# -*- coding: utf-8 -*-

'''
This module provides an access to the HITRAN data.
Data is downloaded and cached.

This module serves as a simple database manager front end.

API is aimed to be RESTful, which means that interaction
between local API and remote data-server will be held 
via sending RESTful queries (API->remote) and
receiving data preferably in text format (remote->API).

Object are supposed to be implemented by structures/dicts
as they are present in almost any programming language.

Trying to retain functional style for this API. 
'''

import sys
import json
import os, os.path
import re
from os import listdir
import numpy as np
from numpy import zeros,array,setdiff1d,ndarray,arange
from numpy import place,where,real,polyval
from numpy import complex128,int64,float64,float32
from numpy import sqrt,abs,exp,pi,log,sin,cos,tan
from numpy import convolve
from numpy import flipud
from numpy.fft import fft,fftshift
from numpy import linspace,floor
from numpy import any,minimum,maximum
from numpy import sort as npsort
from bisect import bisect
from warnings import warn,simplefilter
from time import time
import pydoc

# Enable warning repetitions
simplefilter('always', UserWarning)

# Python 3 compatibility
try:
    import urllib.request as urllib2
except ImportError:
    import urllib2
if 'io' in sys.modules: # define open using Linux-style line endings
    import io
    def open_(*args,**argv):
        argv.update(dict(newline='\n'))
        return io.open(*args,**argv)
else:
    open_ = open

HAPI_VERSION = '1.2.0.0'; __version__ = HAPI_VERSION
HAPI_HISTORY = [
'FIXED GRID BUG (ver. 1.1.0.1)',
'FIXED OUTPUT FORMAT FOR CROSS-SECTIONS (ver. 1.1.0.1)',
'ADDED CPF BY SCHREIER (JQSRT_112_2011) (ver. 1.1.0.2)',
'OPTIMIZED EXPRESSION EVALUATIONS FOR SELECT (ver. 1.1.0.3)',
'ADDED SUPPORT FOR MIXTURES (ver. 1.1.0.4)',
'ADDED SUPPORT FOR USER-DEFINED ENV DEPENDENCES (ver. 1.1.0.5)',
'ADDED PROFILE SELECTION (ALPHA) (ver. 1.1.0.6)',
'ADDED METADATA FOR HTP, FIXED NORMALIZATION IN CONVOLVESPECTRUMSAME (ver. 1.1.0.7)',
'FIXED A "LONELY HEADER" BUG IN CACHE2STORAGE (ver. 1.1.0.7.1)',
'ADDED SUPPORT FOR PHOSGENE AND CYANOGEN (ver. 1.1.0.7.2)',
'OPTIMIZED STORAGE2CACHE (by Nils-Holger Loeber) (ver. 1.1.0.7.3)',
'ADDED SKIPABLE PARAMETERS IN HEADERS (ver. 1.1.0.7.4)',
'ADDED SUPPORT FOR FORTRAN D-NOTATION (ver. 1.1.0.7.5)',
'ADDED SUPPORT FOR WEIRD-FORMATTED INTENSITY VALUES E.G. "2.700-164" (ver. 1.1.0.7.6)',
'ADDED TIPS-2017 (ver. 1.1.0.8)',
'ADDED SUPPORT FOR CUSTOM EXTENSIONS OF THE DATA FILES (ver. 1.1.0.8.1)',
'FIXED LINK TO (2,0) ISOTOPOLOGUE IN TIPS-2017 (ver. 1.1.0.8.2)',
'ADDED SAVEHEADER FUNCTION (ver. 1.1.0.8.3)',
'ADDED METADATA FOR SF6 (ver. 1.1.0.8.4)',
'ADDED D2O ISOTOPOLOGUE OF WATER TO DESCRIPTION (ver. 1.1.0.8.5)',
'FIXED LINE ENDINGS IN STORAGE2CACHE AND QUERYHITRAN (ver. 1.1.0.8.6)',
'ADDED SUPPORT FOR NON-INTEGER LOCAL ISO IDS (ver. 1.1.0.8.7)',
'FIXED PARAMETER NAME CASE BUG (by Robert J. Hargreaves) (ver. 1.1.0.8.8)',
'CAST LOCAL_ISO_ID=0 TO 10 FOR CARBON DIOXIDE (ver. 1.1.0.8.9)',
'USING NUMPY.ARRAYS FOR NUMERIC COLUMNS OF LOCAL_TABLE_CACHE (ver. 1.1.0.9.0)',
'ADDED DESCRIPTIONS FOR BROADENING BY H2O (ver. 1.1.0.9.1)',
'ADDED PROXY SUPPORT IN FETCH AND FETCH_BY_IDS (ver. 1.1.0.9.2)',
'ADDED LIMIT FOR NUMBER OF LINES DURING TABLE READ (ver. 1.1.0.9.3)',
'FIXED ABSOLUTE PATH BUG IN TABLE NAMES (ver. 1.1.0.9.4)',
'CORRECTED ABUNDANCE OF THE HD ISOTOPOLOGUE (ver. 1.1.0.9.5)',
'ADDED UNIFIED INTERFACES FOR ABSCOEF AND XSC CALCULATIONS (ver. 1.1.0.9.6)',
'ADDED PARLISTS FOR LINE MIXING (VOIGT AND SDVOIGT) (ver. 1.1.0.9.7)',
'ADDED SUPPORT FOR ROSENKRANZ LM PARAMETERS TO PCQSDHC AND LORENTZ (ver. 1.1.1.0)',
'FIXED THE TYPEERROR IN ARANGE (ver. 1.1.2.0)',
'ADDED NEW FUNCTIONAL INTERFACES FOR ALL CROSS-SECTION CALCULATING ROUTINES (ver. 1.2.0.0)',
]

# version header
print('HAPI version: %s' % HAPI_VERSION)
print('To get the most up-to-date version please check http://hitran.org/hapi')
print('ATTENTION: Python versions of partition sums from TIPS-2021 are now available in HAPI code')
#print('ATTENTION: Python versions of partition sums from TIPS-2017 are available at http://hitran.org/suppl/TIPS/')
#print('           To use them in HAPI ver. 1.1.0.7, use partitionFunction parameter of the absorptionCoefficient_ routine.')
print('')
print('           MIT license: Copyright 2021 HITRAN team, see more at http://hitran.org. ')
print('')
print('           If you use HAPI in your research or software development,')
print('           please cite it using the following reference:')
print('           R.V. Kochanov, I.E. Gordon, L.S. Rothman, P. Wcislo, C. Hill, J.S. Wilzewski,')
print('           HITRAN Application Programming Interface (HAPI): A comprehensive approach')
print('           to working with spectroscopic data, J. Quant. Spectrosc. Radiat. Transfer 177, 15-30 (2016)')
print('           DOI: 10.1016/j.jqsrt.2016.03.005')
print('')
print('           ATTENTION: This is the core version of the HITRAN Application Programming Interface.')
print('                      For more efficient implementation of the absorption coefficient routine, ')
print('                      as well as for new profiles, parameters and other functional,')
print('                      please consider using HAPI2 extension library.')
print('                      HAPI2 package is available at http://github.com/hitranonline/hapi2')
print('')


# define precision
__ComplexType__ = complex128
__IntegerType__ = int64
__FloatType__ = float64

# define zero
cZero = __FloatType__(0.)

# physical constants
cBolts = 1.380648813E-16 # erg/K, CGS
cc = 2.99792458e10 # cm/s, CGS
hh = 6.626196e-27 # erg*s, CGS

# computational constants
cSqrtLn2divSqrtPi = 0.469718639319144059835
cLn2 = 0.6931471805599
cSqrtLn2 = 0.8325546111577
cSqrt2Ln2 = 1.1774100225

# initialize global variables
VARIABLES = {}

VARIABLES['DEBUG'] = False
if VARIABLES['DEBUG']: warn('DEBUG is set to True!')

GLOBAL_DEBUG = False
if GLOBAL_DEBUG: warn('GLOBAL_DEBUG is set to True!')

FLAG_DEBUG_PROFILE = False
FLAG_DEBUG_LADDER = False

LOCAL_HOST = 'http://localhost'

# DEBUG switch
if GLOBAL_DEBUG:
   GLOBAL_HOST = LOCAL_HOST+':8000' # localhost
else:
   GLOBAL_HOST = 'http://hitran.org'

VARIABLES['PROXY'] = {}
# EXAMPLE OF PROXY:
# VARIABLES['PROXY'] = {'http': '127.0.0.1:80'}
   
# make it changeable
VARIABLES['GLOBAL_HOST'] = GLOBAL_HOST

# display the fetch URL (debug)
VARIABLES['DISPLAY_FETCH_URL'] = False

# In this "robust" version of arange the grid doesn't suffer 
# from the shift of the nodes due to error accumulation.
# This effect is pronounced only if the step is sufficiently small.
def arange_(lower,upper,step):
    npnt = floor((upper-lower)/step)+1
    npnt = int(npnt) # cast to integer to avoid type errors
    upper_new = lower + step*(npnt-1)
    if abs((upper-upper_new)-step) < 1e-10:
        upper_new += step
        npnt += 1    
    return linspace(lower,upper_new,npnt)

# ---------------------------------------------------------------
# ---------------------------------------------------------------
# LOCAL DATABASE MANAGEMENT SYSTEM
# ---------------------------------------------------------------
# ---------------------------------------------------------------

# ---------------------------------------------------------------
# DATABASE BACKEND: simple text files, parsed into a python lists
# Use a directory as a database. Each table is stored in a 
# separate text file. Parameters in text are position-fixed.

BACKEND_DATABASE_NAME_DEFAULT = '.'

VARIABLES['BACKEND_DATABASE_NAME'] = BACKEND_DATABASE_NAME_DEFAULT

# For this node local DB is schema-dependent!
LOCAL_TABLE_CACHE = {
   'sampletab' : { # table
      'header' : { # header
         'order' : ('column1','column2','column3'),
         'format' : {
            'column1' : '%10d',
            'column2' : '%20f',
            'column3' : '%30s' 
         },
         'default' : {
            'column1' : 0,
            'column2' : 0.0,
            'column3' : ''
         },
         'number_of_rows' : 3,
         'size_in_bytes' : None,
         'table_name' : 'sampletab',
         'table_type' : 'strict'
      }, # /header
      'data' : {
         'column1' : [1,2,3],
         'column2' : [10.5,11.5,12.5],
         'column3' : ['one','two','three']
      }, # /data
   } # /table
} # hash-map of tables

# FORMAT CONVERSION LAYER

# converts between TRANSPORT_FORMAT and OBJECT_FORMAT
HITRAN_FORMAT_160 = {
   'M'          : {'pos' :   1,   'len' :  2,   'format' : '%2d' },
   'I'          : {'pos' :   3,   'len' :  1,   'format' : '%1d' },
   'nu'         : {'pos' :   4,   'len' : 12,   'format' : '%12f'},
   'S'          : {'pos' :  16,   'len' : 10,   'format' : '%10f'},
   'R'          : {'pos' :  26,   'len' :  0,   'format' : '%0f' },
   'A'          : {'pos' :  26,   'len' : 10,   'format' : '%10f'},
   'gamma_air'  : {'pos' :  36,   'len' :  5,   'format' : '%5f' },
   'gamma_self' : {'pos' :  41,   'len' :  5,   'format' : '%5f' },
   'E_'         : {'pos' :  46,   'len' : 10,   'format' : '%10f'},
   'n_air'      : {'pos' :  56,   'len' :  4,   'format' : '%4f' },
   'delta_air'  : {'pos' :  60,   'len' :  8,   'format' : '%8f' },
   'V'          : {'pos' :  68,   'len' : 15,   'format' : '%15s'},
   'V_'         : {'pos' :  83,   'len' : 15,   'format' : '%15s'},
   'Q'          : {'pos' :  98,   'len' : 15,   'format' : '%15s'},
   'Q_'         : {'pos' : 113,   'len' : 15,   'format' : '%15s'},
   'Ierr'       : {'pos' : 128,   'len' :  6,   'format' : '%6s' },
   'Iref'       : {'pos' : 134,   'len' : 12,   'format' : '%12s'},
   'flag'       : {'pos' : 146,   'len' :  1,   'format' : '%1s' },
   'g'          : {'pos' : 147,   'len' :  7,   'format' : '%7f' },
   'g_'         : {'pos' : 154,   'len' :  7,   'format' : '%7f' }
}

# This should be generating from the server's response
HITRAN_DEFAULT_HEADER = {
  "table_type": "column-fixed", 
  "size_in_bytes": -1, 
  "table_name": "###", 
  "number_of_rows": -1, 
  "order": [
    "molec_id", 
    "local_iso_id", 
    "nu", 
    "sw", 
    "a", 
    "gamma_air", 
    "gamma_self", 
    "elower", 
    "n_air", 
    "delta_air", 
    "global_upper_quanta", 
    "global_lower_quanta", 
    "local_upper_quanta", 
    "local_lower_quanta", 
    "ierr", 
    "iref", 
    "line_mixing_flag", 
    "gp", 
    "gpp"
  ],
  "format": {
    "a": "%10.3E", 
    "gamma_air": "%5.4f", 
    "gp": "%7.1f", 
    "local_iso_id": "%1d", 
    "molec_id": "%2d", 
    "sw": "%10.3E", 
    "local_lower_quanta": "%15s", 
    "local_upper_quanta": "%15s", 
    "gpp": "%7.1f", 
    "elower": "%10.4f", 
    "n_air": "%4.2f", 
    "delta_air": "%8.6f", 
    "global_upper_quanta": "%15s", 
    "iref": "%12s", 
    "line_mixing_flag": "%1s", 
    "ierr": "%6s", 
    "nu": "%12.6f", 
    "gamma_self": "%5.3f", 
    "global_lower_quanta": "%15s"
  }, 
  "default": {
    "a": 0.0, 
    "gamma_air": 0.0, 
    "gp": "FFF", 
    "local_iso_id": 0, 
    "molec_id": 0, 
    "sw": 0.0, 
    "local_lower_quanta": "000", 
    "local_upper_quanta": "000", 
    "gpp": "FFF", 
    "elower": 0.0, 
    "n_air": 0.0, 
    "delta_air": 0.0, 
    "global_upper_quanta": "000", 
    "iref": "EEE", 
    "line_mixing_flag": "EEE", 
    "ierr": "EEE", 
    "nu": 0.0, 
    "gamma_self": 0.0, 
    "global_lower_quanta": "000"
  },
  "description": {
    "a": "Einstein A-coefficient in s-1", 
    "gamma_air": "Air-broadened Lorentzian half-width at half-maximum at p = 1 atm and T = 296 K", 
    "gp": "Upper state degeneracy", 
    "local_iso_id": "Integer ID of a particular Isotopologue, unique only to a given molecule, in order or abundance (1 = most abundant)", 
    "molec_id": "The HITRAN integer ID for this molecule in all its isotopologue forms", 
    "sw": "Line intensity, multiplied by isotopologue abundance, at T = 296 K", 
    "local_lower_quanta": "Rotational, hyperfine and other quantum numbers and labels for the lower state of a transition", 
    "local_upper_quanta": "Rotational, hyperfine and other quantum numbers and labels for the upper state of a transition", 
    "gpp": "Lower state degeneracy", 
    "elower": "Lower-state energy", 
    "n_air": "Temperature exponent for the air-broadened HWHM", 
    "delta_air": "Pressure shift induced by air, referred to p=1 atm", 
    "global_upper_quanta": "Electronic and vibrational quantum numbers and labels for the upper state of a transition", 
    "iref": "Ordered list of reference identifiers for transition parameters", 
    "line_mixing_flag": "A flag indicating the presence of additional data and code relating to line-mixing", 
    "ierr": "Ordered list of indices corresponding to uncertainty estimates of transition parameters", 
    "nu": "Transition wavenumber", 
    "gamma_self": "Self-broadened HWHM at 1 atm pressure and 296 K", 
    "global_lower_quanta": "Electronic and vibrational quantum numbers and labels for the lower state of a transition"
  },
  "position": {
    "molec_id": 0,
    "local_iso_id": 2,
    "nu": 3,
    "sw": 15,
    "a": 25,
    "gamma_air": 35,
    "gamma_self": 40,
    "elower": 45,
    "n_air": 55,
    "delta_air": 59,
    "global_upper_quanta": 67,
    "global_lower_quanta": 82,
    "local_upper_quanta": 97,
    "local_lower_quanta": 112,
    "ierr": 127,
    "iref": 133,
    "line_mixing_flag": 145,
    "gp": 146,
    "gpp": 153,
  },
  'cast': {
    "molec_id": "uint8",
    "local_iso_id": "uint8",
    "nu": "float32",
    "sw": "float62",
    "a": "float62",
    "gamma_air": "float16",
    "gamma_self": "float16",
    "elower": "float32",
    "n_air": "float16",
    "delta_air": "float16",
    "global_upper_quanta": "str",
    "global_lower_quanta": "str",
    "local_upper_quanta": "str",
    "local_upper_quanta": "str",
    "ierr": "str",
    "iref": "str",
    "line_mixing_flag": "str",
    "gp": "int16",
    "gpp": "int16",  
  }
}

#class CaselessDict(dict):                   
#    def __getitem__(self,key):
#        return super(CaselessDict,self).__getitem__(key.lower())
#    def __setitem__(self,key,val):
#        super(CaselessDict,self).__setitem__(key.lower(),val)
#    def __contains__(self,key):
#        return super(CaselessDict,self).__contains__(key.lower())
#    def has_key(self,key):
#        return super(CaselessDict,self).has_key(key.lower())
#    def get(self,key,default=None):
#        return super(CaselessDict,self).get(key.lower(),default)
#    def __init__(self,dct=None):
#        if dct is not None:
#            for key in dct:
#                super(CaselessDict,self).__setitem__(key.lower(),dct[key])

class CaseInsensitiveDict(dict):
    """
    https://gist.github.com/m000/acbb31b9eca92c1da795 (c) Manolis Stamatogiannakis.
    """
    @classmethod
    def _k(cls, key):
        return key.lower() if isinstance(key, str) else key

    def __init__(self, *args, **kwargs):
        super(CaseInsensitiveDict, self).__init__(*args, **kwargs)
        self._convert_keys()
    def __getitem__(self, key):
        return super(CaseInsensitiveDict, self).__getitem__(self.__class__._k(key))
    def __setitem__(self, key, value):
        super(CaseInsensitiveDict, self).__setitem__(self.__class__._k(key), value)
    def __delitem__(self, key):
        return super(CaseInsensitiveDict, self).__delitem__(self.__class__._k(key))
    def __contains__(self, key):
        return super(CaseInsensitiveDict, self).__contains__(self.__class__._k(key))
    def has_key(self, key):
        return super(CaseInsensitiveDict, self).has_key(self.__class__._k(key))
    def pop(self, key, *args, **kwargs):
        return super(CaseInsensitiveDict, self).pop(self.__class__._k(key), *args, **kwargs)
    def get(self, key, *args, **kwargs):
        return super(CaseInsensitiveDict, self).get(self.__class__._k(key), *args, **kwargs)
    def setdefault(self, key, *args, **kwargs):
        return super(CaseInsensitiveDict, self).setdefault(self.__class__._k(key), *args, **kwargs)
    def update(self, E, **F):
        super(CaseInsensitiveDict, self).update(self.__class__(E))
        super(CaseInsensitiveDict, self).update(self.__class__(**F))
    def _convert_keys(self):
        for k in list(self.keys()):
            v = super(CaseInsensitiveDict, self).pop(k)
            self.__setitem__(k, v)

CaselessDict = CaseInsensitiveDict
            
PARAMETER_META = CaselessDict(
{
  "global_iso_id" : {
    "default_fmt" : "%5d",
  },
  "molec_id" : {
    "default_fmt" : "%2d",
  },
  "local_iso_id" : {
    "default_fmt" : "%1d",
  },
  "nu" : {
    "default_fmt" : "%12.6f",
  },
  "sw" : {
    "default_fmt" : "%10.3e",
  },
  "a" : {
    "default_fmt" : "%10.3e",
  },
  "gamma_air" : {
    "default_fmt" : "%6.4f",
  },
  "gamma_self" : {
    "default_fmt" : "%5.3f",
  },
  "n_air" : {
    "default_fmt" : "%7.4f",
  },
  "delta_air" : {
    "default_fmt" : "%9.6f",
  },
  "elower" : {
    "default_fmt" : "%10.4f",
  },
  "gp" : {
    "default_fmt" : "%5d",
  },
  "gpp" : {
    "default_fmt" : "%5d",
  },
  "global_upper_quanta" : {
    "default_fmt" : "%15s",
  },
  "global_lower_quanta" : {
    "default_fmt" : "%15s",
  },
  "local_upper_quanta" : {
    "default_fmt" : "%15s",
  },
  "local_lower_quanta" : {
    "default_fmt" : "%15s",
  },
  "line_mixing_flag" : {
    "default_fmt" : "%1s",
  },
  "ierr" : {
    "default_fmt" : "%s",
  },
  "iref" : {
    "default_fmt" : "%s",
  },
  "deltap_air" : {
    "default_fmt" : "%10.3e",
  },
  "n_self" : {
    "default_fmt" : "%7.4f",
  },
  "delta_self" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_self" : {
    "default_fmt" : "%10.3e",
  },
  "SD_air" : {
    "default_fmt" : "%9.6f",
  },
  "SD_self" : {
    "default_fmt" : "%9.6f",
  },
  "beta_g_air" : {
    "default_fmt" : "%9.6f",
  },
  "y_self" : {
    "default_fmt" : "%10.3e",
  },
  "y_air" : {
    "default_fmt" : "%10.3e",
  },
  "statep" : {
    "default_fmt" : "%256s",
  },
  "statepp" : {
    "default_fmt" : "%256s",
  },
  "beta_g_self" : {
    "default_fmt" : "%9.6f",
  },
  "trans_id" : {
    "default_fmt" : "%12d",
  },
  "par_line" : {
    "default_fmt" : "%160s",
  },
  "gamma_H2" : {
    "default_fmt" : "%6.4f",
  },
  "n_H2" : {
    "default_fmt" : "%7.4f",
  },
  "delta_H2" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_H2" : {
    "default_fmt" : "%10.3e",
  },
  "gamma_He": {
    "default_fmt" : "%6.4f",
  },
  "n_He" : {
    "default_fmt" : "%7.4f",
  },
  "delta_He" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_CO2" : {
    "default_fmt" : "%6.4f",
  },
  "n_CO2" : {
    "default_fmt" : "%7.4f",
  },
  "delta_CO2" : {
    "default_fmt" : "%9.6f",
  }, 
  "gamma_HT_0_self_50" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_self_50" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_self_50" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_self_50" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_self_50" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_self_50" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_0_self_150" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_self_150" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_self_150" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_self_150" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_self_150" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_self_150" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_0_self_296" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_self_296" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_self_296" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_self_296" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_self_296" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_self_296" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_0_self_700" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_self_700" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_self_700" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_self_700" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_self_700" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_self_700" : {
    "default_fmt" : "%9.6f",
  },
  "nu_HT_self" : {
    "default_fmt" : "%6.4f",
  },
  "kappa_HT_self" : {
    "default_fmt" : "%9.6f",
  },
  "eta_HT_self" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_0_air_50" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_air_50" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_air_50" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_air_50" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_air_50" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_air_50" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_0_air_150" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_air_150" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_air_150" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_air_150" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_air_150" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_air_150" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_0_air_296" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_air_296" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_air_296" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_air_296" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_air_296" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_air_296" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_0_air_700" : {
    "default_fmt" : "%6.4f",
  },
  "n_HT_air_700" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_HT_2_air_700" : {
    "default_fmt" : "%6.4f",
  },
  "delta_HT_0_air_700" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_HT_air_700" : {
    "default_fmt" : "%9.6f",
  },
  "delta_HT_2_air_700" : {
    "default_fmt" : "%9.6f",
  },
  "nu_HT_air" : {
    "default_fmt" : "%6.4f",
  },
  "kappa_HT_air" : {
    "default_fmt" : "%9.6f",
  },
  "eta_HT_air" : {
    "default_fmt" : "%9.6f",
  },  
  "gamma_H2O" : {
    "default_fmt" : "%6.4f",
  },
  "n_H2O" : {
    "default_fmt" : "%9.6f",
  },
  "Y_SDV_air_296" : {
    "default_fmt" : "%10.3e",
  },
  "Y_SDV_self_296" : {
    "default_fmt" : "%10.3e",
  },
  "n_Y_SDV_air_296" : {
    "default_fmt" : "%6.4e",
  },
  "n_Y_SDV_self_296" : {
    "default_fmt" : "%6.4e",
  },
  "Y_HT_air_296" : {
    "default_fmt" : "%10.3e",
  },
  "Y_HT_self_296" : {
    "default_fmt" : "%10.3e",
  },
  "gamma_SDV_0_air_296" : {
    "default_fmt" : "%6.4f",
  },
  "gamma_SDV_0_self_296" : {
    "default_fmt" : "%6.4f",
  },
  "n_SDV_air_296" : {
    "default_fmt" : "%9.6f",
  },
  "n_SDV_self_296" : {
    "default_fmt" : "%9.6f",
  },
  "gamma_SDV_2_air_296" : {
    "default_fmt" : "%6.4f",
  },
  "gamma_SDV_2_self_296" : {
    "default_fmt" : "%6.4f",
  },
  "n_gamma_SDV_2_air_296" : {
    "default_fmt" : "%6.4f",
  },
  "n_gamma_SDV_2_self_296" : {
    "default_fmt" : "%6.4f",
  },
  "delta_SDV_0_air_296" : {
    "default_fmt" : "%9.6f",
  },
  "delta_SDV_0_self_296" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_SDV_air_296" : {
    "default_fmt" : "%9.6f",
  },
  "deltap_SDV_self_296" : {
    "default_fmt" : "%9.6f",
  },
})

def getFullTableAndHeaderName(TableName,ext=None):
    #print('TableName=',TableName)
    if ext is None: ext = 'data'
    flag_abspath = False # check if the supplied table name already contains absolute path
    if os.path.isabs(TableName): flag_abspath = True        
    fullpath_data = TableName + '.' + ext
    if not flag_abspath: fullpath_data = os.path.join(VARIABLES['BACKEND_DATABASE_NAME'],fullpath_data)
    if not os.path.isfile(fullpath_data):
        fullpath_data = VARIABLES['BACKEND_DATABASE_NAME'] + '/' + TableName + '.par'
        if not os.path.isfile(fullpath_data) and TableName!='sampletab':
            raise Exception('Lonely header \"%s\"' % fullpath_data)
    fullpath_header = TableName + '.header'
    if not flag_abspath: fullpath_header = os.path.join(VARIABLES['BACKEND_DATABASE_NAME'],fullpath_header)
    return fullpath_data,fullpath_header

def getParameterFormat(ParameterName,TableName):
    return LOCAL_TABLE_CACHE[TableName]['header']['format']

def getTableHeader(TableName):
    return LOCAL_TABLE_CACHE[TableName]['header']

def getRowObject(RowID,TableName):
    # return RowObject from TableObject in CACHE
    RowObject = []
    for par_name in LOCAL_TABLE_CACHE[TableName]['header']['order']:
        par_value = LOCAL_TABLE_CACHE[TableName]['data'][par_name][RowID]
        par_format = LOCAL_TABLE_CACHE[TableName]['header']['format'][par_name]
        RowObject.append((par_name,par_value,par_format))
    return RowObject

# INCREASE ROW COUNT
def addRowObject(RowObject,TableName):
    #print 'addRowObject: '
    #print 'RowObject: '+str(RowObject)
    #print 'TableName:'+TableName
    for par_name,par_value,par_format in RowObject:
        #print 'par_name,par_value,par_format: '+str((par_name,par_value,par_format))
        #print '>>> '+ str(LOCAL_TABLE_CACHE[TableName]['data'][par_name])
        #LOCAL_TABLE_CACHE[TableName]['data'][par_name] += [par_value]
        LOCAL_TABLE_CACHE[TableName]['data'][par_name].append(par_value)

def setRowObject(RowID,RowObject,TableName):
    number_of_rows = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
    if RowID >= 0 and RowID < number_of_rows:
       for par_name,par_value,par_format in RowObject:
           LOCAL_TABLE_CACHE[TableName]['data'][par_name][RowID] = par_value
    else:
       # !!! XXX ATTENTION: THIS IS A TEMPORARY INSERTION XXX !!!
       LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows'] += 1
       addRowObject(RowObject,TableName)

def getDefaultRowObject(TableName):
    # get a default RowObject from a table
    RowObject = []
    for par_name in LOCAL_TABLE_CACHE[TableName]['header']['order']:
        par_value = LOCAL_TABLE_CACHE[TableName]['header']['default'][par_name]
        par_format = LOCAL_TABLE_CACHE[TableName]['header']['format'][par_name]
        RowObject.append((par_name,par_value,par_format))
    return RowObject

def subsetOfRowObject(ParameterNames,RowObject):
    # return a subset of RowObject according to 
    #RowObjectNew = []
    #for par_name,par_value,par_format in RowObject:
    #     if par_name in ParameterNames:
    #        RowObjectNew.append((par_name,par_value,par_format))
    #return RowObjectNew
    dct = {}
    for par_name,par_value,par_format in RowObject:
        dct[par_name] = (par_name,par_value,par_format)
    RowObjectNew = []
    for par_name in ParameterNames:
        RowObjectNew.append(dct[par_name])
    return RowObjectNew

#FORMAT_PYTHON_REGEX = '^\%([0-9]*)\.?([0-9]*)([dfs])$'
FORMAT_PYTHON_REGEX = '^\%(\d*)(\.(\d*))?([edfsEDFS])$'

# Fortran string formatting
#  based on a pythonic format string
def formatString(par_format,par_value,lang='FORTRAN'):
    # Fortran format rules:
    #  %M.NP
    #        M - total field length (optional)
    #             (minus sign included in M)
    #        . - decimal ceparator (optional)
    #        N - number of digits after . (optional)
    #        P - [dfs] int/float/string
    # PYTHON RULE: if N is abcent, default value is 6
    regex = FORMAT_PYTHON_REGEX
    (lng,trail,lngpnt,ty) = re.search(regex,par_format).groups()
    if type(par_value) is np.ma.core.MaskedConstant:
        result = '%%%ds' % lng % '#'
        return result
    result = par_format % par_value
    if ty.lower() in set(['f','e']):
       lng = int(lng) if lng else 0
       lngpnt = int(lngpnt) if lngpnt else 0
       result = par_format % par_value
       res = result.strip()
       if lng==lngpnt+1:
          if res[0:1]=='0':
             result =  '%%%ds' % lng % res[1:]
       if par_value<0:
          if res[1:2]=='0':
             result = '%%%ds' % lng % (res[0:1]+res[2:])
    return result

def putRowObjectToString(RowObject):
    # serialize RowObject to string
    # TODO: support different languages (C,Fortran)
    output_string = ''
    for par_name,par_value,par_format in RowObject:
        # Python formatting
        #output_string += par_format % par_value
        # Fortran formatting
        #print 'par_name,par_value,par_format: '+str((par_name,par_value,par_format))
        output_string += formatString(par_format,par_value)
    return output_string

# Parameter nicknames are hard-coded.
PARAMETER_NICKNAMES = {
    "a": "A", 
    "gamma_air": "gair", 
    "gp": "g", 
    "local_iso_id": "I", 
    "molec_id": "M", 
    "sw": "S", 
    "local_lower_quanta": "Q_", 
    "local_upper_quanta": "Q", 
    "gpp": "g_", 
    "elower": "E_", 
    "n_air": "nair", 
    "delta_air": "dair", 
    "global_upper_quanta": "V", 
    "iref": "Iref", 
    "line_mixing_flag": "f", 
    "ierr": "ierr", 
    "nu": "nu", 
    "gamma_self": "gsel", 
    "global_lower_quanta": "V_"
}  

def putTableHeaderToString(TableName):
    output_string = ''
    regex = FORMAT_PYTHON_REGEX
    for par_name in LOCAL_TABLE_CACHE[TableName]['header']['order']:
        par_format = LOCAL_TABLE_CACHE[TableName]['header']['format'][par_name]
        (lng,trail,lngpnt,ty) = re.search(regex,par_format).groups()
        fmt = '%%%ss' % lng
        try:
            par_name_short = PARAMETER_NICKNAMES[par_name]
        except:
            par_name_short = par_name
        #output_string += fmt % par_name
        output_string += (fmt % par_name_short)[:int(lng)]
    return output_string

def getRowObjectFromString(input_string,TableName):
    # restore RowObject from string, get formats and names in TableName
    #print 'getRowObjectFromString:'
    pos = 0
    RowObject = []
    for par_name in LOCAL_TABLE_CACHE[TableName]['header']['order']:
        par_format = LOCAL_TABLE_CACHE[TableName]['header']['format'][par_name]
        regex = '^\%([0-9]+)\.?[0-9]*([dfs])$' #
        regex = FORMAT_PYTHON_REGEX
        (lng,trail,lngpnt,ty) = re.search(regex,par_format).groups()
        lng = int(lng)
        par_value = input_string[pos:(pos+lng)]
        if ty=='d': # integer value
           par_value = int(par_value)
        elif ty.lower() in set(['e','f']): # float value
           par_value = float(par_value)
        elif ty=='s': # string value
           pass # don't strip string value
        else:
           print('err1')
           raise Exception('Format \"%s\" is unknown' % par_format)
        RowObject.append((par_name,par_value,par_format))
        pos += lng
    # Do the same but now for extra (comma-separated) parameters
    if 'extra' in set(LOCAL_TABLE_CACHE[TableName]['header']):
        csv_chunks = input_string.split(LOCAL_TABLE_CACHE[TableName]['header'].\
                                        get('extra_separator',','))
        # Disregard the first "column-fixed" container if it presents:
        if LOCAL_TABLE_CACHE[TableName]['header'].get('order',[]):
            pos = 1
        else:
            pos = 0
        for par_name in LOCAL_TABLE_CACHE[TableName]['header']['extra']:
            par_format = LOCAL_TABLE_CACHE[TableName]['header']['extra_format'][par_name]
            regex = '^\%([0-9]+)\.?[0-9]*([dfs])$' #
            regex = FORMAT_PYTHON_REGEX
            (lng,trail,lngpnt,ty) = re.search(regex,par_format).groups()
            lng = int(lng) 
            par_value = csv_chunks[pos]
            if ty=='d': # integer value
                try:
                    par_value = int(par_value)
                except ValueError:
                    #par_value = 0
                    par_value = np.nan
            elif ty.lower() in set(['e','f']): # float value
                try:
                    par_value = float(par_value)
                except ValueError:
                    #par_value = 0.0
                    par_value = np.nan
            elif ty=='s': # string value
                pass # don't strip string value
            else:
                print('err')
                raise Exception('Format \"%s\" is unknown' % par_format)
            RowObject.append((par_name,par_value,par_format))
            pos += 1   
    return RowObject

# Conversion between OBJECT_FORMAT and STORAGE_FORMAT
# This will substitute putTableToStorage and getTableFromStorage
def cache2storage(TableName):
    try:
       os.mkdir(VARIABLES['BACKEND_DATABASE_NAME'])
    except:
       pass
    #fullpath_data,fullpath_header = getFullTableAndHeaderName(TableName) # "lonely header" bug
    fullpath_data = VARIABLES['BACKEND_DATABASE_NAME'] + '/' + TableName + '.data' # bugfix
    fullpath_header = VARIABLES['BACKEND_DATABASE_NAME'] + '/' + TableName + '.header' # bugfix
    OutfileData = open(fullpath_data,'w')
    OutfileHeader = open(fullpath_header,'w')
    # write table data
    line_count = 1
    line_number = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
    for RowID in range(0,line_number):
        line_count += 1
        RowObject = getRowObject(RowID,TableName)
        raw_string = putRowObjectToString(RowObject)
        OutfileData.write(raw_string+'\n')
    # write table header
    TableHeader = getTableHeader(TableName)
    OutfileHeader.write(json.dumps(TableHeader,indent=2))
    
def storage2cache(TableName,cast=True,ext=None,nlines=None,pos=None):
    """ edited by NHL
    TableName: name of the HAPI table to read in
    ext: file extension
    nlines: number of line in the block; if None, read all line at once 
    pos: file position to seek
    """
    #print 'storage2cache:'
    #print('TableName',TableName)
    if nlines is not None:
        print('WARNING: storage2cache is reading the block of maximum %d lines'%nlines)
    fullpath_data,fullpath_header = getFullTableAndHeaderName(TableName,ext)
    if TableName in LOCAL_TABLE_CACHE and \
       'filehandler' in LOCAL_TABLE_CACHE[TableName] and \
       LOCAL_TABLE_CACHE[TableName]['filehandler'] is not None:
        InfileData = LOCAL_TABLE_CACHE[TableName]['filehandler']
    else:
        InfileData = open_(fullpath_data,'r')            
    InfileHeader = open(fullpath_header,'r')
    #try:
    header_text = InfileHeader.read()
    try:
        Header = json.loads(header_text)
    except:
        print('HEADER:')
        print(header_text)
        raise Exception('Invalid header')
    #print 'Header:'+str(Header)
    LOCAL_TABLE_CACHE[TableName] = {}
    LOCAL_TABLE_CACHE[TableName]['header'] = Header
    LOCAL_TABLE_CACHE[TableName]['data'] = CaselessDict()
    LOCAL_TABLE_CACHE[TableName]['filehandler'] = InfileData
    # Check if Header['order'] and Header['extra'] contain
    #  parameters with same names, raise exception if true.
    #intersct = set(Header['order']).intersection(set(Header.get('extra',[])))
    intersct = set(Header.get('order',[])).intersection(set(Header.get('extra',[])))
    if intersct:
        raise Exception('Parameters with the same names: {}'.format(intersct))
    # initialize empty data to avoid problems
    glob_order = []; glob_format = {}; glob_default = {}
    if "order" in LOCAL_TABLE_CACHE[TableName]['header'].keys():
        glob_order += LOCAL_TABLE_CACHE[TableName]['header']['order']
        glob_format.update(LOCAL_TABLE_CACHE[TableName]['header']['format'])
        glob_default.update(LOCAL_TABLE_CACHE[TableName]['header']['default'])
        for par_name in LOCAL_TABLE_CACHE[TableName]['header']['order']:
            LOCAL_TABLE_CACHE[TableName]['data'][par_name] = []
    if "extra" in LOCAL_TABLE_CACHE[TableName]['header'].keys():
        glob_order += LOCAL_TABLE_CACHE[TableName]['header']['extra']
        glob_format.update(LOCAL_TABLE_CACHE[TableName]['header']['extra_format'])
        for par_name in LOCAL_TABLE_CACHE[TableName]['header']['extra']:
            glob_default[par_name] = PARAMETER_META[par_name]['default_fmt']
            LOCAL_TABLE_CACHE[TableName]['data'][par_name] = []
    
    header = LOCAL_TABLE_CACHE[TableName]['header']
    if 'extra' in header and header['extra']:
        line_count = 0
        flag_EOF = False
        #line_number = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
        #for line in InfileData:
        while True:
            #print '%d line from %d' % (line_count,line_number)
            #print 'line: '+line #
            if nlines is not None and line_count>=nlines: break
            line = InfileData.readline()
            if line=='': # end of file is represented by an empty string
                flag_EOF = True
                break 
            try:
                RowObject = getRowObjectFromString(line,TableName)
                line_count += 1
            except:
                continue
            #print 'RowObject: '+str(RowObject)
            addRowObject(RowObject,TableName)
        #except:
        #    raise Exception('TABLE FETCHING ERROR')
        LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows'] = line_count
    else:
        quantities = header['order']
        formats = [header['format'][qnt] for qnt in quantities]
        types = {'d':int, 'f':float, 'E':float, 's':str}
        converters = []
        end = 0
        for qnt, fmt in zip(quantities, formats):
            # pre-defined positions are needed to skip the existing parameters in headers (new feature)
            if 'position' in header:
                start = header['position'][qnt]
            else:
                start = end
            dtype = types[fmt[-1]]
            aux = fmt[fmt.index('%')+1:-1]
            if '.' in aux:
                aux = aux[:aux.index('.')]
            size = int(aux)
            end = start + size
            def cfunc(line, dtype=dtype, start=start, end=end, qnt=qnt):
                # return dtype(line[start:end]) # this will fail on the float number with D exponent (Fortran notation)
                if dtype==float:
                    try:
                        return dtype(line[start:end])
                    except ValueError: # possible D exponent instead of E 
                        try:
                            return dtype(line[start:end].replace('D','E'))
                        except ValueError: # this is a special case and it should not be in the main version tree!
                            # Dealing with the weird and unparsable intensity format such as "2.700-164, i.e with no E or D characters.
                            res = re.search('(\d\.\d\d\d)\-(\d\d\d)',line[start:end])
                            if res:
                                return dtype(res.group(1)+'E-'+res.group(2))
                            else:
                                raise Exception('PARSE ERROR: unknown format of the par value (%s)'%line[start:end])
                elif dtype==int and qnt=='local_iso_id':
                    if line[start:end]=='0': return 10
                    try:
                        return dtype(line[start:end])
                    except ValueError:
                        # convert letters to numbers: A->11, B->12, etc... ; .par file must be in ASCII or Unicode.
                        return 11+ord(line[start:end])-ord('A')
                else:
                    return dtype(line[start:end])
            #cfunc.__doc__ = 'converter {} {}'.format(qnt, fmt) # doesn't work in earlier versions of Python
            converters.append(cfunc)
            #start = end
        #data_matrix = [[cvt(line) for cvt in converters] for line in InfileData]
        flag_EOF = False
        line_count = 0
        data_matrix = []
        while True:
            if nlines is not None and line_count>=nlines: break   
            line = InfileData.readline()
            if line=='': # end of file is represented by an empty string
                flag_EOF = True
                break 
            data_matrix.append([cvt(line) for cvt in converters])
            line_count += 1
        data_columns = zip(*data_matrix)
        for qnt, col in zip(quantities, data_columns):
            #LOCAL_TABLE_CACHE[TableName]['data'][qnt].extend(col) # old code
            if type(col[0]) in {int,float}:
                LOCAL_TABLE_CACHE[TableName]['data'][qnt] = np.array(col) # new code
            else:
                LOCAL_TABLE_CACHE[TableName]['data'][qnt].extend(col) # old code
            #LOCAL_TABLE_CACHE[TableName]['data'][qnt] = list(col)
            #LOCAL_TABLE_CACHE[TableName]['data'][qnt] = col
        header['number_of_rows'] = line_count = (
            len(LOCAL_TABLE_CACHE[TableName]['data'][quantities[0]]))
            
    # Convert all columns to numpy arrays
    par_names = LOCAL_TABLE_CACHE[TableName]['header']['order']
    if 'extra' in header and header['extra']:
        par_names += LOCAL_TABLE_CACHE[TableName]['header']['extra']
    for par_name in par_names:
        column = LOCAL_TABLE_CACHE[TableName]['data'][par_name]
        LOCAL_TABLE_CACHE[TableName]['data'][par_name] = np.array(column)                    
            
    # Additionally: convert numeric arrays in "extra" part of the LOCAL_TABLE_CACHE to masked arrays.
    # This is done to avoid "nan" values in the arithmetic operations involving these columns.
    if 'extra' in header and header['extra']:
        for par_name in LOCAL_TABLE_CACHE[TableName]['header']['extra']:
            par_format = LOCAL_TABLE_CACHE[TableName]['header']['extra_format'][par_name]
            regex = FORMAT_PYTHON_REGEX
            (lng,trail,lngpnt,ty) = re.search(regex,par_format).groups()
            if ty.lower() in ['d','e','f']:
                column = LOCAL_TABLE_CACHE[TableName]['data'][par_name]
                colmask = np.isnan(column)
                LOCAL_TABLE_CACHE[TableName]['data'][par_name] = np.ma.array(column,mask=colmask)
    
    # Delete all character-separated values, treat them as column-fixed.
    try:
        del LOCAL_TABLE_CACHE[TableName]['header']['extra']
        del LOCAL_TABLE_CACHE[TableName]['header']['extra_format']
        del LOCAL_TABLE_CACHE[TableName]['header']['extra_separator']
    except:
        pass
    # Update header.order/format with header.extra/format if exist.
    LOCAL_TABLE_CACHE[TableName]['header']['order'] = glob_order
    LOCAL_TABLE_CACHE[TableName]['header']['format'] = glob_format
    LOCAL_TABLE_CACHE[TableName]['header']['default'] = glob_default
    if flag_EOF:
        InfileData.close()
        LOCAL_TABLE_CACHE[TableName]['filehandler'] = None
    InfileHeader.close()
    print('                     Lines parsed: %d' % line_count)
    return flag_EOF    
    
## old version based on regular expressions    
#def storage2cache(TableName):
#    fullpath_data,fullpath_header = getFullTableAndHeaderName(TableName)
#    InfileData = open(fullpath_data,'r')
#    InfileHeader = open(fullpath_header,'r')
#    #try:
#    header_text = InfileHeader.read()
#    try:
#        Header = json.loads(header_text)
#    except:
#        print('HEADER:')
#        print(header_text)
#        raise Exception('Invalid header')
#    LOCAL_TABLE_CACHE[TableName] = {}
#    LOCAL_TABLE_CACHE[TableName]['header'] = Header
#    LOCAL_TABLE_CACHE[TableName]['data'] = {}
#    # Check if Header['order'] and Header['extra'] contain
#    #  parameters with same names, raise exception if true.
#    intersct = set(Header.get('order',[])).intersection(set(Header.get('extra',[])))
#    if intersct:
#        raise Exception('Parameters with the same names: {}'.format(intersct))
#    # initialize empty data to avoid problems
#    glob_order = []; glob_format = {}; glob_default = {}
#    if "order" in LOCAL_TABLE_CACHE[TableName]['header'].keys():
#        glob_order += LOCAL_TABLE_CACHE[TableName]['header']['order']
#        glob_format.update(LOCAL_TABLE_CACHE[TableName]['header']['format'])
#        glob_default.update(LOCAL_TABLE_CACHE[TableName]['header']['default'])
#        for par_name in LOCAL_TABLE_CACHE[TableName]['header']['order']:
#            LOCAL_TABLE_CACHE[TableName]['data'][par_name] = []
#    if "extra" in LOCAL_TABLE_CACHE[TableName]['header'].keys():
#        glob_order += LOCAL_TABLE_CACHE[TableName]['header']['extra']
#        glob_format.update(LOCAL_TABLE_CACHE[TableName]['header']['extra_format'])
#        for par_name in LOCAL_TABLE_CACHE[TableName]['header']['extra']:
#            glob_default[par_name] = PARAMETER_META[par_name]['default_fmt']
#            LOCAL_TABLE_CACHE[TableName]['data'][par_name] = []
#    line_count = 0
#    for line in InfileData:
#        try:
#            RowObject = getRowObjectFromString(line,TableName)
#            line_count += 1
#        except:
#            continue
#        addRowObject(RowObject,TableName)
#    LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows'] = line_count
#    # Delete all character-separated values, treat them as column-fixed.
#    try:
#        del LOCAL_TABLE_CACHE[TableName]['header']['extra']
#        del LOCAL_TABLE_CACHE[TableName]['header']['extra_format']
#        del LOCAL_TABLE_CACHE[TableName]['header']['extra_separator']
#    except:
#        pass
#    # Update header.order/format with header.extra/format if exist.
#    LOCAL_TABLE_CACHE[TableName]['header']['order'] = glob_order
#    LOCAL_TABLE_CACHE[TableName]['header']['format'] = glob_format
#    LOCAL_TABLE_CACHE[TableName]['header']['default'] = glob_default
#    InfileData.close()
#    InfileHeader.close()
#    print('                     Lines parsed: %d' % line_count)
#    pass

# / FORMAT CONVERSION LAYER    
    
def getTableNamesFromStorage(StorageName):
    file_names = listdir(StorageName)
    table_names = []
    for file_name in file_names:
        matchObject = re.search('(.+)\.header$',file_name)
        if matchObject:
           table_names.append(matchObject.group(1))
    return table_names

# FIX POSSIBLE BUG: SIMILAR NAMES OF .PAR AND .DATA FILES
# BUG FIXED BY INTRODUCING A PRIORITY:
#   *.data files have more priority than *.par files
#   See getFullTableAndHeaderName function for explanation
def scanForNewParfiles(StorageName):
    file_names = listdir(StorageName)
    headers = {} # without extensions!
    parfiles_without_header = []
    for file_name in file_names:
        # create dictionary of unique headers
        try:
            fname,fext = re.search('(.+)\.(\w+)',file_name).groups()
        except:
            continue
        if fext == 'header': headers[fname] = True
    for file_name in file_names:
        # check if extension is 'par' and the header is absent
        try:
            fname,fext = re.search('(.+)\.(\w+)',file_name).groups()
        except:
            continue
        if fext == 'par' and fname not in headers:
            parfiles_without_header.append(fname)
    return parfiles_without_header

def createHeader(TableName):
    fname = TableName+'.header'
    fp = open(VARIABLES['BACKEND_DATABASE_NAME']+'/'+fname,'w')
    if os.path.isfile(TableName):
        raise Exception('File \"%s\" already exists!' % fname)
    fp.write(json.dumps(HITRAN_DEFAULT_HEADER,indent=2))
    fp.close()

def loadCache():
    print('Using '+VARIABLES['BACKEND_DATABASE_NAME']+'\n')
    LOCAL_TABLE_CACHE = {}
    table_names = getTableNamesFromStorage(VARIABLES['BACKEND_DATABASE_NAME'])
    parfiles_without_header = scanForNewParfiles(VARIABLES['BACKEND_DATABASE_NAME'])
    # create headers for new parfiles
    for tab_name in parfiles_without_header:
        # get name without 'par' extension
        createHeader(tab_name)
        table_names.append(tab_name)
    for TableName in table_names:
        print(TableName)
        storage2cache(TableName)

def saveCache():
    try:
        # delete query buffer
        del LOCAL_TABLE_CACHE[QUERY_BUFFER]
    except:
        pass
    for TableName in LOCAL_TABLE_CACHE:
        print(TableName)
        cache2storage(TableName)

# DB backend level, start transaction
def databaseBegin(db=None):
    if db:
       VARIABLES['BACKEND_DATABASE_NAME'] = db
    else:
       VARIABLES['BACKEND_DATABASE_NAME'] = BACKEND_DATABASE_NAME_DEFAULT
    if not os.path.exists(VARIABLES['BACKEND_DATABASE_NAME']):
       os.mkdir(VARIABLES['BACKEND_DATABASE_NAME'])
    loadCache()

# DB backend level, end transaction
def databaseCommit():
    saveCache()

# ----------------------------------------------------
# ----------------------------------------------------
# CONDITIONS
# ----------------------------------------------------
# ----------------------------------------------------
# ----------------------------------------------------
# hierarchic query.condition language:
# Conditions: CONS = ('and', ('=','p1','p2'), ('<','p1',13))
# String literals are distinguished from variable names 
#  by using the operation ('STRING','some_string')
# ----------------------------------------------------

# necessary conditions for hitranonline:
SAMPLE_CONDITIONS = ('AND',('SET','internal_iso_id',[1,2,3,4,5,6]),('>=','nu',0),('<=','nu',100))

# sample hitranonline protocol
# http://hitran.cloudapp.net/lbl/5?output_format_id=1&iso_ids_list=5&numin=0&numax=100&access=api&key=e20e4bd3-e12c-4931-99e0-4c06e88536bd

CONDITION_OPERATIONS = set(['AND','OR','NOT','RANGE','IN','<','>','<=','>=','==','!=','LIKE','STR','+','-','*','/','MATCH','SEARCH','FINDALL'])

# Operations used in Condition verification
# Basic scheme: operationXXX(args),
# where args - list/array of arguments (>=1)

def operationAND(args):
    # any number if arguments
    for arg in args:
        if not arg:
           return False
    return True

def operationOR(args):
    # any number of arguments
    for arg in args:
        if arg:
           return True
    return False

def operationNOT(arg):
    # one argument
    return not arg

def operationRANGE(x,x_min,x_max):
    return x_min <= x <= x_max
    
def operationSUBSET(arg1,arg2):
    # True if arg1 is subset of arg2
    # arg1 is an element
    # arg2 is a set
    return arg1 in arg2

def operationLESS(args):
    # any number of args
    for i in range(1,len(args)):
        if args[i-1] >= args[i]:
           return False
    return True

def operationMORE(args):
    # any number of args
    for i in range(1,len(args)):
        if args[i-1] <= args[i]:
           return False
    return True

def operationLESSOREQUAL(args):
    # any number of args
    for i in range(1,len(args)):
        if args[i-1] > args[i]:
           return False
    return True

def operationMOREOREQUAL(args):
    # any number of args
    for i in range(1,len(args)):
        if args[i-1] < args[i]:
           return False
    return True

def operationEQUAL(args):
    # any number of args
    for i in range(1,len(args)):
        if args[i] != args[i-1]:
           return False
    return True

def operationNOTEQUAL(arg1,arg2):
    return arg1 != arg2
    
def operationSUM(args):
    # any numbers of arguments
    if type(args[0]) in set([int,float]):
       result = 0
    elif type(args[0]) in set([str,unicode]):
       result = ''
    else:
       raise Exception('SUM error: unknown arg type')
    for arg in args:
        result += arg
    return result

def operationDIFF(arg1,arg2):
    return arg1-arg2

def operationMUL(args):
    # any numbers of arguments
    if type(args[0]) in set([int,float]):
       result = 1
    else:
       raise Exception('MUL error: unknown arg type')
    for arg in args:
        result *= arg
    return result

def operationDIV(arg1,arg2):
    return arg1/arg2

def operationSTR(arg):
    # transform arg to str
    if type(arg)!=str:
       raise Exception('Type mismatch: STR')
    return arg

def operationSET(arg):
    # transform arg to list
    if type(arg) not in set([list,tuple,set]):
        raise Exception('Type mismatch: SET')
    return list(arg)

def operationMATCH(arg1,arg2):
    # Match regex (arg1) and string (arg2)
    #return bool(re.match(arg1,arg2)) # works wrong
    return bool(re.search(arg1,arg2))

def operationSEARCH(arg1,arg2):
    # Search regex (arg1) in string (arg2)
    # Output list of entries
    group = re.search(arg1,arg2).groups()
    result = []
    for item in group:
        result.append(('STR',item))
    return result

def operationFINDALL(arg1,arg2):
    # Search all groups of a regex
    # Output a list of groups of entries
    # XXX: If a group has more than 1 entry,
    #    there could be potential problems
    list_of_groups = re.findall(arg1,arg2)
    result = []
    for item in list_of_groups:
        result.append(('STR',item))
    return result

def operationLIST(args):
    # args is a list: do nothing (almost)
    return list(args)

# /operations

# GROUPING ---------------------------------------------- 

GROUP_INDEX = {}
# GROUP_INDEX has the following structure:
#  GROUP_INDEX[KEY] = VALUE
#    KEY = table line values
#    VALUE = {'FUNCTIONS':DICT,'FLAG':LOGICAL,'ROWID':INTEGER}
#      FUNCTIONS = {'FUNC_NAME':DICT}
#            FUNC_NAME = {'FLAG':LOGICAL,'NAME':STRING}

# name and default value
GROUP_FUNCTION_NAMES = { 'COUNT' :  0,
                         'SUM'   :  0,
                         'MUL'   :  1,
                         'AVG'   :  0,
                         'MIN'   : +1e100,
                         'MAX'   : -1e100,
                         'SSQ'   : 0,
                       }

def clearGroupIndex():
    #GROUP_INDEX = {}
    for key in GROUP_INDEX.keys():
        del GROUP_INDEX[key]

def getValueFromGroupIndex(GroupIndexKey,FunctionName):
    # If no such index_key, create it and return a value
    if FunctionName not in GROUP_FUNCTION_NAMES:
       raise Exception('No such function \"%s\"' % FunctionName)
    # In the case if NewRowObjectDefault is requested
    if not GroupIndexKey:
       return GROUP_FUNCTION_NAMES[FunctionName]
    if FunctionName not in GROUP_INDEX[GroupIndexKey]['FUNCTIONS']:
       GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName] = {}
       GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName]['FLAG'] = True
       GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName]['VALUE'] = \
         GROUP_FUNCTION_NAMES[FunctionName]
    return GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName]['VALUE']

def setValueToGroupIndex(GroupIndexKey,FunctionName,Value):
    GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName]['VALUE'] = Value

GROUP_DESC = {}
def initializeGroup(GroupIndexKey):
    if GroupIndexKey not in GROUP_INDEX:
        print('GROUP_DESC[COUNT]='+str(GROUP_DESC['COUNT']))
        GROUP_INDEX[GroupIndexKey] = {}
        GROUP_INDEX[GroupIndexKey]['FUNCTIONS'] = {}
        GROUP_INDEX[GroupIndexKey]['ROWID'] = len(GROUP_INDEX) - 1
    for FunctionName in GROUP_FUNCTION_NAMES:
        # initialize function flags (UpdateFlag)
        if FunctionName in GROUP_INDEX[GroupIndexKey]['FUNCTIONS']:
           GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName]['FLAG'] = True
    print('initializeGroup: GROUP_INDEX='+str(GROUP_INDEX))

def groupCOUNT(GroupIndexKey):
    FunctionName = 'COUNT'
    Value = getValueFromGroupIndex(GroupIndexKey,FunctionName)
    if GroupIndexKey:
       if GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName]['FLAG']:
          GROUP_INDEX[GroupIndexKey]['FUNCTIONS'][FunctionName]['FLAG'] = False
          Value = Value + 1
          setValueToGroupIndex(GroupIndexKey,FunctionName,Value)
    return Value

def groupSUM():
    pass

def groupMUL():
    pass

def groupAVG():
    pass

def groupMIN():
    pass

def groupMAX():
    pass

def groupSSQ():
    pass

OPERATORS = {\
# List
'LIST' : lambda args : operationLIST(args),
# And
'&' : lambda args : operationAND(args),
'&&' : lambda args : operationAND(args),
'AND' : lambda args : operationAND(args),
# Or
'|' : lambda args : operationOR(args),
'||' : lambda args : operationOR(args),
'OR' : lambda args : operationOR(args),
# Not
'!' : lambda args : operationNOT(args[0]),
'NOT' : lambda args : operationNOT(args[0]),
# Between
'RANGE' : lambda args : operationRANGE(args[0],args[1],args[2]),
'BETWEEN' : lambda args : operationRANGE(args[0],args[1],args[2]),
# Subset
'IN' : lambda args : operationSUBSET(args[0],args[1]),
'SUBSET': lambda args : operationSUBSET(args[0],args[1]),
# Less
'<' : lambda args : operationLESS(args),
'LESS' : lambda args : operationLESS(args),
'LT'  : lambda args : operationLESS(args),
# More
'>' : lambda args : operationMORE(args),
'MORE' : lambda args : operationMORE(args),
'MT'   : lambda args : operationMORE(args),
# Less or equal
'<=' : lambda args : operationLESSOREQUAL(args),
'LESSOREQUAL' : lambda args : operationLESSOREQUAL(args),
'LTE' : lambda args : operationLESSOREQUAL(args),
# More or equal
'>=' : lambda args : operationMOREOREQUAL(args),
'MOREOREQUAL' : lambda args : operationMOREOREQUAL(args),
'MTE' : lambda args : operationMOREOREQUAL(args),
# Equal
'=' : lambda args : operationEQUAL(args),
'==' : lambda args : operationEQUAL(args),
'EQ' : lambda args : operationEQUAL(args),
'EQUAL' : lambda args : operationEQUAL(args),
'EQUALS' : lambda args : operationEQUAL(args),
# Not equal
'!=' : lambda args : operationNOTEQUAL(args[0],args[1]),
'<>' : lambda args : operationNOTEQUAL(args[0],args[1]),
'~=' : lambda args : operationNOTEQUAL(args[0],args[1]),
'NE' : lambda args : operationNOTEQUAL(args[0],args[1]),
'NOTEQUAL' : lambda args : operationNOTEQUAL(args[0],args[1]),
# Plus
'+' : lambda args : operationSUM(args),
'SUM' : lambda args : operationSUM(args),
# Minus
'-' : lambda args : operationDIFF(args[0],args[1]),
'DIFF' : lambda args : operationDIFF(args[0],args[1]),
# Mul
'*' : lambda args : operationMUL(args),
'MUL' : lambda args : operationMUL(args),
# Div
'/' : lambda args : operationDIV(args[0],args[1]),
'DIV' : lambda args : operationDIV(args[0],args[1]),
# Regexp match
'MATCH' : lambda args : operationMATCH(args[0],args[1]),
'LIKE' : lambda args : operationMATCH(args[0],args[1]),
# Regexp search
'SEARCH' : lambda args : operationSEARCH(args[0],args[1]),
# Regexp findal
'FINDALL' : lambda args : operationFINDALL(args[0],args[1]),
# Group count
'COUNT' : lambda args : groupCOUNT(args[0]),
}
    
# new evaluateExpression function,
#  accounting for groups
"""
def evaluateExpression(root,VarDictionary,GroupIndexKey=None):
    # input = local tree root
    # XXX: this could be very slow due to passing
    #      every time VarDictionary as a parameter
    # Two special cases: 1) root=varname
    #                    2) root=list/tuple
    # These cases must be processed in a separate way
    if type(root) in set([list,tuple]):
       # root is not a leaf
       head = root[0].upper()
       # string constants are treated specially
       if head in set(['STR','STRING']): # one arg
          return operationSTR(root[1])
       elif head in set(['SET']):
          return operationSET(root[1])
       tail = root[1:]
       args = []
       # evaluate arguments recursively
       for element in tail: # resolve tree by recursion
           args.append(evaluateExpression(element,VarDictionary,GroupIndexKey))
       # call functions with evaluated arguments
       if head in set(['LIST']): # list arg
          return operationLIST(args)
       elif head in set(['&','&&','AND']): # many args 
          return operationAND(args)
       elif head in set(['|','||','OR']): # many args
          return operationOR(args)
       elif head in set(['!','NOT']): # one args
          return operationNOT(args[0])
       elif head in set(['RANGE','BETWEEN']): # three args
          return operationRANGE(args[0],args[1],args[2])
       elif head in set(['IN','SUBSET']): # two args
          return operationSUBSET(args[0],args[1])
       elif head in set(['<','LESS','LT']): # many args
          return operationLESS(args)
       elif head in set(['>','MORE','MT']): # many args
          return operationMORE(args)
       elif head in set(['<=','LESSOREQUAL','LTE']): # many args
          return operationLESSOREQUAL(args)
       elif head in set(['>=','MOREOREQUAL','MTE']): # many args
          return operationMOREOREQUAL(args)
       elif head in set(['=','==','EQ','EQUAL','EQUALS']): # many args
          return operationEQUAL(args)
       elif head in set(['!=','<>','~=','NE','NOTEQUAL']): # two args
          return operationNOTEQUAL(args[0],args[1])
       elif head in set(['+','SUM']): # many args
          return operationSUM(args)
       elif head in set(['-','DIFF']): # two args
          return operationDIFF(args[0],args[1])
       elif head in set(['*','MUL']): # many args
          return operationMUL(args)
       elif head in set(['/','DIV']): # two args
          return operationDIV(args[0],args[1])
       elif head in set(['MATCH','LIKE']): # two args
          return operationMATCH(args[0],args[1])
       elif head in set(['SEARCH']): # two args
          return operationSEARCH(args[0],args[1])
       elif head in set(['FINDALL']): # two args
          return operationFINDALL(args[0],args[1])
       # --- GROUPING OPERATIONS ---
       elif head in set(['COUNT']):
          return groupCOUNT(GroupIndexKey)
       else:
          raise Exception('Unknown operator: %s' % root[0])
    elif type(root)==str:
       # root is a par_name
       return VarDictionary[root]
    else: 
       # root is a non-string constant
       return root
"""

def evaluateExpression(root,VarDictionary,GroupIndexKey=None):
    # input = local tree root
    # XXX: this could be very slow due to passing
    #      every time VarDictionary as a parameter
    # Two special cases: 1) root=varname
    #                    2) root=list/tuple
    # These cases must be processed in a separate way
    if type(root) in set([list,tuple]):
        # root is not a leaf
        head = root[0].upper()
        # string constants are treated specially
        if head in set(['STR','STRING']): # one arg
            return operationSTR(root[1])
        elif head in set(['SET']):
            return operationSET(root[1])
        tail = root[1:]
        args = []
        # evaluate arguments recursively
        for element in tail: # resolve tree by recursion
            args.append(evaluateExpression(element,VarDictionary,GroupIndexKey))
        # call functions with evaluated arguments
        try:
            return OPERATORS[head](args)
        except KeyError:
            raise Exception('Unknown operator: %s' % head)
    elif type(root)==str:
       # root is a par_name
       return VarDictionary[root]
    else: 
       # root is a non-string constant
       return root

def getVarDictionary(RowObject):
    # get VarDict from RowObject
    # VarDict: par_name => par_value
    VarDictionary = {}
    for par_name,par_value,par_format in RowObject:
        VarDictionary[par_name] = par_value
    return VarDictionary

def checkRowObject(RowObject,Conditions,VarDictionary):
    #VarDictionary = getVarDictionary(RowObject)   
    if Conditions:
       Flag = evaluateExpression(Conditions,VarDictionary)
    else:
       Flag=True
    return Flag

# ----------------------------------------------------
# /CONDITIONS
# ----------------------------------------------------


# ----------------------------------------------------
# PARAMETER NAMES (includeing creation of new ones)
# ----------------------------------------------------

# Bind an expression to a new parameter
#   in a form: ('BIND','new_par',('some_exp',...))
def operationBIND(parname,Expression,VarDictionary):
    pass

# This section is for more detailed processing of parlists. 

# Table creation must include not only subsets of 
#   existing parameters, but also new parameters
#   derived from functions on a special prefix language
# For this reason subsetOfRowObject(..) must be substituted
#   by newRowObject(ParameterNames,RowObject)

# For parsing use the function evaluateExpression

# Get names from expression.
#  Must merge this one with evaluateExrpression.
# This is VERY LIMITED version of what will be 
#  when make the language parser is implemented.
# For more ideas and info see LANGUAGE_REFERENCE

# more advansed version of expression evaluator
def evaluateExpressionPAR(ParameterNames,VarDictionary=None):
    # RETURN: 1) Upper-level Expression names
    #         2) Upper-level Expression values
    # Is it reasonable to pass a Context to every parse function?
    # For now the function does the following:
    #   1) iterates through all UPPER-LEVEL list elements
    #   2) if element is a par name: return par name
    #      if element is an BIND expression: return bind name
    #              (see operationBIND)
    #   3) if element is an anonymous expression: return #N(=1,2,3...)
    # N.B. Binds can be only on the 0-th level of Expression    
    pass

def getContextFormat(RowObject):
    # Get context format from the whole RowObject
    ContextFormat = {}
    for par_name,par_value,par_format in RowObject:
        ContextFormat[par_name] = par_format
    return ContextFormat

def getDefaultFormat(Type):
    if Type is int:
       return '%10d'
    elif Type is float:
       return '%25.15E'
    elif Type is str:
       return '%20s'
    elif Type is bool:
       return '%2d'
    else:
       raise Exception('Unknown type')
     
def getDefaultValue(Type):
    if Type is int:
       return 0
    elif Type is float:
       return 0.0
    elif Type is str:
       return ''
    elif Type is bool:
       return False
    else:
       raise Exception('Unknown type')

# VarDictionary = Context (this name is more suitable)

# GroupIndexKey is a key to special structure/dictionary GROUP_INDEX.
# GROUP_INDEX contains information needed to calculate streamed group functions
#  such as COUNT, AVG, MIN, MAX etc...

def newRowObject(ParameterNames,RowObject,VarDictionary,ContextFormat,GroupIndexKey=None):
    # Return a subset of RowObject according to 
    # ParameterNames include either par names
    #  or expressions containing par names literals
    # ContextFormat contains format for ParNames
    anoncount = 0
    RowObjectNew = []
    for expr in ParameterNames:
        if type(expr) in set([list,tuple]): # bind
           head = expr[0]
           if head in set(['let','bind','LET','BIND']):
              par_name = expr[1]
              par_expr = expr[2]
           else:
              par_name = "#%d" % anoncount
              anoncount += 1
              par_expr = expr
           par_value = evaluateExpression(par_expr,VarDictionary,GroupIndexKey)
           try:
              par_format = expr[3]
           except:
              par_format = getDefaultFormat(type(par_value))
        else: # parname
           par_name = expr
           par_value = VarDictionary[par_name]
           par_format = ContextFormat[par_name]
        RowObjectNew.append((par_name,par_value,par_format))
    return RowObjectNew

# ----------------------------------------------------
# /PARAMETER NAMES
# ----------------------------------------------------


# ----------------------------------------------------
# OPERATIONS ON TABLES
# ----------------------------------------------------

QUERY_BUFFER = '__BUFFER__'

def getTableList():
    return LOCAL_TABLE_CACHE.keys()

def describeTable(TableName):
    """
    INPUT PARAMETERS: 
        TableName: name of the table to describe
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Print information about table, including 
        parameter names, formats and wavenumber range.
    ---
    EXAMPLE OF USAGE:
        describeTable('sampletab')
    ---
    """
    print('-----------------------------------------')
    print(TableName+' summary:')
    try:
       print('-----------------------------------------')
       print('Comment: \n'+LOCAL_TABLE_CACHE[TableName]['header']['comment'])
    except:
       pass
    print('Number of rows: '+str(LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']))
    print('Table type: '+str(LOCAL_TABLE_CACHE[TableName]['header']['table_type']))
    print('-----------------------------------------')
    print('            PAR_NAME           PAR_FORMAT')
    print('')
    for par_name in LOCAL_TABLE_CACHE[TableName]['header']['order']:
        par_format = LOCAL_TABLE_CACHE[TableName]['header']['format'][par_name]
        print('%20s %20s' % (par_name,par_format))
    print('-----------------------------------------')

# Write a table to File or STDOUT
def outputTable(TableName,Conditions=None,File=None,Header=True):
    # Display or record table with condition checking
    if File:
       Header = False
       OutputFile = open(File,'w')
    if Header:
       headstr = putTableHeaderToString(TableName)
       if File:
          OutputFile.write(headstr)
       else:
          print(headstr)
    for RowID in range(0,LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']):
        RowObject = getRowObject(RowID,TableName)
        VarDictionary = getVarDictionary(RowObject)
        VarDictionary['LineNumber'] = RowID
        if not checkRowObject(RowObject,Conditions,VarDictionary):
           continue
        raw_string = putRowObjectToString(RowObject)
        if File:
           OutputFile.write(raw_string+'\n')
        else:
           print(raw_string)

# Create table "prototype-based" way
def createTable(TableName,RowObjectDefault):
    # create a Table based on a RowObjectDefault
    LOCAL_TABLE_CACHE[TableName] = {}
    header_order = []
    header_format = {}
    header_default = {}
    data = {}
    for par_name,par_value,par_format in RowObjectDefault:
        header_order.append(par_name)
        header_format[par_name] = par_format
        header_default[par_name] = par_value
        data[par_name] = []
    #header_order = tuple(header_order) # XXX ?
    LOCAL_TABLE_CACHE[TableName]['header']={}
    LOCAL_TABLE_CACHE[TableName]['header']['order'] = header_order 
    LOCAL_TABLE_CACHE[TableName]['header']['format'] = header_format
    LOCAL_TABLE_CACHE[TableName]['header']['default'] = header_default
    LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows'] = 0
    LOCAL_TABLE_CACHE[TableName]['header']['size_in_bytes'] = 0
    LOCAL_TABLE_CACHE[TableName]['header']['table_name'] = TableName
    LOCAL_TABLE_CACHE[TableName]['header']['table_type'] = 'column-fixed'
    LOCAL_TABLE_CACHE[TableName]['data'] = data
    

# simple "drop table" capability
def dropTable(TableName):
    """
    INPUT PARAMETERS: 
        TableName:  name of the table to delete
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Deletes a table from local database.
    ---
    EXAMPLE OF USAGE:
        dropTable('some_dummy_table')
    ---
    """
    # delete Table from both Cache and Storage
    try:
       #LOCAL_TABLE_CACHE[TableName] = {}
       del LOCAL_TABLE_CACHE[TableName]
    except:
       pass
    # delete from storage
    pass # TODO

# Returns a column corresponding to parameter name
def getColumn(TableName,ParameterName):
    """
    INPUT PARAMETERS: 
        TableName:      source table name     (required)
        ParameterName:  name of column to get (required)
    OUTPUT PARAMETERS: 
        ColumnData:     list of values from specified column 
    ---
    DESCRIPTION:
        Returns a column with a name ParameterName from
        table TableName. Column is returned as a list of values.
    ---
    EXAMPLE OF USAGE:
        p1 = getColumn('sampletab','p1')
    ---
    """
    return LOCAL_TABLE_CACHE[TableName]['data'][ParameterName]

# Returns a list of columns corresponding to parameter names
def getColumns(TableName,ParameterNames):
    """
    INPUT PARAMETERS: 
        TableName:       source table name           (required)
        ParameterNames:  list of column names to get (required)
    OUTPUT PARAMETERS: 
        ListColumnData:   tuple of lists of values from specified column 
    ---
    DESCRIPTION:
        Returns columns with a names in ParameterNames from
        table TableName. Columns are returned as a tuple of lists.
    ---
    EXAMPLE OF USAGE:
        p1,p2,p3 = getColumns('sampletab',('p1','p2','p3'))
    ---
    """
    Columns = []
    for par_name in ParameterNames:
        Columns.append(LOCAL_TABLE_CACHE[TableName]['data'][par_name])
    return Columns

def addColumn(TableName,ParameterName,Before=None,Expression=None,Type=None,Default=None,Format=None):
    if ParameterName in LOCAL_TABLE_CACHE[TableName]['header']['format']:
       raise Exception('Column \"%s\" already exists' % ParameterName)
    if not Type: Type = float
    if not Default: Default = getDefaultValue(Type)
    if not Format: Format = getDefaultFormat(Type)
    number_of_rows = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
    # Mess with data
    if not Expression:
       LOCAL_TABLE_CACHE[TableName]['data'][ParameterName]=[Default for i in range(0,number_of_rows)]
    else:
       data = []
       for RowID in range(0,number_of_rows):
           RowObject = getRowObject(RowID,TableName)
           VarDictionary = getVarDictionary(RowObject)
           VarDictionary['LineNumber'] = RowID
           par_value = evaluateExpression(Expression,VarDictionary)
           data.append(par_value)
           LOCAL_TABLE_CACHE[TableName]['data'][ParameterName] = data
    # Mess with header
    header_order = LOCAL_TABLE_CACHE[TableName]['header']['order']
    if not Before: 
       header_order.append(ParameterName)
    else:
       #i = 0
       #for par_name in header_order:
       #    if par_name == Before: break
       #    i += 1
       i = header_order.index(Before)
       header_order = header_order[:i] + [ParameterName,] + header_order[i:]
    LOCAL_TABLE_CACHE[TableName]['header']['order'] = header_order
    LOCAL_TABLE_CACHE[TableName]['header']['format'][ParameterName] = Format
    LOCAL_TABLE_CACHE[TableName]['header']['default'][ParameterName] = Default
   

def deleteColumn(TableName,ParameterName):
    if ParameterName not in LOCAL_TABLE_CACHE[TableName]['header']['format']:
       raise Exception('No such column \"%s\"' % ParameterName)
    # Mess with data
    i = LOCAL_TABLE_CACHE[TableName]['header']['order'].index(ParameterName)
    del LOCAL_TABLE_CACHE[TableName]['header']['order'][i]
    del LOCAL_TABLE_CACHE[TableName]['header']['format'][ParameterName]
    del LOCAL_TABLE_CACHE[TableName]['header']['default'][ParameterName]
    if not LOCAL_TABLE_CACHE[TableName]['header']['order']:
       LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows'] = 0
    # Mess with header
    del LOCAL_TABLE_CACHE[TableName]['data'][ParameterName]

def deleteColumns(TableName,ParameterNames):
    if type(ParameterNames) not in set([list,tuple,set]):
       ParameterNames = [ParameterNames]
    for ParameterName in ParameterNames:
        deleteColumn(TableName,ParameterName)

def renameColumn(TableName,OldParameterName,NewParameterName):
    pass

def insertRow():
    pass

def deleteRows(TableName,ParameterNames,Conditions):
    pass

# select from table to another table
def selectInto(DestinationTableName,TableName,ParameterNames,Conditions):
    # TableName must refer to an existing table in cache!!
    # Conditions = Restrictables in specific format
    # Sample conditions: cond = {'par1':{'range',[b_lo,b_hi]},'par2':b}
    # return structure similar to TableObject and put it to QUERY_BUFFER
    # if ParameterNames is '*' then all parameters are used
    #table_columns = LOCAL_TABLE_CACHE[TableName]['data'].keys()
    #table_length = len(TableObject['header']['number_of_rows'])
    #if ParameterNames=='*':
    #   ParameterNames = table_columns
    # check if Conditions contain elements which are not in the TableObject
    #condition_variables = getConditionVariables(Conditions)
    #strange_pars = set(condition_variables)-set(table_variables)
    #if strange_pars: 
    #   raise Exception('The following parameters are not in the table \"%s\"' % (TableName,list(strange_pars)))
    # do full scan each time
    if DestinationTableName == TableName:
       raise Exception('Selecting into source table is forbidden')
    table_length = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
    row_count = 0
    for RowID in range(0,table_length):
        RowObject = getRowObject(RowID,TableName)
        VarDictionary = getVarDictionary(RowObject)
        VarDictionary['LineNumber'] = RowID
        ContextFormat = getContextFormat(RowObject)
        RowObjectNew = newRowObject(ParameterNames,RowObject,VarDictionary,ContextFormat)
        if checkRowObject(RowObject,Conditions,VarDictionary):
           addRowObject(RowObjectNew,DestinationTableName)
           row_count += 1
    LOCAL_TABLE_CACHE[DestinationTableName]['header']['number_of_rows'] += row_count

def length(TableName):
    tab_len = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
    #print(str(tab_len)+' rows in '+TableName)
    return tab_len

# Select parameters from a table with certain conditions.
# Parameters can be the names or expressions.
# Conditions contain a list of expressions in a special language.
# Set Output to False to suppress output
# Set File=FileName to redirect output to a file.
def select(TableName,DestinationTableName=QUERY_BUFFER,ParameterNames=None,Conditions=None,Output=True,File=None):
    """
    INPUT PARAMETERS: 
        TableName:            name of source table              (required)
        DestinationTableName: name of resulting table           (optional)
        ParameterNames:       list of parameters or expressions (optional)
        Conditions:           list of logincal expressions      (optional)
        Output:   enable (True) or suppress (False) text output (optional)
        File:     enable (True) or suppress (False) file output (optional)
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Select or filter the data in some table 
        either to standard output or to file (if specified)
    ---
    EXAMPLE OF USAGE:
        select('sampletab',DestinationTableName='outtab',ParameterNames=(p1,p2),
                Conditions=(('and',('>=','p1',1),('<',('*','p1','p2'),20))))
        Conditions means (p1>=1 and p1*p2<20)
    ---
    """
    # TODO: Variables defined in ParameterNames ('LET') MUST BE VISIBLE IN Conditions !!
    # check if table exists
    if TableName not in LOCAL_TABLE_CACHE.keys():
        raise Exception('%s: no such table. Check tableList() for more info.' % TableName)
    if not ParameterNames: ParameterNames=LOCAL_TABLE_CACHE[TableName]['header']['order']
    LOCAL_TABLE_CACHE[DestinationTableName] = {} # clear QUERY_BUFFER for the new result
    RowObjectDefault = getDefaultRowObject(TableName)
    VarDictionary = getVarDictionary(RowObjectDefault)
    ContextFormat = getContextFormat(RowObjectDefault)
    RowObjectDefaultNew = newRowObject(ParameterNames,RowObjectDefault,VarDictionary,ContextFormat)
    dropTable(DestinationTableName) # redundant
    createTable(DestinationTableName,RowObjectDefaultNew)
    selectInto(DestinationTableName,TableName,ParameterNames,Conditions)
    if DestinationTableName!=QUERY_BUFFER:
        if File: outputTable(DestinationTableName,File=File)
    elif Output:
        outputTable(DestinationTableName,File=File)

# SORTING ===========================================================

def arrangeTable(TableName,DestinationTableName=None,RowIDList=None):
    #print 'AT/'
    #print 'AT: RowIDList = '+str(RowIDList)
    # make a subset of table rows according to RowIDList
    if not DestinationTableName:
       DestinationTableName = TableName
    if DestinationTableName != TableName:
       dropTable(DestinationTableName)
       LOCAL_TABLE_CACHE[DestinationTableName]['header']=LOCAL_TABLE_CACHE[TableName]['header']
       LOCAL_TABLE_CACHE[DestinationTableName]['data']={}
    LOCAL_TABLE_CACHE[DestinationTableName]['header']['number_of_rows'] = len(RowIDList)
    #print 'AT: RowIDList = '+str(RowIDList)
    for par_name in LOCAL_TABLE_CACHE[DestinationTableName]['header']['order']:
        par_data = LOCAL_TABLE_CACHE[TableName]['data'][par_name]
        LOCAL_TABLE_CACHE[DestinationTableName]['data'][par_name] = [par_data[i] for i in RowIDList]
    
def compareLESS(RowObject1,RowObject2,ParameterNames):
    #print 'CL/'
    # arg1 and arg2 are RowObjects
    # Compare them according to ParameterNames
    # Simple validity check:
    #if len(arg1) != len(arg2):
    #   raise Exception('Arguments have different lengths')
    #RowObject1Subset = subsetOfRowObject(ParameterNames,RowObject1)
    #RowObject2Subset = subsetOfRowObject(ParameterNames,RowObject2)
    #return RowObject1Subset < RowObject2Subset
    row1 = []
    row2 = []
    #n = len(RowObject1)
    #for i in range(0,n):
    #    par_name1 = RowObject1[i][0]
    #    if par_name1 in ParameterNames:
    #       par_value1 = RowObject1[i][1]
    #       par_value2 = RowObject2[i][1]
    #       row1 += [par_value1]
    #       row2 += [par_value2]
    VarDictionary1 = getVarDictionary(RowObject1)
    VarDictionary2 = getVarDictionary(RowObject2)
    for par_name in ParameterNames:
        par_value1 = VarDictionary1[par_name]
        par_value2 = VarDictionary2[par_name]
        row1 += [par_value1]
        row2 += [par_value2]
    Flag = row1 < row2
    return Flag

def quickSort(index,TableName,ParameterNames,Accending=True):
    # ParameterNames: names of parameters which are
    #  taking part in the sorting
    if index == []:
       return []
    else:
       PivotID = index[0]
       Pivot = getRowObject(PivotID,TableName)
       lesser_index = []
       greater_index = [];
       for RowID in index[1:]:
           RowObject = getRowObject(RowID,TableName)           
           if compareLESS(RowObject,Pivot,ParameterNames): 
              lesser_index += [RowID]
           else:
              greater_index += [RowID]
       lesser = quickSort(lesser_index,TableName,ParameterNames,Accending)
       greater = quickSort(greater_index,TableName,ParameterNames,Accending)
       if Accending:
          return lesser + [PivotID] + greater
       else:
          return greater + [PivotID] + lesser

# Sorting must work well on the table itself!
def sort(TableName,DestinationTableName=None,ParameterNames=None,Accending=True,Output=False,File=None):
    """
    INPUT PARAMETERS: 
        TableName:                name of source table          (required)
        DestinationTableName:     name of resulting table       (optional)
        ParameterNames:       list of parameters or expressions to sort by    (optional)
        Accending:       sort in ascending (True) or descending (False) order (optional)
        Output:   enable (True) or suppress (False) text output (optional)
        File:     enable (True) or suppress (False) file output (optional)
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Sort a table by a list of it's parameters or expressions.
        The sorted table is saved in DestinationTableName (if specified).
    ---
    EXAMPLE OF USAGE:
        sort('sampletab',ParameterNames=(p1,('+',p1,p2)))
    ---
    """
    number_of_rows = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
    index = range(0,number_of_rows)
    if not DestinationTableName:
       DestinationTableName = TableName
    # if names are not provided use all parameters in sorting
    if not ParameterNames:
       ParameterNames = LOCAL_TABLE_CACHE[TableName]['header']['order']
    elif type(ParameterNames) not in set([list,tuple]):
       ParameterNames = [ParameterNames] # fix of stupid bug where ('p1',) != ('p1')
    index_sorted = quickSort(index,TableName,ParameterNames,Accending)
    arrangeTable(TableName,DestinationTableName,index_sorted)
    if Output:
       outputTable(DestinationTableName,File=File)

# /SORTING ==========================================================
    

# GROUPING ==========================================================

# GROUP_INDEX global auxiliary structure is a Dictionary,
#   which has the following properties:
#      1) Each key is a composite variable:
#          [array of values of ParameterNames variable
#           STREAM_UPDATE_FLAG]
#      2) Each value is an index in LOCAL_TABLE_CACHE[TableName]['data'][...],
#          corresponding to this key
#   STREAM_UPDATE_FLAG = TRUE if value in GROUP_INDEX needs updating
#                      = FALSE otherwise
#   If no grouping variables are specified (GroupParameterNames==None)
#    than the following key is used: "__GLOBAL__"


def group(TableName,DestinationTableName=QUERY_BUFFER,ParameterNames=None,GroupParameterNames=None,File=None,Output=True):
    """
    INPUT PARAMETERS: 
        TableName:                name of source table          (required)
        DestinationTableName:     name of resulting table       (optional)
        ParameterNames:       list of parameters or expressions to take       (optional)
        GroupParameterNames:  list of parameters or expressions to group by   (optional)
        Accending:       sort in ascending (True) or descending (False) order (optional)
        Output:   enable (True) or suppress (False) text output (optional)
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        none
    ---
    EXAMPLE OF USAGE:
        group('sampletab',ParameterNames=('p1',('sum','p2')),GroupParameterNames=('p1'))
        ... makes grouping by p1,p2. For each group it calculates sum of p2 values.
    ---
    """
    # Implements such functions as:
    # count,sum,avg,min,max,ssq etc...
    # 1) ParameterNames can contain group functions
    # 2) GroupParameterNames can't contain group functions
    # 3) If ParameterNames contains parameters defined by LET directive,
    #    it IS visible in the sub-context of GroupParameterNames
    # 4) Parameters defined in GroupParameterNames are NOT visible in ParameterNames
    # 5) ParameterNames variable represents the structure of the resulting table/collection
    # 6) GroupParameterNames can contain either par_names or expressions with par_names
    # Clear old GROUP_INDEX value
    clearGroupIndex()
    # Consistency check
    if TableName == DestinationTableName:
       raise Exception('TableName and DestinationTableName must be different')
    #if not ParameterNames: ParameterNames=LOCAL_TABLE_CACHE[TableName]['header']['order']
    # Prepare the new DestinationTable
    RowObjectDefault = getDefaultRowObject(TableName)
    VarDictionary = getVarDictionary(RowObjectDefault)
    ContextFormat = getContextFormat(RowObjectDefault)
    RowObjectDefaultNew = newRowObject(ParameterNames,RowObjectDefault,VarDictionary,ContextFormat)
    dropTable(DestinationTableName) # redundant
    createTable(DestinationTableName,RowObjectDefaultNew)
    # Loop through rows of source Table
    # On each iteration group functions update GROUP_INDEX (see description above)
    number_of_rows = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']   
    # STAGE 1: CREATE GROUPS
    print('LOOP:')
    for RowID in range(0,number_of_rows):
        print('--------------------------------')
        print('RowID='+str(RowID))
        RowObject = getRowObject(RowID,TableName) # RowObject from source table
        VarDictionary = getVarDictionary(RowObject)
        print('VarDictionary='+str(VarDictionary))
        # This is a trick which makes evaluateExpression function
        #   not consider first expression as an operation
        GroupParameterNames_ = ['LIST'] + list(GroupParameterNames)
        GroupIndexKey = evaluateExpression(GroupParameterNames_,VarDictionary)
        # List is an unhashable type in Python!
        GroupIndexKey = tuple(GroupIndexKey)       
        initializeGroup(GroupIndexKey)
        print('GROUP_INDEX='+str(GROUP_INDEX))
        ContextFormat = getContextFormat(RowObject)
        RowObjectNew = newRowObject(ParameterNames,RowObject,VarDictionary,ContextFormat,GroupIndexKey)
        RowIDGroup = GROUP_INDEX[GroupIndexKey]['ROWID']
        setRowObject(RowIDGroup,RowObjectNew,DestinationTableName)
    # Output result if required
    if Output and DestinationTableName==QUERY_BUFFER:
       outputTable(DestinationTableName,File=File)

# /GROUPING =========================================================

# EXTRACTING ========================================================

REGEX_INTEGER = '[+-]?\d+'
REGEX_STRING = '[^\s]+'
REGEX_FLOAT_F = '[+-]?\d*\.?\d+'
REGEX_FLOAT_E = '[+-]?\d*\.?\d+[eEfF]?[+-]?\d+' 

REGEX_INTEGER_FIXCOL = lambda n: '\d{%d}' % n
REGEX_STRING_FIXCOL = lambda n: '[^\s]{%d}' % n
REGEX_FLOAT_F_FIXCOL = lambda n: '[\+\-\.\d]{%d}' % n
REGEX_FLOAT_E_FIXCOL = lambda n: '[\+\-\.\deEfF]{%d}' % n

# Extract sub-columns from string column
def extractColumns(TableName,SourceParameterName,ParameterFormats,ParameterNames=None,FixCol=False):
    """
    INPUT PARAMETERS: 
        TableName:             name of source table              (required)
        SourceParameterName:   name of source column to process  (required)
        ParameterFormats:      c formats of unpacked parameters  (required)
        ParameterNames:        list of resulting parameter names (optional)
        FixCol:      column-fixed (True) format of source column (optional)
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Note, that this function is aimed to do some extra job on
        interpreting string parameters which is normally supposed
        to be done by the user.
    ---
    EXAMPLE OF USAGE:
        extractColumns('sampletab',SourceParameterName='p5',
                        ParameterFormats=('%d','%d','%d'),
                        ParameterNames=('p5_1','p5_2','p5_3'))
        This example extracts three integer parameters from
        a source column 'p5' and puts results in ('p5_1','p5_2','p5_3').
    ---
    """
    # ParameterNames = just the names without expressions
    # ParFormats contains python formats for par extraction
    # Example: ParameterNames=('v1','v2','v3')
    #          ParameterFormats=('%1s','%1s','%1s')
    # By default the format of parameters is column-fixed
    if type(LOCAL_TABLE_CACHE[TableName]['header']['default'][SourceParameterName]) not in set([str,unicode]):
       raise Exception('Source parameter must be a string')
    i=-1
    # bug when (a,) != (a)
    if ParameterNames and type(ParameterNames) not in set([list,tuple]):
       ParameterNames = [ParameterNames]
    if ParameterFormats and type(ParameterFormats) not in set([list,tuple]):
       ParameterFormats = [ParameterFormats]
    # if ParameterNames is empty, fill it with #1-2-3-...
    if not ParameterNames:
       ParameterNames = []
       # using naming convension #i, i=0,1,2,3...
       for par_format in ParameterFormats:
           while True:
                 i+=1
                 par_name = '#%d' % i
                 fmt = LOCAL_TABLE_CACHE[TableName]['header']['format'].get(par_name,None)
                 if not fmt: break
           ParameterNames.append(par_name)
    # check if ParameterNames are valid
    Intersection = set(ParameterNames).intersection(LOCAL_TABLE_CACHE[TableName]['header']['order'])
    if Intersection:
       raise Exception('Parameters %s already exist' % str(list(Intersection)))
    # loop over ParameterNames to prepare LOCAL_TABLE_CACHE
    i=0
    for par_name in ParameterNames:  
        par_format = ParameterFormats[i]     
        LOCAL_TABLE_CACHE[TableName]['header']['format'][par_name]=par_format
        LOCAL_TABLE_CACHE[TableName]['data'][par_name]=[] 
        i+=1
    # append new parameters in order list
    LOCAL_TABLE_CACHE[TableName]['header']['order'] += ParameterNames
    # cope with default values
    i=0
    format_regex = []
    format_types = []
    for par_format in ParameterFormats:
        par_name = ParameterNames[i]
        regex = FORMAT_PYTHON_REGEX
        (lng,trail,lngpnt,ty) = re.search(regex,par_format).groups()
        ty = ty.lower()
        if ty == 'd':
           par_type = int
           if FixCol:
              format_regex_part = REGEX_INTEGER_FIXCOL(lng)
           else:
              format_regex_part = REGEX_INTEGER
        elif ty == 's':
           par_type = str
           if FixCol:
              format_regex_part = REGEX_STRING_FIXCOL(lng)
           else:
              format_regex_part = REGEX_STRING
        elif ty == 'f':
           par_type = float
           if FixCol:
              format_regex_part = REGEX_FLOAT_F_FIXCOL(lng)
           else:
              format_regex_part = REGEX_FLOAT_F
        elif ty == 'e':
           par_type = float
           if FixCol:
              format_regex_part = REGEX_FLOAT_E_FIXCOL(lng)
           else:
              format_regex_part = REGEX_FLOAT_E
        else:
           raise Exception('Unknown data type')
        format_regex.append('('+format_regex_part+')')
        format_types.append(par_type)
        def_val = getDefaultValue(par_type)
        LOCAL_TABLE_CACHE[TableName]['header']['default'][par_name]=def_val
        i+=1
    format_regex = '\s*'.join(format_regex)
    # loop through values of SourceParameter
    for SourceParameterString in LOCAL_TABLE_CACHE[TableName]['data'][SourceParameterName]:
        try:
           ExtractedValues = list(re.search(format_regex,SourceParameterString).groups())
        except:
           raise Exception('Error with line \"%s\"' % SourceParameterString)
        i=0
        # loop through all parameters which are supposed to be extracted
        for par_name in ParameterNames:
            par_value = format_types[i](ExtractedValues[i])
            LOCAL_TABLE_CACHE[TableName]['data'][par_name].append(par_value)
            i+=1
    # explicitly check that number of rows are equal
    number_of_rows = LOCAL_TABLE_CACHE[TableName]['header']['number_of_rows']
    number_of_rows2 = len(LOCAL_TABLE_CACHE[TableName]['data'][SourceParameterName])
    number_of_rows3 = len(LOCAL_TABLE_CACHE[TableName]['data'][ParameterNames[0]])
    if not (number_of_rows == number_of_rows2 == number_of_rows3):
       raise Exception('Error while extracting parameters: check your regexp')

# Split string columns into sub-columns with given names
def splitColumn(TableName,SourceParameterName,ParameterNames,Splitter):
    pass

# /EXTRACTING =======================================================

# ---------------------------------------------------------------
# ---------------------------------------------------------------
# /LOCAL DATABASE MANAGEMENT SYSTEM
# ---------------------------------------------------------------
# ---------------------------------------------------------------


# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# GLOBAL API FUNCTIONS
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------

def mergeParlist(*arg):
    # Merge parlists and remove duplicates.
    # Argument contains a list of lists/tuples.
    container = []
    for a in arg:
        container += list(a)
    result = []
    index = set()
    for par_name in container:
        if par_name not in index:
            index.add(par_name)
            result.append(par_name)
    return result

# Define parameter groups to simplify the usage of fetch_
# "Long term" core version includes templates for the Parlists instead of listing the broadeners explicitly.
# Each template parameter has '%s' in place of a broadener, i.e. 'gamma_%s' instead of 'gamma_air' 

# ======================================================
# CODE FOR GENERATING PARAMETER NAMES FOR LINE PROFILES
# NAME: GENERATE_PARLIST
# NOTE: THIS CODE DOESN'T COVER NON-PROFILE PARAMETERS 
#       SUCH AS QUANTA, LOCAL AND GLOBAL IDS ETC...
# NOTE2: THIS CODE DOESN'T GUARANTEE THAT THE GENERATED
#        PARAMETER NAMES WILL EXIST IN THE HITRANONLINE 
#        DATABASE!
#        TO GET THE REAL PARAMETER NAMES PLEASE EITHER
#        USE THE EXTENSION OF THE HITRAN APPLICATION
#        PROGRAMMING INTERFACE:
#             (http://github.org/hitranonline/hapi2) ...
#        ... OR LOOK FOR THE CUSTOM USER FORMAT WEB PAGE
#        ON HITRANONLINE WEBSITE (http://hitran.org).
# ======================================================

VOIGT_PROFILE_TEMPLATE = ['gamma_%s','n_%s','delta_%s','deltap_%s']

SDVOIGT_PROFILE_TEMPLATE = [
    'gamma_SDV_0_%s_%d','n_SDV_%s_%d', # HWHM AND ITS T-DEPENDENCE
    'gamma_SDV_2_%s_%d','n_gamma_SDV_2_%s_%d', # SPEED-DEPENDENCE OF HWHM AND ITS T-DEPENDENCE
    'delta_SDV_%s_%d','deltap_SDV_%s_%d', # SHIFT AND ITS T-DEPENDENCE
    'SD_%s' # UNITLESS SDV PARAMETER
    ]

HT_PROFILE_TEMPLATE = [
    'gamma_HT_0_%s_%d','n_HT_%s_%d', # HWHM AND ITS T-DEPENDENCE
    'gamma_HT_2_%s_%d','n_gamma_HT_2_%s_%d', # SPEED-DEPENDENCE OF HWHM AND ITS T-DEPENDENCE
    'delta_HT_%s_%d','deltap_HT_%s_%d', # SHIFT AND ITS T-DEPENDENCE
    ]

def apply_env(template,broadener,Tref):
    args = []
    if '%s' in template:
        args.append(broadener)
    if '%d'  in template:
        args.append(Tref)
    return template%tuple(args)

def generate_parlist(profile,broadener,Tref):
    PROFILE_MAP = {
        'voigt': VOIGT_PROFILE_TEMPLATE,
        'vp': VOIGT_PROFILE_TEMPLATE,
        'sdvoigt': SDVOIGT_PROFILE_TEMPLATE,
        'sdvp': SDVOIGT_PROFILE_TEMPLATE,
        'ht': HT_PROFILE_TEMPLATE,
        'htp': HT_PROFILE_TEMPLATE,
    }
    return [apply_env(template,broadener,Tref) \
        for template in PROFILE_MAP[profile.lower()]] 
    
# generate_parlist('Voigt','air',296)  =>   gamma_air,
    
# ====================================================================        
# PARLISTS FOR EACH BROADENER EXPLICITLY (FOR BACKWARDS COMPATIBILITY)
# ====================================================================        

# Define parameter groups to simplify the usage of fetch_
PARLIST_DOTPAR = ['par_line',]
PARLIST_ID = ['trans_id',]
PARLIST_STANDARD = ['molec_id','local_iso_id','nu','sw','a','elower','gamma_air',
                    'delta_air','gamma_self','n_air','n_self','gp','gpp']
PARLIST_LABELS = ['statep','statepp']
#PARLIST_LINEMIXING = ['y_air','y_self']

PARLIST_VOIGT_AIR = ['gamma_air','delta_air','deltap_air','n_air']
PARLIST_VOIGT_SELF = ['gamma_self','delta_self','deltap_self','n_self']
PARLIST_VOIGT_H2 = ['gamma_H2','delta_H2','deltap_H2','n_H2']
PARLIST_VOIGT_CO2 = ['gamma_CO2','delta_CO2','n_CO2']
PARLIST_VOIGT_HE = ['gamma_He','delta_He','n_He']
PARLIST_VOIGT_H2O = ['gamma_H2O','n_H2O']
PARLIST_VOIGT_LINEMIXING_AIR = ['y_air']
PARLIST_VOIGT_LINEMIXING_SELF = ['y_self']
PARLIST_VOIGT_LINEMIXING_ALL = mergeParlist(PARLIST_VOIGT_LINEMIXING_AIR,
                                            PARLIST_VOIGT_LINEMIXING_SELF)
PARLIST_VOIGT_ALL = mergeParlist(PARLIST_VOIGT_AIR,PARLIST_VOIGT_SELF,
                                 PARLIST_VOIGT_H2,PARLIST_VOIGT_CO2,
                                 PARLIST_VOIGT_HE,PARLIST_VOIGT_H2O,
                                 PARLIST_VOIGT_LINEMIXING_ALL)

#PARLIST_SDVOIGT_AIR = ['gamma_air','delta_air','deltap_air','n_air','SD_air']
#PARLIST_SDVOIGT_AIR = ['gamma_SDV_0_air_296','n_SDV_air_296',
#                       'gamma_SDV_2_air_296','n_gamma_SDV_2_air_296', # n_SDV_2_air_296 ?
PARLIST_SDVOIGT_AIR = ['gamma_SDV_0_air_296',  # don't include temperature exponents while they are absent in the database
                       'gamma_SDV_2_air_296',  # don't include temperature exponents while they are absent in the database
                       'delta_SDV_0_air_296','deltap_SDV_air_296','SD_air']
#PARLIST_SDVOIGT_SELF = ['gamma_self','delta_self','deltap_self','n_self','SD_self']
#PARLIST_SDVOIGT_SELF = ['gamma_SDV_0_self_296','n_SDV_self_296',
#                       'gamma_SDV_2_self_296','n_gamma_SDV_2_self_296', # n_SDV_2_self_296 ?
PARLIST_SDVOIGT_SELF = ['gamma_SDV_0_self_296', # don't include temperature exponents while they are absent in the database
                       'gamma_SDV_2_self_296',  # don't include temperature exponents while they are absent in the database
                       'delta_SDV_0_self_296','deltap_SDV_self_296','SD_self']
PARLIST_SDVOIGT_H2 = []
PARLIST_SDVOIGT_CO2 = []
PARLIST_SDVOIGT_HE = []
#PARLIST_SDVOIGT_LINEMIXING_AIR = ['Y_SDV_air_296','n_Y_SDV_air_296']
PARLIST_SDVOIGT_LINEMIXING_AIR = ['Y_SDV_air_296'] # don't include temperature exponents while they are absent in the database
#PARLIST_SDVOIGT_LINEMIXING_SELF = ['Y_SDV_self_296','n_Y_SDV_self_296']
PARLIST_SDVOIGT_LINEMIXING_SELF = ['Y_SDV_self_296'] # don't include temperature exponents while they are absent in the database
PARLIST_SDVOIGT_LINEMIXING_ALL = mergeParlist(PARLIST_SDVOIGT_LINEMIXING_AIR,
                                              PARLIST_SDVOIGT_LINEMIXING_SELF)
PARLIST_SDVOIGT_ALL = mergeParlist(PARLIST_SDVOIGT_AIR,PARLIST_SDVOIGT_SELF,
                                   PARLIST_SDVOIGT_H2,PARLIST_SDVOIGT_CO2,
                                   PARLIST_SDVOIGT_HE,PARLIST_SDVOIGT_LINEMIXING_ALL)

PARLIST_GALATRY_AIR = ['gamma_air','delta_air','deltap_air','n_air','beta_g_air']
PARLIST_GALATRY_SELF = ['gamma_self','delta_self','deltap_self','n_self','beta_g_self']
PARLIST_GALATRY_H2 = []
PARLIST_GALATRY_CO2 = []
PARLIST_GALATRY_HE = []
PARLIST_GALATRY_ALL = mergeParlist(PARLIST_GALATRY_AIR,PARLIST_GALATRY_SELF,
                                   PARLIST_GALATRY_H2,PARLIST_GALATRY_CO2,
                                   PARLIST_GALATRY_HE)

PARLIST_HT_SELF = ['gamma_HT_0_self_50','n_HT_self_50','gamma_HT_2_self_50',
                   'delta_HT_0_self_50','deltap_HT_self_50','delta_HT_2_self_50',
                   'gamma_HT_0_self_150','n_HT_self_150','gamma_HT_2_self_150',
                   'delta_HT_0_self_150','deltap_HT_self_150','delta_HT_2_self_150',
                   'gamma_HT_0_self_296','n_HT_self_296','gamma_HT_2_self_296',
                   'delta_HT_0_self_296','deltap_HT_self_296','delta_HT_2_self_296',
                   'gamma_HT_0_self_700','n_HT_self_700','gamma_HT_2_self_700',
                   'delta_HT_0_self_700','deltap_HT_self_700','delta_HT_2_self_700',
                   'nu_HT_self','kappa_HT_self','eta_HT_self','Y_HT_self_296']
#PARLIST_HT_AIR = ['gamma_HT_0_air_50','n_HT_air_50','gamma_HT_2_air_50',
#                  'delta_HT_0_air_50','deltap_HT_air_50','delta_HT_2_air_50',
#                  'gamma_HT_0_air_150','n_HT_air_150','gamma_HT_2_air_150',
#                  'delta_HT_0_air_150','deltap_HT_air_150','delta_HT_2_air_150',
#                  'gamma_HT_0_air_296','n_HT_air_296','gamma_HT_2_air_296',
#                  'delta_HT_0_air_296','deltap_HT_air_296','delta_HT_2_air_296',
#                  'gamma_HT_0_air_700','n_HT_air_700','gamma_HT_2_air_700',
#                  'delta_HT_0_air_700','deltap_HT_air_700','delta_HT_2_air_700',
#                  'nu_HT_air','kappa_HT_air','eta_HT_air']
PARLIST_HT_AIR = ['gamma_HT_0_air_296','n_HT_air_296','gamma_HT_2_air_296',
                  'delta_HT_0_air_296','deltap_HT_air_296','delta_HT_2_air_296',
                  'nu_HT_air','kappa_HT_air','eta_HT_air','Y_HT_air_296']
PARLIST_HT_ALL = mergeParlist(PARLIST_HT_SELF,PARLIST_HT_AIR)
                                   
PARLIST_ALL = mergeParlist(PARLIST_ID,PARLIST_DOTPAR,PARLIST_STANDARD,
                           PARLIST_LABELS,PARLIST_VOIGT_ALL,
                           PARLIST_SDVOIGT_ALL,PARLIST_GALATRY_ALL,
                           PARLIST_HT_ALL)

# ====================================================================        
# PARLISTS FOR EACH BROADENER EXPLICITLY (FOR BACKWARDS COMPATIBILITY)
# ====================================================================        
                           
PARAMETER_GROUPS = {
  'par_line' : PARLIST_DOTPAR,
  '160-char' : PARLIST_DOTPAR,
  '.par' : PARLIST_DOTPAR,
  'id' : PARLIST_ID,
  'standard' : PARLIST_STANDARD,
  'labels' : PARLIST_LABELS,
  #'linemixing' : PARLIST_LINEMIXING,
  'voigt_air' : PARLIST_VOIGT_AIR,
  'voigt_self' : PARLIST_VOIGT_SELF,
  'voigt_h2' : PARLIST_VOIGT_H2,
  'voigt_co2' : PARLIST_VOIGT_CO2,
  'voigt_he' : PARLIST_VOIGT_HE,
  'voigt_h2o' : PARLIST_VOIGT_H2O,
  'voigt_linemixing_air': PARLIST_VOIGT_LINEMIXING_AIR,
  'voigt_linemixing_self': PARLIST_VOIGT_LINEMIXING_SELF,
  'voigt_linemixing': PARLIST_VOIGT_LINEMIXING_ALL,
  'voigt' : PARLIST_VOIGT_ALL,
  'sdvoigt_air' : PARLIST_SDVOIGT_AIR,
  'sdvoigt_self' : PARLIST_SDVOIGT_SELF,
  'sdvoigt_h2' : PARLIST_SDVOIGT_H2,
  'sdvoigt_co2' : PARLIST_SDVOIGT_CO2,
  'sdvoigt_he' : PARLIST_SDVOIGT_HE,
  'sdvoigt_linemixing_air': PARLIST_SDVOIGT_LINEMIXING_AIR,
  'sdvoigt_linemixing_self': PARLIST_SDVOIGT_LINEMIXING_SELF,
  'sdvoigt_linemixing': PARLIST_SDVOIGT_LINEMIXING_ALL,
  'sdvoigt' : PARLIST_SDVOIGT_ALL,
  'galatry_air' : PARLIST_GALATRY_AIR,
  'galatry_self' : PARLIST_GALATRY_SELF,
  'galatry_h2' : PARLIST_GALATRY_H2,
  'galatry_co2' : PARLIST_GALATRY_CO2,
  'galatry_he' : PARLIST_GALATRY_HE,
  'galatry' : PARLIST_GALATRY_ALL,
  'ht' : PARLIST_HT_ALL,
  'all' : PARLIST_ALL
}

def prepareParlist(pargroups=[],params=[],dotpar=True):
    # Apply defaults
    parlist_default = []
    if dotpar:
        parlist_default += ['par_line']
    #parlist_default += PARAMETER_GROUPS['id']
    
    # Make a dictionary of "assumed" parameters.
    ASSUMED_PARAMS = {}
    if 'par_line' in set(parlist_default):
        ASSUMED_PARAMS = HITRAN_DEFAULT_HEADER['format']
    
    parlist = parlist_default
    
    # Iterate over parameter groups.
    for pargroup in pargroups:
        pargroup = pargroup.lower()
        parlist += PARAMETER_GROUPS[pargroup]
        
    # Iterate over single parameters.
    for param in params:
        #param = param.lower()
        parlist.append(param)
        
    # Clean up parameter list.
    parlist = mergeParlist(parlist)
    result = []
    for param in parlist:
        if param not in ASSUMED_PARAMS:
            result.append(param)
    
    return result

def prepareHeader(parlist):
    HEADER = {'table_name':'','number_of_rows':-1,'format':{},
              'default':{},'table_type':'column-fixed',
              'size_in_bytes':-1,'order':[],'description':{}}
    
    # Add column-fixed 160-character part, if specified in parlist.
    if 'par_line' in set(parlist):
        HEADER['order'] = HITRAN_DEFAULT_HEADER['order']
        HEADER['format'] = HITRAN_DEFAULT_HEADER['format']
        HEADER['default'] = HITRAN_DEFAULT_HEADER['default']
        HEADER['description'] = HITRAN_DEFAULT_HEADER['description']
        HEADER['position'] = HITRAN_DEFAULT_HEADER['position']

    # Insert all other parameters in the "extra" section of the header.
    plist = [v for v in parlist if v!='par_line']
    HEADER['extra'] = []
    HEADER['extra_format'] = {}
    HEADER['extra_separator'] = ','
    for param in plist:
        param = param.lower()
        HEADER['extra'].append(param)
        HEADER['extra_format'][param] = PARAMETER_META[param]['default_fmt']
        
    return HEADER
        
def queryHITRAN(TableName,iso_id_list,numin,numax,pargroups=[],params=[],dotpar=True,head=False):
    ParameterList = prepareParlist(pargroups=pargroups,params=params,dotpar=dotpar)
    TableHeader = prepareHeader(ParameterList)
    TableHeader['table_name'] = TableName
    DataFileName = VARIABLES['BACKEND_DATABASE_NAME'] + '/' + TableName + '.data'
    HeaderFileName = VARIABLES['BACKEND_DATABASE_NAME'] + '/' + TableName + '.header'
    # create URL
    iso_id_list_str = [str(iso_id) for iso_id in iso_id_list]
    iso_id_list_str = ','.join(iso_id_list_str)
    print('\nData is fetched from %s\n'%VARIABLES['GLOBAL_HOST'])
    if pargroups or params: # custom par search
        url = VARIABLES['GLOBAL_HOST'] + '/lbl/api?' + \
        'iso_ids_list=' + iso_id_list_str + '&' + \
        'numin=' + str(numin) + '&' + \
        'numax=' + str(numax) + '&' + \
        'head=' + str(head) + '&' + \
        'fixwidth=0&sep=[comma]&' +\
        'request_params=' + ','.join(ParameterList)
    else: # old-fashioned .par search
        url = VARIABLES['GLOBAL_HOST'] + '/lbl/api?' + \
        'iso_ids_list=' + iso_id_list_str + '&' + \
        'numin=' + str(numin) + '&' + \
        'numax=' + str(numax)
    #raise Exception(url)
    # Download data by chunks.
    if VARIABLES['DISPLAY_FETCH_URL']: print(url+'\n')
    try:       
        # Proxy handling # https://stackoverflow.com/questions/1450132/proxy-with-urllib2
        if VARIABLES['PROXY']:
            print('Using proxy '+str(VARIABLES['PROXY']))
            proxy = urllib2.ProxyHandler(VARIABLES['PROXY'])
            opener = urllib2.build_opener(proxy)
            urllib2.install_opener(opener)            
        req = urllib2.urlopen(url)
    except urllib2.HTTPError:
        raise Exception('Failed to retrieve data for given parameters.')
    except urllib2.URLError:
        raise Exception('Cannot connect to %s. Try again or edit GLOBAL_HOST variable.' % GLOBAL_HOST)
    CHUNK = 64 * 1024
    print('BEGIN DOWNLOAD: '+TableName)
    with open_(DataFileName,'w') as fp:
       while True:
          chunk = req.read(CHUNK)
          if not chunk: break
          fp.write(chunk.decode('utf-8'))
          print('  %d bytes written to %s' % (CHUNK,DataFileName))
    with open(HeaderFileName,'w') as fp:       
       fp.write(json.dumps(TableHeader,indent=2))
       print('Header written to %s' % HeaderFileName)
    print('END DOWNLOAD')
    # Set comment
    # Get this table to LOCAL_TABLE_CACHE
    storage2cache(TableName)
    print('PROCESSED')

def saveHeader(TableName):
    ParameterList = prepareParlist(dotpar=True)    
    TableHeader = prepareHeader(ParameterList)
    with open(TableName+'.header','w') as fp:
       fp.write(json.dumps(TableHeader,indent=2))
    
# ---------- DATABASE FRONTEND END -------------

# simple implementation of getting a line list from a remote server
def getLinelist(local_name,query,api_key):
    return makeQuery(local_name)

# -------------------------------------------------------------------
# -------------------------------------------------------------------
# / GLOBABL API FUNCTIONS
# -------------------------------------------------------------------
# -------------------------------------------------------------------



# ---------------- FILTER ---------------------------------------------

def filter(TableName,Conditions):
    select(TableName=TableName,Conditions=Conditions,Output=False)

# ---------------------- ISO.PY ---------------------------------------

ISO_ID_INDEX = {

'M':0,
'I':1,
'iso_name':2,
'abundance':3,
'mass':4,
'mol_name':5

}

ISO_INDEX = {

'id':0,
'iso_name':1,
'abundance':2,
'mass':3,
'mol_name':4

}

#  M   I             id   iso_name                   abundance      mass           mol_name

ISO = {
(  1,  1 ):    [      1,  'H2(16O)',                 9.973173E-01,  1.801056E+01,  'H2O'         ], 
(  1,  2 ):    [      2,  'H2(18O)',                 1.999827E-03,  2.001481E+01,  'H2O'         ], 
(  1,  3 ):    [      3,  'H2(17O)',                 3.718841E-04,  1.901478E+01,  'H2O'         ], 
(  1,  4 ):    [      4,  'HD(16O)',                 3.106928E-04,  1.901674E+01,  'H2O'         ], 
(  1,  5 ):    [      5,  'HD(18O)',                 6.230031E-07,  2.102098E+01,  'H2O'         ], 
(  1,  6 ):    [      6,  'HD(17O)',                 1.158526E-07,  2.002096E+01,  'H2O'         ], 
(  1,  7 ):    [    129,  'D2(16O)',                 2.419741E-08,  2.002292E+01,  'H2O'         ], 
(  2,  1 ):    [      7,  '(12C)(16O)2',             9.842043E-01,  4.398983E+01,  'CO2'         ], 
(  2,  2 ):    [      8,  '(13C)(16O)2',             1.105736E-02,  4.499318E+01,  'CO2'         ], 
(  2,  3 ):    [      9,  '(16O)(12C)(18O)',         3.947066E-03,  4.599408E+01,  'CO2'         ], 
(  2,  4 ):    [     10,  '(16O)(12C)(17O)',         7.339890E-04,  4.499404E+01,  'CO2'         ], 
(  2,  5 ):    [     11,  '(16O)(13C)(18O)',         4.434456E-05,  4.699743E+01,  'CO2'         ], 
(  2,  6 ):    [     12,  '(16O)(13C)(17O)',         8.246233E-06,  4.599740E+01,  'CO2'         ], 
(  2,  7 ):    [     13,  '(12C)(18O)2',             3.957340E-06,  4.799832E+01,  'CO2'         ], 
(  2,  8 ):    [     14,  '(17O)(12C)(18O)',         1.471799E-06,  4.699829E+01,  'CO2'         ], 
(  2,  9 ):    [    121,  '(12C)(17O)2',             1.368466E-07,  4.599826E+01,  'CO2'         ], 
(  2, 10 ):    [     15,  '(13C)(18O)2',             4.446000E-08,  4.900167E+01,  'CO2'         ], 
(  2, 11 ):    [    120,  '(18O)(13C)(17O)',         1.653540E-08,  4.800165E+01,  'CO2'         ], 
(  2, 12 ):    [    122,  '(13C)(17O)2',             1.537446E-09,  4.700162E+01,  'CO2'         ], 
(  3,  1 ):    [     16,  '(16O)3',                  9.929009E-01,  4.798474E+01,  'O3'          ], 
(  3,  2 ):    [     17,  '(16O)(16O)(18O)',         3.981942E-03,  4.998899E+01,  'O3'          ], 
(  3,  3 ):    [     18,  '(16O)(18O)(16O)',         1.990971E-03,  4.998899E+01,  'O3'          ], 
(  3,  4 ):    [     19,  '(16O)(16O)(17O)',         7.404746E-04,  4.898896E+01,  'O3'          ], 
(  3,  5 ):    [     20,  '(16O)(17O)(16O)',         3.702373E-04,  4.898896E+01,  'O3'          ], 
(  4,  1 ):    [     21,  '(14N)2(16O)',             9.903328E-01,  4.400106E+01,  'N2O'         ], 
(  4,  2 ):    [     22,  '(14N)(15N)(16O)',         3.640926E-03,  4.499810E+01,  'N2O'         ], 
(  4,  3 ):    [     23,  '(15N)(14N)(16O)',         3.640926E-03,  4.499810E+01,  'N2O'         ], 
(  4,  4 ):    [     24,  '(14N)2(18O)',             1.985822E-03,  4.600531E+01,  'N2O'         ], 
(  4,  5 ):    [     25,  '(14N)2(17O)',             3.692797E-04,  4.500528E+01,  'N2O'         ], 
(  5,  1 ):    [     26,  '(12C)(16O)',              9.865444E-01,  2.799491E+01,  'CO'          ], 
(  5,  2 ):    [     27,  '(13C)(16O)',              1.108364E-02,  2.899827E+01,  'CO'          ], 
(  5,  3 ):    [     28,  '(12C)(18O)',              1.978224E-03,  2.999916E+01,  'CO'          ], 
(  5,  4 ):    [     29,  '(12C)(17O)',              3.678671E-04,  2.899913E+01,  'CO'          ], 
(  5,  5 ):    [     30,  '(13C)(18O)',              2.222500E-05,  3.100252E+01,  'CO'          ], 
(  5,  6 ):    [     31,  '(13C)(17O)',              4.132920E-06,  3.000249E+01,  'CO'          ], 
(  6,  1 ):    [     32,  '(12C)H4',                 9.882741E-01,  1.603130E+01,  'CH4'         ], 
(  6,  2 ):    [     33,  '(13C)H4',                 1.110308E-02,  1.703466E+01,  'CH4'         ], 
(  6,  3 ):    [     34,  '(12C)H3D',                6.157511E-04,  1.703748E+01,  'CH4'         ], 
(  6,  4 ):    [     35,  '(13C)H3D',                6.917852E-06,  1.804083E+01,  'CH4'         ], 
(  7,  1 ):    [     36,  '(16O)2',                  9.952616E-01,  3.198983E+01,  'O2'          ], 
(  7,  2 ):    [     37,  '(16O)(18O)',              3.991410E-03,  3.399408E+01,  'O2'          ], 
(  7,  3 ):    [     38,  '(16O)(17O)',              7.422352E-04,  3.299404E+01,  'O2'          ], 
(  8,  1 ):    [     39,  '(14N)(16O)',              9.939737E-01,  2.999799E+01,  'NO'          ], 
(  8,  2 ):    [     40,  '(15N)(16O)',              3.654311E-03,  3.099502E+01,  'NO'          ], 
(  8,  3 ):    [     41,  '(14N)(18O)',              1.993122E-03,  3.200223E+01,  'NO'          ], 
(  9,  1 ):    [     42,  '(32S)(16O)2',             9.456777E-01,  6.396190E+01,  'SO2'         ], 
(  9,  2 ):    [     43,  '(34S)(16O)2',             4.195028E-02,  6.595770E+01,  'SO2'         ], 
(  9,  3 ):    [    137,  '(33S)(16O)2',             7.464462E-03,  6.496129E+01,  'SO2'         ], 
(  9,  4 ):    [    138,  '(16O)(32S)(18O)',         3.792558E-03,  6.596615E+01,  'SO2'         ], 
( 10,  1 ):    [     44,  '(14N)(16O)2',             9.916160E-01,  4.599290E+01,  'NO2'         ], 
( 10,  2 ):    [    130,  '(15N)(16O)2',             3.645643E-03,  4.698994E+01,  'NO2'         ], 
( 11,  1 ):    [     45,  '(14N)H3',                 9.958716E-01,  1.702655E+01,  'NH3'         ], 
( 11,  2 ):    [     46,  '(15N)H3',                 3.661289E-03,  1.802358E+01,  'NH3'         ], 
( 12,  1 ):    [     47,  'H(14N)(16O)3',            9.891098E-01,  6.299564E+01,  'HNO3'        ], 
( 12,  2 ):    [    117,  'H(15N)(16O)3',            3.636429E-03,  6.399268E+01,  'HNO3'        ], 
( 13,  1 ):    [     48,  '(16O)H',                  9.974726E-01,  1.700274E+01,  'OH'          ], 
( 13,  2 ):    [     49,  '(18O)H',                  2.000138E-03,  1.900699E+01,  'OH'          ], 
( 13,  3 ):    [     50,  '(16O)D',                  1.553706E-04,  1.800891E+01,  'OH'          ], 
( 14,  1 ):    [     51,  'H(19F)',                  9.998443E-01,  2.000623E+01,  'HF'          ], 
( 14,  2 ):    [    110,  'D(19F)',                  1.557410E-04,  2.101240E+01,  'HF'          ], 
( 15,  1 ):    [     52,  'H(35Cl)',                 7.575870E-01,  3.597668E+01,  'HCl'         ], 
( 15,  2 ):    [     53,  'H(37Cl)',                 2.422573E-01,  3.797373E+01,  'HCl'         ], 
( 15,  3 ):    [    107,  'D(35Cl)',                 1.180050E-04,  3.698285E+01,  'HCl'         ], 
( 15,  4 ):    [    108,  'D(37Cl)',                 3.773502E-05,  3.897990E+01,  'HCl'         ], 
( 16,  1 ):    [     54,  'H(79Br)',                 5.067811E-01,  7.992616E+01,  'HBr'         ], 
( 16,  2 ):    [     55,  'H(81Br)',                 4.930632E-01,  8.192412E+01,  'HBr'         ], 
( 16,  3 ):    [    111,  'D(79Br)',                 7.893838E-05,  8.093234E+01,  'HBr'         ], 
( 16,  4 ):    [    112,  'D(81Br)',                 7.680162E-05,  8.293029E+01,  'HBr'         ], 
( 17,  1 ):    [     56,  'H(127I)',                 9.998443E-01,  1.279123E+02,  'HI'          ], 
( 17,  2 ):    [    113,  'D(127I)',                 1.557410E-04,  1.289185E+02,  'HI'          ], 
( 18,  1 ):    [     57,  '(35Cl)(16O)',             7.559077E-01,  5.096377E+01,  'ClO'         ], 
( 18,  2 ):    [     58,  '(37Cl)(16O)',             2.417203E-01,  5.296082E+01,  'ClO'         ], 
( 19,  1 ):    [     59,  '(16O)(12C)(32S)',         9.373947E-01,  5.996699E+01,  'OCS'         ], 
( 19,  2 ):    [     60,  '(16O)(12C)(34S)',         4.158284E-02,  6.196278E+01,  'OCS'         ], 
( 19,  3 ):    [     61,  '(16O)(13C)(32S)',         1.053146E-02,  6.097034E+01,  'OCS'         ], 
( 19,  4 ):    [     62,  '(16O)(12C)(33S)',         7.399083E-03,  6.096637E+01,  'OCS'         ], 
( 19,  5 ):    [     63,  '(18O)(12C)(32S)',         1.879670E-03,  6.197123E+01,  'OCS'         ], 
( 19,  6 ):    [    135,  '(16O)(13C)(34S)',         4.671757E-04,  6.296614E+01,  'OCS'         ], 
( 20,  1 ):    [     64,  'H2(12C)(16O)',            9.862371E-01,  3.001056E+01,  'H2CO'        ], 
( 20,  2 ):    [     65,  'H2(13C)(16O)',            1.108020E-02,  3.101392E+01,  'H2CO'        ], 
( 20,  3 ):    [     66,  'H2(12C)(18O)',            1.977609E-03,  3.201481E+01,  'H2CO'        ], 
( 21,  1 ):    [     67,  'H(16O)(35Cl)',            7.557900E-01,  5.197159E+01,  'HOCl'        ], 
( 21,  2 ):    [     68,  'H(16O)(37Cl)',            2.416826E-01,  5.396864E+01,  'HOCl'        ], 
( 22,  1 ):    [     69,  '(14N)2',                  9.926874E-01,  2.800615E+01,  'N2'          ], 
( 22,  2 ):    [    118,  '(14N)(15N)',              7.299165E-03,  2.900318E+01,  'N2'          ], 
( 23,  1 ):    [     70,  'H(12C)(14N)',             9.851143E-01,  2.701090E+01,  'HCN'         ], 
( 23,  2 ):    [     71,  'H(13C)(14N)',             1.106758E-02,  2.801425E+01,  'HCN'         ], 
( 23,  3 ):    [     72,  'H(12C)(15N)',             3.621740E-03,  2.800793E+01,  'HCN'         ], 
( 24,  1 ):    [     73,  '(12C)H3(35Cl)',           7.489369E-01,  4.999233E+01,  'CH3Cl'       ], 
( 24,  2 ):    [     74,  '(12C)H3(37Cl)',           2.394912E-01,  5.198938E+01,  'CH3Cl'       ], 
( 25,  1 ):    [     75,  'H2(16O)2',                9.949516E-01,  3.400548E+01,  'H2O2'        ], 
( 26,  1 ):    [     76,  '(12C)2H2',                9.775989E-01,  2.601565E+01,  'C2H2'        ], 
( 26,  2 ):    [     77,  '(12C)(13C)H2',            2.196629E-02,  2.701900E+01,  'C2H2'        ], 
( 26,  3 ):    [    105,  '(12C)2HD',                3.045499E-04,  2.702182E+01,  'C2H2'        ], 
( 27,  1 ):    [     78,  '(12C)2H6',                9.769900E-01,  3.004695E+01,  'C2H6'        ], 
( 27,  2 ):    [    106,  '(12C)H3(13C)H3',          2.195261E-02,  3.105031E+01,  'C2H6'        ], 
( 28,  1 ):    [     79,  '(31P)H3',                 9.995329E-01,  3.399724E+01,  'PH3'         ], 
( 29,  1 ):    [     80,  '(12C)(16O)(19F)2',        9.865444E-01,  6.599172E+01,  'COF2'        ], 
( 29,  2 ):    [    119,  '(13C)(16O)(19F)2',        1.108366E-02,  6.699508E+01,  'COF2'        ], 
( 30,  1 ):    [    126,  '(32S)(19F)6',             9.501800E-01,  1.459625E+02,  'SF6'         ], 
( 31,  1 ):    [     81,  'H2(32S)',                 9.498841E-01,  3.398772E+01,  'H2S'         ], 
( 31,  2 ):    [     82,  'H2(34S)',                 4.213687E-02,  3.598351E+01,  'H2S'         ], 
( 31,  3 ):    [     83,  'H2(33S)',                 7.497664E-03,  3.498710E+01,  'H2S'         ], 
( 32,  1 ):    [     84,  'H(12C)(16O)(16O)H',       9.838977E-01,  4.600548E+01,  'HCOOH'       ], 
( 33,  1 ):    [     85,  'H(16O)2',                 9.951066E-01,  3.299766E+01,  'HO2'         ], 
( 34,  1 ):    [     86,  '(16O)',                   9.976280E-01,  1.599492E+01,  'O'           ], 
( 35,  1 ):    [    127,  '(35Cl)(16O)(14N)(16O)2',  7.495702E-01,  9.695667E+01,  'ClONO2'      ], 
( 35,  2 ):    [    128,  '(37Cl)(16O)(14N)(16O)2',  2.396937E-01,  9.895372E+01,  'ClONO2'      ], 
( 36,  1 ):    [     87,  '(14N)(16O)+',             9.939737E-01,  2.999799E+01,  'NOp'         ], 
( 37,  1 ):    [     88,  'H(16O)(79Br)',            5.055790E-01,  9.592108E+01,  'HOBr'        ], 
( 37,  2 ):    [     89,  'H(16O)(81Br)',            4.918937E-01,  9.791903E+01,  'HOBr'        ], 
( 38,  1 ):    [     90,  '(12C)2H4',                9.772944E-01,  2.803130E+01,  'C2H4'        ], 
( 38,  2 ):    [     91,  '(12C)H2(13C)H2',          2.195946E-02,  2.903466E+01,  'C2H4'        ], 
( 39,  1 ):    [     92,  '(12C)H3(16O)H',           9.859299E-01,  3.202622E+01,  'CH3OH'       ], 
( 40,  1 ):    [     93,  '(12C)H3(79Br)',           5.009946E-01,  9.394181E+01,  'CH3Br'       ], 
( 40,  2 ):    [     94,  '(12C)H3(81Br)',           4.874334E-01,  9.593976E+01,  'CH3Br'       ], 
( 41,  1 ):    [     95,  '(12C)H3(12C)(14N)',       9.738662E-01,  4.102655E+01,  'CH3CN'       ], 
( 42,  1 ):    [     96,  '(12C)(19F)4',             9.888900E-01,  8.799362E+01,  'CF4'         ], 
( 43,  1 ):    [    116,  '(12C)4H2',                9.559980E-01,  5.001565E+01,  'C4H2'        ], 
( 44,  1 ):    [    109,  'H(12C)3(14N)',            9.633460E-01,  5.101090E+01,  'HC3N'        ], 
( 45,  1 ):    [    103,  'H2',                      9.996885E-01,  2.015650E+00,  'H2'          ], 
( 45,  2 ):    [    115,  'HD',                      3.114316E-04,  3.021825E+00,  'H2'          ], 
( 46,  1 ):    [     97,  '(12C)(32S)',              9.396236E-01,  4.397207E+01,  'CS'          ], 
( 46,  2 ):    [     98,  '(12C)(34S)',              4.168171E-02,  4.596787E+01,  'CS'          ], 
( 46,  3 ):    [     99,  '(13C)(32S)',              1.055650E-02,  4.497543E+01,  'CS'          ], 
( 46,  4 ):    [    100,  '(12C)(33S)',              7.416675E-03,  4.497146E+01,  'CS'          ], 
( 47,  1 ):    [    114,  '(32S)(16O)3',             9.434345E-01,  7.995682E+01,  'SO3'         ], 
( 48,  1 ):    [    123,  '(12C)2(14N)2',            9.707524E-01,  5.200615E+01,  'C2N2'        ], 
( 49,  1 ):    [    124,  '(12C)(16O)(35Cl)2',       5.663918E-01,  9.793262E+01,  'COCl2'       ], 
( 49,  2 ):    [    125,  '(12C)(16O)(35Cl)(37Cl)',  3.622350E-01,  9.992967E+01,  'COCl2'       ], 
( 50,  1 ):    [    146,  '(32S)(16O)',              9.479262E-01,  4.796699E+01,  'SO'          ], 
( 50,  2 ):    [    147,  '(34S)(16O)',              4.205002E-02,  4.996278E+01,  'SO'          ], 
( 50,  3 ):    [    148,  '(32S)(18O)',              1.900788E-03,  4.997123E+01,  'SO'          ], 
( 51,  1 ):    [    144,  '(12C)H3(19F)',            9.884280E-01,  3.402188E+01,  'CH3F'        ], 
( 52,  1 ):    [    139,  '(74Ge)H4',                3.651724E-01,  7.795248E+01,  'GeH4'        ], 
( 52,  2 ):    [    140,  '(72Ge)H4',                2.741292E-01,  7.595338E+01,  'GeH4'        ], 
( 52,  3 ):    [    141,  '(70Ge)H4',                2.050722E-01,  7.395555E+01,  'GeH4'        ], 
( 52,  4 ):    [    142,  '(73Ge)H4',                7.755167E-02,  7.695476E+01,  'GeH4'        ], 
( 52,  5 ):    [    143,  '(76Ge)H4',                7.755167E-02,  7.995270E+01,  'GeH4'        ], 
( 53,  1 ):    [    131,  '(12C)(32S)2',             8.928115E-01,  7.594414E+01,  'CS2'         ], 
( 53,  2 ):    [    132,  '(32S)(12C)(34S)',         7.921026E-02,  7.793994E+01,  'CS2'         ], 
( 53,  3 ):    [    133,  '(32S)(12C)(33S)',         1.409435E-02,  7.694353E+01,  'CS2'         ], 
( 53,  4 ):    [    134,  '(13C)(32S)2',             1.003057E-02,  7.694750E+01,  'CS2'         ], 
( 54,  1 ):    [    145,  '(12C)H3(127I)',           9.884280E-01,  1.419279E+02,  'CH3I'        ], 
( 55,  1 ):    [    136,  '(14N)(19F)3',             9.963370E-01,  7.099829E+01,  'NF3'         ], 
}

# calculate ISO_ID instead of repeating the same information twice
ISO_ID = {}
for mol_id,iso_id in ISO:
    ln = ISO[(mol_id,iso_id)]
    glob_iso_id = ln[0]
    ln_ = [mol_id,iso_id]+ln[1:]
    ISO_ID[glob_iso_id] = ln_

def print_iso():
    print('The dictionary \"ISO\" contains information on isotopologues in HITRAN\n')
    print('   M    I          id                  iso_name   abundance      mass        mol_name')
    for i in ISO:
        ab = ISO[i][ISO_INDEX['abundance']]
        ma = ISO[i][ISO_INDEX['mass']]
        ab = ab if ab else -1
        ma = ma if ma else -1
        print('%4i %4i     : %5i %25s %10f %10f %15s' % (i[0],i[1],ISO[i][ISO_INDEX['id']],ISO[i][ISO_INDEX['iso_name']],ab,ma,ISO[i][ISO_INDEX['mol_name']]))

def print_iso_id():
    print('The dictionary \"ISO_ID\" contains information on \"global\" IDs of isotopologues in HITRAN\n')
    print('   id            M    I                    iso_name       abundance       mass        mol_name')
    for i in ISO_ID:
        ab = ISO_ID[i][ISO_ID_INDEX['abundance']]
        ma = ISO_ID[i][ISO_ID_INDEX['mass']]
        ab = ab if ab else -1
        ma = ma if ma else -1
        print('%5i     :   %4i %4i   %25s %15.10f %10f %15s' % (i,ISO_ID[i][ISO_ID_INDEX['M']],ISO_ID[i][ISO_ID_INDEX['I']],ISO_ID[i][ISO_ID_INDEX['iso_name']],ab,ma,ISO_ID[i][ISO_ID_INDEX['mol_name']]))

profiles = 'profiles'
def print_profiles():
    print('Profiles available:')
    print('  HT        : PROFILE_HT')
    print('  SDRautian : PROFILE_SDRAUTIAN')
    print('  Rautian   : PROFILE_RAUTIAN')
    print('  SDVoigt   : PROFILE_SDVOIGT')
    print('  Voigt     : PROFILE_VOIGT')
    print('  Lorentz   : PROFILE_LORENTZ')
    print('  Doppler   : PROFILE_DOPPLER')

slit_functions = 'slit_functions'
def print_slit_functions():
    print('  RECTANGULAR : SLIT_RECTANGULAR')
    print('  TRIANGULAR  : SLIT_TRIANGULAR')
    print('  GAUSSIAN    : SLIT_GAUSSIAN')
    print('  DIFFRACTION : SLIT_DIFFRACTION')
    print('  MICHELSON   : SLIT_MICHELSON')
    print('  DISPERSION/LORENTZ : SLIT_DISPERSION')

tutorial='tutorial'
units='units'
index='index'
data='data'
spectra='spectra'
plotting='plotting'
python='python'

python_tutorial_text = \
"""
THIS TUTORIAL IS TAKEN FROM http://www.stavros.io/tutorials/python/
AUTHOR: Stavros Korokithakis


----- LEARN PYTHON IN 10 MINUTES -----


PRELIMINARY STUFF

So, you want to learn the Python programming language but can't find a concise 
and yet full-featured tutorial. This tutorial will attempt to teach you Python in 10 minutes. 
It's probably not so much a tutorial as it is a cross between a tutorial and a cheatsheet, 
so it will just show you some basic concepts to start you off. Obviously, if you want to 
really learn a language you need to program in it for a while. I will assume that you are 
already familiar with programming and will, therefore, skip most of the non-language-specific stuff. 
The important keywords will be highlighted so you can easily spot them. Also, pay attention because, 
due to the terseness of this tutorial, some things will be introduced directly in code and only 
briefly commented on.


PROPERTIES

Python is strongly typed (i.e. types are enforced), dynamically, implicitly typed (i.e. you don't 
have to declare variables), case sensitive (i.e. var and VAR are two different variables) and 
object-oriented (i.e. everything is an object). 


GETTING HELP

Help in Python is always available right in the interpreter. If you want to know how an object works, 
all you have to do is call help(<object>)! Also useful are dir(), which shows you all the object's methods, 
and <object>.__doc__, which shows you its documentation string: 

>>> help(5)
Help on int object:
(etc etc)

>>> dir(5)
['__abs__', '__add__', ...]

>>> abs.__doc__
'abs(number) -> number

Return the absolute value of the argument.'


SYNTAX

Python has no mandatory statement termination characters and blocks are specified by indentation. 
Indent to begin a block, dedent to end one. Statements that expect an indentation level end in a colon (:). 
Comments start with the pound (#) sign and are single-line, multi-line strings are used for multi-line comments. 
Values are assigned (in fact, objects are bound to names) with the _equals_ sign ("="), and equality testing is 
done using two _equals_ signs ("=="). You can increment/decrement values using the += and -= operators respectively 
by the right-hand amount. This works on many datatypes, strings included. You can also use multiple variables on one 
line. For example: 

>>> myvar = 3
>>> myvar += 2
>>> myvar
5

>>> myvar -= 1
>>> myvar
4

\"\"\"This is a multiline comment.
The following lines concatenate the two strings.\"\"\"

>>> mystring = "Hello"
>>> mystring += " world."
>>> print mystring
Hello world.

# This swaps the variables in one line(!).
# It doesn't violate strong typing because values aren't
# actually being assigned, but new objects are bound to
# the old names.
>>> myvar, mystring = mystring, myvar


DATA TYPES

The data structures available in python are lists, tuples and dictionaries. 
Sets are available in the sets library (but are built-in in Python 2.5 and later). 
Lists are like one-dimensional arrays (but you can also have lists of other lists), 
dictionaries are associative arrays (a.k.a. hash tables) and tuples are immutable 
one-dimensional arrays (Python "arrays" can be of any type, so you can mix e.g. integers, 
strings, etc in lists/dictionaries/tuples). The index of the first item in all array types is 0. 
Negative numbers count from the end towards the beginning, -1 is the last item. Variables 
can point to functions. The usage is as follows:

>>> sample = [1, ["another", "list"], ("a", "tuple")]
>>> mylist = ["List item 1", 2, 3.14]
>>> mylist[0] = "List item 1 again" # We're changing the item.
>>> mylist[-1] = 3.21 # Here, we refer to the last item.
>>> mydict = {"Key 1": "Value 1", 2: 3, "pi": 3.14}
>>> mydict["pi"] = 3.15 # This is how you change dictionary values.
>>> mytuple = (1, 2, 3)
>>> myfunction = len
>>> print myfunction(mylist)
3


You can access array ranges using a colon (:). Leaving the start index empty assumes the first item, 
leaving the end index assumes the last item. Negative indexes count from the last item backwards 
(thus -1 is the last item) like so:

>>> mylist = ["List item 1", 2, 3.14]
>>> print mylist[:]
['List item 1', 2, 3.1400000000000001]

>>> print mylist[0:2]
['List item 1', 2]

>>> print mylist[-3:-1]
['List item 1', 2]

>>> print mylist[1:]
[2, 3.14]

# Adding a third parameter, "step" will have Python step in
# N item increments, rather than 1.
# E.g., this will return the first item, then go to the third and
# return that (so, items 0 and 2 in 0-indexing).
>>> print mylist[::2]
['List item 1', 3.14]


STRINGS

Its strings can use either single or double quotation marks, and you can have quotation 
marks of one kind inside a string that uses the other kind (i.e. "He said 'hello'." is valid). 
Multiline strings are enclosed in _triple double (or single) quotes_ (\"\"\"). 
Python supports Unicode out of the box, using the syntax u"This is a unicode string". 
To fill a string with values, you use the % (modulo) operator and a tuple. 
Each %s gets replaced with an item from the tuple, left to right, and you can also use 
dictionary substitutions, like so:

>>>print "Name: %s\
Number: %s\
String: %s" % (myclass.name, 3, 3 * "-")

Name: Poromenos
Number: 3
String: ---

strString = \"\"\"This is
a multiline
string.\"\"\"

# WARNING: Watch out for the trailing s in "%(key)s".
>>> print "This %(verb)s a %(noun)s." % {"noun": "test", "verb": "is"}
This is a test.


FLOW CONTROL STATEMENTS

Flow control statements are if, for, and while. There is no select; instead, use if. 
Use for to enumerate through members of a list. To obtain a list of numbers, 
use range(<number>). These statements' syntax is thus:

rangelist = range(10)
>>> print rangelist
[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

>>> for number in rangelist:
        # Check if number is one of
        # the numbers in the tuple.
        if number in (3, 4, 7, 9):
            # "Break" terminates a for without
            # executing the "else" clause.
            break
        else:
            # "Continue" starts the next iteration
            # of the loop. It's rather useless here,
            # as it's the last statement of the loop.
            continue
    else:
        # The "else" clause is optional and is
        # executed only if the loop didn't "break".
        pass # Do nothing

>>> if rangelist[1] == 2:
        print "The second item (lists are 0-based) is 2"
    elif rangelist[1] == 3:
        print "The second item (lists are 0-based) is 3"
    else:
        print "Dunno"

>>> while rangelist[1] == 1:
        pass


FUNCTIONS

Functions are declared with the "def" keyword. Optional arguments are set in 
the function declaration after the mandatory arguments by being assigned a default 
value. For named arguments, the name of the argument is assigned a value. 
Functions can return a tuple (and using tuple unpacking you can effectively return 
multiple values). Lambda functions are ad hoc functions that are comprised of 
a single statement. Parameters are passed by reference, but immutable types (tuples, 
ints, strings, etc) *cannot be changed*. This is because only the memory location of 
the item is passed, and binding another object to a variable discards the old one, 
so immutable types are replaced. For example:

# Same as def funcvar(x): return x + 1
>>> funcvar = lambda x: x + 1
>>> print funcvar(1)
2

# an_int and a_string are optional, they have default values
# if one is not passed (2 and "A default string", respectively).
>>> def passing_example(a_list, an_int=2, a_string="A default string"):
        a_list.append("A new item")
        an_int = 4
        return a_list, an_int, a_string

>>> my_list = [1, 2, 3]
>>> my_int = 10
>>> print passing_example(my_list, my_int)
([1, 2, 3, 'A new item'], 4, "A default string")

>>> my_list
[1, 2, 3, 'A new item']

>>> my_int
10


CLASSES

Python supports a limited form of multiple inheritance in classes. 
Private variables and methods can be declared (by convention, this is not enforced 
by the language) by adding at least two leading underscores and at most one trailing 
one (e.g. "__spam"). We can also bind arbitrary names to class instances. 
An example follows:

>>> class MyClass(object):
        common = 10
        def __init__(self):
            self.myvariable = 3
        def myfunction(self, arg1, arg2):
            return self.myvariable

# This is the class instantiation
>>> classinstance = MyClass()
>>> classinstance.myfunction(1, 2)
3

# This variable is shared by all classes.
>>> classinstance2 = MyClass()
>>> classinstance.common
10

>>> classinstance2.common
10

# Note how we use the class name
# instead of the instance.
>>> MyClass.common = 30
>>> classinstance.common
30

>>> classinstance2.common
30

# This will not update the variable on the class,
# instead it will bind a new object to the old
# variable name.
>>> classinstance.common = 10
>>> classinstance.common
10

>>> classinstance2.common
30

>>> MyClass.common = 50
# This has not changed, because "common" is
# now an instance variable.
>>> classinstance.common
10

>>> classinstance2.common
50

# This class inherits from MyClass. The example
# class above inherits from "object", which makes
# it what's called a "new-style class".
# Multiple inheritance is declared as:
# class OtherClass(MyClass1, MyClass2, MyClassN)
>>> class OtherClass(MyClass):
        # The "self" argument is passed automatically
        # and refers to the class instance, so you can set
        # instance variables as above, but from inside the class.
        def __init__(self, arg1):
            self.myvariable = 3
            print arg1

>>> classinstance = OtherClass("hello")
hello

>>> classinstance.myfunction(1, 2)
3

# This class doesn't have a .test member, but
# we can add one to the instance anyway. Note
# that this will only be a member of classinstance.
>>> classinstance.test = 10
>>> classinstance.test
10


EXCEPTIONS

Exceptions in Python are handled with try-except [exceptionname] blocks:

>>> def some_function():
        try:
            # Division by zero raises an exception
            10 / 0
        except ZeroDivisionError:
            print "Oops, invalid."
        else:
            # Exception didn't occur, we're good.
            pass
        finally:
            # This is executed after the code block is run
            # and all exceptions have been handled, even
            # if a new exception is raised while handling.
            print "We're done with that."

>>> some_function()
Oops, invalid.

We're done with that.


IMPORTING:

External libraries are used with the import [libname] keyword. 
You can also use from [libname] import [funcname] for individual functions. 
Here is an example:

>>> import random
>>> from time import clock

>>> randomint = random.randint(1, 100)
>>> print randomint
64


FILE I/O

Python has a wide array of libraries built in. As an example, here is how serializing 
(converting data structures to strings using the pickle library) with file I/O is used:

>>> import pickle
>>> mylist = ["This", "is", 4, 13327]
# Open the file C:\\binary.dat for writing. The letter r before the
# filename string is used to prevent backslash escaping.
>>> yfile = open(r"C:\\binary.dat", "w")
>>> pickle.dump(mylist, myfile)
>>> myfile.close()

>>> myfile = open(r"C:\\text.txt", "w")
>>> myfile.write("This is a sample string")
>>> myfile.close()

>>> myfile = open(r"C:\\text.txt")
>>> print myfile.read()
'This is a sample string'

>>> myfile.close()

# Open the file for reading.
>>> myfile = open(r"C:\\binary.dat")
>>> loadedlist = pickle.load(myfile)
>>> myfile.close()
>>> print loadedlist
['This', 'is', 4, 13327]


MISCELLANEOUS

    -> Conditions can be chained. 1 < a < 3 checks 
       that a is both less than 3 and greater than 1.
    -> You can use del to delete variables or items in arrays.
    -> List comprehensions provide a powerful way to create 
       and manipulate lists. They consist of an expression 
       followed by a for clause followed by zero or more 
       if or for clauses, like so:

>>> lst1 = [1, 2, 3]
>>> lst2 = [3, 4, 5]
>>> print [x * y for x in lst1 for y in lst2]
[3, 4, 5, 6, 8, 10, 9, 12, 15]

>>> print [x for x in lst1 if 4 > x > 1]
[2, 3]

# Check if a condition is true for any items.
# "any" returns true if any item in the list is true.
>>> any([i % 3 for i in [3, 3, 4, 4, 3]])
True

# This is because 4 % 3 = 1, and 1 is true, so any()
# returns True.

# Check for how many items a condition is true.
>>> sum(1 for i in [3, 3, 4, 4, 3] if i == 4)
2

>>> del lst1[0]
>>> print lst1
[2, 3]

>>> del lst1



    -> Global variables are declared outside of functions 
       and can be read without any special declarations, 
       but if you want to write to them you must declare them 
       at the beginning of the function with the "global" keyword, 
       otherwise Python will bind that object to a new local 
       variable (be careful of that, it's a small catch that can 
       get you if you don't know it). For example:

>>> number = 5

>>> def myfunc():
        # This will print 5.
        print number

>>> def anotherfunc():
        # This raises an exception because the variable has not
        # been bound before printing. Python knows that it an
        # object will be bound to it later and creates a new, local
        # object instead of accessing the global one.
        print number
        number = 3

>>> def yetanotherfunc():
        global number
        # This will correctly change the global.
        number = 3


EPILOGUE

This tutorial is not meant to be an exhaustive list of all (or even a subset) of Python. 
Python has a vast array of libraries and much much more functionality which you will 
have to discover through other means, such as the excellent book Dive into Python. 
I hope I have made your transition in Python easier. Please leave comments if you believe 
there is something that could be improved or added or if there is anything else 
you would like to see (classes, error handling, anything). 

"""
def print_python_tutorial():
    pydoc.pager(python_tutorial_text)

data_tutorial_text = \
"""

ACCESS YOUR DATA!

Welcome to tutorial on retrieving and processing the data from HITRANonline.


  ///////////////
 /// PREFACE ///
///////////////

HITRANonline API is a set of routines in Python which is aimed to 
provide a remote access to functionality and data given by a new project 
HITRANonline (http://hitranazure.cloudapp.net).

At the present moment the API can download, filter and process data on 
molecular and atomic line-by-line spectra which is provided by HITRANonline portal.

One of the major purposes of introducing API is extending a functionality 
of the main site, particularly providing a possibility to calculate several 
types of high- and low-resolution spectra based on a flexible HT lineshape. 

Each feature of API is represented by a Python function with a set of parameters 
providing a flexible approach to the task.


  ///////////////////////
 /// FEATURE SUMMARY ///
///////////////////////

1) Downloading line-by-line data from the HITRANonline site to local database.
2) Filtering and processing the data in SQL-like fashion.
3) Conventional Python structures (lists, tuples, dictionaries) for representing 
   a spectroscopic data.
4) Possibility to use a large set of third-party Python libraries to work with a data
5) Python implementation of an HT (Hartmann-Tran [1]) lineshape which is used in spectra.
   simulations. This lineshape can also be reduced to a number of conventional 
   line profiles such as Gaussian (Doppler), Lorentzian, Voigt, Rautian, 
   Speed-dependent Voigt and Rautian.
6) Python implementation of total internal partition sums (TIPS-2011 [2]) 
   which is used in spectra simulations.
7) High-resolution spectra simulation accounting pressure, 
   temperature and optical path length. The following spectral functions 
   can be calculated:
      a) absorption coefficient
      b) absorption spectrum
      c) transmittance spectrum
      d) radiance spectrum
8) Low-resolution spectra simulation using a number of apparatus functions.
9) Possibility to extend with the user's functionality by adding custom lineshapes, 
   partitions sums and apparatus functions.

References:

[1] N.H. Ngo, D. Lisak, H. Tran, J.-M. Hartmann.
    An isolated line-shape model to go beyond the Voigt profile in 
    spectroscopic databases and radiative transfer codes.
    JQSRT, Volume 129, November 2013, Pages 89–100
    http://dx.doi.org/10.1016/j.jqsrt.2013.05.034

[2] A. L. Laraia, R. R. Gamache, J. Lamouroux, I. E. Gordon, L. S. Rothman.
    Total internal partition sums to support planetary remote sensing.
    Icarus, Volume 215, Issue 1, September 2011, Pages 391–400
    http://dx.doi.org/10.1016/j.icarus.2011.06.004

_______________________________________________________________________


This tutorial will give you an insight of how to use HAPI for Python.

First, let's choose a folder for our local database. Every time you start
your Python project, you have to specify explicitly the name of the 
database folder.

>>> db_begin('data')

So, let's download some data from the server and do some processing on it.
Suppose that we want to get line by line data on the main isotopologue of H2O.

For retrieving the data to the local database, user have to specify the following parameters:
1) Name of the local table which will store the downloaded data.
2) Either a pair of molecule and isotopologue HITRAN numbers (M and I), 
   or a "global" isotopologue ID (iso_id).
3) Wavenumber range (nu_min and nu_max)

N.B. If you specify the name which already exists in the database, 
the existing table with that name will be overrided. 

To get additional information on function fetch,
call getHelp:

>>> getHelp(fetch)
...

To download the data, simply call the function "fetch".
This will establish a connection with the main server and get the data using
the parameters listed above:

>>> fetch('H2O',1,1,3400,4100)
BEGIN DOWNLOAD: H2O
  65536 bytes written to data/H2O.data
  65536 bytes written to data/H2O.data
  65536 bytes written to data/H2O.data
...
  65536 bytes written to data/H2O.data
  65536 bytes written to data/H2O.data
  65536 bytes written to data/H2O.data
Header written to data/H2O.header
END DOWNLOAD
                     Lines parsed: 7524
PROCESSED

The output is shown right after the console line ">>>".
To check the file that you've just downloaded you can open the database
folder. The new plain text file should have a name "H2O.data" and
it should contain line-by-line data in HITRAN format.

N.B. If we want several isotopologues in one table, we should
use fetch_by_ids instead of just fetch. Fetch_by_ids takes a "global" 
isotopologue ID numbers as an input instead of HITRAN's "local" identification.
See getHelp(fetch_by_ids) to get more information on this.

To get a list of tables which are already in the database,
use tableList() function (it takes no arguments):
>>> tableList()

To learn about the table we just downloaded, let's use a function "describeTable".

>>> describeTable('H2O')
-----------------------------------------
H2O summary:
-----------------------------------------
Comment: 
Contains lines for H2(16O)
 in 3400.000-4100.000 wavenumber range
Number of rows: 7524
Table type: column-fixed
-----------------------------------------
            PAR_NAME           PAR_FORMAT

            molec_id                  %2d
        local_iso_id                  %1d
                  nu               %12.6f
                  sw               %10.3E
                   a               %10.3E
           gamma_air                %5.4f
          gamma_self                %5.3f
              elower               %10.4f
               n_air                %4.2f
           delta_air                %8.6f
 global_upper_quanta                 %15s
 global_lower_quanta                 %15s
  local_upper_quanta                 %15s
  local_lower_quanta                 %15s
                ierr                  %6s
                iref                 %12s
    line_mixing_flag                  %1s
                  gp                %7.1f
                 gpp                %7.1f
-----------------------------------------

This output tells how many rows are currenty in the table H2O, which 
wavenumber range was used by fetch(). Also this gives a basic information 
about parameters stored in the table.

So, having the table downloaded, one can perform different operations on it
using API.

Here is a list of operations currently available with API:
1) FILTERING 
2) OUTPUTTING
3) SORTING
4) GROUPING


  ////////////////////////////////
 /// FILTERING AND OUTPUTTING ///
////////////////////////////////

The table data can be filtered with the help of select() function.

Use simple select() call to output the table content:

>>> select('H2O')
MI          nu         S         A gair gsel        E_nair    dair  ...
11 1000.288940 1.957E-24 2.335E-02.07100.350 1813.22270.680.008260  ...
11 1000.532321 2.190E-28 1.305E-05.04630.281 2144.04590.39-.011030  ...
...

This will display the list of line parameters containing in the table "H2O".

That's the simplest way of using the function select(). Full information
on control parameters can be obtained via getHelp(select) statement.

Suppose that we need a lines from a table within some wavenumber range. 
That's what filtering is for. Let's apply a simple range filter on a table.

>>> select('H2O',Conditions=('between','nu',4000,4100))
MI          nu         S         A gair gsel        E_nair    dair     
 11 4000.188800 1.513E-25 1.105E-02.03340.298 1581.33570.51-.013910 ...
 11 4000.204070 3.482E-24 8.479E-03.08600.454  586.47920.61-.007000 ...
 11 4000.469910 3.268E-23 1.627E+00.05410.375 1255.91150.56-.013050 ...
......

As a result of this operation, we see a list of lines of H2O table,
whose wavenumbers lie between 4000 cm-1 and 4100 cm-1.
The condition is taken as an input parameter to API function "select".

To specify a subset of columns to display, use another control parameter - 
ParameterNames:

>>> select('H2O',ParameterNames=('nu','sw'),Conditions=('between','nu',4000,4100))

The usage of ParameterNames is outlined below in the section "Specifying a list 
of parameters". So far it worth mentioning that this parameter is a part 
of a powerful tool for displaying and processing tables from database.

In the next section we will show how to create quieries 
with more complex conditions.


  ////////////////////////////
 /// FILTERING CONDITIONS ///
////////////////////////////

Let's analyze the last example of filtering. Condition input variable is
as follows:

                    ('between','nu',4000,4100)

Thus, this is a python list (or tuple), containing logical expressions
defined under column names of the table. For example, 'nu' is a name of 
the column in 'H2O' table, and this column contains a transition wavenumber.
The structure of a simple condition is as follows:

                    (OPERATION,ARG1,ARG2,...)
                    
Where OPERATION must be in a set of predefined operations (see below),
and ARG1,ARG2 etc. are the arguments for this operation.
Conditions can be nested, i.e. ARG can itself be a condition (see examples).
The following operations are available in select (case insensitive):


DESCRIPTION                   LITERAL                     EXAMPLE
---------------------------------------------------------------------------------
Range:               'RANGE','BETWEEN':         ('BETWEEN','nu',0,1000)
Subset:              'IN','SUBSET':             ('IN','local_iso_id',[1,2,3,4])
And:                 '&','&&','AND':            ('AND',('<','nu',1000),('>','nu',10))
Or:                  '|','||','OR':             ('OR',('>','nu',1000),('<','nu',10))
Not:                 '!','NOT':                 ('NOT',('IN','local_iso_id',[1,2,3]))
Less than:           '<','LESS','LT':                 ('<','nu',1000)
More than:           '>','MORE','MT':                 ('>','sw',1.0e-20)
Less or equal than:  '<=','LESSOREQUAL','LTE':        ('<=','local_iso_id',10)
More or equal than   '>=','MOREOREQUAL','MTE':        ('>=','sw',1e-20)
Equal:               '=','==','EQ','EQUAL','EQUALS':  ('<=','local_iso_id',10)
Not equal:           '!=','<>','~=','NE','NOTEQUAL':  ('!=','local_iso_id',1)
Summation:           '+','SUM':                 ('+','v1','v2','v3')
Difference:          '-','DIFF':                ('-','nu','elow')
Multiplication:      '*','MUL':                 ('*','sw',0.98)
Division:            '/','DIV':                 ('/','A',2)
Cast to string:      'STR','STRING':            ('STR','some_string')
Cast to Python list  'LIST':                    ('LIST',[1,2,3,4,5])
Match regexp         'MATCH','LIKE':            ('MATCH','\w+','some string')
Search single match: 'SEARCH':                  ('SEARCH','\d \d \d','1 2 3 4')
Search all matches:  'FINDALL':                 ('FINDALL','\d','1 2 3 4 5')
Count within group:  'COUNT' :                  ('COUNT','local_iso_id')
---------------------------------------------------------------------------------
   
Let's create a query with more complex condition. Suppese that we are 
interested in all lines between 3500 and 4000 with 1e-19 intensity cutoff.
The query will look like this:

>>> Cond = ('AND',('BETWEEN','nu',3500,4000),('>=','Sw',1e-19))
>>> select('H2O',Conditions=Cond,DestinationTableName='tmp')

Here, apart from other parameters, we have used a new parameter 
DestinationTableName. This parameter contains a name of the table
where we want to put a result of the query. Thus we have chosen 
a name 'tmp' for a new table.


  ////////////////////////////////////
 /// ACCESSING COLUMNS IN A TABLE ///
////////////////////////////////////

To get an access to particular table column (or columns) all we need
is to get a column from a table and put it to Python variable.

For this purpose, there exist two functions:

  getColumn(...)
  getColumns(...)

The first one returns just one column at a time. The second one returns
a list of solumns.

So, here are some examples of how to use both:

>>> nu1 = getColumn('H2O','nu')
>>> nu2,sw2 = getColumns('H2O',['nu','sw'])

N.B. If you don't remember exact names of columns in a particular table,
use describeTable to get an info on it's structure!


  ///////////////////////////////////////
 /// SPECIFYING A LIST OF PARAMETERS ///
///////////////////////////////////////

Suppose that we want not only select a set of parameters/columns
from a table, but do a certain transformations with them (for example,
multiply column on a coefficient, or add one column to another etc...).
We can make it in two ways. First, we can extract a column from table
using one of the functions (getColumn or getColumns) and do the rest 
in Python. The second way is to do it on the level of select.
The select function has a control parameter "ParameterNames", which 
makes it possible to specify parameters we want to be selected, 
and evaluate some simple arithmetic expressions with them.

Assume that we need only wavenumber and intensity from H2O table.
Also we need to scale an intensity to the unitary abundance. To do so,
we must divide an 'sw' parameter by it's natural abundance (0.99731) for 
principal isotopologue of water).

Thus, we have to select two columns:  
wavenumber (nu) and scaled intensity (sw/0.99731)
>>> select('H2O',)


  ////////////////////////////
 /// SAVING QUERY TO DISK ///
////////////////////////////

To quickly save a result of a query to disk, the user can take an 
advantage of an additional parameter "File".
If this parameter is presented in function call, then the query is 
saved to file with the name which was specified in "File".

For example, select all lines from H2O and save the result in file 'H2O.txt':
>>> select('H2O',File='H2O.txt')


  ////////////////////////////////////////////
 /// GETTING INFORMATION ON ISOTOPOLOGUES ///
////////////////////////////////////////////

API provides the following auxillary information about isotopologues
present in HITRAN. Corresponding functions use the standard HITRAN
molecule-isotopologue notation:

1) Natural abundances
>>> abundance(mol_id,iso_id)

2) Molecular masses
>>> molecularMass(mol_id,iso_id)

3) Molecule names
>>> moleculeName(mol_id,iso_id)

4) Isotopologue names
>>> isotopologueName(mol_id,iso_id)

5) ISO_ID
>>> getHelp(ISO_ID)

The latter is a dictionary, which contain all information about 
isotopologues concentrated in one place.

"""
def print_data_tutorial():
    pydoc.pager(data_tutorial_text)

spectra_tutorial_text = \
"""

CALCULATE YOUR SPECTRA!

Welcome to tutorial on calculating a spectra from line-by-line data.


  ///////////////
 /// PREFACE ///
///////////////

This tutorial will demonstrate how to use different lineshapes and partition
functions, and how to calculate synthetic spectra with respect to different 
instruments. It will be shown how to combine different parameters of spectral 
calculation to achieve better precision and performance for cross sections.

API provides a powerful tool to calculate cross-sections based on line-by-line
data containing in HITRAN. This features:

*) Python implementation of an HT (Hartmann-Tran [1]) lineshape which is used in 
   spectra simulations. This lineshape can also be reduced to a number of 
   conventional    line profiles such as Gaussian (Doppler), Lorentzian, Voigt, 
   Rautian, Speed-dependent Voigt and Rautian.
*) Python implementation of total internal partition sums (TIPS-2011 [2]) 
   which is used in spectra simulations.
*) High-resolution spectra simulation accounting pressure, 
   temperature and optical path length. The following spectral functions 
   can be calculated:
      a) absorption coefficient
      b) absorption spectrum
      c) transmittance spectrum
      d) radiance spectrum
*) Low-resolution spectra simulation using a number of apparatus functions.
*) Possibility to extend with the user's functionality by adding custom lineshapes, 
   partitions sums and apparatus functions.
*) An approach to function code is aimed to be flexible enough yet hopefully 
   intuitive.

References:

[1] N.H. Ngo, D. Lisak, H. Tran, J.-M. Hartmann.
    An isolated line-shape model to go beyond the Voigt profile in 
    spectroscopic databases and radiative transfer codes.
    JQSRT, Volume 129, November 2013, Pages 89–100
    http://dx.doi.org/10.1016/j.jqsrt.2013.05.034

[2] A. L. Laraia, R. R. Gamache, J. Lamouroux, I. E. Gordon, L. S. Rothman.
    Total internal partition sums to support planetary remote sensing.
    Icarus, Volume 215, Issue 1, September 2011, Pages 391–400
    http://dx.doi.org/10.1016/j.icarus.2011.06.004

            
  ///////////////////////////
 /// USING LINE PROFILES ///
///////////////////////////

Several lineshape (line profile) families are currently available:
1) Gaussian (Doppler) profile
2) Lorentzian profile
3) Voigt profile
4) Speed-dependent Voigt profile
5) Rautian profile
6) Speed-dependent Rautian profile
7) HT profile (Hartmann-Tran)

Each profile has it's own uniwue set of parameters. Normally one should
use profile parameters only in conjunction with their "native" profiles.

So, let's start exploring the available profiles using getHelp:
>>> getHelp(profiles)
Profiles available:
  HTP       : PROFILE_HT
  SDRautian : PROFILE_SDRAUTIAN
  Rautian   : PROFILE_RAUTIAN
  SDVoigt   : PROFILE_SDVOIGT
  Voigt     : PROFILE_VOIGT
  Lorentz   : PROFILE_LORENTZ
  Doppler   : PROFILE_DOPPLER

Output gives all available profiles. We can get additional info on each
of them just by calling getHelp(ProfileName):
>>> getHelp(PROFILE_HT)

Line profiles, adapted for using with HAPI, are written in Python and
heavily using the numerical library "Numpy". This means that the user
can calculate multiple values of particular profile at once having just
pasted a numpy array as a wavenumber grid (array). Let's give a short 
example of how to calculate HT profile on a numpy array.

>>> from numpy import arange
    w0 = 1000.
    GammaD = 0.005
    Gamma0 = 0.2
    Gamma2 = 0.01 * Gamma0
    Delta0 = 0.002
    Delta2 = 0.001 * Delta0
    nuVC = 0.2
    eta = 0.5
    Dw = 1.
    ww = arange(w0-Dw, w0+Dw, 0.01)  # GRID WITH THE STEP 0.01 
    l1 = PROFILE_HT(w0,GammaD,Gamma0,Gamma2,Delta0,Delta2,nuVC,eta,ww)[0]
    # now l1 contains values of HT profile calculates on the grid ww
    
On additional information about parameters see getHelp(PROFILE_HT).

It worth noting that PROFILE_HT returns 2 entities: real and imaginary part
of lineshape (as it described in the article given in preface). Apart from
HT, all other profiles return just one entity (the real part).


  ////////////////////////////
 /// USING PARTITION SUMS ///
////////////////////////////

As it was mentioned in the preface to this tutorial, the partition sums
are taken from the TIPS-2011 (the link is given above). Partition sums 
are taken for those isotopologues, which are present in HITRAN and in
TIPS-2011 simultaneousely.

N.B. Partition sums are omitted for the following isotopologues which
are in HITRAN at the moment:

ID       M     I         ISO                MOL
--------------------------------------------------
117      12    2     H(15N)(16O)3           HNO3
110      14    2     D(19F)                 HF
107      15    3     D(35Cl)                HCl
108      15    4     D(37Cl)                HCl
111      16    3     D(79Br)                HBr
112      16    4     D(81Br)                HBr
113      17    2     D(127I)                HI
118      22    2     (14N)(15N)             N2
119      29    2     (13C)(16O)(19F)2       COF2
 86      34    1     (16O)                  O
 92      39    1     (12C)H3(16O)H          CH3OH
114      47    1     (32S)(16O)3            SO3
--------------------------------------------------

The data on these isotopologues is not present in TIPS-2011 but is 
present in HITRAN. We're planning to add these molecules after TIPS-2013
is released.

To calculate a partition sum for most of the isotopologues in HITRAN,
we will use a function partitionSum (use getHelp for detailed info).
Let's just mention that 
The syntax is as follows: partitionSum(M,I,T), where M,I - standard 
HITRAN molecule-isotopologue notation, T - definition of temperature
range.

Usecase 1: temperatuer is defined by a list:
>>> Q = partitionSum(1,1,[70,80,90])

Usecase 2: temperature is defined by bounds and the step:
>>> T,Q = partiionSum(1,1,[70,3000],step=1.0)

In the latter example we calculate a partition sum on a range of
temperatures from 70K to 3000K using a step 1.0 K, and having arrays 
of temperature (T) and partition sum (Q) at the output.


  ///////////////////////////////////////////
 /// CALCULATING ABSORPTION COEFFICIENTS ///
///////////////////////////////////////////

Currently API can calculate the following spectral function at arbitrary
thermodynamic parameters:

1) Absorption coefficient
2) Absorption spectrum
3) Transmittance spectrum
4) Radiance spectrum

All these functions can be calculated with or without accounting of 
an instrument properties (apparatus function, resolution, path length etc...)

As it well known, the spectral functions such as absorption,
transmittance, and radiance spectra, are calculated on the basis
of the absorption coefficient. By that resaon, absorption coefficient
is the most important part of simulating a cross section. This part of
tutorial is devoted to demonstration how to calculate absorption 
coefficient from the HITRAN line-by-line data. Here we give a brief 
insight on basic parameters of calculation procedure, talk about some 
useful practices and precautions.

To calculate an absorption coefficient, we can use one of the following
functions:

-> absorptionCoefficient_HT
-> absorptionCoefficient_Voigt
-> absorptionCoefficient_Lorentz
-> absorptionCoefficient_Doppler

Each of these function calculates cross sections using different
lineshapes (the names a quite self-explanatory).
You can get detailed information on using each of these functions
by calling getHelp(function_name).

Let's look more closely to the cross sections based on the Lorentz profile.
For doing that, let's have a table downloaded from HITRANonline.

# get data on CO2 main isotopologue in the range 2000-2100 cm-1
>>> fetch('CO2',2,1,2000,2100)

OK, now we're ready to run a fast example of how to calculate an
absorption coefficient cross section:

>>> nu,coef = absorptionCoefficient_Lorentz(SourceTables='CO2')

This example calculates a Lorentz cross section using the whole set of 
lines in the "co2" table. This is the simplest possible way to use these
functions, because major part of parameters bound to their default values.

If we have matplotlib installed, then we can visualize it using a plotter:
>>> from pylab import plot
>>> plot(nu,coef) 

API provides a flexible control over a calculation procedure. This control
can be achieved by using a number of input parameters. So, let's dig 
into the depth of the settings.

The input parameters of absorptionCoefficient_Lorentz are as follows:

Name                          Default value
-------------------------------------------------------------------
SourceTables                  '__BUFFER__'
Components                    All isotopologues in SourceTables 
partitionFunction             PYTIPS
Environment                   {'T':296.,'p':1.}
WavenumberRange               depends on Components
WavenumberStep                0.01 cm-1
WavenumberWing                10 cm-1
WavenumberWingHW              50 HWHMs
IntensityThreshold            0 cm/molec
GammaL                        'gamma_air'
HITRAN_units                  True 
File                          None
Format                        '%e %e'
-------------------------------------------------------------------

Newt we'll give a brief explanation for each parameter. After each description
we'll make some notes about the usage of the correspondent parameter.


SourceTables:     (required parameter)
   
  List of source tables to take line-by-line data from.
  NOTE: User must provide at least one table in the list.

Components:    (optional parameter)

  List of tuples (M,I,D) to consider in cross section calculation.
  M here is a molecule number, I is an isotopologue number, 
  D is an abundance of the component.
  NOTE: If this input contains more than one tuple, then the output 
        is an absorption coefficient for mixture of corresponding gases.
  NOTE2: If omitted, then all data from the source tables is involved.

partitionFunction:    (optional parameter)

  Instance of partition function of the following format:
  Func(M,I,T), where Func - numae of function, (M,I) - HITRAN numbers
  for molecule and isotopologue, T - temperature.
  Function must return only one output - value of partition sum.
  NOTE: Deafult value is PYTIPS - python version of TIPS-2011

Environment:    (optional parameter)

  Python dictionary containing value of pressure and temperature.
  The format is as follows: Environment = {'p':pval,'T':tval}, 
  where "pval" and "tval" are corresponding values in atm and K 
  respectively.
  NOTE: Default value is {'p':1.0,'T':296.0}

WavenumberRange:    (optional parameter)

  List containing minimum and maximum value of wavenumber to consider
  in cross-section calculation. All lines that are out of htese bounds
  will be skipped. The firmat is as follows: WavenumberRange=[wn_low,wn_high]
  NOTE: If this parameter os skipped, then min and max are taken 
  from the data from SourceTables. Deprecated name is OmegaRange.

WavenumberStep:    (optional parameter)

  Value for the wavenumber step. 
  NOTE: Default value is 0.01 cm-1.
  NOTE2: Normally user would want to take the step under 0.001 when
         calculating absorption coefficient with Doppler profile 
         because of very narrow spectral lines. Deprecated name is OmegaStep.

WavenumberWing:    (optional parameter)

  Absolute value of the line wing in cm-1, i.e. distance from the center 
  of each line to the most far point where the profile is considered 
  to be non zero. Deprecated name is OmegaStep.
  NOTE: if omitted, then only OmegaWingHW is taken into account.

WavenumberWingHW:    (optional parameter)

  Relative value of the line wing in halfwidths. Deprecated name is OmegaWingHW.
  NOTE: The resulting wing is a maximum value from both OmegaWing and
  OmegaWingHW.

IntensityThreshold:    (optional parameter)

  Absolute value of minimum intensity in cm/molec to consider.
  NOTE: default value is 0.

GammaL:    (optional parameter)

  This is the name of broadening parameter to consider a "Lorentzian"
  part in the Voigt profile. In the current 160-char format there is 
  a choise between "gamma_air" and "gamma_self".
  NOTE: If the table has custom columns with a broadening coefficients,
        the user can specify the name of this column in GammaL. This
        would let the function calculate an absorption with custom
        broadening parameter.

HITRAN_units:    (optional parameter)

  Logical flag for units, in which the absorption coefficient shoould be 
  calculated. Currently, the choises are: cm^2/molec (if True) and
  cm-1 (if False).
  NOTE: to calculate other spectral functions like transmitance,
  radiance and absorption spectra, user should set HITRAN_units to False.

File:    (optional parameter)

  The name of the file to save the calculated absorption coefficient.
  The file is saved only if this parameter is specified.

Format:    (optional parameter)

  C-style format for the text data to be saved. Default value is "%e %e".
  NOTE: C-style output format specification (which are mostly valid for Python) 
        can be found, for instance, by the link: 
  http://www.gnu.org/software/libc/manual/html_node/Formatted-Output.html


N.B. Other functions such as absorptionCoefficient_Voigt(_HT,_Doppler) have
identical parameter sets so the description is the same for each function.


  ///////////////////////////////////////////////////////////////////
 /// CALCULATING ABSORPTION, TRANSMITTANCE, AND RADIANCE SPECTRA ///
///////////////////////////////////////////////////////////////////

Let's calculate an absorption, transmittance, and radiance
spectra on the basis of apsorption coefficient. In order to be consistent
with internal API's units, we need to have an absorption coefficient cm-1:

>>> nu,coef = absorptionCoefficient_Lorentz(SourceTables='CO2',HITRAN_units=False)

To calculate absorption spectrum, use the function absorptionSpectrum():
>>> nu,absorp = absorptionSpectrum(nu,coef) 

To calculate transmittance spectrum, use function transmittanceSpectrum():
>>> nu,trans = transmittanceSpectrum(nu,coef) 

To calculate radiance spectrum, use function radianceSpectrum():
>>> nu,radi = radianceSpectrum(nu,coef) 


The last three commands used a default path length (1 m).
To see complete info on all three functions, look for section
"calculating spectra" in getHelp()

Generally, all these three functions use similar set of parameters:

Wavenumber:       (required parameter) 

  Wavenumber grid to for spectrum. Deprecated name is Omegas.

AbsorptionCoefficient        (optional parameter)

  Absorption coefficient as input.

Environment={'T': 296.0, 'l': 100.0}       (optional parameter) 

  Environmental parameters for calculating  spectrum.
  This parameter is a bit specific for each of functions:
  For absorptionSpectrum() and transmittanceSpectrum() the default
  value is as follows: Environment={'l': 100.0}
  For transmittanceSpectrum() the default value, besides path length,
  contains a temperature: Environment={'T': 296.0, 'l': 100.0}
  NOTE: temperature must be equal to that which was used in 
  absorptionCoefficient_ routine!

File         (optional parameter)

  Filename of output file for calculated spectrum.
  If omitted, then the file is not created.

Format        (optional parameter)

  C-style format for spectra output file.
  NOTE: Default value is as follows: Format='%e %e'


  ///////////////////////////////////////
 /// APPLYING INSTRUMENTAL FUNCTIONS ///
///////////////////////////////////////

For comparison of the theoretical spectra with the real-world 
instruments output it's necessary to take into account instrumental resolution.
For this purpose HAPI has a function convolveSpectrum() which can emulate
spectra with lower resolution using custom instrumental functions.

The following instrumental functions are available:
1) Rectangular
2) Triangular
3) Gaussian
4) Diffraction
5) Michelson
6) Dispersion
7) Lorentz

To get a description of each instrumental function we can use getHelp():
>>> getHelp(slit_functions)
  RECTANGULAR : SLIT_RECTANGULAR
  TRIANGULAR  : SLIT_TRIANGULAR
  GAUSSIAN    : SLIT_GAUSSIAN
  DIFFRACTION : SLIT_DIFFRACTION
  MICHELSON   : SLIT_MICHELSON
  DISPERSION/LORENTZ : SLIT_DISPERSION
  
For instance,
>>> getHelp(SLIT_MICHELSON)
... will give a datailed info about Michelson's instrumental function.


The function convolveSpectrum() convolutes a high-resulution spectrum
with one of supplied instrumental (slit) functions. The folowing 
parameters of this function are provided:

Wavenumber     (required parameter)
  
  Array of wavenumbers in high-resolution input spectrum.
  Deprecated name is Omega.

CrossSection     (required parameter)

  Values of high-resolution input spectrum.

Resolution     (optional parameter)

  This parameter is passed to the slit function. It represents
  the resolution of corresponding instrument.
  NOTE: default value is 0.1 cm-1

AF_wing     (optional parameter)

  Width of an instrument function where it is considered non-zero.
  NOTE: default value is 10.0 cm-1

SlitFunction     (optional parameter)

  Custom instrumental function to convolve with spectrum.
  Format of the instrumental function must be as follows:
  Func(x,g), where Func - function name, x - wavenumber,
  g - resolution.
  NOTE: if omitted, then the default value is SLIT_RECTANGULAR


Before using the convolution procedure it worth giving some practical 
advices and remarks: 
1) Quality of a convolution depends on many things: quality of calculated 
spectra, width of AF_wing and WavenumberRange, Resolution, WavenumberStep etc ...
Most of these factors are taken from previus stages of spectral calculation.
Right choise of all these factors is crucial for the correct computation.
2) Dispersion, Diffraction and Michelson AF's don't work well in narrow 
wavenumber range because of their broad wings.
3) Generally one must consider WavenumberRange and AF_wing as wide as possible.
4) After applying a convolution, the resulting spectral range for 
the lower-resolution spectra is reduced by the doubled value of AF_wing.
For this reason, try to make an initial spectral range for high-resolution
spectrum (absorption, transmittance, radiance) sufficiently broad.

The following command will calculate a lower-resolution spectra from 
the CO2 transmittance, which was calculated in a previous section. 
The Spectral resolution is 1 cm-1, 

>>> nu_,trans_,i1,i2,slit = convolveSpectrum(nu,trans)

The outputs are: 

nu_, trans_ - wavenumbers and transmittance for the resulting 
              low-resolution spectrum.

i1,i2 - indexes for initial nu,trans spectrum denoting the part of 
        wavenumber range which was taken for lower resolution spectrum.
        => Low-res spectrum is calculated on nu[i1:i2]

Note, than to achieve more flexibility, one have to specify most of 
the optional parameters. For instance, more complete call is as follows:
>>> nu_,trans_,i1,i2,slit = convolveSpectrum(nu,trans,SlitFunction=SLIT_MICHELSON,Resolution=1.0,AF_wing=20.0)

"""
def print_spectra_tutorial():
    pydoc.pager(spectra_tutorial_text)

plotting_tutorial_text = \
"""

PLOTTING THE SPECTRA WITH MATPLOTLIB

This tutorial briefly explains how to make plots using
the Matplotlib - Python library for plotting.

Prerequisites:
   To tun through this tutorial, user must have the following
   Python libraries installed:
   1) Matplotlib
       Matplotlib can be obtained by the link http://matplotlib.org/ 
   2) Numpy  (required by HAPI itself)
       Numpy can be obtained via pip:  
          sudo pip install numpy (under Linux and Mac)
          pip install numpy (under Windows)
       Or by the link http://www.numpy.org/
       
As an option, user can download one of the many scientific Python
distributions, such as Anaconda, Canopy etc...

So, let's calculate plot the basic entities which ar provided by HAPI.
To do so, we will do all necessary steps to download, filter and 
calculate cross sections "from scratch". To demonstrate the different
possibilities of matplotlib, we will mostly use Pylab - a part of 
Matplotlib with the interface similar to Matlab. Please note, that it's 
not the only way to use Matplotlib. More information can be found on it's site.

The next part is a step-by-step guide, demonstrating basic possilities
of HITRANonline API in conjunction with Matplotlib.

First, do some preliminary imports:
>>> from hapi import *
>>> from pylab import show,plot,subplot,xlim,ylim,title,legend,xlabel,ylabel,hold

Start the database 'data':
>>> db_begin('data') 

Download lines for main isotopologue of ozone in [3900,4050] range:
>>> fetch('O3',3,1,3900,4050)

PLot a sick spectrum using the function getStickXY()
>>> x,y = getStickXY('O3')
>>> plot(x,y); show()

Zoom in spectral region [4020,4035] cm-1:
>>> plot(x,y); xlim([4020,4035]); show()

Calculate and plot difference between Voigt and Lorentzian lineshape:
>>> wn = arange(3002,3008,0.01) # get wavenumber range of interest
>>> voi = PROFILE_VOIGT(3005,0.1,0.3,wn)[0]   # calc Voigt
>>> lor = PROFILE_LORENTZ(3005,0.3,wn)   # calc Lorentz
>>> diff = voi-lor    # calc difference
>>> subplot(2,1,1)   # upper panel
>>> plot(wn,voi,'red',wn,lor,'blue')  # plot both profiles
>>> legend(['Voigt','Lorentz'])   # show legend
>>> title('Voigt and Lorentz profiles')   # show title
>>> subplot(2,1,2)   # lower panel
>>> plot(wn,diff)   # plot diffenence
>>> title('Voigt-Lorentz residual')   # show title
>>> show()   # show all figures

Calculate and plot absorption coefficients for ozone using Voigt 
profile. Spectra are calculated for 4 cases of thermodynamic parameters: 
(1 atm, 296 K), (5 atm, 296 K), (1 atm, 500 K), and (5 atm, 500 K)
>>> nu1,coef1 = absorptionCoefficient_Voigt(((3,1),),'O3',
        WavenumberStep=0.01,HITRAN_units=False,GammaL='gamma_self',
        Environment={'p':1,'T':296.})
>>> nu2,coef2 = absorptionCoefficient_Voigt(((3,1),),'O3',
        WavenumberStep=0.01,HITRAN_units=False,GammaL='gamma_self',
        Environment={'p':5,'T':296.})
>>> nu3,coef3 = absorptionCoefficient_Voigt(((3,1),),'O3',
        WavenumberStep=0.01,HITRAN_units=False,GammaL='gamma_self',
        Environment={'p':1,'T':500.})
>>> nu4,coef4 = absorptionCoefficient_Voigt(((3,1),),'O3',
        WavenumberStep=0.01,HITRAN_units=False,GammaL='gamma_self',
        Environment={'p':5,'T':500.})
>>> subplot(2,2,1); plot(nu1,coef1); title('O3 k(w): p=1 atm, T=296K')
>>> subplot(2,2,2); plot(nu2,coef2); title('O3 k(w): p=5 atm, T=296K')
>>> subplot(2,2,3); plot(nu3,coef3); title('O3 k(w): p=1 atm, T=500K')
>>> subplot(2,2,4); plot(nu4,coef4); title('O3 k(w): p=5 atm, T=500K')
>>> show()

Calculate and plot absorption, transmittance and radiance spectra for 1 atm 
and 296K. Path length is set to 10 m.
>>> nu,absorp = absorptionSpectrum(nu1,coef1,Environment={'l':1000.})
>>> nu,transm = transmittanceSpectrum(nu1,coef1,Environment={'l':1000.})
>>> nu,radian = radianceSpectrum(nu1,coef1,Environment={'l':1000.,'T':296.})
>>> subplot(2,2,1); plot(nu1,coef1,'r'); title('O3 k(w): p=1 atm, T=296K')
>>> subplot(2,2,2); plot(nu,absorp,'g'); title('O3 absorption: p=1 atm, T=296K')
>>> subplot(2,2,3); plot(nu,transm,'b'); title('O3 transmittance: p=1 atm, T=296K')
>>> subplot(2,2,4); plot(nu,radian,'y'); title('O3 radiance: p=1 atm, T=296K')
>>> show()

Calculate and compare high resolution spectrum for O3 with lower resolution
spectrum convoluted with an instrumental function of ideal Michelson interferometer.
>>> nu_,trans_,i1,i2,slit = convolveSpectrum(nu,transm,SlitFunction=SLIT_MICHELSON,Resolution=1.0,AF_wing=20.0)
>>> plot(nu,transm,'red',nu_,trans_,'blue'); legend(['HI-RES','Michelson']); show()

"""
def print_plotting_tutorial():
    pydoc.pager(plotting_tutorial_text)

def getHelp(arg=None):
    """
    This function provides interactive manuals and tutorials.
    """
    if arg==None:
        print('--------------------------------------------------------------')
        print('Hello, this is an interactive help system of HITRANonline API.')
        print('--------------------------------------------------------------')
        print('Run getHelp(.) with one of the following arguments:')
        print('    tutorial  -  interactive tutorials on HAPI')
        print('    units     -  units used in calculations')
        print('    index     -  index of available HAPI functions')
    elif arg=='tutorial':
        print('-----------------------------------')
        print('This is a tutorial section of help.')
        print('-----------------------------------')
        print('Please choose the subject of tutorial:')
        print('    data      -  downloading the data and working with it')
        print('    spectra   -  calculating spectral functions')
        print('    plotting  -  visualizing data with matplotlib')
        print('    python    -  Python quick start guide')
    elif arg=='python':
        print_python_tutorial()
    elif arg=='data':
        print_data_tutorial()
    elif arg=='spectra':
        print_spectra_tutorial()
    elif arg=='plotting':
        print_plotting_tutorial()
    elif arg=='index':
        print('------------------------------')
        print('FETCHING DATA:')
        print('------------------------------')
        print('  fetch')
        print('  fetch_by_ids')
        print('')
        print('------------------------------')
        print('WORKING WITH DATA:')
        print('------------------------------')
        print('  db_begin')
        print('  db_commit')
        print('  tableList')
        print('  describe')
        print('  select')
        print('  sort')
        print('  extractColumns')
        print('  getColumn')
        print('  getColumns')
        print('  dropTable')
        print('')
        print('------------------------------')
        print('CALCULATING SPECTRA:')
        print('------------------------------')
        print('  profiles')
        print('  partitionSum')
        print('  absorptionCoefficient_HT')
        print('  absorptionCoefficient_Voigt')
        print('  absorptionCoefficient_SDVoigt')
        print('  absorptionCoefficient_Lorentz')
        print('  absorptionCoefficient_Doppler')
        print('  transmittanceSpectrum')
        print('  absorptionSpectrum')
        print('  radianceSpectrum')
        print('')
        print('------------------------------')
        print('CONVOLVING SPECTRA:')
        print('------------------------------')
        print('  convolveSpectrum')
        print('  slit_functions')
        print('')
        print('------------------------------')
        print('INFO ON ISOTOPOLOGUES:')
        print('------------------------------')
        print('  ISO_ID')
        print('  abundance')
        print('  molecularMass')
        print('  moleculeName')
        print('  isotopologueName')
        print('')
        print('------------------------------')
        print('MISCELLANEOUS:')
        print('------------------------------')
        print('  getStickXY')
        print('  read_hotw')
    elif arg == ISO:
        print_iso()
    elif arg == ISO_ID:
        print_iso_id()
    elif arg == profiles:
        print_profiles()
    elif arg == slit_functions:
        print_slit_functions()
    else:
       help(arg)

    

# Get atmospheric (natural) abundance
# for a specified isotopologue
# M - molecule number
# I - isotopologue number
def abundance(M,I):
    """
    INPUT PARAMETERS: 
        M: HITRAN molecule number
        I: HITRAN isotopologue number
    OUTPUT PARAMETERS: 
        Abbundance: natural abundance
    ---
    DESCRIPTION:
        Return natural (Earth) abundance of HITRAN isotolopogue.
    ---
    EXAMPLE OF USAGE:
        ab = abundance(1,1) # H2O
    ---
    """
    return ISO[(M,I)][ISO_INDEX['abundance']]

# Get molecular mass
# for a specified isotopologue
# M - molecule number
# I - isotopologue number
def molecularMass(M,I):
    """
    INPUT PARAMETERS: 
        M: HITRAN molecule number
        I: HITRAN isotopologue number
    OUTPUT PARAMETERS: 
        MolMass: molecular mass
    ---
    DESCRIPTION:
        Return molecular mass of HITRAN isotolopogue.
    ---
    EXAMPLE OF USAGE:
        mass = molecularMass(1,1) # H2O
    ---
    """
    return ISO[(M,I)][ISO_INDEX['mass']]

# Get molecule name
# for a specified isotopologue
# M - molecule number
# I - isotopologue number
def moleculeName(M):
    """
    INPUT PARAMETERS: 
        M: HITRAN molecule number
    OUTPUT PARAMETERS: 
        MolName: molecular name
    ---
    DESCRIPTION:
        Return name of HITRAN molecule.
    ---
    EXAMPLE OF USAGE:
        molname = moleculeName(1) # H2O
    ---
    """
    return ISO[(M,1)][ISO_INDEX['mol_name']]

# Get isotopologue name
# for a specified isotopologue
# M - molecule number
# I - isotopologue number
def isotopologueName(M,I):
    """
    INPUT PARAMETERS: 
        M: HITRAN molecule number
        I: HITRAN isotopologue number
    OUTPUT PARAMETERS: 
        IsoMass: isotopologue mass
    ---
    DESCRIPTION:
        Return name of HITRAN isotolopogue.
    ---
    EXAMPLE OF USAGE:
        isoname = isotopologueName(1,1) # H2O
    ---
    """
    return ISO[(M,I)][ISO_INDEX['iso_name']]

# ----------------------- table list ----------------------------------
def tableList():
    """
    INPUT PARAMETERS: 
        none
    OUTPUT PARAMETERS: 
        TableList: a list of available tables
    ---
    DESCRIPTION:
        Return a list of tables present in database.
    ---
    EXAMPLE OF USAGE:
        lst = tableList()
    ---
    """

    return getTableList()

# ----------------------- describe ----------------------------------
def describe(TableName):
    """
    INPUT PARAMETERS: 
        TableName: name of the table to describe
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Print information about table, including 
        parameter names, formats and wavenumber range.
    ---
    EXAMPLE OF USAGE:
        describe('sampletab')
    ---
    """
    describeTable(TableName)

# ---------------------- /ISO.PY ---------------------------------------

def db_begin(db=None):
    """
    INPUT PARAMETERS: 
        db: database name (optional)
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Open a database connection. A database is stored 
        in a folder given in db input parameter.
        Default=data
    ---
    EXAMPLE OF USAGE:
        db_begin('bar')
    ---
    """
    databaseBegin(db)

def db_commit():
    """
    INPUT PARAMETERS: 
        none
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Commit all changes made to opened database.
        All tables will be saved in corresponding files.
    ---
    EXAMPLE OF USAGE:
        db_commit()
    ---
    """
    databaseCommit()

# ------------------ QUERY HITRAN ---------------------------------------

def comment(TableName,Comment):
    LOCAL_TABLE_CACHE[TableName]['header']['comment'] = Comment

def fetch_by_ids(TableName,iso_id_list,numin,numax,ParameterGroups=[],Parameters=[]):
    """
    INPUT PARAMETERS: 
        TableName:   local table name to fetch in (required)
        iso_id_list: list of isotopologue id's    (required)
        numin:       lower wavenumber bound       (required)
        numax:       upper wavenumber bound       (required)
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Download line-by-line data from HITRANonline server
        and save it to local table. The input parameter iso_id_list
        contains list of "global" isotopologue Ids (see help on ISO_ID).
        Note: this function is required if user wants to download
        multiple species into single table.
    ---
    EXAMPLE OF USAGE:
        fetch_by_ids('water',[1,2,3,4],4000,4100)
    ---
    """
    if type(iso_id_list) not in set([list,tuple]):
       iso_id_list = [iso_id_list]
    queryHITRAN(TableName,iso_id_list,numin,numax,
                pargroups=ParameterGroups,params=Parameters)
    iso_names = [ISO_ID[i][ISO_ID_INDEX['iso_name']] for i in iso_id_list]
    Comment = 'Contains lines for '+','.join(iso_names)
    Comment += ('\n in %.3f-%.3f wavenumber range' % (numin,numax))
    comment(TableName,Comment)

#def queryHITRAN(TableName,iso_id_list,numin,numax):
def fetch(TableName,M,I,numin,numax,ParameterGroups=[],Parameters=[]):
    """
    INPUT PARAMETERS: 
        TableName:   local table name to fetch in (required)
        M:           HITRAN molecule number       (required)
        I:           HITRAN isotopologue number   (required)
        numin:       lower wavenumber bound       (required)
        numax:       upper wavenumber bound       (required)
    OUTPUT PARAMETERS: 
        none
    ---
    DESCRIPTION:
        Download line-by-line data from HITRANonline server
        and save it to local table. The input parameters M and I
        are the HITRAN molecule and isotopologue numbers.
        This function results in a table containing single 
        isotopologue specie. To have multiple species in a 
        single table use fetch_by_ids instead.
    ---
    EXAMPLE OF USAGE:
        fetch('HOH',1,1,4000,4100)
    ---
    """
    queryHITRAN(TableName,[ISO[(M,I)][ISO_INDEX['id']]],numin,numax,
                pargroups=ParameterGroups,params=Parameters)
    iso_name = ISO[(M,I)][ISO_INDEX['iso_name']]
    Comment = 'Contains lines for '+iso_name
    Comment += ('\n in %.3f-%.3f wavenumber range' % (numin,numax))
    comment(TableName,Comment)

# ------------------ partition sum --------------------------------------

# ------------------- LAGRANGE INTERPOLATION ----------------------

#def AtoB(aa,bb,A,B,npt)
def AtoB(aa,A,B,npt):
#***************************
#...LaGrange 3- and 4-point interpolation
#...arrays A and B are the npt data points,  given aa, a value of the 
#...A variable, the routine will find the corresponding bb value
#
#...input:  aa
#...output: bb 
    for I in range(2,npt+1):
        if A[I-1] >= aa:
            if I < 3 or I == npt:
                J = I
                if I < 3: J = 3
                if I == npt: J = npt
                J = J-1   # zero index correction
                A0D1=A[J-2]-A[J-1]
                if A0D1 == 0.0: A0D1=0.0001
                A0D2=A[J-2]-A[J]
                if A0D2 == 0.0: A0D2=0.0000
                A1D1=A[J-1]-A[J-2]
                if A1D1 == 0.0: A1D1=0.0001
                A1D2=A[J-1]-A[J]
                if A1D2 == 0.0: A1D2=0.0001
                A2D1=A[J]-A[J-2]
                if A2D1 == 0.0: A2D1=0.0001
                A2D2=A[J]-A[J-1]
                if A2D2 == 0.0: A2D2=0.0001

                A0=(aa-A[J-1])*(aa-A[J])/(A0D1*A0D2)
                A1=(aa-A[J-2])*(aa-A[J])/(A1D1*A1D2)
                A2=(aa-A[J-2])*(aa-A[J-1])/(A2D1*A2D2)

                bb = A0*B[J-2] + A1*B[J-1] + A2*B[J]

            else:
                J = I
                J = J-1   # zero index correction
                A0D1=A[J-2]-A[J-1]
                if A0D1 == 0.0: A0D1=0.0001
                A0D2=A[J-2]-A[J]
                if A0D2 == 0.0: A0D2=0.0001
                A0D3 = (A[J-2]-A[J+1])
                if A0D3 == 0.0: A0D3=0.0001
                A1D1=A[J-1]-A[J-2]
                if A1D1 == 0.0: A1D1=0.0001
                A1D2=A[J-1]-A[J]
                if A1D2 == 0.0: A1D2=0.0001
                A1D3 = A[J-1]-A[J+1]
                if A1D3 == 0.0: A1D3=0.0001

                A2D1=A[J]-A[J-2]
                if A2D1 == 0.0: A2D1=0.0001
                A2D2=A[J]-A[J-1]
                if A2D2 == 0.0: A2D2=0.0001
                A2D3 = A[J]-A[J+1]
                if A2D3 == 0.0: A2D3=0.0001

                A3D1 = A[J+1]-A[J-2]
                if A3D1 == 0.0: A3D1=0.0001
                A3D2 = A[J+1]-A[J-1]
                if A3D2 == 0.0: A3D2=0.0001
                A3D3 = A[J+1]-A[J]
                if A3D3 == 0.0: A3D3=0.0001

                A0=(aa-A[J-1])*(aa-A[J])*(aa-A[J+1])
                A0=A0/(A0D1*A0D2*A0D3)
                A1=(aa-A[J-2])*(aa-A[J])*(aa-A[J+1])
                A1=A1/(A1D1*A1D2*A1D3)
                A2=(aa-A[J-2])*(aa-A[J-1])*(aa-A[J+1])
                A2=A2/(A2D1*A2D2*A2D3)
                A3=(aa-A[J-2])*(aa-A[J-1])*(aa-A[J])
                A3=A3/(A3D1*A3D2*A3D3)

                bb = A0*B[J-2] + A1*B[J-1] + A2*B[J] + A3*B[J+1]

            break

    return bb


#  --------------- ISOTOPOLOGUE HASH ----------------------

TIPS_ISO_HASH = {}

#  --------------- STATISTICAL WEIGHT HASH ----------------------

TIPS_GSI_HASH = {}

#  --------------- INTERPOLATION NODES ----------------------

Tdat = __FloatType__(  [60.,  85., 110., 135., 160., 185., 210., 235.,
                       260., 285., 310., 335., 360., 385., 410., 435., 460., 485.,
                       510., 535., 560., 585., 610., 635., 660., 685., 710., 735.,
                       760., 785., 810., 835., 860., 885., 910., 935., 960., 985.,
                      1010.,1035.,1060.,1085.,1110.,1135.,1160.,1185.,1210.,1235.,
                      1260.,1285.,1310.,1335.,1360.,1385.,1410.,1435.,1460.,1485.,
                      1510.,1535.,1560.,1585.,1610.,1635.,1660.,1685.,1710.,1735.,
                      1760.,1785.,1810.,1835.,1860.,1885.,1910.,1935.,1960.,1985.,
                      2010.,2035.,2060.,2085.,2110.,2135.,2160.,2185.,2210.,2235.,
                      2260.,2285.,2310.,2335.,2360.,2385.,2410.,2435.,2460.,2485.,
                      2510.,2535.,2560.,2585.,2610.,2635.,2660.,2685.,2710.,2735.,
                      2760.,2785.,2810.,2835.,2860.,2885.,2910.,2935.,2960.,2985.,
                      3010.]  )

TIPS_NPT = len(Tdat)


# REMARK
# float32 gives exactly the same results as fortran TIPS, because 
# all constants in the fortran code given as xx.xxE+-XX, i.e. 
# in single precision. By this fact all unsignificant figures 
# over single precision are filled with digital garbage


#  --------------- H2O 161: M = 1, I = 1 ---------------------
M = 1
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.16824E+02, 0.27771E+02, 0.40408E+02,
      0.54549E+02, 0.70054E+02, 0.86817E+02, 0.10475E+03, 0.12380E+03,
      0.14391E+03, 0.16503E+03, 0.18714E+03, 0.21021E+03, 0.23425E+03,
      0.25924E+03, 0.28518E+03, 0.31209E+03, 0.33997E+03, 0.36883E+03,
      0.39870E+03, 0.42959E+03, 0.46152E+03, 0.49452E+03, 0.52860E+03,
      0.56380E+03, 0.60015E+03, 0.63766E+03, 0.67637E+03, 0.71631E+03,
      0.75750E+03, 0.79999E+03, 0.84380E+03, 0.88897E+03, 0.93553E+03,
      0.98353E+03, 0.10330E+04, 0.10840E+04, 0.11365E+04, 0.11906E+04,
      0.12463E+04, 0.13037E+04, 0.13628E+04, 0.14237E+04, 0.14863E+04,
      0.15509E+04, 0.16173E+04, 0.16856E+04, 0.17559E+04, 0.18283E+04,
      0.19028E+04, 0.19793E+04, 0.20581E+04, 0.21391E+04, 0.22224E+04,
      0.23080E+04, 0.24067E+04, 0.24975E+04, 0.25908E+04, 0.26867E+04,
      0.27853E+04, 0.28865E+04, 0.29904E+04, 0.30972E+04, 0.32068E+04,
      0.33194E+04, 0.34349E+04, 0.35535E+04, 0.36752E+04, 0.38001E+04,
      0.39282E+04, 0.40597E+04, 0.41945E+04, 0.43327E+04, 0.44745E+04,
      0.46199E+04, 0.47688E+04, 0.49215E+04, 0.50780E+04, 0.52384E+04,
      0.54027E+04, 0.55710E+04, 0.57434E+04, 0.59200E+04, 0.61008E+04,
      0.62859E+04, 0.64754E+04, 0.66693E+04, 0.68679E+04, 0.70710E+04,
      0.72788E+04, 0.74915E+04, 0.77090E+04, 0.79315E+04, 0.81590E+04,
      0.83917E+04, 0.86296E+04, 0.88728E+04, 0.91214E+04, 0.93755E+04,
      0.96351E+04, 0.99005E+04, 0.10171E+05, 0.10448E+05, 0.10731E+05,
      0.11020E+05, 0.11315E+05, 0.11617E+05, 0.11924E+05, 0.12238E+05,
      0.12559E+05, 0.12886E+05, 0.13220E+05, 0.13561E+05, 0.13909E+05,
      0.14263E+05, 0.14625E+05, 0.14995E+05, 0.15371E+05, 0.15755E+05,
      0.16147E+05])


#  --------------- H2O 181: M = 1, I = 2 ---------------------
M = 1
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.15960E+02, 0.26999E+02, 0.39743E+02,
      0.54003E+02, 0.69639E+02, 0.86543E+02, 0.10463E+03, 0.12384E+03,
      0.14412E+03, 0.16542E+03, 0.18773E+03, 0.21103E+03, 0.23531E+03,
      0.26057E+03, 0.28681E+03, 0.31406E+03, 0.34226E+03, 0.37130E+03,
      0.40135E+03, 0.43243E+03, 0.46456E+03, 0.49777E+03, 0.53206E+03,
      0.56748E+03, 0.60405E+03, 0.64179E+03, 0.68074E+03, 0.72093E+03,
      0.76238E+03, 0.80513E+03, 0.84922E+03, 0.89467E+03, 0.94152E+03,
      0.98982E+03, 0.10396E+04, 0.10909E+04, 0.11437E+04, 0.11982E+04,
      0.12543E+04, 0.13120E+04, 0.13715E+04, 0.14328E+04, 0.14959E+04,
      0.15608E+04, 0.16276E+04, 0.16964E+04, 0.17672E+04, 0.18401E+04,
      0.19151E+04, 0.19922E+04, 0.20715E+04, 0.21531E+04, 0.22370E+04,
      0.23232E+04, 0.24118E+04, 0.25030E+04, 0.25967E+04, 0.26929E+04,
      0.27918E+04, 0.28934E+04, 0.29978E+04, 0.31050E+04, 0.32151E+04,
      0.33281E+04, 0.34441E+04, 0.35632E+04, 0.36854E+04, 0.38108E+04,
      0.39395E+04, 0.40715E+04, 0.42070E+04, 0.43459E+04, 0.44883E+04,
      0.46343E+04, 0.47840E+04, 0.49374E+04, 0.50946E+04, 0.52558E+04,
      0.54209E+04, 0.55900E+04, 0.57632E+04, 0.59407E+04, 0.61224E+04,
      0.63084E+04, 0.64988E+04, 0.66938E+04, 0.68933E+04, 0.70975E+04,
      0.73064E+04, 0.75202E+04, 0.77389E+04, 0.79625E+04, 0.81913E+04,
      0.84252E+04, 0.86644E+04, 0.89089E+04, 0.91588E+04, 0.94143E+04,
      0.96754E+04, 0.99422E+04, 0.10215E+05, 0.10493E+05, 0.10778E+05,
      0.11068E+05, 0.11365E+05, 0.11668E+05, 0.11977E+05, 0.12293E+05,
      0.12616E+05, 0.12945E+05, 0.13281E+05, 0.13624E+05, 0.13973E+05,
      0.14330E+05, 0.14694E+05, 0.15066E+05, 0.15445E+05, 0.15831E+05,
      0.16225E+05])


#  --------------- H2O 171: M = 1, I = 3 ---------------------
M = 1
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.95371E+02, 0.16134E+03, 0.23750E+03,
      0.32273E+03, 0.41617E+03, 0.51722E+03, 0.62540E+03, 0.74036E+03,
      0.86185E+03, 0.98970E+03, 0.11238E+04, 0.12642E+04, 0.14097E+04,
      0.15599E+04, 0.17159E+04, 0.18777E+04, 0.20453E+04, 0.22188E+04,
      0.23983E+04, 0.25840E+04, 0.27760E+04, 0.29743E+04, 0.31792E+04,
      0.33907E+04, 0.36091E+04, 0.38346E+04, 0.40672E+04, 0.43072E+04,
      0.45547E+04, 0.48100E+04, 0.50732E+04, 0.53446E+04, 0.56244E+04,
      0.59128E+04, 0.62100E+04, 0.65162E+04, 0.68317E+04, 0.71567E+04,
      0.74915E+04, 0.78363E+04, 0.81914E+04, 0.85571E+04, 0.89335E+04,
      0.93211E+04, 0.97200E+04, 0.10131E+05, 0.10553E+05, 0.10988E+05,
      0.11435E+05, 0.11895E+05, 0.12368E+05, 0.12855E+05, 0.13356E+05,
      0.13870E+05, 0.14399E+05, 0.14943E+05, 0.15502E+05, 0.16076E+05,
      0.16666E+05, 0.17272E+05, 0.17895E+05, 0.18534E+05, 0.19191E+05,
      0.19865E+05, 0.20557E+05, 0.21267E+05, 0.21996E+05, 0.22744E+05,
      0.23512E+05, 0.24299E+05, 0.25106E+05, 0.25935E+05, 0.26784E+05,
      0.27655E+05, 0.28547E+05, 0.29462E+05, 0.30400E+05, 0.31361E+05,
      0.32345E+05, 0.33353E+05, 0.34386E+05, 0.35444E+05, 0.36527E+05,
      0.37637E+05, 0.38772E+05, 0.39934E+05, 0.41124E+05, 0.42341E+05,
      0.43587E+05, 0.44861E+05, 0.46165E+05, 0.47498E+05, 0.48862E+05,
      0.50256E+05, 0.51682E+05, 0.53139E+05, 0.54629E+05, 0.56152E+05,
      0.57708E+05, 0.59299E+05, 0.60923E+05, 0.62583E+05, 0.64279E+05,
      0.66011E+05, 0.67779E+05, 0.69585E+05, 0.71429E+05, 0.73312E+05,
      0.75234E+05, 0.77195E+05, 0.79197E+05, 0.81240E+05, 0.83325E+05,
      0.85452E+05, 0.87622E+05, 0.89835E+05, 0.92093E+05, 0.94395E+05,
      0.96743E+05])


#  --------------- H2O 162: M = 1, I = 4 ---------------------
M = 1
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.75792E+02, 0.12986E+03, 0.19244E+03,
      0.26253E+03, 0.33942E+03, 0.42259E+03, 0.51161E+03, 0.60619E+03,
      0.70609E+03, 0.81117E+03, 0.92132E+03, 0.10365E+04, 0.11567E+04,
      0.12820E+04, 0.14124E+04, 0.15481E+04, 0.16891E+04, 0.18355E+04,
      0.19876E+04, 0.21455E+04, 0.23092E+04, 0.24791E+04, 0.26551E+04,
      0.28376E+04, 0.30268E+04, 0.32258E+04, 0.34288E+04, 0.36392E+04,
      0.38571E+04, 0.40828E+04, 0.43165E+04, 0.45584E+04, 0.48089E+04,
      0.50681E+04, 0.53363E+04, 0.56139E+04, 0.59009E+04, 0.61979E+04,
      0.65049E+04, 0.68224E+04, 0.71506E+04, 0.74898E+04, 0.78403E+04,
      0.82024E+04, 0.85765E+04, 0.89628E+04, 0.93618E+04, 0.97736E+04,
      0.10199E+05, 0.10637E+05, 0.11090E+05, 0.11557E+05, 0.12039E+05,
      0.12535E+05, 0.13047E+05, 0.13575E+05, 0.14119E+05, 0.14679E+05,
      0.15257E+05, 0.15851E+05, 0.16464E+05, 0.17094E+05, 0.17743E+05,
      0.18411E+05, 0.19098E+05, 0.19805E+05, 0.20532E+05, 0.21280E+05,
      0.22049E+05, 0.22840E+05, 0.23652E+05, 0.24487E+05, 0.25345E+05,
      0.26227E+05, 0.27132E+05, 0.28062E+05, 0.29016E+05, 0.29997E+05,
      0.31002E+05, 0.32035E+05, 0.33094E+05, 0.34180E+05, 0.35295E+05,
      0.36438E+05, 0.37610E+05, 0.38812E+05, 0.40044E+05, 0.41306E+05,
      0.42600E+05, 0.43926E+05, 0.45284E+05, 0.46675E+05, 0.48100E+05,
      0.49559E+05, 0.51053E+05, 0.52583E+05, 0.54148E+05, 0.55750E+05,
      0.57390E+05, 0.59067E+05, 0.60783E+05, 0.62539E+05, 0.64334E+05,
      0.66170E+05, 0.68047E+05, 0.69967E+05, 0.71929E+05, 0.73934E+05,
      0.75983E+05, 0.78078E+05, 0.80217E+05, 0.82403E+05, 0.84636E+05,
      0.86917E+05, 0.89246E+05, 0.91625E+05, 0.94053E+05, 0.96533E+05,
      0.99064E+05])


#  --------------- H2O 182: M = 1, I = 5 ---------------------
M = 1
I = 5
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.82770E+02, 0.13749E+03, 0.20083E+03,
      0.27176E+03, 0.34955E+03, 0.43370E+03, 0.52376E+03, 0.61944E+03,
      0.72050E+03, 0.82679E+03, 0.93821E+03, 0.10547E+04, 0.11763E+04,
      0.13031E+04, 0.14350E+04, 0.15723E+04, 0.17150E+04, 0.18633E+04,
      0.20172E+04, 0.21770E+04, 0.23429E+04, 0.25149E+04, 0.26934E+04,
      0.28784E+04, 0.30702E+04, 0.32690E+04, 0.34750E+04, 0.36885E+04,
      0.39096E+04, 0.41386E+04, 0.43758E+04, 0.46213E+04, 0.48755E+04,
      0.51386E+04, 0.54109E+04, 0.56927E+04, 0.59841E+04, 0.62856E+04,
      0.65973E+04, 0.69197E+04, 0.72529E+04, 0.75973E+04, 0.79533E+04,
      0.83210E+04, 0.87009E+04, 0.90933E+04, 0.94985E+04, 0.99168E+04,
      0.10348E+05, 0.10794E+05, 0.11254E+05, 0.11728E+05, 0.12217E+05,
      0.12722E+05, 0.13242E+05, 0.13778E+05, 0.14331E+05, 0.14900E+05,
      0.15486E+05, 0.16091E+05, 0.16713E+05, 0.17353E+05, 0.18012E+05,
      0.18691E+05, 0.19389E+05, 0.20108E+05, 0.20847E+05, 0.21607E+05,
      0.22388E+05, 0.23191E+05, 0.24017E+05, 0.24866E+05, 0.25738E+05,
      0.26633E+05, 0.27553E+05, 0.28498E+05, 0.29468E+05, 0.30464E+05,
      0.31486E+05, 0.32536E+05, 0.33612E+05, 0.34716E+05, 0.35849E+05,
      0.37011E+05, 0.38202E+05, 0.39424E+05, 0.40676E+05, 0.41959E+05,
      0.43274E+05, 0.44622E+05, 0.46002E+05, 0.47416E+05, 0.48864E+05,
      0.50348E+05, 0.51866E+05, 0.53421E+05, 0.55012E+05, 0.56640E+05,
      0.58307E+05, 0.60012E+05, 0.61757E+05, 0.63541E+05, 0.65366E+05,
      0.67233E+05, 0.69141E+05, 0.71092E+05, 0.73087E+05, 0.75125E+05,
      0.77209E+05, 0.79338E+05, 0.81513E+05, 0.83736E+05, 0.86006E+05,
      0.88324E+05, 0.90693E+05, 0.93111E+05, 0.95580E+05, 0.98100E+05,
      0.10067E+06])


#  --------------- H2O 172: M = 1, I = 6 ---------------------
M = 1
I = 6
TIPS_GSI_HASH[(M,I)] = __FloatType__(36.)
TIPS_ISO_HASH[(M,I)] = float32([0.49379E+03, 0.82021E+03, 0.11980E+04,
      0.16211E+04, 0.20851E+04, 0.25870E+04, 0.31242E+04, 0.36949E+04,
      0.42977E+04, 0.49317E+04, 0.55963E+04, 0.62911E+04, 0.70164E+04,
      0.77722E+04, 0.85591E+04, 0.93777E+04, 0.10228E+05, 0.11112E+05,
      0.12030E+05, 0.12983E+05, 0.13971E+05, 0.14997E+05, 0.16061E+05,
      0.17163E+05, 0.18306E+05, 0.19491E+05, 0.20719E+05, 0.21991E+05,
      0.23309E+05, 0.24673E+05, 0.26086E+05, 0.27549E+05, 0.29064E+05,
      0.30631E+05, 0.32254E+05, 0.33932E+05, 0.35669E+05, 0.37464E+05,
      0.39321E+05, 0.41242E+05, 0.43227E+05, 0.45279E+05, 0.47399E+05,
      0.49589E+05, 0.51852E+05, 0.54189E+05, 0.56602E+05, 0.59094E+05,
      0.61666E+05, 0.64320E+05, 0.67058E+05, 0.69883E+05, 0.72796E+05,
      0.75801E+05, 0.78899E+05, 0.82092E+05, 0.85382E+05, 0.88773E+05,
      0.92266E+05, 0.95863E+05, 0.99568E+05, 0.10338E+06, 0.10731E+06,
      0.11135E+06, 0.11551E+06, 0.11979E+06, 0.12419E+06, 0.12871E+06,
      0.13337E+06, 0.13815E+06, 0.14307E+06, 0.14812E+06, 0.15331E+06,
      0.15865E+06, 0.16412E+06, 0.16975E+06, 0.17553E+06, 0.18146E+06,
      0.18754E+06, 0.19379E+06, 0.20020E+06, 0.20678E+06, 0.21352E+06,
      0.22044E+06, 0.22753E+06, 0.23480E+06, 0.24226E+06, 0.24990E+06,
      0.25773E+06, 0.26575E+06, 0.27397E+06, 0.28239E+06, 0.29102E+06,
      0.29985E+06, 0.30889E+06, 0.31814E+06, 0.32762E+06, 0.33731E+06,
      0.34724E+06, 0.35739E+06, 0.36777E+06, 0.37840E+06, 0.38926E+06,
      0.40038E+06, 0.41174E+06, 0.42335E+06, 0.43523E+06, 0.44737E+06,
      0.45977E+06, 0.47245E+06, 0.48540E+06, 0.49863E+06, 0.51214E+06,
      0.52595E+06, 0.54005E+06, 0.55444E+06, 0.56914E+06, 0.58415E+06,
      0.59947E+06])


#  --------------- CO2 626: M = 2, I = 1 ---------------------
M = 2
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.53642E+02, 0.75947E+02, 0.98292E+02,
      0.12078E+03, 0.14364E+03, 0.16714E+03, 0.19160E+03, 0.21731E+03,
      0.24454E+03, 0.27355E+03, 0.30456E+03, 0.33778E+03, 0.37343E+03,
      0.41170E+03, 0.45280E+03, 0.49692E+03, 0.54427E+03, 0.59505E+03,
      0.64948E+03, 0.70779E+03, 0.77019E+03, 0.83693E+03, 0.90825E+03,
      0.98440E+03, 0.10656E+04, 0.11522E+04, 0.12445E+04, 0.13427E+04,
      0.14471E+04, 0.15580E+04, 0.16759E+04, 0.18009E+04, 0.19334E+04,
      0.20739E+04, 0.22225E+04, 0.23798E+04, 0.25462E+04, 0.27219E+04,
      0.29074E+04, 0.31032E+04, 0.33097E+04, 0.35272E+04, 0.37564E+04,
      0.39976E+04, 0.42514E+04, 0.45181E+04, 0.47985E+04, 0.50929E+04,
      0.54019E+04, 0.57260E+04, 0.60659E+04, 0.64221E+04, 0.67952E+04,
      0.71859E+04, 0.75946E+04, 0.80222E+04, 0.84691E+04, 0.89362E+04,
      0.94241E+04, 0.99335E+04, 0.10465E+05, 0.11020E+05, 0.11598E+05,
      0.12201E+05, 0.12828E+05, 0.13482E+05, 0.14163E+05, 0.14872E+05,
      0.15609E+05, 0.16376E+05, 0.17173E+05, 0.18001E+05, 0.18861E+05,
      0.19754E+05, 0.20682E+05, 0.21644E+05, 0.22643E+05, 0.23678E+05,
      0.24752E+05, 0.25865E+05, 0.27018E+05, 0.28212E+05, 0.29449E+05,
      0.30730E+05, 0.32055E+05, 0.33426E+05, 0.34845E+05, 0.36312E+05,
      0.37828E+05, 0.39395E+05, 0.41015E+05, 0.42688E+05, 0.44416E+05,
      0.46199E+05, 0.48041E+05, 0.49942E+05, 0.51902E+05, 0.53925E+05,
      0.56011E+05, 0.58162E+05, 0.60379E+05, 0.62664E+05, 0.65019E+05,
      0.67444E+05, 0.69942E+05, 0.72515E+05, 0.75163E+05, 0.77890E+05,
      0.80695E+05, 0.83582E+05, 0.86551E+05, 0.89605E+05, 0.92746E+05,
      0.95975E+05, 0.99294E+05, 0.10271E+06, 0.10621E+06, 0.10981E+06,
      0.11351E+06])

      
#  --------------- CO2 636: M = 2, I = 2 ---------------------
M = 2
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.10728E+03, 0.15189E+03, 0.19659E+03,
      0.24164E+03, 0.28753E+03, 0.33486E+03, 0.38429E+03, 0.43643E+03,
      0.49184E+03, 0.55104E+03, 0.61449E+03, 0.68263E+03, 0.75589E+03,
      0.83468E+03, 0.91943E+03, 0.10106E+04, 0.11085E+04, 0.12137E+04,
      0.13266E+04, 0.14477E+04, 0.15774E+04, 0.17163E+04, 0.18649E+04,
      0.20237E+04, 0.21933E+04, 0.23743E+04, 0.25673E+04, 0.27729E+04,
      0.29917E+04, 0.32245E+04, 0.34718E+04, 0.37345E+04, 0.40132E+04,
      0.43087E+04, 0.46218E+04, 0.49533E+04, 0.53041E+04, 0.56749E+04,
      0.60668E+04, 0.64805E+04, 0.69171E+04, 0.73774E+04, 0.78626E+04,
      0.83736E+04, 0.89114E+04, 0.94772E+04, 0.10072E+05, 0.10697E+05,
      0.11353E+05, 0.12042E+05, 0.12765E+05, 0.13523E+05, 0.14317E+05,
      0.15148E+05, 0.16019E+05, 0.16930E+05, 0.17883E+05, 0.18879E+05,
      0.19920E+05, 0.21008E+05, 0.22143E+05, 0.23328E+05, 0.24563E+05,
      0.25852E+05, 0.27195E+05, 0.28594E+05, 0.30051E+05, 0.31568E+05,
      0.33146E+05, 0.34788E+05, 0.36496E+05, 0.38271E+05, 0.40115E+05,
      0.42031E+05, 0.44021E+05, 0.46086E+05, 0.48230E+05, 0.50453E+05,
      0.52759E+05, 0.55150E+05, 0.57628E+05, 0.60195E+05, 0.62854E+05,
      0.65608E+05, 0.68459E+05, 0.71409E+05, 0.74461E+05, 0.77618E+05,
      0.80883E+05, 0.84258E+05, 0.87746E+05, 0.91350E+05, 0.95073E+05,
      0.98918E+05, 0.10289E+06, 0.10698E+06, 0.11121E+06, 0.11558E+06,
      0.12008E+06, 0.12472E+06, 0.12950E+06, 0.13443E+06, 0.13952E+06,
      0.14475E+06, 0.15015E+06, 0.15571E+06, 0.16143E+06, 0.16732E+06,
      0.17338E+06, 0.17962E+06, 0.18604E+06, 0.19264E+06, 0.19943E+06,
      0.20642E+06, 0.21360E+06, 0.22098E+06, 0.22856E+06, 0.23636E+06,
      0.24436E+06])
    
    
#  --------------- CO2 628: M = 2, I = 3 ---------------------
M = 2
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.11368E+03, 0.16096E+03, 0.20833E+03,
      0.25603E+03, 0.30452E+03, 0.35442E+03, 0.40640E+03, 0.46110E+03,
      0.51910E+03, 0.58093E+03, 0.64709E+03, 0.71804E+03, 0.79422E+03,
      0.87607E+03, 0.96402E+03, 0.10585E+04, 0.11600E+04, 0.12689E+04,
      0.13857E+04, 0.15108E+04, 0.16449E+04, 0.17883E+04, 0.19416E+04,
      0.21054E+04, 0.22803E+04, 0.24668E+04, 0.26655E+04, 0.28770E+04,
      0.31021E+04, 0.33414E+04, 0.35956E+04, 0.38654E+04, 0.41516E+04,
      0.44549E+04, 0.47761E+04, 0.51160E+04, 0.54755E+04, 0.58555E+04,
      0.62568E+04, 0.66804E+04, 0.71273E+04, 0.75982E+04, 0.80944E+04,
      0.86169E+04, 0.91666E+04, 0.97446E+04, 0.10352E+05, 0.10990E+05,
      0.11660E+05, 0.12363E+05, 0.13101E+05, 0.13874E+05, 0.14683E+05,
      0.15531E+05, 0.16418E+05, 0.17347E+05, 0.18317E+05, 0.19332E+05,
      0.20392E+05, 0.21499E+05, 0.22654E+05, 0.23859E+05, 0.25116E+05,
      0.26426E+05, 0.27792E+05, 0.29214E+05, 0.30695E+05, 0.32236E+05,
      0.33840E+05, 0.35508E+05, 0.37242E+05, 0.39045E+05, 0.40917E+05,
      0.42862E+05, 0.44881E+05, 0.46977E+05, 0.49152E+05, 0.51407E+05,
      0.53746E+05, 0.56171E+05, 0.58683E+05, 0.61286E+05, 0.63981E+05,
      0.66772E+05, 0.69661E+05, 0.72650E+05, 0.75742E+05, 0.78940E+05,
      0.82246E+05, 0.85664E+05, 0.89196E+05, 0.92845E+05, 0.96613E+05,
      0.10050E+06, 0.10452E+06, 0.10867E+06, 0.11295E+06, 0.11736E+06,
      0.12191E+06, 0.12661E+06, 0.13145E+06, 0.13643E+06, 0.14157E+06,
      0.14687E+06, 0.15232E+06, 0.15794E+06, 0.16372E+06, 0.16968E+06,
      0.17580E+06, 0.18211E+06, 0.18859E+06, 0.19526E+06, 0.20213E+06,
      0.20918E+06, 0.21643E+06, 0.22388E+06, 0.23154E+06, 0.23941E+06,
      0.24750E+06])
    
    
#  --------------- CO2 627: M = 2, I = 4 ---------------------
M = 2
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.66338E+03, 0.93923E+03, 0.12156E+04,
      0.14938E+04, 0.17766E+04, 0.20676E+04, 0.23705E+04, 0.26891E+04,
      0.30267E+04, 0.33866E+04, 0.37714E+04, 0.41839E+04, 0.46267E+04,
      0.51023E+04, 0.56132E+04, 0.61618E+04, 0.67508E+04, 0.73827E+04,
      0.80603E+04, 0.87863E+04, 0.95636E+04, 0.10395E+05, 0.11284E+05,
      0.12233E+05, 0.13246E+05, 0.14326E+05, 0.15477E+05, 0.16702E+05,
      0.18005E+05, 0.19390E+05, 0.20861E+05, 0.22422E+05, 0.24077E+05,
      0.25832E+05, 0.27689E+05, 0.29655E+05, 0.31734E+05, 0.33931E+05,
      0.36250E+05, 0.38698E+05, 0.41280E+05, 0.44002E+05, 0.46869E+05,
      0.49886E+05, 0.53062E+05, 0.56400E+05, 0.59909E+05, 0.63594E+05,
      0.67462E+05, 0.71521E+05, 0.75777E+05, 0.80238E+05, 0.84911E+05,
      0.89804E+05, 0.94925E+05, 0.10028E+06, 0.10588E+06, 0.11173E+06,
      0.11785E+06, 0.12423E+06, 0.13090E+06, 0.13785E+06, 0.14510E+06,
      0.15265E+06, 0.16053E+06, 0.16873E+06, 0.17727E+06, 0.18615E+06,
      0.19540E+06, 0.20501E+06, 0.21501E+06, 0.22540E+06, 0.23619E+06,
      0.24740E+06, 0.25904E+06, 0.27112E+06, 0.28365E+06, 0.29664E+06,
      0.31012E+06, 0.32409E+06, 0.33856E+06, 0.35356E+06, 0.36908E+06,
      0.38516E+06, 0.40180E+06, 0.41902E+06, 0.43683E+06, 0.45525E+06,
      0.47429E+06, 0.49397E+06, 0.51431E+06, 0.53532E+06, 0.55702E+06,
      0.57943E+06, 0.60256E+06, 0.62644E+06, 0.65107E+06, 0.67648E+06,
      0.70269E+06, 0.72972E+06, 0.75758E+06, 0.78629E+06, 0.81588E+06,
      0.84636E+06, 0.87775E+06, 0.91008E+06, 0.94337E+06, 0.97763E+06,
      0.10129E+07, 0.10492E+07, 0.10865E+07, 0.11249E+07, 0.11644E+07,
      0.12050E+07, 0.12467E+07, 0.12896E+07, 0.13337E+07, 0.13789E+07,
      0.14255E+07])
    

#  --------------- CO2 638: M = 2, I = 5 ---------------------
M = 2
I = 5
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.22737E+03, 0.32194E+03, 0.41671E+03,
      0.51226E+03, 0.60963E+03, 0.71017E+03, 0.81528E+03, 0.92628E+03,
      0.10444E+04, 0.11707E+04, 0.13061E+04, 0.14518E+04, 0.16085E+04,
      0.17772E+04, 0.19588E+04, 0.21542E+04, 0.23644E+04, 0.25903E+04,
      0.28330E+04, 0.30934E+04, 0.33726E+04, 0.36717E+04, 0.39918E+04,
      0.43342E+04, 0.47001E+04, 0.50907E+04, 0.55074E+04, 0.59515E+04,
      0.64244E+04, 0.69276E+04, 0.74626E+04, 0.80310E+04, 0.86344E+04,
      0.92744E+04, 0.99528E+04, 0.10671E+05, 0.11432E+05, 0.12236E+05,
      0.13086E+05, 0.13984E+05, 0.14932E+05, 0.15932E+05, 0.16985E+05,
      0.18096E+05, 0.19265E+05, 0.20495E+05, 0.21788E+05, 0.23148E+05,
      0.24576E+05, 0.26075E+05, 0.27648E+05, 0.29298E+05, 0.31027E+05,
      0.32839E+05, 0.34736E+05, 0.36721E+05, 0.38798E+05, 0.40970E+05,
      0.43240E+05, 0.45611E+05, 0.48087E+05, 0.50671E+05, 0.53368E+05,
      0.56180E+05, 0.59111E+05, 0.62165E+05, 0.65347E+05, 0.68659E+05,
      0.72107E+05, 0.75694E+05, 0.79425E+05, 0.83303E+05, 0.87334E+05,
      0.91522E+05, 0.95872E+05, 0.10039E+06, 0.10507E+06, 0.10994E+06,
      0.11498E+06, 0.12021E+06, 0.12563E+06, 0.13125E+06, 0.13707E+06,
      0.14309E+06, 0.14933E+06, 0.15579E+06, 0.16247E+06, 0.16938E+06,
      0.17653E+06, 0.18392E+06, 0.19156E+06, 0.19946E+06, 0.20761E+06,
      0.21604E+06, 0.22473E+06, 0.23371E+06, 0.24298E+06, 0.25254E+06,
      0.26240E+06, 0.27258E+06, 0.28307E+06, 0.29388E+06, 0.30502E+06,
      0.31651E+06, 0.32834E+06, 0.34052E+06, 0.35307E+06, 0.36599E+06,
      0.37929E+06, 0.39298E+06, 0.40706E+06, 0.42155E+06, 0.43645E+06,
      0.45178E+06, 0.46753E+06, 0.48373E+06, 0.50038E+06, 0.51748E+06,
      0.53506E+06])


#  --------------- CO2 637: M = 2, I = 6 ---------------------
M = 2
I = 6
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.13267E+04, 0.18785E+04, 0.24314E+04,
      0.29888E+04, 0.35566E+04, 0.41426E+04, 0.47550E+04, 0.54013E+04,
      0.60886E+04, 0.68232E+04, 0.76109E+04, 0.84574E+04, 0.93678E+04,
      0.10348E+05, 0.11402E+05, 0.12536E+05, 0.13755E+05, 0.15065E+05,
      0.16471E+05, 0.17980E+05, 0.19598E+05, 0.21330E+05, 0.23184E+05,
      0.25166E+05, 0.27283E+05, 0.29543E+05, 0.31953E+05, 0.34521E+05,
      0.37256E+05, 0.40164E+05, 0.43256E+05, 0.46541E+05, 0.50026E+05,
      0.53723E+05, 0.57641E+05, 0.61790E+05, 0.66180E+05, 0.70823E+05,
      0.75729E+05, 0.80910E+05, 0.86378E+05, 0.92145E+05, 0.98224E+05,
      0.10463E+06, 0.11137E+06, 0.11846E+06, 0.12592E+06, 0.13375E+06,
      0.14198E+06, 0.15062E+06, 0.15969E+06, 0.16920E+06, 0.17916E+06,
      0.18959E+06, 0.20052E+06, 0.21196E+06, 0.22392E+06, 0.23642E+06,
      0.24949E+06, 0.26314E+06, 0.27740E+06, 0.29227E+06, 0.30779E+06,
      0.32398E+06, 0.34085E+06, 0.35842E+06, 0.37673E+06, 0.39579E+06,
      0.41563E+06, 0.43626E+06, 0.45772E+06, 0.48003E+06, 0.50322E+06,
      0.52730E+06, 0.55232E+06, 0.57829E+06, 0.60524E+06, 0.63320E+06,
      0.66219E+06, 0.69226E+06, 0.72342E+06, 0.75571E+06, 0.78916E+06,
      0.82380E+06, 0.85966E+06, 0.89678E+06, 0.93518E+06, 0.97490E+06,
      0.10160E+07, 0.10585E+07, 0.11023E+07, 0.11477E+07, 0.11946E+07,
      0.12430E+07, 0.12929E+07, 0.13445E+07, 0.13977E+07, 0.14526E+07,
      0.15093E+07, 0.15677E+07, 0.16280E+07, 0.16901E+07, 0.17541E+07,
      0.18200E+07, 0.18880E+07, 0.19579E+07, 0.20300E+07, 0.21042E+07,
      0.21805E+07, 0.22591E+07, 0.23400E+07, 0.24232E+07, 0.25087E+07,
      0.25967E+07, 0.26871E+07, 0.27801E+07, 0.28757E+07, 0.29739E+07,
      0.30747E+07])


#  --------------- CO2 828: M = 2, I = 7 ---------------------
M = 2
I = 7
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.60334E+02, 0.85430E+02, 0.11058E+03,
      0.13590E+03, 0.16167E+03, 0.18821E+03, 0.21588E+03, 0.24502E+03,
      0.27595E+03, 0.30896E+03, 0.34431E+03, 0.38225E+03, 0.42301E+03,
      0.46684E+03, 0.51397E+03, 0.56464E+03, 0.61907E+03, 0.67753E+03,
      0.74027E+03, 0.80753E+03, 0.87961E+03, 0.95676E+03, 0.10393E+04,
      0.11275E+04, 0.12217E+04, 0.13222E+04, 0.14293E+04, 0.15434E+04,
      0.16648E+04, 0.17940E+04, 0.19312E+04, 0.20769E+04, 0.22315E+04,
      0.23954E+04, 0.25691E+04, 0.27529E+04, 0.29474E+04, 0.31530E+04,
      0.33702E+04, 0.35995E+04, 0.38414E+04, 0.40965E+04, 0.43654E+04,
      0.46484E+04, 0.49464E+04, 0.52598E+04, 0.55892E+04, 0.59353E+04,
      0.62988E+04, 0.66803E+04, 0.70804E+04, 0.74998E+04, 0.79394E+04,
      0.83998E+04, 0.88817E+04, 0.93859E+04, 0.99132E+04, 0.10464E+05,
      0.11040E+05, 0.11642E+05, 0.12270E+05, 0.12925E+05, 0.13609E+05,
      0.14321E+05, 0.15064E+05, 0.15838E+05, 0.16643E+05, 0.17482E+05,
      0.18355E+05, 0.19263E+05, 0.20207E+05, 0.21188E+05, 0.22208E+05,
      0.23267E+05, 0.24366E+05, 0.25508E+05, 0.26692E+05, 0.27921E+05,
      0.29195E+05, 0.30516E+05, 0.31886E+05, 0.33304E+05, 0.34773E+05,
      0.36294E+05, 0.37869E+05, 0.39499E+05, 0.41185E+05, 0.42929E+05,
      0.44732E+05, 0.46596E+05, 0.48522E+05, 0.50513E+05, 0.52569E+05,
      0.54692E+05, 0.56884E+05, 0.59146E+05, 0.61481E+05, 0.63890E+05,
      0.66375E+05, 0.68937E+05, 0.71578E+05, 0.74301E+05, 0.77107E+05,
      0.79998E+05, 0.82976E+05, 0.86043E+05, 0.89201E+05, 0.92452E+05,
      0.95799E+05, 0.99242E+05, 0.10278E+06, 0.10643E+06, 0.11018E+06,
      0.11403E+06, 0.11799E+06, 0.12206E+06, 0.12625E+06, 0.13055E+06,
      0.13497E+06])


#  --------------- CO2 728: M = 2, I = 8 ---------------------
M = 2
I = 8
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.70354E+03, 0.99615E+03, 0.12893E+04,
      0.15846E+04, 0.18848E+04, 0.21940E+04, 0.25162E+04, 0.28554E+04,
      0.32152E+04, 0.35991E+04, 0.40099E+04, 0.44507E+04, 0.49242E+04,
      0.54332E+04, 0.59802E+04, 0.65681E+04, 0.71996E+04, 0.78776E+04,
      0.86050E+04, 0.93847E+04, 0.10220E+05, 0.11114E+05, 0.12070E+05,
      0.13091E+05, 0.14182E+05, 0.15345E+05, 0.16585E+05, 0.17906E+05,
      0.19311E+05, 0.20805E+05, 0.22393E+05, 0.24078E+05, 0.25865E+05,
      0.27760E+05, 0.29768E+05, 0.31893E+05, 0.34140E+05, 0.36516E+05,
      0.39025E+05, 0.41674E+05, 0.44469E+05, 0.47416E+05, 0.50520E+05,
      0.53789E+05, 0.57229E+05, 0.60847E+05, 0.64650E+05, 0.68645E+05,
      0.72840E+05, 0.77242E+05, 0.81859E+05, 0.86699E+05, 0.91770E+05,
      0.97081E+05, 0.10264E+06, 0.10846E+06, 0.11454E+06, 0.12090E+06,
      0.12754E+06, 0.13447E+06, 0.14171E+06, 0.14927E+06, 0.15715E+06,
      0.16536E+06, 0.17392E+06, 0.18284E+06, 0.19213E+06, 0.20179E+06,
      0.21185E+06, 0.22231E+06, 0.23319E+06, 0.24450E+06, 0.25625E+06,
      0.26845E+06, 0.28112E+06, 0.29427E+06, 0.30791E+06, 0.32206E+06,
      0.33674E+06, 0.35196E+06, 0.36772E+06, 0.38406E+06, 0.40098E+06,
      0.41850E+06, 0.43663E+06, 0.45539E+06, 0.47480E+06, 0.49488E+06,
      0.51564E+06, 0.53710E+06, 0.55928E+06, 0.58219E+06, 0.60586E+06,
      0.63029E+06, 0.65553E+06, 0.68157E+06, 0.70844E+06, 0.73616E+06,
      0.76476E+06, 0.79424E+06, 0.82464E+06, 0.85597E+06, 0.88826E+06,
      0.92153E+06, 0.95580E+06, 0.99108E+06, 0.10274E+07, 0.10648E+07,
      0.11033E+07, 0.11429E+07, 0.11837E+07, 0.12256E+07, 0.12687E+07,
      0.13131E+07, 0.13586E+07, 0.14055E+07, 0.14536E+07, 0.15031E+07,
      0.15539E+07])


#  --------------- CO2 727: M = 2, I = 9 ---------------------
M = 2
I = 9
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.20518E+04, 0.29051E+04, 0.37601E+04,
      0.46209E+04, 0.54961E+04, 0.63969E+04, 0.73353E+04, 0.83227E+04,
      0.93698E+04, 0.10486E+05, 0.11681E+05, 0.12962E+05, 0.14337E+05,
      0.15815E+05, 0.17403E+05, 0.19110E+05, 0.20942E+05, 0.22909E+05,
      0.25018E+05, 0.27278E+05, 0.29699E+05, 0.32290E+05, 0.35060E+05,
      0.38019E+05, 0.41177E+05, 0.44545E+05, 0.48135E+05, 0.51957E+05,
      0.56023E+05, 0.60346E+05, 0.64938E+05, 0.69812E+05, 0.74981E+05,
      0.80461E+05, 0.86264E+05, 0.92406E+05, 0.98902E+05, 0.10577E+06,
      0.11302E+06, 0.12067E+06, 0.12875E+06, 0.13726E+06, 0.14622E+06,
      0.15566E+06, 0.16559E+06, 0.17604E+06, 0.18702E+06, 0.19855E+06,
      0.21066E+06, 0.22336E+06, 0.23669E+06, 0.25065E+06, 0.26528E+06,
      0.28061E+06, 0.29664E+06, 0.31342E+06, 0.33096E+06, 0.34930E+06,
      0.36845E+06, 0.38845E+06, 0.40933E+06, 0.43111E+06, 0.45383E+06,
      0.47751E+06, 0.50219E+06, 0.52790E+06, 0.55466E+06, 0.58252E+06,
      0.61151E+06, 0.64166E+06, 0.67300E+06, 0.70558E+06, 0.73943E+06,
      0.77458E+06, 0.81108E+06, 0.84896E+06, 0.88827E+06, 0.92904E+06,
      0.97131E+06, 0.10151E+07, 0.10605E+07, 0.11076E+07, 0.11563E+07,
      0.12068E+07, 0.12590E+07, 0.13130E+07, 0.13689E+07, 0.14267E+07,
      0.14865E+07, 0.15483E+07, 0.16121E+07, 0.16781E+07, 0.17462E+07,
      0.18165E+07, 0.18892E+07, 0.19641E+07, 0.20415E+07, 0.21213E+07,
      0.22036E+07, 0.22884E+07, 0.23759E+07, 0.24661E+07, 0.25590E+07,
      0.26547E+07, 0.27533E+07, 0.28549E+07, 0.29594E+07, 0.30670E+07,
      0.31778E+07, 0.32918E+07, 0.34090E+07, 0.35296E+07, 0.36536E+07,
      0.37812E+07, 0.39123E+07, 0.40470E+07, 0.41855E+07, 0.43278E+07,
      0.44739E+07])


#  --------------- CO2 838: M = 2, I = 10 ---------------------
M = 2
I = 10
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.12066E+03, 0.17085E+03, 0.22116E+03,
      0.27190E+03, 0.32364E+03, 0.37711E+03, 0.43305E+03, 0.49219E+03,
      0.55516E+03, 0.62256E+03, 0.69492E+03, 0.77276E+03, 0.85657E+03,
      0.94685E+03, 0.10441E+04, 0.11488E+04, 0.12614E+04, 0.13826E+04,
      0.15127E+04, 0.16525E+04, 0.18024E+04, 0.19630E+04, 0.21351E+04,
      0.23191E+04, 0.25158E+04, 0.27260E+04, 0.29502E+04, 0.31892E+04,
      0.34438E+04, 0.37148E+04, 0.40031E+04, 0.43094E+04, 0.46346E+04,
      0.49797E+04, 0.53455E+04, 0.57331E+04, 0.61434E+04, 0.65775E+04,
      0.70364E+04, 0.75212E+04, 0.80330E+04, 0.85730E+04, 0.91424E+04,
      0.97423E+04, 0.10374E+05, 0.11039E+05, 0.11738E+05, 0.12474E+05,
      0.13246E+05, 0.14057E+05, 0.14908E+05, 0.15801E+05, 0.16737E+05,
      0.17717E+05, 0.18744E+05, 0.19819E+05, 0.20944E+05, 0.22120E+05,
      0.23349E+05, 0.24634E+05, 0.25975E+05, 0.27376E+05, 0.28837E+05,
      0.30361E+05, 0.31950E+05, 0.33605E+05, 0.35330E+05, 0.37126E+05,
      0.38996E+05, 0.40942E+05, 0.42965E+05, 0.45069E+05, 0.47256E+05,
      0.49528E+05, 0.51888E+05, 0.54338E+05, 0.56882E+05, 0.59521E+05,
      0.62259E+05, 0.65097E+05, 0.68040E+05, 0.71090E+05, 0.74249E+05,
      0.77522E+05, 0.80910E+05, 0.84417E+05, 0.88046E+05, 0.91801E+05,
      0.95684E+05, 0.99699E+05, 0.10385E+06, 0.10814E+06, 0.11257E+06,
      0.11715E+06, 0.12187E+06, 0.12675E+06, 0.13179E+06, 0.13699E+06,
      0.14235E+06, 0.14788E+06, 0.15358E+06, 0.15946E+06, 0.16552E+06,
      0.17176E+06, 0.17819E+06, 0.18482E+06, 0.19164E+06, 0.19867E+06,
      0.20590E+06, 0.21335E+06, 0.22101E+06, 0.22889E+06, 0.23699E+06,
      0.24533E+06, 0.25390E+06, 0.26271E+06, 0.27177E+06, 0.28108E+06,
      0.29064E+06])

#  --------------- CO2 838: M = 2, I = 0 ALIAS-----------------
TIPS_GSI_HASH[(M,0)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,0)] = TIPS_ISO_HASH[(M,I)]
      
#  --------------- CO2 837: M = 2, I = 11 ---------------------
M = 2
I = 11
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.14071E+04, 0.19923E+04, 0.25789E+04,
      0.31704E+04, 0.37733E+04, 0.43962E+04, 0.50477E+04, 0.57360E+04,
      0.64687E+04, 0.72525E+04, 0.80938E+04, 0.89984E+04, 0.99723E+04,
      0.11021E+05, 0.12150E+05, 0.13366E+05, 0.14673E+05, 0.16079E+05,
      0.17589E+05, 0.19211E+05, 0.20949E+05, 0.22812E+05, 0.24807E+05,
      0.26940E+05, 0.29221E+05, 0.31656E+05, 0.34254E+05, 0.37023E+05,
      0.39972E+05, 0.43111E+05, 0.46449E+05, 0.49996E+05, 0.53762E+05,
      0.57756E+05, 0.61991E+05, 0.66477E+05, 0.71226E+05, 0.76249E+05,
      0.81558E+05, 0.87167E+05, 0.93088E+05, 0.99334E+05, 0.10592E+06,
      0.11286E+06, 0.12016E+06, 0.12785E+06, 0.13594E+06, 0.14444E+06,
      0.15337E+06, 0.16274E+06, 0.17258E+06, 0.18290E+06, 0.19371E+06,
      0.20504E+06, 0.21691E+06, 0.22933E+06, 0.24233E+06, 0.25592E+06,
      0.27012E+06, 0.28496E+06, 0.30046E+06, 0.31663E+06, 0.33351E+06,
      0.35111E+06, 0.36946E+06, 0.38858E+06, 0.40850E+06, 0.42924E+06,
      0.45083E+06, 0.47329E+06, 0.49666E+06, 0.52095E+06, 0.54620E+06,
      0.57243E+06, 0.59967E+06, 0.62796E+06, 0.65732E+06, 0.68778E+06,
      0.71938E+06, 0.75214E+06, 0.78611E+06, 0.82131E+06, 0.85777E+06,
      0.89553E+06, 0.93463E+06, 0.97511E+06, 0.10170E+07, 0.10603E+07,
      0.11051E+07, 0.11514E+07, 0.11993E+07, 0.12488E+07, 0.12999E+07,
      0.13527E+07, 0.14073E+07, 0.14636E+07, 0.15217E+07, 0.15816E+07,
      0.16435E+07, 0.17072E+07, 0.17730E+07, 0.18408E+07, 0.19107E+07,
      0.19827E+07, 0.20569E+07, 0.21334E+07, 0.22121E+07, 0.22931E+07,
      0.23765E+07, 0.24624E+07, 0.25507E+07, 0.26416E+07, 0.27351E+07,
      0.28312E+07, 0.29301E+07, 0.30317E+07, 0.31361E+07, 0.32434E+07,
      0.33537E+07])

    
#  --------------- O3 666: M = 3, I = 1 ---------------------
M = 3
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.30333E+03, 0.51126E+03, 0.75274E+03,
      0.10241E+04, 0.13236E+04, 0.16508E+04, 0.20068E+04, 0.23935E+04,
      0.28136E+04, 0.32703E+04, 0.37672E+04, 0.43082E+04, 0.48975E+04,
      0.55395E+04, 0.62386E+04, 0.69996E+04, 0.78272E+04, 0.87264E+04,
      0.97026E+04, 0.10761E+05, 0.11907E+05, 0.13146E+05, 0.14485E+05,
      0.15929E+05, 0.17484E+05, 0.19158E+05, 0.20957E+05, 0.22887E+05,
      0.24956E+05, 0.27172E+05, 0.29541E+05, 0.32072E+05, 0.34773E+05,
      0.37652E+05, 0.40718E+05, 0.43979E+05, 0.47444E+05, 0.51123E+05,
      0.55026E+05, 0.59161E+05, 0.63540E+05, 0.68172E+05, 0.73069E+05,
      0.78240E+05, 0.83698E+05, 0.89453E+05, 0.95517E+05, 0.10190E+06,
      0.10862E+06, 0.11569E+06, 0.12311E+06, 0.13091E+06, 0.13909E+06,
      0.14767E+06, 0.15666E+06, 0.16608E+06, 0.17594E+06, 0.18626E+06,
      0.19706E+06, 0.20834E+06, 0.22012E+06, 0.23242E+06, 0.24526E+06,
      0.25866E+06, 0.27262E+06, 0.28717E+06, 0.30233E+06, 0.31811E+06,
      0.33453E+06, 0.35161E+06, 0.36937E+06, 0.38784E+06, 0.40702E+06,
      0.42694E+06, 0.44762E+06, 0.46909E+06, 0.49135E+06, 0.51444E+06,
      0.53838E+06, 0.56318E+06, 0.58887E+06, 0.61548E+06, 0.64303E+06,
      0.67153E+06, 0.70102E+06, 0.73153E+06, 0.76306E+06, 0.79566E+06,
      0.82934E+06, 0.86413E+06, 0.90006E+06, 0.93716E+06, 0.97545E+06,
      0.10150E+07, 0.10557E+07, 0.10977E+07, 0.11411E+07, 0.11858E+07,
      0.12318E+07, 0.12792E+07, 0.13281E+07, 0.13784E+07, 0.14302E+07,
      0.14835E+07, 0.15384E+07, 0.15948E+07, 0.16529E+07, 0.17126E+07,
      0.17740E+07, 0.18371E+07, 0.19020E+07, 0.19686E+07, 0.20371E+07,
      0.21074E+07, 0.21797E+07, 0.22538E+07, 0.23300E+07, 0.24081E+07,
      0.24883E+07])


#  --------------- O3 668: M = 3, I = 2 ---------------------
M = 3
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.64763E+03, 0.10916E+04, 0.16073E+04,
      0.21870E+04, 0.28271E+04, 0.35272E+04, 0.42900E+04, 0.51197E+04,
      0.60225E+04, 0.70057E+04, 0.80771E+04, 0.92455E+04, 0.10520E+05,
      0.11911E+05, 0.13427E+05, 0.15079E+05, 0.16878E+05, 0.18834E+05,
      0.20960E+05, 0.23267E+05, 0.25767E+05, 0.28472E+05, 0.31397E+05,
      0.34553E+05, 0.37957E+05, 0.41620E+05, 0.45559E+05, 0.49790E+05,
      0.54327E+05, 0.59187E+05, 0.64387E+05, 0.69944E+05, 0.75877E+05,
      0.82203E+05, 0.88943E+05, 0.96114E+05, 0.10374E+06, 0.11184E+06,
      0.12043E+06, 0.12954E+06, 0.13918E+06, 0.14939E+06, 0.16018E+06,
      0.17159E+06, 0.18362E+06, 0.19632E+06, 0.20970E+06, 0.22380E+06,
      0.23863E+06, 0.25423E+06, 0.27063E+06, 0.28786E+06, 0.30594E+06,
      0.32490E+06, 0.34478E+06, 0.36561E+06, 0.38743E+06, 0.41026E+06,
      0.43413E+06, 0.45909E+06, 0.48517E+06, 0.51241E+06, 0.54084E+06,
      0.57049E+06, 0.60141E+06, 0.63365E+06, 0.66722E+06, 0.70219E+06,
      0.73858E+06, 0.77644E+06, 0.81581E+06, 0.85674E+06, 0.89927E+06,
      0.94345E+06, 0.98932E+06, 0.10369E+07, 0.10863E+07, 0.11375E+07,
      0.11906E+07, 0.12457E+07, 0.13027E+07, 0.13618E+07, 0.14229E+07,
      0.14862E+07, 0.15517E+07, 0.16194E+07, 0.16894E+07, 0.17618E+07,
      0.18366E+07, 0.19139E+07, 0.19937E+07, 0.20761E+07, 0.21612E+07,
      0.22490E+07, 0.23395E+07, 0.24330E+07, 0.25293E+07, 0.26286E+07,
      0.27309E+07, 0.28363E+07, 0.29449E+07, 0.30568E+07, 0.31720E+07,
      0.32905E+07, 0.34125E+07, 0.35381E+07, 0.36672E+07, 0.38000E+07,
      0.39366E+07, 0.40770E+07, 0.42213E+07, 0.43696E+07, 0.45220E+07,
      0.46785E+07, 0.48392E+07, 0.50043E+07, 0.51737E+07, 0.53476E+07,
      0.55261E+07])


#  --------------- O3 686: M = 3, I = 3 ---------------------
M = 3
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.31656E+03, 0.53355E+03, 0.78557E+03,
      0.10688E+04, 0.13815E+04, 0.17235E+04, 0.20960E+04, 0.25011E+04,
      0.29420E+04, 0.34223E+04, 0.39459E+04, 0.45172E+04, 0.51408E+04,
      0.58213E+04, 0.65639E+04, 0.73735E+04, 0.82555E+04, 0.92152E+04,
      0.10259E+05, 0.11391E+05, 0.12619E+05, 0.13949E+05, 0.15387E+05,
      0.16940E+05, 0.18614E+05, 0.20417E+05, 0.22357E+05, 0.24440E+05,
      0.26675E+05, 0.29070E+05, 0.31633E+05, 0.34374E+05, 0.37299E+05,
      0.40420E+05, 0.43746E+05, 0.47285E+05, 0.51049E+05, 0.55047E+05,
      0.59289E+05, 0.63788E+05, 0.68554E+05, 0.73598E+05, 0.78932E+05,
      0.84568E+05, 0.90519E+05, 0.96796E+05, 0.10341E+06, 0.11039E+06,
      0.11772E+06, 0.12544E+06, 0.13356E+06, 0.14208E+06, 0.15103E+06,
      0.16041E+06, 0.17026E+06, 0.18057E+06, 0.19137E+06, 0.20268E+06,
      0.21450E+06, 0.22687E+06, 0.23979E+06, 0.25328E+06, 0.26736E+06,
      0.28206E+06, 0.29738E+06, 0.31336E+06, 0.33000E+06, 0.34733E+06,
      0.36537E+06, 0.38414E+06, 0.40366E+06, 0.42396E+06, 0.44505E+06,
      0.46696E+06, 0.48971E+06, 0.51332E+06, 0.53782E+06, 0.56323E+06,
      0.58958E+06, 0.61689E+06, 0.64518E+06, 0.67448E+06, 0.70482E+06,
      0.73623E+06, 0.76872E+06, 0.80234E+06, 0.83710E+06, 0.87303E+06,
      0.91017E+06, 0.94853E+06, 0.98816E+06, 0.10291E+07, 0.10713E+07,
      0.11149E+07, 0.11599E+07, 0.12063E+07, 0.12541E+07, 0.13034E+07,
      0.13542E+07, 0.14066E+07, 0.14606E+07, 0.15161E+07, 0.15733E+07,
      0.16322E+07, 0.16928E+07, 0.17552E+07, 0.18194E+07, 0.18854E+07,
      0.19532E+07, 0.20230E+07, 0.20947E+07, 0.21684E+07, 0.22441E+07,
      0.23219E+07, 0.24018E+07, 0.24838E+07, 0.25680E+07, 0.26545E+07,
      0.27432E+07])


#  --------------- O3 667: M = 3, I = 4 ---------------------
M = 3
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.37657E+04, 0.63472E+04, 0.93454E+04,
      0.12715E+05, 0.16435E+05, 0.20502E+05, 0.24929E+05, 0.29742E+05,
      0.34975E+05, 0.40668E+05, 0.46868E+05, 0.53624E+05, 0.60990E+05,
      0.69018E+05, 0.77768E+05, 0.87296E+05, 0.97666E+05, 0.10894E+06,
      0.12118E+06, 0.13446E+06, 0.14885E+06, 0.16441E+06, 0.18123E+06,
      0.19938E+06, 0.21894E+06, 0.23998E+06, 0.26261E+06, 0.28690E+06,
      0.31295E+06, 0.34084E+06, 0.37068E+06, 0.40256E+06, 0.43659E+06,
      0.47287E+06, 0.51151E+06, 0.55262E+06, 0.59632E+06, 0.64272E+06,
      0.69194E+06, 0.74412E+06, 0.79937E+06, 0.85783E+06, 0.91963E+06,
      0.98492E+06, 0.10538E+07, 0.11265E+07, 0.12031E+07, 0.12837E+07,
      0.13686E+07, 0.14579E+07, 0.15517E+07, 0.16502E+07, 0.17536E+07,
      0.18621E+07, 0.19758E+07, 0.20949E+07, 0.22196E+07, 0.23501E+07,
      0.24866E+07, 0.26292E+07, 0.27783E+07, 0.29339E+07, 0.30963E+07,
      0.32658E+07, 0.34425E+07, 0.36266E+07, 0.38184E+07, 0.40181E+07,
      0.42260E+07, 0.44422E+07, 0.46671E+07, 0.49008E+07, 0.51437E+07,
      0.53959E+07, 0.56578E+07, 0.59296E+07, 0.62116E+07, 0.65040E+07,
      0.68071E+07, 0.71213E+07, 0.74468E+07, 0.77838E+07, 0.81328E+07,
      0.84939E+07, 0.88676E+07, 0.92541E+07, 0.96536E+07, 0.10067E+08,
      0.10493E+08, 0.10934E+08, 0.11390E+08, 0.11860E+08, 0.12345E+08,
      0.12846E+08, 0.13363E+08, 0.13895E+08, 0.14445E+08, 0.15011E+08,
      0.15595E+08, 0.16196E+08, 0.16815E+08, 0.17453E+08, 0.18110E+08,
      0.18786E+08, 0.19482E+08, 0.20198E+08, 0.20934E+08, 0.21691E+08,
      0.22470E+08, 0.23270E+08, 0.24093E+08, 0.24939E+08, 0.25807E+08,
      0.26699E+08, 0.27616E+08, 0.28556E+08, 0.29522E+08, 0.30514E+08,
      0.31531E+08])


#  --------------- O3 676: M = 3, I = 5 ---------------------
M = 3
I = 5
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.18608E+04, 0.31363E+04, 0.46177E+04,
      0.62826E+04, 0.81202E+04, 0.10129E+05, 0.12316E+05, 0.14693E+05,
      0.17277E+05, 0.20089E+05, 0.23153E+05, 0.26492E+05, 0.30133E+05,
      0.34103E+05, 0.38430E+05, 0.43145E+05, 0.48277E+05, 0.53858E+05,
      0.59920E+05, 0.66497E+05, 0.73624E+05, 0.81336E+05, 0.89671E+05,
      0.98668E+05, 0.10836E+06, 0.11880E+06, 0.13002E+06, 0.14207E+06,
      0.15500E+06, 0.16884E+06, 0.18365E+06, 0.19947E+06, 0.21636E+06,
      0.23438E+06, 0.25356E+06, 0.27398E+06, 0.29568E+06, 0.31873E+06,
      0.34318E+06, 0.36911E+06, 0.39656E+06, 0.42561E+06, 0.45632E+06,
      0.48877E+06, 0.52302E+06, 0.55914E+06, 0.59722E+06, 0.63732E+06,
      0.67952E+06, 0.72390E+06, 0.77055E+06, 0.81954E+06, 0.87097E+06,
      0.92491E+06, 0.98146E+06, 0.10407E+07, 0.11027E+07, 0.11677E+07,
      0.12356E+07, 0.13066E+07, 0.13807E+07, 0.14582E+07, 0.15390E+07,
      0.16233E+07, 0.17113E+07, 0.18029E+07, 0.18984E+07, 0.19978E+07,
      0.21012E+07, 0.22089E+07, 0.23208E+07, 0.24372E+07, 0.25581E+07,
      0.26837E+07, 0.28141E+07, 0.29494E+07, 0.30898E+07, 0.32354E+07,
      0.33864E+07, 0.35428E+07, 0.37049E+07, 0.38728E+07, 0.40466E+07,
      0.42264E+07, 0.44125E+07, 0.46050E+07, 0.48040E+07, 0.50098E+07,
      0.52224E+07, 0.54420E+07, 0.56689E+07, 0.59031E+07, 0.61449E+07,
      0.63943E+07, 0.66517E+07, 0.69172E+07, 0.71909E+07, 0.74731E+07,
      0.77639E+07, 0.80635E+07, 0.83721E+07, 0.86900E+07, 0.90172E+07,
      0.93541E+07, 0.97008E+07, 0.10058E+08, 0.10424E+08, 0.10802E+08,
      0.11190E+08, 0.11589E+08, 0.11999E+08, 0.12420E+08, 0.12853E+08,
      0.13298E+08, 0.13755E+08, 0.14223E+08, 0.14705E+08, 0.15199E+08,
      0.15706E+08])


#  --------------- O3 886: M = 3, I = 6 ---------------------
M = 3
I = 6
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.67639E+03, 0.11401E+04, 0.16787E+04,
      0.22843E+04, 0.29532E+04, 0.36856E+04, 0.44842E+04, 0.53545E+04,
      0.63030E+04, 0.73381E+04, 0.84686E+04, 0.97040E+04, 0.11054E+05,
      0.12530E+05, 0.14143E+05, 0.15903E+05, 0.17823E+05, 0.19915E+05,
      0.22190E+05, 0.24663E+05, 0.27346E+05, 0.30254E+05, 0.33400E+05,
      0.36800E+05, 0.40469E+05, 0.44423E+05, 0.48678E+05, 0.53251E+05,
      0.58160E+05, 0.63423E+05, 0.69058E+05, 0.75085E+05, 0.81524E+05,
      0.88395E+05, 0.95719E+05, 0.10352E+06, 0.11181E+06, 0.12063E+06,
      0.12999E+06, 0.13991E+06, 0.15043E+06, 0.16157E+06, 0.17335E+06,
      0.18580E+06, 0.19895E+06, 0.21283E+06, 0.22746E+06, 0.24288E+06,
      0.25911E+06, 0.27619E+06, 0.29415E+06, 0.31301E+06, 0.33283E+06,
      0.35362E+06, 0.37542E+06, 0.39827E+06, 0.42221E+06, 0.44726E+06,
      0.47348E+06, 0.50089E+06, 0.52954E+06, 0.55947E+06, 0.59072E+06,
      0.62332E+06, 0.65733E+06, 0.69279E+06, 0.72973E+06, 0.76821E+06,
      0.80827E+06, 0.84996E+06, 0.89332E+06, 0.93840E+06, 0.98526E+06,
      0.10339E+07, 0.10845E+07, 0.11370E+07, 0.11914E+07, 0.12479E+07,
      0.13065E+07, 0.13672E+07, 0.14302E+07, 0.14953E+07, 0.15628E+07,
      0.16327E+07, 0.17050E+07, 0.17798E+07, 0.18571E+07, 0.19371E+07,
      0.20197E+07, 0.21051E+07, 0.21933E+07, 0.22844E+07, 0.23785E+07,
      0.24755E+07, 0.25757E+07, 0.26790E+07, 0.27855E+07, 0.28954E+07,
      0.30086E+07, 0.31253E+07, 0.32455E+07, 0.33693E+07, 0.34967E+07,
      0.36280E+07, 0.37631E+07, 0.39021E+07, 0.40451E+07, 0.41922E+07,
      0.43435E+07, 0.44990E+07, 0.46589E+07, 0.48232E+07, 0.49920E+07,
      0.51654E+07, 0.53436E+07, 0.55265E+07, 0.57143E+07, 0.59071E+07,
      0.61050E+07])


#  --------------- O3 868: M = 3, I = 7 ---------------------
M = 3
I = 7
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.34615E+03, 0.58348E+03, 0.85915E+03,
      0.11692E+04, 0.15117E+04, 0.18868E+04, 0.22960E+04, 0.27419E+04,
      0.32278E+04, 0.37579E+04, 0.43366E+04, 0.49686E+04, 0.56591E+04,
      0.64134E+04, 0.72369E+04, 0.81354E+04, 0.91148E+04, 0.10181E+05,
      0.11341E+05, 0.12600E+05, 0.13966E+05, 0.15446E+05, 0.17046E+05,
      0.18775E+05, 0.20640E+05, 0.22649E+05, 0.24810E+05, 0.27132E+05,
      0.29624E+05, 0.32295E+05, 0.35154E+05, 0.38211E+05, 0.41475E+05,
      0.44958E+05, 0.48670E+05, 0.52621E+05, 0.56823E+05, 0.61288E+05,
      0.66026E+05, 0.71052E+05, 0.76376E+05, 0.82011E+05, 0.87972E+05,
      0.94271E+05, 0.10092E+06, 0.10794E+06, 0.11534E+06, 0.12313E+06,
      0.13134E+06, 0.13997E+06, 0.14905E+06, 0.15858E+06, 0.16859E+06,
      0.17909E+06, 0.19010E+06, 0.20164E+06, 0.21373E+06, 0.22638E+06,
      0.23962E+06, 0.25346E+06, 0.26792E+06, 0.28302E+06, 0.29879E+06,
      0.31524E+06, 0.33240E+06, 0.35029E+06, 0.36892E+06, 0.38833E+06,
      0.40853E+06, 0.42956E+06, 0.45142E+06, 0.47416E+06, 0.49778E+06,
      0.52233E+06, 0.54781E+06, 0.57427E+06, 0.60172E+06, 0.63019E+06,
      0.65971E+06, 0.69031E+06, 0.72201E+06, 0.75485E+06, 0.78886E+06,
      0.82405E+06, 0.86048E+06, 0.89815E+06, 0.93711E+06, 0.97739E+06,
      0.10190E+07, 0.10620E+07, 0.11065E+07, 0.11523E+07, 0.11997E+07,
      0.12485E+07, 0.12990E+07, 0.13510E+07, 0.14046E+07, 0.14599E+07,
      0.15169E+07, 0.15756E+07, 0.16361E+07, 0.16984E+07, 0.17626E+07,
      0.18287E+07, 0.18966E+07, 0.19666E+07, 0.20386E+07, 0.21126E+07,
      0.21887E+07, 0.22669E+07, 0.23474E+07, 0.24300E+07, 0.25150E+07,
      0.26022E+07, 0.26919E+07, 0.27839E+07, 0.28784E+07, 0.29753E+07,
      0.30749E+07])


#  --------------- O3 678: M = 3, I = 8 ---------------------
M = 3
I = 8
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.39745E+04, 0.66993E+04, 0.98642E+04,
      0.13422E+05, 0.17352E+05, 0.21652E+05, 0.26339E+05, 0.31442E+05,
      0.37000E+05, 0.43058E+05, 0.49669E+05, 0.56885E+05, 0.64766E+05,
      0.73372E+05, 0.82765E+05, 0.93011E+05, 0.10418E+06, 0.11633E+06,
      0.12955E+06, 0.14390E+06, 0.15946E+06, 0.17632E+06, 0.19455E+06,
      0.21424E+06, 0.23547E+06, 0.25835E+06, 0.28296E+06, 0.30939E+06,
      0.33776E+06, 0.36816E+06, 0.40070E+06, 0.43549E+06, 0.47264E+06,
      0.51228E+06, 0.55451E+06, 0.59947E+06, 0.64728E+06, 0.69807E+06,
      0.75198E+06, 0.80915E+06, 0.86971E+06, 0.93381E+06, 0.10016E+07,
      0.10733E+07, 0.11489E+07, 0.12287E+07, 0.13128E+07, 0.14015E+07,
      0.14948E+07, 0.15930E+07, 0.16961E+07, 0.18045E+07, 0.19183E+07,
      0.20378E+07, 0.21629E+07, 0.22942E+07, 0.24316E+07, 0.25754E+07,
      0.27258E+07, 0.28831E+07, 0.30475E+07, 0.32192E+07, 0.33984E+07,
      0.35855E+07, 0.37805E+07, 0.39838E+07, 0.41956E+07, 0.44162E+07,
      0.46458E+07, 0.48847E+07, 0.51332E+07, 0.53916E+07, 0.56601E+07,
      0.59390E+07, 0.62286E+07, 0.65292E+07, 0.68412E+07, 0.71647E+07,
      0.75002E+07, 0.78479E+07, 0.82081E+07, 0.85813E+07, 0.89676E+07,
      0.93676E+07, 0.97814E+07, 0.10209E+08, 0.10652E+08, 0.11110E+08,
      0.11583E+08, 0.12071E+08, 0.12576E+08, 0.13097E+08, 0.13635E+08,
      0.14190E+08, 0.14763E+08, 0.15354E+08, 0.15963E+08, 0.16592E+08,
      0.17239E+08, 0.17906E+08, 0.18593E+08, 0.19301E+08, 0.20030E+08,
      0.20780E+08, 0.21553E+08, 0.22347E+08, 0.23165E+08, 0.24006E+08,
      0.24870E+08, 0.25759E+08, 0.26673E+08, 0.27612E+08, 0.28577E+08,
      0.29568E+08, 0.30585E+08, 0.31631E+08, 0.32704E+08, 0.33805E+08,
      0.34936E+08])


#  --------------- O3 768: M = 3, I = 9 ---------------------
M = 3
I = 9
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.40228E+04, 0.67808E+04, 0.99842E+04,
      0.13586E+05, 0.17564E+05, 0.21919E+05, 0.26665E+05, 0.31833E+05,
      0.37461E+05, 0.43596E+05, 0.50286E+05, 0.57589E+05, 0.65562E+05,
      0.74264E+05, 0.83761E+05, 0.94115E+05, 0.10540E+06, 0.11767E+06,
      0.13102E+06, 0.14550E+06, 0.16121E+06, 0.17822E+06, 0.19661E+06,
      0.21646E+06, 0.23788E+06, 0.26094E+06, 0.28574E+06, 0.31239E+06,
      0.34097E+06, 0.37160E+06, 0.40437E+06, 0.43941E+06, 0.47683E+06,
      0.51673E+06, 0.55925E+06, 0.60451E+06, 0.65262E+06, 0.70374E+06,
      0.75799E+06, 0.81550E+06, 0.87643E+06, 0.94092E+06, 0.10091E+07,
      0.10812E+07, 0.11572E+07, 0.12375E+07, 0.13221E+07, 0.14112E+07,
      0.15050E+07, 0.16037E+07, 0.17074E+07, 0.18164E+07, 0.19307E+07,
      0.20507E+07, 0.21765E+07, 0.23084E+07, 0.24464E+07, 0.25909E+07,
      0.27421E+07, 0.29001E+07, 0.30652E+07, 0.32377E+07, 0.34177E+07,
      0.36055E+07, 0.38014E+07, 0.40055E+07, 0.42182E+07, 0.44397E+07,
      0.46703E+07, 0.49102E+07, 0.51597E+07, 0.54191E+07, 0.56886E+07,
      0.59686E+07, 0.62593E+07, 0.65611E+07, 0.68742E+07, 0.71989E+07,
      0.75356E+07, 0.78846E+07, 0.82461E+07, 0.86206E+07, 0.90083E+07,
      0.94097E+07, 0.98249E+07, 0.10254E+08, 0.10699E+08, 0.11158E+08,
      0.11632E+08, 0.12123E+08, 0.12629E+08, 0.13152E+08, 0.13691E+08,
      0.14248E+08, 0.14823E+08, 0.15416E+08, 0.16027E+08, 0.16657E+08,
      0.17307E+08, 0.17976E+08, 0.18665E+08, 0.19375E+08, 0.20106E+08,
      0.20858E+08, 0.21633E+08, 0.22430E+08, 0.23250E+08, 0.24093E+08,
      0.24960E+08, 0.25851E+08, 0.26767E+08, 0.27709E+08, 0.28676E+08,
      0.29670E+08, 0.30691E+08, 0.31739E+08, 0.32815E+08, 0.33919E+08,
      0.35053E+08])


#  --------------- O3 786: M = 3, I = 10 ---------------------
M = 3
I = 10
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.39315E+04, 0.66267E+04, 0.97569E+04,
      0.13276E+05, 0.17162E+05, 0.21414E+05, 0.26048E+05, 0.31094E+05,
      0.36590E+05, 0.42581E+05, 0.49120E+05, 0.56260E+05, 0.64061E+05,
      0.72580E+05, 0.81882E+05, 0.92031E+05, 0.10309E+06, 0.11514E+06,
      0.12824E+06, 0.14247E+06, 0.15791E+06, 0.17463E+06, 0.19272E+06,
      0.21226E+06, 0.23333E+06, 0.25604E+06, 0.28047E+06, 0.30673E+06,
      0.33490E+06, 0.36510E+06, 0.39743E+06, 0.43200E+06, 0.46892E+06,
      0.50831E+06, 0.55029E+06, 0.59498E+06, 0.64251E+06, 0.69301E+06,
      0.74662E+06, 0.80347E+06, 0.86370E+06, 0.92747E+06, 0.99491E+06,
      0.10662E+07, 0.11414E+07, 0.12208E+07, 0.13046E+07, 0.13928E+07,
      0.14856E+07, 0.15833E+07, 0.16860E+07, 0.17939E+07, 0.19072E+07,
      0.20261E+07, 0.21508E+07, 0.22814E+07, 0.24182E+07, 0.25614E+07,
      0.27112E+07, 0.28679E+07, 0.30316E+07, 0.32026E+07, 0.33811E+07,
      0.35674E+07, 0.37617E+07, 0.39642E+07, 0.41752E+07, 0.43950E+07,
      0.46237E+07, 0.48618E+07, 0.51094E+07, 0.53668E+07, 0.56343E+07,
      0.59123E+07, 0.62009E+07, 0.65005E+07, 0.68113E+07, 0.71338E+07,
      0.74681E+07, 0.78147E+07, 0.81737E+07, 0.85457E+07, 0.89308E+07,
      0.93295E+07, 0.97420E+07, 0.10169E+08, 0.10610E+08, 0.11066E+08,
      0.11538E+08, 0.12025E+08, 0.12528E+08, 0.13048E+08, 0.13584E+08,
      0.14138E+08, 0.14709E+08, 0.15298E+08, 0.15906E+08, 0.16532E+08,
      0.17178E+08, 0.17843E+08, 0.18528E+08, 0.19234E+08, 0.19961E+08,
      0.20710E+08, 0.21480E+08, 0.22272E+08, 0.23088E+08, 0.23926E+08,
      0.24789E+08, 0.25675E+08, 0.26587E+08, 0.27523E+08, 0.28485E+08,
      0.29474E+08, 0.30489E+08, 0.31532E+08, 0.32603E+08, 0.33701E+08,
      0.34829E+08])


#  --------------- O3 776: M = 3, I = 11 ---------------------
M = 3
I = 11
TIPS_GSI_HASH[(M,I)] = __FloatType__(36.)
TIPS_ISO_HASH[(M,I)] = float32([0.23106E+05, 0.38945E+05, 0.57342E+05,
      0.78021E+05, 0.10085E+06, 0.12582E+06, 0.15302E+06, 0.18262E+06,
      0.21482E+06, 0.24989E+06, 0.28812E+06, 0.32983E+06, 0.37535E+06,
      0.42501E+06, 0.47919E+06, 0.53825E+06, 0.60258E+06, 0.67256E+06,
      0.74862E+06, 0.83118E+06, 0.92069E+06, 0.10176E+07, 0.11223E+07,
      0.12354E+07, 0.13574E+07, 0.14887E+07, 0.16299E+07, 0.17816E+07,
      0.19443E+07, 0.21187E+07, 0.23052E+07, 0.25047E+07, 0.27176E+07,
      0.29447E+07, 0.31866E+07, 0.34441E+07, 0.37179E+07, 0.40087E+07,
      0.43173E+07, 0.46444E+07, 0.49910E+07, 0.53578E+07, 0.57456E+07,
      0.61554E+07, 0.65880E+07, 0.70444E+07, 0.75255E+07, 0.80322E+07,
      0.85656E+07, 0.91266E+07, 0.97163E+07, 0.10336E+08, 0.10986E+08,
      0.11668E+08, 0.12383E+08, 0.13133E+08, 0.13918E+08, 0.14739E+08,
      0.15598E+08, 0.16496E+08, 0.17435E+08, 0.18415E+08, 0.19438E+08,
      0.20505E+08, 0.21619E+08, 0.22779E+08, 0.23987E+08, 0.25246E+08,
      0.26556E+08, 0.27920E+08, 0.29337E+08, 0.30811E+08, 0.32343E+08,
      0.33934E+08, 0.35585E+08, 0.37300E+08, 0.39079E+08, 0.40924E+08,
      0.42837E+08, 0.44819E+08, 0.46873E+08, 0.49001E+08, 0.51203E+08,
      0.53483E+08, 0.55842E+08, 0.58282E+08, 0.60805E+08, 0.63414E+08,
      0.66109E+08, 0.68894E+08, 0.71770E+08, 0.74740E+08, 0.77806E+08,
      0.80970E+08, 0.84234E+08, 0.87600E+08, 0.91072E+08, 0.94651E+08,
      0.98339E+08, 0.10214E+09, 0.10605E+09, 0.11009E+09, 0.11424E+09,
      0.11851E+09, 0.12291E+09, 0.12744E+09, 0.13209E+09, 0.13688E+09,
      0.14180E+09, 0.14687E+09, 0.15207E+09, 0.15742E+09, 0.16291E+09,
      0.16855E+09, 0.17435E+09, 0.18030E+09, 0.18641E+09, 0.19268E+09,
      0.19912E+09])


#  --------------- O3 767: M = 3, I = 12 ---------------------
M = 3
I = 12
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.11692E+05, 0.19707E+05, 0.29017E+05,
      0.39482E+05, 0.51038E+05, 0.63680E+05, 0.77450E+05, 0.92432E+05,
      0.10873E+06, 0.12649E+06, 0.14584E+06, 0.16694E+06, 0.18996E+06,
      0.21507E+06, 0.24245E+06, 0.27229E+06, 0.30478E+06, 0.34013E+06,
      0.37853E+06, 0.42020E+06, 0.46536E+06, 0.51424E+06, 0.56708E+06,
      0.62411E+06, 0.68559E+06, 0.75178E+06, 0.82296E+06, 0.89939E+06,
      0.98137E+06, 0.10692E+07, 0.11631E+07, 0.12636E+07, 0.13708E+07,
      0.14851E+07, 0.16069E+07, 0.17365E+07, 0.18742E+07, 0.20206E+07,
      0.21758E+07, 0.23404E+07, 0.25148E+07, 0.26992E+07, 0.28943E+07,
      0.31004E+07, 0.33179E+07, 0.35474E+07, 0.37892E+07, 0.40440E+07,
      0.43121E+07, 0.45940E+07, 0.48904E+07, 0.52017E+07, 0.55285E+07,
      0.58713E+07, 0.62306E+07, 0.66071E+07, 0.70014E+07, 0.74140E+07,
      0.78456E+07, 0.82967E+07, 0.87681E+07, 0.92604E+07, 0.97742E+07,
      0.10310E+08, 0.10869E+08, 0.11452E+08, 0.12059E+08, 0.12691E+08,
      0.13348E+08, 0.14033E+08, 0.14745E+08, 0.15484E+08, 0.16253E+08,
      0.17052E+08, 0.17881E+08, 0.18741E+08, 0.19634E+08, 0.20560E+08,
      0.21520E+08, 0.22515E+08, 0.23546E+08, 0.24613E+08, 0.25718E+08,
      0.26862E+08, 0.28046E+08, 0.29270E+08, 0.30536E+08, 0.31845E+08,
      0.33197E+08, 0.34594E+08, 0.36037E+08, 0.37527E+08, 0.39065E+08,
      0.40652E+08, 0.42289E+08, 0.43977E+08, 0.45719E+08, 0.47514E+08,
      0.49363E+08, 0.51270E+08, 0.53233E+08, 0.55255E+08, 0.57337E+08,
      0.59480E+08, 0.61686E+08, 0.63956E+08, 0.66290E+08, 0.68691E+08,
      0.71160E+08, 0.73699E+08, 0.76307E+08, 0.78988E+08, 0.81743E+08,
      0.84572E+08, 0.87478E+08, 0.90462E+08, 0.93525E+08, 0.96669E+08,
      0.99896E+08])


#  --------------- O3 888: M = 3, I = 13 ---------------------
M = 3
I = 13
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.36175E+03, 0.60978E+03, 0.89790E+03,
      0.12219E+04, 0.15802E+04, 0.19728E+04, 0.24016E+04, 0.28696E+04,
      0.33807E+04, 0.39394E+04, 0.45506E+04, 0.52196E+04, 0.59521E+04,
      0.67538E+04, 0.76308E+04, 0.85894E+04, 0.96361E+04, 0.10777E+05,
      0.12021E+05, 0.13373E+05, 0.14841E+05, 0.16434E+05, 0.18158E+05,
      0.20023E+05, 0.22037E+05, 0.24208E+05, 0.26547E+05, 0.29061E+05,
      0.31762E+05, 0.34659E+05, 0.37762E+05, 0.41083E+05, 0.44632E+05,
      0.48421E+05, 0.52462E+05, 0.56766E+05, 0.61346E+05, 0.66215E+05,
      0.71386E+05, 0.76873E+05, 0.82688E+05, 0.88848E+05, 0.95365E+05,
      0.10226E+06, 0.10954E+06, 0.11722E+06, 0.12532E+06, 0.13387E+06,
      0.14286E+06, 0.15233E+06, 0.16229E+06, 0.17275E+06, 0.18374E+06,
      0.19528E+06, 0.20737E+06, 0.22006E+06, 0.23335E+06, 0.24726E+06,
      0.26182E+06, 0.27705E+06, 0.29297E+06, 0.30960E+06, 0.32696E+06,
      0.34509E+06, 0.36399E+06, 0.38371E+06, 0.40425E+06, 0.42566E+06,
      0.44794E+06, 0.47114E+06, 0.49527E+06, 0.52036E+06, 0.54644E+06,
      0.57354E+06, 0.60169E+06, 0.63091E+06, 0.66124E+06, 0.69270E+06,
      0.72533E+06, 0.75916E+06, 0.79421E+06, 0.83053E+06, 0.86814E+06,
      0.90708E+06, 0.94737E+06, 0.98907E+06, 0.10322E+07, 0.10768E+07,
      0.11229E+07, 0.11705E+07, 0.12197E+07, 0.12705E+07, 0.13230E+07,
      0.13771E+07, 0.14330E+07, 0.14906E+07, 0.15501E+07, 0.16114E+07,
      0.16745E+07, 0.17397E+07, 0.18067E+07, 0.18759E+07, 0.19470E+07,
      0.20203E+07, 0.20957E+07, 0.21733E+07, 0.22532E+07, 0.23353E+07,
      0.24198E+07, 0.25067E+07, 0.25960E+07, 0.26878E+07, 0.27821E+07,
      0.28790E+07, 0.29785E+07, 0.30807E+07, 0.31857E+07, 0.32934E+07,
      0.34040E+07])


#  --------------- O3 887: M = 3, I = 14 ---------------------
M = 3
I = 14
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.42000E+04, 0.70796E+04, 0.10424E+05,
      0.14186E+05, 0.18342E+05, 0.22896E+05, 0.27866E+05, 0.33285E+05,
      0.39199E+05, 0.45659E+05, 0.52720E+05, 0.60444E+05, 0.68895E+05,
      0.78139E+05, 0.88246E+05, 0.99288E+05, 0.11134E+06, 0.12447E+06,
      0.13877E+06, 0.15431E+06, 0.17119E+06, 0.18949E+06, 0.20930E+06,
      0.23071E+06, 0.25383E+06, 0.27875E+06, 0.30558E+06, 0.33442E+06,
      0.36539E+06, 0.39861E+06, 0.43418E+06, 0.47224E+06, 0.51291E+06,
      0.55632E+06, 0.60260E+06, 0.65189E+06, 0.70434E+06, 0.76008E+06,
      0.81927E+06, 0.88206E+06, 0.94862E+06, 0.10191E+07, 0.10937E+07,
      0.11725E+07, 0.12558E+07, 0.13436E+07, 0.14363E+07, 0.15340E+07,
      0.16368E+07, 0.17450E+07, 0.18588E+07, 0.19784E+07, 0.21040E+07,
      0.22358E+07, 0.23741E+07, 0.25190E+07, 0.26708E+07, 0.28297E+07,
      0.29961E+07, 0.31700E+07, 0.33518E+07, 0.35417E+07, 0.37400E+07,
      0.39469E+07, 0.41628E+07, 0.43878E+07, 0.46224E+07, 0.48667E+07,
      0.51210E+07, 0.53858E+07, 0.56611E+07, 0.59475E+07, 0.62451E+07,
      0.65544E+07, 0.68755E+07, 0.72089E+07, 0.75550E+07, 0.79139E+07,
      0.82861E+07, 0.86720E+07, 0.90719E+07, 0.94861E+07, 0.99151E+07,
      0.10359E+08, 0.10819E+08, 0.11294E+08, 0.11786E+08, 0.12294E+08,
      0.12820E+08, 0.13363E+08, 0.13924E+08, 0.14503E+08, 0.15101E+08,
      0.15719E+08, 0.16356E+08, 0.17013E+08, 0.17690E+08, 0.18389E+08,
      0.19109E+08, 0.19851E+08, 0.20616E+08, 0.21404E+08, 0.22215E+08,
      0.23050E+08, 0.23910E+08, 0.24794E+08, 0.25704E+08, 0.26640E+08,
      0.27603E+08, 0.28593E+08, 0.29610E+08, 0.30656E+08, 0.31731E+08,
      0.32835E+08, 0.33969E+08, 0.35133E+08, 0.36329E+08, 0.37556E+08,
      0.38816E+08])


#  --------------- O3 878: M = 3, I = 15 ---------------------
M = 3
I = 15
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.21250E+04, 0.35820E+04, 0.52744E+04,
      0.71778E+04, 0.92814E+04, 0.11586E+05, 0.14102E+05, 0.16845E+05,
      0.19839E+05, 0.23108E+05, 0.26680E+05, 0.30588E+05, 0.34861E+05,
      0.39534E+05, 0.44642E+05, 0.50219E+05, 0.56305E+05, 0.62937E+05,
      0.70155E+05, 0.78001E+05, 0.86516E+05, 0.95747E+05, 0.10574E+06,
      0.11653E+06, 0.12819E+06, 0.14075E+06, 0.15427E+06, 0.16881E+06,
      0.18441E+06, 0.20114E+06, 0.21906E+06, 0.23823E+06, 0.25871E+06,
      0.28056E+06, 0.30386E+06, 0.32867E+06, 0.35507E+06, 0.38312E+06,
      0.41291E+06, 0.44450E+06, 0.47799E+06, 0.51344E+06, 0.55095E+06,
      0.59060E+06, 0.63248E+06, 0.67667E+06, 0.72327E+06, 0.77238E+06,
      0.82409E+06, 0.87850E+06, 0.93571E+06, 0.99583E+06, 0.10590E+07,
      0.11252E+07, 0.11947E+07, 0.12675E+07, 0.13438E+07, 0.14237E+07,
      0.15072E+07, 0.15946E+07, 0.16859E+07, 0.17814E+07, 0.18810E+07,
      0.19849E+07, 0.20934E+07, 0.22064E+07, 0.23242E+07, 0.24469E+07,
      0.25747E+07, 0.27076E+07, 0.28459E+07, 0.29897E+07, 0.31391E+07,
      0.32944E+07, 0.34557E+07, 0.36231E+07, 0.37968E+07, 0.39770E+07,
      0.41639E+07, 0.43576E+07, 0.45583E+07, 0.47663E+07, 0.49816E+07,
      0.52045E+07, 0.54352E+07, 0.56739E+07, 0.59207E+07, 0.61759E+07,
      0.64396E+07, 0.67121E+07, 0.69936E+07, 0.72844E+07, 0.75845E+07,
      0.78943E+07, 0.82139E+07, 0.85436E+07, 0.88837E+07, 0.92342E+07,
      0.95956E+07, 0.99680E+07, 0.10352E+08, 0.10747E+08, 0.11154E+08,
      0.11573E+08, 0.12004E+08, 0.12448E+08, 0.12904E+08, 0.13374E+08,
      0.13857E+08, 0.14353E+08, 0.14864E+08, 0.15388E+08, 0.15927E+08,
      0.16481E+08, 0.17050E+08, 0.17634E+08, 0.18234E+08, 0.18849E+08,
      0.19481E+08])


#  --------------- O3 778: M = 3, I = 16 ---------------------
M = 3
I = 16
TIPS_GSI_HASH[(M,I)] = __FloatType__(36.)
TIPS_ISO_HASH[(M,I)] = float32([0.24692E+05, 0.41621E+05, 0.61284E+05,
      0.83394E+05, 0.10782E+06, 0.13457E+06, 0.16375E+06, 0.19554E+06,
      0.23020E+06, 0.26801E+06, 0.30930E+06, 0.35443E+06, 0.40375E+06,
      0.45763E+06, 0.51650E+06, 0.58075E+06, 0.65080E+06, 0.72711E+06,
      0.81012E+06, 0.90030E+06, 0.99815E+06, 0.11042E+07, 0.12189E+07,
      0.13428E+07, 0.14765E+07, 0.16206E+07, 0.17757E+07, 0.19423E+07,
      0.21212E+07, 0.23129E+07, 0.25181E+07, 0.27377E+07, 0.29721E+07,
      0.32223E+07, 0.34890E+07, 0.37729E+07, 0.40750E+07, 0.43959E+07,
      0.47365E+07, 0.50978E+07, 0.54807E+07, 0.58860E+07, 0.63147E+07,
      0.67678E+07, 0.72463E+07, 0.77512E+07, 0.82836E+07, 0.88445E+07,
      0.94351E+07, 0.10056E+08, 0.10710E+08, 0.11396E+08, 0.12117E+08,
      0.12873E+08, 0.13666E+08, 0.14497E+08, 0.15367E+08, 0.16279E+08,
      0.17232E+08, 0.18229E+08, 0.19271E+08, 0.20359E+08, 0.21495E+08,
      0.22681E+08, 0.23917E+08, 0.25206E+08, 0.26549E+08, 0.27948E+08,
      0.29404E+08, 0.30920E+08, 0.32496E+08, 0.34135E+08, 0.35838E+08,
      0.37608E+08, 0.39445E+08, 0.41353E+08, 0.43332E+08, 0.45385E+08,
      0.47514E+08, 0.49721E+08, 0.52007E+08, 0.54376E+08, 0.56829E+08,
      0.59367E+08, 0.61995E+08, 0.64712E+08, 0.67523E+08, 0.70429E+08,
      0.73432E+08, 0.76535E+08, 0.79740E+08, 0.83050E+08, 0.86467E+08,
      0.89993E+08, 0.93632E+08, 0.97385E+08, 0.10126E+09, 0.10525E+09,
      0.10936E+09, 0.11360E+09, 0.11796E+09, 0.12246E+09, 0.12709E+09,
      0.13186E+09, 0.13677E+09, 0.14182E+09, 0.14701E+09, 0.15236E+09,
      0.15785E+09, 0.16350E+09, 0.16931E+09, 0.17528E+09, 0.18141E+09,
      0.18771E+09, 0.19418E+09, 0.20082E+09, 0.20764E+09, 0.21465E+09,
      0.22183E+09])


#  --------------- O3 787: M = 3, I = 17 ---------------------
M = 3
I = 17
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.12211E+05, 0.20582E+05, 0.30305E+05,
      0.41237E+05, 0.53314E+05, 0.66536E+05, 0.80957E+05, 0.96672E+05,
      0.11380E+06, 0.13250E+06, 0.15292E+06, 0.17524E+06, 0.19965E+06,
      0.22632E+06, 0.25546E+06, 0.28728E+06, 0.32199E+06, 0.35980E+06,
      0.40094E+06, 0.44565E+06, 0.49417E+06, 0.54676E+06, 0.60366E+06,
      0.66516E+06, 0.73152E+06, 0.80305E+06, 0.88002E+06, 0.96276E+06,
      0.10516E+07, 0.11468E+07, 0.12488E+07, 0.13578E+07, 0.14743E+07,
      0.15987E+07, 0.17312E+07, 0.18723E+07, 0.20225E+07, 0.21820E+07,
      0.23514E+07, 0.25310E+07, 0.27214E+07, 0.29230E+07, 0.31362E+07,
      0.33616E+07, 0.35997E+07, 0.38509E+07, 0.41158E+07, 0.43949E+07,
      0.46887E+07, 0.49980E+07, 0.53231E+07, 0.56647E+07, 0.60234E+07,
      0.63998E+07, 0.67946E+07, 0.72084E+07, 0.76418E+07, 0.80955E+07,
      0.85702E+07, 0.90666E+07, 0.95854E+07, 0.10127E+08, 0.10693E+08,
      0.11284E+08, 0.11900E+08, 0.12542E+08, 0.13211E+08, 0.13907E+08,
      0.14633E+08, 0.15388E+08, 0.16173E+08, 0.16990E+08, 0.17838E+08,
      0.18720E+08, 0.19636E+08, 0.20586E+08, 0.21573E+08, 0.22596E+08,
      0.23657E+08, 0.24757E+08, 0.25896E+08, 0.27077E+08, 0.28299E+08,
      0.29565E+08, 0.30874E+08, 0.32229E+08, 0.33630E+08, 0.35079E+08,
      0.36576E+08, 0.38123E+08, 0.39721E+08, 0.41371E+08, 0.43075E+08,
      0.44833E+08, 0.46647E+08, 0.48518E+08, 0.50448E+08, 0.52438E+08,
      0.54489E+08, 0.56603E+08, 0.58780E+08, 0.61023E+08, 0.63332E+08,
      0.65710E+08, 0.68157E+08, 0.70676E+08, 0.73266E+08, 0.75931E+08,
      0.78672E+08, 0.81490E+08, 0.84386E+08, 0.87363E+08, 0.90422E+08,
      0.93564E+08, 0.96791E+08, 0.10011E+09, 0.10351E+09, 0.10700E+09,
      0.11059E+09])


#  --------------- O3 777: M = 3, I = 18 ---------------------
M = 3
I = 18
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.71750E+05, 0.12094E+06, 0.17807E+06,
      0.24230E+06, 0.31324E+06, 0.39088E+06, 0.47550E+06, 0.56764E+06,
      0.66800E+06, 0.77740E+06, 0.89677E+06, 0.10271E+07, 0.11694E+07,
      0.13249E+07, 0.14945E+07, 0.16796E+07, 0.18813E+07, 0.21009E+07,
      0.23396E+07, 0.25989E+07, 0.28801E+07, 0.31847E+07, 0.35140E+07,
      0.38698E+07, 0.42535E+07, 0.46669E+07, 0.51115E+07, 0.55893E+07,
      0.61019E+07, 0.66513E+07, 0.72393E+07, 0.78680E+07, 0.85395E+07,
      0.92558E+07, 0.10019E+08, 0.10832E+08, 0.11696E+08, 0.12614E+08,
      0.13588E+08, 0.14621E+08, 0.15716E+08, 0.16875E+08, 0.18100E+08,
      0.19395E+08, 0.20762E+08, 0.22205E+08, 0.23726E+08, 0.25328E+08,
      0.27015E+08, 0.28789E+08, 0.30654E+08, 0.32614E+08, 0.34671E+08,
      0.36830E+08, 0.39093E+08, 0.41465E+08, 0.43949E+08, 0.46549E+08,
      0.49269E+08, 0.52112E+08, 0.55084E+08, 0.58188E+08, 0.61428E+08,
      0.64809E+08, 0.68335E+08, 0.72010E+08, 0.75840E+08, 0.79828E+08,
      0.83979E+08, 0.88299E+08, 0.92792E+08, 0.97463E+08, 0.10232E+09,
      0.10736E+09, 0.11260E+09, 0.11803E+09, 0.12367E+09, 0.12952E+09,
      0.13559E+09, 0.14187E+09, 0.14839E+09, 0.15513E+09, 0.16212E+09,
      0.16935E+09, 0.17683E+09, 0.18457E+09, 0.19257E+09, 0.20085E+09,
      0.20940E+09, 0.21824E+09, 0.22736E+09, 0.23678E+09, 0.24651E+09,
      0.25655E+09, 0.26691E+09, 0.27759E+09, 0.28861E+09, 0.29997E+09,
      0.31167E+09, 0.32374E+09, 0.33616E+09, 0.34896E+09, 0.36214E+09,
      0.37571E+09, 0.38967E+09, 0.40404E+09, 0.41882E+09, 0.43403E+09,
      0.44966E+09, 0.46573E+09, 0.48226E+09, 0.49923E+09, 0.51668E+09,
      0.53460E+09, 0.55301E+09, 0.57191E+09, 0.59131E+09, 0.61123E+09,
      0.63167E+09])


#  --------------- N2O 446: M = 4, I = 1 ---------------------
M = 4
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(9.)
TIPS_ISO_HASH[(M,I)] = float32([0.89943E+03, 0.12734E+04, 0.16489E+04,
      0.20293E+04, 0.24205E+04, 0.28289E+04, 0.32609E+04, 0.37222E+04,
      0.42180E+04, 0.47529E+04, 0.53312E+04, 0.59572E+04, 0.66348E+04,
      0.73683E+04, 0.81616E+04, 0.90190E+04, 0.99450E+04, 0.10944E+05,
      0.12021E+05, 0.13180E+05, 0.14426E+05, 0.15766E+05, 0.17203E+05,
      0.18745E+05, 0.20396E+05, 0.22162E+05, 0.24051E+05, 0.26069E+05,
      0.28222E+05, 0.30517E+05, 0.32962E+05, 0.35564E+05, 0.38331E+05,
      0.41271E+05, 0.44393E+05, 0.47704E+05, 0.51214E+05, 0.54932E+05,
      0.58868E+05, 0.63030E+05, 0.67429E+05, 0.72075E+05, 0.76979E+05,
      0.82151E+05, 0.87604E+05, 0.93348E+05, 0.99395E+05, 0.10576E+06,
      0.11245E+06, 0.11948E+06, 0.12686E+06, 0.13461E+06, 0.14275E+06,
      0.15128E+06, 0.16021E+06, 0.16958E+06, 0.17938E+06, 0.18964E+06,
      0.20037E+06, 0.21159E+06, 0.22331E+06, 0.23556E+06, 0.24834E+06,
      0.26169E+06, 0.27561E+06, 0.29012E+06, 0.30525E+06, 0.32101E+06,
      0.33743E+06, 0.35452E+06, 0.37230E+06, 0.39080E+06, 0.41004E+06,
      0.43004E+06, 0.45082E+06, 0.47241E+06, 0.49483E+06, 0.51810E+06,
      0.54225E+06, 0.56730E+06, 0.59329E+06, 0.62022E+06, 0.64814E+06,
      0.67707E+06, 0.70703E+06, 0.73806E+06, 0.77018E+06, 0.80342E+06,
      0.83781E+06, 0.87338E+06, 0.91016E+06, 0.94818E+06, 0.98748E+06,
      0.10281E+07, 0.10700E+07, 0.11133E+07, 0.11581E+07, 0.12042E+07,
      0.12519E+07, 0.13010E+07, 0.13517E+07, 0.14040E+07, 0.14579E+07,
      0.15134E+07, 0.15707E+07, 0.16297E+07, 0.16905E+07, 0.17530E+07,
      0.18175E+07, 0.18838E+07, 0.19521E+07, 0.20224E+07, 0.20947E+07,
      0.21690E+07, 0.22455E+07, 0.23242E+07, 0.24050E+07, 0.24881E+07,
      0.25735E+07])


#  --------------- N2O 456: M = 4, I = 2 ---------------------
M = 4
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.59966E+03, 0.84903E+03, 0.10995E+04,
      0.13538E+04, 0.16158E+04, 0.18903E+04, 0.21815E+04, 0.24934E+04,
      0.28295E+04, 0.31927E+04, 0.35862E+04, 0.40128E+04, 0.44752E+04,
      0.49763E+04, 0.55189E+04, 0.61059E+04, 0.67404E+04, 0.74256E+04,
      0.81646E+04, 0.89609E+04, 0.98180E+04, 0.10740E+05, 0.11729E+05,
      0.12791E+05, 0.13930E+05, 0.15149E+05, 0.16453E+05, 0.17847E+05,
      0.19335E+05, 0.20922E+05, 0.22614E+05, 0.24416E+05, 0.26333E+05,
      0.28371E+05, 0.30535E+05, 0.32833E+05, 0.35269E+05, 0.37851E+05,
      0.40585E+05, 0.43478E+05, 0.46537E+05, 0.49769E+05, 0.53182E+05,
      0.56783E+05, 0.60580E+05, 0.64582E+05, 0.68796E+05, 0.73232E+05,
      0.77898E+05, 0.82803E+05, 0.87957E+05, 0.93369E+05, 0.99048E+05,
      0.10501E+06, 0.11125E+06, 0.11780E+06, 0.12465E+06, 0.13182E+06,
      0.13933E+06, 0.14718E+06, 0.15539E+06, 0.16396E+06, 0.17291E+06,
      0.18226E+06, 0.19201E+06, 0.20218E+06, 0.21278E+06, 0.22383E+06,
      0.23534E+06, 0.24733E+06, 0.25980E+06, 0.27278E+06, 0.28628E+06,
      0.30032E+06, 0.31491E+06, 0.33007E+06, 0.34581E+06, 0.36216E+06,
      0.37912E+06, 0.39673E+06, 0.41499E+06, 0.43392E+06, 0.45355E+06,
      0.47389E+06, 0.49496E+06, 0.51678E+06, 0.53937E+06, 0.56276E+06,
      0.58695E+06, 0.61199E+06, 0.63788E+06, 0.66464E+06, 0.69231E+06,
      0.72090E+06, 0.75044E+06, 0.78094E+06, 0.81244E+06, 0.84496E+06,
      0.87853E+06, 0.91316E+06, 0.94889E+06, 0.98573E+06, 0.10237E+07,
      0.10629E+07, 0.11033E+07, 0.11449E+07, 0.11877E+07, 0.12319E+07,
      0.12773E+07, 0.13241E+07, 0.13723E+07, 0.14219E+07, 0.14729E+07,
      0.15254E+07, 0.15793E+07, 0.16349E+07, 0.16919E+07, 0.17506E+07,
      0.18109E+07])


#  --------------- N2O 546: M = 4, I = 3 ---------------------
M = 4
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.62051E+03, 0.87856E+03, 0.11377E+04,
      0.14003E+04, 0.16705E+04, 0.19529E+04, 0.22518E+04, 0.25713E+04,
      0.29149E+04, 0.32859E+04, 0.36873E+04, 0.41220E+04, 0.45929E+04,
      0.51028E+04, 0.56547E+04, 0.62515E+04, 0.68963E+04, 0.75923E+04,
      0.83428E+04, 0.91511E+04, 0.10021E+05, 0.10956E+05, 0.11960E+05,
      0.13036E+05, 0.14190E+05, 0.15425E+05, 0.16746E+05, 0.18158E+05,
      0.19664E+05, 0.21271E+05, 0.22984E+05, 0.24806E+05, 0.26745E+05,
      0.28806E+05, 0.30995E+05, 0.33317E+05, 0.35780E+05, 0.38389E+05,
      0.41151E+05, 0.44073E+05, 0.47162E+05, 0.50425E+05, 0.53871E+05,
      0.57505E+05, 0.61338E+05, 0.65375E+05, 0.69628E+05, 0.74102E+05,
      0.78808E+05, 0.83755E+05, 0.88951E+05, 0.94407E+05, 0.10013E+06,
      0.10614E+06, 0.11243E+06, 0.11902E+06, 0.12593E+06, 0.13316E+06,
      0.14072E+06, 0.14862E+06, 0.15689E+06, 0.16552E+06, 0.17453E+06,
      0.18394E+06, 0.19376E+06, 0.20399E+06, 0.21466E+06, 0.22578E+06,
      0.23737E+06, 0.24942E+06, 0.26198E+06, 0.27503E+06, 0.28861E+06,
      0.30273E+06, 0.31741E+06, 0.33265E+06, 0.34848E+06, 0.36492E+06,
      0.38197E+06, 0.39967E+06, 0.41803E+06, 0.43706E+06, 0.45679E+06,
      0.47723E+06, 0.49840E+06, 0.52033E+06, 0.54303E+06, 0.56653E+06,
      0.59084E+06, 0.61599E+06, 0.64200E+06, 0.66888E+06, 0.69667E+06,
      0.72539E+06, 0.75506E+06, 0.78569E+06, 0.81733E+06, 0.84998E+06,
      0.88369E+06, 0.91846E+06, 0.95433E+06, 0.99132E+06, 0.10295E+07,
      0.10688E+07, 0.11093E+07, 0.11511E+07, 0.11941E+07, 0.12384E+07,
      0.12840E+07, 0.13310E+07, 0.13793E+07, 0.14291E+07, 0.14803E+07,
      0.15329E+07, 0.15871E+07, 0.16428E+07, 0.17000E+07, 0.17589E+07,
      0.18194E+07])


#  --------------- N2O 448: M = 4, I = 4 ---------------------
M = 4
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(9.)
TIPS_ISO_HASH[(M,I)] = float32([0.95253E+03, 0.13487E+04, 0.17465E+04,
      0.21498E+04, 0.25648E+04, 0.29986E+04, 0.34580E+04, 0.39493E+04,
      0.44779E+04, 0.50488E+04, 0.56669E+04, 0.63366E+04, 0.70625E+04,
      0.78488E+04, 0.87003E+04, 0.96216E+04, 0.10617E+05, 0.11692E+05,
      0.12852E+05, 0.14102E+05, 0.15447E+05, 0.16893E+05, 0.18446E+05,
      0.20112E+05, 0.21898E+05, 0.23811E+05, 0.25856E+05, 0.28042E+05,
      0.30377E+05, 0.32866E+05, 0.35520E+05, 0.38345E+05, 0.41351E+05,
      0.44545E+05, 0.47939E+05, 0.51540E+05, 0.55359E+05, 0.59405E+05,
      0.63689E+05, 0.68222E+05, 0.73015E+05, 0.78078E+05, 0.83424E+05,
      0.89064E+05, 0.95012E+05, 0.10128E+06, 0.10788E+06, 0.11482E+06,
      0.12213E+06, 0.12981E+06, 0.13788E+06, 0.14635E+06, 0.15524E+06,
      0.16456E+06, 0.17433E+06, 0.18457E+06, 0.19530E+06, 0.20652E+06,
      0.21827E+06, 0.23055E+06, 0.24338E+06, 0.25679E+06, 0.27079E+06,
      0.28541E+06, 0.30066E+06, 0.31656E+06, 0.33314E+06, 0.35042E+06,
      0.36841E+06, 0.38715E+06, 0.40666E+06, 0.42695E+06, 0.44805E+06,
      0.46999E+06, 0.49279E+06, 0.51649E+06, 0.54109E+06, 0.56664E+06,
      0.59315E+06, 0.62066E+06, 0.64919E+06, 0.67877E+06, 0.70943E+06,
      0.74121E+06, 0.77413E+06, 0.80822E+06, 0.84351E+06, 0.88004E+06,
      0.91783E+06, 0.95693E+06, 0.99737E+06, 0.10392E+07, 0.10824E+07,
      0.11270E+07, 0.11732E+07, 0.12208E+07, 0.12700E+07, 0.13208E+07,
      0.13732E+07, 0.14272E+07, 0.14830E+07, 0.15405E+07, 0.15999E+07,
      0.16610E+07, 0.17240E+07, 0.17890E+07, 0.18559E+07, 0.19248E+07,
      0.19957E+07, 0.20687E+07, 0.21439E+07, 0.22213E+07, 0.23009E+07,
      0.23828E+07, 0.24671E+07, 0.25537E+07, 0.26428E+07, 0.27343E+07,
      0.28284E+07])


#  --------------- N2O 447: M = 4, I = 5 ---------------------
M = 4
I = 5
TIPS_GSI_HASH[(M,I)] = __FloatType__(54.)
TIPS_ISO_HASH[(M,I)] = float32([0.55598E+04, 0.78718E+04, 0.10193E+05,
      0.12546E+05, 0.14966E+05, 0.17495E+05, 0.20171E+05, 0.23031E+05,
      0.26106E+05, 0.29426E+05, 0.33018E+05, 0.36908E+05, 0.41121E+05,
      0.45684E+05, 0.50622E+05, 0.55962E+05, 0.61731E+05, 0.67958E+05,
      0.74671E+05, 0.81902E+05, 0.89681E+05, 0.98043E+05, 0.10702E+06,
      0.11665E+06, 0.12697E+06, 0.13801E+06, 0.14983E+06, 0.16244E+06,
      0.17591E+06, 0.19028E+06, 0.20558E+06, 0.22188E+06, 0.23920E+06,
      0.25762E+06, 0.27718E+06, 0.29793E+06, 0.31993E+06, 0.34323E+06,
      0.36791E+06, 0.39401E+06, 0.42160E+06, 0.45074E+06, 0.48151E+06,
      0.51397E+06, 0.54819E+06, 0.58424E+06, 0.62221E+06, 0.66215E+06,
      0.70416E+06, 0.74832E+06, 0.79470E+06, 0.84340E+06, 0.89450E+06,
      0.94808E+06, 0.10042E+07, 0.10631E+07, 0.11247E+07, 0.11892E+07,
      0.12567E+07, 0.13272E+07, 0.14009E+07, 0.14779E+07, 0.15583E+07,
      0.16422E+07, 0.17298E+07, 0.18211E+07, 0.19163E+07, 0.20154E+07,
      0.21187E+07, 0.22263E+07, 0.23382E+07, 0.24546E+07, 0.25757E+07,
      0.27016E+07, 0.28324E+07, 0.29683E+07, 0.31095E+07, 0.32560E+07,
      0.34081E+07, 0.35659E+07, 0.37295E+07, 0.38991E+07, 0.40750E+07,
      0.42572E+07, 0.44459E+07, 0.46414E+07, 0.48437E+07, 0.50531E+07,
      0.52698E+07, 0.54939E+07, 0.57257E+07, 0.59653E+07, 0.62129E+07,
      0.64688E+07, 0.67331E+07, 0.70061E+07, 0.72880E+07, 0.75790E+07,
      0.78792E+07, 0.81891E+07, 0.85086E+07, 0.88382E+07, 0.91780E+07,
      0.95283E+07, 0.98893E+07, 0.10261E+08, 0.10644E+08, 0.11039E+08,
      0.11445E+08, 0.11864E+08, 0.12294E+08, 0.12738E+08, 0.13194E+08,
      0.13663E+08, 0.14145E+08, 0.14641E+08, 0.15151E+08, 0.15675E+08,
      0.16214E+08])


#  --------------- CO 26: M = 5, I = 1 ---------------------
M = 5
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.21948E+02, 0.30961E+02, 0.39980E+02,
      0.49004E+02, 0.58035E+02, 0.67071E+02, 0.76112E+02, 0.85160E+02,
      0.94213E+02, 0.10327E+03, 0.11234E+03, 0.12142E+03, 0.13050E+03,
      0.13960E+03, 0.14872E+03, 0.15787E+03, 0.16704E+03, 0.17624E+03,
      0.18548E+03, 0.19477E+03, 0.20411E+03, 0.21350E+03, 0.22295E+03,
      0.23248E+03, 0.24207E+03, 0.25175E+03, 0.26151E+03, 0.27136E+03,
      0.28130E+03, 0.29134E+03, 0.30148E+03, 0.31172E+03, 0.32207E+03,
      0.33253E+03, 0.34312E+03, 0.35381E+03, 0.36463E+03, 0.37557E+03,
      0.38663E+03, 0.39782E+03, 0.40914E+03, 0.42060E+03, 0.43218E+03,
      0.44389E+03, 0.45575E+03, 0.46774E+03, 0.47987E+03, 0.49213E+03,
      0.50454E+03, 0.51708E+03, 0.52978E+03, 0.54261E+03, 0.55559E+03,
      0.56871E+03, 0.58198E+03, 0.59540E+03, 0.60896E+03, 0.62267E+03,
      0.63653E+03, 0.65055E+03, 0.66470E+03, 0.67901E+03, 0.69347E+03,
      0.70808E+03, 0.72284E+03, 0.73776E+03, 0.75283E+03, 0.76805E+03,
      0.78342E+03, 0.79895E+03, 0.81463E+03, 0.83047E+03, 0.84646E+03,
      0.86260E+03, 0.87891E+03, 0.89536E+03, 0.91197E+03, 0.92874E+03,
      0.94566E+03, 0.96275E+03, 0.97998E+03, 0.99738E+03, 0.10149E+04,
      0.10326E+04, 0.10505E+04, 0.10685E+04, 0.10867E+04, 0.11051E+04,
      0.11236E+04, 0.11422E+04, 0.11611E+04, 0.11800E+04, 0.11992E+04,
      0.12185E+04, 0.12380E+04, 0.12576E+04, 0.12774E+04, 0.12973E+04,
      0.13174E+04, 0.13377E+04, 0.13581E+04, 0.13787E+04, 0.13994E+04,
      0.14203E+04, 0.14414E+04, 0.14627E+04, 0.14841E+04, 0.15056E+04,
      0.15273E+04, 0.15492E+04, 0.15713E+04, 0.15935E+04, 0.16159E+04,
      0.16384E+04, 0.16611E+04, 0.16840E+04, 0.17070E+04, 0.17302E+04,
      0.17536E+04])


#  --------------- CO 36: M = 5, I = 2 ---------------------
M = 5
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.45888E+02, 0.64745E+02, 0.83615E+02,
      0.10250E+03, 0.12139E+03, 0.14030E+03, 0.15921E+03, 0.17814E+03,
      0.19708E+03, 0.21604E+03, 0.23501E+03, 0.25400E+03, 0.27302E+03,
      0.29207E+03, 0.31117E+03, 0.33031E+03, 0.34952E+03, 0.36880E+03,
      0.38817E+03, 0.40764E+03, 0.42723E+03, 0.44694E+03, 0.46679E+03,
      0.48679E+03, 0.50696E+03, 0.52730E+03, 0.54783E+03, 0.56855E+03,
      0.58948E+03, 0.61061E+03, 0.63198E+03, 0.65357E+03, 0.67539E+03,
      0.69747E+03, 0.71979E+03, 0.74237E+03, 0.76521E+03, 0.78832E+03,
      0.81169E+03, 0.83534E+03, 0.85927E+03, 0.88348E+03, 0.90798E+03,
      0.93277E+03, 0.95784E+03, 0.98322E+03, 0.10089E+04, 0.10349E+04,
      0.10611E+04, 0.10877E+04, 0.11146E+04, 0.11418E+04, 0.11693E+04,
      0.11971E+04, 0.12253E+04, 0.12537E+04, 0.12825E+04, 0.13115E+04,
      0.13409E+04, 0.13707E+04, 0.14007E+04, 0.14311E+04, 0.14617E+04,
      0.14928E+04, 0.15241E+04, 0.15558E+04, 0.15877E+04, 0.16200E+04,
      0.16527E+04, 0.16857E+04, 0.17190E+04, 0.17526E+04, 0.17866E+04,
      0.18209E+04, 0.18555E+04, 0.18905E+04, 0.19258E+04, 0.19614E+04,
      0.19974E+04, 0.20337E+04, 0.20703E+04, 0.21073E+04, 0.21446E+04,
      0.21823E+04, 0.22203E+04, 0.22586E+04, 0.22973E+04, 0.23363E+04,
      0.23756E+04, 0.24153E+04, 0.24553E+04, 0.24957E+04, 0.25364E+04,
      0.25775E+04, 0.26189E+04, 0.26606E+04, 0.27027E+04, 0.27451E+04,
      0.27879E+04, 0.28310E+04, 0.28745E+04, 0.29183E+04, 0.29625E+04,
      0.30070E+04, 0.30518E+04, 0.30970E+04, 0.31425E+04, 0.31885E+04,
      0.32347E+04, 0.32813E+04, 0.33282E+04, 0.33755E+04, 0.34231E+04,
      0.34711E+04, 0.35194E+04, 0.35681E+04, 0.36172E+04, 0.36666E+04,
      0.37163E+04])


#  --------------- CO 28: M = 5, I = 3 ---------------------
M = 5
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.23030E+02, 0.32495E+02, 0.41966E+02,
      0.51443E+02, 0.60926E+02, 0.70415E+02, 0.79910E+02, 0.89410E+02,
      0.98918E+02, 0.10843E+03, 0.11795E+03, 0.12749E+03, 0.13703E+03,
      0.14659E+03, 0.15618E+03, 0.16579E+03, 0.17543E+03, 0.18511E+03,
      0.19483E+03, 0.20461E+03, 0.21444E+03, 0.22434E+03, 0.23430E+03,
      0.24435E+03, 0.25447E+03, 0.26468E+03, 0.27499E+03, 0.28540E+03,
      0.29591E+03, 0.30652E+03, 0.31725E+03, 0.32810E+03, 0.33906E+03,
      0.35014E+03, 0.36136E+03, 0.37270E+03, 0.38417E+03, 0.39577E+03,
      0.40752E+03, 0.41940E+03, 0.43142E+03, 0.44358E+03, 0.45589E+03,
      0.46834E+03, 0.48094E+03, 0.49369E+03, 0.50659E+03, 0.51964E+03,
      0.53284E+03, 0.54619E+03, 0.55971E+03, 0.57337E+03, 0.58719E+03,
      0.60117E+03, 0.61530E+03, 0.62959E+03, 0.64405E+03, 0.65866E+03,
      0.67343E+03, 0.68837E+03, 0.70346E+03, 0.71872E+03, 0.73414E+03,
      0.74972E+03, 0.76547E+03, 0.78138E+03, 0.79745E+03, 0.81369E+03,
      0.83010E+03, 0.84667E+03, 0.86341E+03, 0.88031E+03, 0.89738E+03,
      0.91462E+03, 0.93202E+03, 0.94960E+03, 0.96734E+03, 0.98524E+03,
      0.10033E+04, 0.10216E+04, 0.10400E+04, 0.10586E+04, 0.10773E+04,
      0.10962E+04, 0.11153E+04, 0.11346E+04, 0.11540E+04, 0.11737E+04,
      0.11934E+04, 0.12134E+04, 0.12335E+04, 0.12538E+04, 0.12743E+04,
      0.12949E+04, 0.13157E+04, 0.13367E+04, 0.13578E+04, 0.13792E+04,
      0.14007E+04, 0.14223E+04, 0.14442E+04, 0.14662E+04, 0.14884E+04,
      0.15108E+04, 0.15333E+04, 0.15560E+04, 0.15789E+04, 0.16020E+04,
      0.16252E+04, 0.16486E+04, 0.16722E+04, 0.16960E+04, 0.17199E+04,
      0.17441E+04, 0.17684E+04, 0.17928E+04, 0.18175E+04, 0.18423E+04,
      0.18673E+04])


#  --------------- CO 27: M = 5, I = 4 ---------------------
M = 5
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.13505E+03, 0.19054E+03, 0.24606E+03,
      0.30161E+03, 0.35720E+03, 0.41283E+03, 0.46848E+03, 0.52418E+03,
      0.57991E+03, 0.63568E+03, 0.69149E+03, 0.74737E+03, 0.80332E+03,
      0.85937E+03, 0.91553E+03, 0.97183E+03, 0.10283E+04, 0.10850E+04,
      0.11420E+04, 0.11992E+04, 0.12568E+04, 0.13147E+04, 0.13730E+04,
      0.14318E+04, 0.14910E+04, 0.15507E+04, 0.16110E+04, 0.16718E+04,
      0.17332E+04, 0.17952E+04, 0.18579E+04, 0.19212E+04, 0.19852E+04,
      0.20499E+04, 0.21153E+04, 0.21815E+04, 0.22484E+04, 0.23161E+04,
      0.23846E+04, 0.24539E+04, 0.25240E+04, 0.25949E+04, 0.26666E+04,
      0.27392E+04, 0.28127E+04, 0.28869E+04, 0.29621E+04, 0.30381E+04,
      0.31150E+04, 0.31928E+04, 0.32715E+04, 0.33511E+04, 0.34316E+04,
      0.35129E+04, 0.35952E+04, 0.36785E+04, 0.37626E+04, 0.38477E+04,
      0.39336E+04, 0.40206E+04, 0.41084E+04, 0.41972E+04, 0.42869E+04,
      0.43776E+04, 0.44692E+04, 0.45618E+04, 0.46553E+04, 0.47498E+04,
      0.48452E+04, 0.49416E+04, 0.50390E+04, 0.51373E+04, 0.52366E+04,
      0.53368E+04, 0.54381E+04, 0.55403E+04, 0.56435E+04, 0.57476E+04,
      0.58527E+04, 0.59588E+04, 0.60659E+04, 0.61739E+04, 0.62829E+04,
      0.63930E+04, 0.65040E+04, 0.66160E+04, 0.67290E+04, 0.68429E+04,
      0.69579E+04, 0.70739E+04, 0.71908E+04, 0.73088E+04, 0.74277E+04,
      0.75477E+04, 0.76686E+04, 0.77905E+04, 0.79135E+04, 0.80374E+04,
      0.81624E+04, 0.82883E+04, 0.84153E+04, 0.85432E+04, 0.86722E+04,
      0.88022E+04, 0.89331E+04, 0.90651E+04, 0.91982E+04, 0.93322E+04,
      0.94672E+04, 0.96033E+04, 0.97404E+04, 0.98785E+04, 0.10018E+05,
      0.10158E+05, 0.10299E+05, 0.10441E+05, 0.10584E+05, 0.10728E+05,
      0.10874E+05])


#  --------------- CO 38: M = 5, I = 5 ---------------------
M = 5
I = 5
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.48264E+02, 0.68112E+02, 0.87974E+02,
      0.10785E+03, 0.12773E+03, 0.14763E+03, 0.16754E+03, 0.18747E+03,
      0.20741E+03, 0.22736E+03, 0.24733E+03, 0.26732E+03, 0.28735E+03,
      0.30741E+03, 0.32752E+03, 0.34770E+03, 0.36794E+03, 0.38828E+03,
      0.40871E+03, 0.42926E+03, 0.44994E+03, 0.47077E+03, 0.49175E+03,
      0.51290E+03, 0.53424E+03, 0.55578E+03, 0.57752E+03, 0.59948E+03,
      0.62166E+03, 0.64409E+03, 0.66676E+03, 0.68969E+03, 0.71287E+03,
      0.73633E+03, 0.76006E+03, 0.78407E+03, 0.80836E+03, 0.83295E+03,
      0.85784E+03, 0.88302E+03, 0.90851E+03, 0.93431E+03, 0.96042E+03,
      0.98686E+03, 0.10136E+04, 0.10407E+04, 0.10681E+04, 0.10958E+04,
      0.11238E+04, 0.11522E+04, 0.11809E+04, 0.12100E+04, 0.12393E+04,
      0.12691E+04, 0.12991E+04, 0.13295E+04, 0.13603E+04, 0.13914E+04,
      0.14228E+04, 0.14546E+04, 0.14867E+04, 0.15192E+04, 0.15520E+04,
      0.15852E+04, 0.16187E+04, 0.16526E+04, 0.16869E+04, 0.17215E+04,
      0.17564E+04, 0.17917E+04, 0.18274E+04, 0.18634E+04, 0.18998E+04,
      0.19365E+04, 0.19736E+04, 0.20111E+04, 0.20489E+04, 0.20871E+04,
      0.21256E+04, 0.21645E+04, 0.22038E+04, 0.22434E+04, 0.22834E+04,
      0.23238E+04, 0.23645E+04, 0.24056E+04, 0.24471E+04, 0.24889E+04,
      0.25311E+04, 0.25736E+04, 0.26166E+04, 0.26599E+04, 0.27035E+04,
      0.27476E+04, 0.27920E+04, 0.28368E+04, 0.28819E+04, 0.29275E+04,
      0.29733E+04, 0.30196E+04, 0.30662E+04, 0.31133E+04, 0.31606E+04,
      0.32084E+04, 0.32565E+04, 0.33050E+04, 0.33539E+04, 0.34032E+04,
      0.34528E+04, 0.35028E+04, 0.35532E+04, 0.36040E+04, 0.36551E+04,
      0.37067E+04, 0.37586E+04, 0.38108E+04, 0.38635E+04, 0.39165E+04,
      0.39699E+04])


#  --------------- CO 37: M = 5, I = 6 ---------------------
M = 5
I = 6
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.28271E+03, 0.39894E+03, 0.51524E+03,
      0.63162E+03, 0.74807E+03, 0.86459E+03, 0.98119E+03, 0.10979E+04,
      0.12146E+04, 0.13314E+04, 0.14484E+04, 0.15654E+04, 0.16826E+04,
      0.18000E+04, 0.19176E+04, 0.20355E+04, 0.21538E+04, 0.22725E+04,
      0.23916E+04, 0.25114E+04, 0.26318E+04, 0.27529E+04, 0.28749E+04,
      0.29977E+04, 0.31215E+04, 0.32463E+04, 0.33721E+04, 0.34991E+04,
      0.36274E+04, 0.37568E+04, 0.38876E+04, 0.40197E+04, 0.41533E+04,
      0.42882E+04, 0.44247E+04, 0.45626E+04, 0.47022E+04, 0.48433E+04,
      0.49860E+04, 0.51304E+04, 0.52763E+04, 0.54240E+04, 0.55735E+04,
      0.57246E+04, 0.58775E+04, 0.60321E+04, 0.61886E+04, 0.63468E+04,
      0.65068E+04, 0.66687E+04, 0.68324E+04, 0.69980E+04, 0.71654E+04,
      0.73347E+04, 0.75058E+04, 0.76789E+04, 0.78539E+04, 0.80307E+04,
      0.82096E+04, 0.83903E+04, 0.85729E+04, 0.87576E+04, 0.89441E+04,
      0.91326E+04, 0.93230E+04, 0.95154E+04, 0.97098E+04, 0.99061E+04,
      0.10104E+05, 0.10305E+05, 0.10507E+05, 0.10711E+05, 0.10918E+05,
      0.11126E+05, 0.11336E+05, 0.11549E+05, 0.11763E+05, 0.11979E+05,
      0.12198E+05, 0.12418E+05, 0.12640E+05, 0.12865E+05, 0.13091E+05,
      0.13320E+05, 0.13550E+05, 0.13783E+05, 0.14018E+05, 0.14254E+05,
      0.14493E+05, 0.14734E+05, 0.14977E+05, 0.15221E+05, 0.15468E+05,
      0.15718E+05, 0.15969E+05, 0.16222E+05, 0.16477E+05, 0.16734E+05,
      0.16994E+05, 0.17255E+05, 0.17519E+05, 0.17784E+05, 0.18052E+05,
      0.18322E+05, 0.18594E+05, 0.18868E+05, 0.19144E+05, 0.19422E+05,
      0.19703E+05, 0.19985E+05, 0.20270E+05, 0.20556E+05, 0.20845E+05,
      0.21136E+05, 0.21429E+05, 0.21724E+05, 0.22021E+05, 0.22320E+05,
      0.22622E+05])


#  --------------- CH4 211: M = 6, I = 1 ---------------------
M = 6
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.54800E+02, 0.91500E+02, 0.13410E+03,
      0.18180E+03, 0.23410E+03, 0.29070E+03, 0.35140E+03, 0.41600E+03,
      0.48450E+03, 0.55720E+03, 0.63420E+03, 0.71600E+03, 0.80310E+03,
      0.89590E+03, 0.99520E+03, 0.11017E+04, 0.12161E+04, 0.13393E+04,
      0.14721E+04, 0.16155E+04, 0.17706E+04, 0.19384E+04, 0.21202E+04,
      0.23172E+04, 0.25307E+04, 0.27624E+04, 0.30137E+04, 0.32864E+04,
      0.35823E+04, 0.39034E+04, 0.42519E+04, 0.46300E+04, 0.50402E+04,
      0.54853E+04, 0.59679E+04, 0.64913E+04, 0.70588E+04, 0.76739E+04,
      0.83404E+04, 0.90625E+04, 0.98446E+04, 0.10691E+05, 0.11608E+05,
      0.12600E+05, 0.13674E+05, 0.14835E+05, 0.16090E+05, 0.17447E+05,
      0.18914E+05, 0.20500E+05, 0.22212E+05, 0.24063E+05, 0.26061E+05,
      0.28218E+05, 0.30548E+05, 0.33063E+05, 0.35778E+05, 0.38708E+05,
      0.41871E+05, 0.45284E+05, 0.48970E+05, 0.52940E+05, 0.57230E+05,
      0.61860E+05, 0.66860E+05, 0.72250E+05, 0.78070E+05, 0.84350E+05,
      0.91130E+05, 0.98450E+05, 0.10635E+06, 0.11488E+06, 0.12408E+06,
      0.13403E+06, 0.14480E+06, 0.15640E+06, 0.16890E+06, 0.18240E+06,
      0.19700E+06, 0.21280E+06, 0.22980E+06, 0.24830E+06, 0.26820E+06,
      0.28970E+06, 0.31290E+06, 0.33800E+06, 0.36520E+06, 0.39450E+06,
      0.42600E+06, 0.46000E+06, 0.49700E+06, 0.53700E+06, 0.58100E+06,
      0.62700E+06, 0.67800E+06, 0.73300E+06, 0.79200E+06, 0.85600E+06,
      0.92500E+06, 0.10000E+07, 0.10800E+07, 0.11670E+07, 0.12610E+07,
      0.13620E+07, 0.14720E+07, 0.15910E+07, 0.17190E+07, 0.18600E+07,
      0.20100E+07, 0.21700E+07, 0.23400E+07, 0.25300E+07, 0.27300E+07,
      0.29500E+07, 0.31800E+07, 0.34300E+07, 0.37000E+07, 0.39900E+07,
      0.42856E+07])


#  --------------- CH4 311: M = 6, I = 2 ---------------------
M = 6
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.10958E+03, 0.18304E+03, 0.26818E+03,
      0.36356E+03, 0.46820E+03, 0.58141E+03, 0.70270E+03, 0.83186E+03,
      0.96893E+03, 0.11142E+04, 0.12682E+04, 0.14316E+04, 0.16055E+04,
      0.17909E+04, 0.19891E+04, 0.22016E+04, 0.24297E+04, 0.26752E+04,
      0.29399E+04, 0.32255E+04, 0.35342E+04, 0.38680E+04, 0.42294E+04,
      0.46208E+04, 0.50449E+04, 0.55046E+04, 0.60030E+04, 0.65434E+04,
      0.71293E+04, 0.77646E+04, 0.84535E+04, 0.92004E+04, 0.10010E+05,
      0.10888E+05, 0.11838E+05, 0.12869E+05, 0.13984E+05, 0.15193E+05,
      0.16501E+05, 0.17916E+05, 0.19448E+05, 0.21104E+05, 0.22895E+05,
      0.24830E+05, 0.26921E+05, 0.29180E+05, 0.31618E+05, 0.34250E+05,
      0.37090E+05, 0.40152E+05, 0.43454E+05, 0.47012E+05, 0.50845E+05,
      0.54973E+05, 0.59416E+05, 0.64197E+05, 0.69340E+05, 0.74870E+05,
      0.80813E+05, 0.87198E+05, 0.94055E+05, 0.10142E+06, 0.10932E+06,
      0.11779E+06, 0.12688E+06, 0.13662E+06, 0.14706E+06, 0.15824E+06,
      0.17021E+06, 0.18302E+06, 0.19673E+06, 0.21139E+06, 0.22706E+06,
      0.24381E+06, 0.26171E+06, 0.28082E+06, 0.30122E+06, 0.32299E+06,
      0.34621E+06, 0.37097E+06, 0.39737E+06, 0.42551E+06, 0.45548E+06,
      0.48739E+06, 0.52136E+06, 0.55752E+06, 0.59598E+06, 0.63688E+06,
      0.68036E+06, 0.72657E+06, 0.77566E+06, 0.82780E+06, 0.88316E+06,
      0.94191E+06, 0.10043E+07, 0.10704E+07, 0.11405E+07, 0.12148E+07,
      0.12936E+07, 0.13770E+07, 0.14654E+07, 0.15589E+07, 0.16579E+07,
      0.17627E+07, 0.18736E+07, 0.19908E+07, 0.21147E+07, 0.22456E+07,
      0.23840E+07, 0.25301E+07, 0.26844E+07, 0.28474E+07, 0.30193E+07,
      0.32007E+07, 0.33921E+07, 0.35939E+07, 0.38067E+07, 0.40310E+07,
      0.42673E+07])


#  --------------- CH4 212: M = 6, I = 3 ---------------------
M = 6
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.44079E+03, 0.73786E+03, 0.10822E+04,
      0.14679E+04, 0.18913E+04, 0.23497E+04, 0.28415E+04, 0.33665E+04,
      0.39257E+04, 0.45211E+04, 0.51562E+04, 0.58349E+04, 0.65624E+04,
      0.73445E+04, 0.81872E+04, 0.90978E+04, 0.10084E+05, 0.11153E+05,
      0.12315E+05, 0.13579E+05, 0.14955E+05, 0.16455E+05, 0.18089E+05,
      0.19871E+05, 0.21816E+05, 0.23937E+05, 0.26251E+05, 0.28776E+05,
      0.31531E+05, 0.34535E+05, 0.37811E+05, 0.41384E+05, 0.45278E+05,
      0.49521E+05, 0.54144E+05, 0.59178E+05, 0.64657E+05, 0.70621E+05,
      0.77108E+05, 0.84161E+05, 0.91828E+05, 0.10016E+06, 0.10921E+06,
      0.11903E+06, 0.12968E+06, 0.14124E+06, 0.15378E+06, 0.16736E+06,
      0.18207E+06, 0.19800E+06, 0.21524E+06, 0.23389E+06, 0.25405E+06,
      0.27585E+06, 0.29939E+06, 0.32482E+06, 0.35226E+06, 0.38186E+06,
      0.41379E+06, 0.44821E+06, 0.48529E+06, 0.52522E+06, 0.56821E+06,
      0.61447E+06, 0.66422E+06, 0.71771E+06, 0.77519E+06, 0.83693E+06,
      0.90323E+06, 0.97438E+06, 0.10507E+07, 0.11326E+07, 0.12203E+07,
      0.13143E+07, 0.14150E+07, 0.15228E+07, 0.16382E+07, 0.17616E+07,
      0.18935E+07, 0.20346E+07, 0.21853E+07, 0.23463E+07, 0.25181E+07,
      0.27016E+07, 0.28973E+07, 0.31060E+07, 0.33284E+07, 0.35655E+07,
      0.38181E+07, 0.40870E+07, 0.43733E+07, 0.46780E+07, 0.50020E+07,
      0.53467E+07, 0.57130E+07, 0.61023E+07, 0.65158E+07, 0.69549E+07,
      0.74211E+07, 0.79158E+07, 0.84407E+07, 0.89973E+07, 0.95874E+07,
      0.10213E+08, 0.10875E+08, 0.11577E+08, 0.12320E+08, 0.13107E+08,
      0.13940E+08, 0.14820E+08, 0.15752E+08, 0.16736E+08, 0.17777E+08,
      0.18877E+08, 0.20038E+08, 0.21265E+08, 0.22560E+08, 0.23927E+08,
      0.25369E+08])


#  --------------- CH4 312: M = 6, I = 4 ---------------------
M = 6
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.88231E+03, 0.14770E+04, 0.21661E+04,
      0.29384E+04, 0.37859E+04, 0.47034E+04, 0.56879E+04, 0.67388E+04,
      0.78581E+04, 0.90501E+04, 0.10321E+05, 0.11680E+05, 0.13136E+05,
      0.14702E+05, 0.16389E+05, 0.18212E+05, 0.20186E+05, 0.22328E+05,
      0.24654E+05, 0.27185E+05, 0.29941E+05, 0.32943E+05, 0.36216E+05,
      0.39786E+05, 0.43681E+05, 0.47930E+05, 0.52567E+05, 0.57625E+05,
      0.63144E+05, 0.69164E+05, 0.75730E+05, 0.82890E+05, 0.90693E+05,
      0.99198E+05, 0.10846E+06, 0.11855E+06, 0.12954E+06, 0.14149E+06,
      0.15450E+06, 0.16864E+06, 0.18402E+06, 0.20072E+06, 0.21886E+06,
      0.23856E+06, 0.25993E+06, 0.28312E+06, 0.30825E+06, 0.33550E+06,
      0.36501E+06, 0.39696E+06, 0.43155E+06, 0.46896E+06, 0.50942E+06,
      0.55315E+06, 0.60039E+06, 0.65141E+06, 0.70648E+06, 0.76589E+06,
      0.82997E+06, 0.89904E+06, 0.97346E+06, 0.10536E+07, 0.11399E+07,
      0.12327E+07, 0.13326E+07, 0.14400E+07, 0.15554E+07, 0.16793E+07,
      0.18124E+07, 0.19553E+07, 0.21085E+07, 0.22729E+07, 0.24490E+07,
      0.26378E+07, 0.28400E+07, 0.30565E+07, 0.32881E+07, 0.35360E+07,
      0.38010E+07, 0.40843E+07, 0.43870E+07, 0.47103E+07, 0.50555E+07,
      0.54239E+07, 0.58169E+07, 0.62361E+07, 0.66830E+07, 0.71592E+07,
      0.76666E+07, 0.82069E+07, 0.87820E+07, 0.93940E+07, 0.10045E+08,
      0.10737E+08, 0.11473E+08, 0.12256E+08, 0.13086E+08, 0.13969E+08,
      0.14905E+08, 0.15899E+08, 0.16954E+08, 0.18072E+08, 0.19258E+08,
      0.20515E+08, 0.21847E+08, 0.23257E+08, 0.24750E+08, 0.26331E+08,
      0.28004E+08, 0.29774E+08, 0.31646E+08, 0.33625E+08, 0.35716E+08,
      0.37926E+08, 0.40261E+08, 0.42726E+08, 0.45329E+08, 0.48077E+08,
      0.50975E+08])


#  --------------- O2 66: M = 7, I = 1 ---------------------
M = 7
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.44334E+02, 0.62460E+02, 0.80596E+02,
      0.98738E+02, 0.11688E+03, 0.13503E+03, 0.15319E+03, 0.17136E+03,
      0.18954E+03, 0.20775E+03, 0.22600E+03, 0.24431E+03, 0.26270E+03,
      0.28119E+03, 0.29981E+03, 0.31857E+03, 0.33750E+03, 0.35662E+03,
      0.37594E+03, 0.39550E+03, 0.41529E+03, 0.43535E+03, 0.45568E+03,
      0.47630E+03, 0.49722E+03, 0.51844E+03, 0.53998E+03, 0.56185E+03,
      0.58406E+03, 0.60660E+03, 0.62949E+03, 0.65274E+03, 0.67635E+03,
      0.70031E+03, 0.72465E+03, 0.74936E+03, 0.77444E+03, 0.79990E+03,
      0.82574E+03, 0.85197E+03, 0.87858E+03, 0.90558E+03, 0.93297E+03,
      0.96076E+03, 0.98895E+03, 0.10175E+04, 0.10465E+04, 0.10759E+04,
      0.11057E+04, 0.11359E+04, 0.11665E+04, 0.11976E+04, 0.12290E+04,
      0.12609E+04, 0.12931E+04, 0.13258E+04, 0.13590E+04, 0.13925E+04,
      0.14265E+04, 0.14609E+04, 0.14958E+04, 0.15311E+04, 0.15669E+04,
      0.16031E+04, 0.16397E+04, 0.16768E+04, 0.17144E+04, 0.17524E+04,
      0.17909E+04, 0.18298E+04, 0.18692E+04, 0.19091E+04, 0.19495E+04,
      0.19904E+04, 0.20318E+04, 0.20736E+04, 0.21160E+04, 0.21588E+04,
      0.22022E+04, 0.22461E+04, 0.22905E+04, 0.23354E+04, 0.23809E+04,
      0.24268E+04, 0.24734E+04, 0.25204E+04, 0.25680E+04, 0.26162E+04,
      0.26649E+04, 0.27142E+04, 0.27641E+04, 0.28145E+04, 0.28655E+04,
      0.29171E+04, 0.29693E+04, 0.30221E+04, 0.30755E+04, 0.31295E+04,
      0.31841E+04, 0.32393E+04, 0.32951E+04, 0.33516E+04, 0.34087E+04,
      0.34665E+04, 0.35249E+04, 0.35839E+04, 0.36436E+04, 0.37040E+04,
      0.37650E+04, 0.38267E+04, 0.38891E+04, 0.39522E+04, 0.40159E+04,
      0.40804E+04, 0.41455E+04, 0.42114E+04, 0.42780E+04, 0.43452E+04,
      0.44132E+04])


#  --------------- O2 68: M = 7, I = 2 ---------------------
M = 7
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.89206E+02, 0.12759E+03, 0.16600E+03,
      0.20442E+03, 0.24285E+03, 0.28128E+03, 0.31973E+03, 0.35821E+03,
      0.39672E+03, 0.43530E+03, 0.47398E+03, 0.51281E+03, 0.55183E+03,
      0.59108E+03, 0.63062E+03, 0.67051E+03, 0.71078E+03, 0.75148E+03,
      0.79265E+03, 0.83435E+03, 0.87659E+03, 0.91941E+03, 0.96285E+03,
      0.10069E+04, 0.10517E+04, 0.10971E+04, 0.11432E+04, 0.11901E+04,
      0.12377E+04, 0.12861E+04, 0.13352E+04, 0.13851E+04, 0.14358E+04,
      0.14872E+04, 0.15395E+04, 0.15926E+04, 0.16466E+04, 0.17013E+04,
      0.17569E+04, 0.18134E+04, 0.18706E+04, 0.19288E+04, 0.19877E+04,
      0.20476E+04, 0.21083E+04, 0.21698E+04, 0.22323E+04, 0.22956E+04,
      0.23598E+04, 0.24248E+04, 0.24908E+04, 0.25576E+04, 0.26253E+04,
      0.26940E+04, 0.27635E+04, 0.28339E+04, 0.29052E+04, 0.29775E+04,
      0.30506E+04, 0.31247E+04, 0.31997E+04, 0.32756E+04, 0.33524E+04,
      0.34302E+04, 0.35089E+04, 0.35885E+04, 0.36691E+04, 0.37506E+04,
      0.38331E+04, 0.39166E+04, 0.40010E+04, 0.40864E+04, 0.41727E+04,
      0.42601E+04, 0.43484E+04, 0.44377E+04, 0.45280E+04, 0.46193E+04,
      0.47116E+04, 0.48049E+04, 0.48992E+04, 0.49946E+04, 0.50909E+04,
      0.51883E+04, 0.52868E+04, 0.53863E+04, 0.54868E+04, 0.55884E+04,
      0.56911E+04, 0.57949E+04, 0.58997E+04, 0.60056E+04, 0.61126E+04,
      0.62207E+04, 0.63298E+04, 0.64401E+04, 0.65516E+04, 0.66641E+04,
      0.67778E+04, 0.68926E+04, 0.70085E+04, 0.71256E+04, 0.72439E+04,
      0.73633E+04, 0.74839E+04, 0.76056E+04, 0.77286E+04, 0.78527E+04,
      0.79781E+04, 0.81046E+04, 0.82324E+04, 0.83613E+04, 0.84915E+04,
      0.86229E+04, 0.87556E+04, 0.88895E+04, 0.90247E+04, 0.91611E+04,
      0.92988E+04])


#  --------------- O2 67: M = 7, I = 3 ---------------------
M = 7
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.52071E+03, 0.74484E+03, 0.96908E+03,
      0.11934E+04, 0.14177E+04, 0.16422E+04, 0.18667E+04, 0.20913E+04,
      0.23161E+04, 0.25413E+04, 0.27671E+04, 0.29936E+04, 0.32212E+04,
      0.34501E+04, 0.36806E+04, 0.39130E+04, 0.41476E+04, 0.43846E+04,
      0.46242E+04, 0.48668E+04, 0.51125E+04, 0.53615E+04, 0.56140E+04,
      0.58701E+04, 0.61300E+04, 0.63938E+04, 0.66617E+04, 0.69337E+04,
      0.72099E+04, 0.74904E+04, 0.77754E+04, 0.80647E+04, 0.83586E+04,
      0.86571E+04, 0.89602E+04, 0.92680E+04, 0.95805E+04, 0.98977E+04,
      0.10220E+05, 0.10547E+05, 0.10878E+05, 0.11215E+05, 0.11556E+05,
      0.11903E+05, 0.12254E+05, 0.12611E+05, 0.12972E+05, 0.13338E+05,
      0.13710E+05, 0.14086E+05, 0.14468E+05, 0.14855E+05, 0.15247E+05,
      0.15644E+05, 0.16046E+05, 0.16453E+05, 0.16866E+05, 0.17283E+05,
      0.17706E+05, 0.18135E+05, 0.18568E+05, 0.19007E+05, 0.19452E+05,
      0.19901E+05, 0.20356E+05, 0.20817E+05, 0.21283E+05, 0.21754E+05,
      0.22231E+05, 0.22713E+05, 0.23201E+05, 0.23695E+05, 0.24194E+05,
      0.24699E+05, 0.25209E+05, 0.25725E+05, 0.26247E+05, 0.26775E+05,
      0.27308E+05, 0.27847E+05, 0.28393E+05, 0.28944E+05, 0.29500E+05,
      0.30063E+05, 0.30632E+05, 0.31207E+05, 0.31788E+05, 0.32375E+05,
      0.32968E+05, 0.33568E+05, 0.34173E+05, 0.34785E+05, 0.35403E+05,
      0.36028E+05, 0.36659E+05, 0.37296E+05, 0.37939E+05, 0.38590E+05,
      0.39246E+05, 0.39909E+05, 0.40579E+05, 0.41256E+05, 0.41939E+05,
      0.42629E+05, 0.43325E+05, 0.44029E+05, 0.44739E+05, 0.45456E+05,
      0.46180E+05, 0.46911E+05, 0.47649E+05, 0.48394E+05, 0.49146E+05,
      0.49905E+05, 0.50671E+05, 0.51445E+05, 0.52226E+05, 0.53014E+05,
      0.53809E+05])


#  --------------- NO 46: M = 8, I = 1 ---------------------
M = 8
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.15840E+03, 0.23971E+03, 0.33080E+03,
      0.42907E+03, 0.53251E+03, 0.63972E+03, 0.74975E+03, 0.86195E+03,
      0.97582E+03, 0.10911E+04, 0.12074E+04, 0.13248E+04, 0.14430E+04,
      0.15621E+04, 0.16820E+04, 0.18027E+04, 0.19243E+04, 0.20468E+04,
      0.21703E+04, 0.22948E+04, 0.24204E+04, 0.25472E+04, 0.26753E+04,
      0.28046E+04, 0.29354E+04, 0.30676E+04, 0.32013E+04, 0.33365E+04,
      0.34734E+04, 0.36120E+04, 0.37522E+04, 0.38942E+04, 0.40379E+04,
      0.41835E+04, 0.43310E+04, 0.44803E+04, 0.46316E+04, 0.47849E+04,
      0.49400E+04, 0.50972E+04, 0.52564E+04, 0.54176E+04, 0.55809E+04,
      0.57462E+04, 0.59137E+04, 0.60832E+04, 0.62548E+04, 0.64286E+04,
      0.66045E+04, 0.67825E+04, 0.69628E+04, 0.71451E+04, 0.73297E+04,
      0.75164E+04, 0.77053E+04, 0.78964E+04, 0.80897E+04, 0.82853E+04,
      0.84830E+04, 0.86830E+04, 0.88852E+04, 0.90896E+04, 0.92963E+04,
      0.95052E+04, 0.97164E+04, 0.99297E+04, 0.10145E+05, 0.10363E+05,
      0.10583E+05, 0.10806E+05, 0.11031E+05, 0.11258E+05, 0.11487E+05,
      0.11718E+05, 0.11952E+05, 0.12188E+05, 0.12426E+05, 0.12667E+05,
      0.12910E+05, 0.13155E+05, 0.13403E+05, 0.13652E+05, 0.13905E+05,
      0.14159E+05, 0.14416E+05, 0.14675E+05, 0.14936E+05, 0.15199E+05,
      0.15465E+05, 0.15733E+05, 0.16004E+05, 0.16277E+05, 0.16552E+05,
      0.16829E+05, 0.17109E+05, 0.17391E+05, 0.17675E+05, 0.17962E+05,
      0.18251E+05, 0.18542E+05, 0.18836E+05, 0.19131E+05, 0.19430E+05,
      0.19730E+05, 0.20033E+05, 0.20338E+05, 0.20646E+05, 0.20955E+05,
      0.21268E+05, 0.21582E+05, 0.21899E+05, 0.22218E+05, 0.22539E+05,
      0.22863E+05, 0.23189E+05, 0.23518E+05, 0.23848E+05, 0.24181E+05,
      0.24517E+05])


#  --------------- NO 56: M = 8, I = 2 ---------------------
M = 8
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.10942E+03, 0.16560E+03, 0.22856E+03,
      0.29647E+03, 0.36795E+03, 0.44204E+03, 0.51808E+03, 0.59561E+03,
      0.67432E+03, 0.75396E+03, 0.83439E+03, 0.91551E+03, 0.99725E+03,
      0.10796E+04, 0.11625E+04, 0.12460E+04, 0.13302E+04, 0.14150E+04,
      0.15005E+04, 0.15868E+04, 0.16739E+04, 0.17618E+04, 0.18506E+04,
      0.19404E+04, 0.20311E+04, 0.21229E+04, 0.22158E+04, 0.23098E+04,
      0.24050E+04, 0.25013E+04, 0.25989E+04, 0.26976E+04, 0.27977E+04,
      0.28991E+04, 0.30018E+04, 0.31058E+04, 0.32112E+04, 0.33180E+04,
      0.34262E+04, 0.35358E+04, 0.36468E+04, 0.37593E+04, 0.38732E+04,
      0.39885E+04, 0.41054E+04, 0.42237E+04, 0.43436E+04, 0.44649E+04,
      0.45877E+04, 0.47121E+04, 0.48379E+04, 0.49654E+04, 0.50943E+04,
      0.52248E+04, 0.53568E+04, 0.54904E+04, 0.56255E+04, 0.57622E+04,
      0.59004E+04, 0.60403E+04, 0.61816E+04, 0.63246E+04, 0.64692E+04,
      0.66152E+04, 0.67630E+04, 0.69123E+04, 0.70631E+04, 0.72156E+04,
      0.73696E+04, 0.75253E+04, 0.76825E+04, 0.78414E+04, 0.80018E+04,
      0.81638E+04, 0.83275E+04, 0.84927E+04, 0.86596E+04, 0.88280E+04,
      0.89981E+04, 0.91698E+04, 0.93430E+04, 0.95180E+04, 0.96945E+04,
      0.98726E+04, 0.10052E+05, 0.10234E+05, 0.10417E+05, 0.10601E+05,
      0.10788E+05, 0.10975E+05, 0.11165E+05, 0.11356E+05, 0.11549E+05,
      0.11743E+05, 0.11939E+05, 0.12137E+05, 0.12336E+05, 0.12537E+05,
      0.12739E+05, 0.12943E+05, 0.13149E+05, 0.13356E+05, 0.13565E+05,
      0.13776E+05, 0.13988E+05, 0.14202E+05, 0.14418E+05, 0.14635E+05,
      0.14853E+05, 0.15074E+05, 0.15296E+05, 0.15520E+05, 0.15745E+05,
      0.15972E+05, 0.16200E+05, 0.16431E+05, 0.16663E+05, 0.16896E+05,
      0.17131E+05])


#  --------------- NO 48: M = 8, I = 3 ---------------------
M = 8
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.16695E+03, 0.25269E+03, 0.34876E+03,
      0.45239E+03, 0.56148E+03, 0.67455E+03, 0.79059E+03, 0.90891E+03,
      0.10290E+04, 0.11506E+04, 0.12733E+04, 0.13971E+04, 0.15219E+04,
      0.16476E+04, 0.17742E+04, 0.19017E+04, 0.20302E+04, 0.21598E+04,
      0.22904E+04, 0.24223E+04, 0.25553E+04, 0.26897E+04, 0.28255E+04,
      0.29628E+04, 0.31016E+04, 0.32420E+04, 0.33842E+04, 0.35280E+04,
      0.36736E+04, 0.38211E+04, 0.39704E+04, 0.41217E+04, 0.42750E+04,
      0.44302E+04, 0.45876E+04, 0.47469E+04, 0.49084E+04, 0.50720E+04,
      0.52378E+04, 0.54058E+04, 0.55759E+04, 0.57483E+04, 0.59230E+04,
      0.60999E+04, 0.62791E+04, 0.64605E+04, 0.66443E+04, 0.68304E+04,
      0.70187E+04, 0.72095E+04, 0.74026E+04, 0.75980E+04, 0.77958E+04,
      0.79960E+04, 0.81986E+04, 0.84036E+04, 0.86109E+04, 0.88207E+04,
      0.90328E+04, 0.92474E+04, 0.94644E+04, 0.96839E+04, 0.99057E+04,
      0.10130E+05, 0.10357E+05, 0.10586E+05, 0.10817E+05, 0.11052E+05,
      0.11288E+05, 0.11527E+05, 0.11768E+05, 0.12012E+05, 0.12259E+05,
      0.12507E+05, 0.12759E+05, 0.13012E+05, 0.13269E+05, 0.13527E+05,
      0.13788E+05, 0.14052E+05, 0.14318E+05, 0.14587E+05, 0.14858E+05,
      0.15131E+05, 0.15408E+05, 0.15686E+05, 0.15967E+05, 0.16251E+05,
      0.16537E+05, 0.16825E+05, 0.17116E+05, 0.17410E+05, 0.17706E+05,
      0.18004E+05, 0.18305E+05, 0.18609E+05, 0.18915E+05, 0.19224E+05,
      0.19535E+05, 0.19848E+05, 0.20164E+05, 0.20483E+05, 0.20804E+05,
      0.21127E+05, 0.21453E+05, 0.21782E+05, 0.22113E+05, 0.22447E+05,
      0.22783E+05, 0.23122E+05, 0.23463E+05, 0.23807E+05, 0.24153E+05,
      0.24502E+05, 0.24853E+05, 0.25207E+05, 0.25563E+05, 0.25922E+05,
      0.26283E+05])


#  --------------- SO2 626: M = 9, I = 1 ---------------------
M = 9
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.52899E+03, 0.89171E+03, 0.13139E+04,
      0.17915E+04, 0.23246E+04, 0.29155E+04, 0.35675E+04, 0.42848E+04,
      0.50723E+04, 0.59352E+04, 0.68794E+04, 0.79109E+04, 0.90366E+04,
      0.10264E+05, 0.11599E+05, 0.13052E+05, 0.14629E+05, 0.16340E+05,
      0.18193E+05, 0.20199E+05, 0.22366E+05, 0.24704E+05, 0.27225E+05,
      0.29938E+05, 0.32855E+05, 0.35987E+05, 0.39346E+05, 0.42944E+05,
      0.46794E+05, 0.50909E+05, 0.55302E+05, 0.59986E+05, 0.64977E+05,
      0.70288E+05, 0.75934E+05, 0.81931E+05, 0.88294E+05, 0.95040E+05,
      0.10219E+06, 0.10975E+06, 0.11774E+06, 0.12619E+06, 0.13511E+06,
      0.14452E+06, 0.15443E+06, 0.16487E+06, 0.17586E+06, 0.18742E+06,
      0.19957E+06, 0.21234E+06, 0.22573E+06, 0.23978E+06, 0.25451E+06,
      0.26995E+06, 0.28611E+06, 0.30302E+06, 0.32071E+06, 0.33920E+06,
      0.35852E+06, 0.37869E+06, 0.39974E+06, 0.42171E+06, 0.44461E+06,
      0.46848E+06, 0.49334E+06, 0.51922E+06, 0.54617E+06, 0.57419E+06,
      0.60334E+06, 0.63363E+06, 0.66511E+06, 0.69780E+06, 0.73174E+06,
      0.76696E+06, 0.80349E+06, 0.84138E+06, 0.88066E+06, 0.92136E+06,
      0.96352E+06, 0.10072E+07, 0.10524E+07, 0.10992E+07, 0.11475E+07,
      0.11976E+07, 0.12493E+07, 0.13028E+07, 0.13580E+07, 0.14151E+07,
      0.14741E+07, 0.15349E+07, 0.15977E+07, 0.16625E+07, 0.17293E+07,
      0.17982E+07, 0.18693E+07, 0.19425E+07, 0.20180E+07, 0.20958E+07,
      0.21758E+07, 0.22583E+07, 0.23432E+07, 0.24305E+07, 0.25204E+07,
      0.26129E+07, 0.27080E+07, 0.28058E+07, 0.29064E+07, 0.30097E+07,
      0.31159E+07, 0.32250E+07, 0.33371E+07, 0.34522E+07, 0.35705E+07,
      0.36918E+07, 0.38164E+07, 0.39442E+07, 0.40754E+07, 0.42099E+07,
      0.43479E+07])


#  --------------- SO2 646: M = 9, I = 2 ---------------------
M = 9
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.53140E+03, 0.89578E+03, 0.13199E+04,
      0.17997E+04, 0.23353E+04, 0.29288E+04, 0.35837E+04, 0.43043E+04,
      0.50953E+04, 0.59621E+04, 0.69104E+04, 0.79465E+04, 0.90772E+04,
      0.10310E+05, 0.11651E+05, 0.13110E+05, 0.14694E+05, 0.16413E+05,
      0.18274E+05, 0.20289E+05, 0.22465E+05, 0.24814E+05, 0.27345E+05,
      0.30070E+05, 0.33000E+05, 0.36145E+05, 0.39519E+05, 0.43133E+05,
      0.46999E+05, 0.51132E+05, 0.55544E+05, 0.60248E+05, 0.65260E+05,
      0.70594E+05, 0.76264E+05, 0.82287E+05, 0.88678E+05, 0.95453E+05,
      0.10263E+06, 0.11022E+06, 0.11825E+06, 0.12674E+06, 0.13569E+06,
      0.14514E+06, 0.15510E+06, 0.16558E+06, 0.17662E+06, 0.18823E+06,
      0.20043E+06, 0.21325E+06, 0.22670E+06, 0.24081E+06, 0.25561E+06,
      0.27111E+06, 0.28733E+06, 0.30432E+06, 0.32208E+06, 0.34065E+06,
      0.36005E+06, 0.38031E+06, 0.40145E+06, 0.42351E+06, 0.44651E+06,
      0.47047E+06, 0.49544E+06, 0.52144E+06, 0.54849E+06, 0.57664E+06,
      0.60591E+06, 0.63633E+06, 0.66794E+06, 0.70077E+06, 0.73485E+06,
      0.77022E+06, 0.80691E+06, 0.84496E+06, 0.88440E+06, 0.92527E+06,
      0.96761E+06, 0.10115E+07, 0.10568E+07, 0.11038E+07, 0.11524E+07,
      0.12027E+07, 0.12546E+07, 0.13083E+07, 0.13638E+07, 0.14211E+07,
      0.14803E+07, 0.15414E+07, 0.16045E+07, 0.16695E+07, 0.17366E+07,
      0.18059E+07, 0.18772E+07, 0.19507E+07, 0.20265E+07, 0.21046E+07,
      0.21850E+07, 0.22678E+07, 0.23531E+07, 0.24408E+07, 0.25310E+07,
      0.26239E+07, 0.27194E+07, 0.28176E+07, 0.29186E+07, 0.30224E+07,
      0.31290E+07, 0.32386E+07, 0.33512E+07, 0.34668E+07, 0.35855E+07,
      0.37074E+07, 0.38324E+07, 0.39608E+07, 0.40925E+07, 0.42276E+07,
      0.43662E+07])

#  --------------- NO2 646: M = 10, I = 1 ---------------------
M = 10
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.12046E+04, 0.20297E+04, 0.29875E+04,
      0.40626E+04, 0.52463E+04, 0.65350E+04, 0.79286E+04, 0.94298E+04,
      0.11043E+05, 0.12776E+05, 0.14634E+05, 0.16627E+05, 0.18765E+05,
      0.21056E+05, 0.23511E+05, 0.26143E+05, 0.28961E+05, 0.31979E+05,
      0.35209E+05, 0.38663E+05, 0.42355E+05, 0.46300E+05, 0.50510E+05,
      0.55001E+05, 0.59787E+05, 0.64884E+05, 0.70308E+05, 0.76075E+05,
      0.82201E+05, 0.88704E+05, 0.95602E+05, 0.10291E+06, 0.11065E+06,
      0.11884E+06, 0.12750E+06, 0.13665E+06, 0.14631E+06, 0.15650E+06,
      0.16724E+06, 0.17856E+06, 0.19047E+06, 0.20301E+06, 0.21618E+06,
      0.23002E+06, 0.24456E+06, 0.25981E+06, 0.27580E+06, 0.29256E+06,
      0.31012E+06, 0.32850E+06, 0.34773E+06, 0.36784E+06, 0.38886E+06,
      0.41082E+06, 0.43374E+06, 0.45766E+06, 0.48262E+06, 0.50863E+06,
      0.53574E+06, 0.56398E+06, 0.59339E+06, 0.62398E+06, 0.65581E+06,
      0.68891E+06, 0.72331E+06, 0.75905E+06, 0.79617E+06, 0.83470E+06,
      0.87469E+06, 0.91617E+06, 0.95919E+06, 0.10038E+07, 0.10500E+07,
      0.10979E+07, 0.11474E+07, 0.11988E+07, 0.12519E+07, 0.13068E+07,
      0.13636E+07, 0.14224E+07, 0.14831E+07, 0.15459E+07, 0.16107E+07,
      0.16776E+07, 0.17467E+07, 0.18180E+07, 0.18916E+07, 0.19675E+07,
      0.20458E+07, 0.21265E+07, 0.22097E+07, 0.22954E+07, 0.23837E+07,
      0.24747E+07, 0.25684E+07, 0.26648E+07, 0.27641E+07, 0.28662E+07,
      0.29713E+07, 0.30794E+07, 0.31905E+07, 0.33048E+07, 0.34223E+07,
      0.35430E+07, 0.36670E+07, 0.37944E+07, 0.39253E+07, 0.40597E+07,
      0.41976E+07, 0.43393E+07, 0.44846E+07, 0.46337E+07, 0.47867E+07,
      0.49437E+07, 0.51046E+07, 0.52696E+07, 0.54388E+07, 0.56122E+07,
      0.57900E+07])


#  --------------- NH3 4111: M = 11, I = 1 ---------------------
M = 11
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.16013E+03, 0.26692E+03, 0.39067E+03,
      0.52933E+03, 0.68153E+03, 0.84641E+03, 0.10234E+04, 0.12125E+04,
      0.14136E+04, 0.16272E+04, 0.18537E+04, 0.20937E+04, 0.23481E+04,
      0.26177E+04, 0.29035E+04, 0.32065E+04, 0.35279E+04, 0.38688E+04,
      0.42304E+04, 0.46141E+04, 0.50212E+04, 0.54531E+04, 0.59114E+04,
      0.63976E+04, 0.69133E+04, 0.74602E+04, 0.80401E+04, 0.86549E+04,
      0.93066E+04, 0.99971E+04, 0.10729E+05, 0.11504E+05, 0.12324E+05,
      0.13193E+05, 0.14112E+05, 0.15085E+05, 0.16114E+05, 0.17201E+05,
      0.18352E+05, 0.19567E+05, 0.20851E+05, 0.22208E+05, 0.23640E+05,
      0.25152E+05, 0.26747E+05, 0.28430E+05, 0.30205E+05, 0.32077E+05,
      0.34050E+05, 0.36128E+05, 0.38317E+05, 0.40623E+05, 0.43050E+05,
      0.45605E+05, 0.48292E+05, 0.51119E+05, 0.54091E+05, 0.57215E+05,
      0.60498E+05, 0.63947E+05, 0.67569E+05, 0.71372E+05, 0.75364E+05,
      0.79552E+05, 0.83946E+05, 0.88553E+05, 0.93384E+05, 0.98447E+05,
      0.10375E+06, 0.10931E+06, 0.11513E+06, 0.12122E+06, 0.12760E+06,
      0.13427E+06, 0.14125E+06, 0.14855E+06, 0.15619E+06, 0.16417E+06,
      0.17250E+06, 0.18121E+06, 0.19031E+06, 0.19981E+06, 0.20973E+06,
      0.22008E+06, 0.23088E+06, 0.24215E+06, 0.25390E+06, 0.26615E+06,
      0.27892E+06, 0.29223E+06, 0.30610E+06, 0.32055E+06, 0.33559E+06,
      0.35125E+06, 0.36756E+06, 0.38453E+06, 0.40219E+06, 0.42056E+06,
      0.43967E+06, 0.45953E+06, 0.48019E+06, 0.50165E+06, 0.52396E+06,
      0.54714E+06, 0.57122E+06, 0.59622E+06, 0.62218E+06, 0.64913E+06,
      0.67710E+06, 0.70613E+06, 0.73624E+06, 0.76748E+06, 0.79988E+06,
      0.83347E+06, 0.86829E+06, 0.90439E+06, 0.94180E+06, 0.98056E+06,
      0.10207E+07])


#  --------------- NH3 5111: M = 11, I = 2 ---------------------
M = 11
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.10697E+03, 0.17832E+03, 0.26100E+03,
      0.35364E+03, 0.45533E+03, 0.56549E+03, 0.68377E+03, 0.81007E+03,
      0.94447E+03, 0.10872E+04, 0.12385E+04, 0.13988E+04, 0.15688E+04,
      0.17490E+04, 0.19399E+04, 0.21424E+04, 0.23571E+04, 0.25848E+04,
      0.28264E+04, 0.30828E+04, 0.33548E+04, 0.36434E+04, 0.39496E+04,
      0.42745E+04, 0.46190E+04, 0.49845E+04, 0.53720E+04, 0.57828E+04,
      0.62182E+04, 0.66796E+04, 0.71684E+04, 0.76862E+04, 0.82344E+04,
      0.88149E+04, 0.94292E+04, 0.10079E+05, 0.10767E+05, 0.11494E+05,
      0.12262E+05, 0.13074E+05, 0.13932E+05, 0.14839E+05, 0.15796E+05,
      0.16806E+05, 0.17872E+05, 0.18997E+05, 0.20183E+05, 0.21434E+05,
      0.22752E+05, 0.24141E+05, 0.25604E+05, 0.27145E+05, 0.28767E+05,
      0.30475E+05, 0.32271E+05, 0.34160E+05, 0.36146E+05, 0.38234E+05,
      0.40428E+05, 0.42733E+05, 0.45154E+05, 0.47696E+05, 0.50364E+05,
      0.53163E+05, 0.56100E+05, 0.59180E+05, 0.62408E+05, 0.65792E+05,
      0.69339E+05, 0.73053E+05, 0.76943E+05, 0.81016E+05, 0.85279E+05,
      0.89740E+05, 0.94406E+05, 0.99287E+05, 0.10439E+06, 0.10972E+06,
      0.11530E+06, 0.12112E+06, 0.12720E+06, 0.13355E+06, 0.14018E+06,
      0.14711E+06, 0.15433E+06, 0.16186E+06, 0.16971E+06, 0.17791E+06,
      0.18645E+06, 0.19534E+06, 0.20462E+06, 0.21428E+06, 0.22434E+06,
      0.23481E+06, 0.24572E+06, 0.25706E+06, 0.26887E+06, 0.28116E+06,
      0.29393E+06, 0.30722E+06, 0.32103E+06, 0.33539E+06, 0.35031E+06,
      0.36581E+06, 0.38191E+06, 0.39864E+06, 0.41600E+06, 0.43403E+06,
      0.45274E+06, 0.47215E+06, 0.49230E+06, 0.51319E+06, 0.53487E+06,
      0.55734E+06, 0.58064E+06, 0.60478E+06, 0.62981E+06, 0.65574E+06,
      0.68260E+06])


#  --------------- HNO3 146: M = 12, I = 1 ---------------------
M = 12
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.15010E+05, 0.25316E+05, 0.37374E+05,
      0.51216E+05, 0.67105E+05, 0.85473E+05, 0.10688E+06, 0.13201E+06,
      0.16165E+06, 0.19671E+06, 0.23825E+06, 0.28749E+06, 0.34583E+06,
      0.41490E+06, 0.49657E+06, 0.59302E+06, 0.70673E+06, 0.84054E+06,
      0.99775E+06, 0.11821E+07, 0.13978E+07, 0.16498E+07, 0.19436E+07,
      0.22855E+07, 0.26825E+07, 0.31428E+07, 0.36753E+07, 0.42903E+07,
      0.49993E+07, 0.58151E+07, 0.67523E+07, 0.78269E+07, 0.90572E+07,
      0.10463E+08, 0.12067E+08, 0.13895E+08, 0.15973E+08, 0.18333E+08,
      0.21009E+08, 0.24039E+08, 0.27464E+08, 0.31331E+08, 0.35690E+08,
      0.40597E+08, 0.46115E+08, 0.52310E+08, 0.59257E+08, 0.67037E+08,
      0.75739E+08, 0.85461E+08, 0.96310E+08, 0.10840E+09, 0.12186E+09,
      0.13683E+09, 0.15346E+09, 0.17191E+09, 0.19236E+09, 0.21501E+09,
      0.24006E+09, 0.26774E+09, 0.29830E+09, 0.33200E+09, 0.36914E+09,
      0.41002E+09, 0.45498E+09, 0.50438E+09, 0.55862E+09, 0.61812E+09,
      0.68332E+09, 0.75473E+09, 0.83286E+09, 0.91828E+09, 0.10116E+10,
      0.11134E+10, 0.12245E+10, 0.13456E+10, 0.14775E+10, 0.16210E+10,
      0.17771E+10, 0.19467E+10, 0.21309E+10, 0.23309E+10, 0.25477E+10,
      0.27827E+10, 0.30372E+10, 0.33127E+10, 0.36107E+10, 0.39329E+10,
      0.42809E+10, 0.46567E+10, 0.50623E+10, 0.54997E+10, 0.59711E+10,
      0.64789E+10, 0.70257E+10, 0.76140E+10, 0.82468E+10, 0.89269E+10,
      0.96575E+10, 0.10442E+11, 0.11284E+11, 0.12187E+11, 0.13155E+11,
      0.14193E+11, 0.15304E+11, 0.16494E+11, 0.17767E+11, 0.19129E+11,
      0.20585E+11, 0.22140E+11, 0.23802E+11, 0.25576E+11, 0.27469E+11,
      0.29489E+11, 0.31642E+11, 0.33937E+11, 0.36382E+11, 0.38985E+11,
      0.41757E+11])


#  --------------- HNO3 156: M = 12, I = 2 --------------------- NOT IN TIPS-2011
M = 12
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- OH 61: M = 13, I = 1 ---------------------
M = 13
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.20066E+02, 0.24774E+02, 0.30309E+02,
      0.36357E+02, 0.42745E+02, 0.49371E+02, 0.56168E+02, 0.63093E+02,
      0.70116E+02, 0.77217E+02, 0.84380E+02, 0.91594E+02, 0.98850E+02,
      0.10614E+03, 0.11346E+03, 0.12081E+03, 0.12818E+03, 0.13557E+03,
      0.14298E+03, 0.15041E+03, 0.15785E+03, 0.16531E+03, 0.17278E+03,
      0.18027E+03, 0.18778E+03, 0.19530E+03, 0.20284E+03, 0.21040E+03,
      0.21797E+03, 0.22556E+03, 0.23318E+03, 0.24082E+03, 0.24848E+03,
      0.25617E+03, 0.26389E+03, 0.27163E+03, 0.27941E+03, 0.28721E+03,
      0.29505E+03, 0.30292E+03, 0.31084E+03, 0.31878E+03, 0.32677E+03,
      0.33480E+03, 0.34287E+03, 0.35099E+03, 0.35915E+03, 0.36736E+03,
      0.37561E+03, 0.38391E+03, 0.39227E+03, 0.40067E+03, 0.40913E+03,
      0.41764E+03, 0.42620E+03, 0.43482E+03, 0.44350E+03, 0.45223E+03,
      0.46102E+03, 0.46987E+03, 0.47878E+03, 0.48775E+03, 0.49679E+03,
      0.50588E+03, 0.51503E+03, 0.52425E+03, 0.53354E+03, 0.54288E+03,
      0.55229E+03, 0.56177E+03, 0.57132E+03, 0.58092E+03, 0.59060E+03,
      0.60035E+03, 0.61016E+03, 0.62004E+03, 0.62999E+03, 0.64001E+03,
      0.65010E+03, 0.66025E+03, 0.67049E+03, 0.68078E+03, 0.69115E+03,
      0.70160E+03, 0.71211E+03, 0.72269E+03, 0.73335E+03, 0.74408E+03,
      0.75488E+03, 0.76576E+03, 0.77671E+03, 0.78773E+03, 0.79883E+03,
      0.81000E+03, 0.82124E+03, 0.83256E+03, 0.84396E+03, 0.85542E+03,
      0.86696E+03, 0.87858E+03, 0.89027E+03, 0.90204E+03, 0.91389E+03,
      0.92580E+03, 0.93781E+03, 0.94988E+03, 0.96203E+03, 0.97425E+03,
      0.98656E+03, 0.99893E+03, 0.10114E+04, 0.10239E+04, 0.10365E+04,
      0.10492E+04, 0.10620E+04, 0.10748E+04, 0.10878E+04, 0.11007E+04,
      0.11138E+04])

#  --------------- OH 81: M = 13, I = 2 ---------------------
M = 13
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.20124E+02, 0.24876E+02, 0.30457E+02,
      0.36553E+02, 0.42991E+02, 0.49666E+02, 0.56513E+02, 0.63489E+02,
      0.70563E+02, 0.77715E+02, 0.84929E+02, 0.92195E+02, 0.99504E+02,
      0.10685E+03, 0.11423E+03, 0.12164E+03, 0.12907E+03, 0.13654E+03,
      0.14403E+03, 0.15154E+03, 0.15909E+03, 0.16666E+03, 0.17427E+03,
      0.18191E+03, 0.18959E+03, 0.19731E+03, 0.20507E+03, 0.21287E+03,
      0.22073E+03, 0.22863E+03, 0.23658E+03, 0.24459E+03, 0.25266E+03,
      0.26078E+03, 0.26897E+03, 0.27722E+03, 0.28554E+03, 0.29393E+03,
      0.30238E+03, 0.31091E+03, 0.31952E+03, 0.32820E+03, 0.33696E+03,
      0.34579E+03, 0.35471E+03, 0.36371E+03, 0.37279E+03, 0.38196E+03,
      0.39121E+03, 0.40055E+03, 0.40998E+03, 0.41949E+03, 0.42910E+03,
      0.43879E+03, 0.44858E+03, 0.45845E+03, 0.46843E+03, 0.47849E+03,
      0.48865E+03, 0.49890E+03, 0.50924E+03, 0.51969E+03, 0.53022E+03,
      0.54086E+03, 0.55159E+03, 0.56242E+03, 0.57335E+03, 0.58437E+03,
      0.59550E+03, 0.60673E+03, 0.61805E+03, 0.62947E+03, 0.64100E+03,
      0.65263E+03, 0.66435E+03, 0.67618E+03, 0.68811E+03, 0.70014E+03,
      0.71228E+03, 0.72451E+03, 0.73685E+03, 0.74929E+03, 0.76184E+03,
      0.77449E+03, 0.78724E+03, 0.80009E+03, 0.81306E+03, 0.82612E+03,
      0.83929E+03, 0.85256E+03, 0.86594E+03, 0.87942E+03, 0.89301E+03,
      0.90670E+03, 0.92050E+03, 0.93440E+03, 0.94841E+03, 0.96253E+03,
      0.97675E+03, 0.99108E+03, 0.10055E+04, 0.10201E+04, 0.10347E+04,
      0.10495E+04, 0.10643E+04, 0.10793E+04, 0.10944E+04, 0.11096E+04,
      0.11248E+04, 0.11402E+04, 0.11558E+04, 0.11714E+04, 0.11871E+04,
      0.12029E+04, 0.12189E+04, 0.12349E+04, 0.12511E+04, 0.12673E+04,
      0.12837E+04])

#  --------------- OH 62: M = 13, I = 3 ---------------------
M = 13
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.41032E+02, 0.54704E+02, 0.70201E+02,
      0.86985E+02, 0.10469E+03, 0.12306E+03, 0.14194E+03, 0.16119E+03,
      0.18075E+03, 0.20054E+03, 0.22053E+03, 0.24068E+03, 0.26096E+03,
      0.28135E+03, 0.30183E+03, 0.32241E+03, 0.34305E+03, 0.36376E+03,
      0.38453E+03, 0.40535E+03, 0.42622E+03, 0.44714E+03, 0.46811E+03,
      0.48913E+03, 0.51019E+03, 0.53131E+03, 0.55246E+03, 0.57368E+03,
      0.59495E+03, 0.61627E+03, 0.63766E+03, 0.65912E+03, 0.68064E+03,
      0.70223E+03, 0.72390E+03, 0.74565E+03, 0.76749E+03, 0.78941E+03,
      0.81143E+03, 0.83355E+03, 0.85578E+03, 0.87810E+03, 0.90054E+03,
      0.92310E+03, 0.94577E+03, 0.96857E+03, 0.99149E+03, 0.10145E+04,
      0.10377E+04, 0.10611E+04, 0.10845E+04, 0.11081E+04, 0.11319E+04,
      0.11558E+04, 0.11798E+04, 0.12040E+04, 0.12284E+04, 0.12529E+04,
      0.12776E+04, 0.13025E+04, 0.13275E+04, 0.13527E+04, 0.13781E+04,
      0.14036E+04, 0.14293E+04, 0.14552E+04, 0.14813E+04, 0.15076E+04,
      0.15340E+04, 0.15606E+04, 0.15874E+04, 0.16144E+04, 0.16416E+04,
      0.16690E+04, 0.16965E+04, 0.17243E+04, 0.17522E+04, 0.17804E+04,
      0.18087E+04, 0.18373E+04, 0.18660E+04, 0.18949E+04, 0.19241E+04,
      0.19534E+04, 0.19829E+04, 0.20127E+04, 0.20426E+04, 0.20727E+04,
      0.21031E+04, 0.21336E+04, 0.21644E+04, 0.21954E+04, 0.22266E+04,
      0.22579E+04, 0.22895E+04, 0.23213E+04, 0.23534E+04, 0.23856E+04,
      0.24180E+04, 0.24506E+04, 0.24835E+04, 0.25166E+04, 0.25499E+04,
      0.25834E+04, 0.26171E+04, 0.26510E+04, 0.26852E+04, 0.27195E+04,
      0.27541E+04, 0.27889E+04, 0.28239E+04, 0.28592E+04, 0.28946E+04,
      0.29303E+04, 0.29661E+04, 0.30023E+04, 0.30386E+04, 0.30751E+04,
      0.31119E+04])


#  --------------- HF 19: M = 14, I = 1 ---------------------
M = 14
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.95958E+01, 0.12933E+02, 0.16295E+02,
      0.19666E+02, 0.23043E+02, 0.26425E+02, 0.29809E+02, 0.33195E+02,
      0.36584E+02, 0.39974E+02, 0.43366E+02, 0.46759E+02, 0.50154E+02,
      0.53550E+02, 0.56947E+02, 0.60346E+02, 0.63746E+02, 0.67148E+02,
      0.70550E+02, 0.73955E+02, 0.77361E+02, 0.80769E+02, 0.84179E+02,
      0.87591E+02, 0.91006E+02, 0.94424E+02, 0.97846E+02, 0.10127E+03,
      0.10470E+03, 0.10813E+03, 0.11157E+03, 0.11502E+03, 0.11847E+03,
      0.12193E+03, 0.12540E+03, 0.12888E+03, 0.13236E+03, 0.13586E+03,
      0.13936E+03, 0.14288E+03, 0.14641E+03, 0.14995E+03, 0.15351E+03,
      0.15708E+03, 0.16066E+03, 0.16426E+03, 0.16788E+03, 0.17151E+03,
      0.17516E+03, 0.17882E+03, 0.18251E+03, 0.18621E+03, 0.18994E+03,
      0.19368E+03, 0.19745E+03, 0.20123E+03, 0.20504E+03, 0.20887E+03,
      0.21272E+03, 0.21659E+03, 0.22049E+03, 0.22441E+03, 0.22836E+03,
      0.23233E+03, 0.23632E+03, 0.24034E+03, 0.24439E+03, 0.24846E+03,
      0.25255E+03, 0.25668E+03, 0.26083E+03, 0.26501E+03, 0.26921E+03,
      0.27344E+03, 0.27770E+03, 0.28199E+03, 0.28631E+03, 0.29066E+03,
      0.29503E+03, 0.29944E+03, 0.30387E+03, 0.30833E+03, 0.31282E+03,
      0.31735E+03, 0.32190E+03, 0.32648E+03, 0.33110E+03, 0.33574E+03,
      0.34042E+03, 0.34512E+03, 0.34986E+03, 0.35463E+03, 0.35943E+03,
      0.36426E+03, 0.36913E+03, 0.37402E+03, 0.37895E+03, 0.38391E+03,
      0.38891E+03, 0.39393E+03, 0.39899E+03, 0.40408E+03, 0.40921E+03,
      0.41436E+03, 0.41955E+03, 0.42478E+03, 0.43004E+03, 0.43533E+03,
      0.44065E+03, 0.44601E+03, 0.45140E+03, 0.45683E+03, 0.46229E+03,
      0.46779E+03, 0.47332E+03, 0.47888E+03, 0.48448E+03, 0.49011E+03,
      0.49578E+03])

#  --------------- HF 29: M = 14, I = 2 --------------------- not in TIPS-2011
M = 14
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- HСl 15: M = 15, I = 1 --------------------
M = 15
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.34775E+02, 0.48060E+02, 0.61370E+02,
      0.74692E+02, 0.88024E+02, 0.10136E+03, 0.11471E+03, 0.12806E+03,
      0.14141E+03, 0.15478E+03, 0.16814E+03, 0.18151E+03, 0.19489E+03,
      0.20827E+03, 0.22166E+03, 0.23506E+03, 0.24847E+03, 0.26189E+03,
      0.27533E+03, 0.28878E+03, 0.30225E+03, 0.31575E+03, 0.32928E+03,
      0.34284E+03, 0.35645E+03, 0.37009E+03, 0.38378E+03, 0.39753E+03,
      0.41134E+03, 0.42521E+03, 0.43914E+03, 0.45316E+03, 0.46725E+03,
      0.48142E+03, 0.49568E+03, 0.51003E+03, 0.52448E+03, 0.53902E+03,
      0.55368E+03, 0.56843E+03, 0.58330E+03, 0.59829E+03, 0.61339E+03,
      0.62862E+03, 0.64396E+03, 0.65944E+03, 0.67504E+03, 0.69078E+03,
      0.70665E+03, 0.72265E+03, 0.73880E+03, 0.75508E+03, 0.77151E+03,
      0.78809E+03, 0.80481E+03, 0.82168E+03, 0.83870E+03, 0.85587E+03,
      0.87320E+03, 0.89068E+03, 0.90832E+03, 0.92611E+03, 0.94407E+03,
      0.96218E+03, 0.98046E+03, 0.99889E+03, 0.10175E+04, 0.10363E+04,
      0.10552E+04, 0.10743E+04, 0.10936E+04, 0.11130E+04, 0.11326E+04,
      0.11524E+04, 0.11723E+04, 0.11924E+04, 0.12127E+04, 0.12332E+04,
      0.12538E+04, 0.12746E+04, 0.12956E+04, 0.13168E+04, 0.13381E+04,
      0.13597E+04, 0.13814E+04, 0.14032E+04, 0.14253E+04, 0.14475E+04,
      0.14700E+04, 0.14926E+04, 0.15153E+04, 0.15383E+04, 0.15615E+04,
      0.15848E+04, 0.16083E+04, 0.16320E+04, 0.16559E+04, 0.16800E+04,
      0.17043E+04, 0.17287E+04, 0.17533E+04, 0.17782E+04, 0.18032E+04,
      0.18284E+04, 0.18538E+04, 0.18794E+04, 0.19051E+04, 0.19311E+04,
      0.19573E+04, 0.19836E+04, 0.20102E+04, 0.20369E+04, 0.20638E+04,
      0.20910E+04, 0.21183E+04, 0.21458E+04, 0.21735E+04, 0.22014E+04,
      0.22295E+04])


#  --------------- HСl 17: M = 15, I = 2 ---------------------
M = 15
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.34823E+02, 0.48128E+02, 0.61458E+02,
      0.74801E+02, 0.88152E+02, 0.10151E+03, 0.11488E+03, 0.12825E+03,
      0.14162E+03, 0.15500E+03, 0.16839E+03, 0.18178E+03, 0.19518E+03,
      0.20858E+03, 0.22199E+03, 0.23541E+03, 0.24884E+03, 0.26228E+03,
      0.27574E+03, 0.28921E+03, 0.30270E+03, 0.31622E+03, 0.32977E+03,
      0.34336E+03, 0.35698E+03, 0.37065E+03, 0.38436E+03, 0.39813E+03,
      0.41196E+03, 0.42585E+03, 0.43981E+03, 0.45384E+03, 0.46796E+03,
      0.48215E+03, 0.49644E+03, 0.51081E+03, 0.52528E+03, 0.53986E+03,
      0.55453E+03, 0.56932E+03, 0.58421E+03, 0.59922E+03, 0.61435E+03,
      0.62960E+03, 0.64498E+03, 0.66048E+03, 0.67611E+03, 0.69187E+03,
      0.70777E+03, 0.72381E+03, 0.73998E+03, 0.75630E+03, 0.77276E+03,
      0.78936E+03, 0.80612E+03, 0.82302E+03, 0.84007E+03, 0.85727E+03,
      0.87463E+03, 0.89215E+03, 0.90982E+03, 0.92765E+03, 0.94563E+03,
      0.96378E+03, 0.98209E+03, 0.10006E+04, 0.10192E+04, 0.10380E+04,
      0.10570E+04, 0.10761E+04, 0.10954E+04, 0.11149E+04, 0.11345E+04,
      0.11543E+04, 0.11743E+04, 0.11945E+04, 0.12148E+04, 0.12353E+04,
      0.12560E+04, 0.12768E+04, 0.12979E+04, 0.13191E+04, 0.13405E+04,
      0.13620E+04, 0.13838E+04, 0.14057E+04, 0.14278E+04, 0.14501E+04,
      0.14726E+04, 0.14952E+04, 0.15180E+04, 0.15410E+04, 0.15642E+04,
      0.15876E+04, 0.16112E+04, 0.16349E+04, 0.16589E+04, 0.16830E+04,
      0.17073E+04, 0.17318E+04, 0.17565E+04, 0.17814E+04, 0.18064E+04,
      0.18317E+04, 0.18572E+04, 0.18828E+04, 0.19086E+04, 0.19346E+04,
      0.19609E+04, 0.19873E+04, 0.20139E+04, 0.20406E+04, 0.20676E+04,
      0.20948E+04, 0.21222E+04, 0.21498E+04, 0.21775E+04, 0.22055E+04,
      0.22337E+04])


#  --------------- HСl 25: M = 15, I = 3 --------------------- not in TIPS-2011
M = 15
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- HСl 27: M = 15, I = 4 --------------------- not in TIPS-2011
M = 15
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- HBr 19: M = 16, I = 1 ---------------------
M = 16
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.42744E+02, 0.59373E+02, 0.76023E+02,
      0.92685E+02, 0.10936E+03, 0.12604E+03, 0.14272E+03, 0.15942E+03,
      0.17612E+03, 0.19282E+03, 0.20954E+03, 0.22626E+03, 0.24299E+03,
      0.25973E+03, 0.27648E+03, 0.29325E+03, 0.31004E+03, 0.32686E+03,
      0.34371E+03, 0.36060E+03, 0.37753E+03, 0.39451E+03, 0.41156E+03,
      0.42868E+03, 0.44587E+03, 0.46314E+03, 0.48051E+03, 0.49798E+03,
      0.51556E+03, 0.53325E+03, 0.55106E+03, 0.56900E+03, 0.58708E+03,
      0.60530E+03, 0.62367E+03, 0.64219E+03, 0.66088E+03, 0.67972E+03,
      0.69874E+03, 0.71793E+03, 0.73730E+03, 0.75685E+03, 0.77659E+03,
      0.79652E+03, 0.81664E+03, 0.83696E+03, 0.85748E+03, 0.87820E+03,
      0.89914E+03, 0.92028E+03, 0.94163E+03, 0.96319E+03, 0.98498E+03,
      0.10070E+04, 0.10292E+04, 0.10516E+04, 0.10743E+04, 0.10972E+04,
      0.11203E+04, 0.11437E+04, 0.11673E+04, 0.11911E+04, 0.12151E+04,
      0.12394E+04, 0.12640E+04, 0.12887E+04, 0.13137E+04, 0.13390E+04,
      0.13645E+04, 0.13902E+04, 0.14162E+04, 0.14424E+04, 0.14689E+04,
      0.14956E+04, 0.15226E+04, 0.15498E+04, 0.15773E+04, 0.16050E+04,
      0.16330E+04, 0.16612E+04, 0.16897E+04, 0.17185E+04, 0.17475E+04,
      0.17767E+04, 0.18062E+04, 0.18360E+04, 0.18660E+04, 0.18963E+04,
      0.19269E+04, 0.19577E+04, 0.19888E+04, 0.20202E+04, 0.20518E+04,
      0.20837E+04, 0.21158E+04, 0.21482E+04, 0.21809E+04, 0.22139E+04,
      0.22471E+04, 0.22806E+04, 0.23143E+04, 0.23484E+04, 0.23827E+04,
      0.24173E+04, 0.24521E+04, 0.24873E+04, 0.25227E+04, 0.25584E+04,
      0.25943E+04, 0.26306E+04, 0.26671E+04, 0.27039E+04, 0.27409E+04,
      0.27783E+04, 0.28159E+04, 0.28538E+04, 0.28920E+04, 0.29305E+04,
      0.29693E+04])


#  --------------- HBr 11: M = 16, I = 2 ---------------------
M = 16
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.42756E+02, 0.59390E+02, 0.76045E+02,
      0.92713E+02, 0.10939E+03, 0.12607E+03, 0.14277E+03, 0.15947E+03,
      0.17617E+03, 0.19288E+03, 0.20960E+03, 0.22633E+03, 0.24306E+03,
      0.25981E+03, 0.27656E+03, 0.29334E+03, 0.31014E+03, 0.32696E+03,
      0.34381E+03, 0.36071E+03, 0.37764E+03, 0.39464E+03, 0.41169E+03,
      0.42881E+03, 0.44601E+03, 0.46329E+03, 0.48066E+03, 0.49813E+03,
      0.51572E+03, 0.53341E+03, 0.55123E+03, 0.56918E+03, 0.58727E+03,
      0.60549E+03, 0.62387E+03, 0.64240E+03, 0.66109E+03, 0.67994E+03,
      0.69896E+03, 0.71816E+03, 0.73754E+03, 0.75710E+03, 0.77684E+03,
      0.79678E+03, 0.81691E+03, 0.83724E+03, 0.85776E+03, 0.87850E+03,
      0.89943E+03, 0.92058E+03, 0.94194E+03, 0.96352E+03, 0.98531E+03,
      0.10073E+04, 0.10295E+04, 0.10520E+04, 0.10747E+04, 0.10976E+04,
      0.11207E+04, 0.11441E+04, 0.11677E+04, 0.11915E+04, 0.12156E+04,
      0.12399E+04, 0.12644E+04, 0.12892E+04, 0.13142E+04, 0.13395E+04,
      0.13650E+04, 0.13907E+04, 0.14167E+04, 0.14429E+04, 0.14694E+04,
      0.14961E+04, 0.15231E+04, 0.15504E+04, 0.15778E+04, 0.16056E+04,
      0.16336E+04, 0.16618E+04, 0.16903E+04, 0.17191E+04, 0.17481E+04,
      0.17773E+04, 0.18069E+04, 0.18367E+04, 0.18667E+04, 0.18970E+04,
      0.19276E+04, 0.19584E+04, 0.19895E+04, 0.20209E+04, 0.20525E+04,
      0.20844E+04, 0.21166E+04, 0.21490E+04, 0.21817E+04, 0.22147E+04,
      0.22479E+04, 0.22814E+04, 0.23152E+04, 0.23492E+04, 0.23835E+04,
      0.24181E+04, 0.24530E+04, 0.24882E+04, 0.25236E+04, 0.25593E+04,
      0.25952E+04, 0.26315E+04, 0.26680E+04, 0.27048E+04, 0.27419E+04,
      0.27793E+04, 0.28169E+04, 0.28549E+04, 0.28931E+04, 0.29316E+04,
      0.29703E+04])


#  --------------- HBr 29: M = 16, I = 3 --------------------- not in TIPS-2011
M = 16
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- HBr 21: M = 16, I = 4 --------------------- not in TIPS-2011
M = 16
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- HI 17: M = 17, I = 1 ---------------------
M = 17
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.82031E+02, 0.11447E+03, 0.14694E+03,
      0.17943E+03, 0.21194E+03, 0.24445E+03, 0.27699E+03, 0.30953E+03,
      0.34209E+03, 0.37466E+03, 0.40725E+03, 0.43986E+03, 0.47249E+03,
      0.50517E+03, 0.53789E+03, 0.57068E+03, 0.60354E+03, 0.63650E+03,
      0.66957E+03, 0.70278E+03, 0.73614E+03, 0.76967E+03, 0.80340E+03,
      0.83735E+03, 0.87153E+03, 0.90596E+03, 0.94067E+03, 0.97566E+03,
      0.10110E+04, 0.10466E+04, 0.10826E+04, 0.11189E+04, 0.11555E+04,
      0.11926E+04, 0.12300E+04, 0.12679E+04, 0.13061E+04, 0.13448E+04,
      0.13839E+04, 0.14235E+04, 0.14635E+04, 0.15039E+04, 0.15448E+04,
      0.15862E+04, 0.16280E+04, 0.16704E+04, 0.17132E+04, 0.17565E+04,
      0.18003E+04, 0.18446E+04, 0.18894E+04, 0.19347E+04, 0.19806E+04,
      0.20269E+04, 0.20738E+04, 0.21212E+04, 0.21691E+04, 0.22176E+04,
      0.22666E+04, 0.23162E+04, 0.23662E+04, 0.24169E+04, 0.24680E+04,
      0.25198E+04, 0.25720E+04, 0.26249E+04, 0.26783E+04, 0.27322E+04,
      0.27867E+04, 0.28418E+04, 0.28975E+04, 0.29537E+04, 0.30105E+04,
      0.30678E+04, 0.31258E+04, 0.31843E+04, 0.32434E+04, 0.33031E+04,
      0.33633E+04, 0.34242E+04, 0.34856E+04, 0.35477E+04, 0.36103E+04,
      0.36735E+04, 0.37373E+04, 0.38018E+04, 0.38668E+04, 0.39324E+04,
      0.39986E+04, 0.40654E+04, 0.41329E+04, 0.42009E+04, 0.42696E+04,
      0.43388E+04, 0.44087E+04, 0.44792E+04, 0.45503E+04, 0.46221E+04,
      0.46944E+04, 0.47674E+04, 0.48410E+04, 0.49152E+04, 0.49901E+04,
      0.50656E+04, 0.51417E+04, 0.52185E+04, 0.52959E+04, 0.53739E+04,
      0.54526E+04, 0.55319E+04, 0.56118E+04, 0.56924E+04, 0.57736E+04,
      0.58555E+04, 0.59380E+04, 0.60212E+04, 0.61050E+04, 0.61895E+04,
      0.62746E+04])


#  --------------- HI 27: M = 17, I = 2 --------------------- not in TIPS-2011
M = 17
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- ClO 56: M = 18, I = 1 ---------------------
M = 18
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.53847E+03, 0.76580E+03, 0.10017E+04,
      0.12511E+04, 0.15168E+04, 0.18001E+04, 0.21014E+04, 0.24206E+04,
      0.27577E+04, 0.31127E+04, 0.34857E+04, 0.38765E+04, 0.42854E+04,
      0.47124E+04, 0.51575E+04, 0.56208E+04, 0.61025E+04, 0.66026E+04,
      0.71211E+04, 0.76582E+04, 0.82138E+04, 0.87882E+04, 0.93813E+04,
      0.99932E+04, 0.10624E+05, 0.11273E+05, 0.11942E+05, 0.12629E+05,
      0.13336E+05, 0.14061E+05, 0.14806E+05, 0.15570E+05, 0.16353E+05,
      0.17155E+05, 0.17976E+05, 0.18816E+05, 0.19676E+05, 0.20555E+05,
      0.21453E+05, 0.22371E+05, 0.23308E+05, 0.24264E+05, 0.25240E+05,
      0.26236E+05, 0.27250E+05, 0.28284E+05, 0.29338E+05, 0.30412E+05,
      0.31505E+05, 0.32617E+05, 0.33749E+05, 0.34901E+05, 0.36072E+05,
      0.37263E+05, 0.38474E+05, 0.39705E+05, 0.40955E+05, 0.42225E+05,
      0.43515E+05, 0.44825E+05, 0.46154E+05, 0.47504E+05, 0.48873E+05,
      0.50262E+05, 0.51672E+05, 0.53101E+05, 0.54549E+05, 0.56019E+05,
      0.57508E+05, 0.59017E+05, 0.60546E+05, 0.62095E+05, 0.63665E+05,
      0.65254E+05, 0.66864E+05, 0.68494E+05, 0.70144E+05, 0.71814E+05,
      0.73504E+05, 0.75215E+05, 0.76946E+05, 0.78698E+05, 0.80470E+05,
      0.82261E+05, 0.84074E+05, 0.85907E+05, 0.87760E+05, 0.89633E+05,
      0.91527E+05, 0.93442E+05, 0.95377E+05, 0.97333E+05, 0.99309E+05,
      0.10131E+06, 0.10332E+06, 0.10536E+06, 0.10742E+06, 0.10950E+06,
      0.11160E+06, 0.11372E+06, 0.11586E+06, 0.11802E+06, 0.12020E+06,
      0.12241E+06, 0.12463E+06, 0.12688E+06, 0.12914E+06, 0.13143E+06,
      0.13374E+06, 0.13607E+06, 0.13842E+06, 0.14079E+06, 0.14318E+06,
      0.14559E+06, 0.14802E+06, 0.15048E+06, 0.15295E+06, 0.15545E+06,
      0.15797E+06])


#  --------------- ClO 76: M = 18, I = 2 ---------------------
M = 18
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.54775E+03, 0.77899E+03, 0.10189E+04,
      0.12726E+04, 0.15430E+04, 0.18313E+04, 0.21378E+04, 0.24627E+04,
      0.28059E+04, 0.31674E+04, 0.35472E+04, 0.39454E+04, 0.43621E+04,
      0.47972E+04, 0.52508E+04, 0.57232E+04, 0.62143E+04, 0.67242E+04,
      0.72531E+04, 0.78010E+04, 0.83678E+04, 0.89537E+04, 0.95589E+04,
      0.10183E+05, 0.10827E+05, 0.11490E+05, 0.12172E+05, 0.12874E+05,
      0.13595E+05, 0.14335E+05, 0.15095E+05, 0.15875E+05, 0.16674E+05,
      0.17493E+05, 0.18332E+05, 0.19190E+05, 0.20068E+05, 0.20965E+05,
      0.21882E+05, 0.22820E+05, 0.23776E+05, 0.24753E+05, 0.25750E+05,
      0.26766E+05, 0.27803E+05, 0.28859E+05, 0.29935E+05, 0.31032E+05,
      0.32148E+05, 0.33284E+05, 0.34441E+05, 0.35617E+05, 0.36814E+05,
      0.38031E+05, 0.39267E+05, 0.40524E+05, 0.41802E+05, 0.43099E+05,
      0.44417E+05, 0.45755E+05, 0.47113E+05, 0.48492E+05, 0.49891E+05,
      0.51310E+05, 0.52750E+05, 0.54210E+05, 0.55690E+05, 0.57191E+05,
      0.58713E+05, 0.60255E+05, 0.61817E+05, 0.63400E+05, 0.65004E+05,
      0.66628E+05, 0.68272E+05, 0.69938E+05, 0.71624E+05, 0.73331E+05,
      0.75058E+05, 0.76806E+05, 0.78575E+05, 0.80364E+05, 0.82175E+05,
      0.84006E+05, 0.85858E+05, 0.87731E+05, 0.89625E+05, 0.91539E+05,
      0.93475E+05, 0.95431E+05, 0.97409E+05, 0.99407E+05, 0.10143E+06,
      0.10347E+06, 0.10553E+06, 0.10761E+06, 0.10972E+06, 0.11184E+06,
      0.11399E+06, 0.11615E+06, 0.11834E+06, 0.12055E+06, 0.12278E+06,
      0.12503E+06, 0.12731E+06, 0.12960E+06, 0.13192E+06, 0.13425E+06,
      0.13661E+06, 0.13899E+06, 0.14139E+06, 0.14382E+06, 0.14626E+06,
      0.14873E+06, 0.15121E+06, 0.15372E+06, 0.15625E+06, 0.15880E+06,
      0.16138E+06])


#  --------------- OCS 622: M = 19, I = 1 ---------------------
M = 19
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.20609E+03, 0.29199E+03, 0.37861E+03,
      0.46737E+03, 0.56024E+03, 0.65929E+03, 0.76649E+03, 0.88361E+03,
      0.10123E+04, 0.11541E+04, 0.13105E+04, 0.14829E+04, 0.16728E+04,
      0.18818E+04, 0.21113E+04, 0.23629E+04, 0.26383E+04, 0.29391E+04,
      0.32672E+04, 0.36245E+04, 0.40128E+04, 0.44343E+04, 0.48911E+04,
      0.53853E+04, 0.59193E+04, 0.64956E+04, 0.71166E+04, 0.77849E+04,
      0.85033E+04, 0.92746E+04, 0.10102E+05, 0.10988E+05, 0.11936E+05,
      0.12949E+05, 0.14032E+05, 0.15186E+05, 0.16416E+05, 0.17726E+05,
      0.19120E+05, 0.20601E+05, 0.22173E+05, 0.23842E+05, 0.25611E+05,
      0.27484E+05, 0.29468E+05, 0.31566E+05, 0.33783E+05, 0.36124E+05,
      0.38595E+05, 0.41202E+05, 0.43949E+05, 0.46842E+05, 0.49888E+05,
      0.53092E+05, 0.56460E+05, 0.59999E+05, 0.63716E+05, 0.67616E+05,
      0.71708E+05, 0.75997E+05, 0.80491E+05, 0.85197E+05, 0.90124E+05,
      0.95278E+05, 0.10067E+06, 0.10630E+06, 0.11219E+06, 0.11833E+06,
      0.12475E+06, 0.13144E+06, 0.13842E+06, 0.14570E+06, 0.15328E+06,
      0.16117E+06, 0.16940E+06, 0.17795E+06, 0.18686E+06, 0.19611E+06,
      0.20574E+06, 0.21574E+06, 0.22613E+06, 0.23692E+06, 0.24813E+06,
      0.25975E+06, 0.27182E+06, 0.28433E+06, 0.29730E+06, 0.31074E+06,
      0.32467E+06, 0.33909E+06, 0.35403E+06, 0.36950E+06, 0.38551E+06,
      0.40207E+06, 0.41920E+06, 0.43691E+06, 0.45522E+06, 0.47415E+06,
      0.49370E+06, 0.51390E+06, 0.53476E+06, 0.55629E+06, 0.57852E+06,
      0.60146E+06, 0.62513E+06, 0.64954E+06, 0.67471E+06, 0.70067E+06,
      0.72742E+06, 0.75499E+06, 0.78339E+06, 0.81265E+06, 0.84279E+06,
      0.87381E+06, 0.90576E+06, 0.93863E+06, 0.97246E+06, 0.10073E+07,
      0.10431E+07])


#  --------------- OCS 624: M = 19, I = 2 ---------------------
M = 19
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.21125E+03, 0.29930E+03, 0.38809E+03,
      0.47911E+03, 0.57437E+03, 0.67603E+03, 0.78610E+03, 0.90643E+03,
      0.10387E+04, 0.11846E+04, 0.13456E+04, 0.15231E+04, 0.17188E+04,
      0.19342E+04, 0.21709E+04, 0.24304E+04, 0.27145E+04, 0.30250E+04,
      0.33638E+04, 0.37328E+04, 0.41339E+04, 0.45694E+04, 0.50415E+04,
      0.55524E+04, 0.61045E+04, 0.67004E+04, 0.73427E+04, 0.80340E+04,
      0.87773E+04, 0.95755E+04, 0.10432E+05, 0.11349E+05, 0.12330E+05,
      0.13380E+05, 0.14500E+05, 0.15696E+05, 0.16970E+05, 0.18327E+05,
      0.19770E+05, 0.21305E+05, 0.22934E+05, 0.24663E+05, 0.26497E+05,
      0.28439E+05, 0.30495E+05, 0.32669E+05, 0.34968E+05, 0.37396E+05,
      0.39958E+05, 0.42661E+05, 0.45510E+05, 0.48511E+05, 0.51669E+05,
      0.54993E+05, 0.58487E+05, 0.62159E+05, 0.66014E+05, 0.70061E+05,
      0.74306E+05, 0.78757E+05, 0.83421E+05, 0.88305E+05, 0.93418E+05,
      0.98767E+05, 0.10436E+06, 0.11021E+06, 0.11632E+06, 0.12270E+06,
      0.12936E+06, 0.13631E+06, 0.14355E+06, 0.15111E+06, 0.15898E+06,
      0.16718E+06, 0.17572E+06, 0.18460E+06, 0.19385E+06, 0.20346E+06,
      0.21346E+06, 0.22385E+06, 0.23464E+06, 0.24585E+06, 0.25748E+06,
      0.26956E+06, 0.28209E+06, 0.29509E+06, 0.30856E+06, 0.32252E+06,
      0.33699E+06, 0.35198E+06, 0.36750E+06, 0.38357E+06, 0.40020E+06,
      0.41741E+06, 0.43521E+06, 0.45362E+06, 0.47264E+06, 0.49231E+06,
      0.51263E+06, 0.53362E+06, 0.55529E+06, 0.57768E+06, 0.60078E+06,
      0.62462E+06, 0.64922E+06, 0.67459E+06, 0.70075E+06, 0.72773E+06,
      0.75554E+06, 0.78419E+06, 0.81372E+06, 0.84413E+06, 0.87546E+06,
      0.90771E+06, 0.94092E+06, 0.97509E+06, 0.10103E+07, 0.10464E+07,
      0.10837E+07])


#  --------------- OCS 632: M = 19, I = 3 ---------------------
M = 19
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.41351E+03, 0.58591E+03, 0.76004E+03,
      0.93907E+03, 0.11273E+04, 0.13289E+04, 0.15481E+04, 0.17884E+04,
      0.20533E+04, 0.23459E+04, 0.26692E+04, 0.30264E+04, 0.34205E+04,
      0.38547E+04, 0.43323E+04, 0.48565E+04, 0.54309E+04, 0.60592E+04,
      0.67451E+04, 0.74928E+04, 0.83064E+04, 0.91903E+04, 0.10149E+05,
      0.11187E+05, 0.12310E+05, 0.13523E+05, 0.14831E+05, 0.16240E+05,
      0.17756E+05, 0.19384E+05, 0.21132E+05, 0.23005E+05, 0.25011E+05,
      0.27157E+05, 0.29449E+05, 0.31896E+05, 0.34506E+05, 0.37286E+05,
      0.40245E+05, 0.43392E+05, 0.46735E+05, 0.50284E+05, 0.54048E+05,
      0.58038E+05, 0.62263E+05, 0.66733E+05, 0.71460E+05, 0.76455E+05,
      0.81728E+05, 0.87292E+05, 0.93159E+05, 0.99341E+05, 0.10585E+06,
      0.11270E+06, 0.11991E+06, 0.12748E+06, 0.13543E+06, 0.14378E+06,
      0.15255E+06, 0.16174E+06, 0.17137E+06, 0.18146E+06, 0.19202E+06,
      0.20308E+06, 0.21465E+06, 0.22674E+06, 0.23937E+06, 0.25257E+06,
      0.26635E+06, 0.28073E+06, 0.29573E+06, 0.31137E+06, 0.32767E+06,
      0.34466E+06, 0.36235E+06, 0.38076E+06, 0.39992E+06, 0.41985E+06,
      0.44057E+06, 0.46211E+06, 0.48450E+06, 0.50775E+06, 0.53189E+06,
      0.55695E+06, 0.58295E+06, 0.60992E+06, 0.63789E+06, 0.66688E+06,
      0.69693E+06, 0.72806E+06, 0.76030E+06, 0.79368E+06, 0.82823E+06,
      0.86399E+06, 0.90097E+06, 0.93923E+06, 0.97878E+06, 0.10197E+07,
      0.10619E+07, 0.11056E+07, 0.11506E+07, 0.11972E+07, 0.12453E+07,
      0.12949E+07, 0.13460E+07, 0.13988E+07, 0.14533E+07, 0.15094E+07,
      0.15673E+07, 0.16270E+07, 0.16884E+07, 0.17518E+07, 0.18170E+07,
      0.18842E+07, 0.19533E+07, 0.20245E+07, 0.20978E+07, 0.21732E+07,
      0.22507E+07])


#  --------------- OCS 623: M = 19, I = 4 ---------------------
M = 19
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.83485E+03, 0.11828E+04, 0.15337E+04,
      0.18934E+04, 0.22697E+04, 0.26712E+04, 0.31059E+04, 0.35809E+04,
      0.41030E+04, 0.46785E+04, 0.53133E+04, 0.60135E+04, 0.67850E+04,
      0.76338E+04, 0.85663E+04, 0.95888E+04, 0.10708E+05, 0.11931E+05,
      0.13265E+05, 0.14718E+05, 0.16298E+05, 0.18012E+05, 0.19870E+05,
      0.21881E+05, 0.24054E+05, 0.26399E+05, 0.28926E+05, 0.31646E+05,
      0.34570E+05, 0.37710E+05, 0.41077E+05, 0.44685E+05, 0.48545E+05,
      0.52672E+05, 0.57078E+05, 0.61780E+05, 0.66790E+05, 0.72125E+05,
      0.77801E+05, 0.83833E+05, 0.90239E+05, 0.97036E+05, 0.10424E+06,
      0.11188E+06, 0.11996E+06, 0.12850E+06, 0.13754E+06, 0.14708E+06,
      0.15715E+06, 0.16777E+06, 0.17896E+06, 0.19076E+06, 0.20317E+06,
      0.21623E+06, 0.22996E+06, 0.24438E+06, 0.25953E+06, 0.27543E+06,
      0.29211E+06, 0.30959E+06, 0.32791E+06, 0.34710E+06, 0.36718E+06,
      0.38820E+06, 0.41017E+06, 0.43314E+06, 0.45713E+06, 0.48219E+06,
      0.50835E+06, 0.53564E+06, 0.56409E+06, 0.59376E+06, 0.62468E+06,
      0.65688E+06, 0.69041E+06, 0.72530E+06, 0.76161E+06, 0.79937E+06,
      0.83862E+06, 0.87941E+06, 0.92179E+06, 0.96581E+06, 0.10115E+07,
      0.10589E+07, 0.11081E+07, 0.11591E+07, 0.12120E+07, 0.12669E+07,
      0.13237E+07, 0.13825E+07, 0.14435E+07, 0.15066E+07, 0.15718E+07,
      0.16394E+07, 0.17093E+07, 0.17815E+07, 0.18562E+07, 0.19334E+07,
      0.20132E+07, 0.20956E+07, 0.21807E+07, 0.22685E+07, 0.23592E+07,
      0.24528E+07, 0.25494E+07, 0.26490E+07, 0.27517E+07, 0.28576E+07,
      0.29667E+07, 0.30792E+07, 0.31951E+07, 0.33145E+07, 0.34374E+07,
      0.35640E+07, 0.36943E+07, 0.38285E+07, 0.39665E+07, 0.41085E+07,
      0.42546E+07])


#  --------------- OCS 822: M = 19, I = 5 ---------------------
M = 19
I = 5
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.21967E+03, 0.31126E+03, 0.40370E+03,
      0.49862E+03, 0.59823E+03, 0.70481E+03, 0.82050E+03, 0.94724E+03,
      0.10868E+04, 0.12409E+04, 0.14112E+04, 0.15993E+04, 0.18067E+04,
      0.20353E+04, 0.22866E+04, 0.25624E+04, 0.28645E+04, 0.31950E+04,
      0.35558E+04, 0.39490E+04, 0.43767E+04, 0.48413E+04, 0.53452E+04,
      0.58909E+04, 0.64810E+04, 0.71182E+04, 0.78053E+04, 0.85454E+04,
      0.93413E+04, 0.10196E+05, 0.11114E+05, 0.12098E+05, 0.13151E+05,
      0.14277E+05, 0.15480E+05, 0.16764E+05, 0.18133E+05, 0.19592E+05,
      0.21144E+05, 0.22794E+05, 0.24548E+05, 0.26409E+05, 0.28383E+05,
      0.30475E+05, 0.32689E+05, 0.35033E+05, 0.37511E+05, 0.40128E+05,
      0.42892E+05, 0.45808E+05, 0.48882E+05, 0.52121E+05, 0.55532E+05,
      0.59121E+05, 0.62895E+05, 0.66861E+05, 0.71028E+05, 0.75402E+05,
      0.79991E+05, 0.84803E+05, 0.89847E+05, 0.95130E+05, 0.10066E+06,
      0.10645E+06, 0.11251E+06, 0.11883E+06, 0.12545E+06, 0.13236E+06,
      0.13957E+06, 0.14710E+06, 0.15495E+06, 0.16313E+06, 0.17166E+06,
      0.18055E+06, 0.18980E+06, 0.19944E+06, 0.20946E+06, 0.21989E+06,
      0.23073E+06, 0.24200E+06, 0.25371E+06, 0.26587E+06, 0.27850E+06,
      0.29161E+06, 0.30521E+06, 0.31931E+06, 0.33394E+06, 0.34910E+06,
      0.36482E+06, 0.38109E+06, 0.39795E+06, 0.41541E+06, 0.43348E+06,
      0.45217E+06, 0.47151E+06, 0.49151E+06, 0.51219E+06, 0.53356E+06,
      0.55565E+06, 0.57847E+06, 0.60204E+06, 0.62637E+06, 0.65149E+06,
      0.67742E+06, 0.70417E+06, 0.73176E+06, 0.76023E+06, 0.78957E+06,
      0.81982E+06, 0.85100E+06, 0.88313E+06, 0.91622E+06, 0.95031E+06,
      0.98541E+06, 0.10216E+07, 0.10587E+07, 0.10970E+07, 0.11364E+07,
      0.11769E+07])


#  --------------- H2CO 126: M = 20, I = 2 ---------------------
M = 20
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.25934E+03, 0.43623E+03, 0.64143E+03,
      0.87152E+03, 0.11241E+04, 0.13975E+04, 0.16906E+04, 0.20029E+04,
      0.23344E+04, 0.26857E+04, 0.30577E+04, 0.34518E+04, 0.38698E+04,
      0.43138E+04, 0.47860E+04, 0.52890E+04, 0.58256E+04, 0.63985E+04,
      0.70109E+04, 0.76660E+04, 0.83673E+04, 0.91184E+04, 0.99230E+04,
      0.10785E+05, 0.11710E+05, 0.12700E+05, 0.13762E+05, 0.14900E+05,
      0.16119E+05, 0.17425E+05, 0.18823E+05, 0.20320E+05, 0.21923E+05,
      0.23637E+05, 0.25471E+05, 0.27432E+05, 0.29527E+05, 0.31765E+05,
      0.34155E+05, 0.36706E+05, 0.39428E+05, 0.42330E+05, 0.45424E+05,
      0.48720E+05, 0.52231E+05, 0.55968E+05, 0.59945E+05, 0.64175E+05,
      0.68672E+05, 0.73450E+05, 0.78526E+05, 0.83915E+05, 0.89634E+05,
      0.95701E+05, 0.10213E+06, 0.10895E+06, 0.11618E+06, 0.12383E+06,
      0.13193E+06, 0.14049E+06, 0.14956E+06, 0.15914E+06, 0.16927E+06,
      0.17997E+06, 0.19127E+06, 0.20320E+06, 0.21578E+06, 0.22906E+06,
      0.24306E+06, 0.25782E+06, 0.27336E+06, 0.28974E+06, 0.30698E+06,
      0.32513E+06, 0.34422E+06, 0.36430E+06, 0.38542E+06, 0.40761E+06,
      0.43093E+06, 0.45542E+06, 0.48114E+06, 0.50813E+06, 0.53646E+06,
      0.56617E+06, 0.59733E+06, 0.63000E+06, 0.66423E+06, 0.70010E+06,
      0.73767E+06, 0.77701E+06, 0.81818E+06, 0.86127E+06, 0.90635E+06,
      0.95349E+06, 0.10028E+07, 0.10543E+07, 0.11082E+07, 0.11644E+07,
      0.12232E+07, 0.12845E+07, 0.13485E+07, 0.14154E+07, 0.14851E+07,
      0.15578E+07, 0.16337E+07, 0.17127E+07, 0.17952E+07, 0.18810E+07,
      0.19705E+07, 0.20637E+07, 0.21607E+07, 0.22617E+07, 0.23669E+07,
      0.24763E+07, 0.25901E+07, 0.27085E+07, 0.28316E+07, 0.29596E+07,
      0.30926E+07])


#  --------------- H2CO 136: M = 20, I = 2 ---------------------
M = 20
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.53173E+03, 0.89447E+03, 0.13153E+04,
      0.17871E+04, 0.23051E+04, 0.28658E+04, 0.34669E+04, 0.41073E+04,
      0.47872E+04, 0.55074E+04, 0.62702E+04, 0.70785E+04, 0.79357E+04,
      0.88462E+04, 0.98147E+04, 0.10846E+05, 0.11946E+05, 0.13121E+05,
      0.14377E+05, 0.15721E+05, 0.17159E+05, 0.18699E+05, 0.20349E+05,
      0.22118E+05, 0.24013E+05, 0.26045E+05, 0.28222E+05, 0.30555E+05,
      0.33055E+05, 0.35733E+05, 0.38601E+05, 0.41671E+05, 0.44958E+05,
      0.48474E+05, 0.52235E+05, 0.56255E+05, 0.60552E+05, 0.65142E+05,
      0.70043E+05, 0.75275E+05, 0.80856E+05, 0.86808E+05, 0.93152E+05,
      0.99913E+05, 0.10711E+06, 0.11478E+06, 0.12293E+06, 0.13161E+06,
      0.14083E+06, 0.15063E+06, 0.16104E+06, 0.17209E+06, 0.18382E+06,
      0.19626E+06, 0.20945E+06, 0.22343E+06, 0.23825E+06, 0.25394E+06,
      0.27054E+06, 0.28812E+06, 0.30671E+06, 0.32636E+06, 0.34713E+06,
      0.36907E+06, 0.39224E+06, 0.41671E+06, 0.44252E+06, 0.46975E+06,
      0.49845E+06, 0.52872E+06, 0.56060E+06, 0.59418E+06, 0.62954E+06,
      0.66676E+06, 0.70591E+06, 0.74710E+06, 0.79040E+06, 0.83591E+06,
      0.88373E+06, 0.93395E+06, 0.98669E+06, 0.10421E+07, 0.11001E+07,
      0.11611E+07, 0.12250E+07, 0.12920E+07, 0.13622E+07, 0.14357E+07,
      0.15128E+07, 0.15934E+07, 0.16779E+07, 0.17662E+07, 0.18587E+07,
      0.19554E+07, 0.20565E+07, 0.21621E+07, 0.22725E+07, 0.23879E+07,
      0.25084E+07, 0.26342E+07, 0.27655E+07, 0.29026E+07, 0.30456E+07,
      0.31947E+07, 0.33502E+07, 0.35124E+07, 0.36814E+07, 0.38575E+07,
      0.40410E+07, 0.42321E+07, 0.44311E+07, 0.46382E+07, 0.48538E+07,
      0.50782E+07, 0.53116E+07, 0.55544E+07, 0.58068E+07, 0.60693E+07,
      0.63421E+07])


#  --------------- H2CO 128: M = 20, I = 3 ---------------------
M = 20
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.27198E+03, 0.45755E+03, 0.67282E+03,
      0.91421E+03, 0.11792E+04, 0.14660E+04, 0.17735E+04, 0.21012E+04,
      0.24490E+04, 0.28175E+04, 0.32077E+04, 0.36212E+04, 0.40598E+04,
      0.45256E+04, 0.50211E+04, 0.55488E+04, 0.61116E+04, 0.67127E+04,
      0.73552E+04, 0.80426E+04, 0.87783E+04, 0.95663E+04, 0.10410E+05,
      0.11315E+05, 0.12285E+05, 0.13324E+05, 0.14438E+05, 0.15632E+05,
      0.16911E+05, 0.18281E+05, 0.19748E+05, 0.21319E+05, 0.23000E+05,
      0.24799E+05, 0.26723E+05, 0.28780E+05, 0.30978E+05, 0.33326E+05,
      0.35834E+05, 0.38510E+05, 0.41365E+05, 0.44410E+05, 0.47656E+05,
      0.51115E+05, 0.54798E+05, 0.58719E+05, 0.62891E+05, 0.67329E+05,
      0.72047E+05, 0.77060E+05, 0.82385E+05, 0.88039E+05, 0.94039E+05,
      0.10040E+06, 0.10715E+06, 0.11431E+06, 0.12189E+06, 0.12991E+06,
      0.13841E+06, 0.14740E+06, 0.15691E+06, 0.16696E+06, 0.17759E+06,
      0.18882E+06, 0.20067E+06, 0.21318E+06, 0.22639E+06, 0.24032E+06,
      0.25501E+06, 0.27049E+06, 0.28680E+06, 0.30398E+06, 0.32207E+06,
      0.34111E+06, 0.36114E+06, 0.38221E+06, 0.40436E+06, 0.42765E+06,
      0.45211E+06, 0.47781E+06, 0.50479E+06, 0.53311E+06, 0.56283E+06,
      0.59400E+06, 0.62669E+06, 0.66097E+06, 0.69688E+06, 0.73451E+06,
      0.77393E+06, 0.81520E+06, 0.85840E+06, 0.90360E+06, 0.95090E+06,
      0.10004E+07, 0.10521E+07, 0.11061E+07, 0.11626E+07, 0.12216E+07,
      0.12833E+07, 0.13476E+07, 0.14148E+07, 0.14849E+07, 0.15581E+07,
      0.16344E+07, 0.17140E+07, 0.17969E+07, 0.18834E+07, 0.19735E+07,
      0.20674E+07, 0.21651E+07, 0.22669E+07, 0.23729E+07, 0.24832E+07,
      0.25980E+07, 0.27174E+07, 0.28416E+07, 0.29708E+07, 0.31050E+07,
      0.32446E+07])


#  --------------- HOCl 165: M = 21, I = 1 ---------------------
M = 21
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.17041E+04, 0.28708E+04, 0.42250E+04,
      0.57456E+04, 0.74211E+04, 0.92470E+04, 0.11225E+05, 0.13359E+05,
      0.15657E+05, 0.18129E+05, 0.20785E+05, 0.23637E+05, 0.26696E+05,
      0.29974E+05, 0.33484E+05, 0.37239E+05, 0.41252E+05, 0.45536E+05,
      0.50105E+05, 0.54973E+05, 0.60152E+05, 0.65659E+05, 0.71507E+05,
      0.77711E+05, 0.84286E+05, 0.91249E+05, 0.98614E+05, 0.10640E+06,
      0.11462E+06, 0.12330E+06, 0.13244E+06, 0.14208E+06, 0.15222E+06,
      0.16289E+06, 0.17411E+06, 0.18589E+06, 0.19825E+06, 0.21123E+06,
      0.22483E+06, 0.23908E+06, 0.25400E+06, 0.26962E+06, 0.28596E+06,
      0.30303E+06, 0.32087E+06, 0.33950E+06, 0.35895E+06, 0.37923E+06,
      0.40038E+06, 0.42243E+06, 0.44539E+06, 0.46930E+06, 0.49419E+06,
      0.52008E+06, 0.54700E+06, 0.57498E+06, 0.60406E+06, 0.63426E+06,
      0.66562E+06, 0.69816E+06, 0.73192E+06, 0.76692E+06, 0.80322E+06,
      0.84083E+06, 0.87979E+06, 0.92014E+06, 0.96192E+06, 0.10052E+07,
      0.10499E+07, 0.10961E+07, 0.11440E+07, 0.11934E+07, 0.12445E+07,
      0.12973E+07, 0.13518E+07, 0.14081E+07, 0.14661E+07, 0.15261E+07,
      0.15879E+07, 0.16516E+07, 0.17174E+07, 0.17851E+07, 0.18550E+07,
      0.19269E+07, 0.20010E+07, 0.20773E+07, 0.21559E+07, 0.22367E+07,
      0.23200E+07, 0.24056E+07, 0.24936E+07, 0.25842E+07, 0.26773E+07,
      0.27730E+07, 0.28714E+07, 0.29724E+07, 0.30763E+07, 0.31829E+07,
      0.32924E+07, 0.34049E+07, 0.35203E+07, 0.36387E+07, 0.37603E+07,
      0.38850E+07, 0.40129E+07, 0.41441E+07, 0.42786E+07, 0.44165E+07,
      0.45579E+07, 0.47028E+07, 0.48512E+07, 0.50033E+07, 0.51592E+07,
      0.53187E+07, 0.54822E+07, 0.56495E+07, 0.58208E+07, 0.59961E+07,
      0.61755E+07])


#  --------------- HOCl 167: M = 21, I = 2 ---------------------
M = 21
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.17342E+04, 0.29215E+04, 0.42998E+04,
      0.58473E+04, 0.75524E+04, 0.94107E+04, 0.11423E+05, 0.13595E+05,
      0.15935E+05, 0.18450E+05, 0.21154E+05, 0.24056E+05, 0.27168E+05,
      0.30505E+05, 0.34077E+05, 0.37899E+05, 0.41983E+05, 0.46343E+05,
      0.50993E+05, 0.55947E+05, 0.61218E+05, 0.66822E+05, 0.72774E+05,
      0.79088E+05, 0.85780E+05, 0.92866E+05, 0.10036E+06, 0.10829E+06,
      0.11665E+06, 0.12548E+06, 0.13479E+06, 0.14460E+06, 0.15492E+06,
      0.16578E+06, 0.17719E+06, 0.18918E+06, 0.20177E+06, 0.21497E+06,
      0.22881E+06, 0.24332E+06, 0.25851E+06, 0.27440E+06, 0.29102E+06,
      0.30840E+06, 0.32656E+06, 0.34552E+06, 0.36531E+06, 0.38595E+06,
      0.40748E+06, 0.42991E+06, 0.45328E+06, 0.47762E+06, 0.50295E+06,
      0.52929E+06, 0.55669E+06, 0.58517E+06, 0.61477E+06, 0.64550E+06,
      0.67741E+06, 0.71053E+06, 0.74489E+06, 0.78052E+06, 0.81745E+06,
      0.85573E+06, 0.89539E+06, 0.93645E+06, 0.97897E+06, 0.10230E+07,
      0.10685E+07, 0.11156E+07, 0.11643E+07, 0.12146E+07, 0.12666E+07,
      0.13203E+07, 0.13757E+07, 0.14330E+07, 0.14921E+07, 0.15531E+07,
      0.16160E+07, 0.16809E+07, 0.17478E+07, 0.18168E+07, 0.18878E+07,
      0.19611E+07, 0.20365E+07, 0.21141E+07, 0.21941E+07, 0.22764E+07,
      0.23611E+07, 0.24482E+07, 0.25378E+07, 0.26300E+07, 0.27248E+07,
      0.28222E+07, 0.29223E+07, 0.30251E+07, 0.31308E+07, 0.32393E+07,
      0.33508E+07, 0.34652E+07, 0.35827E+07, 0.37032E+07, 0.38269E+07,
      0.39539E+07, 0.40840E+07, 0.42176E+07, 0.43545E+07, 0.44948E+07,
      0.46387E+07, 0.47861E+07, 0.49372E+07, 0.50920E+07, 0.52506E+07,
      0.54130E+07, 0.55793E+07, 0.57496E+07, 0.59239E+07, 0.61024E+07,
      0.62850E+07])


#  --------------- N2 44: M = 22, I = 1 ---------------------
M = 22
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.95487E+02, 0.13466E+03, 0.17386E+03,
      0.21307E+03, 0.25230E+03, 0.29154E+03, 0.33080E+03, 0.37008E+03,
      0.40937E+03, 0.44868E+03, 0.48800E+03, 0.52736E+03, 0.56674E+03,
      0.60616E+03, 0.64562E+03, 0.68515E+03, 0.72475E+03, 0.76445E+03,
      0.80426E+03, 0.84420E+03, 0.88430E+03, 0.92457E+03, 0.96505E+03,
      0.10057E+04, 0.10467E+04, 0.10879E+04, 0.11293E+04, 0.11711E+04,
      0.12132E+04, 0.12556E+04, 0.12984E+04, 0.13416E+04, 0.13851E+04,
      0.14291E+04, 0.14734E+04, 0.15182E+04, 0.15635E+04, 0.16091E+04,
      0.16553E+04, 0.17019E+04, 0.17490E+04, 0.17965E+04, 0.18446E+04,
      0.18932E+04, 0.19422E+04, 0.19918E+04, 0.20419E+04, 0.20926E+04,
      0.21437E+04, 0.21954E+04, 0.22477E+04, 0.23004E+04, 0.23538E+04,
      0.24077E+04, 0.24621E+04, 0.25171E+04, 0.25727E+04, 0.26288E+04,
      0.26856E+04, 0.27428E+04, 0.28007E+04, 0.28591E+04, 0.29181E+04,
      0.29777E+04, 0.30379E+04, 0.30986E+04, 0.31600E+04, 0.32219E+04,
      0.32844E+04, 0.33475E+04, 0.34112E+04, 0.34755E+04, 0.35404E+04,
      0.36059E+04, 0.36720E+04, 0.37387E+04, 0.38060E+04, 0.38739E+04,
      0.39424E+04, 0.40115E+04, 0.40812E+04, 0.41515E+04, 0.42224E+04,
      0.42939E+04, 0.43661E+04, 0.44388E+04, 0.45122E+04, 0.45861E+04,
      0.46607E+04, 0.47359E+04, 0.48117E+04, 0.48882E+04, 0.49652E+04,
      0.50428E+04, 0.51211E+04, 0.52000E+04, 0.52795E+04, 0.53596E+04,
      0.54404E+04, 0.55217E+04, 0.56037E+04, 0.56863E+04, 0.57695E+04,
      0.58533E+04, 0.59378E+04, 0.60229E+04, 0.61086E+04, 0.61950E+04,
      0.62819E+04, 0.63695E+04, 0.64577E+04, 0.65465E+04, 0.66360E+04,
      0.67261E+04, 0.68168E+04, 0.69081E+04, 0.70001E+04, 0.70927E+04,
      0.71859E+04])


#  --------------- N2 45: M = 22, I = 2 --------------------- not in TIPS-2011
M = 22
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- HCN 124: M = 23, I = 1 ---------------------
M = 23
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.17143E+03, 0.24209E+03, 0.31285E+03,
      0.38392E+03, 0.45582E+03, 0.52929E+03, 0.60515E+03, 0.68424E+03,
      0.76731E+03, 0.85505E+03, 0.94805E+03, 0.10468E+04, 0.11519E+04,
      0.12637E+04, 0.13826E+04, 0.15090E+04, 0.16435E+04, 0.17863E+04,
      0.19378E+04, 0.20985E+04, 0.22689E+04, 0.24492E+04, 0.26401E+04,
      0.28418E+04, 0.30550E+04, 0.32801E+04, 0.35176E+04, 0.37680E+04,
      0.40318E+04, 0.43097E+04, 0.46021E+04, 0.49097E+04, 0.52330E+04,
      0.55727E+04, 0.59294E+04, 0.63038E+04, 0.66964E+04, 0.71081E+04,
      0.75396E+04, 0.79915E+04, 0.84646E+04, 0.89596E+04, 0.94774E+04,
      0.10019E+05, 0.10585E+05, 0.11176E+05, 0.11793E+05, 0.12437E+05,
      0.13108E+05, 0.13809E+05, 0.14540E+05, 0.15301E+05, 0.16094E+05,
      0.16919E+05, 0.17779E+05, 0.18673E+05, 0.19603E+05, 0.20570E+05,
      0.21575E+05, 0.22619E+05, 0.23704E+05, 0.24831E+05, 0.26000E+05,
      0.27213E+05, 0.28472E+05, 0.29778E+05, 0.31131E+05, 0.32534E+05,
      0.33987E+05, 0.35493E+05, 0.37052E+05, 0.38666E+05, 0.40336E+05,
      0.42064E+05, 0.43852E+05, 0.45701E+05, 0.47612E+05, 0.49587E+05,
      0.51629E+05, 0.53738E+05, 0.55916E+05, 0.58165E+05, 0.60486E+05,
      0.62883E+05, 0.65355E+05, 0.67905E+05, 0.70536E+05, 0.73249E+05,
      0.76045E+05, 0.78927E+05, 0.81897E+05, 0.84957E+05, 0.88108E+05,
      0.91354E+05, 0.94696E+05, 0.98136E+05, 0.10168E+06, 0.10532E+06,
      0.10907E+06, 0.11292E+06, 0.11689E+06, 0.12096E+06, 0.12516E+06,
      0.12946E+06, 0.13389E+06, 0.13844E+06, 0.14311E+06, 0.14791E+06,
      0.15284E+06, 0.15790E+06, 0.16310E+06, 0.16843E+06, 0.17391E+06,
      0.17953E+06, 0.18529E+06, 0.19120E+06, 0.19726E+06, 0.20348E+06,
      0.20986E+06])


#  --------------- HCN 134: M = 23, I = 2 ---------------------
M = 23
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.35186E+03, 0.49693E+03, 0.64221E+03,
      0.78815E+03, 0.93585E+03, 0.10868E+04, 0.12428E+04, 0.14056E+04,
      0.15766E+04, 0.17574E+04, 0.19491E+04, 0.21528E+04, 0.23695E+04,
      0.26002E+04, 0.28457E+04, 0.31068E+04, 0.33845E+04, 0.36795E+04,
      0.39926E+04, 0.43249E+04, 0.46770E+04, 0.50500E+04, 0.54447E+04,
      0.58621E+04, 0.63032E+04, 0.67690E+04, 0.72606E+04, 0.77789E+04,
      0.83252E+04, 0.89005E+04, 0.95062E+04, 0.10143E+05, 0.10813E+05,
      0.11517E+05, 0.12256E+05, 0.13032E+05, 0.13846E+05, 0.14699E+05,
      0.15593E+05, 0.16530E+05, 0.17511E+05, 0.18538E+05, 0.19612E+05,
      0.20734E+05, 0.21908E+05, 0.23134E+05, 0.24414E+05, 0.25750E+05,
      0.27145E+05, 0.28599E+05, 0.30115E+05, 0.31694E+05, 0.33340E+05,
      0.35054E+05, 0.36838E+05, 0.38694E+05, 0.40625E+05, 0.42633E+05,
      0.44720E+05, 0.46889E+05, 0.49142E+05, 0.51481E+05, 0.53910E+05,
      0.56430E+05, 0.59045E+05, 0.61757E+05, 0.64568E+05, 0.67482E+05,
      0.70502E+05, 0.73630E+05, 0.76869E+05, 0.80223E+05, 0.83694E+05,
      0.87285E+05, 0.91000E+05, 0.94843E+05, 0.98815E+05, 0.10292E+06,
      0.10716E+06, 0.11155E+06, 0.11608E+06, 0.12075E+06, 0.12558E+06,
      0.13056E+06, 0.13570E+06, 0.14100E+06, 0.14647E+06, 0.15211E+06,
      0.15793E+06, 0.16392E+06, 0.17009E+06, 0.17646E+06, 0.18301E+06,
      0.18976E+06, 0.19671E+06, 0.20387E+06, 0.21123E+06, 0.21881E+06,
      0.22660E+06, 0.23462E+06, 0.24287E+06, 0.25135E+06, 0.26007E+06,
      0.26903E+06, 0.27824E+06, 0.28771E+06, 0.29743E+06, 0.30742E+06,
      0.31767E+06, 0.32820E+06, 0.33901E+06, 0.35011E+06, 0.36150E+06,
      0.37319E+06, 0.38518E+06, 0.39749E+06, 0.41010E+06, 0.42304E+06,
      0.43631E+06])


#  --------------- HCN 135: M = 23, I = 3 ---------------------
M = 23
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.11863E+03, 0.16755E+03, 0.21653E+03,
      0.26576E+03, 0.31559E+03, 0.36656E+03, 0.41926E+03, 0.47428E+03,
      0.53214E+03, 0.59333E+03, 0.65824E+03, 0.72727E+03, 0.80074E+03,
      0.87898E+03, 0.96227E+03, 0.10509E+04, 0.11452E+04, 0.12454E+04,
      0.13518E+04, 0.14647E+04, 0.15844E+04, 0.17112E+04, 0.18455E+04,
      0.19875E+04, 0.21377E+04, 0.22962E+04, 0.24636E+04, 0.26402E+04,
      0.28263E+04, 0.30224E+04, 0.32289E+04, 0.34461E+04, 0.36745E+04,
      0.39145E+04, 0.41667E+04, 0.44314E+04, 0.47092E+04, 0.50005E+04,
      0.53059E+04, 0.56259E+04, 0.59609E+04, 0.63116E+04, 0.66785E+04,
      0.70622E+04, 0.74633E+04, 0.78823E+04, 0.83200E+04, 0.87769E+04,
      0.92536E+04, 0.97509E+04, 0.10269E+05, 0.10810E+05, 0.11373E+05,
      0.11959E+05, 0.12570E+05, 0.13205E+05, 0.13866E+05, 0.14554E+05,
      0.15268E+05, 0.16011E+05, 0.16782E+05, 0.17583E+05, 0.18415E+05,
      0.19279E+05, 0.20174E+05, 0.21103E+05, 0.22067E+05, 0.23065E+05,
      0.24100E+05, 0.25172E+05, 0.26282E+05, 0.27432E+05, 0.28622E+05,
      0.29853E+05, 0.31127E+05, 0.32445E+05, 0.33807E+05, 0.35215E+05,
      0.36670E+05, 0.38174E+05, 0.39727E+05, 0.41330E+05, 0.42986E+05,
      0.44695E+05, 0.46459E+05, 0.48278E+05, 0.50155E+05, 0.52091E+05,
      0.54086E+05, 0.56143E+05, 0.58263E+05, 0.60447E+05, 0.62696E+05,
      0.65013E+05, 0.67399E+05, 0.69856E+05, 0.72384E+05, 0.74986E+05,
      0.77663E+05, 0.80416E+05, 0.83249E+05, 0.86161E+05, 0.89156E+05,
      0.92233E+05, 0.95397E+05, 0.98648E+05, 0.10199E+06, 0.10542E+06,
      0.10894E+06, 0.11256E+06, 0.11627E+06, 0.12009E+06, 0.12400E+06,
      0.12802E+06, 0.13214E+06, 0.13636E+06, 0.14070E+06, 0.14515E+06,
      0.14971E+06])


#  --------------- CH3Cl 215: M = 24, I = 1 ---------------------
M = 24
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.50529E+04, 0.85123E+04, 0.12528E+05,
      0.17036E+05, 0.22005E+05, 0.27429E+05, 0.33325E+05, 0.39734E+05,
      0.46713E+05, 0.54336E+05, 0.62690E+05, 0.71876E+05, 0.82006E+05,
      0.93204E+05, 0.10560E+06, 0.11936E+06, 0.13463E+06, 0.15158E+06,
      0.17043E+06, 0.19137E+06, 0.21464E+06, 0.24049E+06, 0.26920E+06,
      0.30107E+06, 0.33642E+06, 0.37563E+06, 0.41907E+06, 0.46719E+06,
      0.52045E+06, 0.57936E+06, 0.64448E+06, 0.71641E+06, 0.79582E+06,
      0.88341E+06, 0.97997E+06, 0.10863E+07, 0.12034E+07, 0.13323E+07,
      0.14739E+07, 0.16295E+07, 0.18003E+07, 0.19877E+07, 0.21932E+07,
      0.24183E+07, 0.26649E+07, 0.29346E+07, 0.32296E+07, 0.35519E+07,
      0.39039E+07, 0.42881E+07, 0.47072E+07, 0.51639E+07, 0.56615E+07,
      0.62032E+07, 0.67926E+07, 0.74335E+07, 0.81299E+07, 0.88862E+07,
      0.97071E+07, 0.10598E+08, 0.11563E+08, 0.12609E+08, 0.13742E+08,
      0.14968E+08, 0.16294E+08, 0.17728E+08, 0.19277E+08, 0.20950E+08,
      0.22756E+08, 0.24704E+08, 0.26805E+08, 0.29069E+08, 0.31507E+08,
      0.34132E+08, 0.36957E+08, 0.39995E+08, 0.43260E+08, 0.46769E+08,
      0.50538E+08, 0.54583E+08, 0.58923E+08, 0.63578E+08, 0.68568E+08,
      0.73914E+08, 0.79640E+08, 0.85770E+08, 0.92329E+08, 0.99345E+08,
      0.10685E+09, 0.11486E+09, 0.12342E+09, 0.13257E+09, 0.14233E+09,
      0.15274E+09, 0.16384E+09, 0.17568E+09, 0.18829E+09, 0.20173E+09,
      0.21604E+09, 0.23127E+09, 0.24748E+09, 0.26471E+09, 0.28304E+09,
      0.30252E+09, 0.32322E+09, 0.34520E+09, 0.36853E+09, 0.39330E+09,
      0.41958E+09, 0.44745E+09, 0.47701E+09, 0.50833E+09, 0.54151E+09,
      0.57667E+09, 0.61389E+09, 0.65329E+09, 0.69498E+09, 0.73909E+09,
      0.78573E+09])


#  --------------- CH3Cl 217: M = 24, I = 2 ---------------------
M = 24
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.51327E+04, 0.86469E+04, 0.12726E+05,
      0.17306E+05, 0.22354E+05, 0.27863E+05, 0.33853E+05, 0.40364E+05,
      0.47453E+05, 0.55197E+05, 0.63684E+05, 0.73016E+05, 0.83306E+05,
      0.94681E+05, 0.10728E+06, 0.12125E+06, 0.13676E+06, 0.15399E+06,
      0.17313E+06, 0.19441E+06, 0.21804E+06, 0.24430E+06, 0.27347E+06,
      0.30584E+06, 0.34176E+06, 0.38158E+06, 0.42572E+06, 0.47460E+06,
      0.52871E+06, 0.58855E+06, 0.65471E+06, 0.72778E+06, 0.80844E+06,
      0.89743E+06, 0.99552E+06, 0.11036E+07, 0.12225E+07, 0.13534E+07,
      0.14973E+07, 0.16553E+07, 0.18289E+07, 0.20193E+07, 0.22280E+07,
      0.24567E+07, 0.27072E+07, 0.29812E+07, 0.32808E+07, 0.36083E+07,
      0.39659E+07, 0.43562E+07, 0.47819E+07, 0.52459E+07, 0.57514E+07,
      0.63017E+07, 0.69005E+07, 0.75515E+07, 0.82590E+07, 0.90273E+07,
      0.98613E+07, 0.10766E+08, 0.11747E+08, 0.12809E+08, 0.13960E+08,
      0.15206E+08, 0.16553E+08, 0.18010E+08, 0.19584E+08, 0.21283E+08,
      0.23118E+08, 0.25097E+08, 0.27231E+08, 0.29531E+08, 0.32008E+08,
      0.34674E+08, 0.37544E+08, 0.40630E+08, 0.43948E+08, 0.47513E+08,
      0.51341E+08, 0.55451E+08, 0.59860E+08, 0.64589E+08, 0.69658E+08,
      0.75089E+08, 0.80906E+08, 0.87134E+08, 0.93797E+08, 0.10092E+09,
      0.10854E+09, 0.11669E+09, 0.12539E+09, 0.13467E+09, 0.14459E+09,
      0.15517E+09, 0.16645E+09, 0.17847E+09, 0.19129E+09, 0.20494E+09,
      0.21948E+09, 0.23495E+09, 0.25141E+09, 0.26893E+09, 0.28754E+09,
      0.30733E+09, 0.32836E+09, 0.35069E+09, 0.37440E+09, 0.39956E+09,
      0.42626E+09, 0.45457E+09, 0.48460E+09, 0.51642E+09, 0.55013E+09,
      0.58585E+09, 0.62366E+09, 0.66369E+09, 0.70605E+09, 0.75085E+09,
      0.79824E+09])


#  --------------- H2O2 1661: M = 25, I = 1 ---------------------
M = 25
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.62392E+03, 0.10958E+04, 0.16692E+04,
      0.23492E+04, 0.31427E+04, 0.40574E+04, 0.51014E+04, 0.62840E+04,
      0.76157E+04, 0.91085E+04, 0.10776E+05, 0.12633E+05, 0.14696E+05,
      0.16983E+05, 0.19515E+05, 0.22312E+05, 0.25396E+05, 0.28792E+05,
      0.32526E+05, 0.36625E+05, 0.41118E+05, 0.46036E+05, 0.51410E+05,
      0.57275E+05, 0.63667E+05, 0.70623E+05, 0.78185E+05, 0.86394E+05,
      0.95295E+05, 0.10493E+06, 0.11536E+06, 0.12662E+06, 0.13878E+06,
      0.15188E+06, 0.16600E+06, 0.18118E+06, 0.19750E+06, 0.21503E+06,
      0.23383E+06, 0.25398E+06, 0.27556E+06, 0.29864E+06, 0.32333E+06,
      0.34970E+06, 0.37784E+06, 0.40786E+06, 0.43985E+06, 0.47392E+06,
      0.51018E+06, 0.54874E+06, 0.58972E+06, 0.63324E+06, 0.67943E+06,
      0.72843E+06, 0.78037E+06, 0.83540E+06, 0.89366E+06, 0.95530E+06,
      0.10205E+07, 0.10894E+07, 0.11622E+07, 0.12391E+07, 0.13202E+07,
      0.14057E+07, 0.14959E+07, 0.15909E+07, 0.16910E+07, 0.17963E+07,
      0.19072E+07, 0.20237E+07, 0.21463E+07, 0.22750E+07, 0.24102E+07,
      0.25522E+07, 0.27012E+07, 0.28575E+07, 0.30213E+07, 0.31931E+07,
      0.33730E+07, 0.35615E+07, 0.37588E+07, 0.39653E+07, 0.41813E+07,
      0.44072E+07, 0.46433E+07, 0.48901E+07, 0.51479E+07, 0.54171E+07,
      0.56982E+07, 0.59915E+07, 0.62976E+07, 0.66167E+07, 0.69495E+07,
      0.72963E+07, 0.76577E+07, 0.80342E+07, 0.84262E+07, 0.88343E+07,
      0.92591E+07, 0.97011E+07, 0.10161E+08, 0.10639E+08, 0.11136E+08,
      0.11652E+08, 0.12189E+08, 0.12746E+08, 0.13325E+08, 0.13926E+08,
      0.14550E+08, 0.15198E+08, 0.15870E+08, 0.16566E+08, 0.17289E+08,
      0.18038E+08, 0.18814E+08, 0.19619E+08, 0.20452E+08, 0.21315E+08,
      0.22209E+08])


#  --------------- C2H2 1221: M = 26, I = 1 ---------------------
M = 26
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.71617E+02, 0.10121E+03, 0.13092E+03,
      0.16104E+03, 0.19218E+03, 0.22509E+03, 0.26062E+03, 0.29959E+03,
      0.34281E+03, 0.39103E+03, 0.44503E+03, 0.50558E+03, 0.57346E+03,
      0.64950E+03, 0.73457E+03, 0.82960E+03, 0.93557E+03, 0.10535E+04,
      0.11846E+04, 0.13301E+04, 0.14911E+04, 0.16692E+04, 0.18658E+04,
      0.20825E+04, 0.23211E+04, 0.25833E+04, 0.28711E+04, 0.31867E+04,
      0.35323E+04, 0.39102E+04, 0.43230E+04, 0.47735E+04, 0.52645E+04,
      0.57991E+04, 0.63807E+04, 0.70127E+04, 0.76988E+04, 0.84430E+04,
      0.92495E+04, 0.10123E+05, 0.11067E+05, 0.12088E+05, 0.13191E+05,
      0.14381E+05, 0.15664E+05, 0.17047E+05, 0.18536E+05, 0.20137E+05,
      0.21859E+05, 0.23710E+05, 0.25696E+05, 0.27827E+05, 0.30112E+05,
      0.32561E+05, 0.35183E+05, 0.37990E+05, 0.40991E+05, 0.44199E+05,
      0.47626E+05, 0.51285E+05, 0.55189E+05, 0.59353E+05, 0.63791E+05,
      0.68518E+05, 0.73551E+05, 0.78908E+05, 0.84604E+05, 0.90661E+05,
      0.97095E+05, 0.10393E+06, 0.11118E+06, 0.11888E+06, 0.12704E+06,
      0.13569E+06, 0.14486E+06, 0.15457E+06, 0.16485E+06, 0.17572E+06,
      0.18722E+06, 0.19938E+06, 0.21223E+06, 0.22581E+06, 0.24014E+06,
      0.25527E+06, 0.27123E+06, 0.28807E+06, 0.30582E+06, 0.32452E+06,
      0.34423E+06, 0.36498E+06, 0.38683E+06, 0.40982E+06, 0.43401E+06,
      0.45944E+06, 0.48618E+06, 0.51428E+06, 0.54380E+06, 0.57480E+06,
      0.60735E+06, 0.64151E+06, 0.67735E+06, 0.71494E+06, 0.75436E+06,
      0.79568E+06, 0.83898E+06, 0.88434E+06, 0.93184E+06, 0.98158E+06,
      0.10336E+07, 0.10881E+07, 0.11451E+07, 0.12047E+07, 0.12670E+07,
      0.13321E+07, 0.14002E+07, 0.14713E+07, 0.15455E+07, 0.16231E+07,
      0.17040E+07])


#  --------------- C2H2 1231: M = 26, I = 2 ---------------------
M = 26
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.28647E+03, 0.40486E+03, 0.52369E+03,
      0.64419E+03, 0.76874E+03, 0.90040E+03, 0.10425E+04, 0.11984E+04,
      0.13713E+04, 0.15642E+04, 0.17802E+04, 0.20223E+04, 0.22939E+04,
      0.25981E+04, 0.29384E+04, 0.33185E+04, 0.37424E+04, 0.42142E+04,
      0.47386E+04, 0.53203E+04, 0.59646E+04, 0.66769E+04, 0.74633E+04,
      0.83302E+04, 0.92845E+04, 0.10333E+05, 0.11485E+05, 0.12747E+05,
      0.14129E+05, 0.15641E+05, 0.17292E+05, 0.19094E+05, 0.21058E+05,
      0.23197E+05, 0.25523E+05, 0.28051E+05, 0.30796E+05, 0.33773E+05,
      0.36999E+05, 0.40492E+05, 0.44270E+05, 0.48354E+05, 0.52765E+05,
      0.57525E+05, 0.62658E+05, 0.68189E+05, 0.74144E+05, 0.80551E+05,
      0.87439E+05, 0.94840E+05, 0.10279E+06, 0.11131E+06, 0.12045E+06,
      0.13025E+06, 0.14074E+06, 0.15196E+06, 0.16397E+06, 0.17680E+06,
      0.19051E+06, 0.20514E+06, 0.22076E+06, 0.23742E+06, 0.25517E+06,
      0.27408E+06, 0.29421E+06, 0.31564E+06, 0.33842E+06, 0.36265E+06,
      0.38839E+06, 0.41572E+06, 0.44474E+06, 0.47553E+06, 0.50818E+06,
      0.54278E+06, 0.57945E+06, 0.61829E+06, 0.65940E+06, 0.70289E+06,
      0.74890E+06, 0.79754E+06, 0.84894E+06, 0.90324E+06, 0.96057E+06,
      0.10211E+07, 0.10849E+07, 0.11523E+07, 0.12233E+07, 0.12981E+07,
      0.13769E+07, 0.14599E+07, 0.15473E+07, 0.16393E+07, 0.17361E+07,
      0.18378E+07, 0.19447E+07, 0.20571E+07, 0.21752E+07, 0.22992E+07,
      0.24294E+07, 0.25661E+07, 0.27094E+07, 0.28598E+07, 0.30175E+07,
      0.31828E+07, 0.33560E+07, 0.35374E+07, 0.37274E+07, 0.39264E+07,
      0.41346E+07, 0.43525E+07, 0.45805E+07, 0.48188E+07, 0.50681E+07,
      0.53286E+07, 0.56008E+07, 0.58852E+07, 0.61823E+07, 0.64924E+07,
      0.68162E+07])


#  --------------- C2H2 1222: M = 26, I = 3 ---------------------
M = 26
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.24843E+03, 0.35373E+03, 0.45997E+03,
      0.56930E+03, 0.68497E+03, 0.81065E+03, 0.94999E+03, 0.11065E+04,
      0.12837E+04, 0.14848E+04, 0.17135E+04, 0.19731E+04, 0.22675E+04,
      0.26205E+04, 0.29999E+04, 0.34276E+04, 0.39086E+04, 0.44486E+04,
      0.50533E+04, 0.57294E+04, 0.64837E+04, 0.73237E+04, 0.82576E+04,
      0.92941E+04, 0.10443E+05, 0.11714E+05, 0.13117E+05, 0.14666E+05,
      0.16373E+05, 0.18250E+05, 0.20313E+05, 0.22578E+05, 0.25060E+05,
      0.27777E+05, 0.30750E+05, 0.33997E+05, 0.37541E+05, 0.41405E+05,
      0.45614E+05, 0.50192E+05, 0.55170E+05, 0.60576E+05, 0.66441E+05,
      0.72799E+05, 0.79686E+05, 0.87140E+05, 0.95199E+05, 0.10391E+06,
      0.11331E+06, 0.12345E+06, 0.13438E+06, 0.14615E+06, 0.15882E+06,
      0.17245E+06, 0.18710E+06, 0.20283E+06, 0.21972E+06, 0.23783E+06,
      0.25724E+06, 0.27804E+06, 0.30030E+06, 0.32411E+06, 0.34958E+06,
      0.37679E+06, 0.40585E+06, 0.43686E+06, 0.46994E+06, 0.50521E+06,
      0.54280E+06, 0.58282E+06, 0.62542E+06, 0.67074E+06, 0.71892E+06,
      0.77013E+06, 0.82453E+06, 0.88228E+06, 0.94356E+06, 0.10086E+07,
      0.10775E+07, 0.11505E+07, 0.12279E+07, 0.13098E+07, 0.13964E+07,
      0.14881E+07, 0.15850E+07, 0.16875E+07, 0.17957E+07, 0.19100E+07,
      0.20307E+07, 0.21580E+07, 0.22923E+07, 0.24339E+07, 0.25831E+07,
      0.27404E+07, 0.29060E+07, 0.30803E+07, 0.32638E+07, 0.34568E+07,
      0.36598E+07, 0.38733E+07, 0.40976E+07, 0.43332E+07, 0.45807E+07,
      0.48406E+07, 0.51133E+07, 0.53995E+07, 0.56997E+07, 0.60144E+07,
      0.63444E+07, 0.66901E+07, 0.70524E+07, 0.74317E+07, 0.78289E+07,
      0.82447E+07, 0.86797E+07, 0.91348E+07, 0.96108E+07, 0.10108E+08,
      0.10629E+08])


#  --------------- C2H6 1221: M = 27, I = 1 ---------------------
M = 27
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.47267E+04, 0.80011E+04, 0.11928E+05,
      0.16564E+05, 0.21985E+05, 0.28287E+05, 0.35590E+05, 0.44049E+05,
      0.53862E+05, 0.65277E+05, 0.78597E+05, 0.94191E+05, 0.11250E+06,
      0.13407E+06, 0.15952E+06, 0.18962E+06, 0.22526E+06, 0.26751E+06,
      0.31763E+06, 0.37714E+06, 0.44780E+06, 0.53174E+06, 0.63145E+06,
      0.74989E+06, 0.89056E+06, 0.10576E+07, 0.12559E+07, 0.14912E+07,
      0.17704E+07, 0.21013E+07, 0.24936E+07, 0.29582E+07, 0.35083E+07,
      0.41591E+07, 0.49286E+07, 0.58379E+07, 0.69116E+07, 0.81787E+07,
      0.96728E+07, 0.11433E+08, 0.13506E+08, 0.15945E+08, 0.18812E+08,
      0.22180E+08, 0.26134E+08, 0.30770E+08, 0.36204E+08, 0.42565E+08,
      0.50008E+08, 0.58708E+08, 0.68868E+08, 0.80725E+08, 0.94548E+08,
      0.11065E+09, 0.12940E+09, 0.15119E+09, 0.17652E+09, 0.20593E+09,
      0.24003E+09, 0.27956E+09, 0.32533E+09, 0.37829E+09, 0.43951E+09,
      0.51021E+09, 0.59180E+09, 0.68588E+09, 0.79427E+09, 0.91904E+09,
      0.10625E+10, 0.12275E+10, 0.14168E+10, 0.16341E+10, 0.18831E+10,
      0.21684E+10, 0.24949E+10, 0.28684E+10, 0.32951E+10, 0.37823E+10,
      0.43382E+10, 0.49719E+10, 0.56938E+10, 0.65156E+10, 0.74502E+10,
      0.85125E+10, 0.97190E+10, 0.11088E+11, 0.12641E+11, 0.14401E+11,
      0.16393E+11, 0.18648E+11, 0.21198E+11, 0.24079E+11, 0.27332E+11,
      0.31003E+11, 0.35142E+11, 0.39807E+11, 0.45060E+11, 0.50972E+11,
      0.57620E+11, 0.65091E+11, 0.73483E+11, 0.82902E+11, 0.93467E+11,
      0.10531E+12, 0.11858E+12, 0.13343E+12, 0.15005E+12, 0.16864E+12,
      0.18941E+12, 0.21260E+12, 0.23849E+12, 0.26737E+12, 0.29957E+12,
      0.33545E+12, 0.37541E+12, 0.41987E+12, 0.46934E+12, 0.52432E+12,
      0.58542E+12])


#  --------------- C2H6 1231: M = 27, I = 2 ---------------------
M = 27
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.24128E+04, 0.40845E+04, 0.60896E+04,
      0.84564E+04, 0.11224E+05, 0.14442E+05, 0.18170E+05, 0.22490E+05,
      0.27501E+05, 0.33329E+05, 0.40131E+05, 0.48094E+05, 0.57446E+05,
      0.68459E+05, 0.81458E+05, 0.96828E+05, 0.11503E+06, 0.13661E+06,
      0.16221E+06, 0.19260E+06, 0.22869E+06, 0.27156E+06, 0.32249E+06,
      0.38298E+06, 0.45483E+06, 0.54015E+06, 0.64144E+06, 0.76164E+06,
      0.90423E+06, 0.10733E+07, 0.12737E+07, 0.15110E+07, 0.17920E+07,
      0.21245E+07, 0.25176E+07, 0.29821E+07, 0.35307E+07, 0.41780E+07,
      0.49414E+07, 0.58408E+07, 0.68999E+07, 0.81461E+07, 0.96110E+07,
      0.11332E+08, 0.13352E+08, 0.15721E+08, 0.18497E+08, 0.21748E+08,
      0.25551E+08, 0.29997E+08, 0.35189E+08, 0.41248E+08, 0.48313E+08,
      0.56542E+08, 0.66122E+08, 0.77262E+08, 0.90206E+08, 0.10523E+09,
      0.12267E+09, 0.14287E+09, 0.16626E+09, 0.19333E+09, 0.22462E+09,
      0.26076E+09, 0.30247E+09, 0.35056E+09, 0.40596E+09, 0.46974E+09,
      0.54310E+09, 0.62740E+09, 0.72420E+09, 0.83527E+09, 0.96260E+09,
      0.11084E+10, 0.12754E+10, 0.14663E+10, 0.16845E+10, 0.19336E+10,
      0.22178E+10, 0.25418E+10, 0.29109E+10, 0.33311E+10, 0.38090E+10,
      0.43522E+10, 0.49691E+10, 0.56693E+10, 0.64633E+10, 0.73631E+10,
      0.83821E+10, 0.95352E+10, 0.10839E+11, 0.12312E+11, 0.13976E+11,
      0.15854E+11, 0.17971E+11, 0.20357E+11, 0.23043E+11, 0.26067E+11,
      0.29467E+11, 0.33289E+11, 0.37581E+11, 0.42399E+11, 0.47804E+11,
      0.53862E+11, 0.60649E+11, 0.68247E+11, 0.76750E+11, 0.86257E+11,
      0.96882E+11, 0.10875E+12, 0.12199E+12, 0.13677E+12, 0.15325E+12,
      0.17160E+12, 0.19204E+12, 0.21480E+12, 0.24010E+12, 0.26824E+12,
      0.29950E+12])


#  --------------- PH3 1111: M = 28, I = 1 ---------------------
M = 28
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.29652E+03, 0.49643E+03, 0.72810E+03,
      0.98777E+03, 0.12729E+04, 0.15820E+04, 0.19145E+04, 0.22708E+04,
      0.26520E+04, 0.30600E+04, 0.34971E+04, 0.39662E+04, 0.44702E+04,
      0.50126E+04, 0.55970E+04, 0.62273E+04, 0.69075E+04, 0.76421E+04,
      0.84357E+04, 0.92933E+04, 0.10220E+05, 0.11222E+05, 0.12304E+05,
      0.13473E+05, 0.14736E+05, 0.16099E+05, 0.17571E+05, 0.19160E+05,
      0.20873E+05, 0.22720E+05, 0.24710E+05, 0.26854E+05, 0.29162E+05,
      0.31646E+05, 0.34317E+05, 0.37188E+05, 0.40273E+05, 0.43585E+05,
      0.47140E+05, 0.50953E+05, 0.55040E+05, 0.59419E+05, 0.64108E+05,
      0.69127E+05, 0.74496E+05, 0.80236E+05, 0.86369E+05, 0.92918E+05,
      0.99909E+05, 0.10737E+06, 0.11532E+06, 0.12380E+06, 0.13282E+06,
      0.14244E+06, 0.15266E+06, 0.16354E+06, 0.17511E+06, 0.18739E+06,
      0.20044E+06, 0.21430E+06, 0.22900E+06, 0.24459E+06, 0.26111E+06,
      0.27862E+06, 0.29716E+06, 0.31680E+06, 0.33757E+06, 0.35954E+06,
      0.38277E+06, 0.40733E+06, 0.43326E+06, 0.46065E+06, 0.48955E+06,
      0.52005E+06, 0.55222E+06, 0.58614E+06, 0.62188E+06, 0.65953E+06,
      0.69917E+06, 0.74091E+06, 0.78483E+06, 0.83103E+06, 0.87960E+06,
      0.93067E+06, 0.98432E+06, 0.10407E+07, 0.10999E+07, 0.11620E+07,
      0.12272E+07, 0.12956E+07, 0.13673E+07, 0.14425E+07, 0.15212E+07,
      0.16038E+07, 0.16902E+07, 0.17808E+07, 0.18755E+07, 0.19746E+07,
      0.20784E+07, 0.21868E+07, 0.23002E+07, 0.24187E+07, 0.25425E+07,
      0.26719E+07, 0.28070E+07, 0.29480E+07, 0.30952E+07, 0.32488E+07,
      0.34091E+07, 0.35762E+07, 0.37504E+07, 0.39320E+07, 0.41213E+07,
      0.43185E+07, 0.45239E+07, 0.47378E+07, 0.49605E+07, 0.51923E+07,
      0.54335E+07])


#  --------------- COF2 269: M = 29, I = 1 ---------------------
M = 29
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.54999E+04, 0.92749E+04, 0.13668E+05,
      0.18643E+05, 0.24224E+05, 0.30487E+05, 0.37547E+05, 0.45543E+05,
      0.54639E+05, 0.65019E+05, 0.76886E+05, 0.90462E+05, 0.10600E+06,
      0.12377E+06, 0.14407E+06, 0.16723E+06, 0.19363E+06, 0.22367E+06,
      0.25780E+06, 0.29650E+06, 0.34031E+06, 0.38982E+06, 0.44568E+06,
      0.50859E+06, 0.57932E+06, 0.65872E+06, 0.74770E+06, 0.84724E+06,
      0.95844E+06, 0.10825E+07, 0.12205E+07, 0.13741E+07, 0.15446E+07,
      0.17336E+07, 0.19428E+07, 0.21742E+07, 0.24296E+07, 0.27113E+07,
      0.30214E+07, 0.33626E+07, 0.37373E+07, 0.41484E+07, 0.45989E+07,
      0.50921E+07, 0.56313E+07, 0.62202E+07, 0.68626E+07, 0.75628E+07,
      0.83251E+07, 0.91542E+07, 0.10055E+08, 0.11033E+08, 0.12093E+08,
      0.13242E+08, 0.14486E+08, 0.15831E+08, 0.17284E+08, 0.18853E+08,
      0.20546E+08, 0.22371E+08, 0.24335E+08, 0.26450E+08, 0.28724E+08,
      0.31167E+08, 0.33790E+08, 0.36605E+08, 0.39623E+08, 0.42856E+08,
      0.46318E+08, 0.50022E+08, 0.53983E+08, 0.58215E+08, 0.62735E+08,
      0.67558E+08, 0.72702E+08, 0.78186E+08, 0.84028E+08, 0.90247E+08,
      0.96865E+08, 0.10390E+09, 0.11138E+09, 0.11933E+09, 0.12777E+09,
      0.13672E+09, 0.14622E+09, 0.15629E+09, 0.16695E+09, 0.17825E+09,
      0.19021E+09, 0.20287E+09, 0.21625E+09, 0.23039E+09, 0.24534E+09,
      0.26113E+09, 0.27779E+09, 0.29538E+09, 0.31392E+09, 0.33348E+09,
      0.35409E+09, 0.37580E+09, 0.39867E+09, 0.42274E+09, 0.44806E+09,
      0.47470E+09, 0.50271E+09, 0.53215E+09, 0.56308E+09, 0.59557E+09,
      0.62968E+09, 0.66548E+09, 0.70304E+09, 0.74243E+09, 0.78374E+09,
      0.82703E+09, 0.87240E+09, 0.91992E+09, 0.96967E+09, 0.10218E+10,
      0.10763E+10])


#  --------------- COF2 369: M = 29, I = 2 --------------------- not in TIPS-2011
M = 29
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- SF6 29: M = 30, I = 1 ---------------------
M = 30
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.46373E+05, 0.78844E+05, 0.11939E+06,
      0.17183E+06, 0.24247E+06, 0.34059E+06, 0.47963E+06, 0.67906E+06,
      0.96713E+06, 0.13848E+07, 0.19911E+07, 0.28714E+07, 0.41481E+07,
      0.59956E+07, 0.86617E+07, 0.12496E+08, 0.17991E+08, 0.25832E+08,
      0.36971E+08, 0.52724E+08, 0.74895E+08, 0.10595E+09, 0.14923E+09,
      0.20925E+09, 0.29208E+09, 0.40582E+09, 0.56124E+09, 0.77259E+09,
      0.10586E+10, 0.14439E+10, 0.19605E+10, 0.26500E+10, 0.35662E+10,
      0.47781E+10, 0.63747E+10, 0.84689E+10, 0.11205E+11, 0.14765E+11,
      0.19378E+11, 0.25336E+11, 0.32998E+11, 0.42819E+11, 0.55361E+11,
      0.71323E+11, 0.91569E+11, 0.11716E+12, 0.14941E+12, 0.18992E+12,
      0.24065E+12, 0.30398E+12, 0.38283E+12, 0.48069E+12, 0.60182E+12,
      0.75136E+12, 0.93546E+12, 0.11615E+13, 0.14384E+13, 0.17767E+13,
      0.21890E+13, 0.26903E+13, 0.32984E+13, 0.40344E+13, 0.49232E+13,
      0.59942E+13, 0.72819E+13, 0.88272E+13, 0.10678E+14, 0.12889E+14,
      0.15527E+14, 0.18666E+14, 0.22397E+14, 0.26823E+14, 0.32062E+14,
      0.38253E+14, 0.45558E+14, 0.54161E+14, 0.64277E+14, 0.76153E+14,
      0.90072E+14, 0.10636E+15, 0.12539E+15, 0.14759E+15, 0.17345E+15,
      0.20354E+15, 0.23848E+15, 0.27902E+15, 0.32597E+15, 0.38028E+15,
      0.44303E+15, 0.51542E+15, 0.59883E+15, 0.69482E+15, 0.80516E+15,
      0.93182E+15, 0.10770E+16, 0.12434E+16, 0.14336E+16, 0.16511E+16,
      0.18992E+16, 0.21821E+16, 0.25043E+16, 0.28709E+16, 0.32875E+16,
      0.37604E+16, 0.42968E+16, 0.49046E+16, 0.55925E+16, 0.63704E+16,
      0.72492E+16, 0.82411E+16, 0.93596E+16, 0.10620E+17, 0.12038E+17,
      0.13633E+17, 0.15425E+17, 0.17438E+17, 0.19694E+17, 0.22224E+17,
      0.25057E+17])


#  --------------- H2S 121: M = 31, I = 1 ---------------------
M = 31
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.47192E+02, 0.78671E+02, 0.11510E+03,
      0.15589E+03, 0.20061E+03, 0.24896E+03, 0.30070E+03, 0.35571E+03,
      0.41386E+03, 0.47513E+03, 0.53951E+03, 0.60703E+03, 0.67772E+03,
      0.75167E+03, 0.82896E+03, 0.90969E+03, 0.99396E+03, 0.10819E+04,
      0.11736E+04, 0.12692E+04, 0.13689E+04, 0.14727E+04, 0.15809E+04,
      0.16937E+04, 0.18111E+04, 0.19333E+04, 0.20606E+04, 0.21931E+04,
      0.23309E+04, 0.24744E+04, 0.26236E+04, 0.27788E+04, 0.29403E+04,
      0.31081E+04, 0.32825E+04, 0.34638E+04, 0.36522E+04, 0.38478E+04,
      0.40510E+04, 0.42619E+04, 0.44808E+04, 0.47080E+04, 0.49437E+04,
      0.51881E+04, 0.54415E+04, 0.57042E+04, 0.59764E+04, 0.62584E+04,
      0.65505E+04, 0.68529E+04, 0.71660E+04, 0.74899E+04, 0.78251E+04,
      0.81718E+04, 0.85303E+04, 0.89008E+04, 0.92838E+04, 0.96795E+04,
      0.10088E+05, 0.10510E+05, 0.10946E+05, 0.11396E+05, 0.11860E+05,
      0.12339E+05, 0.12833E+05, 0.13342E+05, 0.13867E+05, 0.14408E+05,
      0.14966E+05, 0.15540E+05, 0.16132E+05, 0.16741E+05, 0.17368E+05,
      0.18013E+05, 0.18677E+05, 0.19361E+05, 0.20064E+05, 0.20786E+05,
      0.21529E+05, 0.22293E+05, 0.23078E+05, 0.23885E+05, 0.24714E+05,
      0.25565E+05, 0.26439E+05, 0.27337E+05, 0.28258E+05, 0.29204E+05,
      0.30174E+05, 0.31170E+05, 0.32191E+05, 0.33239E+05, 0.34313E+05,
      0.35414E+05, 0.36543E+05, 0.37700E+05, 0.38886E+05, 0.40101E+05,
      0.41346E+05, 0.42621E+05, 0.43926E+05, 0.45263E+05, 0.46631E+05,
      0.48033E+05, 0.49466E+05, 0.50934E+05, 0.52435E+05, 0.53971E+05,
      0.55542E+05, 0.57149E+05, 0.58792E+05, 0.60472E+05, 0.62190E+05,
      0.63946E+05, 0.65740E+05, 0.67574E+05, 0.69448E+05, 0.71362E+05,
      0.73318E+05])


#  --------------- H2S 141: M = 31, I = 2 ---------------------
M = 31
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.47310E+02, 0.78869E+02, 0.11539E+03,
      0.15628E+03, 0.20112E+03, 0.24959E+03, 0.30147E+03, 0.35661E+03,
      0.41491E+03, 0.47634E+03, 0.54088E+03, 0.60857E+03, 0.67945E+03,
      0.75359E+03, 0.83107E+03, 0.91201E+03, 0.99649E+03, 0.10846E+04,
      0.11766E+04, 0.12724E+04, 0.13724E+04, 0.14765E+04, 0.15850E+04,
      0.16980E+04, 0.18157E+04, 0.19382E+04, 0.20658E+04, 0.21987E+04,
      0.23369E+04, 0.24807E+04, 0.26303E+04, 0.27859E+04, 0.29478E+04,
      0.31160E+04, 0.32909E+04, 0.34727E+04, 0.36615E+04, 0.38576E+04,
      0.40613E+04, 0.42728E+04, 0.44923E+04, 0.47200E+04, 0.49563E+04,
      0.52013E+04, 0.54554E+04, 0.57188E+04, 0.59917E+04, 0.62744E+04,
      0.65672E+04, 0.68704E+04, 0.71843E+04, 0.75090E+04, 0.78451E+04,
      0.81926E+04, 0.85520E+04, 0.89236E+04, 0.93075E+04, 0.97042E+04,
      0.10114E+05, 0.10537E+05, 0.10974E+05, 0.11425E+05, 0.11890E+05,
      0.12370E+05, 0.12866E+05, 0.13376E+05, 0.13903E+05, 0.14445E+05,
      0.15004E+05, 0.15580E+05, 0.16173E+05, 0.16784E+05, 0.17412E+05,
      0.18059E+05, 0.18725E+05, 0.19410E+05, 0.20115E+05, 0.20839E+05,
      0.21584E+05, 0.22350E+05, 0.23137E+05, 0.23946E+05, 0.24777E+05,
      0.25630E+05, 0.26507E+05, 0.27407E+05, 0.28330E+05, 0.29278E+05,
      0.30251E+05, 0.31249E+05, 0.32273E+05, 0.33324E+05, 0.34401E+05,
      0.35505E+05, 0.36637E+05, 0.37797E+05, 0.38985E+05, 0.40204E+05,
      0.41451E+05, 0.42729E+05, 0.44038E+05, 0.45379E+05, 0.46751E+05,
      0.48155E+05, 0.49593E+05, 0.51064E+05, 0.52569E+05, 0.54109E+05,
      0.55684E+05, 0.57295E+05, 0.58943E+05, 0.60627E+05, 0.62349E+05,
      0.64109E+05, 0.65908E+05, 0.67747E+05, 0.69625E+05, 0.71544E+05,
      0.73505E+05])


#  --------------- H2S 131: M = 30, I = 3 ---------------------
M = 31
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.18901E+03, 0.31509E+03, 0.46102E+03,
      0.62437E+03, 0.80349E+03, 0.99713E+03, 0.12044E+04, 0.14247E+04,
      0.16576E+04, 0.19030E+04, 0.21609E+04, 0.24313E+04, 0.27145E+04,
      0.30106E+04, 0.33202E+04, 0.36436E+04, 0.39811E+04, 0.43332E+04,
      0.47005E+04, 0.50835E+04, 0.54827E+04, 0.58987E+04, 0.63321E+04,
      0.67836E+04, 0.72538E+04, 0.77434E+04, 0.82532E+04, 0.87838E+04,
      0.93360E+04, 0.99106E+04, 0.10508E+05, 0.11130E+05, 0.11777E+05,
      0.12449E+05, 0.13147E+05, 0.13874E+05, 0.14628E+05, 0.15412E+05,
      0.16225E+05, 0.17070E+05, 0.17947E+05, 0.18857E+05, 0.19801E+05,
      0.20780E+05, 0.21795E+05, 0.22847E+05, 0.23937E+05, 0.25067E+05,
      0.26236E+05, 0.27448E+05, 0.28702E+05, 0.29999E+05, 0.31342E+05,
      0.32730E+05, 0.34166E+05, 0.35650E+05, 0.37184E+05, 0.38769E+05,
      0.40406E+05, 0.42097E+05, 0.43842E+05, 0.45644E+05, 0.47503E+05,
      0.49421E+05, 0.51399E+05, 0.53439E+05, 0.55542E+05, 0.57709E+05,
      0.59942E+05, 0.62242E+05, 0.64611E+05, 0.67051E+05, 0.69563E+05,
      0.72148E+05, 0.74808E+05, 0.77545E+05, 0.80360E+05, 0.83255E+05,
      0.86232E+05, 0.89291E+05, 0.92435E+05, 0.95667E+05, 0.98986E+05,
      0.10240E+06, 0.10590E+06, 0.10949E+06, 0.11318E+06, 0.11697E+06,
      0.12086E+06, 0.12484E+06, 0.12893E+06, 0.13313E+06, 0.13743E+06,
      0.14184E+06, 0.14637E+06, 0.15100E+06, 0.15575E+06, 0.16062E+06,
      0.16560E+06, 0.17071E+06, 0.17594E+06, 0.18129E+06, 0.18677E+06,
      0.19238E+06, 0.19813E+06, 0.20400E+06, 0.21002E+06, 0.21617E+06,
      0.22246E+06, 0.22890E+06, 0.23548E+06, 0.24221E+06, 0.24909E+06,
      0.25612E+06, 0.26331E+06, 0.27065E+06, 0.27816E+06, 0.28583E+06,
      0.29366E+06])


#  --------------- HCOOH 126: M = 32, I = 1 ---------------------
M = 32
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.31899E+04, 0.53773E+04, 0.79205E+04,
      0.10792E+05, 0.13993E+05, 0.17550E+05, 0.21509E+05, 0.25930E+05,
      0.30885E+05, 0.36460E+05, 0.42750E+05, 0.49864E+05, 0.57926E+05,
      0.67071E+05, 0.77453E+05, 0.89243E+05, 0.10263E+06, 0.11783E+06,
      0.13507E+06, 0.15462E+06, 0.17676E+06, 0.20183E+06, 0.23018E+06,
      0.26221E+06, 0.29836E+06, 0.33911E+06, 0.38501E+06, 0.43664E+06,
      0.49467E+06, 0.55981E+06, 0.63286E+06, 0.71470E+06, 0.80628E+06,
      0.90865E+06, 0.10230E+07, 0.11505E+07, 0.12927E+07, 0.14509E+07,
      0.16269E+07, 0.18225E+07, 0.20396E+07, 0.22804E+07, 0.25472E+07,
      0.28425E+07, 0.31692E+07, 0.35301E+07, 0.39285E+07, 0.43681E+07,
      0.48525E+07, 0.53858E+07, 0.59727E+07, 0.66178E+07, 0.73265E+07,
      0.81042E+07, 0.89571E+07, 0.98918E+07, 0.10915E+08, 0.12035E+08,
      0.13259E+08, 0.14597E+08, 0.16057E+08, 0.17650E+08, 0.19387E+08,
      0.21279E+08, 0.23339E+08, 0.25579E+08, 0.28016E+08, 0.30663E+08,
      0.33536E+08, 0.36655E+08, 0.40037E+08, 0.43701E+08, 0.47671E+08,
      0.51967E+08, 0.56614E+08, 0.61639E+08, 0.67068E+08, 0.72930E+08,
      0.79257E+08, 0.86082E+08, 0.93439E+08, 0.10137E+09, 0.10990E+09,
      0.11909E+09, 0.12898E+09, 0.13960E+09, 0.15102E+09, 0.16329E+09,
      0.17646E+09, 0.19059E+09, 0.20575E+09, 0.22200E+09, 0.23941E+09,
      0.25806E+09, 0.27802E+09, 0.29938E+09, 0.32223E+09, 0.34666E+09,
      0.37276E+09, 0.40064E+09, 0.43041E+09, 0.46218E+09, 0.49607E+09,
      0.53221E+09, 0.57074E+09, 0.61179E+09, 0.65551E+09, 0.70206E+09,
      0.75159E+09, 0.80430E+09, 0.86034E+09, 0.91992E+09, 0.98324E+09,
      0.10505E+10, 0.11219E+10, 0.11977E+10, 0.12782E+10, 0.13635E+10,
      0.14540E+10])


#  --------------- HO2 166: M = 33, I = 1 ---------------------
M = 33
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.39277E+03, 0.66062E+03, 0.97123E+03,
      0.13194E+04, 0.17014E+04, 0.21148E+04, 0.25578E+04, 0.30296E+04,
      0.35297E+04, 0.40585E+04, 0.46167E+04, 0.52055E+04, 0.58264E+04,
      0.64809E+04, 0.71707E+04, 0.78978E+04, 0.86641E+04, 0.94715E+04,
      0.10322E+05, 0.11218E+05, 0.12161E+05, 0.13154E+05, 0.14198E+05,
      0.15296E+05, 0.16449E+05, 0.17661E+05, 0.18933E+05, 0.20267E+05,
      0.21666E+05, 0.23133E+05, 0.24669E+05, 0.26277E+05, 0.27960E+05,
      0.29720E+05, 0.31560E+05, 0.33482E+05, 0.35489E+05, 0.37584E+05,
      0.39769E+05, 0.42048E+05, 0.44423E+05, 0.46898E+05, 0.49475E+05,
      0.52157E+05, 0.54948E+05, 0.57850E+05, 0.60868E+05, 0.64003E+05,
      0.67261E+05, 0.70643E+05, 0.74154E+05, 0.77797E+05, 0.81575E+05,
      0.85492E+05, 0.89553E+05, 0.93760E+05, 0.98118E+05, 0.10263E+06,
      0.10730E+06, 0.11213E+06, 0.11713E+06, 0.12230E+06, 0.12765E+06,
      0.13317E+06, 0.13888E+06, 0.14478E+06, 0.15086E+06, 0.15715E+06,
      0.16363E+06, 0.17032E+06, 0.17723E+06, 0.18434E+06, 0.19168E+06,
      0.19924E+06, 0.20704E+06, 0.21506E+06, 0.22333E+06, 0.23185E+06,
      0.24061E+06, 0.24963E+06, 0.25891E+06, 0.26846E+06, 0.27828E+06,
      0.28838E+06, 0.29876E+06, 0.30943E+06, 0.32039E+06, 0.33166E+06,
      0.34323E+06, 0.35512E+06, 0.36732E+06, 0.37985E+06, 0.39271E+06,
      0.40590E+06, 0.41944E+06, 0.43333E+06, 0.44758E+06, 0.46219E+06,
      0.47717E+06, 0.49252E+06, 0.50826E+06, 0.52439E+06, 0.54091E+06,
      0.55784E+06, 0.57518E+06, 0.59293E+06, 0.61112E+06, 0.62973E+06,
      0.64878E+06, 0.66828E+06, 0.68824E+06, 0.70866E+06, 0.72955E+06,
      0.75091E+06, 0.77276E+06, 0.79511E+06, 0.81795E+06, 0.84131E+06,
      0.86518E+06])


#  --------------- O 6: M = 34, I = 1 --------------------- not in TIPS-2011
M = 34
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- ClONO2 5646: M = 35, I = 1 ---------------------
M = 35
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.11444E+06, 0.21121E+06, 0.34858E+06,
      0.53934E+06, 0.80041E+06, 0.11539E+07, 0.16286E+07, 0.22614E+07,
      0.30992E+07, 0.42015E+07, 0.56426E+07, 0.75152E+07, 0.99344E+07,
      0.13042E+08, 0.17012E+08, 0.22058E+08, 0.28437E+08, 0.36463E+08,
      0.46514E+08, 0.59042E+08, 0.74589E+08, 0.93801E+08, 0.11744E+09,
      0.14643E+09, 0.18181E+09, 0.22486E+09, 0.27705E+09, 0.34009E+09,
      0.41598E+09, 0.50705E+09, 0.61599E+09, 0.74590E+09, 0.90037E+09,
      0.10835E+10, 0.13001E+10, 0.15554E+10, 0.18556E+10, 0.22079E+10,
      0.26200E+10, 0.31012E+10, 0.36615E+10, 0.43126E+10, 0.50675E+10,
      0.59409E+10, 0.69492E+10, 0.81110E+10, 0.94469E+10, 0.10980E+11,
      0.12736E+11, 0.14745E+11, 0.17037E+11, 0.19649E+11, 0.22620E+11,
      0.25994E+11, 0.29819E+11, 0.34150E+11, 0.39044E+11, 0.44568E+11,
      0.50794E+11, 0.57799E+11, 0.65672E+11, 0.74506E+11, 0.84408E+11,
      0.95490E+11, 0.10788E+12, 0.12171E+12, 0.13713E+12, 0.15431E+12,
      0.17342E+12, 0.19465E+12, 0.21822E+12, 0.24435E+12, 0.27329E+12,
      0.30530E+12, 0.34069E+12, 0.37976E+12, 0.42286E+12, 0.47034E+12,
      0.52262E+12, 0.58012E+12, 0.64330E+12, 0.71267E+12, 0.78875E+12,
      0.87214E+12, 0.96344E+12, 0.10633E+13, 0.11725E+13, 0.12918E+13,
      0.14220E+13, 0.15640E+13, 0.17188E+13, 0.18873E+13, 0.20706E+13,
      0.22700E+13, 0.24866E+13, 0.27218E+13, 0.29771E+13, 0.32538E+13,
      0.35537E+13, 0.38784E+13, 0.42299E+13, 0.46100E+13, 0.50208E+13,
      0.54645E+13, 0.59435E+13, 0.64603E+13, 0.70175E+13, 0.76180E+13,
      0.82647E+13, 0.89608E+13, 0.97097E+13, 0.10515E+14, 0.11380E+14,
      0.12310E+14, 0.13307E+14, 0.14378E+14, 0.15526E+14, 0.16756E+14,
      0.18075E+14])


#  --------------- ClONO2 7646: M = 35, I = 2 ---------------------
M = 35
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.11735E+06, 0.21659E+06, 0.35745E+06,
      0.55307E+06, 0.82078E+06, 0.11833E+07, 0.16700E+07, 0.23189E+07,
      0.31781E+07, 0.43084E+07, 0.57862E+07, 0.77065E+07, 0.10187E+08,
      0.13374E+08, 0.17445E+08, 0.22619E+08, 0.29161E+08, 0.37391E+08,
      0.47698E+08, 0.60545E+08, 0.76487E+08, 0.96188E+08, 0.12043E+09,
      0.15015E+09, 0.18644E+09, 0.23059E+09, 0.28410E+09, 0.34874E+09,
      0.42657E+09, 0.51995E+09, 0.63167E+09, 0.76489E+09, 0.92329E+09,
      0.11111E+10, 0.13331E+10, 0.15950E+10, 0.19029E+10, 0.22641E+10,
      0.26867E+10, 0.31801E+10, 0.37547E+10, 0.44224E+10, 0.51965E+10,
      0.60921E+10, 0.71261E+10, 0.83174E+10, 0.96873E+10, 0.11260E+11,
      0.13061E+11, 0.15120E+11, 0.17471E+11, 0.20149E+11, 0.23196E+11,
      0.26656E+11, 0.30578E+11, 0.35019E+11, 0.40038E+11, 0.45703E+11,
      0.52087E+11, 0.59270E+11, 0.67343E+11, 0.76403E+11, 0.86556E+11,
      0.97921E+11, 0.11062E+12, 0.12481E+12, 0.14062E+12, 0.15824E+12,
      0.17783E+12, 0.19961E+12, 0.22377E+12, 0.25057E+12, 0.28024E+12,
      0.31308E+12, 0.34936E+12, 0.38943E+12, 0.43362E+12, 0.48232E+12,
      0.53593E+12, 0.59489E+12, 0.65968E+12, 0.73081E+12, 0.80883E+12,
      0.89434E+12, 0.98797E+12, 0.10904E+13, 0.12024E+13, 0.13247E+13,
      0.14582E+13, 0.16038E+13, 0.17625E+13, 0.19353E+13, 0.21233E+13,
      0.23278E+13, 0.25499E+13, 0.27911E+13, 0.30528E+13, 0.33366E+13,
      0.36442E+13, 0.39772E+13, 0.43376E+13, 0.47273E+13, 0.51486E+13,
      0.56036E+13, 0.60948E+13, 0.66248E+13, 0.71962E+13, 0.78119E+13,
      0.84751E+13, 0.91889E+13, 0.99569E+13, 0.10783E+14, 0.11670E+14,
      0.12623E+14, 0.13646E+14, 0.14744E+14, 0.15921E+14, 0.17183E+14,
      0.18535E+14])



#  --------------- NOp 46: M = 36, I = 1 ---------------------
M = 36
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.63956E+02, 0.90185E+02, 0.11642E+03,
      0.14265E+03, 0.16889E+03, 0.19513E+03, 0.22138E+03, 0.24763E+03,
      0.27388E+03, 0.30013E+03, 0.32639E+03, 0.35266E+03, 0.37894E+03,
      0.40523E+03, 0.43155E+03, 0.45790E+03, 0.48429E+03, 0.51074E+03,
      0.53725E+03, 0.56383E+03, 0.59052E+03, 0.61731E+03, 0.64422E+03,
      0.67127E+03, 0.69846E+03, 0.72582E+03, 0.75335E+03, 0.78108E+03,
      0.80901E+03, 0.83715E+03, 0.86552E+03, 0.89413E+03, 0.92298E+03,
      0.95208E+03, 0.98144E+03, 0.10111E+04, 0.10410E+04, 0.10712E+04,
      0.11017E+04, 0.11325E+04, 0.11636E+04, 0.11950E+04, 0.12268E+04,
      0.12588E+04, 0.12912E+04, 0.13239E+04, 0.13570E+04, 0.13903E+04,
      0.14241E+04, 0.14581E+04, 0.14926E+04, 0.15273E+04, 0.15624E+04,
      0.15979E+04, 0.16337E+04, 0.16699E+04, 0.17065E+04, 0.17434E+04,
      0.17806E+04, 0.18183E+04, 0.18563E+04, 0.18947E+04, 0.19334E+04,
      0.19725E+04, 0.20120E+04, 0.20519E+04, 0.20921E+04, 0.21327E+04,
      0.21737E+04, 0.22151E+04, 0.22568E+04, 0.22990E+04, 0.23415E+04,
      0.23844E+04, 0.24276E+04, 0.24713E+04, 0.25153E+04, 0.25598E+04,
      0.26046E+04, 0.26497E+04, 0.26953E+04, 0.27413E+04, 0.27876E+04,
      0.28343E+04, 0.28815E+04, 0.29290E+04, 0.29769E+04, 0.30251E+04,
      0.30738E+04, 0.31229E+04, 0.31723E+04, 0.32222E+04, 0.32724E+04,
      0.33230E+04, 0.33740E+04, 0.34254E+04, 0.34772E+04, 0.35294E+04,
      0.35819E+04, 0.36349E+04, 0.36883E+04, 0.37420E+04, 0.37961E+04,
      0.38507E+04, 0.39056E+04, 0.39609E+04, 0.40166E+04, 0.40727E+04,
      0.41292E+04, 0.41861E+04, 0.42434E+04, 0.43010E+04, 0.43591E+04,
      0.44176E+04, 0.44764E+04, 0.45357E+04, 0.45953E+04, 0.46554E+04,
      0.47158E+04])


#  --------------- HOBr 169: M = 37, I = 1 ---------------------
M = 37
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.24445E+04, 0.41206E+04, 0.60683E+04,
      0.82610E+04, 0.10689E+05, 0.13352E+05, 0.16261E+05, 0.19427E+05,
      0.22867E+05, 0.26600E+05, 0.30643E+05, 0.35018E+05, 0.39745E+05,
      0.44844E+05, 0.50338E+05, 0.56249E+05, 0.62599E+05, 0.69410E+05,
      0.76706E+05, 0.84509E+05, 0.92845E+05, 0.10174E+06, 0.11121E+06,
      0.12128E+06, 0.13199E+06, 0.14335E+06, 0.15540E+06, 0.16815E+06,
      0.18165E+06, 0.19591E+06, 0.21096E+06, 0.22684E+06, 0.24358E+06,
      0.26120E+06, 0.27974E+06, 0.29922E+06, 0.31969E+06, 0.34118E+06,
      0.36372E+06, 0.38735E+06, 0.41210E+06, 0.43800E+06, 0.46511E+06,
      0.49345E+06, 0.52307E+06, 0.55400E+06, 0.58628E+06, 0.61997E+06,
      0.65509E+06, 0.69170E+06, 0.72984E+06, 0.76954E+06, 0.81087E+06,
      0.85386E+06, 0.89856E+06, 0.94502E+06, 0.99329E+06, 0.10434E+07,
      0.10955E+07, 0.11495E+07, 0.12055E+07, 0.12636E+07, 0.13238E+07,
      0.13862E+07, 0.14508E+07, 0.15177E+07, 0.15870E+07, 0.16587E+07,
      0.17328E+07, 0.18095E+07, 0.18888E+07, 0.19707E+07, 0.20554E+07,
      0.21428E+07, 0.22331E+07, 0.23263E+07, 0.24225E+07, 0.25217E+07,
      0.26241E+07, 0.27296E+07, 0.28385E+07, 0.29506E+07, 0.30662E+07,
      0.31853E+07, 0.33079E+07, 0.34341E+07, 0.35641E+07, 0.36979E+07,
      0.38355E+07, 0.39771E+07, 0.41228E+07, 0.42725E+07, 0.44265E+07,
      0.45848E+07, 0.47474E+07, 0.49145E+07, 0.50862E+07, 0.52624E+07,
      0.54435E+07, 0.56293E+07, 0.58201E+07, 0.60159E+07, 0.62168E+07,
      0.64229E+07, 0.66343E+07, 0.68511E+07, 0.70734E+07, 0.73013E+07,
      0.75349E+07, 0.77742E+07, 0.80196E+07, 0.82709E+07, 0.85283E+07,
      0.87920E+07, 0.90620E+07, 0.93385E+07, 0.96215E+07, 0.99112E+07,
      0.10208E+08])


#  --------------- HOBr 161: M = 37, I = 2 ---------------------
M = 37
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(8.)
TIPS_ISO_HASH[(M,I)] = float32([0.24350E+04, 0.41047E+04, 0.60448E+04,
      0.82291E+04, 0.10648E+05, 0.13301E+05, 0.16200E+05, 0.19355E+05,
      0.22784E+05, 0.26504E+05, 0.30534E+05, 0.34895E+05, 0.39607E+05,
      0.44691E+05, 0.50169E+05, 0.56063E+05, 0.62394E+05, 0.69186E+05,
      0.76461E+05, 0.84243E+05, 0.92555E+05, 0.10142E+06, 0.11087E+06,
      0.12091E+06, 0.13159E+06, 0.14292E+06, 0.15494E+06, 0.16766E+06,
      0.18112E+06, 0.19534E+06, 0.21036E+06, 0.22620E+06, 0.24289E+06,
      0.26047E+06, 0.27896E+06, 0.29840E+06, 0.31882E+06, 0.34025E+06,
      0.36274E+06, 0.38630E+06, 0.41099E+06, 0.43683E+06, 0.46387E+06,
      0.49215E+06, 0.52169E+06, 0.55255E+06, 0.58475E+06, 0.61836E+06,
      0.65340E+06, 0.68992E+06, 0.72796E+06, 0.76757E+06, 0.80880E+06,
      0.85169E+06, 0.89628E+06, 0.94263E+06, 0.99079E+06, 0.10408E+07,
      0.10927E+07, 0.11466E+07, 0.12025E+07, 0.12605E+07, 0.13205E+07,
      0.13828E+07, 0.14472E+07, 0.15140E+07, 0.15831E+07, 0.16546E+07,
      0.17286E+07, 0.18051E+07, 0.18842E+07, 0.19660E+07, 0.20504E+07,
      0.21377E+07, 0.22277E+07, 0.23207E+07, 0.24167E+07, 0.25157E+07,
      0.26178E+07, 0.27231E+07, 0.28317E+07, 0.29436E+07, 0.30589E+07,
      0.31777E+07, 0.33001E+07, 0.34260E+07, 0.35557E+07, 0.36892E+07,
      0.38265E+07, 0.39678E+07, 0.41131E+07, 0.42626E+07, 0.44162E+07,
      0.45741E+07, 0.47364E+07, 0.49031E+07, 0.50744E+07, 0.52503E+07,
      0.54309E+07, 0.56164E+07, 0.58067E+07, 0.60021E+07, 0.62025E+07,
      0.64081E+07, 0.66191E+07, 0.68354E+07, 0.70572E+07, 0.72846E+07,
      0.75177E+07, 0.77565E+07, 0.80013E+07, 0.82521E+07, 0.85090E+07,
      0.87721E+07, 0.90415E+07, 0.93173E+07, 0.95997E+07, 0.98888E+07,
      0.10185E+08])


#  --------------- C2H4 221: M = 38, I = 1 ---------------------
M = 38
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.95843E+03, 0.16137E+04, 0.23744E+04,
      0.32285E+04, 0.41694E+04, 0.51963E+04, 0.63143E+04, 0.75337E+04,
      0.88702E+04, 0.10344E+05, 0.11978E+05, 0.13802E+05, 0.15846E+05,
      0.18145E+05, 0.20740E+05, 0.23675E+05, 0.27000E+05, 0.30770E+05,
      0.35048E+05, 0.39905E+05, 0.45420E+05, 0.51680E+05, 0.58786E+05,
      0.66850E+05, 0.75997E+05, 0.86369E+05, 0.98123E+05, 0.11144E+06,
      0.12651E+06, 0.14356E+06, 0.16284E+06, 0.18463E+06, 0.20923E+06,
      0.23699E+06, 0.26831E+06, 0.30360E+06, 0.34334E+06, 0.38808E+06,
      0.43840E+06, 0.49495E+06, 0.55847E+06, 0.62976E+06, 0.70973E+06,
      0.79935E+06, 0.89973E+06, 0.10121E+07, 0.11378E+07, 0.12782E+07,
      0.14351E+07, 0.16102E+07, 0.18055E+07, 0.20231E+07, 0.22656E+07,
      0.25354E+07, 0.28356E+07, 0.31692E+07, 0.35398E+07, 0.39511E+07,
      0.44074E+07, 0.49132E+07, 0.54736E+07, 0.60940E+07, 0.67803E+07,
      0.75392E+07, 0.83776E+07, 0.93035E+07, 0.10325E+08, 0.11452E+08,
      0.12694E+08, 0.14062E+08, 0.15567E+08, 0.17224E+08, 0.19045E+08,
      0.21046E+08, 0.23243E+08, 0.25655E+08, 0.28300E+08, 0.31200E+08,
      0.34377E+08, 0.37856E+08, 0.41662E+08, 0.45826E+08, 0.50378E+08,
      0.55351E+08, 0.60781E+08, 0.66707E+08, 0.73172E+08, 0.80219E+08,
      0.87899E+08, 0.96262E+08, 0.10537E+09, 0.11527E+09, 0.12604E+09,
      0.13775E+09, 0.15047E+09, 0.16428E+09, 0.17927E+09, 0.19553E+09,
      0.21316E+09, 0.23226E+09, 0.25296E+09, 0.27537E+09, 0.29963E+09,
      0.32587E+09, 0.35425E+09, 0.38492E+09, 0.41805E+09, 0.45383E+09,
      0.49246E+09, 0.53413E+09, 0.57908E+09, 0.62754E+09, 0.67977E+09,
      0.73602E+09, 0.79660E+09, 0.86179E+09, 0.93194E+09, 0.10074E+10,
      0.10885E+10])


#  --------------- C2H4 231: M = 38, I = 2 ---------------------
M = 38
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.39228E+04, 0.66051E+04, 0.97190E+04,
      0.13215E+05, 0.17066E+05, 0.21270E+05, 0.25846E+05, 0.30838E+05,
      0.36309E+05, 0.42341E+05, 0.49032E+05, 0.56496E+05, 0.64862E+05,
      0.74275E+05, 0.84897E+05, 0.96912E+05, 0.11052E+06, 0.12595E+06,
      0.14347E+06, 0.16335E+06, 0.18592E+06, 0.21155E+06, 0.24064E+06,
      0.27365E+06, 0.31109E+06, 0.35354E+06, 0.40166E+06, 0.45615E+06,
      0.51785E+06, 0.58765E+06, 0.66657E+06, 0.75575E+06, 0.85646E+06,
      0.97011E+06, 0.10983E+07, 0.12428E+07, 0.14055E+07, 0.15886E+07,
      0.17945E+07, 0.20260E+07, 0.22861E+07, 0.25779E+07, 0.29052E+07,
      0.32721E+07, 0.36830E+07, 0.41429E+07, 0.46573E+07, 0.52323E+07,
      0.58744E+07, 0.65912E+07, 0.73906E+07, 0.82816E+07, 0.92740E+07,
      0.10379E+08, 0.11607E+08, 0.12973E+08, 0.14490E+08, 0.16174E+08,
      0.18042E+08, 0.20112E+08, 0.22406E+08, 0.24945E+08, 0.27755E+08,
      0.30861E+08, 0.34293E+08, 0.38083E+08, 0.42266E+08, 0.46878E+08,
      0.51961E+08, 0.57560E+08, 0.63724E+08, 0.70504E+08, 0.77959E+08,
      0.86150E+08, 0.95145E+08, 0.10502E+09, 0.11585E+09, 0.12772E+09,
      0.14072E+09, 0.15496E+09, 0.17054E+09, 0.18759E+09, 0.20622E+09,
      0.22658E+09, 0.24880E+09, 0.27306E+09, 0.29952E+09, 0.32837E+09,
      0.35981E+09, 0.39404E+09, 0.43131E+09, 0.47186E+09, 0.51595E+09,
      0.56387E+09, 0.61594E+09, 0.67247E+09, 0.73382E+09, 0.80038E+09,
      0.87255E+09, 0.95076E+09, 0.10355E+10, 0.11272E+10, 0.12265E+10,
      0.13339E+10, 0.14501E+10, 0.15756E+10, 0.17113E+10, 0.18577E+10,
      0.20159E+10, 0.21865E+10, 0.23705E+10, 0.25688E+10, 0.27826E+10,
      0.30129E+10, 0.32608E+10, 0.35277E+10, 0.38149E+10, 0.41237E+10,
      0.44557E+10])


#  --------------- CH3OH 2161: M = 39, I = 1 --------------------- not in TIPS-2011
M = 39
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


#  --------------- CH3Br 219: M = 40, I = 1 ---------------------
M = 40
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.70299E+04, 0.11847E+05, 0.17442E+05,
      0.23741E+05, 0.30723E+05, 0.38408E+05, 0.46851E+05, 0.56138E+05,
      0.66375E+05, 0.77692E+05, 0.90239E+05, 0.10418E+06, 0.11972E+06,
      0.13704E+06, 0.15639E+06, 0.17801E+06, 0.20218E+06, 0.22920E+06,
      0.25940E+06, 0.29316E+06, 0.33087E+06, 0.37296E+06, 0.41992E+06,
      0.47229E+06, 0.53062E+06, 0.59557E+06, 0.66781E+06, 0.74812E+06,
      0.83731E+06, 0.93629E+06, 0.10461E+07, 0.11677E+07, 0.13023E+07,
      0.14513E+07, 0.16159E+07, 0.17978E+07, 0.19985E+07, 0.22199E+07,
      0.24638E+07, 0.27324E+07, 0.30280E+07, 0.33529E+07, 0.37099E+07,
      0.41019E+07, 0.45319E+07, 0.50034E+07, 0.55199E+07, 0.60853E+07,
      0.67039E+07, 0.73801E+07, 0.81189E+07, 0.89255E+07, 0.98056E+07,
      0.10765E+08, 0.11811E+08, 0.12949E+08, 0.14188E+08, 0.15535E+08,
      0.17000E+08, 0.18590E+08, 0.20317E+08, 0.22190E+08, 0.24220E+08,
      0.26421E+08, 0.28804E+08, 0.31383E+08, 0.34173E+08, 0.37189E+08,
      0.40448E+08, 0.43967E+08, 0.47765E+08, 0.51862E+08, 0.56280E+08,
      0.61040E+08, 0.66167E+08, 0.71686E+08, 0.77624E+08, 0.84009E+08,
      0.90873E+08, 0.98247E+08, 0.10616E+09, 0.11466E+09, 0.12378E+09,
      0.13356E+09, 0.14403E+09, 0.15526E+09, 0.16728E+09, 0.18014E+09,
      0.19391E+09, 0.20863E+09, 0.22436E+09, 0.24117E+09, 0.25913E+09,
      0.27830E+09, 0.29875E+09, 0.32057E+09, 0.34384E+09, 0.36864E+09,
      0.39506E+09, 0.42320E+09, 0.45316E+09, 0.48504E+09, 0.51896E+09,
      0.55502E+09, 0.59336E+09, 0.63410E+09, 0.67738E+09, 0.72334E+09,
      0.77212E+09, 0.82388E+09, 0.87879E+09, 0.93701E+09, 0.99873E+09,
      0.10641E+10, 0.11334E+10, 0.12068E+10, 0.12845E+10, 0.13667E+10,
      0.14536E+10])


#  --------------- CH3Br 211: M = 40, I = 2 ---------------------
M = 40
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.70566E+04, 0.11892E+05, 0.17508E+05,
      0.23832E+05, 0.30841E+05, 0.38557E+05, 0.47036E+05, 0.56362E+05,
      0.66644E+05, 0.78011E+05, 0.90615E+05, 0.10462E+06, 0.12023E+06,
      0.13763E+06, 0.15707E+06, 0.17880E+06, 0.20308E+06, 0.23023E+06,
      0.26059E+06, 0.29451E+06, 0.33240E+06, 0.37471E+06, 0.42191E+06,
      0.47453E+06, 0.53316E+06, 0.59843E+06, 0.67104E+06, 0.75176E+06,
      0.84141E+06, 0.94090E+06, 0.10512E+07, 0.11735E+07, 0.13088E+07,
      0.14585E+07, 0.16241E+07, 0.18069E+07, 0.20086E+07, 0.22312E+07,
      0.24764E+07, 0.27464E+07, 0.30435E+07, 0.33702E+07, 0.37291E+07,
      0.41231E+07, 0.45554E+07, 0.50294E+07, 0.55486E+07, 0.61171E+07,
      0.67389E+07, 0.74188E+07, 0.81616E+07, 0.89725E+07, 0.98573E+07,
      0.10822E+08, 0.11873E+08, 0.13018E+08, 0.14263E+08, 0.15618E+08,
      0.17090E+08, 0.18689E+08, 0.20425E+08, 0.22308E+08, 0.24350E+08,
      0.26563E+08, 0.28959E+08, 0.31552E+08, 0.34357E+08, 0.37389E+08,
      0.40666E+08, 0.44204E+08, 0.48023E+08, 0.52143E+08, 0.56585E+08,
      0.61371E+08, 0.66526E+08, 0.72076E+08, 0.78046E+08, 0.84467E+08,
      0.91369E+08, 0.98783E+08, 0.10674E+09, 0.11529E+09, 0.12446E+09,
      0.13429E+09, 0.14482E+09, 0.15611E+09, 0.16820E+09, 0.18113E+09,
      0.19497E+09, 0.20978E+09, 0.22560E+09, 0.24250E+09, 0.26056E+09,
      0.27983E+09, 0.30040E+09, 0.32234E+09, 0.34574E+09, 0.37068E+09,
      0.39725E+09, 0.42555E+09, 0.45567E+09, 0.48773E+09, 0.52184E+09,
      0.55811E+09, 0.59666E+09, 0.63763E+09, 0.68115E+09, 0.72736E+09,
      0.77642E+09, 0.82847E+09, 0.88368E+09, 0.94223E+09, 0.10043E+10,
      0.10701E+10, 0.11397E+10, 0.12135E+10, 0.12916E+10, 0.13743E+10,
      0.14618E+10])


#  --------------- CH3CN 2124: M = 41, I = 1 ---------------------
M = 41
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(3.)
TIPS_ISO_HASH[(M,I)] = float32([0.54361E+04, 0.91953E+04, 0.13708E+05,
      0.19097E+05, 0.25531E+05, 0.33206E+05, 0.42337E+05, 0.53173E+05,
      0.66002E+05, 0.81163E+05, 0.99053E+05, 0.12014E+06, 0.14496E+06,
      0.17414E+06, 0.20843E+06, 0.24866E+06, 0.29580E+06, 0.35099E+06,
      0.41551E+06, 0.49085E+06, 0.57871E+06, 0.68104E+06, 0.80008E+06,
      0.93836E+06, 0.10988E+07, 0.12848E+07, 0.14999E+07, 0.17487E+07,
      0.20359E+07, 0.23670E+07, 0.27484E+07, 0.31871E+07, 0.36912E+07,
      0.42697E+07, 0.49328E+07, 0.56921E+07, 0.65605E+07, 0.75526E+07,
      0.86847E+07, 0.99753E+07, 0.11445E+08, 0.13116E+08, 0.15016E+08,
      0.17172E+08, 0.19617E+08, 0.22386E+08, 0.25520E+08, 0.29063E+08,
      0.33064E+08, 0.37578E+08, 0.42667E+08, 0.48397E+08, 0.54844E+08,
      0.62090E+08, 0.70228E+08, 0.79358E+08, 0.89592E+08, 0.10105E+09,
      0.11388E+09, 0.12822E+09, 0.14424E+09, 0.16212E+09, 0.18205E+09,
      0.20427E+09, 0.22900E+09, 0.25652E+09, 0.28710E+09, 0.32107E+09,
      0.35877E+09, 0.40059E+09, 0.44692E+09, 0.49822E+09, 0.55500E+09,
      0.61777E+09, 0.68712E+09, 0.76370E+09, 0.84819E+09, 0.94135E+09,
      0.10440E+10, 0.11570E+10, 0.12814E+10, 0.14181E+10, 0.15684E+10,
      0.17334E+10, 0.19145E+10, 0.21131E+10, 0.23308E+10, 0.25693E+10,
      0.28304E+10, 0.31161E+10, 0.34285E+10, 0.37698E+10, 0.41426E+10,
      0.45496E+10, 0.49935E+10, 0.54776E+10, 0.60051E+10, 0.65796E+10,
      0.72049E+10, 0.78853E+10, 0.86251E+10, 0.94291E+10, 0.10303E+11,
      0.11251E+11, 0.12280E+11, 0.13396E+11, 0.14606E+11, 0.15916E+11,
      0.17336E+11, 0.18873E+11, 0.20536E+11, 0.22334E+11, 0.24278E+11,
      0.26379E+11, 0.28647E+11, 0.31096E+11, 0.33739E+11, 0.36589E+11,
      0.39661E+11])


#  --------------- CH3CN 2134: M = 41, I = 2 --------------------- not in HITRAN-2012
M = 41
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.10906E+05, 0.18458E+05, 0.27552E+05,
      0.38455E+05, 0.51523E+05, 0.67161E+05, 0.85818E+05, 0.10801E+06,
      0.13434E+06, 0.16550E+06, 0.20234E+06, 0.24581E+06, 0.29705E+06,
      0.35737E+06, 0.42831E+06, 0.51162E+06, 0.60936E+06, 0.72387E+06,
      0.85786E+06, 0.10145E+07, 0.11972E+07, 0.14102E+07, 0.16582E+07,
      0.19465E+07, 0.22813E+07, 0.26695E+07, 0.31190E+07, 0.36390E+07,
      0.42397E+07, 0.49328E+07, 0.57314E+07, 0.66507E+07, 0.77076E+07,
      0.89211E+07, 0.10313E+08, 0.11907E+08, 0.13732E+08, 0.15817E+08,
      0.18198E+08, 0.20914E+08, 0.24007E+08, 0.27527E+08, 0.31529E+08,
      0.36073E+08, 0.41228E+08, 0.47070E+08, 0.53683E+08, 0.61162E+08,
      0.69612E+08, 0.79149E+08, 0.89903E+08, 0.10202E+09, 0.11565E+09,
      0.13098E+09, 0.14820E+09, 0.16753E+09, 0.18921E+09, 0.21349E+09,
      0.24066E+09, 0.27106E+09, 0.30502E+09, 0.34293E+09, 0.38523E+09,
      0.43237E+09, 0.48486E+09, 0.54328E+09, 0.60823E+09, 0.68039E+09,
      0.76049E+09, 0.84935E+09, 0.94784E+09, 0.10569E+10, 0.11777E+10,
      0.13112E+10, 0.14588E+10, 0.16217E+10, 0.18016E+10, 0.19999E+10,
      0.22185E+10, 0.24592E+10, 0.27241E+10, 0.30155E+10, 0.33357E+10,
      0.36875E+10, 0.40736E+10, 0.44971E+10, 0.49615E+10, 0.54702E+10,
      0.60273E+10, 0.66369E+10, 0.73035E+10, 0.80322E+10, 0.88282E+10,
      0.96972E+10, 0.10645E+11, 0.11679E+11, 0.12806E+11, 0.14034E+11,
      0.15370E+11, 0.16824E+11, 0.18406E+11, 0.20125E+11, 0.21992E+11,
      0.24020E+11, 0.26221E+11, 0.28608E+11, 0.31197E+11, 0.34002E+11,
      0.37040E+11, 0.40330E+11, 0.43889E+11, 0.47739E+11, 0.51902E+11,
      0.56400E+11, 0.61259E+11, 0.66504E+11, 0.72165E+11, 0.78272E+11,
      0.84856E+11])


#  --------------- CH3CN 3124: M = 41, I = 3 --------------------- not in HITRAN-2012
M = 41
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.11223E+05, 0.18985E+05, 0.28307E+05,
      0.39441E+05, 0.52744E+05, 0.68620E+05, 0.87523E+05, 0.10997E+06,
      0.13658E+06, 0.16806E+06, 0.20524E+06, 0.24910E+06, 0.30080E+06,
      0.36165E+06, 0.43319E+06, 0.51722E+06, 0.61579E+06, 0.73127E+06,
      0.86640E+06, 0.10243E+07, 0.12086E+07, 0.14234E+07, 0.16735E+07,
      0.19642E+07, 0.23017E+07, 0.26931E+07, 0.31464E+07, 0.36706E+07,
      0.42762E+07, 0.49749E+07, 0.57801E+07, 0.67069E+07, 0.77722E+07,
      0.89955E+07, 0.10398E+08, 0.12006E+08, 0.13845E+08, 0.15947E+08,
      0.18346E+08, 0.21083E+08, 0.24201E+08, 0.27748E+08, 0.31781E+08,
      0.36361E+08, 0.41556E+08, 0.47442E+08, 0.54106E+08, 0.61643E+08,
      0.70157E+08, 0.79767E+08, 0.90604E+08, 0.10281E+09, 0.11655E+09,
      0.13199E+09, 0.14935E+09, 0.16882E+09, 0.19065E+09, 0.21512E+09,
      0.24250E+09, 0.27312E+09, 0.30733E+09, 0.34553E+09, 0.38814E+09,
      0.43562E+09, 0.48851E+09, 0.54736E+09, 0.61279E+09, 0.68548E+09,
      0.76617E+09, 0.85568E+09, 0.95489E+09, 0.10648E+10, 0.11864E+10,
      0.13209E+10, 0.14695E+10, 0.16337E+10, 0.18148E+10, 0.20146E+10,
      0.22348E+10, 0.24772E+10, 0.27441E+10, 0.30375E+10, 0.33601E+10,
      0.37143E+10, 0.41032E+10, 0.45298E+10, 0.49975E+10, 0.55099E+10,
      0.60709E+10, 0.66849E+10, 0.73563E+10, 0.80902E+10, 0.88918E+10,
      0.97670E+10, 0.10722E+11, 0.11763E+11, 0.12898E+11, 0.14134E+11,
      0.15480E+11, 0.16945E+11, 0.18537E+11, 0.20269E+11, 0.22149E+11,
      0.24191E+11, 0.26408E+11, 0.28812E+11, 0.31419E+11, 0.34244E+11,
      0.37303E+11, 0.40616E+11, 0.44201E+11, 0.48078E+11, 0.52269E+11,
      0.56799E+11, 0.61692E+11, 0.66974E+11, 0.72675E+11, 0.78824E+11,
      0.85454E+11])


#  --------------- CH3CN 3134: M = 41, I = 4 --------------------- not in HITRAN-2012
M = 41
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.22522E+05, 0.38117E+05, 0.56899E+05,
      0.79412E+05, 0.10640E+06, 0.13870E+06, 0.17726E+06, 0.22314E+06,
      0.27761E+06, 0.34214E+06, 0.41847E+06, 0.50862E+06, 0.61497E+06,
      0.74028E+06, 0.88774E+06, 0.10611E+07, 0.12646E+07, 0.15031E+07,
      0.17825E+07, 0.21092E+07, 0.24908E+07, 0.29358E+07, 0.34541E+07,
      0.40571E+07, 0.47576E+07, 0.55703E+07, 0.65120E+07, 0.76018E+07,
      0.88614E+07, 0.10315E+08, 0.11992E+08, 0.13922E+08, 0.16142E+08,
      0.18693E+08, 0.21619E+08, 0.24973E+08, 0.28812E+08, 0.33202E+08,
      0.38216E+08, 0.43936E+08, 0.50455E+08, 0.57876E+08, 0.66315E+08,
      0.75901E+08, 0.86779E+08, 0.99110E+08, 0.11307E+09, 0.12887E+09,
      0.14672E+09, 0.16688E+09, 0.18961E+09, 0.21523E+09, 0.24407E+09,
      0.27651E+09, 0.31295E+09, 0.35387E+09, 0.39975E+09, 0.45118E+09,
      0.50875E+09, 0.57315E+09, 0.64512E+09, 0.72549E+09, 0.81517E+09,
      0.91514E+09, 0.10265E+10, 0.11504E+10, 0.12883E+10, 0.14414E+10,
      0.16115E+10, 0.18001E+10, 0.20093E+10, 0.22410E+10, 0.24975E+10,
      0.27812E+10, 0.30948E+10, 0.34412E+10, 0.38235E+10, 0.42452E+10,
      0.47101E+10, 0.52220E+10, 0.57856E+10, 0.64055E+10, 0.70869E+10,
      0.78355E+10, 0.86574E+10, 0.95591E+10, 0.10548E+11, 0.11631E+11,
      0.12817E+11, 0.14116E+11, 0.15536E+11, 0.17088E+11, 0.18785E+11,
      0.20636E+11, 0.22657E+11, 0.24861E+11, 0.27264E+11, 0.29881E+11,
      0.32730E+11, 0.35832E+11, 0.39205E+11, 0.42871E+11, 0.46855E+11,
      0.51182E+11, 0.55878E+11, 0.60973E+11, 0.66497E+11, 0.72484E+11,
      0.78970E+11, 0.85992E+11, 0.93592E+11, 0.10181E+12, 0.11070E+12,
      0.12031E+12, 0.13069E+12, 0.14189E+12, 0.15398E+12, 0.16703E+12,
      0.18110E+12])


#  --------------- CF4 29: M = 42, I = 1 ---------------------
M = 42
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.76233E+04, 0.12867E+05, 0.19059E+05,
      0.26316E+05, 0.34895E+05, 0.45145E+05, 0.57461E+05, 0.72259E+05,
      0.89950E+05, 0.11092E+06, 0.13550E+06, 0.16399E+06, 0.19658E+06,
      0.23341E+06, 0.27457E+06, 0.32004E+06, 0.36978E+06, 0.42369E+06,
      0.48161E+06, 0.54338E+06, 0.60880E+06, 0.67764E+06, 0.55684E+07,
      0.71250E+07, 0.90615E+07, 0.11458E+08, 0.14407E+08, 0.18021E+08,
      0.22428E+08, 0.27778E+08, 0.34247E+08, 0.42038E+08, 0.51386E+08,
      0.62559E+08, 0.75869E+08, 0.91670E+08, 0.11037E+09, 0.13242E+09,
      0.15836E+09, 0.18878E+09, 0.22436E+09, 0.26584E+09, 0.31410E+09,
      0.37008E+09, 0.43488E+09, 0.50970E+09, 0.59589E+09, 0.69496E+09,
      0.80858E+09, 0.93863E+09, 0.10872E+10, 0.12565E+10, 0.14491E+10,
      0.16679E+10, 0.19159E+10, 0.21966E+10, 0.25136E+10, 0.28711E+10,
      0.32740E+10, 0.37260E+10, 0.42340E+10, 0.48030E+10, 0.54400E+10,
      0.61520E+10, 0.69470E+10, 0.78320E+10, 0.88170E+10, 0.99120E+10,
      0.11130E+11, 0.12470E+11, 0.13970E+11, 0.15620E+11, 0.17440E+11,
      0.19450E+11, 0.21670E+11, 0.24100E+11, 0.26790E+11, 0.29730E+11,
      0.33000E+11, 0.36500E+11, 0.40400E+11, 0.44600E+11, 0.49300E+11,
      0.54300E+11, 0.59800E+11, 0.65800E+11, 0.72400E+11, 0.79500E+11,
      0.87200E+11, 0.95500E+11, 0.10500E+12, 0.11400E+12, 0.12500E+12,
      0.13600E+12, 0.14900E+12, 0.16200E+12, 0.17700E+12, 0.19200E+12,
      0.21000E+12, 0.23000E+12, 0.25000E+12, 0.27000E+12, 0.29000E+12,
      0.31000E+12, 0.34000E+12, 0.36000E+12, 0.39000E+12, 0.42000E+12,
      0.46000E+12, 0.49000E+12, 0.53000E+12, 0.57000E+12, 0.61000E+12,
      0.66000E+12, 0.70000E+12, 0.75000E+12, 0.81000E+12, 0.86000E+12,
      0.93000E+12])


#  --------------- C4H2 1221: M = 43, I = 1 ---------------------
M = 43
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.57628E+03, 0.84874E+03, 0.11789E+04,
      0.15952E+04, 0.21317E+04, 0.28324E+04, 0.37543E+04, 0.49705E+04,
      0.65754E+04, 0.86894E+04, 0.11466E+05, 0.15099E+05, 0.19834E+05,
      0.25980E+05, 0.33920E+05, 0.44132E+05, 0.57210E+05, 0.73884E+05,
      0.95049E+05, 0.12180E+06, 0.15548E+06, 0.19771E+06, 0.25045E+06,
      0.31606E+06, 0.39739E+06, 0.49786E+06, 0.62152E+06, 0.77324E+06,
      0.95878E+06, 0.11850E+07, 0.14599E+07, 0.17930E+07, 0.21956E+07,
      0.26807E+07, 0.32637E+07, 0.39626E+07, 0.47983E+07, 0.57951E+07,
      0.69813E+07, 0.83896E+07, 0.10058E+08, 0.12030E+08, 0.14356E+08,
      0.17093E+08, 0.20309E+08, 0.24079E+08, 0.28491E+08, 0.33644E+08,
      0.39651E+08, 0.46642E+08, 0.54764E+08, 0.64184E+08, 0.75091E+08,
      0.87699E+08, 0.10225E+09, 0.11902E+09, 0.13832E+09, 0.16049E+09,
      0.18593E+09, 0.21507E+09, 0.24841E+09, 0.28650E+09, 0.32996E+09,
      0.37949E+09, 0.43586E+09, 0.49993E+09, 0.57266E+09, 0.65513E+09,
      0.74852E+09, 0.85418E+09, 0.97356E+09, 0.11083E+10, 0.12602E+10,
      0.14313E+10, 0.16238E+10, 0.18401E+10, 0.20829E+10, 0.23553E+10,
      0.26605E+10, 0.30021E+10, 0.33841E+10, 0.38109E+10, 0.42874E+10,
      0.48187E+10, 0.54107E+10, 0.60698E+10, 0.68029E+10, 0.76176E+10,
      0.85223E+10, 0.95260E+10, 0.10639E+11, 0.11871E+11, 0.13236E+11,
      0.14744E+11, 0.16412E+11, 0.18253E+11, 0.20285E+11, 0.22526E+11,
      0.24995E+11, 0.27714E+11, 0.30705E+11, 0.33995E+11, 0.37609E+11,
      0.41579E+11, 0.45934E+11, 0.50711E+11, 0.55947E+11, 0.61681E+11,
      0.67957E+11, 0.74824E+11, 0.82330E+11, 0.90532E+11, 0.99487E+11,
      0.10926E+12, 0.11992E+12, 0.13154E+12, 0.14420E+12, 0.15799E+12,
      0.17299E+12])


#  --------------- HC3N 12224: M = 44, I = 1 --------------------- 1224 in HITRAN, 12224 in TIPS
M = 44
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.16683E+04, 0.24538E+04, 0.33995E+04,
      0.45769E+04, 0.60637E+04, 0.79533E+04, 0.10360E+05, 0.13422E+05,
      0.17311E+05, 0.22232E+05, 0.28434E+05, 0.36215E+05, 0.45932E+05,
      0.58011E+05, 0.72958E+05, 0.91370E+05, 0.11395E+06, 0.14153E+06,
      0.17507E+06, 0.21570E+06, 0.26475E+06, 0.32372E+06, 0.39440E+06,
      0.47881E+06, 0.57930E+06, 0.69856E+06, 0.83968E+06, 0.10062E+07,
      0.12021E+07, 0.14320E+07, 0.17011E+07, 0.20153E+07, 0.23812E+07,
      0.28065E+07, 0.32996E+07, 0.38701E+07, 0.45287E+07, 0.52876E+07,
      0.61602E+07, 0.71616E+07, 0.83088E+07, 0.96206E+07, 0.11118E+08,
      0.12824E+08, 0.14765E+08, 0.16969E+08, 0.19469E+08, 0.22299E+08,
      0.25498E+08, 0.29110E+08, 0.33181E+08, 0.37763E+08, 0.42914E+08,
      0.48697E+08, 0.55180E+08, 0.62440E+08, 0.70558E+08, 0.79627E+08,
      0.89743E+08, 0.10102E+09, 0.11356E+09, 0.12752E+09, 0.14301E+09,
      0.16020E+09, 0.17925E+09, 0.20035E+09, 0.22367E+09, 0.24945E+09,
      0.27790E+09, 0.30928E+09, 0.34385E+09, 0.38191E+09, 0.42376E+09,
      0.46975E+09, 0.52023E+09, 0.57562E+09, 0.63632E+09, 0.70279E+09,
      0.77553E+09, 0.85506E+09, 0.94195E+09, 0.10368E+10, 0.11403E+10,
      0.12531E+10, 0.13759E+10, 0.15097E+10, 0.16552E+10, 0.18133E+10,
      0.19851E+10, 0.21715E+10, 0.23738E+10, 0.25931E+10, 0.28307E+10,
      0.30879E+10, 0.33662E+10, 0.36672E+10, 0.39926E+10, 0.43439E+10,
      0.47233E+10, 0.51325E+10, 0.55738E+10, 0.60493E+10, 0.65615E+10,
      0.71129E+10, 0.77061E+10, 0.83441E+10, 0.90298E+10, 0.97664E+10,
      0.10557E+11, 0.11406E+11, 0.12317E+11, 0.13293E+11, 0.14339E+11,
      0.15459E+11, 0.16659E+11, 0.17942E+11, 0.19316E+11, 0.20784E+11,
      0.22353E+11])


#  --------------- HC3N 12234: M = 44, I = 2 --------------------- see above
M = 44
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.33507E+04, 0.49290E+04, 0.68293E+04,
      0.91959E+04, 0.12185E+05, 0.15986E+05, 0.20828E+05, 0.26993E+05,
      0.34824E+05, 0.44739E+05, 0.57239E+05, 0.72931E+05, 0.92539E+05,
      0.11693E+06, 0.14713E+06, 0.18435E+06, 0.23004E+06, 0.28588E+06,
      0.35384E+06, 0.43625E+06, 0.53580E+06, 0.65562E+06, 0.79933E+06,
      0.97115E+06, 0.11759E+07, 0.14191E+07, 0.17073E+07, 0.20476E+07,
      0.24486E+07, 0.29196E+07, 0.34716E+07, 0.41169E+07, 0.48696E+07,
      0.57453E+07, 0.67621E+07, 0.79402E+07, 0.93022E+07, 0.10874E+08,
      0.12684E+08, 0.14764E+08, 0.17150E+08, 0.19884E+08, 0.23009E+08,
      0.26576E+08, 0.30641E+08, 0.35265E+08, 0.40518E+08, 0.46477E+08,
      0.53225E+08, 0.60856E+08, 0.69475E+08, 0.79195E+08, 0.90143E+08,
      0.10246E+09, 0.11629E+09, 0.13182E+09, 0.14921E+09, 0.16868E+09,
      0.19045E+09, 0.21477E+09, 0.24189E+09, 0.27211E+09, 0.30575E+09,
      0.34316E+09, 0.38471E+09, 0.43083E+09, 0.48196E+09, 0.53858E+09,
      0.60125E+09, 0.67052E+09, 0.74704E+09, 0.83148E+09, 0.92459E+09,
      0.10272E+10, 0.11401E+10, 0.12643E+10, 0.14007E+10, 0.15506E+10,
      0.17150E+10, 0.18953E+10, 0.20928E+10, 0.23090E+10, 0.25456E+10,
      0.28042E+10, 0.30867E+10, 0.33951E+10, 0.37316E+10, 0.40984E+10,
      0.44981E+10, 0.49332E+10, 0.54067E+10, 0.59216E+10, 0.64812E+10,
      0.70890E+10, 0.77488E+10, 0.84645E+10, 0.92405E+10, 0.10081E+11,
      0.10992E+11, 0.11978E+11, 0.13044E+11, 0.14197E+11, 0.15443E+11,
      0.16789E+11, 0.18243E+11, 0.19810E+11, 0.21501E+11, 0.23324E+11,
      0.25288E+11, 0.27403E+11, 0.29680E+11, 0.32130E+11, 0.34764E+11,
      0.37596E+11, 0.40639E+11, 0.43907E+11, 0.47416E+11, 0.51181E+11,
      0.55220E+11])


#  --------------- HC3N 12324: M = 44, I = 3 --------------------- see above
M = 44
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.33506E+04, 0.49280E+04, 0.68267E+04,
      0.91901E+04, 0.12174E+05, 0.15966E+05, 0.20793E+05, 0.26936E+05,
      0.34734E+05, 0.44598E+05, 0.57026E+05, 0.72612E+05, 0.92071E+05,
      0.11625E+06, 0.14616E+06, 0.18298E+06, 0.22813E+06, 0.28323E+06,
      0.35022E+06, 0.43133E+06, 0.52918E+06, 0.64677E+06, 0.78761E+06,
      0.95571E+06, 0.11557E+07, 0.13929E+07, 0.16734E+07, 0.20041E+07,
      0.23929E+07, 0.28488E+07, 0.33820E+07, 0.40040E+07, 0.47280E+07,
      0.55686E+07, 0.65423E+07, 0.76678E+07, 0.89661E+07, 0.10460E+08,
      0.12177E+08, 0.14145E+08, 0.16397E+08, 0.18970E+08, 0.21903E+08,
      0.25242E+08, 0.29036E+08, 0.33339E+08, 0.38214E+08, 0.43726E+08,
      0.49949E+08, 0.56965E+08, 0.64864E+08, 0.73743E+08, 0.83711E+08,
      0.94886E+08, 0.10740E+09, 0.12139E+09, 0.13701E+09, 0.15443E+09,
      0.17384E+09, 0.19543E+09, 0.21943E+09, 0.24607E+09, 0.27561E+09,
      0.30832E+09, 0.34452E+09, 0.38453E+09, 0.42870E+09, 0.47742E+09,
      0.53110E+09, 0.59020E+09, 0.65518E+09, 0.72659E+09, 0.80496E+09,
      0.89092E+09, 0.98510E+09, 0.10882E+10, 0.12010E+10, 0.13242E+10,
      0.14588E+10, 0.16056E+10, 0.17657E+10, 0.19401E+10, 0.21299E+10,
      0.23363E+10, 0.25606E+10, 0.28043E+10, 0.30687E+10, 0.33553E+10,
      0.36660E+10, 0.40024E+10, 0.43665E+10, 0.47601E+10, 0.51856E+10,
      0.56450E+10, 0.61408E+10, 0.66756E+10, 0.72520E+10, 0.78729E+10,
      0.85413E+10, 0.92604E+10, 0.10034E+11, 0.10864E+11, 0.11757E+11,
      0.12714E+11, 0.13742E+11, 0.14843E+11, 0.16023E+11, 0.17287E+11,
      0.18640E+11, 0.20087E+11, 0.21634E+11, 0.23288E+11, 0.25054E+11,
      0.26939E+11, 0.28950E+11, 0.31096E+11, 0.33382E+11, 0.35819E+11,
      0.38413E+11])


#  --------------- HC3N 13224: M = 44, I = 4 --------------------- see above
M = 44
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(12.)
TIPS_ISO_HASH[(M,I)] = float32([0.34439E+04, 0.50672E+04, 0.70230E+04,
      0.94603E+04, 0.12542E+05, 0.16462E+05, 0.21461E+05, 0.27833E+05,
      0.35935E+05, 0.46204E+05, 0.59168E+05, 0.75463E+05, 0.95854E+05,
      0.12126E+06, 0.15276E+06, 0.19165E+06, 0.23947E+06, 0.29802E+06,
      0.36943E+06, 0.45619E+06, 0.56121E+06, 0.68789E+06, 0.84018E+06,
      0.10227E+07, 0.12407E+07, 0.15003E+07, 0.18086E+07, 0.21738E+07,
      0.26052E+07, 0.31134E+07, 0.37106E+07, 0.44109E+07, 0.52300E+07,
      0.61861E+07, 0.72996E+07, 0.85939E+07, 0.10095E+08, 0.11833E+08,
      0.13841E+08, 0.16158E+08, 0.18825E+08, 0.21890E+08, 0.25407E+08,
      0.29436E+08, 0.34045E+08, 0.39308E+08, 0.45309E+08, 0.52143E+08,
      0.59912E+08, 0.68734E+08, 0.78737E+08, 0.90065E+08, 0.10288E+09,
      0.11735E+09, 0.13367E+09, 0.15206E+09, 0.17277E+09, 0.19604E+09,
      0.22217E+09, 0.25148E+09, 0.28432E+09, 0.32108E+09, 0.36218E+09,
      0.40809E+09, 0.45932E+09, 0.51644E+09, 0.58004E+09, 0.65082E+09,
      0.72950E+09, 0.81690E+09, 0.91388E+09, 0.10214E+10, 0.11405E+10,
      0.12724E+10, 0.14182E+10, 0.15794E+10, 0.17573E+10, 0.19536E+10,
      0.21701E+10, 0.24086E+10, 0.26711E+10, 0.29599E+10, 0.32774E+10,
      0.36262E+10, 0.40090E+10, 0.44290E+10, 0.48895E+10, 0.53939E+10,
      0.59462E+10, 0.65504E+10, 0.72111E+10, 0.79332E+10, 0.87217E+10,
      0.95823E+10, 0.10521E+11, 0.11544E+11, 0.12659E+11, 0.13874E+11,
      0.15195E+11, 0.16632E+11, 0.18194E+11, 0.19892E+11, 0.21735E+11,
      0.23736E+11, 0.25907E+11, 0.28260E+11, 0.30810E+11, 0.33572E+11,
      0.36563E+11, 0.39799E+11, 0.43299E+11, 0.47083E+11, 0.51172E+11,
      0.55588E+11, 0.60355E+11, 0.65500E+11, 0.71049E+11, 0.77031E+11,
      0.83478E+11])


#  --------------- HC3N 12225: M = 44, I = 5 --------------------- see above
M = 44
I = 5
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.11455E+04, 0.16850E+04, 0.23345E+04,
      0.31432E+04, 0.41647E+04, 0.54630E+04, 0.71168E+04, 0.92219E+04,
      0.11895E+05, 0.15279E+05, 0.19545E+05, 0.24897E+05, 0.31584E+05,
      0.39899E+05, 0.50190E+05, 0.62871E+05, 0.78428E+05, 0.97434E+05,
      0.12056E+06, 0.14859E+06, 0.18243E+06, 0.22314E+06, 0.27194E+06,
      0.33026E+06, 0.39972E+06, 0.48219E+06, 0.57983E+06, 0.69509E+06,
      0.83077E+06, 0.99009E+06, 0.11767E+07, 0.13946E+07, 0.16487E+07,
      0.19441E+07, 0.22868E+07, 0.26836E+07, 0.31420E+07, 0.36704E+07,
      0.42786E+07, 0.49770E+07, 0.57776E+07, 0.66938E+07, 0.77404E+07,
      0.89339E+07, 0.10293E+08, 0.11837E+08, 0.13590E+08, 0.15576E+08,
      0.17823E+08, 0.20362E+08, 0.23227E+08, 0.26454E+08, 0.30085E+08,
      0.34166E+08, 0.38745E+08, 0.43877E+08, 0.49622E+08, 0.56046E+08,
      0.63219E+08, 0.71222E+08, 0.80138E+08, 0.90062E+08, 0.10110E+09,
      0.11335E+09, 0.12695E+09, 0.14202E+09, 0.15870E+09, 0.17716E+09,
      0.19756E+09, 0.22009E+09, 0.24493E+09, 0.27232E+09, 0.30247E+09,
      0.33565E+09, 0.37211E+09, 0.41217E+09, 0.45613E+09, 0.50433E+09,
      0.55714E+09, 0.61497E+09, 0.67823E+09, 0.74739E+09, 0.82293E+09,
      0.90540E+09, 0.99536E+09, 0.10934E+10, 0.12002E+10, 0.13165E+10,
      0.14430E+10, 0.15805E+10, 0.17299E+10, 0.18922E+10, 0.20682E+10,
      0.22591E+10, 0.24660E+10, 0.26901E+10, 0.29326E+10, 0.31951E+10,
      0.34788E+10, 0.37854E+10, 0.41166E+10, 0.44741E+10, 0.48598E+10,
      0.52758E+10, 0.57240E+10, 0.62069E+10, 0.67269E+10, 0.72864E+10,
      0.78882E+10, 0.85352E+10, 0.92305E+10, 0.99773E+10, 0.10779E+11,
      0.11639E+11, 0.12562E+11, 0.13552E+11, 0.14612E+11, 0.15748E+11,
      0.16964E+11])


#  --------------- HC3N 22224: M = 44, I = 6 --------------------- see above
M = 44
I = 6
TIPS_GSI_HASH[(M,I)] = __FloatType__(9.)
TIPS_ISO_HASH[(M,I)] = float32([0.27029E+04, 0.39999E+04, 0.55894E+04,
      0.76092E+04, 0.10219E+05, 0.13616E+05, 0.18042E+05, 0.23798E+05,
      0.31255E+05, 0.40867E+05, 0.53189E+05, 0.68897E+05, 0.88807E+05,
      0.11390E+06, 0.14537E+06, 0.18461E+06, 0.23330E+06, 0.29342E+06,
      0.36733E+06, 0.45779E+06, 0.56802E+06, 0.70182E+06, 0.86361E+06,
      0.10585E+07, 0.12925E+07, 0.15725E+07, 0.19064E+07, 0.23034E+07,
      0.27739E+07, 0.33302E+07, 0.39858E+07, 0.47566E+07, 0.56604E+07,
      0.67176E+07, 0.79511E+07, 0.93872E+07, 0.11055E+08, 0.12989E+08,
      0.15225E+08, 0.17806E+08, 0.20779E+08, 0.24197E+08, 0.28119E+08,
      0.32612E+08, 0.37749E+08, 0.43612E+08, 0.50294E+08, 0.57895E+08,
      0.66528E+08, 0.76318E+08, 0.87403E+08, 0.99937E+08, 0.11409E+09,
      0.13004E+09, 0.14800E+09, 0.16819E+09, 0.19086E+09, 0.21629E+09,
      0.24476E+09, 0.27661E+09, 0.31219E+09, 0.35189E+09, 0.39615E+09,
      0.44542E+09, 0.50021E+09, 0.56108E+09, 0.62862E+09, 0.70350E+09,
      0.78641E+09, 0.87814E+09, 0.97952E+09, 0.10915E+10, 0.12149E+10,
      0.13510E+10, 0.15008E+10, 0.16656E+10, 0.18468E+10, 0.20457E+10,
      0.22640E+10, 0.25032E+10, 0.27653E+10, 0.30522E+10, 0.33659E+10,
      0.37088E+10, 0.40832E+10, 0.44917E+10, 0.49371E+10, 0.54224E+10,
      0.59508E+10, 0.65256E+10, 0.71507E+10, 0.78298E+10, 0.85671E+10,
      0.93672E+10, 0.10235E+11, 0.11175E+11, 0.12193E+11, 0.13295E+11,
      0.14487E+11, 0.15776E+11, 0.17168E+11, 0.18671E+11, 0.20293E+11,
      0.22043E+11, 0.23929E+11, 0.25960E+11, 0.28148E+11, 0.30502E+11,
      0.33034E+11, 0.35756E+11, 0.38681E+11, 0.41823E+11, 0.45195E+11,
      0.48812E+11, 0.52692E+11, 0.56850E+11, 0.61306E+11, 0.66076E+11,
      0.71183E+11])


#  --------------- H2 11: M = 45, I = 1 --------------------- 
M = 45
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.15265E+01, 0.22243E+01, 0.29619E+01,
      0.36724E+01, 0.43456E+01, 0.49880E+01, 0.56090E+01, 0.62165E+01,
      0.68161E+01, 0.74113E+01, 0.80044E+01, 0.85966E+01, 0.91887E+01,
      0.97810E+01, 0.10374E+02, 0.10967E+02, 0.11561E+02, 0.12156E+02,
      0.12751E+02, 0.13347E+02, 0.13944E+02, 0.14541E+02, 0.15139E+02,
      0.15738E+02, 0.16337E+02, 0.16937E+02, 0.17538E+02, 0.18140E+02,
      0.18743E+02, 0.19346E+02, 0.19951E+02, 0.20556E+02, 0.21163E+02,
      0.21771E+02, 0.22379E+02, 0.22990E+02, 0.23601E+02, 0.24214E+02,
      0.24829E+02, 0.25445E+02, 0.26063E+02, 0.26683E+02, 0.27304E+02,
      0.27928E+02, 0.28553E+02, 0.29181E+02, 0.29811E+02, 0.30443E+02,
      0.31078E+02, 0.31715E+02, 0.32355E+02, 0.32997E+02, 0.33643E+02,
      0.34291E+02, 0.34942E+02, 0.35596E+02, 0.36253E+02, 0.36914E+02,
      0.37578E+02, 0.38245E+02, 0.38916E+02, 0.39590E+02, 0.40268E+02,
      0.40949E+02, 0.41635E+02, 0.42324E+02, 0.43017E+02, 0.43715E+02,
      0.44416E+02, 0.45122E+02, 0.45831E+02, 0.46546E+02, 0.47264E+02,
      0.47987E+02, 0.48714E+02, 0.49446E+02, 0.50183E+02, 0.50925E+02,
      0.51671E+02, 0.52422E+02, 0.53178E+02, 0.53939E+02, 0.54705E+02,
      0.55476E+02, 0.56252E+02, 0.57033E+02, 0.57820E+02, 0.58612E+02,
      0.59409E+02, 0.60212E+02, 0.61020E+02, 0.61833E+02, 0.62652E+02,
      0.63477E+02, 0.64308E+02, 0.65144E+02, 0.65986E+02, 0.66833E+02,
      0.67687E+02, 0.68546E+02, 0.69411E+02, 0.70283E+02, 0.71160E+02,
      0.72043E+02, 0.72933E+02, 0.73829E+02, 0.74730E+02, 0.75638E+02,
      0.76553E+02, 0.77473E+02, 0.78400E+02, 0.79333E+02, 0.80273E+02,
      0.81219E+02, 0.82172E+02, 0.83131E+02, 0.84097E+02, 0.85069E+02,
      0.86048E+02])


#  --------------- H2 12: M = 45, I = 2 --------------------- 
M = 45
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(6.)
TIPS_ISO_HASH[(M,I)] = float32([0.81692E+01, 0.10308E+02, 0.12557E+02,
      0.14848E+02, 0.17159E+02, 0.19482E+02, 0.21815E+02, 0.24153E+02,
      0.26497E+02, 0.28845E+02, 0.31197E+02, 0.33552E+02, 0.35910E+02,
      0.38272E+02, 0.40636E+02, 0.43002E+02, 0.45372E+02, 0.47744E+02,
      0.50119E+02, 0.52496E+02, 0.54877E+02, 0.57261E+02, 0.59649E+02,
      0.62040E+02, 0.64435E+02, 0.66835E+02, 0.69240E+02, 0.71650E+02,
      0.74066E+02, 0.76489E+02, 0.78918E+02, 0.81354E+02, 0.83799E+02,
      0.86252E+02, 0.88715E+02, 0.91187E+02, 0.93669E+02, 0.96163E+02,
      0.98668E+02, 0.10118E+03, 0.10371E+03, 0.10626E+03, 0.10881E+03,
      0.11138E+03, 0.11397E+03, 0.11657E+03, 0.11919E+03, 0.12182E+03,
      0.12447E+03, 0.12714E+03, 0.12982E+03, 0.13252E+03, 0.13524E+03,
      0.13798E+03, 0.14074E+03, 0.14352E+03, 0.14632E+03, 0.14914E+03,
      0.15198E+03, 0.15484E+03, 0.15772E+03, 0.16062E+03, 0.16355E+03,
      0.16649E+03, 0.16946E+03, 0.17246E+03, 0.17547E+03, 0.17851E+03,
      0.18157E+03, 0.18466E+03, 0.18777E+03, 0.19090E+03, 0.19406E+03,
      0.19725E+03, 0.20045E+03, 0.20369E+03, 0.20695E+03, 0.21023E+03,
      0.21354E+03, 0.21687E+03, 0.22024E+03, 0.22362E+03, 0.22704E+03,
      0.23048E+03, 0.23394E+03, 0.23744E+03, 0.24096E+03, 0.24451E+03,
      0.24808E+03, 0.25169E+03, 0.25532E+03, 0.25897E+03, 0.26266E+03,
      0.26638E+03, 0.27012E+03, 0.27389E+03, 0.27769E+03, 0.28152E+03,
      0.28537E+03, 0.28926E+03, 0.29317E+03, 0.29712E+03, 0.30109E+03,
      0.30509E+03, 0.30913E+03, 0.31319E+03, 0.31728E+03, 0.32140E+03,
      0.32555E+03, 0.32974E+03, 0.33395E+03, 0.33819E+03, 0.34246E+03,
      0.34677E+03, 0.35110E+03, 0.35547E+03, 0.35987E+03, 0.36429E+03,
      0.36875E+03])


#  --------------- CS 22: M = 46, I = 1 --------------------- 
M = 46
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.51416E+02, 0.72723E+02, 0.94044E+02,
      0.11538E+03, 0.13673E+03, 0.15810E+03, 0.17949E+03, 0.20093E+03,
      0.22245E+03, 0.24407E+03, 0.26582E+03, 0.28776E+03, 0.30992E+03,
      0.33233E+03, 0.35504E+03, 0.37807E+03, 0.40147E+03, 0.42525E+03,
      0.44944E+03, 0.47406E+03, 0.49914E+03, 0.52468E+03, 0.55071E+03,
      0.57723E+03, 0.60427E+03, 0.63183E+03, 0.65991E+03, 0.68854E+03,
      0.71771E+03, 0.74743E+03, 0.77771E+03, 0.80855E+03, 0.83996E+03,
      0.87193E+03, 0.90449E+03, 0.93762E+03, 0.97134E+03, 0.10056E+04,
      0.10405E+04, 0.10760E+04, 0.11121E+04, 0.11487E+04, 0.11860E+04,
      0.12239E+04, 0.12623E+04, 0.13014E+04, 0.13410E+04, 0.13813E+04,
      0.14222E+04, 0.14637E+04, 0.15057E+04, 0.15484E+04, 0.15917E+04,
      0.16357E+04, 0.16802E+04, 0.17253E+04, 0.17711E+04, 0.18175E+04,
      0.18645E+04, 0.19121E+04, 0.19603E+04, 0.20091E+04, 0.20586E+04,
      0.21087E+04, 0.21594E+04, 0.22107E+04, 0.22626E+04, 0.23152E+04,
      0.23684E+04, 0.24222E+04, 0.24767E+04, 0.25317E+04, 0.25874E+04,
      0.26438E+04, 0.27007E+04, 0.27583E+04, 0.28165E+04, 0.28754E+04,
      0.29348E+04, 0.29949E+04, 0.30557E+04, 0.31170E+04, 0.31790E+04,
      0.32417E+04, 0.33049E+04, 0.33688E+04, 0.34334E+04, 0.34986E+04,
      0.35644E+04, 0.36308E+04, 0.36979E+04, 0.37656E+04, 0.38340E+04,
      0.39030E+04, 0.39727E+04, 0.40430E+04, 0.41139E+04, 0.41855E+04,
      0.42577E+04, 0.43306E+04, 0.44041E+04, 0.44782E+04, 0.45530E+04,
      0.46284E+04, 0.47045E+04, 0.47813E+04, 0.48587E+04, 0.49367E+04,
      0.50154E+04, 0.50947E+04, 0.51747E+04, 0.52553E+04, 0.53366E+04,
      0.54185E+04, 0.55011E+04, 0.55844E+04, 0.56683E+04, 0.57528E+04,
      0.58380E+04])


#  --------------- CS 24: M = 46, I = 2 --------------------- 
M = 46
I = 2
TIPS_GSI_HASH[(M,I)] = __FloatType__(1.)
TIPS_ISO_HASH[(M,I)] = float32([0.52247E+02, 0.73900E+02, 0.95568E+02,
      0.11725E+03, 0.13895E+03, 0.16066E+03, 0.18241E+03, 0.20420E+03,
      0.22607E+03, 0.24805E+03, 0.27018E+03, 0.29249E+03, 0.31503E+03,
      0.33784E+03, 0.36096E+03, 0.38442E+03, 0.40824E+03, 0.43247E+03,
      0.45712E+03, 0.48221E+03, 0.50778E+03, 0.53382E+03, 0.56037E+03,
      0.58743E+03, 0.61501E+03, 0.64312E+03, 0.67179E+03, 0.70100E+03,
      0.73077E+03, 0.76111E+03, 0.79202E+03, 0.82351E+03, 0.85559E+03,
      0.88824E+03, 0.92149E+03, 0.95533E+03, 0.98977E+03, 0.10248E+04,
      0.10605E+04, 0.10967E+04, 0.11336E+04, 0.11710E+04, 0.12091E+04,
      0.12478E+04, 0.12871E+04, 0.13270E+04, 0.13675E+04, 0.14087E+04,
      0.14505E+04, 0.14929E+04, 0.15359E+04, 0.15795E+04, 0.16238E+04,
      0.16687E+04, 0.17142E+04, 0.17604E+04, 0.18071E+04, 0.18546E+04,
      0.19026E+04, 0.19513E+04, 0.20006E+04, 0.20505E+04, 0.21011E+04,
      0.21523E+04, 0.22042E+04, 0.22566E+04, 0.23098E+04, 0.23635E+04,
      0.24179E+04, 0.24730E+04, 0.25286E+04, 0.25850E+04, 0.26419E+04,
      0.26995E+04, 0.27578E+04, 0.28167E+04, 0.28762E+04, 0.29364E+04,
      0.29972E+04, 0.30587E+04, 0.31208E+04, 0.31836E+04, 0.32470E+04,
      0.33111E+04, 0.33758E+04, 0.34412E+04, 0.35072E+04, 0.35739E+04,
      0.36412E+04, 0.37092E+04, 0.37778E+04, 0.38471E+04, 0.39171E+04,
      0.39877E+04, 0.40589E+04, 0.41309E+04, 0.42034E+04, 0.42767E+04,
      0.43505E+04, 0.44251E+04, 0.45003E+04, 0.45762E+04, 0.46527E+04,
      0.47299E+04, 0.48077E+04, 0.48863E+04, 0.49654E+04, 0.50453E+04,
      0.51258E+04, 0.52070E+04, 0.52888E+04, 0.53713E+04, 0.54545E+04,
      0.55383E+04, 0.56229E+04, 0.57080E+04, 0.57939E+04, 0.58804E+04,
      0.59676E+04])


#  --------------- CS 32: M = 46, I = 3 --------------------- 
M = 46
I = 3
TIPS_GSI_HASH[(M,I)] = __FloatType__(2.)
TIPS_ISO_HASH[(M,I)] = float32([0.10889E+03, 0.15403E+03, 0.19920E+03,
      0.24440E+03, 0.28964E+03, 0.33491E+03, 0.38026E+03, 0.42571E+03,
      0.47134E+03, 0.51722E+03, 0.56342E+03, 0.61005E+03, 0.65719E+03,
      0.70493E+03, 0.75334E+03, 0.80249E+03, 0.85245E+03, 0.90329E+03,
      0.95504E+03, 0.10078E+04, 0.10615E+04, 0.11163E+04, 0.11721E+04,
      0.12291E+04, 0.12872E+04, 0.13464E+04, 0.14068E+04, 0.14684E+04,
      0.15311E+04, 0.15951E+04, 0.16604E+04, 0.17268E+04, 0.17945E+04,
      0.18635E+04, 0.19337E+04, 0.20051E+04, 0.20779E+04, 0.21519E+04,
      0.22272E+04, 0.23038E+04, 0.23817E+04, 0.24609E+04, 0.25414E+04,
      0.26232E+04, 0.27064E+04, 0.27908E+04, 0.28765E+04, 0.29636E+04,
      0.30520E+04, 0.31417E+04, 0.32327E+04, 0.33251E+04, 0.34188E+04,
      0.35138E+04, 0.36102E+04, 0.37079E+04, 0.38070E+04, 0.39074E+04,
      0.40091E+04, 0.41122E+04, 0.42166E+04, 0.43224E+04, 0.44295E+04,
      0.45380E+04, 0.46478E+04, 0.47590E+04, 0.48715E+04, 0.49854E+04,
      0.51007E+04, 0.52173E+04, 0.53353E+04, 0.54547E+04, 0.55754E+04,
      0.56975E+04, 0.58210E+04, 0.59458E+04, 0.60720E+04, 0.61996E+04,
      0.63285E+04, 0.64589E+04, 0.65906E+04, 0.67236E+04, 0.68581E+04,
      0.69940E+04, 0.71312E+04, 0.72698E+04, 0.74098E+04, 0.75512E+04,
      0.76940E+04, 0.78381E+04, 0.79837E+04, 0.81307E+04, 0.82790E+04,
      0.84287E+04, 0.85799E+04, 0.87324E+04, 0.88864E+04, 0.90417E+04,
      0.91984E+04, 0.93566E+04, 0.95161E+04, 0.96771E+04, 0.98394E+04,
      0.10003E+05, 0.10168E+05, 0.10335E+05, 0.10503E+05, 0.10672E+05,
      0.10843E+05, 0.11015E+05, 0.11189E+05, 0.11364E+05, 0.11541E+05,
      0.11719E+05, 0.11898E+05, 0.12079E+05, 0.12261E+05, 0.12444E+05,
      0.12630E+05])


#  --------------- CS 23: M = 46, I = 4 --------------------- 
M = 46
I = 4
TIPS_GSI_HASH[(M,I)] = __FloatType__(4.)
TIPS_ISO_HASH[(M,I)] = float32([0.20737E+03, 0.29330E+03, 0.37930E+03,
      0.46535E+03, 0.55145E+03, 0.63764E+03, 0.72394E+03, 0.81043E+03,
      0.89722E+03, 0.98443E+03, 0.10722E+04, 0.11607E+04, 0.12501E+04,
      0.13406E+04, 0.14323E+04, 0.15253E+04, 0.16197E+04, 0.17158E+04,
      0.18135E+04, 0.19129E+04, 0.20142E+04, 0.21174E+04, 0.22226E+04,
      0.23298E+04, 0.24391E+04, 0.25504E+04, 0.26639E+04, 0.27796E+04,
      0.28976E+04, 0.30177E+04, 0.31401E+04, 0.32648E+04, 0.33918E+04,
      0.35211E+04, 0.36527E+04, 0.37867E+04, 0.39231E+04, 0.40618E+04,
      0.42029E+04, 0.43463E+04, 0.44922E+04, 0.46405E+04, 0.47912E+04,
      0.49443E+04, 0.50999E+04, 0.52579E+04, 0.54183E+04, 0.55812E+04,
      0.57465E+04, 0.59143E+04, 0.60846E+04, 0.62573E+04, 0.64325E+04,
      0.66102E+04, 0.67903E+04, 0.69729E+04, 0.71581E+04, 0.73457E+04,
      0.75358E+04, 0.77284E+04, 0.79235E+04, 0.81211E+04, 0.83212E+04,
      0.85239E+04, 0.87290E+04, 0.89367E+04, 0.91469E+04, 0.93596E+04,
      0.95748E+04, 0.97926E+04, 0.10013E+05, 0.10236E+05, 0.10461E+05,
      0.10689E+05, 0.10920E+05, 0.11153E+05, 0.11388E+05, 0.11626E+05,
      0.11867E+05, 0.12110E+05, 0.12356E+05, 0.12604E+05, 0.12855E+05,
      0.13109E+05, 0.13365E+05, 0.13623E+05, 0.13884E+05, 0.14148E+05,
      0.14415E+05, 0.14683E+05, 0.14955E+05, 0.15229E+05, 0.15506E+05,
      0.15785E+05, 0.16067E+05, 0.16351E+05, 0.16638E+05, 0.16928E+05,
      0.17220E+05, 0.17515E+05, 0.17813E+05, 0.18113E+05, 0.18416E+05,
      0.18721E+05, 0.19029E+05, 0.19340E+05, 0.19653E+05, 0.19969E+05,
      0.20287E+05, 0.20608E+05, 0.20932E+05, 0.21258E+05, 0.21587E+05,
      0.21919E+05, 0.22253E+05, 0.22590E+05, 0.22930E+05, 0.23272E+05,
      0.23617E+05])


#  --------------- SO3 26: M = 46, I = 1 --------------------- not in TIPS-2011
M = 47
I = 1
TIPS_GSI_HASH[(M,I)] = __FloatType__(0.)
TIPS_ISO_HASH[(M,I)] = float32([0.])


# NOT IN HITRAN, BUT PRESENT IN TIPS-2011
#                                                        ... extracted from iso_comparison
#
# id       M    I     COMMENT          TIPS_M    TIPS_I        iso_name                 abundance          mass       mol_name
#101    1001    1    not in HITRAN      45                     H                        \N                 \N              H
#                
#102    1002    1    not in HITRAN      45                     He                       \N                 \N              He
#                
#104    1018    1    not in HITRAN      45                     Ar                       \N                 \N              Ar
#                
#                    not in HITRAN      45         4224                                                                    C2N2
#                    not in HITRAN      45         5225                                                                    C2N2
#
#                    not in HITRAN      48         26                                                                      SO
#                    not in HITRAN      48         46                                                                      SO
#                    not in HITRAN      48         28                                                                      SO
#
#                    not in HITRAN      49         1221                                                                    C3H4
#
#                    not in HITRAN      50         2111                                                                    CH3
#
#                    not in HITRAN      51         222                                                                     CS2
#                    not in HITRAN      51         224                                                                     CS2
#                    not in HITRAN      51         223                                                                     CS2
#                    not in HITRAN      51         232                                                                     CS2

def BD_TIPS_2011_PYTHON(M,I,T):

    # out of temperature range
    if T<70. or T>3000.:
        raise Exception('TIPS: T must be between 70K and 3000K.')
    
    try:
        # get statistical weight for specified isotopologue
        gi = TIPS_GSI_HASH[(M,I)]
        # interpolate partition sum for specified isotopologue
        Qt = AtoB(T,Tdat,TIPS_ISO_HASH[(M,I)],TIPS_NPT)
    except KeyError:
        raise Exception('TIPS: no data for M,I = %d,%d.' % (M,I))
    
    return gi,Qt

#  --------------- /TIPS-2011 IMPLEMENTATION ----------------------

#  --------------- TIPS-2017 IMPLEMENTATION ----------------------

#  --------------- TIPS-2017 GRID TABLE --------------------------
TIPS_2017_ISOT = {}
TIPS_2017_ISOT[0] = float64([
    1.0, 20.0, 40.0, 60.0, 80.0, 100.0, 120.0, 140.0, 160.0, 180.0,
    200.0, 220.0, 240.0, 260.0, 280.0, 300.0, 320.0, 340.0, 360.0, 380.0,
    400.0, 420.0, 440.0, 460.0, 480.0, 500.0, 520.0, 540.0, 560.0, 580.0,
    600.0, 620.0, 640.0, 660.0, 680.0, 700.0, 720.0, 740.0, 760.0, 780.0,
    800.0, 820.0, 840.0, 860.0, 880.0, 900.0, 920.0, 940.0, 960.0, 980.0,
    1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
    1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
    1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
    1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
    1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
    2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
    2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
    2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
    2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
    2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2900.0, 2920.0, 2940.0, 2960.0, 2980.0,
    3000.0, 3020.0, 3040.0, 3060.0, 3080.0, 3100.0, 3120.0, 3140.0, 3160.0, 3180.0,
    3200.0, 3220.0, 3240.0, 3260.0, 3280.0, 3300.0, 3320.0, 3340.0, 3360.0, 3380.0,
    3400.0, 3420.0, 3440.0, 3460.0, 3480.0, 3500.0, 3520.0, 3540.0, 3560.0, 3580.0,
    3600.0, 3620.0, 3640.0, 3660.0, 3680.0, 3700.0, 3720.0, 3740.0, 3760.0, 3780.0,
    3800.0, 3820.0, 3840.0, 3860.0, 3880.0, 3900.0, 3920.0, 3940.0, 3960.0, 3980.0,
    4000.0, 4020.0, 4040.0, 4060.0, 4080.0, 4100.0, 4120.0, 4140.0, 4160.0, 4180.0,
    4200.0, 4220.0, 4240.0, 4260.0, 4280.0, 4300.0, 4320.0, 4340.0, 4360.0, 4380.0,
    4400.0, 4420.0, 4440.0, 4460.0, 4480.0, 4500.0, 4520.0, 4540.0, 4560.0, 4580.0,
    4600.0, 4620.0, 4640.0, 4660.0, 4680.0, 4700.0, 4720.0, 4740.0, 4760.0, 4780.0,
    4800.0, 4820.0, 4840.0, 4860.0, 4880.0, 4900.0, 4920.0, 4940.0, 4960.0, 4980.0,
    5000.0,
])
TIPS_2017_ISOT[1] = float64([
    1.0, 20.0, 40.0, 60.0, 80.0, 100.0, 120.0, 140.0, 160.0, 180.0,
    200.0, 220.0, 240.0, 260.0, 280.0, 300.0, 320.0, 340.0, 360.0, 380.0,
    400.0, 420.0, 440.0, 460.0, 480.0, 500.0, 520.0, 540.0, 560.0, 580.0,
    600.0, 620.0, 640.0, 660.0, 680.0, 700.0, 720.0, 740.0, 760.0, 780.0,
    800.0, 820.0, 840.0, 860.0, 880.0, 900.0, 920.0, 940.0, 960.0, 980.0,
    1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
    1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
    1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
    1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
    1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
    2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
    2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
    2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
    2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
    2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2900.0, 2920.0, 2940.0, 2960.0, 2980.0,
    3000.0, 3020.0, 3040.0, 3060.0, 3080.0, 3100.0, 3120.0, 3140.0, 3160.0, 3180.0,
    3200.0, 3220.0, 3240.0, 3260.0, 3280.0, 3300.0, 3320.0, 3340.0, 3360.0, 3380.0,
    3400.0, 3420.0, 3440.0, 3460.0, 3480.0, 3500.0, 3520.0, 3540.0, 3560.0, 3580.0,
    3600.0, 3620.0, 3640.0, 3660.0, 3680.0, 3700.0, 3720.0, 3740.0, 3760.0, 3780.0,
    3800.0, 3820.0, 3840.0, 3860.0, 3880.0, 3900.0, 3920.0, 3940.0, 3960.0, 3980.0,
    4000.0, 4020.0, 4040.0, 4060.0, 4080.0, 4100.0, 4120.0, 4140.0, 4160.0, 4180.0,
    4200.0, 4220.0, 4240.0, 4260.0, 4280.0, 4300.0, 4320.0, 4340.0, 4360.0, 4380.0,
    4400.0, 4420.0, 4440.0, 4460.0, 4480.0, 4500.0, 4520.0, 4540.0, 4560.0, 4580.0,
    4600.0, 4620.0, 4640.0, 4660.0, 4680.0, 4700.0, 4720.0, 4740.0, 4760.0, 4780.0,
    4800.0, 4820.0, 4840.0, 4860.0, 4880.0, 4900.0, 4920.0, 4940.0, 4960.0, 4980.0,
    5000.0, 5020.0, 5040.0, 5060.0, 5080.0, 5100.0, 5120.0, 5140.0, 5160.0, 5180.0,
    5200.0, 5220.0, 5240.0, 5260.0, 5280.0, 5300.0, 5320.0, 5340.0, 5360.0, 5380.0,
    5400.0, 5420.0, 5440.0, 5460.0, 5480.0, 5500.0, 5520.0, 5540.0, 5560.0, 5580.0,
    5600.0, 5620.0, 5640.0, 5660.0, 5680.0, 5700.0, 5720.0, 5740.0, 5760.0, 5780.0,
    5800.0, 5820.0, 5840.0, 5860.0, 5880.0, 5900.0, 5920.0, 5940.0, 5960.0, 5980.0,
    6000.0,
])
TIPS_2017_ISOT[2] = float64([
    1.0, 20.0, 40.0, 60.0, 80.0, 100.0, 120.0, 140.0, 160.0, 180.0,
    200.0, 220.0, 240.0, 260.0, 280.0, 300.0, 320.0, 340.0, 360.0, 380.0,
    400.0, 420.0, 440.0, 460.0, 480.0, 500.0, 520.0, 540.0, 560.0, 580.0,
    600.0, 620.0, 640.0, 660.0, 680.0, 700.0, 720.0, 740.0, 760.0, 780.0,
    800.0, 820.0, 840.0, 860.0, 880.0, 900.0, 920.0, 940.0, 960.0, 980.0,
    1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
    1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
    1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
    1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
    1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
    2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
    2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
    2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
    2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
    2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2900.0, 2920.0, 2940.0, 2960.0, 2980.0,
    3000.0, 3020.0, 3040.0, 3060.0, 3080.0, 3100.0, 3120.0, 3140.0, 3160.0, 3180.0,
    3200.0, 3220.0, 3240.0, 3260.0, 3280.0, 3300.0, 3320.0, 3340.0, 3360.0, 3380.0,
    3400.0, 3420.0, 3440.0, 3460.0, 3480.0, 3500.0,
])
TIPS_2017_ISOT[3] = float64([
    1.0, 20.0, 40.0, 60.0, 80.0, 100.0, 120.0, 140.0, 160.0, 180.0,
    200.0, 220.0, 240.0, 260.0, 280.0, 300.0, 320.0, 340.0, 360.0, 380.0,
    400.0, 420.0, 440.0, 460.0, 480.0, 500.0, 520.0, 540.0, 560.0, 580.0,
    600.0, 620.0, 640.0, 660.0, 680.0, 700.0, 720.0, 740.0, 760.0, 780.0,
    800.0, 820.0, 840.0, 860.0, 880.0, 900.0, 920.0, 940.0, 960.0, 980.0,
    1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
    1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
    1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
    1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
    1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
    2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
    2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
    2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
    2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
    2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2900.0, 2920.0, 2940.0, 2960.0, 2980.0,
    3000.0, 3020.0, 3040.0, 3060.0, 3080.0, 3100.0, 3120.0, 3140.0, 3160.0, 3180.0,
    3200.0, 3220.0, 3240.0, 3260.0, 3280.0, 3300.0, 3320.0, 3340.0, 3360.0, 3380.0,
    3400.0, 3420.0, 3440.0, 3460.0, 3480.0, 3500.0, 3520.0, 3540.0, 3560.0, 3580.0,
    3600.0, 3620.0, 3640.0, 3660.0, 3680.0, 3700.0, 3720.0, 3740.0, 3760.0, 3780.0,
    3800.0, 3820.0, 3840.0, 3860.0, 3880.0, 3900.0, 3920.0, 3940.0, 3960.0, 3980.0,
    4000.0, 4020.0, 4040.0, 4060.0, 4080.0, 4100.0, 4120.0, 4140.0, 4160.0, 4180.0,
    4200.0, 4220.0, 4240.0, 4260.0, 4280.0, 4300.0, 4320.0, 4340.0, 4360.0, 4380.0,
    4400.0, 4420.0, 4440.0, 4460.0, 4480.0, 4500.0, 4520.0, 4540.0, 4560.0, 4580.0,
    4600.0, 4620.0, 4640.0, 4660.0, 4680.0, 4700.0, 4720.0, 4740.0, 4760.0, 4780.0,
    4800.0, 4820.0, 4840.0, 4860.0, 4880.0, 4900.0, 4920.0, 4940.0, 4960.0, 4980.0,
    5000.0, 5020.0, 5040.0, 5060.0, 5080.0, 5100.0, 5120.0, 5140.0, 5160.0, 5180.0,
    5200.0, 5220.0, 5240.0, 5260.0, 5280.0, 5300.0, 5320.0, 5340.0, 5360.0, 5380.0,
    5400.0, 5420.0, 5440.0, 5460.0, 5480.0, 5500.0, 5520.0, 5540.0, 5560.0, 5580.0,
    5600.0, 5620.0, 5640.0, 5660.0, 5680.0, 5700.0, 5720.0, 5740.0, 5760.0, 5780.0,
    5800.0, 5820.0, 5840.0, 5860.0, 5880.0, 5900.0, 5920.0, 5940.0, 5960.0, 5980.0,
    6000.0, 6020.0, 6040.0, 6060.0, 6080.0, 6100.0, 6120.0, 6140.0, 6160.0, 6180.0,
    6200.0, 6220.0, 6240.0, 6260.0, 6280.0, 6300.0, 6320.0, 6340.0, 6360.0, 6380.0,
    6400.0, 6420.0, 6440.0, 6460.0, 6480.0, 6500.0, 6520.0, 6540.0, 6560.0, 6580.0,
    6600.0, 6620.0, 6640.0, 6660.0, 6680.0, 6700.0, 6720.0, 6740.0, 6760.0, 6780.0,
    6800.0, 6820.0, 6840.0, 6860.0, 6880.0, 6900.0, 6920.0, 6940.0, 6960.0, 6980.0,
    7000.0, 7020.0, 7040.0, 7060.0, 7080.0, 7100.0, 7120.0, 7140.0, 7160.0, 7180.0,
    7200.0, 7220.0, 7240.0, 7260.0, 7280.0, 7300.0, 7320.0, 7340.0, 7360.0, 7380.0,
    7400.0, 7420.0, 7440.0, 7460.0, 7480.0, 7500.0, 7520.0, 7540.0, 7560.0, 7580.0,
    7600.0, 7620.0, 7640.0, 7660.0, 7680.0, 7700.0, 7720.0, 7740.0, 7760.0, 7780.0,
    7800.0, 7820.0, 7840.0, 7860.0, 7880.0, 7900.0, 7920.0, 7940.0, 7960.0, 7980.0,
    8000.0, 8020.0, 8040.0, 8060.0, 8080.0, 8100.0, 8120.0, 8140.0, 8160.0, 8180.0,
    8200.0, 8220.0, 8240.0, 8260.0, 8280.0, 8300.0, 8320.0, 8340.0, 8360.0, 8380.0,
    8400.0, 8420.0, 8440.0, 8460.0, 8480.0, 8500.0, 8520.0, 8540.0, 8560.0, 8580.0,
    8600.0, 8620.0, 8640.0, 8660.0, 8680.0, 8700.0, 8720.0, 8740.0, 8760.0, 8780.0,
    8800.0, 8820.0, 8840.0, 8860.0, 8880.0, 8900.0, 8920.0, 8940.0, 8960.0, 8980.0,
    9000.0,
])
TIPS_2017_ISOT[4] = float64([
    1.0, 20.0, 40.0, 60.0, 80.0, 100.0, 120.0, 140.0, 160.0, 180.0,
    200.0, 220.0, 240.0, 260.0, 280.0, 300.0, 320.0, 340.0, 360.0, 380.0,
    400.0, 420.0, 440.0, 460.0, 480.0, 500.0, 520.0, 540.0, 560.0, 580.0,
    600.0, 620.0, 640.0, 660.0, 680.0, 700.0, 720.0, 740.0, 760.0, 780.0,
    800.0, 820.0, 840.0, 860.0, 880.0, 900.0, 920.0, 940.0, 960.0, 980.0,
    1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
    1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
    1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
    1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
    1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
    2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
    2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
    2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
    2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
    2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2900.0, 2920.0, 2940.0, 2960.0, 2980.0,
    3000.0, 3020.0, 3040.0, 3060.0, 3080.0, 3100.0, 3120.0, 3140.0, 3160.0, 3180.0,
    3200.0, 3220.0, 3240.0, 3260.0, 3280.0, 3300.0, 3320.0, 3340.0, 3360.0, 3380.0,
    3400.0, 3420.0, 3440.0, 3460.0, 3480.0, 3500.0, 3520.0, 3540.0, 3560.0, 3580.0,
    3600.0, 3620.0, 3640.0, 3660.0, 3680.0, 3700.0, 3720.0, 3740.0, 3760.0, 3780.0,
    3800.0, 3820.0, 3840.0, 3860.0, 3880.0, 3900.0, 3920.0, 3940.0, 3960.0, 3980.0,
    4000.0, 4020.0, 4040.0, 4060.0, 4080.0, 4100.0, 4120.0, 4140.0, 4160.0, 4180.0,
    4200.0, 4220.0, 4240.0, 4260.0, 4280.0, 4300.0, 4320.0, 4340.0, 4360.0, 4380.0,
    4400.0, 4420.0, 4440.0, 4460.0, 4480.0, 4500.0,
])
TIPS_2017_ISOT[5] = float64([
    1.0, 20.0, 40.0, 60.0, 80.0, 100.0, 120.0, 140.0, 160.0, 180.0,
    200.0, 220.0, 240.0, 260.0, 280.0, 300.0, 320.0, 340.0, 360.0, 380.0,
    400.0, 420.0, 440.0, 460.0, 480.0, 500.0, 520.0, 540.0, 560.0, 580.0,
    600.0, 620.0, 640.0, 660.0, 680.0, 700.0, 720.0, 740.0, 760.0, 780.0,
    800.0, 820.0, 840.0, 860.0, 880.0, 900.0, 920.0, 940.0, 960.0, 980.0,
    1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
    1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
    1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
    1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
    1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
    2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
    2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
    2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
    2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
    2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2900.0, 2920.0, 2940.0, 2960.0, 2980.0,
    3000.0, 3020.0, 3040.0, 3060.0, 3080.0, 3100.0, 3120.0, 3140.0, 3160.0, 3180.0,
    3200.0, 3220.0, 3240.0, 3260.0, 3280.0, 3300.0, 3320.0, 3340.0, 3360.0, 3380.0,
    3400.0, 3420.0, 3440.0, 3460.0, 3480.0, 3500.0, 3520.0, 3540.0, 3560.0, 3580.0,
    3600.0, 3620.0, 3640.0, 3660.0, 3680.0, 3700.0, 3720.0, 3740.0, 3760.0, 3780.0,
    3800.0, 3820.0, 3840.0, 3860.0, 3880.0, 3900.0, 3920.0, 3940.0, 3960.0, 3980.0,
    4000.0, 4020.0, 4040.0, 4060.0, 4080.0, 4100.0, 4120.0, 4140.0, 4160.0, 4180.0,
    4200.0, 4220.0, 4240.0, 4260.0, 4280.0, 4300.0, 4320.0, 4340.0, 4360.0, 4380.0,
    4400.0, 4420.0, 4440.0, 4460.0, 4480.0, 4500.0, 4520.0, 4540.0, 4560.0, 4580.0,
    4600.0, 4620.0, 4640.0, 4660.0, 4680.0, 4700.0, 4720.0, 4740.0, 4760.0, 4780.0,
    4800.0, 4820.0, 4840.0, 4860.0, 4880.0, 4900.0, 4920.0, 4940.0, 4960.0, 4980.0,
    5000.0, 5020.0, 5040.0, 5060.0, 5080.0, 5100.0, 5120.0, 5140.0, 5160.0, 5180.0,
    5200.0, 5220.0, 5240.0, 5260.0, 5280.0, 5300.0, 5320.0, 5340.0, 5360.0, 5380.0,
    5400.0, 5420.0, 5440.0, 5460.0, 5480.0, 5500.0, 5520.0, 5540.0, 5560.0, 5580.0,
    5600.0, 5620.0, 5640.0, 5660.0, 5680.0, 5700.0, 5720.0, 5740.0, 5760.0, 5780.0,
    5800.0, 5820.0, 5840.0, 5860.0, 5880.0, 5900.0, 5920.0, 5940.0, 5960.0, 5980.0,
    6000.0, 6020.0, 6040.0, 6060.0, 6080.0, 6100.0, 6120.0, 6140.0, 6160.0, 6180.0,
    6200.0, 6220.0, 6240.0, 6260.0, 6280.0, 6300.0, 6320.0, 6340.0, 6360.0, 6380.0,
    6400.0, 6420.0, 6440.0, 6460.0, 6480.0, 6500.0, 6520.0, 6540.0, 6560.0, 6580.0,
    6600.0, 6620.0, 6640.0, 6660.0, 6680.0, 6700.0, 6720.0, 6740.0, 6760.0, 6780.0,
    6800.0, 6820.0, 6840.0, 6860.0, 6880.0, 6900.0, 6920.0, 6940.0, 6960.0, 6980.0,
    7000.0, 7020.0, 7040.0, 7060.0, 7080.0, 7100.0, 7120.0, 7140.0, 7160.0, 7180.0,
    7200.0, 7220.0, 7240.0, 7260.0, 7280.0, 7300.0, 7320.0, 7340.0, 7360.0, 7380.0,
    7400.0, 7420.0, 7440.0, 7460.0, 7480.0, 7500.0,
])
TIPS_2017_ISOT[6] = float64([
    1.0, 20.0, 40.0, 60.0, 80.0, 100.0, 120.0, 140.0, 160.0, 180.0,
    200.0, 220.0, 240.0, 260.0, 280.0, 300.0, 320.0, 340.0, 360.0, 380.0,
    400.0, 420.0, 440.0, 460.0, 480.0, 500.0, 520.0, 540.0, 560.0, 580.0,
    600.0, 620.0, 640.0, 660.0, 680.0, 700.0, 720.0, 740.0, 760.0, 780.0,
    800.0, 820.0, 840.0, 860.0, 880.0, 900.0, 920.0, 940.0, 960.0, 980.0,
    1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
    1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
    1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
    1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
    1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
    2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
    2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
    2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
    2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
    2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2900.0, 2920.0, 2940.0, 2960.0, 2980.0,
    3000.0, 3020.0, 3040.0, 3060.0, 3080.0, 3100.0, 3120.0, 3140.0, 3160.0, 3180.0,
    3200.0, 3220.0, 3240.0, 3260.0, 3280.0, 3300.0, 3320.0, 3340.0, 3360.0, 3380.0,
    3400.0, 3420.0, 3440.0, 3460.0, 3480.0, 3500.0, 3520.0, 3540.0, 3560.0, 3580.0,
    3600.0, 3620.0, 3640.0, 3660.0, 3680.0, 3700.0, 3720.0, 3740.0, 3760.0, 3780.0,
    3800.0, 3820.0, 3840.0, 3860.0, 3880.0, 3900.0, 3920.0, 3940.0, 3960.0, 3980.0,
    4000.0,
])

#  --------------- TIPS-2017 PARTITION SUMS ----------------------
TIPS_2017_ISOT_HASH = {}
TIPS_2017_ISOQ_HASH = {}

#  ---------------------- M = 1, I = 1 ---------------------------
M = 1
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.000000E+00, 3.348920E+00, 9.416960E+00, 1.681863E+01, 2.543182E+01, 3.515313E+01,
    4.587953E+01, 5.752827E+01, 7.003401E+01, 8.334408E+01, 9.741515E+01, 1.122112E+02,
    1.277022E+02, 1.438633E+02, 1.606747E+02, 1.781206E+02, 1.961892E+02, 2.148721E+02,
    2.341643E+02, 2.540633E+02, 2.745691E+02, 2.956837E+02, 3.174110E+02, 3.397563E+02,
    3.627261E+02, 3.863281E+02, 4.105710E+02, 4.354642E+02, 4.610180E+02, 4.872432E+02,
    5.141514E+02, 5.417546E+02, 5.700656E+02, 5.990973E+02, 6.288636E+02, 6.593787E+02,
    6.906572E+02, 7.227143E+02, 7.555656E+02, 7.892274E+02, 8.237162E+02, 8.590491E+02,
    8.952437E+02, 9.323179E+02, 9.702901E+02, 1.009179E+03, 1.049005E+03, 1.089786E+03,
    1.131543E+03, 1.174297E+03, 1.218069E+03, 1.262880E+03, 1.308752E+03, 1.355706E+03,
    1.403766E+03, 1.452954E+03, 1.503294E+03, 1.554809E+03, 1.607523E+03, 1.661461E+03,
    1.716647E+03, 1.773106E+03, 1.830864E+03, 1.889947E+03, 1.950381E+03, 2.012191E+03,
    2.075405E+03, 2.140051E+03, 2.206155E+03, 2.273745E+03, 2.342850E+03, 2.413498E+03,
    2.485717E+03, 2.559538E+03, 2.634989E+03, 2.712101E+03, 2.790904E+03, 2.871427E+03,
    2.953704E+03, 3.037763E+03, 3.123638E+03, 3.211360E+03, 3.300961E+03, 3.392475E+03,
    3.485933E+03, 3.581371E+03, 3.678821E+03, 3.778318E+03, 3.879895E+03, 3.983589E+03,
    4.089435E+03, 4.197467E+03, 4.307722E+03, 4.420237E+03, 4.535049E+03, 4.652193E+03,
    4.771709E+03, 4.893634E+03, 5.018006E+03, 5.144865E+03, 5.274249E+03, 5.406197E+03,
    5.540751E+03, 5.677950E+03, 5.817835E+03, 5.960448E+03, 6.105829E+03, 6.254022E+03,
    6.405068E+03, 6.559010E+03, 6.715891E+03, 6.875757E+03, 7.038649E+03, 7.204614E+03,
    7.373696E+03, 7.545940E+03, 7.721393E+03, 7.900101E+03, 8.082111E+03, 8.267469E+03,
    8.456223E+03, 8.648422E+03, 8.844114E+03, 9.043348E+03, 9.246173E+03, 9.452640E+03,
    9.662798E+03, 9.876698E+03, 1.009439E+04, 1.031593E+04, 1.054137E+04, 1.077075E+04,
    1.100414E+04, 1.124158E+04, 1.148314E+04, 1.172886E+04, 1.197879E+04, 1.223300E+04,
    1.249154E+04, 1.275447E+04, 1.302184E+04, 1.329370E+04, 1.357012E+04, 1.385116E+04,
    1.413687E+04, 1.442730E+04, 1.472253E+04, 1.502260E+04, 1.532759E+04, 1.563754E+04,
    1.595251E+04, 1.627258E+04, 1.659780E+04, 1.692823E+04, 1.726394E+04, 1.760498E+04,
    1.795142E+04, 1.830332E+04, 1.866075E+04, 1.902377E+04, 1.939243E+04, 1.976682E+04,
    2.014699E+04, 2.053301E+04, 2.092493E+04, 2.132284E+04, 2.172678E+04, 2.213684E+04,
    2.255307E+04, 2.297555E+04, 2.340433E+04, 2.383949E+04, 2.428109E+04, 2.472920E+04,
    2.518388E+04, 2.564522E+04, 2.611326E+04, 2.658809E+04, 2.706977E+04, 2.755836E+04,
    2.805395E+04, 2.855658E+04, 2.906634E+04, 2.958330E+04, 3.010751E+04, 3.063906E+04,
    3.117801E+04, 3.172442E+04, 3.227838E+04, 3.283994E+04, 3.340917E+04, 3.398616E+04,
    3.457096E+04, 3.516364E+04, 3.576428E+04, 3.637294E+04, 3.698969E+04, 3.761461E+04,
    3.824775E+04, 3.888919E+04, 3.953900E+04, 4.019725E+04, 4.086401E+04, 4.153933E+04,
    4.222330E+04, 4.291598E+04, 4.361744E+04, 4.432774E+04, 4.504696E+04, 4.577516E+04,
    4.651241E+04, 4.725878E+04, 4.801433E+04, 4.877912E+04, 4.955323E+04, 5.033673E+04,
    5.112967E+04, 5.193213E+04, 5.274416E+04, 5.356584E+04, 5.439722E+04, 5.523838E+04,
    5.608937E+04, 5.695027E+04, 5.782113E+04, 5.870201E+04, 5.959298E+04, 6.049411E+04,
    6.140545E+04, 6.232706E+04, 6.325900E+04, 6.420135E+04, 6.515415E+04, 6.611746E+04,
    6.709136E+04, 6.807588E+04, 6.907110E+04, 7.007707E+04, 7.109385E+04, 7.212149E+04,
    7.316006E+04, 7.420960E+04, 7.527018E+04, 7.634185E+04, 7.742466E+04, 7.851867E+04,
    7.962393E+04, 8.074050E+04, 8.186842E+04, 8.300775E+04, 8.415854E+04,
])

#  ---------------------- M = 1, I = 2 ---------------------------
M = 1
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.000000E+00, 3.372790E+00, 9.492280E+00, 1.695557E+01, 2.564104E+01, 3.544414E+01,
    4.626091E+01, 5.800788E+01, 7.061916E+01, 8.404163E+01, 9.823160E+01, 1.131528E+02,
    1.287749E+02, 1.450731E+02, 1.620274E+02, 1.796219E+02, 1.978450E+02, 2.166883E+02,
    2.361468E+02, 2.562183E+02, 2.769027E+02, 2.982024E+02, 3.201213E+02, 3.426649E+02,
    3.658398E+02, 3.896541E+02, 4.141165E+02, 4.392366E+02, 4.650249E+02, 4.914924E+02,
    5.186508E+02, 5.465124E+02, 5.750899E+02, 6.043966E+02, 6.344463E+02, 6.652532E+02,
    6.968322E+02, 7.291983E+02, 7.623672E+02, 7.963551E+02, 8.311783E+02, 8.668538E+02,
    9.033990E+02, 9.408315E+02, 9.791695E+02, 1.018432E+03, 1.058637E+03, 1.099804E+03,
    1.141952E+03, 1.185103E+03, 1.229276E+03, 1.274491E+03, 1.320770E+03, 1.368133E+03,
    1.416603E+03, 1.466202E+03, 1.516951E+03, 1.568873E+03, 1.621992E+03, 1.676329E+03,
    1.731908E+03, 1.788753E+03, 1.846888E+03, 1.906337E+03, 1.967124E+03, 2.029275E+03,
    2.092814E+03, 2.157766E+03, 2.224157E+03, 2.292012E+03, 2.361359E+03, 2.432222E+03,
    2.504629E+03, 2.578606E+03, 2.654181E+03, 2.731380E+03, 2.810232E+03, 2.890764E+03,
    2.973005E+03, 3.056982E+03, 3.142725E+03, 3.230262E+03, 3.319623E+03, 3.410837E+03,
    3.503933E+03, 3.598943E+03, 3.695895E+03, 3.794822E+03, 3.895752E+03, 3.998719E+03,
    4.103752E+03, 4.210885E+03, 4.320147E+03, 4.431573E+03, 4.545194E+03, 4.661044E+03,
    4.779155E+03, 4.899561E+03, 5.022295E+03, 5.147392E+03, 5.274886E+03, 5.404811E+03,
    5.537202E+03, 5.672095E+03, 5.809525E+03, 5.949527E+03, 6.092139E+03, 6.237395E+03,
    6.385334E+03, 6.535992E+03, 6.689406E+03, 6.845614E+03, 7.004655E+03, 7.166565E+03,
    7.331385E+03, 7.499153E+03, 7.669907E+03, 7.843689E+03, 8.020537E+03, 8.200491E+03,
    8.383594E+03, 8.569884E+03, 8.759404E+03, 8.952195E+03, 9.148299E+03, 9.347757E+03,
    9.550614E+03, 9.756911E+03, 9.966691E+03, 1.018000E+04, 1.039688E+04, 1.061737E+04,
    1.084152E+04, 1.106938E+04, 1.130099E+04, 1.153639E+04, 1.177564E+04, 1.201877E+04,
    1.226583E+04, 1.251688E+04, 1.277195E+04, 1.303110E+04, 1.329438E+04, 1.356182E+04,
    1.383348E+04, 1.410942E+04, 1.438967E+04, 1.467429E+04, 1.496333E+04, 1.525683E+04,
    1.555486E+04, 1.585745E+04, 1.616467E+04, 1.647655E+04, 1.679316E+04, 1.711455E+04,
    1.744077E+04, 1.777186E+04, 1.810790E+04, 1.844892E+04, 1.879498E+04, 1.914614E+04,
    1.950245E+04, 1.986396E+04, 2.023074E+04, 2.060283E+04, 2.098029E+04, 2.136318E+04,
    2.175156E+04, 2.214547E+04, 2.254498E+04, 2.295015E+04, 2.336103E+04, 2.377768E+04,
    2.420015E+04, 2.462851E+04, 2.506282E+04, 2.550313E+04, 2.594950E+04, 2.640200E+04,
    2.686067E+04, 2.732559E+04, 2.779681E+04, 2.827439E+04, 2.875840E+04, 2.924889E+04,
    2.974592E+04, 3.024956E+04, 3.075987E+04, 3.127691E+04, 3.180074E+04, 3.233143E+04,
    3.286903E+04, 3.341361E+04, 3.396524E+04, 3.452397E+04, 3.508988E+04, 3.566301E+04,
    3.624345E+04, 3.683124E+04, 3.742646E+04, 3.802917E+04, 3.863943E+04, 3.925731E+04,
    3.988288E+04, 4.051619E+04, 4.115732E+04, 4.180633E+04, 4.246328E+04, 4.312824E+04,
    4.380128E+04, 4.448245E+04, 4.517184E+04, 4.586950E+04, 4.657550E+04, 4.728991E+04,
    4.801279E+04, 4.874421E+04, 4.948423E+04, 5.023293E+04, 5.099037E+04, 5.175661E+04,
    5.253172E+04, 5.331578E+04, 5.410884E+04, 5.491098E+04, 5.572226E+04, 5.654275E+04,
    5.737251E+04, 5.821161E+04, 5.906013E+04, 5.991812E+04, 6.078566E+04, 6.166281E+04,
    6.254964E+04, 6.344622E+04, 6.435261E+04, 6.526889E+04, 6.619511E+04, 6.713135E+04,
    6.807767E+04, 6.903414E+04, 7.000083E+04, 7.097781E+04, 7.196513E+04, 7.296288E+04,
    7.397111E+04, 7.498990E+04, 7.601930E+04, 7.705940E+04, 7.811024E+04,
])

#  ---------------------- M = 1, I = 3 ---------------------------
M = 1
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.000000E+00, 2.016901E+01, 5.674024E+01, 1.013453E+02, 1.532533E+02, 2.118401E+02,
    2.764846E+02, 3.466880E+02, 4.220566E+02, 5.022729E+02, 5.870757E+02, 6.762480E+02,
    7.696091E+02, 8.670100E+02, 9.683305E+02, 1.073476E+03, 1.182375E+03, 1.294979E+03,
    1.411255E+03, 1.531189E+03, 1.654781E+03, 1.782042E+03, 1.912994E+03, 2.047669E+03,
    2.186104E+03, 2.328346E+03, 2.474446E+03, 2.624459E+03, 2.778445E+03, 2.936469E+03,
    3.098600E+03, 3.264908E+03, 3.435470E+03, 3.610362E+03, 3.789666E+03, 3.973467E+03,
    4.161852E+03, 4.354909E+03, 4.552733E+03, 4.755418E+03, 4.963062E+03, 5.175767E+03,
    5.393635E+03, 5.616772E+03, 5.845286E+03, 6.079288E+03, 6.318891E+03, 6.564209E+03,
    6.815361E+03, 7.072464E+03, 7.335642E+03, 7.605018E+03, 7.880717E+03, 8.162866E+03,
    8.451596E+03, 8.747038E+03, 9.049324E+03, 9.358591E+03, 9.674973E+03, 9.998611E+03,
    1.032964E+04, 1.066821E+04, 1.101447E+04, 1.136854E+04, 1.173059E+04, 1.210077E+04,
    1.247921E+04, 1.286608E+04, 1.326152E+04, 1.366570E+04, 1.407876E+04, 1.450087E+04,
    1.493219E+04, 1.537288E+04, 1.582310E+04, 1.628301E+04, 1.675279E+04, 1.723259E+04,
    1.772260E+04, 1.822297E+04, 1.873389E+04, 1.925553E+04, 1.978806E+04, 2.033166E+04,
    2.088651E+04, 2.145280E+04, 2.203069E+04, 2.262039E+04, 2.322207E+04, 2.383593E+04,
    2.446215E+04, 2.510092E+04, 2.575244E+04, 2.641690E+04, 2.709449E+04, 2.778543E+04,
    2.848989E+04, 2.920810E+04, 2.994025E+04, 3.068655E+04, 3.144720E+04, 3.222241E+04,
    3.301240E+04, 3.381738E+04, 3.463755E+04, 3.547315E+04, 3.632438E+04, 3.719147E+04,
    3.807464E+04, 3.897411E+04, 3.989012E+04, 4.082287E+04, 4.177262E+04, 4.273959E+04,
    4.372400E+04, 4.472611E+04, 4.574614E+04, 4.678433E+04, 4.784094E+04, 4.891619E+04,
    5.001034E+04, 5.112363E+04, 5.225631E+04, 5.340864E+04, 5.458086E+04, 5.577324E+04,
    5.698602E+04, 5.821948E+04, 5.947386E+04, 6.074944E+04, 6.204648E+04, 6.336525E+04,
    6.470602E+04, 6.606905E+04, 6.745462E+04, 6.886301E+04, 7.029450E+04, 7.174936E+04,
    7.322787E+04, 7.473032E+04, 7.625700E+04, 7.780820E+04, 7.938419E+04, 8.098528E+04,
    8.261176E+04, 8.426392E+04, 8.594206E+04, 8.764648E+04, 8.937748E+04, 9.113537E+04,
    9.292045E+04, 9.473303E+04, 9.657342E+04, 9.844193E+04, 1.003389E+05, 1.022646E+05,
    1.042193E+05, 1.062035E+05, 1.082173E+05, 1.102612E+05, 1.123354E+05, 1.144403E+05,
    1.165763E+05, 1.187435E+05, 1.209425E+05, 1.231734E+05, 1.254367E+05, 1.277326E+05,
    1.300616E+05, 1.324240E+05, 1.348200E+05, 1.372501E+05, 1.397146E+05, 1.422138E+05,
    1.447482E+05, 1.473179E+05, 1.499235E+05, 1.525652E+05, 1.552435E+05, 1.579586E+05,
    1.607109E+05, 1.635008E+05, 1.663287E+05, 1.691948E+05, 1.720997E+05, 1.750436E+05,
    1.780269E+05, 1.810500E+05, 1.841132E+05, 1.872170E+05, 1.903617E+05, 1.935477E+05,
    1.967753E+05, 2.000449E+05, 2.033570E+05, 2.067118E+05, 2.101098E+05, 2.135514E+05,
    2.170369E+05, 2.205667E+05, 2.241412E+05, 2.277608E+05, 2.314260E+05, 2.351369E+05,
    2.388942E+05, 2.426981E+05, 2.465490E+05, 2.504474E+05, 2.543936E+05, 2.583880E+05,
    2.624310E+05, 2.665231E+05, 2.706645E+05, 2.748558E+05, 2.790973E+05, 2.833893E+05,
    2.877324E+05, 2.921269E+05, 2.965732E+05, 3.010717E+05, 3.056228E+05, 3.102268E+05,
    3.148843E+05, 3.195956E+05, 3.243612E+05, 3.291813E+05, 3.340565E+05, 3.389870E+05,
    3.439734E+05, 3.490161E+05, 3.541153E+05, 3.592716E+05, 3.644854E+05, 3.697570E+05,
    3.750868E+05, 3.804753E+05, 3.859229E+05, 3.914300E+05, 3.969969E+05, 4.026240E+05,
    4.083119E+05, 4.140609E+05, 4.198713E+05, 4.257436E+05, 4.316783E+05, 4.376756E+05,
    4.437360E+05, 4.498599E+05, 4.560478E+05, 4.622999E+05, 4.686168E+05,
])

#  ---------------------- M = 1, I = 4 ---------------------------
M = 1
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.000000E+00, 1.766833E+01, 4.570241E+01, 8.177074E+01, 1.242917E+02, 1.724141E+02,
    2.255614E+02, 2.833095E+02, 3.453297E+02, 4.113595E+02, 4.811860E+02, 5.546366E+02,
    6.315735E+02, 7.118902E+02, 7.955081E+02, 8.823735E+02, 9.724548E+02, 1.065740E+03,
    1.162232E+03, 1.261951E+03, 1.364927E+03, 1.471199E+03, 1.580818E+03, 1.693839E+03,
    1.810326E+03, 1.930347E+03, 2.053977E+03, 2.181293E+03, 2.312379E+03, 2.447320E+03,
    2.586208E+03, 2.729136E+03, 2.876201E+03, 3.027504E+03, 3.183148E+03, 3.343240E+03,
    3.507890E+03, 3.677211E+03, 3.851318E+03, 4.030329E+03, 4.214365E+03, 4.403551E+03,
    4.598012E+03, 4.797878E+03, 5.003279E+03, 5.214351E+03, 5.431230E+03, 5.654054E+03,
    5.882966E+03, 6.118108E+03, 6.359628E+03, 6.607674E+03, 6.862397E+03, 7.123950E+03,
    7.392489E+03, 7.668172E+03, 7.951159E+03, 8.241613E+03, 8.539699E+03, 8.845584E+03,
    9.159437E+03, 9.481431E+03, 9.811739E+03, 1.015054E+04, 1.049801E+04, 1.085433E+04,
    1.121969E+04, 1.159426E+04, 1.197825E+04, 1.237183E+04, 1.277521E+04, 1.318858E+04,
    1.361213E+04, 1.404607E+04, 1.449060E+04, 1.494593E+04, 1.541225E+04, 1.588979E+04,
    1.637875E+04, 1.687936E+04, 1.739182E+04, 1.791635E+04, 1.845319E+04, 1.900254E+04,
    1.956465E+04, 2.013974E+04, 2.072804E+04, 2.132979E+04, 2.194522E+04, 2.257457E+04,
    2.321808E+04, 2.387600E+04, 2.454858E+04, 2.523606E+04, 2.593868E+04, 2.665672E+04,
    2.739041E+04, 2.814002E+04, 2.890581E+04, 2.968803E+04, 3.048695E+04, 3.130284E+04,
    3.213596E+04, 3.298659E+04, 3.385498E+04, 3.474142E+04, 3.564619E+04, 3.656955E+04,
    3.751178E+04, 3.847316E+04, 3.945398E+04, 4.045452E+04, 4.147505E+04, 4.251587E+04,
    4.357726E+04, 4.465950E+04, 4.576289E+04, 4.688772E+04, 4.803426E+04, 4.920283E+04,
    5.039369E+04, 5.160716E+04, 5.284352E+04, 5.410306E+04, 5.538609E+04, 5.669288E+04,
    5.802374E+04, 5.937896E+04, 6.075884E+04, 6.216367E+04, 6.359374E+04, 6.504935E+04,
    6.653080E+04, 6.803837E+04, 6.957236E+04, 7.113307E+04, 7.272078E+04, 7.433578E+04,
    7.597838E+04, 7.764885E+04, 7.934749E+04, 8.107459E+04, 8.283042E+04, 8.461529E+04,
    8.642947E+04, 8.827324E+04, 9.014689E+04, 9.205070E+04, 9.398494E+04, 9.594990E+04,
    9.794585E+04, 9.997307E+04, 1.020318E+05, 1.041224E+05, 1.062450E+05, 1.084000E+05,
    1.105876E+05, 1.128080E+05, 1.150616E+05, 1.173485E+05, 1.196691E+05, 1.220236E+05,
    1.244122E+05, 1.268353E+05, 1.292929E+05, 1.317854E+05, 1.343131E+05, 1.368761E+05,
    1.394747E+05, 1.421091E+05, 1.447796E+05, 1.474863E+05, 1.502295E+05, 1.530095E+05,
    1.558264E+05, 1.586804E+05, 1.615718E+05, 1.645008E+05, 1.674676E+05, 1.704723E+05,
    1.735152E+05, 1.765964E+05, 1.797163E+05, 1.828748E+05, 1.860723E+05, 1.893090E+05,
    1.925849E+05, 1.959003E+05, 1.992553E+05, 2.026501E+05, 2.060849E+05, 2.095599E+05,
    2.130751E+05, 2.166308E+05, 2.202271E+05, 2.238641E+05, 2.275421E+05, 2.312610E+05,
    2.350212E+05, 2.388226E+05, 2.426655E+05, 2.465499E+05, 2.504760E+05, 2.544439E+05,
    2.584538E+05, 2.625057E+05, 2.665997E+05, 2.707359E+05, 2.749145E+05, 2.791356E+05,
    2.833992E+05, 2.877055E+05, 2.920544E+05, 2.964462E+05, 3.008809E+05, 3.053586E+05,
    3.098792E+05, 3.144431E+05, 3.190500E+05, 3.237003E+05, 3.283938E+05, 3.331308E+05,
    3.379111E+05, 3.427350E+05, 3.476023E+05, 3.525132E+05, 3.574677E+05, 3.624659E+05,
    3.675078E+05, 3.725934E+05, 3.777227E+05, 3.828958E+05, 3.881127E+05, 3.933734E+05,
    3.986779E+05, 4.040263E+05, 4.094186E+05, 4.148547E+05, 4.203346E+05, 4.258585E+05,
    4.314262E+05, 4.370377E+05, 4.426932E+05, 4.483925E+05, 4.541356E+05, 4.599225E+05,
    4.657532E+05, 4.716277E+05, 4.775460E+05, 4.835080E+05, 4.895137E+05,
])

#  ---------------------- M = 1, I = 5 ---------------------------
M = 1
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.000000E+00, 1.786086E+01, 4.625151E+01, 8.276956E+01, 1.258219E+02, 1.745467E+02,
    2.283599E+02, 2.868320E+02, 3.496303E+02, 4.164889E+02, 4.871921E+02, 5.615653E+02,
    6.394690E+02, 7.207956E+02, 8.054658E+02, 8.934261E+02, 9.846456E+02, 1.079113E+03,
    1.176835E+03, 1.277832E+03, 1.382138E+03, 1.489795E+03, 1.600858E+03, 1.715385E+03,
    1.833446E+03, 1.955112E+03, 2.080462E+03, 2.209579E+03, 2.342549E+03, 2.479463E+03,
    2.620417E+03, 2.765507E+03, 2.914836E+03, 3.068509E+03, 3.226632E+03, 3.389316E+03,
    3.556674E+03, 3.728824E+03, 3.905883E+03, 4.087973E+03, 4.275219E+03, 4.467747E+03,
    4.665685E+03, 4.869166E+03, 5.078323E+03, 5.293293E+03, 5.514213E+03, 5.741224E+03,
    5.974468E+03, 6.214091E+03, 6.460239E+03, 6.713060E+03, 6.972705E+03, 7.239327E+03,
    7.513080E+03, 7.794120E+03, 8.082606E+03, 8.378697E+03, 8.682555E+03, 8.994343E+03,
    9.314226E+03, 9.642370E+03, 9.978944E+03, 1.032412E+04, 1.067806E+04, 1.104095E+04,
    1.141296E+04, 1.179427E+04, 1.218504E+04, 1.258547E+04, 1.299573E+04, 1.341601E+04,
    1.384648E+04, 1.428733E+04, 1.473876E+04, 1.520094E+04, 1.567407E+04, 1.615834E+04,
    1.665393E+04, 1.716105E+04, 1.767988E+04, 1.821063E+04, 1.875349E+04, 1.930866E+04,
    1.987633E+04, 2.045672E+04, 2.105002E+04, 2.165643E+04, 2.227617E+04, 2.290944E+04,
    2.355644E+04, 2.421738E+04, 2.489248E+04, 2.558195E+04, 2.628600E+04, 2.700484E+04,
    2.773869E+04, 2.848776E+04, 2.925227E+04, 3.003245E+04, 3.082850E+04, 3.164066E+04,
    3.246914E+04, 3.331417E+04, 3.417596E+04, 3.505476E+04, 3.595078E+04, 3.686425E+04,
    3.779540E+04, 3.874447E+04, 3.971168E+04, 4.069726E+04, 4.170145E+04, 4.272449E+04,
    4.376661E+04, 4.482804E+04, 4.590903E+04, 4.700981E+04, 4.813063E+04, 4.927172E+04,
    5.043333E+04, 5.161570E+04, 5.281908E+04, 5.404371E+04, 5.528984E+04, 5.655772E+04,
    5.784759E+04, 5.915970E+04, 6.049431E+04, 6.185167E+04, 6.323202E+04, 6.463563E+04,
    6.606275E+04, 6.751363E+04, 6.898853E+04, 7.048770E+04, 7.201142E+04, 7.355993E+04,
    7.513350E+04, 7.673238E+04, 7.835685E+04, 8.000716E+04, 8.168358E+04, 8.338637E+04,
    8.511579E+04, 8.687212E+04, 8.865563E+04, 9.046658E+04, 9.230523E+04, 9.417187E+04,
    9.606675E+04, 9.799016E+04, 9.994237E+04, 1.019236E+05, 1.039343E+05, 1.059745E+05,
    1.080446E+05, 1.101449E+05, 1.122756E+05, 1.144371E+05, 1.166295E+05, 1.188533E+05,
    1.211086E+05, 1.233957E+05, 1.257149E+05, 1.280666E+05, 1.304509E+05, 1.328682E+05,
    1.353187E+05, 1.378027E+05, 1.403206E+05, 1.428725E+05, 1.454589E+05, 1.480799E+05,
    1.507358E+05, 1.534269E+05, 1.561536E+05, 1.589161E+05, 1.617147E+05, 1.645496E+05,
    1.674212E+05, 1.703297E+05, 1.732755E+05, 1.762588E+05, 1.792799E+05, 1.823391E+05,
    1.854366E+05, 1.885729E+05, 1.917480E+05, 1.949625E+05, 1.982164E+05, 2.015102E+05,
    2.048440E+05, 2.082182E+05, 2.116331E+05, 2.150890E+05, 2.185861E+05, 2.221247E+05,
    2.257051E+05, 2.293277E+05, 2.329926E+05, 2.367001E+05, 2.404506E+05, 2.442444E+05,
    2.480816E+05, 2.519626E+05, 2.558877E+05, 2.598572E+05, 2.638713E+05, 2.679303E+05,
    2.720345E+05, 2.761841E+05, 2.803795E+05, 2.846209E+05, 2.889086E+05, 2.932429E+05,
    2.976240E+05, 3.020522E+05, 3.065279E+05, 3.110511E+05, 3.156223E+05, 3.202417E+05,
    3.249096E+05, 3.296262E+05, 3.343917E+05, 3.392066E+05, 3.440709E+05, 3.489851E+05,
    3.539492E+05, 3.589637E+05, 3.640287E+05, 3.691446E+05, 3.743115E+05, 3.795298E+05,
    3.847996E+05, 3.901213E+05, 3.954950E+05, 4.009211E+05, 4.063997E+05, 4.119311E+05,
    4.175156E+05, 4.231535E+05, 4.288448E+05, 4.345899E+05, 4.403891E+05, 4.462425E+05,
    4.521504E+05, 4.581130E+05, 4.641305E+05, 4.702033E+05, 4.763314E+05,
])

#  ---------------------- M = 1, I = 6 ---------------------------
M = 1
I = 6
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.600000E+01, 1.066180E+02, 2.759495E+02, 4.937802E+02, 7.505847E+02, 1.041223E+03,
    1.362217E+03, 1.711020E+03, 2.085674E+03, 2.484623E+03, 2.906617E+03, 3.350642E+03,
    3.815891E+03, 4.301738E+03, 4.807723E+03, 5.333532E+03, 5.878982E+03, 6.444007E+03,
    7.028640E+03, 7.633001E+03, 8.257283E+03, 8.901742E+03, 9.566686E+03, 1.025247E+04,
    1.095949E+04, 1.168818E+04, 1.243899E+04, 1.321241E+04, 1.400896E+04, 1.482918E+04,
    1.567361E+04, 1.654285E+04, 1.743748E+04, 1.835812E+04, 1.930541E+04, 2.027999E+04,
    2.128254E+04, 2.231374E+04, 2.337427E+04, 2.446487E+04, 2.558624E+04, 2.673914E+04,
    2.792432E+04, 2.914254E+04, 3.039459E+04, 3.168126E+04, 3.300335E+04, 3.436168E+04,
    3.575707E+04, 3.719038E+04, 3.866244E+04, 4.017412E+04, 4.172629E+04, 4.331984E+04,
    4.495565E+04, 4.663462E+04, 4.835768E+04, 5.012574E+04, 5.193973E+04, 5.380059E+04,
    5.570927E+04, 5.766673E+04, 5.967393E+04, 6.173186E+04, 6.384149E+04, 6.600381E+04,
    6.821984E+04, 7.049057E+04, 7.281703E+04, 7.520023E+04, 7.764122E+04, 8.014104E+04,
    8.270072E+04, 8.532134E+04, 8.800395E+04, 9.074963E+04, 9.355945E+04, 9.643451E+04,
    9.937589E+04, 1.023847E+05, 1.054620E+05, 1.086090E+05, 1.118268E+05, 1.151165E+05,
    1.184792E+05, 1.219162E+05, 1.254284E+05, 1.290172E+05, 1.326836E+05, 1.364289E+05,
    1.402542E+05, 1.441607E+05, 1.481496E+05, 1.522221E+05, 1.563793E+05, 1.606226E+05,
    1.649532E+05, 1.693722E+05, 1.738809E+05, 1.784805E+05, 1.831723E+05, 1.879575E+05,
    1.928374E+05, 1.978133E+05, 2.028864E+05, 2.080579E+05, 2.133293E+05, 2.187018E+05,
    2.241766E+05, 2.297550E+05, 2.354385E+05, 2.412283E+05, 2.471256E+05, 2.531319E+05,
    2.592485E+05, 2.654767E+05, 2.718178E+05, 2.782733E+05, 2.848444E+05, 2.915325E+05,
    2.983391E+05, 3.052654E+05, 3.123128E+05, 3.194828E+05, 3.267767E+05, 3.341959E+05,
    3.417418E+05, 3.494159E+05, 3.572195E+05, 3.651541E+05, 3.732210E+05, 3.814218E+05,
    3.897578E+05, 3.982305E+05, 4.068414E+05, 4.155918E+05, 4.244833E+05, 4.335172E+05,
    4.426952E+05, 4.520186E+05, 4.614889E+05, 4.711076E+05, 4.808762E+05, 4.907962E+05,
    5.008691E+05, 5.110963E+05, 5.214794E+05, 5.320199E+05, 5.427193E+05, 5.535792E+05,
    5.646009E+05, 5.757862E+05, 5.871364E+05, 5.986532E+05, 6.103381E+05, 6.221926E+05,
    6.342182E+05, 6.464166E+05, 6.587892E+05, 6.713377E+05, 6.840635E+05, 6.969684E+05,
    7.100537E+05, 7.233211E+05, 7.367722E+05, 7.504086E+05, 7.642317E+05, 7.782433E+05,
    7.924449E+05, 8.068380E+05, 8.214243E+05, 8.362053E+05, 8.511827E+05, 8.663581E+05,
    8.817330E+05, 8.973090E+05, 9.130878E+05, 9.290709E+05, 9.452599E+05, 9.616565E+05,
    9.782622E+05, 9.950787E+05, 1.012108E+06, 1.029350E+06, 1.046809E+06, 1.064484E+06,
    1.082379E+06, 1.100493E+06, 1.118830E+06, 1.137390E+06, 1.156176E+06, 1.175188E+06,
    1.194429E+06, 1.213900E+06, 1.233602E+06, 1.253538E+06, 1.273708E+06, 1.294115E+06,
    1.314760E+06, 1.335644E+06, 1.356770E+06, 1.378138E+06, 1.399750E+06, 1.421609E+06,
    1.443715E+06, 1.466070E+06, 1.488676E+06, 1.511534E+06, 1.534646E+06, 1.558013E+06,
    1.581636E+06, 1.605519E+06, 1.629661E+06, 1.654065E+06, 1.678732E+06, 1.703663E+06,
    1.728861E+06, 1.754326E+06, 1.780061E+06, 1.806066E+06, 1.832343E+06, 1.858894E+06,
    1.885720E+06, 1.912822E+06, 1.940203E+06, 1.967863E+06, 1.995805E+06, 2.024028E+06,
    2.052536E+06, 2.081329E+06, 2.110410E+06, 2.139778E+06, 2.169436E+06, 2.199385E+06,
    2.229627E+06, 2.260162E+06, 2.290993E+06, 2.322120E+06, 2.353546E+06, 2.385271E+06,
    2.417296E+06, 2.449624E+06, 2.482255E+06, 2.515191E+06, 2.548433E+06, 2.581982E+06,
    2.615840E+06, 2.650008E+06, 2.684487E+06, 2.719279E+06, 2.754384E+06,
])

#  ---------------------- M = 1, I = 7 ---------------------------
M = 1
I = 7
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.000000E+00, 2.020566E+01, 5.328177E+01, 9.597421E+01, 1.463666E+02, 2.034368E+02,
    2.664953E+02, 3.350360E+02, 4.086708E+02, 4.870968E+02, 5.700805E+02, 6.574474E+02,
    7.490750E+02, 8.448862E+02, 9.448428E+02, 1.048939E+03, 1.157194E+03, 1.269651E+03,
    1.386368E+03, 1.507418E+03, 1.632887E+03, 1.762868E+03, 1.897464E+03, 2.036785E+03,
    2.180948E+03, 2.330078E+03, 2.484305E+03, 2.643764E+03, 2.808599E+03, 2.978957E+03,
    3.154991E+03, 3.336863E+03, 3.524736E+03, 3.718782E+03, 3.919176E+03, 4.126099E+03,
    4.339738E+03, 4.560283E+03, 4.787929E+03, 5.022879E+03, 5.265336E+03, 5.515511E+03,
    5.773617E+03, 6.039873E+03, 6.314501E+03, 6.597728E+03, 6.889787E+03, 7.190910E+03,
    7.501338E+03, 7.821315E+03, 8.151087E+03, 8.490906E+03, 8.841027E+03, 9.201709E+03,
    9.573217E+03, 9.955817E+03, 1.034978E+04, 1.075538E+04, 1.117291E+04, 1.160263E+04,
    1.204485E+04, 1.249984E+04, 1.296792E+04, 1.344937E+04, 1.394451E+04, 1.445364E+04,
    1.497708E+04, 1.551513E+04, 1.606813E+04, 1.663640E+04, 1.722027E+04, 1.782008E+04,
    1.843615E+04, 1.906885E+04, 1.971852E+04, 2.038550E+04, 2.107016E+04, 2.177286E+04,
    2.249397E+04, 2.323386E+04, 2.399290E+04, 2.477148E+04, 2.556999E+04, 2.638881E+04,
    2.722834E+04, 2.808898E+04, 2.897114E+04, 2.987523E+04, 3.080167E+04, 3.175088E+04,
    3.272328E+04, 3.371931E+04, 3.473940E+04, 3.578400E+04, 3.685355E+04, 3.794852E+04,
    3.906935E+04, 4.021651E+04, 4.139047E+04, 4.259171E+04, 4.382070E+04, 4.507794E+04,
    4.636392E+04, 4.767913E+04, 4.902407E+04, 5.039927E+04, 5.180522E+04, 5.324247E+04,
    5.471152E+04, 5.621291E+04, 5.774719E+04, 5.931490E+04, 6.091658E+04, 6.255280E+04,
    6.422412E+04, 6.593111E+04, 6.767435E+04, 6.945442E+04, 7.127190E+04, 7.312739E+04,
    7.502150E+04, 7.695482E+04, 7.892798E+04, 8.094160E+04, 8.299630E+04, 8.509271E+04,
    8.723149E+04, 8.941326E+04, 9.163870E+04, 9.390846E+04, 9.622320E+04, 9.858361E+04,
    1.009904E+05, 1.034441E+05, 1.059457E+05, 1.084956E+05, 1.110947E+05, 1.137436E+05,
    1.164431E+05, 1.191939E+05, 1.219968E+05, 1.248525E+05, 1.277617E+05, 1.307252E+05,
    1.337438E+05, 1.368183E+05, 1.399493E+05, 1.431378E+05, 1.463845E+05, 1.496903E+05,
    1.530558E+05, 1.564820E+05, 1.599697E+05, 1.635197E+05, 1.671329E+05, 1.708101E+05,
    1.745521E+05, 1.783599E+05, 1.822343E+05, 1.861762E+05, 1.901864E+05, 1.942659E+05,
    1.984156E+05, 2.026364E+05, 2.069291E+05, 2.112949E+05, 2.157345E+05, 2.202489E+05,
    2.248391E+05, 2.295060E+05, 2.342506E+05, 2.390739E+05, 2.439768E+05, 2.489604E+05,
    2.540256E+05, 2.591735E+05, 2.644051E+05, 2.697213E+05, 2.751233E+05, 2.806120E+05,
    2.861886E+05, 2.918540E+05, 2.976093E+05, 3.034557E+05, 3.093941E+05, 3.154257E+05,
    3.215515E+05, 3.277727E+05, 3.340904E+05, 3.405057E+05, 3.470196E+05, 3.536334E+05,
    3.603482E+05, 3.671652E+05, 3.740854E+05, 3.811100E+05, 3.882403E+05, 3.954774E+05,
    4.028224E+05, 4.102766E+05, 4.178412E+05, 4.255174E+05, 4.333064E+05, 4.412094E+05,
    4.492276E+05, 4.573623E+05, 4.656147E+05, 4.739861E+05, 4.824777E+05, 4.910907E+05,
    4.998266E+05, 5.086864E+05, 5.176716E+05, 5.267833E+05, 5.360229E+05, 5.453917E+05,
    5.548910E+05, 5.645222E+05, 5.742864E+05, 5.841851E+05, 5.942195E+05, 6.043911E+05,
    6.147011E+05, 6.251509E+05, 6.357419E+05, 6.464753E+05, 6.573526E+05, 6.683752E+05,
    6.795444E+05, 6.908615E+05, 7.023280E+05, 7.139453E+05, 7.257147E+05, 7.376376E+05,
    7.497155E+05, 7.619497E+05, 7.743416E+05, 7.868928E+05, 7.996044E+05, 8.124781E+05,
    8.255152E+05, 8.387171E+05, 8.520852E+05, 8.656211E+05, 8.793261E+05, 8.932017E+05,
    9.072493E+05, 9.214703E+05, 9.358662E+05, 9.504385E+05, 9.651886E+05, 9.801180E+05,
    9.952280E+05, 1.010520E+06, 1.025996E+06, 1.041657E+06, 1.057504E+06, 1.073540E+06,
    1.089765E+06, 1.106181E+06, 1.122789E+06, 1.139591E+06, 1.156589E+06, 1.173783E+06,
    1.191175E+06, 1.208768E+06, 1.226561E+06, 1.244557E+06, 1.262758E+06, 1.281164E+06,
    1.299777E+06, 1.318598E+06, 1.337630E+06, 1.356873E+06, 1.376329E+06, 1.396000E+06,
    1.415886E+06, 1.435990E+06, 1.456312E+06, 1.476855E+06, 1.497619E+06, 1.518607E+06,
    1.539819E+06, 1.561257E+06, 1.582923E+06, 1.604818E+06, 1.626943E+06, 1.649300E+06,
    1.671891E+06, 1.694716E+06, 1.717777E+06, 1.741076E+06, 1.764614E+06, 1.788392E+06,
    1.812412E+06, 1.836675E+06, 1.861184E+06, 1.885938E+06, 1.910939E+06, 1.936190E+06,
    1.961691E+06,
])

#  ---------------------- M = 1, I = 8 ---------------------------
M = 1
I = 8
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.000000E+00, 2.048931E+01, 5.406267E+01, 9.739573E+01, 1.485457E+02, 2.064750E+02,
    2.704833E+02, 3.400571E+02, 4.148027E+02, 4.944135E+02, 5.786535E+02, 6.673477E+02,
    7.603741E+02, 8.576570E+02, 9.591602E+02, 1.064880E+03, 1.174840E+03, 1.289086E+03,
    1.407678E+03, 1.530695E+03, 1.658222E+03, 1.790360E+03, 1.927213E+03, 2.068896E+03,
    2.215529E+03, 2.367241E+03, 2.524165E+03, 2.686441E+03, 2.854217E+03, 3.027645E+03,
    3.206883E+03, 3.392094E+03, 3.583450E+03, 3.781124E+03, 3.985298E+03, 4.196157E+03,
    4.413893E+03, 4.638702E+03, 4.870784E+03, 5.110345E+03, 5.357596E+03, 5.612751E+03,
    5.876031E+03, 6.147658E+03, 6.427862E+03, 6.716875E+03, 7.014934E+03, 7.322280E+03,
    7.639158E+03, 7.965818E+03, 8.302513E+03, 8.649501E+03, 9.007044E+03, 9.375407E+03,
    9.754862E+03, 1.014568E+04, 1.054814E+04, 1.096253E+04, 1.138913E+04, 1.182824E+04,
    1.228015E+04, 1.274515E+04, 1.322355E+04, 1.371567E+04, 1.422181E+04, 1.474229E+04,
    1.527743E+04, 1.582755E+04, 1.639299E+04, 1.697409E+04, 1.757117E+04, 1.818460E+04,
    1.881470E+04, 1.946184E+04, 2.012638E+04, 2.080867E+04, 2.150909E+04, 2.222801E+04,
    2.296581E+04, 2.372286E+04, 2.449955E+04, 2.529629E+04, 2.611346E+04, 2.695146E+04,
    2.781071E+04, 2.869162E+04, 2.959460E+04, 3.052008E+04, 3.146849E+04, 3.244026E+04,
    3.343582E+04, 3.445564E+04, 3.550014E+04, 3.656980E+04, 3.766507E+04, 3.878642E+04,
    3.993432E+04, 4.110925E+04, 4.231169E+04, 4.354213E+04, 4.480107E+04, 4.608901E+04,
    4.740645E+04, 4.875391E+04, 5.013191E+04, 5.154097E+04, 5.298163E+04, 5.445442E+04,
    5.595988E+04, 5.749856E+04, 5.907103E+04, 6.067783E+04, 6.231955E+04, 6.399675E+04,
    6.571001E+04, 6.745993E+04, 6.924709E+04, 7.107210E+04, 7.293556E+04, 7.483809E+04,
    7.678030E+04, 7.876284E+04, 8.078632E+04, 8.285139E+04, 8.495869E+04, 8.710888E+04,
    8.930263E+04, 9.154060E+04, 9.382346E+04, 9.615189E+04, 9.852659E+04, 1.009483E+05,
    1.034176E+05, 1.059353E+05, 1.085021E+05, 1.111187E+05, 1.137859E+05, 1.165043E+05,
    1.192748E+05, 1.220981E+05, 1.249750E+05, 1.279061E+05, 1.308924E+05, 1.339345E+05,
    1.370334E+05, 1.401897E+05, 1.434043E+05, 1.466780E+05, 1.500116E+05, 1.534060E+05,
    1.568620E+05, 1.603804E+05, 1.639621E+05, 1.676080E+05, 1.713189E+05, 1.750957E+05,
    1.789393E+05, 1.828506E+05, 1.868305E+05, 1.908799E+05, 1.949997E+05, 1.991908E+05,
    2.034543E+05, 2.077909E+05, 2.122018E+05, 2.166877E+05, 2.212498E+05, 2.258889E+05,
    2.306061E+05, 2.354024E+05, 2.402787E+05, 2.452361E+05, 2.502755E+05, 2.553980E+05,
    2.606047E+05, 2.658966E+05, 2.712746E+05, 2.767400E+05, 2.822937E+05, 2.879368E+05,
    2.936704E+05, 2.994957E+05, 3.054136E+05, 3.114253E+05, 3.175319E+05, 3.237345E+05,
    3.300344E+05, 3.364325E+05, 3.429300E+05, 3.495282E+05, 3.562281E+05, 3.630310E+05,
    3.699380E+05, 3.769503E+05, 3.840690E+05, 3.912955E+05, 3.986309E+05, 4.060763E+05,
    4.136332E+05, 4.213026E+05, 4.290858E+05, 4.369840E+05, 4.449986E+05, 4.531307E+05,
    4.613817E+05, 4.697528E+05, 4.782453E+05, 4.868605E+05, 4.955996E+05, 5.044641E+05,
    5.134551E+05, 5.225741E+05, 5.318222E+05, 5.412010E+05, 5.507116E+05, 5.603555E+05,
    5.701339E+05, 5.800483E+05, 5.901000E+05, 6.002904E+05, 6.106208E+05, 6.210926E+05,
    6.317072E+05, 6.424660E+05, 6.533704E+05, 6.644218E+05, 6.756215E+05, 6.869711E+05,
    6.984719E+05, 7.101253E+05, 7.219327E+05, 7.338957E+05, 7.460156E+05, 7.582938E+05,
    7.707319E+05, 7.833312E+05, 7.960932E+05, 8.090193E+05, 8.221111E+05, 8.353700E+05,
    8.487974E+05, 8.623948E+05, 8.761637E+05, 8.901056E+05, 9.042219E+05, 9.185141E+05,
    9.329838E+05, 9.476323E+05, 9.624612E+05, 9.774720E+05, 9.926662E+05, 1.008045E+06,
    1.023611E+06, 1.039364E+06, 1.055307E+06, 1.071440E+06, 1.087766E+06, 1.104286E+06,
    1.121001E+06, 1.137913E+06, 1.155024E+06, 1.172334E+06, 1.189846E+06, 1.207561E+06,
    1.225480E+06, 1.243606E+06, 1.261938E+06, 1.280480E+06, 1.299233E+06, 1.318197E+06,
    1.337375E+06, 1.356767E+06, 1.376377E+06, 1.396204E+06, 1.416251E+06, 1.436519E+06,
    1.457010E+06, 1.477725E+06, 1.498665E+06, 1.519833E+06, 1.541229E+06, 1.562855E+06,
    1.584712E+06, 1.606803E+06, 1.629128E+06, 1.651689E+06, 1.674488E+06, 1.697526E+06,
    1.720804E+06, 1.744324E+06, 1.768088E+06, 1.792097E+06, 1.816351E+06, 1.840854E+06,
    1.865606E+06, 1.890609E+06, 1.915864E+06, 1.941372E+06, 1.967136E+06, 1.993156E+06,
    2.019434E+06,
])

#  ---------------------- M = 1, I = 9 ---------------------------
M = 1
I = 9
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.600000E+01, 1.221318E+02, 3.221607E+02, 5.803382E+02, 8.850833E+02, 1.230215E+03,
    1.611562E+03, 2.026065E+03, 2.471377E+03, 2.945671E+03, 3.447538E+03, 3.975930E+03,
    4.530112E+03, 5.109629E+03, 5.714256E+03, 6.343965E+03, 6.998887E+03, 7.679283E+03,
    8.385518E+03, 9.118039E+03, 9.877365E+03, 1.066407E+04, 1.147879E+04, 1.232218E+04,
    1.319497E+04, 1.409792E+04, 1.503180E+04, 1.599746E+04, 1.699576E+04, 1.802760E+04,
    1.909392E+04, 2.019568E+04, 2.133390E+04, 2.250960E+04, 2.372386E+04, 2.497778E+04,
    2.627249E+04, 2.760915E+04, 2.898894E+04, 3.041308E+04, 3.188283E+04, 3.339945E+04,
    3.496424E+04, 3.657853E+04, 3.824366E+04, 3.996102E+04, 4.173201E+04, 4.355806E+04,
    4.544061E+04, 4.738114E+04, 4.938116E+04, 5.144219E+04, 5.356578E+04, 5.575351E+04,
    5.800696E+04, 6.032778E+04, 6.271759E+04, 6.517809E+04, 6.771096E+04, 7.031792E+04,
    7.300073E+04, 7.576115E+04, 7.860099E+04, 8.152207E+04, 8.452624E+04, 8.761536E+04,
    9.079135E+04, 9.405614E+04, 9.741167E+04, 1.008599E+05, 1.044029E+05, 1.080427E+05,
    1.117813E+05, 1.156208E+05, 1.195634E+05, 1.236111E+05, 1.277662E+05, 1.320309E+05,
    1.364074E+05, 1.408980E+05, 1.455049E+05, 1.502304E+05, 1.550770E+05, 1.600471E+05,
    1.651429E+05, 1.703670E+05, 1.757218E+05, 1.812099E+05, 1.868337E+05, 1.925958E+05,
    1.984989E+05, 2.045456E+05, 2.107385E+05, 2.170803E+05, 2.235737E+05, 2.302216E+05,
    2.370267E+05, 2.439918E+05, 2.511198E+05, 2.584135E+05, 2.658760E+05, 2.735102E+05,
    2.813191E+05, 2.893056E+05, 2.974730E+05, 3.058242E+05, 3.143624E+05, 3.230909E+05,
    3.320128E+05, 3.411313E+05, 3.504498E+05, 3.599715E+05, 3.696999E+05, 3.796383E+05,
    3.897902E+05, 4.001590E+05, 4.107482E+05, 4.215615E+05, 4.326023E+05, 4.438744E+05,
    4.553813E+05, 4.671268E+05, 4.791146E+05, 4.913485E+05, 5.038324E+05, 5.165701E+05,
    5.295655E+05, 5.428226E+05, 5.563453E+05, 5.701377E+05, 5.842039E+05, 5.985480E+05,
    6.131740E+05, 6.280863E+05, 6.432891E+05, 6.587866E+05, 6.745832E+05, 6.906833E+05,
    7.070913E+05, 7.238115E+05, 7.408486E+05, 7.582071E+05, 7.758916E+05, 7.939067E+05,
    8.122571E+05, 8.309475E+05, 8.499827E+05, 8.693675E+05, 8.891068E+05, 9.092054E+05,
    9.296684E+05, 9.505008E+05, 9.717075E+05, 9.932938E+05, 1.015265E+06, 1.037625E+06,
    1.060381E+06, 1.083537E+06, 1.107099E+06, 1.131072E+06, 1.155461E+06, 1.180272E+06,
    1.205511E+06, 1.231183E+06, 1.257293E+06, 1.283848E+06, 1.310853E+06, 1.338313E+06,
    1.366236E+06, 1.394625E+06, 1.423488E+06, 1.452831E+06, 1.482659E+06, 1.512978E+06,
    1.543794E+06, 1.575115E+06, 1.606945E+06, 1.639291E+06, 1.672160E+06, 1.705557E+06,
    1.739489E+06, 1.773963E+06, 1.808986E+06, 1.844562E+06, 1.880700E+06, 1.917406E+06,
    1.954686E+06, 1.992547E+06, 2.030996E+06, 2.070041E+06, 2.109686E+06, 2.149941E+06,
    2.190810E+06, 2.232303E+06, 2.274425E+06, 2.317183E+06, 2.360586E+06, 2.404639E+06,
    2.449351E+06, 2.494728E+06, 2.540778E+06, 2.587508E+06, 2.634926E+06, 2.683039E+06,
    2.731855E+06, 2.781380E+06, 2.831623E+06, 2.882592E+06, 2.934293E+06, 2.986735E+06,
    3.039925E+06, 3.093872E+06, 3.148582E+06, 3.204065E+06, 3.260327E+06, 3.317376E+06,
    3.375221E+06, 3.433870E+06, 3.493331E+06, 3.553611E+06, 3.614719E+06, 3.676663E+06,
    3.739451E+06, 3.803092E+06, 3.867593E+06, 3.932963E+06, 3.999210E+06, 4.066343E+06,
    4.134370E+06, 4.203299E+06, 4.273139E+06, 4.343898E+06, 4.415584E+06, 4.488207E+06,
    4.561774E+06, 4.636295E+06, 4.711778E+06, 4.788230E+06, 4.865663E+06, 4.944082E+06,
    5.023498E+06, 5.103920E+06, 5.185355E+06, 5.267812E+06, 5.351301E+06, 5.435830E+06,
    5.521408E+06, 5.608043E+06, 5.695745E+06, 5.784521E+06, 5.874383E+06, 5.965337E+06,
    6.057392E+06, 6.150559E+06, 6.244845E+06, 6.340260E+06, 6.436812E+06, 6.534510E+06,
    6.633364E+06, 6.733382E+06, 6.834573E+06, 6.936945E+06, 7.040509E+06, 7.145273E+06,
    7.251246E+06, 7.358436E+06, 7.466853E+06, 7.576506E+06, 7.687404E+06, 7.799555E+06,
    7.912969E+06, 8.027654E+06, 8.143620E+06, 8.260875E+06, 8.379428E+06, 8.499289E+06,
    8.620465E+06, 8.742967E+06, 8.866803E+06, 8.991981E+06, 9.118511E+06, 9.246401E+06,
    9.375661E+06, 9.506299E+06, 9.638324E+06, 9.771745E+06, 9.906570E+06, 1.004281E+07,
    1.018047E+07, 1.031956E+07, 1.046009E+07, 1.060207E+07, 1.074551E+07, 1.089041E+07,
    1.103679E+07, 1.118465E+07, 1.133400E+07, 1.148485E+07, 1.163721E+07, 1.179108E+07,
    1.194648E+07,
])

#  ---------------------- M = 2, I = 1 ---------------------------
M = 2
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.172300E+00, 1.797937E+01, 3.579152E+01, 5.360487E+01, 7.142001E+01, 8.924629E+01,
    1.071216E+02, 1.251286E+02, 1.433912E+02, 1.620592E+02, 1.812909E+02, 2.012420E+02,
    2.220599E+02, 2.438818E+02, 2.668355E+02, 2.910405E+02, 3.166103E+02, 3.436540E+02,
    3.722775E+02, 4.025853E+02, 4.346809E+02, 4.686683E+02, 5.046524E+02, 5.427394E+02,
    5.830375E+02, 6.256573E+02, 6.707119E+02, 7.183172E+02, 7.685924E+02, 8.216598E+02,
    8.776450E+02, 9.366774E+02, 9.988899E+02, 1.064419E+03, 1.133406E+03, 1.205995E+03,
    1.282335E+03, 1.362579E+03, 1.446883E+03, 1.535408E+03, 1.628322E+03, 1.725793E+03,
    1.827996E+03, 1.935111E+03, 2.047321E+03, 2.164814E+03, 2.287785E+03, 2.416430E+03,
    2.550954E+03, 2.691564E+03, 2.838473E+03, 2.991899E+03, 3.152065E+03, 3.319200E+03,
    3.493538E+03, 3.675317E+03, 3.864782E+03, 4.062182E+03, 4.267774E+03, 4.481819E+03,
    4.704582E+03, 4.936336E+03, 5.177359E+03, 5.427935E+03, 5.688354E+03, 5.958911E+03,
    6.239909E+03, 6.531654E+03, 6.834460E+03, 7.148647E+03, 7.474541E+03, 7.812474E+03,
    8.162786E+03, 8.525820E+03, 8.901928E+03, 9.291468E+03, 9.694803E+03, 1.011231E+04,
    1.054435E+04, 1.099132E+04, 1.145362E+04, 1.193162E+04, 1.242575E+04, 1.293641E+04,
    1.346401E+04, 1.400899E+04, 1.457177E+04, 1.515280E+04, 1.575251E+04, 1.637136E+04,
    1.700982E+04, 1.766834E+04, 1.834740E+04, 1.904748E+04, 1.976907E+04, 2.051267E+04,
    2.127878E+04, 2.206790E+04, 2.288057E+04, 2.371729E+04, 2.457861E+04, 2.546506E+04,
    2.637719E+04, 2.731556E+04, 2.828073E+04, 2.927327E+04, 3.029376E+04, 3.134277E+04,
    3.242091E+04, 3.352878E+04, 3.466698E+04, 3.583614E+04, 3.703686E+04, 3.826979E+04,
    3.953557E+04, 4.083483E+04, 4.216825E+04, 4.353647E+04, 4.494018E+04, 4.638004E+04,
    4.785674E+04, 4.937098E+04, 5.092347E+04, 5.251490E+04, 5.414600E+04, 5.581749E+04,
    5.753010E+04, 5.928458E+04, 6.108167E+04, 6.292213E+04, 6.480672E+04, 6.673621E+04,
    6.871139E+04, 7.073304E+04, 7.280195E+04, 7.491893E+04, 7.708478E+04, 7.930032E+04,
    8.156638E+04, 8.388379E+04, 8.625338E+04, 8.867600E+04, 9.115251E+04, 9.368376E+04,
    9.627062E+04, 9.891397E+04, 1.016147E+05, 1.043737E+05, 1.071918E+05, 1.100700E+05,
    1.130091E+05, 1.160101E+05, 1.190739E+05, 1.222014E+05, 1.253936E+05, 1.286514E+05,
    1.319757E+05, 1.353675E+05, 1.388278E+05, 1.423574E+05, 1.459575E+05, 1.496289E+05,
    1.533726E+05, 1.571897E+05, 1.610810E+05, 1.650477E+05, 1.690906E+05, 1.732109E+05,
    1.774094E+05, 1.816873E+05, 1.860455E+05, 1.904851E+05, 1.950070E+05, 1.996124E+05,
    2.043022E+05, 2.090774E+05, 2.139392E+05, 2.188885E+05, 2.239265E+05, 2.290540E+05,
    2.342723E+05, 2.395823E+05, 2.449850E+05, 2.504817E+05, 2.560732E+05, 2.617607E+05,
    2.675453E+05, 2.734279E+05, 2.794097E+05, 2.854917E+05, 2.916749E+05, 2.979606E+05,
    3.043496E+05, 3.108431E+05, 3.174422E+05, 3.241479E+05, 3.309613E+05, 3.378834E+05,
    3.449153E+05, 3.520581E+05, 3.593129E+05, 3.666806E+05, 3.741624E+05, 3.817594E+05,
    3.894725E+05, 3.973029E+05, 4.052515E+05, 4.133195E+05, 4.215079E+05, 4.298178E+05,
    4.382501E+05, 4.468060E+05, 4.554865E+05, 4.642925E+05, 4.732252E+05, 4.822856E+05,
    4.914747E+05, 5.007935E+05, 5.102430E+05, 5.198243E+05, 5.295383E+05, 5.393861E+05,
    5.493687E+05, 5.594870E+05, 5.697421E+05, 5.801349E+05, 5.906665E+05, 6.013377E+05,
    6.121497E+05, 6.231032E+05, 6.341994E+05, 6.454391E+05, 6.568232E+05, 6.683529E+05,
    6.800289E+05, 6.918522E+05, 7.038237E+05, 7.159444E+05, 7.282151E+05, 7.406368E+05,
    7.532103E+05, 7.659366E+05, 7.788165E+05, 7.918509E+05, 8.050406E+05, 8.183866E+05,
    8.318897E+05, 8.455507E+05, 8.593704E+05, 8.733498E+05, 8.874895E+05,
])

#  ---------------------- M = 2, I = 2 ---------------------------
M = 2
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.344550E+00, 3.595709E+01, 7.157976E+01, 1.072048E+02, 1.428342E+02, 1.784919E+02,
    2.142697E+02, 2.503579E+02, 2.870319E+02, 3.246141E+02, 3.634386E+02, 4.038283E+02,
    4.460854E+02, 4.904894E+02, 5.372995E+02, 5.867589E+02, 6.390984E+02, 6.945407E+02,
    7.533034E+02, 8.156015E+02, 8.816500E+02, 9.516650E+02, 1.025865E+03, 1.104474E+03,
    1.187719E+03, 1.275832E+03, 1.369052E+03, 1.467626E+03, 1.571805E+03, 1.681848E+03,
    1.798023E+03, 1.920605E+03, 2.049877E+03, 2.186128E+03, 2.329660E+03, 2.480780E+03,
    2.639806E+03, 2.807062E+03, 2.982884E+03, 3.167617E+03, 3.361614E+03, 3.565239E+03,
    3.778863E+03, 4.002871E+03, 4.237655E+03, 4.483619E+03, 4.741175E+03, 5.010747E+03,
    5.292769E+03, 5.587687E+03, 5.895957E+03, 6.218045E+03, 6.554429E+03, 6.905599E+03,
    7.272055E+03, 7.654309E+03, 8.052885E+03, 8.468319E+03, 8.901159E+03, 9.351963E+03,
    9.821303E+03, 1.030976E+04, 1.081794E+04, 1.134645E+04, 1.189590E+04, 1.246693E+04,
    1.306020E+04, 1.367635E+04, 1.431607E+04, 1.498004E+04, 1.566896E+04, 1.638355E+04,
    1.712453E+04, 1.789264E+04, 1.868864E+04, 1.951329E+04, 2.036739E+04, 2.125172E+04,
    2.216710E+04, 2.311436E+04, 2.409433E+04, 2.510787E+04, 2.615584E+04, 2.723914E+04,
    2.835866E+04, 2.951531E+04, 3.071003E+04, 3.194376E+04, 3.321745E+04, 3.453208E+04,
    3.588864E+04, 3.728814E+04, 3.873160E+04, 4.022005E+04, 4.175454E+04, 4.333615E+04,
    4.496596E+04, 4.664506E+04, 4.837457E+04, 5.015563E+04, 5.198938E+04, 5.387698E+04,
    5.581962E+04, 5.781850E+04, 5.987482E+04, 6.198981E+04, 6.416473E+04, 6.640083E+04,
    6.869940E+04, 7.106172E+04, 7.348912E+04, 7.598292E+04, 7.854447E+04, 8.117512E+04,
    8.387627E+04, 8.664930E+04, 8.949564E+04, 9.241670E+04, 9.541394E+04, 9.848883E+04,
    1.016428E+05, 1.048775E+05, 1.081942E+05, 1.115947E+05, 1.150803E+05, 1.186528E+05,
    1.223136E+05, 1.260644E+05, 1.299067E+05, 1.338423E+05, 1.378728E+05, 1.419997E+05,
    1.462249E+05, 1.505501E+05, 1.549768E+05, 1.595069E+05, 1.641422E+05, 1.688843E+05,
    1.737350E+05, 1.786963E+05, 1.837698E+05, 1.889574E+05, 1.942609E+05, 1.996823E+05,
    2.052233E+05, 2.108859E+05, 2.166720E+05, 2.225835E+05, 2.286223E+05, 2.347904E+05,
    2.410897E+05, 2.475223E+05, 2.540900E+05, 2.607950E+05, 2.676393E+05, 2.746248E+05,
    2.817536E+05, 2.890278E+05, 2.964495E+05, 3.040207E+05, 3.117435E+05, 3.196200E+05,
    3.276524E+05, 3.358427E+05, 3.441932E+05, 3.527060E+05, 3.613832E+05, 3.702270E+05,
    3.792395E+05, 3.884231E+05, 3.977798E+05, 4.073119E+05, 4.170216E+05, 4.269111E+05,
    4.369827E+05, 4.472386E+05, 4.576810E+05, 4.683123E+05, 4.791346E+05, 4.901502E+05,
    5.013614E+05, 5.127705E+05, 5.243797E+05, 5.361914E+05, 5.482078E+05, 5.604312E+05,
    5.728640E+05, 5.855084E+05, 5.983667E+05, 6.114413E+05, 6.247344E+05, 6.382484E+05,
    6.519855E+05, 6.659481E+05, 6.801385E+05, 6.945590E+05, 7.092119E+05, 7.240996E+05,
    7.392243E+05, 7.545883E+05, 7.701940E+05, 7.860436E+05, 8.021395E+05, 8.184840E+05,
    8.350793E+05, 8.519277E+05, 8.690316E+05, 8.863932E+05, 9.040148E+05, 9.218986E+05,
    9.400470E+05, 9.584622E+05, 9.771464E+05, 9.961019E+05, 1.015331E+06, 1.034836E+06,
    1.054619E+06, 1.074682E+06, 1.095027E+06, 1.115657E+06, 1.136574E+06, 1.157780E+06,
    1.179277E+06, 1.201067E+06, 1.223153E+06, 1.245537E+06, 1.268220E+06, 1.291205E+06,
    1.314494E+06, 1.338089E+06, 1.361993E+06, 1.386206E+06, 1.410732E+06, 1.435573E+06,
    1.460729E+06, 1.486204E+06, 1.512000E+06, 1.538118E+06, 1.564560E+06, 1.591328E+06,
    1.618425E+06, 1.645852E+06, 1.673611E+06, 1.701705E+06, 1.730134E+06, 1.758901E+06,
    1.788007E+06, 1.817455E+06, 1.847247E+06, 1.877383E+06, 1.907867E+06,
])

#  ---------------------- M = 2, I = 3 ---------------------------
M = 2
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.260550E+00, 3.809072E+01, 7.584727E+01, 1.136063E+02, 1.513692E+02, 1.891574E+02,
    2.270552E+02, 2.652444E+02, 3.039959E+02, 3.436341E+02, 3.845016E+02, 4.269345E+02,
    4.712510E+02, 5.177476E+02, 5.667010E+02, 6.183711E+02, 6.730046E+02, 7.308391E+02,
    7.921059E+02, 8.570332E+02, 9.258477E+02, 9.987770E+02, 1.076051E+03, 1.157902E+03,
    1.244567E+03, 1.336290E+03, 1.433318E+03, 1.535906E+03, 1.644315E+03, 1.758814E+03,
    1.879680E+03, 2.007196E+03, 2.141654E+03, 2.283356E+03, 2.432611E+03, 2.589737E+03,
    2.755060E+03, 2.928918E+03, 3.111655E+03, 3.303627E+03, 3.505199E+03, 3.716746E+03,
    3.938651E+03, 4.171310E+03, 4.415128E+03, 4.670521E+03, 4.937913E+03, 5.217743E+03,
    5.510457E+03, 5.816514E+03, 6.136385E+03, 6.470548E+03, 6.819498E+03, 7.183738E+03,
    7.563783E+03, 7.960160E+03, 8.373410E+03, 8.804082E+03, 9.252740E+03, 9.719961E+03,
    1.020633E+04, 1.071245E+04, 1.123894E+04, 1.178641E+04, 1.235551E+04, 1.294689E+04,
    1.356122E+04, 1.419917E+04, 1.486143E+04, 1.554872E+04, 1.626173E+04, 1.700122E+04,
    1.776792E+04, 1.856258E+04, 1.938600E+04, 2.023894E+04, 2.112221E+04, 2.203662E+04,
    2.298300E+04, 2.396219E+04, 2.497505E+04, 2.602245E+04, 2.710526E+04, 2.822438E+04,
    2.938072E+04, 3.057522E+04, 3.180880E+04, 3.308241E+04, 3.439702E+04, 3.575361E+04,
    3.715316E+04, 3.859669E+04, 4.008520E+04, 4.161973E+04, 4.320132E+04, 4.483101E+04,
    4.650988E+04, 4.823901E+04, 5.001948E+04, 5.185240E+04, 5.373887E+04, 5.568001E+04,
    5.767697E+04, 5.973088E+04, 6.184289E+04, 6.401416E+04, 6.624588E+04, 6.853920E+04,
    7.089533E+04, 7.331546E+04, 7.580078E+04, 7.835251E+04, 8.097186E+04, 8.366006E+04,
    8.641834E+04, 8.924792E+04, 9.215004E+04, 9.512594E+04, 9.817687E+04, 1.013041E+05,
    1.045088E+05, 1.077923E+05, 1.111559E+05, 1.146007E+05, 1.181281E+05, 1.217393E+05,
    1.254356E+05, 1.292182E+05, 1.330883E+05, 1.370473E+05, 1.410964E+05, 1.452368E+05,
    1.494697E+05, 1.537965E+05, 1.582183E+05, 1.627364E+05, 1.673520E+05, 1.720663E+05,
    1.768805E+05, 1.817959E+05, 1.868137E+05, 1.919349E+05, 1.971610E+05, 2.024929E+05,
    2.079318E+05, 2.134791E+05, 2.191356E+05, 2.249027E+05, 2.307815E+05, 2.367730E+05,
    2.428784E+05, 2.490987E+05, 2.554351E+05, 2.618886E+05, 2.684603E+05, 2.751512E+05,
    2.819624E+05, 2.888948E+05, 2.959496E+05, 3.031276E+05, 3.104298E+05, 3.178573E+05,
    3.254109E+05, 3.330916E+05, 3.409004E+05, 3.488380E+05, 3.569055E+05, 3.651036E+05,
    3.734333E+05, 3.818953E+05, 3.904905E+05, 3.992197E+05, 4.080837E+05, 4.170833E+05,
    4.262192E+05, 4.354922E+05,
])

#  ---------------------- M = 2, I = 4 ---------------------------
M = 2
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.325894E+01, 2.223057E+02, 4.426057E+02, 6.629202E+02, 8.832573E+02, 1.103737E+03,
    1.324841E+03, 1.547611E+03, 1.773607E+03, 2.004700E+03, 2.242871E+03, 2.490065E+03,
    2.748119E+03, 3.018750E+03, 3.303554E+03, 3.604030E+03, 3.921599E+03, 4.257630E+03,
    4.613454E+03, 4.990381E+03, 5.389715E+03, 5.812764E+03, 6.260847E+03, 6.735300E+03,
    7.237487E+03, 7.768800E+03, 8.330662E+03, 8.924535E+03, 9.551919E+03, 1.021436E+04,
    1.091343E+04, 1.165077E+04, 1.242805E+04, 1.324700E+04, 1.410938E+04, 1.501703E+04,
    1.597182E+04, 1.697567E+04, 1.803057E+04, 1.913855E+04, 2.030170E+04, 2.152217E+04,
    2.280215E+04, 2.414392E+04, 2.554979E+04, 2.702215E+04, 2.856342E+04, 3.017611E+04,
    3.186279E+04, 3.362608E+04, 3.546867E+04, 3.739332E+04, 3.940283E+04, 4.150010E+04,
    4.368808E+04, 4.596979E+04, 4.834831E+04, 5.082679E+04, 5.340848E+04, 5.609665E+04,
    5.889468E+04, 6.180600E+04, 6.483413E+04, 6.798265E+04, 7.125521E+04, 7.465556E+04,
    7.818749E+04, 8.185488E+04, 8.566170E+04, 8.961199E+04, 9.370984E+04, 9.795946E+04,
    1.023651E+05, 1.069311E+05, 1.116619E+05, 1.165621E+05, 1.216361E+05, 1.268886E+05,
    1.323244E+05, 1.379484E+05, 1.437653E+05, 1.497802E+05, 1.559982E+05, 1.624244E+05,
    1.690639E+05, 1.759222E+05, 1.830045E+05, 1.903164E+05, 1.978633E+05, 2.056509E+05,
    2.136848E+05, 2.219708E+05, 2.305147E+05, 2.393225E+05, 2.484001E+05, 2.577535E+05,
    2.673889E+05, 2.773125E+05, 2.875304E+05, 2.980491E+05, 3.088749E+05, 3.200142E+05,
    3.314736E+05, 3.432596E+05, 3.553788E+05, 3.678379E+05, 3.806436E+05, 3.938026E+05,
    4.073219E+05, 4.212082E+05, 4.354685E+05, 4.501096E+05, 4.651387E+05, 4.805627E+05,
    4.963887E+05, 5.126236E+05, 5.292748E+05, 5.463492E+05, 5.638540E+05, 5.817965E+05,
    6.001837E+05, 6.190230E+05, 6.383214E+05, 6.580863E+05, 6.783248E+05, 6.990442E+05,
    7.202517E+05, 7.419545E+05, 7.641599E+05, 7.868749E+05, 8.101069E+05, 8.338629E+05,
    8.581502E+05, 8.829758E+05, 9.083470E+05, 9.342706E+05, 9.607539E+05, 9.878038E+05,
    1.015427E+06, 1.043631E+06, 1.072423E+06, 1.101809E+06, 1.131795E+06, 1.162390E+06,
    1.193600E+06, 1.225430E+06, 1.257889E+06, 1.290982E+06, 1.324715E+06, 1.359096E+06,
    1.394131E+06, 1.429826E+06, 1.466187E+06, 1.503220E+06, 1.540932E+06, 1.579328E+06,
    1.618415E+06, 1.658198E+06, 1.698683E+06, 1.739876E+06, 1.781782E+06, 1.824407E+06,
    1.867756E+06, 1.911835E+06, 1.956650E+06, 2.002204E+06, 2.048503E+06, 2.095553E+06,
    2.143358E+06, 2.191923E+06, 2.241253E+06, 2.291353E+06, 2.342226E+06, 2.393878E+06,
    2.446312E+06, 2.499534E+06,
])

#  ---------------------- M = 2, I = 5 ---------------------------
M = 2
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.521080E+00, 7.618089E+01, 1.516935E+02, 2.272111E+02, 3.027382E+02, 3.783295E+02,
    4.541900E+02, 5.307384E+02, 6.085758E+02, 6.884015E+02, 7.709370E+02, 8.568792E+02,
    9.468802E+02, 1.041544E+03, 1.141433E+03, 1.247075E+03, 1.358972E+03, 1.477611E+03,
    1.603466E+03, 1.737009E+03, 1.878710E+03, 2.029043E+03, 2.188489E+03, 2.357536E+03,
    2.536684E+03, 2.726445E+03, 2.927343E+03, 3.139917E+03, 3.364724E+03, 3.602332E+03,
    3.853330E+03, 4.118322E+03, 4.397931E+03, 4.692798E+03, 5.003582E+03, 5.330962E+03,
    5.675636E+03, 6.038323E+03, 6.419760E+03, 6.820707E+03, 7.241943E+03, 7.684269E+03,
    8.148509E+03, 8.635506E+03, 9.146128E+03, 9.681264E+03, 1.024183E+04, 1.082875E+04,
    1.144299E+04, 1.208554E+04, 1.275739E+04, 1.345959E+04, 1.419317E+04, 1.495924E+04,
    1.575889E+04, 1.659324E+04, 1.746347E+04, 1.837075E+04, 1.931629E+04, 2.030133E+04,
    2.132713E+04, 2.239497E+04, 2.350618E+04, 2.466211E+04, 2.586412E+04, 2.711361E+04,
    2.841201E+04, 2.976079E+04, 3.116142E+04, 3.261543E+04, 3.412435E+04, 3.568975E+04,
    3.731326E+04, 3.899648E+04, 4.074109E+04, 4.254878E+04, 4.442127E+04, 4.636031E+04,
    4.836769E+04, 5.044520E+04, 5.259471E+04, 5.481807E+04, 5.711720E+04, 5.949401E+04,
    6.195048E+04, 6.448859E+04, 6.711037E+04, 6.981786E+04, 7.261315E+04, 7.549834E+04,
    7.847557E+04, 8.154701E+04, 8.471484E+04, 8.798130E+04, 9.134862E+04, 9.481909E+04,
    9.839500E+04, 1.020787E+05, 1.058725E+05, 1.097788E+05, 1.138000E+05, 1.179385E+05,
    1.221969E+05, 1.265774E+05, 1.310827E+05, 1.357152E+05, 1.404776E+05, 1.453722E+05,
    1.504017E+05, 1.555688E+05, 1.608759E+05, 1.663257E+05, 1.719208E+05, 1.776640E+05,
    1.835577E+05, 1.896048E+05, 1.958079E+05, 2.021696E+05, 2.086927E+05, 2.153799E+05,
    2.222339E+05, 2.292574E+05, 2.364532E+05, 2.438239E+05, 2.513722E+05, 2.591010E+05,
    2.670130E+05, 2.751108E+05, 2.833972E+05, 2.918749E+05, 3.005467E+05, 3.094152E+05,
    3.184831E+05, 3.277532E+05, 3.372281E+05, 3.469105E+05, 3.568031E+05, 3.669086E+05,
    3.772295E+05, 3.877685E+05, 3.985282E+05, 4.095113E+05, 4.207202E+05, 4.321576E+05,
    4.438260E+05, 4.557279E+05, 4.678659E+05, 4.802424E+05, 4.928599E+05, 5.057207E+05,
    5.188275E+05, 5.321824E+05, 5.457879E+05, 5.596464E+05, 5.737601E+05, 5.881313E+05,
    6.027624E+05, 6.176554E+05, 6.328126E+05, 6.482363E+05, 6.639284E+05, 6.798912E+05,
    6.961267E+05, 7.126369E+05, 7.294239E+05, 7.464896E+05, 7.638361E+05, 7.814651E+05,
    7.993786E+05, 8.175784E+05, 8.360663E+05, 8.548442E+05, 8.739137E+05, 8.932765E+05,
    9.129343E+05, 9.328888E+05,
])

#  ---------------------- M = 2, I = 6 ---------------------------
M = 2
I = 6
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.651606E+01, 4.445925E+02, 8.851739E+02, 1.325785E+03, 1.766449E+03, 2.207477E+03,
    2.650035E+03, 3.096522E+03, 3.550398E+03, 4.015699E+03, 4.496597E+03, 4.997122E+03,
    5.521046E+03, 6.071862E+03, 6.652812E+03, 7.266940E+03, 7.917143E+03, 8.606215E+03,
    9.336891E+03, 1.011188E+04, 1.093387E+04, 1.180560E+04, 1.272982E+04, 1.370933E+04,
    1.474700E+04, 1.584577E+04, 1.700865E+04, 1.823873E+04, 1.953919E+04, 2.091330E+04,
    2.236443E+04, 2.389604E+04, 2.551171E+04, 2.721511E+04, 2.901001E+04, 3.090030E+04,
    3.288998E+04, 3.498317E+04, 3.718410E+04, 3.949710E+04, 4.192665E+04, 4.447733E+04,
    4.715386E+04, 4.996106E+04, 5.290390E+04, 5.598748E+04, 5.921700E+04, 6.259783E+04,
    6.613544E+04, 6.983547E+04, 7.370367E+04, 7.774594E+04, 8.196832E+04, 8.637699E+04,
    9.097828E+04, 9.577865E+04, 1.007847E+05, 1.060033E+05, 1.114413E+05, 1.171057E+05,
    1.230038E+05, 1.291430E+05, 1.355307E+05, 1.421748E+05, 1.490831E+05, 1.562635E+05,
    1.637242E+05, 1.714737E+05, 1.795203E+05, 1.878728E+05, 1.965400E+05, 2.055309E+05,
    2.148547E+05, 2.245208E+05, 2.345385E+05, 2.449177E+05, 2.556682E+05, 2.668000E+05,
    2.783233E+05, 2.902486E+05, 3.025862E+05, 3.153471E+05, 3.285420E+05, 3.421821E+05,
    3.562785E+05, 3.708427E+05, 3.858862E+05, 4.014209E+05, 4.174586E+05, 4.340114E+05,
    4.510916E+05, 4.687115E+05, 4.868838E+05, 5.056212E+05, 5.249365E+05, 5.448429E+05,
    5.653535E+05, 5.864817E+05, 6.082409E+05, 6.306448E+05, 6.537072E+05, 6.774420E+05,
    7.018632E+05, 7.269850E+05, 7.528217E+05, 7.793877E+05, 8.066975E+05, 8.347658E+05,
    8.636073E+05, 8.932368E+05, 9.236693E+05, 9.549198E+05, 9.870035E+05, 1.019935E+06,
    1.053731E+06, 1.088405E+06, 1.123974E+06, 1.160452E+06, 1.197856E+06, 1.236200E+06,
    1.275501E+06, 1.315774E+06, 1.357034E+06, 1.399297E+06, 1.442580E+06, 1.486897E+06,
    1.532264E+06, 1.578697E+06, 1.626211E+06, 1.674823E+06, 1.724548E+06, 1.775400E+06,
    1.827397E+06, 1.880553E+06, 1.934884E+06, 1.990405E+06, 2.047132E+06, 2.105079E+06,
    2.164262E+06, 2.224697E+06, 2.286397E+06, 2.349379E+06, 2.413656E+06, 2.479244E+06,
    2.546157E+06, 2.614410E+06, 2.684016E+06, 2.754992E+06, 2.827349E+06, 2.901104E+06,
    2.976268E+06, 3.052857E+06, 3.130883E+06, 3.210361E+06, 3.291303E+06, 3.373723E+06,
    3.457633E+06, 3.543047E+06, 3.629976E+06, 3.718434E+06, 3.808433E+06, 3.899985E+06,
    3.993102E+06, 4.087795E+06, 4.184076E+06, 4.281956E+06, 4.381448E+06, 4.482561E+06,
    4.585306E+06, 4.689695E+06, 4.795737E+06, 4.903442E+06, 5.012821E+06, 5.123884E+06,
    5.236639E+06, 5.351097E+06,
])

#  ---------------------- M = 2, I = 7 ---------------------------
M = 2
I = 7
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.250890E+00, 2.020899E+01, 4.025104E+01, 6.029436E+01, 8.033986E+01, 1.003996E+02,
    1.205209E+02, 1.408040E+02, 1.613971E+02, 1.824767E+02, 2.042283E+02, 2.268340E+02,
    2.504658E+02, 2.752848E+02, 3.014409E+02, 3.290757E+02, 3.583237E+02, 3.893145E+02,
    4.221748E+02, 4.570293E+02, 4.940023E+02, 5.332187E+02, 5.748044E+02, 6.188873E+02,
    6.655979E+02, 7.150694E+02, 7.674382E+02, 8.228445E+02, 8.814321E+02, 9.433489E+02,
    1.008747E+03, 1.077783E+03, 1.150618E+03, 1.227417E+03, 1.308352E+03, 1.393596E+03,
    1.483332E+03, 1.577744E+03, 1.677024E+03, 1.781366E+03, 1.890973E+03, 2.006051E+03,
    2.126813E+03, 2.253477E+03, 2.386266E+03, 2.525410E+03, 2.671145E+03, 2.823711E+03,
    2.983356E+03, 3.150333E+03, 3.324902E+03, 3.507329E+03, 3.697886E+03, 3.896851E+03,
    4.104509E+03, 4.321152E+03, 4.547077E+03, 4.782590E+03, 5.028001E+03, 5.283630E+03,
    5.549800E+03, 5.826843E+03, 6.115099E+03, 6.414913E+03, 6.726639E+03, 7.050636E+03,
    7.387272E+03, 7.736920E+03, 8.099964E+03, 8.476792E+03, 8.867801E+03, 9.273394E+03,
    9.693984E+03, 1.012999E+04, 1.058184E+04, 1.104996E+04, 1.153481E+04, 1.203682E+04,
    1.255646E+04, 1.309419E+04, 1.365049E+04, 1.422584E+04, 1.482072E+04, 1.543563E+04,
    1.607109E+04, 1.672760E+04, 1.740569E+04, 1.810588E+04, 1.882871E+04, 1.957473E+04,
    2.034450E+04, 2.113858E+04, 2.195754E+04, 2.280195E+04, 2.367242E+04, 2.456953E+04,
    2.549389E+04, 2.644612E+04, 2.742683E+04, 2.843666E+04, 2.947625E+04, 3.054624E+04,
    3.164730E+04, 3.278009E+04, 3.394529E+04, 3.514357E+04, 3.637564E+04, 3.764218E+04,
    3.894392E+04, 4.028157E+04, 4.165585E+04, 4.306751E+04, 4.451729E+04, 4.600595E+04,
    4.753424E+04, 4.910294E+04, 5.071284E+04, 5.236471E+04, 5.405937E+04, 5.579761E+04,
    5.758027E+04, 5.940815E+04, 6.128210E+04, 6.320296E+04, 6.517158E+04, 6.718883E+04,
    6.925557E+04, 7.137269E+04, 7.354107E+04, 7.576161E+04, 7.803522E+04, 8.036280E+04,
    8.274529E+04, 8.518362E+04, 8.767872E+04, 9.023154E+04, 9.284305E+04, 9.551420E+04,
    9.824598E+04, 1.010394E+05, 1.038953E+05, 1.068149E+05, 1.097991E+05, 1.128489E+05,
    1.159653E+05, 1.191494E+05, 1.224023E+05, 1.257249E+05, 1.291182E+05, 1.325835E+05,
    1.361217E+05, 1.397340E+05, 1.434213E+05, 1.471849E+05, 1.510257E+05, 1.549449E+05,
    1.589437E+05, 1.630231E+05, 1.671842E+05, 1.714282E+05, 1.757562E+05, 1.801694E+05,
    1.846689E+05, 1.892559E+05, 1.939315E+05, 1.986969E+05, 2.035533E+05, 2.085017E+05,
    2.135436E+05, 2.186799E+05, 2.239119E+05, 2.292408E+05, 2.346678E+05, 2.401941E+05,
    2.458208E+05, 2.515493E+05, 2.573807E+05, 2.633163E+05, 2.693572E+05, 2.755047E+05,
    2.817600E+05, 2.881243E+05, 2.945989E+05, 3.011850E+05, 3.078838E+05, 3.146966E+05,
    3.216246E+05, 3.286691E+05, 3.358312E+05, 3.431123E+05, 3.505135E+05, 3.580362E+05,
    3.656815E+05, 3.734508E+05, 3.813451E+05, 3.893659E+05, 3.975143E+05, 4.057916E+05,
    4.141989E+05, 4.227377E+05, 4.314090E+05, 4.402142E+05, 4.491544E+05, 4.582309E+05,
    4.674450E+05, 4.767978E+05, 4.862906E+05, 4.959246E+05, 5.057011E+05, 5.156212E+05,
    5.256861E+05, 5.358972E+05, 5.462555E+05, 5.567623E+05, 5.674188E+05, 5.782262E+05,
    5.891856E+05, 6.002984E+05, 6.115655E+05, 6.229883E+05, 6.345679E+05, 6.463054E+05,
    6.582021E+05, 6.702590E+05, 6.824774E+05, 6.948584E+05, 7.074030E+05, 7.201125E+05,
    7.329880E+05, 7.460305E+05, 7.592412E+05, 7.726213E+05, 7.861717E+05, 7.998937E+05,
    8.137882E+05, 8.278563E+05, 8.420992E+05, 8.565179E+05, 8.711134E+05, 8.858868E+05,
    9.008391E+05, 9.159714E+05, 9.312846E+05, 9.467798E+05, 9.624580E+05, 9.783202E+05,
    9.943673E+05, 1.010600E+06, 1.027020E+06, 1.043628E+06, 1.060425E+06,
])

#  ---------------------- M = 2, I = 8 ---------------------------
M = 2
I = 8
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.391189E+01, 2.356850E+02, 4.693664E+02, 7.030628E+02, 9.367842E+02, 1.170667E+03,
    1.405248E+03, 1.641677E+03, 1.881655E+03, 2.127217E+03, 2.380506E+03, 2.643624E+03,
    2.918557E+03, 3.207165E+03, 3.511177E+03, 3.832224E+03, 4.171854E+03, 4.531559E+03,
    4.912793E+03, 5.316991E+03, 5.745580E+03, 6.199992E+03, 6.681674E+03, 7.192093E+03,
    7.732745E+03, 8.305157E+03, 8.910895E+03, 9.551564E+03, 1.022881E+04, 1.094434E+04,
    1.169988E+04, 1.249723E+04, 1.333824E+04, 1.422480E+04, 1.515886E+04, 1.614244E+04,
    1.717760E+04, 1.826646E+04, 1.941121E+04, 2.061408E+04, 2.187739E+04, 2.320350E+04,
    2.459484E+04, 2.605391E+04, 2.758326E+04, 2.918551E+04, 3.086338E+04, 3.261960E+04,
    3.445702E+04, 3.637852E+04, 3.838709E+04, 4.048575E+04, 4.267762E+04, 4.496589E+04,
    4.735382E+04, 4.984473E+04, 5.244204E+04, 5.514923E+04, 5.796986E+04, 6.090757E+04,
    6.396608E+04, 6.714918E+04, 7.046075E+04, 7.390475E+04, 7.748521E+04, 8.120625E+04,
    8.507207E+04, 8.908696E+04, 9.325528E+04, 9.758148E+04, 1.020701E+05, 1.067258E+05,
    1.115532E+05, 1.165571E+05, 1.217425E+05, 1.271142E+05, 1.326774E+05, 1.384371E+05,
    1.443986E+05, 1.505672E+05, 1.569483E+05, 1.635474E+05, 1.703700E+05, 1.774218E+05,
    1.847086E+05, 1.922361E+05, 2.000103E+05, 2.080372E+05, 2.163229E+05, 2.248735E+05,
    2.336952E+05, 2.427945E+05, 2.521776E+05, 2.618512E+05, 2.718217E+05, 2.820959E+05,
    2.926803E+05, 3.035819E+05, 3.148074E+05, 3.263639E+05, 3.382582E+05, 3.504976E+05,
    3.630890E+05, 3.760398E+05, 3.893571E+05, 4.030483E+05, 4.171208E+05, 4.315819E+05,
    4.464392E+05, 4.617002E+05, 4.773724E+05, 4.934635E+05, 5.099811E+05, 5.269329E+05,
    5.443266E+05, 5.621700E+05, 5.804710E+05, 5.992372E+05, 6.184765E+05, 6.381968E+05,
    6.584059E+05, 6.791119E+05, 7.003224E+05, 7.220455E+05, 7.442890E+05, 7.670610E+05,
    7.903691E+05, 8.142215E+05, 8.386259E+05, 8.635903E+05, 8.891225E+05, 9.152303E+05,
    9.419216E+05, 9.692043E+05, 9.970860E+05, 1.025575E+06, 1.054678E+06, 1.084403E+06,
    1.114758E+06, 1.145751E+06, 1.177388E+06, 1.209678E+06, 1.242628E+06, 1.276246E+06,
    1.310538E+06, 1.345512E+06, 1.381175E+06, 1.417535E+06, 1.454598E+06, 1.492372E+06,
    1.530863E+06, 1.570079E+06, 1.610026E+06, 1.650710E+06, 1.692140E+06, 1.734320E+06,
    1.777258E+06, 1.820960E+06, 1.865433E+06, 1.910682E+06, 1.956714E+06, 2.003535E+06,
    2.051150E+06, 2.099566E+06, 2.148789E+06, 2.198824E+06, 2.249676E+06, 2.301352E+06,
    2.353856E+06, 2.407194E+06, 2.461371E+06, 2.516392E+06, 2.572262E+06, 2.628986E+06,
    2.686569E+06, 2.745015E+06,
])

#  ---------------------- M = 2, I = 9 ---------------------------
M = 2
I = 9
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.124397E+01, 6.874652E+02, 1.368916E+03, 2.050410E+03, 2.731976E+03, 3.413998E+03,
    4.098007E+03, 4.787289E+03, 5.486743E+03, 6.202233E+03, 6.939947E+03, 7.705959E+03,
    8.506016E+03, 9.345477E+03, 1.022934E+04, 1.116230E+04, 1.214882E+04, 1.319320E+04,
    1.429960E+04, 1.547216E+04, 1.671497E+04, 1.803216E+04, 1.942786E+04, 2.090629E+04,
    2.247174E+04, 2.412860E+04, 2.588134E+04, 2.773459E+04, 2.969305E+04, 3.176160E+04,
    3.394522E+04, 3.624906E+04, 3.867840E+04, 4.123867E+04, 4.393547E+04, 4.677456E+04,
    4.976184E+04, 5.290340E+04, 5.620549E+04, 5.967453E+04, 6.331712E+04, 6.714003E+04,
    7.115023E+04, 7.535485E+04, 7.976121E+04, 8.437683E+04, 8.920942E+04, 9.426687E+04,
    9.955728E+04, 1.050889E+05, 1.108703E+05, 1.169102E+05, 1.232174E+05, 1.298010E+05,
    1.366705E+05, 1.438352E+05, 1.513051E+05, 1.590899E+05, 1.672000E+05, 1.756458E+05,
    1.844378E+05, 1.935870E+05, 2.031044E+05, 2.130014E+05, 2.232894E+05, 2.339804E+05,
    2.450863E+05, 2.566195E+05, 2.685923E+05, 2.810176E+05, 2.939083E+05, 3.072777E+05,
    3.211394E+05, 3.355069E+05, 3.503944E+05, 3.658161E+05, 3.817865E+05, 3.983203E+05,
    4.154326E+05, 4.331386E+05, 4.514539E+05, 4.703944E+05, 4.899760E+05, 5.102151E+05,
    5.311284E+05, 5.527328E+05, 5.750453E+05, 5.980834E+05, 6.218648E+05, 6.464075E+05,
    6.717298E+05, 6.978501E+05, 7.247874E+05, 7.525606E+05, 7.811893E+05, 8.106929E+05,
    8.410916E+05, 8.724056E+05, 9.046553E+05, 9.378615E+05, 9.720455E+05, 1.007229E+06,
    1.043432E+06, 1.080679E+06, 1.118991E+06, 1.158390E+06, 1.198899E+06, 1.240543E+06,
    1.283343E+06, 1.327324E+06, 1.372511E+06, 1.418926E+06, 1.466595E+06, 1.515544E+06,
    1.565796E+06, 1.617378E+06, 1.670316E+06, 1.724635E+06, 1.780363E+06, 1.837526E+06,
    1.896150E+06, 1.956265E+06, 2.017896E+06, 2.081072E+06, 2.145822E+06, 2.212173E+06,
    2.280156E+06, 2.349798E+06, 2.421130E+06, 2.494182E+06, 2.568982E+06, 2.645562E+06,
    2.723953E+06, 2.804185E+06, 2.886290E+06, 2.970298E+06, 3.056243E+06, 3.144156E+06,
    3.234069E+06, 3.326015E+06, 3.420028E+06, 3.516140E+06, 3.614384E+06, 3.714796E+06,
    3.817408E+06, 3.922256E+06, 4.029373E+06, 4.138795E+06, 4.250556E+06, 4.364693E+06,
    4.481240E+06, 4.600234E+06, 4.721710E+06, 4.845706E+06, 4.972258E+06, 5.101402E+06,
    5.233175E+06, 5.367616E+06, 5.504761E+06, 5.644648E+06, 5.787315E+06, 5.932800E+06,
    6.081142E+06, 6.232379E+06, 6.386550E+06, 6.543694E+06, 6.703849E+06, 6.867056E+06,
    7.033353E+06, 7.202781E+06, 7.375378E+06, 7.551186E+06, 7.730243E+06, 7.912590E+06,
    8.098268E+06, 8.287317E+06, 8.479777E+06, 8.675690E+06, 8.875095E+06, 9.078035E+06,
    9.284549E+06, 9.494679E+06, 9.708467E+06, 9.925953E+06, 1.014718E+07, 1.037219E+07,
    1.060102E+07, 1.083371E+07, 1.107031E+07, 1.131086E+07, 1.155539E+07, 1.180396E+07,
    1.205660E+07, 1.231335E+07, 1.257426E+07, 1.283936E+07, 1.310870E+07, 1.338233E+07,
    1.366027E+07, 1.394258E+07, 1.422929E+07, 1.452045E+07, 1.481610E+07, 1.511628E+07,
    1.542103E+07, 1.573039E+07, 1.604440E+07, 1.636310E+07, 1.668655E+07, 1.701476E+07,
    1.734779E+07, 1.768569E+07, 1.802847E+07, 1.837620E+07, 1.872890E+07, 1.908663E+07,
    1.944941E+07, 1.981729E+07, 2.019030E+07, 2.056850E+07, 2.095191E+07, 2.134058E+07,
    2.173454E+07, 2.213383E+07, 2.253850E+07, 2.294858E+07, 2.336411E+07, 2.378512E+07,
    2.421166E+07, 2.464377E+07, 2.508147E+07, 2.552481E+07, 2.597382E+07, 2.642854E+07,
    2.688901E+07, 2.735527E+07, 2.782734E+07, 2.830527E+07, 2.878909E+07, 2.927883E+07,
    2.977453E+07, 3.027623E+07, 3.078396E+07, 3.129774E+07, 3.181763E+07, 3.234365E+07,
    3.287583E+07, 3.341421E+07, 3.395881E+07, 3.450968E+07, 3.506684E+07,
])

#  ---------------------- M = 2, I = 10 ---------------------------
M = 2
I = 10
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.501700E+00, 4.041606E+01, 8.049824E+01, 1.205830E+02, 1.606730E+02, 2.007995E+02,
    2.410771E+02, 2.817371E+02, 3.231082E+02, 3.655703E+02, 4.095143E+02, 4.553170E+02,
    5.033312E+02, 5.538845E+02, 6.072820E+02, 6.638110E+02, 7.237459E+02, 7.873522E+02,
    8.548902E+02, 9.266178E+02, 1.002793E+03, 1.083677E+03, 1.169533E+03, 1.260630E+03,
    1.357243E+03, 1.459653E+03, 1.568150E+03, 1.683030E+03, 1.804598E+03, 1.933171E+03,
    2.069070E+03, 2.212630E+03, 2.364195E+03, 2.524117E+03, 2.692761E+03, 2.870501E+03,
    3.057723E+03, 3.254823E+03, 3.462209E+03, 3.680301E+03, 3.909530E+03, 4.150338E+03,
    4.403179E+03, 4.668522E+03, 4.946845E+03, 5.238639E+03, 5.544410E+03, 5.864675E+03,
    6.199964E+03, 6.550821E+03, 6.917802E+03, 7.301478E+03, 7.702433E+03, 8.121265E+03,
    8.558585E+03, 9.015020E+03, 9.491209E+03, 9.987807E+03, 1.050548E+04, 1.104492E+04,
    1.160682E+04, 1.219189E+04, 1.280087E+04, 1.343449E+04, 1.409352E+04, 1.477873E+04,
    1.549091E+04, 1.623087E+04, 1.699943E+04, 1.779742E+04, 1.862571E+04, 1.948515E+04,
    2.037665E+04, 2.130110E+04, 2.225942E+04, 2.325254E+04, 2.428143E+04, 2.534705E+04,
    2.645038E+04, 2.759244E+04, 2.877424E+04, 2.999682E+04, 3.126123E+04, 3.256856E+04,
    3.391989E+04, 3.531632E+04, 3.675900E+04, 3.824905E+04, 3.978765E+04, 4.137597E+04,
    4.301520E+04, 4.470658E+04, 4.645132E+04, 4.825069E+04, 5.010596E+04, 5.201841E+04,
    5.398935E+04, 5.602011E+04, 5.811204E+04, 6.026650E+04, 6.248487E+04, 6.476855E+04,
    6.711897E+04, 6.953757E+04, 7.202580E+04, 7.458515E+04, 7.721711E+04, 7.992319E+04,
    8.270494E+04, 8.556390E+04, 8.850166E+04, 9.151980E+04, 9.461994E+04, 9.780372E+04,
    1.010728E+05, 1.044288E+05, 1.078734E+05, 1.114085E+05, 1.150356E+05, 1.187565E+05,
    1.225730E+05, 1.264870E+05, 1.305001E+05, 1.346143E+05, 1.388313E+05, 1.431531E+05,
    1.475815E+05, 1.521185E+05, 1.567659E+05, 1.615257E+05, 1.663998E+05, 1.713903E+05,
    1.764992E+05, 1.817284E+05, 1.870800E+05, 1.925560E+05, 1.981586E+05, 2.038898E+05,
    2.097518E+05, 2.157466E+05, 2.218764E+05, 2.281434E+05, 2.345498E+05, 2.410978E+05,
    2.477895E+05, 2.546272E+05, 2.616132E+05, 2.687498E+05, 2.760392E+05, 2.834837E+05,
    2.910857E+05, 2.988474E+05, 3.067713E+05, 3.148597E+05, 3.231150E+05, 3.315396E+05,
    3.401358E+05, 3.489062E+05, 3.578531E+05, 3.669790E+05, 3.762864E+05, 3.857777E+05,
    3.954555E+05, 4.053222E+05, 4.153804E+05, 4.256326E+05, 4.360813E+05, 4.467291E+05,
    4.575786E+05, 4.686322E+05, 4.798927E+05, 4.913626E+05, 5.030444E+05, 5.149409E+05,
    5.270546E+05, 5.393881E+05, 5.519441E+05, 5.647252E+05, 5.777342E+05, 5.909735E+05,
    6.044459E+05, 6.181541E+05, 6.321007E+05, 6.462885E+05, 6.607200E+05, 6.753979E+05,
    6.903251E+05, 7.055041E+05, 7.209376E+05, 7.366284E+05, 7.525791E+05, 7.687924E+05,
    7.852711E+05, 8.020178E+05, 8.190353E+05, 8.363262E+05, 8.538932E+05, 8.717391E+05,
    8.898665E+05, 9.082781E+05, 9.269767E+05, 9.459648E+05, 9.652453E+05, 9.848207E+05,
    1.004694E+06, 1.024867E+06, 1.045343E+06, 1.066125E+06, 1.087216E+06, 1.108617E+06,
    1.130332E+06, 1.152363E+06, 1.174712E+06, 1.197384E+06, 1.220379E+06, 1.243701E+06,
    1.267352E+06, 1.291335E+06, 1.315653E+06, 1.340307E+06, 1.365301E+06, 1.390637E+06,
    1.416318E+06, 1.442345E+06, 1.468723E+06, 1.495452E+06, 1.522536E+06, 1.549977E+06,
    1.577777E+06, 1.605939E+06, 1.634466E+06, 1.663359E+06, 1.692621E+06, 1.722255E+06,
    1.752263E+06, 1.782647E+06, 1.813409E+06, 1.844552E+06, 1.876079E+06, 1.907990E+06,
    1.940290E+06, 1.972979E+06, 2.006060E+06, 2.039536E+06, 2.073408E+06, 2.107678E+06,
    2.142350E+06, 2.177424E+06, 2.212904E+06, 2.248791E+06, 2.285086E+06,
])

#  --------------- CO2 838: M = 2, I = 0 ALIAS-----------------
TIPS_2017_ISOT_HASH[(M,0)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,0)] = TIPS_2017_ISOQ_HASH[(M,I)]

#  ---------------------- M = 2, I = 11 ---------------------------
M = 2
I = 11
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.782335E+01, 4.713557E+02, 9.387029E+02, 1.406080E+03, 1.873518E+03, 2.341368E+03,
    2.810932E+03, 3.284859E+03, 3.766925E+03, 4.261513E+03, 4.773136E+03, 5.306147E+03,
    5.864623E+03, 6.452344E+03, 7.072828E+03, 7.729387E+03, 8.425175E+03, 9.163245E+03,
    9.946589E+03, 1.077817E+04, 1.166095E+04, 1.259791E+04, 1.359209E+04, 1.464656E+04,
    1.576448E+04, 1.694906E+04, 1.820363E+04, 1.953159E+04, 2.093643E+04, 2.242176E+04,
    2.399128E+04, 2.564882E+04, 2.739831E+04, 2.924378E+04, 3.118942E+04, 3.323950E+04,
    3.539843E+04, 3.767076E+04, 4.006115E+04, 4.257440E+04, 4.521543E+04, 4.798931E+04,
    5.090124E+04, 5.395655E+04, 5.716074E+04, 6.051942E+04, 6.403836E+04, 6.772347E+04,
    7.158083E+04, 7.561663E+04, 7.983725E+04, 8.424921E+04, 8.885918E+04, 9.367400E+04,
    9.870066E+04, 1.039463E+05, 1.094183E+05, 1.151240E+05, 1.210712E+05, 1.272677E+05,
    1.337214E+05, 1.404405E+05, 1.474333E+05, 1.547084E+05, 1.622744E+05, 1.701402E+05,
    1.783147E+05, 1.868073E+05, 1.956274E+05, 2.047844E+05, 2.142882E+05, 2.241486E+05,
    2.343760E+05, 2.449804E+05, 2.559725E+05, 2.673630E+05, 2.791627E+05, 2.913827E+05,
    3.040342E+05, 3.171288E+05, 3.306779E+05, 3.446935E+05, 3.591876E+05, 3.741723E+05,
    3.896601E+05, 4.056634E+05, 4.221951E+05, 4.392682E+05, 4.568957E+05, 4.750909E+05,
    4.938673E+05, 5.132387E+05, 5.332187E+05, 5.538215E+05, 5.750612E+05, 5.969522E+05,
    6.195090E+05, 6.427462E+05, 6.666788E+05, 6.913216E+05, 7.166898E+05, 7.427988E+05,
    7.696639E+05, 7.973007E+05, 8.257250E+05, 8.549525E+05, 8.849992E+05, 9.158811E+05,
    9.476145E+05, 9.802157E+05, 1.013701E+06, 1.048087E+06, 1.083390E+06, 1.119627E+06,
    1.156815E+06, 1.194970E+06, 1.234110E+06, 1.274250E+06, 1.315409E+06, 1.357604E+06,
    1.400851E+06, 1.445167E+06, 1.490570E+06, 1.537077E+06, 1.584704E+06, 1.633470E+06,
    1.683392E+06, 1.734485E+06, 1.786769E+06, 1.840259E+06, 1.894973E+06, 1.950928E+06,
    2.008141E+06, 2.066630E+06, 2.126410E+06, 2.187498E+06, 2.249913E+06, 2.313669E+06,
    2.378785E+06, 2.445276E+06, 2.513159E+06, 2.582450E+06, 2.653165E+06, 2.725322E+06,
    2.798935E+06, 2.874020E+06, 2.950594E+06, 3.028672E+06, 3.108269E+06, 3.189401E+06,
    3.272083E+06, 3.356330E+06, 3.442157E+06, 3.529579E+06, 3.618610E+06, 3.709264E+06,
    3.801557E+06, 3.895501E+06, 3.991111E+06, 4.088400E+06, 4.187382E+06, 4.288070E+06,
    4.390478E+06, 4.494617E+06, 4.600501E+06, 4.708142E+06, 4.817553E+06, 4.928745E+06,
    5.041730E+06, 5.156520E+06, 5.273126E+06, 5.391560E+06, 5.511832E+06, 5.633953E+06,
    5.757934E+06, 5.883784E+06,
])

#  ---------------------- M = 2, I = 12 ---------------------------
M = 2
I = 12
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.248682E+01, 1.374885E+03, 2.737741E+03, 4.100686E+03, 5.463802E+03, 6.828082E+03,
    8.197234E+03, 9.578835E+03, 1.098375E+04, 1.242462E+04, 1.391448E+04, 1.546591E+04,
    1.709070E+04, 1.879977E+04, 2.060326E+04, 2.251071E+04, 2.453122E+04, 2.667355E+04,
    2.894631E+04, 3.135799E+04, 3.391712E+04, 3.663224E+04, 3.951206E+04, 4.256541E+04,
    4.580131E+04, 4.922903E+04, 5.285804E+04, 5.669811E+04, 6.075926E+04, 6.505180E+04,
    6.958637E+04, 7.437390E+04, 7.942564E+04, 8.475318E+04, 9.036846E+04, 9.628373E+04,
    1.025116E+05, 1.090652E+05, 1.159576E+05, 1.232028E+05, 1.308148E+05, 1.388081E+05,
    1.471975E+05, 1.559983E+05, 1.652263E+05, 1.748975E+05, 1.850284E+05, 1.956358E+05,
    2.067372E+05, 2.183504E+05, 2.304934E+05, 2.431851E+05, 2.564443E+05, 2.702908E+05,
    2.847445E+05, 2.998258E+05, 3.155556E+05, 3.319555E+05, 3.490472E+05, 3.668531E+05,
    3.853960E+05, 4.046992E+05, 4.247867E+05, 4.456827E+05, 4.674120E+05, 4.900000E+05,
    5.134725E+05, 5.378560E+05, 5.631772E+05, 5.894637E+05, 6.167433E+05, 6.450447E+05,
    6.743967E+05, 7.048290E+05, 7.363717E+05, 7.690554E+05, 8.029115E+05, 8.379716E+05,
    8.742681E+05, 9.118340E+05, 9.507027E+05, 9.909083E+05, 1.032485E+06, 1.075469E+06,
    1.119896E+06, 1.165801E+06, 1.213222E+06, 1.262197E+06, 1.312763E+06, 1.364960E+06,
    1.418826E+06, 1.474402E+06, 1.531729E+06, 1.590847E+06, 1.651799E+06, 1.714626E+06,
    1.779372E+06, 1.846081E+06, 1.914797E+06, 1.985564E+06, 2.058429E+06, 2.133437E+06,
    2.210636E+06, 2.290073E+06, 2.371795E+06, 2.455852E+06, 2.542294E+06, 2.631169E+06,
    2.722529E+06, 2.816425E+06, 2.912910E+06, 3.012035E+06, 3.113855E+06, 3.218422E+06,
    3.325792E+06, 3.436020E+06, 3.549161E+06, 3.665273E+06, 3.784412E+06, 3.906637E+06,
    4.032006E+06, 4.160578E+06, 4.292413E+06, 4.427572E+06, 4.566115E+06, 4.708105E+06,
    4.853604E+06, 5.002674E+06, 5.155381E+06, 5.311788E+06, 5.471961E+06, 5.635964E+06,
    5.803865E+06, 5.975731E+06, 6.151628E+06, 6.331626E+06, 6.515793E+06, 6.704199E+06,
    6.896914E+06, 7.094009E+06, 7.295554E+06, 7.501623E+06, 7.712287E+06, 7.927620E+06,
    8.147695E+06, 8.372588E+06, 8.602372E+06, 8.837123E+06, 9.076918E+06, 9.321833E+06,
    9.571945E+06, 9.827332E+06, 1.008807E+07, 1.035424E+07, 1.062593E+07, 1.090320E+07,
    1.118615E+07, 1.147484E+07, 1.176937E+07, 1.206982E+07, 1.237626E+07, 1.268879E+07,
    1.300747E+07, 1.333241E+07, 1.366367E+07, 1.400135E+07, 1.434553E+07, 1.469629E+07,
    1.505372E+07, 1.541791E+07, 1.578894E+07, 1.616690E+07, 1.655188E+07, 1.694395E+07,
    1.734322E+07, 1.774977E+07, 1.816368E+07, 1.858505E+07, 1.901396E+07, 1.945050E+07,
    1.989476E+07, 2.034683E+07, 2.080679E+07, 2.127475E+07, 2.175079E+07, 2.223499E+07,
    2.272745E+07, 2.322825E+07, 2.373750E+07, 2.425527E+07, 2.478166E+07, 2.531676E+07,
    2.586066E+07, 2.641344E+07, 2.697521E+07, 2.754605E+07, 2.812604E+07, 2.871529E+07,
    2.931387E+07, 2.992189E+07, 3.053943E+07, 3.116658E+07, 3.180343E+07, 3.245007E+07,
    3.310659E+07, 3.377308E+07, 3.444963E+07, 3.513633E+07, 3.583326E+07, 3.654053E+07,
    3.725820E+07, 3.798638E+07, 3.872515E+07, 3.947459E+07, 4.023480E+07, 4.100587E+07,
    4.178787E+07, 4.258090E+07, 4.338505E+07, 4.420039E+07, 4.502701E+07, 4.586501E+07,
    4.671445E+07, 4.757544E+07, 4.844805E+07, 4.933236E+07, 5.022846E+07, 5.113643E+07,
    5.205636E+07, 5.298832E+07, 5.393239E+07, 5.488867E+07, 5.585722E+07, 5.683812E+07,
    5.783147E+07, 5.883732E+07, 5.985577E+07, 6.088689E+07, 6.193076E+07, 6.298745E+07,
    6.405704E+07, 6.513961E+07, 6.623523E+07, 6.734397E+07, 6.846590E+07, 6.960111E+07,
    7.074966E+07, 7.191162E+07, 7.308707E+07, 7.427607E+07, 7.547870E+07,
])

#  ---------------------- M = 2, I = 13 ---------------------------
M = 2
I = 13
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.172250E+00, 1.797782E+01, 3.578844E+01, 5.360026E+01, 7.141461E+01, 8.924655E+01,
    1.071502E+02, 1.252330E+02, 1.436455E+02, 1.625593E+02, 1.821490E+02, 2.025809E+02,
    2.240094E+02, 2.465762E+02, 2.704128E+02, 2.956425E+02, 3.223827E+02, 3.507470E+02,
    3.808470E+02, 4.127935E+02, 4.466974E+02, 4.826712E+02, 5.208288E+02, 5.612866E+02,
    6.041637E+02, 6.495824E+02, 6.976684E+02, 7.485507E+02, 8.023624E+02, 8.592405E+02,
    9.193259E+02, 9.827640E+02, 1.049704E+03, 1.120301E+03, 1.194713E+03, 1.273103E+03,
    1.355638E+03, 1.442493E+03, 1.533844E+03, 1.629875E+03, 1.730772E+03, 1.836729E+03,
    1.947943E+03, 2.064618E+03, 2.186963E+03, 2.315191E+03, 2.449522E+03, 2.590182E+03,
    2.737401E+03, 2.891416E+03, 3.052469E+03, 3.220808E+03, 3.396688E+03, 3.580369E+03,
    3.772118E+03, 3.972206E+03, 4.180913E+03, 4.398524E+03, 4.625331E+03, 4.861631E+03,
    5.107729E+03, 5.363936E+03, 5.630570E+03, 5.907955E+03, 6.196423E+03, 6.496312E+03,
    6.807968E+03, 7.131742E+03, 7.467993E+03, 7.817088E+03, 8.179401E+03, 8.555312E+03,
    8.945209E+03, 9.349488E+03, 9.768552E+03, 1.020281E+04, 1.065268E+04, 1.111859E+04,
    1.160098E+04, 1.210027E+04, 1.261693E+04, 1.315140E+04, 1.370416E+04, 1.427567E+04,
    1.486641E+04, 1.547688E+04, 1.610757E+04, 1.675898E+04, 1.743163E+04, 1.812603E+04,
    1.884273E+04, 1.958224E+04, 2.034513E+04, 2.113194E+04, 2.194324E+04, 2.277960E+04,
    2.364159E+04, 2.452981E+04, 2.544485E+04, 2.638733E+04, 2.735784E+04, 2.835702E+04,
    2.938550E+04, 3.044392E+04, 3.153292E+04, 3.265317E+04, 3.380534E+04, 3.499009E+04,
    3.620811E+04, 3.746011E+04, 3.874677E+04, 4.006882E+04, 4.142697E+04, 4.282196E+04,
    4.425451E+04, 4.572539E+04, 4.723535E+04, 4.878515E+04, 5.037557E+04, 5.200740E+04,
    5.368142E+04, 5.539845E+04, 5.715929E+04, 5.896477E+04, 6.081571E+04, 6.271296E+04,
    6.465735E+04, 6.664976E+04, 6.869105E+04, 7.078209E+04, 7.292377E+04, 7.511697E+04,
    7.736262E+04, 7.966160E+04, 8.201486E+04, 8.442330E+04, 8.688789E+04, 8.940955E+04,
    9.198924E+04, 9.462794E+04, 9.732660E+04, 1.000862E+05, 1.029078E+05, 1.057923E+05,
    1.087407E+05, 1.117541E+05, 1.148335E+05, 1.179799E+05, 1.211944E+05, 1.244779E+05,
    1.278316E+05, 1.312566E+05, 1.347538E+05, 1.383243E+05, 1.419693E+05, 1.456898E+05,
    1.494870E+05, 1.533619E+05, 1.573156E+05, 1.613494E+05, 1.654642E+05, 1.696612E+05,
    1.739416E+05, 1.783064E+05, 1.827570E+05, 1.872943E+05, 1.919196E+05, 1.966340E+05,
    2.014388E+05, 2.063350E+05, 2.113239E+05, 2.164066E+05, 2.215843E+05, 2.268583E+05,
    2.322298E+05, 2.376998E+05, 2.432697E+05, 2.489407E+05, 2.547139E+05, 2.605906E+05,
    2.665720E+05, 2.726593E+05, 2.788538E+05, 2.851567E+05, 2.915692E+05, 2.980925E+05,
    3.047279E+05, 3.114766E+05, 3.183399E+05, 3.253190E+05, 3.324151E+05, 3.396295E+05,
    3.469634E+05, 3.544181E+05, 3.619948E+05, 3.696947E+05, 3.775191E+05, 3.854693E+05,
    3.935464E+05, 4.017517E+05, 4.100865E+05, 4.185520E+05, 4.271494E+05, 4.358800E+05,
    4.447450E+05, 4.537456E+05, 4.628831E+05, 4.721587E+05, 4.815736E+05, 4.911291E+05,
    5.008263E+05, 5.106664E+05, 5.206508E+05, 5.307806E+05, 5.410569E+05, 5.514811E+05,
    5.620543E+05, 5.727776E+05, 5.836524E+05, 5.946797E+05, 6.058608E+05, 6.171968E+05,
    6.286889E+05, 6.403382E+05, 6.521460E+05, 6.641133E+05, 6.762413E+05, 6.885312E+05,
    7.009841E+05, 7.136011E+05, 7.263834E+05, 7.393320E+05, 7.524481E+05, 7.657327E+05,
    7.791870E+05, 7.928120E+05, 8.066089E+05, 8.205787E+05, 8.347224E+05, 8.490412E+05,
    8.635361E+05, 8.782080E+05, 8.930581E+05, 9.080874E+05, 9.232969E+05, 9.386875E+05,
    9.542604E+05, 9.700165E+05, 9.859567E+05, 1.002082E+06, 1.018394E+06,
])

#  ---------------------- M = 3, I = 1 ---------------------------
M = 3
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    7.847400E-01, 5.870075E+01, 1.653093E+02, 3.033348E+02, 4.668337E+02, 6.523999E+02,
    8.578395E+02, 1.081788E+03, 1.323572E+03, 1.583129E+03, 1.860885E+03, 2.157688E+03,
    2.474797E+03, 2.813601E+03, 3.175846E+03, 3.563351E+03, 3.978042E+03, 4.422045E+03,
    4.897528E+03, 5.406676E+03, 5.951807E+03, 6.535268E+03, 7.159589E+03, 7.827197E+03,
    8.540615E+03, 9.302558E+03, 1.011573E+04, 1.098290E+04, 1.190686E+04, 1.289056E+04,
    1.393708E+04, 1.504939E+04, 1.623081E+04, 1.748441E+04, 1.881364E+04, 2.022192E+04,
    2.171273E+04, 2.328963E+04, 2.495636E+04, 2.671676E+04, 2.857462E+04, 3.053392E+04,
    3.259874E+04, 3.477325E+04, 3.706184E+04, 3.946872E+04, 4.199829E+04, 4.465523E+04,
    4.744417E+04, 5.037000E+04, 5.343750E+04, 5.665155E+04, 6.001736E+04, 6.354022E+04,
    6.722531E+04, 7.107790E+04, 7.510385E+04, 7.930836E+04, 8.369781E+04, 8.827733E+04,
    9.305350E+04, 9.803196E+04, 1.032190E+05, 1.086212E+05, 1.142448E+05, 1.200965E+05,
    1.261830E+05, 1.325109E+05, 1.390870E+05, 1.459185E+05, 1.530127E+05, 1.603767E+05,
    1.680182E+05, 1.759443E+05, 1.841627E+05, 1.926817E+05, 2.015087E+05, 2.106519E+05,
    2.201195E+05, 2.299199E+05, 2.400610E+05, 2.505524E+05, 2.614014E+05, 2.726184E+05,
    2.842108E+05, 2.961892E+05, 3.085615E+05, 3.213380E+05, 3.345278E+05, 3.481407E+05,
    3.621862E+05, 3.766745E+05, 3.916156E+05, 4.070192E+05, 4.228968E+05, 4.392584E+05,
    4.561139E+05, 4.734750E+05, 4.913523E+05, 5.097566E+05, 5.286996E+05, 5.481927E+05,
    5.682468E+05, 5.888737E+05, 6.100856E+05, 6.318949E+05, 6.543128E+05, 6.773516E+05,
    7.010245E+05, 7.253433E+05, 7.503210E+05, 7.759709E+05, 8.023056E+05, 8.293382E+05,
    8.570820E+05, 8.855514E+05, 9.147587E+05, 9.447191E+05, 9.754458E+05, 1.006954E+06,
    1.039256E+06, 1.072368E+06, 1.106304E+06, 1.141080E+06, 1.176708E+06, 1.213207E+06,
    1.250589E+06, 1.288871E+06, 1.328070E+06, 1.368200E+06, 1.409276E+06, 1.451317E+06,
    1.494338E+06, 1.538354E+06, 1.583385E+06, 1.629444E+06, 1.676552E+06, 1.724723E+06,
    1.773976E+06, 1.824329E+06, 1.875798E+06, 1.928402E+06, 1.982159E+06, 2.037088E+06,
    2.093206E+06, 2.150533E+06, 2.209086E+06, 2.268887E+06, 2.329953E+06, 2.392303E+06,
    2.455959E+06, 2.520938E+06, 2.587261E+06, 2.654949E+06, 2.724021E+06, 2.794498E+06,
    2.866401E+06, 2.939752E+06, 3.014570E+06, 3.090877E+06, 3.168695E+06, 3.248044E+06,
    3.328948E+06, 3.411429E+06, 3.495507E+06, 3.581206E+06, 3.668550E+06, 3.757559E+06,
    3.848259E+06, 3.940671E+06, 4.034818E+06, 4.130727E+06, 4.228419E+06, 4.327918E+06,
    4.429248E+06, 4.532437E+06,
])

#  ---------------------- M = 3, I = 2 ---------------------------
M = 3
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.659650E+00, 1.252867E+02, 3.529131E+02, 6.476329E+02, 9.967517E+02, 1.393009E+03,
    1.831769E+03, 2.310206E+03, 2.827051E+03, 3.382309E+03, 3.977121E+03, 4.613505E+03,
    5.294212E+03, 6.022501E+03, 6.802194E+03, 7.637375E+03, 8.532274E+03, 9.491628E+03,
    1.052016E+04, 1.162272E+04, 1.280444E+04, 1.407051E+04, 1.542641E+04, 1.687762E+04,
    1.842974E+04, 2.008860E+04, 2.186018E+04, 2.375072E+04, 2.576662E+04, 2.791400E+04,
    3.019968E+04, 3.263083E+04, 3.521391E+04, 3.795663E+04, 4.086590E+04, 4.394976E+04,
    4.721594E+04, 5.067202E+04, 5.432657E+04, 5.818808E+04, 6.226493E+04, 6.656593E+04,
    7.110012E+04, 7.587705E+04, 8.090577E+04, 8.619627E+04, 9.175872E+04, 9.760273E+04,
    1.037389E+05, 1.101777E+05, 1.169305E+05, 1.240078E+05, 1.314214E+05, 1.391823E+05,
    1.473028E+05, 1.557947E+05, 1.646703E+05, 1.739424E+05, 1.836239E+05, 1.937268E+05,
    2.042656E+05, 2.152535E+05, 2.267043E+05, 2.386323E+05, 2.510512E+05, 2.639756E+05,
    2.774213E+05, 2.914026E+05, 3.059358E+05, 3.210359E+05, 3.367181E+05, 3.530002E+05,
    3.698980E+05, 3.874283E+05, 4.056074E+05, 4.244543E+05, 4.439858E+05, 4.642198E+05,
    4.851738E+05, 5.068680E+05, 5.293193E+05, 5.525478E+05, 5.765737E+05, 6.014149E+05,
    6.270935E+05, 6.536272E+05, 6.810391E+05, 7.093488E+05, 7.385776E+05, 7.687471E+05,
    7.998788E+05, 8.319959E+05, 8.651198E+05, 8.992744E+05, 9.344815E+05, 9.707647E+05,
    1.008148E+06, 1.046657E+06, 1.086313E+06, 1.127143E+06, 1.169172E+06, 1.212425E+06,
    1.256927E+06, 1.302705E+06, 1.349784E+06, 1.398192E+06, 1.447956E+06, 1.499104E+06,
    1.551662E+06, 1.605658E+06, 1.661122E+06, 1.718084E+06, 1.776569E+06, 1.836610E+06,
    1.898235E+06, 1.961475E+06, 2.026362E+06, 2.092923E+06, 2.161193E+06, 2.231202E+06,
    2.302981E+06, 2.376565E+06, 2.451985E+06, 2.529274E+06, 2.608467E+06, 2.689597E+06,
    2.772695E+06, 2.857802E+06, 2.944948E+06, 3.034171E+06, 3.125505E+06, 3.218987E+06,
    3.314654E+06, 3.412542E+06, 3.512687E+06, 3.615129E+06, 3.719906E+06, 3.827055E+06,
    3.936617E+06, 4.048628E+06, 4.163130E+06, 4.280164E+06, 4.399769E+06, 4.521984E+06,
    4.646853E+06, 4.774418E+06, 4.904720E+06, 5.037801E+06, 5.173705E+06, 5.312474E+06,
    5.454153E+06, 5.598785E+06, 5.746414E+06, 5.897090E+06, 6.050851E+06, 6.207749E+06,
    6.367826E+06, 6.531132E+06, 6.697714E+06, 6.867614E+06, 7.040889E+06, 7.217578E+06,
    7.397739E+06, 7.581418E+06, 7.768659E+06, 7.959519E+06, 8.154048E+06, 8.352294E+06,
    8.554311E+06, 8.760149E+06, 8.969865E+06, 9.183502E+06, 9.401127E+06, 9.622785E+06,
    9.848527E+06, 1.007842E+07,
])

#  ---------------------- M = 3, I = 3 ---------------------------
M = 3
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.185900E-01, 6.125610E+01, 1.725127E+02, 3.165570E+02, 4.871861E+02, 6.808519E+02,
    8.952625E+02, 1.129039E+03, 1.381537E+03, 1.652726E+03, 1.943193E+03, 2.253943E+03,
    2.586326E+03, 2.942026E+03, 3.322888E+03, 3.730960E+03, 4.168424E+03, 4.637562E+03,
    5.140769E+03, 5.680414E+03, 6.259100E+03, 6.879368E+03, 7.543867E+03, 8.255453E+03,
    9.016771E+03, 9.830870E+03, 1.070057E+04, 1.162904E+04, 1.261929E+04, 1.367462E+04,
    1.479833E+04, 1.599377E+04, 1.726436E+04, 1.861379E+04, 2.004575E+04, 2.156398E+04,
    2.317217E+04, 2.487462E+04, 2.667503E+04, 2.857789E+04, 3.058721E+04, 3.270766E+04,
    3.494358E+04, 3.729938E+04, 3.978003E+04, 4.239034E+04, 4.513515E+04, 4.801943E+04,
    5.104863E+04, 5.422773E+04, 5.756221E+04, 6.105763E+04, 6.471951E+04, 6.855364E+04,
    7.256596E+04, 7.676252E+04, 8.114927E+04, 8.573247E+04, 9.051853E+04, 9.551399E+04,
    1.007252E+05, 1.061593E+05, 1.118231E+05, 1.177230E+05, 1.238669E+05, 1.302620E+05,
    1.369151E+05, 1.438344E+05, 1.510269E+05, 1.585011E+05, 1.662646E+05, 1.743254E+05,
    1.826918E+05, 1.913721E+05, 2.003749E+05, 2.097086E+05, 2.193821E+05, 2.294048E+05,
    2.397852E+05, 2.505325E+05, 2.616565E+05, 2.731663E+05, 2.850718E+05, 2.973825E+05,
    3.101087E+05, 3.232600E+05, 3.368470E+05, 3.508807E+05, 3.653706E+05, 3.803279E+05,
    3.957632E+05, 4.116883E+05, 4.281136E+05, 4.450504E+05, 4.625108E+05, 4.805059E+05,
    4.990478E+05, 5.181485E+05, 5.378202E+05, 5.580750E+05, 5.789261E+05, 6.003846E+05,
    6.224646E+05, 6.451784E+05, 6.685396E+05, 6.925614E+05, 7.172569E+05, 7.426405E+05,
    7.687248E+05, 7.955251E+05, 8.230548E+05, 8.513281E+05, 8.803602E+05, 9.101655E+05,
    9.407590E+05, 9.721548E+05, 1.004370E+06, 1.037418E+06, 1.071315E+06, 1.106078E+06,
    1.141721E+06, 1.178261E+06, 1.215715E+06, 1.254098E+06, 1.293428E+06, 1.333722E+06,
    1.374996E+06, 1.417268E+06, 1.460554E+06, 1.504874E+06, 1.550244E+06, 1.596682E+06,
    1.644207E+06, 1.692838E+06, 1.742592E+06, 1.793488E+06, 1.845546E+06, 1.898785E+06,
    1.953223E+06, 2.008881E+06, 2.065779E+06, 2.123935E+06, 2.183371E+06, 2.244106E+06,
    2.306163E+06, 2.369560E+06, 2.434319E+06, 2.500462E+06, 2.568009E+06, 2.636982E+06,
    2.707404E+06, 2.779295E+06, 2.852679E+06, 2.927578E+06, 3.004013E+06, 3.082010E+06,
    3.161589E+06, 3.242776E+06, 3.325591E+06, 3.410061E+06, 3.496209E+06, 3.584059E+06,
    3.673634E+06, 3.764960E+06, 3.858061E+06, 3.952965E+06, 4.049693E+06, 4.148272E+06,
    4.248730E+06, 4.351088E+06, 4.455377E+06, 4.561620E+06, 4.669848E+06, 4.780082E+06,
    4.892352E+06, 5.006686E+06,
])

#  ---------------------- M = 3, I = 4 ---------------------------
M = 3
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    9.693860E+00, 7.286110E+02, 2.052138E+03, 3.765742E+03, 5.795622E+03, 8.099566E+03,
    1.065031E+04, 1.343144E+04, 1.643497E+04, 1.966034E+04, 2.311386E+04, 2.680652E+04,
    3.075398E+04, 3.497494E+04, 3.949041E+04, 4.432421E+04, 4.950089E+04, 5.504705E+04,
    6.098959E+04, 6.735674E+04, 7.417719E+04, 8.148200E+04, 8.930109E+04, 9.766575E+04,
    1.066095E+05, 1.161652E+05, 1.263658E+05, 1.372482E+05, 1.488478E+05, 1.612019E+05,
    1.743472E+05, 1.883247E+05, 2.031740E+05, 2.189350E+05, 2.356501E+05, 2.533639E+05,
    2.721204E+05, 2.919662E+05, 3.129454E+05, 3.351083E+05, 3.585027E+05, 3.831788E+05,
    4.091905E+05, 4.365882E+05, 4.654258E+05, 4.957624E+05, 5.276496E+05, 5.611468E+05,
    5.963170E+05, 6.332148E+05, 6.719059E+05, 7.124520E+05, 7.549189E+05, 7.993710E+05,
    8.458764E+05, 8.945049E+05, 9.453249E+05, 9.984083E+05, 1.053826E+06, 1.111656E+06,
    1.171972E+06, 1.234856E+06, 1.300379E+06, 1.368623E+06, 1.439674E+06, 1.513611E+06,
    1.590524E+06, 1.670492E+06, 1.753610E+06, 1.839962E+06, 1.929642E+06, 2.022739E+06,
    2.119351E+06, 2.219570E+06, 2.323499E+06, 2.431227E+06, 2.542864E+06, 2.658513E+06,
    2.778268E+06, 2.902239E+06, 3.030540E+06, 3.163267E+06, 3.300539E+06, 3.442465E+06,
    3.589168E+06, 3.740750E+06, 3.897328E+06, 4.059035E+06, 4.225983E+06, 4.398292E+06,
    4.576090E+06, 4.759502E+06, 4.948660E+06, 5.143690E+06, 5.344725E+06, 5.551892E+06,
    5.765331E+06, 5.985180E+06, 6.211579E+06, 6.444666E+06, 6.684586E+06, 6.931482E+06,
    7.185500E+06, 7.446782E+06, 7.715490E+06, 7.991764E+06, 8.275772E+06, 8.567664E+06,
    8.867593E+06, 9.175712E+06, 9.492203E+06, 9.817227E+06, 1.015093E+07, 1.049349E+07,
    1.084509E+07, 1.120589E+07, 1.157607E+07, 1.195579E+07, 1.234524E+07, 1.274460E+07,
    1.315405E+07, 1.357378E+07, 1.400397E+07, 1.444480E+07, 1.489648E+07, 1.535919E+07,
    1.583314E+07, 1.631850E+07, 1.681549E+07, 1.732430E+07, 1.784514E+07, 1.837821E+07,
    1.892372E+07, 1.948191E+07, 2.005293E+07, 2.063704E+07, 2.123444E+07, 2.184536E+07,
    2.247001E+07, 2.310862E+07, 2.376142E+07, 2.442864E+07, 2.511048E+07, 2.580720E+07,
    2.651904E+07, 2.724621E+07, 2.798898E+07, 2.874757E+07, 2.952224E+07, 3.031321E+07,
    3.112077E+07, 3.194514E+07, 3.278657E+07, 3.364534E+07, 3.452170E+07, 3.541590E+07,
    3.632821E+07, 3.725890E+07, 3.820824E+07, 3.917648E+07, 4.016391E+07, 4.117082E+07,
    4.219747E+07, 4.324413E+07, 4.431109E+07, 4.539866E+07, 4.650710E+07, 4.763672E+07,
    4.878780E+07, 4.996063E+07, 5.115550E+07, 5.237277E+07, 5.361268E+07, 5.487554E+07,
    5.616170E+07, 5.747145E+07,
])

#  ---------------------- M = 3, I = 5 ---------------------------
M = 3
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.813030E+00, 3.600892E+02, 1.014082E+03, 1.860805E+03, 2.863799E+03, 4.002166E+03,
    5.262502E+03, 6.636539E+03, 8.120243E+03, 9.713429E+03, 1.141910E+04, 1.324287E+04,
    1.519237E+04, 1.727719E+04, 1.950780E+04, 2.189589E+04, 2.445388E+04, 2.719480E+04,
    3.013257E+04, 3.328074E+04, 3.665424E+04, 4.026795E+04, 4.413675E+04, 4.827698E+04,
    5.270423E+04, 5.743513E+04, 6.248716E+04, 6.787719E+04, 7.362388E+04, 7.974527E+04,
    8.626022E+04, 9.318860E+04, 1.005494E+05, 1.083638E+05, 1.166537E+05, 1.254391E+05,
    1.347427E+05, 1.445876E+05, 1.549967E+05, 1.659941E+05, 1.776035E+05, 1.898519E+05,
    2.027633E+05, 2.163637E+05, 2.306819E+05, 2.457443E+05, 2.615788E+05, 2.782148E+05,
    2.956814E+05, 3.140094E+05, 3.332292E+05, 3.533726E+05, 3.744711E+05, 3.965582E+05,
    4.196678E+05, 4.438332E+05, 4.690883E+05, 4.954722E+05, 5.230185E+05, 5.517647E+05,
    5.817489E+05, 6.130100E+05, 6.455856E+05, 6.795180E+05, 7.148483E+05, 7.516143E+05,
    7.898604E+05, 8.296320E+05, 8.709688E+05, 9.139178E+05, 9.585242E+05, 1.004832E+06,
    1.052892E+06, 1.102747E+06, 1.154451E+06, 1.208049E+06, 1.263593E+06, 1.321131E+06,
    1.380722E+06, 1.442410E+06, 1.506255E+06, 1.572304E+06, 1.640622E+06, 1.711256E+06,
    1.784265E+06, 1.859711E+06, 1.937648E+06, 2.018139E+06, 2.101238E+06, 2.187013E+06,
    2.275522E+06, 2.366832E+06, 2.461001E+06, 2.558097E+06, 2.658186E+06, 2.761333E+06,
    2.867604E+06, 2.977071E+06, 3.089804E+06, 3.205868E+06, 3.325339E+06, 3.448284E+06,
    3.574781E+06, 3.704902E+06, 3.838724E+06, 3.976317E+06, 4.117767E+06, 4.263141E+06,
    4.412525E+06, 4.565996E+06, 4.723636E+06, 4.885531E+06, 5.051753E+06, 5.222396E+06,
    5.397538E+06, 5.577272E+06, 5.761674E+06, 5.950846E+06, 6.144860E+06, 6.343822E+06,
    6.547813E+06, 6.756924E+06, 6.971254E+06, 7.190894E+06, 7.415940E+06, 7.646489E+06,
    7.882637E+06, 8.124482E+06, 8.372123E+06, 8.625660E+06, 8.885193E+06, 9.150831E+06,
    9.422673E+06, 9.700822E+06, 9.985389E+06, 1.027648E+07, 1.057420E+07, 1.087865E+07,
    1.118997E+07, 1.150823E+07, 1.183358E+07, 1.216611E+07, 1.250594E+07, 1.285320E+07,
    1.320799E+07, 1.357043E+07, 1.394065E+07, 1.431876E+07, 1.470489E+07, 1.509916E+07,
    1.550169E+07, 1.591261E+07, 1.633204E+07, 1.676013E+07, 1.719697E+07, 1.764274E+07,
    1.809752E+07, 1.856147E+07, 1.903474E+07, 1.951743E+07, 2.000969E+07, 2.051167E+07,
    2.102349E+07, 2.154530E+07, 2.207724E+07, 2.261945E+07, 2.317208E+07, 2.373528E+07,
    2.430918E+07, 2.489392E+07, 2.548968E+07, 2.609659E+07, 2.671481E+07, 2.734449E+07,
    2.798577E+07, 2.863883E+07,
])

#  ---------------------- M = 3, I = 6 ---------------------------
M = 3
I = 6
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.732560E+00, 1.308439E+02, 3.685813E+02, 6.763945E+02, 1.041035E+03, 1.454902E+03,
    1.913200E+03, 2.413032E+03, 2.953209E+03, 3.533915E+03, 4.156499E+03, 4.823380E+03,
    5.537674E+03, 6.303045E+03, 7.123765E+03, 8.004388E+03, 8.949564E+03, 9.964533E+03,
    1.105444E+04, 1.222466E+04, 1.348084E+04, 1.482862E+04, 1.627401E+04, 1.782304E+04,
    1.948187E+04, 2.125695E+04, 2.315481E+04, 2.518216E+04, 2.734599E+04, 2.965343E+04,
    3.211177E+04, 3.472865E+04, 3.751164E+04, 4.046894E+04, 4.360818E+04, 4.693842E+04,
    5.046789E+04, 5.420551E+04, 5.816009E+04, 6.234127E+04, 6.675826E+04, 7.142107E+04,
    7.633970E+04, 8.152423E+04, 8.698542E+04, 9.273349E+04, 9.877997E+04, 1.051355E+05,
    1.118121E+05, 1.188218E+05, 1.261758E+05, 1.338869E+05, 1.419674E+05, 1.504307E+05,
    1.592888E+05, 1.685564E+05, 1.782460E+05, 1.883721E+05, 1.989491E+05, 2.099906E+05,
    2.215123E+05, 2.335285E+05, 2.460551E+05, 2.591078E+05, 2.727019E+05, 2.868541E+05,
    3.015810E+05, 3.168993E+05, 3.328262E+05, 3.493782E+05, 3.665743E+05, 3.844320E+05,
    4.029702E+05, 4.222066E+05, 4.421602E+05, 4.628517E+05, 4.842990E+05, 5.065236E+05,
    5.295438E+05, 5.533829E+05, 5.780588E+05, 6.035954E+05, 6.300121E+05, 6.573328E+05,
    6.855785E+05, 7.147714E+05, 7.449354E+05, 7.760936E+05, 8.082695E+05, 8.414866E+05,
    8.757691E+05, 9.111436E+05, 9.476326E+05, 9.852625E+05, 1.024060E+06, 1.064049E+06,
    1.105257E+06, 1.147712E+06, 1.191440E+06, 1.236468E+06, 1.282825E+06, 1.330539E+06,
    1.379637E+06, 1.430151E+06, 1.482109E+06, 1.535539E+06, 1.590474E+06, 1.646944E+06,
    1.704977E+06, 1.764609E+06, 1.825868E+06, 1.888788E+06, 1.953399E+06, 2.019738E+06,
    2.087834E+06, 2.157723E+06, 2.229439E+06, 2.303016E+06, 2.378487E+06, 2.455892E+06,
    2.535263E+06, 2.616637E+06, 2.700050E+06, 2.785540E+06, 2.873144E+06, 2.962900E+06,
    3.054845E+06, 3.149018E+06, 3.245459E+06, 3.344205E+06, 3.445299E+06, 3.548780E+06,
    3.654689E+06, 3.763068E+06, 3.873955E+06, 3.987395E+06, 4.103430E+06, 4.222104E+06,
    4.343458E+06, 4.467538E+06, 4.594386E+06, 4.724049E+06, 4.856572E+06, 4.991999E+06,
    5.130375E+06, 5.271750E+06, 5.416171E+06, 5.563681E+06, 5.714331E+06, 5.868170E+06,
    6.025247E+06, 6.185609E+06, 6.349307E+06, 6.516392E+06, 6.686913E+06, 6.860922E+06,
    7.038471E+06, 7.219613E+06, 7.404399E+06, 7.592883E+06, 7.785118E+06, 7.981157E+06,
    8.181058E+06, 8.384875E+06, 8.592660E+06, 8.804475E+06, 9.020369E+06, 9.240407E+06,
    9.464641E+06, 9.693130E+06, 9.925935E+06, 1.016311E+07, 1.040473E+07, 1.065083E+07,
    1.090149E+07, 1.115676E+07,
])

#  ---------------------- M = 3, I = 7 ---------------------------
M = 3
I = 7
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.789300E-01, 6.694173E+01, 1.886102E+02, 3.461476E+02, 5.327717E+02, 7.445995E+02,
    9.791830E+02, 1.235085E+03, 1.511700E+03, 1.809143E+03, 2.128107E+03, 2.469769E+03,
    2.835738E+03, 3.227819E+03, 3.648133E+03, 4.098969E+03, 4.582699E+03, 5.101875E+03,
    5.659118E+03, 6.257169E+03, 6.898826E+03, 7.586970E+03, 8.324625E+03, 9.114762E+03,
    9.960542E+03, 1.086520E+04, 1.183213E+04, 1.286461E+04, 1.396617E+04, 1.514050E+04,
    1.639117E+04, 1.772203E+04, 1.913689E+04, 2.064001E+04, 2.223524E+04, 2.392689E+04,
    2.571922E+04, 2.761689E+04, 2.962419E+04, 3.174614E+04, 3.398705E+04, 3.635233E+04,
    3.884663E+04, 4.147543E+04, 4.424376E+04, 4.715716E+04, 5.022104E+04, 5.344119E+04,
    5.682326E+04, 6.037348E+04, 6.409755E+04, 6.800167E+04, 7.209225E+04, 7.637581E+04,
    8.085871E+04, 8.554797E+04, 9.045026E+04, 9.557287E+04, 1.009224E+05, 1.065065E+05,
    1.123325E+05, 1.184081E+05, 1.247407E+05, 1.313384E+05, 1.382093E+05, 1.453615E+05,
    1.528030E+05, 1.605426E+05, 1.685888E+05, 1.769504E+05, 1.856360E+05, 1.946552E+05,
    2.040167E+05, 2.137304E+05, 2.238055E+05, 2.342516E+05, 2.450786E+05, 2.562968E+05,
    2.679164E+05, 2.799473E+05, 2.924003E+05, 3.052861E+05, 3.186154E+05, 3.323992E+05,
    3.466491E+05, 3.613758E+05, 3.765914E+05, 3.923068E+05, 4.085344E+05, 4.252868E+05,
    4.425748E+05, 4.604120E+05, 4.788108E+05, 4.977832E+05, 5.173427E+05, 5.375024E+05,
    5.582752E+05, 5.796753E+05, 6.017155E+05, 6.244101E+05, 6.477726E+05, 6.718181E+05,
    6.965603E+05, 7.220136E+05, 7.481935E+05, 7.751140E+05, 8.027916E+05, 8.312397E+05,
    8.604757E+05, 8.905141E+05, 9.213715E+05, 9.530639E+05, 9.856067E+05, 1.019018E+06,
    1.053313E+06, 1.088509E+06, 1.124624E+06, 1.161674E+06, 1.199678E+06, 1.238652E+06,
    1.278615E+06, 1.319584E+06, 1.361580E+06, 1.404619E+06, 1.448720E+06, 1.493902E+06,
    1.540186E+06, 1.587589E+06, 1.636131E+06, 1.685832E+06, 1.736714E+06, 1.788795E+06,
    1.842095E+06, 1.896636E+06, 1.952439E+06, 2.009525E+06, 2.067914E+06, 2.127629E+06,
    2.188690E+06, 2.251121E+06, 2.314944E+06, 2.380180E+06, 2.446852E+06, 2.514985E+06,
    2.584598E+06, 2.655719E+06, 2.728370E+06, 2.802572E+06, 2.878352E+06, 2.955734E+06,
    3.034741E+06, 3.115400E+06, 3.197733E+06, 3.281769E+06, 3.367530E+06, 3.455043E+06,
    3.544334E+06, 3.635429E+06, 3.728356E+06, 3.823139E+06, 3.919806E+06, 4.018383E+06,
    4.118900E+06, 4.221384E+06, 4.325860E+06, 4.432360E+06, 4.540909E+06, 4.651539E+06,
    4.764276E+06, 4.879150E+06, 4.996191E+06, 5.115428E+06, 5.236890E+06, 5.360610E+06,
    5.486615E+06, 5.614939E+06,
])

#  ---------------------- M = 3, I = 8 ---------------------------
M = 3
I = 8
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.018317E+01, 7.688685E+02, 2.165826E+03, 3.974547E+03, 6.117122E+03, 8.549075E+03,
    1.124182E+04, 1.417849E+04, 1.735159E+04, 2.076146E+04, 2.441595E+04, 2.832833E+04,
    3.251593E+04, 3.699983E+04, 4.180437E+04, 4.695509E+04, 5.247930E+04, 5.840655E+04,
    6.476616E+04, 7.158997E+04, 7.890924E+04, 8.675751E+04, 9.516855E+04, 1.041762E+05,
    1.138172E+05, 1.241287E+05, 1.351464E+05, 1.469105E+05, 1.594614E+05, 1.728378E+05,
    1.870849E+05, 2.022417E+05, 2.183567E+05, 2.354731E+05, 2.536375E+05, 2.728988E+05,
    2.933058E+05, 3.149078E+05, 3.377591E+05, 3.619112E+05, 3.874208E+05, 4.143397E+05,
    4.427289E+05, 4.726444E+05, 5.041454E+05, 5.372972E+05, 5.721616E+05, 6.087995E+05,
    6.472799E+05, 6.876690E+05, 7.300362E+05, 7.744501E+05, 8.209831E+05, 8.697093E+05,
    9.207011E+05, 9.740386E+05, 1.029797E+06, 1.088057E+06, 1.148900E+06, 1.212405E+06,
    1.278663E+06, 1.347752E+06, 1.419769E+06, 1.494792E+06, 1.572920E+06, 1.654247E+06,
    1.738862E+06, 1.826861E+06, 1.918342E+06, 2.013409E+06, 2.112162E+06, 2.214696E+06,
    2.321129E+06, 2.431558E+06, 2.546090E+06, 2.664846E+06, 2.787926E+06, 2.915445E+06,
    3.047527E+06, 3.184286E+06, 3.325838E+06, 3.472305E+06, 3.623810E+06, 3.780482E+06,
    3.942443E+06, 4.109824E+06, 4.282759E+06, 4.461371E+06, 4.645800E+06, 4.836192E+06,
    5.032673E+06, 5.235386E+06, 5.444471E+06, 5.660087E+06, 5.882366E+06, 6.111462E+06,
    6.347516E+06, 6.590696E+06, 6.841155E+06, 7.099044E+06, 7.364517E+06, 7.637744E+06,
    7.918894E+06, 8.208116E+06, 8.505584E+06, 8.811474E+06, 9.125951E+06, 9.449192E+06,
    9.781375E+06, 1.012267E+07, 1.047327E+07, 1.083334E+07, 1.120309E+07, 1.158269E+07,
    1.197233E+07, 1.237221E+07, 1.278251E+07, 1.320344E+07, 1.363520E+07, 1.407798E+07,
    1.453199E+07, 1.499743E+07, 1.547452E+07, 1.596346E+07, 1.646446E+07, 1.697775E+07,
    1.750353E+07, 1.804203E+07, 1.859347E+07, 1.915807E+07, 1.973607E+07, 2.032768E+07,
    2.093316E+07, 2.155272E+07, 2.218661E+07, 2.283506E+07, 2.349832E+07, 2.417662E+07,
    2.487023E+07, 2.557938E+07, 2.630433E+07, 2.704534E+07, 2.780265E+07, 2.857654E+07,
    2.936727E+07, 3.017509E+07, 3.100027E+07, 3.184310E+07, 3.270384E+07, 3.358276E+07,
    3.448014E+07, 3.539626E+07, 3.633142E+07, 3.728588E+07, 3.825994E+07, 3.925390E+07,
    4.026805E+07, 4.130268E+07, 4.235810E+07, 4.343460E+07, 4.453249E+07, 4.565210E+07,
    4.679370E+07, 4.795764E+07, 4.914420E+07, 5.035375E+07, 5.158655E+07, 5.284297E+07,
    5.412334E+07, 5.542796E+07, 5.675719E+07, 5.811133E+07, 5.949078E+07, 6.089583E+07,
    6.232682E+07, 6.378415E+07,
])

#  ---------------------- M = 3, I = 9 ---------------------------
M = 3
I = 9
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.025939E+01, 7.780853E+02, 2.192024E+03, 4.022766E+03, 6.191507E+03, 8.653040E+03,
    1.137884E+04, 1.435181E+04, 1.756428E+04, 2.101726E+04, 2.471805E+04, 2.867999E+04,
    3.292087E+04, 3.746130E+04, 4.232569E+04, 4.753963E+04, 5.313097E+04, 5.912840E+04,
    6.556172E+04, 7.246219E+04, 7.986305E+04, 8.779593E+04, 9.629532E+04, 1.053959E+05,
    1.151338E+05, 1.255461E+05, 1.366706E+05, 1.485447E+05, 1.612110E+05, 1.747082E+05,
    1.890807E+05, 2.043701E+05, 2.206199E+05, 2.378786E+05, 2.561919E+05, 2.756073E+05,
    2.961737E+05, 3.179441E+05, 3.409693E+05, 3.653014E+05, 3.909969E+05, 4.181098E+05,
    4.467016E+05, 4.768262E+05, 5.085454E+05, 5.419201E+05, 5.770169E+05, 6.138970E+05,
    6.526250E+05, 6.932718E+05, 7.359051E+05, 7.805932E+05, 8.274118E+05, 8.764327E+05,
    9.277281E+05, 9.813788E+05, 1.037460E+06, 1.096053E+06, 1.157238E+06, 1.221100E+06,
    1.287723E+06, 1.357190E+06, 1.429589E+06, 1.505014E+06, 1.583553E+06, 1.665298E+06,
    1.750344E+06, 1.838791E+06, 1.930734E+06, 2.026272E+06, 2.125506E+06, 2.228539E+06,
    2.335477E+06, 2.446431E+06, 2.561504E+06, 2.680806E+06, 2.804454E+06, 2.932553E+06,
    3.065232E+06, 3.202599E+06, 3.344773E+06, 3.491880E+06, 3.644044E+06, 3.801383E+06,
    3.964036E+06, 4.132119E+06, 4.305769E+06, 4.485124E+06, 4.670306E+06, 4.861464E+06,
    5.058736E+06, 5.262256E+06, 5.472173E+06, 5.688624E+06, 5.911762E+06, 6.141735E+06,
    6.378700E+06, 6.622796E+06, 6.874190E+06, 7.133033E+06, 7.399489E+06, 7.673712E+06,
    7.955873E+06, 8.246134E+06, 8.544672E+06, 8.851639E+06, 9.167217E+06, 9.491582E+06,
    9.824914E+06, 1.016738E+07, 1.051917E+07, 1.088046E+07, 1.125145E+07, 1.163231E+07,
    1.202323E+07, 1.242442E+07, 1.283606E+07, 1.325835E+07, 1.369150E+07, 1.413568E+07,
    1.459113E+07, 1.505804E+07, 1.553661E+07, 1.602706E+07, 1.652961E+07, 1.704446E+07,
    1.757184E+07, 1.811196E+07, 1.866504E+07, 1.923134E+07, 1.981104E+07, 2.040440E+07,
    2.101163E+07, 2.163299E+07, 2.226870E+07, 2.291900E+07, 2.358413E+07, 2.426437E+07,
    2.495991E+07, 2.567104E+07, 2.639800E+07, 2.714105E+07, 2.790045E+07, 2.867645E+07,
    2.946930E+07, 3.027929E+07, 3.110670E+07, 3.195175E+07, 3.281477E+07, 3.369600E+07,
    3.459571E+07, 3.551422E+07, 3.645178E+07, 3.740870E+07, 3.838525E+07, 3.938174E+07,
    4.039843E+07, 4.143568E+07, 4.249373E+07, 4.357290E+07, 4.467351E+07, 4.579585E+07,
    4.694025E+07, 4.810702E+07, 4.929646E+07, 5.050889E+07, 5.174464E+07, 5.300407E+07,
    5.428747E+07, 5.559516E+07, 5.692750E+07, 5.828480E+07, 5.966743E+07, 6.107572E+07,
    6.251002E+07, 6.397068E+07,
])

#  ---------------------- M = 3, I = 10 ---------------------------
M = 3
I = 10
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.011611E+01, 7.606432E+02, 2.142444E+03, 3.931508E+03, 6.050781E+03, 8.456192E+03,
    1.111961E+04, 1.402401E+04, 1.716179E+04, 2.053370E+04, 2.414710E+04, 2.801498E+04,
    3.215563E+04, 3.658950E+04, 4.134081E+04, 4.643537E+04, 5.190096E+04, 5.776617E+04,
    6.406074E+04, 7.081585E+04, 7.806331E+04, 8.583661E+04, 9.416875E+04, 1.030942E+05,
    1.126489E+05, 1.228698E+05, 1.337943E+05, 1.454598E+05, 1.579072E+05, 1.711776E+05,
    1.853113E+05, 2.003519E+05, 2.163455E+05, 2.333335E+05, 2.513666E+05, 2.704892E+05,
    2.907529E+05, 3.122071E+05, 3.349040E+05, 3.588951E+05, 3.842348E+05, 4.109803E+05,
    4.391888E+05, 4.689179E+05, 5.002271E+05, 5.331765E+05, 5.678301E+05, 6.042531E+05,
    6.425100E+05, 6.826686E+05, 7.247962E+05, 7.689631E+05, 8.152419E+05, 8.637038E+05,
    9.144236E+05, 9.674780E+05, 1.022945E+06, 1.080905E+06, 1.141440E+06, 1.204627E+06,
    1.270556E+06, 1.339309E+06, 1.410973E+06, 1.485641E+06, 1.563401E+06, 1.644344E+06,
    1.728569E+06, 1.816162E+06, 1.907235E+06, 2.001877E+06, 2.100190E+06, 2.202280E+06,
    2.308247E+06, 2.418202E+06, 2.532251E+06, 2.650506E+06, 2.773076E+06, 2.900076E+06,
    3.031614E+06, 3.167824E+06, 3.308806E+06, 3.454693E+06, 3.605607E+06, 3.761672E+06,
    3.923009E+06, 4.089756E+06, 4.262030E+06, 4.439977E+06, 4.623729E+06, 4.813416E+06,
    5.009182E+06, 5.211159E+06, 5.419501E+06, 5.634346E+06, 5.855846E+06, 6.084142E+06,
    6.319381E+06, 6.561730E+06, 6.811327E+06, 7.068344E+06, 7.332930E+06, 7.605253E+06,
    7.885465E+06, 8.173747E+06, 8.470249E+06, 8.775154E+06, 9.088627E+06, 9.410835E+06,
    9.741974E+06, 1.008220E+07, 1.043171E+07, 1.079068E+07, 1.115929E+07, 1.153774E+07,
    1.192620E+07, 1.232488E+07, 1.273397E+07, 1.315366E+07, 1.358416E+07, 1.402564E+07,
    1.447834E+07, 1.494244E+07, 1.541817E+07, 1.590572E+07, 1.640531E+07, 1.691716E+07,
    1.744149E+07, 1.797850E+07, 1.852842E+07, 1.909149E+07, 1.966793E+07, 2.025794E+07,
    2.086181E+07, 2.147971E+07, 2.211193E+07, 2.275867E+07, 2.342019E+07, 2.409675E+07,
    2.478856E+07, 2.549589E+07, 2.621900E+07, 2.695813E+07, 2.771354E+07, 2.848549E+07,
    2.927424E+07, 3.008006E+07, 3.090322E+07, 3.174399E+07, 3.260262E+07, 3.347942E+07,
    3.437463E+07, 3.528858E+07, 3.622149E+07, 3.717369E+07, 3.814547E+07, 3.913711E+07,
    4.014888E+07, 4.118112E+07, 4.223410E+07, 4.330813E+07, 4.440353E+07, 4.552057E+07,
    4.665960E+07, 4.782093E+07, 4.900486E+07, 5.021170E+07, 5.144178E+07, 5.269544E+07,
    5.397301E+07, 5.527477E+07, 5.660111E+07, 5.795235E+07, 5.932882E+07, 6.073086E+07,
    6.215883E+07, 6.361306E+07,
])

#  ---------------------- M = 3, I = 11 ---------------------------
M = 3
I = 11
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.946791E+01, 4.470503E+03, 1.259147E+04, 2.310592E+04, 3.556102E+04, 4.969775E+04,
    6.535012E+04, 8.241681E+04, 1.008519E+05, 1.206550E+05, 1.418675E+05, 1.645612E+05,
    1.888379E+05, 2.148151E+05, 2.426307E+05, 2.724293E+05, 3.043712E+05, 3.386193E+05,
    3.753466E+05, 4.147317E+05, 4.569565E+05, 5.022089E+05, 5.506812E+05, 6.025756E+05,
    6.580938E+05, 7.174414E+05, 7.808443E+05, 8.485166E+05, 9.206880E+05, 9.975858E+05,
    1.079455E+06, 1.166548E+06, 1.259104E+06, 1.357392E+06, 1.461671E+06, 1.572220E+06,
    1.689327E+06, 1.813266E+06, 1.944345E+06, 2.082848E+06, 2.229110E+06, 2.383425E+06,
    2.546143E+06, 2.717572E+06, 2.898074E+06, 3.087996E+06, 3.287693E+06, 3.497522E+06,
    3.717868E+06, 3.949102E+06, 4.191641E+06, 4.445858E+06, 4.712167E+06, 4.991009E+06,
    5.282761E+06, 5.587915E+06, 5.906867E+06, 6.240090E+06, 6.588050E+06, 6.951201E+06,
    7.330046E+06, 7.725058E+06, 8.136736E+06, 8.565590E+06, 9.012166E+06, 9.476920E+06,
    9.960460E+06, 1.046329E+07, 1.098601E+07, 1.152911E+07, 1.209326E+07, 1.267897E+07,
    1.328686E+07, 1.391754E+07, 1.457164E+07, 1.524976E+07, 1.595255E+07, 1.668068E+07,
    1.743475E+07, 1.821548E+07, 1.902353E+07, 1.985957E+07, 2.072435E+07, 2.161852E+07,
    2.254284E+07, 2.349807E+07, 2.448487E+07, 2.550405E+07, 2.655638E+07, 2.764263E+07,
    2.876356E+07, 2.992002E+07, 3.111277E+07, 3.234267E+07, 3.361055E+07, 3.491722E+07,
    3.626357E+07, 3.765045E+07, 3.907876E+07, 4.054937E+07, 4.206321E+07, 4.362119E+07,
    4.522422E+07, 4.687322E+07, 4.856921E+07, 5.031312E+07, 5.210590E+07, 5.394861E+07,
    5.584216E+07, 5.778765E+07, 5.978600E+07, 6.183837E+07, 6.394578E+07, 6.610924E+07,
    6.832982E+07, 7.060868E+07, 7.294696E+07, 7.534565E+07, 7.780593E+07, 8.032895E+07,
    8.291590E+07, 8.556789E+07, 8.828611E+07, 9.107179E+07, 9.392614E+07, 9.685037E+07,
    9.984571E+07, 1.029134E+08, 1.060547E+08, 1.092709E+08, 1.125633E+08, 1.159332E+08,
    1.193819E+08, 1.229108E+08, 1.265212E+08, 1.302143E+08, 1.339918E+08, 1.378548E+08,
    1.418050E+08, 1.458435E+08, 1.499719E+08, 1.541916E+08, 1.585041E+08, 1.629109E+08,
    1.674134E+08, 1.720131E+08, 1.767117E+08, 1.815105E+08, 1.864112E+08, 1.914154E+08,
    1.965245E+08, 2.017402E+08, 2.070642E+08, 2.124980E+08, 2.180433E+08, 2.237017E+08,
    2.294748E+08, 2.353645E+08, 2.413725E+08, 2.475002E+08, 2.537497E+08, 2.601226E+08,
    2.666206E+08, 2.732456E+08, 2.799992E+08, 2.868835E+08, 2.939002E+08, 3.010511E+08,
    3.083381E+08, 3.157630E+08, 3.233278E+08, 3.310344E+08, 3.388847E+08, 3.468806E+08,
    3.550242E+08, 3.633173E+08,
])

#  ---------------------- M = 3, I = 12 ---------------------------
M = 3
I = 12
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.995096E+01, 2.261799E+03, 6.371201E+03, 1.169187E+04, 1.799462E+04, 2.514837E+04,
    3.306945E+04, 4.170726E+04, 5.103813E+04, 6.106308E+04, 7.180236E+04, 8.329235E+04,
    9.558279E+04, 1.087346E+05, 1.228140E+05, 1.378944E+05, 1.540562E+05, 1.713824E+05,
    1.899567E+05, 2.098700E+05, 2.312133E+05, 2.540827E+05, 2.785728E+05, 3.047847E+05,
    3.328220E+05, 3.627880E+05, 3.947933E+05, 4.289450E+05, 4.653615E+05, 5.041570E+05,
    5.454511E+05, 5.893697E+05, 6.360411E+05, 6.855926E+05, 7.381579E+05, 7.938731E+05,
    8.528868E+05, 9.153318E+05, 9.813678E+05, 1.051138E+06, 1.124804E+06, 1.202521E+06,
    1.284451E+06, 1.370767E+06, 1.461638E+06, 1.557240E+06, 1.657755E+06, 1.763358E+06,
    1.874240E+06, 1.990604E+06, 2.112627E+06, 2.240529E+06, 2.374501E+06, 2.514755E+06,
    2.661508E+06, 2.814979E+06, 2.975376E+06, 3.142946E+06, 3.317913E+06, 3.500504E+06,
    3.690974E+06, 3.889547E+06, 4.096506E+06, 4.312075E+06, 4.536516E+06, 4.770115E+06,
    5.013131E+06, 5.265828E+06, 5.528484E+06, 5.801396E+06, 6.084839E+06, 6.379123E+06,
    6.684533E+06, 7.001385E+06, 7.329976E+06, 7.670618E+06, 8.023632E+06, 8.389355E+06,
    8.768103E+06, 9.160211E+06, 9.566016E+06, 9.985881E+06, 1.042016E+07, 1.086916E+07,
    1.133330E+07, 1.181291E+07, 1.230839E+07, 1.282011E+07, 1.334843E+07, 1.389376E+07,
    1.445651E+07, 1.503703E+07, 1.563578E+07, 1.625316E+07, 1.688956E+07, 1.754542E+07,
    1.822117E+07, 1.891726E+07, 1.963410E+07, 2.037217E+07, 2.113191E+07, 2.191375E+07,
    2.271818E+07, 2.354568E+07, 2.439673E+07, 2.527179E+07, 2.617137E+07, 2.709593E+07,
    2.804603E+07, 2.902211E+07, 3.002473E+07, 3.105443E+07, 3.211166E+07, 3.319703E+07,
    3.431104E+07, 3.545426E+07, 3.662721E+07, 3.783046E+07, 3.906458E+07, 4.033016E+07,
    4.162776E+07, 4.295797E+07, 4.432139E+07, 4.571860E+07, 4.715020E+07, 4.861682E+07,
    5.011909E+07, 5.165761E+07, 5.323299E+07, 5.484596E+07, 5.649708E+07, 5.818704E+07,
    5.991646E+07, 6.168607E+07, 6.349652E+07, 6.534847E+07, 6.724265E+07, 6.917967E+07,
    7.116036E+07, 7.318530E+07, 7.525531E+07, 7.737105E+07, 7.953329E+07, 8.174279E+07,
    8.400021E+07, 8.630635E+07, 8.866200E+07, 9.106790E+07, 9.352481E+07, 9.603355E+07,
    9.859486E+07, 1.012096E+08, 1.038786E+08, 1.066026E+08, 1.093824E+08, 1.122189E+08,
    1.151128E+08, 1.180652E+08, 1.210768E+08, 1.241484E+08, 1.272810E+08, 1.304754E+08,
    1.337324E+08, 1.370531E+08, 1.404382E+08, 1.438888E+08, 1.474056E+08, 1.509897E+08,
    1.546420E+08, 1.583634E+08, 1.621548E+08, 1.660172E+08, 1.699516E+08, 1.739590E+08,
    1.780403E+08, 1.821965E+08,
])

#  ---------------------- M = 3, I = 13 ---------------------------
M = 3
I = 13
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    9.181300E-01, 6.995503E+01, 1.971073E+02, 3.617463E+02, 5.567837E+02, 7.781615E+02,
    1.023352E+03, 1.290876E+03, 1.580177E+03, 1.891452E+03, 2.225587E+03, 2.583933E+03,
    2.968275E+03, 3.380727E+03, 3.823605E+03, 4.299458E+03, 4.810934E+03, 5.360874E+03,
    5.952091E+03, 6.587671E+03, 7.270592E+03, 8.004158E+03, 8.791511E+03, 9.636056E+03,
    1.054129E+04, 1.151069E+04, 1.254790E+04, 1.365681E+04, 1.484101E+04, 1.610476E+04,
    1.745189E+04, 1.888674E+04, 2.041352E+04, 2.203664E+04, 2.376087E+04, 2.559056E+04,
    2.753068E+04, 2.958620E+04, 3.176202E+04, 3.406331E+04, 3.649554E+04, 3.906410E+04,
    4.177453E+04, 4.463243E+04, 4.764399E+04, 5.081487E+04, 5.415128E+04, 5.765957E+04,
    6.134601E+04, 6.521766E+04, 6.928060E+04, 7.354192E+04, 7.800896E+04, 8.268836E+04,
    8.758771E+04, 9.271449E+04, 9.807632E+04, 1.036807E+05, 1.095361E+05, 1.156503E+05,
    1.220315E+05, 1.286881E+05, 1.356287E+05, 1.428623E+05, 1.503977E+05, 1.582439E+05,
    1.664102E+05, 1.749058E+05, 1.837406E+05, 1.929240E+05, 2.024661E+05, 2.123774E+05,
    2.226672E+05, 2.333470E+05, 2.444267E+05, 2.559177E+05, 2.678304E+05, 2.801759E+05,
    2.929662E+05, 3.062125E+05, 3.199267E+05, 3.341200E+05, 3.488054E+05, 3.639945E+05,
    3.797001E+05, 3.959348E+05, 4.127111E+05, 4.300426E+05, 4.479421E+05, 4.664231E+05,
    4.854994E+05, 5.051847E+05, 5.254935E+05, 5.464391E+05, 5.680363E+05, 5.902997E+05,
    6.132443E+05, 6.368844E+05, 6.612361E+05, 6.863149E+05, 7.121362E+05, 7.387153E+05,
    7.660688E+05, 7.942129E+05, 8.231639E+05, 8.529386E+05, 8.835533E+05, 9.150268E+05,
    9.473747E+05, 9.806152E+05, 1.014766E+06, 1.049845E+06, 1.085871E+06, 1.122862E+06,
    1.160837E+06, 1.199813E+06, 1.239812E+06, 1.280852E+06, 1.322952E+06, 1.366132E+06,
    1.410413E+06, 1.455814E+06, 1.502357E+06, 1.550060E+06, 1.598947E+06, 1.649037E+06,
    1.700354E+06, 1.752916E+06, 1.806748E+06, 1.861869E+06, 1.918305E+06, 1.976076E+06,
    2.035207E+06, 2.095720E+06, 2.157636E+06, 2.220983E+06, 2.285782E+06, 2.352058E+06,
    2.419834E+06, 2.489135E+06, 2.559988E+06, 2.632416E+06, 2.706446E+06, 2.782102E+06,
    2.859408E+06, 2.938395E+06, 3.019085E+06, 3.101507E+06, 3.185689E+06, 3.271654E+06,
    3.359434E+06, 3.449053E+06, 3.540540E+06, 3.633925E+06, 3.729233E+06, 3.826498E+06,
    3.925744E+06, 4.027002E+06, 4.130304E+06, 4.235674E+06, 4.343148E+06, 4.452752E+06,
    4.564520E+06, 4.678482E+06, 4.794667E+06, 4.913111E+06, 5.033839E+06, 5.156890E+06,
    5.282292E+06, 5.410079E+06, 5.540285E+06, 5.672941E+06, 5.808079E+06, 5.945739E+06,
    6.085949E+06, 6.228746E+06,
])

#  ---------------------- M = 3, I = 14 ---------------------------
M = 3
I = 14
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.070955E+01, 8.123365E+02, 2.288590E+03, 4.200027E+03, 6.464366E+03, 9.034498E+03,
    1.188072E+04, 1.498574E+04, 1.834233E+04, 2.195242E+04, 2.582553E+04, 2.997699E+04,
    3.442690E+04, 3.919918E+04, 4.432047E+04, 4.981933E+04, 5.572680E+04, 6.207426E+04,
    6.889543E+04, 7.622379E+04, 8.409527E+04, 9.254631E+04, 1.016143E+05, 1.113367E+05,
    1.217532E+05, 1.329047E+05, 1.448337E+05, 1.575806E+05, 1.711918E+05, 1.857116E+05,
    2.011872E+05, 2.176646E+05, 2.351958E+05, 2.538270E+05, 2.736142E+05, 2.946102E+05,
    3.168671E+05, 3.404431E+05, 3.653942E+05, 3.917795E+05, 4.196636E+05, 4.491018E+05,
    4.801635E+05, 5.129110E+05, 5.474123E+05, 5.837346E+05, 6.219462E+05, 6.621222E+05,
    7.043368E+05, 7.486602E+05, 7.951695E+05, 8.439459E+05, 8.950674E+05, 9.486189E+05,
    1.004678E+06, 1.063332E+06, 1.124673E+06, 1.188782E+06, 1.255752E+06, 1.325673E+06,
    1.398645E+06, 1.474761E+06, 1.554118E+06, 1.636818E+06, 1.722956E+06, 1.812642E+06,
    1.905980E+06, 2.003073E+06, 2.104038E+06, 2.208977E+06, 2.318006E+06, 2.431244E+06,
    2.548807E+06, 2.670808E+06, 2.797373E+06, 2.928627E+06, 3.064690E+06, 3.205688E+06,
    3.351759E+06, 3.503023E+06, 3.659621E+06, 3.821690E+06, 3.989358E+06, 4.162770E+06,
    4.342073E+06, 4.527402E+06, 4.718908E+06, 4.916745E+06, 5.121048E+06, 5.331982E+06,
    5.549699E+06, 5.774354E+06, 6.006112E+06, 6.245127E+06, 6.491574E+06, 6.745604E+06,
    7.007404E+06, 7.277128E+06, 7.554956E+06, 7.841067E+06, 8.135632E+06, 8.438833E+06,
    8.750860E+06, 9.071888E+06, 9.402110E+06, 9.741713E+06, 1.009090E+07, 1.044984E+07,
    1.081877E+07, 1.119784E+07, 1.158730E+07, 1.198732E+07, 1.239813E+07, 1.281992E+07,
    1.325293E+07, 1.369734E+07, 1.415340E+07, 1.462130E+07, 1.510128E+07, 1.559357E+07,
    1.609838E+07, 1.661596E+07, 1.714652E+07, 1.769032E+07, 1.824757E+07, 1.881854E+07,
    1.940346E+07, 2.000258E+07, 2.061614E+07, 2.124441E+07, 2.188760E+07, 2.254603E+07,
    2.321992E+07, 2.390954E+07, 2.461516E+07, 2.533705E+07, 2.607547E+07, 2.683070E+07,
    2.760302E+07, 2.839271E+07, 2.920004E+07, 3.002532E+07, 3.086881E+07, 3.173082E+07,
    3.261163E+07, 3.351154E+07, 3.443087E+07, 3.536991E+07, 3.632895E+07, 3.730831E+07,
    3.830832E+07, 3.932926E+07, 4.037147E+07, 4.143527E+07, 4.252097E+07, 4.362893E+07,
    4.475943E+07, 4.591285E+07, 4.708949E+07, 4.828972E+07, 4.951385E+07, 5.076226E+07,
    5.203526E+07, 5.333325E+07, 5.465654E+07, 5.600550E+07, 5.738049E+07, 5.878190E+07,
    6.021006E+07, 6.166536E+07, 6.314819E+07, 6.465888E+07, 6.619786E+07, 6.776550E+07,
    6.936214E+07, 7.098826E+07,
])

#  ---------------------- M = 3, I = 15 ---------------------------
M = 3
I = 15
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.394640E+00, 4.109442E+02, 1.157868E+03, 2.124994E+03, 3.270687E+03, 4.571107E+03,
    6.011346E+03, 7.582597E+03, 9.281391E+03, 1.110871E+04, 1.306931E+04, 1.517082E+04,
    1.742335E+04, 1.983895E+04, 2.243062E+04, 2.521306E+04, 2.820139E+04, 3.141182E+04,
    3.486102E+04, 3.856575E+04, 4.254401E+04, 4.681405E+04, 5.139459E+04, 5.630527E+04,
    6.156522E+04, 6.719486E+04, 7.321552E+04, 7.964854E+04, 8.651617E+04, 9.384115E+04,
    1.016460E+05, 1.099563E+05, 1.187949E+05, 1.281892E+05, 1.381630E+05, 1.487449E+05,
    1.599612E+05, 1.718402E+05, 1.844110E+05, 1.977042E+05, 2.117483E+05, 2.265752E+05,
    2.422176E+05, 2.587079E+05, 2.760779E+05, 2.943649E+05, 3.136008E+05, 3.338232E+05,
    3.550696E+05, 3.773763E+05, 4.007822E+05, 4.253252E+05, 4.510473E+05, 4.779882E+05,
    5.061912E+05, 5.356961E+05, 5.665499E+05, 5.987935E+05, 6.324759E+05, 6.676398E+05,
    7.043359E+05, 7.426104E+05, 7.825101E+05, 8.240889E+05, 8.673977E+05, 9.124835E+05,
    9.594033E+05, 1.008211E+06, 1.058958E+06, 1.111704E+06, 1.166502E+06, 1.223411E+06,
    1.282491E+06, 1.343799E+06, 1.407399E+06, 1.473350E+06, 1.541716E+06, 1.612559E+06,
    1.685944E+06, 1.761942E+06, 1.840612E+06, 1.922026E+06, 2.006251E+06, 2.093359E+06,
    2.183422E+06, 2.276511E+06, 2.372694E+06, 2.472055E+06, 2.574663E+06, 2.680597E+06,
    2.789932E+06, 2.902747E+06, 3.019125E+06, 3.139149E+06, 3.262894E+06, 3.390446E+06,
    3.521894E+06, 3.657318E+06, 3.796809E+06, 3.940448E+06, 4.088333E+06, 4.240552E+06,
    4.397193E+06, 4.558348E+06, 4.724115E+06, 4.894587E+06, 5.069865E+06, 5.250039E+06,
    5.435212E+06, 5.625483E+06, 5.820957E+06, 6.021730E+06, 6.227909E+06, 6.439603E+06,
    6.656912E+06, 6.879946E+06, 7.108816E+06, 7.343626E+06, 7.584494E+06, 7.831525E+06,
    8.084849E+06, 8.344561E+06, 8.610791E+06, 8.883657E+06, 9.163268E+06, 9.449758E+06,
    9.743239E+06, 1.004384E+07, 1.035168E+07, 1.066689E+07, 1.098959E+07, 1.131992E+07,
    1.165801E+07, 1.200399E+07, 1.235798E+07, 1.272013E+07, 1.309056E+07, 1.346943E+07,
    1.385686E+07, 1.425299E+07, 1.465797E+07, 1.507195E+07, 1.549505E+07, 1.592745E+07,
    1.636926E+07, 1.682065E+07, 1.728178E+07, 1.775276E+07, 1.823379E+07, 1.872501E+07,
    1.922656E+07, 1.973862E+07, 2.026134E+07, 2.079487E+07, 2.133938E+07, 2.189504E+07,
    2.246202E+07, 2.304046E+07, 2.363056E+07, 2.423247E+07, 2.484636E+07, 2.547242E+07,
    2.611081E+07, 2.676171E+07, 2.742530E+07, 2.810175E+07, 2.879126E+07, 2.949400E+07,
    3.021015E+07, 3.093989E+07, 3.168343E+07, 3.244094E+07, 3.321261E+07, 3.399866E+07,
    3.479924E+07, 3.561458E+07,
])

#  ---------------------- M = 3, I = 16 ---------------------------
M = 3
I = 16
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.295915E+01, 4.775780E+03, 1.345460E+04, 2.469181E+04, 3.800370E+04, 5.311278E+04,
    6.984465E+04, 8.809583E+04, 1.078215E+05, 1.290311E+05, 1.517738E+05, 1.761371E+05,
    2.022330E+05, 2.301970E+05, 2.601801E+05, 2.923500E+05, 3.268756E+05, 3.639441E+05,
    4.037454E+05, 4.464741E+05, 4.923320E+05, 5.415271E+05, 5.942775E+05, 6.508028E+05,
    7.113265E+05, 7.760820E+05, 8.453078E+05, 9.192467E+05, 9.981526E+05, 1.082292E+06,
    1.171923E+06, 1.267315E+06, 1.368758E+06, 1.476543E+06, 1.590956E+06, 1.712308E+06,
    1.840917E+06, 1.977087E+06, 2.121159E+06, 2.273473E+06, 2.434375E+06, 2.604205E+06,
    2.783338E+06, 2.972142E+06, 3.171006E+06, 3.380315E+06, 3.600476E+06, 3.831875E+06,
    4.074950E+06, 4.330119E+06, 4.597824E+06, 4.878515E+06, 5.172635E+06, 5.480655E+06,
    5.803069E+06, 6.140331E+06, 6.492956E+06, 6.861430E+06, 7.246294E+06, 7.648073E+06,
    8.067282E+06, 8.504471E+06, 8.960216E+06, 9.435065E+06, 9.929615E+06, 1.044443E+07,
    1.098014E+07, 1.153735E+07, 1.211666E+07, 1.271871E+07, 1.334416E+07, 1.399362E+07,
    1.466782E+07, 1.536741E+07, 1.609309E+07, 1.684551E+07, 1.762546E+07, 1.843360E+07,
    1.927066E+07, 2.013746E+07, 2.103470E+07, 2.196320E+07, 2.292366E+07, 2.391697E+07,
    2.494387E+07, 2.600520E+07, 2.710179E+07, 2.823452E+07, 2.940420E+07, 3.061173E+07,
    3.185798E+07, 3.314380E+07, 3.447017E+07, 3.583801E+07, 3.724820E+07, 3.870172E+07,
    4.019952E+07, 4.174256E+07, 4.333188E+07, 4.496840E+07, 4.665318E+07, 4.838728E+07,
    5.017165E+07, 5.200740E+07, 5.389561E+07, 5.583735E+07, 5.783364E+07, 5.988569E+07,
    6.199461E+07, 6.416146E+07, 6.638749E+07, 6.867377E+07, 7.102154E+07, 7.343199E+07,
    7.590634E+07, 7.844571E+07, 8.105146E+07, 8.372481E+07, 8.646697E+07, 8.927926E+07,
    9.216301E+07, 9.511942E+07, 9.814997E+07, 1.012559E+08, 1.044386E+08, 1.076993E+08,
    1.110396E+08, 1.144608E+08, 1.179644E+08, 1.215516E+08, 1.252241E+08, 1.289833E+08,
    1.328306E+08, 1.367675E+08, 1.407957E+08, 1.449164E+08, 1.491314E+08, 1.534422E+08,
    1.578503E+08, 1.623573E+08, 1.669650E+08, 1.716746E+08, 1.764882E+08, 1.814073E+08,
    1.864334E+08, 1.915684E+08, 1.968138E+08, 2.021716E+08, 2.076433E+08, 2.132308E+08,
    2.189357E+08, 2.247600E+08, 2.307054E+08, 2.367737E+08, 2.429668E+08, 2.492865E+08,
    2.557347E+08, 2.623134E+08, 2.690243E+08, 2.758695E+08, 2.828508E+08, 2.899701E+08,
    2.972298E+08, 3.046315E+08, 3.121773E+08, 3.198694E+08, 3.277097E+08, 3.357002E+08,
    3.438432E+08, 3.521405E+08, 3.605945E+08, 3.692073E+08, 3.779810E+08, 3.869178E+08,
    3.960200E+08, 4.052896E+08,
])

#  ---------------------- M = 3, I = 17 ---------------------------
M = 3
I = 17
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.126567E+01, 2.362010E+03, 6.653739E+03, 1.221051E+04, 1.879316E+04, 2.626447E+04,
    3.453787E+04, 4.356149E+04, 5.331364E+04, 6.379765E+04, 7.503938E+04, 8.708147E+04,
    9.998024E+04, 1.138041E+05, 1.286285E+05, 1.445357E+05, 1.616128E+05, 1.799519E+05,
    1.996462E+05, 2.207944E+05, 2.434973E+05, 2.678588E+05, 2.939855E+05, 3.219866E+05,
    3.519763E+05, 3.840710E+05, 4.183868E+05, 4.550447E+05, 4.941742E+05, 5.359053E+05,
    5.803658E+05, 6.276941E+05, 6.780312E+05, 7.315213E+05, 7.883119E+05, 8.485581E+05,
    9.124080E+05, 9.800237E+05, 1.051575E+06, 1.127220E+06, 1.207142E+06, 1.291512E+06,
    1.380510E+06, 1.474325E+06, 1.573144E+06, 1.677168E+06, 1.786591E+06, 1.901612E+06,
    2.022451E+06, 2.149307E+06, 2.282409E+06, 2.421973E+06, 2.568228E+06, 2.721418E+06,
    2.881762E+06, 3.049503E+06, 3.224910E+06, 3.408205E+06, 3.599666E+06, 3.799557E+06,
    4.008133E+06, 4.225669E+06, 4.452440E+06, 4.688742E+06, 4.934864E+06, 5.191084E+06,
    5.457716E+06, 5.735050E+06, 6.023406E+06, 6.323116E+06, 6.634468E+06, 6.957803E+06,
    7.293467E+06, 7.641780E+06, 8.003085E+06, 8.377742E+06, 8.766122E+06, 9.168553E+06,
    9.585412E+06, 1.001709E+07, 1.046394E+07, 1.092639E+07, 1.140478E+07, 1.189953E+07,
    1.241103E+07, 1.293971E+07, 1.348598E+07, 1.405025E+07, 1.463297E+07, 1.523455E+07,
    1.585545E+07, 1.649609E+07, 1.715694E+07, 1.783848E+07, 1.854115E+07, 1.926541E+07,
    2.001179E+07, 2.078071E+07, 2.157272E+07, 2.238827E+07, 2.322791E+07, 2.409212E+07,
    2.498146E+07, 2.589638E+07, 2.683750E+07, 2.780528E+07, 2.880035E+07, 2.982319E+07,
    3.087441E+07, 3.195455E+07, 3.306416E+07, 3.420390E+07, 3.537427E+07, 3.657594E+07,
    3.780946E+07, 3.907546E+07, 4.037457E+07, 4.170740E+07, 4.307458E+07, 4.447674E+07,
    4.591456E+07, 4.738867E+07, 4.889974E+07, 5.044841E+07, 5.203540E+07, 5.366135E+07,
    5.532700E+07, 5.703306E+07, 5.878013E+07, 6.056905E+07, 6.240048E+07, 6.427519E+07,
    6.619386E+07, 6.815728E+07, 7.016617E+07, 7.222130E+07, 7.432351E+07, 7.647348E+07,
    7.867203E+07, 8.091997E+07, 8.321813E+07, 8.556722E+07, 8.796816E+07, 9.042171E+07,
    9.292876E+07, 9.549011E+07, 9.810667E+07, 1.007792E+08, 1.035087E+08, 1.062959E+08,
    1.091418E+08, 1.120472E+08, 1.150131E+08, 1.180403E+08, 1.211298E+08, 1.242826E+08,
    1.274995E+08, 1.307815E+08, 1.341295E+08, 1.375446E+08, 1.410276E+08, 1.445796E+08,
    1.482015E+08, 1.518944E+08, 1.556593E+08, 1.594971E+08, 1.634089E+08, 1.673958E+08,
    1.714587E+08, 1.755987E+08, 1.798170E+08, 1.841145E+08, 1.884924E+08, 1.929516E+08,
    1.974934E+08, 2.021189E+08,
])

#  ---------------------- M = 3, I = 18 ---------------------------
M = 3
I = 18
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.837627E+02, 1.387967E+04, 3.909803E+04, 7.174973E+04, 1.104283E+05, 1.543309E+05,
    2.029434E+05, 2.559580E+05, 3.132406E+05, 3.748050E+05, 4.407836E+05, 5.114192E+05,
    5.870305E+05, 6.680003E+05, 7.547560E+05, 8.477726E+05, 9.475368E+05, 1.054582E+06,
    1.169449E+06, 1.292692E+06, 1.424898E+06, 1.566662E+06, 1.718594E+06, 1.881319E+06,
    2.055491E+06, 2.241760E+06, 2.440823E+06, 2.653357E+06, 2.880106E+06, 3.121812E+06,
    3.379209E+06, 3.653089E+06, 3.944236E+06, 4.253526E+06, 4.581753E+06, 4.929803E+06,
    5.298553E+06, 5.688930E+06, 6.101879E+06, 6.538337E+06, 6.999325E+06, 7.485814E+06,
    7.998834E+06, 8.539473E+06, 9.108804E+06, 9.707966E+06, 1.033803E+07, 1.100023E+07,
    1.169569E+07, 1.242568E+07, 1.319142E+07, 1.399419E+07, 1.483526E+07, 1.571596E+07,
    1.663761E+07, 1.760170E+07, 1.860954E+07, 1.966258E+07, 2.076236E+07, 2.191031E+07,
    2.310791E+07, 2.435683E+07, 2.565854E+07, 2.701471E+07, 2.842701E+07, 2.989710E+07,
    3.142661E+07, 3.301744E+07, 3.467120E+07, 3.638967E+07, 3.817485E+07, 4.002844E+07,
    4.195248E+07, 4.394874E+07, 4.601924E+07, 4.816607E+07, 5.039106E+07, 5.269646E+07,
    5.508422E+07, 5.755659E+07, 6.011559E+07, 6.276353E+07, 6.550260E+07, 6.833496E+07,
    7.126300E+07, 7.428912E+07, 7.741560E+07, 8.064479E+07, 8.397913E+07, 8.742121E+07,
    9.097347E+07, 9.463835E+07, 9.841865E+07, 1.023169E+08, 1.063354E+08, 1.104775E+08,
    1.147455E+08, 1.191421E+08, 1.236703E+08, 1.283330E+08, 1.331328E+08, 1.380728E+08,
    1.431559E+08, 1.483852E+08, 1.537637E+08, 1.592942E+08, 1.649801E+08, 1.708245E+08,
    1.768305E+08, 1.830015E+08, 1.893405E+08, 1.958510E+08, 2.025364E+08, 2.093999E+08,
    2.164450E+08, 2.236751E+08, 2.310939E+08, 2.387049E+08, 2.465116E+08, 2.545176E+08,
    2.627268E+08, 2.711426E+08, 2.797689E+08, 2.886098E+08, 2.976687E+08, 3.069497E+08,
    3.164567E+08, 3.261937E+08, 3.361646E+08, 3.463737E+08, 3.568248E+08, 3.675224E+08,
    3.784706E+08, 3.896733E+08, 4.011353E+08, 4.128607E+08, 4.248538E+08, 4.371189E+08,
    4.496609E+08, 4.624839E+08, 4.755926E+08, 4.889916E+08, 5.026858E+08, 5.166797E+08,
    5.309777E+08, 5.455850E+08, 5.605064E+08, 5.757467E+08, 5.913112E+08, 6.072043E+08,
    6.234311E+08, 6.399973E+08, 6.569072E+08, 6.741666E+08, 6.917803E+08, 7.097538E+08,
    7.280923E+08, 7.468014E+08, 7.658867E+08, 7.853528E+08, 8.052060E+08, 8.254517E+08,
    8.460953E+08, 8.671429E+08, 8.885996E+08, 9.104718E+08, 9.327650E+08, 9.554851E+08,
    9.786380E+08, 1.002230E+09, 1.026267E+09, 1.050754E+09, 1.075698E+09, 1.101107E+09,
    1.126984E+09, 1.153337E+09,
])

#  ---------------------- M = 4, I = 1 ---------------------------
M = 4
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.834054E+01, 3.016016E+02, 6.001924E+02, 8.988074E+02, 1.197498E+03, 1.496709E+03,
    1.797823E+03, 2.103285E+03, 2.416237E+03, 2.740046E+03, 3.077980E+03, 3.433067E+03,
    3.808070E+03, 4.205517E+03, 4.627758E+03, 5.077011E+03, 5.555411E+03, 6.065047E+03,
    6.607987E+03, 7.186308E+03, 7.802105E+03, 8.457512E+03, 9.154706E+03, 9.895923E+03,
    1.068346E+04, 1.151968E+04, 1.240701E+04, 1.334798E+04, 1.434516E+04, 1.540125E+04,
    1.651898E+04, 1.770123E+04, 1.895091E+04, 2.027106E+04, 2.166480E+04, 2.313534E+04,
    2.468602E+04, 2.632023E+04, 2.804150E+04, 2.985343E+04, 3.175977E+04, 3.376432E+04,
    3.587102E+04, 3.808392E+04, 4.040717E+04, 4.284502E+04, 4.540186E+04, 4.808216E+04,
    5.089054E+04, 5.383169E+04, 5.691045E+04, 6.013176E+04, 6.350068E+04, 6.702236E+04,
    7.070211E+04, 7.454531E+04, 7.855746E+04, 8.274419E+04, 8.711122E+04, 9.166437E+04,
    9.640958E+04, 1.013529E+05, 1.065004E+05, 1.118584E+05, 1.174331E+05, 1.232310E+05,
    1.292585E+05, 1.355222E+05, 1.420287E+05, 1.487848E+05, 1.557971E+05, 1.630726E+05,
    1.706179E+05, 1.784402E+05, 1.865463E+05, 1.949432E+05, 2.036378E+05, 2.126373E+05,
    2.219487E+05, 2.315789E+05, 2.415351E+05, 2.518242E+05, 2.624534E+05, 2.734295E+05,
    2.847595E+05, 2.964505E+05, 3.085092E+05, 3.209425E+05, 3.337573E+05, 3.469602E+05,
    3.605579E+05, 3.745571E+05, 3.889642E+05, 4.037857E+05, 4.190280E+05, 4.346973E+05,
    4.507998E+05, 4.673415E+05, 4.843284E+05, 5.017665E+05, 5.196613E+05, 5.380185E+05,
    5.568437E+05, 5.761421E+05, 5.959191E+05, 6.161798E+05, 6.369291E+05, 6.581719E+05,
    6.799128E+05, 7.021566E+05, 7.249075E+05, 7.481698E+05, 7.719478E+05, 7.962454E+05,
    8.210663E+05, 8.464145E+05, 8.722932E+05, 8.987061E+05, 9.256562E+05, 9.531467E+05,
    9.811806E+05, 1.009761E+06, 1.038889E+06, 1.068569E+06, 1.098803E+06, 1.129593E+06,
    1.160940E+06, 1.192847E+06, 1.225316E+06, 1.258347E+06, 1.291944E+06, 1.326106E+06,
    1.360835E+06, 1.396133E+06, 1.431999E+06, 1.468436E+06, 1.505443E+06, 1.543020E+06,
    1.581169E+06, 1.619890E+06, 1.659183E+06, 1.699047E+06, 1.739482E+06, 1.780489E+06,
    1.822067E+06, 1.864215E+06, 1.906934E+06, 1.950221E+06, 1.994077E+06, 2.038500E+06,
    2.083489E+06, 2.129044E+06, 2.175162E+06, 2.221843E+06, 2.269086E+06, 2.316888E+06,
    2.365247E+06, 2.414164E+06, 2.463635E+06, 2.513658E+06, 2.564232E+06, 2.615355E+06,
    2.667024E+06, 2.719237E+06, 2.771992E+06, 2.825287E+06, 2.879119E+06, 2.933486E+06,
    2.988385E+06, 3.043814E+06, 3.099769E+06, 3.156248E+06, 3.213249E+06, 3.270768E+06,
    3.328803E+06, 3.387350E+06, 3.446406E+06, 3.505969E+06, 3.566036E+06, 3.626603E+06,
    3.687666E+06, 3.749224E+06, 3.811272E+06, 3.873807E+06, 3.936826E+06, 4.000325E+06,
    4.064302E+06, 4.128752E+06, 4.193672E+06, 4.259058E+06, 4.324908E+06, 4.391217E+06,
    4.457982E+06, 4.525200E+06, 4.592866E+06, 4.660977E+06, 4.729530E+06, 4.798520E+06,
    4.867944E+06, 4.937799E+06, 5.008080E+06, 5.078784E+06, 5.149907E+06, 5.221446E+06,
    5.293396E+06, 5.365753E+06, 5.438515E+06, 5.511677E+06, 5.585236E+06, 5.659187E+06,
    5.733527E+06, 5.808252E+06, 5.883358E+06, 5.958842E+06, 6.034699E+06, 6.110926E+06,
    6.187520E+06, 6.264475E+06, 6.341789E+06, 6.419458E+06, 6.497478E+06, 6.575844E+06,
    6.654554E+06, 6.733604E+06, 6.812990E+06, 6.892708E+06, 6.972754E+06, 7.053125E+06,
    7.133816E+06, 7.214826E+06, 7.296148E+06, 7.377781E+06, 7.459719E+06, 7.541961E+06,
    7.624501E+06, 7.707336E+06, 7.790463E+06, 7.873879E+06, 7.957578E+06, 8.041559E+06,
    8.125817E+06, 8.210349E+06, 8.295151E+06, 8.380220E+06, 8.465552E+06, 8.551144E+06,
    8.636993E+06, 8.723094E+06, 8.809445E+06, 8.896042E+06, 8.982882E+06,
])

#  ---------------------- M = 4, I = 2 ---------------------------
M = 4
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.222856E+01, 2.011405E+02, 4.003536E+02, 5.996615E+02, 7.991092E+02, 9.990614E+02,
    1.200554E+03, 1.405397E+03, 1.615831E+03, 1.834200E+03, 2.062784E+03, 2.303629E+03,
    2.558624E+03, 2.829471E+03, 3.117766E+03, 3.425050E+03, 3.752753E+03, 4.102350E+03,
    4.475234E+03, 4.872840E+03, 5.296650E+03, 5.748165E+03, 6.228913E+03, 6.740417E+03,
    7.284339E+03, 7.862305E+03, 8.476054E+03, 9.127358E+03, 9.818007E+03, 1.054988E+04,
    1.132500E+04, 1.214527E+04, 1.301286E+04, 1.392980E+04, 1.489831E+04, 1.592072E+04,
    1.699933E+04, 1.813650E+04, 1.933465E+04, 2.059648E+04, 2.192444E+04, 2.332135E+04,
    2.478986E+04, 2.633284E+04, 2.795313E+04, 2.965383E+04, 3.143800E+04, 3.330869E+04,
    3.526918E+04, 3.732275E+04, 3.947275E+04, 4.172267E+04, 4.407610E+04, 4.653671E+04,
    4.910823E+04, 5.179425E+04, 5.459898E+04, 5.752616E+04, 6.058007E+04, 6.376470E+04,
    6.708449E+04, 7.054360E+04, 7.414663E+04, 7.789815E+04, 8.180281E+04, 8.586522E+04,
    9.009033E+04, 9.448303E+04, 9.904846E+04, 1.037917E+05, 1.087180E+05, 1.138327E+05,
    1.191414E+05, 1.246497E+05, 1.303630E+05, 1.362872E+05, 1.424284E+05, 1.487924E+05,
    1.553853E+05, 1.622135E+05, 1.692834E+05, 1.766012E+05, 1.841737E+05, 1.920075E+05,
    2.001097E+05, 2.084867E+05, 2.171461E+05, 2.260948E+05, 2.353401E+05, 2.448897E+05,
    2.547509E+05, 2.649314E+05, 2.754390E+05, 2.862820E+05, 2.974681E+05, 3.090057E+05,
    3.209029E+05, 3.331687E+05, 3.458111E+05, 3.588390E+05, 3.722616E+05, 3.860877E+05,
    4.003264E+05, 4.149873E+05, 4.300793E+05, 4.456126E+05, 4.615966E+05, 4.780413E+05,
    4.949566E+05, 5.123527E+05, 5.302399E+05, 5.486289E+05, 5.675301E+05, 5.869545E+05,
    6.069124E+05, 6.274157E+05, 6.484752E+05, 6.701022E+05, 6.923088E+05, 7.151059E+05,
    7.385061E+05, 7.625212E+05, 7.871631E+05, 8.124445E+05, 8.383776E+05, 8.649755E+05,
    8.922511E+05, 9.202168E+05, 9.488864E+05, 9.782731E+05, 1.008390E+06, 1.039252E+06,
    1.070871E+06, 1.103263E+06, 1.136442E+06, 1.170421E+06, 1.205216E+06, 1.240841E+06,
    1.277312E+06, 1.314642E+06, 1.352849E+06, 1.391947E+06, 1.431951E+06, 1.472878E+06,
    1.514745E+06, 1.557566E+06, 1.601359E+06, 1.646140E+06, 1.691926E+06, 1.738734E+06,
    1.786582E+06, 1.835486E+06, 1.885465E+06, 1.936536E+06, 1.988717E+06, 2.042026E+06,
    2.096482E+06, 2.152104E+06, 2.208910E+06, 2.266919E+06, 2.326151E+06, 2.386625E+06,
    2.448361E+06, 2.511378E+06, 2.575697E+06, 2.641338E+06, 2.708322E+06, 2.776668E+06,
    2.846400E+06, 2.917536E+06, 2.990100E+06, 3.064111E+06, 3.139593E+06, 3.216567E+06,
    3.295056E+06, 3.375082E+06,
])

#  ---------------------- M = 4, I = 3 ---------------------------
M = 4
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.256526E+01, 2.080869E+02, 4.142500E+02, 6.205111E+02, 8.269083E+02, 1.033772E+03,
    1.242093E+03, 1.453602E+03, 1.670525E+03, 1.895214E+03, 2.129981E+03, 2.376914E+03,
    2.637982E+03, 2.914901E+03, 3.209368E+03, 3.522887E+03, 3.857001E+03, 4.213185E+03,
    4.592889E+03, 4.997581E+03, 5.428733E+03, 5.887874E+03, 6.376560E+03, 6.896343E+03,
    7.448865E+03, 8.035829E+03, 8.658960E+03, 9.319999E+03, 1.002082E+04, 1.076333E+04,
    1.154942E+04, 1.238119E+04, 1.326072E+04, 1.419008E+04, 1.517143E+04, 1.620721E+04,
    1.729969E+04, 1.845127E+04, 1.966438E+04, 2.094161E+04, 2.228561E+04, 2.369906E+04,
    2.518473E+04, 2.674545E+04, 2.838410E+04, 3.010380E+04, 3.190749E+04, 3.379832E+04,
    3.577964E+04, 3.785476E+04, 4.002697E+04, 4.229982E+04, 4.467683E+04, 4.716170E+04,
    4.975823E+04, 5.247004E+04, 5.530117E+04, 5.825564E+04, 6.133755E+04, 6.455106E+04,
    6.790052E+04, 7.139005E+04, 7.502442E+04, 7.880813E+04, 8.274562E+04, 8.684195E+04,
    9.110174E+04, 9.553009E+04, 1.001320E+05, 1.049128E+05, 1.098775E+05, 1.150316E+05,
    1.203805E+05, 1.259299E+05, 1.316857E+05, 1.376532E+05, 1.438385E+05, 1.502479E+05,
    1.568873E+05, 1.637628E+05, 1.708812E+05, 1.782486E+05, 1.858719E+05, 1.937574E+05,
    2.019124E+05, 2.103435E+05, 2.190582E+05, 2.280632E+05, 2.373661E+05, 2.469743E+05,
    2.568955E+05, 2.671372E+05, 2.777073E+05, 2.886138E+05, 2.998648E+05, 3.114686E+05,
    3.234334E+05, 3.357677E+05, 3.484802E+05, 3.615796E+05, 3.750749E+05, 3.889752E+05,
    4.032894E+05, 4.180269E+05, 4.331973E+05, 4.488102E+05, 4.648751E+05, 4.814022E+05,
    4.984014E+05, 5.158830E+05, 5.338571E+05, 5.523341E+05, 5.713253E+05, 5.908406E+05,
    6.108916E+05, 6.314891E+05, 6.526444E+05, 6.743689E+05, 6.966742E+05, 7.195719E+05,
    7.430743E+05, 7.671928E+05, 7.919402E+05, 8.173284E+05, 8.433701E+05, 8.700783E+05,
    8.974653E+05, 9.255449E+05, 9.543294E+05, 9.838329E+05, 1.014069E+06, 1.045050E+06,
    1.076792E+06, 1.109308E+06, 1.142611E+06, 1.176718E+06, 1.211642E+06, 1.247397E+06,
    1.284000E+06, 1.321465E+06, 1.359807E+06, 1.399043E+06, 1.439187E+06, 1.480256E+06,
    1.522265E+06, 1.565232E+06, 1.609171E+06, 1.654101E+06, 1.700038E+06, 1.746998E+06,
    1.795000E+06, 1.844060E+06, 1.894197E+06, 1.945428E+06, 1.997770E+06, 2.051243E+06,
    2.105865E+06, 2.161654E+06, 2.218630E+06, 2.276810E+06, 2.336216E+06, 2.396864E+06,
    2.458777E+06, 2.521974E+06, 2.586474E+06, 2.652298E+06, 2.719466E+06, 2.788000E+06,
    2.857920E+06, 2.929247E+06, 3.002003E+06, 3.076209E+06, 3.151888E+06, 3.229061E+06,
    3.307750E+06, 3.387980E+06,
])

#  ---------------------- M = 4, I = 4 ---------------------------
M = 4
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.920009E+01, 3.193816E+02, 6.358818E+02, 9.525320E+02, 1.269392E+03, 1.586983E+03,
    1.906819E+03, 2.231621E+03, 2.564811E+03, 2.910018E+03, 3.270780E+03, 3.650417E+03,
    4.051873E+03, 4.477876E+03, 4.931017E+03, 5.413766E+03, 5.928387E+03, 6.477164E+03,
    7.062454E+03, 7.686493E+03, 8.351633E+03, 9.060190E+03, 9.814583E+03, 1.061722E+04,
    1.147081E+04, 1.237778E+04, 1.334087E+04, 1.436287E+04, 1.544662E+04, 1.659506E+04,
    1.781131E+04, 1.909836E+04, 2.045953E+04, 2.189821E+04, 2.341770E+04, 2.502151E+04,
    2.671353E+04, 2.849723E+04, 3.037651E+04, 3.235551E+04, 3.443819E+04, 3.662862E+04,
    3.893132E+04, 4.135055E+04, 4.389090E+04, 4.655709E+04, 4.935379E+04, 5.228607E+04,
    5.535866E+04, 5.857697E+04, 6.194628E+04, 6.547190E+04, 6.915948E+04, 7.301461E+04,
    7.704295E+04, 8.125083E+04, 8.564401E+04, 9.022885E+04, 9.501173E+04, 9.999909E+04,
    1.051977E+05, 1.106143E+05, 1.162557E+05, 1.221292E+05, 1.282420E+05, 1.346015E+05,
    1.412153E+05, 1.480910E+05, 1.552365E+05, 1.626599E+05, 1.703693E+05, 1.783731E+05,
    1.866800E+05, 1.952985E+05, 2.042375E+05, 2.135062E+05, 2.231135E+05, 2.330688E+05,
    2.433819E+05, 2.540625E+05, 2.651203E+05, 2.765653E+05, 2.884082E+05, 3.006592E+05,
    3.133289E+05, 3.264282E+05, 3.399683E+05, 3.539598E+05, 3.684149E+05, 3.833445E+05,
    3.987610E+05, 4.146759E+05, 4.311017E+05, 4.480505E+05, 4.655352E+05, 4.835685E+05,
    5.021632E+05, 5.213328E+05, 5.410906E+05, 5.614503E+05, 5.824256E+05, 6.040307E+05,
    6.262797E+05, 6.491874E+05, 6.727680E+05, 6.970374E+05, 7.220099E+05, 7.477012E+05,
    7.741267E+05, 8.013024E+05, 8.292446E+05, 8.579696E+05, 8.874938E+05, 9.178335E+05,
    9.490066E+05, 9.810301E+05, 1.013921E+06, 1.047698E+06, 1.082378E+06, 1.117980E+06,
    1.154522E+06, 1.192024E+06, 1.230503E+06, 1.269980E+06, 1.310473E+06, 1.352004E+06,
    1.394590E+06, 1.438254E+06, 1.483015E+06, 1.528894E+06, 1.575913E+06, 1.624092E+06,
    1.673454E+06, 1.724020E+06, 1.775813E+06, 1.828855E+06, 1.883168E+06, 1.938776E+06,
    1.995702E+06, 2.053969E+06, 2.113602E+06, 2.174624E+06, 2.237060E+06, 2.300936E+06,
    2.366274E+06, 2.433103E+06, 2.501445E+06, 2.571329E+06, 2.642780E+06, 2.715823E+06,
    2.790487E+06, 2.866799E+06, 2.944785E+06, 3.024473E+06, 3.105892E+06, 3.189071E+06,
    3.274037E+06, 3.360820E+06, 3.449449E+06, 3.539953E+06, 3.632364E+06, 3.726710E+06,
    3.823023E+06, 3.921334E+06, 4.021673E+06, 4.124073E+06, 4.228565E+06, 4.335183E+06,
    4.443957E+06, 4.554922E+06, 4.668111E+06, 4.783556E+06, 4.901294E+06, 5.021357E+06,
    5.143780E+06, 5.268599E+06,
])

#  ---------------------- M = 4, I = 5 ---------------------------
M = 4
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.126859E+02, 1.864509E+03, 3.711701E+03, 5.559770E+03, 7.409060E+03, 9.262436E+03,
    1.112884E+04, 1.302350E+04, 1.496642E+04, 1.697860E+04, 1.908053E+04, 2.129100E+04,
    2.362782E+04, 2.610612E+04, 2.874132E+04, 3.154718E+04, 3.453696E+04, 3.772415E+04,
    4.112140E+04, 4.474253E+04, 4.860004E+04, 5.270834E+04, 5.708061E+04, 6.173070E+04,
    6.667417E+04, 7.192540E+04, 7.749930E+04, 8.341287E+04, 8.968139E+04, 9.632295E+04,
    1.033539E+05, 1.107929E+05, 1.186581E+05, 1.269691E+05, 1.357447E+05, 1.450058E+05,
    1.547736E+05, 1.650686E+05, 1.759141E+05, 1.873318E+05, 1.993451E+05, 2.119782E+05,
    2.252566E+05, 2.392045E+05, 2.538486E+05, 2.692152E+05, 2.853310E+05, 3.022263E+05,
    3.199273E+05, 3.384653E+05, 3.578704E+05, 3.781728E+05, 3.994046E+05, 4.215990E+05,
    4.447877E+05, 4.690061E+05, 4.942895E+05, 5.206713E+05, 5.481903E+05, 5.768826E+05,
    6.067872E+05, 6.379417E+05, 6.703877E+05, 7.041640E+05, 7.393128E+05, 7.758767E+05,
    8.138982E+05, 8.534227E+05, 8.944954E+05, 9.371609E+05, 9.814658E+05, 1.027460E+06,
    1.075192E+06, 1.124709E+06, 1.176064E+06, 1.229308E+06, 1.284495E+06, 1.341676E+06,
    1.400908E+06, 1.462245E+06, 1.525743E+06, 1.591463E+06, 1.659462E+06, 1.729799E+06,
    1.802536E+06, 1.877734E+06, 1.955456E+06, 2.035768E+06, 2.118733E+06, 2.204417E+06,
    2.292890E+06, 2.384219E+06, 2.478473E+06, 2.575724E+06, 2.676044E+06, 2.779507E+06,
    2.886184E+06, 2.996153E+06, 3.109493E+06, 3.226277E+06, 3.346586E+06, 3.470502E+06,
    3.598108E+06, 3.729483E+06, 3.864712E+06, 4.003881E+06, 4.147080E+06, 4.294392E+06,
    4.445907E+06, 4.601719E+06, 4.761916E+06, 4.926595E+06, 5.095849E+06, 5.269773E+06,
    5.448463E+06, 5.632024E+06, 5.820548E+06, 6.014144E+06, 6.212908E+06, 6.416949E+06,
    6.626371E+06, 6.841280E+06, 7.061789E+06, 7.288003E+06, 7.520035E+06, 7.758000E+06,
    8.002011E+06, 8.252184E+06, 8.508638E+06, 8.771489E+06, 9.040859E+06, 9.316869E+06,
    9.599643E+06, 9.889311E+06, 1.018599E+07, 1.048982E+07, 1.080092E+07, 1.111943E+07,
    1.144548E+07, 1.177920E+07, 1.212074E+07, 1.247022E+07, 1.282778E+07, 1.319359E+07,
    1.356776E+07, 1.395045E+07, 1.434180E+07, 1.474197E+07, 1.515110E+07, 1.556933E+07,
    1.599684E+07, 1.643377E+07, 1.688027E+07, 1.733652E+07, 1.780266E+07, 1.827886E+07,
    1.876528E+07, 1.926210E+07, 1.976946E+07, 2.028756E+07, 2.081655E+07, 2.135661E+07,
    2.190792E+07, 2.247066E+07, 2.304498E+07, 2.363109E+07, 2.422917E+07, 2.483940E+07,
    2.546196E+07, 2.609704E+07, 2.674484E+07, 2.740554E+07, 2.807935E+07, 2.876645E+07,
    2.946704E+07, 3.018133E+07,
])

#  ---------------------- M = 5, I = 1 ---------------------------
M = 5
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.011870E+00, 7.573520E+00, 1.480009E+01, 2.203056E+01, 2.926249E+01, 3.649540E+01,
    4.372913E+01, 5.096362E+01, 5.819883E+01, 6.543475E+01, 7.267137E+01, 7.990872E+01,
    8.714687E+01, 9.438597E+01, 1.016263E+02, 1.088684E+02, 1.161130E+02, 1.233610E+02,
    1.306138E+02, 1.378730E+02, 1.451405E+02, 1.524185E+02, 1.597093E+02, 1.670157E+02,
    1.743404E+02, 1.816863E+02, 1.890565E+02, 1.964539E+02, 2.038817E+02, 2.113428E+02,
    2.188405E+02, 2.263775E+02, 2.339567E+02, 2.415811E+02, 2.492532E+02, 2.569757E+02,
    2.647511E+02, 2.725817E+02, 2.804698E+02, 2.884176E+02, 2.964271E+02, 3.045004E+02,
    3.126391E+02, 3.208450E+02, 3.291199E+02, 3.374652E+02, 3.458824E+02, 3.543729E+02,
    3.629381E+02, 3.715790E+02, 3.802970E+02, 3.890930E+02, 3.979682E+02, 4.069234E+02,
    4.159597E+02, 4.250779E+02, 4.342787E+02, 4.435630E+02, 4.529316E+02, 4.623850E+02,
    4.719240E+02, 4.815492E+02, 4.912611E+02, 5.010603E+02, 5.109473E+02, 5.209227E+02,
    5.309869E+02, 5.411404E+02, 5.513836E+02, 5.617168E+02, 5.721405E+02, 5.826552E+02,
    5.932609E+02, 6.039583E+02, 6.147475E+02, 6.256289E+02, 6.366028E+02, 6.476694E+02,
    6.588291E+02, 6.700820E+02, 6.814285E+02, 6.928687E+02, 7.044030E+02, 7.160315E+02,
    7.277544E+02, 7.395719E+02, 7.514843E+02, 7.634917E+02, 7.755944E+02, 7.877924E+02,
    8.000860E+02, 8.124754E+02, 8.249606E+02, 8.375419E+02, 8.502194E+02, 8.629933E+02,
    8.758637E+02, 8.888307E+02, 9.018945E+02, 9.150553E+02, 9.283131E+02, 9.416680E+02,
    9.551203E+02, 9.686700E+02, 9.823173E+02, 9.960622E+02, 1.009905E+03, 1.023845E+03,
    1.037884E+03, 1.052021E+03, 1.066256E+03, 1.080589E+03, 1.095020E+03, 1.109551E+03,
    1.124179E+03, 1.138907E+03, 1.153733E+03, 1.168658E+03, 1.183682E+03, 1.198806E+03,
    1.214028E+03, 1.229350E+03, 1.244771E+03, 1.260292E+03, 1.275912E+03, 1.291632E+03,
    1.307451E+03, 1.323370E+03, 1.339390E+03, 1.355509E+03, 1.371728E+03, 1.388047E+03,
    1.404467E+03, 1.420987E+03, 1.437607E+03, 1.454327E+03, 1.471149E+03, 1.488070E+03,
    1.505093E+03, 1.522216E+03, 1.539440E+03, 1.556765E+03, 1.574191E+03, 1.591718E+03,
    1.609346E+03, 1.627076E+03, 1.644907E+03, 1.662839E+03, 1.680872E+03, 1.699007E+03,
    1.717244E+03, 1.735582E+03, 1.754022E+03, 1.772564E+03, 1.791208E+03, 1.809954E+03,
    1.828801E+03, 1.847751E+03, 1.866803E+03, 1.885957E+03, 1.905214E+03, 1.924573E+03,
    1.944034E+03, 1.963598E+03, 1.983264E+03, 2.003034E+03, 2.022906E+03, 2.042880E+03,
    2.062958E+03, 2.083138E+03, 2.103422E+03, 2.123809E+03, 2.144299E+03, 2.164892E+03,
    2.185588E+03, 2.206388E+03, 2.227291E+03, 2.248298E+03, 2.269408E+03, 2.290622E+03,
    2.311939E+03, 2.333361E+03, 2.354886E+03, 2.376515E+03, 2.398249E+03, 2.420086E+03,
    2.442027E+03, 2.464073E+03, 2.486223E+03, 2.508477E+03, 2.530836E+03, 2.553299E+03,
    2.575867E+03, 2.598539E+03, 2.621316E+03, 2.644198E+03, 2.667185E+03, 2.690276E+03,
    2.713473E+03, 2.736775E+03, 2.760181E+03, 2.783693E+03, 2.807310E+03, 2.831033E+03,
    2.854861E+03, 2.878794E+03, 2.902833E+03, 2.926977E+03, 2.951227E+03, 2.975583E+03,
    3.000045E+03, 3.024612E+03, 3.049286E+03, 3.074065E+03, 3.098951E+03, 3.123943E+03,
    3.149041E+03, 3.174245E+03, 3.199555E+03, 3.224973E+03, 3.250496E+03, 3.276126E+03,
    3.301863E+03, 3.327706E+03, 3.353657E+03, 3.379714E+03, 3.405878E+03, 3.432149E+03,
    3.458527E+03, 3.485012E+03, 3.511605E+03, 3.538304E+03, 3.565111E+03, 3.592026E+03,
    3.619048E+03, 3.646177E+03, 3.673414E+03, 3.700759E+03, 3.728212E+03, 3.755772E+03,
    3.783440E+03, 3.811216E+03, 3.839101E+03, 3.867093E+03, 3.895193E+03, 3.923402E+03,
    3.951719E+03, 3.980145E+03, 4.008679E+03, 4.037321E+03, 4.066072E+03, 4.094932E+03,
    4.123900E+03, 4.152977E+03, 4.182163E+03, 4.211458E+03, 4.240862E+03, 4.270375E+03,
    4.299997E+03, 4.329729E+03, 4.359569E+03, 4.389519E+03, 4.419579E+03, 4.449747E+03,
    4.480026E+03, 4.510414E+03, 4.540912E+03, 4.571519E+03, 4.602236E+03, 4.633064E+03,
    4.664001E+03, 4.695048E+03, 4.726205E+03, 4.757472E+03, 4.788850E+03, 4.820338E+03,
    4.851936E+03, 4.883645E+03, 4.915464E+03, 4.947394E+03, 4.979434E+03, 5.011585E+03,
    5.043847E+03, 5.076220E+03, 5.108703E+03, 5.141298E+03, 5.174003E+03, 5.206820E+03,
    5.239748E+03, 5.272787E+03, 5.305937E+03, 5.339199E+03, 5.372572E+03, 5.406056E+03,
    5.439652E+03, 5.473360E+03, 5.507179E+03, 5.541110E+03, 5.575153E+03, 5.609308E+03,
    5.643574E+03, 5.677953E+03, 5.712444E+03, 5.747047E+03, 5.781762E+03, 5.816589E+03,
    5.851528E+03, 5.886580E+03, 5.921745E+03, 5.957021E+03, 5.992411E+03, 6.027913E+03,
    6.063527E+03, 6.099255E+03, 6.135095E+03, 6.171048E+03, 6.207114E+03, 6.243293E+03,
    6.279585E+03, 6.315990E+03, 6.352508E+03, 6.389140E+03, 6.425885E+03, 6.462743E+03,
    6.499714E+03, 6.536799E+03, 6.573998E+03, 6.611310E+03, 6.648735E+03, 6.686275E+03,
    6.723928E+03, 6.761694E+03, 6.799575E+03, 6.837570E+03, 6.875678E+03, 6.913901E+03,
    6.952238E+03, 6.990688E+03, 7.029253E+03, 7.067933E+03, 7.106726E+03, 7.145634E+03,
    7.184656E+03, 7.223793E+03, 7.263044E+03, 7.302410E+03, 7.341891E+03, 7.381486E+03,
    7.421195E+03, 7.461020E+03, 7.500959E+03, 7.541014E+03, 7.581183E+03, 7.621467E+03,
    7.661866E+03, 7.702380E+03, 7.743010E+03, 7.783754E+03, 7.824614E+03, 7.865589E+03,
    7.906680E+03, 7.947885E+03, 7.989206E+03, 8.030643E+03, 8.072195E+03, 8.113863E+03,
    8.155646E+03, 8.197545E+03, 8.239560E+03, 8.281690E+03, 8.323936E+03, 8.366298E+03,
    8.408776E+03, 8.451369E+03, 8.494079E+03, 8.536905E+03, 8.579847E+03, 8.622904E+03,
    8.666078E+03, 8.709368E+03, 8.752775E+03, 8.796297E+03, 8.839936E+03, 8.883691E+03,
    8.927563E+03, 8.971551E+03, 9.015655E+03, 9.059876E+03, 9.104213E+03, 9.148667E+03,
    9.193238E+03, 9.237925E+03, 9.282729E+03, 9.327650E+03, 9.372687E+03, 9.417841E+03,
    9.463112E+03, 9.508500E+03, 9.554005E+03, 9.599626E+03, 9.645365E+03, 9.691221E+03,
    9.737193E+03, 9.783283E+03, 9.829489E+03, 9.875813E+03, 9.922254E+03, 9.968812E+03,
    1.001549E+04, 1.006228E+04, 1.010919E+04, 1.015622E+04, 1.020336E+04, 1.025062E+04,
    1.029800E+04, 1.034550E+04, 1.039311E+04, 1.044085E+04, 1.048869E+04, 1.053666E+04,
    1.058475E+04, 1.063295E+04, 1.068127E+04, 1.072970E+04, 1.077826E+04, 1.082693E+04,
    1.087572E+04, 1.092463E+04, 1.097365E+04, 1.102280E+04, 1.107206E+04, 1.112144E+04,
    1.117093E+04, 1.122055E+04, 1.127028E+04, 1.132013E+04, 1.137010E+04, 1.142018E+04,
    1.147039E+04, 1.152071E+04, 1.157115E+04, 1.162170E+04, 1.167238E+04, 1.172317E+04,
    1.177409E+04, 1.182512E+04, 1.187626E+04, 1.192753E+04, 1.197891E+04, 1.203042E+04,
    1.208204E+04,
])

#  ---------------------- M = 5, I = 2 ---------------------------
M = 5
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.030290E+00, 1.581148E+01, 3.093044E+01, 4.605695E+01, 6.118637E+01, 7.631778E+01,
    9.145088E+01, 1.065855E+02, 1.217217E+02, 1.368594E+02, 1.519985E+02, 1.671392E+02,
    1.822816E+02, 1.974261E+02, 2.125736E+02, 2.277251E+02, 2.428826E+02, 2.580484E+02,
    2.732257E+02, 2.884181E+02, 3.036302E+02, 3.188669E+02, 3.341338E+02, 3.494367E+02,
    3.647819E+02, 3.801760E+02, 3.956256E+02, 4.111372E+02, 4.267178E+02, 4.423738E+02,
    4.581119E+02, 4.739383E+02, 4.898593E+02, 5.058809E+02, 5.220087E+02, 5.382483E+02,
    5.546050E+02, 5.710838E+02, 5.876893E+02, 6.044262E+02, 6.212987E+02, 6.383109E+02,
    6.554665E+02, 6.727693E+02, 6.902226E+02, 7.078295E+02, 7.255932E+02, 7.435165E+02,
    7.616020E+02, 7.798524E+02, 7.982699E+02, 8.168568E+02, 8.356152E+02, 8.545471E+02,
    8.736544E+02, 8.929388E+02, 9.124020E+02, 9.320457E+02, 9.518712E+02, 9.718800E+02,
    9.920733E+02, 1.012453E+03, 1.033019E+03, 1.053773E+03, 1.074717E+03, 1.095851E+03,
    1.117176E+03, 1.138693E+03, 1.160404E+03, 1.182308E+03, 1.204406E+03, 1.226700E+03,
    1.249191E+03, 1.271877E+03, 1.294762E+03, 1.317844E+03, 1.341125E+03, 1.364605E+03,
    1.388285E+03, 1.412165E+03, 1.436246E+03, 1.460528E+03, 1.485012E+03, 1.509698E+03,
    1.534586E+03, 1.559678E+03, 1.584973E+03, 1.610472E+03, 1.636175E+03, 1.662083E+03,
    1.688195E+03, 1.714513E+03, 1.741037E+03, 1.767766E+03, 1.794701E+03, 1.821843E+03,
    1.849192E+03, 1.876748E+03, 1.904512E+03, 1.932483E+03, 1.960662E+03, 1.989049E+03,
    2.017644E+03, 2.046449E+03, 2.075462E+03, 2.104685E+03, 2.134116E+03, 2.163758E+03,
    2.193609E+03, 2.223671E+03, 2.253943E+03, 2.284425E+03, 2.315119E+03, 2.346023E+03,
    2.377138E+03, 2.408465E+03, 2.440003E+03, 2.471753E+03, 2.503715E+03, 2.535889E+03,
    2.568276E+03, 2.600875E+03, 2.633687E+03, 2.666711E+03, 2.699949E+03, 2.733400E+03,
    2.767065E+03, 2.800943E+03, 2.835035E+03, 2.869341E+03, 2.903861E+03, 2.938595E+03,
    2.973544E+03, 3.008708E+03, 3.044086E+03, 3.079680E+03, 3.115488E+03, 3.151512E+03,
    3.187752E+03, 3.224207E+03, 3.260878E+03, 3.297764E+03, 3.334867E+03, 3.372187E+03,
    3.409722E+03, 3.447475E+03, 3.485444E+03, 3.523630E+03, 3.562033E+03, 3.600653E+03,
    3.639491E+03, 3.678547E+03, 3.717820E+03, 3.757310E+03, 3.797019E+03, 3.836946E+03,
    3.877092E+03, 3.917456E+03, 3.958038E+03, 3.998839E+03, 4.039860E+03, 4.081099E+03,
    4.122557E+03, 4.164235E+03, 4.206133E+03, 4.248250E+03, 4.290587E+03, 4.333143E+03,
    4.375921E+03, 4.418918E+03, 4.462136E+03, 4.505574E+03, 4.549233E+03, 4.593113E+03,
    4.637214E+03, 4.681536E+03, 4.726079E+03, 4.770844E+03, 4.815831E+03, 4.861039E+03,
    4.906469E+03, 4.952121E+03, 4.997996E+03, 5.044093E+03, 5.090412E+03, 5.136954E+03,
    5.183719E+03, 5.230707E+03, 5.277917E+03, 5.325351E+03, 5.373009E+03, 5.420890E+03,
    5.468995E+03, 5.517323E+03, 5.565876E+03, 5.614652E+03, 5.663653E+03, 5.712879E+03,
    5.762329E+03, 5.812003E+03, 5.861903E+03, 5.912027E+03, 5.962377E+03, 6.012952E+03,
    6.063752E+03, 6.114778E+03, 6.166030E+03, 6.217508E+03, 6.269212E+03, 6.321142E+03,
    6.373298E+03, 6.425681E+03, 6.478290E+03, 6.531126E+03, 6.584189E+03, 6.637479E+03,
    6.690997E+03, 6.744742E+03, 6.798714E+03, 6.852914E+03, 6.907342E+03, 6.961997E+03,
    7.016881E+03, 7.071993E+03, 7.127334E+03, 7.182903E+03, 7.238700E+03, 7.294727E+03,
    7.350982E+03, 7.407467E+03, 7.464181E+03, 7.521124E+03, 7.578297E+03, 7.635699E+03,
    7.693332E+03, 7.751194E+03, 7.809287E+03, 7.867610E+03, 7.926163E+03, 7.984947E+03,
    8.043962E+03, 8.103207E+03, 8.162684E+03, 8.222391E+03, 8.282330E+03, 8.342501E+03,
    8.402903E+03, 8.463537E+03, 8.524402E+03, 8.585500E+03, 8.646830E+03, 8.708392E+03,
    8.770187E+03, 8.832214E+03, 8.894474E+03, 8.956967E+03, 9.019693E+03, 9.082652E+03,
    9.145844E+03, 9.209270E+03, 9.272930E+03, 9.336823E+03, 9.400950E+03, 9.465311E+03,
    9.529906E+03, 9.594736E+03, 9.659800E+03, 9.725098E+03, 9.790631E+03, 9.856400E+03,
    9.922403E+03, 9.988641E+03, 1.005511E+04, 1.012182E+04, 1.018877E+04, 1.025595E+04,
    1.032336E+04, 1.039102E+04, 1.045890E+04, 1.052703E+04, 1.059539E+04, 1.066399E+04,
    1.073282E+04, 1.080189E+04, 1.087120E+04, 1.094074E+04, 1.101052E+04, 1.108054E+04,
    1.115080E+04, 1.122129E+04, 1.129202E+04, 1.136299E+04, 1.143420E+04, 1.150565E+04,
    1.157733E+04, 1.164925E+04, 1.172141E+04, 1.179381E+04, 1.186645E+04, 1.193932E+04,
    1.201244E+04, 1.208579E+04, 1.215939E+04, 1.223322E+04, 1.230729E+04, 1.238161E+04,
    1.245616E+04, 1.253095E+04, 1.260598E+04, 1.268125E+04, 1.275676E+04, 1.283252E+04,
    1.290851E+04, 1.298474E+04, 1.306122E+04, 1.313793E+04, 1.321489E+04, 1.329208E+04,
    1.336952E+04, 1.344720E+04, 1.352512E+04, 1.360328E+04, 1.368169E+04, 1.376033E+04,
    1.383922E+04, 1.391835E+04, 1.399772E+04, 1.407733E+04, 1.415719E+04, 1.423729E+04,
    1.431763E+04, 1.439821E+04, 1.447903E+04, 1.456010E+04, 1.464141E+04, 1.472297E+04,
    1.480477E+04, 1.488681E+04, 1.496909E+04, 1.505162E+04, 1.513439E+04, 1.521740E+04,
    1.530066E+04, 1.538416E+04, 1.546791E+04, 1.555190E+04, 1.563614E+04, 1.572061E+04,
    1.580534E+04, 1.589030E+04, 1.597551E+04, 1.606097E+04, 1.614667E+04, 1.623262E+04,
    1.631881E+04, 1.640524E+04, 1.649192E+04, 1.657885E+04, 1.666602E+04, 1.675344E+04,
    1.684110E+04, 1.692901E+04, 1.701716E+04, 1.710556E+04, 1.719420E+04, 1.728309E+04,
    1.737222E+04, 1.746161E+04, 1.755123E+04, 1.764111E+04, 1.773123E+04, 1.782159E+04,
    1.791221E+04, 1.800306E+04, 1.809417E+04, 1.818552E+04, 1.827712E+04, 1.836897E+04,
    1.846106E+04, 1.855340E+04, 1.864598E+04, 1.873882E+04, 1.883190E+04, 1.892522E+04,
    1.901880E+04, 1.911262E+04, 1.920669E+04, 1.930100E+04, 1.939557E+04, 1.949038E+04,
    1.958544E+04, 1.968074E+04, 1.977629E+04, 1.987210E+04, 1.996815E+04, 2.006444E+04,
    2.016099E+04, 2.025778E+04, 2.035482E+04, 2.045211E+04, 2.054965E+04, 2.064743E+04,
    2.074547E+04, 2.084375E+04, 2.094228E+04, 2.104106E+04, 2.114008E+04, 2.123936E+04,
    2.133888E+04, 2.143865E+04, 2.153867E+04, 2.163894E+04, 2.173946E+04, 2.184022E+04,
    2.194124E+04, 2.204250E+04, 2.214402E+04, 2.224578E+04, 2.234779E+04, 2.245005E+04,
    2.255255E+04, 2.265531E+04, 2.275832E+04, 2.286157E+04, 2.296507E+04, 2.306883E+04,
    2.317283E+04, 2.327708E+04, 2.338158E+04, 2.348633E+04, 2.359133E+04, 2.369657E+04,
    2.380207E+04, 2.390782E+04, 2.401381E+04, 2.412006E+04, 2.422655E+04, 2.433329E+04,
    2.444028E+04, 2.454752E+04, 2.465502E+04, 2.476276E+04, 2.487074E+04, 2.497898E+04,
    2.508747E+04, 2.519621E+04, 2.530519E+04, 2.541443E+04, 2.552392E+04, 2.563365E+04,
    2.574363E+04,
])

#  ---------------------- M = 5, I = 3 ---------------------------
M = 5
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.015450E+00, 7.934590E+00, 1.552296E+01, 2.311511E+01, 3.070871E+01, 3.830330E+01,
    4.589875E+01, 5.349498E+01, 6.109197E+01, 6.868969E+01, 7.628815E+01, 8.388739E+01,
    9.148750E+01, 9.908871E+01, 1.066914E+02, 1.142961E+02, 1.219039E+02, 1.295159E+02,
    1.371337E+02, 1.447592E+02, 1.523947E+02, 1.600426E+02, 1.677058E+02, 1.753873E+02,
    1.830902E+02, 1.908178E+02, 1.985735E+02, 2.063605E+02, 2.141824E+02, 2.220424E+02,
    2.299438E+02, 2.378898E+02, 2.458836E+02, 2.539281E+02, 2.620262E+02, 2.701807E+02,
    2.783942E+02, 2.866693E+02, 2.950083E+02, 3.034135E+02, 3.118870E+02, 3.204310E+02,
    3.290472E+02, 3.377375E+02, 3.465036E+02, 3.553472E+02, 3.642697E+02, 3.732725E+02,
    3.823570E+02, 3.915246E+02, 4.007762E+02, 4.101132E+02, 4.195365E+02, 4.290471E+02,
    4.386461E+02, 4.483341E+02, 4.581122E+02, 4.679810E+02, 4.779414E+02, 4.879940E+02,
    4.981395E+02, 5.083786E+02, 5.187117E+02, 5.291395E+02, 5.396626E+02, 5.502813E+02,
    5.609963E+02, 5.718079E+02, 5.827166E+02, 5.937229E+02, 6.048270E+02, 6.160294E+02,
    6.273304E+02, 6.387304E+02, 6.502297E+02, 6.618287E+02, 6.735275E+02, 6.853265E+02,
    6.972260E+02, 7.092263E+02, 7.213275E+02, 7.335299E+02, 7.458338E+02, 7.582394E+02,
    7.707469E+02, 7.833566E+02, 7.960685E+02, 8.088830E+02, 8.218001E+02, 8.348201E+02,
    8.479432E+02, 8.611695E+02, 8.744993E+02, 8.879325E+02, 9.014695E+02, 9.151104E+02,
    9.288553E+02, 9.427043E+02, 9.566576E+02, 9.707154E+02, 9.848778E+02, 9.991449E+02,
    1.013517E+03, 1.027994E+03, 1.042576E+03, 1.057263E+03, 1.072055E+03, 1.086953E+03,
    1.101957E+03, 1.117066E+03, 1.132281E+03, 1.147601E+03, 1.163028E+03, 1.178561E+03,
    1.194200E+03, 1.209945E+03, 1.225797E+03, 1.241756E+03, 1.257820E+03, 1.273992E+03,
    1.290270E+03, 1.306656E+03, 1.323148E+03, 1.339747E+03, 1.356454E+03, 1.373267E+03,
    1.390188E+03, 1.407217E+03, 1.424353E+03, 1.441596E+03, 1.458948E+03, 1.476407E+03,
    1.493974E+03, 1.511649E+03, 1.529431E+03, 1.547322E+03, 1.565322E+03, 1.583429E+03,
    1.601645E+03, 1.619969E+03, 1.638402E+03, 1.656944E+03, 1.675594E+03, 1.694353E+03,
    1.713221E+03, 1.732197E+03, 1.751283E+03, 1.770478E+03, 1.789782E+03, 1.809195E+03,
    1.828718E+03, 1.848350E+03, 1.868091E+03, 1.887942E+03, 1.907903E+03, 1.927973E+03,
    1.948153E+03, 1.968443E+03, 1.988843E+03, 2.009353E+03, 2.029973E+03, 2.050703E+03,
    2.071543E+03, 2.092494E+03, 2.113555E+03, 2.134727E+03, 2.156009E+03, 2.177402E+03,
    2.198905E+03, 2.220519E+03, 2.242244E+03, 2.264080E+03, 2.286027E+03, 2.308085E+03,
    2.330255E+03, 2.352535E+03, 2.374927E+03, 2.397430E+03, 2.420044E+03, 2.442771E+03,
    2.465608E+03, 2.488557E+03, 2.511619E+03, 2.534791E+03, 2.558076E+03, 2.581473E+03,
    2.604982E+03, 2.628603E+03, 2.652336E+03, 2.676181E+03, 2.700139E+03, 2.724209E+03,
    2.748391E+03, 2.772686E+03, 2.797094E+03, 2.821615E+03, 2.846248E+03, 2.870994E+03,
    2.895853E+03, 2.920825E+03, 2.945910E+03, 2.971108E+03, 2.996420E+03, 3.021844E+03,
    3.047382E+03, 3.073034E+03, 3.098799E+03, 3.124678E+03, 3.150670E+03, 3.176776E+03,
    3.202996E+03, 3.229330E+03, 3.255778E+03, 3.282340E+03, 3.309015E+03, 3.335806E+03,
    3.362710E+03, 3.389729E+03, 3.416862E+03, 3.444109E+03, 3.471471E+03, 3.498948E+03,
    3.526540E+03, 3.554246E+03, 3.582067E+03, 3.610003E+03, 3.638054E+03, 3.666220E+03,
    3.694501E+03, 3.722898E+03, 3.751409E+03, 3.780036E+03, 3.808779E+03, 3.837637E+03,
    3.866611E+03, 3.895700E+03, 3.924905E+03, 3.954226E+03, 3.983662E+03, 4.013215E+03,
    4.042883E+03, 4.072668E+03, 4.102569E+03, 4.132586E+03, 4.162720E+03, 4.192969E+03,
    4.223336E+03, 4.253818E+03, 4.284418E+03, 4.315134E+03, 4.345967E+03, 4.376916E+03,
    4.407983E+03, 4.439166E+03, 4.470467E+03, 4.501884E+03, 4.533419E+03, 4.565071E+03,
    4.596841E+03, 4.628727E+03, 4.660731E+03, 4.692853E+03, 4.725092E+03, 4.757449E+03,
    4.789924E+03, 4.822517E+03, 4.855227E+03, 4.888055E+03, 4.921002E+03, 4.954066E+03,
    4.987249E+03, 5.020550E+03, 5.053969E+03, 5.087506E+03, 5.121162E+03, 5.154936E+03,
    5.188829E+03, 5.222841E+03, 5.256971E+03, 5.291221E+03, 5.325588E+03, 5.360075E+03,
    5.394681E+03, 5.429406E+03, 5.464250E+03, 5.499213E+03, 5.534296E+03, 5.569497E+03,
    5.604818E+03, 5.640259E+03, 5.675819E+03, 5.711498E+03, 5.747298E+03, 5.783217E+03,
    5.819255E+03, 5.855414E+03, 5.891692E+03, 5.928090E+03, 5.964609E+03, 6.001247E+03,
    6.038006E+03, 6.074885E+03, 6.111884E+03, 6.149003E+03, 6.186243E+03, 6.223603E+03,
    6.261084E+03, 6.298685E+03, 6.336407E+03, 6.374250E+03, 6.412213E+03, 6.450298E+03,
    6.488503E+03, 6.526829E+03, 6.565276E+03, 6.603844E+03, 6.642533E+03, 6.681344E+03,
    6.720275E+03, 6.759328E+03, 6.798502E+03, 6.837798E+03, 6.877215E+03, 6.916754E+03,
    6.956414E+03, 6.996196E+03, 7.036099E+03, 7.076125E+03, 7.116272E+03, 7.156541E+03,
    7.196931E+03, 7.237444E+03, 7.278079E+03, 7.318835E+03, 7.359714E+03, 7.400715E+03,
    7.441839E+03, 7.483084E+03, 7.524452E+03, 7.565942E+03, 7.607555E+03, 7.649290E+03,
    7.691147E+03, 7.733127E+03, 7.775230E+03, 7.817455E+03, 7.859804E+03, 7.902274E+03,
    7.944868E+03, 7.987585E+03, 8.030424E+03, 8.073386E+03, 8.116471E+03, 8.159680E+03,
    8.203011E+03, 8.246466E+03, 8.290043E+03, 8.333744E+03, 8.377568E+03, 8.421516E+03,
    8.465586E+03, 8.509781E+03, 8.554098E+03, 8.598539E+03, 8.643104E+03, 8.687792E+03,
    8.732603E+03, 8.777538E+03, 8.822597E+03, 8.867780E+03, 8.913086E+03, 8.958516E+03,
    9.004070E+03, 9.049748E+03, 9.095549E+03, 9.141475E+03, 9.187524E+03, 9.233697E+03,
    9.279995E+03, 9.326416E+03, 9.372962E+03, 9.419631E+03, 9.466425E+03, 9.513343E+03,
    9.560385E+03, 9.607552E+03, 9.654843E+03, 9.702258E+03, 9.749797E+03, 9.797461E+03,
    9.845249E+03, 9.893161E+03, 9.941198E+03, 9.989360E+03, 1.003765E+04, 1.008606E+04,
    1.013459E+04, 1.018325E+04, 1.023204E+04, 1.028094E+04, 1.032998E+04, 1.037914E+04,
    1.042842E+04, 1.047783E+04, 1.052736E+04, 1.057702E+04, 1.062680E+04, 1.067670E+04,
    1.072674E+04, 1.077689E+04, 1.082717E+04, 1.087758E+04, 1.092811E+04, 1.097877E+04,
    1.102955E+04, 1.108045E+04, 1.113149E+04, 1.118264E+04, 1.123392E+04, 1.128533E+04,
    1.133686E+04, 1.138851E+04, 1.144030E+04, 1.149220E+04, 1.154423E+04, 1.159639E+04,
    1.164867E+04, 1.170108E+04, 1.175361E+04, 1.180627E+04, 1.185905E+04, 1.191196E+04,
    1.196499E+04, 1.201815E+04, 1.207143E+04, 1.212484E+04, 1.217837E+04, 1.223203E+04,
    1.228581E+04, 1.233972E+04, 1.239375E+04, 1.244791E+04, 1.250220E+04, 1.255661E+04,
    1.261114E+04, 1.266580E+04, 1.272059E+04, 1.277550E+04, 1.283053E+04, 1.288570E+04,
    1.294098E+04,
])

#  ---------------------- M = 5, I = 4 ---------------------------
M = 5
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.081920E+00, 4.656398E+01, 9.104865E+01, 1.355563E+02, 1.800726E+02, 2.245949E+02,
    2.691223E+02, 3.136542E+02, 3.581906E+02, 4.027313E+02, 4.472763E+02, 4.918259E+02,
    5.363805E+02, 5.809412E+02, 6.255100E+02, 6.700904E+02, 7.146872E+02, 7.593070E+02,
    8.039587E+02, 8.486526E+02, 8.934013E+02, 9.382189E+02, 9.831211E+02, 1.028125E+03,
    1.073248E+03, 1.118509E+03, 1.163927E+03, 1.209521E+03, 1.255311E+03, 1.301316E+03,
    1.347555E+03, 1.394046E+03, 1.440807E+03, 1.487857E+03, 1.535211E+03, 1.582885E+03,
    1.630896E+03, 1.679258E+03, 1.727985E+03, 1.777089E+03, 1.826585E+03, 1.876483E+03,
    1.926795E+03, 1.977532E+03, 2.028704E+03, 2.080320E+03, 2.132389E+03, 2.184919E+03,
    2.237919E+03, 2.291396E+03, 2.345357E+03, 2.399809E+03, 2.454758E+03, 2.510210E+03,
    2.566171E+03, 2.622645E+03, 2.679638E+03, 2.737155E+03, 2.795199E+03, 2.853775E+03,
    2.912888E+03, 2.972540E+03, 3.032735E+03, 3.093477E+03, 3.154769E+03, 3.216614E+03,
    3.279014E+03, 3.341973E+03, 3.405493E+03, 3.469576E+03, 3.534226E+03, 3.599443E+03,
    3.665230E+03, 3.731590E+03, 3.798524E+03, 3.866034E+03, 3.934122E+03, 4.002789E+03,
    4.072038E+03, 4.141869E+03, 4.212285E+03, 4.283286E+03, 4.354874E+03, 4.427051E+03,
    4.499818E+03, 4.573175E+03, 4.647125E+03, 4.721668E+03, 4.796805E+03, 4.872538E+03,
    4.948867E+03, 5.025794E+03, 5.103320E+03, 5.181445E+03, 5.260171E+03, 5.339498E+03,
    5.419427E+03, 5.499959E+03, 5.581096E+03, 5.662837E+03, 5.745183E+03, 5.828137E+03,
    5.911697E+03, 5.995865E+03, 6.080641E+03, 6.166027E+03, 6.252023E+03, 6.338630E+03,
    6.425848E+03, 6.513677E+03, 6.602120E+03, 6.691176E+03, 6.780846E+03, 6.871130E+03,
    6.962030E+03, 7.053545E+03, 7.145676E+03, 7.238425E+03, 7.331791E+03, 7.425775E+03,
    7.520378E+03, 7.615600E+03, 7.711441E+03, 7.807903E+03, 7.904986E+03, 8.002690E+03,
    8.101016E+03, 8.199964E+03, 8.299536E+03, 8.399730E+03, 8.500549E+03, 8.601992E+03,
    8.704060E+03, 8.806753E+03, 8.910073E+03, 9.014018E+03, 9.118591E+03, 9.223791E+03,
    9.329619E+03, 9.436075E+03, 9.543160E+03, 9.650874E+03, 9.759219E+03, 9.868193E+03,
    9.977798E+03, 1.008803E+04, 1.019890E+04, 1.031040E+04, 1.042253E+04, 1.053530E+04,
    1.064870E+04, 1.076273E+04, 1.087740E+04, 1.099270E+04, 1.110864E+04, 1.122521E+04,
    1.134242E+04, 1.146026E+04, 1.157875E+04, 1.169787E+04, 1.181763E+04, 1.193802E+04,
    1.205906E+04, 1.218073E+04, 1.230305E+04, 1.242600E+04, 1.254960E+04, 1.267383E+04,
    1.279871E+04, 1.292423E+04, 1.305039E+04, 1.317719E+04, 1.330464E+04, 1.343273E+04,
    1.356147E+04, 1.369085E+04, 1.382087E+04, 1.395154E+04, 1.408286E+04, 1.421482E+04,
    1.434743E+04, 1.448068E+04, 1.461459E+04, 1.474914E+04, 1.488434E+04, 1.502019E+04,
    1.515669E+04, 1.529383E+04, 1.543163E+04, 1.557008E+04, 1.570918E+04, 1.584893E+04,
    1.598933E+04, 1.613039E+04, 1.627209E+04, 1.641445E+04, 1.655747E+04, 1.670114E+04,
    1.684546E+04, 1.699044E+04, 1.713607E+04, 1.728236E+04, 1.742930E+04, 1.757690E+04,
    1.772516E+04, 1.787408E+04, 1.802365E+04, 1.817388E+04, 1.832477E+04, 1.847632E+04,
    1.862853E+04, 1.878140E+04, 1.893493E+04, 1.908912E+04, 1.924398E+04, 1.939949E+04,
    1.955567E+04, 1.971250E+04, 1.987001E+04, 2.002817E+04, 2.018700E+04, 2.034649E+04,
    2.050665E+04, 2.066748E+04, 2.082897E+04, 2.099112E+04, 2.115394E+04, 2.131743E+04,
    2.148159E+04, 2.164641E+04, 2.181190E+04, 2.197807E+04, 2.214490E+04, 2.231239E+04,
    2.248056E+04, 2.264940E+04, 2.281891E+04, 2.298910E+04, 2.315995E+04, 2.333147E+04,
    2.350367E+04, 2.367654E+04, 2.385009E+04, 2.402431E+04, 2.419920E+04, 2.437476E+04,
    2.455101E+04, 2.472792E+04, 2.490552E+04, 2.508379E+04, 2.526273E+04, 2.544236E+04,
    2.562266E+04, 2.580364E+04, 2.598529E+04, 2.616763E+04, 2.635065E+04, 2.653434E+04,
    2.671872E+04, 2.690377E+04, 2.708951E+04, 2.727593E+04, 2.746303E+04, 2.765081E+04,
    2.783928E+04, 2.802843E+04, 2.821826E+04, 2.840877E+04, 2.859997E+04, 2.879186E+04,
    2.898443E+04, 2.917768E+04, 2.937162E+04, 2.956625E+04, 2.976157E+04, 2.995757E+04,
    3.015426E+04, 3.035163E+04, 3.054970E+04, 3.074845E+04, 3.094790E+04, 3.114803E+04,
    3.134885E+04, 3.155037E+04, 3.175257E+04, 3.195547E+04, 3.215905E+04, 3.236333E+04,
    3.256830E+04, 3.277397E+04, 3.298033E+04, 3.318738E+04, 3.339512E+04, 3.360356E+04,
    3.381270E+04, 3.402252E+04, 3.423305E+04, 3.444427E+04, 3.465619E+04, 3.486880E+04,
    3.508211E+04, 3.529612E+04, 3.551082E+04, 3.572623E+04, 3.594233E+04, 3.615913E+04,
    3.637663E+04, 3.659483E+04, 3.681373E+04, 3.703333E+04, 3.725363E+04, 3.747464E+04,
    3.769634E+04, 3.791875E+04, 3.814186E+04, 3.836567E+04, 3.859018E+04, 3.881540E+04,
    3.904132E+04, 3.926794E+04, 3.949527E+04, 3.972330E+04, 3.995204E+04, 4.018149E+04,
    4.041164E+04, 4.064249E+04, 4.087405E+04, 4.110632E+04, 4.133930E+04, 4.157298E+04,
    4.180737E+04, 4.204247E+04, 4.227828E+04, 4.251480E+04, 4.275202E+04, 4.298996E+04,
    4.322860E+04, 4.346796E+04, 4.370802E+04, 4.394880E+04, 4.419028E+04, 4.443248E+04,
    4.467539E+04, 4.491901E+04, 4.516335E+04, 4.540839E+04, 4.565415E+04, 4.590062E+04,
    4.614781E+04, 4.639571E+04, 4.664432E+04, 4.689365E+04, 4.714369E+04, 4.739445E+04,
    4.764592E+04, 4.789811E+04, 4.815101E+04, 4.840463E+04, 4.865897E+04, 4.891402E+04,
    4.916979E+04, 4.942627E+04, 4.968348E+04, 4.994140E+04, 5.020004E+04, 5.045939E+04,
    5.071947E+04, 5.098026E+04, 5.124178E+04, 5.150401E+04, 5.176696E+04, 5.203063E+04,
    5.229502E+04, 5.256014E+04, 5.282597E+04, 5.309252E+04, 5.335979E+04, 5.362779E+04,
    5.389650E+04, 5.416594E+04, 5.443610E+04, 5.470698E+04, 5.497858E+04, 5.525091E+04,
    5.552396E+04, 5.579773E+04, 5.607222E+04, 5.634744E+04, 5.662338E+04, 5.690004E+04,
    5.717743E+04, 5.745555E+04, 5.773438E+04, 5.801394E+04, 5.829423E+04, 5.857524E+04,
    5.885698E+04, 5.913944E+04, 5.942262E+04, 5.970654E+04, 5.999117E+04, 6.027654E+04,
    6.056263E+04, 6.084944E+04, 6.113698E+04, 6.142525E+04, 6.171425E+04, 6.200397E+04,
    6.229442E+04, 6.258560E+04, 6.287750E+04, 6.317013E+04, 6.346349E+04, 6.375758E+04,
    6.405239E+04, 6.434794E+04, 6.464421E+04, 6.494121E+04, 6.523894E+04, 6.553739E+04,
    6.583658E+04, 6.613649E+04, 6.643713E+04, 6.673850E+04, 6.704060E+04, 6.734343E+04,
    6.764699E+04, 6.795128E+04, 6.825630E+04, 6.856205E+04, 6.886853E+04, 6.917573E+04,
    6.948367E+04, 6.979234E+04, 7.010173E+04, 7.041186E+04, 7.072272E+04, 7.103431E+04,
    7.134663E+04, 7.165967E+04, 7.197345E+04, 7.228796E+04, 7.260320E+04, 7.291917E+04,
    7.323588E+04, 7.355331E+04, 7.387147E+04, 7.419036E+04, 7.450999E+04, 7.483035E+04,
    7.515143E+04,
])

#  ---------------------- M = 5, I = 5 ---------------------------
M = 5
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.039420E+00, 1.660365E+01, 3.251624E+01, 4.843614E+01, 6.435894E+01, 8.028378E+01,
    9.621037E+01, 1.121386E+02, 1.280684E+02, 1.439997E+02, 1.599326E+02, 1.758672E+02,
    1.918036E+02, 2.077426E+02, 2.236851E+02, 2.396326E+02, 2.555873E+02, 2.715523E+02,
    2.875312E+02, 3.035287E+02, 3.195498E+02, 3.356005E+02, 3.516871E+02, 3.678163E+02,
    3.839953E+02, 4.002313E+02, 4.165317E+02, 4.329040E+02, 4.493555E+02, 4.658935E+02,
    4.825251E+02, 4.992572E+02, 5.160964E+02, 5.330491E+02, 5.501215E+02, 5.673195E+02,
    5.846486E+02, 6.021141E+02, 6.197210E+02, 6.374741E+02, 6.553778E+02, 6.734364E+02,
    6.916539E+02, 7.100339E+02, 7.285802E+02, 7.472958E+02, 7.661840E+02, 7.852478E+02,
    8.044898E+02, 8.239127E+02, 8.435189E+02, 8.633107E+02, 8.832903E+02, 9.034597E+02,
    9.238208E+02, 9.443754E+02, 9.651252E+02, 9.860718E+02, 1.007217E+03, 1.028562E+03,
    1.050107E+03, 1.071856E+03, 1.093807E+03, 1.115964E+03, 1.138326E+03, 1.160895E+03,
    1.183672E+03, 1.206658E+03, 1.229853E+03, 1.253259E+03, 1.276876E+03, 1.300705E+03,
    1.324747E+03, 1.349002E+03, 1.373471E+03, 1.398155E+03, 1.423054E+03, 1.448169E+03,
    1.473501E+03, 1.499050E+03, 1.524816E+03, 1.550800E+03, 1.577002E+03, 1.603424E+03,
    1.630065E+03, 1.656925E+03, 1.684006E+03, 1.711308E+03, 1.738830E+03, 1.766574E+03,
    1.794539E+03, 1.822727E+03, 1.851137E+03, 1.879770E+03, 1.908625E+03, 1.937705E+03,
    1.967007E+03, 1.996534E+03, 2.026285E+03, 2.056261E+03, 2.086461E+03, 2.116887E+03,
    2.147537E+03, 2.178414E+03, 2.209516E+03, 2.240845E+03, 2.272400E+03, 2.304181E+03,
    2.336189E+03, 2.368425E+03, 2.400887E+03, 2.433577E+03, 2.466495E+03, 2.499641E+03,
    2.533015E+03, 2.566618E+03, 2.600449E+03, 2.634509E+03, 2.668798E+03, 2.703316E+03,
    2.738063E+03, 2.773040E+03, 2.808247E+03, 2.843684E+03, 2.879351E+03, 2.915249E+03,
    2.951377E+03, 2.987736E+03, 3.024325E+03, 3.061146E+03, 3.098198E+03, 3.135482E+03,
    3.172998E+03, 3.210745E+03, 3.248724E+03, 3.286935E+03, 3.325379E+03, 3.364056E+03,
    3.402965E+03, 3.442107E+03, 3.481482E+03, 3.521090E+03, 3.560932E+03, 3.601007E+03,
    3.641316E+03, 3.681859E+03, 3.722636E+03, 3.763647E+03, 3.804893E+03, 3.846373E+03,
    3.888088E+03, 3.930038E+03, 3.972223E+03, 4.014644E+03, 4.057299E+03, 4.100190E+03,
    4.143317E+03, 4.186680E+03, 4.230279E+03, 4.274114E+03, 4.318186E+03, 4.362494E+03,
    4.407039E+03, 4.451821E+03, 4.496839E+03, 4.542095E+03, 4.587588E+03, 4.633319E+03,
    4.679287E+03, 4.725494E+03, 4.771938E+03, 4.818620E+03, 4.865541E+03, 4.912700E+03,
    4.960098E+03, 5.007734E+03, 5.055610E+03, 5.103724E+03, 5.152078E+03, 5.200672E+03,
    5.249504E+03, 5.298577E+03, 5.347889E+03, 5.397442E+03, 5.447235E+03, 5.497268E+03,
    5.547541E+03, 5.598056E+03, 5.648811E+03, 5.699807E+03, 5.751044E+03, 5.802523E+03,
    5.854243E+03, 5.906205E+03, 5.958408E+03, 6.010853E+03, 6.063541E+03, 6.116471E+03,
    6.169643E+03, 6.223058E+03, 6.276715E+03, 6.330616E+03, 6.384759E+03, 6.439146E+03,
    6.493776E+03, 6.548649E+03, 6.603766E+03, 6.659127E+03, 6.714732E+03, 6.770581E+03,
    6.826675E+03, 6.883013E+03, 6.939595E+03, 6.996422E+03, 7.053495E+03, 7.110812E+03,
    7.168374E+03, 7.226182E+03, 7.284235E+03, 7.342534E+03, 7.401079E+03, 7.459870E+03,
    7.518907E+03, 7.578191E+03, 7.637721E+03, 7.697497E+03, 7.757520E+03, 7.817791E+03,
    7.878308E+03, 7.939072E+03, 8.000084E+03, 8.061344E+03, 8.122851E+03, 8.184606E+03,
    8.246609E+03, 8.308860E+03, 8.371360E+03, 8.434108E+03, 8.497104E+03, 8.560349E+03,
    8.623844E+03, 8.687587E+03, 8.751580E+03, 8.815821E+03, 8.880313E+03, 8.945054E+03,
    9.010045E+03, 9.075286E+03, 9.140777E+03, 9.206518E+03, 9.272509E+03, 9.338752E+03,
    9.405244E+03, 9.471988E+03, 9.538983E+03, 9.606229E+03, 9.673726E+03, 9.741475E+03,
    9.809475E+03, 9.877727E+03, 9.946231E+03, 1.001499E+04, 1.008399E+04, 1.015325E+04,
    1.022277E+04, 1.029253E+04, 1.036255E+04, 1.043282E+04, 1.050335E+04, 1.057412E+04,
    1.064515E+04, 1.071644E+04, 1.078797E+04, 1.085976E+04, 1.093181E+04, 1.100411E+04,
    1.107666E+04, 1.114947E+04, 1.122253E+04, 1.129585E+04, 1.136942E+04, 1.144324E+04,
    1.151732E+04, 1.159166E+04, 1.166625E+04, 1.174110E+04, 1.181620E+04, 1.189155E+04,
    1.196717E+04, 1.204304E+04, 1.211916E+04, 1.219554E+04, 1.227218E+04, 1.234907E+04,
    1.242622E+04, 1.250363E+04, 1.258129E+04, 1.265921E+04, 1.273739E+04, 1.281582E+04,
    1.289451E+04, 1.297346E+04, 1.305267E+04, 1.313213E+04, 1.321185E+04, 1.329183E+04,
    1.337207E+04, 1.345256E+04, 1.353332E+04, 1.361433E+04, 1.369560E+04, 1.377713E+04,
    1.385891E+04, 1.394096E+04, 1.402326E+04, 1.410583E+04, 1.418865E+04, 1.427173E+04,
    1.435507E+04, 1.443867E+04, 1.452253E+04, 1.460665E+04, 1.469103E+04, 1.477567E+04,
    1.486057E+04, 1.494572E+04, 1.503114E+04, 1.511682E+04, 1.520276E+04, 1.528896E+04,
    1.537542E+04, 1.546214E+04, 1.554912E+04, 1.563636E+04, 1.572387E+04, 1.581163E+04,
    1.589966E+04, 1.598794E+04, 1.607649E+04, 1.616530E+04, 1.625437E+04, 1.634370E+04,
    1.643329E+04, 1.652315E+04, 1.661326E+04, 1.670364E+04, 1.679428E+04, 1.688518E+04,
    1.697635E+04, 1.706777E+04, 1.715946E+04, 1.725141E+04, 1.734363E+04, 1.743610E+04,
    1.752884E+04, 1.762184E+04, 1.771510E+04, 1.780863E+04, 1.790242E+04, 1.799647E+04,
    1.809078E+04, 1.818536E+04, 1.828020E+04, 1.837531E+04, 1.847067E+04, 1.856630E+04,
    1.866220E+04, 1.875836E+04, 1.885478E+04, 1.895146E+04, 1.904841E+04, 1.914562E+04,
    1.924309E+04, 1.934083E+04, 1.943883E+04, 1.953710E+04, 1.963563E+04, 1.973443E+04,
    1.983348E+04, 1.993281E+04, 2.003239E+04, 2.013224E+04, 2.023236E+04, 2.033273E+04,
    2.043338E+04, 2.053428E+04, 2.063546E+04, 2.073689E+04, 2.083859E+04, 2.094056E+04,
    2.104279E+04, 2.114528E+04, 2.124804E+04, 2.135106E+04, 2.145435E+04, 2.155790E+04,
    2.166172E+04, 2.176580E+04, 2.187014E+04, 2.197476E+04, 2.207963E+04, 2.218477E+04,
    2.229018E+04, 2.239585E+04, 2.250178E+04, 2.260798E+04, 2.271445E+04, 2.282118E+04,
    2.292817E+04, 2.303543E+04, 2.314296E+04, 2.325075E+04, 2.335880E+04, 2.346712E+04,
    2.357571E+04, 2.368455E+04, 2.379367E+04, 2.390305E+04, 2.401269E+04, 2.412260E+04,
    2.423278E+04, 2.434322E+04, 2.445392E+04, 2.456489E+04, 2.467613E+04, 2.478763E+04,
    2.489939E+04, 2.501142E+04, 2.512372E+04, 2.523628E+04, 2.534910E+04, 2.546220E+04,
    2.557555E+04, 2.568917E+04, 2.580306E+04, 2.591721E+04, 2.603162E+04, 2.614630E+04,
    2.626125E+04, 2.637646E+04, 2.649193E+04, 2.660767E+04, 2.672367E+04, 2.683994E+04,
    2.695648E+04, 2.707328E+04, 2.719034E+04, 2.730767E+04, 2.742526E+04, 2.754312E+04,
    2.766124E+04,
])

#  ---------------------- M = 5, I = 6 ---------------------------
M = 5
I = 6
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.220898E+01, 9.732970E+01, 1.905089E+02, 2.837327E+02, 3.769738E+02, 4.702271E+02,
    5.634907E+02, 6.567639E+02, 7.500463E+02, 8.433378E+02, 9.366383E+02, 1.029949E+03,
    1.123270E+03, 1.216605E+03, 1.309959E+03, 1.403341E+03, 1.496762E+03, 1.590239E+03,
    1.683792E+03, 1.777446E+03, 1.871230E+03, 1.965177E+03, 2.059322E+03, 2.153704E+03,
    2.248362E+03, 2.343338E+03, 2.438674E+03, 2.534412E+03, 2.630596E+03, 2.727266E+03,
    2.824463E+03, 2.922228E+03, 3.020599E+03, 3.119613E+03, 3.219306E+03, 3.319712E+03,
    3.420863E+03, 3.522791E+03, 3.625525E+03, 3.729092E+03, 3.833519E+03, 3.938831E+03,
    4.045052E+03, 4.152203E+03, 4.260305E+03, 4.369377E+03, 4.479439E+03, 4.590508E+03,
    4.702599E+03, 4.815729E+03, 4.929912E+03, 5.045161E+03, 5.161489E+03, 5.278909E+03,
    5.397431E+03, 5.517067E+03, 5.637826E+03, 5.759718E+03, 5.882752E+03, 6.006937E+03,
    6.132280E+03, 6.258790E+03, 6.386472E+03, 6.515335E+03, 6.645385E+03, 6.776627E+03,
    6.909068E+03, 7.042712E+03, 7.177566E+03, 7.313634E+03, 7.450922E+03, 7.589432E+03,
    7.729171E+03, 7.870141E+03, 8.012348E+03, 8.155794E+03, 8.300483E+03, 8.446420E+03,
    8.593606E+03, 8.742045E+03, 8.891741E+03, 9.042696E+03, 9.194913E+03, 9.348394E+03,
    9.503144E+03, 9.659162E+03, 9.816454E+03, 9.975020E+03, 1.013486E+04, 1.029599E+04,
    1.045839E+04, 1.062208E+04, 1.078705E+04, 1.095331E+04, 1.112086E+04, 1.128970E+04,
    1.145983E+04, 1.163126E+04, 1.180398E+04, 1.197801E+04, 1.215333E+04, 1.232995E+04,
    1.250788E+04, 1.268711E+04, 1.286765E+04, 1.304949E+04, 1.323264E+04, 1.341711E+04,
    1.360288E+04, 1.378997E+04, 1.397837E+04, 1.416809E+04, 1.435913E+04, 1.455148E+04,
    1.474515E+04, 1.494015E+04, 1.513646E+04, 1.533410E+04, 1.553307E+04, 1.573336E+04,
    1.593497E+04, 1.613792E+04, 1.634219E+04, 1.654779E+04, 1.675473E+04, 1.696300E+04,
    1.717260E+04, 1.738353E+04, 1.759580E+04, 1.780941E+04, 1.802436E+04, 1.824064E+04,
    1.845827E+04, 1.867723E+04, 1.889754E+04, 1.911919E+04, 1.934219E+04, 1.956653E+04,
    1.979221E+04, 2.001925E+04, 2.024763E+04, 2.047736E+04, 2.070844E+04, 2.094087E+04,
    2.117466E+04, 2.140979E+04, 2.164629E+04, 2.188413E+04, 2.212333E+04, 2.236389E+04,
    2.260581E+04, 2.284909E+04, 2.309373E+04, 2.333972E+04, 2.358708E+04, 2.383580E+04,
    2.408589E+04, 2.433734E+04, 2.459015E+04, 2.484434E+04, 2.509989E+04, 2.535680E+04,
    2.561509E+04, 2.587475E+04, 2.613578E+04, 2.639818E+04, 2.666195E+04, 2.692710E+04,
    2.719362E+04, 2.746152E+04, 2.773080E+04, 2.800145E+04, 2.827348E+04, 2.854690E+04,
    2.882169E+04, 2.909786E+04, 2.937542E+04, 2.965436E+04, 2.993468E+04, 3.021639E+04,
    3.049948E+04, 3.078396E+04, 3.106983E+04, 3.135709E+04, 3.164574E+04, 3.193578E+04,
    3.222721E+04, 3.252003E+04, 3.281424E+04, 3.310985E+04, 3.340686E+04, 3.370526E+04,
    3.400506E+04, 3.430625E+04, 3.460885E+04, 3.491284E+04, 3.521824E+04, 3.552504E+04,
    3.583324E+04, 3.614284E+04, 3.645385E+04, 3.676626E+04, 3.708008E+04, 3.739530E+04,
    3.771194E+04, 3.802998E+04, 3.834943E+04, 3.867030E+04, 3.899257E+04, 3.931626E+04,
    3.964136E+04, 3.996788E+04, 4.029581E+04, 4.062516E+04, 4.095592E+04, 4.128810E+04,
    4.162170E+04, 4.195673E+04, 4.229317E+04, 4.263103E+04, 4.297032E+04, 4.331103E+04,
    4.365316E+04, 4.399672E+04, 4.434171E+04, 4.468812E+04, 4.503596E+04, 4.538523E+04,
    4.573593E+04, 4.608806E+04, 4.644162E+04, 4.679662E+04, 4.715304E+04, 4.751091E+04,
    4.787020E+04, 4.823094E+04, 4.859311E+04, 4.895671E+04, 4.932176E+04, 4.968825E+04,
    5.005618E+04, 5.042554E+04, 5.079636E+04, 5.116861E+04, 5.154231E+04, 5.191745E+04,
    5.229404E+04, 5.267208E+04, 5.305156E+04, 5.343249E+04, 5.381488E+04, 5.419871E+04,
    5.458399E+04, 5.497073E+04, 5.535892E+04, 5.574856E+04, 5.613965E+04, 5.653221E+04,
    5.692621E+04, 5.732168E+04, 5.771860E+04, 5.811699E+04, 5.851683E+04, 5.891813E+04,
    5.932090E+04, 5.972512E+04, 6.013081E+04, 6.053797E+04, 6.094659E+04, 6.135667E+04,
    6.176822E+04, 6.218124E+04, 6.259573E+04, 6.301168E+04, 6.342911E+04, 6.384801E+04,
    6.426837E+04, 6.469022E+04, 6.511353E+04, 6.553832E+04, 6.596458E+04, 6.639232E+04,
    6.682153E+04, 6.725222E+04, 6.768439E+04, 6.811804E+04, 6.855317E+04, 6.898978E+04,
    6.942787E+04, 6.986744E+04, 7.030849E+04, 7.075103E+04, 7.119505E+04, 7.164056E+04,
    7.208755E+04, 7.253603E+04, 7.298599E+04, 7.343745E+04, 7.389039E+04, 7.434483E+04,
    7.480075E+04, 7.525816E+04, 7.571707E+04, 7.617747E+04, 7.663936E+04, 7.710275E+04,
    7.756763E+04, 7.803400E+04, 7.850187E+04, 7.897124E+04, 7.944211E+04, 7.991447E+04,
    8.038834E+04, 8.086370E+04, 8.134057E+04, 8.181893E+04, 8.229880E+04, 8.278016E+04,
    8.326304E+04, 8.374741E+04, 8.423329E+04, 8.472068E+04, 8.520957E+04, 8.569996E+04,
    8.619187E+04, 8.668528E+04, 8.718020E+04, 8.767663E+04, 8.817457E+04, 8.867401E+04,
    8.917497E+04, 8.967744E+04, 9.018143E+04, 9.068692E+04, 9.119393E+04, 9.170245E+04,
    9.221249E+04, 9.272404E+04, 9.323710E+04, 9.375169E+04, 9.426779E+04, 9.478540E+04,
    9.530454E+04, 9.582519E+04, 9.634736E+04, 9.687105E+04, 9.739626E+04, 9.792299E+04,
    9.845125E+04, 9.898102E+04, 9.951232E+04, 1.000451E+05, 1.005795E+05, 1.011153E+05,
    1.016527E+05, 1.021916E+05, 1.027321E+05, 1.032740E+05, 1.038175E+05, 1.043626E+05,
    1.049091E+05, 1.054572E+05, 1.060068E+05, 1.065579E+05, 1.071105E+05, 1.076647E+05,
    1.082204E+05, 1.087777E+05, 1.093365E+05, 1.098968E+05, 1.104586E+05, 1.110220E+05,
    1.115869E+05, 1.121533E+05, 1.127213E+05, 1.132908E+05, 1.138618E+05, 1.144344E+05,
    1.150085E+05, 1.155841E+05, 1.161613E+05, 1.167400E+05, 1.173202E+05, 1.179020E+05,
    1.184853E+05, 1.190701E+05, 1.196565E+05, 1.202444E+05, 1.208339E+05, 1.214249E+05,
    1.220174E+05, 1.226115E+05, 1.232071E+05, 1.238043E+05, 1.244030E+05, 1.250032E+05,
    1.256050E+05, 1.262083E+05, 1.268131E+05, 1.274195E+05, 1.280275E+05, 1.286369E+05,
    1.292479E+05, 1.298605E+05, 1.304746E+05, 1.310902E+05, 1.317074E+05, 1.323261E+05,
    1.329464E+05, 1.335682E+05, 1.341915E+05, 1.348164E+05, 1.354429E+05, 1.360708E+05,
    1.367004E+05, 1.373314E+05, 1.379640E+05, 1.385982E+05, 1.392339E+05, 1.398711E+05,
    1.405099E+05, 1.411502E+05, 1.417921E+05, 1.424355E+05, 1.430804E+05, 1.437269E+05,
    1.443750E+05, 1.450246E+05, 1.456757E+05, 1.463284E+05, 1.469826E+05, 1.476384E+05,
    1.482957E+05, 1.489545E+05, 1.496149E+05, 1.502769E+05, 1.509404E+05, 1.516054E+05,
    1.522720E+05, 1.529401E+05, 1.536098E+05, 1.542810E+05, 1.549538E+05, 1.556281E+05,
    1.563039E+05, 1.569813E+05, 1.576602E+05, 1.583407E+05, 1.590227E+05, 1.597063E+05,
    1.603914E+05,
])

#  ---------------------- M = 5, I = 7 ---------------------------
M = 5
I = 7
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.018640E+00, 8.214730E+00, 1.608377E+01, 2.395650E+01, 3.183067E+01, 3.970586E+01,
    4.758192E+01, 5.545878E+01, 6.333642E+01, 7.121483E+01, 7.909401E+01, 8.697401E+01,
    9.485494E+01, 1.027371E+02, 1.106209E+02, 1.185071E+02, 1.263968E+02, 1.342914E+02,
    1.421927E+02, 1.501028E+02, 1.580244E+02, 1.659601E+02, 1.739131E+02, 1.818867E+02,
    1.898844E+02, 1.979096E+02, 2.059660E+02, 2.140572E+02, 2.221869E+02, 2.303586E+02,
    2.385758E+02, 2.468418E+02, 2.551601E+02, 2.635336E+02, 2.719655E+02, 2.804586E+02,
    2.890157E+02, 2.976394E+02, 3.063322E+02, 3.150964E+02, 3.239342E+02, 3.328478E+02,
    3.418392E+02, 3.509100E+02, 3.600623E+02, 3.692975E+02, 3.786172E+02, 3.880229E+02,
    3.975159E+02, 4.070976E+02, 4.167692E+02, 4.265318E+02, 4.363864E+02, 4.463342E+02,
    4.563759E+02, 4.665126E+02, 4.767451E+02, 4.870742E+02, 4.975006E+02, 5.080251E+02,
    5.186483E+02, 5.293708E+02, 5.401933E+02, 5.511163E+02, 5.621404E+02, 5.732661E+02,
    5.844938E+02, 5.958240E+02, 6.072572E+02, 6.187938E+02, 6.304342E+02, 6.421787E+02,
    6.540277E+02, 6.659816E+02, 6.780407E+02, 6.902053E+02, 7.024757E+02, 7.148522E+02,
    7.273350E+02, 7.399245E+02, 7.526209E+02, 7.654244E+02, 7.783353E+02, 7.913538E+02,
    8.044801E+02, 8.177145E+02, 8.310571E+02, 8.445081E+02, 8.580677E+02, 8.717362E+02,
    8.855137E+02, 8.994003E+02, 9.133963E+02, 9.275017E+02, 9.417169E+02, 9.560418E+02,
    9.704767E+02, 9.850217E+02, 9.996770E+02, 1.014443E+03, 1.029319E+03, 1.044306E+03,
    1.059403E+03, 1.074612E+03, 1.089932E+03, 1.105363E+03, 1.120905E+03, 1.136558E+03,
    1.152323E+03, 1.168200E+03, 1.184189E+03, 1.200289E+03, 1.216502E+03, 1.232826E+03,
    1.249263E+03, 1.265812E+03, 1.282473E+03, 1.299247E+03, 1.316134E+03, 1.333133E+03,
    1.350245E+03, 1.367470E+03, 1.384808E+03, 1.402259E+03, 1.419823E+03, 1.437501E+03,
    1.455292E+03, 1.473196E+03, 1.491214E+03, 1.509346E+03, 1.527592E+03, 1.545951E+03,
    1.564424E+03, 1.583011E+03, 1.601713E+03, 1.620528E+03, 1.639458E+03, 1.658502E+03,
    1.677661E+03, 1.696934E+03, 1.716321E+03, 1.735824E+03, 1.755441E+03, 1.775173E+03,
    1.795020E+03, 1.814982E+03, 1.835060E+03, 1.855252E+03, 1.875560E+03, 1.895983E+03,
    1.916521E+03, 1.937175E+03, 1.957945E+03, 1.978830E+03, 1.999832E+03, 2.020949E+03,
    2.042182E+03, 2.063530E+03, 2.084995E+03, 2.106577E+03, 2.128274E+03, 2.150088E+03,
    2.172018E+03, 2.194065E+03, 2.216228E+03, 2.238508E+03, 2.260905E+03, 2.283418E+03,
    2.306048E+03, 2.328796E+03, 2.351660E+03, 2.374642E+03, 2.397740E+03, 2.420956E+03,
    2.444290E+03, 2.467740E+03, 2.491309E+03, 2.514994E+03, 2.538798E+03, 2.562719E+03,
    2.586758E+03, 2.610915E+03, 2.635190E+03, 2.659584E+03, 2.684095E+03, 2.708724E+03,
    2.733472E+03, 2.758338E+03, 2.783322E+03, 2.808426E+03, 2.833647E+03, 2.858988E+03,
    2.884447E+03, 2.910025E+03, 2.935722E+03, 2.961537E+03, 2.987472E+03, 3.013526E+03,
    3.039700E+03, 3.065992E+03, 3.092404E+03, 3.118936E+03, 3.145587E+03, 3.172357E+03,
    3.199248E+03, 3.226258E+03, 3.253388E+03, 3.280638E+03, 3.308007E+03, 3.335497E+03,
    3.363107E+03, 3.390838E+03, 3.418688E+03, 3.446659E+03, 3.474751E+03, 3.502963E+03,
    3.531296E+03, 3.559749E+03, 3.588323E+03, 3.617018E+03, 3.645834E+03, 3.674771E+03,
    3.703829E+03, 3.733008E+03, 3.762308E+03, 3.791730E+03, 3.821273E+03, 3.850937E+03,
    3.880723E+03, 3.910631E+03, 3.940660E+03, 3.970811E+03, 4.001084E+03, 4.031479E+03,
    4.061996E+03, 4.092635E+03, 4.123396E+03, 4.154279E+03, 4.185284E+03, 4.216412E+03,
    4.247663E+03, 4.279035E+03, 4.310531E+03, 4.342149E+03, 4.373890E+03, 4.405753E+03,
    4.437740E+03, 4.469850E+03, 4.502082E+03, 4.534438E+03, 4.566917E+03, 4.599519E+03,
    4.632244E+03, 4.665093E+03, 4.698065E+03, 4.731161E+03, 4.764381E+03, 4.797724E+03,
    4.831191E+03, 4.864782E+03, 4.898496E+03, 4.932335E+03, 4.966298E+03, 5.000385E+03,
    5.034596E+03, 5.068931E+03, 5.103391E+03, 5.137975E+03, 5.172683E+03, 5.207516E+03,
    5.242474E+03, 5.277557E+03, 5.312764E+03, 5.348096E+03, 5.383553E+03, 5.419135E+03,
    5.454842E+03, 5.490674E+03, 5.526631E+03, 5.562713E+03, 5.598921E+03, 5.635254E+03,
    5.671712E+03, 5.708296E+03, 5.745006E+03, 5.781841E+03, 5.818802E+03, 5.855889E+03,
    5.893102E+03, 5.930440E+03, 5.967904E+03, 6.005495E+03, 6.043211E+03, 6.081054E+03,
    6.119023E+03, 6.157118E+03, 6.195340E+03, 6.233688E+03, 6.272162E+03, 6.310763E+03,
    6.349490E+03, 6.388345E+03, 6.427325E+03, 6.466433E+03, 6.505668E+03, 6.545029E+03,
    6.584517E+03, 6.624133E+03, 6.663875E+03, 6.703745E+03, 6.743741E+03, 6.783865E+03,
    6.824117E+03, 6.864495E+03, 6.905001E+03, 6.945635E+03, 6.986396E+03, 7.027284E+03,
    7.068301E+03, 7.109445E+03, 7.150716E+03, 7.192116E+03, 7.233643E+03, 7.275298E+03,
    7.317081E+03, 7.358993E+03, 7.401032E+03, 7.443199E+03, 7.485495E+03, 7.527918E+03,
    7.570470E+03, 7.613151E+03, 7.655959E+03, 7.698896E+03, 7.741962E+03, 7.785156E+03,
    7.828478E+03, 7.871929E+03, 7.915509E+03, 7.959218E+03, 8.003055E+03, 8.047021E+03,
    8.091116E+03, 8.135339E+03, 8.179692E+03, 8.224173E+03, 8.268784E+03, 8.313523E+03,
    8.358392E+03, 8.403390E+03, 8.448517E+03, 8.493773E+03, 8.539158E+03, 8.584673E+03,
    8.630317E+03, 8.676090E+03, 8.721993E+03, 8.768025E+03, 8.814186E+03, 8.860478E+03,
    8.906898E+03, 8.953448E+03, 9.000128E+03, 9.046938E+03, 9.093877E+03, 9.140946E+03,
    9.188144E+03, 9.235473E+03, 9.282931E+03, 9.330519E+03, 9.378237E+03, 9.426085E+03,
    9.474063E+03, 9.522171E+03, 9.570408E+03, 9.618776E+03, 9.667274E+03, 9.715902E+03,
    9.764660E+03, 9.813548E+03, 9.862566E+03, 9.911715E+03, 9.960994E+03, 1.001040E+04,
    1.005994E+04, 1.010961E+04, 1.015941E+04, 1.020934E+04, 1.025940E+04, 1.030959E+04,
    1.035991E+04, 1.041037E+04, 1.046095E+04, 1.051166E+04, 1.056250E+04, 1.061348E+04,
    1.066458E+04, 1.071581E+04, 1.076718E+04, 1.081867E+04, 1.087030E+04, 1.092206E+04,
    1.097395E+04, 1.102596E+04, 1.107811E+04, 1.113039E+04, 1.118280E+04, 1.123534E+04,
    1.128801E+04, 1.134081E+04, 1.139375E+04, 1.144681E+04, 1.150000E+04, 1.155333E+04,
    1.160678E+04, 1.166037E+04, 1.171409E+04, 1.176793E+04, 1.182191E+04, 1.187602E+04,
    1.193026E+04, 1.198463E+04, 1.203913E+04, 1.209377E+04, 1.214853E+04, 1.220342E+04,
    1.225845E+04, 1.231360E+04, 1.236889E+04, 1.242431E+04, 1.247985E+04, 1.253553E+04,
    1.259134E+04, 1.264728E+04, 1.270335E+04, 1.275955E+04, 1.281589E+04, 1.287235E+04,
    1.292894E+04, 1.298567E+04, 1.304252E+04, 1.309951E+04, 1.315662E+04, 1.321387E+04,
    1.327125E+04, 1.332876E+04, 1.338640E+04, 1.344417E+04, 1.350207E+04, 1.356010E+04,
    1.361826E+04,
])

#  ---------------------- M = 5, I = 8 ---------------------------
M = 5
I = 8
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.024260E+00, 8.644900E+00, 1.694486E+01, 2.524838E+01, 3.355335E+01, 4.185935E+01,
    5.016626E+01, 5.847401E+01, 6.678257E+01, 7.509194E+01, 8.340213E+01, 9.171321E+01,
    1.000254E+02, 1.083389E+02, 1.166545E+02, 1.249731E+02, 1.332960E+02, 1.416249E+02,
    1.499621E+02, 1.583101E+02, 1.666720E+02, 1.750509E+02, 1.834505E+02, 1.918746E+02,
    2.003270E+02, 2.088118E+02, 2.173329E+02, 2.258944E+02, 2.345004E+02, 2.431546E+02,
    2.518609E+02, 2.606229E+02, 2.694442E+02, 2.783282E+02, 2.872781E+02, 2.962969E+02,
    3.053877E+02, 3.145531E+02, 3.237958E+02, 3.331182E+02, 3.425226E+02, 3.520113E+02,
    3.615863E+02, 3.712495E+02, 3.810028E+02, 3.908478E+02, 4.007862E+02, 4.108194E+02,
    4.209489E+02, 4.311760E+02, 4.415019E+02, 4.519278E+02, 4.624548E+02, 4.730840E+02,
    4.838163E+02, 4.946525E+02, 5.055937E+02, 5.166406E+02, 5.277939E+02, 5.390545E+02,
    5.504229E+02, 5.618998E+02, 5.734859E+02, 5.851816E+02, 5.969876E+02, 6.089044E+02,
    6.209324E+02, 6.330722E+02, 6.453241E+02, 6.576887E+02, 6.701662E+02, 6.827571E+02,
    6.954618E+02, 7.082806E+02, 7.212138E+02, 7.342617E+02, 7.474247E+02, 7.607031E+02,
    7.740971E+02, 7.876070E+02, 8.012330E+02, 8.149755E+02, 8.288346E+02, 8.428107E+02,
    8.569038E+02, 8.711142E+02, 8.854422E+02, 8.998879E+02, 9.144515E+02, 9.291333E+02,
    9.439334E+02, 9.588519E+02, 9.738891E+02, 9.890452E+02, 1.004320E+03, 1.019714E+03,
    1.035228E+03, 1.050861E+03, 1.066613E+03, 1.082486E+03, 1.098478E+03, 1.114590E+03,
    1.130822E+03, 1.147175E+03, 1.163648E+03, 1.180242E+03, 1.196956E+03, 1.213792E+03,
    1.230748E+03, 1.247825E+03, 1.265023E+03, 1.282343E+03, 1.299784E+03, 1.317346E+03,
    1.335031E+03, 1.352836E+03, 1.370764E+03, 1.388814E+03, 1.406986E+03, 1.425280E+03,
    1.443696E+03, 1.462234E+03, 1.480895E+03, 1.499679E+03, 1.518585E+03, 1.537614E+03,
    1.556766E+03, 1.576041E+03, 1.595439E+03, 1.614960E+03, 1.634605E+03, 1.654372E+03,
    1.674264E+03, 1.694278E+03, 1.714417E+03, 1.734679E+03, 1.755065E+03, 1.775574E+03,
    1.796208E+03, 1.816966E+03, 1.837848E+03, 1.858855E+03, 1.879985E+03, 1.901240E+03,
    1.922620E+03, 1.944124E+03, 1.965753E+03, 1.987507E+03, 2.009386E+03, 2.031390E+03,
    2.053519E+03, 2.075772E+03, 2.098152E+03, 2.120656E+03, 2.143286E+03, 2.166041E+03,
    2.188922E+03, 2.211929E+03, 2.235061E+03, 2.258320E+03, 2.281704E+03, 2.305214E+03,
    2.328850E+03, 2.352613E+03, 2.376502E+03, 2.400517E+03, 2.424658E+03, 2.448926E+03,
    2.473321E+03, 2.497842E+03, 2.522490E+03, 2.547265E+03, 2.572167E+03, 2.597196E+03,
    2.622352E+03, 2.647635E+03, 2.673046E+03, 2.698584E+03, 2.724249E+03, 2.750042E+03,
    2.775962E+03, 2.802011E+03, 2.828187E+03, 2.854490E+03, 2.880922E+03, 2.907482E+03,
    2.934170E+03, 2.960986E+03, 2.987930E+03, 3.015003E+03, 3.042204E+03, 3.069534E+03,
    3.096992E+03, 3.124579E+03, 3.152295E+03, 3.180140E+03, 3.208113E+03, 3.236216E+03,
    3.264448E+03, 3.292809E+03, 3.321299E+03, 3.349918E+03, 3.378667E+03, 3.407546E+03,
    3.436554E+03, 3.465692E+03, 3.494959E+03, 3.524357E+03, 3.553884E+03, 3.583542E+03,
    3.613329E+03, 3.643247E+03, 3.673295E+03, 3.703473E+03, 3.733781E+03, 3.764221E+03,
    3.794790E+03, 3.825491E+03, 3.856322E+03, 3.887284E+03, 3.918377E+03, 3.949600E+03,
    3.980955E+03, 4.012441E+03, 4.044059E+03, 4.075807E+03, 4.107687E+03, 4.139699E+03,
    4.171842E+03, 4.204116E+03, 4.236523E+03, 4.269061E+03, 4.301731E+03, 4.334532E+03,
    4.367466E+03, 4.400532E+03, 4.433731E+03, 4.467061E+03, 4.500524E+03, 4.534119E+03,
    4.567847E+03, 4.601707E+03, 4.635700E+03, 4.669825E+03, 4.704084E+03, 4.738475E+03,
    4.772999E+03, 4.807656E+03, 4.842447E+03, 4.877370E+03, 4.912427E+03, 4.947617E+03,
    4.982940E+03, 5.018397E+03, 5.053988E+03, 5.089712E+03, 5.125570E+03, 5.161561E+03,
    5.197686E+03, 5.233946E+03, 5.270339E+03, 5.306866E+03, 5.343528E+03, 5.380323E+03,
    5.417253E+03, 5.454318E+03, 5.491516E+03, 5.528849E+03, 5.566317E+03, 5.603919E+03,
    5.641656E+03, 5.679528E+03, 5.717535E+03, 5.755676E+03, 5.793953E+03, 5.832364E+03,
    5.870911E+03, 5.909592E+03, 5.948409E+03, 5.987362E+03, 6.026449E+03, 6.065672E+03,
    6.105031E+03, 6.144525E+03, 6.184155E+03, 6.223920E+03, 6.263821E+03, 6.303858E+03,
    6.344031E+03, 6.384340E+03, 6.424784E+03, 6.465365E+03, 6.506082E+03, 6.546935E+03,
    6.587924E+03, 6.629050E+03, 6.670312E+03, 6.711710E+03, 6.753245E+03, 6.794916E+03,
    6.836724E+03, 6.878669E+03, 6.920750E+03, 6.962968E+03, 7.005323E+03, 7.047814E+03,
    7.090443E+03, 7.133208E+03, 7.176111E+03, 7.219151E+03, 7.262327E+03, 7.305641E+03,
    7.349092E+03, 7.392681E+03, 7.436407E+03, 7.480270E+03, 7.524270E+03, 7.568408E+03,
    7.612684E+03, 7.657097E+03, 7.701647E+03, 7.746336E+03, 7.791162E+03, 7.836126E+03,
    7.881227E+03, 7.926467E+03, 7.971844E+03, 8.017359E+03, 8.063013E+03, 8.108804E+03,
    8.154733E+03, 8.200801E+03, 8.247006E+03, 8.293350E+03, 8.339832E+03, 8.386452E+03,
    8.433210E+03, 8.480107E+03, 8.527142E+03, 8.574316E+03, 8.621628E+03, 8.669078E+03,
    8.716667E+03, 8.764395E+03, 8.812261E+03, 8.860266E+03, 8.908409E+03, 8.956691E+03,
    9.005112E+03, 9.053671E+03, 9.102369E+03, 9.151206E+03, 9.200182E+03, 9.249297E+03,
    9.298551E+03, 9.347943E+03, 9.397474E+03, 9.447145E+03, 9.496954E+03, 9.546902E+03,
    9.596990E+03, 9.647216E+03, 9.697582E+03, 9.748087E+03, 9.798730E+03, 9.849513E+03,
    9.900435E+03, 9.951496E+03, 1.000270E+04, 1.005404E+04, 1.010552E+04, 1.015713E+04,
    1.020889E+04, 1.026079E+04, 1.031282E+04, 1.036500E+04, 1.041731E+04, 1.046977E+04,
    1.052236E+04, 1.057509E+04, 1.062797E+04, 1.068098E+04, 1.073413E+04, 1.078742E+04,
    1.084085E+04, 1.089442E+04, 1.094813E+04, 1.100198E+04, 1.105596E+04, 1.111009E+04,
    1.116436E+04, 1.121876E+04, 1.127331E+04, 1.132799E+04, 1.138282E+04, 1.143778E+04,
    1.149289E+04, 1.154813E+04, 1.160351E+04, 1.165903E+04, 1.171470E+04, 1.177050E+04,
    1.182644E+04, 1.188252E+04, 1.193874E+04, 1.199510E+04, 1.205159E+04, 1.210823E+04,
    1.216501E+04, 1.222193E+04, 1.227898E+04, 1.233618E+04, 1.239351E+04, 1.245099E+04,
    1.250860E+04, 1.256636E+04, 1.262425E+04, 1.268228E+04, 1.274045E+04, 1.279876E+04,
    1.285722E+04, 1.291581E+04, 1.297453E+04, 1.303340E+04, 1.309241E+04, 1.315156E+04,
    1.321084E+04, 1.327027E+04, 1.332983E+04, 1.338954E+04, 1.344938E+04, 1.350936E+04,
    1.356949E+04, 1.362975E+04, 1.369015E+04, 1.375069E+04, 1.381136E+04, 1.387218E+04,
    1.393314E+04, 1.399423E+04, 1.405547E+04, 1.411684E+04, 1.417835E+04, 1.424000E+04,
    1.430179E+04, 1.436372E+04, 1.442579E+04, 1.448799E+04, 1.455034E+04, 1.461282E+04,
    1.467545E+04,
])

#  ---------------------- M = 5, I = 9 ---------------------------
M = 5
I = 9
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.128630E+00, 5.062329E+01, 9.917483E+01, 1.477481E+02, 1.963300E+02, 2.449180E+02,
    2.935114E+02, 3.421098E+02, 3.907129E+02, 4.393208E+02, 4.879334E+02, 5.365511E+02,
    5.851749E+02, 6.338065E+02, 6.824492E+02, 7.311080E+02, 7.797902E+02, 8.285052E+02,
    8.772651E+02, 9.260840E+02, 9.749786E+02, 1.023967E+03, 1.073070E+03, 1.122307E+03,
    1.171702E+03, 1.221277E+03, 1.271055E+03, 1.321059E+03, 1.371312E+03, 1.421836E+03,
    1.472653E+03, 1.523784E+03, 1.575250E+03, 1.627071E+03, 1.679265E+03, 1.731850E+03,
    1.784843E+03, 1.838260E+03, 1.892117E+03, 1.946428E+03, 2.001207E+03, 2.056467E+03,
    2.112219E+03, 2.168475E+03, 2.225246E+03, 2.282542E+03, 2.340373E+03, 2.398746E+03,
    2.457671E+03, 2.517156E+03, 2.577207E+03, 2.637832E+03, 2.699037E+03, 2.760829E+03,
    2.823213E+03, 2.886195E+03, 2.949779E+03, 3.013971E+03, 3.078775E+03, 3.144196E+03,
    3.210237E+03, 3.276902E+03, 3.344196E+03, 3.412121E+03, 3.480680E+03, 3.549878E+03,
    3.619716E+03, 3.690198E+03, 3.761326E+03, 3.833102E+03, 3.905530E+03, 3.978611E+03,
    4.052348E+03, 4.126742E+03, 4.201796E+03, 4.277511E+03, 4.353890E+03, 4.430934E+03,
    4.508645E+03, 4.587024E+03, 4.666073E+03, 4.745793E+03, 4.826187E+03, 4.907254E+03,
    4.988997E+03, 5.071417E+03, 5.154515E+03, 5.238293E+03, 5.322751E+03, 5.407890E+03,
    5.493712E+03, 5.580218E+03, 5.667408E+03, 5.755285E+03, 5.843848E+03, 5.933099E+03,
    6.023038E+03, 6.113667E+03, 6.204986E+03, 6.296996E+03, 6.389699E+03, 6.483094E+03,
    6.577183E+03, 6.671967E+03, 6.767446E+03, 6.863621E+03, 6.960492E+03, 7.058061E+03,
    7.156329E+03, 7.255295E+03, 7.354961E+03, 7.455327E+03, 7.556394E+03, 7.658162E+03,
    7.760633E+03, 7.863807E+03, 7.967684E+03, 8.072266E+03, 8.177552E+03, 8.283544E+03,
    8.390242E+03, 8.497647E+03, 8.605758E+03, 8.714578E+03, 8.824106E+03, 8.934343E+03,
    9.045289E+03, 9.156946E+03, 9.269313E+03, 9.382392E+03, 9.496182E+03, 9.610685E+03,
    9.725901E+03, 9.841830E+03, 9.958473E+03, 1.007583E+04, 1.019390E+04, 1.031269E+04,
    1.043220E+04, 1.055242E+04, 1.067336E+04, 1.079501E+04, 1.091739E+04, 1.104048E+04,
    1.116429E+04, 1.128882E+04, 1.141408E+04, 1.154005E+04, 1.166674E+04, 1.179416E+04,
    1.192230E+04, 1.205116E+04, 1.218075E+04, 1.231106E+04, 1.244209E+04, 1.257385E+04,
    1.270633E+04, 1.283954E+04, 1.297348E+04, 1.310814E+04, 1.324353E+04, 1.337965E+04,
    1.351649E+04, 1.365407E+04, 1.379237E+04, 1.393140E+04, 1.407117E+04, 1.421166E+04,
    1.435289E+04, 1.449485E+04, 1.463754E+04, 1.478096E+04, 1.492512E+04, 1.507001E+04,
    1.521563E+04, 1.536199E+04, 1.550909E+04, 1.565692E+04, 1.580548E+04, 1.595479E+04,
    1.610483E+04, 1.625560E+04, 1.640712E+04, 1.655937E+04, 1.671237E+04, 1.686610E+04,
    1.702057E+04, 1.717579E+04, 1.733174E+04, 1.748844E+04, 1.764588E+04, 1.780406E+04,
    1.796298E+04, 1.812265E+04, 1.828306E+04, 1.844421E+04, 1.860611E+04, 1.876876E+04,
    1.893215E+04, 1.909629E+04, 1.926117E+04, 1.942680E+04, 1.959318E+04, 1.976031E+04,
    1.992819E+04, 2.009681E+04, 2.026619E+04, 2.043631E+04, 2.060719E+04, 2.077882E+04,
    2.095120E+04, 2.112433E+04, 2.129821E+04, 2.147285E+04, 2.164824E+04, 2.182438E+04,
    2.200128E+04, 2.217893E+04, 2.235734E+04, 2.253651E+04, 2.271643E+04, 2.289711E+04,
    2.307854E+04, 2.326073E+04, 2.344368E+04, 2.362739E+04, 2.381186E+04, 2.399709E+04,
    2.418308E+04, 2.436983E+04, 2.455734E+04, 2.474561E+04, 2.493464E+04, 2.512444E+04,
    2.531500E+04, 2.550632E+04, 2.569841E+04, 2.589126E+04, 2.608487E+04, 2.627925E+04,
    2.647440E+04, 2.667031E+04, 2.686699E+04, 2.706444E+04, 2.726265E+04, 2.746163E+04,
    2.766138E+04, 2.786190E+04, 2.806319E+04, 2.826524E+04, 2.846807E+04, 2.867167E+04,
    2.887604E+04, 2.908118E+04, 2.928709E+04, 2.949378E+04, 2.970124E+04, 2.990947E+04,
    3.011847E+04, 3.032825E+04, 3.053880E+04, 3.075013E+04, 3.096224E+04, 3.117512E+04,
    3.138877E+04, 3.160321E+04, 3.181842E+04, 3.203441E+04, 3.225117E+04, 3.246872E+04,
    3.268704E+04, 3.290614E+04, 3.312602E+04, 3.334669E+04, 3.356813E+04, 3.379035E+04,
    3.401336E+04, 3.423714E+04, 3.446171E+04, 3.468706E+04, 3.491320E+04, 3.514011E+04,
    3.536782E+04, 3.559630E+04, 3.582557E+04, 3.605563E+04, 3.628647E+04, 3.651809E+04,
    3.675050E+04, 3.698370E+04, 3.721769E+04, 3.745246E+04, 3.768802E+04, 3.792437E+04,
    3.816150E+04, 3.839943E+04, 3.863814E+04, 3.887764E+04, 3.911794E+04, 3.935902E+04,
    3.960089E+04, 3.984356E+04, 4.008701E+04, 4.033126E+04, 4.057630E+04, 4.082213E+04,
    4.106875E+04, 4.131617E+04, 4.156438E+04, 4.181338E+04, 4.206318E+04, 4.231377E+04,
    4.256515E+04, 4.281734E+04, 4.307031E+04, 4.332408E+04, 4.357865E+04, 4.383401E+04,
    4.409017E+04, 4.434713E+04, 4.460488E+04, 4.486344E+04, 4.512278E+04, 4.538293E+04,
    4.564388E+04, 4.590562E+04, 4.616816E+04, 4.643150E+04, 4.669564E+04, 4.696058E+04,
    4.722632E+04, 4.749286E+04, 4.776021E+04, 4.802835E+04, 4.829729E+04, 4.856703E+04,
    4.883758E+04, 4.910893E+04, 4.938108E+04, 4.965403E+04, 4.992778E+04, 5.020234E+04,
    5.047770E+04, 5.075386E+04, 5.103083E+04, 5.130860E+04, 5.158717E+04, 5.186655E+04,
    5.214674E+04, 5.242772E+04, 5.270951E+04, 5.299211E+04, 5.327551E+04, 5.355972E+04,
    5.384474E+04, 5.413056E+04, 5.441718E+04, 5.470461E+04, 5.499285E+04, 5.528189E+04,
    5.557175E+04, 5.586240E+04, 5.615387E+04, 5.644614E+04, 5.673922E+04, 5.703311E+04,
    5.732781E+04, 5.762331E+04, 5.791962E+04, 5.821674E+04, 5.851467E+04, 5.881341E+04,
    5.911295E+04, 5.941331E+04, 5.971447E+04, 6.001644E+04, 6.031922E+04, 6.062281E+04,
    6.092721E+04, 6.123242E+04, 6.153844E+04, 6.184527E+04, 6.215291E+04, 6.246136E+04,
    6.277062E+04, 6.308069E+04, 6.339157E+04, 6.370326E+04, 6.401576E+04, 6.432907E+04,
    6.464320E+04, 6.495813E+04, 6.527387E+04, 6.559043E+04, 6.590779E+04, 6.622597E+04,
    6.654496E+04, 6.686476E+04, 6.718537E+04, 6.750679E+04, 6.782902E+04, 6.815206E+04,
    6.847592E+04, 6.880058E+04, 6.912606E+04, 6.945235E+04, 6.977945E+04, 7.010736E+04,
    7.043609E+04, 7.076562E+04, 7.109597E+04, 7.142713E+04, 7.175910E+04, 7.209188E+04,
    7.242547E+04, 7.275988E+04, 7.309509E+04, 7.343112E+04, 7.376796E+04, 7.410561E+04,
    7.444407E+04, 7.478335E+04, 7.512343E+04, 7.546433E+04, 7.580603E+04, 7.614855E+04,
    7.649188E+04, 7.683602E+04, 7.718098E+04, 7.752674E+04, 7.787331E+04, 7.822070E+04,
    7.856889E+04, 7.891790E+04, 7.926772E+04, 7.961835E+04, 7.996979E+04, 8.032204E+04,
    8.067510E+04, 8.102897E+04, 8.138365E+04, 8.173914E+04, 8.209544E+04, 8.245255E+04,
    8.281047E+04, 8.316920E+04, 8.352874E+04, 8.388909E+04, 8.425025E+04, 8.461221E+04,
    8.497499E+04,
])

#  ---------------------- M = 6, I = 1 ---------------------------
M = 6
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.000000E+00, 1.273695E+01, 3.049472E+01, 5.478766E+01, 8.367753E+01, 1.164151E+02,
    1.525835E+02, 1.918873E+02, 2.341011E+02, 2.790524E+02, 3.266165E+02, 3.767179E+02,
    4.293326E+02, 4.844902E+02, 5.422734E+02, 6.028155E+02, 6.662963E+02, 7.329384E+02,
    8.030016E+02, 8.767796E+02, 9.545964E+02, 1.036804E+03, 1.123780E+03, 1.215929E+03,
    1.313679E+03, 1.417485E+03, 1.527828E+03, 1.645218E+03, 1.770193E+03, 1.903324E+03,
    2.045215E+03, 2.196504E+03, 2.357871E+03, 2.530032E+03, 2.713751E+03, 2.909836E+03,
    3.119144E+03, 3.342585E+03, 3.581125E+03, 3.835788E+03, 4.107662E+03, 4.397900E+03,
    4.707726E+03, 5.038440E+03, 5.391417E+03, 5.768118E+03, 6.170091E+03, 6.598978E+03,
    7.056516E+03, 7.544548E+03, 8.065026E+03, 8.620015E+03, 9.211704E+03, 9.842405E+03,
    1.051457E+04, 1.123079E+04, 1.199379E+04, 1.280649E+04, 1.367192E+04, 1.459334E+04,
    1.557414E+04, 1.661792E+04, 1.772850E+04, 1.890986E+04, 2.016624E+04, 2.150208E+04,
    2.292207E+04, 2.443115E+04, 2.603450E+04, 2.773758E+04, 2.954614E+04, 3.150651E+04,
    3.355181E+04, 3.572292E+04, 3.802703E+04, 4.047168E+04, 4.306485E+04, 4.581488E+04,
    4.873055E+04, 5.182111E+04, 5.509623E+04, 5.856610E+04, 6.224140E+04, 6.613336E+04,
    7.025373E+04, 7.461487E+04, 7.922970E+04, 8.411182E+04, 8.927543E+04, 9.473544E+04,
    1.005075E+05, 1.066079E+05, 1.130538E+05, 1.198631E+05, 1.270545E+05, 1.346478E+05,
    1.426634E+05, 1.511227E+05, 1.600483E+05, 1.694635E+05, 1.793929E+05, 1.898620E+05,
    2.008977E+05, 2.125277E+05, 2.247814E+05, 2.376890E+05, 2.512823E+05, 2.655945E+05,
    2.806600E+05, 2.965148E+05, 3.131965E+05, 3.307442E+05, 3.491987E+05, 3.686022E+05,
    3.889992E+05, 4.104356E+05, 4.329593E+05, 4.566203E+05, 4.814703E+05, 5.075634E+05,
    5.349557E+05, 5.637057E+05, 5.938740E+05, 6.255236E+05, 6.587202E+05, 6.935319E+05,
    7.300294E+05, 7.682862E+05, 8.083785E+05, 8.503856E+05, 8.943897E+05, 9.404760E+05,
    9.887330E+05, 1.039252E+06, 1.092129E+06, 1.147463E+06, 1.205355E+06, 1.265911E+06,
    1.329242E+06, 1.395461E+06, 1.464686E+06, 1.537038E+06, 1.612646E+06, 1.691638E+06,
    1.774151E+06, 1.860324E+06, 1.950303E+06, 2.044237E+06, 2.142281E+06, 2.244596E+06,
    2.351347E+06, 2.462706E+06, 2.578850E+06, 2.699960E+06, 2.826228E+06, 2.957846E+06,
    3.095017E+06, 3.237950E+06, 3.386857E+06, 3.541963E+06, 3.703494E+06, 3.871687E+06,
    4.046785E+06, 4.229040E+06, 4.418710E+06, 4.616063E+06, 4.821373E+06, 5.034924E+06,
    5.257009E+06, 5.487928E+06, 5.727992E+06, 5.977520E+06, 6.236841E+06, 6.506295E+06,
    6.786230E+06, 7.077006E+06,
])

#  ---------------------- M = 6, I = 2 ---------------------------
M = 6
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.000001E+01, 2.547411E+01, 6.098959E+01, 1.095756E+02, 1.673556E+02, 2.328309E+02,
    3.051677E+02, 3.837737E+02, 4.682005E+02, 5.581019E+02, 6.532270E+02, 7.534273E+02,
    8.586439E+02, 9.689291E+02, 1.084461E+03, 1.205482E+03, 1.332363E+03, 1.465530E+03,
    1.605503E+03, 1.752860E+03, 1.908229E+03, 2.072336E+03, 2.245903E+03, 2.429737E+03,
    2.624664E+03, 2.831592E+03, 3.051489E+03, 3.285303E+03, 3.534174E+03, 3.799126E+03,
    4.081393E+03, 4.382239E+03, 4.702959E+03, 5.044945E+03, 5.409694E+03, 5.798822E+03,
    6.213891E+03, 6.656804E+03, 7.129283E+03, 7.633410E+03, 8.171277E+03, 8.745069E+03,
    9.357183E+03, 1.001001E+04, 1.070636E+04, 1.144891E+04, 1.224067E+04, 1.308471E+04,
    1.398443E+04, 1.494334E+04, 1.596502E+04, 1.705351E+04, 1.821287E+04, 1.944758E+04,
    2.076216E+04, 2.216156E+04, 2.365078E+04, 2.523547E+04, 2.692124E+04, 2.871415E+04,
    3.062057E+04, 3.264725E+04, 3.480122E+04, 3.708988E+04, 3.952125E+04, 4.210358E+04,
    4.484543E+04, 4.775615E+04, 5.084522E+04, 5.412274E+04, 5.759952E+04, 6.128669E+04,
    6.519606E+04, 6.933999E+04, 7.373142E+04, 7.838393E+04, 8.331199E+04, 8.853045E+04,
    9.405527E+04, 9.990284E+04, 1.060906E+05, 1.126365E+05, 1.195598E+05, 1.268805E+05,
    1.346193E+05, 1.427982E+05, 1.514402E+05, 1.605692E+05, 1.702104E+05, 1.803902E+05,
    1.911358E+05, 2.024762E+05, 2.144415E+05, 2.270628E+05, 2.403733E+05, 2.544072E+05,
    2.692002E+05, 2.847901E+05, 3.012156E+05, 3.185180E+05, 3.367395E+05, 3.559247E+05,
    3.761201E+05, 3.973740E+05, 4.197368E+05, 4.432611E+05, 4.680017E+05, 4.940161E+05,
    5.213635E+05, 5.501059E+05, 5.803083E+05, 6.120376E+05, 6.453640E+05, 6.803606E+05,
    7.171028E+05, 7.556704E+05, 7.961449E+05, 8.386119E+05, 8.831604E+05, 9.298828E+05,
    9.788752E+05, 1.030238E+06, 1.084073E+06, 1.140491E+06, 1.199602E+06, 1.261522E+06,
    1.326373E+06, 1.394281E+06, 1.465374E+06, 1.539790E+06, 1.617667E+06, 1.699151E+06,
    1.784394E+06, 1.873552E+06, 1.966787E+06, 2.064268E+06, 2.166169E+06, 2.272673E+06,
    2.383965E+06, 2.500240E+06, 2.621701E+06, 2.748556E+06, 2.881020E+06, 3.019317E+06,
    3.163680E+06, 3.314348E+06, 3.471569E+06, 3.635602E+06, 3.806711E+06, 3.985172E+06,
    4.171270E+06, 4.365299E+06, 4.567564E+06, 4.778380E+06, 4.998073E+06, 5.226979E+06,
    5.465447E+06, 5.713837E+06, 5.972520E+06, 6.241880E+06, 6.522314E+06, 6.814234E+06,
    7.118060E+06, 7.434232E+06, 7.763201E+06, 8.105433E+06, 8.461408E+06, 8.831625E+06,
    9.216596E+06, 9.616850E+06, 1.003293E+07, 1.046541E+07, 1.091485E+07, 1.138187E+07,
    1.186708E+07, 1.237112E+07,
])

#  ---------------------- M = 6, I = 3 ---------------------------
M = 6
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[4]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.136790E+00, 9.190122E+01, 2.433209E+02, 4.407898E+02, 6.744479E+02, 9.391965E+02,
    1.231752E+03, 1.549735E+03, 1.891357E+03, 2.255315E+03, 2.640780E+03, 3.047413E+03,
    3.475380E+03, 3.925354E+03, 4.398488E+03, 4.896378E+03, 5.421024E+03, 5.974778E+03,
    6.560313E+03, 7.180591E+03, 7.838835E+03, 8.538523E+03, 9.283379E+03, 1.007737E+04,
    1.092472E+04, 1.182991E+04, 1.279770E+04, 1.383315E+04, 1.494161E+04, 1.612880E+04,
    1.740075E+04, 1.876392E+04, 2.022515E+04, 2.179171E+04, 2.347136E+04, 2.527233E+04,
    2.720338E+04, 2.927384E+04, 3.149364E+04, 3.387330E+04, 3.642407E+04, 3.915786E+04,
    4.208734E+04, 4.522600E+04, 4.858815E+04, 5.218897E+04, 5.604463E+04, 6.017224E+04,
    6.458999E+04, 6.931716E+04, 7.437420E+04, 7.978279E+04, 8.556591E+04, 9.174789E+04,
    9.835451E+04, 1.054131E+05, 1.129524E+05, 1.210032E+05, 1.295976E+05, 1.387699E+05,
    1.485562E+05, 1.589946E+05, 1.701255E+05, 1.819914E+05, 1.946373E+05, 2.081105E+05,
    2.224612E+05, 2.377420E+05, 2.540087E+05, 2.713198E+05, 2.897370E+05, 3.093255E+05,
    3.301538E+05, 3.522940E+05, 3.758220E+05, 4.008176E+05, 4.273649E+05, 4.555521E+05,
    4.854721E+05, 5.172225E+05, 5.509057E+05, 5.866294E+05, 6.245067E+05, 6.646560E+05,
    7.072021E+05, 7.522754E+05, 8.000130E+05, 8.505587E+05, 9.040628E+05, 9.606834E+05,
    1.020586E+06, 1.083943E+06, 1.150937E+06, 1.221758E+06, 1.296603E+06, 1.375682E+06,
    1.459212E+06, 1.547420E+06, 1.640545E+06, 1.738836E+06, 1.842553E+06, 1.951967E+06,
    2.067363E+06, 2.189037E+06, 2.317299E+06, 2.452473E+06, 2.594895E+06, 2.744919E+06,
    2.902912E+06, 3.069257E+06, 3.244354E+06, 3.428620E+06, 3.622489E+06, 3.826415E+06,
    4.040869E+06, 4.266343E+06, 4.503349E+06, 4.752420E+06, 5.014112E+06, 5.289002E+06,
    5.577692E+06, 5.880808E+06, 6.199001E+06, 6.532948E+06, 6.883354E+06, 7.250950E+06,
    7.636499E+06, 8.040792E+06, 8.464650E+06, 8.908929E+06, 9.374515E+06, 9.862331E+06,
    1.037333E+07, 1.090852E+07, 1.146891E+07, 1.205559E+07, 1.266966E+07, 1.331228E+07,
    1.398464E+07, 1.468798E+07, 1.542360E+07, 1.619281E+07, 1.699702E+07, 1.783764E+07,
    1.871616E+07, 1.963413E+07, 2.059313E+07, 2.159483E+07, 2.264092E+07, 2.373319E+07,
    2.487345E+07, 2.606363E+07, 2.730567E+07, 2.860161E+07, 2.995356E+07, 3.136369E+07,
    3.283426E+07, 3.436760E+07, 3.596611E+07, 3.763229E+07, 3.936871E+07, 4.117804E+07,
    4.306303E+07, 4.502651E+07, 4.707144E+07, 4.920084E+07, 5.141786E+07, 5.372573E+07,
    5.612780E+07, 5.862752E+07, 6.122847E+07, 6.393432E+07, 6.674887E+07, 6.967606E+07,
    7.271992E+07, 7.588465E+07, 7.917453E+07, 8.259403E+07, 8.614773E+07, 8.984036E+07,
    9.367680E+07, 9.766207E+07, 1.018014E+08, 1.061000E+08, 1.105636E+08, 1.151977E+08,
    1.200082E+08, 1.250011E+08, 1.301826E+08, 1.355593E+08, 1.411375E+08, 1.469242E+08,
    1.529262E+08, 1.591509E+08, 1.656056E+08, 1.722979E+08, 1.792357E+08, 1.864271E+08,
    1.938803E+08, 2.016041E+08, 2.096070E+08, 2.178983E+08, 2.264872E+08, 2.353833E+08,
    2.445965E+08, 2.541369E+08, 2.640150E+08, 2.742414E+08, 2.848271E+08, 2.957836E+08,
    3.071224E+08, 3.188555E+08, 3.309952E+08, 3.435542E+08, 3.565454E+08, 3.699822E+08,
    3.838782E+08, 3.982476E+08, 4.131049E+08, 4.284647E+08, 4.443425E+08, 4.607539E+08,
    4.777149E+08, 4.952421E+08, 5.133524E+08, 5.320631E+08,
])

#  ---------------------- M = 6, I = 4 ---------------------------
M = 6
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.626998E+01, 1.839479E+02, 4.870414E+02, 8.823115E+02, 1.350022E+03, 1.879967E+03,
    2.465569E+03, 3.102094E+03, 3.785903E+03, 4.514479E+03, 5.286106E+03, 6.100196E+03,
    6.957045E+03, 7.858055E+03, 8.805535E+03, 9.802766E+03, 1.085371E+04, 1.196313E+04,
    1.313628E+04, 1.437935E+04, 1.569859E+04, 1.710107E+04, 1.859418E+04, 2.018607E+04,
    2.188496E+04, 2.370019E+04, 2.564086E+04, 2.771745E+04, 2.994063E+04, 3.232161E+04,
    3.487298E+04, 3.760723E+04, 4.053859E+04, 4.368088E+04, 4.705047E+04, 5.066330E+04,
    5.453713E+04, 5.869100E+04, 6.314418E+04, 6.791804E+04, 7.303545E+04, 7.852005E+04,
    8.439702E+04, 9.069327E+04, 9.743844E+04, 1.046622E+05, 1.123971E+05, 1.206774E+05,
    1.295392E+05, 1.390219E+05, 1.491664E+05, 1.600155E+05, 1.716162E+05, 1.840160E+05,
    1.972679E+05, 2.114260E+05, 2.265476E+05, 2.426952E+05, 2.599326E+05, 2.783288E+05,
    2.979556E+05, 3.188895E+05, 3.412118E+05, 3.650082E+05, 3.903670E+05, 4.173853E+05,
    4.461617E+05, 4.768023E+05, 5.094192E+05, 5.441294E+05, 5.810569E+05, 6.203311E+05,
    6.620900E+05, 7.064783E+05, 7.536471E+05, 8.037577E+05, 8.569763E+05, 9.134819E+05,
    9.734592E+05, 1.037104E+06, 1.104621E+06, 1.176227E+06, 1.252147E+06, 1.332619E+06,
    1.417893E+06, 1.508228E+06, 1.603903E+06, 1.705201E+06, 1.812425E+06, 1.925893E+06,
    2.045933E+06, 2.172894E+06, 2.307139E+06, 2.449047E+06, 2.599017E+06, 2.757465E+06,
    2.924828E+06, 3.101559E+06, 3.288137E+06, 3.485058E+06, 3.692845E+06, 3.912041E+06,
    4.143216E+06, 4.386961E+06, 4.643897E+06, 4.914672E+06, 5.199959E+06, 5.500466E+06,
    5.816930E+06, 6.150113E+06, 6.500818E+06, 6.869878E+06, 7.258162E+06, 7.666580E+06,
    8.096072E+06, 8.547624E+06, 9.022260E+06, 9.521045E+06, 1.004509E+07, 1.059556E+07,
    1.117365E+07, 1.178061E+07, 1.241774E+07, 1.308641E+07, 1.378802E+07, 1.452403E+07,
    1.529597E+07, 1.610542E+07, 1.695402E+07, 1.784349E+07, 1.877560E+07, 1.975219E+07,
    2.077518E+07, 2.184655E+07, 2.296837E+07, 2.414278E+07, 2.537200E+07, 2.665835E+07,
    2.800420E+07, 2.941204E+07, 3.088444E+07, 3.242407E+07, 3.403370E+07, 3.571618E+07,
    3.747450E+07, 3.931172E+07, 4.123103E+07, 4.323575E+07, 4.532928E+07, 4.751517E+07,
    4.979709E+07, 5.217884E+07, 5.466433E+07, 5.725764E+07, 5.996298E+07, 6.278469E+07,
    6.572729E+07, 6.879543E+07, 7.199392E+07, 7.532775E+07, 7.880208E+07, 8.242220E+07,
    8.619364E+07, 9.012207E+07, 9.421338E+07, 9.847362E+07, 1.029091E+08, 1.075262E+08,
    1.123317E+08, 1.173325E+08, 1.225357E+08, 1.279487E+08, 1.335791E+08, 1.394346E+08,
    1.455235E+08, 1.518540E+08,
])

#  ---------------------- M = 7, I = 1 ---------------------------
M = 7
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.259272E+00, 1.541160E+01, 2.984283E+01, 4.432568E+01, 5.882297E+01, 7.332720E+01,
    8.783584E+01, 1.023478E+02, 1.168627E+02, 1.313804E+02, 1.459015E+02, 1.604275E+02,
    1.749609E+02, 1.895058E+02, 2.040679E+02, 2.186540E+02, 2.332724E+02, 2.479320E+02,
    2.626424E+02, 2.774132E+02, 2.922540E+02, 3.071739E+02, 3.221816E+02, 3.372850E+02,
    3.524912E+02, 3.678063E+02, 3.832358E+02, 3.987841E+02, 4.144551E+02, 4.302515E+02,
    4.461757E+02, 4.622292E+02, 4.784130E+02, 4.947276E+02, 5.111729E+02, 5.277485E+02,
    5.444538E+02, 5.612875E+02, 5.782484E+02, 5.953349E+02, 6.125453E+02, 6.298776E+02,
    6.473299E+02, 6.649000E+02, 6.825859E+02, 7.003851E+02, 7.182956E+02, 7.363149E+02,
    7.544408E+02, 7.726710E+02, 7.910032E+02, 8.094352E+02, 8.279648E+02, 8.465897E+02,
    8.653078E+02, 8.841171E+02, 9.030155E+02, 9.220009E+02, 9.410715E+02, 9.602254E+02,
    9.794607E+02, 9.987757E+02, 1.018169E+03, 1.037638E+03, 1.057182E+03, 1.076799E+03,
    1.096488E+03, 1.116247E+03, 1.136075E+03, 1.155971E+03, 1.175934E+03, 1.195961E+03,
    1.216053E+03, 1.236207E+03, 1.256424E+03, 1.276701E+03, 1.297039E+03, 1.317435E+03,
    1.337890E+03, 1.358403E+03, 1.378972E+03, 1.399598E+03, 1.420279E+03, 1.441014E+03,
    1.461804E+03, 1.482647E+03, 1.503544E+03, 1.524493E+03, 1.545494E+03, 1.566547E+03,
    1.587651E+03, 1.608806E+03, 1.630012E+03, 1.651268E+03, 1.672575E+03, 1.693931E+03,
    1.715336E+03, 1.736791E+03, 1.758295E+03, 1.779848E+03, 1.801450E+03, 1.823100E+03,
    1.844800E+03, 1.866547E+03, 1.888343E+03, 1.910188E+03, 1.932080E+03, 1.954021E+03,
    1.976010E+03, 1.998048E+03, 2.020133E+03, 2.042267E+03, 2.064449E+03, 2.086680E+03,
    2.108959E+03, 2.131286E+03, 2.153662E+03, 2.176086E+03, 2.198559E+03, 2.221081E+03,
    2.243651E+03, 2.266271E+03, 2.288939E+03, 2.311657E+03, 2.334424E+03, 2.357240E+03,
    2.380105E+03, 2.403021E+03, 2.425986E+03, 2.449001E+03, 2.472066E+03, 2.495181E+03,
    2.518346E+03, 2.541562E+03, 2.564828E+03, 2.588145E+03, 2.611513E+03, 2.634932E+03,
    2.658402E+03, 2.681924E+03, 2.705497E+03, 2.729122E+03, 2.752798E+03, 2.776526E+03,
    2.800307E+03, 2.824140E+03, 2.848025E+03, 2.871962E+03, 2.895953E+03, 2.919996E+03,
    2.944092E+03, 2.968242E+03, 2.992444E+03, 3.016701E+03, 3.041010E+03, 3.065374E+03,
    3.089791E+03, 3.114262E+03, 3.138788E+03, 3.163367E+03, 3.188002E+03, 3.212690E+03,
    3.237434E+03, 3.262232E+03, 3.287085E+03, 3.311993E+03, 3.336957E+03, 3.361975E+03,
    3.387050E+03, 3.412179E+03, 3.437365E+03, 3.462606E+03, 3.487903E+03, 3.513256E+03,
    3.538665E+03, 3.564130E+03, 3.589652E+03, 3.615230E+03, 3.640865E+03, 3.666556E+03,
    3.692304E+03, 3.718109E+03, 3.743970E+03, 3.769889E+03, 3.795864E+03, 3.821897E+03,
    3.847987E+03, 3.874134E+03, 3.900338E+03, 3.926600E+03, 3.952920E+03, 3.979297E+03,
    4.005731E+03, 4.032224E+03, 4.058774E+03, 4.085382E+03, 4.112047E+03, 4.138771E+03,
    4.165553E+03, 4.192392E+03, 4.219290E+03, 4.246246E+03, 4.273260E+03, 4.300332E+03,
    4.327463E+03, 4.354651E+03, 4.381898E+03, 4.409204E+03, 4.436567E+03, 4.463989E+03,
    4.491470E+03, 4.519009E+03, 4.546606E+03, 4.574262E+03, 4.601977E+03, 4.629750E+03,
    4.657581E+03, 4.685471E+03, 4.713419E+03, 4.741426E+03, 4.769492E+03, 4.797616E+03,
    4.825799E+03, 4.854040E+03, 4.882340E+03, 4.910698E+03, 4.939115E+03, 4.967591E+03,
    4.996125E+03, 5.024717E+03, 5.053368E+03, 5.082078E+03, 5.110846E+03, 5.139672E+03,
    5.168557E+03, 5.197500E+03, 5.226502E+03, 5.255562E+03, 5.284680E+03, 5.313857E+03,
    5.343091E+03, 5.372384E+03, 5.401736E+03, 5.431145E+03, 5.460613E+03, 5.490138E+03,
    5.519722E+03, 5.549364E+03, 5.579063E+03, 5.608821E+03, 5.638636E+03, 5.668509E+03,
    5.698440E+03, 5.728429E+03, 5.758475E+03, 5.788579E+03, 5.818741E+03, 5.848960E+03,
    5.879236E+03, 5.909570E+03, 5.939961E+03, 5.970409E+03, 6.000915E+03, 6.031477E+03,
    6.062097E+03, 6.092774E+03, 6.123507E+03, 6.154298E+03, 6.185145E+03, 6.216049E+03,
    6.247009E+03, 6.278026E+03, 6.309100E+03, 6.340229E+03, 6.371416E+03, 6.402658E+03,
    6.433957E+03, 6.465311E+03, 6.496722E+03, 6.528188E+03, 6.559710E+03, 6.591288E+03,
    6.622922E+03, 6.654611E+03, 6.686355E+03, 6.718155E+03, 6.750010E+03, 6.781920E+03,
    6.813886E+03, 6.845906E+03, 6.877981E+03, 6.910111E+03, 6.942296E+03, 6.974535E+03,
    7.006828E+03, 7.039176E+03, 7.071578E+03, 7.104035E+03, 7.136545E+03, 7.169109E+03,
    7.201728E+03, 7.234399E+03, 7.267125E+03, 7.299904E+03, 7.332736E+03, 7.365622E+03,
    7.398560E+03, 7.431552E+03, 7.464597E+03, 7.497694E+03, 7.530844E+03, 7.564047E+03,
    7.597302E+03, 7.630610E+03, 7.663969E+03, 7.697381E+03, 7.730845E+03, 7.764360E+03,
    7.797928E+03, 7.831547E+03, 7.865217E+03, 7.898939E+03, 7.932712E+03, 7.966535E+03,
    8.000410E+03, 8.034336E+03, 8.068313E+03, 8.102340E+03, 8.136417E+03, 8.170545E+03,
    8.204723E+03, 8.238951E+03, 8.273228E+03, 8.307556E+03, 8.341933E+03, 8.376360E+03,
    8.410836E+03, 8.445361E+03, 8.479935E+03, 8.514559E+03, 8.549231E+03, 8.583951E+03,
    8.618721E+03, 8.653538E+03, 8.688404E+03, 8.723318E+03, 8.758280E+03, 8.793289E+03,
    8.828347E+03, 8.863452E+03, 8.898604E+03, 8.933803E+03, 8.969050E+03, 9.004343E+03,
    9.039683E+03, 9.075070E+03, 9.110503E+03, 9.145983E+03, 9.181509E+03, 9.217080E+03,
    9.252698E+03, 9.288362E+03, 9.324071E+03, 9.359825E+03, 9.395625E+03, 9.431470E+03,
    9.467359E+03, 9.503294E+03, 9.539273E+03, 9.575297E+03, 9.611365E+03, 9.647478E+03,
    9.683634E+03, 9.719834E+03, 9.756078E+03, 9.792366E+03,
])

#  ---------------------- M = 7, I = 2 ---------------------------
M = 7
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.562445E+00, 3.092123E+01, 6.151168E+01, 9.218904E+01, 1.228922E+02, 1.536083E+02,
    1.843329E+02, 2.150641E+02, 2.458012E+02, 2.765444E+02, 3.072954E+02, 3.380581E+02,
    3.688395E+02, 3.996500E+02, 4.305039E+02, 4.614188E+02, 4.924152E+02, 5.235163E+02,
    5.547468E+02, 5.861323E+02, 6.176990E+02, 6.494730E+02, 6.814797E+02, 7.137437E+02,
    7.462888E+02, 7.791373E+02, 8.123102E+02, 8.458274E+02, 8.797073E+02, 9.139671E+02,
    9.486225E+02, 9.836883E+02, 1.019178E+03, 1.055104E+03, 1.091478E+03, 1.128311E+03,
    1.165612E+03, 1.203390E+03, 1.241654E+03, 1.280411E+03, 1.319669E+03, 1.359433E+03,
    1.399711E+03, 1.440506E+03, 1.481826E+03, 1.523674E+03, 1.566055E+03, 1.608973E+03,
    1.652432E+03, 1.696437E+03, 1.740989E+03, 1.786093E+03, 1.831752E+03, 1.877969E+03,
    1.924746E+03, 1.972085E+03, 2.019991E+03, 2.068465E+03, 2.117509E+03, 2.167125E+03,
    2.217317E+03, 2.268085E+03, 2.319433E+03, 2.371362E+03, 2.423874E+03, 2.476971E+03,
    2.530656E+03, 2.584929E+03, 2.639793E+03, 2.695250E+03, 2.751302E+03, 2.807950E+03,
    2.865197E+03, 2.923044E+03, 2.981493E+03, 3.040546E+03, 3.100206E+03, 3.160473E+03,
    3.221350E+03, 3.282838E+03, 3.344940E+03, 3.407658E+03, 3.470993E+03, 3.534948E+03,
    3.599524E+03, 3.664723E+03, 3.730548E+03, 3.797000E+03, 3.864082E+03, 3.931795E+03,
    4.000141E+03, 4.069124E+03, 4.138744E+03, 4.209003E+03, 4.279905E+03, 4.351450E+03,
    4.423642E+03, 4.496482E+03, 4.569972E+03, 4.644115E+03, 4.718912E+03, 4.794367E+03,
    4.870480E+03, 4.947254E+03, 5.024692E+03, 5.102796E+03, 5.181567E+03, 5.261008E+03,
    5.341122E+03, 5.421910E+03, 5.503374E+03, 5.585517E+03, 5.668341E+03, 5.751848E+03,
    5.836040E+03, 5.920920E+03, 6.006490E+03, 6.092751E+03, 6.179707E+03, 6.267359E+03,
    6.355709E+03, 6.444760E+03, 6.534513E+03, 6.624971E+03, 6.716136E+03, 6.808010E+03,
    6.900596E+03, 6.993894E+03, 7.087908E+03, 7.182639E+03, 7.278090E+03, 7.374263E+03,
    7.471159E+03, 7.568780E+03, 7.667129E+03, 7.766208E+03, 7.866019E+03, 7.966563E+03,
    8.067842E+03, 8.169859E+03, 8.272615E+03, 8.376113E+03, 8.480353E+03, 8.585339E+03,
    8.691071E+03, 8.797553E+03, 8.904784E+03, 9.012769E+03, 9.121507E+03, 9.231001E+03,
    9.341252E+03, 9.452263E+03, 9.564035E+03, 9.676570E+03, 9.789869E+03, 9.903935E+03,
    1.001877E+04, 1.013437E+04, 1.025074E+04, 1.036789E+04, 1.048581E+04, 1.060450E+04,
    1.072397E+04, 1.084422E+04, 1.096526E+04, 1.108707E+04, 1.120966E+04, 1.133304E+04,
    1.145721E+04, 1.158216E+04, 1.170790E+04, 1.183443E+04, 1.196175E+04, 1.208986E+04,
    1.221877E+04, 1.234847E+04, 1.247897E+04, 1.261026E+04, 1.274236E+04, 1.287525E+04,
    1.300894E+04, 1.314344E+04, 1.327874E+04, 1.341484E+04, 1.355175E+04, 1.368946E+04,
    1.382798E+04, 1.396731E+04, 1.410745E+04, 1.424840E+04, 1.439016E+04, 1.453273E+04,
    1.467611E+04, 1.482031E+04, 1.496532E+04, 1.511115E+04, 1.525780E+04, 1.540526E+04,
    1.555354E+04, 1.570264E+04, 1.585255E+04, 1.600329E+04, 1.615485E+04, 1.630723E+04,
    1.646043E+04, 1.661446E+04, 1.676931E+04, 1.692498E+04, 1.708148E+04, 1.723880E+04,
    1.739695E+04, 1.755593E+04, 1.771573E+04, 1.787636E+04, 1.803782E+04, 1.820010E+04,
    1.836322E+04, 1.852716E+04, 1.869193E+04, 1.885754E+04, 1.902397E+04, 1.919124E+04,
    1.935933E+04, 1.952826E+04, 1.969802E+04, 1.986861E+04, 2.004004E+04, 2.021229E+04,
    2.038538E+04, 2.055931E+04, 2.073406E+04, 2.090965E+04, 2.108607E+04, 2.126333E+04,
    2.144142E+04, 2.162034E+04, 2.180010E+04, 2.198070E+04, 2.216212E+04, 2.234438E+04,
    2.252748E+04, 2.271141E+04, 2.289618E+04, 2.308177E+04, 2.326821E+04, 2.345548E+04,
    2.364358E+04, 2.383251E+04, 2.402229E+04, 2.421289E+04, 2.440433E+04, 2.459660E+04,
    2.478971E+04, 2.498365E+04, 2.517842E+04, 2.537403E+04, 2.557046E+04, 2.576774E+04,
    2.596584E+04, 2.616477E+04, 2.636454E+04, 2.656514E+04, 2.676657E+04, 2.696883E+04,
    2.717193E+04, 2.737585E+04, 2.758060E+04, 2.778618E+04, 2.799260E+04, 2.819984E+04,
    2.840791E+04, 2.861680E+04, 2.882653E+04, 2.903708E+04, 2.924846E+04, 2.946066E+04,
    2.967369E+04, 2.988754E+04, 3.010222E+04, 3.031773E+04, 3.053405E+04, 3.075120E+04,
    3.096917E+04, 3.118797E+04, 3.140758E+04, 3.162802E+04, 3.184927E+04, 3.207135E+04,
    3.229424E+04, 3.251795E+04, 3.274248E+04, 3.296783E+04, 3.319399E+04, 3.342096E+04,
    3.364875E+04, 3.387736E+04, 3.410677E+04, 3.433700E+04, 3.456804E+04, 3.479989E+04,
    3.503255E+04, 3.526602E+04, 3.550030E+04, 3.573539E+04, 3.597128E+04, 3.620797E+04,
    3.644547E+04, 3.668378E+04, 3.692289E+04, 3.716280E+04, 3.740351E+04, 3.764502E+04,
    3.788733E+04, 3.813044E+04, 3.837434E+04, 3.861904E+04, 3.886454E+04, 3.911083E+04,
    3.935791E+04, 3.960579E+04, 3.985445E+04, 4.010391E+04, 4.035416E+04, 4.060519E+04,
    4.085701E+04, 4.110962E+04, 4.136301E+04, 4.161719E+04, 4.187214E+04, 4.212788E+04,
    4.238440E+04, 4.264170E+04, 4.289977E+04, 4.315863E+04, 4.341825E+04, 4.367866E+04,
    4.393983E+04, 4.420178E+04, 4.446450E+04, 4.472798E+04, 4.499224E+04, 4.525726E+04,
    4.552305E+04, 4.578960E+04, 4.605692E+04, 4.632499E+04, 4.659383E+04, 4.686343E+04,
    4.713378E+04, 4.740489E+04, 4.767676E+04, 4.794938E+04, 4.822276E+04, 4.849688E+04,
    4.877176E+04, 4.904738E+04, 4.932375E+04, 4.960087E+04, 4.987873E+04, 5.015733E+04,
    5.043668E+04, 5.071676E+04, 5.099758E+04, 5.127915E+04, 5.156144E+04, 5.184447E+04,
    5.212824E+04, 5.241273E+04, 5.269796E+04, 5.298391E+04, 5.327059E+04, 5.355800E+04,
    5.384613E+04, 5.413498E+04, 5.442456E+04, 5.471485E+04,
])

#  ---------------------- M = 7, I = 3 ---------------------------
M = 7
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.092314E+01, 1.808102E+02, 3.594239E+02, 5.385443E+02, 7.178153E+02, 8.971617E+02,
    1.076558E+03, 1.255992E+03, 1.435461E+03, 1.614965E+03, 1.794512E+03, 1.974123E+03,
    2.153834E+03, 2.333703E+03, 2.513805E+03, 2.694239E+03, 2.875119E+03, 3.056572E+03,
    3.238738E+03, 3.421762E+03, 3.605792E+03, 3.790976E+03, 3.977461E+03, 4.165389E+03,
    4.354895E+03, 4.546108E+03, 4.739152E+03, 4.934142E+03, 5.131184E+03, 5.330380E+03,
    5.531821E+03, 5.735595E+03, 5.941780E+03, 6.150451E+03, 6.361674E+03, 6.575513E+03,
    6.792026E+03, 7.011265E+03, 7.233281E+03, 7.458117E+03, 7.685816E+03, 7.916417E+03,
    8.149955E+03, 8.386465E+03, 8.625976E+03, 8.868517E+03, 9.114116E+03, 9.362797E+03,
    9.614584E+03, 9.869498E+03, 1.012756E+04, 1.038879E+04, 1.065321E+04, 1.092083E+04,
    1.119167E+04, 1.146574E+04, 1.174307E+04, 1.202366E+04, 1.230753E+04, 1.259470E+04,
    1.288517E+04, 1.317895E+04, 1.347607E+04, 1.377654E+04, 1.408035E+04, 1.438754E+04,
    1.469810E+04, 1.501205E+04, 1.532940E+04, 1.565016E+04, 1.597435E+04, 1.630197E+04,
    1.663303E+04, 1.696755E+04, 1.730554E+04, 1.764700E+04, 1.799195E+04, 1.834041E+04,
    1.869237E+04, 1.904786E+04, 1.940687E+04, 1.976944E+04, 2.013556E+04, 2.050525E+04,
    2.087851E+04, 2.125537E+04, 2.163583E+04, 2.201990E+04, 2.240760E+04, 2.279894E+04,
    2.319392E+04, 2.359257E+04, 2.399490E+04, 2.440090E+04, 2.481061E+04, 2.522403E+04,
    2.564117E+04, 2.606205E+04, 2.648668E+04, 2.691506E+04, 2.734723E+04, 2.778317E+04,
    2.822292E+04, 2.866648E+04, 2.911386E+04, 2.956508E+04, 3.002015E+04, 3.047908E+04,
    3.094189E+04, 3.140859E+04, 3.187919E+04, 3.235370E+04, 3.283214E+04, 3.331452E+04,
    3.380086E+04, 3.429116E+04, 3.478544E+04, 3.528371E+04, 3.578598E+04, 3.629228E+04,
    3.680260E+04, 3.731697E+04, 3.783539E+04, 3.835788E+04, 3.888445E+04, 3.941511E+04,
    3.994988E+04, 4.048877E+04, 4.103179E+04, 4.157894E+04, 4.213026E+04, 4.268574E+04,
    4.324540E+04, 4.380925E+04, 4.437731E+04, 4.494957E+04, 4.552607E+04, 4.610680E+04,
    4.669179E+04, 4.728103E+04, 4.787455E+04, 4.847235E+04, 4.907444E+04, 4.968084E+04,
    5.029156E+04, 5.090661E+04, 5.152600E+04, 5.214974E+04, 5.277783E+04, 5.341030E+04,
    5.404715E+04, 5.468839E+04, 5.533403E+04, 5.598409E+04, 5.663857E+04, 5.729748E+04,
    5.796083E+04, 5.862863E+04, 5.930089E+04, 5.997762E+04, 6.065884E+04, 6.134454E+04,
    6.203474E+04, 6.272945E+04, 6.342867E+04, 6.413242E+04, 6.484070E+04, 6.555352E+04,
    6.627089E+04, 6.699282E+04, 6.771932E+04, 6.845039E+04, 6.918604E+04, 6.992629E+04,
    7.067113E+04, 7.142058E+04, 7.217464E+04, 7.293332E+04, 7.369663E+04, 7.446457E+04,
    7.523716E+04, 7.601439E+04, 7.679628E+04, 7.758283E+04, 7.837405E+04, 7.916995E+04,
    7.997052E+04, 8.077579E+04, 8.158575E+04, 8.240040E+04, 8.321976E+04, 8.404384E+04,
    8.487263E+04, 8.570614E+04, 8.654437E+04, 8.738735E+04, 8.823506E+04, 8.908751E+04,
    8.994471E+04, 9.080666E+04, 9.167337E+04, 9.254484E+04, 9.342108E+04, 9.430209E+04,
    9.518787E+04, 9.607843E+04, 9.697378E+04, 9.787391E+04, 9.877883E+04, 9.968855E+04,
    1.006031E+05, 1.015224E+05, 1.024465E+05, 1.033754E+05, 1.043092E+05, 1.052477E+05,
    1.061911E+05, 1.071393E+05, 1.080923E+05, 1.090502E+05, 1.100128E+05, 1.109803E+05,
    1.119527E+05, 1.129299E+05, 1.139119E+05, 1.148987E+05, 1.158904E+05, 1.168869E+05,
    1.178883E+05, 1.188945E+05, 1.199056E+05, 1.209215E+05, 1.219423E+05, 1.229679E+05,
    1.239984E+05, 1.250338E+05, 1.260739E+05, 1.271190E+05, 1.281689E+05, 1.292237E+05,
    1.302833E+05, 1.313477E+05, 1.324171E+05, 1.334913E+05, 1.345703E+05, 1.356543E+05,
    1.367430E+05, 1.378367E+05, 1.389352E+05, 1.400385E+05, 1.411467E+05, 1.422598E+05,
    1.433777E+05, 1.445005E+05, 1.456282E+05, 1.467607E+05, 1.478980E+05, 1.490402E+05,
    1.501873E+05, 1.513392E+05, 1.524960E+05, 1.536576E+05, 1.548240E+05, 1.559954E+05,
    1.571715E+05, 1.583525E+05, 1.595383E+05, 1.607290E+05, 1.619245E+05, 1.631249E+05,
    1.643301E+05, 1.655401E+05, 1.667549E+05, 1.679746E+05, 1.691991E+05, 1.704284E+05,
    1.716626E+05, 1.729015E+05, 1.741453E+05, 1.753939E+05, 1.766473E+05, 1.779055E+05,
    1.791685E+05, 1.804363E+05, 1.817088E+05, 1.829862E+05, 1.842684E+05, 1.855554E+05,
    1.868471E+05, 1.881436E+05, 1.894449E+05, 1.907510E+05, 1.920619E+05, 1.933775E+05,
    1.946978E+05, 1.960229E+05, 1.973528E+05, 1.986874E+05, 2.000268E+05, 2.013709E+05,
    2.027197E+05, 2.040733E+05, 2.054316E+05, 2.067946E+05, 2.081624E+05, 2.095348E+05,
    2.109120E+05, 2.122938E+05, 2.136804E+05, 2.150716E+05, 2.164676E+05, 2.178682E+05,
    2.192735E+05, 2.206834E+05, 2.220981E+05, 2.235174E+05, 2.249413E+05, 2.263699E+05,
    2.278031E+05, 2.292410E+05, 2.306835E+05, 2.321307E+05, 2.335825E+05, 2.350388E+05,
    2.364998E+05, 2.379654E+05, 2.394356E+05, 2.409104E+05, 2.423898E+05, 2.438737E+05,
    2.453622E+05, 2.468553E+05, 2.483530E+05, 2.498552E+05, 2.513619E+05, 2.528732E+05,
    2.543890E+05, 2.559094E+05, 2.574342E+05, 2.589636E+05, 2.604975E+05, 2.620359E+05,
    2.635788E+05, 2.651261E+05, 2.666780E+05, 2.682343E+05, 2.697950E+05, 2.713603E+05,
    2.729299E+05, 2.745041E+05, 2.760826E+05, 2.776656E+05, 2.792530E+05, 2.808448E+05,
    2.824410E+05, 2.840416E+05, 2.856466E+05, 2.872560E+05, 2.888698E+05, 2.904879E+05,
    2.921104E+05, 2.937372E+05, 2.953684E+05, 2.970038E+05, 2.986437E+05, 3.002878E+05,
    3.019362E+05, 3.035890E+05, 3.052460E+05, 3.069073E+05, 3.085729E+05, 3.102428E+05,
    3.119169E+05, 3.135953E+05, 3.152779E+05, 3.169647E+05,
])

#  ---------------------- M = 7, I = 3 ---------------------------
M = 7
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.092314E+01, 1.808102E+02, 3.594239E+02, 5.385443E+02, 7.178153E+02, 8.971617E+02,
    1.076558E+03, 1.255992E+03, 1.435461E+03, 1.614965E+03, 1.794512E+03, 1.974123E+03,
    2.153834E+03, 2.333703E+03, 2.513805E+03, 2.694239E+03, 2.875119E+03, 3.056572E+03,
    3.238738E+03, 3.421762E+03, 3.605792E+03, 3.790976E+03, 3.977461E+03, 4.165389E+03,
    4.354895E+03, 4.546108E+03, 4.739152E+03, 4.934142E+03, 5.131184E+03, 5.330380E+03,
    5.531821E+03, 5.735595E+03, 5.941780E+03, 6.150451E+03, 6.361674E+03, 6.575513E+03,
    6.792026E+03, 7.011265E+03, 7.233281E+03, 7.458117E+03, 7.685816E+03, 7.916417E+03,
    8.149955E+03, 8.386465E+03, 8.625976E+03, 8.868517E+03, 9.114116E+03, 9.362797E+03,
    9.614584E+03, 9.869498E+03, 1.012756E+04, 1.038879E+04, 1.065321E+04, 1.092083E+04,
    1.119167E+04, 1.146574E+04, 1.174307E+04, 1.202366E+04, 1.230753E+04, 1.259470E+04,
    1.288517E+04, 1.317895E+04, 1.347607E+04, 1.377654E+04, 1.408035E+04, 1.438754E+04,
    1.469810E+04, 1.501205E+04, 1.532940E+04, 1.565016E+04, 1.597435E+04, 1.630197E+04,
    1.663303E+04, 1.696755E+04, 1.730554E+04, 1.764700E+04, 1.799195E+04, 1.834041E+04,
    1.869237E+04, 1.904786E+04, 1.940687E+04, 1.976944E+04, 2.013556E+04, 2.050525E+04,
    2.087851E+04, 2.125537E+04, 2.163583E+04, 2.201990E+04, 2.240760E+04, 2.279894E+04,
    2.319392E+04, 2.359257E+04, 2.399490E+04, 2.440090E+04, 2.481061E+04, 2.522403E+04,
    2.564117E+04, 2.606205E+04, 2.648668E+04, 2.691506E+04, 2.734723E+04, 2.778317E+04,
    2.822292E+04, 2.866648E+04, 2.911386E+04, 2.956508E+04, 3.002015E+04, 3.047908E+04,
    3.094189E+04, 3.140859E+04, 3.187919E+04, 3.235370E+04, 3.283214E+04, 3.331452E+04,
    3.380086E+04, 3.429116E+04, 3.478544E+04, 3.528371E+04, 3.578598E+04, 3.629228E+04,
    3.680260E+04, 3.731697E+04, 3.783539E+04, 3.835788E+04, 3.888445E+04, 3.941511E+04,
    3.994988E+04, 4.048877E+04, 4.103179E+04, 4.157894E+04, 4.213026E+04, 4.268574E+04,
    4.324540E+04, 4.380925E+04, 4.437731E+04, 4.494957E+04, 4.552607E+04, 4.610680E+04,
    4.669179E+04, 4.728103E+04, 4.787455E+04, 4.847235E+04, 4.907444E+04, 4.968084E+04,
    5.029156E+04, 5.090661E+04, 5.152600E+04, 5.214974E+04, 5.277783E+04, 5.341030E+04,
    5.404715E+04, 5.468839E+04, 5.533403E+04, 5.598409E+04, 5.663857E+04, 5.729748E+04,
    5.796083E+04, 5.862863E+04, 5.930089E+04, 5.997762E+04, 6.065884E+04, 6.134454E+04,
    6.203474E+04, 6.272945E+04, 6.342867E+04, 6.413242E+04, 6.484070E+04, 6.555352E+04,
    6.627089E+04, 6.699282E+04, 6.771932E+04, 6.845039E+04, 6.918604E+04, 6.992629E+04,
    7.067113E+04, 7.142058E+04, 7.217464E+04, 7.293332E+04, 7.369663E+04, 7.446457E+04,
    7.523716E+04, 7.601439E+04, 7.679628E+04, 7.758283E+04, 7.837405E+04, 7.916995E+04,
    7.997052E+04, 8.077579E+04, 8.158575E+04, 8.240040E+04, 8.321976E+04, 8.404384E+04,
    8.487263E+04, 8.570614E+04, 8.654437E+04, 8.738735E+04, 8.823506E+04, 8.908751E+04,
    8.994471E+04, 9.080666E+04, 9.167337E+04, 9.254484E+04, 9.342108E+04, 9.430209E+04,
    9.518787E+04, 9.607843E+04, 9.697378E+04, 9.787391E+04, 9.877883E+04, 9.968855E+04,
    1.006031E+05, 1.015224E+05, 1.024465E+05, 1.033754E+05, 1.043092E+05, 1.052477E+05,
    1.061911E+05, 1.071393E+05, 1.080923E+05, 1.090502E+05, 1.100128E+05, 1.109803E+05,
    1.119527E+05, 1.129299E+05, 1.139119E+05, 1.148987E+05, 1.158904E+05, 1.168869E+05,
    1.178883E+05, 1.188945E+05, 1.199056E+05, 1.209215E+05, 1.219423E+05, 1.229679E+05,
    1.239984E+05, 1.250338E+05, 1.260739E+05, 1.271190E+05, 1.281689E+05, 1.292237E+05,
    1.302833E+05, 1.313477E+05, 1.324171E+05, 1.334913E+05, 1.345703E+05, 1.356543E+05,
    1.367430E+05, 1.378367E+05, 1.389352E+05, 1.400385E+05, 1.411467E+05, 1.422598E+05,
    1.433777E+05, 1.445005E+05, 1.456282E+05, 1.467607E+05, 1.478980E+05, 1.490402E+05,
    1.501873E+05, 1.513392E+05, 1.524960E+05, 1.536576E+05, 1.548240E+05, 1.559954E+05,
    1.571715E+05, 1.583525E+05, 1.595383E+05, 1.607290E+05, 1.619245E+05, 1.631249E+05,
    1.643301E+05, 1.655401E+05, 1.667549E+05, 1.679746E+05, 1.691991E+05, 1.704284E+05,
    1.716626E+05, 1.729015E+05, 1.741453E+05, 1.753939E+05, 1.766473E+05, 1.779055E+05,
    1.791685E+05, 1.804363E+05, 1.817088E+05, 1.829862E+05, 1.842684E+05, 1.855554E+05,
    1.868471E+05, 1.881436E+05, 1.894449E+05, 1.907510E+05, 1.920619E+05, 1.933775E+05,
    1.946978E+05, 1.960229E+05, 1.973528E+05, 1.986874E+05, 2.000268E+05, 2.013709E+05,
    2.027197E+05, 2.040733E+05, 2.054316E+05, 2.067946E+05, 2.081624E+05, 2.095348E+05,
    2.109120E+05, 2.122938E+05, 2.136804E+05, 2.150716E+05, 2.164676E+05, 2.178682E+05,
    2.192735E+05, 2.206834E+05, 2.220981E+05, 2.235174E+05, 2.249413E+05, 2.263699E+05,
    2.278031E+05, 2.292410E+05, 2.306835E+05, 2.321307E+05, 2.335825E+05, 2.350388E+05,
    2.364998E+05, 2.379654E+05, 2.394356E+05, 2.409104E+05, 2.423898E+05, 2.438737E+05,
    2.453622E+05, 2.468553E+05, 2.483530E+05, 2.498552E+05, 2.513619E+05, 2.528732E+05,
    2.543890E+05, 2.559094E+05, 2.574342E+05, 2.589636E+05, 2.604975E+05, 2.620359E+05,
    2.635788E+05, 2.651261E+05, 2.666780E+05, 2.682343E+05, 2.697950E+05, 2.713603E+05,
    2.729299E+05, 2.745041E+05, 2.760826E+05, 2.776656E+05, 2.792530E+05, 2.808448E+05,
    2.824410E+05, 2.840416E+05, 2.856466E+05, 2.872560E+05, 2.888698E+05, 2.904879E+05,
    2.921104E+05, 2.937372E+05, 2.953684E+05, 2.970038E+05, 2.986437E+05, 3.002878E+05,
    3.019362E+05, 3.035890E+05, 3.052460E+05, 3.069073E+05, 3.085729E+05, 3.102428E+05,
    3.119169E+05, 3.135953E+05, 3.152779E+05, 3.169647E+05,
])

#  ---------------------- M = 7, I = 4 ---------------------------
M = 7
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.270969E+00, 1.687773E+01, 3.312258E+01, 4.941808E+01, 6.572814E+01, 8.204535E+01,
    9.836721E+01, 1.146927E+02, 1.310214E+02, 1.473535E+02, 1.636904E+02, 1.800346E+02,
    1.963907E+02, 2.127654E+02, 2.291674E+02, 2.456073E+02, 2.620976E+02, 2.786516E+02,
    2.952836E+02, 3.120083E+02, 3.288405E+02, 3.457946E+02, 3.628847E+02, 3.801244E+02,
    3.975266E+02, 4.151032E+02, 4.328657E+02, 4.508247E+02, 4.689898E+02, 4.873702E+02,
    5.059742E+02, 5.248095E+02, 5.438832E+02, 5.632018E+02, 5.827712E+02, 6.025970E+02,
    6.226842E+02, 6.430374E+02, 6.636609E+02, 6.845586E+02, 7.057342E+02, 7.271909E+02,
    7.489318E+02, 7.709599E+02, 7.932777E+02, 8.158878E+02, 8.387923E+02, 8.619935E+02,
    8.854934E+02, 9.092938E+02, 9.333964E+02, 9.578031E+02, 9.825153E+02, 1.007535E+03,
    1.032862E+03, 1.058500E+03, 1.084448E+03, 1.110709E+03, 1.137284E+03, 1.164173E+03,
    1.191378E+03, 1.218900E+03, 1.246741E+03, 1.274900E+03, 1.303379E+03, 1.332180E+03,
    1.361303E+03, 1.390749E+03, 1.420519E+03, 1.450614E+03, 1.481036E+03, 1.511784E+03,
    1.542861E+03, 1.574267E+03, 1.606003E+03, 1.638069E+03, 1.670468E+03, 1.703200E+03,
    1.736266E+03, 1.769667E+03, 1.803403E+03, 1.837477E+03, 1.871889E+03, 1.906639E+03,
    1.941730E+03, 1.977162E+03, 2.012936E+03, 2.049053E+03, 2.085515E+03, 2.122321E+03,
    2.159474E+03, 2.196974E+03, 2.234823E+03, 2.273021E+03, 2.311570E+03, 2.350471E+03,
    2.389724E+03, 2.429331E+03, 2.469293E+03, 2.509612E+03, 2.550287E+03, 2.591320E+03,
    2.632713E+03, 2.674467E+03, 2.716582E+03, 2.759059E+03, 2.801900E+03, 2.845107E+03,
    2.888679E+03, 2.932618E+03, 2.976925E+03, 3.021602E+03, 3.066649E+03, 3.112067E+03,
    3.157858E+03, 3.204022E+03, 3.250562E+03, 3.297476E+03, 3.344768E+03, 3.392438E+03,
    3.440486E+03, 3.488915E+03, 3.537724E+03, 3.586916E+03, 3.636491E+03, 3.686450E+03,
    3.736794E+03, 3.787524E+03, 3.838641E+03, 3.890147E+03, 3.942042E+03, 3.994327E+03,
    4.047003E+03, 4.100071E+03, 4.153533E+03, 4.207388E+03, 4.261638E+03, 4.316284E+03,
    4.371327E+03, 4.426768E+03, 4.482608E+03, 4.538846E+03, 4.595486E+03, 4.652526E+03,
    4.709969E+03, 4.767815E+03, 4.826064E+03, 4.884718E+03, 4.943778E+03, 5.003244E+03,
    5.063117E+03, 5.123398E+03, 5.184088E+03, 5.245187E+03, 5.306697E+03, 5.368617E+03,
    5.430949E+03, 5.493693E+03, 5.556851E+03, 5.620423E+03, 5.684409E+03, 5.748810E+03,
    5.813627E+03, 5.878861E+03, 5.944512E+03, 6.010580E+03, 6.077068E+03, 6.143974E+03,
    6.211300E+03, 6.279047E+03, 6.347214E+03, 6.415803E+03, 6.484814E+03, 6.554247E+03,
    6.624104E+03, 6.694384E+03, 6.765089E+03, 6.836218E+03, 6.907772E+03, 6.979752E+03,
    7.052159E+03, 7.124992E+03, 7.198252E+03, 7.271939E+03, 7.346055E+03, 7.420599E+03,
    7.495571E+03, 7.570973E+03, 7.646805E+03, 7.723066E+03, 7.799758E+03, 7.876881E+03,
    7.954435E+03, 8.032420E+03, 8.110837E+03, 8.189686E+03, 8.268968E+03, 8.348682E+03,
    8.428830E+03, 8.509410E+03, 8.590424E+03, 8.671872E+03, 8.753754E+03, 8.836071E+03,
    8.918822E+03, 9.002007E+03, 9.085628E+03, 9.169684E+03, 9.254175E+03, 9.339102E+03,
    9.424465E+03, 9.510263E+03, 9.596498E+03, 9.683169E+03, 9.770276E+03, 9.857820E+03,
    9.945801E+03, 1.003422E+04, 1.012307E+04, 1.021236E+04, 1.030209E+04, 1.039226E+04,
    1.048286E+04, 1.057390E+04, 1.066537E+04, 1.075729E+04, 1.084964E+04, 1.094243E+04,
    1.103566E+04, 1.112932E+04, 1.122342E+04, 1.131796E+04, 1.141293E+04, 1.150835E+04,
    1.160420E+04, 1.170048E+04, 1.179721E+04, 1.189437E+04, 1.199197E+04, 1.209001E+04,
    1.218848E+04, 1.228739E+04, 1.238673E+04, 1.248652E+04, 1.258674E+04, 1.268739E+04,
    1.278848E+04, 1.289001E+04, 1.299198E+04, 1.309438E+04, 1.319721E+04, 1.330048E+04,
    1.340419E+04, 1.350833E+04, 1.361290E+04, 1.371791E+04, 1.382336E+04, 1.392924E+04,
    1.403555E+04, 1.414230E+04, 1.424948E+04, 1.435709E+04, 1.446514E+04, 1.457362E+04,
    1.468253E+04, 1.479187E+04, 1.490165E+04, 1.501185E+04, 1.512249E+04, 1.523356E+04,
    1.534506E+04, 1.545699E+04, 1.556935E+04, 1.568214E+04, 1.579536E+04, 1.590901E+04,
    1.602309E+04, 1.613759E+04, 1.625253E+04, 1.636789E+04, 1.648367E+04, 1.659989E+04,
    1.671653E+04, 1.683360E+04, 1.695109E+04, 1.706900E+04, 1.718735E+04, 1.730611E+04,
    1.742530E+04, 1.754492E+04, 1.766495E+04, 1.778541E+04, 1.790629E+04, 1.802759E+04,
    1.814932E+04, 1.827146E+04, 1.839402E+04, 1.851701E+04, 1.864041E+04, 1.876423E+04,
    1.888847E+04, 1.901313E+04, 1.913820E+04, 1.926369E+04, 1.938959E+04, 1.951592E+04,
    1.964265E+04, 1.976980E+04, 1.989737E+04, 2.002534E+04, 2.015373E+04, 2.028253E+04,
    2.041175E+04, 2.054137E+04, 2.067141E+04, 2.080185E+04, 2.093270E+04, 2.106396E+04,
    2.119563E+04, 2.132771E+04, 2.146019E+04, 2.159308E+04, 2.172638E+04, 2.186007E+04,
    2.199418E+04, 2.212868E+04, 2.226359E+04, 2.239890E+04, 2.253461E+04, 2.267073E+04,
    2.280724E+04, 2.294415E+04, 2.308146E+04, 2.321917E+04, 2.335728E+04, 2.349578E+04,
    2.363468E+04, 2.377397E+04, 2.391366E+04, 2.405374E+04, 2.419421E+04, 2.433508E+04,
    2.447634E+04, 2.461799E+04, 2.476002E+04, 2.490245E+04, 2.504527E+04, 2.518847E+04,
    2.533206E+04, 2.547604E+04, 2.562040E+04, 2.576515E+04, 2.591028E+04, 2.605580E+04,
    2.620169E+04, 2.634797E+04, 2.649463E+04, 2.664167E+04, 2.678908E+04, 2.693688E+04,
    2.708505E+04, 2.723360E+04, 2.738253E+04, 2.753183E+04, 2.768151E+04, 2.783155E+04,
    2.798198E+04, 2.813277E+04, 2.828393E+04, 2.843547E+04, 2.858737E+04, 2.873964E+04,
    2.889228E+04, 2.904528E+04, 2.919865E+04, 2.935239E+04,
])

#  ---------------------- M = 7, I = 5 ---------------------------
M = 7
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.157469E+01, 1.910172E+02, 3.804688E+02, 5.704593E+02, 7.606097E+02, 9.508401E+02,
    1.141123E+03, 1.331447E+03, 1.521808E+03, 1.712208E+03, 1.902659E+03, 2.093190E+03,
    2.283847E+03, 2.474702E+03, 2.665851E+03, 2.857410E+03, 3.049515E+03, 3.242316E+03,
    3.435973E+03, 3.630652E+03, 3.826519E+03, 4.023741E+03, 4.222477E+03, 4.422884E+03,
    4.625110E+03, 4.829294E+03, 5.035569E+03, 5.244058E+03, 5.454873E+03, 5.668122E+03,
    5.883902E+03, 6.102304E+03, 6.323410E+03, 6.547298E+03, 6.774037E+03, 7.003694E+03,
    7.236326E+03, 7.471990E+03, 7.710735E+03, 7.952609E+03, 8.197654E+03, 8.445909E+03,
    8.697412E+03, 8.952197E+03, 9.210295E+03, 9.471735E+03, 9.736545E+03, 1.000475E+04,
    1.027637E+04, 1.055144E+04, 1.082997E+04, 1.111198E+04, 1.139749E+04, 1.168652E+04,
    1.197908E+04, 1.227520E+04, 1.257488E+04, 1.287815E+04, 1.318501E+04, 1.349548E+04,
    1.380958E+04, 1.412731E+04, 1.444869E+04, 1.477374E+04, 1.510246E+04, 1.543486E+04,
    1.577096E+04, 1.611078E+04, 1.645431E+04, 1.680158E+04, 1.715259E+04, 1.750736E+04,
    1.786589E+04, 1.822821E+04, 1.859431E+04, 1.896422E+04, 1.933794E+04, 1.971548E+04,
    2.009687E+04, 2.048210E+04, 2.087119E+04, 2.126415E+04, 2.166100E+04, 2.206174E+04,
    2.246639E+04, 2.287496E+04, 2.328746E+04, 2.370391E+04, 2.412431E+04, 2.454868E+04,
    2.497703E+04, 2.540938E+04, 2.584574E+04, 2.628611E+04, 2.673051E+04, 2.717896E+04,
    2.763147E+04, 2.808804E+04, 2.854870E+04, 2.901346E+04, 2.948232E+04, 2.995531E+04,
    3.043243E+04, 3.091370E+04, 3.139913E+04, 3.188873E+04, 3.238252E+04, 3.288051E+04,
    3.338272E+04, 3.388915E+04, 3.439982E+04, 3.491474E+04, 3.543392E+04, 3.595739E+04,
    3.648514E+04, 3.701720E+04, 3.755358E+04, 3.809428E+04, 3.863933E+04, 3.918873E+04,
    3.974250E+04, 4.030065E+04, 4.086319E+04, 4.143014E+04, 4.200151E+04, 4.257730E+04,
    4.315754E+04, 4.374223E+04, 4.433139E+04, 4.492502E+04, 4.552315E+04, 4.612578E+04,
    4.673292E+04, 4.734459E+04, 4.796079E+04, 4.858154E+04, 4.920686E+04, 4.983674E+04,
    5.047121E+04, 5.111027E+04, 5.175393E+04, 5.240221E+04, 5.305511E+04, 5.371265E+04,
    5.437484E+04, 5.504169E+04, 5.571320E+04, 5.638939E+04, 5.707027E+04, 5.775584E+04,
    5.844613E+04, 5.914113E+04, 5.984085E+04, 6.054532E+04, 6.125452E+04, 6.196849E+04,
    6.268721E+04, 6.341071E+04, 6.413899E+04, 6.487206E+04, 6.560993E+04, 6.635261E+04,
    6.710010E+04, 6.785242E+04, 6.860957E+04, 6.937155E+04, 7.013839E+04, 7.091008E+04,
    7.168663E+04, 7.246805E+04, 7.325435E+04, 7.404554E+04, 7.484161E+04, 7.564259E+04,
    7.644847E+04, 7.725926E+04, 7.807497E+04, 7.889561E+04, 7.972118E+04, 8.055169E+04,
    8.138714E+04, 8.222754E+04, 8.307289E+04, 8.392321E+04, 8.477849E+04, 8.563874E+04,
    8.650398E+04, 8.737419E+04, 8.824940E+04, 8.912959E+04, 9.001479E+04, 9.090498E+04,
    9.180018E+04, 9.270040E+04, 9.360563E+04, 9.451588E+04, 9.543115E+04, 9.635146E+04,
    9.727679E+04, 9.820716E+04, 9.914257E+04, 1.000830E+05, 1.010285E+05, 1.019791E+05,
    1.029347E+05, 1.038953E+05, 1.048611E+05, 1.058318E+05, 1.068077E+05, 1.077886E+05,
    1.087746E+05, 1.097656E+05, 1.107617E+05, 1.117630E+05, 1.127692E+05, 1.137806E+05,
    1.147971E+05, 1.158186E+05, 1.168452E+05, 1.178769E+05, 1.189137E+05, 1.199556E+05,
    1.210026E+05, 1.220546E+05, 1.231118E+05, 1.241741E+05, 1.252414E+05, 1.263139E+05,
    1.273914E+05, 1.284741E+05, 1.295618E+05, 1.306547E+05, 1.317526E+05, 1.328556E+05,
    1.339638E+05, 1.350770E+05, 1.361953E+05, 1.373188E+05, 1.384473E+05, 1.395809E+05,
    1.407197E+05, 1.418635E+05, 1.430124E+05, 1.441664E+05, 1.453255E+05, 1.464897E+05,
    1.476590E+05, 1.488334E+05, 1.500129E+05, 1.511974E+05, 1.523871E+05, 1.535818E+05,
    1.547816E+05, 1.559865E+05, 1.571964E+05, 1.584115E+05, 1.596316E+05, 1.608568E+05,
    1.620871E+05, 1.633224E+05, 1.645628E+05, 1.658083E+05, 1.670588E+05, 1.683144E+05,
    1.695750E+05, 1.708407E+05, 1.721114E+05, 1.733872E+05, 1.746680E+05, 1.759539E+05,
    1.772448E+05, 1.785407E+05, 1.798417E+05, 1.811477E+05, 1.824587E+05, 1.837748E+05,
    1.850958E+05, 1.864219E+05, 1.877530E+05, 1.890890E+05, 1.904301E+05, 1.917762E+05,
    1.931273E+05, 1.944833E+05, 1.958443E+05, 1.972104E+05, 1.985814E+05, 1.999573E+05,
    2.013382E+05, 2.027241E+05, 2.041150E+05, 2.055107E+05, 2.069115E+05, 2.083171E+05,
    2.097278E+05, 2.111433E+05, 2.125638E+05, 2.139891E+05, 2.154194E+05, 2.168546E+05,
    2.182947E+05, 2.197397E+05, 2.211896E+05, 2.226444E+05, 2.241041E+05, 2.255686E+05,
    2.270380E+05, 2.285122E+05, 2.299914E+05, 2.314753E+05, 2.329641E+05, 2.344578E+05,
    2.359562E+05, 2.374595E+05, 2.389676E+05, 2.404806E+05, 2.419983E+05, 2.435208E+05,
    2.450481E+05, 2.465802E+05, 2.481171E+05, 2.496587E+05, 2.512051E+05, 2.527563E+05,
    2.543122E+05, 2.558729E+05, 2.574382E+05, 2.590083E+05, 2.605832E+05, 2.621627E+05,
    2.637469E+05, 2.653359E+05, 2.669295E+05, 2.685278E+05, 2.701308E+05, 2.717384E+05,
    2.733507E+05, 2.749677E+05, 2.765893E+05, 2.782155E+05, 2.798463E+05, 2.814818E+05,
    2.831219E+05, 2.847665E+05, 2.864158E+05, 2.880697E+05, 2.897281E+05, 2.913911E+05,
    2.930586E+05, 2.947307E+05, 2.964074E+05, 2.980885E+05, 2.997742E+05, 3.014644E+05,
    3.031591E+05, 3.048584E+05, 3.065620E+05, 3.082702E+05, 3.099829E+05, 3.117000E+05,
    3.134215E+05, 3.151475E+05, 3.168780E+05, 3.186128E+05, 3.203521E+05, 3.220958E+05,
    3.238439E+05, 3.255963E+05, 3.273532E+05, 3.291144E+05, 3.308799E+05, 3.326499E+05,
    3.344241E+05, 3.362027E+05, 3.379856E+05, 3.397728E+05,
])

#  ---------------------- M = 7, I = 5 ---------------------------
M = 7
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.157469E+01, 1.910172E+02, 3.804688E+02, 5.704593E+02, 7.606097E+02, 9.508401E+02,
    1.141123E+03, 1.331447E+03, 1.521808E+03, 1.712208E+03, 1.902659E+03, 2.093190E+03,
    2.283847E+03, 2.474702E+03, 2.665851E+03, 2.857410E+03, 3.049515E+03, 3.242316E+03,
    3.435973E+03, 3.630652E+03, 3.826519E+03, 4.023741E+03, 4.222477E+03, 4.422884E+03,
    4.625110E+03, 4.829294E+03, 5.035569E+03, 5.244058E+03, 5.454873E+03, 5.668122E+03,
    5.883902E+03, 6.102304E+03, 6.323410E+03, 6.547298E+03, 6.774037E+03, 7.003694E+03,
    7.236326E+03, 7.471990E+03, 7.710735E+03, 7.952609E+03, 8.197654E+03, 8.445909E+03,
    8.697412E+03, 8.952197E+03, 9.210295E+03, 9.471735E+03, 9.736545E+03, 1.000475E+04,
    1.027637E+04, 1.055144E+04, 1.082997E+04, 1.111198E+04, 1.139749E+04, 1.168652E+04,
    1.197908E+04, 1.227520E+04, 1.257488E+04, 1.287815E+04, 1.318501E+04, 1.349548E+04,
    1.380958E+04, 1.412731E+04, 1.444869E+04, 1.477374E+04, 1.510246E+04, 1.543486E+04,
    1.577096E+04, 1.611078E+04, 1.645431E+04, 1.680158E+04, 1.715259E+04, 1.750736E+04,
    1.786589E+04, 1.822821E+04, 1.859431E+04, 1.896422E+04, 1.933794E+04, 1.971548E+04,
    2.009687E+04, 2.048210E+04, 2.087119E+04, 2.126415E+04, 2.166100E+04, 2.206174E+04,
    2.246639E+04, 2.287496E+04, 2.328746E+04, 2.370391E+04, 2.412431E+04, 2.454868E+04,
    2.497703E+04, 2.540938E+04, 2.584574E+04, 2.628611E+04, 2.673051E+04, 2.717896E+04,
    2.763147E+04, 2.808804E+04, 2.854870E+04, 2.901346E+04, 2.948232E+04, 2.995531E+04,
    3.043243E+04, 3.091370E+04, 3.139913E+04, 3.188873E+04, 3.238252E+04, 3.288051E+04,
    3.338272E+04, 3.388915E+04, 3.439982E+04, 3.491474E+04, 3.543392E+04, 3.595739E+04,
    3.648514E+04, 3.701720E+04, 3.755358E+04, 3.809428E+04, 3.863933E+04, 3.918873E+04,
    3.974250E+04, 4.030065E+04, 4.086319E+04, 4.143014E+04, 4.200151E+04, 4.257730E+04,
    4.315754E+04, 4.374223E+04, 4.433139E+04, 4.492502E+04, 4.552315E+04, 4.612578E+04,
    4.673292E+04, 4.734459E+04, 4.796079E+04, 4.858154E+04, 4.920686E+04, 4.983674E+04,
    5.047121E+04, 5.111027E+04, 5.175393E+04, 5.240221E+04, 5.305511E+04, 5.371265E+04,
    5.437484E+04, 5.504169E+04, 5.571320E+04, 5.638939E+04, 5.707027E+04, 5.775584E+04,
    5.844613E+04, 5.914113E+04, 5.984085E+04, 6.054532E+04, 6.125452E+04, 6.196849E+04,
    6.268721E+04, 6.341071E+04, 6.413899E+04, 6.487206E+04, 6.560993E+04, 6.635261E+04,
    6.710010E+04, 6.785242E+04, 6.860957E+04, 6.937155E+04, 7.013839E+04, 7.091008E+04,
    7.168663E+04, 7.246805E+04, 7.325435E+04, 7.404554E+04, 7.484161E+04, 7.564259E+04,
    7.644847E+04, 7.725926E+04, 7.807497E+04, 7.889561E+04, 7.972118E+04, 8.055169E+04,
    8.138714E+04, 8.222754E+04, 8.307289E+04, 8.392321E+04, 8.477849E+04, 8.563874E+04,
    8.650398E+04, 8.737419E+04, 8.824940E+04, 8.912959E+04, 9.001479E+04, 9.090498E+04,
    9.180018E+04, 9.270040E+04, 9.360563E+04, 9.451588E+04, 9.543115E+04, 9.635146E+04,
    9.727679E+04, 9.820716E+04, 9.914257E+04, 1.000830E+05, 1.010285E+05, 1.019791E+05,
    1.029347E+05, 1.038953E+05, 1.048611E+05, 1.058318E+05, 1.068077E+05, 1.077886E+05,
    1.087746E+05, 1.097656E+05, 1.107617E+05, 1.117630E+05, 1.127692E+05, 1.137806E+05,
    1.147971E+05, 1.158186E+05, 1.168452E+05, 1.178769E+05, 1.189137E+05, 1.199556E+05,
    1.210026E+05, 1.220546E+05, 1.231118E+05, 1.241741E+05, 1.252414E+05, 1.263139E+05,
    1.273914E+05, 1.284741E+05, 1.295618E+05, 1.306547E+05, 1.317526E+05, 1.328556E+05,
    1.339638E+05, 1.350770E+05, 1.361953E+05, 1.373188E+05, 1.384473E+05, 1.395809E+05,
    1.407197E+05, 1.418635E+05, 1.430124E+05, 1.441664E+05, 1.453255E+05, 1.464897E+05,
    1.476590E+05, 1.488334E+05, 1.500129E+05, 1.511974E+05, 1.523871E+05, 1.535818E+05,
    1.547816E+05, 1.559865E+05, 1.571964E+05, 1.584115E+05, 1.596316E+05, 1.608568E+05,
    1.620871E+05, 1.633224E+05, 1.645628E+05, 1.658083E+05, 1.670588E+05, 1.683144E+05,
    1.695750E+05, 1.708407E+05, 1.721114E+05, 1.733872E+05, 1.746680E+05, 1.759539E+05,
    1.772448E+05, 1.785407E+05, 1.798417E+05, 1.811477E+05, 1.824587E+05, 1.837748E+05,
    1.850958E+05, 1.864219E+05, 1.877530E+05, 1.890890E+05, 1.904301E+05, 1.917762E+05,
    1.931273E+05, 1.944833E+05, 1.958443E+05, 1.972104E+05, 1.985814E+05, 1.999573E+05,
    2.013382E+05, 2.027241E+05, 2.041150E+05, 2.055107E+05, 2.069115E+05, 2.083171E+05,
    2.097278E+05, 2.111433E+05, 2.125638E+05, 2.139891E+05, 2.154194E+05, 2.168546E+05,
    2.182947E+05, 2.197397E+05, 2.211896E+05, 2.226444E+05, 2.241041E+05, 2.255686E+05,
    2.270380E+05, 2.285122E+05, 2.299914E+05, 2.314753E+05, 2.329641E+05, 2.344578E+05,
    2.359562E+05, 2.374595E+05, 2.389676E+05, 2.404806E+05, 2.419983E+05, 2.435208E+05,
    2.450481E+05, 2.465802E+05, 2.481171E+05, 2.496587E+05, 2.512051E+05, 2.527563E+05,
    2.543122E+05, 2.558729E+05, 2.574382E+05, 2.590083E+05, 2.605832E+05, 2.621627E+05,
    2.637469E+05, 2.653359E+05, 2.669295E+05, 2.685278E+05, 2.701308E+05, 2.717384E+05,
    2.733507E+05, 2.749677E+05, 2.765893E+05, 2.782155E+05, 2.798463E+05, 2.814818E+05,
    2.831219E+05, 2.847665E+05, 2.864158E+05, 2.880697E+05, 2.897281E+05, 2.913911E+05,
    2.930586E+05, 2.947307E+05, 2.964074E+05, 2.980885E+05, 2.997742E+05, 3.014644E+05,
    3.031591E+05, 3.048584E+05, 3.065620E+05, 3.082702E+05, 3.099829E+05, 3.117000E+05,
    3.134215E+05, 3.151475E+05, 3.168780E+05, 3.186128E+05, 3.203521E+05, 3.220958E+05,
    3.238439E+05, 3.255963E+05, 3.273532E+05, 3.291144E+05, 3.308799E+05, 3.326499E+05,
    3.344241E+05, 3.362027E+05, 3.379856E+05, 3.397728E+05,
])

#  ---------------------- M = 7, I = 6 ---------------------------
M = 7
I = 6
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[5]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    7.197400E+01, 5.586187E+02, 1.111229E+03, 1.665410E+03, 2.220057E+03, 2.774938E+03,
    3.329973E+03, 3.885127E+03, 4.440388E+03, 4.995759E+03, 5.551272E+03, 6.106999E+03,
    6.663066E+03, 7.219666E+03, 7.777057E+03, 8.335560E+03, 8.895550E+03, 9.457446E+03,
    1.002170E+04, 1.058877E+04, 1.115914E+04, 1.173327E+04, 1.231163E+04, 1.289467E+04,
    1.348281E+04, 1.407645E+04, 1.467598E+04, 1.528176E+04, 1.589412E+04, 1.651337E+04,
    1.713979E+04, 1.777365E+04, 1.841520E+04, 1.906465E+04, 1.972223E+04, 2.038811E+04,
    2.106247E+04, 2.174549E+04, 2.243730E+04, 2.313805E+04, 2.384786E+04, 2.456684E+04,
    2.529512E+04, 2.603279E+04, 2.677994E+04, 2.753666E+04, 2.830303E+04, 2.907913E+04,
    2.986503E+04, 3.066080E+04, 3.146649E+04, 3.228216E+04, 3.310788E+04, 3.394370E+04,
    3.478965E+04, 3.564581E+04, 3.651220E+04, 3.738887E+04, 3.827587E+04, 3.917323E+04,
    4.008100E+04, 4.099922E+04, 4.192792E+04, 4.286713E+04, 4.381690E+04, 4.477727E+04,
    4.574825E+04, 4.672990E+04, 4.772225E+04, 4.872532E+04, 4.973916E+04, 5.076379E+04,
    5.179925E+04, 5.284558E+04, 5.390280E+04, 5.497096E+04, 5.605008E+04, 5.714021E+04,
    5.824136E+04, 5.935359E+04, 6.047692E+04, 6.161139E+04, 6.275703E+04, 6.391387E+04,
    6.508197E+04, 6.626134E+04, 6.745202E+04, 6.865406E+04, 6.986749E+04, 7.109234E+04,
    7.232865E+04, 7.357646E+04, 7.483581E+04, 7.610672E+04, 7.738925E+04, 7.868343E+04,
    7.998930E+04, 8.130689E+04, 8.263624E+04, 8.397740E+04, 8.533040E+04, 8.669528E+04,
    8.807207E+04, 8.946082E+04, 9.086157E+04, 9.227436E+04, 9.369922E+04, 9.513619E+04,
    9.658532E+04, 9.804664E+04, 9.952019E+04, 1.010060E+05, 1.025041E+05, 1.040146E+05,
    1.055375E+05, 1.070728E+05, 1.086206E+05, 1.101808E+05, 1.117536E+05, 1.133390E+05,
    1.149371E+05, 1.165477E+05, 1.181711E+05, 1.198072E+05, 1.214561E+05, 1.231178E+05,
    1.247924E+05, 1.264798E+05, 1.281802E+05, 1.298935E+05, 1.316198E+05, 1.333592E+05,
    1.351116E+05, 1.368771E+05, 1.386558E+05, 1.404477E+05, 1.422528E+05, 1.440711E+05,
    1.459027E+05, 1.477476E+05, 1.496059E+05, 1.514775E+05, 1.533626E+05, 1.552611E+05,
    1.571732E+05, 1.590987E+05, 1.610378E+05, 1.629905E+05, 1.649567E+05, 1.669367E+05,
    1.689303E+05, 1.709376E+05, 1.729586E+05, 1.749935E+05, 1.770421E+05, 1.791045E+05,
    1.811808E+05, 1.832710E+05, 1.853751E+05, 1.874931E+05, 1.896251E+05, 1.917711E+05,
    1.939311E+05, 1.961052E+05, 1.982934E+05, 2.004956E+05, 2.027120E+05, 2.049425E+05,
    2.071872E+05, 2.094461E+05, 2.117192E+05, 2.140066E+05, 2.163082E+05, 2.186241E+05,
    2.209544E+05, 2.232989E+05, 2.256579E+05, 2.280312E+05, 2.304189E+05, 2.328211E+05,
    2.352376E+05, 2.376687E+05, 2.401142E+05, 2.425742E+05, 2.450488E+05, 2.475379E+05,
    2.500415E+05, 2.525597E+05, 2.550925E+05, 2.576400E+05, 2.602020E+05, 2.627787E+05,
    2.653700E+05, 2.679761E+05, 2.705968E+05, 2.732322E+05, 2.758823E+05, 2.785471E+05,
    2.812268E+05, 2.839211E+05, 2.866302E+05, 2.893542E+05, 2.920929E+05, 2.948464E+05,
    2.976148E+05, 3.003980E+05, 3.031960E+05, 3.060089E+05, 3.088366E+05, 3.116792E+05,
    3.145368E+05, 3.174092E+05, 3.202965E+05, 3.231987E+05, 3.261158E+05, 3.290479E+05,
    3.319949E+05, 3.349569E+05, 3.379338E+05, 3.409257E+05, 3.439325E+05, 3.469543E+05,
    3.499911E+05, 3.530429E+05, 3.561096E+05, 3.591914E+05, 3.622881E+05, 3.653999E+05,
    3.685267E+05, 3.716684E+05, 3.748252E+05, 3.779970E+05, 3.811839E+05, 3.843857E+05,
    3.876026E+05, 3.908345E+05, 3.940814E+05, 3.973434E+05, 4.006204E+05, 4.039124E+05,
    4.072195E+05, 4.105416E+05, 4.138787E+05, 4.172308E+05, 4.205980E+05, 4.239803E+05,
    4.273775E+05, 4.307898E+05, 4.342171E+05, 4.376594E+05, 4.411168E+05, 4.445892E+05,
    4.480765E+05, 4.515790E+05, 4.550964E+05, 4.586288E+05, 4.621762E+05, 4.657386E+05,
    4.693160E+05, 4.729085E+05, 4.765158E+05, 4.801382E+05, 4.837756E+05, 4.874279E+05,
    4.910951E+05, 4.947774E+05, 4.984745E+05, 5.021866E+05, 5.059137E+05, 5.096557E+05,
    5.134126E+05, 5.171844E+05, 5.209711E+05, 5.247727E+05, 5.285892E+05, 5.324206E+05,
    5.362668E+05, 5.401279E+05, 5.440038E+05, 5.478946E+05, 5.518002E+05, 5.557207E+05,
    5.596559E+05, 5.636060E+05, 5.675708E+05, 5.715504E+05, 5.755448E+05, 5.795539E+05,
    5.835777E+05, 5.876163E+05, 5.916696E+05, 5.957376E+05, 5.998203E+05, 6.039177E+05,
    6.080298E+05, 6.121564E+05, 6.162978E+05, 6.204537E+05, 6.246243E+05, 6.288094E+05,
    6.330091E+05, 6.372234E+05, 6.414523E+05, 6.456957E+05, 6.499535E+05, 6.542259E+05,
    6.585128E+05, 6.628142E+05, 6.671300E+05, 6.714602E+05, 6.758049E+05, 6.801639E+05,
    6.845374E+05, 6.889252E+05, 6.933274E+05, 6.977439E+05, 7.021747E+05, 7.066198E+05,
    7.110792E+05, 7.155528E+05, 7.200407E+05, 7.245429E+05, 7.290592E+05, 7.335897E+05,
    7.381343E+05, 7.426932E+05, 7.472661E+05, 7.518531E+05, 7.564543E+05, 7.610694E+05,
    7.656987E+05, 7.703419E+05, 7.749992E+05, 7.796704E+05, 7.843556E+05, 7.890547E+05,
    7.937678E+05, 7.984947E+05, 8.032355E+05, 8.079901E+05, 8.127586E+05, 8.175409E+05,
    8.223370E+05, 8.271468E+05, 8.319703E+05, 8.368076E+05, 8.416586E+05, 8.465232E+05,
    8.514015E+05, 8.562933E+05, 8.611988E+05, 8.661178E+05, 8.710504E+05, 8.759965E+05,
    8.809561E+05, 8.859292E+05, 8.909157E+05, 8.959156E+05, 9.009290E+05, 9.059557E+05,
    9.109957E+05, 9.160491E+05, 9.211157E+05, 9.261956E+05, 9.312888E+05, 9.363952E+05,
    9.415147E+05, 9.466474E+05, 9.517933E+05, 9.569522E+05, 9.621243E+05, 9.673094E+05,
    9.725075E+05, 9.777186E+05, 9.829427E+05, 9.881797E+05,
])

#  ---------------------- M = 8, I = 1 ---------------------------
M = 8
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.187882E+01, 5.438944E+01, 1.046755E+02, 1.605493E+02, 2.237848E+02, 2.936074E+02,
    3.693790E+02, 4.493987E+02, 5.325091E+02, 6.180282E+02, 7.054511E+02, 7.943992E+02,
    8.845954E+02, 9.758218E+02, 1.067921E+03, 1.160750E+03, 1.254256E+03, 1.348352E+03,
    1.443007E+03, 1.538179E+03, 1.633904E+03, 1.730134E+03, 1.826913E+03, 1.924263E+03,
    2.022194E+03, 2.120743E+03, 2.219926E+03, 2.319814E+03, 2.420412E+03, 2.521749E+03,
    2.623906E+03, 2.726868E+03, 2.830696E+03, 2.935398E+03, 3.041038E+03, 3.147623E+03,
    3.255221E+03, 3.363806E+03, 3.473413E+03, 3.584110E+03, 3.695900E+03, 3.808781E+03,
    3.922787E+03, 4.037949E+03, 4.154301E+03, 4.271797E+03, 4.390503E+03, 4.510451E+03,
    4.631630E+03, 4.754023E+03, 4.877700E+03, 5.002642E+03, 5.128876E+03, 5.256379E+03,
    5.385224E+03, 5.515334E+03, 5.646783E+03, 5.779540E+03, 5.913678E+03, 6.049113E+03,
    6.185916E+03, 6.324108E+03, 6.463595E+03, 6.604508E+03, 6.746748E+03, 6.890451E+03,
    7.035454E+03, 7.181892E+03, 7.329656E+03, 7.478888E+03, 7.629473E+03, 7.781490E+03,
    7.934955E+03, 8.089742E+03, 8.246004E+03, 8.403681E+03, 8.562787E+03, 8.723261E+03,
    8.885184E+03, 9.048571E+03, 9.213358E+03, 9.379626E+03, 9.547237E+03, 9.716351E+03,
    9.886903E+03, 1.005890E+04, 1.023227E+04, 1.040718E+04, 1.058348E+04, 1.076127E+04,
    1.094045E+04, 1.112113E+04, 1.130323E+04, 1.148684E+04, 1.167188E+04, 1.185836E+04,
    1.204629E+04, 1.223566E+04, 1.242649E+04, 1.261887E+04, 1.281263E+04, 1.300787E+04,
    1.320458E+04, 1.340278E+04, 1.360238E+04, 1.380347E+04, 1.400606E+04, 1.421006E+04,
    1.441557E+04, 1.462260E+04, 1.483105E+04, 1.504093E+04, 1.525234E+04, 1.546529E+04,
    1.567956E+04, 1.589539E+04, 1.611265E+04, 1.633148E+04, 1.655176E+04, 1.677349E+04,
    1.699667E+04, 1.722133E+04, 1.744744E+04, 1.767514E+04, 1.790419E+04, 1.813483E+04,
    1.836696E+04, 1.860044E+04, 1.883553E+04, 1.907199E+04, 1.931006E+04, 1.954950E+04,
    1.979055E+04, 2.003299E+04, 2.027692E+04, 2.052237E+04, 2.076931E+04, 2.101764E+04,
    2.126762E+04, 2.151898E+04, 2.177186E+04, 2.202612E+04, 2.228205E+04, 2.253936E+04,
    2.279821E+04, 2.305845E+04, 2.332023E+04, 2.358354E+04, 2.384840E+04, 2.411465E+04,
    2.438245E+04, 2.465165E+04, 2.492240E+04, 2.519456E+04, 2.546842E+04, 2.574354E+04,
    2.602037E+04, 2.629846E+04, 2.657826E+04, 2.685948E+04, 2.714211E+04, 2.742631E+04,
    2.771194E+04, 2.799913E+04, 2.828791E+04, 2.857810E+04, 2.886972E+04, 2.916292E+04,
    2.945755E+04, 2.975376E+04, 3.005141E+04, 3.035064E+04, 3.065130E+04, 3.095340E+04,
    3.125708E+04, 3.156221E+04,
])

#  ---------------------- M = 8, I = 2 ---------------------------
M = 8
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    7.941320E+00, 3.745917E+01, 7.221608E+01, 1.108280E+02, 1.545250E+02, 2.027736E+02,
    2.552148E+02, 3.105134E+02, 3.679482E+02, 4.270480E+02, 4.874637E+02, 5.489392E+02,
    6.112725E+02, 6.743180E+02, 7.379665E+02, 8.021363E+02, 8.667588E+02, 9.318076E+02,
    9.972455E+02, 1.063065E+03, 1.129260E+03, 1.195844E+03, 1.262803E+03, 1.330177E+03,
    1.397975E+03, 1.466208E+03, 1.534920E+03, 1.604131E+03, 1.673861E+03, 1.744132E+03,
    1.814968E+03, 1.886392E+03, 1.958449E+03, 2.031145E+03, 2.104487E+03, 2.178540E+03,
    2.253291E+03, 2.328765E+03, 2.404963E+03, 2.481933E+03, 2.559679E+03, 2.638222E+03,
    2.717563E+03, 2.797724E+03, 2.878728E+03, 2.960569E+03, 3.043241E+03, 3.126792E+03,
    3.211244E+03, 3.296556E+03, 3.382746E+03, 3.469862E+03, 3.557861E+03, 3.646792E+03,
    3.736671E+03, 3.827446E+03, 3.919168E+03, 4.011816E+03, 4.105405E+03, 4.199946E+03,
    4.295455E+03, 4.391908E+03, 4.489355E+03, 4.587730E+03, 4.687084E+03, 4.787429E+03,
    4.888735E+03, 4.991012E+03, 5.094314E+03, 5.198564E+03, 5.303815E+03, 5.410032E+03,
    5.517270E+03, 5.625490E+03, 5.734750E+03, 5.844962E+03, 5.956183E+03, 6.068418E+03,
    6.181628E+03, 6.295921E+03, 6.411150E+03, 6.527425E+03, 6.644701E+03, 6.762984E+03,
    6.882281E+03, 7.002598E+03, 7.123940E+03, 7.246257E+03, 7.369611E+03, 7.494009E+03,
    7.619456E+03, 7.745839E+03, 7.873343E+03, 8.001789E+03, 8.131307E+03, 8.261840E+03,
    8.393389E+03, 8.525963E+03, 8.659562E+03, 8.794193E+03, 8.929860E+03, 9.066566E+03,
    9.204314E+03, 9.343044E+03, 9.482892E+03, 9.623727E+03, 9.765553E+03, 9.908441E+03,
    1.005239E+04, 1.019735E+04, 1.034337E+04, 1.049040E+04, 1.063844E+04, 1.078756E+04,
    1.093769E+04, 1.108883E+04, 1.124107E+04, 1.139433E+04, 1.154862E+04, 1.170400E+04,
    1.186034E+04, 1.201778E+04, 1.217626E+04, 1.233577E+04, 1.249631E+04, 1.265790E+04,
    1.282053E+04, 1.298420E+04, 1.314892E+04, 1.331460E+04, 1.348141E+04, 1.364928E+04,
    1.381820E+04, 1.398809E+04, 1.415904E+04, 1.433114E+04, 1.450421E+04, 1.467825E+04,
    1.485345E+04, 1.502963E+04, 1.520687E+04, 1.538519E+04, 1.556458E+04, 1.574495E+04,
    1.592640E+04, 1.610883E+04, 1.629244E+04, 1.647703E+04, 1.666261E+04, 1.684928E+04,
    1.703703E+04, 1.722587E+04, 1.741570E+04, 1.760653E+04, 1.779854E+04, 1.799145E+04,
    1.818556E+04, 1.838056E+04, 1.857677E+04, 1.877397E+04, 1.897217E+04, 1.917148E+04,
    1.937178E+04, 1.957319E+04, 1.977572E+04, 1.997924E+04, 2.018376E+04, 2.038940E+04,
    2.059605E+04, 2.080370E+04, 2.101258E+04, 2.122235E+04, 2.143324E+04, 2.164514E+04,
    2.185817E+04, 2.207220E+04,
])

#  ---------------------- M = 8, I = 3 ---------------------------
M = 8
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.189184E+01, 5.706583E+01, 1.101111E+02, 1.690336E+02, 2.357137E+02, 3.093392E+02,
    3.894382E+02, 4.738273E+02, 5.614765E+02, 6.516668E+02, 7.438655E+02, 8.376816E+02,
    9.328071E+02, 1.029019E+03, 1.126152E+03, 1.224093E+03, 1.322726E+03, 1.422012E+03,
    1.521894E+03, 1.622376E+03, 1.723436E+03, 1.825094E+03, 1.927347E+03, 2.030239E+03,
    2.133784E+03, 2.238044E+03, 2.343022E+03, 2.448769E+03, 2.555318E+03, 2.662726E+03,
    2.771034E+03, 2.880252E+03, 2.990447E+03, 3.101631E+03, 3.213843E+03, 3.327092E+03,
    3.441482E+03, 3.556955E+03, 3.673617E+03, 3.791438E+03, 3.910456E+03, 4.030706E+03,
    4.152188E+03, 4.274977E+03, 4.399026E+03, 4.524369E+03, 4.651037E+03, 4.779063E+03,
    4.908435E+03, 5.039134E+03, 5.171234E+03, 5.304767E+03, 5.439664E+03, 5.575945E+03,
    5.713690E+03, 5.852870E+03, 5.993512E+03, 6.135528E+03, 6.279050E+03, 6.424044E+03,
    6.570531E+03, 6.718415E+03, 6.867831E+03, 7.018739E+03, 7.171158E+03, 7.325106E+03,
    7.480473E+03, 7.637402E+03, 7.795844E+03, 7.955817E+03, 8.117334E+03, 8.280340E+03,
    8.444852E+03, 8.610949E+03, 8.778504E+03, 8.947672E+03, 9.118396E+03, 9.290609E+03,
    9.464401E+03, 9.639785E+03, 9.816691E+03, 9.995128E+03, 1.017511E+04, 1.035664E+04,
    1.053982E+04, 1.072449E+04, 1.091073E+04, 1.109857E+04, 1.128800E+04, 1.147896E+04,
    1.167153E+04, 1.186564E+04, 1.206128E+04, 1.225857E+04, 1.245741E+04, 1.265782E+04,
    1.285979E+04, 1.306344E+04, 1.326857E+04, 1.347539E+04, 1.368371E+04, 1.389363E+04,
    1.410515E+04, 1.431819E+04, 1.453295E+04, 1.474923E+04, 1.496703E+04, 1.518647E+04,
    1.540755E+04, 1.563017E+04, 1.585444E+04, 1.608026E+04, 1.630775E+04, 1.653679E+04,
    1.676739E+04, 1.699955E+04, 1.723340E+04, 1.746882E+04, 1.770581E+04, 1.794450E+04,
    1.818465E+04, 1.842652E+04, 1.866997E+04, 1.891501E+04, 1.916153E+04, 1.940978E+04,
    1.965962E+04, 1.991107E+04, 2.016414E+04, 2.041882E+04, 2.067511E+04, 2.093290E+04,
    2.119244E+04, 2.145347E+04, 2.171627E+04, 2.198056E+04, 2.224635E+04, 2.251392E+04,
    2.278313E+04, 2.305384E+04, 2.332620E+04, 2.360007E+04, 2.387573E+04, 2.415291E+04,
    2.443174E+04, 2.471209E+04, 2.499410E+04, 2.527778E+04, 2.556313E+04, 2.584999E+04,
    2.613838E+04, 2.642845E+04, 2.672019E+04, 2.701361E+04, 2.730857E+04, 2.760506E+04,
    2.790322E+04, 2.820309E+04, 2.850449E+04, 2.880758E+04, 2.911222E+04, 2.941855E+04,
    2.972642E+04, 3.003600E+04, 3.034711E+04, 3.065978E+04, 3.097415E+04, 3.129023E+04,
    3.160787E+04, 3.192705E+04, 3.224796E+04, 3.257041E+04, 3.289459E+04, 3.322032E+04,
    3.354777E+04, 3.387661E+04,
])

#  ---------------------- M = 9, I = 1 ---------------------------
M = 9
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.454300E+00, 1.022319E+02, 2.882009E+02, 5.289590E+02, 8.141879E+02, 1.138246E+03,
    1.498364E+03, 1.893680E+03, 2.324649E+03, 2.792534E+03, 3.299086E+03, 3.846299E+03,
    4.436548E+03, 5.072346E+03, 5.756332E+03, 6.491539E+03, 7.281118E+03, 8.128285E+03,
    9.036571E+03, 1.000965E+04, 1.105141E+04, 1.216570E+04, 1.335683E+04, 1.462901E+04,
    1.598660E+04, 1.743436E+04, 1.897692E+04, 2.061904E+04, 2.236584E+04, 2.422246E+04,
    2.619414E+04, 2.828645E+04, 3.050468E+04, 3.285482E+04, 3.534284E+04, 3.797440E+04,
    4.075593E+04, 4.369377E+04, 4.679420E+04, 5.006431E+04, 5.351042E+04, 5.713968E+04,
    6.095944E+04, 6.497664E+04, 6.919892E+04, 7.363414E+04, 7.828963E+04, 8.317352E+04,
    8.829416E+04, 9.365966E+04, 9.927870E+04, 1.051597E+05, 1.113113E+05, 1.177434E+05,
    1.244642E+05, 1.314837E+05, 1.388114E+05, 1.464570E+05, 1.544299E+05, 1.627414E+05,
    1.714005E+05, 1.804188E+05, 1.898065E+05, 1.995743E+05, 2.097337E+05, 2.202957E+05,
    2.312719E+05, 2.426739E+05, 2.545138E+05, 2.668032E+05, 2.795549E+05, 2.927811E+05,
    3.064944E+05, 3.207079E+05, 3.354346E+05, 3.506881E+05, 3.664818E+05, 3.828290E+05,
    3.997436E+05, 4.172406E+05, 4.353337E+05, 4.540374E+05, 4.733667E+05, 4.933375E+05,
    5.139636E+05, 5.352603E+05, 5.572451E+05, 5.799322E+05, 6.033387E+05, 6.274802E+05,
    6.523742E+05, 6.780366E+05, 7.044845E+05, 7.317353E+05, 7.598073E+05, 7.887176E+05,
    8.184840E+05, 8.491238E+05, 8.806575E+05, 9.131028E+05, 9.464777E+05, 9.808023E+05,
    1.016096E+06, 1.052379E+06, 1.089669E+06, 1.127988E+06, 1.167357E+06, 1.207794E+06,
    1.249321E+06, 1.291961E+06, 1.335732E+06, 1.380659E+06, 1.426761E+06, 1.474061E+06,
    1.522583E+06, 1.572347E+06, 1.623377E+06, 1.675696E+06, 1.729329E+06, 1.784297E+06,
    1.840624E+06, 1.898336E+06, 1.957457E+06, 2.018012E+06, 2.080025E+06, 2.143521E+06,
    2.208524E+06, 2.275064E+06, 2.343163E+06, 2.412851E+06, 2.484152E+06, 2.557091E+06,
    2.631700E+06, 2.708003E+06, 2.786027E+06, 2.865802E+06, 2.947356E+06, 3.030718E+06,
    3.115914E+06, 3.202976E+06, 3.291932E+06, 3.382810E+06, 3.475643E+06, 3.570459E+06,
    3.667290E+06, 3.766164E+06, 3.867117E+06, 3.970174E+06, 4.075372E+06, 4.182739E+06,
    4.292312E+06, 4.404117E+06, 4.518192E+06, 4.634568E+06, 4.753277E+06, 4.874358E+06,
    4.997837E+06, 5.123754E+06, 5.252144E+06, 5.383038E+06, 5.516472E+06, 5.652484E+06,
    5.791109E+06, 5.932381E+06, 6.076339E+06, 6.223016E+06, 6.372454E+06, 6.524687E+06,
    6.679752E+06, 6.837689E+06, 6.998536E+06, 7.162331E+06, 7.329112E+06, 7.498922E+06,
    7.671795E+06, 7.847772E+06,
])

#  ---------------------- M = 9, I = 2 ---------------------------
M = 9
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.314830E+00, 1.027046E+02, 2.895361E+02, 5.313995E+02, 8.179047E+02, 1.143446E+03,
    1.505213E+03, 1.902320E+03, 2.335259E+03, 2.805284E+03, 3.314118E+03, 3.863828E+03,
    4.456728E+03, 5.095327E+03, 5.782419E+03, 6.520963E+03, 7.314000E+03, 8.164932E+03,
    9.077250E+03, 1.005464E+04, 1.110102E+04, 1.222023E+04, 1.341664E+04, 1.469432E+04,
    1.605802E+04, 1.751204E+04, 1.906127E+04, 2.071061E+04, 2.246511E+04, 2.432974E+04,
    2.631009E+04, 2.841143E+04, 3.063941E+04, 3.299970E+04, 3.549845E+04, 3.814136E+04,
    4.093511E+04, 4.388563E+04, 4.699944E+04, 5.028368E+04, 5.374467E+04, 5.738960E+04,
    6.122559E+04, 6.526014E+04, 6.950067E+04, 7.395475E+04, 7.863036E+04, 8.353538E+04,
    8.867784E+04, 9.406655E+04, 9.970955E+04, 1.056157E+05, 1.117939E+05, 1.182531E+05,
    1.250031E+05, 1.320525E+05, 1.394112E+05, 1.470895E+05, 1.550966E+05, 1.634432E+05,
    1.721394E+05, 1.811964E+05, 1.906239E+05, 2.004330E+05, 2.106356E+05, 2.212424E+05,
    2.322657E+05, 2.437162E+05, 2.556059E+05, 2.679482E+05, 2.807536E+05, 2.940361E+05,
    3.078079E+05, 3.220813E+05, 3.368710E+05, 3.521889E+05, 3.680495E+05, 3.844658E+05,
    4.014520E+05, 4.190232E+05, 4.371929E+05, 4.559761E+05, 4.753874E+05, 4.954424E+05,
    5.161552E+05, 5.375425E+05, 5.596200E+05, 5.824029E+05, 6.059083E+05, 6.301519E+05,
    6.551502E+05, 6.809212E+05, 7.074811E+05, 7.348473E+05, 7.630374E+05, 7.920695E+05,
    8.219613E+05, 8.527311E+05, 8.843978E+05, 9.169790E+05, 9.504949E+05, 9.849644E+05,
    1.020406E+06, 1.056841E+06, 1.094290E+06, 1.132770E+06, 1.172304E+06, 1.212911E+06,
    1.254614E+06, 1.297432E+06, 1.341388E+06, 1.386503E+06, 1.432799E+06, 1.480299E+06,
    1.529023E+06, 1.578997E+06, 1.630242E+06, 1.682781E+06, 1.736639E+06, 1.791837E+06,
    1.848402E+06, 1.906357E+06, 1.965726E+06, 2.026534E+06, 2.088807E+06, 2.152569E+06,
    2.217847E+06, 2.284665E+06, 2.353051E+06, 2.423030E+06, 2.494629E+06, 2.567875E+06,
    2.642796E+06, 2.719418E+06, 2.797771E+06, 2.877882E+06, 2.959776E+06, 3.043487E+06,
    3.129040E+06, 3.216467E+06, 3.305794E+06, 3.397055E+06, 3.490276E+06, 3.585490E+06,
    3.682726E+06, 3.782016E+06, 3.883388E+06, 3.986878E+06, 4.092517E+06, 4.200333E+06,
    4.310363E+06, 4.422637E+06, 4.537190E+06, 4.654052E+06, 4.773258E+06, 4.894845E+06,
    5.018842E+06, 5.145286E+06, 5.274211E+06, 5.405652E+06, 5.539647E+06, 5.676225E+06,
    5.815431E+06, 5.957293E+06, 6.101851E+06, 6.249144E+06, 6.399204E+06, 6.552073E+06,
    6.707787E+06, 6.866385E+06, 7.027904E+06, 7.192382E+06, 7.359862E+06, 7.530380E+06,
    7.703973E+06, 7.880689E+06,
])

#  ---------------------- M = 10, I = 1 ---------------------------
M = 10
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.067800E+00, 2.332671E+02, 6.566540E+02, 1.204608E+03, 1.853462E+03, 2.589584E+03,
    3.403999E+03, 4.290795E+03, 5.246348E+03, 6.268939E+03, 7.358458E+03, 8.515917E+03,
    9.743304E+03, 1.104335E+04, 1.241945E+04, 1.387519E+04, 1.541486E+04, 1.704306E+04,
    1.876472E+04, 2.058489E+04, 2.250873E+04, 2.454224E+04, 2.669106E+04, 2.896127E+04,
    3.135884E+04, 3.389046E+04, 3.656272E+04, 3.938207E+04, 4.235539E+04, 4.549001E+04,
    4.879273E+04, 5.227161E+04, 5.593368E+04, 5.978687E+04, 6.383887E+04, 6.809790E+04,
    7.257236E+04, 7.727028E+04, 8.220086E+04, 8.737241E+04, 9.279450E+04, 9.847505E+04,
    1.044246E+05, 1.106525E+05, 1.171684E+05, 1.239821E+05, 1.311035E+05, 1.385434E+05,
    1.463120E+05, 1.544204E+05, 1.628794E+05, 1.717004E+05, 1.808944E+05, 1.904724E+05,
    2.004476E+05, 2.108313E+05, 2.216352E+05, 2.328730E+05, 2.445559E+05, 2.566978E+05,
    2.693113E+05, 2.824111E+05, 2.960088E+05, 3.101196E+05, 3.247566E+05, 3.399344E+05,
    3.556676E+05, 3.719701E+05, 3.888595E+05, 4.063475E+05, 4.244521E+05, 4.431882E+05,
    4.625704E+05, 4.826163E+05, 5.033429E+05, 5.247666E+05, 5.469033E+05, 5.697703E+05,
    5.933851E+05, 6.177656E+05, 6.429297E+05, 6.688959E+05, 6.956828E+05, 7.233073E+05,
    7.517906E+05, 7.811517E+05, 8.114085E+05, 8.425809E+05, 8.746909E+05, 9.077571E+05,
    9.417998E+05, 9.768401E+05, 1.012899E+06, 1.050001E+06, 1.088162E+06, 1.127407E+06,
    1.167759E+06, 1.209239E+06, 1.251871E+06, 1.295677E+06, 1.340680E+06, 1.386904E+06,
    1.434373E+06, 1.483110E+06, 1.533142E+06, 1.584490E+06, 1.637182E+06, 1.691244E+06,
    1.746699E+06, 1.803572E+06, 1.861893E+06, 1.921685E+06, 1.982978E+06, 2.045797E+06,
    2.110167E+06, 2.176121E+06, 2.243683E+06, 2.312882E+06, 2.383749E+06, 2.456307E+06,
    2.530594E+06, 2.606631E+06, 2.684450E+06, 2.764086E+06, 2.845564E+06, 2.928916E+06,
    3.014177E+06, 3.101370E+06, 3.190535E+06, 3.281700E+06, 3.374897E+06, 3.470160E+06,
    3.567520E+06, 3.667012E+06, 3.768671E+06, 3.872526E+06, 3.978618E+06, 4.086977E+06,
    4.197639E+06, 4.310638E+06, 4.426011E+06, 4.543793E+06, 4.664020E+06, 4.786729E+06,
    4.911957E+06, 5.039746E+06, 5.170124E+06, 5.303134E+06, 5.438818E+06, 5.577205E+06,
    5.718344E+06, 5.862272E+06, 6.009019E+06, 6.158638E+06, 6.311162E+06, 6.466636E+06,
    6.625096E+06, 6.786586E+06, 6.951149E+06, 7.118822E+06, 7.289654E+06, 7.463683E+06,
    7.640954E+06, 7.821510E+06, 8.005394E+06, 8.192658E+06, 8.383334E+06, 8.577472E+06,
    8.775125E+06, 8.976324E+06, 9.181131E+06, 9.389576E+06, 9.601719E+06, 9.817605E+06,
    1.003727E+07, 1.026078E+07,
])

#  ---------------------- M = 11, I = 1 ---------------------------
M = 11
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.834090E+00, 3.544986E+01, 8.933083E+01, 1.601234E+02, 2.441067E+02, 3.392668E+02,
    4.443978E+02, 5.586602E+02, 6.814457E+02, 8.123287E+02, 9.510416E+02, 1.097456E+03,
    1.251567E+03, 1.413475E+03, 1.583372E+03, 1.761529E+03, 1.948287E+03, 2.144046E+03,
    2.349255E+03, 2.564410E+03, 2.790043E+03, 3.026722E+03, 3.275047E+03, 3.535644E+03,
    3.809167E+03, 4.096297E+03, 4.397737E+03, 4.714220E+03, 5.046499E+03, 5.395358E+03,
    5.761605E+03, 6.146077E+03, 6.549640E+03, 6.973192E+03, 7.417660E+03, 7.884006E+03,
    8.373224E+03, 8.886347E+03, 9.424443E+03, 9.988617E+03, 1.058002E+04, 1.119983E+04,
    1.184929E+04, 1.252967E+04, 1.324228E+04, 1.398851E+04, 1.476975E+04, 1.558747E+04,
    1.644319E+04, 1.733848E+04, 1.827495E+04, 1.925429E+04, 2.027822E+04, 2.134854E+04,
    2.246709E+04, 2.363579E+04, 2.485661E+04, 2.613158E+04, 2.746282E+04, 2.885248E+04,
    3.030281E+04, 3.181610E+04, 3.339475E+04, 3.504120E+04, 3.675797E+04, 3.854766E+04,
    4.041296E+04, 4.235662E+04, 4.438148E+04, 4.649045E+04, 4.868654E+04, 5.097282E+04,
    5.335248E+04, 5.582878E+04, 5.840505E+04, 6.108474E+04, 6.387137E+04, 6.676857E+04,
    6.978004E+04, 7.290961E+04, 7.616117E+04, 7.953873E+04, 8.304638E+04, 8.668833E+04,
    9.046886E+04, 9.439239E+04, 9.846341E+04, 1.026865E+05, 1.070664E+05, 1.116079E+05,
    1.163160E+05, 1.211955E+05, 1.262517E+05, 1.314898E+05, 1.369150E+05, 1.425329E+05,
    1.483489E+05, 1.543688E+05, 1.605982E+05, 1.670429E+05, 1.737090E+05, 1.806025E+05,
    1.877295E+05, 1.950963E+05, 2.027093E+05, 2.105748E+05, 2.186995E+05, 2.270900E+05,
    2.357531E+05, 2.446956E+05, 2.539243E+05, 2.634465E+05, 2.732691E+05, 2.833994E+05,
    2.938447E+05, 3.046123E+05, 3.157098E+05, 3.271446E+05, 3.389245E+05, 3.510570E+05,
    3.635501E+05, 3.764116E+05, 3.896493E+05, 4.032714E+05, 4.172859E+05, 4.317008E+05,
    4.465246E+05, 4.617653E+05, 4.774313E+05, 4.935310E+05, 5.100729E+05, 5.270654E+05,
    5.445171E+05, 5.624366E+05, 5.808324E+05, 5.997132E+05, 6.190879E+05, 6.389651E+05,
    6.593535E+05, 6.802621E+05, 7.016996E+05, 7.236749E+05, 7.461970E+05, 7.692747E+05,
    7.929169E+05, 8.171327E+05, 8.419309E+05, 8.673206E+05, 8.933106E+05, 9.199101E+05,
    9.471280E+05, 9.749732E+05, 1.003455E+06, 1.032582E+06, 1.062363E+06, 1.092807E+06,
    1.123924E+06, 1.155722E+06, 1.188210E+06, 1.221397E+06, 1.255292E+06, 1.289903E+06,
    1.325240E+06, 1.361311E+06, 1.398126E+06, 1.435692E+06, 1.474019E+06, 1.513114E+06,
    1.552988E+06, 1.593648E+06, 1.635103E+06, 1.677361E+06, 1.720431E+06, 1.764321E+06,
    1.809040E+06, 1.854596E+06, 1.900996E+06, 1.948250E+06, 1.996365E+06, 2.045349E+06,
    2.095211E+06, 2.145958E+06, 2.197599E+06, 2.250140E+06, 2.303590E+06, 2.357956E+06,
    2.413246E+06, 2.469467E+06, 2.526627E+06, 2.584734E+06, 2.643794E+06, 2.703815E+06,
    2.764803E+06, 2.826766E+06, 2.889712E+06, 2.953646E+06, 3.018575E+06, 3.084507E+06,
    3.151448E+06, 3.219404E+06, 3.288382E+06, 3.358388E+06, 3.429429E+06, 3.501511E+06,
    3.574640E+06, 3.648822E+06, 3.724062E+06, 3.800367E+06, 3.877743E+06, 3.956195E+06,
    4.035729E+06, 4.116350E+06, 4.198063E+06, 4.280875E+06, 4.364789E+06, 4.449812E+06,
    4.535948E+06, 4.623202E+06, 4.711579E+06, 4.801083E+06, 4.891720E+06, 4.983494E+06,
    5.076409E+06, 5.170470E+06, 5.265680E+06, 5.362045E+06, 5.459568E+06, 5.558252E+06,
    5.658103E+06, 5.759124E+06, 5.861317E+06, 5.964688E+06, 6.069240E+06, 6.174975E+06,
    6.281898E+06, 6.390011E+06, 6.499317E+06, 6.609820E+06, 6.721523E+06, 6.834428E+06,
    6.948538E+06, 7.063856E+06, 7.180384E+06, 7.298125E+06, 7.417081E+06, 7.537255E+06,
    7.658648E+06, 7.781264E+06, 7.905103E+06, 8.030168E+06, 8.156460E+06, 8.283982E+06,
    8.412735E+06, 8.542721E+06, 8.673941E+06, 8.806397E+06, 8.940089E+06, 9.075020E+06,
    9.211190E+06, 9.348600E+06, 9.487252E+06, 9.627147E+06, 9.768284E+06, 9.910666E+06,
    1.005429E+07, 1.019916E+07, 1.034528E+07, 1.049264E+07, 1.064125E+07, 1.079111E+07,
    1.094222E+07, 1.109457E+07, 1.124817E+07, 1.140301E+07, 1.155911E+07, 1.171645E+07,
    1.187503E+07, 1.203487E+07, 1.219595E+07, 1.235827E+07, 1.252184E+07, 1.268666E+07,
    1.285272E+07, 1.302002E+07, 1.318857E+07, 1.335836E+07, 1.352938E+07, 1.370165E+07,
    1.387516E+07, 1.404990E+07, 1.422589E+07, 1.440310E+07, 1.458155E+07, 1.476124E+07,
    1.494215E+07, 1.512430E+07, 1.530767E+07, 1.549227E+07, 1.567809E+07, 1.586514E+07,
    1.605341E+07,
])

#  ---------------------- M = 11, I = 2 ---------------------------
M = 11
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.689360E+00, 2.371743E+01, 5.973090E+01, 1.070476E+02, 1.631783E+02, 2.267778E+02,
    2.970405E+02, 3.734061E+02, 4.554693E+02, 5.429478E+02, 6.356653E+02, 7.335395E+02,
    8.365694E+02, 9.448250E+02, 1.058437E+03, 1.177588E+03, 1.302505E+03, 1.433451E+03,
    1.570721E+03, 1.714634E+03, 1.865531E+03, 2.023770E+03, 2.189723E+03, 2.363775E+03,
    2.546320E+03, 2.737759E+03, 2.938503E+03, 3.148970E+03, 3.369585E+03, 3.600778E+03,
    3.842989E+03, 4.096664E+03, 4.362256E+03, 4.640230E+03, 4.931055E+03, 5.235214E+03,
    5.553197E+03, 5.885507E+03, 6.232655E+03, 6.595167E+03, 6.973580E+03, 7.368444E+03,
    7.780322E+03, 8.209792E+03, 8.657444E+03, 9.123885E+03, 9.609736E+03, 1.011563E+04,
    1.064223E+04, 1.119020E+04, 1.176022E+04, 1.235300E+04, 1.296927E+04, 1.360974E+04,
    1.427520E+04, 1.496640E+04, 1.568415E+04, 1.642926E+04, 1.720255E+04, 1.800489E+04,
    1.883715E+04, 1.970021E+04, 2.059500E+04, 2.152244E+04, 2.248350E+04, 2.347915E+04,
    2.451040E+04, 2.557826E+04, 2.668378E+04, 2.782802E+04, 2.901208E+04, 3.023707E+04,
    3.150413E+04, 3.281442E+04, 3.416912E+04, 3.556945E+04, 3.701663E+04, 3.851194E+04,
    4.005664E+04, 4.165205E+04, 4.329951E+04, 4.500037E+04, 4.675603E+04, 4.856789E+04,
    5.043739E+04, 5.236600E+04, 5.435520E+04, 5.640652E+04, 5.852151E+04, 6.070172E+04,
    6.294876E+04, 6.526426E+04, 6.764987E+04, 7.010727E+04, 7.263815E+04, 7.524427E+04,
    7.792737E+04, 8.068925E+04, 8.353171E+04, 8.645661E+04, 8.946581E+04, 9.256120E+04,
    9.574471E+04, 9.901829E+04, 1.023839E+05, 1.058436E+05, 1.093993E+05, 1.130531E+05,
    1.168072E+05, 1.206635E+05, 1.246243E+05, 1.286917E+05, 1.328678E+05, 1.371549E+05,
    1.415551E+05, 1.460708E+05, 1.507042E+05, 1.554576E+05, 1.603333E+05, 1.653337E+05,
    1.704610E+05, 1.757178E+05, 1.811063E+05, 1.866290E+05, 1.922883E+05, 1.980868E+05,
    2.040268E+05, 2.101108E+05, 2.163414E+05, 2.227210E+05, 2.292523E+05, 2.359378E+05,
    2.427799E+05, 2.497814E+05, 2.569448E+05, 2.642727E+05, 2.717677E+05, 2.794324E+05,
    2.872695E+05, 2.952817E+05, 3.034715E+05, 3.118417E+05, 3.203948E+05, 3.291337E+05,
    3.380608E+05, 3.471791E+05, 3.564910E+05, 3.659994E+05, 3.757069E+05, 3.856162E+05,
    3.957300E+05, 4.060510E+05, 4.165819E+05, 4.273253E+05, 4.382841E+05, 4.494608E+05,
    4.608582E+05, 4.724789E+05, 4.843256E+05, 4.964010E+05, 5.087078E+05, 5.212486E+05,
    5.340260E+05, 5.470427E+05, 5.603014E+05, 5.738046E+05, 5.875549E+05, 6.015551E+05,
    6.158075E+05, 6.303150E+05, 6.450799E+05, 6.601048E+05, 6.753923E+05, 6.909449E+05,
    7.067651E+05, 7.228554E+05, 7.392182E+05, 7.558561E+05, 7.727714E+05, 7.899665E+05,
    8.074439E+05, 8.252060E+05, 8.432551E+05, 8.615934E+05, 8.802235E+05, 8.991475E+05,
    9.183677E+05, 9.378864E+05, 9.577058E+05, 9.778282E+05, 9.982556E+05, 1.018990E+06,
    1.040034E+06, 1.061390E+06, 1.083059E+06, 1.105044E+06, 1.127346E+06, 1.149969E+06,
    1.172913E+06, 1.196180E+06, 1.219773E+06, 1.243694E+06, 1.267944E+06, 1.292525E+06,
    1.317439E+06, 1.342688E+06, 1.368273E+06, 1.394197E+06, 1.420461E+06, 1.447066E+06,
    1.474015E+06, 1.501309E+06, 1.528949E+06, 1.556938E+06, 1.585276E+06, 1.613965E+06,
    1.643007E+06, 1.672403E+06, 1.702154E+06, 1.732262E+06, 1.762729E+06, 1.793555E+06,
    1.824742E+06, 1.856291E+06, 1.888203E+06, 1.920480E+06, 1.953123E+06, 1.986132E+06,
    2.019510E+06, 2.053256E+06, 2.087373E+06, 2.121861E+06, 2.156721E+06, 2.191954E+06,
    2.227562E+06, 2.263544E+06, 2.299902E+06, 2.336637E+06, 2.373749E+06, 2.411240E+06,
    2.449110E+06, 2.487359E+06, 2.525989E+06, 2.565001E+06, 2.604394E+06, 2.644169E+06,
    2.684328E+06, 2.724871E+06, 2.765797E+06, 2.807108E+06, 2.848805E+06, 2.890886E+06,
    2.933354E+06, 2.976209E+06, 3.019450E+06, 3.063078E+06, 3.107094E+06, 3.151498E+06,
    3.196289E+06, 3.241469E+06, 3.287037E+06, 3.332994E+06, 3.379340E+06, 3.426074E+06,
    3.473198E+06, 3.520711E+06, 3.568613E+06, 3.616905E+06, 3.665585E+06, 3.714655E+06,
    3.764114E+06, 3.813962E+06, 3.864200E+06, 3.914826E+06, 3.965841E+06, 4.017245E+06,
    4.069037E+06, 4.121218E+06, 4.173787E+06, 4.226743E+06, 4.280088E+06, 4.333819E+06,
    4.387938E+06, 4.442443E+06, 4.497335E+06, 4.552613E+06, 4.608276E+06, 4.664325E+06,
    4.720758E+06, 4.777576E+06, 4.834778E+06, 4.892363E+06, 4.950330E+06, 5.008681E+06,
    5.067413E+06, 5.126526E+06, 5.186021E+06, 5.245895E+06, 5.306149E+06, 5.366782E+06,
    5.427793E+06,
])

#  ---------------------- M = 12, I = 1 ---------------------------
M = 12
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.573524E+01, 2.896824E+03, 8.174539E+03, 1.500830E+04, 2.310835E+04, 3.233901E+04,
    4.267884E+04, 5.420733E+04, 6.708747E+04, 8.154826E+04, 9.787188E+04, 1.163871E+05,
    1.374678E+05, 1.615349E+05, 1.890610E+05, 2.205764E+05, 2.566761E+05, 2.980283E+05,
    3.453830E+05, 3.995815E+05, 4.615675E+05, 5.323980E+05, 6.132560E+05, 7.054636E+05,
    8.104973E+05, 9.300032E+05, 1.065814E+06, 1.219969E+06, 1.394732E+06, 1.592616E+06,
    1.816403E+06, 2.069175E+06, 2.354336E+06, 2.675644E+06, 3.037246E+06, 3.443706E+06,
    3.900048E+06, 4.411790E+06, 4.984997E+06, 5.626315E+06, 6.343026E+06, 7.143101E+06,
    8.035255E+06, 9.029012E+06, 1.013476E+07, 1.136382E+07, 1.272855E+07, 1.424236E+07,
    1.591986E+07, 1.777692E+07, 1.983076E+07, 2.210007E+07, 2.460508E+07, 2.736772E+07,
    3.041171E+07, 3.376270E+07, 3.744839E+07, 4.149872E+07, 4.594593E+07, 5.082484E+07,
    5.617291E+07, 6.203049E+07, 6.844098E+07, 7.545107E+07, 8.311086E+07, 9.147421E+07,
    1.005989E+08, 1.105468E+08, 1.213843E+08, 1.331825E+08, 1.460176E+08, 1.599709E+08,
    1.751294E+08, 1.915862E+08, 2.094406E+08, 2.287987E+08, 2.497734E+08, 2.724856E+08,
    2.970635E+08, 3.236443E+08, 3.523735E+08, 3.834064E+08, 4.169078E+08, 4.530531E+08,
    4.920286E+08, 5.340321E+08, 5.792739E+08, 6.279767E+08, 6.803770E+08, 7.367252E+08,
    7.972871E+08, 8.623440E+08, 9.321937E+08, 1.007151E+09, 1.087551E+09, 1.173744E+09,
    1.266104E+09, 1.365025E+09, 1.470923E+09, 1.584237E+09, 1.705430E+09, 1.834993E+09,
    1.973439E+09, 2.121315E+09, 2.279192E+09, 2.447675E+09, 2.627399E+09, 2.819035E+09,
    3.023286E+09, 3.240894E+09, 3.472638E+09, 3.719338E+09, 3.981857E+09, 4.261099E+09,
    4.558015E+09, 4.873603E+09, 5.208913E+09, 5.565043E+09, 5.943148E+09, 6.344438E+09,
    6.770183E+09, 7.221711E+09, 7.700418E+09, 8.207763E+09, 8.745275E+09, 9.314557E+09,
    9.917283E+09, 1.055521E+10, 1.123017E+10, 1.194408E+10, 1.269895E+10, 1.349688E+10,
    1.434006E+10, 1.523078E+10, 1.617143E+10, 1.716451E+10, 1.821263E+10, 1.931852E+10,
    2.048501E+10, 2.171506E+10, 2.301179E+10, 2.437840E+10, 2.581826E+10, 2.733487E+10,
    2.893189E+10, 3.061313E+10, 3.238254E+10, 3.424425E+10, 3.620258E+10, 3.826198E+10,
    4.042712E+10, 4.270285E+10, 4.509420E+10, 4.760643E+10, 5.024498E+10, 5.301553E+10,
    5.592397E+10, 5.897644E+10, 6.217929E+10, 6.553915E+10, 6.906289E+10, 7.275761E+10,
    7.663079E+10, 8.069007E+10, 8.494343E+10, 8.939920E+10, 9.406595E+10, 9.895261E+10,
    1.040684E+11, 1.094230E+11, 1.150262E+11, 1.208886E+11, 1.270205E+11, 1.334333E+11,
    1.401384E+11, 1.471476E+11,
])

#  ---------------------- M = 12, I = 2 ---------------------------
M = 12
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.382603E+01, 1.931437E+03, 5.450319E+03, 1.000669E+04, 1.540736E+04, 2.156205E+04,
    2.845720E+04, 3.614739E+04, 4.474377E+04, 5.440219E+04, 6.531440E+04, 7.770355E+04,
    9.182310E+04, 1.079584E+05, 1.264298E+05, 1.475967E+05, 1.718627E+05, 1.996812E+05,
    2.315613E+05, 2.680746E+05, 3.098624E+05, 3.576437E+05, 4.122233E+05, 4.745014E+05,
    5.454834E+05, 6.262912E+05, 7.181744E+05, 8.225232E+05, 9.408828E+05, 1.074968E+06,
    1.226680E+06, 1.398121E+06, 1.591620E+06, 1.809745E+06, 2.055330E+06, 2.331499E+06,
    2.641689E+06, 2.989677E+06, 3.379614E+06, 3.816053E+06, 4.303982E+06, 4.848864E+06,
    5.456671E+06, 6.133931E+06, 6.887769E+06, 7.725955E+06, 8.656957E+06, 9.689991E+06,
    1.083508E+07, 1.210311E+07, 1.350592E+07, 1.505634E+07, 1.676827E+07, 1.865678E+07,
    2.073817E+07, 2.303008E+07, 2.555154E+07, 2.832314E+07, 3.136707E+07, 3.470727E+07,
    3.836952E+07, 4.238158E+07, 4.677333E+07, 5.157688E+07, 5.682677E+07, 6.256005E+07,
    6.881650E+07, 7.563878E+07, 8.307263E+07, 9.116702E+07, 9.997437E+07, 1.095508E+08,
    1.199563E+08, 1.312550E+08, 1.435154E+08, 1.568106E+08, 1.712185E+08, 1.868225E+08,
    2.037110E+08, 2.219786E+08, 2.417258E+08, 2.630596E+08, 2.860939E+08, 3.109496E+08,
    3.377555E+08, 3.666480E+08, 3.977722E+08, 4.312820E+08, 4.673407E+08, 5.061213E+08,
    5.478072E+08, 5.925928E+08, 6.406837E+08, 6.922978E+08, 7.476655E+08, 8.070303E+08,
    8.706501E+08, 9.387968E+08, 1.011758E+09, 1.089838E+09, 1.173356E+09, 1.262651E+09,
    1.358080E+09, 1.460018E+09, 1.568862E+09, 1.685029E+09, 1.808960E+09, 1.941117E+09,
    2.081987E+09, 2.232084E+09, 2.391947E+09, 2.562142E+09, 2.743268E+09, 2.935948E+09,
    3.140842E+09, 3.358640E+09, 3.590069E+09, 3.835888E+09, 4.096897E+09, 4.373935E+09,
    4.667879E+09, 4.979648E+09, 5.310211E+09, 5.660576E+09, 6.031804E+09, 6.425001E+09,
    6.841331E+09, 7.282006E+09, 7.748297E+09, 8.241535E+09, 8.763108E+09, 9.314471E+09,
    9.897139E+09, 1.051270E+10, 1.116282E+10, 1.184921E+10, 1.257370E+10, 1.333817E+10,
    1.414458E+10, 1.499499E+10, 1.589154E+10, 1.683648E+10, 1.783212E+10, 1.888090E+10,
    1.998535E+10, 2.114810E+10, 2.237192E+10, 2.365965E+10, 2.501428E+10, 2.643891E+10,
    2.793676E+10, 2.951121E+10, 3.116574E+10, 3.290399E+10, 3.472976E+10, 3.664695E+10,
    3.865967E+10, 4.077217E+10, 4.298886E+10, 4.531432E+10, 4.775333E+10, 5.031083E+10,
    5.299197E+10, 5.580208E+10, 5.874669E+10, 6.183156E+10, 6.506265E+10, 6.844617E+10,
    7.198850E+10, 7.569632E+10, 7.957652E+10, 8.363629E+10, 8.788300E+10, 9.232438E+10,
    9.696836E+10, 1.018233E+11,
])

#  ---------------------- M = 13, I = 1 ---------------------------
M = 13
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.538496E+01, 1.602673E+01, 1.728778E+01, 2.005293E+01, 2.374009E+01, 2.800797E+01,
    3.266623E+01, 3.759941E+01, 4.273326E+01, 4.801797E+01, 5.341894E+01, 5.891142E+01,
    6.447729E+01, 7.010295E+01, 7.577806E+01, 8.149456E+01, 8.724613E+01, 9.302771E+01,
    9.883525E+01, 1.046654E+02, 1.105156E+02, 1.163834E+02, 1.222671E+02, 1.281652E+02,
    1.340764E+02, 1.399998E+02, 1.459345E+02, 1.518800E+02, 1.578359E+02, 1.638019E+02,
    1.697777E+02, 1.757636E+02, 1.817594E+02, 1.877655E+02, 1.937821E+02, 1.998096E+02,
    2.058484E+02, 2.118992E+02, 2.179624E+02, 2.240388E+02, 2.301290E+02, 2.362337E+02,
    2.423537E+02, 2.484898E+02, 2.546428E+02, 2.608134E+02, 2.670027E+02, 2.732114E+02,
    2.794403E+02, 2.856903E+02, 2.919624E+02, 2.982572E+02, 3.045757E+02, 3.109188E+02,
    3.172871E+02, 3.236817E+02, 3.301032E+02, 3.365525E+02, 3.430303E+02, 3.495374E+02,
    3.560745E+02, 3.626425E+02, 3.692419E+02, 3.758735E+02, 3.825380E+02, 3.892360E+02,
    3.959682E+02, 4.027353E+02, 4.095377E+02, 4.163762E+02, 4.232513E+02, 4.301635E+02,
    4.371135E+02, 4.441017E+02, 4.511287E+02, 4.581950E+02, 4.653011E+02, 4.724475E+02,
    4.796345E+02, 4.868628E+02, 4.941326E+02, 5.014445E+02, 5.087989E+02, 5.161961E+02,
    5.236366E+02, 5.311207E+02, 5.386489E+02, 5.462214E+02, 5.538387E+02, 5.615011E+02,
    5.692089E+02, 5.769624E+02, 5.847620E+02, 5.926080E+02, 6.005006E+02, 6.084403E+02,
    6.164272E+02, 6.244617E+02, 6.325440E+02, 6.406744E+02, 6.488532E+02, 6.570806E+02,
    6.653569E+02, 6.736823E+02, 6.820571E+02, 6.904815E+02, 6.989557E+02, 7.074801E+02,
    7.160547E+02, 7.246798E+02, 7.333557E+02, 7.420826E+02, 7.508606E+02, 7.596900E+02,
    7.685709E+02, 7.775036E+02, 7.864883E+02, 7.955252E+02, 8.046144E+02, 8.137561E+02,
    8.229506E+02, 8.321980E+02, 8.414985E+02, 8.508522E+02, 8.602594E+02, 8.697202E+02,
    8.792348E+02, 8.888034E+02, 8.984261E+02, 9.081031E+02, 9.178345E+02, 9.276206E+02,
    9.374615E+02, 9.473573E+02, 9.573083E+02, 9.673144E+02, 9.773761E+02, 9.874933E+02,
    9.976662E+02, 1.007895E+03, 1.018180E+03, 1.028521E+03, 1.038918E+03, 1.049372E+03,
    1.059883E+03, 1.070450E+03, 1.081075E+03, 1.091756E+03, 1.102495E+03, 1.113291E+03,
    1.124144E+03, 1.135055E+03, 1.146024E+03, 1.157051E+03, 1.168136E+03, 1.179280E+03,
    1.190482E+03, 1.201742E+03, 1.213061E+03, 1.224439E+03, 1.235875E+03, 1.247371E+03,
    1.258926E+03, 1.270541E+03, 1.282215E+03, 1.293948E+03, 1.305742E+03, 1.317595E+03,
    1.329509E+03, 1.341483E+03, 1.353517E+03, 1.365612E+03, 1.377767E+03, 1.389983E+03,
    1.402260E+03, 1.414599E+03, 1.426998E+03, 1.439459E+03, 1.451981E+03, 1.464566E+03,
    1.477211E+03, 1.489919E+03, 1.502689E+03, 1.515521E+03, 1.528416E+03, 1.541373E+03,
    1.554393E+03, 1.567476E+03, 1.580621E+03, 1.593830E+03, 1.607102E+03, 1.620437E+03,
    1.633836E+03, 1.647299E+03, 1.660825E+03, 1.674416E+03, 1.688070E+03, 1.701789E+03,
    1.715573E+03, 1.729421E+03, 1.743334E+03, 1.757311E+03, 1.771354E+03, 1.785462E+03,
    1.799635E+03, 1.813874E+03, 1.828179E+03, 1.842549E+03, 1.856985E+03, 1.871488E+03,
    1.886056E+03, 1.900691E+03, 1.915393E+03, 1.930162E+03, 1.944997E+03, 1.959899E+03,
    1.974869E+03, 1.989906E+03, 2.005010E+03, 2.020183E+03, 2.035423E+03, 2.050731E+03,
    2.066107E+03, 2.081552E+03, 2.097065E+03, 2.112647E+03, 2.128297E+03, 2.144017E+03,
    2.159805E+03, 2.175663E+03, 2.191591E+03, 2.207588E+03, 2.223654E+03, 2.239791E+03,
    2.255998E+03, 2.272275E+03, 2.288623E+03, 2.305041E+03, 2.321530E+03, 2.338090E+03,
    2.354721E+03, 2.371423E+03, 2.388197E+03, 2.405042E+03, 2.421959E+03, 2.438948E+03,
    2.456009E+03, 2.473143E+03, 2.490349E+03, 2.507627E+03, 2.524978E+03, 2.542402E+03,
    2.559899E+03, 2.577470E+03, 2.595114E+03, 2.612831E+03, 2.630622E+03, 2.648488E+03,
    2.666427E+03, 2.684441E+03, 2.702529E+03, 2.720691E+03, 2.738929E+03, 2.757241E+03,
    2.775629E+03, 2.794091E+03, 2.812630E+03, 2.831243E+03, 2.849933E+03, 2.868698E+03,
    2.887540E+03, 2.906458E+03, 2.925453E+03, 2.944524E+03, 2.963671E+03, 2.982896E+03,
    3.002198E+03, 3.021577E+03, 3.041034E+03, 3.060568E+03, 3.080180E+03, 3.099870E+03,
    3.119638E+03, 3.139484E+03, 3.159409E+03, 3.179412E+03, 3.199494E+03, 3.219655E+03,
    3.239895E+03, 3.260214E+03, 3.280613E+03, 3.301091E+03, 3.321649E+03, 3.342287E+03,
    3.363005E+03, 3.383803E+03, 3.404681E+03, 3.425640E+03, 3.446680E+03, 3.467800E+03,
    3.489001E+03, 3.510284E+03, 3.531647E+03, 3.553093E+03, 3.574619E+03, 3.596228E+03,
    3.617918E+03, 3.639691E+03, 3.661545E+03, 3.683482E+03, 3.705502E+03, 3.727604E+03,
    3.749788E+03, 3.772056E+03, 3.794407E+03, 3.816841E+03, 3.839358E+03, 3.861959E+03,
    3.884644E+03, 3.907412E+03, 3.930264E+03, 3.953201E+03, 3.976221E+03, 3.999326E+03,
    4.022515E+03, 4.045789E+03, 4.069147E+03, 4.092591E+03, 4.116119E+03, 4.139733E+03,
    4.163432E+03, 4.187216E+03, 4.211086E+03, 4.235041E+03, 4.259082E+03, 4.283209E+03,
    4.307422E+03, 4.331721E+03, 4.356107E+03, 4.380578E+03, 4.405137E+03, 4.429781E+03,
    4.454513E+03, 4.479331E+03, 4.504237E+03, 4.529229E+03, 4.554309E+03, 4.579476E+03,
    4.604730E+03, 4.630072E+03, 4.655502E+03, 4.681019E+03, 4.706624E+03, 4.732317E+03,
    4.758098E+03, 4.783967E+03, 4.809924E+03, 4.835970E+03, 4.862104E+03, 4.888326E+03,
    4.914637E+03, 4.941037E+03, 4.967526E+03, 4.994104E+03, 5.020770E+03, 5.047526E+03,
    5.074371E+03, 5.101305E+03, 5.128328E+03, 5.155441E+03, 5.182643E+03, 5.209935E+03,
    5.237316E+03, 5.264787E+03, 5.292348E+03, 5.319999E+03, 5.347739E+03, 5.375570E+03,
    5.403491E+03, 5.431502E+03, 5.459603E+03, 5.487794E+03, 5.516076E+03, 5.544448E+03,
    5.572910E+03, 5.601464E+03, 5.630107E+03, 5.658842E+03, 5.687667E+03, 5.716582E+03,
    5.745589E+03, 5.774686E+03, 5.803874E+03, 5.833154E+03, 5.862524E+03, 5.891985E+03,
    5.921537E+03, 5.951181E+03, 5.980916E+03, 6.010741E+03, 6.040658E+03, 6.070667E+03,
    6.100766E+03, 6.130957E+03, 6.161240E+03, 6.191614E+03, 6.222079E+03, 6.252635E+03,
    6.283284E+03, 6.314023E+03, 6.344855E+03, 6.375777E+03, 6.406792E+03, 6.437898E+03,
    6.469095E+03, 6.500385E+03, 6.531765E+03, 6.563238E+03, 6.594802E+03, 6.626458E+03,
    6.658206E+03, 6.690045E+03, 6.721976E+03, 6.753999E+03, 6.786113E+03, 6.818319E+03,
    6.850617E+03, 6.883007E+03, 6.915488E+03, 6.948061E+03, 6.980726E+03, 7.013482E+03,
    7.046330E+03, 7.079270E+03, 7.112302E+03, 7.145425E+03, 7.178640E+03, 7.211946E+03,
    7.245344E+03, 7.278834E+03, 7.312415E+03, 7.346088E+03, 7.379852E+03, 7.413708E+03,
    7.447655E+03, 7.481694E+03, 7.515824E+03, 7.550045E+03, 7.584358E+03, 7.618763E+03,
    7.653258E+03,
])

#  ---------------------- M = 13, I = 2 ---------------------------
M = 13
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.597389E+01, 1.605971E+01, 1.732986E+01, 2.012387E+01, 2.384501E+01, 2.814934E+01,
    3.284552E+01, 3.781754E+01, 4.299087E+01, 4.831550E+01, 5.375673E+01, 5.928972E+01,
    6.489630E+01, 7.056284E+01, 7.627897E+01, 8.203662E+01, 8.783033E+01, 9.365333E+01,
    9.950433E+01, 1.053783E+02, 1.112735E+02, 1.171880E+02, 1.231211E+02, 1.290729E+02,
    1.350413E+02, 1.410255E+02, 1.470291E+02, 1.530492E+02, 1.590887E+02, 1.651462E+02,
    1.712253E+02, 1.773248E+02, 1.834470E+02, 1.895945E+02, 1.957642E+02, 2.019629E+02,
    2.081875E+02, 2.144411E+02, 2.207268E+02, 2.270436E+02, 2.333928E+02, 2.397777E+02,
    2.461972E+02, 2.526550E+02, 2.591499E+02, 2.656857E+02, 2.722610E+02, 2.788770E+02,
    2.855378E+02, 2.922419E+02, 2.989905E+02, 3.057878E+02, 3.126291E+02, 3.195186E+02,
    3.264576E+02, 3.334441E+02, 3.404823E+02, 3.475737E+02, 3.547126E+02, 3.619069E+02,
    3.691508E+02, 3.764523E+02, 3.838055E+02, 3.912149E+02, 3.986779E+02, 4.061993E+02,
    4.137762E+02, 4.214096E+02, 4.290963E+02, 4.368453E+02, 4.446536E+02, 4.525179E+02,
    4.604431E+02, 4.684258E+02, 4.764670E+02, 4.845716E+02, 4.927317E+02, 5.009570E+02,
    5.092437E+02, 5.175879E+02, 5.259950E+02, 5.344656E+02, 5.429958E+02, 5.515910E+02,
    5.602469E+02, 5.689641E+02, 5.777484E+02, 5.865952E+02, 5.955001E+02, 6.044738E+02,
    6.135065E+02, 6.226095E+02, 6.317725E+02, 6.410014E+02, 6.502913E+02, 6.596482E+02,
    6.690726E+02, 6.785594E+02, 6.881090E+02, 6.977277E+02, 7.074100E+02, 7.171565E+02,
    7.269735E+02, 7.368494E+02, 7.467968E+02, 7.568101E+02, 7.668831E+02, 7.770291E+02,
    7.872422E+02, 7.975161E+02, 8.078577E+02, 8.182674E+02, 8.287457E+02, 8.392861E+02,
    8.498957E+02, 8.605749E+02, 8.713171E+02, 8.821296E+02, 8.930058E+02, 9.039529E+02,
    9.149714E+02, 9.260471E+02, 9.372022E+02, 9.484149E+02, 9.597003E+02, 9.710512E+02,
    9.824755E+02, 9.939582E+02, 1.005515E+03, 1.017146E+03, 1.028836E+03, 1.040601E+03,
    1.052425E+03, 1.064325E+03, 1.076292E+03, 1.088328E+03, 1.100432E+03, 1.112604E+03,
    1.124845E+03, 1.137155E+03, 1.149542E+03, 1.161991E+03, 1.174509E+03, 1.187097E+03,
    1.199746E+03, 1.212474E+03, 1.225273E+03, 1.238133E+03, 1.251065E+03, 1.264076E+03,
    1.277150E+03, 1.290286E+03, 1.303503E+03, 1.316783E+03, 1.330135E+03, 1.343559E+03,
    1.357046E+03, 1.370616E+03, 1.384249E+03, 1.397945E+03, 1.411724E+03, 1.425567E+03,
    1.439474E+03, 1.453465E+03, 1.467520E+03, 1.481639E+03, 1.495832E+03, 1.510100E+03,
    1.524443E+03, 1.538850E+03, 1.553323E+03, 1.567870E+03, 1.582494E+03, 1.597182E+03,
    1.611947E+03, 1.626776E+03,
])

#  ---------------------- M = 13, I = 3 ---------------------------
M = 13
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.398602E+01, 2.529396E+01, 3.190248E+01, 4.103232E+01, 5.180328E+01, 6.382165E+01,
    7.678389E+01, 9.046186E+01, 1.046887E+02, 1.193429E+02, 1.343350E+02, 1.495984E+02,
    1.650827E+02, 1.807493E+02, 1.965682E+02, 2.125155E+02, 2.285724E+02, 2.447236E+02,
    2.609566E+02, 2.772613E+02, 2.936290E+02, 3.100527E+02, 3.265296E+02, 3.430483E+02,
    3.596111E+02, 3.762111E+02, 3.928490E+02, 4.095183E+02, 4.262208E+02, 4.429551E+02,
    4.597196E+02, 4.765132E+02, 4.933399E+02, 5.101943E+02, 5.270811E+02, 5.439947E+02,
    5.609459E+02, 5.779297E+02, 5.949463E+02, 6.120082E+02, 6.290982E+02, 6.462353E+02,
    6.634146E+02, 6.806371E+02, 6.979040E+02, 7.152234E+02, 7.325899E+02, 7.500121E+02,
    7.674917E+02, 7.850226E+02, 8.026143E+02, 8.202684E+02, 8.379869E+02, 8.557801E+02,
    8.736330E+02, 8.915558E+02, 9.095506E+02, 9.276191E+02, 9.457727E+02, 9.639948E+02,
    9.823062E+02, 1.000699E+03, 1.019167E+03, 1.037730E+03, 1.056380E+03, 1.075110E+03,
    1.093943E+03, 1.112858E+03, 1.131880E+03, 1.150989E+03, 1.170197E+03, 1.189507E+03,
    1.208908E+03, 1.228415E+03, 1.248030E+03, 1.267742E+03, 1.287565E+03, 1.307488E+03,
    1.327527E+03, 1.347670E+03, 1.367918E+03, 1.388287E+03, 1.408751E+03, 1.429339E+03,
    1.450040E+03, 1.470854E+03, 1.491784E+03, 1.512831E+03, 1.533996E+03, 1.555282E+03,
    1.576689E+03, 1.598204E+03, 1.619844E+03, 1.641594E+03, 1.663488E+03, 1.685495E+03,
    1.707617E+03, 1.729870E+03, 1.752241E+03, 1.774729E+03, 1.797354E+03, 1.820116E+03,
    1.842984E+03, 1.865991E+03, 1.889123E+03, 1.912380E+03, 1.935764E+03, 1.959275E+03,
    1.982915E+03, 2.006703E+03, 2.030604E+03, 2.054636E+03, 2.078802E+03, 2.103083E+03,
    2.127519E+03, 2.152071E+03, 2.176760E+03, 2.201588E+03, 2.226555E+03, 2.251642E+03,
    2.276871E+03, 2.302222E+03, 2.327716E+03, 2.353353E+03, 2.379116E+03, 2.405003E+03,
    2.431038E+03, 2.457199E+03, 2.483508E+03, 2.509946E+03, 2.536534E+03, 2.563252E+03,
    2.590101E+03, 2.617102E+03, 2.644213E+03, 2.671478E+03, 2.698899E+03, 2.726432E+03,
    2.754121E+03, 2.781922E+03, 2.809883E+03, 2.837979E+03, 2.866213E+03, 2.894608E+03,
    2.923117E+03, 2.951765E+03, 2.980577E+03, 3.009504E+03, 3.038597E+03, 3.067806E+03,
    3.097157E+03, 3.126676E+03, 3.156312E+03, 3.186092E+03, 3.216016E+03, 3.246085E+03,
    3.276300E+03, 3.306660E+03, 3.337166E+03, 3.367793E+03, 3.398568E+03, 3.429490E+03,
    3.460562E+03, 3.491782E+03, 3.523124E+03, 3.554617E+03, 3.586261E+03, 3.618055E+03,
    3.649974E+03, 3.682073E+03, 3.714268E+03, 3.746645E+03, 3.779146E+03, 3.811803E+03,
    3.844614E+03, 3.877551E+03,
])

#  ---------------------- M = 14, I = 1 ---------------------------
M = 14
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.000000E+00, 4.626210E+00, 6.976320E+00, 9.595800E+00, 1.226348E+01, 1.494872E+01,
    1.764266E+01, 2.034174E+01, 2.304424E+01, 2.574926E+01, 2.845624E+01, 3.116485E+01,
    3.387487E+01, 3.658615E+01, 3.929857E+01, 4.201207E+01, 4.472657E+01, 4.744205E+01,
    5.015846E+01, 5.287579E+01, 5.559401E+01, 5.831312E+01, 6.103312E+01, 6.375401E+01,
    6.647583E+01, 6.919858E+01, 7.192233E+01, 7.464713E+01, 7.737305E+01, 8.010018E+01,
    8.282863E+01, 8.555853E+01, 8.829002E+01, 9.102328E+01, 9.375848E+01, 9.649584E+01,
    9.923557E+01, 1.019779E+02, 1.047231E+02, 1.074715E+02, 1.102232E+02, 1.129788E+02,
    1.157383E+02, 1.185022E+02, 1.212707E+02, 1.240443E+02, 1.268232E+02, 1.296079E+02,
    1.323986E+02, 1.351957E+02, 1.379995E+02, 1.408105E+02, 1.436289E+02, 1.464552E+02,
    1.492896E+02, 1.521325E+02, 1.549843E+02, 1.578453E+02, 1.607158E+02, 1.635962E+02,
    1.664868E+02, 1.693878E+02, 1.722998E+02, 1.752228E+02, 1.781573E+02, 1.811035E+02,
    1.840617E+02, 1.870323E+02, 1.900154E+02, 1.930114E+02, 1.960205E+02, 1.990429E+02,
    2.020790E+02, 2.051289E+02, 2.081930E+02, 2.112714E+02, 2.143644E+02, 2.174721E+02,
    2.205948E+02, 2.237328E+02, 2.268862E+02, 2.300551E+02, 2.332399E+02, 2.364407E+02,
    2.396577E+02, 2.428910E+02, 2.461408E+02, 2.494073E+02, 2.526907E+02, 2.559911E+02,
    2.593087E+02, 2.626436E+02, 2.659960E+02, 2.693660E+02, 2.727538E+02, 2.761594E+02,
    2.795831E+02, 2.830250E+02, 2.864851E+02, 2.899636E+02, 2.934607E+02, 2.969764E+02,
    3.005109E+02, 3.040643E+02, 3.076367E+02, 3.112281E+02, 3.148388E+02, 3.184687E+02,
    3.221181E+02, 3.257869E+02, 3.294754E+02, 3.331835E+02, 3.369115E+02, 3.406592E+02,
    3.444270E+02, 3.482148E+02, 3.520228E+02, 3.558509E+02, 3.596994E+02, 3.635682E+02,
    3.674576E+02, 3.713674E+02, 3.752979E+02, 3.792490E+02, 3.832209E+02, 3.872136E+02,
    3.912273E+02, 3.952619E+02, 3.993176E+02, 4.033944E+02, 4.074924E+02, 4.116116E+02,
    4.157521E+02, 4.199140E+02, 4.240974E+02, 4.283022E+02, 4.325286E+02, 4.367767E+02,
    4.410464E+02, 4.453379E+02, 4.496512E+02, 4.539863E+02, 4.583434E+02, 4.627224E+02,
    4.671235E+02, 4.715467E+02, 4.759920E+02, 4.804595E+02, 4.849493E+02, 4.894613E+02,
    4.939958E+02, 4.985526E+02, 5.031320E+02, 5.077338E+02, 5.123582E+02, 5.170052E+02,
    5.216749E+02, 5.263673E+02, 5.310825E+02, 5.358205E+02, 5.405813E+02, 5.453651E+02,
    5.501718E+02, 5.550016E+02, 5.598544E+02, 5.647303E+02, 5.696293E+02, 5.745516E+02,
    5.794971E+02, 5.844659E+02, 5.894580E+02, 5.944735E+02, 5.995124E+02, 6.045748E+02,
    6.096607E+02, 6.147701E+02, 6.199032E+02, 6.250599E+02, 6.302403E+02, 6.354444E+02,
    6.406723E+02, 6.459240E+02, 6.511995E+02, 6.564990E+02, 6.618224E+02, 6.671698E+02,
    6.725412E+02, 6.779366E+02, 6.833562E+02, 6.887999E+02, 6.942678E+02, 6.997600E+02,
    7.052764E+02, 7.108171E+02, 7.163821E+02, 7.219715E+02, 7.275854E+02, 7.332237E+02,
    7.388865E+02, 7.445739E+02, 7.502858E+02, 7.560223E+02, 7.617835E+02, 7.675694E+02,
    7.733800E+02, 7.792153E+02, 7.850755E+02, 7.909605E+02, 7.968704E+02, 8.028052E+02,
    8.087649E+02, 8.147496E+02, 8.207593E+02, 8.267940E+02, 8.328539E+02, 8.389388E+02,
    8.450490E+02, 8.511843E+02, 8.573448E+02, 8.635306E+02, 8.697416E+02, 8.759780E+02,
    8.822397E+02, 8.885269E+02, 8.948394E+02, 9.011774E+02, 9.075409E+02, 9.139299E+02,
    9.203444E+02, 9.267845E+02, 9.332502E+02, 9.397416E+02, 9.462586E+02, 9.528014E+02,
    9.593698E+02, 9.659640E+02, 9.725841E+02, 9.792299E+02, 9.859015E+02, 9.925991E+02,
    9.993225E+02, 1.006072E+03, 1.012847E+03, 1.019649E+03, 1.026476E+03, 1.033329E+03,
    1.040209E+03, 1.047114E+03, 1.054046E+03, 1.061004E+03, 1.067988E+03, 1.074998E+03,
    1.082034E+03, 1.089097E+03, 1.096186E+03, 1.103301E+03, 1.110443E+03, 1.117611E+03,
    1.124805E+03, 1.132026E+03, 1.139273E+03, 1.146546E+03, 1.153846E+03, 1.161173E+03,
    1.168525E+03, 1.175905E+03, 1.183311E+03, 1.190743E+03, 1.198203E+03, 1.205688E+03,
    1.213201E+03, 1.220740E+03, 1.228306E+03, 1.235898E+03, 1.243517E+03, 1.251163E+03,
    1.258835E+03, 1.266535E+03, 1.274261E+03, 1.282013E+03, 1.289793E+03, 1.297600E+03,
    1.305433E+03, 1.313293E+03, 1.321180E+03, 1.329094E+03, 1.337035E+03, 1.345003E+03,
    1.352997E+03, 1.361019E+03, 1.369067E+03, 1.377143E+03, 1.385245E+03, 1.393375E+03,
    1.401531E+03, 1.409715E+03, 1.417925E+03, 1.426163E+03, 1.434428E+03, 1.442719E+03,
    1.451038E+03,
])

#  ---------------------- M = 14, I = 2 ---------------------------
M = 14
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.000000E+00, 1.005366E+01, 1.753287E+01, 2.515838E+01, 3.281701E+01, 4.048954E+01,
    4.816994E+01, 5.585570E+01, 6.354558E+01, 7.123892E+01, 7.893531E+01, 8.663450E+01,
    9.433634E+01, 1.020407E+02, 1.097475E+02, 1.174568E+02, 1.251685E+02, 1.328827E+02,
    1.405995E+02, 1.483191E+02, 1.560419E+02, 1.637681E+02, 1.714984E+02, 1.792334E+02,
    1.869738E+02, 1.947206E+02, 2.024749E+02, 2.102378E+02, 2.180106E+02, 2.257947E+02,
    2.335917E+02, 2.414030E+02, 2.492305E+02, 2.570757E+02, 2.649405E+02, 2.728267E+02,
    2.807361E+02, 2.886704E+02, 2.966316E+02, 3.046215E+02, 3.126418E+02, 3.206944E+02,
    3.287810E+02, 3.369034E+02, 3.450632E+02, 3.532621E+02, 3.615017E+02, 3.697836E+02,
    3.781092E+02, 3.864802E+02, 3.948978E+02, 4.033635E+02, 4.118787E+02, 4.204445E+02,
    4.290623E+02, 4.377333E+02, 4.464585E+02, 4.552392E+02, 4.640764E+02, 4.729710E+02,
    4.819242E+02, 4.909369E+02, 5.000099E+02, 5.091442E+02, 5.183406E+02, 5.276000E+02,
    5.369230E+02, 5.463105E+02, 5.557632E+02, 5.652818E+02, 5.748670E+02, 5.845194E+02,
    5.942396E+02, 6.040283E+02, 6.138860E+02, 6.238133E+02, 6.338107E+02, 6.438787E+02,
    6.540179E+02, 6.642288E+02, 6.745117E+02, 6.848672E+02, 6.952957E+02, 7.057977E+02,
    7.163734E+02, 7.270235E+02, 7.377481E+02, 7.485477E+02, 7.594228E+02, 7.703735E+02,
    7.814004E+02, 7.925036E+02, 8.036835E+02, 8.149406E+02, 8.262749E+02, 8.376870E+02,
    8.491770E+02, 8.607452E+02, 8.723920E+02, 8.841176E+02, 8.959222E+02, 9.078062E+02,
    9.197698E+02, 9.318132E+02, 9.439367E+02, 9.561405E+02, 9.684249E+02, 9.807901E+02,
    9.932363E+02, 1.005764E+03, 1.018373E+03, 1.031063E+03, 1.043836E+03, 1.056690E+03,
    1.069627E+03, 1.082647E+03, 1.095749E+03, 1.108934E+03, 1.122202E+03, 1.135553E+03,
    1.148988E+03, 1.162506E+03, 1.176109E+03, 1.189795E+03, 1.203566E+03, 1.217421E+03,
    1.231360E+03, 1.245384E+03, 1.259493E+03, 1.273688E+03, 1.287967E+03, 1.302332E+03,
    1.316782E+03, 1.331318E+03, 1.345940E+03, 1.360648E+03, 1.375442E+03, 1.390322E+03,
    1.405289E+03, 1.420342E+03, 1.435482E+03, 1.450709E+03, 1.466023E+03, 1.481425E+03,
    1.496913E+03, 1.512490E+03, 1.528153E+03, 1.543905E+03, 1.559744E+03, 1.575672E+03,
    1.591687E+03, 1.607791E+03, 1.623983E+03, 1.640264E+03, 1.656633E+03, 1.673092E+03,
    1.689639E+03, 1.706275E+03, 1.723000E+03, 1.739815E+03, 1.756718E+03, 1.773712E+03,
    1.790795E+03, 1.807968E+03, 1.825230E+03, 1.842583E+03, 1.860025E+03, 1.877558E+03,
    1.895181E+03, 1.912894E+03, 1.930698E+03, 1.948593E+03, 1.966578E+03, 1.984654E+03,
    2.002820E+03, 2.021078E+03, 2.039427E+03, 2.057867E+03, 2.076398E+03, 2.095020E+03,
    2.113734E+03, 2.132540E+03, 2.151437E+03, 2.170425E+03, 2.189506E+03, 2.208678E+03,
    2.227942E+03, 2.247298E+03, 2.266747E+03, 2.286287E+03, 2.305920E+03, 2.325645E+03,
    2.345462E+03, 2.365371E+03, 2.385374E+03, 2.405468E+03, 2.425656E+03, 2.445936E+03,
    2.466309E+03, 2.486774E+03, 2.507333E+03, 2.527984E+03, 2.548729E+03, 2.569566E+03,
    2.590496E+03, 2.611520E+03, 2.632637E+03, 2.653847E+03, 2.675150E+03, 2.696547E+03,
    2.718037E+03, 2.739621E+03, 2.761298E+03, 2.783068E+03, 2.804932E+03, 2.826890E+03,
    2.848941E+03, 2.871086E+03, 2.893324E+03, 2.915656E+03, 2.938082E+03, 2.960602E+03,
    2.983215E+03, 3.005923E+03, 3.028724E+03, 3.051619E+03, 3.074608E+03, 3.097690E+03,
    3.120867E+03, 3.144138E+03, 3.167502E+03, 3.190961E+03, 3.214513E+03, 3.238160E+03,
    3.261900E+03, 3.285735E+03, 3.309663E+03, 3.333686E+03, 3.357802E+03, 3.382013E+03,
    3.406318E+03, 3.430716E+03, 3.455209E+03, 3.479796E+03, 3.504477E+03, 3.529252E+03,
    3.554120E+03, 3.579083E+03, 3.604140E+03, 3.629291E+03, 3.654536E+03, 3.679875E+03,
    3.705308E+03, 3.730835E+03, 3.756455E+03, 3.782170E+03, 3.807979E+03, 3.833881E+03,
    3.859877E+03, 3.885968E+03, 3.912152E+03, 3.938429E+03, 3.964801E+03, 3.991266E+03,
    4.017826E+03, 4.044478E+03, 4.071225E+03, 4.098065E+03, 4.124999E+03, 4.152026E+03,
    4.179147E+03, 4.206362E+03, 4.233670E+03, 4.261071E+03, 4.288566E+03, 4.316154E+03,
    4.343836E+03, 4.371610E+03, 4.399478E+03, 4.427440E+03, 4.455494E+03, 4.483642E+03,
    4.511883E+03, 4.540216E+03, 4.568643E+03, 4.597163E+03, 4.625776E+03, 4.654481E+03,
    4.683279E+03, 4.712171E+03, 4.741154E+03, 4.770231E+03, 4.799400E+03, 4.828661E+03,
    4.858015E+03, 4.887462E+03, 4.917001E+03, 4.946632E+03, 4.976356E+03, 5.006171E+03,
    5.036079E+03,
])

#  ---------------------- M = 15, I = 1 ---------------------------
M = 15
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.000000E+00, 1.379368E+01, 2.419193E+01, 3.477493E+01, 4.540023E+01, 5.604342E+01,
    6.669682E+01, 7.735721E+01, 8.802303E+01, 9.869341E+01, 1.093679E+02, 1.200460E+02,
    1.307278E+02, 1.414129E+02, 1.521013E+02, 1.627930E+02, 1.734880E+02, 1.841863E+02,
    1.948883E+02, 2.055941E+02, 2.163042E+02, 2.270192E+02, 2.377399E+02, 2.484672E+02,
    2.592022E+02, 2.699463E+02, 2.807009E+02, 2.914679E+02, 3.022490E+02, 3.130462E+02,
    3.238617E+02, 3.346978E+02, 3.455568E+02, 3.564412E+02, 3.673535E+02, 3.782961E+02,
    3.892717E+02, 4.002829E+02, 4.113322E+02, 4.224223E+02, 4.335555E+02, 4.447346E+02,
    4.559619E+02, 4.672398E+02, 4.785708E+02, 4.899571E+02, 5.014011E+02, 5.129049E+02,
    5.244706E+02, 5.361002E+02, 5.477959E+02, 5.595596E+02, 5.713930E+02, 5.832980E+02,
    5.952764E+02, 6.073297E+02, 6.194597E+02, 6.316679E+02, 6.439557E+02, 6.563247E+02,
    6.687761E+02, 6.813114E+02, 6.939317E+02, 7.066385E+02, 7.194327E+02, 7.323156E+02,
    7.452883E+02, 7.583518E+02, 7.715072E+02, 7.847553E+02, 7.980973E+02, 8.115339E+02,
    8.250660E+02, 8.386946E+02, 8.524203E+02, 8.662441E+02, 8.801667E+02, 8.941887E+02,
    9.083110E+02, 9.225341E+02, 9.368588E+02, 9.512857E+02, 9.658154E+02, 9.804486E+02,
    9.951858E+02, 1.010028E+03, 1.024974E+03, 1.040027E+03, 1.055186E+03, 1.070451E+03,
    1.085824E+03, 1.101304E+03, 1.116893E+03, 1.132590E+03, 1.148396E+03, 1.164312E+03,
    1.180338E+03, 1.196474E+03, 1.212720E+03, 1.229078E+03, 1.245547E+03, 1.262129E+03,
    1.278822E+03, 1.295628E+03, 1.312548E+03, 1.329580E+03, 1.346727E+03, 1.363987E+03,
    1.381362E+03, 1.398851E+03, 1.416456E+03, 1.434176E+03, 1.452011E+03, 1.469963E+03,
    1.488031E+03, 1.506216E+03, 1.524517E+03, 1.542936E+03, 1.561472E+03, 1.580126E+03,
    1.598899E+03, 1.617789E+03, 1.636798E+03, 1.655927E+03, 1.675174E+03, 1.694541E+03,
    1.714027E+03, 1.733634E+03, 1.753361E+03, 1.773209E+03, 1.793177E+03, 1.813266E+03,
    1.833477E+03, 1.853809E+03, 1.874263E+03, 1.894840E+03, 1.915538E+03, 1.936359E+03,
    1.957303E+03, 1.978370E+03, 1.999561E+03, 2.020875E+03, 2.042312E+03, 2.063874E+03,
    2.085560E+03, 2.107371E+03, 2.129306E+03, 2.151366E+03, 2.173551E+03, 2.195862E+03,
    2.218299E+03, 2.240861E+03, 2.263549E+03, 2.286364E+03, 2.309305E+03, 2.332374E+03,
    2.355569E+03, 2.378891E+03, 2.402340E+03, 2.425918E+03, 2.449623E+03, 2.473456E+03,
    2.497418E+03, 2.521507E+03, 2.545726E+03, 2.570074E+03, 2.594550E+03, 2.619156E+03,
    2.643891E+03, 2.668756E+03, 2.693751E+03, 2.718877E+03, 2.744132E+03, 2.769518E+03,
    2.795034E+03, 2.820682E+03, 2.846460E+03, 2.872370E+03, 2.898411E+03, 2.924584E+03,
    2.950888E+03, 2.977325E+03, 3.003893E+03, 3.030594E+03, 3.057428E+03, 3.084394E+03,
    3.111494E+03, 3.138726E+03, 3.166091E+03, 3.193590E+03, 3.221223E+03, 3.248989E+03,
    3.276889E+03, 3.304923E+03, 3.333091E+03, 3.361394E+03, 3.389831E+03, 3.418403E+03,
    3.447110E+03, 3.475952E+03, 3.504929E+03, 3.534041E+03, 3.563288E+03, 3.592672E+03,
    3.622191E+03, 3.651846E+03, 3.681637E+03, 3.711564E+03, 3.741627E+03, 3.771827E+03,
    3.802164E+03, 3.832637E+03, 3.863247E+03, 3.893994E+03, 3.924878E+03, 3.955899E+03,
    3.987057E+03, 4.018353E+03, 4.049787E+03, 4.081358E+03, 4.113067E+03, 4.144914E+03,
    4.176899E+03, 4.209023E+03, 4.241284E+03, 4.273684E+03, 4.306222E+03, 4.338898E+03,
    4.371714E+03, 4.404668E+03, 4.437761E+03, 4.470993E+03, 4.504364E+03, 4.537874E+03,
    4.571523E+03, 4.605311E+03, 4.639239E+03, 4.673306E+03, 4.707513E+03, 4.741860E+03,
    4.776346E+03, 4.810971E+03, 4.845737E+03, 4.880642E+03, 4.915688E+03, 4.950873E+03,
    4.986199E+03, 5.021664E+03, 5.057270E+03, 5.093016E+03, 5.128903E+03, 5.164930E+03,
    5.201097E+03, 5.237404E+03, 5.273852E+03, 5.310441E+03, 5.347170E+03, 5.384040E+03,
    5.421050E+03, 5.458201E+03, 5.495493E+03, 5.532926E+03, 5.570499E+03, 5.608213E+03,
    5.646068E+03, 5.684064E+03, 5.722201E+03, 5.760479E+03, 5.798897E+03, 5.837457E+03,
    5.876157E+03, 5.914998E+03, 5.953981E+03, 5.993104E+03, 6.032368E+03, 6.071773E+03,
    6.111319E+03, 6.151006E+03, 6.190834E+03, 6.230803E+03, 6.270913E+03, 6.311164E+03,
    6.351555E+03, 6.392088E+03, 6.432761E+03, 6.473575E+03, 6.514531E+03, 6.555626E+03,
    6.596863E+03, 6.638241E+03, 6.679759E+03, 6.721418E+03, 6.763217E+03, 6.805157E+03,
    6.847238E+03, 6.889459E+03, 6.931821E+03, 6.974323E+03, 7.016966E+03, 7.059749E+03,
    7.102672E+03,
])

#  ---------------------- M = 15, I = 2 ---------------------------
M = 15
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.000000E+00, 1.380884E+01, 2.422364E+01, 3.482280E+01, 4.546420E+01, 5.612347E+01,
    6.679294E+01, 7.746940E+01, 8.815129E+01, 9.883774E+01, 1.095283E+02, 1.202225E+02,
    1.309203E+02, 1.416215E+02, 1.523260E+02, 1.630338E+02, 1.737449E+02, 1.844594E+02,
    1.951774E+02, 2.058993E+02, 2.166256E+02, 2.273568E+02, 2.380936E+02, 2.488371E+02,
    2.595884E+02, 2.703487E+02, 2.811197E+02, 2.919031E+02, 3.027006E+02, 3.135144E+02,
    3.243465E+02, 3.351994E+02, 3.460752E+02, 3.569765E+02, 3.679058E+02, 3.788657E+02,
    3.898586E+02, 4.008872E+02, 4.119542E+02, 4.230620E+02, 4.342132E+02, 4.454103E+02,
    4.566559E+02, 4.679523E+02, 4.793019E+02, 4.907071E+02, 5.021701E+02, 5.136932E+02,
    5.252783E+02, 5.369277E+02, 5.486432E+02, 5.604270E+02, 5.722807E+02, 5.842062E+02,
    5.962054E+02, 6.082797E+02, 6.204309E+02, 6.326605E+02, 6.449701E+02, 6.573609E+02,
    6.698345E+02, 6.823922E+02, 6.950352E+02, 7.077648E+02, 7.205821E+02, 7.334884E+02,
    7.464847E+02, 7.595720E+02, 7.727515E+02, 7.860240E+02, 7.993905E+02, 8.128519E+02,
    8.264092E+02, 8.400631E+02, 8.538144E+02, 8.676641E+02, 8.816127E+02, 8.956611E+02,
    9.098099E+02, 9.240599E+02, 9.384118E+02, 9.528660E+02, 9.674234E+02, 9.820845E+02,
    9.968498E+02, 1.011720E+03, 1.026696E+03, 1.041777E+03, 1.056965E+03, 1.072260E+03,
    1.087662E+03, 1.103173E+03, 1.118791E+03, 1.134519E+03, 1.150356E+03, 1.166303E+03,
    1.182360E+03, 1.198527E+03, 1.214806E+03, 1.231195E+03, 1.247697E+03, 1.264311E+03,
    1.281037E+03, 1.297877E+03, 1.314829E+03, 1.331896E+03, 1.349076E+03, 1.366370E+03,
    1.383780E+03, 1.401304E+03, 1.418943E+03, 1.436699E+03, 1.454570E+03, 1.472557E+03,
    1.490661E+03, 1.508882E+03, 1.527220E+03, 1.545676E+03, 1.564249E+03, 1.582941E+03,
    1.601751E+03, 1.620679E+03, 1.639727E+03, 1.658894E+03, 1.678180E+03, 1.697586E+03,
    1.717112E+03, 1.736758E+03, 1.756525E+03, 1.776412E+03, 1.796421E+03, 1.816551E+03,
    1.836803E+03, 1.857176E+03, 1.877672E+03, 1.898290E+03, 1.919031E+03, 1.939894E+03,
    1.960881E+03, 1.981991E+03, 2.003225E+03, 2.024582E+03, 2.046063E+03, 2.067669E+03,
    2.089399E+03, 2.111255E+03, 2.133235E+03, 2.155340E+03, 2.177571E+03, 2.199928E+03,
    2.222410E+03, 2.245019E+03, 2.267754E+03, 2.290615E+03, 2.313604E+03, 2.336719E+03,
    2.359962E+03, 2.383332E+03, 2.406830E+03, 2.430456E+03, 2.454210E+03, 2.478092E+03,
    2.502103E+03, 2.526243E+03, 2.550512E+03, 2.574909E+03, 2.599436E+03, 2.624093E+03,
    2.648880E+03, 2.673796E+03, 2.698843E+03, 2.724020E+03, 2.749328E+03, 2.774766E+03,
    2.800335E+03, 2.826036E+03, 2.851868E+03, 2.877831E+03, 2.903926E+03, 2.930154E+03,
    2.956513E+03, 2.983004E+03, 3.009628E+03, 3.036385E+03, 3.063274E+03, 3.090296E+03,
    3.117452E+03, 3.144741E+03, 3.172163E+03, 3.199719E+03, 3.227409E+03, 3.255233E+03,
    3.283191E+03, 3.311284E+03, 3.339510E+03, 3.367872E+03, 3.396369E+03, 3.425000E+03,
    3.453767E+03, 3.482669E+03, 3.511706E+03, 3.540879E+03, 3.570188E+03, 3.599632E+03,
    3.629213E+03, 3.658929E+03, 3.688783E+03, 3.718772E+03, 3.748898E+03, 3.779161E+03,
    3.809561E+03, 3.840098E+03, 3.870771E+03, 3.901583E+03, 3.932531E+03, 3.963617E+03,
    3.994841E+03, 4.026202E+03, 4.057701E+03, 4.089338E+03, 4.121113E+03, 4.153027E+03,
    4.185079E+03, 4.217269E+03, 4.249598E+03, 4.282065E+03, 4.314671E+03, 4.347416E+03,
    4.380300E+03, 4.413323E+03, 4.446485E+03, 4.479786E+03, 4.513226E+03, 4.546806E+03,
    4.580526E+03, 4.614385E+03, 4.648383E+03, 4.682522E+03, 4.716800E+03, 4.751218E+03,
    4.785776E+03, 4.820474E+03, 4.855312E+03, 4.890290E+03, 4.925409E+03, 4.960667E+03,
    4.996067E+03, 5.031606E+03, 5.067286E+03, 5.103107E+03, 5.139068E+03, 5.175170E+03,
    5.211412E+03, 5.247795E+03, 5.284319E+03, 5.320984E+03, 5.357790E+03, 5.394736E+03,
    5.431824E+03, 5.469052E+03, 5.506421E+03, 5.543932E+03, 5.581583E+03, 5.619376E+03,
    5.657310E+03, 5.695385E+03, 5.733601E+03, 5.771958E+03, 5.810456E+03, 5.849096E+03,
    5.887876E+03, 5.926798E+03, 5.965862E+03, 6.005066E+03, 6.044412E+03, 6.083898E+03,
    6.123527E+03, 6.163296E+03, 6.203206E+03, 6.243258E+03, 6.283451E+03, 6.323785E+03,
    6.364261E+03, 6.404877E+03, 6.445635E+03, 6.486534E+03, 6.527574E+03, 6.568755E+03,
    6.610077E+03, 6.651540E+03, 6.693144E+03, 6.734889E+03, 6.776775E+03, 6.818802E+03,
    6.860969E+03, 6.903278E+03, 6.945727E+03, 6.988317E+03, 7.031048E+03, 7.073919E+03,
    7.116931E+03,
])

#  ---------------------- M = 15, I = 3 ---------------------------
M = 15
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.200001E+01, 3.527472E+01, 6.604716E+01, 9.694745E+01, 1.278841E+02, 1.588401E+02,
    1.898098E+02, 2.207909E+02, 2.517823E+02, 2.827832E+02, 3.137934E+02, 3.448129E+02,
    3.758418E+02, 4.068811E+02, 4.379322E+02, 4.689977E+02, 5.000812E+02, 5.311880E+02,
    5.623246E+02, 5.934988E+02, 6.247200E+02, 6.559986E+02, 6.873464E+02, 7.187757E+02,
    7.502996E+02, 7.819318E+02, 8.136862E+02, 8.455769E+02, 8.776178E+02, 9.098230E+02,
    9.422062E+02, 9.747808E+02, 1.007560E+03, 1.040555E+03, 1.073780E+03, 1.107246E+03,
    1.140963E+03, 1.174943E+03, 1.209195E+03, 1.243729E+03, 1.278555E+03, 1.313680E+03,
    1.349113E+03, 1.384862E+03, 1.420935E+03, 1.457337E+03, 1.494076E+03, 1.531157E+03,
    1.568588E+03, 1.606372E+03, 1.644516E+03, 1.683025E+03, 1.721902E+03, 1.761153E+03,
    1.800782E+03, 1.840792E+03, 1.881188E+03, 1.921974E+03, 1.963151E+03, 2.004725E+03,
    2.046697E+03, 2.089072E+03, 2.131851E+03, 2.175038E+03, 2.218634E+03, 2.262643E+03,
    2.307067E+03, 2.351908E+03, 2.397169E+03, 2.442851E+03, 2.488956E+03, 2.535486E+03,
    2.582444E+03, 2.629831E+03, 2.677648E+03, 2.725898E+03, 2.774582E+03, 2.823702E+03,
    2.873259E+03, 2.923254E+03, 2.973690E+03, 3.024567E+03, 3.075888E+03, 3.127652E+03,
    3.179862E+03, 3.232519E+03, 3.285625E+03, 3.339179E+03, 3.393184E+03, 3.447641E+03,
    3.502550E+03, 3.557914E+03, 3.613733E+03, 3.670008E+03, 3.726740E+03, 3.783930E+03,
    3.841580E+03, 3.899690E+03, 3.958261E+03, 4.017295E+03, 4.076791E+03, 4.136753E+03,
    4.197179E+03, 4.258071E+03, 4.319431E+03, 4.381258E+03, 4.443554E+03, 4.506320E+03,
    4.569557E+03, 4.633264E+03, 4.697444E+03, 4.762098E+03, 4.827225E+03, 4.892827E+03,
    4.958904E+03, 5.025457E+03, 5.092488E+03, 5.159997E+03, 5.227984E+03, 5.296451E+03,
    5.365397E+03, 5.434825E+03, 5.504734E+03, 5.575125E+03, 5.646000E+03, 5.717358E+03,
    5.789200E+03, 5.861527E+03, 5.934340E+03, 6.007640E+03, 6.081426E+03, 6.155700E+03,
    6.230462E+03, 6.305713E+03, 6.381453E+03, 6.457684E+03, 6.534405E+03, 6.611617E+03,
    6.689320E+03, 6.767516E+03, 6.846205E+03, 6.925387E+03, 7.005063E+03, 7.085233E+03,
    7.165898E+03, 7.247058E+03, 7.328713E+03, 7.410865E+03, 7.493514E+03, 7.576660E+03,
    7.660303E+03, 7.744444E+03, 7.829083E+03, 7.914221E+03, 7.999858E+03, 8.085994E+03,
    8.172630E+03, 8.259766E+03, 8.347402E+03, 8.435540E+03, 8.524178E+03, 8.613317E+03,
    8.702958E+03, 8.793101E+03, 8.883746E+03, 8.974893E+03, 9.066542E+03, 9.158695E+03,
    9.251350E+03, 9.344509E+03, 9.438170E+03, 9.532336E+03, 9.627005E+03, 9.722177E+03,
    9.817854E+03, 9.914034E+03, 1.001072E+04, 1.010791E+04, 1.020560E+04, 1.030380E+04,
    1.040250E+04, 1.050171E+04, 1.060142E+04, 1.070163E+04, 1.080235E+04, 1.090358E+04,
    1.100530E+04, 1.110754E+04, 1.121027E+04, 1.131352E+04, 1.141726E+04, 1.152151E+04,
    1.162627E+04, 1.173152E+04, 1.183728E+04, 1.194355E+04, 1.205032E+04, 1.215759E+04,
    1.226537E+04, 1.237364E+04, 1.248242E+04, 1.259171E+04, 1.270149E+04, 1.281178E+04,
    1.292257E+04, 1.303386E+04, 1.314566E+04, 1.325795E+04, 1.337075E+04, 1.348404E+04,
    1.359784E+04, 1.371213E+04, 1.382693E+04, 1.394222E+04, 1.405802E+04, 1.417431E+04,
    1.429110E+04, 1.440838E+04, 1.452617E+04, 1.464445E+04, 1.476323E+04, 1.488250E+04,
    1.500227E+04, 1.512254E+04, 1.524329E+04, 1.536455E+04, 1.548629E+04, 1.560853E+04,
    1.573127E+04, 1.585449E+04, 1.597821E+04, 1.610241E+04, 1.622711E+04, 1.635230E+04,
    1.647798E+04, 1.660414E+04, 1.673080E+04, 1.685794E+04, 1.698556E+04, 1.711368E+04,
    1.724228E+04, 1.737136E+04, 1.750093E+04, 1.763099E+04, 1.776152E+04, 1.789254E+04,
    1.802404E+04, 1.815602E+04, 1.828848E+04, 1.842142E+04, 1.855484E+04, 1.868874E+04,
    1.882311E+04, 1.895796E+04, 1.909329E+04, 1.922909E+04, 1.936536E+04, 1.950211E+04,
    1.963933E+04, 1.977703E+04, 1.991519E+04, 2.005382E+04, 2.019293E+04, 2.033250E+04,
    2.047254E+04, 2.061304E+04, 2.075401E+04, 2.089545E+04, 2.103735E+04, 2.117971E+04,
    2.132254E+04, 2.146582E+04, 2.160957E+04, 2.175378E+04, 2.189844E+04, 2.204356E+04,
    2.218914E+04, 2.233518E+04, 2.248166E+04, 2.262861E+04, 2.277600E+04, 2.292385E+04,
    2.307215E+04, 2.322090E+04, 2.337009E+04, 2.351974E+04, 2.366983E+04, 2.382036E+04,
    2.397134E+04, 2.412277E+04, 2.427464E+04, 2.442694E+04, 2.457969E+04, 2.473288E+04,
    2.488651E+04, 2.504057E+04, 2.519507E+04, 2.535001E+04, 2.550537E+04, 2.566118E+04,
    2.581741E+04,
])

#  ---------------------- M = 15, I = 4 ---------------------------
M = 15
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.200001E+01, 3.536447E+01, 6.622832E+01, 9.721965E+01, 1.282473E+02, 1.592942E+02,
    1.903549E+02, 2.214270E+02, 2.525093E+02, 2.836013E+02, 3.147026E+02, 3.458131E+02,
    3.769332E+02, 4.080636E+02, 4.392059E+02, 4.703627E+02, 5.015378E+02, 5.327363E+02,
    5.639649E+02, 5.952315E+02, 6.265456E+02, 6.579177E+02, 6.893595E+02, 7.208836E+02,
    7.525032E+02, 7.842320E+02, 8.160840E+02, 8.480734E+02, 8.802143E+02, 9.125207E+02,
    9.450063E+02, 9.776848E+02, 1.010569E+03, 1.043672E+03, 1.077005E+03, 1.110580E+03,
    1.144409E+03, 1.178502E+03, 1.212869E+03, 1.247520E+03, 1.282464E+03, 1.317709E+03,
    1.353264E+03, 1.389137E+03, 1.425335E+03, 1.461864E+03, 1.498732E+03, 1.535944E+03,
    1.573507E+03, 1.611425E+03, 1.649705E+03, 1.688351E+03, 1.727367E+03, 1.766759E+03,
    1.806529E+03, 1.846683E+03, 1.887223E+03, 1.928154E+03, 1.969478E+03, 2.011199E+03,
    2.053320E+03, 2.095842E+03, 2.138770E+03, 2.182105E+03, 2.225850E+03, 2.270007E+03,
    2.314578E+03, 2.359564E+03, 2.404969E+03, 2.450792E+03, 2.497037E+03, 2.543704E+03,
    2.590795E+03, 2.638311E+03, 2.686254E+03, 2.734623E+03, 2.783422E+03, 2.832649E+03,
    2.882307E+03, 2.932396E+03, 2.982916E+03, 3.033868E+03, 3.085254E+03, 3.137072E+03,
    3.189325E+03, 3.242011E+03, 3.295131E+03, 3.348686E+03, 3.402676E+03, 3.457100E+03,
    3.511959E+03, 3.567253E+03, 3.622981E+03, 3.679143E+03, 3.735740E+03, 3.792771E+03,
    3.850235E+03, 3.908133E+03, 3.966464E+03, 4.025226E+03, 4.084421E+03, 4.144048E+03,
    4.204105E+03, 4.264592E+03, 4.325508E+03, 4.386854E+03, 4.448627E+03, 4.510828E+03,
    4.573454E+03, 4.636507E+03, 4.699984E+03, 4.763884E+03, 4.828207E+03, 4.892952E+03,
    4.958117E+03, 5.023701E+03, 5.089704E+03, 5.156124E+03, 5.222959E+03, 5.290210E+03,
    5.357874E+03, 5.425950E+03, 5.494437E+03, 5.563333E+03, 5.632638E+03, 5.702349E+03,
    5.772466E+03, 5.842987E+03, 5.913910E+03, 5.985234E+03, 6.056957E+03, 6.129079E+03,
    6.201596E+03, 6.274509E+03, 6.347814E+03, 6.421511E+03, 6.495598E+03, 6.570073E+03,
    6.644935E+03, 6.720182E+03, 6.795811E+03, 6.871822E+03, 6.948213E+03, 7.024981E+03,
    7.102125E+03, 7.179643E+03, 7.257534E+03, 7.335796E+03, 7.414426E+03, 7.493422E+03,
    7.572784E+03, 7.652509E+03, 7.732594E+03, 7.813039E+03, 7.893841E+03, 7.974999E+03,
    8.056509E+03, 8.138371E+03, 8.220583E+03, 8.303141E+03, 8.386045E+03, 8.469292E+03,
    8.552881E+03, 8.636808E+03, 8.721073E+03, 8.805673E+03, 8.890606E+03, 8.975870E+03,
    9.061463E+03, 9.147383E+03, 9.233628E+03, 9.320195E+03, 9.407082E+03, 9.494288E+03,
    9.581810E+03, 9.669647E+03, 9.757795E+03, 9.846253E+03, 9.935019E+03, 1.002409E+04,
    1.011347E+04, 1.020314E+04, 1.029312E+04, 1.038339E+04, 1.047396E+04, 1.056482E+04,
    1.065597E+04, 1.074740E+04, 1.083913E+04, 1.093114E+04, 1.102343E+04, 1.111600E+04,
    1.120884E+04, 1.130197E+04, 1.139536E+04, 1.148903E+04, 1.158296E+04, 1.167716E+04,
    1.177163E+04, 1.186635E+04, 1.196134E+04, 1.205658E+04, 1.215208E+04, 1.224784E+04,
    1.234384E+04, 1.244010E+04, 1.253660E+04, 1.263334E+04, 1.273033E+04, 1.282756E+04,
    1.292502E+04, 1.302272E+04, 1.312066E+04, 1.321883E+04, 1.331722E+04, 1.341585E+04,
    1.351469E+04, 1.361377E+04, 1.371306E+04, 1.381257E+04, 1.391230E+04, 1.401224E+04,
    1.411240E+04, 1.421276E+04, 1.431333E+04, 1.441411E+04, 1.451509E+04, 1.461628E+04,
    1.471766E+04, 1.481925E+04, 1.492102E+04, 1.502299E+04, 1.512516E+04, 1.522751E+04,
    1.533005E+04, 1.543277E+04, 1.553568E+04, 1.563877E+04, 1.574203E+04, 1.584548E+04,
    1.594910E+04, 1.605289E+04, 1.615685E+04, 1.626099E+04, 1.636528E+04, 1.646975E+04,
    1.657438E+04, 1.667916E+04, 1.678411E+04, 1.688922E+04, 1.699448E+04, 1.709989E+04,
    1.720545E+04, 1.731117E+04, 1.741703E+04, 1.752304E+04, 1.762919E+04, 1.773548E+04,
    1.784191E+04, 1.794848E+04, 1.805519E+04, 1.816203E+04, 1.826900E+04, 1.837611E+04,
    1.848334E+04, 1.859070E+04, 1.869818E+04, 1.880579E+04, 1.891352E+04, 1.902137E+04,
    1.912934E+04, 1.923742E+04, 1.934562E+04, 1.945393E+04, 1.956235E+04, 1.967088E+04,
    1.977952E+04, 1.988826E+04, 1.999711E+04, 2.010606E+04, 2.021511E+04, 2.032426E+04,
    2.043350E+04, 2.054284E+04, 2.065228E+04, 2.076181E+04, 2.087143E+04, 2.098113E+04,
    2.109093E+04, 2.120081E+04, 2.131078E+04, 2.142082E+04, 2.153095E+04, 2.164116E+04,
    2.175144E+04, 2.186181E+04, 2.197224E+04, 2.208275E+04, 2.219333E+04, 2.230399E+04,
    2.241471E+04,
])

#  ---------------------- M = 16, I = 1 ---------------------------
M = 16
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.000000E+00, 1.634928E+01, 2.947690E+01, 4.274338E+01, 5.604397E+01, 6.935994E+01,
    8.268533E+01, 9.601763E+01, 1.093556E+02, 1.226986E+02, 1.360462E+02, 1.493982E+02,
    1.627544E+02, 1.761147E+02, 1.894792E+02, 2.028480E+02, 2.162214E+02, 2.295997E+02,
    2.429839E+02, 2.563748E+02, 2.697738E+02, 2.831826E+02, 2.966031E+02, 3.100376E+02,
    3.234889E+02, 3.369598E+02, 3.504537E+02, 3.639739E+02, 3.775240E+02, 3.911080E+02,
    4.047298E+02, 4.183933E+02, 4.321026E+02, 4.458619E+02, 4.596752E+02, 4.735466E+02,
    4.874801E+02, 5.014796E+02, 5.155490E+02, 5.296922E+02, 5.439127E+02, 5.582142E+02,
    5.726001E+02, 5.870738E+02, 6.016384E+02, 6.162971E+02, 6.310528E+02, 6.459085E+02,
    6.608669E+02, 6.759306E+02, 6.911022E+02, 7.063841E+02, 7.217786E+02, 7.372879E+02,
    7.529142E+02, 7.686595E+02, 7.845258E+02, 8.005148E+02, 8.166285E+02, 8.328685E+02,
    8.492364E+02, 8.657339E+02, 8.823623E+02, 8.991233E+02, 9.160180E+02, 9.330480E+02,
    9.502144E+02, 9.675184E+02, 9.849613E+02, 1.002544E+03, 1.020268E+03, 1.038134E+03,
    1.056143E+03, 1.074296E+03, 1.092594E+03, 1.111039E+03, 1.129630E+03, 1.148368E+03,
    1.167255E+03, 1.186292E+03, 1.205478E+03, 1.224815E+03, 1.244304E+03, 1.263945E+03,
    1.283738E+03, 1.303686E+03, 1.323787E+03, 1.344044E+03, 1.364456E+03, 1.385024E+03,
    1.405748E+03, 1.426630E+03, 1.447670E+03, 1.468868E+03, 1.490225E+03, 1.511742E+03,
    1.533418E+03, 1.555255E+03, 1.577253E+03, 1.599413E+03, 1.621734E+03, 1.644218E+03,
    1.666865E+03, 1.689675E+03, 1.712649E+03, 1.735787E+03, 1.759090E+03, 1.782559E+03,
    1.806192E+03, 1.829993E+03, 1.853959E+03, 1.878093E+03, 1.902393E+03, 1.926862E+03,
    1.951499E+03, 1.976304E+03, 2.001279E+03, 2.026423E+03, 2.051737E+03, 2.077221E+03,
    2.102875E+03, 2.128701E+03, 2.154698E+03, 2.180867E+03, 2.207208E+03, 2.233722E+03,
    2.260408E+03, 2.287269E+03, 2.314302E+03, 2.341510E+03, 2.368893E+03, 2.396450E+03,
    2.424183E+03, 2.452091E+03, 2.480175E+03, 2.508436E+03, 2.536873E+03, 2.565488E+03,
    2.594280E+03, 2.623250E+03, 2.652398E+03, 2.681725E+03, 2.711231E+03, 2.740916E+03,
    2.770781E+03, 2.800826E+03, 2.831052E+03, 2.861458E+03, 2.892045E+03, 2.922814E+03,
    2.953765E+03, 2.984899E+03, 3.016214E+03, 3.047713E+03, 3.079395E+03, 3.111261E+03,
    3.143311E+03, 3.175545E+03, 3.207964E+03, 3.240568E+03, 3.273358E+03, 3.306334E+03,
    3.339495E+03, 3.372843E+03, 3.406378E+03, 3.440101E+03, 3.474010E+03, 3.508108E+03,
    3.542394E+03, 3.576869E+03, 3.611532E+03, 3.646385E+03, 3.681428E+03, 3.716660E+03,
    3.752083E+03, 3.787696E+03, 3.823500E+03, 3.859496E+03, 3.895683E+03, 3.932063E+03,
    3.968634E+03, 4.005398E+03, 4.042355E+03, 4.079506E+03, 4.116850E+03, 4.154388E+03,
    4.192120E+03, 4.230046E+03, 4.268168E+03, 4.306485E+03, 4.344997E+03, 4.383705E+03,
    4.422609E+03, 4.461709E+03, 4.501007E+03, 4.540501E+03, 4.580192E+03, 4.620081E+03,
    4.660168E+03, 4.700454E+03, 4.740937E+03, 4.781620E+03, 4.822501E+03, 4.863582E+03,
    4.904862E+03, 4.946342E+03, 4.988023E+03, 5.029903E+03, 5.071985E+03, 5.114267E+03,
    5.156751E+03, 5.199436E+03, 5.242323E+03, 5.285411E+03, 5.328702E+03, 5.372196E+03,
    5.415892E+03, 5.459791E+03, 5.503893E+03, 5.548199E+03, 5.592708E+03, 5.637422E+03,
    5.682339E+03, 5.727461E+03, 5.772787E+03, 5.818318E+03, 5.864053E+03, 5.909994E+03,
    5.956141E+03, 6.002493E+03, 6.049051E+03, 6.095814E+03, 6.142784E+03, 6.189960E+03,
    6.237343E+03, 6.284932E+03, 6.332728E+03, 6.380732E+03, 6.428942E+03, 6.477360E+03,
    6.525985E+03, 6.574818E+03, 6.623859E+03, 6.673107E+03, 6.722564E+03, 6.772229E+03,
    6.822102E+03, 6.872184E+03, 6.922475E+03, 6.972974E+03, 7.023682E+03, 7.074599E+03,
    7.125725E+03, 7.177060E+03, 7.228605E+03, 7.280359E+03, 7.332322E+03, 7.384495E+03,
    7.436877E+03, 7.489470E+03, 7.542272E+03, 7.595283E+03, 7.648505E+03, 7.701937E+03,
    7.755579E+03, 7.809431E+03, 7.863493E+03, 7.917765E+03, 7.972247E+03, 8.026940E+03,
    8.081842E+03, 8.136956E+03, 8.192279E+03, 8.247813E+03, 8.303557E+03, 8.359511E+03,
    8.415676E+03, 8.472051E+03, 8.528637E+03, 8.585433E+03, 8.642439E+03, 8.699655E+03,
    8.757082E+03, 8.814719E+03, 8.872566E+03, 8.930624E+03, 8.988891E+03, 9.047369E+03,
    9.106056E+03, 9.164954E+03, 9.224061E+03, 9.283378E+03, 9.342905E+03, 9.402642E+03,
    9.462588E+03, 9.522744E+03, 9.583109E+03, 9.643684E+03, 9.704468E+03, 9.765461E+03,
    9.826663E+03,
])

#  ---------------------- M = 16, I = 2 ---------------------------
M = 16
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.000000E+00, 1.635327E+01, 2.948508E+01, 4.275570E+01, 5.606042E+01, 6.938053E+01,
    8.271004E+01, 9.604648E+01, 1.093886E+02, 1.227357E+02, 1.360875E+02, 1.494436E+02,
    1.628039E+02, 1.761684E+02, 1.895370E+02, 2.029099E+02, 2.162874E+02, 2.296699E+02,
    2.430582E+02, 2.564533E+02, 2.698565E+02, 2.832694E+02, 2.966941E+02, 3.101328E+02,
    3.235883E+02, 3.370634E+02, 3.505615E+02, 3.640860E+02, 3.776405E+02, 3.912288E+02,
    4.048549E+02, 4.185229E+02, 4.322366E+02, 4.460004E+02, 4.598182E+02, 4.736942E+02,
    4.876323E+02, 5.016365E+02, 5.157107E+02, 5.298586E+02, 5.440840E+02, 5.583904E+02,
    5.727812E+02, 5.872599E+02, 6.018296E+02, 6.164934E+02, 6.312544E+02, 6.461153E+02,
    6.610791E+02, 6.761482E+02, 6.913252E+02, 7.066126E+02, 7.220127E+02, 7.375277E+02,
    7.531597E+02, 7.689108E+02, 7.847830E+02, 8.007780E+02, 8.168977E+02, 8.331437E+02,
    8.495178E+02, 8.660215E+02, 8.826562E+02, 8.994235E+02, 9.163248E+02, 9.333612E+02,
    9.505342E+02, 9.678449E+02, 9.852945E+02, 1.002884E+03, 1.020615E+03, 1.038488E+03,
    1.056504E+03, 1.074664E+03, 1.092970E+03, 1.111421E+03, 1.130019E+03, 1.148765E+03,
    1.167660E+03, 1.186704E+03, 1.205898E+03, 1.225242E+03, 1.244739E+03, 1.264388E+03,
    1.284189E+03, 1.304145E+03, 1.324254E+03, 1.344519E+03, 1.364939E+03, 1.385515E+03,
    1.406248E+03, 1.427139E+03, 1.448187E+03, 1.469393E+03, 1.490759E+03, 1.512284E+03,
    1.533969E+03, 1.555815E+03, 1.577822E+03, 1.599990E+03, 1.622320E+03, 1.644813E+03,
    1.667469E+03, 1.690288E+03, 1.713271E+03, 1.736419E+03, 1.759731E+03, 1.783208E+03,
    1.806851E+03, 1.830660E+03, 1.854636E+03, 1.878779E+03, 1.903089E+03, 1.927566E+03,
    1.952212E+03, 1.977026E+03, 2.002010E+03, 2.027162E+03, 2.052485E+03, 2.077977E+03,
    2.103640E+03, 2.129474E+03, 2.155479E+03, 2.181656E+03, 2.208005E+03, 2.234526E+03,
    2.261220E+03, 2.288087E+03, 2.315127E+03, 2.342341E+03, 2.369730E+03, 2.397293E+03,
    2.425030E+03, 2.452943E+03, 2.481032E+03, 2.509296E+03, 2.537737E+03, 2.566354E+03,
    2.595148E+03, 2.624119E+03, 2.653268E+03, 2.682595E+03, 2.712100E+03, 2.741783E+03,
    2.771646E+03, 2.801687E+03, 2.831908E+03, 2.862308E+03, 2.892889E+03, 2.923650E+03,
    2.954592E+03, 2.985715E+03, 3.017019E+03, 3.048505E+03, 3.080172E+03, 3.112022E+03,
    3.144054E+03, 3.176269E+03, 3.208667E+03, 3.241248E+03, 3.274013E+03, 3.306962E+03,
    3.340095E+03, 3.373412E+03, 3.406913E+03, 3.440600E+03, 3.474472E+03, 3.508529E+03,
    3.542772E+03, 3.577201E+03, 3.611816E+03, 3.646618E+03, 3.681606E+03, 3.716781E+03,
    3.752144E+03, 3.787693E+03, 3.823430E+03, 3.859355E+03, 3.895469E+03, 3.931770E+03,
    3.968260E+03, 4.004938E+03, 4.041806E+03, 4.078862E+03, 4.116108E+03, 4.153544E+03,
    4.191169E+03, 4.228984E+03, 4.266989E+03, 4.305184E+03, 4.343570E+03, 4.382146E+03,
    4.420914E+03, 4.459872E+03, 4.499021E+03, 4.538362E+03, 4.577894E+03, 4.617618E+03,
    4.657534E+03, 4.697641E+03, 4.737941E+03, 4.778433E+03, 4.819117E+03, 4.859994E+03,
    4.901064E+03, 4.942326E+03, 4.983781E+03, 5.025429E+03, 5.067270E+03, 5.109305E+03,
    5.151533E+03, 5.193954E+03, 5.236569E+03, 5.279378E+03, 5.322380E+03, 5.365577E+03,
    5.408967E+03, 5.452551E+03, 5.496329E+03, 5.540302E+03, 5.584469E+03, 5.628830E+03,
    5.673385E+03, 5.718135E+03, 5.763080E+03, 5.808219E+03, 5.853552E+03, 5.899081E+03,
    5.944804E+03, 5.990722E+03, 6.036834E+03, 6.083141E+03, 6.129644E+03, 6.176341E+03,
    6.223233E+03, 6.270319E+03, 6.317601E+03, 6.365078E+03, 6.412749E+03, 6.460616E+03,
    6.508677E+03, 6.556933E+03, 6.605384E+03, 6.654030E+03, 6.702871E+03, 6.751907E+03,
    6.801137E+03, 6.850563E+03, 6.900183E+03, 6.949997E+03, 7.000007E+03, 7.050211E+03,
    7.100609E+03, 7.151202E+03, 7.201990E+03, 7.252972E+03, 7.304148E+03, 7.355518E+03,
    7.407083E+03, 7.458842E+03, 7.510794E+03, 7.562941E+03, 7.615282E+03, 7.667816E+03,
    7.720544E+03, 7.773466E+03, 7.826581E+03, 7.879889E+03, 7.933391E+03, 7.987085E+03,
    8.040973E+03, 8.095054E+03, 8.149327E+03, 8.203793E+03, 8.258452E+03, 8.313303E+03,
    8.368346E+03, 8.423581E+03, 8.479008E+03, 8.534627E+03, 8.590438E+03, 8.646440E+03,
    8.702634E+03, 8.759019E+03, 8.815594E+03, 8.872361E+03, 8.929318E+03, 8.986466E+03,
    9.043803E+03, 9.101331E+03, 9.159049E+03, 9.216957E+03, 9.275054E+03, 9.333341E+03,
    9.391817E+03, 9.450481E+03, 9.509335E+03, 9.568377E+03, 9.627607E+03, 9.687025E+03,
    9.746631E+03,
])

#  ---------------------- M = 16, I = 3 ---------------------------
M = 16
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.200018E+01, 4.353101E+01, 8.267877E+01, 1.219284E+02, 1.612106E+02, 2.005120E+02,
    2.398284E+02, 2.791580E+02, 3.184998E+02, 3.578535E+02, 3.972192E+02, 4.365975E+02,
    4.759903E+02, 5.154010E+02, 5.548351E+02, 5.943007E+02, 6.338083E+02, 6.733713E+02,
    7.130053E+02, 7.527282E+02, 7.925598E+02, 8.325212E+02, 8.726345E+02, 9.129224E+02,
    9.534080E+02, 9.941141E+02, 1.035064E+03, 1.076278E+03, 1.117780E+03, 1.159590E+03,
    1.201727E+03, 1.244211E+03, 1.287059E+03, 1.330289E+03, 1.373916E+03, 1.417957E+03,
    1.462425E+03, 1.507334E+03, 1.552696E+03, 1.598524E+03, 1.644828E+03, 1.691619E+03,
    1.738907E+03, 1.786701E+03, 1.835009E+03, 1.883840E+03, 1.933201E+03, 1.983100E+03,
    2.033542E+03, 2.084536E+03, 2.136085E+03, 2.188197E+03, 2.240876E+03, 2.294127E+03,
    2.347956E+03, 2.402366E+03, 2.457362E+03, 2.512948E+03, 2.569127E+03, 2.625905E+03,
    2.683283E+03, 2.741265E+03, 2.799856E+03, 2.859056E+03, 2.918871E+03, 2.979301E+03,
    3.040351E+03, 3.102023E+03, 3.164318E+03, 3.227241E+03, 3.290792E+03, 3.354975E+03,
    3.419791E+03, 3.485243E+03, 3.551333E+03, 3.618062E+03, 3.685434E+03, 3.753448E+03,
    3.822109E+03, 3.891417E+03, 3.961373E+03, 4.031981E+03, 4.103242E+03, 4.175156E+03,
    4.247727E+03, 4.320955E+03, 4.394842E+03, 4.469390E+03, 4.544599E+03, 4.620473E+03,
    4.697012E+03, 4.774217E+03, 4.852090E+03, 4.930633E+03, 5.009846E+03, 5.089732E+03,
    5.170291E+03, 5.251524E+03, 5.333434E+03, 5.416021E+03, 5.499287E+03, 5.583232E+03,
    5.667858E+03, 5.753167E+03, 5.839159E+03, 5.925836E+03, 6.013198E+03, 6.101247E+03,
    6.189984E+03, 6.279409E+03, 6.369525E+03, 6.460332E+03, 6.551831E+03, 6.644024E+03,
    6.736910E+03, 6.830491E+03, 6.924769E+03, 7.019743E+03, 7.115415E+03, 7.211786E+03,
    7.308857E+03, 7.406628E+03, 7.505101E+03, 7.604275E+03, 7.704153E+03, 7.804734E+03,
    7.906020E+03, 8.008011E+03, 8.110707E+03, 8.214111E+03, 8.318221E+03, 8.423040E+03,
    8.528567E+03, 8.634803E+03, 8.741749E+03, 8.849406E+03, 8.957773E+03, 9.066852E+03,
    9.176642E+03, 9.287145E+03, 9.398361E+03, 9.510290E+03, 9.622933E+03, 9.736290E+03,
    9.850361E+03, 9.965148E+03, 1.008065E+04, 1.019687E+04, 1.031380E+04, 1.043145E+04,
    1.054981E+04, 1.066889E+04, 1.078869E+04, 1.090921E+04, 1.103044E+04, 1.115239E+04,
    1.127506E+04, 1.139844E+04, 1.152254E+04, 1.164736E+04, 1.177290E+04, 1.189916E+04,
    1.202613E+04, 1.215382E+04, 1.228223E+04, 1.241135E+04, 1.254120E+04, 1.267176E+04,
    1.280303E+04, 1.293503E+04, 1.306774E+04, 1.320117E+04, 1.333531E+04, 1.347018E+04,
    1.360575E+04, 1.374205E+04, 1.387906E+04, 1.401678E+04, 1.415522E+04, 1.429438E+04,
    1.443424E+04, 1.457483E+04, 1.471612E+04, 1.485813E+04, 1.500085E+04, 1.514429E+04,
    1.528843E+04, 1.543329E+04, 1.557886E+04, 1.572514E+04, 1.587213E+04, 1.601983E+04,
    1.616823E+04, 1.631735E+04, 1.646717E+04, 1.661770E+04, 1.676894E+04, 1.692088E+04,
    1.707353E+04, 1.722688E+04, 1.738094E+04, 1.753570E+04, 1.769116E+04, 1.784733E+04,
    1.800419E+04, 1.816176E+04, 1.832002E+04, 1.847899E+04, 1.863865E+04, 1.879901E+04,
    1.896006E+04, 1.912181E+04, 1.928426E+04, 1.944740E+04, 1.961123E+04, 1.977575E+04,
    1.994097E+04, 2.010688E+04, 2.027347E+04, 2.044076E+04, 2.060873E+04, 2.077739E+04,
    2.094673E+04, 2.111676E+04, 2.128747E+04, 2.145887E+04, 2.163095E+04, 2.180371E+04,
    2.197714E+04, 2.215126E+04, 2.232606E+04, 2.250153E+04, 2.267767E+04, 2.285449E+04,
    2.303199E+04, 2.321016E+04, 2.338899E+04, 2.356850E+04, 2.374868E+04, 2.392952E+04,
    2.411104E+04, 2.429321E+04, 2.447606E+04, 2.465956E+04, 2.484373E+04, 2.502856E+04,
    2.521404E+04, 2.540019E+04, 2.558699E+04, 2.577445E+04, 2.596257E+04, 2.615133E+04,
    2.634075E+04, 2.653083E+04, 2.672155E+04, 2.691292E+04, 2.710493E+04, 2.729760E+04,
    2.749091E+04, 2.768486E+04, 2.787945E+04, 2.807469E+04, 2.827056E+04, 2.846707E+04,
    2.866422E+04, 2.886201E+04, 2.906042E+04, 2.925948E+04, 2.945916E+04, 2.965947E+04,
    2.986042E+04, 3.006199E+04, 3.026418E+04, 3.046700E+04, 3.067045E+04, 3.087451E+04,
    3.107920E+04, 3.128450E+04, 3.149043E+04, 3.169697E+04, 3.190412E+04, 3.211189E+04,
    3.232027E+04, 3.252925E+04, 3.273885E+04, 3.294906E+04, 3.315987E+04, 3.337129E+04,
    3.358331E+04, 3.379593E+04, 3.400915E+04, 3.422297E+04, 3.443739E+04, 3.465240E+04,
    3.486801E+04, 3.508421E+04, 3.530100E+04, 3.551839E+04, 3.573636E+04, 3.595491E+04,
    3.617405E+04,
])

#  ---------------------- M = 16, I = 4 ---------------------------
M = 16
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.200018E+01, 4.355487E+01, 8.272675E+01, 1.220005E+02, 1.613067E+02, 2.006322E+02,
    2.399726E+02, 2.793263E+02, 3.186922E+02, 3.580700E+02, 3.974597E+02, 4.368621E+02,
    4.762790E+02, 5.157139E+02, 5.551722E+02, 5.946621E+02, 6.341940E+02, 6.737815E+02,
    7.134401E+02, 7.531878E+02, 7.930444E+02, 8.330310E+02, 8.731698E+02, 9.134835E+02,
    9.539953E+02, 9.947280E+02, 1.035704E+03, 1.076947E+03, 1.118476E+03, 1.160314E+03,
    1.202480E+03, 1.244992E+03, 1.287870E+03, 1.331130E+03, 1.374788E+03, 1.418860E+03,
    1.463360E+03, 1.508301E+03, 1.553696E+03, 1.599557E+03, 1.645895E+03, 1.692720E+03,
    1.740043E+03, 1.787872E+03, 1.836217E+03, 1.885084E+03, 1.934483E+03, 1.984419E+03,
    2.034900E+03, 2.085932E+03, 2.137521E+03, 2.189673E+03, 2.242393E+03, 2.295685E+03,
    2.349556E+03, 2.404008E+03, 2.459047E+03, 2.514677E+03, 2.570901E+03, 2.627723E+03,
    2.685146E+03, 2.743175E+03, 2.801812E+03, 2.861060E+03, 2.920922E+03, 2.981401E+03,
    3.042499E+03, 3.104220E+03, 3.166566E+03, 3.229539E+03, 3.293142E+03, 3.357377E+03,
    3.422246E+03, 3.487751E+03, 3.553894E+03, 3.620678E+03, 3.688104E+03, 3.756175E+03,
    3.824891E+03, 3.894256E+03, 3.964270E+03, 4.034936E+03, 4.106255E+03, 4.178229E+03,
    4.250859E+03, 4.324148E+03, 4.398096E+03, 4.472705E+03, 4.547977E+03, 4.623914E+03,
    4.700516E+03, 4.777785E+03, 4.855723E+03, 4.934331E+03, 5.013611E+03, 5.093563E+03,
    5.174189E+03, 5.255491E+03, 5.337469E+03, 5.420125E+03, 5.503460E+03, 5.587476E+03,
    5.672173E+03, 5.757554E+03, 5.843618E+03, 5.930367E+03, 6.017803E+03, 6.105926E+03,
    6.194737E+03, 6.284239E+03, 6.374431E+03, 6.465314E+03, 6.556890E+03, 6.649160E+03,
    6.742125E+03, 6.835786E+03, 6.930143E+03, 7.025198E+03, 7.120951E+03, 7.217404E+03,
    7.314556E+03, 7.412410E+03, 7.510966E+03, 7.610225E+03, 7.710187E+03, 7.810854E+03,
    7.912226E+03, 8.014303E+03, 8.117087E+03, 8.220578E+03, 8.324778E+03, 8.429685E+03,
    8.535302E+03, 8.641629E+03, 8.748666E+03, 8.856414E+03, 8.964874E+03, 9.074045E+03,
    9.183929E+03, 9.294526E+03, 9.405837E+03, 9.517861E+03, 9.630600E+03, 9.744054E+03,
    9.858222E+03, 9.973107E+03, 1.008871E+04, 1.020502E+04, 1.032206E+04, 1.043980E+04,
    1.055827E+04, 1.067745E+04, 1.079735E+04, 1.091797E+04, 1.103931E+04, 1.116136E+04,
    1.128413E+04, 1.140762E+04, 1.153183E+04, 1.165676E+04, 1.178240E+04, 1.190877E+04,
    1.203585E+04, 1.216365E+04, 1.229217E+04, 1.242140E+04, 1.255136E+04, 1.268203E+04,
    1.281342E+04, 1.294553E+04, 1.307835E+04, 1.321189E+04, 1.334615E+04, 1.348113E+04,
    1.361683E+04, 1.375324E+04, 1.389036E+04, 1.402821E+04, 1.416676E+04, 1.430604E+04,
    1.444603E+04, 1.458673E+04, 1.472815E+04, 1.487028E+04, 1.501313E+04, 1.515669E+04,
    1.530096E+04, 1.544594E+04, 1.559163E+04, 1.573804E+04, 1.588516E+04, 1.603299E+04,
    1.618152E+04, 1.633077E+04, 1.648072E+04, 1.663138E+04, 1.678275E+04, 1.693483E+04,
    1.708761E+04, 1.724110E+04, 1.739530E+04, 1.755019E+04, 1.770579E+04, 1.786210E+04,
    1.801910E+04, 1.817681E+04, 1.833522E+04, 1.849432E+04, 1.865413E+04, 1.881464E+04,
    1.897584E+04, 1.913774E+04, 1.930033E+04, 1.946362E+04, 1.962761E+04, 1.979228E+04,
    1.995766E+04, 2.012372E+04, 2.029047E+04, 2.045791E+04, 2.062605E+04, 2.079487E+04,
    2.096438E+04, 2.113457E+04, 2.130545E+04, 2.147701E+04, 2.164926E+04, 2.182219E+04,
    2.199580E+04, 2.217009E+04, 2.234507E+04, 2.252072E+04, 2.269704E+04, 2.287405E+04,
    2.305173E+04, 2.323008E+04, 2.340911E+04, 2.358881E+04, 2.376918E+04, 2.395022E+04,
    2.413193E+04, 2.431431E+04, 2.449736E+04, 2.468107E+04, 2.486545E+04, 2.505049E+04,
    2.523619E+04, 2.542255E+04, 2.560958E+04, 2.579726E+04, 2.598560E+04, 2.617460E+04,
    2.636425E+04, 2.655456E+04, 2.674552E+04, 2.693713E+04, 2.712940E+04, 2.732231E+04,
    2.751587E+04, 2.771008E+04, 2.790494E+04, 2.810044E+04, 2.829658E+04, 2.849337E+04,
    2.869080E+04, 2.888886E+04, 2.908757E+04, 2.928691E+04, 2.948689E+04, 2.968751E+04,
    2.988876E+04, 3.009064E+04, 3.029315E+04, 3.049629E+04, 3.070006E+04, 3.090446E+04,
    3.110948E+04, 3.131513E+04, 3.152140E+04, 3.172830E+04, 3.193581E+04, 3.214394E+04,
    3.235270E+04, 3.256206E+04, 3.277205E+04, 3.298265E+04, 3.319386E+04, 3.340568E+04,
    3.361811E+04, 3.383115E+04, 3.404480E+04, 3.425905E+04, 3.447391E+04, 3.468937E+04,
    3.490544E+04, 3.512210E+04, 3.533937E+04, 3.555723E+04, 3.577568E+04, 3.599474E+04,
    3.621438E+04,
])

#  ---------------------- M = 17, I = 1 ---------------------------
M = 17
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.200000E+01, 3.036750E+01, 5.612178E+01, 8.203022E+01, 1.079799E+02, 1.339502E+02,
    1.599342E+02, 1.859291E+02, 2.119337E+02, 2.379470E+02, 2.639687E+02, 2.899986E+02,
    3.160366E+02, 3.420830E+02, 3.681386E+02, 3.942045E+02, 4.202826E+02, 4.463759E+02,
    4.724880E+02, 4.986238E+02, 5.247889E+02, 5.509901E+02, 5.772350E+02, 6.035318E+02,
    6.298897E+02, 6.563182E+02, 6.828272E+02, 7.094270E+02, 7.361281E+02, 7.629408E+02,
    7.898758E+02, 8.169433E+02, 8.441535E+02, 8.715165E+02, 8.990419E+02, 9.267392E+02,
    9.546175E+02, 9.826856E+02, 1.010952E+03, 1.039424E+03, 1.068111E+03, 1.097019E+03,
    1.126156E+03, 1.155527E+03, 1.185141E+03, 1.215002E+03, 1.245116E+03, 1.275490E+03,
    1.306128E+03, 1.337035E+03, 1.368216E+03, 1.399675E+03, 1.431417E+03, 1.463446E+03,
    1.495766E+03, 1.528380E+03, 1.561292E+03, 1.594505E+03, 1.628022E+03, 1.661847E+03,
    1.695982E+03, 1.730431E+03, 1.765195E+03, 1.800278E+03, 1.835682E+03, 1.871408E+03,
    1.907461E+03, 1.943841E+03, 1.980550E+03, 2.017592E+03, 2.054967E+03, 2.092677E+03,
    2.130725E+03, 2.169112E+03, 2.207840E+03, 2.246911E+03, 2.286325E+03, 2.326085E+03,
    2.366192E+03, 2.406647E+03, 2.447452E+03, 2.488609E+03, 2.530118E+03, 2.571981E+03,
    2.614199E+03, 2.656774E+03, 2.699706E+03, 2.742997E+03, 2.786647E+03, 2.830659E+03,
    2.875033E+03, 2.919770E+03, 2.964871E+03, 3.010338E+03, 3.056170E+03, 3.102370E+03,
    3.148938E+03, 3.195875E+03, 3.243182E+03, 3.290860E+03, 3.338910E+03, 3.387332E+03,
    3.436128E+03, 3.485298E+03, 3.534843E+03, 3.584764E+03, 3.635061E+03, 3.685735E+03,
    3.736788E+03, 3.788220E+03, 3.840030E+03, 3.892221E+03, 3.944793E+03, 3.997746E+03,
    4.051081E+03, 4.104799E+03, 4.158899E+03, 4.213384E+03, 4.268253E+03, 4.323507E+03,
    4.379146E+03, 4.435171E+03, 4.491582E+03, 4.548380E+03, 4.605566E+03, 4.663139E+03,
    4.721100E+03, 4.779450E+03, 4.838189E+03, 4.897317E+03, 4.956835E+03, 5.016743E+03,
    5.077042E+03, 5.137731E+03, 5.198811E+03, 5.260282E+03, 5.322144E+03, 5.384399E+03,
    5.447045E+03, 5.510084E+03, 5.573515E+03, 5.637339E+03, 5.701556E+03, 5.766165E+03,
    5.831168E+03, 5.896563E+03, 5.962352E+03, 6.028535E+03, 6.095111E+03, 6.162081E+03,
    6.229444E+03, 6.297201E+03, 6.365351E+03, 6.433896E+03, 6.502834E+03, 6.572165E+03,
    6.641891E+03, 6.712010E+03, 6.782522E+03, 6.853428E+03, 6.924728E+03, 6.996420E+03,
    7.068506E+03, 7.140985E+03, 7.213857E+03, 7.287122E+03, 7.360779E+03, 7.434829E+03,
    7.509271E+03, 7.584105E+03, 7.659331E+03, 7.734948E+03, 7.810957E+03, 7.887357E+03,
    7.964148E+03, 8.041329E+03, 8.118901E+03, 8.196862E+03, 8.275214E+03, 8.353954E+03,
    8.433084E+03, 8.512603E+03, 8.592509E+03, 8.672804E+03, 8.753486E+03, 8.834556E+03,
    8.916012E+03, 8.997855E+03, 9.080084E+03, 9.162699E+03, 9.245698E+03, 9.329082E+03,
    9.412851E+03, 9.497003E+03, 9.581539E+03, 9.666457E+03, 9.751757E+03, 9.837440E+03,
    9.923503E+03, 1.000995E+04, 1.009677E+04, 1.018398E+04, 1.027156E+04, 1.035952E+04,
    1.044786E+04, 1.053658E+04, 1.062567E+04, 1.071515E+04, 1.080499E+04, 1.089521E+04,
    1.098581E+04, 1.107678E+04, 1.116812E+04, 1.125984E+04, 1.135192E+04, 1.144438E+04,
    1.153721E+04, 1.163041E+04, 1.172398E+04, 1.181791E+04, 1.191222E+04, 1.200689E+04,
    1.210193E+04, 1.219733E+04, 1.229310E+04, 1.238923E+04, 1.248573E+04, 1.258259E+04,
    1.267981E+04, 1.277739E+04, 1.287533E+04, 1.297363E+04, 1.307230E+04, 1.317132E+04,
    1.327069E+04, 1.337043E+04, 1.347052E+04, 1.357096E+04, 1.367176E+04, 1.377291E+04,
    1.387441E+04, 1.397627E+04, 1.407848E+04, 1.418103E+04, 1.428394E+04, 1.438719E+04,
    1.449080E+04, 1.459474E+04, 1.469904E+04, 1.480368E+04, 1.490866E+04, 1.501399E+04,
    1.511966E+04, 1.522567E+04, 1.533202E+04, 1.543871E+04, 1.554573E+04, 1.565310E+04,
    1.576080E+04, 1.586884E+04, 1.597722E+04, 1.608592E+04, 1.619497E+04, 1.630434E+04,
    1.641404E+04, 1.652408E+04, 1.663444E+04, 1.674513E+04, 1.685615E+04, 1.696750E+04,
    1.707917E+04, 1.719117E+04, 1.730349E+04, 1.741613E+04, 1.752909E+04, 1.764238E+04,
    1.775598E+04, 1.786991E+04, 1.798415E+04, 1.809870E+04, 1.821358E+04, 1.832876E+04,
    1.844426E+04, 1.856008E+04, 1.867620E+04, 1.879264E+04, 1.890938E+04, 1.902644E+04,
    1.914380E+04, 1.926147E+04, 1.937944E+04, 1.949772E+04, 1.961630E+04, 1.973518E+04,
    1.985437E+04, 1.997385E+04, 2.009364E+04, 2.021372E+04, 2.033410E+04, 2.045477E+04,
    2.057574E+04,
])

#  ---------------------- M = 17, I = 2 ---------------------------
M = 17
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.800464E+01, 8.321075E+01, 1.599981E+02, 2.369086E+02, 3.138651E+02, 3.908526E+02,
    4.678662E+02, 5.449041E+02, 6.219653E+02, 6.990504E+02, 7.761621E+02, 8.533068E+02,
    9.304964E+02, 1.007750E+03, 1.085094E+03, 1.162563E+03, 1.240199E+03, 1.318049E+03,
    1.396167E+03, 1.474609E+03, 1.553433E+03, 1.632699E+03, 1.712467E+03, 1.792793E+03,
    1.873736E+03, 1.955349E+03, 2.037686E+03, 2.120794E+03, 2.204722E+03, 2.289512E+03,
    2.375206E+03, 2.461843E+03, 2.549457E+03, 2.638082E+03, 2.727749E+03, 2.818487E+03,
    2.910323E+03, 3.003280E+03, 3.097383E+03, 3.192652E+03, 3.289107E+03, 3.386767E+03,
    3.485650E+03, 3.585770E+03, 3.687143E+03, 3.789784E+03, 3.893705E+03, 3.998918E+03,
    4.105435E+03, 4.213267E+03, 4.322425E+03, 4.432917E+03, 4.544753E+03, 4.657942E+03,
    4.772491E+03, 4.888409E+03, 5.005702E+03, 5.124379E+03, 5.244445E+03, 5.365907E+03,
    5.488771E+03, 5.613043E+03, 5.738728E+03, 5.865832E+03, 5.994360E+03, 6.124317E+03,
    6.255707E+03, 6.388535E+03, 6.522805E+03, 6.658522E+03, 6.795690E+03, 6.934312E+03,
    7.074392E+03, 7.215933E+03, 7.358940E+03, 7.503415E+03, 7.649362E+03, 7.796783E+03,
    7.945681E+03, 8.096060E+03, 8.247921E+03, 8.401268E+03, 8.556102E+03, 8.712426E+03,
    8.870242E+03, 9.029552E+03, 9.190358E+03, 9.352662E+03, 9.516465E+03, 9.681769E+03,
    9.848576E+03, 1.001689E+04, 1.018670E+04, 1.035802E+04, 1.053085E+04, 1.070519E+04,
    1.088103E+04, 1.105839E+04, 1.123725E+04, 1.141763E+04, 1.159951E+04, 1.178291E+04,
    1.196781E+04, 1.215423E+04, 1.234216E+04, 1.253160E+04, 1.272255E+04, 1.291501E+04,
    1.310899E+04, 1.330447E+04, 1.350145E+04, 1.369995E+04, 1.389995E+04, 1.410146E+04,
    1.430448E+04, 1.450899E+04, 1.471501E+04, 1.492253E+04, 1.513156E+04, 1.534207E+04,
    1.555409E+04, 1.576760E+04, 1.598261E+04, 1.619910E+04, 1.641709E+04, 1.663656E+04,
    1.685752E+04, 1.707996E+04, 1.730388E+04, 1.752928E+04, 1.775616E+04, 1.798452E+04,
    1.821434E+04, 1.844564E+04, 1.867840E+04, 1.891262E+04, 1.914831E+04, 1.938545E+04,
    1.962405E+04, 1.986410E+04, 2.010561E+04, 2.034856E+04, 2.059295E+04, 2.083878E+04,
    2.108605E+04, 2.133476E+04, 2.158489E+04, 2.183646E+04, 2.208945E+04, 2.234386E+04,
    2.259968E+04, 2.285692E+04, 2.311558E+04, 2.337563E+04, 2.363710E+04, 2.389996E+04,
    2.416422E+04, 2.442987E+04, 2.469691E+04, 2.496533E+04, 2.523514E+04, 2.550632E+04,
    2.577888E+04, 2.605280E+04, 2.632810E+04, 2.660475E+04, 2.688276E+04, 2.716213E+04,
    2.744285E+04, 2.772491E+04, 2.800832E+04, 2.829306E+04, 2.857914E+04, 2.886655E+04,
    2.915528E+04, 2.944534E+04, 2.973671E+04, 3.002940E+04, 3.032340E+04, 3.061870E+04,
    3.091531E+04, 3.121321E+04, 3.151240E+04, 3.181289E+04, 3.211465E+04, 3.241770E+04,
    3.272203E+04, 3.302762E+04, 3.333449E+04, 3.364262E+04, 3.395200E+04, 3.426265E+04,
    3.457454E+04, 3.488768E+04, 3.520206E+04, 3.551768E+04, 3.583454E+04, 3.615262E+04,
    3.647193E+04, 3.679246E+04, 3.711421E+04, 3.743717E+04, 3.776134E+04, 3.808672E+04,
    3.841329E+04, 3.874106E+04, 3.907002E+04, 3.940017E+04, 3.973150E+04, 4.006401E+04,
    4.039770E+04, 4.073255E+04, 4.106857E+04, 4.140576E+04, 4.174410E+04, 4.208359E+04,
    4.242424E+04, 4.276602E+04, 4.310895E+04, 4.345302E+04, 4.379822E+04, 4.414455E+04,
    4.449200E+04, 4.484057E+04, 4.519026E+04, 4.554105E+04, 4.589296E+04, 4.624597E+04,
    4.660008E+04, 4.695528E+04, 4.731158E+04, 4.766896E+04, 4.802742E+04, 4.838697E+04,
    4.874758E+04, 4.910927E+04, 4.947202E+04, 4.983584E+04, 5.020071E+04, 5.056664E+04,
    5.093362E+04, 5.130164E+04, 5.167071E+04, 5.204081E+04, 5.241195E+04, 5.278412E+04,
    5.315731E+04, 5.353153E+04, 5.390676E+04, 5.428300E+04, 5.466026E+04, 5.503852E+04,
    5.541778E+04, 5.579805E+04, 5.617930E+04, 5.656155E+04, 5.694478E+04, 5.732899E+04,
    5.771418E+04, 5.810034E+04, 5.848748E+04, 5.887558E+04, 5.926464E+04, 5.965467E+04,
    6.004564E+04, 6.043757E+04, 6.083045E+04, 6.122426E+04, 6.161902E+04, 6.201471E+04,
    6.241133E+04, 6.280888E+04, 6.320736E+04, 6.360675E+04, 6.400706E+04, 6.440828E+04,
    6.481042E+04, 6.521345E+04, 6.561739E+04, 6.602222E+04, 6.642795E+04, 6.683456E+04,
    6.724206E+04, 6.765045E+04, 6.805971E+04, 6.846985E+04, 6.888085E+04, 6.929273E+04,
    6.970546E+04, 7.011906E+04, 7.053351E+04, 7.094882E+04, 7.136497E+04, 7.178197E+04,
    7.219981E+04, 7.261848E+04, 7.303799E+04, 7.345833E+04, 7.387950E+04, 7.430148E+04,
    7.472429E+04,
])

#  ---------------------- M = 18, I = 1 ---------------------------
M = 18
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    9.761590E+00, 1.797541E+02, 3.589432E+02, 5.384749E+02, 7.198461E+02, 9.059401E+02,
    1.099605E+03, 1.302867E+03, 1.516834E+03, 1.742050E+03, 1.978735E+03, 2.226901E+03,
    2.486580E+03, 2.757704E+03, 3.040307E+03, 3.334331E+03, 3.639872E+03, 3.956877E+03,
    4.285427E+03, 4.625509E+03, 4.977243E+03, 5.340628E+03, 5.715693E+03, 6.102457E+03,
    6.501039E+03, 6.911435E+03, 7.333681E+03, 7.767794E+03, 8.213845E+03, 8.671895E+03,
    9.141829E+03, 9.623827E+03, 1.011782E+04, 1.062389E+04, 1.114211E+04, 1.167236E+04,
    1.221467E+04, 1.276928E+04, 1.333592E+04, 1.391481E+04, 1.450597E+04, 1.510916E+04,
    1.572473E+04, 1.635254E+04, 1.699269E+04, 1.764503E+04, 1.830963E+04, 1.898657E+04,
    1.967596E+04, 2.037757E+04, 2.109164E+04, 2.181793E+04, 2.255682E+04, 2.330792E+04,
    2.407159E+04, 2.484758E+04, 2.563595E+04, 2.643692E+04, 2.725021E+04, 2.807621E+04,
    2.891445E+04, 2.976534E+04, 3.062873E+04, 3.150468E+04, 3.239304E+04, 3.329404E+04,
    3.420753E+04, 3.513376E+04, 3.607234E+04, 3.702375E+04, 3.798758E+04, 3.896408E+04,
    3.995331E+04, 4.095507E+04, 4.196962E+04, 4.299675E+04, 4.403651E+04, 4.508917E+04,
    4.615426E+04, 4.723232E+04, 4.832287E+04, 4.942645E+04, 5.054257E+04, 5.167152E+04,
    5.281306E+04, 5.396749E+04, 5.513482E+04, 5.631483E+04, 5.750780E+04, 5.871349E+04,
    5.993189E+04, 6.116335E+04, 6.240759E+04, 6.366463E+04, 6.493478E+04, 6.621748E+04,
    6.751335E+04, 6.882212E+04, 7.014378E+04, 7.147838E+04, 7.282593E+04, 7.418643E+04,
    7.555994E+04, 7.694646E+04, 7.834567E+04, 7.975826E+04, 8.118359E+04, 8.262236E+04,
    8.407389E+04, 8.553818E+04, 8.701598E+04, 8.850660E+04, 9.001039E+04, 9.152739E+04,
    9.305725E+04, 9.460073E+04, 9.615671E+04, 9.772636E+04, 9.930890E+04, 1.009044E+05,
    1.025132E+05, 1.041354E+05, 1.057705E+05, 1.074191E+05, 1.090806E+05, 1.107556E+05,
    1.124435E+05, 1.141450E+05, 1.158599E+05, 1.175875E+05, 1.193290E+05, 1.210832E+05,
    1.228509E+05, 1.246323E+05, 1.264267E+05, 1.282343E+05, 1.300555E+05, 1.318899E+05,
    1.337380E+05, 1.355993E+05, 1.374737E+05, 1.393619E+05, 1.412634E+05, 1.431781E+05,
    1.451065E+05, 1.470483E+05, 1.490034E+05, 1.509722E+05, 1.529544E+05, 1.549505E+05,
    1.569593E+05, 1.589821E+05, 1.610182E+05, 1.630683E+05, 1.651312E+05, 1.672086E+05,
    1.692989E+05, 1.714027E+05, 1.735204E+05, 1.756516E+05, 1.777969E+05, 1.799551E+05,
    1.821273E+05, 1.843131E+05, 1.865124E+05, 1.887258E+05, 1.909522E+05, 1.931928E+05,
    1.954469E+05, 1.977152E+05, 1.999966E+05, 2.022921E+05, 2.046013E+05, 2.069242E+05,
    2.092612E+05, 2.116114E+05,
])

#  ---------------------- M = 18, I = 2 ---------------------------
M = 18
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    9.912400E+00, 1.828410E+02, 3.651190E+02, 5.477455E+02, 7.322431E+02, 9.215448E+02,
    1.118548E+03, 1.325314E+03, 1.543000E+03, 1.772137E+03, 2.012972E+03, 2.265546E+03,
    2.529857E+03, 2.805894E+03, 3.093647E+03, 3.393126E+03, 3.704301E+03, 4.027259E+03,
    4.362060E+03, 4.708654E+03, 5.067118E+03, 5.437544E+03, 5.819917E+03, 6.214315E+03,
    6.620748E+03, 7.039331E+03, 7.469975E+03, 7.912762E+03, 8.367835E+03, 8.835036E+03,
    9.314552E+03, 9.806333E+03, 1.031039E+04, 1.082689E+04, 1.135564E+04, 1.189679E+04,
    1.245038E+04, 1.301635E+04, 1.359485E+04, 1.418577E+04, 1.478913E+04, 1.540504E+04,
    1.603337E+04, 1.667435E+04, 1.732783E+04, 1.799389E+04, 1.867251E+04, 1.936376E+04,
    2.006759E+04, 2.078409E+04, 2.151317E+04, 2.225490E+04, 2.300937E+04, 2.377649E+04,
    2.455631E+04, 2.534873E+04, 2.615398E+04, 2.697195E+04, 2.780269E+04, 2.864608E+04,
    2.950235E+04, 3.037136E+04, 3.125337E+04, 3.214803E+04, 3.305538E+04, 3.397565E+04,
    3.490891E+04, 3.585478E+04, 3.681371E+04, 3.778554E+04, 3.877029E+04, 3.976780E+04,
    4.077831E+04, 4.180164E+04, 4.283805E+04, 4.388733E+04, 4.494977E+04, 4.602491E+04,
    4.711326E+04, 4.821437E+04, 4.932877E+04, 5.045597E+04, 5.159626E+04, 5.274966E+04,
    5.391623E+04, 5.509570E+04, 5.628837E+04, 5.749399E+04, 5.871259E+04, 5.994477E+04,
    6.118969E+04, 6.244793E+04, 6.371925E+04, 6.500366E+04, 6.630117E+04, 6.761214E+04,
    6.893627E+04, 7.027357E+04, 7.162408E+04, 7.298748E+04, 7.436445E+04, 7.575468E+04,
    7.715820E+04, 7.857503E+04, 8.000484E+04, 8.144834E+04, 8.290486E+04, 8.437476E+04,
    8.585806E+04, 8.735480E+04, 8.886462E+04, 9.038828E+04, 9.192505E+04, 9.347494E+04,
    9.503836E+04, 9.661531E+04, 9.820585E+04, 9.980955E+04, 1.014269E+05, 1.030574E+05,
    1.047016E+05, 1.063594E+05, 1.080305E+05, 1.097153E+05, 1.114134E+05, 1.131248E+05,
    1.148503E+05, 1.165888E+05, 1.183415E+05, 1.201071E+05, 1.218866E+05, 1.236798E+05,
    1.254865E+05, 1.273067E+05, 1.291407E+05, 1.309881E+05, 1.328495E+05, 1.347244E+05,
    1.366128E+05, 1.385151E+05, 1.404309E+05, 1.423608E+05, 1.443038E+05, 1.462613E+05,
    1.482318E+05, 1.502165E+05, 1.522147E+05, 1.542271E+05, 1.562526E+05, 1.582928E+05,
    1.603461E+05, 1.624136E+05, 1.644948E+05, 1.665896E+05, 1.686987E+05, 1.708211E+05,
    1.729576E+05, 1.751085E+05, 1.772726E+05, 1.794510E+05, 1.816432E+05, 1.838497E+05,
    1.860695E+05, 1.883038E+05, 1.905518E+05, 1.928137E+05, 1.950900E+05, 1.973803E+05,
    1.996838E+05, 2.020024E+05, 2.043343E+05, 2.066802E+05, 2.090406E+05, 2.114150E+05,
    2.138033E+05, 2.162063E+05,
])

#  ---------------------- M = 19, I = 1 ---------------------------
M = 19
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.780860E+00, 6.888728E+01, 1.374712E+02, 2.060890E+02, 2.747808E+02, 3.438010E+02,
    4.137509E+02, 4.855483E+02, 5.602403E+02, 6.388943E+02, 7.225221E+02, 8.120553E+02,
    9.083745E+02, 1.012293E+03, 1.124615E+03, 1.246077E+03, 1.377438E+03, 1.519448E+03,
    1.672847E+03, 1.838414E+03, 2.016938E+03, 2.209189E+03, 2.416025E+03, 2.638267E+03,
    2.876824E+03, 3.132602E+03, 3.406543E+03, 3.699623E+03, 4.012839E+03, 4.347286E+03,
    4.703992E+03, 5.084126E+03, 5.488848E+03, 5.919326E+03, 6.376821E+03, 6.862632E+03,
    7.378065E+03, 7.924484E+03, 8.503314E+03, 9.115988E+03, 9.764033E+03, 1.044898E+04,
    1.117241E+04, 1.193599E+04, 1.274135E+04, 1.359026E+04, 1.448448E+04, 1.542590E+04,
    1.641632E+04, 1.745769E+04, 1.855200E+04, 1.970131E+04, 2.090764E+04, 2.217321E+04,
    2.350017E+04, 2.489078E+04, 2.634733E+04, 2.787217E+04, 2.946776E+04, 3.113658E+04,
    3.288109E+04, 3.470398E+04, 3.660778E+04, 3.859534E+04, 4.066933E+04, 4.283266E+04,
    4.508820E+04, 4.743892E+04, 4.988780E+04, 5.243797E+04, 5.509261E+04, 5.785493E+04,
    6.072821E+04, 6.371578E+04, 6.682109E+04, 7.004764E+04, 7.339906E+04, 7.687883E+04,
    8.049086E+04, 8.423872E+04, 8.812646E+04, 9.215786E+04, 9.633693E+04, 1.006678E+05,
    1.051547E+05, 1.098017E+05, 1.146132E+05, 1.195935E+05, 1.247472E+05, 1.300787E+05,
    1.355927E+05, 1.412939E+05, 1.471870E+05, 1.532770E+05, 1.595687E+05, 1.660673E+05,
    1.727778E+05, 1.797055E+05, 1.868555E+05, 1.942334E+05, 2.018446E+05, 2.096945E+05,
    2.177890E+05, 2.261336E+05, 2.347342E+05, 2.435967E+05, 2.527273E+05, 2.621318E+05,
    2.718165E+05, 2.817878E+05, 2.920519E+05, 3.026154E+05, 3.134849E+05, 3.246670E+05,
    3.361684E+05, 3.479962E+05, 3.601571E+05, 3.726584E+05, 3.855070E+05, 3.987105E+05,
    4.122760E+05, 4.262112E+05, 4.405236E+05, 4.552207E+05, 4.703105E+05, 4.858008E+05,
    5.016998E+05, 5.180154E+05, 5.347559E+05, 5.519297E+05, 5.695451E+05, 5.876108E+05,
    6.061353E+05, 6.251276E+05, 6.445965E+05, 6.645510E+05, 6.850001E+05, 7.059533E+05,
    7.274196E+05, 7.494089E+05, 7.719305E+05, 7.949943E+05, 8.186099E+05, 8.427874E+05,
    8.675368E+05, 8.928685E+05, 9.187926E+05, 9.453196E+05, 9.724601E+05, 1.000225E+06,
    1.028624E+06, 1.057670E+06, 1.087373E+06, 1.117744E+06, 1.148794E+06, 1.180535E+06,
    1.212979E+06, 1.246138E+06, 1.280022E+06, 1.314645E+06, 1.350018E+06, 1.386153E+06,
    1.423064E+06, 1.460762E+06, 1.499260E+06, 1.538571E+06, 1.578708E+06, 1.619685E+06,
    1.661513E+06, 1.704207E+06, 1.747780E+06, 1.792246E+06, 1.837618E+06, 1.883911E+06,
    1.931138E+06, 1.979315E+06,
])

#  ---------------------- M = 19, I = 2 ---------------------------
M = 19
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.866200E+00, 7.060513E+01, 1.409078E+02, 2.112453E+02, 2.816616E+02, 3.524124E+02,
    4.241291E+02, 4.977537E+02, 5.743694E+02, 6.550831E+02, 7.409406E+02, 8.329051E+02,
    9.318879E+02, 1.038741E+03, 1.154268E+03, 1.279272E+03, 1.414522E+03, 1.560780E+03,
    1.718840E+03, 1.889483E+03, 2.073538E+03, 2.271818E+03, 2.485183E+03, 2.714529E+03,
    2.960748E+03, 3.224797E+03, 3.507665E+03, 3.810366E+03, 4.133950E+03, 4.479493E+03,
    4.848138E+03, 5.241034E+03, 5.659402E+03, 6.104497E+03, 6.577577E+03, 7.080010E+03,
    7.613174E+03, 8.178454E+03, 8.777326E+03, 9.411328E+03, 1.008198E+04, 1.079091E+04,
    1.153976E+04, 1.233025E+04, 1.316410E+04, 1.404310E+04, 1.496915E+04, 1.594410E+04,
    1.696994E+04, 1.804866E+04, 1.918233E+04, 2.037303E+04, 2.162293E+04, 2.293432E+04,
    2.430941E+04, 2.575059E+04, 2.726019E+04, 2.884073E+04, 3.049469E+04, 3.222463E+04,
    3.403321E+04, 3.592313E+04, 3.789715E+04, 3.995806E+04, 4.210879E+04, 4.435223E+04,
    4.669141E+04, 4.912946E+04, 5.166950E+04, 5.431473E+04, 5.706844E+04, 5.993400E+04,
    6.291482E+04, 6.601437E+04, 6.923620E+04, 7.258402E+04, 7.606148E+04, 7.967242E+04,
    8.342064E+04, 8.731008E+04, 9.134474E+04, 9.552874E+04, 9.986620E+04, 1.043614E+05,
    1.090186E+05, 1.138423E+05, 1.188369E+05, 1.240069E+05, 1.293571E+05, 1.348921E+05,
    1.406168E+05, 1.465359E+05, 1.526546E+05, 1.589779E+05, 1.655109E+05, 1.722588E+05,
    1.792271E+05, 1.864210E+05, 1.938462E+05, 2.015082E+05, 2.094126E+05, 2.175653E+05,
    2.259721E+05, 2.346390E+05, 2.435721E+05, 2.527774E+05, 2.622614E+05, 2.720302E+05,
    2.820903E+05, 2.924483E+05, 3.031110E+05, 3.140847E+05, 3.253767E+05, 3.369937E+05,
    3.489427E+05, 3.612310E+05, 3.738659E+05, 3.868545E+05, 4.002045E+05, 4.139232E+05,
    4.280186E+05, 4.424983E+05, 4.573703E+05, 4.726424E+05, 4.883229E+05, 5.044200E+05,
    5.209419E+05, 5.378972E+05, 5.552945E+05, 5.731422E+05, 5.914493E+05, 6.102247E+05,
    6.294775E+05, 6.492166E+05, 6.694515E+05, 6.901914E+05, 7.114458E+05, 7.332246E+05,
    7.555370E+05, 7.783934E+05, 8.018036E+05, 8.257776E+05, 8.503257E+05, 8.754583E+05,
    9.011859E+05, 9.275190E+05, 9.544684E+05, 9.820450E+05, 1.010260E+06, 1.039124E+06,
    1.068649E+06, 1.098845E+06, 1.129726E+06, 1.161301E+06, 1.193583E+06, 1.226584E+06,
    1.260317E+06, 1.294792E+06, 1.330023E+06, 1.366022E+06, 1.402801E+06, 1.440374E+06,
    1.478753E+06, 1.517952E+06, 1.557983E+06, 1.598860E+06, 1.640596E+06, 1.683205E+06,
    1.726701E+06, 1.771098E+06, 1.816409E+06, 1.862649E+06, 1.909833E+06, 1.957974E+06,
    2.007088E+06, 2.057189E+06,
])

#  ---------------------- M = 19, I = 3 ---------------------------
M = 19
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    7.583710E+00, 1.382173E+02, 2.758282E+02, 4.135070E+02, 5.513676E+02, 6.900135E+02,
    8.308340E+02, 9.758329E+02, 1.127263E+03, 1.287312E+03, 1.458080E+03, 1.641493E+03,
    1.839335E+03, 2.053277E+03, 2.284972E+03, 2.535982E+03, 2.807868E+03, 3.102189E+03,
    3.420546E+03, 3.764540E+03, 4.135842E+03, 4.536154E+03, 4.967247E+03, 5.430907E+03,
    5.929040E+03, 6.463587E+03, 7.036596E+03, 7.650158E+03, 8.306416E+03, 9.007672E+03,
    9.756218E+03, 1.055447E+04, 1.140497E+04, 1.231027E+04, 1.327302E+04, 1.429603E+04,
    1.538213E+04, 1.653432E+04, 1.775555E+04, 1.904896E+04, 2.041788E+04, 2.186555E+04,
    2.339541E+04, 2.501105E+04, 2.671603E+04, 2.851411E+04, 3.040920E+04, 3.240513E+04,
    3.450607E+04, 3.671615E+04, 3.903959E+04, 4.148094E+04, 4.404455E+04, 4.673515E+04,
    4.955745E+04, 5.251625E+04, 5.561666E+04, 5.886377E+04, 6.226278E+04, 6.581899E+04,
    6.953795E+04, 7.342535E+04, 7.748685E+04, 8.172822E+04, 8.615570E+04, 9.077534E+04,
    9.559340E+04, 1.006162E+05, 1.058507E+05, 1.113031E+05, 1.169806E+05, 1.228899E+05,
    1.290384E+05, 1.354334E+05, 1.420821E+05, 1.489923E+05, 1.561716E+05, 1.636280E+05,
    1.713695E+05, 1.794043E+05, 1.877407E+05, 1.963875E+05, 2.053530E+05, 2.146464E+05,
    2.242765E+05, 2.342527E+05, 2.445840E+05, 2.552802E+05, 2.663510E+05, 2.778062E+05,
    2.896558E+05, 3.019102E+05, 3.145796E+05, 3.276747E+05, 3.412063E+05, 3.551851E+05,
    3.696226E+05, 3.845298E+05, 3.999184E+05, 4.158000E+05, 4.321865E+05, 4.490901E+05,
    4.665230E+05, 4.844975E+05, 5.030266E+05, 5.221230E+05, 5.417996E+05, 5.620701E+05,
    5.829476E+05, 6.044458E+05, 6.265791E+05, 6.493608E+05, 6.728060E+05, 6.969287E+05,
    7.217437E+05, 7.472664E+05, 7.735113E+05, 8.004945E+05, 8.282311E+05, 8.567373E+05,
    8.860289E+05, 9.161222E+05, 9.470339E+05, 9.787808E+05, 1.011380E+06, 1.044848E+06,
    1.079203E+06, 1.114462E+06, 1.150644E+06, 1.187766E+06, 1.225847E+06, 1.264906E+06,
    1.304961E+06, 1.346032E+06, 1.388138E+06, 1.431299E+06, 1.475534E+06, 1.520864E+06,
    1.567309E+06, 1.614891E+06, 1.663628E+06, 1.713544E+06, 1.764659E+06, 1.816995E+06,
    1.870575E+06, 1.925419E+06, 1.981552E+06, 2.038995E+06, 2.097772E+06, 2.157905E+06,
    2.219420E+06, 2.282339E+06, 2.346687E+06, 2.412488E+06, 2.479766E+06, 2.548548E+06,
    2.618858E+06, 2.690721E+06, 2.764165E+06, 2.839214E+06, 2.915895E+06, 2.994236E+06,
    3.074263E+06, 3.156003E+06, 3.239484E+06, 3.324735E+06, 3.411783E+06, 3.500657E+06,
    3.591387E+06, 3.683999E+06, 3.778527E+06, 3.874996E+06, 3.973440E+06, 4.073888E+06,
    4.176370E+06, 4.280919E+06,
])

#  ---------------------- M = 19, I = 4 ---------------------------
M = 19
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.529716E+01, 2.790462E+02, 5.568808E+02, 8.348524E+02, 1.113124E+03, 1.392726E+03,
    1.676127E+03, 1.967024E+03, 2.269724E+03, 2.588528E+03, 2.927570E+03, 3.290640E+03,
    3.681356E+03, 4.103003E+03, 4.558813E+03, 5.051868E+03, 5.585175E+03, 6.161870E+03,
    6.784985E+03, 7.457593E+03, 8.182895E+03, 8.964139E+03, 9.804763E+03, 1.070818E+04,
    1.167802E+04, 1.271790E+04, 1.383181E+04, 1.502371E+04, 1.629759E+04, 1.765790E+04,
    1.910900E+04, 2.065548E+04, 2.230214E+04, 2.405374E+04, 2.591542E+04, 2.789238E+04,
    2.999010E+04, 3.221416E+04, 3.457015E+04, 3.706422E+04, 3.970237E+04, 4.249085E+04,
    4.543618E+04, 4.854516E+04, 5.182439E+04, 5.528126E+04, 5.892283E+04, 6.275655E+04,
    6.679011E+04, 7.103145E+04, 7.548857E+04, 8.016983E+04, 8.508371E+04, 9.023910E+04,
    9.564456E+04, 1.013096E+05, 1.072436E+05, 1.134560E+05, 1.199569E+05, 1.267562E+05,
    1.338645E+05, 1.412920E+05, 1.490498E+05, 1.571490E+05, 1.656006E+05, 1.744166E+05,
    1.836087E+05, 1.931887E+05, 2.031692E+05, 2.135629E+05, 2.243826E+05, 2.356413E+05,
    2.473527E+05, 2.595302E+05, 2.721881E+05, 2.853405E+05, 2.990019E+05, 3.131871E+05,
    3.279114E+05, 3.431903E+05, 3.590394E+05, 3.754746E+05, 3.925124E+05, 4.101694E+05,
    4.284625E+05, 4.474091E+05, 4.670263E+05, 4.873326E+05, 5.083460E+05, 5.300849E+05,
    5.525683E+05, 5.758153E+05, 5.998453E+05, 6.246787E+05, 6.503350E+05, 6.768352E+05,
    7.042000E+05, 7.324508E+05, 7.616091E+05, 7.916970E+05, 8.227364E+05, 8.547503E+05,
    8.877616E+05, 9.217938E+05, 9.568705E+05, 9.930163E+05, 1.030255E+06, 1.068612E+06,
    1.108113E+06, 1.148782E+06, 1.190646E+06, 1.233733E+06, 1.278068E+06, 1.323678E+06,
    1.370591E+06, 1.418836E+06, 1.468441E+06, 1.519434E+06, 1.571845E+06, 1.625704E+06,
    1.681041E+06, 1.737886E+06, 1.796270E+06, 1.856224E+06, 1.917781E+06, 1.980972E+06,
    2.045832E+06, 2.112391E+06, 2.180684E+06, 2.250746E+06, 2.322610E+06, 2.396312E+06,
    2.471886E+06, 2.549370E+06, 2.628798E+06, 2.710208E+06, 2.793637E+06, 2.879123E+06,
    2.966704E+06, 3.056419E+06, 3.148307E+06, 3.242406E+06, 3.338759E+06, 3.437405E+06,
    3.538386E+06, 3.641742E+06, 3.747517E+06, 3.855753E+06, 3.966492E+06, 4.079779E+06,
    4.195658E+06, 4.314174E+06, 4.435371E+06, 4.559296E+06, 4.685994E+06, 4.815512E+06,
    4.947899E+06, 5.083202E+06, 5.221468E+06, 5.362749E+06, 5.507091E+06, 5.654546E+06,
    5.805164E+06, 5.958997E+06, 6.116096E+06, 6.276514E+06, 6.440302E+06, 6.607517E+06,
    6.778209E+06, 6.952435E+06, 7.130249E+06, 7.311707E+06, 7.496867E+06, 7.685785E+06,
    7.878517E+06, 8.075122E+06,
])

#  ---------------------- M = 19, I = 5 ---------------------------
M = 19
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.005750E+00, 7.341299E+01, 1.465248E+02, 2.196725E+02, 2.929080E+02, 3.665343E+02,
    4.412543E+02, 5.180980E+02, 5.982340E+02, 6.828353E+02, 7.730027E+02, 8.697668E+02,
    9.740684E+02, 1.086823E+03, 1.208879E+03, 1.341069E+03, 1.484232E+03, 1.639180E+03,
    1.806745E+03, 1.987801E+03, 2.183188E+03, 2.393832E+03, 2.620622E+03, 2.864546E+03,
    3.126558E+03, 3.407697E+03, 3.709043E+03, 4.031658E+03, 4.376687E+03, 4.745342E+03,
    5.138807E+03, 5.558356E+03, 6.005318E+03, 6.481015E+03, 6.986846E+03, 7.524300E+03,
    8.094828E+03, 8.699968E+03, 9.341331E+03, 1.002056E+04, 1.073933E+04, 1.149937E+04,
    1.230251E+04, 1.315057E+04, 1.404547E+04, 1.498914E+04, 1.598358E+04, 1.703092E+04,
    1.813322E+04, 1.929266E+04, 2.051152E+04, 2.179207E+04, 2.313669E+04, 2.454777E+04,
    2.602783E+04, 2.757939E+04, 2.920503E+04, 3.090744E+04, 3.268941E+04, 3.455366E+04,
    3.650309E+04, 3.854064E+04, 4.066930E+04, 4.289217E+04, 4.521234E+04, 4.763307E+04,
    5.015761E+04, 5.278935E+04, 5.553171E+04, 5.838817E+04, 6.136237E+04, 6.445786E+04,
    6.767842E+04, 7.102788E+04, 7.451010E+04, 7.812908E+04, 8.188878E+04, 8.579338E+04,
    8.984714E+04, 9.405423E+04, 9.841911E+04, 1.029462E+05, 1.076400E+05, 1.125052E+05,
    1.175464E+05, 1.227686E+05, 1.281765E+05, 1.337751E+05, 1.395696E+05, 1.455650E+05,
    1.517667E+05, 1.581798E+05, 1.648099E+05, 1.716626E+05, 1.787433E+05, 1.860579E+05,
    1.936120E+05, 2.014118E+05, 2.094632E+05, 2.177721E+05, 2.263450E+05, 2.351881E+05,
    2.443078E+05, 2.537105E+05, 2.634030E+05, 2.733918E+05, 2.836841E+05, 2.942864E+05,
    3.052060E+05, 3.164500E+05, 3.280255E+05, 3.399401E+05, 3.522011E+05, 3.648163E+05,
    3.777931E+05, 3.911396E+05, 4.048633E+05, 4.189728E+05, 4.334758E+05, 4.483808E+05,
    4.636960E+05, 4.794302E+05, 4.955917E+05, 5.121894E+05, 5.292323E+05, 5.467291E+05,
    5.646892E+05, 5.831215E+05, 6.020357E+05, 6.214410E+05, 6.413472E+05, 6.617640E+05,
    6.827011E+05, 7.041687E+05, 7.261769E+05, 7.487358E+05, 7.718559E+05, 7.955477E+05,
    8.198219E+05, 8.446893E+05, 8.701605E+05, 8.962469E+05, 9.229596E+05, 9.503098E+05,
    9.783092E+05, 1.006969E+06, 1.036302E+06, 1.066319E+06, 1.097032E+06, 1.128454E+06,
    1.160596E+06, 1.193472E+06, 1.227094E+06, 1.261474E+06, 1.296626E+06, 1.332563E+06,
    1.369297E+06, 1.406843E+06, 1.445213E+06, 1.484422E+06, 1.524483E+06, 1.565410E+06,
    1.607217E+06, 1.649919E+06, 1.693530E+06, 1.738064E+06, 1.783536E+06, 1.829962E+06,
    1.877355E+06, 1.925733E+06, 1.975109E+06, 2.025500E+06, 2.076920E+06, 2.129387E+06,
    2.182916E+06, 2.237523E+06,
])

#  ---------------------- M = 20, I = 1 ---------------------------
M = 20
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.091110E+00, 4.947081E+01, 1.416480E+02, 2.592507E+02, 3.983023E+02, 5.559709E+02,
    7.302920E+02, 9.198257E+02, 1.123477E+03, 1.340418E+03, 1.570058E+03, 1.812038E+03,
    2.066230E+03, 2.332728E+03, 2.611836E+03, 2.904051E+03, 3.210045E+03, 3.530639E+03,
    3.866789E+03, 4.219566E+03, 4.590146E+03, 4.979793E+03, 5.389860E+03, 5.821776E+03,
    6.277046E+03, 6.757251E+03, 7.264045E+03, 7.799159E+03, 8.364401E+03, 8.961659E+03,
    9.592904E+03, 1.026019E+04, 1.096512E+04, 1.170966E+04, 1.249659E+04, 1.332840E+04,
    1.420757E+04, 1.513680E+04, 1.611862E+04, 1.715616E+04, 1.825218E+04, 1.940974E+04,
    2.063218E+04, 2.192282E+04, 2.328522E+04, 2.472293E+04, 2.623979E+04, 2.783973E+04,
    2.952702E+04, 3.130573E+04, 3.318041E+04, 3.515575E+04, 3.723663E+04, 3.942790E+04,
    4.173473E+04, 4.416289E+04, 4.671773E+04, 4.940519E+04, 5.223118E+04, 5.520223E+04,
    5.832512E+04, 6.160614E+04, 6.505245E+04, 6.867173E+04, 7.247112E+04, 7.645903E+04,
    8.064278E+04, 8.503157E+04, 8.963402E+04, 9.445875E+04, 9.951574E+04, 1.048142E+05,
    1.103643E+05, 1.161765E+05, 1.222616E+05, 1.286309E+05, 1.352955E+05, 1.422674E+05,
    1.495587E+05, 1.571827E+05, 1.651519E+05, 1.734802E+05, 1.821813E+05, 1.912699E+05,
    2.007604E+05, 2.106690E+05, 2.210108E+05, 2.318027E+05, 2.430609E+05, 2.548036E+05,
    2.670483E+05, 2.798136E+05, 2.931184E+05, 3.069823E+05, 3.214256E+05, 3.364693E+05,
    3.521348E+05, 3.684439E+05, 3.854199E+05, 4.030855E+05, 4.214650E+05, 4.405834E+05,
    4.604660E+05, 4.811396E+05, 5.026303E+05, 5.249660E+05, 5.481756E+05, 5.722878E+05,
    5.973333E+05, 6.233430E+05, 6.503482E+05, 6.783817E+05, 7.074775E+05, 7.376698E+05,
    7.689940E+05, 8.014861E+05, 8.351831E+05, 8.701245E+05, 9.063479E+05, 9.438955E+05,
    9.828070E+05, 1.023125E+06, 1.064895E+06, 1.108158E+06, 1.152963E+06, 1.199355E+06,
    1.247383E+06, 1.297095E+06, 1.348543E+06, 1.401778E+06, 1.456853E+06, 1.513823E+06,
    1.572742E+06, 1.633669E+06, 1.696660E+06, 1.761777E+06, 1.829081E+06, 1.898632E+06,
    1.970497E+06, 2.044741E+06, 2.121430E+06, 2.200634E+06, 2.282422E+06, 2.366867E+06,
    2.454042E+06, 2.544023E+06, 2.636886E+06, 2.732710E+06, 2.831576E+06, 2.933566E+06,
    3.038763E+06, 3.147255E+06, 3.259128E+06, 3.374473E+06, 3.493381E+06, 3.615946E+06,
    3.742264E+06, 3.872434E+06, 4.006554E+06, 4.144726E+06, 4.287056E+06, 4.433650E+06,
    4.584616E+06, 4.740066E+06, 4.900110E+06, 5.064870E+06, 5.234458E+06, 5.408999E+06,
    5.588613E+06, 5.773427E+06, 5.963569E+06, 6.159169E+06, 6.360361E+06, 6.567283E+06,
    6.780068E+06, 6.998864E+06,
])

#  ---------------------- M = 20, I = 2 ---------------------------
M = 20
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.519830E+00, 1.041424E+02, 2.906766E+02, 5.317342E+02, 8.170004E+02, 1.140498E+03,
    1.498191E+03, 1.887134E+03, 2.305068E+03, 2.750313E+03, 3.221668E+03, 3.718356E+03,
    4.240095E+03, 4.787152E+03, 5.360055E+03, 5.959825E+03, 6.587858E+03, 7.245886E+03,
    7.935732E+03, 8.659593E+03, 9.420003E+03, 1.021942E+04, 1.106064E+04, 1.194646E+04,
    1.288013E+04, 1.386475E+04, 1.490370E+04, 1.600056E+04, 1.715893E+04, 1.838256E+04,
    1.967551E+04, 2.104202E+04, 2.248633E+04, 2.401319E+04, 2.562698E+04, 2.733280E+04,
    2.913575E+04, 3.104133E+04, 3.305479E+04, 3.518252E+04, 3.743016E+04, 3.980401E+04,
    4.231092E+04, 4.495767E+04, 4.775159E+04, 5.069997E+04, 5.381064E+04, 5.709169E+04,
    6.055190E+04, 6.419956E+04, 6.804403E+04, 7.209494E+04, 7.636229E+04, 8.085600E+04,
    8.558671E+04, 9.056622E+04, 9.580554E+04, 1.013168E+05, 1.071122E+05, 1.132050E+05,
    1.196093E+05, 1.263378E+05, 1.334053E+05, 1.408275E+05, 1.486191E+05, 1.567972E+05,
    1.653770E+05, 1.743773E+05, 1.838157E+05, 1.937100E+05, 2.040806E+05, 2.149463E+05,
    2.263282E+05, 2.382475E+05, 2.507265E+05, 2.637882E+05, 2.774557E+05, 2.917531E+05,
    3.067057E+05, 3.223406E+05, 3.386834E+05, 3.557626E+05, 3.736064E+05, 3.922448E+05,
    4.117074E+05, 4.320273E+05, 4.532358E+05, 4.753672E+05, 4.984549E+05, 5.225361E+05,
    5.476468E+05, 5.738251E+05, 6.011099E+05, 6.295411E+05, 6.591607E+05, 6.900115E+05,
    7.221373E+05, 7.555835E+05, 7.903966E+05, 8.266243E+05, 8.643161E+05, 9.035230E+05,
    9.442971E+05, 9.866935E+05, 1.030765E+06, 1.076570E+06, 1.124167E+06, 1.173615E+06,
    1.224977E+06, 1.278317E+06, 1.333697E+06, 1.391187E+06, 1.450855E+06, 1.512772E+06,
    1.577010E+06, 1.643643E+06, 1.712747E+06, 1.784403E+06, 1.858688E+06, 1.935688E+06,
    2.015486E+06, 2.098169E+06, 2.183827E+06, 2.272550E+06, 2.364432E+06, 2.459570E+06,
    2.558063E+06, 2.660012E+06, 2.765517E+06, 2.874690E+06, 2.987634E+06, 3.104464E+06,
    3.225293E+06, 3.350239E+06, 3.479418E+06, 3.612957E+06, 3.750979E+06, 3.893612E+06,
    4.040990E+06, 4.193245E+06, 4.350513E+06, 4.512941E+06, 4.680667E+06, 4.853844E+06,
    5.032618E+06, 5.217145E+06, 5.407584E+06, 5.604095E+06, 5.806845E+06, 6.016000E+06,
    6.231733E+06, 6.454222E+06, 6.683645E+06, 6.920189E+06, 7.164040E+06, 7.415389E+06,
    7.674437E+06, 7.941381E+06, 8.216427E+06, 8.499785E+06, 8.791667E+06, 9.092294E+06,
    9.401888E+06, 9.720676E+06, 1.004889E+07, 1.038677E+07, 1.073455E+07, 1.109249E+07,
    1.146083E+07, 1.183984E+07, 1.222977E+07, 1.263090E+07, 1.304350E+07, 1.346784E+07,
    1.390421E+07, 1.435290E+07,
])

#  ---------------------- M = 20, I = 3 ---------------------------
M = 20
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    7.773000E-01, 5.324867E+01, 1.486677E+02, 2.719834E+02, 4.179176E+02, 5.834117E+02,
    7.664008E+02, 9.653781E+02, 1.179188E+03, 1.406969E+03, 1.648109E+03, 1.902210E+03,
    2.169127E+03, 2.448996E+03, 2.742088E+03, 3.048926E+03, 3.370223E+03, 3.706865E+03,
    4.059785E+03, 4.430108E+03, 4.819128E+03, 5.228105E+03, 5.658465E+03, 6.111648E+03,
    6.589308E+03, 7.093032E+03, 7.624553E+03, 8.185700E+03, 8.778316E+03, 9.404317E+03,
    1.006578E+04, 1.076488E+04, 1.150378E+04, 1.228491E+04, 1.311052E+04, 1.398321E+04,
    1.490558E+04, 1.588047E+04, 1.691055E+04, 1.799908E+04, 1.914896E+04, 2.036341E+04,
    2.164594E+04, 2.300000E+04, 2.442935E+04, 2.593773E+04, 2.752913E+04, 2.920771E+04,
    3.097792E+04, 3.284406E+04, 3.481086E+04, 3.688330E+04, 3.906645E+04, 4.136541E+04,
    4.378561E+04, 4.633311E+04, 4.901353E+04, 5.183307E+04, 5.479795E+04, 5.791503E+04,
    6.119141E+04, 6.463369E+04, 6.824939E+04, 7.204656E+04, 7.603270E+04, 8.021661E+04,
    8.460600E+04, 8.921050E+04, 9.403916E+04, 9.910103E+04, 1.044066E+05, 1.099654E+05,
    1.157884E+05, 1.218862E+05, 1.282704E+05, 1.349528E+05, 1.419450E+05, 1.492595E+05,
    1.569092E+05, 1.649080E+05, 1.732688E+05, 1.820065E+05, 1.911353E+05, 2.006707E+05,
    2.106276E+05, 2.210232E+05, 2.318735E+05, 2.431958E+05, 2.550074E+05, 2.673273E+05,
    2.801738E+05, 2.935665E+05, 3.075254E+05, 3.220707E+05, 3.372240E+05, 3.530071E+05,
    3.694426E+05, 3.865535E+05, 4.043638E+05, 4.228977E+05, 4.421807E+05, 4.622389E+05,
    4.830988E+05, 5.047886E+05, 5.273357E+05, 5.507693E+05, 5.751197E+05, 6.004172E+05,
    6.266939E+05, 6.539820E+05, 6.823147E+05, 7.117262E+05, 7.422522E+05, 7.739285E+05,
    8.067927E+05, 8.408819E+05, 8.762352E+05, 9.128942E+05, 9.508983E+05, 9.902916E+05,
    1.031116E+06, 1.073416E+06, 1.117238E+06, 1.162629E+06, 1.209636E+06, 1.258308E+06,
    1.308697E+06, 1.360853E+06, 1.414829E+06, 1.470682E+06, 1.528464E+06, 1.588234E+06,
    1.650050E+06, 1.713972E+06, 1.780059E+06, 1.848378E+06, 1.918989E+06, 1.991960E+06,
    2.067358E+06, 2.145251E+06, 2.225709E+06, 2.308807E+06, 2.394615E+06, 2.483212E+06,
    2.574672E+06, 2.669076E+06, 2.766504E+06, 2.867038E+06, 2.970764E+06, 3.077768E+06,
    3.188136E+06, 3.301961E+06, 3.419333E+06, 3.540348E+06, 3.665102E+06, 3.793692E+06,
    3.926220E+06, 4.062788E+06, 4.203501E+06, 4.348466E+06, 4.497792E+06, 4.651592E+06,
    4.809980E+06, 4.973071E+06, 5.140983E+06, 5.313842E+06, 5.491767E+06, 5.674888E+06,
    5.863331E+06, 6.057231E+06, 6.256719E+06, 6.461935E+06, 6.673017E+06, 6.890111E+06,
    7.113357E+06, 7.342907E+06,
])

#  ---------------------- M = 21, I = 1 ---------------------------
M = 21
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.981460E+00, 3.304167E+02, 9.292313E+02, 1.704088E+03, 2.621559E+03, 3.662421E+03,
    4.814121E+03, 6.068541E+03, 7.421071E+03, 8.869781E+03, 1.041512E+04, 1.205911E+04,
    1.380519E+04, 1.565730E+04, 1.762056E+04, 1.970014E+04, 2.190181E+04, 2.423163E+04,
    2.669554E+04, 2.929991E+04, 3.205132E+04, 3.495586E+04, 3.802056E+04, 4.125226E+04,
    4.465708E+04, 4.824259E+04, 5.201564E+04, 5.598309E+04, 6.015219E+04, 6.453072E+04,
    6.912550E+04, 7.394438E+04, 7.899525E+04, 8.428610E+04, 8.982435E+04, 9.561869E+04,
    1.016773E+05, 1.080083E+05, 1.146215E+05, 1.215237E+05, 1.287266E+05, 1.362371E+05,
    1.440654E+05, 1.522203E+05, 1.607132E+05, 1.695523E+05, 1.787484E+05, 1.883127E+05,
    1.982535E+05, 2.085833E+05, 2.193115E+05, 2.304511E+05, 2.420107E+05, 2.540037E+05,
    2.664413E+05, 2.793348E+05, 2.926958E+05, 3.065372E+05, 3.208723E+05, 3.357116E+05,
    3.510703E+05, 3.669592E+05, 3.833922E+05, 4.003837E+05, 4.179467E+05, 4.360939E+05,
    4.548404E+05, 4.741996E+05, 4.941865E+05, 5.148171E+05, 5.361032E+05, 5.580627E+05,
    5.807100E+05, 6.040616E+05, 6.281301E+05, 6.529321E+05, 6.784872E+05, 7.048083E+05,
    7.319154E+05, 7.598219E+05, 7.885455E+05, 8.181023E+05, 8.485133E+05, 8.797949E+05,
    9.119639E+05, 9.450366E+05, 9.790353E+05, 1.013977E+06, 1.049882E+06, 1.086768E+06,
    1.124652E+06, 1.163556E+06, 1.203500E+06, 1.244505E+06, 1.286588E+06, 1.329770E+06,
    1.374075E+06, 1.419521E+06, 1.466130E+06, 1.513921E+06, 1.562920E+06, 1.613148E+06,
    1.664626E+06, 1.717375E+06, 1.771419E+06, 1.826780E+06, 1.883484E+06, 1.941552E+06,
    2.001009E+06, 2.061877E+06, 2.124181E+06, 2.187948E+06, 2.253196E+06, 2.319956E+06,
    2.388251E+06, 2.458107E+06, 2.529549E+06, 2.602607E+06, 2.677298E+06, 2.753658E+06,
    2.831706E+06, 2.911473E+06, 2.992987E+06, 3.076272E+06, 3.161364E+06, 3.248277E+06,
    3.337053E+06, 3.427717E+06, 3.520289E+06, 3.614807E+06, 3.711301E+06, 3.809798E+06,
    3.910326E+06, 4.012918E+06, 4.117603E+06, 4.224413E+06, 4.333378E+06, 4.444528E+06,
    4.557903E+06, 4.673519E+06, 4.791422E+06, 4.911642E+06, 5.034208E+06, 5.159155E+06,
    5.286513E+06, 5.416319E+06, 5.548609E+06, 5.683412E+06, 5.820768E+06, 5.960706E+06,
    6.103265E+06, 6.248481E+06, 6.396387E+06, 6.547017E+06, 6.700413E+06, 6.856604E+06,
    7.015634E+06, 7.177539E+06, 7.342350E+06, 7.510115E+06, 7.680862E+06, 7.854632E+06,
    8.031473E+06, 8.211412E+06, 8.394491E+06, 8.580756E+06, 8.770230E+06, 8.962977E+06,
    9.159022E+06, 9.358408E+06, 9.561181E+06, 9.767369E+06, 9.977034E+06, 1.019021E+07,
    1.040692E+07, 1.062723E+07,
])

#  ---------------------- M = 21, I = 2 ---------------------------
M = 21
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.058120E+00, 3.362054E+02, 9.456074E+02, 1.734179E+03, 2.667896E+03, 3.727193E+03,
    4.899294E+03, 6.175937E+03, 7.552431E+03, 9.026809E+03, 1.059953E+04, 1.227265E+04,
    1.404968E+04, 1.593461E+04, 1.793265E+04, 2.004908E+04, 2.228977E+04, 2.466088E+04,
    2.716846E+04, 2.981898E+04, 3.261916E+04, 3.557517E+04, 3.869418E+04, 4.198316E+04,
    4.544832E+04, 4.909737E+04, 5.293730E+04, 5.697506E+04, 6.121805E+04, 6.567419E+04,
    7.035040E+04, 7.525467E+04, 8.039507E+04, 8.577969E+04, 9.141609E+04, 9.731312E+04,
    1.034791E+05, 1.099223E+05, 1.166527E+05, 1.236773E+05, 1.310079E+05, 1.386514E+05,
    1.466185E+05, 1.549179E+05, 1.635614E+05, 1.725571E+05, 1.819162E+05, 1.916501E+05,
    2.017671E+05, 2.122799E+05, 2.231982E+05, 2.345353E+05, 2.462997E+05, 2.585053E+05,
    2.711634E+05, 2.842854E+05, 2.978833E+05, 3.119700E+05, 3.265592E+05, 3.416615E+05,
    3.572924E+05, 3.734629E+05, 3.901872E+05, 4.074799E+05, 4.253541E+05, 4.438230E+05,
    4.629018E+05, 4.826041E+05, 5.029453E+05, 5.239416E+05, 5.456049E+05, 5.679538E+05,
    5.910025E+05, 6.147679E+05, 6.392630E+05, 6.645047E+05, 6.905128E+05, 7.173004E+05,
    7.448879E+05, 7.732890E+05, 8.025217E+05, 8.326025E+05, 8.635525E+05, 8.953886E+05,
    9.281278E+05, 9.617867E+05, 9.963880E+05, 1.031949E+06, 1.068491E+06, 1.106030E+06,
    1.144586E+06, 1.184179E+06, 1.224831E+06, 1.266563E+06, 1.309392E+06, 1.353339E+06,
    1.398430E+06, 1.444682E+06, 1.492116E+06, 1.540754E+06, 1.590622E+06, 1.641741E+06,
    1.694130E+06, 1.747816E+06, 1.802817E+06, 1.859159E+06, 1.916869E+06, 1.975965E+06,
    2.036476E+06, 2.098423E+06, 2.161832E+06, 2.226729E+06, 2.293134E+06, 2.361077E+06,
    2.430583E+06, 2.501677E+06, 2.574385E+06, 2.648739E+06, 2.724753E+06, 2.802466E+06,
    2.881898E+06, 2.963079E+06, 3.046038E+06, 3.130799E+06, 3.217399E+06, 3.305853E+06,
    3.396203E+06, 3.488473E+06, 3.582687E+06, 3.678880E+06, 3.777084E+06, 3.877327E+06,
    3.979638E+06, 4.084047E+06, 4.190589E+06, 4.299291E+06, 4.410188E+06, 4.523309E+06,
    4.638693E+06, 4.756358E+06, 4.876351E+06, 4.998702E+06, 5.123440E+06, 5.250602E+06,
    5.380218E+06, 5.512325E+06, 5.646960E+06, 5.784152E+06, 5.923943E+06, 6.066361E+06,
    6.211446E+06, 6.359237E+06, 6.509766E+06, 6.663065E+06, 6.819180E+06, 6.978140E+06,
    7.139989E+06, 7.304764E+06, 7.472497E+06, 7.643236E+06, 7.817009E+06, 7.993858E+06,
    8.173833E+06, 8.356963E+06, 8.543287E+06, 8.732853E+06, 8.925687E+06, 9.121849E+06,
    9.321369E+06, 9.524290E+06, 9.730657E+06, 9.940501E+06, 1.015388E+07, 1.037083E+07,
    1.059139E+07, 1.081561E+07,
])

#  ---------------------- M = 22, I = 1 ---------------------------
M = 22
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.029370E+00, 3.298562E+01, 6.440722E+01, 9.584644E+01, 1.272919E+02, 1.587414E+02,
    1.901941E+02, 2.216498E+02, 2.531083E+02, 2.845695E+02, 3.160334E+02, 3.475000E+02,
    3.789694E+02, 4.104418E+02, 4.419180E+02, 4.733988E+02, 5.048860E+02, 5.363820E+02,
    5.678901E+02, 5.994148E+02, 6.309613E+02, 6.625362E+02, 6.941467E+02, 7.258012E+02,
    7.575086E+02, 7.892788E+02, 8.211221E+02, 8.530492E+02, 8.850710E+02, 9.171988E+02,
    9.494439E+02, 9.818175E+02, 1.014331E+03, 1.046995E+03, 1.079820E+03, 1.112818E+03,
    1.145998E+03, 1.179370E+03, 1.212943E+03, 1.246728E+03, 1.280732E+03, 1.314964E+03,
    1.349432E+03, 1.384144E+03, 1.419107E+03, 1.454327E+03, 1.489813E+03, 1.525569E+03,
    1.561602E+03, 1.597917E+03, 1.634520E+03, 1.671416E+03, 1.708610E+03, 1.746106E+03,
    1.783909E+03, 1.822022E+03, 1.860449E+03, 1.899195E+03, 1.938262E+03, 1.977654E+03,
    2.017374E+03, 2.057425E+03, 2.097810E+03, 2.138531E+03, 2.179590E+03, 2.220991E+03,
    2.262736E+03, 2.304826E+03, 2.347264E+03, 2.390051E+03, 2.433189E+03, 2.476681E+03,
    2.520527E+03, 2.564729E+03, 2.609289E+03, 2.654208E+03, 2.699487E+03, 2.745128E+03,
    2.791131E+03, 2.837498E+03, 2.884230E+03, 2.931327E+03, 2.978791E+03, 3.026622E+03,
    3.074822E+03, 3.123390E+03, 3.172328E+03, 3.221636E+03, 3.271315E+03, 3.321366E+03,
    3.371788E+03, 3.422582E+03, 3.473749E+03, 3.525290E+03, 3.577203E+03, 3.629490E+03,
    3.682151E+03, 3.735186E+03, 3.788596E+03, 3.842379E+03, 3.896538E+03, 3.951070E+03,
    4.005977E+03, 4.061259E+03, 4.116915E+03, 4.172946E+03, 4.229351E+03, 4.286130E+03,
    4.343283E+03, 4.400810E+03, 4.458710E+03, 4.516984E+03, 4.575631E+03, 4.634651E+03,
    4.694043E+03, 4.753807E+03, 4.813944E+03, 4.874451E+03, 4.935329E+03, 4.996578E+03,
    5.058196E+03, 5.120184E+03, 5.182541E+03, 5.245267E+03, 5.308360E+03, 5.371821E+03,
    5.435648E+03, 5.499841E+03, 5.564399E+03, 5.629323E+03, 5.694610E+03, 5.760261E+03,
    5.826274E+03, 5.892649E+03, 5.959385E+03, 6.026482E+03, 6.093938E+03, 6.161753E+03,
    6.229926E+03, 6.298457E+03, 6.367343E+03, 6.436585E+03, 6.506181E+03, 6.576131E+03,
    6.646434E+03, 6.717089E+03, 6.788095E+03, 6.859450E+03, 6.931155E+03, 7.003207E+03,
    7.075607E+03, 7.148353E+03, 7.221444E+03, 7.294879E+03, 7.368657E+03, 7.442778E+03,
    7.517239E+03, 7.592040E+03, 7.667180E+03, 7.742658E+03, 7.818473E+03, 7.894623E+03,
    7.971108E+03, 8.047926E+03, 8.125077E+03, 8.202560E+03, 8.280372E+03, 8.358513E+03,
    8.436983E+03, 8.515779E+03, 8.594900E+03, 8.674346E+03, 8.754116E+03, 8.834207E+03,
    8.914620E+03, 8.995352E+03, 9.076403E+03, 9.157772E+03, 9.239456E+03, 9.321456E+03,
    9.403770E+03, 9.486396E+03, 9.569334E+03, 9.652582E+03, 9.736138E+03, 9.820003E+03,
    9.904175E+03, 9.988651E+03, 1.007343E+04, 1.015852E+04, 1.024390E+04, 1.032959E+04,
    1.041557E+04, 1.050186E+04, 1.058844E+04, 1.067531E+04, 1.076248E+04, 1.084994E+04,
    1.093770E+04, 1.102574E+04, 1.111408E+04, 1.120270E+04, 1.129161E+04, 1.138080E+04,
    1.147028E+04, 1.156004E+04, 1.165009E+04, 1.174041E+04, 1.183101E+04, 1.192189E+04,
    1.201305E+04, 1.210448E+04, 1.219619E+04, 1.228817E+04, 1.238042E+04, 1.247295E+04,
    1.256574E+04, 1.265880E+04, 1.275212E+04, 1.284571E+04, 1.293957E+04, 1.303369E+04,
    1.312807E+04, 1.322271E+04, 1.331760E+04, 1.341276E+04, 1.350817E+04, 1.360384E+04,
    1.369976E+04, 1.379594E+04, 1.389237E+04, 1.398904E+04, 1.408597E+04, 1.418314E+04,
    1.428056E+04, 1.437823E+04, 1.447614E+04, 1.457429E+04, 1.467269E+04, 1.477132E+04,
    1.487019E+04, 1.496930E+04, 1.506865E+04, 1.516823E+04, 1.526805E+04, 1.536810E+04,
    1.546838E+04, 1.556890E+04, 1.566964E+04, 1.577061E+04, 1.587180E+04, 1.597322E+04,
    1.607487E+04, 1.617674E+04, 1.627883E+04, 1.638114E+04, 1.648367E+04, 1.658642E+04,
    1.668939E+04, 1.679257E+04, 1.689597E+04, 1.699958E+04, 1.710340E+04, 1.720744E+04,
    1.731168E+04, 1.741613E+04, 1.752079E+04, 1.762566E+04, 1.773073E+04, 1.783601E+04,
    1.794149E+04, 1.804717E+04, 1.815305E+04, 1.825913E+04, 1.836541E+04, 1.847189E+04,
    1.857856E+04, 1.868542E+04, 1.879248E+04, 1.889974E+04, 1.900718E+04, 1.911481E+04,
    1.922264E+04, 1.933065E+04, 1.943885E+04, 1.954723E+04, 1.965580E+04, 1.976455E+04,
    1.987348E+04, 1.998260E+04, 2.009189E+04, 2.020137E+04, 2.031102E+04, 2.042085E+04,
    2.053085E+04, 2.064103E+04, 2.075138E+04, 2.086191E+04, 2.097260E+04, 2.108347E+04,
    2.119450E+04, 2.130571E+04, 2.141708E+04, 2.152862E+04, 2.164032E+04, 2.175218E+04,
    2.186421E+04, 2.197640E+04, 2.208875E+04, 2.220126E+04, 2.231393E+04, 2.242675E+04,
    2.253974E+04, 2.265287E+04, 2.276617E+04, 2.287961E+04, 2.299321E+04, 2.310696E+04,
    2.322086E+04, 2.333491E+04, 2.344911E+04, 2.356345E+04, 2.367794E+04, 2.379258E+04,
    2.390736E+04, 2.402228E+04, 2.413735E+04, 2.425256E+04, 2.436791E+04, 2.448339E+04,
    2.459902E+04, 2.471478E+04, 2.483069E+04, 2.494672E+04, 2.506289E+04, 2.517920E+04,
    2.529563E+04, 2.541220E+04, 2.552890E+04, 2.564573E+04, 2.576269E+04, 2.587978E+04,
    2.599699E+04, 2.611433E+04, 2.623180E+04, 2.634939E+04, 2.646710E+04, 2.658493E+04,
    2.670289E+04, 2.682097E+04, 2.693917E+04, 2.705748E+04, 2.717592E+04, 2.729447E+04,
    2.741314E+04, 2.753192E+04, 2.765082E+04, 2.776984E+04, 2.788896E+04, 2.800820E+04,
    2.812754E+04, 2.824700E+04, 2.836657E+04, 2.848625E+04, 2.860603E+04, 2.872592E+04,
    2.884592E+04, 2.896602E+04, 2.908623E+04, 2.920654E+04, 2.932695E+04, 2.944746E+04,
    2.956808E+04, 2.968880E+04, 2.980961E+04, 2.993053E+04, 3.005154E+04, 3.017265E+04,
    3.029385E+04, 3.041515E+04, 3.053655E+04, 3.065804E+04, 3.077962E+04, 3.090129E+04,
    3.102306E+04, 3.114492E+04, 3.126686E+04, 3.138890E+04, 3.151103E+04, 3.163324E+04,
    3.175554E+04, 3.187793E+04, 3.200040E+04, 3.212295E+04, 3.224560E+04, 3.236832E+04,
    3.249113E+04, 3.261402E+04, 3.273698E+04, 3.286004E+04, 3.298317E+04, 3.310637E+04,
    3.322966E+04, 3.335303E+04, 3.347647E+04, 3.359999E+04, 3.372358E+04, 3.384725E+04,
    3.397099E+04, 3.409481E+04, 3.421870E+04, 3.434266E+04, 3.446669E+04, 3.459079E+04,
    3.471497E+04, 3.483921E+04, 3.496352E+04, 3.508790E+04, 3.521235E+04, 3.533686E+04,
    3.546144E+04, 3.558609E+04, 3.571080E+04, 3.583558E+04, 3.596041E+04, 3.608532E+04,
    3.621028E+04, 3.633531E+04, 3.646039E+04, 3.658554E+04, 3.671075E+04, 3.683601E+04,
    3.696134E+04, 3.708672E+04, 3.721216E+04, 3.733766E+04, 3.746321E+04, 3.758882E+04,
    3.771449E+04, 3.784021E+04, 3.796598E+04, 3.809181E+04, 3.821769E+04, 3.834362E+04,
    3.846960E+04, 3.859563E+04, 3.872172E+04, 3.884785E+04, 3.897404E+04, 3.910027E+04,
    3.922655E+04,
])

#  ---------------------- M = 22, I = 2 ---------------------------
M = 22
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.071020E+00, 4.541678E+01, 8.875137E+01, 1.321089E+02, 1.754746E+02, 2.188458E+02,
    2.622213E+02, 3.056010E+02, 3.489845E+02, 3.923717E+02, 4.357626E+02, 4.791572E+02,
    5.225558E+02, 5.659587E+02, 6.093669E+02, 6.527822E+02, 6.962070E+02, 7.396453E+02,
    7.831022E+02, 8.265842E+02, 8.700997E+02, 9.136581E+02, 9.572704E+02, 1.000949E+03,
    1.044707E+03, 1.088558E+03, 1.132518E+03, 1.176603E+03, 1.220827E+03, 1.265207E+03,
    1.309760E+03, 1.354500E+03, 1.399444E+03, 1.444608E+03, 1.490006E+03, 1.535654E+03,
    1.581565E+03, 1.627754E+03, 1.674233E+03, 1.721016E+03, 1.768115E+03, 1.815541E+03,
    1.863305E+03, 1.911418E+03, 1.959890E+03, 2.008730E+03, 2.057948E+03, 2.107552E+03,
    2.157550E+03, 2.207950E+03, 2.258760E+03, 2.309985E+03, 2.361634E+03, 2.413711E+03,
    2.466223E+03, 2.519175E+03, 2.572573E+03, 2.626421E+03, 2.680724E+03, 2.735487E+03,
    2.790713E+03, 2.846407E+03, 2.902573E+03, 2.959214E+03, 3.016333E+03, 3.073933E+03,
    3.132018E+03, 3.190590E+03, 3.249653E+03, 3.309208E+03, 3.369258E+03, 3.429805E+03,
    3.490852E+03, 3.552400E+03, 3.614451E+03, 3.677007E+03, 3.740070E+03, 3.803642E+03,
    3.867723E+03, 3.932316E+03, 3.997421E+03, 4.063040E+03, 4.129174E+03, 4.195823E+03,
    4.262990E+03, 4.330675E+03, 4.398878E+03, 4.467600E+03, 4.536843E+03, 4.606606E+03,
    4.676891E+03, 4.747698E+03, 4.819026E+03, 4.890878E+03, 4.963252E+03, 5.036150E+03,
    5.109571E+03, 5.183515E+03, 5.257983E+03, 5.332975E+03, 5.408491E+03, 5.484530E+03,
    5.561093E+03, 5.638179E+03, 5.715789E+03, 5.793921E+03, 5.872577E+03, 5.951754E+03,
    6.031454E+03, 6.111676E+03, 6.192419E+03, 6.273682E+03, 6.355466E+03, 6.437769E+03,
    6.520592E+03, 6.603933E+03, 6.687791E+03, 6.772167E+03, 6.857060E+03, 6.942467E+03,
    7.028390E+03, 7.114827E+03, 7.201776E+03, 7.289238E+03, 7.377212E+03, 7.465695E+03,
    7.554688E+03, 7.644190E+03, 7.734198E+03, 7.824713E+03, 7.915734E+03, 8.007258E+03,
    8.099285E+03, 8.191814E+03, 8.284844E+03, 8.378373E+03, 8.472401E+03, 8.566925E+03,
    8.661945E+03, 8.757460E+03, 8.853467E+03, 8.949967E+03, 9.046956E+03, 9.144435E+03,
    9.242402E+03, 9.340855E+03, 9.439793E+03, 9.539214E+03, 9.639117E+03, 9.739501E+03,
    9.840364E+03, 9.941705E+03, 1.004352E+04, 1.014581E+04, 1.024858E+04, 1.035181E+04,
    1.045552E+04, 1.055969E+04, 1.066433E+04, 1.076944E+04, 1.087501E+04, 1.098104E+04,
    1.108753E+04, 1.119448E+04, 1.130189E+04, 1.140975E+04, 1.151807E+04, 1.162684E+04,
    1.173606E+04, 1.184573E+04, 1.195584E+04, 1.206641E+04, 1.217741E+04, 1.228886E+04,
    1.240074E+04, 1.251307E+04, 1.262583E+04, 1.273903E+04, 1.285266E+04, 1.296672E+04,
    1.308121E+04, 1.319613E+04, 1.331148E+04, 1.342725E+04, 1.354344E+04, 1.366005E+04,
    1.377709E+04, 1.389454E+04, 1.401240E+04, 1.413069E+04, 1.424938E+04, 1.436848E+04,
    1.448799E+04, 1.460791E+04, 1.472823E+04, 1.484896E+04, 1.497009E+04, 1.509161E+04,
    1.521354E+04, 1.533586E+04, 1.545857E+04, 1.558168E+04, 1.570518E+04, 1.582906E+04,
    1.595333E+04, 1.607799E+04, 1.620303E+04, 1.632846E+04, 1.645426E+04, 1.658044E+04,
    1.670699E+04, 1.683392E+04, 1.696123E+04, 1.708890E+04, 1.721694E+04, 1.734535E+04,
    1.747412E+04, 1.760326E+04, 1.773276E+04, 1.786262E+04, 1.799284E+04, 1.812341E+04,
    1.825434E+04, 1.838562E+04, 1.851725E+04, 1.864923E+04, 1.878155E+04, 1.891423E+04,
    1.904724E+04, 1.918060E+04, 1.931430E+04, 1.944833E+04, 1.958271E+04, 1.971742E+04,
    1.985246E+04, 1.998783E+04, 2.012353E+04, 2.025956E+04, 2.039591E+04, 2.053259E+04,
    2.066960E+04, 2.080692E+04, 2.094456E+04, 2.108252E+04, 2.122079E+04, 2.135938E+04,
    2.149828E+04, 2.163749E+04, 2.177701E+04, 2.191683E+04, 2.205696E+04, 2.219740E+04,
    2.233813E+04, 2.247917E+04, 2.262050E+04, 2.276213E+04, 2.290406E+04, 2.304628E+04,
    2.318879E+04, 2.333159E+04, 2.347468E+04, 2.361805E+04, 2.376171E+04, 2.390565E+04,
    2.404987E+04, 2.419438E+04, 2.433916E+04, 2.448422E+04, 2.462955E+04, 2.477516E+04,
    2.492103E+04, 2.506718E+04, 2.521360E+04, 2.536028E+04, 2.550723E+04, 2.565444E+04,
    2.580191E+04, 2.594964E+04, 2.609763E+04, 2.624588E+04, 2.639439E+04, 2.654314E+04,
    2.669215E+04, 2.684141E+04, 2.699092E+04, 2.714068E+04, 2.729068E+04, 2.744093E+04,
    2.759142E+04, 2.774215E+04, 2.789312E+04, 2.804433E+04, 2.819578E+04, 2.834746E+04,
    2.849937E+04, 2.865152E+04, 2.880390E+04, 2.895650E+04, 2.910934E+04, 2.926240E+04,
    2.941568E+04, 2.956919E+04, 2.972292E+04, 2.987687E+04, 3.003104E+04, 3.018542E+04,
    3.034002E+04, 3.049484E+04, 3.064987E+04, 3.080511E+04, 3.096055E+04, 3.111621E+04,
    3.127208E+04, 3.142815E+04, 3.158442E+04, 3.174090E+04, 3.189758E+04, 3.205446E+04,
    3.221153E+04, 3.236881E+04, 3.252628E+04, 3.268394E+04, 3.284180E+04, 3.299985E+04,
    3.315809E+04, 3.331652E+04, 3.347514E+04, 3.363394E+04, 3.379293E+04, 3.395210E+04,
    3.411145E+04, 3.427098E+04, 3.443070E+04, 3.459059E+04, 3.475066E+04, 3.491090E+04,
    3.507132E+04, 3.523191E+04, 3.539268E+04, 3.555361E+04, 3.571471E+04, 3.587598E+04,
    3.603742E+04, 3.619902E+04, 3.636079E+04, 3.652272E+04, 3.668481E+04, 3.684706E+04,
    3.700947E+04, 3.717204E+04, 3.733477E+04, 3.749765E+04, 3.766069E+04, 3.782387E+04,
    3.798721E+04, 3.815070E+04, 3.831434E+04, 3.847813E+04, 3.864207E+04, 3.880615E+04,
    3.897037E+04, 3.913474E+04, 3.929926E+04, 3.946391E+04, 3.962870E+04, 3.979363E+04,
    3.995870E+04, 4.012391E+04, 4.028925E+04, 4.045473E+04, 4.062034E+04, 4.078608E+04,
    4.095195E+04, 4.111796E+04, 4.128409E+04, 4.145035E+04, 4.161673E+04, 4.178324E+04,
    4.194988E+04, 4.211664E+04, 4.228352E+04, 4.245053E+04, 4.261765E+04, 4.278489E+04,
    4.295225E+04, 4.311973E+04, 4.328733E+04, 4.345503E+04, 4.362286E+04, 4.379079E+04,
    4.395884E+04, 4.412700E+04, 4.429527E+04, 4.446365E+04, 4.463213E+04, 4.480073E+04,
    4.496943E+04, 4.513823E+04, 4.530714E+04, 4.547615E+04, 4.564526E+04, 4.581448E+04,
    4.598379E+04, 4.615320E+04, 4.632272E+04, 4.649233E+04, 4.666203E+04, 4.683183E+04,
    4.700173E+04, 4.717172E+04, 4.734180E+04, 4.751197E+04, 4.768224E+04, 4.785259E+04,
    4.802304E+04, 4.819357E+04, 4.836419E+04, 4.853489E+04, 4.870568E+04, 4.887656E+04,
    4.904752E+04, 4.921856E+04, 4.938969E+04, 4.956089E+04, 4.973218E+04, 4.990354E+04,
    5.007499E+04, 5.024651E+04, 5.041811E+04, 5.058978E+04, 5.076153E+04, 5.093336E+04,
    5.110526E+04, 5.127723E+04, 5.144927E+04, 5.162139E+04, 5.179357E+04, 5.196583E+04,
    5.213815E+04, 5.231054E+04, 5.248300E+04, 5.265553E+04, 5.282812E+04, 5.300078E+04,
    5.317350E+04, 5.334628E+04, 5.351913E+04, 5.369204E+04, 5.386501E+04, 5.403804E+04,
    5.421113E+04,
])

#  ---------------------- M = 22, I = 3 ---------------------------
M = 22
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[3]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.042930E+00, 1.565169E+01, 3.061038E+01, 4.557650E+01, 6.054536E+01, 7.551604E+01,
    9.048823E+01, 1.054618E+02, 1.204367E+02, 1.354129E+02, 1.503903E+02, 1.653691E+02,
    1.803492E+02, 1.953309E+02, 2.103145E+02, 2.253008E+02, 2.402907E+02, 2.552858E+02,
    2.702880E+02, 2.852999E+02, 3.003246E+02, 3.153656E+02, 3.304270E+02, 3.455134E+02,
    3.606296E+02, 3.757809E+02, 3.909726E+02, 4.062104E+02, 4.214999E+02, 4.368470E+02,
    4.522573E+02, 4.677366E+02, 4.832903E+02, 4.989241E+02, 5.146432E+02, 5.304529E+02,
    5.463581E+02, 5.623636E+02, 5.784740E+02, 5.946939E+02, 6.110273E+02, 6.274783E+02,
    6.440507E+02, 6.607482E+02, 6.775743E+02, 6.945321E+02, 7.116249E+02, 7.288554E+02,
    7.462266E+02, 7.637410E+02, 7.814012E+02, 7.992095E+02, 8.171680E+02, 8.352790E+02,
    8.535443E+02, 8.719660E+02, 8.905456E+02, 9.092849E+02, 9.281856E+02, 9.472490E+02,
    9.664766E+02, 9.858697E+02, 1.005430E+03, 1.025157E+03, 1.045054E+03, 1.065122E+03,
    1.085360E+03, 1.105770E+03, 1.126354E+03, 1.147111E+03, 1.168043E+03, 1.189150E+03,
    1.210433E+03, 1.231893E+03, 1.253530E+03, 1.275346E+03, 1.297340E+03, 1.319512E+03,
    1.341865E+03, 1.364397E+03, 1.387110E+03, 1.410003E+03, 1.433077E+03, 1.456333E+03,
    1.479771E+03, 1.503390E+03, 1.527192E+03, 1.551176E+03, 1.575343E+03, 1.599692E+03,
    1.624225E+03, 1.648941E+03, 1.673840E+03, 1.698922E+03, 1.724188E+03, 1.749637E+03,
    1.775269E+03, 1.801085E+03, 1.827085E+03, 1.853267E+03, 1.879634E+03, 1.906183E+03,
    1.932916E+03, 1.959832E+03, 1.986931E+03, 2.014212E+03, 2.041677E+03, 2.069324E+03,
    2.097154E+03, 2.125166E+03, 2.153359E+03, 2.181735E+03, 2.210292E+03, 2.239031E+03,
    2.267951E+03, 2.297051E+03, 2.326332E+03, 2.355794E+03, 2.385435E+03, 2.415256E+03,
    2.445256E+03, 2.475436E+03, 2.505794E+03, 2.536330E+03, 2.567044E+03, 2.597935E+03,
    2.629004E+03, 2.660250E+03, 2.691671E+03, 2.723269E+03, 2.755043E+03, 2.786991E+03,
    2.819114E+03, 2.851411E+03, 2.883882E+03, 2.916526E+03, 2.949343E+03, 2.982332E+03,
    3.015493E+03, 3.048826E+03, 3.082329E+03, 3.116002E+03, 3.149846E+03, 3.183859E+03,
    3.218040E+03, 3.252390E+03, 3.286907E+03, 3.321592E+03, 3.356443E+03, 3.391461E+03,
    3.426643E+03, 3.461991E+03, 3.497504E+03, 3.533180E+03, 3.569019E+03, 3.605021E+03,
    3.641186E+03, 3.677511E+03, 3.713998E+03, 3.750645E+03, 3.787452E+03, 3.824418E+03,
    3.861542E+03, 3.898825E+03, 3.936265E+03, 3.973861E+03, 4.011614E+03, 4.049522E+03,
    4.087585E+03, 4.125803E+03, 4.164174E+03, 4.202698E+03, 4.241375E+03, 4.280203E+03,
    4.319182E+03, 4.358312E+03, 4.397592E+03, 4.437021E+03, 4.476599E+03, 4.516325E+03,
    4.556198E+03, 4.596218E+03, 4.636384E+03, 4.676695E+03, 4.717151E+03, 4.757752E+03,
    4.798495E+03, 4.839382E+03, 4.880411E+03, 4.921581E+03, 4.962892E+03, 5.004344E+03,
    5.045935E+03, 5.087665E+03, 5.129534E+03, 5.171540E+03, 5.213683E+03, 5.255963E+03,
    5.298378E+03, 5.340928E+03, 5.383613E+03, 5.426432E+03, 5.469384E+03, 5.512468E+03,
    5.555684E+03, 5.599031E+03, 5.642509E+03, 5.686116E+03, 5.729853E+03, 5.773719E+03,
    5.817712E+03, 5.861833E+03, 5.906080E+03, 5.950453E+03, 5.994952E+03, 6.039576E+03,
    6.084323E+03, 6.129194E+03, 6.174188E+03, 6.219304E+03, 6.264541E+03, 6.309899E+03,
    6.355378E+03, 6.400976E+03, 6.446693E+03, 6.492528E+03, 6.538481E+03, 6.584551E+03,
    6.630738E+03, 6.677040E+03, 6.723457E+03, 6.769989E+03, 6.816635E+03, 6.863394E+03,
    6.910266E+03, 6.957250E+03, 7.004345E+03, 7.051551E+03, 7.098867E+03, 7.146292E+03,
    7.193827E+03, 7.241469E+03, 7.289220E+03, 7.337077E+03, 7.385041E+03, 7.433111E+03,
    7.481286E+03, 7.529565E+03, 7.577949E+03, 7.626436E+03, 7.675025E+03, 7.723717E+03,
    7.772511E+03, 7.821405E+03, 7.870400E+03, 7.919495E+03, 7.968689E+03, 8.017981E+03,
    8.067371E+03, 8.116859E+03, 8.166444E+03, 8.216125E+03, 8.265901E+03, 8.315773E+03,
    8.365739E+03, 8.415798E+03, 8.465952E+03, 8.516198E+03, 8.566536E+03, 8.616965E+03,
    8.667486E+03, 8.718097E+03, 8.768798E+03, 8.819588E+03, 8.870467E+03, 8.921434E+03,
    8.972489E+03, 9.023631E+03, 9.074859E+03, 9.126174E+03, 9.177573E+03, 9.229058E+03,
    9.280626E+03, 9.332279E+03, 9.384014E+03, 9.435833E+03, 9.487733E+03, 9.539715E+03,
    9.591778E+03, 9.643921E+03, 9.696144E+03, 9.748447E+03, 9.800828E+03, 9.853288E+03,
    9.905826E+03, 9.958440E+03, 1.001113E+04, 1.006390E+04, 1.011674E+04, 1.016966E+04,
    1.022266E+04, 1.027572E+04, 1.032886E+04, 1.038208E+04, 1.043536E+04, 1.048872E+04,
    1.054215E+04, 1.059566E+04, 1.064923E+04, 1.070287E+04, 1.075658E+04, 1.081036E+04,
    1.086421E+04, 1.091813E+04, 1.097212E+04, 1.102617E+04, 1.108029E+04, 1.113447E+04,
    1.118872E+04, 1.124304E+04, 1.129742E+04, 1.135187E+04, 1.140637E+04, 1.146095E+04,
    1.151558E+04, 1.157028E+04, 1.162504E+04, 1.167986E+04, 1.173474E+04, 1.178968E+04,
    1.184468E+04, 1.189975E+04, 1.195487E+04, 1.201005E+04, 1.206528E+04, 1.212058E+04,
    1.217593E+04, 1.223134E+04, 1.228681E+04, 1.234233E+04, 1.239791E+04, 1.245354E+04,
    1.250923E+04, 1.256497E+04, 1.262077E+04, 1.267661E+04, 1.273252E+04, 1.278847E+04,
    1.284448E+04, 1.290054E+04, 1.295665E+04, 1.301281E+04, 1.306902E+04, 1.312528E+04,
    1.318159E+04, 1.323795E+04, 1.329436E+04, 1.335081E+04, 1.340732E+04, 1.346387E+04,
    1.352047E+04, 1.357712E+04, 1.363381E+04, 1.369055E+04, 1.374734E+04, 1.380417E+04,
    1.386104E+04, 1.391796E+04, 1.397492E+04, 1.403193E+04, 1.408898E+04, 1.414607E+04,
    1.420321E+04, 1.426039E+04, 1.431761E+04, 1.437487E+04, 1.443217E+04, 1.448951E+04,
    1.454690E+04, 1.460432E+04, 1.466178E+04, 1.471928E+04, 1.477682E+04, 1.483440E+04,
    1.489202E+04, 1.494968E+04, 1.500737E+04, 1.506510E+04, 1.512287E+04, 1.518067E+04,
    1.523851E+04, 1.529638E+04, 1.535429E+04, 1.541224E+04, 1.547022E+04, 1.552823E+04,
    1.558628E+04, 1.564436E+04, 1.570248E+04, 1.576063E+04, 1.581881E+04, 1.587702E+04,
    1.593527E+04, 1.599354E+04, 1.605185E+04, 1.611019E+04, 1.616856E+04, 1.622697E+04,
    1.628540E+04, 1.634386E+04, 1.640235E+04, 1.646087E+04, 1.651942E+04, 1.657800E+04,
    1.663660E+04, 1.669524E+04, 1.675390E+04, 1.681259E+04, 1.687130E+04, 1.693005E+04,
    1.698882E+04, 1.704761E+04, 1.710643E+04, 1.716528E+04, 1.722415E+04, 1.728305E+04,
    1.734197E+04, 1.740092E+04, 1.745989E+04, 1.751889E+04, 1.757791E+04, 1.763695E+04,
    1.769602E+04, 1.775510E+04, 1.781422E+04, 1.787335E+04, 1.793250E+04, 1.799168E+04,
    1.805088E+04, 1.811010E+04, 1.816934E+04, 1.822860E+04, 1.828789E+04, 1.834719E+04,
    1.840651E+04, 1.846585E+04, 1.852522E+04, 1.858460E+04, 1.864400E+04, 1.870342E+04,
    1.876285E+04,
])

#  ---------------------- M = 23, I = 1 ---------------------------
M = 23
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.255910E+00, 5.846743E+01, 1.148737E+02, 1.712988E+02, 2.277330E+02, 2.841901E+02,
    3.407448E+02, 3.975806E+02, 4.549981E+02, 5.133834E+02, 5.731623E+02, 6.347625E+02,
    6.985902E+02, 7.650186E+02, 8.343861E+02, 9.069982E+02, 9.831325E+02, 1.063044E+03,
    1.146969E+03, 1.235132E+03, 1.327747E+03, 1.425022E+03, 1.527162E+03, 1.634371E+03,
    1.746850E+03, 1.864805E+03, 1.988442E+03, 2.117972E+03, 2.253608E+03, 2.395569E+03,
    2.544078E+03, 2.699363E+03, 2.861660E+03, 3.031206E+03, 3.208249E+03, 3.393039E+03,
    3.585834E+03, 3.786899E+03, 3.996503E+03, 4.214924E+03, 4.442444E+03, 4.679352E+03,
    4.925946E+03, 5.182528E+03, 5.449407E+03, 5.726899E+03, 6.015328E+03, 6.315021E+03,
    6.626316E+03, 6.949556E+03, 7.285090E+03, 7.644438E+03, 8.017771E+03, 8.405197E+03,
    8.807106E+03, 9.223898E+03, 9.655973E+03, 1.010375E+04, 1.056765E+04, 1.104809E+04,
    1.154553E+04, 1.206039E+04, 1.259313E+04, 1.314422E+04, 1.371412E+04, 1.430330E+04,
    1.491225E+04, 1.554147E+04, 1.619144E+04, 1.686268E+04, 1.755572E+04, 1.827106E+04,
    1.900924E+04, 1.977081E+04, 2.055631E+04, 2.136630E+04, 2.220136E+04, 2.306204E+04,
    2.394895E+04, 2.486265E+04, 2.580377E+04, 2.677291E+04, 2.777069E+04, 2.879773E+04,
    2.985468E+04, 3.094216E+04, 3.206084E+04, 3.321139E+04, 3.439446E+04, 3.561075E+04,
    3.686094E+04, 3.814573E+04, 3.946582E+04, 4.082193E+04, 4.221480E+04, 4.364516E+04,
    4.511375E+04, 4.662130E+04, 4.816861E+04, 4.975646E+04, 5.138558E+04, 5.305681E+04,
    5.477092E+04, 5.652873E+04, 5.833107E+04, 6.017876E+04, 6.207265E+04, 6.401357E+04,
    6.600238E+04, 6.803996E+04, 7.012718E+04, 7.226494E+04, 7.445411E+04, 7.669563E+04,
    7.899039E+04, 8.133932E+04, 8.374336E+04, 8.620347E+04, 8.872059E+04, 9.129568E+04,
    9.392974E+04, 9.662374E+04, 9.937866E+04, 1.021955E+05, 1.050753E+05, 1.080191E+05,
    1.110280E+05, 1.141028E+05, 1.172448E+05, 1.204550E+05, 1.237344E+05, 1.270842E+05,
    1.305054E+05, 1.339991E+05, 1.375665E+05, 1.412087E+05, 1.449268E+05, 1.487220E+05,
    1.525953E+05, 1.565481E+05, 1.605814E+05, 1.646964E+05, 1.688944E+05, 1.731765E+05,
    1.775439E+05, 1.819979E+05, 1.865398E+05, 1.911707E+05, 1.958919E+05, 2.007047E+05,
    2.056102E+05, 2.106100E+05, 2.157051E+05, 2.208970E+05, 2.261869E+05, 2.315762E+05,
    2.370662E+05, 2.426582E+05, 2.483535E+05, 2.541537E+05, 2.600599E+05, 2.660736E+05,
    2.721962E+05, 2.784291E+05, 2.847737E+05, 2.912314E+05, 2.978036E+05, 3.044918E+05,
    3.112975E+05, 3.182219E+05, 3.252667E+05, 3.324334E+05, 3.397233E+05, 3.471381E+05,
    3.546790E+05, 3.623478E+05,
])

#  ---------------------- M = 23, I = 2 ---------------------------
M = 23
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.256912E+01, 1.198179E+02, 2.355169E+02, 3.512532E+02, 4.670077E+02, 5.828098E+02,
    6.988156E+02, 8.154062E+02, 9.332049E+02, 1.053010E+03, 1.175699E+03, 1.302157E+03,
    1.433218E+03, 1.569655E+03, 1.712166E+03, 1.861382E+03, 2.017879E+03, 2.182186E+03,
    2.354797E+03, 2.536179E+03, 2.726782E+03, 2.927041E+03, 3.137388E+03, 3.358249E+03,
    3.590052E+03, 3.833228E+03, 4.088213E+03, 4.355451E+03, 4.635393E+03, 4.928501E+03,
    5.235245E+03, 5.556109E+03, 5.891584E+03, 6.242178E+03, 6.608407E+03, 6.990802E+03,
    7.389906E+03, 7.806274E+03, 8.240476E+03, 8.693093E+03, 9.164721E+03, 9.655968E+03,
    1.016746E+04, 1.069982E+04, 1.125372E+04, 1.182980E+04, 1.242875E+04, 1.305125E+04,
    1.369802E+04, 1.436977E+04, 1.506723E+04, 1.581858E+04, 1.659528E+04, 1.740150E+04,
    1.823806E+04, 1.910581E+04, 2.000560E+04, 2.093831E+04, 2.190483E+04, 2.290607E+04,
    2.394295E+04, 2.501642E+04, 2.612743E+04, 2.727695E+04, 2.846597E+04, 2.969552E+04,
    3.096661E+04, 3.228030E+04, 3.363762E+04, 3.503967E+04, 3.648755E+04, 3.798238E+04,
    3.952527E+04, 4.111738E+04, 4.275989E+04, 4.445397E+04, 4.620085E+04, 4.800173E+04,
    4.985785E+04, 5.177049E+04, 5.374092E+04, 5.577045E+04, 5.786038E+04, 6.001205E+04,
    6.222683E+04, 6.450609E+04, 6.685123E+04, 6.926364E+04, 7.174476E+04, 7.429607E+04,
    7.691903E+04, 7.961513E+04, 8.238587E+04, 8.523279E+04, 8.815748E+04, 9.116146E+04,
    9.424635E+04, 9.741373E+04, 1.006653E+05, 1.040027E+05, 1.074275E+05, 1.109415E+05,
    1.145465E+05, 1.182440E+05, 1.220360E+05, 1.259241E+05, 1.299102E+05, 1.339961E+05,
    1.381836E+05, 1.424746E+05, 1.468710E+05, 1.513747E+05, 1.559876E+05, 1.607117E+05,
    1.655489E+05, 1.705012E+05, 1.755707E+05, 1.807593E+05, 1.860692E+05, 1.915024E+05,
    1.970610E+05, 2.027471E+05, 2.085628E+05, 2.145104E+05, 2.205921E+05, 2.268099E+05,
    2.331663E+05, 2.396632E+05, 2.463032E+05, 2.530885E+05, 2.600213E+05, 2.671041E+05,
    2.743392E+05, 2.817289E+05, 2.892757E+05, 2.969820E+05, 3.048502E+05, 3.128829E+05,
    3.210824E+05, 3.294513E+05, 3.379922E+05, 3.467075E+05, 3.556000E+05, 3.646721E+05,
    3.739265E+05, 3.833658E+05, 3.929927E+05, 4.028098E+05, 4.128199E+05, 4.230257E+05,
    4.334300E+05, 4.440355E+05, 4.548450E+05, 4.658613E+05, 4.770871E+05, 4.885256E+05,
    5.001793E+05, 5.120512E+05, 5.241444E+05, 5.364618E+05, 5.490060E+05, 5.617803E+05,
    5.747876E+05, 5.880310E+05, 6.015134E+05, 6.152380E+05, 6.292077E+05, 6.434256E+05,
    6.578952E+05, 6.726192E+05, 6.876007E+05, 7.028432E+05, 7.183499E+05, 7.341236E+05,
    7.501680E+05, 7.664860E+05,
])

#  ---------------------- M = 23, I = 3 ---------------------------
M = 23
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.705510E+00, 4.011176E+01, 7.886627E+01, 1.176487E+02, 1.564527E+02, 1.952881E+02,
    2.342069E+02, 2.733368E+02, 3.128870E+02, 3.531270E+02, 3.943565E+02, 4.368808E+02,
    4.809954E+02, 5.269788E+02, 5.750913E+02, 6.255753E+02, 6.786581E+02, 7.345535E+02,
    7.934654E+02, 8.555889E+02, 9.211128E+02, 9.902210E+02, 1.063094E+03, 1.139909E+03,
    1.220842E+03, 1.306069E+03, 1.395765E+03, 1.490107E+03, 1.589273E+03, 1.693442E+03,
    1.802795E+03, 1.917518E+03, 2.037796E+03, 2.163821E+03, 2.295788E+03, 2.433892E+03,
    2.578337E+03, 2.729329E+03, 2.887077E+03, 3.051796E+03, 3.223707E+03, 3.403033E+03,
    3.590003E+03, 3.784852E+03, 3.987816E+03, 4.199140E+03, 4.419076E+03, 4.647874E+03,
    4.885797E+03, 5.133106E+03, 5.390076E+03, 5.656980E+03, 5.934099E+03, 6.221720E+03,
    6.520136E+03, 6.829645E+03, 7.150549E+03, 7.483158E+03, 7.827786E+03, 8.184755E+03,
    8.554390E+03, 8.937025E+03, 9.332999E+03, 9.742652E+03, 1.016634E+04, 1.060441E+04,
    1.105723E+04, 1.152517E+04, 1.200860E+04, 1.250790E+04, 1.302346E+04, 1.355567E+04,
    1.410492E+04, 1.467163E+04, 1.525621E+04, 1.585907E+04, 1.648063E+04, 1.712133E+04,
    1.778161E+04, 1.846190E+04, 1.916266E+04, 1.988433E+04, 2.062740E+04, 2.139232E+04,
    2.217957E+04, 2.298962E+04, 2.382298E+04, 2.468014E+04, 2.556160E+04, 2.646787E+04,
    2.739946E+04, 2.835689E+04, 2.934071E+04, 3.035144E+04, 3.138963E+04, 3.245582E+04,
    3.355059E+04, 3.467448E+04, 3.582807E+04, 3.701195E+04, 3.822668E+04, 3.947288E+04,
    4.075113E+04, 4.206204E+04, 4.340624E+04, 4.478434E+04, 4.619696E+04, 4.764476E+04,
    4.912835E+04, 5.064840E+04, 5.220557E+04, 5.380051E+04, 5.543391E+04, 5.710644E+04,
    5.881878E+04, 6.057165E+04, 6.236571E+04, 6.420171E+04, 6.608034E+04, 6.800233E+04,
    6.996841E+04, 7.197933E+04, 7.403584E+04, 7.613867E+04, 7.828858E+04, 8.048637E+04,
    8.273280E+04, 8.502866E+04, 8.737470E+04, 8.977178E+04, 9.222067E+04, 9.472220E+04,
    9.727719E+04, 9.988645E+04, 1.025508E+05, 1.052712E+05, 1.080484E+05, 1.108832E+05,
    1.137766E+05, 1.167294E+05, 1.197425E+05, 1.228168E+05, 1.259531E+05, 1.291525E+05,
    1.324157E+05, 1.357438E+05, 1.391376E+05, 1.425981E+05, 1.461262E+05, 1.497229E+05,
    1.533892E+05, 1.571259E+05, 1.609341E+05, 1.648147E+05, 1.687687E+05, 1.727973E+05,
    1.769011E+05, 1.810815E+05, 1.853392E+05, 1.896755E+05, 1.940913E+05, 1.985876E+05,
    2.031654E+05, 2.078260E+05, 2.125702E+05, 2.173992E+05, 2.223140E+05, 2.273157E+05,
    2.324055E+05, 2.375843E+05, 2.428533E+05, 2.482137E+05, 2.536665E+05, 2.592128E+05,
    2.648538E+05, 2.705906E+05,
])

#  ---------------------- M = 24, I = 1 ---------------------------
M = 24
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.292033E+01, 1.005064E+03, 2.758565E+03, 5.052901E+03, 7.773275E+03, 1.085963E+04,
    1.427439E+04, 1.799370E+04, 2.200520E+04, 2.630720E+04, 3.090790E+04, 3.582448E+04,
    4.108212E+04, 4.671306E+04, 5.275584E+04, 5.925467E+04, 6.625898E+04, 7.382313E+04,
    8.200629E+04, 9.087247E+04, 1.004906E+05, 1.109347E+05, 1.222845E+05, 1.346250E+05,
    1.480481E+05, 1.626518E+05, 1.785416E+05, 1.958306E+05, 2.146405E+05, 2.351016E+05,
    2.573542E+05, 2.815485E+05, 3.078462E+05, 3.364206E+05, 3.674579E+05, 4.011576E+05,
    4.377339E+05, 4.774164E+05, 5.204511E+05, 5.671017E+05, 6.176507E+05, 6.724001E+05,
    7.316734E+05, 7.958165E+05, 8.651993E+05, 9.402168E+05, 1.021291E+06, 1.108873E+06,
    1.203444E+06, 1.305515E+06, 1.415636E+06, 1.534387E+06, 1.662391E+06, 1.800308E+06,
    1.948842E+06, 2.108742E+06, 2.280804E+06, 2.465875E+06, 2.664853E+06, 2.878694E+06,
    3.108412E+06, 3.355083E+06, 3.619849E+06, 3.903920E+06, 4.208579E+06, 4.535186E+06,
    4.885181E+06, 5.260089E+06, 5.661522E+06, 6.091189E+06, 6.550894E+06, 7.042545E+06,
    7.568161E+06, 8.129872E+06, 8.729927E+06, 9.370703E+06, 1.005471E+07, 1.078458E+07,
    1.156312E+07, 1.239325E+07, 1.327809E+07, 1.422089E+07, 1.522509E+07, 1.629431E+07,
    1.743237E+07, 1.864327E+07, 1.993123E+07, 2.130068E+07, 2.275629E+07, 2.430295E+07,
    2.594581E+07, 2.769028E+07, 2.954203E+07, 3.150702E+07, 3.359151E+07, 3.580204E+07,
    3.814551E+07, 4.062912E+07, 4.326044E+07, 4.604738E+07, 4.899825E+07, 5.212174E+07,
    5.542695E+07, 5.892342E+07, 6.262112E+07, 6.653048E+07, 7.066241E+07, 7.502834E+07,
    7.964020E+07, 8.451045E+07, 8.965214E+07, 9.507889E+07, 1.008049E+08, 1.068451E+08,
    1.132149E+08, 1.199306E+08, 1.270089E+08, 1.344676E+08, 1.423251E+08, 1.506005E+08,
    1.593137E+08, 1.684857E+08, 1.781381E+08, 1.882936E+08, 1.989757E+08, 2.102091E+08,
    2.220191E+08, 2.344325E+08, 2.474770E+08, 2.611813E+08, 2.755755E+08, 2.906907E+08,
    3.065595E+08, 3.232155E+08, 3.406938E+08, 3.590310E+08, 3.782647E+08, 3.984346E+08,
    4.195814E+08, 4.417476E+08, 4.649774E+08, 4.893165E+08, 5.148126E+08, 5.415149E+08,
    5.694747E+08, 5.987452E+08, 6.293814E+08, 6.614407E+08, 6.949822E+08, 7.300674E+08,
    7.667602E+08, 8.051266E+08, 8.452350E+08, 8.871565E+08, 9.309645E+08, 9.767353E+08,
    1.024548E+09, 1.074483E+09, 1.126626E+09, 1.181065E+09, 1.237890E+09, 1.297194E+09,
    1.359076E+09, 1.423634E+09, 1.490973E+09, 1.561201E+09, 1.634429E+09, 1.710770E+09,
    1.790345E+09, 1.873276E+09, 1.959690E+09, 2.049717E+09, 2.143494E+09, 2.241160E+09,
    2.342859E+09, 2.448743E+09,
])

#  ---------------------- M = 24, I = 2 ---------------------------
M = 24
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.320610E+01, 1.020802E+03, 2.802064E+03, 5.132731E+03, 7.896193E+03, 1.103144E+04,
    1.450032E+04, 1.827857E+04, 2.235364E+04, 2.672383E+04, 3.139746E+04, 3.639199E+04,
    4.173298E+04, 4.745320E+04, 5.359179E+04, 6.019368E+04, 6.730905E+04, 7.499315E+04,
    8.330609E+04, 9.231288E+04, 1.020835E+05, 1.126934E+05, 1.242231E+05, 1.367594E+05,
    1.503953E+05, 1.652307E+05, 1.813726E+05, 1.989359E+05, 2.180442E+05, 2.388299E+05,
    2.614355E+05, 2.860137E+05, 3.127287E+05, 3.417565E+05, 3.732862E+05, 4.075207E+05,
    4.446774E+05, 4.849896E+05, 5.287073E+05, 5.760983E+05, 6.274494E+05, 6.830678E+05,
    7.432819E+05, 8.084431E+05, 8.789272E+05, 9.551356E+05, 1.037497E+06, 1.126469E+06,
    1.222541E+06, 1.326233E+06, 1.438102E+06, 1.558738E+06, 1.688774E+06, 1.828881E+06,
    1.979774E+06, 2.142213E+06, 2.317007E+06, 2.505017E+06, 2.707155E+06, 2.924392E+06,
    3.157758E+06, 3.408347E+06, 3.677317E+06, 3.965900E+06, 4.275399E+06, 4.607193E+06,
    4.962748E+06, 5.343611E+06, 5.751421E+06, 6.187913E+06, 6.654922E+06, 7.154384E+06,
    7.688351E+06, 8.258986E+06, 8.868575E+06, 9.519533E+06, 1.021440E+07, 1.095588E+07,
    1.174678E+07, 1.259011E+07, 1.348901E+07, 1.444679E+07, 1.546695E+07, 1.655317E+07,
    1.770932E+07, 1.893946E+07, 2.024790E+07, 2.163912E+07, 2.311786E+07, 2.468911E+07,
    2.635809E+07, 2.813029E+07, 3.001148E+07, 3.200771E+07, 3.412534E+07, 3.637102E+07,
    3.875175E+07, 4.127485E+07, 4.394801E+07, 4.677927E+07, 4.977706E+07, 5.295023E+07,
    5.630800E+07, 5.986008E+07, 6.361659E+07, 6.758813E+07, 7.178578E+07, 7.622116E+07,
    8.090637E+07, 8.585410E+07, 9.107759E+07, 9.659067E+07, 1.024078E+08, 1.085441E+08,
    1.150152E+08, 1.218377E+08, 1.290287E+08, 1.366061E+08, 1.445886E+08, 1.529956E+08,
    1.618476E+08, 1.711655E+08, 1.809715E+08, 1.912887E+08, 2.021408E+08, 2.135529E+08,
    2.255509E+08, 2.381619E+08, 2.514140E+08, 2.653365E+08, 2.799598E+08, 2.953157E+08,
    3.114371E+08, 3.283583E+08, 3.461149E+08, 3.647439E+08, 3.842840E+08, 4.047750E+08,
    4.262585E+08, 4.487777E+08, 4.723774E+08, 4.971041E+08, 5.230062E+08, 5.501338E+08,
    5.785389E+08, 6.082756E+08, 6.393998E+08, 6.719697E+08, 7.060455E+08, 7.416896E+08,
    7.789669E+08, 8.179445E+08, 8.586919E+08, 9.012813E+08, 9.457873E+08, 9.922872E+08,
    1.040861E+09, 1.091593E+09, 1.144567E+09, 1.199873E+09, 1.257603E+09, 1.317853E+09,
    1.380720E+09, 1.446308E+09, 1.514720E+09, 1.586067E+09, 1.660462E+09, 1.738021E+09,
    1.818864E+09, 1.903117E+09, 1.990908E+09, 2.082370E+09, 2.177642E+09, 2.276865E+09,
    2.380187E+09, 2.487758E+09,
])

#  ---------------------- M = 25, I = 1 ---------------------------
M = 25
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.770090E+00, 9.937734E+01, 3.215728E+02, 6.242169E+02, 9.940464E+02, 1.428783E+03,
    1.930438E+03, 2.502407E+03, 3.148468E+03, 3.872571E+03, 4.678893E+03, 5.571933E+03,
    6.556583E+03, 7.638169E+03, 8.822458E+03, 1.011566E+04, 1.152440E+04, 1.305573E+04,
    1.471706E+04, 1.651621E+04, 1.846133E+04, 2.056093E+04, 2.282389E+04, 2.525940E+04,
    2.787700E+04, 3.068660E+04, 3.369842E+04, 3.692304E+04, 4.037142E+04, 4.405485E+04,
    4.798501E+04, 5.217394E+04, 5.663409E+04, 6.137826E+04, 6.641968E+04, 7.177196E+04,
    7.744912E+04, 8.346559E+04, 8.983619E+04, 9.657618E+04, 1.037012E+05, 1.112273E+05,
    1.191709E+05, 1.275489E+05, 1.363785E+05, 1.456772E+05, 1.554631E+05, 1.657543E+05,
    1.765695E+05, 1.879275E+05, 1.998476E+05, 2.123492E+05, 2.254519E+05, 2.391757E+05,
    2.535406E+05, 2.685667E+05, 2.842744E+05, 3.006842E+05, 3.178165E+05, 3.356919E+05,
    3.543309E+05, 3.737539E+05, 3.939815E+05, 4.150340E+05, 4.369316E+05, 4.596945E+05,
    4.833425E+05, 5.078955E+05, 5.333729E+05, 5.597939E+05, 5.871776E+05, 6.155426E+05,
    6.449072E+05, 6.752896E+05, 7.067073E+05, 7.391776E+05, 7.727174E+05, 8.073431E+05,
    8.430708E+05, 8.799161E+05, 9.178940E+05, 9.570193E+05, 9.973060E+05, 1.038768E+06,
    1.081418E+06, 1.125270E+06, 1.170335E+06, 1.216625E+06, 1.264150E+06, 1.312923E+06,
    1.362953E+06, 1.414249E+06, 1.466821E+06, 1.520677E+06, 1.575826E+06, 1.632274E+06,
    1.690030E+06, 1.749098E+06, 1.809486E+06, 1.871200E+06, 1.934243E+06, 1.998620E+06,
    2.064337E+06, 2.131395E+06, 2.199799E+06, 2.269551E+06, 2.340654E+06, 2.413108E+06,
    2.486916E+06, 2.562078E+06, 2.638594E+06, 2.716466E+06, 2.795692E+06, 2.876272E+06,
    2.958205E+06, 3.041488E+06, 3.126122E+06, 3.212102E+06, 3.299428E+06, 3.388095E+06,
    3.478101E+06, 3.569442E+06, 3.662114E+06, 3.756113E+06, 3.851435E+06, 3.948076E+06,
    4.046029E+06, 4.145290E+06, 4.245854E+06, 4.347714E+06, 4.450865E+06, 4.555301E+06,
    4.661014E+06, 4.767999E+06, 4.876249E+06, 4.985756E+06, 5.096513E+06, 5.208514E+06,
    5.321749E+06, 5.436213E+06, 5.551896E+06, 5.668791E+06, 5.786890E+06, 5.906183E+06,
    6.026663E+06, 6.148322E+06, 6.271150E+06, 6.395138E+06, 6.520279E+06, 6.646561E+06,
    6.773978E+06, 6.902519E+06, 7.032175E+06, 7.162937E+06, 7.294795E+06, 7.427739E+06,
    7.561761E+06, 7.696851E+06, 7.832999E+06, 7.970195E+06, 8.108429E+06, 8.247692E+06,
    8.387974E+06, 8.529265E+06, 8.671555E+06, 8.814834E+06, 8.959093E+06, 9.104320E+06,
    9.250507E+06, 9.397644E+06, 9.545720E+06, 9.694725E+06, 9.844650E+06, 9.995484E+06,
    1.014722E+07, 1.029984E+07, 1.045334E+07, 1.060772E+07, 1.076295E+07, 1.091903E+07,
    1.107595E+07, 1.123370E+07, 1.139227E+07, 1.155166E+07, 1.171184E+07, 1.187281E+07,
    1.203457E+07, 1.219709E+07, 1.236038E+07, 1.252442E+07, 1.268921E+07, 1.285472E+07,
    1.302096E+07, 1.318792E+07, 1.335558E+07, 1.352393E+07, 1.369297E+07, 1.386269E+07,
    1.403307E+07, 1.420412E+07, 1.437581E+07, 1.454814E+07, 1.472111E+07, 1.489470E+07,
    1.506890E+07, 1.524370E+07, 1.541910E+07, 1.559509E+07, 1.577166E+07, 1.594880E+07,
    1.612649E+07, 1.630475E+07, 1.648354E+07, 1.666287E+07, 1.684273E+07, 1.702311E+07,
    1.720400E+07, 1.738539E+07, 1.756728E+07, 1.774966E+07, 1.793251E+07, 1.811584E+07,
    1.829962E+07, 1.848387E+07, 1.866856E+07, 1.885369E+07, 1.903926E+07, 1.922525E+07,
    1.941166E+07, 1.959847E+07, 1.978569E+07, 1.997331E+07, 2.016131E+07, 2.034970E+07,
    2.053846E+07, 2.072759E+07, 2.091708E+07, 2.110692E+07, 2.129710E+07, 2.148763E+07,
    2.167849E+07, 2.186967E+07, 2.206118E+07, 2.225300E+07, 2.244512E+07, 2.263755E+07,
    2.283027E+07, 2.302327E+07, 2.321656E+07, 2.341012E+07, 2.360396E+07, 2.379805E+07,
    2.399240E+07, 2.418700E+07, 2.438185E+07, 2.457694E+07, 2.477226E+07, 2.496781E+07,
    2.516358E+07, 2.535956E+07, 2.555576E+07, 2.575216E+07, 2.594876E+07, 2.614556E+07,
    2.634255E+07, 2.653972E+07, 2.673707E+07, 2.693459E+07, 2.713228E+07, 2.733013E+07,
    2.752815E+07, 2.772631E+07, 2.792462E+07, 2.812308E+07, 2.832168E+07, 2.852041E+07,
    2.871927E+07, 2.891825E+07, 2.911736E+07, 2.931658E+07, 2.951591E+07, 2.971535E+07,
    2.991488E+07, 3.011452E+07, 3.031425E+07, 3.051407E+07, 3.071398E+07, 3.091396E+07,
    3.111402E+07, 3.131416E+07, 3.151436E+07, 3.171463E+07, 3.191495E+07, 3.211534E+07,
    3.231577E+07, 3.251626E+07, 3.271679E+07, 3.291736E+07, 3.311797E+07, 3.331861E+07,
    3.351929E+07,
])

#  ---------------------- M = 26, I = 1 ---------------------------
M = 26
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.304830E+00, 2.430655E+01, 4.793073E+01, 7.156039E+01, 9.519594E+01, 1.188684E+02,
    1.426920E+02, 1.668995E+02, 1.918282E+02, 2.178787E+02, 2.454753E+02, 2.750411E+02,
    3.069871E+02, 3.417113E+02, 3.796013E+02, 4.210401E+02, 4.664113E+02, 5.161047E+02,
    5.705201E+02, 6.300721E+02, 6.951927E+02, 7.663344E+02, 8.439731E+02, 9.286097E+02,
    1.020773E+03, 1.121022E+03, 1.229946E+03, 1.348169E+03, 1.476351E+03, 1.615189E+03,
    1.765420E+03, 1.927822E+03, 2.103220E+03, 2.292482E+03, 2.496526E+03, 2.716320E+03,
    2.952886E+03, 3.207303E+03, 3.480706E+03, 3.774293E+03, 4.089326E+03, 4.427131E+03,
    4.789107E+03, 5.176724E+03, 5.591528E+03, 6.035144E+03, 6.509280E+03, 7.015728E+03,
    7.556371E+03, 8.133184E+03, 8.748238E+03, 9.403705E+03, 1.010186E+04, 1.084508E+04,
    1.163587E+04, 1.247683E+04, 1.337069E+04, 1.432031E+04, 1.532865E+04, 1.639884E+04,
    1.753411E+04, 1.873786E+04, 2.001359E+04, 2.136500E+04, 2.279588E+04, 2.431024E+04,
    2.591218E+04, 2.760600E+04, 2.939617E+04, 3.128728E+04, 3.328414E+04, 3.539170E+04,
    3.761510E+04, 3.995963E+04, 4.243078E+04, 4.503422E+04, 4.777578E+04, 5.066149E+04,
    5.369755E+04, 5.689035E+04, 6.024647E+04, 6.377264E+04, 6.747581E+04, 7.136309E+04,
    7.544178E+04, 7.971935E+04, 8.420347E+04, 8.890196E+04, 9.382283E+04, 9.897425E+04,
    1.043646E+05, 1.100024E+05, 1.158962E+05, 1.220551E+05, 1.284878E+05, 1.352037E+05,
    1.422121E+05, 1.495223E+05, 1.571439E+05, 1.650868E+05, 1.733608E+05, 1.819759E+05,
    1.909421E+05, 2.002699E+05, 2.099695E+05, 2.200513E+05, 2.305261E+05, 2.414044E+05,
    2.526970E+05, 2.644148E+05, 2.765688E+05, 2.891699E+05, 3.022293E+05, 3.157581E+05,
    3.297675E+05, 3.442689E+05, 3.592735E+05, 3.747927E+05, 3.908380E+05, 4.074208E+05,
    4.245525E+05, 4.422448E+05, 4.605090E+05, 4.793568E+05, 4.987996E+05, 5.188490E+05,
    5.395165E+05, 5.608137E+05, 5.827520E+05, 6.053430E+05, 6.285981E+05, 6.525288E+05,
    6.771464E+05, 7.024624E+05, 7.284880E+05, 7.552345E+05, 7.827132E+05, 8.109352E+05,
    8.399115E+05, 8.696534E+05, 9.001717E+05, 9.314773E+05, 9.635811E+05, 9.964938E+05,
    1.030226E+06, 1.064788E+06, 1.100191E+06, 1.136445E+06, 1.173561E+06, 1.211548E+06,
    1.250416E+06, 1.290176E+06, 1.330837E+06, 1.372410E+06, 1.414904E+06, 1.458328E+06,
    1.502692E+06, 1.548005E+06, 1.594276E+06, 1.641515E+06, 1.689730E+06, 1.738930E+06,
    1.789124E+06, 1.840321E+06, 1.892528E+06, 1.945754E+06, 2.000007E+06, 2.055296E+06,
    2.111627E+06, 2.169010E+06, 2.227450E+06, 2.286957E+06, 2.347537E+06, 2.409197E+06,
    2.471944E+06, 2.535786E+06, 2.600728E+06, 2.666778E+06, 2.733942E+06, 2.802227E+06,
    2.871638E+06, 2.942181E+06, 3.013863E+06, 3.086689E+06, 3.160664E+06, 3.235795E+06,
    3.312086E+06, 3.389542E+06, 3.468170E+06, 3.547972E+06, 3.628955E+06, 3.711122E+06,
    3.794479E+06, 3.879029E+06, 3.964776E+06, 4.051725E+06, 4.139880E+06, 4.229243E+06,
    4.319820E+06, 4.411612E+06, 4.504625E+06, 4.598859E+06, 4.694320E+06, 4.791009E+06,
    4.888930E+06, 4.988084E+06, 5.088475E+06, 5.190105E+06, 5.292976E+06, 5.397090E+06,
    5.502450E+06, 5.609057E+06, 5.716912E+06, 5.826018E+06, 5.936376E+06, 6.047987E+06,
    6.160852E+06, 6.274973E+06, 6.390351E+06, 6.506986E+06, 6.624880E+06, 6.744032E+06,
    6.864444E+06, 6.986115E+06, 7.109047E+06, 7.233239E+06, 7.358692E+06, 7.485405E+06,
    7.613378E+06, 7.742611E+06, 7.873104E+06, 8.004857E+06, 8.137868E+06, 8.272137E+06,
    8.407663E+06, 8.544446E+06, 8.682485E+06, 8.821778E+06, 8.962325E+06, 9.104124E+06,
    9.247174E+06, 9.391473E+06, 9.537021E+06, 9.683815E+06, 9.831853E+06, 9.981135E+06,
    1.013166E+07, 1.028342E+07, 1.043642E+07, 1.059066E+07, 1.074612E+07,
])

#  ---------------------- M = 26, I = 2 ---------------------------
M = 26
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.814510E+00, 9.724635E+01, 1.917632E+02, 2.864734E+02, 3.811700E+02, 4.760528E+02,
    5.715776E+02, 6.686900E+02, 7.687384E+02, 8.733388E+02, 9.841987E+02, 1.103046E+03,
    1.231520E+03, 1.371261E+03, 1.523833E+03, 1.690802E+03, 1.873730E+03, 2.074207E+03,
    2.293885E+03, 2.534441E+03, 2.797671E+03, 3.085414E+03, 3.399618E+03, 3.742372E+03,
    4.115810E+03, 4.522255E+03, 4.964139E+03, 5.444011E+03, 5.964565E+03, 6.528734E+03,
    7.139491E+03, 7.800087E+03, 8.513884E+03, 9.284496E+03, 1.011567E+04, 1.101143E+04,
    1.197594E+04, 1.301371E+04, 1.412933E+04, 1.532784E+04, 1.661437E+04, 1.799441E+04,
    1.947370E+04, 2.105830E+04, 2.275461E+04, 2.456929E+04, 2.650943E+04, 2.858230E+04,
    3.079576E+04, 3.315789E+04, 3.567723E+04, 3.836273E+04, 4.122372E+04, 4.427006E+04,
    4.751198E+04, 5.096026E+04, 5.462619E+04, 5.852141E+04, 6.265841E+04, 6.704991E+04,
    7.170941E+04, 7.665096E+04, 8.188914E+04, 8.743929E+04, 9.331736E+04, 9.953997E+04,
    1.061245E+05, 1.130890E+05, 1.204523E+05, 1.282340E+05, 1.364547E+05, 1.451355E+05,
    1.542986E+05, 1.639671E+05, 1.741648E+05, 1.849166E+05, 1.962486E+05, 2.081876E+05,
    2.207615E+05, 2.339994E+05, 2.479314E+05, 2.625888E+05, 2.780041E+05, 2.942110E+05,
    3.112445E+05, 3.291406E+05, 3.479371E+05, 3.676729E+05, 3.883884E+05, 4.101252E+05,
    4.329270E+05, 4.568381E+05, 4.819053E+05, 5.081766E+05, 5.357017E+05, 5.645321E+05,
    5.947207E+05, 6.263228E+05, 6.593952E+05, 6.939969E+05, 7.301883E+05, 7.680325E+05,
    8.075944E+05, 8.489406E+05, 8.921407E+05, 9.372660E+05, 9.843902E+05, 1.033589E+06,
    1.084942E+06, 1.138530E+06, 1.194435E+06, 1.252745E+06, 1.313548E+06, 1.376935E+06,
    1.443002E+06, 1.511846E+06, 1.583566E+06, 1.658265E+06, 1.736052E+06, 1.817034E+06,
    1.901325E+06, 1.989042E+06, 2.080302E+06, 2.175231E+06, 2.273953E+06, 2.376600E+06,
    2.483305E+06, 2.594207E+06, 2.709447E+06, 2.829171E+06, 2.953528E+06, 3.082674E+06,
    3.216765E+06, 3.355965E+06, 3.500442E+06, 3.650366E+06, 3.805915E+06, 3.967268E+06,
    4.134614E+06, 4.308144E+06, 4.488050E+06, 4.674537E+06, 4.867812E+06, 5.068086E+06,
    5.275577E+06, 5.490508E+06, 5.713108E+06, 5.943613E+06, 6.182262E+06, 6.429305E+06,
    6.684995E+06, 6.949588E+06, 7.223356E+06, 7.506568E+06, 7.799506E+06, 8.102456E+06,
    8.415711E+06, 8.739574E+06, 9.074353E+06, 9.420361E+06, 9.777928E+06, 1.014738E+07,
    1.052905E+07, 1.092330E+07, 1.133048E+07, 1.175095E+07, 1.218509E+07, 1.263327E+07,
    1.309589E+07, 1.357335E+07, 1.406605E+07, 1.457441E+07, 1.509887E+07, 1.563985E+07,
    1.619781E+07, 1.677320E+07,
])

#  ---------------------- M = 26, I = 3 ---------------------------
M = 26
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.044140E+00, 8.016384E+01, 1.642897E+02, 2.484331E+02, 3.326393E+02, 4.172555E+02,
    5.031646E+02, 5.918139E+02, 6.849749E+02, 7.845357E+02, 8.923827E+02, 1.010349E+03,
    1.140206E+03, 1.283685E+03, 1.442530E+03, 1.618488E+03, 1.813370E+03, 2.029067E+03,
    2.267536E+03, 2.550029E+03, 2.842636E+03, 3.164913E+03, 3.519348E+03, 3.908620E+03,
    4.335576E+03, 4.803251E+03, 5.314814E+03, 5.873741E+03, 6.483650E+03, 7.148457E+03,
    7.872235E+03, 8.659382E+03, 9.514548E+03, 1.044272E+04, 1.144906E+04, 1.253918E+04,
    1.371897E+04, 1.499462E+04, 1.637280E+04, 1.786041E+04, 1.946491E+04, 2.119405E+04,
    2.305609E+04, 2.505975E+04, 2.721420E+04, 2.952910E+04, 3.201468E+04, 3.468164E+04,
    3.754137E+04, 4.060572E+04, 4.388721E+04, 4.739912E+04, 5.115514E+04, 5.516994E+04,
    5.945871E+04, 6.403752E+04, 6.892318E+04, 7.413333E+04, 7.968642E+04, 8.560184E+04,
    9.189984E+04, 9.860168E+04, 1.057296E+05, 1.133067E+05, 1.213575E+05, 1.299074E+05,
    1.389828E+05, 1.486115E+05, 1.588224E+05, 1.696459E+05, 1.811134E+05, 1.932580E+05,
    2.061137E+05, 2.197165E+05, 2.341036E+05, 2.493138E+05, 2.653877E+05, 2.823672E+05,
    3.002963E+05, 3.192205E+05, 3.391872E+05, 3.602457E+05, 3.824474E+05, 4.058456E+05,
    4.304954E+05, 4.564546E+05, 4.837827E+05, 5.125418E+05, 5.427962E+05, 5.746128E+05,
    6.080607E+05, 6.432115E+05, 6.801401E+05, 7.189233E+05, 7.596412E+05, 8.023769E+05,
    8.472160E+05, 8.942474E+05, 9.435629E+05, 9.952583E+05, 1.049431E+06, 1.106185E+06,
    1.165624E+06, 1.227858E+06, 1.292999E+06, 1.361164E+06, 1.432473E+06, 1.507052E+06,
    1.585028E+06, 1.666535E+06, 1.751710E+06, 1.840694E+06, 1.933633E+06, 2.030680E+06,
    2.131987E+06, 2.237718E+06, 2.348038E+06, 2.463117E+06, 2.583132E+06, 2.708264E+06,
    2.838701E+06, 2.974635E+06, 3.116266E+06, 3.263798E+06, 3.417442E+06, 3.577415E+06,
    3.743943E+06, 3.917253E+06, 4.097585E+06, 4.285180E+06, 4.480292E+06, 4.683179E+06,
    4.894105E+06, 5.113345E+06, 5.341180E+06, 5.577898E+06, 5.823797E+06, 6.079183E+06,
    6.344370E+06, 6.619679E+06, 6.905443E+06, 7.202001E+06, 7.509705E+06, 7.828915E+06,
    8.159999E+06, 8.503333E+06, 8.859312E+06, 9.228330E+06, 9.610799E+06, 1.000714E+07,
    1.041778E+07, 1.084317E+07, 1.128377E+07, 1.174002E+07, 1.221242E+07, 1.270146E+07,
    1.320763E+07, 1.373146E+07, 1.427346E+07, 1.483419E+07, 1.541419E+07, 1.601404E+07,
    1.663432E+07, 1.727563E+07, 1.793856E+07, 1.862376E+07, 1.933187E+07, 2.006354E+07,
    2.081943E+07, 2.160025E+07, 2.240670E+07, 2.323950E+07, 2.409938E+07, 2.498711E+07,
    2.590344E+07, 2.684920E+07,
])

#  ---------------------- M = 27, I = 1 ---------------------------
M = 27
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.500304E+01, 9.183261E+02, 2.577299E+03, 4.726665E+03, 7.295077E+03, 1.027634E+04,
    1.369384E+04, 1.758296E+04, 2.198535E+04, 2.695083E+04, 3.253906E+04, 3.882423E+04,
    4.589599E+04, 5.386197E+04, 6.285216E+04, 7.301797E+04, 8.453841E+04, 9.762032E+04,
    1.125046E+05, 1.294682E+05, 1.488302E+05, 1.709580E+05, 1.962724E+05, 2.252593E+05,
    2.584720E+05, 2.965481E+05, 3.402164E+05, 3.903158E+05, 4.478043E+05, 5.137787E+05,
    5.894956E+05, 6.763969E+05, 7.761243E+05, 8.905622E+05, 1.021863E+06, 1.172482E+06,
    1.345229E+06, 1.543303E+06, 1.770360E+06, 2.030568E+06, 2.328675E+06, 2.670086E+06,
    3.060957E+06, 3.508289E+06, 4.020046E+06, 4.605271E+06, 5.274240E+06, 6.038612E+06,
    6.911616E+06, 7.908245E+06, 9.045487E+06, 1.034258E+07, 1.182128E+07, 1.350621E+07,
    1.542519E+07, 1.760965E+07, 2.009505E+07, 2.292142E+07, 2.613387E+07, 2.978325E+07,
    3.392680E+07, 3.862896E+07, 4.396218E+07, 5.000792E+07, 5.685769E+07, 6.461421E+07,
    7.339274E+07, 8.332252E+07, 9.454840E+07, 1.072326E+08, 1.215567E+08, 1.377238E+08,
    1.559610E+08, 1.765221E+08, 1.996904E+08, 2.257822E+08, 2.551502E+08, 2.881877E+08,
    3.253328E+08, 3.670735E+08, 4.139525E+08, 4.665738E+08, 5.256087E+08, 5.918032E+08,
    6.659852E+08, 7.490741E+08, 8.420894E+08, 9.461613E+08, 1.062542E+09, 1.192619E+09,
    1.337927E+09, 1.500164E+09, 1.681208E+09, 1.883134E+09, 2.108234E+09, 2.359039E+09,
    2.638341E+09, 2.949219E+09, 3.295067E+09, 3.679627E+09, 4.107016E+09, 4.581768E+09,
    5.108871E+09, 5.693809E+09, 6.342612E+09, 7.061902E+09, 7.858951E+09, 8.741742E+09,
    9.719033E+09, 1.080043E+10, 1.199645E+10, 1.331863E+10, 1.477960E+10, 1.639318E+10,
    1.817449E+10, 2.014008E+10, 2.230804E+10, 2.469810E+10, 2.733187E+10, 3.023290E+10,
    3.342690E+10, 3.694193E+10, 4.080858E+10, 4.506017E+10, 4.973304E+10, 5.486675E+10,
    6.050436E+10, 6.669275E+10, 7.348290E+10, 8.093024E+10, 8.909503E+10, 9.804276E+10,
    1.078445E+11, 1.185775E+11, 1.303256E+11, 1.431796E+11, 1.572382E+11, 1.726082E+11,
    1.894054E+11, 2.077554E+11, 2.277941E+11, 2.496685E+11, 2.735379E+11, 2.995744E+11,
    3.279644E+11, 3.589092E+11, 3.926263E+11, 4.293508E+11, 4.693366E+11, 5.128578E+11,
    5.602101E+11, 6.117127E+11, 6.677096E+11, 7.285721E+11, 7.947001E+11, 8.665246E+11,
    9.445098E+11, 1.029156E+12, 1.121000E+12, 1.220623E+12, 1.328646E+12, 1.445740E+12,
    1.572626E+12, 1.710077E+12, 1.858927E+12, 2.020068E+12, 2.194462E+12, 2.383137E+12,
    2.587201E+12, 2.807841E+12, 3.046329E+12, 3.304031E+12, 3.582410E+12, 3.883036E+12,
    4.207589E+12, 4.557870E+12,
])

#  ---------------------- M = 27, I = 2 ---------------------------
M = 27
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    7.614480E+00, 4.686721E+02, 1.315543E+03, 2.412801E+03, 3.724027E+03, 5.246064E+03,
    6.990860E+03, 8.976483E+03, 1.122421E+04, 1.375949E+04, 1.661280E+04, 1.982202E+04,
    2.343294E+04, 2.750053E+04, 3.209119E+04, 3.728225E+04, 4.316512E+04, 4.984547E+04,
    5.744633E+04, 6.610916E+04, 7.599690E+04, 8.729731E+04, 1.002252E+05, 1.150289E+05,
    1.319909E+05, 1.514370E+05, 1.737394E+05, 1.993267E+05, 2.286883E+05, 2.623844E+05,
    3.010571E+05, 3.454428E+05, 3.963803E+05, 4.548324E+05, 5.218983E+05, 5.988332E+05,
    6.870716E+05, 7.882488E+05, 9.042323E+05, 1.037152E+06, 1.189433E+06, 1.363837E+06,
    1.563510E+06, 1.792029E+06, 2.053463E+06, 2.352434E+06, 2.694190E+06, 3.084692E+06,
    3.530698E+06, 4.039869E+06, 4.620887E+06, 5.283581E+06, 6.039074E+06, 6.899946E+06,
    7.880414E+06, 8.996537E+06, 1.026644E+07, 1.171059E+07, 1.335203E+07, 1.521675E+07,
    1.733400E+07, 1.973673E+07, 2.246196E+07, 2.555133E+07, 2.905161E+07, 3.301531E+07,
    3.750133E+07, 4.257574E+07, 4.831258E+07, 5.479478E+07, 6.211514E+07, 7.037750E+07,
    7.969796E+07, 9.020622E+07, 1.020472E+08, 1.153824E+08, 1.303924E+08, 1.472781E+08,
    1.662635E+08, 1.875980E+08, 2.115592E+08, 2.384560E+08, 2.686314E+08, 3.024668E+08,
    3.403857E+08, 3.828581E+08, 4.304053E+08, 4.836051E+08, 5.430980E+08, 6.095931E+08,
    6.838754E+08, 7.668134E+08, 8.593671E+08, 9.625978E+08, 1.077678E+09, 1.205900E+09,
    1.348694E+09, 1.507634E+09, 1.684456E+09, 1.881072E+09, 2.099589E+09, 2.342326E+09,
    2.611834E+09, 2.910918E+09, 3.242661E+09, 3.610451E+09, 4.018007E+09, 4.469413E+09,
    4.969148E+09, 5.522123E+09, 6.133724E+09, 6.809848E+09, 7.556956E+09, 8.382119E+09,
    9.293073E+09, 1.029828E+10, 1.140699E+10, 1.262932E+10, 1.397629E+10, 1.545998E+10,
    1.709352E+10, 1.889128E+10, 2.086891E+10, 2.304345E+10, 2.543350E+10, 2.805929E+10,
    3.094287E+10, 3.410821E+10, 3.758140E+10, 4.139081E+10, 4.556728E+10, 5.014429E+10,
    5.515826E+10, 6.064867E+10, 6.665842E+10, 7.323403E+10, 8.042594E+10, 8.828887E+10,
    9.688209E+10, 1.062698E+11, 1.165216E+11, 1.277127E+11, 1.399247E+11, 1.532456E+11,
    1.677709E+11, 1.836035E+11, 2.008548E+11, 2.196451E+11, 2.401044E+11, 2.623730E+11,
    2.866023E+11, 3.129555E+11, 3.416090E+11, 3.727526E+11, 4.065911E+11, 4.433452E+11,
    4.832525E+11, 5.265688E+11, 5.735698E+11, 6.245518E+11, 6.798340E+11, 7.397592E+11,
    8.046964E+11, 8.750421E+11, 9.512223E+11, 1.033695E+12, 1.122951E+12, 1.219518E+12,
    1.323963E+12, 1.436894E+12, 1.558962E+12, 1.690867E+12, 1.833358E+12, 1.987238E+12,
    2.153368E+12, 2.332671E+12,
])

#  ---------------------- M = 28, I = 1 ---------------------------
M = 28
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[4]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.000210E+00, 6.080058E+01, 1.634756E+02, 2.965282E+02, 4.537635E+02, 6.319162E+02,
    8.288067E+02, 1.042867E+03, 1.272974E+03, 1.518410E+03, 1.778850E+03, 2.054349E+03,
    2.345319E+03, 2.652478E+03, 2.976809E+03, 3.319511E+03, 3.681956E+03, 4.065668E+03,
    4.472291E+03, 4.903585E+03, 5.361415E+03, 5.847747E+03, 6.364650E+03, 6.914302E+03,
    7.498988E+03, 8.121115E+03, 8.783208E+03, 9.487927E+03, 1.023807E+04, 1.103656E+04,
    1.188652E+04, 1.279117E+04, 1.375395E+04, 1.477846E+04, 1.586846E+04, 1.702794E+04,
    1.826107E+04, 1.957224E+04, 2.096604E+04, 2.244732E+04, 2.402115E+04, 2.569284E+04,
    2.746798E+04, 2.935242E+04, 3.135229E+04, 3.347401E+04, 3.572430E+04, 3.811021E+04,
    4.063911E+04, 4.331870E+04, 4.615704E+04, 4.916257E+04, 5.234409E+04, 5.571079E+04,
    5.927230E+04, 6.303863E+04, 6.702025E+04, 7.122808E+04, 7.567351E+04, 8.036840E+04,
    8.532511E+04, 9.055653E+04, 9.607607E+04, 1.018977E+05, 1.080359E+05, 1.145057E+05,
    1.213230E+05, 1.285039E+05, 1.360654E+05, 1.440252E+05, 1.524013E+05, 1.612128E+05,
    1.704793E+05, 1.802210E+05, 1.904590E+05, 2.012151E+05, 2.125118E+05, 2.243723E+05,
    2.368209E+05, 2.498822E+05, 2.635820E+05, 2.779468E+05, 2.930039E+05, 3.087813E+05,
    3.253080E+05, 3.426140E+05, 3.607297E+05, 3.796869E+05, 3.995178E+05, 4.202557E+05,
    4.419347E+05, 4.645899E+05, 4.882572E+05, 5.129732E+05, 5.387755E+05, 5.657027E+05,
    5.937941E+05, 6.230898E+05, 6.536308E+05, 6.854591E+05, 7.186173E+05, 7.531488E+05,
    7.890981E+05, 8.265101E+05, 8.654309E+05, 9.059069E+05, 9.479856E+05, 9.917151E+05,
    1.037144E+06, 1.084322E+06, 1.133300E+06, 1.184128E+06, 1.236857E+06, 1.291540E+06,
    1.348229E+06, 1.406978E+06, 1.467840E+06, 1.530870E+06, 1.596122E+06, 1.663652E+06,
    1.733515E+06, 1.805768E+06, 1.880466E+06, 1.957667E+06, 2.037427E+06, 2.119805E+06,
    2.204858E+06, 2.292643E+06, 2.383220E+06, 2.476646E+06, 2.572979E+06, 2.672280E+06,
    2.774606E+06, 2.880016E+06, 2.988569E+06, 3.100324E+06, 3.215340E+06, 3.333676E+06,
    3.455390E+06, 3.580542E+06, 3.709190E+06, 3.841392E+06, 3.977207E+06, 4.116693E+06,
    4.259908E+06, 4.406909E+06, 4.557755E+06, 4.712501E+06, 4.871206E+06, 5.033924E+06,
    5.200714E+06, 5.371630E+06, 5.546727E+06, 5.726062E+06, 5.909687E+06, 6.097659E+06,
    6.290029E+06, 6.486851E+06, 6.688177E+06, 6.894061E+06, 7.104553E+06, 7.319705E+06,
    7.539566E+06, 7.764186E+06, 7.993616E+06, 8.227903E+06, 8.467096E+06, 8.711241E+06,
    8.960386E+06, 9.214576E+06, 9.473858E+06, 9.738275E+06, 1.000787E+07, 1.028269E+07,
    1.056277E+07, 1.084817E+07, 1.113891E+07, 1.143504E+07, 1.173659E+07, 1.204362E+07,
    1.235615E+07, 1.267422E+07, 1.299787E+07, 1.332714E+07, 1.366206E+07, 1.400266E+07,
    1.434898E+07, 1.470106E+07, 1.505891E+07, 1.542258E+07, 1.579209E+07, 1.616748E+07,
    1.654877E+07, 1.693599E+07, 1.732917E+07, 1.772834E+07, 1.813351E+07, 1.854472E+07,
    1.896199E+07, 1.938534E+07, 1.981480E+07, 2.025039E+07, 2.069212E+07, 2.114002E+07,
    2.159411E+07, 2.205441E+07, 2.252093E+07, 2.299369E+07, 2.347271E+07, 2.395800E+07,
    2.444958E+07, 2.494747E+07, 2.545167E+07, 2.596220E+07, 2.647907E+07, 2.700230E+07,
    2.753188E+07, 2.806784E+07, 2.861019E+07, 2.915892E+07, 2.971405E+07, 3.027559E+07,
    3.084354E+07, 3.141790E+07, 3.199869E+07, 3.258591E+07,
])

#  ---------------------- M = 29, I = 1 ---------------------------
M = 29
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.300330E+01, 1.060780E+03, 2.994480E+03, 5.499119E+03, 8.466905E+03, 1.183885E+04,
    1.558590E+04, 1.970577E+04, 2.422011E+04, 2.917030E+04, 3.461299E+04, 4.061669E+04,
    4.725957E+04, 5.462826E+04, 6.281747E+04, 7.193003E+04, 8.207734E+04, 9.337996E+04,
    1.059684E+05, 1.199837E+05, 1.355789E+05, 1.529192E+05, 1.721835E+05, 1.935653E+05,
    2.172735E+05, 2.435340E+05, 2.725902E+05, 3.047047E+05, 3.401604E+05, 3.792616E+05,
    4.223355E+05, 4.697336E+05, 5.218332E+05, 5.790385E+05, 6.417827E+05, 7.105294E+05,
    7.857744E+05, 8.680471E+05, 9.579127E+05, 1.055974E+06, 1.162873E+06, 1.279295E+06,
    1.405966E+06, 1.543660E+06, 1.693200E+06, 1.855457E+06, 2.031357E+06, 2.221881E+06,
    2.428067E+06, 2.651016E+06, 2.891891E+06, 3.151923E+06, 3.432411E+06, 3.734728E+06,
    4.060320E+06, 4.410715E+06, 4.787525E+06, 5.192441E+06, 5.627252E+06, 6.093834E+06,
    6.594162E+06, 7.130313E+06, 7.704467E+06, 8.318917E+06, 8.976065E+06, 9.678431E+06,
    1.042866E+07, 1.122953E+07, 1.208393E+07, 1.299491E+07, 1.396564E+07, 1.499946E+07,
    1.609985E+07, 1.727043E+07, 1.851502E+07, 1.983759E+07, 2.124227E+07, 2.273339E+07,
    2.431547E+07, 2.599322E+07, 2.777152E+07, 2.965551E+07, 3.165049E+07, 3.376202E+07,
    3.599588E+07, 3.835806E+07, 4.085482E+07, 4.349265E+07, 4.627831E+07, 4.921882E+07,
    5.232149E+07, 5.559387E+07, 5.904385E+07, 6.267958E+07, 6.650956E+07, 7.054254E+07,
    7.478767E+07, 7.925441E+07, 8.395253E+07, 8.889220E+07, 9.408394E+07, 9.953866E+07,
    1.052676E+08, 1.112825E+08, 1.175954E+08, 1.242189E+08, 1.311658E+08, 1.384496E+08,
    1.460840E+08, 1.540834E+08, 1.624626E+08, 1.712367E+08, 1.804217E+08, 1.900338E+08,
    2.000898E+08, 2.106070E+08, 2.216035E+08, 2.330976E+08, 2.451085E+08, 2.576558E+08,
    2.707598E+08, 2.844415E+08, 2.987224E+08, 3.136247E+08, 3.291713E+08, 3.453860E+08,
    3.622928E+08, 3.799169E+08, 3.982840E+08, 4.174208E+08, 4.373543E+08, 4.581129E+08,
    4.797253E+08, 5.022216E+08, 5.256319E+08, 5.499880E+08, 5.753222E+08, 6.016678E+08,
    6.290589E+08, 6.575307E+08, 6.871194E+08, 7.178619E+08, 7.497965E+08, 7.829623E+08,
    8.173995E+08, 8.531493E+08, 8.902543E+08, 9.287578E+08, 9.687047E+08, 1.010141E+09,
    1.053113E+09, 1.097669E+09, 1.143859E+09, 1.191734E+09, 1.241345E+09, 1.292747E+09,
    1.345992E+09, 1.401139E+09, 1.458245E+09, 1.517367E+09, 1.578567E+09, 1.641907E+09,
    1.707449E+09, 1.775259E+09, 1.845404E+09, 1.917950E+09, 1.992969E+09, 2.070532E+09,
    2.150711E+09, 2.233580E+09, 2.319218E+09, 2.407703E+09, 2.499113E+09, 2.593533E+09,
    2.691044E+09, 2.791734E+09,
])

#  ---------------------- M = 29, I = 2 ---------------------------
M = 29
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.600659E+01, 2.121560E+03, 5.988960E+03, 1.099824E+04, 1.693381E+04, 2.367770E+04,
    3.117187E+04, 3.941173E+04, 4.844066E+04, 5.834143E+04, 6.922734E+04, 8.123543E+04,
    9.452204E+04, 1.092604E+05, 1.256400E+05, 1.438665E+05, 1.641626E+05, 1.867695E+05,
    2.119482E+05, 2.399810E+05, 2.711735E+05, 3.058566E+05, 3.443879E+05, 3.871544E+05,
    4.345740E+05, 4.870983E+05, 5.452144E+05, 6.094476E+05, 6.803632E+05, 7.585704E+05,
    8.447233E+05, 9.395250E+05, 1.043730E+06, 1.158147E+06, 1.283642E+06, 1.421143E+06,
    1.571641E+06, 1.736195E+06, 1.915934E+06, 2.112067E+06, 2.325876E+06, 2.558730E+06,
    2.812083E+06, 3.087485E+06, 3.386577E+06, 3.711105E+06, 4.062920E+06, 4.443983E+06,
    4.856372E+06, 5.302289E+06, 5.784058E+06, 6.304143E+06, 6.865140E+06, 7.469795E+06,
    8.121003E+06, 8.821819E+06, 9.575464E+06, 1.038532E+07, 1.125497E+07, 1.218817E+07,
    1.318886E+07, 1.426120E+07, 1.540954E+07, 1.663848E+07, 1.795281E+07, 1.935759E+07,
    2.085809E+07, 2.245987E+07, 2.416872E+07, 2.599072E+07, 2.793224E+07, 2.999994E+07,
    3.220077E+07, 3.454200E+07, 3.703124E+07, 3.967644E+07, 4.248586E+07, 4.546818E+07,
    4.863242E+07, 5.198798E+07, 5.554466E+07, 5.931272E+07, 6.330278E+07, 6.752592E+07,
    7.199374E+07, 7.671818E+07, 8.171180E+07, 8.698757E+07, 9.255900E+07, 9.844013E+07,
    1.046456E+08, 1.111905E+08, 1.180905E+08, 1.253621E+08, 1.330222E+08, 1.410883E+08,
    1.495787E+08, 1.585123E+08, 1.679087E+08, 1.777882E+08, 1.881719E+08, 1.990815E+08,
    2.105395E+08, 2.225695E+08, 2.351955E+08, 2.484426E+08, 2.623366E+08, 2.769043E+08,
    2.921733E+08, 3.081724E+08, 3.249309E+08, 3.424795E+08, 3.608497E+08, 3.800740E+08,
    4.001862E+08, 4.212210E+08, 4.432141E+08, 4.662025E+08, 4.902245E+08, 5.153195E+08,
    5.415277E+08, 5.688914E+08, 5.974535E+08, 6.272584E+08, 6.583520E+08, 6.907814E+08,
    7.245953E+08, 7.598439E+08, 7.965784E+08, 8.348521E+08, 8.747197E+08, 9.162371E+08,
    9.594623E+08, 1.004455E+09, 1.051276E+09, 1.099989E+09, 1.150657E+09, 1.203349E+09,
    1.258132E+09, 1.315075E+09, 1.374253E+09, 1.435738E+09, 1.499608E+09, 1.565940E+09,
    1.634815E+09, 1.706315E+09, 1.780525E+09, 1.857533E+09, 1.937427E+09, 2.020299E+09,
    2.106243E+09, 2.195356E+09, 2.287737E+09, 2.383486E+09, 2.482710E+09, 2.585513E+09,
    2.692005E+09, 2.802299E+09, 2.916510E+09, 3.034755E+09, 3.157156E+09, 3.283835E+09,
    3.414921E+09, 3.550541E+09, 3.690830E+09, 3.835924E+09, 3.985962E+09, 4.141087E+09,
    4.301445E+09, 4.467185E+09, 4.638461E+09, 4.815431E+09, 4.998252E+09, 5.187090E+09,
    5.382113E+09, 5.583493E+09,
])

#  ---------------------- M = 30, I = 1 ---------------------------
M = 30
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.021972E+02, 8.926883E+03, 2.522955E+04, 4.637336E+04, 7.178418E+04, 1.020099E+05,
    1.386432E+05, 1.842557E+05, 2.424692E+05, 3.181910E+05, 4.180157E+05, 5.508060E+05,
    7.284907E+05, 9.671311E+05, 1.288340E+06, 1.721151E+06, 2.304480E+06, 3.090346E+06,
    4.148075E+06, 5.569754E+06, 7.477295E+06, 1.003153E+07, 1.344388E+07, 1.799125E+07,
    2.403505E+07, 3.204519E+07, 4.263047E+07, 5.657681E+07, 7.489511E+07, 9.888099E+07,
    1.301893E+08, 1.709260E+08, 2.237627E+08, 2.920765E+08, 3.801222E+08, 4.932437E+08,
    6.381305E+08, 8.231307E+08, 1.058630E+09, 1.357512E+09, 1.735707E+09, 2.212863E+09,
    2.813136E+09, 3.566151E+09, 4.508136E+09, 5.683276E+09, 7.145326E+09, 8.959512E+09,
    1.120479E+10, 1.397648E+10, 1.738943E+10, 2.158165E+10, 2.671863E+10, 3.299836E+10,
    4.065719E+10, 4.997670E+10, 6.129157E+10, 7.499890E+10, 9.156880E+10, 1.115568E+11,
    1.356180E+11, 1.645237E+11, 1.991801E+11, 2.406500E+11, 2.901778E+11, 3.492179E+11,
    4.194675E+11, 5.029034E+11, 6.018245E+11, 7.189005E+11, 8.572261E+11, 1.020383E+12,
    1.212513E+12, 1.438393E+12, 1.703531E+12, 2.014262E+12, 2.377871E+12, 2.802713E+12,
    3.298368E+12, 3.875798E+12, 4.547536E+12, 5.327887E+12, 6.233164E+12, 7.281945E+12,
    8.495361E+12, 9.897418E+12, 1.151536E+13, 1.338007E+13, 1.552651E+13, 1.799421E+13,
    2.082787E+13, 2.407789E+13, 2.780112E+13, 3.206156E+13, 3.693125E+13, 4.249113E+13,
    4.883210E+13, 5.605613E+13, 6.427748E+13, 7.362411E+13, 8.423914E+13, 9.628255E+13,
    1.099330E+14, 1.253896E+14, 1.428748E+14, 1.626357E+14, 1.849479E+14, 2.101173E+14,
    2.384842E+14, 2.704260E+14, 3.063617E+14, 3.467555E+14, 3.921214E+14, 4.430285E+14,
    5.001060E+14, 5.640492E+14, 6.356260E+14, 7.156838E+14, 8.051568E+14, 9.050747E+14,
    1.016571E+15, 1.140894E+15, 1.279415E+15, 1.433643E+15, 1.605233E+15, 1.796004E+15,
    2.007949E+15, 2.243255E+15, 2.504315E+15, 2.793752E+15, 3.114435E+15, 3.469502E+15,
    3.862381E+15, 4.296819E+15, 4.776905E+15, 5.307101E+15, 5.892273E+15, 6.537723E+15,
    7.249229E+15, 8.033080E+15, 8.896120E+15, 9.845792E+15, 1.089019E+16, 1.203810E+16,
    1.329907E+16, 1.468347E+16, 1.620253E+16, 1.786845E+16, 1.969444E+16, 2.169481E+16,
    2.388505E+16, 2.628195E+16, 2.890364E+16, 3.176975E+16, 3.490150E+16, 3.832178E+16,
    4.205536E+16, 4.612893E+16, 5.057131E+16, 5.541360E+16, 6.068930E+16, 6.643451E+16,
    7.268815E+16, 7.949209E+16, 8.689142E+16, 9.493463E+16, 1.036739E+17, 1.131653E+17,
    1.234691E+17, 1.346500E+17, 1.467777E+17, 1.599265E+17, 1.741768E+17, 1.896144E+17,
    2.063313E+17, 2.244263E+17,
])

#  ---------------------- M = 31, I = 1 ---------------------------
M = 31
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[6]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.000000E+00, 9.577640E+00, 2.619804E+01, 4.723879E+01, 7.205446E+01, 1.001556E+02,
    1.312023E+02, 1.649444E+02, 2.011901E+02, 2.397898E+02, 2.806276E+02, 3.236172E+02,
    3.686976E+02, 4.158306E+02, 4.649971E+02, 5.161940E+02, 5.694318E+02, 6.247317E+02,
    6.821239E+02, 7.416459E+02, 8.033414E+02, 8.672592E+02, 9.334532E+02, 1.001981E+03,
    1.072904E+03, 1.146289E+03, 1.222203E+03, 1.300719E+03, 1.381913E+03, 1.465863E+03,
    1.552650E+03, 1.642361E+03, 1.735081E+03, 1.830901E+03, 1.929915E+03, 2.032218E+03,
    2.137907E+03, 2.247085E+03, 2.359854E+03, 2.476320E+03, 2.596589E+03, 2.720774E+03,
    2.848985E+03, 2.981338E+03, 3.117948E+03, 3.258936E+03, 3.404422E+03, 3.554529E+03,
    3.709382E+03, 3.869109E+03, 4.033838E+03, 4.203702E+03, 4.378833E+03, 4.559368E+03,
    4.745444E+03, 4.937200E+03, 5.134779E+03, 5.338325E+03, 5.547982E+03, 5.763900E+03,
    5.986229E+03, 6.215121E+03, 6.450730E+03, 6.693213E+03, 6.942728E+03, 7.199438E+03,
    7.463503E+03, 7.735091E+03, 8.014367E+03, 8.301502E+03, 8.596667E+03, 8.900036E+03,
    9.211786E+03, 9.532093E+03, 9.861140E+03, 1.019911E+04, 1.054618E+04, 1.090255E+04,
    1.126840E+04, 1.164392E+04, 1.202931E+04, 1.242476E+04, 1.283047E+04, 1.324664E+04,
    1.367348E+04, 1.411118E+04, 1.455995E+04, 1.502000E+04, 1.549154E+04, 1.597478E+04,
    1.646995E+04, 1.697724E+04, 1.749688E+04, 1.802909E+04, 1.857410E+04, 1.913212E+04,
    1.970338E+04, 2.028811E+04, 2.088653E+04, 2.149888E+04, 2.212539E+04, 2.276628E+04,
    2.342180E+04, 2.409219E+04, 2.477767E+04, 2.547848E+04, 2.619488E+04, 2.692709E+04,
    2.767536E+04, 2.843993E+04, 2.922104E+04, 3.001895E+04, 3.083389E+04, 3.166610E+04,
    3.251585E+04, 3.338337E+04, 3.426891E+04, 3.517272E+04, 3.609505E+04, 3.703614E+04,
    3.799625E+04, 3.897562E+04, 3.997449E+04, 4.099313E+04, 4.203177E+04, 4.309066E+04,
    4.417005E+04, 4.527018E+04, 4.639131E+04, 4.753367E+04, 4.869751E+04, 4.988308E+04,
    5.109061E+04, 5.232035E+04, 5.357253E+04, 5.484740E+04, 5.614520E+04, 5.746615E+04,
    5.881050E+04, 6.017847E+04, 6.157031E+04, 6.298623E+04, 6.442647E+04, 6.589125E+04,
    6.738080E+04, 6.889533E+04, 7.043508E+04, 7.200025E+04, 7.359107E+04, 7.520774E+04,
    7.685048E+04, 7.851950E+04, 8.021500E+04, 8.193718E+04, 8.368625E+04, 8.546241E+04,
    8.726584E+04, 8.909676E+04, 9.095533E+04, 9.284176E+04, 9.475623E+04, 9.669891E+04,
    9.866999E+04, 1.006696E+05, 1.026980E+05, 1.047553E+05, 1.068417E+05, 1.089574E+05,
    1.111024E+05, 1.132770E+05, 1.154813E+05, 1.177155E+05, 1.199797E+05, 1.222740E+05,
    1.245986E+05, 1.269537E+05, 1.293393E+05, 1.317557E+05, 1.342028E+05, 1.366809E+05,
    1.391901E+05, 1.417304E+05, 1.443021E+05, 1.469051E+05, 1.495397E+05, 1.522058E+05,
    1.549037E+05, 1.576334E+05, 1.603950E+05, 1.631886E+05, 1.660143E+05, 1.688721E+05,
    1.717622E+05, 1.746846E+05, 1.776393E+05, 1.806266E+05, 1.836463E+05, 1.866986E+05,
    1.897836E+05, 1.929013E+05, 1.960517E+05,
])

#  ---------------------- M = 31, I = 2 ---------------------------
M = 31
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.000000E+00, 9.858680E+00, 2.626126E+01, 4.730993E+01, 7.213596E+01, 1.002319E+02,
    1.312533E+02, 1.649477E+02, 2.011172E+02, 2.396146E+02, 2.803171E+02, 3.231407E+02,
    3.680172E+02, 4.149141E+02, 4.638055E+02, 5.146919E+02, 5.675764E+02, 6.224870E+02,
    6.794483E+02, 7.384938E+02, 7.996744E+02, 8.630347E+02, 9.286157E+02, 9.964913E+02,
    1.066714E+03, 1.139338E+03, 1.214443E+03, 1.292091E+03, 1.372359E+03, 1.455313E+03,
    1.541049E+03, 1.629623E+03, 1.721154E+03, 1.815684E+03, 1.913338E+03, 2.014177E+03,
    2.118315E+03, 2.225852E+03, 2.336870E+03, 2.451471E+03, 2.569761E+03, 2.691824E+03,
    2.817792E+03, 2.947774E+03, 3.081835E+03, 3.220139E+03, 3.362775E+03, 3.509861E+03,
    3.661488E+03, 3.817802E+03, 3.978929E+03, 4.144958E+03, 4.316016E+03, 4.492262E+03,
    4.673789E+03, 4.860762E+03, 5.053242E+03, 5.251395E+03, 5.455392E+03, 5.665333E+03,
    5.881313E+03, 6.103551E+03, 6.332104E+03, 6.567196E+03, 6.808930E+03, 7.057402E+03,
    7.312850E+03, 7.575326E+03, 7.845073E+03, 8.122195E+03, 8.406841E+03, 8.699164E+03,
    8.999319E+03, 9.307513E+03, 9.623851E+03, 9.948547E+03, 1.028170E+04, 1.062348E+04,
    1.097410E+04, 1.133374E+04, 1.170255E+04, 1.208063E+04, 1.246830E+04, 1.286558E+04,
    1.327280E+04, 1.369003E+04, 1.411755E+04, 1.455545E+04, 1.500391E+04, 1.546326E+04,
    1.593352E+04, 1.641497E+04, 1.690779E+04, 1.741224E+04, 1.792835E+04, 1.845655E+04,
    1.899688E+04, 1.954952E+04, 2.011484E+04, 2.069287E+04, 2.128398E+04, 2.188828E+04,
    2.250597E+04, 2.313735E+04, 2.378263E+04, 2.444192E+04, 2.511562E+04, 2.580384E+04,
    2.650680E+04, 2.722481E+04, 2.795798E+04, 2.870665E+04, 2.947101E+04, 3.025131E+04,
    3.104787E+04, 3.186080E+04, 3.269035E+04, 3.353694E+04, 3.440060E+04, 3.528166E+04,
    3.618048E+04, 3.709719E+04, 3.803214E+04, 3.898543E+04, 3.995757E+04, 4.094855E+04,
    4.195886E+04, 4.298876E+04, 4.403836E+04, 4.510792E+04, 4.619795E+04, 4.730858E+04,
    4.844006E+04, 4.959280E+04, 5.076704E+04, 5.196291E+04, 5.318083E+04, 5.442120E+04,
    5.568414E+04, 5.697007E+04, 5.827913E+04, 5.961172E+04, 6.096828E+04, 6.234896E+04,
    6.375399E+04, 6.518386E+04, 6.663882E+04, 6.811918E+04, 6.962521E+04, 7.115721E+04,
    7.271564E+04, 7.430080E+04, 7.591297E+04, 7.755248E+04, 7.921976E+04, 8.091481E+04,
    8.263842E+04, 8.439059E+04, 8.617196E+04, 8.798264E+04, 8.982298E+04, 9.169347E+04,
    9.359443E+04, 9.552618E+04, 9.748903E+04, 9.948332E+04, 1.015096E+05, 1.035681E+05,
    1.056590E+05, 1.077831E+05, 1.099405E+05, 1.121316E+05, 1.143570E+05, 1.166166E+05,
    1.189111E+05, 1.212410E+05,
])

#  ---------------------- M = 31, I = 3 ---------------------------
M = 31
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.000000E+00, 3.938981E+01, 1.049203E+02, 1.890120E+02, 2.881945E+02, 4.004404E+02,
    5.243733E+02, 6.589857E+02, 8.034856E+02, 9.572857E+02, 1.119896E+03, 1.290979E+03,
    1.470264E+03, 1.657621E+03, 1.852946E+03, 2.056241E+03, 2.267518E+03, 2.486890E+03,
    2.714455E+03, 2.950346E+03, 3.194767E+03, 3.447896E+03, 3.709897E+03, 3.981065E+03,
    4.261609E+03, 4.551747E+03, 4.851797E+03, 5.162008E+03, 5.482685E+03, 5.814092E+03,
    6.156613E+03, 6.510473E+03, 6.876144E+03, 7.253798E+03, 7.643934E+03, 8.046793E+03,
    8.462831E+03, 8.892449E+03, 9.335972E+03, 9.793812E+03, 1.026639E+04, 1.075404E+04,
    1.125729E+04, 1.177658E+04, 1.231216E+04, 1.286470E+04, 1.343454E+04, 1.402216E+04,
    1.462791E+04, 1.525240E+04, 1.589611E+04, 1.655942E+04, 1.724280E+04, 1.794692E+04,
    1.867213E+04, 1.941910E+04, 2.018807E+04, 2.097971E+04, 2.179469E+04, 2.263342E+04,
    2.349628E+04, 2.438413E+04, 2.529722E+04, 2.623643E+04, 2.720217E+04, 2.819483E+04,
    2.921536E+04, 3.026397E+04, 3.134163E+04, 3.244875E+04, 3.358594E+04, 3.475379E+04,
    3.595293E+04, 3.718418E+04, 3.844797E+04, 3.974515E+04, 4.107614E+04, 4.244157E+04,
    4.384233E+04, 4.527909E+04, 4.675250E+04, 4.826299E+04, 4.981175E+04, 5.139893E+04,
    5.302576E+04, 5.469267E+04, 5.640061E+04, 5.815005E+04, 5.994168E+04, 6.177680E+04,
    6.365556E+04, 6.557899E+04, 6.754782E+04, 6.956311E+04, 7.162501E+04, 7.373522E+04,
    7.589386E+04, 7.810171E+04, 8.036020E+04, 8.266947E+04, 8.503099E+04, 8.744521E+04,
    8.991295E+04, 9.243536E+04, 9.501329E+04, 9.764719E+04, 1.003387E+05, 1.030882E+05,
    1.058965E+05, 1.087650E+05, 1.116941E+05, 1.146851E+05, 1.177388E+05, 1.208561E+05,
    1.240384E+05, 1.272861E+05, 1.306002E+05, 1.339824E+05, 1.374328E+05, 1.409527E+05,
    1.445436E+05, 1.482059E+05, 1.519411E+05, 1.557495E+05, 1.596333E+05, 1.635923E+05,
    1.676286E+05, 1.717431E+05, 1.759363E+05, 1.802093E+05, 1.845641E+05, 1.890011E+05,
    1.935214E+05, 1.981267E+05, 2.028178E+05, 2.075955E+05, 2.124611E+05, 2.174165E+05,
    2.224620E+05, 2.275994E+05, 2.328292E+05, 2.381530E+05, 2.435725E+05, 2.490884E+05,
    2.547017E+05, 2.604140E+05, 2.662267E+05, 2.721408E+05, 2.781576E+05, 2.842780E+05,
    2.905041E+05, 2.968369E+05, 3.032776E+05, 3.098275E+05, 3.164884E+05, 3.232603E+05,
    3.301462E+05, 3.371462E+05, 3.442629E+05, 3.514967E+05, 3.588490E+05, 3.663217E+05,
    3.739162E+05, 3.816337E+05, 3.894754E+05, 3.974428E+05, 4.055377E+05, 4.137616E+05,
    4.221151E+05, 4.306009E+05, 4.392197E+05, 4.479736E+05, 4.568640E+05, 4.658915E+05,
    4.750581E+05, 4.843661E+05,
])

#  ---------------------- M = 32, I = 1 ---------------------------
M = 32
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    8.098530E+00, 6.168274E+02, 1.738066E+03, 3.189875E+03, 4.909798E+03, 6.862997E+03,
    9.030079E+03, 1.140495E+04, 1.399259E+04, 1.680815E+04, 1.987372E+04, 2.321812E+04,
    2.687559E+04, 3.088517E+04, 3.529073E+04, 4.014075E+04, 4.548994E+04, 5.139632E+04,
    5.792593E+04, 6.514813E+04, 7.314122E+04, 8.198985E+04, 9.178577E+04, 1.026295E+05,
    1.146311E+05, 1.279089E+05, 1.425943E+05, 1.588277E+05, 1.767626E+05, 1.965672E+05,
    2.184237E+05, 2.425277E+05, 2.690945E+05, 2.983565E+05, 3.305651E+05, 3.659933E+05,
    4.049357E+05, 4.477128E+05, 4.946682E+05, 5.461776E+05, 6.026400E+05, 6.644928E+05,
    7.322038E+05, 8.062787E+05, 8.872603E+05, 9.757342E+05, 1.072333E+06, 1.177728E+06,
    1.292652E+06, 1.417886E+06, 1.554264E+06, 1.702691E+06, 1.864125E+06, 2.039604E+06,
    2.230231E+06, 2.437189E+06, 2.661747E+06, 2.905255E+06, 3.169158E+06, 3.455004E+06,
    3.764439E+06, 4.099224E+06, 4.461229E+06, 4.852460E+06, 5.275039E+06, 5.731238E+06,
    6.223470E+06, 6.754297E+06, 7.326455E+06, 7.942838E+06, 8.606536E+06, 9.320818E+06,
    1.008916E+07, 1.091524E+07, 1.180298E+07, 1.275651E+07, 1.378023E+07, 1.487879E+07,
    1.605711E+07, 1.732042E+07, 1.867421E+07, 2.012433E+07, 2.167694E+07, 2.333855E+07,
    2.511606E+07, 2.701671E+07, 2.904820E+07, 3.121862E+07, 3.353650E+07, 3.601086E+07,
    3.865118E+07, 4.146747E+07, 4.447026E+07, 4.767066E+07, 5.108035E+07, 5.471163E+07,
    5.857740E+07, 6.269129E+07, 6.706761E+07, 7.172137E+07, 7.666838E+07, 8.192520E+07,
    8.750927E+07, 9.343889E+07, 9.973322E+07, 1.064124E+08, 1.134976E+08, 1.210109E+08,
    1.289756E+08, 1.374159E+08, 1.463573E+08, 1.558267E+08, 1.658518E+08, 1.764620E+08,
    1.876879E+08, 1.995616E+08, 2.121165E+08, 2.253878E+08, 2.394120E+08, 2.542275E+08,
    2.698744E+08, 2.863944E+08, 3.038314E+08, 3.222307E+08, 3.416402E+08, 3.621095E+08,
    3.836906E+08, 4.064374E+08, 4.304066E+08, 4.556569E+08, 4.822498E+08, 5.102492E+08,
    5.397218E+08, 5.707372E+08, 6.033677E+08, 6.376886E+08, 6.737783E+08, 7.117187E+08,
    7.515949E+08, 7.934953E+08, 8.375118E+08, 8.837404E+08, 9.322806E+08, 9.832362E+08,
    1.036715E+09, 1.092828E+09, 1.151692E+09, 1.213428E+09, 1.278162E+09, 1.346024E+09,
    1.417150E+09, 1.491679E+09, 1.569758E+09, 1.651538E+09, 1.737177E+09, 1.826838E+09,
    1.920689E+09, 2.018907E+09, 2.121673E+09, 2.229176E+09, 2.341612E+09, 2.459182E+09,
    2.582099E+09, 2.710579E+09, 2.844846E+09, 2.985137E+09, 3.131691E+09, 3.284760E+09,
    3.444603E+09, 3.611489E+09, 3.785695E+09, 3.967509E+09, 4.157229E+09, 4.355164E+09,
    4.561629E+09, 4.776956E+09,
])

#  ---------------------- M = 33, I = 1 ---------------------------
M = 33
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.594200E-01, 7.688782E+01, 2.147062E+02, 3.927665E+02, 6.034216E+02, 8.422352E+02,
    1.106209E+03, 1.393154E+03, 1.701421E+03, 2.029715E+03, 2.377155E+03, 2.743100E+03,
    3.127307E+03, 3.529659E+03, 3.950367E+03, 4.389807E+03, 4.848487E+03, 5.327087E+03,
    5.826359E+03, 6.347192E+03, 6.890410E+03, 7.456986E+03, 8.047900E+03, 8.664120E+03,
    9.306654E+03, 9.976636E+03, 1.067496E+04, 1.140281E+04, 1.216121E+04, 1.295117E+04,
    1.377383E+04, 1.463032E+04, 1.552167E+04, 1.644903E+04, 1.741356E+04, 1.841643E+04,
    1.945864E+04, 2.054151E+04, 2.166621E+04, 2.283391E+04, 2.404598E+04, 2.530358E+04,
    2.660771E+04, 2.796015E+04, 2.936189E+04, 3.081411E+04, 3.231868E+04, 3.387635E+04,
    3.548880E+04, 3.715773E+04, 3.888414E+04, 4.066950E+04, 4.251530E+04, 4.442335E+04,
    4.639489E+04, 4.843149E+04, 5.053503E+04, 5.270646E+04, 5.494802E+04, 5.726067E+04,
    5.964671E+04, 6.210782E+04, 6.464494E+04, 6.726087E+04, 6.995658E+04, 7.273377E+04,
    7.559498E+04, 7.854117E+04, 8.157495E+04, 8.469771E+04, 8.791125E+04, 9.121830E+04,
    9.461985E+04, 9.811822E+04, 1.017153E+05, 1.054135E+05, 1.092148E+05, 1.131206E+05,
    1.171339E+05, 1.212563E+05, 1.254897E+05, 1.298368E+05, 1.342996E+05, 1.388801E+05,
    1.435812E+05, 1.484048E+05, 1.533526E+05, 1.584280E+05, 1.636325E+05, 1.689691E+05,
    1.744392E+05, 1.800464E+05, 1.857931E+05, 1.916809E+05, 1.977128E+05, 2.038912E+05,
    2.102190E+05, 2.166981E+05, 2.233322E+05, 2.301232E+05, 2.370735E+05, 2.441870E+05,
    2.514648E+05, 2.589109E+05, 2.665280E+05, 2.743178E+05, 2.822846E+05, 2.904301E+05,
    2.987578E+05, 3.072706E+05, 3.159718E+05, 3.248634E+05, 3.339491E+05, 3.432315E+05,
    3.527136E+05, 3.623999E+05, 3.722915E+05, 3.823931E+05, 3.927067E+05, 4.032371E+05,
    4.139854E+05, 4.249565E+05, 4.361534E+05, 4.475791E+05, 4.592368E+05, 4.711305E+05,
    4.832632E+05, 4.956373E+05, 5.082588E+05, 5.211290E+05, 5.342531E+05, 5.476335E+05,
    5.612732E+05, 5.751779E+05, 5.893485E+05, 6.037919E+05, 6.185091E+05, 6.335047E+05,
    6.487834E+05, 6.643485E+05, 6.802023E+05, 6.963508E+05, 7.127974E+05, 7.295461E+05,
    7.465986E+05, 7.639627E+05, 7.816395E+05, 7.996338E+05, 8.179508E+05, 8.365924E+05,
    8.555657E+05, 8.748729E+05, 8.945175E+05, 9.145066E+05, 9.348407E+05, 9.555269E+05,
    9.765689E+05, 9.979710E+05, 1.019738E+06, 1.041873E+06, 1.064382E+06, 1.087269E+06,
    1.110538E+06, 1.134193E+06, 1.158240E+06, 1.182684E+06, 1.207529E+06, 1.232780E+06,
    1.258440E+06, 1.284515E+06, 1.311010E+06, 1.337930E+06, 1.365279E+06, 1.393063E+06,
    1.421286E+06, 1.449955E+06,
])

#  ---------------------- M = 35, I = 1 ---------------------------
M = 35
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.408818E+02, 2.081987E+04, 5.959366E+04, 1.144363E+05, 1.889620E+05, 2.880202E+05,
    4.176320E+05, 5.853070E+05, 8.004051E+05, 1.074538E+06, 1.422010E+06, 1.860338E+06,
    2.410828E+06, 3.099228E+06, 3.956556E+06, 5.019996E+06, 6.333930E+06, 7.951208E+06,
    9.934433E+06, 1.235765E+07, 1.530808E+07, 1.888813E+07, 2.321778E+07, 2.843710E+07,
    3.470920E+07, 4.222351E+07, 5.119940E+07, 6.189045E+07, 7.458876E+07, 8.963031E+07,
    1.074004E+08, 1.283401E+08, 1.529530E+08, 1.818133E+08, 2.155737E+08, 2.549750E+08,
    3.008566E+08, 3.541675E+08, 4.159785E+08, 4.874961E+08, 5.700773E+08, 6.652451E+08,
    7.747069E+08, 9.003730E+08, 1.044378E+09, 1.209104E+09, 1.397202E+09, 1.611625E+09,
    1.855648E+09, 2.132908E+09, 2.447432E+09, 2.803673E+09, 3.206556E+09, 3.661513E+09,
    4.174529E+09, 4.752195E+09, 5.401760E+09, 6.131182E+09, 6.949194E+09, 7.865367E+09,
    8.890177E+09, 1.003509E+10, 1.131261E+10, 1.273641E+10, 1.432140E+10, 1.608379E+10,
    1.804126E+10, 2.021301E+10, 2.261991E+10, 2.528461E+10, 2.823167E+10, 3.148772E+10,
    3.508157E+10, 3.904440E+10, 4.340993E+10, 4.821455E+10, 5.349755E+10, 5.930134E+10,
    6.567158E+10, 7.265748E+10, 8.031197E+10, 8.869200E+10, 9.785877E+10, 1.078780E+11,
    1.188203E+11, 1.307613E+11, 1.437821E+11, 1.579696E+11, 1.734170E+11, 1.902238E+11,
    2.084964E+11, 2.283487E+11, 2.499022E+11, 2.732868E+11, 2.986409E+11, 3.261123E+11,
    3.558583E+11, 3.880465E+11, 4.228556E+11, 4.604755E+11, 5.011085E+11, 5.449694E+11,
    5.922865E+11, 6.433023E+11, 6.982744E+11, 7.574760E+11, 8.211970E+11, 8.897447E+11,
    9.634444E+11, 1.042642E+12, 1.127701E+12, 1.219010E+12, 1.316977E+12, 1.422034E+12,
    1.534639E+12, 1.655275E+12, 1.784452E+12, 1.922709E+12, 2.070615E+12, 2.228770E+12,
    2.397806E+12, 2.578390E+12, 2.771227E+12, 2.977056E+12, 3.196658E+12, 3.430854E+12,
    3.680508E+12, 3.946531E+12, 4.229878E+12, 4.531554E+12, 4.852619E+12, 5.194181E+12,
    5.557407E+12, 5.943522E+12, 6.353813E+12, 6.789628E+12, 7.252383E+12, 7.743565E+12,
    8.264728E+12, 8.817504E+12, 9.403604E+12, 1.002482E+13, 1.068303E+13, 1.138019E+13,
    1.211836E+13, 1.289970E+13, 1.372645E+13, 1.460097E+13, 1.552571E+13, 1.650326E+13,
    1.753631E+13, 1.862765E+13, 1.978023E+13, 2.099711E+13, 2.228149E+13, 2.363672E+13,
    2.506627E+13, 2.657379E+13, 2.816307E+13, 2.983806E+13, 3.160289E+13, 3.346185E+13,
    3.541944E+13, 3.748031E+13, 3.964933E+13, 4.193156E+13, 4.433228E+13, 4.685697E+13,
    4.951135E+13, 5.230137E+13, 5.523320E+13, 5.831329E+13, 6.154832E+13, 6.494525E+13,
    6.851131E+13, 7.225402E+13,
])

#  ---------------------- M = 35, I = 2 ---------------------------
M = 35
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.468098E+02, 2.134900E+04, 6.110945E+04, 1.173478E+05, 1.937703E+05, 2.953497E+05,
    4.282605E+05, 6.002031E+05, 8.207761E+05, 1.101887E+06, 1.458203E+06, 1.907688E+06,
    2.472190E+06, 3.178113E+06, 4.057263E+06, 5.147772E+06, 6.495151E+06, 8.153595E+06,
    1.018730E+07, 1.267221E+07, 1.569774E+07, 1.936891E+07, 2.380877E+07, 2.916094E+07,
    3.559270E+07, 4.329828E+07, 5.250266E+07, 6.346585E+07, 7.648739E+07, 9.191183E+07,
    1.101343E+08, 1.316070E+08, 1.568465E+08, 1.864414E+08, 2.210611E+08, 2.614653E+08,
    3.085149E+08, 3.631828E+08, 4.265672E+08, 4.999054E+08, 5.845887E+08, 6.821789E+08,
    7.944272E+08, 9.232921E+08, 1.070963E+09, 1.239882E+09, 1.432769E+09, 1.652649E+09,
    1.902884E+09, 2.187202E+09, 2.509732E+09, 2.875042E+09, 3.288180E+09, 3.754718E+09,
    4.280793E+09, 4.873164E+09, 5.539263E+09, 6.287254E+09, 7.126088E+09, 8.065583E+09,
    9.116481E+09, 1.029053E+10, 1.160058E+10, 1.306063E+10, 1.468596E+10, 1.649322E+10,
    1.850051E+10, 2.072754E+10, 2.319571E+10, 2.592825E+10, 2.895033E+10, 3.228926E+10,
    3.597459E+10, 4.003830E+10, 4.451495E+10, 4.944187E+10, 5.485937E+10, 6.081089E+10,
    6.734329E+10, 7.450702E+10, 8.235636E+10, 9.094971E+10, 1.003498E+11, 1.106241E+11,
    1.218449E+11, 1.340899E+11, 1.474421E+11, 1.619909E+11, 1.778314E+11, 1.950660E+11,
    2.138038E+11, 2.341615E+11, 2.562637E+11, 2.802435E+11, 3.062431E+11, 3.344137E+11,
    3.649168E+11, 3.979245E+11, 4.336196E+11, 4.721973E+11, 5.138646E+11, 5.588420E+11,
    6.073635E+11, 6.596780E+11, 7.160494E+11, 7.767581E+11, 8.421012E+11, 9.123937E+11,
    9.879697E+11, 1.069183E+12, 1.156408E+12, 1.250041E+12, 1.350501E+12, 1.458233E+12,
    1.573704E+12, 1.697411E+12, 1.829877E+12, 1.971654E+12, 2.123324E+12, 2.285505E+12,
    2.458844E+12, 2.644025E+12, 2.841771E+12, 3.052839E+12, 3.278031E+12, 3.518189E+12,
    3.774199E+12, 4.046993E+12, 4.337553E+12, 4.646909E+12, 4.976146E+12, 5.326403E+12,
    5.698874E+12, 6.094819E+12, 6.515555E+12, 6.962464E+12, 7.436999E+12, 7.940684E+12,
    8.475113E+12, 9.041962E+12, 9.642982E+12, 1.028001E+13, 1.095497E+13, 1.166988E+13,
    1.242685E+13, 1.322807E+13, 1.407587E+13, 1.497265E+13, 1.592093E+13, 1.692337E+13,
    1.798271E+13, 1.910183E+13, 2.028375E+13, 2.153161E+13, 2.284869E+13, 2.423842E+13,
    2.570436E+13, 2.725025E+13, 2.887999E+13, 3.059761E+13, 3.240737E+13, 3.431365E+13,
    3.632107E+13, 3.843441E+13, 4.065863E+13, 4.299897E+13, 4.546079E+13, 4.804976E+13,
    5.077171E+13, 5.363275E+13, 5.663921E+13, 5.979770E+13, 6.311508E+13, 6.659849E+13,
    7.025532E+13, 7.409331E+13,
])

#  ---------------------- M = 36, I = 1 ---------------------------
M = 36
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.029540E+00, 2.200969E+01, 4.297675E+01, 6.395552E+01, 8.493845E+01, 1.059240E+02,
    1.269117E+02, 1.479015E+02, 1.688930E+02, 1.898864E+02, 2.108815E+02, 2.318784E+02,
    2.528769E+02, 2.738772E+02, 2.948822E+02, 3.158861E+02, 3.368985E+02, 3.579134E+02,
    3.789385E+02, 3.999708E+02, 4.210197E+02, 4.420868E+02, 4.631741E+02, 4.842933E+02,
    5.054422E+02, 5.266341E+02, 5.478727E+02, 5.691675E+02, 5.905169E+02, 6.119432E+02,
    6.334393E+02, 6.550162E+02, 6.766925E+02, 6.984603E+02, 7.203322E+02, 7.423211E+02,
    7.644255E+02, 7.866515E+02, 8.090126E+02, 8.315068E+02, 8.541484E+02, 8.769350E+02,
    8.998723E+02, 9.229750E+02, 9.462310E+02, 9.696643E+02, 9.932616E+02, 1.017048E+03,
    1.041000E+03, 1.065141E+03, 1.089479E+03, 1.113997E+03, 1.138710E+03, 1.163612E+03,
    1.188731E+03, 1.214038E+03, 1.239547E+03, 1.265277E+03, 1.291206E+03, 1.317352E+03,
    1.343706E+03, 1.370272E+03, 1.397067E+03, 1.424069E+03, 1.451294E+03, 1.478747E+03,
    1.506416E+03, 1.534307E+03, 1.562435E+03, 1.590776E+03, 1.619347E+03, 1.648150E+03,
    1.677191E+03, 1.706455E+03, 1.735962E+03, 1.765682E+03, 1.795650E+03, 1.825853E+03,
    1.856277E+03, 1.886958E+03, 1.917863E+03, 1.949013E+03, 1.980393E+03, 2.012022E+03,
    2.043868E+03, 2.075985E+03, 2.108322E+03, 2.140900E+03, 2.173719E+03, 2.206782E+03,
    2.240090E+03, 2.273646E+03, 2.307451E+03, 2.341488E+03, 2.375759E+03, 2.410284E+03,
    2.445065E+03, 2.480064E+03, 2.515321E+03, 2.550840E+03, 2.586581E+03, 2.622585E+03,
    2.658813E+03, 2.695308E+03, 2.732051E+03, 2.769042E+03, 2.806260E+03, 2.843730E+03,
    2.881473E+03, 2.919448E+03, 2.957655E+03, 2.996140E+03, 3.034859E+03, 3.073813E+03,
    3.113025E+03, 3.152497E+03, 3.192207E+03, 3.232179E+03, 3.272390E+03, 3.312840E+03,
    3.353556E+03, 3.394513E+03, 3.435739E+03, 3.477180E+03, 3.518892E+03, 3.560874E+03,
    3.603075E+03, 3.645548E+03, 3.688268E+03, 3.731208E+03, 3.774424E+03, 3.817915E+03,
    3.861629E+03, 3.905593E+03, 3.949807E+03, 3.994273E+03, 4.039020E+03, 4.083990E+03,
    4.129214E+03, 4.174693E+03, 4.220425E+03, 4.266414E+03, 4.312628E+03, 4.359128E+03,
    4.405856E+03, 4.452842E+03, 4.500085E+03, 4.547587E+03, 4.595349E+03, 4.643340E+03,
    4.691592E+03, 4.740104E+03, 4.788878E+03, 4.837883E+03, 4.887149E+03, 4.936647E+03,
    4.986441E+03, 5.036467E+03, 5.086725E+03, 5.137247E+03, 5.188036E+03, 5.239091E+03,
    5.290379E+03, 5.341935E+03, 5.393723E+03, 5.445780E+03, 5.498071E+03, 5.550631E+03,
    5.603462E+03, 5.656527E+03, 5.709827E+03, 5.763397E+03, 5.817240E+03, 5.871319E+03,
    5.925670E+03, 5.980257E+03,
])

#  ---------------------- M = 37, I = 1 ---------------------------
M = 37
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.785130E+00, 4.726760E+02, 1.331909E+03, 2.444466E+03, 3.762449E+03, 5.258748E+03,
    6.916853E+03, 8.727864E+03, 1.068858E+04, 1.280031E+04, 1.506728E+04, 1.749553E+04,
    2.009268E+04, 2.286748E+04, 2.582902E+04, 2.898719E+04, 3.235219E+04, 3.593458E+04,
    3.974451E+04, 4.379322E+04, 4.809173E+04, 5.265152E+04, 5.748313E+04, 6.259898E+04,
    6.800999E+04, 7.372804E+04, 7.976499E+04, 8.613334E+04, 9.284498E+04, 9.991164E+04,
    1.073465E+05, 1.151622E+05, 1.233719E+05, 1.319883E+05, 1.410250E+05, 1.504953E+05,
    1.604130E+05, 1.707918E+05, 1.816466E+05, 1.929925E+05, 2.048425E+05, 2.172130E+05,
    2.301186E+05, 2.435762E+05, 2.576005E+05, 2.722077E+05, 2.874142E+05, 3.032379E+05,
    3.196943E+05, 3.368020E+05, 3.545764E+05, 3.730385E+05, 3.922061E+05, 4.120955E+05,
    4.327284E+05, 4.541215E+05, 4.762955E+05, 4.992694E+05, 5.230665E+05, 5.477044E+05,
    5.732050E+05, 5.995885E+05, 6.268775E+05, 6.550925E+05, 6.842570E+05, 7.143946E+05,
    7.455270E+05, 7.776759E+05, 8.108659E+05, 8.451225E+05, 8.804679E+05, 9.169282E+05,
    9.545270E+05, 9.932902E+05, 1.033245E+06, 1.074416E+06, 1.116828E+06, 1.160511E+06,
    1.205491E+06, 1.251794E+06, 1.299452E+06, 1.348487E+06, 1.398931E+06, 1.450812E+06,
    1.504163E+06, 1.559011E+06, 1.615386E+06, 1.673318E+06, 1.732838E+06, 1.793976E+06,
    1.856768E+06, 1.921245E+06, 1.987436E+06, 2.055375E+06, 2.125098E+06, 2.196636E+06,
    2.270020E+06, 2.345293E+06, 2.422481E+06, 2.501629E+06, 2.582761E+06, 2.665923E+06,
    2.751144E+06, 2.838463E+06, 2.927919E+06, 3.019552E+06, 3.113392E+06, 3.209490E+06,
    3.307870E+06, 3.408585E+06, 3.511666E+06, 3.617155E+06, 3.725094E+06, 3.835528E+06,
    3.948487E+06, 4.064024E+06, 4.182175E+06, 4.302986E+06, 4.426502E+06, 4.552764E+06,
    4.681813E+06, 4.813699E+06, 4.948460E+06, 5.086155E+06, 5.226813E+06, 5.370497E+06,
    5.517235E+06, 5.667093E+06, 5.820104E+06, 5.976328E+06, 6.135809E+06, 6.298591E+06,
    6.464727E+06, 6.634271E+06, 6.807274E+06, 6.983775E+06, 7.163838E+06, 7.347517E+06,
    7.534852E+06, 7.725898E+06, 7.920720E+06, 8.119366E+06, 8.321879E+06, 8.528331E+06,
    8.738766E+06, 8.953250E+06, 9.171827E+06, 9.394565E+06, 9.621507E+06, 9.852731E+06,
    1.008828E+07, 1.032821E+07, 1.057259E+07, 1.082148E+07, 1.107494E+07, 1.133303E+07,
    1.159581E+07, 1.186333E+07, 1.213567E+07, 1.241289E+07, 1.269505E+07, 1.298221E+07,
    1.327444E+07, 1.357180E+07, 1.387436E+07, 1.418219E+07, 1.449535E+07, 1.481390E+07,
    1.513792E+07, 1.546748E+07, 1.580263E+07, 1.614346E+07, 1.649004E+07, 1.684241E+07,
    1.720068E+07, 1.756490E+07,
])

#  ---------------------- M = 37, I = 2 ---------------------------
M = 37
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.739990E+00, 4.708308E+02, 1.326733E+03, 2.434981E+03, 3.747861E+03, 5.238419E+03,
    6.890124E+03, 8.694231E+03, 1.064761E+04, 1.275163E+04, 1.501028E+04, 1.743002E+04,
    2.001840E+04, 2.278379E+04, 2.573568E+04, 2.888372E+04, 3.223779E+04, 3.580868E+04,
    3.960721E+04, 4.364360E+04, 4.792918E+04, 5.247539E+04, 5.729323E+04, 6.239415E+04,
    6.778948E+04, 7.349165E+04, 7.951198E+04, 8.586227E+04, 9.255496E+04, 9.960247E+04,
    1.070173E+05, 1.148120E+05, 1.229998E+05, 1.315924E+05, 1.406050E+05, 1.500502E+05,
    1.599417E+05, 1.702941E+05, 1.811203E+05, 1.924363E+05, 2.042563E+05, 2.165956E+05,
    2.294686E+05, 2.428909E+05, 2.568798E+05, 2.714501E+05, 2.866181E+05, 3.024019E+05,
    3.188166E+05, 3.358826E+05, 3.536137E+05, 3.720290E+05, 3.911480E+05, 4.109904E+05,
    4.315709E+05, 4.529114E+05, 4.750329E+05, 4.979522E+05, 5.216906E+05, 5.462676E+05,
    5.717072E+05, 5.980275E+05, 6.252486E+05, 6.533981E+05, 6.824923E+05, 7.125570E+05,
    7.436139E+05, 7.756873E+05, 8.087991E+05, 8.429747E+05, 8.782365E+05, 9.146103E+05,
    9.521193E+05, 9.907930E+05, 1.030652E+06, 1.071728E+06, 1.114040E+06, 1.157621E+06,
    1.202496E+06, 1.248693E+06, 1.296235E+06, 1.345157E+06, 1.395485E+06, 1.447246E+06,
    1.500473E+06, 1.555196E+06, 1.611440E+06, 1.669235E+06, 1.728620E+06, 1.789621E+06,
    1.852269E+06, 1.916593E+06, 1.982635E+06, 2.050416E+06, 2.119981E+06, 2.191356E+06,
    2.264576E+06, 2.339672E+06, 2.416687E+06, 2.495652E+06, 2.576605E+06, 2.659573E+06,
    2.744601E+06, 2.831725E+06, 2.920982E+06, 3.012404E+06, 3.106038E+06, 3.201913E+06,
    3.300075E+06, 3.400561E+06, 3.503412E+06, 3.608665E+06, 3.716362E+06, 3.826546E+06,
    3.939250E+06, 4.054530E+06, 4.172418E+06, 4.292959E+06, 4.416198E+06, 4.542175E+06,
    4.670937E+06, 4.802531E+06, 4.936999E+06, 5.074379E+06, 5.214727E+06, 5.358088E+06,
    5.504508E+06, 5.654031E+06, 5.806702E+06, 5.962583E+06, 6.121705E+06, 6.284127E+06,
    6.449902E+06, 6.619068E+06, 6.791685E+06, 6.967806E+06, 7.147471E+06, 7.330735E+06,
    7.517662E+06, 7.708297E+06, 7.902682E+06, 8.100891E+06, 8.302962E+06, 8.508957E+06,
    8.718940E+06, 8.932943E+06, 9.151047E+06, 9.373291E+06, 9.599747E+06, 9.830455E+06,
    1.006549E+07, 1.030491E+07, 1.054875E+07, 1.079710E+07, 1.105000E+07, 1.130753E+07,
    1.156973E+07, 1.183667E+07, 1.210843E+07, 1.238504E+07, 1.266658E+07, 1.295312E+07,
    1.324472E+07, 1.354143E+07, 1.384333E+07, 1.415049E+07, 1.446297E+07, 1.478083E+07,
    1.510415E+07, 1.543299E+07, 1.576743E+07, 1.610752E+07, 1.645334E+07, 1.680496E+07,
    1.716245E+07, 1.752589E+07,
])

#  ---------------------- M = 38, I = 1 ---------------------------
M = 38
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.964040E+00, 1.867813E+02, 5.232315E+02, 9.584283E+02, 1.473741E+03, 2.058415E+03,
    2.705341E+03, 3.409831E+03, 4.169420E+03, 4.983909E+03, 5.855633E+03, 6.789060E+03,
    7.791043E+03, 8.870171E+03, 1.003696E+04, 1.130344E+04, 1.268335E+04, 1.419187E+04,
    1.584565E+04, 1.766292E+04, 1.966394E+04, 2.187063E+04, 2.430711E+04, 2.699984E+04,
    2.997740E+04, 3.327167E+04, 3.691750E+04, 4.095290E+04, 4.541954E+04, 5.036362E+04,
    5.583538E+04, 6.189039E+04, 6.858858E+04, 7.599728E+04, 8.418919E+04, 9.324438E+04,
    1.032502E+05, 1.143029E+05, 1.265073E+05, 1.399784E+05, 1.548424E+05, 1.712363E+05,
    1.893101E+05, 2.092280E+05, 2.311692E+05, 2.553286E+05, 2.819194E+05, 3.111737E+05,
    3.433445E+05, 3.787068E+05, 4.175601E+05, 4.602299E+05, 5.070709E+05, 5.584681E+05,
    6.148394E+05, 6.766383E+05, 7.443566E+05, 8.185285E+05, 8.997333E+05, 9.885977E+05,
    1.085799E+06, 1.192073E+06, 1.308215E+06, 1.435085E+06, 1.573609E+06, 1.724794E+06,
    1.889722E+06, 2.069561E+06, 2.265577E+06, 2.479128E+06, 2.711680E+06, 2.964814E+06,
    3.240234E+06, 3.539768E+06, 3.865391E+06, 4.219220E+06, 4.603536E+06, 5.020787E+06,
    5.473605E+06, 5.964816E+06, 6.497450E+06, 7.074761E+06, 7.700237E+06, 8.377621E+06,
    9.110918E+06, 9.904421E+06, 1.076273E+07, 1.169076E+07, 1.269377E+07, 1.377741E+07,
    1.494767E+07, 1.621101E+07, 1.757429E+07, 1.904485E+07, 2.063052E+07, 2.233968E+07,
    2.418123E+07, 2.616470E+07, 2.830021E+07, 3.059859E+07, 3.307135E+07, 3.573073E+07,
    3.858981E+07, 4.166247E+07, 4.496350E+07, 4.850860E+07, 5.231451E+07, 5.639898E+07,
    6.078088E+07, 6.548028E+07, 7.051845E+07, 7.591801E+07, 8.170292E+07, 8.789865E+07,
    9.453217E+07, 1.016321E+08, 1.092288E+08, 1.173543E+08, 1.260428E+08, 1.353302E+08,
    1.452548E+08, 1.558569E+08, 1.671794E+08, 1.792673E+08, 1.921686E+08, 2.059338E+08,
    2.206164E+08, 2.362728E+08, 2.529626E+08, 2.707490E+08, 2.896984E+08, 3.098811E+08,
    3.313711E+08, 3.542468E+08, 3.785905E+08, 4.044891E+08, 4.320345E+08, 4.613233E+08,
    4.924573E+08, 5.255437E+08, 5.606957E+08, 5.980320E+08, 6.376782E+08, 6.797660E+08,
    7.244340E+08, 7.718283E+08, 8.221023E+08, 8.754174E+08, 9.319435E+08, 9.918586E+08,
    1.055351E+09, 1.122616E+09, 1.193862E+09, 1.269305E+09, 1.349174E+09, 1.433708E+09,
    1.523159E+09, 1.617789E+09, 1.717876E+09, 1.823709E+09, 1.935593E+09, 2.053846E+09,
    2.178804E+09, 2.310816E+09, 2.450248E+09, 2.597487E+09, 2.752934E+09, 2.917011E+09,
    3.090161E+09, 3.272845E+09, 3.465547E+09, 3.668772E+09, 3.883052E+09, 4.108941E+09,
    4.347015E+09, 4.597885E+09,
])

#  ---------------------- M = 38, I = 2 ---------------------------
M = 38
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.204615E+01, 7.642634E+02, 2.141386E+03, 3.922752E+03, 6.032083E+03, 8.425355E+03,
    1.107346E+04, 1.395721E+04, 1.706650E+04, 2.040053E+04, 2.396885E+04, 2.778974E+04,
    3.189127E+04, 3.630859E+04, 4.108475E+04, 4.626896E+04, 5.191754E+04, 5.809252E+04,
    6.486214E+04, 7.230097E+04, 8.049201E+04, 8.952494E+04, 9.949845E+04, 1.105209E+05,
    1.227093E+05, 1.361942E+05, 1.511181E+05, 1.676367E+05, 1.859206E+05, 2.061588E+05,
    2.285571E+05, 2.533428E+05, 2.807614E+05, 3.110884E+05, 3.446214E+05, 3.816882E+05,
    4.226462E+05, 4.678897E+05, 5.178476E+05, 5.729908E+05, 6.338355E+05, 7.009427E+05,
    7.749268E+05, 8.564597E+05, 9.462743E+05, 1.045169E+06, 1.154017E+06, 1.273768E+06,
    1.405457E+06, 1.550210E+06, 1.709254E+06, 1.883920E+06, 2.075661E+06, 2.286052E+06,
    2.516805E+06, 2.769775E+06, 3.046976E+06, 3.350594E+06, 3.683001E+06, 4.046763E+06,
    4.444651E+06, 4.879678E+06, 5.355099E+06, 5.874432E+06, 6.441474E+06, 7.060342E+06,
    7.735464E+06, 8.471627E+06, 9.274007E+06, 1.014816E+07, 1.110010E+07, 1.213630E+07,
    1.326371E+07, 1.448984E+07, 1.582276E+07, 1.727114E+07, 1.884431E+07, 2.055231E+07,
    2.240589E+07, 2.441664E+07, 2.659695E+07, 2.896014E+07, 3.152050E+07, 3.429333E+07,
    3.729505E+07, 4.054321E+07, 4.405664E+07, 4.785548E+07, 5.196127E+07, 5.639707E+07,
    6.118750E+07, 6.635891E+07, 7.193943E+07, 7.795909E+07, 8.444997E+07, 9.144632E+07,
    9.898462E+07, 1.071038E+08, 1.158455E+08, 1.252538E+08, 1.353759E+08, 1.462619E+08,
    1.579654E+08, 1.705432E+08, 1.840558E+08, 1.985675E+08, 2.141468E+08, 2.308664E+08,
    2.488035E+08, 2.680402E+08, 2.886638E+08, 3.107666E+08, 3.344469E+08, 3.598088E+08,
    3.869628E+08, 4.160260E+08, 4.471227E+08, 4.803842E+08, 5.159500E+08, 5.539677E+08,
    5.945935E+08, 6.379928E+08, 6.843406E+08, 7.338221E+08, 7.866329E+08, 8.429801E+08,
    9.030826E+08, 9.671712E+08, 1.035490E+09, 1.108298E+09, 1.185867E+09, 1.268484E+09,
    1.356452E+09, 1.450092E+09, 1.549742E+09, 1.655757E+09, 1.768513E+09, 1.888406E+09,
    2.015851E+09, 2.151289E+09, 2.295182E+09, 2.448017E+09, 2.610306E+09, 2.782591E+09,
    2.965437E+09, 3.159443E+09, 3.365238E+09, 3.583481E+09, 3.814868E+09, 4.060128E+09,
    4.320030E+09, 4.595378E+09, 4.887019E+09, 5.195844E+09, 5.522783E+09, 5.868820E+09,
    6.234981E+09, 6.622346E+09, 7.032047E+09, 7.465271E+09, 7.923262E+09, 8.407328E+09,
    8.918836E+09, 9.459220E+09, 1.002998E+10, 1.063270E+10, 1.126901E+10, 1.194066E+10,
    1.264944E+10, 1.339724E+10, 1.418606E+10, 1.501796E+10, 1.589510E+10, 1.681977E+10,
    1.779432E+10, 1.882124E+10,
])

#  ---------------------- M = 39, I = 1 ---------------------------
M = 39
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.441130E+01, 8.984785E+02, 2.519098E+03, 4.622384E+03, 7.153276E+03, 1.012212E+04,
    1.356235E+04, 1.751151E+04, 2.200662E+04, 2.708504E+04, 3.278708E+04, 3.915884E+04,
    4.625465E+04, 5.413906E+04, 6.288836E+04, 7.259164E+04, 8.335181E+04, 9.528638E+04,
    1.085284E+05, 1.232272E+05, 1.395500E+05, 1.576826E+05, 1.778316E+05, 2.002254E+05,
    2.251168E+05, 2.527848E+05, 2.835372E+05, 3.177134E+05, 3.556871E+05, 3.978696E+05,
    4.447142E+05, 4.967184E+05, 5.544300E+05, 6.184503E+05, 6.894398E+05, 7.681233E+05,
    8.552960E+05, 9.518298E+05, 1.058680E+06, 1.176894E+06, 1.307616E+06, 1.452099E+06,
    1.611715E+06, 1.787960E+06, 1.982471E+06, 2.197034E+06, 2.433598E+06, 2.694287E+06,
    2.981419E+06, 3.297516E+06, 3.645326E+06, 4.027840E+06, 4.448308E+06, 4.910266E+06,
    5.417556E+06, 5.974349E+06, 6.585171E+06, 7.254935E+06, 7.988968E+06, 8.793039E+06,
    9.673404E+06, 1.063683E+07, 1.169064E+07, 1.284277E+07, 1.410177E+07, 1.547691E+07,
    1.697818E+07, 1.861638E+07, 2.040317E+07, 2.235109E+07, 2.447370E+07, 2.678560E+07,
    2.930251E+07, 3.204136E+07, 3.502035E+07, 3.825912E+07, 4.177870E+07, 4.560178E+07,
    4.975264E+07, 5.425747E+07, 5.914429E+07, 6.444319E+07, 7.018647E+07, 7.640872E+07,
    8.314707E+07, 9.044122E+07, 9.833376E+07, 1.068702E+08, 1.160993E+08, 1.260732E+08,
    1.368476E+08, 1.484821E+08, 1.610404E+08, 1.745904E+08, 1.892048E+08, 2.049609E+08,
    2.219414E+08, 2.402345E+08, 2.599341E+08, 2.811403E+08, 3.039598E+08, 3.285064E+08,
    3.549009E+08, 3.832720E+08, 4.137567E+08, 4.465008E+08, 4.816591E+08, 5.193963E+08,
    5.598872E+08, 6.033178E+08, 6.498853E+08, 6.997992E+08, 7.532821E+08, 8.105694E+08,
    8.719115E+08, 9.375733E+08, 1.007836E+09, 1.082998E+09, 1.163375E+09, 1.249300E+09,
    1.341128E+09, 1.439233E+09, 1.544011E+09, 1.655881E+09, 1.775287E+09, 1.902698E+09,
    2.038608E+09, 2.183542E+09, 2.338052E+09, 2.502722E+09, 2.678169E+09, 2.865044E+09,
    3.064034E+09, 3.275863E+09, 3.501296E+09, 3.741140E+09, 3.996242E+09, 4.267501E+09,
    4.555859E+09, 4.862310E+09, 5.187903E+09, 5.533738E+09, 5.900977E+09, 6.290843E+09,
    6.704620E+09, 7.143662E+09, 7.609388E+09, 8.103298E+09, 8.626961E+09, 9.182033E+09,
    9.770245E+09, 1.039343E+10, 1.105349E+10, 1.175245E+10, 1.249242E+10, 1.327562E+10,
    1.410437E+10, 1.498112E+10, 1.590843E+10, 1.688899E+10, 1.792560E+10, 1.902123E+10,
    2.017898E+10, 2.140208E+10, 2.269394E+10, 2.405813E+10, 2.549835E+10, 2.701854E+10,
    2.862278E+10, 3.031533E+10, 3.210070E+10, 3.398356E+10, 3.596882E+10, 3.806161E+10,
    4.026729E+10, 4.259149E+10,
])

#  ---------------------- M = 40, I = 1 ---------------------------
M = 40
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.004878E+01, 1.395096E+03, 3.835885E+03, 7.029882E+03, 1.081742E+04, 1.511632E+04,
    1.987927E+04, 2.508270E+04, 3.072275E+04, 3.681224E+04, 4.337811E+04, 5.045934E+04,
    5.810536E+04, 6.637497E+04, 7.533555E+04, 8.506260E+04, 9.563945E+04, 1.071573E+05,
    1.197152E+05, 1.334205E+05, 1.483892E+05, 1.647463E+05, 1.826266E+05, 2.021755E+05,
    2.235493E+05, 2.469166E+05, 2.724590E+05, 3.003719E+05, 3.308659E+05, 3.641678E+05,
    4.005214E+05, 4.401896E+05, 4.834550E+05, 5.306217E+05, 5.820165E+05, 6.379909E+05,
    6.989225E+05, 7.652168E+05, 8.373091E+05, 9.156666E+05, 1.000790E+06, 1.093217E+06,
    1.193523E+06, 1.302323E+06, 1.420279E+06, 1.548095E+06, 1.686527E+06, 1.836381E+06,
    1.998520E+06, 2.173864E+06, 2.363397E+06, 2.568166E+06, 2.789290E+06, 3.027961E+06,
    3.285448E+06, 3.563104E+06, 3.862368E+06, 4.184771E+06, 4.531944E+06, 4.905616E+06,
    5.307629E+06, 5.739936E+06, 6.204613E+06, 6.703863E+06, 7.240020E+06, 7.815564E+06,
    8.433119E+06, 9.095469E+06, 9.805559E+06, 1.056651E+07, 1.138163E+07, 1.225440E+07,
    1.318853E+07, 1.418791E+07, 1.525668E+07, 1.639920E+07, 1.762006E+07, 1.892414E+07,
    2.031655E+07, 2.180272E+07, 2.338834E+07, 2.507943E+07, 2.688234E+07, 2.880373E+07,
    3.085064E+07, 3.303048E+07, 3.535103E+07, 3.782050E+07, 4.044751E+07, 4.324112E+07,
    4.621087E+07, 4.936677E+07, 5.271934E+07, 5.627963E+07, 6.005924E+07, 6.407034E+07,
    6.832572E+07, 7.283878E+07, 7.762356E+07, 8.269480E+07, 8.806795E+07, 9.375919E+07,
    9.978547E+07, 1.061645E+08, 1.129150E+08, 1.200563E+08, 1.276088E+08, 1.355938E+08,
    1.440336E+08, 1.529514E+08, 1.623717E+08, 1.723198E+08, 1.828225E+08, 1.939074E+08,
    2.056036E+08, 2.179413E+08, 2.309523E+08, 2.446695E+08, 2.591273E+08, 2.743617E+08,
    2.904102E+08, 3.073119E+08, 3.251074E+08, 3.438393E+08, 3.635518E+08, 3.842910E+08,
    4.061049E+08, 4.290435E+08, 4.531588E+08, 4.785052E+08, 5.051389E+08, 5.331187E+08,
    5.625056E+08, 5.933632E+08, 6.257574E+08, 6.597570E+08, 6.954334E+08, 7.328607E+08,
    7.721161E+08, 8.132796E+08, 8.564345E+08, 9.016672E+08, 9.490674E+08, 9.987283E+08,
    1.050747E+09, 1.105223E+09, 1.162261E+09, 1.221968E+09, 1.284458E+09, 1.349847E+09,
    1.418253E+09, 1.489804E+09, 1.564628E+09, 1.642860E+09, 1.724638E+09, 1.810107E+09,
    1.899416E+09, 1.992720E+09, 2.090179E+09, 2.191959E+09, 2.298231E+09, 2.409173E+09,
    2.524970E+09, 2.645811E+09, 2.771894E+09, 2.903422E+09, 3.040606E+09, 3.183664E+09,
    3.332821E+09, 3.488310E+09, 3.650373E+09, 3.819257E+09, 3.995222E+09, 4.178531E+09,
    4.369461E+09, 4.568295E+09,
])

#  ---------------------- M = 40, I = 2 ---------------------------
M = 40
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.014586E+01, 1.400380E+03, 3.850470E+03, 7.056645E+03, 1.085863E+04, 1.517397E+04,
    1.995524E+04, 2.517891E+04, 3.084126E+04, 3.695530E+04, 4.354822E+04, 5.065922E+04,
    5.833802E+04, 6.664374E+04, 7.564409E+04, 8.541493E+04, 9.604001E+04, 1.076110E+05,
    1.202274E+05, 1.339971E+05, 1.490367E+05, 1.654718E+05, 1.834380E+05, 2.030813E+05,
    2.245589E+05, 2.480404E+05, 2.737081E+05, 3.017585E+05, 3.324034E+05, 3.658707E+05,
    4.024056E+05, 4.422723E+05, 4.857550E+05, 5.331592E+05, 5.848137E+05, 6.410718E+05,
    7.023131E+05, 7.689453E+05, 8.414060E+05, 9.201650E+05, 1.005726E+06, 1.098629E+06,
    1.199452E+06, 1.308815E+06, 1.427382E+06, 1.555862E+06, 1.695014E+06, 1.845650E+06,
    2.008636E+06, 2.184898E+06, 2.375424E+06, 2.581269E+06, 2.803557E+06, 3.043485E+06,
    3.302331E+06, 3.581455E+06, 3.882303E+06, 4.206416E+06, 4.555432E+06, 4.931091E+06,
    5.335245E+06, 5.769857E+06, 6.237015E+06, 6.738933E+06, 7.277960E+06, 7.856588E+06,
    8.477456E+06, 9.143363E+06, 9.857272E+06, 1.062232E+07, 1.144183E+07, 1.231931E+07,
    1.325848E+07, 1.426327E+07, 1.533782E+07, 1.648652E+07, 1.771400E+07, 1.902515E+07,
    2.042513E+07, 2.191937E+07, 2.351362E+07, 2.521392E+07, 2.702665E+07, 2.895852E+07,
    3.101660E+07, 3.320835E+07, 3.554159E+07, 3.802456E+07, 4.066595E+07, 4.347487E+07,
    4.646089E+07, 4.963410E+07, 5.300508E+07, 5.658492E+07, 6.038531E+07, 6.441848E+07,
    6.869728E+07, 7.323518E+07, 7.804633E+07, 8.314553E+07, 8.854833E+07, 9.427099E+07,
    1.003306E+08, 1.067449E+08, 1.135327E+08, 1.207135E+08, 1.283078E+08, 1.363370E+08,
    1.448235E+08, 1.537908E+08, 1.632633E+08, 1.732667E+08, 1.838276E+08, 1.949741E+08,
    2.067353E+08, 2.191416E+08, 2.322250E+08, 2.460185E+08, 2.605568E+08, 2.758761E+08,
    2.920141E+08, 3.090099E+08, 3.269047E+08, 3.457411E+08, 3.655636E+08, 3.864186E+08,
    4.083543E+08, 4.314211E+08, 4.556712E+08, 4.811593E+08, 5.079420E+08, 5.360784E+08,
    5.656298E+08, 5.966602E+08, 6.292359E+08, 6.634261E+08, 6.993024E+08, 7.369397E+08,
    7.764152E+08, 8.178097E+08, 8.612069E+08, 9.066936E+08, 9.543600E+08, 1.004300E+09,
    1.056611E+09, 1.111393E+09, 1.168752E+09, 1.228795E+09, 1.291637E+09, 1.357393E+09,
    1.426185E+09, 1.498139E+09, 1.573384E+09, 1.652057E+09, 1.734296E+09, 1.820247E+09,
    1.910060E+09, 2.003890E+09, 2.101899E+09, 2.204253E+09, 2.311125E+09, 2.422694E+09,
    2.539145E+09, 2.660669E+09, 2.787464E+09, 2.919735E+09, 3.057694E+09, 3.201561E+09,
    3.351562E+09, 3.507931E+09, 3.670911E+09, 3.840752E+09, 4.017712E+09, 4.202060E+09,
    4.394071E+09, 4.594031E+09,
])

#  ---------------------- M = 41, I = 1 ---------------------------
M = 41
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.315131E+01, 1.079811E+03, 2.965691E+03, 5.436069E+03, 8.385490E+03, 1.180644E+04,
    1.574958E+04, 2.029388E+04, 2.553119E+04, 3.156166E+04, 3.849468E+04, 4.645162E+04,
    5.556906E+04, 6.600179E+04, 7.792578E+04, 9.154102E+04, 1.070745E+05, 1.247831E+05,
    1.449571E+05, 1.679240E+05, 1.940520E+05, 2.237552E+05, 2.574980E+05, 2.958013E+05,
    3.392479E+05, 3.884899E+05, 4.442559E+05, 5.073594E+05, 5.787080E+05, 6.593131E+05,
    7.503014E+05, 8.529264E+05, 9.685822E+05, 1.098818E+06, 1.245353E+06, 1.410094E+06,
    1.595158E+06, 1.802885E+06, 2.035868E+06, 2.296976E+06, 2.589376E+06, 2.916568E+06,
    3.282415E+06, 3.691174E+06, 4.147540E+06, 4.656680E+06, 5.224279E+06, 5.856591E+06,
    6.560485E+06, 7.343508E+06, 8.213936E+06, 9.180849E+06, 1.025420E+07, 1.144487E+07,
    1.276479E+07, 1.422701E+07, 1.584577E+07, 1.763664E+07, 1.961662E+07, 2.180426E+07,
    2.421975E+07, 2.688513E+07, 2.982439E+07, 3.306362E+07, 3.663123E+07, 4.055806E+07,
    4.487764E+07, 4.962639E+07, 5.484381E+07, 6.057274E+07, 6.685963E+07, 7.375479E+07,
    8.131271E+07, 8.959236E+07, 9.865750E+07, 1.085771E+08, 1.194257E+08, 1.312836E+08,
    1.442379E+08, 1.583821E+08, 1.738174E+08, 1.906527E+08, 2.090052E+08, 2.290015E+08,
    2.507774E+08, 2.744793E+08, 3.002646E+08, 3.283022E+08, 3.587741E+08, 3.918751E+08,
    4.278148E+08, 4.668179E+08, 5.091253E+08, 5.549954E+08, 6.047049E+08, 6.585505E+08,
    7.168495E+08, 7.799418E+08, 8.481907E+08, 9.219851E+08, 1.001740E+09, 1.087900E+09,
    1.180939E+09, 1.281364E+09, 1.389714E+09, 1.506566E+09, 1.632536E+09, 1.768278E+09,
    1.914492E+09, 2.071922E+09, 2.241360E+09, 2.423650E+09, 2.619690E+09, 2.830436E+09,
    3.056904E+09, 3.300174E+09, 3.561394E+09, 3.841784E+09, 4.142639E+09, 4.465335E+09,
    4.811331E+09, 5.182177E+09, 5.579515E+09, 6.005089E+09, 6.460744E+09, 6.948439E+09,
    7.470246E+09, 8.028363E+09, 8.625114E+09, 9.262960E+09, 9.944506E+09, 1.067251E+10,
    1.144988E+10, 1.227970E+10, 1.316522E+10, 1.410990E+10, 1.511735E+10, 1.619142E+10,
    1.733617E+10, 1.855587E+10, 1.985503E+10, 2.123842E+10, 2.271106E+10, 2.427825E+10,
    2.594556E+10, 2.771889E+10, 2.960444E+10, 3.160873E+10, 3.373864E+10, 3.600142E+10,
    3.840468E+10, 4.095644E+10, 4.366516E+10, 4.653969E+10, 4.958938E+10, 5.282404E+10,
    5.625398E+10, 5.989006E+10, 6.374364E+10, 6.782671E+10, 7.215183E+10, 7.673218E+10,
    8.158161E+10, 8.671466E+10, 9.214658E+10, 9.789336E+10, 1.039718E+11, 1.103994E+11,
    1.171948E+11, 1.243772E+11, 1.319668E+11, 1.399850E+11, 1.484539E+11, 1.573968E+11,
    1.668382E+11, 1.768036E+11,
])

#  ---------------------- M = 41, I = 2 ---------------------------
M = 41
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.756468E+01, 2.228120E+03, 6.122297E+03, 1.122300E+04, 1.731317E+04, 2.437845E+04,
    3.252465E+04, 4.191614E+04, 5.274423E+04, 6.521817E+04, 7.956715E+04, 9.604612E+04,
    1.149423E+05, 1.365814E+05, 1.613342E+05, 1.896217E+05, 2.219220E+05, 2.587765E+05,
    3.007971E+05, 3.486737E+05, 4.031834E+05, 4.651993E+05, 5.357021E+05, 6.157912E+05,
    7.066979E+05, 8.098005E+05, 9.266395E+05, 1.058936E+06, 1.208610E+06, 1.377803E+06,
    1.568902E+06, 1.784563E+06, 2.027740E+06, 2.301720E+06, 2.610149E+06, 2.957077E+06,
    3.346993E+06, 3.784872E+06, 4.276222E+06, 4.827137E+06, 5.444356E+06, 6.135322E+06,
    6.908250E+06, 7.772205E+06, 8.737178E+06, 9.814173E+06, 1.101530E+07, 1.235389E+07,
    1.384459E+07, 1.550347E+07, 1.734820E+07, 1.939814E+07, 2.167453E+07, 2.420060E+07,
    2.700184E+07, 3.010607E+07, 3.354375E+07, 3.734815E+07, 4.155558E+07, 4.620567E+07,
    5.134166E+07, 5.701064E+07, 6.326393E+07, 7.015737E+07, 7.775173E+07, 8.611306E+07,
    9.531315E+07, 1.054300E+08, 1.165481E+08, 1.287594E+08, 1.421633E+08, 1.568678E+08,
    1.729895E+08, 1.906549E+08, 2.100008E+08, 2.311751E+08, 2.543376E+08, 2.796608E+08,
    3.073314E+08, 3.375503E+08, 3.705345E+08, 4.065179E+08, 4.457524E+08, 4.885096E+08,
    5.350814E+08, 5.857824E+08, 6.409505E+08, 7.009493E+08, 7.661692E+08, 8.370298E+08,
    9.139812E+08, 9.975068E+08, 1.088125E+09, 1.186391E+09, 1.292901E+09, 1.408292E+09,
    1.533248E+09, 1.668500E+09, 1.814831E+09, 1.973077E+09, 2.144133E+09, 2.328955E+09,
    2.528564E+09, 2.744052E+09, 2.976580E+09, 3.227394E+09, 3.497816E+09, 3.789260E+09,
    4.103232E+09, 4.441336E+09, 4.805281E+09, 5.196886E+09, 5.618088E+09, 6.070948E+09,
    6.557656E+09, 7.080543E+09, 7.642086E+09, 8.244916E+09, 8.891828E+09, 9.585791E+09,
    1.032996E+10, 1.112766E+10, 1.198246E+10, 1.289812E+10, 1.387861E+10, 1.492818E+10,
    1.605128E+10, 1.725268E+10, 1.853739E+10, 1.991072E+10, 2.137831E+10, 2.294611E+10,
    2.462041E+10, 2.640788E+10, 2.831554E+10, 3.035083E+10, 3.252162E+10, 3.483620E+10,
    3.730333E+10, 3.993226E+10, 4.273276E+10, 4.571512E+10, 4.889021E+10, 5.226948E+10,
    5.586501E+10, 5.968954E+10, 6.375646E+10, 6.807992E+10, 7.267480E+10, 7.755676E+10,
    8.274231E+10, 8.824879E+10, 9.409449E+10, 1.002986E+11, 1.068814E+11, 1.138640E+11,
    1.212689E+11, 1.291194E+11, 1.374404E+11, 1.462576E+11, 1.555982E+11, 1.654910E+11,
    1.759658E+11, 1.870541E+11, 1.987891E+11, 2.112053E+11, 2.243391E+11, 2.382286E+11,
    2.529139E+11, 2.684368E+11, 2.848412E+11, 3.021733E+11, 3.204811E+11, 3.398152E+11,
    3.602284E+11, 3.817762E+11,
])

#  ---------------------- M = 41, I = 3 ---------------------------
M = 41
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.640829E+01, 2.165303E+03, 5.949088E+03, 1.090578E+04, 1.682930E+04, 2.371551E+04,
    3.167737E+04, 4.088260E+04, 5.152311E+04, 6.380713E+04, 7.796181E+04, 9.423931E+04,
    1.129235E+05, 1.343366E+05, 1.588447E+05, 1.868645E+05, 2.188692E+05, 2.553945E+05,
    2.970463E+05, 3.445084E+05, 3.985504E+05, 4.600380E+05, 5.299431E+05, 6.093559E+05,
    6.994976E+05, 8.017346E+05, 9.175952E+05, 1.048786E+06, 1.197212E+06, 1.364999E+06,
    1.554512E+06, 1.768388E+06, 2.009559E+06, 2.281283E+06, 2.587180E+06, 2.931268E+06,
    3.318001E+06, 3.752316E+06, 4.239679E+06, 4.786137E+06, 5.398376E+06, 6.083784E+06,
    6.850514E+06, 7.707561E+06, 8.664841E+06, 9.733275E+06, 1.092489E+07, 1.225290E+07,
    1.373184E+07, 1.537769E+07, 1.720796E+07, 1.924188E+07, 2.150051E+07, 2.400695E+07,
    2.678645E+07, 2.986667E+07, 3.327782E+07, 3.705292E+07, 4.122803E+07, 4.584250E+07,
    5.093921E+07, 5.656495E+07, 6.277064E+07, 6.961172E+07, 7.714850E+07, 8.544659E+07,
    9.457722E+07, 1.046178E+08, 1.156524E+08, 1.277720E+08, 1.410755E+08, 1.556700E+08,
    1.716714E+08, 1.892051E+08, 2.084071E+08, 2.294241E+08, 2.524148E+08, 2.775507E+08,
    3.050167E+08, 3.350126E+08, 3.677537E+08, 4.034723E+08, 4.424186E+08, 4.848620E+08,
    5.310927E+08, 5.814227E+08, 6.361877E+08, 6.957486E+08, 7.604932E+08, 8.308379E+08,
    9.072299E+08, 9.901490E+08, 1.080110E+09, 1.177664E+09, 1.283403E+09, 1.397960E+09,
    1.522014E+09, 1.656291E+09, 1.801568E+09, 1.958675E+09, 2.128501E+09, 2.311996E+09,
    2.510172E+09, 2.724114E+09, 2.954978E+09, 3.203997E+09, 3.472486E+09, 3.761848E+09,
    4.073580E+09, 4.409273E+09, 4.770625E+09, 5.159443E+09, 5.577650E+09, 6.027291E+09,
    6.510543E+09, 7.029720E+09, 7.587282E+09, 8.185841E+09, 8.828174E+09, 9.517228E+09,
    1.025613E+10, 1.104820E+10, 1.189697E+10, 1.280617E+10, 1.377975E+10, 1.482192E+10,
    1.593712E+10, 1.713006E+10, 1.840573E+10, 1.976942E+10, 2.122670E+10, 2.278350E+10,
    2.444605E+10, 2.622098E+10, 2.811528E+10, 3.013632E+10, 3.229192E+10, 3.459030E+10,
    3.704019E+10, 3.965075E+10, 4.243169E+10, 4.539323E+10, 4.854616E+10, 5.190187E+10,
    5.547234E+10, 5.927022E+10, 6.330882E+10, 6.760219E+10, 7.216510E+10, 7.701311E+10,
    8.216261E+10, 8.763084E+10, 9.343594E+10, 9.959700E+10, 1.061341E+11, 1.130683E+11,
    1.204218E+11, 1.282180E+11, 1.364812E+11, 1.452374E+11, 1.545134E+11, 1.643377E+11,
    1.747400E+11, 1.857517E+11, 1.974056E+11, 2.097360E+11, 2.227790E+11, 2.365727E+11,
    2.511566E+11, 2.665724E+11, 2.828637E+11, 3.000762E+11, 3.182578E+11, 3.374587E+11,
    3.577313E+11, 3.791307E+11,
])

#  ---------------------- M = 41, I = 4 ---------------------------
M = 41
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    9.539916E+01, 4.470892E+03, 1.228501E+04, 2.252154E+04, 3.475468E+04, 4.897554E+04,
    6.541725E+04, 8.442665E+04, 1.064017E+05, 1.317753E+05, 1.610212E+05, 1.946663E+05,
    2.333045E+05, 2.776101E+05, 3.283502E+05, 3.863976E+05, 4.527432E+05, 5.285101E+05,
    6.149681E+05, 7.135503E+05, 8.258710E+05, 9.537460E+05, 1.099215E+06, 1.264566E+06,
    1.452364E+06, 1.665478E+06, 1.907122E+06, 2.180882E+06, 2.490766E+06, 2.841242E+06,
    3.237293E+06, 3.684466E+06, 4.188938E+06, 4.757571E+06, 5.397996E+06, 6.118680E+06,
    6.929018E+06, 7.839421E+06, 8.861424E+06, 1.000779E+07, 1.129264E+07, 1.273156E+07,
    1.434179E+07, 1.614232E+07, 1.815412E+07, 2.040028E+07, 2.290621E+07, 2.569988E+07,
    2.881206E+07, 3.227651E+07, 3.613034E+07, 4.041424E+07, 4.517284E+07, 5.045502E+07,
    5.631431E+07, 6.280930E+07, 7.000402E+07, 7.796850E+07, 8.677916E+07, 9.651945E+07,
    1.072804E+08, 1.191612E+08, 1.322698E+08, 1.467241E+08, 1.626520E+08, 1.801928E+08,
    1.994978E+08, 2.207313E+08, 2.440718E+08, 2.697130E+08, 2.978648E+08, 3.287546E+08,
    3.626291E+08, 3.997549E+08, 4.404208E+08, 4.849391E+08, 5.336473E+08, 5.869099E+08,
    6.451208E+08, 7.087047E+08, 7.781201E+08, 8.538612E+08, 9.364606E+08, 1.026492E+09,
    1.124573E+09, 1.231368E+09, 1.347593E+09, 1.474015E+09, 1.611462E+09, 1.760820E+09,
    1.923041E+09, 2.099150E+09, 2.290241E+09, 2.497492E+09, 2.722164E+09, 2.965606E+09,
    3.229265E+09, 3.514690E+09, 3.823537E+09, 4.157580E+09, 4.518712E+09, 4.908959E+09,
    5.330485E+09, 5.785601E+09, 6.276774E+09, 6.806635E+09, 7.377994E+09, 7.993846E+09,
    8.657381E+09, 9.372002E+09, 1.014133E+10, 1.096923E+10, 1.185980E+10, 1.281742E+10,
    1.384672E+10, 1.495267E+10, 1.614050E+10, 1.741580E+10, 1.878451E+10, 2.025292E+10,
    2.182772E+10, 2.351600E+10, 2.532529E+10, 2.726358E+10, 2.933934E+10, 3.156152E+10,
    3.393966E+10, 3.648380E+10, 3.920463E+10, 4.211343E+10, 4.522214E+10, 4.854343E+10,
    5.209065E+10, 5.587795E+10, 5.992028E+10, 6.423345E+10, 6.883414E+10, 7.373998E+10,
    7.896960E+10, 8.454265E+10, 9.047989E+10, 9.680319E+10, 1.035357E+11, 1.107016E+11,
    1.183268E+11, 1.264382E+11, 1.350644E+11, 1.442354E+11, 1.539828E+11, 1.643401E+11,
    1.753422E+11, 1.870260E+11, 1.994305E+11, 2.125965E+11, 2.265670E+11, 2.413872E+11,
    2.571045E+11, 2.737690E+11, 2.914332E+11, 3.101521E+11, 3.299836E+11, 3.509886E+11,
    3.732309E+11, 3.967775E+11, 4.216988E+11, 4.480684E+11, 4.759638E+11, 5.054661E+11,
    5.366605E+11, 5.696362E+11, 6.044865E+11, 6.413096E+11, 6.802081E+11, 7.212894E+11,
    7.646663E+11, 8.104566E+11,
])

#  ---------------------- M = 42, I = 1 ---------------------------
M = 42
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00, 0.000000E+00,
    0.000000E+00, 0.000000E+00,
])

#  ---------------------- M = 43, I = 1 ---------------------------
M = 43
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.019112E+01, 1.905753E+02, 3.807665E+02, 5.762817E+02, 7.905060E+02, 1.038159E+03,
    1.333345E+03, 1.691617E+03, 2.131702E+03, 2.676627E+03, 3.354709E+03, 4.200671E+03,
    5.256974E+03, 6.575423E+03, 8.219051E+03, 1.026434E+04, 1.280384E+04, 1.594915E+04,
    1.983447E+04, 2.462068E+04, 3.050003E+04, 3.770160E+04, 4.649756E+04, 5.721039E+04,
    7.022113E+04, 8.597887E+04, 1.050115E+05, 1.279381E+05, 1.554830E+05, 1.884915E+05,
    2.279483E+05, 2.749977E+05, 3.309668E+05, 3.973918E+05, 4.760472E+05, 5.689788E+05,
    6.785415E+05, 8.074402E+05, 9.587774E+05, 1.136105E+06, 1.343483E+06, 1.585544E+06,
    1.867567E+06, 2.195558E+06, 2.576338E+06, 3.017647E+06, 3.528251E+06, 4.118066E+06,
    4.798292E+06, 5.581567E+06, 6.482126E+06, 7.515994E+06, 8.701176E+06, 1.005789E+07,
    1.160879E+07, 1.337926E+07, 1.539769E+07, 1.769582E+07, 2.030904E+07, 2.327685E+07,
    2.664322E+07, 3.045709E+07, 3.477285E+07, 3.965090E+07, 4.515827E+07, 5.136922E+07,
    5.836599E+07, 6.623958E+07, 7.509054E+07, 8.502994E+07, 9.618032E+07, 1.086768E+08,
    1.226681E+08, 1.383180E+08, 1.558067E+08, 1.753321E+08, 1.971114E+08, 2.213829E+08,
    2.484083E+08, 2.784735E+08, 3.118921E+08, 3.490069E+08, 3.901925E+08, 4.358585E+08,
    4.864516E+08, 5.424591E+08, 6.044125E+08, 6.728905E+08, 7.485237E+08, 8.319970E+08,
    9.240564E+08, 1.025512E+09, 1.137243E+09, 1.260205E+09, 1.395433E+09, 1.544051E+09,
    1.707273E+09, 1.886418E+09, 2.082910E+09, 2.298291E+09, 2.534229E+09, 2.792522E+09,
    3.075116E+09, 3.384109E+09, 3.721764E+09, 4.090522E+09, 4.493014E+09, 4.932072E+09,
    5.410747E+09, 5.932319E+09, 6.500319E+09, 7.118541E+09, 7.791064E+09, 8.522268E+09,
    9.316850E+09, 1.017986E+10, 1.111670E+10, 1.213319E+10, 1.323554E+10, 1.443039E+10,
    1.572490E+10, 1.712669E+10, 1.864393E+10, 2.028536E+10, 2.206030E+10, 2.397874E+10,
    2.605133E+10, 2.828946E+10, 3.070527E+10, 3.331171E+10, 3.612261E+10, 3.915271E+10,
    4.241771E+10, 4.593434E+10, 4.972043E+10, 5.379494E+10, 5.817805E+10, 6.289125E+10,
    6.795741E+10, 7.340077E+10, 7.924717E+10, 8.552402E+10, 9.226047E+10, 9.948742E+10,
    1.072377E+11, 1.155462E+11, 1.244497E+11, 1.339876E+11, 1.442013E+11, 1.551348E+11,
    1.668348E+11, 1.793507E+11, 1.927348E+11, 2.070423E+11, 2.223319E+11, 2.386657E+11,
    2.561090E+11, 2.747312E+11, 2.946056E+11, 3.158095E+11, 3.384248E+11, 3.625376E+11,
    3.882395E+11, 4.156265E+11, 4.448002E+11, 4.758678E+11, 5.089422E+11, 5.441429E+11,
    5.815953E+11, 6.214321E+11, 6.637927E+11, 7.088241E+11, 7.566812E+11, 8.075271E+11,
    8.615336E+11, 9.188811E+11,
])

#  ---------------------- M = 44, I = 1 ---------------------------
M = 44
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.957662E+01, 5.518291E+02, 1.102633E+03, 1.668279E+03, 2.286281E+03, 2.997450E+03,
    3.838554E+03, 4.846743E+03, 6.063697E+03, 7.537981E+03, 9.326765E+03, 1.149723E+04,
    1.412808E+04, 1.731129E+04, 2.115428E+04, 2.578174E+04, 3.133815E+04, 3.799064E+04,
    4.593183E+04, 5.538330E+04, 6.659913E+04, 7.986998E+04, 9.552786E+04, 1.139507E+05,
    1.355685E+05, 1.608685E+05, 1.904027E+05, 2.247946E+05, 2.647478E+05, 3.110535E+05,
    3.646019E+05, 4.263913E+05, 4.975399E+05, 5.792987E+05, 6.730643E+05, 7.803938E+05,
    9.030217E+05, 1.042876E+06, 1.202097E+06, 1.383060E+06, 1.588392E+06, 1.821003E+06,
    2.084105E+06, 2.381241E+06, 2.716320E+06, 3.093641E+06, 3.517933E+06, 3.994391E+06,
    4.528714E+06, 5.127151E+06, 5.796541E+06, 6.544370E+06, 7.378816E+06, 8.308808E+06,
    9.344088E+06, 1.049527E+07, 1.177392E+07, 1.319260E+07, 1.476499E+07, 1.650593E+07,
    1.843154E+07, 2.055927E+07, 2.290806E+07, 2.549840E+07, 2.835243E+07, 3.149414E+07,
    3.494939E+07, 3.874612E+07, 4.291446E+07, 4.748690E+07, 5.249840E+07, 5.798665E+07,
    6.399214E+07, 7.055842E+07, 7.773230E+07, 8.556400E+07, 9.410744E+07, 1.034204E+08,
    1.135649E+08, 1.246073E+08, 1.366186E+08, 1.496748E+08, 1.638572E+08, 1.792528E+08,
    1.959542E+08, 2.140605E+08, 2.336775E+08, 2.549180E+08, 2.779020E+08, 3.027576E+08,
    3.296212E+08, 3.586378E+08, 3.899621E+08, 4.237580E+08, 4.602001E+08, 4.994740E+08,
    5.417764E+08, 5.873165E+08, 6.363159E+08, 6.890100E+08, 7.456480E+08, 8.064942E+08,
    8.718285E+08, 9.419473E+08, 1.017164E+09, 1.097811E+09, 1.184239E+09, 1.276819E+09,
    1.375943E+09, 1.482026E+09, 1.595504E+09, 1.716839E+09, 1.846519E+09, 1.985058E+09,
    2.132997E+09, 2.290908E+09, 2.459392E+09, 2.639085E+09, 2.830653E+09, 3.034800E+09,
    3.252265E+09, 3.483827E+09, 3.730304E+09, 3.992557E+09, 4.271490E+09, 4.568055E+09,
    4.883248E+09, 5.218118E+09, 5.573767E+09, 5.951348E+09, 6.352075E+09, 6.777217E+09,
    7.228108E+09, 7.706145E+09, 8.212792E+09, 8.749583E+09, 9.318127E+09, 9.920104E+09,
    1.055728E+10, 1.123149E+10, 1.194468E+10, 1.269885E+10, 1.349612E+10, 1.433869E+10,
    1.522888E+10, 1.616909E+10, 1.716185E+10, 1.820978E+10, 1.931564E+10, 2.048228E+10,
    2.171272E+10, 2.301006E+10, 2.437758E+10, 2.581868E+10, 2.733690E+10, 2.893594E+10,
    3.061966E+10, 3.239208E+10, 3.425739E+10, 3.621994E+10, 3.828429E+10, 4.045517E+10,
    4.273749E+10, 4.513640E+10, 4.765723E+10, 5.030553E+10, 5.308708E+10, 5.600790E+10,
    5.907422E+10, 6.229255E+10, 6.566964E+10, 6.921252E+10, 7.292848E+10, 7.682509E+10,
    8.091023E+10, 8.519208E+10,
])

#  ---------------------- M = 44, I = 2 ---------------------------
M = 44
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.026479E+01, 3.788677E+02, 7.570816E+02, 1.145507E+03, 1.569915E+03, 2.058345E+03,
    2.636068E+03, 3.328628E+03, 4.164690E+03, 5.177665E+03, 6.406892E+03, 7.898611E+03,
    9.707022E+03, 1.189546E+04, 1.453793E+04, 1.772040E+04, 2.154245E+04, 2.611938E+04,
    3.158409E+04, 3.808955E+04, 4.581120E+04, 5.494989E+04, 6.573509E+04, 7.842821E+04,
    9.332667E+04, 1.107679E+05, 1.311340E+05, 1.548574E+05, 1.824256E+05, 2.143877E+05,
    2.513616E+05, 2.940409E+05, 3.432024E+05, 3.997164E+05, 4.645548E+05, 5.388019E+05,
    6.236663E+05, 7.204925E+05, 8.307742E+05, 9.561694E+05, 1.098515E+06, 1.259846E+06,
    1.442409E+06, 1.648688E+06, 1.881421E+06, 2.143626E+06, 2.438622E+06, 2.770061E+06,
    3.141951E+06, 3.558689E+06, 4.025097E+06, 4.546452E+06, 5.128527E+06, 5.777632E+06,
    6.500654E+06, 7.305109E+06, 8.199187E+06, 9.191807E+06, 1.029267E+07, 1.151234E+07,
    1.286226E+07, 1.435488E+07, 1.600369E+07, 1.782331E+07, 1.982959E+07, 2.203965E+07,
    2.447203E+07, 2.714675E+07, 3.008544E+07, 3.331144E+07, 3.684992E+07, 4.072800E+07,
    4.497489E+07, 4.962204E+07, 5.470328E+07, 6.025497E+07, 6.631617E+07, 7.292885E+07,
    8.013801E+07, 8.799194E+07, 9.654241E+07, 1.058449E+08, 1.159587E+08, 1.269474E+08,
    1.388789E+08, 1.518261E+08, 1.658664E+08, 1.810828E+08, 1.975638E+08, 2.154040E+08,
    2.347039E+08, 2.555711E+08, 2.781199E+08, 3.024721E+08, 3.287576E+08, 3.571142E+08,
    3.876889E+08, 4.206376E+08, 4.561262E+08, 4.943310E+08, 5.354389E+08, 5.796485E+08,
    6.271706E+08, 6.782284E+08, 7.330590E+08, 7.919132E+08, 8.550571E+08, 9.227721E+08,
    9.953563E+08, 1.073125E+09, 1.156413E+09, 1.245571E+09, 1.340974E+09, 1.443015E+09,
    1.552110E+09, 1.668700E+09, 1.793248E+09, 1.926244E+09, 2.068205E+09, 2.219676E+09,
    2.381232E+09, 2.553477E+09, 2.737051E+09, 2.932625E+09, 3.140908E+09, 3.362643E+09,
    3.598617E+09, 3.849653E+09, 4.116620E+09, 4.400430E+09, 4.702043E+09, 5.022468E+09,
    5.362763E+09, 5.724042E+09, 6.107473E+09, 6.514284E+09, 6.945762E+09, 7.403258E+09,
    7.888191E+09, 8.402048E+09, 8.946386E+09, 9.522842E+09, 1.013313E+10, 1.077904E+10,
    1.146245E+10, 1.218534E+10, 1.294977E+10, 1.375789E+10, 1.461196E+10, 1.551435E+10,
    1.646753E+10, 1.747409E+10, 1.853673E+10, 1.965827E+10, 2.084169E+10, 2.209005E+10,
    2.340660E+10, 2.479470E+10, 2.625787E+10, 2.779978E+10, 2.942429E+10, 3.113539E+10,
    3.293726E+10, 3.483426E+10, 3.683096E+10, 3.893208E+10, 4.114258E+10, 4.346763E+10,
    4.591259E+10, 4.848308E+10, 5.118493E+10, 5.402423E+10, 5.700733E+10, 6.014081E+10,
    6.343157E+10, 6.688675E+10,
])

#  ---------------------- M = 44, I = 3 ---------------------------
M = 44
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.938131E+01, 1.108247E+03, 2.214517E+03, 3.350727E+03, 4.592300E+03, 6.021313E+03,
    7.711766E+03, 9.738490E+03, 1.218549E+04, 1.515071E+04, 1.874951E+04, 2.311752E+04,
    2.841381E+04, 3.482427E+04, 4.256628E+04, 5.189234E+04, 6.309521E+04, 7.651387E+04,
    9.253932E+04, 1.116218E+05, 1.342779E+05, 1.610995E+05, 1.927630E+05, 2.300395E+05,
    2.738067E+05, 3.250609E+05, 3.849317E+05, 4.546970E+05, 5.357995E+05, 6.298651E+05,
    7.387238E+05, 8.644319E+05, 1.009295E+06, 1.175895E+06, 1.367122E+06, 1.586198E+06,
    1.836720E+06, 2.122693E+06, 2.448568E+06, 2.819290E+06, 3.240346E+06, 3.717812E+06,
    4.258411E+06, 4.869576E+06, 5.559511E+06, 6.337263E+06, 7.212800E+06, 8.197089E+06,
    9.302184E+06, 1.054133E+07, 1.192904E+07, 1.348124E+07, 1.521536E+07, 1.715048E+07,
    1.930742E+07, 2.170896E+07, 2.437993E+07, 2.734741E+07, 3.064087E+07, 3.429243E+07,
    3.833699E+07, 4.281248E+07, 4.776010E+07, 5.322453E+07, 5.925423E+07, 6.590171E+07,
    7.322381E+07, 8.128204E+07, 9.014290E+07, 9.987824E+07, 1.105657E+08, 1.222889E+08,
    1.351382E+08, 1.492109E+08, 1.646119E+08, 1.814539E+08, 1.998583E+08, 2.199558E+08,
    2.418865E+08, 2.658011E+08, 2.918612E+08, 3.202405E+08, 3.511248E+08, 3.847136E+08,
    4.212204E+08, 4.608738E+08, 5.039185E+08, 5.506162E+08, 6.012466E+08, 6.561088E+08,
    7.155218E+08, 7.798268E+08, 8.493872E+08, 9.245912E+08, 1.005852E+09, 1.093611E+09,
    1.188338E+09, 1.290532E+09, 1.400726E+09, 1.519485E+09, 1.647413E+09, 1.785148E+09,
    1.933372E+09, 2.092806E+09, 2.264219E+09, 2.448424E+09, 2.646286E+09, 2.858721E+09,
    3.086701E+09, 3.331257E+09, 3.593479E+09, 3.874525E+09, 4.175617E+09, 4.498052E+09,
    4.843200E+09, 5.212513E+09, 5.607523E+09, 6.029851E+09, 6.481212E+09, 6.963415E+09,
    7.478371E+09, 8.028099E+09, 8.614729E+09, 9.240508E+09, 9.907808E+09, 1.061913E+10,
    1.137711E+10, 1.218453E+10, 1.304431E+10, 1.395953E+10, 1.493346E+10, 1.596950E+10,
    1.707126E+10, 1.824252E+10, 1.948726E+10, 2.080968E+10, 2.221419E+10, 2.370541E+10,
    2.528822E+10, 2.696772E+10, 2.874930E+10, 3.063859E+10, 3.264151E+10, 3.476429E+10,
    3.701345E+10, 3.939584E+10, 4.191864E+10, 4.458939E+10, 4.741597E+10, 5.040668E+10,
    5.357018E+10, 5.691557E+10, 6.045237E+10, 6.419056E+10, 6.814058E+10, 7.231337E+10,
    7.672038E+10, 8.137359E+10, 8.628553E+10, 9.146932E+10, 9.693866E+10, 1.027079E+11,
    1.087920E+11, 1.152067E+11, 1.219684E+11, 1.290941E+11, 1.366017E+11, 1.445099E+11,
    1.528383E+11, 1.616071E+11, 1.708377E+11, 1.805521E+11, 1.907737E+11, 2.015265E+11,
    2.128357E+11, 2.247276E+11,
])

#  ---------------------- M = 44, I = 4 ---------------------------
M = 44
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.938661E+01, 1.108336E+03, 2.214589E+03, 3.350570E+03, 4.591574E+03, 6.019498E+03,
    7.708091E+03, 9.731833E+03, 1.217425E+04, 1.513263E+04, 1.872146E+04, 2.307521E+04,
    2.835138E+04, 3.473390E+04, 4.243751E+04, 5.171140E+04, 6.284408E+04, 7.616915E+04,
    9.207088E+04, 1.109911E+05, 1.334360E+05, 1.599844E+05, 1.912971E+05, 2.281258E+05,
    2.713248E+05, 3.218620E+05, 3.808325E+05, 4.494733E+05, 5.291779E+05, 6.215133E+05,
    7.282402E+05, 8.513320E+05, 9.929969E+05, 1.155703E+06, 1.342204E+06, 1.555566E+06,
    1.799202E+06, 2.076901E+06, 2.392867E+06, 2.751758E+06, 3.158724E+06, 3.619459E+06,
    4.140242E+06, 4.727999E+06, 5.390348E+06, 6.135671E+06, 6.973171E+06, 7.912946E+06,
    8.966061E+06, 1.014463E+07, 1.146190E+07, 1.293235E+07, 1.457177E+07, 1.639738E+07,
    1.842795E+07, 2.068389E+07, 2.318741E+07, 2.596262E+07, 2.903569E+07, 3.243502E+07,
    3.619134E+07, 4.033796E+07, 4.491089E+07, 4.994905E+07, 5.549449E+07, 6.159258E+07,
    6.829223E+07, 7.564616E+07, 8.371114E+07, 9.254824E+07, 1.022231E+08, 1.128064E+08,
    1.243737E+08, 1.370064E+08, 1.507918E+08, 1.658230E+08, 1.822004E+08, 2.000307E+08,
    2.194287E+08, 2.405167E+08, 2.634254E+08, 2.882945E+08, 3.152730E+08, 3.445196E+08,
    3.762037E+08, 4.105057E+08, 4.476175E+08, 4.877435E+08, 5.311009E+08, 5.779207E+08,
    6.284482E+08, 6.829439E+08, 7.416843E+08, 8.049627E+08, 8.730900E+08, 9.463957E+08,
    1.025229E+09, 1.109959E+09, 1.200977E+09, 1.298696E+09, 1.403554E+09, 1.516013E+09,
    1.636560E+09, 1.765712E+09, 1.904012E+09, 2.052033E+09, 2.210381E+09, 2.379692E+09,
    2.560640E+09, 2.753931E+09, 2.960311E+09, 3.180562E+09, 3.415509E+09, 3.666020E+09,
    3.933004E+09, 4.217420E+09, 4.520273E+09, 4.842618E+09, 5.185562E+09, 5.550269E+09,
    5.937956E+09, 6.349900E+09, 6.787441E+09, 7.251982E+09, 7.744989E+09, 8.268002E+09,
    8.822629E+09, 9.410552E+09, 1.003353E+10, 1.069341E+10, 1.139211E+10, 1.213163E+10,
    1.291408E+10, 1.374165E+10, 1.461662E+10, 1.554138E+10, 1.651842E+10, 1.755032E+10,
    1.863980E+10, 1.978967E+10, 2.100286E+10, 2.228244E+10, 2.363158E+10, 2.505360E+10,
    2.655195E+10, 2.813020E+10, 2.979208E+10, 3.154147E+10, 3.338240E+10, 3.531905E+10,
    3.735576E+10, 3.949704E+10, 4.174758E+10, 4.411223E+10, 4.659605E+10, 4.920424E+10,
    5.194225E+10, 5.481567E+10, 5.783035E+10, 6.099230E+10, 6.430779E+10, 6.778327E+10,
    7.142544E+10, 7.524123E+10, 7.923781E+10, 8.342260E+10, 8.780325E+10, 9.238769E+10,
    9.718412E+10, 1.022010E+11, 1.074470E+11, 1.129313E+11, 1.186630E+11, 1.246519E+11,
    1.309079E+11, 1.374411E+11,
])

#  ---------------------- M = 44, I = 5 ---------------------------
M = 44
I = 5
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.090077E+01, 1.138773E+03, 2.275792E+03, 3.443921E+03, 4.720878E+03, 6.191305E+03,
    7.931657E+03, 1.001940E+04, 1.254164E+04, 1.560008E+04, 1.931466E+04, 2.382667E+04,
    2.930200E+04, 3.593494E+04, 4.395300E+04, 5.362102E+04, 6.524665E+04, 7.918681E+04,
    9.585408E+04, 1.157246E+05, 1.393459E+05, 1.673468E+05, 2.004476E+05, 2.394715E+05,
    2.853580E+05, 3.391763E+05, 4.021417E+05, 4.756331E+05, 5.612115E+05, 6.606413E+05,
    7.759143E+05, 9.092750E+05, 1.063248E+06, 1.240670E+06, 1.444723E+06, 1.678972E+06,
    1.947406E+06, 2.254480E+06, 2.605165E+06, 3.005004E+06, 3.460163E+06, 3.977501E+06,
    4.564632E+06, 5.230003E+06, 5.982976E+06, 6.833909E+06, 7.794262E+06, 8.876687E+06,
    1.009515E+07, 1.146505E+07, 1.300334E+07, 1.472869E+07, 1.666160E+07, 1.882462E+07,
    2.124247E+07, 2.394228E+07, 2.695376E+07, 3.030945E+07, 3.404492E+07, 3.819908E+07,
    4.281440E+07, 4.793725E+07, 5.361820E+07, 5.991235E+07, 6.687972E+07, 7.458561E+07,
    8.310104E+07, 9.250322E+07, 1.028760E+08, 1.143103E+08, 1.269049E+08, 1.407668E+08,
    1.560118E+08, 1.727656E+08, 1.911638E+08, 2.113534E+08, 2.334931E+08, 2.577542E+08,
    2.843221E+08, 3.133963E+08, 3.451922E+08, 3.799422E+08, 4.178961E+08, 4.593235E+08,
    5.045140E+08, 5.537796E+08, 6.074553E+08, 6.659014E+08, 7.295048E+08, 7.986808E+08,
    8.738752E+08, 9.555661E+08, 1.044266E+09, 1.140524E+09, 1.244930E+09, 1.358112E+09,
    1.480745E+09, 1.613551E+09, 1.757301E+09, 1.912819E+09, 2.080987E+09, 2.262745E+09,
    2.459099E+09, 2.671121E+09, 2.899954E+09, 3.146818E+09, 3.413014E+09, 3.699928E+09,
    4.009035E+09, 4.341907E+09, 4.700218E+09, 5.085748E+09, 5.500390E+09, 5.946158E+09,
    6.425194E+09, 6.939770E+09, 7.492304E+09, 8.085362E+09, 8.721666E+09, 9.404108E+09,
    1.013575E+10, 1.091986E+10, 1.175986E+10, 1.265943E+10, 1.362243E+10, 1.465296E+10,
    1.575536E+10, 1.693424E+10, 1.819446E+10, 1.954117E+10, 2.097981E+10, 2.251614E+10,
    2.415625E+10, 2.590657E+10, 2.777389E+10, 2.976540E+10, 3.188867E+10, 3.415170E+10,
    3.656294E+10, 3.913129E+10, 4.186616E+10, 4.477745E+10, 4.787562E+10, 5.117167E+10,
    5.467720E+10, 5.840445E+10, 6.236629E+10, 6.657628E+10, 7.104869E+10, 7.579854E+10,
    8.084163E+10, 8.619460E+10, 9.187493E+10, 9.790101E+10, 1.042922E+11, 1.110687E+11,
    1.182520E+11, 1.258645E+11, 1.339296E+11, 1.424723E+11, 1.515183E+11, 1.610951E+11,
    1.712311E+11, 1.819564E+11, 1.933025E+11, 2.053023E+11, 2.179905E+11, 2.314034E+11,
    2.455790E+11, 2.605572E+11, 2.763796E+11, 2.930901E+11, 3.107345E+11, 3.293607E+11,
    3.490188E+11, 3.697615E+11,
])

#  ---------------------- M = 44, I = 6 ---------------------------
M = 44
I = 6
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    4.755283E+01, 8.917192E+02, 1.782495E+03, 2.702949E+03, 3.721375E+03, 4.909994E+03,
    6.337485E+03, 8.078305E+03, 1.021922E+04, 1.286259E+04, 1.613004E+04, 2.016576E+04,
    2.514031E+04, 3.125496E+04, 3.874699E+04, 4.789457E+04, 5.902344E+04, 7.251376E+04,
    8.880723E+04, 1.084165E+05, 1.319339E+05, 1.600430E+05, 1.935301E+05, 2.332972E+05,
    2.803772E+05, 3.359502E+05, 4.013610E+05, 4.781392E+05, 5.680217E+05, 6.729762E+05,
    7.952284E+05, 9.372903E+05, 1.101993E+06, 1.292522E+06, 1.512454E+06, 1.765801E+06,
    2.057055E+06, 2.391235E+06, 2.773948E+06, 3.211440E+06, 3.710667E+06, 4.279359E+06,
    4.926097E+06, 5.660397E+06, 6.492793E+06, 7.434939E+06, 8.499703E+06, 9.701287E+06,
    1.105534E+07, 1.257909E+07, 1.429147E+07, 1.621329E+07, 1.836739E+07, 2.077878E+07,
    2.347487E+07, 2.648563E+07, 2.984384E+07, 3.358525E+07, 3.774889E+07, 4.237729E+07,
    4.751677E+07, 5.321771E+07, 5.953488E+07, 6.652777E+07, 7.426095E+07, 8.280442E+07,
    9.223405E+07, 1.026320E+08, 1.140870E+08, 1.266953E+08, 1.405605E+08, 1.557947E+08,
    1.725188E+08, 1.908630E+08, 2.109676E+08, 2.329838E+08, 2.570740E+08, 2.834131E+08,
    3.121888E+08, 3.436027E+08, 3.778713E+08, 4.152266E+08, 4.559173E+08, 5.002100E+08,
    5.483899E+08, 6.007624E+08, 6.576539E+08, 7.194134E+08, 7.864137E+08, 8.590528E+08,
    9.377556E+08, 1.022975E+09, 1.115194E+09, 1.214928E+09, 1.322724E+09, 1.439167E+09,
    1.564877E+09, 1.700514E+09, 1.846781E+09, 2.004423E+09, 2.174233E+09, 2.357051E+09,
    2.553770E+09, 2.765338E+09, 2.992756E+09, 3.237090E+09, 3.499466E+09, 3.781077E+09,
    4.083186E+09, 4.407130E+09, 4.754323E+09, 5.126259E+09, 5.524518E+09, 5.950769E+09,
    6.406775E+09, 6.894396E+09, 7.415596E+09, 7.972447E+09, 8.567134E+09, 9.201960E+09,
    9.879354E+09, 1.060187E+10, 1.137221E+10, 1.219320E+10, 1.306783E+10, 1.399924E+10,
    1.499073E+10, 1.604578E+10, 1.716804E+10, 1.836135E+10, 1.962973E+10, 2.097741E+10,
    2.240886E+10, 2.392872E+10, 2.554190E+10, 2.725353E+10, 2.906899E+10, 3.099394E+10,
    3.303430E+10, 3.519625E+10, 3.748631E+10, 3.991127E+10, 4.247826E+10, 4.519474E+10,
    4.806852E+10, 5.110775E+10, 5.432098E+10, 5.771714E+10, 6.130557E+10, 6.509603E+10,
    6.909872E+10, 7.332428E+10, 7.778385E+10, 8.248906E+10, 8.745202E+10, 9.268540E+10,
    9.820242E+10, 1.040169E+11, 1.101431E+11, 1.165961E+11, 1.233916E+11, 1.305458E+11,
    1.380757E+11, 1.459989E+11, 1.543341E+11, 1.631002E+11, 1.723174E+11, 1.820064E+11,
    1.921888E+11, 2.028874E+11, 2.141254E+11, 2.259273E+11, 2.383185E+11, 2.513254E+11,
    2.649754E+11, 2.792970E+11,
])

#  ---------------------- M = 45, I = 1 ---------------------------
M = 45
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.000000E+00, 1.001788E+00, 1.126881E+00, 1.526206E+00, 2.077149E+00, 2.667730E+00,
    3.249956E+00, 3.809254E+00, 4.344827E+00, 4.860660E+00, 5.361676E+00, 5.852264E+00,
    6.335855E+00, 6.814950E+00, 7.291287E+00, 7.766034E+00, 8.239959E+00, 8.713552E+00,
    9.187121E+00, 9.660856E+00, 1.013487E+01, 1.060924E+01, 1.108399E+01, 1.155914E+01,
    1.203472E+01, 1.251071E+01, 1.298713E+01, 1.346397E+01, 1.394124E+01, 1.441895E+01,
    1.489709E+01, 1.537567E+01, 1.585471E+01, 1.633422E+01, 1.681420E+01, 1.729468E+01,
    1.777567E+01, 1.825719E+01, 1.873927E+01, 1.922193E+01, 1.970520E+01, 2.018910E+01,
    2.067367E+01, 2.115894E+01, 2.164494E+01, 2.213171E+01, 2.261929E+01, 2.310771E+01,
    2.359701E+01, 2.408724E+01, 2.457844E+01, 2.507064E+01, 2.556390E+01, 2.605825E+01,
    2.655374E+01, 2.705042E+01, 2.754832E+01, 2.804751E+01, 2.854802E+01, 2.904991E+01,
    2.955321E+01, 3.005797E+01, 3.056425E+01, 3.107208E+01, 3.158152E+01, 3.209261E+01,
    3.260540E+01, 3.311994E+01, 3.363627E+01, 3.415443E+01, 3.467448E+01, 3.519645E+01,
    3.572040E+01, 3.624636E+01, 3.677439E+01, 3.730452E+01, 3.783680E+01, 3.837127E+01,
    3.890798E+01, 3.944696E+01, 3.998826E+01, 4.053192E+01, 4.107798E+01, 4.162648E+01,
    4.217746E+01, 4.273095E+01, 4.328701E+01, 4.384566E+01, 4.440694E+01, 4.497089E+01,
    4.553755E+01, 4.610696E+01, 4.667914E+01, 4.725414E+01, 4.783199E+01, 4.841272E+01,
    4.899637E+01, 4.958297E+01, 5.017256E+01, 5.076516E+01, 5.136081E+01, 5.195955E+01,
    5.256139E+01, 5.316638E+01, 5.377454E+01, 5.438591E+01, 5.500051E+01, 5.561837E+01,
    5.623952E+01, 5.686400E+01, 5.749182E+01, 5.812302E+01, 5.875762E+01, 5.939565E+01,
    6.003713E+01, 6.068210E+01, 6.133059E+01, 6.198260E+01, 6.263818E+01, 6.329734E+01,
    6.396011E+01, 6.462652E+01, 6.529658E+01, 6.597033E+01, 6.664778E+01, 6.732897E+01,
    6.801391E+01, 6.870262E+01, 6.939513E+01, 7.009147E+01, 7.079164E+01, 7.149568E+01,
    7.220361E+01, 7.291544E+01, 7.363121E+01, 7.435092E+01, 7.507461E+01, 7.580228E+01,
    7.653397E+01, 7.726969E+01, 7.800946E+01, 7.875330E+01, 7.950123E+01, 8.025328E+01,
    8.100945E+01, 8.176978E+01, 8.253427E+01, 8.330295E+01, 8.407583E+01, 8.485294E+01,
    8.563429E+01, 8.641990E+01, 8.720979E+01, 8.800398E+01, 8.880249E+01, 8.960532E+01,
    9.041251E+01, 9.122406E+01, 9.204000E+01, 9.286034E+01, 9.368510E+01, 9.451430E+01,
    9.534795E+01, 9.618607E+01, 9.702868E+01, 9.787579E+01, 9.872743E+01, 9.958360E+01,
    1.004443E+02, 1.013096E+02, 1.021795E+02, 1.030540E+02, 1.039331E+02, 1.048168E+02,
    1.057052E+02, 1.065983E+02, 1.074960E+02, 1.083984E+02, 1.093056E+02, 1.102175E+02,
    1.111341E+02, 1.120555E+02, 1.129816E+02, 1.139126E+02, 1.148484E+02, 1.157890E+02,
    1.167344E+02, 1.176847E+02, 1.186398E+02, 1.195998E+02, 1.205648E+02, 1.215346E+02,
    1.225094E+02, 1.234891E+02, 1.244738E+02, 1.254635E+02, 1.264581E+02, 1.274578E+02,
    1.284624E+02, 1.294721E+02, 1.304869E+02, 1.315067E+02, 1.325316E+02, 1.335616E+02,
    1.345967E+02, 1.356369E+02, 1.366822E+02, 1.377327E+02, 1.387884E+02, 1.398492E+02,
    1.409153E+02, 1.419865E+02, 1.430630E+02, 1.441447E+02, 1.452317E+02, 1.463239E+02,
    1.474214E+02, 1.485242E+02, 1.496323E+02, 1.507458E+02, 1.518645E+02, 1.529887E+02,
    1.541181E+02, 1.552530E+02, 1.563932E+02, 1.575389E+02, 1.586900E+02, 1.598465E+02,
    1.610084E+02, 1.621758E+02, 1.633487E+02, 1.645271E+02, 1.657110E+02, 1.669004E+02,
    1.680953E+02, 1.692957E+02, 1.705017E+02, 1.717133E+02, 1.729305E+02, 1.741532E+02,
    1.753815E+02, 1.766155E+02, 1.778551E+02, 1.791004E+02, 1.803513E+02, 1.816078E+02,
    1.828701E+02, 1.841380E+02, 1.854117E+02, 1.866910E+02, 1.879761E+02, 1.892670E+02,
    1.905636E+02, 1.918659E+02, 1.931741E+02, 1.944880E+02, 1.958078E+02, 1.971333E+02,
    1.984647E+02, 1.998019E+02, 2.011450E+02, 2.024939E+02, 2.038487E+02, 2.052094E+02,
    2.065760E+02, 2.079485E+02, 2.093269E+02, 2.107112E+02, 2.121014E+02, 2.134977E+02,
    2.148998E+02, 2.163080E+02, 2.177221E+02, 2.191422E+02, 2.205683E+02, 2.220004E+02,
    2.234386E+02, 2.248827E+02, 2.263329E+02, 2.277892E+02, 2.292515E+02, 2.307199E+02,
    2.321943E+02, 2.336749E+02, 2.351615E+02, 2.366543E+02, 2.381531E+02, 2.396581E+02,
    2.411692E+02, 2.426864E+02, 2.442098E+02, 2.457394E+02, 2.472751E+02, 2.488169E+02,
    2.503650E+02, 2.519192E+02, 2.534797E+02, 2.550463E+02, 2.566191E+02, 2.581982E+02,
    2.597835E+02,
])

#  ---------------------- M = 45, I = 2 ---------------------------
M = 45
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[1]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.000000E+00, 6.029347E+00, 6.728818E+00, 8.168283E+00, 9.865998E+00, 1.164890E+01,
    1.346792E+01, 1.530608E+01, 1.715597E+01, 1.901383E+01, 2.087753E+01, 2.274575E+01,
    2.461767E+01, 2.649271E+01, 2.837047E+01, 3.025068E+01, 3.213312E+01, 3.401765E+01,
    3.590412E+01, 3.779247E+01, 3.968262E+01, 4.157454E+01, 4.346818E+01, 4.536356E+01,
    4.726067E+01, 4.915955E+01, 5.106023E+01, 5.296280E+01, 5.486733E+01, 5.677392E+01,
    5.868270E+01, 6.059382E+01, 6.250744E+01, 6.442372E+01, 6.634288E+01, 6.826512E+01,
    7.019067E+01, 7.211977E+01, 7.405266E+01, 7.598962E+01, 7.793091E+01, 7.987682E+01,
    8.182762E+01, 8.378361E+01, 8.574509E+01, 8.771234E+01, 8.968568E+01, 9.166539E+01,
    9.365178E+01, 9.564515E+01, 9.764579E+01, 9.965399E+01, 1.016701E+02, 1.036943E+02,
    1.057269E+02, 1.077682E+02, 1.098186E+02, 1.118781E+02, 1.139472E+02, 1.160261E+02,
    1.181150E+02, 1.202141E+02, 1.223238E+02, 1.244442E+02, 1.265756E+02, 1.287182E+02,
    1.308723E+02, 1.330380E+02, 1.352155E+02, 1.374051E+02, 1.396070E+02, 1.418213E+02,
    1.440482E+02, 1.462879E+02, 1.485407E+02, 1.508066E+02, 1.530859E+02, 1.553786E+02,
    1.576850E+02, 1.600053E+02, 1.623395E+02, 1.646878E+02, 1.670504E+02, 1.694274E+02,
    1.718190E+02, 1.742252E+02, 1.766463E+02, 1.790823E+02, 1.815333E+02, 1.839996E+02,
    1.864811E+02, 1.889781E+02, 1.914906E+02, 1.940188E+02, 1.965627E+02, 1.991225E+02,
    2.016983E+02, 2.042901E+02, 2.068981E+02, 2.095224E+02, 2.121631E+02, 2.148202E+02,
    2.174939E+02, 2.201843E+02, 2.228913E+02, 2.256153E+02, 2.283561E+02, 2.311140E+02,
    2.338889E+02, 2.366811E+02, 2.394905E+02, 2.423172E+02, 2.451614E+02, 2.480230E+02,
    2.509023E+02, 2.537992E+02, 2.567138E+02, 2.596462E+02, 2.625966E+02, 2.655649E+02,
    2.685512E+02, 2.715556E+02, 2.745782E+02, 2.776191E+02, 2.806782E+02, 2.837558E+02,
    2.868518E+02, 2.899664E+02, 2.930995E+02, 2.962513E+02, 2.994219E+02, 3.026112E+02,
    3.058195E+02, 3.090466E+02, 3.122928E+02, 3.155580E+02, 3.188423E+02, 3.221459E+02,
    3.254687E+02, 3.288109E+02, 3.321724E+02, 3.355534E+02, 3.389539E+02, 3.423740E+02,
    3.458138E+02, 3.492732E+02, 3.527525E+02, 3.562516E+02, 3.597706E+02, 3.633095E+02,
    3.668685E+02, 3.704476E+02, 3.740468E+02, 3.776663E+02, 3.813061E+02, 3.849662E+02,
    3.886467E+02, 3.923476E+02, 3.960692E+02, 3.998113E+02, 4.035740E+02, 4.073576E+02,
    4.111618E+02, 4.149870E+02, 4.188330E+02, 4.227001E+02, 4.265882E+02, 4.304974E+02,
    4.344277E+02, 4.383793E+02, 4.423522E+02, 4.463464E+02, 4.503621E+02, 4.543992E+02,
    4.584579E+02, 4.625383E+02, 4.666403E+02, 4.707640E+02, 4.749095E+02, 4.790770E+02,
    4.832663E+02, 4.874777E+02, 4.917111E+02, 4.959667E+02, 5.002444E+02, 5.045444E+02,
    5.088667E+02, 5.132115E+02, 5.175787E+02, 5.219684E+02, 5.263806E+02, 5.308156E+02,
    5.352732E+02, 5.397537E+02, 5.442570E+02, 5.487832E+02, 5.533323E+02, 5.579046E+02,
    5.624999E+02, 5.671184E+02, 5.717602E+02, 5.764253E+02, 5.811137E+02, 5.858256E+02,
    5.905610E+02, 5.953200E+02, 6.001026E+02, 6.049090E+02, 6.097391E+02, 6.145930E+02,
    6.194709E+02, 6.243727E+02, 6.292986E+02, 6.342486E+02, 6.392228E+02, 6.442212E+02,
    6.492439E+02, 6.542909E+02, 6.593625E+02, 6.644585E+02, 6.695791E+02, 6.747243E+02,
    6.798943E+02, 6.850890E+02, 6.903085E+02, 6.955530E+02, 7.008224E+02, 7.061168E+02,
    7.114364E+02, 7.167811E+02, 7.221510E+02, 7.275463E+02, 7.329669E+02, 7.384129E+02,
    7.438844E+02, 7.493815E+02, 7.549042E+02, 7.604525E+02, 7.660266E+02, 7.716266E+02,
    7.772523E+02, 7.829041E+02, 7.885818E+02, 7.942856E+02, 8.000155E+02, 8.057716E+02,
    8.115539E+02, 8.173626E+02, 8.231976E+02, 8.290591E+02, 8.349470E+02, 8.408615E+02,
    8.468026E+02, 8.527704E+02, 8.587650E+02, 8.647863E+02, 8.708345E+02, 8.769095E+02,
    8.830116E+02, 8.891406E+02, 8.952968E+02, 9.014801E+02, 9.076906E+02, 9.139283E+02,
    9.201934E+02, 9.264858E+02, 9.328056E+02, 9.391529E+02, 9.455278E+02, 9.519302E+02,
    9.583602E+02, 9.648179E+02, 9.713034E+02, 9.778167E+02, 9.843578E+02, 9.909268E+02,
    9.975237E+02, 1.004149E+03, 1.010802E+03, 1.017483E+03, 1.024192E+03, 1.030929E+03,
    1.037695E+03, 1.044489E+03, 1.051311E+03, 1.058162E+03, 1.065041E+03, 1.071948E+03,
    1.078884E+03, 1.085849E+03, 1.092842E+03, 1.099863E+03, 1.106914E+03, 1.113993E+03,
    1.121101E+03, 1.128237E+03, 1.135403E+03, 1.142597E+03, 1.149821E+03, 1.157073E+03,
    1.164354E+03,
])

#  ---------------------- M = 46, I = 1 ---------------------------
M = 46
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.290560E+00, 1.735586E+01, 3.438114E+01, 5.141645E+01, 6.846079E+01, 8.551397E+01,
    1.025760E+02, 1.196469E+02, 1.367275E+02, 1.538201E+02, 1.709292E+02, 1.880624E+02,
    2.052309E+02, 2.224487E+02, 2.397327E+02, 2.571016E+02, 2.745752E+02, 2.921739E+02,
    3.099177E+02, 3.278264E+02, 3.459187E+02, 3.642124E+02, 3.827237E+02, 4.014681E+02,
    4.204596E+02, 4.397109E+02, 4.592337E+02, 4.790386E+02, 4.991353E+02, 5.195323E+02,
    5.402377E+02, 5.612585E+02, 5.826011E+02, 6.042715E+02, 6.262748E+02, 6.486160E+02,
    6.712993E+02, 6.943287E+02, 7.177078E+02, 7.414400E+02, 7.655282E+02, 7.899751E+02,
    8.147835E+02, 8.399554E+02, 8.654931E+02, 8.913986E+02, 9.176737E+02, 9.443201E+02,
    9.713393E+02, 9.987328E+02, 1.026502E+03, 1.054648E+03, 1.083173E+03, 1.112076E+03,
    1.141360E+03, 1.171026E+03, 1.201073E+03, 1.231504E+03, 1.262319E+03, 1.293518E+03,
    1.325104E+03, 1.357075E+03, 1.389434E+03, 1.422180E+03, 1.455315E+03, 1.488838E+03,
    1.522751E+03, 1.557055E+03, 1.591749E+03, 1.626834E+03, 1.662311E+03, 1.698180E+03,
    1.734442E+03, 1.771097E+03, 1.808146E+03, 1.845590E+03, 1.883427E+03, 1.921660E+03,
    1.960288E+03, 1.999313E+03, 2.038733E+03, 2.078551E+03, 2.118765E+03, 2.159377E+03,
    2.200387E+03, 2.241795E+03, 2.283602E+03, 2.325808E+03, 2.368413E+03, 2.411418E+03,
    2.454824E+03, 2.498630E+03, 2.542836E+03, 2.587444E+03, 2.632454E+03, 2.677865E+03,
    2.723679E+03, 2.769895E+03, 2.816514E+03, 2.863536E+03, 2.910962E+03, 2.958792E+03,
    3.007026E+03, 3.055665E+03, 3.104709E+03, 3.154158E+03, 3.204012E+03, 3.254272E+03,
    3.304939E+03, 3.356012E+03, 3.407492E+03, 3.459379E+03, 3.511673E+03, 3.564376E+03,
    3.617486E+03, 3.671005E+03, 3.724933E+03, 3.779269E+03, 3.834015E+03, 3.889171E+03,
    3.944736E+03, 4.000712E+03, 4.057099E+03, 4.113896E+03, 4.171105E+03, 4.228725E+03,
    4.286756E+03, 4.345200E+03, 4.404057E+03, 4.463326E+03, 4.523008E+03, 4.583104E+03,
    4.643613E+03, 4.704536E+03, 4.765873E+03, 4.827625E+03, 4.889792E+03, 4.952374E+03,
    5.015371E+03, 5.078784E+03, 5.142613E+03, 5.206859E+03, 5.271521E+03, 5.336600E+03,
    5.402097E+03, 5.468011E+03, 5.534343E+03, 5.601093E+03, 5.668261E+03, 5.735848E+03,
    5.803855E+03, 5.872280E+03, 5.941126E+03, 6.010391E+03, 6.080076E+03, 6.150183E+03,
    6.220709E+03, 6.291658E+03, 6.363027E+03, 6.434819E+03, 6.507032E+03, 6.579668E+03,
    6.652726E+03, 6.726207E+03, 6.800112E+03, 6.874440E+03, 6.949192E+03, 7.024368E+03,
    7.099969E+03, 7.175994E+03, 7.252444E+03, 7.329320E+03, 7.406621E+03, 7.484349E+03,
    7.562502E+03, 7.641082E+03,
])

#  ---------------------- M = 46, I = 2 ---------------------------
M = 46
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.301900E+00, 1.763255E+01, 3.493474E+01, 5.224710E+01, 6.956862E+01, 8.689912E+01,
    1.042386E+02, 1.215871E+02, 1.389456E+02, 1.563165E+02, 1.737047E+02, 1.911184E+02,
    2.085692E+02, 2.260721E+02, 2.436447E+02, 2.613063E+02, 2.790777E+02, 2.969798E+02,
    3.150333E+02, 3.332584E+02, 3.516744E+02, 3.702993E+02, 3.891500E+02, 4.082419E+02,
    4.275893E+02, 4.472052E+02, 4.671014E+02, 4.872886E+02, 5.077767E+02, 5.285742E+02,
    5.496893E+02, 5.711290E+02, 5.928998E+02, 6.150077E+02, 6.374580E+02, 6.602555E+02,
    6.834045E+02, 7.069091E+02, 7.307729E+02, 7.549992E+02, 7.795910E+02, 8.045510E+02,
    8.298819E+02, 8.555859E+02, 8.816653E+02, 9.081219E+02, 9.349576E+02, 9.621742E+02,
    9.897732E+02, 1.017756E+03, 1.046124E+03, 1.074879E+03, 1.104021E+03, 1.133552E+03,
    1.163473E+03, 1.193785E+03, 1.224489E+03, 1.255586E+03, 1.287076E+03, 1.318961E+03,
    1.351240E+03, 1.383916E+03, 1.416989E+03, 1.450458E+03, 1.484326E+03, 1.518592E+03,
    1.553258E+03, 1.588323E+03, 1.623789E+03, 1.659656E+03, 1.695923E+03, 1.732593E+03,
    1.769666E+03, 1.807141E+03, 1.845020E+03, 1.883302E+03, 1.921989E+03, 1.961081E+03,
    2.000577E+03, 2.040480E+03, 2.080788E+03, 2.121503E+03, 2.162625E+03, 2.204154E+03,
    2.246091E+03, 2.288436E+03, 2.331189E+03, 2.374352E+03, 2.417923E+03, 2.461904E+03,
    2.506295E+03, 2.551096E+03, 2.596308E+03, 2.641930E+03, 2.687965E+03, 2.734411E+03,
    2.781269E+03, 2.828539E+03, 2.876222E+03, 2.924319E+03, 2.972829E+03, 3.021752E+03,
    3.071090E+03, 3.120842E+03, 3.171009E+03, 3.221591E+03, 3.272589E+03, 3.324002E+03,
    3.375832E+03, 3.428078E+03, 3.480740E+03, 3.533820E+03, 3.587317E+03, 3.641232E+03,
    3.695565E+03, 3.750317E+03, 3.805487E+03, 3.861076E+03, 3.917084E+03, 3.973512E+03,
    4.030360E+03, 4.087629E+03, 4.145318E+03, 4.203427E+03, 4.261959E+03, 4.320911E+03,
    4.380286E+03, 4.440082E+03, 4.500301E+03, 4.560943E+03, 4.622008E+03, 4.683497E+03,
    4.745409E+03, 4.807745E+03, 4.870506E+03, 4.933691E+03, 4.997301E+03, 5.061336E+03,
    5.125797E+03, 5.190684E+03, 5.255997E+03, 5.321737E+03, 5.387903E+03, 5.454497E+03,
    5.521518E+03, 5.588967E+03, 5.656843E+03, 5.725148E+03, 5.793882E+03, 5.863045E+03,
    5.932637E+03, 6.002659E+03, 6.073110E+03, 6.143992E+03, 6.215304E+03, 6.287047E+03,
    6.359221E+03, 6.431827E+03, 6.504864E+03, 6.578333E+03, 6.652235E+03, 6.726569E+03,
    6.801336E+03, 6.876537E+03, 6.952171E+03, 7.028239E+03, 7.104741E+03, 7.181677E+03,
    7.259049E+03, 7.336855E+03, 7.415097E+03, 7.493774E+03, 7.572888E+03, 7.652438E+03,
    7.732424E+03, 7.812847E+03,
])

#  ---------------------- M = 46, I = 3 ---------------------------
M = 46
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.665010E+00, 3.673026E+01, 7.280106E+01, 1.088928E+02, 1.450036E+02, 1.811332E+02,
    2.172814E+02, 2.534487E+02, 2.896373E+02, 3.258533E+02, 3.621084E+02, 3.984219E+02,
    4.348208E+02, 4.713391E+02, 5.080160E+02, 5.448948E+02, 5.820205E+02, 6.194387E+02,
    6.571942E+02, 6.953301E+02, 7.338872E+02, 7.729036E+02, 8.124146E+02, 8.524529E+02,
    8.930479E+02, 9.342266E+02, 9.760135E+02, 1.018431E+03, 1.061498E+03, 1.105234E+03,
    1.149654E+03, 1.194773E+03, 1.240604E+03, 1.287160E+03, 1.334451E+03, 1.382487E+03,
    1.431277E+03, 1.480828E+03, 1.531149E+03, 1.582246E+03, 1.634124E+03, 1.686790E+03,
    1.740249E+03, 1.794505E+03, 1.849563E+03, 1.905427E+03, 1.962100E+03, 2.019586E+03,
    2.077888E+03, 2.137009E+03, 2.196951E+03, 2.257718E+03, 2.319312E+03, 2.381735E+03,
    2.444990E+03, 2.509077E+03, 2.574000E+03, 2.639760E+03, 2.706358E+03, 2.773797E+03,
    2.842078E+03, 2.911202E+03, 2.981172E+03, 3.051987E+03, 3.123651E+03, 3.196163E+03,
    3.269525E+03, 3.343739E+03, 3.418805E+03, 3.494725E+03, 3.571500E+03, 3.649130E+03,
    3.727618E+03, 3.806963E+03, 3.887167E+03, 3.968231E+03, 4.050155E+03, 4.132942E+03,
    4.216590E+03, 4.301103E+03, 4.386479E+03, 4.472721E+03, 4.559829E+03, 4.647804E+03,
    4.736646E+03, 4.826357E+03, 4.916938E+03, 5.008388E+03, 5.100710E+03, 5.193903E+03,
    5.287969E+03, 5.382908E+03, 5.478720E+03, 5.575408E+03, 5.672971E+03, 5.771411E+03,
    5.870727E+03, 5.970921E+03, 6.071993E+03, 6.173945E+03, 6.276777E+03, 6.380489E+03,
    6.485082E+03, 6.590557E+03, 6.696916E+03, 6.804157E+03, 6.912283E+03, 7.021293E+03,
    7.131189E+03, 7.241971E+03, 7.353640E+03, 7.466197E+03, 7.579641E+03, 7.693975E+03,
    7.809198E+03, 7.925312E+03, 8.042316E+03, 8.160212E+03, 8.279001E+03, 8.398682E+03,
    8.519257E+03, 8.640727E+03, 8.763091E+03, 8.886351E+03, 9.010508E+03, 9.135561E+03,
    9.261512E+03, 9.388361E+03, 9.516109E+03, 9.644757E+03, 9.774306E+03, 9.904755E+03,
    1.003611E+04, 1.016836E+04, 1.030151E+04, 1.043557E+04, 1.057054E+04, 1.070641E+04,
    1.084318E+04, 1.098086E+04, 1.111945E+04, 1.125894E+04, 1.139935E+04, 1.154066E+04,
    1.168288E+04, 1.182601E+04, 1.197005E+04, 1.211501E+04, 1.226087E+04, 1.240765E+04,
    1.255535E+04, 1.270395E+04, 1.285347E+04, 1.300391E+04, 1.315526E+04, 1.330753E+04,
    1.346072E+04, 1.361482E+04, 1.376984E+04, 1.392579E+04, 1.408265E+04, 1.424043E+04,
    1.439914E+04, 1.455876E+04, 1.471931E+04, 1.488078E+04, 1.504318E+04, 1.520650E+04,
    1.537074E+04, 1.553591E+04, 1.570201E+04, 1.586903E+04, 1.603699E+04, 1.620587E+04,
    1.637567E+04, 1.654641E+04,
])

#  ---------------------- M = 46, I = 4 ---------------------------
M = 46
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    5.185410E+00, 6.999021E+01, 1.386585E+02, 2.073673E+02, 2.761123E+02, 3.448931E+02,
    4.137093E+02, 4.825616E+02, 5.514533E+02, 6.203936E+02, 6.894014E+02, 7.585087E+02,
    8.277606E+02, 8.972156E+02, 9.669423E+02, 1.037017E+03, 1.107521E+03, 1.178536E+03,
    1.250145E+03, 1.322428E+03, 1.395459E+03, 1.469311E+03, 1.544051E+03, 1.619739E+03,
    1.696433E+03, 1.774184E+03, 1.853038E+03, 1.933040E+03, 2.014227E+03, 2.096634E+03,
    2.180293E+03, 2.265233E+03, 2.351479E+03, 2.439054E+03, 2.527982E+03, 2.618279E+03,
    2.709964E+03, 2.803053E+03, 2.897560E+03, 2.993499E+03, 3.090881E+03, 3.189718E+03,
    3.290019E+03, 3.391794E+03, 3.495052E+03, 3.599800E+03, 3.706046E+03, 3.813797E+03,
    3.923059E+03, 4.033837E+03, 4.146138E+03, 4.259966E+03, 4.375326E+03, 4.492223E+03,
    4.610661E+03, 4.730643E+03, 4.852175E+03, 4.975258E+03, 5.099898E+03, 5.226096E+03,
    5.353856E+03, 5.483181E+03, 5.614074E+03, 5.746537E+03, 5.880573E+03, 6.016185E+03,
    6.153375E+03, 6.292145E+03, 6.432497E+03, 6.574434E+03, 6.717958E+03, 6.863070E+03,
    7.009774E+03, 7.158070E+03, 7.307961E+03, 7.459448E+03, 7.612533E+03, 7.767219E+03,
    7.923506E+03, 8.081397E+03, 8.240893E+03, 8.401995E+03, 8.564707E+03, 8.729028E+03,
    8.894960E+03, 9.062506E+03, 9.231667E+03, 9.402443E+03, 9.574837E+03, 9.748851E+03,
    9.924485E+03, 1.010174E+04, 1.028062E+04, 1.046112E+04, 1.064325E+04, 1.082701E+04,
    1.101240E+04, 1.119942E+04, 1.138807E+04, 1.157835E+04, 1.177027E+04, 1.196382E+04,
    1.215901E+04, 1.235584E+04, 1.255431E+04, 1.275442E+04, 1.295617E+04, 1.315956E+04,
    1.336460E+04, 1.357129E+04, 1.377962E+04, 1.398961E+04, 1.420124E+04, 1.441453E+04,
    1.462946E+04, 1.484606E+04, 1.506430E+04, 1.528421E+04, 1.550577E+04, 1.572899E+04,
    1.595387E+04, 1.618041E+04, 1.640861E+04, 1.663848E+04, 1.687002E+04, 1.710322E+04,
    1.733808E+04, 1.757462E+04, 1.781283E+04, 1.805271E+04, 1.829426E+04, 1.853748E+04,
    1.878238E+04, 1.902896E+04, 1.927721E+04, 1.952715E+04, 1.977876E+04, 2.003205E+04,
    2.028703E+04, 2.054369E+04, 2.080203E+04, 2.106207E+04, 2.132378E+04, 2.158719E+04,
    2.185229E+04, 2.211908E+04, 2.238756E+04, 2.265773E+04, 2.292960E+04, 2.320316E+04,
    2.347842E+04, 2.375538E+04, 2.403404E+04, 2.431440E+04, 2.459646E+04, 2.488023E+04,
    2.516569E+04, 2.545287E+04, 2.574175E+04, 2.603234E+04, 2.632464E+04, 2.661864E+04,
    2.691436E+04, 2.721180E+04, 2.751094E+04, 2.781181E+04, 2.811438E+04, 2.841868E+04,
    2.872470E+04, 2.903243E+04, 2.934189E+04, 2.965306E+04, 2.996597E+04, 3.028059E+04,
    3.059695E+04, 3.091503E+04,
])

#  ---------------------- M = 47, I = 1 ---------------------------
M = 47
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.414270E+00, 1.057361E+02, 2.984678E+02, 5.479928E+02, 8.436907E+02, 1.180696E+03,
    1.558672E+03, 1.981177E+03, 2.454584E+03, 2.987094E+03, 3.588121E+03, 4.268040E+03,
    5.038147E+03, 5.910737E+03, 6.899230E+03, 8.018317E+03, 9.284110E+03, 1.071430E+04,
    1.232829E+04, 1.414738E+04, 1.619491E+04, 1.849642E+04, 2.107982E+04, 2.397557E+04,
    2.721685E+04, 3.083979E+04, 3.488362E+04, 3.939089E+04, 4.440770E+04, 4.998392E+04,
    5.617341E+04, 6.303426E+04, 7.062902E+04, 7.884530E+04, 8.807839E+04, 9.825772E+04,
    1.094663E+05, 1.217930E+05, 1.353330E+05, 1.501882E+05, 1.664674E+05, 1.842866E+05,
    2.037697E+05, 2.250484E+05, 2.482633E+05, 2.735634E+05, 3.011073E+05, 3.310633E+05,
    3.636098E+05, 3.989359E+05, 4.372419E+05, 4.787394E+05, 5.236525E+05, 5.722175E+05,
    6.246841E+05, 6.813156E+05, 7.423894E+05, 8.081977E+05, 8.790482E+05, 9.552643E+05,
    1.037186E+06, 1.125171E+06, 1.219593E+06, 1.320847E+06, 1.429344E+06, 1.545517E+06,
    1.669818E+06, 1.802721E+06, 1.944721E+06, 2.096336E+06, 2.258109E+06, 2.430603E+06,
    2.614408E+06, 2.810141E+06, 3.018444E+06, 3.239985E+06, 3.475461E+06, 3.725598E+06,
    3.991152E+06, 4.272907E+06, 4.571682E+06, 4.888326E+06, 5.223722E+06, 5.578785E+06,
    5.954468E+06, 6.351757E+06, 6.771677E+06, 7.215291E+06, 7.683698E+06, 8.178039E+06,
    8.699497E+06, 9.249293E+06, 9.828694E+06, 1.043901E+07, 1.108160E+07, 1.175785E+07,
    1.246923E+07, 1.321721E+07, 1.400336E+07, 1.482927E+07, 1.569657E+07, 1.660698E+07,
    1.756225E+07, 1.856419E+07, 1.961465E+07, 2.071558E+07, 2.186893E+07, 2.307677E+07,
    2.434119E+07, 2.566435E+07, 2.704850E+07, 2.849591E+07, 3.000895E+07, 3.159005E+07,
    3.324171E+07, 3.496649E+07, 3.676703E+07, 3.864605E+07, 4.060633E+07, 4.265074E+07,
    4.478221E+07, 4.700377E+07, 4.931851E+07, 5.172961E+07, 5.424035E+07, 5.685405E+07,
    5.957418E+07, 6.240423E+07, 6.534783E+07, 6.840867E+07, 7.159056E+07, 7.489738E+07,
    7.833310E+07, 8.190183E+07, 8.560772E+07, 8.945507E+07, 9.344824E+07, 9.759174E+07,
    1.018901E+08, 1.063481E+08, 1.109706E+08, 1.157623E+08, 1.207284E+08, 1.258740E+08,
    1.312043E+08, 1.367248E+08, 1.424409E+08, 1.483582E+08, 1.544824E+08, 1.608195E+08,
    1.673754E+08, 1.741563E+08, 1.811683E+08, 1.884179E+08, 1.959115E+08, 2.036558E+08,
    2.116577E+08, 2.199239E+08, 2.284616E+08, 2.372780E+08, 2.463805E+08, 2.557765E+08,
    2.654736E+08, 2.754798E+08, 2.858029E+08, 2.964510E+08, 3.074325E+08, 3.187557E+08,
    3.304293E+08, 3.424619E+08, 3.548625E+08, 3.676402E+08, 3.808043E+08, 3.943641E+08,
    4.083293E+08, 4.227097E+08,
])

#  ---------------------- M = 48, I = 1 ---------------------------
M = 48
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.147898E+01, 3.996775E+02, 7.982281E+02, 1.204932E+03, 1.643306E+03, 2.140834E+03,
    2.722097E+03, 3.410126E+03, 4.228639E+03, 5.203092E+03, 6.361663E+03, 7.735468E+03,
    9.359297E+03, 1.127186E+04, 1.351617E+04, 1.614009E+04, 1.919639E+04, 2.274374E+04,
    2.684666E+04, 3.157632E+04, 3.701089E+04, 4.323629E+04, 5.034661E+04, 5.844492E+04,
    6.764383E+04, 7.806626E+04, 8.984612E+04, 1.031293E+05, 1.180743E+05, 1.348531E+05,
    1.536525E+05, 1.746748E+05, 1.981385E+05, 2.242802E+05, 2.533549E+05, 2.856378E+05,
    3.214253E+05, 3.610363E+05, 4.048136E+05, 4.531253E+05, 5.063665E+05, 5.649605E+05,
    6.293605E+05, 7.000516E+05, 7.775523E+05, 8.624165E+05, 9.552351E+05, 1.056639E+06,
    1.167298E+06, 1.287929E+06, 1.419291E+06, 1.562193E+06, 1.717494E+06, 1.886106E+06,
    2.068996E+06, 2.267189E+06, 2.481773E+06, 2.713896E+06, 2.964776E+06, 3.235699E+06,
    3.528026E+06, 3.843193E+06, 4.182715E+06, 4.548192E+06, 4.941312E+06, 5.363849E+06,
    5.817677E+06, 6.304765E+06, 6.827188E+06, 7.387122E+06, 7.986861E+06, 8.628813E+06,
    9.315504E+06, 1.004959E+07, 1.083385E+07, 1.167121E+07, 1.256472E+07, 1.351760E+07,
    1.453319E+07, 1.561502E+07, 1.676676E+07, 1.799226E+07, 1.929555E+07, 2.068082E+07,
    2.215247E+07, 2.371510E+07, 2.537348E+07, 2.713262E+07, 2.899773E+07, 3.097426E+07,
    3.306787E+07, 3.528449E+07, 3.763024E+07, 4.011156E+07, 4.273513E+07, 4.550790E+07,
    4.843709E+07, 5.153021E+07, 5.479511E+07, 5.823992E+07, 6.187306E+07, 6.570334E+07,
    6.973986E+07, 7.399210E+07, 7.846990E+07, 8.318344E+07, 8.814330E+07, 9.336047E+07,
    9.884634E+07, 1.046127E+08, 1.106718E+08, 1.170363E+08, 1.237193E+08, 1.307344E+08,
    1.380958E+08, 1.458179E+08, 1.539159E+08, 1.624053E+08, 1.713024E+08, 1.806237E+08,
    1.903866E+08, 2.006089E+08, 2.113091E+08, 2.225062E+08, 2.342199E+08, 2.464705E+08,
    2.592791E+08, 2.726673E+08, 2.866575E+08, 3.012728E+08, 3.165370E+08, 3.324747E+08,
    3.491113E+08, 3.664730E+08, 3.845867E+08, 4.034802E+08, 4.231821E+08, 4.437221E+08,
    4.651304E+08, 4.874384E+08, 5.106785E+08, 5.348838E+08, 5.600887E+08, 5.863282E+08,
    6.136387E+08, 6.420574E+08, 6.716228E+08, 7.023745E+08, 7.343529E+08, 7.675998E+08,
    8.021584E+08, 8.380725E+08, 8.753878E+08, 9.141511E+08, 9.544098E+08, 9.962137E+08,
    1.039613E+09, 1.084660E+09, 1.131409E+09, 1.179913E+09, 1.230230E+09, 1.282416E+09,
    1.336532E+09, 1.392638E+09, 1.450797E+09, 1.511073E+09, 1.573532E+09, 1.638241E+09,
    1.705270E+09, 1.774690E+09, 1.846572E+09, 1.920993E+09, 1.998028E+09, 2.077756E+09,
    2.160258E+09, 2.245616E+09,
])

#  ---------------------- M = 48, I = 2 ---------------------------
M = 48
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.010601E+01, 1.888698E+02, 3.772493E+02, 5.694819E+02, 7.766830E+02, 1.011842E+03,
    1.286579E+03, 1.611778E+03, 1.998651E+03, 2.459229E+03, 3.006830E+03, 3.656162E+03,
    4.423668E+03, 5.327646E+03, 6.388423E+03, 7.628619E+03, 9.073184E+03, 1.074984E+04,
    1.268909E+04, 1.492457E+04, 1.749322E+04, 2.043566E+04, 2.379635E+04, 2.762402E+04,
    3.197188E+04, 3.689804E+04, 4.246576E+04, 4.874404E+04, 5.580776E+04, 6.373828E+04,
    7.262379E+04, 8.255991E+04, 9.364995E+04, 1.060058E+05, 1.197478E+05, 1.350062E+05,
    1.519210E+05, 1.706431E+05, 1.913342E+05, 2.141685E+05, 2.393327E+05, 2.670268E+05,
    2.974652E+05, 3.308769E+05, 3.675071E+05, 4.076176E+05, 4.514877E+05, 4.994153E+05,
    5.517179E+05, 6.087329E+05, 6.708202E+05, 7.383616E+05, 8.117633E+05, 8.914563E+05,
    9.778976E+05, 1.071572E+06, 1.172993E+06, 1.282703E+06, 1.401279E+06, 1.529328E+06,
    1.667493E+06, 1.816453E+06, 1.976924E+06, 2.149662E+06, 2.335465E+06, 2.535172E+06,
    2.749668E+06, 2.979884E+06, 3.226799E+06, 3.491445E+06, 3.774903E+06, 4.078312E+06,
    4.402866E+06, 4.749820E+06, 5.120489E+06, 5.516252E+06, 5.938556E+06, 6.388917E+06,
    6.868920E+06, 7.380229E+06, 7.924578E+06, 8.503790E+06, 9.119763E+06, 9.774486E+06,
    1.047004E+07, 1.120858E+07, 1.199238E+07, 1.282380E+07, 1.370531E+07, 1.463948E+07,
    1.562898E+07, 1.667661E+07, 1.778528E+07, 1.895802E+07, 2.019800E+07, 2.150848E+07,
    2.289289E+07, 2.435479E+07, 2.589786E+07, 2.752596E+07, 2.924309E+07, 3.105337E+07,
    3.296113E+07, 3.497084E+07, 3.708715E+07, 3.931488E+07, 4.165904E+07, 4.412480E+07,
    4.671754E+07, 4.944286E+07, 5.230652E+07, 5.531451E+07, 5.847304E+07, 6.178856E+07,
    6.526767E+07, 6.891730E+07, 7.274457E+07, 7.675684E+07, 8.096177E+07, 8.536722E+07,
    8.998134E+07, 9.481260E+07, 9.986970E+07, 1.051616E+08, 1.106977E+08, 1.164876E+08,
    1.225411E+08, 1.288686E+08, 1.354806E+08, 1.423880E+08, 1.496021E+08, 1.571345E+08,
    1.649972E+08, 1.732026E+08, 1.817634E+08, 1.906927E+08, 2.000042E+08, 2.097116E+08,
    2.198294E+08, 2.303725E+08, 2.413560E+08, 2.527958E+08, 2.647078E+08, 2.771089E+08,
    2.900161E+08, 3.034471E+08, 3.174201E+08, 3.319536E+08, 3.470669E+08, 3.627796E+08,
    3.791123E+08, 3.960856E+08, 4.137211E+08, 4.320410E+08, 4.510675E+08, 4.708244E+08,
    4.913352E+08, 5.126249E+08, 5.347184E+08, 5.576418E+08, 5.814215E+08, 6.060851E+08,
    6.316606E+08, 6.581766E+08, 6.856628E+08, 7.141496E+08, 7.436678E+08, 7.742496E+08,
    8.059274E+08, 8.387350E+08, 8.727071E+08, 9.078783E+08, 9.442853E+08, 9.819650E+08,
    1.020955E+09, 1.061295E+09,
])

#  ---------------------- M = 49, I = 1 ---------------------------
M = 49
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.721481E+02, 1.484065E+04, 4.195358E+04, 7.716245E+04, 1.194582E+05, 1.691203E+05,
    2.272411E+05, 2.954664E+05, 3.758697E+05, 4.709005E+05, 5.833741E+05, 7.164874E+05,
    8.738508E+05, 1.059529E+06, 1.278091E+06, 1.534657E+06, 1.834964E+06, 2.185421E+06,
    2.593175E+06, 3.066183E+06, 3.613284E+06, 4.244280E+06, 4.970014E+06, 5.802462E+06,
    6.754827E+06, 7.841634E+06, 9.078840E+06, 1.048393E+07, 1.207606E+07, 1.387613E+07,
    1.590698E+07, 1.819346E+07, 2.076262E+07, 2.364381E+07, 2.686889E+07, 3.047235E+07,
    3.449152E+07, 3.896669E+07, 4.394135E+07, 4.946237E+07, 5.558021E+07, 6.234908E+07,
    6.982725E+07, 7.807722E+07, 8.716595E+07, 9.716517E+07, 1.081516E+08, 1.202071E+08,
    1.334193E+08, 1.478815E+08, 1.636932E+08, 1.809603E+08, 1.997955E+08, 2.203186E+08,
    2.426568E+08, 2.669453E+08, 2.933272E+08, 3.219545E+08, 3.529879E+08, 3.865976E+08,
    4.229636E+08, 4.622762E+08, 5.047363E+08, 5.505561E+08, 5.999592E+08, 6.531817E+08,
    7.104720E+08, 7.720921E+08, 8.383173E+08, 9.094375E+08, 9.857572E+08, 1.067597E+09,
    1.155292E+09, 1.249196E+09, 1.349679E+09, 1.457130E+09, 1.571954E+09, 1.694580E+09,
    1.825452E+09, 1.965038E+09, 2.113827E+09, 2.272331E+09, 2.441083E+09, 2.620643E+09,
    2.811592E+09, 3.014541E+09, 3.230126E+09, 3.459007E+09, 3.701878E+09, 3.959458E+09,
    4.232500E+09, 4.521784E+09, 4.828127E+09, 5.152375E+09, 5.495412E+09, 5.858154E+09,
    6.241558E+09, 6.646615E+09, 7.074357E+09, 7.525854E+09, 8.002221E+09, 8.504613E+09,
    9.034227E+09, 9.592310E+09, 1.018015E+10, 1.079909E+10, 1.145051E+10, 1.213586E+10,
    1.285662E+10, 1.361435E+10, 1.441063E+10, 1.524712E+10, 1.612554E+10, 1.704766E+10,
    1.801533E+10, 1.903042E+10, 2.009491E+10, 2.121083E+10, 2.238027E+10, 2.360541E+10,
    2.488848E+10, 2.623178E+10, 2.763772E+10, 2.910876E+10, 3.064744E+10, 3.225639E+10,
    3.393831E+10, 3.569601E+10, 3.753238E+10, 3.945037E+10, 4.145306E+10, 4.354361E+10,
    4.572526E+10, 4.800138E+10, 5.037543E+10, 5.285095E+10, 5.543162E+10, 5.812120E+10,
    6.092359E+10, 6.384278E+10, 6.688289E+10, 7.004814E+10, 7.334289E+10, 7.677162E+10,
    8.033894E+10, 8.404958E+10, 8.790839E+10, 9.192039E+10, 9.609072E+10, 1.004247E+11,
    1.049276E+11, 1.096052E+11, 1.144631E+11, 1.195073E+11, 1.247437E+11, 1.301785E+11,
    1.358181E+11, 1.416690E+11, 1.477380E+11, 1.540319E+11, 1.605577E+11, 1.673227E+11,
    1.743343E+11, 1.816000E+11, 1.891278E+11, 1.969255E+11, 2.050014E+11, 2.133638E+11,
    2.220214E+11, 2.309830E+11, 2.402576E+11, 2.498545E+11, 2.597830E+11, 2.700530E+11,
    2.806744E+11, 2.916573E+11, 3.030122E+11, 3.147497E+11, 3.268807E+11, 3.394164E+11,
    3.523683E+11, 3.657479E+11, 3.795672E+11, 3.938387E+11, 4.085747E+11, 4.237879E+11,
    4.394915E+11, 4.556990E+11, 4.724240E+11, 4.896804E+11, 5.074825E+11, 5.258451E+11,
    5.447830E+11, 5.643114E+11, 5.844460E+11, 6.052027E+11, 6.265977E+11, 6.486480E+11,
    6.713700E+11, 6.947814E+11, 7.188998E+11, 7.437436E+11, 7.693308E+11, 7.956805E+11,
    8.228118E+11, 8.507446E+11, 8.794988E+11, 9.090947E+11, 9.395534E+11, 9.708961E+11,
    1.003145E+12, 1.036321E+12, 1.070448E+12, 1.105549E+12, 1.141647E+12, 1.178766E+12,
    1.216931E+12, 1.256166E+12, 1.296498E+12, 1.337952E+12, 1.380554E+12, 1.424331E+12,
    1.469312E+12, 1.515524E+12, 1.562995E+12, 1.611755E+12, 1.661833E+12, 1.713259E+12,
    1.766065E+12, 1.820282E+12, 1.875941E+12, 1.933075E+12, 1.991718E+12, 2.051902E+12,
    2.113663E+12, 2.177035E+12, 2.242055E+12, 2.308758E+12, 2.377181E+12, 2.447362E+12,
    2.519339E+12, 2.593152E+12, 2.668840E+12, 2.746443E+12, 2.826002E+12, 2.907559E+12,
    2.991157E+12, 3.076840E+12, 3.164651E+12, 3.254635E+12, 3.346837E+12,
])

#  ---------------------- M = 49, I = 2 ---------------------------
M = 49
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[0]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.536788E+02, 3.051435E+04, 8.626393E+04, 1.586606E+05, 2.456296E+05, 3.477452E+05,
    4.672541E+05, 6.075406E+05, 7.728694E+05, 9.682784E+05, 1.199558E+06, 1.473286E+06,
    1.796889E+06, 2.178729E+06, 2.628202E+06, 3.155848E+06, 3.773466E+06, 4.494242E+06,
    5.332883E+06, 6.305759E+06, 7.431058E+06, 8.728947E+06, 1.022174E+07, 1.193408E+07,
    1.389313E+07, 1.612879E+07, 1.867388E+07, 2.156440E+07, 2.483975E+07, 2.854298E+07,
    3.272105E+07, 3.742513E+07, 4.271087E+07, 4.863870E+07, 5.527418E+07, 6.268830E+07,
    7.095785E+07, 8.016580E+07, 9.040167E+07, 1.017619E+08, 1.143503E+08, 1.282786E+08,
    1.436666E+08, 1.606431E+08, 1.793457E+08, 1.999222E+08, 2.225305E+08, 2.473392E+08,
    2.745284E+08, 3.042904E+08, 3.368300E+08, 3.723651E+08, 4.111276E+08, 4.533644E+08,
    4.993372E+08, 5.493242E+08, 6.036203E+08, 6.625380E+08, 7.264084E+08, 7.955819E+08,
    8.704289E+08, 9.513412E+08, 1.038732E+09, 1.133039E+09, 1.234722E+09, 1.344267E+09,
    1.462186E+09, 1.589017E+09, 1.725329E+09, 1.871716E+09, 2.028808E+09, 2.197261E+09,
    2.377770E+09, 2.571060E+09, 2.777894E+09, 2.999070E+09, 3.235427E+09, 3.487843E+09,
    3.757235E+09, 4.044568E+09, 4.350846E+09, 4.677124E+09, 5.024499E+09, 5.394123E+09,
    5.787198E+09, 6.204973E+09, 6.648762E+09, 7.119927E+09, 7.619892E+09, 8.150140E+09,
    8.712220E+09, 9.307740E+09, 9.938379E+09, 1.060588E+10, 1.131207E+10, 1.205882E+10,
    1.284811E+10, 1.368199E+10, 1.456256E+10, 1.549205E+10, 1.647274E+10, 1.750701E+10,
    1.859732E+10, 1.974625E+10, 2.095645E+10, 2.223068E+10, 2.357178E+10, 2.498274E+10,
    2.646661E+10, 2.802658E+10, 2.966593E+10, 3.138808E+10, 3.319655E+10, 3.509501E+10,
    3.708723E+10, 3.917710E+10, 4.136869E+10, 4.366616E+10, 4.607384E+10, 4.859618E+10,
    5.123780E+10, 5.400345E+10, 5.689808E+10, 5.992673E+10, 6.309465E+10, 6.640727E+10,
    6.987015E+10, 7.348904E+10, 7.726990E+10, 8.121884E+10, 8.534217E+10, 8.964640E+10,
    9.413822E+10, 9.882455E+10, 1.037125E+11, 1.088094E+11, 1.141229E+11, 1.196605E+11,
    1.254305E+11, 1.314410E+11, 1.377004E+11, 1.442175E+11, 1.510013E+11, 1.580609E+11,
    1.654059E+11, 1.730460E+11, 1.809912E+11, 1.892519E+11, 1.978386E+11, 2.067621E+11,
    2.160338E+11, 2.256649E+11, 2.356674E+11, 2.460533E+11, 2.568351E+11, 2.680255E+11,
    2.796376E+11, 2.916849E+11, 3.041811E+11, 3.171404E+11, 3.305773E+11, 3.445067E+11,
    3.589439E+11, 3.739045E+11, 3.894045E+11, 4.054605E+11, 4.220892E+11, 4.393081E+11,
    4.571347E+11, 4.755873E+11, 4.946844E+11, 5.144452E+11, 5.348889E+11, 5.560357E+11,
    5.779061E+11, 6.005211E+11, 6.239019E+11, 6.480705E+11, 6.730496E+11, 6.988620E+11,
    7.255312E+11, 7.530815E+11, 7.815372E+11, 8.109238E+11, 8.412671E+11, 8.725931E+11,
    9.049290E+11, 9.383023E+11, 9.727413E+11, 1.008275E+12, 1.044932E+12, 1.082743E+12,
    1.121739E+12, 1.161951E+12, 1.203411E+12, 1.246152E+12, 1.290208E+12, 1.335613E+12,
    1.382402E+12, 1.430610E+12, 1.480274E+12, 1.531431E+12, 1.584120E+12, 1.638379E+12,
    1.694247E+12, 1.751766E+12, 1.810976E+12, 1.871920E+12, 1.934640E+12, 1.999181E+12,
    2.065587E+12, 2.133904E+12, 2.204179E+12, 2.276458E+12, 2.350791E+12, 2.427227E+12,
    2.505817E+12, 2.586611E+12, 2.669663E+12, 2.755024E+12, 2.842752E+12, 2.932900E+12,
    3.025525E+12, 3.120685E+12, 3.218439E+12, 3.318847E+12, 3.421969E+12, 3.527869E+12,
    3.636609E+12, 3.748253E+12, 3.862869E+12, 3.980522E+12, 4.101281E+12, 4.225217E+12,
    4.352398E+12, 4.482897E+12, 4.616789E+12, 4.754148E+12, 4.895049E+12, 5.039569E+12,
    5.187790E+12, 5.339789E+12, 5.495649E+12, 5.655455E+12, 5.819289E+12, 5.987238E+12,
    6.159389E+12, 6.335833E+12, 6.516660E+12, 6.701962E+12, 6.891832E+12,
])

#  ---------------------- M = 50, I = 1 ---------------------------
M = 50
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.769600E+00, 4.269987E+01, 9.814561E+01, 1.552356E+02, 2.128150E+02, 2.706077E+02,
    3.285146E+02, 3.864951E+02, 4.445326E+02, 5.026278E+02, 5.608132E+02, 6.191261E+02,
    6.776212E+02, 7.363736E+02, 7.954647E+02, 8.549797E+02, 9.149952E+02, 9.755928E+02,
    1.036859E+03, 1.098863E+03, 1.161685E+03, 1.225362E+03, 1.289972E+03, 1.355557E+03,
    1.422167E+03, 1.489851E+03, 1.558626E+03, 1.628532E+03, 1.699607E+03, 1.771871E+03,
    1.845355E+03, 1.920070E+03, 1.996043E+03, 2.073277E+03, 2.151791E+03, 2.231603E+03,
    2.312747E+03, 2.395195E+03, 2.478975E+03, 2.564097E+03, 2.650564E+03, 2.738380E+03,
    2.827570E+03, 2.918108E+03, 3.010041E+03, 3.103340E+03, 3.198024E+03, 3.294087E+03,
    3.391545E+03, 3.490387E+03, 3.590631E+03, 3.692292E+03, 3.795325E+03, 3.899805E+03,
    4.005654E+03, 4.112914E+03, 4.221598E+03, 4.331719E+03, 4.443221E+03, 4.556148E+03,
    4.670476E+03, 4.786249E+03, 4.903443E+03, 5.022065E+03, 5.142089E+03, 5.263558E+03,
    5.386445E+03, 5.510755E+03, 5.636496E+03, 5.763636E+03, 5.892261E+03, 6.022257E+03,
    6.153753E+03, 6.286630E+03, 6.420936E+03, 6.556719E+03, 6.693898E+03, 6.832523E+03,
    6.972598E+03, 7.114083E+03, 7.257028E+03, 7.401392E+03, 7.547226E+03, 7.694488E+03,
    7.843181E+03, 7.993308E+03, 8.144875E+03, 8.297883E+03, 8.452338E+03, 8.608242E+03,
    8.765548E+03, 8.924362E+03, 9.084530E+03, 9.246215E+03, 9.409312E+03, 9.573879E+03,
    9.739863E+03, 9.907323E+03, 1.007621E+04, 1.024651E+04, 1.041830E+04, 1.059152E+04,
    1.076618E+04, 1.094226E+04, 1.111984E+04, 1.129880E+04, 1.147925E+04, 1.166115E+04,
    1.184449E+04, 1.202927E+04, 1.221551E+04, 1.240319E+04, 1.259232E+04, 1.278283E+04,
    1.297487E+04, 1.316836E+04, 1.336323E+04, 1.355957E+04, 1.375736E+04, 1.395662E+04,
    1.415733E+04, 1.435944E+04, 1.456301E+04, 1.476805E+04, 1.497455E+04, 1.518245E+04,
    1.539182E+04, 1.560258E+04, 1.581489E+04, 1.602852E+04, 1.624369E+04, 1.646026E+04,
    1.667823E+04, 1.689775E+04, 1.711859E+04, 1.734099E+04, 1.756470E+04, 1.778997E+04,
    1.801664E+04, 1.824470E+04, 1.847424E+04, 1.870518E+04, 1.893760E+04, 1.917141E+04,
    1.940670E+04, 1.964339E+04, 1.988156E+04, 2.012112E+04, 2.036208E+04, 2.060452E+04,
    2.084835E+04, 2.109366E+04, 2.134037E+04, 2.158846E+04, 2.183804E+04, 2.208901E+04,
    2.234136E+04, 2.259519E+04, 2.285042E+04, 2.310712E+04, 2.336520E+04, 2.362467E+04,
    2.388553E+04, 2.414786E+04, 2.441157E+04, 2.467677E+04, 2.494324E+04, 2.521119E+04,
    2.548061E+04, 2.575131E+04, 2.602348E+04, 2.629703E+04, 2.657195E+04, 2.684834E+04,
    2.712610E+04, 2.740522E+04,
])

#  ---------------------- M = 50, I = 2 ---------------------------
M = 50
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.784190E+00, 4.344159E+01, 9.997170E+01, 1.581917E+02, 2.169147E+02, 2.758572E+02,
    3.349170E+02, 3.940525E+02, 4.532464E+02, 5.125045E+02, 5.718560E+02, 6.313451E+02,
    6.910295E+02, 7.509947E+02, 8.113115E+02, 8.720752E+02, 9.333752E+02, 9.952881E+02,
    1.057893E+03, 1.121284E+03, 1.185511E+03, 1.250646E+03, 1.316747E+03, 1.383869E+03,
    1.452039E+03, 1.521334E+03, 1.591771E+03, 1.663379E+03, 1.736195E+03, 1.810241E+03,
    1.885547E+03, 1.962127E+03, 2.039987E+03, 2.119187E+03, 2.199692E+03, 2.281556E+03,
    2.364777E+03, 2.449343E+03, 2.535286E+03, 2.622613E+03, 2.711330E+03, 2.801463E+03,
    2.892990E+03, 2.985909E+03, 3.080268E+03, 3.176036E+03, 3.273235E+03, 3.371855E+03,
    3.471916E+03, 3.573406E+03, 3.676342E+03, 3.780709E+03, 3.886523E+03, 3.993797E+03,
    4.102516E+03, 4.212690E+03, 4.324334E+03, 4.437392E+03, 4.551942E+03, 4.667963E+03,
    4.785429E+03, 4.904350E+03, 5.024735E+03, 5.146593E+03, 5.269896E+03, 5.394689E+03,
    5.520982E+03, 5.648704E+03, 5.777902E+03, 5.908582E+03, 6.040712E+03, 6.174338E+03,
    6.309425E+03, 6.446022E+03, 6.584049E+03, 6.723599E+03, 6.864589E+03, 7.007068E+03,
    7.151042E+03, 7.296470E+03, 7.443403E+03, 7.591798E+03, 7.741660E+03, 7.893042E+03,
    8.045899E+03, 8.200235E+03, 8.356054E+03, 8.513307E+03, 8.672103E+03, 8.832393E+03,
    8.994127E+03, 9.157362E+03, 9.322101E+03, 9.488294E+03, 9.655997E+03, 9.825158E+03,
    9.995836E+03, 1.016798E+04, 1.034164E+04, 1.051677E+04, 1.069338E+04, 1.087151E+04,
    1.105106E+04, 1.123214E+04, 1.141471E+04, 1.159882E+04, 1.178434E+04, 1.197142E+04,
    1.215991E+04, 1.234990E+04, 1.254144E+04, 1.273440E+04, 1.292893E+04, 1.312488E+04,
    1.332233E+04, 1.352128E+04, 1.372172E+04, 1.392367E+04, 1.412705E+04, 1.433200E+04,
    1.453839E+04, 1.474621E+04, 1.495561E+04, 1.516644E+04, 1.537878E+04, 1.559263E+04,
    1.580792E+04, 1.602472E+04, 1.624304E+04, 1.646279E+04, 1.668405E+04, 1.690675E+04,
    1.713097E+04, 1.735671E+04, 1.758388E+04, 1.781249E+04, 1.804261E+04, 1.827426E+04,
    1.850734E+04, 1.874195E+04, 1.897798E+04, 1.921554E+04, 1.945454E+04, 1.969497E+04,
    1.993692E+04, 2.018039E+04, 2.042530E+04, 2.067164E+04, 2.091950E+04, 2.116879E+04,
    2.141952E+04, 2.167176E+04, 2.192553E+04, 2.218063E+04, 2.243725E+04, 2.269540E+04,
    2.295488E+04, 2.321596E+04, 2.347839E+04, 2.374232E+04, 2.400769E+04, 2.427447E+04,
    2.454277E+04, 2.481249E+04, 2.508362E+04, 2.535627E+04, 2.563034E+04, 2.590582E+04,
    2.618281E+04, 2.646111E+04, 2.674103E+04, 2.702225E+04, 2.730487E+04, 2.758901E+04,
    2.787455E+04, 2.816159E+04,
])

#  ---------------------- M = 50, I = 3 ---------------------------
M = 50
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    1.829360E+00, 4.567662E+01, 1.054716E+02, 1.670941E+02, 2.292611E+02, 2.916655E+02,
    3.541968E+02, 4.168097E+02, 4.794904E+02, 5.422517E+02, 6.051222E+02, 6.681570E+02,
    7.314428E+02, 7.950479E+02, 8.590830E+02, 9.236403E+02, 9.888198E+02, 1.054709E+03,
    1.121396E+03, 1.188975E+03, 1.257504E+03, 1.327060E+03, 1.397697E+03, 1.469475E+03,
    1.542453E+03, 1.616657E+03, 1.692120E+03, 1.768904E+03, 1.847020E+03, 1.926507E+03,
    2.007363E+03, 2.089638E+03, 2.173342E+03, 2.258478E+03, 2.345089E+03, 2.433172E+03,
    2.522743E+03, 2.613793E+03, 2.706354E+03, 2.800433E+03, 2.896036E+03, 2.993166E+03,
    3.091851E+03, 3.192062E+03, 3.293823E+03, 3.397129E+03, 3.502001E+03, 3.608430E+03,
    3.716436E+03, 3.826007E+03, 3.937129E+03, 4.049850E+03, 4.164154E+03, 4.280024E+03,
    4.397473E+03, 4.516514E+03, 4.637127E+03, 4.759358E+03, 4.883149E+03, 5.008545E+03,
    5.135559E+03, 5.264164E+03, 5.394332E+03, 5.526108E+03, 5.659504E+03, 5.794488E+03,
    5.931067E+03, 6.069251E+03, 6.209046E+03, 6.350461E+03, 6.493459E+03, 6.638047E+03,
    6.784276E+03, 6.932107E+03, 7.081546E+03, 7.232601E+03, 7.385228E+03, 7.539480E+03,
    7.695363E+03, 7.852884E+03, 8.011947E+03, 8.172707E+03, 8.335018E+03, 8.498934E+03,
    8.664511E+03, 8.831702E+03, 9.000458E+03, 9.170889E+03, 9.342891E+03, 9.516521E+03,
    9.691784E+03, 9.868684E+03, 1.004717E+04, 1.022729E+04, 1.040901E+04, 1.059237E+04,
    1.077733E+04, 1.096388E+04, 1.115209E+04, 1.134190E+04, 1.153338E+04, 1.172646E+04,
    1.192108E+04, 1.211738E+04, 1.231535E+04, 1.251487E+04, 1.271601E+04, 1.291883E+04,
    1.312320E+04, 1.332925E+04, 1.353687E+04, 1.374610E+04, 1.395703E+04, 1.416951E+04,
    1.438362E+04, 1.459936E+04, 1.481666E+04, 1.503566E+04, 1.525623E+04, 1.547842E+04,
    1.570225E+04, 1.592764E+04, 1.615467E+04, 1.638333E+04, 1.661363E+04, 1.684549E+04,
    1.707899E+04, 1.731414E+04, 1.755084E+04, 1.778910E+04, 1.802909E+04, 1.827064E+04,
    1.851375E+04, 1.875850E+04, 1.900489E+04, 1.925285E+04, 1.950236E+04, 1.975360E+04,
    2.000631E+04, 2.026067E+04, 2.051667E+04, 2.077423E+04, 2.103334E+04, 2.129409E+04,
    2.155649E+04, 2.182035E+04, 2.208594E+04, 2.235299E+04, 2.262168E+04, 2.289202E+04,
    2.316380E+04, 2.343732E+04, 2.371229E+04, 2.398890E+04, 2.426705E+04, 2.454684E+04,
    2.482816E+04, 2.511102E+04, 2.539552E+04, 2.568155E+04, 2.596912E+04, 2.625831E+04,
    2.654893E+04, 2.684129E+04, 2.713506E+04, 2.743046E+04, 2.772739E+04, 2.802583E+04,
    2.832590E+04, 2.862748E+04, 2.893058E+04, 2.923518E+04, 2.954141E+04, 2.984914E+04,
    3.015838E+04, 3.046922E+04,
])

#  ---------------------- M = 51, I = 1 ---------------------------
M = 51
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.242436E+01, 7.725486E+02, 2.121419E+03, 3.890178E+03, 6.013062E+03, 8.504252E+03,
    1.142366E+04, 1.485658E+04, 1.890626E+04, 2.369326E+04, 2.935867E+04, 3.606709E+04,
    4.401067E+04, 5.341521E+04, 6.454346E+04, 7.771136E+04, 9.328406E+04, 1.116825E+05,
    1.334005E+05, 1.590135E+05, 1.891903E+05, 2.247090E+05, 2.664698E+05, 3.155170E+05,
    3.730570E+05, 4.404840E+05, 5.194095E+05, 6.116882E+05, 7.194554E+05, 8.451682E+05,
    9.916491E+05, 1.162137E+06, 1.360343E+06, 1.590521E+06, 1.857532E+06, 2.166934E+06,
    2.525070E+06, 2.939175E+06, 3.417490E+06, 3.969394E+06, 4.605551E+06, 5.338073E+06,
    6.180709E+06, 7.149044E+06, 8.260724E+06, 9.535725E+06, 1.099663E+07, 1.266894E+07,
    1.458144E+07, 1.676658E+07, 1.926092E+07, 2.210560E+07, 2.534690E+07, 2.903680E+07,
    3.323367E+07, 3.800297E+07, 4.341808E+07, 4.956117E+07, 5.652419E+07, 6.440992E+07,
    7.333322E+07, 8.342230E+07, 9.482017E+07, 1.076862E+08, 1.221980E+08, 1.385530E+08,
    1.569711E+08, 1.776964E+08, 2.010001E+08, 2.271832E+08, 2.565793E+08, 2.895582E+08,
    3.265296E+08, 3.679465E+08, 4.143103E+08, 4.661747E+08, 5.241516E+08, 5.889162E+08,
    6.612133E+08, 7.418637E+08, 8.317719E+08, 9.319336E+08, 1.043444E+09, 1.167508E+09,
    1.305450E+09, 1.458722E+09, 1.628922E+09, 1.817798E+09, 2.027272E+09, 2.259444E+09,
    2.516619E+09, 2.801317E+09, 3.116294E+09, 3.464566E+09, 3.849427E+09, 4.274473E+09,
    4.743631E+09, 5.261186E+09, 5.831808E+09, 6.460589E+09, 7.153075E+09, 7.915304E+09,
    8.753845E+09, 9.675848E+09, 1.068908E+10, 1.180199E+10, 1.302374E+10, 1.436429E+10,
    1.583444E+10, 1.744592E+10, 1.921141E+10, 2.114470E+10, 2.326068E+10, 2.557551E+10,
    2.810664E+10, 3.087298E+10, 3.389497E+10, 3.719468E+10, 4.079597E+10, 4.472462E+10,
    4.900843E+10, 5.367742E+10, 5.876395E+10, 6.430294E+10, 7.033198E+10, 7.689160E+10,
    8.402544E+10, 9.178048E+10, 1.002073E+11, 1.093602E+11, 1.192976E+11, 1.300825E+11,
    1.417824E+11, 1.544698E+11, 1.682225E+11, 1.831241E+11, 1.992643E+11, 2.167394E+11,
    2.356524E+11, 2.561139E+11, 2.782424E+11, 3.021646E+11, 3.280164E+11, 3.559434E+11,
    3.861009E+11, 4.186556E+11, 4.537853E+11, 4.916803E+11, 5.325440E+11, 5.765937E+11,
    6.240612E+11, 6.751945E+11, 7.302577E+11, 7.895332E+11, 8.533219E+11, 9.219446E+11,
    9.957437E+11, 1.075084E+12, 1.160353E+12, 1.251966E+12, 1.350364E+12, 1.456014E+12,
    1.569417E+12, 1.691105E+12, 1.821642E+12, 1.961629E+12, 2.111705E+12, 2.272549E+12,
    2.444884E+12, 2.629474E+12, 2.827135E+12, 3.038731E+12, 3.265180E+12, 3.507453E+12,
    3.766586E+12, 4.043672E+12,
])

#  ---------------------- M = 52, I = 1 ---------------------------
M = 52
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.000000E-06, 9.439835E+00, 2.871728E+01, 5.406319E+01, 8.449555E+01, 1.193488E+02,
    1.582205E+02, 2.008949E+02, 2.472961E+02, 2.974472E+02, 3.514358E+02, 4.093897E+02,
    4.714617E+02, 5.378208E+02, 6.086474E+02, 6.841307E+02, 7.644677E+02, 8.498620E+02,
    9.405242E+02, 1.036671E+03, 1.138525E+03, 1.246317E+03, 1.360281E+03, 1.480661E+03,
    1.607706E+03, 1.741671E+03, 1.882822E+03, 2.031428E+03, 2.187770E+03, 2.352137E+03,
    2.524823E+03, 2.706136E+03, 2.896391E+03, 3.095913E+03, 3.305037E+03, 3.524108E+03,
    3.753482E+03, 3.993526E+03, 4.244617E+03, 4.507146E+03, 4.781511E+03, 5.068125E+03,
    5.367413E+03, 5.679810E+03, 6.005763E+03, 6.345735E+03, 6.700197E+03, 7.069636E+03,
    7.454550E+03, 7.855452E+03, 8.272866E+03, 8.707330E+03, 9.159397E+03, 9.629633E+03,
    1.011862E+04, 1.062694E+04, 1.115522E+04, 1.170407E+04, 1.227414E+04, 1.286606E+04,
    1.348052E+04, 1.411819E+04, 1.477977E+04, 1.546598E+04, 1.617755E+04, 1.691522E+04,
    1.767975E+04, 1.847193E+04, 1.929255E+04, 2.014242E+04, 2.102237E+04, 2.193326E+04,
    2.287594E+04, 2.385130E+04, 2.486024E+04, 2.590368E+04, 2.698256E+04, 2.809783E+04,
    2.925048E+04, 3.044150E+04, 3.167190E+04, 3.294272E+04, 3.425501E+04, 3.560985E+04,
    3.700835E+04, 3.845160E+04, 3.994076E+04, 4.147698E+04, 4.306145E+04, 4.469536E+04,
    4.637995E+04, 4.811646E+04, 4.990616E+04, 5.175035E+04, 5.365034E+04, 5.560747E+04,
    5.762312E+04, 5.969865E+04, 6.183550E+04, 6.403509E+04, 6.629888E+04, 6.862837E+04,
    7.102507E+04, 7.349051E+04, 7.602625E+04, 7.863389E+04, 8.131504E+04, 8.407135E+04,
    8.690448E+04, 8.981613E+04, 9.280803E+04, 9.588192E+04, 9.903959E+04, 1.022828E+05,
    1.056135E+05, 1.090335E+05, 1.125446E+05, 1.161488E+05, 1.198481E+05, 1.236444E+05,
    1.275397E+05, 1.315362E+05, 1.356358E+05, 1.398406E+05, 1.441529E+05, 1.485747E+05,
    1.531082E+05, 1.577558E+05, 1.625195E+05, 1.674018E+05, 1.724050E+05, 1.775314E+05,
    1.827833E+05, 1.881633E+05, 1.936738E+05, 1.993172E+05, 2.050961E+05, 2.110130E+05,
    2.170706E+05, 2.232714E+05, 2.296181E+05, 2.361134E+05, 2.427600E+05, 2.495607E+05,
    2.565183E+05, 2.636356E+05, 2.709155E+05, 2.783609E+05, 2.859747E+05, 2.937599E+05,
    3.017195E+05, 3.098566E+05, 3.181742E+05, 3.266755E+05, 3.353636E+05, 3.442417E+05,
    3.533131E+05, 3.625811E+05, 3.720488E+05, 3.817198E+05, 3.915973E+05, 4.016848E+05,
    4.119858E+05, 4.225038E+05, 4.332422E+05, 4.442047E+05, 4.553950E+05, 4.668165E+05,
    4.784731E+05, 4.903685E+05, 5.025065E+05, 5.148908E+05, 5.275254E+05, 5.404141E+05,
    5.535608E+05, 5.669696E+05,
])

#  ---------------------- M = 53, I = 1 ---------------------------
M = 53
I = 1
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    3.357230E+00, 6.388395E+01, 1.276299E+02, 1.914331E+02, 2.556212E+02, 3.212115E+02,
    3.897614E+02, 4.629667E+02, 5.424324E+02, 6.296189E+02, 7.258625E+02, 8.324166E+02,
    9.504891E+02, 1.081274E+03, 1.225973E+03, 1.385815E+03, 1.562070E+03, 1.756055E+03,
    1.969145E+03, 2.202779E+03, 2.458461E+03, 2.737768E+03, 3.042351E+03, 3.373938E+03,
    3.734336E+03, 4.125436E+03, 4.549211E+03, 5.007721E+03, 5.503115E+03, 6.037631E+03,
    6.613599E+03, 7.233444E+03, 7.899685E+03, 8.614941E+03, 9.381928E+03, 1.020347E+04,
    1.108248E+04, 1.202199E+04, 1.302514E+04, 1.409517E+04, 1.523543E+04, 1.644939E+04,
    1.774064E+04, 1.911288E+04, 2.056992E+04, 2.211570E+04, 2.375428E+04, 2.548986E+04,
    2.732674E+04, 2.926936E+04, 3.132230E+04, 3.349026E+04, 3.577808E+04, 3.819073E+04,
    4.073333E+04, 4.341113E+04, 4.622952E+04, 4.919405E+04, 5.231040E+04, 5.558441E+04,
    5.902205E+04, 6.262946E+04, 6.641295E+04, 7.037894E+04, 7.453405E+04, 7.888504E+04,
    8.343883E+04, 8.820253E+04, 9.318338E+04, 9.838881E+04, 1.038264E+05, 1.095040E+05,
    1.154294E+05, 1.216108E+05, 1.280566E+05, 1.347751E+05, 1.417750E+05, 1.490652E+05,
    1.566547E+05, 1.645527E+05, 1.727687E+05, 1.813122E+05, 1.901930E+05, 1.994211E+05,
    2.090067E+05, 2.189602E+05, 2.292922E+05, 2.400135E+05, 2.511350E+05, 2.626681E+05,
    2.746241E+05, 2.870146E+05, 2.998515E+05, 3.131469E+05, 3.269131E+05, 3.411625E+05,
    3.559080E+05, 3.711624E+05, 3.869389E+05, 4.032510E+05, 4.201123E+05, 4.375367E+05,
    4.555383E+05, 4.741314E+05, 4.933307E+05, 5.131510E+05, 5.336073E+05, 5.547150E+05,
    5.764896E+05, 5.989470E+05, 6.221033E+05, 6.459748E+05, 6.705781E+05, 6.959299E+05,
    7.220475E+05, 7.489482E+05, 7.766496E+05, 8.051697E+05, 8.345266E+05, 8.647387E+05,
    8.958248E+05, 9.278039E+05, 9.606952E+05, 9.945184E+05, 1.029293E+06, 1.065040E+06,
    1.101778E+06, 1.139530E+06, 1.178315E+06, 1.218156E+06, 1.259073E+06, 1.301088E+06,
    1.344225E+06, 1.388504E+06, 1.433950E+06, 1.480585E+06, 1.528432E+06, 1.577515E+06,
    1.627859E+06, 1.679487E+06, 1.732424E+06, 1.786695E+06, 1.842326E+06, 1.899341E+06,
    1.957768E+06, 2.017632E+06, 2.078959E+06, 2.141778E+06, 2.206114E+06, 2.271996E+06,
    2.339451E+06, 2.408509E+06, 2.479196E+06, 2.551543E+06, 2.625579E+06, 2.701333E+06,
    2.778836E+06, 2.858117E+06, 2.939208E+06, 3.022139E+06, 3.106942E+06, 3.193650E+06,
    3.282293E+06, 3.372905E+06, 3.465518E+06, 3.560166E+06, 3.656882E+06, 3.755701E+06,
    3.856657E+06, 3.959784E+06, 4.065119E+06, 4.172696E+06, 4.282551E+06, 4.394721E+06,
    4.509242E+06, 4.626152E+06,
])

#  ---------------------- M = 53, I = 2 ---------------------------
M = 53
I = 2
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.908470E+00, 1.316557E+02, 2.630372E+02, 3.945385E+02, 5.268460E+02, 6.620835E+02,
    8.034970E+02, 9.546200E+02, 1.118799E+03, 1.299078E+03, 1.498248E+03, 1.718924E+03,
    1.963631E+03, 2.234865E+03, 2.535144E+03, 2.867044E+03, 3.233225E+03, 3.636454E+03,
    4.079620E+03, 4.565744E+03, 5.097990E+03, 5.679674E+03, 6.314270E+03, 7.005414E+03,
    7.756911E+03, 8.572742E+03, 9.457061E+03, 1.041421E+04, 1.144871E+04, 1.256529E+04,
    1.376884E+04, 1.506449E+04, 1.645754E+04, 1.795352E+04, 1.955815E+04, 2.127738E+04,
    2.311737E+04, 2.508451E+04, 2.718541E+04, 2.942693E+04, 3.181613E+04, 3.436034E+04,
    3.706712E+04, 3.994428E+04, 4.299988E+04, 4.624224E+04, 4.967993E+04, 5.332180E+04,
    5.717695E+04, 6.125477E+04, 6.556491E+04, 7.011731E+04, 7.492218E+04, 7.999005E+04,
    8.533172E+04, 9.095828E+04, 9.688115E+04, 1.031120E+05, 1.096629E+05, 1.165462E+05,
    1.237744E+05, 1.313607E+05, 1.393182E+05, 1.476606E+05, 1.564019E+05, 1.655564E+05,
    1.751387E+05, 1.851638E+05, 1.956471E+05, 2.066043E+05, 2.180514E+05, 2.300049E+05,
    2.424815E+05, 2.554985E+05, 2.690734E+05, 2.832241E+05, 2.979690E+05, 3.133267E+05,
    3.293164E+05, 3.459575E+05, 3.632700E+05, 3.812742E+05, 3.999909E+05, 4.194410E+05,
    4.396463E+05, 4.606288E+05, 4.824107E+05, 5.050151E+05, 5.284651E+05, 5.527846E+05,
    5.779977E+05, 6.041290E+05, 6.312037E+05, 6.592472E+05, 6.882857E+05, 7.183455E+05,
    7.494537E+05, 7.816377E+05, 8.149253E+05, 8.493451E+05, 8.849259E+05, 9.216971E+05,
    9.596886E+05, 9.989308E+05, 1.039455E+06, 1.081291E+06, 1.124473E+06, 1.169032E+06,
    1.215002E+06, 1.262415E+06, 1.311307E+06, 1.361711E+06, 1.413663E+06, 1.467198E+06,
    1.522353E+06, 1.579164E+06, 1.637669E+06, 1.697906E+06, 1.759913E+06, 1.823730E+06,
    1.889395E+06, 1.956950E+06, 2.026435E+06, 2.097891E+06, 2.171361E+06, 2.246887E+06,
    2.324512E+06, 2.404281E+06, 2.486237E+06, 2.570426E+06, 2.656894E+06, 2.745687E+06,
    2.836852E+06, 2.930436E+06, 3.026488E+06, 3.125057E+06, 3.226192E+06, 3.329943E+06,
    3.436362E+06, 3.545501E+06, 3.657410E+06, 3.772143E+06, 3.889755E+06, 4.010298E+06,
    4.133829E+06, 4.260402E+06, 4.390074E+06, 4.522903E+06, 4.658945E+06, 4.798260E+06,
    4.940907E+06, 5.086945E+06, 5.236436E+06, 5.389440E+06, 5.546021E+06, 5.706240E+06,
    5.870161E+06, 6.037850E+06, 6.209371E+06, 6.384789E+06, 6.564172E+06, 6.747587E+06,
    6.935102E+06, 7.126787E+06, 7.322711E+06, 7.522944E+06, 7.727559E+06, 7.936627E+06,
    8.150221E+06, 8.368415E+06, 8.591283E+06, 8.818902E+06, 9.051347E+06, 9.288696E+06,
    9.531026E+06, 9.778416E+06,
])

#  ---------------------- M = 53, I = 3 ---------------------------
M = 53
I = 3
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    2.724759E+01, 5.188821E+02, 1.036664E+03, 1.554914E+03, 2.076315E+03, 2.609185E+03,
    3.166242E+03, 3.761337E+03, 4.407580E+03, 5.116904E+03, 5.900230E+03, 6.767805E+03,
    7.729507E+03, 8.795104E+03, 9.974440E+03, 1.127758E+04, 1.271492E+04, 1.429726E+04,
    1.603589E+04, 1.794258E+04, 2.002969E+04, 2.231015E+04, 2.479752E+04, 2.750597E+04,
    3.045034E+04, 3.364615E+04, 3.710960E+04, 4.085761E+04, 4.490782E+04, 4.927862E+04,
    5.398913E+04, 5.905927E+04, 6.450975E+04, 7.036207E+04, 7.663855E+04, 8.336236E+04,
    9.055751E+04, 9.824887E+04, 1.064622E+05, 1.152242E+05, 1.245624E+05, 1.345053E+05,
    1.450823E+05, 1.563239E+05, 1.682615E+05, 1.809275E+05, 1.943551E+05, 2.085790E+05,
    2.236344E+05, 2.395580E+05, 2.563872E+05, 2.741609E+05, 2.929188E+05, 3.127018E+05,
    3.335519E+05, 3.555124E+05, 3.786276E+05, 4.029431E+05, 4.285056E+05, 4.553632E+05,
    4.835651E+05, 5.131617E+05, 5.442048E+05, 5.767474E+05, 6.108439E+05, 6.465498E+05,
    6.839223E+05, 7.230196E+05, 7.639014E+05, 8.066289E+05, 8.512645E+05, 8.978721E+05,
    9.465171E+05, 9.972663E+05, 1.050188E+06, 1.105352E+06, 1.162829E+06, 1.222692E+06,
    1.285016E+06, 1.349877E+06, 1.417351E+06, 1.487518E+06, 1.560458E+06, 1.636254E+06,
    1.714990E+06, 1.796750E+06, 1.881623E+06, 1.969697E+06, 2.061062E+06, 2.155812E+06,
    2.254039E+06, 2.355840E+06, 2.461313E+06, 2.570556E+06, 2.683671E+06, 2.800761E+06,
    2.921930E+06, 3.047286E+06, 3.176937E+06, 3.310992E+06, 3.449566E+06, 3.592771E+06,
    3.740724E+06, 3.893544E+06, 4.051350E+06, 4.214264E+06, 4.382412E+06, 4.555918E+06,
    4.734911E+06, 4.919521E+06, 5.109882E+06, 5.306126E+06, 5.508392E+06, 5.716817E+06,
    5.931542E+06, 6.152711E+06, 6.380469E+06, 6.614963E+06, 6.856342E+06, 7.104760E+06,
    7.360369E+06, 7.623326E+06, 7.893790E+06, 8.171922E+06, 8.457886E+06, 8.751846E+06,
    9.053972E+06, 9.364433E+06, 9.683403E+06, 1.001106E+07, 1.034757E+07, 1.069313E+07,
    1.104791E+07, 1.141209E+07, 1.178588E+07, 1.216945E+07, 1.256300E+07, 1.296673E+07,
    1.338083E+07, 1.380550E+07, 1.424095E+07, 1.468738E+07, 1.514500E+07, 1.561402E+07,
    1.609465E+07, 1.658712E+07, 1.709163E+07, 1.760842E+07, 1.813770E+07, 1.867971E+07,
    1.923467E+07, 1.980281E+07, 2.038438E+07, 2.097961E+07, 2.158874E+07, 2.221202E+07,
    2.284969E+07, 2.350201E+07, 2.416922E+07, 2.485159E+07, 2.554937E+07, 2.626282E+07,
    2.699222E+07, 2.773782E+07, 2.849990E+07, 2.927873E+07, 3.007460E+07, 3.088777E+07,
    3.171854E+07, 3.256719E+07, 3.343401E+07, 3.431930E+07, 3.522335E+07, 3.614645E+07,
    3.708892E+07, 3.805106E+07,
])

#  ---------------------- M = 53, I = 4 ---------------------------
M = 53
I = 4
TIPS_2017_ISOT_HASH[(M,I)] = TIPS_2017_ISOT[2]
TIPS_2017_ISOQ_HASH[(M,I)] = float64([
    6.713360E+00, 1.277459E+02, 2.552158E+02, 3.828120E+02, 5.112785E+02, 6.428265E+02,
    7.807297E+02, 9.284754E+02, 1.089325E+03, 1.266231E+03, 1.461891E+03, 1.678831E+03,
    1.919481E+03, 2.186220E+03, 2.481426E+03, 2.807498E+03, 3.166878E+03, 3.562071E+03,
    3.995649E+03, 4.470269E+03, 4.988676E+03, 5.553712E+03, 6.168321E+03, 6.835557E+03,
    7.558585E+03, 8.340692E+03, 9.185283E+03, 1.009589E+04, 1.107619E+04, 1.212996E+04,
    1.326116E+04, 1.447384E+04, 1.577225E+04, 1.716074E+04, 1.864383E+04, 2.022620E+04,
    2.191268E+04, 2.370826E+04, 2.561808E+04, 2.764746E+04, 2.980188E+04, 3.208700E+04,
    3.450865E+04, 3.707282E+04, 3.978569E+04, 4.265362E+04, 4.568317E+04, 4.888106E+04,
    5.225421E+04, 5.580974E+04, 5.955495E+04, 6.349734E+04, 6.764462E+04, 7.200469E+04,
    7.658567E+04, 8.139588E+04, 8.644384E+04, 9.173829E+04, 9.728821E+04, 1.031028E+05,
    1.091913E+05, 1.155636E+05, 1.222293E+05, 1.291987E+05, 1.364819E+05, 1.440896E+05,
    1.520325E+05, 1.603216E+05, 1.689683E+05, 1.779840E+05, 1.873805E+05, 1.971699E+05,
    2.073643E+05, 2.179763E+05, 2.290187E+05, 2.405046E+05, 2.524472E+05, 2.648602E+05,
    2.777574E+05, 2.911528E+05, 3.050611E+05, 3.194967E+05, 3.344747E+05, 3.500103E+05,
    3.661191E+05, 3.828169E+05, 4.001198E+05, 4.180442E+05, 4.366069E+05, 4.558248E+05,
    4.757153E+05, 4.962961E+05, 5.175850E+05, 5.396004E+05, 5.623607E+05, 5.858850E+05,
    6.101923E+05, 6.353023E+05, 6.612348E+05, 6.880100E+05, 7.156484E+05, 7.441709E+05,
    7.735987E+05, 8.039534E+05, 8.352568E+05, 8.675311E+05, 9.007991E+05, 9.350835E+05,
    9.704078E+05, 1.006796E+06, 1.044271E+06, 1.082858E+06, 1.122582E+06, 1.163467E+06,
    1.205540E+06, 1.248826E+06, 1.293351E+06, 1.339142E+06, 1.386227E+06, 1.434632E+06,
    1.484385E+06, 1.535514E+06, 1.588049E+06, 1.642018E+06, 1.697450E+06, 1.754375E+06,
    1.812824E+06, 1.872828E+06, 1.934416E+06, 1.997621E+06, 2.062474E+06, 2.129008E+06,
    2.197256E+06, 2.267251E+06, 2.339026E+06, 2.412615E+06, 2.488053E+06, 2.565375E+06,
    2.644616E+06, 2.725811E+06, 2.808998E+06, 2.894213E+06, 2.981493E+06, 3.070876E+06,
    3.162399E+06, 3.256102E+06, 3.352023E+06, 3.450202E+06, 3.550679E+06, 3.653494E+06,
    3.758688E+06, 3.866303E+06, 3.976381E+06, 4.088964E+06, 4.204094E+06, 4.321816E+06,
    4.442173E+06, 4.565209E+06, 4.690970E+06, 4.819501E+06, 4.950847E+06, 5.085056E+06,
    5.222174E+06, 5.362248E+06, 5.505328E+06, 5.651460E+06, 5.800694E+06, 5.953081E+06,
    6.108669E+06, 6.267510E+06, 6.429655E+06, 6.595155E+06, 6.764063E+06, 6.936432E+06,
    7.112315E+06, 7.291766E+06,
])

def BD_TIPS_2017_PYTHON(M,I,T):
    # get temperature grid
    TT = TIPS_2017_ISOT_HASH[(M,I)]
    Tmin = min(TT); Tmax = max(TT)
    
    # out of temperature range
    if T<Tmin or T>Tmax:
        raise Exception('TIPS2017: T(%.1fK) must be between %.1fK and %.1fK.'%(T,Tmin,Tmax))
    
    try:
        # get statistical weight for specified isotopologue
        #gi = TIPS_GSI_HASH[(M,I)] # Take from TIPS-2011?
        # interpolate partition sum for specified isotopologue
        Qt = AtoB(T,TT,TIPS_2017_ISOQ_HASH[(M,I)],len(TT))
    except KeyError:
        raise Exception('TIPS2017: no data for M,I = %d,%d.' % (M,I))
    
    return None,Qt

def BD_TIPS_2017_PYTHON_SLICE(M,I,T,n=20): # testing
    """
    Calculate partition sum using Lagrange interpolation
    with slicing algorithm, taking (i_T-n,i_T+n) points,
    where i_T is the index of the array value closest to T.
    """
    # get temperature grid
    TT = TIPS_2017_ISOT_HASH[(M,I)]
    Tmin = min(TT); Tmax = max(TT); NT = len(TT)
    
    # get partition sum
    QQ = TIPS_2017_ISOQ_HASH[(M,I)]
    
    # slice temperature grid and partition sum
    i_T = np.searchsorted(TT,T)
    TT_ = TT[max(i_T-n,0):min(i_T+n,NT)]
    QQ_ = QQ[max(i_T-n,0):min(i_T+n,NT)]
    
    # out of temperature range
    if T<Tmin or T>Tmax:
        raise Exception('TIPS2017: T(%.1fK) must be between %.1fK and %.1fK.'%(T,Tmin,Tmax))
    
    try:
        # get statistical weight for specified isotopologue
        #gi = TIPS_GSI_HASH[(M,I)] # Take from TIPS-2011?
        # interpolate partition sum for specified isotopologue
        Qt = AtoB(T,TT_,QQ_,len(TT_))
    except KeyError:
        raise Exception('TIPS2017: no data for M,I = %d,%d.' % (M,I))
    
    return None,Qt    

#  --------------- /TIPS-2017 IMPLEMENTATION ----------------------

# ========================= TIPS2021 PARTITION SUMS =========================
# Gamache et al. (2021), JQSRT 271, 107713 (doi: 10.1016/j.jqsrt.2021.107713)
# Data obtained from: https://zenodo.org/record/4708099 
# ===========================================================================

TIPS_2021_ISOT_HASH = {}
TIPS_2021_ISOQ_HASH = {}

TIPS_2021_ISOT = {}

TIPS_2021_ISOT[0] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0,
])
TIPS_2021_ISOT[1] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0,
])
TIPS_2021_ISOT[2] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0,
])
TIPS_2021_ISOT[3] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0, 3020.0, 3030.0, 3040.0, 3050.0, 3060.0, 3070.0, 3080.0, 3090.0,
 3100.0, 3110.0, 3120.0, 3130.0, 3140.0, 3150.0, 3160.0, 3170.0, 3180.0, 3190.0,
 3200.0, 3210.0, 3220.0, 3230.0, 3240.0, 3250.0, 3260.0, 3270.0, 3280.0, 3290.0,
 3300.0, 3310.0, 3320.0, 3330.0, 3340.0, 3350.0, 3360.0, 3370.0, 3380.0, 3390.0,
 3400.0, 3410.0, 3420.0, 3430.0, 3440.0, 3450.0, 3460.0, 3470.0, 3480.0, 3490.0,
 3500.0,
])
TIPS_2021_ISOT[4] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0, 3020.0, 3030.0, 3040.0, 3050.0, 3060.0, 3070.0, 3080.0, 3090.0,
 3100.0, 3110.0, 3120.0, 3130.0, 3140.0, 3150.0, 3160.0, 3170.0, 3180.0, 3190.0,
 3200.0, 3210.0, 3220.0, 3230.0, 3240.0, 3250.0, 3260.0, 3270.0, 3280.0, 3290.0,
 3300.0, 3310.0, 3320.0, 3330.0, 3340.0, 3350.0, 3360.0, 3370.0, 3380.0, 3390.0,
 3400.0, 3410.0, 3420.0, 3430.0, 3440.0, 3450.0, 3460.0, 3470.0, 3480.0, 3490.0,
 3500.0, 3510.0, 3520.0, 3530.0, 3540.0, 3550.0, 3560.0, 3570.0, 3580.0, 3590.0,
 3600.0, 3610.0, 3620.0, 3630.0, 3640.0, 3650.0, 3660.0, 3670.0, 3680.0, 3690.0,
 3700.0, 3710.0, 3720.0, 3730.0, 3740.0, 3750.0, 3760.0, 3770.0, 3780.0, 3790.0,
 3800.0, 3810.0, 3820.0, 3830.0, 3840.0, 3850.0, 3860.0, 3870.0, 3880.0, 3890.0,
 3900.0, 3910.0, 3920.0, 3930.0, 3940.0, 3950.0, 3960.0, 3970.0, 3980.0, 3990.0,
 4000.0,
])
TIPS_2021_ISOT[5] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0, 3020.0, 3030.0, 3040.0, 3050.0, 3060.0, 3070.0, 3080.0, 3090.0,
 3100.0, 3110.0, 3120.0, 3130.0, 3140.0, 3150.0, 3160.0, 3170.0, 3180.0, 3190.0,
 3200.0, 3210.0, 3220.0, 3230.0, 3240.0, 3250.0, 3260.0, 3270.0, 3280.0, 3290.0,
 3300.0, 3310.0, 3320.0, 3330.0, 3340.0, 3350.0, 3360.0, 3370.0, 3380.0, 3390.0,
 3400.0, 3410.0, 3420.0, 3430.0, 3440.0, 3450.0, 3460.0, 3470.0, 3480.0, 3490.0,
 3500.0, 3510.0, 3520.0, 3530.0, 3540.0, 3550.0, 3560.0, 3570.0, 3580.0, 3590.0,
 3600.0, 3610.0, 3620.0, 3630.0, 3640.0, 3650.0, 3660.0, 3670.0, 3680.0, 3690.0,
 3700.0, 3710.0, 3720.0, 3730.0, 3740.0, 3750.0, 3760.0, 3770.0, 3780.0, 3790.0,
 3800.0, 3810.0, 3820.0, 3830.0, 3840.0, 3850.0, 3860.0, 3870.0, 3880.0, 3890.0,
 3900.0, 3910.0, 3920.0, 3930.0, 3940.0, 3950.0, 3960.0, 3970.0, 3980.0, 3990.0,
 4000.0, 4010.0, 4020.0, 4030.0, 4040.0, 4050.0, 4060.0, 4070.0, 4080.0, 4090.0,
 4100.0, 4110.0, 4120.0, 4130.0, 4140.0, 4150.0, 4160.0, 4170.0, 4180.0, 4190.0,
 4200.0, 4210.0, 4220.0, 4230.0, 4240.0, 4250.0, 4260.0, 4270.0, 4280.0, 4290.0,
 4300.0, 4310.0, 4320.0, 4330.0, 4340.0, 4350.0, 4360.0, 4370.0, 4380.0, 4390.0,
 4400.0, 4410.0, 4420.0, 4430.0, 4440.0, 4450.0, 4460.0, 4470.0, 4480.0, 4490.0,
 4500.0,
])
TIPS_2021_ISOT[6] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0, 3020.0, 3030.0, 3040.0, 3050.0, 3060.0, 3070.0, 3080.0, 3090.0,
 3100.0, 3110.0, 3120.0, 3130.0, 3140.0, 3150.0, 3160.0, 3170.0, 3180.0, 3190.0,
 3200.0, 3210.0, 3220.0, 3230.0, 3240.0, 3250.0, 3260.0, 3270.0, 3280.0, 3290.0,
 3300.0, 3310.0, 3320.0, 3330.0, 3340.0, 3350.0, 3360.0, 3370.0, 3380.0, 3390.0,
 3400.0, 3410.0, 3420.0, 3430.0, 3440.0, 3450.0, 3460.0, 3470.0, 3480.0, 3490.0,
 3500.0, 3510.0, 3520.0, 3530.0, 3540.0, 3550.0, 3560.0, 3570.0, 3580.0, 3590.0,
 3600.0, 3610.0, 3620.0, 3630.0, 3640.0, 3650.0, 3660.0, 3670.0, 3680.0, 3690.0,
 3700.0, 3710.0, 3720.0, 3730.0, 3740.0, 3750.0, 3760.0, 3770.0, 3780.0, 3790.0,
 3800.0, 3810.0, 3820.0, 3830.0, 3840.0, 3850.0, 3860.0, 3870.0, 3880.0, 3890.0,
 3900.0, 3910.0, 3920.0, 3930.0, 3940.0, 3950.0, 3960.0, 3970.0, 3980.0, 3990.0,
 4000.0, 4010.0, 4020.0, 4030.0, 4040.0, 4050.0, 4060.0, 4070.0, 4080.0, 4090.0,
 4100.0, 4110.0, 4120.0, 4130.0, 4140.0, 4150.0, 4160.0, 4170.0, 4180.0, 4190.0,
 4200.0, 4210.0, 4220.0, 4230.0, 4240.0, 4250.0, 4260.0, 4270.0, 4280.0, 4290.0,
 4300.0, 4310.0, 4320.0, 4330.0, 4340.0, 4350.0, 4360.0, 4370.0, 4380.0, 4390.0,
 4400.0, 4410.0, 4420.0, 4430.0, 4440.0, 4450.0, 4460.0, 4470.0, 4480.0, 4490.0,
 4500.0, 4510.0, 4520.0, 4530.0, 4540.0, 4550.0, 4560.0, 4570.0, 4580.0, 4590.0,
 4600.0, 4610.0, 4620.0, 4630.0, 4640.0, 4650.0, 4660.0, 4670.0, 4680.0, 4690.0,
 4700.0, 4710.0, 4720.0, 4730.0, 4740.0, 4750.0, 4760.0, 4770.0, 4780.0, 4790.0,
 4800.0, 4810.0, 4820.0, 4830.0, 4840.0, 4850.0, 4860.0, 4870.0, 4880.0, 4890.0,
 4900.0, 4910.0, 4920.0, 4930.0, 4940.0, 4950.0, 4960.0, 4970.0, 4980.0, 4990.0,
 5000.0,
])
TIPS_2021_ISOT[7] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0, 3020.0, 3030.0, 3040.0, 3050.0, 3060.0, 3070.0, 3080.0, 3090.0,
 3100.0, 3110.0, 3120.0, 3130.0, 3140.0, 3150.0, 3160.0, 3170.0, 3180.0, 3190.0,
 3200.0, 3210.0, 3220.0, 3230.0, 3240.0, 3250.0, 3260.0, 3270.0, 3280.0, 3290.0,
 3300.0, 3310.0, 3320.0, 3330.0, 3340.0, 3350.0, 3360.0, 3370.0, 3380.0, 3390.0,
 3400.0, 3410.0, 3420.0, 3430.0, 3440.0, 3450.0, 3460.0, 3470.0, 3480.0, 3490.0,
 3500.0, 3510.0, 3520.0, 3530.0, 3540.0, 3550.0, 3560.0, 3570.0, 3580.0, 3590.0,
 3600.0, 3610.0, 3620.0, 3630.0, 3640.0, 3650.0, 3660.0, 3670.0, 3680.0, 3690.0,
 3700.0, 3710.0, 3720.0, 3730.0, 3740.0, 3750.0, 3760.0, 3770.0, 3780.0, 3790.0,
 3800.0, 3810.0, 3820.0, 3830.0, 3840.0, 3850.0, 3860.0, 3870.0, 3880.0, 3890.0,
 3900.0, 3910.0, 3920.0, 3930.0, 3940.0, 3950.0, 3960.0, 3970.0, 3980.0, 3990.0,
 4000.0, 4010.0, 4020.0, 4030.0, 4040.0, 4050.0, 4060.0, 4070.0, 4080.0, 4090.0,
 4100.0, 4110.0, 4120.0, 4130.0, 4140.0, 4150.0, 4160.0, 4170.0, 4180.0, 4190.0,
 4200.0, 4210.0, 4220.0, 4230.0, 4240.0, 4250.0, 4260.0, 4270.0, 4280.0, 4290.0,
 4300.0, 4310.0, 4320.0, 4330.0, 4340.0, 4350.0, 4360.0, 4370.0, 4380.0, 4390.0,
 4400.0, 4410.0, 4420.0, 4430.0, 4440.0, 4450.0, 4460.0, 4470.0, 4480.0, 4490.0,
 4500.0, 4510.0, 4520.0, 4530.0, 4540.0, 4550.0, 4560.0, 4570.0, 4580.0, 4590.0,
 4600.0, 4610.0, 4620.0, 4630.0, 4640.0, 4650.0, 4660.0, 4670.0, 4680.0, 4690.0,
 4700.0, 4710.0, 4720.0, 4730.0, 4740.0, 4750.0, 4760.0, 4770.0, 4780.0, 4790.0,
 4800.0, 4810.0, 4820.0, 4830.0, 4840.0, 4850.0, 4860.0, 4870.0, 4880.0, 4890.0,
 4900.0, 4910.0, 4920.0, 4930.0, 4940.0, 4950.0, 4960.0, 4970.0, 4980.0, 4990.0,
 5000.0, 5010.0, 5020.0, 5030.0, 5040.0, 5050.0, 5060.0, 5070.0, 5080.0, 5090.0,
 5100.0, 5110.0, 5120.0, 5130.0, 5140.0, 5150.0, 5160.0, 5170.0, 5180.0, 5190.0,
 5200.0, 5210.0, 5220.0, 5230.0, 5240.0, 5250.0, 5260.0, 5270.0, 5280.0, 5290.0,
 5300.0, 5310.0, 5320.0, 5330.0, 5340.0, 5350.0, 5360.0, 5370.0, 5380.0, 5390.0,
 5400.0, 5410.0, 5420.0, 5430.0, 5440.0, 5450.0, 5460.0, 5470.0, 5480.0, 5490.0,
 5500.0, 5510.0, 5520.0, 5530.0, 5540.0, 5550.0, 5560.0, 5570.0, 5580.0, 5590.0,
 5600.0, 5610.0, 5620.0, 5630.0, 5640.0, 5650.0, 5660.0, 5670.0, 5680.0, 5690.0,
 5700.0, 5710.0, 5720.0, 5730.0, 5740.0, 5750.0, 5760.0, 5770.0, 5780.0, 5790.0,
 5800.0, 5810.0, 5820.0, 5830.0, 5840.0, 5850.0, 5860.0, 5870.0, 5880.0, 5890.0,
 5900.0, 5910.0, 5920.0, 5930.0, 5940.0, 5950.0, 5960.0, 5970.0, 5980.0, 5990.0,
 6000.0,
])
TIPS_2021_ISOT[8] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0, 3020.0, 3030.0, 3040.0, 3050.0, 3060.0, 3070.0, 3080.0, 3090.0,
 3100.0, 3110.0, 3120.0, 3130.0, 3140.0, 3150.0, 3160.0, 3170.0, 3180.0, 3190.0,
 3200.0, 3210.0, 3220.0, 3230.0, 3240.0, 3250.0, 3260.0, 3270.0, 3280.0, 3290.0,
 3300.0, 3310.0, 3320.0, 3330.0, 3340.0, 3350.0, 3360.0, 3370.0, 3380.0, 3390.0,
 3400.0, 3410.0, 3420.0, 3430.0, 3440.0, 3450.0, 3460.0, 3470.0, 3480.0, 3490.0,
 3500.0, 3510.0, 3520.0, 3530.0, 3540.0, 3550.0, 3560.0, 3570.0, 3580.0, 3590.0,
 3600.0, 3610.0, 3620.0, 3630.0, 3640.0, 3650.0, 3660.0, 3670.0, 3680.0, 3690.0,
 3700.0, 3710.0, 3720.0, 3730.0, 3740.0, 3750.0, 3760.0, 3770.0, 3780.0, 3790.0,
 3800.0, 3810.0, 3820.0, 3830.0, 3840.0, 3850.0, 3860.0, 3870.0, 3880.0, 3890.0,
 3900.0, 3910.0, 3920.0, 3930.0, 3940.0, 3950.0, 3960.0, 3970.0, 3980.0, 3990.0,
 4000.0, 4010.0, 4020.0, 4030.0, 4040.0, 4050.0, 4060.0, 4070.0, 4080.0, 4090.0,
 4100.0, 4110.0, 4120.0, 4130.0, 4140.0, 4150.0, 4160.0, 4170.0, 4180.0, 4190.0,
 4200.0, 4210.0, 4220.0, 4230.0, 4240.0, 4250.0, 4260.0, 4270.0, 4280.0, 4290.0,
 4300.0, 4310.0, 4320.0, 4330.0, 4340.0, 4350.0, 4360.0, 4370.0, 4380.0, 4390.0,
 4400.0, 4410.0, 4420.0, 4430.0, 4440.0, 4450.0, 4460.0, 4470.0, 4480.0, 4490.0,
 4500.0, 4510.0, 4520.0, 4530.0, 4540.0, 4550.0, 4560.0, 4570.0, 4580.0, 4590.0,
 4600.0, 4610.0, 4620.0, 4630.0, 4640.0, 4650.0, 4660.0, 4670.0, 4680.0, 4690.0,
 4700.0, 4710.0, 4720.0, 4730.0, 4740.0, 4750.0, 4760.0, 4770.0, 4780.0, 4790.0,
 4800.0, 4810.0, 4820.0, 4830.0, 4840.0, 4850.0, 4860.0, 4870.0, 4880.0, 4890.0,
 4900.0, 4910.0, 4920.0, 4930.0, 4940.0, 4950.0, 4960.0, 4970.0, 4980.0, 4990.0,
 5000.0, 5010.0, 5020.0, 5030.0, 5040.0, 5050.0, 5060.0, 5070.0, 5080.0, 5090.0,
 5100.0, 5110.0, 5120.0, 5130.0, 5140.0, 5150.0, 5160.0, 5170.0, 5180.0, 5190.0,
 5200.0, 5210.0, 5220.0, 5230.0, 5240.0, 5250.0, 5260.0, 5270.0, 5280.0, 5290.0,
 5300.0, 5310.0, 5320.0, 5330.0, 5340.0, 5350.0, 5360.0, 5370.0, 5380.0, 5390.0,
 5400.0, 5410.0, 5420.0, 5430.0, 5440.0, 5450.0, 5460.0, 5470.0, 5480.0, 5490.0,
 5500.0, 5510.0, 5520.0, 5530.0, 5540.0, 5550.0, 5560.0, 5570.0, 5580.0, 5590.0,
 5600.0, 5610.0, 5620.0, 5630.0, 5640.0, 5650.0, 5660.0, 5670.0, 5680.0, 5690.0,
 5700.0, 5710.0, 5720.0, 5730.0, 5740.0, 5750.0, 5760.0, 5770.0, 5780.0, 5790.0,
 5800.0, 5810.0, 5820.0, 5830.0, 5840.0, 5850.0, 5860.0, 5870.0, 5880.0, 5890.0,
 5900.0, 5910.0, 5920.0, 5930.0, 5940.0, 5950.0, 5960.0, 5970.0, 5980.0, 5990.0,
 6000.0, 6010.0, 6020.0, 6030.0, 6040.0, 6050.0, 6060.0, 6070.0, 6080.0, 6090.0,
 6100.0, 6110.0, 6120.0, 6130.0, 6140.0, 6150.0, 6160.0, 6170.0, 6180.0, 6190.0,
 6200.0, 6210.0, 6220.0, 6230.0, 6240.0, 6250.0, 6260.0, 6270.0, 6280.0, 6290.0,
 6300.0, 6310.0, 6320.0, 6330.0, 6340.0, 6350.0, 6360.0, 6370.0, 6380.0, 6390.0,
 6400.0, 6410.0, 6420.0, 6430.0, 6440.0, 6450.0, 6460.0, 6470.0, 6480.0, 6490.0,
 6500.0, 6510.0, 6520.0, 6530.0, 6540.0, 6550.0, 6560.0, 6570.0, 6580.0, 6590.0,
 6600.0, 6610.0, 6620.0, 6630.0, 6640.0, 6650.0, 6660.0, 6670.0, 6680.0, 6690.0,
 6700.0, 6710.0, 6720.0, 6730.0, 6740.0, 6750.0, 6760.0, 6770.0, 6780.0, 6790.0,
 6800.0, 6810.0, 6820.0, 6830.0, 6840.0, 6850.0, 6860.0, 6870.0, 6880.0, 6890.0,
 6900.0, 6910.0, 6920.0, 6930.0, 6940.0, 6950.0, 6960.0, 6970.0, 6980.0, 6990.0,
 7000.0, 7010.0, 7020.0, 7030.0, 7040.0, 7050.0, 7060.0, 7070.0, 7080.0, 7090.0,
 7100.0, 7110.0, 7120.0, 7130.0, 7140.0, 7150.0, 7160.0, 7170.0, 7180.0, 7190.0,
 7200.0, 7210.0, 7220.0, 7230.0, 7240.0, 7250.0, 7260.0, 7270.0, 7280.0, 7290.0,
 7300.0, 7310.0, 7320.0, 7330.0, 7340.0, 7350.0, 7360.0, 7370.0, 7380.0, 7390.0,
 7400.0, 7410.0, 7420.0, 7430.0, 7440.0, 7450.0, 7460.0, 7470.0, 7480.0, 7490.0,
 7500.0,
])
TIPS_2021_ISOT[9] = float64([
    1.0,   10.0,   20.0,   30.0,   40.0,   50.0,   60.0,   70.0,   80.0,   90.0,
  100.0,  110.0,  120.0,  130.0,  140.0,  150.0,  160.0,  170.0,  180.0,  190.0,
  200.0,  210.0,  220.0,  230.0,  240.0,  250.0,  260.0,  270.0,  280.0,  290.0,
  300.0,  310.0,  320.0,  330.0,  340.0,  350.0,  360.0,  370.0,  380.0,  390.0,
  400.0,  410.0,  420.0,  430.0,  440.0,  450.0,  460.0,  470.0,  480.0,  490.0,
  500.0,  510.0,  520.0,  530.0,  540.0,  550.0,  560.0,  570.0,  580.0,  590.0,
  600.0,  610.0,  620.0,  630.0,  640.0,  650.0,  660.0,  670.0,  680.0,  690.0,
  700.0,  710.0,  720.0,  730.0,  740.0,  750.0,  760.0,  770.0,  780.0,  790.0,
  800.0,  810.0,  820.0,  830.0,  840.0,  850.0,  860.0,  870.0,  880.0,  890.0,
  900.0,  910.0,  920.0,  930.0,  940.0,  950.0,  960.0,  970.0,  980.0,  990.0,
 1000.0, 1010.0, 1020.0, 1030.0, 1040.0, 1050.0, 1060.0, 1070.0, 1080.0, 1090.0,
 1100.0, 1110.0, 1120.0, 1130.0, 1140.0, 1150.0, 1160.0, 1170.0, 1180.0, 1190.0,
 1200.0, 1210.0, 1220.0, 1230.0, 1240.0, 1250.0, 1260.0, 1270.0, 1280.0, 1290.0,
 1300.0, 1310.0, 1320.0, 1330.0, 1340.0, 1350.0, 1360.0, 1370.0, 1380.0, 1390.0,
 1400.0, 1410.0, 1420.0, 1430.0, 1440.0, 1450.0, 1460.0, 1470.0, 1480.0, 1490.0,
 1500.0, 1510.0, 1520.0, 1530.0, 1540.0, 1550.0, 1560.0, 1570.0, 1580.0, 1590.0,
 1600.0, 1610.0, 1620.0, 1630.0, 1640.0, 1650.0, 1660.0, 1670.0, 1680.0, 1690.0,
 1700.0, 1710.0, 1720.0, 1730.0, 1740.0, 1750.0, 1760.0, 1770.0, 1780.0, 1790.0,
 1800.0, 1810.0, 1820.0, 1830.0, 1840.0, 1850.0, 1860.0, 1870.0, 1880.0, 1890.0,
 1900.0, 1910.0, 1920.0, 1930.0, 1940.0, 1950.0, 1960.0, 1970.0, 1980.0, 1990.0,
 2000.0, 2010.0, 2020.0, 2030.0, 2040.0, 2050.0, 2060.0, 2070.0, 2080.0, 2090.0,
 2100.0, 2110.0, 2120.0, 2130.0, 2140.0, 2150.0, 2160.0, 2170.0, 2180.0, 2190.0,
 2200.0, 2210.0, 2220.0, 2230.0, 2240.0, 2250.0, 2260.0, 2270.0, 2280.0, 2290.0,
 2300.0, 2310.0, 2320.0, 2330.0, 2340.0, 2350.0, 2360.0, 2370.0, 2380.0, 2390.0,
 2400.0, 2410.0, 2420.0, 2430.0, 2440.0, 2450.0, 2460.0, 2470.0, 2480.0, 2490.0,
 2500.0, 2510.0, 2520.0, 2530.0, 2540.0, 2550.0, 2560.0, 2570.0, 2580.0, 2590.0,
 2600.0, 2610.0, 2620.0, 2630.0, 2640.0, 2650.0, 2660.0, 2670.0, 2680.0, 2690.0,
 2700.0, 2710.0, 2720.0, 2730.0, 2740.0, 2750.0, 2760.0, 2770.0, 2780.0, 2790.0,
 2800.0, 2810.0, 2820.0, 2830.0, 2840.0, 2850.0, 2860.0, 2870.0, 2880.0, 2890.0,
 2900.0, 2910.0, 2920.0, 2930.0, 2940.0, 2950.0, 2960.0, 2970.0, 2980.0, 2990.0,
 3000.0, 3010.0, 3020.0, 3030.0, 3040.0, 3050.0, 3060.0, 3070.0, 3080.0, 3090.0,
 3100.0, 3110.0, 3120.0, 3130.0, 3140.0, 3150.0, 3160.0, 3170.0, 3180.0, 3190.0,
 3200.0, 3210.0, 3220.0, 3230.0, 3240.0, 3250.0, 3260.0, 3270.0, 3280.0, 3290.0,
 3300.0, 3310.0, 3320.0, 3330.0, 3340.0, 3350.0, 3360.0, 3370.0, 3380.0, 3390.0,
 3400.0, 3410.0, 3420.0, 3430.0, 3440.0, 3450.0, 3460.0, 3470.0, 3480.0, 3490.0,
 3500.0, 3510.0, 3520.0, 3530.0, 3540.0, 3550.0, 3560.0, 3570.0, 3580.0, 3590.0,
 3600.0, 3610.0, 3620.0, 3630.0, 3640.0, 3650.0, 3660.0, 3670.0, 3680.0, 3690.0,
 3700.0, 3710.0, 3720.0, 3730.0, 3740.0, 3750.0, 3760.0, 3770.0, 3780.0, 3790.0,
 3800.0, 3810.0, 3820.0, 3830.0, 3840.0, 3850.0, 3860.0, 3870.0, 3880.0, 3890.0,
 3900.0, 3910.0, 3920.0, 3930.0, 3940.0, 3950.0, 3960.0, 3970.0, 3980.0, 3990.0,
 4000.0, 4010.0, 4020.0, 4030.0, 4040.0, 4050.0, 4060.0, 4070.0, 4080.0, 4090.0,
 4100.0, 4110.0, 4120.0, 4130.0, 4140.0, 4150.0, 4160.0, 4170.0, 4180.0, 4190.0,
 4200.0, 4210.0, 4220.0, 4230.0, 4240.0, 4250.0, 4260.0, 4270.0, 4280.0, 4290.0,
 4300.0, 4310.0, 4320.0, 4330.0, 4340.0, 4350.0, 4360.0, 4370.0, 4380.0, 4390.0,
 4400.0, 4410.0, 4420.0, 4430.0, 4440.0, 4450.0, 4460.0, 4470.0, 4480.0, 4490.0,
 4500.0, 4510.0, 4520.0, 4530.0, 4540.0, 4550.0, 4560.0, 4570.0, 4580.0, 4590.0,
 4600.0, 4610.0, 4620.0, 4630.0, 4640.0, 4650.0, 4660.0, 4670.0, 4680.0, 4690.0,
 4700.0, 4710.0, 4720.0, 4730.0, 4740.0, 4750.0, 4760.0, 4770.0, 4780.0, 4790.0,
 4800.0, 4810.0, 4820.0, 4830.0, 4840.0, 4850.0, 4860.0, 4870.0, 4880.0, 4890.0,
 4900.0, 4910.0, 4920.0, 4930.0, 4940.0, 4950.0, 4960.0, 4970.0, 4980.0, 4990.0,
 5000.0, 5010.0, 5020.0, 5030.0, 5040.0, 5050.0, 5060.0, 5070.0, 5080.0, 5090.0,
 5100.0, 5110.0, 5120.0, 5130.0, 5140.0, 5150.0, 5160.0, 5170.0, 5180.0, 5190.0,
 5200.0, 5210.0, 5220.0, 5230.0, 5240.0, 5250.0, 5260.0, 5270.0, 5280.0, 5290.0,
 5300.0, 5310.0, 5320.0, 5330.0, 5340.0, 5350.0, 5360.0, 5370.0, 5380.0, 5390.0,
 5400.0, 5410.0, 5420.0, 5430.0, 5440.0, 5450.0, 5460.0, 5470.0, 5480.0, 5490.0,
 5500.0, 5510.0, 5520.0, 5530.0, 5540.0, 5550.0, 5560.0, 5570.0, 5580.0, 5590.0,
 5600.0, 5610.0, 5620.0, 5630.0, 5640.0, 5650.0, 5660.0, 5670.0, 5680.0, 5690.0,
 5700.0, 5710.0, 5720.0, 5730.0, 5740.0, 5750.0, 5760.0, 5770.0, 5780.0, 5790.0,
 5800.0, 5810.0, 5820.0, 5830.0, 5840.0, 5850.0, 5860.0, 5870.0, 5880.0, 5890.0,
 5900.0, 5910.0, 5920.0, 5930.0, 5940.0, 5950.0, 5960.0, 5970.0, 5980.0, 5990.0,
 6000.0, 6010.0, 6020.0, 6030.0, 6040.0, 6050.0, 6060.0, 6070.0, 6080.0, 6090.0,
 6100.0, 6110.0, 6120.0, 6130.0, 6140.0, 6150.0, 6160.0, 6170.0, 6180.0, 6190.0,
 6200.0, 6210.0, 6220.0, 6230.0, 6240.0, 6250.0, 6260.0, 6270.0, 6280.0, 6290.0,
 6300.0, 6310.0, 6320.0, 6330.0, 6340.0, 6350.0, 6360.0, 6370.0, 6380.0, 6390.0,
 6400.0, 6410.0, 6420.0, 6430.0, 6440.0, 6450.0, 6460.0, 6470.0, 6480.0, 6490.0,
 6500.0, 6510.0, 6520.0, 6530.0, 6540.0, 6550.0, 6560.0, 6570.0, 6580.0, 6590.0,
 6600.0, 6610.0, 6620.0, 6630.0, 6640.0, 6650.0, 6660.0, 6670.0, 6680.0, 6690.0,
 6700.0, 6710.0, 6720.0, 6730.0, 6740.0, 6750.0, 6760.0, 6770.0, 6780.0, 6790.0,
 6800.0, 6810.0, 6820.0, 6830.0, 6840.0, 6850.0, 6860.0, 6870.0, 6880.0, 6890.0,
 6900.0, 6910.0, 6920.0, 6930.0, 6940.0, 6950.0, 6960.0, 6970.0, 6980.0, 6990.0,
 7000.0, 7010.0, 7020.0, 7030.0, 7040.0, 7050.0, 7060.0, 7070.0, 7080.0, 7090.0,
 7100.0, 7110.0, 7120.0, 7130.0, 7140.0, 7150.0, 7160.0, 7170.0, 7180.0, 7190.0,
 7200.0, 7210.0, 7220.0, 7230.0, 7240.0, 7250.0, 7260.0, 7270.0, 7280.0, 7290.0,
 7300.0, 7310.0, 7320.0, 7330.0, 7340.0, 7350.0, 7360.0, 7370.0, 7380.0, 7390.0,
 7400.0, 7410.0, 7420.0, 7430.0, 7440.0, 7450.0, 7460.0, 7470.0, 7480.0, 7490.0,
 7500.0, 7510.0, 7520.0, 7530.0, 7540.0, 7550.0, 7560.0, 7570.0, 7580.0, 7590.0,
 7600.0, 7610.0, 7620.0, 7630.0, 7640.0, 7650.0, 7660.0, 7670.0, 7680.0, 7690.0,
 7700.0, 7710.0, 7720.0, 7730.0, 7740.0, 7750.0, 7760.0, 7770.0, 7780.0, 7790.0,
 7800.0, 7810.0, 7820.0, 7830.0, 7840.0, 7850.0, 7860.0, 7870.0, 7880.0, 7890.0,
 7900.0, 7910.0, 7920.0, 7930.0, 7940.0, 7950.0, 7960.0, 7970.0, 7980.0, 7990.0,
 8000.0, 8010.0, 8020.0, 8030.0, 8040.0, 8050.0, 8060.0, 8070.0, 8080.0, 8090.0,
 8100.0, 8110.0, 8120.0, 8130.0, 8140.0, 8150.0, 8160.0, 8170.0, 8180.0, 8190.0,
 8200.0, 8210.0, 8220.0, 8230.0, 8240.0, 8250.0, 8260.0, 8270.0, 8280.0, 8290.0,
 8300.0, 8310.0, 8320.0, 8330.0, 8340.0, 8350.0, 8360.0, 8370.0, 8380.0, 8390.0,
 8400.0, 8410.0, 8420.0, 8430.0, 8440.0, 8450.0, 8460.0, 8470.0, 8480.0, 8490.0,
 8500.0, 8510.0, 8520.0, 8530.0, 8540.0, 8550.0, 8560.0, 8570.0, 8580.0, 8590.0,
 8600.0, 8610.0, 8620.0, 8630.0, 8640.0, 8650.0, 8660.0, 8670.0, 8680.0, 8690.0,
 8700.0, 8710.0, 8720.0, 8730.0, 8740.0, 8750.0, 8760.0, 8770.0, 8780.0, 8790.0,
 8800.0, 8810.0, 8820.0, 8830.0, 8840.0, 8850.0, 8860.0, 8870.0, 8880.0, 8890.0,
 8900.0, 8910.0, 8920.0, 8930.0, 8940.0, 8950.0, 8960.0, 8970.0, 8980.0, 8990.0,
 9000.0,
])

# ===========================================================================

# ============================ M = 1, I = 1 ============================
M = 1
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.000000E+00, 1.328370E+00, 3.348920E+00, 6.191840E+00, 9.416960E+00, 1.296157E+01, 
   1.681863E+01, 2.097967E+01, 2.543182E+01, 3.016085E+01, 3.515313E+01, 4.039634E+01, 
   4.587953E+01, 5.159304E+01, 5.752827E+01, 6.367755E+01, 7.003401E+01, 7.659140E+01, 
   8.334408E+01, 9.028689E+01, 9.741515E+01, 1.047245E+02, 1.122112E+02, 1.198714E+02, 
   1.277022E+02, 1.357003E+02, 1.438633E+02, 1.521888E+02, 1.606747E+02, 1.693192E+02, 
   1.781206E+02, 1.870777E+02, 1.961892E+02, 2.054543E+02, 2.148721E+02, 2.244423E+02, 
   2.341643E+02, 2.440380E+02, 2.540633E+02, 2.642403E+02, 2.745691E+02, 2.850501E+02, 
   2.956837E+02, 3.064705E+02, 3.174110E+02, 3.285060E+02, 3.397563E+02, 3.511627E+02, 
   3.627261E+02, 3.744476E+02, 3.863281E+02, 3.983689E+02, 4.105710E+02, 4.229357E+02, 
   4.354642E+02, 4.481579E+02, 4.610180E+02, 4.740460E+02, 4.872432E+02, 5.006112E+02, 
   5.141514E+02, 5.278654E+02, 5.417546E+02, 5.558208E+02, 5.700656E+02, 5.844905E+02, 
   5.990973E+02, 6.138878E+02, 6.288636E+02, 6.440267E+02, 6.593787E+02, 6.749216E+02, 
   6.906572E+02, 7.065874E+02, 7.227143E+02, 7.390397E+02, 7.555656E+02, 7.722942E+02, 
   7.892274E+02, 8.063674E+02, 8.237162E+02, 8.412761E+02, 8.590491E+02, 8.770376E+02, 
   8.952437E+02, 9.136697E+02, 9.323179E+02, 9.511905E+02, 9.702901E+02, 9.896188E+02, 
   1.009179E+03, 1.028974E+03, 1.049005E+03, 1.069275E+03, 1.089786E+03, 1.110541E+03, 
   1.131543E+03, 1.152795E+03, 1.174297E+03, 1.196055E+03, 1.218069E+03, 1.240343E+03, 
   1.262880E+03, 1.285682E+03, 1.308752E+03, 1.332092E+03, 1.355706E+03, 1.379596E+03, 
   1.403766E+03, 1.428217E+03, 1.452954E+03, 1.477978E+03, 1.503294E+03, 1.528903E+03, 
   1.554809E+03, 1.581014E+03, 1.607523E+03, 1.634337E+03, 1.661461E+03, 1.688896E+03, 
   1.716647E+03, 1.744716E+03, 1.773106E+03, 1.801821E+03, 1.830864E+03, 1.860239E+03, 
   1.889947E+03, 1.919993E+03, 1.950381E+03, 1.981112E+03, 2.012191E+03, 2.043621E+03, 
   2.075405E+03, 2.107548E+03, 2.140051E+03, 2.172919E+03, 2.206155E+03, 2.239762E+03, 
   2.273745E+03, 2.308106E+03, 2.342850E+03, 2.377979E+03, 2.413498E+03, 2.449409E+03, 
   2.485717E+03, 2.522426E+03, 2.559538E+03, 2.597058E+03, 2.634989E+03, 2.673336E+03, 
   2.712101E+03, 2.751289E+03, 2.790904E+03, 2.830948E+03, 2.871427E+03, 2.912345E+03, 
   2.953704E+03, 2.995508E+03, 3.037763E+03, 3.080472E+03, 3.123638E+03, 3.167266E+03, 
   3.211360E+03, 3.255923E+03, 3.300961E+03, 3.346477E+03, 3.392475E+03, 3.438959E+03, 
   3.485933E+03, 3.533403E+03, 3.581371E+03, 3.629842E+03, 3.678821E+03, 3.728311E+03, 
   3.778318E+03, 3.828844E+03, 3.879895E+03, 3.931476E+03, 3.983589E+03, 4.036241E+03, 
   4.089435E+03, 4.143175E+03, 4.197467E+03, 4.252315E+03, 4.307722E+03, 4.363695E+03, 
   4.420237E+03, 4.477354E+03, 4.535049E+03, 4.593327E+03, 4.652193E+03, 4.711652E+03, 
   4.771709E+03, 4.832368E+03, 4.893634E+03, 4.955512E+03, 5.018006E+03, 5.081122E+03, 
   5.144865E+03, 5.209239E+03, 5.274249E+03, 5.339900E+03, 5.406197E+03, 5.473146E+03, 
   5.540751E+03, 5.609018E+03, 5.677950E+03, 5.747555E+03, 5.817835E+03, 5.888798E+03, 
   5.960448E+03, 6.032790E+03, 6.105829E+03, 6.179572E+03, 6.254022E+03, 6.329185E+03, 
   6.405068E+03, 6.481674E+03, 6.559010E+03, 6.637080E+03, 6.715891E+03, 6.795448E+03, 
   6.875757E+03, 6.956822E+03, 7.038649E+03, 7.121245E+03, 7.204614E+03, 7.288763E+03, 
   7.373696E+03, 7.459420E+03, 7.545940E+03, 7.633263E+03, 7.721393E+03, 7.810338E+03, 
   7.900101E+03, 7.990690E+03, 8.082111E+03, 8.174368E+03, 8.267469E+03, 8.361418E+03, 
   8.456223E+03, 8.551889E+03, 8.648422E+03, 8.745828E+03, 8.844114E+03, 8.943285E+03, 
   9.043348E+03, 9.144309E+03, 9.246173E+03, 9.348948E+03, 9.452640E+03, 9.557254E+03, 
   9.662798E+03, 9.769277E+03, 9.876698E+03, 9.985068E+03, 1.009439E+04, 1.020468E+04, 
   1.031593E+04, 1.042816E+04, 1.054137E+04, 1.065556E+04, 1.077075E+04, 1.088694E+04, 
   1.100414E+04, 1.112235E+04, 1.124158E+04, 1.136184E+04, 1.148314E+04, 1.160547E+04, 
   1.172886E+04, 1.185329E+04, 1.197879E+04, 1.210536E+04, 1.223300E+04, 1.236173E+04, 
   1.249154E+04, 1.262245E+04, 1.275447E+04, 1.288759E+04, 1.302184E+04, 1.315720E+04, 
   1.329370E+04, 1.343134E+04, 1.357012E+04, 1.371006E+04, 1.385116E+04, 1.399342E+04, 
   1.413687E+04, 1.428149E+04, 1.442730E+04, 1.457431E+04, 1.472253E+04, 1.487196E+04, 
   1.502260E+04, 1.517448E+04, 1.532759E+04, 1.548194E+04, 1.563754E+04, 1.579439E+04, 
   1.595251E+04, 1.611191E+04, 1.627258E+04, 1.643454E+04, 1.659780E+04, 1.676236E+04, 
   1.692823E+04, 1.709542E+04, 1.726394E+04, 1.743379E+04, 1.760498E+04, 1.777752E+04, 
   1.795142E+04, 1.812668E+04, 1.830332E+04, 1.848134E+04, 1.866075E+04, 1.884155E+04, 
   1.902377E+04, 1.920739E+04, 1.939243E+04, 1.957891E+04, 1.976682E+04, 1.995618E+04, 
   2.014699E+04, 2.033926E+04, 2.053301E+04, 2.072823E+04, 2.092493E+04, 2.112313E+04, 
   2.132284E+04, 2.152405E+04, 2.172678E+04, 2.193104E+04, 2.213684E+04, 2.234418E+04, 
   2.255307E+04, 2.276353E+04, 2.297555E+04, 2.318914E+04, 2.340433E+04, 2.362111E+04, 
   2.383949E+04, 2.405948E+04, 2.428109E+04, 2.450432E+04, 2.472920E+04, 2.495571E+04, 
   2.518388E+04, 2.541372E+04, 2.564522E+04, 2.587840E+04, 2.611326E+04, 2.634983E+04, 
   2.658809E+04, 2.682807E+04, 2.706977E+04, 2.731320E+04, 2.755836E+04, 2.780528E+04, 
   2.805395E+04, 2.830438E+04, 2.855658E+04, 2.881057E+04, 2.906634E+04, 2.932392E+04, 
   2.958330E+04, 2.984449E+04, 3.010751E+04, 3.037237E+04, 3.063906E+04, 3.090760E+04, 
   3.117801E+04, 3.145028E+04, 3.172442E+04, 3.200045E+04, 3.227838E+04, 3.255820E+04, 
   3.283994E+04, 3.312359E+04, 3.340917E+04, 3.369669E+04, 3.398616E+04, 3.427758E+04, 
   3.457096E+04, 3.486631E+04, 3.516364E+04, 3.546296E+04, 3.576428E+04, 3.606760E+04, 
   3.637294E+04, 3.668030E+04, 3.698969E+04, 3.730112E+04, 3.761461E+04, 3.793014E+04, 
   3.824775E+04, 3.856743E+04, 3.888919E+04, 3.921305E+04, 3.953900E+04, 3.986707E+04, 
   4.019725E+04, 4.052956E+04, 4.086401E+04, 4.120059E+04, 4.153933E+04, 4.188023E+04, 
   4.222330E+04, 4.256855E+04, 4.291598E+04, 4.326561E+04, 4.361744E+04, 4.397148E+04, 
   4.432774E+04, 4.468624E+04, 4.504696E+04, 4.540994E+04, 4.577516E+04, 4.614265E+04, 
   4.651241E+04, 4.688445E+04, 4.725878E+04, 4.763540E+04, 4.801433E+04, 4.839556E+04, 
   4.877912E+04, 4.916501E+04, 4.955323E+04, 4.994381E+04, 5.033673E+04, 5.073202E+04, 
   5.112967E+04, 5.152971E+04, 5.193213E+04, 5.233694E+04, 5.274416E+04, 5.315379E+04, 
   5.356584E+04, 5.398031E+04, 5.439722E+04, 5.481658E+04, 5.523838E+04, 5.566264E+04, 
   5.608937E+04, 5.651858E+04, 5.695027E+04, 5.738445E+04, 5.782113E+04, 5.826031E+04, 
   5.870201E+04, 5.914623E+04, 5.959298E+04, 6.004227E+04, 6.049411E+04, 6.094850E+04, 
   6.140545E+04, 6.186496E+04, 6.232706E+04, 6.279173E+04, 6.325900E+04, 6.372887E+04, 
   6.420135E+04, 6.467644E+04, 6.515415E+04, 6.563449E+04, 6.611746E+04, 6.660308E+04, 
   6.709136E+04, 6.758229E+04, 6.807588E+04, 6.857215E+04, 6.907110E+04, 6.957274E+04, 
   7.007707E+04, 7.058411E+04, 7.109385E+04, 7.160631E+04, 7.212149E+04, 7.263941E+04, 
   7.316006E+04, 7.368346E+04, 7.420960E+04, 7.473851E+04, 7.527018E+04, 7.580463E+04, 
   7.634185E+04, 7.688186E+04, 7.742466E+04, 7.797026E+04, 7.851867E+04, 7.906989E+04, 
   7.962393E+04, 8.018080E+04, 8.074050E+04, 8.130304E+04, 8.186842E+04, 8.243666E+04, 
   8.300775E+04, 8.358171E+04, 8.415854E+04, 
])
# ============================ M = 1, I = 2 ============================
M = 1
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.000000E+00, 1.331920E+00, 3.372790E+00, 6.240440E+00, 9.492280E+00, 1.306630E+01, 
   1.695557E+01, 2.115149E+01, 2.564104E+01, 3.040983E+01, 3.544414E+01, 4.073150E+01, 
   4.626091E+01, 5.202258E+01, 5.800788E+01, 6.420905E+01, 7.061916E+01, 7.723191E+01, 
   8.404163E+01, 9.104310E+01, 9.823160E+01, 1.056028E+02, 1.131528E+02, 1.208779E+02, 
   1.287749E+02, 1.368409E+02, 1.450731E+02, 1.534693E+02, 1.620274E+02, 1.707455E+02, 
   1.796219E+02, 1.886555E+02, 1.978450E+02, 2.071895E+02, 2.166883E+02, 2.263409E+02, 
   2.361468E+02, 2.461060E+02, 2.562183E+02, 2.664838E+02, 2.769027E+02, 2.874755E+02, 
   2.982024E+02, 3.090842E+02, 3.201213E+02, 3.313146E+02, 3.426649E+02, 3.541729E+02, 
   3.658398E+02, 3.776665E+02, 3.896541E+02, 4.018037E+02, 4.141165E+02, 4.265937E+02, 
   4.392366E+02, 4.520466E+02, 4.650249E+02, 4.781730E+02, 4.914924E+02, 5.049845E+02, 
   5.186508E+02, 5.324929E+02, 5.465124E+02, 5.607108E+02, 5.750899E+02, 5.896512E+02, 
   6.043966E+02, 6.193277E+02, 6.344463E+02, 6.497542E+02, 6.652532E+02, 6.809452E+02, 
   6.968322E+02, 7.129159E+02, 7.291983E+02, 7.456814E+02, 7.623672E+02, 7.792578E+02, 
   7.963551E+02, 8.136612E+02, 8.311783E+02, 8.489084E+02, 8.668538E+02, 8.850166E+02, 
   9.033990E+02, 9.220032E+02, 9.408315E+02, 9.598862E+02, 9.791695E+02, 9.986839E+02, 
   1.018432E+03, 1.038415E+03, 1.058637E+03, 1.079099E+03, 1.099804E+03, 1.120754E+03, 
   1.141952E+03, 1.163401E+03, 1.185103E+03, 1.207060E+03, 1.229276E+03, 1.251751E+03, 
   1.274491E+03, 1.297496E+03, 1.320770E+03, 1.344314E+03, 1.368133E+03, 1.392229E+03, 
   1.416603E+03, 1.441260E+03, 1.466202E+03, 1.491431E+03, 1.516951E+03, 1.542764E+03, 
   1.568873E+03, 1.595282E+03, 1.621992E+03, 1.649006E+03, 1.676329E+03, 1.703961E+03, 
   1.731908E+03, 1.760171E+03, 1.788753E+03, 1.817658E+03, 1.846888E+03, 1.876447E+03, 
   1.906337E+03, 1.936562E+03, 1.967124E+03, 1.998028E+03, 2.029275E+03, 2.060869E+03, 
   2.092814E+03, 2.125112E+03, 2.157766E+03, 2.190780E+03, 2.224157E+03, 2.257900E+03, 
   2.292012E+03, 2.326498E+03, 2.361359E+03, 2.396599E+03, 2.432222E+03, 2.468231E+03, 
   2.504629E+03, 2.541420E+03, 2.578606E+03, 2.616192E+03, 2.654181E+03, 2.692576E+03, 
   2.731380E+03, 2.770598E+03, 2.810232E+03, 2.850286E+03, 2.890764E+03, 2.931669E+03, 
   2.973005E+03, 3.014775E+03, 3.056982E+03, 3.099631E+03, 3.142725E+03, 3.186267E+03, 
   3.230262E+03, 3.274713E+03, 3.319623E+03, 3.364996E+03, 3.410837E+03, 3.457148E+03, 
   3.503933E+03, 3.551197E+03, 3.598943E+03, 3.647174E+03, 3.695895E+03, 3.745110E+03, 
   3.794822E+03, 3.845035E+03, 3.895752E+03, 3.946979E+03, 3.998719E+03, 4.050975E+03, 
   4.103752E+03, 4.157054E+03, 4.210885E+03, 4.265248E+03, 4.320147E+03, 4.375588E+03, 
   4.431573E+03, 4.488107E+03, 4.545194E+03, 4.602839E+03, 4.661044E+03, 4.719815E+03, 
   4.779155E+03, 4.839069E+03, 4.899561E+03, 4.960635E+03, 5.022295E+03, 5.084546E+03, 
   5.147392E+03, 5.210837E+03, 5.274886E+03, 5.339542E+03, 5.404811E+03, 5.470696E+03, 
   5.537202E+03, 5.604333E+03, 5.672095E+03, 5.740490E+03, 5.809525E+03, 5.879202E+03, 
   5.949527E+03, 6.020504E+03, 6.092139E+03, 6.164434E+03, 6.237395E+03, 6.311027E+03, 
   6.385334E+03, 6.460321E+03, 6.535992E+03, 6.612352E+03, 6.689406E+03, 6.767159E+03, 
   6.845614E+03, 6.924778E+03, 7.004655E+03, 7.085249E+03, 7.166565E+03, 7.248609E+03, 
   7.331385E+03, 7.414898E+03, 7.499153E+03, 7.584154E+03, 7.669907E+03, 7.756417E+03, 
   7.843689E+03, 7.931727E+03, 8.020537E+03, 8.110123E+03, 8.200491E+03, 8.291647E+03, 
   8.383594E+03, 8.476338E+03, 8.569884E+03, 8.664238E+03, 8.759404E+03, 8.855388E+03, 
   8.952195E+03, 9.049830E+03, 9.148299E+03, 9.247606E+03, 9.347757E+03, 9.448758E+03, 
   9.550614E+03, 9.653329E+03, 9.756911E+03, 9.861363E+03, 9.966691E+03, 1.007290E+04, 
   1.018000E+04, 1.028799E+04, 1.039688E+04, 1.050667E+04, 1.061737E+04, 1.072899E+04, 
   1.084152E+04, 1.095499E+04, 1.106938E+04, 1.118471E+04, 1.130099E+04, 1.141821E+04, 
   1.153639E+04, 1.165553E+04, 1.177564E+04, 1.189671E+04, 1.201877E+04, 1.214181E+04, 
   1.226583E+04, 1.239086E+04, 1.251688E+04, 1.264391E+04, 1.277195E+04, 1.290101E+04, 
   1.303110E+04, 1.316222E+04, 1.329438E+04, 1.342757E+04, 1.356182E+04, 1.369712E+04, 
   1.383348E+04, 1.397091E+04, 1.410942E+04, 1.424900E+04, 1.438967E+04, 1.453143E+04, 
   1.467429E+04, 1.481825E+04, 1.496333E+04, 1.510952E+04, 1.525683E+04, 1.540528E+04, 
   1.555486E+04, 1.570558E+04, 1.585745E+04, 1.601048E+04, 1.616467E+04, 1.632002E+04, 
   1.647655E+04, 1.663426E+04, 1.679316E+04, 1.695326E+04, 1.711455E+04, 1.727705E+04, 
   1.744077E+04, 1.760570E+04, 1.777186E+04, 1.793926E+04, 1.810790E+04, 1.827778E+04, 
   1.844892E+04, 1.862131E+04, 1.879498E+04, 1.896992E+04, 1.914614E+04, 1.932364E+04, 
   1.950245E+04, 1.968255E+04, 1.986396E+04, 2.004669E+04, 2.023074E+04, 2.041611E+04, 
   2.060283E+04, 2.079089E+04, 2.098029E+04, 2.117105E+04, 2.136318E+04, 2.155668E+04, 
   2.175156E+04, 2.194782E+04, 2.214547E+04, 2.234452E+04, 2.254498E+04, 2.274686E+04, 
   2.295015E+04, 2.315487E+04, 2.336103E+04, 2.356863E+04, 2.377768E+04, 2.398818E+04, 
   2.420015E+04, 2.441359E+04, 2.462851E+04, 2.484492E+04, 2.506282E+04, 2.528222E+04, 
   2.550313E+04, 2.572555E+04, 2.594950E+04, 2.617498E+04, 2.640200E+04, 2.663056E+04, 
   2.686067E+04, 2.709235E+04, 2.732559E+04, 2.756041E+04, 2.779681E+04, 2.803480E+04, 
   2.827439E+04, 2.851559E+04, 2.875840E+04, 2.900283E+04, 2.924889E+04, 2.949658E+04, 
   2.974592E+04, 2.999691E+04, 3.024956E+04, 3.050388E+04, 3.075987E+04, 3.101754E+04, 
   3.127691E+04, 3.153797E+04, 3.180074E+04, 3.206522E+04, 3.233143E+04, 3.259936E+04, 
   3.286903E+04, 3.314045E+04, 3.341361E+04, 3.368854E+04, 3.396524E+04, 3.424371E+04, 
   3.452397E+04, 3.480602E+04, 3.508988E+04, 3.537554E+04, 3.566301E+04, 3.595231E+04, 
   3.624345E+04, 3.653642E+04, 3.683124E+04, 3.712792E+04, 3.742646E+04, 3.772687E+04, 
   3.802917E+04, 3.833335E+04, 3.863943E+04, 3.894741E+04, 3.925731E+04, 3.956913E+04, 
   3.988288E+04, 4.019856E+04, 4.051619E+04, 4.083577E+04, 4.115732E+04, 4.148083E+04, 
   4.180633E+04, 4.213380E+04, 4.246328E+04, 4.279475E+04, 4.312824E+04, 4.346374E+04, 
   4.380128E+04, 4.414084E+04, 4.448245E+04, 4.482612E+04, 4.517184E+04, 4.551963E+04, 
   4.586950E+04, 4.622146E+04, 4.657550E+04, 4.693165E+04, 4.728991E+04, 4.765029E+04, 
   4.801279E+04, 4.837743E+04, 4.874421E+04, 4.911314E+04, 4.948423E+04, 4.985749E+04, 
   5.023293E+04, 5.061055E+04, 5.099037E+04, 5.137238E+04, 5.175661E+04, 5.214305E+04, 
   5.253172E+04, 5.292263E+04, 5.331578E+04, 5.371118E+04, 5.410884E+04, 5.450877E+04, 
   5.491098E+04, 5.531547E+04, 5.572226E+04, 5.613135E+04, 5.654275E+04, 5.695646E+04, 
   5.737251E+04, 5.779089E+04, 5.821161E+04, 5.863469E+04, 5.906013E+04, 5.948794E+04, 
   5.991812E+04, 6.035070E+04, 6.078566E+04, 6.122303E+04, 6.166281E+04, 6.210501E+04, 
   6.254964E+04, 6.299671E+04, 6.344622E+04, 6.389819E+04, 6.435261E+04, 6.480951E+04, 
   6.526889E+04, 6.573075E+04, 6.619511E+04, 6.666197E+04, 6.713135E+04, 6.760324E+04, 
   6.807767E+04, 6.855463E+04, 6.903414E+04, 6.951620E+04, 7.000083E+04, 7.048803E+04, 
   7.097781E+04, 7.147017E+04, 7.196513E+04, 7.246270E+04, 7.296288E+04, 7.346568E+04, 
   7.397111E+04, 7.447918E+04, 7.498990E+04, 7.550327E+04, 7.601930E+04, 7.653801E+04, 
   7.705940E+04, 7.758347E+04, 7.811024E+04, 
])
# ============================ M = 1, I = 3 ============================
M = 1
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.000000E+00, 7.981410E+00, 2.016901E+01, 3.730489E+01, 5.674024E+01, 7.810101E+01, 
   1.013453E+02, 1.264220E+02, 1.532533E+02, 1.817534E+02, 2.118401E+02, 2.434392E+02, 
   2.764846E+02, 3.109182E+02, 3.466880E+02, 3.837480E+02, 4.220566E+02, 4.615762E+02, 
   5.022729E+02, 5.441155E+02, 5.870757E+02, 6.311277E+02, 6.762480E+02, 7.224149E+02, 
   7.696091E+02, 8.178127E+02, 8.670100E+02, 9.171868E+02, 9.683305E+02, 1.020430E+03, 
   1.073476E+03, 1.127460E+03, 1.182375E+03, 1.238216E+03, 1.294979E+03, 1.352659E+03, 
   1.411255E+03, 1.470765E+03, 1.531189E+03, 1.592527E+03, 1.654781E+03, 1.717952E+03, 
   1.782042E+03, 1.847055E+03, 1.912994E+03, 1.979864E+03, 2.047669E+03, 2.116414E+03, 
   2.186104E+03, 2.256747E+03, 2.328346E+03, 2.400911E+03, 2.474446E+03, 2.548959E+03, 
   2.624459E+03, 2.700951E+03, 2.778445E+03, 2.856948E+03, 2.936469E+03, 3.017017E+03, 
   3.098600E+03, 3.181227E+03, 3.264908E+03, 3.349652E+03, 3.435470E+03, 3.522369E+03, 
   3.610362E+03, 3.699457E+03, 3.789666E+03, 3.880999E+03, 3.973467E+03, 4.067081E+03, 
   4.161852E+03, 4.257790E+03, 4.354909E+03, 4.453219E+03, 4.552733E+03, 4.653462E+03, 
   4.755418E+03, 4.858614E+03, 4.963062E+03, 5.068776E+03, 5.175767E+03, 5.284049E+03, 
   5.393635E+03, 5.504538E+03, 5.616772E+03, 5.730350E+03, 5.845286E+03, 5.961594E+03, 
   6.079288E+03, 6.198382E+03, 6.318891E+03, 6.440828E+03, 6.564209E+03, 6.689048E+03, 
   6.815361E+03, 6.943161E+03, 7.072464E+03, 7.203286E+03, 7.335642E+03, 7.469548E+03, 
   7.605018E+03, 7.742069E+03, 7.880717E+03, 8.020977E+03, 8.162866E+03, 8.306401E+03, 
   8.451596E+03, 8.598470E+03, 8.747038E+03, 8.897317E+03, 9.049324E+03, 9.203076E+03, 
   9.358591E+03, 9.515884E+03, 9.674973E+03, 9.835877E+03, 9.998611E+03, 1.016319E+04, 
   1.032964E+04, 1.049798E+04, 1.066821E+04, 1.084037E+04, 1.101447E+04, 1.119052E+04, 
   1.136854E+04, 1.154856E+04, 1.173059E+04, 1.191465E+04, 1.210077E+04, 1.228894E+04, 
   1.247921E+04, 1.267158E+04, 1.286608E+04, 1.306272E+04, 1.326152E+04, 1.346251E+04, 
   1.366570E+04, 1.387111E+04, 1.407876E+04, 1.428868E+04, 1.450087E+04, 1.471537E+04, 
   1.493219E+04, 1.515135E+04, 1.537288E+04, 1.559679E+04, 1.582310E+04, 1.605183E+04, 
   1.628301E+04, 1.651665E+04, 1.675279E+04, 1.699142E+04, 1.723259E+04, 1.747631E+04, 
   1.772260E+04, 1.797148E+04, 1.822297E+04, 1.847710E+04, 1.873389E+04, 1.899336E+04, 
   1.925553E+04, 1.952042E+04, 1.978806E+04, 2.005846E+04, 2.033166E+04, 2.060767E+04, 
   2.088651E+04, 2.116821E+04, 2.145280E+04, 2.174028E+04, 2.203069E+04, 2.232406E+04, 
   2.262039E+04, 2.291972E+04, 2.322207E+04, 2.352747E+04, 2.383593E+04, 2.414748E+04, 
   2.446215E+04, 2.477995E+04, 2.510092E+04, 2.542507E+04, 2.575244E+04, 2.608304E+04, 
   2.641690E+04, 2.675404E+04, 2.709449E+04, 2.743828E+04, 2.778543E+04, 2.813596E+04, 
   2.848989E+04, 2.884727E+04, 2.920810E+04, 2.957242E+04, 2.994025E+04, 3.031162E+04, 
   3.068655E+04, 3.106506E+04, 3.144720E+04, 3.183297E+04, 3.222241E+04, 3.261555E+04, 
   3.301240E+04, 3.341300E+04, 3.381738E+04, 3.422555E+04, 3.463755E+04, 3.505341E+04, 
   3.547315E+04, 3.589680E+04, 3.632438E+04, 3.675593E+04, 3.719147E+04, 3.763103E+04, 
   3.807464E+04, 3.852233E+04, 3.897411E+04, 3.943003E+04, 3.989012E+04, 4.035439E+04, 
   4.082287E+04, 4.129561E+04, 4.177262E+04, 4.225394E+04, 4.273959E+04, 4.322960E+04, 
   4.372400E+04, 4.422283E+04, 4.472611E+04, 4.523387E+04, 4.574614E+04, 4.626295E+04, 
   4.678433E+04, 4.731032E+04, 4.784094E+04, 4.837621E+04, 4.891619E+04, 4.946088E+04, 
   5.001034E+04, 5.056457E+04, 5.112363E+04, 5.168753E+04, 5.225631E+04, 5.283000E+04, 
   5.340864E+04, 5.399224E+04, 5.458086E+04, 5.517451E+04, 5.577324E+04, 5.637706E+04, 
   5.698602E+04, 5.760015E+04, 5.821948E+04, 5.884404E+04, 5.947386E+04, 6.010899E+04, 
   6.074944E+04, 6.139527E+04, 6.204648E+04, 6.270314E+04, 6.336525E+04, 6.403287E+04, 
   6.470602E+04, 6.538473E+04, 6.606905E+04, 6.675900E+04, 6.745462E+04, 6.815595E+04, 
   6.886301E+04, 6.957585E+04, 7.029450E+04, 7.101899E+04, 7.174936E+04, 7.248564E+04, 
   7.322787E+04, 7.397609E+04, 7.473032E+04, 7.549062E+04, 7.625700E+04, 7.702952E+04, 
   7.780820E+04, 7.859308E+04, 7.938419E+04, 8.018158E+04, 8.098528E+04, 8.179533E+04, 
   8.261176E+04, 8.343461E+04, 8.426392E+04, 8.509972E+04, 8.594206E+04, 8.679096E+04, 
   8.764648E+04, 8.850864E+04, 8.937748E+04, 9.025305E+04, 9.113537E+04, 9.202449E+04, 
   9.292045E+04, 9.382328E+04, 9.473303E+04, 9.564973E+04, 9.657342E+04, 9.750414E+04, 
   9.844193E+04, 9.938683E+04, 1.003389E+05, 1.012981E+05, 1.022646E+05, 1.032383E+05, 
   1.042193E+05, 1.052077E+05, 1.062035E+05, 1.072067E+05, 1.082173E+05, 1.092355E+05, 
   1.102612E+05, 1.112945E+05, 1.123354E+05, 1.133840E+05, 1.144403E+05, 1.155044E+05, 
   1.165763E+05, 1.176559E+05, 1.187435E+05, 1.198390E+05, 1.209425E+05, 1.220539E+05, 
   1.231734E+05, 1.243010E+05, 1.254367E+05, 1.265806E+05, 1.277326E+05, 1.288930E+05, 
   1.300616E+05, 1.312386E+05, 1.324240E+05, 1.336178E+05, 1.348200E+05, 1.360308E+05, 
   1.372501E+05, 1.384780E+05, 1.397146E+05, 1.409599E+05, 1.422138E+05, 1.434766E+05, 
   1.447482E+05, 1.460286E+05, 1.473179E+05, 1.486162E+05, 1.499235E+05, 1.512398E+05, 
   1.525652E+05, 1.538998E+05, 1.552435E+05, 1.565964E+05, 1.579586E+05, 1.593301E+05, 
   1.607109E+05, 1.621011E+05, 1.635008E+05, 1.649100E+05, 1.663287E+05, 1.677569E+05, 
   1.691948E+05, 1.706424E+05, 1.720997E+05, 1.735667E+05, 1.750436E+05, 1.765303E+05, 
   1.780269E+05, 1.795334E+05, 1.810500E+05, 1.825766E+05, 1.841132E+05, 1.856600E+05, 
   1.872170E+05, 1.887842E+05, 1.903617E+05, 1.919495E+05, 1.935477E+05, 1.951562E+05, 
   1.967753E+05, 1.984048E+05, 2.000449E+05, 2.016956E+05, 2.033570E+05, 2.050290E+05, 
   2.067118E+05, 2.084054E+05, 2.101098E+05, 2.118251E+05, 2.135514E+05, 2.152886E+05, 
   2.170369E+05, 2.187962E+05, 2.205667E+05, 2.223483E+05, 2.241412E+05, 2.259454E+05, 
   2.277608E+05, 2.295877E+05, 2.314260E+05, 2.332757E+05, 2.351369E+05, 2.370097E+05, 
   2.388942E+05, 2.407903E+05, 2.426981E+05, 2.446176E+05, 2.465490E+05, 2.484922E+05, 
   2.504474E+05, 2.524145E+05, 2.543936E+05, 2.563847E+05, 2.583880E+05, 2.604034E+05, 
   2.624310E+05, 2.644709E+05, 2.665231E+05, 2.685876E+05, 2.706645E+05, 2.727539E+05, 
   2.748558E+05, 2.769702E+05, 2.790973E+05, 2.812370E+05, 2.833893E+05, 2.855545E+05, 
   2.877324E+05, 2.899232E+05, 2.921269E+05, 2.943435E+05, 2.965732E+05, 2.988159E+05, 
   3.010717E+05, 3.033406E+05, 3.056228E+05, 3.079181E+05, 3.102268E+05, 3.125489E+05, 
   3.148843E+05, 3.172332E+05, 3.195956E+05, 3.219716E+05, 3.243612E+05, 3.267644E+05, 
   3.291813E+05, 3.316120E+05, 3.340565E+05, 3.365148E+05, 3.389870E+05, 3.414732E+05, 
   3.439734E+05, 3.464877E+05, 3.490161E+05, 3.515586E+05, 3.541153E+05, 3.566863E+05, 
   3.592716E+05, 3.618713E+05, 3.644854E+05, 3.671139E+05, 3.697570E+05, 3.724146E+05, 
   3.750868E+05, 3.777737E+05, 3.804753E+05, 3.831917E+05, 3.859229E+05, 3.886690E+05, 
   3.914300E+05, 3.942059E+05, 3.969969E+05, 3.998029E+05, 4.026240E+05, 4.054604E+05, 
   4.083119E+05, 4.111787E+05, 4.140609E+05, 4.169584E+05, 4.198713E+05, 4.227997E+05, 
   4.257436E+05, 4.287031E+05, 4.316783E+05, 4.346691E+05, 4.376756E+05, 4.406979E+05, 
   4.437360E+05, 4.467900E+05, 4.498599E+05, 4.529458E+05, 4.560478E+05, 4.591658E+05, 
   4.622999E+05, 4.654502E+05, 4.686168E+05, 
])
# ============================ M = 1, I = 4 ============================
M = 1
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.000000E+00, 8.396180E+00, 1.766833E+01, 3.050885E+01, 4.570241E+01, 6.285970E+01, 
   8.177074E+01, 1.022865E+02, 1.242917E+02, 1.476933E+02, 1.724141E+02, 1.983889E+02, 
   2.255614E+02, 2.538827E+02, 2.833095E+02, 3.138033E+02, 3.453297E+02, 3.778577E+02, 
   4.113595E+02, 4.458098E+02, 4.811860E+02, 5.174676E+02, 5.546366E+02, 5.926766E+02, 
   6.315735E+02, 6.713149E+02, 7.118902E+02, 7.532904E+02, 7.955081E+02, 8.385373E+02, 
   8.823735E+02, 9.270133E+02, 9.724548E+02, 1.018697E+03, 1.065740E+03, 1.113584E+03, 
   1.162232E+03, 1.211687E+03, 1.261951E+03, 1.313030E+03, 1.364927E+03, 1.417648E+03, 
   1.471199E+03, 1.525587E+03, 1.580818E+03, 1.636899E+03, 1.693839E+03, 1.751645E+03, 
   1.810326E+03, 1.869890E+03, 1.930347E+03, 1.991706E+03, 2.053977E+03, 2.117169E+03, 
   2.181293E+03, 2.246359E+03, 2.312379E+03, 2.379362E+03, 2.447320E+03, 2.516265E+03, 
   2.586208E+03, 2.657161E+03, 2.729136E+03, 2.802145E+03, 2.876201E+03, 2.951316E+03, 
   3.027504E+03, 3.104777E+03, 3.183148E+03, 3.262631E+03, 3.343240E+03, 3.424989E+03, 
   3.507890E+03, 3.591960E+03, 3.677211E+03, 3.763659E+03, 3.851318E+03, 3.940203E+03, 
   4.030329E+03, 4.121711E+03, 4.214365E+03, 4.308307E+03, 4.403551E+03, 4.500114E+03, 
   4.598012E+03, 4.697261E+03, 4.797878E+03, 4.899878E+03, 5.003279E+03, 5.108098E+03, 
   5.214351E+03, 5.322056E+03, 5.431230E+03, 5.541890E+03, 5.654054E+03, 5.767740E+03, 
   5.882966E+03, 5.999749E+03, 6.118108E+03, 6.238062E+03, 6.359628E+03, 6.482826E+03, 
   6.607674E+03, 6.734191E+03, 6.862397E+03, 6.992310E+03, 7.123950E+03, 7.257337E+03, 
   7.392489E+03, 7.529428E+03, 7.668172E+03, 7.808743E+03, 7.951159E+03, 8.095443E+03, 
   8.241613E+03, 8.389692E+03, 8.539699E+03, 8.691656E+03, 8.845584E+03, 9.001504E+03, 
   9.159437E+03, 9.319406E+03, 9.481431E+03, 9.645535E+03, 9.811739E+03, 9.980067E+03, 
   1.015054E+04, 1.032318E+04, 1.049801E+04, 1.067505E+04, 1.085433E+04, 1.103587E+04, 
   1.121969E+04, 1.140581E+04, 1.159426E+04, 1.178507E+04, 1.197825E+04, 1.217383E+04, 
   1.237183E+04, 1.257229E+04, 1.277521E+04, 1.298064E+04, 1.318858E+04, 1.339907E+04, 
   1.361213E+04, 1.382779E+04, 1.404607E+04, 1.426700E+04, 1.449060E+04, 1.471690E+04, 
   1.494593E+04, 1.517770E+04, 1.541225E+04, 1.564961E+04, 1.588979E+04, 1.613283E+04, 
   1.637875E+04, 1.662759E+04, 1.687936E+04, 1.713409E+04, 1.739182E+04, 1.765256E+04, 
   1.791635E+04, 1.818322E+04, 1.845319E+04, 1.872628E+04, 1.900254E+04, 1.928199E+04, 
   1.956465E+04, 1.985056E+04, 2.013974E+04, 2.043222E+04, 2.072804E+04, 2.102722E+04, 
   2.132979E+04, 2.163578E+04, 2.194522E+04, 2.225814E+04, 2.257457E+04, 2.289454E+04, 
   2.321808E+04, 2.354523E+04, 2.387600E+04, 2.421045E+04, 2.454858E+04, 2.489044E+04, 
   2.523606E+04, 2.558546E+04, 2.593868E+04, 2.629576E+04, 2.665672E+04, 2.702159E+04, 
   2.739041E+04, 2.776321E+04, 2.814002E+04, 2.852087E+04, 2.890581E+04, 2.929485E+04, 
   2.968803E+04, 3.008539E+04, 3.048695E+04, 3.089276E+04, 3.130284E+04, 3.171723E+04, 
   3.213596E+04, 3.255907E+04, 3.298659E+04, 3.341855E+04, 3.385498E+04, 3.429593E+04, 
   3.474142E+04, 3.519150E+04, 3.564619E+04, 3.610552E+04, 3.656955E+04, 3.703829E+04, 
   3.751178E+04, 3.799006E+04, 3.847316E+04, 3.896113E+04, 3.945398E+04, 3.995177E+04, 
   4.045452E+04, 4.096227E+04, 4.147505E+04, 4.199291E+04, 4.251587E+04, 4.304397E+04, 
   4.357726E+04, 4.411575E+04, 4.465950E+04, 4.520854E+04, 4.576289E+04, 4.632261E+04, 
   4.688772E+04, 4.745826E+04, 4.803426E+04, 4.861577E+04, 4.920283E+04, 4.979545E+04, 
   5.039369E+04, 5.099758E+04, 5.160716E+04, 5.222246E+04, 5.284352E+04, 5.347038E+04, 
   5.410306E+04, 5.474162E+04, 5.538609E+04, 5.603649E+04, 5.669288E+04, 5.735528E+04, 
   5.802374E+04, 5.869829E+04, 5.937896E+04, 6.006580E+04, 6.075884E+04, 6.145812E+04, 
   6.216367E+04, 6.287553E+04, 6.359374E+04, 6.431833E+04, 6.504935E+04, 6.578683E+04, 
   6.653080E+04, 6.728130E+04, 6.803837E+04, 6.880204E+04, 6.957236E+04, 7.034936E+04, 
   7.113307E+04, 7.192353E+04, 7.272078E+04, 7.352485E+04, 7.433578E+04, 7.515361E+04, 
   7.597838E+04, 7.681011E+04, 7.764885E+04, 7.849463E+04, 7.934749E+04, 8.020746E+04, 
   8.107459E+04, 8.194889E+04, 8.283042E+04, 8.371921E+04, 8.461529E+04, 8.551870E+04, 
   8.642947E+04, 8.734763E+04, 8.827324E+04, 8.920631E+04, 9.014689E+04, 9.109501E+04, 
   9.205070E+04, 9.301400E+04, 9.398494E+04, 9.496357E+04, 9.594990E+04, 9.694399E+04, 
   9.794585E+04, 9.895554E+04, 9.997307E+04, 1.009985E+05, 1.020318E+05, 1.030731E+05, 
   1.041224E+05, 1.051797E+05, 1.062450E+05, 1.073184E+05, 1.084000E+05, 1.094897E+05, 
   1.105876E+05, 1.116937E+05, 1.128080E+05, 1.139306E+05, 1.150616E+05, 1.162009E+05, 
   1.173485E+05, 1.185046E+05, 1.196691E+05, 1.208421E+05, 1.220236E+05, 1.232136E+05, 
   1.244122E+05, 1.256194E+05, 1.268353E+05, 1.280597E+05, 1.292929E+05, 1.305348E+05, 
   1.317854E+05, 1.330449E+05, 1.343131E+05, 1.355901E+05, 1.368761E+05, 1.381709E+05, 
   1.394747E+05, 1.407874E+05, 1.421091E+05, 1.434398E+05, 1.447796E+05, 1.461284E+05, 
   1.474863E+05, 1.488533E+05, 1.502295E+05, 1.516149E+05, 1.530095E+05, 1.544133E+05, 
   1.558264E+05, 1.572487E+05, 1.586804E+05, 1.601214E+05, 1.615718E+05, 1.630316E+05, 
   1.645008E+05, 1.659794E+05, 1.674676E+05, 1.689652E+05, 1.704723E+05, 1.719889E+05, 
   1.735152E+05, 1.750510E+05, 1.765964E+05, 1.781515E+05, 1.797163E+05, 1.812907E+05, 
   1.828748E+05, 1.844687E+05, 1.860723E+05, 1.876857E+05, 1.893090E+05, 1.909420E+05, 
   1.925849E+05, 1.942376E+05, 1.959003E+05, 1.975728E+05, 1.992553E+05, 2.009477E+05, 
   2.026501E+05, 2.043625E+05, 2.060849E+05, 2.078174E+05, 2.095599E+05, 2.113125E+05, 
   2.130751E+05, 2.148479E+05, 2.166308E+05, 2.184239E+05, 2.202271E+05, 2.220405E+05, 
   2.238641E+05, 2.256980E+05, 2.275421E+05, 2.293964E+05, 2.312610E+05, 2.331359E+05, 
   2.350212E+05, 2.369167E+05, 2.388226E+05, 2.407388E+05, 2.426655E+05, 2.446025E+05, 
   2.465499E+05, 2.485077E+05, 2.504760E+05, 2.524547E+05, 2.544439E+05, 2.564436E+05, 
   2.584538E+05, 2.604745E+05, 2.625057E+05, 2.645474E+05, 2.665997E+05, 2.686625E+05, 
   2.707359E+05, 2.728199E+05, 2.749145E+05, 2.770198E+05, 2.791356E+05, 2.812621E+05, 
   2.833992E+05, 2.855470E+05, 2.877055E+05, 2.898746E+05, 2.920544E+05, 2.942450E+05, 
   2.964462E+05, 2.986582E+05, 3.008809E+05, 3.031144E+05, 3.053586E+05, 3.076135E+05, 
   3.098792E+05, 3.121558E+05, 3.144431E+05, 3.167411E+05, 3.190500E+05, 3.213698E+05, 
   3.237003E+05, 3.260417E+05, 3.283938E+05, 3.307569E+05, 3.331308E+05, 3.355155E+05, 
   3.379111E+05, 3.403176E+05, 3.427350E+05, 3.451632E+05, 3.476023E+05, 3.500523E+05, 
   3.525132E+05, 3.549850E+05, 3.574677E+05, 3.599614E+05, 3.624659E+05, 3.649814E+05, 
   3.675078E+05, 3.700451E+05, 3.725934E+05, 3.751526E+05, 3.777227E+05, 3.803038E+05, 
   3.828958E+05, 3.854988E+05, 3.881127E+05, 3.907376E+05, 3.933734E+05, 3.960202E+05, 
   3.986779E+05, 4.013466E+05, 4.040263E+05, 4.067170E+05, 4.094186E+05, 4.121311E+05, 
   4.148547E+05, 4.175892E+05, 4.203346E+05, 4.230911E+05, 4.258585E+05, 4.286368E+05, 
   4.314262E+05, 4.342265E+05, 4.370377E+05, 4.398600E+05, 4.426932E+05, 4.455373E+05, 
   4.483925E+05, 4.512585E+05, 4.541356E+05, 4.570235E+05, 4.599225E+05, 4.628324E+05, 
   4.657532E+05, 4.686850E+05, 4.716277E+05, 4.745814E+05, 4.775460E+05, 4.805215E+05, 
   4.835080E+05, 4.865054E+05, 4.895137E+05, 
])
# ============================ M = 1, I = 5 ============================
M = 1
I = 5
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.000000E+00, 8.444960E+00, 1.786086E+01, 3.086810E+01, 4.625151E+01, 6.362259E+01, 
   8.276956E+01, 1.035416E+02, 1.258219E+02, 1.495163E+02, 1.745467E+02, 2.008468E+02, 
   2.283599E+02, 2.570362E+02, 2.868320E+02, 3.177083E+02, 3.496303E+02, 3.825666E+02, 
   4.164889E+02, 4.513717E+02, 4.871921E+02, 5.239294E+02, 5.615653E+02, 6.000832E+02, 
   6.394690E+02, 6.797100E+02, 7.207956E+02, 7.627166E+02, 8.054658E+02, 8.490371E+02, 
   8.934261E+02, 9.386295E+02, 9.846456E+02, 1.031473E+03, 1.079113E+03, 1.127566E+03, 
   1.176835E+03, 1.226923E+03, 1.277832E+03, 1.329569E+03, 1.382138E+03, 1.435544E+03, 
   1.489795E+03, 1.544897E+03, 1.600858E+03, 1.657684E+03, 1.715385E+03, 1.773970E+03, 
   1.833446E+03, 1.893824E+03, 1.955112E+03, 2.017322E+03, 2.080462E+03, 2.144544E+03, 
   2.209579E+03, 2.275576E+03, 2.342549E+03, 2.410507E+03, 2.479463E+03, 2.549429E+03, 
   2.620417E+03, 2.692439E+03, 2.765507E+03, 2.839636E+03, 2.914836E+03, 2.991123E+03, 
   3.068509E+03, 3.147007E+03, 3.226632E+03, 3.307397E+03, 3.389316E+03, 3.472404E+03, 
   3.556674E+03, 3.642143E+03, 3.728824E+03, 3.816732E+03, 3.905883E+03, 3.996292E+03, 
   4.087973E+03, 4.180944E+03, 4.275219E+03, 4.370815E+03, 4.467747E+03, 4.566031E+03, 
   4.665685E+03, 4.766725E+03, 4.869166E+03, 4.973027E+03, 5.078323E+03, 5.185073E+03, 
   5.293293E+03, 5.403000E+03, 5.514213E+03, 5.626948E+03, 5.741224E+03, 5.857058E+03, 
   5.974468E+03, 6.093473E+03, 6.214091E+03, 6.336340E+03, 6.460239E+03, 6.585806E+03, 
   6.713060E+03, 6.842020E+03, 6.972705E+03, 7.105134E+03, 7.239327E+03, 7.375302E+03, 
   7.513080E+03, 7.652679E+03, 7.794120E+03, 7.937423E+03, 8.082606E+03, 8.229691E+03, 
   8.378697E+03, 8.529645E+03, 8.682555E+03, 8.837447E+03, 8.994343E+03, 9.153262E+03, 
   9.314226E+03, 9.477255E+03, 9.642370E+03, 9.809593E+03, 9.978944E+03, 1.015045E+04, 
   1.032412E+04, 1.049998E+04, 1.067806E+04, 1.085838E+04, 1.104095E+04, 1.122581E+04, 
   1.141296E+04, 1.160244E+04, 1.179427E+04, 1.198846E+04, 1.218504E+04, 1.238404E+04, 
   1.258547E+04, 1.278936E+04, 1.299573E+04, 1.320460E+04, 1.341601E+04, 1.362996E+04, 
   1.384648E+04, 1.406560E+04, 1.428733E+04, 1.451171E+04, 1.473876E+04, 1.496849E+04, 
   1.520094E+04, 1.543613E+04, 1.567407E+04, 1.591480E+04, 1.615834E+04, 1.640471E+04, 
   1.665393E+04, 1.690604E+04, 1.716105E+04, 1.741899E+04, 1.767988E+04, 1.794376E+04, 
   1.821063E+04, 1.848053E+04, 1.875349E+04, 1.902952E+04, 1.930866E+04, 1.959092E+04, 
   1.987633E+04, 2.016492E+04, 2.045672E+04, 2.075174E+04, 2.105002E+04, 2.135157E+04, 
   2.165643E+04, 2.196462E+04, 2.227617E+04, 2.259110E+04, 2.290944E+04, 2.323121E+04, 
   2.355644E+04, 2.388515E+04, 2.421738E+04, 2.455315E+04, 2.489248E+04, 2.523541E+04, 
   2.558195E+04, 2.593214E+04, 2.628600E+04, 2.664356E+04, 2.700484E+04, 2.736987E+04, 
   2.773869E+04, 2.811131E+04, 2.848776E+04, 2.886807E+04, 2.925227E+04, 2.964039E+04, 
   3.003245E+04, 3.042848E+04, 3.082850E+04, 3.123255E+04, 3.164066E+04, 3.205284E+04, 
   3.246914E+04, 3.288957E+04, 3.331417E+04, 3.374295E+04, 3.417596E+04, 3.461322E+04, 
   3.505476E+04, 3.550060E+04, 3.595078E+04, 3.640532E+04, 3.686425E+04, 3.732760E+04, 
   3.779540E+04, 3.826768E+04, 3.874447E+04, 3.922579E+04, 3.971168E+04, 4.020216E+04, 
   4.069726E+04, 4.119701E+04, 4.170145E+04, 4.221060E+04, 4.272449E+04, 4.324315E+04, 
   4.376661E+04, 4.429489E+04, 4.482804E+04, 4.536607E+04, 4.590903E+04, 4.645693E+04, 
   4.700981E+04, 4.756770E+04, 4.813063E+04, 4.869862E+04, 4.927172E+04, 4.984995E+04, 
   5.043333E+04, 5.102191E+04, 5.161570E+04, 5.221475E+04, 5.281908E+04, 5.342873E+04, 
   5.404371E+04, 5.466408E+04, 5.528984E+04, 5.592105E+04, 5.655772E+04, 5.719989E+04, 
   5.784759E+04, 5.850085E+04, 5.915970E+04, 5.982418E+04, 6.049431E+04, 6.117013E+04, 
   6.185167E+04, 6.253895E+04, 6.323202E+04, 6.393090E+04, 6.463563E+04, 6.534623E+04, 
   6.606275E+04, 6.678520E+04, 6.751363E+04, 6.824806E+04, 6.898853E+04, 6.973506E+04, 
   7.048770E+04, 7.124648E+04, 7.201142E+04, 7.278256E+04, 7.355993E+04, 7.434357E+04, 
   7.513350E+04, 7.592976E+04, 7.673238E+04, 7.754140E+04, 7.835685E+04, 7.917876E+04, 
   8.000716E+04, 8.084209E+04, 8.168358E+04, 8.253166E+04, 8.338637E+04, 8.424773E+04, 
   8.511579E+04, 8.599058E+04, 8.687212E+04, 8.776046E+04, 8.865563E+04, 8.955766E+04, 
   9.046658E+04, 9.138242E+04, 9.230523E+04, 9.323504E+04, 9.417187E+04, 9.511576E+04, 
   9.606675E+04, 9.702488E+04, 9.799016E+04, 9.896265E+04, 9.994237E+04, 1.009294E+05, 
   1.019236E+05, 1.029253E+05, 1.039343E+05, 1.049507E+05, 1.059745E+05, 1.070058E+05, 
   1.080446E+05, 1.090910E+05, 1.101449E+05, 1.112064E+05, 1.122756E+05, 1.133525E+05, 
   1.144371E+05, 1.155294E+05, 1.166295E+05, 1.177375E+05, 1.188533E+05, 1.199770E+05, 
   1.211086E+05, 1.222481E+05, 1.233957E+05, 1.245513E+05, 1.257149E+05, 1.268867E+05, 
   1.280666E+05, 1.292546E+05, 1.304509E+05, 1.316554E+05, 1.328682E+05, 1.340893E+05, 
   1.353187E+05, 1.365565E+05, 1.378027E+05, 1.390574E+05, 1.403206E+05, 1.415923E+05, 
   1.428725E+05, 1.441614E+05, 1.454589E+05, 1.467650E+05, 1.480799E+05, 1.494034E+05, 
   1.507358E+05, 1.520769E+05, 1.534269E+05, 1.547858E+05, 1.561536E+05, 1.575304E+05, 
   1.589161E+05, 1.603109E+05, 1.617147E+05, 1.631276E+05, 1.645496E+05, 1.659808E+05, 
   1.674212E+05, 1.688708E+05, 1.703297E+05, 1.717980E+05, 1.732755E+05, 1.747624E+05, 
   1.762588E+05, 1.777646E+05, 1.792799E+05, 1.808047E+05, 1.823391E+05, 1.838830E+05, 
   1.854366E+05, 1.869999E+05, 1.885729E+05, 1.901556E+05, 1.917480E+05, 1.933503E+05, 
   1.949625E+05, 1.965845E+05, 1.982164E+05, 1.998583E+05, 2.015102E+05, 2.031721E+05, 
   2.048440E+05, 2.065261E+05, 2.082182E+05, 2.099206E+05, 2.116331E+05, 2.133559E+05, 
   2.150890E+05, 2.168324E+05, 2.185861E+05, 2.203502E+05, 2.221247E+05, 2.239097E+05, 
   2.257051E+05, 2.275111E+05, 2.293277E+05, 2.311548E+05, 2.329926E+05, 2.348410E+05, 
   2.367001E+05, 2.385700E+05, 2.404506E+05, 2.423421E+05, 2.442444E+05, 2.461575E+05, 
   2.480816E+05, 2.500166E+05, 2.519626E+05, 2.539197E+05, 2.558877E+05, 2.578669E+05, 
   2.598572E+05, 2.618586E+05, 2.638713E+05, 2.658951E+05, 2.679303E+05, 2.699767E+05, 
   2.720345E+05, 2.741036E+05, 2.761841E+05, 2.782761E+05, 2.803795E+05, 2.824944E+05, 
   2.846209E+05, 2.867590E+05, 2.889086E+05, 2.910699E+05, 2.932429E+05, 2.954276E+05, 
   2.976240E+05, 2.998322E+05, 3.020522E+05, 3.042841E+05, 3.065279E+05, 3.087835E+05, 
   3.110511E+05, 3.133307E+05, 3.156223E+05, 3.179260E+05, 3.202417E+05, 3.225696E+05, 
   3.249096E+05, 3.272618E+05, 3.296262E+05, 3.320028E+05, 3.343917E+05, 3.367930E+05, 
   3.392066E+05, 3.416325E+05, 3.440709E+05, 3.465217E+05, 3.489851E+05, 3.514609E+05, 
   3.539492E+05, 3.564502E+05, 3.589637E+05, 3.614899E+05, 3.640287E+05, 3.665803E+05, 
   3.691446E+05, 3.717217E+05, 3.743115E+05, 3.769142E+05, 3.795298E+05, 3.821582E+05, 
   3.847996E+05, 3.874539E+05, 3.901213E+05, 3.928016E+05, 3.954950E+05, 3.982015E+05, 
   4.009211E+05, 4.036538E+05, 4.063997E+05, 4.091588E+05, 4.119311E+05, 4.147167E+05, 
   4.175156E+05, 4.203279E+05, 4.231535E+05, 4.259924E+05, 4.288448E+05, 4.317106E+05, 
   4.345899E+05, 4.374828E+05, 4.403891E+05, 4.433090E+05, 4.462425E+05, 4.491896E+05, 
   4.521504E+05, 4.551248E+05, 4.581130E+05, 4.611149E+05, 4.641305E+05, 4.671600E+05, 
   4.702033E+05, 4.732604E+05, 4.763314E+05, 
])
# ============================ M = 1, I = 6 ============================
M = 1
I = 6
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.600000E+01, 5.053060E+01, 1.066180E+02, 1.841882E+02, 2.759495E+02, 3.795687E+02, 
   4.937802E+02, 6.176844E+02, 7.505847E+02, 8.919192E+02, 1.041223E+03, 1.198101E+03, 
   1.362217E+03, 1.533276E+03, 1.711020E+03, 1.895221E+03, 2.085674E+03, 2.282196E+03, 
   2.484623E+03, 2.692808E+03, 2.906617E+03, 3.125930E+03, 3.350642E+03, 3.580657E+03, 
   3.815891E+03, 4.056272E+03, 4.301738E+03, 4.552237E+03, 4.807723E+03, 5.068164E+03, 
   5.333532E+03, 5.603809E+03, 5.878982E+03, 6.159048E+03, 6.444007E+03, 6.733867E+03, 
   7.028640E+03, 7.328344E+03, 7.633001E+03, 7.942638E+03, 8.257283E+03, 8.576972E+03, 
   8.901742E+03, 9.231632E+03, 9.566686E+03, 9.906950E+03, 1.025247E+04, 1.060330E+04, 
   1.095949E+04, 1.132110E+04, 1.168818E+04, 1.206079E+04, 1.243899E+04, 1.282284E+04, 
   1.321241E+04, 1.360777E+04, 1.400896E+04, 1.441608E+04, 1.482918E+04, 1.524833E+04, 
   1.567361E+04, 1.610509E+04, 1.654285E+04, 1.698695E+04, 1.743748E+04, 1.789451E+04, 
   1.835812E+04, 1.882839E+04, 1.930541E+04, 1.978925E+04, 2.027999E+04, 2.077773E+04, 
   2.128254E+04, 2.179452E+04, 2.231374E+04, 2.284029E+04, 2.337427E+04, 2.391577E+04, 
   2.446487E+04, 2.502166E+04, 2.558624E+04, 2.615871E+04, 2.673914E+04, 2.732765E+04, 
   2.792432E+04, 2.852925E+04, 2.914254E+04, 2.976429E+04, 3.039459E+04, 3.103355E+04, 
   3.168126E+04, 3.233782E+04, 3.300335E+04, 3.367793E+04, 3.436168E+04, 3.505469E+04, 
   3.575707E+04, 3.646893E+04, 3.719038E+04, 3.792151E+04, 3.866244E+04, 3.941327E+04, 
   4.017412E+04, 4.094509E+04, 4.172629E+04, 4.251784E+04, 4.331984E+04, 4.413240E+04, 
   4.495565E+04, 4.578968E+04, 4.663462E+04, 4.749058E+04, 4.835768E+04, 4.923603E+04, 
   5.012574E+04, 5.102693E+04, 5.193973E+04, 5.286424E+04, 5.380059E+04, 5.474889E+04, 
   5.570927E+04, 5.668184E+04, 5.766673E+04, 5.866405E+04, 5.967393E+04, 6.069649E+04, 
   6.173186E+04, 6.278015E+04, 6.384149E+04, 6.491600E+04, 6.600381E+04, 6.710505E+04, 
   6.821984E+04, 6.934830E+04, 7.049057E+04, 7.164677E+04, 7.281703E+04, 7.400147E+04, 
   7.520023E+04, 7.641344E+04, 7.764122E+04, 7.888371E+04, 8.014104E+04, 8.141333E+04, 
   8.270072E+04, 8.400335E+04, 8.532134E+04, 8.665483E+04, 8.800395E+04, 8.936884E+04, 
   9.074963E+04, 9.214645E+04, 9.355945E+04, 9.498876E+04, 9.643451E+04, 9.789684E+04, 
   9.937589E+04, 1.008718E+05, 1.023847E+05, 1.039147E+05, 1.054620E+05, 1.070268E+05, 
   1.086090E+05, 1.102090E+05, 1.118268E+05, 1.134626E+05, 1.151165E+05, 1.167887E+05, 
   1.184792E+05, 1.201884E+05, 1.219162E+05, 1.236628E+05, 1.254284E+05, 1.272132E+05, 
   1.290172E+05, 1.308406E+05, 1.326836E+05, 1.345463E+05, 1.364289E+05, 1.383315E+05, 
   1.402542E+05, 1.421972E+05, 1.441607E+05, 1.461448E+05, 1.481496E+05, 1.501753E+05, 
   1.522221E+05, 1.542900E+05, 1.563793E+05, 1.584902E+05, 1.606226E+05, 1.627769E+05, 
   1.649532E+05, 1.671515E+05, 1.693722E+05, 1.716152E+05, 1.738809E+05, 1.761692E+05, 
   1.784805E+05, 1.808148E+05, 1.831723E+05, 1.855531E+05, 1.879575E+05, 1.903855E+05, 
   1.928374E+05, 1.953133E+05, 1.978133E+05, 2.003376E+05, 2.028864E+05, 2.054598E+05, 
   2.080579E+05, 2.106811E+05, 2.133293E+05, 2.160028E+05, 2.187018E+05, 2.214263E+05, 
   2.241766E+05, 2.269528E+05, 2.297550E+05, 2.325836E+05, 2.354385E+05, 2.383200E+05, 
   2.412283E+05, 2.441634E+05, 2.471256E+05, 2.501151E+05, 2.531319E+05, 2.561764E+05, 
   2.592485E+05, 2.623486E+05, 2.654767E+05, 2.686331E+05, 2.718178E+05, 2.750312E+05, 
   2.782733E+05, 2.815443E+05, 2.848444E+05, 2.881738E+05, 2.915325E+05, 2.949209E+05, 
   2.983391E+05, 3.017872E+05, 3.052654E+05, 3.087739E+05, 3.123128E+05, 3.158824E+05, 
   3.194828E+05, 3.231142E+05, 3.267767E+05, 3.304705E+05, 3.341959E+05, 3.379529E+05, 
   3.417418E+05, 3.455628E+05, 3.494159E+05, 3.533014E+05, 3.572195E+05, 3.611703E+05, 
   3.651541E+05, 3.691709E+05, 3.732210E+05, 3.773046E+05, 3.814218E+05, 3.855728E+05, 
   3.897578E+05, 3.939770E+05, 3.982305E+05, 4.025186E+05, 4.068414E+05, 4.111990E+05, 
   4.155918E+05, 4.200198E+05, 4.244833E+05, 4.289823E+05, 4.335172E+05, 4.380881E+05, 
   4.426952E+05, 4.473386E+05, 4.520186E+05, 4.567353E+05, 4.614889E+05, 4.662796E+05, 
   4.711076E+05, 4.759731E+05, 4.808762E+05, 4.858172E+05, 4.907962E+05, 4.958134E+05, 
   5.008691E+05, 5.059633E+05, 5.110963E+05, 5.162683E+05, 5.214794E+05, 5.267299E+05, 
   5.320199E+05, 5.373497E+05, 5.427193E+05, 5.481291E+05, 5.535792E+05, 5.590697E+05, 
   5.646009E+05, 5.701730E+05, 5.757862E+05, 5.814406E+05, 5.871364E+05, 5.928739E+05, 
   5.986532E+05, 6.044745E+05, 6.103381E+05, 6.162440E+05, 6.221926E+05, 6.281839E+05, 
   6.342182E+05, 6.402957E+05, 6.464166E+05, 6.525810E+05, 6.587892E+05, 6.650414E+05, 
   6.713377E+05, 6.776783E+05, 6.840635E+05, 6.904935E+05, 6.969684E+05, 7.034884E+05, 
   7.100537E+05, 7.166646E+05, 7.233211E+05, 7.300236E+05, 7.367722E+05, 7.435671E+05, 
   7.504086E+05, 7.572967E+05, 7.642317E+05, 7.712139E+05, 7.782433E+05, 7.853202E+05, 
   7.924449E+05, 7.996174E+05, 8.068380E+05, 8.141069E+05, 8.214243E+05, 8.287904E+05, 
   8.362053E+05, 8.436694E+05, 8.511827E+05, 8.587456E+05, 8.663581E+05, 8.740205E+05, 
   8.817330E+05, 8.894957E+05, 8.973090E+05, 9.051729E+05, 9.130878E+05, 9.210537E+05, 
   9.290709E+05, 9.371395E+05, 9.452599E+05, 9.534322E+05, 9.616565E+05, 9.699331E+05, 
   9.782622E+05, 9.866440E+05, 9.950787E+05, 1.003566E+06, 1.012108E+06, 1.020702E+06, 
   1.029350E+06, 1.038053E+06, 1.046809E+06, 1.055619E+06, 1.064484E+06, 1.073404E+06, 
   1.082379E+06, 1.091408E+06, 1.100493E+06, 1.109634E+06, 1.118830E+06, 1.128082E+06, 
   1.137390E+06, 1.146755E+06, 1.156176E+06, 1.165654E+06, 1.175188E+06, 1.184780E+06, 
   1.194429E+06, 1.204136E+06, 1.213900E+06, 1.223722E+06, 1.233602E+06, 1.243541E+06, 
   1.253538E+06, 1.263593E+06, 1.273708E+06, 1.283882E+06, 1.294115E+06, 1.304407E+06, 
   1.314760E+06, 1.325172E+06, 1.335644E+06, 1.346177E+06, 1.356770E+06, 1.367423E+06, 
   1.378138E+06, 1.388913E+06, 1.399750E+06, 1.410649E+06, 1.421609E+06, 1.432631E+06, 
   1.443715E+06, 1.454861E+06, 1.466070E+06, 1.477342E+06, 1.488676E+06, 1.500073E+06, 
   1.511534E+06, 1.523058E+06, 1.534646E+06, 1.546297E+06, 1.558013E+06, 1.569792E+06, 
   1.581636E+06, 1.593545E+06, 1.605519E+06, 1.617557E+06, 1.629661E+06, 1.641830E+06, 
   1.654065E+06, 1.666365E+06, 1.678732E+06, 1.691164E+06, 1.703663E+06, 1.716229E+06, 
   1.728861E+06, 1.741560E+06, 1.754326E+06, 1.767160E+06, 1.780061E+06, 1.793029E+06, 
   1.806066E+06, 1.819170E+06, 1.832343E+06, 1.845584E+06, 1.858894E+06, 1.872272E+06, 
   1.885720E+06, 1.899236E+06, 1.912822E+06, 1.926478E+06, 1.940203E+06, 1.953998E+06, 
   1.967863E+06, 1.981799E+06, 1.995805E+06, 2.009881E+06, 2.024028E+06, 2.038247E+06, 
   2.052536E+06, 2.066897E+06, 2.081329E+06, 2.095834E+06, 2.110410E+06, 2.125058E+06, 
   2.139778E+06, 2.154571E+06, 2.169436E+06, 2.184374E+06, 2.199385E+06, 2.214469E+06, 
   2.229627E+06, 2.244857E+06, 2.260162E+06, 2.275540E+06, 2.290993E+06, 2.306519E+06, 
   2.322120E+06, 2.337796E+06, 2.353546E+06, 2.369371E+06, 2.385271E+06, 2.401246E+06, 
   2.417296E+06, 2.433422E+06, 2.449624E+06, 2.465901E+06, 2.482255E+06, 2.498685E+06, 
   2.515191E+06, 2.531773E+06, 2.548433E+06, 2.565169E+06, 2.581982E+06, 2.598872E+06, 
   2.615840E+06, 2.632885E+06, 2.650008E+06, 2.667209E+06, 2.684487E+06, 2.701844E+06, 
   2.719279E+06, 2.736792E+06, 2.754384E+06, 
])
# ============================ M = 1, I = 7 ============================
M = 1
I = 7
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.000000E+00, 9.129250E+00, 2.020566E+01, 3.533602E+01, 5.328177E+01, 7.358062E+01, 
   9.597421E+01, 1.202823E+02, 1.463666E+02, 1.741155E+02, 2.034368E+02, 2.342525E+02, 
   2.664953E+02, 3.001069E+02, 3.350360E+02, 3.712373E+02, 4.086708E+02, 4.473010E+02, 
   4.870968E+02, 5.280312E+02, 5.700805E+02, 6.132248E+02, 6.574474E+02, 7.027344E+02, 
   7.490750E+02, 7.964609E+02, 8.448862E+02, 8.943474E+02, 9.448428E+02, 9.963726E+02, 
   1.048939E+03, 1.102544E+03, 1.157194E+03, 1.212894E+03, 1.269651E+03, 1.327472E+03, 
   1.386368E+03, 1.446346E+03, 1.507418E+03, 1.569595E+03, 1.632887E+03, 1.697307E+03, 
   1.762868E+03, 1.829582E+03, 1.897464E+03, 1.966526E+03, 2.036785E+03, 2.108254E+03, 
   2.180948E+03, 2.254885E+03, 2.330078E+03, 2.406546E+03, 2.484305E+03, 2.563372E+03, 
   2.643764E+03, 2.725501E+03, 2.808599E+03, 2.893078E+03, 2.978957E+03, 3.066254E+03, 
   3.154991E+03, 3.245187E+03, 3.336863E+03, 3.430039E+03, 3.524736E+03, 3.620977E+03, 
   3.718782E+03, 3.818174E+03, 3.919176E+03, 4.021810E+03, 4.126099E+03, 4.232067E+03, 
   4.339738E+03, 4.449135E+03, 4.560283E+03, 4.673206E+03, 4.787929E+03, 4.904479E+03, 
   5.022879E+03, 5.143156E+03, 5.265336E+03, 5.389446E+03, 5.515511E+03, 5.643559E+03, 
   5.773617E+03, 5.905712E+03, 6.039873E+03, 6.176126E+03, 6.314501E+03, 6.455025E+03, 
   6.597728E+03, 6.742639E+03, 6.889787E+03, 7.039200E+03, 7.190910E+03, 7.344946E+03, 
   7.501338E+03, 7.660118E+03, 7.821315E+03, 7.984961E+03, 8.151087E+03, 8.319725E+03, 
   8.490906E+03, 8.664662E+03, 8.841027E+03, 9.020031E+03, 9.201709E+03, 9.386093E+03, 
   9.573217E+03, 9.763113E+03, 9.955817E+03, 1.015136E+04, 1.034978E+04, 1.055111E+04, 
   1.075538E+04, 1.096264E+04, 1.117291E+04, 1.138623E+04, 1.160263E+04, 1.182216E+04, 
   1.204485E+04, 1.227073E+04, 1.249984E+04, 1.273223E+04, 1.296792E+04, 1.320696E+04, 
   1.344937E+04, 1.369521E+04, 1.394451E+04, 1.419731E+04, 1.445364E+04, 1.471355E+04, 
   1.497708E+04, 1.524426E+04, 1.551513E+04, 1.578974E+04, 1.606813E+04, 1.635034E+04, 
   1.663640E+04, 1.692637E+04, 1.722027E+04, 1.751816E+04, 1.782008E+04, 1.812606E+04, 
   1.843615E+04, 1.875040E+04, 1.906885E+04, 1.939154E+04, 1.971852E+04, 2.004982E+04, 
   2.038550E+04, 2.072560E+04, 2.107016E+04, 2.141923E+04, 2.177286E+04, 2.213109E+04, 
   2.249397E+04, 2.286155E+04, 2.323386E+04, 2.361096E+04, 2.399290E+04, 2.437973E+04, 
   2.477148E+04, 2.516822E+04, 2.556999E+04, 2.597683E+04, 2.638881E+04, 2.680596E+04, 
   2.722834E+04, 2.765600E+04, 2.808898E+04, 2.852735E+04, 2.897114E+04, 2.942042E+04, 
   2.987523E+04, 3.033563E+04, 3.080167E+04, 3.127340E+04, 3.175088E+04, 3.223415E+04, 
   3.272328E+04, 3.321831E+04, 3.371931E+04, 3.422632E+04, 3.473940E+04, 3.525861E+04, 
   3.578400E+04, 3.631563E+04, 3.685355E+04, 3.739783E+04, 3.794852E+04, 3.850567E+04, 
   3.906935E+04, 3.963961E+04, 4.021651E+04, 4.080011E+04, 4.139047E+04, 4.198765E+04, 
   4.259171E+04, 4.320271E+04, 4.382070E+04, 4.444576E+04, 4.507794E+04, 4.571731E+04, 
   4.636392E+04, 4.701784E+04, 4.767913E+04, 4.834785E+04, 4.902407E+04, 4.970786E+04, 
   5.039927E+04, 5.109837E+04, 5.180522E+04, 5.251990E+04, 5.324247E+04, 5.397298E+04, 
   5.471152E+04, 5.545814E+04, 5.621291E+04, 5.697591E+04, 5.774719E+04, 5.852683E+04, 
   5.931490E+04, 6.011146E+04, 6.091658E+04, 6.173034E+04, 6.255280E+04, 6.338404E+04, 
   6.422412E+04, 6.507312E+04, 6.593111E+04, 6.679817E+04, 6.767435E+04, 6.855974E+04, 
   6.945442E+04, 7.035844E+04, 7.127190E+04, 7.219486E+04, 7.312739E+04, 7.406958E+04, 
   7.502150E+04, 7.598322E+04, 7.695482E+04, 7.793638E+04, 7.892798E+04, 7.992969E+04, 
   8.094160E+04, 8.196377E+04, 8.299630E+04, 8.403925E+04, 8.509271E+04, 8.615676E+04, 
   8.723149E+04, 8.831696E+04, 8.941326E+04, 9.052048E+04, 9.163870E+04, 9.276800E+04, 
   9.390846E+04, 9.506016E+04, 9.622320E+04, 9.739766E+04, 9.858361E+04, 9.978115E+04, 
   1.009904E+05, 1.022113E+05, 1.034441E+05, 1.046889E+05, 1.059457E+05, 1.072145E+05, 
   1.084956E+05, 1.097889E+05, 1.110947E+05, 1.124129E+05, 1.137436E+05, 1.150870E+05, 
   1.164431E+05, 1.178121E+05, 1.191939E+05, 1.205888E+05, 1.219968E+05, 1.234180E+05, 
   1.248525E+05, 1.263004E+05, 1.277617E+05, 1.292366E+05, 1.307252E+05, 1.322276E+05, 
   1.337438E+05, 1.352740E+05, 1.368183E+05, 1.383767E+05, 1.399493E+05, 1.415364E+05, 
   1.431378E+05, 1.447539E+05, 1.463845E+05, 1.480300E+05, 1.496903E+05, 1.513655E+05, 
   1.530558E+05, 1.547613E+05, 1.564820E+05, 1.582182E+05, 1.599697E+05, 1.617369E+05, 
   1.635197E+05, 1.653184E+05, 1.671329E+05, 1.689635E+05, 1.708101E+05, 1.726730E+05, 
   1.745521E+05, 1.764478E+05, 1.783599E+05, 1.802887E+05, 1.822343E+05, 1.841967E+05, 
   1.861762E+05, 1.881727E+05, 1.901864E+05, 1.922174E+05, 1.942659E+05, 1.963319E+05, 
   1.984156E+05, 2.005170E+05, 2.026364E+05, 2.047737E+05, 2.069291E+05, 2.091028E+05, 
   2.112949E+05, 2.135054E+05, 2.157345E+05, 2.179823E+05, 2.202489E+05, 2.225344E+05, 
   2.248391E+05, 2.271629E+05, 2.295060E+05, 2.318685E+05, 2.342506E+05, 2.366523E+05, 
   2.390739E+05, 2.415153E+05, 2.439768E+05, 2.464584E+05, 2.489604E+05, 2.514827E+05, 
   2.540256E+05, 2.565892E+05, 2.591735E+05, 2.617788E+05, 2.644051E+05, 2.670525E+05, 
   2.697213E+05, 2.724115E+05, 2.751233E+05, 2.778567E+05, 2.806120E+05, 2.833893E+05, 
   2.861886E+05, 2.890101E+05, 2.918540E+05, 2.947204E+05, 2.976093E+05, 3.005211E+05, 
   3.034557E+05, 3.064133E+05, 3.093941E+05, 3.123982E+05, 3.154257E+05, 3.184768E+05, 
   3.215515E+05, 3.246501E+05, 3.277727E+05, 3.309194E+05, 3.340904E+05, 3.372858E+05, 
   3.405057E+05, 3.437502E+05, 3.470196E+05, 3.503140E+05, 3.536334E+05, 3.569781E+05, 
   3.603482E+05, 3.637439E+05, 3.671652E+05, 3.706123E+05, 3.740854E+05, 3.775846E+05, 
   3.811100E+05, 3.846619E+05, 3.882403E+05, 3.918454E+05, 3.954774E+05, 3.991363E+05, 
   4.028224E+05, 4.065358E+05, 4.102766E+05, 4.140451E+05, 4.178412E+05, 4.216653E+05, 
   4.255174E+05, 4.293977E+05, 4.333064E+05, 4.372435E+05, 4.412094E+05, 4.452040E+05, 
   4.492276E+05, 4.532803E+05, 4.573623E+05, 4.614737E+05, 4.656147E+05, 4.697854E+05, 
   4.739861E+05, 4.782168E+05, 4.824777E+05, 4.867689E+05, 4.910907E+05, 4.954432E+05, 
   4.998266E+05, 5.042409E+05, 5.086864E+05, 5.131632E+05, 5.176716E+05, 5.222115E+05, 
   5.267833E+05, 5.313870E+05, 5.360229E+05, 5.406911E+05, 5.453917E+05, 5.501250E+05, 
   5.548910E+05, 5.596900E+05, 5.645222E+05, 5.693876E+05, 5.742864E+05, 5.792188E+05, 
   5.841851E+05, 5.891852E+05, 5.942195E+05, 5.992881E+05, 6.043911E+05, 6.095287E+05, 
   6.147011E+05, 6.199084E+05, 6.251509E+05, 6.304286E+05, 6.357419E+05, 6.410907E+05, 
   6.464753E+05, 6.518959E+05, 6.573526E+05, 6.628457E+05, 6.683752E+05, 6.739414E+05, 
   6.795444E+05, 6.851844E+05, 6.908615E+05, 6.965760E+05, 7.023280E+05, 7.081177E+05, 
   7.139453E+05, 7.198109E+05, 7.257147E+05, 7.316569E+05, 7.376376E+05, 7.436571E+05, 
   7.497155E+05, 7.558130E+05, 7.619497E+05, 7.681259E+05, 7.743416E+05, 7.805972E+05, 
   7.868928E+05, 7.932284E+05, 7.996044E+05, 8.060209E+05, 8.124781E+05, 8.189761E+05, 
   8.255152E+05, 8.320954E+05, 8.387171E+05, 8.453803E+05, 8.520852E+05, 8.588321E+05, 
   8.656211E+05, 8.724524E+05, 8.793261E+05, 8.862425E+05, 8.932017E+05, 9.002039E+05, 
   9.072493E+05, 9.143380E+05, 9.214703E+05, 9.286463E+05, 9.358662E+05, 9.431303E+05, 
   9.504385E+05, 9.577913E+05, 9.651886E+05, 9.726308E+05, 9.801180E+05, 9.876503E+05, 
   9.952280E+05, 1.002851E+06, 1.010520E+06, 1.018235E+06, 1.025996E+06, 1.033803E+06, 
   1.041657E+06, 1.049557E+06, 1.057504E+06, 1.065499E+06, 1.073540E+06, 1.081629E+06, 
   1.089765E+06, 1.097949E+06, 1.106181E+06, 1.114461E+06, 1.122789E+06, 1.131166E+06, 
   1.139591E+06, 1.148065E+06, 1.156589E+06, 1.165161E+06, 1.173783E+06, 1.182454E+06, 
   1.191175E+06, 1.199946E+06, 1.208768E+06, 1.217639E+06, 1.226561E+06, 1.235534E+06, 
   1.244557E+06, 1.253632E+06, 1.262758E+06, 1.271935E+06, 1.281164E+06, 1.290444E+06, 
   1.299777E+06, 1.309161E+06, 1.318598E+06, 1.328088E+06, 1.337630E+06, 1.347225E+06, 
   1.356873E+06, 1.366574E+06, 1.376329E+06, 1.386137E+06, 1.396000E+06, 1.405916E+06, 
   1.415886E+06, 1.425910E+06, 1.435990E+06, 1.446123E+06, 1.456312E+06, 1.466556E+06, 
   1.476855E+06, 1.487209E+06, 1.497619E+06, 1.508085E+06, 1.518607E+06, 1.529185E+06, 
   1.539819E+06, 1.550510E+06, 1.561257E+06, 1.572062E+06, 1.582923E+06, 1.593842E+06, 
   1.604818E+06, 1.615852E+06, 1.626943E+06, 1.638093E+06, 1.649300E+06, 1.660566E+06, 
   1.671891E+06, 1.683274E+06, 1.694716E+06, 1.706217E+06, 1.717777E+06, 1.729397E+06, 
   1.741076E+06, 1.752815E+06, 1.764614E+06, 1.776473E+06, 1.788392E+06, 1.800372E+06, 
   1.812412E+06, 1.824513E+06, 1.836675E+06, 1.848899E+06, 1.861184E+06, 1.873530E+06, 
   1.885938E+06, 1.898407E+06, 1.910939E+06, 1.923533E+06, 1.936190E+06, 1.948909E+06, 
   1.961691E+06, 
])
# ============================ M = 1, I = 8 ============================
#M = 1
#I = 8
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   6.000000E+00, 9.217470E+00, 2.048931E+01, 3.584812E+01, 5.406267E+01, 7.466592E+01, 
#   9.739573E+01, 1.220692E+02, 1.485457E+02, 1.767122E+02, 2.064750E+02, 2.377547E+02, 
#   2.704833E+02, 3.046015E+02, 3.400571E+02, 3.768044E+02, 4.148027E+02, 4.540162E+02, 
#   4.944135E+02, 5.359671E+02, 5.786535E+02, 6.224527E+02, 6.673477E+02, 7.133251E+02, 
#   7.603741E+02, 8.084866E+02, 8.576570E+02, 9.078819E+02, 9.591602E+02, 1.011492E+03, 
#   1.064880E+03, 1.119328E+03, 1.174840E+03, 1.231424E+03, 1.289086E+03, 1.347834E+03, 
#   1.407678E+03, 1.468628E+03, 1.530695E+03, 1.593889E+03, 1.658222E+03, 1.723709E+03, 
#   1.790360E+03, 1.858190E+03, 1.927213E+03, 1.997443E+03, 2.068896E+03, 2.141586E+03, 
#   2.215529E+03, 2.290742E+03, 2.367241E+03, 2.445042E+03, 2.524165E+03, 2.604625E+03, 
#   2.686441E+03, 2.769633E+03, 2.854217E+03, 2.940215E+03, 3.027645E+03, 3.116527E+03, 
#   3.206883E+03, 3.298731E+03, 3.392094E+03, 3.486993E+03, 3.583450E+03, 3.681486E+03, 
#   3.781124E+03, 3.882387E+03, 3.985298E+03, 4.089880E+03, 4.196157E+03, 4.304154E+03, 
#   4.413893E+03, 4.525401E+03, 4.638702E+03, 4.753821E+03, 4.870784E+03, 4.989616E+03, 
#   5.110345E+03, 5.232996E+03, 5.357596E+03, 5.484172E+03, 5.612751E+03, 5.743362E+03, 
#   5.876031E+03, 6.010787E+03, 6.147658E+03, 6.286674E+03, 6.427862E+03, 6.571253E+03, 
#   6.716875E+03, 6.864759E+03, 7.014934E+03, 7.167431E+03, 7.322280E+03, 7.479512E+03, 
#   7.639158E+03, 7.801250E+03, 7.965818E+03, 8.132895E+03, 8.302513E+03, 8.474704E+03, 
#   8.649501E+03, 8.826937E+03, 9.007044E+03, 9.189856E+03, 9.375407E+03, 9.563731E+03, 
#   9.754862E+03, 9.948834E+03, 1.014568E+04, 1.034544E+04, 1.054814E+04, 1.075383E+04, 
#   1.096253E+04, 1.117429E+04, 1.138913E+04, 1.160711E+04, 1.182824E+04, 1.205257E+04, 
#   1.228015E+04, 1.251099E+04, 1.274515E+04, 1.298266E+04, 1.322355E+04, 1.346788E+04, 
#   1.371567E+04, 1.396697E+04, 1.422181E+04, 1.448023E+04, 1.474229E+04, 1.500800E+04, 
#   1.527743E+04, 1.555059E+04, 1.582755E+04, 1.610834E+04, 1.639299E+04, 1.668156E+04, 
#   1.697409E+04, 1.727061E+04, 1.757117E+04, 1.787582E+04, 1.818460E+04, 1.849754E+04, 
#   1.881470E+04, 1.913612E+04, 1.946184E+04, 1.979191E+04, 2.012638E+04, 2.046528E+04, 
#   2.080867E+04, 2.115660E+04, 2.150909E+04, 2.186622E+04, 2.222801E+04, 2.259453E+04, 
#   2.296581E+04, 2.334190E+04, 2.372286E+04, 2.410873E+04, 2.449955E+04, 2.489539E+04, 
#   2.529629E+04, 2.570229E+04, 2.611346E+04, 2.652983E+04, 2.695146E+04, 2.737840E+04, 
#   2.781071E+04, 2.824843E+04, 2.869162E+04, 2.914032E+04, 2.959460E+04, 3.005450E+04, 
#   3.052008E+04, 3.099139E+04, 3.146849E+04, 3.195142E+04, 3.244026E+04, 3.293504E+04, 
#   3.343582E+04, 3.394267E+04, 3.445564E+04, 3.497478E+04, 3.550014E+04, 3.603180E+04, 
#   3.656980E+04, 3.711421E+04, 3.766507E+04, 3.822246E+04, 3.878642E+04, 3.935702E+04, 
#   3.993432E+04, 4.051837E+04, 4.110925E+04, 4.170700E+04, 4.231169E+04, 4.292338E+04, 
#   4.354213E+04, 4.416800E+04, 4.480107E+04, 4.544138E+04, 4.608901E+04, 4.674401E+04, 
#   4.740645E+04, 4.807640E+04, 4.875391E+04, 4.943906E+04, 5.013191E+04, 5.083253E+04, 
#   5.154097E+04, 5.225732E+04, 5.298163E+04, 5.371397E+04, 5.445442E+04, 5.520303E+04, 
#   5.595988E+04, 5.672503E+04, 5.749856E+04, 5.828054E+04, 5.907103E+04, 5.987010E+04, 
#   6.067783E+04, 6.149429E+04, 6.231955E+04, 6.315368E+04, 6.399675E+04, 6.484883E+04, 
#   6.571001E+04, 6.658035E+04, 6.745993E+04, 6.834881E+04, 6.924709E+04, 7.015482E+04, 
#   7.107210E+04, 7.199898E+04, 7.293556E+04, 7.388190E+04, 7.483809E+04, 7.580420E+04, 
#   7.678030E+04, 7.776649E+04, 7.876284E+04, 7.976942E+04, 8.078632E+04, 8.181361E+04, 
#   8.285139E+04, 8.389972E+04, 8.495869E+04, 8.602838E+04, 8.710888E+04, 8.820027E+04, 
#   8.930263E+04, 9.041604E+04, 9.154060E+04, 9.267637E+04, 9.382346E+04, 9.498193E+04, 
#   9.615189E+04, 9.733342E+04, 9.852659E+04, 9.973151E+04, 1.009483E+05, 1.021769E+05, 
#   1.034176E+05, 1.046703E+05, 1.059353E+05, 1.072125E+05, 1.085021E+05, 1.098041E+05, 
#   1.111187E+05, 1.124459E+05, 1.137859E+05, 1.151386E+05, 1.165043E+05, 1.178830E+05, 
#   1.192748E+05, 1.206798E+05, 1.220981E+05, 1.235298E+05, 1.249750E+05, 1.264337E+05, 
#   1.279061E+05, 1.293923E+05, 1.308924E+05, 1.324064E+05, 1.339345E+05, 1.354768E+05, 
#   1.370334E+05, 1.386043E+05, 1.401897E+05, 1.417897E+05, 1.434043E+05, 1.450337E+05, 
#   1.466780E+05, 1.483373E+05, 1.500116E+05, 1.517012E+05, 1.534060E+05, 1.551263E+05, 
#   1.568620E+05, 1.586133E+05, 1.603804E+05, 1.621633E+05, 1.639621E+05, 1.657770E+05, 
#   1.676080E+05, 1.694553E+05, 1.713189E+05, 1.731990E+05, 1.750957E+05, 1.770091E+05, 
#   1.789393E+05, 1.808865E+05, 1.828506E+05, 1.848319E+05, 1.868305E+05, 1.888465E+05, 
#   1.908799E+05, 1.929309E+05, 1.949997E+05, 1.970863E+05, 1.991908E+05, 2.013135E+05, 
#   2.034543E+05, 2.056134E+05, 2.077909E+05, 2.099870E+05, 2.122018E+05, 2.144353E+05, 
#   2.166877E+05, 2.189592E+05, 2.212498E+05, 2.235597E+05, 2.258889E+05, 2.282377E+05, 
#   2.306061E+05, 2.329943E+05, 2.354024E+05, 2.378305E+05, 2.402787E+05, 2.427472E+05, 
#   2.452361E+05, 2.477454E+05, 2.502755E+05, 2.528263E+05, 2.553980E+05, 2.579908E+05, 
#   2.606047E+05, 2.632399E+05, 2.658966E+05, 2.685748E+05, 2.712746E+05, 2.739963E+05, 
#   2.767400E+05, 2.795057E+05, 2.822937E+05, 2.851040E+05, 2.879368E+05, 2.907922E+05, 
#   2.936704E+05, 2.965715E+05, 2.994957E+05, 3.024430E+05, 3.054136E+05, 3.084076E+05, 
#   3.114253E+05, 3.144667E+05, 3.175319E+05, 3.206211E+05, 3.237345E+05, 3.268722E+05, 
#   3.300344E+05, 3.332211E+05, 3.364325E+05, 3.396688E+05, 3.429300E+05, 3.462165E+05, 
#   3.495282E+05, 3.528654E+05, 3.562281E+05, 3.596166E+05, 3.630310E+05, 3.664714E+05, 
#   3.699380E+05, 3.734309E+05, 3.769503E+05, 3.804963E+05, 3.840690E+05, 3.876687E+05, 
#   3.912955E+05, 3.949495E+05, 3.986309E+05, 4.023398E+05, 4.060763E+05, 4.098408E+05, 
#   4.136332E+05, 4.174537E+05, 4.213026E+05, 4.251799E+05, 4.290858E+05, 4.330204E+05, 
#   4.369840E+05, 4.409767E+05, 4.449986E+05, 4.490499E+05, 4.531307E+05, 4.572413E+05, 
#   4.613817E+05, 4.655522E+05, 4.697528E+05, 4.739838E+05, 4.782453E+05, 4.825375E+05, 
#   4.868605E+05, 4.912145E+05, 4.955996E+05, 5.000161E+05, 5.044641E+05, 5.089437E+05, 
#   5.134551E+05, 5.179985E+05, 5.225741E+05, 5.271819E+05, 5.318222E+05, 5.364952E+05, 
#   5.412010E+05, 5.459397E+05, 5.507116E+05, 5.555168E+05, 5.603555E+05, 5.652278E+05, 
#   5.701339E+05, 5.750740E+05, 5.800483E+05, 5.850569E+05, 5.901000E+05, 5.951778E+05, 
#   6.002904E+05, 6.054380E+05, 6.106208E+05, 6.158389E+05, 6.210926E+05, 6.263819E+05, 
#   6.317072E+05, 6.370685E+05, 6.424660E+05, 6.478999E+05, 6.533704E+05, 6.588776E+05, 
#   6.644218E+05, 6.700030E+05, 6.756215E+05, 6.812775E+05, 6.869711E+05, 6.927025E+05, 
#   6.984719E+05, 7.042794E+05, 7.101253E+05, 7.160097E+05, 7.219327E+05, 7.278947E+05, 
#   7.338957E+05, 7.399359E+05, 7.460156E+05, 7.521348E+05, 7.582938E+05, 7.644928E+05, 
#   7.707319E+05, 7.770113E+05, 7.833312E+05, 7.896917E+05, 7.960932E+05, 8.025356E+05, 
#   8.090193E+05, 8.155444E+05, 8.221111E+05, 8.287196E+05, 8.353700E+05, 8.420625E+05, 
#   8.487974E+05, 8.555747E+05, 8.623948E+05, 8.692577E+05, 8.761637E+05, 8.831129E+05, 
#   8.901056E+05, 8.971418E+05, 9.042219E+05, 9.113459E+05, 9.185141E+05, 9.257267E+05, 
#   9.329838E+05, 9.402856E+05, 9.476323E+05, 9.550241E+05, 9.624612E+05, 9.699438E+05, 
#   9.774720E+05, 9.850461E+05, 9.926662E+05, 1.000333E+06, 1.008045E+06, 1.015805E+06, 
#   1.023611E+06, 1.031464E+06, 1.039364E+06, 1.047312E+06, 1.055307E+06, 1.063350E+06, 
#   1.071440E+06, 1.079579E+06, 1.087766E+06, 1.096002E+06, 1.104286E+06, 1.112619E+06, 
#   1.121001E+06, 1.129433E+06, 1.137913E+06, 1.146444E+06, 1.155024E+06, 1.163654E+06, 
#   1.172334E+06, 1.181065E+06, 1.189846E+06, 1.198678E+06, 1.207561E+06, 1.216495E+06, 
#   1.225480E+06, 1.234517E+06, 1.243606E+06, 1.252746E+06, 1.261938E+06, 1.271183E+06, 
#   1.280480E+06, 1.289830E+06, 1.299233E+06, 1.308688E+06, 1.318197E+06, 1.327759E+06, 
#   1.337375E+06, 1.347044E+06, 1.356767E+06, 1.366545E+06, 1.376377E+06, 1.386263E+06, 
#   1.396204E+06, 1.406200E+06, 1.416251E+06, 1.426358E+06, 1.436519E+06, 1.446737E+06, 
#   1.457010E+06, 1.467339E+06, 1.477725E+06, 1.488167E+06, 1.498665E+06, 1.509220E+06, 
#   1.519833E+06, 1.530502E+06, 1.541229E+06, 1.552013E+06, 1.562855E+06, 1.573754E+06, 
#   1.584712E+06, 1.595728E+06, 1.606803E+06, 1.617936E+06, 1.629128E+06, 1.640379E+06, 
#   1.651689E+06, 1.663059E+06, 1.674488E+06, 1.685977E+06, 1.697526E+06, 1.709135E+06, 
#   1.720804E+06, 1.732534E+06, 1.744324E+06, 1.756176E+06, 1.768088E+06, 1.780062E+06, 
#   1.792097E+06, 1.804193E+06, 1.816351E+06, 1.828572E+06, 1.840854E+06, 1.853199E+06, 
#   1.865606E+06, 1.878076E+06, 1.890609E+06, 1.903205E+06, 1.915864E+06, 1.928586E+06, 
#   1.941372E+06, 1.954222E+06, 1.967136E+06, 1.980113E+06, 1.993156E+06, 2.006262E+06, 
#   2.019434E+06, 
#])
# ============================ M = 1, I = 9 ============================
#M = 1
#I = 9
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   3.600000E+01, 5.505365E+01, 1.221318E+02, 2.136367E+02, 3.221607E+02, 4.449152E+02, 
#   5.803382E+02, 7.273400E+02, 8.850833E+02, 1.052894E+03, 1.230215E+03, 1.416572E+03, 
#   1.611562E+03, 1.814829E+03, 2.026065E+03, 2.244995E+03, 2.471377E+03, 2.704998E+03, 
#   2.945671E+03, 3.193232E+03, 3.447538E+03, 3.708471E+03, 3.975930E+03, 4.249832E+03, 
#   4.530112E+03, 4.816723E+03, 5.109629E+03, 5.408810E+03, 5.714256E+03, 6.025970E+03, 
#   6.343965E+03, 6.668261E+03, 6.998887E+03, 7.335880E+03, 7.679283E+03, 8.029144E+03, 
#   8.385518E+03, 8.748461E+03, 9.118039E+03, 9.494316E+03, 9.877365E+03, 1.026726E+04, 
#   1.066407E+04, 1.106789E+04, 1.147879E+04, 1.189686E+04, 1.232218E+04, 1.275486E+04, 
#   1.319497E+04, 1.364263E+04, 1.409792E+04, 1.456094E+04, 1.503180E+04, 1.551061E+04, 
#   1.599746E+04, 1.649247E+04, 1.699576E+04, 1.750743E+04, 1.802760E+04, 1.855639E+04, 
#   1.909392E+04, 1.964030E+04, 2.019568E+04, 2.076017E+04, 2.133390E+04, 2.191700E+04, 
#   2.250960E+04, 2.311185E+04, 2.372386E+04, 2.434580E+04, 2.497778E+04, 2.561997E+04, 
#   2.627249E+04, 2.693550E+04, 2.760915E+04, 2.829357E+04, 2.898894E+04, 2.969539E+04, 
#   3.041308E+04, 3.114218E+04, 3.188283E+04, 3.263520E+04, 3.339945E+04, 3.417574E+04, 
#   3.496424E+04, 3.576511E+04, 3.657853E+04, 3.740465E+04, 3.824366E+04, 3.909573E+04, 
#   3.996102E+04, 4.083973E+04, 4.173201E+04, 4.263806E+04, 4.355806E+04, 4.449218E+04, 
#   4.544061E+04, 4.640354E+04, 4.738114E+04, 4.837362E+04, 4.938116E+04, 5.040396E+04, 
#   5.144219E+04, 5.249607E+04, 5.356578E+04, 5.465153E+04, 5.575351E+04, 5.687192E+04, 
#   5.800696E+04, 5.915885E+04, 6.032778E+04, 6.151395E+04, 6.271759E+04, 6.393890E+04, 
#   6.517809E+04, 6.643537E+04, 6.771096E+04, 6.900507E+04, 7.031792E+04, 7.164973E+04, 
#   7.300073E+04, 7.437113E+04, 7.576115E+04, 7.717103E+04, 7.860099E+04, 8.005126E+04, 
#   8.152207E+04, 8.301365E+04, 8.452624E+04, 8.606006E+04, 8.761536E+04, 8.919238E+04, 
#   9.079135E+04, 9.241253E+04, 9.405614E+04, 9.572244E+04, 9.741167E+04, 9.912408E+04, 
#   1.008599E+05, 1.026194E+05, 1.044029E+05, 1.062106E+05, 1.080427E+05, 1.098995E+05, 
#   1.117813E+05, 1.136883E+05, 1.156208E+05, 1.175791E+05, 1.195634E+05, 1.215739E+05, 
#   1.236111E+05, 1.256751E+05, 1.277662E+05, 1.298847E+05, 1.320309E+05, 1.342050E+05, 
#   1.364074E+05, 1.386383E+05, 1.408980E+05, 1.431867E+05, 1.455049E+05, 1.478527E+05, 
#   1.502304E+05, 1.526385E+05, 1.550770E+05, 1.575465E+05, 1.600471E+05, 1.625791E+05, 
#   1.651429E+05, 1.677387E+05, 1.703670E+05, 1.730279E+05, 1.757218E+05, 1.784490E+05, 
#   1.812099E+05, 1.840046E+05, 1.868337E+05, 1.896973E+05, 1.925958E+05, 1.955296E+05, 
#   1.984989E+05, 2.015042E+05, 2.045456E+05, 2.076236E+05, 2.107385E+05, 2.138906E+05, 
#   2.170803E+05, 2.203079E+05, 2.235737E+05, 2.268782E+05, 2.302216E+05, 2.336043E+05, 
#   2.370267E+05, 2.404890E+05, 2.439918E+05, 2.475352E+05, 2.511198E+05, 2.547457E+05, 
#   2.584135E+05, 2.621235E+05, 2.658760E+05, 2.696715E+05, 2.735102E+05, 2.773926E+05, 
#   2.813191E+05, 2.852899E+05, 2.893056E+05, 2.933665E+05, 2.974730E+05, 3.016254E+05, 
#   3.058242E+05, 3.100697E+05, 3.143624E+05, 3.187027E+05, 3.230909E+05, 3.275275E+05, 
#   3.320128E+05, 3.365473E+05, 3.411313E+05, 3.457653E+05, 3.504498E+05, 3.551850E+05, 
#   3.599715E+05, 3.648097E+05, 3.696999E+05, 3.746427E+05, 3.796383E+05, 3.846874E+05, 
#   3.897902E+05, 3.949473E+05, 4.001590E+05, 4.054258E+05, 4.107482E+05, 4.161266E+05, 
#   4.215615E+05, 4.270532E+05, 4.326023E+05, 4.382092E+05, 4.438744E+05, 4.495982E+05, 
#   4.553813E+05, 4.612240E+05, 4.671268E+05, 4.730902E+05, 4.791146E+05, 4.852006E+05, 
#   4.913485E+05, 4.975590E+05, 5.038324E+05, 5.101693E+05, 5.165701E+05, 5.230354E+05, 
#   5.295655E+05, 5.361611E+05, 5.428226E+05, 5.495505E+05, 5.563453E+05, 5.632076E+05, 
#   5.701377E+05, 5.771363E+05, 5.842039E+05, 5.913409E+05, 5.985480E+05, 6.058255E+05, 
#   6.131740E+05, 6.205941E+05, 6.280863E+05, 6.356511E+05, 6.432891E+05, 6.510007E+05, 
#   6.587866E+05, 6.666473E+05, 6.745832E+05, 6.825951E+05, 6.906833E+05, 6.988485E+05, 
#   7.070913E+05, 7.154121E+05, 7.238115E+05, 7.322902E+05, 7.408486E+05, 7.494874E+05, 
#   7.582071E+05, 7.670083E+05, 7.758916E+05, 7.848575E+05, 7.939067E+05, 8.030397E+05, 
#   8.122571E+05, 8.215595E+05, 8.309475E+05, 8.404217E+05, 8.499827E+05, 8.596311E+05, 
#   8.693675E+05, 8.791925E+05, 8.891068E+05, 8.991109E+05, 9.092054E+05, 9.193911E+05, 
#   9.296684E+05, 9.400381E+05, 9.505008E+05, 9.610571E+05, 9.717075E+05, 9.824529E+05, 
#   9.932938E+05, 1.004231E+06, 1.015265E+06, 1.026396E+06, 1.037625E+06, 1.048954E+06, 
#   1.060381E+06, 1.071909E+06, 1.083537E+06, 1.095267E+06, 1.107099E+06, 1.119034E+06, 
#   1.131072E+06, 1.143214E+06, 1.155461E+06, 1.167814E+06, 1.180272E+06, 1.192838E+06, 
#   1.205511E+06, 1.218292E+06, 1.231183E+06, 1.244183E+06, 1.257293E+06, 1.270515E+06, 
#   1.283848E+06, 1.297294E+06, 1.310853E+06, 1.324526E+06, 1.338313E+06, 1.352216E+06, 
#   1.366236E+06, 1.380372E+06, 1.394625E+06, 1.408997E+06, 1.423488E+06, 1.438099E+06, 
#   1.452831E+06, 1.467684E+06, 1.482659E+06, 1.497756E+06, 1.512978E+06, 1.528323E+06, 
#   1.543794E+06, 1.559391E+06, 1.575115E+06, 1.590966E+06, 1.606945E+06, 1.623053E+06, 
#   1.639291E+06, 1.655660E+06, 1.672160E+06, 1.688792E+06, 1.705557E+06, 1.722456E+06, 
#   1.739489E+06, 1.756658E+06, 1.773963E+06, 1.791406E+06, 1.808986E+06, 1.826704E+06, 
#   1.844562E+06, 1.862561E+06, 1.880700E+06, 1.898982E+06, 1.917406E+06, 1.935974E+06, 
#   1.954686E+06, 1.973543E+06, 1.992547E+06, 2.011698E+06, 2.030996E+06, 2.050444E+06, 
#   2.070041E+06, 2.089788E+06, 2.109686E+06, 2.129737E+06, 2.149941E+06, 2.170298E+06, 
#   2.190810E+06, 2.211478E+06, 2.232303E+06, 2.253285E+06, 2.274425E+06, 2.295724E+06, 
#   2.317183E+06, 2.338804E+06, 2.360586E+06, 2.382531E+06, 2.404639E+06, 2.426912E+06, 
#   2.449351E+06, 2.471956E+06, 2.494728E+06, 2.517669E+06, 2.540778E+06, 2.564058E+06, 
#   2.587508E+06, 2.611131E+06, 2.634926E+06, 2.658895E+06, 2.683039E+06, 2.707359E+06, 
#   2.731855E+06, 2.756528E+06, 2.781380E+06, 2.806411E+06, 2.831623E+06, 2.857016E+06, 
#   2.882592E+06, 2.908350E+06, 2.934293E+06, 2.960421E+06, 2.986735E+06, 3.013236E+06, 
#   3.039925E+06, 3.066804E+06, 3.093872E+06, 3.121131E+06, 3.148582E+06, 3.176226E+06, 
#   3.204065E+06, 3.232098E+06, 3.260327E+06, 3.288752E+06, 3.317376E+06, 3.346199E+06, 
#   3.375221E+06, 3.404445E+06, 3.433870E+06, 3.463498E+06, 3.493331E+06, 3.523368E+06, 
#   3.553611E+06, 3.584061E+06, 3.614719E+06, 3.645586E+06, 3.676663E+06, 3.707951E+06, 
#   3.739451E+06, 3.771164E+06, 3.803092E+06, 3.835234E+06, 3.867593E+06, 3.900169E+06, 
#   3.932963E+06, 3.965976E+06, 3.999210E+06, 4.032665E+06, 4.066343E+06, 4.100244E+06, 
#   4.134370E+06, 4.168721E+06, 4.203299E+06, 4.238104E+06, 4.273139E+06, 4.308403E+06, 
#   4.343898E+06, 4.379624E+06, 4.415584E+06, 4.451778E+06, 4.488207E+06, 4.524872E+06, 
#   4.561774E+06, 4.598915E+06, 4.636295E+06, 4.673915E+06, 4.711778E+06, 4.749882E+06, 
#   4.788230E+06, 4.826824E+06, 4.865663E+06, 4.904748E+06, 4.944082E+06, 4.983665E+06, 
#   5.023498E+06, 5.063583E+06, 5.103920E+06, 5.144510E+06, 5.185355E+06, 5.226455E+06, 
#   5.267812E+06, 5.309427E+06, 5.351301E+06, 5.393435E+06, 5.435830E+06, 5.478487E+06, 
#   5.521408E+06, 5.564592E+06, 5.608043E+06, 5.651760E+06, 5.695745E+06, 5.739998E+06, 
#   5.784521E+06, 5.829316E+06, 5.874383E+06, 5.919722E+06, 5.965337E+06, 6.011226E+06, 
#   6.057392E+06, 6.103836E+06, 6.150559E+06, 6.197562E+06, 6.244845E+06, 6.292411E+06, 
#   6.340260E+06, 6.388393E+06, 6.436812E+06, 6.485517E+06, 6.534510E+06, 6.583792E+06, 
#   6.633364E+06, 6.683227E+06, 6.733382E+06, 6.783830E+06, 6.834573E+06, 6.885611E+06, 
#   6.936945E+06, 6.988578E+06, 7.040509E+06, 7.092741E+06, 7.145273E+06, 7.198108E+06, 
#   7.251246E+06, 7.304688E+06, 7.358436E+06, 7.412491E+06, 7.466853E+06, 7.521525E+06, 
#   7.576506E+06, 7.631799E+06, 7.687404E+06, 7.743322E+06, 7.799555E+06, 7.856104E+06, 
#   7.912969E+06, 7.970152E+06, 8.027654E+06, 8.085477E+06, 8.143620E+06, 8.202086E+06, 
#   8.260875E+06, 8.319989E+06, 8.379428E+06, 8.439195E+06, 8.499289E+06, 8.559712E+06, 
#   8.620465E+06, 8.681550E+06, 8.742967E+06, 8.804718E+06, 8.866803E+06, 8.929223E+06, 
#   8.991981E+06, 9.055076E+06, 9.118511E+06, 9.182285E+06, 9.246401E+06, 9.310859E+06, 
#   9.375661E+06, 9.440807E+06, 9.506299E+06, 9.572137E+06, 9.638324E+06, 9.704859E+06, 
#   9.771745E+06, 9.838981E+06, 9.906570E+06, 9.974512E+06, 1.004281E+07, 1.011146E+07, 
#   1.018047E+07, 1.024984E+07, 1.031956E+07, 1.038965E+07, 1.046009E+07, 1.053090E+07, 
#   1.060207E+07, 1.067361E+07, 1.074551E+07, 1.081778E+07, 1.089041E+07, 1.096341E+07, 
#   1.103679E+07, 1.111053E+07, 1.118465E+07, 1.125913E+07, 1.133400E+07, 1.140923E+07, 
#   1.148485E+07, 1.156084E+07, 1.163721E+07, 1.171395E+07, 1.179108E+07, 1.186859E+07, 
#   1.194648E+07, 
#])
# ============================ M = 2, I = 1 ============================
M = 2
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.172300E+00, 9.074330E+00, 1.797937E+01, 2.688526E+01, 3.579152E+01, 4.469806E+01, 
   5.360487E+01, 6.251202E+01, 7.142001E+01, 8.033032E+01, 8.924629E+01, 9.817378E+01, 
   1.071216E+02, 1.161017E+02, 1.251286E+02, 1.342190E+02, 1.433912E+02, 1.526647E+02, 
   1.620592E+02, 1.715947E+02, 1.812909E+02, 1.911671E+02, 2.012420E+02, 2.115338E+02, 
   2.220599E+02, 2.328372E+02, 2.438818E+02, 2.552095E+02, 2.668355E+02, 2.787743E+02, 
   2.910405E+02, 3.036479E+02, 3.166103E+02, 3.299412E+02, 3.436540E+02, 3.577617E+02, 
   3.722775E+02, 3.872144E+02, 4.025853E+02, 4.184031E+02, 4.346809E+02, 4.514316E+02, 
   4.686683E+02, 4.864042E+02, 5.046524E+02, 5.234263E+02, 5.427394E+02, 5.626052E+02, 
   5.830375E+02, 6.040502E+02, 6.256573E+02, 6.478731E+02, 6.707119E+02, 6.941884E+02, 
   7.183172E+02, 7.431135E+02, 7.685924E+02, 7.947693E+02, 8.216598E+02, 8.492796E+02, 
   8.776450E+02, 9.067721E+02, 9.366774E+02, 9.673777E+02, 9.988899E+02, 1.031231E+03, 
   1.064419E+03, 1.098472E+03, 1.133406E+03, 1.169241E+03, 1.205995E+03, 1.243687E+03, 
   1.282335E+03, 1.321959E+03, 1.362579E+03, 1.404213E+03, 1.446883E+03, 1.490607E+03, 
   1.535408E+03, 1.581306E+03, 1.628322E+03, 1.676477E+03, 1.725793E+03, 1.776292E+03, 
   1.827996E+03, 1.880928E+03, 1.935111E+03, 1.990567E+03, 2.047321E+03, 2.105395E+03, 
   2.164814E+03, 2.225603E+03, 2.287785E+03, 2.351386E+03, 2.416430E+03, 2.482945E+03, 
   2.550954E+03, 2.620485E+03, 2.691564E+03, 2.764217E+03, 2.838473E+03, 2.914357E+03, 
   2.991899E+03, 3.071125E+03, 3.152065E+03, 3.234747E+03, 3.319200E+03, 3.405454E+03, 
   3.493538E+03, 3.583482E+03, 3.675317E+03, 3.769073E+03, 3.864782E+03, 3.962474E+03, 
   4.062182E+03, 4.163938E+03, 4.267774E+03, 4.373723E+03, 4.481819E+03, 4.592093E+03, 
   4.704582E+03, 4.819317E+03, 4.936336E+03, 5.055671E+03, 5.177359E+03, 5.301435E+03, 
   5.427935E+03, 5.556896E+03, 5.688354E+03, 5.822347E+03, 5.958911E+03, 6.098086E+03, 
   6.239909E+03, 6.384418E+03, 6.531654E+03, 6.681654E+03, 6.834460E+03, 6.990110E+03, 
   7.148647E+03, 7.310110E+03, 7.474541E+03, 7.641982E+03, 7.812474E+03, 7.986061E+03, 
   8.162786E+03, 8.342691E+03, 8.525820E+03, 8.712217E+03, 8.901928E+03, 9.094996E+03, 
   9.291468E+03, 9.491388E+03, 9.694803E+03, 9.901760E+03, 1.011231E+04, 1.032649E+04, 
   1.054435E+04, 1.076595E+04, 1.099132E+04, 1.122053E+04, 1.145362E+04, 1.169063E+04, 
   1.193162E+04, 1.217665E+04, 1.242575E+04, 1.267899E+04, 1.293641E+04, 1.319807E+04, 
   1.346401E+04, 1.373430E+04, 1.400899E+04, 1.428813E+04, 1.457177E+04, 1.485998E+04, 
   1.515280E+04, 1.545029E+04, 1.575251E+04, 1.605951E+04, 1.637136E+04, 1.668811E+04, 
   1.700982E+04, 1.733654E+04, 1.766834E+04, 1.800527E+04, 1.834740E+04, 1.869478E+04, 
   1.904748E+04, 1.940556E+04, 1.976907E+04, 2.013809E+04, 2.051267E+04, 2.089288E+04, 
   2.127878E+04, 2.167043E+04, 2.206790E+04, 2.247126E+04, 2.288057E+04, 2.329589E+04, 
   2.371729E+04, 2.414484E+04, 2.457861E+04, 2.501866E+04, 2.546506E+04, 2.591788E+04, 
   2.637719E+04, 2.684306E+04, 2.731556E+04, 2.779476E+04, 2.828073E+04, 2.877355E+04, 
   2.927327E+04, 2.977998E+04, 3.029376E+04, 3.081466E+04, 3.134277E+04, 3.187817E+04, 
   3.242091E+04, 3.297109E+04, 3.352878E+04, 3.409405E+04, 3.466698E+04, 3.524765E+04, 
   3.583614E+04, 3.643251E+04, 3.703686E+04, 3.764926E+04, 3.826979E+04, 3.889853E+04, 
   3.953557E+04, 4.018097E+04, 4.083483E+04, 4.149723E+04, 4.216825E+04, 4.284797E+04, 
   4.353647E+04, 4.423385E+04, 4.494018E+04, 4.565554E+04, 4.638004E+04, 4.711374E+04, 
   4.785674E+04, 4.860913E+04, 4.937098E+04, 5.014240E+04, 5.092347E+04, 5.171427E+04, 
   5.251490E+04, 5.332544E+04, 5.414600E+04, 5.497665E+04, 5.581749E+04, 5.666861E+04, 
   5.753010E+04, 5.840206E+04, 5.928458E+04, 6.017775E+04, 6.108167E+04, 6.199643E+04, 
   6.292213E+04, 6.385886E+04, 6.480672E+04, 6.576580E+04, 6.673621E+04, 6.771804E+04, 
   6.871139E+04, 6.971636E+04, 7.073304E+04, 7.176154E+04, 7.280195E+04, 7.385438E+04, 
   7.491893E+04, 7.599570E+04, 7.708478E+04, 7.818629E+04, 7.930032E+04, 8.042699E+04, 
   8.156638E+04, 8.271861E+04, 8.388379E+04, 8.506201E+04, 8.625338E+04, 8.745801E+04, 
   8.867600E+04, 8.990746E+04, 9.115251E+04, 9.241124E+04, 9.368376E+04, 9.497018E+04, 
   9.627062E+04, 9.758518E+04, 9.891397E+04, 1.002571E+05, 1.016147E+05, 1.029868E+05, 
   1.043737E+05, 1.057753E+05, 1.071918E+05, 1.086233E+05, 1.100700E+05, 1.115319E+05, 
   1.130091E+05, 1.145018E+05, 1.160101E+05, 1.175341E+05, 1.190739E+05, 1.206296E+05, 
   1.222014E+05, 1.237894E+05, 1.253936E+05, 1.270142E+05, 1.286514E+05, 1.303052E+05, 
   1.319757E+05, 1.336631E+05, 1.353675E+05, 1.370890E+05, 1.388278E+05, 1.405839E+05, 
   1.423574E+05, 1.441486E+05, 1.459575E+05, 1.477842E+05, 1.496289E+05, 1.514917E+05, 
   1.533726E+05, 1.552719E+05, 1.571897E+05, 1.591260E+05, 1.610810E+05, 1.630549E+05, 
   1.650477E+05, 1.670596E+05, 1.690906E+05, 1.711410E+05, 1.732109E+05, 1.753003E+05, 
   1.774094E+05, 1.795384E+05, 1.816873E+05, 1.838563E+05, 1.860455E+05, 1.882551E+05, 
   1.904851E+05, 1.927357E+05, 1.950070E+05, 1.972992E+05, 1.996124E+05, 2.019467E+05, 
   2.043022E+05, 2.066791E+05, 2.090774E+05, 2.114975E+05, 2.139392E+05, 2.164029E+05, 
   2.188885E+05, 2.213964E+05, 2.239265E+05, 2.264790E+05, 2.290540E+05, 2.316517E+05, 
   2.342723E+05, 2.369157E+05, 2.395823E+05, 2.422720E+05, 2.449850E+05, 2.477216E+05, 
   2.504817E+05, 2.532655E+05, 2.560732E+05, 2.589049E+05, 2.617607E+05, 2.646408E+05, 
   2.675453E+05, 2.704743E+05, 2.734279E+05, 2.764063E+05, 2.794097E+05, 2.824381E+05, 
   2.854917E+05, 2.885706E+05, 2.916749E+05, 2.948049E+05, 2.979606E+05, 3.011421E+05, 
   3.043496E+05, 3.075833E+05, 3.108431E+05, 3.141294E+05, 3.174422E+05, 3.207817E+05, 
   3.241479E+05, 3.275411E+05, 3.309613E+05, 3.344087E+05, 3.378834E+05, 3.413856E+05, 
   3.449153E+05, 3.484728E+05, 3.520581E+05, 3.556714E+05, 3.593129E+05, 3.629825E+05, 
   3.666806E+05, 3.704072E+05, 3.741624E+05, 3.779464E+05, 3.817594E+05, 3.856013E+05, 
   3.894725E+05, 3.933730E+05, 3.973029E+05, 4.012623E+05, 4.052515E+05, 4.092705E+05, 
   4.133195E+05, 4.173986E+05, 4.215079E+05, 4.256476E+05, 4.298178E+05, 4.340186E+05, 
   4.382501E+05, 4.425126E+05, 4.468060E+05, 4.511306E+05, 4.554865E+05, 4.598737E+05, 
   4.642925E+05, 4.687430E+05, 4.732252E+05, 4.777394E+05, 4.822856E+05, 4.868640E+05, 
   4.914747E+05, 4.961178E+05, 5.007935E+05, 5.055018E+05, 5.102430E+05, 5.150171E+05, 
   5.198243E+05, 5.246646E+05, 5.295383E+05, 5.344454E+05, 5.393861E+05, 5.443605E+05, 
   5.493687E+05, 5.544108E+05, 5.594870E+05, 5.645974E+05, 5.697421E+05, 5.749212E+05, 
   5.801349E+05, 5.853833E+05, 5.906665E+05, 5.959846E+05, 6.013377E+05, 6.067261E+05, 
   6.121497E+05, 6.176087E+05, 6.231032E+05, 6.286334E+05, 6.341994E+05, 6.398012E+05, 
   6.454391E+05, 6.511130E+05, 6.568232E+05, 6.625698E+05, 6.683529E+05, 6.741725E+05, 
   6.800289E+05, 6.859221E+05, 6.918522E+05, 6.978194E+05, 7.038237E+05, 7.098653E+05, 
   7.159444E+05, 7.220609E+05, 7.282151E+05, 7.344070E+05, 7.406368E+05, 7.469045E+05, 
   7.532103E+05, 7.595543E+05, 7.659366E+05, 7.723573E+05, 7.788165E+05, 7.853143E+05, 
   7.918509E+05, 7.984263E+05, 8.050406E+05, 8.116940E+05, 8.183866E+05, 8.251184E+05, 
   8.318897E+05, 8.387004E+05, 8.455507E+05, 8.524406E+05, 8.593704E+05, 8.663401E+05, 
   8.733498E+05, 8.803995E+05, 8.874895E+05, 
])
# ============================ M = 2, I = 2 ============================
M = 2
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.344550E+00, 1.814784E+01, 3.595709E+01, 5.376805E+01, 7.157976E+01, 8.939203E+01, 
   1.072048E+02, 1.250184E+02, 1.428342E+02, 1.606559E+02, 1.784919E+02, 1.963562E+02, 
   2.142697E+02, 2.322594E+02, 2.503579E+02, 2.686022E+02, 2.870319E+02, 3.056885E+02, 
   3.246141E+02, 3.438505E+02, 3.634386E+02, 3.834184E+02, 4.038283E+02, 4.247054E+02, 
   4.460854E+02, 4.680024E+02, 4.904894E+02, 5.135781E+02, 5.372995E+02, 5.616834E+02, 
   5.867589E+02, 6.125545E+02, 6.390984E+02, 6.664180E+02, 6.945407E+02, 7.234935E+02, 
   7.533034E+02, 7.839971E+02, 8.156015E+02, 8.481435E+02, 8.816500E+02, 9.161481E+02, 
   9.516650E+02, 9.882282E+02, 1.025865E+03, 1.064605E+03, 1.104474E+03, 1.145503E+03, 
   1.187719E+03, 1.231152E+03, 1.275832E+03, 1.321788E+03, 1.369052E+03, 1.417654E+03, 
   1.467626E+03, 1.518999E+03, 1.571805E+03, 1.626077E+03, 1.681848E+03, 1.739152E+03, 
   1.798023E+03, 1.858496E+03, 1.920605E+03, 1.984387E+03, 2.049877E+03, 2.117111E+03, 
   2.186128E+03, 2.256965E+03, 2.329660E+03, 2.404252E+03, 2.480780E+03, 2.559285E+03, 
   2.639806E+03, 2.722384E+03, 2.807062E+03, 2.893881E+03, 2.982884E+03, 3.074115E+03, 
   3.167617E+03, 3.263435E+03, 3.361614E+03, 3.462200E+03, 3.565239E+03, 3.670777E+03, 
   3.778863E+03, 3.889545E+03, 4.002871E+03, 4.118891E+03, 4.237655E+03, 4.359214E+03, 
   4.483619E+03, 4.610921E+03, 4.741175E+03, 4.874432E+03, 5.010747E+03, 5.150174E+03, 
   5.292769E+03, 5.438588E+03, 5.587687E+03, 5.740124E+03, 5.895957E+03, 6.055244E+03, 
   6.218045E+03, 6.384420E+03, 6.554429E+03, 6.728135E+03, 6.905599E+03, 7.086884E+03, 
   7.272055E+03, 7.461175E+03, 7.654309E+03, 7.851524E+03, 8.052885E+03, 8.258461E+03, 
   8.468319E+03, 8.682529E+03, 8.901159E+03, 9.124280E+03, 9.351963E+03, 9.584280E+03, 
   9.821303E+03, 1.006311E+04, 1.030976E+04, 1.056135E+04, 1.081794E+04, 1.107962E+04, 
   1.134645E+04, 1.161852E+04, 1.189590E+04, 1.217868E+04, 1.246693E+04, 1.276075E+04, 
   1.306020E+04, 1.336537E+04, 1.367635E+04, 1.399323E+04, 1.431607E+04, 1.464498E+04, 
   1.498004E+04, 1.532134E+04, 1.566896E+04, 1.602300E+04, 1.638355E+04, 1.675069E+04, 
   1.712453E+04, 1.750514E+04, 1.789264E+04, 1.828710E+04, 1.868864E+04, 1.909733E+04, 
   1.951329E+04, 1.993661E+04, 2.036739E+04, 2.080572E+04, 2.125172E+04, 2.170548E+04, 
   2.216710E+04, 2.263669E+04, 2.311436E+04, 2.360020E+04, 2.409433E+04, 2.459685E+04, 
   2.510787E+04, 2.562749E+04, 2.615584E+04, 2.669302E+04, 2.723914E+04, 2.779432E+04, 
   2.835866E+04, 2.893229E+04, 2.951531E+04, 3.010786E+04, 3.071003E+04, 3.132196E+04, 
   3.194376E+04, 3.257555E+04, 3.321745E+04, 3.386958E+04, 3.453208E+04, 3.520506E+04, 
   3.588864E+04, 3.658296E+04, 3.728814E+04, 3.800431E+04, 3.873160E+04, 3.947013E+04, 
   4.022005E+04, 4.098147E+04, 4.175454E+04, 4.253939E+04, 4.333615E+04, 4.414496E+04, 
   4.496596E+04, 4.579927E+04, 4.664506E+04, 4.750344E+04, 4.837457E+04, 4.925858E+04, 
   5.015563E+04, 5.106584E+04, 5.198938E+04, 5.292637E+04, 5.387698E+04, 5.484135E+04, 
   5.581962E+04, 5.681196E+04, 5.781850E+04, 5.883940E+04, 5.987482E+04, 6.092490E+04, 
   6.198981E+04, 6.306970E+04, 6.416473E+04, 6.527506E+04, 6.640083E+04, 6.754223E+04, 
   6.869940E+04, 6.987251E+04, 7.106172E+04, 7.226721E+04, 7.348912E+04, 7.472764E+04, 
   7.598292E+04, 7.725514E+04, 7.854447E+04, 7.985107E+04, 8.117512E+04, 8.251680E+04, 
   8.387627E+04, 8.525371E+04, 8.664930E+04, 8.806322E+04, 8.949564E+04, 9.094674E+04, 
   9.241670E+04, 9.390571E+04, 9.541394E+04, 9.694159E+04, 9.848883E+04, 1.000559E+05, 
   1.016428E+05, 1.032500E+05, 1.048775E+05, 1.065255E+05, 1.081942E+05, 1.098839E+05, 
   1.115947E+05, 1.133268E+05, 1.150803E+05, 1.168556E+05, 1.186528E+05, 1.204720E+05, 
   1.223136E+05, 1.241776E+05, 1.260644E+05, 1.279740E+05, 1.299067E+05, 1.318628E+05, 
   1.338423E+05, 1.358456E+05, 1.378728E+05, 1.399241E+05, 1.419997E+05, 1.441000E+05, 
   1.462249E+05, 1.483749E+05, 1.505501E+05, 1.527506E+05, 1.549768E+05, 1.572288E+05, 
   1.595069E+05, 1.618113E+05, 1.641422E+05, 1.664997E+05, 1.688843E+05, 1.712960E+05, 
   1.737350E+05, 1.762017E+05, 1.786963E+05, 1.812189E+05, 1.837698E+05, 1.863492E+05, 
   1.889574E+05, 1.915945E+05, 1.942609E+05, 1.969568E+05, 1.996823E+05, 2.024377E+05, 
   2.052233E+05, 2.080393E+05, 2.108859E+05, 2.137634E+05, 2.166720E+05, 2.196119E+05, 
   2.225835E+05, 2.255868E+05, 2.286223E+05, 2.316900E+05, 2.347904E+05, 2.379235E+05, 
   2.410897E+05, 2.442892E+05, 2.475223E+05, 2.507891E+05, 2.540900E+05, 2.574253E+05, 
   2.607950E+05, 2.641996E+05, 2.676393E+05, 2.711142E+05, 2.746248E+05, 2.781712E+05, 
   2.817536E+05, 2.853724E+05, 2.890278E+05, 2.927201E+05, 2.964495E+05, 3.002162E+05, 
   3.040207E+05, 3.078630E+05, 3.117435E+05, 3.156624E+05, 3.196200E+05, 3.236166E+05, 
   3.276524E+05, 3.317277E+05, 3.358427E+05, 3.399978E+05, 3.441932E+05, 3.484292E+05, 
   3.527060E+05, 3.570239E+05, 3.613832E+05, 3.657841E+05, 3.702270E+05, 3.747120E+05, 
   3.792395E+05, 3.838098E+05, 3.884231E+05, 3.930797E+05, 3.977798E+05, 4.025238E+05, 
   4.073119E+05, 4.121444E+05, 4.170216E+05, 4.219438E+05, 4.269111E+05, 4.319240E+05, 
   4.369827E+05, 4.420875E+05, 4.472386E+05, 4.524364E+05, 4.576810E+05, 4.629729E+05, 
   4.683123E+05, 4.736994E+05, 4.791346E+05, 4.846181E+05, 4.901502E+05, 4.957312E+05, 
   5.013614E+05, 5.070410E+05, 5.127705E+05, 5.185499E+05, 5.243797E+05, 5.302601E+05, 
   5.361914E+05, 5.421738E+05, 5.482078E+05, 5.542935E+05, 5.604312E+05, 5.666213E+05, 
   5.728640E+05, 5.791596E+05, 5.855084E+05, 5.919107E+05, 5.983667E+05, 6.048768E+05, 
   6.114413E+05, 6.180604E+05, 6.247344E+05, 6.314636E+05, 6.382484E+05, 6.450889E+05, 
   6.519855E+05, 6.589385E+05, 6.659481E+05, 6.730147E+05, 6.801385E+05, 6.873199E+05, 
   6.945590E+05, 7.018563E+05, 7.092119E+05, 7.166263E+05, 7.240996E+05, 7.316322E+05, 
   7.392243E+05, 7.468762E+05, 7.545883E+05, 7.623608E+05, 7.701940E+05, 7.780882E+05, 
   7.860436E+05, 7.940607E+05, 8.021395E+05, 8.102805E+05, 8.184840E+05, 8.267501E+05, 
   8.350793E+05, 8.434717E+05, 8.519277E+05, 8.604476E+05, 8.690316E+05, 8.776800E+05, 
   8.863932E+05, 8.951713E+05, 9.040148E+05, 9.129238E+05, 9.218986E+05, 9.309396E+05, 
   9.400470E+05, 9.492211E+05, 9.584622E+05, 9.677705E+05, 9.771464E+05, 9.865901E+05, 
   9.961019E+05, 1.005682E+06, 1.015331E+06, 1.025049E+06, 1.034836E+06, 1.044692E+06, 
   1.054619E+06, 1.064615E+06, 1.074682E+06, 1.084819E+06, 1.095027E+06, 1.105306E+06, 
   1.115657E+06, 1.126080E+06, 1.136574E+06, 1.147141E+06, 1.157780E+06, 1.168492E+06, 
   1.179277E+06, 1.190135E+06, 1.201067E+06, 1.212073E+06, 1.223153E+06, 1.234308E+06, 
   1.245537E+06, 1.256841E+06, 1.268220E+06, 1.279675E+06, 1.291205E+06, 1.302812E+06, 
   1.314494E+06, 1.326253E+06, 1.338089E+06, 1.350003E+06, 1.361993E+06, 1.374061E+06, 
   1.386206E+06, 1.398430E+06, 1.410732E+06, 1.423113E+06, 1.435573E+06, 1.448111E+06, 
   1.460729E+06, 1.473427E+06, 1.486204E+06, 1.499062E+06, 1.512000E+06, 1.525018E+06, 
   1.538118E+06, 1.551298E+06, 1.564560E+06, 1.577903E+06, 1.591328E+06, 1.604836E+06, 
   1.618425E+06, 1.632097E+06, 1.645852E+06, 1.659690E+06, 1.673611E+06, 1.687616E+06, 
   1.701705E+06, 1.715877E+06, 1.730134E+06, 1.744475E+06, 1.758901E+06, 1.773411E+06, 
   1.788007E+06, 1.802689E+06, 1.817455E+06, 1.832308E+06, 1.847247E+06, 1.862272E+06, 
   1.877383E+06, 1.892581E+06, 1.907867E+06, 
])
# ============================ M = 2, I = 3 ============================
M = 2
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.260550E+00, 1.921448E+01, 3.809072E+01, 5.696862E+01, 7.584727E+01, 9.472651E+01, 
   1.136063E+02, 1.324868E+02, 1.513692E+02, 1.702569E+02, 1.891574E+02, 2.080836E+02, 
   2.270552E+02, 2.460982E+02, 2.652444E+02, 2.845304E+02, 3.039959E+02, 3.236828E+02, 
   3.436341E+02, 3.638929E+02, 3.845016E+02, 4.055020E+02, 4.269345E+02, 4.488383E+02, 
   4.712510E+02, 4.942090E+02, 5.177476E+02, 5.419006E+02, 5.667010E+02, 5.921808E+02, 
   6.183711E+02, 6.453024E+02, 6.730046E+02, 7.015071E+02, 7.308391E+02, 7.610291E+02, 
   7.921059E+02, 8.240978E+02, 8.570332E+02, 8.909404E+02, 9.258477E+02, 9.617837E+02, 
   9.987770E+02, 1.036856E+03, 1.076051E+03, 1.116389E+03, 1.157902E+03, 1.200617E+03, 
   1.244567E+03, 1.289781E+03, 1.336290E+03, 1.384125E+03, 1.433318E+03, 1.483901E+03, 
   1.535906E+03, 1.589366E+03, 1.644315E+03, 1.700786E+03, 1.758814E+03, 1.818434E+03, 
   1.879680E+03, 1.942588E+03, 2.007196E+03, 2.073539E+03, 2.141654E+03, 2.211581E+03, 
   2.283356E+03, 2.357020E+03, 2.432611E+03, 2.510170E+03, 2.589737E+03, 2.671353E+03, 
   2.755060E+03, 2.840901E+03, 2.928918E+03, 3.019154E+03, 3.111655E+03, 3.206464E+03, 
   3.303627E+03, 3.403190E+03, 3.505199E+03, 3.609702E+03, 3.716746E+03, 3.826379E+03, 
   3.938651E+03, 4.053611E+03, 4.171310E+03, 4.291799E+03, 4.415128E+03, 4.541351E+03, 
   4.670521E+03, 4.802690E+03, 4.937913E+03, 5.076246E+03, 5.217743E+03, 5.362461E+03, 
   5.510457E+03, 5.661789E+03, 5.816514E+03, 5.974693E+03, 6.136385E+03, 6.301649E+03, 
   6.470548E+03, 6.643144E+03, 6.819498E+03, 6.999675E+03, 7.183738E+03, 7.371752E+03, 
   7.563783E+03, 7.759897E+03, 7.960160E+03, 8.164642E+03, 8.373410E+03, 8.586533E+03, 
   8.804082E+03, 9.026127E+03, 9.252740E+03, 9.483994E+03, 9.719961E+03, 9.960715E+03, 
   1.020633E+04, 1.045688E+04, 1.071245E+04, 1.097311E+04, 1.123894E+04, 1.151001E+04, 
   1.178641E+04, 1.206822E+04, 1.235551E+04, 1.264838E+04, 1.294689E+04, 1.325115E+04, 
   1.356122E+04, 1.387720E+04, 1.419917E+04, 1.452722E+04, 1.486143E+04, 1.520190E+04, 
   1.554872E+04, 1.590196E+04, 1.626173E+04, 1.662812E+04, 1.700122E+04, 1.738112E+04, 
   1.776792E+04, 1.816171E+04, 1.856258E+04, 1.897065E+04, 1.938600E+04, 1.980873E+04, 
   2.023894E+04, 2.067673E+04, 2.112221E+04, 2.157547E+04, 2.203662E+04, 2.250576E+04, 
   2.298300E+04, 2.346844E+04, 2.396219E+04, 2.446436E+04, 2.497505E+04, 2.549438E+04, 
   2.602245E+04, 2.655937E+04, 2.710526E+04, 2.766022E+04, 2.822438E+04, 2.879784E+04, 
   2.938072E+04, 2.997314E+04, 3.057522E+04, 3.118706E+04, 3.180880E+04, 3.244054E+04, 
   3.308241E+04, 3.373453E+04, 3.439702E+04, 3.507001E+04, 3.575361E+04, 3.644795E+04, 
   3.715316E+04, 3.786937E+04, 3.859669E+04, 3.933526E+04, 4.008520E+04, 4.084665E+04, 
   4.161973E+04, 4.240457E+04, 4.320132E+04, 4.401008E+04, 4.483101E+04, 4.566423E+04, 
   4.650988E+04, 4.736810E+04, 4.823901E+04, 4.912276E+04, 5.001948E+04, 5.092932E+04, 
   5.185240E+04, 5.278887E+04, 5.373887E+04, 5.470254E+04, 5.568001E+04, 5.667144E+04, 
   5.767697E+04, 5.869673E+04, 5.973088E+04, 6.077955E+04, 6.184289E+04, 6.292105E+04, 
   6.401416E+04, 6.512239E+04, 6.624588E+04, 6.738476E+04, 6.853920E+04, 6.970934E+04, 
   7.089533E+04, 7.209732E+04, 7.331546E+04, 7.454989E+04, 7.580078E+04, 7.706827E+04, 
   7.835251E+04, 7.965366E+04, 8.097186E+04, 8.230728E+04, 8.366006E+04, 8.503037E+04, 
   8.641834E+04, 8.782414E+04, 8.924792E+04, 9.068983E+04, 9.215004E+04, 9.362869E+04, 
   9.512594E+04, 9.664195E+04, 9.817687E+04, 9.973087E+04, 1.013041E+05, 1.028967E+05, 
   1.045088E+05, 1.061407E+05, 1.077923E+05, 1.094640E+05, 1.111559E+05, 1.128681E+05, 
   1.146007E+05, 1.163540E+05, 1.181281E+05, 1.199232E+05, 1.217393E+05, 1.235768E+05, 
   1.254356E+05, 1.273160E+05, 1.292182E+05, 1.311422E+05, 1.330883E+05, 1.350567E+05, 
   1.370473E+05, 1.390605E+05, 1.410964E+05, 1.431551E+05, 1.452368E+05, 1.473416E+05, 
   1.494697E+05, 1.516213E+05, 1.537965E+05, 1.559954E+05, 1.582183E+05, 1.604652E+05, 
   1.627364E+05, 1.650319E+05, 1.673520E+05, 1.696967E+05, 1.720663E+05, 1.744608E+05, 
   1.768805E+05, 1.793255E+05, 1.817959E+05, 1.842919E+05, 1.868137E+05, 1.893613E+05, 
   1.919349E+05, 1.945348E+05, 1.971610E+05, 1.998136E+05, 2.024929E+05, 2.051989E+05, 
   2.079318E+05, 2.106918E+05, 2.134791E+05, 2.162936E+05, 2.191356E+05, 2.220053E+05, 
   2.249027E+05, 2.278281E+05, 2.307815E+05, 2.337631E+05, 2.367730E+05, 2.398114E+05, 
   2.428784E+05, 2.459741E+05, 2.490987E+05, 2.522523E+05, 2.554351E+05, 2.586472E+05, 
   2.618886E+05, 2.651596E+05, 2.684603E+05, 2.717908E+05, 2.751512E+05, 2.785417E+05, 
   2.819624E+05, 2.854134E+05, 2.888948E+05, 2.924069E+05, 2.959496E+05, 2.995231E+05, 
   3.031276E+05, 3.067631E+05, 3.104298E+05, 3.141278E+05, 3.178573E+05, 3.216183E+05, 
   3.254109E+05, 3.292353E+05, 3.330916E+05, 3.369799E+05, 3.409004E+05, 3.448530E+05, 
   3.488380E+05, 3.528555E+05, 3.569055E+05, 3.609881E+05, 3.651036E+05, 3.692519E+05, 
   3.734333E+05, 3.776477E+05, 3.818953E+05, 3.861762E+05, 3.904905E+05, 3.948383E+05, 
   3.992197E+05, 4.036348E+05, 4.080837E+05, 4.125665E+05, 4.170833E+05, 4.216342E+05, 
   4.262192E+05, 4.308385E+05, 4.354922E+05, 
])
# ============================ M = 2, I = 4 ============================
M = 2
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.325894E+01, 1.121680E+02, 2.223057E+02, 3.324534E+02, 4.426057E+02, 5.527613E+02, 
   6.629202E+02, 7.730834E+02, 8.832573E+02, 9.934611E+02, 1.103737E+03, 1.214160E+03, 
   1.324841E+03, 1.435931E+03, 1.547611E+03, 1.660093E+03, 1.773607E+03, 1.888393E+03, 
   2.004700E+03, 2.122777E+03, 2.242871E+03, 2.365222E+03, 2.490065E+03, 2.617624E+03, 
   2.748119E+03, 2.881760E+03, 3.018750E+03, 3.159285E+03, 3.303554E+03, 3.451743E+03, 
   3.604030E+03, 3.760591E+03, 3.921599E+03, 4.087223E+03, 4.257630E+03, 4.432986E+03, 
   4.613454E+03, 4.799198E+03, 4.990381E+03, 5.187166E+03, 5.389715E+03, 5.598193E+03, 
   5.812764E+03, 6.033593E+03, 6.260847E+03, 6.494692E+03, 6.735300E+03, 6.982841E+03, 
   7.237487E+03, 7.499415E+03, 7.768800E+03, 8.045822E+03, 8.330662E+03, 8.623505E+03, 
   8.924535E+03, 9.233943E+03, 9.551919E+03, 9.878658E+03, 1.021436E+04, 1.055921E+04, 
   1.091343E+04, 1.127721E+04, 1.165077E+04, 1.203431E+04, 1.242805E+04, 1.283220E+04, 
   1.324700E+04, 1.367265E+04, 1.410938E+04, 1.455743E+04, 1.501703E+04, 1.548842E+04, 
   1.597182E+04, 1.646749E+04, 1.697567E+04, 1.749662E+04, 1.803057E+04, 1.857780E+04, 
   1.913855E+04, 1.971310E+04, 2.030170E+04, 2.090463E+04, 2.152217E+04, 2.215458E+04, 
   2.280215E+04, 2.346517E+04, 2.414392E+04, 2.483870E+04, 2.554979E+04, 2.627751E+04, 
   2.702215E+04, 2.778401E+04, 2.856342E+04, 2.936068E+04, 3.017611E+04, 3.101004E+04, 
   3.186279E+04, 3.273469E+04, 3.362608E+04, 3.453729E+04, 3.546867E+04, 3.642056E+04, 
   3.739332E+04, 3.838729E+04, 3.940283E+04, 4.044031E+04, 4.150010E+04, 4.258257E+04, 
   4.368808E+04, 4.481703E+04, 4.596979E+04, 4.714675E+04, 4.834831E+04, 4.957485E+04, 
   5.082679E+04, 5.210453E+04, 5.340848E+04, 5.473904E+04, 5.609665E+04, 5.748172E+04, 
   5.889468E+04, 6.033596E+04, 6.180600E+04, 6.330524E+04, 6.483413E+04, 6.639311E+04, 
   6.798265E+04, 6.960319E+04, 7.125521E+04, 7.293918E+04, 7.465556E+04, 7.640483E+04, 
   7.818749E+04, 8.000400E+04, 8.185488E+04, 8.374061E+04, 8.566170E+04, 8.761866E+04, 
   8.961199E+04, 9.164221E+04, 9.370984E+04, 9.581541E+04, 9.795946E+04, 1.001425E+05, 
   1.023651E+05, 1.046278E+05, 1.069311E+05, 1.092756E+05, 1.116619E+05, 1.140905E+05, 
   1.165621E+05, 1.190770E+05, 1.216361E+05, 1.242397E+05, 1.268886E+05, 1.295833E+05, 
   1.323244E+05, 1.351126E+05, 1.379484E+05, 1.408324E+05, 1.437653E+05, 1.467477E+05, 
   1.497802E+05, 1.528635E+05, 1.559982E+05, 1.591849E+05, 1.624244E+05, 1.657172E+05, 
   1.690639E+05, 1.724654E+05, 1.759222E+05, 1.794350E+05, 1.830045E+05, 1.866314E+05, 
   1.903164E+05, 1.940601E+05, 1.978633E+05, 2.017266E+05, 2.056509E+05, 2.096367E+05, 
   2.136848E+05, 2.177959E+05, 2.219708E+05, 2.262101E+05, 2.305147E+05, 2.348853E+05, 
   2.393225E+05, 2.438272E+05, 2.484001E+05, 2.530419E+05, 2.577535E+05, 2.625356E+05, 
   2.673889E+05, 2.723143E+05, 2.773125E+05, 2.823843E+05, 2.875304E+05, 2.927518E+05, 
   2.980491E+05, 3.034232E+05, 3.088749E+05, 3.144050E+05, 3.200142E+05, 3.257035E+05, 
   3.314736E+05, 3.373254E+05, 3.432596E+05, 3.492771E+05, 3.553788E+05, 3.615654E+05, 
   3.678379E+05, 3.741970E+05, 3.806436E+05, 3.871785E+05, 3.938026E+05, 4.005168E+05, 
   4.073219E+05, 4.142187E+05, 4.212082E+05, 4.282911E+05, 4.354685E+05, 4.427410E+05, 
   4.501096E+05, 4.575753E+05, 4.651387E+05, 4.728009E+05, 4.805627E+05, 4.884250E+05, 
   4.963887E+05, 5.044546E+05, 5.126236E+05, 5.208967E+05, 5.292748E+05, 5.377586E+05, 
   5.463492E+05, 5.550474E+05, 5.638540E+05, 5.727701E+05, 5.817965E+05, 5.909341E+05, 
   6.001837E+05, 6.095464E+05, 6.190230E+05, 6.286144E+05, 6.383214E+05, 6.481451E+05, 
   6.580863E+05, 6.681459E+05, 6.783248E+05, 6.886240E+05, 6.990442E+05, 7.095865E+05, 
   7.202517E+05, 7.310408E+05, 7.419545E+05, 7.529939E+05, 7.641599E+05, 7.754532E+05, 
   7.868749E+05, 7.984258E+05, 8.101069E+05, 8.219189E+05, 8.338629E+05, 8.459397E+05, 
   8.581502E+05, 8.704953E+05, 8.829758E+05, 8.955928E+05, 9.083470E+05, 9.212393E+05, 
   9.342706E+05, 9.474419E+05, 9.607539E+05, 9.742076E+05, 9.878038E+05, 1.001543E+06, 
   1.015427E+06, 1.029456E+06, 1.043631E+06, 1.057953E+06, 1.072423E+06, 1.087041E+06, 
   1.101809E+06, 1.116726E+06, 1.131795E+06, 1.147016E+06, 1.162390E+06, 1.177918E+06, 
   1.193600E+06, 1.209437E+06, 1.225430E+06, 1.241581E+06, 1.257889E+06, 1.274355E+06, 
   1.290982E+06, 1.307768E+06, 1.324715E+06, 1.341825E+06, 1.359096E+06, 1.376532E+06, 
   1.394131E+06, 1.411896E+06, 1.429826E+06, 1.447923E+06, 1.466187E+06, 1.484619E+06, 
   1.503220E+06, 1.521991E+06, 1.540932E+06, 1.560044E+06, 1.579328E+06, 1.598785E+06, 
   1.618415E+06, 1.638219E+06, 1.658198E+06, 1.678352E+06, 1.698683E+06, 1.719191E+06, 
   1.739876E+06, 1.760739E+06, 1.781782E+06, 1.803004E+06, 1.824407E+06, 1.845991E+06, 
   1.867756E+06, 1.889704E+06, 1.911835E+06, 1.934150E+06, 1.956650E+06, 1.979334E+06, 
   2.002204E+06, 2.025260E+06, 2.048503E+06, 2.071934E+06, 2.095553E+06, 2.119361E+06, 
   2.143358E+06, 2.167546E+06, 2.191923E+06, 2.216493E+06, 2.241253E+06, 2.266207E+06, 
   2.291353E+06, 2.316692E+06, 2.342226E+06, 2.367954E+06, 2.393878E+06, 2.419997E+06, 
   2.446312E+06, 2.472824E+06, 2.499534E+06, 
])
# ============================ M = 2, I = 5 ============================
M = 2
I = 5
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.521080E+00, 3.842867E+01, 7.618089E+01, 1.139364E+02, 1.516935E+02, 1.894517E+02, 
   2.272111E+02, 2.649722E+02, 3.027382E+02, 3.405178E+02, 3.783295E+02, 4.162050E+02, 
   4.541900E+02, 4.923440E+02, 5.307384E+02, 5.694535E+02, 6.085758E+02, 6.481948E+02, 
   6.884015E+02, 7.292860E+02, 7.709370E+02, 8.134404E+02, 8.568792E+02, 9.013335E+02, 
   9.468802E+02, 9.935933E+02, 1.041544E+03, 1.090802E+03, 1.141433E+03, 1.193503E+03, 
   1.247075E+03, 1.302211E+03, 1.358972E+03, 1.417419E+03, 1.477611E+03, 1.539607E+03, 
   1.603466E+03, 1.669247E+03, 1.737009E+03, 1.806810E+03, 1.878710E+03, 1.952768E+03, 
   2.029043E+03, 2.107597E+03, 2.188489E+03, 2.271781E+03, 2.357536E+03, 2.445816E+03, 
   2.536684E+03, 2.630205E+03, 2.726445E+03, 2.825468E+03, 2.927343E+03, 3.032136E+03, 
   3.139917E+03, 3.250756E+03, 3.364724E+03, 3.481891E+03, 3.602332E+03, 3.726120E+03, 
   3.853330E+03, 3.984038E+03, 4.118322E+03, 4.256260E+03, 4.397931E+03, 4.543417E+03, 
   4.692798E+03, 4.846159E+03, 5.003582E+03, 5.165155E+03, 5.330962E+03, 5.501093E+03, 
   5.675636E+03, 5.854682E+03, 6.038323E+03, 6.226651E+03, 6.419760E+03, 6.617746E+03, 
   6.820707E+03, 7.028739E+03, 7.241943E+03, 7.460419E+03, 7.684269E+03, 7.913597E+03, 
   8.148509E+03, 8.389109E+03, 8.635506E+03, 8.887809E+03, 9.146128E+03, 9.410575E+03, 
   9.681264E+03, 9.958309E+03, 1.024183E+04, 1.053193E+04, 1.082875E+04, 1.113239E+04, 
   1.144299E+04, 1.176066E+04, 1.208554E+04, 1.241774E+04, 1.275739E+04, 1.310463E+04, 
   1.345959E+04, 1.382239E+04, 1.419317E+04, 1.457208E+04, 1.495924E+04, 1.535479E+04, 
   1.575889E+04, 1.617165E+04, 1.659324E+04, 1.702380E+04, 1.746347E+04, 1.791240E+04, 
   1.837075E+04, 1.883866E+04, 1.931629E+04, 1.980379E+04, 2.030133E+04, 2.080905E+04, 
   2.132713E+04, 2.185571E+04, 2.239497E+04, 2.294507E+04, 2.350618E+04, 2.407847E+04, 
   2.466211E+04, 2.525727E+04, 2.586412E+04, 2.648284E+04, 2.711361E+04, 2.775661E+04, 
   2.841201E+04, 2.908001E+04, 2.976079E+04, 3.045453E+04, 3.116142E+04, 3.188166E+04, 
   3.261543E+04, 3.336292E+04, 3.412435E+04, 3.489989E+04, 3.568975E+04, 3.649414E+04, 
   3.731326E+04, 3.814730E+04, 3.899648E+04, 3.986101E+04, 4.074109E+04, 4.163695E+04, 
   4.254878E+04, 4.347682E+04, 4.442127E+04, 4.538236E+04, 4.636031E+04, 4.735535E+04, 
   4.836769E+04, 4.939756E+04, 5.044520E+04, 5.151084E+04, 5.259471E+04, 5.369704E+04, 
   5.481807E+04, 5.595804E+04, 5.711720E+04, 5.829577E+04, 5.949401E+04, 6.071216E+04, 
   6.195048E+04, 6.320920E+04, 6.448859E+04, 6.578889E+04, 6.711037E+04, 6.845327E+04, 
   6.981786E+04, 7.120440E+04, 7.261315E+04, 7.404437E+04, 7.549834E+04, 7.697532E+04, 
   7.847557E+04, 7.999938E+04, 8.154701E+04, 8.311874E+04, 8.471484E+04, 8.633560E+04, 
   8.798130E+04, 8.965221E+04, 9.134862E+04, 9.307082E+04, 9.481909E+04, 9.659372E+04, 
   9.839500E+04, 1.002232E+05, 1.020787E+05, 1.039617E+05, 1.058725E+05, 1.078114E+05, 
   1.097788E+05, 1.117749E+05, 1.138000E+05, 1.158545E+05, 1.179385E+05, 1.200526E+05, 
   1.221969E+05, 1.243717E+05, 1.265774E+05, 1.288143E+05, 1.310827E+05, 1.333829E+05, 
   1.357152E+05, 1.380800E+05, 1.404776E+05, 1.429082E+05, 1.453722E+05, 1.478700E+05, 
   1.504017E+05, 1.529679E+05, 1.555688E+05, 1.582047E+05, 1.608759E+05, 1.635828E+05, 
   1.663257E+05, 1.691049E+05, 1.719208E+05, 1.747737E+05, 1.776640E+05, 1.805919E+05, 
   1.835577E+05, 1.865619E+05, 1.896048E+05, 1.926867E+05, 1.958079E+05, 1.989687E+05, 
   2.021696E+05, 2.054108E+05, 2.086927E+05, 2.120156E+05, 2.153799E+05, 2.187859E+05, 
   2.222339E+05, 2.257243E+05, 2.292574E+05, 2.328336E+05, 2.364532E+05, 2.401165E+05, 
   2.438239E+05, 2.475757E+05, 2.513722E+05, 2.552139E+05, 2.591010E+05, 2.630340E+05, 
   2.670130E+05, 2.710385E+05, 2.751108E+05, 2.792303E+05, 2.833972E+05, 2.876120E+05, 
   2.918749E+05, 2.961864E+05, 3.005467E+05, 3.049562E+05, 3.094152E+05, 3.139241E+05, 
   3.184831E+05, 3.230927E+05, 3.277532E+05, 3.324649E+05, 3.372281E+05, 3.420432E+05, 
   3.469105E+05, 3.518304E+05, 3.568031E+05, 3.618291E+05, 3.669086E+05, 3.720419E+05, 
   3.772295E+05, 3.824716E+05, 3.877685E+05, 3.931206E+05, 3.985282E+05, 4.039917E+05, 
   4.095113E+05, 4.150873E+05, 4.207202E+05, 4.264102E+05, 4.321576E+05, 4.379628E+05, 
   4.438260E+05, 4.497476E+05, 4.557279E+05, 4.617673E+05, 4.678659E+05, 4.740242E+05, 
   4.802424E+05, 4.865208E+05, 4.928599E+05, 4.992597E+05, 5.057207E+05, 5.122432E+05, 
   5.188275E+05, 5.254738E+05, 5.321824E+05, 5.389537E+05, 5.457879E+05, 5.526854E+05, 
   5.596464E+05, 5.666712E+05, 5.737601E+05, 5.809134E+05, 5.881313E+05, 5.954142E+05, 
   6.027624E+05, 6.101760E+05, 6.176554E+05, 6.252009E+05, 6.328126E+05, 6.404910E+05, 
   6.482363E+05, 6.560486E+05, 6.639284E+05, 6.718758E+05, 6.798912E+05, 6.879747E+05, 
   6.961267E+05, 7.043473E+05, 7.126369E+05, 7.209957E+05, 7.294239E+05, 7.379218E+05, 
   7.464896E+05, 7.551276E+05, 7.638361E+05, 7.726151E+05, 7.814651E+05, 7.903861E+05, 
   7.993786E+05, 8.084426E+05, 8.175784E+05, 8.267862E+05, 8.360663E+05, 8.454189E+05, 
   8.548442E+05, 8.643423E+05, 8.739137E+05, 8.835583E+05, 8.932765E+05, 9.030684E+05, 
   9.129343E+05, 9.228744E+05, 9.328888E+05, 
])
# ============================ M = 2, I = 6 ============================
M = 2
I = 6
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.651606E+01, 2.243264E+02, 4.445925E+02, 6.648787E+02, 8.851739E+02, 1.105476E+03, 
   1.325785E+03, 1.546103E+03, 1.766449E+03, 1.986872E+03, 2.207477E+03, 2.428444E+03, 
   2.650035E+03, 2.872591E+03, 3.096522E+03, 3.322292E+03, 3.550398E+03, 3.781359E+03, 
   4.015699E+03, 4.253941E+03, 4.496597E+03, 4.744163E+03, 4.997122E+03, 5.255935E+03, 
   5.521046E+03, 5.792885E+03, 6.071862E+03, 6.358376E+03, 6.652812E+03, 6.955545E+03, 
   7.266940E+03, 7.587356E+03, 7.917143E+03, 8.256649E+03, 8.606215E+03, 8.966183E+03, 
   9.336891E+03, 9.718676E+03, 1.011188E+04, 1.051683E+04, 1.093387E+04, 1.136335E+04, 
   1.180560E+04, 1.226097E+04, 1.272982E+04, 1.321248E+04, 1.370933E+04, 1.422071E+04, 
   1.474700E+04, 1.528857E+04, 1.584577E+04, 1.641901E+04, 1.700865E+04, 1.761510E+04, 
   1.823873E+04, 1.887996E+04, 1.953919E+04, 2.021683E+04, 2.091330E+04, 2.162902E+04, 
   2.236443E+04, 2.311996E+04, 2.389604E+04, 2.469315E+04, 2.551171E+04, 2.635221E+04, 
   2.721511E+04, 2.810088E+04, 2.901001E+04, 2.994298E+04, 3.090030E+04, 3.188246E+04, 
   3.288998E+04, 3.392338E+04, 3.498317E+04, 3.606990E+04, 3.718410E+04, 3.832631E+04, 
   3.949710E+04, 4.069702E+04, 4.192665E+04, 4.318656E+04, 4.447733E+04, 4.579957E+04, 
   4.715386E+04, 4.854082E+04, 4.996106E+04, 5.141521E+04, 5.290390E+04, 5.442778E+04, 
   5.598748E+04, 5.758366E+04, 5.921700E+04, 6.088816E+04, 6.259783E+04, 6.434669E+04, 
   6.613544E+04, 6.796480E+04, 6.983547E+04, 7.174818E+04, 7.370367E+04, 7.570267E+04, 
   7.774594E+04, 7.983423E+04, 8.196832E+04, 8.414897E+04, 8.637699E+04, 8.865315E+04, 
   9.097828E+04, 9.335317E+04, 9.577865E+04, 9.825556E+04, 1.007847E+05, 1.033670E+05, 
   1.060033E+05, 1.086944E+05, 1.114413E+05, 1.142447E+05, 1.171057E+05, 1.200251E+05, 
   1.230038E+05, 1.260428E+05, 1.291430E+05, 1.323053E+05, 1.355307E+05, 1.388203E+05, 
   1.421748E+05, 1.455954E+05, 1.490831E+05, 1.526388E+05, 1.562635E+05, 1.599583E+05, 
   1.637242E+05, 1.675624E+05, 1.714737E+05, 1.754593E+05, 1.795203E+05, 1.836578E+05, 
   1.878728E+05, 1.921665E+05, 1.965400E+05, 2.009945E+05, 2.055309E+05, 2.101506E+05, 
   2.148547E+05, 2.196444E+05, 2.245208E+05, 2.294851E+05, 2.345385E+05, 2.396823E+05, 
   2.449177E+05, 2.502459E+05, 2.556682E+05, 2.611858E+05, 2.668000E+05, 2.725121E+05, 
   2.783233E+05, 2.842351E+05, 2.902486E+05, 2.963652E+05, 3.025862E+05, 3.089131E+05, 
   3.153471E+05, 3.218896E+05, 3.285420E+05, 3.353057E+05, 3.421821E+05, 3.491725E+05, 
   3.562785E+05, 3.635014E+05, 3.708427E+05, 3.783038E+05, 3.858862E+05, 3.935914E+05, 
   4.014209E+05, 4.093761E+05, 4.174586E+05, 4.256698E+05, 4.340114E+05, 4.424848E+05, 
   4.510916E+05, 4.598333E+05, 4.687115E+05, 4.777278E+05, 4.868838E+05, 4.961810E+05, 
   5.056212E+05, 5.152058E+05, 5.249365E+05, 5.348150E+05, 5.448429E+05, 5.550219E+05, 
   5.653535E+05, 5.758396E+05, 5.864817E+05, 5.972816E+05, 6.082409E+05, 6.193614E+05, 
   6.306448E+05, 6.420929E+05, 6.537072E+05, 6.654897E+05, 6.774420E+05, 6.895660E+05, 
   7.018632E+05, 7.143357E+05, 7.269850E+05, 7.398131E+05, 7.528217E+05, 7.660127E+05, 
   7.793877E+05, 7.929487E+05, 8.066975E+05, 8.206359E+05, 8.347658E+05, 8.490890E+05, 
   8.636073E+05, 8.783226E+05, 8.932368E+05, 9.083518E+05, 9.236693E+05, 9.391914E+05, 
   9.549198E+05, 9.708566E+05, 9.870035E+05, 1.003362E+06, 1.019935E+06, 1.036724E+06, 
   1.053731E+06, 1.070957E+06, 1.088405E+06, 1.106077E+06, 1.123974E+06, 1.142098E+06, 
   1.160452E+06, 1.179038E+06, 1.197856E+06, 1.216910E+06, 1.236200E+06, 1.255730E+06, 
   1.275501E+06, 1.295515E+06, 1.315774E+06, 1.336279E+06, 1.357034E+06, 1.378039E+06, 
   1.399297E+06, 1.420810E+06, 1.442580E+06, 1.464608E+06, 1.486897E+06, 1.509448E+06, 
   1.532264E+06, 1.555346E+06, 1.578697E+06, 1.602318E+06, 1.626211E+06, 1.650379E+06, 
   1.674823E+06, 1.699545E+06, 1.724548E+06, 1.749832E+06, 1.775400E+06, 1.801255E+06, 
   1.827397E+06, 1.853829E+06, 1.880553E+06, 1.907571E+06, 1.934884E+06, 1.962495E+06, 
   1.990405E+06, 2.018617E+06, 2.047132E+06, 2.075952E+06, 2.105079E+06, 2.134515E+06, 
   2.164262E+06, 2.194322E+06, 2.224697E+06, 2.255388E+06, 2.286397E+06, 2.317727E+06, 
   2.349379E+06, 2.381354E+06, 2.413656E+06, 2.446285E+06, 2.479244E+06, 2.512534E+06, 
   2.546157E+06, 2.580115E+06, 2.614410E+06, 2.649043E+06, 2.684016E+06, 2.719332E+06, 
   2.754992E+06, 2.790997E+06, 2.827349E+06, 2.864051E+06, 2.901104E+06, 2.938509E+06, 
   2.976268E+06, 3.014384E+06, 3.052857E+06, 3.091690E+06, 3.130883E+06, 3.170440E+06, 
   3.210361E+06, 3.250648E+06, 3.291303E+06, 3.332328E+06, 3.373723E+06, 3.415491E+06, 
   3.457633E+06, 3.500151E+06, 3.543047E+06, 3.586321E+06, 3.629976E+06, 3.674014E+06, 
   3.718434E+06, 3.763241E+06, 3.808433E+06, 3.854014E+06, 3.899985E+06, 3.946347E+06, 
   3.993102E+06, 4.040250E+06, 4.087795E+06, 4.135736E+06, 4.184076E+06, 4.232815E+06, 
   4.281956E+06, 4.331500E+06, 4.381448E+06, 4.431801E+06, 4.482561E+06, 4.533729E+06, 
   4.585306E+06, 4.637294E+06, 4.689695E+06, 4.742508E+06, 4.795737E+06, 4.849381E+06, 
   4.903442E+06, 4.957922E+06, 5.012821E+06, 5.068142E+06, 5.123884E+06, 5.180049E+06, 
   5.236639E+06, 5.293655E+06, 5.351097E+06, 
])
# ============================ M = 2, I = 7 ============================
M = 2
I = 7
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.250890E+00, 1.018897E+01, 2.020899E+01, 3.022982E+01, 4.025104E+01, 5.027255E+01, 
   6.029436E+01, 7.031658E+01, 8.033986E+01, 9.036614E+01, 1.003996E+02, 1.104476E+02, 
   1.205209E+02, 1.306339E+02, 1.408040E+02, 1.510512E+02, 1.613971E+02, 1.718645E+02, 
   1.824767E+02, 1.932569E+02, 2.042283E+02, 2.154134E+02, 2.268340E+02, 2.385113E+02, 
   2.504658E+02, 2.627173E+02, 2.752848E+02, 2.881867E+02, 3.014409E+02, 3.150649E+02, 
   3.290757E+02, 3.434898E+02, 3.583237E+02, 3.735933E+02, 3.893145E+02, 4.055031E+02, 
   4.221748E+02, 4.393450E+02, 4.570293E+02, 4.752432E+02, 4.940023E+02, 5.133222E+02, 
   5.332187E+02, 5.537074E+02, 5.748044E+02, 5.965256E+02, 6.188873E+02, 6.419059E+02, 
   6.655979E+02, 6.899801E+02, 7.150694E+02, 7.408830E+02, 7.674382E+02, 7.947528E+02, 
   8.228445E+02, 8.517315E+02, 8.814321E+02, 9.119649E+02, 9.433489E+02, 9.756031E+02, 
   1.008747E+03, 1.042800E+03, 1.077783E+03, 1.113715E+03, 1.150618E+03, 1.188511E+03, 
   1.227417E+03, 1.267357E+03, 1.308352E+03, 1.350424E+03, 1.393596E+03, 1.437891E+03, 
   1.483332E+03, 1.529942E+03, 1.577744E+03, 1.626764E+03, 1.677024E+03, 1.728550E+03, 
   1.781366E+03, 1.835499E+03, 1.890973E+03, 1.947815E+03, 2.006051E+03, 2.065708E+03, 
   2.126813E+03, 2.189394E+03, 2.253477E+03, 2.319092E+03, 2.386266E+03, 2.455029E+03, 
   2.525410E+03, 2.597439E+03, 2.671145E+03, 2.746558E+03, 2.823711E+03, 2.902633E+03, 
   2.983356E+03, 3.065912E+03, 3.150333E+03, 3.236652E+03, 3.324902E+03, 3.415117E+03, 
   3.507329E+03, 3.601574E+03, 3.697886E+03, 3.796300E+03, 3.896851E+03, 3.999575E+03, 
   4.104509E+03, 4.211689E+03, 4.321152E+03, 4.432935E+03, 4.547077E+03, 4.663616E+03, 
   4.782590E+03, 4.904039E+03, 5.028001E+03, 5.154518E+03, 5.283630E+03, 5.415376E+03, 
   5.549800E+03, 5.686941E+03, 5.826843E+03, 5.969548E+03, 6.115099E+03, 6.263540E+03, 
   6.414913E+03, 6.569265E+03, 6.726639E+03, 6.887081E+03, 7.050636E+03, 7.217351E+03, 
   7.387272E+03, 7.560446E+03, 7.736920E+03, 7.916744E+03, 8.099964E+03, 8.286630E+03, 
   8.476792E+03, 8.670499E+03, 8.867801E+03, 9.068749E+03, 9.273394E+03, 9.481789E+03, 
   9.693984E+03, 9.910034E+03, 1.012999E+04, 1.035391E+04, 1.058184E+04, 1.081384E+04, 
   1.104996E+04, 1.129027E+04, 1.153481E+04, 1.178364E+04, 1.203682E+04, 1.229441E+04, 
   1.255646E+04, 1.282303E+04, 1.309419E+04, 1.336999E+04, 1.365049E+04, 1.393575E+04, 
   1.422584E+04, 1.452080E+04, 1.482072E+04, 1.512564E+04, 1.543563E+04, 1.575076E+04, 
   1.607109E+04, 1.639668E+04, 1.672760E+04, 1.706391E+04, 1.740569E+04, 1.775298E+04, 
   1.810588E+04, 1.846443E+04, 1.882871E+04, 1.919879E+04, 1.957473E+04, 1.995662E+04, 
   2.034450E+04, 2.073847E+04, 2.113858E+04, 2.154491E+04, 2.195754E+04, 2.237653E+04, 
   2.280195E+04, 2.323389E+04, 2.367242E+04, 2.411761E+04, 2.456953E+04, 2.502826E+04, 
   2.549389E+04, 2.596648E+04, 2.644612E+04, 2.693287E+04, 2.742683E+04, 2.792806E+04, 
   2.843666E+04, 2.895269E+04, 2.947625E+04, 3.000740E+04, 3.054624E+04, 3.109285E+04, 
   3.164730E+04, 3.220969E+04, 3.278009E+04, 3.335860E+04, 3.394529E+04, 3.454025E+04, 
   3.514357E+04, 3.575534E+04, 3.637564E+04, 3.700455E+04, 3.764218E+04, 3.828861E+04, 
   3.894392E+04, 3.960821E+04, 4.028157E+04, 4.096408E+04, 4.165585E+04, 4.235696E+04, 
   4.306751E+04, 4.378759E+04, 4.451729E+04, 4.525671E+04, 4.600595E+04, 4.676509E+04, 
   4.753424E+04, 4.831349E+04, 4.910294E+04, 4.990269E+04, 5.071284E+04, 5.153348E+04, 
   5.236471E+04, 5.320664E+04, 5.405937E+04, 5.492299E+04, 5.579761E+04, 5.668334E+04, 
   5.758027E+04, 5.848850E+04, 5.940815E+04, 6.033931E+04, 6.128210E+04, 6.223661E+04, 
   6.320296E+04, 6.418125E+04, 6.517158E+04, 6.617407E+04, 6.718883E+04, 6.821596E+04, 
   6.925557E+04, 7.030778E+04, 7.137269E+04, 7.245042E+04, 7.354107E+04, 7.464476E+04, 
   7.576161E+04, 7.689172E+04, 7.803522E+04, 7.919220E+04, 8.036280E+04, 8.154713E+04, 
   8.274529E+04, 8.395742E+04, 8.518362E+04, 8.642401E+04, 8.767872E+04, 8.894785E+04, 
   9.023154E+04, 9.152990E+04, 9.284305E+04, 9.417111E+04, 9.551420E+04, 9.687245E+04, 
   9.824598E+04, 9.963491E+04, 1.010394E+05, 1.024595E+05, 1.038953E+05, 1.053471E+05, 
   1.068149E+05, 1.082989E+05, 1.097991E+05, 1.113157E+05, 1.128489E+05, 1.143987E+05, 
   1.159653E+05, 1.175489E+05, 1.191494E+05, 1.207672E+05, 1.224023E+05, 1.240548E+05, 
   1.257249E+05, 1.274126E+05, 1.291182E+05, 1.308418E+05, 1.325835E+05, 1.343434E+05, 
   1.361217E+05, 1.379185E+05, 1.397340E+05, 1.415682E+05, 1.434213E+05, 1.452935E+05, 
   1.471849E+05, 1.490956E+05, 1.510257E+05, 1.529755E+05, 1.549449E+05, 1.569343E+05, 
   1.589437E+05, 1.609732E+05, 1.630231E+05, 1.650933E+05, 1.671842E+05, 1.692958E+05, 
   1.714282E+05, 1.735816E+05, 1.757562E+05, 1.779521E+05, 1.801694E+05, 1.824083E+05, 
   1.846689E+05, 1.869514E+05, 1.892559E+05, 1.915826E+05, 1.939315E+05, 1.963029E+05, 
   1.986969E+05, 2.011136E+05, 2.035533E+05, 2.060159E+05, 2.085017E+05, 2.110109E+05, 
   2.135436E+05, 2.160998E+05, 2.186799E+05, 2.212838E+05, 2.239119E+05, 2.265642E+05, 
   2.292408E+05, 2.319420E+05, 2.346678E+05, 2.374184E+05, 2.401941E+05, 2.429948E+05, 
   2.458208E+05, 2.486723E+05, 2.515493E+05, 2.544521E+05, 2.573807E+05, 2.603354E+05, 
   2.633163E+05, 2.663235E+05, 2.693572E+05, 2.724175E+05, 2.755047E+05, 2.786188E+05, 
   2.817600E+05, 2.849284E+05, 2.881243E+05, 2.913477E+05, 2.945989E+05, 2.978779E+05, 
   3.011850E+05, 3.045202E+05, 3.078838E+05, 3.112759E+05, 3.146966E+05, 3.181461E+05, 
   3.216246E+05, 3.251322E+05, 3.286691E+05, 3.322353E+05, 3.358312E+05, 3.394568E+05, 
   3.431123E+05, 3.467978E+05, 3.505135E+05, 3.542596E+05, 3.580362E+05, 3.618435E+05, 
   3.656815E+05, 3.695506E+05, 3.734508E+05, 3.773822E+05, 3.813451E+05, 3.853396E+05, 
   3.893659E+05, 3.934241E+05, 3.975143E+05, 4.016367E+05, 4.057916E+05, 4.099789E+05, 
   4.141989E+05, 4.184518E+05, 4.227377E+05, 4.270567E+05, 4.314090E+05, 4.357948E+05, 
   4.402142E+05, 4.446673E+05, 4.491544E+05, 4.536755E+05, 4.582309E+05, 4.628207E+05, 
   4.674450E+05, 4.721040E+05, 4.767978E+05, 4.815266E+05, 4.862906E+05, 4.910899E+05, 
   4.959246E+05, 5.007950E+05, 5.057011E+05, 5.106431E+05, 5.156212E+05, 5.206355E+05, 
   5.256861E+05, 5.307733E+05, 5.358972E+05, 5.410578E+05, 5.462555E+05, 5.514903E+05, 
   5.567623E+05, 5.620718E+05, 5.674188E+05, 5.728036E+05, 5.782262E+05, 5.836868E+05, 
   5.891856E+05, 5.947228E+05, 6.002984E+05, 6.059126E+05, 6.115655E+05, 6.172574E+05, 
   6.229883E+05, 6.287584E+05, 6.345679E+05, 6.404169E+05, 6.463054E+05, 6.522338E+05, 
   6.582021E+05, 6.642105E+05, 6.702590E+05, 6.763480E+05, 6.824774E+05, 6.886475E+05, 
   6.948584E+05, 7.011101E+05, 7.074030E+05, 7.137371E+05, 7.201125E+05, 7.265294E+05, 
   7.329880E+05, 7.394883E+05, 7.460305E+05, 7.526148E+05, 7.592412E+05, 7.659100E+05, 
   7.726213E+05, 7.793751E+05, 7.861717E+05, 7.930112E+05, 7.998937E+05, 8.068193E+05, 
   8.137882E+05, 8.208005E+05, 8.278563E+05, 8.349559E+05, 8.420992E+05, 8.492865E+05, 
   8.565179E+05, 8.637935E+05, 8.711134E+05, 8.784778E+05, 8.858868E+05, 8.933405E+05, 
   9.008391E+05, 9.083827E+05, 9.159714E+05, 9.236053E+05, 9.312846E+05, 9.390094E+05, 
   9.467798E+05, 9.545960E+05, 9.624580E+05, 9.703660E+05, 9.783202E+05, 9.863206E+05, 
   9.943673E+05, 1.002461E+06, 1.010600E+06, 1.018787E+06, 1.027020E+06, 1.035301E+06, 
   1.043628E+06, 1.052003E+06, 1.060425E+06, 
])
# ============================ M = 2, I = 8 ============================
M = 2
I = 8
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.391189E+01, 1.188565E+02, 2.356850E+02, 3.525234E+02, 4.693664E+02, 5.862129E+02, 
   7.030628E+02, 8.199175E+02, 9.367842E+02, 1.053685E+03, 1.170667E+03, 1.287813E+03, 
   1.405248E+03, 1.523136E+03, 1.641677E+03, 1.761100E+03, 1.881655E+03, 2.003605E+03, 
   2.127217E+03, 2.252763E+03, 2.380506E+03, 2.510709E+03, 2.643624E+03, 2.779494E+03, 
   2.918557E+03, 3.061041E+03, 3.207165E+03, 3.357141E+03, 3.511177E+03, 3.669473E+03, 
   3.832224E+03, 3.999622E+03, 4.171854E+03, 4.349105E+03, 4.531559E+03, 4.719395E+03, 
   4.912793E+03, 5.111932E+03, 5.316991E+03, 5.528147E+03, 5.745580E+03, 5.969468E+03, 
   6.199992E+03, 6.437333E+03, 6.681674E+03, 6.933199E+03, 7.192093E+03, 7.458545E+03, 
   7.732745E+03, 8.014884E+03, 8.305157E+03, 8.603761E+03, 8.910895E+03, 9.226761E+03, 
   9.551564E+03, 9.885512E+03, 1.022881E+04, 1.058168E+04, 1.094434E+04, 1.131699E+04, 
   1.169988E+04, 1.209321E+04, 1.249723E+04, 1.291216E+04, 1.333824E+04, 1.377570E+04, 
   1.422480E+04, 1.468577E+04, 1.515886E+04, 1.564434E+04, 1.614244E+04, 1.665344E+04, 
   1.717760E+04, 1.771518E+04, 1.826646E+04, 1.883171E+04, 1.941121E+04, 2.000524E+04, 
   2.061408E+04, 2.123804E+04, 2.187739E+04, 2.253245E+04, 2.320350E+04, 2.389086E+04, 
   2.459484E+04, 2.531575E+04, 2.605391E+04, 2.680963E+04, 2.758326E+04, 2.837510E+04, 
   2.918551E+04, 3.001482E+04, 3.086338E+04, 3.173152E+04, 3.261960E+04, 3.352798E+04, 
   3.445702E+04, 3.540707E+04, 3.637852E+04, 3.737173E+04, 3.838709E+04, 3.942496E+04, 
   4.048575E+04, 4.156984E+04, 4.267762E+04, 4.380951E+04, 4.496589E+04, 4.614719E+04, 
   4.735382E+04, 4.858619E+04, 4.984473E+04, 5.112987E+04, 5.244204E+04, 5.378168E+04, 
   5.514923E+04, 5.654514E+04, 5.796986E+04, 5.942385E+04, 6.090757E+04, 6.242149E+04, 
   6.396608E+04, 6.554182E+04, 6.714918E+04, 6.878866E+04, 7.046075E+04, 7.216595E+04, 
   7.390475E+04, 7.567767E+04, 7.748521E+04, 7.932790E+04, 8.120625E+04, 8.312080E+04, 
   8.507207E+04, 8.706061E+04, 8.908696E+04, 9.115166E+04, 9.325528E+04, 9.539836E+04, 
   9.758148E+04, 9.980520E+04, 1.020701E+05, 1.043768E+05, 1.067258E+05, 1.091177E+05, 
   1.115532E+05, 1.140328E+05, 1.165571E+05, 1.191268E+05, 1.217425E+05, 1.244047E+05, 
   1.271142E+05, 1.298715E+05, 1.326774E+05, 1.355323E+05, 1.384371E+05, 1.413923E+05, 
   1.443986E+05, 1.474567E+05, 1.505672E+05, 1.537308E+05, 1.569483E+05, 1.602202E+05, 
   1.635474E+05, 1.669304E+05, 1.703700E+05, 1.738669E+05, 1.774218E+05, 1.810355E+05, 
   1.847086E+05, 1.884419E+05, 1.922361E+05, 1.960920E+05, 2.000103E+05, 2.039918E+05, 
   2.080372E+05, 2.121473E+05, 2.163229E+05, 2.205647E+05, 2.248735E+05, 2.292500E+05, 
   2.336952E+05, 2.382098E+05, 2.427945E+05, 2.474502E+05, 2.521776E+05, 2.569777E+05, 
   2.618512E+05, 2.667989E+05, 2.718217E+05, 2.769204E+05, 2.820959E+05, 2.873489E+05, 
   2.926803E+05, 2.980910E+05, 3.035819E+05, 3.091537E+05, 3.148074E+05, 3.205439E+05, 
   3.263639E+05, 3.322684E+05, 3.382582E+05, 3.443343E+05, 3.504976E+05, 3.567488E+05, 
   3.630890E+05, 3.695190E+05, 3.760398E+05, 3.826522E+05, 3.893571E+05, 3.961555E+05, 
   4.030483E+05, 4.100364E+05, 4.171208E+05, 4.243023E+05, 4.315819E+05, 4.389606E+05, 
   4.464392E+05, 4.540188E+05, 4.617002E+05, 4.694844E+05, 4.773724E+05, 4.853651E+05, 
   4.934635E+05, 5.016685E+05, 5.099811E+05, 5.184022E+05, 5.269329E+05, 5.355740E+05, 
   5.443266E+05, 5.531916E+05, 5.621700E+05, 5.712628E+05, 5.804710E+05, 5.897954E+05, 
   5.992372E+05, 6.087972E+05, 6.184765E+05, 6.282760E+05, 6.381968E+05, 6.482398E+05, 
   6.584059E+05, 6.686963E+05, 6.791119E+05, 6.896536E+05, 7.003224E+05, 7.111194E+05, 
   7.220455E+05, 7.331017E+05, 7.442890E+05, 7.556085E+05, 7.670610E+05, 7.786475E+05, 
   7.903691E+05, 8.022268E+05, 8.142215E+05, 8.263542E+05, 8.386259E+05, 8.510376E+05, 
   8.635903E+05, 8.762849E+05, 8.891225E+05, 9.021039E+05, 9.152303E+05, 9.285025E+05, 
   9.419216E+05, 9.554886E+05, 9.692043E+05, 9.830698E+05, 9.970860E+05, 1.011254E+06, 
   1.025575E+06, 1.040049E+06, 1.054678E+06, 1.069462E+06, 1.084403E+06, 1.099501E+06, 
   1.114758E+06, 1.130174E+06, 1.145751E+06, 1.161488E+06, 1.177388E+06, 1.193451E+06, 
   1.209678E+06, 1.226070E+06, 1.242628E+06, 1.259353E+06, 1.276246E+06, 1.293307E+06, 
   1.310538E+06, 1.327939E+06, 1.345512E+06, 1.363257E+06, 1.381175E+06, 1.399268E+06, 
   1.417535E+06, 1.435978E+06, 1.454598E+06, 1.473396E+06, 1.492372E+06, 1.511528E+06, 
   1.530863E+06, 1.550380E+06, 1.570079E+06, 1.589961E+06, 1.610026E+06, 1.630275E+06, 
   1.650710E+06, 1.671332E+06, 1.692140E+06, 1.713136E+06, 1.734320E+06, 1.755694E+06, 
   1.777258E+06, 1.799013E+06, 1.820960E+06, 1.843100E+06, 1.865433E+06, 1.887960E+06, 
   1.910682E+06, 1.933600E+06, 1.956714E+06, 1.980025E+06, 2.003535E+06, 2.027243E+06, 
   2.051150E+06, 2.075258E+06, 2.099566E+06, 2.124076E+06, 2.148789E+06, 2.173704E+06, 
   2.198824E+06, 2.224147E+06, 2.249676E+06, 2.275411E+06, 2.301352E+06, 2.327500E+06, 
   2.353856E+06, 2.380420E+06, 2.407194E+06, 2.434177E+06, 2.461371E+06, 2.488775E+06, 
   2.516392E+06, 2.544220E+06, 2.572262E+06, 2.600517E+06, 2.628986E+06, 2.657670E+06, 
   2.686569E+06, 2.715684E+06, 2.745015E+06, 
])
# ============================ M = 2, I = 9 ============================
M = 2
I = 9
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.124397E+01, 3.467766E+02, 6.874652E+02, 1.028184E+03, 1.368916E+03, 1.709658E+03, 
   2.050410E+03, 2.391176E+03, 2.731976E+03, 3.072872E+03, 3.413998E+03, 3.755592E+03, 
   4.098007E+03, 4.441714E+03, 4.787289E+03, 5.135392E+03, 5.486743E+03, 5.842097E+03, 
   6.202233E+03, 6.567926E+03, 6.939947E+03, 7.319048E+03, 7.705959E+03, 8.101388E+03, 
   8.506016E+03, 8.920501E+03, 9.345477E+03, 9.781559E+03, 1.022934E+04, 1.068940E+04, 
   1.116230E+04, 1.164860E+04, 1.214882E+04, 1.266352E+04, 1.319320E+04, 1.373839E+04, 
   1.429960E+04, 1.487735E+04, 1.547216E+04, 1.608453E+04, 1.671497E+04, 1.736401E+04, 
   1.803216E+04, 1.871993E+04, 1.942786E+04, 2.015647E+04, 2.090629E+04, 2.167787E+04, 
   2.247174E+04, 2.328846E+04, 2.412860E+04, 2.499270E+04, 2.588134E+04, 2.679511E+04, 
   2.773459E+04, 2.870037E+04, 2.969305E+04, 3.071326E+04, 3.176160E+04, 3.283871E+04, 
   3.394522E+04, 3.508179E+04, 3.624906E+04, 3.744771E+04, 3.867840E+04, 3.994182E+04, 
   4.123867E+04, 4.256965E+04, 4.393547E+04, 4.533686E+04, 4.677456E+04, 4.824930E+04, 
   4.976184E+04, 5.131295E+04, 5.290340E+04, 5.453398E+04, 5.620549E+04, 5.791873E+04, 
   5.967453E+04, 6.147371E+04, 6.331712E+04, 6.520560E+04, 6.714003E+04, 6.912128E+04, 
   7.115023E+04, 7.322778E+04, 7.535485E+04, 7.753234E+04, 7.976121E+04, 8.204239E+04, 
   8.437683E+04, 8.676552E+04, 8.920942E+04, 9.170953E+04, 9.426687E+04, 9.688244E+04, 
   9.955728E+04, 1.022924E+05, 1.050889E+05, 1.079479E+05, 1.108703E+05, 1.138574E+05, 
   1.169102E+05, 1.200298E+05, 1.232174E+05, 1.264741E+05, 1.298010E+05, 1.331994E+05, 
   1.366705E+05, 1.402153E+05, 1.438352E+05, 1.475314E+05, 1.513051E+05, 1.551575E+05, 
   1.590899E+05, 1.631037E+05, 1.672000E+05, 1.713803E+05, 1.756458E+05, 1.799979E+05, 
   1.844378E+05, 1.889671E+05, 1.935870E+05, 1.982990E+05, 2.031044E+05, 2.080047E+05, 
   2.130014E+05, 2.180958E+05, 2.232894E+05, 2.285838E+05, 2.339804E+05, 2.394808E+05, 
   2.450863E+05, 2.507987E+05, 2.566195E+05, 2.625501E+05, 2.685923E+05, 2.747476E+05, 
   2.810176E+05, 2.874039E+05, 2.939083E+05, 3.005323E+05, 3.072777E+05, 3.141462E+05, 
   3.211394E+05, 3.282590E+05, 3.355069E+05, 3.428848E+05, 3.503944E+05, 3.580376E+05, 
   3.658161E+05, 3.737318E+05, 3.817865E+05, 3.899820E+05, 3.983203E+05, 4.068032E+05, 
   4.154326E+05, 4.242104E+05, 4.331386E+05, 4.422191E+05, 4.514539E+05, 4.608450E+05, 
   4.703944E+05, 4.801040E+05, 4.899760E+05, 5.000123E+05, 5.102151E+05, 5.205865E+05, 
   5.311284E+05, 5.418432E+05, 5.527328E+05, 5.637994E+05, 5.750453E+05, 5.864725E+05, 
   5.980834E+05, 6.098801E+05, 6.218648E+05, 6.340399E+05, 6.464075E+05, 6.589700E+05, 
   6.717298E+05, 6.846890E+05, 6.978501E+05, 7.112154E+05, 7.247874E+05, 7.385683E+05, 
   7.525606E+05, 7.667668E+05, 7.811893E+05, 7.958305E+05, 8.106929E+05, 8.257792E+05, 
   8.410916E+05, 8.566329E+05, 8.724056E+05, 8.884121E+05, 9.046553E+05, 9.211375E+05, 
   9.378615E+05, 9.548300E+05, 9.720455E+05, 9.895108E+05, 1.007229E+06, 1.025201E+06, 
   1.043432E+06, 1.061924E+06, 1.080679E+06, 1.099700E+06, 1.118991E+06, 1.138553E+06, 
   1.158390E+06, 1.178504E+06, 1.198899E+06, 1.219578E+06, 1.240543E+06, 1.261797E+06, 
   1.283343E+06, 1.305185E+06, 1.327324E+06, 1.349765E+06, 1.372511E+06, 1.395563E+06, 
   1.418926E+06, 1.442602E+06, 1.466595E+06, 1.490908E+06, 1.515544E+06, 1.540505E+06, 
   1.565796E+06, 1.591419E+06, 1.617378E+06, 1.643676E+06, 1.670316E+06, 1.697301E+06, 
   1.724635E+06, 1.752322E+06, 1.780363E+06, 1.808763E+06, 1.837526E+06, 1.866654E+06, 
   1.896150E+06, 1.926020E+06, 1.956265E+06, 1.986889E+06, 2.017896E+06, 2.049289E+06, 
   2.081072E+06, 2.113249E+06, 2.145822E+06, 2.178796E+06, 2.212173E+06, 2.245959E+06, 
   2.280156E+06, 2.314768E+06, 2.349798E+06, 2.385251E+06, 2.421130E+06, 2.457439E+06, 
   2.494182E+06, 2.531361E+06, 2.568982E+06, 2.607048E+06, 2.645562E+06, 2.684529E+06, 
   2.723953E+06, 2.763837E+06, 2.804185E+06, 2.845001E+06, 2.886290E+06, 2.928054E+06, 
   2.970298E+06, 3.013027E+06, 3.056243E+06, 3.099951E+06, 3.144156E+06, 3.188860E+06, 
   3.234069E+06, 3.279786E+06, 3.326015E+06, 3.372761E+06, 3.420028E+06, 3.467819E+06, 
   3.516140E+06, 3.564993E+06, 3.614384E+06, 3.664317E+06, 3.714796E+06, 3.765825E+06, 
   3.817408E+06, 3.869550E+06, 3.922256E+06, 3.975528E+06, 4.029373E+06, 4.083794E+06, 
   4.138795E+06, 4.194381E+06, 4.250556E+06, 4.307325E+06, 4.364693E+06, 4.422663E+06, 
   4.481240E+06, 4.540429E+06, 4.600234E+06, 4.660660E+06, 4.721710E+06, 4.783391E+06, 
   4.845706E+06, 4.908660E+06, 4.972258E+06, 5.036503E+06, 5.101402E+06, 5.166957E+06, 
   5.233175E+06, 5.300060E+06, 5.367616E+06, 5.435848E+06, 5.504761E+06, 5.574359E+06, 
   5.644648E+06, 5.715631E+06, 5.787315E+06, 5.859703E+06, 5.932800E+06, 6.006612E+06, 
   6.081142E+06, 6.156396E+06, 6.232379E+06, 6.309096E+06, 6.386550E+06, 6.464748E+06, 
   6.543694E+06, 6.623393E+06, 6.703849E+06, 6.785069E+06, 6.867056E+06, 6.949816E+06, 
   7.033353E+06, 7.117673E+06, 7.202781E+06, 7.288681E+06, 7.375378E+06, 7.462878E+06, 
   7.551186E+06, 7.640306E+06, 7.730243E+06, 7.821003E+06, 7.912590E+06, 8.005010E+06, 
   8.098268E+06, 8.192369E+06, 8.287317E+06, 8.383118E+06, 8.479777E+06, 8.577299E+06, 
   8.675690E+06, 8.774953E+06, 8.875095E+06, 8.976120E+06, 9.078035E+06, 9.180842E+06, 
   9.284549E+06, 9.389160E+06, 9.494679E+06, 9.601113E+06, 9.708467E+06, 9.816745E+06, 
   9.925953E+06, 1.003610E+07, 1.014718E+07, 1.025921E+07, 1.037219E+07, 1.048612E+07, 
   1.060102E+07, 1.071688E+07, 1.083371E+07, 1.095152E+07, 1.107031E+07, 1.119009E+07, 
   1.131086E+07, 1.143262E+07, 1.155539E+07, 1.167917E+07, 1.180396E+07, 1.192977E+07, 
   1.205660E+07, 1.218446E+07, 1.231335E+07, 1.244328E+07, 1.257426E+07, 1.270628E+07, 
   1.283936E+07, 1.297350E+07, 1.310870E+07, 1.324498E+07, 1.338233E+07, 1.352076E+07, 
   1.366027E+07, 1.380088E+07, 1.394258E+07, 1.408538E+07, 1.422929E+07, 1.437432E+07, 
   1.452045E+07, 1.466771E+07, 1.481610E+07, 1.496562E+07, 1.511628E+07, 1.526808E+07, 
   1.542103E+07, 1.557513E+07, 1.573039E+07, 1.588681E+07, 1.604440E+07, 1.620316E+07, 
   1.636310E+07, 1.652423E+07, 1.668655E+07, 1.685005E+07, 1.701476E+07, 1.718067E+07, 
   1.734779E+07, 1.751613E+07, 1.768569E+07, 1.785646E+07, 1.802847E+07, 1.820172E+07, 
   1.837620E+07, 1.855193E+07, 1.872890E+07, 1.890713E+07, 1.908663E+07, 1.926738E+07, 
   1.944941E+07, 1.963271E+07, 1.981729E+07, 2.000315E+07, 2.019030E+07, 2.037875E+07, 
   2.056850E+07, 2.075955E+07, 2.095191E+07, 2.114558E+07, 2.134058E+07, 2.153689E+07, 
   2.173454E+07, 2.193352E+07, 2.213383E+07, 2.233549E+07, 2.253850E+07, 2.274286E+07, 
   2.294858E+07, 2.315566E+07, 2.336411E+07, 2.357393E+07, 2.378512E+07, 2.399770E+07, 
   2.421166E+07, 2.442702E+07, 2.464377E+07, 2.486192E+07, 2.508147E+07, 2.530243E+07, 
   2.552481E+07, 2.574860E+07, 2.597382E+07, 2.620047E+07, 2.642854E+07, 2.665806E+07, 
   2.688901E+07, 2.712142E+07, 2.735527E+07, 2.759057E+07, 2.782734E+07, 2.806557E+07, 
   2.830527E+07, 2.854644E+07, 2.878909E+07, 2.903321E+07, 2.927883E+07, 2.952593E+07, 
   2.977453E+07, 3.002463E+07, 3.027623E+07, 3.052934E+07, 3.078396E+07, 3.104009E+07, 
   3.129774E+07, 3.155692E+07, 3.181763E+07, 3.207987E+07, 3.234365E+07, 3.260897E+07, 
   3.287583E+07, 3.314424E+07, 3.341421E+07, 3.368573E+07, 3.395881E+07, 3.423346E+07, 
   3.450968E+07, 3.478747E+07, 3.506684E+07, 
])
# ============================ M = 2, I = 10 ============================
M = 2
I = 10
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.501700E+00, 2.037698E+01, 4.041606E+01, 6.045677E+01, 8.049824E+01, 1.005403E+02, 
   1.205830E+02, 1.406266E+02, 1.606730E+02, 1.807271E+02, 2.007995E+02, 2.209077E+02, 
   2.410771E+02, 2.613406E+02, 2.817371E+02, 3.023107E+02, 3.231082E+02, 3.441783E+02, 
   3.655703E+02, 3.873330E+02, 4.095143E+02, 4.321606E+02, 4.553170E+02, 4.790266E+02, 
   5.033312E+02, 5.282709E+02, 5.538845E+02, 5.802094E+02, 6.072820E+02, 6.351377E+02, 
   6.638110E+02, 6.933360E+02, 7.237459E+02, 7.550738E+02, 7.873522E+02, 8.206136E+02, 
   8.548902E+02, 8.902142E+02, 9.266178E+02, 9.641335E+02, 1.002793E+03, 1.042630E+03, 
   1.083677E+03, 1.125967E+03, 1.169533E+03, 1.214410E+03, 1.260630E+03, 1.308230E+03, 
   1.357243E+03, 1.407706E+03, 1.459653E+03, 1.513122E+03, 1.568150E+03, 1.624773E+03, 
   1.683030E+03, 1.742958E+03, 1.804598E+03, 1.867989E+03, 1.933171E+03, 2.000184E+03, 
   2.069070E+03, 2.139871E+03, 2.212630E+03, 2.287390E+03, 2.364195E+03, 2.443089E+03, 
   2.524117E+03, 2.607325E+03, 2.692761E+03, 2.780470E+03, 2.870501E+03, 2.962902E+03, 
   3.057723E+03, 3.155013E+03, 3.254823E+03, 3.357204E+03, 3.462209E+03, 3.569891E+03, 
   3.680301E+03, 3.793496E+03, 3.909530E+03, 4.028458E+03, 4.150338E+03, 4.275225E+03, 
   4.403179E+03, 4.534258E+03, 4.668522E+03, 4.806030E+03, 4.946845E+03, 5.091027E+03, 
   5.238639E+03, 5.389746E+03, 5.544410E+03, 5.702698E+03, 5.864675E+03, 6.030408E+03, 
   6.199964E+03, 6.373412E+03, 6.550821E+03, 6.732261E+03, 6.917802E+03, 7.107517E+03, 
   7.301478E+03, 7.499759E+03, 7.702433E+03, 7.909577E+03, 8.121265E+03, 8.337575E+03, 
   8.558585E+03, 8.784373E+03, 9.015020E+03, 9.250604E+03, 9.491209E+03, 9.736915E+03, 
   9.987807E+03, 1.024397E+04, 1.050548E+04, 1.077244E+04, 1.104492E+04, 1.132302E+04, 
   1.160682E+04, 1.189642E+04, 1.219189E+04, 1.249335E+04, 1.280087E+04, 1.311455E+04, 
   1.343449E+04, 1.376078E+04, 1.409352E+04, 1.443280E+04, 1.477873E+04, 1.513140E+04, 
   1.549091E+04, 1.585737E+04, 1.623087E+04, 1.661152E+04, 1.699943E+04, 1.739469E+04, 
   1.779742E+04, 1.820772E+04, 1.862571E+04, 1.905148E+04, 1.948515E+04, 1.992684E+04, 
   2.037665E+04, 2.083470E+04, 2.130110E+04, 2.177597E+04, 2.225942E+04, 2.275157E+04, 
   2.325254E+04, 2.376246E+04, 2.428143E+04, 2.480959E+04, 2.534705E+04, 2.589394E+04, 
   2.645038E+04, 2.701651E+04, 2.759244E+04, 2.817830E+04, 2.877424E+04, 2.938036E+04, 
   2.999682E+04, 3.062373E+04, 3.126123E+04, 3.190946E+04, 3.256856E+04, 3.323865E+04, 
   3.391989E+04, 3.461240E+04, 3.531632E+04, 3.603181E+04, 3.675900E+04, 3.749803E+04, 
   3.824905E+04, 3.901221E+04, 3.978765E+04, 4.057552E+04, 4.137597E+04, 4.218914E+04, 
   4.301520E+04, 4.385430E+04, 4.470658E+04, 4.557220E+04, 4.645132E+04, 4.734410E+04, 
   4.825069E+04, 4.917126E+04, 5.010596E+04, 5.105495E+04, 5.201841E+04, 5.299648E+04, 
   5.398935E+04, 5.499717E+04, 5.602011E+04, 5.705834E+04, 5.811204E+04, 5.918136E+04, 
   6.026650E+04, 6.136760E+04, 6.248487E+04, 6.361846E+04, 6.476855E+04, 6.593533E+04, 
   6.711897E+04, 6.831966E+04, 6.953757E+04, 7.077289E+04, 7.202580E+04, 7.329650E+04, 
   7.458515E+04, 7.589196E+04, 7.721711E+04, 7.856079E+04, 7.992319E+04, 8.130451E+04, 
   8.270494E+04, 8.412467E+04, 8.556390E+04, 8.702283E+04, 8.850166E+04, 9.000058E+04, 
   9.151980E+04, 9.305952E+04, 9.461994E+04, 9.620127E+04, 9.780372E+04, 9.942748E+04, 
   1.010728E+05, 1.027398E+05, 1.044288E+05, 1.061399E+05, 1.078734E+05, 1.096296E+05, 
   1.114085E+05, 1.132104E+05, 1.150356E+05, 1.168842E+05, 1.187565E+05, 1.206527E+05, 
   1.225730E+05, 1.245177E+05, 1.264870E+05, 1.284810E+05, 1.305001E+05, 1.325445E+05, 
   1.346143E+05, 1.367098E+05, 1.388313E+05, 1.409790E+05, 1.431531E+05, 1.453539E+05, 
   1.475815E+05, 1.498363E+05, 1.521185E+05, 1.544282E+05, 1.567659E+05, 1.591316E+05, 
   1.615257E+05, 1.639483E+05, 1.663998E+05, 1.688804E+05, 1.713903E+05, 1.739298E+05, 
   1.764992E+05, 1.790986E+05, 1.817284E+05, 1.843887E+05, 1.870800E+05, 1.898023E+05, 
   1.925560E+05, 1.953414E+05, 1.981586E+05, 2.010080E+05, 2.038898E+05, 2.068043E+05, 
   2.097518E+05, 2.127324E+05, 2.157466E+05, 2.187945E+05, 2.218764E+05, 2.249926E+05, 
   2.281434E+05, 2.313291E+05, 2.345498E+05, 2.378059E+05, 2.410978E+05, 2.444255E+05, 
   2.477895E+05, 2.511900E+05, 2.546272E+05, 2.581016E+05, 2.616132E+05, 2.651626E+05, 
   2.687498E+05, 2.723753E+05, 2.760392E+05, 2.797419E+05, 2.834837E+05, 2.872649E+05, 
   2.910857E+05, 2.949464E+05, 2.988474E+05, 3.027890E+05, 3.067713E+05, 3.107948E+05, 
   3.148597E+05, 3.189664E+05, 3.231150E+05, 3.273060E+05, 3.315396E+05, 3.358161E+05, 
   3.401358E+05, 3.444991E+05, 3.489062E+05, 3.533574E+05, 3.578531E+05, 3.623935E+05, 
   3.669790E+05, 3.716099E+05, 3.762864E+05, 3.810089E+05, 3.857777E+05, 3.905932E+05, 
   3.954555E+05, 4.003651E+05, 4.053222E+05, 4.103272E+05, 4.153804E+05, 4.204821E+05, 
   4.256326E+05, 4.308322E+05, 4.360813E+05, 4.413802E+05, 4.467291E+05, 4.521285E+05, 
   4.575786E+05, 4.630797E+05, 4.686322E+05, 4.742365E+05, 4.798927E+05, 4.856013E+05, 
   4.913626E+05, 4.971768E+05, 5.030444E+05, 5.089657E+05, 5.149409E+05, 5.209704E+05, 
   5.270546E+05, 5.331937E+05, 5.393881E+05, 5.456381E+05, 5.519441E+05, 5.583064E+05, 
   5.647252E+05, 5.712011E+05, 5.777342E+05, 5.843249E+05, 5.909735E+05, 5.976804E+05, 
   6.044459E+05, 6.112704E+05, 6.181541E+05, 6.250975E+05, 6.321007E+05, 6.391643E+05, 
   6.462885E+05, 6.534736E+05, 6.607200E+05, 6.680280E+05, 6.753979E+05, 6.828302E+05, 
   6.903251E+05, 6.978829E+05, 7.055041E+05, 7.131888E+05, 7.209376E+05, 7.287507E+05, 
   7.366284E+05, 7.445711E+05, 7.525791E+05, 7.606528E+05, 7.687924E+05, 7.769984E+05, 
   7.852711E+05, 7.936108E+05, 8.020178E+05, 8.104926E+05, 8.190353E+05, 8.276464E+05, 
   8.363262E+05, 8.450750E+05, 8.538932E+05, 8.627812E+05, 8.717391E+05, 8.807675E+05, 
   8.898665E+05, 8.990366E+05, 9.082781E+05, 9.175914E+05, 9.269767E+05, 9.364344E+05, 
   9.459648E+05, 9.555683E+05, 9.652453E+05, 9.749959E+05, 9.848207E+05, 9.947198E+05, 
   1.004694E+06, 1.014743E+06, 1.024867E+06, 1.035067E+06, 1.045343E+06, 1.055696E+06, 
   1.066125E+06, 1.076632E+06, 1.087216E+06, 1.097877E+06, 1.108617E+06, 1.119435E+06, 
   1.130332E+06, 1.141307E+06, 1.152363E+06, 1.163497E+06, 1.174712E+06, 1.186008E+06, 
   1.197384E+06, 1.208841E+06, 1.220379E+06, 1.231999E+06, 1.243701E+06, 1.255485E+06, 
   1.267352E+06, 1.279302E+06, 1.291335E+06, 1.303452E+06, 1.315653E+06, 1.327938E+06, 
   1.340307E+06, 1.352762E+06, 1.365301E+06, 1.377926E+06, 1.390637E+06, 1.403434E+06, 
   1.416318E+06, 1.429288E+06, 1.442345E+06, 1.455490E+06, 1.468723E+06, 1.482043E+06, 
   1.495452E+06, 1.508949E+06, 1.522536E+06, 1.536212E+06, 1.549977E+06, 1.563832E+06, 
   1.577777E+06, 1.591813E+06, 1.605939E+06, 1.620157E+06, 1.634466E+06, 1.648866E+06, 
   1.663359E+06, 1.677944E+06, 1.692621E+06, 1.707392E+06, 1.722255E+06, 1.737212E+06, 
   1.752263E+06, 1.767408E+06, 1.782647E+06, 1.797981E+06, 1.813409E+06, 1.828933E+06, 
   1.844552E+06, 1.860267E+06, 1.876079E+06, 1.891986E+06, 1.907990E+06, 1.924091E+06, 
   1.940290E+06, 1.956585E+06, 1.972979E+06, 1.989470E+06, 2.006060E+06, 2.022748E+06, 
   2.039536E+06, 2.056422E+06, 2.073408E+06, 2.090493E+06, 2.107678E+06, 2.124964E+06, 
   2.142350E+06, 2.159837E+06, 2.177424E+06, 2.195113E+06, 2.212904E+06, 2.230796E+06, 
   2.248791E+06, 2.266887E+06, 2.285086E+06, 
])
# ============================ M = 2, I = 11 ============================
M = 2
I = 11
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.782335E+01, 2.377062E+02, 4.713557E+02, 7.050247E+02, 9.387029E+02, 1.172388E+03, 
   1.406080E+03, 1.639784E+03, 1.873518E+03, 2.107340E+03, 2.341368E+03, 2.575802E+03, 
   2.810932E+03, 3.047135E+03, 3.284859E+03, 3.524608E+03, 3.766925E+03, 4.012371E+03, 
   4.261513E+03, 4.514917E+03, 4.773136E+03, 5.036706E+03, 5.306147E+03, 5.581959E+03, 
   5.864623E+03, 6.154603E+03, 6.452344E+03, 6.758280E+03, 7.072828E+03, 7.396398E+03, 
   7.729387E+03, 8.072184E+03, 8.425175E+03, 8.788737E+03, 9.163245E+03, 9.549073E+03, 
   9.946589E+03, 1.035616E+04, 1.077817E+04, 1.121297E+04, 1.166095E+04, 1.212247E+04, 
   1.259791E+04, 1.308766E+04, 1.359209E+04, 1.411159E+04, 1.464656E+04, 1.519739E+04, 
   1.576448E+04, 1.634823E+04, 1.694906E+04, 1.756739E+04, 1.820363E+04, 1.885822E+04, 
   1.953159E+04, 2.022417E+04, 2.093643E+04, 2.166880E+04, 2.242176E+04, 2.319576E+04, 
   2.399128E+04, 2.480881E+04, 2.564882E+04, 2.651182E+04, 2.739831E+04, 2.830879E+04, 
   2.924378E+04, 3.020382E+04, 3.118942E+04, 3.220113E+04, 3.323950E+04, 3.430508E+04, 
   3.539843E+04, 3.652014E+04, 3.767076E+04, 3.885090E+04, 4.006115E+04, 4.130211E+04, 
   4.257440E+04, 4.387863E+04, 4.521543E+04, 4.658544E+04, 4.798931E+04, 4.942769E+04, 
   5.090124E+04, 5.241063E+04, 5.395655E+04, 5.553969E+04, 5.716074E+04, 5.882041E+04, 
   6.051942E+04, 6.225849E+04, 6.403836E+04, 6.585977E+04, 6.772347E+04, 6.963024E+04, 
   7.158083E+04, 7.357603E+04, 7.561663E+04, 7.770343E+04, 7.983725E+04, 8.201890E+04, 
   8.424921E+04, 8.652902E+04, 8.885918E+04, 9.124055E+04, 9.367400E+04, 9.616040E+04, 
   9.870066E+04, 1.012957E+05, 1.039463E+05, 1.066535E+05, 1.094183E+05, 1.122415E+05, 
   1.151240E+05, 1.180670E+05, 1.210712E+05, 1.241378E+05, 1.272677E+05, 1.304619E+05, 
   1.337214E+05, 1.370473E+05, 1.404405E+05, 1.439022E+05, 1.474333E+05, 1.510351E+05, 
   1.547084E+05, 1.584545E+05, 1.622744E+05, 1.661693E+05, 1.701402E+05, 1.741883E+05, 
   1.783147E+05, 1.825207E+05, 1.868073E+05, 1.911758E+05, 1.956274E+05, 2.001631E+05, 
   2.047844E+05, 2.094923E+05, 2.142882E+05, 2.191732E+05, 2.241486E+05, 2.292158E+05, 
   2.343760E+05, 2.396304E+05, 2.449804E+05, 2.504274E+05, 2.559725E+05, 2.616173E+05, 
   2.673630E+05, 2.732110E+05, 2.791627E+05, 2.852195E+05, 2.913827E+05, 2.976538E+05, 
   3.040342E+05, 3.105254E+05, 3.171288E+05, 3.238458E+05, 3.306779E+05, 3.376267E+05, 
   3.446935E+05, 3.518800E+05, 3.591876E+05, 3.666178E+05, 3.741723E+05, 3.818525E+05, 
   3.896601E+05, 3.975965E+05, 4.056634E+05, 4.138624E+05, 4.221951E+05, 4.306632E+05, 
   4.392682E+05, 4.480118E+05, 4.568957E+05, 4.659215E+05, 4.750909E+05, 4.844056E+05, 
   4.938673E+05, 5.034778E+05, 5.132387E+05, 5.231517E+05, 5.332187E+05, 5.434414E+05, 
   5.538215E+05, 5.643609E+05, 5.750612E+05, 5.859244E+05, 5.969522E+05, 6.081465E+05, 
   6.195090E+05, 6.310416E+05, 6.427462E+05, 6.546246E+05, 6.666788E+05, 6.789104E+05, 
   6.913216E+05, 7.039141E+05, 7.166898E+05, 7.296508E+05, 7.427988E+05, 7.561359E+05, 
   7.696639E+05, 7.833849E+05, 7.973007E+05, 8.114135E+05, 8.257250E+05, 8.402373E+05, 
   8.549525E+05, 8.698724E+05, 8.849992E+05, 9.003347E+05, 9.158811E+05, 9.316404E+05, 
   9.476145E+05, 9.638056E+05, 9.802157E+05, 9.968468E+05, 1.013701E+06, 1.030780E+06, 
   1.048087E+06, 1.065623E+06, 1.083390E+06, 1.101391E+06, 1.119627E+06, 1.138101E+06, 
   1.156815E+06, 1.175770E+06, 1.194970E+06, 1.214416E+06, 1.234110E+06, 1.254054E+06, 
   1.274250E+06, 1.294702E+06, 1.315409E+06, 1.336376E+06, 1.357604E+06, 1.379095E+06, 
   1.400851E+06, 1.422874E+06, 1.445167E+06, 1.467731E+06, 1.490570E+06, 1.513684E+06, 
   1.537077E+06, 1.560749E+06, 1.584704E+06, 1.608944E+06, 1.633470E+06, 1.658285E+06, 
   1.683392E+06, 1.708791E+06, 1.734485E+06, 1.760477E+06, 1.786769E+06, 1.813362E+06, 
   1.840259E+06, 1.867462E+06, 1.894973E+06, 1.922795E+06, 1.950928E+06, 1.979377E+06, 
   2.008141E+06, 2.037225E+06, 2.066630E+06, 2.096357E+06, 2.126410E+06, 2.156789E+06, 
   2.187498E+06, 2.218539E+06, 2.249913E+06, 2.281622E+06, 2.313669E+06, 2.346056E+06, 
   2.378785E+06, 2.411857E+06, 2.445276E+06, 2.479042E+06, 2.513159E+06, 2.547627E+06, 
   2.582450E+06, 2.617628E+06, 2.653165E+06, 2.689062E+06, 2.725322E+06, 2.761945E+06, 
   2.798935E+06, 2.836292E+06, 2.874020E+06, 2.912120E+06, 2.950594E+06, 2.989444E+06, 
   3.028672E+06, 3.068279E+06, 3.108269E+06, 3.148642E+06, 3.189401E+06, 3.230547E+06, 
   3.272083E+06, 3.314010E+06, 3.356330E+06, 3.399045E+06, 3.442157E+06, 3.485668E+06, 
   3.529579E+06, 3.573892E+06, 3.618610E+06, 3.663733E+06, 3.709264E+06, 3.755205E+06, 
   3.801557E+06, 3.848321E+06, 3.895501E+06, 3.943097E+06, 3.991111E+06, 4.039545E+06, 
   4.088400E+06, 4.137679E+06, 4.187382E+06, 4.237512E+06, 4.288070E+06, 4.339058E+06, 
   4.390478E+06, 4.442330E+06, 4.494617E+06, 4.547340E+06, 4.600501E+06, 4.654101E+06, 
   4.708142E+06, 4.762626E+06, 4.817553E+06, 4.872925E+06, 4.928745E+06, 4.985012E+06, 
   5.041730E+06, 5.098899E+06, 5.156520E+06, 5.214595E+06, 5.273126E+06, 5.332114E+06, 
   5.391560E+06, 5.451466E+06, 5.511832E+06, 5.572661E+06, 5.633953E+06, 5.695710E+06, 
   5.757934E+06, 5.820625E+06, 5.883784E+06, 
])
# ============================ M = 2, I = 12 ============================
M = 2
I = 12
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.248682E+01, 6.935304E+02, 1.374885E+03, 2.056299E+03, 2.737741E+03, 3.419203E+03, 
   4.100686E+03, 4.782200E+03, 5.463802E+03, 6.145650E+03, 6.828082E+03, 7.511667E+03, 
   8.197234E+03, 8.885860E+03, 9.578835E+03, 1.027761E+04, 1.098375E+04, 1.169887E+04, 
   1.242462E+04, 1.316262E+04, 1.391448E+04, 1.468174E+04, 1.546591E+04, 1.626843E+04, 
   1.709070E+04, 1.793405E+04, 1.879977E+04, 1.968910E+04, 2.060326E+04, 2.154342E+04, 
   2.251071E+04, 2.350628E+04, 2.453122E+04, 2.558662E+04, 2.667355E+04, 2.779309E+04, 
   2.894631E+04, 3.013425E+04, 3.135799E+04, 3.261859E+04, 3.391712E+04, 3.525464E+04, 
   3.663224E+04, 3.805102E+04, 3.951206E+04, 4.101648E+04, 4.256541E+04, 4.415997E+04, 
   4.580131E+04, 4.749061E+04, 4.922903E+04, 5.101777E+04, 5.285804E+04, 5.475107E+04, 
   5.669811E+04, 5.870041E+04, 6.075926E+04, 6.287595E+04, 6.505180E+04, 6.728816E+04, 
   6.958637E+04, 7.194782E+04, 7.437390E+04, 7.686603E+04, 7.942564E+04, 8.205420E+04, 
   8.475318E+04, 8.752410E+04, 9.036846E+04, 9.328781E+04, 9.628373E+04, 9.935780E+04, 
   1.025116E+05, 1.057469E+05, 1.090652E+05, 1.124682E+05, 1.159576E+05, 1.195353E+05, 
   1.232028E+05, 1.269621E+05, 1.308148E+05, 1.347629E+05, 1.388081E+05, 1.429523E+05, 
   1.471975E+05, 1.515455E+05, 1.559983E+05, 1.605580E+05, 1.652263E+05, 1.700055E+05, 
   1.748975E+05, 1.799044E+05, 1.850284E+05, 1.902714E+05, 1.956358E+05, 2.011237E+05, 
   2.067372E+05, 2.124787E+05, 2.183504E+05, 2.243545E+05, 2.304934E+05, 2.367695E+05, 
   2.431851E+05, 2.497425E+05, 2.564443E+05, 2.632929E+05, 2.702908E+05, 2.774405E+05, 
   2.847445E+05, 2.922054E+05, 2.998258E+05, 3.076083E+05, 3.155556E+05, 3.236705E+05, 
   3.319555E+05, 3.404135E+05, 3.490472E+05, 3.578594E+05, 3.668531E+05, 3.760309E+05, 
   3.853960E+05, 3.949511E+05, 4.046992E+05, 4.146434E+05, 4.247867E+05, 4.351321E+05, 
   4.456827E+05, 4.564416E+05, 4.674120E+05, 4.785970E+05, 4.900000E+05, 5.016240E+05, 
   5.134725E+05, 5.255487E+05, 5.378560E+05, 5.503977E+05, 5.631772E+05, 5.761981E+05, 
   5.894637E+05, 6.029776E+05, 6.167433E+05, 6.307645E+05, 6.450447E+05, 6.595875E+05, 
   6.743967E+05, 6.894759E+05, 7.048290E+05, 7.204596E+05, 7.363717E+05, 7.525690E+05, 
   7.690554E+05, 7.858350E+05, 8.029115E+05, 8.202890E+05, 8.379716E+05, 8.559633E+05, 
   8.742681E+05, 8.928903E+05, 9.118340E+05, 9.311034E+05, 9.507027E+05, 9.706362E+05, 
   9.909083E+05, 1.011523E+06, 1.032485E+06, 1.053799E+06, 1.075469E+06, 1.097500E+06, 
   1.119896E+06, 1.142661E+06, 1.165801E+06, 1.189320E+06, 1.213222E+06, 1.237513E+06, 
   1.262197E+06, 1.287278E+06, 1.312763E+06, 1.338655E+06, 1.364960E+06, 1.391682E+06, 
   1.418826E+06, 1.446398E+06, 1.474402E+06, 1.502844E+06, 1.531729E+06, 1.561062E+06, 
   1.590847E+06, 1.621091E+06, 1.651799E+06, 1.682975E+06, 1.714626E+06, 1.746757E+06, 
   1.779372E+06, 1.812479E+06, 1.846081E+06, 1.880185E+06, 1.914797E+06, 1.949921E+06, 
   1.985564E+06, 2.021732E+06, 2.058429E+06, 2.095662E+06, 2.133437E+06, 2.171760E+06, 
   2.210636E+06, 2.250072E+06, 2.290073E+06, 2.330645E+06, 2.371795E+06, 2.413529E+06, 
   2.455852E+06, 2.498772E+06, 2.542294E+06, 2.586424E+06, 2.631169E+06, 2.676535E+06, 
   2.722529E+06, 2.769157E+06, 2.816425E+06, 2.864341E+06, 2.912910E+06, 2.962139E+06, 
   3.012035E+06, 3.062605E+06, 3.113855E+06, 3.165791E+06, 3.218422E+06, 3.271753E+06, 
   3.325792E+06, 3.380545E+06, 3.436020E+06, 3.492223E+06, 3.549161E+06, 3.606842E+06, 
   3.665273E+06, 3.724461E+06, 3.784412E+06, 3.845136E+06, 3.906637E+06, 3.968925E+06, 
   4.032006E+06, 4.095888E+06, 4.160578E+06, 4.226084E+06, 4.292413E+06, 4.359573E+06, 
   4.427572E+06, 4.496416E+06, 4.566115E+06, 4.636675E+06, 4.708105E+06, 4.780412E+06, 
   4.853604E+06, 4.927689E+06, 5.002674E+06, 5.078569E+06, 5.155381E+06, 5.233118E+06, 
   5.311788E+06, 5.391400E+06, 5.471961E+06, 5.553480E+06, 5.635964E+06, 5.719423E+06, 
   5.803865E+06, 5.889298E+06, 5.975731E+06, 6.063171E+06, 6.151628E+06, 6.241111E+06, 
   6.331626E+06, 6.423184E+06, 6.515793E+06, 6.609462E+06, 6.704199E+06, 6.800014E+06, 
   6.896914E+06, 6.994910E+06, 7.094009E+06, 7.194221E+06, 7.295554E+06, 7.398019E+06, 
   7.501623E+06, 7.606376E+06, 7.712287E+06, 7.819365E+06, 7.927620E+06, 8.037060E+06, 
   8.147695E+06, 8.259535E+06, 8.372588E+06, 8.486864E+06, 8.602372E+06, 8.719122E+06, 
   8.837123E+06, 8.956385E+06, 9.076918E+06, 9.198731E+06, 9.321833E+06, 9.446234E+06, 
   9.571945E+06, 9.698974E+06, 9.827332E+06, 9.957028E+06, 1.008807E+07, 1.022047E+07, 
   1.035424E+07, 1.048939E+07, 1.062593E+07, 1.076386E+07, 1.090320E+07, 1.104396E+07, 
   1.118615E+07, 1.132977E+07, 1.147484E+07, 1.162137E+07, 1.176937E+07, 1.191885E+07, 
   1.206982E+07, 1.222229E+07, 1.237626E+07, 1.253176E+07, 1.268879E+07, 1.284735E+07, 
   1.300747E+07, 1.316915E+07, 1.333241E+07, 1.349724E+07, 1.366367E+07, 1.383170E+07, 
   1.400135E+07, 1.417262E+07, 1.434553E+07, 1.452008E+07, 1.469629E+07, 1.487417E+07, 
   1.505372E+07, 1.523497E+07, 1.541791E+07, 1.560256E+07, 1.578894E+07, 1.597705E+07, 
   1.616690E+07, 1.635851E+07, 1.655188E+07, 1.674702E+07, 1.694395E+07, 1.714268E+07, 
   1.734322E+07, 1.754558E+07, 1.774977E+07, 1.795580E+07, 1.816368E+07, 1.837343E+07, 
   1.858505E+07, 1.879855E+07, 1.901396E+07, 1.923127E+07, 1.945050E+07, 1.967166E+07, 
   1.989476E+07, 2.011981E+07, 2.034683E+07, 2.057582E+07, 2.080679E+07, 2.103977E+07, 
   2.127475E+07, 2.151175E+07, 2.175079E+07, 2.199186E+07, 2.223499E+07, 2.248018E+07, 
   2.272745E+07, 2.297680E+07, 2.322825E+07, 2.348182E+07, 2.373750E+07, 2.399531E+07, 
   2.425527E+07, 2.451738E+07, 2.478166E+07, 2.504812E+07, 2.531676E+07, 2.558760E+07, 
   2.586066E+07, 2.613593E+07, 2.641344E+07, 2.669320E+07, 2.697521E+07, 2.725949E+07, 
   2.754605E+07, 2.783489E+07, 2.812604E+07, 2.841950E+07, 2.871529E+07, 2.901341E+07, 
   2.931387E+07, 2.961670E+07, 2.992189E+07, 3.022947E+07, 3.053943E+07, 3.085180E+07, 
   3.116658E+07, 3.148379E+07, 3.180343E+07, 3.212552E+07, 3.245007E+07, 3.277709E+07, 
   3.310659E+07, 3.343858E+07, 3.377308E+07, 3.411009E+07, 3.444963E+07, 3.479171E+07, 
   3.513633E+07, 3.548351E+07, 3.583326E+07, 3.618560E+07, 3.654053E+07, 3.689806E+07, 
   3.725820E+07, 3.762097E+07, 3.798638E+07, 3.835443E+07, 3.872515E+07, 3.909853E+07, 
   3.947459E+07, 3.985335E+07, 4.023480E+07, 4.061897E+07, 4.100587E+07, 4.139550E+07, 
   4.178787E+07, 4.218300E+07, 4.258090E+07, 4.298158E+07, 4.338505E+07, 4.379131E+07, 
   4.420039E+07, 4.461229E+07, 4.502701E+07, 4.544458E+07, 4.586501E+07, 4.628829E+07, 
   4.671445E+07, 4.714350E+07, 4.757544E+07, 4.801029E+07, 4.844805E+07, 4.888874E+07, 
   4.933236E+07, 4.977893E+07, 5.022846E+07, 5.068096E+07, 5.113643E+07, 5.159490E+07, 
   5.205636E+07, 5.252083E+07, 5.298832E+07, 5.345884E+07, 5.393239E+07, 5.440900E+07, 
   5.488867E+07, 5.537140E+07, 5.585722E+07, 5.634612E+07, 5.683812E+07, 5.733324E+07, 
   5.783147E+07, 5.833283E+07, 5.883732E+07, 5.934497E+07, 5.985577E+07, 6.036974E+07, 
   6.088689E+07, 6.140723E+07, 6.193076E+07, 6.245750E+07, 6.298745E+07, 6.352063E+07, 
   6.405704E+07, 6.459670E+07, 6.513961E+07, 6.568578E+07, 6.623523E+07, 6.678795E+07, 
   6.734397E+07, 6.790328E+07, 6.846590E+07, 6.903184E+07, 6.960111E+07, 7.017371E+07, 
   7.074966E+07, 7.132896E+07, 7.191162E+07, 7.249765E+07, 7.308707E+07, 7.367987E+07, 
   7.427607E+07, 7.487568E+07, 7.547870E+07, 
])
# ============================ M = 2, I = 13 ============================
#M = 2
#I = 13
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.172250E+00, 9.073560E+00, 1.797782E+01, 2.688294E+01, 3.578844E+01, 4.469421E+01, 
#   5.360026E+01, 6.250677E+01, 7.141461E+01, 8.032621E+01, 8.924655E+01, 9.818393E+01, 
#   1.071502E+02, 1.161606E+02, 1.252330E+02, 1.343876E+02, 1.436455E+02, 1.530287E+02, 
#   1.625593E+02, 1.722590E+02, 1.821490E+02, 1.922497E+02, 2.025809E+02, 2.131615E+02, 
#   2.240094E+02, 2.351421E+02, 2.465762E+02, 2.583279E+02, 2.704128E+02, 2.828461E+02, 
#   2.956425E+02, 3.088166E+02, 3.223827E+02, 3.363548E+02, 3.507470E+02, 3.655731E+02, 
#   3.808470E+02, 3.965825E+02, 4.127935E+02, 4.294938E+02, 4.466974E+02, 4.644185E+02, 
#   4.826712E+02, 5.014698E+02, 5.208288E+02, 5.407628E+02, 5.612866E+02, 5.824152E+02, 
#   6.041637E+02, 6.265476E+02, 6.495824E+02, 6.732840E+02, 6.976684E+02, 7.227518E+02, 
#   7.485507E+02, 7.750819E+02, 8.023624E+02, 8.304094E+02, 8.592405E+02, 8.888733E+02, 
#   9.193259E+02, 9.506167E+02, 9.827640E+02, 1.015787E+03, 1.049704E+03, 1.084536E+03, 
#   1.120301E+03, 1.157020E+03, 1.194713E+03, 1.233400E+03, 1.273103E+03, 1.313842E+03, 
#   1.355638E+03, 1.398515E+03, 1.442493E+03, 1.487596E+03, 1.533844E+03, 1.581263E+03, 
#   1.629875E+03, 1.679703E+03, 1.730772E+03, 1.783106E+03, 1.836729E+03, 1.891666E+03, 
#   1.947943E+03, 2.005585E+03, 2.064618E+03, 2.125069E+03, 2.186963E+03, 2.250328E+03, 
#   2.315191E+03, 2.381580E+03, 2.449522E+03, 2.519047E+03, 2.590182E+03, 2.662957E+03, 
#   2.737401E+03, 2.813544E+03, 2.891416E+03, 2.971047E+03, 3.052469E+03, 3.135712E+03, 
#   3.220808E+03, 3.307789E+03, 3.396688E+03, 3.487537E+03, 3.580369E+03, 3.675218E+03, 
#   3.772118E+03, 3.871102E+03, 3.972206E+03, 4.075465E+03, 4.180913E+03, 4.288588E+03, 
#   4.398524E+03, 4.510760E+03, 4.625331E+03, 4.742275E+03, 4.861631E+03, 4.983436E+03, 
#   5.107729E+03, 5.234549E+03, 5.363936E+03, 5.495929E+03, 5.630570E+03, 5.767898E+03, 
#   5.907955E+03, 6.050783E+03, 6.196423E+03, 6.344919E+03, 6.496312E+03, 6.650647E+03, 
#   6.807968E+03, 6.968318E+03, 7.131742E+03, 7.298285E+03, 7.467993E+03, 7.640912E+03, 
#   7.817088E+03, 7.996569E+03, 8.179401E+03, 8.365632E+03, 8.555312E+03, 8.748488E+03, 
#   8.945209E+03, 9.145526E+03, 9.349488E+03, 9.557146E+03, 9.768552E+03, 9.983756E+03, 
#   1.020281E+04, 1.042577E+04, 1.065268E+04, 1.088361E+04, 1.111859E+04, 1.135770E+04, 
#   1.160098E+04, 1.184848E+04, 1.210027E+04, 1.235640E+04, 1.261693E+04, 1.288191E+04, 
#   1.315140E+04, 1.342546E+04, 1.370416E+04, 1.398754E+04, 1.427567E+04, 1.456860E+04, 
#   1.486641E+04, 1.516915E+04, 1.547688E+04, 1.578967E+04, 1.610757E+04, 1.643065E+04, 
#   1.675898E+04, 1.709262E+04, 1.743163E+04, 1.777608E+04, 1.812603E+04, 1.848156E+04, 
#   1.884273E+04, 1.920960E+04, 1.958224E+04, 1.996073E+04, 2.034513E+04, 2.073551E+04, 
#   2.113194E+04, 2.153449E+04, 2.194324E+04, 2.235825E+04, 2.277960E+04, 2.320735E+04, 
#   2.364159E+04, 2.408238E+04, 2.452981E+04, 2.498394E+04, 2.544485E+04, 2.591262E+04, 
#   2.638733E+04, 2.686904E+04, 2.735784E+04, 2.785381E+04, 2.835702E+04, 2.886756E+04, 
#   2.938550E+04, 2.991093E+04, 3.044392E+04, 3.098456E+04, 3.153292E+04, 3.208910E+04, 
#   3.265317E+04, 3.322522E+04, 3.380534E+04, 3.439360E+04, 3.499009E+04, 3.559490E+04, 
#   3.620811E+04, 3.682982E+04, 3.746011E+04, 3.809906E+04, 3.874677E+04, 3.940333E+04, 
#   4.006882E+04, 4.074334E+04, 4.142697E+04, 4.211981E+04, 4.282196E+04, 4.353349E+04, 
#   4.425451E+04, 4.498512E+04, 4.572539E+04, 4.647544E+04, 4.723535E+04, 4.800522E+04, 
#   4.878515E+04, 4.957523E+04, 5.037557E+04, 5.118626E+04, 5.200740E+04, 5.283909E+04, 
#   5.368142E+04, 5.453451E+04, 5.539845E+04, 5.627334E+04, 5.715929E+04, 5.805640E+04, 
#   5.896477E+04, 5.988450E+04, 6.081571E+04, 6.175849E+04, 6.271296E+04, 6.367921E+04, 
#   6.465735E+04, 6.564750E+04, 6.664976E+04, 6.766424E+04, 6.869105E+04, 6.973030E+04, 
#   7.078209E+04, 7.184654E+04, 7.292377E+04, 7.401387E+04, 7.511697E+04, 7.623318E+04, 
#   7.736262E+04, 7.850538E+04, 7.966160E+04, 8.083139E+04, 8.201486E+04, 8.321212E+04, 
#   8.442330E+04, 8.564852E+04, 8.688789E+04, 8.814152E+04, 8.940955E+04, 9.069208E+04, 
#   9.198924E+04, 9.330115E+04, 9.462794E+04, 9.596971E+04, 9.732660E+04, 9.869873E+04, 
#   1.000862E+05, 1.014892E+05, 1.029078E+05, 1.043421E+05, 1.057923E+05, 1.072584E+05, 
#   1.087407E+05, 1.102392E+05, 1.117541E+05, 1.132855E+05, 1.148335E+05, 1.163983E+05, 
#   1.179799E+05, 1.195786E+05, 1.211944E+05, 1.228275E+05, 1.244779E+05, 1.261460E+05, 
#   1.278316E+05, 1.295351E+05, 1.312566E+05, 1.329961E+05, 1.347538E+05, 1.365298E+05, 
#   1.383243E+05, 1.401374E+05, 1.419693E+05, 1.438201E+05, 1.456898E+05, 1.475788E+05, 
#   1.494870E+05, 1.514147E+05, 1.533619E+05, 1.553289E+05, 1.573156E+05, 1.593224E+05, 
#   1.613494E+05, 1.633966E+05, 1.654642E+05, 1.675523E+05, 1.696612E+05, 1.717909E+05, 
#   1.739416E+05, 1.761134E+05, 1.783064E+05, 1.805209E+05, 1.827570E+05, 1.850147E+05, 
#   1.872943E+05, 1.895959E+05, 1.919196E+05, 1.942656E+05, 1.966340E+05, 1.990250E+05, 
#   2.014388E+05, 2.038754E+05, 2.063350E+05, 2.088178E+05, 2.113239E+05, 2.138534E+05, 
#   2.164066E+05, 2.189835E+05, 2.215843E+05, 2.242092E+05, 2.268583E+05, 2.295318E+05, 
#   2.322298E+05, 2.349524E+05, 2.376998E+05, 2.404722E+05, 2.432697E+05, 2.460925E+05, 
#   2.489407E+05, 2.518144E+05, 2.547139E+05, 2.576392E+05, 2.605906E+05, 2.635681E+05, 
#   2.665720E+05, 2.696023E+05, 2.726593E+05, 2.757431E+05, 2.788538E+05, 2.819916E+05, 
#   2.851567E+05, 2.883491E+05, 2.915692E+05, 2.948169E+05, 2.980925E+05, 3.013961E+05, 
#   3.047279E+05, 3.080880E+05, 3.114766E+05, 3.148939E+05, 3.183399E+05, 3.218149E+05, 
#   3.253190E+05, 3.288523E+05, 3.324151E+05, 3.360074E+05, 3.396295E+05, 3.432814E+05, 
#   3.469634E+05, 3.506756E+05, 3.544181E+05, 3.581911E+05, 3.619948E+05, 3.658292E+05, 
#   3.696947E+05, 3.735913E+05, 3.775191E+05, 3.814784E+05, 3.854693E+05, 3.894919E+05, 
#   3.935464E+05, 3.976329E+05, 4.017517E+05, 4.059029E+05, 4.100865E+05, 4.143028E+05, 
#   4.185520E+05, 4.228342E+05, 4.271494E+05, 4.314980E+05, 4.358800E+05, 4.402956E+05, 
#   4.447450E+05, 4.492283E+05, 4.537456E+05, 4.582972E+05, 4.628831E+05, 4.675036E+05, 
#   4.721587E+05, 4.768487E+05, 4.815736E+05, 4.863337E+05, 4.911291E+05, 4.959599E+05, 
#   5.008263E+05, 5.057284E+05, 5.106664E+05, 5.156405E+05, 5.206508E+05, 5.256974E+05, 
#   5.307806E+05, 5.359003E+05, 5.410569E+05, 5.462505E+05, 5.514811E+05, 5.567490E+05, 
#   5.620543E+05, 5.673971E+05, 5.727776E+05, 5.781960E+05, 5.836524E+05, 5.891469E+05, 
#   5.946797E+05, 6.002510E+05, 6.058608E+05, 6.115093E+05, 6.171968E+05, 6.229232E+05, 
#   6.286889E+05, 6.344938E+05, 6.403382E+05, 6.462222E+05, 6.521460E+05, 6.581096E+05, 
#   6.641133E+05, 6.701572E+05, 6.762413E+05, 6.823660E+05, 6.885312E+05, 6.947372E+05, 
#   7.009841E+05, 7.072721E+05, 7.136011E+05, 7.199715E+05, 7.263834E+05, 7.328368E+05, 
#   7.393320E+05, 7.458690E+05, 7.524481E+05, 7.590692E+05, 7.657327E+05, 7.724386E+05, 
#   7.791870E+05, 7.859781E+05, 7.928120E+05, 7.996889E+05, 8.066089E+05, 8.135721E+05, 
#   8.205787E+05, 8.276288E+05, 8.347224E+05, 8.418599E+05, 8.490412E+05, 8.562666E+05, 
#   8.635361E+05, 8.708498E+05, 8.782080E+05, 8.856107E+05, 8.930581E+05, 9.005503E+05, 
#   9.080874E+05, 9.156695E+05, 9.232969E+05, 9.309695E+05, 9.386875E+05, 9.464511E+05, 
#   9.542604E+05, 9.621155E+05, 9.700165E+05, 9.779635E+05, 9.859567E+05, 9.939962E+05, 
#   1.002082E+06, 1.010214E+06, 1.018394E+06, 
#])
# ============================ M = 3, I = 1 ============================
M = 3
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.816030E-01, 2.089358E+01, 5.855067E+01, 1.072484E+02, 1.648941E+02, 2.302763E+02, 
   3.025774E+02, 3.811977E+02, 4.656732E+02, 5.556340E+02, 6.507802E+02, 7.508695E+02, 
   8.557101E+02, 9.651560E+02, 1.079104E+03, 1.197490E+03, 1.320290E+03, 1.447512E+03, 
   1.579196E+03, 1.715412E+03, 1.856257E+03, 2.001851E+03, 2.152334E+03, 2.307866E+03, 
   2.468623E+03, 2.634796E+03, 2.806589E+03, 2.984217E+03, 3.167905E+03, 3.357887E+03, 
   3.554405E+03, 3.757709E+03, 3.968052E+03, 4.185698E+03, 4.410912E+03, 4.643966E+03, 
   4.885137E+03, 5.134702E+03, 5.392953E+03, 5.660173E+03, 5.936659E+03, 6.222707E+03, 
   6.518619E+03, 6.824701E+03, 7.141263E+03, 7.468618E+03, 7.807082E+03, 8.156980E+03, 
   8.518638E+03, 8.892386E+03, 9.278557E+03, 9.677494E+03, 1.008954E+04, 1.051504E+04, 
   1.095435E+04, 1.140783E+04, 1.187583E+04, 1.235874E+04, 1.285691E+04, 1.337073E+04, 
   1.390058E+04, 1.444684E+04, 1.500991E+04, 1.559018E+04, 1.618806E+04, 1.680395E+04, 
   1.743827E+04, 1.809144E+04, 1.876388E+04, 1.945601E+04, 2.016828E+04, 2.090112E+04, 
   2.165499E+04, 2.243032E+04, 2.322758E+04, 2.404724E+04, 2.488975E+04, 2.575560E+04, 
   2.664527E+04, 2.755924E+04, 2.849799E+04, 2.946205E+04, 3.045190E+04, 3.146805E+04, 
   3.251103E+04, 3.358135E+04, 3.467954E+04, 3.580614E+04, 3.696169E+04, 3.814674E+04, 
   3.936183E+04, 4.060753E+04, 4.188440E+04, 4.319302E+04, 4.453396E+04, 4.590782E+04, 
   4.731517E+04, 4.875663E+04, 5.023278E+04, 5.174425E+04, 5.329165E+04, 
])
# ============================ M = 3, I = 2 ============================
M = 3
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.683569E+00, 4.539552E+01, 1.272772E+02, 2.331750E+02, 3.585355E+02, 5.007233E+02, 
   6.579601E+02, 8.289410E+02, 1.012658E+03, 1.208308E+03, 1.415243E+03, 1.632943E+03, 
   1.861003E+03, 2.099119E+03, 2.347084E+03, 2.604780E+03, 2.872173E+03, 3.149307E+03, 
   3.436292E+03, 3.733305E+03, 4.040579E+03, 4.358398E+03, 4.687091E+03, 5.027027E+03, 
   5.378612E+03, 5.742285E+03, 6.118504E+03, 6.507762E+03, 6.910567E+03, 7.327449E+03, 
   7.758952E+03, 8.205636E+03, 8.668079E+03, 9.146869E+03, 9.642602E+03, 1.015589E+04, 
   1.068735E+04, 1.123762E+04, 1.180733E+04, 1.239714E+04, 1.300771E+04, 1.363970E+04, 
   1.429379E+04, 1.497067E+04, 1.567103E+04, 1.639558E+04, 1.714504E+04, 1.792012E+04, 
   1.872157E+04, 1.955013E+04, 2.040655E+04, 2.129160E+04, 2.220604E+04, 2.315067E+04, 
   2.412629E+04, 2.513369E+04, 2.617369E+04, 2.724713E+04, 2.835484E+04, 2.949767E+04, 
   3.067649E+04, 3.189216E+04, 3.314558E+04, 3.443763E+04, 3.576924E+04, 3.714132E+04, 
   3.855479E+04, 4.001062E+04, 4.150976E+04, 4.305316E+04, 4.464183E+04, 4.627675E+04, 
   4.795894E+04, 4.968941E+04, 5.146918E+04, 5.329933E+04, 5.518090E+04, 5.711498E+04, 
   5.910263E+04, 6.114498E+04, 6.324310E+04, 6.539817E+04, 6.761130E+04, 6.988364E+04, 
   7.221639E+04, 7.461069E+04, 7.706777E+04, 7.958882E+04, 8.217510E+04, 8.482778E+04, 
   8.754817E+04, 9.033753E+04, 9.319712E+04, 9.612825E+04, 9.913224E+04, 1.022104E+05, 
   1.053641E+05, 1.085946E+05, 1.119034E+05, 1.152918E+05, 1.187612E+05, 
])
# ============================ M = 3, I = 3 ============================
M = 3
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.301881E-01, 2.219188E+01, 6.219313E+01, 1.139229E+02, 1.751581E+02, 2.446117E+02, 
   3.214152E+02, 4.049315E+02, 4.946679E+02, 5.902319E+02, 6.913059E+02, 7.976344E+02, 
   9.090165E+02, 1.025302E+03, 1.146386E+03, 1.272209E+03, 1.402753E+03, 1.538037E+03, 
   1.678117E+03, 1.823079E+03, 1.973040E+03, 2.128140E+03, 2.288545E+03, 2.454439E+03, 
   2.626025E+03, 2.803521E+03, 2.987161E+03, 3.177186E+03, 3.373852E+03, 3.577422E+03, 
   3.788169E+03, 4.006372E+03, 4.232318E+03, 4.466299E+03, 4.708611E+03, 4.959560E+03, 
   5.219453E+03, 5.488603E+03, 5.767329E+03, 6.055951E+03, 6.354800E+03, 6.664202E+03, 
   6.984496E+03, 7.316023E+03, 7.659127E+03, 8.014158E+03, 8.381470E+03, 8.761421E+03, 
   9.154375E+03, 9.560701E+03, 9.980776E+03, 1.041497E+04, 1.086368E+04, 1.132728E+04, 
   1.180618E+04, 1.230077E+04, 1.281145E+04, 1.333864E+04, 1.388275E+04, 1.444420E+04, 
   1.502342E+04, 1.562085E+04, 1.623692E+04, 1.687208E+04, 1.752678E+04, 1.820147E+04, 
   1.889662E+04, 1.961270E+04, 2.035017E+04, 2.110953E+04, 2.189126E+04, 2.269585E+04, 
   2.352380E+04, 2.437562E+04, 2.525183E+04, 2.615294E+04, 2.707947E+04, 2.803197E+04, 
   2.901096E+04, 3.001700E+04, 3.105064E+04, 3.211244E+04, 3.320296E+04, 3.432278E+04, 
   3.547248E+04, 3.665263E+04, 3.786386E+04, 3.910673E+04, 4.038187E+04, 4.168989E+04, 
   4.303142E+04, 4.440708E+04, 4.581751E+04, 4.726335E+04, 4.874526E+04, 5.026387E+04, 
   5.181988E+04, 5.341396E+04, 5.504676E+04, 5.671901E+04, 5.843137E+04, 
])
# ============================ M = 3, I = 4 ============================
M = 3
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   9.840201E+00, 2.642019E+02, 7.405728E+02, 1.356639E+03, 2.085916E+03, 2.913078E+03, 
   3.827779E+03, 4.822429E+03, 5.891163E+03, 7.029303E+03, 8.233069E+03, 9.499418E+03, 
   1.082595E+04, 1.221087E+04, 1.365291E+04, 1.515134E+04, 1.670589E+04, 1.831676E+04, 
   1.998452E+04, 2.171011E+04, 2.349486E+04, 2.534032E+04, 2.724836E+04, 2.922106E+04, 
   3.126072E+04, 3.336981E+04, 3.555100E+04, 3.780703E+04, 4.014084E+04, 4.255544E+04, 
   4.505396E+04, 4.763958E+04, 5.031563E+04, 5.308546E+04, 5.595247E+04, 5.892020E+04, 
   6.199219E+04, 6.517203E+04, 6.846342E+04, 7.187008E+04, 7.539576E+04, 7.904431E+04, 
   8.281961E+04, 8.672558E+04, 9.076623E+04, 9.494557E+04, 9.926772E+04, 1.037368E+05, 
   1.083570E+05, 1.131326E+05, 1.180680E+05, 1.231674E+05, 1.284354E+05, 1.338764E+05, 
   1.394949E+05, 1.452956E+05, 1.512831E+05, 1.574623E+05, 1.638378E+05, 1.704145E+05, 
   1.771974E+05, 1.841915E+05, 1.914018E+05, 1.988333E+05, 2.064915E+05, 2.143814E+05, 
   2.225085E+05, 2.308780E+05, 2.394956E+05, 2.483667E+05, 2.574970E+05, 2.668921E+05, 
   2.765578E+05, 2.864998E+05, 2.967242E+05, 3.072369E+05, 3.180439E+05, 3.291515E+05, 
   3.405656E+05, 3.522928E+05, 3.643393E+05, 3.767116E+05, 3.894161E+05, 4.024593E+05, 
   4.158482E+05, 4.295894E+05, 4.436896E+05, 4.581558E+05, 4.729950E+05, 4.882142E+05, 
   5.038208E+05, 5.198216E+05, 5.362242E+05, 5.530360E+05, 5.702644E+05, 5.879170E+05, 
   6.060012E+05, 6.245253E+05, 6.434964E+05, 6.629231E+05, 6.828128E+05, 
])
# ============================ M = 3, I = 5 ============================
M = 3
I = 5
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.883311E+00, 1.305120E+02, 3.657480E+02, 6.699543E+02, 1.030057E+03, 1.438490E+03, 
   1.890143E+03, 2.381272E+03, 2.908979E+03, 3.470953E+03, 4.065325E+03, 4.690586E+03, 
   5.345546E+03, 6.029306E+03, 6.741237E+03, 7.480967E+03, 8.248361E+03, 9.043504E+03, 
   9.866689E+03, 1.071838E+04, 1.159924E+04, 1.251003E+04, 1.345171E+04, 1.442529E+04, 
   1.543194E+04, 1.647289E+04, 1.754946E+04, 1.866305E+04, 1.981511E+04, 2.100714E+04, 
   2.224070E+04, 2.351740E+04, 2.483886E+04, 2.620679E+04, 2.762287E+04, 2.908886E+04, 
   3.060652E+04, 3.217765E+04, 3.380409E+04, 3.548768E+04, 3.723030E+04, 3.903386E+04, 
   4.090028E+04, 4.283153E+04, 4.482960E+04, 4.689647E+04, 4.903420E+04, 5.124486E+04, 
   5.353051E+04, 5.589329E+04, 5.833534E+04, 6.085884E+04, 6.346599E+04, 6.615902E+04, 
   6.894021E+04, 7.181183E+04, 7.477620E+04, 7.783571E+04, 8.099271E+04, 8.424964E+04, 
   8.760896E+04, 9.107312E+04, 9.464465E+04, 9.832612E+04, 1.021201E+05, 1.060292E+05, 
   1.100561E+05, 1.142034E+05, 1.184740E+05, 1.228705E+05, 1.273957E+05, 1.320525E+05, 
   1.368437E+05, 1.417723E+05, 1.468411E+05, 1.520532E+05, 1.574116E+05, 1.629192E+05, 
   1.685793E+05, 1.743949E+05, 1.803692E+05, 1.865054E+05, 1.928067E+05, 1.992764E+05, 
   2.059179E+05, 2.127344E+05, 2.197295E+05, 2.269065E+05, 2.342689E+05, 2.418202E+05, 
   2.495641E+05, 2.575040E+05, 2.656436E+05, 2.739867E+05, 2.825370E+05, 2.912980E+05, 
   3.002738E+05, 3.094683E+05, 3.188851E+05, 3.285283E+05, 3.384019E+05, 
])
# ============================ M = 3, I = 6 ============================
#M = 3
#I = 6
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.756011E+00, 4.734568E+01, 1.327529E+02, 2.432114E+02, 3.739714E+02, 5.222839E+02, 
#   6.862938E+02, 8.646401E+02, 1.056272E+03, 1.260352E+03, 1.476207E+03, 1.703301E+03, 
#   1.941215E+03, 2.189644E+03, 2.448384E+03, 2.717332E+03, 2.996469E+03, 3.285865E+03, 
#   3.585662E+03, 3.896072E+03, 4.217370E+03, 4.549883E+03, 4.893989E+03, 5.250108E+03, 
#   5.618697E+03, 6.000244E+03, 6.395271E+03, 6.804318E+03, 7.227954E+03, 7.666765E+03, 
#   8.121350E+03, 8.592331E+03, 9.080340E+03, 9.586021E+03, 1.011003E+04, 1.065305E+04, 
#   1.121575E+04, 1.179882E+04, 1.240297E+04, 1.302890E+04, 1.367735E+04, 1.434903E+04, 
#   1.504470E+04, 1.576510E+04, 1.651100E+04, 1.728317E+04, 1.808239E+04, 1.890946E+04, 
#   1.976518E+04, 2.065037E+04, 2.156585E+04, 2.251248E+04, 2.349108E+04, 2.450253E+04, 
#   2.554770E+04, 2.662747E+04, 2.774274E+04, 2.889443E+04, 3.008345E+04, 3.131075E+04, 
#   3.257725E+04, 3.388394E+04, 3.523177E+04, 3.662175E+04, 3.805487E+04, 3.953214E+04, 
#   4.105459E+04, 4.262327E+04, 4.423923E+04, 4.590353E+04, 4.761726E+04, 4.938152E+04, 
#   5.119742E+04, 5.306609E+04, 5.498866E+04, 5.696630E+04, 5.900016E+04, 6.109145E+04, 
#   6.324133E+04, 6.545106E+04, 6.772185E+04, 7.005494E+04, 7.245158E+04, 7.491306E+04, 
#   7.744069E+04, 8.003574E+04, 8.269957E+04, 8.543347E+04, 8.823884E+04, 9.111701E+04, 
#   9.406939E+04, 9.709738E+04, 1.002024E+05, 1.033859E+05, 1.066492E+05, 1.099940E+05, 
#   1.134216E+05, 1.169335E+05, 1.205313E+05, 1.242165E+05, 1.279907E+05, 
#])
# ============================ M = 3, I = 7 ============================
#M = 3
#I = 7
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   8.920031E-01, 2.424564E+01, 6.801072E+01, 1.246169E+02, 1.916289E+02, 2.676376E+02, 
#   3.516918E+02, 4.430943E+02, 5.413061E+02, 6.458996E+02, 7.565312E+02, 8.729275E+02, 
#   9.948782E+02, 1.122230E+03, 1.254882E+03, 1.392784E+03, 1.535929E+03, 1.684354E+03, 
#   1.838131E+03, 1.997367E+03, 2.162201E+03, 2.332797E+03, 2.509343E+03, 2.692050E+03, 
#   2.881145E+03, 3.076875E+03, 3.279498E+03, 3.489287E+03, 3.706525E+03, 3.931506E+03, 
#   4.164532E+03, 4.405913E+03, 4.655968E+03, 4.915024E+03, 5.183409E+03, 5.461461E+03, 
#   5.749523E+03, 6.047944E+03, 6.357078E+03, 6.677283E+03, 7.008925E+03, 7.352372E+03, 
#   7.707999E+03, 8.076186E+03, 8.457316E+03, 8.851784E+03, 9.259981E+03, 9.682310E+03, 
#   1.011918E+04, 1.057100E+04, 1.103818E+04, 1.152116E+04, 1.202036E+04, 1.253622E+04, 
#   1.306917E+04, 1.361967E+04, 1.418816E+04, 1.477512E+04, 1.538099E+04, 1.600626E+04, 
#   1.665141E+04, 1.731692E+04, 1.800327E+04, 1.871098E+04, 1.944054E+04, 2.019247E+04, 
#   2.096727E+04, 2.176549E+04, 2.258766E+04, 2.343431E+04, 2.430598E+04, 2.520323E+04, 
#   2.612663E+04, 2.707674E+04, 2.805413E+04, 2.905940E+04, 3.009312E+04, 3.115590E+04, 
#   3.224834E+04, 3.337106E+04, 3.452466E+04, 3.570979E+04, 3.692707E+04, 3.817715E+04, 
#   3.946069E+04, 4.077833E+04, 4.213075E+04, 4.351861E+04, 4.494261E+04, 4.640342E+04, 
#   4.790177E+04, 4.943833E+04, 5.101383E+04, 5.262900E+04, 5.428456E+04, 5.598125E+04, 
#   5.771981E+04, 5.950102E+04, 6.132561E+04, 6.319439E+04, 6.510810E+04, 
#])
# ============================ M = 3, I = 8 ============================
#M = 3
#I = 8
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.032224E+01, 2.782087E+02, 7.800385E+02, 1.429057E+03, 2.197359E+03, 3.068793E+03, 
#   4.032456E+03, 5.080355E+03, 6.206310E+03, 7.405404E+03, 8.673675E+03, 1.000795E+04, 
#   1.140576E+04, 1.286528E+04, 1.438527E+04, 1.596507E+04, 1.760453E+04, 1.930398E+04, 
#   2.106417E+04, 2.288629E+04, 2.477185E+04, 2.672268E+04, 2.874091E+04, 3.082890E+04, 
#   3.298924E+04, 3.522472E+04, 3.753828E+04, 3.993302E+04, 4.241217E+04, 4.497907E+04, 
#   4.763717E+04, 5.039000E+04, 5.324121E+04, 5.619448E+04, 5.925359E+04, 6.242237E+04, 
#   6.570476E+04, 6.910467E+04, 7.262618E+04, 7.627334E+04, 8.005032E+04, 8.396132E+04, 
#   8.801058E+04, 9.220244E+04, 9.654126E+04, 1.010315E+05, 1.056776E+05, 1.104842E+05, 
#   1.154558E+05, 1.205972E+05, 1.259131E+05, 1.314083E+05, 1.370877E+05, 1.429562E+05, 
#   1.490188E+05, 1.552807E+05, 1.617469E+05, 1.684226E+05, 1.753132E+05, 1.824241E+05, 
#   1.897605E+05, 1.973282E+05, 2.051324E+05, 2.131792E+05, 2.214740E+05, 2.300227E+05, 
#   2.388312E+05, 2.479055E+05, 2.572516E+05, 2.668755E+05, 2.767837E+05, 2.869822E+05, 
#   2.974773E+05, 3.082758E+05, 3.193840E+05, 3.308084E+05, 3.425559E+05, 3.546332E+05, 
#   3.670472E+05, 3.798047E+05, 3.929128E+05, 4.063788E+05, 4.202095E+05, 4.344126E+05, 
#   4.489953E+05, 4.639651E+05, 4.793296E+05, 4.950962E+05, 5.112730E+05, 5.278675E+05, 
#   5.448879E+05, 5.623420E+05, 5.802379E+05, 5.985839E+05, 6.173884E+05, 6.366594E+05, 
#   6.564056E+05, 6.766356E+05, 6.973580E+05, 7.185816E+05, 7.403151E+05, 
#])
# ============================ M = 3, I = 9 ============================
#M = 3
#I = 9
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.040186E+01, 2.814360E+02, 7.892482E+02, 1.446027E+03, 2.223527E+03, 3.105399E+03, 
#   4.080613E+03, 5.141076E+03, 6.280535E+03, 7.494020E+03, 8.777526E+03, 1.012786E+04, 
#   1.154254E+04, 1.301974E+04, 1.455823E+04, 1.615736E+04, 1.781698E+04, 1.953744E+04, 
#   2.131951E+04, 2.316436E+04, 2.507351E+04, 2.704881E+04, 2.909238E+04, 3.120656E+04, 
#   3.339397E+04, 3.565736E+04, 3.799969E+04, 4.042406E+04, 4.293369E+04, 4.553192E+04, 
#   4.822222E+04, 5.100811E+04, 5.389324E+04, 5.688131E+04, 5.997610E+04, 6.318146E+04, 
#   6.650132E+04, 6.993964E+04, 7.350048E+04, 7.718793E+04, 8.100616E+04, 8.495937E+04, 
#   8.905185E+04, 9.328794E+04, 9.767201E+04, 1.022086E+05, 1.069021E+05, 1.117571E+05, 
#   1.167783E+05, 1.219705E+05, 1.273382E+05, 1.328865E+05, 1.386201E+05, 1.445441E+05, 
#   1.506634E+05, 1.569832E+05, 1.635086E+05, 1.702449E+05, 1.771973E+05, 1.843714E+05, 
#   1.917724E+05, 1.994060E+05, 2.072776E+05, 2.153932E+05, 2.237582E+05, 2.323787E+05, 
#   2.412605E+05, 2.504095E+05, 2.598319E+05, 2.695338E+05, 2.795214E+05, 2.898010E+05, 
#   3.003790E+05, 3.112619E+05, 3.224561E+05, 3.339684E+05, 3.458054E+05, 3.579741E+05, 
#   3.704811E+05, 3.833336E+05, 3.965384E+05, 4.101031E+05, 4.240344E+05, 4.383398E+05, 
#   4.530269E+05, 4.681029E+05, 4.835757E+05, 4.994527E+05, 5.157418E+05, 5.324506E+05, 
#   5.495875E+05, 5.671601E+05, 5.851768E+05, 6.036457E+05, 6.225750E+05, 6.419733E+05, 
#   6.618490E+05, 6.822109E+05, 7.030672E+05, 7.244271E+05, 7.462993E+05, 
#])
# ============================ M = 3, I = 10 ============================
#M = 3
#I = 10
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.025754E+01, 2.754415E+02, 7.721314E+02, 1.414482E+03, 2.174879E+03, 3.037341E+03, 
#   3.991077E+03, 5.028178E+03, 6.142526E+03, 7.329254E+03, 8.584430E+03, 9.904907E+03, 
#   1.128822E+04, 1.273256E+04, 1.423667E+04, 1.579990E+04, 1.742206E+04, 1.910351E+04, 
#   2.084498E+04, 2.264764E+04, 2.451302E+04, 2.644295E+04, 2.843953E+04, 3.050517E+04, 
#   3.264244E+04, 3.485413E+04, 3.714319E+04, 3.951272E+04, 4.196595E+04, 4.450622E+04, 
#   4.713698E+04, 4.986176E+04, 5.268419E+04, 5.560797E+04, 5.863687E+04, 6.177470E+04, 
#   6.502539E+04, 6.839284E+04, 7.188114E+04, 7.549432E+04, 7.923653E+04, 8.311195E+04, 
#   8.712481E+04, 9.127946E+04, 9.558022E+04, 1.000315E+05, 1.046379E+05, 1.094038E+05, 
#   1.143338E+05, 1.194328E+05, 1.247052E+05, 1.301561E+05, 1.357901E+05, 1.416124E+05, 
#   1.476277E+05, 1.538412E+05, 1.602581E+05, 1.668834E+05, 1.737226E+05, 1.807809E+05, 
#   1.880637E+05, 1.955766E+05, 2.033251E+05, 2.113147E+05, 2.195514E+05, 2.280407E+05, 
#   2.367887E+05, 2.458012E+05, 2.550843E+05, 2.646440E+05, 2.744866E+05, 2.846183E+05, 
#   2.950454E+05, 3.057745E+05, 3.168118E+05, 3.281642E+05, 3.398382E+05, 3.518407E+05, 
#   3.641783E+05, 3.768582E+05, 3.898872E+05, 4.032725E+05, 4.170214E+05, 4.311409E+05, 
#   4.456386E+05, 4.605218E+05, 4.757982E+05, 4.914751E+05, 5.075607E+05, 5.240624E+05, 
#   5.409882E+05, 5.583462E+05, 5.761445E+05, 5.943911E+05, 6.130944E+05, 6.322626E+05, 
#   6.519042E+05, 6.720280E+05, 6.926425E+05, 7.137563E+05, 7.353782E+05, 
#])
# ============================ M = 3, I = 11 ============================
#M = 3
#I = 11
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   6.033293E+01, 1.619938E+03, 4.540936E+03, 8.318534E+03, 1.279034E+04, 1.786236E+04, 
#   2.347116E+04, 2.957020E+04, 3.612352E+04, 4.310246E+04, 5.048387E+04, 5.824916E+04, 
#   6.638374E+04, 7.487670E+04, 8.372060E+04, 9.291114E+04, 1.024470E+05, 1.123298E+05, 
#   1.225635E+05, 1.331545E+05, 1.441111E+05, 1.554437E+05, 1.671642E+05, 1.792859E+05, 
#   1.918235E+05, 2.047929E+05, 2.182108E+05, 2.320949E+05, 2.464635E+05, 2.613360E+05, 
#   2.767318E+05, 2.926713E+05, 3.091753E+05, 3.262649E+05, 3.439618E+05, 3.622879E+05, 
#   3.812657E+05, 4.009177E+05, 4.212672E+05, 4.423375E+05, 4.641525E+05, 4.867360E+05, 
#   5.101126E+05, 5.343069E+05, 5.593441E+05, 5.852495E+05, 6.120488E+05, 6.397681E+05, 
#   6.684339E+05, 6.980729E+05, 7.287124E+05, 7.603799E+05, 7.931029E+05, 8.269102E+05, 
#   8.618301E+05, 8.978916E+05, 9.351243E+05, 9.735580E+05, 1.013223E+06, 1.054149E+06, 
#   1.096369E+06, 1.139913E+06, 1.184812E+06, 1.231100E+06, 1.278810E+06, 1.327974E+06, 
#   1.378625E+06, 1.430799E+06, 1.484529E+06, 1.539851E+06, 1.596799E+06, 1.655410E+06, 
#   1.715720E+06, 1.777766E+06, 1.841585E+06, 1.907215E+06, 1.974694E+06, 2.044060E+06, 
#   2.115352E+06, 2.188612E+06, 2.263878E+06, 2.341191E+06, 2.420593E+06, 2.502124E+06, 
#   2.585828E+06, 2.671745E+06, 2.759921E+06, 2.850398E+06, 2.943220E+06, 3.038432E+06, 
#   3.136080E+06, 3.236209E+06, 3.338866E+06, 3.444095E+06, 3.551946E+06, 3.662466E+06, 
#   3.775702E+06, 3.891706E+06, 4.010524E+06, 4.132208E+06, 4.256807E+06, 
#])
# ============================ M = 3, I = 12 ============================
#M = 3
#I = 12
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   3.040601E+01, 8.200722E+02, 2.299310E+03, 4.212419E+03, 6.477137E+03, 9.045854E+03, 
#   1.188644E+04, 1.497532E+04, 1.829429E+04, 2.182884E+04, 2.556727E+04, 2.950019E+04, 
#   3.362029E+04, 3.792207E+04, 4.240184E+04, 4.705745E+04, 5.188832E+04, 5.689523E+04, 
#   6.208021E+04, 6.744646E+04, 7.299821E+04, 7.874060E+04, 8.467959E+04, 9.082188E+04, 
#   9.717478E+04, 1.037462E+05, 1.105446E+05, 1.175788E+05, 1.248579E+05, 1.323916E+05, 
#   1.401898E+05, 1.482625E+05, 1.566202E+05, 1.652735E+05, 1.742332E+05, 1.835105E+05, 
#   1.931165E+05, 2.030626E+05, 2.133604E+05, 2.240217E+05, 2.350584E+05, 2.464826E+05, 
#   2.583066E+05, 2.705427E+05, 2.832036E+05, 2.963021E+05, 3.098510E+05, 3.238634E+05, 
#   3.383528E+05, 3.533325E+05, 3.688162E+05, 3.848176E+05, 4.013509E+05, 4.184302E+05, 
#   4.360700E+05, 4.542847E+05, 4.730893E+05, 4.924985E+05, 5.125278E+05, 5.331923E+05, 
#   5.545078E+05, 5.764901E+05, 5.991551E+05, 6.225192E+05, 6.465987E+05, 6.714103E+05, 
#   6.969709E+05, 7.232976E+05, 7.504077E+05, 7.783188E+05, 8.070487E+05, 8.366153E+05, 
#   8.670370E+05, 8.983321E+05, 9.305194E+05, 9.636180E+05, 9.976467E+05, 1.032625E+06, 
#   1.068574E+06, 1.105511E+06, 1.143458E+06, 1.182434E+06, 1.222462E+06, 1.263560E+06, 
#   1.305752E+06, 1.349057E+06, 1.393498E+06, 1.439096E+06, 1.485874E+06, 1.533854E+06, 
#   1.583059E+06, 1.633511E+06, 1.685235E+06, 1.738253E+06, 1.792589E+06, 1.848267E+06, 
#   1.905310E+06, 1.963746E+06, 2.023596E+06, 2.084888E+06, 2.147645E+06, 
#])
# ============================ M = 3, I = 13 ============================
#M = 3
#I = 13
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   9.309759E-01, 2.531152E+01, 7.100520E+01, 1.301065E+02, 2.000728E+02, 2.794324E+02, 
#   3.671926E+02, 4.626250E+02, 5.651676E+02, 6.743742E+02, 7.898873E+02, 9.114249E+02, 
#   1.038771E+03, 1.171770E+03, 1.310326E+03, 1.454395E+03, 1.603982E+03, 1.759137E+03, 
#   1.919951E+03, 2.086553E+03, 2.259104E+03, 2.437793E+03, 2.622836E+03, 2.814471E+03, 
#   3.012957E+03, 3.218568E+03, 3.431595E+03, 3.652341E+03, 3.881122E+03, 4.118263E+03, 
#   4.364099E+03, 4.618972E+03, 4.883234E+03, 5.157244E+03, 5.441365E+03, 5.735966E+03, 
#   6.041427E+03, 6.358130E+03, 6.686463E+03, 7.026818E+03, 7.379600E+03, 7.745210E+03, 
#   8.124060E+03, 8.516567E+03, 8.923154E+03, 9.344249E+03, 9.780287E+03, 1.023171E+04, 
#   1.069896E+04, 1.118249E+04, 1.168277E+04, 1.220025E+04, 1.273542E+04, 1.328874E+04, 
#   1.386071E+04, 1.445181E+04, 1.506255E+04, 1.569343E+04, 1.634497E+04, 1.701768E+04, 
#   1.771209E+04, 1.842874E+04, 1.916817E+04, 1.993093E+04, 2.071758E+04, 2.152868E+04, 
#   2.236480E+04, 2.322653E+04, 2.411444E+04, 2.502915E+04, 2.597125E+04, 2.694135E+04, 
#   2.794008E+04, 2.896806E+04, 3.002592E+04, 3.111432E+04, 3.223390E+04, 3.338532E+04, 
#   3.456926E+04, 3.578639E+04, 3.703740E+04, 3.832299E+04, 3.964384E+04, 4.100069E+04, 
#   4.239425E+04, 4.382524E+04, 4.529440E+04, 4.680250E+04, 4.835027E+04, 4.993848E+04, 
#   5.156791E+04, 5.323933E+04, 5.495356E+04, 5.671137E+04, 5.851359E+04, 6.036102E+04, 
#   6.225450E+04, 6.419488E+04, 6.618297E+04, 6.821968E+04, 7.030582E+04, 
#])
# ============================ M = 3, I = 14 ============================
#M = 3
#I = 14
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.084643E+01, 2.934593E+02, 8.230106E+02, 1.507912E+03, 2.318707E+03, 3.238346E+03, 
#   4.255325E+03, 5.361204E+03, 6.549469E+03, 7.814941E+03, 9.153461E+03, 1.056173E+04, 
#   1.203721E+04, 1.357805E+04, 1.518306E+04, 1.685167E+04, 1.858386E+04, 2.038015E+04, 
#   2.224151E+04, 2.416933E+04, 2.616542E+04, 2.823190E+04, 3.037120E+04, 3.258599E+04, 
#   3.487921E+04, 3.725396E+04, 3.971354E+04, 4.226142E+04, 4.490118E+04, 4.763651E+04, 
#   5.047125E+04, 5.340928E+04, 5.645462E+04, 5.961137E+04, 6.288366E+04, 6.627573E+04, 
#   6.979188E+04, 7.343648E+04, 7.721396E+04, 8.112881E+04, 8.518561E+04, 8.938897E+04, 
#   9.374358E+04, 9.825419E+04, 1.029256E+05, 1.077628E+05, 1.127706E+05, 1.179541E+05, 
#   1.233183E+05, 1.288685E+05, 1.346099E+05, 1.405477E+05, 1.466874E+05, 1.530344E+05, 
#   1.595943E+05, 1.663725E+05, 1.733749E+05, 1.806072E+05, 1.880752E+05, 1.957849E+05, 
#   2.037422E+05, 2.119532E+05, 2.204242E+05, 2.291613E+05, 2.381710E+05, 2.474596E+05, 
#   2.570336E+05, 2.668997E+05, 2.770645E+05, 2.875348E+05, 2.983175E+05, 3.094196E+05, 
#   3.208480E+05, 3.326100E+05, 3.447128E+05, 3.571637E+05, 3.699700E+05, 3.831393E+05, 
#   3.966793E+05, 4.105977E+05, 4.249020E+05, 4.396007E+05, 4.547011E+05, 4.702117E+05, 
#   4.861407E+05, 5.024963E+05, 5.192868E+05, 5.365208E+05, 5.542069E+05, 5.723538E+05, 
#   5.909702E+05, 6.100649E+05, 6.296473E+05, 6.497260E+05, 6.703105E+05, 6.914101E+05, 
#   7.130340E+05, 7.351921E+05, 7.578936E+05, 7.811485E+05, 8.049666E+05, 
#])
# ============================ M = 3, I = 15 ============================
#M = 3
#I = 15
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   5.470224E+00, 1.486652E+02, 4.170261E+02, 7.641279E+02, 1.175038E+03, 1.641114E+03, 
#   2.156526E+03, 2.716996E+03, 3.319223E+03, 3.960583E+03, 4.638976E+03, 5.352734E+03, 
#   6.100578E+03, 6.881587E+03, 7.695167E+03, 8.541034E+03, 9.419186E+03, 1.032988E+04, 
#   1.127362E+04, 1.225110E+04, 1.326323E+04, 1.431106E+04, 1.539582E+04, 1.651885E+04, 
#   1.768160E+04, 1.888566E+04, 2.013266E+04, 2.142433E+04, 2.276248E+04, 2.414896E+04, 
#   2.558568E+04, 2.707462E+04, 2.861777E+04, 3.021720E+04, 3.187499E+04, 3.359326E+04, 
#   3.537418E+04, 3.721994E+04, 3.913277E+04, 4.111494E+04, 4.316873E+04, 4.529647E+04, 
#   4.750054E+04, 4.978331E+04, 5.214721E+04, 5.459470E+04, 5.712827E+04, 5.975045E+04, 
#   6.246379E+04, 6.527091E+04, 6.817443E+04, 7.117702E+04, 7.428139E+04, 7.749029E+04, 
#   8.080648E+04, 8.423280E+04, 8.777211E+04, 9.142730E+04, 9.520132E+04, 9.909715E+04, 
#   1.031178E+05, 1.072663E+05, 1.115459E+05, 1.159595E+05, 1.205105E+05, 1.252021E+05, 
#   1.300375E+05, 1.350201E+05, 1.401531E+05, 1.454401E+05, 1.508845E+05, 1.564898E+05, 
#   1.622595E+05, 1.681973E+05, 1.743066E+05, 1.805914E+05, 1.870552E+05, 1.937019E+05, 
#   2.005352E+05, 2.075591E+05, 2.147775E+05, 2.221943E+05, 2.298136E+05, 2.376394E+05, 
#   2.456758E+05, 2.539271E+05, 2.623974E+05, 2.710910E+05, 2.800123E+05, 2.891655E+05, 
#   2.985552E+05, 3.081856E+05, 3.180616E+05, 3.281874E+05, 3.385680E+05, 3.492077E+05, 
#   3.601115E+05, 3.712840E+05, 3.827302E+05, 3.944550E+05, 4.064631E+05, 
#])
# ============================ M = 3, I = 16 ============================
#M = 3
#I = 16
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   6.265142E+01, 1.678857E+03, 4.706439E+03, 8.621861E+03, 1.325682E+04, 1.851389E+04, 
#   2.432733E+04, 3.064889E+04, 3.744135E+04, 4.467509E+04, 5.232627E+04, 6.037598E+04, 
#   6.880951E+04, 7.761618E+04, 8.678894E+04, 9.632416E+04, 1.062214E+05, 1.164832E+05, 
#   1.271145E+05, 1.381229E+05, 1.495182E+05, 1.613118E+05, 1.735170E+05, 1.861485E+05, 
#   1.992225E+05, 2.127561E+05, 2.267676E+05, 2.412761E+05, 2.563014E+05, 2.718642E+05, 
#   2.879857E+05, 3.046875E+05, 3.219921E+05, 3.399222E+05, 3.585010E+05, 3.777519E+05, 
#   3.976990E+05, 4.183668E+05, 4.397797E+05, 4.619632E+05, 4.849424E+05, 5.087433E+05, 
#   5.333921E+05, 5.589152E+05, 5.853396E+05, 6.126924E+05, 6.410015E+05, 6.702946E+05, 
#   7.006002E+05, 7.319472E+05, 7.643648E+05, 7.978824E+05, 8.325302E+05, 8.683383E+05, 
#   9.053377E+05, 9.435596E+05, 9.830358E+05, 1.023798E+06, 1.065879E+06, 1.109312E+06, 
#   1.154130E+06, 1.200368E+06, 1.248059E+06, 1.297238E+06, 1.347940E+06, 1.400203E+06, 
#   1.454060E+06, 1.509550E+06, 1.566710E+06, 1.625577E+06, 1.686189E+06, 1.748585E+06, 
#   1.812805E+06, 1.878888E+06, 1.946874E+06, 2.016804E+06, 2.088719E+06, 2.162662E+06, 
#   2.238673E+06, 2.316797E+06, 2.397076E+06, 2.479555E+06, 2.564277E+06, 2.651288E+06, 
#   2.740634E+06, 2.832359E+06, 2.926512E+06, 3.023138E+06, 3.122288E+06, 3.224005E+06, 
#   3.328343E+06, 3.435348E+06, 3.545073E+06, 3.657565E+06, 3.772878E+06, 3.891062E+06, 
#   4.012169E+06, 4.136253E+06, 4.263368E+06, 4.393567E+06, 4.526904E+06, 
#])
# ============================ M = 3, I = 17 ============================
#M = 3
#I = 17
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   3.170713E+01, 8.552206E+02, 2.398013E+03, 4.393338E+03, 6.755395E+03, 9.434524E+03, 
#   1.239721E+04, 1.561888E+04, 1.908053E+04, 2.276706E+04, 2.666631E+04, 3.076859E+04, 
#   3.506635E+04, 3.955411E+04, 4.422821E+04, 4.908676E+04, 5.412950E+04, 5.935768E+04, 
#   6.477392E+04, 7.038207E+04, 7.618711E+04, 8.219498E+04, 8.841255E+04, 9.484741E+04, 
#   1.015078E+05, 1.084028E+05, 1.155415E+05, 1.229339E+05, 1.305904E+05, 1.385213E+05, 
#   1.467377E+05, 1.552509E+05, 1.640721E+05, 1.732132E+05, 1.826860E+05, 1.925028E+05, 
#   2.026757E+05, 2.132173E+05, 2.241404E+05, 2.354578E+05, 2.471825E+05, 2.593279E+05, 
#   2.719073E+05, 2.849344E+05, 2.984231E+05, 3.123871E+05, 3.268408E+05, 3.417987E+05, 
#   3.572751E+05, 3.732849E+05, 3.898431E+05, 4.069648E+05, 4.246656E+05, 4.429608E+05, 
#   4.618663E+05, 4.813983E+05, 5.015729E+05, 5.224067E+05, 5.439163E+05, 5.661186E+05, 
#   5.890311E+05, 6.126706E+05, 6.370552E+05, 6.622028E+05, 6.881313E+05, 7.148591E+05, 
#   7.424048E+05, 7.707874E+05, 8.000261E+05, 8.301398E+05, 8.611487E+05, 8.930721E+05, 
#   9.259307E+05, 9.597445E+05, 9.945342E+05, 1.030321E+06, 1.067126E+06, 1.104970E+06, 
#   1.143875E+06, 1.183865E+06, 1.224959E+06, 1.267181E+06, 1.310554E+06, 1.355102E+06, 
#   1.400847E+06, 1.447813E+06, 1.496024E+06, 1.545504E+06, 1.596278E+06, 1.648370E+06, 
#   1.701806E+06, 1.756611E+06, 1.812811E+06, 1.870432E+06, 1.929499E+06, 1.990040E+06, 
#   2.052081E+06, 2.115650E+06, 2.180773E+06, 2.247480E+06, 2.315797E+06, 
#])
# ============================ M = 3, I = 18 ============================
#M = 3
#I = 18
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.864598E+02, 5.029074E+03, 1.410094E+04, 2.583372E+04, 3.972289E+04, 5.547647E+04, 
#   7.289739E+04, 9.184110E+04, 1.121959E+05, 1.338730E+05, 1.568006E+05, 1.809216E+05, 
#   2.061912E+05, 2.325766E+05, 2.600555E+05, 2.886161E+05, 3.182556E+05, 3.489801E+05, 
#   3.808036E+05, 4.137473E+05, 4.478387E+05, 4.831112E+05, 5.196032E+05, 5.573578E+05, 
#   5.964214E+05, 6.368451E+05, 6.786818E+05, 7.219878E+05, 7.668217E+05, 8.132437E+05, 
#   8.613168E+05, 9.111043E+05, 9.626726E+05, 1.016088E+06, 1.071420E+06, 1.128737E+06, 
#   1.188111E+06, 1.249612E+06, 1.313314E+06, 1.379291E+06, 1.447618E+06, 1.518371E+06, 
#   1.591628E+06, 1.667465E+06, 1.745963E+06, 1.827203E+06, 1.911265E+06, 1.998231E+06, 
#   2.088186E+06, 2.181214E+06, 2.277402E+06, 2.376836E+06, 2.479605E+06, 2.585797E+06, 
#   2.695505E+06, 2.808819E+06, 2.925833E+06, 3.046641E+06, 3.171340E+06, 3.300026E+06, 
#   3.432797E+06, 3.569753E+06, 3.710995E+06, 3.856626E+06, 4.006750E+06, 4.161471E+06, 
#   4.320895E+06, 4.485133E+06, 4.654292E+06, 4.828483E+06, 5.007818E+06, 5.192412E+06, 
#   5.382378E+06, 5.577837E+06, 5.778902E+06, 5.985696E+06, 6.198339E+06, 6.416956E+06, 
#   6.641666E+06, 6.872600E+06, 7.109881E+06, 7.353643E+06, 7.604013E+06, 7.861122E+06, 
#   8.125110E+06, 8.396104E+06, 8.674247E+06, 8.959673E+06, 9.252525E+06, 9.552945E+06, 
#   9.861075E+06, 1.017706E+07, 1.050105E+07, 1.083318E+07, 1.117362E+07, 1.152251E+07, 
#   1.188001E+07, 1.224627E+07, 1.262144E+07, 1.300569E+07, 1.339918E+07, 
#])
# ============================ M = 4, I = 1 ============================
M = 4
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.834054E+01, 1.523262E+02, 3.016016E+02, 4.508934E+02, 6.001924E+02, 7.494969E+02, 
   8.988074E+02, 1.048131E+03, 1.197498E+03, 1.346979E+03, 1.496709E+03, 1.646898E+03, 
   1.797823E+03, 1.949825E+03, 2.103285E+03, 2.258615E+03, 2.416237E+03, 2.576575E+03, 
   2.740046E+03, 2.907051E+03, 3.077980E+03, 3.253200E+03, 3.433067E+03, 3.617916E+03, 
   3.808070E+03, 4.003838E+03, 4.205517E+03, 4.413398E+03, 4.627758E+03, 4.848873E+03, 
   5.077011E+03, 5.312437E+03, 5.555411E+03, 5.806195E+03, 6.065047E+03, 6.332224E+03, 
   6.607987E+03, 6.892595E+03, 7.186308E+03, 7.489390E+03, 7.802105E+03, 8.124722E+03, 
   8.457512E+03, 8.800747E+03, 9.154706E+03, 9.519670E+03, 9.895923E+03, 1.028375E+04, 
   1.068346E+04, 1.109533E+04, 1.151968E+04, 1.195680E+04, 1.240701E+04, 1.287063E+04, 
   1.334798E+04, 1.383938E+04, 1.434516E+04, 1.486567E+04, 1.540125E+04, 1.595223E+04, 
   1.651898E+04, 1.710186E+04, 1.770123E+04, 1.831745E+04, 1.895091E+04, 1.960198E+04, 
   2.027106E+04, 2.095853E+04, 2.166480E+04, 2.239026E+04, 2.313534E+04, 2.390046E+04, 
   2.468602E+04, 2.549246E+04, 2.632023E+04, 2.716976E+04, 2.804150E+04, 2.893590E+04, 
   2.985343E+04, 3.079456E+04, 3.175977E+04, 3.274952E+04, 3.376432E+04, 3.480465E+04, 
   3.587102E+04, 3.696394E+04, 3.808392E+04, 3.923149E+04, 4.040717E+04, 4.161150E+04, 
   4.284502E+04, 4.410829E+04, 4.540186E+04, 4.672629E+04, 4.808216E+04, 4.947005E+04, 
   5.089054E+04, 5.234422E+04, 5.383169E+04, 5.535356E+04, 5.691045E+04, 5.850297E+04, 
   6.013176E+04, 6.179745E+04, 6.350068E+04, 6.524210E+04, 6.702236E+04, 6.884215E+04, 
   7.070211E+04, 7.260294E+04, 7.454531E+04, 7.652992E+04, 7.855746E+04, 8.062865E+04, 
   8.274419E+04, 8.490480E+04, 8.711122E+04, 8.936416E+04, 9.166437E+04, 9.401259E+04, 
   9.640958E+04, 9.885608E+04, 1.013529E+05, 1.039007E+05, 1.065004E+05, 1.091527E+05, 
   1.118584E+05, 1.146182E+05, 1.174331E+05, 1.203037E+05, 1.232310E+05, 1.262156E+05, 
   1.292585E+05, 1.323604E+05, 1.355222E+05, 1.387447E+05, 1.420287E+05, 1.453751E+05, 
   1.487848E+05, 1.522585E+05, 1.557971E+05, 1.594015E+05, 1.630726E+05, 1.668111E+05, 
   1.706179E+05, 1.744940E+05, 1.784402E+05, 1.824573E+05, 1.865463E+05, 1.907079E+05, 
   1.949432E+05, 1.992528E+05, 2.036378E+05, 2.080990E+05, 2.126373E+05, 2.172536E+05, 
   2.219487E+05, 2.267235E+05, 2.315789E+05, 2.365158E+05, 2.415351E+05, 2.466376E+05, 
   2.518242E+05, 2.570959E+05, 2.624534E+05, 2.678976E+05, 2.734295E+05, 2.790498E+05, 
   2.847595E+05, 2.905594E+05, 2.964505E+05, 3.024334E+05, 3.085092E+05, 3.146786E+05, 
   3.209425E+05, 3.273018E+05, 3.337573E+05, 3.403098E+05, 3.469602E+05, 3.537093E+05, 
   3.605579E+05, 3.675069E+05, 3.745571E+05, 3.817092E+05, 3.889642E+05, 3.963228E+05, 
   4.037857E+05, 4.113539E+05, 4.190280E+05, 4.268089E+05, 4.346973E+05, 4.426940E+05, 
   4.507998E+05, 4.590153E+05, 4.673415E+05, 4.757789E+05, 4.843284E+05, 4.929907E+05, 
   5.017665E+05, 5.106564E+05, 5.196613E+05, 5.287817E+05, 5.380185E+05, 5.473723E+05, 
   5.568437E+05, 5.664334E+05, 5.761421E+05, 5.859705E+05, 5.959191E+05, 6.059887E+05, 
   6.161798E+05, 6.264931E+05, 6.369291E+05, 6.474885E+05, 6.581719E+05, 6.689798E+05, 
   6.799128E+05, 6.909716E+05, 7.021566E+05, 7.134683E+05, 7.249075E+05, 7.364745E+05, 
   7.481698E+05, 7.599941E+05, 7.719478E+05, 7.840314E+05, 7.962454E+05, 8.085902E+05, 
   8.210663E+05, 8.336743E+05, 8.464145E+05, 8.592873E+05, 8.722932E+05, 8.854327E+05, 
   8.987061E+05, 9.121138E+05, 9.256562E+05, 9.393337E+05, 9.531467E+05, 9.670956E+05, 
   9.811806E+05, 9.954022E+05, 1.009761E+06, 1.024256E+06, 1.038889E+06, 1.053660E+06, 
   1.068569E+06, 1.083617E+06, 1.098803E+06, 1.114128E+06, 1.129593E+06, 1.145196E+06, 
   1.160940E+06, 1.176823E+06, 1.192847E+06, 1.209011E+06, 1.225316E+06, 1.241761E+06, 
   1.258347E+06, 1.275075E+06, 1.291944E+06, 1.308954E+06, 1.326106E+06, 1.343400E+06, 
   1.360835E+06, 1.378413E+06, 1.396133E+06, 1.413995E+06, 1.431999E+06, 1.450146E+06, 
   1.468436E+06, 1.486868E+06, 1.505443E+06, 1.524160E+06, 1.543020E+06, 1.562023E+06, 
   1.581169E+06, 1.600458E+06, 1.619890E+06, 1.639465E+06, 1.659183E+06, 1.679043E+06, 
   1.699047E+06, 1.719193E+06, 1.739482E+06, 1.759914E+06, 1.780489E+06, 1.801207E+06, 
   1.822067E+06, 1.843070E+06, 1.864215E+06, 1.885503E+06, 1.906934E+06, 1.928506E+06, 
   1.950221E+06, 1.972078E+06, 1.994077E+06, 2.016217E+06, 2.038500E+06, 2.060924E+06, 
   2.083489E+06, 2.106196E+06, 2.129044E+06, 2.152032E+06, 2.175162E+06, 2.198432E+06, 
   2.221843E+06, 2.245394E+06, 2.269086E+06, 2.292917E+06, 2.316888E+06, 2.340998E+06, 
   2.365247E+06, 2.389636E+06, 2.414164E+06, 2.438830E+06, 2.463635E+06, 2.488577E+06, 
   2.513658E+06, 2.538876E+06, 2.564232E+06, 2.589725E+06, 2.615355E+06, 2.641121E+06, 
   2.667024E+06, 2.693063E+06, 2.719237E+06, 2.745547E+06, 2.771992E+06, 2.798573E+06, 
   2.825287E+06, 2.852136E+06, 2.879119E+06, 2.906236E+06, 2.933486E+06, 2.960869E+06, 
   2.988385E+06, 3.016033E+06, 3.043814E+06, 3.071726E+06, 3.099769E+06, 3.127943E+06, 
   3.156248E+06, 3.184684E+06, 3.213249E+06, 3.241944E+06, 3.270768E+06, 3.299721E+06, 
   3.328803E+06, 3.358012E+06, 3.387350E+06, 3.416815E+06, 3.446406E+06, 3.476125E+06, 
   3.505969E+06, 3.535940E+06, 3.566036E+06, 3.596257E+06, 3.626603E+06, 3.657073E+06, 
   3.687666E+06, 3.718384E+06, 3.749224E+06, 3.780187E+06, 3.811272E+06, 3.842479E+06, 
   3.873807E+06, 3.905256E+06, 3.936826E+06, 3.968516E+06, 4.000325E+06, 4.032254E+06, 
   4.064302E+06, 4.096468E+06, 4.128752E+06, 4.161153E+06, 4.193672E+06, 4.226307E+06, 
   4.259058E+06, 4.291926E+06, 4.324908E+06, 4.358006E+06, 4.391217E+06, 4.424543E+06, 
   4.457982E+06, 4.491535E+06, 4.525200E+06, 4.558977E+06, 4.592866E+06, 4.626866E+06, 
   4.660977E+06, 4.695199E+06, 4.729530E+06, 4.763971E+06, 4.798520E+06, 4.833178E+06, 
   4.867944E+06, 4.902818E+06, 4.937799E+06, 4.972887E+06, 5.008080E+06, 5.043380E+06, 
   5.078784E+06, 5.114294E+06, 5.149907E+06, 5.185625E+06, 5.221446E+06, 5.257369E+06, 
   5.293396E+06, 5.329524E+06, 5.365753E+06, 5.402084E+06, 5.438515E+06, 5.475046E+06, 
   5.511677E+06, 5.548407E+06, 5.585236E+06, 5.622162E+06, 5.659187E+06, 5.696308E+06, 
   5.733527E+06, 5.770841E+06, 5.808252E+06, 5.845757E+06, 5.883358E+06, 5.921053E+06, 
   5.958842E+06, 5.996724E+06, 6.034699E+06, 6.072767E+06, 6.110926E+06, 6.149177E+06, 
   6.187520E+06, 6.225952E+06, 6.264475E+06, 6.303088E+06, 6.341789E+06, 6.380579E+06, 
   6.419458E+06, 6.458424E+06, 6.497478E+06, 6.536618E+06, 6.575844E+06, 6.615157E+06, 
   6.654554E+06, 6.694037E+06, 6.733604E+06, 6.773255E+06, 6.812990E+06, 6.852807E+06, 
   6.892708E+06, 6.932690E+06, 6.972754E+06, 7.012899E+06, 7.053125E+06, 7.093431E+06, 
   7.133816E+06, 7.174282E+06, 7.214826E+06, 7.255448E+06, 7.296148E+06, 7.336926E+06, 
   7.377781E+06, 7.418712E+06, 7.459719E+06, 7.500802E+06, 7.541961E+06, 7.583194E+06, 
   7.624501E+06, 7.665882E+06, 7.707336E+06, 7.748864E+06, 7.790463E+06, 7.832135E+06, 
   7.873879E+06, 7.915693E+06, 7.957578E+06, 7.999534E+06, 8.041559E+06, 8.083654E+06, 
   8.125817E+06, 8.168049E+06, 8.210349E+06, 8.252716E+06, 8.295151E+06, 8.337652E+06, 
   8.380220E+06, 8.422853E+06, 8.465552E+06, 8.508316E+06, 8.551144E+06, 8.594036E+06, 
   8.636993E+06, 8.680012E+06, 8.723094E+06, 8.766238E+06, 8.809445E+06, 8.852713E+06, 
   8.896042E+06, 8.939432E+06, 8.982882E+06, 
])
# ============================ M = 4, I = 2 ============================
M = 4
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.222765E+01, 1.015679E+02, 2.011218E+02, 3.007066E+02, 4.003161E+02, 4.999492E+02, 
   5.996065E+02, 6.992945E+02, 7.990374E+02, 8.988927E+02, 9.989652E+02, 1.099412E+03, 
   1.200442E+03, 1.302300E+03, 1.405262E+03, 1.509620E+03, 1.615671E+03, 1.723709E+03, 
   1.834022E+03, 1.946887E+03, 2.062569E+03, 2.181320E+03, 2.303380E+03, 2.428977E+03, 
   2.558331E+03, 2.691651E+03, 2.829140E+03, 2.970992E+03, 3.117401E+03, 3.268551E+03, 
   3.424627E+03, 3.585810E+03, 3.752280E+03, 3.924217E+03, 4.101798E+03, 4.285203E+03, 
   4.474610E+03, 4.670201E+03, 4.872157E+03, 5.080660E+03, 5.295897E+03, 5.518054E+03, 
   5.747321E+03, 5.983889E+03, 6.227952E+03, 6.479709E+03, 6.739359E+03, 7.007105E+03, 
   7.283152E+03, 7.567711E+03, 7.860994E+03, 8.163217E+03, 8.474598E+03, 8.795360E+03, 
   9.125730E+03, 9.465938E+03, 9.816216E+03, 1.017680E+04, 1.054794E+04, 1.092987E+04, 
   1.132284E+04, 1.172710E+04, 1.214292E+04, 1.257055E+04, 1.301025E+04, 1.346229E+04, 
   1.392695E+04, 1.440451E+04, 1.489523E+04, 1.539942E+04, 1.591734E+04, 1.644931E+04, 
   1.699562E+04, 1.755656E+04, 1.813245E+04, 1.872360E+04, 1.933031E+04, 1.995292E+04, 
   2.059174E+04, 2.124710E+04, 2.191933E+04, 2.260878E+04, 2.331578E+04, 2.404068E+04, 
   2.478383E+04, 2.554560E+04, 2.632633E+04, 2.712640E+04, 2.794617E+04, 2.878602E+04, 
   2.964634E+04, 3.052750E+04, 3.142989E+04, 3.235392E+04, 3.329997E+04, 3.426846E+04, 
   3.525980E+04, 3.627440E+04, 3.731268E+04, 3.837507E+04, 3.946199E+04, 4.057389E+04, 
   4.171120E+04, 4.287437E+04, 4.406385E+04, 4.528011E+04, 4.652359E+04, 4.779478E+04, 
   4.909414E+04, 5.042215E+04, 5.177930E+04, 5.316608E+04, 5.458298E+04, 5.603051E+04, 
   5.750917E+04, 5.901947E+04, 6.056193E+04, 6.213708E+04, 6.374545E+04, 6.538757E+04, 
   6.706399E+04, 6.877525E+04, 7.052190E+04, 7.230451E+04, 7.412364E+04, 7.597986E+04, 
   7.787375E+04, 7.980590E+04, 8.177689E+04, 8.378732E+04, 8.583780E+04, 8.792892E+04, 
   9.006131E+04, 9.223559E+04, 9.445238E+04, 9.671232E+04, 9.901605E+04, 1.013642E+05, 
   1.037575E+05, 1.061965E+05, 1.086819E+05, 1.112144E+05, 1.137947E+05, 1.164234E+05, 
   1.191013E+05, 1.218290E+05, 1.246073E+05, 1.274368E+05, 1.303184E+05, 1.332526E+05, 
   1.362403E+05, 1.392821E+05, 1.423790E+05, 1.455315E+05, 1.487404E+05, 1.520066E+05, 
   1.553308E+05, 1.587137E+05, 1.621562E+05, 1.656591E+05, 1.692232E+05, 1.728492E+05, 
   1.765380E+05, 1.802905E+05, 1.841075E+05, 1.879897E+05, 1.919381E+05, 1.959535E+05, 
   2.000368E+05, 2.041888E+05, 2.084104E+05, 2.127026E+05, 2.170661E+05, 2.215020E+05, 
   2.260111E+05, 2.305943E+05, 2.352526E+05, 2.399869E+05, 2.447981E+05, 2.496872E+05, 
   2.546551E+05, 2.597029E+05, 2.648314E+05, 2.700417E+05, 2.753347E+05, 2.807114E+05, 
   2.861730E+05, 2.917202E+05, 2.973543E+05, 3.030762E+05, 3.088869E+05, 3.147875E+05, 
   3.207791E+05, 3.268627E+05, 3.330394E+05, 3.393103E+05, 3.456764E+05, 3.521388E+05, 
   3.586988E+05, 3.653573E+05, 3.721155E+05, 3.789745E+05, 3.859356E+05, 3.929997E+05, 
   4.001681E+05, 4.074420E+05, 4.148224E+05, 4.223107E+05, 4.299080E+05, 4.376155E+05, 
   4.454344E+05, 4.533659E+05, 4.614113E+05, 4.695718E+05, 4.778486E+05, 4.862431E+05, 
   4.947564E+05, 5.033899E+05, 5.121448E+05, 5.210224E+05, 5.300241E+05, 5.391511E+05, 
   5.484048E+05, 5.577865E+05, 5.672975E+05, 5.769392E+05, 5.867129E+05, 5.966201E+05, 
   6.066621E+05, 6.168403E+05, 6.271560E+05, 6.376108E+05, 6.482060E+05, 6.589430E+05, 
   6.698233E+05, 6.808483E+05, 6.920195E+05, 7.033384E+05, 7.148064E+05, 7.264251E+05, 
   7.381958E+05, 7.501202E+05, 7.621998E+05, 7.744360E+05, 7.868304E+05, 7.993847E+05, 
   8.121002E+05, 8.249786E+05, 8.380215E+05, 8.512305E+05, 8.646072E+05, 8.781531E+05, 
   8.918699E+05, 9.057593E+05, 9.198228E+05, 9.340622E+05, 9.484791E+05, 9.630751E+05, 
   9.778520E+05, 9.928115E+05, 1.007955E+06, 1.023285E+06, 1.038802E+06, 1.054509E+06, 
   1.070407E+06, 1.086498E+06, 1.102784E+06, 1.119266E+06, 1.135947E+06, 1.152828E+06, 
   1.169910E+06, 1.187197E+06, 1.204689E+06, 1.222388E+06, 1.240297E+06, 1.258417E+06, 
   1.276750E+06, 1.295298E+06, 1.314063E+06, 1.333047E+06, 1.352252E+06, 1.371679E+06, 
   1.391331E+06, 1.411209E+06, 1.431316E+06, 1.451654E+06, 1.472224E+06, 1.493029E+06, 
   1.514071E+06, 1.535351E+06, 1.556872E+06, 1.578635E+06, 1.600643E+06, 1.622899E+06, 
   1.645403E+06, 1.668159E+06, 1.691167E+06, 1.714431E+06, 1.737953E+06, 1.761734E+06, 
   1.785778E+06, 1.810085E+06, 1.834658E+06, 1.859500E+06, 1.884613E+06, 1.909998E+06, 
   1.935659E+06, 1.961597E+06, 1.987815E+06, 2.014314E+06, 2.041098E+06, 2.068169E+06, 
   2.095528E+06, 2.123178E+06, 2.151123E+06, 2.179363E+06, 2.207901E+06, 2.236740E+06, 
   2.265882E+06, 2.295330E+06, 2.325085E+06, 2.355151E+06, 2.385530E+06, 2.416224E+06, 
   2.447235E+06, 2.478567E+06, 2.510222E+06, 2.542202E+06, 2.574509E+06, 2.607147E+06, 
   2.640118E+06, 2.673424E+06, 2.707069E+06, 2.741054E+06, 2.775382E+06, 2.810056E+06, 
   2.845079E+06, 2.880452E+06, 2.916180E+06, 2.952264E+06, 2.988708E+06, 3.025513E+06, 
   3.062683E+06, 3.100220E+06, 3.138127E+06, 3.176408E+06, 3.215064E+06, 3.254098E+06, 
   3.293514E+06, 3.333313E+06, 3.373500E+06, 3.414076E+06, 3.455045E+06, 3.496410E+06, 
   3.538172E+06, 3.580337E+06, 3.622905E+06, 3.665880E+06, 3.709266E+06, 3.753065E+06, 
   3.797280E+06, 3.841913E+06, 3.886969E+06, 3.932450E+06, 3.978359E+06, 4.024699E+06, 
   4.071473E+06, 4.118685E+06, 4.166337E+06, 4.214432E+06, 4.262975E+06, 4.311966E+06, 
   4.361411E+06, 4.411312E+06, 4.461673E+06, 4.512495E+06, 4.563784E+06, 4.615541E+06, 
   4.667771E+06, 4.720476E+06, 4.773660E+06, 4.827325E+06, 4.881476E+06, 4.936116E+06, 
   4.991248E+06, 5.046875E+06, 5.103001E+06, 5.159629E+06, 5.216762E+06, 5.274405E+06, 
   5.332559E+06, 5.391230E+06, 5.450420E+06, 5.510132E+06, 5.570371E+06, 5.631140E+06, 
   5.692442E+06, 5.754281E+06, 5.816660E+06, 5.879583E+06, 5.943054E+06, 6.007076E+06, 
   6.071653E+06, 6.136788E+06, 6.202486E+06, 6.268749E+06, 6.335581E+06, 6.402987E+06, 
   6.470970E+06, 6.539534E+06, 6.608682E+06, 6.678418E+06, 6.748746E+06, 6.819670E+06, 
   6.891193E+06, 6.963320E+06, 7.036054E+06, 7.109400E+06, 7.183360E+06, 7.257940E+06, 
   7.333142E+06, 7.408972E+06, 7.485432E+06, 7.562527E+06, 7.640261E+06, 7.718638E+06, 
   7.797661E+06, 7.877335E+06, 7.957665E+06, 8.038653E+06, 8.120304E+06, 8.202623E+06, 
   8.285613E+06, 8.369279E+06, 8.453624E+06, 8.538653E+06, 8.624370E+06, 8.710780E+06, 
   8.797886E+06, 8.885692E+06, 8.974204E+06, 9.063426E+06, 9.153360E+06, 9.244013E+06, 
   9.335389E+06, 9.427491E+06, 9.520324E+06, 9.613892E+06, 9.708201E+06, 9.803253E+06, 
   9.899055E+06, 9.995610E+06, 1.009292E+07, 1.019100E+07, 1.028984E+07, 1.038945E+07, 
   1.048984E+07, 1.059101E+07, 1.069296E+07, 1.079571E+07, 1.089925E+07, 1.100358E+07, 
   1.110872E+07, 1.121467E+07, 1.132144E+07, 1.142902E+07, 1.153742E+07, 1.164665E+07, 
   1.175671E+07, 1.186761E+07, 1.197935E+07, 1.209194E+07, 1.220538E+07, 1.231967E+07, 
   1.243483E+07, 1.255085E+07, 1.266775E+07, 1.278552E+07, 1.290417E+07, 1.302370E+07, 
   1.314413E+07, 1.326546E+07, 1.338768E+07, 1.351081E+07, 1.363486E+07, 1.375982E+07, 
   1.388570E+07, 1.401250E+07, 1.414024E+07, 1.426892E+07, 1.439854E+07, 1.452910E+07, 
   1.466062E+07, 1.479310E+07, 1.492654E+07, 1.506095E+07, 1.519633E+07, 1.533270E+07, 
   1.547004E+07, 1.560838E+07, 1.574771E+07, 
])
# ============================ M = 4, I = 3 ============================
M = 4
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.256432E+01, 1.050397E+02, 2.080675E+02, 3.111266E+02, 4.142112E+02, 5.173201E+02, 
   6.204538E+02, 7.236178E+02, 8.268336E+02, 9.301535E+02, 1.033675E+03, 1.137547E+03, 
   1.241969E+03, 1.347182E+03, 1.453455E+03, 1.561078E+03, 1.670348E+03, 1.781563E+03, 
   1.895012E+03, 2.010979E+03, 2.129734E+03, 2.251535E+03, 2.376628E+03, 2.505250E+03, 
   2.637625E+03, 2.773970E+03, 2.914492E+03, 3.059395E+03, 3.208875E+03, 3.363124E+03, 
   3.522331E+03, 3.686683E+03, 3.856363E+03, 4.031555E+03, 4.212441E+03, 4.399205E+03, 
   4.592030E+03, 4.791098E+03, 4.996595E+03, 5.208707E+03, 5.427621E+03, 5.653529E+03, 
   5.886621E+03, 6.127092E+03, 6.375138E+03, 6.630959E+03, 6.894758E+03, 7.166739E+03, 
   7.447109E+03, 7.736081E+03, 8.033869E+03, 8.340690E+03, 8.656764E+03, 8.982318E+03, 
   9.317577E+03, 9.662775E+03, 1.001815E+04, 1.038393E+04, 1.076036E+04, 1.114770E+04, 
   1.154618E+04, 1.195607E+04, 1.237762E+04, 1.281110E+04, 1.325676E+04, 1.371488E+04, 
   1.418573E+04, 1.466960E+04, 1.516675E+04, 1.567749E+04, 1.620209E+04, 1.674086E+04, 
   1.729409E+04, 1.786208E+04, 1.844515E+04, 1.904360E+04, 1.965775E+04, 2.028792E+04, 
   2.093445E+04, 2.159765E+04, 2.227785E+04, 2.297542E+04, 2.369067E+04, 2.442396E+04, 
   2.517565E+04, 2.594610E+04, 2.673565E+04, 2.754469E+04, 2.837358E+04, 2.922271E+04, 
   3.009244E+04, 3.098317E+04, 3.189529E+04, 3.282920E+04, 3.378529E+04, 3.476398E+04, 
   3.576568E+04, 3.679079E+04, 3.783975E+04, 3.891298E+04, 4.001092E+04, 4.113399E+04, 
   4.228265E+04, 4.345734E+04, 4.465851E+04, 4.588663E+04, 4.714215E+04, 4.842555E+04, 
   4.973730E+04, 5.107789E+04, 5.244779E+04, 5.384750E+04, 5.527752E+04, 5.673835E+04, 
   5.823050E+04, 5.975448E+04, 6.131082E+04, 6.290003E+04, 6.452265E+04, 6.617922E+04, 
   6.787028E+04, 6.959638E+04, 7.135808E+04, 7.315593E+04, 7.499051E+04, 7.686238E+04, 
   7.877212E+04, 8.072034E+04, 8.270760E+04, 8.473451E+04, 8.680168E+04, 8.890971E+04, 
   9.105922E+04, 9.325084E+04, 9.548518E+04, 9.776290E+04, 1.000846E+05, 1.024510E+05, 
   1.048627E+05, 1.073204E+05, 1.098247E+05, 1.123763E+05, 1.149760E+05, 1.176243E+05, 
   1.203220E+05, 1.230698E+05, 1.258683E+05, 1.287184E+05, 1.316207E+05, 1.345760E+05, 
   1.375849E+05, 1.406483E+05, 1.437668E+05, 1.469413E+05, 1.501725E+05, 1.534612E+05, 
   1.568081E+05, 1.602140E+05, 1.636798E+05, 1.672062E+05, 1.707940E+05, 1.744441E+05, 
   1.781572E+05, 1.819343E+05, 1.857760E+05, 1.896833E+05, 1.936571E+05, 1.976981E+05, 
   2.018073E+05, 2.059855E+05, 2.102336E+05, 2.145524E+05, 2.189430E+05, 2.234061E+05, 
   2.279428E+05, 2.325538E+05, 2.372402E+05, 2.420029E+05, 2.468428E+05, 2.517608E+05, 
   2.567580E+05, 2.618353E+05, 2.669937E+05, 2.722341E+05, 2.775575E+05, 2.829650E+05, 
   2.884575E+05, 2.940361E+05, 2.997018E+05, 3.054556E+05, 3.112985E+05, 3.172317E+05, 
   3.232561E+05, 3.293729E+05, 3.355830E+05, 3.418876E+05, 3.482878E+05, 3.547847E+05, 
   3.613793E+05, 3.680728E+05, 3.748664E+05, 3.817611E+05, 3.887581E+05, 3.958585E+05, 
   4.030635E+05, 4.103743E+05, 4.177921E+05, 4.253180E+05, 4.329532E+05, 4.406989E+05, 
   4.485564E+05, 4.565268E+05, 4.646115E+05, 4.728115E+05, 4.811283E+05, 4.895630E+05, 
   4.981170E+05, 5.067914E+05, 5.155875E+05, 5.245068E+05, 5.335504E+05, 5.427198E+05, 
   5.520161E+05, 5.614408E+05, 5.709952E+05, 5.806806E+05, 5.904984E+05, 6.004500E+05, 
   6.105367E+05, 6.207600E+05, 6.311213E+05, 6.416218E+05, 6.522632E+05, 6.630468E+05, 
   6.739740E+05, 6.850463E+05, 6.962652E+05, 7.076321E+05, 7.191485E+05, 7.308159E+05, 
   7.426357E+05, 7.546096E+05, 7.667390E+05, 7.790255E+05, 7.914705E+05, 8.040757E+05, 
   8.168426E+05, 8.297727E+05, 8.428677E+05, 8.561292E+05, 8.695587E+05, 8.831578E+05, 
   8.969282E+05, 9.108716E+05, 9.249895E+05, 9.392837E+05, 9.537557E+05, 9.684073E+05, 
   9.832401E+05, 9.982558E+05, 1.013456E+06, 1.028843E+06, 1.044418E+06, 1.060183E+06, 
   1.076139E+06, 1.092289E+06, 1.108634E+06, 1.125175E+06, 1.141916E+06, 1.158857E+06, 
   1.176000E+06, 1.193347E+06, 1.210901E+06, 1.228662E+06, 1.246633E+06, 1.264816E+06, 
   1.283212E+06, 1.301824E+06, 1.320653E+06, 1.339701E+06, 1.358970E+06, 1.378462E+06, 
   1.398180E+06, 1.418124E+06, 1.438297E+06, 1.458702E+06, 1.479339E+06, 1.500211E+06, 
   1.521321E+06, 1.542669E+06, 1.564259E+06, 1.586091E+06, 1.608169E+06, 1.630495E+06, 
   1.653069E+06, 1.675896E+06, 1.698976E+06, 1.722311E+06, 1.745905E+06, 1.769759E+06, 
   1.793875E+06, 1.818256E+06, 1.842903E+06, 1.867819E+06, 1.893006E+06, 1.918466E+06, 
   1.944202E+06, 1.970216E+06, 1.996510E+06, 2.023087E+06, 2.049948E+06, 2.077096E+06, 
   2.104533E+06, 2.132262E+06, 2.160285E+06, 2.188604E+06, 2.217222E+06, 2.246141E+06, 
   2.275364E+06, 2.304892E+06, 2.334729E+06, 2.364876E+06, 2.395337E+06, 2.426114E+06, 
   2.457208E+06, 2.488624E+06, 2.520362E+06, 2.552426E+06, 2.584819E+06, 2.617542E+06, 
   2.650598E+06, 2.683990E+06, 2.717721E+06, 2.751793E+06, 2.786209E+06, 2.820970E+06, 
   2.856081E+06, 2.891544E+06, 2.927360E+06, 2.963534E+06, 3.000067E+06, 3.036962E+06, 
   3.074223E+06, 3.111851E+06, 3.149850E+06, 3.188222E+06, 3.226970E+06, 3.266098E+06, 
   3.305606E+06, 3.345500E+06, 3.385780E+06, 3.426451E+06, 3.467515E+06, 3.508975E+06, 
   3.550833E+06, 3.593094E+06, 3.635759E+06, 3.678831E+06, 3.722314E+06, 3.766211E+06, 
   3.810524E+06, 3.855256E+06, 3.900411E+06, 3.945991E+06, 3.992000E+06, 4.038440E+06, 
   4.085316E+06, 4.132628E+06, 4.180382E+06, 4.228579E+06, 4.277224E+06, 4.326318E+06, 
   4.375866E+06, 4.425871E+06, 4.476335E+06, 4.527262E+06, 4.578656E+06, 4.630518E+06, 
   4.682854E+06, 4.735665E+06, 4.788955E+06, 4.842728E+06, 4.896986E+06, 4.951733E+06, 
   5.006973E+06, 5.062709E+06, 5.118943E+06, 5.175680E+06, 5.232924E+06, 5.290676E+06, 
   5.348941E+06, 5.407723E+06, 5.467024E+06, 5.526848E+06, 5.587199E+06, 5.648080E+06, 
   5.709494E+06, 5.771446E+06, 5.833939E+06, 5.896976E+06, 5.960561E+06, 6.024698E+06, 
   6.089390E+06, 6.154641E+06, 6.220454E+06, 6.286833E+06, 6.353782E+06, 6.421305E+06, 
   6.489405E+06, 6.558086E+06, 6.627352E+06, 6.697207E+06, 6.767654E+06, 6.838697E+06, 
   6.910340E+06, 6.982586E+06, 7.055441E+06, 7.128907E+06, 7.202988E+06, 7.277689E+06, 
   7.353013E+06, 7.428965E+06, 7.505547E+06, 7.582765E+06, 7.660622E+06, 7.739122E+06, 
   7.818269E+06, 7.898067E+06, 7.978520E+06, 8.059634E+06, 8.141410E+06, 8.223854E+06, 
   8.306970E+06, 8.390761E+06, 8.475232E+06, 8.560388E+06, 8.646232E+06, 8.732769E+06, 
   8.820003E+06, 8.907937E+06, 8.996577E+06, 9.085927E+06, 9.175991E+06, 9.266773E+06, 
   9.358277E+06, 9.450509E+06, 9.543472E+06, 9.637171E+06, 9.731610E+06, 9.826793E+06, 
   9.922726E+06, 1.001941E+07, 1.011686E+07, 1.021506E+07, 1.031404E+07, 1.041378E+07, 
   1.051430E+07, 1.061561E+07, 1.071769E+07, 1.082057E+07, 1.092424E+07, 1.102872E+07, 
   1.113399E+07, 1.124007E+07, 1.134697E+07, 1.145469E+07, 1.156322E+07, 1.167259E+07, 
   1.178279E+07, 1.189382E+07, 1.200570E+07, 1.211842E+07, 1.223200E+07, 1.234643E+07, 
   1.246173E+07, 1.257789E+07, 1.269492E+07, 1.281282E+07, 1.293161E+07, 1.305129E+07, 
   1.317185E+07, 1.329332E+07, 1.341568E+07, 1.353895E+07, 1.366313E+07, 1.378823E+07, 
   1.391425E+07, 1.404120E+07, 1.416908E+07, 1.429789E+07, 1.442765E+07, 1.455836E+07, 
   1.469002E+07, 1.482264E+07, 1.495622E+07, 1.509077E+07, 1.522629E+07, 1.536279E+07, 
   1.550028E+07, 1.563876E+07, 1.577824E+07, 
])
# ============================ M = 4, I = 4 ============================
M = 4
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.919864E+01, 1.611838E+02, 3.193518E+02, 4.775675E+02, 6.358222E+02, 7.941142E+02, 
   9.524442E+02, 1.110821E+03, 1.269278E+03, 1.427897E+03, 1.586830E+03, 1.746306E+03, 
   1.906636E+03, 2.068189E+03, 2.231386E+03, 2.396671E+03, 2.564504E+03, 2.735343E+03, 
   2.909637E+03, 3.087823E+03, 3.270318E+03, 3.457522E+03, 3.649817E+03, 3.847567E+03, 
   4.051122E+03, 4.260818E+03, 4.476979E+03, 4.699920E+03, 4.929946E+03, 5.167356E+03, 
   5.412446E+03, 5.665505E+03, 5.926820E+03, 6.196678E+03, 6.475363E+03, 6.763159E+03, 
   7.060351E+03, 7.367227E+03, 7.684073E+03, 8.011180E+03, 8.348840E+03, 8.697348E+03, 
   9.057004E+03, 9.428110E+03, 9.810971E+03, 1.020590E+04, 1.061321E+04, 1.103321E+04, 
   1.146624E+04, 1.191262E+04, 1.237268E+04, 1.284677E+04, 1.333522E+04, 1.383839E+04, 
   1.435662E+04, 1.489028E+04, 1.543974E+04, 1.600536E+04, 1.658752E+04, 1.718660E+04, 
   1.780300E+04, 1.843710E+04, 1.908931E+04, 1.976004E+04, 2.044969E+04, 2.115868E+04, 
   2.188745E+04, 2.263642E+04, 2.340603E+04, 2.419672E+04, 2.500896E+04, 2.584319E+04, 
   2.669987E+04, 2.757949E+04, 2.848252E+04, 2.940944E+04, 3.036075E+04, 3.133694E+04, 
   3.233852E+04, 3.336601E+04, 3.441992E+04, 3.550077E+04, 3.660911E+04, 3.774548E+04, 
   3.891041E+04, 4.010448E+04, 4.132824E+04, 4.258226E+04, 4.386713E+04, 4.518342E+04, 
   4.653173E+04, 4.791267E+04, 4.932683E+04, 5.077484E+04, 5.225733E+04, 5.377492E+04, 
   5.532825E+04, 5.691797E+04, 5.854474E+04, 6.020923E+04, 6.191209E+04, 6.365403E+04, 
   6.543571E+04, 6.725785E+04, 6.912113E+04, 7.102629E+04, 7.297404E+04, 7.496511E+04, 
   7.700024E+04, 7.908018E+04, 8.120568E+04, 8.337751E+04, 8.559644E+04, 8.786326E+04, 
   9.017875E+04, 9.254372E+04, 9.495897E+04, 9.742533E+04, 9.994361E+04, 1.025147E+05, 
   1.051393E+05, 1.078184E+05, 1.105529E+05, 1.133436E+05, 1.161913E+05, 1.190970E+05, 
   1.220616E+05, 1.250860E+05, 1.281711E+05, 1.313179E+05, 1.345272E+05, 1.378000E+05, 
   1.411374E+05, 1.445402E+05, 1.480094E+05, 1.515460E+05, 1.551511E+05, 1.588256E+05, 
   1.625706E+05, 1.663870E+05, 1.702760E+05, 1.742385E+05, 1.782757E+05, 1.823886E+05, 
   1.865782E+05, 1.908458E+05, 1.951923E+05, 1.996189E+05, 2.041266E+05, 2.087168E+05, 
   2.133904E+05, 2.181487E+05, 2.229928E+05, 2.279238E+05, 2.329431E+05, 2.380517E+05, 
   2.432509E+05, 2.485419E+05, 2.539260E+05, 2.594043E+05, 2.649782E+05, 2.706488E+05, 
   2.764176E+05, 2.822857E+05, 2.882545E+05, 2.943252E+05, 3.004993E+05, 3.067780E+05, 
   3.131627E+05, 3.196548E+05, 3.262555E+05, 3.329664E+05, 3.397888E+05, 3.467240E+05, 
   3.537736E+05, 3.609389E+05, 3.682214E+05, 3.756226E+05, 3.831439E+05, 3.907868E+05, 
   3.985527E+05, 4.064433E+05, 4.144600E+05, 4.226042E+05, 4.308777E+05, 4.392819E+05, 
   4.478184E+05, 4.564888E+05, 4.652947E+05, 4.742376E+05, 4.833193E+05, 4.925412E+05, 
   5.019052E+05, 5.114128E+05, 5.210656E+05, 5.308655E+05, 5.408140E+05, 5.509129E+05, 
   5.611640E+05, 5.715689E+05, 5.821294E+05, 5.928473E+05, 6.037243E+05, 6.147623E+05, 
   6.259630E+05, 6.373283E+05, 6.488599E+05, 6.605598E+05, 6.724298E+05, 6.844718E+05, 
   6.966877E+05, 7.090793E+05, 7.216486E+05, 7.343975E+05, 7.473280E+05, 7.604419E+05, 
   7.737414E+05, 7.872284E+05, 8.009048E+05, 8.147728E+05, 8.288343E+05, 8.430913E+05, 
   8.575460E+05, 8.722004E+05, 8.870566E+05, 9.021168E+05, 9.173829E+05, 9.328572E+05, 
   9.485419E+05, 9.644390E+05, 9.805508E+05, 9.968794E+05, 1.013427E+06, 1.030196E+06, 
   1.047189E+06, 1.064407E+06, 1.081853E+06, 1.099530E+06, 1.117440E+06, 1.135584E+06, 
   1.153966E+06, 1.172587E+06, 1.191450E+06, 1.210558E+06, 1.229913E+06, 1.249516E+06, 
   1.269372E+06, 1.289482E+06, 1.309848E+06, 1.330473E+06, 1.351359E+06, 1.372510E+06, 
   1.393928E+06, 1.415614E+06, 1.437572E+06, 1.459804E+06, 1.482314E+06, 1.505102E+06, 
   1.528173E+06, 1.551529E+06, 1.575172E+06, 1.599104E+06, 1.623330E+06, 1.647851E+06, 
   1.672671E+06, 1.697791E+06, 1.723215E+06, 1.748946E+06, 1.774986E+06, 1.801338E+06, 
   1.828004E+06, 1.854989E+06, 1.882294E+06, 1.909923E+06, 1.937879E+06, 1.966163E+06, 
   1.994780E+06, 2.023732E+06, 2.053023E+06, 2.082655E+06, 2.112631E+06, 2.142954E+06, 
   2.173627E+06, 2.204654E+06, 2.236037E+06, 2.267780E+06, 2.299886E+06, 2.332357E+06, 
   2.365197E+06, 2.398410E+06, 2.431997E+06, 2.465964E+06, 2.500312E+06, 2.535045E+06, 
   2.570166E+06, 2.605679E+06, 2.641586E+06, 2.677892E+06, 2.714600E+06, 2.751712E+06, 
   2.789233E+06, 2.827165E+06, 2.865513E+06, 2.904279E+06, 2.943467E+06, 2.983080E+06, 
   3.023123E+06, 3.063598E+06, 3.104509E+06, 3.145860E+06, 3.187653E+06, 3.229894E+06, 
   3.272585E+06, 3.315730E+06, 3.359332E+06, 3.403396E+06, 3.447925E+06, 3.492923E+06, 
   3.538393E+06, 3.584339E+06, 3.630766E+06, 3.677676E+06, 3.725074E+06, 3.772963E+06, 
   3.821348E+06, 3.870232E+06, 3.919619E+06, 3.969513E+06, 4.019918E+06, 4.070838E+06, 
   4.122277E+06, 4.174238E+06, 4.226727E+06, 4.279747E+06, 4.333302E+06, 4.387396E+06, 
   4.442033E+06, 4.497218E+06, 4.552954E+06, 4.609246E+06, 4.666097E+06, 4.723513E+06, 
   4.781498E+06, 4.840054E+06, 4.899188E+06, 4.958903E+06, 5.019204E+06, 5.080094E+06, 
   5.141579E+06, 5.203662E+06, 5.266349E+06, 5.329643E+06, 5.393548E+06, 5.458071E+06, 
   5.523214E+06, 5.588983E+06, 5.655381E+06, 5.722415E+06, 5.790087E+06, 5.858403E+06, 
   5.927368E+06, 5.996986E+06, 6.067262E+06, 6.138200E+06, 6.209805E+06, 6.282082E+06, 
   6.355036E+06, 6.428672E+06, 6.502993E+06, 6.578006E+06, 6.653716E+06, 6.730126E+06, 
   6.807242E+06, 6.885069E+06, 6.963611E+06, 7.042875E+06, 7.122864E+06, 7.203584E+06, 
   7.285040E+06, 7.367236E+06, 7.450179E+06, 7.533873E+06, 7.618324E+06, 7.703536E+06, 
   7.789514E+06, 7.876265E+06, 7.963793E+06, 8.052103E+06, 8.141201E+06, 8.231092E+06, 
   8.321782E+06, 8.413275E+06, 8.505578E+06, 8.598695E+06, 8.692632E+06, 8.787395E+06, 
   8.882989E+06, 8.979420E+06, 9.076692E+06, 9.174812E+06, 9.273786E+06, 9.373618E+06, 
   9.474316E+06, 9.575883E+06, 9.678326E+06, 9.781651E+06, 9.885863E+06, 9.990968E+06, 
   1.009697E+07, 1.020388E+07, 1.031170E+07, 1.042044E+07, 1.053010E+07, 1.064068E+07, 
   1.075221E+07, 1.086467E+07, 1.097808E+07, 1.109244E+07, 1.120776E+07, 1.132404E+07, 
   1.144129E+07, 1.155952E+07, 1.167874E+07, 1.179894E+07, 1.192014E+07, 1.204234E+07, 
   1.216554E+07, 1.228977E+07, 1.241501E+07, 1.254128E+07, 1.266858E+07, 1.279692E+07, 
   1.292631E+07, 1.305675E+07, 1.318825E+07, 1.332081E+07, 1.345445E+07, 1.358916E+07, 
   1.372496E+07, 1.386186E+07, 1.399985E+07, 1.413894E+07, 1.427915E+07, 1.442048E+07, 
   1.456293E+07, 1.470651E+07, 1.485124E+07, 1.499711E+07, 1.514413E+07, 1.529231E+07, 
   1.544166E+07, 1.559218E+07, 1.574388E+07, 1.589677E+07, 1.605086E+07, 1.620614E+07, 
   1.636264E+07, 1.652035E+07, 1.667928E+07, 1.683944E+07, 1.700085E+07, 1.716349E+07, 
   1.732739E+07, 1.749255E+07, 1.765897E+07, 1.782667E+07, 1.799565E+07, 1.816592E+07, 
   1.833748E+07, 1.851035E+07, 1.868453E+07, 1.886003E+07, 1.903686E+07, 1.921502E+07, 
   1.939452E+07, 1.957537E+07, 1.975757E+07, 1.994115E+07, 2.012609E+07, 2.031242E+07, 
   2.050013E+07, 2.068924E+07, 2.087975E+07, 2.107167E+07, 2.126502E+07, 2.145979E+07, 
   2.165600E+07, 2.185365E+07, 2.205275E+07, 2.225331E+07, 2.245534E+07, 2.265885E+07, 
   2.286384E+07, 2.307032E+07, 2.327830E+07, 2.348780E+07, 2.369880E+07, 2.391134E+07, 
   2.412541E+07, 2.434102E+07, 2.455818E+07, 
])
# ============================ M = 4, I = 5 ============================
M = 4
I = 5
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.126775E+02, 9.412225E+02, 1.864335E+03, 2.787730E+03, 3.711353E+03, 4.635194E+03, 
   5.559257E+03, 6.483591E+03, 7.408383E+03, 8.334099E+03, 9.261600E+03, 1.019221E+04, 
   1.112770E+04, 1.207021E+04, 1.302215E+04, 1.398610E+04, 1.496471E+04, 1.596064E+04, 
   1.697650E+04, 1.801479E+04, 1.907796E+04, 2.016831E+04, 2.128805E+04, 2.243929E+04, 
   2.362406E+04, 2.484430E+04, 2.610187E+04, 2.739858E+04, 2.873621E+04, 3.011648E+04, 
   3.154108E+04, 3.301168E+04, 3.452994E+04, 3.609750E+04, 3.771599E+04, 3.938705E+04, 
   4.111232E+04, 4.289345E+04, 4.473208E+04, 4.662989E+04, 4.858855E+04, 5.060976E+04, 
   5.269524E+04, 5.484671E+04, 5.706596E+04, 5.935474E+04, 6.171486E+04, 6.414817E+04, 
   6.665651E+04, 6.924177E+04, 7.190586E+04, 7.465073E+04, 7.747834E+04, 8.039070E+04, 
   8.338985E+04, 8.647784E+04, 8.965677E+04, 9.292877E+04, 9.629601E+04, 9.976067E+04, 
   1.033250E+05, 1.069912E+05, 1.107617E+05, 1.146388E+05, 1.186247E+05, 1.227220E+05, 
   1.269330E+05, 1.312604E+05, 1.357064E+05, 1.402738E+05, 1.449651E+05, 1.497829E+05, 
   1.547299E+05, 1.598088E+05, 1.650224E+05, 1.703734E+05, 1.758646E+05, 1.814989E+05, 
   1.872792E+05, 1.932085E+05, 1.992897E+05, 2.055258E+05, 2.119200E+05, 2.184752E+05, 
   2.251947E+05, 2.320817E+05, 2.391393E+05, 2.463709E+05, 2.537797E+05, 2.613692E+05, 
   2.691427E+05, 2.771036E+05, 2.852555E+05, 2.936019E+05, 3.021463E+05, 3.108925E+05, 
   3.198440E+05, 3.290046E+05, 3.383780E+05, 3.479680E+05, 3.577786E+05, 3.678135E+05, 
   3.780768E+05, 3.885723E+05, 3.993043E+05, 4.102767E+05, 4.214937E+05, 4.329595E+05, 
   4.446783E+05, 4.566544E+05, 4.688921E+05, 4.813958E+05, 4.941699E+05, 5.072190E+05, 
   5.205475E+05, 5.341601E+05, 5.480613E+05, 5.622559E+05, 5.767486E+05, 5.915441E+05, 
   6.066474E+05, 6.220632E+05, 6.377967E+05, 6.538527E+05, 6.702363E+05, 6.869526E+05, 
   7.040069E+05, 7.214042E+05, 7.391499E+05, 7.572493E+05, 7.757077E+05, 7.945307E+05, 
   8.137237E+05, 8.332922E+05, 8.532420E+05, 8.735785E+05, 8.943075E+05, 9.154349E+05, 
   9.369665E+05, 9.589080E+05, 9.812656E+05, 1.004045E+06, 1.027253E+06, 1.050895E+06, 
   1.074977E+06, 1.099506E+06, 1.124488E+06, 1.149929E+06, 1.175836E+06, 1.202215E+06, 
   1.229072E+06, 1.256416E+06, 1.284251E+06, 1.312585E+06, 1.341425E+06, 1.370777E+06, 
   1.400649E+06, 1.431047E+06, 1.461978E+06, 1.493450E+06, 1.525470E+06, 1.558044E+06, 
   1.591181E+06, 1.624888E+06, 1.659171E+06, 1.694040E+06, 1.729500E+06, 1.765560E+06, 
   1.802228E+06, 1.839511E+06, 1.877418E+06, 1.915955E+06, 1.955131E+06, 1.994955E+06, 
   2.035434E+06, 2.076576E+06, 2.118389E+06, 2.160883E+06, 2.204065E+06, 2.247944E+06, 
   2.292528E+06, 2.337827E+06, 2.383848E+06, 2.430600E+06, 2.478092E+06, 2.526334E+06, 
   2.575334E+06, 2.625101E+06, 2.675644E+06, 2.726972E+06, 2.779096E+06, 2.832023E+06, 
   2.885763E+06, 2.940327E+06, 2.995723E+06, 3.051961E+06, 3.109051E+06, 3.167002E+06, 
   3.225825E+06, 3.285529E+06, 3.346125E+06, 3.407622E+06, 3.470030E+06, 3.533361E+06, 
   3.597624E+06, 3.662829E+06, 3.728988E+06, 3.796110E+06, 3.864207E+06, 3.933288E+06, 
   4.003366E+06, 4.074450E+06, 4.146552E+06, 4.219682E+06, 4.293852E+06, 4.369074E+06, 
   4.445357E+06, 4.522714E+06, 4.601157E+06, 4.680696E+06, 4.761344E+06, 4.843111E+06, 
   4.926010E+06, 5.010053E+06, 5.095252E+06, 5.181618E+06, 5.269164E+06, 5.357902E+06, 
   5.447844E+06, 5.539003E+06, 5.631391E+06, 5.725020E+06, 5.819904E+06, 5.916056E+06, 
   6.013486E+06, 6.112210E+06, 6.212240E+06, 6.313588E+06, 6.416268E+06, 6.520294E+06, 
   6.625678E+06, 6.732434E+06, 6.840576E+06, 6.950117E+06, 7.061071E+06, 7.173452E+06, 
   7.287273E+06, 7.402549E+06, 7.519294E+06, 7.637522E+06, 7.757246E+06, 7.878482E+06, 
   8.001244E+06, 8.125547E+06, 8.251405E+06, 8.378832E+06, 8.507845E+06, 8.638456E+06, 
   8.770683E+06, 8.904539E+06, 9.040040E+06, 9.177202E+06, 9.316039E+06, 9.456567E+06, 
   9.598802E+06, 9.742760E+06, 9.888455E+06, 1.003590E+07, 1.018512E+07, 1.033613E+07, 
   1.048894E+07, 1.064357E+07, 1.080003E+07, 1.095834E+07, 1.111853E+07, 1.128059E+07, 
   1.144456E+07, 1.161045E+07, 1.177827E+07, 1.194805E+07, 1.211979E+07, 1.229352E+07, 
   1.246926E+07, 1.264702E+07, 1.282682E+07, 1.300868E+07, 1.319261E+07, 1.337863E+07, 
   1.356677E+07, 1.375703E+07, 1.394944E+07, 1.414402E+07, 1.434079E+07, 1.453975E+07, 
   1.474094E+07, 1.494437E+07, 1.515006E+07, 1.535802E+07, 1.556828E+07, 1.578086E+07, 
   1.599578E+07, 1.621305E+07, 1.643270E+07, 1.665474E+07, 1.687919E+07, 1.710608E+07, 
   1.733542E+07, 1.756724E+07, 1.780156E+07, 1.803838E+07, 1.827774E+07, 1.851966E+07, 
   1.876416E+07, 1.901125E+07, 1.926096E+07, 1.951331E+07, 1.976832E+07, 2.002601E+07, 
   2.028640E+07, 2.054952E+07, 2.081539E+07, 2.108402E+07, 2.135544E+07, 2.162967E+07, 
   2.190674E+07, 2.218666E+07, 2.246946E+07, 2.275516E+07, 2.304378E+07, 2.333535E+07, 
   2.362989E+07, 2.392741E+07, 2.422796E+07, 2.453153E+07, 2.483817E+07, 2.514790E+07, 
   2.546073E+07, 2.577669E+07, 2.609580E+07, 2.641810E+07, 2.674359E+07, 2.707232E+07, 
   2.740429E+07, 2.773954E+07, 2.807809E+07, 2.841996E+07, 2.876518E+07, 2.911378E+07, 
   2.946577E+07, 2.982119E+07, 3.018006E+07, 3.054240E+07, 3.090824E+07, 3.127760E+07, 
   3.165052E+07, 3.202702E+07, 3.240711E+07, 3.279084E+07, 3.317822E+07, 3.356928E+07, 
   3.396406E+07, 3.436256E+07, 3.476483E+07, 3.517089E+07, 3.558076E+07, 3.599447E+07, 
   3.641206E+07, 3.683354E+07, 3.725895E+07, 3.768831E+07, 3.812165E+07, 3.855900E+07, 
   3.900038E+07, 3.944583E+07, 3.989537E+07, 4.034904E+07, 4.080685E+07, 4.126884E+07, 
   4.173504E+07, 4.220548E+07, 4.268019E+07, 4.315918E+07, 4.364251E+07, 4.413019E+07, 
   4.462225E+07, 4.511872E+07, 4.561964E+07, 4.612504E+07, 4.663494E+07, 4.714937E+07, 
   4.766837E+07, 4.819196E+07, 4.872018E+07, 4.925306E+07, 4.979063E+07, 5.033292E+07, 
   5.087995E+07, 5.143178E+07, 5.198841E+07, 5.254990E+07, 5.311626E+07, 5.368753E+07, 
   5.426375E+07, 5.484494E+07, 5.543114E+07, 5.602237E+07, 5.661869E+07, 5.722011E+07, 
   5.782666E+07, 5.843840E+07, 5.905533E+07, 5.967751E+07, 6.030496E+07, 6.093772E+07, 
   6.157582E+07, 6.221930E+07, 6.286818E+07, 6.352251E+07, 6.418232E+07, 6.484764E+07, 
   6.551851E+07, 6.619497E+07, 6.687704E+07, 6.756477E+07, 6.825819E+07, 6.895733E+07, 
   6.966224E+07, 7.037294E+07, 7.108947E+07, 7.181188E+07, 7.254019E+07, 7.327444E+07, 
   7.401468E+07, 7.476093E+07, 7.551324E+07, 7.627163E+07, 7.703615E+07, 7.780684E+07, 
   7.858374E+07, 7.936687E+07, 8.015628E+07, 8.095201E+07, 8.175410E+07, 8.256258E+07, 
   8.337749E+07, 8.419888E+07, 8.502677E+07, 8.586122E+07, 8.670225E+07, 8.754991E+07, 
   8.840424E+07, 8.926528E+07, 9.013307E+07, 9.100764E+07, 9.188905E+07, 9.277732E+07, 
   9.367250E+07, 9.457463E+07, 9.548375E+07, 9.639990E+07, 9.732313E+07, 9.825347E+07, 
   9.919096E+07, 1.001357E+08, 1.010876E+08, 1.020468E+08, 1.030134E+08, 1.039872E+08, 
   1.049686E+08, 1.059573E+08, 1.069536E+08, 1.079574E+08, 1.089688E+08, 1.099878E+08, 
   1.110144E+08, 1.120488E+08, 1.130910E+08, 1.141409E+08, 1.151987E+08, 1.162644E+08, 
   1.173380E+08, 1.184196E+08, 1.195092E+08, 1.206068E+08, 1.217126E+08, 1.228266E+08, 
   1.239487E+08, 1.250791E+08, 1.262178E+08, 1.273649E+08, 1.285203E+08, 1.296842E+08, 
   1.308565E+08, 1.320374E+08, 1.332268E+08, 1.344249E+08, 1.356317E+08, 1.368471E+08, 
   1.380714E+08, 1.393044E+08, 1.405463E+08, 
])
# ============================ M = 5, I = 1 ============================
M = 5
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.011872E+00, 3.968093E+00, 7.573516E+00, 1.118591E+01, 1.480009E+01, 1.841509E+01, 
   2.203056E+01, 2.564639E+01, 2.926249E+01, 3.287884E+01, 3.649540E+01, 4.011217E+01, 
   4.372913E+01, 4.734629E+01, 5.096362E+01, 5.458114E+01, 5.819883E+01, 6.181670E+01, 
   6.543475E+01, 6.905297E+01, 7.267137E+01, 7.628995E+01, 7.990872E+01, 8.352769E+01, 
   8.714687E+01, 9.076628E+01, 9.438597E+01, 9.800596E+01, 1.016263E+02, 1.052471E+02, 
   1.088684E+02, 1.124903E+02, 1.161130E+02, 1.197365E+02, 1.233610E+02, 1.269867E+02, 
   1.306138E+02, 1.342425E+02, 1.378730E+02, 1.415056E+02, 1.451405E+02, 1.487781E+02, 
   1.524185E+02, 1.560621E+02, 1.597093E+02, 1.633604E+02, 1.670157E+02, 1.706756E+02, 
   1.743404E+02, 1.780105E+02, 1.816863E+02, 1.853682E+02, 1.890564E+02, 1.927516E+02, 
   1.964539E+02, 2.001638E+02, 2.038817E+02, 2.076079E+02, 2.113428E+02, 2.150869E+02, 
   2.188405E+02, 2.226038E+02, 2.263775E+02, 2.301616E+02, 2.339567E+02, 2.377631E+02, 
   2.415811E+02, 2.454110E+02, 2.492532E+02, 2.531080E+02, 2.569757E+02, 2.608566E+02, 
   2.647511E+02, 2.686593E+02, 2.725817E+02, 2.765184E+02, 2.804698E+02, 2.844361E+02, 
   2.884176E+02, 2.924145E+02, 2.964272E+02, 3.004557E+02, 3.045004E+02, 3.085614E+02, 
   3.126391E+02, 3.167335E+02, 3.208450E+02, 3.249737E+02, 3.291199E+02, 3.332836E+02, 
   3.374652E+02, 3.416647E+02, 3.458824E+02, 3.501184E+02, 3.543729E+02, 3.586461E+02, 
   3.629381E+02, 3.672490E+02, 3.715790E+02, 3.759283E+02, 3.802970E+02, 3.846852E+02, 
   3.890930E+02, 3.935206E+02, 3.979682E+02, 4.024357E+02, 4.069234E+02, 4.114314E+02, 
   4.159597E+02, 4.205085E+02, 4.250779E+02, 4.296679E+02, 4.342787E+02, 4.389104E+02, 
   4.435631E+02, 4.482368E+02, 4.529316E+02, 4.576477E+02, 4.623850E+02, 4.671438E+02, 
   4.719240E+02, 4.767258E+02, 4.815492E+02, 4.863942E+02, 4.912611E+02, 4.961497E+02, 
   5.010603E+02, 5.059928E+02, 5.109473E+02, 5.159240E+02, 5.209227E+02, 5.259437E+02, 
   5.309869E+02, 5.360525E+02, 5.411404E+02, 5.462507E+02, 5.513836E+02, 5.565389E+02, 
   5.617168E+02, 5.669173E+02, 5.721405E+02, 5.773865E+02, 5.826551E+02, 5.879466E+02, 
   5.932610E+02, 5.985982E+02, 6.039583E+02, 6.093414E+02, 6.147475E+02, 6.201767E+02, 
   6.256289E+02, 6.311043E+02, 6.366028E+02, 6.421245E+02, 6.476694E+02, 6.532376E+02, 
   6.588291E+02, 6.644439E+02, 6.700820E+02, 6.757436E+02, 6.814285E+02, 6.871369E+02, 
   6.928687E+02, 6.986241E+02, 7.044030E+02, 7.102054E+02, 7.160315E+02, 7.218811E+02, 
   7.277544E+02, 7.336513E+02, 7.395719E+02, 7.455162E+02, 7.514843E+02, 7.574761E+02, 
   7.634917E+02, 7.695311E+02, 7.755944E+02, 7.816815E+02, 7.877924E+02, 7.939273E+02, 
   8.000860E+02, 8.062687E+02, 8.124754E+02, 8.187060E+02, 8.249606E+02, 8.312392E+02, 
   8.375419E+02, 8.438686E+02, 8.502194E+02, 8.565943E+02, 8.629933E+02, 8.694164E+02, 
   8.758637E+02, 8.823351E+02, 8.888307E+02, 8.953505E+02, 9.018945E+02, 9.084628E+02, 
   9.150553E+02, 9.216720E+02, 9.283131E+02, 9.349784E+02, 9.416680E+02, 9.483820E+02, 
   9.551203E+02, 9.618830E+02, 9.686700E+02, 9.754814E+02, 9.823173E+02, 9.891775E+02, 
   9.960622E+02, 1.002971E+03, 1.009905E+03, 1.016863E+03, 1.023845E+03, 1.030852E+03, 
   1.037884E+03, 1.044940E+03, 1.052021E+03, 1.059126E+03, 1.066256E+03, 1.073410E+03, 
   1.080589E+03, 1.087792E+03, 1.095020E+03, 1.102273E+03, 1.109551E+03, 1.116853E+03, 
   1.124179E+03, 1.131531E+03, 1.138907E+03, 1.146308E+03, 1.153733E+03, 1.161183E+03, 
   1.168658E+03, 1.176158E+03, 1.183682E+03, 1.191232E+03, 1.198806E+03, 1.206405E+03, 
   1.214028E+03, 1.221677E+03, 1.229350E+03, 1.237048E+03, 1.244771E+03, 1.252519E+03, 
   1.260292E+03, 1.268089E+03, 1.275912E+03, 1.283759E+03, 1.291632E+03, 1.299529E+03, 
   1.307451E+03, 1.315398E+03, 1.323370E+03, 1.331368E+03, 1.339390E+03, 1.347437E+03, 
   1.355509E+03, 1.363606E+03, 1.371728E+03, 1.379875E+03, 1.388047E+03, 1.396245E+03, 
   1.404467E+03, 1.412714E+03, 1.420987E+03, 1.429284E+03, 1.437607E+03, 1.445955E+03, 
   1.454327E+03, 1.462726E+03, 1.471149E+03, 1.479597E+03, 1.488070E+03, 1.496569E+03, 
   1.505093E+03, 1.513642E+03, 1.522216E+03, 1.530816E+03, 1.539440E+03, 1.548090E+03, 
   1.556765E+03, 1.565466E+03, 1.574191E+03, 1.582942E+03, 1.591718E+03, 1.600520E+03, 
   1.609346E+03, 1.618199E+03, 1.627076E+03, 1.635979E+03, 1.644907E+03, 1.653860E+03, 
   1.662839E+03, 1.671843E+03, 1.680872E+03, 1.689927E+03, 1.699007E+03, 1.708113E+03, 
   1.717244E+03, 1.726400E+03, 1.735582E+03, 1.744790E+03, 1.754022E+03, 1.763280E+03, 
   1.772564E+03, 1.781873E+03, 1.791208E+03, 1.800568E+03, 1.809954E+03, 1.819365E+03, 
   1.828801E+03, 1.838263E+03, 1.847751E+03, 1.857264E+03, 1.866803E+03, 1.876367E+03, 
   1.885957E+03, 1.895573E+03, 1.905214E+03, 1.914880E+03, 1.924573E+03, 1.934291E+03, 
   1.944034E+03, 1.953803E+03, 1.963598E+03, 1.973418E+03, 1.983264E+03, 1.993136E+03, 
   2.003034E+03, 2.012957E+03, 2.022906E+03, 2.032880E+03, 2.042880E+03, 2.052906E+03, 
   2.062958E+03, 2.073035E+03, 2.083138E+03, 2.093267E+03, 2.103422E+03, 2.113602E+03, 
   2.123809E+03, 2.134041E+03, 2.144299E+03, 2.154582E+03, 2.164892E+03, 2.175227E+03, 
   2.185588E+03, 2.195975E+03, 2.206388E+03, 2.216826E+03, 2.227291E+03, 2.237781E+03, 
   2.248298E+03, 2.258840E+03, 2.269408E+03, 2.280002E+03, 2.290622E+03, 2.301268E+03, 
   2.311939E+03, 2.322637E+03, 2.333361E+03, 2.344111E+03, 2.354886E+03, 2.365688E+03, 
   2.376515E+03, 2.387369E+03, 2.398249E+03, 2.409154E+03, 2.420086E+03, 2.431044E+03, 
   2.442027E+03, 2.453037E+03, 2.464073E+03, 2.475135E+03, 2.486223E+03, 2.497337E+03, 
   2.508477E+03, 2.519644E+03, 2.530836E+03, 2.542055E+03, 2.553299E+03, 2.564570E+03, 
   2.575867E+03, 2.587190E+03, 2.598539E+03, 2.609915E+03, 2.621316E+03, 2.632744E+03, 
   2.644198E+03, 2.655678E+03, 2.667185E+03, 2.678718E+03, 2.690276E+03, 2.701862E+03, 
   2.713473E+03, 2.725111E+03, 2.736775E+03, 2.748465E+03, 2.760181E+03, 2.771924E+03, 
   2.783693E+03, 2.795489E+03, 2.807310E+03, 2.819158E+03, 2.831033E+03, 2.842934E+03, 
   2.854861E+03, 2.866814E+03, 2.878794E+03, 2.890800E+03, 2.902833E+03, 2.914892E+03, 
   2.926977E+03, 2.939089E+03, 2.951227E+03, 2.963392E+03, 2.975583E+03, 2.987801E+03, 
   3.000045E+03, 3.012315E+03, 3.024612E+03, 3.036936E+03, 3.049286E+03, 3.061662E+03, 
   3.074065E+03, 3.086495E+03, 3.098951E+03, 3.111434E+03, 3.123943E+03, 3.136478E+03, 
   3.149041E+03, 3.161629E+03, 3.174245E+03, 3.186887E+03, 3.199555E+03, 3.212251E+03, 
   3.224973E+03, 3.237721E+03, 3.250496E+03, 3.263298E+03, 3.276126E+03, 3.288981E+03, 
   3.301863E+03, 3.314771E+03, 3.327706E+03, 3.340668E+03, 3.353657E+03, 3.366672E+03, 
   3.379714E+03, 3.392783E+03, 3.405878E+03, 3.419000E+03, 3.432149E+03, 3.445325E+03, 
   3.458527E+03, 3.471756E+03, 3.485012E+03, 3.498295E+03, 3.511605E+03, 3.524941E+03, 
   3.538304E+03, 3.551694E+03, 3.565111E+03, 3.578555E+03, 3.592026E+03, 3.605523E+03, 
   3.619048E+03, 3.632599E+03, 3.646177E+03, 3.659782E+03, 3.673414E+03, 3.687073E+03, 
   3.700759E+03, 3.714472E+03, 3.728212E+03, 3.741978E+03, 3.755772E+03, 3.769593E+03, 
   3.783440E+03, 3.797315E+03, 3.811216E+03, 3.825145E+03, 3.839101E+03, 3.853083E+03, 
   3.867093E+03, 3.881130E+03, 3.895193E+03, 3.909284E+03, 3.923402E+03, 3.937547E+03, 
   3.951719E+03, 3.965918E+03, 3.980145E+03, 3.994398E+03, 4.008679E+03, 4.022986E+03, 
   4.037321E+03, 4.051683E+03, 4.066072E+03, 4.080488E+03, 4.094932E+03, 4.109402E+03, 
   4.123900E+03, 4.138425E+03, 4.152977E+03, 4.167557E+03, 4.182163E+03, 4.196797E+03, 
   4.211458E+03, 4.226146E+03, 4.240862E+03, 4.255605E+03, 4.270375E+03, 4.285173E+03, 
   4.299997E+03, 4.314849E+03, 4.329729E+03, 4.344635E+03, 4.359569E+03, 4.374531E+03, 
   4.389519E+03, 4.404535E+03, 4.419579E+03, 4.434649E+03, 4.449747E+03, 4.464873E+03, 
   4.480026E+03, 4.495206E+03, 4.510414E+03, 4.525649E+03, 4.540912E+03, 4.556202E+03, 
   4.571519E+03, 4.586864E+03, 4.602236E+03, 4.617636E+03, 4.633064E+03, 4.648518E+03, 
   4.664001E+03, 4.679511E+03, 4.695048E+03, 4.710613E+03, 4.726205E+03, 4.741825E+03, 
   4.757472E+03, 4.773148E+03, 4.788850E+03, 4.804580E+03, 4.820338E+03, 4.836123E+03, 
   4.851936E+03, 4.867777E+03, 4.883645E+03, 4.899541E+03, 4.915464E+03, 4.931415E+03, 
   4.947394E+03, 4.963400E+03, 4.979434E+03, 4.995496E+03, 5.011585E+03, 5.027702E+03, 
   5.043847E+03, 5.060020E+03, 5.076220E+03, 5.092448E+03, 5.108703E+03, 5.124987E+03, 
   5.141298E+03, 5.157637E+03, 5.174003E+03, 5.190398E+03, 5.206820E+03, 5.223270E+03, 
   5.239748E+03, 5.256253E+03, 5.272787E+03, 5.289348E+03, 5.305937E+03, 5.322554E+03, 
   5.339199E+03, 5.355871E+03, 5.372572E+03, 5.389300E+03, 5.406056E+03, 5.422840E+03, 
   5.439652E+03, 5.456492E+03, 5.473360E+03, 5.490255E+03, 5.507179E+03, 5.524131E+03, 
   5.541110E+03, 5.558118E+03, 5.575153E+03, 5.592216E+03, 5.609308E+03, 5.626427E+03, 
   5.643574E+03, 5.660750E+03, 5.677953E+03, 5.695184E+03, 5.712444E+03, 5.729731E+03, 
   5.747047E+03, 5.764390E+03, 5.781762E+03, 5.799161E+03, 5.816589E+03, 5.834044E+03, 
   5.851528E+03, 5.869040E+03, 5.886580E+03, 5.904148E+03, 5.921745E+03, 5.939369E+03, 
   5.957021E+03, 5.974702E+03, 5.992411E+03, 6.010148E+03, 6.027913E+03, 6.045706E+03, 
   6.063527E+03, 6.081377E+03, 6.099255E+03, 6.117161E+03, 6.135095E+03, 6.153057E+03, 
   6.171048E+03, 6.189067E+03, 6.207114E+03, 6.225189E+03, 6.243293E+03, 6.261425E+03, 
   6.279585E+03, 6.297774E+03, 6.315990E+03, 6.334235E+03, 6.352508E+03, 6.370810E+03, 
   6.389140E+03, 6.407498E+03, 6.425885E+03, 6.444300E+03, 6.462743E+03, 6.481214E+03, 
   6.499714E+03, 6.518243E+03, 6.536799E+03, 6.555384E+03, 6.573998E+03, 6.592639E+03, 
   6.611310E+03, 6.630008E+03, 6.648735E+03, 6.667491E+03, 6.686275E+03, 6.705087E+03, 
   6.723928E+03, 6.742797E+03, 6.761694E+03, 6.780621E+03, 6.799575E+03, 6.818558E+03, 
   6.837570E+03, 6.856610E+03, 6.875678E+03, 6.894775E+03, 6.913901E+03, 6.933055E+03, 
   6.952238E+03, 6.971449E+03, 6.990688E+03, 7.009957E+03, 7.029253E+03, 7.048579E+03, 
   7.067933E+03, 7.087315E+03, 7.106726E+03, 7.126166E+03, 7.145634E+03, 7.165131E+03, 
   7.184656E+03, 7.204210E+03, 7.223793E+03, 7.243404E+03, 7.263044E+03, 7.282713E+03, 
   7.302410E+03, 7.322136E+03, 7.341891E+03, 7.361674E+03, 7.381486E+03, 7.401326E+03, 
   7.421195E+03, 7.441093E+03, 7.461020E+03, 7.480975E+03, 7.500959E+03, 7.520972E+03, 
   7.541014E+03, 7.561084E+03, 7.581183E+03, 7.601311E+03, 7.621467E+03, 7.641652E+03, 
   7.661866E+03, 7.682109E+03, 7.702380E+03, 7.722681E+03, 7.743010E+03, 7.763368E+03, 
   7.783754E+03, 7.804170E+03, 7.824614E+03, 7.845087E+03, 7.865589E+03, 7.886120E+03, 
   7.906680E+03, 7.927268E+03, 7.947885E+03, 7.968531E+03, 7.989206E+03, 8.009910E+03, 
   8.030643E+03, 8.051405E+03, 8.072195E+03, 8.093015E+03, 8.113863E+03, 8.134740E+03, 
   8.155646E+03, 8.176581E+03, 8.197545E+03, 8.218538E+03, 8.239560E+03, 8.260610E+03, 
   8.281690E+03, 8.302798E+03, 8.323936E+03, 8.345102E+03, 8.366298E+03, 8.387522E+03, 
   8.408776E+03, 8.430058E+03, 8.451369E+03, 8.472710E+03, 8.494079E+03, 8.515477E+03, 
   8.536905E+03, 8.558361E+03, 8.579847E+03, 8.601361E+03, 8.622904E+03, 8.644477E+03, 
   8.666078E+03, 8.687709E+03, 8.709368E+03, 8.731057E+03, 8.752775E+03, 8.774521E+03, 
   8.796297E+03, 8.818102E+03, 8.839936E+03, 8.861799E+03, 8.883691E+03, 8.905612E+03, 
   8.927563E+03, 8.949542E+03, 8.971551E+03, 8.993588E+03, 9.015655E+03, 9.037751E+03, 
   9.059876E+03, 9.082030E+03, 9.104213E+03, 9.126426E+03, 9.148667E+03, 9.170938E+03, 
   9.193238E+03, 9.215567E+03, 9.237925E+03, 9.260313E+03, 9.282729E+03, 9.305175E+03, 
   9.327650E+03, 9.350154E+03, 9.372687E+03, 9.395250E+03, 9.417841E+03, 9.440462E+03, 
   9.463112E+03, 9.485792E+03, 9.508500E+03, 9.531238E+03, 9.554005E+03, 9.576801E+03, 
   9.599626E+03, 9.622481E+03, 9.645365E+03, 9.668278E+03, 9.691221E+03, 9.714192E+03, 
   9.737193E+03, 9.760223E+03, 9.783283E+03, 9.806371E+03, 9.829489E+03, 9.852637E+03, 
   9.875813E+03, 9.899019E+03, 9.922254E+03, 9.945519E+03, 9.968812E+03, 9.992136E+03, 
   1.001549E+04, 1.003887E+04, 1.006228E+04, 1.008572E+04, 1.010919E+04, 1.013269E+04, 
   1.015622E+04, 1.017978E+04, 1.020336E+04, 1.022698E+04, 1.025062E+04, 1.027430E+04, 
   1.029800E+04, 1.032174E+04, 1.034550E+04, 1.036929E+04, 1.039311E+04, 1.041697E+04, 
   1.044085E+04, 1.046476E+04, 1.048869E+04, 1.051266E+04, 1.053666E+04, 1.056069E+04, 
   1.058475E+04, 1.060883E+04, 1.063295E+04, 1.065709E+04, 1.068127E+04, 1.070547E+04, 
   1.072970E+04, 1.075397E+04, 1.077826E+04, 1.080258E+04, 1.082693E+04, 1.085131E+04, 
   1.087572E+04, 1.090016E+04, 1.092463E+04, 1.094913E+04, 1.097365E+04, 1.099821E+04, 
   1.102280E+04, 1.104741E+04, 1.107206E+04, 1.109673E+04, 1.112144E+04, 1.114617E+04, 
   1.117093E+04, 1.119572E+04, 1.122055E+04, 1.124540E+04, 1.127028E+04, 1.129519E+04, 
   1.132013E+04, 1.134510E+04, 1.137010E+04, 1.139512E+04, 1.142018E+04, 1.144527E+04, 
   1.147039E+04, 1.149553E+04, 1.152071E+04, 1.154591E+04, 1.157115E+04, 1.159641E+04, 
   1.162170E+04, 1.164703E+04, 1.167238E+04, 1.169776E+04, 1.172317E+04, 1.174862E+04, 
   1.177409E+04, 1.179959E+04, 1.182512E+04, 1.185067E+04, 1.187626E+04, 1.190188E+04, 
   1.192753E+04, 1.195321E+04, 1.197891E+04, 1.200465E+04, 1.203042E+04, 1.205621E+04, 
   1.208204E+04, 
])
# ============================ M = 5, I = 2 ============================
M = 5
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.030286E+00, 8.267022E+00, 1.581148E+01, 2.336923E+01, 3.093044E+01, 3.849322E+01, 
   4.605695E+01, 5.362138E+01, 6.118637E+01, 6.875185E+01, 7.631778E+01, 8.388413E+01, 
   9.145088E+01, 9.901802E+01, 1.065855E+02, 1.141535E+02, 1.217217E+02, 1.292904E+02, 
   1.368594E+02, 1.444287E+02, 1.519985E+02, 1.595686E+02, 1.671392E+02, 1.747101E+02, 
   1.822815E+02, 1.898535E+02, 1.974261E+02, 2.049994E+02, 2.125736E+02, 2.201487E+02, 
   2.277251E+02, 2.353030E+02, 2.428826E+02, 2.504643E+02, 2.580484E+02, 2.656354E+02, 
   2.732257E+02, 2.808197E+02, 2.884181E+02, 2.960214E+02, 3.036302E+02, 3.112452E+02, 
   3.188669E+02, 3.264962E+02, 3.341338E+02, 3.417803E+02, 3.494367E+02, 3.571036E+02, 
   3.647819E+02, 3.724725E+02, 3.801760E+02, 3.878934E+02, 3.956256E+02, 4.033732E+02, 
   4.111372E+02, 4.189185E+02, 4.267178E+02, 4.345360E+02, 4.423738E+02, 4.502322E+02, 
   4.581119E+02, 4.660137E+02, 4.739383E+02, 4.818866E+02, 4.898593E+02, 4.978572E+02, 
   5.058809E+02, 5.139312E+02, 5.220087E+02, 5.301142E+02, 5.382484E+02, 5.464117E+02, 
   5.546050E+02, 5.628288E+02, 5.710838E+02, 5.793704E+02, 5.876893E+02, 5.960411E+02, 
   6.044262E+02, 6.128452E+02, 6.212987E+02, 6.297871E+02, 6.383109E+02, 6.468705E+02, 
   6.554665E+02, 6.640993E+02, 6.727693E+02, 6.814769E+02, 6.902226E+02, 6.990066E+02, 
   7.078295E+02, 7.166916E+02, 7.255932E+02, 7.345348E+02, 7.435165E+02, 7.525388E+02, 
   7.616020E+02, 7.707065E+02, 7.798524E+02, 7.890401E+02, 7.982699E+02, 8.075420E+02, 
   8.168568E+02, 8.262144E+02, 8.356152E+02, 8.450593E+02, 8.545471E+02, 8.640787E+02, 
   8.736544E+02, 8.832743E+02, 8.929388E+02, 9.026480E+02, 9.124020E+02, 9.222012E+02, 
   9.320457E+02, 9.419356E+02, 9.518712E+02, 9.618526E+02, 9.718799E+02, 9.819535E+02, 
   9.920733E+02, 1.002240E+03, 1.012453E+03, 1.022712E+03, 1.033019E+03, 1.043372E+03, 
   1.053773E+03, 1.064221E+03, 1.074717E+03, 1.085260E+03, 1.095851E+03, 1.106490E+03, 
   1.117176E+03, 1.127911E+03, 1.138693E+03, 1.149524E+03, 1.160404E+03, 1.171331E+03, 
   1.182308E+03, 1.193333E+03, 1.204406E+03, 1.215529E+03, 1.226700E+03, 1.237921E+03, 
   1.249191E+03, 1.260509E+03, 1.271877E+03, 1.283295E+03, 1.294762E+03, 1.306278E+03, 
   1.317844E+03, 1.329460E+03, 1.341125E+03, 1.352840E+03, 1.364605E+03, 1.376420E+03, 
   1.388285E+03, 1.400200E+03, 1.412165E+03, 1.424180E+03, 1.436246E+03, 1.448362E+03, 
   1.460528E+03, 1.472745E+03, 1.485012E+03, 1.497330E+03, 1.509698E+03, 1.522117E+03, 
   1.534586E+03, 1.547107E+03, 1.559678E+03, 1.572300E+03, 1.584973E+03, 1.597697E+03, 
   1.610472E+03, 1.623298E+03, 1.636175E+03, 1.649103E+03, 1.662083E+03, 1.675114E+03, 
   1.688195E+03, 1.701329E+03, 1.714513E+03, 1.727749E+03, 1.741037E+03, 1.754376E+03, 
   1.767766E+03, 1.781208E+03, 1.794701E+03, 1.808247E+03, 1.821843E+03, 1.835492E+03, 
   1.849192E+03, 1.862944E+03, 1.876748E+03, 1.890604E+03, 1.904512E+03, 1.918471E+03, 
   1.932483E+03, 1.946546E+03, 1.960662E+03, 1.974829E+03, 1.989049E+03, 2.003321E+03, 
   2.017644E+03, 2.032021E+03, 2.046449E+03, 2.060929E+03, 2.075462E+03, 2.090047E+03, 
   2.104685E+03, 2.119374E+03, 2.134116E+03, 2.148911E+03, 2.163758E+03, 2.178657E+03, 
   2.193609E+03, 2.208614E+03, 2.223671E+03, 2.238781E+03, 2.253943E+03, 2.269158E+03, 
   2.284425E+03, 2.299746E+03, 2.315119E+03, 2.330544E+03, 2.346023E+03, 2.361554E+03, 
   2.377138E+03, 2.392775E+03, 2.408465E+03, 2.424207E+03, 2.440003E+03, 2.455852E+03, 
   2.471753E+03, 2.487708E+03, 2.503715E+03, 2.519776E+03, 2.535889E+03, 2.552056E+03, 
   2.568276E+03, 2.584549E+03, 2.600875E+03, 2.617254E+03, 2.633687E+03, 2.650172E+03, 
   2.666711E+03, 2.683304E+03, 2.699949E+03, 2.716648E+03, 2.733400E+03, 2.750206E+03, 
   2.767065E+03, 2.783977E+03, 2.800943E+03, 2.817962E+03, 2.835035E+03, 2.852161E+03, 
   2.869341E+03, 2.886574E+03, 2.903861E+03, 2.921201E+03, 2.938595E+03, 2.956043E+03, 
   2.973544E+03, 2.991099E+03, 3.008708E+03, 3.026370E+03, 3.044086E+03, 3.061856E+03, 
   3.079680E+03, 3.097557E+03, 3.115488E+03, 3.133473E+03, 3.151512E+03, 3.169605E+03, 
   3.187752E+03, 3.205952E+03, 3.224207E+03, 3.242515E+03, 3.260878E+03, 3.279294E+03, 
   3.297764E+03, 3.316289E+03, 3.334867E+03, 3.353500E+03, 3.372187E+03, 3.390927E+03, 
   3.409722E+03, 3.428572E+03, 3.447475E+03, 3.466432E+03, 3.485444E+03, 3.504510E+03, 
   3.523630E+03, 3.542804E+03, 3.562033E+03, 3.581316E+03, 3.600653E+03, 3.620045E+03, 
   3.639491E+03, 3.658992E+03, 3.678547E+03, 3.698156E+03, 3.717820E+03, 3.737538E+03, 
   3.757310E+03, 3.777138E+03, 3.797019E+03, 3.816956E+03, 3.836946E+03, 3.856992E+03, 
   3.877092E+03, 3.897246E+03, 3.917456E+03, 3.937720E+03, 3.958038E+03, 3.978411E+03, 
   3.998839E+03, 4.019322E+03, 4.039860E+03, 4.060452E+03, 4.081099E+03, 4.101801E+03, 
   4.122557E+03, 4.143369E+03, 4.164235E+03, 4.185156E+03, 4.206133E+03, 4.227164E+03, 
   4.248250E+03, 4.269391E+03, 4.290587E+03, 4.311838E+03, 4.333143E+03, 4.354504E+03, 
   4.375921E+03, 4.397392E+03, 4.418918E+03, 4.440499E+03, 4.462136E+03, 4.483827E+03, 
   4.505574E+03, 4.527376E+03, 4.549233E+03, 4.571145E+03, 4.593113E+03, 4.615136E+03, 
   4.637214E+03, 4.659347E+03, 4.681536E+03, 4.703780E+03, 4.726079E+03, 4.748434E+03, 
   4.770844E+03, 4.793310E+03, 4.815831E+03, 4.838407E+03, 4.861039E+03, 4.883726E+03, 
   4.906469E+03, 4.929268E+03, 4.952121E+03, 4.975031E+03, 4.997996E+03, 5.021016E+03, 
   5.044093E+03, 5.067225E+03, 5.090412E+03, 5.113655E+03, 5.136954E+03, 5.160309E+03, 
   5.183719E+03, 5.207185E+03, 5.230707E+03, 5.254284E+03, 5.277917E+03, 5.301607E+03, 
   5.325351E+03, 5.349152E+03, 5.373009E+03, 5.396922E+03, 5.420890E+03, 5.444914E+03, 
   5.468995E+03, 5.493131E+03, 5.517323E+03, 5.541571E+03, 5.565876E+03, 5.590236E+03, 
   5.614652E+03, 5.639125E+03, 5.663653E+03, 5.688238E+03, 5.712879E+03, 5.737576E+03, 
   5.762329E+03, 5.787138E+03, 5.812003E+03, 5.836925E+03, 5.861903E+03, 5.886937E+03, 
   5.912027E+03, 5.937174E+03, 5.962377E+03, 5.987636E+03, 6.012952E+03, 6.038324E+03, 
   6.063752E+03, 6.089237E+03, 6.114778E+03, 6.140376E+03, 6.166030E+03, 6.191741E+03, 
   6.217508E+03, 6.243332E+03, 6.269212E+03, 6.295148E+03, 6.321142E+03, 6.347191E+03, 
   6.373298E+03, 6.399461E+03, 6.425681E+03, 6.451957E+03, 6.478290E+03, 6.504680E+03, 
   6.531126E+03, 6.557629E+03, 6.584189E+03, 6.610806E+03, 6.637479E+03, 6.664210E+03, 
   6.690997E+03, 6.717841E+03, 6.744742E+03, 6.771699E+03, 6.798714E+03, 6.825785E+03, 
   6.852914E+03, 6.880099E+03, 6.907342E+03, 6.934641E+03, 6.961997E+03, 6.989411E+03, 
   7.016881E+03, 7.044409E+03, 7.071993E+03, 7.099635E+03, 7.127334E+03, 7.155089E+03, 
   7.182903E+03, 7.210773E+03, 7.238700E+03, 7.266685E+03, 7.294727E+03, 7.322826E+03, 
   7.350982E+03, 7.379196E+03, 7.407467E+03, 7.435795E+03, 7.464181E+03, 7.492624E+03, 
   7.521124E+03, 7.549682E+03, 7.578297E+03, 7.606969E+03, 7.635699E+03, 7.664487E+03, 
   7.693332E+03, 7.722234E+03, 7.751194E+03, 7.780212E+03, 7.809287E+03, 7.838420E+03, 
   7.867610E+03, 7.896858E+03, 7.926163E+03, 7.955526E+03, 7.984947E+03, 8.014426E+03, 
   8.043962E+03, 8.073556E+03, 8.103207E+03, 8.132917E+03, 8.162684E+03, 8.192509E+03, 
   8.222391E+03, 8.252332E+03, 8.282330E+03, 8.312387E+03, 8.342501E+03, 8.372673E+03, 
   8.402903E+03, 8.433191E+03, 8.463537E+03, 8.493941E+03, 8.524402E+03, 8.554922E+03, 
   8.585500E+03, 8.616136E+03, 8.646830E+03, 8.677582E+03, 8.708392E+03, 8.739260E+03, 
   8.770187E+03, 8.801171E+03, 8.832214E+03, 8.863315E+03, 8.894474E+03, 8.925691E+03, 
   8.956967E+03, 8.988301E+03, 9.019693E+03, 9.051143E+03, 9.082652E+03, 9.114219E+03, 
   9.145844E+03, 9.177528E+03, 9.209270E+03, 9.241071E+03, 9.272930E+03, 9.304847E+03, 
   9.336823E+03, 9.368857E+03, 9.400950E+03, 9.433101E+03, 9.465311E+03, 9.497579E+03, 
   9.529906E+03, 9.562292E+03, 9.594736E+03, 9.627238E+03, 9.659800E+03, 9.692420E+03, 
   9.725098E+03, 9.757835E+03, 9.790631E+03, 9.823486E+03, 9.856400E+03, 9.889372E+03, 
   9.922403E+03, 9.955492E+03, 9.988641E+03, 1.002185E+04, 1.005511E+04, 1.008844E+04, 
   1.012182E+04, 1.015527E+04, 1.018877E+04, 1.022233E+04, 1.025595E+04, 1.028963E+04, 
   1.032336E+04, 1.035716E+04, 1.039102E+04, 1.042493E+04, 1.045890E+04, 1.049294E+04, 
   1.052703E+04, 1.056118E+04, 1.059539E+04, 1.062966E+04, 1.066399E+04, 1.069837E+04, 
   1.073282E+04, 1.076732E+04, 1.080189E+04, 1.083651E+04, 1.087120E+04, 1.090594E+04, 
   1.094074E+04, 1.097560E+04, 1.101052E+04, 1.104550E+04, 1.108054E+04, 1.111564E+04, 
   1.115080E+04, 1.118602E+04, 1.122129E+04, 1.125663E+04, 1.129202E+04, 1.132748E+04, 
   1.136299E+04, 1.139857E+04, 1.143420E+04, 1.146989E+04, 1.150565E+04, 1.154146E+04, 
   1.157733E+04, 1.161326E+04, 1.164925E+04, 1.168530E+04, 1.172141E+04, 1.175758E+04, 
   1.179381E+04, 1.183010E+04, 1.186645E+04, 1.190286E+04, 1.193932E+04, 1.197585E+04, 
   1.201244E+04, 1.204909E+04, 1.208579E+04, 1.212256E+04, 1.215939E+04, 1.219627E+04, 
   1.223322E+04, 1.227023E+04, 1.230729E+04, 1.234442E+04, 1.238161E+04, 1.241885E+04, 
   1.245616E+04, 1.249352E+04, 1.253095E+04, 1.256844E+04, 1.260598E+04, 1.264359E+04, 
   1.268125E+04, 1.271898E+04, 1.275676E+04, 1.279461E+04, 1.283252E+04, 1.287048E+04, 
   1.290851E+04, 1.294660E+04, 1.298474E+04, 1.302295E+04, 1.306122E+04, 1.309954E+04, 
   1.313793E+04, 1.317638E+04, 1.321489E+04, 1.325346E+04, 1.329208E+04, 1.333077E+04, 
   1.336952E+04, 1.340833E+04, 1.344720E+04, 1.348613E+04, 1.352512E+04, 1.356417E+04, 
   1.360328E+04, 1.364245E+04, 1.368169E+04, 1.372098E+04, 1.376033E+04, 1.379975E+04, 
   1.383922E+04, 1.387875E+04, 1.391835E+04, 1.395800E+04, 1.399772E+04, 1.403750E+04, 
   1.407733E+04, 1.411723E+04, 1.415719E+04, 1.419721E+04, 1.423729E+04, 1.427743E+04, 
   1.431763E+04, 1.435789E+04, 1.439821E+04, 1.443859E+04, 1.447903E+04, 1.451954E+04, 
   1.456010E+04, 1.460073E+04, 1.464141E+04, 1.468216E+04, 1.472297E+04, 1.476384E+04, 
   1.480477E+04, 1.484576E+04, 1.488681E+04, 1.492792E+04, 1.496909E+04, 1.501032E+04, 
   1.505162E+04, 1.509297E+04, 1.513439E+04, 1.517587E+04, 1.521740E+04, 1.525900E+04, 
   1.530066E+04, 1.534238E+04, 1.538416E+04, 1.542601E+04, 1.546791E+04, 1.550988E+04, 
   1.555190E+04, 1.559399E+04, 1.563614E+04, 1.567834E+04, 1.572061E+04, 1.576294E+04, 
   1.580534E+04, 1.584779E+04, 1.589030E+04, 1.593288E+04, 1.597551E+04, 1.601821E+04, 
   1.606097E+04, 1.610379E+04, 1.614667E+04, 1.618961E+04, 1.623262E+04, 1.627568E+04, 
   1.631881E+04, 1.636200E+04, 1.640524E+04, 1.644855E+04, 1.649192E+04, 1.653536E+04, 
   1.657885E+04, 1.662240E+04, 1.666602E+04, 1.670970E+04, 1.675344E+04, 1.679724E+04, 
   1.684110E+04, 1.688502E+04, 1.692901E+04, 1.697305E+04, 1.701716E+04, 1.706133E+04, 
   1.710556E+04, 1.714985E+04, 1.719420E+04, 1.723861E+04, 1.728309E+04, 1.732763E+04, 
   1.737222E+04, 1.741688E+04, 1.746161E+04, 1.750639E+04, 1.755123E+04, 1.759614E+04, 
   1.764111E+04, 1.768614E+04, 1.773123E+04, 1.777638E+04, 1.782159E+04, 1.786687E+04, 
   1.791221E+04, 1.795760E+04, 1.800306E+04, 1.804859E+04, 1.809417E+04, 1.813982E+04, 
   1.818552E+04, 1.823129E+04, 1.827712E+04, 1.832301E+04, 1.836897E+04, 1.841498E+04, 
   1.846106E+04, 1.850720E+04, 1.855340E+04, 1.859966E+04, 1.864598E+04, 1.869237E+04, 
   1.873882E+04, 1.878532E+04, 1.883190E+04, 1.887853E+04, 1.892522E+04, 1.897198E+04, 
   1.901880E+04, 1.906568E+04, 1.911262E+04, 1.915962E+04, 1.920669E+04, 1.925381E+04, 
   1.930100E+04, 1.934825E+04, 1.939557E+04, 1.944294E+04, 1.949038E+04, 1.953787E+04, 
   1.958544E+04, 1.963306E+04, 1.968074E+04, 1.972849E+04, 1.977629E+04, 1.982416E+04, 
   1.987210E+04, 1.992009E+04, 1.996815E+04, 2.001626E+04, 2.006444E+04, 2.011268E+04, 
   2.016099E+04, 2.020935E+04, 2.025778E+04, 2.030627E+04, 2.035482E+04, 2.040343E+04, 
   2.045211E+04, 2.050085E+04, 2.054965E+04, 2.059851E+04, 2.064743E+04, 2.069642E+04, 
   2.074547E+04, 2.079458E+04, 2.084375E+04, 2.089298E+04, 2.094228E+04, 2.099163E+04, 
   2.104106E+04, 2.109054E+04, 2.114008E+04, 2.118969E+04, 2.123936E+04, 2.128909E+04, 
   2.133888E+04, 2.138873E+04, 2.143865E+04, 2.148863E+04, 2.153867E+04, 2.158878E+04, 
   2.163894E+04, 2.168917E+04, 2.173946E+04, 2.178981E+04, 2.184022E+04, 2.189070E+04, 
   2.194124E+04, 2.199184E+04, 2.204250E+04, 2.209323E+04, 2.214402E+04, 2.219487E+04, 
   2.224578E+04, 2.229675E+04, 2.234779E+04, 2.239889E+04, 2.245005E+04, 2.250127E+04, 
   2.255255E+04, 2.260390E+04, 2.265531E+04, 2.270678E+04, 2.275832E+04, 2.280991E+04, 
   2.286157E+04, 2.291329E+04, 2.296507E+04, 2.301692E+04, 2.306883E+04, 2.312080E+04, 
   2.317283E+04, 2.322492E+04, 2.327708E+04, 2.332930E+04, 2.338158E+04, 2.343392E+04, 
   2.348633E+04, 2.353880E+04, 2.359133E+04, 2.364392E+04, 2.369657E+04, 2.374929E+04, 
   2.380207E+04, 2.385491E+04, 2.390782E+04, 2.396078E+04, 2.401381E+04, 2.406690E+04, 
   2.412006E+04, 2.417327E+04, 2.422655E+04, 2.427989E+04, 2.433329E+04, 2.438676E+04, 
   2.444028E+04, 2.449387E+04, 2.454752E+04, 2.460124E+04, 2.465502E+04, 2.470885E+04, 
   2.476276E+04, 2.481672E+04, 2.487074E+04, 2.492483E+04, 2.497898E+04, 2.503320E+04, 
   2.508747E+04, 2.514181E+04, 2.519621E+04, 2.525067E+04, 2.530519E+04, 2.535978E+04, 
   2.541443E+04, 2.546914E+04, 2.552392E+04, 2.557875E+04, 2.563365E+04, 2.568861E+04, 
   2.574363E+04, 
])
# ============================ M = 5, I = 3 ============================
M = 5
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.015451E+00, 4.147875E+00, 7.934585E+00, 1.172791E+01, 1.552296E+01, 1.931880E+01, 
   2.311511E+01, 2.691177E+01, 3.070871E+01, 3.450589E+01, 3.830330E+01, 4.210093E+01, 
   4.589875E+01, 4.969677E+01, 5.349498E+01, 5.729338E+01, 6.109197E+01, 6.489073E+01, 
   6.868969E+01, 7.248883E+01, 7.628815E+01, 8.008767E+01, 8.388739E+01, 8.768733E+01, 
   9.148750E+01, 9.528795E+01, 9.908871E+01, 1.028898E+02, 1.066914E+02, 1.104934E+02, 
   1.142961E+02, 1.180996E+02, 1.219039E+02, 1.257093E+02, 1.295159E+02, 1.333240E+02, 
   1.371337E+02, 1.409454E+02, 1.447592E+02, 1.485755E+02, 1.523947E+02, 1.562169E+02, 
   1.600426E+02, 1.638721E+02, 1.677058E+02, 1.715441E+02, 1.753873E+02, 1.792359E+02, 
   1.830902E+02, 1.869507E+02, 1.908178E+02, 1.946919E+02, 1.985735E+02, 2.024629E+02, 
   2.063605E+02, 2.102669E+02, 2.141824E+02, 2.181074E+02, 2.220424E+02, 2.259877E+02, 
   2.299438E+02, 2.339111E+02, 2.378898E+02, 2.418806E+02, 2.458836E+02, 2.498993E+02, 
   2.539281E+02, 2.579703E+02, 2.620262E+02, 2.660963E+02, 2.701807E+02, 2.742799E+02, 
   2.783942E+02, 2.825239E+02, 2.866693E+02, 2.908307E+02, 2.950083E+02, 2.992025E+02, 
   3.034135E+02, 3.076416E+02, 3.118870E+02, 3.161501E+02, 3.204310E+02, 3.247299E+02, 
   3.290472E+02, 3.333830E+02, 3.377375E+02, 3.421110E+02, 3.465036E+02, 3.509156E+02, 
   3.553472E+02, 3.597985E+02, 3.642697E+02, 3.687609E+02, 3.732725E+02, 3.778045E+02, 
   3.823570E+02, 3.869304E+02, 3.915246E+02, 3.961398E+02, 4.007762E+02, 4.054340E+02, 
   4.101132E+02, 4.148140E+02, 4.195365E+02, 4.242808E+02, 4.290471E+02, 4.338355E+02, 
   4.386461E+02, 4.434789E+02, 4.483341E+02, 4.532119E+02, 4.581122E+02, 4.630352E+02, 
   4.679810E+02, 4.729498E+02, 4.779414E+02, 4.829562E+02, 4.879940E+02, 4.930551E+02, 
   4.981395E+02, 5.032473E+02, 5.083786E+02, 5.135333E+02, 5.187117E+02, 5.239138E+02, 
   5.291395E+02, 5.343891E+02, 5.396626E+02, 5.449600E+02, 5.502813E+02, 5.556268E+02, 
   5.609963E+02, 5.663900E+02, 5.718079E+02, 5.772501E+02, 5.827166E+02, 5.882075E+02, 
   5.937229E+02, 5.992627E+02, 6.048270E+02, 6.104159E+02, 6.160294E+02, 6.216676E+02, 
   6.273304E+02, 6.330180E+02, 6.387304E+02, 6.444676E+02, 6.502297E+02, 6.560167E+02, 
   6.618287E+02, 6.676656E+02, 6.735275E+02, 6.794145E+02, 6.853265E+02, 6.912637E+02, 
   6.972260E+02, 7.032135E+02, 7.092262E+02, 7.152642E+02, 7.213275E+02, 7.274160E+02, 
   7.335299E+02, 7.396692E+02, 7.458338E+02, 7.520239E+02, 7.582394E+02, 7.644804E+02, 
   7.707469E+02, 7.770390E+02, 7.833566E+02, 7.896997E+02, 7.960685E+02, 8.024629E+02, 
   8.088830E+02, 8.153287E+02, 8.218001E+02, 8.282972E+02, 8.348201E+02, 8.413688E+02, 
   8.479432E+02, 8.545435E+02, 8.611695E+02, 8.678215E+02, 8.744993E+02, 8.812029E+02, 
   8.879325E+02, 8.946881E+02, 9.014695E+02, 9.082770E+02, 9.151104E+02, 9.219698E+02, 
   9.288553E+02, 9.357668E+02, 9.427043E+02, 9.496679E+02, 9.566576E+02, 9.636735E+02, 
   9.707154E+02, 9.777835E+02, 9.848778E+02, 9.919982E+02, 9.991449E+02, 1.006318E+03, 
   1.013517E+03, 1.020742E+03, 1.027994E+03, 1.035271E+03, 1.042576E+03, 1.049906E+03, 
   1.057263E+03, 1.064646E+03, 1.072055E+03, 1.079491E+03, 1.086953E+03, 1.094442E+03, 
   1.101957E+03, 1.109498E+03, 1.117066E+03, 1.124660E+03, 1.132281E+03, 1.139928E+03, 
   1.147601E+03, 1.155302E+03, 1.163028E+03, 1.170781E+03, 1.178561E+03, 1.186367E+03, 
   1.194200E+03, 1.202060E+03, 1.209945E+03, 1.217858E+03, 1.225797E+03, 1.233763E+03, 
   1.241756E+03, 1.249775E+03, 1.257820E+03, 1.265893E+03, 1.273992E+03, 1.282118E+03, 
   1.290270E+03, 1.298450E+03, 1.306656E+03, 1.314888E+03, 1.323148E+03, 1.331434E+03, 
   1.339747E+03, 1.348087E+03, 1.356454E+03, 1.364847E+03, 1.373267E+03, 1.381714E+03, 
   1.390188E+03, 1.398689E+03, 1.407217E+03, 1.415771E+03, 1.424353E+03, 1.432961E+03, 
   1.441596E+03, 1.450258E+03, 1.458948E+03, 1.467664E+03, 1.476407E+03, 1.485177E+03, 
   1.493974E+03, 1.502798E+03, 1.511649E+03, 1.520526E+03, 1.529431E+03, 1.538363E+03, 
   1.547322E+03, 1.556309E+03, 1.565322E+03, 1.574362E+03, 1.583429E+03, 1.592524E+03, 
   1.601645E+03, 1.610794E+03, 1.619969E+03, 1.629172E+03, 1.638402E+03, 1.647659E+03, 
   1.656944E+03, 1.666255E+03, 1.675594E+03, 1.684960E+03, 1.694353E+03, 1.703773E+03, 
   1.713221E+03, 1.722695E+03, 1.732197E+03, 1.741727E+03, 1.751283E+03, 1.760867E+03, 
   1.770478E+03, 1.780116E+03, 1.789782E+03, 1.799475E+03, 1.809195E+03, 1.818943E+03, 
   1.828718E+03, 1.838520E+03, 1.848350E+03, 1.858207E+03, 1.868091E+03, 1.878003E+03, 
   1.887942E+03, 1.897909E+03, 1.907903E+03, 1.917924E+03, 1.927973E+03, 1.938049E+03, 
   1.948153E+03, 1.958284E+03, 1.968443E+03, 1.978629E+03, 1.988843E+03, 1.999084E+03, 
   2.009353E+03, 2.019649E+03, 2.029973E+03, 2.040324E+03, 2.050703E+03, 2.061109E+03, 
   2.071543E+03, 2.082005E+03, 2.092494E+03, 2.103011E+03, 2.113555E+03, 2.124127E+03, 
   2.134727E+03, 2.145354E+03, 2.156009E+03, 2.166691E+03, 2.177402E+03, 2.188140E+03, 
   2.198905E+03, 2.209698E+03, 2.220519E+03, 2.231368E+03, 2.242244E+03, 2.253149E+03, 
   2.264080E+03, 2.275040E+03, 2.286027E+03, 2.297043E+03, 2.308085E+03, 2.319156E+03, 
   2.330255E+03, 2.341381E+03, 2.352535E+03, 2.363717E+03, 2.374927E+03, 2.386164E+03, 
   2.397430E+03, 2.408723E+03, 2.420044E+03, 2.431394E+03, 2.442771E+03, 2.454175E+03, 
   2.465608E+03, 2.477069E+03, 2.488557E+03, 2.500074E+03, 2.511619E+03, 2.523191E+03, 
   2.534791E+03, 2.546420E+03, 2.558076E+03, 2.569761E+03, 2.581473E+03, 2.593213E+03, 
   2.604982E+03, 2.616778E+03, 2.628603E+03, 2.640455E+03, 2.652336E+03, 2.664244E+03, 
   2.676181E+03, 2.688146E+03, 2.700139E+03, 2.712160E+03, 2.724209E+03, 2.736286E+03, 
   2.748391E+03, 2.760525E+03, 2.772686E+03, 2.784876E+03, 2.797094E+03, 2.809340E+03, 
   2.821615E+03, 2.833917E+03, 2.846248E+03, 2.858607E+03, 2.870994E+03, 2.883409E+03, 
   2.895853E+03, 2.908325E+03, 2.920825E+03, 2.933353E+03, 2.945910E+03, 2.958495E+03, 
   2.971108E+03, 2.983750E+03, 2.996420E+03, 3.009118E+03, 3.021844E+03, 3.034599E+03, 
   3.047382E+03, 3.060194E+03, 3.073034E+03, 3.085902E+03, 3.098799E+03, 3.111724E+03, 
   3.124678E+03, 3.137660E+03, 3.150670E+03, 3.163709E+03, 3.176776E+03, 3.189872E+03, 
   3.202996E+03, 3.216149E+03, 3.229330E+03, 3.242540E+03, 3.255778E+03, 3.269044E+03, 
   3.282340E+03, 3.295663E+03, 3.309015E+03, 3.322396E+03, 3.335806E+03, 3.349243E+03, 
   3.362710E+03, 3.376205E+03, 3.389729E+03, 3.403281E+03, 3.416862E+03, 3.430471E+03, 
   3.444109E+03, 3.457776E+03, 3.471471E+03, 3.485195E+03, 3.498948E+03, 3.512730E+03, 
   3.526540E+03, 3.540378E+03, 3.554246E+03, 3.568142E+03, 3.582067E+03, 3.596021E+03, 
   3.610003E+03, 3.624014E+03, 3.638054E+03, 3.652123E+03, 3.666220E+03, 3.680346E+03, 
   3.694501E+03, 3.708685E+03, 3.722898E+03, 3.737139E+03, 3.751409E+03, 3.765708E+03, 
   3.780036E+03, 3.794393E+03, 3.808779E+03, 3.823193E+03, 3.837637E+03, 3.852109E+03, 
   3.866611E+03, 3.881141E+03, 3.895700E+03, 3.910288E+03, 3.924905E+03, 3.939551E+03, 
   3.954226E+03, 3.968929E+03, 3.983662E+03, 3.998424E+03, 4.013215E+03, 4.028035E+03, 
   4.042883E+03, 4.057761E+03, 4.072668E+03, 4.087604E+03, 4.102569E+03, 4.117563E+03, 
   4.132586E+03, 4.147638E+03, 4.162720E+03, 4.177830E+03, 4.192969E+03, 4.208138E+03, 
   4.223336E+03, 4.238562E+03, 4.253818E+03, 4.269104E+03, 4.284418E+03, 4.299761E+03, 
   4.315134E+03, 4.330536E+03, 4.345967E+03, 4.361427E+03, 4.376916E+03, 4.392435E+03, 
   4.407983E+03, 4.423560E+03, 4.439166E+03, 4.454802E+03, 4.470467E+03, 4.486161E+03, 
   4.501884E+03, 4.517637E+03, 4.533419E+03, 4.549231E+03, 4.565071E+03, 4.580941E+03, 
   4.596841E+03, 4.612769E+03, 4.628727E+03, 4.644715E+03, 4.660731E+03, 4.676778E+03, 
   4.692853E+03, 4.708958E+03, 4.725092E+03, 4.741256E+03, 4.757449E+03, 4.773672E+03, 
   4.789924E+03, 4.806206E+03, 4.822517E+03, 4.838857E+03, 4.855227E+03, 4.871626E+03, 
   4.888055E+03, 4.904514E+03, 4.921002E+03, 4.937519E+03, 4.954066E+03, 4.970643E+03, 
   4.987249E+03, 5.003884E+03, 5.020550E+03, 5.037244E+03, 5.053969E+03, 5.070723E+03, 
   5.087506E+03, 5.104319E+03, 5.121162E+03, 5.138034E+03, 5.154936E+03, 5.171868E+03, 
   5.188829E+03, 5.205820E+03, 5.222841E+03, 5.239891E+03, 5.256971E+03, 5.274081E+03, 
   5.291221E+03, 5.308390E+03, 5.325588E+03, 5.342817E+03, 5.360075E+03, 5.377363E+03, 
   5.394681E+03, 5.412029E+03, 5.429406E+03, 5.446813E+03, 5.464250E+03, 5.481717E+03, 
   5.499213E+03, 5.516739E+03, 5.534296E+03, 5.551881E+03, 5.569497E+03, 5.587143E+03, 
   5.604818E+03, 5.622524E+03, 5.640259E+03, 5.658024E+03, 5.675819E+03, 5.693644E+03, 
   5.711498E+03, 5.729383E+03, 5.747298E+03, 5.765242E+03, 5.783217E+03, 5.801221E+03, 
   5.819255E+03, 5.837319E+03, 5.855414E+03, 5.873538E+03, 5.891692E+03, 5.909876E+03, 
   5.928090E+03, 5.946335E+03, 5.964609E+03, 5.982913E+03, 6.001247E+03, 6.019612E+03, 
   6.038006E+03, 6.056430E+03, 6.074885E+03, 6.093369E+03, 6.111884E+03, 6.130428E+03, 
   6.149003E+03, 6.167608E+03, 6.186243E+03, 6.204908E+03, 6.223603E+03, 6.242328E+03, 
   6.261084E+03, 6.279870E+03, 6.298685E+03, 6.317531E+03, 6.336407E+03, 6.355313E+03, 
   6.374250E+03, 6.393217E+03, 6.412213E+03, 6.431240E+03, 6.450298E+03, 6.469385E+03, 
   6.488503E+03, 6.507651E+03, 6.526829E+03, 6.546037E+03, 6.565276E+03, 6.584545E+03, 
   6.603844E+03, 6.623173E+03, 6.642533E+03, 6.661923E+03, 6.681344E+03, 6.700794E+03, 
   6.720275E+03, 6.739787E+03, 6.759328E+03, 6.778900E+03, 6.798502E+03, 6.818135E+03, 
   6.837798E+03, 6.857492E+03, 6.877215E+03, 6.896969E+03, 6.916754E+03, 6.936569E+03, 
   6.956414E+03, 6.976290E+03, 6.996196E+03, 7.016132E+03, 7.036099E+03, 7.056097E+03, 
   7.076125E+03, 7.096183E+03, 7.116272E+03, 7.136391E+03, 7.156541E+03, 7.176721E+03, 
   7.196931E+03, 7.217172E+03, 7.237444E+03, 7.257746E+03, 7.278079E+03, 7.298442E+03, 
   7.318835E+03, 7.339260E+03, 7.359714E+03, 7.380200E+03, 7.400715E+03, 7.421262E+03, 
   7.441839E+03, 7.462446E+03, 7.483084E+03, 7.503753E+03, 7.524452E+03, 7.545182E+03, 
   7.565942E+03, 7.586733E+03, 7.607555E+03, 7.628407E+03, 7.649290E+03, 7.670203E+03, 
   7.691147E+03, 7.712122E+03, 7.733127E+03, 7.754163E+03, 7.775230E+03, 7.796327E+03, 
   7.817455E+03, 7.838614E+03, 7.859804E+03, 7.881024E+03, 7.902274E+03, 7.923556E+03, 
   7.944868E+03, 7.966211E+03, 7.987585E+03, 8.008989E+03, 8.030424E+03, 8.051890E+03, 
   8.073386E+03, 8.094913E+03, 8.116471E+03, 8.138060E+03, 8.159680E+03, 8.181330E+03, 
   8.203011E+03, 8.224723E+03, 8.246466E+03, 8.268239E+03, 8.290043E+03, 8.311878E+03, 
   8.333744E+03, 8.355641E+03, 8.377568E+03, 8.399527E+03, 8.421516E+03, 8.443536E+03, 
   8.465586E+03, 8.487668E+03, 8.509781E+03, 8.531924E+03, 8.554098E+03, 8.576303E+03, 
   8.598539E+03, 8.620806E+03, 8.643104E+03, 8.665432E+03, 8.687792E+03, 8.710182E+03, 
   8.732603E+03, 8.755055E+03, 8.777538E+03, 8.800052E+03, 8.822597E+03, 8.845173E+03, 
   8.867780E+03, 8.890417E+03, 8.913086E+03, 8.935786E+03, 8.958516E+03, 8.981278E+03, 
   9.004070E+03, 9.026893E+03, 9.049748E+03, 9.072633E+03, 9.095549E+03, 9.118496E+03, 
   9.141475E+03, 9.164484E+03, 9.187524E+03, 9.210595E+03, 9.233697E+03, 9.256831E+03, 
   9.279995E+03, 9.303190E+03, 9.326416E+03, 9.349674E+03, 9.372962E+03, 9.396281E+03, 
   9.419631E+03, 9.443013E+03, 9.466425E+03, 9.489869E+03, 9.513343E+03, 9.536849E+03, 
   9.560385E+03, 9.583953E+03, 9.607552E+03, 9.631182E+03, 9.654843E+03, 9.678535E+03, 
   9.702258E+03, 9.726012E+03, 9.749797E+03, 9.773613E+03, 9.797461E+03, 9.821339E+03, 
   9.845249E+03, 9.869190E+03, 9.893161E+03, 9.917164E+03, 9.941198E+03, 9.965264E+03, 
   9.989360E+03, 1.001349E+04, 1.003765E+04, 1.006184E+04, 1.008606E+04, 1.011031E+04, 
   1.013459E+04, 1.015891E+04, 1.018325E+04, 1.020763E+04, 1.023204E+04, 1.025647E+04, 
   1.028094E+04, 1.030545E+04, 1.032998E+04, 1.035454E+04, 1.037914E+04, 1.040376E+04, 
   1.042842E+04, 1.045311E+04, 1.047783E+04, 1.050258E+04, 1.052736E+04, 1.055217E+04, 
   1.057702E+04, 1.060189E+04, 1.062680E+04, 1.065174E+04, 1.067670E+04, 1.070170E+04, 
   1.072674E+04, 1.075180E+04, 1.077689E+04, 1.080202E+04, 1.082717E+04, 1.085236E+04, 
   1.087758E+04, 1.090283E+04, 1.092811E+04, 1.095342E+04, 1.097877E+04, 1.100414E+04, 
   1.102955E+04, 1.105499E+04, 1.108045E+04, 1.110595E+04, 1.113149E+04, 1.115705E+04, 
   1.118264E+04, 1.120827E+04, 1.123392E+04, 1.125961E+04, 1.128533E+04, 1.131108E+04, 
   1.133686E+04, 1.136267E+04, 1.138851E+04, 1.141439E+04, 1.144030E+04, 1.146623E+04, 
   1.149220E+04, 1.151820E+04, 1.154423E+04, 1.157030E+04, 1.159639E+04, 1.162251E+04, 
   1.164867E+04, 1.167486E+04, 1.170108E+04, 1.172733E+04, 1.175361E+04, 1.177992E+04, 
   1.180627E+04, 1.183264E+04, 1.185905E+04, 1.188549E+04, 1.191196E+04, 1.193846E+04, 
   1.196499E+04, 1.199155E+04, 1.201815E+04, 1.204477E+04, 1.207143E+04, 1.209812E+04, 
   1.212484E+04, 1.215159E+04, 1.217837E+04, 1.220518E+04, 1.223203E+04, 1.225890E+04, 
   1.228581E+04, 1.231275E+04, 1.233972E+04, 1.236672E+04, 1.239375E+04, 1.242082E+04, 
   1.244791E+04, 1.247504E+04, 1.250220E+04, 1.252939E+04, 1.255661E+04, 1.258386E+04, 
   1.261114E+04, 1.263846E+04, 1.266580E+04, 1.269318E+04, 1.272059E+04, 1.274803E+04, 
   1.277550E+04, 1.280300E+04, 1.283053E+04, 1.285810E+04, 1.288570E+04, 1.291332E+04, 
   1.294098E+04, 
])
# ============================ M = 5, I = 4 ============================
M = 5
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.081919E+00, 2.436760E+01, 4.656398E+01, 6.880105E+01, 9.104865E+01, 1.133010E+02, 
   1.355563E+02, 1.578136E+02, 1.800726E+02, 2.023331E+02, 2.245949E+02, 2.468580E+02, 
   2.691223E+02, 2.913877E+02, 3.136542E+02, 3.359218E+02, 3.581906E+02, 3.804604E+02, 
   4.027313E+02, 4.250033E+02, 4.472763E+02, 4.695505E+02, 4.918259E+02, 5.141025E+02, 
   5.363805E+02, 5.586599E+02, 5.809412E+02, 6.032244E+02, 6.255100E+02, 6.477985E+02, 
   6.700904E+02, 6.923864E+02, 7.146872E+02, 7.369937E+02, 7.593070E+02, 7.816283E+02, 
   8.039586E+02, 8.262996E+02, 8.486526E+02, 8.710192E+02, 8.934013E+02, 9.158005E+02, 
   9.382189E+02, 9.606584E+02, 9.831211E+02, 1.005609E+03, 1.028125E+03, 1.050670E+03, 
   1.073248E+03, 1.095860E+03, 1.118509E+03, 1.141197E+03, 1.163927E+03, 1.186700E+03, 
   1.209521E+03, 1.232390E+03, 1.255311E+03, 1.278285E+03, 1.301316E+03, 1.324405E+03, 
   1.347555E+03, 1.370767E+03, 1.394046E+03, 1.417392E+03, 1.440807E+03, 1.464295E+03, 
   1.487857E+03, 1.511495E+03, 1.535211E+03, 1.559007E+03, 1.582885E+03, 1.606848E+03, 
   1.630896E+03, 1.655032E+03, 1.679258E+03, 1.703575E+03, 1.727985E+03, 1.752489E+03, 
   1.777089E+03, 1.801788E+03, 1.826585E+03, 1.851483E+03, 1.876483E+03, 1.901587E+03, 
   1.926795E+03, 1.952110E+03, 1.977532E+03, 2.003063E+03, 2.028704E+03, 2.054456E+03, 
   2.080320E+03, 2.106297E+03, 2.132389E+03, 2.158596E+03, 2.184919E+03, 2.211360E+03, 
   2.237919E+03, 2.264598E+03, 2.291396E+03, 2.318316E+03, 2.345357E+03, 2.372522E+03, 
   2.399809E+03, 2.427221E+03, 2.454758E+03, 2.482421E+03, 2.510210E+03, 2.538127E+03, 
   2.566171E+03, 2.594344E+03, 2.622645E+03, 2.651077E+03, 2.679638E+03, 2.708331E+03, 
   2.737155E+03, 2.766111E+03, 2.795199E+03, 2.824420E+03, 2.853775E+03, 2.883264E+03, 
   2.912888E+03, 2.942646E+03, 2.972540E+03, 3.002569E+03, 3.032735E+03, 3.063038E+03, 
   3.093477E+03, 3.124054E+03, 3.154769E+03, 3.185622E+03, 3.216614E+03, 3.247744E+03, 
   3.279014E+03, 3.310424E+03, 3.341973E+03, 3.373663E+03, 3.405493E+03, 3.437464E+03, 
   3.469576E+03, 3.501830E+03, 3.534226E+03, 3.566763E+03, 3.599443E+03, 3.632265E+03, 
   3.665230E+03, 3.698339E+03, 3.731590E+03, 3.764985E+03, 3.798524E+03, 3.832207E+03, 
   3.866034E+03, 3.900006E+03, 3.934122E+03, 3.968383E+03, 4.002789E+03, 4.037341E+03, 
   4.072038E+03, 4.106881E+03, 4.141869E+03, 4.177004E+03, 4.212285E+03, 4.247712E+03, 
   4.283286E+03, 4.319007E+03, 4.354874E+03, 4.390889E+03, 4.427051E+03, 4.463361E+03, 
   4.499818E+03, 4.536422E+03, 4.573175E+03, 4.610076E+03, 4.647125E+03, 4.684322E+03, 
   4.721668E+03, 4.759162E+03, 4.796805E+03, 4.834597E+03, 4.872538E+03, 4.910628E+03, 
   4.948867E+03, 4.987256E+03, 5.025794E+03, 5.064482E+03, 5.103320E+03, 5.142307E+03, 
   5.181445E+03, 5.220733E+03, 5.260171E+03, 5.299759E+03, 5.339498E+03, 5.379387E+03, 
   5.419427E+03, 5.459618E+03, 5.499959E+03, 5.540452E+03, 5.581096E+03, 5.621891E+03, 
   5.662837E+03, 5.703934E+03, 5.745183E+03, 5.786584E+03, 5.828137E+03, 5.869841E+03, 
   5.911697E+03, 5.953705E+03, 5.995865E+03, 6.038177E+03, 6.080641E+03, 6.123258E+03, 
   6.166027E+03, 6.208949E+03, 6.252023E+03, 6.295250E+03, 6.338630E+03, 6.382162E+03, 
   6.425848E+03, 6.469686E+03, 6.513677E+03, 6.557822E+03, 6.602120E+03, 6.646571E+03, 
   6.691176E+03, 6.735934E+03, 6.780846E+03, 6.825911E+03, 6.871130E+03, 6.916503E+03, 
   6.962030E+03, 7.007710E+03, 7.053545E+03, 7.099533E+03, 7.145676E+03, 7.191973E+03, 
   7.238425E+03, 7.285031E+03, 7.331791E+03, 7.378706E+03, 7.425775E+03, 7.472999E+03, 
   7.520378E+03, 7.567911E+03, 7.615600E+03, 7.663443E+03, 7.711441E+03, 7.759595E+03, 
   7.807903E+03, 7.856367E+03, 7.904986E+03, 7.953760E+03, 8.002690E+03, 8.051775E+03, 
   8.101016E+03, 8.150412E+03, 8.199964E+03, 8.249672E+03, 8.299536E+03, 8.349555E+03, 
   8.399730E+03, 8.450062E+03, 8.500549E+03, 8.551192E+03, 8.601992E+03, 8.652948E+03, 
   8.704060E+03, 8.755328E+03, 8.806753E+03, 8.858335E+03, 8.910073E+03, 8.961967E+03, 
   9.014018E+03, 9.066226E+03, 9.118591E+03, 9.171113E+03, 9.223791E+03, 9.276626E+03, 
   9.329619E+03, 9.382768E+03, 9.436075E+03, 9.489539E+03, 9.543160E+03, 9.596939E+03, 
   9.650874E+03, 9.704968E+03, 9.759219E+03, 9.813627E+03, 9.868193E+03, 9.922916E+03, 
   9.977798E+03, 1.003284E+04, 1.008803E+04, 1.014339E+04, 1.019890E+04, 1.025457E+04, 
   1.031040E+04, 1.036639E+04, 1.042253E+04, 1.047884E+04, 1.053530E+04, 1.059192E+04, 
   1.064870E+04, 1.070564E+04, 1.076273E+04, 1.081999E+04, 1.087740E+04, 1.093497E+04, 
   1.099270E+04, 1.105059E+04, 1.110864E+04, 1.116684E+04, 1.122521E+04, 1.128373E+04, 
   1.134242E+04, 1.140126E+04, 1.146026E+04, 1.151943E+04, 1.157875E+04, 1.163823E+04, 
   1.169787E+04, 1.175767E+04, 1.181763E+04, 1.187774E+04, 1.193802E+04, 1.199846E+04, 
   1.205906E+04, 1.211981E+04, 1.218073E+04, 1.224181E+04, 1.230305E+04, 1.236444E+04, 
   1.242600E+04, 1.248772E+04, 1.254960E+04, 1.261163E+04, 1.267383E+04, 1.273619E+04, 
   1.279871E+04, 1.286139E+04, 1.292423E+04, 1.298723E+04, 1.305039E+04, 1.311371E+04, 
   1.317719E+04, 1.324084E+04, 1.330464E+04, 1.336861E+04, 1.343273E+04, 1.349702E+04, 
   1.356147E+04, 1.362608E+04, 1.369085E+04, 1.375578E+04, 1.382087E+04, 1.388613E+04, 
   1.395154E+04, 1.401712E+04, 1.408286E+04, 1.414876E+04, 1.421482E+04, 1.428104E+04, 
   1.434743E+04, 1.441398E+04, 1.448068E+04, 1.454756E+04, 1.461459E+04, 1.468178E+04, 
   1.474914E+04, 1.481666E+04, 1.488434E+04, 1.495218E+04, 1.502019E+04, 1.508835E+04, 
   1.515669E+04, 1.522518E+04, 1.529383E+04, 1.536265E+04, 1.543163E+04, 1.550077E+04, 
   1.557008E+04, 1.563955E+04, 1.570918E+04, 1.577897E+04, 1.584893E+04, 1.591905E+04, 
   1.598933E+04, 1.605978E+04, 1.613039E+04, 1.620116E+04, 1.627209E+04, 1.634319E+04, 
   1.641445E+04, 1.648588E+04, 1.655747E+04, 1.662922E+04, 1.670114E+04, 1.677322E+04, 
   1.684546E+04, 1.691787E+04, 1.699044E+04, 1.706317E+04, 1.713607E+04, 1.720913E+04, 
   1.728236E+04, 1.735575E+04, 1.742930E+04, 1.750302E+04, 1.757690E+04, 1.765095E+04, 
   1.772516E+04, 1.779954E+04, 1.787408E+04, 1.794878E+04, 1.802365E+04, 1.809868E+04, 
   1.817388E+04, 1.824925E+04, 1.832477E+04, 1.840047E+04, 1.847632E+04, 1.855235E+04, 
   1.862853E+04, 1.870488E+04, 1.878140E+04, 1.885808E+04, 1.893493E+04, 1.901195E+04, 
   1.908912E+04, 1.916647E+04, 1.924398E+04, 1.932165E+04, 1.939949E+04, 1.947749E+04, 
   1.955567E+04, 1.963400E+04, 1.971250E+04, 1.979117E+04, 1.987001E+04, 1.994901E+04, 
   2.002817E+04, 2.010750E+04, 2.018700E+04, 2.026666E+04, 2.034649E+04, 2.042649E+04, 
   2.050665E+04, 2.058698E+04, 2.066748E+04, 2.074814E+04, 2.082897E+04, 2.090996E+04, 
   2.099112E+04, 2.107245E+04, 2.115394E+04, 2.123560E+04, 2.131743E+04, 2.139943E+04, 
   2.148159E+04, 2.156392E+04, 2.164641E+04, 2.172907E+04, 2.181190E+04, 2.189490E+04, 
   2.197807E+04, 2.206140E+04, 2.214490E+04, 2.222856E+04, 2.231239E+04, 2.239640E+04, 
   2.248056E+04, 2.256490E+04, 2.264940E+04, 2.273407E+04, 2.281891E+04, 2.290392E+04, 
   2.298910E+04, 2.307444E+04, 2.315995E+04, 2.324563E+04, 2.333147E+04, 2.341749E+04, 
   2.350367E+04, 2.359002E+04, 2.367654E+04, 2.376323E+04, 2.385009E+04, 2.393711E+04, 
   2.402431E+04, 2.411167E+04, 2.419920E+04, 2.428690E+04, 2.437476E+04, 2.446280E+04, 
   2.455101E+04, 2.463938E+04, 2.472792E+04, 2.481664E+04, 2.490552E+04, 2.499457E+04, 
   2.508379E+04, 2.517318E+04, 2.526273E+04, 2.535246E+04, 2.544236E+04, 2.553242E+04, 
   2.562266E+04, 2.571306E+04, 2.580364E+04, 2.589438E+04, 2.598529E+04, 2.607638E+04, 
   2.616763E+04, 2.625905E+04, 2.635065E+04, 2.644241E+04, 2.653434E+04, 2.662645E+04, 
   2.671872E+04, 2.681116E+04, 2.690377E+04, 2.699656E+04, 2.708951E+04, 2.718264E+04, 
   2.727593E+04, 2.736939E+04, 2.746303E+04, 2.755684E+04, 2.765081E+04, 2.774496E+04, 
   2.783928E+04, 2.793377E+04, 2.802843E+04, 2.812326E+04, 2.821826E+04, 2.831343E+04, 
   2.840877E+04, 2.850429E+04, 2.859997E+04, 2.869583E+04, 2.879186E+04, 2.888806E+04, 
   2.898443E+04, 2.908097E+04, 2.917768E+04, 2.927457E+04, 2.937162E+04, 2.946885E+04, 
   2.956625E+04, 2.966382E+04, 2.976157E+04, 2.985948E+04, 2.995757E+04, 3.005583E+04, 
   3.015426E+04, 3.025286E+04, 3.035163E+04, 3.045058E+04, 3.054970E+04, 3.064899E+04, 
   3.074845E+04, 3.084809E+04, 3.094790E+04, 3.104788E+04, 3.114803E+04, 3.124836E+04, 
   3.134885E+04, 3.144952E+04, 3.155037E+04, 3.165138E+04, 3.175257E+04, 3.185393E+04, 
   3.195547E+04, 3.205717E+04, 3.215905E+04, 3.226111E+04, 3.236333E+04, 3.246573E+04, 
   3.256830E+04, 3.267105E+04, 3.277397E+04, 3.287706E+04, 3.298033E+04, 3.308376E+04, 
   3.318738E+04, 3.329116E+04, 3.339512E+04, 3.349925E+04, 3.360356E+04, 3.370804E+04, 
   3.381270E+04, 3.391752E+04, 3.402252E+04, 3.412770E+04, 3.423305E+04, 3.433857E+04, 
   3.444427E+04, 3.455014E+04, 3.465619E+04, 3.476241E+04, 3.486880E+04, 3.497537E+04, 
   3.508211E+04, 3.518903E+04, 3.529612E+04, 3.540338E+04, 3.551082E+04, 3.561844E+04, 
   3.572623E+04, 3.583419E+04, 3.594233E+04, 3.605064E+04, 3.615913E+04, 3.626779E+04, 
   3.637663E+04, 3.648564E+04, 3.659483E+04, 3.670419E+04, 3.681373E+04, 3.692345E+04, 
   3.703333E+04, 3.714340E+04, 3.725363E+04, 3.736405E+04, 3.747464E+04, 3.758540E+04, 
   3.769634E+04, 3.780746E+04, 3.791875E+04, 3.803021E+04, 3.814186E+04, 3.825367E+04, 
   3.836567E+04, 3.847783E+04, 3.859018E+04, 3.870270E+04, 3.881540E+04, 3.892827E+04, 
   3.904132E+04, 3.915454E+04, 3.926794E+04, 3.938152E+04, 3.949527E+04, 3.960920E+04, 
   3.972330E+04, 3.983758E+04, 3.995204E+04, 4.006668E+04, 4.018149E+04, 4.029647E+04, 
   4.041164E+04, 4.052698E+04, 4.064249E+04, 4.075818E+04, 4.087405E+04, 4.099010E+04, 
   4.110632E+04, 4.122272E+04, 4.133930E+04, 4.145605E+04, 4.157298E+04, 4.169009E+04, 
   4.180737E+04, 4.192483E+04, 4.204247E+04, 4.216029E+04, 4.227828E+04, 4.239645E+04, 
   4.251480E+04, 4.263332E+04, 4.275202E+04, 4.287090E+04, 4.298996E+04, 4.310919E+04, 
   4.322860E+04, 4.334819E+04, 4.346796E+04, 4.358790E+04, 4.370802E+04, 4.382832E+04, 
   4.394880E+04, 4.406945E+04, 4.419028E+04, 4.431129E+04, 4.443248E+04, 4.455385E+04, 
   4.467539E+04, 4.479711E+04, 4.491901E+04, 4.504109E+04, 4.516335E+04, 4.528578E+04, 
   4.540839E+04, 4.553118E+04, 4.565415E+04, 4.577730E+04, 4.590062E+04, 4.602413E+04, 
   4.614781E+04, 4.627167E+04, 4.639571E+04, 4.651993E+04, 4.664432E+04, 4.676890E+04, 
   4.689365E+04, 4.701858E+04, 4.714369E+04, 4.726898E+04, 4.739445E+04, 4.752010E+04, 
   4.764592E+04, 4.777193E+04, 4.789811E+04, 4.802447E+04, 4.815101E+04, 4.827773E+04, 
   4.840463E+04, 4.853171E+04, 4.865897E+04, 4.878640E+04, 4.891402E+04, 4.904181E+04, 
   4.916979E+04, 4.929794E+04, 4.942627E+04, 4.955479E+04, 4.968348E+04, 4.981235E+04, 
   4.994140E+04, 5.007063E+04, 5.020004E+04, 5.032963E+04, 5.045939E+04, 5.058934E+04, 
   5.071947E+04, 5.084978E+04, 5.098026E+04, 5.111093E+04, 5.124178E+04, 5.137280E+04, 
   5.150401E+04, 5.163540E+04, 5.176696E+04, 5.189871E+04, 5.203063E+04, 5.216274E+04, 
   5.229502E+04, 5.242749E+04, 5.256014E+04, 5.269296E+04, 5.282597E+04, 5.295915E+04, 
   5.309252E+04, 5.322607E+04, 5.335979E+04, 5.349370E+04, 5.362779E+04, 5.376205E+04, 
   5.389650E+04, 5.403113E+04, 5.416594E+04, 5.430093E+04, 5.443610E+04, 5.457145E+04, 
   5.470698E+04, 5.484269E+04, 5.497858E+04, 5.511466E+04, 5.525091E+04, 5.538734E+04, 
   5.552396E+04, 5.566075E+04, 5.579773E+04, 5.593488E+04, 5.607222E+04, 5.620974E+04, 
   5.634744E+04, 5.648532E+04, 5.662338E+04, 5.676162E+04, 5.690004E+04, 5.703865E+04, 
   5.717743E+04, 5.731640E+04, 5.745555E+04, 5.759487E+04, 5.773438E+04, 5.787407E+04, 
   5.801394E+04, 5.815400E+04, 5.829423E+04, 5.843464E+04, 5.857524E+04, 5.871602E+04, 
   5.885698E+04, 5.899812E+04, 5.913944E+04, 5.928094E+04, 5.942262E+04, 5.956449E+04, 
   5.970654E+04, 5.984876E+04, 5.999117E+04, 6.013376E+04, 6.027654E+04, 6.041949E+04, 
   6.056263E+04, 6.070594E+04, 6.084944E+04, 6.099312E+04, 6.113698E+04, 6.128103E+04, 
   6.142525E+04, 6.156966E+04, 6.171425E+04, 6.185902E+04, 6.200397E+04, 6.214911E+04, 
   6.229442E+04, 6.243992E+04, 6.258560E+04, 6.273146E+04, 6.287750E+04, 6.302373E+04, 
   6.317013E+04, 6.331672E+04, 6.346349E+04, 6.361045E+04, 6.375758E+04, 6.390490E+04, 
   6.405239E+04, 6.420007E+04, 6.434794E+04, 6.449598E+04, 6.464421E+04, 6.479262E+04, 
   6.494121E+04, 6.508998E+04, 6.523894E+04, 6.538807E+04, 6.553739E+04, 6.568689E+04, 
   6.583658E+04, 6.598644E+04, 6.613649E+04, 6.628672E+04, 6.643713E+04, 6.658773E+04, 
   6.673850E+04, 6.688946E+04, 6.704060E+04, 6.719193E+04, 6.734343E+04, 6.749512E+04, 
   6.764699E+04, 6.779905E+04, 6.795128E+04, 6.810370E+04, 6.825630E+04, 6.840908E+04, 
   6.856205E+04, 6.871520E+04, 6.886853E+04, 6.902204E+04, 6.917573E+04, 6.932961E+04, 
   6.948367E+04, 6.963791E+04, 6.979234E+04, 6.994694E+04, 7.010173E+04, 7.025671E+04, 
   7.041186E+04, 7.056720E+04, 7.072272E+04, 7.087842E+04, 7.103431E+04, 7.119037E+04, 
   7.134663E+04, 7.150306E+04, 7.165967E+04, 7.181647E+04, 7.197345E+04, 7.213062E+04, 
   7.228796E+04, 7.244549E+04, 7.260320E+04, 7.276110E+04, 7.291917E+04, 7.307743E+04, 
   7.323588E+04, 7.339450E+04, 7.355331E+04, 7.371230E+04, 7.387147E+04, 7.403083E+04, 
   7.419036E+04, 7.435009E+04, 7.450999E+04, 7.467008E+04, 7.483035E+04, 7.499080E+04, 
   7.515143E+04, 
])
# ============================ M = 5, I = 5 ============================
M = 5
I = 5
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.039417E+00, 8.661610E+00, 1.660365E+01, 2.455829E+01, 3.251624E+01, 4.047572E+01, 
   4.843614E+01, 5.639726E+01, 6.435894E+01, 7.232113E+01, 8.028378E+01, 8.824686E+01, 
   9.621037E+01, 1.041743E+02, 1.121386E+02, 1.201033E+02, 1.280684E+02, 1.360338E+02, 
   1.439997E+02, 1.519660E+02, 1.599326E+02, 1.678997E+02, 1.758672E+02, 1.838351E+02, 
   1.918036E+02, 1.997728E+02, 2.077426E+02, 2.157133E+02, 2.236851E+02, 2.316581E+02, 
   2.396326E+02, 2.476089E+02, 2.555873E+02, 2.635683E+02, 2.715523E+02, 2.795398E+02, 
   2.875312E+02, 2.955274E+02, 3.035287E+02, 3.115360E+02, 3.195498E+02, 3.275711E+02, 
   3.356005E+02, 3.436389E+02, 3.516871E+02, 3.597459E+02, 3.678163E+02, 3.758991E+02, 
   3.839953E+02, 3.921057E+02, 4.002313E+02, 4.083730E+02, 4.165317E+02, 4.247084E+02, 
   4.329040E+02, 4.411194E+02, 4.493555E+02, 4.576133E+02, 4.658935E+02, 4.741972E+02, 
   4.825251E+02, 4.908781E+02, 4.992572E+02, 5.076630E+02, 5.160964E+02, 5.245582E+02, 
   5.330491E+02, 5.415700E+02, 5.501215E+02, 5.587045E+02, 5.673195E+02, 5.759673E+02, 
   5.846486E+02, 5.933640E+02, 6.021141E+02, 6.108996E+02, 6.197210E+02, 6.285790E+02, 
   6.374741E+02, 6.464068E+02, 6.553778E+02, 6.643875E+02, 6.734364E+02, 6.825250E+02, 
   6.916539E+02, 7.008234E+02, 7.100339E+02, 7.192861E+02, 7.285801E+02, 7.379166E+02, 
   7.472958E+02, 7.567182E+02, 7.661840E+02, 7.756938E+02, 7.852478E+02, 7.948464E+02, 
   8.044898E+02, 8.141785E+02, 8.239127E+02, 8.336928E+02, 8.435189E+02, 8.533915E+02, 
   8.633107E+02, 8.732769E+02, 8.832903E+02, 8.933512E+02, 9.034597E+02, 9.136162E+02, 
   9.238208E+02, 9.340738E+02, 9.443754E+02, 9.547258E+02, 9.651252E+02, 9.755738E+02, 
   9.860718E+02, 9.966194E+02, 1.007217E+03, 1.017864E+03, 1.028562E+03, 1.039309E+03, 
   1.050107E+03, 1.060956E+03, 1.071856E+03, 1.082806E+03, 1.093807E+03, 1.104860E+03, 
   1.115964E+03, 1.127119E+03, 1.138326E+03, 1.149585E+03, 1.160895E+03, 1.172258E+03, 
   1.183672E+03, 1.195139E+03, 1.206658E+03, 1.218229E+03, 1.229853E+03, 1.241530E+03, 
   1.253259E+03, 1.265041E+03, 1.276876E+03, 1.288764E+03, 1.300705E+03, 1.312699E+03, 
   1.324747E+03, 1.336848E+03, 1.349002E+03, 1.361210E+03, 1.373471E+03, 1.385786E+03, 
   1.398155E+03, 1.410578E+03, 1.423054E+03, 1.435585E+03, 1.448169E+03, 1.460808E+03, 
   1.473501E+03, 1.486248E+03, 1.499050E+03, 1.511906E+03, 1.524816E+03, 1.537781E+03, 
   1.550800E+03, 1.563874E+03, 1.577002E+03, 1.590186E+03, 1.603424E+03, 1.616717E+03, 
   1.630065E+03, 1.643468E+03, 1.656925E+03, 1.670438E+03, 1.684006E+03, 1.697629E+03, 
   1.711308E+03, 1.725041E+03, 1.738830E+03, 1.752674E+03, 1.766574E+03, 1.780529E+03, 
   1.794539E+03, 1.808605E+03, 1.822727E+03, 1.836904E+03, 1.851137E+03, 1.865425E+03, 
   1.879770E+03, 1.894170E+03, 1.908625E+03, 1.923137E+03, 1.937705E+03, 1.952328E+03, 
   1.967007E+03, 1.981743E+03, 1.996534E+03, 2.011382E+03, 2.026285E+03, 2.041245E+03, 
   2.056261E+03, 2.071333E+03, 2.086461E+03, 2.101646E+03, 2.116887E+03, 2.132184E+03, 
   2.147537E+03, 2.162947E+03, 2.178414E+03, 2.193937E+03, 2.209516E+03, 2.225152E+03, 
   2.240845E+03, 2.256594E+03, 2.272400E+03, 2.288262E+03, 2.304181E+03, 2.320157E+03, 
   2.336189E+03, 2.352279E+03, 2.368425E+03, 2.384628E+03, 2.400887E+03, 2.417204E+03, 
   2.433577E+03, 2.450008E+03, 2.466495E+03, 2.483040E+03, 2.499641E+03, 2.516300E+03, 
   2.533015E+03, 2.549788E+03, 2.566618E+03, 2.583505E+03, 2.600449E+03, 2.617450E+03, 
   2.634509E+03, 2.651624E+03, 2.668798E+03, 2.686028E+03, 2.703316E+03, 2.720661E+03, 
   2.738063E+03, 2.755523E+03, 2.773040E+03, 2.790615E+03, 2.808247E+03, 2.825937E+03, 
   2.843684E+03, 2.861489E+03, 2.879351E+03, 2.897271E+03, 2.915249E+03, 2.933284E+03, 
   2.951377E+03, 2.969527E+03, 2.987736E+03, 3.006002E+03, 3.024325E+03, 3.042707E+03, 
   3.061146E+03, 3.079643E+03, 3.098198E+03, 3.116811E+03, 3.135482E+03, 3.154211E+03, 
   3.172998E+03, 3.191842E+03, 3.210745E+03, 3.229705E+03, 3.248724E+03, 3.267801E+03, 
   3.286935E+03, 3.306128E+03, 3.325379E+03, 3.344688E+03, 3.364056E+03, 3.383481E+03, 
   3.402965E+03, 3.422507E+03, 3.442107E+03, 3.461765E+03, 3.481482E+03, 3.501257E+03, 
   3.521090E+03, 3.540982E+03, 3.560932E+03, 3.580940E+03, 3.601007E+03, 3.621132E+03, 
   3.641316E+03, 3.661558E+03, 3.681859E+03, 3.702218E+03, 3.722636E+03, 3.743112E+03, 
   3.763647E+03, 3.784241E+03, 3.804893E+03, 3.825604E+03, 3.846373E+03, 3.867201E+03, 
   3.888088E+03, 3.909034E+03, 3.930038E+03, 3.951101E+03, 3.972223E+03, 3.993404E+03, 
   4.014644E+03, 4.035942E+03, 4.057299E+03, 4.078715E+03, 4.100190E+03, 4.121724E+03, 
   4.143317E+03, 4.164969E+03, 4.186680E+03, 4.208450E+03, 4.230279E+03, 4.252167E+03, 
   4.274114E+03, 4.296121E+03, 4.318186E+03, 4.340310E+03, 4.362494E+03, 4.384737E+03, 
   4.407039E+03, 4.429400E+03, 4.451821E+03, 4.474300E+03, 4.496839E+03, 4.519437E+03, 
   4.542095E+03, 4.564812E+03, 4.587588E+03, 4.610424E+03, 4.633319E+03, 4.656274E+03, 
   4.679287E+03, 4.702361E+03, 4.725494E+03, 4.748686E+03, 4.771938E+03, 4.795249E+03, 
   4.818620E+03, 4.842051E+03, 4.865541E+03, 4.889091E+03, 4.912700E+03, 4.936369E+03, 
   4.960098E+03, 4.983886E+03, 5.007734E+03, 5.031642E+03, 5.055610E+03, 5.079637E+03, 
   5.103724E+03, 5.127871E+03, 5.152078E+03, 5.176345E+03, 5.200672E+03, 5.225058E+03, 
   5.249504E+03, 5.274011E+03, 5.298577E+03, 5.323203E+03, 5.347889E+03, 5.372636E+03, 
   5.397442E+03, 5.422308E+03, 5.447235E+03, 5.472221E+03, 5.497268E+03, 5.522374E+03, 
   5.547541E+03, 5.572768E+03, 5.598056E+03, 5.623403E+03, 5.648811E+03, 5.674279E+03, 
   5.699807E+03, 5.725395E+03, 5.751044E+03, 5.776753E+03, 5.802523E+03, 5.828353E+03, 
   5.854243E+03, 5.880194E+03, 5.906205E+03, 5.932276E+03, 5.958408E+03, 5.984601E+03, 
   6.010853E+03, 6.037167E+03, 6.063541E+03, 6.089976E+03, 6.116471E+03, 6.143027E+03, 
   6.169643E+03, 6.196320E+03, 6.223058E+03, 6.249856E+03, 6.276715E+03, 6.303635E+03, 
   6.330616E+03, 6.357657E+03, 6.384759E+03, 6.411922E+03, 6.439146E+03, 6.466430E+03, 
   6.493776E+03, 6.521182E+03, 6.548649E+03, 6.576177E+03, 6.603766E+03, 6.631416E+03, 
   6.659127E+03, 6.686899E+03, 6.714732E+03, 6.742626E+03, 6.770581E+03, 6.798598E+03, 
   6.826675E+03, 6.854813E+03, 6.883013E+03, 6.911273E+03, 6.939595E+03, 6.967978E+03, 
   6.996422E+03, 7.024928E+03, 7.053495E+03, 7.082123E+03, 7.110812E+03, 7.139562E+03, 
   7.168374E+03, 7.197247E+03, 7.226182E+03, 7.255178E+03, 7.284235E+03, 7.313354E+03, 
   7.342534E+03, 7.371776E+03, 7.401079E+03, 7.430444E+03, 7.459870E+03, 7.489358E+03, 
   7.518907E+03, 7.548518E+03, 7.578191E+03, 7.607925E+03, 7.637721E+03, 7.667578E+03, 
   7.697497E+03, 7.727478E+03, 7.757520E+03, 7.787625E+03, 7.817791E+03, 7.848018E+03, 
   7.878308E+03, 7.908659E+03, 7.939072E+03, 7.969547E+03, 8.000084E+03, 8.030683E+03, 
   8.061344E+03, 8.092066E+03, 8.122851E+03, 8.153697E+03, 8.184606E+03, 8.215576E+03, 
   8.246609E+03, 8.277704E+03, 8.308860E+03, 8.340079E+03, 8.371360E+03, 8.402703E+03, 
   8.434108E+03, 8.465575E+03, 8.497104E+03, 8.528696E+03, 8.560349E+03, 8.592065E+03, 
   8.623844E+03, 8.655684E+03, 8.687587E+03, 8.719552E+03, 8.751580E+03, 8.783669E+03, 
   8.815821E+03, 8.848036E+03, 8.880313E+03, 8.912652E+03, 8.945054E+03, 8.977518E+03, 
   9.010045E+03, 9.042634E+03, 9.075286E+03, 9.108000E+03, 9.140777E+03, 9.173616E+03, 
   9.206518E+03, 9.239482E+03, 9.272509E+03, 9.305599E+03, 9.338752E+03, 9.371967E+03, 
   9.405244E+03, 9.438585E+03, 9.471988E+03, 9.505454E+03, 9.538983E+03, 9.572575E+03, 
   9.606229E+03, 9.639946E+03, 9.673726E+03, 9.707569E+03, 9.741475E+03, 9.775443E+03, 
   9.809475E+03, 9.843569E+03, 9.877727E+03, 9.911947E+03, 9.946231E+03, 9.980577E+03, 
   1.001499E+04, 1.004946E+04, 1.008399E+04, 1.011859E+04, 1.015325E+04, 1.018798E+04, 
   1.022277E+04, 1.025762E+04, 1.029253E+04, 1.032751E+04, 1.036255E+04, 1.039765E+04, 
   1.043282E+04, 1.046805E+04, 1.050335E+04, 1.053870E+04, 1.057412E+04, 1.060961E+04, 
   1.064515E+04, 1.068076E+04, 1.071644E+04, 1.075217E+04, 1.078797E+04, 1.082384E+04, 
   1.085976E+04, 1.089576E+04, 1.093181E+04, 1.096793E+04, 1.100411E+04, 1.104035E+04, 
   1.107666E+04, 1.111303E+04, 1.114947E+04, 1.118597E+04, 1.122253E+04, 1.125916E+04, 
   1.129585E+04, 1.133260E+04, 1.136942E+04, 1.140630E+04, 1.144324E+04, 1.148025E+04, 
   1.151732E+04, 1.155446E+04, 1.159166E+04, 1.162892E+04, 1.166625E+04, 1.170364E+04, 
   1.174110E+04, 1.177861E+04, 1.181620E+04, 1.185384E+04, 1.189155E+04, 1.192933E+04, 
   1.196717E+04, 1.200507E+04, 1.204304E+04, 1.208107E+04, 1.211916E+04, 1.215732E+04, 
   1.219554E+04, 1.223383E+04, 1.227218E+04, 1.231059E+04, 1.234907E+04, 1.238761E+04, 
   1.242622E+04, 1.246489E+04, 1.250363E+04, 1.254243E+04, 1.258129E+04, 1.262022E+04, 
   1.265921E+04, 1.269827E+04, 1.273739E+04, 1.277657E+04, 1.281582E+04, 1.285513E+04, 
   1.289451E+04, 1.293395E+04, 1.297346E+04, 1.301303E+04, 1.305267E+04, 1.309237E+04, 
   1.313213E+04, 1.317196E+04, 1.321185E+04, 1.325181E+04, 1.329183E+04, 1.333192E+04, 
   1.337207E+04, 1.341228E+04, 1.345256E+04, 1.349291E+04, 1.353332E+04, 1.357379E+04, 
   1.361433E+04, 1.365493E+04, 1.369560E+04, 1.373633E+04, 1.377713E+04, 1.381799E+04, 
   1.385891E+04, 1.389990E+04, 1.394096E+04, 1.398208E+04, 1.402326E+04, 1.406451E+04, 
   1.410583E+04, 1.414720E+04, 1.418865E+04, 1.423016E+04, 1.427173E+04, 1.431337E+04, 
   1.435507E+04, 1.439684E+04, 1.443867E+04, 1.448057E+04, 1.452253E+04, 1.456456E+04, 
   1.460665E+04, 1.464881E+04, 1.469103E+04, 1.473332E+04, 1.477567E+04, 1.481808E+04, 
   1.486057E+04, 1.490311E+04, 1.494572E+04, 1.498840E+04, 1.503114E+04, 1.507395E+04, 
   1.511682E+04, 1.515976E+04, 1.520276E+04, 1.524583E+04, 1.528896E+04, 1.533216E+04, 
   1.537542E+04, 1.541875E+04, 1.546214E+04, 1.550560E+04, 1.554912E+04, 1.559271E+04, 
   1.563636E+04, 1.568008E+04, 1.572387E+04, 1.576772E+04, 1.581163E+04, 1.585561E+04, 
   1.589966E+04, 1.594377E+04, 1.598794E+04, 1.603218E+04, 1.607649E+04, 1.612086E+04, 
   1.616530E+04, 1.620980E+04, 1.625437E+04, 1.629900E+04, 1.634370E+04, 1.638846E+04, 
   1.643329E+04, 1.647819E+04, 1.652315E+04, 1.656817E+04, 1.661326E+04, 1.665842E+04, 
   1.670364E+04, 1.674893E+04, 1.679428E+04, 1.683970E+04, 1.688518E+04, 1.693073E+04, 
   1.697635E+04, 1.702203E+04, 1.706777E+04, 1.711358E+04, 1.715946E+04, 1.720540E+04, 
   1.725141E+04, 1.729749E+04, 1.734363E+04, 1.738983E+04, 1.743610E+04, 1.748244E+04, 
   1.752884E+04, 1.757531E+04, 1.762184E+04, 1.766844E+04, 1.771510E+04, 1.776183E+04, 
   1.780863E+04, 1.785549E+04, 1.790242E+04, 1.794941E+04, 1.799647E+04, 1.804359E+04, 
   1.809078E+04, 1.813804E+04, 1.818536E+04, 1.823275E+04, 1.828020E+04, 1.832772E+04, 
   1.837531E+04, 1.842296E+04, 1.847067E+04, 1.851846E+04, 1.856630E+04, 1.861422E+04, 
   1.866220E+04, 1.871024E+04, 1.875836E+04, 1.880653E+04, 1.885478E+04, 1.890309E+04, 
   1.895146E+04, 1.899990E+04, 1.904841E+04, 1.909698E+04, 1.914562E+04, 1.919432E+04, 
   1.924309E+04, 1.929193E+04, 1.934083E+04, 1.938980E+04, 1.943883E+04, 1.948793E+04, 
   1.953710E+04, 1.958633E+04, 1.963563E+04, 1.968500E+04, 1.973443E+04, 1.978392E+04, 
   1.983348E+04, 1.988311E+04, 1.993281E+04, 1.998257E+04, 2.003239E+04, 2.008228E+04, 
   2.013224E+04, 2.018227E+04, 2.023236E+04, 2.028251E+04, 2.033273E+04, 2.038302E+04, 
   2.043338E+04, 2.048380E+04, 2.053428E+04, 2.058484E+04, 2.063546E+04, 2.068614E+04, 
   2.073689E+04, 2.078771E+04, 2.083859E+04, 2.088954E+04, 2.094056E+04, 2.099164E+04, 
   2.104279E+04, 2.109400E+04, 2.114528E+04, 2.119663E+04, 2.124804E+04, 2.129952E+04, 
   2.135106E+04, 2.140267E+04, 2.145435E+04, 2.150609E+04, 2.155790E+04, 2.160978E+04, 
   2.166172E+04, 2.171372E+04, 2.176580E+04, 2.181794E+04, 2.187014E+04, 2.192242E+04, 
   2.197476E+04, 2.202716E+04, 2.207963E+04, 2.213217E+04, 2.218477E+04, 2.223744E+04, 
   2.229018E+04, 2.234298E+04, 2.239585E+04, 2.244878E+04, 2.250178E+04, 2.255485E+04, 
   2.260798E+04, 2.266118E+04, 2.271445E+04, 2.276778E+04, 2.282118E+04, 2.287464E+04, 
   2.292817E+04, 2.298177E+04, 2.303543E+04, 2.308916E+04, 2.314296E+04, 2.319682E+04, 
   2.325075E+04, 2.330474E+04, 2.335880E+04, 2.341293E+04, 2.346712E+04, 2.352138E+04, 
   2.357571E+04, 2.363010E+04, 2.368455E+04, 2.373908E+04, 2.379367E+04, 2.384833E+04, 
   2.390305E+04, 2.395784E+04, 2.401269E+04, 2.406762E+04, 2.412260E+04, 2.417766E+04, 
   2.423278E+04, 2.428797E+04, 2.434322E+04, 2.439854E+04, 2.445392E+04, 2.450938E+04, 
   2.456489E+04, 2.462048E+04, 2.467613E+04, 2.473185E+04, 2.478763E+04, 2.484348E+04, 
   2.489939E+04, 2.495538E+04, 2.501142E+04, 2.506754E+04, 2.512372E+04, 2.517997E+04, 
   2.523628E+04, 2.529266E+04, 2.534910E+04, 2.540562E+04, 2.546220E+04, 2.551884E+04, 
   2.557555E+04, 2.563233E+04, 2.568917E+04, 2.574608E+04, 2.580306E+04, 2.586010E+04, 
   2.591721E+04, 2.597438E+04, 2.603162E+04, 2.608893E+04, 2.614630E+04, 2.620374E+04, 
   2.626125E+04, 2.631882E+04, 2.637646E+04, 2.643416E+04, 2.649193E+04, 2.654977E+04, 
   2.660767E+04, 2.666564E+04, 2.672367E+04, 2.678178E+04, 2.683994E+04, 2.689818E+04, 
   2.695648E+04, 2.701484E+04, 2.707328E+04, 2.713177E+04, 2.719034E+04, 2.724897E+04, 
   2.730767E+04, 2.736643E+04, 2.742526E+04, 2.748416E+04, 2.754312E+04, 2.760215E+04, 
   2.766124E+04, 
])
# ============================ M = 5, I = 6 ============================
M = 5
I = 6
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.220898E+01, 5.082776E+01, 9.732970E+01, 1.439092E+02, 1.905089E+02, 2.371180E+02, 
   2.837327E+02, 3.303516E+02, 3.769738E+02, 4.235991E+02, 4.702271E+02, 5.168577E+02, 
   5.634907E+02, 6.101261E+02, 6.567639E+02, 7.034039E+02, 7.500463E+02, 7.966909E+02, 
   8.433378E+02, 8.899869E+02, 9.366383E+02, 9.832922E+02, 1.029949E+03, 1.076608E+03, 
   1.123270E+03, 1.169935E+03, 1.216605E+03, 1.263279E+03, 1.309959E+03, 1.356646E+03, 
   1.403341E+03, 1.450046E+03, 1.496762E+03, 1.543492E+03, 1.590239E+03, 1.637004E+03, 
   1.683792E+03, 1.730605E+03, 1.777446E+03, 1.824320E+03, 1.871230E+03, 1.918181E+03, 
   1.965177E+03, 2.012223E+03, 2.059322E+03, 2.106481E+03, 2.153704E+03, 2.200996E+03, 
   2.248362E+03, 2.295807E+03, 2.343338E+03, 2.390958E+03, 2.438674E+03, 2.486490E+03, 
   2.534412E+03, 2.582446E+03, 2.630596E+03, 2.678867E+03, 2.727266E+03, 2.775796E+03, 
   2.824463E+03, 2.873272E+03, 2.922228E+03, 2.971335E+03, 3.020599E+03, 3.070023E+03, 
   3.119613E+03, 3.169372E+03, 3.219306E+03, 3.269418E+03, 3.319712E+03, 3.370192E+03, 
   3.420863E+03, 3.471728E+03, 3.522791E+03, 3.574055E+03, 3.625525E+03, 3.677202E+03, 
   3.729092E+03, 3.781196E+03, 3.833519E+03, 3.886063E+03, 3.938831E+03, 3.991827E+03, 
   4.045052E+03, 4.098510E+03, 4.152203E+03, 4.206133E+03, 4.260305E+03, 4.314718E+03, 
   4.369377E+03, 4.424284E+03, 4.479439E+03, 4.534847E+03, 4.590508E+03, 4.646425E+03, 
   4.702599E+03, 4.759034E+03, 4.815729E+03, 4.872688E+03, 4.929912E+03, 4.987402E+03, 
   5.045161E+03, 5.103190E+03, 5.161489E+03, 5.220062E+03, 5.278909E+03, 5.338032E+03, 
   5.397431E+03, 5.457109E+03, 5.517067E+03, 5.577305E+03, 5.637826E+03, 5.698630E+03, 
   5.759718E+03, 5.821092E+03, 5.882752E+03, 5.944700E+03, 6.006937E+03, 6.069463E+03, 
   6.132280E+03, 6.195389E+03, 6.258790E+03, 6.322484E+03, 6.386472E+03, 6.450756E+03, 
   6.515335E+03, 6.580211E+03, 6.645385E+03, 6.710856E+03, 6.776627E+03, 6.842697E+03, 
   6.909068E+03, 6.975739E+03, 7.042712E+03, 7.109988E+03, 7.177566E+03, 7.245448E+03, 
   7.313634E+03, 7.382125E+03, 7.450922E+03, 7.520024E+03, 7.589432E+03, 7.659148E+03, 
   7.729171E+03, 7.799502E+03, 7.870141E+03, 7.941090E+03, 8.012348E+03, 8.083916E+03, 
   8.155794E+03, 8.227983E+03, 8.300483E+03, 8.373295E+03, 8.446420E+03, 8.519856E+03, 
   8.593606E+03, 8.667669E+03, 8.742045E+03, 8.816736E+03, 8.891741E+03, 8.967061E+03, 
   9.042696E+03, 9.118646E+03, 9.194913E+03, 9.271495E+03, 9.348394E+03, 9.425610E+03, 
   9.503144E+03, 9.580994E+03, 9.659162E+03, 9.737649E+03, 9.816454E+03, 9.895577E+03, 
   9.975020E+03, 1.005478E+04, 1.013486E+04, 1.021526E+04, 1.029599E+04, 1.037703E+04, 
   1.045839E+04, 1.054007E+04, 1.062208E+04, 1.070440E+04, 1.078705E+04, 1.087002E+04, 
   1.095331E+04, 1.103692E+04, 1.112086E+04, 1.120512E+04, 1.128970E+04, 1.137460E+04, 
   1.145983E+04, 1.154538E+04, 1.163126E+04, 1.171746E+04, 1.180398E+04, 1.189083E+04, 
   1.197801E+04, 1.206550E+04, 1.215333E+04, 1.224148E+04, 1.232995E+04, 1.241875E+04, 
   1.250788E+04, 1.259733E+04, 1.268711E+04, 1.277721E+04, 1.286765E+04, 1.295841E+04, 
   1.304949E+04, 1.314090E+04, 1.323264E+04, 1.332471E+04, 1.341711E+04, 1.350983E+04, 
   1.360288E+04, 1.369626E+04, 1.378997E+04, 1.388401E+04, 1.397837E+04, 1.407307E+04, 
   1.416809E+04, 1.426345E+04, 1.435913E+04, 1.445514E+04, 1.455148E+04, 1.464815E+04, 
   1.474515E+04, 1.484249E+04, 1.494015E+04, 1.503814E+04, 1.513646E+04, 1.523512E+04, 
   1.533410E+04, 1.543342E+04, 1.553307E+04, 1.563305E+04, 1.573336E+04, 1.583400E+04, 
   1.593497E+04, 1.603628E+04, 1.613792E+04, 1.623989E+04, 1.634219E+04, 1.644483E+04, 
   1.654779E+04, 1.665109E+04, 1.675473E+04, 1.685869E+04, 1.696300E+04, 1.706763E+04, 
   1.717260E+04, 1.727790E+04, 1.738353E+04, 1.748950E+04, 1.759580E+04, 1.770244E+04, 
   1.780941E+04, 1.791672E+04, 1.802436E+04, 1.813233E+04, 1.824064E+04, 1.834929E+04, 
   1.845827E+04, 1.856758E+04, 1.867723E+04, 1.878722E+04, 1.889754E+04, 1.900820E+04, 
   1.911919E+04, 1.923052E+04, 1.934219E+04, 1.945419E+04, 1.956653E+04, 1.967920E+04, 
   1.979221E+04, 1.990556E+04, 2.001925E+04, 2.013327E+04, 2.024763E+04, 2.036232E+04, 
   2.047736E+04, 2.059273E+04, 2.070844E+04, 2.082449E+04, 2.094087E+04, 2.105760E+04, 
   2.117466E+04, 2.129206E+04, 2.140979E+04, 2.152787E+04, 2.164629E+04, 2.176504E+04, 
   2.188413E+04, 2.200356E+04, 2.212333E+04, 2.224345E+04, 2.236389E+04, 2.248468E+04, 
   2.260581E+04, 2.272728E+04, 2.284909E+04, 2.297124E+04, 2.309373E+04, 2.321655E+04, 
   2.333972E+04, 2.346323E+04, 2.358708E+04, 2.371127E+04, 2.383580E+04, 2.396068E+04, 
   2.408589E+04, 2.421144E+04, 2.433734E+04, 2.446358E+04, 2.459015E+04, 2.471707E+04, 
   2.484434E+04, 2.497194E+04, 2.509989E+04, 2.522817E+04, 2.535680E+04, 2.548578E+04, 
   2.561509E+04, 2.574475E+04, 2.587475E+04, 2.600509E+04, 2.613578E+04, 2.626681E+04, 
   2.639818E+04, 2.652989E+04, 2.666195E+04, 2.679435E+04, 2.692710E+04, 2.706019E+04, 
   2.719362E+04, 2.732740E+04, 2.746152E+04, 2.759599E+04, 2.773080E+04, 2.786595E+04, 
   2.800145E+04, 2.813730E+04, 2.827348E+04, 2.841002E+04, 2.854690E+04, 2.868412E+04, 
   2.882169E+04, 2.895960E+04, 2.909786E+04, 2.923647E+04, 2.937542E+04, 2.951471E+04, 
   2.965436E+04, 2.979434E+04, 2.993468E+04, 3.007536E+04, 3.021639E+04, 3.035776E+04, 
   3.049948E+04, 3.064155E+04, 3.078396E+04, 3.092672E+04, 3.106983E+04, 3.121329E+04, 
   3.135709E+04, 3.150124E+04, 3.164574E+04, 3.179058E+04, 3.193578E+04, 3.208132E+04, 
   3.222721E+04, 3.237344E+04, 3.252003E+04, 3.266696E+04, 3.281424E+04, 3.296187E+04, 
   3.310985E+04, 3.325818E+04, 3.340686E+04, 3.355588E+04, 3.370526E+04, 3.385498E+04, 
   3.400506E+04, 3.415548E+04, 3.430625E+04, 3.445738E+04, 3.460885E+04, 3.476067E+04, 
   3.491284E+04, 3.506537E+04, 3.521824E+04, 3.537146E+04, 3.552504E+04, 3.567896E+04, 
   3.583324E+04, 3.598786E+04, 3.614284E+04, 3.629817E+04, 3.645385E+04, 3.660988E+04, 
   3.676626E+04, 3.692299E+04, 3.708008E+04, 3.723752E+04, 3.739530E+04, 3.755345E+04, 
   3.771194E+04, 3.787078E+04, 3.802998E+04, 3.818953E+04, 3.834943E+04, 3.850969E+04, 
   3.867030E+04, 3.883126E+04, 3.899257E+04, 3.915424E+04, 3.931626E+04, 3.947863E+04, 
   3.964136E+04, 3.980444E+04, 3.996788E+04, 4.013167E+04, 4.029581E+04, 4.046031E+04, 
   4.062516E+04, 4.079036E+04, 4.095592E+04, 4.112183E+04, 4.128810E+04, 4.145473E+04, 
   4.162170E+04, 4.178904E+04, 4.195673E+04, 4.212477E+04, 4.229317E+04, 4.246192E+04, 
   4.263103E+04, 4.280050E+04, 4.297032E+04, 4.314049E+04, 4.331103E+04, 4.348192E+04, 
   4.365316E+04, 4.382476E+04, 4.399672E+04, 4.416904E+04, 4.434171E+04, 4.451473E+04, 
   4.468812E+04, 4.486186E+04, 4.503596E+04, 4.521042E+04, 4.538523E+04, 4.556040E+04, 
   4.573593E+04, 4.591182E+04, 4.608806E+04, 4.626466E+04, 4.644162E+04, 4.661894E+04, 
   4.679662E+04, 4.697465E+04, 4.715304E+04, 4.733179E+04, 4.751091E+04, 4.769037E+04, 
   4.787020E+04, 4.805039E+04, 4.823094E+04, 4.841184E+04, 4.859311E+04, 4.877473E+04, 
   4.895671E+04, 4.913906E+04, 4.932176E+04, 4.950483E+04, 4.968825E+04, 4.987203E+04, 
   5.005618E+04, 5.024068E+04, 5.042554E+04, 5.061077E+04, 5.079636E+04, 5.098230E+04, 
   5.116861E+04, 5.135528E+04, 5.154231E+04, 5.172970E+04, 5.191745E+04, 5.210557E+04, 
   5.229404E+04, 5.248288E+04, 5.267208E+04, 5.286164E+04, 5.305156E+04, 5.324185E+04, 
   5.343249E+04, 5.362350E+04, 5.381488E+04, 5.400661E+04, 5.419871E+04, 5.439117E+04, 
   5.458399E+04, 5.477718E+04, 5.497073E+04, 5.516464E+04, 5.535892E+04, 5.555355E+04, 
   5.574856E+04, 5.594392E+04, 5.613965E+04, 5.633575E+04, 5.653221E+04, 5.672903E+04, 
   5.692621E+04, 5.712376E+04, 5.732168E+04, 5.751996E+04, 5.771860E+04, 5.791761E+04, 
   5.811699E+04, 5.831672E+04, 5.851683E+04, 5.871730E+04, 5.891813E+04, 5.911933E+04, 
   5.932090E+04, 5.952283E+04, 5.972512E+04, 5.992779E+04, 6.013081E+04, 6.033421E+04, 
   6.053797E+04, 6.074209E+04, 6.094659E+04, 6.115145E+04, 6.135667E+04, 6.156226E+04, 
   6.176822E+04, 6.197455E+04, 6.218124E+04, 6.238830E+04, 6.259573E+04, 6.280352E+04, 
   6.301168E+04, 6.322021E+04, 6.342911E+04, 6.363837E+04, 6.384801E+04, 6.405801E+04, 
   6.426837E+04, 6.447911E+04, 6.469022E+04, 6.490169E+04, 6.511353E+04, 6.532574E+04, 
   6.553832E+04, 6.575126E+04, 6.596458E+04, 6.617826E+04, 6.639232E+04, 6.660674E+04, 
   6.682153E+04, 6.703669E+04, 6.725222E+04, 6.746812E+04, 6.768439E+04, 6.790103E+04, 
   6.811804E+04, 6.833542E+04, 6.855317E+04, 6.877129E+04, 6.898978E+04, 6.920864E+04, 
   6.942787E+04, 6.964747E+04, 6.986744E+04, 7.008778E+04, 7.030849E+04, 7.052957E+04, 
   7.075103E+04, 7.097285E+04, 7.119505E+04, 7.141762E+04, 7.164056E+04, 7.186387E+04, 
   7.208755E+04, 7.231160E+04, 7.253603E+04, 7.276083E+04, 7.298599E+04, 7.321154E+04, 
   7.343745E+04, 7.366373E+04, 7.389039E+04, 7.411742E+04, 7.434483E+04, 7.457260E+04, 
   7.480075E+04, 7.502927E+04, 7.525816E+04, 7.548743E+04, 7.571707E+04, 7.594708E+04, 
   7.617747E+04, 7.640823E+04, 7.663936E+04, 7.687087E+04, 7.710275E+04, 7.733500E+04, 
   7.756763E+04, 7.780063E+04, 7.803400E+04, 7.826775E+04, 7.850187E+04, 7.873637E+04, 
   7.897124E+04, 7.920649E+04, 7.944211E+04, 7.967810E+04, 7.991447E+04, 8.015122E+04, 
   8.038834E+04, 8.062583E+04, 8.086370E+04, 8.110195E+04, 8.134057E+04, 8.157956E+04, 
   8.181893E+04, 8.205868E+04, 8.229880E+04, 8.253929E+04, 8.278016E+04, 8.302141E+04, 
   8.326304E+04, 8.350504E+04, 8.374741E+04, 8.399016E+04, 8.423329E+04, 8.447680E+04, 
   8.472068E+04, 8.496493E+04, 8.520957E+04, 8.545458E+04, 8.569996E+04, 8.594573E+04, 
   8.619187E+04, 8.643839E+04, 8.668528E+04, 8.693255E+04, 8.718020E+04, 8.742823E+04, 
   8.767663E+04, 8.792541E+04, 8.817457E+04, 8.842410E+04, 8.867401E+04, 8.892431E+04, 
   8.917497E+04, 8.942602E+04, 8.967744E+04, 8.992925E+04, 9.018143E+04, 9.043398E+04, 
   9.068692E+04, 9.094024E+04, 9.119393E+04, 9.144800E+04, 9.170245E+04, 9.195728E+04, 
   9.221249E+04, 9.246807E+04, 9.272404E+04, 9.298038E+04, 9.323710E+04, 9.349421E+04, 
   9.375169E+04, 9.400955E+04, 9.426779E+04, 9.452641E+04, 9.478540E+04, 9.504478E+04, 
   9.530454E+04, 9.556467E+04, 9.582519E+04, 9.608609E+04, 9.634736E+04, 9.660902E+04, 
   9.687105E+04, 9.713347E+04, 9.739626E+04, 9.765944E+04, 9.792299E+04, 9.818693E+04, 
   9.845125E+04, 9.871594E+04, 9.898102E+04, 9.924648E+04, 9.951232E+04, 9.977853E+04, 
   1.000451E+05, 1.003121E+05, 1.005795E+05, 1.008472E+05, 1.011153E+05, 1.013838E+05, 
   1.016527E+05, 1.019220E+05, 1.021916E+05, 1.024617E+05, 1.027321E+05, 1.030029E+05, 
   1.032740E+05, 1.035456E+05, 1.038175E+05, 1.040899E+05, 1.043626E+05, 1.046356E+05, 
   1.049091E+05, 1.051829E+05, 1.054572E+05, 1.057318E+05, 1.060068E+05, 1.062821E+05, 
   1.065579E+05, 1.068340E+05, 1.071105E+05, 1.073874E+05, 1.076647E+05, 1.079424E+05, 
   1.082204E+05, 1.084989E+05, 1.087777E+05, 1.090569E+05, 1.093365E+05, 1.096164E+05, 
   1.098968E+05, 1.101775E+05, 1.104586E+05, 1.107401E+05, 1.110220E+05, 1.113042E+05, 
   1.115869E+05, 1.118699E+05, 1.121533E+05, 1.124371E+05, 1.127213E+05, 1.130058E+05, 
   1.132908E+05, 1.135761E+05, 1.138618E+05, 1.141479E+05, 1.144344E+05, 1.147212E+05, 
   1.150085E+05, 1.152961E+05, 1.155841E+05, 1.158725E+05, 1.161613E+05, 1.164504E+05, 
   1.167400E+05, 1.170299E+05, 1.173202E+05, 1.176109E+05, 1.179020E+05, 1.181934E+05, 
   1.184853E+05, 1.187775E+05, 1.190701E+05, 1.193631E+05, 1.196565E+05, 1.199503E+05, 
   1.202444E+05, 1.205390E+05, 1.208339E+05, 1.211292E+05, 1.214249E+05, 1.217210E+05, 
   1.220174E+05, 1.223143E+05, 1.226115E+05, 1.229091E+05, 1.232071E+05, 1.235055E+05, 
   1.238043E+05, 1.241034E+05, 1.244030E+05, 1.247029E+05, 1.250032E+05, 1.253039E+05, 
   1.256050E+05, 1.259064E+05, 1.262083E+05, 1.265105E+05, 1.268131E+05, 1.271161E+05, 
   1.274195E+05, 1.277233E+05, 1.280275E+05, 1.283320E+05, 1.286369E+05, 1.289422E+05, 
   1.292479E+05, 1.295540E+05, 1.298605E+05, 1.301673E+05, 1.304746E+05, 1.307822E+05, 
   1.310902E+05, 1.313986E+05, 1.317074E+05, 1.320166E+05, 1.323261E+05, 1.326361E+05, 
   1.329464E+05, 1.332571E+05, 1.335682E+05, 1.338797E+05, 1.341915E+05, 1.345038E+05, 
   1.348164E+05, 1.351295E+05, 1.354429E+05, 1.357567E+05, 1.360708E+05, 1.363854E+05, 
   1.367004E+05, 1.370157E+05, 1.373314E+05, 1.376475E+05, 1.379640E+05, 1.382809E+05, 
   1.385982E+05, 1.389158E+05, 1.392339E+05, 1.395523E+05, 1.398711E+05, 1.401903E+05, 
   1.405099E+05, 1.408299E+05, 1.411502E+05, 1.414709E+05, 1.417921E+05, 1.421136E+05, 
   1.424355E+05, 1.427578E+05, 1.430804E+05, 1.434035E+05, 1.437269E+05, 1.440508E+05, 
   1.443750E+05, 1.446996E+05, 1.450246E+05, 1.453499E+05, 1.456757E+05, 1.460019E+05, 
   1.463284E+05, 1.466553E+05, 1.469826E+05, 1.473103E+05, 1.476384E+05, 1.479668E+05, 
   1.482957E+05, 1.486249E+05, 1.489545E+05, 1.492846E+05, 1.496149E+05, 1.499457E+05, 
   1.502769E+05, 1.506084E+05, 1.509404E+05, 1.512727E+05, 1.516054E+05, 1.519385E+05, 
   1.522720E+05, 1.526059E+05, 1.529401E+05, 1.532748E+05, 1.536098E+05, 1.539452E+05, 
   1.542810E+05, 1.546172E+05, 1.549538E+05, 1.552907E+05, 1.556281E+05, 1.559658E+05, 
   1.563039E+05, 1.566424E+05, 1.569813E+05, 1.573206E+05, 1.576602E+05, 1.580003E+05, 
   1.583407E+05, 1.586815E+05, 1.590227E+05, 1.593643E+05, 1.597063E+05, 1.600486E+05, 
   1.603914E+05, 
])
# ============================ M = 5, I = 7 ============================
#M = 5
#I = 7
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.018641E+00, 4.287414E+00, 8.214728E+00, 1.214842E+01, 1.608377E+01, 2.001990E+01, 
#   2.395650E+01, 2.789344E+01, 3.183067E+01, 3.576815E+01, 3.970586E+01, 4.364378E+01, 
#   4.758192E+01, 5.152025E+01, 5.545878E+01, 5.939751E+01, 6.333642E+01, 6.727553E+01, 
#   7.121483E+01, 7.515432E+01, 7.909401E+01, 8.303390E+01, 8.697401E+01, 9.091434E+01, 
#   9.485494E+01, 9.879584E+01, 1.027371E+02, 1.066787E+02, 1.106209E+02, 1.145637E+02, 
#   1.185071E+02, 1.224515E+02, 1.263968E+02, 1.303434E+02, 1.342914E+02, 1.382411E+02, 
#   1.421927E+02, 1.461465E+02, 1.501028E+02, 1.540620E+02, 1.580244E+02, 1.619903E+02, 
#   1.659601E+02, 1.699343E+02, 1.739131E+02, 1.778971E+02, 1.818867E+02, 1.858823E+02, 
#   1.898844E+02, 1.938933E+02, 1.979096E+02, 2.019336E+02, 2.059660E+02, 2.100070E+02, 
#   2.140572E+02, 2.181170E+02, 2.221869E+02, 2.262673E+02, 2.303586E+02, 2.344613E+02, 
#   2.385758E+02, 2.427025E+02, 2.468418E+02, 2.509942E+02, 2.551601E+02, 2.593397E+02, 
#   2.635336E+02, 2.677421E+02, 2.719655E+02, 2.762042E+02, 2.804586E+02, 2.847290E+02, 
#   2.890157E+02, 2.933191E+02, 2.976394E+02, 3.019770E+02, 3.063322E+02, 3.107052E+02, 
#   3.150964E+02, 3.195060E+02, 3.239342E+02, 3.283814E+02, 3.328478E+02, 3.373337E+02, 
#   3.418391E+02, 3.463645E+02, 3.509100E+02, 3.554759E+02, 3.600623E+02, 3.646694E+02, 
#   3.692974E+02, 3.739466E+02, 3.786172E+02, 3.833092E+02, 3.880229E+02, 3.927584E+02, 
#   3.975159E+02, 4.022956E+02, 4.070976E+02, 4.119221E+02, 4.167692E+02, 4.216390E+02, 
#   4.265318E+02, 4.314475E+02, 4.363864E+02, 4.413486E+02, 4.463342E+02, 4.513432E+02, 
#   4.563759E+02, 4.614324E+02, 4.665126E+02, 4.716169E+02, 4.767451E+02, 4.818976E+02, 
#   4.870742E+02, 4.922752E+02, 4.975006E+02, 5.027506E+02, 5.080251E+02, 5.133243E+02, 
#   5.186483E+02, 5.239971E+02, 5.293708E+02, 5.347695E+02, 5.401933E+02, 5.456422E+02, 
#   5.511163E+02, 5.566157E+02, 5.621404E+02, 5.676905E+02, 5.732661E+02, 5.788672E+02, 
#   5.844938E+02, 5.901461E+02, 5.958240E+02, 6.015277E+02, 6.072572E+02, 6.130126E+02, 
#   6.187938E+02, 6.246010E+02, 6.304342E+02, 6.362934E+02, 6.421787E+02, 6.480901E+02, 
#   6.540277E+02, 6.599916E+02, 6.659816E+02, 6.719980E+02, 6.780407E+02, 6.841098E+02, 
#   6.902053E+02, 6.963272E+02, 7.024757E+02, 7.086506E+02, 7.148522E+02, 7.210803E+02, 
#   7.273350E+02, 7.336164E+02, 7.399245E+02, 7.462593E+02, 7.526209E+02, 7.590092E+02, 
#   7.654244E+02, 7.718664E+02, 7.783353E+02, 7.848311E+02, 7.913538E+02, 7.979035E+02, 
#   8.044801E+02, 8.110838E+02, 8.177145E+02, 8.243722E+02, 8.310571E+02, 8.377690E+02, 
#   8.445081E+02, 8.512743E+02, 8.580677E+02, 8.648884E+02, 8.717362E+02, 8.786113E+02, 
#   8.855137E+02, 8.924433E+02, 8.994003E+02, 9.063846E+02, 9.133963E+02, 9.204353E+02, 
#   9.275017E+02, 9.345956E+02, 9.417168E+02, 9.488656E+02, 9.560418E+02, 9.632455E+02, 
#   9.704767E+02, 9.777354E+02, 9.850217E+02, 9.923356E+02, 9.996770E+02, 1.007046E+03, 
#   1.014443E+03, 1.021867E+03, 1.029319E+03, 1.036798E+03, 1.044306E+03, 1.051841E+03, 
#   1.059403E+03, 1.066994E+03, 1.074612E+03, 1.082258E+03, 1.089932E+03, 1.097633E+03, 
#   1.105363E+03, 1.113120E+03, 1.120905E+03, 1.128718E+03, 1.136558E+03, 1.144427E+03, 
#   1.152323E+03, 1.160248E+03, 1.168200E+03, 1.176181E+03, 1.184189E+03, 1.192225E+03, 
#   1.200289E+03, 1.208381E+03, 1.216502E+03, 1.224650E+03, 1.232826E+03, 1.241030E+03, 
#   1.249263E+03, 1.257523E+03, 1.265812E+03, 1.274128E+03, 1.282473E+03, 1.290846E+03, 
#   1.299247E+03, 1.307676E+03, 1.316134E+03, 1.324619E+03, 1.333133E+03, 1.341675E+03, 
#   1.350245E+03, 1.358843E+03, 1.367470E+03, 1.376125E+03, 1.384808E+03, 1.393519E+03, 
#   1.402259E+03, 1.411027E+03, 1.419823E+03, 1.428648E+03, 1.437501E+03, 1.446382E+03, 
#   1.455292E+03, 1.464230E+03, 1.473196E+03, 1.482191E+03, 1.491214E+03, 1.500266E+03, 
#   1.509346E+03, 1.518455E+03, 1.527592E+03, 1.536757E+03, 1.545951E+03, 1.555173E+03, 
#   1.564424E+03, 1.573703E+03, 1.583011E+03, 1.592348E+03, 1.601713E+03, 1.611106E+03, 
#   1.620528E+03, 1.629979E+03, 1.639458E+03, 1.648966E+03, 1.658502E+03, 1.668067E+03, 
#   1.677661E+03, 1.687283E+03, 1.696934E+03, 1.706613E+03, 1.716321E+03, 1.726058E+03, 
#   1.735824E+03, 1.745618E+03, 1.755441E+03, 1.765293E+03, 1.775173E+03, 1.785082E+03, 
#   1.795020E+03, 1.804987E+03, 1.814982E+03, 1.825007E+03, 1.835060E+03, 1.845141E+03, 
#   1.855252E+03, 1.865392E+03, 1.875560E+03, 1.885757E+03, 1.895983E+03, 1.906238E+03, 
#   1.916521E+03, 1.926834E+03, 1.937175E+03, 1.947546E+03, 1.957945E+03, 1.968373E+03, 
#   1.978830E+03, 1.989317E+03, 1.999832E+03, 2.010376E+03, 2.020949E+03, 2.031551E+03, 
#   2.042182E+03, 2.052841E+03, 2.063530E+03, 2.074248E+03, 2.084995E+03, 2.095772E+03, 
#   2.106577E+03, 2.117411E+03, 2.128274E+03, 2.139167E+03, 2.150088E+03, 2.161039E+03, 
#   2.172018E+03, 2.183027E+03, 2.194065E+03, 2.205132E+03, 2.216228E+03, 2.227354E+03, 
#   2.238508E+03, 2.249692E+03, 2.260905E+03, 2.272147E+03, 2.283418E+03, 2.294719E+03, 
#   2.306048E+03, 2.317408E+03, 2.328796E+03, 2.340213E+03, 2.351660E+03, 2.363136E+03, 
#   2.374642E+03, 2.386176E+03, 2.397740E+03, 2.409334E+03, 2.420956E+03, 2.432608E+03, 
#   2.444290E+03, 2.456000E+03, 2.467740E+03, 2.479510E+03, 2.491309E+03, 2.503137E+03, 
#   2.514994E+03, 2.526882E+03, 2.538798E+03, 2.550744E+03, 2.562719E+03, 2.574724E+03, 
#   2.586758E+03, 2.598822E+03, 2.610915E+03, 2.623038E+03, 2.635190E+03, 2.647372E+03, 
#   2.659584E+03, 2.671824E+03, 2.684095E+03, 2.696395E+03, 2.708724E+03, 2.721083E+03, 
#   2.733472E+03, 2.745890E+03, 2.758338E+03, 2.770815E+03, 2.783322E+03, 2.795859E+03, 
#   2.808426E+03, 2.821022E+03, 2.833647E+03, 2.846303E+03, 2.858988E+03, 2.871702E+03, 
#   2.884447E+03, 2.897221E+03, 2.910025E+03, 2.922858E+03, 2.935722E+03, 2.948615E+03, 
#   2.961537E+03, 2.974490E+03, 2.987472E+03, 3.000484E+03, 3.013526E+03, 3.026598E+03, 
#   3.039700E+03, 3.052831E+03, 3.065992E+03, 3.079183E+03, 3.092404E+03, 3.105655E+03, 
#   3.118936E+03, 3.132246E+03, 3.145587E+03, 3.158957E+03, 3.172357E+03, 3.185788E+03, 
#   3.199248E+03, 3.212738E+03, 3.226258E+03, 3.239808E+03, 3.253388E+03, 3.266998E+03, 
#   3.280638E+03, 3.294308E+03, 3.308007E+03, 3.321737E+03, 3.335497E+03, 3.349287E+03, 
#   3.363107E+03, 3.376958E+03, 3.390838E+03, 3.404748E+03, 3.418688E+03, 3.432659E+03, 
#   3.446659E+03, 3.460690E+03, 3.474751E+03, 3.488842E+03, 3.502963E+03, 3.517114E+03, 
#   3.531296E+03, 3.545507E+03, 3.559749E+03, 3.574021E+03, 3.588323E+03, 3.602655E+03, 
#   3.617018E+03, 3.631411E+03, 3.645834E+03, 3.660287E+03, 3.674771E+03, 3.689284E+03, 
#   3.703829E+03, 3.718403E+03, 3.733008E+03, 3.747643E+03, 3.762308E+03, 3.777004E+03, 
#   3.791730E+03, 3.806486E+03, 3.821273E+03, 3.836090E+03, 3.850937E+03, 3.865815E+03, 
#   3.880723E+03, 3.895662E+03, 3.910631E+03, 3.925630E+03, 3.940660E+03, 3.955720E+03, 
#   3.970811E+03, 3.985932E+03, 4.001084E+03, 4.016266E+03, 4.031479E+03, 4.046722E+03, 
#   4.061996E+03, 4.077300E+03, 4.092635E+03, 4.108000E+03, 4.123396E+03, 4.138822E+03, 
#   4.154279E+03, 4.169766E+03, 4.185284E+03, 4.200833E+03, 4.216412E+03, 4.232022E+03, 
#   4.247663E+03, 4.263334E+03, 4.279035E+03, 4.294768E+03, 4.310531E+03, 4.326325E+03, 
#   4.342149E+03, 4.358004E+03, 4.373890E+03, 4.389806E+03, 4.405753E+03, 4.421731E+03, 
#   4.437740E+03, 4.453779E+03, 4.469850E+03, 4.485950E+03, 4.502082E+03, 4.518245E+03, 
#   4.534438E+03, 4.550662E+03, 4.566917E+03, 4.583202E+03, 4.599519E+03, 4.615866E+03, 
#   4.632244E+03, 4.648653E+03, 4.665093E+03, 4.681564E+03, 4.698065E+03, 4.714598E+03, 
#   4.731161E+03, 4.747756E+03, 4.764381E+03, 4.781037E+03, 4.797724E+03, 4.814442E+03, 
#   4.831191E+03, 4.847971E+03, 4.864782E+03, 4.881623E+03, 4.898496E+03, 4.915400E+03, 
#   4.932335E+03, 4.949301E+03, 4.966298E+03, 4.983326E+03, 5.000385E+03, 5.017475E+03, 
#   5.034596E+03, 5.051748E+03, 5.068931E+03, 5.086145E+03, 5.103391E+03, 5.120667E+03, 
#   5.137975E+03, 5.155314E+03, 5.172683E+03, 5.190084E+03, 5.207516E+03, 5.224980E+03, 
#   5.242474E+03, 5.260000E+03, 5.277557E+03, 5.295145E+03, 5.312764E+03, 5.330414E+03, 
#   5.348096E+03, 5.365809E+03, 5.383553E+03, 5.401328E+03, 5.419135E+03, 5.436972E+03, 
#   5.454842E+03, 5.472742E+03, 5.490674E+03, 5.508636E+03, 5.526631E+03, 5.544656E+03, 
#   5.562713E+03, 5.580801E+03, 5.598921E+03, 5.617072E+03, 5.635254E+03, 5.653467E+03, 
#   5.671712E+03, 5.689989E+03, 5.708296E+03, 5.726636E+03, 5.745006E+03, 5.763408E+03, 
#   5.781841E+03, 5.800306E+03, 5.818802E+03, 5.837330E+03, 5.855889E+03, 5.874480E+03, 
#   5.893102E+03, 5.911755E+03, 5.930440E+03, 5.949156E+03, 5.967904E+03, 5.986684E+03, 
#   6.005495E+03, 6.024337E+03, 6.043211E+03, 6.062117E+03, 6.081054E+03, 6.100023E+03, 
#   6.119023E+03, 6.138055E+03, 6.157118E+03, 6.176213E+03, 6.195340E+03, 6.214498E+03, 
#   6.233688E+03, 6.252909E+03, 6.272162E+03, 6.291447E+03, 6.310763E+03, 6.330111E+03, 
#   6.349490E+03, 6.368902E+03, 6.388345E+03, 6.407819E+03, 6.427325E+03, 6.446863E+03, 
#   6.466433E+03, 6.486035E+03, 6.505668E+03, 6.525332E+03, 6.545029E+03, 6.564757E+03, 
#   6.584517E+03, 6.604309E+03, 6.624133E+03, 6.643988E+03, 6.663875E+03, 6.683794E+03, 
#   6.703745E+03, 6.723727E+03, 6.743741E+03, 6.763787E+03, 6.783865E+03, 6.803975E+03, 
#   6.824117E+03, 6.844290E+03, 6.864495E+03, 6.884732E+03, 6.905001E+03, 6.925302E+03, 
#   6.945635E+03, 6.965999E+03, 6.986396E+03, 7.006824E+03, 7.027284E+03, 7.047777E+03, 
#   7.068301E+03, 7.088857E+03, 7.109445E+03, 7.130064E+03, 7.150716E+03, 7.171400E+03, 
#   7.192116E+03, 7.212863E+03, 7.233643E+03, 7.254455E+03, 7.275298E+03, 7.296174E+03, 
#   7.317081E+03, 7.338021E+03, 7.358993E+03, 7.379996E+03, 7.401032E+03, 7.422099E+03, 
#   7.443199E+03, 7.464331E+03, 7.485495E+03, 7.506690E+03, 7.527918E+03, 7.549178E+03, 
#   7.570470E+03, 7.591794E+03, 7.613151E+03, 7.634539E+03, 7.655959E+03, 7.677412E+03, 
#   7.698896E+03, 7.720413E+03, 7.741962E+03, 7.763543E+03, 7.785156E+03, 7.806801E+03, 
#   7.828478E+03, 7.850188E+03, 7.871929E+03, 7.893703E+03, 7.915509E+03, 7.937347E+03, 
#   7.959218E+03, 7.981120E+03, 8.003055E+03, 8.025022E+03, 8.047021E+03, 8.069052E+03, 
#   8.091116E+03, 8.113211E+03, 8.135339E+03, 8.157499E+03, 8.179692E+03, 8.201916E+03, 
#   8.224173E+03, 8.246462E+03, 8.268784E+03, 8.291137E+03, 8.313523E+03, 8.335941E+03, 
#   8.358392E+03, 8.380875E+03, 8.403390E+03, 8.425937E+03, 8.448517E+03, 8.471128E+03, 
#   8.493773E+03, 8.516449E+03, 8.539158E+03, 8.561899E+03, 8.584673E+03, 8.607478E+03, 
#   8.630317E+03, 8.653187E+03, 8.676090E+03, 8.699025E+03, 8.721993E+03, 8.744993E+03, 
#   8.768025E+03, 8.791089E+03, 8.814186E+03, 8.837316E+03, 8.860478E+03, 8.883672E+03, 
#   8.906898E+03, 8.930157E+03, 8.953448E+03, 8.976772E+03, 9.000128E+03, 9.023517E+03, 
#   9.046938E+03, 9.070391E+03, 9.093877E+03, 9.117395E+03, 9.140946E+03, 9.164529E+03, 
#   9.188144E+03, 9.211792E+03, 9.235473E+03, 9.259186E+03, 9.282931E+03, 9.306709E+03, 
#   9.330519E+03, 9.354362E+03, 9.378237E+03, 9.402145E+03, 9.426085E+03, 9.450058E+03, 
#   9.474063E+03, 9.498100E+03, 9.522171E+03, 9.546273E+03, 9.570408E+03, 9.594576E+03, 
#   9.618776E+03, 9.643009E+03, 9.667274E+03, 9.691572E+03, 9.715902E+03, 9.740265E+03, 
#   9.764660E+03, 9.789088E+03, 9.813548E+03, 9.838041E+03, 9.862566E+03, 9.887124E+03, 
#   9.911715E+03, 9.936338E+03, 9.960994E+03, 9.985682E+03, 1.001040E+04, 1.003516E+04, 
#   1.005994E+04, 1.008476E+04, 1.010961E+04, 1.013449E+04, 1.015941E+04, 1.018436E+04, 
#   1.020934E+04, 1.023435E+04, 1.025940E+04, 1.028448E+04, 1.030959E+04, 1.033474E+04, 
#   1.035991E+04, 1.038512E+04, 1.041037E+04, 1.043564E+04, 1.046095E+04, 1.048629E+04, 
#   1.051166E+04, 1.053706E+04, 1.056250E+04, 1.058797E+04, 1.061348E+04, 1.063901E+04, 
#   1.066458E+04, 1.069018E+04, 1.071581E+04, 1.074148E+04, 1.076718E+04, 1.079291E+04, 
#   1.081867E+04, 1.084447E+04, 1.087030E+04, 1.089616E+04, 1.092206E+04, 1.094799E+04, 
#   1.097395E+04, 1.099994E+04, 1.102596E+04, 1.105202E+04, 1.107811E+04, 1.110423E+04, 
#   1.113039E+04, 1.115658E+04, 1.118280E+04, 1.120905E+04, 1.123534E+04, 1.126166E+04, 
#   1.128801E+04, 1.131440E+04, 1.134081E+04, 1.136726E+04, 1.139375E+04, 1.142026E+04, 
#   1.144681E+04, 1.147339E+04, 1.150000E+04, 1.152665E+04, 1.155333E+04, 1.158004E+04, 
#   1.160678E+04, 1.163356E+04, 1.166037E+04, 1.168721E+04, 1.171409E+04, 1.174099E+04, 
#   1.176793E+04, 1.179491E+04, 1.182191E+04, 1.184895E+04, 1.187602E+04, 1.190313E+04, 
#   1.193026E+04, 1.195743E+04, 1.198463E+04, 1.201187E+04, 1.203913E+04, 1.206643E+04, 
#   1.209377E+04, 1.212113E+04, 1.214853E+04, 1.217596E+04, 1.220342E+04, 1.223092E+04, 
#   1.225845E+04, 1.228601E+04, 1.231360E+04, 1.234123E+04, 1.236889E+04, 1.239658E+04, 
#   1.242431E+04, 1.245206E+04, 1.247985E+04, 1.250768E+04, 1.253553E+04, 1.256342E+04, 
#   1.259134E+04, 1.261929E+04, 1.264728E+04, 1.267530E+04, 1.270335E+04, 1.273144E+04, 
#   1.275955E+04, 1.278770E+04, 1.281589E+04, 1.284410E+04, 1.287235E+04, 1.290063E+04, 
#   1.292894E+04, 1.295729E+04, 1.298567E+04, 1.301408E+04, 1.304252E+04, 1.307100E+04, 
#   1.309951E+04, 1.312805E+04, 1.315662E+04, 1.318523E+04, 1.321387E+04, 1.324254E+04, 
#   1.327125E+04, 1.329999E+04, 1.332876E+04, 1.335756E+04, 1.338640E+04, 1.341527E+04, 
#   1.344417E+04, 1.347310E+04, 1.350207E+04, 1.353107E+04, 1.356010E+04, 1.358917E+04, 
#   1.361826E+04, 
#])
# ============================ M = 5, I = 8 ============================
#M = 5
#I = 8
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.024263E+00, 4.501758E+00, 8.644905E+00, 1.279408E+01, 1.694486E+01, 2.109639E+01, 
#   2.524838E+01, 2.940072E+01, 3.355335E+01, 3.770623E+01, 4.185935E+01, 4.601270E+01, 
#   5.016626E+01, 5.432003E+01, 5.847401E+01, 6.262819E+01, 6.678257E+01, 7.093716E+01, 
#   7.509194E+01, 7.924693E+01, 8.340213E+01, 8.755755E+01, 9.171321E+01, 9.586913E+01, 
#   1.000254E+02, 1.041819E+02, 1.083389E+02, 1.124964E+02, 1.166545E+02, 1.208133E+02, 
#   1.249731E+02, 1.291339E+02, 1.332960E+02, 1.374595E+02, 1.416249E+02, 1.457923E+02, 
#   1.499621E+02, 1.541346E+02, 1.583101E+02, 1.624891E+02, 1.666720E+02, 1.708591E+02, 
#   1.750509E+02, 1.792479E+02, 1.834505E+02, 1.876593E+02, 1.918746E+02, 1.960970E+02, 
#   2.003270E+02, 2.045651E+02, 2.088118E+02, 2.130675E+02, 2.173329E+02, 2.216084E+02, 
#   2.258944E+02, 2.301916E+02, 2.345004E+02, 2.388212E+02, 2.431546E+02, 2.475010E+02, 
#   2.518609E+02, 2.562347E+02, 2.606229E+02, 2.650259E+02, 2.694442E+02, 2.738782E+02, 
#   2.783282E+02, 2.827947E+02, 2.872781E+02, 2.917787E+02, 2.962969E+02, 3.008331E+02, 
#   3.053877E+02, 3.099609E+02, 3.145531E+02, 3.191646E+02, 3.237958E+02, 3.284468E+02, 
#   3.331182E+02, 3.378100E+02, 3.425226E+02, 3.472563E+02, 3.520113E+02, 3.567879E+02, 
#   3.615863E+02, 3.664068E+02, 3.712495E+02, 3.761148E+02, 3.810028E+02, 3.859137E+02, 
#   3.908478E+02, 3.958052E+02, 4.007862E+02, 4.057909E+02, 4.108194E+02, 4.158720E+02, 
#   4.209489E+02, 4.260502E+02, 4.311760E+02, 4.363265E+02, 4.415019E+02, 4.467023E+02, 
#   4.519278E+02, 4.571786E+02, 4.624548E+02, 4.677566E+02, 4.730840E+02, 4.784372E+02, 
#   4.838163E+02, 4.892213E+02, 4.946525E+02, 5.001100E+02, 5.055937E+02, 5.111039E+02, 
#   5.166406E+02, 5.222039E+02, 5.277939E+02, 5.334108E+02, 5.390545E+02, 5.447252E+02, 
#   5.504229E+02, 5.561477E+02, 5.618998E+02, 5.676792E+02, 5.734859E+02, 5.793200E+02, 
#   5.851816E+02, 5.910708E+02, 5.969876E+02, 6.029321E+02, 6.089044E+02, 6.149045E+02, 
#   6.209324E+02, 6.269883E+02, 6.330722E+02, 6.391841E+02, 6.453241E+02, 6.514923E+02, 
#   6.576887E+02, 6.639133E+02, 6.701662E+02, 6.764475E+02, 6.827571E+02, 6.890952E+02, 
#   6.954618E+02, 7.018569E+02, 7.082806E+02, 7.147328E+02, 7.212138E+02, 7.277234E+02, 
#   7.342617E+02, 7.408288E+02, 7.474247E+02, 7.540495E+02, 7.607031E+02, 7.673856E+02, 
#   7.740971E+02, 7.808375E+02, 7.876070E+02, 7.944055E+02, 8.012330E+02, 8.080897E+02, 
#   8.149755E+02, 8.218905E+02, 8.288346E+02, 8.358080E+02, 8.428107E+02, 8.498426E+02, 
#   8.569038E+02, 8.639943E+02, 8.711142E+02, 8.782635E+02, 8.854422E+02, 8.926503E+02, 
#   8.998879E+02, 9.071550E+02, 9.144515E+02, 9.217777E+02, 9.291333E+02, 9.365185E+02, 
#   9.439334E+02, 9.513778E+02, 9.588519E+02, 9.663557E+02, 9.738891E+02, 9.814523E+02, 
#   9.890452E+02, 9.966678E+02, 1.004320E+03, 1.012002E+03, 1.019714E+03, 1.027456E+03, 
#   1.035228E+03, 1.043029E+03, 1.050861E+03, 1.058722E+03, 1.066613E+03, 1.074535E+03, 
#   1.082486E+03, 1.090467E+03, 1.098478E+03, 1.106519E+03, 1.114590E+03, 1.122691E+03, 
#   1.130822E+03, 1.138984E+03, 1.147175E+03, 1.155397E+03, 1.163648E+03, 1.171930E+03, 
#   1.180242E+03, 1.188584E+03, 1.196956E+03, 1.205359E+03, 1.213792E+03, 1.222254E+03, 
#   1.230748E+03, 1.239271E+03, 1.247825E+03, 1.256409E+03, 1.265023E+03, 1.273668E+03, 
#   1.282343E+03, 1.291048E+03, 1.299784E+03, 1.308550E+03, 1.317346E+03, 1.326173E+03, 
#   1.335031E+03, 1.343918E+03, 1.352836E+03, 1.361785E+03, 1.370764E+03, 1.379774E+03, 
#   1.388814E+03, 1.397885E+03, 1.406986E+03, 1.416117E+03, 1.425280E+03, 1.434472E+03, 
#   1.443696E+03, 1.452950E+03, 1.462234E+03, 1.471550E+03, 1.480895E+03, 1.490272E+03, 
#   1.499679E+03, 1.509117E+03, 1.518585E+03, 1.528084E+03, 1.537614E+03, 1.547175E+03, 
#   1.556766E+03, 1.566388E+03, 1.576041E+03, 1.585725E+03, 1.595439E+03, 1.605184E+03, 
#   1.614960E+03, 1.624767E+03, 1.634605E+03, 1.644473E+03, 1.654372E+03, 1.664303E+03, 
#   1.674264E+03, 1.684256E+03, 1.694278E+03, 1.704332E+03, 1.714417E+03, 1.724532E+03, 
#   1.734679E+03, 1.744856E+03, 1.755065E+03, 1.765304E+03, 1.775574E+03, 1.785876E+03, 
#   1.796208E+03, 1.806572E+03, 1.816966E+03, 1.827392E+03, 1.837848E+03, 1.848336E+03, 
#   1.858855E+03, 1.869404E+03, 1.879985E+03, 1.890597E+03, 1.901240E+03, 1.911915E+03, 
#   1.922620E+03, 1.933357E+03, 1.944124E+03, 1.954923E+03, 1.965753E+03, 1.976615E+03, 
#   1.987507E+03, 1.998431E+03, 2.009386E+03, 2.020372E+03, 2.031390E+03, 2.042438E+03, 
#   2.053519E+03, 2.064630E+03, 2.075772E+03, 2.086946E+03, 2.098152E+03, 2.109388E+03, 
#   2.120656E+03, 2.131955E+03, 2.143286E+03, 2.154648E+03, 2.166041E+03, 2.177466E+03, 
#   2.188922E+03, 2.200410E+03, 2.211929E+03, 2.223479E+03, 2.235061E+03, 2.246675E+03, 
#   2.258320E+03, 2.269996E+03, 2.281704E+03, 2.293443E+03, 2.305214E+03, 2.317016E+03, 
#   2.328850E+03, 2.340716E+03, 2.352613E+03, 2.364541E+03, 2.376502E+03, 2.388493E+03, 
#   2.400517E+03, 2.412572E+03, 2.424658E+03, 2.436776E+03, 2.448926E+03, 2.461108E+03, 
#   2.473321E+03, 2.485566E+03, 2.497842E+03, 2.510150E+03, 2.522490E+03, 2.534862E+03, 
#   2.547265E+03, 2.559700E+03, 2.572167E+03, 2.584666E+03, 2.597196E+03, 2.609758E+03, 
#   2.622352E+03, 2.634978E+03, 2.647635E+03, 2.660325E+03, 2.673046E+03, 2.685799E+03, 
#   2.698584E+03, 2.711401E+03, 2.724249E+03, 2.737130E+03, 2.750042E+03, 2.762986E+03, 
#   2.775962E+03, 2.788971E+03, 2.802011E+03, 2.815083E+03, 2.828187E+03, 2.841323E+03, 
#   2.854490E+03, 2.867690E+03, 2.880922E+03, 2.894186E+03, 2.907482E+03, 2.920810E+03, 
#   2.934170E+03, 2.947562E+03, 2.960986E+03, 2.974442E+03, 2.987930E+03, 3.001451E+03, 
#   3.015003E+03, 3.028588E+03, 3.042204E+03, 3.055853E+03, 3.069534E+03, 3.083247E+03, 
#   3.096992E+03, 3.110770E+03, 3.124579E+03, 3.138421E+03, 3.152295E+03, 3.166201E+03, 
#   3.180140E+03, 3.194110E+03, 3.208113E+03, 3.222149E+03, 3.236216E+03, 3.250316E+03, 
#   3.264448E+03, 3.278612E+03, 3.292809E+03, 3.307038E+03, 3.321299E+03, 3.335593E+03, 
#   3.349918E+03, 3.364277E+03, 3.378667E+03, 3.393091E+03, 3.407546E+03, 3.422034E+03, 
#   3.436554E+03, 3.451107E+03, 3.465692E+03, 3.480309E+03, 3.494959E+03, 3.509642E+03, 
#   3.524357E+03, 3.539104E+03, 3.553884E+03, 3.568697E+03, 3.583542E+03, 3.598419E+03, 
#   3.613329E+03, 3.628272E+03, 3.643247E+03, 3.658254E+03, 3.673295E+03, 3.688367E+03, 
#   3.703473E+03, 3.718611E+03, 3.733781E+03, 3.748985E+03, 3.764221E+03, 3.779489E+03, 
#   3.794790E+03, 3.810124E+03, 3.825491E+03, 3.840890E+03, 3.856322E+03, 3.871786E+03, 
#   3.887284E+03, 3.902814E+03, 3.918377E+03, 3.933972E+03, 3.949600E+03, 3.965261E+03, 
#   3.980955E+03, 3.996682E+03, 4.012441E+03, 4.028234E+03, 4.044059E+03, 4.059917E+03, 
#   4.075807E+03, 4.091731E+03, 4.107687E+03, 4.123676E+03, 4.139699E+03, 4.155754E+03, 
#   4.171842E+03, 4.187962E+03, 4.204116E+03, 4.220303E+03, 4.236523E+03, 4.252775E+03, 
#   4.269061E+03, 4.285379E+03, 4.301731E+03, 4.318115E+03, 4.334532E+03, 4.350983E+03, 
#   4.367466E+03, 4.383983E+03, 4.400532E+03, 4.417115E+03, 4.433731E+03, 4.450379E+03, 
#   4.467061E+03, 4.483776E+03, 4.500524E+03, 4.517305E+03, 4.534119E+03, 4.550966E+03, 
#   4.567847E+03, 4.584760E+03, 4.601707E+03, 4.618687E+03, 4.635700E+03, 4.652746E+03, 
#   4.669825E+03, 4.686938E+03, 4.704084E+03, 4.721263E+03, 4.738475E+03, 4.755720E+03, 
#   4.772999E+03, 4.790311E+03, 4.807656E+03, 4.825035E+03, 4.842447E+03, 4.859892E+03, 
#   4.877370E+03, 4.894882E+03, 4.912427E+03, 4.930005E+03, 4.947617E+03, 4.965262E+03, 
#   4.982940E+03, 5.000652E+03, 5.018397E+03, 5.036176E+03, 5.053988E+03, 5.071833E+03, 
#   5.089712E+03, 5.107624E+03, 5.125570E+03, 5.143549E+03, 5.161561E+03, 5.179607E+03, 
#   5.197686E+03, 5.215799E+03, 5.233946E+03, 5.252126E+03, 5.270339E+03, 5.288586E+03, 
#   5.306866E+03, 5.325180E+03, 5.343528E+03, 5.361909E+03, 5.380323E+03, 5.398772E+03, 
#   5.417253E+03, 5.435769E+03, 5.454318E+03, 5.472900E+03, 5.491516E+03, 5.510166E+03, 
#   5.528849E+03, 5.547566E+03, 5.566317E+03, 5.585101E+03, 5.603919E+03, 5.622771E+03, 
#   5.641656E+03, 5.660575E+03, 5.679528E+03, 5.698515E+03, 5.717535E+03, 5.736589E+03, 
#   5.755676E+03, 5.774798E+03, 5.793953E+03, 5.813142E+03, 5.832364E+03, 5.851621E+03, 
#   5.870911E+03, 5.890235E+03, 5.909592E+03, 5.928984E+03, 5.948409E+03, 5.967869E+03, 
#   5.987362E+03, 6.006889E+03, 6.026449E+03, 6.046044E+03, 6.065672E+03, 6.085335E+03, 
#   6.105031E+03, 6.124761E+03, 6.144525E+03, 6.164323E+03, 6.184155E+03, 6.204020E+03, 
#   6.223920E+03, 6.243854E+03, 6.263821E+03, 6.283823E+03, 6.303858E+03, 6.323927E+03, 
#   6.344031E+03, 6.364168E+03, 6.384340E+03, 6.404545E+03, 6.424784E+03, 6.445058E+03, 
#   6.465365E+03, 6.485707E+03, 6.506082E+03, 6.526491E+03, 6.546935E+03, 6.567413E+03, 
#   6.587924E+03, 6.608470E+03, 6.629050E+03, 6.649664E+03, 6.670312E+03, 6.690994E+03, 
#   6.711710E+03, 6.732461E+03, 6.753245E+03, 6.774064E+03, 6.794916E+03, 6.815803E+03, 
#   6.836724E+03, 6.857679E+03, 6.878669E+03, 6.899692E+03, 6.920750E+03, 6.941842E+03, 
#   6.962968E+03, 6.984128E+03, 7.005323E+03, 7.026551E+03, 7.047814E+03, 7.069112E+03, 
#   7.090443E+03, 7.111809E+03, 7.133208E+03, 7.154643E+03, 7.176111E+03, 7.197614E+03, 
#   7.219151E+03, 7.240722E+03, 7.262327E+03, 7.283967E+03, 7.305641E+03, 7.327350E+03, 
#   7.349092E+03, 7.370869E+03, 7.392681E+03, 7.414526E+03, 7.436407E+03, 7.458321E+03, 
#   7.480270E+03, 7.502253E+03, 7.524270E+03, 7.546322E+03, 7.568408E+03, 7.590529E+03, 
#   7.612684E+03, 7.634873E+03, 7.657097E+03, 7.679355E+03, 7.701647E+03, 7.723974E+03, 
#   7.746336E+03, 7.768732E+03, 7.791162E+03, 7.813627E+03, 7.836126E+03, 7.858659E+03, 
#   7.881227E+03, 7.903830E+03, 7.926467E+03, 7.949138E+03, 7.971844E+03, 7.994584E+03, 
#   8.017359E+03, 8.040169E+03, 8.063013E+03, 8.085891E+03, 8.108804E+03, 8.131751E+03, 
#   8.154733E+03, 8.177750E+03, 8.200801E+03, 8.223886E+03, 8.247006E+03, 8.270161E+03, 
#   8.293350E+03, 8.316573E+03, 8.339832E+03, 8.363125E+03, 8.386452E+03, 8.409814E+03, 
#   8.433210E+03, 8.456641E+03, 8.480107E+03, 8.503607E+03, 8.527142E+03, 8.550712E+03, 
#   8.574316E+03, 8.597955E+03, 8.621628E+03, 8.645336E+03, 8.669078E+03, 8.692856E+03, 
#   8.716667E+03, 8.740514E+03, 8.764395E+03, 8.788311E+03, 8.812261E+03, 8.836246E+03, 
#   8.860266E+03, 8.884320E+03, 8.908409E+03, 8.932533E+03, 8.956691E+03, 8.980884E+03, 
#   9.005112E+03, 9.029374E+03, 9.053671E+03, 9.078003E+03, 9.102369E+03, 9.126771E+03, 
#   9.151206E+03, 9.175677E+03, 9.200182E+03, 9.224722E+03, 9.249297E+03, 9.273906E+03, 
#   9.298551E+03, 9.323229E+03, 9.347943E+03, 9.372691E+03, 9.397474E+03, 9.422292E+03, 
#   9.447145E+03, 9.472032E+03, 9.496954E+03, 9.521911E+03, 9.546902E+03, 9.571929E+03, 
#   9.596990E+03, 9.622086E+03, 9.647216E+03, 9.672382E+03, 9.697582E+03, 9.722817E+03, 
#   9.748087E+03, 9.773391E+03, 9.798730E+03, 9.824104E+03, 9.849513E+03, 9.874957E+03, 
#   9.900435E+03, 9.925948E+03, 9.951496E+03, 9.977079E+03, 1.000270E+04, 1.002835E+04, 
#   1.005404E+04, 1.007976E+04, 1.010552E+04, 1.013131E+04, 1.015713E+04, 1.018300E+04, 
#   1.020889E+04, 1.023482E+04, 1.026079E+04, 1.028679E+04, 1.031282E+04, 1.033889E+04, 
#   1.036500E+04, 1.039114E+04, 1.041731E+04, 1.044352E+04, 1.046977E+04, 1.049605E+04, 
#   1.052236E+04, 1.054871E+04, 1.057509E+04, 1.060151E+04, 1.062797E+04, 1.065445E+04, 
#   1.068098E+04, 1.070754E+04, 1.073413E+04, 1.076076E+04, 1.078742E+04, 1.081412E+04, 
#   1.084085E+04, 1.086762E+04, 1.089442E+04, 1.092126E+04, 1.094813E+04, 1.097503E+04, 
#   1.100198E+04, 1.102895E+04, 1.105596E+04, 1.108301E+04, 1.111009E+04, 1.113721E+04, 
#   1.116436E+04, 1.119154E+04, 1.121876E+04, 1.124602E+04, 1.127331E+04, 1.130063E+04, 
#   1.132799E+04, 1.135539E+04, 1.138282E+04, 1.141028E+04, 1.143778E+04, 1.146532E+04, 
#   1.149289E+04, 1.152049E+04, 1.154813E+04, 1.157580E+04, 1.160351E+04, 1.163126E+04, 
#   1.165903E+04, 1.168685E+04, 1.171470E+04, 1.174258E+04, 1.177050E+04, 1.179845E+04, 
#   1.182644E+04, 1.185446E+04, 1.188252E+04, 1.191061E+04, 1.193874E+04, 1.196690E+04, 
#   1.199510E+04, 1.202333E+04, 1.205159E+04, 1.207990E+04, 1.210823E+04, 1.213660E+04, 
#   1.216501E+04, 1.219345E+04, 1.222193E+04, 1.225044E+04, 1.227898E+04, 1.230756E+04, 
#   1.233618E+04, 1.236483E+04, 1.239351E+04, 1.242223E+04, 1.245099E+04, 1.247978E+04, 
#   1.250860E+04, 1.253746E+04, 1.256636E+04, 1.259529E+04, 1.262425E+04, 1.265325E+04, 
#   1.268228E+04, 1.271135E+04, 1.274045E+04, 1.276959E+04, 1.279876E+04, 1.282797E+04, 
#   1.285722E+04, 1.288649E+04, 1.291581E+04, 1.294515E+04, 1.297453E+04, 1.300395E+04, 
#   1.303340E+04, 1.306289E+04, 1.309241E+04, 1.312197E+04, 1.315156E+04, 1.318118E+04, 
#   1.321084E+04, 1.324054E+04, 1.327027E+04, 1.330004E+04, 1.332983E+04, 1.335967E+04, 
#   1.338954E+04, 1.341944E+04, 1.344938E+04, 1.347936E+04, 1.350936E+04, 1.353941E+04, 
#   1.356949E+04, 1.359960E+04, 1.362975E+04, 1.365993E+04, 1.369015E+04, 1.372040E+04, 
#   1.375069E+04, 1.378101E+04, 1.381136E+04, 1.384176E+04, 1.387218E+04, 1.390264E+04, 
#   1.393314E+04, 1.396367E+04, 1.399423E+04, 1.402483E+04, 1.405547E+04, 1.408614E+04, 
#   1.411684E+04, 1.414758E+04, 1.417835E+04, 1.420916E+04, 1.424000E+04, 1.427088E+04, 
#   1.430179E+04, 1.433274E+04, 1.436372E+04, 1.439474E+04, 1.442579E+04, 1.445687E+04, 
#   1.448799E+04, 1.451915E+04, 1.455034E+04, 1.458156E+04, 1.461282E+04, 1.464412E+04, 
#   1.467545E+04, 
#])
# ============================ M = 5, I = 9 ============================
#M = 5
#I = 9
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   6.128633E+00, 2.638957E+01, 5.062329E+01, 7.489416E+01, 9.917483E+01, 1.234600E+02, 
#   1.477480E+02, 1.720382E+02, 1.963300E+02, 2.206233E+02, 2.449180E+02, 2.692141E+02, 
#   2.935114E+02, 3.178100E+02, 3.421098E+02, 3.664108E+02, 3.907129E+02, 4.150162E+02, 
#   4.393208E+02, 4.636265E+02, 4.879334E+02, 5.122416E+02, 5.365511E+02, 5.608621E+02, 
#   5.851749E+02, 6.094895E+02, 6.338065E+02, 6.581262E+02, 6.824492E+02, 7.067762E+02, 
#   7.311080E+02, 7.554457E+02, 7.797902E+02, 8.041429E+02, 8.285052E+02, 8.528787E+02, 
#   8.772651E+02, 9.016662E+02, 9.260840E+02, 9.505208E+02, 9.749786E+02, 9.994600E+02, 
#   1.023967E+03, 1.048503E+03, 1.073070E+03, 1.097670E+03, 1.122307E+03, 1.146984E+03, 
#   1.171702E+03, 1.196466E+03, 1.221277E+03, 1.246140E+03, 1.271055E+03, 1.296028E+03, 
#   1.321059E+03, 1.346153E+03, 1.371312E+03, 1.396539E+03, 1.421836E+03, 1.447207E+03, 
#   1.472653E+03, 1.498178E+03, 1.523784E+03, 1.549474E+03, 1.575250E+03, 1.601115E+03, 
#   1.627071E+03, 1.653120E+03, 1.679265E+03, 1.705507E+03, 1.731850E+03, 1.758294E+03, 
#   1.784843E+03, 1.811497E+03, 1.838260E+03, 1.865133E+03, 1.892117E+03, 1.919215E+03, 
#   1.946428E+03, 1.973759E+03, 2.001207E+03, 2.028776E+03, 2.056467E+03, 2.084281E+03, 
#   2.112219E+03, 2.140284E+03, 2.168475E+03, 2.196796E+03, 2.225246E+03, 2.253828E+03, 
#   2.282542E+03, 2.311390E+03, 2.340373E+03, 2.369491E+03, 2.398746E+03, 2.428139E+03, 
#   2.457671E+03, 2.487343E+03, 2.517156E+03, 2.547110E+03, 2.577207E+03, 2.607447E+03, 
#   2.637832E+03, 2.668362E+03, 2.699037E+03, 2.729860E+03, 2.760829E+03, 2.791947E+03, 
#   2.823213E+03, 2.854629E+03, 2.886195E+03, 2.917911E+03, 2.949779E+03, 2.981799E+03, 
#   3.013971E+03, 3.046296E+03, 3.078775E+03, 3.111408E+03, 3.144196E+03, 3.177138E+03, 
#   3.210237E+03, 3.243491E+03, 3.276902E+03, 3.310470E+03, 3.344196E+03, 3.378079E+03, 
#   3.412121E+03, 3.446321E+03, 3.480680E+03, 3.515199E+03, 3.549878E+03, 3.584717E+03, 
#   3.619716E+03, 3.654876E+03, 3.690198E+03, 3.725681E+03, 3.761326E+03, 3.797133E+03, 
#   3.833102E+03, 3.869235E+03, 3.905530E+03, 3.941989E+03, 3.978611E+03, 4.015397E+03, 
#   4.052348E+03, 4.089462E+03, 4.126742E+03, 4.164186E+03, 4.201796E+03, 4.239571E+03, 
#   4.277511E+03, 4.315618E+03, 4.353890E+03, 4.392329E+03, 4.430934E+03, 4.469706E+03, 
#   4.508645E+03, 4.547751E+03, 4.587024E+03, 4.626464E+03, 4.666073E+03, 4.705849E+03, 
#   4.745793E+03, 4.785906E+03, 4.826187E+03, 4.866636E+03, 4.907254E+03, 4.948041E+03, 
#   4.988997E+03, 5.030123E+03, 5.071417E+03, 5.112881E+03, 5.154515E+03, 5.196319E+03, 
#   5.238293E+03, 5.280437E+03, 5.322751E+03, 5.365235E+03, 5.407890E+03, 5.450716E+03, 
#   5.493712E+03, 5.536879E+03, 5.580218E+03, 5.623727E+03, 5.667408E+03, 5.711261E+03, 
#   5.755285E+03, 5.799480E+03, 5.843848E+03, 5.888387E+03, 5.933099E+03, 5.977982E+03, 
#   6.023038E+03, 6.068266E+03, 6.113667E+03, 6.159240E+03, 6.204986E+03, 6.250905E+03, 
#   6.296996E+03, 6.343261E+03, 6.389699E+03, 6.436310E+03, 6.483094E+03, 6.530052E+03, 
#   6.577183E+03, 6.624488E+03, 6.671967E+03, 6.719619E+03, 6.767446E+03, 6.815446E+03, 
#   6.863621E+03, 6.911969E+03, 6.960492E+03, 7.009189E+03, 7.058061E+03, 7.107108E+03, 
#   7.156329E+03, 7.205724E+03, 7.255295E+03, 7.305040E+03, 7.354961E+03, 7.405056E+03, 
#   7.455327E+03, 7.505772E+03, 7.556394E+03, 7.607190E+03, 7.658162E+03, 7.709310E+03, 
#   7.760633E+03, 7.812132E+03, 7.863807E+03, 7.915658E+03, 7.967684E+03, 8.019887E+03, 
#   8.072266E+03, 8.124821E+03, 8.177552E+03, 8.230460E+03, 8.283544E+03, 8.336805E+03, 
#   8.390242E+03, 8.443856E+03, 8.497647E+03, 8.551614E+03, 8.605758E+03, 8.660080E+03, 
#   8.714578E+03, 8.769253E+03, 8.824106E+03, 8.879136E+03, 8.934343E+03, 8.989727E+03, 
#   9.045289E+03, 9.101029E+03, 9.156946E+03, 9.213041E+03, 9.269313E+03, 9.325764E+03, 
#   9.382392E+03, 9.439198E+03, 9.496182E+03, 9.553344E+03, 9.610685E+03, 9.668204E+03, 
#   9.725901E+03, 9.783776E+03, 9.841830E+03, 9.900062E+03, 9.958473E+03, 1.001706E+04, 
#   1.007583E+04, 1.013478E+04, 1.019390E+04, 1.025321E+04, 1.031269E+04, 1.037235E+04, 
#   1.043220E+04, 1.049222E+04, 1.055242E+04, 1.061280E+04, 1.067336E+04, 1.073409E+04, 
#   1.079501E+04, 1.085611E+04, 1.091739E+04, 1.097884E+04, 1.104048E+04, 1.110230E+04, 
#   1.116429E+04, 1.122647E+04, 1.128882E+04, 1.135136E+04, 1.141408E+04, 1.147697E+04, 
#   1.154005E+04, 1.160331E+04, 1.166674E+04, 1.173036E+04, 1.179416E+04, 1.185814E+04, 
#   1.192230E+04, 1.198664E+04, 1.205116E+04, 1.211586E+04, 1.218075E+04, 1.224581E+04, 
#   1.231106E+04, 1.237648E+04, 1.244209E+04, 1.250788E+04, 1.257385E+04, 1.264000E+04, 
#   1.270633E+04, 1.277284E+04, 1.283954E+04, 1.290642E+04, 1.297348E+04, 1.304072E+04, 
#   1.310814E+04, 1.317574E+04, 1.324353E+04, 1.331150E+04, 1.337965E+04, 1.344798E+04, 
#   1.351649E+04, 1.358519E+04, 1.365407E+04, 1.372313E+04, 1.379237E+04, 1.386180E+04, 
#   1.393140E+04, 1.400120E+04, 1.407117E+04, 1.414133E+04, 1.421166E+04, 1.428219E+04, 
#   1.435289E+04, 1.442378E+04, 1.449485E+04, 1.456610E+04, 1.463754E+04, 1.470916E+04, 
#   1.478096E+04, 1.485295E+04, 1.492512E+04, 1.499747E+04, 1.507001E+04, 1.514273E+04, 
#   1.521563E+04, 1.528872E+04, 1.536199E+04, 1.543545E+04, 1.550909E+04, 1.558291E+04, 
#   1.565692E+04, 1.573111E+04, 1.580548E+04, 1.588004E+04, 1.595479E+04, 1.602971E+04, 
#   1.610483E+04, 1.618012E+04, 1.625560E+04, 1.633127E+04, 1.640712E+04, 1.648315E+04, 
#   1.655937E+04, 1.663578E+04, 1.671237E+04, 1.678914E+04, 1.686610E+04, 1.694325E+04, 
#   1.702057E+04, 1.709809E+04, 1.717579E+04, 1.725367E+04, 1.733174E+04, 1.741000E+04, 
#   1.748844E+04, 1.756706E+04, 1.764588E+04, 1.772487E+04, 1.780406E+04, 1.788343E+04, 
#   1.796298E+04, 1.804272E+04, 1.812265E+04, 1.820276E+04, 1.828306E+04, 1.836354E+04, 
#   1.844421E+04, 1.852507E+04, 1.860611E+04, 1.868734E+04, 1.876876E+04, 1.885036E+04, 
#   1.893215E+04, 1.901413E+04, 1.909629E+04, 1.917864E+04, 1.926117E+04, 1.934390E+04, 
#   1.942680E+04, 1.950990E+04, 1.959318E+04, 1.967665E+04, 1.976031E+04, 1.984416E+04, 
#   1.992819E+04, 2.001241E+04, 2.009681E+04, 2.018141E+04, 2.026619E+04, 2.035116E+04, 
#   2.043631E+04, 2.052166E+04, 2.060719E+04, 2.069291E+04, 2.077882E+04, 2.086491E+04, 
#   2.095120E+04, 2.103767E+04, 2.112433E+04, 2.121118E+04, 2.129821E+04, 2.138544E+04, 
#   2.147285E+04, 2.156045E+04, 2.164824E+04, 2.173622E+04, 2.182438E+04, 2.191274E+04, 
#   2.200128E+04, 2.209001E+04, 2.217893E+04, 2.226804E+04, 2.235734E+04, 2.244683E+04, 
#   2.253651E+04, 2.262637E+04, 2.271643E+04, 2.280667E+04, 2.289711E+04, 2.298773E+04, 
#   2.307854E+04, 2.316954E+04, 2.326073E+04, 2.335211E+04, 2.344368E+04, 2.353544E+04, 
#   2.362739E+04, 2.371953E+04, 2.381186E+04, 2.390438E+04, 2.399709E+04, 2.408999E+04, 
#   2.418308E+04, 2.427636E+04, 2.436983E+04, 2.446349E+04, 2.455734E+04, 2.465138E+04, 
#   2.474561E+04, 2.484003E+04, 2.493464E+04, 2.502945E+04, 2.512444E+04, 2.521962E+04, 
#   2.531500E+04, 2.541056E+04, 2.550632E+04, 2.560227E+04, 2.569841E+04, 2.579474E+04, 
#   2.589126E+04, 2.598797E+04, 2.608487E+04, 2.618197E+04, 2.627925E+04, 2.637673E+04, 
#   2.647440E+04, 2.657226E+04, 2.667031E+04, 2.676856E+04, 2.686699E+04, 2.696562E+04, 
#   2.706444E+04, 2.716345E+04, 2.726265E+04, 2.736204E+04, 2.746163E+04, 2.756141E+04, 
#   2.766138E+04, 2.776154E+04, 2.786190E+04, 2.796245E+04, 2.806319E+04, 2.816412E+04, 
#   2.826524E+04, 2.836656E+04, 2.846807E+04, 2.856977E+04, 2.867167E+04, 2.877376E+04, 
#   2.887604E+04, 2.897851E+04, 2.908118E+04, 2.918404E+04, 2.928709E+04, 2.939034E+04, 
#   2.949378E+04, 2.959741E+04, 2.970124E+04, 2.980525E+04, 2.990947E+04, 3.001387E+04, 
#   3.011847E+04, 3.022326E+04, 3.032825E+04, 3.043343E+04, 3.053880E+04, 3.064437E+04, 
#   3.075013E+04, 3.085609E+04, 3.096224E+04, 3.106858E+04, 3.117512E+04, 3.128185E+04, 
#   3.138877E+04, 3.149589E+04, 3.160321E+04, 3.171072E+04, 3.181842E+04, 3.192631E+04, 
#   3.203441E+04, 3.214269E+04, 3.225117E+04, 3.235985E+04, 3.246872E+04, 3.257778E+04, 
#   3.268704E+04, 3.279649E+04, 3.290614E+04, 3.301598E+04, 3.312602E+04, 3.323626E+04, 
#   3.334669E+04, 3.345731E+04, 3.356813E+04, 3.367914E+04, 3.379035E+04, 3.390176E+04, 
#   3.401336E+04, 3.412515E+04, 3.423714E+04, 3.434933E+04, 3.446171E+04, 3.457429E+04, 
#   3.468706E+04, 3.480003E+04, 3.491320E+04, 3.502656E+04, 3.514011E+04, 3.525387E+04, 
#   3.536782E+04, 3.548196E+04, 3.559630E+04, 3.571084E+04, 3.582557E+04, 3.594050E+04, 
#   3.605563E+04, 3.617095E+04, 3.628647E+04, 3.640218E+04, 3.651809E+04, 3.663420E+04, 
#   3.675050E+04, 3.686700E+04, 3.698370E+04, 3.710060E+04, 3.721769E+04, 3.733497E+04, 
#   3.745246E+04, 3.757014E+04, 3.768802E+04, 3.780609E+04, 3.792437E+04, 3.804283E+04, 
#   3.816150E+04, 3.828036E+04, 3.839943E+04, 3.851868E+04, 3.863814E+04, 3.875779E+04, 
#   3.887764E+04, 3.899769E+04, 3.911794E+04, 3.923838E+04, 3.935902E+04, 3.947986E+04, 
#   3.960089E+04, 3.972212E+04, 3.984356E+04, 3.996518E+04, 4.008701E+04, 4.020904E+04, 
#   4.033126E+04, 4.045368E+04, 4.057630E+04, 4.069911E+04, 4.082213E+04, 4.094534E+04, 
#   4.106875E+04, 4.119236E+04, 4.131617E+04, 4.144017E+04, 4.156438E+04, 4.168878E+04, 
#   4.181338E+04, 4.193818E+04, 4.206318E+04, 4.218837E+04, 4.231377E+04, 4.243936E+04, 
#   4.256515E+04, 4.269115E+04, 4.281734E+04, 4.294372E+04, 4.307031E+04, 4.319710E+04, 
#   4.332408E+04, 4.345127E+04, 4.357865E+04, 4.370623E+04, 4.383401E+04, 4.396199E+04, 
#   4.409017E+04, 4.421855E+04, 4.434713E+04, 4.447591E+04, 4.460488E+04, 4.473406E+04, 
#   4.486344E+04, 4.499301E+04, 4.512278E+04, 4.525276E+04, 4.538293E+04, 4.551330E+04, 
#   4.564388E+04, 4.577465E+04, 4.590562E+04, 4.603679E+04, 4.616816E+04, 4.629973E+04, 
#   4.643150E+04, 4.656347E+04, 4.669564E+04, 4.682801E+04, 4.696058E+04, 4.709335E+04, 
#   4.722632E+04, 4.735949E+04, 4.749286E+04, 4.762643E+04, 4.776021E+04, 4.789418E+04, 
#   4.802835E+04, 4.816272E+04, 4.829729E+04, 4.843206E+04, 4.856703E+04, 4.870221E+04, 
#   4.883758E+04, 4.897315E+04, 4.910893E+04, 4.924490E+04, 4.938108E+04, 4.951745E+04, 
#   4.965403E+04, 4.979080E+04, 4.992778E+04, 5.006496E+04, 5.020234E+04, 5.033992E+04, 
#   5.047770E+04, 5.061568E+04, 5.075386E+04, 5.089225E+04, 5.103083E+04, 5.116961E+04, 
#   5.130860E+04, 5.144779E+04, 5.158717E+04, 5.172676E+04, 5.186655E+04, 5.200654E+04, 
#   5.214674E+04, 5.228713E+04, 5.242772E+04, 5.256852E+04, 5.270951E+04, 5.285071E+04, 
#   5.299211E+04, 5.313371E+04, 5.327551E+04, 5.341752E+04, 5.355972E+04, 5.370213E+04, 
#   5.384474E+04, 5.398754E+04, 5.413056E+04, 5.427377E+04, 5.441718E+04, 5.456080E+04, 
#   5.470461E+04, 5.484863E+04, 5.499285E+04, 5.513727E+04, 5.528189E+04, 5.542672E+04, 
#   5.557175E+04, 5.571697E+04, 5.586240E+04, 5.600804E+04, 5.615387E+04, 5.629991E+04, 
#   5.644614E+04, 5.659258E+04, 5.673922E+04, 5.688607E+04, 5.703311E+04, 5.718036E+04, 
#   5.732781E+04, 5.747546E+04, 5.762331E+04, 5.777136E+04, 5.791962E+04, 5.806808E+04, 
#   5.821674E+04, 5.836560E+04, 5.851467E+04, 5.866394E+04, 5.881341E+04, 5.896308E+04, 
#   5.911295E+04, 5.926303E+04, 5.941331E+04, 5.956379E+04, 5.971447E+04, 5.986535E+04, 
#   6.001644E+04, 6.016773E+04, 6.031922E+04, 6.047092E+04, 6.062281E+04, 6.077491E+04, 
#   6.092721E+04, 6.107972E+04, 6.123242E+04, 6.138533E+04, 6.153844E+04, 6.169176E+04, 
#   6.184527E+04, 6.199899E+04, 6.215291E+04, 6.230704E+04, 6.246136E+04, 6.261589E+04, 
#   6.277062E+04, 6.292555E+04, 6.308069E+04, 6.323603E+04, 6.339157E+04, 6.354732E+04, 
#   6.370326E+04, 6.385941E+04, 6.401576E+04, 6.417232E+04, 6.432907E+04, 6.448603E+04, 
#   6.464320E+04, 6.480056E+04, 6.495813E+04, 6.511590E+04, 6.527387E+04, 6.543205E+04, 
#   6.559043E+04, 6.574901E+04, 6.590779E+04, 6.606678E+04, 6.622597E+04, 6.638536E+04, 
#   6.654496E+04, 6.670475E+04, 6.686476E+04, 6.702496E+04, 6.718537E+04, 6.734597E+04, 
#   6.750679E+04, 6.766780E+04, 6.782902E+04, 6.799044E+04, 6.815206E+04, 6.831389E+04, 
#   6.847592E+04, 6.863815E+04, 6.880058E+04, 6.896322E+04, 6.912606E+04, 6.928911E+04, 
#   6.945235E+04, 6.961580E+04, 6.977945E+04, 6.994331E+04, 7.010736E+04, 7.027162E+04, 
#   7.043609E+04, 7.060075E+04, 7.076562E+04, 7.093070E+04, 7.109597E+04, 7.126145E+04, 
#   7.142713E+04, 7.159301E+04, 7.175910E+04, 7.192539E+04, 7.209188E+04, 7.225857E+04, 
#   7.242547E+04, 7.259257E+04, 7.275988E+04, 7.292738E+04, 7.309509E+04, 7.326301E+04, 
#   7.343112E+04, 7.359944E+04, 7.376796E+04, 7.393668E+04, 7.410561E+04, 7.427474E+04, 
#   7.444407E+04, 7.461361E+04, 7.478335E+04, 7.495329E+04, 7.512343E+04, 7.529378E+04, 
#   7.546433E+04, 7.563508E+04, 7.580603E+04, 7.597719E+04, 7.614855E+04, 7.632012E+04, 
#   7.649188E+04, 7.666385E+04, 7.683602E+04, 7.700840E+04, 7.718098E+04, 7.735376E+04, 
#   7.752674E+04, 7.769992E+04, 7.787331E+04, 7.804690E+04, 7.822070E+04, 7.839470E+04, 
#   7.856889E+04, 7.874330E+04, 7.891790E+04, 7.909271E+04, 7.926772E+04, 7.944293E+04, 
#   7.961835E+04, 7.979397E+04, 7.996979E+04, 8.014581E+04, 8.032204E+04, 8.049846E+04, 
#   8.067510E+04, 8.085193E+04, 8.102897E+04, 8.120620E+04, 8.138365E+04, 8.156129E+04, 
#   8.173914E+04, 8.191719E+04, 8.209544E+04, 8.227389E+04, 8.245255E+04, 8.263141E+04, 
#   8.281047E+04, 8.298973E+04, 8.316920E+04, 8.334887E+04, 8.352874E+04, 8.370881E+04, 
#   8.388909E+04, 8.406956E+04, 8.425025E+04, 8.443113E+04, 8.461221E+04, 8.479350E+04, 
#   8.497499E+04, 
#])
# ============================ M = 6, I = 1 ============================
M = 6
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[1]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.000003E+00, 7.272368E+00, 1.273767E+01, 2.059396E+01, 3.049704E+01, 4.199408E+01, 
   5.479199E+01, 6.872307E+01, 8.368420E+01, 9.960328E+01, 1.164244E+02, 1.341017E+02, 
   1.525958E+02, 1.718728E+02, 1.919028E+02, 2.126595E+02, 2.341200E+02, 2.562642E+02, 
   2.790750E+02, 3.025383E+02, 3.266430E+02, 3.513813E+02, 3.767486E+02, 4.027434E+02, 
   4.293678E+02, 4.566272E+02, 4.845303E+02, 5.130891E+02, 5.423188E+02, 5.722376E+02, 
   6.028667E+02, 6.342299E+02, 6.663540E+02, 6.992679E+02, 7.330031E+02, 7.675933E+02, 
   8.030742E+02, 8.394836E+02, 8.768610E+02, 9.152480E+02, 9.546876E+02, 9.952248E+02, 
   1.036906E+03, 1.079779E+03, 1.123895E+03, 1.169303E+03, 1.216057E+03, 1.264211E+03, 
   1.313822E+03, 1.364947E+03, 1.417645E+03, 1.471977E+03, 1.528007E+03, 1.585798E+03, 
   1.645417E+03, 1.706933E+03, 1.770415E+03, 1.835937E+03, 1.903571E+03, 1.973396E+03, 
   2.045490E+03, 2.119933E+03, 2.196810E+03, 2.276206E+03, 2.358210E+03, 2.442914E+03, 
   2.530410E+03, 2.620795E+03, 2.714170E+03, 2.810636E+03, 2.910300E+03, 3.013269E+03, 
   3.119657E+03, 3.229578E+03, 3.343152E+03, 3.460500E+03, 3.581750E+03, 3.707031E+03, 
   3.836476E+03, 3.970225E+03, 4.108418E+03, 4.251203E+03, 4.398729E+03, 4.551153E+03, 
   4.708633E+03, 4.871335E+03, 5.039427E+03, 5.213084E+03, 5.392486E+03, 5.577818E+03, 
   5.769269E+03, 5.967037E+03, 6.171321E+03, 6.382330E+03, 6.600278E+03, 6.825383E+03, 
   7.057872E+03, 7.297977E+03, 7.545937E+03, 7.801998E+03, 8.066413E+03, 8.339441E+03, 
   8.621350E+03, 8.912415E+03, 9.212917E+03, 9.523146E+03, 9.843401E+03, 1.017399E+04, 
   1.051522E+04, 1.086742E+04, 1.123092E+04, 1.160606E+04, 1.199319E+04, 1.239267E+04, 
   1.280487E+04, 1.323015E+04, 1.366892E+04, 1.412156E+04, 1.458849E+04, 1.507012E+04, 
   1.556688E+04, 1.607920E+04, 1.660754E+04, 1.715235E+04, 1.771410E+04, 1.829329E+04, 
   1.889039E+04, 1.950591E+04, 2.014038E+04, 2.079431E+04, 2.146826E+04, 2.216335E+04, 
   2.287987E+04, 2.361843E+04, 2.437969E+04, 2.516428E+04, 2.597287E+04, 2.680616E+04, 
   2.766484E+04, 2.854964E+04, 2.946131E+04, 3.040060E+04, 3.136829E+04, 3.236518E+04, 
   3.339210E+04, 3.444989E+04, 3.553941E+04, 3.666155E+04, 3.781722E+04, 3.900735E+04, 
   4.023289E+04, 4.149482E+04, 4.279415E+04, 4.413190E+04, 4.550914E+04, 4.692694E+04, 
   4.838640E+04, 4.988867E+04, 5.143491E+04, 5.302631E+04, 5.466408E+04, 5.634949E+04, 
   5.808381E+04, 5.986835E+04, 6.170446E+04, 6.359351E+04, 6.553691E+04, 6.753612E+04, 
   6.959259E+04, 7.170786E+04, 7.388346E+04, 7.612098E+04, 7.842204E+04, 8.078831E+04, 
   8.322148E+04, 8.572329E+04, 8.829553E+04, 9.094001E+04, 9.365859E+04, 9.645319E+04, 
   9.932574E+04, 1.022783E+05, 1.053128E+05, 1.084313E+05, 1.116361E+05, 1.149293E+05, 
   1.183131E+05, 1.217898E+05, 1.253617E+05, 1.290312E+05, 1.328007E+05, 1.366728E+05, 
   1.406500E+05, 1.447348E+05, 1.489299E+05, 1.532380E+05, 1.576620E+05, 1.622045E+05, 
   1.668686E+05, 1.716571E+05, 1.765730E+05, 1.816196E+05, 1.867998E+05, 1.921168E+05, 
   1.975741E+05, 2.031748E+05, 2.089224E+05, 2.148204E+05, 2.208724E+05, 2.270819E+05, 
   2.334526E+05, 2.399884E+05, 2.466930E+05, 2.535704E+05, 2.606246E+05, 2.678596E+05, 
   2.752797E+05, 2.828890E+05, 2.906919E+05, 2.986928E+05, 3.068962E+05, 3.153067E+05, 
   3.239289E+05, 3.327676E+05, 3.418276E+05, 3.511140E+05, 3.606316E+05, 3.703857E+05, 
   3.803814E+05, 3.906241E+05, 4.011192E+05, 4.118722E+05, 4.228887E+05, 4.341745E+05, 
   4.457354E+05, 4.575772E+05, 4.697061E+05, 4.821281E+05, 4.948496E+05, 5.078768E+05, 
   5.212163E+05, 5.348746E+05, 5.488584E+05, 5.631746E+05, 5.778300E+05, 5.928318E+05, 
   6.081870E+05, 6.239031E+05, 6.399873E+05, 6.564473E+05, 6.732906E+05, 
])
# ============================ M = 6, I = 2 ============================
M = 6
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[1]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.000001E+01, 1.454474E+01, 2.547534E+01, 4.118793E+01, 6.099408E+01, 8.398817E+01, 
   1.095840E+02, 1.374461E+02, 1.673684E+02, 1.992066E+02, 2.328489E+02, 2.682034E+02, 
   3.051916E+02, 3.437455E+02, 3.838055E+02, 4.253190E+02, 4.682400E+02, 5.125284E+02, 
   5.581499E+02, 6.050765E+02, 6.532860E+02, 7.027627E+02, 7.534972E+02, 8.054868E+02, 
   8.587356E+02, 9.132544E+02, 9.690606E+02, 1.026178E+03, 1.084638E+03, 1.144475E+03, 
   1.205733E+03, 1.268460E+03, 1.332708E+03, 1.398536E+03, 1.466006E+03, 1.535187E+03, 
   1.606148E+03, 1.678967E+03, 1.753722E+03, 1.830496E+03, 1.909375E+03, 1.990450E+03, 
   2.073812E+03, 2.159559E+03, 2.247789E+03, 2.338605E+03, 2.432114E+03, 2.528422E+03, 
   2.627644E+03, 2.729893E+03, 2.835289E+03, 2.943954E+03, 3.056013E+03, 3.171595E+03, 
   3.290834E+03, 3.413865E+03, 3.540830E+03, 3.671873E+03, 3.807143E+03, 3.946792E+03, 
   4.090980E+03, 4.239867E+03, 4.393620E+03, 4.552413E+03, 4.716421E+03, 4.885827E+03, 
   5.060819E+03, 5.241590E+03, 5.428340E+03, 5.621272E+03, 5.820599E+03, 6.026538E+03, 
   6.239313E+03, 6.459156E+03, 6.686303E+03, 6.921000E+03, 7.163500E+03, 7.414061E+03, 
   7.672953E+03, 7.940450E+03, 8.216837E+03, 8.502406E+03, 8.797459E+03, 9.102306E+03, 
   9.417266E+03, 9.742669E+03, 1.007885E+04, 1.042617E+04, 1.078497E+04, 1.115564E+04, 
   1.153854E+04, 1.193407E+04, 1.234264E+04, 1.276466E+04, 1.320056E+04, 1.365077E+04, 
   1.411574E+04, 1.459595E+04, 1.509187E+04, 1.560400E+04, 1.613283E+04, 1.667888E+04, 
   1.724270E+04, 1.782483E+04, 1.842583E+04, 1.904629E+04, 1.968680E+04, 2.034797E+04, 
   2.103044E+04, 2.173484E+04, 2.246184E+04, 2.321212E+04, 2.398638E+04, 2.478534E+04, 
   2.560973E+04, 2.646031E+04, 2.733784E+04, 2.824313E+04, 2.917699E+04, 3.014024E+04, 
   3.113375E+04, 3.215840E+04, 3.321507E+04, 3.430470E+04, 3.542821E+04, 3.658657E+04, 
   3.778077E+04, 3.901183E+04, 4.028076E+04, 4.158863E+04, 4.293652E+04, 4.432669E+04, 
   4.575973E+04, 4.723687E+04, 4.875937E+04, 5.032855E+04, 5.194574E+04, 5.361231E+04, 
   5.532968E+04, 5.709929E+04, 5.892262E+04, 6.080119E+04, 6.273657E+04, 6.473036E+04, 
   6.678420E+04, 6.889978E+04, 7.107883E+04, 7.332311E+04, 7.563444E+04, 7.801469E+04, 
   8.046577E+04, 8.298964E+04, 8.558830E+04, 8.826381E+04, 9.101828E+04, 9.385387E+04, 
   9.677280E+04, 9.977734E+04, 1.028698E+05, 1.060526E+05, 1.093282E+05, 1.126990E+05, 
   1.161676E+05, 1.197367E+05, 1.234089E+05, 1.271870E+05, 1.310738E+05, 1.350722E+05, 
   1.391852E+05, 1.434157E+05, 1.477669E+05, 1.522420E+05, 1.568441E+05, 1.615766E+05, 
   1.664430E+05, 1.714466E+05, 1.765911E+05, 1.818800E+05, 1.873172E+05, 1.929064E+05, 
   1.986515E+05, 2.045565E+05, 2.106255E+05, 2.168627E+05, 2.232722E+05, 2.298586E+05, 
   2.366262E+05, 2.435795E+05, 2.507234E+05, 2.580624E+05, 2.656015E+05, 2.733457E+05, 
   2.812999E+05, 2.894696E+05, 2.978598E+05, 3.064761E+05, 3.153239E+05, 3.244090E+05, 
   3.337371E+05, 3.433142E+05, 3.531461E+05, 3.632391E+05, 3.735995E+05, 3.842337E+05, 
   3.951481E+05, 4.063496E+05, 4.178448E+05, 4.296409E+05, 4.417448E+05, 4.541637E+05, 
   4.669052E+05, 4.799767E+05, 4.933860E+05, 5.071408E+05, 5.212491E+05, 5.357192E+05, 
   5.505593E+05, 5.657780E+05, 5.813838E+05, 5.973857E+05, 6.137925E+05, 6.306134E+05, 
   6.478578E+05, 6.655352E+05, 6.836553E+05, 7.022279E+05, 7.212632E+05, 7.407713E+05, 
   7.607628E+05, 7.812482E+05, 8.022384E+05, 8.237444E+05, 8.457775E+05, 8.683491E+05, 
   8.914708E+05, 9.151544E+05, 9.394122E+05, 9.642562E+05, 9.896991E+05, 1.015754E+06, 
   1.042433E+06, 1.069749E+06, 1.097717E+06, 1.126349E+06, 1.155660E+06, 1.185664E+06, 
   1.216374E+06, 1.247806E+06, 1.279975E+06, 1.312895E+06, 1.346581E+06, 
])
# ============================ M = 6, I = 3 ============================
M = 6
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[1]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.136787E+00, 3.851666E+01, 9.190122E+01, 1.609080E+02, 2.433209E+02, 3.370859E+02, 
   4.407898E+02, 5.534753E+02, 6.744479E+02, 8.031682E+02, 9.391965E+02, 1.082163E+03, 
   1.231752E+03, 1.387689E+03, 1.549735E+03, 1.717683E+03, 1.891357E+03, 2.070607E+03, 
   2.255315E+03, 2.445393E+03, 2.640781E+03, 2.841453E+03, 3.047414E+03, 3.258701E+03, 
   3.475383E+03, 3.697559E+03, 3.925360E+03, 4.158944E+03, 4.398498E+03, 4.644236E+03, 
   4.896397E+03, 5.155242E+03, 5.421056E+03, 5.694144E+03, 5.974832E+03, 6.263464E+03, 
   6.560403E+03, 6.866029E+03, 7.180737E+03, 7.504940E+03, 7.839066E+03, 8.183561E+03, 
   8.538882E+03, 8.905506E+03, 9.283922E+03, 9.674638E+03, 1.007817E+04, 1.049507E+04, 
   1.092588E+04, 1.137118E+04, 1.183156E+04, 1.230763E+04, 1.280000E+04, 1.330934E+04, 
   1.383631E+04, 1.438158E+04, 1.494588E+04, 1.552992E+04, 1.613447E+04, 1.676030E+04, 
   1.740822E+04, 1.807904E+04, 1.877362E+04, 1.949285E+04, 2.023763E+04, 2.100889E+04, 
   2.180760E+04, 2.263477E+04, 2.349142E+04, 2.437860E+04, 2.529743E+04, 2.624903E+04, 
   2.723457E+04, 2.825525E+04, 2.931231E+04, 3.040705E+04, 3.154077E+04, 3.271486E+04, 
   3.393072E+04, 3.518980E+04, 3.649361E+04, 3.784369E+04, 3.924164E+04, 4.068910E+04, 
   4.218778E+04, 4.373944E+04, 4.534586E+04, 4.700893E+04, 4.873057E+04, 5.051275E+04, 
   5.235752E+04, 5.426699E+04, 5.624333E+04, 5.828878E+04, 6.040565E+04, 6.259631E+04, 
   6.486323E+04, 6.720893E+04, 6.963601E+04, 7.214715E+04, 7.474512E+04, 7.743278E+04, 
   8.021305E+04, 8.308895E+04, 8.606361E+04, 8.914022E+04, 9.232209E+04, 9.561261E+04, 
   9.901529E+04, 1.025337E+05, 1.061716E+05, 1.099329E+05, 1.138213E+05, 1.178410E+05, 
   1.219961E+05, 1.262910E+05, 1.307299E+05, 1.353176E+05, 1.400585E+05, 1.449576E+05, 
   1.500198E+05, 1.552501E+05, 1.606538E+05, 1.662362E+05, 1.720029E+05, 1.779595E+05, 
   1.841120E+05, 1.904661E+05, 1.970283E+05, 2.038047E+05, 2.108019E+05, 2.180266E+05, 
   2.254857E+05, 2.331862E+05, 2.411354E+05, 2.493408E+05, 2.578100E+05, 2.665509E+05, 
   2.755716E+05, 2.848803E+05, 2.944856E+05, 3.043962E+05, 3.146212E+05, 3.251696E+05, 
   3.360510E+05, 3.472752E+05, 3.588519E+05, 3.707915E+05, 3.831045E+05, 3.958015E+05, 
   4.088938E+05, 4.223924E+05, 4.363091E+05, 4.506558E+05, 4.654446E+05, 4.806881E+05, 
   4.963991E+05, 5.125907E+05, 5.292765E+05, 5.464702E+05, 5.641859E+05, 5.824382E+05, 
   6.012420E+05, 6.206123E+05, 6.405649E+05, 6.611157E+05, 6.822811E+05, 7.040776E+05, 
   7.265226E+05, 7.496336E+05, 7.734284E+05, 7.979255E+05, 8.231438E+05, 8.491023E+05, 
   8.758210E+05, 9.033198E+05, 9.316194E+05, 9.607409E+05, 9.907059E+05, 1.021536E+06, 
   1.053255E+06, 1.085885E+06, 1.119449E+06, 1.153972E+06, 1.189478E+06, 1.225993E+06, 
   1.263543E+06, 1.302152E+06, 1.341850E+06, 1.382661E+06, 1.424616E+06, 1.467742E+06, 
   1.512068E+06, 1.557625E+06, 1.604443E+06, 1.652552E+06, 1.701986E+06, 1.752774E+06, 
   1.804952E+06, 1.858553E+06, 1.913610E+06, 1.970160E+06, 2.028237E+06, 2.087878E+06, 
   2.149122E+06, 2.212004E+06, 2.276564E+06, 2.342842E+06, 2.410878E+06, 2.480712E+06, 
   2.552386E+06, 2.625943E+06, 2.701426E+06, 2.778879E+06, 2.858348E+06, 2.939877E+06, 
   3.023513E+06, 3.109305E+06, 3.197300E+06, 3.287547E+06, 3.380097E+06, 3.475000E+06, 
   3.572308E+06, 3.672075E+06, 3.774353E+06, 3.879197E+06, 3.986664E+06, 4.096808E+06, 
   4.209689E+06, 4.325364E+06, 4.443893E+06, 4.565335E+06, 4.689754E+06, 4.817210E+06, 
   4.947767E+06, 5.081491E+06, 5.218445E+06, 5.358698E+06, 5.502316E+06, 5.649368E+06, 
   5.799924E+06, 5.954055E+06, 6.111833E+06, 6.273330E+06, 6.438622E+06, 6.607782E+06, 
   6.780888E+06, 6.958018E+06, 7.139249E+06, 7.324662E+06, 7.514338E+06, 
])
# ============================ M = 6, I = 4 ============================
M = 6
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[1]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.626995E+01, 7.708984E+01, 1.839494E+02, 3.220799E+02, 4.870457E+02, 6.747349E+02, 
   8.823193E+02, 1.107882E+03, 1.350034E+03, 1.607694E+03, 1.879984E+03, 2.166162E+03, 
   2.465597E+03, 2.777738E+03, 3.102111E+03, 3.438300E+03, 3.785955E+03, 4.144780E+03, 
   4.514542E+03, 4.895066E+03, 5.286243E+03, 5.688022E+03, 6.100423E+03, 6.523527E+03, 
   6.957478E+03, 7.402486E+03, 7.858818E+03, 8.326804E+03, 8.806825E+03, 9.299319E+03, 
   9.804771E+03, 1.032372E+04, 1.085674E+04, 1.140444E+04, 1.196751E+04, 1.254662E+04, 
   1.314253E+04, 1.375599E+04, 1.438781E+04, 1.503882E+04, 1.570990E+04, 1.640194E+04, 
   1.711587E+04, 1.785265E+04, 1.861328E+04, 1.939879E+04, 2.021022E+04, 2.104867E+04, 
   2.191527E+04, 2.281118E+04, 2.373758E+04, 2.469571E+04, 2.568685E+04, 2.671229E+04, 
   2.777339E+04, 2.887155E+04, 3.000820E+04, 3.118481E+04, 3.240292E+04, 3.366410E+04, 
   3.496997E+04, 3.632222E+04, 3.772257E+04, 3.917280E+04, 4.067477E+04, 4.223037E+04, 
   4.384156E+04, 4.551036E+04, 4.723888E+04, 4.902926E+04, 5.088373E+04, 5.280458E+04, 
   5.479420E+04, 5.685503E+04, 5.898958E+04, 6.120048E+04, 6.349040E+04, 6.586213E+04, 
   6.831854E+04, 7.086256E+04, 7.349727E+04, 7.622579E+04, 7.905139E+04, 8.197741E+04, 
   8.500730E+04, 8.814462E+04, 9.139307E+04, 9.475642E+04, 9.823858E+04, 1.018436E+05, 
   1.055756E+05, 1.094389E+05, 1.134380E+05, 1.175773E+05, 1.218615E+05, 1.262956E+05, 
   1.308845E+05, 1.356333E+05, 1.405474E+05, 1.456322E+05, 1.508933E+05, 1.563366E+05, 
   1.619680E+05, 1.677936E+05, 1.738199E+05, 1.800533E+05, 1.865005E+05, 1.931686E+05, 
   2.000645E+05, 2.071957E+05, 2.145697E+05, 2.221944E+05, 2.300776E+05, 2.382276E+05, 
   2.466530E+05, 2.553625E+05, 2.643650E+05, 2.736698E+05, 2.832864E+05, 2.932246E+05, 
   3.034944E+05, 3.141063E+05, 3.250708E+05, 3.363989E+05, 3.481018E+05, 3.601912E+05, 
   3.726790E+05, 3.855773E+05, 3.988987E+05, 4.126562E+05, 4.268631E+05, 4.415330E+05, 
   4.566800E+05, 4.723184E+05, 4.884630E+05, 5.051292E+05, 5.223324E+05, 5.400888E+05, 
   5.584149E+05, 5.773275E+05, 5.968440E+05, 6.169823E+05, 6.377607E+05, 6.591980E+05, 
   6.813135E+05, 7.041270E+05, 7.276588E+05, 7.519298E+05, 7.769614E+05, 8.027755E+05, 
   8.293946E+05, 8.568419E+05, 8.851410E+05, 9.143162E+05, 9.443924E+05, 9.753951E+05, 
   1.007351E+06, 1.040286E+06, 1.074228E+06, 1.109205E+06, 1.145246E+06, 1.182382E+06, 
   1.220641E+06, 1.260055E+06, 1.300656E+06, 1.342477E+06, 1.385551E+06, 1.429911E+06, 
   1.475594E+06, 1.522634E+06, 1.571069E+06, 1.620936E+06, 1.672273E+06, 1.725120E+06, 
   1.779518E+06, 1.835506E+06, 1.893127E+06, 1.952424E+06, 2.013442E+06, 2.076224E+06, 
   2.140819E+06, 2.207271E+06, 2.275630E+06, 2.345944E+06, 2.418264E+06, 2.492642E+06, 
   2.569129E+06, 2.647780E+06, 2.728648E+06, 2.811791E+06, 2.897265E+06, 2.985128E+06, 
   3.075441E+06, 3.168264E+06, 3.263659E+06, 3.361689E+06, 3.462420E+06, 3.565917E+06, 
   3.672247E+06, 3.781480E+06, 3.893686E+06, 4.008936E+06, 4.127303E+06, 4.248861E+06, 
   4.373687E+06, 4.501858E+06, 4.633453E+06, 4.768551E+06, 4.907236E+06, 5.049590E+06, 
   5.195700E+06, 5.345650E+06, 5.499531E+06, 5.657431E+06, 5.819443E+06, 5.985660E+06, 
   6.156177E+06, 6.331090E+06, 6.510499E+06, 6.694504E+06, 6.883206E+06, 7.076710E+06, 
   7.275122E+06, 7.478548E+06, 7.687099E+06, 7.900887E+06, 8.120023E+06, 8.344624E+06, 
   8.574806E+06, 8.810690E+06, 9.052395E+06, 9.300046E+06, 9.553767E+06, 9.813686E+06, 
   1.007993E+07, 1.035264E+07, 1.063193E+07, 1.091796E+07, 1.121084E+07, 1.151074E+07, 
   1.181778E+07, 1.213212E+07, 1.245389E+07, 1.278325E+07, 1.312035E+07, 1.346534E+07, 
   1.381838E+07, 1.417962E+07, 1.454923E+07, 1.492737E+07, 1.531421E+07, 
])
# ============================ M = 7, I = 1 ============================
M = 7
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[8]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.259272E+00, 8.305762E+00, 1.541160E+01, 2.261460E+01, 2.984283E+01, 3.708152E+01, 
   4.432568E+01, 5.157319E+01, 5.882297E+01, 6.607443E+01, 7.332720E+01, 8.058105E+01, 
   8.783584E+01, 9.509146E+01, 1.023478E+02, 1.096049E+02, 1.168627E+02, 1.241212E+02, 
   1.313804E+02, 1.386405E+02, 1.459015E+02, 1.531638E+02, 1.604275E+02, 1.676930E+02, 
   1.749609E+02, 1.822316E+02, 1.895058E+02, 1.967843E+02, 2.040679E+02, 2.113575E+02, 
   2.186540E+02, 2.259587E+02, 2.332724E+02, 2.405965E+02, 2.479320E+02, 2.552803E+02, 
   2.626424E+02, 2.700196E+02, 2.774132E+02, 2.848242E+02, 2.922540E+02, 2.997035E+02, 
   3.071739E+02, 3.146663E+02, 3.221816E+02, 3.297209E+02, 3.372850E+02, 3.448749E+02, 
   3.524912E+02, 3.601348E+02, 3.678063E+02, 3.755064E+02, 3.832358E+02, 3.909948E+02, 
   3.987841E+02, 4.066041E+02, 4.144551E+02, 4.223374E+02, 4.302515E+02, 4.381975E+02, 
   4.461757E+02, 4.541862E+02, 4.622292E+02, 4.703048E+02, 4.784130E+02, 4.865539E+02, 
   4.947276E+02, 5.029339E+02, 5.111729E+02, 5.194445E+02, 5.277485E+02, 5.360850E+02, 
   5.444538E+02, 5.528547E+02, 5.612875E+02, 5.697522E+02, 5.782484E+02, 5.867761E+02, 
   5.953349E+02, 6.039247E+02, 6.125453E+02, 6.211963E+02, 6.298776E+02, 6.385889E+02, 
   6.473299E+02, 6.561004E+02, 6.649000E+02, 6.737286E+02, 6.825859E+02, 6.914715E+02, 
   7.003851E+02, 7.093266E+02, 7.182956E+02, 7.272917E+02, 7.363149E+02, 7.453646E+02, 
   7.544408E+02, 7.635430E+02, 7.726710E+02, 7.818245E+02, 7.910032E+02, 8.002069E+02, 
   8.094352E+02, 8.186879E+02, 8.279648E+02, 8.372654E+02, 8.465897E+02, 8.559372E+02, 
   8.653078E+02, 8.747012E+02, 8.841171E+02, 8.935553E+02, 9.030155E+02, 9.124974E+02, 
   9.220009E+02, 9.315257E+02, 9.410715E+02, 9.506382E+02, 9.602254E+02, 9.698330E+02, 
   9.794607E+02, 9.891084E+02, 9.987757E+02, 1.008463E+03, 1.018169E+03, 1.027894E+03, 
   1.037638E+03, 1.047401E+03, 1.057182E+03, 1.066981E+03, 1.076799E+03, 1.086635E+03, 
   1.096488E+03, 1.106359E+03, 1.116247E+03, 1.126153E+03, 1.136075E+03, 1.146015E+03, 
   1.155971E+03, 1.165944E+03, 1.175934E+03, 1.185939E+03, 1.195961E+03, 1.205999E+03, 
   1.216053E+03, 1.226122E+03, 1.236207E+03, 1.246308E+03, 1.256424E+03, 1.266555E+03, 
   1.276701E+03, 1.286862E+03, 1.297039E+03, 1.307230E+03, 1.317435E+03, 1.327656E+03, 
   1.337890E+03, 1.348140E+03, 1.358403E+03, 1.368681E+03, 1.378972E+03, 1.389278E+03, 
   1.399598E+03, 1.409931E+03, 1.420279E+03, 1.430640E+03, 1.441014E+03, 1.451402E+03, 
   1.461804E+03, 1.472219E+03, 1.482647E+03, 1.493089E+03, 1.503544E+03, 1.514012E+03, 
   1.524493E+03, 1.534987E+03, 1.545494E+03, 1.556014E+03, 1.566547E+03, 1.577093E+03, 
   1.587651E+03, 1.598222E+03, 1.608806E+03, 1.619403E+03, 1.630012E+03, 1.640634E+03, 
   1.651268E+03, 1.661915E+03, 1.672575E+03, 1.683246E+03, 1.693931E+03, 1.704627E+03, 
   1.715336E+03, 1.726057E+03, 1.736791E+03, 1.747537E+03, 1.758295E+03, 1.769065E+03, 
   1.779848E+03, 1.790643E+03, 1.801450E+03, 1.812269E+03, 1.823100E+03, 1.833944E+03, 
   1.844800E+03, 1.855667E+03, 1.866547E+03, 1.877439E+03, 1.888343E+03, 1.899259E+03, 
   1.910188E+03, 1.921128E+03, 1.932080E+03, 1.943045E+03, 1.954021E+03, 1.965010E+03, 
   1.976010E+03, 1.987023E+03, 1.998048E+03, 2.009084E+03, 2.020133E+03, 2.031194E+03, 
   2.042267E+03, 2.053352E+03, 2.064449E+03, 2.075559E+03, 2.086680E+03, 2.097813E+03, 
   2.108959E+03, 2.120116E+03, 2.131286E+03, 2.142468E+03, 2.153662E+03, 2.164868E+03, 
   2.176086E+03, 2.187317E+03, 2.198559E+03, 2.209814E+03, 2.221081E+03, 2.232360E+03, 
   2.243651E+03, 2.254955E+03, 2.266271E+03, 2.277599E+03, 2.288939E+03, 2.300292E+03, 
   2.311657E+03, 2.323034E+03, 2.334424E+03, 2.345826E+03, 2.357240E+03, 2.368666E+03, 
   2.380105E+03, 2.391557E+03, 2.403021E+03, 2.414497E+03, 2.425986E+03, 2.437487E+03, 
   2.449001E+03, 2.460527E+03, 2.472066E+03, 2.483617E+03, 2.495181E+03, 2.506757E+03, 
   2.518346E+03, 2.529947E+03, 2.541562E+03, 2.553189E+03, 2.564828E+03, 2.576480E+03, 
   2.588145E+03, 2.599823E+03, 2.611513E+03, 2.623216E+03, 2.634932E+03, 2.646661E+03, 
   2.658402E+03, 2.670157E+03, 2.681924E+03, 2.693704E+03, 2.705497E+03, 2.717303E+03, 
   2.729122E+03, 2.740953E+03, 2.752798E+03, 2.764656E+03, 2.776526E+03, 2.788410E+03, 
   2.800307E+03, 2.812217E+03, 2.824140E+03, 2.836076E+03, 2.848025E+03, 2.859987E+03, 
   2.871962E+03, 2.883951E+03, 2.895953E+03, 2.907968E+03, 2.919996E+03, 2.932038E+03, 
   2.944092E+03, 2.956160E+03, 2.968242E+03, 2.980336E+03, 2.992444E+03, 3.004566E+03, 
   3.016701E+03, 3.028849E+03, 3.041010E+03, 3.053185E+03, 3.065374E+03, 3.077576E+03, 
   3.089791E+03, 3.102020E+03, 3.114262E+03, 3.126518E+03, 3.138788E+03, 3.151071E+03, 
   3.163367E+03, 3.175678E+03, 3.188002E+03, 3.200339E+03, 3.212690E+03, 3.225055E+03, 
   3.237434E+03, 3.249826E+03, 3.262232E+03, 3.274652E+03, 3.287085E+03, 3.299532E+03, 
   3.311993E+03, 3.324468E+03, 3.336957E+03, 3.349459E+03, 3.361975E+03, 3.374506E+03, 
   3.387050E+03, 3.399608E+03, 3.412179E+03, 3.424765E+03, 3.437365E+03, 3.449978E+03, 
   3.462606E+03, 3.475247E+03, 3.487903E+03, 3.500572E+03, 3.513256E+03, 3.525954E+03, 
   3.538665E+03, 3.551391E+03, 3.564130E+03, 3.576884E+03, 3.589652E+03, 3.602434E+03, 
   3.615230E+03, 3.628040E+03, 3.640865E+03, 3.653703E+03, 3.666556E+03, 3.679423E+03, 
   3.692304E+03, 3.705199E+03, 3.718109E+03, 3.731032E+03, 3.743970E+03, 3.756922E+03, 
   3.769889E+03, 3.782869E+03, 3.795864E+03, 3.808873E+03, 3.821897E+03, 3.834935E+03, 
   3.847987E+03, 3.861053E+03, 3.874134E+03, 3.887229E+03, 3.900338E+03, 3.913462E+03, 
   3.926600E+03, 3.939753E+03, 3.952920E+03, 3.966101E+03, 3.979297E+03, 3.992507E+03, 
   4.005731E+03, 4.018970E+03, 4.032224E+03, 4.045491E+03, 4.058774E+03, 4.072070E+03, 
   4.085382E+03, 4.098707E+03, 4.112047E+03, 4.125402E+03, 4.138771E+03, 4.152155E+03, 
   4.165553E+03, 4.178965E+03, 4.192392E+03, 4.205834E+03, 4.219290E+03, 4.232761E+03, 
   4.246246E+03, 4.259746E+03, 4.273260E+03, 4.286789E+03, 4.300332E+03, 4.313890E+03, 
   4.327463E+03, 4.341050E+03, 4.354651E+03, 4.368268E+03, 4.381898E+03, 4.395544E+03, 
   4.409204E+03, 4.422878E+03, 4.436567E+03, 4.450271E+03, 4.463989E+03, 4.477722E+03, 
   4.491470E+03, 4.505232E+03, 4.519009E+03, 4.532800E+03, 4.546606E+03, 4.560427E+03, 
   4.574262E+03, 4.588112E+03, 4.601977E+03, 4.615856E+03, 4.629750E+03, 4.643658E+03, 
   4.657581E+03, 4.671519E+03, 4.685471E+03, 4.699438E+03, 4.713419E+03, 4.727416E+03, 
   4.741426E+03, 4.755452E+03, 4.769492E+03, 4.783547E+03, 4.797616E+03, 4.811700E+03, 
   4.825799E+03, 4.839912E+03, 4.854040E+03, 4.868183E+03, 4.882340E+03, 4.896512E+03, 
   4.910698E+03, 4.924900E+03, 4.939115E+03, 4.953346E+03, 4.967591E+03, 4.981850E+03, 
   4.996125E+03, 5.010414E+03, 5.024717E+03, 5.039035E+03, 5.053368E+03, 5.067716E+03, 
   5.082078E+03, 5.096454E+03, 5.110846E+03, 5.125252E+03, 5.139672E+03, 5.154107E+03, 
   5.168557E+03, 5.183021E+03, 5.197500E+03, 5.211994E+03, 5.226502E+03, 5.241024E+03, 
   5.255562E+03, 5.270114E+03, 5.284680E+03, 5.299261E+03, 5.313857E+03, 5.328467E+03, 
   5.343091E+03, 5.357731E+03, 5.372384E+03, 5.387053E+03, 5.401736E+03, 5.416433E+03, 
   5.431145E+03, 5.445872E+03, 5.460613E+03, 5.475368E+03, 5.490138E+03, 5.504923E+03, 
   5.519722E+03, 5.534536E+03, 5.549364E+03, 5.564206E+03, 5.579063E+03, 5.593935E+03, 
   5.608821E+03, 5.623721E+03, 5.638636E+03, 5.653566E+03, 5.668509E+03, 5.683468E+03, 
   5.698440E+03, 5.713428E+03, 5.728429E+03, 5.743445E+03, 5.758475E+03, 5.773520E+03, 
   5.788579E+03, 5.803653E+03, 5.818741E+03, 5.833843E+03, 5.848960E+03, 5.864091E+03, 
   5.879236E+03, 5.894396E+03, 5.909570E+03, 5.924758E+03, 5.939961E+03, 5.955178E+03, 
   5.970409E+03, 5.985655E+03, 6.000915E+03, 6.016189E+03, 6.031477E+03, 6.046780E+03, 
   6.062097E+03, 6.077428E+03, 6.092774E+03, 6.108133E+03, 6.123507E+03, 6.138895E+03, 
   6.154298E+03, 6.169714E+03, 6.185145E+03, 6.200590E+03, 6.216049E+03, 6.231522E+03, 
   6.247009E+03, 6.262511E+03, 6.278026E+03, 6.293556E+03, 6.309100E+03, 6.324658E+03, 
   6.340229E+03, 6.355816E+03, 6.371416E+03, 6.387030E+03, 6.402658E+03, 6.418300E+03, 
   6.433957E+03, 6.449627E+03, 6.465311E+03, 6.481009E+03, 6.496722E+03, 6.512448E+03, 
   6.528188E+03, 6.543942E+03, 6.559710E+03, 6.575492E+03, 6.591288E+03, 6.607098E+03, 
   6.622922E+03, 6.638759E+03, 6.654611E+03, 6.670476E+03, 6.686355E+03, 6.702248E+03, 
   6.718155E+03, 6.734076E+03, 6.750010E+03, 6.765958E+03, 6.781920E+03, 6.797896E+03, 
   6.813886E+03, 6.829889E+03, 6.845906E+03, 6.861937E+03, 6.877981E+03, 6.894039E+03, 
   6.910111E+03, 6.926197E+03, 6.942296E+03, 6.958408E+03, 6.974535E+03, 6.990675E+03, 
   7.006828E+03, 7.022996E+03, 7.039176E+03, 7.055371E+03, 7.071578E+03, 7.087800E+03, 
   7.104035E+03, 7.120283E+03, 7.136545E+03, 7.152821E+03, 7.169109E+03, 7.185412E+03, 
   7.201728E+03, 7.218057E+03, 7.234399E+03, 7.250755E+03, 7.267125E+03, 7.283508E+03, 
   7.299904E+03, 7.316313E+03, 7.332736E+03, 7.349172E+03, 7.365622E+03, 7.382084E+03, 
   7.398560E+03, 7.415049E+03, 7.431552E+03, 7.448068E+03, 7.464597E+03, 7.481139E+03, 
   7.497694E+03, 7.514263E+03, 7.530844E+03, 7.547439E+03, 7.564047E+03, 7.580668E+03, 
   7.597302E+03, 7.613949E+03, 7.630610E+03, 7.647283E+03, 7.663969E+03, 7.680669E+03, 
   7.697381E+03, 7.714106E+03, 7.730845E+03, 7.747596E+03, 7.764360E+03, 7.781138E+03, 
   7.797928E+03, 7.814731E+03, 7.831547E+03, 7.848375E+03, 7.865217E+03, 7.882071E+03, 
   7.898939E+03, 7.915819E+03, 7.932712E+03, 7.949617E+03, 7.966535E+03, 7.983467E+03, 
   8.000410E+03, 8.017367E+03, 8.034336E+03, 8.051318E+03, 8.068313E+03, 8.085320E+03, 
   8.102340E+03, 8.119372E+03, 8.136417E+03, 8.153475E+03, 8.170545E+03, 8.187627E+03, 
   8.204723E+03, 8.221830E+03, 8.238951E+03, 8.256083E+03, 8.273228E+03, 8.290386E+03, 
   8.307556E+03, 8.324738E+03, 8.341933E+03, 8.359140E+03, 8.376360E+03, 8.393592E+03, 
   8.410836E+03, 8.428092E+03, 8.445361E+03, 8.462642E+03, 8.479935E+03, 8.497241E+03, 
   8.514559E+03, 8.531889E+03, 8.549231E+03, 8.566585E+03, 8.583951E+03, 8.601330E+03, 
   8.618721E+03, 8.636123E+03, 8.653538E+03, 8.670965E+03, 8.688404E+03, 8.705855E+03, 
   8.723318E+03, 8.740793E+03, 8.758280E+03, 8.775779E+03, 8.793289E+03, 8.810812E+03, 
   8.828347E+03, 8.845893E+03, 8.863452E+03, 8.881022E+03, 8.898604E+03, 8.916198E+03, 
   8.933803E+03, 8.951421E+03, 8.969050E+03, 8.986690E+03, 9.004343E+03, 9.022007E+03, 
   9.039683E+03, 9.057371E+03, 9.075070E+03, 9.092781E+03, 9.110503E+03, 9.128237E+03, 
   9.145983E+03, 9.163740E+03, 9.181509E+03, 9.199289E+03, 9.217080E+03, 9.234884E+03, 
   9.252698E+03, 9.270524E+03, 9.288362E+03, 9.306210E+03, 9.324071E+03, 9.341942E+03, 
   9.359825E+03, 9.377719E+03, 9.395625E+03, 9.413542E+03, 9.431470E+03, 9.449409E+03, 
   9.467359E+03, 9.485321E+03, 9.503294E+03, 9.521278E+03, 9.539273E+03, 9.557280E+03, 
   9.575297E+03, 9.593326E+03, 9.611365E+03, 9.629416E+03, 9.647478E+03, 9.665550E+03, 
   9.683634E+03, 9.701729E+03, 9.719834E+03, 9.737951E+03, 9.756078E+03, 9.774217E+03, 
   9.792366E+03, 
])
# ============================ M = 7, I = 2 ============================
M = 7
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[8]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.562445E+00, 1.579406E+01, 3.092123E+01, 4.619570E+01, 6.151168E+01, 7.684554E+01, 
   9.218904E+01, 1.075385E+02, 1.228922E+02, 1.382490E+02, 1.536083E+02, 1.689697E+02, 
   1.843329E+02, 1.996977E+02, 2.150641E+02, 2.304319E+02, 2.458012E+02, 2.611720E+02, 
   2.765444E+02, 2.919187E+02, 3.072954E+02, 3.226749E+02, 3.380581E+02, 3.534459E+02, 
   3.688395E+02, 3.842403E+02, 3.996500E+02, 4.150705E+02, 4.305039E+02, 4.459525E+02, 
   4.614188E+02, 4.769054E+02, 4.924152E+02, 5.079512E+02, 5.235163E+02, 5.391138E+02, 
   5.547468E+02, 5.704185E+02, 5.861323E+02, 6.018914E+02, 6.176990E+02, 6.335585E+02, 
   6.494730E+02, 6.654457E+02, 6.814797E+02, 6.975780E+02, 7.137437E+02, 7.299797E+02, 
   7.462888E+02, 7.626738E+02, 7.791373E+02, 7.956819E+02, 8.123102E+02, 8.290246E+02, 
   8.458274E+02, 8.627209E+02, 8.797073E+02, 8.967887E+02, 9.139671E+02, 9.312444E+02, 
   9.486225E+02, 9.661033E+02, 9.836883E+02, 1.001379E+03, 1.019178E+03, 1.037086E+03, 
   1.055104E+03, 1.073235E+03, 1.091478E+03, 1.109837E+03, 1.128311E+03, 1.146902E+03, 
   1.165612E+03, 1.184441E+03, 1.203390E+03, 1.222461E+03, 1.241654E+03, 1.260971E+03, 
   1.280411E+03, 1.299977E+03, 1.319669E+03, 1.339487E+03, 1.359433E+03, 1.379507E+03, 
   1.399711E+03, 1.420043E+03, 1.440506E+03, 1.461100E+03, 1.481826E+03, 1.502683E+03, 
   1.523674E+03, 1.544797E+03, 1.566055E+03, 1.587446E+03, 1.608973E+03, 1.630635E+03, 
   1.652432E+03, 1.674366E+03, 1.696437E+03, 1.718644E+03, 1.740989E+03, 1.763472E+03, 
   1.786093E+03, 1.808853E+03, 1.831752E+03, 1.854791E+03, 1.877969E+03, 1.901287E+03, 
   1.924746E+03, 1.948345E+03, 1.972085E+03, 1.995967E+03, 2.019991E+03, 2.044157E+03, 
   2.068465E+03, 2.092915E+03, 2.117509E+03, 2.142245E+03, 2.167125E+03, 2.192149E+03, 
   2.217317E+03, 2.242629E+03, 2.268085E+03, 2.293687E+03, 2.319433E+03, 2.345325E+03, 
   2.371362E+03, 2.397545E+03, 2.423874E+03, 2.450349E+03, 2.476971E+03, 2.503740E+03, 
   2.530656E+03, 2.557718E+03, 2.584929E+03, 2.612287E+03, 2.639793E+03, 2.667447E+03, 
   2.695250E+03, 2.723201E+03, 2.751302E+03, 2.779551E+03, 2.807950E+03, 2.836498E+03, 
   2.865197E+03, 2.894045E+03, 2.923044E+03, 2.952193E+03, 2.981493E+03, 3.010944E+03, 
   3.040546E+03, 3.070300E+03, 3.100206E+03, 3.130263E+03, 3.160473E+03, 3.190835E+03, 
   3.221350E+03, 3.252017E+03, 3.282838E+03, 3.313812E+03, 3.344940E+03, 3.376222E+03, 
   3.407658E+03, 3.439248E+03, 3.470993E+03, 3.502893E+03, 3.534948E+03, 3.567158E+03, 
   3.599524E+03, 3.632045E+03, 3.664723E+03, 3.697557E+03, 3.730548E+03, 3.763695E+03, 
   3.797000E+03, 3.830462E+03, 3.864082E+03, 3.897859E+03, 3.931795E+03, 3.965889E+03, 
   4.000141E+03, 4.034553E+03, 4.069124E+03, 4.103854E+03, 4.138744E+03, 4.173793E+03, 
   4.209003E+03, 4.244374E+03, 4.279905E+03, 4.315597E+03, 4.351450E+03, 4.387465E+03, 
   4.423642E+03, 4.459981E+03, 4.496482E+03, 4.533145E+03, 4.569972E+03, 4.606962E+03, 
   4.644115E+03, 4.681432E+03, 4.718912E+03, 4.756557E+03, 4.794367E+03, 4.832341E+03, 
   4.870480E+03, 4.908784E+03, 4.947254E+03, 4.985890E+03, 5.024692E+03, 5.063661E+03, 
   5.102796E+03, 5.142098E+03, 5.181567E+03, 5.221204E+03, 5.261008E+03, 5.300981E+03, 
   5.341122E+03, 5.381431E+03, 5.421910E+03, 5.462557E+03, 5.503374E+03, 5.544360E+03, 
   5.585517E+03, 5.626844E+03, 5.668341E+03, 5.710009E+03, 5.751848E+03, 5.793858E+03, 
   5.836040E+03, 5.878394E+03, 5.920920E+03, 5.963619E+03, 6.006490E+03, 6.049534E+03, 
   6.092751E+03, 6.136142E+03, 6.179707E+03, 6.223446E+03, 6.267359E+03, 6.311446E+03, 
   6.355709E+03, 6.400147E+03, 6.444760E+03, 6.489548E+03, 6.534513E+03, 6.579654E+03, 
   6.624971E+03, 6.670465E+03, 6.716136E+03, 6.761984E+03, 6.808010E+03, 6.854214E+03, 
   6.900596E+03, 6.947156E+03, 6.993894E+03, 7.040812E+03, 7.087908E+03, 7.135184E+03, 
   7.182639E+03, 7.230275E+03, 7.278090E+03, 7.326086E+03, 7.374263E+03, 7.422620E+03, 
   7.471159E+03, 7.519879E+03, 7.568780E+03, 7.617864E+03, 7.667129E+03, 7.716577E+03, 
   7.766208E+03, 7.816022E+03, 7.866019E+03, 7.916199E+03, 7.966563E+03, 8.017110E+03, 
   8.067842E+03, 8.118758E+03, 8.169859E+03, 8.221144E+03, 8.272615E+03, 8.324271E+03, 
   8.376113E+03, 8.428140E+03, 8.480353E+03, 8.532753E+03, 8.585339E+03, 8.638112E+03, 
   8.691071E+03, 8.744218E+03, 8.797553E+03, 8.851075E+03, 8.904784E+03, 8.958682E+03, 
   9.012769E+03, 9.067043E+03, 9.121507E+03, 9.176159E+03, 9.231001E+03, 9.286032E+03, 
   9.341252E+03, 9.396663E+03, 9.452263E+03, 9.508054E+03, 9.564035E+03, 9.620207E+03, 
   9.676570E+03, 9.733124E+03, 9.789869E+03, 9.846806E+03, 9.903935E+03, 9.961255E+03, 
   1.001877E+04, 1.007647E+04, 1.013437E+04, 1.019246E+04, 1.025074E+04, 1.030922E+04, 
   1.036789E+04, 1.042675E+04, 1.048581E+04, 1.054506E+04, 1.060450E+04, 1.066414E+04, 
   1.072397E+04, 1.078400E+04, 1.084422E+04, 1.090464E+04, 1.096526E+04, 1.102606E+04, 
   1.108707E+04, 1.114827E+04, 1.120966E+04, 1.127125E+04, 1.133304E+04, 1.139503E+04, 
   1.145721E+04, 1.151958E+04, 1.158216E+04, 1.164493E+04, 1.170790E+04, 1.177106E+04, 
   1.183443E+04, 1.189799E+04, 1.196175E+04, 1.202571E+04, 1.208986E+04, 1.215422E+04, 
   1.221877E+04, 1.228352E+04, 1.234847E+04, 1.241362E+04, 1.247897E+04, 1.254452E+04, 
   1.261026E+04, 1.267621E+04, 1.274236E+04, 1.280870E+04, 1.287525E+04, 1.294200E+04, 
   1.300894E+04, 1.307609E+04, 1.314344E+04, 1.321099E+04, 1.327874E+04, 1.334669E+04, 
   1.341484E+04, 1.348319E+04, 1.355175E+04, 1.362050E+04, 1.368946E+04, 1.375862E+04, 
   1.382798E+04, 1.389754E+04, 1.396731E+04, 1.403728E+04, 1.410745E+04, 1.417782E+04, 
   1.424840E+04, 1.431918E+04, 1.439016E+04, 1.446134E+04, 1.453273E+04, 1.460432E+04, 
   1.467611E+04, 1.474811E+04, 1.482031E+04, 1.489272E+04, 1.496532E+04, 1.503814E+04, 
   1.511115E+04, 1.518437E+04, 1.525780E+04, 1.533143E+04, 1.540526E+04, 1.547930E+04, 
   1.555354E+04, 1.562798E+04, 1.570264E+04, 1.577749E+04, 1.585255E+04, 1.592782E+04, 
   1.600329E+04, 1.607897E+04, 1.615485E+04, 1.623094E+04, 1.630723E+04, 1.638373E+04, 
   1.646043E+04, 1.653734E+04, 1.661446E+04, 1.669178E+04, 1.676931E+04, 1.684704E+04, 
   1.692498E+04, 1.700313E+04, 1.708148E+04, 1.716004E+04, 1.723880E+04, 1.731777E+04, 
   1.739695E+04, 1.747634E+04, 1.755593E+04, 1.763572E+04, 1.771573E+04, 1.779594E+04, 
   1.787636E+04, 1.795698E+04, 1.803782E+04, 1.811886E+04, 1.820010E+04, 1.828156E+04, 
   1.836322E+04, 1.844509E+04, 1.852716E+04, 1.860944E+04, 1.869193E+04, 1.877463E+04, 
   1.885754E+04, 1.894065E+04, 1.902397E+04, 1.910750E+04, 1.919124E+04, 1.927518E+04, 
   1.935933E+04, 1.944369E+04, 1.952826E+04, 1.961304E+04, 1.969802E+04, 1.978321E+04, 
   1.986861E+04, 1.995422E+04, 2.004004E+04, 2.012606E+04, 2.021229E+04, 2.029873E+04, 
   2.038538E+04, 2.047224E+04, 2.055931E+04, 2.064658E+04, 2.073406E+04, 2.082175E+04, 
   2.090965E+04, 2.099776E+04, 2.108607E+04, 2.117460E+04, 2.126333E+04, 2.135227E+04, 
   2.144142E+04, 2.153078E+04, 2.162034E+04, 2.171012E+04, 2.180010E+04, 2.189029E+04, 
   2.198070E+04, 2.207130E+04, 2.216212E+04, 2.225315E+04, 2.234438E+04, 2.243583E+04, 
   2.252748E+04, 2.261934E+04, 2.271141E+04, 2.280369E+04, 2.289618E+04, 2.298887E+04, 
   2.308177E+04, 2.317489E+04, 2.326821E+04, 2.336174E+04, 2.345548E+04, 2.354942E+04, 
   2.364358E+04, 2.373794E+04, 2.383251E+04, 2.392730E+04, 2.402229E+04, 2.411748E+04, 
   2.421289E+04, 2.430851E+04, 2.440433E+04, 2.450036E+04, 2.459660E+04, 2.469305E+04, 
   2.478971E+04, 2.488657E+04, 2.498365E+04, 2.508093E+04, 2.517842E+04, 2.527612E+04, 
   2.537403E+04, 2.547214E+04, 2.557046E+04, 2.566900E+04, 2.576774E+04, 2.586668E+04, 
   2.596584E+04, 2.606520E+04, 2.616477E+04, 2.626455E+04, 2.636454E+04, 2.646474E+04, 
   2.656514E+04, 2.666575E+04, 2.676657E+04, 2.686760E+04, 2.696883E+04, 2.707028E+04, 
   2.717193E+04, 2.727378E+04, 2.737585E+04, 2.747812E+04, 2.758060E+04, 2.768329E+04, 
   2.778618E+04, 2.788929E+04, 2.799260E+04, 2.809611E+04, 2.819984E+04, 2.830377E+04, 
   2.840791E+04, 2.851225E+04, 2.861680E+04, 2.872156E+04, 2.882653E+04, 2.893170E+04, 
   2.903708E+04, 2.914266E+04, 2.924846E+04, 2.935445E+04, 2.946066E+04, 2.956707E+04, 
   2.967369E+04, 2.978051E+04, 2.988754E+04, 2.999478E+04, 3.010222E+04, 3.020987E+04, 
   3.031773E+04, 3.042579E+04, 3.053405E+04, 3.064253E+04, 3.075120E+04, 3.086009E+04, 
   3.096917E+04, 3.107847E+04, 3.118797E+04, 3.129767E+04, 3.140758E+04, 3.151770E+04, 
   3.162802E+04, 3.173854E+04, 3.184927E+04, 3.196021E+04, 3.207135E+04, 3.218269E+04, 
   3.229424E+04, 3.240600E+04, 3.251795E+04, 3.263012E+04, 3.274248E+04, 3.285505E+04, 
   3.296783E+04, 3.308081E+04, 3.319399E+04, 3.330737E+04, 3.342096E+04, 3.353476E+04, 
   3.364875E+04, 3.376295E+04, 3.387736E+04, 3.399196E+04, 3.410677E+04, 3.422179E+04, 
   3.433700E+04, 3.445242E+04, 3.456804E+04, 3.468387E+04, 3.479989E+04, 3.491612E+04, 
   3.503255E+04, 3.514919E+04, 3.526602E+04, 3.538306E+04, 3.550030E+04, 3.561774E+04, 
   3.573539E+04, 3.585323E+04, 3.597128E+04, 3.608952E+04, 3.620797E+04, 3.632662E+04, 
   3.644547E+04, 3.656453E+04, 3.668378E+04, 3.680323E+04, 3.692289E+04, 3.704274E+04, 
   3.716280E+04, 3.728305E+04, 3.740351E+04, 3.752416E+04, 3.764502E+04, 3.776607E+04, 
   3.788733E+04, 3.800878E+04, 3.813044E+04, 3.825229E+04, 3.837434E+04, 3.849659E+04, 
   3.861904E+04, 3.874169E+04, 3.886454E+04, 3.898758E+04, 3.911083E+04, 3.923427E+04, 
   3.935791E+04, 3.948175E+04, 3.960579E+04, 3.973002E+04, 3.985445E+04, 3.997908E+04, 
   4.010391E+04, 4.022894E+04, 4.035416E+04, 4.047958E+04, 4.060519E+04, 4.073100E+04, 
   4.085701E+04, 4.098322E+04, 4.110962E+04, 4.123622E+04, 4.136301E+04, 4.149000E+04, 
   4.161719E+04, 4.174457E+04, 4.187214E+04, 4.199991E+04, 4.212788E+04, 4.225604E+04, 
   4.238440E+04, 4.251295E+04, 4.264170E+04, 4.277064E+04, 4.289977E+04, 4.302910E+04, 
   4.315863E+04, 4.328834E+04, 4.341825E+04, 4.354836E+04, 4.367866E+04, 4.380915E+04, 
   4.393983E+04, 4.407071E+04, 4.420178E+04, 4.433304E+04, 4.446450E+04, 4.459614E+04, 
   4.472798E+04, 4.486001E+04, 4.499224E+04, 4.512465E+04, 4.525726E+04, 4.539006E+04, 
   4.552305E+04, 4.565623E+04, 4.578960E+04, 4.592316E+04, 4.605692E+04, 4.619086E+04, 
   4.632499E+04, 4.645932E+04, 4.659383E+04, 4.672853E+04, 4.686343E+04, 4.699851E+04, 
   4.713378E+04, 4.726924E+04, 4.740489E+04, 4.754073E+04, 4.767676E+04, 4.781298E+04, 
   4.794938E+04, 4.808598E+04, 4.822276E+04, 4.835973E+04, 4.849688E+04, 4.863423E+04, 
   4.877176E+04, 4.890947E+04, 4.904738E+04, 4.918547E+04, 4.932375E+04, 4.946222E+04, 
   4.960087E+04, 4.973970E+04, 4.987873E+04, 5.001794E+04, 5.015733E+04, 5.029691E+04, 
   5.043668E+04, 5.057663E+04, 5.071676E+04, 5.085708E+04, 5.099758E+04, 5.113827E+04, 
   5.127915E+04, 5.142020E+04, 5.156144E+04, 5.170287E+04, 5.184447E+04, 5.198626E+04, 
   5.212824E+04, 5.227039E+04, 5.241273E+04, 5.255525E+04, 5.269796E+04, 5.284084E+04, 
   5.298391E+04, 5.312716E+04, 5.327059E+04, 5.341421E+04, 5.355800E+04, 5.370197E+04, 
   5.384613E+04, 5.399047E+04, 5.413498E+04, 5.427968E+04, 5.442456E+04, 5.456961E+04, 
   5.471485E+04, 
])
# ============================ M = 7, I = 3 ============================
M = 7
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[8]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.092314E+01, 9.248086E+01, 1.808102E+02, 2.699961E+02, 3.594239E+02, 4.489560E+02, 
   5.385443E+02, 6.281677E+02, 7.178153E+02, 8.074812E+02, 8.971617E+02, 9.868544E+02, 
   1.076558E+03, 1.166271E+03, 1.255992E+03, 1.345723E+03, 1.435461E+03, 1.525209E+03, 
   1.614965E+03, 1.704732E+03, 1.794512E+03, 1.884307E+03, 1.974123E+03, 2.063963E+03, 
   2.153834E+03, 2.243744E+03, 2.333703E+03, 2.423719E+03, 2.513805E+03, 2.603974E+03, 
   2.694239E+03, 2.784616E+03, 2.875119E+03, 2.965765E+03, 3.056572E+03, 3.147557E+03, 
   3.238738E+03, 3.330133E+03, 3.421762E+03, 3.513642E+03, 3.605792E+03, 3.698231E+03, 
   3.790976E+03, 3.884047E+03, 3.977461E+03, 4.071236E+03, 4.165389E+03, 4.259936E+03, 
   4.354895E+03, 4.450280E+03, 4.546108E+03, 4.642394E+03, 4.739152E+03, 4.836397E+03, 
   4.934142E+03, 5.032400E+03, 5.131184E+03, 5.230507E+03, 5.330380E+03, 5.430814E+03, 
   5.531821E+03, 5.633411E+03, 5.735595E+03, 5.838381E+03, 5.941780E+03, 6.045800E+03, 
   6.150451E+03, 6.255739E+03, 6.361674E+03, 6.468263E+03, 6.575513E+03, 6.683432E+03, 
   6.792026E+03, 6.901302E+03, 7.011265E+03, 7.121923E+03, 7.233281E+03, 7.345343E+03, 
   7.458117E+03, 7.571606E+03, 7.685816E+03, 7.800751E+03, 7.916417E+03, 8.032817E+03, 
   8.149955E+03, 8.267837E+03, 8.386465E+03, 8.505843E+03, 8.625976E+03, 8.746866E+03, 
   8.868517E+03, 8.990933E+03, 9.114116E+03, 9.238070E+03, 9.362797E+03, 9.488301E+03, 
   9.614584E+03, 9.741649E+03, 9.869498E+03, 9.998135E+03, 1.012756E+04, 1.025778E+04, 
   1.038879E+04, 1.052060E+04, 1.065321E+04, 1.078662E+04, 1.092083E+04, 1.105584E+04, 
   1.119167E+04, 1.132830E+04, 1.146574E+04, 1.160400E+04, 1.174307E+04, 1.188296E+04, 
   1.202366E+04, 1.216519E+04, 1.230753E+04, 1.245070E+04, 1.259470E+04, 1.273952E+04, 
   1.288517E+04, 1.303164E+04, 1.317895E+04, 1.332710E+04, 1.347607E+04, 1.362589E+04, 
   1.377654E+04, 1.392802E+04, 1.408035E+04, 1.423352E+04, 1.438754E+04, 1.454239E+04, 
   1.469810E+04, 1.485465E+04, 1.501205E+04, 1.517030E+04, 1.532940E+04, 1.548935E+04, 
   1.565016E+04, 1.581183E+04, 1.597435E+04, 1.613773E+04, 1.630197E+04, 1.646707E+04, 
   1.663303E+04, 1.679986E+04, 1.696755E+04, 1.713611E+04, 1.730554E+04, 1.747583E+04, 
   1.764700E+04, 1.781904E+04, 1.799195E+04, 1.816574E+04, 1.834041E+04, 1.851595E+04, 
   1.869237E+04, 1.886967E+04, 1.904786E+04, 1.922692E+04, 1.940687E+04, 1.958771E+04, 
   1.976944E+04, 1.995205E+04, 2.013556E+04, 2.031996E+04, 2.050525E+04, 2.069143E+04, 
   2.087851E+04, 2.106649E+04, 2.125537E+04, 2.144515E+04, 2.163583E+04, 2.182741E+04, 
   2.201990E+04, 2.221330E+04, 2.240760E+04, 2.260281E+04, 2.279894E+04, 2.299597E+04, 
   2.319392E+04, 2.339279E+04, 2.359257E+04, 2.379327E+04, 2.399490E+04, 2.419744E+04, 
   2.440090E+04, 2.460530E+04, 2.481061E+04, 2.501686E+04, 2.522403E+04, 2.543214E+04, 
   2.564117E+04, 2.585114E+04, 2.606205E+04, 2.627389E+04, 2.648668E+04, 2.670040E+04, 
   2.691506E+04, 2.713067E+04, 2.734723E+04, 2.756472E+04, 2.778317E+04, 2.800257E+04, 
   2.822292E+04, 2.844422E+04, 2.866648E+04, 2.888969E+04, 2.911386E+04, 2.933899E+04, 
   2.956508E+04, 2.979213E+04, 3.002015E+04, 3.024913E+04, 3.047908E+04, 3.071000E+04, 
   3.094189E+04, 3.117475E+04, 3.140859E+04, 3.164340E+04, 3.187919E+04, 3.211595E+04, 
   3.235370E+04, 3.259243E+04, 3.283214E+04, 3.307284E+04, 3.331452E+04, 3.355719E+04, 
   3.380086E+04, 3.404551E+04, 3.429116E+04, 3.453780E+04, 3.478544E+04, 3.503407E+04, 
   3.528371E+04, 3.553435E+04, 3.578598E+04, 3.603863E+04, 3.629228E+04, 3.654694E+04, 
   3.680260E+04, 3.705928E+04, 3.731697E+04, 3.757567E+04, 3.783539E+04, 3.809613E+04, 
   3.835788E+04, 3.862065E+04, 3.888445E+04, 3.914927E+04, 3.941511E+04, 3.968198E+04, 
   3.994988E+04, 4.021881E+04, 4.048877E+04, 4.075976E+04, 4.103179E+04, 4.130485E+04, 
   4.157894E+04, 4.185408E+04, 4.213026E+04, 4.240748E+04, 4.268574E+04, 4.296505E+04, 
   4.324540E+04, 4.352680E+04, 4.380925E+04, 4.409275E+04, 4.437731E+04, 4.466291E+04, 
   4.494957E+04, 4.523729E+04, 4.552607E+04, 4.581591E+04, 4.610680E+04, 4.639876E+04, 
   4.669179E+04, 4.698587E+04, 4.728103E+04, 4.757725E+04, 4.787455E+04, 4.817291E+04, 
   4.847235E+04, 4.877286E+04, 4.907444E+04, 4.937710E+04, 4.968084E+04, 4.998566E+04, 
   5.029156E+04, 5.059855E+04, 5.090661E+04, 5.121576E+04, 5.152600E+04, 5.183732E+04, 
   5.214974E+04, 5.246324E+04, 5.277783E+04, 5.309352E+04, 5.341030E+04, 5.372818E+04, 
   5.404715E+04, 5.436722E+04, 5.468839E+04, 5.501066E+04, 5.533403E+04, 5.565851E+04, 
   5.598409E+04, 5.631077E+04, 5.663857E+04, 5.696747E+04, 5.729748E+04, 5.762860E+04, 
   5.796083E+04, 5.829417E+04, 5.862863E+04, 5.896420E+04, 5.930089E+04, 5.963870E+04, 
   5.997762E+04, 6.031767E+04, 6.065884E+04, 6.100113E+04, 6.134454E+04, 6.168908E+04, 
   6.203474E+04, 6.238153E+04, 6.272945E+04, 6.307849E+04, 6.342867E+04, 6.377998E+04, 
   6.413242E+04, 6.448599E+04, 6.484070E+04, 6.519654E+04, 6.555352E+04, 6.591163E+04, 
   6.627089E+04, 6.663129E+04, 6.699282E+04, 6.735550E+04, 6.771932E+04, 6.808428E+04, 
   6.845039E+04, 6.881764E+04, 6.918604E+04, 6.955559E+04, 6.992629E+04, 7.029813E+04, 
   7.067113E+04, 7.104528E+04, 7.142058E+04, 7.179703E+04, 7.217464E+04, 7.255340E+04, 
   7.293332E+04, 7.331440E+04, 7.369663E+04, 7.408002E+04, 7.446457E+04, 7.485029E+04, 
   7.523716E+04, 7.562519E+04, 7.601439E+04, 7.640476E+04, 7.679628E+04, 7.718897E+04, 
   7.758283E+04, 7.797786E+04, 7.837405E+04, 7.877142E+04, 7.916995E+04, 7.956965E+04, 
   7.997052E+04, 8.037257E+04, 8.077579E+04, 8.118018E+04, 8.158575E+04, 8.199249E+04, 
   8.240040E+04, 8.280949E+04, 8.321976E+04, 8.363121E+04, 8.404384E+04, 8.445764E+04, 
   8.487263E+04, 8.528879E+04, 8.570614E+04, 8.612466E+04, 8.654437E+04, 8.696527E+04, 
   8.738735E+04, 8.781061E+04, 8.823506E+04, 8.866069E+04, 8.908751E+04, 8.951551E+04, 
   8.994471E+04, 9.037509E+04, 9.080666E+04, 9.123942E+04, 9.167337E+04, 9.210851E+04, 
   9.254484E+04, 9.298236E+04, 9.342108E+04, 9.386098E+04, 9.430209E+04, 9.474438E+04, 
   9.518787E+04, 9.563255E+04, 9.607843E+04, 9.652551E+04, 9.697378E+04, 9.742324E+04, 
   9.787391E+04, 9.832577E+04, 9.877883E+04, 9.923309E+04, 9.968855E+04, 1.001452E+05, 
   1.006031E+05, 1.010621E+05, 1.015224E+05, 1.019838E+05, 1.024465E+05, 1.029104E+05, 
   1.033754E+05, 1.038417E+05, 1.043092E+05, 1.047779E+05, 1.052477E+05, 1.057188E+05, 
   1.061911E+05, 1.066646E+05, 1.071393E+05, 1.076152E+05, 1.080923E+05, 1.085706E+05, 
   1.090502E+05, 1.095309E+05, 1.100128E+05, 1.104960E+05, 1.109803E+05, 1.114659E+05, 
   1.119527E+05, 1.124407E+05, 1.129299E+05, 1.134203E+05, 1.139119E+05, 1.144047E+05, 
   1.148987E+05, 1.153940E+05, 1.158904E+05, 1.163881E+05, 1.168869E+05, 1.173870E+05, 
   1.178883E+05, 1.183908E+05, 1.188945E+05, 1.193995E+05, 1.199056E+05, 1.204130E+05, 
   1.209215E+05, 1.214313E+05, 1.219423E+05, 1.224545E+05, 1.229679E+05, 1.234826E+05, 
   1.239984E+05, 1.245155E+05, 1.250338E+05, 1.255532E+05, 1.260739E+05, 1.265959E+05, 
   1.271190E+05, 1.276433E+05, 1.281689E+05, 1.286957E+05, 1.292237E+05, 1.297529E+05, 
   1.302833E+05, 1.308149E+05, 1.313477E+05, 1.318818E+05, 1.324171E+05, 1.329536E+05, 
   1.334913E+05, 1.340302E+05, 1.345703E+05, 1.351117E+05, 1.356543E+05, 1.361980E+05, 
   1.367430E+05, 1.372892E+05, 1.378367E+05, 1.383853E+05, 1.389352E+05, 1.394862E+05, 
   1.400385E+05, 1.405920E+05, 1.411467E+05, 1.417027E+05, 1.422598E+05, 1.428182E+05, 
   1.433777E+05, 1.439385E+05, 1.445005E+05, 1.450637E+05, 1.456282E+05, 1.461938E+05, 
   1.467607E+05, 1.473287E+05, 1.478980E+05, 1.484685E+05, 1.490402E+05, 1.496132E+05, 
   1.501873E+05, 1.507626E+05, 1.513392E+05, 1.519170E+05, 1.524960E+05, 1.530762E+05, 
   1.536576E+05, 1.542402E+05, 1.548240E+05, 1.554091E+05, 1.559954E+05, 1.565828E+05, 
   1.571715E+05, 1.577614E+05, 1.583525E+05, 1.589448E+05, 1.595383E+05, 1.601331E+05, 
   1.607290E+05, 1.613262E+05, 1.619245E+05, 1.625241E+05, 1.631249E+05, 1.637269E+05, 
   1.643301E+05, 1.649345E+05, 1.655401E+05, 1.661469E+05, 1.667549E+05, 1.673642E+05, 
   1.679746E+05, 1.685863E+05, 1.691991E+05, 1.698132E+05, 1.704284E+05, 1.710449E+05, 
   1.716626E+05, 1.722814E+05, 1.729015E+05, 1.735228E+05, 1.741453E+05, 1.747690E+05, 
   1.753939E+05, 1.760200E+05, 1.766473E+05, 1.772758E+05, 1.779055E+05, 1.785364E+05, 
   1.791685E+05, 1.798018E+05, 1.804363E+05, 1.810720E+05, 1.817088E+05, 1.823469E+05, 
   1.829862E+05, 1.836267E+05, 1.842684E+05, 1.849113E+05, 1.855554E+05, 1.862006E+05, 
   1.868471E+05, 1.874948E+05, 1.881436E+05, 1.887937E+05, 1.894449E+05, 1.900974E+05, 
   1.907510E+05, 1.914058E+05, 1.920619E+05, 1.927191E+05, 1.933775E+05, 1.940370E+05, 
   1.946978E+05, 1.953598E+05, 1.960229E+05, 1.966873E+05, 1.973528E+05, 1.980195E+05, 
   1.986874E+05, 1.993565E+05, 2.000268E+05, 2.006983E+05, 2.013709E+05, 2.020447E+05, 
   2.027197E+05, 2.033959E+05, 2.040733E+05, 2.047519E+05, 2.054316E+05, 2.061125E+05, 
   2.067946E+05, 2.074779E+05, 2.081624E+05, 2.088480E+05, 2.095348E+05, 2.102228E+05, 
   2.109120E+05, 2.116023E+05, 2.122938E+05, 2.129865E+05, 2.136804E+05, 2.143754E+05, 
   2.150716E+05, 2.157690E+05, 2.164676E+05, 2.171673E+05, 2.178682E+05, 2.185702E+05, 
   2.192735E+05, 2.199779E+05, 2.206834E+05, 2.213902E+05, 2.220981E+05, 2.228071E+05, 
   2.235174E+05, 2.242287E+05, 2.249413E+05, 2.256550E+05, 2.263699E+05, 2.270859E+05, 
   2.278031E+05, 2.285215E+05, 2.292410E+05, 2.299617E+05, 2.306835E+05, 2.314065E+05, 
   2.321307E+05, 2.328560E+05, 2.335825E+05, 2.343101E+05, 2.350388E+05, 2.357688E+05, 
   2.364998E+05, 2.372320E+05, 2.379654E+05, 2.386999E+05, 2.394356E+05, 2.401724E+05, 
   2.409104E+05, 2.416495E+05, 2.423898E+05, 2.431312E+05, 2.438737E+05, 2.446174E+05, 
   2.453622E+05, 2.461082E+05, 2.468553E+05, 2.476036E+05, 2.483530E+05, 2.491035E+05, 
   2.498552E+05, 2.506080E+05, 2.513619E+05, 2.521170E+05, 2.528732E+05, 2.536305E+05, 
   2.543890E+05, 2.551486E+05, 2.559094E+05, 2.566712E+05, 2.574342E+05, 2.581984E+05, 
   2.589636E+05, 2.597300E+05, 2.604975E+05, 2.612661E+05, 2.620359E+05, 2.628068E+05, 
   2.635788E+05, 2.643519E+05, 2.651261E+05, 2.659015E+05, 2.666780E+05, 2.674556E+05, 
   2.682343E+05, 2.690141E+05, 2.697950E+05, 2.705771E+05, 2.713603E+05, 2.721446E+05, 
   2.729299E+05, 2.737165E+05, 2.745041E+05, 2.752928E+05, 2.760826E+05, 2.768736E+05, 
   2.776656E+05, 2.784588E+05, 2.792530E+05, 2.800484E+05, 2.808448E+05, 2.816424E+05, 
   2.824410E+05, 2.832408E+05, 2.840416E+05, 2.848436E+05, 2.856466E+05, 2.864508E+05, 
   2.872560E+05, 2.880624E+05, 2.888698E+05, 2.896783E+05, 2.904879E+05, 2.912986E+05, 
   2.921104E+05, 2.929232E+05, 2.937372E+05, 2.945522E+05, 2.953684E+05, 2.961856E+05, 
   2.970038E+05, 2.978232E+05, 2.986437E+05, 2.994652E+05, 3.002878E+05, 3.011115E+05, 
   3.019362E+05, 3.027621E+05, 3.035890E+05, 3.044170E+05, 3.052460E+05, 3.060762E+05, 
   3.069073E+05, 3.077396E+05, 3.085729E+05, 3.094073E+05, 3.102428E+05, 3.110793E+05, 
   3.119169E+05, 3.127556E+05, 3.135953E+05, 3.144361E+05, 3.152779E+05, 3.161208E+05, 
   3.169647E+05, 
])
# ============================ M = 7, I = 4 ============================
#M = 7
#I = 4
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[8]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.270969E+00, 8.859605E+00, 1.687773E+01, 2.498788E+01, 3.312258E+01, 4.126759E+01, 
#   4.941808E+01, 5.757195E+01, 6.572814E+01, 7.388606E+01, 8.204535E+01, 9.020578E+01, 
#   9.836721E+01, 1.065295E+02, 1.146927E+02, 1.228566E+02, 1.310214E+02, 1.391870E+02, 
#   1.473535E+02, 1.555212E+02, 1.636904E+02, 1.718613E+02, 1.800346E+02, 1.882108E+02, 
#   1.963907E+02, 2.045752E+02, 2.127654E+02, 2.209623E+02, 2.291674E+02, 2.373819E+02, 
#   2.456073E+02, 2.538453E+02, 2.620976E+02, 2.703657E+02, 2.786516E+02, 2.869569E+02, 
#   2.952836E+02, 3.036334E+02, 3.120083E+02, 3.204100E+02, 3.288405E+02, 3.373014E+02, 
#   3.457946E+02, 3.543218E+02, 3.628847E+02, 3.714850E+02, 3.801244E+02, 3.888044E+02, 
#   3.975266E+02, 4.062923E+02, 4.151032E+02, 4.239606E+02, 4.328657E+02, 4.418200E+02, 
#   4.508247E+02, 4.598809E+02, 4.689898E+02, 4.781525E+02, 4.873702E+02, 4.966437E+02, 
#   5.059742E+02, 5.153625E+02, 5.248095E+02, 5.343161E+02, 5.438832E+02, 5.535115E+02, 
#   5.632018E+02, 5.729548E+02, 5.827712E+02, 5.926518E+02, 6.025970E+02, 6.126077E+02, 
#   6.226842E+02, 6.328273E+02, 6.430374E+02, 6.533151E+02, 6.636609E+02, 6.740753E+02, 
#   6.845586E+02, 6.951115E+02, 7.057342E+02, 7.164272E+02, 7.271909E+02, 7.380256E+02, 
#   7.489318E+02, 7.599098E+02, 7.709599E+02, 7.820824E+02, 7.932777E+02, 8.045461E+02, 
#   8.158878E+02, 8.273031E+02, 8.387923E+02, 8.503557E+02, 8.619935E+02, 8.737060E+02, 
#   8.854934E+02, 8.973559E+02, 9.092938E+02, 9.213072E+02, 9.333964E+02, 9.455617E+02, 
#   9.578031E+02, 9.701209E+02, 9.825153E+02, 9.949864E+02, 1.007535E+03, 1.020160E+03, 
#   1.032862E+03, 1.045642E+03, 1.058500E+03, 1.071435E+03, 1.084448E+03, 1.097540E+03, 
#   1.110709E+03, 1.123957E+03, 1.137284E+03, 1.150689E+03, 1.164173E+03, 1.177736E+03, 
#   1.191378E+03, 1.205100E+03, 1.218900E+03, 1.232781E+03, 1.246741E+03, 1.260780E+03, 
#   1.274900E+03, 1.289100E+03, 1.303379E+03, 1.317740E+03, 1.332180E+03, 1.346701E+03, 
#   1.361303E+03, 1.375985E+03, 1.390749E+03, 1.405593E+03, 1.420519E+03, 1.435526E+03, 
#   1.450614E+03, 1.465784E+03, 1.481036E+03, 1.496369E+03, 1.511784E+03, 1.527282E+03, 
#   1.542861E+03, 1.558523E+03, 1.574267E+03, 1.590093E+03, 1.606003E+03, 1.621995E+03, 
#   1.638069E+03, 1.654227E+03, 1.670468E+03, 1.686792E+03, 1.703200E+03, 1.719691E+03, 
#   1.736266E+03, 1.752924E+03, 1.769667E+03, 1.786493E+03, 1.803403E+03, 1.820398E+03, 
#   1.837477E+03, 1.854641E+03, 1.871889E+03, 1.889222E+03, 1.906639E+03, 1.924142E+03, 
#   1.941730E+03, 1.959403E+03, 1.977162E+03, 1.995006E+03, 2.012936E+03, 2.030952E+03, 
#   2.049053E+03, 2.067241E+03, 2.085515E+03, 2.103875E+03, 2.122321E+03, 2.140854E+03, 
#   2.159474E+03, 2.178181E+03, 2.196974E+03, 2.215855E+03, 2.234823E+03, 2.253878E+03, 
#   2.273021E+03, 2.292252E+03, 2.311570E+03, 2.330976E+03, 2.350471E+03, 2.370053E+03, 
#   2.389724E+03, 2.409483E+03, 2.429331E+03, 2.449268E+03, 2.469293E+03, 2.489408E+03, 
#   2.509612E+03, 2.529904E+03, 2.550287E+03, 2.570759E+03, 2.591320E+03, 2.611972E+03, 
#   2.632713E+03, 2.653545E+03, 2.674467E+03, 2.695479E+03, 2.716582E+03, 2.737775E+03, 
#   2.759059E+03, 2.780434E+03, 2.801900E+03, 2.823458E+03, 2.845107E+03, 2.866847E+03, 
#   2.888679E+03, 2.910602E+03, 2.932618E+03, 2.954725E+03, 2.976925E+03, 2.999217E+03, 
#   3.021602E+03, 3.044079E+03, 3.066649E+03, 3.089311E+03, 3.112067E+03, 3.134916E+03, 
#   3.157858E+03, 3.180893E+03, 3.204022E+03, 3.227245E+03, 3.250562E+03, 3.273972E+03, 
#   3.297476E+03, 3.321075E+03, 3.344768E+03, 3.368556E+03, 3.392438E+03, 3.416414E+03, 
#   3.440486E+03, 3.464653E+03, 3.488915E+03, 3.513272E+03, 3.537724E+03, 3.562272E+03, 
#   3.586916E+03, 3.611655E+03, 3.636491E+03, 3.661422E+03, 3.686450E+03, 3.711573E+03, 
#   3.736794E+03, 3.762110E+03, 3.787524E+03, 3.813034E+03, 3.838641E+03, 3.864346E+03, 
#   3.890147E+03, 3.916046E+03, 3.942042E+03, 3.968136E+03, 3.994327E+03, 4.020616E+03, 
#   4.047003E+03, 4.073488E+03, 4.100071E+03, 4.126753E+03, 4.153533E+03, 4.180411E+03, 
#   4.207388E+03, 4.234464E+03, 4.261638E+03, 4.288912E+03, 4.316284E+03, 4.343756E+03, 
#   4.371327E+03, 4.398998E+03, 4.426768E+03, 4.454638E+03, 4.482608E+03, 4.510677E+03, 
#   4.538846E+03, 4.567116E+03, 4.595486E+03, 4.623956E+03, 4.652526E+03, 4.681197E+03, 
#   4.709969E+03, 4.738841E+03, 4.767815E+03, 4.796889E+03, 4.826064E+03, 4.855341E+03, 
#   4.884718E+03, 4.914198E+03, 4.943778E+03, 4.973460E+03, 5.003244E+03, 5.033130E+03, 
#   5.063117E+03, 5.093207E+03, 5.123398E+03, 5.153692E+03, 5.184088E+03, 5.214586E+03, 
#   5.245187E+03, 5.275891E+03, 5.306697E+03, 5.337605E+03, 5.368617E+03, 5.399731E+03, 
#   5.430949E+03, 5.462270E+03, 5.493693E+03, 5.525221E+03, 5.556851E+03, 5.588585E+03, 
#   5.620423E+03, 5.652364E+03, 5.684409E+03, 5.716557E+03, 5.748810E+03, 5.781166E+03, 
#   5.813627E+03, 5.846192E+03, 5.878861E+03, 5.911634E+03, 5.944512E+03, 5.977494E+03, 
#   6.010580E+03, 6.043772E+03, 6.077068E+03, 6.110469E+03, 6.143974E+03, 6.177585E+03, 
#   6.211300E+03, 6.245121E+03, 6.279047E+03, 6.313078E+03, 6.347214E+03, 6.381456E+03, 
#   6.415803E+03, 6.450256E+03, 6.484814E+03, 6.519478E+03, 6.554247E+03, 6.589123E+03, 
#   6.624104E+03, 6.659191E+03, 6.694384E+03, 6.729683E+03, 6.765089E+03, 6.800600E+03, 
#   6.836218E+03, 6.871942E+03, 6.907772E+03, 6.943709E+03, 6.979752E+03, 7.015902E+03, 
#   7.052159E+03, 7.088522E+03, 7.124992E+03, 7.161568E+03, 7.198252E+03, 7.235042E+03, 
#   7.271939E+03, 7.308943E+03, 7.346055E+03, 7.383273E+03, 7.420599E+03, 7.458031E+03, 
#   7.495571E+03, 7.533219E+03, 7.570973E+03, 7.608835E+03, 7.646805E+03, 7.684882E+03, 
#   7.723066E+03, 7.761359E+03, 7.799758E+03, 7.838266E+03, 7.876881E+03, 7.915604E+03, 
#   7.954435E+03, 7.993374E+03, 8.032420E+03, 8.071575E+03, 8.110837E+03, 8.150208E+03, 
#   8.189686E+03, 8.229273E+03, 8.268968E+03, 8.308771E+03, 8.348682E+03, 8.388702E+03, 
#   8.428830E+03, 8.469066E+03, 8.509410E+03, 8.549863E+03, 8.590424E+03, 8.631094E+03, 
#   8.671872E+03, 8.712759E+03, 8.753754E+03, 8.794858E+03, 8.836071E+03, 8.877392E+03, 
#   8.918822E+03, 8.960360E+03, 9.002007E+03, 9.043763E+03, 9.085628E+03, 9.127602E+03, 
#   9.169684E+03, 9.211875E+03, 9.254175E+03, 9.296584E+03, 9.339102E+03, 9.381729E+03, 
#   9.424465E+03, 9.467310E+03, 9.510263E+03, 9.553326E+03, 9.596498E+03, 9.639779E+03, 
#   9.683169E+03, 9.726668E+03, 9.770276E+03, 9.813994E+03, 9.857820E+03, 9.901756E+03, 
#   9.945801E+03, 9.989955E+03, 1.003422E+04, 1.007859E+04, 1.012307E+04, 1.016766E+04, 
#   1.021236E+04, 1.025717E+04, 1.030209E+04, 1.034712E+04, 1.039226E+04, 1.043750E+04, 
#   1.048286E+04, 1.052832E+04, 1.057390E+04, 1.061958E+04, 1.066537E+04, 1.071128E+04, 
#   1.075729E+04, 1.080341E+04, 1.084964E+04, 1.089598E+04, 1.094243E+04, 1.098899E+04, 
#   1.103566E+04, 1.108243E+04, 1.112932E+04, 1.117631E+04, 1.122342E+04, 1.127063E+04, 
#   1.131796E+04, 1.136539E+04, 1.141293E+04, 1.146059E+04, 1.150835E+04, 1.155622E+04, 
#   1.160420E+04, 1.165229E+04, 1.170048E+04, 1.174879E+04, 1.179721E+04, 1.184574E+04, 
#   1.189437E+04, 1.194312E+04, 1.199197E+04, 1.204093E+04, 1.209001E+04, 1.213919E+04, 
#   1.218848E+04, 1.223788E+04, 1.228739E+04, 1.233701E+04, 1.238673E+04, 1.243657E+04, 
#   1.248652E+04, 1.253657E+04, 1.258674E+04, 1.263701E+04, 1.268739E+04, 1.273788E+04, 
#   1.278848E+04, 1.283919E+04, 1.289001E+04, 1.294094E+04, 1.299198E+04, 1.304312E+04, 
#   1.309438E+04, 1.314574E+04, 1.319721E+04, 1.324879E+04, 1.330048E+04, 1.335228E+04, 
#   1.340419E+04, 1.345620E+04, 1.350833E+04, 1.356056E+04, 1.361290E+04, 1.366535E+04, 
#   1.371791E+04, 1.377058E+04, 1.382336E+04, 1.387624E+04, 1.392924E+04, 1.398234E+04, 
#   1.403555E+04, 1.408887E+04, 1.414230E+04, 1.419583E+04, 1.424948E+04, 1.430323E+04, 
#   1.435709E+04, 1.441106E+04, 1.446514E+04, 1.451932E+04, 1.457362E+04, 1.462802E+04, 
#   1.468253E+04, 1.473715E+04, 1.479187E+04, 1.484670E+04, 1.490165E+04, 1.495670E+04, 
#   1.501185E+04, 1.506712E+04, 1.512249E+04, 1.517797E+04, 1.523356E+04, 1.528926E+04, 
#   1.534506E+04, 1.540097E+04, 1.545699E+04, 1.551312E+04, 1.556935E+04, 1.562569E+04, 
#   1.568214E+04, 1.573870E+04, 1.579536E+04, 1.585213E+04, 1.590901E+04, 1.596599E+04, 
#   1.602309E+04, 1.608029E+04, 1.613759E+04, 1.619501E+04, 1.625253E+04, 1.631015E+04, 
#   1.636789E+04, 1.642573E+04, 1.648367E+04, 1.654173E+04, 1.659989E+04, 1.665815E+04, 
#   1.671653E+04, 1.677501E+04, 1.683360E+04, 1.689229E+04, 1.695109E+04, 1.700999E+04, 
#   1.706900E+04, 1.712812E+04, 1.718735E+04, 1.724668E+04, 1.730611E+04, 1.736565E+04, 
#   1.742530E+04, 1.748506E+04, 1.754492E+04, 1.760488E+04, 1.766495E+04, 1.772513E+04, 
#   1.778541E+04, 1.784580E+04, 1.790629E+04, 1.796689E+04, 1.802759E+04, 1.808840E+04, 
#   1.814932E+04, 1.821034E+04, 1.827146E+04, 1.833269E+04, 1.839402E+04, 1.845546E+04, 
#   1.851701E+04, 1.857866E+04, 1.864041E+04, 1.870227E+04, 1.876423E+04, 1.882630E+04, 
#   1.888847E+04, 1.895074E+04, 1.901313E+04, 1.907561E+04, 1.913820E+04, 1.920089E+04, 
#   1.926369E+04, 1.932659E+04, 1.938959E+04, 1.945270E+04, 1.951592E+04, 1.957923E+04, 
#   1.964265E+04, 1.970618E+04, 1.976980E+04, 1.983353E+04, 1.989737E+04, 1.996130E+04, 
#   2.002534E+04, 2.008949E+04, 2.015373E+04, 2.021808E+04, 2.028253E+04, 2.034709E+04, 
#   2.041175E+04, 2.047651E+04, 2.054137E+04, 2.060634E+04, 2.067141E+04, 2.073658E+04, 
#   2.080185E+04, 2.086723E+04, 2.093270E+04, 2.099828E+04, 2.106396E+04, 2.112975E+04, 
#   2.119563E+04, 2.126162E+04, 2.132771E+04, 2.139390E+04, 2.146019E+04, 2.152659E+04, 
#   2.159308E+04, 2.165968E+04, 2.172638E+04, 2.179317E+04, 2.186007E+04, 2.192708E+04, 
#   2.199418E+04, 2.206138E+04, 2.212868E+04, 2.219609E+04, 2.226359E+04, 2.233120E+04, 
#   2.239890E+04, 2.246671E+04, 2.253461E+04, 2.260262E+04, 2.267073E+04, 2.273893E+04, 
#   2.280724E+04, 2.287565E+04, 2.294415E+04, 2.301276E+04, 2.308146E+04, 2.315027E+04, 
#   2.321917E+04, 2.328817E+04, 2.335728E+04, 2.342648E+04, 2.349578E+04, 2.356518E+04, 
#   2.363468E+04, 2.370427E+04, 2.377397E+04, 2.384376E+04, 2.391366E+04, 2.398365E+04, 
#   2.405374E+04, 2.412393E+04, 2.419421E+04, 2.426460E+04, 2.433508E+04, 2.440566E+04, 
#   2.447634E+04, 2.454711E+04, 2.461799E+04, 2.468896E+04, 2.476002E+04, 2.483119E+04, 
#   2.490245E+04, 2.497381E+04, 2.504527E+04, 2.511682E+04, 2.518847E+04, 2.526022E+04, 
#   2.533206E+04, 2.540400E+04, 2.547604E+04, 2.554817E+04, 2.562040E+04, 2.569273E+04, 
#   2.576515E+04, 2.583767E+04, 2.591028E+04, 2.598299E+04, 2.605580E+04, 2.612870E+04, 
#   2.620169E+04, 2.627478E+04, 2.634797E+04, 2.642125E+04, 2.649463E+04, 2.656810E+04, 
#   2.664167E+04, 2.671533E+04, 2.678908E+04, 2.686294E+04, 2.693688E+04, 2.701092E+04, 
#   2.708505E+04, 2.715928E+04, 2.723360E+04, 2.730802E+04, 2.738253E+04, 2.745713E+04, 
#   2.753183E+04, 2.760662E+04, 2.768151E+04, 2.775648E+04, 2.783155E+04, 2.790672E+04, 
#   2.798198E+04, 2.805733E+04, 2.813277E+04, 2.820830E+04, 2.828393E+04, 2.835965E+04, 
#   2.843547E+04, 2.851137E+04, 2.858737E+04, 2.866346E+04, 2.873964E+04, 2.881591E+04, 
#   2.889228E+04, 2.896873E+04, 2.904528E+04, 2.912192E+04, 2.919865E+04, 2.927548E+04, 
#   2.935239E+04, 
#])
# ============================ M = 7, I = 5 ============================
#M = 7
#I = 5
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[8]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   2.157469E+01, 9.733414E+01, 1.910172E+02, 2.856143E+02, 3.804688E+02, 4.754342E+02, 
#   5.704593E+02, 6.655216E+02, 7.606097E+02, 8.557171E+02, 9.508401E+02, 1.045976E+03, 
#   1.141123E+03, 1.236281E+03, 1.331447E+03, 1.426623E+03, 1.521808E+03, 1.617003E+03, 
#   1.712208E+03, 1.807426E+03, 1.902659E+03, 1.997912E+03, 2.093190E+03, 2.188498E+03, 
#   2.283847E+03, 2.379244E+03, 2.474702E+03, 2.570233E+03, 2.665851E+03, 2.761571E+03, 
#   2.857410E+03, 2.953385E+03, 3.049515E+03, 3.145819E+03, 3.242316E+03, 3.339028E+03, 
#   3.435973E+03, 3.533175E+03, 3.630652E+03, 3.728427E+03, 3.826519E+03, 3.924950E+03, 
#   4.023741E+03, 4.122909E+03, 4.222477E+03, 4.322462E+03, 4.422884E+03, 4.523761E+03, 
#   4.625110E+03, 4.726949E+03, 4.829294E+03, 4.932163E+03, 5.035569E+03, 5.139530E+03, 
#   5.244058E+03, 5.349168E+03, 5.454873E+03, 5.561187E+03, 5.668122E+03, 5.775690E+03, 
#   5.883902E+03, 5.992770E+03, 6.102304E+03, 6.212514E+03, 6.323410E+03, 6.435002E+03, 
#   6.547298E+03, 6.660307E+03, 6.774037E+03, 6.888497E+03, 7.003694E+03, 7.119634E+03, 
#   7.236326E+03, 7.353776E+03, 7.471990E+03, 7.590974E+03, 7.710735E+03, 7.831278E+03, 
#   7.952609E+03, 8.074732E+03, 8.197654E+03, 8.321378E+03, 8.445909E+03, 8.571253E+03, 
#   8.697412E+03, 8.824393E+03, 8.952197E+03, 9.080830E+03, 9.210295E+03, 9.340596E+03, 
#   9.471735E+03, 9.603717E+03, 9.736545E+03, 9.870222E+03, 1.000475E+04, 1.014013E+04, 
#   1.027637E+04, 1.041348E+04, 1.055144E+04, 1.069027E+04, 1.082997E+04, 1.097054E+04, 
#   1.111198E+04, 1.125429E+04, 1.139749E+04, 1.154156E+04, 1.168652E+04, 1.183236E+04, 
#   1.197908E+04, 1.212670E+04, 1.227520E+04, 1.242459E+04, 1.257488E+04, 1.272607E+04, 
#   1.287815E+04, 1.303113E+04, 1.318501E+04, 1.333979E+04, 1.349548E+04, 1.365208E+04, 
#   1.380958E+04, 1.396799E+04, 1.412731E+04, 1.428755E+04, 1.444869E+04, 1.461076E+04, 
#   1.477374E+04, 1.493764E+04, 1.510246E+04, 1.526820E+04, 1.543486E+04, 1.560245E+04, 
#   1.577096E+04, 1.594041E+04, 1.611078E+04, 1.628208E+04, 1.645431E+04, 1.662748E+04, 
#   1.680158E+04, 1.697661E+04, 1.715259E+04, 1.732950E+04, 1.750736E+04, 1.768615E+04, 
#   1.786589E+04, 1.804658E+04, 1.822821E+04, 1.841078E+04, 1.859431E+04, 1.877879E+04, 
#   1.896422E+04, 1.915060E+04, 1.933794E+04, 1.952623E+04, 1.971548E+04, 1.990569E+04, 
#   2.009687E+04, 2.028900E+04, 2.048210E+04, 2.067616E+04, 2.087119E+04, 2.106718E+04, 
#   2.126415E+04, 2.146209E+04, 2.166100E+04, 2.186088E+04, 2.206174E+04, 2.226357E+04, 
#   2.246639E+04, 2.267018E+04, 2.287496E+04, 2.308072E+04, 2.328746E+04, 2.349519E+04, 
#   2.370391E+04, 2.391361E+04, 2.412431E+04, 2.433600E+04, 2.454868E+04, 2.476236E+04, 
#   2.497703E+04, 2.519271E+04, 2.540938E+04, 2.562706E+04, 2.584574E+04, 2.606542E+04, 
#   2.628611E+04, 2.650781E+04, 2.673051E+04, 2.695423E+04, 2.717896E+04, 2.740471E+04, 
#   2.763147E+04, 2.785925E+04, 2.808804E+04, 2.831786E+04, 2.854870E+04, 2.878057E+04, 
#   2.901346E+04, 2.924738E+04, 2.948232E+04, 2.971830E+04, 2.995531E+04, 3.019335E+04, 
#   3.043243E+04, 3.067255E+04, 3.091370E+04, 3.115589E+04, 3.139913E+04, 3.164341E+04, 
#   3.188873E+04, 3.213510E+04, 3.238252E+04, 3.263099E+04, 3.288051E+04, 3.313109E+04, 
#   3.338272E+04, 3.363540E+04, 3.388915E+04, 3.414395E+04, 3.439982E+04, 3.465675E+04, 
#   3.491474E+04, 3.517380E+04, 3.543392E+04, 3.569512E+04, 3.595739E+04, 3.622073E+04, 
#   3.648514E+04, 3.675063E+04, 3.701720E+04, 3.728485E+04, 3.755358E+04, 3.782339E+04, 
#   3.809428E+04, 3.836626E+04, 3.863933E+04, 3.891349E+04, 3.918873E+04, 3.946507E+04, 
#   3.974250E+04, 4.002103E+04, 4.030065E+04, 4.058137E+04, 4.086319E+04, 4.114612E+04, 
#   4.143014E+04, 4.171527E+04, 4.200151E+04, 4.228885E+04, 4.257730E+04, 4.286686E+04, 
#   4.315754E+04, 4.344933E+04, 4.374223E+04, 4.403625E+04, 4.433139E+04, 4.462764E+04, 
#   4.492502E+04, 4.522352E+04, 4.552315E+04, 4.582390E+04, 4.612578E+04, 4.642878E+04, 
#   4.673292E+04, 4.703819E+04, 4.734459E+04, 4.765212E+04, 4.796079E+04, 4.827060E+04, 
#   4.858154E+04, 4.889363E+04, 4.920686E+04, 4.952123E+04, 4.983674E+04, 5.015340E+04, 
#   5.047121E+04, 5.079016E+04, 5.111027E+04, 5.143152E+04, 5.175393E+04, 5.207749E+04, 
#   5.240221E+04, 5.272808E+04, 5.305511E+04, 5.338330E+04, 5.371265E+04, 5.404317E+04, 
#   5.437484E+04, 5.470768E+04, 5.504169E+04, 5.537686E+04, 5.571320E+04, 5.605071E+04, 
#   5.638939E+04, 5.672924E+04, 5.707027E+04, 5.741247E+04, 5.775584E+04, 5.810040E+04, 
#   5.844613E+04, 5.879304E+04, 5.914113E+04, 5.949040E+04, 5.984085E+04, 6.019249E+04, 
#   6.054532E+04, 6.089933E+04, 6.125452E+04, 6.161091E+04, 6.196849E+04, 6.232725E+04, 
#   6.268721E+04, 6.304837E+04, 6.341071E+04, 6.377426E+04, 6.413899E+04, 6.450493E+04, 
#   6.487206E+04, 6.524040E+04, 6.560993E+04, 6.598067E+04, 6.635261E+04, 6.672575E+04, 
#   6.710010E+04, 6.747566E+04, 6.785242E+04, 6.823039E+04, 6.860957E+04, 6.898995E+04, 
#   6.937155E+04, 6.975436E+04, 7.013839E+04, 7.052362E+04, 7.091008E+04, 7.129775E+04, 
#   7.168663E+04, 7.207673E+04, 7.246805E+04, 7.286059E+04, 7.325435E+04, 7.364933E+04, 
#   7.404554E+04, 7.444296E+04, 7.484161E+04, 7.524149E+04, 7.564259E+04, 7.604492E+04, 
#   7.644847E+04, 7.685325E+04, 7.725926E+04, 7.766650E+04, 7.807497E+04, 7.848468E+04, 
#   7.889561E+04, 7.930778E+04, 7.972118E+04, 8.013582E+04, 8.055169E+04, 8.096879E+04, 
#   8.138714E+04, 8.180672E+04, 8.222754E+04, 8.264959E+04, 8.307289E+04, 8.349743E+04, 
#   8.392321E+04, 8.435023E+04, 8.477849E+04, 8.520800E+04, 8.563874E+04, 8.607074E+04, 
#   8.650398E+04, 8.693846E+04, 8.737419E+04, 8.781117E+04, 8.824940E+04, 8.868887E+04, 
#   8.912959E+04, 8.957156E+04, 9.001479E+04, 9.045926E+04, 9.090498E+04, 9.135196E+04, 
#   9.180018E+04, 9.224966E+04, 9.270040E+04, 9.315239E+04, 9.360563E+04, 9.406013E+04, 
#   9.451588E+04, 9.497289E+04, 9.543115E+04, 9.589068E+04, 9.635146E+04, 9.681349E+04, 
#   9.727679E+04, 9.774135E+04, 9.820716E+04, 9.867424E+04, 9.914257E+04, 9.961217E+04, 
#   1.000830E+05, 1.005551E+05, 1.010285E+05, 1.015032E+05, 1.019791E+05, 1.024562E+05, 
#   1.029347E+05, 1.034144E+05, 1.038953E+05, 1.043776E+05, 1.048611E+05, 1.053458E+05, 
#   1.058318E+05, 1.063191E+05, 1.068077E+05, 1.072975E+05, 1.077886E+05, 1.082809E+05, 
#   1.087746E+05, 1.092695E+05, 1.097656E+05, 1.102631E+05, 1.107617E+05, 1.112617E+05, 
#   1.117630E+05, 1.122655E+05, 1.127692E+05, 1.132743E+05, 1.137806E+05, 1.142882E+05, 
#   1.147971E+05, 1.153072E+05, 1.158186E+05, 1.163313E+05, 1.168452E+05, 1.173604E+05, 
#   1.178769E+05, 1.183947E+05, 1.189137E+05, 1.194340E+05, 1.199556E+05, 1.204784E+05, 
#   1.210026E+05, 1.215280E+05, 1.220546E+05, 1.225826E+05, 1.231118E+05, 1.236423E+05, 
#   1.241741E+05, 1.247071E+05, 1.252414E+05, 1.257770E+05, 1.263139E+05, 1.268520E+05, 
#   1.273914E+05, 1.279321E+05, 1.284741E+05, 1.290173E+05, 1.295618E+05, 1.301076E+05, 
#   1.306547E+05, 1.312030E+05, 1.317526E+05, 1.323035E+05, 1.328556E+05, 1.334091E+05, 
#   1.339638E+05, 1.345198E+05, 1.350770E+05, 1.356355E+05, 1.361953E+05, 1.367564E+05, 
#   1.373188E+05, 1.378824E+05, 1.384473E+05, 1.390135E+05, 1.395809E+05, 1.401497E+05, 
#   1.407197E+05, 1.412909E+05, 1.418635E+05, 1.424373E+05, 1.430124E+05, 1.435888E+05, 
#   1.441664E+05, 1.447453E+05, 1.453255E+05, 1.459070E+05, 1.464897E+05, 1.470737E+05, 
#   1.476590E+05, 1.482456E+05, 1.488334E+05, 1.494225E+05, 1.500129E+05, 1.506045E+05, 
#   1.511974E+05, 1.517916E+05, 1.523871E+05, 1.529838E+05, 1.535818E+05, 1.541810E+05, 
#   1.547816E+05, 1.553834E+05, 1.559865E+05, 1.565908E+05, 1.571964E+05, 1.578033E+05, 
#   1.584115E+05, 1.590209E+05, 1.596316E+05, 1.602436E+05, 1.608568E+05, 1.614713E+05, 
#   1.620871E+05, 1.627041E+05, 1.633224E+05, 1.639420E+05, 1.645628E+05, 1.651849E+05, 
#   1.658083E+05, 1.664329E+05, 1.670588E+05, 1.676859E+05, 1.683144E+05, 1.689440E+05, 
#   1.695750E+05, 1.702072E+05, 1.708407E+05, 1.714754E+05, 1.721114E+05, 1.727487E+05, 
#   1.733872E+05, 1.740270E+05, 1.746680E+05, 1.753103E+05, 1.759539E+05, 1.765987E+05, 
#   1.772448E+05, 1.778921E+05, 1.785407E+05, 1.791906E+05, 1.798417E+05, 1.804941E+05, 
#   1.811477E+05, 1.818026E+05, 1.824587E+05, 1.831161E+05, 1.837748E+05, 1.844347E+05, 
#   1.850958E+05, 1.857582E+05, 1.864219E+05, 1.870868E+05, 1.877530E+05, 1.884204E+05, 
#   1.890890E+05, 1.897590E+05, 1.904301E+05, 1.911025E+05, 1.917762E+05, 1.924511E+05, 
#   1.931273E+05, 1.938047E+05, 1.944833E+05, 1.951632E+05, 1.958443E+05, 1.965267E+05, 
#   1.972104E+05, 1.978952E+05, 1.985814E+05, 1.992687E+05, 1.999573E+05, 2.006472E+05, 
#   2.013382E+05, 2.020306E+05, 2.027241E+05, 2.034189E+05, 2.041150E+05, 2.048122E+05, 
#   2.055107E+05, 2.062105E+05, 2.069115E+05, 2.076137E+05, 2.083171E+05, 2.090218E+05, 
#   2.097278E+05, 2.104349E+05, 2.111433E+05, 2.118529E+05, 2.125638E+05, 2.132758E+05, 
#   2.139891E+05, 2.147037E+05, 2.154194E+05, 2.161364E+05, 2.168546E+05, 2.175741E+05, 
#   2.182947E+05, 2.190166E+05, 2.197397E+05, 2.204641E+05, 2.211896E+05, 2.219164E+05, 
#   2.226444E+05, 2.233736E+05, 2.241041E+05, 2.248357E+05, 2.255686E+05, 2.263027E+05, 
#   2.270380E+05, 2.277745E+05, 2.285122E+05, 2.292512E+05, 2.299914E+05, 2.307327E+05, 
#   2.314753E+05, 2.322191E+05, 2.329641E+05, 2.337103E+05, 2.344578E+05, 2.352064E+05, 
#   2.359562E+05, 2.367073E+05, 2.374595E+05, 2.382130E+05, 2.389676E+05, 2.397235E+05, 
#   2.404806E+05, 2.412388E+05, 2.419983E+05, 2.427590E+05, 2.435208E+05, 2.442839E+05, 
#   2.450481E+05, 2.458136E+05, 2.465802E+05, 2.473481E+05, 2.481171E+05, 2.488873E+05, 
#   2.496587E+05, 2.504313E+05, 2.512051E+05, 2.519801E+05, 2.527563E+05, 2.535337E+05, 
#   2.543122E+05, 2.550919E+05, 2.558729E+05, 2.566550E+05, 2.574382E+05, 2.582227E+05, 
#   2.590083E+05, 2.597952E+05, 2.605832E+05, 2.613723E+05, 2.621627E+05, 2.629542E+05, 
#   2.637469E+05, 2.645408E+05, 2.653359E+05, 2.661321E+05, 2.669295E+05, 2.677281E+05, 
#   2.685278E+05, 2.693287E+05, 2.701308E+05, 2.709340E+05, 2.717384E+05, 2.725440E+05, 
#   2.733507E+05, 2.741586E+05, 2.749677E+05, 2.757779E+05, 2.765893E+05, 2.774018E+05, 
#   2.782155E+05, 2.790303E+05, 2.798463E+05, 2.806635E+05, 2.814818E+05, 2.823013E+05, 
#   2.831219E+05, 2.839436E+05, 2.847665E+05, 2.855906E+05, 2.864158E+05, 2.872422E+05, 
#   2.880697E+05, 2.888983E+05, 2.897281E+05, 2.905590E+05, 2.913911E+05, 2.922243E+05, 
#   2.930586E+05, 2.938941E+05, 2.947307E+05, 2.955685E+05, 2.964074E+05, 2.972474E+05, 
#   2.980885E+05, 2.989308E+05, 2.997742E+05, 3.006188E+05, 3.014644E+05, 3.023112E+05, 
#   3.031591E+05, 3.040082E+05, 3.048584E+05, 3.057096E+05, 3.065620E+05, 3.074156E+05, 
#   3.082702E+05, 3.091260E+05, 3.099829E+05, 3.108409E+05, 3.117000E+05, 3.125602E+05, 
#   3.134215E+05, 3.142840E+05, 3.151475E+05, 3.160122E+05, 3.168780E+05, 3.177449E+05, 
#   3.186128E+05, 3.194819E+05, 3.203521E+05, 3.212234E+05, 3.220958E+05, 3.229693E+05, 
#   3.238439E+05, 3.247196E+05, 3.255963E+05, 3.264742E+05, 3.273532E+05, 3.282332E+05, 
#   3.291144E+05, 3.299966E+05, 3.308799E+05, 3.317644E+05, 3.326499E+05, 3.335364E+05, 
#   3.344241E+05, 3.353129E+05, 3.362027E+05, 3.370936E+05, 3.379856E+05, 3.388787E+05, 
#   3.397728E+05, 
#])
# ============================ M = 7, I = 6 ============================
#M = 7
#I = 6
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[8]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   7.197400E+01, 2.853555E+02, 5.586187E+02, 8.345484E+02, 1.111229E+03, 1.388232E+03, 
#   1.665410E+03, 1.942696E+03, 2.220057E+03, 2.497475E+03, 2.774938E+03, 3.052439E+03, 
#   3.329973E+03, 3.607536E+03, 3.885127E+03, 4.162745E+03, 4.440388E+03, 4.718059E+03, 
#   4.995759E+03, 5.273495E+03, 5.551272E+03, 5.829102E+03, 6.106999E+03, 6.384979E+03, 
#   6.663066E+03, 6.941285E+03, 7.219666E+03, 7.498244E+03, 7.777057E+03, 8.056147E+03, 
#   8.335560E+03, 8.615344E+03, 8.895550E+03, 9.176232E+03, 9.457446E+03, 9.739248E+03, 
#   1.002170E+04, 1.030485E+04, 1.058877E+04, 1.087351E+04, 1.115914E+04, 1.144570E+04, 
#   1.173327E+04, 1.202189E+04, 1.231163E+04, 1.260254E+04, 1.289467E+04, 1.318807E+04, 
#   1.348281E+04, 1.377892E+04, 1.407645E+04, 1.437546E+04, 1.467598E+04, 1.497807E+04, 
#   1.528176E+04, 1.558710E+04, 1.589412E+04, 1.620287E+04, 1.651337E+04, 1.682567E+04, 
#   1.713979E+04, 1.745577E+04, 1.777365E+04, 1.809345E+04, 1.841520E+04, 1.873892E+04, 
#   1.906465E+04, 1.939241E+04, 1.972223E+04, 2.005412E+04, 2.038811E+04, 2.072422E+04, 
#   2.106247E+04, 2.140289E+04, 2.174549E+04, 2.209029E+04, 2.243730E+04, 2.278655E+04, 
#   2.313805E+04, 2.349181E+04, 2.384786E+04, 2.420620E+04, 2.456684E+04, 2.492982E+04, 
#   2.529512E+04, 2.566278E+04, 2.603279E+04, 2.640517E+04, 2.677994E+04, 2.715710E+04, 
#   2.753666E+04, 2.791864E+04, 2.830303E+04, 2.868986E+04, 2.907913E+04, 2.947085E+04, 
#   2.986503E+04, 3.026168E+04, 3.066080E+04, 3.106240E+04, 3.146649E+04, 3.187307E+04, 
#   3.228216E+04, 3.269376E+04, 3.310788E+04, 3.352452E+04, 3.394370E+04, 3.436540E+04, 
#   3.478965E+04, 3.521645E+04, 3.564581E+04, 3.607772E+04, 3.651220E+04, 3.694925E+04, 
#   3.738887E+04, 3.783108E+04, 3.827587E+04, 3.872325E+04, 3.917323E+04, 3.962582E+04, 
#   4.008100E+04, 4.053880E+04, 4.099922E+04, 4.146225E+04, 4.192792E+04, 4.239621E+04, 
#   4.286713E+04, 4.334070E+04, 4.381690E+04, 4.429576E+04, 4.477727E+04, 4.526143E+04, 
#   4.574825E+04, 4.623774E+04, 4.672990E+04, 4.722474E+04, 4.772225E+04, 4.822244E+04, 
#   4.872532E+04, 4.923089E+04, 4.973916E+04, 5.025012E+04, 5.076379E+04, 5.128016E+04, 
#   5.179925E+04, 5.232105E+04, 5.284558E+04, 5.337283E+04, 5.390280E+04, 5.443551E+04, 
#   5.497096E+04, 5.550915E+04, 5.605008E+04, 5.659377E+04, 5.714021E+04, 5.768940E+04, 
#   5.824136E+04, 5.879609E+04, 5.935359E+04, 5.991386E+04, 6.047692E+04, 6.104276E+04, 
#   6.161139E+04, 6.218281E+04, 6.275703E+04, 6.333405E+04, 6.391387E+04, 6.449651E+04, 
#   6.508197E+04, 6.567024E+04, 6.626134E+04, 6.685526E+04, 6.745202E+04, 6.805162E+04, 
#   6.865406E+04, 6.925935E+04, 6.986749E+04, 7.047848E+04, 7.109234E+04, 7.170906E+04, 
#   7.232865E+04, 7.295112E+04, 7.357646E+04, 7.420469E+04, 7.483581E+04, 7.546982E+04, 
#   7.610672E+04, 7.674654E+04, 7.738925E+04, 7.803489E+04, 7.868343E+04, 7.933490E+04, 
#   7.998930E+04, 8.064663E+04, 8.130689E+04, 8.197009E+04, 8.263624E+04, 8.330535E+04, 
#   8.397740E+04, 8.465242E+04, 8.533040E+04, 8.601135E+04, 8.669528E+04, 8.738218E+04, 
#   8.807207E+04, 8.876495E+04, 8.946082E+04, 9.015970E+04, 9.086157E+04, 9.156646E+04, 
#   9.227436E+04, 9.298527E+04, 9.369922E+04, 9.441619E+04, 9.513619E+04, 9.585923E+04, 
#   9.658532E+04, 9.731445E+04, 9.804664E+04, 9.878188E+04, 9.952019E+04, 1.002616E+05, 
#   1.010060E+05, 1.017535E+05, 1.025041E+05, 1.032578E+05, 1.040146E+05, 1.047745E+05, 
#   1.055375E+05, 1.063036E+05, 1.070728E+05, 1.078451E+05, 1.086206E+05, 1.093991E+05, 
#   1.101808E+05, 1.109657E+05, 1.117536E+05, 1.125448E+05, 1.133390E+05, 1.141365E+05, 
#   1.149371E+05, 1.157408E+05, 1.165477E+05, 1.173578E+05, 1.181711E+05, 1.189876E+05, 
#   1.198072E+05, 1.206301E+05, 1.214561E+05, 1.222854E+05, 1.231178E+05, 1.239535E+05, 
#   1.247924E+05, 1.256345E+05, 1.264798E+05, 1.273284E+05, 1.281802E+05, 1.290352E+05, 
#   1.298935E+05, 1.307550E+05, 1.316198E+05, 1.324879E+05, 1.333592E+05, 1.342338E+05, 
#   1.351116E+05, 1.359927E+05, 1.368771E+05, 1.377648E+05, 1.386558E+05, 1.395501E+05, 
#   1.404477E+05, 1.413486E+05, 1.422528E+05, 1.431603E+05, 1.440711E+05, 1.449852E+05, 
#   1.459027E+05, 1.468235E+05, 1.477476E+05, 1.486751E+05, 1.496059E+05, 1.505400E+05, 
#   1.514775E+05, 1.524184E+05, 1.533626E+05, 1.543102E+05, 1.552611E+05, 1.562155E+05, 
#   1.571732E+05, 1.581342E+05, 1.590987E+05, 1.600665E+05, 1.610378E+05, 1.620124E+05, 
#   1.629905E+05, 1.639719E+05, 1.649567E+05, 1.659450E+05, 1.669367E+05, 1.679318E+05, 
#   1.689303E+05, 1.699322E+05, 1.709376E+05, 1.719464E+05, 1.729586E+05, 1.739743E+05, 
#   1.749935E+05, 1.760160E+05, 1.770421E+05, 1.780716E+05, 1.791045E+05, 1.801409E+05, 
#   1.811808E+05, 1.822242E+05, 1.832710E+05, 1.843213E+05, 1.853751E+05, 1.864324E+05, 
#   1.874931E+05, 1.885574E+05, 1.896251E+05, 1.906964E+05, 1.917711E+05, 1.928494E+05, 
#   1.939311E+05, 1.950164E+05, 1.961052E+05, 1.971975E+05, 1.982934E+05, 1.993927E+05, 
#   2.004956E+05, 2.016020E+05, 2.027120E+05, 2.038255E+05, 2.049425E+05, 2.060631E+05, 
#   2.071872E+05, 2.083149E+05, 2.094461E+05, 2.105809E+05, 2.117192E+05, 2.128611E+05, 
#   2.140066E+05, 2.151556E+05, 2.163082E+05, 2.174644E+05, 2.186241E+05, 2.197874E+05, 
#   2.209544E+05, 2.221249E+05, 2.232989E+05, 2.244766E+05, 2.256579E+05, 2.268427E+05, 
#   2.280312E+05, 2.292232E+05, 2.304189E+05, 2.316182E+05, 2.328211E+05, 2.340275E+05, 
#   2.352376E+05, 2.364514E+05, 2.376687E+05, 2.388896E+05, 2.401142E+05, 2.413424E+05, 
#   2.425742E+05, 2.438097E+05, 2.450488E+05, 2.462915E+05, 2.475379E+05, 2.487879E+05, 
#   2.500415E+05, 2.512988E+05, 2.525597E+05, 2.538243E+05, 2.550925E+05, 2.563644E+05, 
#   2.576400E+05, 2.589192E+05, 2.602020E+05, 2.614885E+05, 2.627787E+05, 2.640725E+05, 
#   2.653700E+05, 2.666712E+05, 2.679761E+05, 2.692846E+05, 2.705968E+05, 2.719126E+05, 
#   2.732322E+05, 2.745554E+05, 2.758823E+05, 2.772129E+05, 2.785471E+05, 2.798851E+05, 
#   2.812268E+05, 2.825721E+05, 2.839211E+05, 2.852738E+05, 2.866302E+05, 2.879904E+05, 
#   2.893542E+05, 2.907217E+05, 2.920929E+05, 2.934678E+05, 2.948464E+05, 2.962287E+05, 
#   2.976148E+05, 2.990045E+05, 3.003980E+05, 3.017951E+05, 3.031960E+05, 3.046006E+05, 
#   3.060089E+05, 3.074209E+05, 3.088366E+05, 3.102561E+05, 3.116792E+05, 3.131061E+05, 
#   3.145368E+05, 3.159711E+05, 3.174092E+05, 3.188509E+05, 3.202965E+05, 3.217457E+05, 
#   3.231987E+05, 3.246554E+05, 3.261158E+05, 3.275800E+05, 3.290479E+05, 3.305196E+05, 
#   3.319949E+05, 3.334740E+05, 3.349569E+05, 3.364435E+05, 3.379338E+05, 3.394279E+05, 
#   3.409257E+05, 3.424272E+05, 3.439325E+05, 3.454415E+05, 3.469543E+05, 3.484708E+05, 
#   3.499911E+05, 3.515151E+05, 3.530429E+05, 3.545744E+05, 3.561096E+05, 3.576486E+05, 
#   3.591914E+05, 3.607379E+05, 3.622881E+05, 3.638421E+05, 3.653999E+05, 3.669614E+05, 
#   3.685267E+05, 3.700957E+05, 3.716684E+05, 3.732450E+05, 3.748252E+05, 3.764093E+05, 
#   3.779970E+05, 3.795886E+05, 3.811839E+05, 3.827829E+05, 3.843857E+05, 3.859923E+05, 
#   3.876026E+05, 3.892167E+05, 3.908345E+05, 3.924561E+05, 3.940814E+05, 3.957105E+05, 
#   3.973434E+05, 3.989800E+05, 4.006204E+05, 4.022645E+05, 4.039124E+05, 4.055641E+05, 
#   4.072195E+05, 4.088786E+05, 4.105416E+05, 4.122082E+05, 4.138787E+05, 4.155529E+05, 
#   4.172308E+05, 4.189126E+05, 4.205980E+05, 4.222873E+05, 4.239803E+05, 4.256770E+05, 
#   4.273775E+05, 4.290818E+05, 4.307898E+05, 4.325016E+05, 4.342171E+05, 4.359364E+05, 
#   4.376594E+05, 4.393862E+05, 4.411168E+05, 4.428511E+05, 4.445892E+05, 4.463310E+05, 
#   4.480765E+05, 4.498259E+05, 4.515790E+05, 4.533358E+05, 4.550964E+05, 4.568607E+05, 
#   4.586288E+05, 4.604006E+05, 4.621762E+05, 4.639555E+05, 4.657386E+05, 4.675255E+05, 
#   4.693160E+05, 4.711104E+05, 4.729085E+05, 4.747103E+05, 4.765158E+05, 4.783252E+05, 
#   4.801382E+05, 4.819550E+05, 4.837756E+05, 4.855998E+05, 4.874279E+05, 4.892596E+05, 
#   4.910951E+05, 4.929344E+05, 4.947774E+05, 4.966241E+05, 4.984745E+05, 5.003287E+05, 
#   5.021866E+05, 5.040483E+05, 5.059137E+05, 5.077828E+05, 5.096557E+05, 5.115323E+05, 
#   5.134126E+05, 5.152966E+05, 5.171844E+05, 5.190759E+05, 5.209711E+05, 5.228700E+05, 
#   5.247727E+05, 5.266791E+05, 5.285892E+05, 5.305030E+05, 5.324206E+05, 5.343418E+05, 
#   5.362668E+05, 5.381955E+05, 5.401279E+05, 5.420640E+05, 5.440038E+05, 5.459474E+05, 
#   5.478946E+05, 5.498456E+05, 5.518002E+05, 5.537586E+05, 5.557207E+05, 5.576864E+05, 
#   5.596559E+05, 5.616291E+05, 5.636060E+05, 5.655865E+05, 5.675708E+05, 5.695587E+05, 
#   5.715504E+05, 5.735457E+05, 5.755448E+05, 5.775475E+05, 5.795539E+05, 5.815640E+05, 
#   5.835777E+05, 5.855952E+05, 5.876163E+05, 5.896411E+05, 5.916696E+05, 5.937018E+05, 
#   5.957376E+05, 5.977772E+05, 5.998203E+05, 6.018672E+05, 6.039177E+05, 6.059719E+05, 
#   6.080298E+05, 6.100913E+05, 6.121564E+05, 6.142253E+05, 6.162978E+05, 6.183739E+05, 
#   6.204537E+05, 6.225372E+05, 6.246243E+05, 6.267150E+05, 6.288094E+05, 6.309075E+05, 
#   6.330091E+05, 6.351145E+05, 6.372234E+05, 6.393360E+05, 6.414523E+05, 6.435722E+05, 
#   6.456957E+05, 6.478228E+05, 6.499535E+05, 6.520879E+05, 6.542259E+05, 6.563676E+05, 
#   6.585128E+05, 6.606617E+05, 6.628142E+05, 6.649703E+05, 6.671300E+05, 6.692933E+05, 
#   6.714602E+05, 6.736307E+05, 6.758049E+05, 6.779826E+05, 6.801639E+05, 6.823489E+05, 
#   6.845374E+05, 6.867295E+05, 6.889252E+05, 6.911245E+05, 6.933274E+05, 6.955338E+05, 
#   6.977439E+05, 6.999575E+05, 7.021747E+05, 7.043955E+05, 7.066198E+05, 7.088477E+05, 
#   7.110792E+05, 7.133142E+05, 7.155528E+05, 7.177950E+05, 7.200407E+05, 7.222900E+05, 
#   7.245429E+05, 7.267992E+05, 7.290592E+05, 7.313227E+05, 7.335897E+05, 7.358602E+05, 
#   7.381343E+05, 7.404120E+05, 7.426932E+05, 7.449779E+05, 7.472661E+05, 7.495578E+05, 
#   7.518531E+05, 7.541519E+05, 7.564543E+05, 7.587601E+05, 7.610694E+05, 7.633823E+05, 
#   7.656987E+05, 7.680185E+05, 7.703419E+05, 7.726688E+05, 7.749992E+05, 7.773330E+05, 
#   7.796704E+05, 7.820113E+05, 7.843556E+05, 7.867034E+05, 7.890547E+05, 7.914095E+05, 
#   7.937678E+05, 7.961295E+05, 7.984947E+05, 8.008634E+05, 8.032355E+05, 8.056111E+05, 
#   8.079901E+05, 8.103727E+05, 8.127586E+05, 8.151480E+05, 8.175409E+05, 8.199372E+05, 
#   8.223370E+05, 8.247402E+05, 8.271468E+05, 8.295569E+05, 8.319703E+05, 8.343873E+05, 
#   8.368076E+05, 8.392314E+05, 8.416586E+05, 8.440892E+05, 8.465232E+05, 8.489606E+05, 
#   8.514015E+05, 8.538457E+05, 8.562933E+05, 8.587444E+05, 8.611988E+05, 8.636566E+05, 
#   8.661178E+05, 8.685824E+05, 8.710504E+05, 8.735218E+05, 8.759965E+05, 8.784746E+05, 
#   8.809561E+05, 8.834410E+05, 8.859292E+05, 8.884208E+05, 8.909157E+05, 8.934140E+05, 
#   8.959156E+05, 8.984206E+05, 9.009290E+05, 9.034406E+05, 9.059557E+05, 9.084740E+05, 
#   9.109957E+05, 9.135207E+05, 9.160491E+05, 9.185807E+05, 9.211157E+05, 9.236540E+05, 
#   9.261956E+05, 9.287405E+05, 9.312888E+05, 9.338403E+05, 9.363952E+05, 9.389533E+05, 
#   9.415147E+05, 9.440794E+05, 9.466474E+05, 9.492187E+05, 9.517933E+05, 9.543711E+05, 
#   9.569522E+05, 9.595366E+05, 9.621243E+05, 9.647152E+05, 9.673094E+05, 9.699068E+05, 
#   9.725075E+05, 9.751114E+05, 9.777186E+05, 9.803290E+05, 9.829427E+05, 9.855596E+05, 
#   9.881797E+05, 
#])
# ============================ M = 8, I = 1 ============================
M = 8
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.191251E+01, 3.039419E+01, 5.510160E+01, 8.016892E+01, 1.060748E+02, 1.334249E+02, 
   1.625521E+02, 1.935309E+02, 2.262845E+02, 2.606669E+02, 2.965100E+02, 3.336478E+02, 
   3.719268E+02, 4.112102E+02, 4.513776E+02, 4.923247E+02, 5.339612E+02, 5.762089E+02, 
   6.190004E+02, 6.622774E+02, 7.059892E+02, 7.500918E+02, 7.945471E+02, 8.393216E+02, 
   8.843864E+02, 9.297160E+02, 9.752885E+02, 1.021085E+03, 1.067088E+03, 1.113283E+03, 
   1.159659E+03, 1.206206E+03, 1.252913E+03, 1.299774E+03, 1.346783E+03, 1.393935E+03, 
   1.441227E+03, 1.488656E+03, 1.536220E+03, 1.583919E+03, 1.631753E+03, 1.679722E+03, 
   1.727827E+03, 1.776070E+03, 1.824453E+03, 1.872979E+03, 1.921651E+03, 1.970471E+03, 
   2.019443E+03, 2.068572E+03, 2.117860E+03, 2.167312E+03, 2.216932E+03, 2.266724E+03, 
   2.316693E+03, 2.366843E+03, 2.417177E+03, 2.467702E+03, 2.518421E+03, 2.569338E+03, 
   2.620458E+03, 2.671785E+03, 2.723323E+03, 2.775077E+03, 2.827051E+03, 2.879248E+03, 
   2.931673E+03, 2.984330E+03, 3.037222E+03, 3.090354E+03, 3.143729E+03, 3.197350E+03, 
   3.251221E+03, 3.305345E+03, 3.359727E+03, 3.414368E+03, 3.469272E+03, 3.524443E+03, 
   3.579882E+03, 3.635594E+03, 3.691580E+03, 3.747844E+03, 3.804389E+03, 3.861216E+03, 
   3.918328E+03, 3.975728E+03, 4.033418E+03, 4.091400E+03, 4.149677E+03, 4.208250E+03, 
   4.267123E+03, 4.326295E+03, 4.385771E+03, 4.445551E+03, 4.505637E+03, 4.566032E+03, 
   4.626736E+03, 4.687752E+03, 4.749081E+03, 4.810724E+03, 4.872684E+03, 4.934962E+03, 
   4.997558E+03, 5.060475E+03, 5.123714E+03, 5.187276E+03, 5.251162E+03, 5.315374E+03, 
   5.379913E+03, 5.444779E+03, 5.509975E+03, 5.575500E+03, 5.641357E+03, 5.707545E+03, 
   5.774067E+03, 5.840923E+03, 5.908114E+03, 5.975641E+03, 6.043504E+03, 6.111706E+03, 
   6.180245E+03, 6.249124E+03, 6.318343E+03, 6.387902E+03, 6.457803E+03, 6.528046E+03, 
   6.598632E+03, 6.669562E+03, 6.740836E+03, 6.812455E+03, 6.884419E+03, 6.956729E+03, 
   7.029386E+03, 7.102390E+03, 7.175742E+03, 7.249442E+03, 7.323491E+03, 7.397889E+03, 
   7.472637E+03, 7.547736E+03, 7.623185E+03, 7.698986E+03, 7.775138E+03, 7.851642E+03, 
   7.928499E+03, 8.005709E+03, 8.083272E+03, 8.161189E+03, 8.239460E+03, 8.318086E+03, 
   8.397066E+03, 8.476402E+03, 8.556093E+03, 8.636141E+03, 8.716544E+03, 8.797304E+03, 
   8.878421E+03, 8.959895E+03, 9.041727E+03, 9.123916E+03, 9.206464E+03, 9.289370E+03, 
   9.372635E+03, 9.456258E+03, 9.540241E+03, 9.624583E+03, 9.709284E+03, 9.794346E+03, 
   9.879768E+03, 9.965550E+03, 1.005169E+04, 1.013820E+04, 1.022506E+04, 1.031229E+04, 
   1.039988E+04, 1.048783E+04, 1.057614E+04, 1.066481E+04, 1.075384E+04, 1.084324E+04, 
   1.093300E+04, 1.102313E+04, 1.111361E+04, 1.120447E+04, 1.129568E+04, 1.138725E+04, 
   1.147920E+04, 1.157150E+04, 1.166417E+04, 1.175720E+04, 1.185060E+04, 1.194436E+04, 
   1.203849E+04, 1.213298E+04, 1.222784E+04, 1.232306E+04, 1.241864E+04, 1.251460E+04, 
   1.261091E+04, 1.270760E+04, 1.280465E+04, 1.290206E+04, 1.299984E+04, 1.309799E+04, 
   1.319650E+04, 1.329538E+04, 1.339463E+04, 1.349424E+04, 1.359422E+04, 1.369457E+04, 
   1.379528E+04, 1.389637E+04, 1.399781E+04, 1.409963E+04, 1.420181E+04, 1.430436E+04, 
   1.440728E+04, 1.451057E+04, 1.461422E+04, 1.471824E+04, 1.482263E+04, 1.492739E+04, 
   1.503252E+04, 1.513801E+04, 1.524387E+04, 1.535010E+04, 1.545670E+04, 1.556367E+04, 
   1.567101E+04, 1.577872E+04, 1.588679E+04, 1.599524E+04, 1.610405E+04, 1.621323E+04, 
   1.632278E+04, 1.643271E+04, 1.654300E+04, 1.665366E+04, 1.676469E+04, 1.687608E+04, 
   1.698785E+04, 1.709999E+04, 1.721250E+04, 1.732538E+04, 1.743863E+04, 1.755225E+04, 
   1.766624E+04, 1.778060E+04, 1.789533E+04, 1.801043E+04, 1.812590E+04, 1.824174E+04, 
   1.835795E+04, 1.847453E+04, 1.859149E+04, 1.870881E+04, 1.882651E+04, 1.894457E+04, 
   1.906301E+04, 1.918182E+04, 1.930099E+04, 1.942054E+04, 1.954047E+04, 1.966076E+04, 
   1.978142E+04, 1.990246E+04, 2.002387E+04, 2.014565E+04, 2.026779E+04, 2.039032E+04, 
   2.051321E+04, 2.063648E+04, 2.076011E+04, 2.088412E+04, 2.100850E+04, 2.113326E+04, 
   2.125838E+04, 2.138388E+04, 2.150975E+04, 2.163599E+04, 2.176260E+04, 2.188959E+04, 
   2.201695E+04, 2.214468E+04, 2.227279E+04, 2.240126E+04, 2.253011E+04, 2.265933E+04, 
   2.278893E+04, 2.291890E+04, 2.304924E+04, 2.317995E+04, 2.331103E+04, 2.344249E+04, 
   2.357433E+04, 2.370653E+04, 2.383911E+04, 2.397206E+04, 2.410538E+04, 2.423908E+04, 
   2.437315E+04, 2.450760E+04, 2.464242E+04, 2.477761E+04, 2.491317E+04, 2.504911E+04, 
   2.518542E+04, 2.532211E+04, 2.545917E+04, 2.559660E+04, 2.573441E+04, 2.587259E+04, 
   2.601114E+04, 2.615007E+04, 2.628938E+04, 2.642905E+04, 2.656910E+04, 2.670953E+04, 
   2.685033E+04, 2.699150E+04, 2.713305E+04, 2.727497E+04, 2.741727E+04, 2.755994E+04, 
   2.770298E+04, 2.784640E+04, 2.799019E+04, 2.813436E+04, 2.827891E+04, 2.842383E+04, 
   2.856912E+04, 2.871478E+04, 2.886083E+04, 2.900724E+04, 2.915404E+04, 2.930120E+04, 
   2.944874E+04, 2.959666E+04, 2.974495E+04, 2.989362E+04, 3.004266E+04, 3.019208E+04, 
   3.034187E+04, 3.049204E+04, 3.064258E+04, 3.079350E+04, 3.094479E+04, 3.109646E+04, 
   3.124851E+04, 3.140093E+04, 3.155372E+04, 3.170690E+04, 3.186044E+04, 3.201436E+04, 
   3.216866E+04, 3.232334E+04, 3.247839E+04, 3.263381E+04, 3.278961E+04, 3.294579E+04, 
   3.310234E+04, 3.325927E+04, 3.341658E+04, 3.357426E+04, 3.373232E+04, 3.389075E+04, 
   3.404956E+04, 3.420875E+04, 3.436831E+04, 3.452825E+04, 3.468856E+04, 3.484925E+04, 
   3.501032E+04, 3.517177E+04, 3.533359E+04, 3.549578E+04, 3.565836E+04, 3.582131E+04, 
   3.598464E+04, 3.614834E+04, 3.631242E+04, 3.647688E+04, 3.664171E+04, 3.680692E+04, 
   3.697251E+04, 3.713848E+04, 3.730482E+04, 3.747154E+04, 3.763863E+04, 3.780611E+04, 
   3.797396E+04, 3.814218E+04, 3.831079E+04, 3.847977E+04, 3.864913E+04, 3.881886E+04, 
   3.898898E+04, 3.915947E+04, 3.933034E+04, 3.950158E+04, 3.967321E+04, 3.984521E+04, 
   4.001759E+04, 4.019034E+04, 4.036348E+04, 4.053699E+04, 4.071088E+04, 4.088515E+04, 
   4.105979E+04, 4.123481E+04, 4.141021E+04, 4.158599E+04, 4.176215E+04, 4.193868E+04, 
   4.211560E+04, 4.229289E+04, 4.247056E+04, 4.264860E+04, 4.282703E+04, 4.300583E+04, 
   4.318501E+04, 4.336457E+04, 4.354451E+04, 4.372483E+04, 4.390552E+04, 4.408660E+04, 
   4.426805E+04, 4.444988E+04, 4.463209E+04, 4.481468E+04, 4.499764E+04, 4.518099E+04, 
   4.536471E+04, 4.554881E+04, 4.573329E+04, 4.591815E+04, 4.610339E+04, 4.628901E+04, 
   4.647501E+04, 4.666138E+04, 4.684814E+04, 4.703527E+04, 4.722278E+04, 4.741068E+04, 
   4.759895E+04, 4.778760E+04, 4.797663E+04, 4.816604E+04, 4.835582E+04, 4.854599E+04, 
   4.873654E+04, 4.892746E+04, 4.911877E+04, 4.931045E+04, 4.950252E+04, 4.969496E+04, 
   4.988779E+04, 5.008099E+04, 5.027457E+04, 5.046853E+04, 5.066288E+04, 5.085760E+04, 
   5.105270E+04, 5.124818E+04, 5.144404E+04, 5.164028E+04, 5.183691E+04, 5.203391E+04, 
   5.223129E+04, 5.242905E+04, 5.262719E+04, 5.282571E+04, 5.302462E+04, 5.322390E+04, 
   5.342356E+04, 5.362360E+04, 5.382403E+04, 5.402483E+04, 5.422601E+04, 5.442758E+04, 
   5.462952E+04, 5.483185E+04, 5.503455E+04, 5.523764E+04, 5.544110E+04, 5.564495E+04, 
   5.584918E+04, 5.605379E+04, 5.625878E+04, 5.646415E+04, 5.666990E+04, 5.687603E+04, 
   5.708254E+04, 5.728944E+04, 5.749671E+04, 5.770437E+04, 5.791240E+04, 5.812082E+04, 
   5.832962E+04, 5.853880E+04, 5.874836E+04, 
])
# ============================ M = 8, I = 2 ============================
M = 8
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.968254E+00, 2.085297E+01, 3.792990E+01, 5.524809E+01, 7.314363E+01, 9.203618E+01, 
   1.121564E+02, 1.335562E+02, 1.561829E+02, 1.799356E+02, 2.046983E+02, 2.303563E+02, 
   2.568035E+02, 2.839452E+02, 3.116984E+02, 3.399909E+02, 3.687601E+02, 3.979521E+02, 
   4.275203E+02, 4.574244E+02, 4.876292E+02, 5.181045E+02, 5.488239E+02, 5.797642E+02, 
   6.109056E+02, 6.422306E+02, 6.737239E+02, 7.053725E+02, 7.371650E+02, 7.690917E+02, 
   8.011441E+02, 8.333154E+02, 8.655995E+02, 8.979917E+02, 9.304882E+02, 9.630858E+02, 
   9.957823E+02, 1.028576E+03, 1.061467E+03, 1.094454E+03, 1.127536E+03, 1.160716E+03, 
   1.193994E+03, 1.227372E+03, 1.260850E+03, 1.294431E+03, 1.328118E+03, 1.361912E+03, 
   1.395816E+03, 1.429833E+03, 1.463965E+03, 1.498216E+03, 1.532589E+03, 1.567086E+03, 
   1.601711E+03, 1.636466E+03, 1.671355E+03, 1.706382E+03, 1.741548E+03, 1.776858E+03, 
   1.812314E+03, 1.847919E+03, 1.883676E+03, 1.919589E+03, 1.955659E+03, 1.991891E+03, 
   2.028286E+03, 2.064847E+03, 2.101578E+03, 2.138480E+03, 2.175556E+03, 2.212809E+03, 
   2.250241E+03, 2.287854E+03, 2.325651E+03, 2.363634E+03, 2.401805E+03, 2.440166E+03, 
   2.478720E+03, 2.517467E+03, 2.556411E+03, 2.595552E+03, 2.634894E+03, 2.674437E+03, 
   2.714183E+03, 2.754134E+03, 2.794291E+03, 2.834657E+03, 2.875231E+03, 2.916017E+03, 
   2.957016E+03, 2.998227E+03, 3.039655E+03, 3.081298E+03, 3.123159E+03, 3.165238E+03, 
   3.207538E+03, 3.250058E+03, 3.292801E+03, 3.335767E+03, 3.378957E+03, 3.422372E+03, 
   3.466014E+03, 3.509882E+03, 3.553979E+03, 3.598304E+03, 3.642859E+03, 3.687645E+03, 
   3.732662E+03, 3.777911E+03, 3.823393E+03, 3.869108E+03, 3.915058E+03, 3.961243E+03, 
   4.007663E+03, 4.054320E+03, 4.101213E+03, 4.148344E+03, 4.195713E+03, 4.243321E+03, 
   4.291167E+03, 4.339254E+03, 4.387581E+03, 4.436148E+03, 4.484957E+03, 4.534007E+03, 
   4.583300E+03, 4.632835E+03, 4.682614E+03, 4.732636E+03, 4.782901E+03, 4.833411E+03, 
   4.884166E+03, 4.935166E+03, 4.986411E+03, 5.037902E+03, 5.089640E+03, 5.141623E+03, 
   5.193854E+03, 5.246332E+03, 5.299057E+03, 5.352030E+03, 5.405251E+03, 5.458721E+03, 
   5.512439E+03, 5.566406E+03, 5.620622E+03, 5.675088E+03, 5.729803E+03, 5.784769E+03, 
   5.839984E+03, 5.895450E+03, 5.951167E+03, 6.007135E+03, 6.063353E+03, 6.119824E+03, 
   6.176545E+03, 6.233519E+03, 6.290744E+03, 6.348221E+03, 6.405951E+03, 6.463934E+03, 
   6.522169E+03, 6.580657E+03, 6.639398E+03, 6.698392E+03, 6.757639E+03, 6.817141E+03, 
   6.876895E+03, 6.936904E+03, 6.997167E+03, 7.057684E+03, 7.118455E+03, 7.179481E+03, 
   7.240761E+03, 7.302296E+03, 7.364086E+03, 7.426130E+03, 7.488430E+03, 7.550985E+03, 
   7.613796E+03, 7.676861E+03, 7.740183E+03, 7.803760E+03, 7.867593E+03, 7.931682E+03, 
   7.996026E+03, 8.060627E+03, 8.125484E+03, 8.190598E+03, 8.255968E+03, 8.321594E+03, 
   8.387477E+03, 8.453616E+03, 8.520013E+03, 8.586666E+03, 8.653576E+03, 8.720743E+03, 
   8.788167E+03, 8.855848E+03, 8.923786E+03, 8.991982E+03, 9.060435E+03, 9.129145E+03, 
   9.198113E+03, 9.267339E+03, 9.336822E+03, 9.406562E+03, 9.476561E+03, 9.546817E+03, 
   9.617331E+03, 9.688103E+03, 9.759133E+03, 9.830421E+03, 9.901967E+03, 9.973771E+03, 
   1.004583E+04, 1.011815E+04, 1.019073E+04, 1.026357E+04, 1.033666E+04, 1.041002E+04, 
   1.048363E+04, 1.055750E+04, 1.063163E+04, 1.070601E+04, 1.078066E+04, 1.085557E+04, 
   1.093073E+04, 1.100615E+04, 1.108183E+04, 1.115777E+04, 1.123396E+04, 1.131042E+04, 
   1.138713E+04, 1.146411E+04, 1.154134E+04, 1.161883E+04, 1.169658E+04, 1.177458E+04, 
   1.185285E+04, 1.193138E+04, 1.201016E+04, 1.208920E+04, 1.216850E+04, 1.224806E+04, 
   1.232788E+04, 1.240796E+04, 1.248829E+04, 1.256889E+04, 1.264974E+04, 1.273085E+04, 
   1.281222E+04, 1.289385E+04, 1.297574E+04, 1.305789E+04, 1.314029E+04, 1.322295E+04, 
   1.330588E+04, 1.338906E+04, 1.347249E+04, 1.355619E+04, 1.364015E+04, 1.372436E+04, 
   1.380883E+04, 1.389356E+04, 1.397855E+04, 1.406380E+04, 1.414931E+04, 1.423507E+04, 
   1.432109E+04, 1.440737E+04, 1.449391E+04, 1.458070E+04, 1.466776E+04, 1.475507E+04, 
   1.484264E+04, 1.493047E+04, 1.501855E+04, 1.510690E+04, 1.519550E+04, 1.528435E+04, 
   1.537347E+04, 1.546284E+04, 1.555247E+04, 1.564236E+04, 1.573251E+04, 1.582291E+04, 
   1.591357E+04, 1.600448E+04, 1.609566E+04, 1.618709E+04, 1.627877E+04, 1.637072E+04, 
   1.646292E+04, 1.655538E+04, 1.664809E+04, 1.674106E+04, 1.683429E+04, 1.692777E+04, 
   1.702151E+04, 1.711550E+04, 1.720975E+04, 1.730426E+04, 1.739902E+04, 1.749404E+04, 
   1.758932E+04, 1.768485E+04, 1.778063E+04, 1.787667E+04, 1.797297E+04, 1.806952E+04, 
   1.816632E+04, 1.826338E+04, 1.836070E+04, 1.845827E+04, 1.855609E+04, 1.865417E+04, 
   1.875251E+04, 1.885109E+04, 1.894994E+04, 1.904903E+04, 1.914838E+04, 1.924798E+04, 
   1.934784E+04, 1.944795E+04, 1.954831E+04, 1.964893E+04, 1.974980E+04, 1.985093E+04, 
   1.995230E+04, 2.005393E+04, 2.015581E+04, 2.025795E+04, 2.036033E+04, 2.046297E+04, 
   2.056586E+04, 2.066900E+04, 2.077240E+04, 2.087604E+04, 2.097994E+04, 2.108409E+04, 
   2.118849E+04, 2.129314E+04, 2.139804E+04, 2.150319E+04, 2.160860E+04, 2.171425E+04, 
   2.182015E+04, 2.192631E+04, 2.203271E+04, 2.213936E+04, 2.224627E+04, 2.235342E+04, 
   2.246082E+04, 2.256847E+04, 2.267637E+04, 2.278452E+04, 2.289292E+04, 2.300156E+04, 
   2.311046E+04, 2.321960E+04, 2.332899E+04, 2.343862E+04, 2.354851E+04, 2.365864E+04, 
   2.376902E+04, 2.387965E+04, 2.399052E+04, 2.410165E+04, 2.421301E+04, 2.432462E+04, 
   2.443649E+04, 2.454859E+04, 2.466094E+04, 2.477354E+04, 2.488638E+04, 2.499947E+04, 
   2.511280E+04, 2.522638E+04, 2.534020E+04, 2.545426E+04, 2.556858E+04, 2.568313E+04, 
   2.579793E+04, 2.591297E+04, 2.602826E+04, 2.614378E+04, 2.625956E+04, 2.637557E+04, 
   2.649183E+04, 2.660833E+04, 2.672507E+04, 2.684205E+04, 2.695928E+04, 2.707674E+04, 
   2.719445E+04, 2.731240E+04, 2.743059E+04, 2.754902E+04, 2.766769E+04, 2.778660E+04, 
   2.790575E+04, 2.802515E+04, 2.814478E+04, 2.826465E+04, 2.838476E+04, 2.850511E+04, 
   2.862569E+04, 2.874652E+04, 2.886758E+04, 2.898888E+04, 2.911042E+04, 2.923220E+04, 
   2.935422E+04, 2.947647E+04, 2.959896E+04, 2.972169E+04, 2.984465E+04, 2.996785E+04, 
   3.009128E+04, 3.021495E+04, 3.033886E+04, 3.046300E+04, 3.058737E+04, 3.071199E+04, 
   3.083683E+04, 3.096191E+04, 3.108722E+04, 3.121277E+04, 3.133855E+04, 3.146457E+04, 
   3.159082E+04, 3.171730E+04, 3.184401E+04, 3.197096E+04, 3.209814E+04, 3.222555E+04, 
   3.235319E+04, 3.248106E+04, 3.260917E+04, 3.273750E+04, 3.286607E+04, 3.299486E+04, 
   3.312389E+04, 3.325315E+04, 3.338263E+04, 3.351235E+04, 3.364229E+04, 3.377247E+04, 
   3.390287E+04, 3.403350E+04, 3.416436E+04, 3.429545E+04, 3.442676E+04, 3.455830E+04, 
   3.469007E+04, 3.482207E+04, 3.495429E+04, 3.508674E+04, 3.521942E+04, 3.535232E+04, 
   3.548545E+04, 3.561880E+04, 3.575238E+04, 3.588618E+04, 3.602020E+04, 3.615446E+04, 
   3.628893E+04, 3.642363E+04, 3.655855E+04, 3.669370E+04, 3.682907E+04, 3.696466E+04, 
   3.710047E+04, 3.723650E+04, 3.737276E+04, 3.750924E+04, 3.764594E+04, 3.778286E+04, 
   3.792001E+04, 3.805737E+04, 3.819495E+04, 3.833276E+04, 3.847078E+04, 3.860902E+04, 
   3.874749E+04, 3.888617E+04, 3.902507E+04, 3.916419E+04, 3.930352E+04, 3.944308E+04, 
   3.958285E+04, 3.972284E+04, 3.986304E+04, 4.000347E+04, 4.014411E+04, 4.028496E+04, 
   4.042603E+04, 4.056732E+04, 4.070882E+04, 
])
# ============================ M = 8, I = 3 ============================
M = 8
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.189636E+01, 3.127447E+01, 5.689275E+01, 8.287098E+01, 1.097148E+02, 1.380539E+02, 
   1.682344E+02, 2.003343E+02, 2.342745E+02, 2.699036E+02, 3.070478E+02, 3.455348E+02, 
   3.852056E+02, 4.259183E+02, 4.675481E+02, 5.099868E+02, 5.531407E+02, 5.969288E+02, 
   6.412812E+02, 6.861373E+02, 7.314446E+02, 7.771578E+02, 8.232370E+02, 8.696480E+02, 
   9.163606E+02, 9.633488E+02, 1.010590E+03, 1.058065E+03, 1.105755E+03, 1.153648E+03, 
   1.201731E+03, 1.249992E+03, 1.298424E+03, 1.347019E+03, 1.395772E+03, 1.444679E+03, 
   1.493735E+03, 1.542940E+03, 1.592292E+03, 1.641790E+03, 1.691435E+03, 1.741228E+03, 
   1.791171E+03, 1.841266E+03, 1.891516E+03, 1.941923E+03, 1.992492E+03, 2.043225E+03, 
   2.094127E+03, 2.145202E+03, 2.196454E+03, 2.247888E+03, 2.299508E+03, 2.351319E+03, 
   2.403326E+03, 2.455533E+03, 2.507945E+03, 2.560567E+03, 2.613403E+03, 2.666459E+03, 
   2.719738E+03, 2.773245E+03, 2.826986E+03, 2.880964E+03, 2.935183E+03, 2.989648E+03, 
   3.044363E+03, 3.099333E+03, 3.154560E+03, 3.210050E+03, 3.265805E+03, 3.321830E+03, 
   3.378128E+03, 3.434703E+03, 3.491557E+03, 3.548695E+03, 3.606120E+03, 3.663834E+03, 
   3.721842E+03, 3.780145E+03, 3.838746E+03, 3.897649E+03, 3.956856E+03, 4.016370E+03, 
   4.076193E+03, 4.136328E+03, 4.196777E+03, 4.257542E+03, 4.318625E+03, 4.380030E+03, 
   4.441757E+03, 4.503809E+03, 4.566188E+03, 4.628896E+03, 4.691934E+03, 4.755305E+03, 
   4.819009E+03, 4.883050E+03, 4.947427E+03, 5.012144E+03, 5.077201E+03, 5.142600E+03, 
   5.208342E+03, 5.274429E+03, 5.340863E+03, 5.407643E+03, 5.474772E+03, 5.542251E+03, 
   5.610081E+03, 5.678262E+03, 5.746798E+03, 5.815687E+03, 5.884932E+03, 5.954533E+03, 
   6.024492E+03, 6.094809E+03, 6.165485E+03, 6.236521E+03, 6.307918E+03, 6.379677E+03, 
   6.451799E+03, 6.524283E+03, 6.597133E+03, 6.670347E+03, 6.743927E+03, 6.817873E+03, 
   6.892186E+03, 6.966867E+03, 7.041916E+03, 7.117334E+03, 7.193122E+03, 7.269280E+03, 
   7.345809E+03, 7.422709E+03, 7.499982E+03, 7.577626E+03, 7.655644E+03, 7.734034E+03, 
   7.812799E+03, 7.891939E+03, 7.971453E+03, 8.051342E+03, 8.131608E+03, 8.212249E+03, 
   8.293267E+03, 8.374662E+03, 8.456435E+03, 8.538585E+03, 8.621114E+03, 8.704021E+03, 
   8.787307E+03, 8.870972E+03, 8.955017E+03, 9.039442E+03, 9.124247E+03, 9.209433E+03, 
   9.294999E+03, 9.380947E+03, 9.467277E+03, 9.553988E+03, 9.641081E+03, 9.728557E+03, 
   9.816415E+03, 9.904656E+03, 9.993280E+03, 1.008229E+04, 1.017168E+04, 1.026145E+04, 
   1.035161E+04, 1.044216E+04, 1.053308E+04, 1.062440E+04, 1.071609E+04, 1.080818E+04, 
   1.090065E+04, 1.099350E+04, 1.108674E+04, 1.118036E+04, 1.127438E+04, 1.136877E+04, 
   1.146356E+04, 1.155873E+04, 1.165429E+04, 1.175023E+04, 1.184657E+04, 1.194328E+04, 
   1.204039E+04, 1.213789E+04, 1.223577E+04, 1.233404E+04, 1.243270E+04, 1.253174E+04, 
   1.263118E+04, 1.273100E+04, 1.283121E+04, 1.293181E+04, 1.303280E+04, 1.313418E+04, 
   1.323595E+04, 1.333811E+04, 1.344065E+04, 1.354359E+04, 1.364691E+04, 1.375063E+04, 
   1.385473E+04, 1.395922E+04, 1.406411E+04, 1.416938E+04, 1.427504E+04, 1.438110E+04, 
   1.448754E+04, 1.459438E+04, 1.470160E+04, 1.480922E+04, 1.491722E+04, 1.502562E+04, 
   1.513440E+04, 1.524358E+04, 1.535315E+04, 1.546311E+04, 1.557346E+04, 1.568420E+04, 
   1.579533E+04, 1.590685E+04, 1.601876E+04, 1.613107E+04, 1.624376E+04, 1.635685E+04, 
   1.647033E+04, 1.658420E+04, 1.669846E+04, 1.681311E+04, 1.692815E+04, 1.704359E+04, 
   1.715942E+04, 1.727563E+04, 1.739224E+04, 1.750924E+04, 1.762663E+04, 1.774442E+04, 
   1.786259E+04, 1.798116E+04, 1.810011E+04, 1.821946E+04, 1.833920E+04, 1.845933E+04, 
   1.857986E+04, 1.870077E+04, 1.882208E+04, 1.894378E+04, 1.906587E+04, 1.918835E+04, 
   1.931122E+04, 1.943448E+04, 1.955813E+04, 1.968218E+04, 1.980662E+04, 1.993144E+04, 
   2.005666E+04, 2.018227E+04, 2.030827E+04, 2.043467E+04, 2.056145E+04, 2.068862E+04, 
   2.081619E+04, 2.094414E+04, 2.107249E+04, 2.120123E+04, 2.133036E+04, 2.145988E+04, 
   2.158978E+04, 2.172008E+04, 2.185078E+04, 2.198186E+04, 2.211333E+04, 2.224519E+04, 
   2.237744E+04, 2.251008E+04, 2.264311E+04, 2.277653E+04, 2.291035E+04, 2.304455E+04, 
   2.317914E+04, 2.331412E+04, 2.344949E+04, 2.358524E+04, 2.372139E+04, 2.385793E+04, 
   2.399486E+04, 2.413217E+04, 2.426988E+04, 2.440797E+04, 2.454645E+04, 2.468532E+04, 
   2.482458E+04, 2.496422E+04, 2.510426E+04, 2.524468E+04, 2.538549E+04, 2.552669E+04, 
   2.566828E+04, 2.581025E+04, 2.595261E+04, 2.609536E+04, 2.623849E+04, 2.638201E+04, 
   2.652592E+04, 2.667022E+04, 2.681490E+04, 2.695997E+04, 2.710542E+04, 2.725126E+04, 
   2.739749E+04, 2.754410E+04, 2.769109E+04, 2.783848E+04, 2.798624E+04, 2.813440E+04, 
   2.828293E+04, 2.843186E+04, 2.858116E+04, 2.873085E+04, 2.888093E+04, 2.903139E+04, 
   2.918223E+04, 2.933346E+04, 2.948506E+04, 2.963706E+04, 2.978943E+04, 2.994219E+04, 
   3.009533E+04, 3.024886E+04, 3.040276E+04, 3.055705E+04, 3.071172E+04, 3.086677E+04, 
   3.102220E+04, 3.117802E+04, 3.133421E+04, 3.149078E+04, 3.164774E+04, 3.180508E+04, 
   3.196279E+04, 3.212089E+04, 3.227936E+04, 3.243822E+04, 3.259745E+04, 3.275706E+04, 
   3.291706E+04, 3.307743E+04, 3.323817E+04, 3.339930E+04, 3.356080E+04, 3.372268E+04, 
   3.388494E+04, 3.404758E+04, 3.421059E+04, 3.437398E+04, 3.453774E+04, 3.470188E+04, 
   3.486640E+04, 3.503129E+04, 3.519656E+04, 3.536220E+04, 3.552821E+04, 3.569460E+04, 
   3.586137E+04, 3.602851E+04, 3.619602E+04, 3.636390E+04, 3.653216E+04, 3.670079E+04, 
   3.686979E+04, 3.703917E+04, 3.720892E+04, 3.737904E+04, 3.754953E+04, 3.772039E+04, 
   3.789162E+04, 3.806322E+04, 3.823519E+04, 3.840753E+04, 3.858025E+04, 3.875333E+04, 
   3.892678E+04, 3.910059E+04, 3.927478E+04, 3.944933E+04, 3.962426E+04, 3.979955E+04, 
   3.997520E+04, 4.015123E+04, 4.032762E+04, 4.050437E+04, 4.068150E+04, 4.085898E+04, 
   4.103684E+04, 4.121505E+04, 4.139364E+04, 4.157258E+04, 4.175189E+04, 4.193157E+04, 
   4.211160E+04, 4.229200E+04, 4.247276E+04, 4.265389E+04, 4.283538E+04, 4.301722E+04, 
   4.319943E+04, 4.338200E+04, 4.356493E+04, 4.374822E+04, 4.393188E+04, 4.411589E+04, 
   4.430026E+04, 4.448498E+04, 4.467007E+04, 4.485552E+04, 4.504132E+04, 4.522748E+04, 
   4.541400E+04, 4.560087E+04, 4.578810E+04, 4.597569E+04, 4.616363E+04, 4.635193E+04, 
   4.654058E+04, 4.672959E+04, 4.691895E+04, 4.710867E+04, 4.729874E+04, 4.748916E+04, 
   4.767993E+04, 4.787106E+04, 4.806254E+04, 4.825437E+04, 4.844655E+04, 4.863908E+04, 
   4.883197E+04, 4.902520E+04, 4.921879E+04, 4.941272E+04, 4.960700E+04, 4.980164E+04, 
   4.999661E+04, 5.019194E+04, 5.038762E+04, 5.058364E+04, 5.078001E+04, 5.097672E+04, 
   5.117379E+04, 5.137119E+04, 5.156894E+04, 5.176704E+04, 5.196548E+04, 5.216427E+04, 
   5.236340E+04, 5.256287E+04, 5.276269E+04, 5.296285E+04, 5.316335E+04, 5.336419E+04, 
   5.356538E+04, 5.376690E+04, 5.396876E+04, 5.417097E+04, 5.437352E+04, 5.457640E+04, 
   5.477962E+04, 5.498319E+04, 5.518709E+04, 5.539133E+04, 5.559590E+04, 5.580082E+04, 
   5.600607E+04, 5.621165E+04, 5.641757E+04, 5.662383E+04, 5.683042E+04, 5.703735E+04, 
   5.724461E+04, 5.745221E+04, 5.766013E+04, 5.786839E+04, 5.807699E+04, 5.828592E+04, 
   5.849517E+04, 5.870476E+04, 5.891468E+04, 5.912493E+04, 5.933551E+04, 5.954642E+04, 
   5.975766E+04, 5.996923E+04, 6.018113E+04, 6.039335E+04, 6.060591E+04, 6.081879E+04, 
   6.103199E+04, 6.124553E+04, 6.145939E+04, 
])
# ============================ M = 9, I = 1 ============================
M = 9
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.310610E+00, 3.639717E+01, 1.022291E+02, 1.873834E+02, 2.881851E+02, 4.025072E+02, 
   5.289155E+02, 6.663657E+02, 8.140731E+02, 9.714526E+02, 1.138087E+03, 1.313701E+03, 
   1.498146E+03, 1.691369E+03, 1.893400E+03, 2.104331E+03, 2.324303E+03, 2.553493E+03, 
   2.792108E+03, 3.040377E+03, 3.298550E+03, 3.566891E+03, 3.845682E+03, 4.135217E+03, 
   4.435804E+03, 4.747766E+03, 5.071436E+03, 5.407162E+03, 5.755305E+03, 6.116236E+03, 
   6.490342E+03, 6.878017E+03, 7.279673E+03, 7.695728E+03, 8.126614E+03, 8.572774E+03, 
   9.034660E+03, 9.512737E+03, 1.000748E+04, 1.051937E+04, 1.104890E+04, 1.159658E+04, 
   1.216291E+04, 1.274842E+04, 1.335365E+04, 1.397913E+04, 1.462541E+04, 1.529305E+04, 
   1.598262E+04, 1.669469E+04, 1.742985E+04, 1.818870E+04, 1.897183E+04, 1.977986E+04, 
   2.061341E+04, 2.147311E+04, 2.235959E+04, 2.327351E+04, 2.421552E+04, 2.518629E+04, 
   2.618649E+04, 2.721680E+04, 2.827792E+04, 2.937055E+04, 3.049540E+04, 3.165319E+04, 
   3.284465E+04, 3.407052E+04, 3.533155E+04, 3.662849E+04, 3.796211E+04, 3.933320E+04, 
   4.074253E+04, 4.219090E+04, 4.367913E+04, 4.520802E+04, 4.677839E+04, 4.839110E+04, 
   5.004697E+04, 5.174688E+04, 5.349167E+04, 5.528224E+04, 5.711946E+04, 5.900423E+04, 
   6.093745E+04, 6.292005E+04, 6.495295E+04, 6.703709E+04, 6.917341E+04, 7.136288E+04, 
   7.360646E+04, 7.590513E+04, 7.825988E+04, 8.067172E+04, 8.314165E+04, 8.567070E+04, 
   8.825990E+04, 9.091029E+04, 9.362293E+04, 9.639888E+04, 9.923923E+04, 1.021451E+05, 
   1.051175E+05, 1.081576E+05, 1.112665E+05, 1.144453E+05, 1.176953E+05, 1.210175E+05, 
   1.244131E+05, 1.278832E+05, 1.314292E+05, 1.350521E+05, 1.387533E+05, 1.425338E+05, 
   1.463950E+05, 1.503381E+05, 1.543644E+05, 1.584750E+05, 1.626714E+05, 1.669548E+05, 
   1.713266E+05, 1.757879E+05, 1.803402E+05, 1.849848E+05, 1.897231E+05, 1.945564E+05, 
   1.994861E+05, 2.045136E+05, 2.096403E+05, 2.148677E+05, 2.201970E+05, 2.256299E+05, 
   2.311676E+05, 2.368118E+05, 2.425639E+05, 2.484253E+05, 2.543976E+05, 2.604823E+05, 
   2.666808E+05, 2.729949E+05, 2.794259E+05, 2.859755E+05, 2.926453E+05, 2.994368E+05, 
   3.063516E+05, 3.133914E+05, 3.205578E+05, 3.278524E+05, 3.352769E+05, 3.428330E+05, 
   3.505223E+05, 3.583465E+05, 3.663074E+05, 3.744067E+05, 3.826460E+05, 3.910272E+05, 
   3.995520E+05, 4.082222E+05, 4.170395E+05, 4.260058E+05, 4.351230E+05, 4.443927E+05, 
   4.538168E+05, 4.633973E+05, 4.731360E+05, 4.830347E+05, 4.930954E+05, 5.033200E+05, 
   5.137103E+05, 5.242684E+05, 5.349962E+05, 5.458956E+05, 5.569686E+05, 5.682172E+05, 
   5.796435E+05, 5.912494E+05, 6.030371E+05, 6.150084E+05, 6.271655E+05, 6.395106E+05, 
   6.520455E+05, 6.647726E+05, 6.776938E+05, 6.908114E+05, 7.041274E+05, 7.176441E+05, 
   7.313636E+05, 7.452881E+05, 7.594198E+05, 7.737609E+05, 7.883138E+05, 8.030805E+05, 
   8.180634E+05, 8.332648E+05, 8.486870E+05, 8.643322E+05, 8.802028E+05, 8.963012E+05, 
   9.126297E+05, 9.291905E+05, 9.459863E+05, 9.630192E+05, 9.802919E+05, 9.978066E+05, 
   1.015566E+06, 1.033572E+06, 1.051828E+06, 1.070335E+06, 1.089097E+06, 1.108116E+06, 
   1.127395E+06, 1.146935E+06, 1.166740E+06, 1.186813E+06, 1.207155E+06, 1.227770E+06, 
   1.248659E+06, 1.269826E+06, 1.291274E+06, 1.313004E+06, 1.335021E+06, 1.357325E+06, 
   1.379921E+06, 1.402811E+06, 1.425997E+06, 1.449483E+06, 1.473270E+06, 1.497363E+06, 
   1.521764E+06, 1.546475E+06, 1.571499E+06, 1.596840E+06, 1.622500E+06, 1.648482E+06, 
   1.674789E+06, 1.701424E+06, 1.728390E+06, 1.755690E+06, 1.783327E+06, 1.811303E+06, 
   1.839622E+06, 1.868287E+06, 1.897301E+06, 1.926667E+06, 1.956388E+06, 1.986466E+06, 
   2.016906E+06, 2.047711E+06, 2.078883E+06, 2.110425E+06, 2.142342E+06, 2.174635E+06, 
   2.207308E+06, 2.240365E+06, 2.273809E+06, 2.307642E+06, 2.341869E+06, 2.376492E+06, 
   2.411515E+06, 2.446941E+06, 2.482773E+06, 2.519016E+06, 2.555671E+06, 2.592743E+06, 
   2.630235E+06, 2.668151E+06, 2.706493E+06, 2.745266E+06, 2.784472E+06, 2.824116E+06, 
   2.864200E+06, 2.904729E+06, 2.945706E+06, 2.987134E+06, 3.029018E+06, 3.071360E+06, 
   3.114164E+06, 3.157434E+06, 3.201173E+06, 3.245386E+06, 3.290076E+06, 3.335246E+06, 
   3.380901E+06, 3.427044E+06, 3.473678E+06, 3.520809E+06, 3.568438E+06, 3.616571E+06, 
   3.665211E+06, 3.714362E+06, 3.764028E+06, 3.814212E+06, 3.864919E+06, 3.916152E+06, 
   3.967916E+06, 4.020214E+06, 4.073050E+06, 4.126429E+06, 4.180354E+06, 4.234829E+06, 
   4.289859E+06, 4.345448E+06, 4.401598E+06, 4.458316E+06, 4.515604E+06, 4.573468E+06, 
   4.631910E+06, 4.690936E+06, 4.750549E+06, 4.810754E+06, 4.871555E+06, 4.932956E+06, 
   4.994962E+06, 5.057576E+06, 5.120803E+06, 5.184648E+06, 5.249114E+06, 5.314206E+06, 
   5.379929E+06, 5.446287E+06, 5.513284E+06, 5.580924E+06, 5.649213E+06, 5.718155E+06, 
   5.787753E+06, 5.858014E+06, 5.928940E+06, 6.000537E+06, 6.072810E+06, 6.145762E+06, 
   6.219400E+06, 6.293726E+06, 6.368746E+06, 6.444464E+06, 6.520886E+06, 6.598015E+06, 
   6.675858E+06, 6.754417E+06, 6.833699E+06, 6.913707E+06, 6.994448E+06, 7.075924E+06, 
   7.158142E+06, 7.241107E+06, 7.324822E+06, 7.409294E+06, 7.494527E+06, 7.580525E+06, 
   7.667295E+06, 7.754840E+06, 7.843166E+06, 7.932278E+06, 8.022181E+06, 8.112880E+06, 
   8.204379E+06, 8.296686E+06, 8.389803E+06, 8.483737E+06, 8.578492E+06, 8.674074E+06, 
   8.770489E+06, 8.867740E+06, 8.965834E+06, 9.064776E+06, 9.164570E+06, 9.265223E+06, 
   9.366740E+06, 9.469126E+06, 9.572386E+06, 9.676525E+06, 9.781550E+06, 9.887465E+06, 
   9.994277E+06, 1.010199E+07, 1.021061E+07, 1.032014E+07, 1.043059E+07, 1.054197E+07, 
   1.065427E+07, 1.076751E+07, 1.088169E+07, 1.099681E+07, 1.111289E+07, 1.122992E+07, 
   1.134792E+07, 1.146688E+07, 1.158682E+07, 1.170774E+07, 1.182965E+07, 1.195254E+07, 
   1.207644E+07, 1.220134E+07, 1.232725E+07, 1.245417E+07, 1.258212E+07, 1.271109E+07, 
   1.284109E+07, 1.297214E+07, 1.310422E+07, 1.323736E+07, 1.337156E+07, 1.350682E+07, 
   1.364315E+07, 1.378056E+07, 1.391904E+07, 1.405862E+07, 1.419928E+07, 1.434105E+07, 
   1.448392E+07, 1.462791E+07, 1.477302E+07, 1.491924E+07, 1.506661E+07, 1.521510E+07, 
   1.536475E+07, 1.551554E+07, 1.566749E+07, 1.582060E+07, 1.597488E+07, 1.613034E+07, 
   1.628698E+07, 1.644481E+07, 1.660383E+07, 1.676406E+07, 1.692549E+07, 1.708814E+07, 
   1.725201E+07, 1.741711E+07, 1.758345E+07, 1.775102E+07, 1.791985E+07, 1.808993E+07, 
   1.826127E+07, 1.843388E+07, 1.860776E+07, 1.878293E+07, 1.895939E+07, 1.913714E+07, 
   1.931620E+07, 1.949656E+07, 1.967824E+07, 1.986125E+07, 2.004558E+07, 2.023125E+07, 
   2.041827E+07, 2.060664E+07, 2.079637E+07, 2.098746E+07, 2.117992E+07, 2.137377E+07, 
   2.156900E+07, 2.176563E+07, 2.196366E+07, 2.216309E+07, 2.236394E+07, 2.256622E+07, 
   2.276992E+07, 2.297507E+07, 2.318166E+07, 2.338970E+07, 2.359920E+07, 2.381016E+07, 
   2.402260E+07, 2.423653E+07, 2.445194E+07, 2.466885E+07, 2.488727E+07, 2.510719E+07, 
   2.532864E+07, 2.555161E+07, 2.577612E+07, 2.600217E+07, 2.622978E+07, 2.645894E+07, 
   2.668966E+07, 2.692196E+07, 2.715584E+07, 2.739130E+07, 2.762837E+07, 2.786704E+07, 
   2.810732E+07, 2.834922E+07, 2.859275E+07, 2.883791E+07, 2.908472E+07, 2.933318E+07, 
   2.958330E+07, 2.983509E+07, 3.008856E+07, 3.034371E+07, 3.060055E+07, 3.085909E+07, 
   3.111934E+07, 3.138131E+07, 3.164500E+07, 3.191042E+07, 3.217759E+07, 3.244650E+07, 
   3.271717E+07, 3.298961E+07, 3.326382E+07, 
])
# ============================ M = 9, I = 2 ============================
M = 9
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.314657E+00, 3.655868E+01, 1.026901E+02, 1.882330E+02, 2.894953E+02, 4.043401E+02, 
   5.313267E+02, 6.694059E+02, 8.177892E+02, 9.758889E+02, 1.143285E+03, 1.319703E+03, 
   1.504990E+03, 1.699095E+03, 1.902048E+03, 2.113940E+03, 2.334913E+03, 2.565144E+03, 
   2.804842E+03, 3.054236E+03, 3.313578E+03, 3.583132E+03, 3.863182E+03, 4.154022E+03, 
   4.455963E+03, 4.769328E+03, 5.094451E+03, 5.431684E+03, 5.781386E+03, 6.143932E+03, 
   6.519709E+03, 6.909116E+03, 7.312562E+03, 7.730470E+03, 8.163273E+03, 8.611415E+03, 
   9.075352E+03, 9.555549E+03, 1.005248E+04, 1.056664E+04, 1.109851E+04, 1.164860E+04, 
   1.221744E+04, 1.280553E+04, 1.341343E+04, 1.404166E+04, 1.469078E+04, 1.536135E+04, 
   1.605395E+04, 1.676914E+04, 1.750753E+04, 1.826970E+04, 1.905626E+04, 1.986782E+04, 
   2.070502E+04, 2.156847E+04, 2.245883E+04, 2.337673E+04, 2.432285E+04, 2.529785E+04, 
   2.630240E+04, 2.733720E+04, 2.840294E+04, 2.950032E+04, 3.063005E+04, 3.179287E+04, 
   3.298950E+04, 3.422069E+04, 3.548718E+04, 3.678975E+04, 3.812915E+04, 3.950617E+04, 
   4.092160E+04, 4.237624E+04, 4.387090E+04, 4.540639E+04, 4.698356E+04, 4.860323E+04, 
   5.026625E+04, 5.197349E+04, 5.372581E+04, 5.552410E+04, 5.736923E+04, 5.926212E+04, 
   6.120367E+04, 6.319481E+04, 6.523646E+04, 6.732956E+04, 6.947507E+04, 7.167395E+04, 
   7.392717E+04, 7.623572E+04, 7.860058E+04, 8.102277E+04, 8.350331E+04, 8.604321E+04, 
   8.864351E+04, 9.130527E+04, 9.402954E+04, 9.681739E+04, 9.966991E+04, 1.025882E+05, 
   1.055733E+05, 1.086264E+05, 1.117486E+05, 1.149411E+05, 1.182049E+05, 1.215413E+05, 
   1.249514E+05, 1.284365E+05, 1.319976E+05, 1.356360E+05, 1.393529E+05, 1.431496E+05, 
   1.470273E+05, 1.509872E+05, 1.550306E+05, 1.591589E+05, 1.633732E+05, 1.676748E+05, 
   1.720652E+05, 1.765456E+05, 1.811173E+05, 1.857817E+05, 1.905401E+05, 1.953940E+05, 
   2.003447E+05, 2.053936E+05, 2.105421E+05, 2.157917E+05, 2.211438E+05, 2.265997E+05, 
   2.321610E+05, 2.378292E+05, 2.436057E+05, 2.494920E+05, 2.554897E+05, 2.616002E+05, 
   2.678252E+05, 2.741660E+05, 2.806243E+05, 2.872018E+05, 2.938998E+05, 3.007202E+05, 
   3.076643E+05, 3.147340E+05, 3.219308E+05, 3.292564E+05, 3.367124E+05, 3.443005E+05, 
   3.520224E+05, 3.598798E+05, 3.678744E+05, 3.760080E+05, 3.842822E+05, 3.926989E+05, 
   4.012598E+05, 4.099667E+05, 4.188214E+05, 4.278257E+05, 4.369814E+05, 4.462903E+05, 
   4.557544E+05, 4.653754E+05, 4.751553E+05, 4.850959E+05, 4.951992E+05, 5.054670E+05, 
   5.159012E+05, 5.265040E+05, 5.372771E+05, 5.482226E+05, 5.593424E+05, 5.706386E+05, 
   5.821132E+05, 5.937681E+05, 6.056055E+05, 6.176274E+05, 6.298359E+05, 6.422331E+05, 
   6.548210E+05, 6.676018E+05, 6.805775E+05, 6.937505E+05, 7.071227E+05, 7.206964E+05, 
   7.344738E+05, 7.484571E+05, 7.626484E+05, 7.770501E+05, 7.916643E+05, 8.064933E+05, 
   8.215394E+05, 8.368049E+05, 8.522921E+05, 8.680033E+05, 8.839408E+05, 9.001070E+05, 
   9.165042E+05, 9.331349E+05, 9.500014E+05, 9.671062E+05, 9.844516E+05, 1.002040E+06, 
   1.019874E+06, 1.037956E+06, 1.056289E+06, 1.074874E+06, 1.093715E+06, 1.112814E+06, 
   1.132174E+06, 1.151797E+06, 1.171685E+06, 1.191842E+06, 1.212270E+06, 1.232971E+06, 
   1.253949E+06, 1.275205E+06, 1.296743E+06, 1.318565E+06, 1.340674E+06, 1.363072E+06, 
   1.385763E+06, 1.408749E+06, 1.432032E+06, 1.455617E+06, 1.479505E+06, 1.503699E+06, 
   1.528202E+06, 1.553017E+06, 1.578146E+06, 1.603594E+06, 1.629362E+06, 1.655453E+06, 
   1.681870E+06, 1.708617E+06, 1.735697E+06, 1.763111E+06, 1.790864E+06, 1.818958E+06, 
   1.847396E+06, 1.876181E+06, 1.905317E+06, 1.934806E+06, 1.964651E+06, 1.994856E+06, 
   2.025424E+06, 2.056358E+06, 2.087661E+06, 2.119336E+06, 2.151386E+06, 2.183815E+06, 
   2.216625E+06, 2.249821E+06, 2.283405E+06, 2.317380E+06, 2.351750E+06, 2.386519E+06, 
   2.421688E+06, 2.457263E+06, 2.493246E+06, 2.529640E+06, 2.566449E+06, 2.603677E+06, 
   2.641326E+06, 2.679400E+06, 2.717904E+06, 2.756839E+06, 2.796210E+06, 2.836019E+06, 
   2.876272E+06, 2.916971E+06, 2.958119E+06, 2.999721E+06, 3.041780E+06, 3.084299E+06, 
   3.127283E+06, 3.170734E+06, 3.214657E+06, 3.259055E+06, 3.303932E+06, 3.349292E+06, 
   3.395138E+06, 3.441474E+06, 3.488304E+06, 3.535631E+06, 3.583460E+06, 3.631795E+06, 
   3.680638E+06, 3.729995E+06, 3.779869E+06, 3.830263E+06, 3.881182E+06, 3.932630E+06, 
   3.984610E+06, 4.037127E+06, 4.090185E+06, 4.143787E+06, 4.197938E+06, 4.252641E+06, 
   4.307901E+06, 4.363722E+06, 4.420108E+06, 4.477063E+06, 4.534591E+06, 4.592697E+06, 
   4.651384E+06, 4.710656E+06, 4.770519E+06, 4.830976E+06, 4.892031E+06, 4.953689E+06, 
   5.015954E+06, 5.078830E+06, 5.142321E+06, 5.206433E+06, 5.271169E+06, 5.336533E+06, 
   5.402531E+06, 5.469166E+06, 5.536443E+06, 5.604367E+06, 5.672941E+06, 5.742171E+06, 
   5.812060E+06, 5.882614E+06, 5.953837E+06, 6.025734E+06, 6.098308E+06, 6.171566E+06, 
   6.245511E+06, 6.320148E+06, 6.395481E+06, 6.471516E+06, 6.548257E+06, 6.625709E+06, 
   6.703876E+06, 6.782764E+06, 6.862377E+06, 6.942720E+06, 7.023797E+06, 7.105614E+06, 
   7.188176E+06, 7.271487E+06, 7.355552E+06, 7.440377E+06, 7.525965E+06, 7.612323E+06, 
   7.699455E+06, 7.787366E+06, 7.876061E+06, 7.965545E+06, 8.055823E+06, 8.146901E+06, 
   8.238783E+06, 8.331474E+06, 8.424980E+06, 8.519306E+06, 8.614457E+06, 8.710439E+06, 
   8.807255E+06, 8.904913E+06, 9.003416E+06, 9.102771E+06, 9.202982E+06, 9.304055E+06, 
   9.405995E+06, 9.508808E+06, 9.612499E+06, 9.717074E+06, 9.822537E+06, 9.928894E+06, 
   1.003615E+07, 1.014431E+07, 1.025339E+07, 1.036338E+07, 1.047429E+07, 1.058613E+07, 
   1.069890E+07, 1.081261E+07, 1.092726E+07, 1.104287E+07, 1.115943E+07, 1.127695E+07, 
   1.139544E+07, 1.151490E+07, 1.163534E+07, 1.175676E+07, 1.187918E+07, 1.200259E+07, 
   1.212700E+07, 1.225242E+07, 1.237885E+07, 1.250631E+07, 1.263478E+07, 1.276429E+07, 
   1.289484E+07, 1.302643E+07, 1.315907E+07, 1.329277E+07, 1.342752E+07, 1.356335E+07, 
   1.370024E+07, 1.383822E+07, 1.397729E+07, 1.411744E+07, 1.425870E+07, 1.440105E+07, 
   1.454452E+07, 1.468911E+07, 1.483482E+07, 1.498166E+07, 1.512963E+07, 1.527875E+07, 
   1.542902E+07, 1.558044E+07, 1.573302E+07, 1.588677E+07, 1.604169E+07, 1.619780E+07, 
   1.635509E+07, 1.651358E+07, 1.667326E+07, 1.683416E+07, 1.699627E+07, 1.715959E+07, 
   1.732415E+07, 1.748994E+07, 1.765696E+07, 1.782524E+07, 1.799476E+07, 1.816555E+07, 
   1.833761E+07, 1.851094E+07, 1.868555E+07, 1.886144E+07, 1.903864E+07, 1.921713E+07, 
   1.939693E+07, 1.957804E+07, 1.976048E+07, 1.994425E+07, 2.012935E+07, 2.031580E+07, 
   2.050359E+07, 2.069275E+07, 2.088326E+07, 2.107515E+07, 2.126842E+07, 2.146307E+07, 
   2.165912E+07, 2.185656E+07, 2.205541E+07, 2.225568E+07, 2.245737E+07, 2.266048E+07, 
   2.286504E+07, 2.307104E+07, 2.327848E+07, 2.348739E+07, 2.369776E+07, 2.390961E+07, 
   2.412294E+07, 2.433775E+07, 2.455406E+07, 2.477187E+07, 2.499120E+07, 2.521204E+07, 
   2.543441E+07, 2.565831E+07, 2.588375E+07, 2.611074E+07, 2.633929E+07, 2.656941E+07, 
   2.680109E+07, 2.703436E+07, 2.726921E+07, 2.750566E+07, 2.774371E+07, 2.798337E+07, 
   2.822465E+07, 2.846756E+07, 2.871210E+07, 2.895828E+07, 2.920612E+07, 2.945562E+07, 
   2.970678E+07, 2.995961E+07, 3.021413E+07, 3.047034E+07, 3.072825E+07, 3.098787E+07, 
   3.124920E+07, 3.151226E+07, 3.177705E+07, 3.204358E+07, 3.231185E+07, 3.258189E+07, 
   3.285368E+07, 3.312726E+07, 3.340261E+07, 
])
# ============================ M = 9, I = 3 ============================
M = 9
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.287149E+00, 1.468630E+02, 4.125020E+02, 7.561100E+02, 1.162858E+03, 1.624162E+03, 
   2.134236E+03, 2.688869E+03, 3.284897E+03, 3.919968E+03, 4.592410E+03, 5.301132E+03, 
   6.045541E+03, 6.825447E+03, 7.640986E+03, 8.492544E+03, 9.380702E+03, 1.030619E+04, 
   1.126986E+04, 1.227264E+04, 1.331555E+04, 1.439968E+04, 1.552617E+04, 1.669621E+04, 
   1.791108E+04, 1.917208E+04, 2.048057E+04, 2.183797E+04, 2.324576E+04, 2.470544E+04, 
   2.621858E+04, 2.778681E+04, 2.941180E+04, 3.109524E+04, 3.283891E+04, 3.464460E+04, 
   3.651417E+04, 3.844949E+04, 4.045251E+04, 4.252519E+04, 4.466954E+04, 4.688762E+04, 
   4.918151E+04, 5.155334E+04, 5.400528E+04, 5.653952E+04, 5.915831E+04, 6.186393E+04, 
   6.465867E+04, 6.754490E+04, 7.052499E+04, 7.360137E+04, 7.677649E+04, 8.005285E+04, 
   8.343296E+04, 8.691941E+04, 9.051479E+04, 9.422174E+04, 9.804292E+04, 1.019811E+05, 
   1.060389E+05, 1.102192E+05, 1.145249E+05, 1.189587E+05, 1.235236E+05, 1.282224E+05, 
   1.330583E+05, 1.380341E+05, 1.431530E+05, 1.484180E+05, 1.538322E+05, 1.593989E+05, 
   1.651212E+05, 1.710024E+05, 1.770457E+05, 1.832545E+05, 1.896321E+05, 1.961821E+05, 
   2.029077E+05, 2.098125E+05, 2.169000E+05, 2.241738E+05, 2.316375E+05, 2.392948E+05, 
   2.471493E+05, 2.552048E+05, 2.634651E+05, 2.719340E+05, 2.806152E+05, 2.895129E+05, 
   2.986309E+05, 3.079732E+05, 3.175438E+05, 3.273468E+05, 3.373864E+05, 3.476667E+05, 
   3.581919E+05, 3.689663E+05, 3.799942E+05, 3.912800E+05, 4.028279E+05, 4.146426E+05, 
   4.267283E+05, 4.390898E+05, 4.517315E+05, 4.646581E+05, 4.778742E+05, 4.913845E+05, 
   5.051939E+05, 5.193071E+05, 5.337289E+05, 5.484644E+05, 5.635183E+05, 5.788957E+05, 
   5.946017E+05, 6.106413E+05, 6.270197E+05, 6.437420E+05, 6.608135E+05, 6.782395E+05, 
   6.960253E+05, 7.141763E+05, 7.326979E+05, 7.515955E+05, 7.708748E+05, 7.905413E+05, 
   8.106006E+05, 8.310583E+05, 8.519203E+05, 8.731922E+05, 8.948800E+05, 9.169894E+05, 
   9.395265E+05, 9.624971E+05, 9.859074E+05, 1.009763E+06, 1.034071E+06, 1.058837E+06, 
   1.084067E+06, 1.109767E+06, 1.135945E+06, 1.162605E+06, 1.189756E+06, 1.217402E+06, 
   1.245551E+06, 1.274210E+06, 1.303384E+06, 1.333081E+06, 1.363308E+06, 1.394071E+06, 
   1.425377E+06, 1.457232E+06, 1.489646E+06, 1.522622E+06, 1.556170E+06, 1.590297E+06, 
   1.625008E+06, 1.660312E+06, 1.696217E+06, 1.732728E+06, 1.769854E+06, 1.807603E+06, 
   1.845981E+06, 1.884996E+06, 1.924656E+06, 1.964969E+06, 2.005942E+06, 2.047584E+06, 
   2.089901E+06, 2.132902E+06, 2.176594E+06, 2.220987E+06, 2.266088E+06, 2.311904E+06, 
   2.358445E+06, 2.405718E+06, 2.453733E+06, 2.502496E+06, 2.552017E+06, 2.602304E+06, 
   2.653365E+06, 2.705210E+06, 2.757847E+06, 2.811284E+06, 2.865530E+06, 2.920595E+06, 
   2.976487E+06, 3.033215E+06, 3.090787E+06, 3.149214E+06, 3.208504E+06, 3.268667E+06, 
   3.329711E+06, 3.391646E+06, 3.454481E+06, 3.518226E+06, 3.582890E+06, 3.648483E+06, 
   3.715015E+06, 3.782494E+06, 3.850931E+06, 3.920336E+06, 3.990718E+06, 4.062088E+06, 
   4.134454E+06, 4.207828E+06, 4.282220E+06, 4.357639E+06, 4.434096E+06, 4.511600E+06, 
   4.590164E+06, 4.669796E+06, 4.750508E+06, 4.832309E+06, 4.915211E+06, 4.999224E+06, 
   5.084358E+06, 5.170626E+06, 5.258036E+06, 5.346602E+06, 5.436332E+06, 5.527239E+06, 
   5.619334E+06, 5.712627E+06, 5.807130E+06, 5.902854E+06, 5.999811E+06, 6.098012E+06, 
   6.197468E+06, 6.298191E+06, 6.400193E+06, 6.503486E+06, 6.608080E+06, 6.713988E+06, 
   6.821223E+06, 6.929795E+06, 7.039716E+06, 7.151000E+06, 7.263658E+06, 7.377702E+06, 
   7.493144E+06, 7.609997E+06, 7.728274E+06, 7.847986E+06, 7.969147E+06, 8.091769E+06, 
   8.215864E+06, 8.341446E+06, 8.468526E+06, 8.597119E+06, 8.727238E+06, 8.858894E+06, 
   8.992101E+06, 9.126872E+06, 9.263222E+06, 9.401162E+06, 9.540706E+06, 9.681868E+06, 
   9.824661E+06, 9.969099E+06, 1.011520E+07, 1.026296E+07, 1.041242E+07, 1.056357E+07, 
   1.071644E+07, 1.087103E+07, 1.102737E+07, 1.118546E+07, 1.134533E+07, 1.150697E+07, 
   1.167042E+07, 1.183568E+07, 1.200277E+07, 1.217169E+07, 1.234248E+07, 1.251514E+07, 
   1.268968E+07, 1.286612E+07, 1.304448E+07, 1.322478E+07, 1.340701E+07, 1.359121E+07, 
   1.377739E+07, 1.396555E+07, 1.415573E+07, 1.434792E+07, 1.454216E+07, 1.473844E+07, 
   1.493680E+07, 1.513724E+07, 1.533979E+07, 1.554444E+07, 1.575124E+07, 1.596018E+07, 
   1.617128E+07, 1.638456E+07, 1.660005E+07, 1.681774E+07, 1.703767E+07, 1.725984E+07, 
   1.748427E+07, 1.771098E+07, 1.793999E+07, 1.817131E+07, 1.840496E+07, 1.864096E+07, 
   1.887932E+07, 1.912006E+07, 1.936320E+07, 1.960875E+07, 1.985674E+07, 2.010717E+07, 
   2.036007E+07, 2.061546E+07, 2.087335E+07, 2.113375E+07, 2.139670E+07, 2.166220E+07, 
   2.193027E+07, 2.220093E+07, 2.247421E+07, 2.275011E+07, 2.302865E+07, 2.330986E+07, 
   2.359375E+07, 2.388034E+07, 2.416966E+07, 2.446170E+07, 2.475651E+07, 2.505409E+07, 
   2.535446E+07, 2.565765E+07, 2.596367E+07, 2.627254E+07, 2.658428E+07, 2.689890E+07, 
   2.721644E+07, 2.753691E+07, 2.786032E+07, 2.818670E+07, 2.851606E+07, 2.884844E+07, 
   2.918384E+07, 2.952228E+07, 2.986379E+07, 3.020839E+07, 3.055609E+07, 3.090692E+07, 
   3.126089E+07, 3.161803E+07, 3.197836E+07, 3.234190E+07, 3.270866E+07, 3.307868E+07, 
   3.345196E+07, 3.382854E+07, 3.420842E+07, 3.459164E+07, 3.497822E+07, 3.536816E+07, 
   3.576151E+07, 3.615827E+07, 3.655847E+07, 3.696213E+07, 3.736927E+07, 3.777992E+07, 
   3.819409E+07, 3.861181E+07, 3.903310E+07, 3.945797E+07, 3.988647E+07, 4.031859E+07, 
   4.075438E+07, 4.119384E+07, 4.163701E+07, 4.208390E+07, 4.253454E+07, 4.298895E+07, 
   4.344716E+07, 4.390918E+07, 4.437503E+07, 4.484475E+07, 4.531836E+07, 4.579587E+07, 
   4.627731E+07, 4.676271E+07, 4.725208E+07, 4.774546E+07, 4.824287E+07, 4.874432E+07, 
   4.924984E+07, 4.975946E+07, 5.027321E+07, 5.079109E+07, 5.131315E+07, 5.183940E+07, 
   5.236986E+07, 5.290457E+07, 5.344354E+07, 5.398681E+07, 5.453439E+07, 5.508631E+07, 
   5.564259E+07, 5.620326E+07, 5.676835E+07, 5.733788E+07, 5.791188E+07, 5.849036E+07, 
   5.907336E+07, 5.966091E+07, 6.025302E+07, 6.084972E+07, 6.145104E+07, 6.205700E+07, 
   6.266764E+07, 6.328297E+07, 6.390302E+07, 6.452782E+07, 6.515739E+07, 6.579177E+07, 
   6.643097E+07, 6.707503E+07, 6.772396E+07, 6.837780E+07, 6.903658E+07, 6.970032E+07, 
   7.036905E+07, 7.104279E+07, 7.172157E+07, 7.240542E+07, 7.309437E+07, 7.378844E+07, 
   7.448766E+07, 7.519207E+07, 7.590168E+07, 7.661652E+07, 7.733663E+07, 7.806203E+07, 
   7.879274E+07, 7.952881E+07, 8.027024E+07, 8.101709E+07, 8.176936E+07, 8.252710E+07, 
   8.329032E+07, 8.405906E+07, 8.483335E+07, 8.561321E+07, 8.639868E+07, 8.718978E+07, 
   8.798654E+07, 8.878900E+07, 8.959718E+07, 9.041111E+07, 9.123082E+07, 9.205634E+07, 
   9.288770E+07, 9.372493E+07, 9.456806E+07, 9.541712E+07, 9.627214E+07, 9.713315E+07, 
   9.800018E+07, 9.887326E+07, 9.975242E+07, 1.006377E+08, 1.015291E+08, 1.024267E+08, 
   1.033305E+08, 1.042405E+08, 1.051568E+08, 1.060794E+08, 1.070084E+08, 1.079436E+08, 
   1.088853E+08, 1.098334E+08, 1.107880E+08, 1.117490E+08, 1.127166E+08, 1.136907E+08, 
   1.146714E+08, 1.156587E+08, 1.166527E+08, 1.176534E+08, 1.186607E+08, 1.196748E+08, 
   1.206957E+08, 1.217234E+08, 1.227579E+08, 1.237993E+08, 1.248476E+08, 1.259029E+08, 
   1.269651E+08, 1.280344E+08, 1.291106E+08, 1.301940E+08, 1.312845E+08, 1.323821E+08, 
   1.334869E+08, 1.345989E+08, 1.357182E+08, 
])
# ============================ M = 9, I = 4 ============================
M = 9
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.792284E+00, 7.808710E+01, 2.194074E+02, 4.022190E+02, 6.186290E+02, 8.640697E+02, 
   1.135463E+03, 1.430569E+03, 1.747716E+03, 2.085669E+03, 2.443562E+03, 2.820844E+03, 
   3.217232E+03, 3.632658E+03, 4.067225E+03, 4.521171E+03, 4.994831E+03, 5.488622E+03, 
   6.003019E+03, 6.538547E+03, 7.095771E+03, 7.675295E+03, 8.277751E+03, 8.903807E+03, 
   9.554158E+03, 1.022953E+04, 1.093068E+04, 1.165838E+04, 1.241346E+04, 1.319675E+04, 
   1.400911E+04, 1.485145E+04, 1.572467E+04, 1.662972E+04, 1.756759E+04, 1.853924E+04, 
   1.954572E+04, 2.058805E+04, 2.166730E+04, 2.278456E+04, 2.394093E+04, 2.513755E+04, 
   2.637555E+04, 2.765612E+04, 2.898045E+04, 3.034975E+04, 3.176524E+04, 3.322819E+04, 
   3.473986E+04, 3.630155E+04, 3.791457E+04, 3.958025E+04, 4.129994E+04, 4.307503E+04, 
   4.490689E+04, 4.679695E+04, 4.874663E+04, 5.075739E+04, 5.283070E+04, 5.496806E+04, 
   5.717097E+04, 5.944098E+04, 6.177964E+04, 6.418853E+04, 6.666924E+04, 6.922338E+04, 
   7.185261E+04, 7.455858E+04, 7.734298E+04, 8.020750E+04, 8.315387E+04, 8.618384E+04, 
   8.929918E+04, 9.250167E+04, 9.579314E+04, 9.917541E+04, 1.026504E+05, 1.062198E+05, 
   1.098858E+05, 1.136501E+05, 1.175147E+05, 1.214816E+05, 1.255528E+05, 1.297303E+05, 
   1.340161E+05, 1.384124E+05, 1.429211E+05, 1.475445E+05, 1.522845E+05, 1.571435E+05, 
   1.621236E+05, 1.672269E+05, 1.724558E+05, 1.778124E+05, 1.832991E+05, 1.889182E+05, 
   1.946719E+05, 2.005627E+05, 2.065929E+05, 2.127649E+05, 2.190812E+05, 2.255442E+05, 
   2.321563E+05, 2.389202E+05, 2.458382E+05, 2.529130E+05, 2.601472E+05, 2.675433E+05, 
   2.751040E+05, 2.828320E+05, 2.907298E+05, 2.988004E+05, 3.070462E+05, 3.154703E+05, 
   3.240752E+05, 3.328639E+05, 3.418392E+05, 3.510039E+05, 3.603610E+05, 3.699134E+05, 
   3.796639E+05, 3.896157E+05, 3.997717E+05, 4.101349E+05, 4.207083E+05, 4.314952E+05, 
   4.424986E+05, 4.537215E+05, 4.651673E+05, 4.768391E+05, 4.887401E+05, 5.008735E+05, 
   5.132428E+05, 5.258511E+05, 5.387017E+05, 5.517982E+05, 5.651438E+05, 5.787419E+05, 
   5.925962E+05, 6.067099E+05, 6.210866E+05, 6.357299E+05, 6.506433E+05, 6.658305E+05, 
   6.812949E+05, 6.970404E+05, 7.130706E+05, 7.293892E+05, 7.459998E+05, 7.629065E+05, 
   7.801128E+05, 7.976226E+05, 8.154399E+05, 8.335685E+05, 8.520122E+05, 8.707752E+05, 
   8.898613E+05, 9.092745E+05, 9.290190E+05, 9.490988E+05, 9.695179E+05, 9.902805E+05, 
   1.011391E+06, 1.032853E+06, 1.054671E+06, 1.076850E+06, 1.099393E+06, 1.122306E+06, 
   1.145591E+06, 1.169254E+06, 1.193300E+06, 1.217732E+06, 1.242555E+06, 1.267774E+06, 
   1.293392E+06, 1.319416E+06, 1.345848E+06, 1.372695E+06, 1.399960E+06, 1.427648E+06, 
   1.455764E+06, 1.484314E+06, 1.513300E+06, 1.542729E+06, 1.572606E+06, 1.602934E+06, 
   1.633720E+06, 1.664967E+06, 1.696682E+06, 1.728869E+06, 1.761533E+06, 1.794679E+06, 
   1.828312E+06, 1.862438E+06, 1.897062E+06, 1.932188E+06, 1.967823E+06, 2.003971E+06, 
   2.040638E+06, 2.077829E+06, 2.115550E+06, 2.153806E+06, 2.192602E+06, 2.231944E+06, 
   2.271838E+06, 2.312288E+06, 2.353302E+06, 2.394884E+06, 2.437039E+06, 2.479774E+06, 
   2.523095E+06, 2.567007E+06, 2.611515E+06, 2.656627E+06, 2.702347E+06, 2.748682E+06, 
   2.795638E+06, 2.843220E+06, 2.891434E+06, 2.940287E+06, 2.989785E+06, 3.039934E+06, 
   3.090740E+06, 3.142209E+06, 3.194347E+06, 3.247161E+06, 3.300657E+06, 3.354842E+06, 
   3.409721E+06, 3.465301E+06, 3.521589E+06, 3.578590E+06, 3.636313E+06, 3.694762E+06, 
   3.753946E+06, 3.813869E+06, 3.874540E+06, 3.935965E+06, 3.998150E+06, 4.061102E+06, 
   4.124828E+06, 4.189335E+06, 4.254630E+06, 4.320720E+06, 4.387612E+06, 4.455312E+06, 
   4.523828E+06, 4.593167E+06, 4.663336E+06, 4.734342E+06, 4.806192E+06, 4.878894E+06, 
   4.952454E+06, 5.026881E+06, 5.102181E+06, 5.178362E+06, 5.255431E+06, 5.333396E+06, 
   5.412264E+06, 5.492042E+06, 5.572739E+06, 5.654362E+06, 5.736919E+06, 5.820417E+06, 
   5.904864E+06, 5.990267E+06, 6.076635E+06, 6.163976E+06, 6.252296E+06, 6.341606E+06, 
   6.431911E+06, 6.523220E+06, 6.615542E+06, 6.708884E+06, 6.803254E+06, 6.898661E+06, 
   6.995112E+06, 7.092617E+06, 7.191183E+06, 7.290819E+06, 7.391533E+06, 7.493333E+06, 
   7.596228E+06, 7.700226E+06, 7.805336E+06, 7.911566E+06, 8.018926E+06, 8.127423E+06, 
   8.237066E+06, 8.347865E+06, 8.459827E+06, 8.572962E+06, 8.687279E+06, 8.802786E+06, 
   8.919492E+06, 9.037406E+06, 9.156538E+06, 9.276896E+06, 9.398489E+06, 9.521327E+06, 
   9.645419E+06, 9.770774E+06, 9.897401E+06, 1.002531E+07, 1.015451E+07, 1.028501E+07, 
   1.041682E+07, 1.054995E+07, 1.068440E+07, 1.082020E+07, 1.095734E+07, 1.109584E+07, 
   1.123571E+07, 1.137696E+07, 1.151959E+07, 1.166362E+07, 1.180905E+07, 1.195590E+07, 
   1.210418E+07, 1.225389E+07, 1.240505E+07, 1.255766E+07, 1.271174E+07, 1.286730E+07, 
   1.302435E+07, 1.318289E+07, 1.334294E+07, 1.350450E+07, 1.366760E+07, 1.383223E+07, 
   1.399841E+07, 1.416616E+07, 1.433547E+07, 1.450636E+07, 1.467884E+07, 1.485292E+07, 
   1.502862E+07, 1.520594E+07, 1.538489E+07, 1.556549E+07, 1.574775E+07, 1.593167E+07, 
   1.611727E+07, 1.630456E+07, 1.649354E+07, 1.668424E+07, 1.687666E+07, 1.707082E+07, 
   1.726672E+07, 1.746437E+07, 1.766380E+07, 1.786500E+07, 1.806799E+07, 1.827278E+07, 
   1.847939E+07, 1.868782E+07, 1.889808E+07, 1.911020E+07, 1.932418E+07, 1.954002E+07, 
   1.975775E+07, 1.997738E+07, 2.019891E+07, 2.042236E+07, 2.064774E+07, 2.087507E+07, 
   2.110435E+07, 2.133560E+07, 2.156882E+07, 2.180404E+07, 2.204126E+07, 2.228050E+07, 
   2.252177E+07, 2.276508E+07, 2.301044E+07, 2.325787E+07, 2.350738E+07, 2.375898E+07, 
   2.401268E+07, 2.426849E+07, 2.452644E+07, 2.478653E+07, 2.504878E+07, 2.531319E+07, 
   2.557978E+07, 2.584857E+07, 2.611956E+07, 2.639277E+07, 2.666822E+07, 2.694591E+07, 
   2.722586E+07, 2.750808E+07, 2.779259E+07, 2.807940E+07, 2.836852E+07, 2.865997E+07, 
   2.895376E+07, 2.924990E+07, 2.954840E+07, 2.984929E+07, 3.015257E+07, 3.045826E+07, 
   3.076637E+07, 3.107692E+07, 3.138991E+07, 3.170537E+07, 3.202330E+07, 3.234373E+07, 
   3.266666E+07, 3.299212E+07, 3.332010E+07, 3.365064E+07, 3.398374E+07, 3.431941E+07, 
   3.465767E+07, 3.499854E+07, 3.534203E+07, 3.568816E+07, 3.603693E+07, 3.638837E+07, 
   3.674248E+07, 3.709930E+07, 3.745881E+07, 3.782105E+07, 3.818603E+07, 3.855376E+07, 
   3.892426E+07, 3.929754E+07, 3.967362E+07, 4.005251E+07, 4.043423E+07, 4.081880E+07, 
   4.120622E+07, 4.159652E+07, 4.198970E+07, 4.238579E+07, 4.278480E+07, 4.318675E+07, 
   4.359165E+07, 4.399952E+07, 4.441037E+07, 4.482421E+07, 4.524108E+07, 4.566097E+07, 
   4.608391E+07, 4.650991E+07, 4.693899E+07, 4.737116E+07, 4.780645E+07, 4.824486E+07, 
   4.868642E+07, 4.913113E+07, 4.957902E+07, 5.003010E+07, 5.048439E+07, 5.094191E+07, 
   5.140266E+07, 5.186668E+07, 5.233397E+07, 5.280455E+07, 5.327843E+07, 5.375565E+07, 
   5.423620E+07, 5.472012E+07, 5.520741E+07, 5.569809E+07, 5.619218E+07, 5.668970E+07, 
   5.719066E+07, 5.769509E+07, 5.820299E+07, 5.871439E+07, 5.922930E+07, 5.974774E+07, 
   6.026974E+07, 6.079529E+07, 6.132443E+07, 6.185718E+07, 6.239354E+07, 6.293353E+07, 
   6.347719E+07, 6.402451E+07, 6.457552E+07, 6.513025E+07, 6.568870E+07, 6.625089E+07, 
   6.681685E+07, 6.738658E+07, 6.796012E+07, 6.853747E+07, 6.911866E+07, 6.970371E+07, 
   7.029262E+07, 7.088543E+07, 7.148215E+07, 7.208279E+07, 7.268739E+07, 7.329594E+07, 
   7.390849E+07, 7.452504E+07, 7.514560E+07, 
])
# ============================ M = 10, I = 1 ============================
M = 10
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.067397E+00, 8.327548E+01, 2.332343E+02, 4.271098E+02, 6.565616E+02, 9.167586E+02, 
   1.204438E+03, 1.517205E+03, 1.853203E+03, 2.210947E+03, 2.589220E+03, 2.987022E+03, 
   3.403534E+03, 3.838095E+03, 4.290193E+03, 4.759444E+03, 5.245589E+03, 5.748470E+03, 
   6.268031E+03, 6.804295E+03, 7.357364E+03, 7.927403E+03, 8.514637E+03, 9.119340E+03, 
   9.741831E+03, 1.038247E+04, 1.104166E+04, 1.171982E+04, 1.241742E+04, 1.313495E+04, 
   1.387291E+04, 1.463185E+04, 1.541232E+04, 1.621490E+04, 1.704019E+04, 1.788881E+04, 
   1.876139E+04, 1.965859E+04, 2.058106E+04, 2.152949E+04, 2.250457E+04, 2.350701E+04, 
   2.453754E+04, 2.559688E+04, 2.668578E+04, 2.780500E+04, 2.895531E+04, 3.013749E+04, 
   3.135233E+04, 3.260064E+04, 3.388322E+04, 3.520090E+04, 3.655452E+04, 3.794493E+04, 
   3.937297E+04, 4.083951E+04, 4.234544E+04, 4.389163E+04, 4.547899E+04, 4.710842E+04, 
   4.878084E+04, 5.049718E+04, 5.225838E+04, 5.406539E+04, 5.591917E+04, 5.782069E+04, 
   5.977093E+04, 6.177088E+04, 6.382155E+04, 6.592396E+04, 6.807912E+04, 7.028808E+04, 
   7.255188E+04, 7.487158E+04, 7.724825E+04, 7.968297E+04, 8.217684E+04, 8.473095E+04, 
   8.734644E+04, 9.002441E+04, 9.276602E+04, 9.557241E+04, 9.844475E+04, 1.013842E+05, 
   1.043920E+05, 1.074692E+05, 1.106172E+05, 1.138372E+05, 1.171303E+05, 1.204978E+05, 
   1.239411E+05, 1.274612E+05, 1.310596E+05, 1.347376E+05, 1.384964E+05, 1.423374E+05, 
   1.462619E+05, 1.502713E+05, 1.543669E+05, 1.585500E+05, 1.628222E+05, 
])
# ============================ M = 10, I = 2 ============================
M = 10
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[0]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.100026E+00, 5.691524E+01, 1.594094E+02, 2.919203E+02, 4.487475E+02, 6.265887E+02, 
   8.232140E+02, 1.036986E+03, 1.266637E+03, 1.511152E+03, 1.769702E+03, 2.041605E+03, 
   2.326307E+03, 2.623366E+03, 2.932438E+03, 3.253273E+03, 3.585704E+03, 3.929639E+03, 
   4.285049E+03, 4.651965E+03, 5.030470E+03, 5.420691E+03, 5.822792E+03, 6.236977E+03, 
   6.663477E+03, 7.102552E+03, 7.554488E+03, 8.019592E+03, 8.498193E+03, 8.990638E+03, 
   9.497292E+03, 1.001854E+04, 1.055477E+04, 1.110640E+04, 1.167386E+04, 1.225758E+04, 
   1.285800E+04, 1.347560E+04, 1.411084E+04, 1.476420E+04, 1.543617E+04, 1.612726E+04, 
   1.683798E+04, 1.756883E+04, 1.832035E+04, 1.909308E+04, 1.988756E+04, 2.070433E+04, 
   2.154397E+04, 2.240702E+04, 2.329408E+04, 2.420572E+04, 2.514253E+04, 2.610511E+04, 
   2.709406E+04, 2.810999E+04, 2.915352E+04, 3.022528E+04, 3.132590E+04, 3.245603E+04, 
   3.361630E+04, 3.480738E+04, 3.602994E+04, 3.728463E+04, 3.857214E+04, 3.989316E+04, 
   4.124838E+04, 4.263850E+04, 4.406422E+04, 4.552628E+04, 4.702538E+04, 4.856227E+04, 
   5.013768E+04, 5.175237E+04, 5.340708E+04, 5.510259E+04, 5.683966E+04, 5.861909E+04, 
   6.044165E+04, 6.230814E+04, 6.421938E+04, 6.617618E+04, 6.817935E+04, 7.022973E+04, 
   7.232816E+04, 7.447549E+04, 7.667258E+04, 7.892029E+04, 8.121949E+04, 8.357108E+04, 
   8.597593E+04, 8.843497E+04, 9.094908E+04, 9.351921E+04, 9.614626E+04, 9.883118E+04, 
   1.015749E+05, 1.043784E+05, 1.072427E+05, 1.101686E+05, 1.131573E+05, 
])
# ============================ M = 11, I = 1 ============================
M = 11
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.834090E+00, 1.620772E+01, 3.544986E+01, 5.997985E+01, 8.933083E+01, 1.228953E+02, 
   1.601234E+02, 2.006201E+02, 2.441067E+02, 2.903764E+02, 3.392668E+02, 3.906447E+02, 
   4.443978E+02, 5.004302E+02, 5.586602E+02, 6.190181E+02, 6.814457E+02, 7.458952E+02, 
   8.123287E+02, 8.807174E+02, 9.510416E+02, 1.023289E+03, 1.097456E+03, 1.173546E+03, 
   1.251567E+03, 1.331537E+03, 1.413475E+03, 1.497409E+03, 1.583372E+03, 1.671398E+03, 
   1.761529E+03, 1.853809E+03, 1.948287E+03, 2.045015E+03, 2.144046E+03, 2.245439E+03, 
   2.349255E+03, 2.455557E+03, 2.564410E+03, 2.675882E+03, 2.790043E+03, 2.906965E+03, 
   3.026722E+03, 3.149390E+03, 3.275047E+03, 3.403771E+03, 3.535644E+03, 3.670748E+03, 
   3.809167E+03, 3.950988E+03, 4.096297E+03, 4.245183E+03, 4.397737E+03, 4.554052E+03, 
   4.714220E+03, 4.878336E+03, 5.046499E+03, 5.218806E+03, 5.395358E+03, 5.576256E+03, 
   5.761605E+03, 5.951509E+03, 6.146077E+03, 6.345417E+03, 6.549640E+03, 6.758861E+03, 
   6.973192E+03, 7.192752E+03, 7.417660E+03, 7.648037E+03, 7.884006E+03, 8.125692E+03, 
   8.373224E+03, 8.626732E+03, 8.886347E+03, 9.152205E+03, 9.424443E+03, 9.703199E+03, 
   9.988617E+03, 1.028084E+04, 1.058002E+04, 1.088630E+04, 1.119983E+04, 1.152078E+04, 
   1.184929E+04, 1.218553E+04, 1.252967E+04, 1.288186E+04, 1.324228E+04, 1.361111E+04, 
   1.398851E+04, 1.437466E+04, 1.476975E+04, 1.517395E+04, 1.558747E+04, 1.601048E+04, 
   1.644319E+04, 1.688579E+04, 1.733848E+04, 1.780146E+04, 1.827495E+04, 1.875915E+04, 
   1.925429E+04, 1.976057E+04, 2.027822E+04, 2.080747E+04, 2.134854E+04, 2.190167E+04, 
   2.246709E+04, 2.304505E+04, 2.363579E+04, 2.423956E+04, 2.485661E+04, 2.548720E+04, 
   2.613158E+04, 2.679003E+04, 2.746282E+04, 2.815021E+04, 2.885248E+04, 2.956992E+04, 
   3.030281E+04, 3.105144E+04, 3.181610E+04, 3.259711E+04, 3.339475E+04, 3.420934E+04, 
   3.504120E+04, 3.589063E+04, 3.675797E+04, 3.764353E+04, 3.854766E+04, 3.947069E+04, 
   4.041296E+04, 4.137482E+04, 4.235662E+04, 4.335872E+04, 4.438148E+04, 4.542527E+04, 
   4.649045E+04, 4.757741E+04, 4.868654E+04, 4.981821E+04, 5.097282E+04, 5.215078E+04, 
   5.335248E+04, 5.457834E+04, 5.582878E+04, 5.710420E+04, 5.840505E+04, 5.973175E+04, 
   6.108474E+04, 6.246446E+04, 6.387137E+04, 6.530592E+04, 6.676857E+04, 6.825978E+04, 
   6.978004E+04, 7.132982E+04, 7.290961E+04, 7.451989E+04, 7.616117E+04, 7.783395E+04, 
   7.953873E+04, 8.127603E+04, 8.304638E+04, 8.485030E+04, 8.668833E+04, 8.856100E+04, 
   9.046886E+04, 9.241247E+04, 9.439239E+04, 9.640918E+04, 9.846341E+04, 1.005557E+05, 
   1.026865E+05, 1.048566E+05, 1.070664E+05, 1.093167E+05, 1.116079E+05, 1.139408E+05, 
   1.163160E+05, 1.187340E+05, 1.211955E+05, 1.237012E+05, 1.262517E+05, 1.288477E+05, 
   1.314898E+05, 1.341787E+05, 1.369150E+05, 1.396995E+05, 1.425329E+05, 1.454158E+05, 
   1.483489E+05, 1.513330E+05, 1.543688E+05, 1.574569E+05, 1.605982E+05, 1.637932E+05, 
   1.670429E+05, 1.703479E+05, 1.737090E+05, 1.771269E+05, 1.806025E+05, 1.841364E+05, 
   1.877295E+05, 1.913825E+05, 1.950963E+05, 1.988716E+05, 2.027093E+05, 2.066101E+05, 
   2.105748E+05, 2.146044E+05, 2.186995E+05, 2.228611E+05, 2.270900E+05, 2.313871E+05, 
   2.357531E+05, 2.401890E+05, 2.446956E+05, 2.492737E+05, 2.539243E+05, 2.586483E+05, 
   2.634465E+05, 2.683198E+05, 2.732691E+05, 2.782953E+05, 2.833994E+05, 2.885822E+05, 
   2.938447E+05, 2.991877E+05, 3.046123E+05, 3.101193E+05, 3.157098E+05, 3.213845E+05, 
   3.271446E+05, 3.329909E+05, 3.389245E+05, 3.449462E+05, 3.510570E+05, 3.572580E+05, 
   3.635501E+05, 3.699343E+05, 3.764116E+05, 3.829829E+05, 3.896493E+05, 3.964118E+05, 
   4.032714E+05, 4.102291E+05, 4.172859E+05, 4.244428E+05, 4.317008E+05, 4.390611E+05, 
   4.465246E+05, 4.540923E+05, 4.617653E+05, 4.695446E+05, 4.774313E+05, 4.854264E+05, 
   4.935310E+05, 5.017462E+05, 5.100729E+05, 5.185123E+05, 5.270654E+05, 5.357333E+05, 
   5.445171E+05, 5.534178E+05, 5.624366E+05, 5.715744E+05, 5.808324E+05, 5.902116E+05, 
   5.997132E+05, 6.093383E+05, 6.190879E+05, 6.289631E+05, 6.389651E+05, 6.490948E+05, 
   6.593535E+05, 6.697422E+05, 6.802621E+05, 6.909142E+05, 7.016996E+05, 7.126195E+05, 
   7.236749E+05, 7.348671E+05, 7.461970E+05, 7.576658E+05, 7.692747E+05, 7.810247E+05, 
   7.929169E+05, 8.049526E+05, 8.171327E+05, 8.294584E+05, 8.419309E+05, 8.545512E+05, 
   8.673206E+05, 8.802400E+05, 8.933106E+05, 9.065336E+05, 9.199101E+05, 9.334412E+05, 
   9.471280E+05, 9.609716E+05, 9.749732E+05, 9.891339E+05, 1.003455E+06, 1.017937E+06, 
   1.032582E+06, 1.047390E+06, 1.062363E+06, 1.077502E+06, 1.092807E+06, 1.108281E+06, 
   1.123924E+06, 1.139737E+06, 1.155722E+06, 1.171879E+06, 1.188210E+06, 1.204716E+06, 
   1.221397E+06, 1.238255E+06, 1.255292E+06, 1.272507E+06, 1.289903E+06, 1.307480E+06, 
   1.325240E+06, 1.343183E+06, 1.361311E+06, 1.379625E+06, 1.398126E+06, 1.416814E+06, 
   1.435692E+06, 1.454760E+06, 1.474019E+06, 1.493470E+06, 1.513114E+06, 1.532954E+06, 
   1.552988E+06, 1.573219E+06, 1.593648E+06, 1.614276E+06, 1.635103E+06, 1.656131E+06, 
   1.677361E+06, 1.698794E+06, 1.720431E+06, 1.742273E+06, 1.764321E+06, 1.786577E+06, 
   1.809040E+06, 1.831713E+06, 1.854596E+06, 1.877690E+06, 1.900996E+06, 1.924516E+06, 
   1.948250E+06, 1.972199E+06, 1.996365E+06, 2.020748E+06, 2.045349E+06, 2.070170E+06, 
   2.095211E+06, 2.120474E+06, 2.145958E+06, 2.171666E+06, 2.197599E+06, 2.223756E+06, 
   2.250140E+06, 2.276751E+06, 2.303590E+06, 2.330658E+06, 2.357956E+06, 2.385485E+06, 
   2.413246E+06, 2.441240E+06, 2.469467E+06, 2.497930E+06, 2.526627E+06, 2.555562E+06, 
   2.584734E+06, 2.614144E+06, 2.643794E+06, 2.673684E+06, 2.703815E+06, 2.734187E+06, 
   2.764803E+06, 2.795662E+06, 2.826766E+06, 2.858116E+06, 2.889712E+06, 2.921555E+06, 
   2.953646E+06, 2.985985E+06, 3.018575E+06, 3.051415E+06, 3.084507E+06, 3.117851E+06, 
   3.151448E+06, 3.185298E+06, 3.219404E+06, 3.253765E+06, 3.288382E+06, 3.323256E+06, 
   3.358388E+06, 3.393779E+06, 3.429429E+06, 3.465340E+06, 3.501511E+06, 3.537944E+06, 
   3.574640E+06, 3.611599E+06, 3.648822E+06, 3.686309E+06, 3.724062E+06, 3.762081E+06, 
   3.800367E+06, 3.838921E+06, 3.877743E+06, 3.916834E+06, 3.956195E+06, 3.995827E+06, 
   4.035729E+06, 4.075903E+06, 4.116350E+06, 4.157070E+06, 4.198063E+06, 4.239332E+06, 
   4.280875E+06, 4.322694E+06, 4.364789E+06, 4.407162E+06, 4.449812E+06, 4.492740E+06, 
   4.535948E+06, 4.579435E+06, 4.623202E+06, 4.667250E+06, 4.711579E+06, 4.756190E+06, 
   4.801083E+06, 4.846260E+06, 4.891720E+06, 4.937465E+06, 4.983494E+06, 5.029809E+06, 
   5.076409E+06, 5.123296E+06, 5.170470E+06, 5.217931E+06, 5.265680E+06, 5.313718E+06, 
   5.362045E+06, 5.410661E+06, 5.459568E+06, 5.508764E+06, 5.558252E+06, 5.608032E+06, 
   5.658103E+06, 5.708467E+06, 5.759124E+06, 5.810074E+06, 5.861317E+06, 5.912856E+06, 
   5.964688E+06, 6.016816E+06, 6.069240E+06, 6.121959E+06, 6.174975E+06, 6.228288E+06, 
   6.281898E+06, 6.335805E+06, 6.390011E+06, 6.444514E+06, 6.499317E+06, 6.554419E+06, 
   6.609820E+06, 6.665521E+06, 6.721523E+06, 6.777825E+06, 6.834428E+06, 6.891332E+06, 
   6.948538E+06, 7.006046E+06, 7.063856E+06, 7.121968E+06, 7.180384E+06, 7.239103E+06, 
   7.298125E+06, 7.357451E+06, 7.417081E+06, 7.477016E+06, 7.537255E+06, 7.597799E+06, 
   7.658648E+06, 7.719803E+06, 7.781264E+06, 7.843030E+06, 7.905103E+06, 7.967482E+06, 
   8.030168E+06, 8.093160E+06, 8.156460E+06, 8.220067E+06, 8.283982E+06, 8.348205E+06, 
   8.412735E+06, 8.477574E+06, 8.542721E+06, 8.608177E+06, 8.673941E+06, 8.740014E+06, 
   8.806397E+06, 8.873088E+06, 8.940089E+06, 9.007400E+06, 9.075020E+06, 9.142950E+06, 
   9.211190E+06, 9.279740E+06, 9.348600E+06, 9.417771E+06, 9.487252E+06, 9.557044E+06, 
   9.627147E+06, 9.697560E+06, 9.768284E+06, 9.839320E+06, 9.910666E+06, 9.982323E+06, 
   1.005429E+07, 1.012657E+07, 1.019916E+07, 1.027207E+07, 1.034528E+07, 1.041881E+07, 
   1.049264E+07, 1.056679E+07, 1.064125E+07, 1.071603E+07, 1.079111E+07, 1.086651E+07, 
   1.094222E+07, 1.101824E+07, 1.109457E+07, 1.117121E+07, 1.124817E+07, 1.132543E+07, 
   1.140301E+07, 1.148090E+07, 1.155911E+07, 1.163762E+07, 1.171645E+07, 1.179558E+07, 
   1.187503E+07, 1.195479E+07, 1.203487E+07, 1.211525E+07, 1.219595E+07, 1.227695E+07, 
   1.235827E+07, 1.243990E+07, 1.252184E+07, 1.260410E+07, 1.268666E+07, 1.276953E+07, 
   1.285272E+07, 1.293622E+07, 1.302002E+07, 1.310414E+07, 1.318857E+07, 1.327331E+07, 
   1.335836E+07, 1.344372E+07, 1.352938E+07, 1.361536E+07, 1.370165E+07, 1.378825E+07, 
   1.387516E+07, 1.396238E+07, 1.404990E+07, 1.413774E+07, 1.422589E+07, 1.431434E+07, 
   1.440310E+07, 1.449217E+07, 1.458155E+07, 1.467124E+07, 1.476124E+07, 1.485154E+07, 
   1.494215E+07, 1.503307E+07, 1.512430E+07, 1.521583E+07, 1.530767E+07, 1.539982E+07, 
   1.549227E+07, 1.558503E+07, 1.567809E+07, 1.577146E+07, 1.586514E+07, 1.595912E+07, 
   1.605341E+07, 
])
# ============================ M = 11, I = 2 ============================
M = 11
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.689360E+00, 1.086509E+01, 2.371743E+01, 4.011229E+01, 5.973090E+01, 8.216524E+01, 
   1.070476E+02, 1.341140E+02, 1.631783E+02, 1.941024E+02, 2.267778E+02, 2.611155E+02, 
   2.970405E+02, 3.344889E+02, 3.734061E+02, 4.137458E+02, 4.554693E+02, 4.985450E+02, 
   5.429478E+02, 5.886589E+02, 6.356653E+02, 6.839597E+02, 7.335395E+02, 7.844071E+02, 
   8.365694E+02, 8.900371E+02, 9.448250E+02, 1.000951E+03, 1.058437E+03, 1.117307E+03, 
   1.177588E+03, 1.239310E+03, 1.302505E+03, 1.367207E+03, 1.433451E+03, 1.501276E+03, 
   1.570721E+03, 1.641826E+03, 1.714634E+03, 1.789187E+03, 1.865531E+03, 1.943709E+03, 
   2.023770E+03, 2.105759E+03, 2.189723E+03, 2.275713E+03, 2.363775E+03, 2.453961E+03, 
   2.546320E+03, 2.640902E+03, 2.737759E+03, 2.836942E+03, 2.938503E+03, 3.042495E+03, 
   3.148970E+03, 3.257982E+03, 3.369585E+03, 3.483832E+03, 3.600778E+03, 3.720478E+03, 
   3.842989E+03, 3.968365E+03, 4.096664E+03, 4.227942E+03, 4.362256E+03, 4.499666E+03, 
   4.640230E+03, 4.784006E+03, 4.931055E+03, 5.081438E+03, 5.235214E+03, 5.392447E+03, 
   5.553197E+03, 5.717529E+03, 5.885507E+03, 6.057193E+03, 6.232655E+03, 6.411957E+03, 
   6.595167E+03, 6.782352E+03, 6.973580E+03, 7.168921E+03, 7.368444E+03, 7.572221E+03, 
   7.780322E+03, 7.992822E+03, 8.209792E+03, 8.431307E+03, 8.657444E+03, 8.888277E+03, 
   9.123885E+03, 9.364345E+03, 9.609736E+03, 9.860139E+03, 1.011563E+04, 1.037630E+04, 
   1.064223E+04, 1.091350E+04, 1.119020E+04, 1.147241E+04, 1.176022E+04, 1.205372E+04, 
   1.235300E+04, 1.265815E+04, 1.296927E+04, 1.328643E+04, 1.360974E+04, 1.393930E+04, 
   1.427520E+04, 1.461753E+04, 1.496640E+04, 1.532191E+04, 1.568415E+04, 1.605323E+04, 
   1.642926E+04, 1.681233E+04, 1.720255E+04, 1.760004E+04, 1.800489E+04, 1.841722E+04, 
   1.883715E+04, 1.926477E+04, 1.970021E+04, 2.014358E+04, 2.059500E+04, 2.105458E+04, 
   2.152244E+04, 2.199871E+04, 2.248350E+04, 2.297694E+04, 2.347915E+04, 2.399026E+04, 
   2.451040E+04, 2.503969E+04, 2.557826E+04, 2.612624E+04, 2.668378E+04, 2.725099E+04, 
   2.782802E+04, 2.841500E+04, 2.901208E+04, 2.961939E+04, 3.023707E+04, 3.086527E+04, 
   3.150413E+04, 3.215380E+04, 3.281442E+04, 3.348614E+04, 3.416912E+04, 3.486351E+04, 
   3.556945E+04, 3.628711E+04, 3.701663E+04, 3.775819E+04, 3.851194E+04, 3.927803E+04, 
   4.005664E+04, 4.084792E+04, 4.165205E+04, 4.246919E+04, 4.329951E+04, 4.414318E+04, 
   4.500037E+04, 4.587126E+04, 4.675603E+04, 4.765484E+04, 4.856789E+04, 4.949534E+04, 
   5.043739E+04, 5.139421E+04, 5.236600E+04, 5.335293E+04, 5.435520E+04, 5.537300E+04, 
   5.640652E+04, 5.745596E+04, 5.852151E+04, 5.960336E+04, 6.070172E+04, 6.181679E+04, 
   6.294876E+04, 6.409785E+04, 6.526426E+04, 6.644820E+04, 6.764987E+04, 6.886949E+04, 
   7.010727E+04, 7.136341E+04, 7.263815E+04, 7.393170E+04, 7.524427E+04, 7.657608E+04, 
   7.792737E+04, 7.929835E+04, 8.068925E+04, 8.210029E+04, 8.353171E+04, 8.498374E+04, 
   8.645661E+04, 8.795055E+04, 8.946581E+04, 9.100261E+04, 9.256120E+04, 9.414182E+04, 
   9.574471E+04, 9.737012E+04, 9.901829E+04, 1.006895E+05, 1.023839E+05, 1.041019E+05, 
   1.058436E+05, 1.076093E+05, 1.093993E+05, 1.112138E+05, 1.130531E+05, 1.149175E+05, 
   1.168072E+05, 1.187224E+05, 1.206635E+05, 1.226307E+05, 1.246243E+05, 1.266445E+05, 
   1.286917E+05, 1.307660E+05, 1.328678E+05, 1.349973E+05, 1.371549E+05, 1.393407E+05, 
   1.415551E+05, 1.437984E+05, 1.460708E+05, 1.483727E+05, 1.507042E+05, 1.530658E+05, 
   1.554576E+05, 1.578800E+05, 1.603333E+05, 1.628178E+05, 1.653337E+05, 1.678813E+05, 
   1.704610E+05, 1.730731E+05, 1.757178E+05, 1.783954E+05, 1.811063E+05, 1.838507E+05, 
   1.866290E+05, 1.894414E+05, 1.922883E+05, 1.951700E+05, 1.980868E+05, 2.010389E+05, 
   2.040268E+05, 2.070506E+05, 2.101108E+05, 2.132076E+05, 2.163414E+05, 2.195124E+05, 
   2.227210E+05, 2.259676E+05, 2.292523E+05, 2.325756E+05, 2.359378E+05, 2.393391E+05, 
   2.427799E+05, 2.462606E+05, 2.497814E+05, 2.533427E+05, 2.569448E+05, 2.605880E+05, 
   2.642727E+05, 2.679991E+05, 2.717677E+05, 2.755787E+05, 2.794324E+05, 2.833293E+05, 
   2.872695E+05, 2.912536E+05, 2.952817E+05, 2.993542E+05, 3.034715E+05, 3.076339E+05, 
   3.118417E+05, 3.160952E+05, 3.203948E+05, 3.247409E+05, 3.291337E+05, 3.335735E+05, 
   3.380608E+05, 3.425959E+05, 3.471791E+05, 3.518107E+05, 3.564910E+05, 3.612205E+05, 
   3.659994E+05, 3.708281E+05, 3.757069E+05, 3.806361E+05, 3.856162E+05, 3.906474E+05, 
   3.957300E+05, 4.008644E+05, 4.060510E+05, 4.112900E+05, 4.165819E+05, 4.219269E+05, 
   4.273253E+05, 4.327776E+05, 4.382841E+05, 4.438450E+05, 4.494608E+05, 4.551317E+05, 
   4.608582E+05, 4.666405E+05, 4.724789E+05, 4.783738E+05, 4.843256E+05, 4.903346E+05, 
   4.964010E+05, 5.025253E+05, 5.087078E+05, 5.149488E+05, 5.212486E+05, 5.276075E+05, 
   5.340260E+05, 5.405043E+05, 5.470427E+05, 5.536416E+05, 5.603014E+05, 5.670222E+05, 
   5.738046E+05, 5.806487E+05, 5.875549E+05, 5.945236E+05, 6.015551E+05, 6.086496E+05, 
   6.158075E+05, 6.230292E+05, 6.303150E+05, 6.376651E+05, 6.450799E+05, 6.525597E+05, 
   6.601048E+05, 6.677156E+05, 6.753923E+05, 6.831353E+05, 6.909449E+05, 6.988214E+05, 
   7.067651E+05, 7.147763E+05, 7.228554E+05, 7.310026E+05, 7.392182E+05, 7.475026E+05, 
   7.558561E+05, 7.642789E+05, 7.727714E+05, 7.813338E+05, 7.899665E+05, 7.986698E+05, 
   8.074439E+05, 8.162892E+05, 8.252060E+05, 8.341945E+05, 8.432551E+05, 8.523879E+05, 
   8.615934E+05, 8.708719E+05, 8.802235E+05, 8.896486E+05, 8.991475E+05, 9.087204E+05, 
   9.183677E+05, 9.280896E+05, 9.378864E+05, 9.477584E+05, 9.577058E+05, 9.677290E+05, 
   9.778282E+05, 9.880036E+05, 9.982556E+05, 1.008584E+06, 1.018990E+06, 1.029474E+06, 
   1.040034E+06, 1.050673E+06, 1.061390E+06, 1.072185E+06, 1.083059E+06, 1.094012E+06, 
   1.105044E+06, 1.116155E+06, 1.127346E+06, 1.138618E+06, 1.149969E+06, 1.161400E+06, 
   1.172913E+06, 1.184506E+06, 1.196180E+06, 1.207936E+06, 1.219773E+06, 1.231692E+06, 
   1.243694E+06, 1.255777E+06, 1.267944E+06, 1.280193E+06, 1.292525E+06, 1.304940E+06, 
   1.317439E+06, 1.330021E+06, 1.342688E+06, 1.355438E+06, 1.368273E+06, 1.381193E+06, 
   1.394197E+06, 1.407286E+06, 1.420461E+06, 1.433721E+06, 1.447066E+06, 1.460498E+06, 
   1.474015E+06, 1.487619E+06, 1.501309E+06, 1.515086E+06, 1.528949E+06, 1.542900E+06, 
   1.556938E+06, 1.571063E+06, 1.585276E+06, 1.599576E+06, 1.613965E+06, 1.628442E+06, 
   1.643007E+06, 1.657660E+06, 1.672403E+06, 1.687234E+06, 1.702154E+06, 1.717163E+06, 
   1.732262E+06, 1.747451E+06, 1.762729E+06, 1.778097E+06, 1.793555E+06, 1.809103E+06, 
   1.824742E+06, 1.840471E+06, 1.856291E+06, 1.872201E+06, 1.888203E+06, 1.904296E+06, 
   1.920480E+06, 1.936756E+06, 1.953123E+06, 1.969581E+06, 1.986132E+06, 2.002775E+06, 
   2.019510E+06, 2.036337E+06, 2.053256E+06, 2.070268E+06, 2.087373E+06, 2.104571E+06, 
   2.121861E+06, 2.139244E+06, 2.156721E+06, 2.174291E+06, 2.191954E+06, 2.209711E+06, 
   2.227562E+06, 2.245506E+06, 2.263544E+06, 2.281676E+06, 2.299902E+06, 2.318222E+06, 
   2.336637E+06, 2.355146E+06, 2.373749E+06, 2.392447E+06, 2.411240E+06, 2.430127E+06, 
   2.449110E+06, 2.468187E+06, 2.487359E+06, 2.506627E+06, 2.525989E+06, 2.545447E+06, 
   2.565001E+06, 2.584649E+06, 2.604394E+06, 2.624234E+06, 2.644169E+06, 2.664201E+06, 
   2.684328E+06, 2.704551E+06, 2.724871E+06, 2.745286E+06, 2.765797E+06, 2.786405E+06, 
   2.807108E+06, 2.827908E+06, 2.848805E+06, 2.869797E+06, 2.890886E+06, 2.912072E+06, 
   2.933354E+06, 2.954733E+06, 2.976209E+06, 2.997781E+06, 3.019450E+06, 3.041216E+06, 
   3.063078E+06, 3.085038E+06, 3.107094E+06, 3.129247E+06, 3.151498E+06, 3.173845E+06, 
   3.196289E+06, 3.218831E+06, 3.241469E+06, 3.264205E+06, 3.287037E+06, 3.309967E+06, 
   3.332994E+06, 3.356118E+06, 3.379340E+06, 3.402659E+06, 3.426074E+06, 3.449588E+06, 
   3.473198E+06, 3.496906E+06, 3.520711E+06, 3.544614E+06, 3.568613E+06, 3.592710E+06, 
   3.616905E+06, 3.641196E+06, 3.665585E+06, 3.690071E+06, 3.714655E+06, 3.739336E+06, 
   3.764114E+06, 3.788990E+06, 3.813962E+06, 3.839032E+06, 3.864200E+06, 3.889464E+06, 
   3.914826E+06, 3.940285E+06, 3.965841E+06, 3.991494E+06, 4.017245E+06, 4.043093E+06, 
   4.069037E+06, 4.095079E+06, 4.121218E+06, 4.147454E+06, 4.173787E+06, 4.200217E+06, 
   4.226743E+06, 4.253367E+06, 4.280088E+06, 4.306905E+06, 4.333819E+06, 4.360830E+06, 
   4.387938E+06, 4.415142E+06, 4.442443E+06, 4.469841E+06, 4.497335E+06, 4.524926E+06, 
   4.552613E+06, 4.580396E+06, 4.608276E+06, 4.636252E+06, 4.664325E+06, 4.692493E+06, 
   4.720758E+06, 4.749119E+06, 4.777576E+06, 4.806129E+06, 4.834778E+06, 4.863522E+06, 
   4.892363E+06, 4.921299E+06, 4.950330E+06, 4.979458E+06, 5.008681E+06, 5.037999E+06, 
   5.067413E+06, 5.096922E+06, 5.126526E+06, 5.156226E+06, 5.186021E+06, 5.215910E+06, 
   5.245895E+06, 5.275975E+06, 5.306149E+06, 5.336418E+06, 5.366782E+06, 5.397240E+06, 
   5.427793E+06, 
])
# ============================ M = 12, I = 1 ============================
M = 12
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.573524E+01, 1.029251E+03, 2.896824E+03, 5.313431E+03, 8.174539E+03, 1.141970E+04, 
   1.500830E+04, 1.891136E+04, 2.310835E+04, 2.758623E+04, 3.233901E+04, 3.736750E+04, 
   4.267884E+04, 4.828605E+04, 5.420733E+04, 6.046551E+04, 6.708747E+04, 7.410380E+04, 
   8.154826E+04, 8.945771E+04, 9.787188E+04, 1.068332E+05, 1.163871E+05, 1.265816E+05, 
   1.374678E+05, 1.490998E+05, 1.615349E+05, 1.748339E+05, 1.890610E+05, 2.042845E+05, 
   2.205764E+05, 2.380133E+05, 2.566761E+05, 2.766508E+05, 2.980283E+05, 3.209051E+05, 
   3.453830E+05, 3.715702E+05, 3.995815E+05, 4.295378E+05, 4.615675E+05, 4.958063E+05, 
   5.323980E+05, 5.714943E+05, 6.132560E+05, 6.578525E+05, 7.054636E+05, 7.562786E+05, 
   8.104973E+05, 8.683312E+05, 9.300032E+05, 9.957481E+05, 1.065814E+06, 1.140462E+06, 
   1.219969E+06, 1.304624E+06, 1.394732E+06, 1.490616E+06, 1.592616E+06, 1.701086E+06, 
   1.816403E+06, 1.938961E+06, 2.069175E+06, 2.207481E+06, 2.354336E+06, 2.510222E+06, 
   2.675644E+06, 2.851134E+06, 3.037246E+06, 3.234567E+06, 3.443706E+06, 3.665308E+06, 
   3.900048E+06, 4.148628E+06, 4.411790E+06, 4.690310E+06, 4.984997E+06, 5.296703E+06, 
   5.626315E+06, 5.974766E+06, 6.343026E+06, 6.732116E+06, 7.143101E+06, 7.577092E+06, 
   8.035255E+06, 8.518805E+06, 9.029012E+06, 9.567200E+06, 1.013476E+07, 1.073313E+07, 
   1.136382E+07, 1.202841E+07, 1.272855E+07, 1.346593E+07, 1.424236E+07, 1.505969E+07, 
   1.591986E+07, 1.682490E+07, 1.777692E+07, 1.877811E+07, 1.983076E+07, 2.093726E+07, 
   2.210007E+07, 2.332178E+07, 2.460508E+07, 2.595275E+07, 2.736772E+07, 2.885299E+07, 
   3.041171E+07, 3.204715E+07, 3.376270E+07, 3.556189E+07, 3.744839E+07, 3.942602E+07, 
   4.149872E+07, 4.367060E+07, 4.594593E+07, 4.832914E+07, 5.082484E+07, 5.343777E+07, 
   5.617291E+07, 5.903537E+07, 6.203049E+07, 6.516378E+07, 6.844098E+07, 7.186803E+07, 
   7.545107E+07, 7.919648E+07, 8.311086E+07, 8.720108E+07, 9.147421E+07, 9.593761E+07, 
   1.005989E+08, 1.054659E+08, 1.105468E+08, 1.158500E+08, 1.213843E+08, 1.271587E+08, 
   1.331825E+08, 1.394655E+08, 1.460176E+08, 1.528491E+08, 1.599709E+08, 1.673938E+08, 
   1.751294E+08, 1.831894E+08, 1.915862E+08, 2.003323E+08, 2.094406E+08, 2.189248E+08, 
   2.287987E+08, 2.390766E+08, 2.497734E+08, 2.609045E+08, 2.724856E+08, 2.845330E+08, 
   2.970635E+08, 3.100947E+08, 3.236443E+08, 3.377309E+08, 3.523735E+08, 3.675919E+08, 
   3.834064E+08, 3.998378E+08, 4.169078E+08, 4.346385E+08, 4.530531E+08, 4.721750E+08, 
   4.920286E+08, 5.126390E+08, 5.340321E+08, 5.562347E+08, 5.792739E+08, 6.031782E+08, 
   6.279767E+08, 6.536994E+08, 6.803770E+08, 7.080413E+08, 7.367252E+08, 7.664622E+08, 
   7.972871E+08, 8.292355E+08, 8.623440E+08, 8.966504E+08, 9.321937E+08, 9.690135E+08, 
   1.007151E+09, 1.046649E+09, 1.087551E+09, 1.129900E+09, 1.173744E+09, 1.219129E+09, 
   1.266104E+09, 1.314719E+09, 1.365025E+09, 1.417075E+09, 1.470923E+09, 1.526624E+09, 
   1.584237E+09, 1.643818E+09, 1.705430E+09, 1.769134E+09, 1.834993E+09, 1.903072E+09, 
   1.973439E+09, 2.046164E+09, 2.121315E+09, 2.198967E+09, 2.279192E+09, 2.362069E+09, 
   2.447675E+09, 2.536091E+09, 2.627399E+09, 2.721685E+09, 2.819035E+09, 2.919538E+09, 
   3.023286E+09, 3.130372E+09, 3.240894E+09, 3.354949E+09, 3.472638E+09, 3.594066E+09, 
   3.719338E+09, 3.848565E+09, 3.981857E+09, 4.119329E+09, 4.261099E+09, 4.407286E+09, 
   4.558015E+09, 4.713411E+09, 4.873603E+09, 5.038726E+09, 5.208913E+09, 5.384305E+09, 
   5.565043E+09, 5.751275E+09, 5.943148E+09, 6.140818E+09, 6.344438E+09, 6.554173E+09, 
   6.770183E+09, 6.992639E+09, 7.221711E+09, 7.457578E+09, 7.700418E+09, 7.950417E+09, 
   8.207763E+09, 8.472650E+09, 8.745275E+09, 9.025842E+09, 9.314557E+09, 9.611632E+09, 
   9.917283E+09, 1.023173E+10, 1.055521E+10, 1.088794E+10, 1.123017E+10, 1.158213E+10, 
   1.194408E+10, 1.231627E+10, 1.269895E+10, 1.309240E+10, 1.349688E+10, 1.391267E+10, 
   1.434006E+10, 1.477933E+10, 1.523078E+10, 1.569471E+10, 1.617143E+10, 1.666125E+10, 
   1.716451E+10, 1.768152E+10, 1.821263E+10, 1.875818E+10, 1.931852E+10, 1.989400E+10, 
   2.048501E+10, 2.109190E+10, 2.171506E+10, 2.235490E+10, 2.301179E+10, 2.368615E+10, 
   2.437840E+10, 2.508896E+10, 2.581826E+10, 2.656675E+10, 2.733487E+10, 2.812310E+10, 
   2.893189E+10, 2.976174E+10, 3.061313E+10, 3.148656E+10, 3.238254E+10, 3.330159E+10, 
   3.424425E+10, 3.521108E+10, 3.620258E+10, 3.721936E+10, 3.826198E+10, 3.933104E+10, 
   4.042712E+10, 4.155085E+10, 4.270285E+10, 4.388375E+10, 4.509420E+10, 4.633487E+10, 
   4.760643E+10, 4.890956E+10, 5.024498E+10, 5.161339E+10, 5.301553E+10, 5.445214E+10, 
   5.592397E+10, 5.743180E+10, 5.897644E+10, 6.055866E+10, 6.217929E+10, 6.383916E+10, 
   6.553915E+10, 6.728008E+10, 6.906289E+10, 7.088842E+10, 7.275761E+10, 7.467143E+10, 
   7.663079E+10, 7.863667E+10, 8.069007E+10, 8.279197E+10, 8.494343E+10, 8.714550E+10, 
   8.939920E+10, 9.170566E+10, 9.406595E+10, 9.648122E+10, 9.895261E+10, 1.014813E+11, 
   1.040684E+11, 1.067153E+11, 1.094230E+11, 1.121929E+11, 1.150262E+11, 1.179243E+11, 
   1.208886E+11, 1.239201E+11, 1.270205E+11, 1.301911E+11, 1.334333E+11, 1.367486E+11, 
   1.401384E+11, 1.436042E+11, 1.471476E+11, 
])
# ============================ M = 12, I = 2 ============================
M = 12
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.382603E+01, 6.862459E+02, 1.931437E+03, 3.542694E+03, 5.450319E+03, 7.614008E+03, 
   1.000669E+04, 1.260902E+04, 1.540736E+04, 1.839301E+04, 2.156205E+04, 2.491514E+04, 
   2.845720E+04, 3.219714E+04, 3.614739E+04, 4.032349E+04, 4.474377E+04, 4.942902E+04, 
   5.440219E+04, 5.968834E+04, 6.531440E+04, 7.130923E+04, 7.770355E+04, 8.452997E+04, 
   9.182310E+04, 9.961964E+04, 1.079584E+05, 1.168806E+05, 1.264298E+05, 1.366522E+05, 
   1.475967E+05, 1.593152E+05, 1.718627E+05, 1.852975E+05, 1.996812E+05, 2.150794E+05, 
   2.315613E+05, 2.492004E+05, 2.680746E+05, 2.882662E+05, 3.098624E+05, 3.329557E+05, 
   3.576437E+05, 3.840298E+05, 4.122233E+05, 4.423398E+05, 4.745014E+05, 5.088373E+05, 
   5.454834E+05, 5.845843E+05, 6.262912E+05, 6.707649E+05, 7.181744E+05, 7.686977E+05, 
   8.225232E+05, 8.798486E+05, 9.408828E+05, 1.005845E+06, 1.074968E+06, 1.148494E+06, 
   1.226680E+06, 1.309794E+06, 1.398121E+06, 1.491959E+06, 1.591620E+06, 1.697433E+06, 
   1.809745E+06, 1.928917E+06, 2.055330E+06, 2.189385E+06, 2.331499E+06, 2.482113E+06, 
   2.641689E+06, 2.810708E+06, 2.989677E+06, 3.179128E+06, 3.379614E+06, 3.591720E+06, 
   3.816053E+06, 4.053252E+06, 4.303982E+06, 4.568943E+06, 4.848864E+06, 5.144506E+06, 
   5.456671E+06, 5.786187E+06, 6.133931E+06, 6.500808E+06, 6.887769E+06, 7.295806E+06, 
   7.725955E+06, 8.179296E+06, 8.656957E+06, 9.160112E+06, 9.689991E+06, 1.024787E+07, 
   1.083508E+07, 1.145301E+07, 1.210311E+07, 1.278689E+07, 1.350592E+07, 1.426184E+07, 
   1.505634E+07, 1.589120E+07, 1.676827E+07, 1.768946E+07, 1.865678E+07, 1.967230E+07, 
   2.073817E+07, 2.185666E+07, 2.303008E+07, 2.426086E+07, 2.555154E+07, 2.690472E+07, 
   2.832314E+07, 2.980961E+07, 3.136707E+07, 3.299857E+07, 3.470727E+07, 3.649645E+07, 
   3.836952E+07, 4.033000E+07, 4.238158E+07, 4.452803E+07, 4.677333E+07, 4.912153E+07, 
   5.157688E+07, 5.414379E+07, 5.682677E+07, 5.963057E+07, 6.256005E+07, 6.562028E+07, 
   6.881650E+07, 7.215413E+07, 7.563878E+07, 7.927627E+07, 8.307263E+07, 8.703406E+07, 
   9.116702E+07, 9.547814E+07, 9.997437E+07, 1.046628E+08, 1.095508E+08, 1.146460E+08, 
   1.199563E+08, 1.254898E+08, 1.312550E+08, 1.372606E+08, 1.435154E+08, 1.500289E+08, 
   1.568106E+08, 1.638704E+08, 1.712185E+08, 1.788656E+08, 1.868225E+08, 1.951004E+08, 
   2.037110E+08, 2.126663E+08, 2.219786E+08, 2.316607E+08, 2.417258E+08, 2.521874E+08, 
   2.630596E+08, 2.743568E+08, 2.860939E+08, 2.982862E+08, 3.109496E+08, 3.241005E+08, 
   3.377555E+08, 3.519320E+08, 3.666480E+08, 3.819217E+08, 3.977722E+08, 4.142190E+08, 
   4.312820E+08, 4.489822E+08, 4.673407E+08, 4.863796E+08, 5.061213E+08, 5.265891E+08, 
   5.478072E+08, 5.697999E+08, 5.925928E+08, 6.162116E+08, 6.406837E+08, 6.660362E+08, 
   6.922978E+08, 7.194975E+08, 7.476655E+08, 7.768325E+08, 8.070303E+08, 8.382917E+08, 
   8.706501E+08, 9.041400E+08, 9.387968E+08, 9.746571E+08, 1.011758E+09, 1.050139E+09, 
   1.089838E+09, 1.130896E+09, 1.173356E+09, 1.217260E+09, 1.262651E+09, 1.309576E+09, 
   1.358080E+09, 1.408211E+09, 1.460018E+09, 1.513551E+09, 1.568862E+09, 1.626003E+09, 
   1.685029E+09, 1.745995E+09, 1.808960E+09, 1.873980E+09, 1.941117E+09, 2.010431E+09, 
   2.081987E+09, 2.155849E+09, 2.232084E+09, 2.310760E+09, 2.391947E+09, 2.475716E+09, 
   2.562142E+09, 2.651301E+09, 2.743268E+09, 2.838123E+09, 2.935948E+09, 3.036826E+09, 
   3.140842E+09, 3.248084E+09, 3.358640E+09, 3.472604E+09, 3.590069E+09, 3.711130E+09, 
   3.835888E+09, 3.964442E+09, 4.096897E+09, 4.233359E+09, 4.373935E+09, 4.518736E+09, 
   4.667879E+09, 4.821476E+09, 4.979648E+09, 5.142519E+09, 5.310211E+09, 5.482853E+09, 
   5.660576E+09, 5.843514E+09, 6.031804E+09, 6.225585E+09, 6.425001E+09, 6.630200E+09, 
   6.841331E+09, 7.058547E+09, 7.282006E+09, 7.511867E+09, 7.748297E+09, 7.991463E+09, 
   8.241535E+09, 8.498690E+09, 8.763108E+09, 9.034973E+09, 9.314471E+09, 9.601794E+09, 
   9.897139E+09, 1.020071E+10, 1.051270E+10, 1.083334E+10, 1.116282E+10, 1.150137E+10, 
   1.184921E+10, 1.220658E+10, 1.257370E+10, 1.295082E+10, 1.333817E+10, 1.373600E+10, 
   1.414458E+10, 1.456415E+10, 1.499499E+10, 1.543736E+10, 1.589154E+10, 1.635782E+10, 
   1.683648E+10, 1.732781E+10, 1.783212E+10, 1.834971E+10, 1.888090E+10, 1.942600E+10, 
   1.998535E+10, 2.055927E+10, 2.114810E+10, 2.175220E+10, 2.237192E+10, 2.300761E+10, 
   2.365965E+10, 2.432841E+10, 2.501428E+10, 2.571764E+10, 2.643891E+10, 2.717847E+10, 
   2.793676E+10, 2.871419E+10, 2.951121E+10, 3.032824E+10, 3.116574E+10, 3.202417E+10, 
   3.290399E+10, 3.380569E+10, 3.472976E+10, 3.567667E+10, 3.664695E+10, 3.764111E+10, 
   3.865967E+10, 3.970318E+10, 4.077217E+10, 4.186721E+10, 4.298886E+10, 4.413769E+10, 
   4.531432E+10, 4.651932E+10, 4.775333E+10, 4.901696E+10, 5.031083E+10, 5.163562E+10, 
   5.299197E+10, 5.438056E+10, 5.580208E+10, 5.725721E+10, 5.874669E+10, 6.027123E+10, 
   6.183156E+10, 6.342845E+10, 6.506265E+10, 6.673496E+10, 6.844617E+10, 7.019706E+10, 
   7.198850E+10, 7.382129E+10, 7.569632E+10, 7.761444E+10, 7.957652E+10, 8.158351E+10, 
   8.363629E+10, 8.573579E+10, 8.788300E+10, 9.007886E+10, 9.232438E+10, 9.462054E+10, 
   9.696836E+10, 9.936892E+10, 1.018233E+11, 
])
# ============================ M = 13, I = 1 ============================
M = 13
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.538496E+01, 1.593640E+01, 1.602673E+01, 1.643215E+01, 1.728778E+01, 1.852414E+01, 
   2.005293E+01, 2.180752E+01, 2.374009E+01, 2.581568E+01, 2.800797E+01, 3.029677E+01, 
   3.266623E+01, 3.510382E+01, 3.759941E+01, 4.014480E+01, 4.273326E+01, 4.535920E+01, 
   4.801797E+01, 5.070565E+01, 5.341894E+01, 5.615500E+01, 5.891142E+01, 6.168612E+01, 
   6.447729E+01, 6.728336E+01, 7.010295E+01, 7.293488E+01, 7.577806E+01, 7.863156E+01, 
   8.149456E+01, 8.436630E+01, 8.724613E+01, 9.013344E+01, 9.302771E+01, 9.592846E+01, 
   9.883525E+01, 1.017477E+02, 1.046654E+02, 1.075882E+02, 1.105156E+02, 1.134474E+02, 
   1.163834E+02, 1.193234E+02, 1.222671E+02, 1.252144E+02, 1.281652E+02, 1.311192E+02, 
   1.340764E+02, 1.370366E+02, 1.399998E+02, 1.429658E+02, 1.459345E+02, 1.489059E+02, 
   1.518800E+02, 1.548567E+02, 1.578359E+02, 1.608176E+02, 1.638019E+02, 1.667886E+02, 
   1.697777E+02, 1.727694E+02, 1.757636E+02, 1.787602E+02, 1.817594E+02, 1.847611E+02, 
   1.877655E+02, 1.907724E+02, 1.937821E+02, 1.967944E+02, 1.998096E+02, 2.028276E+02, 
   2.058484E+02, 2.088723E+02, 2.118992E+02, 2.149292E+02, 2.179624E+02, 2.209990E+02, 
   2.240388E+02, 2.270821E+02, 2.301290E+02, 2.331795E+02, 2.362337E+02, 2.392917E+02, 
   2.423537E+02, 2.454197E+02, 2.484898E+02, 2.515641E+02, 2.546428E+02, 2.577258E+02, 
   2.608134E+02, 2.639057E+02, 2.670027E+02, 2.701046E+02, 2.732114E+02, 2.763232E+02, 
   2.794403E+02, 2.825626E+02, 2.856903E+02, 2.888235E+02, 2.919624E+02, 2.951069E+02, 
   2.982572E+02, 3.014135E+02, 3.045757E+02, 3.077441E+02, 3.109188E+02, 3.140997E+02, 
   3.172871E+02, 3.204811E+02, 3.236817E+02, 3.268890E+02, 3.301032E+02, 3.333243E+02, 
   3.365525E+02, 3.397878E+02, 3.430303E+02, 3.462801E+02, 3.495374E+02, 3.528022E+02, 
   3.560745E+02, 3.593546E+02, 3.626425E+02, 3.659382E+02, 3.692419E+02, 3.725537E+02, 
   3.758735E+02, 3.792016E+02, 3.825380E+02, 3.858828E+02, 3.892360E+02, 3.925978E+02, 
   3.959682E+02, 3.993474E+02, 4.027353E+02, 4.061320E+02, 4.095377E+02, 4.129524E+02, 
   4.163762E+02, 4.198091E+02, 4.232513E+02, 4.267027E+02, 4.301635E+02, 4.336337E+02, 
   4.371135E+02, 4.406028E+02, 4.441017E+02, 4.476103E+02, 4.511287E+02, 4.546569E+02, 
   4.581950E+02, 4.617431E+02, 4.653011E+02, 4.688692E+02, 4.724475E+02, 4.760359E+02, 
   4.796345E+02, 4.832435E+02, 4.868628E+02, 4.904925E+02, 4.941326E+02, 4.977833E+02, 
   5.014445E+02, 5.051164E+02, 5.087989E+02, 5.124921E+02, 5.161961E+02, 5.199109E+02, 
   5.236366E+02, 5.273732E+02, 5.311207E+02, 5.348793E+02, 5.386489E+02, 5.424296E+02, 
   5.462214E+02, 5.500245E+02, 5.538387E+02, 5.576643E+02, 5.615011E+02, 5.653493E+02, 
   5.692089E+02, 5.730799E+02, 5.769624E+02, 5.808564E+02, 5.847620E+02, 5.886792E+02, 
   5.926080E+02, 5.965485E+02, 6.005006E+02, 6.044646E+02, 6.084403E+02, 6.124278E+02, 
   6.164272E+02, 6.204385E+02, 6.244617E+02, 6.284968E+02, 6.325440E+02, 6.366032E+02, 
   6.406744E+02, 6.447577E+02, 6.488532E+02, 6.529608E+02, 6.570806E+02, 6.612126E+02, 
   6.653569E+02, 6.695134E+02, 6.736823E+02, 6.778635E+02, 6.820571E+02, 6.862631E+02, 
   6.904815E+02, 6.947124E+02, 6.989557E+02, 7.032116E+02, 7.074801E+02, 7.117611E+02, 
   7.160547E+02, 7.203609E+02, 7.246798E+02, 7.290114E+02, 7.333557E+02, 7.377128E+02, 
   7.420826E+02, 7.464652E+02, 7.508606E+02, 7.552689E+02, 7.596900E+02, 7.641240E+02, 
   7.685709E+02, 7.730308E+02, 7.775036E+02, 7.819895E+02, 7.864883E+02, 7.910002E+02, 
   7.955252E+02, 8.000632E+02, 8.046144E+02, 8.091787E+02, 8.137561E+02, 8.183468E+02, 
   8.229506E+02, 8.275677E+02, 8.321980E+02, 8.368416E+02, 8.414985E+02, 8.461687E+02, 
   8.508522E+02, 8.555491E+02, 8.602594E+02, 8.649831E+02, 8.697202E+02, 8.744708E+02, 
   8.792348E+02, 8.840123E+02, 8.888034E+02, 8.936079E+02, 8.984261E+02, 9.032578E+02, 
   9.081031E+02, 9.129620E+02, 9.178345E+02, 9.227207E+02, 9.276206E+02, 9.325342E+02, 
   9.374615E+02, 9.424025E+02, 9.473573E+02, 9.523259E+02, 9.573083E+02, 9.623044E+02, 
   9.673144E+02, 9.723383E+02, 9.773761E+02, 9.824277E+02, 9.874933E+02, 9.925728E+02, 
   9.976662E+02, 1.002774E+03, 1.007895E+03, 1.013031E+03, 1.018180E+03, 1.023343E+03, 
   1.028521E+03, 1.033713E+03, 1.038918E+03, 1.044138E+03, 1.049372E+03, 1.054621E+03, 
   1.059883E+03, 1.065160E+03, 1.070450E+03, 1.075755E+03, 1.081075E+03, 1.086408E+03, 
   1.091756E+03, 1.097118E+03, 1.102495E+03, 1.107885E+03, 1.113291E+03, 1.118710E+03, 
   1.124144E+03, 1.129593E+03, 1.135055E+03, 1.140533E+03, 1.146024E+03, 1.151531E+03, 
   1.157051E+03, 1.162587E+03, 1.168136E+03, 1.173701E+03, 1.179280E+03, 1.184873E+03, 
   1.190482E+03, 1.196104E+03, 1.201742E+03, 1.207394E+03, 1.213061E+03, 1.218742E+03, 
   1.224439E+03, 1.230150E+03, 1.235875E+03, 1.241616E+03, 1.247371E+03, 1.253141E+03, 
   1.258926E+03, 1.264726E+03, 1.270541E+03, 1.276370E+03, 1.282215E+03, 1.288074E+03, 
   1.293948E+03, 1.299838E+03, 1.305742E+03, 1.311661E+03, 1.317595E+03, 1.323545E+03, 
   1.329509E+03, 1.335488E+03, 1.341483E+03, 1.347492E+03, 1.353517E+03, 1.359557E+03, 
   1.365612E+03, 1.371682E+03, 1.377767E+03, 1.383868E+03, 1.389983E+03, 1.396114E+03, 
   1.402260E+03, 1.408422E+03, 1.414599E+03, 1.420791E+03, 1.426998E+03, 1.433221E+03, 
   1.439459E+03, 1.445713E+03, 1.451981E+03, 1.458266E+03, 1.464566E+03, 1.470881E+03, 
   1.477211E+03, 1.483558E+03, 1.489919E+03, 1.496296E+03, 1.502689E+03, 1.509098E+03, 
   1.515521E+03, 1.521961E+03, 1.528416E+03, 1.534887E+03, 1.541373E+03, 1.547875E+03, 
   1.554393E+03, 1.560926E+03, 1.567476E+03, 1.574040E+03, 1.580621E+03, 1.587218E+03, 
   1.593830E+03, 1.600458E+03, 1.607102E+03, 1.613762E+03, 1.620437E+03, 1.627129E+03, 
   1.633836E+03, 1.640559E+03, 1.647299E+03, 1.654054E+03, 1.660825E+03, 1.667613E+03, 
   1.674416E+03, 1.681235E+03, 1.688070E+03, 1.694922E+03, 1.701789E+03, 1.708673E+03, 
   1.715573E+03, 1.722489E+03, 1.729421E+03, 1.736369E+03, 1.743334E+03, 1.750314E+03, 
   1.757311E+03, 1.764325E+03, 1.771354E+03, 1.778400E+03, 1.785462E+03, 1.792541E+03, 
   1.799635E+03, 1.806747E+03, 1.813874E+03, 1.821018E+03, 1.828179E+03, 1.835356E+03, 
   1.842549E+03, 1.849759E+03, 1.856985E+03, 1.864228E+03, 1.871488E+03, 1.878764E+03, 
   1.886056E+03, 1.893366E+03, 1.900691E+03, 1.908034E+03, 1.915393E+03, 1.922769E+03, 
   1.930162E+03, 1.937571E+03, 1.944997E+03, 1.952440E+03, 1.959899E+03, 1.967376E+03, 
   1.974869E+03, 1.982379E+03, 1.989906E+03, 1.997450E+03, 2.005010E+03, 2.012588E+03, 
   2.020183E+03, 2.027794E+03, 2.035423E+03, 2.043068E+03, 2.050731E+03, 2.058410E+03, 
   2.066107E+03, 2.073821E+03, 2.081552E+03, 2.089300E+03, 2.097065E+03, 2.104847E+03, 
   2.112647E+03, 2.120463E+03, 2.128297E+03, 2.136148E+03, 2.144017E+03, 2.151902E+03, 
   2.159805E+03, 2.167726E+03, 2.175663E+03, 2.183618E+03, 2.191591E+03, 2.199580E+03, 
   2.207588E+03, 2.215612E+03, 2.223654E+03, 2.231714E+03, 2.239791E+03, 2.247886E+03, 
   2.255998E+03, 2.264128E+03, 2.272275E+03, 2.280440E+03, 2.288623E+03, 2.296823E+03, 
   2.305041E+03, 2.313277E+03, 2.321530E+03, 2.329801E+03, 2.338090E+03, 2.346396E+03, 
   2.354721E+03, 2.363063E+03, 2.371423E+03, 2.379801E+03, 2.388197E+03, 2.396611E+03, 
   2.405042E+03, 2.413492E+03, 2.421959E+03, 2.430445E+03, 2.438948E+03, 2.447470E+03, 
   2.456009E+03, 2.464567E+03, 2.473143E+03, 2.481737E+03, 2.490349E+03, 2.498979E+03, 
   2.507627E+03, 2.516293E+03, 2.524978E+03, 2.533681E+03, 2.542402E+03, 2.551142E+03, 
   2.559899E+03, 2.568675E+03, 2.577470E+03, 2.586283E+03, 2.595114E+03, 2.603963E+03, 
   2.612831E+03, 2.621718E+03, 2.630622E+03, 2.639546E+03, 2.648488E+03, 2.657448E+03, 
   2.666427E+03, 2.675424E+03, 2.684441E+03, 2.693475E+03, 2.702529E+03, 2.711601E+03, 
   2.720691E+03, 2.729801E+03, 2.738929E+03, 2.748075E+03, 2.757241E+03, 2.766425E+03, 
   2.775629E+03, 2.784850E+03, 2.794091E+03, 2.803351E+03, 2.812630E+03, 2.821927E+03, 
   2.831243E+03, 2.840579E+03, 2.849933E+03, 2.859306E+03, 2.868698E+03, 2.878110E+03, 
   2.887540E+03, 2.896990E+03, 2.906458E+03, 2.915946E+03, 2.925453E+03, 2.934978E+03, 
   2.944524E+03, 2.954088E+03, 2.963671E+03, 2.973274E+03, 2.982896E+03, 2.992537E+03, 
   3.002198E+03, 3.011878E+03, 3.021577E+03, 3.031296E+03, 3.041034E+03, 3.050791E+03, 
   3.060568E+03, 3.070364E+03, 3.080180E+03, 3.090015E+03, 3.099870E+03, 3.109744E+03, 
   3.119638E+03, 3.129551E+03, 3.139484E+03, 3.149437E+03, 3.159409E+03, 3.169401E+03, 
   3.179412E+03, 3.189443E+03, 3.199494E+03, 3.209565E+03, 3.219655E+03, 3.229765E+03, 
   3.239895E+03, 3.250045E+03, 3.260214E+03, 3.270404E+03, 3.280613E+03, 3.290842E+03, 
   3.301091E+03, 3.311360E+03, 3.321649E+03, 3.331958E+03, 3.342287E+03, 3.352636E+03, 
   3.363005E+03, 3.373394E+03, 3.383803E+03, 3.394232E+03, 3.404681E+03, 3.415150E+03, 
   3.425640E+03, 3.436150E+03, 3.446680E+03, 3.457230E+03, 3.467800E+03, 3.478390E+03, 
   3.489001E+03, 3.499632E+03, 3.510284E+03, 3.520955E+03, 3.531647E+03, 3.542360E+03, 
   3.553093E+03, 3.563846E+03, 3.574619E+03, 3.585413E+03, 3.596228E+03, 3.607063E+03, 
   3.617918E+03, 3.628794E+03, 3.639691E+03, 3.650608E+03, 3.661545E+03, 3.672503E+03, 
   3.683482E+03, 3.694482E+03, 3.705502E+03, 3.716542E+03, 3.727604E+03, 3.738686E+03, 
   3.749788E+03, 3.760912E+03, 3.772056E+03, 3.783221E+03, 3.794407E+03, 3.805614E+03, 
   3.816841E+03, 3.828089E+03, 3.839358E+03, 3.850648E+03, 3.861959E+03, 3.873291E+03, 
   3.884644E+03, 3.896017E+03, 3.907412E+03, 3.918828E+03, 3.930264E+03, 3.941722E+03, 
   3.953201E+03, 3.964700E+03, 3.976221E+03, 3.987763E+03, 3.999326E+03, 4.010910E+03, 
   4.022515E+03, 4.034141E+03, 4.045789E+03, 4.057458E+03, 4.069147E+03, 4.080858E+03, 
   4.092591E+03, 4.104344E+03, 4.116119E+03, 4.127915E+03, 4.139733E+03, 4.151572E+03, 
   4.163432E+03, 4.175313E+03, 4.187216E+03, 4.199140E+03, 4.211086E+03, 4.223052E+03, 
   4.235041E+03, 4.247051E+03, 4.259082E+03, 4.271135E+03, 4.283209E+03, 4.295305E+03, 
   4.307422E+03, 4.319561E+03, 4.331721E+03, 4.343903E+03, 4.356107E+03, 4.368332E+03, 
   4.380578E+03, 4.392847E+03, 4.405137E+03, 4.417448E+03, 4.429781E+03, 4.442136E+03, 
   4.454513E+03, 4.466911E+03, 4.479331E+03, 4.491773E+03, 4.504237E+03, 4.516722E+03, 
   4.529229E+03, 4.541758E+03, 4.554309E+03, 4.566882E+03, 4.579476E+03, 4.592092E+03, 
   4.604730E+03, 4.617390E+03, 4.630072E+03, 4.642776E+03, 4.655502E+03, 4.668249E+03, 
   4.681019E+03, 4.693810E+03, 4.706624E+03, 4.719459E+03, 4.732317E+03, 4.745196E+03, 
   4.758098E+03, 4.771021E+03, 4.783967E+03, 4.796934E+03, 4.809924E+03, 4.822936E+03, 
   4.835970E+03, 4.849026E+03, 4.862104E+03, 4.875204E+03, 4.888326E+03, 4.901471E+03, 
   4.914637E+03, 4.927826E+03, 4.941037E+03, 4.954271E+03, 4.967526E+03, 4.980804E+03, 
   4.994104E+03, 5.007426E+03, 5.020770E+03, 5.034137E+03, 5.047526E+03, 5.060937E+03, 
   5.074371E+03, 5.087826E+03, 5.101305E+03, 5.114805E+03, 5.128328E+03, 5.141873E+03, 
   5.155441E+03, 5.169030E+03, 5.182643E+03, 5.196277E+03, 5.209935E+03, 5.223614E+03, 
   5.237316E+03, 5.251040E+03, 5.264787E+03, 5.278556E+03, 5.292348E+03, 5.306162E+03, 
   5.319999E+03, 5.333858E+03, 5.347739E+03, 5.361643E+03, 5.375570E+03, 5.389519E+03, 
   5.403491E+03, 5.417485E+03, 5.431502E+03, 5.445541E+03, 5.459603E+03, 5.473687E+03, 
   5.487794E+03, 5.501924E+03, 5.516076E+03, 5.530251E+03, 5.544448E+03, 5.558668E+03, 
   5.572910E+03, 5.587176E+03, 5.601464E+03, 5.615774E+03, 5.630107E+03, 5.644463E+03, 
   5.658842E+03, 5.673243E+03, 5.687667E+03, 5.702113E+03, 5.716582E+03, 5.731074E+03, 
   5.745589E+03, 5.760126E+03, 5.774686E+03, 5.789269E+03, 5.803874E+03, 5.818503E+03, 
   5.833154E+03, 5.847827E+03, 5.862524E+03, 5.877243E+03, 5.891985E+03, 5.906750E+03, 
   5.921537E+03, 5.936348E+03, 5.951181E+03, 5.966037E+03, 5.980916E+03, 5.995817E+03, 
   6.010741E+03, 6.025688E+03, 6.040658E+03, 6.055651E+03, 6.070667E+03, 6.085705E+03, 
   6.100766E+03, 6.115850E+03, 6.130957E+03, 6.146087E+03, 6.161240E+03, 6.176415E+03, 
   6.191614E+03, 6.206835E+03, 6.222079E+03, 6.237346E+03, 6.252635E+03, 6.267948E+03, 
   6.283284E+03, 6.298642E+03, 6.314023E+03, 6.329427E+03, 6.344855E+03, 6.360305E+03, 
   6.375777E+03, 6.391273E+03, 6.406792E+03, 6.422333E+03, 6.437898E+03, 6.453485E+03, 
   6.469095E+03, 6.484728E+03, 6.500385E+03, 6.516064E+03, 6.531765E+03, 6.547490E+03, 
   6.563238E+03, 6.579009E+03, 6.594802E+03, 6.610619E+03, 6.626458E+03, 6.642320E+03, 
   6.658206E+03, 6.674114E+03, 6.690045E+03, 6.705999E+03, 6.721976E+03, 6.737976E+03, 
   6.753999E+03, 6.770044E+03, 6.786113E+03, 6.802205E+03, 6.818319E+03, 6.834457E+03, 
   6.850617E+03, 6.866801E+03, 6.883007E+03, 6.899236E+03, 6.915488E+03, 6.931763E+03, 
   6.948061E+03, 6.964382E+03, 6.980726E+03, 6.997093E+03, 7.013482E+03, 7.029895E+03, 
   7.046330E+03, 7.062789E+03, 7.079270E+03, 7.095774E+03, 7.112302E+03, 7.128852E+03, 
   7.145425E+03, 7.162021E+03, 7.178640E+03, 7.195281E+03, 7.211946E+03, 7.228634E+03, 
   7.245344E+03, 7.262077E+03, 7.278834E+03, 7.295613E+03, 7.312415E+03, 7.329240E+03, 
   7.346088E+03, 7.362958E+03, 7.379852E+03, 7.396768E+03, 7.413708E+03, 7.430670E+03, 
   7.447655E+03, 7.464663E+03, 7.481694E+03, 7.498747E+03, 7.515824E+03, 7.532923E+03, 
   7.550045E+03, 7.567190E+03, 7.584358E+03, 7.601549E+03, 7.618763E+03, 7.635999E+03, 
   7.653258E+03, 
])
# ============================ M = 13, I = 2 ============================
M = 13
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.597389E+01, 1.599754E+01, 1.605971E+01, 1.646471E+01, 1.732980E+01, 1.857958E+01, 
   2.012374E+01, 2.189487E+01, 2.384480E+01, 2.593834E+01, 2.814905E+01, 3.045662E+01, 
   3.284515E+01, 3.530202E+01, 3.781709E+01, 4.038212E+01, 4.299034E+01, 4.563615E+01, 
   4.831489E+01, 5.102262E+01, 5.375603E+01, 5.651228E+01, 5.928894E+01, 6.208393E+01, 
   6.489545E+01, 6.772191E+01, 7.056194E+01, 7.341434E+01, 7.627806E+01, 7.915215E+01, 
   8.203579E+01, 8.492826E+01, 8.782891E+01, 9.073715E+01, 9.365249E+01, 9.657447E+01, 
   9.950270E+01, 1.024368E+02, 1.053765E+02, 1.083216E+02, 1.112717E+02, 1.142268E+02, 
   1.171865E+02, 1.201509E+02, 1.231198E+02, 1.260931E+02, 1.290707E+02, 1.320526E+02, 
   1.350388E+02, 1.380293E+02, 1.410241E+02, 1.440232E+02, 1.470268E+02, 1.500347E+02, 
   1.530472E+02, 1.560643E+02, 1.590861E+02, 1.621128E+02, 1.651443E+02, 1.681809E+02, 
   1.712227E+02, 1.742698E+02, 1.773224E+02, 1.803806E+02, 1.834445E+02, 1.865144E+02, 
   1.895904E+02, 1.926726E+02, 1.957613E+02, 1.988565E+02, 2.019585E+02, 2.050674E+02, 
   2.081834E+02, 2.113067E+02, 2.144374E+02, 2.175757E+02, 2.207218E+02, 2.238759E+02, 
   2.270381E+02, 2.302086E+02, 2.333876E+02, 2.365751E+02, 2.397715E+02, 2.429768E+02, 
   2.461912E+02, 2.494149E+02, 2.526480E+02, 2.558907E+02, 2.591431E+02, 2.624054E+02, 
   2.656778E+02, 2.689603E+02, 2.722531E+02, 2.755563E+02, 2.788702E+02, 2.821947E+02, 
   2.855302E+02, 2.888766E+02, 2.922341E+02, 2.956028E+02, 2.989830E+02, 3.023745E+02, 
   3.057777E+02, 3.091926E+02, 3.126193E+02, 3.160580E+02, 3.195087E+02, 3.229715E+02, 
   3.264466E+02, 3.299340E+02, 3.334339E+02, 3.369463E+02, 3.404714E+02, 3.440092E+02, 
   3.475598E+02, 3.511233E+02, 3.546999E+02, 3.582895E+02, 3.618923E+02, 3.655083E+02, 
   3.691377E+02, 3.727804E+02, 3.764367E+02, 3.801065E+02, 3.837899E+02, 3.874870E+02, 
   3.911979E+02, 3.949226E+02, 3.986612E+02, 4.024138E+02, 4.061804E+02, 4.099611E+02, 
   4.137560E+02, 4.175650E+02, 4.213884E+02, 4.252260E+02, 4.290781E+02, 4.329445E+02, 
   4.368255E+02, 4.407210E+02, 4.446310E+02, 4.485558E+02, 4.524952E+02, 4.564493E+02, 
   4.604182E+02, 4.644020E+02, 4.684006E+02, 4.724141E+02, 4.764426E+02, 4.804861E+02, 
   4.845447E+02, 4.886183E+02, 4.927070E+02, 4.968109E+02, 5.009300E+02, 5.050643E+02, 
   5.092139E+02, 5.133788E+02, 5.175591E+02, 5.217547E+02, 5.259657E+02, 5.301922E+02, 
   5.344341E+02, 5.386915E+02, 5.429645E+02, 5.472531E+02, 5.515572E+02, 5.558769E+02, 
   5.602124E+02, 5.645634E+02, 5.689303E+02, 5.733128E+02, 5.777111E+02, 5.821252E+02, 
   5.865551E+02, 5.910008E+02, 5.954624E+02, 5.999399E+02, 6.044333E+02, 6.089427E+02, 
   6.134680E+02, 6.180093E+02, 6.225666E+02, 6.271399E+02, 6.317292E+02, 6.363347E+02, 
   6.409562E+02, 6.455938E+02, 6.502476E+02, 6.549175E+02, 6.596036E+02, 6.643059E+02, 
   6.690244E+02, 6.737591E+02, 6.785101E+02, 6.832773E+02, 6.880608E+02, 6.928606E+02, 
   6.976767E+02, 7.025092E+02, 7.073580E+02, 7.122232E+02, 7.171047E+02, 7.220027E+02, 
   7.269171E+02, 7.318479E+02, 7.367952E+02, 7.417589E+02, 7.467391E+02, 7.517358E+02, 
   7.567490E+02, 7.617787E+02, 7.668250E+02, 7.718878E+02, 7.769672E+02, 7.820631E+02, 
   7.871757E+02, 7.923048E+02, 7.974506E+02, 8.026130E+02, 8.077921E+02, 8.129878E+02, 
   8.182002E+02, 8.234292E+02, 8.286750E+02, 8.339375E+02, 8.392167E+02, 8.445126E+02, 
   8.498253E+02, 8.551547E+02, 8.605009E+02, 8.658639E+02, 8.712437E+02, 8.766403E+02, 
   8.820536E+02, 8.874839E+02, 8.929309E+02, 8.983948E+02, 9.038756E+02, 9.093732E+02, 
   9.148877E+02, 9.204191E+02, 9.259675E+02, 9.315327E+02, 9.371148E+02, 9.427139E+02, 
   9.483299E+02, 9.539629E+02, 9.596128E+02, 9.652797E+02, 9.709636E+02, 9.766645E+02, 
   9.823824E+02, 9.881173E+02, 9.938692E+02, 9.996382E+02, 1.005424E+03, 1.011227E+03, 
   1.017047E+03, 1.022884E+03, 1.028739E+03, 1.034610E+03, 1.040498E+03, 1.046404E+03, 
   1.052327E+03, 1.058266E+03, 1.064223E+03, 1.070197E+03, 1.076189E+03, 1.082197E+03, 
   1.088222E+03, 1.094265E+03, 1.100325E+03, 1.106402E+03, 1.112496E+03, 1.118608E+03, 
   1.124737E+03, 1.130882E+03, 1.137046E+03, 1.143226E+03, 1.149424E+03, 1.155639E+03, 
   1.161871E+03, 1.168120E+03, 1.174387E+03, 1.180671E+03, 1.186972E+03, 1.193291E+03, 
   1.199627E+03, 1.205980E+03, 1.212351E+03, 1.218739E+03, 1.225144E+03, 1.231567E+03, 
   1.238007E+03, 1.244464E+03, 1.250939E+03, 1.257431E+03, 1.263941E+03, 1.270468E+03, 
   1.277012E+03, 1.283574E+03, 1.290153E+03, 1.296749E+03, 1.303364E+03, 1.309995E+03, 
   1.316644E+03, 1.323310E+03, 1.329994E+03, 1.336696E+03, 1.343415E+03, 1.350151E+03, 
   1.356905E+03, 1.363676E+03, 1.370465E+03, 1.377272E+03, 1.384095E+03, 1.390937E+03, 
   1.397796E+03, 1.404673E+03, 1.411567E+03, 1.418478E+03, 1.425408E+03, 1.432355E+03, 
   1.439319E+03, 1.446301E+03, 1.453301E+03, 1.460318E+03, 1.467353E+03, 1.474405E+03, 
   1.481475E+03, 1.488563E+03, 1.495669E+03, 1.502792E+03, 1.509932E+03, 1.517091E+03, 
   1.524267E+03, 1.531461E+03, 1.538672E+03, 1.545901E+03, 1.553148E+03, 1.560413E+03, 
   1.567695E+03, 1.574995E+03, 1.582313E+03, 1.589648E+03, 1.597001E+03, 1.604372E+03, 
   1.611761E+03, 1.619168E+03, 1.626592E+03, 1.634034E+03, 1.641494E+03, 1.648971E+03, 
   1.656467E+03, 1.663980E+03, 1.671511E+03, 1.679060E+03, 1.686627E+03, 1.694211E+03, 
   1.701814E+03, 1.709434E+03, 1.717072E+03, 1.724728E+03, 1.732402E+03, 1.740094E+03, 
   1.747804E+03, 1.755531E+03, 1.763277E+03, 1.771040E+03, 1.778821E+03, 1.786621E+03, 
   1.794438E+03, 1.802273E+03, 1.810126E+03, 1.817997E+03, 1.825886E+03, 1.833793E+03, 
   1.841718E+03, 1.849661E+03, 1.857621E+03, 1.865600E+03, 1.873597E+03, 1.881612E+03, 
   1.889645E+03, 1.897696E+03, 1.905765E+03, 1.913852E+03, 1.921957E+03, 1.930080E+03, 
   1.938221E+03, 1.946381E+03, 1.954558E+03, 1.962753E+03, 1.970967E+03, 1.979198E+03, 
   1.987448E+03, 1.995716E+03, 2.004002E+03, 2.012306E+03, 2.020628E+03, 2.028969E+03, 
   2.037327E+03, 2.045704E+03, 2.054099E+03, 2.062512E+03, 2.070943E+03, 2.079392E+03, 
   2.087860E+03, 2.096345E+03, 2.104849E+03, 2.113372E+03, 2.121912E+03, 2.130471E+03, 
   2.139048E+03, 2.147643E+03, 2.156256E+03, 2.164888E+03, 2.173538E+03, 2.182206E+03, 
   2.190893E+03, 2.199597E+03, 2.208320E+03, 2.217062E+03, 2.225822E+03, 2.234600E+03, 
   2.243396E+03, 2.252211E+03, 2.261044E+03, 2.269895E+03, 2.278765E+03, 2.287653E+03, 
   2.296560E+03, 2.305485E+03, 2.314428E+03, 2.323390E+03, 2.332370E+03, 2.341368E+03, 
   2.350385E+03, 2.359421E+03, 2.368475E+03, 2.377547E+03, 2.386638E+03, 2.395747E+03, 
   2.404874E+03, 2.414021E+03, 2.423185E+03, 2.432368E+03, 2.441570E+03, 2.450790E+03, 
   2.460029E+03, 2.469286E+03, 2.478562E+03, 2.487856E+03, 2.497169E+03, 2.506501E+03, 
   2.515851E+03, 2.525219E+03, 2.534606E+03, 2.544012E+03, 2.553436E+03, 2.562879E+03, 
   2.572341E+03, 2.581821E+03, 2.591320E+03, 2.600838E+03, 2.610374E+03, 2.619928E+03, 
   2.629502E+03, 2.639094E+03, 2.648705E+03, 2.658335E+03, 2.667983E+03, 2.677650E+03, 
   2.687335E+03, 2.697040E+03, 2.706763E+03, 2.716505E+03, 2.726266E+03, 2.736045E+03, 
   2.745843E+03, 2.755660E+03, 2.765496E+03, 2.775350E+03, 2.785224E+03, 2.795116E+03, 
   2.805027E+03, 2.814957E+03, 2.824906E+03, 2.834873E+03, 2.844859E+03, 2.854865E+03, 
   2.864889E+03, 2.874932E+03, 2.884994E+03, 2.895075E+03, 2.905174E+03, 2.915293E+03, 
   2.925431E+03, 2.935587E+03, 2.945763E+03, 
])
# ============================ M = 13, I = 3 ============================
M = 13
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.398602E+01, 2.404395E+01, 2.529391E+01, 2.815478E+01, 3.190227E+01, 3.623179E+01, 
   4.103194E+01, 4.623929E+01, 5.180270E+01, 5.767669E+01, 6.382085E+01, 7.019978E+01, 
   7.678288E+01, 8.354395E+01, 9.046062E+01, 9.751381E+01, 1.046873E+02, 1.119670E+02, 
   1.193412E+02, 1.267995E+02, 1.343330E+02, 1.419342E+02, 1.495962E+02, 1.573133E+02, 
   1.650803E+02, 1.728928E+02, 1.807468E+02, 1.886387E+02, 1.965655E+02, 2.045244E+02, 
   2.125129E+02, 2.205288E+02, 2.285702E+02, 2.366353E+02, 2.447225E+02, 2.528305E+02, 
   2.609579E+02, 2.691036E+02, 2.772666E+02, 2.854462E+02, 2.936413E+02, 3.018515E+02, 
   3.100761E+02, 3.183146E+02, 3.265665E+02, 3.348315E+02, 3.431093E+02, 3.513996E+02, 
   3.597024E+02, 3.680174E+02, 3.763446E+02, 3.846839E+02, 3.930355E+02, 4.013993E+02, 
   4.097755E+02, 4.181642E+02, 4.265656E+02, 4.349798E+02, 4.434071E+02, 4.518478E+02, 
   4.603021E+02, 4.687704E+02, 4.772529E+02, 4.857500E+02, 4.942620E+02, 5.027894E+02, 
   5.113324E+02, 5.198916E+02, 5.284672E+02, 5.370597E+02, 5.456695E+02, 5.542971E+02, 
   5.629428E+02, 5.716071E+02, 5.802905E+02, 5.889933E+02, 5.977160E+02, 6.064591E+02, 
   6.152229E+02, 6.240080E+02, 6.328147E+02, 6.416434E+02, 6.504947E+02, 6.593690E+02, 
   6.682665E+02, 6.771879E+02, 6.861335E+02, 6.951037E+02, 7.040989E+02, 7.131195E+02, 
   7.221659E+02, 7.312386E+02, 7.403378E+02, 7.494640E+02, 7.586176E+02, 7.677989E+02, 
   7.770082E+02, 7.862460E+02, 7.955126E+02, 8.048082E+02, 8.141334E+02, 8.234883E+02, 
   8.328734E+02, 8.422889E+02, 8.517352E+02, 8.612126E+02, 8.707213E+02, 8.802617E+02, 
   8.898340E+02, 8.994386E+02, 9.090757E+02, 9.187456E+02, 9.284485E+02, 9.381848E+02, 
   9.479546E+02, 9.577583E+02, 9.675961E+02, 9.774682E+02, 9.873748E+02, 9.973162E+02, 
   1.007293E+03, 1.017304E+03, 1.027352E+03, 1.037434E+03, 1.047553E+03, 1.057708E+03, 
   1.067899E+03, 1.078126E+03, 1.088390E+03, 1.098692E+03, 1.109030E+03, 1.119405E+03, 
   1.129818E+03, 1.140268E+03, 1.150756E+03, 1.161282E+03, 1.171845E+03, 1.182448E+03, 
   1.193088E+03, 1.203767E+03, 1.214485E+03, 1.225241E+03, 1.236037E+03, 1.246871E+03, 
   1.257745E+03, 1.268658E+03, 1.279610E+03, 1.290603E+03, 1.301635E+03, 1.312706E+03, 
   1.323818E+03, 1.334970E+03, 1.346162E+03, 1.357395E+03, 1.368668E+03, 1.379981E+03, 
   1.391335E+03, 1.402730E+03, 1.414166E+03, 1.425643E+03, 1.437160E+03, 1.448719E+03, 
   1.460320E+03, 1.471961E+03, 1.483644E+03, 1.495369E+03, 1.507135E+03, 1.518943E+03, 
   1.530792E+03, 1.542684E+03, 1.554617E+03, 1.566593E+03, 1.578610E+03, 1.590670E+03, 
   1.602772E+03, 1.614916E+03, 1.627103E+03, 1.639332E+03, 1.651604E+03, 1.663919E+03, 
   1.676276E+03, 1.688676E+03, 1.701119E+03, 1.713604E+03, 1.726133E+03, 1.738704E+03, 
   1.751319E+03, 1.763977E+03, 1.776678E+03, 1.789422E+03, 1.802210E+03, 1.815041E+03, 
   1.827915E+03, 1.840833E+03, 1.853794E+03, 1.866799E+03, 1.879848E+03, 1.892940E+03, 
   1.906076E+03, 1.919256E+03, 1.932480E+03, 1.945748E+03, 1.959059E+03, 1.972415E+03, 
   1.985814E+03, 1.999258E+03, 2.012746E+03, 2.026278E+03, 2.039854E+03, 2.053474E+03, 
   2.067139E+03, 2.080848E+03, 2.094602E+03, 2.108400E+03, 2.122242E+03, 2.136129E+03, 
   2.150061E+03, 2.164037E+03, 2.178057E+03, 2.192123E+03, 2.206233E+03, 2.220388E+03, 
   2.234587E+03, 2.248832E+03, 2.263121E+03, 2.277455E+03, 2.291834E+03, 2.306258E+03, 
   2.320728E+03, 2.335242E+03, 2.349801E+03, 2.364405E+03, 2.379055E+03, 2.393749E+03, 
   2.408489E+03, 2.423274E+03, 2.438104E+03, 2.452980E+03, 2.467901E+03, 2.482867E+03, 
   2.497879E+03, 2.512936E+03, 2.528038E+03, 2.543186E+03, 2.558380E+03, 2.573619E+03, 
   2.588903E+03, 2.604233E+03, 2.619609E+03, 2.635031E+03, 2.650498E+03, 2.666011E+03, 
   2.681569E+03, 2.697174E+03, 2.712824E+03, 2.728520E+03, 2.744262E+03, 2.760049E+03, 
   2.775883E+03, 2.791763E+03, 2.807688E+03, 2.823660E+03, 2.839677E+03, 2.855740E+03, 
   2.871850E+03, 2.888006E+03, 2.904207E+03, 2.920455E+03, 2.936749E+03, 2.953089E+03, 
   2.969476E+03, 2.985908E+03, 3.002387E+03, 3.018912E+03, 3.035484E+03, 3.052101E+03, 
   3.068766E+03, 3.085476E+03, 3.102233E+03, 3.119036E+03, 3.135886E+03, 3.152782E+03, 
   3.169725E+03, 3.186714E+03, 3.203750E+03, 3.220832E+03, 3.237961E+03, 3.255136E+03, 
   3.272358E+03, 3.289627E+03, 3.306942E+03, 3.324304E+03, 3.341713E+03, 3.359168E+03, 
   3.376671E+03, 3.394220E+03, 3.411815E+03, 3.429458E+03, 3.447147E+03, 3.464884E+03, 
   3.482667E+03, 3.500497E+03, 3.518374E+03, 3.536298E+03, 3.554269E+03, 3.572287E+03, 
   3.590352E+03, 3.608463E+03, 3.626622E+03, 3.644828E+03, 3.663082E+03, 3.681382E+03, 
   3.699729E+03, 3.718124E+03, 3.736565E+03, 3.755054E+03, 3.773590E+03, 3.792173E+03, 
   3.810804E+03, 3.829482E+03, 3.848207E+03, 3.866979E+03, 3.885799E+03, 3.904666E+03, 
   3.923581E+03, 3.942542E+03, 3.961552E+03, 3.980608E+03, 3.999712E+03, 4.018864E+03, 
   4.038063E+03, 4.057309E+03, 4.076604E+03, 4.095945E+03, 4.115334E+03, 4.134771E+03, 
   4.154256E+03, 4.173788E+03, 4.193367E+03, 4.212994E+03, 4.232669E+03, 4.252392E+03, 
   4.272162E+03, 4.291981E+03, 4.311846E+03, 4.331760E+03, 4.351722E+03, 4.371731E+03, 
   4.391788E+03, 4.411893E+03, 4.432046E+03, 4.452246E+03, 4.472495E+03, 4.492791E+03, 
   4.513136E+03, 4.533528E+03, 4.553969E+03, 4.574457E+03, 4.594994E+03, 4.615578E+03, 
   4.636211E+03, 4.656891E+03, 4.677620E+03, 4.698397E+03, 4.719222E+03, 4.740095E+03, 
   4.761016E+03, 4.781986E+03, 4.803003E+03, 4.824069E+03, 4.845183E+03, 4.866346E+03, 
   4.887556E+03, 4.908815E+03, 4.930123E+03, 4.951478E+03, 4.972882E+03, 4.994335E+03, 
   5.015836E+03, 5.037385E+03, 5.058982E+03, 5.080628E+03, 5.102323E+03, 5.124066E+03, 
   5.145858E+03, 5.167698E+03, 5.189586E+03, 5.211523E+03, 5.233509E+03, 5.255544E+03, 
   5.277627E+03, 5.299758E+03, 5.321938E+03, 5.344167E+03, 5.366445E+03, 5.388771E+03, 
   5.411146E+03, 5.433570E+03, 5.456043E+03, 5.478564E+03, 5.501134E+03, 5.523753E+03, 
   5.546421E+03, 5.569138E+03, 5.591904E+03, 5.614718E+03, 5.637581E+03, 5.660494E+03, 
   5.683455E+03, 5.706465E+03, 5.729525E+03, 5.752633E+03, 5.775790E+03, 5.798996E+03, 
   5.822252E+03, 5.845556E+03, 5.868910E+03, 5.892313E+03, 5.915764E+03, 5.939265E+03, 
   5.962816E+03, 5.986415E+03, 6.010064E+03, 6.033761E+03, 6.057509E+03, 6.081305E+03, 
   6.105151E+03, 6.129045E+03, 6.152990E+03, 6.176983E+03, 6.201026E+03, 6.225119E+03, 
   6.249261E+03, 6.273452E+03, 6.297692E+03, 6.321982E+03, 6.346322E+03, 6.370711E+03, 
   6.395150E+03, 6.419638E+03, 6.444175E+03, 6.468763E+03, 6.493399E+03, 6.518086E+03, 
   6.542822E+03, 6.567608E+03, 6.592443E+03, 6.617328E+03, 6.642263E+03, 6.667247E+03, 
   6.692281E+03, 6.717365E+03, 6.742499E+03, 6.767683E+03, 6.792916E+03, 6.818199E+03, 
   6.843532E+03, 6.868915E+03, 6.894348E+03, 6.919831E+03, 6.945363E+03, 6.970946E+03, 
   6.996578E+03, 7.022261E+03, 7.047993E+03, 7.073776E+03, 7.099608E+03, 7.125491E+03, 
   7.151424E+03, 7.177407E+03, 7.203440E+03, 7.229523E+03, 7.255656E+03, 7.281839E+03, 
   7.308073E+03, 7.334357E+03, 7.360691E+03, 7.387075E+03, 7.413510E+03, 7.439994E+03, 
   7.466529E+03, 7.493115E+03, 7.519751E+03, 7.546437E+03, 7.573173E+03, 7.599960E+03, 
   7.626798E+03, 7.653686E+03, 7.680624E+03, 7.707613E+03, 7.734652E+03, 7.761742E+03, 
   7.788882E+03, 7.816073E+03, 7.843314E+03, 7.870606E+03, 7.897949E+03, 7.925342E+03, 
   7.952786E+03, 7.980280E+03, 8.007825E+03, 
])
# ============================ M = 14, I = 1 ============================
M = 14
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.000000E+00, 4.032390E+00, 4.626210E+00, 5.725030E+00, 6.976320E+00, 8.275850E+00, 
   9.595800E+00, 1.092645E+01, 1.226348E+01, 1.360466E+01, 1.494872E+01, 1.629489E+01, 
   1.764266E+01, 1.899169E+01, 2.034174E+01, 2.169263E+01, 2.304424E+01, 2.439648E+01, 
   2.574926E+01, 2.710253E+01, 2.845624E+01, 2.981036E+01, 3.116485E+01, 3.251970E+01, 
   3.387487E+01, 3.523036E+01, 3.658615E+01, 3.794222E+01, 3.929857E+01, 4.065519E+01, 
   4.201207E+01, 4.336920E+01, 4.472657E+01, 4.608419E+01, 4.744205E+01, 4.880014E+01, 
   5.015846E+01, 5.151701E+01, 5.287579E+01, 5.423478E+01, 5.559401E+01, 5.695345E+01, 
   5.831312E+01, 5.967301E+01, 6.103312E+01, 6.239345E+01, 6.375401E+01, 6.511480E+01, 
   6.647583E+01, 6.783708E+01, 6.919858E+01, 7.056033E+01, 7.192233E+01, 7.328460E+01, 
   7.464713E+01, 7.600994E+01, 7.737305E+01, 7.873645E+01, 8.010018E+01, 8.146423E+01, 
   8.282863E+01, 8.419339E+01, 8.555853E+01, 8.692407E+01, 8.829002E+01, 8.965642E+01, 
   9.102328E+01, 9.239063E+01, 9.375848E+01, 9.512688E+01, 9.649584E+01, 9.786539E+01, 
   9.923557E+01, 1.006064E+02, 1.019779E+02, 1.033501E+02, 1.047231E+02, 1.060969E+02, 
   1.074715E+02, 1.088469E+02, 1.102232E+02, 1.116005E+02, 1.129788E+02, 1.143580E+02, 
   1.157383E+02, 1.171197E+02, 1.185022E+02, 1.198858E+02, 1.212707E+02, 1.226569E+02, 
   1.240443E+02, 1.254331E+02, 1.268232E+02, 1.282148E+02, 1.296079E+02, 1.310024E+02, 
   1.323986E+02, 1.337963E+02, 1.351957E+02, 1.365967E+02, 1.379995E+02, 1.394041E+02, 
   1.408105E+02, 1.422187E+02, 1.436289E+02, 1.450410E+02, 1.464552E+02, 1.478713E+02, 
   1.492896E+02, 1.507099E+02, 1.521325E+02, 1.535573E+02, 1.549843E+02, 1.564136E+02, 
   1.578453E+02, 1.592793E+02, 1.607158E+02, 1.621547E+02, 1.635962E+02, 1.650402E+02, 
   1.664868E+02, 1.679360E+02, 1.693878E+02, 1.708424E+02, 1.722998E+02, 1.737599E+02, 
   1.752228E+02, 1.766886E+02, 1.781573E+02, 1.796289E+02, 1.811035E+02, 1.825811E+02, 
   1.840617E+02, 1.855455E+02, 1.870323E+02, 1.885223E+02, 1.900154E+02, 1.915118E+02, 
   1.930114E+02, 1.945143E+02, 1.960205E+02, 1.975300E+02, 1.990429E+02, 2.005592E+02, 
   2.020790E+02, 2.036022E+02, 2.051289E+02, 2.066592E+02, 2.081930E+02, 2.097304E+02, 
   2.112714E+02, 2.128160E+02, 2.143644E+02, 2.159164E+02, 2.174721E+02, 2.190316E+02, 
   2.205948E+02, 2.221619E+02, 2.237328E+02, 2.253075E+02, 2.268862E+02, 2.284687E+02, 
   2.300551E+02, 2.316455E+02, 2.332399E+02, 2.348383E+02, 2.364407E+02, 2.380471E+02, 
   2.396577E+02, 2.412723E+02, 2.428910E+02, 2.445138E+02, 2.461408E+02, 2.477720E+02, 
   2.494073E+02, 2.510469E+02, 2.526907E+02, 2.543388E+02, 2.559911E+02, 2.576478E+02, 
   2.593087E+02, 2.609740E+02, 2.626436E+02, 2.643176E+02, 2.659960E+02, 2.676788E+02, 
   2.693660E+02, 2.710577E+02, 2.727538E+02, 2.744543E+02, 2.761594E+02, 2.778690E+02, 
   2.795831E+02, 2.813018E+02, 2.830250E+02, 2.847527E+02, 2.864851E+02, 2.882221E+02, 
   2.899636E+02, 2.917099E+02, 2.934607E+02, 2.952163E+02, 2.969764E+02, 2.987414E+02, 
   3.005109E+02, 3.022853E+02, 3.040643E+02, 3.058481E+02, 3.076367E+02, 3.094300E+02, 
   3.112281E+02, 3.130310E+02, 3.148388E+02, 3.166513E+02, 3.184687E+02, 3.202910E+02, 
   3.221181E+02, 3.239501E+02, 3.257869E+02, 3.276287E+02, 3.294754E+02, 3.313270E+02, 
   3.331835E+02, 3.350450E+02, 3.369115E+02, 3.387829E+02, 3.406592E+02, 3.425406E+02, 
   3.444270E+02, 3.463184E+02, 3.482148E+02, 3.501163E+02, 3.520228E+02, 3.539343E+02, 
   3.558509E+02, 3.577726E+02, 3.596994E+02, 3.616313E+02, 3.635682E+02, 3.655103E+02, 
   3.674576E+02, 3.694099E+02, 3.713674E+02, 3.733301E+02, 3.752979E+02, 3.772708E+02, 
   3.792490E+02, 3.812323E+02, 3.832209E+02, 3.852147E+02, 3.872136E+02, 3.892179E+02, 
   3.912273E+02, 3.932420E+02, 3.952619E+02, 3.972871E+02, 3.993176E+02, 4.013533E+02, 
   4.033944E+02, 4.054407E+02, 4.074924E+02, 4.095493E+02, 4.116116E+02, 4.136792E+02, 
   4.157521E+02, 4.178304E+02, 4.199140E+02, 4.220030E+02, 4.240974E+02, 4.261971E+02, 
   4.283022E+02, 4.304127E+02, 4.325286E+02, 4.346499E+02, 4.367767E+02, 4.389088E+02, 
   4.410464E+02, 4.431894E+02, 4.453379E+02, 4.474918E+02, 4.496512E+02, 4.518160E+02, 
   4.539863E+02, 4.561621E+02, 4.583434E+02, 4.605301E+02, 4.627224E+02, 4.649202E+02, 
   4.671235E+02, 4.693323E+02, 4.715467E+02, 4.737666E+02, 4.759920E+02, 4.782230E+02, 
   4.804595E+02, 4.827016E+02, 4.849493E+02, 4.872025E+02, 4.894613E+02, 4.917258E+02, 
   4.939958E+02, 4.962714E+02, 4.985526E+02, 5.008395E+02, 5.031320E+02, 5.054300E+02, 
   5.077338E+02, 5.100432E+02, 5.123582E+02, 5.146789E+02, 5.170052E+02, 5.193372E+02, 
   5.216749E+02, 5.240182E+02, 5.263673E+02, 5.287220E+02, 5.310825E+02, 5.334486E+02, 
   5.358205E+02, 5.381980E+02, 5.405813E+02, 5.429703E+02, 5.453651E+02, 5.477656E+02, 
   5.501718E+02, 5.525838E+02, 5.550016E+02, 5.574251E+02, 5.598544E+02, 5.622894E+02, 
   5.647303E+02, 5.671769E+02, 5.696293E+02, 5.720875E+02, 5.745516E+02, 5.770214E+02, 
   5.794971E+02, 5.819785E+02, 5.844659E+02, 5.869590E+02, 5.894580E+02, 5.919628E+02, 
   5.944735E+02, 5.969900E+02, 5.995124E+02, 6.020406E+02, 6.045748E+02, 6.071148E+02, 
   6.096607E+02, 6.122124E+02, 6.147701E+02, 6.173337E+02, 6.199032E+02, 6.224786E+02, 
   6.250599E+02, 6.276471E+02, 6.302403E+02, 6.328393E+02, 6.354444E+02, 6.380553E+02, 
   6.406723E+02, 6.432951E+02, 6.459240E+02, 6.485588E+02, 6.511995E+02, 6.538463E+02, 
   6.564990E+02, 6.591577E+02, 6.618224E+02, 6.644931E+02, 6.671698E+02, 6.698525E+02, 
   6.725412E+02, 6.752359E+02, 6.779366E+02, 6.806434E+02, 6.833562E+02, 6.860751E+02, 
   6.887999E+02, 6.915309E+02, 6.942678E+02, 6.970109E+02, 6.997600E+02, 7.025151E+02, 
   7.052764E+02, 7.080437E+02, 7.108171E+02, 7.135965E+02, 7.163821E+02, 7.191738E+02, 
   7.219715E+02, 7.247754E+02, 7.275854E+02, 7.304015E+02, 7.332237E+02, 7.360520E+02, 
   7.388865E+02, 7.417271E+02, 7.445739E+02, 7.474267E+02, 7.502858E+02, 7.531510E+02, 
   7.560223E+02, 7.588998E+02, 7.617835E+02, 7.646734E+02, 7.675694E+02, 7.704716E+02, 
   7.733800E+02, 7.762946E+02, 7.792153E+02, 7.821423E+02, 7.850755E+02, 7.880149E+02, 
   7.909605E+02, 7.939123E+02, 7.968704E+02, 7.998347E+02, 8.028052E+02, 8.057819E+02, 
   8.087649E+02, 8.117541E+02, 8.147496E+02, 8.177513E+02, 8.207593E+02, 8.237735E+02, 
   8.267940E+02, 8.298208E+02, 8.328539E+02, 8.358932E+02, 8.389388E+02, 8.419908E+02, 
   8.450490E+02, 8.481135E+02, 8.511843E+02, 8.542614E+02, 8.573448E+02, 8.604345E+02, 
   8.635306E+02, 8.666329E+02, 8.697416E+02, 8.728566E+02, 8.759780E+02, 8.791057E+02, 
   8.822397E+02, 8.853801E+02, 8.885269E+02, 8.916800E+02, 8.948394E+02, 8.980052E+02, 
   9.011774E+02, 9.043560E+02, 9.075409E+02, 9.107322E+02, 9.139299E+02, 9.171339E+02, 
   9.203444E+02, 9.235613E+02, 9.267845E+02, 9.300142E+02, 9.332502E+02, 9.364927E+02, 
   9.397416E+02, 9.429969E+02, 9.462586E+02, 9.495268E+02, 9.528014E+02, 9.560824E+02, 
   9.593698E+02, 9.626637E+02, 9.659640E+02, 9.692708E+02, 9.725841E+02, 9.759037E+02, 
   9.792299E+02, 9.825625E+02, 9.859015E+02, 9.892471E+02, 9.925991E+02, 9.959576E+02, 
   9.993225E+02, 1.002694E+03, 1.006072E+03, 1.009456E+03, 1.012847E+03, 1.016245E+03, 
   1.019649E+03, 1.023059E+03, 1.026476E+03, 1.029899E+03, 1.033329E+03, 1.036766E+03, 
   1.040209E+03, 1.043658E+03, 1.047114E+03, 1.050577E+03, 1.054046E+03, 1.057522E+03, 
   1.061004E+03, 1.064493E+03, 1.067988E+03, 1.071490E+03, 1.074998E+03, 1.078513E+03, 
   1.082034E+03, 1.085562E+03, 1.089097E+03, 1.092638E+03, 1.096186E+03, 1.099740E+03, 
   1.103301E+03, 1.106869E+03, 1.110443E+03, 1.114023E+03, 1.117611E+03, 1.121204E+03, 
   1.124805E+03, 1.128412E+03, 1.132026E+03, 1.135646E+03, 1.139273E+03, 1.142906E+03, 
   1.146546E+03, 1.150193E+03, 1.153846E+03, 1.157506E+03, 1.161173E+03, 1.164846E+03, 
   1.168525E+03, 1.172212E+03, 1.175905E+03, 1.179605E+03, 1.183311E+03, 1.187024E+03, 
   1.190743E+03, 1.194470E+03, 1.198203E+03, 1.201942E+03, 1.205688E+03, 1.209441E+03, 
   1.213201E+03, 1.216967E+03, 1.220740E+03, 1.224519E+03, 1.228306E+03, 1.232098E+03, 
   1.235898E+03, 1.239704E+03, 1.243517E+03, 1.247337E+03, 1.251163E+03, 1.254996E+03, 
   1.258835E+03, 1.262682E+03, 1.266535E+03, 1.270394E+03, 1.274261E+03, 1.278134E+03, 
   1.282013E+03, 1.285900E+03, 1.289793E+03, 1.293693E+03, 1.297600E+03, 1.301513E+03, 
   1.305433E+03, 1.309360E+03, 1.313293E+03, 1.317233E+03, 1.321180E+03, 1.325134E+03, 
   1.329094E+03, 1.333061E+03, 1.337035E+03, 1.341015E+03, 1.345003E+03, 1.348997E+03, 
   1.352997E+03, 1.357005E+03, 1.361019E+03, 1.365040E+03, 1.369067E+03, 1.373102E+03, 
   1.377143E+03, 1.381191E+03, 1.385245E+03, 1.389307E+03, 1.393375E+03, 1.397450E+03, 
   1.401531E+03, 1.405620E+03, 1.409715E+03, 1.413817E+03, 1.417925E+03, 1.422041E+03, 
   1.426163E+03, 1.430292E+03, 1.434428E+03, 1.438570E+03, 1.442719E+03, 1.446875E+03, 
   1.451038E+03, 
])
# ============================ M = 14, I = 2 ============================
M = 14
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.000000E+00, 6.793590E+00, 1.005366E+01, 1.375436E+01, 1.753287E+01, 2.133900E+01, 
   2.515838E+01, 2.898529E+01, 3.281701E+01, 3.665204E+01, 4.048954E+01, 4.432896E+01, 
   4.816994E+01, 5.201225E+01, 5.585570E+01, 5.970017E+01, 6.354558E+01, 6.739185E+01, 
   7.123892E+01, 7.508675E+01, 7.893531E+01, 8.278457E+01, 8.663450E+01, 9.048510E+01, 
   9.433634E+01, 9.818821E+01, 1.020407E+02, 1.058938E+02, 1.097475E+02, 1.136019E+02, 
   1.174568E+02, 1.213123E+02, 1.251685E+02, 1.290253E+02, 1.328827E+02, 1.367408E+02, 
   1.405995E+02, 1.444590E+02, 1.483191E+02, 1.521801E+02, 1.560419E+02, 1.599045E+02, 
   1.637681E+02, 1.676327E+02, 1.714984E+02, 1.753653E+02, 1.792334E+02, 1.831029E+02, 
   1.869738E+02, 1.908464E+02, 1.947206E+02, 1.985968E+02, 2.024749E+02, 2.063552E+02, 
   2.102378E+02, 2.141228E+02, 2.180106E+02, 2.219011E+02, 2.257947E+02, 2.296915E+02, 
   2.335917E+02, 2.374954E+02, 2.414030E+02, 2.453146E+02, 2.492305E+02, 2.531508E+02, 
   2.570757E+02, 2.610056E+02, 2.649405E+02, 2.688809E+02, 2.728267E+02, 2.767784E+02, 
   2.807361E+02, 2.847000E+02, 2.886704E+02, 2.926476E+02, 2.966316E+02, 3.006229E+02, 
   3.046215E+02, 3.086277E+02, 3.126418E+02, 3.166640E+02, 3.206944E+02, 3.247333E+02, 
   3.287810E+02, 3.328376E+02, 3.369034E+02, 3.409785E+02, 3.450632E+02, 3.491576E+02, 
   3.532621E+02, 3.573767E+02, 3.615017E+02, 3.656372E+02, 3.697836E+02, 3.739408E+02, 
   3.781092E+02, 3.822889E+02, 3.864802E+02, 3.906831E+02, 3.948978E+02, 3.991246E+02, 
   4.033635E+02, 4.076148E+02, 4.118787E+02, 4.161552E+02, 4.204445E+02, 4.247468E+02, 
   4.290623E+02, 4.333911E+02, 4.377333E+02, 4.420890E+02, 4.464585E+02, 4.508419E+02, 
   4.552392E+02, 4.596507E+02, 4.640764E+02, 4.685165E+02, 4.729710E+02, 4.774403E+02, 
   4.819242E+02, 4.864231E+02, 4.909369E+02, 4.954658E+02, 5.000099E+02, 5.045694E+02, 
   5.091442E+02, 5.137346E+02, 5.183406E+02, 5.229624E+02, 5.276000E+02, 5.322535E+02, 
   5.369230E+02, 5.416087E+02, 5.463105E+02, 5.510287E+02, 5.557632E+02, 5.605143E+02, 
   5.652818E+02, 5.700661E+02, 5.748670E+02, 5.796848E+02, 5.845194E+02, 5.893710E+02, 
   5.942396E+02, 5.991254E+02, 6.040283E+02, 6.089485E+02, 6.138860E+02, 6.188409E+02, 
   6.238133E+02, 6.288032E+02, 6.338107E+02, 6.388359E+02, 6.438787E+02, 6.489394E+02, 
   6.540179E+02, 6.591144E+02, 6.642288E+02, 6.693612E+02, 6.745117E+02, 6.796804E+02, 
   6.848672E+02, 6.900723E+02, 6.952957E+02, 7.005375E+02, 7.057977E+02, 7.110763E+02, 
   7.163734E+02, 7.216891E+02, 7.270235E+02, 7.323764E+02, 7.377481E+02, 7.431385E+02, 
   7.485477E+02, 7.539758E+02, 7.594228E+02, 7.648887E+02, 7.703735E+02, 7.758774E+02, 
   7.814004E+02, 7.869424E+02, 7.925036E+02, 7.980839E+02, 8.036835E+02, 8.093024E+02, 
   8.149406E+02, 8.205981E+02, 8.262749E+02, 8.319712E+02, 8.376870E+02, 8.434222E+02, 
   8.491770E+02, 8.549513E+02, 8.607452E+02, 8.665588E+02, 8.723920E+02, 8.782449E+02, 
   8.841176E+02, 8.900100E+02, 8.959222E+02, 9.018543E+02, 9.078062E+02, 9.137780E+02, 
   9.197698E+02, 9.257815E+02, 9.318132E+02, 9.378649E+02, 9.439367E+02, 9.500285E+02, 
   9.561405E+02, 9.622726E+02, 9.684249E+02, 9.745974E+02, 9.807901E+02, 9.870030E+02, 
   9.932363E+02, 9.994898E+02, 1.005764E+03, 1.012058E+03, 1.018373E+03, 1.024708E+03, 
   1.031063E+03, 1.037439E+03, 1.043836E+03, 1.050253E+03, 1.056690E+03, 1.063148E+03, 
   1.069627E+03, 1.076127E+03, 1.082647E+03, 1.089187E+03, 1.095749E+03, 1.102331E+03, 
   1.108934E+03, 1.115557E+03, 1.122202E+03, 1.128867E+03, 1.135553E+03, 1.142260E+03, 
   1.148988E+03, 1.155737E+03, 1.162506E+03, 1.169297E+03, 1.176109E+03, 1.182941E+03, 
   1.189795E+03, 1.196670E+03, 1.203566E+03, 1.210483E+03, 1.217421E+03, 1.224380E+03, 
   1.231360E+03, 1.238362E+03, 1.245384E+03, 1.252428E+03, 1.259493E+03, 1.266580E+03, 
   1.273688E+03, 1.280817E+03, 1.287967E+03, 1.295139E+03, 1.302332E+03, 1.309546E+03, 
   1.316782E+03, 1.324039E+03, 1.331318E+03, 1.338618E+03, 1.345940E+03, 1.353283E+03, 
   1.360648E+03, 1.368034E+03, 1.375442E+03, 1.382871E+03, 1.390322E+03, 1.397794E+03, 
   1.405289E+03, 1.412805E+03, 1.420342E+03, 1.427901E+03, 1.435482E+03, 1.443085E+03, 
   1.450709E+03, 1.458356E+03, 1.466023E+03, 1.473713E+03, 1.481425E+03, 1.489158E+03, 
   1.496913E+03, 1.504691E+03, 1.512490E+03, 1.520311E+03, 1.528153E+03, 1.536018E+03, 
   1.543905E+03, 1.551814E+03, 1.559744E+03, 1.567697E+03, 1.575672E+03, 1.583668E+03, 
   1.591687E+03, 1.599728E+03, 1.607791E+03, 1.615876E+03, 1.623983E+03, 1.632112E+03, 
   1.640264E+03, 1.648438E+03, 1.656633E+03, 1.664851E+03, 1.673092E+03, 1.681354E+03, 
   1.689639E+03, 1.697946E+03, 1.706275E+03, 1.714626E+03, 1.723000E+03, 1.731396E+03, 
   1.739815E+03, 1.748255E+03, 1.756718E+03, 1.765204E+03, 1.773712E+03, 1.782242E+03, 
   1.790795E+03, 1.799370E+03, 1.807968E+03, 1.816588E+03, 1.825230E+03, 1.833895E+03, 
   1.842583E+03, 1.851293E+03, 1.860025E+03, 1.868780E+03, 1.877558E+03, 1.886358E+03, 
   1.895181E+03, 1.904026E+03, 1.912894E+03, 1.921785E+03, 1.930698E+03, 1.939634E+03, 
   1.948593E+03, 1.957574E+03, 1.966578E+03, 1.975604E+03, 1.984654E+03, 1.993726E+03, 
   2.002820E+03, 2.011938E+03, 2.021078E+03, 2.030241E+03, 2.039427E+03, 2.048635E+03, 
   2.057867E+03, 2.067121E+03, 2.076398E+03, 2.085698E+03, 2.095020E+03, 2.104366E+03, 
   2.113734E+03, 2.123126E+03, 2.132540E+03, 2.141977E+03, 2.151437E+03, 2.160920E+03, 
   2.170425E+03, 2.179954E+03, 2.189506E+03, 2.199080E+03, 2.208678E+03, 2.218299E+03, 
   2.227942E+03, 2.237609E+03, 2.247298E+03, 2.257011E+03, 2.266747E+03, 2.276505E+03, 
   2.286287E+03, 2.296092E+03, 2.305920E+03, 2.315771E+03, 2.325645E+03, 2.335542E+03, 
   2.345462E+03, 2.355405E+03, 2.365371E+03, 2.375361E+03, 2.385374E+03, 2.395409E+03, 
   2.405468E+03, 2.415550E+03, 2.425656E+03, 2.435784E+03, 2.445936E+03, 2.456111E+03, 
   2.466309E+03, 2.476530E+03, 2.486774E+03, 2.497042E+03, 2.507333E+03, 2.517647E+03, 
   2.527984E+03, 2.538345E+03, 2.548729E+03, 2.559136E+03, 2.569566E+03, 2.580020E+03, 
   2.590496E+03, 2.600997E+03, 2.611520E+03, 2.622067E+03, 2.632637E+03, 2.643230E+03, 
   2.653847E+03, 2.664487E+03, 2.675150E+03, 2.685837E+03, 2.696547E+03, 2.707281E+03, 
   2.718037E+03, 2.728817E+03, 2.739621E+03, 2.750448E+03, 2.761298E+03, 2.772171E+03, 
   2.783068E+03, 2.793988E+03, 2.804932E+03, 2.815899E+03, 2.826890E+03, 2.837904E+03, 
   2.848941E+03, 2.860001E+03, 2.871086E+03, 2.882193E+03, 2.893324E+03, 2.904478E+03, 
   2.915656E+03, 2.926857E+03, 2.938082E+03, 2.949330E+03, 2.960602E+03, 2.971897E+03, 
   2.983215E+03, 2.994557E+03, 3.005923E+03, 3.017311E+03, 3.028724E+03, 3.040159E+03, 
   3.051619E+03, 3.063101E+03, 3.074608E+03, 3.086137E+03, 3.097690E+03, 3.109267E+03, 
   3.120867E+03, 3.132490E+03, 3.144138E+03, 3.155808E+03, 3.167502E+03, 3.179220E+03, 
   3.190961E+03, 3.202725E+03, 3.214513E+03, 3.226325E+03, 3.238160E+03, 3.250018E+03, 
   3.261900E+03, 3.273806E+03, 3.285735E+03, 3.297687E+03, 3.309663E+03, 3.321663E+03, 
   3.333686E+03, 3.345732E+03, 3.357802E+03, 3.369896E+03, 3.382013E+03, 3.394154E+03, 
   3.406318E+03, 3.418505E+03, 3.430716E+03, 3.442951E+03, 3.455209E+03, 3.467491E+03, 
   3.479796E+03, 3.492125E+03, 3.504477E+03, 3.516852E+03, 3.529252E+03, 3.541674E+03, 
   3.554120E+03, 3.566590E+03, 3.579083E+03, 3.591600E+03, 3.604140E+03, 3.616704E+03, 
   3.629291E+03, 3.641902E+03, 3.654536E+03, 3.667194E+03, 3.679875E+03, 3.692580E+03, 
   3.705308E+03, 3.718059E+03, 3.730835E+03, 3.743633E+03, 3.756455E+03, 3.769301E+03, 
   3.782170E+03, 3.795063E+03, 3.807979E+03, 3.820918E+03, 3.833881E+03, 3.846867E+03, 
   3.859877E+03, 3.872911E+03, 3.885968E+03, 3.899048E+03, 3.912152E+03, 3.925279E+03, 
   3.938429E+03, 3.951604E+03, 3.964801E+03, 3.978022E+03, 3.991266E+03, 4.004534E+03, 
   4.017826E+03, 4.031140E+03, 4.044478E+03, 4.057840E+03, 4.071225E+03, 4.084633E+03, 
   4.098065E+03, 4.111520E+03, 4.124999E+03, 4.138501E+03, 4.152026E+03, 4.165575E+03, 
   4.179147E+03, 4.192743E+03, 4.206362E+03, 4.220004E+03, 4.233670E+03, 4.247359E+03, 
   4.261071E+03, 4.274807E+03, 4.288566E+03, 4.302348E+03, 4.316154E+03, 4.329983E+03, 
   4.343836E+03, 4.357711E+03, 4.371610E+03, 4.385533E+03, 4.399478E+03, 4.413447E+03, 
   4.427440E+03, 4.441455E+03, 4.455494E+03, 4.469556E+03, 4.483642E+03, 4.497751E+03, 
   4.511883E+03, 4.526038E+03, 4.540216E+03, 4.554418E+03, 4.568643E+03, 4.582891E+03, 
   4.597163E+03, 4.611458E+03, 4.625776E+03, 4.640117E+03, 4.654481E+03, 4.668869E+03, 
   4.683279E+03, 4.697713E+03, 4.712171E+03, 4.726651E+03, 4.741154E+03, 4.755681E+03, 
   4.770231E+03, 4.784804E+03, 4.799400E+03, 4.814019E+03, 4.828661E+03, 4.843327E+03, 
   4.858015E+03, 4.872727E+03, 4.887462E+03, 4.902220E+03, 4.917001E+03, 4.931805E+03, 
   4.946632E+03, 4.961482E+03, 4.976356E+03, 4.991252E+03, 5.006171E+03, 5.021114E+03, 
   5.036079E+03, 
])
# ============================ M = 15, I = 1 ============================
M = 15
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.000000E+00, 9.195100E+00, 1.379368E+01, 1.894390E+01, 2.419193E+01, 2.947499E+01, 
   3.477493E+01, 4.008450E+01, 4.540023E+01, 5.072023E+01, 5.604342E+01, 6.136910E+01, 
   6.669682E+01, 7.202626E+01, 7.735721E+01, 8.268950E+01, 8.802303E+01, 9.335769E+01, 
   9.869341E+01, 1.040302E+02, 1.093679E+02, 1.147065E+02, 1.200460E+02, 1.253865E+02, 
   1.307278E+02, 1.360699E+02, 1.414129E+02, 1.467567E+02, 1.521013E+02, 1.574467E+02, 
   1.627930E+02, 1.681401E+02, 1.734880E+02, 1.788367E+02, 1.841863E+02, 1.895368E+02, 
   1.948883E+02, 2.002407E+02, 2.055941E+02, 2.109486E+02, 2.163042E+02, 2.216611E+02, 
   2.270192E+02, 2.323788E+02, 2.377399E+02, 2.431027E+02, 2.484672E+02, 2.538336E+02, 
   2.592022E+02, 2.645730E+02, 2.699463E+02, 2.753222E+02, 2.807009E+02, 2.860827E+02, 
   2.914679E+02, 2.968565E+02, 3.022490E+02, 3.076454E+02, 3.130462E+02, 3.184515E+02, 
   3.238617E+02, 3.292771E+02, 3.346978E+02, 3.401243E+02, 3.455568E+02, 3.509957E+02, 
   3.564412E+02, 3.618937E+02, 3.673535E+02, 3.728208E+02, 3.782961E+02, 3.837796E+02, 
   3.892717E+02, 3.947727E+02, 4.002829E+02, 4.058026E+02, 4.113322E+02, 4.168720E+02, 
   4.224223E+02, 4.279833E+02, 4.335555E+02, 4.391392E+02, 4.447346E+02, 4.503420E+02, 
   4.559619E+02, 4.615944E+02, 4.672398E+02, 4.728985E+02, 4.785708E+02, 4.842569E+02, 
   4.899571E+02, 4.956718E+02, 5.014011E+02, 5.071454E+02, 5.129049E+02, 5.186798E+02, 
   5.244706E+02, 5.302773E+02, 5.361002E+02, 5.419397E+02, 5.477959E+02, 5.536691E+02, 
   5.595596E+02, 5.654674E+02, 5.713930E+02, 5.773364E+02, 5.832980E+02, 5.892779E+02, 
   5.952764E+02, 6.012936E+02, 6.073297E+02, 6.133850E+02, 6.194597E+02, 6.255539E+02, 
   6.316679E+02, 6.378017E+02, 6.439557E+02, 6.501300E+02, 6.563247E+02, 6.625400E+02, 
   6.687761E+02, 6.750332E+02, 6.813114E+02, 6.876109E+02, 6.939317E+02, 7.002742E+02, 
   7.066385E+02, 7.130246E+02, 7.194327E+02, 7.258630E+02, 7.323156E+02, 7.387907E+02, 
   7.452883E+02, 7.518086E+02, 7.583518E+02, 7.649179E+02, 7.715072E+02, 7.781196E+02, 
   7.847553E+02, 7.914145E+02, 7.980973E+02, 8.048037E+02, 8.115339E+02, 8.182879E+02, 
   8.250660E+02, 8.318682E+02, 8.386946E+02, 8.455453E+02, 8.524203E+02, 8.593199E+02, 
   8.662441E+02, 8.731930E+02, 8.801667E+02, 8.871652E+02, 8.941887E+02, 9.012373E+02, 
   9.083110E+02, 9.154099E+02, 9.225341E+02, 9.296837E+02, 9.368588E+02, 9.440594E+02, 
   9.512857E+02, 9.585377E+02, 9.658154E+02, 9.731191E+02, 9.804486E+02, 9.878042E+02, 
   9.951858E+02, 1.002594E+03, 1.010028E+03, 1.017488E+03, 1.024974E+03, 1.032488E+03, 
   1.040027E+03, 1.047593E+03, 1.055186E+03, 1.062805E+03, 1.070451E+03, 1.078124E+03, 
   1.085824E+03, 1.093551E+03, 1.101304E+03, 1.109085E+03, 1.116893E+03, 1.124728E+03, 
   1.132590E+03, 1.140480E+03, 1.148396E+03, 1.156340E+03, 1.164312E+03, 1.172311E+03, 
   1.180338E+03, 1.188392E+03, 1.196474E+03, 1.204583E+03, 1.212720E+03, 1.220885E+03, 
   1.229078E+03, 1.237299E+03, 1.245547E+03, 1.253824E+03, 1.262129E+03, 1.270461E+03, 
   1.278822E+03, 1.287211E+03, 1.295628E+03, 1.304074E+03, 1.312548E+03, 1.321050E+03, 
   1.329580E+03, 1.338139E+03, 1.346727E+03, 1.355342E+03, 1.363987E+03, 1.372660E+03, 
   1.381362E+03, 1.390092E+03, 1.398851E+03, 1.407639E+03, 1.416456E+03, 1.425301E+03, 
   1.434176E+03, 1.443079E+03, 1.452011E+03, 1.460973E+03, 1.469963E+03, 1.478983E+03, 
   1.488031E+03, 1.497109E+03, 1.506216E+03, 1.515352E+03, 1.524517E+03, 1.533712E+03, 
   1.542936E+03, 1.552189E+03, 1.561472E+03, 1.570785E+03, 1.580126E+03, 1.589498E+03, 
   1.598899E+03, 1.608329E+03, 1.617789E+03, 1.627279E+03, 1.636798E+03, 1.646348E+03, 
   1.655927E+03, 1.665535E+03, 1.675174E+03, 1.684842E+03, 1.694541E+03, 1.704269E+03, 
   1.714027E+03, 1.723816E+03, 1.733634E+03, 1.743483E+03, 1.753361E+03, 1.763270E+03, 
   1.773209E+03, 1.783178E+03, 1.793177E+03, 1.803206E+03, 1.813266E+03, 1.823356E+03, 
   1.833477E+03, 1.843628E+03, 1.853809E+03, 1.864021E+03, 1.874263E+03, 1.884536E+03, 
   1.894840E+03, 1.905174E+03, 1.915538E+03, 1.925933E+03, 1.936359E+03, 1.946816E+03, 
   1.957303E+03, 1.967821E+03, 1.978370E+03, 1.988950E+03, 1.999561E+03, 2.010202E+03, 
   2.020875E+03, 2.031578E+03, 2.042312E+03, 2.053078E+03, 2.063874E+03, 2.074701E+03, 
   2.085560E+03, 2.096450E+03, 2.107371E+03, 2.118323E+03, 2.129306E+03, 2.140320E+03, 
   2.151366E+03, 2.162443E+03, 2.173551E+03, 2.184691E+03, 2.195862E+03, 2.207065E+03, 
   2.218299E+03, 2.229564E+03, 2.240861E+03, 2.252189E+03, 2.263549E+03, 2.274941E+03, 
   2.286364E+03, 2.297819E+03, 2.309305E+03, 2.320824E+03, 2.332374E+03, 2.343955E+03, 
   2.355569E+03, 2.367214E+03, 2.378891E+03, 2.390600E+03, 2.402340E+03, 2.414113E+03, 
   2.425918E+03, 2.437754E+03, 2.449623E+03, 2.461523E+03, 2.473456E+03, 2.485421E+03, 
   2.497418E+03, 2.509446E+03, 2.521507E+03, 2.533601E+03, 2.545726E+03, 2.557884E+03, 
   2.570074E+03, 2.582296E+03, 2.594550E+03, 2.606837E+03, 2.619156E+03, 2.631508E+03, 
   2.643891E+03, 2.656308E+03, 2.668756E+03, 2.681238E+03, 2.693751E+03, 2.706298E+03, 
   2.718877E+03, 2.731488E+03, 2.744132E+03, 2.756808E+03, 2.769518E+03, 2.782260E+03, 
   2.795034E+03, 2.807842E+03, 2.820682E+03, 2.833554E+03, 2.846460E+03, 2.859398E+03, 
   2.872370E+03, 2.885374E+03, 2.898411E+03, 2.911481E+03, 2.924584E+03, 2.937719E+03, 
   2.950888E+03, 2.964090E+03, 2.977325E+03, 2.990593E+03, 3.003893E+03, 3.017227E+03, 
   3.030594E+03, 3.043995E+03, 3.057428E+03, 3.070895E+03, 3.084394E+03, 3.097927E+03, 
   3.111494E+03, 3.125093E+03, 3.138726E+03, 3.152392E+03, 3.166091E+03, 3.179824E+03, 
   3.193590E+03, 3.207390E+03, 3.221223E+03, 3.235089E+03, 3.248989E+03, 3.262922E+03, 
   3.276889E+03, 3.290889E+03, 3.304923E+03, 3.318990E+03, 3.333091E+03, 3.347226E+03, 
   3.361394E+03, 3.375596E+03, 3.389831E+03, 3.404100E+03, 3.418403E+03, 3.432740E+03, 
   3.447110E+03, 3.461514E+03, 3.475952E+03, 3.490423E+03, 3.504929E+03, 3.519468E+03, 
   3.534041E+03, 3.548648E+03, 3.563288E+03, 3.577963E+03, 3.592672E+03, 3.607414E+03, 
   3.622191E+03, 3.637001E+03, 3.651846E+03, 3.666724E+03, 3.681637E+03, 3.696583E+03, 
   3.711564E+03, 3.726579E+03, 3.741627E+03, 3.756710E+03, 3.771827E+03, 3.786978E+03, 
   3.802164E+03, 3.817383E+03, 3.832637E+03, 3.847925E+03, 3.863247E+03, 3.878603E+03, 
   3.893994E+03, 3.909418E+03, 3.924878E+03, 3.940371E+03, 3.955899E+03, 3.971461E+03, 
   3.987057E+03, 4.002688E+03, 4.018353E+03, 4.034053E+03, 4.049787E+03, 4.065555E+03, 
   4.081358E+03, 4.097196E+03, 4.113067E+03, 4.128974E+03, 4.144914E+03, 4.160890E+03, 
   4.176899E+03, 4.192944E+03, 4.209023E+03, 4.225136E+03, 4.241284E+03, 4.257466E+03, 
   4.273684E+03, 4.289935E+03, 4.306222E+03, 4.322543E+03, 4.338898E+03, 4.355289E+03, 
   4.371714E+03, 4.388174E+03, 4.404668E+03, 4.421197E+03, 4.437761E+03, 4.454359E+03, 
   4.470993E+03, 4.487661E+03, 4.504364E+03, 4.521101E+03, 4.537874E+03, 4.554681E+03, 
   4.571523E+03, 4.588400E+03, 4.605311E+03, 4.622258E+03, 4.639239E+03, 4.656255E+03, 
   4.673306E+03, 4.690392E+03, 4.707513E+03, 4.724669E+03, 4.741860E+03, 4.759085E+03, 
   4.776346E+03, 4.793641E+03, 4.810971E+03, 4.828337E+03, 4.845737E+03, 4.863172E+03, 
   4.880642E+03, 4.898148E+03, 4.915688E+03, 4.933263E+03, 4.950873E+03, 4.968519E+03, 
   4.986199E+03, 5.003914E+03, 5.021664E+03, 5.039450E+03, 5.057270E+03, 5.075126E+03, 
   5.093016E+03, 5.110942E+03, 5.128903E+03, 5.146899E+03, 5.164930E+03, 5.182996E+03, 
   5.201097E+03, 5.219233E+03, 5.237404E+03, 5.255611E+03, 5.273852E+03, 5.292129E+03, 
   5.310441E+03, 5.328788E+03, 5.347170E+03, 5.365587E+03, 5.384040E+03, 5.402528E+03, 
   5.421050E+03, 5.439608E+03, 5.458201E+03, 5.476830E+03, 5.495493E+03, 5.514192E+03, 
   5.532926E+03, 5.551695E+03, 5.570499E+03, 5.589339E+03, 5.608213E+03, 5.627123E+03, 
   5.646068E+03, 5.665049E+03, 5.684064E+03, 5.703115E+03, 5.722201E+03, 5.741322E+03, 
   5.760479E+03, 5.779670E+03, 5.798897E+03, 5.818159E+03, 5.837457E+03, 5.856789E+03, 
   5.876157E+03, 5.895560E+03, 5.914998E+03, 5.934472E+03, 5.953981E+03, 5.973525E+03, 
   5.993104E+03, 6.012718E+03, 6.032368E+03, 6.052053E+03, 6.071773E+03, 6.091528E+03, 
   6.111319E+03, 6.131145E+03, 6.151006E+03, 6.170902E+03, 6.190834E+03, 6.210801E+03, 
   6.230803E+03, 6.250840E+03, 6.270913E+03, 6.291021E+03, 6.311164E+03, 6.331342E+03, 
   6.351555E+03, 6.371804E+03, 6.392088E+03, 6.412407E+03, 6.432761E+03, 6.453151E+03, 
   6.473575E+03, 6.494035E+03, 6.514531E+03, 6.535061E+03, 6.555626E+03, 6.576227E+03, 
   6.596863E+03, 6.617534E+03, 6.638241E+03, 6.658982E+03, 6.679759E+03, 6.700571E+03, 
   6.721418E+03, 6.742300E+03, 6.763217E+03, 6.784170E+03, 6.805157E+03, 6.826180E+03, 
   6.847238E+03, 6.868331E+03, 6.889459E+03, 6.910623E+03, 6.931821E+03, 6.953055E+03, 
   6.974323E+03, 6.995627E+03, 7.016966E+03, 7.038340E+03, 7.059749E+03, 7.081193E+03, 
   7.102672E+03, 
])
# ============================ M = 15, I = 2 ============================
M = 15
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.000000E+00, 9.200550E+00, 1.380884E+01, 1.896745E+01, 2.422364E+01, 2.951479E+01, 
   3.482280E+01, 4.014042E+01, 4.546420E+01, 5.079225E+01, 5.612347E+01, 6.145719E+01, 
   6.679294E+01, 7.213042E+01, 7.746940E+01, 8.280973E+01, 8.815129E+01, 9.349398E+01, 
   9.883774E+01, 1.041825E+02, 1.095283E+02, 1.148749E+02, 1.202225E+02, 1.255710E+02, 
   1.309203E+02, 1.362705E+02, 1.416215E+02, 1.469734E+02, 1.523260E+02, 1.576795E+02, 
   1.630338E+02, 1.683889E+02, 1.737449E+02, 1.791017E+02, 1.844594E+02, 1.898179E+02, 
   1.951774E+02, 2.005379E+02, 2.058993E+02, 2.112619E+02, 2.166256E+02, 2.219905E+02, 
   2.273568E+02, 2.327244E+02, 2.380936E+02, 2.434645E+02, 2.488371E+02, 2.542117E+02, 
   2.595884E+02, 2.649673E+02, 2.703487E+02, 2.757328E+02, 2.811197E+02, 2.865097E+02, 
   2.919031E+02, 2.973000E+02, 3.027006E+02, 3.081053E+02, 3.135144E+02, 3.189280E+02, 
   3.243465E+02, 3.297702E+02, 3.351994E+02, 3.406343E+02, 3.460752E+02, 3.515225E+02, 
   3.569765E+02, 3.624375E+02, 3.679058E+02, 3.733818E+02, 3.788657E+02, 3.843578E+02, 
   3.898586E+02, 3.953683E+02, 4.008872E+02, 4.064157E+02, 4.119542E+02, 4.175028E+02, 
   4.230620E+02, 4.286320E+02, 4.342132E+02, 4.398058E+02, 4.454103E+02, 4.510269E+02, 
   4.566559E+02, 4.622976E+02, 4.679523E+02, 4.736203E+02, 4.793019E+02, 4.849975E+02, 
   4.907071E+02, 4.964313E+02, 5.021701E+02, 5.079240E+02, 5.136932E+02, 5.194778E+02, 
   5.252783E+02, 5.310948E+02, 5.369277E+02, 5.427771E+02, 5.486432E+02, 5.545265E+02, 
   5.604270E+02, 5.663450E+02, 5.722807E+02, 5.782344E+02, 5.842062E+02, 5.901965E+02, 
   5.962054E+02, 6.022330E+02, 6.082797E+02, 6.143456E+02, 6.204309E+02, 6.265358E+02, 
   6.326605E+02, 6.388052E+02, 6.449701E+02, 6.511552E+02, 6.573609E+02, 6.635873E+02, 
   6.698345E+02, 6.761028E+02, 6.823922E+02, 6.887029E+02, 6.950352E+02, 7.013891E+02, 
   7.077648E+02, 7.141624E+02, 7.205821E+02, 7.270241E+02, 7.334884E+02, 7.399752E+02, 
   7.464847E+02, 7.530169E+02, 7.595720E+02, 7.661502E+02, 7.727515E+02, 7.793760E+02, 
   7.860240E+02, 7.926954E+02, 7.993905E+02, 8.061093E+02, 8.128519E+02, 8.196185E+02, 
   8.264092E+02, 8.332240E+02, 8.400631E+02, 8.469265E+02, 8.538144E+02, 8.607269E+02, 
   8.676641E+02, 8.746260E+02, 8.816127E+02, 8.886244E+02, 8.956611E+02, 9.027229E+02, 
   9.098099E+02, 9.169223E+02, 9.240599E+02, 9.312231E+02, 9.384118E+02, 9.456261E+02, 
   9.528660E+02, 9.601318E+02, 9.674234E+02, 9.747409E+02, 9.820845E+02, 9.894541E+02, 
   9.968498E+02, 1.004272E+03, 1.011720E+03, 1.019195E+03, 1.026696E+03, 1.034223E+03, 
   1.041777E+03, 1.049358E+03, 1.056965E+03, 1.064599E+03, 1.072260E+03, 1.079948E+03, 
   1.087662E+03, 1.095404E+03, 1.103173E+03, 1.110968E+03, 1.118791E+03, 1.126642E+03, 
   1.134519E+03, 1.142424E+03, 1.150356E+03, 1.158316E+03, 1.166303E+03, 1.174317E+03, 
   1.182360E+03, 1.190429E+03, 1.198527E+03, 1.206652E+03, 1.214806E+03, 1.222986E+03, 
   1.231195E+03, 1.239432E+03, 1.247697E+03, 1.255990E+03, 1.264311E+03, 1.272660E+03, 
   1.281037E+03, 1.289443E+03, 1.297877E+03, 1.306339E+03, 1.314829E+03, 1.323348E+03, 
   1.331896E+03, 1.340471E+03, 1.349076E+03, 1.357709E+03, 1.366370E+03, 1.375061E+03, 
   1.383780E+03, 1.392527E+03, 1.401304E+03, 1.410109E+03, 1.418943E+03, 1.427806E+03, 
   1.436699E+03, 1.445620E+03, 1.454570E+03, 1.463549E+03, 1.472557E+03, 1.481595E+03, 
   1.490661E+03, 1.499757E+03, 1.508882E+03, 1.518037E+03, 1.527220E+03, 1.536434E+03, 
   1.545676E+03, 1.554948E+03, 1.564249E+03, 1.573580E+03, 1.582941E+03, 1.592331E+03, 
   1.601751E+03, 1.611200E+03, 1.620679E+03, 1.630188E+03, 1.639727E+03, 1.649295E+03, 
   1.658894E+03, 1.668522E+03, 1.678180E+03, 1.687868E+03, 1.697586E+03, 1.707334E+03, 
   1.717112E+03, 1.726920E+03, 1.736758E+03, 1.746626E+03, 1.756525E+03, 1.766453E+03, 
   1.776412E+03, 1.786402E+03, 1.796421E+03, 1.806471E+03, 1.816551E+03, 1.826662E+03, 
   1.836803E+03, 1.846974E+03, 1.857176E+03, 1.867409E+03, 1.877672E+03, 1.887966E+03, 
   1.898290E+03, 1.908645E+03, 1.919031E+03, 1.929447E+03, 1.939894E+03, 1.950372E+03, 
   1.960881E+03, 1.971421E+03, 1.981991E+03, 1.992592E+03, 2.003225E+03, 2.013888E+03, 
   2.024582E+03, 2.035307E+03, 2.046063E+03, 2.056851E+03, 2.067669E+03, 2.078519E+03, 
   2.089399E+03, 2.100311E+03, 2.111255E+03, 2.122229E+03, 2.133235E+03, 2.144272E+03, 
   2.155340E+03, 2.166440E+03, 2.177571E+03, 2.188733E+03, 2.199928E+03, 2.211153E+03, 
   2.222410E+03, 2.233699E+03, 2.245019E+03, 2.256370E+03, 2.267754E+03, 2.279169E+03, 
   2.290615E+03, 2.302094E+03, 2.313604E+03, 2.325146E+03, 2.336719E+03, 2.348325E+03, 
   2.359962E+03, 2.371631E+03, 2.383332E+03, 2.395065E+03, 2.406830E+03, 2.418627E+03, 
   2.430456E+03, 2.442317E+03, 2.454210E+03, 2.466135E+03, 2.478092E+03, 2.490082E+03, 
   2.502103E+03, 2.514157E+03, 2.526243E+03, 2.538361E+03, 2.550512E+03, 2.562694E+03, 
   2.574909E+03, 2.587157E+03, 2.599436E+03, 2.611749E+03, 2.624093E+03, 2.636470E+03, 
   2.648880E+03, 2.661322E+03, 2.673796E+03, 2.686303E+03, 2.698843E+03, 2.711415E+03, 
   2.724020E+03, 2.736657E+03, 2.749328E+03, 2.762031E+03, 2.774766E+03, 2.787534E+03, 
   2.800335E+03, 2.813169E+03, 2.826036E+03, 2.838936E+03, 2.851868E+03, 2.864833E+03, 
   2.877831E+03, 2.890862E+03, 2.903926E+03, 2.917024E+03, 2.930154E+03, 2.943317E+03, 
   2.956513E+03, 2.969742E+03, 2.983004E+03, 2.996300E+03, 3.009628E+03, 3.022990E+03, 
   3.036385E+03, 3.049813E+03, 3.063274E+03, 3.076768E+03, 3.090296E+03, 3.103857E+03, 
   3.117452E+03, 3.131080E+03, 3.144741E+03, 3.158435E+03, 3.172163E+03, 3.185924E+03, 
   3.199719E+03, 3.213547E+03, 3.227409E+03, 3.241304E+03, 3.255233E+03, 3.269195E+03, 
   3.283191E+03, 3.297220E+03, 3.311284E+03, 3.325380E+03, 3.339510E+03, 3.353674E+03, 
   3.367872E+03, 3.382104E+03, 3.396369E+03, 3.410667E+03, 3.425000E+03, 3.439366E+03, 
   3.453767E+03, 3.468201E+03, 3.482669E+03, 3.497170E+03, 3.511706E+03, 3.526275E+03, 
   3.540879E+03, 3.555516E+03, 3.570188E+03, 3.584893E+03, 3.599632E+03, 3.614405E+03, 
   3.629213E+03, 3.644054E+03, 3.658929E+03, 3.673839E+03, 3.688783E+03, 3.703760E+03, 
   3.718772E+03, 3.733818E+03, 3.748898E+03, 3.764013E+03, 3.779161E+03, 3.794344E+03, 
   3.809561E+03, 3.824812E+03, 3.840098E+03, 3.855417E+03, 3.870771E+03, 3.886160E+03, 
   3.901583E+03, 3.917040E+03, 3.932531E+03, 3.948057E+03, 3.963617E+03, 3.979212E+03, 
   3.994841E+03, 4.010504E+03, 4.026202E+03, 4.041934E+03, 4.057701E+03, 4.073502E+03, 
   4.089338E+03, 4.105209E+03, 4.121113E+03, 4.137053E+03, 4.153027E+03, 4.169036E+03, 
   4.185079E+03, 4.201157E+03, 4.217269E+03, 4.233416E+03, 4.249598E+03, 4.265814E+03, 
   4.282065E+03, 4.298351E+03, 4.314671E+03, 4.331026E+03, 4.347416E+03, 4.363841E+03, 
   4.380300E+03, 4.396794E+03, 4.413323E+03, 4.429886E+03, 4.446485E+03, 4.463118E+03, 
   4.479786E+03, 4.496489E+03, 4.513226E+03, 4.529999E+03, 4.546806E+03, 4.563649E+03, 
   4.580526E+03, 4.597438E+03, 4.614385E+03, 4.631367E+03, 4.648383E+03, 4.665435E+03, 
   4.682522E+03, 4.699643E+03, 4.716800E+03, 4.733991E+03, 4.751218E+03, 4.768479E+03, 
   4.785776E+03, 4.803107E+03, 4.820474E+03, 4.837875E+03, 4.855312E+03, 4.872783E+03, 
   4.890290E+03, 4.907832E+03, 4.925409E+03, 4.943020E+03, 4.960667E+03, 4.978349E+03, 
   4.996067E+03, 5.013819E+03, 5.031606E+03, 5.049429E+03, 5.067286E+03, 5.085179E+03, 
   5.103107E+03, 5.121070E+03, 5.139068E+03, 5.157101E+03, 5.175170E+03, 5.193273E+03, 
   5.211412E+03, 5.229586E+03, 5.247795E+03, 5.266040E+03, 5.284319E+03, 5.302634E+03, 
   5.320984E+03, 5.339369E+03, 5.357790E+03, 5.376245E+03, 5.394736E+03, 5.413262E+03, 
   5.431824E+03, 5.450420E+03, 5.469052E+03, 5.487719E+03, 5.506421E+03, 5.525159E+03, 
   5.543932E+03, 5.562740E+03, 5.581583E+03, 5.600462E+03, 5.619376E+03, 5.638325E+03, 
   5.657310E+03, 5.676330E+03, 5.695385E+03, 5.714475E+03, 5.733601E+03, 5.752762E+03, 
   5.771958E+03, 5.791189E+03, 5.810456E+03, 5.829758E+03, 5.849096E+03, 5.868468E+03, 
   5.887876E+03, 5.907320E+03, 5.926798E+03, 5.946312E+03, 5.965862E+03, 5.985446E+03, 
   6.005066E+03, 6.024721E+03, 6.044412E+03, 6.064137E+03, 6.083898E+03, 6.103695E+03, 
   6.123527E+03, 6.143394E+03, 6.163296E+03, 6.183234E+03, 6.203206E+03, 6.223215E+03, 
   6.243258E+03, 6.263337E+03, 6.283451E+03, 6.303601E+03, 6.323785E+03, 6.344005E+03, 
   6.364261E+03, 6.384551E+03, 6.404877E+03, 6.425238E+03, 6.445635E+03, 6.466067E+03, 
   6.486534E+03, 6.507036E+03, 6.527574E+03, 6.548146E+03, 6.568755E+03, 6.589398E+03, 
   6.610077E+03, 6.630791E+03, 6.651540E+03, 6.672324E+03, 6.693144E+03, 6.713999E+03, 
   6.734889E+03, 6.755814E+03, 6.776775E+03, 6.797771E+03, 6.818802E+03, 6.839868E+03, 
   6.860969E+03, 6.882106E+03, 6.903278E+03, 6.924485E+03, 6.945727E+03, 6.967005E+03, 
   6.988317E+03, 7.009665E+03, 7.031048E+03, 7.052466E+03, 7.073919E+03, 7.095408E+03, 
   7.116931E+03, 
])
# ============================ M = 15, I = 3 ============================
M = 15
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.200001E+01, 2.020781E+01, 3.527472E+01, 5.062917E+01, 6.604716E+01, 8.149058E+01, 
   9.694745E+01, 1.124128E+02, 1.278841E+02, 1.433602E+02, 1.588401E+02, 1.743234E+02, 
   1.898098E+02, 2.052990E+02, 2.207909E+02, 2.362853E+02, 2.517823E+02, 2.672815E+02, 
   2.827832E+02, 2.982872E+02, 3.137934E+02, 3.293020E+02, 3.448129E+02, 3.603261E+02, 
   3.758418E+02, 3.913601E+02, 4.068811E+02, 4.224050E+02, 4.379322E+02, 4.534629E+02, 
   4.689977E+02, 4.845369E+02, 5.000812E+02, 5.156314E+02, 5.311880E+02, 5.467521E+02, 
   5.623246E+02, 5.779064E+02, 5.934988E+02, 6.091029E+02, 6.247200E+02, 6.403514E+02, 
   6.559986E+02, 6.716631E+02, 6.873464E+02, 7.030500E+02, 7.187757E+02, 7.345250E+02, 
   7.502996E+02, 7.661013E+02, 7.819318E+02, 7.977929E+02, 8.136862E+02, 8.296136E+02, 
   8.455769E+02, 8.615777E+02, 8.776178E+02, 8.936991E+02, 9.098230E+02, 9.259915E+02, 
   9.422062E+02, 9.584688E+02, 9.747808E+02, 9.911439E+02, 1.007560E+03, 1.024030E+03, 
   1.040555E+03, 1.057138E+03, 1.073780E+03, 1.090482E+03, 1.107246E+03, 1.124072E+03, 
   1.140963E+03, 1.157919E+03, 1.174943E+03, 1.192034E+03, 1.209195E+03, 1.226426E+03, 
   1.243729E+03, 1.261105E+03, 1.278555E+03, 1.296079E+03, 1.313680E+03, 1.331358E+03, 
   1.349113E+03, 1.366948E+03, 1.384862E+03, 1.402858E+03, 1.420935E+03, 1.439094E+03, 
   1.457337E+03, 1.475664E+03, 1.494076E+03, 1.512573E+03, 1.531157E+03, 1.549829E+03, 
   1.568588E+03, 1.587435E+03, 1.606372E+03, 1.625399E+03, 1.644516E+03, 1.663725E+03, 
   1.683025E+03, 1.702417E+03, 1.721902E+03, 1.741481E+03, 1.761153E+03, 1.780920E+03, 
   1.800782E+03, 1.820739E+03, 1.840792E+03, 1.860942E+03, 1.881188E+03, 1.901532E+03, 
   1.921974E+03, 1.942513E+03, 1.963151E+03, 1.983888E+03, 2.004725E+03, 2.025661E+03, 
   2.046697E+03, 2.067834E+03, 2.089072E+03, 2.110411E+03, 2.131851E+03, 2.153393E+03, 
   2.175038E+03, 2.196785E+03, 2.218634E+03, 2.240587E+03, 2.262643E+03, 2.284803E+03, 
   2.307067E+03, 2.329436E+03, 2.351908E+03, 2.374486E+03, 2.397169E+03, 2.419957E+03, 
   2.442851E+03, 2.465850E+03, 2.488956E+03, 2.512168E+03, 2.535486E+03, 2.558912E+03, 
   2.582444E+03, 2.606084E+03, 2.629831E+03, 2.653685E+03, 2.677648E+03, 2.701719E+03, 
   2.725898E+03, 2.750186E+03, 2.774582E+03, 2.799087E+03, 2.823702E+03, 2.848426E+03, 
   2.873259E+03, 2.898202E+03, 2.923254E+03, 2.948417E+03, 2.973690E+03, 2.999074E+03, 
   3.024567E+03, 3.050172E+03, 3.075888E+03, 3.101714E+03, 3.127652E+03, 3.153702E+03, 
   3.179862E+03, 3.206135E+03, 3.232519E+03, 3.259016E+03, 3.285625E+03, 3.312346E+03, 
   3.339179E+03, 3.366125E+03, 3.393184E+03, 3.420356E+03, 3.447641E+03, 3.475039E+03, 
   3.502550E+03, 3.530175E+03, 3.557914E+03, 3.585766E+03, 3.613733E+03, 3.641813E+03, 
   3.670008E+03, 3.698316E+03, 3.726740E+03, 3.755277E+03, 3.783930E+03, 3.812697E+03, 
   3.841580E+03, 3.870577E+03, 3.899690E+03, 3.928918E+03, 3.958261E+03, 3.987720E+03, 
   4.017295E+03, 4.046985E+03, 4.076791E+03, 4.106714E+03, 4.136753E+03, 4.166908E+03, 
   4.197179E+03, 4.227567E+03, 4.258071E+03, 4.288693E+03, 4.319431E+03, 4.350286E+03, 
   4.381258E+03, 4.412348E+03, 4.443554E+03, 4.474878E+03, 4.506320E+03, 4.537880E+03, 
   4.569557E+03, 4.601352E+03, 4.633264E+03, 4.665295E+03, 4.697444E+03, 4.729712E+03, 
   4.762098E+03, 4.794602E+03, 4.827225E+03, 4.859966E+03, 4.892827E+03, 4.925806E+03, 
   4.958904E+03, 4.992121E+03, 5.025457E+03, 5.058913E+03, 5.092488E+03, 5.126183E+03, 
   5.159997E+03, 5.193930E+03, 5.227984E+03, 5.262157E+03, 5.296451E+03, 5.330864E+03, 
   5.365397E+03, 5.400051E+03, 5.434825E+03, 5.469719E+03, 5.504734E+03, 5.539869E+03, 
   5.575125E+03, 5.610502E+03, 5.646000E+03, 5.681618E+03, 5.717358E+03, 5.753218E+03, 
   5.789200E+03, 5.825303E+03, 5.861527E+03, 5.897873E+03, 5.934340E+03, 5.970929E+03, 
   6.007640E+03, 6.044472E+03, 6.081426E+03, 6.118502E+03, 6.155700E+03, 6.193020E+03, 
   6.230462E+03, 6.268026E+03, 6.305713E+03, 6.343522E+03, 6.381453E+03, 6.419507E+03, 
   6.457684E+03, 6.495983E+03, 6.534405E+03, 6.572949E+03, 6.611617E+03, 6.650407E+03, 
   6.689320E+03, 6.728357E+03, 6.767516E+03, 6.806799E+03, 6.846205E+03, 6.885734E+03, 
   6.925387E+03, 6.965163E+03, 7.005063E+03, 7.045086E+03, 7.085233E+03, 7.125503E+03, 
   7.165898E+03, 7.206416E+03, 7.247058E+03, 7.287824E+03, 7.328713E+03, 7.369727E+03, 
   7.410865E+03, 7.452128E+03, 7.493514E+03, 7.535025E+03, 7.576660E+03, 7.618419E+03, 
   7.660303E+03, 7.702311E+03, 7.744444E+03, 7.786701E+03, 7.829083E+03, 7.871590E+03, 
   7.914221E+03, 7.956977E+03, 7.999858E+03, 8.042863E+03, 8.085994E+03, 8.129249E+03, 
   8.172630E+03, 8.216135E+03, 8.259766E+03, 8.303522E+03, 8.347402E+03, 8.391408E+03, 
   8.435540E+03, 8.479796E+03, 8.524178E+03, 8.568685E+03, 8.613317E+03, 8.658075E+03, 
   8.702958E+03, 8.747967E+03, 8.793101E+03, 8.838360E+03, 8.883746E+03, 8.929256E+03, 
   8.974893E+03, 9.020655E+03, 9.066542E+03, 9.112556E+03, 9.158695E+03, 9.204960E+03, 
   9.251350E+03, 9.297866E+03, 9.344509E+03, 9.391277E+03, 9.438170E+03, 9.485190E+03, 
   9.532336E+03, 9.579607E+03, 9.627005E+03, 9.674528E+03, 9.722177E+03, 9.769952E+03, 
   9.817854E+03, 9.865881E+03, 9.914034E+03, 9.962314E+03, 1.001072E+04, 1.005925E+04, 
   1.010791E+04, 1.015669E+04, 1.020560E+04, 1.025464E+04, 1.030380E+04, 1.035309E+04, 
   1.040250E+04, 1.045204E+04, 1.050171E+04, 1.055150E+04, 1.060142E+04, 1.065146E+04, 
   1.070163E+04, 1.075193E+04, 1.080235E+04, 1.085290E+04, 1.090358E+04, 1.095438E+04, 
   1.100530E+04, 1.105636E+04, 1.110754E+04, 1.115884E+04, 1.121027E+04, 1.126183E+04, 
   1.131352E+04, 1.136533E+04, 1.141726E+04, 1.146932E+04, 1.152151E+04, 1.157383E+04, 
   1.162627E+04, 1.167883E+04, 1.173152E+04, 1.178434E+04, 1.183728E+04, 1.189035E+04, 
   1.194355E+04, 1.199687E+04, 1.205032E+04, 1.210389E+04, 1.215759E+04, 1.221142E+04, 
   1.226537E+04, 1.231944E+04, 1.237364E+04, 1.242797E+04, 1.248242E+04, 1.253700E+04, 
   1.259171E+04, 1.264654E+04, 1.270149E+04, 1.275658E+04, 1.281178E+04, 1.286711E+04, 
   1.292257E+04, 1.297816E+04, 1.303386E+04, 1.308970E+04, 1.314566E+04, 1.320174E+04, 
   1.325795E+04, 1.331429E+04, 1.337075E+04, 1.342733E+04, 1.348404E+04, 1.354088E+04, 
   1.359784E+04, 1.365492E+04, 1.371213E+04, 1.376947E+04, 1.382693E+04, 1.388451E+04, 
   1.394222E+04, 1.400006E+04, 1.405802E+04, 1.411610E+04, 1.417431E+04, 1.423264E+04, 
   1.429110E+04, 1.434968E+04, 1.440838E+04, 1.446721E+04, 1.452617E+04, 1.458525E+04, 
   1.464445E+04, 1.470378E+04, 1.476323E+04, 1.482280E+04, 1.488250E+04, 1.494232E+04, 
   1.500227E+04, 1.506234E+04, 1.512254E+04, 1.518285E+04, 1.524329E+04, 1.530386E+04, 
   1.536455E+04, 1.542536E+04, 1.548629E+04, 1.554735E+04, 1.560853E+04, 1.566984E+04, 
   1.573127E+04, 1.579282E+04, 1.585449E+04, 1.591629E+04, 1.597821E+04, 1.604025E+04, 
   1.610241E+04, 1.616470E+04, 1.622711E+04, 1.628964E+04, 1.635230E+04, 1.641508E+04, 
   1.647798E+04, 1.654100E+04, 1.660414E+04, 1.666741E+04, 1.673080E+04, 1.679430E+04, 
   1.685794E+04, 1.692169E+04, 1.698556E+04, 1.704956E+04, 1.711368E+04, 1.717792E+04, 
   1.724228E+04, 1.730676E+04, 1.737136E+04, 1.743609E+04, 1.750093E+04, 1.756590E+04, 
   1.763099E+04, 1.769619E+04, 1.776152E+04, 1.782697E+04, 1.789254E+04, 1.795823E+04, 
   1.802404E+04, 1.808997E+04, 1.815602E+04, 1.822219E+04, 1.828848E+04, 1.835489E+04, 
   1.842142E+04, 1.848807E+04, 1.855484E+04, 1.862173E+04, 1.868874E+04, 1.875586E+04, 
   1.882311E+04, 1.889048E+04, 1.895796E+04, 1.902556E+04, 1.909329E+04, 1.916113E+04, 
   1.922909E+04, 1.929717E+04, 1.936536E+04, 1.943368E+04, 1.950211E+04, 1.957066E+04, 
   1.963933E+04, 1.970812E+04, 1.977703E+04, 1.984605E+04, 1.991519E+04, 1.998445E+04, 
   2.005382E+04, 2.012332E+04, 2.019293E+04, 2.026265E+04, 2.033250E+04, 2.040246E+04, 
   2.047254E+04, 2.054273E+04, 2.061304E+04, 2.068347E+04, 2.075401E+04, 2.082467E+04, 
   2.089545E+04, 2.096634E+04, 2.103735E+04, 2.110847E+04, 2.117971E+04, 2.125107E+04, 
   2.132254E+04, 2.139412E+04, 2.146582E+04, 2.153764E+04, 2.160957E+04, 2.168161E+04, 
   2.175378E+04, 2.182605E+04, 2.189844E+04, 2.197094E+04, 2.204356E+04, 2.211629E+04, 
   2.218914E+04, 2.226210E+04, 2.233518E+04, 2.240836E+04, 2.248166E+04, 2.255508E+04, 
   2.262861E+04, 2.270225E+04, 2.277600E+04, 2.284987E+04, 2.292385E+04, 2.299794E+04, 
   2.307215E+04, 2.314647E+04, 2.322090E+04, 2.329544E+04, 2.337009E+04, 2.344486E+04, 
   2.351974E+04, 2.359473E+04, 2.366983E+04, 2.374504E+04, 2.382036E+04, 2.389580E+04, 
   2.397134E+04, 2.404700E+04, 2.412277E+04, 2.419865E+04, 2.427464E+04, 2.435074E+04, 
   2.442694E+04, 2.450326E+04, 2.457969E+04, 2.465623E+04, 2.473288E+04, 2.480964E+04, 
   2.488651E+04, 2.496349E+04, 2.504057E+04, 2.511777E+04, 2.519507E+04, 2.527248E+04, 
   2.535001E+04, 2.542764E+04, 2.550537E+04, 2.558322E+04, 2.566118E+04, 2.573924E+04, 
   2.581741E+04, 
])
# ============================ M = 15, I = 4 ============================
M = 15
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.200001E+01, 2.025058E+01, 3.536447E+01, 5.076473E+01, 6.622832E+01, 8.171728E+01, 
   9.721965E+01, 1.127305E+02, 1.282473E+02, 1.437688E+02, 1.592942E+02, 1.748230E+02, 
   1.903549E+02, 2.058896E+02, 2.214270E+02, 2.369669E+02, 2.525093E+02, 2.680541E+02, 
   2.836013E+02, 2.991508E+02, 3.147026E+02, 3.302567E+02, 3.458131E+02, 3.613719E+02, 
   3.769332E+02, 3.924970E+02, 4.080636E+02, 4.236331E+02, 4.392059E+02, 4.547823E+02, 
   4.703627E+02, 4.859477E+02, 5.015378E+02, 5.171338E+02, 5.327363E+02, 5.483464E+02, 
   5.639649E+02, 5.795929E+02, 5.952315E+02, 6.108820E+02, 6.265456E+02, 6.422237E+02, 
   6.579177E+02, 6.736291E+02, 6.893595E+02, 7.051104E+02, 7.208836E+02, 7.366806E+02, 
   7.525032E+02, 7.683531E+02, 7.842320E+02, 8.001417E+02, 8.160840E+02, 8.320607E+02, 
   8.480734E+02, 8.641240E+02, 8.802143E+02, 8.963459E+02, 9.125207E+02, 9.287402E+02, 
   9.450063E+02, 9.613206E+02, 9.776848E+02, 9.941003E+02, 1.010569E+03, 1.027092E+03, 
   1.043672E+03, 1.060309E+03, 1.077005E+03, 1.093761E+03, 1.110580E+03, 1.127462E+03, 
   1.144409E+03, 1.161422E+03, 1.178502E+03, 1.195651E+03, 1.212869E+03, 1.230158E+03, 
   1.247520E+03, 1.264955E+03, 1.282464E+03, 1.300048E+03, 1.317709E+03, 1.335447E+03, 
   1.353264E+03, 1.371160E+03, 1.389137E+03, 1.407195E+03, 1.425335E+03, 1.443557E+03, 
   1.461864E+03, 1.480255E+03, 1.498732E+03, 1.517294E+03, 1.535944E+03, 1.554681E+03, 
   1.573507E+03, 1.592421E+03, 1.611425E+03, 1.630520E+03, 1.649705E+03, 1.668982E+03, 
   1.688351E+03, 1.707812E+03, 1.727367E+03, 1.747016E+03, 1.766759E+03, 1.786596E+03, 
   1.806529E+03, 1.826558E+03, 1.846683E+03, 1.866905E+03, 1.887223E+03, 1.907640E+03, 
   1.928154E+03, 1.948767E+03, 1.969478E+03, 1.990289E+03, 2.011199E+03, 2.032209E+03, 
   2.053320E+03, 2.074531E+03, 2.095842E+03, 2.117256E+03, 2.138770E+03, 2.160387E+03, 
   2.182105E+03, 2.203926E+03, 2.225850E+03, 2.247877E+03, 2.270007E+03, 2.292240E+03, 
   2.314578E+03, 2.337019E+03, 2.359564E+03, 2.382214E+03, 2.404969E+03, 2.427828E+03, 
   2.450792E+03, 2.473862E+03, 2.497037E+03, 2.520318E+03, 2.543704E+03, 2.567197E+03, 
   2.590795E+03, 2.614500E+03, 2.638311E+03, 2.662229E+03, 2.686254E+03, 2.710385E+03, 
   2.734623E+03, 2.758969E+03, 2.783422E+03, 2.807982E+03, 2.832649E+03, 2.857424E+03, 
   2.882307E+03, 2.907297E+03, 2.932396E+03, 2.957602E+03, 2.982916E+03, 3.008338E+03, 
   3.033868E+03, 3.059507E+03, 3.085254E+03, 3.111109E+03, 3.137072E+03, 3.163144E+03, 
   3.189325E+03, 3.215614E+03, 3.242011E+03, 3.268517E+03, 3.295131E+03, 3.321855E+03, 
   3.348686E+03, 3.375627E+03, 3.402676E+03, 3.429834E+03, 3.457100E+03, 3.484475E+03, 
   3.511959E+03, 3.539552E+03, 3.567253E+03, 3.595063E+03, 3.622981E+03, 3.651008E+03, 
   3.679143E+03, 3.707388E+03, 3.735740E+03, 3.764201E+03, 3.792771E+03, 3.821449E+03, 
   3.850235E+03, 3.879130E+03, 3.908133E+03, 3.937244E+03, 3.966464E+03, 3.995791E+03, 
   4.025226E+03, 4.054770E+03, 4.084421E+03, 4.114181E+03, 4.144048E+03, 4.174022E+03, 
   4.204105E+03, 4.234294E+03, 4.264592E+03, 4.294996E+03, 4.325508E+03, 4.356128E+03, 
   4.386854E+03, 4.417687E+03, 4.448627E+03, 4.479674E+03, 4.510828E+03, 4.542088E+03, 
   4.573454E+03, 4.604928E+03, 4.636507E+03, 4.668192E+03, 4.699984E+03, 4.731881E+03, 
   4.763884E+03, 4.795993E+03, 4.828207E+03, 4.860527E+03, 4.892952E+03, 4.925482E+03, 
   4.958117E+03, 4.990856E+03, 5.023701E+03, 5.056650E+03, 5.089704E+03, 5.122862E+03, 
   5.156124E+03, 5.189489E+03, 5.222959E+03, 5.256533E+03, 5.290210E+03, 5.323990E+03, 
   5.357874E+03, 5.391860E+03, 5.425950E+03, 5.460142E+03, 5.494437E+03, 5.528834E+03, 
   5.563333E+03, 5.597934E+03, 5.632638E+03, 5.667443E+03, 5.702349E+03, 5.737357E+03, 
   5.772466E+03, 5.807676E+03, 5.842987E+03, 5.878398E+03, 5.913910E+03, 5.949522E+03, 
   5.985234E+03, 6.021046E+03, 6.056957E+03, 6.092968E+03, 6.129079E+03, 6.165288E+03, 
   6.201596E+03, 6.238003E+03, 6.274509E+03, 6.311112E+03, 6.347814E+03, 6.384614E+03, 
   6.421511E+03, 6.458506E+03, 6.495598E+03, 6.532787E+03, 6.570073E+03, 6.607456E+03, 
   6.644935E+03, 6.682510E+03, 6.720182E+03, 6.757949E+03, 6.795811E+03, 6.833769E+03, 
   6.871822E+03, 6.909970E+03, 6.948213E+03, 6.986550E+03, 7.024981E+03, 7.063506E+03, 
   7.102125E+03, 7.140838E+03, 7.179643E+03, 7.218542E+03, 7.257534E+03, 7.296619E+03, 
   7.335796E+03, 7.375065E+03, 7.414426E+03, 7.453878E+03, 7.493422E+03, 7.533058E+03, 
   7.572784E+03, 7.612601E+03, 7.652509E+03, 7.692506E+03, 7.732594E+03, 7.772772E+03, 
   7.813039E+03, 7.853396E+03, 7.893841E+03, 7.934376E+03, 7.974999E+03, 8.015710E+03, 
   8.056509E+03, 8.097396E+03, 8.138371E+03, 8.179433E+03, 8.220583E+03, 8.261819E+03, 
   8.303141E+03, 8.344550E+03, 8.386045E+03, 8.427626E+03, 8.469292E+03, 8.511044E+03, 
   8.552881E+03, 8.594802E+03, 8.636808E+03, 8.678899E+03, 8.721073E+03, 8.763332E+03, 
   8.805673E+03, 8.848098E+03, 8.890606E+03, 8.933197E+03, 8.975870E+03, 9.018626E+03, 
   9.061463E+03, 9.104383E+03, 9.147383E+03, 9.190465E+03, 9.233628E+03, 9.276871E+03, 
   9.320195E+03, 9.363599E+03, 9.407082E+03, 9.450646E+03, 9.494288E+03, 9.538010E+03, 
   9.581810E+03, 9.625690E+03, 9.669647E+03, 9.713682E+03, 9.757795E+03, 9.801986E+03, 
   9.846253E+03, 9.890598E+03, 9.935019E+03, 9.979517E+03, 1.002409E+04, 1.006874E+04, 
   1.011347E+04, 1.015827E+04, 1.020314E+04, 1.024809E+04, 1.029312E+04, 1.033822E+04, 
   1.038339E+04, 1.042864E+04, 1.047396E+04, 1.051935E+04, 1.056482E+04, 1.061035E+04, 
   1.065597E+04, 1.070165E+04, 1.074740E+04, 1.079323E+04, 1.083913E+04, 1.088510E+04, 
   1.093114E+04, 1.097725E+04, 1.102343E+04, 1.106968E+04, 1.111600E+04, 1.116239E+04, 
   1.120884E+04, 1.125537E+04, 1.130197E+04, 1.134863E+04, 1.139536E+04, 1.144216E+04, 
   1.148903E+04, 1.153596E+04, 1.158296E+04, 1.163003E+04, 1.167716E+04, 1.172436E+04, 
   1.177163E+04, 1.181896E+04, 1.186635E+04, 1.191381E+04, 1.196134E+04, 1.200893E+04, 
   1.205658E+04, 1.210430E+04, 1.215208E+04, 1.219993E+04, 1.224784E+04, 1.229581E+04, 
   1.234384E+04, 1.239194E+04, 1.244010E+04, 1.248832E+04, 1.253660E+04, 1.258494E+04, 
   1.263334E+04, 1.268180E+04, 1.273033E+04, 1.277891E+04, 1.282756E+04, 1.287626E+04, 
   1.292502E+04, 1.297384E+04, 1.302272E+04, 1.307166E+04, 1.312066E+04, 1.316971E+04, 
   1.321883E+04, 1.326800E+04, 1.331722E+04, 1.336651E+04, 1.341585E+04, 1.346524E+04, 
   1.351469E+04, 1.356420E+04, 1.361377E+04, 1.366338E+04, 1.371306E+04, 1.376279E+04, 
   1.381257E+04, 1.386241E+04, 1.391230E+04, 1.396224E+04, 1.401224E+04, 1.406229E+04, 
   1.411240E+04, 1.416255E+04, 1.421276E+04, 1.426302E+04, 1.431333E+04, 1.436370E+04, 
   1.441411E+04, 1.446458E+04, 1.451509E+04, 1.456566E+04, 1.461628E+04, 1.466695E+04, 
   1.471766E+04, 1.476843E+04, 1.481925E+04, 1.487011E+04, 1.492102E+04, 1.497198E+04, 
   1.502299E+04, 1.507405E+04, 1.512516E+04, 1.517631E+04, 1.522751E+04, 1.527875E+04, 
   1.533005E+04, 1.538139E+04, 1.543277E+04, 1.548420E+04, 1.553568E+04, 1.558720E+04, 
   1.563877E+04, 1.569038E+04, 1.574203E+04, 1.579373E+04, 1.584548E+04, 1.589727E+04, 
   1.594910E+04, 1.600097E+04, 1.605289E+04, 1.610485E+04, 1.615685E+04, 1.620890E+04, 
   1.626099E+04, 1.631311E+04, 1.636528E+04, 1.641750E+04, 1.646975E+04, 1.652204E+04, 
   1.657438E+04, 1.662675E+04, 1.667916E+04, 1.673162E+04, 1.678411E+04, 1.683665E+04, 
   1.688922E+04, 1.694183E+04, 1.699448E+04, 1.704716E+04, 1.709989E+04, 1.715265E+04, 
   1.720545E+04, 1.725829E+04, 1.731117E+04, 1.736408E+04, 1.741703E+04, 1.747002E+04, 
   1.752304E+04, 1.757609E+04, 1.762919E+04, 1.768232E+04, 1.773548E+04, 1.778868E+04, 
   1.784191E+04, 1.789518E+04, 1.794848E+04, 1.800182E+04, 1.805519E+04, 1.810859E+04, 
   1.816203E+04, 1.821550E+04, 1.826900E+04, 1.832254E+04, 1.837611E+04, 1.842971E+04, 
   1.848334E+04, 1.853700E+04, 1.859070E+04, 1.864443E+04, 1.869818E+04, 1.875197E+04, 
   1.880579E+04, 1.885964E+04, 1.891352E+04, 1.896743E+04, 1.902137E+04, 1.907534E+04, 
   1.912934E+04, 1.918336E+04, 1.923742E+04, 1.929150E+04, 1.934562E+04, 1.939976E+04, 
   1.945393E+04, 1.950812E+04, 1.956235E+04, 1.961660E+04, 1.967088E+04, 1.972518E+04, 
   1.977952E+04, 1.983387E+04, 1.988826E+04, 1.994267E+04, 1.999711E+04, 2.005157E+04, 
   2.010606E+04, 2.016057E+04, 2.021511E+04, 2.026967E+04, 2.032426E+04, 2.037887E+04, 
   2.043350E+04, 2.048816E+04, 2.054284E+04, 2.059755E+04, 2.065228E+04, 2.070703E+04, 
   2.076181E+04, 2.081661E+04, 2.087143E+04, 2.092627E+04, 2.098113E+04, 2.103602E+04, 
   2.109093E+04, 2.114586E+04, 2.120081E+04, 2.125578E+04, 2.131078E+04, 2.136579E+04, 
   2.142082E+04, 2.147588E+04, 2.153095E+04, 2.158605E+04, 2.164116E+04, 2.169629E+04, 
   2.175144E+04, 2.180662E+04, 2.186181E+04, 2.191702E+04, 2.197224E+04, 2.202749E+04, 
   2.208275E+04, 2.213804E+04, 2.219333E+04, 2.224865E+04, 2.230399E+04, 2.235934E+04, 
   2.241471E+04, 
])
# ============================ M = 16, I = 1 ============================
M = 16
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.000000E+00, 1.020069E+01, 1.634928E+01, 2.287719E+01, 2.947690E+01, 3.610348E+01, 
   4.274338E+01, 4.939110E+01, 5.604397E+01, 6.270053E+01, 6.935994E+01, 7.602166E+01, 
   8.268533E+01, 8.935071E+01, 9.601763E+01, 1.026860E+02, 1.093556E+02, 1.160265E+02, 
   1.226986E+02, 1.293719E+02, 1.360462E+02, 1.427217E+02, 1.493982E+02, 1.560758E+02, 
   1.627544E+02, 1.694341E+02, 1.761147E+02, 1.827964E+02, 1.894792E+02, 1.961631E+02, 
   2.028480E+02, 2.095341E+02, 2.162214E+02, 2.229099E+02, 2.295997E+02, 2.362911E+02, 
   2.429839E+02, 2.496784E+02, 2.563748E+02, 2.630732E+02, 2.697738E+02, 2.764769E+02, 
   2.831826E+02, 2.898912E+02, 2.966031E+02, 3.033184E+02, 3.100376E+02, 3.167610E+02, 
   3.234889E+02, 3.302217E+02, 3.369598E+02, 3.437037E+02, 3.504537E+02, 3.572102E+02, 
   3.639739E+02, 3.707450E+02, 3.775240E+02, 3.843116E+02, 3.911080E+02, 3.979139E+02, 
   4.047298E+02, 4.115561E+02, 4.183933E+02, 4.252420E+02, 4.321026E+02, 4.389758E+02, 
   4.458619E+02, 4.527615E+02, 4.596752E+02, 4.666034E+02, 4.735466E+02, 4.805053E+02, 
   4.874801E+02, 4.944713E+02, 5.014796E+02, 5.085053E+02, 5.155490E+02, 5.226112E+02, 
   5.296922E+02, 5.367926E+02, 5.439127E+02, 5.510531E+02, 5.582142E+02, 5.653964E+02, 
   5.726001E+02, 5.798257E+02, 5.870738E+02, 5.943445E+02, 6.016384E+02, 6.089558E+02, 
   6.162971E+02, 6.236626E+02, 6.310528E+02, 6.384680E+02, 6.459085E+02, 6.533747E+02, 
   6.608669E+02, 6.683854E+02, 6.759306E+02, 6.835028E+02, 6.911022E+02, 6.987292E+02, 
   7.063841E+02, 7.140671E+02, 7.217786E+02, 7.295188E+02, 7.372879E+02, 7.450863E+02, 
   7.529142E+02, 7.607719E+02, 7.686595E+02, 7.765774E+02, 7.845258E+02, 7.925048E+02, 
   8.005148E+02, 8.085560E+02, 8.166285E+02, 8.247326E+02, 8.328685E+02, 8.410364E+02, 
   8.492364E+02, 8.574689E+02, 8.657339E+02, 8.740316E+02, 8.823623E+02, 8.907262E+02, 
   8.991233E+02, 9.075538E+02, 9.160180E+02, 9.245160E+02, 9.330480E+02, 9.416140E+02, 
   9.502144E+02, 9.588491E+02, 9.675184E+02, 9.762224E+02, 9.849613E+02, 9.937351E+02, 
   1.002544E+03, 1.011388E+03, 1.020268E+03, 1.029183E+03, 1.038134E+03, 1.047121E+03, 
   1.056143E+03, 1.065202E+03, 1.074296E+03, 1.083427E+03, 1.092594E+03, 1.101798E+03, 
   1.111039E+03, 1.120316E+03, 1.129630E+03, 1.138980E+03, 1.148368E+03, 1.157793E+03, 
   1.167255E+03, 1.176755E+03, 1.186292E+03, 1.195866E+03, 1.205478E+03, 1.215128E+03, 
   1.224815E+03, 1.234540E+03, 1.244304E+03, 1.254105E+03, 1.263945E+03, 1.273822E+03, 
   1.283738E+03, 1.293693E+03, 1.303686E+03, 1.313717E+03, 1.323787E+03, 1.333896E+03, 
   1.344044E+03, 1.354230E+03, 1.364456E+03, 1.374720E+03, 1.385024E+03, 1.395367E+03, 
   1.405748E+03, 1.416170E+03, 1.426630E+03, 1.437131E+03, 1.447670E+03, 1.458249E+03, 
   1.468868E+03, 1.479527E+03, 1.490225E+03, 1.500964E+03, 1.511742E+03, 1.522560E+03, 
   1.533418E+03, 1.544317E+03, 1.555255E+03, 1.566234E+03, 1.577253E+03, 1.588313E+03, 
   1.599413E+03, 1.610553E+03, 1.621734E+03, 1.632956E+03, 1.644218E+03, 1.655521E+03, 
   1.666865E+03, 1.678249E+03, 1.689675E+03, 1.701141E+03, 1.712649E+03, 1.724197E+03, 
   1.735787E+03, 1.747418E+03, 1.759090E+03, 1.770804E+03, 1.782559E+03, 1.794355E+03, 
   1.806192E+03, 1.818072E+03, 1.829993E+03, 1.841955E+03, 1.853959E+03, 1.866005E+03, 
   1.878093E+03, 1.890222E+03, 1.902393E+03, 1.914607E+03, 1.926862E+03, 1.939160E+03, 
   1.951499E+03, 1.963881E+03, 1.976304E+03, 1.988771E+03, 2.001279E+03, 2.013830E+03, 
   2.026423E+03, 2.039059E+03, 2.051737E+03, 2.064457E+03, 2.077221E+03, 2.090027E+03, 
   2.102875E+03, 2.115767E+03, 2.128701E+03, 2.141678E+03, 2.154698E+03, 2.167761E+03, 
   2.180867E+03, 2.194016E+03, 2.207208E+03, 2.220443E+03, 2.233722E+03, 2.247044E+03, 
   2.260408E+03, 2.273817E+03, 2.287269E+03, 2.300764E+03, 2.314302E+03, 2.327885E+03, 
   2.341510E+03, 2.355180E+03, 2.368893E+03, 2.382650E+03, 2.396450E+03, 2.410294E+03, 
   2.424183E+03, 2.438115E+03, 2.452091E+03, 2.466111E+03, 2.480175E+03, 2.494284E+03, 
   2.508436E+03, 2.522633E+03, 2.536873E+03, 2.551159E+03, 2.565488E+03, 2.579862E+03, 
   2.594280E+03, 2.608743E+03, 2.623250E+03, 2.637802E+03, 2.652398E+03, 2.667039E+03, 
   2.681725E+03, 2.696456E+03, 2.711231E+03, 2.726051E+03, 2.740916E+03, 2.755826E+03, 
   2.770781E+03, 2.785781E+03, 2.800826E+03, 2.815916E+03, 2.831052E+03, 2.846232E+03, 
   2.861458E+03, 2.876729E+03, 2.892045E+03, 2.907407E+03, 2.922814E+03, 2.938267E+03, 
   2.953765E+03, 2.969309E+03, 2.984899E+03, 3.000534E+03, 3.016214E+03, 3.031941E+03, 
   3.047713E+03, 3.063531E+03, 3.079395E+03, 3.095305E+03, 3.111261E+03, 3.127263E+03, 
   3.143311E+03, 3.159405E+03, 3.175545E+03, 3.191732E+03, 3.207964E+03, 3.224243E+03, 
   3.240568E+03, 3.256940E+03, 3.273358E+03, 3.289823E+03, 3.306334E+03, 3.322891E+03, 
   3.339495E+03, 3.356146E+03, 3.372843E+03, 3.389587E+03, 3.406378E+03, 3.423216E+03, 
   3.440101E+03, 3.457032E+03, 3.474010E+03, 3.491036E+03, 3.508108E+03, 3.525228E+03, 
   3.542394E+03, 3.559608E+03, 3.576869E+03, 3.594177E+03, 3.611532E+03, 3.628935E+03, 
   3.646385E+03, 3.663883E+03, 3.681428E+03, 3.699020E+03, 3.716660E+03, 3.734348E+03, 
   3.752083E+03, 3.769866E+03, 3.787696E+03, 3.805574E+03, 3.823500E+03, 3.841474E+03, 
   3.859496E+03, 3.877566E+03, 3.895683E+03, 3.913849E+03, 3.932063E+03, 3.950324E+03, 
   3.968634E+03, 3.986992E+03, 4.005398E+03, 4.023853E+03, 4.042355E+03, 4.060906E+03, 
   4.079506E+03, 4.098154E+03, 4.116850E+03, 4.135595E+03, 4.154388E+03, 4.173229E+03, 
   4.192120E+03, 4.211059E+03, 4.230046E+03, 4.249083E+03, 4.268168E+03, 4.287302E+03, 
   4.306485E+03, 4.325716E+03, 4.344997E+03, 4.364326E+03, 4.383705E+03, 4.403132E+03, 
   4.422609E+03, 4.442135E+03, 4.461709E+03, 4.481333E+03, 4.501007E+03, 4.520729E+03, 
   4.540501E+03, 4.560322E+03, 4.580192E+03, 4.600112E+03, 4.620081E+03, 4.640100E+03, 
   4.660168E+03, 4.680286E+03, 4.700454E+03, 4.720671E+03, 4.740937E+03, 4.761253E+03, 
   4.781620E+03, 4.802035E+03, 4.822501E+03, 4.843016E+03, 4.863582E+03, 4.884197E+03, 
   4.904862E+03, 4.925577E+03, 4.946342E+03, 4.967157E+03, 4.988023E+03, 5.008938E+03, 
   5.029903E+03, 5.050919E+03, 5.071985E+03, 5.093101E+03, 5.114267E+03, 5.135484E+03, 
   5.156751E+03, 5.178068E+03, 5.199436E+03, 5.220854E+03, 5.242323E+03, 5.263842E+03, 
   5.285411E+03, 5.307032E+03, 5.328702E+03, 5.350424E+03, 5.372196E+03, 5.394019E+03, 
   5.415892E+03, 5.437816E+03, 5.459791E+03, 5.481817E+03, 5.503893E+03, 5.526021E+03, 
   5.548199E+03, 5.570428E+03, 5.592708E+03, 5.615039E+03, 5.637422E+03, 5.659855E+03, 
   5.682339E+03, 5.704874E+03, 5.727461E+03, 5.750098E+03, 5.772787E+03, 5.795527E+03, 
   5.818318E+03, 5.841160E+03, 5.864053E+03, 5.886998E+03, 5.909994E+03, 5.933042E+03, 
   5.956141E+03, 5.979291E+03, 6.002493E+03, 6.025746E+03, 6.049051E+03, 6.072407E+03, 
   6.095814E+03, 6.119273E+03, 6.142784E+03, 6.166346E+03, 6.189960E+03, 6.213626E+03, 
   6.237343E+03, 6.261112E+03, 6.284932E+03, 6.308804E+03, 6.332728E+03, 6.356704E+03, 
   6.380732E+03, 6.404811E+03, 6.428942E+03, 6.453125E+03, 6.477360E+03, 6.501646E+03, 
   6.525985E+03, 6.550375E+03, 6.574818E+03, 6.599312E+03, 6.623859E+03, 6.648457E+03, 
   6.673107E+03, 6.697810E+03, 6.722564E+03, 6.747370E+03, 6.772229E+03, 6.797140E+03, 
   6.822102E+03, 6.847117E+03, 6.872184E+03, 6.897303E+03, 6.922475E+03, 6.947698E+03, 
   6.972974E+03, 6.998302E+03, 7.023682E+03, 7.049114E+03, 7.074599E+03, 7.100136E+03, 
   7.125725E+03, 7.151366E+03, 7.177060E+03, 7.202806E+03, 7.228605E+03, 7.254455E+03, 
   7.280359E+03, 7.306314E+03, 7.332322E+03, 7.358382E+03, 7.384495E+03, 7.410660E+03, 
   7.436877E+03, 7.463147E+03, 7.489470E+03, 7.515844E+03, 7.542272E+03, 7.568751E+03, 
   7.595283E+03, 7.621868E+03, 7.648505E+03, 7.675195E+03, 7.701937E+03, 7.728732E+03, 
   7.755579E+03, 7.782478E+03, 7.809431E+03, 7.836435E+03, 7.863493E+03, 7.890602E+03, 
   7.917765E+03, 7.944980E+03, 7.972247E+03, 7.999567E+03, 8.026940E+03, 8.054365E+03, 
   8.081842E+03, 8.109373E+03, 8.136956E+03, 8.164591E+03, 8.192279E+03, 8.220020E+03, 
   8.247813E+03, 8.275659E+03, 8.303557E+03, 8.331508E+03, 8.359511E+03, 8.387568E+03, 
   8.415676E+03, 8.443838E+03, 8.472051E+03, 8.500318E+03, 8.528637E+03, 8.557009E+03, 
   8.585433E+03, 8.613910E+03, 8.642439E+03, 8.671021E+03, 8.699655E+03, 8.728342E+03, 
   8.757082E+03, 8.785874E+03, 8.814719E+03, 8.843616E+03, 8.872566E+03, 8.901569E+03, 
   8.930624E+03, 8.959731E+03, 8.988891E+03, 9.018103E+03, 9.047369E+03, 9.076686E+03, 
   9.106056E+03, 9.135479E+03, 9.164954E+03, 9.194481E+03, 9.224061E+03, 9.253693E+03, 
   9.283378E+03, 9.313116E+03, 9.342905E+03, 9.372747E+03, 9.402642E+03, 9.432589E+03, 
   9.462588E+03, 9.492640E+03, 9.522744E+03, 9.552901E+03, 9.583109E+03, 9.613371E+03, 
   9.643684E+03, 9.674050E+03, 9.704468E+03, 9.734938E+03, 9.765461E+03, 9.796036E+03, 
   9.826663E+03, 
])
# ============================ M = 16, I = 2 ============================
M = 16
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.000000E+00, 1.020237E+01, 1.635327E+01, 2.288329E+01, 2.948508E+01, 3.611373E+01, 
   4.275570E+01, 4.940549E+01, 5.606042E+01, 6.271905E+01, 6.938053E+01, 7.604431E+01, 
   8.271004E+01, 8.937749E+01, 9.604648E+01, 1.027169E+02, 1.093886E+02, 1.160616E+02, 
   1.227357E+02, 1.294110E+02, 1.360875E+02, 1.427650E+02, 1.494436E+02, 1.561232E+02, 
   1.628039E+02, 1.694856E+02, 1.761684E+02, 1.828521E+02, 1.895370E+02, 1.962229E+02, 
   2.029099E+02, 2.095981E+02, 2.162874E+02, 2.229780E+02, 2.296699E+02, 2.363633E+02, 
   2.430582E+02, 2.497548E+02, 2.564533E+02, 2.631538E+02, 2.698565E+02, 2.765616E+02, 
   2.832694E+02, 2.899801E+02, 2.966941E+02, 3.034115E+02, 3.101328E+02, 3.168583E+02, 
   3.235883E+02, 3.303232E+02, 3.370634E+02, 3.438094E+02, 3.505615E+02, 3.573202E+02, 
   3.640860E+02, 3.708593E+02, 3.776405E+02, 3.844302E+02, 3.912288E+02, 3.980369E+02, 
   4.048549E+02, 4.116834E+02, 4.185229E+02, 4.253737E+02, 4.322366E+02, 4.391120E+02, 
   4.460004E+02, 4.529023E+02, 4.598182E+02, 4.667487E+02, 4.736942E+02, 4.806552E+02, 
   4.876323E+02, 4.946259E+02, 5.016365E+02, 5.086646E+02, 5.157107E+02, 5.227752E+02, 
   5.298586E+02, 5.369614E+02, 5.440840E+02, 5.512268E+02, 5.583904E+02, 5.655750E+02, 
   5.727812E+02, 5.800094E+02, 5.872599E+02, 5.945332E+02, 6.018296E+02, 6.091495E+02, 
   6.164934E+02, 6.238616E+02, 6.312544E+02, 6.386722E+02, 6.461153E+02, 6.535842E+02, 
   6.610791E+02, 6.686003E+02, 6.761482E+02, 6.837230E+02, 6.913252E+02, 6.989550E+02, 
   7.066126E+02, 7.142985E+02, 7.220127E+02, 7.297557E+02, 7.375277E+02, 7.453290E+02, 
   7.531597E+02, 7.610203E+02, 7.689108E+02, 7.768317E+02, 7.847830E+02, 7.927650E+02, 
   8.007780E+02, 8.088221E+02, 8.168977E+02, 8.250048E+02, 8.331437E+02, 8.413147E+02, 
   8.495178E+02, 8.577533E+02, 8.660215E+02, 8.743224E+02, 8.826562E+02, 8.910232E+02, 
   8.994235E+02, 9.078573E+02, 9.163248E+02, 9.248260E+02, 9.333612E+02, 9.419306E+02, 
   9.505342E+02, 9.591722E+02, 9.678449E+02, 9.765522E+02, 9.852945E+02, 9.940717E+02, 
   1.002884E+03, 1.011732E+03, 1.020615E+03, 1.029534E+03, 1.038488E+03, 1.047478E+03, 
   1.056504E+03, 1.065566E+03, 1.074664E+03, 1.083799E+03, 1.092970E+03, 1.102177E+03, 
   1.111421E+03, 1.120702E+03, 1.130019E+03, 1.139374E+03, 1.148765E+03, 1.158194E+03, 
   1.167660E+03, 1.177163E+03, 1.186704E+03, 1.196282E+03, 1.205898E+03, 1.215551E+03, 
   1.225242E+03, 1.234972E+03, 1.244739E+03, 1.254544E+03, 1.264388E+03, 1.274269E+03, 
   1.284189E+03, 1.294148E+03, 1.304145E+03, 1.314180E+03, 1.324254E+03, 1.334367E+03, 
   1.344519E+03, 1.354709E+03, 1.364939E+03, 1.375208E+03, 1.385515E+03, 1.395862E+03, 
   1.406248E+03, 1.416674E+03, 1.427139E+03, 1.437643E+03, 1.448187E+03, 1.458770E+03, 
   1.469393E+03, 1.480056E+03, 1.490759E+03, 1.501502E+03, 1.512284E+03, 1.523107E+03, 
   1.533969E+03, 1.544872E+03, 1.555815E+03, 1.566798E+03, 1.577822E+03, 1.588886E+03, 
   1.599990E+03, 1.611135E+03, 1.622320E+03, 1.633546E+03, 1.644813E+03, 1.656121E+03, 
   1.667469E+03, 1.678858E+03, 1.690288E+03, 1.701759E+03, 1.713271E+03, 1.724824E+03, 
   1.736419E+03, 1.748054E+03, 1.759731E+03, 1.771449E+03, 1.783208E+03, 1.795009E+03, 
   1.806851E+03, 1.818735E+03, 1.830660E+03, 1.842627E+03, 1.854636E+03, 1.866686E+03, 
   1.878779E+03, 1.890913E+03, 1.903089E+03, 1.915306E+03, 1.927566E+03, 1.939868E+03, 
   1.952212E+03, 1.964598E+03, 1.977026E+03, 1.989497E+03, 2.002010E+03, 2.014565E+03, 
   2.027162E+03, 2.039802E+03, 2.052485E+03, 2.065210E+03, 2.077977E+03, 2.090787E+03, 
   2.103640E+03, 2.116536E+03, 2.129474E+03, 2.142455E+03, 2.155479E+03, 2.168546E+03, 
   2.181656E+03, 2.194809E+03, 2.208005E+03, 2.221244E+03, 2.234526E+03, 2.247851E+03, 
   2.261220E+03, 2.274632E+03, 2.288087E+03, 2.301585E+03, 2.315127E+03, 2.328713E+03, 
   2.342341E+03, 2.356014E+03, 2.369730E+03, 2.383489E+03, 2.397293E+03, 2.411140E+03, 
   2.425030E+03, 2.438965E+03, 2.452943E+03, 2.466966E+03, 2.481032E+03, 2.495142E+03, 
   2.509296E+03, 2.523495E+03, 2.537737E+03, 2.552023E+03, 2.566354E+03, 2.580729E+03, 
   2.595148E+03, 2.609612E+03, 2.624119E+03, 2.638672E+03, 2.653268E+03, 2.667909E+03, 
   2.682595E+03, 2.697325E+03, 2.712100E+03, 2.726919E+03, 2.741783E+03, 2.756692E+03, 
   2.771646E+03, 2.786644E+03, 2.801687E+03, 2.816775E+03, 2.831908E+03, 2.847086E+03, 
   2.862308E+03, 2.877576E+03, 2.892889E+03, 2.908247E+03, 2.923650E+03, 2.939099E+03, 
   2.954592E+03, 2.970131E+03, 2.985715E+03, 3.001344E+03, 3.017019E+03, 3.032739E+03, 
   3.048505E+03, 3.064316E+03, 3.080172E+03, 3.096074E+03, 3.112022E+03, 3.128015E+03, 
   3.144054E+03, 3.160139E+03, 3.176269E+03, 3.192445E+03, 3.208667E+03, 3.224935E+03, 
   3.241248E+03, 3.257608E+03, 3.274013E+03, 3.290464E+03, 3.306962E+03, 3.323505E+03, 
   3.340095E+03, 3.356730E+03, 3.373412E+03, 3.390139E+03, 3.406913E+03, 3.423734E+03, 
   3.440600E+03, 3.457513E+03, 3.474472E+03, 3.491478E+03, 3.508529E+03, 3.525628E+03, 
   3.542772E+03, 3.559964E+03, 3.577201E+03, 3.594486E+03, 3.611816E+03, 3.629194E+03, 
   3.646618E+03, 3.664089E+03, 3.681606E+03, 3.699170E+03, 3.716781E+03, 3.734439E+03, 
   3.752144E+03, 3.769895E+03, 3.787693E+03, 3.805538E+03, 3.823430E+03, 3.841369E+03, 
   3.859355E+03, 3.877388E+03, 3.895469E+03, 3.913596E+03, 3.931770E+03, 3.949991E+03, 
   3.968260E+03, 3.986575E+03, 4.004938E+03, 4.023348E+03, 4.041806E+03, 4.060310E+03, 
   4.078862E+03, 4.097462E+03, 4.116108E+03, 4.134802E+03, 4.153544E+03, 4.172332E+03, 
   4.191169E+03, 4.210052E+03, 4.228984E+03, 4.247962E+03, 4.266989E+03, 4.286063E+03, 
   4.305184E+03, 4.324353E+03, 4.343570E+03, 4.362834E+03, 4.382146E+03, 4.401506E+03, 
   4.420914E+03, 4.440369E+03, 4.459872E+03, 4.479423E+03, 4.499021E+03, 4.518668E+03, 
   4.538362E+03, 4.558104E+03, 4.577894E+03, 4.597732E+03, 4.617618E+03, 4.637552E+03, 
   4.657534E+03, 4.677564E+03, 4.697641E+03, 4.717767E+03, 4.737941E+03, 4.758163E+03, 
   4.778433E+03, 4.798751E+03, 4.819117E+03, 4.839532E+03, 4.859994E+03, 4.880505E+03, 
   4.901064E+03, 4.921671E+03, 4.942326E+03, 4.963029E+03, 4.983781E+03, 5.004581E+03, 
   5.025429E+03, 5.046326E+03, 5.067270E+03, 5.088264E+03, 5.109305E+03, 5.130395E+03, 
   5.151533E+03, 5.172719E+03, 5.193954E+03, 5.215238E+03, 5.236569E+03, 5.257949E+03, 
   5.279378E+03, 5.300855E+03, 5.322380E+03, 5.343954E+03, 5.365577E+03, 5.387247E+03, 
   5.408967E+03, 5.430735E+03, 5.452551E+03, 5.474416E+03, 5.496329E+03, 5.518291E+03, 
   5.540302E+03, 5.562361E+03, 5.584469E+03, 5.606625E+03, 5.628830E+03, 5.651083E+03, 
   5.673385E+03, 5.695736E+03, 5.718135E+03, 5.740583E+03, 5.763080E+03, 5.785625E+03, 
   5.808219E+03, 5.830861E+03, 5.853552E+03, 5.876292E+03, 5.899081E+03, 5.921918E+03, 
   5.944804E+03, 5.967738E+03, 5.990722E+03, 6.013753E+03, 6.036834E+03, 6.059963E+03, 
   6.083141E+03, 6.106368E+03, 6.129644E+03, 6.152968E+03, 6.176341E+03, 6.199762E+03, 
   6.223233E+03, 6.246752E+03, 6.270319E+03, 6.293936E+03, 6.317601E+03, 6.341315E+03, 
   6.365078E+03, 6.388889E+03, 6.412749E+03, 6.436658E+03, 6.460616E+03, 6.484622E+03, 
   6.508677E+03, 6.532781E+03, 6.556933E+03, 6.581134E+03, 6.605384E+03, 6.629683E+03, 
   6.654030E+03, 6.678426E+03, 6.702871E+03, 6.727365E+03, 6.751907E+03, 6.776498E+03, 
   6.801137E+03, 6.825826E+03, 6.850563E+03, 6.875348E+03, 6.900183E+03, 6.925066E+03, 
   6.949997E+03, 6.974978E+03, 7.000007E+03, 7.025084E+03, 7.050211E+03, 7.075386E+03, 
   7.100609E+03, 7.125882E+03, 7.151202E+03, 7.176572E+03, 7.201990E+03, 7.227456E+03, 
   7.252972E+03, 7.278536E+03, 7.304148E+03, 7.329809E+03, 7.355518E+03, 7.381276E+03, 
   7.407083E+03, 7.432938E+03, 7.458842E+03, 7.484794E+03, 7.510794E+03, 7.536844E+03, 
   7.562941E+03, 7.589087E+03, 7.615282E+03, 7.641525E+03, 7.667816E+03, 7.694156E+03, 
   7.720544E+03, 7.746981E+03, 7.773466E+03, 7.799999E+03, 7.826581E+03, 7.853211E+03, 
   7.879889E+03, 7.906616E+03, 7.933391E+03, 7.960214E+03, 7.987085E+03, 8.014005E+03, 
   8.040973E+03, 8.067989E+03, 8.095054E+03, 8.122166E+03, 8.149327E+03, 8.176536E+03, 
   8.203793E+03, 8.231098E+03, 8.258452E+03, 8.285853E+03, 8.313303E+03, 8.340800E+03, 
   8.368346E+03, 8.395940E+03, 8.423581E+03, 8.451271E+03, 8.479008E+03, 8.506794E+03, 
   8.534627E+03, 8.562509E+03, 8.590438E+03, 8.618415E+03, 8.646440E+03, 8.674513E+03, 
   8.702634E+03, 8.730802E+03, 8.759019E+03, 8.787283E+03, 8.815594E+03, 8.843954E+03, 
   8.872361E+03, 8.900815E+03, 8.929318E+03, 8.957868E+03, 8.986466E+03, 9.015111E+03, 
   9.043803E+03, 9.072544E+03, 9.101331E+03, 9.130167E+03, 9.159049E+03, 9.187980E+03, 
   9.216957E+03, 9.245982E+03, 9.275054E+03, 9.304174E+03, 9.333341E+03, 9.362555E+03, 
   9.391817E+03, 9.421125E+03, 9.450481E+03, 9.479884E+03, 9.509335E+03, 9.538832E+03, 
   9.568377E+03, 9.597968E+03, 9.627607E+03, 9.657292E+03, 9.687025E+03, 9.716805E+03, 
   9.746631E+03, 
])
# ============================ M = 16, I = 3 ============================
M = 16
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.200018E+01, 2.419175E+01, 4.353101E+01, 6.308027E+01, 8.267877E+01, 1.022979E+02, 
   1.219284E+02, 1.415666E+02, 1.612106E+02, 1.808592E+02, 2.005120E+02, 2.201685E+02, 
   2.398284E+02, 2.594916E+02, 2.791580E+02, 2.988274E+02, 3.184998E+02, 3.381752E+02, 
   3.578535E+02, 3.775348E+02, 3.972192E+02, 4.169067E+02, 4.365975E+02, 4.562919E+02, 
   4.759903E+02, 4.956931E+02, 5.154010E+02, 5.351147E+02, 5.548351E+02, 5.745634E+02, 
   5.943007E+02, 6.140485E+02, 6.338083E+02, 6.535820E+02, 6.733713E+02, 6.931783E+02, 
   7.130053E+02, 7.328544E+02, 7.527282E+02, 7.726291E+02, 7.925598E+02, 8.125229E+02, 
   8.325212E+02, 8.525574E+02, 8.726345E+02, 8.927551E+02, 9.129224E+02, 9.331390E+02, 
   9.534080E+02, 9.737320E+02, 9.941141E+02, 1.014557E+03, 1.035064E+03, 1.055637E+03, 
   1.076278E+03, 1.096992E+03, 1.117780E+03, 1.138645E+03, 1.159590E+03, 1.180616E+03, 
   1.201727E+03, 1.222924E+03, 1.244211E+03, 1.265588E+03, 1.287059E+03, 1.308625E+03, 
   1.330289E+03, 1.352052E+03, 1.373916E+03, 1.395884E+03, 1.417957E+03, 1.440137E+03, 
   1.462425E+03, 1.484823E+03, 1.507334E+03, 1.529958E+03, 1.552696E+03, 1.575551E+03, 
   1.598524E+03, 1.621616E+03, 1.644828E+03, 1.668162E+03, 1.691619E+03, 1.715200E+03, 
   1.738907E+03, 1.762740E+03, 1.786701E+03, 1.810790E+03, 1.835009E+03, 1.859359E+03, 
   1.883840E+03, 1.908454E+03, 1.933201E+03, 1.958083E+03, 1.983100E+03, 2.008253E+03, 
   2.033542E+03, 2.058970E+03, 2.084536E+03, 2.110241E+03, 2.136085E+03, 2.162070E+03, 
   2.188197E+03, 2.214465E+03, 2.240876E+03, 2.267430E+03, 2.294127E+03, 2.320969E+03, 
   2.347956E+03, 2.375088E+03, 2.402366E+03, 2.429790E+03, 2.457362E+03, 2.485081E+03, 
   2.512948E+03, 2.540963E+03, 2.569127E+03, 2.597441E+03, 2.625905E+03, 2.654518E+03, 
   2.683283E+03, 2.712198E+03, 2.741265E+03, 2.770484E+03, 2.799856E+03, 2.829379E+03, 
   2.859056E+03, 2.888887E+03, 2.918871E+03, 2.949009E+03, 2.979301E+03, 3.009749E+03, 
   3.040351E+03, 3.071109E+03, 3.102023E+03, 3.133092E+03, 3.164318E+03, 3.195701E+03, 
   3.227241E+03, 3.258938E+03, 3.290792E+03, 3.322805E+03, 3.354975E+03, 3.387304E+03, 
   3.419791E+03, 3.452438E+03, 3.485243E+03, 3.518208E+03, 3.551333E+03, 3.584618E+03, 
   3.618062E+03, 3.651668E+03, 3.685434E+03, 3.719360E+03, 3.753448E+03, 3.787698E+03, 
   3.822109E+03, 3.856682E+03, 3.891417E+03, 3.926314E+03, 3.961373E+03, 3.996596E+03, 
   4.031981E+03, 4.067530E+03, 4.103242E+03, 4.139117E+03, 4.175156E+03, 4.211359E+03, 
   4.247727E+03, 4.284258E+03, 4.320955E+03, 4.357816E+03, 4.394842E+03, 4.432033E+03, 
   4.469390E+03, 4.506912E+03, 4.544599E+03, 4.582453E+03, 4.620473E+03, 4.658659E+03, 
   4.697012E+03, 4.735531E+03, 4.774217E+03, 4.813070E+03, 4.852090E+03, 4.891278E+03, 
   4.930633E+03, 4.970156E+03, 5.009846E+03, 5.049705E+03, 5.089732E+03, 5.129927E+03, 
   5.170291E+03, 5.210823E+03, 5.251524E+03, 5.292395E+03, 5.333434E+03, 5.374643E+03, 
   5.416021E+03, 5.457569E+03, 5.499287E+03, 5.541174E+03, 5.583232E+03, 5.625460E+03, 
   5.667858E+03, 5.710427E+03, 5.753167E+03, 5.796077E+03, 5.839159E+03, 5.882412E+03, 
   5.925836E+03, 5.969431E+03, 6.013198E+03, 6.057136E+03, 6.101247E+03, 6.145529E+03, 
   6.189984E+03, 6.234610E+03, 6.279409E+03, 6.324381E+03, 6.369525E+03, 6.414842E+03, 
   6.460332E+03, 6.505995E+03, 6.551831E+03, 6.597841E+03, 6.644024E+03, 6.690380E+03, 
   6.736910E+03, 6.783614E+03, 6.830491E+03, 6.877543E+03, 6.924769E+03, 6.972169E+03, 
   7.019743E+03, 7.067492E+03, 7.115415E+03, 7.163514E+03, 7.211786E+03, 7.260234E+03, 
   7.308857E+03, 7.357655E+03, 7.406628E+03, 7.455777E+03, 7.505101E+03, 7.554600E+03, 
   7.604275E+03, 7.654126E+03, 7.704153E+03, 7.754355E+03, 7.804734E+03, 7.855289E+03, 
   7.906020E+03, 7.956927E+03, 8.008011E+03, 8.059271E+03, 8.110707E+03, 8.162321E+03, 
   8.214111E+03, 8.266078E+03, 8.318221E+03, 8.370542E+03, 8.423040E+03, 8.475715E+03, 
   8.528567E+03, 8.581597E+03, 8.634803E+03, 8.688188E+03, 8.741749E+03, 8.795489E+03, 
   8.849406E+03, 8.903501E+03, 8.957773E+03, 9.012224E+03, 9.066852E+03, 9.121658E+03, 
   9.176642E+03, 9.231805E+03, 9.287145E+03, 9.342664E+03, 9.398361E+03, 9.454236E+03, 
   9.510290E+03, 9.566522E+03, 9.622933E+03, 9.679522E+03, 9.736290E+03, 9.793236E+03, 
   9.850361E+03, 9.907665E+03, 9.965148E+03, 1.002281E+04, 1.008065E+04, 1.013867E+04, 
   1.019687E+04, 1.025524E+04, 1.031380E+04, 1.037253E+04, 1.043145E+04, 1.049054E+04, 
   1.054981E+04, 1.060926E+04, 1.066889E+04, 1.072870E+04, 1.078869E+04, 1.084886E+04, 
   1.090921E+04, 1.096973E+04, 1.103044E+04, 1.109133E+04, 1.115239E+04, 1.121363E+04, 
   1.127506E+04, 1.133666E+04, 1.139844E+04, 1.146040E+04, 1.152254E+04, 1.158486E+04, 
   1.164736E+04, 1.171004E+04, 1.177290E+04, 1.183594E+04, 1.189916E+04, 1.196255E+04, 
   1.202613E+04, 1.208988E+04, 1.215382E+04, 1.221793E+04, 1.228223E+04, 1.234670E+04, 
   1.241135E+04, 1.247619E+04, 1.254120E+04, 1.260639E+04, 1.267176E+04, 1.273731E+04, 
   1.280303E+04, 1.286894E+04, 1.293503E+04, 1.300130E+04, 1.306774E+04, 1.313437E+04, 
   1.320117E+04, 1.326815E+04, 1.333531E+04, 1.340266E+04, 1.347018E+04, 1.353788E+04, 
   1.360575E+04, 1.367381E+04, 1.374205E+04, 1.381046E+04, 1.387906E+04, 1.394783E+04, 
   1.401678E+04, 1.408591E+04, 1.415522E+04, 1.422471E+04, 1.429438E+04, 1.436422E+04, 
   1.443424E+04, 1.450445E+04, 1.457483E+04, 1.464539E+04, 1.471612E+04, 1.478704E+04, 
   1.485813E+04, 1.492940E+04, 1.500085E+04, 1.507248E+04, 1.514429E+04, 1.521627E+04, 
   1.528843E+04, 1.536077E+04, 1.543329E+04, 1.550599E+04, 1.557886E+04, 1.565191E+04, 
   1.572514E+04, 1.579855E+04, 1.587213E+04, 1.594589E+04, 1.601983E+04, 1.609394E+04, 
   1.616823E+04, 1.624270E+04, 1.631735E+04, 1.639217E+04, 1.646717E+04, 1.654235E+04, 
   1.661770E+04, 1.669323E+04, 1.676894E+04, 1.684482E+04, 1.692088E+04, 1.699712E+04, 
   1.707353E+04, 1.715012E+04, 1.722688E+04, 1.730382E+04, 1.738094E+04, 1.745823E+04, 
   1.753570E+04, 1.761334E+04, 1.769116E+04, 1.776916E+04, 1.784733E+04, 1.792567E+04, 
   1.800419E+04, 1.808289E+04, 1.816176E+04, 1.824080E+04, 1.832002E+04, 1.839942E+04, 
   1.847899E+04, 1.855873E+04, 1.863865E+04, 1.871874E+04, 1.879901E+04, 1.887945E+04, 
   1.896006E+04, 1.904085E+04, 1.912181E+04, 1.920295E+04, 1.928426E+04, 1.936574E+04, 
   1.944740E+04, 1.952923E+04, 1.961123E+04, 1.969341E+04, 1.977575E+04, 1.985828E+04, 
   1.994097E+04, 2.002384E+04, 2.010688E+04, 2.019009E+04, 2.027347E+04, 2.035703E+04, 
   2.044076E+04, 2.052466E+04, 2.060873E+04, 2.069297E+04, 2.077739E+04, 2.086197E+04, 
   2.094673E+04, 2.103166E+04, 2.111676E+04, 2.120203E+04, 2.128747E+04, 2.137309E+04, 
   2.145887E+04, 2.154482E+04, 2.163095E+04, 2.171724E+04, 2.180371E+04, 2.189034E+04, 
   2.197714E+04, 2.206412E+04, 2.215126E+04, 2.223857E+04, 2.232606E+04, 2.241371E+04, 
   2.250153E+04, 2.258952E+04, 2.267767E+04, 2.276600E+04, 2.285449E+04, 2.294316E+04, 
   2.303199E+04, 2.312099E+04, 2.321016E+04, 2.329949E+04, 2.338899E+04, 2.347866E+04, 
   2.356850E+04, 2.365851E+04, 2.374868E+04, 2.383902E+04, 2.392952E+04, 2.402020E+04, 
   2.411104E+04, 2.420204E+04, 2.429321E+04, 2.438455E+04, 2.447606E+04, 2.456773E+04, 
   2.465956E+04, 2.475156E+04, 2.484373E+04, 2.493606E+04, 2.502856E+04, 2.512122E+04, 
   2.521404E+04, 2.530703E+04, 2.540019E+04, 2.549351E+04, 2.558699E+04, 2.568064E+04, 
   2.577445E+04, 2.586843E+04, 2.596257E+04, 2.605687E+04, 2.615133E+04, 2.624596E+04, 
   2.634075E+04, 2.643571E+04, 2.653083E+04, 2.662611E+04, 2.672155E+04, 2.681715E+04, 
   2.691292E+04, 2.700884E+04, 2.710493E+04, 2.720119E+04, 2.729760E+04, 2.739417E+04, 
   2.749091E+04, 2.758780E+04, 2.768486E+04, 2.778207E+04, 2.787945E+04, 2.797699E+04, 
   2.807469E+04, 2.817254E+04, 2.827056E+04, 2.836874E+04, 2.846707E+04, 2.856557E+04, 
   2.866422E+04, 2.876303E+04, 2.886201E+04, 2.896114E+04, 2.906042E+04, 2.915987E+04, 
   2.925948E+04, 2.935924E+04, 2.945916E+04, 2.955924E+04, 2.965947E+04, 2.975987E+04, 
   2.986042E+04, 2.996112E+04, 3.006199E+04, 3.016301E+04, 3.026418E+04, 3.036551E+04, 
   3.046700E+04, 3.056865E+04, 3.067045E+04, 3.077240E+04, 3.087451E+04, 3.097678E+04, 
   3.107920E+04, 3.118177E+04, 3.128450E+04, 3.138739E+04, 3.149043E+04, 3.159362E+04, 
   3.169697E+04, 3.180047E+04, 3.190412E+04, 3.200793E+04, 3.211189E+04, 3.221600E+04, 
   3.232027E+04, 3.242468E+04, 3.252925E+04, 3.263398E+04, 3.273885E+04, 3.284388E+04, 
   3.294906E+04, 3.305439E+04, 3.315987E+04, 3.326550E+04, 3.337129E+04, 3.347722E+04, 
   3.358331E+04, 3.368954E+04, 3.379593E+04, 3.390247E+04, 3.400915E+04, 3.411599E+04, 
   3.422297E+04, 3.433011E+04, 3.443739E+04, 3.454482E+04, 3.465240E+04, 3.476013E+04, 
   3.486801E+04, 3.497604E+04, 3.508421E+04, 3.519253E+04, 3.530100E+04, 3.540962E+04, 
   3.551839E+04, 3.562730E+04, 3.573636E+04, 3.584556E+04, 3.595491E+04, 3.606441E+04, 
   3.617405E+04, 
])
# ============================ M = 16, I = 4 ============================
M = 16
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.200018E+01, 2.420337E+01, 4.355487E+01, 6.311620E+01, 8.272675E+01, 1.023579E+02, 
   1.220005E+02, 1.416507E+02, 1.613067E+02, 1.809674E+02, 2.006322E+02, 2.203007E+02, 
   2.399726E+02, 2.596479E+02, 2.793263E+02, 2.990077E+02, 3.186922E+02, 3.383796E+02, 
   3.580700E+02, 3.777633E+02, 3.974597E+02, 4.171592E+02, 4.368621E+02, 4.565686E+02, 
   4.762790E+02, 4.959939E+02, 5.157139E+02, 5.354397E+02, 5.551722E+02, 5.749126E+02, 
   5.946621E+02, 6.144220E+02, 6.341940E+02, 6.539799E+02, 6.737815E+02, 6.936008E+02, 
   7.134401E+02, 7.333016E+02, 7.531878E+02, 7.731012E+02, 7.930444E+02, 8.130200E+02, 
   8.330310E+02, 8.530799E+02, 8.731698E+02, 8.933034E+02, 9.134835E+02, 9.337132E+02, 
   9.539953E+02, 9.743326E+02, 9.947280E+02, 1.015184E+03, 1.035704E+03, 1.056291E+03, 
   1.076947E+03, 1.097674E+03, 1.118476E+03, 1.139355E+03, 1.160314E+03, 1.181354E+03, 
   1.202480E+03, 1.223691E+03, 1.244992E+03, 1.266384E+03, 1.287870E+03, 1.309451E+03, 
   1.331130E+03, 1.352908E+03, 1.374788E+03, 1.396772E+03, 1.418860E+03, 1.441056E+03, 
   1.463360E+03, 1.485774E+03, 1.508301E+03, 1.530941E+03, 1.553696E+03, 1.576567E+03, 
   1.599557E+03, 1.622666E+03, 1.645895E+03, 1.669246E+03, 1.692720E+03, 1.716319E+03, 
   1.740043E+03, 1.763894E+03, 1.787872E+03, 1.811980E+03, 1.836217E+03, 1.860585E+03, 
   1.885084E+03, 1.909717E+03, 1.934483E+03, 1.959383E+03, 1.984419E+03, 2.009591E+03, 
   2.034900E+03, 2.060347E+03, 2.085932E+03, 2.111657E+03, 2.137521E+03, 2.163527E+03, 
   2.189673E+03, 2.215962E+03, 2.242393E+03, 2.268967E+03, 2.295685E+03, 2.322548E+03, 
   2.349556E+03, 2.376709E+03, 2.404008E+03, 2.431454E+03, 2.459047E+03, 2.486788E+03, 
   2.514677E+03, 2.542714E+03, 2.570901E+03, 2.599237E+03, 2.627723E+03, 2.656359E+03, 
   2.685146E+03, 2.714085E+03, 2.743175E+03, 2.772417E+03, 2.801812E+03, 2.831359E+03, 
   2.861060E+03, 2.890914E+03, 2.920922E+03, 2.951084E+03, 2.981401E+03, 3.011872E+03, 
   3.042499E+03, 3.073282E+03, 3.104220E+03, 3.135315E+03, 3.166566E+03, 3.197974E+03, 
   3.229539E+03, 3.261262E+03, 3.293142E+03, 3.325181E+03, 3.357377E+03, 3.389732E+03, 
   3.422246E+03, 3.454919E+03, 3.487751E+03, 3.520743E+03, 3.553894E+03, 3.587206E+03, 
   3.620678E+03, 3.654311E+03, 3.688104E+03, 3.722059E+03, 3.756175E+03, 3.790452E+03, 
   3.824891E+03, 3.859493E+03, 3.894256E+03, 3.929182E+03, 3.964270E+03, 3.999521E+03, 
   4.034936E+03, 4.070514E+03, 4.106255E+03, 4.142160E+03, 4.178229E+03, 4.214462E+03, 
   4.250859E+03, 4.287421E+03, 4.324148E+03, 4.361039E+03, 4.398096E+03, 4.435318E+03, 
   4.472705E+03, 4.510258E+03, 4.547977E+03, 4.585863E+03, 4.623914E+03, 4.662132E+03, 
   4.700516E+03, 4.739067E+03, 4.777785E+03, 4.816671E+03, 4.855723E+03, 4.894944E+03, 
   4.934331E+03, 4.973887E+03, 5.013611E+03, 5.053503E+03, 5.093563E+03, 5.133792E+03, 
   5.174189E+03, 5.214755E+03, 5.255491E+03, 5.296395E+03, 5.337469E+03, 5.378712E+03, 
   5.420125E+03, 5.461708E+03, 5.503460E+03, 5.545383E+03, 5.587476E+03, 5.629739E+03, 
   5.672173E+03, 5.714778E+03, 5.757554E+03, 5.800500E+03, 5.843618E+03, 5.886907E+03, 
   5.930367E+03, 5.973999E+03, 6.017803E+03, 6.061778E+03, 6.105926E+03, 6.150246E+03, 
   6.194737E+03, 6.239402E+03, 6.284239E+03, 6.329248E+03, 6.374431E+03, 6.419786E+03, 
   6.465314E+03, 6.511016E+03, 6.556890E+03, 6.602939E+03, 6.649160E+03, 6.695556E+03, 
   6.742125E+03, 6.788869E+03, 6.835786E+03, 6.882877E+03, 6.930143E+03, 6.977583E+03, 
   7.025198E+03, 7.072987E+03, 7.120951E+03, 7.169090E+03, 7.217404E+03, 7.265892E+03, 
   7.314556E+03, 7.363396E+03, 7.412410E+03, 7.461601E+03, 7.510966E+03, 7.560508E+03, 
   7.610225E+03, 7.660118E+03, 7.710187E+03, 7.760433E+03, 7.810854E+03, 7.861452E+03, 
   7.912226E+03, 7.963176E+03, 8.014303E+03, 8.065607E+03, 8.117087E+03, 8.168744E+03, 
   8.220578E+03, 8.272590E+03, 8.324778E+03, 8.377143E+03, 8.429685E+03, 8.482405E+03, 
   8.535302E+03, 8.588377E+03, 8.641629E+03, 8.695059E+03, 8.748666E+03, 8.802451E+03, 
   8.856414E+03, 8.910555E+03, 8.964874E+03, 9.019370E+03, 9.074045E+03, 9.128898E+03, 
   9.183929E+03, 9.239139E+03, 9.294526E+03, 9.350092E+03, 9.405837E+03, 9.461760E+03, 
   9.517861E+03, 9.574141E+03, 9.630600E+03, 9.687238E+03, 9.744054E+03, 9.801049E+03, 
   9.858222E+03, 9.915575E+03, 9.973107E+03, 1.003082E+04, 1.008871E+04, 1.014678E+04, 
   1.020502E+04, 1.026345E+04, 1.032206E+04, 1.038084E+04, 1.043980E+04, 1.049895E+04, 
   1.055827E+04, 1.061777E+04, 1.067745E+04, 1.073731E+04, 1.079735E+04, 1.085757E+04, 
   1.091797E+04, 1.097855E+04, 1.103931E+04, 1.110025E+04, 1.116136E+04, 1.122266E+04, 
   1.128413E+04, 1.134579E+04, 1.140762E+04, 1.146964E+04, 1.153183E+04, 1.159421E+04, 
   1.165676E+04, 1.171949E+04, 1.178240E+04, 1.184550E+04, 1.190877E+04, 1.197222E+04, 
   1.203585E+04, 1.209966E+04, 1.216365E+04, 1.222782E+04, 1.229217E+04, 1.235669E+04, 
   1.242140E+04, 1.248629E+04, 1.255136E+04, 1.261660E+04, 1.268203E+04, 1.274763E+04, 
   1.281342E+04, 1.287938E+04, 1.294553E+04, 1.301185E+04, 1.307835E+04, 1.314503E+04, 
   1.321189E+04, 1.327893E+04, 1.334615E+04, 1.341355E+04, 1.348113E+04, 1.354889E+04, 
   1.361683E+04, 1.368494E+04, 1.375324E+04, 1.382171E+04, 1.389036E+04, 1.395920E+04, 
   1.402821E+04, 1.409740E+04, 1.416676E+04, 1.423631E+04, 1.430604E+04, 1.437594E+04, 
   1.444603E+04, 1.451629E+04, 1.458673E+04, 1.465735E+04, 1.472815E+04, 1.479913E+04, 
   1.487028E+04, 1.494161E+04, 1.501313E+04, 1.508482E+04, 1.515669E+04, 1.522873E+04, 
   1.530096E+04, 1.537336E+04, 1.544594E+04, 1.551870E+04, 1.559163E+04, 1.566475E+04, 
   1.573804E+04, 1.581151E+04, 1.588516E+04, 1.595898E+04, 1.603299E+04, 1.610716E+04, 
   1.618152E+04, 1.625606E+04, 1.633077E+04, 1.640566E+04, 1.648072E+04, 1.655597E+04, 
   1.663138E+04, 1.670698E+04, 1.678275E+04, 1.685870E+04, 1.693483E+04, 1.701113E+04, 
   1.708761E+04, 1.716427E+04, 1.724110E+04, 1.731811E+04, 1.739530E+04, 1.747266E+04, 
   1.755019E+04, 1.762790E+04, 1.770579E+04, 1.778386E+04, 1.786210E+04, 1.794051E+04, 
   1.801910E+04, 1.809787E+04, 1.817681E+04, 1.825593E+04, 1.833522E+04, 1.841468E+04, 
   1.849432E+04, 1.857414E+04, 1.865413E+04, 1.873430E+04, 1.881464E+04, 1.889515E+04, 
   1.897584E+04, 1.905670E+04, 1.913774E+04, 1.921895E+04, 1.930033E+04, 1.938189E+04, 
   1.946362E+04, 1.954553E+04, 1.962761E+04, 1.970986E+04, 1.979228E+04, 1.987488E+04, 
   1.995766E+04, 2.004060E+04, 2.012372E+04, 2.020701E+04, 2.029047E+04, 2.037411E+04, 
   2.045791E+04, 2.054189E+04, 2.062605E+04, 2.071037E+04, 2.079487E+04, 2.087954E+04, 
   2.096438E+04, 2.104939E+04, 2.113457E+04, 2.121992E+04, 2.130545E+04, 2.139115E+04, 
   2.147701E+04, 2.156305E+04, 2.164926E+04, 2.173564E+04, 2.182219E+04, 2.190891E+04, 
   2.199580E+04, 2.208286E+04, 2.217009E+04, 2.225750E+04, 2.234507E+04, 2.243281E+04, 
   2.252072E+04, 2.260880E+04, 2.269704E+04, 2.278546E+04, 2.287405E+04, 2.296280E+04, 
   2.305173E+04, 2.314082E+04, 2.323008E+04, 2.331951E+04, 2.340911E+04, 2.349888E+04, 
   2.358881E+04, 2.367891E+04, 2.376918E+04, 2.385962E+04, 2.395022E+04, 2.404099E+04, 
   2.413193E+04, 2.422304E+04, 2.431431E+04, 2.440575E+04, 2.449736E+04, 2.458913E+04, 
   2.468107E+04, 2.477317E+04, 2.486545E+04, 2.495788E+04, 2.505049E+04, 2.514325E+04, 
   2.523619E+04, 2.532929E+04, 2.542255E+04, 2.551598E+04, 2.560958E+04, 2.570333E+04, 
   2.579726E+04, 2.589135E+04, 2.598560E+04, 2.608002E+04, 2.617460E+04, 2.626934E+04, 
   2.636425E+04, 2.645932E+04, 2.655456E+04, 2.664996E+04, 2.674552E+04, 2.684124E+04, 
   2.693713E+04, 2.703318E+04, 2.712940E+04, 2.722577E+04, 2.732231E+04, 2.741901E+04, 
   2.751587E+04, 2.761290E+04, 2.771008E+04, 2.780743E+04, 2.790494E+04, 2.800261E+04, 
   2.810044E+04, 2.819843E+04, 2.829658E+04, 2.839490E+04, 2.849337E+04, 2.859200E+04, 
   2.869080E+04, 2.878975E+04, 2.888886E+04, 2.898814E+04, 2.908757E+04, 2.918716E+04, 
   2.928691E+04, 2.938683E+04, 2.948689E+04, 2.958712E+04, 2.968751E+04, 2.978805E+04, 
   2.988876E+04, 2.998962E+04, 3.009064E+04, 3.019182E+04, 3.029315E+04, 3.039464E+04, 
   3.049629E+04, 3.059810E+04, 3.070006E+04, 3.080218E+04, 3.090446E+04, 3.100689E+04, 
   3.110948E+04, 3.121223E+04, 3.131513E+04, 3.141819E+04, 3.152140E+04, 3.162477E+04, 
   3.172830E+04, 3.183198E+04, 3.193581E+04, 3.203980E+04, 3.214394E+04, 3.224824E+04, 
   3.235270E+04, 3.245730E+04, 3.256206E+04, 3.266698E+04, 3.277205E+04, 3.287727E+04, 
   3.298265E+04, 3.308817E+04, 3.319386E+04, 3.329969E+04, 3.340568E+04, 3.351182E+04, 
   3.361811E+04, 3.372455E+04, 3.383115E+04, 3.393790E+04, 3.404480E+04, 3.415185E+04, 
   3.425905E+04, 3.436641E+04, 3.447391E+04, 3.458157E+04, 3.468937E+04, 3.479733E+04, 
   3.490544E+04, 3.501369E+04, 3.512210E+04, 3.523066E+04, 3.533937E+04, 3.544822E+04, 
   3.555723E+04, 3.566638E+04, 3.577568E+04, 3.588514E+04, 3.599474E+04, 3.610449E+04, 
   3.621438E+04, 
])
# ============================ M = 17, I = 1 ============================
M = 17
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.200000E+01, 1.790094E+01, 3.036750E+01, 4.320574E+01, 5.612178E+01, 6.906819E+01, 
   8.203022E+01, 9.500179E+01, 1.079799E+02, 1.209630E+02, 1.339502E+02, 1.469407E+02, 
   1.599342E+02, 1.729304E+02, 1.859291E+02, 1.989303E+02, 2.119337E+02, 2.249393E+02, 
   2.379470E+02, 2.509568E+02, 2.639687E+02, 2.769826E+02, 2.899986E+02, 3.030166E+02, 
   3.160366E+02, 3.290587E+02, 3.420830E+02, 3.551096E+02, 3.681386E+02, 3.811701E+02, 
   3.942045E+02, 4.072418E+02, 4.202826E+02, 4.333272E+02, 4.463759E+02, 4.594293E+02, 
   4.724880E+02, 4.855527E+02, 4.986238E+02, 5.117023E+02, 5.247889E+02, 5.378846E+02, 
   5.509901E+02, 5.641066E+02, 5.772350E+02, 5.903763E+02, 6.035318E+02, 6.167025E+02, 
   6.298897E+02, 6.430945E+02, 6.563182E+02, 6.695620E+02, 6.828272E+02, 6.961151E+02, 
   7.094270E+02, 7.227642E+02, 7.361281E+02, 7.495199E+02, 7.629408E+02, 7.763924E+02, 
   7.898758E+02, 8.033923E+02, 8.169433E+02, 8.305299E+02, 8.441535E+02, 8.578153E+02, 
   8.715165E+02, 8.852583E+02, 8.990419E+02, 9.128685E+02, 9.267392E+02, 9.406551E+02, 
   9.546175E+02, 9.686273E+02, 9.826856E+02, 9.967934E+02, 1.010952E+03, 1.025162E+03, 
   1.039424E+03, 1.053741E+03, 1.068111E+03, 1.082537E+03, 1.097019E+03, 1.111558E+03, 
   1.126156E+03, 1.140812E+03, 1.155527E+03, 1.170304E+03, 1.185141E+03, 1.200040E+03, 
   1.215002E+03, 1.230027E+03, 1.245116E+03, 1.260271E+03, 1.275490E+03, 1.290776E+03, 
   1.306128E+03, 1.321547E+03, 1.337035E+03, 1.352591E+03, 1.368216E+03, 1.383910E+03, 
   1.399675E+03, 1.415511E+03, 1.431417E+03, 1.447396E+03, 1.463446E+03, 1.479570E+03, 
   1.495766E+03, 1.512036E+03, 1.528380E+03, 1.544798E+03, 1.561292E+03, 1.577860E+03, 
   1.594505E+03, 1.611225E+03, 1.628022E+03, 1.644896E+03, 1.661847E+03, 1.678876E+03, 
   1.695982E+03, 1.713167E+03, 1.730431E+03, 1.747773E+03, 1.765195E+03, 1.782697E+03, 
   1.800278E+03, 1.817940E+03, 1.835682E+03, 1.853504E+03, 1.871408E+03, 1.889394E+03, 
   1.907461E+03, 1.925610E+03, 1.943841E+03, 1.962154E+03, 1.980550E+03, 1.999029E+03, 
   2.017592E+03, 2.036237E+03, 2.054967E+03, 2.073780E+03, 2.092677E+03, 2.111659E+03, 
   2.130725E+03, 2.149876E+03, 2.169112E+03, 2.188434E+03, 2.207840E+03, 2.227333E+03, 
   2.246911E+03, 2.266575E+03, 2.286325E+03, 2.306162E+03, 2.326085E+03, 2.346095E+03, 
   2.366192E+03, 2.386376E+03, 2.406647E+03, 2.427006E+03, 2.447452E+03, 2.467986E+03, 
   2.488609E+03, 2.509319E+03, 2.530118E+03, 2.551005E+03, 2.571981E+03, 2.593045E+03, 
   2.614199E+03, 2.635442E+03, 2.656774E+03, 2.678195E+03, 2.699706E+03, 2.721306E+03, 
   2.742997E+03, 2.764777E+03, 2.786647E+03, 2.808608E+03, 2.830659E+03, 2.852801E+03, 
   2.875033E+03, 2.897356E+03, 2.919770E+03, 2.942275E+03, 2.964871E+03, 2.987559E+03, 
   3.010338E+03, 3.033208E+03, 3.056170E+03, 3.079224E+03, 3.102370E+03, 3.125608E+03, 
   3.148938E+03, 3.172361E+03, 3.195875E+03, 3.219483E+03, 3.243182E+03, 3.266975E+03, 
   3.290860E+03, 3.314839E+03, 3.338910E+03, 3.363074E+03, 3.387332E+03, 3.411683E+03, 
   3.436128E+03, 3.460666E+03, 3.485298E+03, 3.510023E+03, 3.534843E+03, 3.559756E+03, 
   3.584764E+03, 3.609865E+03, 3.635061E+03, 3.660351E+03, 3.685735E+03, 3.711215E+03, 
   3.736788E+03, 3.762456E+03, 3.788220E+03, 3.814077E+03, 3.840030E+03, 3.866078E+03, 
   3.892221E+03, 3.918459E+03, 3.944793E+03, 3.971222E+03, 3.997746E+03, 4.024366E+03, 
   4.051081E+03, 4.077892E+03, 4.104799E+03, 4.131801E+03, 4.158899E+03, 4.186094E+03, 
   4.213384E+03, 4.240770E+03, 4.268253E+03, 4.295832E+03, 4.323507E+03, 4.351278E+03, 
   4.379146E+03, 4.407110E+03, 4.435171E+03, 4.463328E+03, 4.491582E+03, 4.519933E+03, 
   4.548380E+03, 4.576925E+03, 4.605566E+03, 4.634304E+03, 4.663139E+03, 4.692071E+03, 
   4.721100E+03, 4.750227E+03, 4.779450E+03, 4.808771E+03, 4.838189E+03, 4.867705E+03, 
   4.897317E+03, 4.927028E+03, 4.956835E+03, 4.986741E+03, 5.016743E+03, 5.046844E+03, 
   5.077042E+03, 5.107337E+03, 5.137731E+03, 5.168222E+03, 5.198811E+03, 5.229497E+03, 
   5.260282E+03, 5.291164E+03, 5.322144E+03, 5.353223E+03, 5.384399E+03, 5.415673E+03, 
   5.447045E+03, 5.478516E+03, 5.510084E+03, 5.541751E+03, 5.573515E+03, 5.605378E+03, 
   5.637339E+03, 5.669398E+03, 5.701556E+03, 5.733811E+03, 5.766165E+03, 5.798617E+03, 
   5.831168E+03, 5.863816E+03, 5.896563E+03, 5.929409E+03, 5.962352E+03, 5.995395E+03, 
   6.028535E+03, 6.061774E+03, 6.095111E+03, 6.128547E+03, 6.162081E+03, 6.195713E+03, 
   6.229444E+03, 6.263273E+03, 6.297201E+03, 6.331227E+03, 6.365351E+03, 6.399574E+03, 
   6.433896E+03, 6.468315E+03, 6.502834E+03, 6.537450E+03, 6.572165E+03, 6.606979E+03, 
   6.641891E+03, 6.676901E+03, 6.712010E+03, 6.747217E+03, 6.782522E+03, 6.817926E+03, 
   6.853428E+03, 6.889029E+03, 6.924728E+03, 6.960525E+03, 6.996420E+03, 7.032414E+03, 
   7.068506E+03, 7.104697E+03, 7.140985E+03, 7.177372E+03, 7.213857E+03, 7.250440E+03, 
   7.287122E+03, 7.323901E+03, 7.360779E+03, 7.397755E+03, 7.434829E+03, 7.472001E+03, 
   7.509271E+03, 7.546639E+03, 7.584105E+03, 7.621669E+03, 7.659331E+03, 7.697090E+03, 
   7.734948E+03, 7.772903E+03, 7.810957E+03, 7.849108E+03, 7.887357E+03, 7.925703E+03, 
   7.964148E+03, 8.002689E+03, 8.041329E+03, 8.080066E+03, 8.118901E+03, 8.157833E+03, 
   8.196862E+03, 8.235989E+03, 8.275214E+03, 8.314535E+03, 8.353954E+03, 8.393471E+03, 
   8.433084E+03, 8.472795E+03, 8.512603E+03, 8.552507E+03, 8.592509E+03, 8.632608E+03, 
   8.672804E+03, 8.713097E+03, 8.753486E+03, 8.793973E+03, 8.834556E+03, 8.875236E+03, 
   8.916012E+03, 8.956886E+03, 8.997855E+03, 9.038922E+03, 9.080084E+03, 9.121343E+03, 
   9.162699E+03, 9.204150E+03, 9.245698E+03, 9.287342E+03, 9.329082E+03, 9.370919E+03, 
   9.412851E+03, 9.454879E+03, 9.497003E+03, 9.539223E+03, 9.581539E+03, 9.623950E+03, 
   9.666457E+03, 9.709059E+03, 9.751757E+03, 9.794551E+03, 9.837440E+03, 9.880424E+03, 
   9.923503E+03, 9.966678E+03, 1.000995E+04, 1.005331E+04, 1.009677E+04, 1.014033E+04, 
   1.018398E+04, 1.022772E+04, 1.027156E+04, 1.031549E+04, 1.035952E+04, 1.040365E+04, 
   1.044786E+04, 1.049217E+04, 1.053658E+04, 1.058108E+04, 1.062567E+04, 1.067036E+04, 
   1.071515E+04, 1.076002E+04, 1.080499E+04, 1.085006E+04, 1.089521E+04, 1.094046E+04, 
   1.098581E+04, 1.103125E+04, 1.107678E+04, 1.112240E+04, 1.116812E+04, 1.121393E+04, 
   1.125984E+04, 1.130583E+04, 1.135192E+04, 1.139811E+04, 1.144438E+04, 1.149075E+04, 
   1.153721E+04, 1.158376E+04, 1.163041E+04, 1.167715E+04, 1.172398E+04, 1.177090E+04, 
   1.181791E+04, 1.186502E+04, 1.191222E+04, 1.195951E+04, 1.200689E+04, 1.205436E+04, 
   1.210193E+04, 1.214958E+04, 1.219733E+04, 1.224517E+04, 1.229310E+04, 1.234112E+04, 
   1.238923E+04, 1.243743E+04, 1.248573E+04, 1.253411E+04, 1.258259E+04, 1.263115E+04, 
   1.267981E+04, 1.272855E+04, 1.277739E+04, 1.282632E+04, 1.287533E+04, 1.292444E+04, 
   1.297363E+04, 1.302292E+04, 1.307230E+04, 1.312176E+04, 1.317132E+04, 1.322096E+04, 
   1.327069E+04, 1.332051E+04, 1.337043E+04, 1.342043E+04, 1.347052E+04, 1.352069E+04, 
   1.357096E+04, 1.362131E+04, 1.367176E+04, 1.372229E+04, 1.377291E+04, 1.382362E+04, 
   1.387441E+04, 1.392530E+04, 1.397627E+04, 1.402733E+04, 1.407848E+04, 1.412971E+04, 
   1.418103E+04, 1.423244E+04, 1.428394E+04, 1.433552E+04, 1.438719E+04, 1.443895E+04, 
   1.449080E+04, 1.454273E+04, 1.459474E+04, 1.464685E+04, 1.469904E+04, 1.475132E+04, 
   1.480368E+04, 1.485613E+04, 1.490866E+04, 1.496128E+04, 1.501399E+04, 1.506678E+04, 
   1.511966E+04, 1.517262E+04, 1.522567E+04, 1.527880E+04, 1.533202E+04, 1.538532E+04, 
   1.543871E+04, 1.549218E+04, 1.554573E+04, 1.559938E+04, 1.565310E+04, 1.570691E+04, 
   1.576080E+04, 1.581478E+04, 1.586884E+04, 1.592299E+04, 1.597722E+04, 1.603153E+04, 
   1.608592E+04, 1.614040E+04, 1.619497E+04, 1.624961E+04, 1.630434E+04, 1.635915E+04, 
   1.641404E+04, 1.646902E+04, 1.652408E+04, 1.657922E+04, 1.663444E+04, 1.668975E+04, 
   1.674513E+04, 1.680060E+04, 1.685615E+04, 1.691179E+04, 1.696750E+04, 1.702330E+04, 
   1.707917E+04, 1.713513E+04, 1.719117E+04, 1.724729E+04, 1.730349E+04, 1.735977E+04, 
   1.741613E+04, 1.747257E+04, 1.752909E+04, 1.758570E+04, 1.764238E+04, 1.769914E+04, 
   1.775598E+04, 1.781290E+04, 1.786991E+04, 1.792699E+04, 1.798415E+04, 1.804139E+04, 
   1.809870E+04, 1.815610E+04, 1.821358E+04, 1.827113E+04, 1.832876E+04, 1.838647E+04, 
   1.844426E+04, 1.850213E+04, 1.856008E+04, 1.861810E+04, 1.867620E+04, 1.873438E+04, 
   1.879264E+04, 1.885097E+04, 1.890938E+04, 1.896787E+04, 1.902644E+04, 1.908508E+04, 
   1.914380E+04, 1.920260E+04, 1.926147E+04, 1.932042E+04, 1.937944E+04, 1.943854E+04, 
   1.949772E+04, 1.955697E+04, 1.961630E+04, 1.967570E+04, 1.973518E+04, 1.979474E+04, 
   1.985437E+04, 1.991407E+04, 1.997385E+04, 2.003371E+04, 2.009364E+04, 2.015364E+04, 
   2.021372E+04, 2.027387E+04, 2.033410E+04, 2.039440E+04, 2.045477E+04, 2.051522E+04, 
   2.057574E+04, 
])
# ============================ M = 17, I = 2 ============================
M = 17
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.800464E+01, 4.507463E+01, 8.321075E+01, 1.215759E+02, 1.599981E+02, 1.984457E+02, 
   2.369086E+02, 2.753824E+02, 3.138651E+02, 3.523553E+02, 3.908526E+02, 4.293563E+02, 
   4.678662E+02, 5.063822E+02, 5.449041E+02, 5.834318E+02, 6.219653E+02, 6.605048E+02, 
   6.990504E+02, 7.376026E+02, 7.761621E+02, 8.147297E+02, 8.533068E+02, 8.918950E+02, 
   9.304964E+02, 9.691138E+02, 1.007750E+03, 1.046409E+03, 1.085094E+03, 1.123811E+03, 
   1.162563E+03, 1.201357E+03, 1.240199E+03, 1.279094E+03, 1.318049E+03, 1.357072E+03, 
   1.396167E+03, 1.435344E+03, 1.474609E+03, 1.513970E+03, 1.553433E+03, 1.593008E+03, 
   1.632699E+03, 1.672517E+03, 1.712467E+03, 1.752556E+03, 1.792793E+03, 1.833184E+03, 
   1.873736E+03, 1.914455E+03, 1.955349E+03, 1.996424E+03, 2.037686E+03, 2.079140E+03, 
   2.120794E+03, 2.162653E+03, 2.204722E+03, 2.247006E+03, 2.289512E+03, 2.332244E+03, 
   2.375206E+03, 2.418404E+03, 2.461843E+03, 2.505525E+03, 2.549457E+03, 2.593641E+03, 
   2.638082E+03, 2.682784E+03, 2.727749E+03, 2.772983E+03, 2.818487E+03, 2.864266E+03, 
   2.910323E+03, 2.956660E+03, 3.003280E+03, 3.050187E+03, 3.097383E+03, 3.144870E+03, 
   3.192652E+03, 3.240730E+03, 3.289107E+03, 3.337786E+03, 3.386767E+03, 3.436055E+03, 
   3.485650E+03, 3.535554E+03, 3.585770E+03, 3.636299E+03, 3.687143E+03, 3.738304E+03, 
   3.789784E+03, 3.841583E+03, 3.893705E+03, 3.946149E+03, 3.998918E+03, 4.052013E+03, 
   4.105435E+03, 4.159186E+03, 4.213267E+03, 4.267680E+03, 4.322425E+03, 4.377503E+03, 
   4.432917E+03, 4.488666E+03, 4.544753E+03, 4.601178E+03, 4.657942E+03, 4.715046E+03, 
   4.772491E+03, 4.830278E+03, 4.888409E+03, 4.946883E+03, 5.005702E+03, 5.064867E+03, 
   5.124379E+03, 5.184238E+03, 5.244445E+03, 5.305001E+03, 5.365907E+03, 5.427164E+03, 
   5.488771E+03, 5.550731E+03, 5.613043E+03, 5.675709E+03, 5.738728E+03, 5.802103E+03, 
   5.865832E+03, 5.929918E+03, 5.994360E+03, 6.059159E+03, 6.124317E+03, 6.189832E+03, 
   6.255707E+03, 6.321941E+03, 6.388535E+03, 6.455489E+03, 6.522805E+03, 6.590483E+03, 
   6.658522E+03, 6.726924E+03, 6.795690E+03, 6.864819E+03, 6.934312E+03, 7.004169E+03, 
   7.074392E+03, 7.144980E+03, 7.215933E+03, 7.287253E+03, 7.358940E+03, 7.430994E+03, 
   7.503415E+03, 7.576204E+03, 7.649362E+03, 7.722888E+03, 7.796783E+03, 7.871047E+03, 
   7.945681E+03, 8.020685E+03, 8.096060E+03, 8.171805E+03, 8.247921E+03, 8.324409E+03, 
   8.401268E+03, 8.478499E+03, 8.556102E+03, 8.634078E+03, 8.712426E+03, 8.791147E+03, 
   8.870242E+03, 8.949710E+03, 9.029552E+03, 9.109768E+03, 9.190358E+03, 9.271323E+03, 
   9.352662E+03, 9.434376E+03, 9.516465E+03, 9.598930E+03, 9.681769E+03, 9.764985E+03, 
   9.848576E+03, 9.932543E+03, 1.001689E+04, 1.010161E+04, 1.018670E+04, 1.027217E+04, 
   1.035802E+04, 1.044425E+04, 1.053085E+04, 1.061783E+04, 1.070519E+04, 1.079292E+04, 
   1.088103E+04, 1.096952E+04, 1.105839E+04, 1.114763E+04, 1.123725E+04, 1.132725E+04, 
   1.141763E+04, 1.150838E+04, 1.159951E+04, 1.169102E+04, 1.178291E+04, 1.187517E+04, 
   1.196781E+04, 1.206083E+04, 1.215423E+04, 1.224801E+04, 1.234216E+04, 1.243669E+04, 
   1.253160E+04, 1.262689E+04, 1.272255E+04, 1.281860E+04, 1.291501E+04, 1.301181E+04, 
   1.310899E+04, 1.320654E+04, 1.330447E+04, 1.340277E+04, 1.350145E+04, 1.360051E+04, 
   1.369995E+04, 1.379976E+04, 1.389995E+04, 1.400052E+04, 1.410146E+04, 1.420278E+04, 
   1.430448E+04, 1.440655E+04, 1.450899E+04, 1.461182E+04, 1.471501E+04, 1.481859E+04, 
   1.492253E+04, 1.502686E+04, 1.513156E+04, 1.523663E+04, 1.534207E+04, 1.544790E+04, 
   1.555409E+04, 1.566066E+04, 1.576760E+04, 1.587492E+04, 1.598261E+04, 1.609067E+04, 
   1.619910E+04, 1.630791E+04, 1.641709E+04, 1.652664E+04, 1.663656E+04, 1.674685E+04, 
   1.685752E+04, 1.696855E+04, 1.707996E+04, 1.719174E+04, 1.730388E+04, 1.741640E+04, 
   1.752928E+04, 1.764254E+04, 1.775616E+04, 1.787016E+04, 1.798452E+04, 1.809924E+04, 
   1.821434E+04, 1.832980E+04, 1.844564E+04, 1.856183E+04, 1.867840E+04, 1.879533E+04, 
   1.891262E+04, 1.903028E+04, 1.914831E+04, 1.926670E+04, 1.938545E+04, 1.950457E+04, 
   1.962405E+04, 1.974390E+04, 1.986410E+04, 1.998467E+04, 2.010561E+04, 2.022690E+04, 
   2.034856E+04, 2.047057E+04, 2.059295E+04, 2.071568E+04, 2.083878E+04, 2.096224E+04, 
   2.108605E+04, 2.121023E+04, 2.133476E+04, 2.145965E+04, 2.158489E+04, 2.171050E+04, 
   2.183646E+04, 2.196277E+04, 2.208945E+04, 2.221647E+04, 2.234386E+04, 2.247159E+04, 
   2.259968E+04, 2.272813E+04, 2.285692E+04, 2.298607E+04, 2.311558E+04, 2.324543E+04, 
   2.337563E+04, 2.350619E+04, 2.363710E+04, 2.376835E+04, 2.389996E+04, 2.403191E+04, 
   2.416422E+04, 2.429687E+04, 2.442987E+04, 2.456321E+04, 2.469691E+04, 2.483094E+04, 
   2.496533E+04, 2.510006E+04, 2.523514E+04, 2.537056E+04, 2.550632E+04, 2.564243E+04, 
   2.577888E+04, 2.591567E+04, 2.605280E+04, 2.619028E+04, 2.632810E+04, 2.646625E+04, 
   2.660475E+04, 2.674359E+04, 2.688276E+04, 2.702228E+04, 2.716213E+04, 2.730232E+04, 
   2.744285E+04, 2.758371E+04, 2.772491E+04, 2.786645E+04, 2.800832E+04, 2.815052E+04, 
   2.829306E+04, 2.843593E+04, 2.857914E+04, 2.872268E+04, 2.886655E+04, 2.901075E+04, 
   2.915528E+04, 2.930015E+04, 2.944534E+04, 2.959086E+04, 2.973671E+04, 2.988289E+04, 
   3.002940E+04, 3.017624E+04, 3.032340E+04, 3.047089E+04, 3.061870E+04, 3.076684E+04, 
   3.091531E+04, 3.106409E+04, 3.121321E+04, 3.136264E+04, 3.151240E+04, 3.166248E+04, 
   3.181289E+04, 3.196361E+04, 3.211465E+04, 3.226602E+04, 3.241770E+04, 3.256970E+04, 
   3.272203E+04, 3.287467E+04, 3.302762E+04, 3.318090E+04, 3.333449E+04, 3.348839E+04, 
   3.364262E+04, 3.379715E+04, 3.395200E+04, 3.410717E+04, 3.426265E+04, 3.441844E+04, 
   3.457454E+04, 3.473095E+04, 3.488768E+04, 3.504472E+04, 3.520206E+04, 3.535972E+04, 
   3.551768E+04, 3.567596E+04, 3.583454E+04, 3.599343E+04, 3.615262E+04, 3.631213E+04, 
   3.647193E+04, 3.663205E+04, 3.679246E+04, 3.695319E+04, 3.711421E+04, 3.727554E+04, 
   3.743717E+04, 3.759911E+04, 3.776134E+04, 3.792388E+04, 3.808672E+04, 3.824985E+04, 
   3.841329E+04, 3.857703E+04, 3.874106E+04, 3.890539E+04, 3.907002E+04, 3.923495E+04, 
   3.940017E+04, 3.956569E+04, 3.973150E+04, 3.989761E+04, 4.006401E+04, 4.023071E+04, 
   4.039770E+04, 4.056498E+04, 4.073255E+04, 4.090042E+04, 4.106857E+04, 4.123702E+04, 
   4.140576E+04, 4.157478E+04, 4.174410E+04, 4.191370E+04, 4.208359E+04, 4.225377E+04, 
   4.242424E+04, 4.259499E+04, 4.276602E+04, 4.293735E+04, 4.310895E+04, 4.328085E+04, 
   4.345302E+04, 4.362548E+04, 4.379822E+04, 4.397124E+04, 4.414455E+04, 4.431813E+04, 
   4.449200E+04, 4.466614E+04, 4.484057E+04, 4.501527E+04, 4.519026E+04, 4.536552E+04, 
   4.554105E+04, 4.571687E+04, 4.589296E+04, 4.606933E+04, 4.624597E+04, 4.642289E+04, 
   4.660008E+04, 4.677754E+04, 4.695528E+04, 4.713329E+04, 4.731158E+04, 4.749013E+04, 
   4.766896E+04, 4.784806E+04, 4.802742E+04, 4.820706E+04, 4.838697E+04, 4.856714E+04, 
   4.874758E+04, 4.892829E+04, 4.910927E+04, 4.929051E+04, 4.947202E+04, 4.965380E+04, 
   4.983584E+04, 5.001814E+04, 5.020071E+04, 5.038355E+04, 5.056664E+04, 5.075000E+04, 
   5.093362E+04, 5.111750E+04, 5.130164E+04, 5.148605E+04, 5.167071E+04, 5.185563E+04, 
   5.204081E+04, 5.222625E+04, 5.241195E+04, 5.259791E+04, 5.278412E+04, 5.297059E+04, 
   5.315731E+04, 5.334429E+04, 5.353153E+04, 5.371901E+04, 5.390676E+04, 5.409475E+04, 
   5.428300E+04, 5.447151E+04, 5.466026E+04, 5.484926E+04, 5.503852E+04, 5.522803E+04, 
   5.541778E+04, 5.560779E+04, 5.579805E+04, 5.598855E+04, 5.617930E+04, 5.637030E+04, 
   5.656155E+04, 5.675304E+04, 5.694478E+04, 5.713676E+04, 5.732899E+04, 5.752146E+04, 
   5.771418E+04, 5.790714E+04, 5.810034E+04, 5.829379E+04, 5.848748E+04, 5.868141E+04, 
   5.887558E+04, 5.906999E+04, 5.926464E+04, 5.945954E+04, 5.965467E+04, 5.985004E+04, 
   6.004564E+04, 6.024149E+04, 6.043757E+04, 6.063389E+04, 6.083045E+04, 6.102724E+04, 
   6.122426E+04, 6.142152E+04, 6.161902E+04, 6.181675E+04, 6.201471E+04, 6.221291E+04, 
   6.241133E+04, 6.260999E+04, 6.280888E+04, 6.300801E+04, 6.320736E+04, 6.340694E+04, 
   6.360675E+04, 6.380679E+04, 6.400706E+04, 6.420756E+04, 6.440828E+04, 6.460924E+04, 
   6.481042E+04, 6.501182E+04, 6.521345E+04, 6.541531E+04, 6.561739E+04, 6.581969E+04, 
   6.602222E+04, 6.622497E+04, 6.642795E+04, 6.663114E+04, 6.683456E+04, 6.703820E+04, 
   6.724206E+04, 6.744615E+04, 6.765045E+04, 6.785497E+04, 6.805971E+04, 6.826467E+04, 
   6.846985E+04, 6.867524E+04, 6.888085E+04, 6.908668E+04, 6.929273E+04, 6.949899E+04, 
   6.970546E+04, 6.991215E+04, 7.011906E+04, 7.032618E+04, 7.053351E+04, 7.074106E+04, 
   7.094882E+04, 7.115679E+04, 7.136497E+04, 7.157336E+04, 7.178197E+04, 7.199078E+04, 
   7.219981E+04, 7.240904E+04, 7.261848E+04, 7.282813E+04, 7.303799E+04, 7.324806E+04, 
   7.345833E+04, 7.366881E+04, 7.387950E+04, 7.409039E+04, 7.430148E+04, 7.451279E+04, 
   7.472429E+04, 
])
# ============================ M = 18, I = 1 ============================
M = 18
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.214341E+01, 1.052089E+02, 1.944698E+02, 2.840543E+02, 3.737290E+02, 4.634821E+02, 
   5.534263E+02, 6.438126E+02, 7.349974E+02, 8.273813E+02, 9.213526E+02, 1.017252E+03, 
   1.115359E+03, 1.215886E+03, 1.318993E+03, 1.424790E+03, 1.533350E+03, 1.644721E+03, 
   1.758929E+03, 1.875989E+03, 1.995905E+03, 2.118677E+03, 2.244300E+03, 2.372771E+03, 
   2.504084E+03, 2.638235E+03, 2.775220E+03, 2.915035E+03, 3.057681E+03, 3.203155E+03, 
   3.351458E+03, 3.502591E+03, 3.656557E+03, 3.813358E+03, 3.972996E+03, 4.135477E+03, 
   4.300803E+03, 4.468978E+03, 4.640008E+03, 4.813896E+03, 4.990647E+03, 5.170265E+03, 
   5.352756E+03, 5.538122E+03, 5.726370E+03, 5.917503E+03, 6.111526E+03, 6.308442E+03, 
   6.508256E+03, 6.710971E+03, 6.916592E+03, 7.125122E+03, 7.336564E+03, 7.550922E+03, 
   7.768200E+03, 7.988401E+03, 8.211527E+03, 8.437581E+03, 8.666567E+03, 8.898488E+03, 
   9.133345E+03, 9.371142E+03, 9.611880E+03, 9.855563E+03, 1.010219E+04, 1.035177E+04, 
   1.060430E+04, 1.085978E+04, 1.111822E+04, 1.137961E+04, 1.164397E+04, 1.191128E+04, 
   1.218155E+04, 1.245479E+04, 1.273100E+04, 1.301017E+04, 1.329231E+04, 1.357742E+04, 
   1.386550E+04, 1.415656E+04, 1.445059E+04, 1.474759E+04, 1.504758E+04, 1.535054E+04, 
   1.565648E+04, 1.596540E+04, 1.627730E+04, 1.659219E+04, 1.691006E+04, 1.723091E+04, 
   1.755475E+04, 1.788158E+04, 1.821140E+04, 1.854420E+04, 1.888000E+04, 1.921878E+04, 
   1.956056E+04, 1.990533E+04, 2.025310E+04, 2.060386E+04, 2.095761E+04, 2.131436E+04, 
   2.167411E+04, 2.203686E+04, 2.240261E+04, 2.277135E+04, 2.314310E+04, 2.351785E+04, 
   2.389560E+04, 2.427635E+04, 2.466010E+04, 2.504686E+04, 2.543663E+04, 2.582940E+04, 
   2.622518E+04, 2.662396E+04, 2.702575E+04, 2.743055E+04, 2.783836E+04, 2.824918E+04, 
   2.866301E+04, 2.907985E+04, 2.949971E+04, 2.992257E+04, 3.034845E+04, 3.077734E+04, 
   3.120925E+04, 3.164417E+04, 3.208211E+04, 3.252306E+04, 3.296703E+04, 3.341402E+04, 
   3.386403E+04, 3.431705E+04, 3.477310E+04, 3.523216E+04, 3.569424E+04, 3.615935E+04, 
   3.662748E+04, 3.709863E+04, 3.757280E+04, 3.805000E+04, 3.853021E+04, 3.901346E+04, 
   3.949973E+04, 3.998902E+04, 4.048135E+04, 4.097669E+04, 4.147507E+04, 4.197647E+04, 
   4.248091E+04, 4.298837E+04, 4.349886E+04, 4.401238E+04, 4.452893E+04, 4.504851E+04, 
   4.557113E+04, 4.609678E+04, 4.662546E+04, 4.715717E+04, 4.769192E+04, 4.822970E+04, 
   4.877052E+04, 4.931437E+04, 4.986126E+04, 5.041119E+04, 5.096415E+04, 5.152015E+04, 
   5.207919E+04, 5.264127E+04, 5.320639E+04, 5.377454E+04, 5.434574E+04, 5.491998E+04, 
   5.549726E+04, 5.607758E+04, 5.666095E+04, 5.724736E+04, 5.783681E+04, 5.842930E+04, 
   5.902484E+04, 5.962343E+04, 6.022506E+04, 6.082974E+04, 6.143746E+04, 6.204823E+04, 
   6.266205E+04, 6.327892E+04, 6.389884E+04, 6.452180E+04, 6.514782E+04, 6.577688E+04, 
   6.640900E+04, 6.704417E+04, 6.768239E+04, 6.832366E+04, 6.896799E+04, 6.961537E+04, 
   7.026580E+04, 7.091929E+04, 7.157583E+04, 7.223543E+04, 7.289808E+04, 7.356379E+04, 
   7.423256E+04, 7.490438E+04, 7.557927E+04, 7.625721E+04, 7.693821E+04, 7.762227E+04, 
   7.830939E+04, 7.899957E+04, 7.969282E+04, 8.038912E+04, 8.108849E+04, 8.179092E+04, 
   8.249641E+04, 8.320497E+04, 8.391659E+04, 8.463128E+04, 8.534903E+04, 8.606985E+04, 
   8.679373E+04, 8.752068E+04, 8.825070E+04, 8.898379E+04, 8.971994E+04, 9.045917E+04, 
   9.120146E+04, 9.194682E+04, 9.269526E+04, 9.344676E+04, 9.420134E+04, 9.495899E+04, 
   9.571971E+04, 9.648350E+04, 9.725037E+04, 9.802031E+04, 9.879333E+04, 9.956942E+04, 
   1.003486E+05, 1.011308E+05, 1.019161E+05, 1.027046E+05, 1.034960E+05, 1.042906E+05, 
   1.050882E+05, 1.058889E+05, 1.066927E+05, 1.074996E+05, 1.083095E+05, 1.091226E+05, 
   1.099387E+05, 1.107579E+05, 1.115802E+05, 1.124055E+05, 1.132340E+05, 1.140655E+05, 
   1.149001E+05, 1.157378E+05, 1.165786E+05, 1.174225E+05, 1.182695E+05, 1.191195E+05, 
   1.199727E+05, 1.208289E+05, 1.216882E+05, 1.225506E+05, 1.234161E+05, 1.242847E+05, 
   1.251564E+05, 1.260311E+05, 1.269090E+05, 1.277900E+05, 1.286740E+05, 1.295612E+05, 
   1.304514E+05, 1.313447E+05, 1.322412E+05, 1.331407E+05, 1.340433E+05, 1.349490E+05, 
   1.358579E+05, 1.367698E+05, 1.376848E+05, 1.386029E+05, 1.395241E+05, 1.404484E+05, 
   1.413758E+05, 1.423064E+05, 1.432400E+05, 1.441767E+05, 1.451165E+05, 1.460594E+05, 
   1.470055E+05, 1.479546E+05, 1.489068E+05, 1.498622E+05, 1.508206E+05, 1.517822E+05, 
   1.527469E+05, 1.537146E+05, 1.546855E+05, 1.556595E+05, 1.566366E+05, 1.576168E+05, 
   1.586001E+05, 1.595865E+05, 1.605761E+05, 1.615687E+05, 1.625645E+05, 1.635634E+05, 
   1.645654E+05, 1.655705E+05, 1.665787E+05, 1.675900E+05, 1.686045E+05, 1.696221E+05, 
   1.706427E+05, 1.716665E+05, 1.726935E+05, 1.737235E+05, 1.747567E+05, 1.757929E+05, 
   1.768323E+05, 1.778749E+05, 1.789205E+05, 1.799692E+05, 1.810211E+05, 1.820761E+05, 
   1.831343E+05, 1.841955E+05, 1.852599E+05, 1.863274E+05, 1.873980E+05, 1.884718E+05, 
   1.895486E+05, 1.906286E+05, 1.917118E+05, 1.927980E+05, 1.938874E+05, 1.949800E+05, 
   1.960756E+05, 1.971744E+05, 1.982763E+05, 1.993813E+05, 2.004895E+05, 2.016008E+05, 
   2.027152E+05, 2.038328E+05, 2.049535E+05, 2.060774E+05, 2.072043E+05, 2.083344E+05, 
   2.094677E+05, 2.106041E+05, 2.117436E+05, 2.128862E+05, 2.140320E+05, 2.151810E+05, 
   2.163330E+05, 2.174883E+05, 2.186466E+05, 2.198081E+05, 2.209727E+05, 2.221405E+05, 
   2.233114E+05, 2.244855E+05, 2.256627E+05, 2.268431E+05, 2.280266E+05, 2.292132E+05, 
   2.304030E+05, 2.315959E+05, 2.327920E+05, 2.339912E+05, 2.351936E+05, 2.363991E+05, 
   2.376078E+05, 2.388196E+05, 2.400346E+05, 2.412527E+05, 2.424740E+05, 2.436984E+05, 
   2.449260E+05, 2.461567E+05, 2.473906E+05, 2.486276E+05, 2.498678E+05, 2.511112E+05, 
   2.523577E+05, 2.536073E+05, 2.548601E+05, 2.561161E+05, 2.573752E+05, 2.586375E+05, 
   2.599030E+05, 2.611716E+05, 2.624433E+05, 2.637183E+05, 2.649963E+05, 2.662776E+05, 
   2.675620E+05, 2.688496E+05, 2.701403E+05, 2.714342E+05, 2.727312E+05, 2.740315E+05, 
   2.753349E+05, 2.766414E+05, 2.779511E+05, 2.792640E+05, 2.805801E+05, 2.818993E+05, 
   2.832217E+05, 2.845472E+05, 2.858759E+05, 2.872078E+05, 2.885429E+05, 2.898811E+05, 
   2.912225E+05, 2.925671E+05, 2.939148E+05, 2.952658E+05, 2.966199E+05, 2.979771E+05, 
   2.993376E+05, 3.007012E+05, 3.020680E+05, 3.034379E+05, 3.048111E+05, 3.061874E+05, 
   3.075669E+05, 3.089495E+05, 3.103354E+05, 3.117244E+05, 3.131166E+05, 3.145120E+05, 
   3.159105E+05, 3.173123E+05, 3.187172E+05, 3.201253E+05, 3.215366E+05, 3.229511E+05, 
   3.243687E+05, 3.257896E+05, 3.272136E+05, 3.286408E+05, 3.300712E+05, 3.315047E+05, 
   3.329415E+05, 3.343814E+05, 3.358246E+05, 3.372709E+05, 3.387204E+05, 3.401731E+05, 
   3.416290E+05, 3.430880E+05, 3.445503E+05, 3.460157E+05, 3.474844E+05, 3.489562E+05, 
   3.504312E+05, 3.519094E+05, 3.533908E+05, 3.548754E+05, 3.563632E+05, 3.578542E+05, 
   3.593483E+05, 3.608457E+05, 3.623463E+05, 3.638500E+05, 3.653570E+05, 3.668671E+05, 
   3.683805E+05, 3.698970E+05, 3.714167E+05, 3.729397E+05, 3.744658E+05, 3.759951E+05, 
   3.775277E+05, 3.790634E+05, 3.806023E+05, 3.821445E+05, 3.836898E+05, 3.852383E+05, 
   3.867900E+05, 3.883450E+05, 3.899031E+05, 3.914644E+05, 3.930290E+05, 3.945967E+05, 
   3.961677E+05, 3.977418E+05, 3.993192E+05, 4.008997E+05, 4.024835E+05, 4.040705E+05, 
   4.056606E+05, 4.072540E+05, 4.088506E+05, 
])
# ============================ M = 18, I = 2 ============================
M = 18
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.225498E+01, 1.067540E+02, 1.975622E+02, 2.886898E+02, 3.799068E+02, 4.712024E+02, 
   5.626919E+02, 6.546304E+02, 7.473806E+02, 8.413500E+02, 9.369337E+02, 1.034479E+03, 
   1.134269E+03, 1.236522E+03, 1.341401E+03, 1.449019E+03, 1.559453E+03, 1.672751E+03, 
   1.788942E+03, 1.908041E+03, 2.030055E+03, 2.154984E+03, 2.282825E+03, 2.413576E+03, 
   2.547231E+03, 2.683787E+03, 2.823240E+03, 2.965589E+03, 3.110831E+03, 3.258968E+03, 
   3.409999E+03, 3.563927E+03, 3.720753E+03, 3.880480E+03, 4.043111E+03, 4.208652E+03, 
   4.377104E+03, 4.548474E+03, 4.722764E+03, 4.899980E+03, 5.080127E+03, 5.263208E+03, 
   5.449229E+03, 5.638194E+03, 5.830108E+03, 6.024974E+03, 6.222797E+03, 6.423582E+03, 
   6.627332E+03, 6.834052E+03, 7.043744E+03, 7.256414E+03, 7.472064E+03, 7.690698E+03, 
   7.912318E+03, 8.136929E+03, 8.364534E+03, 8.595135E+03, 8.828735E+03, 9.065337E+03, 
   9.304944E+03, 9.547558E+03, 9.793182E+03, 1.004182E+04, 1.029347E+04, 1.054814E+04, 
   1.080582E+04, 1.106653E+04, 1.133026E+04, 1.159701E+04, 1.186679E+04, 1.213960E+04, 
   1.241544E+04, 1.269431E+04, 1.297621E+04, 1.326115E+04, 1.354913E+04, 1.384014E+04, 
   1.413419E+04, 1.443129E+04, 1.473142E+04, 1.503460E+04, 1.534083E+04, 1.565010E+04, 
   1.596242E+04, 1.627778E+04, 1.659620E+04, 1.691766E+04, 1.724218E+04, 1.756975E+04, 
   1.790038E+04, 1.823406E+04, 1.857080E+04, 1.891059E+04, 1.925344E+04, 1.959935E+04, 
   1.994832E+04, 2.030035E+04, 2.065545E+04, 2.101360E+04, 2.137482E+04, 2.173911E+04, 
   2.210646E+04, 2.247687E+04, 2.285035E+04, 2.322690E+04, 2.360652E+04, 2.398921E+04, 
   2.437497E+04, 2.476379E+04, 2.515569E+04, 2.555066E+04, 2.594871E+04, 2.634983E+04, 
   2.675402E+04, 2.716129E+04, 2.757163E+04, 2.798505E+04, 2.840155E+04, 2.882113E+04, 
   2.924378E+04, 2.966952E+04, 3.009833E+04, 3.053022E+04, 3.096520E+04, 3.140326E+04, 
   3.184440E+04, 3.228862E+04, 3.273593E+04, 3.318632E+04, 3.363979E+04, 3.409636E+04, 
   3.455601E+04, 3.501874E+04, 3.548456E+04, 3.595348E+04, 3.642548E+04, 3.690057E+04, 
   3.737874E+04, 3.786001E+04, 3.834438E+04, 3.883183E+04, 3.932237E+04, 3.981601E+04, 
   4.031274E+04, 4.081257E+04, 4.131549E+04, 4.182150E+04, 4.233061E+04, 4.284282E+04, 
   4.335812E+04, 4.387653E+04, 4.439803E+04, 4.492262E+04, 4.545032E+04, 4.598112E+04, 
   4.651501E+04, 4.705201E+04, 4.759211E+04, 4.813532E+04, 4.868162E+04, 4.923103E+04, 
   4.978354E+04, 5.033915E+04, 5.089787E+04, 5.145970E+04, 5.202463E+04, 5.259266E+04, 
   5.316381E+04, 5.373806E+04, 5.431542E+04, 5.489588E+04, 5.547946E+04, 5.606615E+04, 
   5.665594E+04, 5.724885E+04, 5.784487E+04, 5.844400E+04, 5.904624E+04, 5.965160E+04, 
   6.026006E+04, 6.087165E+04, 6.148634E+04, 6.210415E+04, 6.272508E+04, 6.334912E+04, 
   6.397628E+04, 6.460656E+04, 6.523995E+04, 6.587647E+04, 6.651610E+04, 6.715885E+04, 
   6.780472E+04, 6.845371E+04, 6.910582E+04, 6.976105E+04, 7.041940E+04, 7.108088E+04, 
   7.174548E+04, 7.241320E+04, 7.308405E+04, 7.375802E+04, 7.443511E+04, 7.511534E+04, 
   7.579868E+04, 7.648516E+04, 7.717476E+04, 7.786749E+04, 7.856334E+04, 7.926233E+04, 
   7.996444E+04, 8.066969E+04, 8.137806E+04, 8.208957E+04, 8.280421E+04, 8.352198E+04, 
   8.424288E+04, 8.496691E+04, 8.569408E+04, 8.642438E+04, 8.715782E+04, 8.789439E+04, 
   8.863409E+04, 8.937693E+04, 9.012291E+04, 9.087203E+04, 9.162429E+04, 9.237968E+04, 
   9.313821E+04, 9.389988E+04, 9.466469E+04, 9.543264E+04, 9.620373E+04, 9.697796E+04, 
   9.775533E+04, 9.853585E+04, 9.931951E+04, 1.001063E+05, 1.008963E+05, 1.016894E+05, 
   1.024856E+05, 1.032850E+05, 1.040875E+05, 1.048932E+05, 1.057020E+05, 1.065140E+05, 
   1.073291E+05, 1.081474E+05, 1.089688E+05, 1.097933E+05, 1.106210E+05, 1.114519E+05, 
   1.122859E+05, 1.131231E+05, 1.139634E+05, 1.148069E+05, 1.156535E+05, 1.165033E+05, 
   1.173562E+05, 1.182123E+05, 1.190716E+05, 1.199340E+05, 1.207996E+05, 1.216683E+05, 
   1.225401E+05, 1.234152E+05, 1.242934E+05, 1.251747E+05, 1.260592E+05, 1.269469E+05, 
   1.278378E+05, 1.287318E+05, 1.296289E+05, 1.305292E+05, 1.314327E+05, 1.323394E+05, 
   1.332492E+05, 1.341621E+05, 1.350783E+05, 1.359976E+05, 1.369201E+05, 1.378457E+05, 
   1.387745E+05, 1.397065E+05, 1.406416E+05, 1.415800E+05, 1.425214E+05, 1.434661E+05, 
   1.444139E+05, 1.453649E+05, 1.463191E+05, 1.472764E+05, 1.482369E+05, 1.492006E+05, 
   1.501675E+05, 1.511375E+05, 1.521107E+05, 1.530871E+05, 1.540667E+05, 1.550494E+05, 
   1.560353E+05, 1.570244E+05, 1.580167E+05, 1.590121E+05, 1.600108E+05, 1.610126E+05, 
   1.620176E+05, 1.630257E+05, 1.640371E+05, 1.650516E+05, 1.660693E+05, 1.670902E+05, 
   1.681143E+05, 1.691416E+05, 1.701720E+05, 1.712057E+05, 1.722425E+05, 1.732825E+05, 
   1.743257E+05, 1.753721E+05, 1.764217E+05, 1.774744E+05, 1.785304E+05, 1.795895E+05, 
   1.806519E+05, 1.817174E+05, 1.827861E+05, 1.838580E+05, 1.849331E+05, 1.860114E+05, 
   1.870929E+05, 1.881775E+05, 1.892654E+05, 1.903565E+05, 1.914507E+05, 1.925482E+05, 
   1.936489E+05, 1.947527E+05, 1.958598E+05, 1.969700E+05, 1.980835E+05, 1.992001E+05, 
   2.003200E+05, 2.014430E+05, 2.025693E+05, 2.036987E+05, 2.048314E+05, 2.059672E+05, 
   2.071063E+05, 2.082485E+05, 2.093940E+05, 2.105427E+05, 2.116946E+05, 2.128497E+05, 
   2.140080E+05, 2.151695E+05, 2.163342E+05, 2.175021E+05, 2.186732E+05, 2.198475E+05, 
   2.210251E+05, 2.222058E+05, 2.233898E+05, 2.245770E+05, 2.257674E+05, 2.269610E+05, 
   2.281578E+05, 2.293578E+05, 2.305611E+05, 2.317675E+05, 2.329772E+05, 2.341901E+05, 
   2.354062E+05, 2.366255E+05, 2.378480E+05, 2.390738E+05, 2.403028E+05, 2.415350E+05, 
   2.427704E+05, 2.440090E+05, 2.452509E+05, 2.464960E+05, 2.477443E+05, 2.489958E+05, 
   2.502505E+05, 2.515085E+05, 2.527697E+05, 2.540341E+05, 2.553018E+05, 2.565726E+05, 
   2.578467E+05, 2.591240E+05, 2.604046E+05, 2.616884E+05, 2.629754E+05, 2.642656E+05, 
   2.655591E+05, 2.668558E+05, 2.681557E+05, 2.694588E+05, 2.707652E+05, 2.720748E+05, 
   2.733877E+05, 2.747038E+05, 2.760231E+05, 2.773457E+05, 2.786714E+05, 2.800005E+05, 
   2.813327E+05, 2.826682E+05, 2.840069E+05, 2.853489E+05, 2.866941E+05, 2.880425E+05, 
   2.893942E+05, 2.907492E+05, 2.921073E+05, 2.934687E+05, 2.948333E+05, 2.962012E+05, 
   2.975723E+05, 2.989467E+05, 3.003243E+05, 3.017052E+05, 3.030893E+05, 3.044766E+05, 
   3.058672E+05, 3.072610E+05, 3.086581E+05, 3.100584E+05, 3.114620E+05, 3.128688E+05, 
   3.142789E+05, 3.156922E+05, 3.171087E+05, 3.185285E+05, 3.199516E+05, 3.213779E+05, 
   3.228075E+05, 3.242403E+05, 3.256764E+05, 3.271157E+05, 3.285582E+05, 3.300041E+05, 
   3.314531E+05, 3.329055E+05, 3.343610E+05, 3.358199E+05, 3.372820E+05, 3.387473E+05, 
   3.402159E+05, 3.416878E+05, 3.431629E+05, 3.446413E+05, 3.461229E+05, 3.476078E+05, 
   3.490960E+05, 3.505874E+05, 3.520820E+05, 3.535800E+05, 3.550812E+05, 3.565856E+05, 
   3.580933E+05, 3.596043E+05, 3.611186E+05, 3.626361E+05, 3.641568E+05, 3.656809E+05, 
   3.672082E+05, 3.687387E+05, 3.702726E+05, 3.718097E+05, 3.733500E+05, 3.748936E+05, 
   3.764405E+05, 3.779907E+05, 3.795441E+05, 3.811008E+05, 3.826608E+05, 3.842240E+05, 
   3.857905E+05, 3.873603E+05, 3.889333E+05, 3.905096E+05, 3.920892E+05, 3.936721E+05, 
   3.952582E+05, 3.968476E+05, 3.984403E+05, 4.000362E+05, 4.016354E+05, 4.032379E+05, 
   4.048437E+05, 4.064527E+05, 4.080650E+05, 4.096806E+05, 4.112995E+05, 4.129216E+05, 
   4.145470E+05, 4.161757E+05, 4.178077E+05, 
])
# ============================ M = 19, I = 1 ============================
M = 19
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.780234E+00, 3.460154E+01, 6.887468E+01, 1.031563E+02, 1.374460E+02, 1.717437E+02, 
   2.060506E+02, 2.403733E+02, 2.747311E+02, 3.091638E+02, 3.437361E+02, 3.785364E+02, 
   4.136732E+02, 4.492686E+02, 4.854534E+02, 5.223618E+02, 5.601283E+02, 5.988851E+02, 
   6.387607E+02, 6.798792E+02, 7.223604E+02, 7.663192E+02, 8.118666E+02, 8.591097E+02, 
   9.081526E+02, 9.590961E+02, 1.012039E+03, 1.067079E+03, 1.124310E+03, 1.183828E+03, 
   1.245726E+03, 1.310098E+03, 1.377035E+03, 1.446633E+03, 1.518984E+03, 1.594183E+03, 
   1.672325E+03, 1.753505E+03, 1.837821E+03, 1.925370E+03, 2.016253E+03, 2.110570E+03, 
   2.208422E+03, 2.309915E+03, 2.415154E+03, 2.524246E+03, 2.637300E+03, 2.754428E+03, 
   2.875742E+03, 3.001357E+03, 3.131390E+03, 3.265961E+03, 3.405190E+03, 3.549200E+03, 
   3.698117E+03, 3.852069E+03, 4.011185E+03, 4.175599E+03, 4.345443E+03, 4.520855E+03, 
   4.701974E+03, 4.888942E+03, 5.081903E+03, 5.281002E+03, 5.486390E+03, 5.698217E+03, 
   5.916637E+03, 6.141807E+03, 6.373885E+03, 6.613034E+03, 6.859417E+03, 7.113201E+03, 
   7.374555E+03, 7.643653E+03, 7.920667E+03, 8.205777E+03, 8.499162E+03, 8.801004E+03, 
   9.111491E+03, 9.430811E+03, 9.759155E+03, 1.009672E+04, 1.044369E+04, 1.080029E+04, 
   1.116670E+04, 1.154313E+04, 1.192980E+04, 1.232691E+04, 1.273468E+04, 1.315333E+04, 
   1.358308E+04, 1.402415E+04, 1.447676E+04, 1.494116E+04, 1.541757E+04, 1.590623E+04, 
   1.640737E+04, 1.692124E+04, 1.744810E+04, 1.798817E+04, 1.854172E+04, 1.910901E+04, 
   1.969029E+04, 2.028583E+04, 2.089588E+04, 2.152072E+04, 2.216063E+04, 2.281587E+04, 
   2.348673E+04, 2.417349E+04, 2.487643E+04, 2.559585E+04, 2.633203E+04, 2.708528E+04, 
   2.785590E+04, 2.864418E+04, 2.945043E+04, 3.027498E+04, 3.111812E+04, 3.198019E+04, 
   3.286150E+04, 3.376237E+04, 3.468314E+04, 3.562414E+04, 3.658571E+04, 3.756818E+04, 
   3.857191E+04, 3.959724E+04, 4.064453E+04, 4.171412E+04, 4.280638E+04, 4.392167E+04, 
   4.506037E+04, 4.622284E+04, 4.740946E+04, 4.862061E+04, 4.985668E+04, 5.111805E+04, 
   5.240511E+04, 5.371826E+04, 5.505791E+04, 5.642445E+04, 5.781829E+04, 5.923986E+04, 
   6.068956E+04, 6.216782E+04, 6.367505E+04, 6.521171E+04, 6.677821E+04, 6.837499E+04, 
   7.000251E+04, 7.166120E+04, 7.335153E+04, 7.507393E+04, 7.682889E+04, 7.861685E+04, 
   8.043830E+04, 8.229370E+04, 8.418354E+04, 8.610830E+04, 8.806846E+04, 9.006452E+04, 
   9.209698E+04, 9.416634E+04, 9.627310E+04, 9.841778E+04, 1.006009E+05, 1.028230E+05, 
   1.050845E+05, 1.073861E+05, 1.097281E+05, 1.121113E+05, 1.145361E+05, 1.170031E+05, 
   1.195129E+05, 1.220659E+05, 1.246628E+05, 1.273041E+05, 1.299904E+05, 1.327223E+05, 
   1.355004E+05, 1.383252E+05, 1.411974E+05, 1.441175E+05, 1.470862E+05, 1.501040E+05, 
   1.531717E+05, 1.562897E+05, 1.594588E+05, 1.626795E+05, 1.659525E+05, 1.692785E+05, 
   1.726581E+05, 1.760919E+05, 1.795806E+05, 1.831248E+05, 1.867253E+05, 1.903827E+05, 
   1.940977E+05, 1.978710E+05, 2.017032E+05, 2.055950E+05, 2.095473E+05, 2.135605E+05, 
   2.176356E+05, 2.217732E+05, 2.259740E+05, 2.302387E+05, 2.345681E+05, 2.389630E+05, 
   2.434240E+05, 2.479520E+05, 2.525476E+05, 2.572117E+05, 2.619450E+05, 2.667483E+05, 
   2.716224E+05, 2.765681E+05, 2.815861E+05, 2.866773E+05, 2.918425E+05, 2.970824E+05, 
   3.023980E+05, 3.077900E+05, 3.132592E+05, 3.188065E+05, 3.244327E+05, 3.301387E+05, 
   3.359254E+05, 3.417935E+05, 3.477441E+05, 3.537778E+05, 3.598957E+05, 3.660986E+05, 
   3.723873E+05, 3.787629E+05, 3.852262E+05, 3.917780E+05, 3.984194E+05, 4.051513E+05, 
   4.119745E+05, 4.188901E+05, 4.258989E+05, 4.330019E+05, 4.402001E+05, 4.474945E+05, 
   4.548859E+05, 4.623755E+05, 4.699640E+05, 4.776527E+05, 4.854424E+05, 4.933342E+05, 
   5.013290E+05, 5.094279E+05, 5.176319E+05, 5.259420E+05, 5.343594E+05, 5.428849E+05, 
   5.515197E+05, 5.602649E+05, 5.691214E+05, 5.780904E+05, 5.871730E+05, 5.963701E+05, 
   6.056830E+05, 6.151128E+05, 6.246604E+05, 6.343271E+05, 6.441140E+05, 6.540222E+05, 
   6.640528E+05, 6.742070E+05, 6.844859E+05, 6.948907E+05, 7.054225E+05, 7.160826E+05, 
   7.268721E+05, 7.377921E+05, 7.488439E+05, 7.600288E+05, 7.713478E+05, 7.828022E+05, 
   7.943933E+05, 8.061222E+05, 8.179902E+05, 8.299986E+05, 8.421486E+05, 8.544415E+05, 
   8.668785E+05, 8.794609E+05, 8.921900E+05, 9.050671E+05, 9.180935E+05, 9.312705E+05, 
   9.445994E+05, 9.580816E+05, 9.717183E+05, 9.855110E+05, 9.994609E+05, 1.013569E+06, 
   1.027838E+06, 1.042268E+06, 1.056860E+06, 1.071617E+06, 1.086539E+06, 1.101628E+06, 
   1.116886E+06, 1.132313E+06, 1.147911E+06, 1.163682E+06, 1.179627E+06, 1.195748E+06, 
   1.212045E+06, 1.228521E+06, 1.245177E+06, 1.262014E+06, 1.279034E+06, 1.296239E+06, 
   1.313629E+06, 1.331207E+06, 1.348974E+06, 1.366931E+06, 1.385080E+06, 1.403423E+06, 
   1.421961E+06, 1.440696E+06, 1.459629E+06, 1.478762E+06, 1.498096E+06, 1.517633E+06, 
   1.537376E+06, 1.557324E+06, 1.577480E+06, 1.597846E+06, 1.618423E+06, 1.639213E+06, 
   1.660218E+06, 1.681438E+06, 1.702877E+06, 1.724535E+06, 1.746415E+06, 1.768517E+06, 
   1.790845E+06, 1.813398E+06, 1.836180E+06, 1.859192E+06, 1.882436E+06, 1.905912E+06, 
   1.929625E+06, 1.953574E+06, 1.977762E+06, 2.002190E+06, 2.026862E+06, 2.051777E+06, 
   2.076938E+06, 2.102347E+06, 2.128006E+06, 2.153917E+06, 2.180081E+06, 2.206500E+06, 
   2.233176E+06, 2.260112E+06, 2.287309E+06, 2.314769E+06, 2.342493E+06, 2.370484E+06, 
   2.398744E+06, 2.427275E+06, 2.456078E+06, 2.485156E+06, 2.514511E+06, 2.544144E+06, 
   2.574058E+06, 2.604254E+06, 2.634736E+06, 2.665503E+06, 2.696560E+06, 2.727906E+06, 
   2.759546E+06, 2.791481E+06, 2.823713E+06, 2.856244E+06, 2.889076E+06, 2.922211E+06, 
   2.955651E+06, 2.989399E+06, 3.023457E+06, 3.057826E+06, 3.092509E+06, 3.127509E+06, 
   3.162827E+06, 3.198465E+06, 3.234426E+06, 3.270712E+06, 3.307324E+06, 3.344267E+06, 
   3.381541E+06, 3.419148E+06, 3.457092E+06, 3.495374E+06, 3.533997E+06, 3.572963E+06, 
   3.612274E+06, 3.651933E+06, 3.691941E+06, 3.732302E+06, 3.773018E+06, 3.814090E+06, 
   3.855522E+06, 3.897315E+06, 3.939472E+06, 3.981996E+06, 4.024889E+06, 4.068153E+06, 
   4.111791E+06, 4.155805E+06, 4.200198E+06, 4.244972E+06, 4.290130E+06, 4.335673E+06, 
   4.381605E+06, 4.427929E+06, 4.474646E+06, 4.521758E+06, 4.569270E+06, 4.617183E+06, 
   4.665500E+06, 4.714223E+06, 4.763355E+06, 4.812899E+06, 4.862857E+06, 4.913232E+06, 
   4.964026E+06, 5.015242E+06, 5.066883E+06, 5.118952E+06, 5.171450E+06, 5.224381E+06, 
   5.277748E+06, 5.331553E+06, 5.385799E+06, 5.440488E+06, 5.495624E+06, 5.551210E+06, 
   5.607247E+06, 5.663738E+06, 5.720688E+06, 5.778097E+06, 5.835970E+06, 5.894309E+06, 
   5.953116E+06, 6.012396E+06, 6.072149E+06, 6.132381E+06, 6.193092E+06, 6.254287E+06, 
   6.315968E+06, 6.378137E+06, 6.440799E+06, 6.503956E+06, 6.567610E+06, 6.631766E+06, 
   6.696425E+06, 6.761591E+06, 6.827267E+06, 6.893456E+06, 6.960161E+06, 7.027385E+06, 
   7.095131E+06, 7.163402E+06, 7.232201E+06, 7.301531E+06, 7.371396E+06, 7.441798E+06, 
   7.512741E+06, 7.584227E+06, 7.656261E+06, 7.728845E+06, 7.801982E+06, 7.875675E+06, 
   7.949928E+06, 8.024744E+06, 8.100126E+06, 8.176078E+06, 8.252602E+06, 8.329703E+06, 
   8.407382E+06, 8.485644E+06, 8.564492E+06, 8.643929E+06, 8.723958E+06, 8.804584E+06, 
   8.885808E+06, 8.967635E+06, 9.050068E+06, 9.133110E+06, 9.216766E+06, 9.301037E+06, 
   9.385928E+06, 9.471442E+06, 9.557583E+06, 
])
# ============================ M = 19, I = 2 ============================
M = 19
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.865556E+00, 3.546016E+01, 7.059221E+01, 1.057330E+02, 1.408820E+02, 1.760392E+02, 
   2.112060E+02, 2.463891E+02, 2.816084E+02, 3.169054E+02, 3.523467E+02, 3.880240E+02, 
   4.240491E+02, 4.605482E+02, 4.976565E+02, 5.355128E+02, 5.742562E+02, 6.140234E+02, 
   6.549474E+02, 6.971567E+02, 7.407752E+02, 7.859220E+02, 8.327119E+02, 8.812560E+02, 
   9.316618E+02, 9.840340E+02, 1.038475E+03, 1.095085E+03, 1.153964E+03, 1.215208E+03, 
   1.278915E+03, 1.345181E+03, 1.414104E+03, 1.485780E+03, 1.560305E+03, 1.637778E+03, 
   1.718297E+03, 1.801961E+03, 1.888871E+03, 1.979129E+03, 2.072837E+03, 2.170100E+03, 
   2.271025E+03, 2.375719E+03, 2.484291E+03, 2.596854E+03, 2.713520E+03, 2.834404E+03, 
   2.959624E+03, 3.089299E+03, 3.223550E+03, 3.362501E+03, 3.506277E+03, 3.655007E+03, 
   3.808820E+03, 3.967848E+03, 4.132228E+03, 4.302096E+03, 4.477591E+03, 4.658856E+03, 
   4.846035E+03, 5.039275E+03, 5.238725E+03, 5.444538E+03, 5.656868E+03, 5.875872E+03, 
   6.101710E+03, 6.334545E+03, 6.574542E+03, 6.821867E+03, 7.076693E+03, 7.339191E+03, 
   7.609539E+03, 7.887914E+03, 8.174499E+03, 8.469477E+03, 8.773036E+03, 9.085367E+03, 
   9.406661E+03, 9.737115E+03, 1.007693E+04, 1.042630E+04, 1.078544E+04, 1.115455E+04, 
   1.153385E+04, 1.192354E+04, 1.232385E+04, 1.273499E+04, 1.315719E+04, 1.359067E+04, 
   1.403567E+04, 1.449242E+04, 1.496114E+04, 1.544209E+04, 1.593550E+04, 1.644162E+04, 
   1.696069E+04, 1.749298E+04, 1.803873E+04, 1.859821E+04, 1.917167E+04, 1.975938E+04, 
   2.036162E+04, 2.097865E+04, 2.161075E+04, 2.225819E+04, 2.292128E+04, 2.360028E+04, 
   2.429549E+04, 2.500720E+04, 2.573571E+04, 2.648132E+04, 2.724434E+04, 2.802508E+04, 
   2.882384E+04, 2.964094E+04, 3.047671E+04, 3.133145E+04, 3.220552E+04, 3.309922E+04, 
   3.401290E+04, 3.494690E+04, 3.590156E+04, 3.687722E+04, 3.787424E+04, 3.889297E+04, 
   3.993376E+04, 4.099699E+04, 4.208302E+04, 4.319221E+04, 4.432494E+04, 4.548160E+04, 
   4.666255E+04, 4.786820E+04, 4.909893E+04, 5.035513E+04, 5.163721E+04, 5.294557E+04, 
   5.428062E+04, 5.564277E+04, 5.703243E+04, 5.845003E+04, 5.989599E+04, 6.137074E+04, 
   6.287471E+04, 6.440836E+04, 6.597210E+04, 6.756641E+04, 6.919171E+04, 7.084848E+04, 
   7.253718E+04, 7.425826E+04, 7.601220E+04, 7.779947E+04, 7.962056E+04, 8.147594E+04, 
   8.336611E+04, 8.529156E+04, 8.725278E+04, 8.925029E+04, 9.128458E+04, 9.335617E+04, 
   9.546557E+04, 9.761332E+04, 9.979993E+04, 1.020259E+05, 1.042919E+05, 1.065983E+05, 
   1.089458E+05, 1.113348E+05, 1.137659E+05, 1.162398E+05, 1.187569E+05, 1.213178E+05, 
   1.239232E+05, 1.265735E+05, 1.292694E+05, 1.320115E+05, 1.348004E+05, 1.376366E+05, 
   1.405209E+05, 1.434537E+05, 1.464357E+05, 1.494676E+05, 1.525499E+05, 1.556834E+05, 
   1.588685E+05, 1.621061E+05, 1.653967E+05, 1.687410E+05, 1.721397E+05, 1.755934E+05, 
   1.791028E+05, 1.826685E+05, 1.862914E+05, 1.899720E+05, 1.937110E+05, 1.975092E+05, 
   2.013672E+05, 2.052859E+05, 2.092658E+05, 2.133077E+05, 2.174124E+05, 2.215805E+05, 
   2.258129E+05, 2.301102E+05, 2.344733E+05, 2.389028E+05, 2.433996E+05, 2.479644E+05, 
   2.525980E+05, 2.573012E+05, 2.620748E+05, 2.669195E+05, 2.718362E+05, 2.768257E+05, 
   2.818888E+05, 2.870262E+05, 2.922389E+05, 2.975277E+05, 3.028934E+05, 3.083368E+05, 
   3.138588E+05, 3.194603E+05, 3.251422E+05, 3.309052E+05, 3.367503E+05, 3.426783E+05, 
   3.486902E+05, 3.547868E+05, 3.609691E+05, 3.672379E+05, 3.735942E+05, 3.800389E+05, 
   3.865728E+05, 3.931971E+05, 3.999125E+05, 4.067201E+05, 4.136208E+05, 4.206155E+05, 
   4.277053E+05, 4.348910E+05, 4.421738E+05, 4.495545E+05, 4.570342E+05, 4.646139E+05, 
   4.722946E+05, 4.800772E+05, 4.879629E+05, 4.959527E+05, 5.040475E+05, 5.122484E+05, 
   5.205566E+05, 5.289730E+05, 5.374987E+05, 5.461348E+05, 5.548823E+05, 5.637424E+05, 
   5.727162E+05, 5.818047E+05, 5.910090E+05, 6.003303E+05, 6.097698E+05, 6.193284E+05, 
   6.290074E+05, 6.388079E+05, 6.487310E+05, 6.587780E+05, 6.689500E+05, 6.792481E+05, 
   6.896736E+05, 7.002276E+05, 7.109113E+05, 7.217260E+05, 7.326728E+05, 7.437531E+05, 
   7.549678E+05, 7.663185E+05, 7.778062E+05, 7.894322E+05, 8.011978E+05, 8.131043E+05, 
   8.251529E+05, 8.373449E+05, 8.496816E+05, 8.621642E+05, 8.747942E+05, 8.875729E+05, 
   9.005014E+05, 9.135812E+05, 9.268136E+05, 9.402000E+05, 9.537417E+05, 9.674400E+05, 
   9.812964E+05, 9.953122E+05, 1.009489E+06, 1.023828E+06, 1.038330E+06, 1.052997E+06, 
   1.067831E+06, 1.082833E+06, 1.098004E+06, 1.113345E+06, 1.128859E+06, 1.144547E+06, 
   1.160409E+06, 1.176448E+06, 1.192666E+06, 1.209063E+06, 1.225641E+06, 1.242401E+06, 
   1.259346E+06, 1.276476E+06, 1.293793E+06, 1.311300E+06, 1.328996E+06, 1.346884E+06, 
   1.364966E+06, 1.383243E+06, 1.401716E+06, 1.420387E+06, 1.439258E+06, 1.458331E+06, 
   1.477607E+06, 1.497087E+06, 1.516774E+06, 1.536668E+06, 1.556772E+06, 1.577088E+06, 
   1.597616E+06, 1.618359E+06, 1.639319E+06, 1.660496E+06, 1.681894E+06, 1.703512E+06, 
   1.725354E+06, 1.747421E+06, 1.769715E+06, 1.792237E+06, 1.814989E+06, 1.837974E+06, 
   1.861192E+06, 1.884646E+06, 1.908337E+06, 1.932268E+06, 1.956440E+06, 1.980854E+06, 
   2.005514E+06, 2.030420E+06, 2.055574E+06, 2.080979E+06, 2.106636E+06, 2.132547E+06, 
   2.158714E+06, 2.185140E+06, 2.211825E+06, 2.238772E+06, 2.265982E+06, 2.293458E+06, 
   2.321202E+06, 2.349216E+06, 2.377501E+06, 2.406060E+06, 2.434895E+06, 2.464007E+06, 
   2.493399E+06, 2.523072E+06, 2.553029E+06, 2.583272E+06, 2.613803E+06, 2.644624E+06, 
   2.675737E+06, 2.707143E+06, 2.738846E+06, 2.770848E+06, 2.803150E+06, 2.835754E+06, 
   2.868663E+06, 2.901879E+06, 2.935404E+06, 2.969240E+06, 3.003389E+06, 3.037854E+06, 
   3.072637E+06, 3.107740E+06, 3.143165E+06, 3.178915E+06, 3.214991E+06, 3.251396E+06, 
   3.288133E+06, 3.325203E+06, 3.362609E+06, 3.400353E+06, 3.438437E+06, 3.476865E+06, 
   3.515637E+06, 3.554757E+06, 3.594226E+06, 3.634048E+06, 3.674224E+06, 3.714758E+06, 
   3.755650E+06, 3.796905E+06, 3.838523E+06, 3.880508E+06, 3.922862E+06, 3.965588E+06, 
   4.008688E+06, 4.052164E+06, 4.096019E+06, 4.140255E+06, 4.184876E+06, 4.229883E+06, 
   4.275279E+06, 4.321066E+06, 4.367248E+06, 4.413826E+06, 4.460803E+06, 4.508183E+06, 
   4.555966E+06, 4.604157E+06, 4.652758E+06, 4.701771E+06, 4.751198E+06, 4.801044E+06, 
   4.851310E+06, 4.901998E+06, 4.953113E+06, 5.004656E+06, 5.056630E+06, 5.109038E+06, 
   5.161882E+06, 5.215166E+06, 5.268892E+06, 5.323062E+06, 5.377681E+06, 5.432749E+06, 
   5.488272E+06, 5.544250E+06, 5.600687E+06, 5.657586E+06, 5.714950E+06, 5.772781E+06, 
   5.831083E+06, 5.889858E+06, 5.949109E+06, 6.008839E+06, 6.069052E+06, 6.129749E+06, 
   6.190934E+06, 6.252611E+06, 6.314781E+06, 6.377449E+06, 6.440616E+06, 6.504286E+06, 
   6.568462E+06, 6.633148E+06, 6.698345E+06, 6.764058E+06, 6.830289E+06, 6.897041E+06, 
   6.964318E+06, 7.032123E+06, 7.100458E+06, 7.169327E+06, 7.238733E+06, 7.308679E+06, 
   7.379169E+06, 7.450205E+06, 7.521791E+06, 7.593931E+06, 7.666626E+06, 7.739881E+06, 
   7.813699E+06, 7.888083E+06, 7.963036E+06, 8.038562E+06, 8.114664E+06, 8.191345E+06, 
   8.268609E+06, 8.346459E+06, 8.424898E+06, 8.503930E+06, 8.583558E+06, 8.663786E+06, 
   8.744616E+06, 8.826053E+06, 8.908100E+06, 8.990761E+06, 9.074038E+06, 9.157935E+06, 
   9.242456E+06, 9.327605E+06, 9.413384E+06, 9.499798E+06, 9.586849E+06, 9.674542E+06, 
   9.762881E+06, 9.851868E+06, 9.941507E+06, 
])
# ============================ M = 19, I = 3 ============================
M = 19
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.582456E+00, 6.942440E+01, 1.381920E+02, 2.069768E+02, 2.757776E+02, 3.445947E+02, 
   4.134315E+02, 4.823053E+02, 5.512650E+02, 6.204071E+02, 6.898825E+02, 7.598924E+02, 
   8.306769E+02, 9.025019E+02, 9.756470E+02, 1.050396E+03, 1.127030E+03, 1.205823E+03, 
   1.287040E+03, 1.370937E+03, 1.457758E+03, 1.547738E+03, 1.641106E+03, 1.738080E+03, 
   1.838874E+03, 1.943696E+03, 2.052750E+03, 2.166235E+03, 2.284351E+03, 2.407293E+03, 
   2.535256E+03, 2.668435E+03, 2.807027E+03, 2.951226E+03, 3.101229E+03, 3.257236E+03, 
   3.419447E+03, 3.588065E+03, 3.763296E+03, 3.945347E+03, 4.134431E+03, 4.330761E+03, 
   4.534556E+03, 4.746037E+03, 4.965429E+03, 5.192962E+03, 5.428870E+03, 5.673389E+03, 
   5.926761E+03, 6.189232E+03, 6.461053E+03, 6.742479E+03, 7.033770E+03, 7.335189E+03, 
   7.647006E+03, 7.969494E+03, 8.302934E+03, 8.647608E+03, 9.003806E+03, 9.371821E+03, 
   9.751952E+03, 1.014450E+04, 1.054979E+04, 1.096812E+04, 1.139981E+04, 1.184519E+04, 
   1.230460E+04, 1.277837E+04, 1.326684E+04, 1.377036E+04, 1.428928E+04, 1.482396E+04, 
   1.537477E+04, 1.594208E+04, 1.652626E+04, 1.712770E+04, 1.774678E+04, 1.838391E+04, 
   1.903947E+04, 1.971388E+04, 2.040756E+04, 2.112091E+04, 2.185437E+04, 2.260836E+04, 
   2.338333E+04, 2.417971E+04, 2.499797E+04, 2.583855E+04, 2.670192E+04, 2.758854E+04, 
   2.849891E+04, 2.943350E+04, 3.039279E+04, 3.137729E+04, 3.238751E+04, 3.342394E+04, 
   3.448712E+04, 3.557755E+04, 3.669579E+04, 3.784235E+04, 3.901779E+04, 4.022267E+04, 
   4.145754E+04, 4.272296E+04, 4.401952E+04, 4.534779E+04, 4.670837E+04, 4.810185E+04, 
   4.952883E+04, 5.098993E+04, 5.248576E+04, 5.401695E+04, 5.558414E+04, 5.718796E+04, 
   5.882908E+04, 6.050814E+04, 6.222581E+04, 6.398276E+04, 6.577969E+04, 6.761726E+04, 
   6.949619E+04, 7.141718E+04, 7.338093E+04, 7.538819E+04, 7.743966E+04, 7.953610E+04, 
   8.167825E+04, 8.386686E+04, 8.610270E+04, 8.838654E+04, 9.071916E+04, 9.310135E+04, 
   9.553391E+04, 9.801764E+04, 1.005534E+05, 1.031419E+05, 1.057841E+05, 1.084808E+05, 
   1.112328E+05, 1.140410E+05, 1.169063E+05, 1.198295E+05, 1.228116E+05, 1.258534E+05, 
   1.289558E+05, 1.321198E+05, 1.353462E+05, 1.386361E+05, 1.419903E+05, 1.454098E+05, 
   1.488955E+05, 1.524486E+05, 1.560698E+05, 1.597602E+05, 1.635209E+05, 1.673527E+05, 
   1.712569E+05, 1.752342E+05, 1.792859E+05, 1.834130E+05, 1.876164E+05, 1.918974E+05, 
   1.962569E+05, 2.006961E+05, 2.052160E+05, 2.098179E+05, 2.145027E+05, 2.192716E+05, 
   2.241258E+05, 2.290665E+05, 2.340947E+05, 2.392116E+05, 2.444185E+05, 2.497166E+05, 
   2.551070E+05, 2.605909E+05, 2.661697E+05, 2.718444E+05, 2.776165E+05, 2.834870E+05, 
   2.894574E+05, 2.955289E+05, 3.017027E+05, 3.079803E+05, 3.143628E+05, 3.208517E+05, 
   3.274482E+05, 3.341538E+05, 3.409697E+05, 3.478974E+05, 3.549382E+05, 3.620936E+05, 
   3.693649E+05, 3.767536E+05, 3.842610E+05, 3.918887E+05, 3.996381E+05, 4.075107E+05, 
   4.155079E+05, 4.236312E+05, 4.318821E+05, 4.402622E+05, 4.487730E+05, 4.574160E+05, 
   4.661927E+05, 4.751048E+05, 4.841538E+05, 4.933413E+05, 5.026688E+05, 5.121381E+05, 
   5.217507E+05, 5.315083E+05, 5.414126E+05, 5.514651E+05, 5.616676E+05, 5.720217E+05, 
   5.825292E+05, 5.931918E+05, 6.040112E+05, 6.149891E+05, 6.261274E+05, 6.374277E+05, 
   6.488919E+05, 6.605217E+05, 6.723190E+05, 6.842855E+05, 6.964232E+05, 7.087338E+05, 
   7.212193E+05, 7.338815E+05, 7.467222E+05, 7.597435E+05, 7.729471E+05, 7.863351E+05, 
   7.999094E+05, 8.136719E+05, 8.276247E+05, 8.417696E+05, 8.561087E+05, 8.706441E+05, 
   8.853777E+05, 9.003115E+05, 9.154477E+05, 9.307883E+05, 9.463355E+05, 9.620912E+05, 
   9.780576E+05, 9.942369E+05, 1.010631E+06, 1.027242E+06, 1.044073E+06, 1.061125E+06, 
   1.078401E+06, 1.095903E+06, 1.113633E+06, 1.131594E+06, 1.149786E+06, 1.168214E+06, 
   1.186880E+06, 1.205784E+06, 1.224931E+06, 1.244322E+06, 1.263959E+06, 1.283846E+06, 
   1.303983E+06, 1.324374E+06, 1.345022E+06, 1.365928E+06, 1.387095E+06, 1.408525E+06, 
   1.430221E+06, 1.452186E+06, 1.474422E+06, 1.496931E+06, 1.519716E+06, 1.542780E+06, 
   1.566124E+06, 1.589753E+06, 1.613668E+06, 1.637872E+06, 1.662367E+06, 1.687157E+06, 
   1.712243E+06, 1.737629E+06, 1.763318E+06, 1.789311E+06, 1.815612E+06, 1.842224E+06, 
   1.869149E+06, 1.896390E+06, 1.923950E+06, 1.951832E+06, 1.980038E+06, 2.008572E+06, 
   2.037435E+06, 2.066632E+06, 2.096165E+06, 2.126037E+06, 2.156251E+06, 2.186810E+06, 
   2.217716E+06, 2.248974E+06, 2.280585E+06, 2.312553E+06, 2.344881E+06, 2.377572E+06, 
   2.410629E+06, 2.444055E+06, 2.477854E+06, 2.512028E+06, 2.546580E+06, 2.581514E+06, 
   2.616834E+06, 2.652541E+06, 2.688640E+06, 2.725133E+06, 2.762024E+06, 2.799316E+06, 
   2.837012E+06, 2.875117E+06, 2.913632E+06, 2.952562E+06, 2.991910E+06, 3.031678E+06, 
   3.071872E+06, 3.112493E+06, 3.153546E+06, 3.195034E+06, 3.236960E+06, 3.279328E+06, 
   3.322142E+06, 3.365404E+06, 3.409119E+06, 3.453290E+06, 3.497921E+06, 3.543016E+06, 
   3.588577E+06, 3.634609E+06, 3.681115E+06, 3.728099E+06, 3.775565E+06, 3.823516E+06, 
   3.871956E+06, 3.920890E+06, 3.970320E+06, 4.020251E+06, 4.070686E+06, 4.121630E+06, 
   4.173085E+06, 4.225057E+06, 4.277548E+06, 4.330564E+06, 4.384107E+06, 4.438182E+06, 
   4.492793E+06, 4.547944E+06, 4.603639E+06, 4.659881E+06, 4.716676E+06, 4.774026E+06, 
   4.831937E+06, 4.890412E+06, 4.949456E+06, 5.009072E+06, 5.069265E+06, 5.130039E+06, 
   5.191399E+06, 5.253348E+06, 5.315891E+06, 5.379032E+06, 5.442776E+06, 5.507127E+06, 
   5.572089E+06, 5.637666E+06, 5.703864E+06, 5.770686E+06, 5.838137E+06, 5.906222E+06, 
   5.974944E+06, 6.044309E+06, 6.114321E+06, 6.184985E+06, 6.256304E+06, 6.328285E+06, 
   6.400930E+06, 6.474246E+06, 6.548237E+06, 6.622907E+06, 6.698261E+06, 6.774304E+06, 
   6.851040E+06, 6.928475E+06, 7.006613E+06, 7.085460E+06, 7.165019E+06, 7.245296E+06, 
   7.326296E+06, 7.408023E+06, 7.490483E+06, 7.573681E+06, 7.657621E+06, 7.742308E+06, 
   7.827749E+06, 7.913946E+06, 8.000907E+06, 8.088636E+06, 8.177137E+06, 8.266417E+06, 
   8.356480E+06, 8.447331E+06, 8.538976E+06, 8.631420E+06, 8.724669E+06, 8.818727E+06, 
   8.913600E+06, 9.009293E+06, 9.105811E+06, 9.203161E+06, 9.301347E+06, 9.400375E+06, 
   9.500250E+06, 9.600978E+06, 9.702564E+06, 9.805014E+06, 9.908334E+06, 1.001253E+07, 
   1.011760E+07, 1.022356E+07, 1.033042E+07, 1.043817E+07, 1.054682E+07, 1.065639E+07, 
   1.076686E+07, 1.087826E+07, 1.099059E+07, 1.110384E+07, 1.121804E+07, 1.133318E+07, 
   1.144927E+07, 1.156631E+07, 1.168432E+07, 1.180329E+07, 1.192324E+07, 1.204417E+07, 
   1.216608E+07, 1.228899E+07, 1.241289E+07, 1.253780E+07, 1.266372E+07, 1.279065E+07, 
   1.291861E+07, 1.304760E+07, 1.317762E+07, 1.330868E+07, 1.344079E+07, 1.357396E+07, 
   1.370818E+07, 1.384348E+07, 1.397984E+07, 1.411729E+07, 1.425582E+07, 1.439545E+07, 
   1.453617E+07, 1.467800E+07, 1.482094E+07, 1.496501E+07, 1.511020E+07, 1.525652E+07, 
   1.540398E+07, 1.555258E+07, 1.570234E+07, 1.585326E+07, 1.600534E+07, 1.615860E+07, 
   1.631303E+07, 1.646866E+07, 1.662547E+07, 1.678349E+07, 1.694271E+07, 1.710315E+07, 
   1.726481E+07, 1.742770E+07, 1.759183E+07, 1.775720E+07, 1.792382E+07, 1.809169E+07, 
   1.826083E+07, 1.843124E+07, 1.860293E+07, 1.877591E+07, 1.895018E+07, 1.912574E+07, 
   1.930262E+07, 1.948081E+07, 1.966033E+07, 1.984118E+07, 2.002336E+07, 2.020689E+07, 
   2.039177E+07, 2.057801E+07, 2.076562E+07, 
])
# ============================ M = 19, I = 4 ============================
M = 19
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.529462E+01, 1.401541E+02, 2.789952E+02, 4.178707E+02, 5.567786E+02, 6.957191E+02, 
   8.346971E+02, 9.737391E+02, 1.112924E+03, 1.252414E+03, 1.392472E+03, 1.533458E+03, 
   1.675813E+03, 1.820034E+03, 1.966653E+03, 2.116216E+03, 2.269270E+03, 2.426353E+03, 
   2.587989E+03, 2.754683E+03, 2.926922E+03, 3.105175E+03, 3.289892E+03, 3.481511E+03, 
   3.680455E+03, 3.887136E+03, 4.101956E+03, 4.325309E+03, 4.557584E+03, 4.799167E+03, 
   5.050438E+03, 5.311777E+03, 5.583565E+03, 5.866181E+03, 6.160007E+03, 6.465426E+03, 
   6.782826E+03, 7.112598E+03, 7.455135E+03, 7.810838E+03, 8.180113E+03, 8.563370E+03, 
   8.961026E+03, 9.373505E+03, 9.801237E+03, 1.024466E+04, 1.070422E+04, 1.118036E+04, 
   1.167356E+04, 1.218427E+04, 1.271298E+04, 1.326016E+04, 1.382632E+04, 1.441195E+04, 
   1.501757E+04, 1.564369E+04, 1.629085E+04, 1.695959E+04, 1.765045E+04, 1.836399E+04, 
   1.910078E+04, 1.986140E+04, 2.064643E+04, 2.145647E+04, 2.229212E+04, 2.315401E+04, 
   2.404277E+04, 2.495902E+04, 2.590342E+04, 2.687663E+04, 2.787931E+04, 2.891215E+04, 
   2.997584E+04, 3.107108E+04, 3.219857E+04, 3.335906E+04, 3.455327E+04, 3.578194E+04, 
   3.704584E+04, 3.834574E+04, 3.968241E+04, 4.105665E+04, 4.246926E+04, 4.392105E+04, 
   4.541287E+04, 4.694553E+04, 4.851991E+04, 5.013685E+04, 5.179724E+04, 5.350197E+04, 
   5.525193E+04, 5.704803E+04, 5.889122E+04, 6.078241E+04, 6.272257E+04, 6.471266E+04, 
   6.675365E+04, 6.884653E+04, 7.099232E+04, 7.319201E+04, 7.544665E+04, 7.775728E+04, 
   8.012495E+04, 8.255073E+04, 8.503571E+04, 8.758098E+04, 9.018765E+04, 9.285686E+04, 
   9.558973E+04, 9.838742E+04, 1.012511E+05, 1.041820E+05, 1.071812E+05, 1.102500E+05, 
   1.133896E+05, 1.166012E+05, 1.198861E+05, 1.232456E+05, 1.266810E+05, 1.301934E+05, 
   1.337844E+05, 1.374551E+05, 1.412070E+05, 1.450414E+05, 1.489596E+05, 1.529631E+05, 
   1.570532E+05, 1.612315E+05, 1.654992E+05, 1.698580E+05, 1.743092E+05, 1.788543E+05, 
   1.834949E+05, 1.882324E+05, 1.930684E+05, 1.980044E+05, 2.030420E+05, 2.081829E+05, 
   2.134285E+05, 2.187805E+05, 2.242406E+05, 2.298104E+05, 2.354915E+05, 2.412857E+05, 
   2.471947E+05, 2.532201E+05, 2.593637E+05, 2.656273E+05, 2.720126E+05, 2.785215E+05, 
   2.851557E+05, 2.919170E+05, 2.988074E+05, 3.058287E+05, 3.129827E+05, 3.202713E+05, 
   3.276965E+05, 3.352603E+05, 3.429644E+05, 3.508111E+05, 3.588021E+05, 3.669396E+05, 
   3.752255E+05, 3.836620E+05, 3.922510E+05, 4.009947E+05, 4.098952E+05, 4.189546E+05, 
   4.281751E+05, 4.375587E+05, 4.471078E+05, 4.568244E+05, 4.667109E+05, 4.767694E+05, 
   4.870023E+05, 4.974118E+05, 5.080002E+05, 5.187699E+05, 5.297231E+05, 5.408624E+05, 
   5.521899E+05, 5.637083E+05, 5.754198E+05, 5.873269E+05, 5.994322E+05, 6.117380E+05, 
   6.242469E+05, 6.369615E+05, 6.498843E+05, 6.630178E+05, 6.763648E+05, 6.899277E+05, 
   7.037092E+05, 7.177121E+05, 7.319389E+05, 7.463924E+05, 7.610753E+05, 7.759904E+05, 
   7.911404E+05, 8.065282E+05, 8.221566E+05, 8.380283E+05, 8.541463E+05, 8.705135E+05, 
   8.871327E+05, 9.040069E+05, 9.211391E+05, 9.385322E+05, 9.561893E+05, 9.741133E+05, 
   9.923074E+05, 1.010774E+06, 1.029518E+06, 1.048540E+06, 1.067846E+06, 1.087436E+06, 
   1.107316E+06, 1.127487E+06, 1.147954E+06, 1.168719E+06, 1.189786E+06, 1.211159E+06, 
   1.232840E+06, 1.254832E+06, 1.277140E+06, 1.299767E+06, 1.322715E+06, 1.345989E+06, 
   1.369592E+06, 1.393528E+06, 1.417800E+06, 1.442412E+06, 1.467366E+06, 1.492668E+06, 
   1.518320E+06, 1.544326E+06, 1.570690E+06, 1.597416E+06, 1.624507E+06, 1.651968E+06, 
   1.679801E+06, 1.708011E+06, 1.736601E+06, 1.765576E+06, 1.794940E+06, 1.824695E+06, 
   1.854847E+06, 1.885399E+06, 1.916356E+06, 1.947720E+06, 1.979498E+06, 2.011691E+06, 
   2.044306E+06, 2.077345E+06, 2.110813E+06, 2.144714E+06, 2.179052E+06, 2.213832E+06, 
   2.249058E+06, 2.284735E+06, 2.320866E+06, 2.357456E+06, 2.394509E+06, 2.432030E+06, 
   2.470024E+06, 2.508494E+06, 2.547445E+06, 2.586882E+06, 2.626810E+06, 2.667233E+06, 
   2.708156E+06, 2.749582E+06, 2.791518E+06, 2.833968E+06, 2.876936E+06, 2.920427E+06, 
   2.964447E+06, 3.009000E+06, 3.054090E+06, 3.099723E+06, 3.145904E+06, 3.192637E+06, 
   3.239928E+06, 3.287782E+06, 3.336204E+06, 3.385198E+06, 3.434771E+06, 3.484926E+06, 
   3.535670E+06, 3.587007E+06, 3.638943E+06, 3.691483E+06, 3.744632E+06, 3.798396E+06, 
   3.852780E+06, 3.907790E+06, 3.963430E+06, 4.019707E+06, 4.076626E+06, 4.134192E+06, 
   4.192411E+06, 4.251288E+06, 4.310830E+06, 4.371042E+06, 4.431929E+06, 4.493497E+06, 
   4.555753E+06, 4.618701E+06, 4.682348E+06, 4.746699E+06, 4.811761E+06, 4.877539E+06, 
   4.944040E+06, 5.011269E+06, 5.079232E+06, 5.147935E+06, 5.217385E+06, 5.287588E+06, 
   5.358549E+06, 5.430276E+06, 5.502774E+06, 5.576049E+06, 5.650108E+06, 5.724957E+06, 
   5.800602E+06, 5.877051E+06, 5.954309E+06, 6.032382E+06, 6.111278E+06, 6.191003E+06, 
   6.271564E+06, 6.352966E+06, 6.435218E+06, 6.518324E+06, 6.602293E+06, 6.687131E+06, 
   6.772845E+06, 6.859441E+06, 6.946927E+06, 7.035309E+06, 7.124594E+06, 7.214790E+06, 
   7.305903E+06, 7.397940E+06, 7.490909E+06, 7.584816E+06, 7.679669E+06, 7.775475E+06, 
   7.872241E+06, 7.969975E+06, 8.068683E+06, 8.168374E+06, 8.269054E+06, 8.370731E+06, 
   8.473412E+06, 8.577105E+06, 8.681818E+06, 8.787558E+06, 8.894332E+06, 9.002149E+06, 
   9.111016E+06, 9.220941E+06, 9.331931E+06, 9.443994E+06, 9.557139E+06, 9.671373E+06, 
   9.786704E+06, 9.903139E+06, 1.002069E+07, 1.013936E+07, 1.025916E+07, 1.038009E+07, 
   1.050217E+07, 1.062541E+07, 1.074981E+07, 1.087537E+07, 1.100212E+07, 1.113005E+07, 
   1.125918E+07, 1.138951E+07, 1.152106E+07, 1.165382E+07, 1.178781E+07, 1.192305E+07, 
   1.205952E+07, 1.219726E+07, 1.233626E+07, 1.247653E+07, 1.261808E+07, 1.276092E+07, 
   1.290506E+07, 1.305051E+07, 1.319728E+07, 1.334537E+07, 1.349480E+07, 1.364558E+07, 
   1.379770E+07, 1.395119E+07, 1.410606E+07, 1.426230E+07, 1.441993E+07, 1.457897E+07, 
   1.473941E+07, 1.490128E+07, 1.506457E+07, 1.522930E+07, 1.539547E+07, 1.556311E+07, 
   1.573221E+07, 1.590279E+07, 1.607485E+07, 1.624841E+07, 1.642348E+07, 1.660006E+07, 
   1.677817E+07, 1.695781E+07, 1.713900E+07, 1.732174E+07, 1.750606E+07, 1.769194E+07, 
   1.787942E+07, 1.806849E+07, 1.825917E+07, 1.845146E+07, 1.864538E+07, 1.884094E+07, 
   1.903815E+07, 1.923702E+07, 1.943756E+07, 1.963978E+07, 1.984369E+07, 2.004930E+07, 
   2.025662E+07, 2.046567E+07, 2.067645E+07, 2.088898E+07, 2.110326E+07, 2.131930E+07, 
   2.153713E+07, 2.175674E+07, 2.197816E+07, 2.220139E+07, 2.242644E+07, 2.265332E+07, 
   2.288205E+07, 2.311263E+07, 2.334508E+07, 2.357941E+07, 2.381564E+07, 2.405376E+07, 
   2.429380E+07, 2.453576E+07, 2.477967E+07, 2.502552E+07, 2.527333E+07, 2.552311E+07, 
   2.577488E+07, 2.602865E+07, 2.628442E+07, 2.654222E+07, 2.680204E+07, 2.706392E+07, 
   2.732784E+07, 2.759384E+07, 2.786192E+07, 2.813210E+07, 2.840438E+07, 2.867878E+07, 
   2.895531E+07, 2.923398E+07, 2.951482E+07, 2.979782E+07, 3.008300E+07, 3.037037E+07, 
   3.065996E+07, 3.095176E+07, 3.124580E+07, 3.154208E+07, 3.184062E+07, 3.214143E+07, 
   3.244453E+07, 3.274993E+07, 3.305764E+07, 3.336767E+07, 3.368004E+07, 3.399476E+07, 
   3.431185E+07, 3.463132E+07, 3.495318E+07, 3.527744E+07, 3.560412E+07, 3.593324E+07, 
   3.626480E+07, 3.659882E+07, 3.693531E+07, 3.727430E+07, 3.761578E+07, 3.795978E+07, 
   3.830632E+07, 3.865539E+07, 3.900702E+07, 
])
# ============================ M = 19, I = 5 ============================
M = 19
I = 5
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.005081E+00, 3.686364E+01, 7.339955E+01, 1.099445E+02, 1.464979E+02, 1.830599E+02, 
   2.196320E+02, 2.562227E+02, 2.928554E+02, 3.295781E+02, 3.664665E+02, 4.036226E+02, 
   4.411696E+02, 4.792452E+02, 5.179950E+02, 5.575679E+02, 5.981119E+02, 6.397721E+02, 
   6.826893E+02, 7.269988E+02, 7.728312E+02, 8.203119E+02, 8.695619E+02, 9.206981E+02, 
   9.738340E+02, 1.029080E+03, 1.086544E+03, 1.146334E+03, 1.208552E+03, 1.273304E+03, 
   1.340693E+03, 1.410821E+03, 1.483792E+03, 1.559709E+03, 1.638675E+03, 1.720796E+03, 
   1.806176E+03, 1.894923E+03, 1.987145E+03, 2.082950E+03, 2.182450E+03, 2.285757E+03, 
   2.392986E+03, 2.504253E+03, 2.619676E+03, 2.739375E+03, 2.863472E+03, 2.992093E+03, 
   3.125362E+03, 3.263410E+03, 3.406368E+03, 3.554369E+03, 3.707548E+03, 3.866045E+03, 
   4.030001E+03, 4.199558E+03, 4.374862E+03, 4.556063E+03, 4.743312E+03, 4.936762E+03, 
   5.136569E+03, 5.342894E+03, 5.555898E+03, 5.775745E+03, 6.002604E+03, 6.236645E+03, 
   6.478041E+03, 6.726968E+03, 6.983605E+03, 7.248135E+03, 7.520742E+03, 7.801614E+03, 
   8.090941E+03, 8.388919E+03, 8.695744E+03, 9.011616E+03, 9.336738E+03, 9.671317E+03, 
   1.001556E+04, 1.036969E+04, 1.073391E+04, 1.110844E+04, 1.149351E+04, 1.188934E+04, 
   1.229616E+04, 1.271420E+04, 1.314371E+04, 1.358491E+04, 1.403805E+04, 1.450338E+04, 
   1.498114E+04, 1.547159E+04, 1.597499E+04, 1.649159E+04, 1.702165E+04, 1.756546E+04, 
   1.812326E+04, 1.869535E+04, 1.928199E+04, 1.988347E+04, 2.050008E+04, 2.113210E+04, 
   2.177982E+04, 2.244355E+04, 2.312357E+04, 2.382021E+04, 2.453375E+04, 2.526453E+04, 
   2.601284E+04, 2.677901E+04, 2.756337E+04, 2.836625E+04, 2.918796E+04, 3.002885E+04, 
   3.088927E+04, 3.176954E+04, 3.267003E+04, 3.359108E+04, 3.453304E+04, 3.549629E+04, 
   3.648117E+04, 3.748807E+04, 3.851735E+04, 3.956940E+04, 4.064458E+04, 4.174330E+04, 
   4.286594E+04, 4.401289E+04, 4.518455E+04, 4.638133E+04, 4.760363E+04, 4.885187E+04, 
   5.012646E+04, 5.142783E+04, 5.275639E+04, 5.411259E+04, 5.549686E+04, 5.690963E+04, 
   5.835135E+04, 5.982247E+04, 6.132344E+04, 6.285473E+04, 6.441679E+04, 6.601010E+04, 
   6.763512E+04, 6.929234E+04, 7.098224E+04, 7.270529E+04, 7.446201E+04, 7.625288E+04, 
   7.807841E+04, 7.993911E+04, 8.183548E+04, 8.376804E+04, 8.573732E+04, 8.774385E+04, 
   8.978816E+04, 9.187078E+04, 9.399226E+04, 9.615315E+04, 9.835399E+04, 1.005954E+05, 
   1.028778E+05, 1.052019E+05, 1.075682E+05, 1.099774E+05, 1.124299E+05, 1.149264E+05, 
   1.174676E+05, 1.200538E+05, 1.226859E+05, 1.253644E+05, 1.280899E+05, 1.308630E+05, 
   1.336845E+05, 1.365548E+05, 1.394747E+05, 1.424448E+05, 1.454657E+05, 1.485381E+05, 
   1.516628E+05, 1.548402E+05, 1.580712E+05, 1.613564E+05, 1.646964E+05, 1.680921E+05, 
   1.715440E+05, 1.750529E+05, 1.786195E+05, 1.822445E+05, 1.859286E+05, 1.896726E+05, 
   1.934772E+05, 1.973432E+05, 2.012712E+05, 2.052621E+05, 2.093165E+05, 2.134354E+05, 
   2.176193E+05, 2.218692E+05, 2.261858E+05, 2.305698E+05, 2.350222E+05, 2.395436E+05, 
   2.441350E+05, 2.487970E+05, 2.535307E+05, 2.583366E+05, 2.632158E+05, 2.681690E+05, 
   2.731972E+05, 2.783011E+05, 2.834815E+05, 2.887395E+05, 2.940759E+05, 2.994915E+05, 
   3.049872E+05, 3.105639E+05, 3.162226E+05, 3.219641E+05, 3.277893E+05, 3.336993E+05, 
   3.396948E+05, 3.457769E+05, 3.519465E+05, 3.582045E+05, 3.645519E+05, 3.709897E+05, 
   3.775188E+05, 3.841402E+05, 3.908550E+05, 3.976640E+05, 4.045683E+05, 4.115689E+05, 
   4.186668E+05, 4.258630E+05, 4.331586E+05, 4.405546E+05, 4.480521E+05, 4.556520E+05, 
   4.633555E+05, 4.711637E+05, 4.790775E+05, 4.870980E+05, 4.952265E+05, 5.034639E+05, 
   5.118114E+05, 5.202700E+05, 5.288409E+05, 5.375253E+05, 5.463242E+05, 5.552388E+05, 
   5.642702E+05, 5.734196E+05, 5.826882E+05, 5.920771E+05, 6.015876E+05, 6.112207E+05, 
   6.209778E+05, 6.308599E+05, 6.408684E+05, 6.510044E+05, 6.612691E+05, 6.716639E+05, 
   6.821899E+05, 6.928484E+05, 7.036406E+05, 7.145679E+05, 7.256314E+05, 7.368326E+05, 
   7.481726E+05, 7.596528E+05, 7.712745E+05, 7.830390E+05, 7.949476E+05, 8.070017E+05, 
   8.192026E+05, 8.315516E+05, 8.440502E+05, 8.566996E+05, 8.695013E+05, 8.824567E+05, 
   8.955671E+05, 9.088339E+05, 9.222586E+05, 9.358425E+05, 9.495872E+05, 9.634939E+05, 
   9.775643E+05, 9.917997E+05, 1.006202E+06, 1.020771E+06, 1.035511E+06, 1.050421E+06, 
   1.065504E+06, 1.080760E+06, 1.096192E+06, 1.111801E+06, 1.127589E+06, 1.143557E+06, 
   1.159706E+06, 1.176038E+06, 1.192555E+06, 1.209259E+06, 1.226150E+06, 1.243231E+06, 
   1.260503E+06, 1.277968E+06, 1.295627E+06, 1.313482E+06, 1.331535E+06, 1.349787E+06, 
   1.368240E+06, 1.386895E+06, 1.405755E+06, 1.424821E+06, 1.444095E+06, 1.463578E+06, 
   1.483272E+06, 1.503179E+06, 1.523301E+06, 1.543638E+06, 1.564194E+06, 1.584970E+06, 
   1.605968E+06, 1.627189E+06, 1.648635E+06, 1.670308E+06, 1.692211E+06, 1.714343E+06, 
   1.736709E+06, 1.759308E+06, 1.782144E+06, 1.805218E+06, 1.828532E+06, 1.852088E+06, 
   1.875888E+06, 1.899933E+06, 1.924226E+06, 1.948768E+06, 1.973562E+06, 1.998609E+06, 
   2.023912E+06, 2.049472E+06, 2.075291E+06, 2.101371E+06, 2.127715E+06, 2.154324E+06, 
   2.181200E+06, 2.208346E+06, 2.235763E+06, 2.263453E+06, 2.291419E+06, 2.319663E+06, 
   2.348186E+06, 2.376991E+06, 2.406080E+06, 2.435455E+06, 2.465118E+06, 2.495071E+06, 
   2.525316E+06, 2.555856E+06, 2.586693E+06, 2.617829E+06, 2.649266E+06, 2.681006E+06, 
   2.713051E+06, 2.745404E+06, 2.778068E+06, 2.811043E+06, 2.844333E+06, 2.877939E+06, 
   2.911865E+06, 2.946112E+06, 2.980682E+06, 3.015579E+06, 3.050803E+06, 3.086359E+06, 
   3.122247E+06, 3.158470E+06, 3.195031E+06, 3.231932E+06, 3.269176E+06, 3.306764E+06, 
   3.344700E+06, 3.382985E+06, 3.421622E+06, 3.460614E+06, 3.499963E+06, 3.539672E+06, 
   3.579742E+06, 3.620177E+06, 3.660979E+06, 3.702150E+06, 3.743694E+06, 3.785612E+06, 
   3.827907E+06, 3.870582E+06, 3.913639E+06, 3.957081E+06, 4.000910E+06, 4.045130E+06, 
   4.089742E+06, 4.134750E+06, 4.180156E+06, 4.225962E+06, 4.272172E+06, 4.318787E+06, 
   4.365812E+06, 4.413248E+06, 4.461098E+06, 4.509365E+06, 4.558052E+06, 4.607161E+06, 
   4.656695E+06, 4.706658E+06, 4.757051E+06, 4.807877E+06, 4.859141E+06, 4.910843E+06, 
   4.962988E+06, 5.015577E+06, 5.068614E+06, 5.122102E+06, 5.176044E+06, 5.230442E+06, 
   5.285300E+06, 5.340620E+06, 5.396406E+06, 5.452660E+06, 5.509385E+06, 5.566584E+06, 
   5.624261E+06, 5.682418E+06, 5.741059E+06, 5.800185E+06, 5.859802E+06, 5.919910E+06, 
   5.980515E+06, 6.041618E+06, 6.103223E+06, 6.165332E+06, 6.227950E+06, 6.291079E+06, 
   6.354722E+06, 6.418883E+06, 6.483565E+06, 6.548770E+06, 6.614503E+06, 6.680766E+06, 
   6.747562E+06, 6.814896E+06, 6.882769E+06, 6.951186E+06, 7.020150E+06, 7.089663E+06, 
   7.159730E+06, 7.230354E+06, 7.301537E+06, 7.373284E+06, 7.445598E+06, 7.518482E+06, 
   7.591940E+06, 7.665974E+06, 7.740589E+06, 7.815788E+06, 7.891574E+06, 7.967950E+06, 
   8.044921E+06, 8.122490E+06, 8.200660E+06, 8.279435E+06, 8.358819E+06, 8.438814E+06, 
   8.519425E+06, 8.600655E+06, 8.682507E+06, 8.764986E+06, 8.848095E+06, 8.931838E+06, 
   9.016218E+06, 9.101238E+06, 9.186904E+06, 9.273217E+06, 9.360183E+06, 9.447804E+06, 
   9.536085E+06, 9.625030E+06, 9.714641E+06, 9.804923E+06, 9.895880E+06, 9.987515E+06, 
   1.007983E+07, 1.017284E+07, 1.026653E+07, 1.036092E+07, 1.045600E+07, 1.055179E+07, 
   1.064828E+07, 1.074549E+07, 1.084340E+07, 
])
# ============================ M = 19, I = 6 ============================
M = 19
I = 6
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.756030E+00, 7.117103E+01, 1.416858E+02, 2.122181E+02, 2.827669E+02, 3.533324E+02, 
   4.239180E+02, 4.945416E+02, 5.652532E+02, 6.361519E+02, 7.073925E+02, 7.791810E+02, 
   8.517638E+02, 9.254135E+02, 1.000417E+03, 1.077065E+03, 1.155646E+03, 1.236441E+03, 
   1.319721E+03, 1.405749E+03, 1.494775E+03, 1.587042E+03, 1.682782E+03, 1.782219E+03, 
   1.885574E+03, 1.993059E+03, 2.104883E+03, 2.221252E+03, 2.342368E+03, 2.468433E+03, 
   2.599647E+03, 2.736210E+03, 2.878322E+03, 3.026185E+03, 3.180000E+03, 3.339970E+03, 
   3.506303E+03, 3.679206E+03, 3.858889E+03, 4.045567E+03, 4.239456E+03, 4.440775E+03, 
   4.649749E+03, 4.866605E+03, 5.091574E+03, 5.324891E+03, 5.566795E+03, 5.817530E+03, 
   6.077344E+03, 6.346489E+03, 6.625222E+03, 6.913804E+03, 7.212502E+03, 7.521587E+03, 
   7.841334E+03, 8.172026E+03, 8.513947E+03, 8.867389E+03, 9.232649E+03, 9.610028E+03, 
   9.999832E+03, 1.040237E+04, 1.081797E+04, 1.124694E+04, 1.168963E+04, 1.214635E+04, 
   1.261745E+04, 1.310328E+04, 1.360419E+04, 1.412052E+04, 1.465266E+04, 1.520096E+04, 
   1.576580E+04, 1.634755E+04, 1.694662E+04, 1.756338E+04, 1.819823E+04, 1.885159E+04, 
   1.952386E+04, 2.021546E+04, 2.092681E+04, 2.165834E+04, 2.241049E+04, 2.318370E+04, 
   2.397843E+04, 2.479511E+04, 2.563423E+04, 2.649624E+04, 2.738162E+04, 2.829086E+04, 
   2.922444E+04, 3.018286E+04, 3.116663E+04, 3.217624E+04, 3.321222E+04, 3.427510E+04, 
   3.536540E+04, 3.648366E+04, 3.763042E+04, 3.880624E+04, 4.001168E+04, 4.124731E+04, 
   4.251369E+04, 4.381141E+04, 4.514107E+04, 4.650325E+04, 4.789856E+04, 4.932761E+04, 
   5.079102E+04, 5.228943E+04, 5.382345E+04, 5.539374E+04, 5.700096E+04, 5.864574E+04, 
   6.032877E+04, 6.205072E+04, 6.381226E+04, 6.561410E+04, 6.745692E+04, 6.934144E+04, 
   7.126837E+04, 7.323844E+04, 7.525238E+04, 7.731092E+04, 7.941482E+04, 8.156483E+04, 
   8.376172E+04, 8.600627E+04, 8.829926E+04, 9.064147E+04, 9.303372E+04, 9.547680E+04, 
   9.797155E+04, 1.005188E+05, 1.031193E+05, 1.057741E+05, 1.084838E+05, 1.112494E+05, 
   1.140718E+05, 1.169518E+05, 1.198904E+05, 1.228884E+05, 1.259467E+05, 1.290663E+05, 
   1.322481E+05, 1.354930E+05, 1.388020E+05, 1.421760E+05, 1.456160E+05, 1.491230E+05, 
   1.526979E+05, 1.563419E+05, 1.600557E+05, 1.638406E+05, 1.676975E+05, 1.716274E+05, 
   1.756314E+05, 1.797106E+05, 1.838660E+05, 1.880986E+05, 1.924097E+05, 1.968002E+05, 
   2.012713E+05, 2.058241E+05, 2.104598E+05, 2.151794E+05, 2.199841E+05, 2.248752E+05, 
   2.298537E+05, 2.349208E+05, 2.400777E+05, 2.453257E+05, 2.506660E+05, 2.560997E+05, 
   2.616281E+05, 2.672525E+05, 2.729741E+05, 2.787942E+05, 2.847140E+05, 2.907349E+05, 
   2.968582E+05, 3.030852E+05, 3.094172E+05, 3.158555E+05, 3.224015E+05, 3.290566E+05, 
   3.358222E+05, 3.426995E+05, 3.496900E+05, 3.567952E+05, 3.640164E+05, 3.713551E+05, 
   3.788127E+05, 3.863907E+05, 3.940905E+05, 4.019137E+05, 4.098617E+05, 4.179360E+05, 
   4.261381E+05, 4.344696E+05, 4.429320E+05, 4.515268E+05, 4.602557E+05, 4.691202E+05, 
   4.781219E+05, 4.872624E+05, 4.965433E+05, 5.059663E+05, 5.155330E+05, 5.252450E+05, 
   5.351040E+05, 5.451117E+05, 5.552698E+05, 5.655801E+05, 5.760441E+05, 5.866637E+05, 
   5.974406E+05, 6.083766E+05, 6.194733E+05, 6.307328E+05, 6.421566E+05, 6.537467E+05, 
   6.655048E+05, 6.774329E+05, 6.895327E+05, 7.018061E+05, 7.142550E+05, 7.268814E+05, 
   7.396870E+05, 7.526739E+05, 7.658440E+05, 7.791992E+05, 7.927415E+05, 8.064729E+05, 
   8.203953E+05, 8.345108E+05, 8.488215E+05, 8.633292E+05, 8.780362E+05, 8.929444E+05, 
   9.080559E+05, 9.233728E+05, 9.388973E+05, 9.546315E+05, 9.705775E+05, 9.867374E+05, 
   1.003114E+06, 1.019708E+06, 1.036523E+06, 1.053560E+06, 1.070823E+06, 1.088312E+06, 
   1.106032E+06, 1.123983E+06, 1.142168E+06, 1.160589E+06, 1.179249E+06, 1.198150E+06, 
   1.217294E+06, 1.236684E+06, 1.256322E+06, 1.276210E+06, 1.296351E+06, 1.316748E+06, 
   1.337402E+06, 1.358317E+06, 1.379494E+06, 1.400937E+06, 1.422647E+06, 1.444627E+06, 
   1.466881E+06, 1.489409E+06, 1.512216E+06, 1.535302E+06, 1.558672E+06, 1.582328E+06, 
   1.606272E+06, 1.630507E+06, 1.655036E+06, 1.679861E+06, 1.704986E+06, 1.730412E+06, 
   1.756142E+06, 1.782180E+06, 1.808528E+06, 1.835189E+06, 1.862165E+06, 1.889460E+06, 
   1.917076E+06, 1.945017E+06, 1.973284E+06, 2.001882E+06, 2.030812E+06, 2.060079E+06, 
   2.089684E+06, 2.119630E+06, 2.149922E+06, 2.180560E+06, 2.211550E+06, 2.242894E+06, 
   2.274594E+06, 2.306654E+06, 2.339077E+06, 2.371866E+06, 2.405024E+06, 2.438555E+06, 
   2.472461E+06, 2.506746E+06, 2.541412E+06, 2.576464E+06, 2.611904E+06, 2.647735E+06, 
   2.683962E+06, 2.720586E+06, 2.757612E+06, 2.795042E+06, 2.832881E+06, 2.871131E+06, 
   2.909796E+06, 2.948879E+06, 2.988384E+06, 3.028314E+06, 3.068672E+06, 3.109462E+06, 
   3.150688E+06, 3.192353E+06, 3.234461E+06, 3.277014E+06, 3.320018E+06, 3.363474E+06, 
   3.407388E+06, 3.451762E+06, 3.496600E+06, 3.541906E+06, 3.587683E+06, 3.633936E+06, 
   3.680668E+06, 3.727882E+06, 3.775584E+06, 3.823775E+06, 3.872460E+06, 3.921644E+06, 
   3.971329E+06, 4.021520E+06, 4.072220E+06, 4.123434E+06, 4.175165E+06, 4.227418E+06, 
   4.280195E+06, 4.333502E+06, 4.387343E+06, 4.441721E+06, 4.496640E+06, 4.552105E+06, 
   4.608119E+06, 4.664687E+06, 4.721813E+06, 4.779501E+06, 4.837755E+06, 4.896580E+06, 
   4.955979E+06, 5.015957E+06, 5.076518E+06, 5.137666E+06, 5.199406E+06, 5.261743E+06, 
   5.324679E+06, 5.388221E+06, 5.452371E+06, 5.517136E+06, 5.582518E+06, 5.648522E+06, 
   5.715154E+06, 5.782418E+06, 5.850317E+06, 5.918856E+06, 5.988042E+06, 6.057876E+06, 
   6.128365E+06, 6.199513E+06, 6.271325E+06, 6.343805E+06, 6.416958E+06, 6.490788E+06, 
   6.565302E+06, 6.640502E+06, 6.716395E+06, 6.792984E+06, 6.870276E+06, 6.948274E+06, 
   7.026983E+06, 7.106409E+06, 7.186556E+06, 7.267429E+06, 7.349034E+06, 7.431375E+06, 
   7.514457E+06, 7.598286E+06, 7.682866E+06, 7.768202E+06, 7.854301E+06, 7.941166E+06, 
   8.028803E+06, 8.117217E+06, 8.206414E+06, 8.296398E+06, 8.387175E+06, 8.478750E+06, 
   8.571129E+06, 8.664316E+06, 8.758318E+06, 8.853139E+06, 8.948786E+06, 9.045262E+06, 
   9.142575E+06, 9.240729E+06, 9.339729E+06, 9.439582E+06, 9.540293E+06, 9.641868E+06, 
   9.744311E+06, 9.847630E+06, 9.951828E+06, 1.005691E+07, 1.016289E+07, 1.026976E+07, 
   1.037754E+07, 1.048623E+07, 1.059583E+07, 1.070635E+07, 1.081780E+07, 1.093018E+07, 
   1.104350E+07, 1.115776E+07, 1.127298E+07, 1.138915E+07, 1.150628E+07, 1.162438E+07, 
   1.174346E+07, 1.186351E+07, 1.198455E+07, 1.210659E+07, 1.222962E+07, 1.235366E+07, 
   1.247871E+07, 1.260477E+07, 1.273186E+07, 1.285998E+07, 1.298914E+07, 1.311934E+07, 
   1.325059E+07, 1.338290E+07, 1.351626E+07, 1.365070E+07, 1.378620E+07, 1.392280E+07, 
   1.406048E+07, 1.419925E+07, 1.433912E+07, 1.448010E+07, 1.462220E+07, 1.476542E+07, 
   1.490976E+07, 1.505524E+07, 1.520186E+07, 1.534963E+07, 1.549855E+07, 1.564864E+07, 
   1.579989E+07, 1.595232E+07, 1.610593E+07, 1.626073E+07, 1.641673E+07, 1.657393E+07, 
   1.673234E+07, 1.689196E+07, 1.705281E+07, 1.721490E+07, 1.737822E+07, 1.754279E+07, 
   1.770861E+07, 1.787569E+07, 1.804404E+07, 1.821366E+07, 1.838456E+07, 1.855676E+07, 
   1.873025E+07, 1.890504E+07, 1.908115E+07, 1.925858E+07, 1.943733E+07, 1.961742E+07, 
   1.979885E+07, 1.998162E+07, 2.016576E+07, 2.035126E+07, 2.053813E+07, 2.072638E+07, 
   2.091602E+07, 2.110705E+07, 2.129949E+07, 
])
# ============================ M = 20, I = 1 ============================
M = 20
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.091110E+00, 1.549359E+01, 4.947081E+01, 9.209028E+01, 1.416480E+02, 1.975455E+02, 
   2.592507E+02, 3.262979E+02, 3.983023E+02, 4.749476E+02, 5.559709E+02, 6.411494E+02, 
   7.302920E+02, 8.232325E+02, 9.198257E+02, 1.019944E+03, 1.123477E+03, 1.230329E+03, 
   1.340418E+03, 1.453678E+03, 1.570058E+03, 1.689519E+03, 1.812038E+03, 1.937607E+03, 
   2.066230E+03, 2.197927E+03, 2.332728E+03, 2.470679E+03, 2.611836E+03, 2.756267E+03, 
   2.904051E+03, 3.055278E+03, 3.210045E+03, 3.368460E+03, 3.530639E+03, 3.696705E+03, 
   3.866789E+03, 4.041028E+03, 4.219566E+03, 4.402554E+03, 4.590146E+03, 4.782503E+03, 
   4.979793E+03, 5.182187E+03, 5.389860E+03, 5.602994E+03, 5.821776E+03, 6.046395E+03, 
   6.277046E+03, 6.513930E+03, 6.757251E+03, 7.007218E+03, 7.264045E+03, 7.527951E+03, 
   7.799159E+03, 8.077898E+03, 8.364401E+03, 8.658907E+03, 8.961659E+03, 9.272906E+03, 
   9.592904E+03, 9.921911E+03, 1.026019E+04, 1.060802E+04, 1.096512E+04, 1.133220E+04, 
   1.170966E+04, 1.209767E+04, 1.249659E+04, 1.290671E+04, 1.332840E+04, 1.376186E+04, 
   1.420757E+04, 1.466575E+04, 1.513680E+04, 1.562090E+04, 1.611862E+04, 1.663026E+04, 
   1.715616E+04, 1.769665E+04, 1.825218E+04, 1.882296E+04, 1.940974E+04, 2.001261E+04, 
   2.063218E+04, 2.126884E+04, 2.192282E+04, 2.259491E+04, 2.328522E+04, 2.399443E+04, 
   2.472293E+04, 2.547130E+04, 2.623979E+04, 2.702912E+04, 2.783973E+04, 2.867222E+04, 
   2.952702E+04, 3.040459E+04, 3.130573E+04, 3.223073E+04, 3.318041E+04, 3.415525E+04, 
   3.515575E+04, 3.618259E+04, 3.723663E+04, 3.831804E+04, 3.942790E+04, 4.056655E+04, 
   4.173473E+04, 4.293338E+04, 4.416289E+04, 4.542400E+04, 4.671773E+04, 4.804445E+04, 
   4.940519E+04, 5.080033E+04, 5.223118E+04, 5.369833E+04, 5.520223E+04, 5.674442E+04, 
   5.832512E+04, 5.994522E+04, 6.160614E+04, 6.330807E+04, 6.505245E+04, 6.684001E+04, 
   6.867173E+04, 7.054835E+04, 7.247112E+04, 7.444108E+04, 7.645903E+04, 7.852572E+04, 
   8.064278E+04, 8.281104E+04, 8.503157E+04, 8.730552E+04, 8.963402E+04, 9.201794E+04, 
   9.445875E+04, 9.695761E+04, 9.951574E+04, 1.021341E+05, 1.048142E+05, 1.075573E+05, 
   1.103643E+05, 1.132370E+05, 1.161765E+05, 1.191842E+05, 1.222616E+05, 1.254099E+05, 
   1.286309E+05, 1.319253E+05, 1.352955E+05, 1.387421E+05, 1.422674E+05, 1.458723E+05, 
   1.495587E+05, 1.533284E+05, 1.571827E+05, 1.611234E+05, 1.651519E+05, 1.692703E+05, 
   1.734802E+05, 1.777830E+05, 1.821813E+05, 1.866760E+05, 1.912699E+05, 1.959638E+05, 
   2.007604E+05, 2.056616E+05, 2.106690E+05, 2.157846E+05, 2.210108E+05, 2.263496E+05, 
   2.318027E+05, 2.373722E+05, 2.430609E+05, 2.488709E+05, 2.548036E+05, 2.608620E+05, 
   2.670483E+05, 2.733644E+05, 2.798136E+05, 2.863973E+05, 2.931184E+05, 2.999790E+05, 
   3.069823E+05, 3.141301E+05, 3.214256E+05, 3.288712E+05, 3.364693E+05, 3.442231E+05, 
   3.521348E+05, 3.602075E+05, 3.684439E+05, 3.768471E+05, 3.854199E+05, 3.941650E+05, 
   4.030855E+05, 4.121846E+05, 4.214650E+05, 4.309303E+05, 4.405834E+05, 4.504275E+05, 
   4.604660E+05, 4.707023E+05, 4.811396E+05, 4.917806E+05, 5.026303E+05, 5.136905E+05, 
   5.249660E+05, 5.364598E+05, 5.481756E+05, 5.601169E+05, 5.722878E+05, 5.846917E+05, 
   5.973333E+05, 6.102159E+05, 6.233430E+05, 6.367190E+05, 6.503482E+05, 6.642343E+05, 
   6.783817E+05, 6.927947E+05, 7.074775E+05, 7.224342E+05, 7.376698E+05, 7.531879E+05, 
   7.689940E+05, 7.850917E+05, 8.014861E+05, 8.181815E+05, 8.351831E+05, 8.524957E+05, 
   8.701245E+05, 8.880732E+05, 9.063479E+05, 9.249535E+05, 9.438955E+05, 9.631783E+05, 
   9.828070E+05, 1.002788E+06, 1.023125E+06, 1.043826E+06, 1.064895E+06, 1.086337E+06, 
   1.108158E+06, 1.130365E+06, 1.152963E+06, 1.175957E+06, 1.199355E+06, 1.223161E+06, 
   1.247383E+06, 1.272025E+06, 1.297095E+06, 1.322599E+06, 1.348543E+06, 1.374934E+06, 
   1.401778E+06, 1.429082E+06, 1.456853E+06, 1.485098E+06, 1.513823E+06, 1.543035E+06, 
   1.572742E+06, 1.602951E+06, 1.633669E+06, 1.664902E+06, 1.696660E+06, 1.728950E+06, 
   1.761777E+06, 1.795151E+06, 1.829081E+06, 1.863571E+06, 1.898632E+06, 1.934272E+06, 
   1.970497E+06, 2.007318E+06, 2.044741E+06, 2.082776E+06, 2.121430E+06, 2.160713E+06, 
   2.200634E+06, 2.241201E+06, 2.282422E+06, 2.324308E+06, 2.366867E+06, 2.410109E+06, 
   2.454042E+06, 2.498677E+06, 2.544023E+06, 2.590089E+06, 2.636886E+06, 2.684423E+06, 
   2.732710E+06, 2.781758E+06, 2.831576E+06, 2.882175E+06, 2.933566E+06, 2.985758E+06, 
   3.038763E+06, 3.092591E+06, 3.147255E+06, 3.202763E+06, 3.259128E+06, 3.316360E+06, 
   3.374473E+06, 3.433476E+06, 3.493381E+06, 3.554201E+06, 3.615946E+06, 3.678630E+06, 
   3.742264E+06, 3.806862E+06, 3.872434E+06, 3.938993E+06, 4.006554E+06, 4.075126E+06, 
   4.144726E+06, 4.215365E+06, 4.287056E+06, 4.359812E+06, 4.433650E+06, 4.508580E+06, 
   4.584616E+06, 4.661773E+06, 4.740066E+06, 4.819506E+06, 4.900110E+06, 4.981894E+06, 
   5.064870E+06, 5.149052E+06, 5.234458E+06, 5.321103E+06, 5.408999E+06, 5.498163E+06, 
   5.588613E+06, 5.680362E+06, 5.773427E+06, 5.867824E+06, 5.963569E+06, 6.060679E+06, 
   6.159169E+06, 6.259059E+06, 6.360361E+06, 6.463098E+06, 6.567283E+06, 6.672933E+06, 
   6.780068E+06, 6.888706E+06, 6.998864E+06, 
])
# ============================ M = 20, I = 2 ============================
M = 20
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.519557E+00, 3.782502E+01, 1.041280E+02, 1.896019E+02, 2.906359E+02, 4.051291E+02, 
   5.316596E+02, 6.691821E+02, 8.168855E+02, 9.741174E+02, 1.140338E+03, 1.315089E+03, 
   1.497981E+03, 1.688673E+03, 1.886866E+03, 2.092301E+03, 2.304749E+03, 2.524015E+03, 
   2.749931E+03, 2.982361E+03, 3.221198E+03, 3.466365E+03, 3.717812E+03, 3.975520E+03, 
   4.239498E+03, 4.509784E+03, 4.786442E+03, 5.069561E+03, 5.359256E+03, 5.655666E+03, 
   5.958950E+03, 6.269290E+03, 6.586886E+03, 6.911958E+03, 7.244739E+03, 7.585482E+03, 
   7.934453E+03, 8.291932E+03, 8.658212E+03, 9.033599E+03, 9.418409E+03, 9.812970E+03, 
   1.021762E+04, 1.063271E+04, 1.105860E+04, 1.149566E+04, 1.194426E+04, 1.240479E+04, 
   1.287765E+04, 1.336324E+04, 1.386199E+04, 1.437431E+04, 1.490064E+04, 1.544142E+04, 
   1.599711E+04, 1.656817E+04, 1.715506E+04, 1.775828E+04, 1.837832E+04, 1.901569E+04, 
   1.967088E+04, 2.034444E+04, 2.103690E+04, 2.174880E+04, 2.248071E+04, 2.323319E+04, 
   2.400684E+04, 2.480225E+04, 2.562002E+04, 2.646078E+04, 2.732517E+04, 2.821382E+04, 
   2.912742E+04, 3.006662E+04, 3.103213E+04, 3.202465E+04, 3.304489E+04, 3.409361E+04, 
   3.517153E+04, 3.627945E+04, 3.741813E+04, 3.858838E+04, 3.979102E+04, 4.102688E+04, 
   4.229680E+04, 4.360167E+04, 4.494237E+04, 4.631980E+04, 4.773489E+04, 4.918858E+04, 
   5.068184E+04, 5.221565E+04, 5.379101E+04, 5.540895E+04, 5.707051E+04, 5.877675E+04, 
   6.052877E+04, 6.232767E+04, 6.417459E+04, 6.607068E+04, 6.801711E+04, 7.001509E+04, 
   7.206584E+04, 7.417062E+04, 7.633069E+04, 7.854735E+04, 8.082193E+04, 8.315578E+04, 
   8.555027E+04, 8.800681E+04, 9.052682E+04, 9.311177E+04, 9.576314E+04, 9.848244E+04, 
   1.012712E+05, 1.041311E+05, 1.070635E+05, 1.100703E+05, 1.131530E+05, 1.163133E+05, 
   1.195530E+05, 1.228739E+05, 1.262776E+05, 1.297661E+05, 1.333412E+05, 1.370047E+05, 
   1.407587E+05, 1.446050E+05, 1.485457E+05, 1.525828E+05, 1.567183E+05, 1.609545E+05, 
   1.652933E+05, 1.697370E+05, 1.742879E+05, 1.789481E+05, 1.837200E+05, 1.886059E+05, 
   1.936082E+05, 1.987294E+05, 2.039718E+05, 2.093380E+05, 2.148305E+05, 2.204520E+05, 
   2.262050E+05, 2.320924E+05, 2.381167E+05, 2.442807E+05, 2.505874E+05, 2.570395E+05, 
   2.636400E+05, 2.703919E+05, 2.772981E+05, 2.843618E+05, 2.915860E+05, 2.989740E+05, 
   3.065288E+05, 3.142540E+05, 3.221526E+05, 3.302282E+05, 3.384842E+05, 3.469241E+05, 
   3.555514E+05, 3.643697E+05, 3.733827E+05, 3.825942E+05, 3.920078E+05, 4.016275E+05, 
   4.114572E+05, 4.215008E+05, 4.317623E+05, 4.422459E+05, 4.529557E+05, 4.638959E+05, 
   4.750709E+05, 4.864849E+05, 4.981424E+05, 5.100479E+05, 5.222059E+05, 5.346211E+05, 
   5.472981E+05, 5.602418E+05, 5.734570E+05, 5.869485E+05, 6.007215E+05, 6.147808E+05, 
   6.291318E+05, 6.437796E+05, 6.587294E+05, 6.739868E+05, 6.895571E+05, 7.054458E+05, 
   7.216586E+05, 7.382011E+05, 7.550792E+05, 7.722988E+05, 7.898656E+05, 8.077859E+05, 
   8.260657E+05, 8.447112E+05, 8.637288E+05, 8.831247E+05, 9.029056E+05, 9.230779E+05, 
   9.436484E+05, 9.646238E+05, 9.860109E+05, 1.007817E+06, 1.030048E+06, 1.052713E+06, 
   1.075818E+06, 1.099370E+06, 1.123377E+06, 1.147846E+06, 1.172786E+06, 1.198204E+06, 
   1.224108E+06, 1.250505E+06, 1.277404E+06, 1.304813E+06, 1.332741E+06, 1.361195E+06, 
   1.390185E+06, 1.419718E+06, 1.449805E+06, 1.480453E+06, 1.511671E+06, 1.543470E+06, 
   1.575857E+06, 1.608842E+06, 1.642436E+06, 1.676646E+06, 1.711484E+06, 1.746959E+06, 
   1.783081E+06, 1.819860E+06, 1.857306E+06, 1.895430E+06, 1.934242E+06, 1.973753E+06, 
   2.013974E+06, 2.054915E+06, 2.096588E+06, 2.139004E+06, 2.182174E+06, 2.226110E+06, 
   2.270824E+06, 2.316326E+06, 2.362629E+06, 2.409746E+06, 2.457688E+06, 2.506468E+06, 
   2.556098E+06, 2.606591E+06, 2.657960E+06, 2.710217E+06, 2.763377E+06, 2.817452E+06, 
   2.872456E+06, 2.928402E+06, 2.985305E+06, 3.043178E+06, 3.102035E+06, 3.161892E+06, 
   3.222761E+06, 3.284659E+06, 3.347599E+06, 3.411597E+06, 3.476668E+06, 3.542828E+06, 
   3.610091E+06, 3.678475E+06, 3.747994E+06, 3.818665E+06, 3.890504E+06, 3.963528E+06, 
   4.037753E+06, 4.113196E+06, 4.189875E+06, 4.267806E+06, 4.347008E+06, 4.427497E+06, 
   4.509293E+06, 4.592412E+06, 4.676873E+06, 4.762695E+06, 4.849896E+06, 4.938496E+06, 
   5.028513E+06, 5.119967E+06, 5.212878E+06, 5.307265E+06, 5.403148E+06, 5.500548E+06, 
   5.599485E+06, 5.699980E+06, 5.802054E+06, 5.905727E+06, 6.011022E+06, 6.117961E+06, 
   6.226564E+06, 6.336854E+06, 6.448853E+06, 6.562585E+06, 6.678072E+06, 6.795336E+06, 
   6.914403E+06, 7.035294E+06, 7.158034E+06, 7.282647E+06, 7.409158E+06, 7.537591E+06, 
   7.667971E+06, 7.800324E+06, 7.934674E+06, 8.071047E+06, 8.209470E+06, 8.349969E+06, 
   8.492570E+06, 8.637301E+06, 8.784188E+06, 8.933258E+06, 9.084540E+06, 9.238061E+06, 
   9.393850E+06, 9.551936E+06, 9.712346E+06, 9.875111E+06, 1.004026E+07, 1.020782E+07, 
   1.037783E+07, 1.055031E+07, 1.072529E+07, 1.090281E+07, 1.108290E+07, 1.126559E+07, 
   1.145090E+07, 1.163889E+07, 1.182956E+07, 1.202297E+07, 1.221914E+07, 1.241810E+07, 
   1.261989E+07, 1.282455E+07, 1.303210E+07, 1.324259E+07, 1.345605E+07, 1.367251E+07, 
   1.389201E+07, 1.411459E+07, 1.434029E+07, 1.456913E+07, 1.480117E+07, 1.503643E+07, 
   1.527495E+07, 1.551678E+07, 1.576195E+07, 1.601050E+07, 1.626248E+07, 1.651791E+07, 
   1.677685E+07, 1.703933E+07, 1.730539E+07, 1.757508E+07, 1.784844E+07, 1.812551E+07, 
   1.840634E+07, 1.869096E+07, 1.897942E+07, 1.927176E+07, 1.956804E+07, 1.986829E+07, 
   2.017256E+07, 2.048089E+07, 2.079334E+07, 2.110994E+07, 2.143076E+07, 2.175582E+07, 
   2.208519E+07, 2.241891E+07, 2.275702E+07, 2.309959E+07, 2.344665E+07, 2.379826E+07, 
   2.415447E+07, 2.451533E+07, 2.488090E+07, 2.525121E+07, 2.562634E+07, 2.600632E+07, 
   2.639122E+07, 2.678109E+07, 2.717598E+07, 2.757595E+07, 2.798105E+07, 2.839134E+07, 
   2.880688E+07, 2.922772E+07, 2.965392E+07, 3.008554E+07, 3.052264E+07, 3.096528E+07, 
   3.141351E+07, 3.186740E+07, 3.232701E+07, 3.279239E+07, 3.326362E+07, 3.374075E+07, 
   3.422385E+07, 3.471298E+07, 3.520820E+07, 3.570958E+07, 3.621718E+07, 3.673107E+07, 
   3.725132E+07, 3.777800E+07, 3.831116E+07, 3.885088E+07, 3.939723E+07, 3.995027E+07, 
   4.051009E+07, 4.107674E+07, 4.165029E+07, 4.223083E+07, 4.281842E+07, 4.341313E+07, 
   4.401504E+07, 4.462423E+07, 4.524076E+07, 4.586471E+07, 4.649617E+07, 4.713520E+07, 
   4.778188E+07, 4.843630E+07, 4.909852E+07, 4.976863E+07, 5.044671E+07, 5.113284E+07, 
   5.182711E+07, 5.252958E+07, 5.324036E+07, 5.395951E+07, 5.468712E+07, 5.542328E+07, 
   5.616808E+07, 5.692159E+07, 5.768392E+07, 5.845513E+07, 5.923533E+07, 6.002460E+07, 
   6.082303E+07, 6.163072E+07, 6.244774E+07, 6.327421E+07, 6.411020E+07, 6.495581E+07, 
   6.581113E+07, 6.667627E+07, 6.755131E+07, 6.843636E+07, 6.933150E+07, 7.023685E+07, 
   7.115249E+07, 7.207852E+07, 7.301506E+07, 7.396219E+07, 7.492003E+07, 7.588866E+07, 
   7.686821E+07, 7.785876E+07, 7.886044E+07, 7.987333E+07, 8.089756E+07, 8.193322E+07, 
   8.298043E+07, 8.403929E+07, 8.510993E+07, 8.619244E+07, 8.728694E+07, 8.839354E+07, 
   8.951236E+07, 9.064351E+07, 9.178711E+07, 9.294327E+07, 9.411211E+07, 9.529375E+07, 
   9.648831E+07, 9.769591E+07, 9.891667E+07, 1.001507E+08, 1.013982E+08, 1.026591E+08, 
   1.039338E+08, 1.052222E+08, 1.065245E+08, 1.078408E+08, 1.091713E+08, 1.105161E+08, 
   1.118754E+08, 1.132492E+08, 1.146376E+08, 
])
# ============================ M = 20, I = 3 ============================
M = 20
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.771648E-01, 1.932880E+01, 5.324128E+01, 9.696334E+01, 1.486468E+02, 2.072166E+02, 
   2.719452E+02, 3.422976E+02, 4.178588E+02, 4.982949E+02, 5.833296E+02, 6.727294E+02, 
   7.662937E+02, 8.638484E+02, 9.652412E+02, 1.070339E+03, 1.179024E+03, 1.291198E+03, 
   1.406774E+03, 1.525683E+03, 1.647869E+03, 1.773294E+03, 1.901931E+03, 2.033772E+03, 
   2.168821E+03, 2.307097E+03, 2.448633E+03, 2.593474E+03, 2.741680E+03, 2.893320E+03, 
   3.048479E+03, 3.207246E+03, 3.369726E+03, 3.536030E+03, 3.706278E+03, 3.880600E+03, 
   4.059131E+03, 4.242015E+03, 4.429401E+03, 4.621447E+03, 4.818313E+03, 5.020168E+03, 
   5.227185E+03, 5.439542E+03, 5.657424E+03, 5.881019E+03, 6.110520E+03, 6.346125E+03, 
   6.588037E+03, 6.836465E+03, 7.091620E+03, 7.353720E+03, 7.622987E+03, 7.899648E+03, 
   8.183934E+03, 8.476083E+03, 8.776336E+03, 9.084941E+03, 9.402149E+03, 9.728219E+03, 
   1.006342E+04, 1.040800E+04, 1.076226E+04, 1.112647E+04, 1.150091E+04, 1.188587E+04, 
   1.228167E+04, 1.268859E+04, 1.310696E+04, 1.353709E+04, 1.397930E+04, 1.443393E+04, 
   1.490132E+04, 1.538182E+04, 1.587576E+04, 1.638353E+04, 1.690548E+04, 1.744200E+04, 
   1.799346E+04, 1.856026E+04, 1.914281E+04, 1.974150E+04, 2.035677E+04, 2.098902E+04, 
   2.163871E+04, 2.230628E+04, 2.299217E+04, 2.369686E+04, 2.442081E+04, 2.516451E+04, 
   2.592846E+04, 2.671315E+04, 2.751909E+04, 2.834682E+04, 2.919687E+04, 3.006977E+04, 
   3.096610E+04, 3.188640E+04, 3.283128E+04, 3.380131E+04, 3.479709E+04, 3.581925E+04, 
   3.686841E+04, 3.794520E+04, 3.905028E+04, 4.018431E+04, 4.134798E+04, 4.254196E+04, 
   4.376697E+04, 4.502373E+04, 4.631296E+04, 4.763540E+04, 4.899183E+04, 5.038302E+04, 
   5.180974E+04, 5.327282E+04, 5.477306E+04, 5.631131E+04, 5.788840E+04, 5.950522E+04, 
   6.116264E+04, 6.286157E+04, 6.460290E+04, 6.638759E+04, 6.821658E+04, 7.009084E+04, 
   7.201135E+04, 7.397912E+04, 7.599516E+04, 7.806051E+04, 8.017625E+04, 8.234343E+04, 
   8.456316E+04, 8.683655E+04, 8.916475E+04, 9.154890E+04, 9.399018E+04, 9.648980E+04, 
   9.904896E+04, 1.016689E+05, 1.043509E+05, 1.070962E+05, 1.099062E+05, 1.127821E+05, 
   1.157254E+05, 1.187373E+05, 1.218193E+05, 1.249728E+05, 1.281993E+05, 1.315002E+05, 
   1.348770E+05, 1.383312E+05, 1.418644E+05, 1.454781E+05, 1.491740E+05, 1.529537E+05, 
   1.568187E+05, 1.607709E+05, 1.648118E+05, 1.689432E+05, 1.731670E+05, 1.774848E+05, 
   1.818985E+05, 1.864099E+05, 1.910209E+05, 1.957335E+05, 2.005494E+05, 2.054709E+05, 
   2.104997E+05, 2.156379E+05, 2.208877E+05, 2.262511E+05, 2.317302E+05, 2.373271E+05, 
   2.430442E+05, 2.488836E+05, 2.548475E+05, 2.609383E+05, 2.671583E+05, 2.735099E+05, 
   2.799954E+05, 2.866174E+05, 2.933782E+05, 3.002804E+05, 3.073266E+05, 3.145193E+05, 
   3.218612E+05, 3.293550E+05, 3.370033E+05, 3.448089E+05, 3.527746E+05, 3.609032E+05, 
   3.691976E+05, 3.776608E+05, 3.862955E+05, 3.951050E+05, 4.040921E+05, 4.132601E+05, 
   4.226120E+05, 4.321510E+05, 4.418803E+05, 4.518032E+05, 4.619230E+05, 4.722431E+05, 
   4.827669E+05, 4.934979E+05, 5.044395E+05, 5.155952E+05, 5.269689E+05, 5.385640E+05, 
   5.503842E+05, 5.624335E+05, 5.747154E+05, 5.872340E+05, 5.999932E+05, 6.129968E+05, 
   6.262489E+05, 6.397537E+05, 6.535152E+05, 6.675377E+05, 6.818254E+05, 6.963826E+05, 
   7.112136E+05, 7.263229E+05, 7.417150E+05, 7.573944E+05, 7.733657E+05, 7.896336E+05, 
   8.062027E+05, 8.230780E+05, 8.402642E+05, 8.577663E+05, 8.755892E+05, 8.937381E+05, 
   9.122179E+05, 9.310338E+05, 9.501912E+05, 9.696953E+05, 9.895515E+05, 1.009765E+06, 
   1.030342E+06, 1.051288E+06, 1.072608E+06, 1.094307E+06, 1.116393E+06, 1.138871E+06, 
   1.161746E+06, 1.185025E+06, 1.208713E+06, 1.232818E+06, 1.257345E+06, 1.282301E+06, 
   1.307691E+06, 1.333523E+06, 1.359804E+06, 1.386538E+06, 1.413735E+06, 1.441399E+06, 
   1.469539E+06, 1.498161E+06, 1.527272E+06, 1.556880E+06, 1.586991E+06, 1.617614E+06, 
   1.648754E+06, 1.680421E+06, 1.712621E+06, 1.745362E+06, 1.778652E+06, 1.812500E+06, 
   1.846911E+06, 1.881896E+06, 1.917462E+06, 1.953617E+06, 1.990370E+06, 2.027728E+06, 
   2.065702E+06, 2.104298E+06, 2.143527E+06, 2.183397E+06, 2.223916E+06, 2.265094E+06, 
   2.306940E+06, 2.349464E+06, 2.392674E+06, 2.436580E+06, 2.481192E+06, 2.526520E+06, 
   2.572572E+06, 2.619360E+06, 2.666893E+06, 2.715181E+06, 2.764234E+06, 2.814064E+06, 
   2.864680E+06, 2.916093E+06, 2.968314E+06, 3.021353E+06, 3.075221E+06, 3.129931E+06, 
   3.185492E+06, 3.241916E+06, 3.299215E+06, 3.357399E+06, 3.416482E+06, 3.476474E+06, 
   3.537388E+06, 3.599236E+06, 3.662030E+06, 3.725781E+06, 3.790504E+06, 3.856210E+06, 
   3.922912E+06, 3.990623E+06, 4.059356E+06, 4.129125E+06, 4.199942E+06, 4.271821E+06, 
   4.344775E+06, 4.418819E+06, 4.493966E+06, 4.570230E+06, 4.647625E+06, 4.726166E+06, 
   4.805868E+06, 4.886744E+06, 4.968809E+06, 5.052079E+06, 5.136569E+06, 5.222294E+06, 
   5.309268E+06, 5.397508E+06, 5.487030E+06, 5.577850E+06, 5.669982E+06, 5.763444E+06, 
   5.858252E+06, 5.954422E+06, 6.051972E+06, 6.150918E+06, 6.251276E+06, 6.353065E+06, 
   6.456302E+06, 6.561003E+06, 6.667188E+06, 6.774874E+06, 6.884078E+06, 6.994820E+06, 
   7.107117E+06, 7.220988E+06, 7.336453E+06, 7.453530E+06, 7.572238E+06, 7.692596E+06, 
   7.814625E+06, 7.938344E+06, 8.063772E+06, 8.190931E+06, 8.319840E+06, 8.450520E+06, 
   8.582992E+06, 8.717276E+06, 8.853394E+06, 8.991367E+06, 9.131217E+06, 9.272965E+06, 
   9.416634E+06, 9.562245E+06, 9.709820E+06, 9.859384E+06, 1.001096E+07, 1.016456E+07, 
   1.032023E+07, 1.047797E+07, 1.063782E+07, 1.079979E+07, 1.096392E+07, 1.113022E+07, 
   1.129873E+07, 1.146946E+07, 1.164244E+07, 1.181769E+07, 1.199525E+07, 1.217513E+07, 
   1.235737E+07, 1.254198E+07, 1.272900E+07, 1.291846E+07, 1.311037E+07, 1.330477E+07, 
   1.350168E+07, 1.370114E+07, 1.390316E+07, 1.410779E+07, 1.431504E+07, 1.452494E+07, 
   1.473753E+07, 1.495283E+07, 1.517087E+07, 1.539169E+07, 1.561531E+07, 1.584176E+07, 
   1.607108E+07, 1.630328E+07, 1.653842E+07, 1.677651E+07, 1.701759E+07, 1.726169E+07, 
   1.750884E+07, 1.775907E+07, 1.801243E+07, 1.826893E+07, 1.852862E+07, 1.879153E+07, 
   1.905769E+07, 1.932713E+07, 1.959990E+07, 1.987602E+07, 2.015553E+07, 2.043846E+07, 
   2.072486E+07, 2.101476E+07, 2.130819E+07, 2.160519E+07, 2.190580E+07, 2.221006E+07, 
   2.251799E+07, 2.282965E+07, 2.314507E+07, 2.346428E+07, 2.378733E+07, 2.411426E+07, 
   2.444510E+07, 2.477990E+07, 2.511869E+07, 2.546152E+07, 2.580842E+07, 2.615944E+07, 
   2.651463E+07, 2.687401E+07, 2.723764E+07, 2.760556E+07, 2.797780E+07, 2.835442E+07, 
   2.873546E+07, 2.912096E+07, 2.951096E+07, 2.990551E+07, 3.030466E+07, 3.070845E+07, 
   3.111692E+07, 3.153013E+07, 3.194812E+07, 3.237094E+07, 3.279863E+07, 3.323124E+07, 
   3.366882E+07, 3.411143E+07, 3.455910E+07, 3.501188E+07, 3.546984E+07, 3.593301E+07, 
   3.640145E+07, 3.687521E+07, 3.735434E+07, 3.783889E+07, 3.832892E+07, 3.882447E+07, 
   3.932560E+07, 3.983237E+07, 4.034482E+07, 4.086302E+07, 4.138701E+07, 4.191685E+07, 
   4.245260E+07, 4.299432E+07, 4.354205E+07, 4.409586E+07, 4.465580E+07, 4.522194E+07, 
   4.579432E+07, 4.637302E+07, 4.695808E+07, 4.754957E+07, 4.814755E+07, 4.875207E+07, 
   4.936321E+07, 4.998101E+07, 5.060555E+07, 5.123688E+07, 5.187508E+07, 5.252019E+07, 
   5.317228E+07, 5.383143E+07, 5.449769E+07, 5.517113E+07, 5.585182E+07, 5.653982E+07, 
   5.723520E+07, 5.793802E+07, 5.864835E+07, 
])
# ============================ M = 21, I = 1 ============================
M = 21
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.980795E+00, 1.181810E+02, 3.303703E+02, 6.045991E+02, 9.291006E+02, 1.297053E+03, 
   1.703849E+03, 2.146102E+03, 2.621196E+03, 3.127032E+03, 3.661905E+03, 4.224417E+03, 
   4.813442E+03, 5.428090E+03, 6.067686E+03, 6.731751E+03, 7.419983E+03, 8.132235E+03, 
   8.868498E+03, 9.628884E+03, 1.041361E+04, 1.122298E+04, 1.205737E+04, 1.291723E+04, 
   1.380307E+04, 1.471544E+04, 1.565492E+04, 1.662215E+04, 1.761778E+04, 1.864248E+04, 
   1.969697E+04, 2.078197E+04, 2.189821E+04, 2.304644E+04, 2.422742E+04, 2.544194E+04, 
   2.669077E+04, 2.797470E+04, 2.929454E+04, 3.065108E+04, 3.204514E+04, 3.347754E+04, 
   3.494911E+04, 3.646067E+04, 3.801305E+04, 3.960711E+04, 4.124368E+04, 4.292363E+04, 
   4.464780E+04, 4.641707E+04, 4.823229E+04, 5.009436E+04, 5.200416E+04, 5.396257E+04, 
   5.597049E+04, 5.802883E+04, 6.013850E+04, 6.230042E+04, 6.451551E+04, 6.678472E+04, 
   6.910899E+04, 7.148926E+04, 7.392650E+04, 7.642168E+04, 7.897577E+04, 8.158977E+04, 
   8.426467E+04, 8.700148E+04, 8.980122E+04, 9.266490E+04, 9.559357E+04, 9.858828E+04, 
   1.016501E+05, 1.047800E+05, 1.079792E+05, 1.112487E+05, 1.145896E+05, 1.180030E+05, 
   1.214901E+05, 1.250519E+05, 1.286896E+05, 1.324045E+05, 1.361975E+05, 1.400699E+05, 
   1.440229E+05, 1.480577E+05, 1.521755E+05, 1.563775E+05, 1.606649E+05, 1.650390E+05, 
   1.695010E+05, 1.740522E+05, 1.786939E+05, 1.834273E+05, 1.882538E+05, 1.931747E+05, 
   1.981912E+05, 2.033048E+05, 2.085168E+05, 2.138284E+05, 2.192412E+05, 2.247565E+05, 
   2.303756E+05, 2.361000E+05, 2.419312E+05, 2.478704E+05, 2.539193E+05, 2.600791E+05, 
   2.663515E+05, 2.727379E+05, 2.792397E+05, 2.858585E+05, 2.925958E+05, 2.994531E+05, 
   3.064319E+05, 3.135339E+05, 3.207606E+05, 3.281135E+05, 3.355943E+05, 3.432045E+05, 
   3.509458E+05, 3.588198E+05, 3.668282E+05, 3.749725E+05, 3.832545E+05, 3.916759E+05, 
   4.002382E+05, 4.089434E+05, 4.177930E+05, 4.267888E+05, 4.359325E+05, 4.452259E+05, 
   4.546708E+05, 4.642689E+05, 4.740221E+05, 4.839321E+05, 4.940008E+05, 5.042300E+05, 
   5.146216E+05, 5.251774E+05, 5.358992E+05, 5.467891E+05, 5.578489E+05, 5.690805E+05, 
   5.804858E+05, 5.920667E+05, 6.038253E+05, 6.157635E+05, 6.278833E+05, 6.401867E+05, 
   6.526756E+05, 6.653522E+05, 6.782183E+05, 6.912762E+05, 7.045279E+05, 7.179754E+05, 
   7.316208E+05, 7.454662E+05, 7.595138E+05, 7.737657E+05, 7.882240E+05, 8.028909E+05, 
   8.177686E+05, 8.328593E+05, 8.481651E+05, 8.636882E+05, 8.794310E+05, 8.953956E+05, 
   9.115844E+05, 9.279995E+05, 9.446433E+05, 9.615181E+05, 9.786261E+05, 9.959698E+05, 
   1.013551E+06, 1.031373E+06, 1.049438E+06, 1.067748E+06, 1.086305E+06, 1.105112E+06, 
   1.124172E+06, 1.143486E+06, 1.163057E+06, 1.182888E+06, 1.202982E+06, 1.223340E+06, 
   1.243965E+06, 1.264860E+06, 1.286028E+06, 1.307470E+06, 1.329190E+06, 1.351190E+06, 
   1.373472E+06, 1.396040E+06, 1.418895E+06, 1.442042E+06, 1.465481E+06, 1.489216E+06, 
   1.513250E+06, 1.537586E+06, 1.562225E+06, 1.587171E+06, 1.612427E+06, 1.637995E+06, 
   1.663878E+06, 1.690079E+06, 1.716601E+06, 1.743446E+06, 1.770618E+06, 1.798119E+06, 
   1.825953E+06, 1.854121E+06, 1.882628E+06, 1.911475E+06, 1.940666E+06, 1.970204E+06, 
   2.000092E+06, 2.030333E+06, 2.060929E+06, 2.091884E+06, 2.123202E+06, 2.154884E+06, 
   2.186934E+06, 2.219355E+06, 2.252150E+06, 2.285323E+06, 2.318876E+06, 2.352813E+06, 
   2.387136E+06, 2.421849E+06, 2.456956E+06, 2.492459E+06, 2.528361E+06, 2.564666E+06, 
   2.601378E+06, 2.638498E+06, 2.676031E+06, 2.713981E+06, 2.752349E+06, 2.791140E+06, 
   2.830358E+06, 2.870004E+06, 2.910084E+06, 2.950599E+06, 2.991555E+06, 3.032953E+06, 
   3.074798E+06, 3.117092E+06, 3.159841E+06, 3.203046E+06, 3.246712E+06, 3.290842E+06, 
   3.335440E+06, 3.380508E+06, 3.426052E+06, 3.472074E+06, 3.518578E+06, 3.565568E+06, 
   3.613047E+06, 3.661019E+06, 3.709488E+06, 3.758458E+06, 3.807931E+06, 3.857913E+06, 
   3.908406E+06, 3.959415E+06, 4.010943E+06, 4.062994E+06, 4.115572E+06, 4.168680E+06, 
   4.222324E+06, 4.276506E+06, 4.331230E+06, 4.386501E+06, 4.442322E+06, 4.498698E+06, 
   4.555631E+06, 4.613127E+06, 4.671189E+06, 4.729822E+06, 4.789028E+06, 4.848813E+06, 
   4.909181E+06, 4.970135E+06, 5.031679E+06, 5.093819E+06, 5.156557E+06, 5.219899E+06, 
   5.283848E+06, 5.348408E+06, 5.413584E+06, 5.479380E+06, 5.545800E+06, 5.612849E+06, 
   5.680531E+06, 5.748850E+06, 5.817810E+06, 5.887416E+06, 5.957672E+06, 6.028582E+06, 
   6.100152E+06, 6.172385E+06, 6.245286E+06, 6.318859E+06, 6.393109E+06, 6.468040E+06, 
   6.543657E+06, 6.619964E+06, 6.696966E+06, 6.774668E+06, 6.853073E+06, 6.932187E+06, 
   7.012015E+06, 7.092560E+06, 7.173828E+06, 7.255823E+06, 7.338550E+06, 7.422013E+06, 
   7.506218E+06, 7.591169E+06, 7.676872E+06, 7.763329E+06, 7.850548E+06, 7.938531E+06, 
   8.027285E+06, 8.116814E+06, 8.207123E+06, 8.298217E+06, 8.390101E+06, 8.482780E+06, 
   8.576258E+06, 8.670541E+06, 8.765634E+06, 8.861541E+06, 8.958268E+06, 9.055820E+06, 
   9.154202E+06, 9.253419E+06, 9.353477E+06, 9.454379E+06, 9.556132E+06, 9.658741E+06, 
   9.762210E+06, 9.866546E+06, 9.971753E+06, 1.007784E+07, 1.018480E+07, 1.029265E+07, 
   1.040140E+07, 1.051104E+07, 1.062159E+07, 1.073304E+07, 1.084541E+07, 1.095870E+07, 
   1.107291E+07, 1.118805E+07, 1.130413E+07, 1.142115E+07, 1.153912E+07, 1.165803E+07, 
   1.177791E+07, 1.189875E+07, 1.202056E+07, 1.214334E+07, 1.226710E+07, 1.239185E+07, 
   1.251759E+07, 1.264433E+07, 1.277207E+07, 1.290082E+07, 1.303059E+07, 1.316137E+07, 
   1.329318E+07, 1.342602E+07, 1.355990E+07, 1.369483E+07, 1.383080E+07, 1.396783E+07, 
   1.410592E+07, 1.424507E+07, 1.438530E+07, 1.452661E+07, 1.466901E+07, 1.481249E+07, 
   1.495708E+07, 1.510276E+07, 1.524956E+07, 1.539747E+07, 1.554650E+07, 1.569666E+07, 
   1.584795E+07, 1.600039E+07, 1.615397E+07, 1.630870E+07, 1.646459E+07, 1.662165E+07, 
   1.677988E+07, 1.693928E+07, 1.709988E+07, 1.726166E+07, 1.742463E+07, 1.758882E+07, 
   1.775420E+07, 1.792081E+07, 1.808864E+07, 1.825770E+07, 1.842799E+07, 1.859953E+07, 
   1.877231E+07, 1.894635E+07, 1.912165E+07, 1.929822E+07, 1.947607E+07, 1.965519E+07, 
   1.983561E+07, 2.001732E+07, 2.020033E+07, 2.038465E+07, 2.057029E+07, 2.075724E+07, 
   2.094553E+07, 2.113515E+07, 2.132612E+07, 2.151843E+07, 2.171210E+07, 2.190714E+07, 
   2.210354E+07, 2.230133E+07, 2.250049E+07, 2.270105E+07, 2.290301E+07, 2.310637E+07, 
   2.331114E+07, 2.351733E+07, 2.372495E+07, 2.393401E+07, 2.414450E+07, 2.435644E+07, 
   2.456984E+07, 2.478470E+07, 2.500103E+07, 2.521884E+07, 2.543813E+07, 2.565891E+07, 
   2.588120E+07, 2.610498E+07, 2.633029E+07, 2.655711E+07, 2.678546E+07, 2.701535E+07, 
   2.724678E+07, 2.747977E+07, 2.771431E+07, 2.795042E+07, 2.818810E+07, 2.842736E+07, 
   2.866822E+07, 2.891066E+07, 2.915472E+07, 2.940038E+07, 2.964767E+07, 2.989658E+07, 
   3.014713E+07, 3.039932E+07, 3.065316E+07, 3.090866E+07, 3.116582E+07, 3.142466E+07, 
   3.168518E+07, 3.194740E+07, 3.221131E+07, 3.247692E+07, 3.274425E+07, 3.301330E+07, 
   3.328409E+07, 3.355661E+07, 3.383087E+07, 3.410689E+07, 3.438468E+07, 3.466423E+07, 
   3.494556E+07, 3.522868E+07, 3.551359E+07, 3.580031E+07, 3.608884E+07, 3.637919E+07, 
   3.667136E+07, 3.696538E+07, 3.726124E+07, 3.755895E+07, 3.785852E+07, 3.815996E+07, 
   3.846328E+07, 3.876849E+07, 3.907560E+07, 3.938461E+07, 3.969553E+07, 4.000837E+07, 
   4.032314E+07, 4.063985E+07, 4.095851E+07, 
])
# ============================ M = 21, I = 2 ============================
M = 21
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.057442E+00, 1.202269E+02, 3.361583E+02, 6.152332E+02, 9.454744E+02, 1.319938E+03, 
   1.733935E+03, 2.184019E+03, 2.667526E+03, 3.182321E+03, 3.726667E+03, 4.299144E+03, 
   4.898603E+03, 5.524140E+03, 6.175067E+03, 6.850898E+03, 7.551324E+03, 8.276195E+03, 
   9.025504E+03, 9.799363E+03, 1.059799E+04, 1.142170E+04, 1.227088E+04, 1.314598E+04, 
   1.404752E+04, 1.497605E+04, 1.593218E+04, 1.691655E+04, 1.792982E+04, 1.897269E+04, 
   2.004587E+04, 2.115009E+04, 2.228611E+04, 2.345469E+04, 2.465660E+04, 2.589264E+04, 
   2.716360E+04, 2.847029E+04, 2.981351E+04, 3.119410E+04, 3.261287E+04, 3.407065E+04, 
   3.556830E+04, 3.710664E+04, 3.868654E+04, 4.030885E+04, 4.197443E+04, 4.368415E+04, 
   4.543888E+04, 4.723950E+04, 4.908690E+04, 5.098197E+04, 5.292561E+04, 5.491873E+04, 
   5.696224E+04, 5.905706E+04, 6.120411E+04, 6.340435E+04, 6.565870E+04, 6.796813E+04, 
   7.033358E+04, 7.275604E+04, 7.523648E+04, 7.777588E+04, 8.037524E+04, 8.303557E+04, 
   8.575788E+04, 8.854320E+04, 9.139255E+04, 9.430699E+04, 9.728757E+04, 1.003354E+05, 
   1.034514E+05, 1.066368E+05, 1.098927E+05, 1.132201E+05, 1.166202E+05, 1.200942E+05, 
   1.236430E+05, 1.272680E+05, 1.309702E+05, 1.347509E+05, 1.386111E+05, 1.425522E+05, 
   1.465753E+05, 1.506816E+05, 1.548723E+05, 1.591488E+05, 1.635122E+05, 1.679638E+05, 
   1.725049E+05, 1.771368E+05, 1.818607E+05, 1.866781E+05, 1.915901E+05, 1.965982E+05, 
   2.017036E+05, 2.069079E+05, 2.122122E+05, 2.176180E+05, 2.231267E+05, 2.287398E+05, 
   2.344585E+05, 2.402844E+05, 2.462189E+05, 2.522634E+05, 2.584194E+05, 2.646885E+05, 
   2.710720E+05, 2.775716E+05, 2.841886E+05, 2.909247E+05, 2.977814E+05, 3.047603E+05, 
   3.118628E+05, 3.190907E+05, 3.264455E+05, 3.339287E+05, 3.415421E+05, 3.492872E+05, 
   3.571657E+05, 3.651793E+05, 3.733296E+05, 3.816183E+05, 3.900471E+05, 3.986177E+05, 
   4.073318E+05, 4.161912E+05, 4.251977E+05, 4.343529E+05, 4.436587E+05, 4.531169E+05, 
   4.627291E+05, 4.724974E+05, 4.824234E+05, 4.925091E+05, 5.027563E+05, 5.131668E+05, 
   5.237426E+05, 5.344854E+05, 5.453974E+05, 5.564803E+05, 5.677361E+05, 5.791667E+05, 
   5.907742E+05, 6.025604E+05, 6.145274E+05, 6.266772E+05, 6.390118E+05, 6.515333E+05, 
   6.642436E+05, 6.771448E+05, 6.902391E+05, 7.035284E+05, 7.170149E+05, 7.307008E+05, 
   7.445881E+05, 7.586789E+05, 7.729755E+05, 7.874800E+05, 8.021946E+05, 8.171215E+05, 
   8.322629E+05, 8.476210E+05, 8.631981E+05, 8.789964E+05, 8.950182E+05, 9.112658E+05, 
   9.277415E+05, 9.444476E+05, 9.613864E+05, 9.785602E+05, 9.959715E+05, 1.013623E+06, 
   1.031516E+06, 1.049654E+06, 1.068039E+06, 1.086673E+06, 1.105559E+06, 1.124700E+06, 
   1.144097E+06, 1.163753E+06, 1.183672E+06, 1.203854E+06, 1.224304E+06, 1.245023E+06, 
   1.266014E+06, 1.287279E+06, 1.308822E+06, 1.330644E+06, 1.352749E+06, 1.375139E+06, 
   1.397816E+06, 1.420784E+06, 1.444045E+06, 1.467601E+06, 1.491456E+06, 1.515612E+06, 
   1.540072E+06, 1.564839E+06, 1.589915E+06, 1.615303E+06, 1.641006E+06, 1.667028E+06, 
   1.693370E+06, 1.720035E+06, 1.747027E+06, 1.774348E+06, 1.802002E+06, 1.829990E+06, 
   1.858317E+06, 1.886985E+06, 1.915997E+06, 1.945355E+06, 1.975064E+06, 2.005126E+06, 
   2.035543E+06, 2.066320E+06, 2.097459E+06, 2.128963E+06, 2.160835E+06, 2.193079E+06, 
   2.225697E+06, 2.258693E+06, 2.292069E+06, 2.325830E+06, 2.359978E+06, 2.394516E+06, 
   2.429448E+06, 2.464776E+06, 2.500505E+06, 2.536637E+06, 2.573176E+06, 2.610125E+06, 
   2.647487E+06, 2.685265E+06, 2.723464E+06, 2.762086E+06, 2.801134E+06, 2.840613E+06, 
   2.880526E+06, 2.920875E+06, 2.961665E+06, 3.002899E+06, 3.044580E+06, 3.086712E+06, 
   3.129298E+06, 3.172343E+06, 3.215849E+06, 3.259820E+06, 3.304260E+06, 3.349172E+06, 
   3.394560E+06, 3.440428E+06, 3.486779E+06, 3.533617E+06, 3.580945E+06, 3.628768E+06, 
   3.677089E+06, 3.725911E+06, 3.775239E+06, 3.825077E+06, 3.875427E+06, 3.926295E+06, 
   3.977683E+06, 4.029596E+06, 4.082037E+06, 4.135011E+06, 4.188521E+06, 4.242571E+06, 
   4.297165E+06, 4.352308E+06, 4.408002E+06, 4.464253E+06, 4.521063E+06, 4.578438E+06, 
   4.636381E+06, 4.694896E+06, 4.753987E+06, 4.813658E+06, 4.873915E+06, 4.934759E+06, 
   4.996197E+06, 5.058231E+06, 5.120867E+06, 5.184108E+06, 5.247958E+06, 5.312423E+06, 
   5.377505E+06, 5.443210E+06, 5.509541E+06, 5.576504E+06, 5.644101E+06, 5.712338E+06, 
   5.781220E+06, 5.850749E+06, 5.920932E+06, 5.991772E+06, 6.063273E+06, 6.135441E+06, 
   6.208279E+06, 6.281792E+06, 6.355986E+06, 6.430863E+06, 6.506429E+06, 6.582688E+06, 
   6.659645E+06, 6.737305E+06, 6.815672E+06, 6.894751E+06, 6.974546E+06, 7.055063E+06, 
   7.136305E+06, 7.218278E+06, 7.300986E+06, 7.384435E+06, 7.468628E+06, 7.553571E+06, 
   7.639269E+06, 7.725726E+06, 7.812947E+06, 7.900937E+06, 7.989702E+06, 8.079245E+06, 
   8.169572E+06, 8.260688E+06, 8.352598E+06, 8.445307E+06, 8.538819E+06, 8.633140E+06, 
   8.728276E+06, 8.824230E+06, 8.921008E+06, 9.018616E+06, 9.117057E+06, 9.216339E+06, 
   9.316465E+06, 9.417440E+06, 9.519271E+06, 9.621962E+06, 9.725519E+06, 9.829946E+06, 
   9.935250E+06, 1.004144E+07, 1.014851E+07, 1.025647E+07, 1.036533E+07, 1.047510E+07, 
   1.058577E+07, 1.069735E+07, 1.080986E+07, 1.092329E+07, 1.103765E+07, 1.115294E+07, 
   1.126918E+07, 1.138636E+07, 1.150450E+07, 1.162360E+07, 1.174365E+07, 1.186468E+07, 
   1.198668E+07, 1.210966E+07, 1.223363E+07, 1.235859E+07, 1.248454E+07, 1.261150E+07, 
   1.273948E+07, 1.286846E+07, 1.299846E+07, 1.312950E+07, 1.326156E+07, 1.339466E+07, 
   1.352881E+07, 1.366401E+07, 1.380026E+07, 1.393758E+07, 1.407596E+07, 1.421542E+07, 
   1.435595E+07, 1.449758E+07, 1.464029E+07, 1.478410E+07, 1.492902E+07, 1.507505E+07, 
   1.522220E+07, 1.537047E+07, 1.551986E+07, 1.567040E+07, 1.582207E+07, 1.597489E+07, 
   1.612887E+07, 1.628400E+07, 1.644031E+07, 1.659778E+07, 1.675644E+07, 1.691628E+07, 
   1.707731E+07, 1.723954E+07, 1.740298E+07, 1.756763E+07, 1.773350E+07, 1.790059E+07, 
   1.806891E+07, 1.823847E+07, 1.840927E+07, 1.858133E+07, 1.875464E+07, 1.892922E+07, 
   1.910506E+07, 1.928219E+07, 1.946060E+07, 1.964030E+07, 1.982129E+07, 2.000360E+07, 
   2.018721E+07, 2.037214E+07, 2.055840E+07, 2.074598E+07, 2.093491E+07, 2.112518E+07, 
   2.131680E+07, 2.150979E+07, 2.170414E+07, 2.189986E+07, 2.209697E+07, 2.229546E+07, 
   2.249534E+07, 2.269663E+07, 2.289933E+07, 2.310344E+07, 2.330898E+07, 2.351594E+07, 
   2.372435E+07, 2.393420E+07, 2.414550E+07, 2.435825E+07, 2.457248E+07, 2.478818E+07, 
   2.500536E+07, 2.522403E+07, 2.544419E+07, 2.566586E+07, 2.588904E+07, 2.611374E+07, 
   2.633996E+07, 2.656772E+07, 2.679701E+07, 2.702786E+07, 2.726026E+07, 2.749422E+07, 
   2.772976E+07, 2.796687E+07, 2.820557E+07, 2.844586E+07, 2.868776E+07, 2.893126E+07, 
   2.917638E+07, 2.942313E+07, 2.967151E+07, 2.992153E+07, 3.017320E+07, 3.042652E+07, 
   3.068151E+07, 3.093817E+07, 3.119651E+07, 3.145654E+07, 3.171826E+07, 3.198169E+07, 
   3.224683E+07, 3.251369E+07, 3.278228E+07, 3.305260E+07, 3.332467E+07, 3.359849E+07, 
   3.387407E+07, 3.415142E+07, 3.443055E+07, 3.471146E+07, 3.499417E+07, 3.527868E+07, 
   3.556500E+07, 3.585314E+07, 3.614310E+07, 3.643490E+07, 3.672854E+07, 3.702404E+07, 
   3.732139E+07, 3.762062E+07, 3.792172E+07, 3.822471E+07, 3.852959E+07, 3.883638E+07, 
   3.914508E+07, 3.945570E+07, 3.976825E+07, 4.008273E+07, 4.039916E+07, 4.071755E+07, 
   4.103790E+07, 4.136023E+07, 4.168454E+07, 
])
# ============================ M = 22, I = 1 ============================
M = 22
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.029370E+00, 1.732193E+01, 3.298562E+01, 4.869230E+01, 6.440722E+01, 8.012577E+01, 
   9.584644E+01, 1.115686E+02, 1.272919E+02, 1.430162E+02, 1.587414E+02, 1.744673E+02, 
   1.901941E+02, 2.059216E+02, 2.216498E+02, 2.373787E+02, 2.531083E+02, 2.688385E+02, 
   2.845695E+02, 3.003011E+02, 3.160334E+02, 3.317664E+02, 3.475000E+02, 3.632343E+02, 
   3.789694E+02, 3.947052E+02, 4.104418E+02, 4.261794E+02, 4.419180E+02, 4.576577E+02, 
   4.733988E+02, 4.891415E+02, 5.048860E+02, 5.206327E+02, 5.363820E+02, 5.521343E+02, 
   5.678901E+02, 5.836501E+02, 5.994148E+02, 6.151849E+02, 6.309613E+02, 6.467448E+02, 
   6.625362E+02, 6.783365E+02, 6.941467E+02, 7.099679E+02, 7.258012E+02, 7.416477E+02, 
   7.575086E+02, 7.733853E+02, 7.892788E+02, 8.051907E+02, 8.211221E+02, 8.370745E+02, 
   8.530492E+02, 8.690475E+02, 8.850710E+02, 9.011209E+02, 9.171988E+02, 9.333060E+02, 
   9.494439E+02, 9.656139E+02, 9.818175E+02, 9.980560E+02, 1.014331E+03, 1.030643E+03, 
   1.046995E+03, 1.063387E+03, 1.079820E+03, 1.096297E+03, 1.112818E+03, 1.129385E+03, 
   1.145998E+03, 1.162659E+03, 1.179370E+03, 1.196131E+03, 1.212943E+03, 1.229809E+03, 
   1.246728E+03, 1.263702E+03, 1.280732E+03, 1.297819E+03, 1.314964E+03, 1.332168E+03, 
   1.349432E+03, 1.366757E+03, 1.384144E+03, 1.401593E+03, 1.419107E+03, 1.436684E+03, 
   1.454327E+03, 1.472037E+03, 1.489813E+03, 1.507657E+03, 1.525569E+03, 1.543550E+03, 
   1.561602E+03, 1.579724E+03, 1.597917E+03, 1.616183E+03, 1.634520E+03, 1.652932E+03, 
   1.671416E+03, 1.689976E+03, 1.708610E+03, 1.727320E+03, 1.746106E+03, 1.764969E+03, 
   1.783909E+03, 1.802926E+03, 1.822022E+03, 1.841196E+03, 1.860449E+03, 1.879782E+03, 
   1.899195E+03, 1.918688E+03, 1.938262E+03, 1.957917E+03, 1.977654E+03, 1.997473E+03, 
   2.017374E+03, 2.037358E+03, 2.057425E+03, 2.077575E+03, 2.097810E+03, 2.118128E+03, 
   2.138531E+03, 2.159018E+03, 2.179590E+03, 2.200248E+03, 2.220991E+03, 2.241821E+03, 
   2.262736E+03, 2.283738E+03, 2.304826E+03, 2.326001E+03, 2.347264E+03, 2.368614E+03, 
   2.390051E+03, 2.411576E+03, 2.433189E+03, 2.454891E+03, 2.476681E+03, 2.498560E+03, 
   2.520527E+03, 2.542584E+03, 2.564729E+03, 2.586964E+03, 2.609289E+03, 2.631704E+03, 
   2.654208E+03, 2.676803E+03, 2.699487E+03, 2.722262E+03, 2.745128E+03, 2.768084E+03, 
   2.791131E+03, 2.814269E+03, 2.837498E+03, 2.860818E+03, 2.884230E+03, 2.907732E+03, 
   2.931327E+03, 2.955013E+03, 2.978791E+03, 3.002660E+03, 3.026622E+03, 3.050676E+03, 
   3.074822E+03, 3.099060E+03, 3.123390E+03, 3.147813E+03, 3.172328E+03, 3.196936E+03, 
   3.221636E+03, 3.246429E+03, 3.271315E+03, 3.296294E+03, 3.321366E+03, 3.346530E+03, 
   3.371788E+03, 3.397139E+03, 3.422582E+03, 3.448119E+03, 3.473749E+03, 3.499473E+03, 
   3.525290E+03, 3.551200E+03, 3.577203E+03, 3.603300E+03, 3.629490E+03, 3.655774E+03, 
   3.682151E+03, 3.708622E+03, 3.735186E+03, 3.761844E+03, 3.788596E+03, 3.815441E+03, 
   3.842379E+03, 3.869412E+03, 3.896538E+03, 3.923757E+03, 3.951070E+03, 3.978477E+03, 
   4.005977E+03, 4.033571E+03, 4.061259E+03, 4.089040E+03, 4.116915E+03, 4.144884E+03, 
   4.172946E+03, 4.201102E+03, 4.229351E+03, 4.257694E+03, 4.286130E+03, 4.314660E+03, 
   4.343283E+03, 4.372000E+03, 4.400810E+03, 4.429713E+03, 4.458710E+03, 4.487801E+03, 
   4.516984E+03, 4.546261E+03, 4.575631E+03, 4.605095E+03, 4.634651E+03, 4.664301E+03, 
   4.694043E+03, 4.723879E+03, 4.753807E+03, 4.783829E+03, 4.813944E+03, 4.844151E+03, 
   4.874451E+03, 4.904844E+03, 4.935329E+03, 4.965907E+03, 4.996578E+03, 5.027341E+03, 
   5.058196E+03, 5.089144E+03, 5.120184E+03, 5.151317E+03, 5.182541E+03, 5.213858E+03, 
   5.245267E+03, 5.276767E+03, 5.308360E+03, 5.340044E+03, 5.371821E+03, 5.403688E+03, 
   5.435648E+03, 5.467699E+03, 5.499841E+03, 5.532074E+03, 5.564399E+03, 5.596816E+03, 
   5.629323E+03, 5.661921E+03, 5.694610E+03, 5.727390E+03, 5.760261E+03, 5.793222E+03, 
   5.826274E+03, 5.859416E+03, 5.892649E+03, 5.925972E+03, 5.959385E+03, 5.992889E+03, 
   6.026482E+03, 6.060165E+03, 6.093938E+03, 6.127801E+03, 6.161753E+03, 6.195795E+03, 
   6.229926E+03, 6.264147E+03, 6.298457E+03, 6.332855E+03, 6.367343E+03, 6.401920E+03, 
   6.436585E+03, 6.471339E+03, 6.506181E+03, 6.541112E+03, 6.576131E+03, 6.611239E+03, 
   6.646434E+03, 6.681718E+03, 6.717089E+03, 6.752548E+03, 6.788095E+03, 6.823729E+03, 
   6.859450E+03, 6.895259E+03, 6.931155E+03, 6.967138E+03, 7.003207E+03, 7.039364E+03, 
   7.075607E+03, 7.111937E+03, 7.148353E+03, 7.184856E+03, 7.221444E+03, 7.258119E+03, 
   7.294879E+03, 7.331725E+03, 7.368657E+03, 7.405675E+03, 7.442778E+03, 7.479966E+03, 
   7.517239E+03, 7.554597E+03, 7.592040E+03, 7.629568E+03, 7.667180E+03, 7.704877E+03, 
   7.742658E+03, 7.780523E+03, 7.818473E+03, 7.856506E+03, 7.894623E+03, 7.932824E+03, 
   7.971108E+03, 8.009476E+03, 8.047926E+03, 8.086460E+03, 8.125077E+03, 8.163777E+03, 
   8.202560E+03, 8.241425E+03, 8.280372E+03, 8.319401E+03, 8.358513E+03, 8.397707E+03, 
   8.436983E+03, 8.476340E+03, 8.515779E+03, 8.555299E+03, 8.594900E+03, 8.634583E+03, 
   8.674346E+03, 8.714191E+03, 8.754116E+03, 8.794121E+03, 8.834207E+03, 8.874374E+03, 
   8.914620E+03, 8.954946E+03, 8.995352E+03, 9.035838E+03, 9.076403E+03, 9.117048E+03, 
   9.157772E+03, 9.198575E+03, 9.239456E+03, 9.280417E+03, 9.321456E+03, 9.362574E+03, 
   9.403770E+03, 9.445044E+03, 9.486396E+03, 9.527826E+03, 9.569334E+03, 9.610919E+03, 
   9.652582E+03, 9.694321E+03, 9.736138E+03, 9.778032E+03, 9.820003E+03, 9.862051E+03, 
   9.904175E+03, 9.946375E+03, 9.988651E+03, 1.003100E+04, 1.007343E+04, 1.011594E+04, 
   1.015852E+04, 1.020117E+04, 1.024390E+04, 1.028671E+04, 1.032959E+04, 1.037254E+04, 
   1.041557E+04, 1.045868E+04, 1.050186E+04, 1.054511E+04, 1.058844E+04, 1.063184E+04, 
   1.067531E+04, 1.071886E+04, 1.076248E+04, 1.080618E+04, 1.084994E+04, 1.089378E+04, 
   1.093770E+04, 1.098168E+04, 1.102574E+04, 1.106987E+04, 1.111408E+04, 1.115835E+04, 
   1.120270E+04, 1.124712E+04, 1.129161E+04, 1.133617E+04, 1.138080E+04, 1.142551E+04, 
   1.147028E+04, 1.151513E+04, 1.156004E+04, 1.160503E+04, 1.165009E+04, 1.169521E+04, 
   1.174041E+04, 1.178568E+04, 1.183101E+04, 1.187642E+04, 1.192189E+04, 1.196744E+04, 
   1.201305E+04, 1.205873E+04, 1.210448E+04, 1.215030E+04, 1.219619E+04, 1.224215E+04, 
   1.228817E+04, 1.233426E+04, 1.238042E+04, 1.242665E+04, 1.247295E+04, 1.251931E+04, 
   1.256574E+04, 1.261223E+04, 1.265880E+04, 1.270543E+04, 1.275212E+04, 1.279888E+04, 
   1.284571E+04, 1.289261E+04, 1.293957E+04, 1.298659E+04, 1.303369E+04, 1.308084E+04, 
   1.312807E+04, 1.317535E+04, 1.322271E+04, 1.327012E+04, 1.331760E+04, 1.336515E+04, 
   1.341276E+04, 1.346044E+04, 1.350817E+04, 1.355598E+04, 1.360384E+04, 1.365177E+04, 
   1.369976E+04, 1.374782E+04, 1.379594E+04, 1.384412E+04, 1.389237E+04, 1.394067E+04, 
   1.398904E+04, 1.403748E+04, 1.408597E+04, 1.413453E+04, 1.418314E+04, 1.423182E+04, 
   1.428056E+04, 1.432937E+04, 1.437823E+04, 1.442715E+04, 1.447614E+04, 1.452518E+04, 
   1.457429E+04, 1.462346E+04, 1.467269E+04, 1.472197E+04, 1.477132E+04, 1.482073E+04, 
   1.487019E+04, 1.491972E+04, 1.496930E+04, 1.501895E+04, 1.506865E+04, 1.511841E+04, 
   1.516823E+04, 1.521811E+04, 1.526805E+04, 1.531805E+04, 1.536810E+04, 1.541821E+04, 
   1.546838E+04, 1.551861E+04, 1.556890E+04, 1.561924E+04, 1.566964E+04, 1.572009E+04, 
   1.577061E+04, 1.582118E+04, 1.587180E+04, 1.592249E+04, 1.597322E+04, 1.602402E+04, 
   1.607487E+04, 1.612578E+04, 1.617674E+04, 1.622776E+04, 1.627883E+04, 1.632996E+04, 
   1.638114E+04, 1.643238E+04, 1.648367E+04, 1.653502E+04, 1.658642E+04, 1.663788E+04, 
   1.668939E+04, 1.674095E+04, 1.679257E+04, 1.684424E+04, 1.689597E+04, 1.694775E+04, 
   1.699958E+04, 1.705147E+04, 1.710340E+04, 1.715539E+04, 1.720744E+04, 1.725953E+04, 
   1.731168E+04, 1.736388E+04, 1.741613E+04, 1.746844E+04, 1.752079E+04, 1.757320E+04, 
   1.762566E+04, 1.767817E+04, 1.773073E+04, 1.778335E+04, 1.783601E+04, 1.788872E+04, 
   1.794149E+04, 1.799430E+04, 1.804717E+04, 1.810009E+04, 1.815305E+04, 1.820607E+04, 
   1.825913E+04, 1.831225E+04, 1.836541E+04, 1.841862E+04, 1.847189E+04, 1.852520E+04, 
   1.857856E+04, 1.863197E+04, 1.868542E+04, 1.873893E+04, 1.879248E+04, 1.884609E+04, 
   1.889974E+04, 1.895343E+04, 1.900718E+04, 1.906097E+04, 1.911481E+04, 1.916870E+04, 
   1.922264E+04, 1.927662E+04, 1.933065E+04, 1.938472E+04, 1.943885E+04, 1.949302E+04, 
   1.954723E+04, 1.960149E+04, 1.965580E+04, 1.971015E+04, 1.976455E+04, 1.981899E+04, 
   1.987348E+04, 1.992802E+04, 1.998260E+04, 2.003722E+04, 2.009189E+04, 2.014661E+04, 
   2.020137E+04, 2.025617E+04, 2.031102E+04, 2.036591E+04, 2.042085E+04, 2.047583E+04, 
   2.053085E+04, 2.058592E+04, 2.064103E+04, 2.069619E+04, 2.075138E+04, 2.080662E+04, 
   2.086191E+04, 2.091723E+04, 2.097260E+04, 2.102802E+04, 2.108347E+04, 2.113897E+04, 
   2.119450E+04, 2.125009E+04, 2.130571E+04, 2.136137E+04, 2.141708E+04, 2.147283E+04, 
   2.152862E+04, 2.158445E+04, 2.164032E+04, 2.169623E+04, 2.175218E+04, 2.180818E+04, 
   2.186421E+04, 2.192028E+04, 2.197640E+04, 2.203255E+04, 2.208875E+04, 2.214498E+04, 
   2.220126E+04, 2.225757E+04, 2.231393E+04, 2.237032E+04, 2.242675E+04, 2.248322E+04, 
   2.253974E+04, 2.259629E+04, 2.265287E+04, 2.270950E+04, 2.276617E+04, 2.282287E+04, 
   2.287961E+04, 2.293639E+04, 2.299321E+04, 2.305007E+04, 2.310696E+04, 2.316389E+04, 
   2.322086E+04, 2.327786E+04, 2.333491E+04, 2.339199E+04, 2.344911E+04, 2.350626E+04, 
   2.356345E+04, 2.362068E+04, 2.367794E+04, 2.373524E+04, 2.379258E+04, 2.384995E+04, 
   2.390736E+04, 2.396480E+04, 2.402228E+04, 2.407980E+04, 2.413735E+04, 2.419494E+04, 
   2.425256E+04, 2.431021E+04, 2.436791E+04, 2.442563E+04, 2.448339E+04, 2.454119E+04, 
   2.459902E+04, 2.465689E+04, 2.471478E+04, 2.477272E+04, 2.483069E+04, 2.488869E+04, 
   2.494672E+04, 2.500479E+04, 2.506289E+04, 2.512103E+04, 2.517920E+04, 2.523740E+04, 
   2.529563E+04, 2.535390E+04, 2.541220E+04, 2.547054E+04, 2.552890E+04, 2.558730E+04, 
   2.564573E+04, 2.570420E+04, 2.576269E+04, 2.582122E+04, 2.587978E+04, 2.593837E+04, 
   2.599699E+04, 2.605565E+04, 2.611433E+04, 2.617305E+04, 2.623180E+04, 2.629058E+04, 
   2.634939E+04, 2.640823E+04, 2.646710E+04, 2.652600E+04, 2.658493E+04, 2.664390E+04, 
   2.670289E+04, 2.676192E+04, 2.682097E+04, 2.688005E+04, 2.693917E+04, 2.699831E+04, 
   2.705748E+04, 2.711669E+04, 2.717592E+04, 2.723518E+04, 2.729447E+04, 2.735379E+04, 
   2.741314E+04, 2.747252E+04, 2.753192E+04, 2.759136E+04, 2.765082E+04, 2.771031E+04, 
   2.776984E+04, 2.782938E+04, 2.788896E+04, 2.794856E+04, 2.800820E+04, 2.806786E+04, 
   2.812754E+04, 2.818726E+04, 2.824700E+04, 2.830677E+04, 2.836657E+04, 2.842640E+04, 
   2.848625E+04, 2.854613E+04, 2.860603E+04, 2.866596E+04, 2.872592E+04, 2.878591E+04, 
   2.884592E+04, 2.890596E+04, 2.896602E+04, 2.902611E+04, 2.908623E+04, 2.914637E+04, 
   2.920654E+04, 2.926673E+04, 2.932695E+04, 2.938719E+04, 2.944746E+04, 2.950776E+04, 
   2.956808E+04, 2.962843E+04, 2.968880E+04, 2.974919E+04, 2.980961E+04, 2.987006E+04, 
   2.993053E+04, 2.999102E+04, 3.005154E+04, 3.011208E+04, 3.017265E+04, 3.023324E+04, 
   3.029385E+04, 3.035449E+04, 3.041515E+04, 3.047584E+04, 3.053655E+04, 3.059728E+04, 
   3.065804E+04, 3.071882E+04, 3.077962E+04, 3.084044E+04, 3.090129E+04, 3.096217E+04, 
   3.102306E+04, 3.108398E+04, 3.114492E+04, 3.120588E+04, 3.126686E+04, 3.132787E+04, 
   3.138890E+04, 3.144995E+04, 3.151103E+04, 3.157212E+04, 3.163324E+04, 3.169438E+04, 
   3.175554E+04, 3.181672E+04, 3.187793E+04, 3.193915E+04, 3.200040E+04, 3.206167E+04, 
   3.212295E+04, 3.218426E+04, 3.224560E+04, 3.230695E+04, 3.236832E+04, 3.242971E+04, 
   3.249113E+04, 3.255256E+04, 3.261402E+04, 3.267549E+04, 3.273698E+04, 3.279850E+04, 
   3.286004E+04, 3.292159E+04, 3.298317E+04, 3.304476E+04, 3.310637E+04, 3.316801E+04, 
   3.322966E+04, 3.329133E+04, 3.335303E+04, 3.341474E+04, 3.347647E+04, 3.353822E+04, 
   3.359999E+04, 3.366177E+04, 3.372358E+04, 3.378541E+04, 3.384725E+04, 3.390911E+04, 
   3.397099E+04, 3.403289E+04, 3.409481E+04, 3.415674E+04, 3.421870E+04, 3.428067E+04, 
   3.434266E+04, 3.440467E+04, 3.446669E+04, 3.452873E+04, 3.459079E+04, 3.465287E+04, 
   3.471497E+04, 3.477708E+04, 3.483921E+04, 3.490136E+04, 3.496352E+04, 3.502570E+04, 
   3.508790E+04, 3.515012E+04, 3.521235E+04, 3.527460E+04, 3.533686E+04, 3.539915E+04, 
   3.546144E+04, 3.552376E+04, 3.558609E+04, 3.564844E+04, 3.571080E+04, 3.577318E+04, 
   3.583558E+04, 3.589799E+04, 3.596041E+04, 3.602286E+04, 3.608532E+04, 3.614779E+04, 
   3.621028E+04, 3.627278E+04, 3.633531E+04, 3.639784E+04, 3.646039E+04, 3.652296E+04, 
   3.658554E+04, 3.664814E+04, 3.671075E+04, 3.677337E+04, 3.683601E+04, 3.689867E+04, 
   3.696134E+04, 3.702402E+04, 3.708672E+04, 3.714943E+04, 3.721216E+04, 3.727490E+04, 
   3.733766E+04, 3.740043E+04, 3.746321E+04, 3.752601E+04, 3.758882E+04, 3.765165E+04, 
   3.771449E+04, 3.777734E+04, 3.784021E+04, 3.790309E+04, 3.796598E+04, 3.802889E+04, 
   3.809181E+04, 3.815474E+04, 3.821769E+04, 3.828064E+04, 3.834362E+04, 3.840660E+04, 
   3.846960E+04, 3.853261E+04, 3.859563E+04, 3.865867E+04, 3.872172E+04, 3.878478E+04, 
   3.884785E+04, 3.891094E+04, 3.897404E+04, 3.903715E+04, 3.910027E+04, 3.916340E+04, 
   3.922655E+04, 
])
# ============================ M = 22, I = 2 ============================
M = 22
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.071020E+00, 2.379651E+01, 4.541678E+01, 6.707875E+01, 8.875137E+01, 1.104287E+02, 
   1.321089E+02, 1.537910E+02, 1.754746E+02, 1.971596E+02, 2.188458E+02, 2.405330E+02, 
   2.622213E+02, 2.839107E+02, 3.056010E+02, 3.272922E+02, 3.489845E+02, 3.706776E+02, 
   3.923717E+02, 4.140667E+02, 4.357626E+02, 4.574594E+02, 4.791572E+02, 5.008560E+02, 
   5.225558E+02, 5.442566E+02, 5.659587E+02, 5.876621E+02, 6.093669E+02, 6.310735E+02, 
   6.527822E+02, 6.744932E+02, 6.962070E+02, 7.179242E+02, 7.396453E+02, 7.613710E+02, 
   7.831022E+02, 8.048396E+02, 8.265842E+02, 8.483372E+02, 8.700997E+02, 8.918729E+02, 
   9.136581E+02, 9.354568E+02, 9.572704E+02, 9.791005E+02, 1.000949E+03, 1.022817E+03, 
   1.044707E+03, 1.066620E+03, 1.088558E+03, 1.110524E+03, 1.132518E+03, 1.154544E+03, 
   1.176603E+03, 1.198696E+03, 1.220827E+03, 1.242997E+03, 1.265207E+03, 1.287461E+03, 
   1.309760E+03, 1.332105E+03, 1.354500E+03, 1.376946E+03, 1.399444E+03, 1.421998E+03, 
   1.444608E+03, 1.467277E+03, 1.490006E+03, 1.512798E+03, 1.535654E+03, 1.558576E+03, 
   1.581565E+03, 1.604624E+03, 1.627754E+03, 1.650957E+03, 1.674233E+03, 1.697586E+03, 
   1.721016E+03, 1.744525E+03, 1.768115E+03, 1.791786E+03, 1.815541E+03, 1.839380E+03, 
   1.863305E+03, 1.887317E+03, 1.911418E+03, 1.935608E+03, 1.959890E+03, 1.984263E+03, 
   2.008730E+03, 2.033291E+03, 2.057948E+03, 2.082701E+03, 2.107552E+03, 2.132501E+03, 
   2.157550E+03, 2.182699E+03, 2.207950E+03, 2.233303E+03, 2.258760E+03, 2.284320E+03, 
   2.309985E+03, 2.335756E+03, 2.361634E+03, 2.387618E+03, 2.413711E+03, 2.439912E+03, 
   2.466223E+03, 2.492644E+03, 2.519175E+03, 2.545818E+03, 2.572573E+03, 2.599440E+03, 
   2.626421E+03, 2.653515E+03, 2.680724E+03, 2.708048E+03, 2.735487E+03, 2.763042E+03, 
   2.790713E+03, 2.818502E+03, 2.846407E+03, 2.874431E+03, 2.902573E+03, 2.930834E+03, 
   2.959214E+03, 2.987713E+03, 3.016333E+03, 3.045072E+03, 3.073933E+03, 3.102915E+03, 
   3.132018E+03, 3.161243E+03, 3.190590E+03, 3.220060E+03, 3.249653E+03, 3.279369E+03, 
   3.309208E+03, 3.339171E+03, 3.369258E+03, 3.399469E+03, 3.429805E+03, 3.460266E+03, 
   3.490852E+03, 3.521563E+03, 3.552400E+03, 3.583362E+03, 3.614451E+03, 3.645666E+03, 
   3.677007E+03, 3.708475E+03, 3.740070E+03, 3.771793E+03, 3.803642E+03, 3.835619E+03, 
   3.867723E+03, 3.899956E+03, 3.932316E+03, 3.964804E+03, 3.997421E+03, 4.030166E+03, 
   4.063040E+03, 4.096042E+03, 4.129174E+03, 4.162434E+03, 4.195823E+03, 4.229342E+03, 
   4.262990E+03, 4.296768E+03, 4.330675E+03, 4.364711E+03, 4.398878E+03, 4.433174E+03, 
   4.467600E+03, 4.502157E+03, 4.536843E+03, 4.571660E+03, 4.606606E+03, 4.641684E+03, 
   4.676891E+03, 4.712229E+03, 4.747698E+03, 4.783297E+03, 4.819026E+03, 4.854887E+03, 
   4.890878E+03, 4.927000E+03, 4.963252E+03, 4.999636E+03, 5.036150E+03, 5.072795E+03, 
   5.109571E+03, 5.146477E+03, 5.183515E+03, 5.220684E+03, 5.257983E+03, 5.295414E+03, 
   5.332975E+03, 5.370667E+03, 5.408491E+03, 5.446445E+03, 5.484530E+03, 5.522746E+03, 
   5.561093E+03, 5.599571E+03, 5.638179E+03, 5.676919E+03, 5.715789E+03, 5.754790E+03, 
   5.793921E+03, 5.833184E+03, 5.872577E+03, 5.912100E+03, 5.951754E+03, 5.991539E+03, 
   6.031454E+03, 6.071500E+03, 6.111676E+03, 6.151982E+03, 6.192419E+03, 6.232985E+03, 
   6.273682E+03, 6.314509E+03, 6.355466E+03, 6.396553E+03, 6.437769E+03, 6.479116E+03, 
   6.520592E+03, 6.562197E+03, 6.603933E+03, 6.645797E+03, 6.687791E+03, 6.729915E+03, 
   6.772167E+03, 6.814549E+03, 6.857060E+03, 6.899699E+03, 6.942467E+03, 6.985364E+03, 
   7.028390E+03, 7.071544E+03, 7.114827E+03, 7.158237E+03, 7.201776E+03, 7.245443E+03, 
   7.289238E+03, 7.333161E+03, 7.377212E+03, 7.421390E+03, 7.465695E+03, 7.510128E+03, 
   7.554688E+03, 7.599375E+03, 7.644190E+03, 7.689131E+03, 7.734198E+03, 7.779393E+03, 
   7.824713E+03, 7.870160E+03, 7.915734E+03, 7.961433E+03, 8.007258E+03, 8.053209E+03, 
   8.099285E+03, 8.145487E+03, 8.191814E+03, 8.238267E+03, 8.284844E+03, 8.331546E+03, 
   8.378373E+03, 8.425325E+03, 8.472401E+03, 8.519601E+03, 8.566925E+03, 8.614373E+03, 
   8.661945E+03, 8.709641E+03, 8.757460E+03, 8.805402E+03, 8.853467E+03, 8.901655E+03, 
   8.949967E+03, 8.998400E+03, 9.046956E+03, 9.095635E+03, 9.144435E+03, 9.193358E+03, 
   9.242402E+03, 9.291568E+03, 9.340855E+03, 9.390263E+03, 9.439793E+03, 9.489443E+03, 
   9.539214E+03, 9.589106E+03, 9.639117E+03, 9.689249E+03, 9.739501E+03, 9.789873E+03, 
   9.840364E+03, 9.890975E+03, 9.941705E+03, 9.992554E+03, 1.004352E+04, 1.009461E+04, 
   1.014581E+04, 1.019714E+04, 1.024858E+04, 1.030014E+04, 1.035181E+04, 1.040361E+04, 
   1.045552E+04, 1.050755E+04, 1.055969E+04, 1.061195E+04, 1.066433E+04, 1.071683E+04, 
   1.076944E+04, 1.082217E+04, 1.087501E+04, 1.092797E+04, 1.098104E+04, 1.103423E+04, 
   1.108753E+04, 1.114095E+04, 1.119448E+04, 1.124813E+04, 1.130189E+04, 1.135577E+04, 
   1.140975E+04, 1.146386E+04, 1.151807E+04, 1.157240E+04, 1.162684E+04, 1.168139E+04, 
   1.173606E+04, 1.179084E+04, 1.184573E+04, 1.190073E+04, 1.195584E+04, 1.201107E+04, 
   1.206641E+04, 1.212185E+04, 1.217741E+04, 1.223308E+04, 1.228886E+04, 1.234474E+04, 
   1.240074E+04, 1.245685E+04, 1.251307E+04, 1.256939E+04, 1.262583E+04, 1.268237E+04, 
   1.273903E+04, 1.279579E+04, 1.285266E+04, 1.290963E+04, 1.296672E+04, 1.302391E+04, 
   1.308121E+04, 1.313862E+04, 1.319613E+04, 1.325375E+04, 1.331148E+04, 1.336931E+04, 
   1.342725E+04, 1.348529E+04, 1.354344E+04, 1.360169E+04, 1.366005E+04, 1.371852E+04, 
   1.377709E+04, 1.383576E+04, 1.389454E+04, 1.395342E+04, 1.401240E+04, 1.407149E+04, 
   1.413069E+04, 1.418998E+04, 1.424938E+04, 1.430888E+04, 1.436848E+04, 1.442819E+04, 
   1.448799E+04, 1.454790E+04, 1.460791E+04, 1.466802E+04, 1.472823E+04, 1.478855E+04, 
   1.484896E+04, 1.490947E+04, 1.497009E+04, 1.503080E+04, 1.509161E+04, 1.515252E+04, 
   1.521354E+04, 1.527465E+04, 1.533586E+04, 1.539717E+04, 1.545857E+04, 1.552008E+04, 
   1.558168E+04, 1.564338E+04, 1.570518E+04, 1.576707E+04, 1.582906E+04, 1.589115E+04, 
   1.595333E+04, 1.601561E+04, 1.607799E+04, 1.614046E+04, 1.620303E+04, 1.626570E+04, 
   1.632846E+04, 1.639131E+04, 1.645426E+04, 1.651730E+04, 1.658044E+04, 1.664367E+04, 
   1.670699E+04, 1.677041E+04, 1.683392E+04, 1.689753E+04, 1.696123E+04, 1.702502E+04, 
   1.708890E+04, 1.715287E+04, 1.721694E+04, 1.728110E+04, 1.734535E+04, 1.740969E+04, 
   1.747412E+04, 1.753865E+04, 1.760326E+04, 1.766797E+04, 1.773276E+04, 1.779764E+04, 
   1.786262E+04, 1.792768E+04, 1.799284E+04, 1.805808E+04, 1.812341E+04, 1.818883E+04, 
   1.825434E+04, 1.831993E+04, 1.838562E+04, 1.845139E+04, 1.851725E+04, 1.858319E+04, 
   1.864923E+04, 1.871535E+04, 1.878155E+04, 1.884785E+04, 1.891423E+04, 1.898069E+04, 
   1.904724E+04, 1.911388E+04, 1.918060E+04, 1.924741E+04, 1.931430E+04, 1.938127E+04, 
   1.944833E+04, 1.951548E+04, 1.958271E+04, 1.965002E+04, 1.971742E+04, 1.978489E+04, 
   1.985246E+04, 1.992010E+04, 1.998783E+04, 2.005564E+04, 2.012353E+04, 2.019150E+04, 
   2.025956E+04, 2.032770E+04, 2.039591E+04, 2.046421E+04, 2.053259E+04, 2.060105E+04, 
   2.066960E+04, 2.073822E+04, 2.080692E+04, 2.087570E+04, 2.094456E+04, 2.101350E+04, 
   2.108252E+04, 2.115162E+04, 2.122079E+04, 2.129005E+04, 2.135938E+04, 2.142879E+04, 
   2.149828E+04, 2.156785E+04, 2.163749E+04, 2.170721E+04, 2.177701E+04, 2.184688E+04, 
   2.191683E+04, 2.198686E+04, 2.205696E+04, 2.212714E+04, 2.219740E+04, 2.226773E+04, 
   2.233813E+04, 2.240861E+04, 2.247917E+04, 2.254980E+04, 2.262050E+04, 2.269128E+04, 
   2.276213E+04, 2.283306E+04, 2.290406E+04, 2.297513E+04, 2.304628E+04, 2.311750E+04, 
   2.318879E+04, 2.326015E+04, 2.333159E+04, 2.340310E+04, 2.347468E+04, 2.354633E+04, 
   2.361805E+04, 2.368984E+04, 2.376171E+04, 2.383365E+04, 2.390565E+04, 2.397773E+04, 
   2.404987E+04, 2.412209E+04, 2.419438E+04, 2.426673E+04, 2.433916E+04, 2.441165E+04, 
   2.448422E+04, 2.455685E+04, 2.462955E+04, 2.470232E+04, 2.477516E+04, 2.484806E+04, 
   2.492103E+04, 2.499407E+04, 2.506718E+04, 2.514036E+04, 2.521360E+04, 2.528690E+04, 
   2.536028E+04, 2.543372E+04, 2.550723E+04, 2.558080E+04, 2.565444E+04, 2.572814E+04, 
   2.580191E+04, 2.587574E+04, 2.594964E+04, 2.602361E+04, 2.609763E+04, 2.617173E+04, 
   2.624588E+04, 2.632010E+04, 2.639439E+04, 2.646873E+04, 2.654314E+04, 2.661762E+04, 
   2.669215E+04, 2.676675E+04, 2.684141E+04, 2.691614E+04, 2.699092E+04, 2.706577E+04, 
   2.714068E+04, 2.721565E+04, 2.729068E+04, 2.736578E+04, 2.744093E+04, 2.751614E+04, 
   2.759142E+04, 2.766676E+04, 2.774215E+04, 2.781761E+04, 2.789312E+04, 2.796870E+04, 
   2.804433E+04, 2.812003E+04, 2.819578E+04, 2.827159E+04, 2.834746E+04, 2.842339E+04, 
   2.849937E+04, 2.857542E+04, 2.865152E+04, 2.872768E+04, 2.880390E+04, 2.888017E+04, 
   2.895650E+04, 2.903289E+04, 2.910934E+04, 2.918584E+04, 2.926240E+04, 2.933901E+04, 
   2.941568E+04, 2.949241E+04, 2.956919E+04, 2.964603E+04, 2.972292E+04, 2.979987E+04, 
   2.987687E+04, 2.995393E+04, 3.003104E+04, 3.010820E+04, 3.018542E+04, 3.026270E+04, 
   3.034002E+04, 3.041740E+04, 3.049484E+04, 3.057233E+04, 3.064987E+04, 3.072746E+04, 
   3.080511E+04, 3.088280E+04, 3.096055E+04, 3.103836E+04, 3.111621E+04, 3.119412E+04, 
   3.127208E+04, 3.135009E+04, 3.142815E+04, 3.150626E+04, 3.158442E+04, 3.166264E+04, 
   3.174090E+04, 3.181921E+04, 3.189758E+04, 3.197599E+04, 3.205446E+04, 3.213297E+04, 
   3.221153E+04, 3.229015E+04, 3.236881E+04, 3.244752E+04, 3.252628E+04, 3.260509E+04, 
   3.268394E+04, 3.276285E+04, 3.284180E+04, 3.292080E+04, 3.299985E+04, 3.307895E+04, 
   3.315809E+04, 3.323728E+04, 3.331652E+04, 3.339580E+04, 3.347514E+04, 3.355451E+04, 
   3.363394E+04, 3.371341E+04, 3.379293E+04, 3.387249E+04, 3.395210E+04, 3.403175E+04, 
   3.411145E+04, 3.419119E+04, 3.427098E+04, 3.435082E+04, 3.443070E+04, 3.451062E+04, 
   3.459059E+04, 3.467060E+04, 3.475066E+04, 3.483076E+04, 3.491090E+04, 3.499109E+04, 
   3.507132E+04, 3.515159E+04, 3.523191E+04, 3.531227E+04, 3.539268E+04, 3.547312E+04, 
   3.555361E+04, 3.563414E+04, 3.571471E+04, 3.579533E+04, 3.587598E+04, 3.595668E+04, 
   3.603742E+04, 3.611820E+04, 3.619902E+04, 3.627989E+04, 3.636079E+04, 3.644174E+04, 
   3.652272E+04, 3.660375E+04, 3.668481E+04, 3.676592E+04, 3.684706E+04, 3.692825E+04, 
   3.700947E+04, 3.709074E+04, 3.717204E+04, 3.725339E+04, 3.733477E+04, 3.741619E+04, 
   3.749765E+04, 3.757915E+04, 3.766069E+04, 3.774226E+04, 3.782387E+04, 3.790552E+04, 
   3.798721E+04, 3.806894E+04, 3.815070E+04, 3.823251E+04, 3.831434E+04, 3.839622E+04, 
   3.847813E+04, 3.856008E+04, 3.864207E+04, 3.872409E+04, 3.880615E+04, 3.888824E+04, 
   3.897037E+04, 3.905254E+04, 3.913474E+04, 3.921698E+04, 3.929926E+04, 3.938156E+04, 
   3.946391E+04, 3.954629E+04, 3.962870E+04, 3.971115E+04, 3.979363E+04, 3.987615E+04, 
   3.995870E+04, 4.004129E+04, 4.012391E+04, 4.020656E+04, 4.028925E+04, 4.037197E+04, 
   4.045473E+04, 4.053752E+04, 4.062034E+04, 4.070319E+04, 4.078608E+04, 4.086900E+04, 
   4.095195E+04, 4.103494E+04, 4.111796E+04, 4.120101E+04, 4.128409E+04, 4.136720E+04, 
   4.145035E+04, 4.153352E+04, 4.161673E+04, 4.169997E+04, 4.178324E+04, 4.186655E+04, 
   4.194988E+04, 4.203325E+04, 4.211664E+04, 4.220007E+04, 4.228352E+04, 4.236701E+04, 
   4.245053E+04, 4.253407E+04, 4.261765E+04, 4.270126E+04, 4.278489E+04, 4.286856E+04, 
   4.295225E+04, 4.303598E+04, 4.311973E+04, 4.320351E+04, 4.328733E+04, 4.337117E+04, 
   4.345503E+04, 4.353893E+04, 4.362286E+04, 4.370681E+04, 4.379079E+04, 4.387480E+04, 
   4.395884E+04, 4.404291E+04, 4.412700E+04, 4.421112E+04, 4.429527E+04, 4.437945E+04, 
   4.446365E+04, 4.454788E+04, 4.463213E+04, 4.471642E+04, 4.480073E+04, 4.488506E+04, 
   4.496943E+04, 4.505381E+04, 4.513823E+04, 4.522267E+04, 4.530714E+04, 4.539163E+04, 
   4.547615E+04, 4.556069E+04, 4.564526E+04, 4.572986E+04, 4.581448E+04, 4.589912E+04, 
   4.598379E+04, 4.606849E+04, 4.615320E+04, 4.623795E+04, 4.632272E+04, 4.640751E+04, 
   4.649233E+04, 4.657717E+04, 4.666203E+04, 4.674692E+04, 4.683183E+04, 4.691677E+04, 
   4.700173E+04, 4.708671E+04, 4.717172E+04, 4.725675E+04, 4.734180E+04, 4.742688E+04, 
   4.751197E+04, 4.759709E+04, 4.768224E+04, 4.776740E+04, 4.785259E+04, 4.793780E+04, 
   4.802304E+04, 4.810829E+04, 4.819357E+04, 4.827887E+04, 4.836419E+04, 4.844953E+04, 
   4.853489E+04, 4.862028E+04, 4.870568E+04, 4.879111E+04, 4.887656E+04, 4.896203E+04, 
   4.904752E+04, 4.913303E+04, 4.921856E+04, 4.930411E+04, 4.938969E+04, 4.947528E+04, 
   4.956089E+04, 4.964653E+04, 4.973218E+04, 4.981785E+04, 4.990354E+04, 4.998926E+04, 
   5.007499E+04, 5.016074E+04, 5.024651E+04, 5.033230E+04, 5.041811E+04, 5.050394E+04, 
   5.058978E+04, 5.067565E+04, 5.076153E+04, 5.084744E+04, 5.093336E+04, 5.101930E+04, 
   5.110526E+04, 5.119123E+04, 5.127723E+04, 5.136324E+04, 5.144927E+04, 5.153532E+04, 
   5.162139E+04, 5.170747E+04, 5.179357E+04, 5.187969E+04, 5.196583E+04, 5.205198E+04, 
   5.213815E+04, 5.222434E+04, 5.231054E+04, 5.239676E+04, 5.248300E+04, 5.256926E+04, 
   5.265553E+04, 5.274182E+04, 5.282812E+04, 5.291444E+04, 5.300078E+04, 5.308713E+04, 
   5.317350E+04, 5.325988E+04, 5.334628E+04, 5.343270E+04, 5.351913E+04, 5.360557E+04, 
   5.369204E+04, 5.377851E+04, 5.386501E+04, 5.395151E+04, 5.403804E+04, 5.412457E+04, 
   5.421113E+04, 
])
# ============================ M = 22, I = 3 ============================
#M = 22
#I = 3
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[9]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.042930E+00, 8.183250E+00, 1.565169E+01, 2.312931E+01, 3.061038E+01, 3.809298E+01, 
#   4.557650E+01, 5.306067E+01, 6.054536E+01, 6.803050E+01, 7.551604E+01, 8.300195E+01, 
#   9.048823E+01, 9.797485E+01, 1.054618E+02, 1.129491E+02, 1.204367E+02, 1.279246E+02, 
#   1.354129E+02, 1.429015E+02, 1.503903E+02, 1.578795E+02, 1.653691E+02, 1.728590E+02, 
#   1.803492E+02, 1.878398E+02, 1.953309E+02, 2.028224E+02, 2.103145E+02, 2.178073E+02, 
#   2.253008E+02, 2.327952E+02, 2.402907E+02, 2.477875E+02, 2.552858E+02, 2.627859E+02, 
#   2.702880E+02, 2.777926E+02, 2.852999E+02, 2.928104E+02, 3.003246E+02, 3.078428E+02, 
#   3.153656E+02, 3.228934E+02, 3.304270E+02, 3.379668E+02, 3.455134E+02, 3.530674E+02, 
#   3.606296E+02, 3.682005E+02, 3.757809E+02, 3.833713E+02, 3.909726E+02, 3.985854E+02, 
#   4.062104E+02, 4.138483E+02, 4.214999E+02, 4.291659E+02, 4.368470E+02, 4.445439E+02, 
#   4.522573E+02, 4.599880E+02, 4.677366E+02, 4.755038E+02, 4.832903E+02, 4.910969E+02, 
#   4.989241E+02, 5.067727E+02, 5.146432E+02, 5.225364E+02, 5.304529E+02, 5.383932E+02, 
#   5.463581E+02, 5.543480E+02, 5.623636E+02, 5.704054E+02, 5.784740E+02, 5.865700E+02, 
#   5.946939E+02, 6.028461E+02, 6.110273E+02, 6.192378E+02, 6.274783E+02, 6.357491E+02, 
#   6.440507E+02, 6.523836E+02, 6.607482E+02, 6.691450E+02, 6.775743E+02, 6.860365E+02, 
#   6.945321E+02, 7.030614E+02, 7.116249E+02, 7.202227E+02, 7.288554E+02, 7.375233E+02, 
#   7.462266E+02, 7.549658E+02, 7.637410E+02, 7.725528E+02, 7.814012E+02, 7.902867E+02, 
#   7.992095E+02, 8.081698E+02, 8.171680E+02, 8.262043E+02, 8.352790E+02, 8.443922E+02, 
#   8.535443E+02, 8.627355E+02, 8.719660E+02, 8.812359E+02, 8.905456E+02, 8.998952E+02, 
#   9.092849E+02, 9.187150E+02, 9.281856E+02, 9.376968E+02, 9.472490E+02, 9.568422E+02, 
#   9.664766E+02, 9.761523E+02, 9.858697E+02, 9.956287E+02, 1.005430E+03, 1.015272E+03, 
#   1.025157E+03, 1.035085E+03, 1.045054E+03, 1.055067E+03, 1.065122E+03, 1.075219E+03, 
#   1.085360E+03, 1.095543E+03, 1.105770E+03, 1.116040E+03, 1.126354E+03, 1.136710E+03, 
#   1.147111E+03, 1.157555E+03, 1.168043E+03, 1.178574E+03, 1.189150E+03, 1.199769E+03, 
#   1.210433E+03, 1.221141E+03, 1.231893E+03, 1.242690E+03, 1.253530E+03, 1.264416E+03, 
#   1.275346E+03, 1.286320E+03, 1.297340E+03, 1.308404E+03, 1.319512E+03, 1.330666E+03, 
#   1.341865E+03, 1.353108E+03, 1.364397E+03, 1.375731E+03, 1.387110E+03, 1.398534E+03, 
#   1.410003E+03, 1.421517E+03, 1.433077E+03, 1.444682E+03, 1.456333E+03, 1.468029E+03, 
#   1.479771E+03, 1.491558E+03, 1.503390E+03, 1.515268E+03, 1.527192E+03, 1.539161E+03, 
#   1.551176E+03, 1.563237E+03, 1.575343E+03, 1.587495E+03, 1.599692E+03, 1.611936E+03, 
#   1.624225E+03, 1.636560E+03, 1.648941E+03, 1.661367E+03, 1.673840E+03, 1.686358E+03, 
#   1.698922E+03, 1.711532E+03, 1.724188E+03, 1.736889E+03, 1.749637E+03, 1.762430E+03, 
#   1.775269E+03, 1.788154E+03, 1.801085E+03, 1.814062E+03, 1.827085E+03, 1.840153E+03, 
#   1.853267E+03, 1.866428E+03, 1.879634E+03, 1.892885E+03, 1.906183E+03, 1.919527E+03, 
#   1.932916E+03, 1.946351E+03, 1.959832E+03, 1.973358E+03, 1.986931E+03, 2.000549E+03, 
#   2.014212E+03, 2.027922E+03, 2.041677E+03, 2.055478E+03, 2.069324E+03, 2.083216E+03, 
#   2.097154E+03, 2.111137E+03, 2.125166E+03, 2.139240E+03, 2.153359E+03, 2.167525E+03, 
#   2.181735E+03, 2.195991E+03, 2.210292E+03, 2.224639E+03, 2.239031E+03, 2.253468E+03, 
#   2.267951E+03, 2.282478E+03, 2.297051E+03, 2.311669E+03, 2.326332E+03, 2.341041E+03, 
#   2.355794E+03, 2.370592E+03, 2.385435E+03, 2.400323E+03, 2.415256E+03, 2.430234E+03, 
#   2.445256E+03, 2.460324E+03, 2.475436E+03, 2.490592E+03, 2.505794E+03, 2.521039E+03, 
#   2.536330E+03, 2.551665E+03, 2.567044E+03, 2.582467E+03, 2.597935E+03, 2.613448E+03, 
#   2.629004E+03, 2.644605E+03, 2.660250E+03, 2.675939E+03, 2.691671E+03, 2.707448E+03, 
#   2.723269E+03, 2.739134E+03, 2.755043E+03, 2.770995E+03, 2.786991E+03, 2.803031E+03, 
#   2.819114E+03, 2.835241E+03, 2.851411E+03, 2.867625E+03, 2.883882E+03, 2.900182E+03, 
#   2.916526E+03, 2.932913E+03, 2.949343E+03, 2.965816E+03, 2.982332E+03, 2.998891E+03, 
#   3.015493E+03, 3.032138E+03, 3.048826E+03, 3.065556E+03, 3.082329E+03, 3.099144E+03, 
#   3.116002E+03, 3.132903E+03, 3.149846E+03, 3.166831E+03, 3.183859E+03, 3.200928E+03, 
#   3.218040E+03, 3.235194E+03, 3.252390E+03, 3.269628E+03, 3.286907E+03, 3.304229E+03, 
#   3.321592E+03, 3.338997E+03, 3.356443E+03, 3.373931E+03, 3.391461E+03, 3.409031E+03, 
#   3.426643E+03, 3.444297E+03, 3.461991E+03, 3.479727E+03, 3.497504E+03, 3.515321E+03, 
#   3.533180E+03, 3.551079E+03, 3.569019E+03, 3.587000E+03, 3.605021E+03, 3.623083E+03, 
#   3.641186E+03, 3.659328E+03, 3.677511E+03, 3.695735E+03, 3.713998E+03, 3.732301E+03, 
#   3.750645E+03, 3.769028E+03, 3.787452E+03, 3.805915E+03, 3.824418E+03, 3.842960E+03, 
#   3.861542E+03, 3.880164E+03, 3.898825E+03, 3.917525E+03, 3.936265E+03, 3.955043E+03, 
#   3.973861E+03, 3.992718E+03, 4.011614E+03, 4.030549E+03, 4.049522E+03, 4.068534E+03, 
#   4.087585E+03, 4.106675E+03, 4.125803E+03, 4.144969E+03, 4.164174E+03, 4.183417E+03, 
#   4.202698E+03, 4.222017E+03, 4.241375E+03, 4.260770E+03, 4.280203E+03, 4.299674E+03, 
#   4.319182E+03, 4.338729E+03, 4.358312E+03, 4.377934E+03, 4.397592E+03, 4.417288E+03, 
#   4.437021E+03, 4.456792E+03, 4.476599E+03, 4.496444E+03, 4.516325E+03, 4.536243E+03, 
#   4.556198E+03, 4.576190E+03, 4.596218E+03, 4.616283E+03, 4.636384E+03, 4.656521E+03, 
#   4.676695E+03, 4.696905E+03, 4.717151E+03, 4.737433E+03, 4.757752E+03, 4.778106E+03, 
#   4.798495E+03, 4.818921E+03, 4.839382E+03, 4.859879E+03, 4.880411E+03, 4.900978E+03, 
#   4.921581E+03, 4.942219E+03, 4.962892E+03, 4.983601E+03, 5.004344E+03, 5.025122E+03, 
#   5.045935E+03, 5.066783E+03, 5.087665E+03, 5.108582E+03, 5.129534E+03, 5.150520E+03, 
#   5.171540E+03, 5.192595E+03, 5.213683E+03, 5.234806E+03, 5.255963E+03, 5.277154E+03, 
#   5.298378E+03, 5.319636E+03, 5.340928E+03, 5.362254E+03, 5.383613E+03, 5.405006E+03, 
#   5.426432E+03, 5.447891E+03, 5.469384E+03, 5.490909E+03, 5.512468E+03, 5.534059E+03, 
#   5.555684E+03, 5.577341E+03, 5.599031E+03, 5.620754E+03, 5.642509E+03, 5.664296E+03, 
#   5.686116E+03, 5.707969E+03, 5.729853E+03, 5.751770E+03, 5.773719E+03, 5.795699E+03, 
#   5.817712E+03, 5.839756E+03, 5.861833E+03, 5.883941E+03, 5.906080E+03, 5.928251E+03, 
#   5.950453E+03, 5.972687E+03, 5.994952E+03, 6.017248E+03, 6.039576E+03, 6.061934E+03, 
#   6.084323E+03, 6.106743E+03, 6.129194E+03, 6.151676E+03, 6.174188E+03, 6.196730E+03, 
#   6.219304E+03, 6.241907E+03, 6.264541E+03, 6.287205E+03, 6.309899E+03, 6.332623E+03, 
#   6.355378E+03, 6.378162E+03, 6.400976E+03, 6.423819E+03, 6.446693E+03, 6.469596E+03, 
#   6.492528E+03, 6.515490E+03, 6.538481E+03, 6.561502E+03, 6.584551E+03, 6.607630E+03, 
#   6.630738E+03, 6.653874E+03, 6.677040E+03, 6.700234E+03, 6.723457E+03, 6.746709E+03, 
#   6.769989E+03, 6.793298E+03, 6.816635E+03, 6.840000E+03, 6.863394E+03, 6.886816E+03, 
#   6.910266E+03, 6.933744E+03, 6.957250E+03, 6.980783E+03, 7.004345E+03, 7.027934E+03, 
#   7.051551E+03, 7.075195E+03, 7.098867E+03, 7.122566E+03, 7.146292E+03, 7.170046E+03, 
#   7.193827E+03, 7.217635E+03, 7.241469E+03, 7.265331E+03, 7.289220E+03, 7.313135E+03, 
#   7.337077E+03, 7.361046E+03, 7.385041E+03, 7.409063E+03, 7.433111E+03, 7.457185E+03, 
#   7.481286E+03, 7.505413E+03, 7.529565E+03, 7.553744E+03, 7.577949E+03, 7.602179E+03, 
#   7.626436E+03, 7.650718E+03, 7.675025E+03, 7.699359E+03, 7.723717E+03, 7.748101E+03, 
#   7.772511E+03, 7.796946E+03, 7.821405E+03, 7.845890E+03, 7.870400E+03, 7.894935E+03, 
#   7.919495E+03, 7.944079E+03, 7.968689E+03, 7.993322E+03, 8.017981E+03, 8.042664E+03, 
#   8.067371E+03, 8.092103E+03, 8.116859E+03, 8.141639E+03, 8.166444E+03, 8.191272E+03, 
#   8.216125E+03, 8.241001E+03, 8.265901E+03, 8.290825E+03, 8.315773E+03, 8.340744E+03, 
#   8.365739E+03, 8.390757E+03, 8.415798E+03, 8.440863E+03, 8.465952E+03, 8.491063E+03, 
#   8.516198E+03, 8.541355E+03, 8.566536E+03, 8.591739E+03, 8.616965E+03, 8.642214E+03, 
#   8.667486E+03, 8.692780E+03, 8.718097E+03, 8.743436E+03, 8.768798E+03, 8.794182E+03, 
#   8.819588E+03, 8.845017E+03, 8.870467E+03, 8.895940E+03, 8.921434E+03, 8.946951E+03, 
#   8.972489E+03, 8.998049E+03, 9.023631E+03, 9.049234E+03, 9.074859E+03, 9.100506E+03, 
#   9.126174E+03, 9.151863E+03, 9.177573E+03, 9.203305E+03, 9.229058E+03, 9.254832E+03, 
#   9.280626E+03, 9.306442E+03, 9.332279E+03, 9.358136E+03, 9.384014E+03, 9.409913E+03, 
#   9.435833E+03, 9.461773E+03, 9.487733E+03, 9.513714E+03, 9.539715E+03, 9.565736E+03, 
#   9.591778E+03, 9.617839E+03, 9.643921E+03, 9.670023E+03, 9.696144E+03, 9.722286E+03, 
#   9.748447E+03, 9.774628E+03, 9.800828E+03, 9.827048E+03, 9.853288E+03, 9.879547E+03, 
#   9.905826E+03, 9.932123E+03, 9.958440E+03, 9.984777E+03, 1.001113E+04, 1.003751E+04, 
#   1.006390E+04, 1.009031E+04, 1.011674E+04, 1.014319E+04, 1.016966E+04, 1.019615E+04, 
#   1.022266E+04, 1.024918E+04, 1.027572E+04, 1.030228E+04, 1.032886E+04, 1.035546E+04, 
#   1.038208E+04, 1.040871E+04, 1.043536E+04, 1.046204E+04, 1.048872E+04, 1.051543E+04, 
#   1.054215E+04, 1.056890E+04, 1.059566E+04, 1.062243E+04, 1.064923E+04, 1.067604E+04, 
#   1.070287E+04, 1.072972E+04, 1.075658E+04, 1.078346E+04, 1.081036E+04, 1.083728E+04, 
#   1.086421E+04, 1.089116E+04, 1.091813E+04, 1.094511E+04, 1.097212E+04, 1.099913E+04, 
#   1.102617E+04, 1.105322E+04, 1.108029E+04, 1.110737E+04, 1.113447E+04, 1.116159E+04, 
#   1.118872E+04, 1.121587E+04, 1.124304E+04, 1.127022E+04, 1.129742E+04, 1.132464E+04, 
#   1.135187E+04, 1.137911E+04, 1.140637E+04, 1.143365E+04, 1.146095E+04, 1.148826E+04, 
#   1.151558E+04, 1.154292E+04, 1.157028E+04, 1.159765E+04, 1.162504E+04, 1.165244E+04, 
#   1.167986E+04, 1.170729E+04, 1.173474E+04, 1.176220E+04, 1.178968E+04, 1.181718E+04, 
#   1.184468E+04, 1.187221E+04, 1.189975E+04, 1.192730E+04, 1.195487E+04, 1.198245E+04, 
#   1.201005E+04, 1.203766E+04, 1.206528E+04, 1.209292E+04, 1.212058E+04, 1.214825E+04, 
#   1.217593E+04, 1.220363E+04, 1.223134E+04, 1.225907E+04, 1.228681E+04, 1.231456E+04, 
#   1.234233E+04, 1.237011E+04, 1.239791E+04, 1.242572E+04, 1.245354E+04, 1.248138E+04, 
#   1.250923E+04, 1.253709E+04, 1.256497E+04, 1.259286E+04, 1.262077E+04, 1.264868E+04, 
#   1.267661E+04, 1.270456E+04, 1.273252E+04, 1.276049E+04, 1.278847E+04, 1.281647E+04, 
#   1.284448E+04, 1.287250E+04, 1.290054E+04, 1.292858E+04, 1.295665E+04, 1.298472E+04, 
#   1.301281E+04, 1.304091E+04, 1.306902E+04, 1.309714E+04, 1.312528E+04, 1.315343E+04, 
#   1.318159E+04, 1.320976E+04, 1.323795E+04, 1.326615E+04, 1.329436E+04, 1.332258E+04, 
#   1.335081E+04, 1.337906E+04, 1.340732E+04, 1.343559E+04, 1.346387E+04, 1.349217E+04, 
#   1.352047E+04, 1.354879E+04, 1.357712E+04, 1.360546E+04, 1.363381E+04, 1.366218E+04, 
#   1.369055E+04, 1.371894E+04, 1.374734E+04, 1.377574E+04, 1.380417E+04, 1.383260E+04, 
#   1.386104E+04, 1.388949E+04, 1.391796E+04, 1.394644E+04, 1.397492E+04, 1.400342E+04, 
#   1.403193E+04, 1.406045E+04, 1.408898E+04, 1.411752E+04, 1.414607E+04, 1.417464E+04, 
#   1.420321E+04, 1.423179E+04, 1.426039E+04, 1.428899E+04, 1.431761E+04, 1.434623E+04, 
#   1.437487E+04, 1.440351E+04, 1.443217E+04, 1.446084E+04, 1.448951E+04, 1.451820E+04, 
#   1.454690E+04, 1.457560E+04, 1.460432E+04, 1.463304E+04, 1.466178E+04, 1.469053E+04, 
#   1.471928E+04, 1.474805E+04, 1.477682E+04, 1.480561E+04, 1.483440E+04, 1.486321E+04, 
#   1.489202E+04, 1.492084E+04, 1.494968E+04, 1.497852E+04, 1.500737E+04, 1.503623E+04, 
#   1.506510E+04, 1.509398E+04, 1.512287E+04, 1.515176E+04, 1.518067E+04, 1.520958E+04, 
#   1.523851E+04, 1.526744E+04, 1.529638E+04, 1.532533E+04, 1.535429E+04, 1.538326E+04, 
#   1.541224E+04, 1.544122E+04, 1.547022E+04, 1.549922E+04, 1.552823E+04, 1.555725E+04, 
#   1.558628E+04, 1.561532E+04, 1.564436E+04, 1.567342E+04, 1.570248E+04, 1.573155E+04, 
#   1.576063E+04, 1.578971E+04, 1.581881E+04, 1.584791E+04, 1.587702E+04, 1.590614E+04, 
#   1.593527E+04, 1.596440E+04, 1.599354E+04, 1.602269E+04, 1.605185E+04, 1.608102E+04, 
#   1.611019E+04, 1.613937E+04, 1.616856E+04, 1.619776E+04, 1.622697E+04, 1.625618E+04, 
#   1.628540E+04, 1.631462E+04, 1.634386E+04, 1.637310E+04, 1.640235E+04, 1.643161E+04, 
#   1.646087E+04, 1.649014E+04, 1.651942E+04, 1.654870E+04, 1.657800E+04, 1.660729E+04, 
#   1.663660E+04, 1.666591E+04, 1.669524E+04, 1.672456E+04, 1.675390E+04, 1.678324E+04, 
#   1.681259E+04, 1.684194E+04, 1.687130E+04, 1.690067E+04, 1.693005E+04, 1.695943E+04, 
#   1.698882E+04, 1.701821E+04, 1.704761E+04, 1.707702E+04, 1.710643E+04, 1.713585E+04, 
#   1.716528E+04, 1.719471E+04, 1.722415E+04, 1.725360E+04, 1.728305E+04, 1.731251E+04, 
#   1.734197E+04, 1.737144E+04, 1.740092E+04, 1.743040E+04, 1.745989E+04, 1.748939E+04, 
#   1.751889E+04, 1.754840E+04, 1.757791E+04, 1.760743E+04, 1.763695E+04, 1.766648E+04, 
#   1.769602E+04, 1.772556E+04, 1.775510E+04, 1.778466E+04, 1.781422E+04, 1.784378E+04, 
#   1.787335E+04, 1.790292E+04, 1.793250E+04, 1.796209E+04, 1.799168E+04, 1.802128E+04, 
#   1.805088E+04, 1.808049E+04, 1.811010E+04, 1.813972E+04, 1.816934E+04, 1.819897E+04, 
#   1.822860E+04, 1.825824E+04, 1.828789E+04, 1.831754E+04, 1.834719E+04, 1.837685E+04, 
#   1.840651E+04, 1.843618E+04, 1.846585E+04, 1.849553E+04, 1.852522E+04, 1.855490E+04, 
#   1.858460E+04, 1.861429E+04, 1.864400E+04, 1.867370E+04, 1.870342E+04, 1.873313E+04, 
#   1.876285E+04, 
#])
# ============================ M = 23, I = 1 ============================
M = 23
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.255910E+00, 3.030021E+01, 5.846743E+01, 8.666632E+01, 1.148737E+02, 1.430850E+02, 
   1.712988E+02, 1.995147E+02, 2.277330E+02, 2.559561E+02, 2.841901E+02, 3.124467E+02, 
   3.407448E+02, 3.691113E+02, 3.975806E+02, 4.261940E+02, 4.549981E+02, 4.840436E+02, 
   5.133834E+02, 5.430716E+02, 5.731623E+02, 6.037087E+02, 6.347625E+02, 6.663738E+02, 
   6.985902E+02, 7.314574E+02, 7.650186E+02, 7.993152E+02, 8.343861E+02, 8.702687E+02, 
   9.069982E+02, 9.446087E+02, 9.831325E+02, 1.022601E+03, 1.063044E+03, 1.104490E+03, 
   1.146969E+03, 1.190507E+03, 1.235132E+03, 1.280869E+03, 1.327747E+03, 1.375789E+03, 
   1.425022E+03, 1.475471E+03, 1.527162E+03, 1.580120E+03, 1.634371E+03, 1.689939E+03, 
   1.746850E+03, 1.805130E+03, 1.864805E+03, 1.925900E+03, 1.988442E+03, 2.052457E+03, 
   2.117972E+03, 2.185013E+03, 2.253608E+03, 2.323784E+03, 2.395569E+03, 2.468990E+03, 
   2.544078E+03, 2.620859E+03, 2.699363E+03, 2.779620E+03, 2.861660E+03, 2.945511E+03, 
   3.031206E+03, 3.118775E+03, 3.208249E+03, 3.299660E+03, 3.393039E+03, 3.488420E+03, 
   3.585834E+03, 3.685316E+03, 3.786899E+03, 3.890617E+03, 3.996503E+03, 4.104594E+03, 
   4.214924E+03, 4.327528E+03, 4.442444E+03, 4.559706E+03, 4.679352E+03, 4.801420E+03, 
   4.925946E+03, 5.052970E+03, 5.182528E+03, 5.314661E+03, 5.449407E+03, 5.586807E+03, 
   5.726899E+03, 5.869726E+03, 6.015328E+03, 6.163745E+03, 6.315021E+03, 6.469197E+03, 
   6.626316E+03, 6.786421E+03, 6.949556E+03, 7.115764E+03, 7.285090E+03, 7.462934E+03, 
   7.644438E+03, 7.829368E+03, 8.017771E+03, 8.209700E+03, 8.405197E+03, 8.604317E+03, 
   8.807106E+03, 9.013616E+03, 9.223898E+03, 9.437998E+03, 9.655973E+03, 9.877874E+03, 
   1.010375E+04, 1.033366E+04, 1.056765E+04, 1.080577E+04, 1.104809E+04, 1.129466E+04, 
   1.154553E+04, 1.180075E+04, 1.206039E+04, 1.232450E+04, 1.259313E+04, 1.286636E+04, 
   1.314422E+04, 1.342679E+04, 1.371412E+04, 1.400627E+04, 1.430330E+04, 1.460527E+04, 
   1.491225E+04, 1.522429E+04, 1.554147E+04, 1.586383E+04, 1.619144E+04, 1.652437E+04, 
   1.686268E+04, 1.720645E+04, 1.755572E+04, 1.791056E+04, 1.827106E+04, 1.863726E+04, 
   1.900924E+04, 1.938706E+04, 1.977081E+04, 2.016053E+04, 2.055631E+04, 2.095821E+04, 
   2.136630E+04, 2.178066E+04, 2.220136E+04, 2.262846E+04, 2.306204E+04, 2.350218E+04, 
   2.394895E+04, 2.440241E+04, 2.486265E+04, 2.532975E+04, 2.580377E+04, 2.628481E+04, 
   2.677291E+04, 2.726818E+04, 2.777069E+04, 2.828052E+04, 2.879773E+04, 2.932242E+04, 
   2.985468E+04, 3.039456E+04, 3.094216E+04, 3.149756E+04, 3.206084E+04, 3.263209E+04, 
   3.321139E+04, 3.379881E+04, 3.439446E+04, 3.499841E+04, 3.561075E+04, 3.623156E+04, 
   3.686094E+04, 3.749896E+04, 3.814573E+04, 3.880131E+04, 3.946582E+04, 4.013933E+04, 
   4.082193E+04, 4.151373E+04, 4.221480E+04, 4.292525E+04, 4.364516E+04, 4.437462E+04, 
   4.511375E+04, 4.586260E+04, 4.662130E+04, 4.738994E+04, 4.816861E+04, 4.895742E+04, 
   4.975646E+04, 5.056580E+04, 5.138558E+04, 5.221588E+04, 5.305681E+04, 5.390845E+04, 
   5.477092E+04, 5.564432E+04, 5.652873E+04, 5.742429E+04, 5.833107E+04, 5.924920E+04, 
   6.017876E+04, 6.111988E+04, 6.207265E+04, 6.303717E+04, 6.401357E+04, 6.500193E+04, 
   6.600238E+04, 6.701501E+04, 6.803996E+04, 6.907732E+04, 7.012718E+04, 7.118969E+04, 
   7.226494E+04, 7.335304E+04, 7.445411E+04, 7.556827E+04, 7.669563E+04, 7.783629E+04, 
   7.899039E+04, 8.015801E+04, 8.133932E+04, 8.253439E+04, 8.374336E+04, 8.496635E+04, 
   8.620347E+04, 8.745484E+04, 8.872059E+04, 9.000083E+04, 9.129568E+04, 9.260529E+04, 
   9.392974E+04, 9.526917E+04, 9.662374E+04, 9.799352E+04, 9.937866E+04, 1.007793E+05, 
   1.021955E+05, 1.036275E+05, 1.050753E+05, 1.065392E+05, 1.080191E+05, 1.095154E+05, 
   1.110280E+05, 1.125571E+05, 1.141028E+05, 1.156654E+05, 1.172448E+05, 1.188413E+05, 
   1.204550E+05, 1.220860E+05, 1.237344E+05, 1.254005E+05, 1.270842E+05, 1.287858E+05, 
   1.305054E+05, 1.322431E+05, 1.339991E+05, 1.357736E+05, 1.375665E+05, 1.393782E+05, 
   1.412087E+05, 1.430582E+05, 1.449268E+05, 1.468147E+05, 1.487220E+05, 1.506488E+05, 
   1.525953E+05, 1.545617E+05, 1.565481E+05, 1.585546E+05, 1.605814E+05, 1.626286E+05, 
   1.646964E+05, 1.667849E+05, 1.688944E+05, 1.710248E+05, 1.731765E+05, 1.753495E+05, 
   1.775439E+05, 1.797600E+05, 1.819979E+05, 1.842578E+05, 1.865398E+05, 1.888440E+05, 
   1.911707E+05, 1.935199E+05, 1.958919E+05, 1.982867E+05, 2.007047E+05, 2.031458E+05, 
   2.056102E+05, 2.080983E+05, 2.106100E+05, 2.131455E+05, 2.157051E+05, 2.182889E+05, 
   2.208970E+05, 2.235296E+05, 2.261869E+05, 2.288691E+05, 2.315762E+05, 2.343085E+05, 
   2.370662E+05, 2.398493E+05, 2.426582E+05, 2.454929E+05, 2.483535E+05, 2.512404E+05, 
   2.541537E+05, 2.570935E+05, 2.600599E+05, 2.630532E+05, 2.660736E+05, 2.691212E+05, 
   2.721962E+05, 2.752988E+05, 2.784291E+05, 2.815874E+05, 2.847737E+05, 2.879883E+05, 
   2.912314E+05, 2.945031E+05, 2.978036E+05, 3.011331E+05, 3.044918E+05, 3.078799E+05, 
   3.112975E+05, 3.147447E+05, 3.182219E+05, 3.217292E+05, 3.252667E+05, 3.288348E+05, 
   3.324334E+05, 3.360629E+05, 3.397233E+05, 3.434150E+05, 3.471381E+05, 3.508926E+05, 
   3.546790E+05, 3.584974E+05, 3.623478E+05, 
])
# ============================ M = 23, I = 2 ============================
M = 23
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.256912E+01, 6.203839E+01, 1.198179E+02, 1.776591E+02, 2.355169E+02, 2.933826E+02, 
   3.512532E+02, 4.091279E+02, 4.670077E+02, 5.248974E+02, 5.828098E+02, 6.407694E+02, 
   6.988156E+02, 7.570040E+02, 8.154062E+02, 8.741077E+02, 9.332049E+02, 9.928024E+02, 
   1.053010E+03, 1.113938E+03, 1.175699E+03, 1.238403E+03, 1.302157E+03, 1.367062E+03, 
   1.433218E+03, 1.500720E+03, 1.569655E+03, 1.640110E+03, 1.712166E+03, 1.785898E+03, 
   1.861382E+03, 1.938686E+03, 2.017879E+03, 2.099024E+03, 2.182186E+03, 2.267424E+03, 
   2.354797E+03, 2.444363E+03, 2.536179E+03, 2.630300E+03, 2.726782E+03, 2.825678E+03, 
   2.927041E+03, 3.030927E+03, 3.137388E+03, 3.246478E+03, 3.358249E+03, 3.472756E+03, 
   3.590052E+03, 3.710191E+03, 3.833228E+03, 3.959216E+03, 4.088213E+03, 4.220272E+03, 
   4.355451E+03, 4.493805E+03, 4.635393E+03, 4.780272E+03, 4.928501E+03, 5.080139E+03, 
   5.235245E+03, 5.393882E+03, 5.556109E+03, 5.721989E+03, 5.891584E+03, 6.064959E+03, 
   6.242178E+03, 6.423305E+03, 6.608407E+03, 6.797551E+03, 6.990802E+03, 7.188231E+03, 
   7.389906E+03, 7.595897E+03, 7.806274E+03, 8.021110E+03, 8.240476E+03, 8.464446E+03, 
   8.693093E+03, 8.926493E+03, 9.164721E+03, 9.407853E+03, 9.655968E+03, 9.909143E+03, 
   1.016746E+04, 1.043099E+04, 1.069982E+04, 1.097404E+04, 1.125372E+04, 1.153894E+04, 
   1.182980E+04, 1.212637E+04, 1.242875E+04, 1.273701E+04, 1.305125E+04, 1.337156E+04, 
   1.369802E+04, 1.403073E+04, 1.436977E+04, 1.471524E+04, 1.506723E+04, 1.544104E+04, 
   1.581858E+04, 1.620329E+04, 1.659528E+04, 1.699465E+04, 1.740150E+04, 1.781594E+04, 
   1.823806E+04, 1.866798E+04, 1.910581E+04, 1.955164E+04, 2.000560E+04, 2.046778E+04, 
   2.093831E+04, 2.141729E+04, 2.190483E+04, 2.240105E+04, 2.290607E+04, 2.342000E+04, 
   2.394295E+04, 2.447505E+04, 2.501642E+04, 2.556717E+04, 2.612743E+04, 2.669731E+04, 
   2.727695E+04, 2.786646E+04, 2.846597E+04, 2.907562E+04, 2.969552E+04, 3.032581E+04, 
   3.096661E+04, 3.161806E+04, 3.228030E+04, 3.295343E+04, 3.363762E+04, 3.433299E+04, 
   3.503967E+04, 3.575782E+04, 3.648755E+04, 3.722902E+04, 3.798238E+04, 3.874774E+04, 
   3.952527E+04, 4.031510E+04, 4.111738E+04, 4.193226E+04, 4.275989E+04, 4.360041E+04, 
   4.445397E+04, 4.532073E+04, 4.620085E+04, 4.709445E+04, 4.800173E+04, 4.892280E+04, 
   4.985785E+04, 5.080703E+04, 5.177049E+04, 5.274841E+04, 5.374092E+04, 5.474821E+04, 
   5.577045E+04, 5.680777E+04, 5.786038E+04, 5.892842E+04, 6.001205E+04, 6.111146E+04, 
   6.222683E+04, 6.335832E+04, 6.450609E+04, 6.567033E+04, 6.685123E+04, 6.804893E+04, 
   6.926364E+04, 7.049552E+04, 7.174476E+04, 7.301155E+04, 7.429607E+04, 7.559851E+04, 
   7.691903E+04, 7.825783E+04, 7.961513E+04, 8.099106E+04, 8.238587E+04, 8.379971E+04, 
   8.523279E+04, 8.668532E+04, 8.815748E+04, 8.964944E+04, 9.116146E+04, 9.269367E+04, 
   9.424635E+04, 9.581962E+04, 9.741373E+04, 9.902889E+04, 1.006653E+05, 1.023232E+05, 
   1.040027E+05, 1.057040E+05, 1.074275E+05, 1.091733E+05, 1.109415E+05, 1.127325E+05, 
   1.145465E+05, 1.163836E+05, 1.182440E+05, 1.201281E+05, 1.220360E+05, 1.239679E+05, 
   1.259241E+05, 1.279048E+05, 1.299102E+05, 1.319406E+05, 1.339961E+05, 1.360770E+05, 
   1.381836E+05, 1.403161E+05, 1.424746E+05, 1.446595E+05, 1.468710E+05, 1.491093E+05, 
   1.513747E+05, 1.536674E+05, 1.559876E+05, 1.583356E+05, 1.607117E+05, 1.631160E+05, 
   1.655489E+05, 1.680105E+05, 1.705012E+05, 1.730212E+05, 1.755707E+05, 1.781500E+05, 
   1.807593E+05, 1.833990E+05, 1.860692E+05, 1.887702E+05, 1.915024E+05, 1.942658E+05, 
   1.970610E+05, 1.998879E+05, 2.027471E+05, 2.056386E+05, 2.085628E+05, 2.115200E+05, 
   2.145104E+05, 2.175344E+05, 2.205921E+05, 2.236838E+05, 2.268099E+05, 2.299706E+05, 
   2.331663E+05, 2.363970E+05, 2.396632E+05, 2.429652E+05, 2.463032E+05, 2.496775E+05, 
   2.530885E+05, 2.565363E+05, 2.600213E+05, 2.635438E+05, 2.671041E+05, 2.707024E+05, 
   2.743392E+05, 2.780146E+05, 2.817289E+05, 2.854825E+05, 2.892757E+05, 2.931087E+05, 
   2.969820E+05, 3.008957E+05, 3.048502E+05, 3.088458E+05, 3.128829E+05, 3.169616E+05, 
   3.210824E+05, 3.252455E+05, 3.294513E+05, 3.337001E+05, 3.379922E+05, 3.423279E+05, 
   3.467075E+05, 3.511314E+05, 3.556000E+05, 3.601134E+05, 3.646721E+05, 3.692763E+05, 
   3.739265E+05, 3.786228E+05, 3.833658E+05, 3.881556E+05, 3.929927E+05, 3.978773E+05, 
   4.028098E+05, 4.077906E+05, 4.128199E+05, 4.178982E+05, 4.230257E+05, 4.282029E+05, 
   4.334300E+05, 4.387075E+05, 4.440355E+05, 4.494145E+05, 4.548450E+05, 4.603270E+05, 
   4.658613E+05, 4.714479E+05, 4.770871E+05, 4.827796E+05, 4.885256E+05, 4.943252E+05, 
   5.001793E+05, 5.060878E+05, 5.120512E+05, 5.180700E+05, 5.241444E+05, 5.302749E+05, 
   5.364618E+05, 5.427052E+05, 5.490060E+05, 5.553642E+05, 5.617803E+05, 5.682546E+05, 
   5.747876E+05, 5.813797E+05, 5.880310E+05, 5.947421E+05, 6.015134E+05, 6.083451E+05, 
   6.152380E+05, 6.221920E+05, 6.292077E+05, 6.362855E+05, 6.434256E+05, 6.506287E+05, 
   6.578952E+05, 6.652251E+05, 6.726192E+05, 6.800774E+05, 6.876007E+05, 6.951892E+05, 
   7.028432E+05, 7.105633E+05, 7.183499E+05, 7.262031E+05, 7.341236E+05, 7.421119E+05, 
   7.501680E+05, 7.582926E+05, 7.664860E+05, 
])
# ============================ M = 23, I = 3 ============================
M = 23
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.705510E+00, 2.076356E+01, 4.011176E+01, 5.948433E+01, 7.886627E+01, 9.825473E+01, 
   1.176487E+02, 1.370479E+02, 1.564527E+02, 1.758646E+02, 1.952881E+02, 2.147311E+02, 
   2.342069E+02, 2.537341E+02, 2.733368E+02, 2.930436E+02, 3.128870E+02, 3.329024E+02, 
   3.531270E+02, 3.735989E+02, 3.943565E+02, 4.154380E+02, 4.368808E+02, 4.587215E+02, 
   4.809954E+02, 5.037367E+02, 5.269788E+02, 5.507533E+02, 5.750913E+02, 6.000224E+02, 
   6.255753E+02, 6.517782E+02, 6.786581E+02, 7.062413E+02, 7.345535E+02, 7.636201E+02, 
   7.934654E+02, 8.241137E+02, 8.555889E+02, 8.879144E+02, 9.211128E+02, 9.552075E+02, 
   9.902210E+02, 1.026176E+03, 1.063094E+03, 1.100997E+03, 1.139909E+03, 1.179849E+03, 
   1.220842E+03, 1.262907E+03, 1.306069E+03, 1.350347E+03, 1.395765E+03, 1.442344E+03, 
   1.490107E+03, 1.539076E+03, 1.589273E+03, 1.640721E+03, 1.693442E+03, 1.747459E+03, 
   1.802795E+03, 1.859474E+03, 1.917518E+03, 1.976950E+03, 2.037796E+03, 2.100078E+03, 
   2.163821E+03, 2.229049E+03, 2.295788E+03, 2.364060E+03, 2.433892E+03, 2.505309E+03, 
   2.578337E+03, 2.653001E+03, 2.729329E+03, 2.807344E+03, 2.887077E+03, 2.968551E+03, 
   3.051796E+03, 3.136839E+03, 3.223707E+03, 3.312429E+03, 3.403033E+03, 3.495548E+03, 
   3.590003E+03, 3.686428E+03, 3.784852E+03, 3.885304E+03, 3.987816E+03, 4.092418E+03, 
   4.199140E+03, 4.308016E+03, 4.419076E+03, 4.532350E+03, 4.647874E+03, 4.765678E+03, 
   4.885797E+03, 5.008261E+03, 5.133106E+03, 5.260367E+03, 5.390076E+03, 5.522268E+03, 
   5.656980E+03, 5.794245E+03, 5.934099E+03, 6.076579E+03, 6.221720E+03, 6.369560E+03, 
   6.520136E+03, 6.673485E+03, 6.829645E+03, 6.988653E+03, 7.150549E+03, 7.315370E+03, 
   7.483158E+03, 7.653950E+03, 7.827786E+03, 8.004708E+03, 8.184755E+03, 8.367969E+03, 
   8.554390E+03, 8.744063E+03, 8.937025E+03, 9.133324E+03, 9.332999E+03, 9.536096E+03, 
   9.742652E+03, 9.952720E+03, 1.016634E+04, 1.038355E+04, 1.060441E+04, 1.082895E+04, 
   1.105723E+04, 1.128929E+04, 1.152517E+04, 1.176492E+04, 1.200860E+04, 1.225624E+04, 
   1.250790E+04, 1.276362E+04, 1.302346E+04, 1.328745E+04, 1.355567E+04, 1.382814E+04, 
   1.410492E+04, 1.438607E+04, 1.467163E+04, 1.496166E+04, 1.525621E+04, 1.555533E+04, 
   1.585907E+04, 1.616749E+04, 1.648063E+04, 1.679856E+04, 1.712133E+04, 1.744900E+04, 
   1.778161E+04, 1.811922E+04, 1.846190E+04, 1.880969E+04, 1.916266E+04, 1.952085E+04, 
   1.988433E+04, 2.025316E+04, 2.062740E+04, 2.100710E+04, 2.139232E+04, 2.178312E+04, 
   2.217957E+04, 2.258171E+04, 2.298962E+04, 2.340336E+04, 2.382298E+04, 2.424856E+04, 
   2.468014E+04, 2.511780E+04, 2.556160E+04, 2.601160E+04, 2.646787E+04, 2.693046E+04, 
   2.739946E+04, 2.787491E+04, 2.835689E+04, 2.884547E+04, 2.934071E+04, 2.984268E+04, 
   3.035144E+04, 3.086706E+04, 3.138963E+04, 3.191919E+04, 3.245582E+04, 3.299960E+04, 
   3.355059E+04, 3.410885E+04, 3.467448E+04, 3.524753E+04, 3.582807E+04, 3.641619E+04, 
   3.701195E+04, 3.761541E+04, 3.822668E+04, 3.884581E+04, 3.947288E+04, 4.010796E+04, 
   4.075113E+04, 4.140247E+04, 4.206204E+04, 4.272995E+04, 4.340624E+04, 4.409102E+04, 
   4.478434E+04, 4.548630E+04, 4.619696E+04, 4.691643E+04, 4.764476E+04, 4.838204E+04, 
   4.912835E+04, 4.988378E+04, 5.064840E+04, 5.142230E+04, 5.220557E+04, 5.299828E+04, 
   5.380051E+04, 5.461237E+04, 5.543391E+04, 5.626524E+04, 5.710644E+04, 5.795759E+04, 
   5.881878E+04, 5.969010E+04, 6.057165E+04, 6.146348E+04, 6.236571E+04, 6.327842E+04, 
   6.420171E+04, 6.513564E+04, 6.608034E+04, 6.703586E+04, 6.800233E+04, 6.897980E+04, 
   6.996841E+04, 7.096821E+04, 7.197933E+04, 7.300183E+04, 7.403584E+04, 7.508140E+04, 
   7.613867E+04, 7.720769E+04, 7.828858E+04, 7.938144E+04, 8.048637E+04, 8.160346E+04, 
   8.273280E+04, 8.387449E+04, 8.502866E+04, 8.619536E+04, 8.737470E+04, 8.856682E+04, 
   8.977178E+04, 9.098972E+04, 9.222067E+04, 9.346482E+04, 9.472220E+04, 9.599299E+04, 
   9.727719E+04, 9.857499E+04, 9.988645E+04, 1.012117E+05, 1.025508E+05, 1.039040E+05, 
   1.052712E+05, 1.066526E+05, 1.080484E+05, 1.094585E+05, 1.108832E+05, 1.123225E+05, 
   1.137766E+05, 1.152455E+05, 1.167294E+05, 1.182283E+05, 1.197425E+05, 1.212719E+05, 
   1.228168E+05, 1.243771E+05, 1.259531E+05, 1.275449E+05, 1.291525E+05, 1.307761E+05, 
   1.324157E+05, 1.340716E+05, 1.357438E+05, 1.374324E+05, 1.391376E+05, 1.408595E+05, 
   1.425981E+05, 1.443537E+05, 1.461262E+05, 1.479160E+05, 1.497229E+05, 1.515473E+05, 
   1.533892E+05, 1.552487E+05, 1.571259E+05, 1.590210E+05, 1.609341E+05, 1.628653E+05, 
   1.648147E+05, 1.667825E+05, 1.687687E+05, 1.707737E+05, 1.727973E+05, 1.748397E+05, 
   1.769011E+05, 1.789817E+05, 1.810815E+05, 1.832006E+05, 1.853392E+05, 1.874975E+05, 
   1.896755E+05, 1.918734E+05, 1.940913E+05, 1.963293E+05, 1.985876E+05, 2.008663E+05, 
   2.031654E+05, 2.054853E+05, 2.078260E+05, 2.101876E+05, 2.125702E+05, 2.149740E+05, 
   2.173992E+05, 2.198458E+05, 2.223140E+05, 2.248039E+05, 2.273157E+05, 2.298495E+05, 
   2.324055E+05, 2.349837E+05, 2.375843E+05, 2.402075E+05, 2.428533E+05, 2.455220E+05, 
   2.482137E+05, 2.509285E+05, 2.536665E+05, 2.564279E+05, 2.592128E+05, 2.620214E+05, 
   2.648538E+05, 2.677102E+05, 2.705906E+05, 
])
# ============================ M = 24, I = 1 ============================
M = 24
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.292049E+01, 3.906739E+02, 1.005073E+03, 1.803271E+03, 2.758590E+03, 3.847277E+03, 
   5.052946E+03, 6.364370E+03, 7.773345E+03, 9.273505E+03, 1.085973E+04, 1.252785E+04, 
   1.427452E+04, 1.609715E+04, 1.799385E+04, 1.996343E+04, 2.200535E+04, 2.411974E+04, 
   2.630733E+04, 2.856944E+04, 3.090796E+04, 3.332531E+04, 3.582441E+04, 3.840864E+04, 
   4.108182E+04, 4.384821E+04, 4.671244E+04, 4.967950E+04, 5.275476E+04, 5.594391E+04, 
   5.925297E+04, 6.268828E+04, 6.625647E+04, 6.996450E+04, 7.381960E+04, 7.782932E+04, 
   8.200151E+04, 8.634430E+04, 9.086615E+04, 9.557583E+04, 1.004824E+05, 1.055954E+05, 
   1.109244E+05, 1.164796E+05, 1.222715E+05, 1.283109E+05, 1.346091E+05, 1.411776E+05, 
   1.480286E+05, 1.551745E+05, 1.626282E+05, 1.704032E+05, 1.785133E+05, 1.869730E+05, 
   1.957970E+05, 2.050010E+05, 2.146008E+05, 2.246131E+05, 2.350550E+05, 2.459444E+05, 
   2.572997E+05, 2.691400E+05, 2.814852E+05, 2.943557E+05, 3.077729E+05, 3.217588E+05, 
   3.363361E+05, 3.515286E+05, 3.673608E+05, 3.838579E+05, 4.010463E+05, 4.189533E+05, 
   4.376069E+05, 4.570363E+05, 4.772718E+05, 4.983445E+05, 5.202869E+05, 5.431324E+05, 
   5.669158E+05, 5.916727E+05, 6.174404E+05, 6.442573E+05, 6.721630E+05, 7.011986E+05, 
   7.314066E+05, 7.628309E+05, 7.955168E+05, 8.295115E+05, 8.648632E+05, 9.016224E+05, 
   9.398407E+05, 9.795718E+05, 1.020871E+06, 1.063795E+06, 1.108404E+06, 1.154758E+06, 
   1.202921E+06, 1.252957E+06, 1.304934E+06, 1.358921E+06, 1.414990E+06, 1.473214E+06, 
   1.533670E+06, 1.596437E+06, 1.661596E+06, 1.729231E+06, 1.799429E+06, 1.872278E+06, 
   1.947870E+06, 2.026301E+06, 2.107669E+06, 2.192074E+06, 2.279620E+06, 2.370416E+06, 
   2.464570E+06, 2.562198E+06, 2.663417E+06, 2.768348E+06, 2.877115E+06, 2.989847E+06, 
   3.106677E+06, 3.227741E+06, 3.353179E+06, 3.483136E+06, 3.617761E+06, 3.757207E+06, 
   3.901632E+06, 4.051198E+06, 4.206074E+06, 4.366431E+06, 4.532446E+06, 4.704302E+06, 
   4.882187E+06, 5.066293E+06, 5.256819E+06, 5.453969E+06, 5.657954E+06, 5.868990E+06, 
   6.087299E+06, 6.313109E+06, 6.546655E+06, 6.788180E+06, 7.037931E+06, 7.296164E+06, 
   7.563141E+06, 7.839132E+06, 8.124414E+06, 8.419272E+06, 8.723998E+06, 9.038892E+06, 
   9.364265E+06, 9.700432E+06, 1.004772E+07, 1.040646E+07, 1.077701E+07, 1.115970E+07, 
   1.155491E+07, 1.196300E+07, 1.238436E+07, 1.281938E+07, 1.326847E+07, 1.373203E+07, 
   1.421048E+07, 1.470427E+07, 1.521384E+07, 1.573965E+07, 1.628216E+07, 1.684187E+07, 
   1.741925E+07, 1.801483E+07, 1.862912E+07, 1.926264E+07, 1.991596E+07, 2.058963E+07, 
   2.128422E+07, 2.200033E+07, 2.273856E+07, 2.349952E+07, 2.428386E+07, 2.509222E+07, 
   2.592526E+07, 2.678368E+07, 2.766818E+07, 2.857946E+07, 2.951827E+07, 3.048535E+07, 
   3.148149E+07, 3.250746E+07, 3.356408E+07, 3.465218E+07, 3.577260E+07, 3.692621E+07, 
   3.811391E+07, 3.933661E+07, 4.059523E+07, 4.189073E+07, 4.322410E+07, 4.459633E+07, 
   4.600844E+07, 4.746149E+07, 4.895654E+07, 5.049470E+07, 5.207708E+07, 5.370485E+07, 
   5.537917E+07, 5.710124E+07, 5.887230E+07, 6.069362E+07, 6.256646E+07, 6.449216E+07, 
   6.647206E+07, 6.850754E+07, 7.060000E+07, 7.275090E+07, 7.496169E+07, 7.723390E+07, 
   7.956905E+07, 8.196872E+07, 8.443453E+07, 8.696812E+07, 8.957116E+07, 9.224539E+07, 
   9.499255E+07, 9.781444E+07, 1.007129E+08, 1.036898E+08, 1.067470E+08, 1.098866E+08, 
   1.131105E+08, 1.164207E+08, 1.198194E+08, 1.233086E+08, 1.268906E+08, 1.305676E+08, 
   1.343418E+08, 1.382155E+08, 1.421912E+08, 1.462713E+08, 1.504582E+08, 1.547544E+08, 
   1.591625E+08, 1.636851E+08, 1.683250E+08, 1.730849E+08, 1.779675E+08, 1.829757E+08, 
   1.881125E+08, 1.933807E+08, 1.987835E+08, 2.043239E+08, 2.100051E+08, 2.158303E+08, 
   2.218028E+08, 2.279259E+08, 2.342031E+08, 2.406379E+08, 2.472338E+08, 2.539945E+08, 
   2.609236E+08, 2.680251E+08, 2.753026E+08, 2.827601E+08, 2.904017E+08, 2.982315E+08, 
   3.062536E+08, 3.144722E+08, 3.228917E+08, 3.315166E+08, 3.403513E+08, 3.494004E+08, 
   3.586686E+08, 3.681608E+08, 3.778816E+08, 3.878362E+08, 3.980296E+08, 4.084669E+08, 
   4.191534E+08, 4.300945E+08, 4.412955E+08, 4.527621E+08, 4.644999E+08, 4.765147E+08, 
   4.888123E+08, 5.013988E+08, 5.142804E+08, 5.274630E+08, 5.409532E+08, 5.547574E+08, 
   5.688822E+08, 5.833342E+08, 5.981202E+08, 6.132473E+08, 6.287224E+08, 6.445529E+08, 
   6.607459E+08, 6.773091E+08, 6.942500E+08, 7.115763E+08, 7.292960E+08, 7.474170E+08, 
   7.659476E+08, 7.848960E+08, 8.042708E+08, 8.240805E+08, 8.443340E+08, 8.650402E+08, 
   8.862081E+08, 9.078471E+08, 9.299665E+08, 9.525760E+08, 9.756852E+08, 9.993043E+08, 
   1.023443E+09, 1.048112E+09, 1.073322E+09, 1.099082E+09, 1.125406E+09, 1.152302E+09, 
   1.179782E+09, 1.207858E+09, 1.236541E+09, 1.265844E+09, 1.295777E+09, 1.326354E+09, 
   1.357587E+09, 1.389488E+09, 1.422071E+09, 1.455348E+09, 1.489332E+09, 1.524038E+09, 
   1.559479E+09, 1.595668E+09, 1.632621E+09, 1.670351E+09, 1.708874E+09, 1.748204E+09, 
   1.788356E+09, 1.829346E+09, 1.871190E+09, 1.913903E+09, 1.957502E+09, 2.002003E+09, 
   2.047423E+09, 2.093780E+09, 2.141090E+09, 2.189371E+09, 2.238641E+09, 2.288918E+09, 
   2.340220E+09, 2.392568E+09, 2.445978E+09, 2.500472E+09, 2.556068E+09, 2.612787E+09, 
   2.670649E+09, 2.729675E+09, 2.789885E+09, 2.851302E+09, 2.913946E+09, 2.977841E+09, 
   3.043007E+09, 3.109469E+09, 3.177248E+09, 3.246370E+09, 3.316856E+09, 3.388733E+09, 
   3.462023E+09, 3.536753E+09, 3.612947E+09, 3.690632E+09, 3.769834E+09, 3.850578E+09, 
   3.932892E+09, 4.016805E+09, 4.102342E+09, 4.189533E+09, 4.278407E+09, 4.368992E+09, 
   4.461318E+09, 4.555415E+09, 4.651313E+09, 4.749044E+09, 4.848640E+09, 4.950131E+09, 
   5.053550E+09, 5.158931E+09, 5.266306E+09, 5.375710E+09, 5.487177E+09, 5.600741E+09, 
   5.716439E+09, 5.834306E+09, 5.954378E+09, 6.076694E+09, 6.201289E+09, 6.328203E+09, 
   6.457475E+09, 6.589142E+09, 6.723247E+09, 6.859828E+09, 6.998927E+09, 7.140585E+09, 
   7.284846E+09, 7.431751E+09, 7.581344E+09, 7.733670E+09, 7.888773E+09, 8.046698E+09, 
   8.207493E+09, 8.371202E+09, 8.537875E+09, 8.707558E+09, 8.880302E+09, 9.056155E+09, 
   9.235167E+09, 9.417390E+09, 9.602875E+09, 9.791675E+09, 9.983842E+09, 1.017943E+10, 
   1.037850E+10, 1.058109E+10, 1.078728E+10, 1.099711E+10, 1.121064E+10, 1.142794E+10, 
   1.164905E+10, 1.187405E+10, 1.210299E+10, 1.233593E+10, 1.257294E+10, 1.281408E+10, 
   1.305942E+10, 1.330901E+10, 1.356294E+10, 1.382126E+10, 1.408404E+10, 1.435135E+10, 
   1.462327E+10, 1.489986E+10, 1.518119E+10, 1.546734E+10, 1.575839E+10, 1.605440E+10, 
   1.635546E+10, 1.666163E+10, 1.697301E+10, 1.728966E+10, 1.761167E+10, 1.793912E+10, 
   1.827210E+10, 1.861067E+10, 1.895494E+10, 1.930498E+10, 1.966089E+10, 2.002275E+10, 
   2.039064E+10, 2.076467E+10, 2.114492E+10, 2.153149E+10, 2.192446E+10, 2.232394E+10, 
   2.273002E+10, 2.314280E+10, 2.356237E+10, 2.398885E+10, 2.442232E+10, 2.486290E+10, 
   2.531068E+10, 2.576578E+10, 2.622830E+10, 2.669835E+10, 2.717604E+10, 2.766147E+10, 
   2.815477E+10, 2.865605E+10, 2.916542E+10, 2.968300E+10, 3.020891E+10, 3.074327E+10, 
   3.128620E+10, 3.183782E+10, 3.239826E+10, 3.296765E+10, 3.354611E+10, 3.413378E+10, 
   3.473078E+10, 3.533724E+10, 3.595331E+10, 3.657912E+10, 3.721481E+10, 3.786051E+10, 
   3.851636E+10, 3.918252E+10, 3.985913E+10, 4.054633E+10, 4.124427E+10, 4.195311E+10, 
   4.267299E+10, 4.340407E+10, 4.414650E+10, 
])
# ============================ M = 24, I = 2 ============================
M = 24
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.320626E+01, 3.967139E+02, 1.020811E+03, 1.831643E+03, 2.802089E+03, 3.908016E+03, 
   5.132777E+03, 6.464971E+03, 7.896264E+03, 9.420189E+03, 1.103154E+04, 1.272609E+04, 
   1.450044E+04, 1.635196E+04, 1.827872E+04, 2.027951E+04, 2.235380E+04, 2.450170E+04, 
   2.672396E+04, 2.902193E+04, 3.139753E+04, 3.385320E+04, 3.639192E+04, 3.901711E+04, 
   4.173268E+04, 4.454293E+04, 4.745257E+04, 5.046667E+04, 5.359069E+04, 5.683041E+04, 
   6.019195E+04, 6.368173E+04, 6.730651E+04, 7.107334E+04, 7.498957E+04, 7.906288E+04, 
   8.330123E+04, 8.771290E+04, 9.230646E+04, 9.709084E+04, 1.020752E+05, 1.072693E+05, 
   1.126828E+05, 1.183261E+05, 1.242099E+05, 1.303451E+05, 1.367432E+05, 1.434159E+05, 
   1.503755E+05, 1.576348E+05, 1.652068E+05, 1.731051E+05, 1.813439E+05, 1.899377E+05, 
   1.989018E+05, 2.082518E+05, 2.180039E+05, 2.281750E+05, 2.387826E+05, 2.498447E+05, 
   2.613802E+05, 2.734084E+05, 2.859494E+05, 2.990242E+05, 3.126542E+05, 3.268620E+05, 
   3.416706E+05, 3.571042E+05, 3.731875E+05, 3.899465E+05, 4.074077E+05, 4.255988E+05, 
   4.445484E+05, 4.642861E+05, 4.848427E+05, 5.062498E+05, 5.285405E+05, 5.517486E+05, 
   5.759093E+05, 6.010592E+05, 6.272359E+05, 6.544784E+05, 6.828270E+05, 7.123234E+05, 
   7.430109E+05, 7.749339E+05, 8.081387E+05, 8.426729E+05, 8.785858E+05, 9.159285E+05, 
   9.547534E+05, 9.951152E+05, 1.037070E+06, 1.080676E+06, 1.125993E+06, 1.173083E+06, 
   1.222010E+06, 1.272841E+06, 1.325643E+06, 1.380487E+06, 1.437446E+06, 1.496594E+06, 
   1.558010E+06, 1.621774E+06, 1.687968E+06, 1.756676E+06, 1.827988E+06, 1.901994E+06, 
   1.978787E+06, 2.058463E+06, 2.141123E+06, 2.226868E+06, 2.315805E+06, 2.408042E+06, 
   2.503692E+06, 2.602870E+06, 2.705696E+06, 2.812293E+06, 2.922788E+06, 3.037310E+06, 
   3.155996E+06, 3.278982E+06, 3.406412E+06, 3.538433E+06, 3.675196E+06, 3.816857E+06, 
   3.963576E+06, 4.115518E+06, 4.272854E+06, 4.435758E+06, 4.604410E+06, 4.778996E+06, 
   4.959706E+06, 5.146737E+06, 5.340289E+06, 5.540572E+06, 5.747797E+06, 5.962185E+06, 
   6.183962E+06, 6.413359E+06, 6.650616E+06, 6.895978E+06, 7.149697E+06, 7.412032E+06, 
   7.683251E+06, 7.963627E+06, 8.253442E+06, 8.552984E+06, 8.862552E+06, 9.182450E+06, 
   9.512992E+06, 9.854501E+06, 1.020731E+07, 1.057175E+07, 1.094818E+07, 1.133696E+07, 
   1.173845E+07, 1.215302E+07, 1.258108E+07, 1.302302E+07, 1.347924E+07, 1.395016E+07, 
   1.443622E+07, 1.493786E+07, 1.545553E+07, 1.598969E+07, 1.654083E+07, 1.710943E+07, 
   1.769599E+07, 1.830103E+07, 1.892508E+07, 1.956868E+07, 2.023238E+07, 2.091676E+07, 
   2.162240E+07, 2.234989E+07, 2.309985E+07, 2.387291E+07, 2.466971E+07, 2.549092E+07, 
   2.633721E+07, 2.720928E+07, 2.810783E+07, 2.903360E+07, 2.998734E+07, 3.096980E+07, 
   3.198177E+07, 3.302405E+07, 3.409747E+07, 3.520287E+07, 3.634110E+07, 3.751306E+07, 
   3.871965E+07, 3.996178E+07, 4.124042E+07, 4.255652E+07, 4.391109E+07, 4.530514E+07, 
   4.673971E+07, 4.821586E+07, 4.973469E+07, 5.129731E+07, 5.290486E+07, 5.455852E+07, 
   5.625946E+07, 5.800892E+07, 5.980815E+07, 6.165843E+07, 6.356106E+07, 6.551739E+07, 
   6.752878E+07, 6.959663E+07, 7.172238E+07, 7.390749E+07, 7.615345E+07, 7.846180E+07, 
   8.083409E+07, 8.327194E+07, 8.577697E+07, 8.835086E+07, 9.099532E+07, 9.371209E+07, 
   9.650295E+07, 9.936973E+07, 1.023143E+08, 1.053386E+08, 1.084444E+08, 1.116340E+08, 
   1.149091E+08, 1.182720E+08, 1.217248E+08, 1.252695E+08, 1.289085E+08, 1.326440E+08, 
   1.364782E+08, 1.404136E+08, 1.444526E+08, 1.485976E+08, 1.528511E+08, 1.572156E+08, 
   1.616939E+08, 1.662885E+08, 1.710023E+08, 1.758379E+08, 1.807982E+08, 1.858861E+08, 
   1.911046E+08, 1.964567E+08, 2.019455E+08, 2.075740E+08, 2.133456E+08, 2.192636E+08, 
   2.253311E+08, 2.315517E+08, 2.379289E+08, 2.444661E+08, 2.511670E+08, 2.580353E+08, 
   2.650747E+08, 2.722892E+08, 2.796826E+08, 2.872588E+08, 2.950221E+08, 3.029765E+08, 
   3.111263E+08, 3.194757E+08, 3.280293E+08, 3.367915E+08, 3.457668E+08, 3.549600E+08, 
   3.643758E+08, 3.740191E+08, 3.838948E+08, 3.940079E+08, 4.043636E+08, 4.149671E+08, 
   4.258237E+08, 4.369390E+08, 4.483184E+08, 4.599676E+08, 4.718923E+08, 4.840984E+08, 
   4.965919E+08, 5.093789E+08, 5.224655E+08, 5.358581E+08, 5.495632E+08, 5.635872E+08, 
   5.779369E+08, 5.926191E+08, 6.076407E+08, 6.230087E+08, 6.387303E+08, 6.548129E+08, 
   6.712639E+08, 6.880909E+08, 7.053016E+08, 7.229040E+08, 7.409059E+08, 7.593156E+08, 
   7.781413E+08, 7.973916E+08, 8.170751E+08, 8.372004E+08, 8.577765E+08, 8.788126E+08, 
   9.003178E+08, 9.223015E+08, 9.447733E+08, 9.677430E+08, 9.912205E+08, 1.015216E+09, 
   1.039739E+09, 1.064801E+09, 1.090413E+09, 1.116584E+09, 1.143326E+09, 1.170651E+09, 
   1.198569E+09, 1.227093E+09, 1.256233E+09, 1.286003E+09, 1.316413E+09, 1.347478E+09, 
   1.379208E+09, 1.411618E+09, 1.444720E+09, 1.478527E+09, 1.513053E+09, 1.548312E+09, 
   1.584318E+09, 1.621084E+09, 1.658626E+09, 1.696957E+09, 1.736094E+09, 1.776050E+09, 
   1.816843E+09, 1.858486E+09, 1.900997E+09, 1.944391E+09, 1.988685E+09, 2.033896E+09, 
   2.080040E+09, 2.127136E+09, 2.175200E+09, 2.224251E+09, 2.274306E+09, 2.325385E+09, 
   2.377505E+09, 2.430687E+09, 2.484949E+09, 2.540312E+09, 2.596795E+09, 2.654418E+09, 
   2.713203E+09, 2.773170E+09, 2.834340E+09, 2.896736E+09, 2.960380E+09, 3.025293E+09, 
   3.091499E+09, 3.159020E+09, 3.227881E+09, 3.298105E+09, 3.369715E+09, 3.442738E+09, 
   3.517198E+09, 3.593119E+09, 3.670529E+09, 3.749453E+09, 3.829917E+09, 3.911950E+09, 
   3.995577E+09, 4.080828E+09, 4.167730E+09, 4.256312E+09, 4.346603E+09, 4.438633E+09, 
   4.532432E+09, 4.628030E+09, 4.725459E+09, 4.824749E+09, 4.925933E+09, 5.029044E+09, 
   5.134113E+09, 5.241175E+09, 5.350264E+09, 5.461413E+09, 5.574658E+09, 5.690035E+09, 
   5.807578E+09, 5.927326E+09, 6.049314E+09, 6.173582E+09, 6.300165E+09, 6.429105E+09, 
   6.560439E+09, 6.694208E+09, 6.830452E+09, 6.969213E+09, 7.110532E+09, 7.254452E+09, 
   7.401014E+09, 7.550264E+09, 7.702245E+09, 7.857002E+09, 8.014581E+09, 8.175027E+09, 
   8.338388E+09, 8.504710E+09, 8.674044E+09, 8.846436E+09, 9.021937E+09, 9.200597E+09, 
   9.382467E+09, 9.567599E+09, 9.756045E+09, 9.947859E+09, 1.014309E+10, 1.034181E+10, 
   1.054405E+10, 1.074988E+10, 1.095936E+10, 1.117254E+10, 1.138948E+10, 1.161024E+10, 
   1.183489E+10, 1.206348E+10, 1.229607E+10, 1.253274E+10, 1.277353E+10, 1.301852E+10, 
   1.326778E+10, 1.352136E+10, 1.377934E+10, 1.404178E+10, 1.430876E+10, 1.458034E+10, 
   1.485660E+10, 1.513761E+10, 1.542344E+10, 1.571416E+10, 1.600985E+10, 1.631060E+10, 
   1.661646E+10, 1.692753E+10, 1.724388E+10, 1.756559E+10, 1.789274E+10, 1.822542E+10, 
   1.856371E+10, 1.890770E+10, 1.925747E+10, 1.961310E+10, 1.997469E+10, 2.034233E+10, 
   2.071611E+10, 2.109611E+10, 2.148244E+10, 2.187518E+10, 2.227443E+10, 2.268029E+10, 
   2.309286E+10, 2.351223E+10, 2.393851E+10, 2.437180E+10, 2.481220E+10, 2.525982E+10, 
   2.571476E+10, 2.617713E+10, 2.664704E+10, 2.712460E+10, 2.760992E+10, 2.810312E+10, 
   2.860430E+10, 2.911359E+10, 2.963110E+10, 3.015696E+10, 3.069127E+10, 3.123417E+10, 
   3.178578E+10, 3.234622E+10, 3.291562E+10, 3.349411E+10, 3.408182E+10, 3.467888E+10, 
   3.528542E+10, 3.590158E+10, 3.652750E+10, 3.716331E+10, 3.780916E+10, 3.846518E+10, 
   3.913153E+10, 3.980834E+10, 4.049576E+10, 4.119395E+10, 4.190305E+10, 4.262322E+10, 
   4.335461E+10, 4.409738E+10, 4.485168E+10, 
])
# ============================ M = 25, I = 1 ============================
M = 25
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.770090E+00, 2.977857E+01, 9.937734E+01, 1.988320E+02, 3.215728E+02, 4.640427E+02, 
   6.242169E+02, 8.010302E+02, 9.940464E+02, 1.203229E+03, 1.428783E+03, 1.671048E+03, 
   1.930438E+03, 2.207401E+03, 2.502407E+03, 2.815933E+03, 3.148468E+03, 3.500509E+03, 
   3.872571E+03, 4.265182E+03, 4.678893E+03, 5.114277E+03, 5.571933E+03, 6.052485E+03, 
   6.556583E+03, 7.084909E+03, 7.638169E+03, 8.217098E+03, 8.822458E+03, 9.455040E+03, 
   1.011566E+04, 1.080516E+04, 1.152440E+04, 1.227429E+04, 1.305573E+04, 1.386967E+04, 
   1.471706E+04, 1.559891E+04, 1.651621E+04, 1.747000E+04, 1.846133E+04, 1.949127E+04, 
   2.056093E+04, 2.167142E+04, 2.282389E+04, 2.401948E+04, 2.525940E+04, 2.654483E+04, 
   2.787700E+04, 2.925717E+04, 3.068660E+04, 3.216658E+04, 3.369842E+04, 3.528345E+04, 
   3.692304E+04, 3.861856E+04, 4.037142E+04, 4.218303E+04, 4.405485E+04, 4.598834E+04, 
   4.798501E+04, 5.004636E+04, 5.217394E+04, 5.436932E+04, 5.663409E+04, 5.896986E+04, 
   6.137826E+04, 6.386098E+04, 6.641968E+04, 6.905610E+04, 7.177196E+04, 7.456904E+04, 
   7.744912E+04, 8.041402E+04, 8.346559E+04, 8.660568E+04, 8.983619E+04, 9.315904E+04, 
   9.657618E+04, 1.000896E+05, 1.037012E+05, 1.074131E+05, 1.112273E+05, 1.151459E+05, 
   1.191709E+05, 1.233046E+05, 1.275489E+05, 1.319062E+05, 1.363785E+05, 1.409681E+05, 
   1.456772E+05, 1.505081E+05, 1.554631E+05, 1.605443E+05, 1.657543E+05, 1.710952E+05, 
   1.765695E+05, 1.821794E+05, 1.879275E+05, 1.938161E+05, 1.998476E+05, 2.060245E+05, 
   2.123492E+05, 2.188242E+05, 2.254519E+05, 2.322350E+05, 2.391757E+05, 2.462767E+05, 
   2.535406E+05, 2.609697E+05, 2.685667E+05, 2.763341E+05, 2.842744E+05, 2.923903E+05, 
   3.006842E+05, 3.091588E+05, 3.178165E+05, 3.266600E+05, 3.356919E+05, 3.449147E+05, 
   3.543309E+05, 3.639431E+05, 3.737539E+05, 3.837659E+05, 3.939815E+05, 4.044034E+05, 
   4.150340E+05, 4.258759E+05, 4.369316E+05, 4.482036E+05, 4.596945E+05, 4.714066E+05, 
   4.833425E+05, 4.955047E+05, 5.078955E+05, 5.205174E+05, 5.333729E+05, 5.464642E+05, 
   5.597939E+05, 5.733642E+05, 5.871776E+05, 6.012362E+05, 6.155426E+05, 6.300988E+05, 
   6.449072E+05, 6.599701E+05, 6.752896E+05, 6.908679E+05, 7.067073E+05, 7.228098E+05, 
   7.391776E+05, 7.558128E+05, 7.727174E+05, 7.898935E+05, 8.073431E+05, 8.250683E+05, 
   8.430708E+05, 8.613528E+05, 8.799161E+05, 8.987625E+05, 9.178940E+05, 9.373123E+05, 
   9.570193E+05, 9.770166E+05, 9.973060E+05, 1.017889E+06, 1.038768E+06, 1.059944E+06, 
   1.081418E+06, 1.103193E+06, 1.125270E+06, 1.147650E+06, 1.170335E+06, 1.193326E+06, 
   1.216625E+06, 1.240232E+06, 1.264150E+06, 1.288380E+06, 1.312923E+06, 1.337780E+06, 
   1.362953E+06, 1.388442E+06, 1.414249E+06, 1.440375E+06, 1.466821E+06, 1.493588E+06, 
   1.520677E+06, 1.548090E+06, 1.575826E+06, 1.603887E+06, 1.632274E+06, 1.660988E+06, 
   1.690030E+06, 1.719399E+06, 1.749098E+06, 1.779127E+06, 1.809486E+06, 1.840177E+06, 
   1.871200E+06, 1.902555E+06, 1.934243E+06, 1.966264E+06, 1.998620E+06, 2.031311E+06, 
   2.064337E+06, 2.097698E+06, 2.131395E+06, 2.165429E+06, 2.199799E+06, 2.234507E+06, 
   2.269551E+06, 2.304934E+06, 2.340654E+06, 2.376712E+06, 2.413108E+06, 2.449842E+06, 
   2.486916E+06, 2.524327E+06, 2.562078E+06, 2.600167E+06, 2.638594E+06, 2.677361E+06, 
   2.716466E+06, 2.755910E+06, 2.795692E+06, 2.835813E+06, 2.876272E+06, 2.917069E+06, 
   2.958205E+06, 2.999678E+06, 3.041488E+06, 3.083637E+06, 3.126122E+06, 3.168944E+06, 
   3.212102E+06, 3.255597E+06, 3.299428E+06, 3.343594E+06, 3.388095E+06, 3.432931E+06, 
   3.478101E+06, 3.523605E+06, 3.569442E+06, 3.615612E+06, 3.662114E+06, 3.708948E+06, 
   3.756113E+06, 3.803609E+06, 3.851435E+06, 3.899591E+06, 3.948076E+06, 3.996889E+06, 
   4.046029E+06, 4.095497E+06, 4.145290E+06, 4.195410E+06, 4.245854E+06, 4.296622E+06, 
   4.347714E+06, 4.399129E+06, 4.450865E+06, 4.502923E+06, 4.555301E+06, 4.607998E+06, 
   4.661014E+06, 4.714348E+06, 4.767999E+06, 4.821966E+06, 4.876249E+06, 4.930845E+06, 
   4.985756E+06, 5.040979E+06, 5.096513E+06, 5.152359E+06, 5.208514E+06, 5.264978E+06, 
   5.321749E+06, 5.378828E+06, 5.436213E+06, 5.493903E+06, 5.551896E+06, 5.610193E+06, 
   5.668791E+06, 5.727690E+06, 5.786890E+06, 5.846387E+06, 5.906183E+06, 5.966275E+06, 
   6.026663E+06, 6.087346E+06, 6.148322E+06, 6.209590E+06, 6.271150E+06, 6.333000E+06, 
   6.395138E+06, 6.457565E+06, 6.520279E+06, 6.583278E+06, 6.646561E+06, 6.710129E+06, 
   6.773978E+06, 6.838108E+06, 6.902519E+06, 6.967208E+06, 7.032175E+06, 7.097418E+06, 
   7.162937E+06, 7.228729E+06, 7.294795E+06, 7.361132E+06, 7.427739E+06, 7.494616E+06, 
   7.561761E+06, 7.629173E+06, 7.696851E+06, 7.764793E+06, 7.832999E+06, 7.901466E+06, 
   7.970195E+06, 8.039183E+06, 8.108429E+06, 8.177933E+06, 8.247692E+06, 8.317706E+06, 
   8.387974E+06, 8.458494E+06, 8.529265E+06, 8.600286E+06, 8.671555E+06, 8.743072E+06, 
   8.814834E+06, 8.886842E+06, 8.959093E+06, 9.031586E+06, 9.104320E+06, 9.177295E+06, 
   9.250507E+06, 9.323958E+06, 9.397644E+06, 9.471565E+06, 9.545720E+06, 9.620107E+06, 
   9.694725E+06, 9.769573E+06, 9.844650E+06, 9.919954E+06, 9.995484E+06, 1.007124E+07, 
   1.014722E+07, 1.022342E+07, 1.029984E+07, 1.037648E+07, 1.045334E+07, 1.053042E+07, 
   1.060772E+07, 1.068522E+07, 1.076295E+07, 1.084088E+07, 1.091903E+07, 1.099739E+07, 
   1.107595E+07, 1.115472E+07, 1.123370E+07, 1.131289E+07, 1.139227E+07, 1.147186E+07, 
   1.155166E+07, 1.163165E+07, 1.171184E+07, 1.179223E+07, 1.187281E+07, 1.195359E+07, 
   1.203457E+07, 1.211573E+07, 1.219709E+07, 1.227864E+07, 1.236038E+07, 1.244231E+07, 
   1.252442E+07, 1.260672E+07, 1.268921E+07, 1.277187E+07, 1.285472E+07, 1.293775E+07, 
   1.302096E+07, 1.310435E+07, 1.318792E+07, 1.327166E+07, 1.335558E+07, 1.343967E+07, 
   1.352393E+07, 1.360836E+07, 1.369297E+07, 1.377774E+07, 1.386269E+07, 1.394780E+07, 
   1.403307E+07, 1.411851E+07, 1.420412E+07, 1.428988E+07, 1.437581E+07, 1.446190E+07, 
   1.454814E+07, 1.463455E+07, 1.472111E+07, 1.480782E+07, 1.489470E+07, 1.498172E+07, 
   1.506890E+07, 1.515622E+07, 1.524370E+07, 1.533133E+07, 1.541910E+07, 1.550702E+07, 
   1.559509E+07, 1.568330E+07, 1.577166E+07, 1.586016E+07, 1.594880E+07, 1.603758E+07, 
   1.612649E+07, 1.621555E+07, 1.630475E+07, 1.639408E+07, 1.648354E+07, 1.657314E+07, 
   1.666287E+07, 1.675274E+07, 1.684273E+07, 1.693286E+07, 1.702311E+07, 1.711349E+07, 
   1.720400E+07, 1.729463E+07, 1.738539E+07, 1.747628E+07, 1.756728E+07, 1.765841E+07, 
   1.774966E+07, 1.784102E+07, 1.793251E+07, 1.802412E+07, 1.811584E+07, 1.820767E+07, 
   1.829962E+07, 1.839169E+07, 1.848387E+07, 1.857616E+07, 1.866856E+07, 1.876107E+07, 
   1.885369E+07, 1.894642E+07, 1.903926E+07, 1.913220E+07, 1.922525E+07, 1.931840E+07, 
   1.941166E+07, 1.950501E+07, 1.959847E+07, 1.969203E+07, 1.978569E+07, 1.987945E+07, 
   1.997331E+07, 2.006726E+07, 2.016131E+07, 2.025546E+07, 2.034970E+07, 2.044403E+07, 
   2.053846E+07, 2.063298E+07, 2.072759E+07, 2.082229E+07, 2.091708E+07, 2.101195E+07, 
   2.110692E+07, 2.120197E+07, 2.129710E+07, 2.139232E+07, 2.148763E+07, 2.158302E+07, 
   2.167849E+07, 2.177404E+07, 2.186967E+07, 2.196539E+07, 2.206118E+07, 2.215705E+07, 
   2.225300E+07, 2.234902E+07, 2.244512E+07, 2.254130E+07, 2.263755E+07, 2.273387E+07, 
   2.283027E+07, 2.292673E+07, 2.302327E+07, 2.311988E+07, 2.321656E+07, 2.331331E+07, 
   2.341012E+07, 2.350701E+07, 2.360396E+07, 2.370097E+07, 2.379805E+07, 2.389519E+07, 
   2.399240E+07, 2.408967E+07, 2.418700E+07, 2.428440E+07, 2.438185E+07, 2.447937E+07, 
   2.457694E+07, 2.467457E+07, 2.477226E+07, 2.487000E+07, 2.496781E+07, 2.506566E+07, 
   2.516358E+07, 2.526154E+07, 2.535956E+07, 2.545763E+07, 2.555576E+07, 2.565394E+07, 
   2.575216E+07, 2.585044E+07, 2.594876E+07, 2.604714E+07, 2.614556E+07, 2.624403E+07, 
   2.634255E+07, 2.644111E+07, 2.653972E+07, 2.663837E+07, 2.673707E+07, 2.683581E+07, 
   2.693459E+07, 2.703341E+07, 2.713228E+07, 2.723119E+07, 2.733013E+07, 2.742912E+07, 
   2.752815E+07, 2.762721E+07, 2.772631E+07, 2.782545E+07, 2.792462E+07, 2.802384E+07, 
   2.812308E+07, 2.822236E+07, 2.832168E+07, 2.842103E+07, 2.852041E+07, 2.861982E+07, 
   2.871927E+07, 2.881875E+07, 2.891825E+07, 2.901779E+07, 2.911736E+07, 2.921695E+07, 
   2.931658E+07, 2.941623E+07, 2.951591E+07, 2.961561E+07, 2.971535E+07, 2.981510E+07, 
   2.991488E+07, 3.001469E+07, 3.011452E+07, 3.021438E+07, 3.031425E+07, 3.041415E+07, 
   3.051407E+07, 3.061401E+07, 3.071398E+07, 3.081396E+07, 3.091396E+07, 3.101398E+07, 
   3.111402E+07, 3.121408E+07, 3.131416E+07, 3.141425E+07, 3.151436E+07, 3.161448E+07, 
   3.171463E+07, 3.181478E+07, 3.191495E+07, 3.201514E+07, 3.211534E+07, 3.221555E+07, 
   3.231577E+07, 3.241601E+07, 3.251626E+07, 3.261652E+07, 3.271679E+07, 3.281707E+07, 
   3.291736E+07, 3.301766E+07, 3.311797E+07, 3.321829E+07, 3.331861E+07, 3.341895E+07, 
   3.351929E+07, 
])
# ============================ M = 26, I = 1 ============================
M = 26
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.304830E+00, 1.250398E+01, 2.430655E+01, 3.611747E+01, 4.793073E+01, 5.974515E+01, 
   7.156039E+01, 8.337674E+01, 9.519594E+01, 1.070230E+02, 1.188684E+02, 1.307497E+02, 
   1.426920E+02, 1.547287E+02, 1.668995E+02, 1.792496E+02, 1.918282E+02, 2.046869E+02, 
   2.178787E+02, 2.314571E+02, 2.454753E+02, 2.599860E+02, 2.750411E+02, 2.906914E+02, 
   3.069871E+02, 3.239775E+02, 3.417113E+02, 3.602366E+02, 3.796013E+02, 3.998532E+02, 
   4.210401E+02, 4.432100E+02, 4.664113E+02, 4.906930E+02, 5.161047E+02, 5.426966E+02, 
   5.705201E+02, 5.996275E+02, 6.300721E+02, 6.619086E+02, 6.951927E+02, 7.299818E+02, 
   7.663344E+02, 8.043110E+02, 8.439731E+02, 8.853843E+02, 9.286097E+02, 9.737164E+02, 
   1.020773E+03, 1.069851E+03, 1.121022E+03, 1.174361E+03, 1.229946E+03, 1.287854E+03, 
   1.348169E+03, 1.410972E+03, 1.476351E+03, 1.544393E+03, 1.615189E+03, 1.688832E+03, 
   1.765420E+03, 1.845049E+03, 1.927822E+03, 2.013844E+03, 2.103220E+03, 2.196062E+03, 
   2.292482E+03, 2.392597E+03, 2.496526E+03, 2.604391E+03, 2.716320E+03, 2.832440E+03, 
   2.952886E+03, 3.077794E+03, 3.207303E+03, 3.341558E+03, 3.480706E+03, 3.624900E+03, 
   3.774293E+03, 3.929048E+03, 4.089326E+03, 4.255296E+03, 4.427131E+03, 4.605007E+03, 
   4.789107E+03, 4.979615E+03, 5.176724E+03, 5.380628E+03, 5.591528E+03, 5.809630E+03, 
   6.035144E+03, 6.268287E+03, 6.509280E+03, 6.758349E+03, 7.015728E+03, 7.281654E+03, 
   7.556371E+03, 7.840129E+03, 8.133184E+03, 8.435798E+03, 8.748238E+03, 9.070780E+03, 
   9.403705E+03, 9.747299E+03, 1.010186E+04, 1.046768E+04, 1.084508E+04, 1.123437E+04, 
   1.163587E+04, 1.204991E+04, 1.247683E+04, 1.291697E+04, 1.337069E+04, 1.383835E+04, 
   1.432031E+04, 1.481695E+04, 1.532865E+04, 1.585582E+04, 1.639884E+04, 1.695813E+04, 
   1.753411E+04, 1.812721E+04, 1.873786E+04, 1.936650E+04, 2.001359E+04, 2.067960E+04, 
   2.136500E+04, 2.207026E+04, 2.279588E+04, 2.354237E+04, 2.431024E+04, 2.509999E+04, 
   2.591218E+04, 2.674733E+04, 2.760600E+04, 2.848876E+04, 2.939617E+04, 3.032881E+04, 
   3.128728E+04, 3.227219E+04, 3.328414E+04, 3.432377E+04, 3.539170E+04, 3.648859E+04, 
   3.761510E+04, 3.877188E+04, 3.995963E+04, 4.117903E+04, 4.243078E+04, 4.371560E+04, 
   4.503422E+04, 4.638736E+04, 4.777578E+04, 4.920023E+04, 5.066149E+04, 5.216033E+04, 
   5.369755E+04, 5.527395E+04, 5.689035E+04, 5.854758E+04, 6.024647E+04, 6.198787E+04, 
   6.377264E+04, 6.560166E+04, 6.747581E+04, 6.939598E+04, 7.136309E+04, 7.337805E+04, 
   7.544178E+04, 7.755523E+04, 7.971935E+04, 8.193511E+04, 8.420347E+04, 8.652542E+04, 
   8.890196E+04, 9.133409E+04, 9.382283E+04, 9.636920E+04, 9.897425E+04, 1.016390E+05, 
   1.043646E+05, 1.071520E+05, 1.100024E+05, 1.129167E+05, 1.158962E+05, 1.189420E+05, 
   1.220551E+05, 1.252366E+05, 1.284878E+05, 1.318098E+05, 1.352037E+05, 1.386708E+05, 
   1.422121E+05, 1.458288E+05, 1.495223E+05, 1.532936E+05, 1.571439E+05, 1.610746E+05, 
   1.650868E+05, 1.691818E+05, 1.733608E+05, 1.776251E+05, 1.819759E+05, 1.864145E+05, 
   1.909421E+05, 1.955602E+05, 2.002699E+05, 2.050725E+05, 2.099695E+05, 2.149620E+05, 
   2.200513E+05, 2.252389E+05, 2.305261E+05, 2.359141E+05, 2.414044E+05, 2.469982E+05, 
   2.526970E+05, 2.585021E+05, 2.644148E+05, 2.704366E+05, 2.765688E+05, 2.828128E+05, 
   2.891699E+05, 2.956416E+05, 3.022293E+05, 3.089343E+05, 3.157581E+05, 3.227020E+05, 
   3.297675E+05, 3.369560E+05, 3.442689E+05, 3.517075E+05, 3.592735E+05, 3.669680E+05, 
   3.747927E+05, 3.827489E+05, 3.908380E+05, 3.990615E+05, 4.074208E+05, 4.159173E+05, 
   4.245525E+05, 4.333279E+05, 4.422448E+05, 4.513047E+05, 4.605090E+05, 4.698592E+05, 
   4.793568E+05, 4.890031E+05, 4.987996E+05, 5.087477E+05, 5.188490E+05, 5.291047E+05, 
   5.395165E+05, 5.500857E+05, 5.608137E+05, 5.717020E+05, 5.827520E+05, 5.939652E+05, 
   6.053430E+05, 6.168868E+05, 6.285981E+05, 6.404783E+05, 6.525288E+05, 6.647510E+05, 
   6.771464E+05, 6.897164E+05, 7.024624E+05, 7.153858E+05, 7.284880E+05, 7.417704E+05, 
   7.552345E+05, 7.688816E+05, 7.827132E+05, 7.967306E+05, 8.109352E+05, 8.253284E+05, 
   8.399115E+05, 8.546861E+05, 8.696534E+05, 8.848148E+05, 9.001717E+05, 9.157254E+05, 
   9.314773E+05, 9.474287E+05, 9.635811E+05, 9.799356E+05, 9.964938E+05, 1.013257E+06, 
   1.030226E+06, 1.047403E+06, 1.064788E+06, 1.082384E+06, 1.100191E+06, 1.118211E+06, 
   1.136445E+06, 1.154895E+06, 1.173561E+06, 1.192445E+06, 1.211548E+06, 1.230871E+06, 
   1.250416E+06, 1.270184E+06, 1.290176E+06, 1.310393E+06, 1.330837E+06, 1.351509E+06, 
   1.372410E+06, 1.393541E+06, 1.414904E+06, 1.436499E+06, 1.458328E+06, 1.480392E+06, 
   1.502692E+06, 1.525229E+06, 1.548005E+06, 1.571020E+06, 1.594276E+06, 1.617774E+06, 
   1.641515E+06, 1.665500E+06, 1.689730E+06, 1.714206E+06, 1.738930E+06, 1.763902E+06, 
   1.789124E+06, 1.814597E+06, 1.840321E+06, 1.866297E+06, 1.892528E+06, 1.919013E+06, 
   1.945754E+06, 1.972752E+06, 2.000007E+06, 2.027522E+06, 2.055296E+06, 2.083331E+06, 
   2.111627E+06, 2.140187E+06, 2.169010E+06, 2.198097E+06, 2.227450E+06, 2.257070E+06, 
   2.286957E+06, 2.317112E+06, 2.347537E+06, 2.378231E+06, 2.409197E+06, 2.440434E+06, 
   2.471944E+06, 2.503728E+06, 2.535786E+06, 2.568119E+06, 2.600728E+06, 2.633614E+06, 
   2.666778E+06, 2.700221E+06, 2.733942E+06, 2.767944E+06, 2.802227E+06, 2.836791E+06, 
   2.871638E+06, 2.906768E+06, 2.942181E+06, 2.977879E+06, 3.013863E+06, 3.050132E+06, 
   3.086689E+06, 3.123532E+06, 3.160664E+06, 3.198085E+06, 3.235795E+06, 3.273795E+06, 
   3.312086E+06, 3.350668E+06, 3.389542E+06, 3.428709E+06, 3.468170E+06, 3.507924E+06, 
   3.547972E+06, 3.588316E+06, 3.628955E+06, 3.669890E+06, 3.711122E+06, 3.752652E+06, 
   3.794479E+06, 3.836604E+06, 3.879029E+06, 3.921752E+06, 3.964776E+06, 4.008100E+06, 
   4.051725E+06, 4.095652E+06, 4.139880E+06, 4.184410E+06, 4.229243E+06, 4.274380E+06, 
   4.319820E+06, 4.365564E+06, 4.411612E+06, 4.457966E+06, 4.504625E+06, 4.551589E+06, 
   4.598859E+06, 4.646436E+06, 4.694320E+06, 4.742511E+06, 4.791009E+06, 4.839815E+06, 
   4.888930E+06, 4.938352E+06, 4.988084E+06, 5.038125E+06, 5.088475E+06, 5.139135E+06, 
   5.190105E+06, 5.241385E+06, 5.292976E+06, 5.344878E+06, 5.397090E+06, 5.449614E+06, 
   5.502450E+06, 5.555597E+06, 5.609057E+06, 5.662828E+06, 5.716912E+06, 5.771309E+06, 
   5.826018E+06, 5.881040E+06, 5.936376E+06, 5.992024E+06, 6.047987E+06, 6.104263E+06, 
   6.160852E+06, 6.217756E+06, 6.274973E+06, 6.332505E+06, 6.390351E+06, 6.448511E+06, 
   6.506986E+06, 6.565776E+06, 6.624880E+06, 6.684298E+06, 6.744032E+06, 6.804080E+06, 
   6.864444E+06, 6.925122E+06, 6.986115E+06, 7.047424E+06, 7.109047E+06, 7.170986E+06, 
   7.233239E+06, 7.295808E+06, 7.358692E+06, 7.421891E+06, 7.485405E+06, 7.549234E+06, 
   7.613378E+06, 7.677837E+06, 7.742611E+06, 7.807700E+06, 7.873104E+06, 7.938823E+06, 
   8.004857E+06, 8.071205E+06, 8.137868E+06, 8.204845E+06, 8.272137E+06, 8.339743E+06, 
   8.407663E+06, 8.475898E+06, 8.544446E+06, 8.613309E+06, 8.682485E+06, 8.751975E+06, 
   8.821778E+06, 8.891895E+06, 8.962325E+06, 9.033068E+06, 9.104124E+06, 9.175493E+06, 
   9.247174E+06, 9.319167E+06, 9.391473E+06, 9.464091E+06, 9.537021E+06, 9.610262E+06, 
   9.683815E+06, 9.757679E+06, 9.831853E+06, 9.906339E+06, 9.981135E+06, 1.005624E+07, 
   1.013166E+07, 1.020738E+07, 1.028342E+07, 1.035977E+07, 1.043642E+07, 1.051338E+07, 
   1.059066E+07, 1.066824E+07, 1.074612E+07, 
])
# ============================ M = 26, I = 2 ============================
M = 26
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   9.038610E+00, 5.002244E+01, 9.724841E+01, 1.445173E+02, 1.918051E+02, 2.391069E+02, 
   2.864215E+02, 3.337499E+02, 3.810993E+02, 4.284898E+02, 4.759633E+02, 5.235902E+02, 
   5.714723E+02, 6.197426E+02, 6.685611E+02, 7.181097E+02, 7.685861E+02, 8.201983E+02, 
   8.731597E+02, 9.276860E+02, 9.839923E+02, 1.042292E+03, 1.102794E+03, 1.165705E+03, 
   1.231229E+03, 1.299565E+03, 1.370912E+03, 1.445466E+03, 1.523421E+03, 1.604972E+03, 
   1.690315E+03, 1.779645E+03, 1.873161E+03, 1.971061E+03, 2.073550E+03, 2.180833E+03, 
   2.293119E+03, 2.410624E+03, 2.533565E+03, 2.662166E+03, 2.796657E+03, 2.937272E+03, 
   3.084251E+03, 3.237843E+03, 3.398300E+03, 3.565882E+03, 3.740858E+03, 3.923503E+03, 
   4.114100E+03, 4.312938E+03, 4.520318E+03, 4.736546E+03, 4.961938E+03, 5.196819E+03, 
   5.441525E+03, 5.696398E+03, 5.961792E+03, 6.238071E+03, 6.525608E+03, 6.824787E+03, 
   7.136004E+03, 7.459665E+03, 7.796187E+03, 8.146000E+03, 8.509544E+03, 8.887273E+03, 
   9.279654E+03, 9.687163E+03, 1.011029E+04, 1.054955E+04, 1.100545E+04, 1.147853E+04, 
   1.196934E+04, 1.247843E+04, 1.300639E+04, 1.355381E+04, 1.412129E+04, 1.470945E+04, 
   1.531895E+04, 1.595044E+04, 1.660458E+04, 1.728208E+04, 1.798365E+04, 1.871000E+04, 
   1.946190E+04, 2.024009E+04, 2.104538E+04, 2.187856E+04, 2.274046E+04, 2.363193E+04, 
   2.455383E+04, 2.550705E+04, 2.649250E+04, 2.751112E+04, 2.856386E+04, 2.965170E+04, 
   3.077565E+04, 3.193672E+04, 3.313599E+04, 3.437451E+04, 3.565339E+04, 3.697377E+04, 
   3.833680E+04, 3.974367E+04, 4.119557E+04, 4.269376E+04, 4.423951E+04, 4.583410E+04, 
   4.747887E+04, 4.917518E+04, 5.092441E+04, 5.272799E+04, 5.458736E+04, 5.650403E+04, 
   5.847949E+04, 6.051532E+04, 6.261309E+04, 6.477443E+04, 6.700100E+04, 6.929450E+04, 
   7.165665E+04, 7.408924E+04, 7.659406E+04, 7.917297E+04, 8.182786E+04, 8.456065E+04, 
   8.737333E+04, 9.026789E+04, 9.324640E+04, 9.631096E+04, 9.946370E+04, 1.027068E+05, 
   1.060426E+05, 1.094732E+05, 1.130010E+05, 1.166285E+05, 1.203579E+05, 1.241919E+05, 
   1.281329E+05, 1.321835E+05, 1.363463E+05, 1.406241E+05, 1.450195E+05, 1.495353E+05, 
   1.541744E+05, 1.589397E+05, 1.638342E+05, 1.688609E+05, 1.740228E+05, 1.793231E+05, 
   1.847650E+05, 1.903517E+05, 1.960867E+05, 2.019732E+05, 2.080148E+05, 2.142149E+05, 
   2.205771E+05, 2.271052E+05, 2.338029E+05, 2.406738E+05, 2.477220E+05, 2.549513E+05, 
   2.623658E+05, 2.699695E+05, 2.777667E+05, 2.857616E+05, 2.939584E+05, 3.023617E+05, 
   3.109758E+05, 3.198054E+05, 3.288550E+05, 3.381296E+05, 3.476337E+05, 3.573724E+05, 
   3.673507E+05, 3.775736E+05, 3.880463E+05, 3.987741E+05, 4.097623E+05, 4.210165E+05, 
   4.325420E+05, 4.443446E+05, 4.564301E+05, 4.688042E+05, 4.814730E+05, 4.944424E+05, 
   5.077186E+05, 5.213080E+05, 5.352168E+05, 5.494515E+05, 5.640187E+05, 5.789252E+05, 
   5.941777E+05, 6.097831E+05, 6.257486E+05, 6.420812E+05, 6.587882E+05, 6.758771E+05, 
   6.933553E+05, 7.112306E+05, 7.295107E+05, 7.482035E+05, 7.673170E+05, 7.868594E+05, 
   8.068390E+05, 8.272642E+05, 8.481436E+05, 8.694859E+05, 8.913000E+05, 9.135948E+05, 
   9.363795E+05, 9.596633E+05, 9.834557E+05, 1.007766E+06, 1.032605E+06, 1.057981E+06, 
   1.083905E+06, 1.110387E+06, 1.137437E+06, 1.165066E+06, 1.193285E+06, 1.222104E+06, 
   1.251535E+06, 1.281588E+06, 1.312275E+06, 1.343607E+06, 1.375597E+06, 1.408255E+06, 
   1.441595E+06, 1.475628E+06, 1.510366E+06, 1.545823E+06, 1.582011E+06, 1.618944E+06, 
   1.656633E+06, 1.695093E+06, 1.734338E+06, 1.774380E+06, 1.815235E+06, 1.856916E+06, 
   1.899437E+06, 1.942814E+06, 1.987060E+06, 2.032192E+06, 2.078224E+06, 2.125172E+06, 
   2.173052E+06, 2.221878E+06, 2.271669E+06, 2.322439E+06, 2.374206E+06, 2.426986E+06, 
   2.480797E+06, 2.535656E+06, 2.591580E+06, 2.648587E+06, 2.706696E+06, 2.765924E+06, 
   2.826291E+06, 2.887814E+06, 2.950514E+06, 3.014409E+06, 3.079519E+06, 3.145865E+06, 
   3.213465E+06, 3.282342E+06, 3.352514E+06, 3.424005E+06, 3.496834E+06, 3.571023E+06, 
   3.646594E+06, 3.723570E+06, 3.801973E+06, 3.881825E+06, 3.963150E+06, 4.045971E+06, 
   4.130312E+06, 4.216197E+06, 4.303650E+06, 4.392696E+06, 4.483359E+06, 4.575666E+06, 
   4.669641E+06, 4.765311E+06, 4.862702E+06, 4.961840E+06, 5.062754E+06, 5.165469E+06, 
   5.270014E+06, 5.376417E+06, 5.484706E+06, 5.594910E+06, 5.707059E+06, 5.821181E+06, 
   5.937306E+06, 6.055466E+06, 6.175690E+06, 6.298009E+06, 6.422456E+06, 6.549062E+06, 
   6.677858E+06, 6.808879E+06, 6.942156E+06, 7.077723E+06, 7.215615E+06, 7.355865E+06, 
   7.498509E+06, 7.643580E+06, 7.791116E+06, 7.941152E+06, 8.093724E+06, 8.248869E+06, 
   8.406625E+06, 8.567030E+06, 8.730121E+06, 8.895938E+06, 9.064519E+06, 9.235905E+06, 
   9.410136E+06, 9.587251E+06, 9.767293E+06, 9.950303E+06, 1.013632E+07, 1.032540E+07, 
   1.051756E+07, 1.071287E+07, 1.091136E+07, 1.111308E+07, 1.131808E+07, 1.152639E+07, 
   1.173806E+07, 1.195315E+07, 1.217170E+07, 1.239376E+07, 1.261937E+07, 1.284859E+07, 
   1.308146E+07, 1.331803E+07, 1.355836E+07, 1.380250E+07, 1.405050E+07, 1.430240E+07, 
   1.455827E+07, 1.481816E+07, 1.508212E+07, 1.535021E+07, 1.562248E+07, 1.589898E+07, 
   1.617979E+07, 1.646494E+07, 1.675451E+07, 1.704854E+07, 1.734711E+07, 1.765026E+07, 
   1.795807E+07, 1.827058E+07, 1.858787E+07, 1.890999E+07, 1.923701E+07, 1.956900E+07, 
   1.990601E+07, 2.024812E+07, 2.059538E+07, 2.094787E+07, 2.130566E+07, 2.166881E+07, 
   2.203739E+07, 2.241148E+07, 2.279114E+07, 2.317644E+07, 2.356745E+07, 2.396426E+07, 
   2.436693E+07, 2.477554E+07, 2.519016E+07, 2.561086E+07, 2.603774E+07, 2.647086E+07, 
   2.691030E+07, 2.735614E+07, 2.780846E+07, 2.826735E+07, 2.873288E+07, 2.920514E+07, 
   2.968422E+07, 3.017018E+07, 3.066313E+07, 3.116315E+07, 3.167032E+07, 3.218473E+07, 
   3.270648E+07, 3.323564E+07, 3.377232E+07, 3.431661E+07, 3.486859E+07, 3.542836E+07, 
   3.599602E+07, 3.657166E+07, 3.715537E+07, 3.774726E+07, 3.834743E+07, 3.895596E+07, 
   3.957298E+07, 4.019856E+07, 4.083282E+07, 4.147587E+07, 4.212780E+07, 4.278872E+07, 
   4.345874E+07, 4.413796E+07, 4.482650E+07, 4.552446E+07, 4.623196E+07, 4.694911E+07, 
   4.767602E+07, 4.841280E+07, 4.915957E+07, 4.991645E+07, 5.068356E+07, 5.146100E+07, 
   5.224892E+07, 5.304741E+07, 5.385662E+07, 5.467665E+07, 5.550764E+07, 5.634971E+07, 
   5.720298E+07, 5.806760E+07, 5.894368E+07, 5.983136E+07, 6.073076E+07, 6.164203E+07, 
   6.256530E+07, 6.350070E+07, 6.444836E+07, 6.540844E+07, 6.638107E+07, 6.736638E+07, 
   6.836453E+07, 6.937565E+07, 7.039989E+07, 7.143740E+07, 7.248833E+07, 7.355282E+07, 
   7.463102E+07, 7.572309E+07, 7.682918E+07, 7.794945E+07, 7.908405E+07, 8.023313E+07, 
   8.139687E+07, 8.257541E+07, 8.376893E+07, 8.497758E+07, 8.620153E+07, 8.744094E+07, 
   8.869600E+07, 8.996685E+07, 9.125368E+07, 9.255666E+07, 9.387596E+07, 9.521176E+07, 
   9.656424E+07, 9.793357E+07, 9.931994E+07, 1.007235E+08, 1.021445E+08, 1.035831E+08, 
   1.050394E+08, 1.065138E+08, 1.080062E+08, 1.095170E+08, 1.110464E+08, 1.125945E+08, 
   1.141615E+08, 1.157477E+08, 1.173532E+08, 1.189782E+08, 1.206230E+08, 1.222877E+08, 
   1.239726E+08, 1.256778E+08, 1.274037E+08, 1.291503E+08, 1.309179E+08, 1.327067E+08, 
   1.345170E+08, 1.363490E+08, 1.382028E+08, 1.400788E+08, 1.419771E+08, 1.438980E+08, 
   1.458416E+08, 1.478083E+08, 1.497983E+08, 1.518117E+08, 1.538490E+08, 1.559102E+08, 
   1.579956E+08, 1.601055E+08, 1.622401E+08, 
])
# ============================ M = 26, I = 3 ============================
M = 26
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.079366E+00, 4.412216E+01, 8.616565E+01, 1.282389E+02, 1.703269E+02, 2.124266E+02, 
   2.545389E+02, 2.966716E+02, 3.388507E+02, 3.811314E+02, 4.236068E+02, 4.664096E+02, 
   5.097084E+02, 5.537007E+02, 5.986048E+02, 6.446518E+02, 6.920797E+02, 7.411282E+02, 
   7.920360E+02, 8.450387E+02, 9.003680E+02, 9.582516E+02, 1.018913E+03, 1.082575E+03, 
   1.149455E+03, 1.219772E+03, 1.293742E+03, 1.371586E+03, 1.453522E+03, 1.539774E+03, 
   1.630566E+03, 1.726130E+03, 1.826698E+03, 1.932512E+03, 2.043815E+03, 2.160860E+03, 
   2.283904E+03, 2.413214E+03, 2.549063E+03, 2.691732E+03, 2.841512E+03, 2.998700E+03, 
   3.163606E+03, 3.336546E+03, 3.517849E+03, 3.707854E+03, 3.906908E+03, 4.115372E+03, 
   4.333618E+03, 4.562029E+03, 4.801000E+03, 5.050941E+03, 5.312271E+03, 5.585426E+03, 
   5.870854E+03, 6.169016E+03, 6.480391E+03, 6.805469E+03, 7.144758E+03, 7.498780E+03, 
   7.868074E+03, 8.253196E+03, 8.654717E+03, 9.073228E+03, 9.509336E+03, 9.963667E+03, 
   1.043687E+04, 1.092960E+04, 1.144254E+04, 1.197641E+04, 1.253192E+04, 1.310982E+04, 
   1.371088E+04, 1.433588E+04, 1.498565E+04, 1.566100E+04, 1.636282E+04, 1.709198E+04, 
   1.784938E+04, 1.863597E+04, 1.945270E+04, 2.030057E+04, 2.118058E+04, 2.209378E+04, 
   2.304125E+04, 2.402408E+04, 2.504340E+04, 2.610038E+04, 2.719621E+04, 2.833211E+04, 
   2.950934E+04, 3.072919E+04, 3.199299E+04, 3.330209E+04, 3.465789E+04, 3.606182E+04, 
   3.751535E+04, 3.901999E+04, 4.057727E+04, 4.218879E+04, 4.385617E+04, 4.558107E+04, 
   4.736520E+04, 4.921032E+04, 5.111820E+04, 5.309070E+04, 5.512970E+04, 5.723712E+04, 
   5.941494E+04, 6.166519E+04, 6.398995E+04, 6.639133E+04, 6.887151E+04, 7.143272E+04, 
   7.407724E+04, 7.680741E+04, 7.962561E+04, 8.253429E+04, 8.553596E+04, 8.863316E+04, 
   9.182854E+04, 9.512476E+04, 9.852457E+04, 1.020308E+05, 1.056462E+05, 1.093739E+05, 
   1.132168E+05, 1.171779E+05, 1.212605E+05, 1.254677E+05, 1.298027E+05, 1.342690E+05, 
   1.388700E+05, 1.436092E+05, 1.484901E+05, 1.535164E+05, 1.586918E+05, 1.640202E+05, 
   1.695055E+05, 1.751515E+05, 1.809625E+05, 1.869426E+05, 1.930959E+05, 1.994268E+05, 
   2.059398E+05, 2.126393E+05, 2.195300E+05, 2.266165E+05, 2.339037E+05, 2.413964E+05, 
   2.490997E+05, 2.570186E+05, 2.651584E+05, 2.735244E+05, 2.821219E+05, 2.909566E+05, 
   3.000340E+05, 3.093599E+05, 3.189401E+05, 3.287807E+05, 3.388878E+05, 3.492674E+05, 
   3.599261E+05, 3.708702E+05, 3.821064E+05, 3.936413E+05, 4.054818E+05, 4.176349E+05, 
   4.301077E+05, 4.429075E+05, 4.560416E+05, 4.695175E+05, 4.833429E+05, 4.975257E+05, 
   5.120738E+05, 5.269952E+05, 5.422983E+05, 5.579915E+05, 5.740833E+05, 5.905825E+05, 
   6.074979E+05, 6.248386E+05, 6.426138E+05, 6.608329E+05, 6.795053E+05, 6.986410E+05, 
   7.182497E+05, 7.383415E+05, 7.589267E+05, 7.800157E+05, 8.016191E+05, 8.237478E+05, 
   8.464127E+05, 8.696250E+05, 8.933962E+05, 9.177378E+05, 9.426615E+05, 9.681795E+05, 
   9.943038E+05, 1.021047E+06, 1.048422E+06, 1.076440E+06, 1.105117E+06, 1.134463E+06, 
   1.164494E+06, 1.195223E+06, 1.226664E+06, 1.258830E+06, 1.291737E+06, 1.325399E+06, 
   1.359831E+06, 1.395048E+06, 1.431066E+06, 1.467901E+06, 1.505567E+06, 1.544082E+06, 
   1.583462E+06, 1.623723E+06, 1.664883E+06, 1.706959E+06, 1.749968E+06, 1.793928E+06, 
   1.838858E+06, 1.884775E+06, 1.931699E+06, 1.979648E+06, 2.028642E+06, 2.078700E+06, 
   2.129842E+06, 2.182089E+06, 2.235461E+06, 2.289978E+06, 2.345662E+06, 2.402535E+06, 
   2.460618E+06, 2.519934E+06, 2.580504E+06, 2.642352E+06, 2.705502E+06, 2.769976E+06, 
   2.835798E+06, 2.902993E+06, 2.971585E+06, 3.041600E+06, 3.113063E+06, 3.185999E+06, 
   3.260435E+06, 3.336397E+06, 3.413912E+06, 3.493008E+06, 3.573712E+06, 3.656052E+06, 
   3.740057E+06, 3.825756E+06, 3.913178E+06, 4.002353E+06, 4.093312E+06, 4.186085E+06, 
   4.280702E+06, 4.377197E+06, 4.475600E+06, 4.575944E+06, 4.678263E+06, 4.782589E+06, 
   4.888957E+06, 4.997400E+06, 5.107955E+06, 5.220655E+06, 5.335537E+06, 5.452638E+06, 
   5.571993E+06, 5.693641E+06, 5.817619E+06, 5.943966E+06, 6.072721E+06, 6.203923E+06, 
   6.337612E+06, 6.473828E+06, 6.612614E+06, 6.754010E+06, 6.898059E+06, 7.044803E+06, 
   7.194286E+06, 7.346552E+06, 7.501646E+06, 7.659612E+06, 7.820497E+06, 7.984346E+06, 
   8.151208E+06, 8.321128E+06, 8.494157E+06, 8.670342E+06, 8.849733E+06, 9.032380E+06, 
   9.218335E+06, 9.407648E+06, 9.600372E+06, 9.796559E+06, 9.996264E+06, 1.019954E+07, 
   1.040644E+07, 1.061703E+07, 1.083135E+07, 1.104947E+07, 1.127144E+07, 1.149732E+07, 
   1.172718E+07, 1.196106E+07, 1.219904E+07, 1.244117E+07, 1.268752E+07, 1.293815E+07, 
   1.319312E+07, 1.345249E+07, 1.371634E+07, 1.398473E+07, 1.425773E+07, 1.453540E+07, 
   1.481781E+07, 1.510504E+07, 1.539715E+07, 1.569422E+07, 1.599631E+07, 1.630351E+07, 
   1.661588E+07, 1.693349E+07, 1.725644E+07, 1.758479E+07, 1.791861E+07, 1.825800E+07, 
   1.860302E+07, 1.895377E+07, 1.931031E+07, 1.967274E+07, 2.004113E+07, 2.041557E+07, 
   2.079615E+07, 2.118296E+07, 2.157607E+07, 2.197558E+07, 2.238158E+07, 2.279416E+07, 
   2.321341E+07, 2.363942E+07, 2.407229E+07, 2.451212E+07, 2.495899E+07, 2.541300E+07, 
   2.587426E+07, 2.634286E+07, 2.681891E+07, 2.730250E+07, 2.779374E+07, 2.829273E+07, 
   2.879958E+07, 2.931439E+07, 2.983727E+07, 3.036834E+07, 3.090769E+07, 3.145544E+07, 
   3.201171E+07, 3.257660E+07, 3.315024E+07, 3.373274E+07, 3.432421E+07, 3.492478E+07, 
   3.553456E+07, 3.615369E+07, 3.678227E+07, 3.742044E+07, 3.806831E+07, 3.872603E+07, 
   3.939371E+07, 4.007148E+07, 4.075949E+07, 4.145785E+07, 4.216671E+07, 4.288619E+07, 
   4.361645E+07, 4.435760E+07, 4.510980E+07, 4.587319E+07, 4.664791E+07, 4.743410E+07, 
   4.823192E+07, 4.904150E+07, 4.986299E+07, 5.069656E+07, 5.154234E+07, 5.240050E+07, 
   5.327119E+07, 5.415457E+07, 5.505080E+07, 5.596003E+07, 5.688243E+07, 5.781816E+07, 
   5.876739E+07, 5.973029E+07, 6.070703E+07, 6.169777E+07, 6.270269E+07, 6.372196E+07, 
   6.475577E+07, 6.580428E+07, 6.686768E+07, 6.794616E+07, 6.903988E+07, 7.014905E+07, 
   7.127384E+07, 7.241444E+07, 7.357106E+07, 7.474387E+07, 7.593308E+07, 7.713888E+07, 
   7.836147E+07, 7.960106E+07, 8.085784E+07, 8.213202E+07, 8.342381E+07, 8.473341E+07, 
   8.606105E+07, 8.740692E+07, 8.877125E+07, 9.015426E+07, 9.155616E+07, 9.297718E+07, 
   9.441754E+07, 9.587747E+07, 9.735719E+07, 9.885694E+07, 1.003770E+08, 1.019174E+08, 
   1.034787E+08, 1.050609E+08, 1.066643E+08, 1.082892E+08, 1.099358E+08, 1.116043E+08, 
   1.132951E+08, 1.150083E+08, 1.167442E+08, 1.185031E+08, 1.202853E+08, 1.220909E+08, 
   1.239203E+08, 1.257738E+08, 1.276515E+08, 1.295539E+08, 1.314811E+08, 1.334334E+08, 
   1.354112E+08, 1.374147E+08, 1.394442E+08, 1.414999E+08, 1.435822E+08, 1.456914E+08, 
   1.478278E+08, 1.499916E+08, 1.521833E+08, 1.544030E+08, 1.566510E+08, 1.589278E+08, 
   1.612336E+08, 1.635688E+08, 1.659336E+08, 1.683284E+08, 1.707535E+08, 1.732092E+08, 
   1.756959E+08, 1.782139E+08, 1.807635E+08, 1.833452E+08, 1.859591E+08, 1.886058E+08, 
   1.912855E+08, 1.939985E+08, 1.967453E+08, 1.995262E+08, 2.023416E+08, 2.051918E+08, 
   2.080773E+08, 2.109983E+08, 2.139552E+08, 2.169486E+08, 2.199786E+08, 2.230457E+08, 
   2.261504E+08, 2.292929E+08, 2.324737E+08, 2.356932E+08, 2.389518E+08, 2.422500E+08, 
   2.455880E+08, 2.489664E+08, 2.523855E+08, 2.558458E+08, 2.593478E+08, 2.628917E+08, 
   2.664781E+08, 2.701074E+08, 2.737801E+08, 
])
# ============================ M = 27, I = 1 ============================
M = 27
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.500294E+01, 3.318138E+02, 9.183182E+02, 1.678091E+03, 2.577273E+03, 3.597153E+03, 
   4.726621E+03, 5.959861E+03, 7.295001E+03, 8.732893E+03, 1.027611E+04, 1.192824E+04, 
   1.369350E+04, 1.557652E+04, 1.758232E+04, 1.971631E+04, 2.198438E+04, 2.439298E+04, 
   2.694922E+04, 2.966098E+04, 3.253697E+04, 3.558683E+04, 3.882120E+04, 4.225182E+04, 
   4.589158E+04, 4.975458E+04, 5.385624E+04, 5.821335E+04, 6.284415E+04, 6.776843E+04, 
   7.300756E+04, 7.858467E+04, 8.452468E+04, 9.085443E+04, 9.760282E+04, 1.048009E+05, 
   1.124821E+05, 1.206821E+05, 1.294394E+05, 1.387952E+05, 1.487938E+05, 1.594826E+05, 
   1.709122E+05, 1.831372E+05, 1.962158E+05, 2.102105E+05, 2.251881E+05, 2.412204E+05, 
   2.583840E+05, 2.767611E+05, 2.964396E+05, 3.175138E+05, 3.400844E+05, 3.642594E+05, 
   3.901542E+05, 4.178926E+05, 4.476068E+05, 4.794385E+05, 5.135390E+05, 5.500706E+05, 
   5.892066E+05, 6.311326E+05, 6.760469E+05, 7.241617E+05, 7.757040E+05, 8.309162E+05, 
   8.900580E+05, 9.534064E+05, 1.021258E+06, 1.093929E+06, 1.171759E+06, 1.255108E+06, 
   1.344364E+06, 1.439939E+06, 1.542274E+06, 1.651839E+06, 1.769138E+06, 1.894706E+06, 
   2.029117E+06, 2.172982E+06, 2.326954E+06, 2.491728E+06, 2.668048E+06, 2.856707E+06, 
   3.058549E+06, 3.274476E+06, 3.505448E+06, 3.752492E+06, 4.016697E+06, 4.299230E+06, 
   4.601331E+06, 4.924321E+06, 5.269609E+06, 5.638695E+06, 6.033176E+06, 6.454755E+06, 
   6.905242E+06, 7.386566E+06, 7.900779E+06, 8.450067E+06, 9.036753E+06, 9.663310E+06, 
   1.033237E+07, 1.104673E+07, 1.180937E+07, 1.262344E+07, 1.349232E+07, 1.441958E+07, 
   1.540901E+07, 1.646467E+07, 1.759082E+07, 1.879204E+07, 2.007317E+07, 2.143934E+07, 
   2.289601E+07, 2.444898E+07, 2.610439E+07, 2.786878E+07, 2.974908E+07, 3.175263E+07, 
   3.388724E+07, 3.616117E+07, 3.858319E+07, 4.116261E+07, 4.390929E+07, 4.683367E+07, 
   4.994685E+07, 5.326056E+07, 5.678723E+07, 6.054006E+07, 6.453300E+07, 6.878083E+07, 
   7.329921E+07, 7.810472E+07, 8.321491E+07, 8.864834E+07, 9.442468E+07, 1.005647E+08, 
   1.070905E+08, 1.140252E+08, 1.213936E+08, 1.292216E+08, 1.375367E+08, 1.463682E+08, 
   1.557467E+08, 1.657047E+08, 1.762767E+08, 1.874989E+08, 1.994097E+08, 2.120496E+08, 
   2.254614E+08, 2.396902E+08, 2.547838E+08, 2.707925E+08, 2.877695E+08, 3.057709E+08, 
   3.248559E+08, 3.450870E+08, 3.665300E+08, 3.892544E+08, 4.133336E+08, 4.388449E+08, 
   4.658696E+08, 4.944939E+08, 5.248081E+08, 5.569078E+08, 5.908935E+08, 6.268712E+08, 
   6.649525E+08, 7.052550E+08, 7.479025E+08, 7.930254E+08, 8.407611E+08, 8.912540E+08, 
   9.446564E+08, 1.001128E+09, 1.060838E+09, 1.123964E+09, 1.190692E+09, 1.261217E+09, 
   1.335748E+09, 1.414500E+09, 1.497702E+09, 1.585594E+09, 1.678428E+09, 1.776470E+09, 
   1.879998E+09, 1.989304E+09, 2.104698E+09, 2.226501E+09, 2.355054E+09, 2.490713E+09, 
   2.633853E+09, 2.784867E+09, 2.944168E+09, 3.112190E+09, 3.289387E+09, 3.476237E+09, 
   3.673242E+09, 3.880927E+09, 4.099844E+09, 4.330572E+09, 4.573717E+09, 4.829917E+09, 
   5.099839E+09, 5.384183E+09, 5.683683E+09, 5.999108E+09, 6.331265E+09, 6.680999E+09, 
   7.049195E+09, 7.436781E+09, 7.844730E+09, 8.274059E+09, 8.725835E+09, 9.201175E+09, 
   9.701250E+09, 1.022728E+10, 1.078056E+10, 1.136242E+10, 1.197426E+10, 1.261757E+10, 
   1.329387E+10, 1.400478E+10, 1.475198E+10, 1.553724E+10, 1.636239E+10, 1.722937E+10, 
   1.814019E+10, 1.909696E+10, 2.010189E+10, 2.115728E+10, 2.226553E+10, 2.342917E+10, 
   2.465082E+10, 2.593324E+10, 2.727931E+10, 2.869201E+10, 3.017449E+10, 3.173003E+10, 
   3.336204E+10, 3.507409E+10, 3.686993E+10, 3.875344E+10, 4.072869E+10, 4.279993E+10, 
   4.497158E+10, 4.724829E+10, 4.963486E+10, 5.213634E+10, 5.475798E+10, 5.750528E+10, 
   6.038394E+10, 6.339993E+10, 6.655947E+10, 6.986907E+10, 7.333547E+10, 7.696574E+10, 
   8.076724E+10, 8.474763E+10, 8.891490E+10, 9.327739E+10, 9.784379E+10, 1.026231E+11, 
   1.076248E+11, 1.128588E+11, 1.183351E+11, 1.240646E+11, 1.300582E+11, 1.363276E+11, 
   1.428848E+11, 1.497423E+11, 1.569133E+11, 1.644112E+11, 1.722503E+11, 1.804452E+11, 
   1.890114E+11, 1.979647E+11, 2.073218E+11, 2.170999E+11, 2.273170E+11, 2.379919E+11, 
   2.491439E+11, 2.607934E+11, 2.729613E+11, 2.856696E+11, 2.989410E+11, 3.127992E+11, 
   3.272688E+11, 3.423753E+11, 3.581455E+11, 3.746069E+11, 3.917884E+11, 4.097196E+11, 
   4.284318E+11, 4.479570E+11, 4.683290E+11, 4.895824E+11, 5.117535E+11, 5.348798E+11, 
   5.590004E+11, 5.841557E+11, 6.103880E+11, 6.377408E+11, 6.662596E+11, 6.959916E+11, 
   7.269856E+11, 7.592924E+11, 7.929648E+11, 8.280575E+11, 8.646273E+11, 9.027332E+11, 
   9.424362E+11, 9.838000E+11, 1.026890E+12, 1.071775E+12, 1.118526E+12, 1.167216E+12, 
   1.217922E+12, 1.270722E+12, 1.325699E+12, 1.382937E+12, 1.442525E+12, 1.504555E+12, 
   1.569120E+12, 1.636320E+12, 1.706256E+12, 1.779034E+12, 1.854763E+12, 1.933557E+12, 
   2.015532E+12, 2.100812E+12, 2.189522E+12, 2.281794E+12, 2.377761E+12, 2.477565E+12, 
   2.581351E+12, 2.689270E+12, 2.801477E+12, 2.918135E+12, 3.039409E+12, 3.165474E+12, 
   3.296509E+12, 3.432699E+12, 3.574237E+12, 3.721321E+12, 3.874158E+12, 4.032960E+12, 
   4.197948E+12, 4.369351E+12, 4.547404E+12, 4.732353E+12, 4.924450E+12, 5.123956E+12, 
   5.331144E+12, 5.546292E+12, 5.769691E+12, 6.001640E+12, 6.242449E+12, 6.492439E+12, 
   6.751942E+12, 7.021300E+12, 7.300868E+12, 7.591012E+12, 7.892111E+12, 8.204558E+12, 
   8.528758E+12, 8.865128E+12, 9.214102E+12, 9.576127E+12, 9.951665E+12, 1.034119E+13, 
   1.074521E+13, 1.116422E+13, 1.159875E+13, 1.204935E+13, 1.251657E+13, 1.300101E+13, 
   1.350326E+13, 1.402394E+13, 1.456369E+13, 1.512317E+13, 1.570307E+13, 1.630409E+13, 
   1.692696E+13, 1.757244E+13, 1.824129E+13, 1.893432E+13, 1.965236E+13, 2.039627E+13, 
   2.116692E+13, 2.196523E+13, 2.279213E+13, 2.364860E+13, 2.453564E+13, 2.545427E+13, 
   2.640557E+13, 2.739064E+13, 2.841061E+13, 2.946664E+13, 3.055996E+13, 3.169180E+13, 
   3.286345E+13, 3.407624E+13, 3.533152E+13, 3.663073E+13, 3.797530E+13, 3.936673E+13, 
   4.080659E+13, 4.229645E+13, 4.383796E+13, 4.543282E+13, 4.708278E+13, 4.878964E+13, 
   5.055526E+13, 5.238155E+13, 5.427048E+13, 5.622409E+13, 5.824448E+13, 6.033380E+13, 
   6.249429E+13, 6.472824E+13, 6.703800E+13, 6.942603E+13, 7.189482E+13, 7.444698E+13, 
   7.708514E+13, 7.981207E+13, 8.263059E+13, 8.554362E+13, 8.855414E+13, 9.166525E+13, 
   9.488014E+13, 9.820207E+13, 1.016344E+14, 1.051807E+14, 1.088444E+14, 1.126293E+14, 
   1.165391E+14, 1.205778E+14, 1.247494E+14, 1.290580E+14, 1.335078E+14, 1.381033E+14, 
   1.428490E+14, 1.477494E+14, 1.528095E+14, 1.580341E+14, 1.634283E+14, 1.689972E+14, 
   1.747463E+14, 1.806810E+14, 1.868071E+14, 1.931304E+14, 1.996568E+14, 2.063926E+14, 
   2.133440E+14, 2.205178E+14, 2.279205E+14, 2.355591E+14, 2.434407E+14, 2.515727E+14, 
   2.599624E+14, 2.686178E+14, 2.775467E+14, 2.867574E+14, 2.962581E+14, 3.060577E+14, 
   3.161649E+14, 3.265890E+14, 3.373392E+14, 3.484254E+14, 3.598573E+14, 3.716452E+14, 
   3.837997E+14, 3.963315E+14, 4.092516E+14, 4.225715E+14, 4.363029E+14, 4.504578E+14, 
   4.650485E+14, 4.800878E+14, 4.955887E+14, 5.115646E+14, 5.280294E+14, 5.449970E+14, 
   5.624822E+14, 5.804997E+14, 5.990650E+14, 6.181939E+14, 6.379024E+14, 6.582073E+14, 
   6.791255E+14, 7.006747E+14, 7.228728E+14, 7.457384E+14, 7.692904E+14, 7.935483E+14, 
   8.185322E+14, 8.442625E+14, 8.707605E+14, 
])
# ============================ M = 27, I = 2 ============================
M = 27
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.613755E+00, 1.692742E+02, 4.686227E+02, 8.564268E+02, 1.315402E+03, 1.835998E+03, 
   2.412544E+03, 3.042071E+03, 3.723626E+03, 4.457642E+03, 5.245434E+03, 6.088837E+03, 
   6.990003E+03, 7.951299E+03, 8.975284E+03, 1.006472E+04, 1.122262E+04, 1.245228E+04, 
   1.375733E+04, 1.514179E+04, 1.661011E+04, 1.816721E+04, 1.981853E+04, 2.157007E+04, 
   2.342840E+04, 2.540072E+04, 2.749492E+04, 2.971957E+04, 3.208397E+04, 3.459824E+04, 
   3.727329E+04, 4.012094E+04, 4.315390E+04, 4.638589E+04, 4.983167E+04, 5.350710E+04, 
   5.742921E+04, 6.161629E+04, 6.608798E+04, 7.086534E+04, 7.597093E+04, 8.142897E+04, 
   8.726538E+04, 9.350797E+04, 1.001865E+05, 1.073329E+05, 1.149813E+05, 1.231683E+05, 
   1.319330E+05, 1.413176E+05, 1.513668E+05, 1.621288E+05, 1.736550E+05, 1.860007E+05, 
   1.992248E+05, 2.133904E+05, 2.285651E+05, 2.448214E+05, 2.622364E+05, 2.808931E+05, 
   3.008801E+05, 3.222920E+05, 3.452303E+05, 3.698033E+05, 3.961270E+05, 4.243251E+05, 
   4.545304E+05, 4.868844E+05, 5.215384E+05, 5.586544E+05, 5.984052E+05, 6.409756E+05, 
   6.865629E+05, 7.353781E+05, 7.876461E+05, 8.436077E+05, 9.035194E+05, 9.676556E+05, 
   1.036309E+06, 1.109791E+06, 1.188437E+06, 1.272601E+06, 1.362663E+06, 1.459028E+06, 
   1.562127E+06, 1.672422E+06, 1.790403E+06, 1.916593E+06, 2.051552E+06, 2.195873E+06, 
   2.350191E+06, 2.515180E+06, 2.691561E+06, 2.880100E+06, 3.081613E+06, 3.296970E+06, 
   3.527096E+06, 3.772976E+06, 4.035660E+06, 4.316263E+06, 4.615973E+06, 4.936054E+06, 
   5.277850E+06, 5.642791E+06, 6.032397E+06, 6.448285E+06, 6.892174E+06, 7.365891E+06, 
   7.871376E+06, 8.410695E+06, 8.986038E+06, 9.599734E+06, 1.025426E+07, 1.095224E+07, 
   1.169646E+07, 1.248988E+07, 1.333566E+07, 1.423711E+07, 1.519780E+07, 1.622146E+07, 
   1.731209E+07, 1.847391E+07, 1.971141E+07, 2.102934E+07, 2.243274E+07, 2.392694E+07, 
   2.551762E+07, 2.721078E+07, 2.901276E+07, 3.093032E+07, 3.297057E+07, 3.514109E+07, 
   3.744986E+07, 3.990537E+07, 4.251658E+07, 4.529298E+07, 4.824463E+07, 5.138215E+07, 
   5.471678E+07, 5.826044E+07, 6.202570E+07, 6.602587E+07, 7.027502E+07, 7.478803E+07, 
   7.958062E+07, 8.466942E+07, 9.007199E+07, 9.580688E+07, 1.018937E+08, 1.083532E+08, 
   1.152072E+08, 1.224788E+08, 1.301923E+08, 1.383737E+08, 1.470499E+08, 1.562497E+08, 
   1.660034E+08, 1.763429E+08, 1.873018E+08, 1.989158E+08, 2.112222E+08, 2.242606E+08, 
   2.380726E+08, 2.527023E+08, 2.681958E+08, 2.846019E+08, 3.019722E+08, 3.203607E+08, 
   3.398244E+08, 3.604237E+08, 3.822216E+08, 4.052850E+08, 4.296840E+08, 4.554925E+08, 
   4.827884E+08, 5.116534E+08, 5.421738E+08, 5.744402E+08, 6.085479E+08, 6.445975E+08, 
   6.826943E+08, 7.229494E+08, 7.654794E+08, 8.104072E+08, 8.578616E+08, 9.079783E+08, 
   9.608998E+08, 1.016776E+09, 1.075764E+09, 1.138029E+09, 1.203745E+09, 1.273094E+09, 
   1.346267E+09, 1.423467E+09, 1.504904E+09, 1.590799E+09, 1.681386E+09, 1.776909E+09, 
   1.877623E+09, 1.983798E+09, 2.095716E+09, 2.213674E+09, 2.337980E+09, 2.468962E+09, 
   2.606960E+09, 2.752332E+09, 2.905454E+09, 3.066720E+09, 3.236541E+09, 3.415349E+09, 
   3.603599E+09, 3.801763E+09, 4.010340E+09, 4.229851E+09, 4.460840E+09, 4.703879E+09, 
   4.959566E+09, 5.228528E+09, 5.511421E+09, 5.808931E+09, 6.121776E+09, 6.450709E+09, 
   6.796518E+09, 7.160024E+09, 7.542091E+09, 7.943619E+09, 8.365552E+09, 8.808875E+09, 
   9.274620E+09, 9.763865E+09, 1.027774E+10, 1.081742E+10, 1.138413E+10, 1.197918E+10, 
   1.260390E+10, 1.325969E+10, 1.394804E+10, 1.467047E+10, 1.542858E+10, 1.622407E+10, 
   1.705867E+10, 1.793421E+10, 1.885260E+10, 1.981583E+10, 2.082600E+10, 2.188525E+10, 
   2.299587E+10, 2.416023E+10, 2.538078E+10, 2.666011E+10, 2.800090E+10, 2.940596E+10, 
   3.087823E+10, 3.242074E+10, 3.403668E+10, 3.572938E+10, 3.750230E+10, 3.935904E+10, 
   4.130336E+10, 4.333921E+10, 4.547065E+10, 4.770196E+10, 5.003758E+10, 5.248214E+10, 
   5.504045E+10, 5.771756E+10, 6.051869E+10, 6.344930E+10, 6.651507E+10, 6.972192E+10, 
   7.307600E+10, 7.658374E+10, 8.025182E+10, 8.408719E+10, 8.809709E+10, 9.228907E+10, 
   9.667096E+10, 1.012509E+11, 1.060375E+11, 1.110395E+11, 1.162660E+11, 1.217268E+11, 
   1.274318E+11, 1.333912E+11, 1.396159E+11, 1.461171E+11, 1.529064E+11, 1.599960E+11, 
   1.673984E+11, 1.751268E+11, 1.831946E+11, 1.916162E+11, 2.004062E+11, 2.095799E+11, 
   2.191532E+11, 2.291426E+11, 2.395652E+11, 2.504388E+11, 2.617821E+11, 2.736142E+11, 
   2.859550E+11, 2.988254E+11, 3.122468E+11, 3.262417E+11, 3.408333E+11, 3.560457E+11, 
   3.719040E+11, 3.884341E+11, 4.056630E+11, 4.236188E+11, 4.423306E+11, 4.618284E+11, 
   4.821437E+11, 5.033088E+11, 5.253576E+11, 5.483250E+11, 5.722472E+11, 5.971619E+11, 
   6.231081E+11, 6.501264E+11, 6.782586E+11, 7.075484E+11, 7.380409E+11, 7.697828E+11, 
   8.028228E+11, 8.372111E+11, 8.729999E+11, 9.102433E+11, 9.489972E+11, 9.893198E+11, 
   1.031271E+12, 1.074914E+12, 1.120312E+12, 1.167533E+12, 1.216646E+12, 1.267723E+12, 
   1.320838E+12, 1.376069E+12, 1.433495E+12, 1.493199E+12, 1.555267E+12, 1.619786E+12, 
   1.686850E+12, 1.756553E+12, 1.828993E+12, 1.904273E+12, 1.982497E+12, 2.063776E+12, 
   2.148221E+12, 2.235950E+12, 2.327083E+12, 2.421747E+12, 2.520070E+12, 2.622187E+12, 
   2.728236E+12, 2.838360E+12, 2.952708E+12, 3.071434E+12, 3.194696E+12, 3.322659E+12, 
   3.455492E+12, 3.593370E+12, 3.736476E+12, 3.884997E+12, 4.039127E+12, 4.199067E+12, 
   4.365024E+12, 4.537213E+12, 4.715855E+12, 4.901179E+12, 5.093423E+12, 5.292830E+12, 
   5.499655E+12, 5.714157E+12, 5.936607E+12, 6.167284E+12, 6.406476E+12, 6.654480E+12, 
   6.911605E+12, 7.178167E+12, 7.454496E+12, 7.740929E+12, 8.037816E+12, 8.345520E+12, 
   8.664413E+12, 8.994880E+12, 9.337319E+12, 9.692142E+12, 1.005977E+13, 1.044065E+13, 
   1.083522E+13, 1.124395E+13, 1.166733E+13, 1.210585E+13, 1.256002E+13, 1.303038E+13, 
   1.351747E+13, 1.402185E+13, 1.454410E+13, 1.508483E+13, 1.564465E+13, 1.622420E+13, 
   1.682414E+13, 1.744514E+13, 1.808792E+13, 1.875319E+13, 1.944169E+13, 2.015420E+13, 
   2.089150E+13, 2.165443E+13, 2.244380E+13, 2.326051E+13, 2.410543E+13, 2.497950E+13, 
   2.588366E+13, 2.681890E+13, 2.778623E+13, 2.878669E+13, 2.982136E+13, 3.089134E+13, 
   3.199777E+13, 3.314182E+13, 3.432472E+13, 3.554771E+13, 3.681207E+13, 3.811913E+13, 
   3.947026E+13, 4.086685E+13, 4.231037E+13, 4.380230E+13, 4.534417E+13, 4.693758E+13, 
   4.858415E+13, 5.028556E+13, 5.204354E+13, 5.385986E+13, 5.573638E+13, 5.767496E+13, 
   5.967756E+13, 6.174617E+13, 6.388286E+13, 6.608974E+13, 6.836901E+13, 7.072290E+13, 
   7.315372E+13, 7.566387E+13, 7.825578E+13, 8.093198E+13, 8.369506E+13, 8.654770E+13, 
   8.949264E+13, 9.253271E+13, 9.567081E+13, 9.890994E+13, 1.022532E+14, 1.057037E+14, 
   1.092647E+14, 1.129397E+14, 1.167319E+14, 1.206450E+14, 1.246827E+14, 1.288486E+14, 
   1.331466E+14, 1.375808E+14, 1.421551E+14, 1.468738E+14, 1.517411E+14, 1.567616E+14, 
   1.619397E+14, 1.672802E+14, 1.727879E+14, 1.784677E+14, 1.843247E+14, 1.903642E+14, 
   1.965914E+14, 2.030121E+14, 2.096318E+14, 2.164563E+14, 2.234918E+14, 2.307442E+14, 
   2.382201E+14, 2.459259E+14, 2.538682E+14, 2.620540E+14, 2.704903E+14, 2.791844E+14, 
   2.881437E+14, 2.973759E+14, 3.068888E+14, 3.166906E+14, 3.267895E+14, 3.371941E+14, 
   3.479130E+14, 3.589554E+14, 3.703304E+14, 3.820474E+14, 3.941164E+14, 4.065472E+14, 
   4.193500E+14, 4.325356E+14, 4.461146E+14, 
])
# ============================ M = 27, I = 3 ============================
#M = 27
#I = 3
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   6.936192E+01, 1.700790E+03, 4.744332E+03, 8.675783E+03, 1.332721E+04, 1.860613E+04, 
#   2.446259E+04, 3.087723E+04, 3.785166E+04, 4.540032E+04, 5.354483E+04, 6.231068E+04, 
#   7.172576E+04, 8.181995E+04, 9.262527E+04, 1.041764E+05, 1.165112E+05, 1.296713E+05, 
#   1.437028E+05, 1.586566E+05, 1.745892E+05, 1.915630E+05, 2.096469E+05, 2.289166E+05, 
#   2.494555E+05, 2.713544E+05, 2.947129E+05, 3.196395E+05, 3.462520E+05, 3.746783E+05, 
#   4.050570E+05, 4.375383E+05, 4.722842E+05, 5.094698E+05, 5.492839E+05, 5.919303E+05, 
#   6.376282E+05, 6.866139E+05, 7.391416E+05, 7.954850E+05, 8.559384E+05, 9.208183E+05, 
#   9.904651E+05, 1.065245E+06, 1.145550E+06, 1.231805E+06, 1.324463E+06, 1.424012E+06, 
#   1.530977E+06, 1.645920E+06, 1.769449E+06, 1.902211E+06, 2.044906E+06, 2.198282E+06, 
#   2.363146E+06, 2.540360E+06, 2.730853E+06, 2.935619E+06, 3.155728E+06, 3.392327E+06, 
#   3.646645E+06, 3.920002E+06, 4.213815E+06, 4.529601E+06, 4.868990E+06, 5.233729E+06, 
#   5.625688E+06, 6.046875E+06, 6.499442E+06, 6.985693E+06, 7.508100E+06, 8.069308E+06, 
#   8.672152E+06, 9.319669E+06, 1.001511E+07, 1.076196E+07, 1.156394E+07, 1.242504E+07, 
#   1.334954E+07, 1.434200E+07, 1.540730E+07, 1.655068E+07, 1.777774E+07, 1.909444E+07, 
#   2.050719E+07, 2.202282E+07, 2.364864E+07, 2.539245E+07, 2.726260E+07, 2.926800E+07, 
#   3.141817E+07, 3.372328E+07, 3.619418E+07, 3.884247E+07, 4.168051E+07, 4.472152E+07, 
#   4.797958E+07, 5.146973E+07, 5.520800E+07, 5.921149E+07, 6.349845E+07, 6.808832E+07, 
#   7.300183E+07, 7.826107E+07, 8.388959E+07, 8.991247E+07, 9.635644E+07, 1.032500E+08, 
#   1.106234E+08, 1.185089E+08, 1.269409E+08, 1.359560E+08, 1.455930E+08, 1.558935E+08, 
#   1.669015E+08, 1.786637E+08, 1.912301E+08, 2.046537E+08, 2.189907E+08, 2.343012E+08, 
#   2.506486E+08, 2.681007E+08, 2.867294E+08, 3.066109E+08, 3.278263E+08, 3.504617E+08, 
#   3.746085E+08, 4.003637E+08, 4.278304E+08, 4.571178E+08, 4.883418E+08, 5.216256E+08, 
#   5.570995E+08, 5.949020E+08, 6.351798E+08, 6.780884E+08, 7.237927E+08, 7.724676E+08, 
#   8.242981E+08, 8.794806E+08, 9.382227E+08, 1.000745E+09, 1.067280E+09, 1.138075E+09, 
#   1.213391E+09, 1.293505E+09, 1.378709E+09, 1.469314E+09, 1.565646E+09, 1.668054E+09, 
#   1.776902E+09, 1.892580E+09, 2.015496E+09, 2.146085E+09, 2.284804E+09, 2.432138E+09, 
#   2.588599E+09, 2.754726E+09, 2.931091E+09, 3.118297E+09, 3.316980E+09, 3.527814E+09, 
#   3.751508E+09, 3.988812E+09, 4.240516E+09, 4.507456E+09, 4.790512E+09, 5.090613E+09, 
#   5.408739E+09, 5.745924E+09, 6.103257E+09, 6.481888E+09, 6.883028E+09, 7.307954E+09, 
#   7.758011E+09, 8.234619E+09, 8.739270E+09, 9.273539E+09, 9.839083E+09, 1.043765E+10, 
#   1.107107E+10, 1.174129E+10, 1.245035E+10, 1.320038E+10, 1.399364E+10, 1.483252E+10, 
#   1.571950E+10, 1.665723E+10, 1.764846E+10, 1.869609E+10, 1.980320E+10, 2.097300E+10, 
#   2.220886E+10, 2.351434E+10, 2.489318E+10, 2.634929E+10, 2.788681E+10, 2.951007E+10, 
#   3.122361E+10, 3.303222E+10, 3.494092E+10, 3.695498E+10, 3.907993E+10, 4.132158E+10, 
#   4.368605E+10, 4.617972E+10, 4.880931E+10, 5.158187E+10, 5.450480E+10, 5.758585E+10, 
#   6.083316E+10, 6.425526E+10, 6.786110E+10, 7.166007E+10, 7.566199E+10, 7.987718E+10, 
#   8.431644E+10, 8.899110E+10, 9.391302E+10, 9.909465E+10, 1.045490E+11, 1.102897E+11, 
#   1.163311E+11, 1.226881E+11, 1.293764E+11, 1.364123E+11, 1.438132E+11, 1.515969E+11, 
#   1.597823E+11, 1.683890E+11, 1.774378E+11, 1.869501E+11, 1.969484E+11, 2.074565E+11, 
#   2.184989E+11, 2.301015E+11, 2.422913E+11, 2.550965E+11, 2.685465E+11, 2.826723E+11, 
#   2.975061E+11, 3.130816E+11, 3.294339E+11, 3.465999E+11, 3.646180E+11, 3.835284E+11, 
#   4.033730E+11, 4.241956E+11, 4.460421E+11, 4.689601E+11, 4.929995E+11, 5.182125E+11, 
#   5.446534E+11, 5.723790E+11, 6.014485E+11, 6.319237E+11, 6.638692E+11, 6.973522E+11, 
#   7.324430E+11, 7.692148E+11, 8.077440E+11, 8.481104E+11, 8.903970E+11, 9.346905E+11, 
#   9.810813E+11, 1.029664E+12, 1.080535E+12, 1.133799E+12, 1.189562E+12, 1.247934E+12, 
#   1.309033E+12, 1.372978E+12, 1.439895E+12, 1.509915E+12, 1.583176E+12, 1.659818E+12, 
#   1.739990E+12, 1.823847E+12, 1.911548E+12, 2.003261E+12, 2.099160E+12, 2.199427E+12, 
#   2.304249E+12, 2.413823E+12, 2.528353E+12, 2.648052E+12, 2.773141E+12, 2.903849E+12, 
#   3.040418E+12, 3.183094E+12, 3.332138E+12, 3.487818E+12, 3.650416E+12, 3.820222E+12, 
#   3.997538E+12, 4.182681E+12, 4.375977E+12, 4.577767E+12, 4.788404E+12, 5.008257E+12, 
#   5.237706E+12, 5.477150E+12, 5.727001E+12, 5.987687E+12, 6.259654E+12, 6.543365E+12, 
#   6.839299E+12, 7.147956E+12, 7.469855E+12, 7.805533E+12, 8.155550E+12, 8.520486E+12, 
#   8.900943E+12, 9.297547E+12, 9.710947E+12, 1.014182E+13, 1.059086E+13, 1.105879E+13, 
#   1.154638E+13, 1.205439E+13, 1.258365E+13, 1.313499E+13, 1.370928E+13, 1.430744E+13, 
#   1.493039E+13, 1.557912E+13, 1.625464E+13, 1.695798E+13, 1.769025E+13, 1.845256E+13, 
#   1.924608E+13, 2.007203E+13, 2.093165E+13, 2.182625E+13, 2.275718E+13, 2.372583E+13, 
#   2.473365E+13, 2.578214E+13, 2.687286E+13, 2.800741E+13, 2.918747E+13, 3.041476E+13, 
#   3.169107E+13, 3.301827E+13, 3.439826E+13, 3.583304E+13, 3.732466E+13, 3.887527E+13, 
#   4.048707E+13, 4.216234E+13, 4.390345E+13, 4.571286E+13, 4.759310E+13, 4.954678E+13, 
#   5.157665E+13, 5.368549E+13, 5.587623E+13, 5.815188E+13, 6.051554E+13, 6.297045E+13, 
#   6.551994E+13, 6.816746E+13, 7.091657E+13, 7.377097E+13, 7.673448E+13, 7.981105E+13, 
#   8.300475E+13, 8.631981E+13, 8.976060E+13, 9.333163E+13, 9.703756E+13, 1.008832E+14, 
#   1.048736E+14, 1.090139E+14, 1.133094E+14, 1.177655E+14, 1.223881E+14, 1.271830E+14, 
#   1.321562E+14, 1.373141E+14, 1.426631E+14, 1.482100E+14, 1.539616E+14, 1.599251E+14, 
#   1.661080E+14, 1.725177E+14, 1.791624E+14, 1.860500E+14, 1.931889E+14, 2.005880E+14, 
#   2.082560E+14, 2.162024E+14, 2.244366E+14, 2.329686E+14, 2.418084E+14, 2.509667E+14, 
#   2.604542E+14, 2.702823E+14, 2.804624E+14, 2.910064E+14, 3.019268E+14, 3.132362E+14, 
#   3.249478E+14, 3.370749E+14, 3.496317E+14, 3.626326E+14, 3.760923E+14, 3.900262E+14, 
#   4.044502E+14, 4.193804E+14, 4.348339E+14, 4.508278E+14, 4.673801E+14, 4.845093E+14, 
#   5.022343E+14, 5.205748E+14, 5.395510E+14, 5.591837E+14, 5.794945E+14, 6.005055E+14, 
#   6.222395E+14, 6.447202E+14, 6.679716E+14, 6.920190E+14, 7.168880E+14, 7.426052E+14, 
#   7.691980E+14, 7.966946E+14, 8.251240E+14, 8.545162E+14, 8.849021E+14, 9.163135E+14, 
#   9.487830E+14, 9.823444E+14, 1.017032E+15, 1.052883E+15, 1.089933E+15, 1.128220E+15, 
#   1.167783E+15, 1.208663E+15, 1.250902E+15, 1.294540E+15, 1.339624E+15, 1.386198E+15, 
#   1.434308E+15, 1.484002E+15, 1.535330E+15, 1.588344E+15, 1.643094E+15, 1.699635E+15, 
#   1.758022E+15, 1.818313E+15, 1.880565E+15, 1.944840E+15, 2.011200E+15, 2.079708E+15, 
#   2.150431E+15, 2.223435E+15, 2.298791E+15, 2.376571E+15, 2.456848E+15, 2.539698E+15, 
#   2.625198E+15, 2.713430E+15, 2.804476E+15, 2.898421E+15, 2.995351E+15, 3.095357E+15, 
#   3.198532E+15, 3.304969E+15, 3.414766E+15, 3.528025E+15, 3.644848E+15, 3.765342E+15, 
#   3.889615E+15, 4.017779E+15, 4.149950E+15, 4.286247E+15, 4.426790E+15, 4.571706E+15, 
#   4.721123E+15, 4.875173E+15, 5.033992E+15, 5.197720E+15, 5.366501E+15, 5.540481E+15, 
#   5.719812E+15, 5.904650E+15, 6.095156E+15, 6.291492E+15, 6.493829E+15, 6.702340E+15, 
#   6.917202E+15, 7.138599E+15, 7.366718E+15, 7.601754E+15, 7.843904E+15, 8.093372E+15, 
#   8.350366E+15, 8.615103E+15, 8.887802E+15, 
#])
# ============================ M = 28, I = 1 ============================
M = 28
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[5]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.000210E+00, 2.500325E+01, 6.080058E+01, 1.076993E+02, 1.634756E+02, 2.267043E+02, 
   2.965282E+02, 3.723620E+02, 4.537635E+02, 5.403790E+02, 6.319162E+02, 7.281288E+02, 
   8.288067E+02, 9.337702E+02, 1.042867E+03, 1.155969E+03, 1.272974E+03, 1.393806E+03, 
   1.518410E+03, 1.646760E+03, 1.778850E+03, 1.914699E+03, 2.054349E+03, 2.197862E+03, 
   2.345319E+03, 2.496819E+03, 2.652478E+03, 2.812427E+03, 2.976809E+03, 3.145782E+03, 
   3.319511E+03, 3.498174E+03, 3.681956E+03, 3.871054E+03, 4.065668E+03, 4.266008E+03, 
   4.472291E+03, 4.684740E+03, 4.903585E+03, 5.129063E+03, 5.361415E+03, 5.600891E+03, 
   5.847747E+03, 6.102244E+03, 6.364650E+03, 6.635242E+03, 6.914302E+03, 7.202118E+03, 
   7.498988E+03, 7.805217E+03, 8.121115E+03, 8.447003E+03, 8.783208E+03, 9.130068E+03, 
   9.487927E+03, 9.857139E+03, 1.023807E+04, 1.063108E+04, 1.103656E+04, 1.145491E+04, 
   1.188652E+04, 1.233180E+04, 1.279117E+04, 1.326508E+04, 1.375395E+04, 1.425826E+04, 
   1.477846E+04, 1.531503E+04, 1.586846E+04, 1.643926E+04, 1.702794E+04, 1.763503E+04, 
   1.826107E+04, 1.890662E+04, 1.957224E+04, 2.025851E+04, 2.096604E+04, 2.169544E+04, 
   2.244732E+04, 2.322234E+04, 2.402115E+04, 2.484442E+04, 2.569284E+04, 2.656712E+04, 
   2.746798E+04, 2.839616E+04, 2.935242E+04, 3.033753E+04, 3.135229E+04, 3.239750E+04, 
   3.347401E+04, 3.458265E+04, 3.572430E+04, 3.689985E+04, 3.811021E+04, 3.935631E+04, 
   4.063911E+04, 4.195957E+04, 4.331870E+04, 4.471751E+04, 4.615704E+04, 4.763836E+04, 
   4.916257E+04, 5.073076E+04, 5.234409E+04, 5.400370E+04, 5.571079E+04, 5.746658E+04, 
   5.927230E+04, 6.112921E+04, 6.303863E+04, 6.500185E+04, 6.702025E+04, 6.909519E+04, 
   7.122808E+04, 7.342037E+04, 7.567351E+04, 7.798901E+04, 8.036840E+04, 8.281323E+04, 
   8.532511E+04, 8.790566E+04, 9.055653E+04, 9.327943E+04, 9.607607E+04, 9.894822E+04, 
   1.018977E+05, 1.049263E+05, 1.080359E+05, 1.112284E+05, 1.145057E+05, 1.178699E+05, 
   1.213230E+05, 1.248669E+05, 1.285039E+05, 1.322360E+05, 1.360654E+05, 1.399944E+05, 
   1.440252E+05, 1.481600E+05, 1.524013E+05, 1.567515E+05, 1.612128E+05, 1.657879E+05, 
   1.704793E+05, 1.752894E+05, 1.802210E+05, 1.852766E+05, 1.904590E+05, 1.957709E+05, 
   2.012151E+05, 2.067944E+05, 2.125118E+05, 2.183701E+05, 2.243723E+05, 2.305216E+05, 
   2.368209E+05, 2.432733E+05, 2.498822E+05, 2.566507E+05, 2.635820E+05, 2.706796E+05, 
   2.779468E+05, 2.853871E+05, 2.930039E+05, 3.008007E+05, 3.087813E+05, 3.169491E+05, 
   3.253080E+05, 3.338617E+05, 3.426140E+05, 3.515687E+05, 3.607297E+05, 3.701011E+05, 
   3.796869E+05, 3.894910E+05, 3.995178E+05, 4.097712E+05, 4.202557E+05, 4.309754E+05, 
   4.419347E+05, 4.531381E+05, 4.645899E+05, 4.762948E+05, 4.882572E+05, 5.004818E+05, 
   5.129732E+05, 5.257362E+05, 5.387755E+05, 5.520961E+05, 5.657027E+05, 5.796004E+05, 
   5.937941E+05, 6.082889E+05, 6.230898E+05, 6.382020E+05, 6.536308E+05, 6.693814E+05, 
   6.854591E+05, 7.018692E+05, 7.186173E+05, 7.357086E+05, 7.531488E+05, 7.709434E+05, 
   7.890981E+05, 8.076184E+05, 8.265101E+05, 8.457790E+05, 8.654309E+05, 8.854715E+05, 
   9.059069E+05, 9.267429E+05, 9.479856E+05, 9.696410E+05, 9.917151E+05, 1.014214E+06, 
   1.037144E+06, 1.060512E+06, 1.084322E+06, 1.108583E+06, 1.133300E+06, 1.158479E+06, 
   1.184128E+06, 1.210251E+06, 1.236857E+06, 1.263951E+06, 1.291540E+06, 1.319630E+06, 
   1.348229E+06, 1.377343E+06, 1.406978E+06, 1.437142E+06, 1.467840E+06, 1.499081E+06, 
   1.530870E+06, 1.563215E+06, 1.596122E+06, 1.629599E+06, 1.663652E+06, 1.698289E+06, 
   1.733515E+06, 1.769339E+06, 1.805768E+06, 1.842808E+06, 1.880466E+06, 1.918750E+06, 
   1.957667E+06, 1.997223E+06, 2.037427E+06, 2.078285E+06, 2.119805E+06, 2.161993E+06, 
   2.204858E+06, 2.248405E+06, 2.292643E+06, 2.337579E+06, 2.383220E+06, 2.429573E+06, 
   2.476646E+06, 2.524445E+06, 2.572979E+06, 2.622255E+06, 2.672280E+06, 2.723061E+06, 
   2.774606E+06, 2.826921E+06, 2.880016E+06, 2.933896E+06, 2.988569E+06, 3.044042E+06, 
   3.100324E+06, 3.157421E+06, 3.215340E+06, 3.274089E+06, 3.333676E+06, 3.394107E+06, 
   3.455390E+06, 3.517533E+06, 3.580542E+06, 3.644425E+06, 3.709190E+06, 3.774843E+06, 
   3.841392E+06, 3.908844E+06, 3.977207E+06, 4.046488E+06, 4.116693E+06, 4.187831E+06, 
   4.259908E+06, 4.332932E+06, 4.406909E+06, 4.481848E+06, 4.557755E+06, 4.634637E+06, 
   4.712501E+06, 4.791355E+06, 4.871206E+06, 4.952060E+06, 5.033924E+06, 5.116807E+06, 
   5.200714E+06, 5.285653E+06, 5.371630E+06, 5.458652E+06, 5.546727E+06, 5.635862E+06, 
   5.726062E+06, 5.817335E+06, 5.909687E+06, 6.003126E+06, 6.097659E+06, 6.193290E+06, 
   6.290029E+06, 6.387880E+06, 6.486851E+06, 6.586948E+06, 6.688177E+06, 6.790546E+06, 
   6.894061E+06, 6.998728E+06, 7.104553E+06, 7.211543E+06, 7.319705E+06, 7.429043E+06, 
   7.539566E+06, 7.651278E+06, 7.764186E+06, 7.878297E+06, 7.993616E+06, 8.110149E+06, 
   8.227903E+06, 8.346883E+06, 8.467096E+06, 8.588546E+06, 8.711241E+06, 8.835186E+06, 
   8.960386E+06, 9.086848E+06, 9.214576E+06, 9.343578E+06, 9.473858E+06, 9.605421E+06, 
   9.738275E+06, 9.872423E+06, 1.000787E+07, 1.014463E+07, 1.028269E+07, 1.042207E+07, 
   1.056277E+07, 1.070480E+07, 1.084817E+07, 1.099287E+07, 1.113891E+07, 1.128630E+07, 
   1.143504E+07, 1.158513E+07, 1.173659E+07, 1.188942E+07, 1.204362E+07, 1.219919E+07, 
   1.235615E+07, 1.251449E+07, 1.267422E+07, 1.283535E+07, 1.299787E+07, 1.316180E+07, 
   1.332714E+07, 1.349389E+07, 1.366206E+07, 1.383165E+07, 1.400266E+07, 1.417511E+07, 
   1.434898E+07, 1.452430E+07, 1.470106E+07, 1.487926E+07, 1.505891E+07, 1.524002E+07, 
   1.542258E+07, 1.560660E+07, 1.579209E+07, 1.597905E+07, 1.616748E+07, 1.635739E+07, 
   1.654877E+07, 1.674164E+07, 1.693599E+07, 1.713184E+07, 1.732917E+07, 1.752801E+07, 
   1.772834E+07, 1.793017E+07, 1.813351E+07, 1.833836E+07, 1.854472E+07, 1.875260E+07, 
   1.896199E+07, 1.917291E+07, 1.938534E+07, 1.959931E+07, 1.981480E+07, 2.003183E+07, 
   2.025039E+07, 2.047048E+07, 2.069212E+07, 2.091530E+07, 2.114002E+07, 2.136629E+07, 
   2.159411E+07, 2.182348E+07, 2.205441E+07, 2.228689E+07, 2.252093E+07, 2.275653E+07, 
   2.299369E+07, 2.323241E+07, 2.347271E+07, 2.371457E+07, 2.395800E+07, 2.420300E+07, 
   2.444958E+07, 2.469774E+07, 2.494747E+07, 2.519878E+07, 2.545167E+07, 2.570614E+07, 
   2.596220E+07, 2.621984E+07, 2.647907E+07, 2.673989E+07, 2.700230E+07, 2.726629E+07, 
   2.753188E+07, 2.779907E+07, 2.806784E+07, 2.833822E+07, 2.861019E+07, 2.888375E+07, 
   2.915892E+07, 2.943568E+07, 2.971405E+07, 2.999402E+07, 3.027559E+07, 3.055876E+07, 
   3.084354E+07, 3.112992E+07, 3.141790E+07, 3.170749E+07, 3.199869E+07, 3.229150E+07, 
   3.258591E+07, 
])
# ============================ M = 29, I = 1 ============================
M = 29
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.300330E+01, 3.767447E+02, 1.060780E+03, 1.946108E+03, 2.994480E+03, 4.183786E+03, 
   5.499119E+03, 6.929584E+03, 8.466905E+03, 1.010482E+04, 1.183885E+04, 1.366623E+04, 
   1.558590E+04, 1.759839E+04, 1.970577E+04, 2.191145E+04, 2.422011E+04, 2.663750E+04, 
   2.917030E+04, 3.182605E+04, 3.461299E+04, 3.754004E+04, 4.061669E+04, 4.385301E+04, 
   4.725957E+04, 5.084745E+04, 5.462826E+04, 5.861406E+04, 6.281747E+04, 6.725158E+04, 
   7.193003E+04, 7.686702E+04, 8.207734E+04, 8.757632E+04, 9.337996E+04, 9.950488E+04, 
   1.059684E+05, 1.127884E+05, 1.199837E+05, 1.275738E+05, 1.355789E+05, 1.440200E+05, 
   1.529192E+05, 1.622991E+05, 1.721835E+05, 1.825970E+05, 1.935653E+05, 2.051149E+05, 
   2.172735E+05, 2.300699E+05, 2.435340E+05, 2.576967E+05, 2.725902E+05, 2.882480E+05, 
   3.047047E+05, 3.219964E+05, 3.401604E+05, 3.592355E+05, 3.792616E+05, 4.002806E+05, 
   4.223355E+05, 4.454712E+05, 4.697336E+05, 4.951711E+05, 5.218332E+05, 5.497712E+05, 
   5.790385E+05, 6.096900E+05, 6.417827E+05, 6.753756E+05, 7.105294E+05, 7.473075E+05, 
   7.857744E+05, 8.259978E+05, 8.680471E+05, 9.119940E+05, 9.579127E+05, 1.005880E+06, 
   1.055974E+06, 1.108277E+06, 1.162873E+06, 1.219850E+06, 1.279295E+06, 1.341302E+06, 
   1.405966E+06, 1.473385E+06, 1.543660E+06, 1.616896E+06, 1.693200E+06, 1.772682E+06, 
   1.855457E+06, 1.941641E+06, 2.031357E+06, 2.124727E+06, 2.221881E+06, 2.322949E+06, 
   2.428067E+06, 2.537375E+06, 2.651016E+06, 2.769138E+06, 2.891891E+06, 3.019433E+06, 
   3.151923E+06, 3.289526E+06, 3.432411E+06, 3.580752E+06, 3.734728E+06, 3.894521E+06, 
   4.060320E+06, 4.232319E+06, 4.410715E+06, 4.595715E+06, 4.787525E+06, 4.986359E+06, 
   5.192441E+06, 5.405994E+06, 5.627252E+06, 5.856450E+06, 6.093834E+06, 6.339652E+06, 
   6.594162E+06, 6.857625E+06, 7.130313E+06, 7.412500E+06, 7.704467E+06, 8.006508E+06, 
   8.318917E+06, 8.641998E+06, 8.976065E+06, 9.321433E+06, 9.678431E+06, 1.004739E+07, 
   1.042866E+07, 1.082259E+07, 1.122953E+07, 1.164985E+07, 1.208393E+07, 1.253215E+07, 
   1.299491E+07, 1.347260E+07, 1.396564E+07, 1.447445E+07, 1.499946E+07, 1.554111E+07, 
   1.609985E+07, 1.667613E+07, 1.727043E+07, 1.788323E+07, 1.851502E+07, 1.916631E+07, 
   1.983759E+07, 2.052940E+07, 2.124227E+07, 2.197675E+07, 2.273339E+07, 2.351277E+07, 
   2.431547E+07, 2.514208E+07, 2.599322E+07, 2.686948E+07, 2.777152E+07, 2.869997E+07, 
   2.965551E+07, 3.063878E+07, 3.165049E+07, 3.269133E+07, 3.376202E+07, 3.486330E+07, 
   3.599588E+07, 3.716054E+07, 3.835806E+07, 3.958922E+07, 4.085482E+07, 4.215568E+07, 
   4.349265E+07, 4.486657E+07, 4.627831E+07, 4.772876E+07, 4.921882E+07, 5.074942E+07, 
   5.232149E+07, 5.393598E+07, 5.559387E+07, 5.729615E+07, 5.904385E+07, 6.083798E+07, 
   6.267958E+07, 6.456974E+07, 6.650956E+07, 6.850011E+07, 7.054254E+07, 7.263800E+07, 
   7.478767E+07, 7.699273E+07, 7.925441E+07, 8.157392E+07, 8.395253E+07, 8.639153E+07, 
   8.889220E+07, 9.145590E+07, 9.408394E+07, 9.677775E+07, 9.953866E+07, 1.023681E+08, 
   1.052676E+08, 1.082386E+08, 1.112825E+08, 1.144010E+08, 1.175954E+08, 1.208675E+08, 
   1.242189E+08, 1.276511E+08, 1.311658E+08, 1.347647E+08, 1.384496E+08, 1.422220E+08, 
   1.460840E+08, 1.500371E+08, 1.540834E+08, 1.582245E+08, 1.624626E+08, 1.667993E+08, 
   1.712367E+08, 1.757769E+08, 1.804217E+08, 1.851733E+08, 1.900338E+08, 1.950053E+08, 
   2.000898E+08, 2.052897E+08, 2.106070E+08, 2.160442E+08, 2.216035E+08, 2.272871E+08, 
   2.330976E+08, 2.390372E+08, 2.451085E+08, 2.513138E+08, 2.576558E+08, 2.641369E+08, 
   2.707598E+08, 2.775271E+08, 2.844415E+08, 2.915056E+08, 2.987224E+08, 3.060944E+08, 
   3.136247E+08, 3.213160E+08, 3.291713E+08, 3.371937E+08, 3.453860E+08, 3.537512E+08, 
   3.622928E+08, 3.710136E+08, 3.799169E+08, 3.890059E+08, 3.982840E+08, 4.077545E+08, 
   4.174208E+08, 4.272862E+08, 4.373543E+08, 4.476286E+08, 4.581129E+08, 4.688105E+08, 
   4.797253E+08, 4.908611E+08, 5.022216E+08, 5.138105E+08, 5.256319E+08, 5.376897E+08, 
   5.499880E+08, 5.625309E+08, 5.753222E+08, 5.883665E+08, 6.016678E+08, 6.152305E+08, 
   6.290589E+08, 6.431575E+08, 6.575307E+08, 6.721832E+08, 6.871194E+08, 7.023441E+08, 
   7.178619E+08, 7.336778E+08, 7.497965E+08, 7.662230E+08, 7.829623E+08, 8.000195E+08, 
   8.173995E+08, 8.351077E+08, 8.531493E+08, 8.715297E+08, 8.902543E+08, 9.093285E+08, 
   9.287578E+08, 9.485481E+08, 9.687047E+08, 9.892336E+08, 1.010141E+09, 1.031432E+09, 
   1.053113E+09, 1.075190E+09, 1.097669E+09, 1.120557E+09, 1.143859E+09, 1.167583E+09, 
   1.191734E+09, 1.216319E+09, 1.241345E+09, 1.266819E+09, 1.292747E+09, 1.319136E+09, 
   1.345992E+09, 1.373325E+09, 1.401139E+09, 1.429444E+09, 1.458245E+09, 1.487550E+09, 
   1.517367E+09, 1.547703E+09, 1.578567E+09, 1.609966E+09, 1.641907E+09, 1.674399E+09, 
   1.707449E+09, 1.741067E+09, 1.775259E+09, 1.810035E+09, 1.845404E+09, 1.881373E+09, 
   1.917950E+09, 1.955147E+09, 1.992969E+09, 2.031428E+09, 2.070532E+09, 2.110289E+09, 
   2.150711E+09, 2.191804E+09, 2.233580E+09, 2.276049E+09, 2.319218E+09, 2.363100E+09, 
   2.407703E+09, 2.453037E+09, 2.499113E+09, 2.545942E+09, 2.593533E+09, 2.641897E+09, 
   2.691044E+09, 2.740986E+09, 2.791734E+09, 
])
# ============================ M = 29, I = 2 ============================
M = 29
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.600659E+01, 7.534893E+02, 2.121560E+03, 3.892216E+03, 5.988960E+03, 8.367572E+03, 
   1.099824E+04, 1.385917E+04, 1.693381E+04, 2.020964E+04, 2.367770E+04, 2.733250E+04, 
   3.117187E+04, 3.519691E+04, 3.941173E+04, 4.382320E+04, 4.844066E+04, 5.327561E+04, 
   5.834143E+04, 6.365316E+04, 6.922734E+04, 7.508175E+04, 8.123543E+04, 8.770848E+04, 
   9.452204E+04, 1.016983E+05, 1.092604E+05, 1.172326E+05, 1.256400E+05, 1.345089E+05, 
   1.438665E+05, 1.537412E+05, 1.641626E+05, 1.751614E+05, 1.867695E+05, 1.990203E+05, 
   2.119482E+05, 2.255893E+05, 2.399810E+05, 2.551622E+05, 2.711735E+05, 2.880570E+05, 
   3.058566E+05, 3.246177E+05, 3.443879E+05, 3.652164E+05, 3.871544E+05, 4.102551E+05, 
   4.345740E+05, 4.601685E+05, 4.870983E+05, 5.154256E+05, 5.452144E+05, 5.765320E+05, 
   6.094476E+05, 6.440331E+05, 6.803632E+05, 7.185157E+05, 7.585704E+05, 8.006109E+05, 
   8.447233E+05, 8.909973E+05, 9.395250E+05, 9.904028E+05, 1.043730E+06, 1.099609E+06, 
   1.158147E+06, 1.219453E+06, 1.283642E+06, 1.350832E+06, 1.421143E+06, 1.494703E+06, 
   1.571641E+06, 1.652092E+06, 1.736195E+06, 1.824093E+06, 1.915934E+06, 2.011873E+06, 
   2.112067E+06, 2.216678E+06, 2.325876E+06, 2.439833E+06, 2.558730E+06, 2.682749E+06, 
   2.812083E+06, 2.946927E+06, 3.087485E+06, 3.233963E+06, 3.386577E+06, 3.545548E+06, 
   3.711105E+06, 3.883481E+06, 4.062920E+06, 4.249668E+06, 4.443983E+06, 4.646127E+06, 
   4.856372E+06, 5.074998E+06, 5.302289E+06, 5.538541E+06, 5.784058E+06, 6.039152E+06, 
   6.304143E+06, 6.579359E+06, 6.865140E+06, 7.161833E+06, 7.469795E+06, 7.789393E+06, 
   8.121003E+06, 8.465013E+06, 8.821819E+06, 9.191830E+06, 9.575464E+06, 9.973147E+06, 
   1.038532E+07, 1.081245E+07, 1.125497E+07, 1.171339E+07, 1.218817E+07, 1.267982E+07, 
   1.318886E+07, 1.371580E+07, 1.426120E+07, 1.482559E+07, 1.540954E+07, 1.601364E+07, 
   1.663848E+07, 1.728466E+07, 1.795281E+07, 1.864357E+07, 1.935759E+07, 2.009553E+07, 
   2.085809E+07, 2.164596E+07, 2.245987E+07, 2.330053E+07, 2.416872E+07, 2.506518E+07, 
   2.599072E+07, 2.694613E+07, 2.793224E+07, 2.894989E+07, 2.999994E+07, 3.108326E+07, 
   3.220077E+07, 3.335336E+07, 3.454200E+07, 3.576763E+07, 3.703124E+07, 3.833384E+07, 
   3.967644E+07, 4.106009E+07, 4.248586E+07, 4.395486E+07, 4.546818E+07, 4.702698E+07, 
   4.863242E+07, 5.028568E+07, 5.198798E+07, 5.374055E+07, 5.554466E+07, 5.740161E+07, 
   5.931272E+07, 6.127931E+07, 6.330278E+07, 6.538450E+07, 6.752592E+07, 6.972852E+07, 
   7.199374E+07, 7.432310E+07, 7.671818E+07, 7.918056E+07, 8.171180E+07, 8.431358E+07, 
   8.698757E+07, 8.973546E+07, 9.255900E+07, 9.545995E+07, 9.844013E+07, 1.015014E+08, 
   1.046456E+08, 1.078746E+08, 1.111905E+08, 1.145951E+08, 1.180905E+08, 1.216789E+08, 
   1.253621E+08, 1.291425E+08, 1.330222E+08, 1.370034E+08, 1.410883E+08, 1.452793E+08, 
   1.495787E+08, 1.539889E+08, 1.585123E+08, 1.631514E+08, 1.679087E+08, 1.727868E+08, 
   1.777882E+08, 1.829157E+08, 1.881719E+08, 1.935595E+08, 1.990815E+08, 2.047405E+08, 
   2.105395E+08, 2.164816E+08, 2.225695E+08, 2.288065E+08, 2.351955E+08, 2.417398E+08, 
   2.484426E+08, 2.553071E+08, 2.623366E+08, 2.695345E+08, 2.769043E+08, 2.844494E+08, 
   2.921733E+08, 3.000798E+08, 3.081724E+08, 3.164548E+08, 3.249309E+08, 3.336044E+08, 
   3.424795E+08, 3.515598E+08, 3.608497E+08, 3.703530E+08, 3.800740E+08, 3.900171E+08, 
   4.001862E+08, 4.105861E+08, 4.212210E+08, 4.320955E+08, 4.432141E+08, 4.545816E+08, 
   4.662025E+08, 4.780819E+08, 4.902245E+08, 5.026354E+08, 5.153195E+08, 5.282818E+08, 
   5.415277E+08, 5.550625E+08, 5.688914E+08, 5.830199E+08, 5.974535E+08, 6.121977E+08, 
   6.272584E+08, 6.426412E+08, 6.583520E+08, 6.743967E+08, 6.907814E+08, 7.075121E+08, 
   7.245953E+08, 7.420372E+08, 7.598439E+08, 7.780220E+08, 7.965784E+08, 8.155195E+08, 
   8.348521E+08, 8.545832E+08, 8.747197E+08, 8.952684E+08, 9.162371E+08, 9.376325E+08, 
   9.594623E+08, 9.817339E+08, 1.004455E+09, 1.027633E+09, 1.051276E+09, 1.075392E+09, 
   1.099989E+09, 1.125074E+09, 1.150657E+09, 1.176746E+09, 1.203349E+09, 1.230475E+09, 
   1.258132E+09, 1.286329E+09, 1.315075E+09, 1.344380E+09, 1.374253E+09, 1.404703E+09, 
   1.435738E+09, 1.467371E+09, 1.499608E+09, 1.532461E+09, 1.565940E+09, 1.600055E+09, 
   1.634815E+09, 1.670231E+09, 1.706315E+09, 1.743076E+09, 1.780525E+09, 1.818674E+09, 
   1.857533E+09, 1.897113E+09, 1.937427E+09, 1.978485E+09, 2.020299E+09, 2.062881E+09, 
   2.106243E+09, 2.150397E+09, 2.195356E+09, 2.241132E+09, 2.287737E+09, 2.335184E+09, 
   2.383486E+09, 2.432657E+09, 2.482710E+09, 2.533657E+09, 2.585513E+09, 2.638291E+09, 
   2.692005E+09, 2.746670E+09, 2.802299E+09, 2.858908E+09, 2.916510E+09, 2.975121E+09, 
   3.034755E+09, 3.095428E+09, 3.157156E+09, 3.219953E+09, 3.283835E+09, 3.348819E+09, 
   3.414921E+09, 3.482156E+09, 3.550541E+09, 3.620094E+09, 3.690830E+09, 3.762768E+09, 
   3.835924E+09, 3.910316E+09, 3.985962E+09, 4.062879E+09, 4.141087E+09, 4.220602E+09, 
   4.301445E+09, 4.383633E+09, 4.467185E+09, 4.552122E+09, 4.638461E+09, 4.726224E+09, 
   4.815431E+09, 4.906099E+09, 4.998252E+09, 5.091908E+09, 5.187090E+09, 5.283818E+09, 
   5.382113E+09, 5.481997E+09, 5.583493E+09, 
])
# ============================ M = 30, I = 1 ============================
M = 30
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.030200E+02, 3.161421E+03, 8.927419E+03, 1.639214E+04, 2.523132E+04, 3.526128E+04, 
   4.637565E+04, 5.853672E+04, 7.177676E+04, 8.619683E+04, 1.019630E+05, 1.193027E+05, 
   1.385034E+05, 1.599127E+05, 1.839423E+05, 2.110738E+05, 2.418668E+05, 2.769703E+05, 
   3.171359E+05, 3.632343E+05, 4.162746E+05, 4.774265E+05, 5.480466E+05, 6.297090E+05, 
   7.242394E+05, 8.337559E+05, 9.607151E+05, 1.107965E+06, 1.278807E+06, 1.477063E+06, 
   1.707158E+06, 1.974214E+06, 2.284147E+06, 2.643794E+06, 3.061047E+06, 3.545009E+06, 
   4.106167E+06, 4.756601E+06, 5.510207E+06, 6.382958E+06, 7.393198E+06, 8.561980E+06, 
   9.913436E+06, 1.147521E+07, 1.327894E+07, 1.536079E+07, 1.776208E+07, 2.052996E+07, 
   2.371822E+07, 2.738808E+07, 3.160928E+07, 3.646110E+07, 4.203360E+07, 4.842902E+07, 
   5.576332E+07, 6.416786E+07, 7.379138E+07, 8.480210E+07, 9.739014E+07, 1.117701E+08, 
   1.281842E+08, 1.469054E+08, 1.682410E+08, 1.925370E+08, 2.201823E+08, 2.516136E+08, 
   2.873214E+08, 3.278556E+08, 3.738326E+08, 4.259424E+08, 4.849571E+08, 5.517399E+08, 
   6.272550E+08, 7.125787E+08, 8.089114E+08, 9.175913E+08, 1.040108E+09, 1.178121E+09, 
   1.333472E+09, 1.508213E+09, 1.704618E+09, 1.925212E+09, 2.172795E+09, 2.450471E+09, 
   2.761674E+09, 3.110207E+09, 3.500274E+09, 3.936522E+09, 4.424082E+09, 4.968616E+09, 
   5.576371E+09, 6.254232E+09, 7.009782E+09, 7.851370E+09, 8.788182E+09, 9.830316E+09, 
   1.098887E+10, 1.227603E+10, 1.370517E+10, 1.529096E+10, 1.704949E+10, 1.899838E+10, 
   2.115692E+10, 2.354623E+10, 2.618941E+10, 2.911170E+10, 3.234070E+10, 3.590654E+10, 
   3.984208E+10, 4.418321E+10, 4.896903E+10, 5.424216E+10, 6.004900E+10, 6.644010E+10, 
   7.347043E+10, 8.119980E+10, 8.969319E+10, 9.902123E+10, 1.092606E+11, 1.204946E+11, 
   1.328135E+11, 1.463153E+11, 1.611061E+11, 1.773009E+11, 1.950245E+11, 2.144115E+11, 
   2.356080E+11, 2.587718E+11, 2.840735E+11, 3.116975E+11, 3.418429E+11, 3.747248E+11, 
   4.105754E+11, 4.496450E+11, 4.922036E+11, 5.385424E+11, 5.889749E+11, 6.438391E+11, 
   7.034988E+11, 7.683454E+11, 8.388001E+11, 9.153161E+11, 9.983803E+11, 1.088516E+12, 
   1.186286E+12, 1.292294E+12, 1.407187E+12, 1.531662E+12, 1.666465E+12, 1.812396E+12, 
   1.970312E+12, 2.141134E+12, 2.325846E+12, 2.525502E+12, 2.741231E+12, 2.974242E+12, 
   3.225827E+12, 3.497369E+12, 3.790346E+12, 4.106337E+12, 4.447031E+12, 4.814230E+12, 
   5.209859E+12, 5.635974E+12, 6.094768E+12, 6.588580E+12, 7.119906E+12, 7.691405E+12, 
   8.305914E+12, 8.966455E+12, 9.676246E+12, 1.043872E+13, 1.125751E+13, 1.213652E+13, 
   1.307988E+13, 1.409197E+13, 1.517749E+13, 1.634139E+13, 1.758897E+13, 1.892584E+13, 
   2.035796E+13, 2.189167E+13, 2.353370E+13, 2.529118E+13, 2.717170E+13, 2.918329E+13, 
   3.133448E+13, 3.363431E+13, 3.609238E+13, 3.871883E+13, 4.152444E+13, 4.452062E+13, 
   4.771944E+13, 5.113370E+13, 5.477695E+13, 5.866351E+13, 6.280855E+13, 6.722812E+13, 
   7.193919E+13, 7.695971E+13, 8.230863E+13, 8.800602E+13, 9.407307E+13, 1.005322E+14, 
   1.074069E+14, 1.147223E+14, 1.225048E+14, 1.307820E+14, 1.395835E+14, 1.489402E+14, 
   1.588847E+14, 1.694516E+14, 1.806772E+14, 1.925998E+14, 2.052598E+14, 2.186998E+14, 
   2.329647E+14, 2.481017E+14, 2.641605E+14, 2.811935E+14, 2.992560E+14, 3.184059E+14, 
   3.387044E+14, 3.602159E+14, 3.830078E+14, 4.071514E+14, 4.327215E+14, 4.597968E+14, 
   4.884599E+14, 5.187977E+14, 5.509017E+14, 5.848676E+14, 6.207965E+14, 6.587940E+14, 
   6.989715E+14, 7.414456E+14, 7.863388E+14, 8.337797E+14, 8.839034E+14, 9.368513E+14, 
   9.927720E+14, 1.051821E+15, 1.114163E+15, 1.179967E+15, 1.249415E+15, 1.322693E+15, 
   1.400000E+15, 1.481542E+15, 1.567536E+15, 1.658207E+15, 1.753795E+15, 1.854548E+15, 
   1.960726E+15, 2.072601E+15, 2.190460E+15, 2.314601E+15, 2.445337E+15, 2.582994E+15, 
   2.727914E+15, 2.880455E+15, 3.040992E+15, 3.209916E+15, 3.387634E+15, 3.574576E+15, 
   3.771187E+15, 3.977935E+15, 4.195307E+15, 4.423813E+15, 4.663985E+15, 4.916380E+15, 
   5.181576E+15, 5.460182E+15, 5.752830E+15, 6.060180E+15, 6.382922E+15, 6.721775E+15, 
   7.077491E+15, 7.450851E+15, 7.842674E+15, 8.253812E+15, 8.685153E+15, 9.137623E+15, 
   9.612190E+15, 1.010986E+16, 1.063168E+16, 1.117876E+16, 1.175222E+16, 1.235325E+16, 
   1.298310E+16, 1.364306E+16, 1.433446E+16, 1.505870E+16, 1.581725E+16, 1.661161E+16, 
   1.744338E+16, 1.831418E+16, 1.922573E+16, 2.017982E+16, 2.117829E+16, 2.222307E+16, 
   2.331617E+16, 2.445967E+16, 2.565575E+16, 2.690665E+16, 2.821473E+16, 2.958242E+16, 
   3.101226E+16, 3.250689E+16, 3.406905E+16, 3.570158E+16, 3.740745E+16, 3.918974E+16, 
   4.105162E+16, 4.299644E+16, 4.502763E+16, 4.714877E+16, 4.936359E+16, 5.167594E+16, 
   5.408984E+16, 5.660946E+16, 5.923910E+16, 6.198326E+16, 6.484659E+16, 6.783393E+16, 
   7.095029E+16, 7.420086E+16, 7.759106E+16, 8.112647E+16, 8.481291E+16, 8.865639E+16, 
   9.266317E+16, 9.683972E+16, 1.011928E+17, 1.057292E+17, 1.104564E+17, 1.153817E+17, 
   1.205129E+17, 1.258580E+17, 1.314254E+17, 1.372237E+17, 1.432619E+17, 1.495491E+17, 
   1.560951E+17, 1.629096E+17, 1.700031E+17, 1.773862E+17, 1.850699E+17, 1.930657E+17, 
   2.013853E+17, 2.100411E+17, 2.190456E+17, 2.284122E+17, 2.381542E+17, 2.482858E+17, 
   2.588216E+17, 2.697765E+17, 2.811661E+17, 2.930066E+17, 3.053147E+17, 3.181074E+17, 
   3.314028E+17, 3.452192E+17, 3.595756E+17, 3.744919E+17, 3.899883E+17, 4.060860E+17, 
   4.228067E+17, 4.401729E+17, 4.582079E+17, 4.769358E+17, 4.963815E+17, 5.165705E+17, 
   5.375294E+17, 5.592857E+17, 5.818677E+17, 6.053046E+17, 6.296267E+17, 6.548652E+17, 
   6.810523E+17, 7.082212E+17, 7.364065E+17, 7.656436E+17, 7.959691E+17, 8.274209E+17, 
   8.600380E+17, 8.938607E+17, 9.289308E+17, 9.652910E+17, 1.002986E+18, 1.042061E+18, 
   1.082563E+18, 1.124542E+18, 1.168047E+18, 1.213131E+18, 1.259846E+18, 1.308248E+18, 
   1.358393E+18, 1.410341E+18, 1.464152E+18, 1.519889E+18, 1.577614E+18, 1.637396E+18, 
   1.699302E+18, 1.763403E+18, 1.829772E+18, 1.898482E+18, 1.969613E+18, 2.043242E+18, 
   2.119453E+18, 2.198329E+18, 2.279958E+18, 2.364429E+18, 2.451835E+18, 2.542272E+18, 
   2.635836E+18, 2.732630E+18, 2.832757E+18, 2.936324E+18, 3.043442E+18, 3.154225E+18, 
   3.268790E+18, 3.387257E+18, 3.509750E+18, 3.636398E+18, 3.767331E+18, 3.902685E+18, 
   4.042600E+18, 4.187219E+18, 4.336690E+18, 4.491164E+18, 4.650800E+18, 4.815756E+18, 
   4.986199E+18, 5.162300E+18, 5.344233E+18, 5.532179E+18, 5.726325E+18, 5.926860E+18, 
   6.133981E+18, 6.347890E+18, 6.568796E+18, 6.796911E+18, 7.032455E+18, 7.275655E+18, 
   7.526742E+18, 7.785956E+18, 8.053543E+18, 8.329754E+18, 8.614850E+18, 8.909098E+18, 
   9.212771E+18, 9.526152E+18, 9.849531E+18, 1.018321E+19, 1.052748E+19, 1.088267E+19, 
   1.124911E+19, 1.162711E+19, 1.201703E+19, 1.241921E+19, 1.283401E+19, 1.326181E+19, 
   1.370299E+19, 1.415792E+19, 1.462703E+19, 1.511071E+19, 1.560940E+19, 1.612351E+19, 
   1.665352E+19, 1.719986E+19, 1.776301E+19, 1.834346E+19, 1.894169E+19, 1.955823E+19, 
   2.019358E+19, 2.084830E+19, 2.152293E+19, 2.221803E+19, 2.293419E+19, 2.367199E+19, 
   2.443207E+19, 2.521503E+19, 2.602152E+19, 2.685221E+19, 2.770778E+19, 2.858891E+19, 
   2.949632E+19, 3.043075E+19, 3.139295E+19, 3.238368E+19, 3.340374E+19, 3.445394E+19, 
   3.553510E+19, 3.664809E+19, 3.779378E+19, 3.897306E+19, 4.018686E+19, 4.143611E+19, 
   4.272179E+19, 4.404489E+19, 4.540643E+19, 
])
# ============================ M = 31, I = 1 ============================
M = 31
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[4]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.000000E+00, 3.233990E+00, 9.577640E+00, 1.729643E+01, 2.619804E+01, 3.620815E+01, 
   4.723879E+01, 5.920975E+01, 7.205446E+01, 8.571826E+01, 1.001556E+02, 1.153280E+02, 
   1.312023E+02, 1.477495E+02, 1.649444E+02, 1.827645E+02, 2.011901E+02, 2.202037E+02, 
   2.397898E+02, 2.599350E+02, 2.806276E+02, 3.018578E+02, 3.236172E+02, 3.458989E+02, 
   3.686976E+02, 3.920091E+02, 4.158306E+02, 4.401602E+02, 4.649971E+02, 4.903414E+02, 
   5.161940E+02, 5.425567E+02, 5.694318E+02, 5.968223E+02, 6.247317E+02, 6.531641E+02, 
   6.821239E+02, 7.116161E+02, 7.416459E+02, 7.722190E+02, 8.033414E+02, 8.350192E+02, 
   8.672592E+02, 9.000682E+02, 9.334532E+02, 9.674216E+02, 1.001981E+03, 1.037139E+03, 
   1.072904E+03, 1.109285E+03, 1.146289E+03, 1.183925E+03, 1.222203E+03, 1.261131E+03, 
   1.300719E+03, 1.340976E+03, 1.381913E+03, 1.423538E+03, 1.465863E+03, 1.508897E+03, 
   1.552650E+03, 1.597135E+03, 1.642361E+03, 1.688339E+03, 1.735081E+03, 1.782598E+03, 
   1.830901E+03, 1.880003E+03, 1.929915E+03, 1.980649E+03, 2.032218E+03, 2.084633E+03, 
   2.137907E+03, 2.192054E+03, 2.247085E+03, 2.303014E+03, 2.359854E+03, 2.417618E+03, 
   2.476320E+03, 2.535972E+03, 2.596589E+03, 2.658185E+03, 2.720774E+03, 2.784369E+03, 
   2.848985E+03, 2.914636E+03, 2.981338E+03, 3.049103E+03, 3.117948E+03, 3.187888E+03, 
   3.258936E+03, 3.331109E+03, 3.404422E+03, 3.478890E+03, 3.554529E+03, 3.631354E+03, 
   3.709382E+03, 3.788628E+03, 3.869109E+03, 3.950840E+03, 4.033838E+03, 4.118120E+03, 
   4.203702E+03, 4.290601E+03, 4.378833E+03, 4.468417E+03, 4.559368E+03, 4.651705E+03, 
   4.745444E+03, 4.840603E+03, 4.937200E+03, 5.035253E+03, 5.134779E+03, 5.235797E+03, 
   5.338325E+03, 5.442380E+03, 5.547982E+03, 5.655149E+03, 5.763900E+03, 5.874254E+03, 
   5.986229E+03, 6.099845E+03, 6.215121E+03, 6.332076E+03, 6.450730E+03, 6.571102E+03, 
   6.693213E+03, 6.817082E+03, 6.942728E+03, 7.070174E+03, 7.199438E+03, 7.330541E+03, 
   7.463503E+03, 7.598346E+03, 7.735091E+03, 7.873757E+03, 8.014367E+03, 8.156942E+03, 
   8.301502E+03, 8.448070E+03, 8.596667E+03, 8.747315E+03, 8.900036E+03, 9.054852E+03, 
   9.211786E+03, 9.370859E+03, 9.532093E+03, 9.695513E+03, 9.861140E+03, 1.002900E+04, 
   1.019911E+04, 1.037149E+04, 1.054618E+04, 1.072319E+04, 1.090255E+04, 1.108428E+04, 
   1.126840E+04, 1.145494E+04, 1.164392E+04, 1.183537E+04, 1.202931E+04, 1.222576E+04, 
   1.242476E+04, 1.262632E+04, 1.283047E+04, 1.303724E+04, 1.324664E+04, 1.345872E+04, 
   1.367348E+04, 1.389096E+04, 1.411118E+04, 1.433417E+04, 1.455995E+04, 1.478855E+04, 
   1.502000E+04, 1.525432E+04, 1.549154E+04, 1.573169E+04, 1.597478E+04, 1.622086E+04, 
   1.646995E+04, 1.672206E+04, 1.697724E+04, 1.723550E+04, 1.749688E+04, 1.776140E+04, 
   1.802909E+04, 1.829998E+04, 1.857410E+04, 1.885147E+04, 1.913212E+04, 1.941608E+04, 
   1.970338E+04, 1.999405E+04, 2.028811E+04, 2.058559E+04, 2.088653E+04, 2.119095E+04, 
   2.149888E+04, 2.181035E+04, 2.212539E+04, 2.244402E+04, 2.276628E+04, 2.309220E+04, 
   2.342180E+04, 2.375512E+04, 2.409219E+04, 2.443303E+04, 2.477767E+04, 2.512614E+04, 
   2.547848E+04, 2.583472E+04, 2.619488E+04, 2.655899E+04, 2.692709E+04, 2.729920E+04, 
   2.767536E+04, 2.805559E+04, 2.843993E+04, 2.882840E+04, 2.922104E+04, 2.961788E+04, 
   3.001895E+04, 3.042427E+04, 3.083389E+04, 3.124782E+04, 3.166610E+04, 3.208877E+04, 
   3.251585E+04, 3.294737E+04, 3.338337E+04, 3.382387E+04, 3.426891E+04, 3.471852E+04, 
   3.517272E+04, 3.563156E+04, 3.609505E+04, 3.656324E+04, 3.703614E+04, 3.751381E+04, 
   3.799625E+04, 3.848351E+04, 3.897562E+04, 3.947260E+04, 3.997449E+04, 4.048133E+04, 
   4.099313E+04, 4.150993E+04, 4.203177E+04, 4.255866E+04, 4.309066E+04, 4.362777E+04, 
   4.417005E+04, 4.471751E+04, 4.527018E+04, 4.582811E+04, 4.639131E+04, 4.695982E+04, 
   4.753367E+04, 4.811289E+04, 4.869751E+04, 4.928756E+04, 4.988308E+04, 5.048408E+04, 
   5.109061E+04, 5.170269E+04, 5.232035E+04, 5.294362E+04, 5.357253E+04, 5.420712E+04, 
   5.484740E+04, 5.549342E+04, 5.614520E+04, 5.680276E+04, 5.746615E+04, 5.813538E+04, 
   5.881050E+04, 5.949152E+04, 6.017847E+04, 6.087139E+04, 6.157031E+04, 6.227524E+04, 
   6.298623E+04, 6.370329E+04, 6.442647E+04, 6.515578E+04, 6.589125E+04, 6.663291E+04, 
   6.738080E+04, 6.813493E+04, 6.889533E+04, 6.966204E+04, 7.043508E+04, 7.121448E+04, 
   7.200025E+04, 7.279244E+04, 7.359107E+04, 7.439616E+04, 7.520774E+04, 7.602584E+04, 
   7.685048E+04, 7.768169E+04, 7.851950E+04, 7.936392E+04, 8.021500E+04, 8.107274E+04, 
   8.193718E+04, 8.280834E+04, 8.368625E+04, 8.457093E+04, 8.546241E+04, 8.636070E+04, 
   8.726584E+04, 8.817785E+04, 8.909676E+04, 9.002257E+04, 9.095533E+04, 9.189505E+04, 
   9.284176E+04, 9.379548E+04, 9.475623E+04, 9.572403E+04, 9.669891E+04, 9.768089E+04, 
   9.866999E+04, 9.966623E+04, 1.006696E+05, 1.016802E+05, 1.026980E+05, 1.037231E+05, 
   1.047553E+05, 1.057949E+05, 1.068417E+05, 1.078959E+05, 1.089574E+05, 1.100262E+05, 
   1.111024E+05, 1.121860E+05, 1.132770E+05, 1.143754E+05, 1.154813E+05, 1.165947E+05, 
   1.177155E+05, 1.188438E+05, 1.199797E+05, 1.211231E+05, 1.222740E+05, 1.234325E+05, 
   1.245986E+05, 1.257724E+05, 1.269537E+05, 1.281427E+05, 1.293393E+05, 1.305437E+05, 
   1.317557E+05, 1.329754E+05, 1.342028E+05, 1.354380E+05, 1.366809E+05, 1.379316E+05, 
   1.391901E+05, 1.404563E+05, 1.417304E+05, 1.430123E+05, 1.443021E+05, 1.455997E+05, 
   1.469051E+05, 1.482184E+05, 1.495397E+05, 1.508688E+05, 1.522058E+05, 1.535508E+05, 
   1.549037E+05, 1.562646E+05, 1.576334E+05, 1.590102E+05, 1.603950E+05, 1.617878E+05, 
   1.631886E+05, 1.645974E+05, 1.660143E+05, 1.674392E+05, 1.688721E+05, 1.703131E+05, 
   1.717622E+05, 1.732193E+05, 1.746846E+05, 1.761579E+05, 1.776393E+05, 1.791289E+05, 
   1.806266E+05, 1.821324E+05, 1.836463E+05, 1.851684E+05, 1.866986E+05, 1.882370E+05, 
   1.897836E+05, 1.913384E+05, 1.929013E+05, 1.944724E+05, 1.960517E+05, 
])
# ============================ M = 31, I = 2 ============================
M = 31
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   -4.868110E+00, 3.941117E+00, 9.857399E+00, 1.739600E+01, 2.625769E+01, 3.626671E+01, 
   4.730342E+01, 5.927943E+01, 7.212597E+01, 8.578750E+01, 1.002180E+02, 1.153783E+02, 
   1.312350E+02, 1.477588E+02, 1.649239E+02, 1.827076E+02, 2.010899E+02, 2.200527E+02, 
   2.395806E+02, 2.596597E+02, 2.802782E+02, 3.014260E+02, 3.230944E+02, 3.452764E+02, 
   3.679665E+02, 3.911605E+02, 4.148552E+02, 4.390486E+02, 4.637399E+02, 4.889290E+02, 
   5.146167E+02, 5.408047E+02, 5.674950E+02, 5.946907E+02, 6.223951E+02, 6.506122E+02, 
   6.793462E+02, 7.086019E+02, 7.383846E+02, 7.686996E+02, 7.995527E+02, 8.309502E+02, 
   8.628982E+02, 8.954036E+02, 9.284731E+02, 9.621139E+02, 9.963333E+02, 1.031139E+03, 
   1.066539E+03, 1.102540E+03, 1.139152E+03, 1.176383E+03, 1.214241E+03, 1.252734E+03, 
   1.291873E+03, 1.331666E+03, 1.372122E+03, 1.413251E+03, 1.455062E+03, 1.497566E+03, 
   1.540773E+03, 1.584692E+03, 1.629335E+03, 1.674710E+03, 1.720830E+03, 1.767705E+03, 
   1.815346E+03, 1.863764E+03, 1.912970E+03, 1.962976E+03, 2.013793E+03, 2.065433E+03, 
   2.117908E+03, 2.171230E+03, 2.225410E+03, 2.280461E+03, 2.336395E+03, 2.393226E+03, 
   2.450964E+03, 2.509624E+03, 2.569217E+03, 2.629757E+03, 2.691257E+03, 2.753730E+03, 
   2.817189E+03, 2.881648E+03, 2.947121E+03, 3.013620E+03, 3.081160E+03, 3.149754E+03, 
   3.219417E+03, 3.290162E+03, 3.362004E+03, 3.434957E+03, 3.509035E+03, 3.584253E+03, 
   3.660625E+03, 3.738167E+03, 3.816892E+03, 3.896817E+03, 3.977955E+03, 4.060322E+03, 
   4.143934E+03, 4.228805E+03, 4.314951E+03, 4.402388E+03, 4.491131E+03, 4.581196E+03, 
   4.672599E+03, 4.765355E+03, 4.859481E+03, 4.954994E+03, 5.051908E+03, 5.150242E+03, 
   5.250010E+03, 5.351231E+03, 5.453919E+03, 5.558093E+03, 5.663769E+03, 5.770965E+03, 
   5.879696E+03, 5.989981E+03, 6.101837E+03, 6.215281E+03, 6.330331E+03, 6.447004E+03, 
   6.565319E+03, 6.685292E+03, 6.806943E+03, 6.930289E+03, 7.055348E+03, 7.182139E+03, 
   7.310680E+03, 7.440989E+03, 7.573086E+03, 7.706988E+03, 7.842715E+03, 7.980287E+03, 
   8.119721E+03, 8.261036E+03, 8.404254E+03, 8.549392E+03, 8.696470E+03, 8.845508E+03, 
   8.996526E+03, 9.149544E+03, 9.304581E+03, 9.461657E+03, 9.620793E+03, 9.782010E+03, 
   9.945327E+03, 1.011076E+04, 1.027834E+04, 1.044809E+04, 1.062001E+04, 1.079414E+04, 
   1.097050E+04, 1.114910E+04, 1.132997E+04, 1.151314E+04, 1.169861E+04, 1.188641E+04, 
   1.207657E+04, 1.226911E+04, 1.246405E+04, 1.266141E+04, 1.286121E+04, 1.306347E+04, 
   1.326823E+04, 1.347550E+04, 1.368530E+04, 1.389766E+04, 1.411259E+04, 1.433013E+04, 
   1.455030E+04, 1.477312E+04, 1.499861E+04, 1.522680E+04, 1.545771E+04, 1.569136E+04, 
   1.592778E+04, 1.616700E+04, 1.640903E+04, 1.665391E+04, 1.690165E+04, 1.715228E+04, 
   1.740583E+04, 1.766232E+04, 1.792177E+04, 1.818422E+04, 1.844968E+04, 1.871818E+04, 
   1.898975E+04, 1.926441E+04, 1.954219E+04, 1.982312E+04, 2.010722E+04, 2.039451E+04, 
   2.068503E+04, 2.097879E+04, 2.127583E+04, 2.157618E+04, 2.187985E+04, 2.218689E+04, 
   2.249730E+04, 2.281113E+04, 2.312839E+04, 2.344912E+04, 2.377335E+04, 2.410109E+04, 
   2.443239E+04, 2.476726E+04, 2.510573E+04, 2.544785E+04, 2.579362E+04, 2.614308E+04, 
   2.649626E+04, 2.685319E+04, 2.721390E+04, 2.757841E+04, 2.794676E+04, 2.831897E+04, 
   2.869508E+04, 2.907511E+04, 2.945910E+04, 2.984706E+04, 3.023904E+04, 3.063507E+04, 
   3.103517E+04, 3.143937E+04, 3.184771E+04, 3.226021E+04, 3.267691E+04, 3.309784E+04, 
   3.352303E+04, 3.395250E+04, 3.438630E+04, 3.482445E+04, 3.526699E+04, 3.571394E+04, 
   3.616534E+04, 3.662121E+04, 3.708161E+04, 3.754654E+04, 3.801606E+04, 3.849018E+04, 
   3.896895E+04, 3.945239E+04, 3.994054E+04, 4.043344E+04, 4.093111E+04, 4.143358E+04, 
   4.194090E+04, 4.245310E+04, 4.297021E+04, 4.349226E+04, 4.401929E+04, 4.455133E+04, 
   4.508842E+04, 4.563059E+04, 4.617788E+04, 4.673032E+04, 4.728795E+04, 4.785080E+04, 
   4.841890E+04, 4.899230E+04, 4.957103E+04, 5.015512E+04, 5.074461E+04, 5.133954E+04, 
   5.193994E+04, 5.254584E+04, 5.315730E+04, 5.377433E+04, 5.439699E+04, 5.502529E+04, 
   5.565930E+04, 5.629903E+04, 5.694453E+04, 5.759583E+04, 5.825298E+04, 5.891600E+04, 
   5.958495E+04, 6.025985E+04, 6.094075E+04, 6.162768E+04, 6.232068E+04, 6.301979E+04, 
   6.372505E+04, 6.443649E+04, 6.515417E+04, 6.587811E+04, 6.660836E+04, 6.734495E+04, 
   6.808793E+04, 6.883734E+04, 6.959321E+04, 7.035558E+04, 7.112450E+04, 7.190001E+04, 
   7.268215E+04, 7.347095E+04, 7.426646E+04, 7.506872E+04, 7.587778E+04, 7.669366E+04, 
   7.751642E+04, 7.834610E+04, 7.918273E+04, 8.002637E+04, 8.087704E+04, 8.173480E+04, 
   8.259969E+04, 8.347174E+04, 8.435101E+04, 8.523754E+04, 8.613136E+04, 8.703252E+04, 
   8.794107E+04, 8.885705E+04, 8.978050E+04, 9.071147E+04, 9.164999E+04, 9.259612E+04, 
   9.354990E+04, 9.451137E+04, 9.548058E+04, 9.645757E+04, 9.744240E+04, 9.843509E+04, 
   9.943570E+04, 1.004443E+05, 1.014609E+05, 1.024855E+05, 1.035182E+05, 1.045591E+05, 
   1.056082E+05, 1.066655E+05, 1.077311E+05, 1.088051E+05, 1.098874E+05, 1.109781E+05, 
   1.120773E+05, 1.131851E+05, 1.143014E+05, 1.154263E+05, 1.165599E+05, 1.177022E+05, 
   1.188532E+05, 1.200131E+05, 1.211818E+05, 1.223594E+05, 1.235460E+05, 1.247416E+05, 
   1.259462E+05, 1.271600E+05, 1.283829E+05, 1.296149E+05, 1.308563E+05, 1.321069E+05, 
   1.333669E+05, 1.346363E+05, 1.359151E+05, 1.372035E+05, 1.385014E+05, 1.398089E+05, 
   1.411260E+05, 1.424529E+05, 1.437895E+05, 1.451359E+05, 1.464922E+05, 1.478584E+05, 
   1.492345E+05, 1.506207E+05, 1.520169E+05, 1.534232E+05, 1.548398E+05, 1.562665E+05, 
   1.577036E+05, 1.591509E+05, 1.606087E+05, 1.620769E+05, 1.635556E+05, 1.650448E+05, 
   1.665446E+05, 1.680551E+05, 1.695763E+05, 1.711082E+05, 1.726510E+05, 1.742047E+05, 
   1.757692E+05, 1.773448E+05, 1.789314E+05, 1.805291E+05, 1.821379E+05, 1.837580E+05, 
   1.853893E+05, 1.870319E+05, 1.886859E+05, 1.903513E+05, 1.920283E+05, 1.937167E+05, 
   1.954168E+05, 1.971285E+05, 1.988519E+05, 2.005872E+05, 2.023342E+05, 2.040931E+05, 
   2.058640E+05, 2.076469E+05, 2.094418E+05, 2.112489E+05, 2.130681E+05, 2.148996E+05, 
   2.167434E+05, 2.185996E+05, 2.204681E+05, 2.223492E+05, 2.242428E+05, 2.261489E+05, 
   2.280678E+05, 2.299993E+05, 2.319436E+05, 2.339008E+05, 2.358708E+05, 2.378539E+05, 
   2.398499E+05, 2.418590E+05, 2.438813E+05, 2.459168E+05, 2.479655E+05, 2.500276E+05, 
   2.521030E+05, 2.541920E+05, 2.562944E+05, 2.584104E+05, 2.605401E+05, 2.626834E+05, 
   2.648405E+05, 2.670115E+05, 2.691963E+05, 2.713952E+05, 2.736080E+05, 2.758349E+05, 
   2.780760E+05, 2.803313E+05, 2.826009E+05, 2.848848E+05, 2.871831E+05, 2.894959E+05, 
   2.918232E+05, 2.941652E+05, 2.965218E+05, 2.988932E+05, 3.012793E+05, 3.036804E+05, 
   3.060963E+05, 3.085273E+05, 3.109734E+05, 3.134345E+05, 3.159109E+05, 3.184026E+05, 
   3.209096E+05, 3.234320E+05, 3.259698E+05, 3.285233E+05, 3.310923E+05, 3.336770E+05, 
   3.362775E+05, 3.388937E+05, 3.415259E+05, 3.441740E+05, 3.468381E+05, 3.495184E+05, 
   3.522148E+05, 3.549274E+05, 3.576563E+05, 3.604016E+05, 3.631634E+05, 3.659416E+05, 
   3.687365E+05, 3.715480E+05, 3.743762E+05, 3.772212E+05, 3.800831E+05, 3.829619E+05, 
   3.858578E+05, 3.887707E+05, 3.917008E+05, 3.946481E+05, 3.976127E+05, 4.005947E+05, 
   4.035942E+05, 4.066111E+05, 4.096457E+05, 4.126979E+05, 4.157679E+05, 4.188557E+05, 
   4.219613E+05, 4.250850E+05, 4.282267E+05, 
])
# ============================ M = 31, I = 3 ============================
M = 31
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   -1.955262E+01, 1.574803E+01, 3.938469E+01, 6.950250E+01, 1.049061E+02, 1.448932E+02, 
   1.889860E+02, 2.368314E+02, 2.881546E+02, 3.427337E+02, 4.003847E+02, 4.609517E+02, 
   5.243006E+02, 5.903144E+02, 6.588905E+02, 7.299379E+02, 8.033764E+02, 8.791346E+02, 
   9.571500E+02, 1.037368E+03, 1.119740E+03, 1.204227E+03, 1.290794E+03, 1.379413E+03, 
   1.470062E+03, 1.562724E+03, 1.657386E+03, 1.754040E+03, 1.852684E+03, 1.953316E+03, 
   2.055940E+03, 2.160563E+03, 2.267193E+03, 2.375842E+03, 2.486523E+03, 2.599252E+03, 
   2.714047E+03, 2.830926E+03, 2.949910E+03, 3.071020E+03, 3.194281E+03, 3.319716E+03, 
   3.447351E+03, 3.577212E+03, 3.709327E+03, 3.843724E+03, 3.980434E+03, 4.119485E+03, 
   4.260909E+03, 4.404739E+03, 4.551006E+03, 4.699744E+03, 4.850989E+03, 5.004774E+03, 
   5.161135E+03, 5.320110E+03, 5.481735E+03, 5.646049E+03, 5.813090E+03, 5.982897E+03, 
   6.155510E+03, 6.330971E+03, 6.509320E+03, 6.690599E+03, 6.874852E+03, 7.062120E+03, 
   7.252449E+03, 7.445882E+03, 7.642464E+03, 7.842242E+03, 8.045260E+03, 8.251565E+03, 
   8.461206E+03, 8.674229E+03, 8.890683E+03, 9.110616E+03, 9.334078E+03, 9.561118E+03, 
   9.791788E+03, 1.002614E+04, 1.026422E+04, 1.050608E+04, 1.075177E+04, 1.100136E+04, 
   1.125488E+04, 1.151240E+04, 1.177397E+04, 1.203964E+04, 1.230947E+04, 1.258350E+04, 
   1.286181E+04, 1.314444E+04, 1.343146E+04, 1.372291E+04, 1.401886E+04, 1.431936E+04, 
   1.462447E+04, 1.493425E+04, 1.524877E+04, 1.556807E+04, 1.589223E+04, 1.622129E+04, 
   1.655532E+04, 1.689439E+04, 1.723855E+04, 1.758786E+04, 1.794240E+04, 1.830221E+04, 
   1.866737E+04, 1.903794E+04, 1.941399E+04, 1.979556E+04, 2.018274E+04, 2.057559E+04, 
   2.097418E+04, 2.137856E+04, 2.178881E+04, 2.220499E+04, 2.262717E+04, 2.305542E+04, 
   2.348981E+04, 2.393041E+04, 2.437728E+04, 2.483050E+04, 2.529013E+04, 2.575625E+04, 
   2.622892E+04, 2.670823E+04, 2.719423E+04, 2.768701E+04, 2.818663E+04, 2.869316E+04, 
   2.920669E+04, 2.972729E+04, 3.025502E+04, 3.078997E+04, 3.133221E+04, 3.188182E+04, 
   3.243887E+04, 3.300344E+04, 3.357560E+04, 3.415544E+04, 3.474302E+04, 3.533844E+04, 
   3.594177E+04, 3.655308E+04, 3.717247E+04, 3.780000E+04, 3.843576E+04, 3.907983E+04, 
   3.973229E+04, 4.039323E+04, 4.106272E+04, 4.174086E+04, 4.242771E+04, 4.312338E+04, 
   4.382793E+04, 4.454146E+04, 4.526406E+04, 4.599580E+04, 4.673677E+04, 4.748707E+04, 
   4.824677E+04, 4.901597E+04, 4.979476E+04, 5.058322E+04, 5.138143E+04, 5.218951E+04, 
   5.300752E+04, 5.383557E+04, 5.467373E+04, 5.552212E+04, 5.638081E+04, 5.724990E+04, 
   5.812949E+04, 5.901966E+04, 5.992051E+04, 6.083214E+04, 6.175464E+04, 6.268810E+04, 
   6.363262E+04, 6.458831E+04, 6.555525E+04, 6.653354E+04, 6.752328E+04, 6.852457E+04, 
   6.953752E+04, 7.056221E+04, 7.159874E+04, 7.264723E+04, 7.370777E+04, 7.478045E+04, 
   7.586539E+04, 7.696269E+04, 7.807245E+04, 7.919476E+04, 8.032975E+04, 8.147751E+04, 
   8.263814E+04, 8.381176E+04, 8.499846E+04, 8.619836E+04, 8.741156E+04, 8.863817E+04, 
   8.987830E+04, 9.113206E+04, 9.239956E+04, 9.368090E+04, 9.497619E+04, 9.628556E+04, 
   9.760910E+04, 9.894694E+04, 1.002992E+05, 1.016659E+05, 1.030473E+05, 1.044434E+05, 
   1.058544E+05, 1.072804E+05, 1.087214E+05, 1.101777E+05, 1.116493E+05, 1.131363E+05, 
   1.146389E+05, 1.161571E+05, 1.176912E+05, 1.192411E+05, 1.208071E+05, 1.223892E+05, 
   1.239877E+05, 1.256025E+05, 1.272338E+05, 1.288818E+05, 1.305466E+05, 1.322282E+05, 
   1.339268E+05, 1.356426E+05, 1.373757E+05, 1.391261E+05, 1.408941E+05, 1.426797E+05, 
   1.444830E+05, 1.463043E+05, 1.481436E+05, 1.500011E+05, 1.518768E+05, 1.537710E+05, 
   1.556837E+05, 1.576151E+05, 1.595653E+05, 1.615344E+05, 1.635226E+05, 1.655301E+05, 
   1.675568E+05, 1.696031E+05, 1.716690E+05, 1.737546E+05, 1.758601E+05, 1.779857E+05, 
   1.801314E+05, 1.822974E+05, 1.844839E+05, 1.866909E+05, 1.889186E+05, 1.911673E+05, 
   1.934369E+05, 1.957276E+05, 1.980397E+05, 2.003732E+05, 2.027282E+05, 2.051050E+05, 
   2.075037E+05, 2.099243E+05, 2.123671E+05, 2.148322E+05, 2.173197E+05, 2.198299E+05, 
   2.223628E+05, 2.249185E+05, 2.274974E+05, 2.300994E+05, 2.327247E+05, 2.353735E+05, 
   2.380460E+05, 2.407423E+05, 2.434625E+05, 2.462068E+05, 2.489754E+05, 2.517684E+05, 
   2.545860E+05, 2.574283E+05, 2.602954E+05, 2.631876E+05, 2.661050E+05, 2.690478E+05, 
   2.720160E+05, 2.750099E+05, 2.780297E+05, 2.810754E+05, 2.841473E+05, 2.872455E+05, 
   2.903702E+05, 2.935215E+05, 2.966997E+05, 2.999048E+05, 3.031370E+05, 3.063965E+05, 
   3.096835E+05, 3.129981E+05, 3.163405E+05, 3.197109E+05, 3.231094E+05, 3.265362E+05, 
   3.299915E+05, 3.334754E+05, 3.369881E+05, 3.405299E+05, 3.441008E+05, 3.477010E+05, 
   3.513307E+05, 3.549901E+05, 3.586793E+05, 3.623986E+05, 3.661480E+05, 3.699279E+05, 
   3.737383E+05, 3.775795E+05, 3.814515E+05, 3.853547E+05, 3.892891E+05, 3.932550E+05, 
   3.972525E+05, 4.012818E+05, 4.053431E+05, 4.094366E+05, 4.135625E+05, 4.177209E+05, 
   4.219120E+05, 4.261361E+05, 4.303932E+05, 4.346837E+05, 4.390076E+05, 4.433652E+05, 
   4.477567E+05, 4.521821E+05, 4.566418E+05, 4.611360E+05, 4.656647E+05, 4.702283E+05, 
   4.748268E+05, 4.794606E+05, 4.841297E+05, 4.888344E+05, 4.935748E+05, 4.983513E+05, 
   5.031639E+05, 5.080128E+05, 5.128984E+05, 5.178206E+05, 5.227799E+05, 5.277762E+05, 
   5.328100E+05, 5.378813E+05, 5.429903E+05, 5.481373E+05, 5.533225E+05, 5.585460E+05, 
   5.638081E+05, 5.691090E+05, 5.744488E+05, 5.798278E+05, 5.852463E+05, 5.907043E+05, 
   5.962021E+05, 6.017399E+05, 6.073180E+05, 6.129365E+05, 6.185956E+05, 6.242956E+05, 
   6.300366E+05, 6.358189E+05, 6.416428E+05, 6.475083E+05, 6.534157E+05, 6.593653E+05, 
   6.653572E+05, 6.713917E+05, 6.774689E+05, 6.835892E+05, 6.897527E+05, 6.959596E+05, 
   7.022102E+05, 7.085047E+05, 7.148432E+05, 7.212261E+05, 7.276536E+05, 7.341258E+05, 
   7.406429E+05, 7.472053E+05, 7.538132E+05, 7.604667E+05, 7.671661E+05, 7.739116E+05, 
   7.807035E+05, 7.875419E+05, 7.944272E+05, 8.013595E+05, 8.083390E+05, 8.153661E+05, 
   8.224408E+05, 8.295636E+05, 8.367345E+05, 8.439539E+05, 8.512219E+05, 8.585388E+05, 
   8.659049E+05, 8.733203E+05, 8.807853E+05, 8.883002E+05, 8.958652E+05, 9.034805E+05, 
   9.111463E+05, 9.188630E+05, 9.266307E+05, 9.344497E+05, 9.423202E+05, 9.502425E+05, 
   9.582168E+05, 9.662434E+05, 9.743225E+05, 9.824544E+05, 9.906392E+05, 9.988773E+05, 
   1.007169E+06, 1.015514E+06, 1.023914E+06, 1.032367E+06, 1.040875E+06, 1.049438E+06, 
   1.058056E+06, 1.066729E+06, 1.075458E+06, 1.084242E+06, 1.093083E+06, 1.101979E+06, 
   1.110933E+06, 1.119943E+06, 1.129010E+06, 1.138134E+06, 1.147316E+06, 1.156556E+06, 
   1.165854E+06, 1.175210E+06, 1.184625E+06, 1.194099E+06, 1.203632E+06, 1.213224E+06, 
   1.222876E+06, 1.232588E+06, 1.242360E+06, 1.252192E+06, 1.262086E+06, 1.272040E+06, 
   1.282056E+06, 1.292133E+06, 1.302272E+06, 1.312473E+06, 1.322736E+06, 1.333062E+06, 
   1.343452E+06, 1.353904E+06, 1.364419E+06, 1.374999E+06, 1.385642E+06, 1.396350E+06, 
   1.407122E+06, 1.417959E+06, 1.428862E+06, 1.439829E+06, 1.450862E+06, 1.461962E+06, 
   1.473127E+06, 1.484360E+06, 1.495658E+06, 1.507024E+06, 1.518458E+06, 1.529959E+06, 
   1.541528E+06, 1.553166E+06, 1.564872E+06, 1.576646E+06, 1.588490E+06, 1.600403E+06, 
   1.612386E+06, 1.624439E+06, 1.636563E+06, 1.648756E+06, 1.661021E+06, 1.673357E+06, 
   1.685764E+06, 1.698244E+06, 1.710795E+06, 
])
# ============================ M = 32, I = 1 ============================
M = 32
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   8.097515E+00, 2.198477E+02, 6.167408E+02, 1.130094E+03, 1.737821E+03, 2.427140E+03, 
   3.189427E+03, 4.018367E+03, 4.909123E+03, 5.857947E+03, 6.862015E+03, 7.919354E+03, 
   9.028806E+03, 1.018999E+04, 1.140324E+04, 1.266956E+04, 1.399055E+04, 1.536833E+04, 
   1.680549E+04, 1.830508E+04, 1.987048E+04, 2.150549E+04, 2.321420E+04, 2.500105E+04, 
   2.687076E+04, 2.882837E+04, 3.087922E+04, 3.302895E+04, 3.528349E+04, 3.764910E+04, 
   4.013233E+04, 4.274006E+04, 4.547951E+04, 4.835822E+04, 5.138411E+04, 5.456544E+04, 
   5.791086E+04, 6.142941E+04, 6.513055E+04, 6.902415E+04, 7.312052E+04, 7.743045E+04, 
   8.196521E+04, 8.673653E+04, 9.175672E+04, 9.703858E+04, 1.025955E+05, 1.084414E+05, 
   1.145910E+05, 1.210594E+05, 1.278626E+05, 1.350170E+05, 1.425399E+05, 1.504495E+05, 
   1.587645E+05, 1.675045E+05, 1.766900E+05, 1.863423E+05, 1.964836E+05, 2.071370E+05, 
   2.183268E+05, 2.300780E+05, 2.424168E+05, 2.553706E+05, 2.689677E+05, 2.832377E+05, 
   2.982114E+05, 3.139210E+05, 3.303996E+05, 3.476822E+05, 3.658048E+05, 3.848049E+05, 
   4.047217E+05, 4.255958E+05, 4.474696E+05, 4.703868E+05, 4.943933E+05, 5.195364E+05, 
   5.458655E+05, 5.734318E+05, 6.022886E+05, 6.324912E+05, 6.640968E+05, 6.971652E+05, 
   7.317583E+05, 7.679402E+05, 8.057776E+05, 8.453396E+05, 8.866981E+05, 9.299274E+05, 
   9.751048E+05, 1.022310E+06, 1.071627E+06, 1.123141E+06, 1.176941E+06, 1.233120E+06, 
   1.291773E+06, 1.353001E+06, 1.416905E+06, 1.483592E+06, 1.553172E+06, 1.625761E+06, 
   1.701475E+06, 1.780438E+06, 1.862775E+06, 1.948619E+06, 2.038105E+06, 2.131373E+06, 
   2.228568E+06, 2.329842E+06, 2.435348E+06, 2.545248E+06, 2.659708E+06, 2.778900E+06, 
   2.903001E+06, 3.032194E+06, 3.166670E+06, 3.306624E+06, 3.452259E+06, 3.603783E+06, 
   3.761413E+06, 3.925372E+06, 4.095890E+06, 4.273205E+06, 4.457562E+06, 4.649215E+06, 
   4.848426E+06, 5.055464E+06, 5.270609E+06, 5.494147E+06, 5.726375E+06, 5.967599E+06, 
   6.218135E+06, 6.478307E+06, 6.748452E+06, 7.028915E+06, 7.320053E+06, 7.622234E+06, 
   7.935836E+06, 8.261250E+06, 8.598879E+06, 8.949137E+06, 9.312452E+06, 9.689262E+06, 
   1.008002E+07, 1.048520E+07, 1.090528E+07, 1.134074E+07, 1.179211E+07, 1.225991E+07, 
   1.274467E+07, 1.324696E+07, 1.376734E+07, 1.430641E+07, 1.486477E+07, 1.544304E+07, 
   1.604187E+07, 1.666192E+07, 1.730386E+07, 1.796839E+07, 1.865623E+07, 1.936812E+07, 
   2.010481E+07, 2.086710E+07, 2.165577E+07, 2.247166E+07, 2.331561E+07, 2.418850E+07, 
   2.509121E+07, 2.602466E+07, 2.698981E+07, 2.798763E+07, 2.901910E+07, 3.008526E+07, 
   3.118715E+07, 3.232586E+07, 3.350249E+07, 3.471819E+07, 3.597412E+07, 3.727148E+07, 
   3.861152E+07, 3.999548E+07, 4.142468E+07, 4.290043E+07, 4.442412E+07, 4.599714E+07, 
   4.762093E+07, 4.929697E+07, 5.102678E+07, 5.281190E+07, 5.465393E+07, 5.655452E+07, 
   5.851532E+07, 6.053807E+07, 6.262452E+07, 6.477649E+07, 6.699582E+07, 6.928441E+07, 
   7.164422E+07, 7.407724E+07, 7.658550E+07, 7.917112E+07, 8.183623E+07, 8.458304E+07, 
   8.741381E+07, 9.033083E+07, 9.333649E+07, 9.643320E+07, 9.962344E+07, 1.029098E+08, 
   1.062948E+08, 1.097811E+08, 1.133716E+08, 1.170689E+08, 1.208760E+08, 1.247957E+08, 
   1.288312E+08, 1.329853E+08, 1.372614E+08, 1.416626E+08, 1.461921E+08, 1.508535E+08, 
   1.556501E+08, 1.605854E+08, 1.656631E+08, 1.708869E+08, 1.762605E+08, 1.817878E+08, 
   1.874728E+08, 1.933194E+08, 1.993320E+08, 2.055146E+08, 2.118716E+08, 2.184074E+08, 
   2.251266E+08, 2.320337E+08, 2.391336E+08, 2.464310E+08, 2.539309E+08, 2.616383E+08, 
   2.695584E+08, 2.776965E+08, 2.860580E+08, 2.946483E+08, 3.034732E+08, 3.125384E+08, 
   3.218497E+08, 3.314131E+08, 3.412349E+08, 3.513212E+08, 3.616786E+08, 3.723134E+08, 
   3.832325E+08, 3.944426E+08, 4.059507E+08, 4.177640E+08, 4.298896E+08, 4.423351E+08, 
   4.551080E+08, 4.682160E+08, 4.816671E+08, 4.954694E+08, 5.096310E+08, 5.241603E+08, 
   5.390660E+08, 5.543568E+08, 5.700417E+08, 5.861298E+08, 6.026304E+08, 6.195529E+08, 
   6.369072E+08, 6.547030E+08, 6.729505E+08, 6.916600E+08, 7.108420E+08, 7.305072E+08, 
   7.506666E+08, 7.713312E+08, 7.925126E+08, 8.142222E+08, 8.364720E+08, 8.592740E+08, 
   8.826405E+08, 9.065840E+08, 9.311174E+08, 9.562538E+08, 9.820063E+08, 1.008389E+09, 
   1.035415E+09, 1.063098E+09, 1.091454E+09, 1.120497E+09, 1.150241E+09, 1.180703E+09, 
   1.211896E+09, 1.243838E+09, 1.276545E+09, 1.310032E+09, 1.344317E+09, 1.379416E+09, 
   1.415348E+09, 1.452129E+09, 1.489778E+09, 1.528313E+09, 1.567753E+09, 1.608117E+09, 
   1.649425E+09, 1.691695E+09, 1.734949E+09, 1.779207E+09, 1.824490E+09, 1.870819E+09, 
   1.918216E+09, 1.966703E+09, 2.016302E+09, 2.067036E+09, 2.118930E+09, 2.172006E+09, 
   2.226288E+09, 2.281802E+09, 2.338572E+09, 2.396624E+09, 2.455984E+09, 2.516678E+09, 
   2.578734E+09, 2.642178E+09, 2.707039E+09, 2.773345E+09, 2.841125E+09, 2.910408E+09, 
   2.981224E+09, 3.053604E+09, 3.127579E+09, 3.203180E+09, 3.280439E+09, 3.359389E+09, 
   3.440064E+09, 3.522496E+09, 3.606721E+09, 3.692773E+09, 3.780688E+09, 3.870503E+09, 
   3.962253E+09, 4.055977E+09, 4.151712E+09, 4.249498E+09, 4.349374E+09, 4.451379E+09, 
   4.555555E+09, 4.661943E+09, 4.770585E+09, 4.881524E+09, 4.994804E+09, 5.110469E+09, 
   5.228564E+09, 5.349135E+09, 5.472228E+09, 5.597892E+09, 5.726173E+09, 5.857121E+09, 
   5.990786E+09, 6.127218E+09, 6.266469E+09, 6.408591E+09, 6.553638E+09, 6.701662E+09, 
   6.852719E+09, 7.006865E+09, 7.164156E+09, 7.324651E+09, 7.488407E+09, 7.655484E+09, 
   7.825943E+09, 7.999845E+09, 8.177252E+09, 8.358228E+09, 8.542837E+09, 8.731145E+09, 
   8.923218E+09, 9.119124E+09, 9.318931E+09, 9.522709E+09, 9.730529E+09, 9.942463E+09, 
   1.015858E+10, 1.037897E+10, 1.060368E+10, 1.083282E+10, 1.106644E+10, 1.130463E+10, 
   1.154747E+10, 1.179505E+10, 1.204744E+10, 1.230472E+10, 1.256700E+10, 1.283434E+10, 
   1.310684E+10, 1.338458E+10, 1.366767E+10, 1.395618E+10, 1.425022E+10, 1.454988E+10, 
   1.485525E+10, 1.516644E+10, 1.548353E+10, 1.580664E+10, 1.613587E+10, 1.647131E+10, 
   1.681308E+10, 1.716128E+10, 1.751602E+10, 1.787742E+10, 1.824557E+10, 1.862060E+10, 
   1.900262E+10, 1.939174E+10, 1.978810E+10, 2.019180E+10, 2.060297E+10, 2.102173E+10, 
   2.144821E+10, 2.188254E+10, 2.232484E+10, 2.277526E+10, 2.323391E+10, 2.370094E+10, 
   2.417648E+10, 2.466068E+10, 2.515367E+10, 2.565559E+10, 2.616660E+10, 2.668684E+10, 
   2.721646E+10, 2.775561E+10, 2.830445E+10, 2.886312E+10, 2.943180E+10, 3.001064E+10, 
   3.059980E+10, 3.119945E+10, 3.180976E+10, 3.243089E+10, 3.306302E+10, 3.370632E+10, 
   3.436097E+10, 3.502715E+10, 3.570505E+10, 3.639484E+10, 3.709671E+10, 3.781086E+10, 
   3.853748E+10, 3.927676E+10, 4.002889E+10, 4.079409E+10, 4.157255E+10, 4.236448E+10, 
   4.317010E+10, 4.398960E+10, 4.482321E+10, 4.567114E+10, 4.653362E+10, 4.741087E+10, 
   4.830311E+10, 4.921058E+10, 5.013351E+10, 5.107213E+10, 5.202669E+10, 5.299743E+10, 
   5.398459E+10, 5.498842E+10, 5.600917E+10, 5.704711E+10, 5.810249E+10, 5.917557E+10, 
   6.026663E+10, 6.137592E+10, 6.250373E+10, 6.365032E+10, 6.481600E+10, 6.600102E+10, 
   6.720569E+10, 6.843030E+10, 6.967514E+10, 7.094052E+10, 7.222673E+10, 7.353409E+10, 
   7.486290E+10, 7.621349E+10, 7.758617E+10, 7.898126E+10, 8.039910E+10, 8.184002E+10, 
   8.330436E+10, 8.479245E+10, 8.630464E+10, 8.784129E+10, 8.940274E+10, 9.098936E+10, 
   9.260152E+10, 9.423956E+10, 9.590389E+10, 
])
# ============================ M = 33, I = 1 ============================
M = 33
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.591148E-01, 2.788376E+01, 7.687714E+01, 1.400317E+02, 2.146761E+02, 2.992540E+02, 
   3.927114E+02, 4.942728E+02, 6.033367E+02, 7.194184E+02, 8.421168E+02, 9.710929E+02, 
   1.106056E+03, 1.246752E+03, 1.392962E+03, 1.544492E+03, 1.701173E+03, 1.862860E+03, 
   2.029427E+03, 2.200772E+03, 2.376810E+03, 2.557476E+03, 2.742721E+03, 2.932517E+03, 
   3.126849E+03, 3.325720E+03, 3.529145E+03, 3.737153E+03, 3.949787E+03, 4.167097E+03, 
   4.389147E+03, 4.616008E+03, 4.847759E+03, 5.084485E+03, 5.326281E+03, 5.573243E+03, 
   5.825474E+03, 6.083082E+03, 6.346176E+03, 6.614871E+03, 6.889284E+03, 7.169532E+03, 
   7.455737E+03, 7.748021E+03, 8.046508E+03, 8.351325E+03, 8.662596E+03, 8.980451E+03, 
   9.305017E+03, 9.636424E+03, 9.974801E+03, 1.032028E+04, 1.067299E+04, 1.103307E+04, 
   1.140064E+04, 1.177585E+04, 1.215882E+04, 1.254969E+04, 1.294859E+04, 1.335566E+04, 
   1.377104E+04, 1.419487E+04, 1.462727E+04, 1.506840E+04, 1.551839E+04, 1.597737E+04, 
   1.644549E+04, 1.692290E+04, 1.740974E+04, 1.790614E+04, 1.841225E+04, 1.892823E+04, 
   1.945421E+04, 1.999034E+04, 2.053677E+04, 2.109366E+04, 2.166114E+04, 2.223938E+04, 
   2.282852E+04, 2.342873E+04, 2.404014E+04, 2.466293E+04, 2.529725E+04, 2.594325E+04, 
   2.660110E+04, 2.727096E+04, 2.795299E+04, 2.864735E+04, 2.935421E+04, 3.007373E+04, 
   3.080609E+04, 3.155144E+04, 3.230997E+04, 3.308183E+04, 3.386721E+04, 3.466628E+04, 
   3.547921E+04, 3.630618E+04, 3.714737E+04, 3.800296E+04, 3.887312E+04, 3.975804E+04, 
   4.065790E+04, 4.157289E+04, 4.250320E+04, 4.344900E+04, 4.441050E+04, 4.538788E+04, 
   4.638132E+04, 4.739104E+04, 4.841721E+04, 4.946004E+04, 5.051973E+04, 5.159647E+04, 
   5.269046E+04, 5.380191E+04, 5.493101E+04, 5.607798E+04, 5.724301E+04, 5.842633E+04, 
   5.962813E+04, 6.084862E+04, 6.208803E+04, 6.334656E+04, 6.462442E+04, 6.592184E+04, 
   6.723903E+04, 6.857621E+04, 6.993360E+04, 7.131142E+04, 7.270991E+04, 7.412928E+04, 
   7.556976E+04, 7.703157E+04, 7.851496E+04, 8.002015E+04, 8.154737E+04, 8.309686E+04, 
   8.466885E+04, 8.626359E+04, 8.788130E+04, 8.952224E+04, 9.118664E+04, 9.287475E+04, 
   9.458681E+04, 9.632307E+04, 9.808378E+04, 9.986918E+04, 1.016795E+05, 1.035151E+05, 
   1.053761E+05, 1.072628E+05, 1.091755E+05, 1.111145E+05, 1.130799E+05, 1.150720E+05, 
   1.170912E+05, 1.191377E+05, 1.212117E+05, 1.233136E+05, 1.254435E+05, 1.276018E+05, 
   1.297887E+05, 1.320045E+05, 1.342495E+05, 1.365240E+05, 1.388282E+05, 1.411625E+05, 
   1.435270E+05, 1.459222E+05, 1.483482E+05, 1.508054E+05, 1.532941E+05, 1.558145E+05, 
   1.583670E+05, 1.609518E+05, 1.635692E+05, 1.662195E+05, 1.689031E+05, 1.716202E+05, 
   1.743712E+05, 1.771563E+05, 1.799758E+05, 1.828301E+05, 1.857194E+05, 1.886441E+05, 
   1.916045E+05, 1.946010E+05, 1.976337E+05, 2.007030E+05, 2.038094E+05, 2.069529E+05, 
   2.101341E+05, 2.133532E+05, 2.166106E+05, 2.199065E+05, 2.232413E+05, 2.266154E+05, 
   2.300290E+05, 2.334826E+05, 2.369764E+05, 2.405108E+05, 2.440860E+05, 2.477026E+05, 
   2.513607E+05, 2.550608E+05, 2.588032E+05, 2.625883E+05, 2.664163E+05, 2.702877E+05, 
   2.742028E+05, 2.781619E+05, 2.821654E+05, 2.862138E+05, 2.903072E+05, 2.944462E+05, 
   2.986310E+05, 3.028620E+05, 3.071396E+05, 3.114642E+05, 3.158361E+05, 3.202557E+05, 
   3.247234E+05, 3.292396E+05, 3.338046E+05, 3.384188E+05, 3.430826E+05, 3.477964E+05, 
   3.525605E+05, 3.573754E+05, 3.622415E+05, 3.671591E+05, 3.721286E+05, 3.771504E+05, 
   3.822250E+05, 3.873526E+05, 3.925338E+05, 3.977689E+05, 4.030583E+05, 4.084024E+05, 
   4.138017E+05, 4.192565E+05, 4.247673E+05, 4.303344E+05, 4.359583E+05, 4.416394E+05, 
   4.473781E+05, 4.531748E+05, 4.590300E+05, 4.649441E+05, 4.709175E+05, 4.769506E+05, 
   4.830439E+05, 4.891978E+05, 4.954127E+05, 5.016890E+05, 5.080273E+05, 5.144278E+05, 
   5.208912E+05, 5.274178E+05, 5.340080E+05, 5.406623E+05, 5.473812E+05, 5.541652E+05, 
   5.610145E+05, 5.679298E+05, 5.749114E+05, 5.819599E+05, 5.890756E+05, 5.962591E+05, 
   6.035108E+05, 6.108312E+05, 6.182207E+05, 6.256798E+05, 6.332090E+05, 6.408087E+05, 
   6.484795E+05, 6.562217E+05, 6.640360E+05, 6.719227E+05, 6.798823E+05, 6.879154E+05, 
   6.960224E+05, 7.042038E+05, 7.124600E+05, 7.207917E+05, 7.291992E+05, 7.376831E+05, 
   7.462439E+05, 7.548821E+05, 7.635981E+05, 7.723925E+05, 7.812657E+05, 7.902184E+05, 
   7.992510E+05, 8.083639E+05, 8.175578E+05, 8.268332E+05, 8.361904E+05, 8.456302E+05, 
   8.551529E+05, 8.647592E+05, 8.744495E+05, 8.842244E+05, 8.940844E+05, 9.040300E+05, 
   9.140618E+05, 9.241803E+05, 9.343860E+05, 9.446796E+05, 9.550614E+05, 9.655321E+05, 
   9.760922E+05, 9.867422E+05, 9.974828E+05, 1.008314E+06, 1.019238E+06, 1.030253E+06, 
   1.041361E+06, 1.052562E+06, 1.063858E+06, 1.075247E+06, 1.086732E+06, 1.098312E+06, 
   1.109988E+06, 1.121761E+06, 1.133632E+06, 1.145600E+06, 1.157666E+06, 1.169831E+06, 
   1.182096E+06, 1.194462E+06, 1.206927E+06, 1.219495E+06, 1.232164E+06, 1.244935E+06, 
   1.257810E+06, 1.270788E+06, 1.283871E+06, 1.297059E+06, 1.310352E+06, 1.323751E+06, 
   1.337257E+06, 1.350871E+06, 1.364592E+06, 1.378422E+06, 1.392360E+06, 1.406409E+06, 
   1.420568E+06, 1.434838E+06, 1.449220E+06, 1.463714E+06, 1.478321E+06, 1.493042E+06, 
   1.507876E+06, 1.522826E+06, 1.537891E+06, 1.553072E+06, 1.568369E+06, 1.583784E+06, 
   1.599317E+06, 1.614969E+06, 1.630740E+06, 1.646631E+06, 1.662642E+06, 1.678775E+06, 
   1.695029E+06, 1.711406E+06, 1.727907E+06, 1.744531E+06, 1.761280E+06, 1.778154E+06, 
   1.795154E+06, 1.812281E+06, 1.829535E+06, 1.846916E+06, 1.864427E+06, 1.882066E+06, 
   1.899836E+06, 1.917736E+06, 1.935768E+06, 1.953931E+06, 1.972228E+06, 1.990657E+06, 
   2.009221E+06, 2.027920E+06, 2.046754E+06, 2.065724E+06, 2.084832E+06, 2.104077E+06, 
   2.123460E+06, 2.142982E+06, 2.162645E+06, 2.182447E+06, 2.202391E+06, 2.222477E+06, 
   2.242705E+06, 2.263077E+06, 2.283592E+06, 2.304253E+06, 2.325059E+06, 2.346012E+06, 
   2.367111E+06, 2.388359E+06, 2.409754E+06, 2.431299E+06, 2.452994E+06, 2.474840E+06, 
   2.496837E+06, 2.518987E+06, 2.541289E+06, 2.563746E+06, 2.586356E+06, 2.609122E+06, 
   2.632044E+06, 2.655123E+06, 2.678360E+06, 2.701754E+06, 2.725308E+06, 2.749022E+06, 
   2.772896E+06, 2.796932E+06, 2.821130E+06, 2.845491E+06, 2.870016E+06, 2.894706E+06, 
   2.919560E+06, 2.944581E+06, 2.969769E+06, 2.995125E+06, 3.020649E+06, 3.046343E+06, 
   3.072206E+06, 3.098241E+06, 3.124448E+06, 3.150826E+06, 3.177379E+06, 3.204106E+06, 
   3.231007E+06, 3.258085E+06, 3.285339E+06, 3.312771E+06, 3.340380E+06, 3.368170E+06, 
   3.396139E+06, 3.424289E+06, 3.452620E+06, 3.481134E+06, 3.509832E+06, 3.538713E+06, 
   3.567780E+06, 3.597032E+06, 3.626472E+06, 3.656098E+06, 3.685914E+06, 3.715918E+06, 
   3.746113E+06, 3.776498E+06, 3.807076E+06, 3.837846E+06, 3.868810E+06, 3.899968E+06, 
   3.931322E+06, 3.962872E+06, 3.994619E+06, 4.026564E+06, 4.058708E+06, 4.091051E+06, 
   4.123595E+06, 4.156341E+06, 4.189289E+06, 4.222441E+06, 4.255797E+06, 4.289357E+06, 
   4.323124E+06, 4.357098E+06, 4.391280E+06, 4.425670E+06, 4.460270E+06, 4.495080E+06, 
   4.530103E+06, 4.565337E+06, 4.600785E+06, 4.636447E+06, 4.672324E+06, 4.708417E+06, 
   4.744728E+06, 4.781256E+06, 4.818003E+06, 4.854970E+06, 4.892158E+06, 4.929567E+06, 
   4.967199E+06, 5.005055E+06, 5.043135E+06, 5.081441E+06, 5.119974E+06, 5.158733E+06, 
   5.197721E+06, 5.236939E+06, 5.276387E+06, 
])
# ============================ M = 35, I = 1 ============================
M = 35
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.408486E+02, 7.370638E+03, 2.081703E+04, 3.833246E+04, 5.958466E+04, 8.480327E+04, 
   1.144161E+05, 1.489328E+05, 1.889250E+05, 2.350292E+05, 2.879545E+05, 3.484919E+05, 
   4.175250E+05, 4.960407E+05, 5.851411E+05, 6.860561E+05, 8.001556E+05, 9.289633E+05, 
   1.074171E+06, 1.237654E+06, 1.421487E+06, 1.627960E+06, 1.859601E+06, 2.119192E+06, 
   2.409795E+06, 2.734771E+06, 3.097810E+06, 3.502956E+06, 3.954635E+06, 4.457691E+06, 
   5.017416E+06, 5.639588E+06, 6.330509E+06, 7.097050E+06, 7.946691E+06, 8.887570E+06, 
   9.928538E+06, 1.107920E+07, 1.235001E+07, 1.375226E+07, 1.529822E+07, 1.700119E+07, 
   1.887552E+07, 2.093678E+07, 2.320174E+07, 2.568856E+07, 2.841681E+07, 3.140759E+07, 
   3.468367E+07, 3.826953E+07, 4.219154E+07, 4.647808E+07, 5.115962E+07, 5.626891E+07, 
   6.184111E+07, 6.791395E+07, 7.452788E+07, 8.172624E+07, 8.955546E+07, 9.806526E+07, 
   1.073088E+08, 1.173429E+08, 1.282284E+08, 1.400301E+08, 1.528173E+08, 1.666640E+08, 
   1.816490E+08, 1.978563E+08, 2.153754E+08, 2.343017E+08, 2.547366E+08, 2.767880E+08, 
   3.005708E+08, 3.262069E+08, 3.538259E+08, 3.835654E+08, 4.155714E+08, 4.499991E+08, 
   4.870125E+08, 5.267860E+08, 5.695042E+08, 6.153624E+08, 6.645677E+08, 7.173392E+08, 
   7.739084E+08, 8.345204E+08, 8.994342E+08, 9.689234E+08, 1.043277E+09, 1.122800E+09, 
   1.207815E+09, 1.298662E+09, 1.395698E+09, 1.499302E+09, 1.609872E+09, 1.727828E+09, 
   1.853610E+09, 1.987686E+09, 2.130544E+09, 2.282700E+09, 2.444694E+09, 2.617099E+09, 
   2.800511E+09, 2.995561E+09, 3.202910E+09, 3.423251E+09, 3.657315E+09, 3.905865E+09, 
   4.169705E+09, 4.449677E+09, 4.746663E+09, 5.061590E+09, 5.395426E+09, 5.749188E+09, 
   6.123942E+09, 6.520801E+09, 6.940932E+09, 7.385557E+09, 7.855954E+09, 8.353459E+09, 
   8.879471E+09, 9.435450E+09, 1.002293E+10, 1.064349E+10, 1.129882E+10, 1.199065E+10, 
   1.272080E+10, 1.349117E+10, 1.430374E+10, 1.516058E+10, 1.606385E+10, 1.701580E+10, 
   1.801877E+10, 1.907522E+10, 2.018768E+10, 2.135883E+10, 2.259142E+10, 2.388835E+10, 
   2.525261E+10, 2.668733E+10, 2.819577E+10, 2.978131E+10, 3.144749E+10, 3.319796E+10, 
   3.503654E+10, 3.696720E+10, 3.899407E+10, 4.112142E+10, 4.335372E+10, 4.569560E+10, 
   4.815186E+10, 5.072750E+10, 5.342772E+10, 5.625789E+10, 5.922362E+10, 6.233070E+10, 
   6.558517E+10, 6.899328E+10, 7.256151E+10, 7.629659E+10, 8.020550E+10, 8.429546E+10, 
   8.857399E+10, 9.304885E+10, 9.772811E+10, 1.026201E+11, 1.077335E+11, 1.130772E+11, 
   1.186605E+11, 1.244931E+11, 1.305849E+11, 1.369461E+11, 1.435875E+11, 1.505201E+11, 
   1.577552E+11, 1.653047E+11, 1.731809E+11, 1.813963E+11, 1.899640E+11, 1.988975E+11, 
   2.082108E+11, 2.179183E+11, 2.280350E+11, 2.385762E+11, 2.495580E+11, 2.609966E+11, 
   2.729093E+11, 2.853135E+11, 2.982273E+11, 3.116694E+11, 3.256593E+11, 3.402169E+11, 
   3.553627E+11, 3.711180E+11, 3.875047E+11, 4.045455E+11, 4.222638E+11, 4.406835E+11, 
   4.598295E+11, 4.797274E+11, 5.004038E+11, 5.218856E+11, 5.442011E+11, 5.673791E+11, 
   5.914495E+11, 6.164430E+11, 6.423912E+11, 6.693268E+11, 6.972832E+11, 7.262953E+11, 
   7.563985E+11, 7.876296E+11, 8.200263E+11, 8.536276E+11, 8.884735E+11, 9.246053E+11, 
   9.620653E+11, 1.000897E+12, 1.041146E+12, 1.082858E+12, 1.126080E+12, 1.170862E+12, 
   1.217254E+12, 1.265308E+12, 1.315076E+12, 1.366614E+12, 1.419978E+12, 1.475226E+12, 
   1.532416E+12, 1.591611E+12, 1.652873E+12, 1.716267E+12, 1.781858E+12, 1.849716E+12, 
   1.919909E+12, 1.992511E+12, 2.067594E+12, 2.145236E+12, 2.225513E+12, 2.308506E+12, 
   2.394296E+12, 2.482969E+12, 2.574610E+12, 2.669309E+12, 2.767157E+12, 2.868248E+12, 
   2.972677E+12, 3.080544E+12, 3.191949E+12, 3.306996E+12, 3.425792E+12, 3.548446E+12, 
   3.675069E+12, 3.805778E+12, 3.940690E+12, 4.079925E+12, 4.223608E+12, 4.371866E+12, 
   4.524828E+12, 4.682629E+12, 4.845405E+12, 5.013297E+12, 5.186448E+12, 5.365006E+12, 
   5.549122E+12, 5.738950E+12, 5.934650E+12, 6.136382E+12, 6.344315E+12, 6.558618E+12, 
   6.779465E+12, 7.007036E+12, 7.241514E+12, 7.483086E+12, 7.731943E+12, 7.988284E+12, 
   8.252308E+12, 8.524222E+12, 8.804237E+12, 9.092569E+12, 9.389438E+12, 9.695071E+12, 
   1.000970E+13, 1.033356E+13, 1.066689E+13, 1.100995E+13, 1.136298E+13, 1.172625E+13, 
   1.210002E+13, 1.248456E+13, 1.288015E+13, 1.328707E+13, 1.370562E+13, 1.413609E+13, 
   1.457879E+13, 1.503402E+13, 1.550210E+13, 1.598337E+13, 1.647814E+13, 1.698676E+13, 
   1.750958E+13, 1.804695E+13, 1.859923E+13, 1.916680E+13, 1.975002E+13, 2.034930E+13, 
   2.096501E+13, 2.159757E+13, 2.224740E+13, 2.291490E+13, 2.360051E+13, 2.430468E+13, 
   2.502784E+13, 2.577046E+13, 2.653300E+13, 2.731595E+13, 2.811980E+13, 2.894503E+13, 
   2.979217E+13, 3.066173E+13, 3.155424E+13, 3.247024E+13, 3.341029E+13, 3.437496E+13, 
   3.536481E+13, 3.638044E+13, 3.742245E+13, 3.849145E+13, 3.958806E+13, 4.071293E+13, 
   4.186671E+13, 4.305006E+13, 4.426365E+13, 4.550819E+13, 4.678437E+13, 4.809292E+13, 
   4.943458E+13, 5.081008E+13, 5.222020E+13, 5.366571E+13, 5.514740E+13, 5.666610E+13, 
   5.822263E+13, 5.981783E+13, 6.145255E+13, 6.312768E+13, 6.484411E+13, 6.660275E+13, 
   6.840453E+13, 7.025040E+13, 7.214132E+13, 7.407827E+13, 7.606226E+13, 7.809432E+13, 
   8.017547E+13, 8.230679E+13, 8.448936E+13, 8.672428E+13, 8.901267E+13, 9.135567E+13, 
   9.375447E+13, 9.621023E+13, 9.872418E+13, 1.012976E+14, 1.039316E+14, 1.066276E+14, 
   1.093868E+14, 1.122107E+14, 1.151005E+14, 1.180576E+14, 1.210834E+14, 1.241794E+14, 
   1.273470E+14, 1.305876E+14, 1.339029E+14, 1.372943E+14, 1.407634E+14, 1.443118E+14, 
   1.479412E+14, 1.516530E+14, 1.554492E+14, 1.593313E+14, 1.633011E+14, 1.673603E+14, 
   1.715109E+14, 1.757546E+14, 1.800932E+14, 1.845288E+14, 1.890632E+14, 1.936984E+14, 
   1.984363E+14, 2.032792E+14, 2.082289E+14, 2.132877E+14, 2.184577E+14, 2.237410E+14, 
   2.291400E+14, 2.346568E+14, 2.402938E+14, 2.460532E+14, 2.519376E+14, 2.579493E+14, 
   2.640908E+14, 2.703646E+14, 2.767732E+14, 2.833192E+14, 2.900053E+14, 2.968342E+14, 
   3.038085E+14, 3.109312E+14, 3.182048E+14, 3.256325E+14, 3.332170E+14, 3.409614E+14, 
   3.488685E+14, 3.569416E+14, 3.651838E+14, 3.735981E+14, 3.821879E+14, 3.909563E+14, 
   3.999068E+14, 4.090427E+14, 4.183674E+14, 4.278845E+14, 4.375974E+14, 4.475098E+14, 
   4.576255E+14, 4.679480E+14, 4.784812E+14, 4.892289E+14, 5.001951E+14, 5.113837E+14, 
   5.227987E+14, 5.344443E+14, 5.463247E+14, 5.584440E+14, 5.708066E+14, 5.834168E+14, 
   5.962792E+14, 6.093981E+14, 6.227782E+14, 6.364242E+14, 6.503407E+14, 6.645326E+14, 
   6.790047E+14, 6.937621E+14, 7.088097E+14, 7.241526E+14, 7.397961E+14, 7.557454E+14, 
   7.720058E+14, 7.885829E+14, 8.054821E+14, 8.227090E+14, 8.402693E+14, 8.581688E+14, 
   8.764134E+14, 8.950090E+14, 9.139616E+14, 9.332775E+14, 9.529628E+14, 9.730239E+14, 
   9.934671E+14, 1.014299E+15, 1.035526E+15, 1.057156E+15, 1.079194E+15, 1.101648E+15, 
   1.124525E+15, 1.147832E+15, 1.171575E+15, 1.195764E+15, 1.220404E+15, 1.245504E+15, 
   1.271071E+15, 1.297114E+15, 1.323639E+15, 1.350655E+15, 1.378170E+15, 1.406192E+15, 
   1.434730E+15, 1.463792E+15, 1.493387E+15, 1.523524E+15, 1.554211E+15, 1.585458E+15, 
   1.617273E+15, 1.649666E+15, 1.682646E+15, 1.716223E+15, 1.750406E+15, 1.785206E+15, 
   1.820632E+15, 1.856694E+15, 1.893402E+15, 1.930768E+15, 1.968801E+15, 2.007512E+15, 
   2.046912E+15, 2.087013E+15, 2.127824E+15, 
])
# ============================ M = 35, I = 2 ============================
M = 35
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.467757E+02, 7.557655E+03, 2.134608E+04, 3.930720E+04, 6.110022E+04, 8.696063E+04, 
   1.173271E+05, 1.527223E+05, 1.937324E+05, 2.410100E+05, 2.952823E+05, 3.573605E+05, 
   4.281507E+05, 5.086648E+05, 6.000331E+05, 7.035166E+05, 8.205203E+05, 9.526066E+05, 
   1.101510E+06, 1.269155E+06, 1.457666E+06, 1.669395E+06, 1.906932E+06, 2.173131E+06, 
   2.471131E+06, 2.804379E+06, 3.176659E+06, 3.592117E+06, 4.055293E+06, 4.571154E+06, 
   5.145126E+06, 5.783135E+06, 6.491643E+06, 7.277696E+06, 8.148963E+06, 9.113792E+06, 
   1.018126E+07, 1.136121E+07, 1.266436E+07, 1.410231E+07, 1.568762E+07, 1.743394E+07, 
   1.935598E+07, 2.146970E+07, 2.379232E+07, 2.634244E+07, 2.914014E+07, 3.220705E+07, 
   3.556652E+07, 3.924366E+07, 4.326551E+07, 4.766115E+07, 5.246186E+07, 5.770121E+07, 
   6.341525E+07, 6.964268E+07, 7.642496E+07, 8.380655E+07, 9.183507E+07, 1.005615E+08, 
   1.100403E+08, 1.203298E+08, 1.314924E+08, 1.435945E+08, 1.567073E+08, 1.709064E+08, 
   1.862728E+08, 2.028927E+08, 2.208578E+08, 2.402658E+08, 2.612209E+08, 2.838336E+08, 
   3.082218E+08, 3.345105E+08, 3.628325E+08, 3.933290E+08, 4.261498E+08, 4.614538E+08, 
   4.994094E+08, 5.401954E+08, 5.840010E+08, 6.310265E+08, 6.814844E+08, 7.355991E+08, 
   7.936083E+08, 8.557632E+08, 9.223294E+08, 9.935875E+08, 1.069834E+09, 1.151381E+09, 
   1.238560E+09, 1.331719E+09, 1.431226E+09, 1.537467E+09, 1.650852E+09, 1.771810E+09, 
   1.900795E+09, 2.038283E+09, 2.184778E+09, 2.340806E+09, 2.506925E+09, 2.683718E+09, 
   2.871799E+09, 3.071814E+09, 3.284440E+09, 3.510391E+09, 3.750413E+09, 4.005290E+09, 
   4.275846E+09, 4.562945E+09, 4.867491E+09, 5.190434E+09, 5.532768E+09, 5.895536E+09, 
   6.279829E+09, 6.686790E+09, 7.117616E+09, 7.573560E+09, 8.055931E+09, 8.566100E+09, 
   9.105502E+09, 9.675634E+09, 1.027806E+10, 1.091443E+10, 1.158644E+10, 1.229588E+10, 
   1.304461E+10, 1.383459E+10, 1.466785E+10, 1.554650E+10, 1.647276E+10, 1.744894E+10, 
   1.847745E+10, 1.956078E+10, 2.070157E+10, 2.190253E+10, 2.316650E+10, 2.449644E+10, 
   2.589543E+10, 2.736667E+10, 2.891351E+10, 3.053941E+10, 3.224800E+10, 3.404303E+10, 
   3.592842E+10, 3.790822E+10, 3.998668E+10, 4.216819E+10, 4.445731E+10, 4.685880E+10, 
   4.937759E+10, 5.201880E+10, 5.478775E+10, 5.768997E+10, 6.073119E+10, 6.391737E+10, 
   6.725468E+10, 7.074955E+10, 7.440861E+10, 7.823877E+10, 8.224718E+10, 8.644126E+10, 
   9.082870E+10, 9.541747E+10, 1.002158E+11, 1.052324E+11, 1.104759E+11, 1.159556E+11, 
   1.216811E+11, 1.276622E+11, 1.339090E+11, 1.404322E+11, 1.472426E+11, 1.543516E+11, 
   1.617710E+11, 1.695127E+11, 1.775893E+11, 1.860138E+11, 1.947996E+11, 2.039606E+11, 
   2.135109E+11, 2.234656E+11, 2.338398E+11, 2.446494E+11, 2.559106E+11, 2.676405E+11, 
   2.798564E+11, 2.925763E+11, 3.058188E+11, 3.196032E+11, 3.339492E+11, 3.488773E+11, 
   3.644087E+11, 3.805650E+11, 3.973689E+11, 4.148435E+11, 4.330128E+11, 4.519014E+11, 
   4.715348E+11, 4.919392E+11, 5.131419E+11, 5.351706E+11, 5.580541E+11, 5.818222E+11, 
   6.065053E+11, 6.321350E+11, 6.587437E+11, 6.863650E+11, 7.150331E+11, 7.447836E+11, 
   7.756532E+11, 8.076792E+11, 8.409007E+11, 8.753573E+11, 9.110903E+11, 9.481418E+11, 
   9.865553E+11, 1.026376E+12, 1.067649E+12, 1.110423E+12, 1.154746E+12, 1.200668E+12, 
   1.248240E+12, 1.297517E+12, 1.348553E+12, 1.401402E+12, 1.456125E+12, 1.512779E+12, 
   1.571425E+12, 1.632127E+12, 1.694948E+12, 1.759956E+12, 1.827217E+12, 1.896802E+12, 
   1.968782E+12, 2.043232E+12, 2.120227E+12, 2.199844E+12, 2.282165E+12, 2.367270E+12, 
   2.455245E+12, 2.546174E+12, 2.640149E+12, 2.737259E+12, 2.837598E+12, 2.941261E+12, 
   3.048349E+12, 3.158961E+12, 3.273202E+12, 3.391178E+12, 3.512998E+12, 3.638774E+12, 
   3.768621E+12, 3.902657E+12, 4.041003E+12, 4.183783E+12, 4.331123E+12, 4.483155E+12, 
   4.640011E+12, 4.801829E+12, 4.968749E+12, 5.140914E+12, 5.318473E+12, 5.501577E+12, 
   5.690379E+12, 5.885040E+12, 6.085721E+12, 6.292589E+12, 6.505815E+12, 6.725572E+12, 
   6.952042E+12, 7.185406E+12, 7.425852E+12, 7.673574E+12, 7.928766E+12, 8.191632E+12, 
   8.462377E+12, 8.741213E+12, 9.028356E+12, 9.324028E+12, 9.628454E+12, 9.941867E+12, 
   1.026450E+13, 1.059661E+13, 1.093843E+13, 1.129022E+13, 1.165224E+13, 1.202475E+13, 
   1.240803E+13, 1.280236E+13, 1.320802E+13, 1.362531E+13, 1.405451E+13, 1.449594E+13, 
   1.494990E+13, 1.541672E+13, 1.589672E+13, 1.639024E+13, 1.689760E+13, 1.741918E+13, 
   1.795530E+13, 1.850635E+13, 1.907269E+13, 1.965470E+13, 2.025277E+13, 2.086730E+13, 
   2.149869E+13, 2.214736E+13, 2.281372E+13, 2.349822E+13, 2.420128E+13, 2.492337E+13, 
   2.566494E+13, 2.642647E+13, 2.720842E+13, 2.801130E+13, 2.883561E+13, 2.968185E+13, 
   3.055056E+13, 3.144225E+13, 3.235748E+13, 3.329680E+13, 3.426078E+13, 3.525000E+13, 
   3.626505E+13, 3.730654E+13, 3.837507E+13, 3.947128E+13, 4.059581E+13, 4.174932E+13, 
   4.293246E+13, 4.414593E+13, 4.539042E+13, 4.666664E+13, 4.797531E+13, 4.931717E+13, 
   5.069298E+13, 5.210349E+13, 5.354950E+13, 5.503181E+13, 5.655123E+13, 5.810859E+13, 
   5.970474E+13, 6.134054E+13, 6.301688E+13, 6.473465E+13, 6.649478E+13, 6.829818E+13, 
   7.014583E+13, 7.203868E+13, 7.397774E+13, 7.596400E+13, 7.799850E+13, 8.008228E+13, 
   8.221641E+13, 8.440198E+13, 8.664011E+13, 8.893192E+13, 9.127856E+13, 9.368121E+13, 
   9.614107E+13, 9.865935E+13, 1.012373E+14, 1.038762E+14, 1.065773E+14, 1.093419E+14, 
   1.121714E+14, 1.150671E+14, 1.180305E+14, 1.210628E+14, 1.241657E+14, 1.273405E+14, 
   1.305887E+14, 1.339119E+14, 1.373115E+14, 1.407893E+14, 1.443467E+14, 1.479854E+14, 
   1.517071E+14, 1.555135E+14, 1.594063E+14, 1.633872E+14, 1.674580E+14, 1.716206E+14, 
   1.758769E+14, 1.802286E+14, 1.846777E+14, 1.892261E+14, 1.938759E+14, 1.986291E+14, 
   2.034877E+14, 2.084538E+14, 2.135296E+14, 2.187172E+14, 2.240187E+14, 2.294366E+14, 
   2.349730E+14, 2.406302E+14, 2.464107E+14, 2.523168E+14, 2.583509E+14, 2.645156E+14, 
   2.708135E+14, 2.772469E+14, 2.838187E+14, 2.905314E+14, 2.973877E+14, 3.043904E+14, 
   3.115423E+14, 3.188462E+14, 3.263050E+14, 3.339218E+14, 3.416994E+14, 3.496408E+14, 
   3.577493E+14, 3.660279E+14, 3.744799E+14, 3.831084E+14, 3.919168E+14, 4.009085E+14, 
   4.100868E+14, 4.194552E+14, 4.290173E+14, 4.387766E+14, 4.487368E+14, 4.589016E+14, 
   4.692747E+14, 4.798600E+14, 4.906613E+14, 5.016827E+14, 5.129280E+14, 5.244014E+14, 
   5.361070E+14, 5.480491E+14, 5.602319E+14, 5.726597E+14, 5.853370E+14, 5.982683E+14, 
   6.114580E+14, 6.249109E+14, 6.386316E+14, 6.526249E+14, 6.668957E+14, 6.814489E+14, 
   6.962894E+14, 7.114224E+14, 7.268531E+14, 7.425866E+14, 7.586283E+14, 7.749836E+14, 
   7.916580E+14, 8.086570E+14, 8.259864E+14, 8.436518E+14, 8.616591E+14, 8.800143E+14, 
   8.987233E+14, 9.177923E+14, 9.372274E+14, 9.570350E+14, 9.772214E+14, 9.977931E+14, 
   1.018757E+15, 1.040119E+15, 1.061887E+15, 1.084067E+15, 1.106666E+15, 1.129691E+15, 
   1.153151E+15, 1.177051E+15, 1.201399E+15, 1.226203E+15, 1.251471E+15, 1.277210E+15, 
   1.303428E+15, 1.330133E+15, 1.357333E+15, 1.385037E+15, 1.413252E+15, 1.441988E+15, 
   1.471252E+15, 1.501054E+15, 1.531403E+15, 1.562307E+15, 1.593775E+15, 1.625817E+15, 
   1.658442E+15, 1.691659E+15, 1.725479E+15, 1.759911E+15, 1.794964E+15, 1.830650E+15, 
   1.866978E+15, 1.903958E+15, 1.941601E+15, 1.979918E+15, 2.018919E+15, 2.058616E+15, 
   2.099019E+15, 2.140140E+15, 2.181990E+15, 
])
# ============================ M = 36, I = 1 ============================
M = 36
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.869088E+00, 1.154951E+01, 2.200772E+01, 3.248752E+01, 4.297280E+01, 5.346050E+01, 
   6.394960E+01, 7.443968E+01, 8.493054E+01, 9.542204E+01, 1.059141E+02, 1.164068E+02, 
   1.268999E+02, 1.373935E+02, 1.478876E+02, 1.583822E+02, 1.688772E+02, 1.793727E+02, 
   1.898686E+02, 2.003649E+02, 2.108617E+02, 2.213590E+02, 2.318567E+02, 2.423548E+02, 
   2.528534E+02, 2.633525E+02, 2.738522E+02, 2.843524E+02, 2.948533E+02, 3.053550E+02, 
   3.158575E+02, 3.263610E+02, 3.368657E+02, 3.473718E+02, 3.578795E+02, 3.683892E+02, 
   3.789011E+02, 3.894156E+02, 3.999332E+02, 4.104542E+02, 4.209792E+02, 4.315088E+02, 
   4.420433E+02, 4.525836E+02, 4.631302E+02, 4.736839E+02, 4.842453E+02, 4.948152E+02, 
   5.053943E+02, 5.159835E+02, 5.265836E+02, 5.371955E+02, 5.478199E+02, 5.584578E+02, 
   5.691101E+02, 5.797777E+02, 5.904614E+02, 6.011622E+02, 6.118811E+02, 6.226188E+02, 
   6.333765E+02, 6.441549E+02, 6.549550E+02, 6.657776E+02, 6.766238E+02, 6.874944E+02, 
   6.983902E+02, 7.093122E+02, 7.202613E+02, 7.312382E+02, 7.422438E+02, 7.532790E+02, 
   7.643445E+02, 7.754412E+02, 7.865699E+02, 7.977312E+02, 8.089261E+02, 8.201552E+02, 
   8.314192E+02, 8.427188E+02, 8.540549E+02, 8.654280E+02, 8.768387E+02, 8.882878E+02, 
   8.997759E+02, 9.113036E+02, 9.228715E+02, 9.344802E+02, 9.461303E+02, 9.578223E+02, 
   9.695567E+02, 9.813342E+02, 9.931551E+02, 1.005020E+03, 1.016930E+03, 1.028884E+03, 
   1.040884E+03, 1.052930E+03, 1.065022E+03, 1.077161E+03, 1.089347E+03, 1.101581E+03, 
   1.113863E+03, 1.126193E+03, 1.138572E+03, 1.151000E+03, 1.163478E+03, 1.176005E+03, 
   1.188582E+03, 1.201210E+03, 1.213889E+03, 1.226619E+03, 1.239400E+03, 1.252233E+03, 
   1.265118E+03, 1.278055E+03, 1.291045E+03, 1.304087E+03, 1.317182E+03, 1.330331E+03, 
   1.343533E+03, 1.356789E+03, 1.370099E+03, 1.383463E+03, 1.396881E+03, 1.410354E+03, 
   1.423882E+03, 1.437465E+03, 1.451103E+03, 1.464796E+03, 1.478545E+03, 1.492350E+03, 
   1.506211E+03, 1.520127E+03, 1.534100E+03, 1.548130E+03, 1.562215E+03, 1.576358E+03, 
   1.590557E+03, 1.604814E+03, 1.619127E+03, 1.633498E+03, 1.647926E+03, 1.662412E+03, 
   1.676955E+03, 1.691556E+03, 1.706215E+03, 1.720932E+03, 1.735707E+03, 1.750541E+03, 
   1.765432E+03, 1.780382E+03, 1.795391E+03, 1.810458E+03, 1.825584E+03, 1.840769E+03, 
   1.856013E+03, 1.871316E+03, 1.886678E+03, 1.902099E+03, 1.917579E+03, 1.933119E+03, 
   1.948718E+03, 1.964377E+03, 1.980095E+03, 1.995873E+03, 2.011711E+03, 2.027608E+03, 
   2.043565E+03, 2.059583E+03, 2.075660E+03, 2.091797E+03, 2.107995E+03, 2.124253E+03, 
   2.140570E+03, 2.156949E+03, 2.173387E+03, 2.189887E+03, 2.206446E+03, 2.223066E+03, 
   2.239747E+03, 2.256488E+03, 2.273290E+03, 2.290153E+03, 2.307077E+03, 2.324061E+03, 
   2.341106E+03, 2.358213E+03, 2.375380E+03, 2.392608E+03, 2.409897E+03, 2.427248E+03, 
   2.444659E+03, 2.462132E+03, 2.479666E+03, 2.497261E+03, 2.514918E+03, 2.532636E+03, 
   2.550415E+03, 2.568255E+03, 2.586157E+03, 2.604121E+03, 2.622146E+03, 2.640233E+03, 
   2.658381E+03, 2.676591E+03, 2.694862E+03, 2.713195E+03, 2.731590E+03, 2.750047E+03, 
   2.768565E+03, 2.787145E+03, 2.805787E+03, 2.824491E+03, 2.843257E+03, 2.862084E+03, 
   2.880974E+03, 2.899925E+03, 2.918939E+03, 2.938014E+03, 2.957151E+03, 2.976351E+03, 
   2.995612E+03, 3.014936E+03, 3.034322E+03, 3.053770E+03, 3.073280E+03, 3.092852E+03, 
   3.112486E+03, 3.132183E+03, 3.151942E+03, 3.171763E+03, 3.191646E+03, 3.211592E+03, 
   3.231600E+03, 3.251670E+03, 3.271803E+03, 3.291998E+03, 3.312256E+03, 3.332576E+03, 
   3.352958E+03, 3.373403E+03, 3.393910E+03, 3.414480E+03, 3.435112E+03, 3.455807E+03, 
   3.476564E+03, 3.497384E+03, 3.518266E+03, 3.539211E+03, 3.560219E+03, 3.581289E+03, 
   3.602422E+03, 3.623617E+03, 3.644875E+03, 3.666196E+03, 3.687579E+03, 3.709026E+03, 
   3.730534E+03, 3.752106E+03, 3.773740E+03, 3.795437E+03, 3.817197E+03, 3.839019E+03, 
   3.860905E+03, 3.882853E+03, 3.904864E+03, 3.926938E+03, 3.949074E+03, 3.971273E+03, 
   3.993536E+03, 4.015861E+03, 4.038249E+03, 4.060700E+03, 4.083214E+03, 4.105790E+03, 
   4.128430E+03, 4.151133E+03, 4.173898E+03, 4.196727E+03, 4.219618E+03, 4.242573E+03, 
   4.265590E+03, 4.288670E+03, 4.311814E+03, 4.335020E+03, 4.358290E+03, 4.381622E+03, 
   4.405018E+03, 4.428476E+03, 4.451998E+03, 4.475583E+03, 4.499231E+03, 4.522942E+03, 
   4.546716E+03, 4.570553E+03, 4.594453E+03, 4.618417E+03, 4.642443E+03, 4.666533E+03, 
   4.690685E+03, 4.714901E+03, 4.739181E+03, 4.763523E+03, 4.787929E+03, 4.812397E+03, 
   4.836929E+03, 4.861524E+03, 4.886183E+03, 4.910904E+03, 4.935689E+03, 4.960537E+03, 
   4.985449E+03, 5.010423E+03, 5.035461E+03, 5.060562E+03, 5.085727E+03, 5.110954E+03, 
   5.136245E+03, 5.161600E+03, 5.187017E+03, 5.212498E+03, 5.238043E+03, 5.263650E+03, 
   5.289321E+03, 5.315056E+03, 5.340853E+03, 5.366715E+03, 5.392639E+03, 5.418627E+03, 
   5.444678E+03, 5.470792E+03, 5.496970E+03, 5.523212E+03, 5.549517E+03, 5.575885E+03, 
   5.602316E+03, 5.628811E+03, 5.655370E+03, 5.681992E+03, 5.708677E+03, 5.735426E+03, 
   5.762239E+03, 5.789114E+03, 5.816054E+03, 5.843056E+03, 5.870123E+03, 5.897252E+03, 
   5.924445E+03, 5.951702E+03, 5.979023E+03, 6.006406E+03, 6.033854E+03, 6.061364E+03, 
   6.088939E+03, 6.116577E+03, 6.144278E+03, 6.172043E+03, 6.199872E+03, 6.227764E+03, 
   6.255720E+03, 6.283739E+03, 6.311822E+03, 6.339968E+03, 6.368178E+03, 6.396452E+03, 
   6.424789E+03, 6.453190E+03, 6.481655E+03, 6.510183E+03, 6.538775E+03, 6.567430E+03, 
   6.596149E+03, 6.624932E+03, 6.653778E+03, 6.682688E+03, 6.711662E+03, 6.740699E+03, 
   6.769800E+03, 6.798965E+03, 6.828193E+03, 6.857485E+03, 6.886841E+03, 6.916261E+03, 
   6.945744E+03, 6.975291E+03, 7.004901E+03, 7.034575E+03, 7.064313E+03, 7.094115E+03, 
   7.123981E+03, 7.153910E+03, 7.183903E+03, 7.213960E+03, 7.244080E+03, 7.274264E+03, 
   7.304512E+03, 7.334824E+03, 7.365200E+03, 7.395639E+03, 7.426142E+03, 7.456709E+03, 
   7.487340E+03, 7.518034E+03, 7.548793E+03, 7.579615E+03, 7.610501E+03, 7.641451E+03, 
   7.672464E+03, 7.703542E+03, 7.734683E+03, 7.765888E+03, 7.797157E+03, 7.828490E+03, 
   7.859887E+03, 7.891347E+03, 7.922872E+03, 7.954460E+03, 7.986112E+03, 8.017828E+03, 
   8.049608E+03, 8.081452E+03, 8.113360E+03, 8.145331E+03, 8.177367E+03, 8.209466E+03, 
   8.241630E+03, 8.273857E+03, 8.306148E+03, 8.338503E+03, 8.370922E+03, 8.403405E+03, 
   8.435952E+03, 8.468563E+03, 8.501238E+03, 8.533976E+03, 8.566779E+03, 8.599646E+03, 
   8.632576E+03, 8.665571E+03, 8.698630E+03, 8.731752E+03, 8.764939E+03, 8.798190E+03, 
   8.831504E+03, 8.864883E+03, 8.898325E+03, 8.931832E+03, 8.965402E+03, 8.999037E+03, 
   9.032736E+03, 9.066498E+03, 9.100325E+03, 9.134216E+03, 9.168170E+03, 9.202189E+03, 
   9.236272E+03, 9.270419E+03, 9.304630E+03, 9.338905E+03, 9.373244E+03, 9.407647E+03, 
   9.442115E+03, 9.476646E+03, 9.511241E+03, 9.545901E+03, 9.580624E+03, 9.615412E+03, 
   9.650264E+03, 9.685180E+03, 9.720160E+03, 9.755204E+03, 9.790312E+03, 9.825485E+03, 
   9.860721E+03, 9.896022E+03, 9.931387E+03, 9.966816E+03, 1.000231E+04, 1.003787E+04, 
   1.007349E+04, 1.010917E+04, 1.014492E+04, 1.018074E+04, 1.021661E+04, 1.025256E+04, 
   1.028856E+04, 1.032463E+04, 1.036077E+04, 1.039697E+04, 1.043323E+04, 1.046956E+04, 
   1.050595E+04, 1.054241E+04, 1.057892E+04, 1.061551E+04, 1.065216E+04, 1.068887E+04, 
   1.072565E+04, 1.076249E+04, 1.079939E+04, 
])
# ============================ M = 37, I = 1 ============================
M = 37
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.784286E+00, 1.684562E+02, 4.726096E+02, 8.660051E+02, 1.331721E+03, 1.859963E+03, 
   2.444122E+03, 3.079355E+03, 3.761935E+03, 4.488927E+03, 5.258023E+03, 6.067445E+03, 
   6.915884E+03, 7.802444E+03, 8.726593E+03, 9.688113E+03, 1.068705E+04, 1.172368E+04, 
   1.279844E+04, 1.391196E+04, 1.506494E+04, 1.625824E+04, 1.749277E+04, 1.876951E+04, 
   2.008951E+04, 2.145388E+04, 2.286376E+04, 2.432031E+04, 2.582475E+04, 2.737832E+04, 
   2.898227E+04, 3.063788E+04, 3.234645E+04, 3.410930E+04, 3.592775E+04, 3.780314E+04, 
   3.973684E+04, 4.173019E+04, 4.378459E+04, 4.590140E+04, 4.808204E+04, 5.032789E+04, 
   5.264038E+04, 5.502091E+04, 5.747093E+04, 5.999186E+04, 6.258515E+04, 6.525225E+04, 
   6.799463E+04, 7.081375E+04, 7.371110E+04, 7.668816E+04, 7.974644E+04, 8.288743E+04, 
   8.611266E+04, 8.942365E+04, 9.282194E+04, 9.630908E+04, 9.988663E+04, 1.035562E+05, 
   1.073192E+05, 1.111775E+05, 1.151325E+05, 1.191859E+05, 1.233393E+05, 1.275944E+05, 
   1.319528E+05, 1.364162E+05, 1.409862E+05, 1.456647E+05, 1.504532E+05, 1.553536E+05, 
   1.603675E+05, 1.654967E+05, 1.707431E+05, 1.761083E+05, 1.815943E+05, 1.872027E+05, 
   1.929355E+05, 1.987946E+05, 2.047817E+05, 2.108988E+05, 2.171478E+05, 2.235306E+05, 
   2.300491E+05, 2.367054E+05, 2.435013E+05, 2.504389E+05, 2.575201E+05, 2.647471E+05, 
   2.721218E+05, 2.796464E+05, 2.873228E+05, 2.951533E+05, 3.031399E+05, 3.112848E+05, 
   3.195900E+05, 3.280579E+05, 3.366906E+05, 3.454903E+05, 3.544593E+05, 3.635998E+05, 
   3.729140E+05, 3.824044E+05, 3.920731E+05, 4.019225E+05, 4.119550E+05, 4.221730E+05, 
   4.325788E+05, 4.431748E+05, 4.539635E+05, 4.649474E+05, 4.761289E+05, 4.875104E+05, 
   4.990946E+05, 5.108839E+05, 5.228810E+05, 5.350883E+05, 5.475085E+05, 5.601441E+05, 
   5.729979E+05, 5.860725E+05, 5.993706E+05, 6.128948E+05, 6.266479E+05, 6.406326E+05, 
   6.548518E+05, 6.693081E+05, 6.840043E+05, 6.989434E+05, 7.141282E+05, 7.295614E+05, 
   7.452461E+05, 7.611852E+05, 7.773814E+05, 7.938380E+05, 8.105577E+05, 8.275436E+05, 
   8.447987E+05, 8.623261E+05, 8.801289E+05, 8.982100E+05, 9.165727E+05, 9.352201E+05, 
   9.541552E+05, 9.733814E+05, 9.929017E+05, 1.012720E+06, 1.032838E+06, 1.053260E+06, 
   1.073990E+06, 1.095030E+06, 1.116384E+06, 1.138055E+06, 1.160047E+06, 1.182362E+06, 
   1.205006E+06, 1.227980E+06, 1.251288E+06, 1.274934E+06, 1.298922E+06, 1.323254E+06, 
   1.347935E+06, 1.372968E+06, 1.398356E+06, 1.424104E+06, 1.450215E+06, 1.476692E+06, 
   1.503540E+06, 1.530762E+06, 1.558361E+06, 1.586342E+06, 1.614709E+06, 1.643464E+06, 
   1.672613E+06, 1.702159E+06, 1.732105E+06, 1.762456E+06, 1.793216E+06, 1.824389E+06, 
   1.855978E+06, 1.887988E+06, 1.920422E+06, 1.953285E+06, 1.986581E+06, 2.020314E+06, 
   2.054488E+06, 2.089108E+06, 2.124177E+06, 2.159699E+06, 2.195679E+06, 2.232122E+06, 
   2.269031E+06, 2.306410E+06, 2.344265E+06, 2.382600E+06, 2.421418E+06, 2.460724E+06, 
   2.500523E+06, 2.540819E+06, 2.581617E+06, 2.622921E+06, 2.664735E+06, 2.707065E+06, 
   2.749915E+06, 2.793289E+06, 2.837192E+06, 2.881630E+06, 2.926605E+06, 2.972124E+06, 
   3.018190E+06, 3.064810E+06, 3.111987E+06, 3.159726E+06, 3.208032E+06, 3.256911E+06, 
   3.306366E+06, 3.356404E+06, 3.407028E+06, 3.458244E+06, 3.510057E+06, 3.562472E+06, 
   3.615493E+06, 3.669127E+06, 3.723378E+06, 3.778251E+06, 3.833752E+06, 3.889885E+06, 
   3.946656E+06, 4.004071E+06, 4.062133E+06, 4.120850E+06, 4.180225E+06, 4.240265E+06, 
   4.300975E+06, 4.362360E+06, 4.424425E+06, 4.487177E+06, 4.550620E+06, 4.614761E+06, 
   4.679604E+06, 4.745156E+06, 4.811421E+06, 4.878406E+06, 4.946116E+06, 5.014557E+06, 
   5.083735E+06, 5.153655E+06, 5.224323E+06, 5.295745E+06, 5.367927E+06, 5.440874E+06, 
   5.514593E+06, 5.589090E+06, 5.664370E+06, 5.740439E+06, 5.817304E+06, 5.894970E+06, 
   5.973444E+06, 6.052731E+06, 6.132838E+06, 6.213771E+06, 6.295535E+06, 6.378138E+06, 
   6.461586E+06, 6.545884E+06, 6.631040E+06, 6.717058E+06, 6.803947E+06, 6.891711E+06, 
   6.980358E+06, 7.069894E+06, 7.160326E+06, 7.251659E+06, 7.343901E+06, 7.437058E+06, 
   7.531137E+06, 7.626144E+06, 7.722085E+06, 7.818969E+06, 7.916800E+06, 8.015587E+06, 
   8.115336E+06, 8.216053E+06, 8.317745E+06, 8.420420E+06, 8.524084E+06, 8.628745E+06, 
   8.734408E+06, 8.841082E+06, 8.948772E+06, 9.057487E+06, 9.167233E+06, 9.278017E+06, 
   9.389847E+06, 9.502730E+06, 9.616672E+06, 9.731681E+06, 9.847765E+06, 9.964930E+06, 
   1.008318E+07, 1.020254E+07, 1.032299E+07, 1.044456E+07, 1.056724E+07, 1.069105E+07, 
   1.081599E+07, 1.094208E+07, 1.106931E+07, 1.119770E+07, 1.132726E+07, 1.145798E+07, 
   1.158989E+07, 1.172298E+07, 1.185727E+07, 1.199276E+07, 1.212946E+07, 1.226738E+07, 
   1.240652E+07, 1.254690E+07, 1.268852E+07, 1.283140E+07, 1.297553E+07, 1.312092E+07, 
   1.326759E+07, 1.341555E+07, 1.356479E+07, 1.371534E+07, 1.386718E+07, 1.402035E+07, 
   1.417484E+07, 1.433066E+07, 1.448782E+07, 1.464633E+07, 1.480620E+07, 1.496743E+07, 
   1.513004E+07, 1.529403E+07, 1.545941E+07, 1.562619E+07, 1.579438E+07, 1.596399E+07, 
   1.613502E+07, 1.630749E+07, 1.648140E+07, 1.665676E+07, 1.683358E+07, 1.701188E+07, 
   1.719165E+07, 1.737291E+07, 1.755566E+07, 1.773992E+07, 1.792570E+07, 1.811300E+07, 
   1.830183E+07, 1.849221E+07, 1.868413E+07, 1.887762E+07, 1.907268E+07, 1.926931E+07, 
   1.946753E+07, 1.966735E+07, 1.986878E+07, 2.007182E+07, 2.027649E+07, 2.048280E+07, 
   2.069075E+07, 2.090035E+07, 2.111162E+07, 2.132456E+07, 2.153919E+07, 2.175550E+07, 
   2.197352E+07, 2.219326E+07, 2.241471E+07, 2.263790E+07, 2.286283E+07, 2.308951E+07, 
   2.331795E+07, 2.354817E+07, 2.378016E+07, 2.401395E+07, 2.424954E+07, 2.448694E+07, 
   2.472617E+07, 2.496723E+07, 2.521013E+07, 2.545488E+07, 2.570150E+07, 2.594999E+07, 
   2.620036E+07, 2.645263E+07, 2.670681E+07, 2.696290E+07, 2.722091E+07, 2.748086E+07, 
   2.774276E+07, 2.800662E+07, 2.827245E+07, 2.854025E+07, 2.881005E+07, 2.908184E+07, 
   2.935565E+07, 2.963148E+07, 2.990934E+07, 3.018925E+07, 3.047122E+07, 3.075524E+07, 
   3.104135E+07, 3.132955E+07, 3.161984E+07, 3.191225E+07, 3.220678E+07, 3.250344E+07, 
   3.280224E+07, 3.310320E+07, 3.340633E+07, 3.371164E+07, 3.401914E+07, 3.432883E+07, 
   3.464074E+07, 3.495488E+07, 3.527125E+07, 3.558987E+07, 3.591075E+07, 3.623390E+07, 
   3.655934E+07, 3.688707E+07, 3.721710E+07, 3.754946E+07, 3.788414E+07, 3.822116E+07, 
   3.856054E+07, 3.890229E+07, 3.924642E+07, 3.959293E+07, 3.994185E+07, 4.029318E+07, 
   4.064694E+07, 4.100314E+07, 4.136179E+07, 4.172290E+07, 4.208649E+07, 4.245257E+07, 
   4.282114E+07, 4.319224E+07, 4.356586E+07, 4.394201E+07, 4.432072E+07, 4.470199E+07, 
   4.508584E+07, 4.547228E+07, 4.586132E+07, 4.625297E+07, 4.664726E+07, 4.704418E+07, 
   4.744376E+07, 4.784600E+07, 4.825092E+07, 4.865854E+07, 4.906886E+07, 4.948190E+07, 
   4.989767E+07, 5.031619E+07, 5.073746E+07, 5.116151E+07, 5.158834E+07, 5.201797E+07, 
   5.245041E+07, 5.288568E+07, 5.332379E+07, 5.376475E+07, 5.420857E+07, 5.465528E+07, 
   5.510488E+07, 5.555738E+07, 5.601281E+07, 5.647117E+07, 5.693248E+07, 5.739675E+07, 
   5.786400E+07, 5.833424E+07, 5.880748E+07, 5.928374E+07, 5.976303E+07, 6.024537E+07, 
   6.073077E+07, 6.121924E+07, 6.171081E+07, 6.220548E+07, 6.270326E+07, 6.320418E+07, 
   6.370824E+07, 6.421547E+07, 6.472587E+07, 6.523946E+07, 6.575625E+07, 6.627627E+07, 
   6.679952E+07, 6.732602E+07, 6.785578E+07, 
])
# ============================ M = 37, I = 2 ============================
M = 37
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.739142E+00, 1.677924E+02, 4.707646E+02, 8.626346E+02, 1.326546E+03, 1.852742E+03, 
   2.434639E+03, 3.067412E+03, 3.747349E+03, 4.471530E+03, 5.237657E+03, 6.043962E+03, 
   6.889145E+03, 7.772320E+03, 8.692963E+03, 9.650864E+03, 1.064607E+04, 1.167887E+04, 
   1.274970E+04, 1.385918E+04, 1.500805E+04, 1.619712E+04, 1.742732E+04, 1.869965E+04, 
   2.001513E+04, 2.137488E+04, 2.278003E+04, 2.423176E+04, 2.573126E+04, 2.727978E+04, 
   2.887857E+04, 3.052890E+04, 3.223207E+04, 3.398939E+04, 3.580218E+04, 3.767179E+04, 
   3.959956E+04, 4.158685E+04, 4.363504E+04, 4.574551E+04, 4.791964E+04, 5.015885E+04, 
   5.246452E+04, 5.483809E+04, 5.728097E+04, 5.979460E+04, 6.238042E+04, 6.503988E+04, 
   6.777443E+04, 7.058555E+04, 7.347471E+04, 7.644339E+04, 7.949309E+04, 8.262532E+04, 
   8.584158E+04, 8.914340E+04, 9.253232E+04, 9.600988E+04, 9.957763E+04, 1.032371E+05, 
   1.069900E+05, 1.108378E+05, 1.147821E+05, 1.188246E+05, 1.229669E+05, 1.272106E+05, 
   1.315573E+05, 1.360088E+05, 1.405668E+05, 1.452328E+05, 1.500087E+05, 1.548962E+05, 
   1.598969E+05, 1.650127E+05, 1.702454E+05, 1.755966E+05, 1.810683E+05, 1.866622E+05, 
   1.923802E+05, 1.982241E+05, 2.041958E+05, 2.102972E+05, 2.165302E+05, 2.228966E+05, 
   2.293985E+05, 2.360378E+05, 2.428164E+05, 2.497363E+05, 2.567996E+05, 2.640083E+05, 
   2.713644E+05, 2.788700E+05, 2.865272E+05, 2.943380E+05, 3.023045E+05, 3.104290E+05, 
   3.187135E+05, 3.271603E+05, 3.357715E+05, 3.445493E+05, 3.534960E+05, 3.626138E+05, 
   3.719050E+05, 3.813719E+05, 3.910167E+05, 4.008419E+05, 4.108497E+05, 4.210425E+05, 
   4.314228E+05, 4.419929E+05, 4.527552E+05, 4.637122E+05, 4.748663E+05, 4.862202E+05, 
   4.977761E+05, 5.095368E+05, 5.215047E+05, 5.336824E+05, 5.460725E+05, 5.586776E+05, 
   5.715003E+05, 5.845433E+05, 5.978093E+05, 6.113010E+05, 6.250210E+05, 6.389721E+05, 
   6.531571E+05, 6.675787E+05, 6.822398E+05, 6.971431E+05, 7.122916E+05, 7.276880E+05, 
   7.433353E+05, 7.592363E+05, 7.753940E+05, 7.918114E+05, 8.084914E+05, 8.254370E+05, 
   8.426512E+05, 8.601371E+05, 8.778977E+05, 8.959361E+05, 9.142554E+05, 9.328587E+05, 
   9.517492E+05, 9.709301E+05, 9.904044E+05, 1.010176E+06, 1.030247E+06, 1.050621E+06, 
   1.071302E+06, 1.092293E+06, 1.113597E+06, 1.135217E+06, 1.157157E+06, 1.179421E+06, 
   1.202011E+06, 1.224932E+06, 1.248186E+06, 1.271777E+06, 1.295708E+06, 1.319984E+06, 
   1.344608E+06, 1.369582E+06, 1.394912E+06, 1.420600E+06, 1.446650E+06, 1.473066E+06, 
   1.499852E+06, 1.527011E+06, 1.554546E+06, 1.582463E+06, 1.610764E+06, 1.639453E+06, 
   1.668535E+06, 1.698012E+06, 1.727890E+06, 1.758171E+06, 1.788860E+06, 1.819961E+06, 
   1.851478E+06, 1.883414E+06, 1.915774E+06, 1.948562E+06, 1.981782E+06, 2.015438E+06, 
   2.049534E+06, 2.084074E+06, 2.119062E+06, 2.154504E+06, 2.190402E+06, 2.226761E+06, 
   2.263586E+06, 2.300881E+06, 2.338649E+06, 2.376896E+06, 2.415626E+06, 2.454843E+06, 
   2.494551E+06, 2.534756E+06, 2.575461E+06, 2.616671E+06, 2.658391E+06, 2.700625E+06, 
   2.743378E+06, 2.786654E+06, 2.830458E+06, 2.874794E+06, 2.919668E+06, 2.965084E+06, 
   3.011047E+06, 3.057561E+06, 3.104632E+06, 3.152263E+06, 3.200461E+06, 3.249229E+06, 
   3.298573E+06, 3.348498E+06, 3.399008E+06, 3.450109E+06, 3.501805E+06, 3.554102E+06, 
   3.607005E+06, 3.660518E+06, 3.714648E+06, 3.769398E+06, 3.824774E+06, 3.880781E+06, 
   3.937425E+06, 3.994711E+06, 4.052644E+06, 4.111229E+06, 4.170472E+06, 4.230377E+06, 
   4.290951E+06, 4.352199E+06, 4.414126E+06, 4.476738E+06, 4.540040E+06, 4.604037E+06, 
   4.668736E+06, 4.734141E+06, 4.800259E+06, 4.867095E+06, 4.934654E+06, 5.002943E+06, 
   5.071967E+06, 5.141731E+06, 5.212242E+06, 5.283506E+06, 5.355528E+06, 5.428313E+06, 
   5.501869E+06, 5.576200E+06, 5.651313E+06, 5.727214E+06, 5.803908E+06, 5.881402E+06, 
   5.959702E+06, 6.038814E+06, 6.118743E+06, 6.199497E+06, 6.281081E+06, 6.363502E+06, 
   6.446765E+06, 6.530877E+06, 6.615844E+06, 6.701673E+06, 6.788370E+06, 6.875940E+06, 
   6.964392E+06, 7.053731E+06, 7.143963E+06, 7.235095E+06, 7.327134E+06, 7.420086E+06, 
   7.513957E+06, 7.608755E+06, 7.704486E+06, 7.801156E+06, 7.898772E+06, 7.997342E+06, 
   8.096872E+06, 8.197367E+06, 8.298837E+06, 8.401286E+06, 8.504723E+06, 8.609154E+06, 
   8.714585E+06, 8.821025E+06, 8.928479E+06, 9.036956E+06, 9.146462E+06, 9.257003E+06, 
   9.368588E+06, 9.481224E+06, 9.594917E+06, 9.709674E+06, 9.825504E+06, 9.942414E+06, 
   1.006041E+07, 1.017950E+07, 1.029969E+07, 1.042099E+07, 1.054341E+07, 1.066695E+07, 
   1.079162E+07, 1.091743E+07, 1.104438E+07, 1.117250E+07, 1.130177E+07, 1.143221E+07, 
   1.156383E+07, 1.169663E+07, 1.183062E+07, 1.196582E+07, 1.210222E+07, 1.223984E+07, 
   1.237868E+07, 1.251876E+07, 1.266007E+07, 1.280263E+07, 1.294645E+07, 1.309153E+07, 
   1.323788E+07, 1.338551E+07, 1.353443E+07, 1.368465E+07, 1.383617E+07, 1.398900E+07, 
   1.414316E+07, 1.429864E+07, 1.445546E+07, 1.461363E+07, 1.477315E+07, 1.493403E+07, 
   1.509629E+07, 1.525992E+07, 1.542494E+07, 1.559136E+07, 1.575919E+07, 1.592843E+07, 
   1.609909E+07, 1.627118E+07, 1.644472E+07, 1.661970E+07, 1.679614E+07, 1.697405E+07, 
   1.715343E+07, 1.733430E+07, 1.751666E+07, 1.770053E+07, 1.788590E+07, 1.807280E+07, 
   1.826122E+07, 1.845119E+07, 1.864270E+07, 1.883577E+07, 1.903040E+07, 1.922661E+07, 
   1.942441E+07, 1.962380E+07, 1.982479E+07, 2.002740E+07, 2.023163E+07, 2.043749E+07, 
   2.064499E+07, 2.085414E+07, 2.106495E+07, 2.127744E+07, 2.149160E+07, 2.170745E+07, 
   2.192500E+07, 2.214426E+07, 2.236524E+07, 2.258795E+07, 2.281240E+07, 2.303859E+07, 
   2.326654E+07, 2.349626E+07, 2.372776E+07, 2.396105E+07, 2.419613E+07, 2.443302E+07, 
   2.467174E+07, 2.491228E+07, 2.515466E+07, 2.539888E+07, 2.564497E+07, 2.589293E+07, 
   2.614277E+07, 2.639450E+07, 2.664813E+07, 2.690367E+07, 2.716113E+07, 2.742052E+07, 
   2.768186E+07, 2.794516E+07, 2.821041E+07, 2.847764E+07, 2.874686E+07, 2.901808E+07, 
   2.929130E+07, 2.956654E+07, 2.984381E+07, 3.012312E+07, 3.040448E+07, 3.068790E+07, 
   3.097339E+07, 3.126097E+07, 3.155065E+07, 3.184243E+07, 3.213633E+07, 3.243236E+07, 
   3.273052E+07, 3.303084E+07, 3.333332E+07, 3.363798E+07, 3.394482E+07, 3.425386E+07, 
   3.456510E+07, 3.487857E+07, 3.519427E+07, 3.551221E+07, 3.583240E+07, 3.615486E+07, 
   3.647960E+07, 3.680664E+07, 3.713597E+07, 3.746761E+07, 3.780158E+07, 3.813789E+07, 
   3.847655E+07, 3.881757E+07, 3.916096E+07, 3.950674E+07, 3.985491E+07, 4.020549E+07, 
   4.055850E+07, 4.091394E+07, 4.127183E+07, 4.163217E+07, 4.199499E+07, 4.236029E+07, 
   4.272808E+07, 4.309839E+07, 4.347121E+07, 4.384657E+07, 4.422447E+07, 4.460493E+07, 
   4.498797E+07, 4.537358E+07, 4.576180E+07, 4.615262E+07, 4.654606E+07, 4.694214E+07, 
   4.734087E+07, 4.774226E+07, 4.814632E+07, 4.855308E+07, 4.896253E+07, 4.937469E+07, 
   4.978958E+07, 5.020721E+07, 5.062759E+07, 5.105074E+07, 5.147666E+07, 5.190538E+07, 
   5.233691E+07, 5.277126E+07, 5.320844E+07, 5.364846E+07, 5.409134E+07, 5.453710E+07, 
   5.498575E+07, 5.543730E+07, 5.589176E+07, 5.634915E+07, 5.680948E+07, 5.727277E+07, 
   5.773903E+07, 5.820827E+07, 5.868051E+07, 5.915577E+07, 5.963404E+07, 6.011536E+07, 
   6.059974E+07, 6.108718E+07, 6.157770E+07, 6.207132E+07, 6.256806E+07, 6.306792E+07, 
   6.357092E+07, 6.407707E+07, 6.458639E+07, 6.509890E+07, 6.561460E+07, 6.613352E+07, 
   6.665566E+07, 6.718105E+07, 6.770969E+07, 
])
# ============================ M = 38, I = 1 ============================
M = 38
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.963689E+00, 6.736587E+01, 1.867552E+02, 3.408688E+02, 5.231580E+02, 7.298226E+02, 
   9.582935E+02, 1.206685E+03, 1.473534E+03, 1.757668E+03, 2.058124E+03, 2.374105E+03, 
   2.704958E+03, 3.050165E+03, 3.409349E+03, 3.782269E+03, 4.168832E+03, 4.569097E+03, 
   4.983270E+03, 5.411709E+03, 5.854921E+03, 6.313548E+03, 6.788379E+03, 7.280320E+03, 
   7.790408E+03, 8.319793E+03, 8.869736E+03, 9.441602E+03, 1.003686E+04, 1.065708E+04, 
   1.130392E+04, 1.197913E+04, 1.268457E+04, 1.342219E+04, 1.419404E+04, 1.500224E+04, 
   1.584907E+04, 1.673685E+04, 1.766805E+04, 1.864525E+04, 1.967114E+04, 2.074854E+04, 
   2.188040E+04, 2.306979E+04, 2.431996E+04, 2.563427E+04, 2.701627E+04, 2.846965E+04, 
   2.999831E+04, 3.160629E+04, 3.329786E+04, 3.507746E+04, 3.694978E+04, 3.891969E+04, 
   4.099232E+04, 4.317304E+04, 4.546747E+04, 4.788149E+04, 5.042130E+04, 5.309333E+04, 
   5.590439E+04, 5.886155E+04, 6.197225E+04, 6.524427E+04, 6.868579E+04, 7.230534E+04, 
   7.611183E+04, 8.011466E+04, 8.432362E+04, 8.874898E+04, 9.340146E+04, 9.829237E+04, 
   1.034334E+05, 1.088369E+05, 1.145158E+05, 1.204835E+05, 1.267541E+05, 1.333425E+05, 
   1.402640E+05, 1.475348E+05, 1.551716E+05, 1.631923E+05, 1.716151E+05, 1.804594E+05, 
   1.897453E+05, 1.994939E+05, 2.097271E+05, 2.204679E+05, 2.317403E+05, 2.435694E+05, 
   2.559813E+05, 2.690033E+05, 2.826640E+05, 2.969931E+05, 3.120218E+05, 3.277823E+05, 
   3.443087E+05, 3.616362E+05, 3.798016E+05, 3.988435E+05, 4.188016E+05, 4.397181E+05, 
   4.616364E+05, 4.846019E+05, 5.086620E+05, 5.338660E+05, 5.602653E+05, 5.879138E+05, 
   6.168670E+05, 6.471835E+05, 6.789234E+05, 7.121501E+05, 7.469294E+05, 7.833296E+05, 
   8.214221E+05, 8.612809E+05, 9.029835E+05, 9.466101E+05, 9.922444E+05, 1.039973E+06, 
   1.089888E+06, 1.142081E+06, 1.196652E+06, 1.253703E+06, 1.313339E+06, 1.375670E+06, 
   1.440811E+06, 1.508882E+06, 1.580005E+06, 1.654310E+06, 1.731930E+06, 1.813004E+06, 
   1.897676E+06, 1.986097E+06, 2.078422E+06, 2.174813E+06, 2.275437E+06, 2.380469E+06, 
   2.490091E+06, 2.604490E+06, 2.723861E+06, 2.848406E+06, 2.978337E+06, 3.113870E+06, 
   3.255233E+06, 3.402660E+06, 3.556394E+06, 3.716688E+06, 3.883803E+06, 4.058014E+06, 
   4.239598E+06, 4.428850E+06, 4.626072E+06, 4.831577E+06, 5.045691E+06, 5.268752E+06, 
   5.501107E+06, 5.743119E+06, 5.995166E+06, 6.257629E+06, 6.530917E+06, 6.815444E+06, 
   7.111643E+06, 7.419959E+06, 7.740854E+06, 8.074809E+06, 8.422321E+06, 8.783900E+06, 
   9.160078E+06, 9.551407E+06, 9.958454E+06, 1.038181E+07, 1.082208E+07, 1.127989E+07, 
   1.175591E+07, 1.225079E+07, 1.276524E+07, 1.329999E+07, 1.385576E+07, 1.443335E+07, 
   1.503353E+07, 1.565715E+07, 1.630504E+07, 1.697808E+07, 1.767719E+07, 1.840331E+07, 
   1.915740E+07, 1.994048E+07, 2.075357E+07, 2.159775E+07, 2.247411E+07, 2.338381E+07, 
   2.432803E+07, 2.530798E+07, 2.632491E+07, 2.738012E+07, 2.847496E+07, 2.961082E+07, 
   3.078911E+07, 3.201130E+07, 3.327893E+07, 3.459356E+07, 3.595680E+07, 3.737034E+07, 
   3.883588E+07, 4.035522E+07, 4.193018E+07, 4.356266E+07, 4.525460E+07, 4.700801E+07, 
   4.882499E+07, 5.070765E+07, 5.265821E+07, 5.467893E+07, 5.677216E+07, 5.894032E+07, 
   6.118588E+07, 6.351143E+07, 6.591961E+07, 6.841312E+07, 7.099478E+07, 7.366750E+07, 
   7.643423E+07, 7.929806E+07, 8.226213E+07, 8.532970E+07, 8.850412E+07, 9.178888E+07, 
   9.518748E+07, 9.870360E+07, 1.023410E+08, 1.061036E+08, 1.099953E+08, 1.140203E+08, 
   1.181829E+08, 1.224872E+08, 1.269380E+08, 1.315396E+08, 1.362969E+08, 1.412148E+08, 
   1.462984E+08, 1.515526E+08, 1.569830E+08, 1.625948E+08, 1.683939E+08, 1.743859E+08, 
   1.805768E+08, 1.869727E+08, 1.935798E+08, 2.004048E+08, 2.074542E+08, 2.147347E+08, 
   2.222536E+08, 2.300179E+08, 2.380352E+08, 2.463129E+08, 2.548590E+08, 2.636816E+08, 
   2.727889E+08, 2.821894E+08, 2.918918E+08, 3.019051E+08, 3.122386E+08, 3.229017E+08, 
   3.339041E+08, 3.452558E+08, 3.569673E+08, 3.690488E+08, 3.815113E+08, 3.943659E+08, 
   4.076240E+08, 4.212973E+08, 4.353979E+08, 4.499380E+08, 4.649306E+08, 4.803883E+08, 
   4.963248E+08, 5.127536E+08, 5.296887E+08, 5.471449E+08, 5.651367E+08, 5.836795E+08, 
   6.027886E+08, 6.224803E+08, 6.427710E+08, 6.636774E+08, 6.852169E+08, 7.074071E+08, 
   7.302664E+08, 7.538133E+08, 7.780668E+08, 8.030466E+08, 8.287730E+08, 8.552664E+08, 
   8.825481E+08, 9.106398E+08, 9.395634E+08, 9.693421E+08, 9.999990E+08, 1.031558E+09, 
   1.064044E+09, 1.097482E+09, 1.131897E+09, 1.167317E+09, 1.203768E+09, 1.241277E+09, 
   1.279874E+09, 1.319588E+09, 1.360448E+09, 1.402484E+09, 1.445730E+09, 1.490215E+09, 
   1.535974E+09, 1.583039E+09, 1.631446E+09, 1.681230E+09, 1.732426E+09, 1.785072E+09, 
   1.839206E+09, 1.894867E+09, 1.952094E+09, 2.010929E+09, 2.071412E+09, 2.133587E+09, 
   2.197497E+09, 2.263187E+09, 2.330704E+09, 2.400092E+09, 2.471402E+09, 2.544681E+09, 
   2.619980E+09, 2.697350E+09, 2.776844E+09, 2.858518E+09, 2.942423E+09, 3.028618E+09, 
   3.117161E+09, 3.208109E+09, 3.301524E+09, 3.397467E+09, 3.496001E+09, 3.597192E+09, 
   3.701105E+09, 3.807808E+09, 3.917370E+09, 4.029861E+09, 4.145355E+09, 4.263925E+09, 
   4.385646E+09, 4.510596E+09, 4.638855E+09, 4.770503E+09, 4.905622E+09, 5.044298E+09, 
   5.186615E+09, 5.332664E+09, 5.482535E+09, 5.636319E+09, 5.794110E+09, 5.956006E+09, 
   6.122104E+09, 6.292508E+09, 6.467316E+09, 6.646638E+09, 6.830576E+09, 7.019245E+09, 
   7.212754E+09, 7.411221E+09, 7.614760E+09, 7.823491E+09, 8.037537E+09, 8.257023E+09, 
   8.482078E+09, 8.712829E+09, 8.949411E+09, 9.191960E+09, 9.440617E+09, 9.695519E+09, 
   9.956814E+09, 1.022465E+10, 1.049918E+10, 1.078055E+10, 1.106893E+10, 1.136447E+10, 
   1.166734E+10, 1.197770E+10, 1.229574E+10, 1.262161E+10, 1.295551E+10, 1.329761E+10, 
   1.364810E+10, 1.400716E+10, 1.437500E+10, 1.475181E+10, 1.513779E+10, 1.553314E+10, 
   1.593809E+10, 1.635284E+10, 1.677761E+10, 1.721263E+10, 1.765811E+10, 1.811431E+10, 
   1.858145E+10, 1.905977E+10, 1.954952E+10, 2.005096E+10, 2.056434E+10, 2.108992E+10, 
   2.162798E+10, 2.217878E+10, 2.274260E+10, 2.331973E+10, 2.391046E+10, 2.451507E+10, 
   2.513389E+10, 2.576720E+10, 2.641533E+10, 2.707859E+10, 2.775730E+10, 2.845182E+10, 
   2.916247E+10, 2.988959E+10, 3.063354E+10, 3.139469E+10, 3.217338E+10, 3.297001E+10, 
   3.378494E+10, 3.461857E+10, 3.547130E+10, 3.634352E+10, 3.723565E+10, 3.814810E+10, 
   3.908131E+10, 4.003572E+10, 4.101175E+10, 4.200987E+10, 4.303054E+10, 4.407421E+10, 
   4.514139E+10, 4.623255E+10, 4.734819E+10, 4.848882E+10, 4.965494E+10, 5.084709E+10, 
   5.206581E+10, 5.331165E+10, 5.458514E+10, 5.588687E+10, 5.721741E+10, 5.857734E+10, 
   5.996727E+10, 6.138782E+10, 6.283958E+10, 6.432321E+10, 6.583936E+10, 6.738865E+10, 
   6.897179E+10, 7.058944E+10, 7.224229E+10, 7.393108E+10, 7.565646E+10, 7.741924E+10, 
   7.922012E+10, 8.105989E+10, 8.293929E+10, 8.485912E+10, 8.682021E+10, 8.882334E+10, 
   9.086936E+10, 9.295910E+10, 9.509344E+10, 9.727327E+10, 9.949944E+10, 1.017729E+11, 
   1.040946E+11, 1.064653E+11, 1.088862E+11, 1.113582E+11, 1.138823E+11, 1.164594E+11, 
   1.190906E+11, 1.217770E+11, 1.245196E+11, 1.273196E+11, 1.301779E+11, 1.330958E+11, 
   1.360744E+11, 1.391148E+11, 1.422183E+11, 1.453859E+11, 1.486191E+11, 1.519189E+11, 
   1.552868E+11, 1.587239E+11, 1.622315E+11, 1.658112E+11, 1.694640E+11, 1.731915E+11, 
   1.769952E+11, 1.808762E+11, 1.848363E+11, 
])
# ============================ M = 38, I = 2 ============================
M = 38
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.204473E+01, 2.755267E+02, 7.641567E+02, 1.394948E+03, 2.141085E+03, 2.987009E+03, 
   3.922200E+03, 4.938938E+03, 6.031238E+03, 7.194295E+03, 8.424168E+03, 9.717596E+03, 
   1.107191E+04, 1.248500E+04, 1.395534E+04, 1.548194E+04, 1.706447E+04, 1.870318E+04, 
   2.039895E+04, 2.215330E+04, 2.396833E+04, 2.584673E+04, 2.779177E+04, 2.980724E+04, 
   3.189742E+04, 3.406710E+04, 3.632151E+04, 3.866631E+04, 4.110759E+04, 4.365183E+04, 
   4.630594E+04, 4.907716E+04, 5.197322E+04, 5.500216E+04, 5.817247E+04, 6.149302E+04, 
   6.497313E+04, 6.862252E+04, 7.245142E+04, 7.647050E+04, 8.069093E+04, 8.512437E+04, 
   8.978312E+04, 9.467991E+04, 9.982819E+04, 1.052420E+05, 1.109359E+05, 1.169255E+05, 
   1.232266E+05, 1.298564E+05, 1.368325E+05, 1.441732E+05, 1.518982E+05, 1.600276E+05, 
   1.685828E+05, 1.775862E+05, 1.870611E+05, 1.970320E+05, 2.075246E+05, 2.185659E+05, 
   2.301840E+05, 2.424085E+05, 2.552703E+05, 2.688020E+05, 2.830374E+05, 2.980120E+05, 
   3.137633E+05, 3.303302E+05, 3.477537E+05, 3.660763E+05, 3.853432E+05, 4.056010E+05, 
   4.268988E+05, 4.492882E+05, 4.728228E+05, 4.975589E+05, 5.235553E+05, 5.508737E+05, 
   5.795783E+05, 6.097369E+05, 6.414194E+05, 6.746999E+05, 7.096551E+05, 7.463656E+05, 
   7.849158E+05, 8.253932E+05, 8.678898E+05, 9.125018E+05, 9.593294E+05, 1.008477E+06, 
   1.060055E+06, 1.114176E+06, 1.170961E+06, 1.230534E+06, 1.293024E+06, 1.358568E+06, 
   1.427307E+06, 1.499389E+06, 1.574968E+06, 1.654205E+06, 1.737268E+06, 1.824331E+06, 
   1.915577E+06, 2.011197E+06, 2.111388E+06, 2.216358E+06, 2.326321E+06, 2.441503E+06, 
   2.562138E+06, 2.688469E+06, 2.820750E+06, 2.959246E+06, 3.104232E+06, 3.255996E+06, 
   3.414837E+06, 3.581064E+06, 3.755003E+06, 3.936990E+06, 4.127376E+06, 4.326526E+06, 
   4.534820E+06, 4.752652E+06, 4.980435E+06, 5.218597E+06, 5.467579E+06, 5.727848E+06, 
   5.999882E+06, 6.284182E+06, 6.581266E+06, 6.891677E+06, 7.215974E+06, 7.554741E+06, 
   7.908583E+06, 8.278131E+06, 8.664039E+06, 9.066986E+06, 9.487676E+06, 9.926844E+06, 
   1.038525E+07, 1.086368E+07, 1.136296E+07, 1.188394E+07, 1.242751E+07, 1.299457E+07, 
   1.358608E+07, 1.420302E+07, 1.484643E+07, 1.551736E+07, 1.621691E+07, 1.694623E+07, 
   1.770650E+07, 1.849895E+07, 1.932485E+07, 2.018552E+07, 2.108234E+07, 2.201671E+07, 
   2.299010E+07, 2.400405E+07, 2.506014E+07, 2.615998E+07, 2.730527E+07, 2.849778E+07, 
   2.973931E+07, 3.103175E+07, 3.237704E+07, 3.377721E+07, 3.523434E+07, 3.675059E+07, 
   3.832819E+07, 3.996947E+07, 4.167682E+07, 4.345272E+07, 4.529972E+07, 4.722050E+07, 
   4.921779E+07, 5.129444E+07, 5.345337E+07, 5.569762E+07, 5.803035E+07, 6.045479E+07, 
   6.297432E+07, 6.559237E+07, 6.831256E+07, 7.113858E+07, 7.407429E+07, 7.712364E+07, 
   8.029069E+07, 8.357972E+07, 8.699506E+07, 9.054124E+07, 9.422293E+07, 9.804492E+07, 
   1.020122E+08, 1.061299E+08, 1.104034E+08, 1.148380E+08, 1.194395E+08, 1.242137E+08, 
   1.291666E+08, 1.343044E+08, 1.396335E+08, 1.451606E+08, 1.508926E+08, 1.568364E+08, 
   1.629992E+08, 1.693887E+08, 1.760126E+08, 1.828788E+08, 1.899956E+08, 1.973714E+08, 
   2.050152E+08, 2.129356E+08, 2.211423E+08, 2.296448E+08, 2.384528E+08, 2.475767E+08, 
   2.570270E+08, 2.668144E+08, 2.769503E+08, 2.874459E+08, 2.983132E+08, 3.095644E+08, 
   3.212122E+08, 3.332694E+08, 3.457494E+08, 3.586659E+08, 3.720331E+08, 3.858657E+08, 
   4.001785E+08, 4.149871E+08, 4.303074E+08, 4.461558E+08, 4.625492E+08, 4.795048E+08, 
   4.970406E+08, 5.151751E+08, 5.339271E+08, 5.533161E+08, 5.733622E+08, 5.940859E+08, 
   6.155087E+08, 6.376523E+08, 6.605390E+08, 6.841921E+08, 7.086354E+08, 7.338932E+08, 
   7.599906E+08, 7.869537E+08, 8.148090E+08, 8.435838E+08, 8.733061E+08, 9.040050E+08, 
   9.357100E+08, 9.684518E+08, 1.002262E+09, 1.037172E+09, 1.073216E+09, 1.110427E+09, 
   1.148842E+09, 1.188494E+09, 1.229422E+09, 1.271664E+09, 1.315259E+09, 1.360246E+09, 
   1.406666E+09, 1.454563E+09, 1.503980E+09, 1.554961E+09, 1.607551E+09, 1.661799E+09, 
   1.717752E+09, 1.775460E+09, 1.834973E+09, 1.896345E+09, 1.959629E+09, 2.024880E+09, 
   2.092153E+09, 2.161509E+09, 2.233005E+09, 2.306703E+09, 2.382667E+09, 2.460959E+09, 
   2.541647E+09, 2.624798E+09, 2.710481E+09, 2.798769E+09, 2.889733E+09, 2.983450E+09, 
   3.079995E+09, 3.179449E+09, 3.281892E+09, 3.387406E+09, 3.496079E+09, 3.607996E+09, 
   3.723247E+09, 3.841924E+09, 3.964121E+09, 4.089935E+09, 4.219465E+09, 4.352811E+09, 
   4.490078E+09, 4.631374E+09, 4.776806E+09, 4.926487E+09, 5.080530E+09, 5.239056E+09, 
   5.402182E+09, 5.570034E+09, 5.742737E+09, 5.920420E+09, 6.103217E+09, 6.291265E+09, 
   6.484700E+09, 6.683667E+09, 6.888311E+09, 7.098785E+09, 7.315238E+09, 7.537828E+09, 
   7.766718E+09, 8.002069E+09, 8.244055E+09, 8.492843E+09, 8.748612E+09, 9.011546E+09, 
   9.281825E+09, 9.559641E+09, 9.845188E+09, 1.013866E+10, 1.044028E+10, 1.075023E+10, 
   1.106873E+10, 1.139601E+10, 1.173229E+10, 1.207779E+10, 1.243274E+10, 1.279739E+10, 
   1.317199E+10, 1.355678E+10, 1.395200E+10, 1.435795E+10, 1.477487E+10, 1.520304E+10, 
   1.564275E+10, 1.609427E+10, 1.655791E+10, 1.703396E+10, 1.752272E+10, 1.802452E+10, 
   1.853967E+10, 1.906851E+10, 1.961136E+10, 2.016858E+10, 2.074050E+10, 2.132749E+10, 
   2.192992E+10, 2.254816E+10, 2.318259E+10, 2.383362E+10, 2.450162E+10, 2.518702E+10, 
   2.589023E+10, 2.661168E+10, 2.735181E+10, 2.811107E+10, 2.888990E+10, 2.968877E+10, 
   3.050817E+10, 3.134858E+10, 3.221049E+10, 3.309441E+10, 3.400087E+10, 3.493038E+10, 
   3.588350E+10, 3.686078E+10, 3.786278E+10, 3.889008E+10, 3.994327E+10, 4.102294E+10, 
   4.212972E+10, 4.326423E+10, 4.442713E+10, 4.561905E+10, 4.684066E+10, 4.809266E+10, 
   4.937573E+10, 5.069060E+10, 5.203798E+10, 5.341862E+10, 5.483328E+10, 5.628273E+10, 
   5.776775E+10, 5.928915E+10, 6.084777E+10, 6.244443E+10, 6.408001E+10, 6.575535E+10, 
   6.747136E+10, 6.922897E+10, 7.102908E+10, 7.287265E+10, 7.476064E+10, 7.669406E+10, 
   7.867389E+10, 8.070118E+10, 8.277699E+10, 8.490233E+10, 8.707838E+10, 8.930618E+10, 
   9.158690E+10, 9.392171E+10, 9.631174E+10, 9.875825E+10, 1.012625E+11, 1.038256E+11, 
   1.064490E+11, 1.091340E+11, 1.118817E+11, 1.146937E+11, 1.175714E+11, 1.205160E+11, 
   1.235291E+11, 1.266122E+11, 1.297666E+11, 1.329940E+11, 1.362959E+11, 1.396740E+11, 
   1.431297E+11, 1.466648E+11, 1.502809E+11, 1.539798E+11, 1.577632E+11, 1.616328E+11, 
   1.655906E+11, 1.696383E+11, 1.737779E+11, 1.780112E+11, 1.823402E+11, 1.867670E+11, 
   1.912934E+11, 1.959217E+11, 2.006539E+11, 2.054922E+11, 2.104388E+11, 2.154959E+11, 
   2.206657E+11, 2.259507E+11, 2.313531E+11, 2.368755E+11, 2.425201E+11, 2.482896E+11, 
   2.541865E+11, 2.602133E+11, 2.663727E+11, 2.726675E+11, 2.791002E+11, 2.856738E+11, 
   2.923911E+11, 2.992550E+11, 3.062683E+11, 3.134342E+11, 3.207557E+11, 3.282359E+11, 
   3.358780E+11, 3.436851E+11, 3.516606E+11, 3.598080E+11, 3.681304E+11, 3.766314E+11, 
   3.853147E+11, 3.941836E+11, 4.032420E+11, 4.124935E+11, 4.219420E+11, 4.315913E+11, 
   4.414453E+11, 4.515082E+11, 4.617838E+11, 4.722764E+11, 4.829902E+11, 4.939296E+11, 
   5.050988E+11, 5.165024E+11, 5.281448E+11, 5.400307E+11, 5.521648E+11, 5.645520E+11, 
   5.771968E+11, 5.901045E+11, 6.032800E+11, 6.167283E+11, 6.304549E+11, 6.444647E+11, 
   6.587637E+11, 6.733568E+11, 6.882497E+11, 7.034485E+11, 7.189586E+11, 7.347858E+11, 
   7.509365E+11, 7.674164E+11, 7.842318E+11, 
])
# ============================ M = 38, I = 3 ============================
#M = 38
#I = 3
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   2.114267E+01, 4.891941E+02, 1.359005E+03, 2.482187E+03, 3.810911E+03, 5.317433E+03, 
#   6.983007E+03, 8.793869E+03, 1.073937E+04, 1.281100E+04, 1.500189E+04, 1.730652E+04, 
#   1.972067E+04, 2.224136E+04, 2.486690E+04, 2.759692E+04, 3.043232E+04, 3.337534E+04, 
#   3.642945E+04, 3.959935E+04, 4.289090E+04, 4.631105E+04, 4.986778E+04, 5.357006E+04, 
#   5.742776E+04, 6.145165E+04, 6.565336E+04, 7.004533E+04, 7.464078E+04, 7.945381E+04, 
#   8.449930E+04, 8.979288E+04, 9.535112E+04, 1.011914E+05, 1.073320E+05, 1.137921E+05, 
#   1.205918E+05, 1.277524E+05, 1.352961E+05, 1.432463E+05, 1.516274E+05, 1.604653E+05, 
#   1.697869E+05, 1.796207E+05, 1.899966E+05, 2.009459E+05, 2.125016E+05, 2.246984E+05, 
#   2.375726E+05, 2.511625E+05, 2.655085E+05, 2.806526E+05, 2.966394E+05, 3.135155E+05, 
#   3.313302E+05, 3.501348E+05, 3.699837E+05, 3.909338E+05, 4.130450E+05, 4.363800E+05, 
#   4.610053E+05, 4.869899E+05, 5.144068E+05, 5.433326E+05, 5.738477E+05, 6.060365E+05, 
#   6.399877E+05, 6.757943E+05, 7.135540E+05, 7.533691E+05, 7.953475E+05, 8.396016E+05, 
#   8.862501E+05, 9.354170E+05, 9.872326E+05, 1.041834E+06, 1.099362E+06, 1.159970E+06, 
#   1.223814E+06, 1.291058E+06, 1.361875E+06, 1.436448E+06, 1.514963E+06, 1.597622E+06, 
#   1.684631E+06, 1.776208E+06, 1.872582E+06, 1.973991E+06, 2.080685E+06, 2.192925E+06, 
#   2.310985E+06, 2.435151E+06, 2.565723E+06, 2.703013E+06, 2.847349E+06, 2.999073E+06, 
#   3.158544E+06, 3.326135E+06, 3.502238E+06, 3.687262E+06, 3.881632E+06, 4.085795E+06, 
#   4.300218E+06, 4.525386E+06, 4.761808E+06, 5.010015E+06, 5.270561E+06, 5.544023E+06, 
#   5.831005E+06, 6.132139E+06, 6.448077E+06, 6.779509E+06, 7.127149E+06, 7.491742E+06, 
#   7.874068E+06, 8.274934E+06, 8.695188E+06, 9.135714E+06, 9.597426E+06, 1.008128E+07, 
#   1.058828E+07, 1.111946E+07, 1.167591E+07, 1.225875E+07, 1.286916E+07, 1.350836E+07, 
#   1.417762E+07, 1.487827E+07, 1.561169E+07, 1.637931E+07, 1.718264E+07, 1.802322E+07, 
#   1.890268E+07, 1.982269E+07, 2.078502E+07, 2.179147E+07, 2.284394E+07, 2.394440E+07, 
#   2.509491E+07, 2.629758E+07, 2.755463E+07, 2.886836E+07, 3.024115E+07, 3.167549E+07, 
#   3.317396E+07, 3.473923E+07, 3.637408E+07, 3.808140E+07, 3.986421E+07, 4.172560E+07, 
#   4.366881E+07, 4.569720E+07, 4.781425E+07, 5.002358E+07, 5.232895E+07, 5.473423E+07, 
#   5.724346E+07, 5.986087E+07, 6.259076E+07, 6.543765E+07, 6.840621E+07, 7.150130E+07, 
#   7.472796E+07, 7.809134E+07, 8.159689E+07, 8.525018E+07, 8.905702E+07, 9.302344E+07, 
#   9.715561E+07, 1.014600E+08, 1.059434E+08, 1.106125E+08, 1.154747E+08, 1.205373E+08, 
#   1.258080E+08, 1.312948E+08, 1.370058E+08, 1.429498E+08, 1.491353E+08, 1.555717E+08, 
#   1.622683E+08, 1.692349E+08, 1.764817E+08, 1.840192E+08, 1.918581E+08, 2.000096E+08, 
#   2.084854E+08, 2.172974E+08, 2.264580E+08, 2.359800E+08, 2.458768E+08, 2.561618E+08, 
#   2.668494E+08, 2.779541E+08, 2.894910E+08, 3.014758E+08, 3.139247E+08, 3.268542E+08, 
#   3.402816E+08, 3.542247E+08, 3.687019E+08, 3.837322E+08, 3.993351E+08, 4.155309E+08, 
#   4.323406E+08, 4.497856E+08, 4.678883E+08, 4.866716E+08, 5.061593E+08, 5.263759E+08, 
#   5.473466E+08, 5.690976E+08, 5.916558E+08, 6.150488E+08, 6.393054E+08, 6.644552E+08, 
#   6.905285E+08, 7.175569E+08, 7.455728E+08, 7.746095E+08, 8.047015E+08, 8.358844E+08, 
#   8.681948E+08, 9.016707E+08, 9.363506E+08, 9.722748E+08, 1.009485E+09, 1.048023E+09, 
#   1.087934E+09, 1.129262E+09, 1.172054E+09, 1.216359E+09, 1.262225E+09, 1.309703E+09, 
#   1.358847E+09, 1.409710E+09, 1.462348E+09, 1.516818E+09, 1.573179E+09, 1.631492E+09, 
#   1.691820E+09, 1.754226E+09, 1.818778E+09, 1.885543E+09, 1.954593E+09, 2.025997E+09, 
#   2.099833E+09, 2.176176E+09, 2.255104E+09, 2.336699E+09, 2.421044E+09, 2.508225E+09, 
#   2.598330E+09, 2.691450E+09, 2.787677E+09, 2.887110E+09, 2.989844E+09, 3.095983E+09, 
#   3.205630E+09, 3.318893E+09, 3.435882E+09, 3.556711E+09, 3.681495E+09, 3.810355E+09, 
#   3.943413E+09, 4.080797E+09, 4.222636E+09, 4.369064E+09, 4.520218E+09, 4.676237E+09, 
#   4.837271E+09, 5.003464E+09, 5.174971E+09, 5.351948E+09, 5.534557E+09, 5.722964E+09, 
#   5.917339E+09, 6.117857E+09, 6.324694E+09, 6.538040E+09, 6.758080E+09, 6.985008E+09, 
#   7.219026E+09, 7.460336E+09, 7.709151E+09, 7.965683E+09, 8.230156E+09, 8.502798E+09, 
#   8.783835E+09, 9.073514E+09, 9.372076E+09, 9.679778E+09, 9.996872E+09, 1.032362E+10, 
#   1.066031E+10, 1.100720E+10, 1.136460E+10, 1.173278E+10, 1.211205E+10, 1.250272E+10, 
#   1.290510E+10, 1.331953E+10, 1.374632E+10, 1.418584E+10, 1.463841E+10, 1.510441E+10, 
#   1.558419E+10, 1.607815E+10, 1.658665E+10, 1.711010E+10, 1.764891E+10, 1.820349E+10, 
#   1.877426E+10, 1.936167E+10, 1.996616E+10, 2.058819E+10, 2.122823E+10, 2.188675E+10, 
#   2.256426E+10, 2.326126E+10, 2.397827E+10, 2.471582E+10, 2.547443E+10, 2.625469E+10, 
#   2.705716E+10, 2.788242E+10, 2.873105E+10, 2.960369E+10, 3.050095E+10, 3.142348E+10, 
#   3.237193E+10, 3.334698E+10, 3.434930E+10, 3.537963E+10, 3.643865E+10, 3.752712E+10, 
#   3.864580E+10, 3.979546E+10, 4.097687E+10, 4.219088E+10, 4.343828E+10, 4.471995E+10, 
#   4.603674E+10, 4.738953E+10, 4.877926E+10, 5.020682E+10, 5.167320E+10, 5.317935E+10, 
#   5.472628E+10, 5.631499E+10, 5.794654E+10, 5.962198E+10, 6.134241E+10, 6.310893E+10, 
#   6.492270E+10, 6.678488E+10, 6.869666E+10, 7.065925E+10, 7.267391E+10, 7.474191E+10, 
#   7.686454E+10, 7.904315E+10, 8.127909E+10, 8.357376E+10, 8.592858E+10, 8.834501E+10, 
#   9.082452E+10, 9.336865E+10, 9.597894E+10, 9.865699E+10, 1.014044E+11, 1.042228E+11, 
#   1.071140E+11, 1.100797E+11, 1.131215E+11, 1.162414E+11, 1.194412E+11, 1.227227E+11, 
#   1.260879E+11, 1.295388E+11, 1.330774E+11, 1.367056E+11, 1.404258E+11, 1.442398E+11, 
#   1.481501E+11, 1.521587E+11, 1.562681E+11, 1.604804E+11, 1.647981E+11, 1.692236E+11, 
#   1.737595E+11, 1.784082E+11, 1.831723E+11, 1.880545E+11, 1.930575E+11, 1.981840E+11, 
#   2.034369E+11, 2.088190E+11, 2.143331E+11, 2.199825E+11, 2.257700E+11, 2.316989E+11, 
#   2.377723E+11, 2.439934E+11, 2.503655E+11, 2.568921E+11, 2.635765E+11, 2.704224E+11, 
#   2.774333E+11, 2.846129E+11, 2.919649E+11, 2.994930E+11, 3.072014E+11, 3.150938E+11, 
#   3.231745E+11, 3.314473E+11, 3.399166E+11, 3.485869E+11, 3.574623E+11, 3.665474E+11, 
#   3.758468E+11, 3.853650E+11, 3.951070E+11, 4.050774E+11, 4.152814E+11, 4.257237E+11, 
#   4.364097E+11, 4.473446E+11, 4.585337E+11, 4.699825E+11, 4.816965E+11, 4.936812E+11, 
#   5.059430E+11, 5.184869E+11, 5.313196E+11, 5.444469E+11, 5.578752E+11, 5.716107E+11, 
#   5.856598E+11, 6.000295E+11, 6.147262E+11, 6.297569E+11, 6.451285E+11, 6.608482E+11, 
#   6.769232E+11, 6.933610E+11, 7.101692E+11, 7.273552E+11, 7.449272E+11, 7.628930E+11, 
#   7.812608E+11, 8.000387E+11, 8.192353E+11, 8.388593E+11, 8.589193E+11, 8.794245E+11, 
#   9.003836E+11, 9.218060E+11, 9.437014E+11, 9.660792E+11, 9.889492E+11, 1.012321E+12, 
#   1.036206E+12, 1.060613E+12, 1.085554E+12, 1.111039E+12, 1.137078E+12, 1.163684E+12, 
#   1.190867E+12, 1.218639E+12, 1.247012E+12, 1.275998E+12, 1.305609E+12, 1.335857E+12, 
#   1.366755E+12, 1.398316E+12, 1.430553E+12, 1.463479E+12, 1.497108E+12, 1.531454E+12, 
#   1.566531E+12, 1.602353E+12, 1.638935E+12, 1.676291E+12, 1.714437E+12, 1.753389E+12, 
#   1.793160E+12, 1.833769E+12, 1.875231E+12, 1.917562E+12, 1.960780E+12, 2.004901E+12, 
#   2.049943E+12, 2.095923E+12, 2.142860E+12, 2.190772E+12, 2.239677E+12, 2.289595E+12, 
#   2.340545E+12, 2.392546E+12, 2.445619E+12, 
#])
# ============================ M = 39, I = 1 ============================
M = 39
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.441130E+01, 3.235283E+02, 8.984785E+02, 1.640815E+03, 2.519098E+03, 3.516040E+03, 
   4.622384E+03, 5.834648E+03, 7.153276E+03, 8.581086E+03, 1.012212E+04, 1.178096E+04, 
   1.356235E+04, 1.547098E+04, 1.751151E+04, 1.968852E+04, 2.200662E+04, 2.447049E+04, 
   2.708504E+04, 2.985539E+04, 3.278708E+04, 3.588607E+04, 3.915884E+04, 4.261247E+04, 
   4.625465E+04, 5.009380E+04, 5.413906E+04, 5.840034E+04, 6.288836E+04, 6.761466E+04, 
   7.259164E+04, 7.783262E+04, 8.335181E+04, 8.916436E+04, 9.528638E+04, 1.017350E+05, 
   1.085284E+05, 1.156856E+05, 1.232272E+05, 1.311744E+05, 1.395500E+05, 1.483776E+05, 
   1.576826E+05, 1.674913E+05, 1.778316E+05, 1.887327E+05, 2.002254E+05, 2.123422E+05, 
   2.251168E+05, 2.385852E+05, 2.527848E+05, 2.677551E+05, 2.835372E+05, 3.001748E+05, 
   3.177134E+05, 3.362007E+05, 3.556871E+05, 3.762249E+05, 3.978696E+05, 4.206792E+05, 
   4.447142E+05, 4.700384E+05, 4.967184E+05, 5.248246E+05, 5.544300E+05, 5.856118E+05, 
   6.184503E+05, 6.530302E+05, 6.894398E+05, 7.277718E+05, 7.681233E+05, 8.105959E+05, 
   8.552960E+05, 9.023352E+05, 9.518298E+05, 1.003902E+06, 1.058680E+06, 1.116298E+06, 
   1.176894E+06, 1.240615E+06, 1.307616E+06, 1.378054E+06, 1.452099E+06, 1.529924E+06, 
   1.611715E+06, 1.697660E+06, 1.787960E+06, 1.882824E+06, 1.982471E+06, 2.087128E+06, 
   2.197034E+06, 2.312437E+06, 2.433598E+06, 2.560787E+06, 2.694287E+06, 2.834395E+06, 
   2.981419E+06, 3.135681E+06, 3.297516E+06, 3.467276E+06, 3.645326E+06, 3.832048E+06, 
   4.027840E+06, 4.233115E+06, 4.448308E+06, 4.673869E+06, 4.910266E+06, 5.157993E+06, 
   5.417556E+06, 5.689491E+06, 5.974349E+06, 6.272709E+06, 6.585171E+06, 6.912363E+06, 
   7.254935E+06, 7.613568E+06, 7.988968E+06, 8.381869E+06, 8.793039E+06, 9.223274E+06, 
   9.673404E+06, 1.014429E+07, 1.063683E+07, 1.115196E+07, 1.169064E+07, 1.225390E+07, 
   1.284277E+07, 1.345835E+07, 1.410177E+07, 1.477422E+07, 1.547691E+07, 1.621112E+07, 
   1.697818E+07, 1.777946E+07, 1.861638E+07, 1.949044E+07, 2.040317E+07, 2.135616E+07, 
   2.235109E+07, 2.338967E+07, 2.447370E+07, 2.560503E+07, 2.678560E+07, 2.801740E+07, 
   2.930251E+07, 3.064308E+07, 3.204136E+07, 3.349965E+07, 3.502035E+07, 3.660599E+07, 
   3.825912E+07, 3.998243E+07, 4.177870E+07, 4.365082E+07, 4.560178E+07, 4.763465E+07, 
   4.975264E+07, 5.195911E+07, 5.425747E+07, 5.665129E+07, 5.914429E+07, 6.174025E+07, 
   6.444319E+07, 6.725716E+07, 7.018647E+07, 7.323546E+07, 7.640872E+07, 7.971097E+07, 
   8.314707E+07, 8.672208E+07, 9.044122E+07, 9.430991E+07, 9.833376E+07, 1.025185E+08, 
   1.068702E+08, 1.113950E+08, 1.160993E+08, 1.209897E+08, 1.260732E+08, 1.313567E+08, 
   1.368476E+08, 1.425535E+08, 1.484821E+08, 1.546416E+08, 1.610404E+08, 1.676870E+08, 
   1.745904E+08, 1.817598E+08, 1.892048E+08, 1.969350E+08, 2.049609E+08, 2.132927E+08, 
   2.219414E+08, 2.309181E+08, 2.402345E+08, 2.499023E+08, 2.599341E+08, 2.703424E+08, 
   2.811403E+08, 2.923415E+08, 3.039598E+08, 3.160099E+08, 3.285064E+08, 3.414648E+08, 
   3.549009E+08, 3.688310E+08, 3.832720E+08, 3.982413E+08, 4.137567E+08, 4.298369E+08, 
   4.465008E+08, 4.637681E+08, 4.816591E+08, 5.001946E+08, 5.193963E+08, 5.392861E+08, 
   5.598872E+08, 5.812229E+08, 6.033178E+08, 6.261966E+08, 6.498853E+08, 6.744104E+08, 
   6.997992E+08, 7.260801E+08, 7.532821E+08, 7.814349E+08, 8.105694E+08, 8.407173E+08, 
   8.719115E+08, 9.041853E+08, 9.375733E+08, 9.721116E+08, 1.007836E+09, 1.044786E+09, 
   1.082998E+09, 1.122514E+09, 1.163375E+09, 1.205622E+09, 1.249300E+09, 1.294453E+09, 
   1.341128E+09, 1.389371E+09, 1.439233E+09, 1.490762E+09, 1.544011E+09, 1.599032E+09, 
   1.655881E+09, 1.714614E+09, 1.775287E+09, 1.837962E+09, 1.902698E+09, 1.969558E+09, 
   2.038608E+09, 2.109914E+09, 2.183542E+09, 2.259564E+09, 2.338052E+09, 2.419079E+09, 
   2.502722E+09, 2.589058E+09, 2.678169E+09, 2.770136E+09, 2.865044E+09, 2.962980E+09, 
   3.064034E+09, 3.168297E+09, 3.275863E+09, 3.386830E+09, 3.501296E+09, 3.619364E+09, 
   3.741140E+09, 3.866729E+09, 3.996242E+09, 4.129794E+09, 4.267501E+09, 4.409482E+09, 
   4.555859E+09, 4.706759E+09, 4.862310E+09, 5.022647E+09, 5.187903E+09, 5.358218E+09, 
   5.533738E+09, 5.714608E+09, 5.900977E+09, 6.093003E+09, 6.290843E+09, 6.494661E+09, 
   6.704620E+09, 6.920895E+09, 7.143662E+09, 7.373097E+09, 7.609388E+09, 7.852725E+09, 
   8.103298E+09, 8.361310E+09, 8.626961E+09, 8.900465E+09, 9.182033E+09, 9.471883E+09, 
   9.770245E+09, 1.007735E+10, 1.039343E+10, 1.071873E+10, 1.105349E+10, 1.139798E+10, 
   1.175245E+10, 1.211718E+10, 1.249242E+10, 1.287848E+10, 1.327562E+10, 1.368416E+10, 
   1.410437E+10, 1.453659E+10, 1.498112E+10, 1.543829E+10, 1.590843E+10, 1.639188E+10, 
   1.688899E+10, 1.740010E+10, 1.792560E+10, 1.846585E+10, 1.902123E+10, 1.959214E+10, 
   2.017898E+10, 2.078215E+10, 2.140208E+10, 2.203920E+10, 2.269394E+10, 2.336677E+10, 
   2.405813E+10, 2.476850E+10, 2.549835E+10, 2.624820E+10, 2.701854E+10, 2.780989E+10, 
   2.862278E+10, 2.945774E+10, 3.031533E+10, 3.119613E+10, 3.210070E+10, 3.302964E+10, 
   3.398356E+10, 3.496307E+10, 3.596882E+10, 3.700144E+10, 3.806161E+10, 3.915000E+10, 
   4.026729E+10, 4.141422E+10, 4.259149E+10, 
])
# ============================ M = 40, I = 1 ============================
M = 40
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.004900E+01, 5.404774E+02, 1.395108E+03, 2.506082E+03, 3.835919E+03, 5.351474E+03, 
   7.029945E+03, 8.855735E+03, 1.081751E+04, 1.290664E+04, 1.511646E+04, 1.744192E+04, 
   1.987944E+04, 2.242676E+04, 2.508287E+04, 2.784786E+04, 3.072288E+04, 3.371002E+04, 
   3.681226E+04, 4.003339E+04, 4.337794E+04, 4.685113E+04, 5.045884E+04, 5.420757E+04, 
   5.810439E+04, 6.215692E+04, 6.637333E+04, 7.076230E+04, 7.533301E+04, 8.009516E+04, 
   8.505890E+04, 9.023489E+04, 9.563428E+04, 1.012687E+05, 1.071503E+05, 1.132916E+05, 
   1.197059E+05, 1.264068E+05, 1.334085E+05, 1.407258E+05, 1.483740E+05, 1.563690E+05, 
   1.647273E+05, 1.734661E+05, 1.826031E+05, 1.921570E+05, 2.021467E+05, 2.125924E+05, 
   2.235145E+05, 2.349346E+05, 2.468748E+05, 2.593584E+05, 2.724092E+05, 2.860521E+05, 
   3.003129E+05, 3.152184E+05, 3.307964E+05, 3.470757E+05, 3.640863E+05, 3.818591E+05, 
   4.004264E+05, 4.198216E+05, 4.400793E+05, 4.612355E+05, 4.833274E+05, 5.063938E+05, 
   5.304746E+05, 5.556115E+05, 5.818476E+05, 6.092275E+05, 6.377975E+05, 6.676056E+05, 
   6.987017E+05, 7.311371E+05, 7.649653E+05, 8.002417E+05, 8.370235E+05, 8.753701E+05, 
   9.153429E+05, 9.570057E+05, 1.000424E+06, 1.045667E+06, 1.092804E+06, 1.141909E+06, 
   1.193058E+06, 1.246328E+06, 1.301800E+06, 1.359560E+06, 1.419692E+06, 1.482287E+06, 
   1.547438E+06, 1.615239E+06, 1.685791E+06, 1.759196E+06, 1.835560E+06, 1.914992E+06, 
   1.997604E+06, 2.083515E+06, 2.172845E+06, 2.265718E+06, 2.362263E+06, 2.462614E+06, 
   2.566907E+06, 2.675284E+06, 2.787893E+06, 2.904884E+06, 3.026413E+06, 3.152641E+06, 
   3.283735E+06, 3.419866E+06, 3.561210E+06, 3.707951E+06, 3.860277E+06, 4.018381E+06, 
   4.182465E+06, 4.352734E+06, 4.529402E+06, 4.712688E+06, 4.902818E+06, 5.100026E+06, 
   5.304552E+06, 5.516643E+06, 5.736555E+06, 5.964552E+06, 6.200902E+06, 6.445887E+06, 
   6.699793E+06, 6.962916E+06, 7.235561E+06, 7.518042E+06, 7.810681E+06, 8.113813E+06, 
   8.427778E+06, 8.752930E+06, 9.089631E+06, 9.438254E+06, 9.799183E+06, 1.017282E+07, 
   1.055955E+07, 1.095982E+07, 1.137404E+07, 1.180266E+07, 1.224613E+07, 1.270493E+07, 
   1.317952E+07, 1.367042E+07, 1.417812E+07, 1.470314E+07, 1.524603E+07, 1.580733E+07, 
   1.638762E+07, 1.698747E+07, 1.760749E+07, 1.824829E+07, 1.891050E+07, 1.959476E+07, 
   2.030176E+07, 2.103216E+07, 2.178669E+07, 2.256604E+07, 2.337098E+07, 2.420225E+07, 
   2.506064E+07, 2.594695E+07, 2.686200E+07, 2.780665E+07, 2.878175E+07, 2.978819E+07, 
   3.082689E+07, 3.189878E+07, 3.300483E+07, 3.414602E+07, 3.532335E+07, 3.653787E+07, 
   3.779064E+07, 3.908275E+07, 4.041532E+07, 4.178948E+07, 4.320643E+07, 4.466736E+07, 
   4.617351E+07, 4.772614E+07, 4.932655E+07, 5.097607E+07, 5.267607E+07, 5.442794E+07, 
   5.623310E+07, 5.809303E+07, 6.000923E+07, 6.198324E+07, 6.401663E+07, 6.611101E+07, 
   6.826805E+07, 7.048942E+07, 7.277688E+07, 7.513218E+07, 7.755716E+07, 8.005366E+07, 
   8.262360E+07, 8.526894E+07, 8.799165E+07, 9.079379E+07, 9.367745E+07, 9.664478E+07, 
   9.969795E+07, 1.028392E+08, 1.060709E+08, 1.093953E+08, 1.128148E+08, 1.163319E+08, 
   1.199491E+08, 1.236690E+08, 1.274942E+08, 1.314274E+08, 1.354714E+08, 1.396289E+08, 
   1.439028E+08, 1.482961E+08, 1.528118E+08, 1.574530E+08, 1.622227E+08, 1.671243E+08, 
   1.721609E+08, 1.773360E+08, 1.826530E+08, 1.881154E+08, 1.937268E+08, 1.994907E+08, 
   2.054111E+08, 2.114916E+08, 2.177363E+08, 2.241491E+08, 2.307340E+08, 2.374953E+08, 
   2.444371E+08, 2.515639E+08, 2.588801E+08, 2.663902E+08, 2.740988E+08, 2.820107E+08, 
   2.901307E+08, 2.984638E+08, 3.070148E+08, 3.157891E+08, 3.247918E+08, 3.340283E+08, 
   3.435041E+08, 3.532247E+08, 3.631959E+08, 3.734234E+08, 3.839132E+08, 3.946713E+08, 
   4.057040E+08, 4.170175E+08, 4.286183E+08, 4.405129E+08, 4.527080E+08, 4.652104E+08, 
   4.780272E+08, 4.911655E+08, 5.046324E+08, 5.184355E+08, 5.325822E+08, 5.470802E+08, 
   5.619374E+08, 5.771618E+08, 5.927616E+08, 6.087451E+08, 6.251208E+08, 6.418972E+08, 
   6.590834E+08, 6.766882E+08, 6.947209E+08, 7.131907E+08, 7.321073E+08, 7.514802E+08, 
   7.713196E+08, 7.916353E+08, 8.124378E+08, 8.337375E+08, 8.555451E+08, 8.778715E+08, 
   9.007278E+08, 9.241253E+08, 9.480754E+08, 9.725901E+08, 9.976811E+08, 1.023361E+09, 
   1.049641E+09, 1.076536E+09, 1.104057E+09, 1.132217E+09, 1.161031E+09, 1.190511E+09, 
   1.220672E+09, 1.251527E+09, 1.283091E+09, 1.315379E+09, 1.348405E+09, 1.382186E+09, 
   1.416735E+09, 1.452069E+09, 1.488205E+09, 1.525157E+09, 1.562944E+09, 1.601581E+09, 
   1.641086E+09, 1.681477E+09, 1.722771E+09, 1.764986E+09, 1.808142E+09, 1.852256E+09, 
   1.897349E+09, 1.943438E+09, 1.990545E+09, 2.038690E+09, 2.087892E+09, 2.138173E+09, 
   2.189554E+09, 2.242057E+09, 2.295703E+09, 2.350516E+09, 2.406517E+09, 2.463730E+09, 
   2.522179E+09, 2.581888E+09, 2.642880E+09, 2.705181E+09, 2.768815E+09, 2.833810E+09, 
   2.900190E+09, 2.967982E+09, 3.037213E+09, 3.107910E+09, 3.180103E+09, 3.253818E+09, 
   3.329084E+09, 3.405932E+09, 3.484391E+09, 3.564490E+09, 3.646262E+09, 3.729737E+09, 
   3.814947E+09, 3.901925E+09, 3.990703E+09, 4.081315E+09, 4.173795E+09, 4.268177E+09, 
   4.364498E+09, 4.462791E+09, 4.563094E+09, 4.665444E+09, 4.769878E+09, 4.876434E+09, 
   4.985151E+09, 5.096068E+09, 5.209225E+09, 5.324663E+09, 5.442423E+09, 5.562547E+09, 
   5.685077E+09, 5.810058E+09, 5.937532E+09, 6.067544E+09, 6.200141E+09, 6.335367E+09, 
   6.473270E+09, 6.613898E+09, 6.757298E+09, 6.903520E+09, 7.052613E+09, 7.204628E+09, 
   7.359617E+09, 7.517632E+09, 7.678725E+09, 7.842951E+09, 8.010364E+09, 8.181020E+09, 
   8.354976E+09, 8.532287E+09, 8.713013E+09, 8.897212E+09, 9.084945E+09, 9.276272E+09, 
   9.471255E+09, 9.669957E+09, 9.872441E+09, 1.007877E+10, 1.028902E+10, 1.050324E+10, 
   1.072151E+10, 1.094389E+10, 1.117046E+10, 1.140128E+10, 1.163644E+10, 1.187599E+10, 
   1.212001E+10, 1.236859E+10, 1.262179E+10, 1.287969E+10, 1.314237E+10, 1.340991E+10, 
   1.368239E+10, 1.395990E+10, 1.424251E+10, 1.453031E+10, 1.482338E+10, 1.512182E+10, 
   1.542571E+10, 1.573514E+10, 1.605019E+10, 1.637097E+10, 1.669757E+10, 1.703007E+10, 
   1.736858E+10, 1.771319E+10, 1.806400E+10, 1.842112E+10, 1.878464E+10, 1.915466E+10, 
   1.953130E+10, 1.991465E+10, 2.030483E+10, 2.070194E+10, 2.110609E+10, 2.151740E+10, 
   2.193598E+10, 2.236194E+10, 2.279540E+10, 2.323649E+10, 2.368531E+10, 2.414200E+10, 
   2.460667E+10, 2.507945E+10, 2.556047E+10, 2.604986E+10, 2.654774E+10, 2.705425E+10, 
   2.756952E+10, 2.809370E+10, 2.862691E+10, 2.916930E+10, 2.972101E+10, 3.028218E+10, 
   3.085295E+10, 3.143349E+10, 3.202392E+10, 3.262442E+10, 3.323512E+10, 3.385619E+10, 
   3.448778E+10, 3.513006E+10, 3.578318E+10, 3.644732E+10, 3.712262E+10, 3.780928E+10, 
   3.850744E+10, 3.921730E+10, 3.993903E+10, 4.067279E+10, 4.141878E+10, 4.217717E+10, 
   4.294815E+10, 4.373191E+10, 4.452864E+10, 4.533853E+10, 4.616177E+10, 4.699857E+10, 
   4.784912E+10, 4.871362E+10, 4.959228E+10, 5.048531E+10, 5.139292E+10, 5.231533E+10, 
   5.325274E+10, 5.420538E+10, 5.517347E+10, 5.615724E+10, 5.715690E+10, 5.817270E+10, 
   5.920486E+10, 6.025363E+10, 6.131924E+10, 6.240194E+10, 6.350196E+10, 6.461956E+10, 
   6.575499E+10, 6.690851E+10, 6.808037E+10, 6.927084E+10, 7.048018E+10, 7.170866E+10, 
   7.295655E+10, 7.422412E+10, 7.551166E+10, 7.681944E+10, 7.814776E+10, 7.949689E+10, 
   8.086714E+10, 8.225879E+10, 8.367216E+10, 
])
# ============================ M = 40, I = 2 ============================
M = 40
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.014608E+01, 5.425089E+02, 1.400392E+03, 2.515599E+03, 3.850504E+03, 5.371838E+03, 
   7.056709E+03, 8.889462E+03, 1.085873E+04, 1.295583E+04, 1.517410E+04, 1.750849E+04, 
   1.995540E+04, 2.251260E+04, 2.517908E+04, 2.795495E+04, 3.084138E+04, 3.384051E+04, 
   3.695533E+04, 4.018965E+04, 4.354804E+04, 4.703576E+04, 5.065872E+04, 5.442344E+04, 
   5.833704E+04, 6.240719E+04, 6.664209E+04, 7.105046E+04, 7.564153E+04, 8.042504E+04, 
   8.541120E+04, 9.061072E+04, 9.603480E+04, 1.016951E+05, 1.076039E+05, 1.137738E+05, 
   1.202181E+05, 1.269504E+05, 1.339851E+05, 1.413369E+05, 1.490214E+05, 1.570545E+05, 
   1.654527E+05, 1.742334E+05, 1.834144E+05, 1.930143E+05, 2.030524E+05, 2.135487E+05, 
   2.245240E+05, 2.359997E+05, 2.479984E+05, 2.605431E+05, 2.736580E+05, 2.873680E+05, 
   3.016992E+05, 3.166784E+05, 3.323335E+05, 3.486937E+05, 3.657888E+05, 3.836501E+05, 
   4.023101E+05, 4.218022E+05, 4.421614E+05, 4.634237E+05, 4.856267E+05, 5.088091E+05, 
   5.330114E+05, 5.582752E+05, 5.846439E+05, 6.121625E+05, 6.408774E+05, 6.708369E+05, 
   7.020911E+05, 7.346917E+05, 7.686925E+05, 8.041490E+05, 8.411189E+05, 8.796618E+05, 
   9.198396E+05, 9.617162E+05, 1.005358E+06, 1.050833E+06, 1.098213E+06, 1.147572E+06, 
   1.198984E+06, 1.252530E+06, 1.308289E+06, 1.366348E+06, 1.426792E+06, 1.489712E+06, 
   1.555201E+06, 1.623355E+06, 1.694275E+06, 1.768062E+06, 1.844824E+06, 1.924670E+06, 
   2.007715E+06, 2.094075E+06, 2.183873E+06, 2.277232E+06, 2.374284E+06, 2.475162E+06, 
   2.580003E+06, 2.688950E+06, 2.802152E+06, 2.919759E+06, 3.041929E+06, 3.168823E+06, 
   3.300609E+06, 3.437459E+06, 3.579551E+06, 3.727068E+06, 3.880201E+06, 4.039143E+06, 
   4.204097E+06, 4.375270E+06, 4.552876E+06, 4.737137E+06, 4.928278E+06, 5.126536E+06, 
   5.332151E+06, 5.545373E+06, 5.766458E+06, 5.995671E+06, 6.233284E+06, 6.479578E+06, 
   6.734841E+06, 6.999372E+06, 7.273476E+06, 7.557470E+06, 7.851679E+06, 8.156436E+06, 
   8.472086E+06, 8.798983E+06, 9.137493E+06, 9.487990E+06, 9.850861E+06, 1.022650E+07, 
   1.061532E+07, 1.101774E+07, 1.143420E+07, 1.186513E+07, 1.231100E+07, 1.277227E+07, 
   1.324943E+07, 1.374297E+07, 1.425342E+07, 1.478128E+07, 1.532711E+07, 1.589145E+07, 
   1.647488E+07, 1.707798E+07, 1.770136E+07, 1.834564E+07, 1.901144E+07, 1.969942E+07, 
   2.041026E+07, 2.114463E+07, 2.190325E+07, 2.268685E+07, 2.349616E+07, 2.433196E+07, 
   2.519502E+07, 2.608616E+07, 2.700621E+07, 2.795600E+07, 2.893642E+07, 2.994835E+07, 
   3.099272E+07, 3.207047E+07, 3.318256E+07, 3.432998E+07, 3.551375E+07, 3.673491E+07, 
   3.799454E+07, 3.929372E+07, 4.063358E+07, 4.201527E+07, 4.343998E+07, 4.490892E+07, 
   4.642332E+07, 4.798447E+07, 4.959366E+07, 5.125224E+07, 5.296157E+07, 5.472306E+07, 
   5.653814E+07, 5.840829E+07, 6.033503E+07, 6.231989E+07, 6.436446E+07, 6.647037E+07, 
   6.863928E+07, 7.087289E+07, 7.317294E+07, 7.554122E+07, 7.797956E+07, 8.048983E+07, 
   8.307394E+07, 8.573387E+07, 8.847160E+07, 9.128921E+07, 9.418879E+07, 9.717250E+07, 
   1.002425E+08, 1.034012E+08, 1.066507E+08, 1.099934E+08, 1.134319E+08, 1.169685E+08, 
   1.206057E+08, 1.243462E+08, 1.281926E+08, 1.321476E+08, 1.362139E+08, 1.403944E+08, 
   1.446921E+08, 1.491097E+08, 1.536505E+08, 1.583174E+08, 1.631136E+08, 1.680423E+08, 
   1.731069E+08, 1.783108E+08, 1.836573E+08, 1.891500E+08, 1.947925E+08, 2.005885E+08, 
   2.065418E+08, 2.126561E+08, 2.189355E+08, 2.253839E+08, 2.320055E+08, 2.388044E+08, 
   2.457849E+08, 2.529514E+08, 2.603083E+08, 2.678602E+08, 2.756118E+08, 2.835677E+08, 
   2.917330E+08, 3.001124E+08, 3.087112E+08, 3.175344E+08, 3.265873E+08, 3.358753E+08, 
   3.454040E+08, 3.551788E+08, 3.652057E+08, 3.754903E+08, 3.860387E+08, 3.968569E+08, 
   4.079512E+08, 4.193279E+08, 4.309935E+08, 4.429546E+08, 4.552179E+08, 4.677902E+08, 
   4.806787E+08, 4.938905E+08, 5.074327E+08, 5.213130E+08, 5.355389E+08, 5.501181E+08, 
   5.650585E+08, 5.803681E+08, 5.960553E+08, 6.121283E+08, 6.285957E+08, 6.454662E+08, 
   6.627487E+08, 6.804522E+08, 6.985859E+08, 7.171593E+08, 7.361820E+08, 7.556636E+08, 
   7.756142E+08, 7.960440E+08, 8.169632E+08, 8.383825E+08, 8.603125E+08, 8.827642E+08, 
   9.057489E+08, 9.292778E+08, 9.533625E+08, 9.780148E+08, 1.003247E+09, 1.029071E+09, 
   1.055499E+09, 1.082545E+09, 1.110220E+09, 1.138539E+09, 1.167515E+09, 1.197161E+09, 
   1.227491E+09, 1.258520E+09, 1.290262E+09, 1.322732E+09, 1.355944E+09, 1.389914E+09, 
   1.424658E+09, 1.460191E+09, 1.496530E+09, 1.533691E+09, 1.571690E+09, 1.610545E+09, 
   1.650273E+09, 1.690891E+09, 1.732418E+09, 1.774872E+09, 1.818271E+09, 1.862634E+09, 
   1.907980E+09, 1.954330E+09, 2.001703E+09, 2.050119E+09, 2.099599E+09, 2.150164E+09, 
   2.201835E+09, 2.254634E+09, 2.308583E+09, 2.363705E+09, 2.420023E+09, 2.477559E+09, 
   2.536338E+09, 2.596384E+09, 2.657720E+09, 2.720373E+09, 2.784368E+09, 2.849729E+09, 
   2.916484E+09, 2.984660E+09, 3.054282E+09, 3.125380E+09, 3.197980E+09, 3.272112E+09, 
   3.347805E+09, 3.425087E+09, 3.503990E+09, 3.584542E+09, 3.666777E+09, 3.750724E+09, 
   3.836417E+09, 3.923887E+09, 4.013167E+09, 4.104292E+09, 4.197296E+09, 4.292213E+09, 
   4.389079E+09, 4.487929E+09, 4.588801E+09, 4.691730E+09, 4.796756E+09, 4.903916E+09, 
   5.013249E+09, 5.124794E+09, 5.238593E+09, 5.354686E+09, 5.473113E+09, 5.593918E+09, 
   5.717144E+09, 5.842833E+09, 5.971030E+09, 6.101780E+09, 6.235129E+09, 6.371123E+09, 
   6.509808E+09, 6.651234E+09, 6.795448E+09, 6.942500E+09, 7.092439E+09, 7.245318E+09, 
   7.401186E+09, 7.560098E+09, 7.722107E+09, 7.887265E+09, 8.055629E+09, 8.227255E+09, 
   8.402198E+09, 8.580517E+09, 8.762270E+09, 8.947516E+09, 9.136316E+09, 9.328730E+09, 
   9.524821E+09, 9.724652E+09, 9.928287E+09, 1.013579E+10, 1.034723E+10, 1.056267E+10, 
   1.078218E+10, 1.100583E+10, 1.123369E+10, 1.146582E+10, 1.170231E+10, 1.194323E+10, 
   1.218864E+10, 1.243863E+10, 1.269327E+10, 1.295264E+10, 1.321682E+10, 1.348588E+10, 
   1.375992E+10, 1.403900E+10, 1.432322E+10, 1.461266E+10, 1.490740E+10, 1.520754E+10, 
   1.551316E+10, 1.582435E+10, 1.614120E+10, 1.646381E+10, 1.679226E+10, 1.712666E+10, 
   1.746710E+10, 1.781367E+10, 1.816649E+10, 1.852564E+10, 1.889123E+10, 1.926336E+10, 
   1.964214E+10, 2.002768E+10, 2.042008E+10, 2.081946E+10, 2.122592E+10, 2.163957E+10, 
   2.206054E+10, 2.248893E+10, 2.292486E+10, 2.336846E+10, 2.381985E+10, 2.427914E+10, 
   2.474646E+10, 2.522194E+10, 2.570571E+10, 2.619789E+10, 2.669861E+10, 2.720801E+10, 
   2.772623E+10, 2.825340E+10, 2.878965E+10, 2.933514E+10, 2.989000E+10, 3.045437E+10, 
   3.102841E+10, 3.161226E+10, 3.220606E+10, 3.280999E+10, 3.342418E+10, 3.404880E+10, 
   3.468400E+10, 3.532995E+10, 3.598680E+10, 3.665473E+10, 3.733390E+10, 3.802447E+10, 
   3.872663E+10, 3.944055E+10, 4.016639E+10, 4.090435E+10, 4.165460E+10, 4.241733E+10, 
   4.319272E+10, 4.398097E+10, 4.478225E+10, 4.559677E+10, 4.642472E+10, 4.726630E+10, 
   4.812171E+10, 4.899116E+10, 4.987485E+10, 5.077299E+10, 5.168580E+10, 5.261348E+10, 
   5.355625E+10, 5.451434E+10, 5.548797E+10, 5.647737E+10, 5.748276E+10, 5.850437E+10, 
   5.954244E+10, 6.059721E+10, 6.166892E+10, 6.275781E+10, 6.386413E+10, 6.498814E+10, 
   6.613007E+10, 6.729019E+10, 6.846877E+10, 6.966605E+10, 7.088232E+10, 7.211783E+10, 
   7.337287E+10, 7.464770E+10, 7.594261E+10, 7.725789E+10, 7.859381E+10, 7.995068E+10, 
   8.132878E+10, 8.272841E+10, 8.414987E+10, 
])
# ============================ M = 41, I = 1 ============================
M = 41
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.315148E+01, 4.188337E+02, 1.079820E+03, 1.938155E+03, 2.965718E+03, 4.137253E+03, 
   5.436115E+03, 6.853217E+03, 8.385537E+03, 1.003483E+04, 1.180643E+04, 1.370813E+04, 
   1.574939E+04, 1.794074E+04, 2.029339E+04, 2.281913E+04, 2.553020E+04, 2.843938E+04, 
   3.155996E+04, 3.490593E+04, 3.849199E+04, 4.233373E+04, 4.644764E+04, 5.085129E+04, 
   5.556339E+04, 6.060385E+04, 6.599395E+04, 7.175635E+04, 7.791521E+04, 8.449629E+04, 
   9.152703E+04, 9.903665E+04, 1.070562E+05, 1.156188E+05, 1.247596E+05, 1.345159E+05, 
   1.449272E+05, 1.560359E+05, 1.678863E+05, 1.805260E+05, 1.940049E+05, 2.083765E+05, 
   2.236968E+05, 2.400256E+05, 2.574261E+05, 2.759651E+05, 2.957132E+05, 3.167454E+05, 
   3.391407E+05, 3.629827E+05, 3.883600E+05, 4.153658E+05, 4.440992E+05, 4.746642E+05, 
   5.071712E+05, 5.417365E+05, 5.784829E+05, 6.175400E+05, 6.590448E+05, 7.031416E+05, 
   7.499827E+05, 7.997288E+05, 8.525491E+05, 9.086224E+05, 9.681369E+05, 1.031291E+06, 
   1.098294E+06, 1.169365E+06, 1.244737E+06, 1.324654E+06, 1.409374E+06, 1.499166E+06, 
   1.594316E+06, 1.695123E+06, 1.801903E+06, 1.914988E+06, 2.034727E+06, 2.161486E+06, 
   2.295651E+06, 2.437627E+06, 2.587841E+06, 2.746740E+06, 2.914794E+06, 3.092497E+06, 
   3.280367E+06, 3.478950E+06, 3.688816E+06, 3.910565E+06, 4.144827E+06, 4.392262E+06, 
   4.653564E+06, 4.929458E+06, 5.220707E+06, 5.528108E+06, 5.852501E+06, 6.194762E+06, 
   6.555810E+06, 6.936611E+06, 7.338171E+06, 7.761550E+06, 8.207853E+06, 8.678240E+06, 
   9.173924E+06, 9.696175E+06, 1.024632E+07, 1.082575E+07, 1.143593E+07, 1.207836E+07, 
   1.275465E+07, 1.346646E+07, 1.421552E+07, 1.500366E+07, 1.583278E+07, 1.670485E+07, 
   1.762196E+07, 1.858627E+07, 1.960005E+07, 2.066566E+07, 2.178556E+07, 2.296235E+07, 
   2.419870E+07, 2.549742E+07, 2.686144E+07, 2.829383E+07, 2.979776E+07, 3.137657E+07, 
   3.303372E+07, 3.477283E+07, 3.659768E+07, 3.851219E+07, 4.052046E+07, 4.262676E+07, 
   4.483554E+07, 4.715145E+07, 4.957930E+07, 5.212413E+07, 5.479117E+07, 5.758589E+07, 
   6.051396E+07, 6.358130E+07, 6.679405E+07, 7.015863E+07, 7.368170E+07, 7.737018E+07, 
   8.123130E+07, 8.527256E+07, 8.950176E+07, 9.392702E+07, 9.855678E+07, 1.033998E+08, 
   1.084652E+08, 1.137625E+08, 1.193014E+08, 1.250923E+08, 1.311458E+08, 1.374729E+08, 
   1.440852E+08, 1.509944E+08, 1.582130E+08, 1.657538E+08, 1.736302E+08, 1.818560E+08, 
   1.904456E+08, 1.994139E+08, 2.087764E+08, 2.185491E+08, 2.287488E+08, 2.393926E+08, 
   2.504985E+08, 2.620851E+08, 2.741717E+08, 2.867783E+08, 2.999256E+08, 3.136350E+08, 
   3.279289E+08, 3.428303E+08, 3.583632E+08, 3.745523E+08, 3.914232E+08, 4.090026E+08, 
   4.273181E+08, 4.463981E+08, 4.662723E+08, 4.869711E+08, 5.085263E+08, 5.309707E+08, 
   5.543382E+08, 5.786640E+08, 6.039844E+08, 6.303371E+08, 6.577610E+08, 6.862964E+08, 
   7.159850E+08, 7.468698E+08, 7.789955E+08, 8.124083E+08, 8.471557E+08, 8.832871E+08, 
   9.208535E+08, 9.599076E+08, 1.000504E+09, 1.042699E+09, 1.086550E+09, 1.132119E+09, 
   1.179466E+09, 1.228656E+09, 1.279757E+09, 1.332835E+09, 1.387962E+09, 1.445211E+09, 
   1.504657E+09, 1.566379E+09, 1.630457E+09, 1.696974E+09, 1.766015E+09, 1.837670E+09, 
   1.912030E+09, 1.989189E+09, 2.069244E+09, 2.152297E+09, 2.238450E+09, 2.327810E+09, 
   2.420489E+09, 2.516599E+09, 2.616258E+09, 2.719587E+09, 2.826711E+09, 2.937759E+09, 
   3.052864E+09, 3.172161E+09, 3.295793E+09, 3.423905E+09, 3.556646E+09, 3.694172E+09, 
   3.836641E+09, 3.984218E+09, 4.137071E+09, 4.295375E+09, 4.459309E+09, 4.629058E+09, 
   4.804812E+09, 4.986768E+09, 5.175128E+09, 5.370100E+09, 5.571897E+09, 5.780741E+09, 
   5.996859E+09, 6.220484E+09, 6.451857E+09, 6.691226E+09, 6.938845E+09, 7.194978E+09, 
   7.459895E+09, 7.733873E+09, 8.017199E+09, 8.310166E+09, 8.613078E+09, 8.926245E+09, 
   9.249989E+09, 9.584639E+09, 9.930534E+09, 1.028802E+10, 1.065746E+10, 1.103922E+10, 
   1.143368E+10, 1.184123E+10, 1.226227E+10, 1.269722E+10, 1.314648E+10, 1.361051E+10, 
   1.408975E+10, 1.458465E+10, 1.509570E+10, 1.562337E+10, 1.616816E+10, 1.673059E+10, 
   1.731119E+10, 1.791048E+10, 1.852904E+10, 1.916744E+10, 1.982624E+10, 2.050607E+10, 
   2.120754E+10, 2.193128E+10, 2.267794E+10, 2.344820E+10, 2.424274E+10, 2.506227E+10, 
   2.590752E+10, 2.677922E+10, 2.767814E+10, 2.860506E+10, 2.956079E+10, 3.054615E+10, 
   3.156200E+10, 3.260919E+10, 3.368863E+10, 3.480122E+10, 3.594791E+10, 3.712966E+10, 
   3.834746E+10, 3.960231E+10, 4.089526E+10, 4.222739E+10, 4.359977E+10, 4.501353E+10, 
   4.646982E+10, 4.796983E+10, 4.951475E+10, 5.110584E+10, 5.274435E+10, 5.443160E+10, 
   5.616892E+10, 5.795768E+10, 5.979928E+10, 6.169517E+10, 6.364681E+10, 6.565571E+10, 
   6.772344E+10, 6.985156E+10, 7.204171E+10, 7.429556E+10, 7.661481E+10, 7.900121E+10, 
   8.145655E+10, 8.398267E+10, 8.658144E+10, 8.925480E+10, 9.200471E+10, 9.483320E+10, 
   9.774232E+10, 1.007342E+11, 1.038110E+11, 1.069750E+11, 1.102284E+11, 1.135736E+11, 
   1.170128E+11, 1.205487E+11, 1.241837E+11, 1.279203E+11, 1.317611E+11, 1.357089E+11, 
   1.397663E+11, 1.439363E+11, 1.482216E+11, 1.526251E+11, 1.571500E+11, 1.617993E+11, 
   1.665761E+11, 1.714836E+11, 1.765253E+11, 1.817043E+11, 1.870242E+11, 1.924885E+11, 
   1.981008E+11, 2.038649E+11, 2.097844E+11, 2.158632E+11, 2.221054E+11, 2.285149E+11, 
   2.350959E+11, 2.418525E+11, 2.487892E+11, 2.559103E+11, 2.632204E+11, 2.707240E+11, 
   2.784260E+11, 2.863310E+11, 2.944441E+11, 3.027703E+11, 3.113147E+11, 3.200826E+11, 
   3.290794E+11, 3.383107E+11, 3.477819E+11, 3.574989E+11, 3.674676E+11, 3.776939E+11, 
   3.881840E+11, 3.989442E+11, 4.099807E+11, 4.213003E+11, 4.329096E+11, 4.448153E+11, 
   4.570245E+11, 4.695443E+11, 4.823819E+11, 4.955449E+11, 5.090407E+11, 5.228772E+11, 
   5.370622E+11, 5.516039E+11, 5.665105E+11, 5.817904E+11, 5.974522E+11, 6.135048E+11, 
   6.299571E+11, 6.468182E+11, 6.640976E+11, 6.818047E+11, 6.999494E+11, 7.185415E+11, 
   7.375913E+11, 7.571091E+11, 7.771055E+11, 7.975913E+11, 8.185775E+11, 8.400754E+11, 
   8.620965E+11, 8.846525E+11, 9.077553E+11, 9.314172E+11, 9.556506E+11, 9.804683E+11, 
   1.005883E+12, 1.031908E+12, 1.058558E+12, 1.085844E+12, 1.113783E+12, 1.142388E+12, 
   1.171673E+12, 1.201654E+12, 1.232346E+12, 1.263764E+12, 1.295924E+12, 1.328843E+12, 
   1.362536E+12, 1.397021E+12, 1.432315E+12, 1.468434E+12, 1.505398E+12, 1.543224E+12, 
   1.581930E+12, 1.621535E+12, 1.662060E+12, 1.703522E+12, 1.745943E+12, 1.789343E+12, 
   1.833742E+12, 1.879162E+12, 1.925624E+12, 1.973150E+12, 2.021763E+12, 2.071485E+12, 
   2.122341E+12, 2.174353E+12, 2.227546E+12, 2.281945E+12, 2.337574E+12, 2.394459E+12, 
   2.452628E+12, 2.512105E+12, 2.572918E+12, 2.635095E+12, 2.698664E+12, 2.763654E+12, 
   2.830093E+12, 2.898013E+12, 2.967442E+12, 3.038412E+12, 3.110954E+12, 3.185101E+12, 
   3.260885E+12, 3.338339E+12, 3.417498E+12, 3.498395E+12, 3.581066E+12, 3.665547E+12, 
   3.751874E+12, 3.840084E+12, 3.930216E+12, 4.022307E+12, 4.116396E+12, 4.212525E+12, 
   4.310733E+12, 4.411062E+12, 4.513554E+12, 4.618251E+12, 4.725199E+12, 4.834440E+12, 
   4.946021E+12, 5.059987E+12, 5.176385E+12, 5.295264E+12, 5.416672E+12, 5.540658E+12, 
   5.667272E+12, 5.796567E+12, 5.928594E+12, 6.063406E+12, 6.201058E+12, 6.341604E+12, 
   6.485100E+12, 6.631605E+12, 6.781174E+12, 6.933869E+12, 7.089748E+12, 7.248872E+12, 
   7.411305E+12, 7.577108E+12, 7.746348E+12, 
])
# ============================ M = 41, I = 2 ============================
#M = 41
#I = 2
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   4.756503E+01, 8.636953E+02, 2.228139E+03, 4.000548E+03, 6.122352E+03, 8.541298E+03, 
#   1.122310E+04, 1.414912E+04, 1.731326E+04, 2.071932E+04, 2.437842E+04, 2.830685E+04, 
#   3.252426E+04, 3.705256E+04, 4.191510E+04, 4.713645E+04, 5.274217E+04, 5.875893E+04, 
#   6.521462E+04, 7.213853E+04, 7.956156E+04, 8.751641E+04, 9.603781E+04, 1.051627E+05, 
#   1.149304E+05, 1.253830E+05, 1.365651E+05, 1.485245E+05, 1.613121E+05, 1.749821E+05, 
#   1.895924E+05, 2.052045E+05, 2.218838E+05, 2.397001E+05, 2.587273E+05, 2.790443E+05, 
#   3.007345E+05, 3.238866E+05, 3.485948E+05, 3.749588E+05, 4.030846E+05, 4.330843E+05, 
#   4.650768E+05, 4.991880E+05, 5.355511E+05, 5.743074E+05, 6.156062E+05, 6.596055E+05, 
#   7.064725E+05, 7.563841E+05, 8.095273E+05, 8.660995E+05, 9.263098E+05, 9.903787E+05, 
#   1.058540E+06, 1.131038E+06, 1.208136E+06, 1.290106E+06, 1.377238E+06, 1.469838E+06, 
#   1.568230E+06, 1.672754E+06, 1.783767E+06, 1.901650E+06, 2.026801E+06, 2.159641E+06, 
#   2.300614E+06, 2.450186E+06, 2.608850E+06, 2.777125E+06, 2.955555E+06, 3.144717E+06, 
#   3.345214E+06, 3.557685E+06, 3.782798E+06, 4.021259E+06, 4.273809E+06, 4.541228E+06, 
#   4.824336E+06, 5.123995E+06, 5.441110E+06, 5.776634E+06, 6.131567E+06, 6.506960E+06, 
#   6.903915E+06, 7.323593E+06, 7.767209E+06, 8.236041E+06, 8.731430E+06, 9.254781E+06, 
#   9.807570E+06, 1.039135E+07, 1.100773E+07, 1.165843E+07, 1.234522E+07, 1.306998E+07, 
#   1.383467E+07, 1.464134E+07, 1.549215E+07, 1.638934E+07, 1.733529E+07, 1.833246E+07, 
#   1.938344E+07, 2.049094E+07, 2.165780E+07, 2.288699E+07, 2.418161E+07, 2.554491E+07, 
#   2.698029E+07, 2.849130E+07, 3.008166E+07, 3.175524E+07, 3.351612E+07, 3.536853E+07, 
#   3.731691E+07, 3.936589E+07, 4.152031E+07, 4.378522E+07, 4.616589E+07, 4.866785E+07, 
#   5.129684E+07, 5.405887E+07, 5.696020E+07, 6.000737E+07, 6.320721E+07, 6.656684E+07, 
#   7.009367E+07, 7.379544E+07, 7.768024E+07, 8.175648E+07, 8.603292E+07, 9.051872E+07, 
#   9.522340E+07, 1.001569E+08, 1.053295E+08, 1.107521E+08, 1.164358E+08, 1.223924E+08, 
#   1.286340E+08, 1.351733E+08, 1.420234E+08, 1.491981E+08, 1.567118E+08, 1.645791E+08, 
#   1.728157E+08, 1.814376E+08, 1.904615E+08, 1.999048E+08, 2.097857E+08, 2.201229E+08, 
#   2.309360E+08, 2.422454E+08, 2.540721E+08, 2.664381E+08, 2.793663E+08, 2.928804E+08, 
#   3.070049E+08, 3.217654E+08, 3.371886E+08, 3.533019E+08, 3.701341E+08, 3.877148E+08, 
#   4.060750E+08, 4.252466E+08, 4.452629E+08, 4.661584E+08, 4.879688E+08, 5.107314E+08, 
#   5.344846E+08, 5.592683E+08, 5.851240E+08, 6.120947E+08, 6.402248E+08, 6.695606E+08, 
#   7.001500E+08, 7.320424E+08, 7.652893E+08, 7.999441E+08, 8.360618E+08, 8.736998E+08, 
#   9.129171E+08, 9.537752E+08, 9.963377E+08, 1.040670E+09, 1.086841E+09, 1.134921E+09, 
#   1.184982E+09, 1.237101E+09, 1.291356E+09, 1.347827E+09, 1.406599E+09, 1.467758E+09, 
#   1.531394E+09, 1.597600E+09, 1.666471E+09, 1.738107E+09, 1.812611E+09, 1.890089E+09, 
#   1.970650E+09, 2.054408E+09, 2.141481E+09, 2.231989E+09, 2.326058E+09, 2.423818E+09, 
#   2.525402E+09, 2.630949E+09, 2.740601E+09, 2.854507E+09, 2.972819E+09, 3.095693E+09, 
#   3.223294E+09, 3.355789E+09, 3.493351E+09, 3.636159E+09, 3.784399E+09, 3.938261E+09, 
#   4.097943E+09, 4.263647E+09, 4.435584E+09, 4.613970E+09, 4.799028E+09, 4.990990E+09, 
#   5.190093E+09, 5.396582E+09, 5.610712E+09, 5.832741E+09, 6.062941E+09, 6.301588E+09, 
#   6.548970E+09, 6.805380E+09, 7.071124E+09, 7.346515E+09, 7.631877E+09, 7.927543E+09, 
#   8.233856E+09, 8.551171E+09, 8.879852E+09, 9.220276E+09, 9.572828E+09, 9.937910E+09, 
#   1.031593E+10, 1.070732E+10, 1.111250E+10, 1.153193E+10, 1.196607E+10, 1.241540E+10, 
#   1.288041E+10, 1.336159E+10, 1.385948E+10, 1.437461E+10, 1.490752E+10, 1.545879E+10, 
#   1.602900E+10, 1.661874E+10, 1.722864E+10, 1.785933E+10, 1.851147E+10, 1.918572E+10, 
#   1.988279E+10, 2.060338E+10, 2.134822E+10, 2.211807E+10, 2.291370E+10, 2.373591E+10, 
#   2.458552E+10, 2.546338E+10, 2.637033E+10, 2.730729E+10, 2.827515E+10, 2.927487E+10, 
#   3.030741E+10, 3.137376E+10, 3.247495E+10, 3.361202E+10, 3.478605E+10, 3.599816E+10, 
#   3.724947E+10, 3.854116E+10, 3.987444E+10, 4.125053E+10, 4.267070E+10, 4.413626E+10, 
#   4.564854E+10, 4.720891E+10, 4.881880E+10, 5.047964E+10, 5.219292E+10, 5.396017E+10, 
#   5.578296E+10, 5.766289E+10, 5.960162E+10, 6.160084E+10, 6.366230E+10, 6.578778E+10, 
#   6.797911E+10, 7.023817E+10, 7.256690E+10, 7.496727E+10, 7.744131E+10, 7.999112E+10, 
#   8.261882E+10, 8.532662E+10, 8.811676E+10, 9.099155E+10, 9.395336E+10, 9.700462E+10, 
#   1.001478E+11, 1.033855E+11, 1.067203E+11, 1.101549E+11, 1.136920E+11, 1.173345E+11, 
#   1.210852E+11, 1.249472E+11, 1.289234E+11, 1.330170E+11, 1.372312E+11, 1.415693E+11, 
#   1.460345E+11, 1.506304E+11, 1.553604E+11, 1.602282E+11, 1.652375E+11, 1.703920E+11, 
#   1.756956E+11, 1.811524E+11, 1.867663E+11, 1.925416E+11, 1.984826E+11, 2.045935E+11, 
#   2.108789E+11, 2.173434E+11, 2.239917E+11, 2.308286E+11, 2.378590E+11, 2.450880E+11, 
#   2.525207E+11, 2.601624E+11, 2.680186E+11, 2.760948E+11, 2.843966E+11, 2.929299E+11, 
#   3.017006E+11, 3.107148E+11, 3.199788E+11, 3.294989E+11, 3.392816E+11, 3.493335E+11, 
#   3.596617E+11, 3.702729E+11, 3.811744E+11, 3.923735E+11, 4.038777E+11, 4.156946E+11, 
#   4.278320E+11, 4.402980E+11, 4.531007E+11, 4.662486E+11, 4.797502E+11, 4.936142E+11, 
#   5.078497E+11, 5.224658E+11, 5.374718E+11, 5.528774E+11, 5.686924E+11, 5.849267E+11, 
#   6.015906E+11, 6.186947E+11, 6.362495E+11, 6.542660E+11, 6.727555E+11, 6.917293E+11, 
#   7.111991E+11, 7.311769E+11, 7.516749E+11, 7.727056E+11, 7.942816E+11, 8.164161E+11, 
#   8.391224E+11, 8.624140E+11, 8.863048E+11, 9.108091E+11, 9.359413E+11, 9.617163E+11, 
#   9.881491E+11, 1.015255E+12, 1.043051E+12, 1.071552E+12, 1.100774E+12, 1.130735E+12, 
#   1.161452E+12, 1.192942E+12, 1.225224E+12, 1.258315E+12, 1.292234E+12, 1.327001E+12, 
#   1.362635E+12, 1.399156E+12, 1.436583E+12, 1.474939E+12, 1.514243E+12, 1.554519E+12, 
#   1.595787E+12, 1.638070E+12, 1.681391E+12, 1.725774E+12, 1.771243E+12, 1.817822E+12, 
#   1.865537E+12, 1.914411E+12, 1.964473E+12, 2.015747E+12, 2.068262E+12, 2.122044E+12, 
#   2.177122E+12, 2.233526E+12, 2.291283E+12, 2.350424E+12, 2.410979E+12, 2.472981E+12, 
#   2.536459E+12, 2.601448E+12, 2.667979E+12, 2.736088E+12, 2.805807E+12, 2.877172E+12, 
#   2.950219E+12, 3.024985E+12, 3.101506E+12, 3.179821E+12, 3.259968E+12, 3.341987E+12, 
#   3.425918E+12, 3.511801E+12, 3.599680E+12, 3.689596E+12, 3.781594E+12, 3.875716E+12, 
#   3.972009E+12, 4.070519E+12, 4.171293E+12, 4.274378E+12, 4.379823E+12, 4.487678E+12, 
#   4.597994E+12, 4.710823E+12, 4.826216E+12, 4.944229E+12, 5.064915E+12, 5.188330E+12, 
#   5.314532E+12, 5.443577E+12, 5.575526E+12, 5.710437E+12, 5.848373E+12, 5.989396E+12, 
#   6.133569E+12, 6.280958E+12, 6.431628E+12, 6.585646E+12, 6.743080E+12, 6.904002E+12, 
#   7.068482E+12, 7.236591E+12, 7.408405E+12, 7.583998E+12, 7.763446E+12, 7.946828E+12, 
#   8.134223E+12, 8.325712E+12, 8.521376E+12, 8.721301E+12, 8.925571E+12, 9.134273E+12, 
#   9.347495E+12, 9.565329E+12, 9.787865E+12, 1.001520E+13, 1.024742E+13, 1.048463E+13, 
#   1.072693E+13, 1.097442E+13, 1.122719E+13, 1.148536E+13, 1.174903E+13, 1.201830E+13, 
#   1.229330E+13, 1.257412E+13, 1.286088E+13, 1.315370E+13, 1.345270E+13, 1.375799E+13, 
#   1.406970E+13, 1.438795E+13, 1.471287E+13, 1.504459E+13, 1.538324E+13, 1.572894E+13, 
#   1.608185E+13, 1.644208E+13, 1.680980E+13, 
#])
# ============================ M = 41, I = 3 ============================
#M = 41
#I = 3
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   4.640863E+01, 8.395187E+02, 2.165321E+03, 3.887494E+03, 5.949141E+03, 8.299582E+03, 
#   1.090587E+04, 1.375072E+04, 1.682938E+04, 2.014691E+04, 2.371546E+04, 2.755208E+04, 
#   3.167695E+04, 3.611225E+04, 4.088151E+04, 4.600925E+04, 5.152098E+04, 5.744318E+04, 
#   6.380350E+04, 7.063096E+04, 7.795612E+04, 8.581133E+04, 9.423091E+04, 1.032514E+05, 
#   1.129116E+05, 1.232531E+05, 1.343201E+05, 1.461598E+05, 1.588225E+05, 1.723620E+05, 
#   1.868353E+05, 2.023033E+05, 2.188310E+05, 2.364873E+05, 2.553454E+05, 2.754834E+05, 
#   2.969840E+05, 3.199350E+05, 3.444298E+05, 3.705672E+05, 3.984522E+05, 4.281959E+05, 
#   4.599161E+05, 4.937378E+05, 5.297930E+05, 5.682218E+05, 6.091721E+05, 6.528008E+05, 
#   6.992737E+05, 7.487660E+05, 8.014633E+05, 8.575615E+05, 9.172679E+05, 9.808012E+05, 
#   1.048393E+06, 1.120287E+06, 1.196742E+06, 1.278030E+06, 1.364438E+06, 1.456270E+06, 
#   1.553846E+06, 1.657504E+06, 1.767599E+06, 1.884508E+06, 2.008627E+06, 2.140373E+06, 
#   2.280186E+06, 2.428529E+06, 2.585892E+06, 2.752788E+06, 2.929758E+06, 3.117375E+06, 
#   3.316237E+06, 3.526977E+06, 3.750259E+06, 3.986783E+06, 4.237285E+06, 4.502539E+06, 
#   4.783358E+06, 5.080597E+06, 5.395156E+06, 5.727979E+06, 6.080059E+06, 6.452439E+06, 
#   6.846214E+06, 7.262533E+06, 7.702605E+06, 8.167696E+06, 8.659138E+06, 9.178326E+06, 
#   9.726724E+06, 1.030587E+07, 1.091737E+07, 1.156292E+07, 1.224429E+07, 1.296334E+07, 
#   1.372200E+07, 1.452233E+07, 1.536645E+07, 1.625661E+07, 1.719514E+07, 1.818451E+07, 
#   1.922728E+07, 2.032615E+07, 2.148392E+07, 2.270354E+07, 2.398810E+07, 2.534082E+07, 
#   2.676507E+07, 2.826438E+07, 2.984244E+07, 3.150310E+07, 3.325040E+07, 3.508854E+07, 
#   3.702193E+07, 3.905516E+07, 4.119303E+07, 4.344058E+07, 4.580302E+07, 4.828584E+07, 
#   5.089474E+07, 5.363568E+07, 5.651489E+07, 5.953885E+07, 6.271435E+07, 6.604844E+07, 
#   6.954849E+07, 7.322219E+07, 7.707756E+07, 8.112294E+07, 8.536705E+07, 8.981896E+07, 
#   9.448814E+07, 9.938445E+07, 1.045181E+08, 1.098999E+08, 1.155409E+08, 1.214528E+08, 
#   1.276475E+08, 1.341378E+08, 1.409366E+08, 1.480577E+08, 1.555151E+08, 1.633237E+08, 
#   1.714988E+08, 1.800564E+08, 1.890131E+08, 1.983862E+08, 2.081936E+08, 2.184540E+08, 
#   2.291868E+08, 2.404123E+08, 2.521513E+08, 2.644258E+08, 2.772583E+08, 2.906724E+08, 
#   3.046926E+08, 3.193441E+08, 3.346535E+08, 3.506481E+08, 3.673563E+08, 3.848076E+08, 
#   4.030327E+08, 4.220634E+08, 4.419326E+08, 4.626748E+08, 4.843252E+08, 5.069210E+08, 
#   5.305002E+08, 5.551025E+08, 5.807691E+08, 6.075426E+08, 6.354673E+08, 6.645889E+08, 
#   6.949551E+08, 7.266150E+08, 7.596197E+08, 7.940221E+08, 8.298770E+08, 8.672412E+08, 
#   9.061735E+08, 9.467349E+08, 9.889884E+08, 1.032999E+09, 1.078835E+09, 1.126567E+09, 
#   1.176266E+09, 1.228007E+09, 1.281870E+09, 1.337933E+09, 1.396280E+09, 1.456997E+09, 
#   1.520174E+09, 1.585902E+09, 1.654276E+09, 1.725396E+09, 1.799364E+09, 1.876283E+09, 
#   1.956265E+09, 2.039421E+09, 2.125867E+09, 2.215725E+09, 2.309119E+09, 2.406177E+09, 
#   2.507032E+09, 2.611822E+09, 2.720689E+09, 2.833779E+09, 2.951243E+09, 3.073238E+09, 
#   3.199926E+09, 3.331473E+09, 3.468052E+09, 3.609840E+09, 3.757022E+09, 3.909786E+09, 
#   4.068328E+09, 4.232850E+09, 4.403562E+09, 4.580676E+09, 4.764417E+09, 4.955012E+09, 
#   5.152698E+09, 5.357718E+09, 5.570325E+09, 5.790777E+09, 6.019341E+09, 6.256293E+09, 
#   6.501918E+09, 6.756509E+09, 7.020368E+09, 7.293806E+09, 7.577145E+09, 7.870716E+09, 
#   8.174859E+09, 8.489927E+09, 8.816282E+09, 9.154297E+09, 9.504356E+09, 9.866856E+09, 
#   1.024221E+10, 1.063082E+10, 1.103315E+10, 1.144962E+10, 1.188069E+10, 1.232685E+10, 
#   1.278858E+10, 1.326637E+10, 1.376075E+10, 1.427225E+10, 1.480141E+10, 1.534880E+10, 
#   1.591499E+10, 1.650058E+10, 1.710619E+10, 1.773244E+10, 1.838000E+10, 1.904951E+10, 
#   1.974168E+10, 2.045721E+10, 2.119682E+10, 2.196126E+10, 2.275131E+10, 2.356776E+10, 
#   2.441141E+10, 2.528310E+10, 2.618370E+10, 2.711409E+10, 2.807518E+10, 2.906789E+10, 
#   3.009320E+10, 3.115209E+10, 3.224557E+10, 3.337469E+10, 3.454051E+10, 3.574414E+10, 
#   3.698670E+10, 3.826937E+10, 3.959333E+10, 4.095980E+10, 4.237006E+10, 4.382538E+10, 
#   4.532711E+10, 4.687660E+10, 4.847525E+10, 5.012450E+10, 5.182584E+10, 5.358077E+10, 
#   5.539085E+10, 5.725769E+10, 5.918291E+10, 6.116821E+10, 6.321532E+10, 6.532600E+10, 
#   6.750208E+10, 6.974542E+10, 7.205795E+10, 7.444162E+10, 7.689846E+10, 7.943054E+10, 
#   8.203999E+10, 8.472897E+10, 8.749973E+10, 9.035455E+10, 9.329579E+10, 9.632586E+10, 
#   9.944724E+10, 1.026624E+11, 1.059741E+11, 1.093849E+11, 1.128974E+11, 1.165147E+11, 
#   1.202394E+11, 1.240746E+11, 1.280233E+11, 1.320886E+11, 1.362736E+11, 1.405816E+11, 
#   1.450159E+11, 1.495799E+11, 1.542772E+11, 1.591113E+11, 1.640859E+11, 1.692048E+11, 
#   1.744718E+11, 1.798908E+11, 1.854659E+11, 1.912013E+11, 1.971012E+11, 2.031699E+11, 
#   2.094119E+11, 2.158317E+11, 2.224341E+11, 2.292237E+11, 2.362056E+11, 2.433847E+11, 
#   2.507661E+11, 2.583551E+11, 2.661571E+11, 2.741775E+11, 2.824221E+11, 2.908966E+11, 
#   2.996068E+11, 3.085589E+11, 3.177590E+11, 3.272134E+11, 3.369287E+11, 3.469115E+11, 
#   3.571685E+11, 3.677066E+11, 3.785331E+11, 3.896550E+11, 4.010800E+11, 4.128156E+11, 
#   4.248695E+11, 4.372497E+11, 4.499644E+11, 4.630218E+11, 4.764305E+11, 4.901992E+11, 
#   5.043368E+11, 5.188524E+11, 5.337553E+11, 5.490551E+11, 5.647613E+11, 5.808841E+11, 
#   5.974336E+11, 6.144201E+11, 6.318544E+11, 6.497472E+11, 6.681097E+11, 6.869532E+11, 
#   7.062894E+11, 7.261301E+11, 7.464875E+11, 7.673738E+11, 7.888019E+11, 8.107846E+11, 
#   8.333351E+11, 8.564669E+11, 8.801940E+11, 9.045302E+11, 9.294902E+11, 9.550885E+11, 
#   9.813402E+11, 1.008261E+12, 1.035866E+12, 1.064171E+12, 1.093193E+12, 1.122949E+12, 
#   1.153456E+12, 1.184731E+12, 1.216791E+12, 1.249656E+12, 1.283343E+12, 1.317872E+12, 
#   1.353262E+12, 1.389532E+12, 1.426704E+12, 1.464797E+12, 1.503833E+12, 1.543833E+12, 
#   1.584818E+12, 1.626812E+12, 1.669838E+12, 1.713918E+12, 1.759076E+12, 1.805336E+12, 
#   1.852724E+12, 1.901265E+12, 1.950984E+12, 2.001909E+12, 2.054064E+12, 2.107480E+12, 
#   2.162182E+12, 2.218200E+12, 2.275562E+12, 2.334300E+12, 2.394442E+12, 2.456020E+12, 
#   2.519065E+12, 2.583610E+12, 2.649688E+12, 2.717331E+12, 2.786574E+12, 2.857453E+12, 
#   2.930002E+12, 3.004257E+12, 3.080257E+12, 3.158038E+12, 3.237638E+12, 3.319098E+12, 
#   3.402456E+12, 3.487755E+12, 3.575034E+12, 3.664338E+12, 3.755708E+12, 3.849189E+12, 
#   3.944826E+12, 4.042665E+12, 4.142752E+12, 4.245134E+12, 4.349861E+12, 4.456982E+12, 
#   4.566547E+12, 4.678607E+12, 4.793215E+12, 4.910424E+12, 5.030288E+12, 5.152864E+12, 
#   5.278206E+12, 5.406373E+12, 5.537424E+12, 5.671417E+12, 5.808414E+12, 5.948478E+12, 
#   6.091670E+12, 6.238056E+12, 6.387700E+12, 6.540671E+12, 6.697035E+12, 6.856862E+12, 
#   7.020224E+12, 7.187190E+12, 7.357836E+12, 7.532234E+12, 7.710463E+12, 7.892598E+12, 
#   8.078719E+12, 8.268906E+12, 8.463241E+12, 8.661807E+12, 8.864689E+12, 9.071973E+12, 
#   9.283747E+12, 9.500101E+12, 9.721125E+12, 9.946914E+12, 1.017756E+13, 1.041316E+13, 
#   1.065381E+13, 1.089962E+13, 1.115068E+13, 1.140709E+13, 1.166897E+13, 1.193642E+13, 
#   1.220954E+13, 1.248846E+13, 1.277327E+13, 1.306411E+13, 1.336107E+13, 1.366430E+13, 
#   1.397389E+13, 1.428998E+13, 1.461270E+13, 1.494217E+13, 1.527852E+13, 1.562188E+13, 
#   1.597239E+13, 1.633018E+13, 1.669540E+13, 
#])
# ============================ M = 41, I = 4 ============================
#M = 41
#I = 4
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   9.539986E+01, 1.733030E+03, 4.470930E+03, 8.027465E+03, 1.228512E+04, 1.713923E+04, 
#   2.252173E+04, 2.839688E+04, 3.475486E+04, 4.160600E+04, 4.897544E+04, 5.689833E+04, 
#   6.541639E+04, 7.457551E+04, 8.442441E+04, 9.501401E+04, 1.063973E+05, 1.186293E+05, 
#   1.317678E+05, 1.458732E+05, 1.610094E+05, 1.772441E+05, 1.946489E+05, 2.133003E+05, 
#   2.332798E+05, 2.546741E+05, 2.775759E+05, 3.020842E+05, 3.283042E+05, 3.563484E+05, 
#   3.863367E+05, 4.183965E+05, 4.526638E+05, 4.892830E+05, 5.284079E+05, 5.702017E+05, 
#   6.148381E+05, 6.625011E+05, 7.133863E+05, 7.677013E+05, 8.256660E+05, 8.875137E+05, 
#   9.534916E+05, 1.023862E+06, 1.098901E+06, 1.178904E+06, 1.264182E+06, 1.355063E+06, 
#   1.451896E+06, 1.555048E+06, 1.664911E+06, 1.781896E+06, 1.906437E+06, 2.038996E+06, 
#   2.180059E+06, 2.330140E+06, 2.489781E+06, 2.659555E+06, 2.840067E+06, 3.031956E+06, 
#   3.235896E+06, 3.452598E+06, 3.682812E+06, 3.927329E+06, 4.186983E+06, 4.462654E+06, 
#   4.755270E+06, 5.065806E+06, 5.395292E+06, 5.744813E+06, 6.115511E+06, 6.508589E+06, 
#   6.925313E+06, 7.367016E+06, 7.835100E+06, 8.331043E+06, 8.856395E+06, 9.412792E+06, 
#   1.000195E+07, 1.062568E+07, 1.128587E+07, 1.198452E+07, 1.272373E+07, 1.350570E+07, 
#   1.433274E+07, 1.520729E+07, 1.613189E+07, 1.710923E+07, 1.814212E+07, 1.923350E+07, 
#   2.038648E+07, 2.160431E+07, 2.289038E+07, 2.424828E+07, 2.568176E+07, 2.719473E+07, 
#   2.879132E+07, 3.047585E+07, 3.225283E+07, 3.412700E+07, 3.610333E+07, 3.818701E+07, 
#   4.038348E+07, 4.269843E+07, 4.513784E+07, 4.770794E+07, 5.041525E+07, 5.326662E+07, 
#   5.626919E+07, 5.943043E+07, 6.275815E+07, 6.626054E+07, 6.994613E+07, 7.382385E+07, 
#   7.790304E+07, 8.219343E+07, 8.670523E+07, 9.144906E+07, 9.643604E+07, 1.016778E+08, 
#   1.071864E+08, 1.129745E+08, 1.190553E+08, 1.254426E+08, 1.321508E+08, 1.391949E+08, 
#   1.465904E+08, 1.543538E+08, 1.625020E+08, 1.710527E+08, 1.800245E+08, 1.894367E+08, 
#   1.993093E+08, 2.096632E+08, 2.205203E+08, 2.319034E+08, 2.438360E+08, 2.563428E+08, 
#   2.694495E+08, 2.831829E+08, 2.975706E+08, 3.126419E+08, 3.284266E+08, 3.449563E+08, 
#   3.622636E+08, 3.803824E+08, 3.993480E+08, 4.191972E+08, 4.399683E+08, 4.617008E+08, 
#   4.844361E+08, 5.082172E+08, 5.330887E+08, 5.590969E+08, 5.862900E+08, 6.147182E+08, 
#   6.444334E+08, 6.754896E+08, 7.079431E+08, 7.418520E+08, 7.772769E+08, 8.142806E+08, 
#   8.529283E+08, 8.932878E+08, 9.354293E+08, 9.794256E+08, 1.025353E+09, 1.073289E+09, 
#   1.123315E+09, 1.175517E+09, 1.229981E+09, 1.286798E+09, 1.346063E+09, 1.407873E+09, 
#   1.472330E+09, 1.539538E+09, 1.609606E+09, 1.682647E+09, 1.758778E+09, 1.838120E+09, 
#   1.920797E+09, 2.006940E+09, 2.096683E+09, 2.190166E+09, 2.287533E+09, 2.388933E+09, 
#   2.494520E+09, 2.604455E+09, 2.718904E+09, 2.838037E+09, 2.962033E+09, 3.091074E+09, 
#   3.225351E+09, 3.365060E+09, 3.510405E+09, 3.661595E+09, 3.818849E+09, 3.982390E+09, 
#   4.152453E+09, 4.329276E+09, 4.513109E+09, 4.704208E+09, 4.902839E+09, 5.109276E+09, 
#   5.323804E+09, 5.546715E+09, 5.778311E+09, 6.018907E+09, 6.268824E+09, 6.528397E+09, 
#   6.797971E+09, 7.077901E+09, 7.368556E+09, 7.670315E+09, 7.983569E+09, 8.308724E+09, 
#   8.646198E+09, 8.996421E+09, 9.359839E+09, 9.736910E+09, 1.012811E+10, 1.053392E+10, 
#   1.095486E+10, 1.139144E+10, 1.184420E+10, 1.231369E+10, 1.280048E+10, 1.330516E+10, 
#   1.382835E+10, 1.437066E+10, 1.493274E+10, 1.551525E+10, 1.611889E+10, 1.674436E+10, 
#   1.739239E+10, 1.806374E+10, 1.875916E+10, 1.947947E+10, 2.022548E+10, 2.099804E+10, 
#   2.179803E+10, 2.262633E+10, 2.348389E+10, 2.437164E+10, 2.529058E+10, 2.624171E+10, 
#   2.722607E+10, 2.824474E+10, 2.929882E+10, 3.038945E+10, 3.151778E+10, 3.268504E+10, 
#   3.389245E+10, 3.514130E+10, 3.643288E+10, 3.776856E+10, 3.914972E+10, 4.057779E+10, 
#   4.205424E+10, 4.358058E+10, 4.515838E+10, 4.678922E+10, 4.847475E+10, 5.021667E+10, 
#   5.201671E+10, 5.387667E+10, 5.579838E+10, 5.778374E+10, 5.983469E+10, 6.195322E+10, 
#   6.414140E+10, 6.640134E+10, 6.873519E+10, 7.114520E+10, 7.363366E+10, 7.620292E+10, 
#   7.885540E+10, 8.159358E+10, 8.442003E+10, 8.733736E+10, 9.034827E+10, 9.345553E+10, 
#   9.666197E+10, 9.997052E+10, 1.033842E+11, 1.069060E+11, 1.105392E+11, 1.142870E+11, 
#   1.181527E+11, 1.221398E+11, 1.262517E+11, 1.304921E+11, 1.348646E+11, 1.393731E+11, 
#   1.440215E+11, 1.488137E+11, 1.537539E+11, 1.588462E+11, 1.640951E+11, 1.695048E+11, 
#   1.750801E+11, 1.808255E+11, 1.867458E+11, 1.928460E+11, 1.991310E+11, 2.056060E+11, 
#   2.122764E+11, 2.191476E+11, 2.262250E+11, 2.335145E+11, 2.410220E+11, 2.487533E+11, 
#   2.567146E+11, 2.649123E+11, 2.733529E+11, 2.820429E+11, 2.909892E+11, 3.001986E+11, 
#   3.096784E+11, 3.194359E+11, 3.294786E+11, 3.398140E+11, 3.504502E+11, 3.613952E+11, 
#   3.726572E+11, 3.842446E+11, 3.961662E+11, 4.084309E+11, 4.210477E+11, 4.340259E+11, 
#   4.473751E+11, 4.611051E+11, 4.752258E+11, 4.897476E+11, 5.046808E+11, 5.200363E+11, 
#   5.358250E+11, 5.520583E+11, 5.687475E+11, 5.859046E+11, 6.035417E+11, 6.216710E+11, 
#   6.403052E+11, 6.594574E+11, 6.791407E+11, 6.993687E+11, 7.201554E+11, 7.415149E+11, 
#   7.634618E+11, 7.860109E+11, 8.091775E+11, 8.329771E+11, 8.574257E+11, 8.825396E+11, 
#   9.083355E+11, 9.348304E+11, 9.620417E+11, 9.899874E+11, 1.018686E+12, 1.048155E+12, 
#   1.078415E+12, 1.109484E+12, 1.141384E+12, 1.174134E+12, 1.207754E+12, 1.242267E+12, 
#   1.277695E+12, 1.314059E+12, 1.351382E+12, 1.389688E+12, 1.429000E+12, 1.469344E+12, 
#   1.510743E+12, 1.553223E+12, 1.596810E+12, 1.641532E+12, 1.687414E+12, 1.734485E+12, 
#   1.782772E+12, 1.832306E+12, 1.883116E+12, 1.935231E+12, 1.988683E+12, 2.043504E+12, 
#   2.099725E+12, 2.157380E+12, 2.216502E+12, 2.277126E+12, 2.339286E+12, 2.403019E+12, 
#   2.468361E+12, 2.535350E+12, 2.604024E+12, 2.674422E+12, 2.746583E+12, 2.820549E+12, 
#   2.896361E+12, 2.974062E+12, 3.053694E+12, 3.135302E+12, 3.218931E+12, 3.304627E+12, 
#   3.392438E+12, 3.482411E+12, 3.574595E+12, 3.669040E+12, 3.765798E+12, 3.864921E+12, 
#   3.966461E+12, 4.070473E+12, 4.177013E+12, 4.286137E+12, 4.397902E+12, 4.512368E+12, 
#   4.629595E+12, 4.749644E+12, 4.872577E+12, 4.998459E+12, 5.127354E+12, 5.259328E+12, 
#   5.394451E+12, 5.532790E+12, 5.674417E+12, 5.819402E+12, 5.967820E+12, 6.119746E+12, 
#   6.275255E+12, 6.434425E+12, 6.597336E+12, 6.764069E+12, 6.934707E+12, 7.109332E+12, 
#   7.288032E+12, 7.470894E+12, 7.658007E+12, 7.849462E+12, 8.045351E+12, 8.245770E+12, 
#   8.450815E+12, 8.660584E+12, 8.875177E+12, 9.094696E+12, 9.319246E+12, 9.548932E+12, 
#   9.783863E+12, 1.002415E+13, 1.026990E+13, 1.052124E+13, 1.077827E+13, 1.104112E+13, 
#   1.130991E+13, 1.158477E+13, 1.186581E+13, 1.215316E+13, 1.244697E+13, 1.274735E+13, 
#   1.305445E+13, 1.336841E+13, 1.368936E+13, 1.401745E+13, 1.435282E+13, 1.469562E+13, 
#   1.504602E+13, 1.540415E+13, 1.577018E+13, 1.614426E+13, 1.652656E+13, 1.691726E+13, 
#   1.731651E+13, 1.772448E+13, 1.814136E+13, 1.856733E+13, 1.900256E+13, 1.944724E+13, 
#   1.990156E+13, 2.036571E+13, 2.083990E+13, 2.132430E+13, 2.181914E+13, 2.232462E+13, 
#   2.284094E+13, 2.336833E+13, 2.390699E+13, 2.445716E+13, 2.501906E+13, 2.559291E+13, 
#   2.617896E+13, 2.677744E+13, 2.738859E+13, 2.801266E+13, 2.864991E+13, 2.930058E+13, 
#   2.996495E+13, 3.064326E+13, 3.133580E+13, 3.204284E+13, 3.276466E+13, 3.350153E+13, 
#   3.425376E+13, 3.502164E+13, 3.580545E+13, 
#])
# ============================ M = 42, I = 1 ============================
M = 42
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[2]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.833894E+01, 5.215695E+02, 1.470224E+03, 2.698003E+03, 4.151658E+03, 5.800427E+03, 
   7.623855E+03, 9.608013E+03, 1.174445E+04, 1.403001E+04, 1.646668E+04, 1.906129E+04, 
   2.182505E+04, 2.477304E+04, 2.792373E+04, 3.129858E+04, 3.492171E+04, 3.881978E+04, 
   4.302183E+04, 4.755933E+04, 5.246621E+04, 5.777900E+04, 6.353697E+04, 6.978236E+04, 
   7.656059E+04, 8.392052E+04, 9.191471E+04, 1.005997E+05, 1.100365E+05, 1.202906E+05, 
   1.314325E+05, 1.435383E+05, 1.566897E+05, 1.709747E+05, 1.864878E+05, 2.033308E+05, 
   2.216130E+05, 2.414519E+05, 2.629737E+05, 2.863138E+05, 3.116176E+05, 3.390410E+05, 
   3.687510E+05, 4.009267E+05, 4.357599E+05, 4.734560E+05, 5.142346E+05, 5.583308E+05, 
   6.059957E+05, 6.574979E+05, 7.131240E+05, 7.731800E+05, 8.379927E+05, 9.079104E+05, 
   9.833044E+05, 1.064571E+06, 1.152130E+06, 1.246433E+06, 1.347956E+06, 1.457207E+06, 
   1.574726E+06, 1.701089E+06, 1.836904E+06, 1.982821E+06, 2.139526E+06, 2.307750E+06, 
   2.488267E+06, 2.681897E+06, 2.889511E+06, 3.112030E+06, 3.350430E+06, 3.605744E+06, 
   3.879066E+06, 4.171552E+06, 4.484425E+06, 4.818979E+06, 5.176578E+06, 5.558666E+06, 
   5.966767E+06, 6.402489E+06, 6.867528E+06, 7.363674E+06, 7.892816E+06, 8.456941E+06, 
   9.058148E+06, 9.698643E+06, 1.038075E+07, 1.110692E+07, 1.187974E+07, 1.270190E+07, 
   1.357626E+07, 1.450581E+07, 1.549372E+07, 1.654328E+07, 1.765797E+07, 1.884145E+07, 
   2.009755E+07, 2.143029E+07, 2.284390E+07, 2.434279E+07, 2.593162E+07, 2.761525E+07, 
   2.939878E+07, 3.128757E+07, 3.328720E+07, 3.540354E+07, 3.764272E+07, 4.001119E+07, 
   4.251565E+07, 4.516314E+07, 4.796102E+07, 5.091697E+07, 5.403904E+07, 5.733564E+07, 
   6.081553E+07, 6.448789E+07, 6.836230E+07, 7.244875E+07, 7.675769E+07, 8.130001E+07, 
   8.608707E+07, 9.113073E+07, 9.644335E+07, 1.020378E+08, 1.079276E+08, 1.141267E+08, 
   1.206496E+08, 1.275117E+08, 1.347287E+08, 1.423172E+08, 1.502942E+08, 1.586777E+08, 
   1.674862E+08, 1.767392E+08, 1.864567E+08, 1.966597E+08, 2.073700E+08, 2.186101E+08, 
   2.304036E+08, 2.427751E+08, 2.557498E+08, 2.693541E+08, 2.836156E+08, 2.985626E+08, 
   3.142247E+08, 3.306326E+08, 3.478183E+08, 3.658146E+08, 3.846560E+08, 4.043779E+08, 
   4.250174E+08, 4.466126E+08, 4.692033E+08, 4.928306E+08, 5.175372E+08, 5.433672E+08, 
   5.703664E+08, 5.985823E+08, 6.280640E+08, 6.588624E+08, 6.910301E+08, 7.246219E+08, 
   7.596941E+08, 7.963052E+08, 8.345157E+08, 8.743882E+08, 9.159875E+08, 9.593806E+08, 
   1.004637E+09, 1.051828E+09, 1.101027E+09, 1.152312E+09, 1.205762E+09, 1.261458E+09, 
   1.319485E+09, 1.379931E+09, 1.442885E+09, 1.508441E+09, 1.576695E+09, 1.647746E+09, 
   1.721698E+09, 1.798657E+09, 1.878730E+09, 1.962033E+09, 2.048680E+09, 2.138794E+09, 
   2.232496E+09, 2.329916E+09, 2.431186E+09, 2.536442E+09, 2.645823E+09, 2.759476E+09, 
   2.877549E+09, 3.000196E+09, 3.127576E+09, 3.259852E+09, 3.397192E+09, 3.539771E+09, 
   3.687767E+09, 3.841364E+09, 4.000752E+09, 4.166126E+09, 4.337688E+09, 4.515644E+09, 
   4.700209E+09, 4.891601E+09, 5.090047E+09, 5.295779E+09, 5.509036E+09, 5.730066E+09, 
   5.959120E+09, 6.196461E+09, 6.442356E+09, 6.697081E+09, 6.960920E+09, 7.234165E+09, 
   7.517115E+09, 7.810079E+09, 8.113374E+09, 8.427326E+09, 8.752271E+09, 9.088551E+09, 
   9.436522E+09, 9.796546E+09, 1.016900E+10, 1.055426E+10, 1.095273E+10, 1.136480E+10, 
   1.179091E+10, 1.223146E+10, 1.268691E+10, 1.315770E+10, 1.364429E+10, 1.414716E+10, 
   1.466680E+10, 1.520370E+10, 1.575838E+10, 1.633137E+10, 1.692320E+10, 1.753443E+10, 
   1.816563E+10, 1.881738E+10, 1.949028E+10, 2.018495E+10, 2.090201E+10, 2.164211E+10, 
   2.240591E+10, 2.319410E+10, 2.400736E+10, 2.484641E+10, 2.571198E+10, 2.660482E+10, 
   2.752570E+10, 2.847541E+10, 2.945476E+10, 3.046456E+10, 3.150568E+10, 3.257897E+10, 
   3.368533E+10, 3.482566E+10, 3.600091E+10, 3.721202E+10, 3.845998E+10, 3.974578E+10, 
   4.107046E+10, 4.243506E+10, 4.384066E+10, 4.528836E+10, 4.677930E+10, 4.831462E+10, 
   4.989550E+10, 5.152316E+10, 5.319884E+10, 5.492379E+10, 5.669931E+10, 5.852674E+10, 
   6.040743E+10, 6.234276E+10, 6.433416E+10, 6.638307E+10, 6.849099E+10, 7.065942E+10, 
   7.288994E+10, 7.518412E+10, 7.754359E+10, 7.997001E+10, 8.246509E+10, 8.503055E+10, 
   8.766817E+10, 9.037977E+10, 9.316720E+10, 9.603236E+10, 9.897719E+10, 1.020037E+11, 
   1.051138E+11, 1.083097E+11, 1.115934E+11, 1.149672E+11, 1.184332E+11, 1.219936E+11, 
   1.256508E+11, 1.294072E+11, 
])
# ============================ M = 43, I = 1 ============================
M = 43
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.019023E+01, 9.561317E+01, 1.905574E+02, 2.855193E+02, 3.807311E+02, 4.770977E+02, 
   5.762323E+02, 6.800505E+02, 7.904508E+02, 9.092264E+02, 1.038109E+03, 1.178855E+03, 
   1.333314E+03, 1.503499E+03, 1.691624E+03, 1.900142E+03, 2.131775E+03, 2.389546E+03, 
   2.676806E+03, 2.997267E+03, 3.355044E+03, 3.754687E+03, 4.201231E+03, 4.700238E+03, 
   5.257853E+03, 5.880860E+03, 6.576743E+03, 7.353751E+03, 8.220972E+03, 9.188411E+03, 
   1.026707E+04, 1.146905E+04, 1.280764E+04, 1.429741E+04, 1.595436E+04, 1.779601E+04, 
   1.984153E+04, 2.211192E+04, 2.463013E+04, 2.742122E+04, 3.051255E+04, 3.393398E+04, 
   3.771805E+04, 4.190019E+04, 4.651899E+04, 5.161643E+04, 5.723813E+04, 6.343364E+04, 
   7.025678E+04, 7.776594E+04, 8.602440E+04, 9.510077E+04, 1.050693E+05, 1.160105E+05, 
   1.280112E+05, 1.411654E+05, 1.555748E+05, 1.713490E+05, 1.886064E+05, 2.074746E+05, 
   2.280914E+05, 2.506051E+05, 2.751752E+05, 3.019740E+05, 3.311862E+05, 3.630108E+05, 
   3.976618E+05, 4.353687E+05, 4.763782E+05, 5.209551E+05, 5.693834E+05, 6.219676E+05, 
   6.790341E+05, 7.409330E+05, 8.080384E+05, 8.807517E+05, 9.595015E+05, 1.044747E+06, 
   1.136979E+06, 1.236722E+06, 1.344534E+06, 1.461017E+06, 1.586806E+06, 1.722584E+06, 
   1.869078E+06, 2.027062E+06, 2.197362E+06, 2.380858E+06, 2.578487E+06, 2.791247E+06, 
   3.020201E+06, 3.266477E+06, 3.531278E+06, 3.815881E+06, 4.121646E+06, 4.450013E+06, 
   4.802517E+06, 5.180784E+06, 5.586543E+06, 6.021625E+06, 6.487975E+06, 6.987658E+06, 
   7.522856E+06, 8.095890E+06, 8.709211E+06, 9.365423E+06, 1.006728E+07, 1.081769E+07, 
   1.161974E+07, 1.247670E+07, 1.339201E+07, 1.436933E+07, 1.541252E+07, 1.652564E+07, 
   1.771302E+07, 1.897920E+07, 2.032898E+07, 2.176742E+07, 2.329990E+07, 2.493206E+07, 
   2.666985E+07, 2.851955E+07, 3.048780E+07, 3.258156E+07, 3.480820E+07, 3.717548E+07, 
   3.969155E+07, 4.236502E+07, 4.520494E+07, 4.822084E+07, 5.142273E+07, 5.482117E+07, 
   5.842725E+07, 6.225265E+07, 6.630963E+07, 7.061106E+07, 7.517052E+07, 8.000223E+07, 
   8.512116E+07, 9.054299E+07, 9.628422E+07, 1.023622E+08, 1.087950E+08, 1.156017E+08, 
   1.228024E+08, 1.304180E+08, 1.384705E+08, 1.469829E+08, 1.559795E+08, 1.654856E+08, 
   1.755278E+08, 1.861337E+08, 1.973326E+08, 2.091551E+08, 2.216330E+08, 2.347997E+08, 
   2.486904E+08, 2.633416E+08, 2.787916E+08, 2.950805E+08, 3.122504E+08, 3.303449E+08, 
   3.494100E+08, 3.694936E+08, 3.906456E+08, 4.129187E+08, 4.363673E+08, 4.610487E+08, 
   4.870224E+08, 5.143507E+08, 5.430988E+08, 5.733345E+08, 6.051288E+08, 6.385555E+08, 
   6.736920E+08, 7.106185E+08, 7.494191E+08, 7.901817E+08, 8.329972E+08, 8.779609E+08, 
   9.251722E+08, 9.747347E+08, 1.026756E+09, 1.081348E+09, 1.138629E+09, 1.198719E+09, 
   1.261747E+09, 1.327844E+09, 1.397148E+09, 1.469802E+09, 1.545956E+09, 1.625764E+09, 
   1.709388E+09, 1.796997E+09, 1.888764E+09, 1.984873E+09, 2.085511E+09, 2.190876E+09, 
   2.301173E+09, 2.416613E+09, 2.537418E+09, 2.663817E+09, 2.796049E+09, 2.934362E+09, 
   3.079014E+09, 3.230272E+09, 3.388414E+09, 3.553729E+09, 3.726516E+09, 3.907086E+09, 
   4.095764E+09, 4.292882E+09, 4.498791E+09, 4.713850E+09, 4.938435E+09, 5.172934E+09, 
   5.417751E+09, 5.673301E+09, 5.940022E+09, 6.218363E+09, 6.508787E+09, 6.811782E+09, 
   7.127845E+09, 7.457498E+09, 7.801278E+09, 8.159746E+09, 8.533474E+09, 8.923065E+09, 
   9.329138E+09, 9.752337E+09, 1.019332E+10, 1.065279E+10, 1.113146E+10, 1.163005E+10, 
   1.214934E+10, 1.269011E+10, 1.325320E+10, 1.383943E+10, 1.444970E+10, 1.508491E+10, 
   1.574600E+10, 1.643394E+10, 1.714974E+10, 1.789443E+10, 1.866909E+10, 1.947482E+10, 
   2.031280E+10, 2.118418E+10, 2.209022E+10, 2.303217E+10, 2.401134E+10, 2.502910E+10, 
   2.608684E+10, 2.718602E+10, 2.832812E+10, 2.951468E+10, 3.074732E+10, 3.202768E+10, 
   3.335744E+10, 3.473839E+10, 3.617233E+10, 3.766113E+10, 3.920672E+10, 4.081112E+10, 
   4.247636E+10, 4.420459E+10, 4.599800E+10, 4.785887E+10, 4.978950E+10, 5.179232E+10, 
   5.386983E+10, 5.602458E+10, 5.825923E+10, 6.057649E+10, 6.297920E+10, 6.547025E+10, 
   6.805263E+10, 7.072944E+10, 7.350384E+10, 7.637913E+10, 7.935868E+10, 8.244599E+10, 
   8.564462E+10, 8.895830E+10, 9.239082E+10, 9.594615E+10, 9.962826E+10, 1.034414E+11, 
   1.073898E+11, 1.114780E+11, 1.157104E+11, 1.200918E+11, 1.246270E+11, 1.293209E+11, 
   1.341788E+11, 1.392058E+11, 1.444074E+11, 1.497892E+11, 1.553570E+11, 1.611166E+11, 
   1.670742E+11, 1.732360E+11, 1.796085E+11, 1.861983E+11, 1.930123E+11, 2.000574E+11, 
   2.073409E+11, 2.148703E+11, 2.226532E+11, 2.306975E+11, 2.390111E+11, 2.476026E+11, 
   2.564803E+11, 2.656532E+11, 2.751302E+11, 2.849207E+11, 2.950342E+11, 3.054805E+11, 
   3.162697E+11, 3.274122E+11, 3.389187E+11, 3.508002E+11, 3.630677E+11, 3.757330E+11, 
   3.888080E+11, 4.023048E+11, 4.162361E+11, 4.306145E+11, 4.454535E+11, 4.607667E+11, 
   4.765678E+11, 4.928714E+11, 5.096920E+11, 5.270451E+11, 5.449458E+11, 5.634102E+11, 
   5.824548E+11, 6.020963E+11, 6.223516E+11, 6.432391E+11, 6.647764E+11, 6.869824E+11, 
   7.098760E+11, 7.334772E+11, 7.578059E+11, 7.828826E+11, 8.087291E+11, 8.353667E+11, 
   8.628175E+11, 8.911051E+11, 9.202524E+11, 9.502837E+11, 9.812237E+11, 1.013098E+12, 
   1.045932E+12, 1.079753E+12, 1.114588E+12, 1.150465E+12, 1.187412E+12, 1.225460E+12, 
   1.264639E+12, 1.304979E+12, 1.346512E+12, 1.389271E+12, 1.433289E+12, 1.478600E+12, 
   1.525239E+12, 1.573243E+12, 1.622647E+12, 1.673490E+12, 1.725810E+12, 1.779647E+12, 
   1.835041E+12, 1.892034E+12, 1.950669E+12, 2.010988E+12, 2.073037E+12, 2.136861E+12, 
   2.202508E+12, 2.270024E+12, 2.339459E+12, 2.410865E+12, 2.484291E+12, 2.559791E+12, 
   2.637418E+12, 2.717229E+12, 2.799279E+12, 2.883626E+12, 2.970331E+12, 3.059452E+12, 
   3.151054E+12, 3.245198E+12, 3.341951E+12, 3.441379E+12, 3.543550E+12, 3.648534E+12, 
   3.756402E+12, 3.867227E+12, 3.981085E+12, 4.098051E+12, 4.218203E+12, 4.341624E+12, 
   4.468393E+12, 4.598593E+12, 4.732312E+12, 4.869637E+12, 5.010657E+12, 5.155464E+12, 
   5.304152E+12, 5.456817E+12, 5.613557E+12, 5.774471E+12, 5.939664E+12, 6.109239E+12, 
   6.283303E+12, 6.461967E+12, 6.645342E+12, 6.833542E+12, 7.026684E+12, 7.224891E+12, 
   7.428281E+12, 7.636981E+12, 7.851120E+12, 8.070825E+12, 8.296233E+12, 8.527479E+12, 
   8.764702E+12, 9.008046E+12, 9.257655E+12, 9.513680E+12, 9.776270E+12, 1.004558E+13, 
   1.032177E+13, 1.060501E+13, 1.089545E+13, 1.119326E+13, 1.149863E+13, 1.181173E+13, 
   1.213272E+13, 1.246182E+13, 1.279919E+13, 1.314502E+13, 1.349953E+13, 1.386290E+13, 
   1.423534E+13, 1.461706E+13, 1.500828E+13, 1.540921E+13, 1.582006E+13, 1.624108E+13, 
   1.667249E+13, 1.711453E+13, 1.756743E+13, 1.803146E+13, 1.850684E+13, 1.899385E+13, 
   1.949275E+13, 2.000380E+13, 2.052728E+13, 2.106346E+13, 2.161263E+13, 2.217508E+13, 
   2.275111E+13, 2.334101E+13, 2.394510E+13, 2.456368E+13, 2.519709E+13, 2.584565E+13, 
   2.650969E+13, 2.718955E+13, 2.788558E+13, 2.859814E+13, 2.932759E+13, 3.007429E+13, 
   3.083862E+13, 3.162097E+13, 3.242172E+13, 3.324128E+13, 3.408006E+13, 3.493848E+13, 
   3.581694E+13, 3.671588E+13, 3.763575E+13, 3.857700E+13, 3.954008E+13, 4.052545E+13, 
   4.153361E+13, 4.256502E+13, 4.362020E+13, 4.469962E+13, 4.580383E+13, 4.693334E+13, 
   4.808868E+13, 4.927039E+13, 5.047903E+13, 5.171518E+13, 5.297940E+13, 5.427230E+13, 
   5.559444E+13, 5.694647E+13, 5.832900E+13, 
])
# ============================ M = 44, I = 1 ============================
M = 44
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.957404E+01, 2.768648E+02, 5.517775E+02, 8.267910E+02, 1.102515E+03, 1.381400E+03, 
   1.667911E+03, 1.967411E+03, 2.285213E+03, 2.626251E+03, 2.995131E+03, 3.396321E+03, 
   3.834329E+03, 4.313851E+03, 4.839876E+03, 5.417769E+03, 6.053325E+03, 6.752828E+03, 
   7.523094E+03, 8.371515E+03, 9.306105E+03, 1.033555E+04, 1.146925E+04, 1.271738E+04, 
   1.409094E+04, 1.560182E+04, 1.726285E+04, 1.908786E+04, 2.109179E+04, 2.329072E+04, 
   2.570195E+04, 2.834412E+04, 3.123725E+04, 3.440286E+04, 3.786404E+04, 4.164560E+04, 
   4.577410E+04, 5.027805E+04, 5.518795E+04, 6.053646E+04, 6.635852E+04, 7.269147E+04, 
   7.957522E+04, 8.705239E+04, 9.516846E+04, 1.039719E+05, 1.135146E+05, 1.238515E+05, 
   1.350414E+05, 1.471467E+05, 1.602341E+05, 1.743741E+05, 1.896420E+05, 2.061176E+05, 
   2.238858E+05, 2.430366E+05, 2.636655E+05, 2.858739E+05, 3.097692E+05, 3.354652E+05, 
   3.630825E+05, 3.927487E+05, 4.245990E+05, 4.587763E+05, 4.954319E+05, 5.347255E+05, 
   5.768260E+05, 6.219118E+05, 6.701713E+05, 7.218035E+05, 7.770181E+05, 8.360364E+05, 
   8.990920E+05, 9.664307E+05, 1.038312E+06, 1.115009E+06, 1.196809E+06, 1.284014E+06, 
   1.376945E+06, 1.475935E+06, 1.581337E+06, 1.693522E+06, 1.812879E+06, 1.939818E+06, 
   2.074769E+06, 2.218183E+06, 2.370534E+06, 2.532319E+06, 2.704061E+06, 2.886307E+06, 
   3.079631E+06, 3.284635E+06, 3.501949E+06, 3.732236E+06, 3.976187E+06, 4.234526E+06, 
   4.508015E+06, 4.797446E+06, 5.103651E+06, 5.427500E+06, 5.769904E+06, 6.131812E+06, 
   6.514221E+06, 6.918169E+06, 7.344742E+06, 7.795075E+06, 8.270354E+06, 8.771815E+06, 
   9.300750E+06, 9.858508E+06, 1.044650E+07, 1.106618E+07, 1.171909E+07, 1.240683E+07, 
   1.313106E+07, 1.389351E+07, 1.469599E+07, 1.554039E+07, 1.642867E+07, 1.736287E+07, 
   1.834512E+07, 1.937764E+07, 2.046273E+07, 2.160280E+07, 2.280034E+07, 2.405794E+07, 
   2.537832E+07, 2.676428E+07, 2.821874E+07, 2.974474E+07, 3.134544E+07, 3.302411E+07, 
   3.478417E+07, 3.662916E+07, 3.856274E+07, 4.058874E+07, 4.271113E+07, 4.493400E+07, 
   4.726165E+07, 4.969849E+07, 5.224913E+07, 5.491833E+07, 5.771104E+07, 6.063239E+07, 
   6.368769E+07, 6.688247E+07, 7.022244E+07, 7.371351E+07, 7.736182E+07, 8.117374E+07, 
   8.515586E+07, 8.931498E+07, 9.365818E+07, 9.819277E+07, 1.029263E+08, 1.078667E+08, 
   1.130219E+08, 1.184005E+08, 1.240111E+08, 1.298626E+08, 1.359644E+08, 1.423261E+08, 
   1.489576E+08, 1.558693E+08, 1.630716E+08, 1.705757E+08, 1.783928E+08, 1.865347E+08, 
   1.950135E+08, 2.038418E+08, 2.130324E+08, 2.225987E+08, 2.325545E+08, 2.429141E+08, 
   2.536922E+08, 2.649039E+08, 2.765650E+08, 2.886916E+08, 3.013003E+08, 3.144085E+08, 
   3.280338E+08, 3.421947E+08, 3.569100E+08, 3.721992E+08, 3.880824E+08, 4.045804E+08, 
   4.217145E+08, 4.395068E+08, 4.579800E+08, 4.771575E+08, 4.970634E+08, 5.177226E+08, 
   5.391606E+08, 5.614040E+08, 5.844797E+08, 6.084159E+08, 6.332413E+08, 6.589857E+08, 
   6.856796E+08, 7.133544E+08, 7.420426E+08, 7.717774E+08, 8.025933E+08, 8.345254E+08, 
   8.676102E+08, 9.018850E+08, 9.373883E+08, 9.741596E+08, 1.012240E+09, 1.051670E+09, 
   1.092495E+09, 1.134757E+09, 1.178502E+09, 1.223778E+09, 1.270632E+09, 1.319114E+09, 
   1.369274E+09, 1.421165E+09, 1.474840E+09, 1.530355E+09, 1.587766E+09, 1.647132E+09, 
   1.708511E+09, 1.771966E+09, 1.837560E+09, 1.905357E+09, 1.975424E+09, 2.047830E+09, 
   2.122643E+09, 2.199937E+09, 2.279785E+09, 2.362263E+09, 2.447449E+09, 2.535423E+09, 
   2.626266E+09, 2.720064E+09, 2.816901E+09, 2.916868E+09, 3.020054E+09, 3.126552E+09, 
   3.236459E+09, 3.349872E+09, 3.466892E+09, 3.587623E+09, 3.712168E+09, 3.840638E+09, 
   3.973143E+09, 4.109797E+09, 4.250716E+09, 4.396022E+09, 4.545835E+09, 4.700282E+09, 
   4.859491E+09, 5.023595E+09, 5.192728E+09, 5.367030E+09, 5.546642E+09, 5.731710E+09, 
   5.922382E+09, 6.118812E+09, 6.321154E+09, 6.529569E+09, 6.744222E+09, 6.965279E+09, 
   7.192913E+09, 7.427299E+09, 7.668617E+09, 7.917052E+09, 8.172792E+09, 8.436030E+09, 
   8.706964E+09, 8.985796E+09, 9.272732E+09, 9.567985E+09, 9.871772E+09, 1.018431E+10, 
   1.050584E+10, 1.083657E+10, 1.117676E+10, 1.152664E+10, 1.188646E+10, 1.225648E+10, 
   1.263695E+10, 1.302814E+10, 1.343032E+10, 1.384378E+10, 1.426879E+10, 1.470564E+10, 
   1.515463E+10, 1.561606E+10, 1.609025E+10, 1.657751E+10, 1.707816E+10, 1.759253E+10, 
   1.812097E+10, 1.866382E+10, 1.922142E+10, 1.979415E+10, 2.038237E+10, 2.098646E+10, 
   2.160679E+10, 2.224378E+10, 2.289780E+10, 2.356928E+10, 2.425864E+10, 2.496630E+10, 
   2.569270E+10, 2.643828E+10, 2.720350E+10, 2.798882E+10, 2.879473E+10, 2.962169E+10, 
   3.047022E+10, 3.134081E+10, 3.223398E+10, 3.315025E+10, 3.409017E+10, 3.505428E+10, 
   3.604313E+10, 3.705731E+10, 3.809739E+10, 3.916397E+10, 4.025766E+10, 4.137906E+10, 
   4.252883E+10, 4.370759E+10, 4.491601E+10, 4.615476E+10, 4.742452E+10, 4.872598E+10, 
   5.005987E+10, 5.142691E+10, 5.282783E+10, 5.426339E+10, 5.573436E+10, 5.724153E+10, 
   5.878570E+10, 6.036767E+10, 6.198829E+10, 6.364841E+10, 6.534888E+10, 6.709059E+10, 
   6.887443E+10, 7.070133E+10, 7.257222E+10, 7.448805E+10, 7.644978E+10, 7.845841E+10, 
   8.051495E+10, 8.262042E+10, 8.477586E+10, 8.698235E+10, 8.924097E+10, 9.155283E+10, 
   9.391904E+10, 9.634078E+10, 9.881919E+10, 1.013555E+11, 1.039508E+11, 1.066066E+11, 
   1.093238E+11, 1.121040E+11, 1.149483E+11, 1.178582E+11, 1.208349E+11, 1.238798E+11, 
   1.269944E+11, 1.301801E+11, 1.334383E+11, 1.367706E+11, 1.401783E+11, 1.436632E+11, 
   1.472266E+11, 1.508704E+11, 1.545959E+11, 1.584050E+11, 1.622992E+11, 1.662804E+11, 
   1.703502E+11, 1.745104E+11, 1.787629E+11, 1.831094E+11, 1.875518E+11, 1.920921E+11, 
   1.967321E+11, 2.014738E+11, 2.063193E+11, 2.112706E+11, 2.163297E+11, 2.214988E+11, 
   2.267800E+11, 2.321755E+11, 2.376875E+11, 2.433183E+11, 2.490702E+11, 2.549454E+11, 
   2.609465E+11, 2.670758E+11, 2.733358E+11, 2.797289E+11, 2.862578E+11, 2.929250E+11, 
   2.997331E+11, 3.066848E+11, 3.137829E+11, 3.210302E+11, 3.284293E+11, 3.359832E+11, 
   3.436948E+11, 3.515671E+11, 3.596030E+11, 3.678057E+11, 3.761782E+11, 3.847236E+11, 
   3.934453E+11, 4.023464E+11, 4.114303E+11, 4.207003E+11, 4.301600E+11, 4.398126E+11, 
   4.496619E+11, 4.597113E+11, 4.699646E+11, 4.804255E+11, 4.910977E+11, 5.019851E+11, 
   5.130915E+11, 5.244210E+11, 5.359775E+11, 5.477652E+11, 5.597881E+11, 5.720506E+11, 
   5.845569E+11, 5.973113E+11, 6.103183E+11, 6.235823E+11, 6.371080E+11, 6.509000E+11, 
   6.649629E+11, 6.793016E+11, 6.939209E+11, 7.088257E+11, 7.240211E+11, 7.395121E+11, 
   7.553040E+11, 7.714019E+11, 7.878112E+11, 8.045373E+11, 8.215857E+11, 8.389619E+11, 
   8.566717E+11, 8.747207E+11, 8.931149E+11, 9.118601E+11, 9.309624E+11, 9.504278E+11, 
   9.702626E+11, 9.904731E+11, 1.011066E+12, 1.032047E+12, 1.053423E+12, 1.075201E+12, 
   1.097388E+12, 1.119990E+12, 1.143014E+12, 1.166468E+12, 1.190359E+12, 1.214694E+12, 
   1.239481E+12, 1.264726E+12, 1.290438E+12, 1.316625E+12, 1.343294E+12, 1.370452E+12, 
   1.398110E+12, 1.426273E+12, 1.454952E+12, 1.484154E+12, 1.513888E+12, 1.544162E+12, 
   1.574986E+12, 1.606368E+12, 1.638317E+12, 1.670843E+12, 1.703956E+12, 1.737663E+12, 
   1.771976E+12, 1.806904E+12, 1.842456E+12, 1.878643E+12, 1.915475E+12, 1.952962E+12, 
   1.991115E+12, 2.029945E+12, 2.069461E+12, 2.109676E+12, 2.150600E+12, 2.192244E+12, 
   2.234620E+12, 2.277739E+12, 2.321613E+12, 
])
# ============================ M = 44, I = 2 ============================
#M = 44
#I = 2
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   2.026302E+01, 1.900655E+02, 3.788323E+02, 5.676714E+02, 7.570114E+02, 9.485593E+02, 
#   1.145410E+03, 1.351269E+03, 1.569800E+03, 1.804400E+03, 2.058243E+03, 2.334406E+03, 
#   2.635997E+03, 2.966254E+03, 3.328620E+03, 3.726796E+03, 4.164787E+03, 4.646930E+03, 
#   5.177937E+03, 5.762915E+03, 6.407404E+03, 7.117410E+03, 7.899435E+03, 8.760518E+03, 
#   9.708269E+03, 1.075091E+04, 1.189733E+04, 1.315711E+04, 1.454059E+04, 1.605890E+04, 
#   1.772403E+04, 1.954891E+04, 2.154740E+04, 2.373442E+04, 2.612599E+04, 2.873931E+04, 
#   3.159281E+04, 3.470627E+04, 3.810087E+04, 4.179926E+04, 4.582572E+04, 5.020619E+04, 
#   5.496838E+04, 6.014192E+04, 6.575841E+04, 7.185160E+04, 7.845746E+04, 8.561432E+04, 
#   9.336306E+04, 1.017472E+05, 1.108129E+05, 1.206096E+05, 1.311895E+05, 1.426084E+05, 
#   1.549253E+05, 1.682030E+05, 1.825082E+05, 1.979115E+05, 2.144880E+05, 2.323170E+05, 
#   2.514827E+05, 2.720744E+05, 2.941864E+05, 3.179187E+05, 3.433769E+05, 3.706728E+05, 
#   3.999246E+05, 4.312572E+05, 4.648025E+05, 5.006996E+05, 5.390956E+05, 5.801458E+05, 
#   6.240135E+05, 6.708716E+05, 7.209017E+05, 7.742956E+05, 8.312552E+05, 8.919931E+05, 
#   9.567332E+05, 1.025711E+06, 1.099175E+06, 1.177384E+06, 1.260615E+06, 1.349154E+06, 
#   1.443304E+06, 1.543384E+06, 1.649727E+06, 1.762684E+06, 1.882624E+06, 2.009933E+06, 
#   2.145017E+06, 2.288299E+06, 2.440226E+06, 2.601266E+06, 2.771908E+06, 2.952664E+06, 
#   3.144072E+06, 3.346695E+06, 3.561122E+06, 3.787970E+06, 4.027883E+06, 4.281536E+06, 
#   4.549635E+06, 4.832919E+06, 5.132159E+06, 5.448161E+06, 5.781768E+06, 6.133860E+06, 
#   6.505357E+06, 6.897219E+06, 7.310449E+06, 7.746092E+06, 8.205240E+06, 8.689034E+06, 
#   9.198659E+06, 9.735357E+06, 1.030042E+07, 1.089519E+07, 1.152108E+07, 1.217955E+07, 
#   1.287212E+07, 1.360037E+07, 1.436597E+07, 1.517064E+07, 1.601616E+07, 1.690441E+07, 
#   1.783732E+07, 1.881692E+07, 1.984530E+07, 2.092465E+07, 2.205724E+07, 2.324544E+07, 
#   2.449171E+07, 2.579859E+07, 2.716875E+07, 2.860493E+07, 3.011000E+07, 3.168693E+07, 
#   3.333882E+07, 3.506887E+07, 3.688041E+07, 3.877690E+07, 4.076193E+07, 4.283921E+07, 
#   4.501260E+07, 4.728612E+07, 4.966392E+07, 5.215030E+07, 5.474973E+07, 5.746685E+07, 
#   6.030645E+07, 6.327350E+07, 6.637317E+07, 6.961079E+07, 7.299189E+07, 7.652221E+07, 
#   8.020768E+07, 8.405444E+07, 8.806886E+07, 9.225753E+07, 9.662726E+07, 1.011851E+08, 
#   1.059384E+08, 1.108946E+08, 1.160617E+08, 1.214476E+08, 1.270607E+08, 1.329097E+08, 
#   1.390036E+08, 1.453514E+08, 1.519630E+08, 1.588479E+08, 1.660166E+08, 1.734795E+08, 
#   1.812476E+08, 1.893320E+08, 1.977444E+08, 2.064969E+08, 2.156018E+08, 2.250719E+08, 
#   2.349204E+08, 2.451611E+08, 2.558079E+08, 2.668754E+08, 2.783786E+08, 2.903331E+08, 
#   3.027547E+08, 3.156600E+08, 3.290660E+08, 3.429901E+08, 3.574505E+08, 3.724659E+08, 
#   3.880554E+08, 4.042389E+08, 4.210368E+08, 4.384702E+08, 4.565608E+08, 4.753309E+08, 
#   4.948036E+08, 5.150027E+08, 5.359527E+08, 5.576787E+08, 5.802068E+08, 6.035636E+08, 
#   6.277767E+08, 6.528745E+08, 6.788861E+08, 7.058418E+08, 7.337722E+08, 7.627095E+08, 
#   7.926863E+08, 8.237364E+08, 8.558945E+08, 8.891963E+08, 9.236787E+08, 9.593794E+08, 
#   9.963373E+08, 1.034592E+09, 1.074186E+09, 1.115161E+09, 1.157559E+09, 1.201427E+09, 
#   1.246810E+09, 1.293755E+09, 1.342311E+09, 1.392528E+09, 1.444458E+09, 1.498153E+09, 
#   1.553667E+09, 1.611056E+09, 1.670378E+09, 1.731692E+09, 1.795057E+09, 1.860536E+09, 
#   1.928192E+09, 1.998092E+09, 2.070302E+09, 2.144892E+09, 2.221933E+09, 2.301497E+09, 
#   2.383659E+09, 2.468496E+09, 2.556086E+09, 2.646512E+09, 2.739855E+09, 2.836200E+09, 
#   2.935636E+09, 3.038252E+09, 3.144140E+09, 3.253395E+09, 3.366113E+09, 3.482393E+09, 
#   3.602338E+09, 3.726052E+09, 3.853643E+09, 3.985220E+09, 4.120896E+09, 4.260787E+09, 
#   4.405012E+09, 4.553691E+09, 4.706949E+09, 4.864915E+09, 5.027720E+09, 5.195496E+09, 
#   5.368382E+09, 5.546520E+09, 5.730052E+09, 5.919128E+09, 6.113899E+09, 6.314521E+09, 
#   6.521152E+09, 6.733956E+09, 6.953099E+09, 7.178754E+09, 7.411094E+09, 7.650300E+09, 
#   7.896556E+09, 8.150050E+09, 8.410974E+09, 8.679527E+09, 8.955909E+09, 9.240329E+09, 
#   9.532997E+09, 9.834131E+09, 1.014395E+10, 1.046269E+10, 1.079057E+10, 1.112784E+10, 
#   1.147474E+10, 1.183152E+10, 1.219843E+10, 1.257573E+10, 1.296370E+10, 1.336260E+10, 
#   1.377271E+10, 1.419433E+10, 1.462773E+10, 1.507323E+10, 1.553112E+10, 1.600173E+10, 
#   1.648536E+10, 1.698236E+10, 1.749304E+10, 1.801776E+10, 1.855686E+10, 1.911071E+10, 
#   1.967966E+10, 2.026410E+10, 2.086440E+10, 2.148095E+10, 2.211416E+10, 2.276443E+10, 
#   2.343218E+10, 2.411784E+10, 2.482184E+10, 2.554462E+10, 2.628665E+10, 2.704839E+10, 
#   2.783030E+10, 2.863289E+10, 2.945664E+10, 3.030206E+10, 3.116966E+10, 3.205999E+10, 
#   3.297357E+10, 3.391096E+10, 3.487272E+10, 3.585942E+10, 3.687167E+10, 3.791004E+10, 
#   3.897517E+10, 4.006767E+10, 4.118818E+10, 4.233735E+10, 4.351586E+10, 4.472438E+10, 
#   4.596360E+10, 4.723424E+10, 4.853701E+10, 4.987266E+10, 5.124194E+10, 5.264561E+10, 
#   5.408448E+10, 5.555932E+10, 5.707097E+10, 5.862026E+10, 6.020804E+10, 6.183518E+10, 
#   6.350256E+10, 6.521109E+10, 6.696169E+10, 6.875531E+10, 7.059291E+10, 7.247546E+10, 
#   7.440396E+10, 7.637944E+10, 7.840294E+10, 8.047551E+10, 8.259824E+10, 8.477224E+10, 
#   8.699862E+10, 8.927854E+10, 9.161316E+10, 9.400368E+10, 9.645133E+10, 9.895733E+10, 
#   1.015230E+11, 1.041495E+11, 1.068383E+11, 1.095906E+11, 1.124079E+11, 1.152915E+11, 
#   1.182429E+11, 1.212634E+11, 1.243547E+11, 1.275181E+11, 1.307553E+11, 1.340677E+11, 
#   1.374570E+11, 1.409248E+11, 1.444727E+11, 1.481025E+11, 1.518158E+11, 1.556144E+11, 
#   1.595002E+11, 1.634748E+11, 1.675402E+11, 1.716982E+11, 1.759508E+11, 1.802999E+11, 
#   1.847475E+11, 1.892956E+11, 1.939464E+11, 1.987020E+11, 2.035643E+11, 2.085358E+11, 
#   2.136185E+11, 2.188148E+11, 2.241269E+11, 2.295573E+11, 2.351083E+11, 2.407823E+11, 
#   2.465819E+11, 2.525095E+11, 2.585678E+11, 2.647593E+11, 2.710868E+11, 2.775530E+11, 
#   2.841605E+11, 2.909123E+11, 2.978112E+11, 3.048602E+11, 3.120621E+11, 3.194201E+11, 
#   3.269372E+11, 3.346165E+11, 3.424613E+11, 3.504747E+11, 3.586602E+11, 3.670209E+11, 
#   3.755605E+11, 3.842823E+11, 3.931900E+11, 4.022870E+11, 4.115771E+11, 4.210641E+11, 
#   4.307516E+11, 4.406437E+11, 4.507442E+11, 4.610571E+11, 4.715865E+11, 4.823366E+11, 
#   4.933115E+11, 5.045156E+11, 5.159531E+11, 5.276287E+11, 5.395467E+11, 5.517117E+11, 
#   5.641285E+11, 5.768018E+11, 5.897364E+11, 6.029372E+11, 6.164092E+11, 6.301575E+11, 
#   6.441873E+11, 6.585039E+11, 6.731125E+11, 6.880186E+11, 7.032278E+11, 7.187456E+11, 
#   7.345778E+11, 7.507302E+11, 7.672087E+11, 7.840193E+11, 8.011682E+11, 8.186614E+11, 
#   8.365054E+11, 8.547065E+11, 8.732713E+11, 8.922064E+11, 9.115186E+11, 9.312146E+11, 
#   9.513016E+11, 9.717865E+11, 9.926765E+11, 1.013979E+12, 1.035701E+12, 1.057851E+12, 
#   1.080436E+12, 1.103464E+12, 1.126943E+12, 1.150880E+12, 1.175285E+12, 1.200166E+12, 
#   1.225530E+12, 1.251386E+12, 1.277744E+12, 1.304612E+12, 1.331999E+12, 1.359914E+12, 
#   1.388367E+12, 1.417367E+12, 1.446924E+12, 1.477047E+12, 1.507746E+12, 1.539032E+12, 
#   1.570914E+12, 1.603404E+12, 1.636510E+12, 1.670246E+12, 1.704620E+12, 1.739644E+12, 
#   1.775330E+12, 1.811690E+12, 1.848733E+12, 1.886473E+12, 1.924921E+12, 1.964089E+12, 
#   2.003990E+12, 2.044637E+12, 2.086041E+12, 
#])
# ============================ M = 44, I = 3 ============================
#M = 44
#I = 3
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   5.937613E+01, 5.560196E+02, 1.108143E+03, 1.660488E+03, 2.214311E+03, 2.774611E+03, 
#   3.350442E+03, 3.952652E+03, 4.591963E+03, 5.278320E+03, 6.021014E+03, 6.829058E+03, 
#   7.711556E+03, 8.677996E+03, 9.738464E+03, 1.090381E+04, 1.218577E+04, 1.359706E+04, 
#   1.515150E+04, 1.686406E+04, 1.875100E+04, 2.082993E+04, 2.311993E+04, 2.564166E+04, 
#   2.841745E+04, 3.147146E+04, 3.482975E+04, 3.852050E+04, 4.257404E+04, 4.702312E+04, 
#   5.190298E+04, 5.725156E+04, 6.310967E+04, 6.952117E+04, 7.653320E+04, 8.419632E+04, 
#   9.256483E+04, 1.016969E+05, 1.116549E+05, 1.225056E+05, 1.343204E+05, 1.471759E+05, 
#   1.611536E+05, 1.763410E+05, 1.928313E+05, 2.107240E+05, 2.301251E+05, 2.511480E+05, 
#   2.739133E+05, 2.985493E+05, 3.251929E+05, 3.539897E+05, 3.850943E+05, 4.186715E+05, 
#   4.548961E+05, 4.939538E+05, 5.360418E+05, 5.813696E+05, 6.301590E+05, 6.826456E+05, 
#   7.390789E+05, 7.997234E+05, 8.648589E+05, 9.347821E+05, 1.009807E+06, 1.090265E+06, 
#   1.176507E+06, 1.268904E+06, 1.367849E+06, 1.473755E+06, 1.587061E+06, 1.708227E+06, 
#   1.837741E+06, 1.976116E+06, 2.123896E+06, 2.281650E+06, 2.449982E+06, 2.629525E+06, 
#   2.820949E+06, 3.024955E+06, 3.242286E+06, 3.473719E+06, 3.720074E+06, 3.982214E+06, 
#   4.261045E+06, 4.557518E+06, 4.872634E+06, 5.207446E+06, 5.563055E+06, 5.940622E+06, 
#   6.341362E+06, 6.766551E+06, 7.217530E+06, 7.695700E+06, 8.202535E+06, 8.739577E+06, 
#   9.308444E+06, 9.910830E+06, 1.054851E+07, 1.122334E+07, 1.193726E+07, 1.269232E+07, 
#   1.349064E+07, 1.433446E+07, 1.522610E+07, 1.616800E+07, 1.716271E+07, 1.821289E+07, 
#   1.932134E+07, 2.049095E+07, 2.172477E+07, 2.302597E+07, 2.439786E+07, 2.584390E+07, 
#   2.736771E+07, 2.897304E+07, 3.066383E+07, 3.244418E+07, 3.431836E+07, 3.629083E+07, 
#   3.836623E+07, 4.054942E+07, 4.284542E+07, 4.525950E+07, 4.779714E+07, 5.046405E+07, 
#   5.326615E+07, 5.620963E+07, 5.930093E+07, 6.254675E+07, 6.595405E+07, 6.953009E+07, 
#   7.328240E+07, 7.721884E+07, 8.134755E+07, 8.567703E+07, 9.021607E+07, 9.497385E+07, 
#   9.995988E+07, 1.051840E+08, 1.106566E+08, 1.163883E+08, 1.223902E+08, 1.286737E+08, 
#   1.352508E+08, 1.421340E+08, 1.493361E+08, 1.568704E+08, 1.647508E+08, 1.729917E+08, 
#   1.816079E+08, 1.906150E+08, 2.000290E+08, 2.098665E+08, 2.201447E+08, 2.308815E+08, 
#   2.420954E+08, 2.538056E+08, 2.660319E+08, 2.787949E+08, 2.921160E+08, 3.060172E+08, 
#   3.205215E+08, 3.356524E+08, 3.514345E+08, 3.678931E+08, 3.850546E+08, 4.029460E+08, 
#   4.215955E+08, 4.410322E+08, 4.612862E+08, 4.823886E+08, 5.043715E+08, 5.272684E+08, 
#   5.511135E+08, 5.759424E+08, 6.017921E+08, 6.287003E+08, 6.567066E+08, 6.858513E+08, 
#   7.161766E+08, 7.477256E+08, 7.805433E+08, 8.146758E+08, 8.501709E+08, 8.870778E+08, 
#   9.254476E+08, 9.653328E+08, 1.006788E+09, 1.049868E+09, 1.094632E+09, 1.141140E+09, 
#   1.189452E+09, 1.239632E+09, 1.291746E+09, 1.345861E+09, 1.402048E+09, 1.460378E+09, 
#   1.520924E+09, 1.583765E+09, 1.648979E+09, 1.716646E+09, 1.786851E+09, 1.859680E+09, 
#   1.935222E+09, 2.013569E+09, 2.094816E+09, 2.179059E+09, 2.266400E+09, 2.356942E+09, 
#   2.450790E+09, 2.548056E+09, 2.648851E+09, 2.753293E+09, 2.861501E+09, 2.973598E+09, 
#   3.089712E+09, 3.209973E+09, 3.334516E+09, 3.463478E+09, 3.597004E+09, 3.735240E+09, 
#   3.878336E+09, 4.026448E+09, 4.179736E+09, 4.338364E+09, 4.502501E+09, 4.672321E+09, 
#   4.848004E+09, 5.029732E+09, 5.217696E+09, 5.412090E+09, 5.613114E+09, 5.820973E+09, 
#   6.035879E+09, 6.258049E+09, 6.487707E+09, 6.725082E+09, 6.970410E+09, 7.223933E+09, 
#   7.485901E+09, 7.756570E+09, 8.036202E+09, 8.325067E+09, 8.623444E+09, 8.931616E+09, 
#   9.249878E+09, 9.578528E+09, 9.917877E+09, 1.026824E+10, 1.062995E+10, 1.100333E+10, 
#   1.138872E+10, 1.178649E+10, 1.219699E+10, 1.262059E+10, 1.305768E+10, 1.350864E+10, 
#   1.397387E+10, 1.445380E+10, 1.494883E+10, 1.545941E+10, 1.598597E+10, 1.652898E+10, 
#   1.708890E+10, 1.766621E+10, 1.826140E+10, 1.887499E+10, 1.950748E+10, 2.015940E+10, 
#   2.083131E+10, 2.152376E+10, 2.223732E+10, 2.297258E+10, 2.373014E+10, 2.451062E+10, 
#   2.531464E+10, 2.614287E+10, 2.699595E+10, 2.787458E+10, 2.877944E+10, 2.971126E+10, 
#   3.067077E+10, 3.165871E+10, 3.267586E+10, 3.372299E+10, 3.480093E+10, 3.591049E+10, 
#   3.705252E+10, 3.822790E+10, 3.943750E+10, 4.068223E+10, 4.196303E+10, 4.328086E+10, 
#   4.463668E+10, 4.603150E+10, 4.746634E+10, 4.894225E+10, 5.046030E+10, 5.202159E+10, 
#   5.362725E+10, 5.527842E+10, 5.697629E+10, 5.872206E+10, 6.051696E+10, 6.236225E+10, 
#   6.425924E+10, 6.620923E+10, 6.821359E+10, 7.027369E+10, 7.239096E+10, 7.456684E+10, 
#   7.680281E+10, 7.910039E+10, 8.146113E+10, 8.388662E+10, 8.637848E+10, 8.893836E+10, 
#   9.156797E+10, 9.426904E+10, 9.704335E+10, 9.989270E+10, 1.028190E+11, 1.058240E+11, 
#   1.089098E+11, 1.120783E+11, 1.153316E+11, 1.186717E+11, 1.221007E+11, 1.256209E+11, 
#   1.292343E+11, 1.329434E+11, 1.367503E+11, 1.406575E+11, 1.446673E+11, 1.487823E+11, 
#   1.530049E+11, 1.573378E+11, 1.617835E+11, 1.663448E+11, 1.710243E+11, 1.758250E+11, 
#   1.807496E+11, 1.858012E+11, 1.909826E+11, 1.962970E+11, 2.017474E+11, 2.073370E+11, 
#   2.130692E+11, 2.189472E+11, 2.249745E+11, 2.311544E+11, 2.374905E+11, 2.439865E+11, 
#   2.506460E+11, 2.574728E+11, 2.644708E+11, 2.716438E+11, 2.789960E+11, 2.865313E+11, 
#   2.942539E+11, 3.021682E+11, 3.102784E+11, 3.185890E+11, 3.271046E+11, 3.358298E+11, 
#   3.447692E+11, 3.539278E+11, 3.633104E+11, 3.729220E+11, 3.827678E+11, 3.928530E+11, 
#   4.031829E+11, 4.137630E+11, 4.245987E+11, 4.356959E+11, 4.470602E+11, 4.586975E+11, 
#   4.706138E+11, 4.828154E+11, 4.953083E+11, 5.080990E+11, 5.211941E+11, 5.346001E+11, 
#   5.483237E+11, 5.623720E+11, 5.767519E+11, 5.914706E+11, 6.065354E+11, 6.219538E+11, 
#   6.377333E+11, 6.538818E+11, 6.704071E+11, 6.873174E+11, 7.046207E+11, 7.223255E+11, 
#   7.404404E+11, 7.589739E+11, 7.779351E+11, 7.973329E+11, 8.171766E+11, 8.374755E+11, 
#   8.582393E+11, 8.794777E+11, 9.012006E+11, 9.234181E+11, 9.461407E+11, 9.693788E+11, 
#   9.931431E+11, 1.017445E+12, 1.042294E+12, 1.067704E+12, 1.093684E+12, 1.120248E+12, 
#   1.147407E+12, 1.175173E+12, 1.203558E+12, 1.232576E+12, 1.262239E+12, 1.292561E+12, 
#   1.323554E+12, 1.355233E+12, 1.387611E+12, 1.420703E+12, 1.454523E+12, 1.489085E+12, 
#   1.524405E+12, 1.560498E+12, 1.597380E+12, 1.635065E+12, 1.673571E+12, 1.712914E+12, 
#   1.753109E+12, 1.794176E+12, 1.836129E+12, 1.878988E+12, 1.922771E+12, 1.967494E+12, 
#   2.013178E+12, 2.059840E+12, 2.107501E+12, 2.156179E+12, 2.205895E+12, 2.256669E+12, 
#   2.308522E+12, 2.361474E+12, 2.415548E+12, 2.470764E+12, 2.527145E+12, 2.584714E+12, 
#   2.643494E+12, 2.703507E+12, 2.764779E+12, 2.827332E+12, 2.891192E+12, 2.956383E+12, 
#   3.022932E+12, 3.090864E+12, 3.160205E+12, 3.230982E+12, 3.303223E+12, 3.376956E+12, 
#   3.452208E+12, 3.529009E+12, 3.607388E+12, 3.687374E+12, 3.768997E+12, 3.852290E+12, 
#   3.937282E+12, 4.024006E+12, 4.112494E+12, 4.202780E+12, 4.294896E+12, 4.388877E+12, 
#   4.484757E+12, 4.582572E+12, 4.682358E+12, 4.784151E+12, 4.887988E+12, 4.993906E+12, 
#   5.101945E+12, 5.212143E+12, 5.324540E+12, 5.439176E+12, 5.556093E+12, 5.675331E+12, 
#   5.796933E+12, 5.920943E+12, 6.047403E+12, 6.176359E+12, 6.307856E+12, 6.441940E+12, 
#   6.578657E+12, 6.718056E+12, 6.860184E+12, 7.005090E+12, 7.152825E+12, 7.303439E+12, 
#   7.456984E+12, 7.613512E+12, 7.773076E+12, 
#])
# ============================ M = 44, I = 4 ============================
#M = 44
#I = 4
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   5.938143E+01, 5.560705E+02, 1.108232E+03, 1.660589E+03, 2.214384E+03, 2.774602E+03, 
#   3.350285E+03, 3.952263E+03, 4.591237E+03, 5.277128E+03, 6.019200E+03, 6.826430E+03, 
#   7.707882E+03, 8.672998E+03, 9.731808E+03, 1.089510E+04, 1.217453E+04, 1.358274E+04, 
#   1.513342E+04, 1.684146E+04, 1.872296E+04, 2.079538E+04, 2.307762E+04, 2.559014E+04, 
#   2.835503E+04, 3.139619E+04, 3.473939E+04, 3.841242E+04, 4.244528E+04, 4.687023E+04, 
#   5.172204E+04, 5.703808E+04, 6.285853E+04, 6.922654E+04, 7.618846E+04, 8.379396E+04, 
#   9.209635E+04, 1.011527E+05, 1.110241E+05, 1.217760E+05, 1.334784E+05, 1.462059E+05, 
#   1.600384E+05, 1.750612E+05, 1.913652E+05, 2.090474E+05, 2.282111E+05, 2.489666E+05, 
#   2.714309E+05, 2.957289E+05, 3.219933E+05, 3.503651E+05, 3.809942E+05, 4.140399E+05, 
#   4.496711E+05, 4.880672E+05, 5.294185E+05, 5.739265E+05, 6.218050E+05, 6.732803E+05, 
#   7.285922E+05, 7.879944E+05, 8.517551E+05, 9.201581E+05, 9.935036E+05, 1.072108E+06, 
#   1.156308E+06, 1.246455E+06, 1.342922E+06, 1.446105E+06, 1.556418E+06, 1.674298E+06, 
#   1.800208E+06, 1.934634E+06, 2.078086E+06, 2.231106E+06, 2.394260E+06, 2.568145E+06, 
#   2.753389E+06, 2.950653E+06, 3.160630E+06, 3.384050E+06, 3.621680E+06, 3.874323E+06, 
#   4.142825E+06, 4.428072E+06, 4.730995E+06, 5.052568E+06, 5.393816E+06, 5.755811E+06, 
#   6.139677E+06, 6.546591E+06, 6.977788E+06, 7.434559E+06, 7.918256E+06, 8.430295E+06, 
#   8.972157E+06, 9.545390E+06, 1.015162E+07, 1.079252E+07, 1.146989E+07, 1.218556E+07, 
#   1.294147E+07, 1.373964E+07, 1.458216E+07, 1.547126E+07, 1.640921E+07, 1.739842E+07, 
#   1.844139E+07, 1.954073E+07, 2.069914E+07, 2.191948E+07, 2.320468E+07, 2.455784E+07, 
#   2.598215E+07, 2.748096E+07, 2.905775E+07, 3.071613E+07, 3.245988E+07, 3.429292E+07, 
#   3.621934E+07, 3.824337E+07, 4.036944E+07, 4.260214E+07, 4.494624E+07, 4.740672E+07, 
#   4.998871E+07, 5.269759E+07, 5.553892E+07, 5.851848E+07, 6.164228E+07, 6.491656E+07, 
#   6.834778E+07, 7.194266E+07, 7.570816E+07, 7.965154E+07, 8.378027E+07, 8.810214E+07, 
#   9.262522E+07, 9.735787E+07, 1.023088E+08, 1.074869E+08, 1.129015E+08, 1.185623E+08, 
#   1.244793E+08, 1.306628E+08, 1.371236E+08, 1.438726E+08, 1.509215E+08, 1.582821E+08, 
#   1.659666E+08, 1.739880E+08, 1.823591E+08, 1.910938E+08, 2.002061E+08, 2.097106E+08, 
#   2.196223E+08, 2.299567E+08, 2.407301E+08, 2.519589E+08, 2.636605E+08, 2.758525E+08, 
#   2.885532E+08, 3.017817E+08, 3.155574E+08, 3.299006E+08, 3.448321E+08, 3.603734E+08, 
#   3.765468E+08, 3.933751E+08, 4.108820E+08, 4.290919E+08, 4.480299E+08, 4.677221E+08, 
#   4.881951E+08, 5.094766E+08, 5.315951E+08, 5.545798E+08, 5.784610E+08, 6.032699E+08, 
#   6.290386E+08, 6.558001E+08, 6.835885E+08, 7.124389E+08, 7.423875E+08, 7.734716E+08, 
#   8.057294E+08, 8.392004E+08, 8.739252E+08, 9.099458E+08, 9.473050E+08, 9.860472E+08, 
#   1.026218E+09, 1.067864E+09, 1.111034E+09, 1.155778E+09, 1.202145E+09, 1.250190E+09, 
#   1.299965E+09, 1.351526E+09, 1.404931E+09, 1.460237E+09, 1.517506E+09, 1.576798E+09, 
#   1.638178E+09, 1.701711E+09, 1.767464E+09, 1.835506E+09, 1.905908E+09, 1.978742E+09, 
#   2.054084E+09, 2.132010E+09, 2.212598E+09, 2.295930E+09, 2.382088E+09, 2.471159E+09, 
#   2.563228E+09, 2.658385E+09, 2.756724E+09, 2.858337E+09, 2.963323E+09, 3.071779E+09, 
#   3.183809E+09, 3.299516E+09, 3.419008E+09, 3.542394E+09, 3.669787E+09, 3.801303E+09, 
#   3.937060E+09, 4.077178E+09, 4.221783E+09, 4.371001E+09, 4.524963E+09, 4.683804E+09, 
#   4.847658E+09, 5.016668E+09, 5.190976E+09, 5.370731E+09, 5.556081E+09, 5.747183E+09, 
#   5.944193E+09, 6.147273E+09, 6.356590E+09, 6.572312E+09, 6.794613E+09, 7.023670E+09, 
#   7.259666E+09, 7.502786E+09, 7.753220E+09, 8.011162E+09, 8.276813E+09, 8.550375E+09, 
#   8.832057E+09, 9.122072E+09, 9.420636E+09, 9.727975E+09, 1.004431E+10, 1.036989E+10, 
#   1.070493E+10, 1.104969E+10, 1.140441E+10, 1.176935E+10, 1.214477E+10, 1.253093E+10, 
#   1.292811E+10, 1.333657E+10, 1.375661E+10, 1.418852E+10, 1.463258E+10, 1.508910E+10, 
#   1.555839E+10, 1.604076E+10, 1.653654E+10, 1.704605E+10, 1.756962E+10, 1.810761E+10, 
#   1.866035E+10, 1.922820E+10, 1.981154E+10, 2.041072E+10, 2.102613E+10, 2.165815E+10, 
#   2.230718E+10, 2.297363E+10, 2.365789E+10, 2.436039E+10, 2.508156E+10, 2.582182E+10, 
#   2.658164E+10, 2.736145E+10, 2.816173E+10, 2.898293E+10, 2.982555E+10, 3.069007E+10, 
#   3.157699E+10, 3.248682E+10, 3.342008E+10, 3.437729E+10, 3.535900E+10, 3.636575E+10, 
#   3.739810E+10, 3.845663E+10, 3.954191E+10, 4.065454E+10, 4.179511E+10, 4.296424E+10, 
#   4.416256E+10, 4.539071E+10, 4.664932E+10, 4.793907E+10, 4.926062E+10, 5.061466E+10, 
#   5.200189E+10, 5.342301E+10, 5.487874E+10, 5.636984E+10, 5.789703E+10, 5.946108E+10, 
#   6.106278E+10, 6.270289E+10, 6.438224E+10, 6.610163E+10, 6.786190E+10, 6.966390E+10, 
#   7.150847E+10, 7.339650E+10, 7.532887E+10, 7.730650E+10, 7.933029E+10, 8.140119E+10, 
#   8.352015E+10, 8.568814E+10, 8.790613E+10, 9.017513E+10, 9.249616E+10, 9.487025E+10, 
#   9.729844E+10, 9.978181E+10, 1.023214E+11, 1.049184E+11, 1.075739E+11, 1.102890E+11, 
#   1.130649E+11, 1.159027E+11, 1.188037E+11, 1.217690E+11, 1.248000E+11, 1.278978E+11, 
#   1.310636E+11, 1.342989E+11, 1.376050E+11, 1.409830E+11, 1.444344E+11, 1.479606E+11, 
#   1.515630E+11, 1.552430E+11, 1.590019E+11, 1.628413E+11, 1.667626E+11, 1.707674E+11, 
#   1.748572E+11, 1.790334E+11, 1.832978E+11, 1.876518E+11, 1.920970E+11, 1.966352E+11, 
#   2.012680E+11, 2.059970E+11, 2.108239E+11, 2.157506E+11, 2.207787E+11, 2.259100E+11, 
#   2.311464E+11, 2.364897E+11, 2.419416E+11, 2.475042E+11, 2.531793E+11, 2.589688E+11, 
#   2.648748E+11, 2.708991E+11, 2.770438E+11, 2.833110E+11, 2.897027E+11, 2.962209E+11, 
#   3.028679E+11, 3.096458E+11, 3.165567E+11, 3.236029E+11, 3.307865E+11, 3.381099E+11, 
#   3.455753E+11, 3.531851E+11, 3.609416E+11, 3.688471E+11, 3.769042E+11, 3.851151E+11, 
#   3.934824E+11, 4.020087E+11, 4.106963E+11, 4.195479E+11, 4.285660E+11, 4.377533E+11, 
#   4.471124E+11, 4.566460E+11, 4.663568E+11, 4.762476E+11, 4.863211E+11, 4.965801E+11, 
#   5.070275E+11, 5.176661E+11, 5.284988E+11, 5.395286E+11, 5.507584E+11, 5.621913E+11, 
#   5.738302E+11, 5.856782E+11, 5.977384E+11, 6.100140E+11, 6.225080E+11, 6.352238E+11, 
#   6.481645E+11, 6.613334E+11, 6.747337E+11, 6.883689E+11, 7.022422E+11, 7.163570E+11, 
#   7.307169E+11, 7.453251E+11, 7.601854E+11, 7.753010E+11, 7.906757E+11, 8.063130E+11, 
#   8.222165E+11, 8.383899E+11, 8.548370E+11, 8.715613E+11, 8.885667E+11, 9.058569E+11, 
#   9.234359E+11, 9.413074E+11, 9.594754E+11, 9.779437E+11, 9.967163E+11, 1.015797E+12, 
#   1.035191E+12, 1.054900E+12, 1.074930E+12, 1.095285E+12, 1.115969E+12, 1.136985E+12, 
#   1.158338E+12, 1.180033E+12, 1.202074E+12, 1.224464E+12, 1.247209E+12, 1.270313E+12, 
#   1.293779E+12, 1.317613E+12, 1.341819E+12, 1.366401E+12, 1.391364E+12, 1.416713E+12, 
#   1.442451E+12, 1.468585E+12, 1.495117E+12, 1.522053E+12, 1.549398E+12, 1.577156E+12, 
#   1.605332E+12, 1.633930E+12, 1.662956E+12, 1.692414E+12, 1.722309E+12, 1.752646E+12, 
#   1.783430E+12, 1.814665E+12, 1.846357E+12, 1.878510E+12, 1.911130E+12, 1.944220E+12, 
#   1.977787E+12, 2.011835E+12, 2.046369E+12, 2.081394E+12, 2.116916E+12, 2.152938E+12, 
#   2.189467E+12, 2.226507E+12, 2.264063E+12, 2.302141E+12, 2.340746E+12, 2.379881E+12, 
#   2.419554E+12, 2.459768E+12, 2.500528E+12, 2.541841E+12, 2.583711E+12, 2.626142E+12, 
#   2.669141E+12, 2.712712E+12, 2.756860E+12, 
#])
# ============================ M = 44, I = 5 ============================
#M = 44
#I = 5
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   6.089545E+01, 5.712682E+02, 1.138666E+03, 1.706332E+03, 2.275581E+03, 2.851569E+03, 
#   3.443628E+03, 4.062932E+03, 4.720532E+03, 5.426688E+03, 6.190997E+03, 7.022774E+03, 
#   7.931441E+03, 8.926822E+03, 1.001937E+04, 1.122035E+04, 1.254193E+04, 1.399732E+04, 
#   1.560089E+04, 1.736821E+04, 1.931620E+04, 2.146321E+04, 2.382914E+04, 2.643554E+04, 
#   2.930574E+04, 3.246499E+04, 3.594057E+04, 3.976195E+04, 4.396098E+04, 4.857198E+04, 
#   5.363197E+04, 5.918083E+04, 6.526153E+04, 7.192025E+04, 7.920670E+04, 8.717429E+04, 
#   9.588037E+04, 1.053865E+05, 1.157587E+05, 1.270679E+05, 1.393898E+05, 1.528058E+05, 
#   1.674027E+05, 1.832737E+05, 2.005182E+05, 2.192425E+05, 2.395601E+05, 2.615923E+05, 
#   2.854683E+05, 3.113262E+05, 3.393131E+05, 3.695855E+05, 4.023103E+05, 4.376654E+05, 
#   4.758397E+05, 5.170344E+05, 5.614634E+05, 6.093538E+05, 6.609472E+05, 7.164999E+05, 
#   7.762842E+05, 8.405888E+05, 9.097202E+05, 9.840032E+05, 1.063782E+06, 1.149422E+06, 
#   1.241309E+06, 1.339852E+06, 1.445484E+06, 1.558664E+06, 1.679876E+06, 1.809633E+06, 
#   1.948476E+06, 2.096978E+06, 2.255744E+06, 2.425410E+06, 2.606653E+06, 2.800184E+06, 
#   3.006751E+06, 3.227149E+06, 3.462210E+06, 3.712816E+06, 3.979893E+06, 4.264418E+06, 
#   4.567420E+06, 4.889983E+06, 5.233246E+06, 5.598411E+06, 5.986740E+06, 6.399560E+06, 
#   6.838269E+06, 7.304335E+06, 7.799301E+06, 8.324789E+06, 8.882501E+06, 9.474227E+06, 
#   1.010185E+07, 1.076733E+07, 1.147274E+07, 1.222027E+07, 1.301217E+07, 1.385085E+07, 
#   1.473880E+07, 1.567865E+07, 1.667316E+07, 1.772521E+07, 1.883782E+07, 2.001415E+07, 
#   2.125752E+07, 2.257140E+07, 2.395941E+07, 2.542536E+07, 2.697323E+07, 2.860718E+07, 
#   3.033154E+07, 3.215089E+07, 3.406996E+07, 3.609374E+07, 3.822742E+07, 4.047642E+07, 
#   4.284643E+07, 4.534336E+07, 4.797341E+07, 5.074303E+07, 5.365896E+07, 5.672826E+07, 
#   5.995825E+07, 6.335661E+07, 6.693134E+07, 7.069077E+07, 7.464359E+07, 7.879889E+07, 
#   8.316611E+07, 8.775511E+07, 9.257615E+07, 9.763994E+07, 1.029576E+08, 1.085408E+08, 
#   1.144016E+08, 1.205526E+08, 1.270069E+08, 1.337782E+08, 1.408806E+08, 1.483290E+08, 
#   1.561388E+08, 1.643259E+08, 1.729070E+08, 1.818994E+08, 1.913212E+08, 2.011910E+08, 
#   2.115284E+08, 2.223535E+08, 2.336874E+08, 2.455520E+08, 2.579699E+08, 2.709648E+08, 
#   2.845612E+08, 2.987845E+08, 3.136612E+08, 3.292187E+08, 3.454855E+08, 3.624912E+08, 
#   3.802665E+08, 3.988432E+08, 4.182545E+08, 4.385346E+08, 4.597192E+08, 4.818451E+08, 
#   5.049506E+08, 5.290755E+08, 5.542609E+08, 5.805495E+08, 6.079855E+08, 6.366149E+08, 
#   6.664850E+08, 6.976453E+08, 7.301468E+08, 7.640422E+08, 7.993865E+08, 8.362363E+08, 
#   8.746503E+08, 9.146896E+08, 9.564169E+08, 9.998977E+08, 1.045199E+09, 1.092392E+09, 
#   1.141548E+09, 1.192741E+09, 1.246050E+09, 1.301555E+09, 1.359339E+09, 1.419486E+09, 
#   1.482087E+09, 1.547233E+09, 1.615018E+09, 1.685542E+09, 1.758904E+09, 1.835211E+09, 
#   1.914570E+09, 1.997094E+09, 2.082898E+09, 2.172102E+09, 2.264830E+09, 2.361209E+09, 
#   2.461371E+09, 2.565453E+09, 2.673596E+09, 2.785944E+09, 2.902649E+09, 3.023864E+09, 
#   3.149751E+09, 3.280474E+09, 3.416204E+09, 3.557117E+09, 3.703395E+09, 3.855225E+09, 
#   4.012802E+09, 4.176324E+09, 4.345997E+09, 4.522035E+09, 4.704657E+09, 4.894088E+09, 
#   5.090563E+09, 5.294321E+09, 5.505610E+09, 5.724687E+09, 5.951816E+09, 6.187267E+09, 
#   6.431321E+09, 6.684267E+09, 6.946404E+09, 7.218037E+09, 7.499482E+09, 7.791067E+09, 
#   8.093125E+09, 8.406004E+09, 8.730059E+09, 9.065658E+09, 9.413178E+09, 9.773008E+09, 
#   1.014555E+10, 1.053122E+10, 1.093043E+10, 1.134364E+10, 1.177128E+10, 1.221382E+10, 
#   1.267174E+10, 1.314553E+10, 1.363570E+10, 1.414276E+10, 1.466726E+10, 1.520974E+10, 
#   1.577077E+10, 1.635094E+10, 1.695084E+10, 1.757109E+10, 1.821232E+10, 1.887520E+10, 
#   1.956039E+10, 2.026858E+10, 2.100048E+10, 2.175682E+10, 2.253836E+10, 2.334587E+10, 
#   2.418013E+10, 2.504197E+10, 2.593222E+10, 2.685175E+10, 2.780144E+10, 2.878221E+10, 
#   2.979498E+10, 3.084071E+10, 3.192040E+10, 3.303507E+10, 3.418574E+10, 3.537350E+10, 
#   3.659944E+10, 3.786469E+10, 3.917042E+10, 4.051781E+10, 4.190809E+10, 4.334251E+10, 
#   4.482236E+10, 4.634897E+10, 4.792370E+10, 4.954794E+10, 5.122313E+10, 5.295074E+10, 
#   5.473227E+10, 5.656928E+10, 5.846336E+10, 6.041614E+10, 6.242928E+10, 6.450452E+10, 
#   6.664361E+10, 6.884837E+10, 7.112064E+10, 7.346233E+10, 7.587540E+10, 7.836184E+10, 
#   8.092371E+10, 8.356312E+10, 8.628222E+10, 8.908324E+10, 9.196845E+10, 9.494016E+10, 
#   9.800078E+10, 1.011528E+11, 1.043986E+11, 1.077408E+11, 1.111822E+11, 1.147253E+11, 
#   1.183730E+11, 1.221280E+11, 1.259934E+11, 1.299720E+11, 1.340670E+11, 1.382814E+11, 
#   1.426185E+11, 1.470816E+11, 1.516740E+11, 1.563992E+11, 1.612608E+11, 1.662623E+11, 
#   1.714074E+11, 1.767000E+11, 1.821440E+11, 1.877432E+11, 1.935019E+11, 1.994242E+11, 
#   2.055144E+11, 2.117768E+11, 2.182159E+11, 2.248364E+11, 2.316429E+11, 2.386403E+11, 
#   2.458335E+11, 2.532274E+11, 2.608274E+11, 2.686386E+11, 2.766666E+11, 2.849167E+11, 
#   2.933947E+11, 3.021064E+11, 3.110577E+11, 3.202547E+11, 3.297036E+11, 3.394107E+11, 
#   3.493825E+11, 3.596258E+11, 3.701472E+11, 3.809538E+11, 3.920527E+11, 4.034511E+11, 
#   4.151565E+11, 4.271765E+11, 4.395189E+11, 4.521917E+11, 4.652030E+11, 4.785611E+11, 
#   4.922745E+11, 5.063521E+11, 5.208025E+11, 5.356351E+11, 5.508590E+11, 5.664837E+11, 
#   5.825191E+11, 5.989750E+11, 6.158615E+11, 6.331892E+11, 6.509685E+11, 6.692103E+11, 
#   6.879258E+11, 7.071261E+11, 7.268230E+11, 7.470282E+11, 7.677538E+11, 7.890122E+11, 
#   8.108159E+11, 8.331779E+11, 8.561113E+11, 8.796295E+11, 9.037464E+11, 9.284759E+11, 
#   9.538324E+11, 9.798305E+11, 1.006485E+12, 1.033812E+12, 1.061826E+12, 1.090543E+12, 
#   1.119980E+12, 1.150153E+12, 1.181079E+12, 1.212776E+12, 1.245261E+12, 1.278552E+12, 
#   1.312668E+12, 1.347628E+12, 1.383450E+12, 1.420154E+12, 1.457761E+12, 1.496290E+12, 
#   1.535763E+12, 1.576201E+12, 1.617626E+12, 1.660059E+12, 1.703524E+12, 1.748043E+12, 
#   1.793640E+12, 1.840340E+12, 1.888166E+12, 1.937143E+12, 1.987298E+12, 2.038656E+12, 
#   2.091244E+12, 2.145089E+12, 2.200218E+12, 2.256661E+12, 2.314445E+12, 2.373600E+12, 
#   2.434156E+12, 2.496144E+12, 2.559594E+12, 2.624539E+12, 2.691011E+12, 2.759043E+12, 
#   2.828668E+12, 2.899922E+12, 2.972839E+12, 3.047454E+12, 3.123805E+12, 3.201929E+12, 
#   3.281863E+12, 3.363646E+12, 3.447318E+12, 3.532918E+12, 3.620488E+12, 3.710069E+12, 
#   3.801704E+12, 3.895436E+12, 3.991309E+12, 4.089369E+12, 4.189662E+12, 4.292233E+12, 
#   4.397132E+12, 4.504406E+12, 4.614105E+12, 4.726280E+12, 4.840981E+12, 4.958262E+12, 
#   5.078176E+12, 5.200777E+12, 5.326121E+12, 5.454264E+12, 5.585264E+12, 5.719179E+12, 
#   5.856069E+12, 5.995996E+12, 6.139020E+12, 6.285206E+12, 6.434618E+12, 6.587320E+12, 
#   6.743380E+12, 6.902867E+12, 7.065848E+12, 7.232395E+12, 7.402580E+12, 7.576475E+12, 
#   7.754156E+12, 7.935697E+12, 8.121177E+12, 8.310674E+12, 8.504267E+12, 8.702040E+12, 
#   8.904074E+12, 9.110455E+12, 9.321268E+12, 9.536600E+12, 9.756542E+12, 9.981185E+12, 
#   1.021062E+13, 1.044494E+13, 1.068425E+13, 1.092863E+13, 1.117819E+13, 1.143304E+13, 
#   1.169327E+13, 1.195898E+13, 1.223029E+13, 1.250731E+13, 1.279014E+13, 1.307889E+13, 
#   1.337368E+13, 1.367463E+13, 1.398186E+13, 1.429548E+13, 1.461562E+13, 1.494240E+13, 
#   1.527595E+13, 1.561640E+13, 1.596388E+13, 
#])
# ============================ M = 44, I = 6 ============================
#M = 44
#I = 6
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   4.754866E+01, 4.472778E+02, 8.916359E+02, 1.336193E+03, 1.782331E+03, 2.235042E+03, 
#   2.702729E+03, 3.194988E+03, 3.721133E+03, 4.289909E+03, 4.909770E+03, 5.589265E+03, 
#   6.337351E+03, 7.163609E+03, 8.078399E+03, 9.092975E+03, 1.021958E+04, 1.147157E+04, 
#   1.286346E+04, 1.441108E+04, 1.613167E+04, 1.804397E+04, 2.016839E+04, 2.252712E+04, 
#   2.514428E+04, 2.804603E+04, 3.126081E+04, 3.481940E+04, 3.875522E+04, 4.310438E+04, 
#   4.790601E+04, 5.320235E+04, 5.903908E+04, 6.546546E+04, 7.253463E+04, 8.030386E+04, 
#   8.883483E+04, 9.819391E+04, 1.084525E+05, 1.196872E+05, 1.319805E+05, 1.454207E+05, 
#   1.601027E+05, 1.761282E+05, 1.936059E+05, 2.126526E+05, 2.333929E+05, 2.559602E+05, 
#   2.804972E+05, 3.071560E+05, 3.360995E+05, 3.675010E+05, 4.015457E+05, 4.384308E+05, 
#   4.783667E+05, 5.215771E+05, 5.683005E+05, 6.187903E+05, 6.733163E+05, 7.321651E+05, 
#   7.956413E+05, 8.640683E+05, 9.377896E+05, 1.017170E+06, 1.102595E+06, 1.194475E+06, 
#   1.293244E+06, 1.399362E+06, 1.513318E+06, 1.635625E+06, 1.766830E+06, 1.907511E+06, 
#   2.058278E+06, 2.219775E+06, 2.392684E+06, 2.577726E+06, 2.775660E+06, 2.987288E+06, 
#   3.213457E+06, 3.455059E+06, 3.713036E+06, 3.988380E+06, 4.282135E+06, 4.595403E+06, 
#   4.929342E+06, 5.285174E+06, 5.664181E+06, 6.067715E+06, 6.497197E+06, 6.954119E+06, 
#   7.440051E+06, 7.956642E+06, 8.505626E+06, 9.088820E+06, 9.708134E+06, 1.036557E+07, 
#   1.106324E+07, 1.180334E+07, 1.258818E+07, 1.342020E+07, 1.430192E+07, 1.523602E+07, 
#   1.622529E+07, 1.727264E+07, 1.838112E+07, 1.955396E+07, 2.079448E+07, 2.210621E+07, 
#   2.349279E+07, 2.495808E+07, 2.650606E+07, 2.814093E+07, 2.986707E+07, 3.168905E+07, 
#   3.361164E+07, 3.563983E+07, 3.777883E+07, 4.003406E+07, 4.241120E+07, 4.491616E+07, 
#   4.755512E+07, 5.033450E+07, 5.326102E+07, 5.634166E+07, 5.958372E+07, 6.299479E+07, 
#   6.658278E+07, 7.035593E+07, 7.432283E+07, 7.849239E+07, 8.287394E+07, 8.747713E+07, 
#   9.231204E+07, 9.738916E+07, 1.027194E+08, 1.083140E+08, 1.141848E+08, 1.203442E+08, 
#   1.268047E+08, 1.335796E+08, 1.406827E+08, 1.481282E+08, 1.559310E+08, 1.641065E+08, 
#   1.726706E+08, 1.816401E+08, 1.910320E+08, 2.008643E+08, 2.111556E+08, 2.219250E+08, 
#   2.331926E+08, 2.449790E+08, 2.573058E+08, 2.701951E+08, 2.836700E+08, 2.977545E+08, 
#   3.124733E+08, 3.278522E+08, 3.439176E+08, 3.606972E+08, 3.782194E+08, 3.965138E+08, 
#   4.156110E+08, 4.355427E+08, 4.563416E+08, 4.780415E+08, 5.006777E+08, 5.242864E+08, 
#   5.489052E+08, 5.745728E+08, 6.013295E+08, 6.292168E+08, 6.582776E+08, 6.885562E+08, 
#   7.200987E+08, 7.529523E+08, 7.871661E+08, 8.227907E+08, 8.598783E+08, 8.984830E+08, 
#   9.386606E+08, 9.804685E+08, 1.023966E+09, 1.069216E+09, 1.116280E+09, 1.165223E+09, 
#   1.216115E+09, 1.269024E+09, 1.324022E+09, 1.381183E+09, 1.440584E+09, 1.502304E+09, 
#   1.566423E+09, 1.633027E+09, 1.702201E+09, 1.774034E+09, 1.848619E+09, 1.926051E+09, 
#   2.006426E+09, 2.089845E+09, 2.176413E+09, 2.266235E+09, 2.359422E+09, 2.456088E+09, 
#   2.556348E+09, 2.660324E+09, 2.768139E+09, 2.879919E+09, 2.995797E+09, 3.115908E+09, 
#   3.240390E+09, 3.369386E+09, 3.503044E+09, 3.641515E+09, 3.784955E+09, 3.933524E+09, 
#   4.087387E+09, 4.246714E+09, 4.411678E+09, 4.582460E+09, 4.759244E+09, 4.942219E+09, 
#   5.131581E+09, 5.327529E+09, 5.530270E+09, 5.740015E+09, 5.956982E+09, 6.181394E+09, 
#   6.413482E+09, 6.653480E+09, 6.901632E+09, 7.158187E+09, 7.423400E+09, 7.697535E+09, 
#   7.980860E+09, 8.273652E+09, 8.576198E+09, 8.888787E+09, 9.211721E+09, 9.545306E+09, 
#   9.889859E+09, 1.024570E+10, 1.061317E+10, 1.099261E+10, 1.138436E+10, 1.178879E+10, 
#   1.220626E+10, 1.263716E+10, 1.308186E+10, 1.354078E+10, 1.401431E+10, 1.450287E+10, 
#   1.500690E+10, 1.552684E+10, 1.606313E+10, 1.661624E+10, 1.718664E+10, 1.777482E+10, 
#   1.838128E+10, 1.900653E+10, 1.965108E+10, 2.031549E+10, 2.100029E+10, 2.170605E+10, 
#   2.243334E+10, 2.318277E+10, 2.395492E+10, 2.475043E+10, 2.556992E+10, 2.641406E+10, 
#   2.728349E+10, 2.817891E+10, 2.910102E+10, 3.005052E+10, 3.102816E+10, 3.203467E+10, 
#   3.307083E+10, 3.413742E+10, 3.523525E+10, 3.636514E+10, 3.752792E+10, 3.872447E+10, 
#   3.995566E+10, 4.122239E+10, 4.252559E+10, 4.386620E+10, 4.524519E+10, 4.666354E+10, 
#   4.812227E+10, 4.962240E+10, 5.116500E+10, 5.275114E+10, 5.438193E+10, 5.605851E+10, 
#   5.778202E+10, 5.955364E+10, 6.137460E+10, 6.324611E+10, 6.516944E+10, 6.714589E+10, 
#   6.917677E+10, 7.126343E+10, 7.340724E+10, 7.560962E+10, 7.787200E+10, 8.019586E+10, 
#   8.258268E+10, 8.503402E+10, 8.755144E+10, 9.013653E+10, 9.279093E+10, 9.551632E+10, 
#   9.831441E+10, 1.011869E+11, 1.041357E+11, 1.071624E+11, 1.102691E+11, 1.134575E+11, 
#   1.167297E+11, 1.200876E+11, 1.235331E+11, 1.270685E+11, 1.306957E+11, 1.344170E+11, 
#   1.382345E+11, 1.421504E+11, 1.461671E+11, 1.502869E+11, 1.545121E+11, 1.588452E+11, 
#   1.632886E+11, 1.678449E+11, 1.725167E+11, 1.773065E+11, 1.822172E+11, 1.872513E+11, 
#   1.924118E+11, 1.977014E+11, 2.031230E+11, 2.086797E+11, 2.143744E+11, 2.202103E+11, 
#   2.261904E+11, 2.323180E+11, 2.385964E+11, 2.450288E+11, 2.516188E+11, 2.583697E+11, 
#   2.652851E+11, 2.723686E+11, 2.796238E+11, 2.870546E+11, 2.946647E+11, 3.024580E+11, 
#   3.104386E+11, 3.186103E+11, 3.269774E+11, 3.355441E+11, 3.443146E+11, 3.532933E+11, 
#   3.624846E+11, 3.718931E+11, 3.815234E+11, 3.913801E+11, 4.014681E+11, 4.117923E+11, 
#   4.223576E+11, 4.331691E+11, 4.442319E+11, 4.555513E+11, 4.671327E+11, 4.789815E+11, 
#   4.911032E+11, 5.035035E+11, 5.161882E+11, 5.291631E+11, 5.424342E+11, 5.560076E+11, 
#   5.698895E+11, 5.840861E+11, 5.986039E+11, 6.134495E+11, 6.286294E+11, 6.441505E+11, 
#   6.600196E+11, 6.762438E+11, 6.928302E+11, 7.097860E+11, 7.271188E+11, 7.448359E+11, 
#   7.629452E+11, 7.814543E+11, 8.003712E+11, 8.197040E+11, 8.394610E+11, 8.596504E+11, 
#   8.802809E+11, 9.013610E+11, 9.228996E+11, 9.449056E+11, 9.673882E+11, 9.903566E+11, 
#   1.013820E+12, 1.037789E+12, 1.062272E+12, 1.087280E+12, 1.112823E+12, 1.138910E+12, 
#   1.165553E+12, 1.192762E+12, 1.220548E+12, 1.248922E+12, 1.277895E+12, 1.307479E+12, 
#   1.337684E+12, 1.368523E+12, 1.400008E+12, 1.432151E+12, 1.464964E+12, 1.498460E+12, 
#   1.532652E+12, 1.567552E+12, 1.603174E+12, 1.639531E+12, 1.676637E+12, 1.714506E+12, 
#   1.753151E+12, 1.792588E+12, 1.832830E+12, 1.873893E+12, 1.915792E+12, 1.958541E+12, 
#   2.002157E+12, 2.046655E+12, 2.092051E+12, 2.138362E+12, 2.185604E+12, 2.233794E+12, 
#   2.282949E+12, 2.333086E+12, 2.384224E+12, 2.436380E+12, 2.489571E+12, 2.543818E+12, 
#   2.599138E+12, 2.655551E+12, 2.713076E+12, 2.771732E+12, 2.831541E+12, 2.892522E+12, 
#   2.954695E+12, 3.018082E+12, 3.082705E+12, 3.148584E+12, 3.215742E+12, 3.284201E+12, 
#   3.353984E+12, 3.425113E+12, 3.497613E+12, 3.571506E+12, 3.646818E+12, 3.723572E+12, 
#   3.801792E+12, 3.881506E+12, 3.962738E+12, 4.045513E+12, 4.129860E+12, 4.215803E+12, 
#   4.303371E+12, 4.392591E+12, 4.483491E+12, 4.576100E+12, 4.670446E+12, 4.766559E+12, 
#   4.864468E+12, 4.964204E+12, 5.065797E+12, 5.169279E+12, 5.274681E+12, 5.382035E+12, 
#   5.491373E+12, 5.602729E+12, 5.716136E+12, 5.831628E+12, 5.949240E+12, 6.069006E+12, 
#   6.190962E+12, 6.315144E+12, 6.441588E+12, 6.570332E+12, 6.701413E+12, 6.834869E+12, 
#   6.970739E+12, 7.109063E+12, 7.249879E+12, 7.393230E+12, 7.539154E+12, 7.687695E+12, 
#   7.838894E+12, 7.992794E+12, 8.149438E+12, 
#])
# ============================ M = 45, I = 1 ============================
M = 45
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.000000E+00, 1.000000E+00, 1.001788E+00, 1.030647E+00, 1.126881E+00, 1.297714E+00, 
   1.526206E+00, 1.791557E+00, 2.077149E+00, 2.371603E+00, 2.667730E+00, 2.961282E+00, 
   3.249956E+00, 3.532703E+00, 3.809254E+00, 4.079808E+00, 4.344827E+00, 4.604900E+00, 
   4.860660E+00, 5.112725E+00, 5.361676E+00, 5.608036E+00, 5.852264E+00, 6.094758E+00, 
   6.335855E+00, 6.575840E+00, 6.814950E+00, 7.053379E+00, 7.291287E+00, 7.528804E+00, 
   7.766034E+00, 8.003064E+00, 8.239959E+00, 8.476774E+00, 8.713552E+00, 8.950325E+00, 
   9.187121E+00, 9.423959E+00, 9.660856E+00, 9.897824E+00, 1.013487E+01, 1.037201E+01, 
   1.060924E+01, 1.084656E+01, 1.108399E+01, 1.132151E+01, 1.155914E+01, 1.179688E+01, 
   1.203472E+01, 1.227266E+01, 1.251071E+01, 1.274887E+01, 1.298713E+01, 1.322550E+01, 
   1.346397E+01, 1.370255E+01, 1.394124E+01, 1.418004E+01, 1.441895E+01, 1.465796E+01, 
   1.489709E+01, 1.513633E+01, 1.537567E+01, 1.561514E+01, 1.585471E+01, 1.609441E+01, 
   1.633422E+01, 1.657415E+01, 1.681420E+01, 1.705438E+01, 1.729468E+01, 1.753511E+01, 
   1.777567E+01, 1.801636E+01, 1.825719E+01, 1.849816E+01, 1.873927E+01, 1.898053E+01, 
   1.922193E+01, 1.946348E+01, 1.970520E+01, 1.994707E+01, 2.018910E+01, 2.043130E+01, 
   2.067367E+01, 2.091622E+01, 2.115894E+01, 2.140185E+01, 2.164494E+01, 2.188823E+01, 
   2.213171E+01, 2.237540E+01, 2.261929E+01, 2.286339E+01, 2.310771E+01, 2.335225E+01, 
   2.359701E+01, 2.384201E+01, 2.408724E+01, 2.433272E+01, 2.457844E+01, 2.482441E+01, 
   2.507064E+01, 2.531713E+01, 2.556390E+01, 2.581093E+01, 2.605825E+01, 2.630585E+01, 
   2.655374E+01, 2.680193E+01, 2.705042E+01, 2.729921E+01, 2.754832E+01, 2.779776E+01, 
   2.804751E+01, 2.829760E+01, 2.854802E+01, 2.879879E+01, 2.904991E+01, 2.930138E+01, 
   2.955321E+01, 2.980540E+01, 3.005797E+01, 3.031092E+01, 3.056425E+01, 3.081797E+01, 
   3.107208E+01, 3.132660E+01, 3.158152E+01, 3.183686E+01, 3.209261E+01, 3.234879E+01, 
   3.260540E+01, 3.286245E+01, 3.311994E+01, 3.337788E+01, 3.363627E+01, 3.389511E+01, 
   3.415443E+01, 3.441421E+01, 3.467448E+01, 3.493522E+01, 3.519645E+01, 3.545817E+01, 
   3.572040E+01, 3.598312E+01, 3.624636E+01, 3.651011E+01, 3.677439E+01, 3.703919E+01, 
   3.730452E+01, 3.757039E+01, 3.783680E+01, 3.810376E+01, 3.837127E+01, 3.863934E+01, 
   3.890798E+01, 3.917718E+01, 3.944696E+01, 3.971732E+01, 3.998826E+01, 4.025979E+01, 
   4.053192E+01, 4.080465E+01, 4.107798E+01, 4.135192E+01, 4.162648E+01, 4.190165E+01, 
   4.217746E+01, 4.245389E+01, 4.273095E+01, 4.300866E+01, 4.328701E+01, 4.356600E+01, 
   4.384566E+01, 4.412597E+01, 4.440694E+01, 4.468858E+01, 4.497089E+01, 4.525388E+01, 
   4.553755E+01, 4.582191E+01, 4.610696E+01, 4.639270E+01, 4.667914E+01, 4.696629E+01, 
   4.725414E+01, 4.754270E+01, 4.783199E+01, 4.812199E+01, 4.841272E+01, 4.870418E+01, 
   4.899637E+01, 4.928930E+01, 4.958297E+01, 4.987739E+01, 5.017256E+01, 5.046848E+01, 
   5.076516E+01, 5.106260E+01, 5.136081E+01, 5.165979E+01, 5.195955E+01, 5.226008E+01, 
   5.256139E+01, 5.286349E+01, 5.316638E+01, 5.347006E+01, 5.377454E+01, 5.407982E+01, 
   5.438591E+01, 5.469280E+01, 5.500051E+01, 5.530903E+01, 5.561837E+01, 5.592853E+01, 
   5.623952E+01, 5.655134E+01, 5.686400E+01, 5.717749E+01, 5.749182E+01, 5.780699E+01, 
   5.812302E+01, 5.843989E+01, 5.875762E+01, 5.907620E+01, 5.939565E+01, 5.971596E+01, 
   6.003713E+01, 6.035918E+01, 6.068210E+01, 6.100590E+01, 6.133059E+01, 6.165615E+01, 
   6.198260E+01, 6.230994E+01, 6.263818E+01, 6.296731E+01, 6.329734E+01, 6.362827E+01, 
   6.396011E+01, 6.429286E+01, 6.462652E+01, 6.496109E+01, 6.529658E+01, 6.563299E+01, 
   6.597033E+01, 6.630859E+01, 6.664778E+01, 6.698791E+01, 6.732897E+01, 6.767097E+01, 
   6.801391E+01, 6.835779E+01, 6.870262E+01, 6.904840E+01, 6.939513E+01, 6.974282E+01, 
   7.009147E+01, 7.044107E+01, 7.079164E+01, 7.114318E+01, 7.149568E+01, 7.184916E+01, 
   7.220361E+01, 7.255904E+01, 7.291544E+01, 7.327283E+01, 7.363121E+01, 7.399057E+01, 
   7.435092E+01, 7.471227E+01, 7.507461E+01, 7.543794E+01, 7.580228E+01, 7.616762E+01, 
   7.653397E+01, 7.690132E+01, 7.726969E+01, 7.763906E+01, 7.800946E+01, 7.838087E+01, 
   7.875330E+01, 7.912675E+01, 7.950123E+01, 7.987674E+01, 8.025328E+01, 8.063085E+01, 
   8.100945E+01, 8.138909E+01, 8.176978E+01, 8.215150E+01, 8.253427E+01, 8.291808E+01, 
   8.330295E+01, 8.368886E+01, 8.407583E+01, 8.446386E+01, 8.485294E+01, 8.524308E+01, 
   8.563429E+01, 8.602656E+01, 8.641990E+01, 8.681431E+01, 8.720979E+01, 8.760635E+01, 
   8.800398E+01, 8.840269E+01, 8.880249E+01, 8.920336E+01, 8.960532E+01, 9.000837E+01, 
   9.041251E+01, 9.081774E+01, 9.122406E+01, 9.163148E+01, 9.204000E+01, 9.244962E+01, 
   9.286034E+01, 9.327217E+01, 9.368510E+01, 9.409914E+01, 9.451430E+01, 9.493057E+01, 
   9.534795E+01, 9.576645E+01, 9.618607E+01, 9.660681E+01, 9.702868E+01, 9.745167E+01, 
   9.787579E+01, 9.830104E+01, 9.872743E+01, 9.915494E+01, 9.958360E+01, 1.000134E+02, 
   1.004443E+02, 1.008764E+02, 1.013096E+02, 1.017440E+02, 1.021795E+02, 1.026162E+02, 
   1.030540E+02, 1.034930E+02, 1.039331E+02, 1.043744E+02, 1.048168E+02, 1.052604E+02, 
   1.057052E+02, 1.061512E+02, 1.065983E+02, 1.070465E+02, 1.074960E+02, 1.079466E+02, 
   1.083984E+02, 1.088514E+02, 1.093056E+02, 1.097609E+02, 1.102175E+02, 1.106752E+02, 
   1.111341E+02, 1.115942E+02, 1.120555E+02, 1.125180E+02, 1.129816E+02, 1.134465E+02, 
   1.139126E+02, 1.143799E+02, 1.148484E+02, 1.153181E+02, 1.157890E+02, 1.162611E+02, 
   1.167344E+02, 1.172089E+02, 1.176847E+02, 1.181616E+02, 1.186398E+02, 1.191192E+02, 
   1.195998E+02, 1.200817E+02, 1.205648E+02, 1.210491E+02, 1.215346E+02, 1.220214E+02, 
   1.225094E+02, 1.229986E+02, 1.234891E+02, 1.239808E+02, 1.244738E+02, 1.249680E+02, 
   1.254635E+02, 1.259602E+02, 1.264581E+02, 1.269573E+02, 1.274578E+02, 1.279595E+02, 
   1.284624E+02, 1.289666E+02, 1.294721E+02, 1.299789E+02, 1.304869E+02, 1.309961E+02, 
   1.315067E+02, 1.320185E+02, 1.325316E+02, 1.330459E+02, 1.335616E+02, 1.340785E+02, 
   1.345967E+02, 1.351161E+02, 1.356369E+02, 1.361589E+02, 1.366822E+02, 1.372068E+02, 
   1.377327E+02, 1.382599E+02, 1.387884E+02, 1.393182E+02, 1.398492E+02, 1.403816E+02, 
   1.409153E+02, 1.414503E+02, 1.419865E+02, 1.425241E+02, 1.430630E+02, 1.436032E+02, 
   1.441447E+02, 1.446875E+02, 1.452317E+02, 1.457771E+02, 1.463239E+02, 1.468720E+02, 
   1.474214E+02, 1.479722E+02, 1.485242E+02, 1.490776E+02, 1.496323E+02, 1.501884E+02, 
   1.507458E+02, 1.513045E+02, 1.518645E+02, 1.524259E+02, 1.529887E+02, 1.535527E+02, 
   1.541181E+02, 1.546849E+02, 1.552530E+02, 1.558224E+02, 1.563932E+02, 1.569654E+02, 
   1.575389E+02, 1.581138E+02, 1.586900E+02, 1.592675E+02, 1.598465E+02, 1.604268E+02, 
   1.610084E+02, 1.615915E+02, 1.621758E+02, 1.627616E+02, 1.633487E+02, 1.639372E+02, 
   1.645271E+02, 1.651183E+02, 1.657110E+02, 1.663050E+02, 1.669004E+02, 1.674971E+02, 
   1.680953E+02, 1.686948E+02, 1.692957E+02, 1.698980E+02, 1.705017E+02, 1.711068E+02, 
   1.717133E+02, 1.723212E+02, 1.729305E+02, 1.735411E+02, 1.741532E+02, 1.747667E+02, 
   1.753815E+02, 1.759978E+02, 1.766155E+02, 1.772346E+02, 1.778551E+02, 1.784770E+02, 
   1.791004E+02, 1.797251E+02, 1.803513E+02, 1.809788E+02, 1.816078E+02, 1.822382E+02, 
   1.828701E+02, 1.835033E+02, 1.841380E+02, 1.847741E+02, 1.854117E+02, 1.860506E+02, 
   1.866910E+02, 1.873329E+02, 1.879761E+02, 1.886208E+02, 1.892670E+02, 1.899146E+02, 
   1.905636E+02, 1.912140E+02, 1.918659E+02, 1.925193E+02, 1.931741E+02, 1.938303E+02, 
   1.944880E+02, 1.951472E+02, 1.958078E+02, 1.964698E+02, 1.971333E+02, 1.977983E+02, 
   1.984647E+02, 1.991326E+02, 1.998019E+02, 2.004727E+02, 2.011450E+02, 2.018187E+02, 
   2.024939E+02, 2.031706E+02, 2.038487E+02, 2.045283E+02, 2.052094E+02, 2.058919E+02, 
   2.065760E+02, 2.072615E+02, 2.079485E+02, 2.086369E+02, 2.093269E+02, 2.100183E+02, 
   2.107112E+02, 2.114056E+02, 2.121014E+02, 2.127988E+02, 2.134977E+02, 2.141980E+02, 
   2.148998E+02, 2.156032E+02, 2.163080E+02, 2.170143E+02, 2.177221E+02, 2.184314E+02, 
   2.191422E+02, 2.198545E+02, 2.205683E+02, 2.212836E+02, 2.220004E+02, 2.227187E+02, 
   2.234386E+02, 2.241599E+02, 2.248827E+02, 2.256071E+02, 2.263329E+02, 2.270603E+02, 
   2.277892E+02, 2.285196E+02, 2.292515E+02, 2.299849E+02, 2.307199E+02, 2.314563E+02, 
   2.321943E+02, 2.329338E+02, 2.336749E+02, 2.344174E+02, 2.351615E+02, 2.359071E+02, 
   2.366543E+02, 2.374029E+02, 2.381531E+02, 2.389048E+02, 2.396581E+02, 2.404129E+02, 
   2.411692E+02, 2.419270E+02, 2.426864E+02, 2.434474E+02, 2.442098E+02, 2.449738E+02, 
   2.457394E+02, 2.465064E+02, 2.472751E+02, 2.480452E+02, 2.488169E+02, 2.495902E+02, 
   2.503650E+02, 2.511413E+02, 2.519192E+02, 2.526987E+02, 2.534797E+02, 2.542622E+02, 
   2.550463E+02, 2.558319E+02, 2.566191E+02, 2.574079E+02, 2.581982E+02, 2.589900E+02, 
   2.597835E+02, 
])
# ============================ M = 45, I = 2 ============================
M = 45
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[7]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.000000E+00, 6.000048E+00, 6.029347E+00, 6.249424E+00, 6.728818E+00, 7.394774E+00, 
   8.168283E+00, 9.000618E+00, 9.865998E+00, 1.075112E+01, 1.164890E+01, 1.255531E+01, 
   1.346792E+01, 1.438519E+01, 1.530608E+01, 1.622985E+01, 1.715597E+01, 1.808406E+01, 
   1.901383E+01, 1.994505E+01, 2.087753E+01, 2.181113E+01, 2.274575E+01, 2.368129E+01, 
   2.461767E+01, 2.555482E+01, 2.649271E+01, 2.743127E+01, 2.837047E+01, 2.931029E+01, 
   3.025068E+01, 3.119164E+01, 3.213312E+01, 3.307513E+01, 3.401765E+01, 3.496064E+01, 
   3.590412E+01, 3.684807E+01, 3.779247E+01, 3.873732E+01, 3.968262E+01, 4.062836E+01, 
   4.157454E+01, 4.252114E+01, 4.346818E+01, 4.441565E+01, 4.536356E+01, 4.631189E+01, 
   4.726067E+01, 4.820988E+01, 4.915955E+01, 5.010966E+01, 5.106023E+01, 5.201128E+01, 
   5.296280E+01, 5.391481E+01, 5.486733E+01, 5.582036E+01, 5.677392E+01, 5.772803E+01, 
   5.868270E+01, 5.963796E+01, 6.059382E+01, 6.155031E+01, 6.250744E+01, 6.346523E+01, 
   6.442372E+01, 6.538293E+01, 6.634288E+01, 6.730360E+01, 6.826512E+01, 6.922747E+01, 
   7.019067E+01, 7.115476E+01, 7.211977E+01, 7.308572E+01, 7.405266E+01, 7.502062E+01, 
   7.598962E+01, 7.695971E+01, 7.793091E+01, 7.890327E+01, 7.987682E+01, 8.085159E+01, 
   8.182762E+01, 8.280495E+01, 8.378361E+01, 8.476365E+01, 8.574509E+01, 8.672798E+01, 
   8.771234E+01, 8.869823E+01, 8.968568E+01, 9.067472E+01, 9.166539E+01, 9.265773E+01, 
   9.365178E+01, 9.464757E+01, 9.564515E+01, 9.664454E+01, 9.764579E+01, 9.864892E+01, 
   9.965399E+01, 1.006610E+02, 1.016701E+02, 1.026811E+02, 1.036943E+02, 1.047095E+02, 
   1.057269E+02, 1.067465E+02, 1.077682E+02, 1.087923E+02, 1.098186E+02, 1.108472E+02, 
   1.118781E+02, 1.129115E+02, 1.139472E+02, 1.149854E+02, 1.160261E+02, 1.170692E+02, 
   1.181150E+02, 1.191632E+02, 1.202141E+02, 1.212676E+02, 1.223238E+02, 1.233826E+02, 
   1.244442E+02, 1.255085E+02, 1.265756E+02, 1.276455E+02, 1.287182E+02, 1.297938E+02, 
   1.308723E+02, 1.319537E+02, 1.330380E+02, 1.341253E+02, 1.352155E+02, 1.363088E+02, 
   1.374051E+02, 1.385045E+02, 1.396070E+02, 1.407126E+02, 1.418213E+02, 1.429331E+02, 
   1.440482E+02, 1.451665E+02, 1.462879E+02, 1.474127E+02, 1.485407E+02, 1.496720E+02, 
   1.508066E+02, 1.519446E+02, 1.530859E+02, 1.542305E+02, 1.553786E+02, 1.565301E+02, 
   1.576850E+02, 1.588434E+02, 1.600053E+02, 1.611706E+02, 1.623395E+02, 1.635119E+02, 
   1.646878E+02, 1.658673E+02, 1.670504E+02, 1.682371E+02, 1.694274E+02, 1.706214E+02, 
   1.718190E+02, 1.730203E+02, 1.742252E+02, 1.754339E+02, 1.766463E+02, 1.778624E+02, 
   1.790823E+02, 1.803059E+02, 1.815333E+02, 1.827645E+02, 1.839996E+02, 1.852384E+02, 
   1.864811E+02, 1.877277E+02, 1.889781E+02, 1.902324E+02, 1.914906E+02, 1.927527E+02, 
   1.940188E+02, 1.952888E+02, 1.965627E+02, 1.978406E+02, 1.991225E+02, 2.004084E+02, 
   2.016983E+02, 2.029922E+02, 2.042901E+02, 2.055921E+02, 2.068981E+02, 2.082082E+02, 
   2.095224E+02, 2.108407E+02, 2.121631E+02, 2.134896E+02, 2.148202E+02, 2.161550E+02, 
   2.174939E+02, 2.188370E+02, 2.201843E+02, 2.215357E+02, 2.228913E+02, 2.242512E+02, 
   2.256153E+02, 2.269836E+02, 2.283561E+02, 2.297329E+02, 2.311140E+02, 2.324993E+02, 
   2.338889E+02, 2.352829E+02, 2.366811E+02, 2.380836E+02, 2.394905E+02, 2.409017E+02, 
   2.423172E+02, 2.437371E+02, 2.451614E+02, 2.465900E+02, 2.480230E+02, 2.494604E+02, 
   2.509023E+02, 2.523485E+02, 2.537992E+02, 2.552543E+02, 2.567138E+02, 2.581778E+02, 
   2.596462E+02, 2.611192E+02, 2.625966E+02, 2.640785E+02, 2.655649E+02, 2.670558E+02, 
   2.685512E+02, 2.700511E+02, 2.715556E+02, 2.730646E+02, 2.745782E+02, 2.760964E+02, 
   2.776191E+02, 2.791464E+02, 2.806782E+02, 2.822147E+02, 2.837558E+02, 2.853015E+02, 
   2.868518E+02, 2.884068E+02, 2.899664E+02, 2.915306E+02, 2.930995E+02, 2.946731E+02, 
   2.962513E+02, 2.978343E+02, 2.994219E+02, 3.010142E+02, 3.026112E+02, 3.042130E+02, 
   3.058195E+02, 3.074307E+02, 3.090466E+02, 3.106673E+02, 3.122928E+02, 3.139230E+02, 
   3.155580E+02, 3.171978E+02, 3.188423E+02, 3.204917E+02, 3.221459E+02, 3.238049E+02, 
   3.254687E+02, 3.271374E+02, 3.288109E+02, 3.304892E+02, 3.321724E+02, 3.338605E+02, 
   3.355534E+02, 3.372512E+02, 3.389539E+02, 3.406615E+02, 3.423740E+02, 3.440914E+02, 
   3.458138E+02, 3.475410E+02, 3.492732E+02, 3.510104E+02, 3.527525E+02, 3.544995E+02, 
   3.562516E+02, 3.580086E+02, 3.597706E+02, 3.615375E+02, 3.633095E+02, 3.650865E+02, 
   3.668685E+02, 3.686555E+02, 3.704476E+02, 3.722447E+02, 3.740468E+02, 3.758540E+02, 
   3.776663E+02, 3.794836E+02, 3.813061E+02, 3.831336E+02, 3.849662E+02, 3.868038E+02, 
   3.886467E+02, 3.904946E+02, 3.923476E+02, 3.942058E+02, 3.960692E+02, 3.979376E+02, 
   3.998113E+02, 4.016901E+02, 4.035740E+02, 4.054632E+02, 4.073576E+02, 4.092571E+02, 
   4.111618E+02, 4.130718E+02, 4.149870E+02, 4.169074E+02, 4.188330E+02, 4.207639E+02, 
   4.227001E+02, 4.246415E+02, 4.265882E+02, 4.285401E+02, 4.304974E+02, 4.324599E+02, 
   4.344277E+02, 4.364008E+02, 4.383793E+02, 4.403631E+02, 4.423522E+02, 4.443466E+02, 
   4.463464E+02, 4.483516E+02, 4.503621E+02, 4.523780E+02, 4.543992E+02, 4.564259E+02, 
   4.584579E+02, 4.604954E+02, 4.625383E+02, 4.645865E+02, 4.666403E+02, 4.686994E+02, 
   4.707640E+02, 4.728340E+02, 4.749095E+02, 4.769905E+02, 4.790770E+02, 4.811689E+02, 
   4.832663E+02, 4.853692E+02, 4.874777E+02, 4.895916E+02, 4.917111E+02, 4.938361E+02, 
   4.959667E+02, 4.981028E+02, 5.002444E+02, 5.023916E+02, 5.045444E+02, 5.067028E+02, 
   5.088667E+02, 5.110363E+02, 5.132115E+02, 5.153923E+02, 5.175787E+02, 5.197707E+02, 
   5.219684E+02, 5.241717E+02, 5.263806E+02, 5.285953E+02, 5.308156E+02, 5.330416E+02, 
   5.352732E+02, 5.375106E+02, 5.397537E+02, 5.420025E+02, 5.442570E+02, 5.465172E+02, 
   5.487832E+02, 5.510549E+02, 5.533323E+02, 5.556156E+02, 5.579046E+02, 5.601994E+02, 
   5.624999E+02, 5.648063E+02, 5.671184E+02, 5.694364E+02, 5.717602E+02, 5.740898E+02, 
   5.764253E+02, 5.787666E+02, 5.811137E+02, 5.834667E+02, 5.858256E+02, 5.881904E+02, 
   5.905610E+02, 5.929376E+02, 5.953200E+02, 5.977084E+02, 6.001026E+02, 6.025028E+02, 
   6.049090E+02, 6.073211E+02, 6.097391E+02, 6.121631E+02, 6.145930E+02, 6.170290E+02, 
   6.194709E+02, 6.219188E+02, 6.243727E+02, 6.268327E+02, 6.292986E+02, 6.317706E+02, 
   6.342486E+02, 6.367326E+02, 6.392228E+02, 6.417189E+02, 6.442212E+02, 6.467295E+02, 
   6.492439E+02, 6.517643E+02, 6.542909E+02, 6.568236E+02, 6.593625E+02, 6.619074E+02, 
   6.644585E+02, 6.670157E+02, 6.695791E+02, 6.721486E+02, 6.747243E+02, 6.773062E+02, 
   6.798943E+02, 6.824885E+02, 6.850890E+02, 6.876956E+02, 6.903085E+02, 6.929276E+02, 
   6.955530E+02, 6.981845E+02, 7.008224E+02, 7.034665E+02, 7.061168E+02, 7.087735E+02, 
   7.114364E+02, 7.141056E+02, 7.167811E+02, 7.194629E+02, 7.221510E+02, 7.248455E+02, 
   7.275463E+02, 7.302534E+02, 7.329669E+02, 7.356867E+02, 7.384129E+02, 7.411455E+02, 
   7.438844E+02, 7.466297E+02, 7.493815E+02, 7.521396E+02, 7.549042E+02, 7.576751E+02, 
   7.604525E+02, 7.632364E+02, 7.660266E+02, 7.688234E+02, 7.716266E+02, 7.744362E+02, 
   7.772523E+02, 7.800750E+02, 7.829041E+02, 7.857397E+02, 7.885818E+02, 7.914304E+02, 
   7.942856E+02, 7.971473E+02, 8.000155E+02, 8.028903E+02, 8.057716E+02, 8.086595E+02, 
   8.115539E+02, 8.144550E+02, 8.173626E+02, 8.202768E+02, 8.231976E+02, 8.261250E+02, 
   8.290591E+02, 8.319997E+02, 8.349470E+02, 8.379010E+02, 8.408615E+02, 8.438288E+02, 
   8.468026E+02, 8.497832E+02, 8.527704E+02, 8.557644E+02, 8.587650E+02, 8.617723E+02, 
   8.647863E+02, 8.678070E+02, 8.708345E+02, 8.738686E+02, 8.769095E+02, 8.799572E+02, 
   8.830116E+02, 8.860727E+02, 8.891406E+02, 8.922153E+02, 8.952968E+02, 8.983851E+02, 
   9.014801E+02, 9.045819E+02, 9.076906E+02, 9.108060E+02, 9.139283E+02, 9.170574E+02, 
   9.201934E+02, 9.233362E+02, 9.264858E+02, 9.296423E+02, 9.328056E+02, 9.359758E+02, 
   9.391529E+02, 9.423369E+02, 9.455278E+02, 9.487255E+02, 9.519302E+02, 9.551417E+02, 
   9.583602E+02, 9.615856E+02, 9.648179E+02, 9.680572E+02, 9.713034E+02, 9.745566E+02, 
   9.778167E+02, 9.810837E+02, 9.843578E+02, 9.876388E+02, 9.909268E+02, 9.942217E+02, 
   9.975237E+02, 1.000833E+03, 1.004149E+03, 1.007472E+03, 1.010802E+03, 1.014139E+03, 
   1.017483E+03, 1.020834E+03, 1.024192E+03, 1.027557E+03, 1.030929E+03, 1.034309E+03, 
   1.037695E+03, 1.041088E+03, 1.044489E+03, 1.047896E+03, 1.051311E+03, 1.054733E+03, 
   1.058162E+03, 1.061597E+03, 1.065041E+03, 1.068491E+03, 1.071948E+03, 1.075412E+03, 
   1.078884E+03, 1.082363E+03, 1.085849E+03, 1.089341E+03, 1.092842E+03, 1.096349E+03, 
   1.099863E+03, 1.103385E+03, 1.106914E+03, 1.110450E+03, 1.113993E+03, 1.117543E+03, 
   1.121101E+03, 1.124665E+03, 1.128237E+03, 1.131817E+03, 1.135403E+03, 1.138996E+03, 
   1.142597E+03, 1.146205E+03, 1.149821E+03, 1.153443E+03, 1.157073E+03, 1.160710E+03, 
   1.164354E+03, 
])
# ============================ M = 46, I = 1 ============================
M = 46
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.290490E+00, 8.848274E+00, 1.735237E+01, 2.586041E+01, 3.437036E+01, 4.288179E+01, 
   5.139459E+01, 5.990868E+01, 6.842405E+01, 7.694069E+01, 8.545856E+01, 9.397769E+01, 
   1.024981E+02, 1.110197E+02, 1.195427E+02, 1.280672E+02, 1.365933E+02, 1.451214E+02, 
   1.536520E+02, 1.621857E+02, 1.707234E+02, 1.792661E+02, 1.878151E+02, 1.963719E+02, 
   2.049382E+02, 2.135158E+02, 2.221069E+02, 2.307134E+02, 2.393378E+02, 2.479824E+02, 
   2.566497E+02, 2.653421E+02, 2.740623E+02, 2.828127E+02, 2.915958E+02, 3.004143E+02, 
   3.092705E+02, 3.181669E+02, 3.271058E+02, 3.360895E+02, 3.451203E+02, 3.542004E+02, 
   3.633317E+02, 3.725165E+02, 3.817564E+02, 3.910535E+02, 4.004095E+02, 4.098261E+02, 
   4.193048E+02, 4.288474E+02, 4.384552E+02, 4.481296E+02, 4.578720E+02, 4.676837E+02, 
   4.775659E+02, 4.875197E+02, 4.975462E+02, 5.076465E+02, 5.178215E+02, 5.280723E+02, 
   5.383997E+02, 5.488045E+02, 5.592877E+02, 5.698498E+02, 5.804917E+02, 5.912141E+02, 
   6.020176E+02, 6.129029E+02, 6.238705E+02, 6.349210E+02, 6.460550E+02, 6.572730E+02, 
   6.685754E+02, 6.799628E+02, 6.914355E+02, 7.029941E+02, 7.146388E+02, 7.263701E+02, 
   7.381885E+02, 7.500941E+02, 7.620874E+02, 7.741686E+02, 7.863381E+02, 7.985962E+02, 
   8.109431E+02, 8.233791E+02, 8.359046E+02, 8.485196E+02, 8.612245E+02, 8.740195E+02, 
   8.869047E+02, 8.998805E+02, 9.129470E+02, 9.261043E+02, 9.393528E+02, 9.526924E+02, 
   9.661236E+02, 9.796463E+02, 9.932607E+02, 1.006967E+03, 1.020765E+03, 1.034656E+03, 
   1.048639E+03, 1.062714E+03, 1.076882E+03, 1.091143E+03, 1.105496E+03, 1.119943E+03, 
   1.134482E+03, 1.149114E+03, 1.163840E+03, 1.178659E+03, 1.193572E+03, 1.208578E+03, 
   1.223678E+03, 1.238871E+03, 1.254158E+03, 1.269539E+03, 1.285014E+03, 1.300584E+03, 
   1.316247E+03, 1.332004E+03, 1.347856E+03, 1.363802E+03, 1.379842E+03, 1.395977E+03, 
   1.412207E+03, 1.428531E+03, 1.444949E+03, 1.461463E+03, 1.478071E+03, 1.494774E+03, 
   1.511572E+03, 1.528465E+03, 1.545453E+03, 1.562536E+03, 1.579714E+03, 1.596987E+03, 
   1.614355E+03, 1.631819E+03, 1.649378E+03, 1.667032E+03, 1.684782E+03, 1.702627E+03, 
   1.720567E+03, 1.738603E+03, 1.756735E+03, 1.774962E+03, 1.793285E+03, 1.811704E+03, 
   1.830218E+03, 1.848828E+03, 1.867534E+03, 1.886336E+03, 1.905233E+03, 1.924227E+03, 
   1.943316E+03, 1.962502E+03, 1.981784E+03, 2.001161E+03, 2.020635E+03, 2.040205E+03, 
   2.059870E+03, 2.079633E+03, 2.099491E+03, 2.119446E+03, 2.139497E+03, 2.159644E+03, 
   2.179887E+03, 2.200227E+03, 2.220664E+03, 2.241197E+03, 2.261826E+03, 2.282552E+03, 
   2.303374E+03, 2.324293E+03, 2.345309E+03, 2.366421E+03, 2.387630E+03, 2.408935E+03, 
   2.430337E+03, 2.451836E+03, 2.473432E+03, 2.495124E+03, 2.516914E+03, 2.538800E+03, 
   2.560783E+03, 2.582863E+03, 2.605040E+03, 2.627314E+03, 2.649684E+03, 2.672152E+03, 
   2.694717E+03, 2.717379E+03, 2.740138E+03, 2.762994E+03, 2.785947E+03, 2.808997E+03, 
   2.832145E+03, 2.855389E+03, 2.878731E+03, 2.902170E+03, 2.925707E+03, 2.949341E+03, 
   2.973072E+03, 2.996900E+03, 3.020826E+03, 3.044849E+03, 3.068970E+03, 3.093188E+03, 
   3.117503E+03, 3.141916E+03, 3.166427E+03, 3.191035E+03, 3.215740E+03, 3.240544E+03, 
   3.265444E+03, 3.290443E+03, 3.315539E+03, 3.340733E+03, 3.366024E+03, 3.391413E+03, 
   3.416900E+03, 3.442485E+03, 3.468167E+03, 3.493947E+03, 3.519825E+03, 3.545801E+03, 
   3.571875E+03, 3.598047E+03, 3.624316E+03, 3.650684E+03, 3.677149E+03, 3.703713E+03, 
   3.730374E+03, 3.757134E+03, 3.783991E+03, 3.810947E+03, 3.838001E+03, 3.865152E+03, 
   3.892402E+03, 3.919751E+03, 3.947197E+03, 3.974741E+03, 4.002384E+03, 4.030125E+03, 
   4.057964E+03, 4.085901E+03, 4.113937E+03, 4.142071E+03, 4.170304E+03, 4.198635E+03, 
   4.227064E+03, 4.255591E+03, 4.284217E+03, 4.312942E+03, 4.341765E+03, 4.370686E+03, 
   4.399706E+03, 4.428824E+03, 4.458042E+03, 4.487357E+03, 4.516771E+03, 4.546284E+03, 
   4.575895E+03, 4.605605E+03, 4.635414E+03, 4.665322E+03, 4.695328E+03, 4.725433E+03, 
   4.755636E+03, 4.785939E+03, 4.816340E+03, 4.846840E+03, 4.877439E+03, 4.908136E+03, 
   4.938933E+03, 4.969829E+03, 5.000823E+03, 5.031916E+03, 5.063109E+03, 5.094400E+03, 
   5.125790E+03, 5.157279E+03, 5.188868E+03, 5.220555E+03, 5.252342E+03, 5.284227E+03, 
   5.316212E+03, 5.348296E+03, 5.380479E+03, 5.412761E+03, 5.445142E+03, 5.477623E+03, 
   5.510202E+03, 5.542881E+03, 5.575660E+03, 5.608537E+03, 5.641514E+03, 5.674591E+03, 
   5.707766E+03, 5.741041E+03, 5.774416E+03, 5.807889E+03, 5.841463E+03, 5.875135E+03, 
   5.908907E+03, 5.942779E+03, 5.976750E+03, 6.010821E+03, 6.044991E+03, 6.079261E+03, 
   6.113630E+03, 6.148099E+03, 6.182668E+03, 6.217336E+03, 6.252104E+03, 6.286972E+03, 
   6.321939E+03, 6.357006E+03, 6.392173E+03, 6.427440E+03, 6.462806E+03, 6.498272E+03, 
   6.533838E+03, 6.569504E+03, 6.605270E+03, 6.641136E+03, 6.677101E+03, 6.713167E+03, 
   6.749332E+03, 6.785597E+03, 6.821962E+03, 6.858428E+03, 6.894993E+03, 6.931658E+03, 
   6.968424E+03, 7.005289E+03, 7.042255E+03, 7.079321E+03, 7.116486E+03, 7.153752E+03, 
   7.191119E+03, 7.228585E+03, 7.266151E+03, 7.303818E+03, 7.341585E+03, 7.379453E+03, 
   7.417420E+03, 7.455488E+03, 7.493656E+03, 7.531925E+03, 7.570294E+03, 7.608763E+03, 
   7.647333E+03, 7.686003E+03, 7.724774E+03, 7.763645E+03, 7.802616E+03, 7.841688E+03, 
   7.880861E+03, 7.920134E+03, 7.959507E+03, 7.998982E+03, 8.038556E+03, 8.078232E+03, 
   8.118008E+03, 8.157884E+03, 8.197862E+03, 8.237940E+03, 8.278119E+03, 8.318398E+03, 
   8.358778E+03, 8.399259E+03, 8.439841E+03, 8.480524E+03, 8.521307E+03, 8.562191E+03, 
   8.603176E+03, 8.644262E+03, 8.685449E+03, 8.726737E+03, 8.768126E+03, 8.809615E+03, 
   8.851206E+03, 8.892897E+03, 8.934690E+03, 8.976584E+03, 9.018578E+03, 9.060674E+03, 
   9.102871E+03, 9.145169E+03, 9.187568E+03, 9.230068E+03, 9.272670E+03, 9.315372E+03, 
   9.358176E+03, 9.401081E+03, 9.444087E+03, 9.487195E+03, 9.530404E+03, 9.573714E+03, 
   9.617125E+03, 9.660638E+03, 9.704252E+03, 9.747967E+03, 9.791784E+03, 9.835702E+03, 
   9.879722E+03, 9.923843E+03, 9.968066E+03, 1.001239E+04, 1.005682E+04, 1.010134E+04, 
   1.014597E+04, 1.019070E+04, 1.023553E+04, 1.028047E+04, 1.032550E+04, 1.037064E+04, 
   1.041588E+04, 1.046122E+04, 1.050666E+04, 1.055220E+04, 1.059784E+04, 1.064359E+04, 
   1.068944E+04, 1.073539E+04, 1.078144E+04, 1.082760E+04, 1.087385E+04, 1.092021E+04, 
   1.096667E+04, 1.101323E+04, 1.105989E+04, 1.110666E+04, 1.115353E+04, 1.120050E+04, 
   1.124757E+04, 1.129474E+04, 1.134201E+04, 1.138939E+04, 1.143687E+04, 1.148445E+04, 
   1.153214E+04, 1.157992E+04, 1.162781E+04, 1.167580E+04, 1.172389E+04, 1.177208E+04, 
   1.182038E+04, 1.186878E+04, 1.191728E+04, 1.196588E+04, 1.201459E+04, 1.206340E+04, 
   1.211231E+04, 1.216132E+04, 1.221043E+04, 1.225965E+04, 1.230897E+04, 1.235839E+04, 
   1.240792E+04, 1.245754E+04, 1.250727E+04, 1.255710E+04, 1.260704E+04, 1.265708E+04, 
   1.270721E+04, 1.275746E+04, 1.280780E+04, 1.285825E+04, 1.290880E+04, 1.295945E+04, 
   1.301020E+04, 1.306106E+04, 1.311202E+04, 1.316308E+04, 1.321425E+04, 1.326552E+04, 
   1.331689E+04, 1.336836E+04, 1.341994E+04, 1.347162E+04, 1.352340E+04, 1.357529E+04, 
   1.362728E+04, 1.367937E+04, 1.373156E+04, 1.378386E+04, 1.383626E+04, 1.388876E+04, 
   1.394137E+04, 1.399408E+04, 1.404689E+04, 1.409980E+04, 1.415282E+04, 1.420594E+04, 
   1.425917E+04, 1.431249E+04, 1.436592E+04, 
])
# ============================ M = 46, I = 2 ============================
M = 46
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.301835E+00, 8.986958E+00, 1.763092E+01, 2.627977E+01, 3.493149E+01, 4.358568E+01, 
   5.224221E+01, 6.090102E+01, 6.956210E+01, 7.822542E+01, 8.689097E+01, 9.555875E+01, 
   1.042288E+02, 1.129011E+02, 1.215757E+02, 1.302528E+02, 1.389326E+02, 1.476154E+02, 
   1.563019E+02, 1.649925E+02, 1.736884E+02, 1.823906E+02, 1.911005E+02, 1.998196E+02, 
   2.085497E+02, 2.172928E+02, 2.260511E+02, 2.348267E+02, 2.436221E+02, 2.524398E+02, 
   2.612823E+02, 2.701523E+02, 2.790523E+02, 2.879850E+02, 2.969530E+02, 3.059589E+02, 
   3.150052E+02, 3.240945E+02, 3.332291E+02, 3.424115E+02, 3.516440E+02, 3.609287E+02, 
   3.702678E+02, 3.796633E+02, 3.891174E+02, 3.986318E+02, 4.082083E+02, 4.178488E+02, 
   4.275548E+02, 4.373280E+02, 4.471699E+02, 4.570818E+02, 4.670653E+02, 4.771216E+02, 
   4.872518E+02, 4.974573E+02, 5.077392E+02, 5.180984E+02, 5.285361E+02, 5.390532E+02, 
   5.496506E+02, 5.603291E+02, 5.710897E+02, 5.819332E+02, 5.928601E+02, 6.038714E+02, 
   6.149676E+02, 6.261495E+02, 6.374175E+02, 6.487724E+02, 6.602146E+02, 6.717448E+02, 
   6.833634E+02, 6.950709E+02, 7.068677E+02, 7.187544E+02, 7.307313E+02, 7.427988E+02, 
   7.549574E+02, 7.672073E+02, 7.795490E+02, 7.919828E+02, 8.045089E+02, 8.171278E+02, 
   8.298397E+02, 8.426449E+02, 8.555437E+02, 8.685363E+02, 8.816231E+02, 8.948041E+02, 
   9.080797E+02, 9.214501E+02, 9.349155E+02, 9.484761E+02, 9.621321E+02, 9.758838E+02, 
   9.897312E+02, 1.003675E+03, 1.017714E+03, 1.031850E+03, 1.046082E+03, 1.060411E+03, 
   1.074837E+03, 1.089360E+03, 1.103980E+03, 1.118697E+03, 1.133511E+03, 1.148423E+03, 
   1.163433E+03, 1.178540E+03, 1.193745E+03, 1.209048E+03, 1.224449E+03, 1.239948E+03, 
   1.255546E+03, 1.271242E+03, 1.287037E+03, 1.302930E+03, 1.318922E+03, 1.335013E+03, 
   1.351202E+03, 1.367491E+03, 1.383878E+03, 1.400365E+03, 1.416951E+03, 1.433637E+03, 
   1.450422E+03, 1.467306E+03, 1.484290E+03, 1.501374E+03, 1.518557E+03, 1.535840E+03, 
   1.553223E+03, 1.570706E+03, 1.588289E+03, 1.605972E+03, 1.623755E+03, 1.641639E+03, 
   1.659623E+03, 1.677707E+03, 1.695891E+03, 1.714176E+03, 1.732562E+03, 1.751048E+03, 
   1.769635E+03, 1.788323E+03, 1.807111E+03, 1.826000E+03, 1.844991E+03, 1.864082E+03, 
   1.883274E+03, 1.902567E+03, 1.921962E+03, 1.941457E+03, 1.961054E+03, 1.980752E+03, 
   2.000552E+03, 2.020453E+03, 2.040455E+03, 2.060559E+03, 2.080765E+03, 2.101072E+03, 
   2.121481E+03, 2.141991E+03, 2.162604E+03, 2.183318E+03, 2.204134E+03, 2.225052E+03, 
   2.246072E+03, 2.267194E+03, 2.288418E+03, 2.309744E+03, 2.331172E+03, 2.352703E+03, 
   2.374335E+03, 2.396070E+03, 2.417908E+03, 2.439848E+03, 2.461890E+03, 2.484035E+03, 
   2.506282E+03, 2.528632E+03, 2.551084E+03, 2.573640E+03, 2.596298E+03, 2.619058E+03, 
   2.641922E+03, 2.664888E+03, 2.687957E+03, 2.711129E+03, 2.734405E+03, 2.757783E+03, 
   2.781264E+03, 2.804848E+03, 2.828536E+03, 2.852327E+03, 2.876220E+03, 2.900218E+03, 
   2.924318E+03, 2.948522E+03, 2.972830E+03, 2.997240E+03, 3.021755E+03, 3.046372E+03, 
   3.071094E+03, 3.095919E+03, 3.120848E+03, 3.145880E+03, 3.171016E+03, 3.196256E+03, 
   3.221600E+03, 3.247048E+03, 3.272599E+03, 3.298255E+03, 3.324014E+03, 3.349878E+03, 
   3.375845E+03, 3.401917E+03, 3.428093E+03, 3.454373E+03, 3.480757E+03, 3.507246E+03, 
   3.533839E+03, 3.560536E+03, 3.587338E+03, 3.614244E+03, 3.641255E+03, 3.668370E+03, 
   3.695589E+03, 3.722914E+03, 3.750343E+03, 3.777876E+03, 3.805514E+03, 3.833258E+03, 
   3.861105E+03, 3.889058E+03, 3.917116E+03, 3.945278E+03, 3.973546E+03, 4.001918E+03, 
   4.030396E+03, 4.058978E+03, 4.087666E+03, 4.116459E+03, 4.145357E+03, 4.174360E+03, 
   4.203469E+03, 4.232683E+03, 4.262002E+03, 4.291426E+03, 4.320957E+03, 4.350592E+03, 
   4.380333E+03, 4.410180E+03, 4.440132E+03, 4.470190E+03, 4.500353E+03, 4.530622E+03, 
   4.560997E+03, 4.591478E+03, 4.622064E+03, 4.652757E+03, 4.683555E+03, 4.714459E+03, 
   4.745469E+03, 4.776586E+03, 4.807808E+03, 4.839136E+03, 4.870571E+03, 4.902111E+03, 
   4.933758E+03, 4.965511E+03, 4.997370E+03, 5.029336E+03, 5.061408E+03, 5.093587E+03, 
   5.125872E+03, 5.158263E+03, 5.190761E+03, 5.223365E+03, 5.256077E+03, 5.288894E+03, 
   5.321819E+03, 5.354850E+03, 5.387988E+03, 5.421232E+03, 5.454583E+03, 5.488042E+03, 
   5.521607E+03, 5.555279E+03, 5.589058E+03, 5.622944E+03, 5.656938E+03, 5.691038E+03, 
   5.725245E+03, 5.759560E+03, 5.793982E+03, 5.828511E+03, 5.863147E+03, 5.897891E+03, 
   5.932742E+03, 5.967700E+03, 6.002766E+03, 6.037940E+03, 6.073220E+03, 6.108609E+03, 
   6.144105E+03, 6.179708E+03, 6.215420E+03, 6.251239E+03, 6.287166E+03, 6.323200E+03, 
   6.359342E+03, 6.395593E+03, 6.431951E+03, 6.468417E+03, 6.504991E+03, 6.541673E+03, 
   6.578463E+03, 6.615361E+03, 6.652368E+03, 6.689482E+03, 6.726705E+03, 6.764036E+03, 
   6.801475E+03, 6.839023E+03, 6.876678E+03, 6.914443E+03, 6.952316E+03, 6.990297E+03, 
   7.028386E+03, 7.066585E+03, 7.104892E+03, 7.143307E+03, 7.181831E+03, 7.220464E+03, 
   7.259206E+03, 7.298056E+03, 7.337015E+03, 7.376083E+03, 7.415260E+03, 7.454546E+03, 
   7.493941E+03, 7.533445E+03, 7.573058E+03, 7.612780E+03, 7.652611E+03, 7.692551E+03, 
   7.732600E+03, 7.772759E+03, 7.813027E+03, 7.853404E+03, 7.893891E+03, 7.934487E+03, 
   7.975192E+03, 8.016007E+03, 8.056932E+03, 8.097966E+03, 8.139109E+03, 8.180362E+03, 
   8.221725E+03, 8.263198E+03, 8.304780E+03, 8.346472E+03, 8.388274E+03, 8.430185E+03, 
   8.472207E+03, 8.514338E+03, 8.556580E+03, 8.598931E+03, 8.641392E+03, 8.683964E+03, 
   8.726646E+03, 8.769437E+03, 8.812339E+03, 8.855352E+03, 8.898474E+03, 8.941707E+03, 
   8.985050E+03, 9.028504E+03, 9.072067E+03, 9.115742E+03, 9.159527E+03, 9.203422E+03, 
   9.247428E+03, 9.291545E+03, 9.335772E+03, 9.380110E+03, 9.424558E+03, 9.469118E+03, 
   9.513788E+03, 9.558569E+03, 9.603461E+03, 9.648464E+03, 9.693578E+03, 9.738802E+03, 
   9.784138E+03, 9.829585E+03, 9.875143E+03, 9.920812E+03, 9.966593E+03, 1.001248E+04, 
   1.005849E+04, 1.010460E+04, 1.015083E+04, 1.019716E+04, 1.024361E+04, 1.029017E+04, 
   1.033684E+04, 1.038362E+04, 1.043052E+04, 1.047752E+04, 1.052464E+04, 1.057187E+04, 
   1.061921E+04, 1.066667E+04, 1.071423E+04, 1.076191E+04, 1.080969E+04, 1.085759E+04, 
   1.090561E+04, 1.095373E+04, 1.100197E+04, 1.105031E+04, 1.109877E+04, 1.114735E+04, 
   1.119603E+04, 1.124483E+04, 1.129374E+04, 1.134276E+04, 1.139189E+04, 1.144114E+04, 
   1.149050E+04, 1.153997E+04, 1.158955E+04, 1.163925E+04, 1.168905E+04, 1.173898E+04, 
   1.178901E+04, 1.183916E+04, 1.188941E+04, 1.193979E+04, 1.199027E+04, 1.204087E+04, 
   1.209158E+04, 1.214240E+04, 1.219334E+04, 1.224439E+04, 1.229555E+04, 1.234682E+04, 
   1.239821E+04, 1.244971E+04, 1.250133E+04, 1.255306E+04, 1.260490E+04, 1.265685E+04, 
   1.270892E+04, 1.276110E+04, 1.281339E+04, 1.286580E+04, 1.291832E+04, 1.297096E+04, 
   1.302370E+04, 1.307657E+04, 1.312954E+04, 1.318263E+04, 1.323583E+04, 1.328915E+04, 
   1.334258E+04, 1.339613E+04, 1.344978E+04, 1.350356E+04, 1.355744E+04, 1.361144E+04, 
   1.366556E+04, 1.371978E+04, 1.377413E+04, 1.382858E+04, 1.388315E+04, 1.393784E+04, 
   1.399264E+04, 1.404755E+04, 1.410258E+04, 1.415772E+04, 1.421298E+04, 1.426835E+04, 
   1.432383E+04, 1.437943E+04, 1.443515E+04, 1.449098E+04, 1.454692E+04, 1.460298E+04, 
   1.465915E+04, 1.471544E+04, 1.477184E+04, 1.482836E+04, 1.488499E+04, 1.494174E+04, 
   1.499860E+04, 1.505558E+04, 1.511267E+04, 
])
# ============================ M = 46, I = 3 ============================
M = 46
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.664867E+00, 1.870584E+01, 3.672687E+01, 5.475763E+01, 7.279427E+01, 9.083602E+01, 
   1.088826E+02, 1.269340E+02, 1.449901E+02, 1.630508E+02, 1.811162E+02, 1.991862E+02, 
   2.172610E+02, 2.353405E+02, 2.534249E+02, 2.715146E+02, 2.896101E+02, 3.077124E+02, 
   3.258227E+02, 3.439426E+02, 3.620744E+02, 3.802207E+02, 3.983846E+02, 4.165698E+02, 
   4.347802E+02, 4.530205E+02, 4.712953E+02, 4.896097E+02, 5.079691E+02, 5.263789E+02, 
   5.448448E+02, 5.633725E+02, 5.819677E+02, 6.006360E+02, 6.193831E+02, 6.382146E+02, 
   6.571360E+02, 6.761525E+02, 6.952693E+02, 7.144916E+02, 7.338240E+02, 7.532714E+02, 
   7.728382E+02, 7.925288E+02, 8.123472E+02, 8.322975E+02, 8.523835E+02, 8.726087E+02, 
   8.929766E+02, 9.134906E+02, 9.341537E+02, 9.549688E+02, 9.759390E+02, 9.970667E+02, 
   1.018355E+03, 1.039805E+03, 1.061421E+03, 1.083203E+03, 1.105155E+03, 1.127278E+03, 
   1.149574E+03, 1.172045E+03, 1.194692E+03, 1.217518E+03, 1.240523E+03, 1.263709E+03, 
   1.287078E+03, 1.310630E+03, 1.334368E+03, 1.358292E+03, 1.382403E+03, 1.406703E+03, 
   1.431193E+03, 1.455872E+03, 1.480744E+03, 1.505807E+03, 1.531064E+03, 1.556515E+03, 
   1.582160E+03, 1.608001E+03, 1.634038E+03, 1.660273E+03, 1.686704E+03, 1.713334E+03, 
   1.740163E+03, 1.767191E+03, 1.794419E+03, 1.821848E+03, 1.849477E+03, 1.877308E+03, 
   1.905341E+03, 1.933576E+03, 1.962014E+03, 1.990655E+03, 2.019500E+03, 2.048549E+03, 
   2.077802E+03, 2.107260E+03, 2.136924E+03, 2.166792E+03, 2.196867E+03, 2.227147E+03, 
   2.257634E+03, 2.288328E+03, 2.319229E+03, 2.350337E+03, 2.381653E+03, 2.413176E+03, 
   2.444907E+03, 2.476847E+03, 2.508996E+03, 2.541353E+03, 2.573919E+03, 2.606695E+03, 
   2.639680E+03, 2.672875E+03, 2.706279E+03, 2.739894E+03, 2.773719E+03, 2.807755E+03, 
   2.842001E+03, 2.876458E+03, 2.911127E+03, 2.946006E+03, 2.981097E+03, 3.016400E+03, 
   3.051914E+03, 3.087640E+03, 3.123579E+03, 3.159729E+03, 3.196092E+03, 3.232668E+03, 
   3.269456E+03, 3.306457E+03, 3.343671E+03, 3.381098E+03, 3.418739E+03, 3.456593E+03, 
   3.494660E+03, 3.532941E+03, 3.571436E+03, 3.610146E+03, 3.649069E+03, 3.688206E+03, 
   3.727558E+03, 3.767124E+03, 3.806905E+03, 3.846900E+03, 3.887110E+03, 3.927536E+03, 
   3.968176E+03, 4.009032E+03, 4.050102E+03, 4.091389E+03, 4.132891E+03, 4.174608E+03, 
   4.216542E+03, 4.258691E+03, 4.301056E+03, 4.343637E+03, 4.386435E+03, 4.429448E+03, 
   4.472679E+03, 4.516125E+03, 4.559789E+03, 4.603669E+03, 4.647766E+03, 4.692080E+03, 
   4.736611E+03, 4.781359E+03, 4.826324E+03, 4.871507E+03, 4.916907E+03, 4.962525E+03, 
   5.008360E+03, 5.054413E+03, 5.100684E+03, 5.147173E+03, 5.193880E+03, 5.240805E+03, 
   5.287948E+03, 5.335309E+03, 5.382889E+03, 5.430688E+03, 5.478705E+03, 5.526941E+03, 
   5.575395E+03, 5.624069E+03, 5.672961E+03, 5.722073E+03, 5.771404E+03, 5.820954E+03, 
   5.870723E+03, 5.920712E+03, 5.970920E+03, 6.021348E+03, 6.071996E+03, 6.122863E+03, 
   6.173950E+03, 6.225258E+03, 6.276785E+03, 6.328533E+03, 6.380500E+03, 6.432688E+03, 
   6.485097E+03, 6.537726E+03, 6.590576E+03, 6.643646E+03, 6.696937E+03, 6.750449E+03, 
   6.804182E+03, 6.858136E+03, 6.912311E+03, 6.966707E+03, 7.021325E+03, 7.076164E+03, 
   7.131224E+03, 7.186507E+03, 7.242010E+03, 7.297735E+03, 7.353683E+03, 7.409852E+03, 
   7.466243E+03, 7.522856E+03, 7.579691E+03, 7.636749E+03, 7.694029E+03, 7.751531E+03, 
   7.809256E+03, 7.867203E+03, 7.925374E+03, 7.983766E+03, 8.042382E+03, 8.101221E+03, 
   8.160282E+03, 8.219567E+03, 8.279075E+03, 8.338806E+03, 8.398760E+03, 8.458938E+03, 
   8.519340E+03, 8.579965E+03, 8.640813E+03, 8.701886E+03, 8.763182E+03, 8.824702E+03, 
   8.886446E+03, 8.948415E+03, 9.010607E+03, 9.073024E+03, 9.135665E+03, 9.198530E+03, 
   9.261620E+03, 9.324935E+03, 9.388474E+03, 9.452238E+03, 9.516227E+03, 9.580441E+03, 
   9.644879E+03, 9.709543E+03, 9.774432E+03, 9.839547E+03, 9.904886E+03, 9.970451E+03, 
   1.003624E+04, 1.010226E+04, 1.016850E+04, 1.023497E+04, 1.030166E+04, 1.036858E+04, 
   1.043573E+04, 1.050310E+04, 1.057069E+04, 1.063852E+04, 1.070657E+04, 1.077484E+04, 
   1.084335E+04, 1.091208E+04, 1.098103E+04, 1.105022E+04, 1.111963E+04, 1.118926E+04, 
   1.125913E+04, 1.132922E+04, 1.139953E+04, 1.147008E+04, 1.154085E+04, 1.161185E+04, 
   1.168308E+04, 1.175453E+04, 1.182622E+04, 1.189813E+04, 1.197026E+04, 1.204263E+04, 
   1.211522E+04, 1.218805E+04, 1.226109E+04, 1.233437E+04, 1.240788E+04, 1.248161E+04, 
   1.255558E+04, 1.262977E+04, 1.270419E+04, 1.277884E+04, 1.285372E+04, 1.292882E+04, 
   1.300416E+04, 1.307972E+04, 1.315552E+04, 1.323154E+04, 1.330779E+04, 1.338427E+04, 
   1.346098E+04, 1.353792E+04, 1.361509E+04, 1.369249E+04, 1.377012E+04, 1.384798E+04, 
   1.392607E+04, 1.400439E+04, 1.408294E+04, 1.416172E+04, 1.424073E+04, 1.431997E+04, 
   1.439944E+04, 1.447914E+04, 1.455907E+04, 1.463923E+04, 1.471963E+04, 1.480025E+04, 
   1.488110E+04, 1.496219E+04, 1.504351E+04, 1.512505E+04, 1.520683E+04, 1.528884E+04, 
   1.537108E+04, 1.545356E+04, 1.553626E+04, 1.561920E+04, 1.570237E+04, 1.578577E+04, 
   1.586940E+04, 1.595326E+04, 1.603736E+04, 1.612168E+04, 1.620624E+04, 1.629103E+04, 
   1.637606E+04, 1.646132E+04, 1.654681E+04, 1.663253E+04, 1.671848E+04, 1.680467E+04, 
   1.689109E+04, 1.697774E+04, 1.706463E+04, 1.715175E+04, 1.723910E+04, 1.732669E+04, 
   1.741450E+04, 1.750256E+04, 1.759084E+04, 1.767936E+04, 1.776811E+04, 1.785710E+04, 
   1.794632E+04, 1.803578E+04, 1.812546E+04, 1.821539E+04, 1.830554E+04, 1.839593E+04, 
   1.848656E+04, 1.857742E+04, 1.866851E+04, 1.875984E+04, 1.885140E+04, 1.894320E+04, 
   1.903523E+04, 1.912750E+04, 1.922000E+04, 1.931274E+04, 1.940571E+04, 1.949892E+04, 
   1.959236E+04, 1.968604E+04, 1.977996E+04, 1.987411E+04, 1.996849E+04, 2.006311E+04, 
   2.015797E+04, 2.025306E+04, 2.034839E+04, 2.044395E+04, 2.053975E+04, 2.063579E+04, 
   2.073206E+04, 2.082857E+04, 2.092531E+04, 2.102230E+04, 2.111951E+04, 2.121697E+04, 
   2.131466E+04, 2.141259E+04, 2.151076E+04, 2.160916E+04, 2.170780E+04, 2.180667E+04, 
   2.190579E+04, 2.200514E+04, 2.210473E+04, 2.220455E+04, 2.230462E+04, 2.240492E+04, 
   2.250546E+04, 2.260624E+04, 2.270725E+04, 2.280850E+04, 2.290999E+04, 2.301172E+04, 
   2.311369E+04, 2.321589E+04, 2.331834E+04, 2.342102E+04, 2.352394E+04, 2.362710E+04, 
   2.373050E+04, 2.383413E+04, 2.393801E+04, 2.404212E+04, 2.414648E+04, 2.425107E+04, 
   2.435590E+04, 2.446097E+04, 2.456628E+04, 2.467183E+04, 2.477762E+04, 2.488365E+04, 
   2.498992E+04, 2.509643E+04, 2.520317E+04, 2.531016E+04, 2.541739E+04, 2.552486E+04, 
   2.563256E+04, 2.574051E+04, 2.584870E+04, 2.595713E+04, 2.606580E+04, 2.617471E+04, 
   2.628386E+04, 2.639325E+04, 2.650289E+04, 2.661276E+04, 2.672287E+04, 2.683323E+04, 
   2.694383E+04, 2.705466E+04, 2.716574E+04, 2.727706E+04, 2.738862E+04, 2.750043E+04, 
   2.761247E+04, 2.772476E+04, 2.783729E+04, 2.795006E+04, 2.806307E+04, 2.817632E+04, 
   2.828982E+04, 2.840356E+04, 2.851754E+04, 2.863176E+04, 2.874623E+04, 2.886094E+04, 
   2.897589E+04, 2.909108E+04, 2.920652E+04, 2.932219E+04, 2.943812E+04, 2.955428E+04, 
   2.967069E+04, 2.978734E+04, 2.990424E+04, 3.002137E+04, 3.013876E+04, 3.025638E+04, 
   3.037425E+04, 3.049236E+04, 3.061072E+04, 3.072931E+04, 3.084816E+04, 3.096725E+04, 
   3.108658E+04, 3.120615E+04, 3.132597E+04, 3.144604E+04, 3.156635E+04, 3.168690E+04, 
   3.180770E+04, 3.192874E+04, 3.205003E+04, 
])
# ============================ M = 46, I = 4 ============================
M = 46
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.185142E+00, 3.567810E+01, 6.998375E+01, 1.043090E+02, 1.386456E+02, 1.729921E+02, 
   2.073478E+02, 2.417127E+02, 2.760865E+02, 3.104692E+02, 3.448607E+02, 3.792611E+02, 
   4.136704E+02, 4.480887E+02, 4.825163E+02, 5.169535E+02, 5.514015E+02, 5.858614E+02, 
   6.203353E+02, 6.548259E+02, 6.893367E+02, 7.238722E+02, 7.584376E+02, 7.930390E+02, 
   8.276833E+02, 8.623782E+02, 8.971321E+02, 9.319538E+02, 9.668528E+02, 1.001839E+03, 
   1.036922E+03, 1.072112E+03, 1.107420E+03, 1.142856E+03, 1.178430E+03, 1.214153E+03, 
   1.250034E+03, 1.286084E+03, 1.322311E+03, 1.358726E+03, 1.395338E+03, 1.432155E+03, 
   1.469186E+03, 1.506439E+03, 1.543921E+03, 1.581641E+03, 1.619606E+03, 1.657822E+03, 
   1.696296E+03, 1.735034E+03, 1.774043E+03, 1.813328E+03, 1.852895E+03, 1.892748E+03, 
   1.932893E+03, 1.973335E+03, 2.014078E+03, 2.055125E+03, 2.096482E+03, 2.138152E+03, 
   2.180139E+03, 2.222446E+03, 2.265077E+03, 2.308034E+03, 2.351321E+03, 2.394940E+03, 
   2.438895E+03, 2.483187E+03, 2.527820E+03, 2.572796E+03, 2.618116E+03, 2.663784E+03, 
   2.709800E+03, 2.756168E+03, 2.802888E+03, 2.849963E+03, 2.897394E+03, 2.945184E+03, 
   2.993332E+03, 3.041842E+03, 3.090714E+03, 3.139949E+03, 3.189550E+03, 3.239517E+03, 
   3.289851E+03, 3.340554E+03, 3.391626E+03, 3.443069E+03, 3.494884E+03, 3.547071E+03, 
   3.599632E+03, 3.652568E+03, 3.705878E+03, 3.759566E+03, 3.813630E+03, 3.868071E+03, 
   3.922892E+03, 3.978091E+03, 4.033671E+03, 4.089631E+03, 4.145972E+03, 4.202695E+03, 
   4.259801E+03, 4.317290E+03, 4.375162E+03, 4.433418E+03, 4.492060E+03, 4.551086E+03, 
   4.610499E+03, 4.670297E+03, 4.730483E+03, 4.791055E+03, 4.852015E+03, 4.913364E+03, 
   4.975100E+03, 5.037226E+03, 5.099741E+03, 5.162646E+03, 5.225941E+03, 5.289627E+03, 
   5.353703E+03, 5.418171E+03, 5.483030E+03, 5.548282E+03, 5.613925E+03, 5.679961E+03, 
   5.746390E+03, 5.813213E+03, 5.880429E+03, 5.948039E+03, 6.016043E+03, 6.084442E+03, 
   6.153235E+03, 6.222423E+03, 6.292007E+03, 6.361987E+03, 6.432362E+03, 6.503134E+03, 
   6.574302E+03, 6.645867E+03, 6.717829E+03, 6.790188E+03, 6.862944E+03, 6.936099E+03, 
   7.009651E+03, 7.083601E+03, 7.157950E+03, 7.232697E+03, 7.307844E+03, 7.383389E+03, 
   7.459334E+03, 7.535679E+03, 7.612423E+03, 7.689568E+03, 7.767112E+03, 7.845057E+03, 
   7.923403E+03, 8.002150E+03, 8.081298E+03, 8.160847E+03, 8.240797E+03, 8.321150E+03, 
   8.401904E+03, 8.483060E+03, 8.564619E+03, 8.646580E+03, 8.728944E+03, 8.811711E+03, 
   8.894881E+03, 8.978454E+03, 9.062431E+03, 9.146812E+03, 9.231596E+03, 9.316785E+03, 
   9.402377E+03, 9.488374E+03, 9.574776E+03, 9.661583E+03, 9.748794E+03, 9.836411E+03, 
   9.924433E+03, 1.001286E+04, 1.010169E+04, 1.019093E+04, 1.028058E+04, 1.037063E+04, 
   1.046109E+04, 1.055195E+04, 1.064322E+04, 1.073490E+04, 1.082699E+04, 1.091948E+04, 
   1.101238E+04, 1.110569E+04, 1.119940E+04, 1.129352E+04, 1.138806E+04, 1.148300E+04, 
   1.157834E+04, 1.167410E+04, 1.177027E+04, 1.186684E+04, 1.196383E+04, 1.206122E+04, 
   1.215902E+04, 1.225724E+04, 1.235586E+04, 1.245489E+04, 1.255433E+04, 1.265418E+04, 
   1.275445E+04, 1.285512E+04, 1.295620E+04, 1.305770E+04, 1.315961E+04, 1.326192E+04, 
   1.336465E+04, 1.346780E+04, 1.357135E+04, 1.367531E+04, 1.377969E+04, 1.388448E+04, 
   1.398968E+04, 1.409529E+04, 1.420132E+04, 1.430776E+04, 1.441461E+04, 1.452188E+04, 
   1.462956E+04, 1.473765E+04, 1.484616E+04, 1.495508E+04, 1.506441E+04, 1.517416E+04, 
   1.528432E+04, 1.539490E+04, 1.550589E+04, 1.561729E+04, 1.572912E+04, 1.584135E+04, 
   1.595400E+04, 1.606707E+04, 1.618055E+04, 1.629445E+04, 1.640877E+04, 1.652350E+04, 
   1.663864E+04, 1.675421E+04, 1.687018E+04, 1.698658E+04, 1.710339E+04, 1.722062E+04, 
   1.733827E+04, 1.745633E+04, 1.757481E+04, 1.769371E+04, 1.781303E+04, 1.793276E+04, 
   1.805292E+04, 1.817349E+04, 1.829448E+04, 1.841588E+04, 1.853771E+04, 1.865996E+04, 
   1.878262E+04, 1.890570E+04, 1.902920E+04, 1.915313E+04, 1.927747E+04, 1.940223E+04, 
   1.952741E+04, 1.965301E+04, 1.977903E+04, 1.990547E+04, 2.003233E+04, 2.015962E+04, 
   2.028732E+04, 2.041544E+04, 2.054399E+04, 2.067296E+04, 2.080235E+04, 2.093215E+04, 
   2.106239E+04, 2.119304E+04, 2.132411E+04, 2.145561E+04, 2.158753E+04, 2.171987E+04, 
   2.185264E+04, 2.198583E+04, 2.211944E+04, 2.225347E+04, 2.238793E+04, 2.252281E+04, 
   2.265811E+04, 2.279384E+04, 2.292999E+04, 2.306656E+04, 2.320356E+04, 2.334099E+04, 
   2.347883E+04, 2.361711E+04, 2.375580E+04, 2.389493E+04, 2.403447E+04, 2.417445E+04, 
   2.431484E+04, 2.445567E+04, 2.459692E+04, 2.473859E+04, 2.488069E+04, 2.502322E+04, 
   2.516617E+04, 2.530955E+04, 2.545336E+04, 2.559759E+04, 2.574225E+04, 2.588734E+04, 
   2.603285E+04, 2.617879E+04, 2.632516E+04, 2.647196E+04, 2.661918E+04, 2.676683E+04, 
   2.691491E+04, 2.706342E+04, 2.721235E+04, 2.736172E+04, 2.751151E+04, 2.766174E+04, 
   2.781239E+04, 2.796347E+04, 2.811498E+04, 2.826692E+04, 2.841929E+04, 2.857208E+04, 
   2.872531E+04, 2.887897E+04, 2.903306E+04, 2.918758E+04, 2.934253E+04, 2.949791E+04, 
   2.965372E+04, 2.980996E+04, 2.996663E+04, 3.012374E+04, 3.028128E+04, 3.043924E+04, 
   3.059764E+04, 3.075647E+04, 3.091573E+04, 3.107543E+04, 3.123556E+04, 3.139612E+04, 
   3.155711E+04, 3.171853E+04, 3.188039E+04, 3.204268E+04, 3.220540E+04, 3.236856E+04, 
   3.253215E+04, 3.269618E+04, 3.286064E+04, 3.302553E+04, 3.319085E+04, 3.335661E+04, 
   3.352281E+04, 3.368944E+04, 3.385650E+04, 3.402400E+04, 3.419194E+04, 3.436031E+04, 
   3.452911E+04, 3.469835E+04, 3.486803E+04, 3.503814E+04, 3.520868E+04, 3.537967E+04, 
   3.555109E+04, 3.572294E+04, 3.589524E+04, 3.606796E+04, 3.624113E+04, 3.641473E+04, 
   3.658877E+04, 3.676325E+04, 3.693817E+04, 3.711352E+04, 3.728931E+04, 3.746554E+04, 
   3.764220E+04, 3.781931E+04, 3.799685E+04, 3.817483E+04, 3.835325E+04, 3.853211E+04, 
   3.871140E+04, 3.889114E+04, 3.907132E+04, 3.925193E+04, 3.943299E+04, 3.961448E+04, 
   3.979641E+04, 3.997879E+04, 4.016160E+04, 4.034486E+04, 4.052855E+04, 4.071269E+04, 
   4.089726E+04, 4.108228E+04, 4.126774E+04, 4.145364E+04, 4.163998E+04, 4.182676E+04, 
   4.201398E+04, 4.220165E+04, 4.238976E+04, 4.257831E+04, 4.276730E+04, 4.295673E+04, 
   4.314661E+04, 4.333693E+04, 4.352769E+04, 4.371889E+04, 4.391054E+04, 4.410264E+04, 
   4.429517E+04, 4.448815E+04, 4.468157E+04, 4.487544E+04, 4.506975E+04, 4.526450E+04, 
   4.545970E+04, 4.565535E+04, 4.585144E+04, 4.604797E+04, 4.624495E+04, 4.644237E+04, 
   4.664024E+04, 4.683855E+04, 4.703731E+04, 4.723652E+04, 4.743617E+04, 4.763627E+04, 
   4.783681E+04, 4.803780E+04, 4.823924E+04, 4.844112E+04, 4.864345E+04, 4.884622E+04, 
   4.904945E+04, 4.925312E+04, 4.945724E+04, 4.966180E+04, 4.986682E+04, 5.007228E+04, 
   5.027819E+04, 5.048455E+04, 5.069135E+04, 5.089861E+04, 5.110631E+04, 5.131446E+04, 
   5.152306E+04, 5.173211E+04, 5.194161E+04, 5.215156E+04, 5.236196E+04, 5.257280E+04, 
   5.278410E+04, 5.299585E+04, 5.320804E+04, 5.342069E+04, 5.363379E+04, 5.384734E+04, 
   5.406134E+04, 5.427579E+04, 5.449069E+04, 5.470604E+04, 5.492185E+04, 5.513810E+04, 
   5.535481E+04, 5.557197E+04, 5.578958E+04, 5.600764E+04, 5.622616E+04, 5.644512E+04, 
   5.666454E+04, 5.688442E+04, 5.710474E+04, 5.732552E+04, 5.754675E+04, 5.776844E+04, 
   5.799058E+04, 5.821317E+04, 5.843621E+04, 5.865971E+04, 5.888367E+04, 5.910807E+04, 
   5.933294E+04, 5.955825E+04, 5.978403E+04, 
])
# ============================ M = 47, I = 1 ============================
M = 47
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[3]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.414270E+00, 3.753885E+01, 1.057361E+02, 1.939873E+02, 2.984678E+02, 4.169656E+02, 
   5.479928E+02, 6.904842E+02, 8.436907E+02, 1.007153E+03, 1.180696E+03, 1.364420E+03, 
   1.558672E+03, 1.764013E+03, 1.981177E+03, 2.211039E+03, 2.454584E+03, 2.712887E+03, 
   2.987094E+03, 3.278415E+03, 3.588121E+03, 3.917535E+03, 4.268040E+03, 4.641076E+03, 
   5.038147E+03, 5.460821E+03, 5.910737E+03, 6.389609E+03, 6.899230E+03, 7.441478E+03, 
   8.018317E+03, 8.631809E+03, 9.284110E+03, 9.977483E+03, 1.071430E+04, 1.149703E+04, 
   1.232829E+04, 1.321079E+04, 1.414738E+04, 1.514105E+04, 1.619491E+04, 1.731223E+04, 
   1.849642E+04, 1.975104E+04, 2.107982E+04, 2.248664E+04, 2.397557E+04, 2.555083E+04, 
   2.721685E+04, 2.897824E+04, 3.083979E+04, 3.280651E+04, 3.488362E+04, 3.707653E+04, 
   3.939089E+04, 4.183257E+04, 4.440770E+04, 4.712262E+04, 4.998392E+04, 5.299848E+04, 
   5.617341E+04, 5.951611E+04, 6.303426E+04, 6.673581E+04, 7.062902E+04, 7.472245E+04, 
   7.884530E+04, 8.334853E+04, 8.807839E+04, 9.304473E+04, 9.825772E+04, 1.037279E+05, 
   1.094663E+05, 1.154840E+05, 1.217930E+05, 1.284051E+05, 1.353330E+05, 1.425896E+05, 
   1.501882E+05, 1.581427E+05, 1.664674E+05, 1.751769E+05, 1.842866E+05, 1.938121E+05, 
   2.037697E+05, 2.141760E+05, 2.250484E+05, 2.364047E+05, 2.482633E+05, 2.606430E+05, 
   2.735634E+05, 2.870446E+05, 3.011073E+05, 3.157729E+05, 3.310633E+05, 3.470012E+05, 
   3.636098E+05, 3.809132E+05, 3.989359E+05, 4.177035E+05, 4.372419E+05, 4.575780E+05, 
   4.787394E+05, 5.007546E+05, 5.236525E+05, 5.474632E+05, 5.722175E+05, 5.979470E+05, 
   6.246841E+05, 6.524622E+05, 6.813156E+05, 7.112793E+05, 7.423894E+05, 7.746829E+05, 
   8.081977E+05, 8.429729E+05, 8.790482E+05, 9.164647E+05, 9.552643E+05, 9.954901E+05, 
   1.037186E+06, 1.080398E+06, 1.125171E+06, 1.171553E+06, 1.219593E+06, 1.269341E+06, 
   1.320847E+06, 1.374163E+06, 1.429344E+06, 1.486443E+06, 1.545517E+06, 1.606622E+06, 
   1.669818E+06, 1.735164E+06, 1.802721E+06, 1.872552E+06, 1.944721E+06, 2.019294E+06, 
   2.096336E+06, 2.175918E+06, 2.258109E+06, 2.342979E+06, 2.430603E+06, 2.521054E+06, 
   2.614408E+06, 2.710744E+06, 2.810141E+06, 2.912680E+06, 3.018444E+06, 3.127517E+06, 
   3.239985E+06, 3.355936E+06, 3.475461E+06, 3.598650E+06, 3.725598E+06, 3.856399E+06, 
   3.991152E+06, 4.129954E+06, 4.272907E+06, 4.420115E+06, 4.571682E+06, 4.727716E+06, 
   4.888326E+06, 5.053624E+06, 5.223722E+06, 5.398736E+06, 5.578785E+06, 5.763988E+06, 
   5.954468E+06, 6.150348E+06, 6.351757E+06, 6.558823E+06, 6.771677E+06, 6.990454E+06, 
   7.215291E+06, 7.446325E+06, 7.683698E+06, 7.927554E+06, 8.178039E+06, 8.435303E+06, 
   8.699497E+06, 8.970774E+06, 9.249293E+06, 9.535212E+06, 9.828694E+06, 1.012990E+07, 
   1.043901E+07, 1.075618E+07, 1.108160E+07, 1.141543E+07, 1.175785E+07, 1.210906E+07, 
   1.246923E+07, 1.283855E+07, 1.321721E+07, 1.360542E+07, 1.400336E+07, 1.441124E+07, 
   1.482927E+07, 1.525764E+07, 1.569657E+07, 1.614628E+07, 1.660698E+07, 1.707890E+07, 
   1.756225E+07, 1.805727E+07, 1.856419E+07, 1.908323E+07, 1.961465E+07, 2.015869E+07, 
   2.071558E+07, 2.128557E+07, 2.186893E+07, 2.246591E+07, 2.307677E+07, 2.370177E+07, 
   2.434119E+07, 2.499529E+07, 2.566435E+07, 2.634866E+07, 2.704850E+07, 2.776415E+07, 
   2.849591E+07, 2.924407E+07, 3.000895E+07, 3.079084E+07, 3.159005E+07, 3.240690E+07, 
   3.324171E+07, 3.409479E+07, 3.496649E+07, 3.585712E+07, 3.676703E+07, 3.769656E+07, 
   3.864605E+07, 3.961586E+07, 4.060633E+07, 4.161784E+07, 4.265074E+07, 4.370540E+07, 
   4.478221E+07, 4.588154E+07, 4.700377E+07, 4.814929E+07, 4.931851E+07, 5.051181E+07, 
   5.172961E+07, 5.297232E+07, 5.424035E+07, 5.553412E+07, 5.685405E+07, 5.820060E+07, 
   5.957418E+07, 6.097524E+07, 6.240423E+07, 6.386161E+07, 6.534783E+07, 6.686336E+07, 
   6.840867E+07, 6.998425E+07, 7.159056E+07, 7.322811E+07, 7.489738E+07, 7.659887E+07, 
   7.833310E+07, 8.010058E+07, 8.190183E+07, 8.373736E+07, 8.560772E+07, 8.751344E+07, 
   8.945507E+07, 9.143315E+07, 9.344824E+07, 9.550092E+07, 9.759174E+07, 9.972129E+07, 
   1.018901E+08, 1.040989E+08, 1.063481E+08, 1.086385E+08, 1.109706E+08, 1.133450E+08, 
   1.157623E+08, 1.182232E+08, 1.207284E+08, 1.232784E+08, 1.258740E+08, 1.285157E+08, 
   1.312043E+08, 1.339404E+08, 1.367248E+08, 1.395580E+08, 1.424409E+08, 1.453740E+08, 
   1.483582E+08, 1.513941E+08, 1.544824E+08, 1.576240E+08, 1.608195E+08, 1.640698E+08, 
   1.673754E+08, 1.707374E+08, 1.741563E+08, 1.776330E+08, 1.811683E+08, 1.847630E+08, 
   1.884179E+08, 1.921338E+08, 1.959115E+08, 1.997519E+08, 2.036558E+08, 2.076241E+08, 
   2.116577E+08, 2.157573E+08, 2.199239E+08, 2.241584E+08, 2.284616E+08, 2.328345E+08, 
   2.372780E+08, 2.417930E+08, 2.463805E+08, 2.510413E+08, 2.557765E+08, 2.605869E+08, 
   2.654736E+08, 2.704376E+08, 2.754798E+08, 2.806012E+08, 2.858029E+08, 2.910858E+08, 
   2.964510E+08, 3.018996E+08, 3.074325E+08, 3.130509E+08, 3.187557E+08, 3.245481E+08, 
   3.304293E+08, 3.364001E+08, 3.424619E+08, 3.486156E+08, 3.548625E+08, 3.612037E+08, 
   3.676402E+08, 3.741734E+08, 3.808043E+08, 3.875341E+08, 3.943641E+08, 4.012954E+08, 
   4.083293E+08, 4.154670E+08, 4.227097E+08, 
])
# ============================ M = 48, I = 1 ============================
M = 48
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.147711E+01, 2.005718E+02, 3.996401E+02, 5.987334E+02, 7.981562E+02, 9.992792E+02, 
   1.204822E+03, 1.418261E+03, 1.643178E+03, 1.882951E+03, 2.140717E+03, 2.419440E+03, 
   2.721994E+03, 3.051242E+03, 3.410091E+03, 3.801534E+03, 4.228684E+03, 4.694797E+03, 
   5.203291E+03, 5.757766E+03, 6.362016E+03, 7.020045E+03, 7.736076E+03, 8.514573E+03, 
   9.360239E+03, 1.027804E+04, 1.127322E+04, 1.235130E+04, 1.351808E+04, 1.477971E+04, 
   1.614262E+04, 1.761360E+04, 1.919979E+04, 2.090867E+04, 2.274813E+04, 2.472644E+04, 
   2.685226E+04, 2.913472E+04, 3.158336E+04, 3.420818E+04, 3.701967E+04, 4.002882E+04, 
   4.324713E+04, 4.668663E+04, 5.035992E+04, 5.428016E+04, 5.846110E+04, 6.291717E+04, 
   6.766336E+04, 7.271542E+04, 7.808968E+04, 8.380330E+04, 8.987409E+04, 9.632066E+04, 
   1.031625E+05, 1.104197E+05, 1.181134E+05, 1.262656E+05, 1.348992E+05, 1.440379E+05, 
   1.537066E+05, 1.639309E+05, 1.747378E+05, 1.861551E+05, 1.982118E+05, 2.109380E+05, 
   2.243651E+05, 2.385255E+05, 2.534530E+05, 2.691826E+05, 2.857507E+05, 3.031951E+05, 
   3.215550E+05, 3.408708E+05, 3.611847E+05, 3.825405E+05, 4.049831E+05, 4.285597E+05, 
   4.533185E+05, 4.793100E+05, 5.065860E+05, 5.352006E+05, 5.652093E+05, 5.966700E+05, 
   6.296422E+05, 6.641874E+05, 7.003696E+05, 7.382547E+05, 7.779108E+05, 8.194081E+05, 
   8.628197E+05, 9.082204E+05, 9.556878E+05, 1.005302E+06, 1.057146E+06, 1.111304E+06, 
   1.167865E+06, 1.226920E+06, 1.288562E+06, 1.352888E+06, 1.419997E+06, 1.489992E+06, 
   1.562979E+06, 1.639067E+06, 1.718368E+06, 1.800998E+06, 1.887076E+06, 1.976725E+06, 
   2.070071E+06, 2.167245E+06, 2.268379E+06, 2.373613E+06, 2.483088E+06, 2.596949E+06, 
   2.715348E+06, 2.838437E+06, 2.966376E+06, 3.099329E+06, 3.237462E+06, 3.380948E+06, 
   3.529965E+06, 3.684693E+06, 3.845323E+06, 4.012043E+06, 4.185052E+06, 4.364553E+06, 
   4.550754E+06, 4.743868E+06, 4.944116E+06, 5.151722E+06, 5.366916E+06, 5.589938E+06, 
   5.821028E+06, 6.060438E+06, 6.308424E+06, 6.565247E+06, 6.831177E+06, 7.106488E+06, 
   7.391467E+06, 7.686404E+06, 7.991591E+06, 8.307337E+06, 8.633956E+06, 8.971764E+06, 
   9.321091E+06, 9.682274E+06, 1.005565E+07, 1.044158E+07, 1.084043E+07, 1.125255E+07, 
   1.167834E+07, 1.211817E+07, 1.257244E+07, 1.304156E+07, 1.352595E+07, 1.402602E+07, 
   1.454222E+07, 1.507498E+07, 1.562477E+07, 1.619205E+07, 1.677729E+07, 1.738098E+07, 
   1.800362E+07, 1.864571E+07, 1.930779E+07, 1.999037E+07, 2.069400E+07, 2.141925E+07, 
   2.216666E+07, 2.293684E+07, 2.373036E+07, 2.454784E+07, 2.538988E+07, 2.625713E+07, 
   2.715023E+07, 2.806984E+07, 2.901664E+07, 2.999131E+07, 3.099454E+07, 3.202708E+07, 
   3.308963E+07, 3.418294E+07, 3.530779E+07, 3.646494E+07, 3.765521E+07, 3.887937E+07, 
   4.013828E+07, 4.143277E+07, 4.276371E+07, 4.413196E+07, 4.553844E+07, 4.698404E+07, 
   4.846971E+07, 4.999640E+07, 5.156506E+07, 5.317669E+07, 5.483231E+07, 5.653292E+07, 
   5.827958E+07, 6.007336E+07, 6.191534E+07, 6.380664E+07, 6.574839E+07, 6.774173E+07, 
   6.978785E+07, 7.188792E+07, 7.404318E+07, 7.625486E+07, 7.852424E+07, 8.085259E+07, 
   8.324124E+07, 8.569149E+07, 8.820473E+07, 9.078236E+07, 9.342576E+07, 9.613637E+07, 
   9.891568E+07, 1.017651E+08, 1.046863E+08, 1.076807E+08, 1.107499E+08, 1.138955E+08, 
   1.171191E+08, 1.204224E+08, 1.238071E+08, 1.272749E+08, 1.308275E+08, 1.344667E+08, 
   1.381943E+08, 1.420122E+08, 1.459222E+08, 1.499263E+08, 1.540263E+08, 1.582242E+08, 
   1.625221E+08, 1.669220E+08, 1.714259E+08, 1.760359E+08, 1.807542E+08, 1.855830E+08, 
   1.905246E+08, 1.955810E+08, 2.007547E+08, 2.060479E+08, 2.114630E+08, 2.170024E+08, 
   2.226686E+08, 2.284640E+08, 2.343912E+08, 2.404528E+08, 2.466513E+08, 2.529893E+08, 
   2.594697E+08, 2.660950E+08, 2.728682E+08, 2.797919E+08, 2.868691E+08, 2.941027E+08, 
   3.014957E+08, 3.090510E+08, 3.167717E+08, 3.246609E+08, 3.327218E+08, 3.409575E+08, 
   3.493713E+08, 3.579665E+08, 3.667465E+08, 3.757146E+08, 3.848742E+08, 3.942290E+08, 
   4.037825E+08, 4.135381E+08, 4.234998E+08, 4.336710E+08, 4.440558E+08, 4.546578E+08, 
   4.654809E+08, 4.765291E+08, 4.878064E+08, 4.993170E+08, 5.110648E+08, 5.230542E+08, 
   5.352892E+08, 5.477743E+08, 5.605138E+08, 5.735123E+08, 5.867741E+08, 6.003038E+08, 
   6.141061E+08, 6.281858E+08, 6.425474E+08, 6.571959E+08, 6.721362E+08, 6.873734E+08, 
   7.029123E+08, 7.187581E+08, 7.349161E+08, 7.513915E+08, 7.681896E+08, 7.853157E+08, 
   8.027758E+08, 8.205746E+08, 8.387185E+08, 8.572130E+08, 8.760636E+08, 8.952766E+08, 
   9.148578E+08, 9.348132E+08, 9.551489E+08, 9.758713E+08, 9.969865E+08, 1.018501E+09, 
   1.040421E+09, 1.062753E+09, 1.085504E+09, 1.108681E+09, 1.132290E+09, 1.156339E+09, 
   1.180834E+09, 1.205782E+09, 1.231191E+09, 1.257068E+09, 1.283419E+09, 1.310254E+09, 
   1.337579E+09, 1.365402E+09, 1.393731E+09, 1.422573E+09, 1.451937E+09, 1.481831E+09, 
   1.512262E+09, 1.543240E+09, 1.574772E+09, 1.606867E+09, 1.639534E+09, 1.672781E+09, 
   1.706617E+09, 1.741051E+09, 1.776093E+09, 1.811751E+09, 1.848034E+09, 1.884953E+09, 
   1.922516E+09, 1.960733E+09, 1.999614E+09, 2.039169E+09, 2.079408E+09, 2.120340E+09, 
   2.161977E+09, 2.204328E+09, 2.247405E+09, 2.291216E+09, 2.335775E+09, 2.381090E+09, 
   2.427175E+09, 2.474038E+09, 2.521692E+09, 2.570149E+09, 2.619418E+09, 2.669515E+09, 
   2.720448E+09, 2.772230E+09, 2.824873E+09, 2.878390E+09, 2.932794E+09, 2.988096E+09, 
   3.044309E+09, 3.101446E+09, 3.159522E+09, 3.218546E+09, 3.278535E+09, 3.339501E+09, 
   3.401459E+09, 3.464421E+09, 3.528400E+09, 3.593413E+09, 3.659474E+09, 3.726594E+09, 
   3.794791E+09, 3.864078E+09, 3.934472E+09, 4.005987E+09, 4.078637E+09, 4.152439E+09, 
   4.227409E+09, 4.303560E+09, 4.380912E+09, 4.459477E+09, 4.539276E+09, 4.620321E+09, 
   4.702631E+09, 4.786224E+09, 4.871113E+09, 4.957321E+09, 5.044860E+09, 5.133752E+09, 
   5.224012E+09, 5.315659E+09, 5.408711E+09, 5.503187E+09, 5.599105E+09, 5.696486E+09, 
   5.795346E+09, 5.895705E+09, 5.997584E+09, 6.101001E+09, 6.205977E+09, 6.312532E+09, 
   6.420685E+09, 6.530459E+09, 6.641874E+09, 6.754949E+09, 6.869706E+09, 6.986168E+09, 
   7.104355E+09, 7.224291E+09, 7.345996E+09, 7.469493E+09, 7.594804E+09, 7.721954E+09, 
   7.850965E+09, 7.981859E+09, 8.114660E+09, 8.249393E+09, 8.386082E+09, 8.524748E+09, 
   8.665420E+09, 8.808122E+09, 8.952878E+09, 9.099711E+09, 9.248652E+09, 9.399720E+09, 
   9.552948E+09, 9.708360E+09, 9.865981E+09, 1.002584E+10, 1.018796E+10, 1.035237E+10, 
   1.051910E+10, 1.068818E+10, 1.085964E+10, 1.103350E+10, 1.120979E+10, 1.138855E+10, 
   1.156979E+10, 1.175356E+10, 1.193988E+10, 1.212878E+10, 1.232029E+10, 1.251445E+10, 
   1.271128E+10, 1.291082E+10, 1.311310E+10, 1.331814E+10, 1.352599E+10, 1.373667E+10, 
   1.395023E+10, 1.416668E+10, 1.438607E+10, 1.460843E+10, 1.483380E+10, 1.506220E+10, 
   1.529368E+10, 1.552827E+10, 1.576600E+10, 1.600692E+10, 1.625105E+10, 1.649844E+10, 
   1.674912E+10, 1.700313E+10, 1.726050E+10, 1.752129E+10, 1.778552E+10, 1.805323E+10, 
   1.832446E+10, 1.859926E+10, 1.887765E+10, 1.915969E+10, 1.944542E+10, 1.973487E+10, 
   2.002808E+10, 2.032510E+10, 2.062598E+10, 2.093075E+10, 2.123945E+10, 2.155214E+10, 
   2.186885E+10, 2.218963E+10, 2.251452E+10, 2.284357E+10, 2.317682E+10, 2.351433E+10, 
   2.385613E+10, 2.420227E+10, 2.455280E+10, 2.490778E+10, 2.526724E+10, 2.563123E+10, 
   2.599981E+10, 2.637302E+10, 2.675092E+10, 
])
# ============================ M = 48, I = 2 ============================
#M = 48
#I = 2
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.010512E+01, 9.475994E+01, 1.888521E+02, 2.829559E+02, 3.772153E+02, 4.722782E+02, 
#   5.694299E+02, 6.703137E+02, 7.766221E+02, 8.899526E+02, 1.011787E+03, 1.143527E+03, 
#   1.286531E+03, 1.442151E+03, 1.611762E+03, 1.796779E+03, 1.998672E+03, 2.218982E+03, 
#   2.459323E+03, 2.721397E+03, 3.006997E+03, 3.318016E+03, 3.656450E+03, 4.024408E+03, 
#   4.424113E+03, 4.857915E+03, 5.328287E+03, 5.837842E+03, 6.389325E+03, 6.985635E+03, 
#   7.629816E+03, 8.325077E+03, 9.074791E+03, 9.882496E+03, 1.075192E+04, 1.168697E+04, 
#   1.269174E+04, 1.377055E+04, 1.492790E+04, 1.616852E+04, 1.749737E+04, 1.891965E+04, 
#   2.044078E+04, 2.206646E+04, 2.380264E+04, 2.565554E+04, 2.763167E+04, 2.973783E+04, 
#   3.198112E+04, 3.436896E+04, 3.690911E+04, 3.960964E+04, 4.247899E+04, 4.552596E+04, 
#   4.875972E+04, 5.218984E+04, 5.582627E+04, 5.967942E+04, 6.376005E+04, 6.807945E+04, 
#   7.264931E+04, 7.748183E+04, 8.258968E+04, 8.798604E+04, 9.368460E+04, 9.969961E+04, 
#   1.060459E+05, 1.127387E+05, 1.197942E+05, 1.272287E+05, 1.350596E+05, 1.433046E+05, 
#   1.519823E+05, 1.611119E+05, 1.707132E+05, 1.808069E+05, 1.914143E+05, 2.025577E+05, 
#   2.142598E+05, 2.265446E+05, 2.394365E+05, 2.529610E+05, 2.671445E+05, 2.820142E+05, 
#   2.975983E+05, 3.139258E+05, 3.310272E+05, 3.489334E+05, 3.676765E+05, 3.872900E+05, 
#   4.078082E+05, 4.292665E+05, 4.517017E+05, 4.751515E+05, 4.996550E+05, 5.252526E+05, 
#   5.519859E+05, 5.798977E+05, 6.090323E+05, 6.394353E+05, 6.711538E+05, 7.042367E+05, 
#   7.387332E+05, 7.746956E+05, 8.121765E+05, 8.512309E+05, 8.919148E+05, 9.342868E+05, 
#   9.784057E+05, 1.024334E+06, 1.072134E+06, 1.121872E+06, 1.173614E+06, 1.227430E+06, 
#   1.283389E+06, 1.341566E+06, 1.402035E+06, 1.464874E+06, 1.530161E+06, 1.597978E+06, 
#   1.668409E+06, 1.741540E+06, 1.817459E+06, 1.896257E+06, 1.978028E+06, 2.062868E+06, 
#   2.150873E+06, 2.242146E+06, 2.336791E+06, 2.434913E+06, 2.536622E+06, 2.642030E+06, 
#   2.751252E+06, 2.864406E+06, 2.981613E+06, 3.102996E+06, 3.228684E+06, 3.358807E+06, 
#   3.493499E+06, 3.632895E+06, 3.777138E+06, 3.926371E+06, 4.080743E+06, 4.240403E+06, 
#   4.405507E+06, 4.576214E+06, 4.752686E+06, 4.935090E+06, 5.123597E+06, 5.318382E+06, 
#   5.519621E+06, 5.727500E+06, 5.942204E+06, 6.163926E+06, 6.392864E+06, 6.629215E+06, 
#   6.873186E+06, 7.124989E+06, 7.384837E+06, 7.652950E+06, 7.929553E+06, 8.214876E+06, 
#   8.509155E+06, 8.812631E+06, 9.125547E+06, 9.448157E+06, 9.780716E+06, 1.012349E+07, 
#   1.047674E+07, 1.084075E+07, 1.121579E+07, 1.160215E+07, 1.200013E+07, 1.241002E+07, 
#   1.283213E+07, 1.326677E+07, 1.371425E+07, 1.417490E+07, 1.464906E+07, 1.513707E+07, 
#   1.563926E+07, 1.615599E+07, 1.668762E+07, 1.723453E+07, 1.779708E+07, 1.837565E+07, 
#   1.897065E+07, 1.958246E+07, 2.021150E+07, 2.085818E+07, 2.152292E+07, 2.220614E+07, 
#   2.290831E+07, 2.362986E+07, 2.437125E+07, 2.513296E+07, 2.591543E+07, 2.671919E+07, 
#   2.754471E+07, 2.839250E+07, 2.926307E+07, 3.015695E+07, 3.107467E+07, 3.201677E+07, 
#   3.298381E+07, 3.397636E+07, 3.499499E+07, 3.604028E+07, 3.711284E+07, 3.821327E+07, 
#   3.934220E+07, 4.050026E+07, 4.168807E+07, 4.290632E+07, 4.415565E+07, 4.543675E+07, 
#   4.675031E+07, 4.809703E+07, 4.947764E+07, 5.089285E+07, 5.234342E+07, 5.383009E+07, 
#   5.535364E+07, 5.691486E+07, 5.851453E+07, 6.015348E+07, 6.183251E+07, 6.355250E+07, 
#   6.531424E+07, 6.711866E+07, 6.896660E+07, 7.085900E+07, 7.279675E+07, 7.478078E+07, 
#   7.681205E+07, 7.889150E+07, 8.102015E+07, 8.319894E+07, 8.542891E+07, 8.771111E+07, 
#   9.004656E+07, 9.243633E+07, 9.488148E+07, 9.738314E+07, 9.994243E+07, 1.025605E+08, 
#   1.052384E+08, 1.079774E+08, 1.107787E+08, 1.136435E+08, 1.165730E+08, 1.195685E+08, 
#   1.226312E+08, 1.257625E+08, 1.289635E+08, 1.322358E+08, 1.355806E+08, 1.389994E+08, 
#   1.424934E+08, 1.460641E+08, 1.497131E+08, 1.534416E+08, 1.572513E+08, 1.611436E+08, 
#   1.651201E+08, 1.691823E+08, 1.733319E+08, 1.775703E+08, 1.818993E+08, 1.863205E+08, 
#   1.908356E+08, 1.954463E+08, 2.001543E+08, 2.049614E+08, 2.098693E+08, 2.148799E+08, 
#   2.199950E+08, 2.252166E+08, 2.305464E+08, 2.359864E+08, 2.415386E+08, 2.472049E+08, 
#   2.529873E+08, 2.588879E+08, 2.649087E+08, 2.710520E+08, 2.773196E+08, 2.837140E+08, 
#   2.902370E+08, 2.968913E+08, 3.036787E+08, 3.106017E+08, 3.176627E+08, 3.248639E+08, 
#   3.322077E+08, 3.396966E+08, 3.473330E+08, 3.551194E+08, 3.630583E+08, 3.711523E+08, 
#   3.794040E+08, 3.878160E+08, 3.963910E+08, 4.051316E+08, 4.140405E+08, 4.231207E+08, 
#   4.323749E+08, 4.418060E+08, 4.514169E+08, 4.612103E+08, 4.711895E+08, 4.813573E+08, 
#   4.917170E+08, 5.022713E+08, 5.130236E+08, 5.239772E+08, 5.351349E+08, 5.465004E+08, 
#   5.580768E+08, 5.698675E+08, 5.818759E+08, 5.941053E+08, 6.065594E+08, 6.192415E+08, 
#   6.321555E+08, 6.453048E+08, 6.586930E+08, 6.723240E+08, 6.862016E+08, 7.003295E+08, 
#   7.147113E+08, 7.293516E+08, 7.442536E+08, 7.594218E+08, 7.748604E+08, 7.905730E+08, 
#   8.065641E+08, 8.228378E+08, 8.393985E+08, 8.562504E+08, 8.733981E+08, 8.908458E+08, 
#   9.085983E+08, 9.266595E+08, 9.450350E+08, 9.637285E+08, 9.827454E+08, 1.002090E+09, 
#   1.021768E+09, 1.041783E+09, 1.062141E+09, 1.082846E+09, 1.103904E+09, 1.125320E+09, 
#   1.147100E+09, 1.169248E+09, 1.191769E+09, 1.214669E+09, 1.237954E+09, 1.261630E+09, 
#   1.285700E+09, 1.310172E+09, 1.335052E+09, 1.360344E+09, 1.386055E+09, 1.412190E+09, 
#   1.438756E+09, 1.465760E+09, 1.493206E+09, 1.521101E+09, 1.549451E+09, 1.578264E+09, 
#   1.607544E+09, 1.637300E+09, 1.667537E+09, 1.698261E+09, 1.729481E+09, 1.761202E+09, 
#   1.793431E+09, 1.826176E+09, 1.859444E+09, 1.893242E+09, 1.927576E+09, 1.962454E+09, 
#   1.997884E+09, 2.033873E+09, 2.070429E+09, 2.107558E+09, 2.145270E+09, 2.183572E+09, 
#   2.222471E+09, 2.261976E+09, 2.302095E+09, 2.342835E+09, 2.384206E+09, 2.426216E+09, 
#   2.468872E+09, 2.512184E+09, 2.556159E+09, 2.600808E+09, 2.646138E+09, 2.692159E+09, 
#   2.738880E+09, 2.786308E+09, 2.834455E+09, 2.883329E+09, 2.932940E+09, 2.983297E+09, 
#   3.034409E+09, 3.086287E+09, 3.138940E+09, 3.192378E+09, 3.246611E+09, 3.301650E+09, 
#   3.357504E+09, 3.414184E+09, 3.471701E+09, 3.530064E+09, 3.589285E+09, 3.649375E+09, 
#   3.710344E+09, 3.772202E+09, 3.834962E+09, 3.898636E+09, 3.963232E+09, 4.028765E+09, 
#   4.095245E+09, 4.162683E+09, 4.231092E+09, 4.300485E+09, 4.370872E+09, 4.442265E+09, 
#   4.514679E+09, 4.588124E+09, 4.662613E+09, 4.738159E+09, 4.814775E+09, 4.892474E+09, 
#   4.971270E+09, 5.051174E+09, 5.132201E+09, 5.214365E+09, 5.297678E+09, 5.382154E+09, 
#   5.467808E+09, 5.554654E+09, 5.642705E+09, 5.731977E+09, 5.822484E+09, 5.914238E+09, 
#   6.007257E+09, 6.101556E+09, 6.197146E+09, 6.294048E+09, 6.392273E+09, 6.491839E+09, 
#   6.592760E+09, 6.695052E+09, 6.798733E+09, 6.903816E+09, 7.010319E+09, 7.118258E+09, 
#   7.227651E+09, 7.338512E+09, 7.450861E+09, 7.564713E+09, 7.680085E+09, 7.796996E+09, 
#   7.915462E+09, 8.035503E+09, 8.157132E+09, 8.280373E+09, 8.405242E+09, 8.531757E+09, 
#   8.659935E+09, 8.789798E+09, 8.921362E+09, 9.054650E+09, 9.189675E+09, 9.326464E+09, 
#   9.465031E+09, 9.605397E+09, 9.747583E+09, 9.891610E+09, 1.003750E+10, 1.018527E+10, 
#   1.033493E+10, 1.048653E+10, 1.064006E+10, 1.079556E+10, 1.095305E+10, 1.111255E+10, 
#   1.127407E+10, 1.143765E+10, 1.160331E+10, 1.177106E+10, 1.194093E+10, 1.211294E+10, 
#   1.228712E+10, 1.246350E+10, 1.264208E+10, 
#])
# ============================ M = 49, I = 1 ============================
M = 49
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.721481E+02, 5.256056E+03, 1.484065E+04, 2.725198E+04, 4.195358E+04, 5.864663E+04, 
   7.716245E+04, 9.743043E+04, 1.194582E+05, 1.433153E+05, 1.691203E+05, 1.970315E+05, 
   2.272411E+05, 2.599702E+05, 2.954664E+05, 3.340012E+05, 3.758697E+05, 4.213894E+05, 
   4.709005E+05, 5.247664E+05, 5.833741E+05, 6.471352E+05, 7.164874E+05, 7.918951E+05, 
   8.738508E+05, 9.628776E+05, 1.059529E+06, 1.164393E+06, 1.278091E+06, 1.401280E+06, 
   1.534657E+06, 1.678959E+06, 1.834964E+06, 2.003495E+06, 2.185421E+06, 2.381658E+06, 
   2.593175E+06, 2.820991E+06, 3.066183E+06, 3.329883E+06, 3.613284E+06, 3.917643E+06, 
   4.244280E+06, 4.594584E+06, 4.970014E+06, 5.372103E+06, 5.802462E+06, 6.262780E+06, 
   6.754827E+06, 7.280464E+06, 7.841634E+06, 8.440382E+06, 9.078840E+06, 9.759244E+06, 
   1.048393E+07, 1.125535E+07, 1.207606E+07, 1.294872E+07, 1.387613E+07, 1.486120E+07, 
   1.590698E+07, 1.701663E+07, 1.819346E+07, 1.944093E+07, 2.076262E+07, 2.216227E+07, 
   2.364381E+07, 2.521127E+07, 2.686889E+07, 2.862106E+07, 3.047235E+07, 3.242753E+07, 
   3.449152E+07, 3.666946E+07, 3.896669E+07, 4.138874E+07, 4.394135E+07, 4.663050E+07, 
   4.946237E+07, 5.244340E+07, 5.558021E+07, 5.887973E+07, 6.234908E+07, 6.599570E+07, 
   6.982725E+07, 7.385167E+07, 7.807722E+07, 8.251236E+07, 8.716595E+07, 9.204707E+07, 
   9.716517E+07, 1.025300E+08, 1.081516E+08, 1.140404E+08, 1.202071E+08, 1.266630E+08, 
   1.334193E+08, 1.404881E+08, 1.478815E+08, 1.556122E+08, 1.636932E+08, 1.721380E+08, 
   1.809603E+08, 1.901746E+08, 1.997955E+08, 2.098383E+08, 2.203186E+08, 2.312526E+08, 
   2.426568E+08, 2.545485E+08, 2.669453E+08, 2.798652E+08, 2.933272E+08, 3.073503E+08, 
   3.219545E+08, 3.371600E+08, 3.529879E+08, 3.694597E+08, 3.865976E+08, 4.044244E+08, 
   4.229636E+08, 4.422393E+08, 4.622762E+08, 4.830998E+08, 5.047363E+08, 5.272126E+08, 
   5.505561E+08, 5.747953E+08, 5.999592E+08, 6.260778E+08, 6.531817E+08, 6.813024E+08, 
   7.104720E+08, 7.407239E+08, 7.720921E+08, 8.046114E+08, 8.383173E+08, 8.732467E+08, 
   9.094375E+08, 9.469277E+08, 9.857572E+08, 1.025966E+09, 1.067597E+09, 1.110691E+09, 
   1.155292E+09, 1.201445E+09, 1.249196E+09, 1.298592E+09, 1.349679E+09, 1.402509E+09, 
   1.457130E+09, 1.513594E+09, 1.571954E+09, 1.632264E+09, 1.694580E+09, 1.758956E+09, 
   1.825452E+09, 1.894125E+09, 1.965038E+09, 2.038251E+09, 2.113827E+09, 2.191832E+09, 
   2.272331E+09, 2.355392E+09, 2.441083E+09, 2.529476E+09, 2.620643E+09, 2.714656E+09, 
   2.811592E+09, 2.911528E+09, 3.014541E+09, 3.120714E+09, 3.230126E+09, 3.342861E+09, 
   3.459007E+09, 3.578649E+09, 3.701878E+09, 3.828783E+09, 3.959458E+09, 4.093998E+09, 
   4.232500E+09, 4.375062E+09, 4.521784E+09, 4.672771E+09, 4.828127E+09, 4.987958E+09, 
   5.152375E+09, 5.321488E+09, 5.495412E+09, 5.674260E+09, 5.858154E+09, 6.047212E+09, 
   6.241558E+09, 6.441317E+09, 6.646615E+09, 6.857584E+09, 7.074357E+09, 7.297068E+09, 
   7.525854E+09, 7.760859E+09, 8.002221E+09, 8.250091E+09, 8.504613E+09, 8.765940E+09, 
   9.034227E+09, 9.309631E+09, 9.592310E+09, 9.882427E+09, 1.018015E+10, 1.048565E+10, 
   1.079909E+10, 1.112065E+10, 1.145051E+10, 1.178885E+10, 1.213586E+10, 1.249172E+10, 
   1.285662E+10, 1.323077E+10, 1.361435E+10, 1.400757E+10, 1.441063E+10, 1.482374E+10, 
   1.524712E+10, 1.568098E+10, 1.612554E+10, 1.658103E+10, 1.704766E+10, 1.752569E+10, 
   1.801533E+10, 1.851682E+10, 1.903042E+10, 1.955637E+10, 2.009491E+10, 2.064631E+10, 
   2.121083E+10, 2.178873E+10, 2.238027E+10, 2.298574E+10, 2.360541E+10, 2.423955E+10, 
   2.488848E+10, 2.555245E+10, 2.623178E+10, 2.692677E+10, 2.763772E+10, 2.836495E+10, 
   2.910876E+10, 2.986948E+10, 3.064744E+10, 3.144296E+10, 3.225639E+10, 3.308806E+10, 
   3.393831E+10, 3.480752E+10, 3.569601E+10, 3.660418E+10, 3.753238E+10, 3.848098E+10, 
   3.945037E+10, 4.044094E+10, 4.145306E+10, 4.248715E+10, 4.354361E+10, 4.462284E+10, 
   4.572526E+10, 4.685129E+10, 4.800138E+10, 4.917594E+10, 5.037543E+10, 5.160027E+10, 
   5.285095E+10, 5.412790E+10, 5.543162E+10, 5.676255E+10, 5.812120E+10, 5.950805E+10, 
   6.092359E+10, 6.236833E+10, 6.384278E+10, 6.534746E+10, 6.688289E+10, 6.844960E+10, 
   7.004814E+10, 7.167905E+10, 7.334289E+10, 7.504023E+10, 7.677162E+10, 7.853767E+10, 
   8.033894E+10, 8.217604E+10, 8.404958E+10, 8.596015E+10, 8.790839E+10, 8.989492E+10, 
   9.192039E+10, 9.398546E+10, 9.609072E+10, 9.823692E+10, 1.004247E+11, 1.026547E+11, 
   1.049276E+11, 1.072443E+11, 1.096052E+11, 1.120113E+11, 1.144631E+11, 1.169616E+11, 
   1.195073E+11, 1.221010E+11, 1.247437E+11, 1.274359E+11, 1.301785E+11, 1.329723E+11, 
   1.358181E+11, 1.387167E+11, 1.416690E+11, 1.446759E+11, 1.477380E+11, 1.508564E+11, 
   1.540319E+11, 1.572653E+11, 1.605577E+11, 1.639098E+11, 1.673227E+11, 1.707972E+11, 
   1.743343E+11, 1.779349E+11, 1.816000E+11, 1.853306E+11, 1.891278E+11, 1.929924E+11, 
   1.969255E+11, 2.009282E+11, 2.050014E+11, 2.091462E+11, 2.133638E+11, 2.176552E+11, 
   2.220214E+11, 2.264637E+11, 2.309830E+11, 2.355806E+11, 2.402576E+11, 2.450151E+11, 
   2.498545E+11, 2.547767E+11, 2.597830E+11, 2.648747E+11, 2.700530E+11, 2.753192E+11, 
   2.806744E+11, 2.861201E+11, 2.916573E+11, 2.972876E+11, 3.030122E+11, 3.088324E+11, 
   3.147497E+11, 3.207653E+11, 3.268807E+11, 3.330973E+11, 3.394164E+11, 3.458396E+11, 
   3.523683E+11, 3.590038E+11, 3.657479E+11, 3.726019E+11, 3.795672E+11, 3.866457E+11, 
   3.938387E+11, 4.011478E+11, 4.085747E+11, 4.161208E+11, 4.237879E+11, 4.315776E+11, 
   4.394915E+11, 4.475316E+11, 4.556990E+11, 4.639959E+11, 4.724240E+11, 4.809849E+11, 
   4.896804E+11, 4.985123E+11, 5.074825E+11, 5.165929E+11, 5.258451E+11, 5.352411E+11, 
   5.447830E+11, 5.544724E+11, 5.643114E+11, 5.743019E+11, 5.844460E+11, 5.947456E+11, 
   6.052027E+11, 6.158194E+11, 6.265977E+11, 6.375399E+11, 6.486480E+11, 6.599239E+11, 
   6.713700E+11, 6.829885E+11, 6.947814E+11, 7.067512E+11, 7.188998E+11, 7.312299E+11, 
   7.437436E+11, 7.564430E+11, 7.693308E+11, 7.824091E+11, 7.956805E+11, 8.091472E+11, 
   8.228118E+11, 8.366768E+11, 8.507446E+11, 8.650177E+11, 8.794988E+11, 8.941901E+11, 
   9.090947E+11, 9.242149E+11, 9.395534E+11, 9.551129E+11, 9.708961E+11, 9.869057E+11, 
   1.003145E+12, 1.019616E+12, 1.036321E+12, 1.053265E+12, 1.070448E+12, 1.087876E+12, 
   1.105549E+12, 1.123472E+12, 1.141647E+12, 1.160077E+12, 1.178766E+12, 1.197716E+12, 
   1.216931E+12, 1.236413E+12, 1.256166E+12, 1.276194E+12, 1.296498E+12, 1.317083E+12, 
   1.337952E+12, 1.359108E+12, 1.380554E+12, 1.402294E+12, 1.424331E+12, 1.446670E+12, 
   1.469312E+12, 1.492262E+12, 1.515524E+12, 1.539100E+12, 1.562995E+12, 1.587212E+12, 
   1.611755E+12, 1.636627E+12, 1.661833E+12, 1.687375E+12, 1.713259E+12, 1.739488E+12, 
   1.766065E+12, 1.792995E+12, 1.820282E+12, 1.847929E+12, 1.875941E+12, 1.904321E+12, 
   1.933075E+12, 1.962206E+12, 1.991718E+12, 2.021615E+12, 2.051902E+12, 2.082584E+12, 
   2.113663E+12, 2.145146E+12, 2.177035E+12, 2.209337E+12, 2.242055E+12, 2.275193E+12, 
   2.308758E+12, 2.342752E+12, 2.377181E+12, 2.412049E+12, 2.447362E+12, 2.483124E+12, 
   2.519339E+12, 2.556014E+12, 2.593152E+12, 2.630759E+12, 2.668840E+12, 2.707399E+12, 
   2.746443E+12, 2.785975E+12, 2.826002E+12, 2.866528E+12, 2.907559E+12, 2.949101E+12, 
   2.991157E+12, 3.033736E+12, 3.076840E+12, 3.120476E+12, 3.164651E+12, 3.209368E+12, 
   3.254635E+12, 3.300455E+12, 3.346837E+12, 
])
# ============================ M = 49, I = 2 ============================
M = 49
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.536788E+02, 1.080671E+04, 3.051435E+04, 5.603449E+04, 8.626393E+04, 1.205883E+05, 
   1.586606E+05, 2.003358E+05, 2.456296E+05, 2.946848E+05, 3.477452E+05, 4.051367E+05, 
   4.672541E+05, 5.345525E+05, 6.075406E+05, 6.867774E+05, 7.728694E+05, 8.664697E+05, 
   9.682784E+05, 1.079043E+06, 1.199558E+06, 1.330673E+06, 1.473286E+06, 1.628354E+06, 
   1.796889E+06, 1.979967E+06, 2.178729E+06, 2.394381E+06, 2.628202E+06, 2.881547E+06, 
   3.155848E+06, 3.452620E+06, 3.773466E+06, 4.120077E+06, 4.494242E+06, 4.897847E+06, 
   5.332883E+06, 5.801449E+06, 6.305759E+06, 6.848144E+06, 7.431058E+06, 8.057087E+06, 
   8.728947E+06, 9.449496E+06, 1.022174E+07, 1.104883E+07, 1.193408E+07, 1.288096E+07, 
   1.389313E+07, 1.497440E+07, 1.612879E+07, 1.736048E+07, 1.867388E+07, 2.007358E+07, 
   2.156440E+07, 2.315137E+07, 2.483975E+07, 2.663504E+07, 2.854298E+07, 3.056956E+07, 
   3.272105E+07, 3.500397E+07, 3.742513E+07, 3.999163E+07, 4.271087E+07, 4.559055E+07, 
   4.863870E+07, 5.186368E+07, 5.527418E+07, 5.887925E+07, 6.268830E+07, 6.671111E+07, 
   7.095785E+07, 7.543910E+07, 8.016580E+07, 8.514940E+07, 9.040167E+07, 9.593494E+07, 
   1.017619E+08, 1.078958E+08, 1.143503E+08, 1.211397E+08, 1.282786E+08, 1.357823E+08, 
   1.436666E+08, 1.519479E+08, 1.606431E+08, 1.697696E+08, 1.793457E+08, 1.893901E+08, 
   1.999222E+08, 2.109621E+08, 2.225305E+08, 2.346487E+08, 2.473392E+08, 2.606245E+08, 
   2.745284E+08, 2.890754E+08, 3.042904E+08, 3.201997E+08, 3.368300E+08, 3.542089E+08, 
   3.723651E+08, 3.913279E+08, 4.111276E+08, 4.317958E+08, 4.533644E+08, 4.758668E+08, 
   4.993372E+08, 5.238110E+08, 5.493242E+08, 5.759145E+08, 6.036203E+08, 6.324811E+08, 
   6.625380E+08, 6.938327E+08, 7.264084E+08, 7.603095E+08, 7.955819E+08, 8.322721E+08, 
   8.704289E+08, 9.101015E+08, 9.513412E+08, 9.942002E+08, 1.038732E+09, 1.084993E+09, 
   1.133039E+09, 1.182929E+09, 1.234722E+09, 1.288481E+09, 1.344267E+09, 1.402147E+09, 
   1.462186E+09, 1.524453E+09, 1.589017E+09, 1.655952E+09, 1.725329E+09, 1.797225E+09, 
   1.871716E+09, 1.948884E+09, 2.028808E+09, 2.111572E+09, 2.197261E+09, 2.285964E+09, 
   2.377770E+09, 2.472770E+09, 2.571060E+09, 2.672735E+09, 2.777894E+09, 2.886638E+09, 
   2.999070E+09, 3.115297E+09, 3.235427E+09, 3.359571E+09, 3.487843E+09, 3.620358E+09, 
   3.757235E+09, 3.898597E+09, 4.044568E+09, 4.195274E+09, 4.350846E+09, 4.511417E+09, 
   4.677124E+09, 4.848103E+09, 5.024499E+09, 5.206456E+09, 5.394123E+09, 5.587653E+09, 
   5.787198E+09, 5.992917E+09, 6.204973E+09, 6.423533E+09, 6.648762E+09, 6.880835E+09, 
   7.119927E+09, 7.366219E+09, 7.619892E+09, 7.881135E+09, 8.150140E+09, 8.427102E+09, 
   8.712220E+09, 9.005696E+09, 9.307740E+09, 9.618561E+09, 9.938379E+09, 1.026741E+10, 
   1.060588E+10, 1.095402E+10, 1.131207E+10, 1.168025E+10, 1.205882E+10, 1.244802E+10, 
   1.284811E+10, 1.325935E+10, 1.368199E+10, 1.411630E+10, 1.456256E+10, 1.502105E+10, 
   1.549205E+10, 1.597584E+10, 1.647274E+10, 1.698302E+10, 1.750701E+10, 1.804500E+10, 
   1.859732E+10, 1.916430E+10, 1.974625E+10, 2.034352E+10, 2.095645E+10, 2.158539E+10, 
   2.223068E+10, 2.289269E+10, 2.357178E+10, 2.426834E+10, 2.498274E+10, 2.571536E+10, 
   2.646661E+10, 2.723688E+10, 2.802658E+10, 2.883612E+10, 2.966593E+10, 3.051643E+10, 
   3.138808E+10, 3.228130E+10, 3.319655E+10, 3.413430E+10, 3.509501E+10, 3.607916E+10, 
   3.708723E+10, 3.811970E+10, 3.917710E+10, 4.025992E+10, 4.136869E+10, 4.250392E+10, 
   4.366616E+10, 4.485594E+10, 4.607384E+10, 4.732039E+10, 4.859618E+10, 4.990179E+10, 
   5.123780E+10, 5.260481E+10, 5.400345E+10, 5.543432E+10, 5.689808E+10, 5.839532E+10, 
   5.992673E+10, 6.149293E+10, 6.309465E+10, 6.473252E+10, 6.640727E+10, 6.811957E+10, 
   6.987015E+10, 7.165972E+10, 7.348904E+10, 7.535884E+10, 7.726990E+10, 7.922297E+10, 
   8.121884E+10, 8.325829E+10, 8.534217E+10, 8.747125E+10, 8.964640E+10, 9.186842E+10, 
   9.413822E+10, 9.645664E+10, 9.882455E+10, 1.012429E+11, 1.037125E+11, 1.062344E+11, 
   1.088094E+11, 1.114386E+11, 1.141229E+11, 1.168632E+11, 1.196605E+11, 1.225160E+11, 
   1.254305E+11, 1.284051E+11, 1.314410E+11, 1.345390E+11, 1.377004E+11, 1.409262E+11, 
   1.442175E+11, 1.475755E+11, 1.510013E+11, 1.544960E+11, 1.580609E+11, 1.616971E+11, 
   1.654059E+11, 1.691884E+11, 1.730460E+11, 1.769798E+11, 1.809912E+11, 1.850815E+11, 
   1.892519E+11, 1.935038E+11, 1.978386E+11, 2.022575E+11, 2.067621E+11, 2.113537E+11, 
   2.160338E+11, 2.208036E+11, 2.256649E+11, 2.306190E+11, 2.356674E+11, 2.408117E+11, 
   2.460533E+11, 2.513939E+11, 2.568351E+11, 2.623784E+11, 2.680255E+11, 2.737780E+11, 
   2.796376E+11, 2.856060E+11, 2.916849E+11, 2.978760E+11, 3.041811E+11, 3.106019E+11, 
   3.171404E+11, 3.237982E+11, 3.305773E+11, 3.374795E+11, 3.445067E+11, 3.516608E+11, 
   3.589439E+11, 3.663578E+11, 3.739045E+11, 3.815860E+11, 3.894045E+11, 3.973620E+11, 
   4.054605E+11, 4.137022E+11, 4.220892E+11, 4.306238E+11, 4.393081E+11, 4.481443E+11, 
   4.571347E+11, 4.662817E+11, 4.755873E+11, 4.850541E+11, 4.946844E+11, 5.044807E+11, 
   5.144452E+11, 5.245804E+11, 5.348889E+11, 5.453732E+11, 5.560357E+11, 5.668792E+11, 
   5.779061E+11, 5.891192E+11, 6.005211E+11, 6.121143E+11, 6.239019E+11, 6.358863E+11, 
   6.480705E+11, 6.604573E+11, 6.730496E+11, 6.858501E+11, 6.988620E+11, 7.120880E+11, 
   7.255312E+11, 7.391947E+11, 7.530815E+11, 7.671947E+11, 7.815372E+11, 7.961127E+11, 
   8.109238E+11, 8.259742E+11, 8.412671E+11, 8.568055E+11, 8.725931E+11, 8.886330E+11, 
   9.049290E+11, 9.214842E+11, 9.383023E+11, 9.553869E+11, 9.727413E+11, 9.903694E+11, 
   1.008275E+12, 1.026461E+12, 1.044932E+12, 1.063691E+12, 1.082743E+12, 1.102091E+12, 
   1.121739E+12, 1.141691E+12, 1.161951E+12, 1.182523E+12, 1.203411E+12, 1.224620E+12, 
   1.246152E+12, 1.268014E+12, 1.290208E+12, 1.312740E+12, 1.335613E+12, 1.358832E+12, 
   1.382402E+12, 1.406326E+12, 1.430610E+12, 1.455258E+12, 1.480274E+12, 1.505664E+12, 
   1.531431E+12, 1.557582E+12, 1.584120E+12, 1.611050E+12, 1.638379E+12, 1.666109E+12, 
   1.694247E+12, 1.722798E+12, 1.751766E+12, 1.781157E+12, 1.810976E+12, 1.841228E+12, 
   1.871920E+12, 1.903055E+12, 1.934640E+12, 1.966680E+12, 1.999181E+12, 2.032148E+12, 
   2.065587E+12, 2.099503E+12, 2.133904E+12, 2.168794E+12, 2.204179E+12, 2.240065E+12, 
   2.276458E+12, 2.313365E+12, 2.350791E+12, 2.388743E+12, 2.427227E+12, 2.466250E+12, 
   2.505817E+12, 2.545935E+12, 2.586611E+12, 2.627851E+12, 2.669663E+12, 2.712052E+12, 
   2.755024E+12, 2.798589E+12, 2.842752E+12, 2.887520E+12, 2.932900E+12, 2.978899E+12, 
   3.025525E+12, 3.072784E+12, 3.120685E+12, 3.169234E+12, 3.218439E+12, 3.268308E+12, 
   3.318847E+12, 3.370065E+12, 3.421969E+12, 3.474568E+12, 3.527869E+12, 3.581880E+12, 
   3.636609E+12, 3.692063E+12, 3.748253E+12, 3.805186E+12, 3.862869E+12, 3.921312E+12, 
   3.980522E+12, 4.040509E+12, 4.101281E+12, 4.162847E+12, 4.225217E+12, 4.288397E+12, 
   4.352398E+12, 4.417229E+12, 4.482897E+12, 4.549414E+12, 4.616789E+12, 4.685030E+12, 
   4.754148E+12, 4.824151E+12, 4.895049E+12, 4.966851E+12, 5.039569E+12, 5.113212E+12, 
   5.187790E+12, 5.263312E+12, 5.339789E+12, 5.417232E+12, 5.495649E+12, 5.575054E+12, 
   5.655455E+12, 5.736863E+12, 5.819289E+12, 5.902744E+12, 5.987238E+12, 6.072783E+12, 
   6.159389E+12, 6.247069E+12, 6.335833E+12, 6.425694E+12, 6.516660E+12, 6.608746E+12, 
   6.701962E+12, 6.796320E+12, 6.891832E+12, 
])
# ============================ M = 50, I = 1 ============================
M = 50
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.769359E+00, 1.755338E+01, 4.269006E+01, 7.002747E+01, 9.812478E+01, 1.265702E+02, 
   1.552039E+02, 1.839512E+02, 2.127725E+02, 2.416450E+02, 2.705545E+02, 2.994919E+02, 
   3.284510E+02, 3.574278E+02, 3.864198E+02, 4.154257E+02, 4.444454E+02, 4.734803E+02, 
   5.025327E+02, 5.316061E+02, 5.607054E+02, 5.898365E+02, 6.190061E+02, 6.482221E+02, 
   6.774931E+02, 7.068283E+02, 7.362375E+02, 7.657308E+02, 7.953185E+02, 8.250112E+02, 
   8.548195E+02, 8.847539E+02, 9.148248E+02, 9.450425E+02, 9.754168E+02, 1.005958E+03, 
   1.036674E+03, 1.067576E+03, 1.098671E+03, 1.129968E+03, 1.161475E+03, 1.193199E+03, 
   1.225149E+03, 1.257330E+03, 1.289749E+03, 1.322413E+03, 1.355327E+03, 1.388497E+03, 
   1.421929E+03, 1.455627E+03, 1.489596E+03, 1.523841E+03, 1.558365E+03, 1.593173E+03, 
   1.628268E+03, 1.663655E+03, 1.699336E+03, 1.735314E+03, 1.771594E+03, 1.808176E+03, 
   1.845065E+03, 1.882262E+03, 1.919771E+03, 1.957592E+03, 1.995729E+03, 2.034184E+03, 
   2.072958E+03, 2.112053E+03, 2.151471E+03, 2.191213E+03, 2.231282E+03, 2.271678E+03, 
   2.312403E+03, 2.353459E+03, 2.394845E+03, 2.436565E+03, 2.478618E+03, 2.521006E+03, 
   2.563730E+03, 2.606791E+03, 2.650189E+03, 2.693927E+03, 2.738003E+03, 2.782420E+03, 
   2.827178E+03, 2.872278E+03, 2.917720E+03, 2.963506E+03, 3.009635E+03, 3.056108E+03, 
   3.102927E+03, 3.150091E+03, 3.197601E+03, 3.245457E+03, 3.293661E+03, 3.342212E+03, 
   3.391111E+03, 3.440358E+03, 3.489954E+03, 3.539899E+03, 3.590194E+03, 3.640838E+03, 
   3.691833E+03, 3.743179E+03, 3.794875E+03, 3.846922E+03, 3.899321E+03, 3.952072E+03, 
   4.005175E+03, 4.058630E+03, 4.112438E+03, 4.166599E+03, 4.221112E+03, 4.275979E+03, 
   4.331200E+03, 4.386775E+03, 4.442703E+03, 4.498985E+03, 4.555622E+03, 4.612614E+03, 
   4.669960E+03, 4.727661E+03, 4.785718E+03, 4.844129E+03, 4.902896E+03, 4.962019E+03, 
   5.021498E+03, 5.081332E+03, 5.141523E+03, 5.202069E+03, 5.262972E+03, 5.324232E+03, 
   5.385848E+03, 5.447821E+03, 5.510150E+03, 5.572837E+03, 5.635881E+03, 5.699282E+03, 
   5.763041E+03, 5.827157E+03, 5.891630E+03, 5.956461E+03, 6.021650E+03, 6.087197E+03, 
   6.153102E+03, 6.219364E+03, 6.285985E+03, 6.352965E+03, 6.420302E+03, 6.487998E+03, 
   6.556053E+03, 6.624466E+03, 6.693238E+03, 6.762369E+03, 6.831859E+03, 6.901707E+03, 
   6.971915E+03, 7.042481E+03, 7.113407E+03, 7.184693E+03, 7.256337E+03, 7.328341E+03, 
   7.400704E+03, 7.473427E+03, 7.546510E+03, 7.619952E+03, 7.693754E+03, 7.767916E+03, 
   7.842438E+03, 7.917320E+03, 7.992561E+03, 8.068163E+03, 8.144125E+03, 8.220447E+03, 
   8.297130E+03, 8.374173E+03, 8.451576E+03, 8.529340E+03, 8.607464E+03, 8.685949E+03, 
   8.764794E+03, 8.844000E+03, 8.923567E+03, 9.003495E+03, 9.083783E+03, 9.164433E+03, 
   9.245443E+03, 9.326815E+03, 9.408547E+03, 9.490641E+03, 9.573096E+03, 9.655912E+03, 
   9.739089E+03, 9.822628E+03, 9.906528E+03, 9.990790E+03, 1.007541E+04, 1.016040E+04, 
   1.024574E+04, 1.033145E+04, 1.041752E+04, 1.050395E+04, 1.059075E+04, 1.067790E+04, 
   1.076542E+04, 1.085330E+04, 1.094154E+04, 1.103014E+04, 1.111910E+04, 1.120843E+04, 
   1.129812E+04, 1.138817E+04, 1.147858E+04, 1.156936E+04, 1.166050E+04, 1.175200E+04, 
   1.184386E+04, 1.193609E+04, 1.202867E+04, 1.212162E+04, 1.221494E+04, 1.230861E+04, 
   1.240265E+04, 1.249705E+04, 1.259182E+04, 1.268694E+04, 1.278243E+04, 1.287829E+04, 
   1.297450E+04, 1.307108E+04, 1.316802E+04, 1.326533E+04, 1.336300E+04, 1.346103E+04, 
   1.355942E+04, 1.365818E+04, 1.375730E+04, 1.385678E+04, 1.395663E+04, 1.405684E+04, 
   1.415742E+04, 1.425835E+04, 1.435966E+04, 1.446132E+04, 1.456335E+04, 1.466574E+04, 
   1.476850E+04, 1.487162E+04, 1.497510E+04, 1.507895E+04, 1.518316E+04, 1.528774E+04, 
   1.539268E+04, 1.549798E+04, 1.560365E+04, 1.570968E+04, 1.581608E+04, 1.592283E+04, 
   1.602996E+04, 1.613745E+04, 1.624530E+04, 1.635352E+04, 1.646210E+04, 1.657104E+04, 
   1.668035E+04, 1.679003E+04, 1.690007E+04, 1.701047E+04, 1.712124E+04, 1.723237E+04, 
   1.734387E+04, 1.745573E+04, 1.756796E+04, 1.768055E+04, 1.779351E+04, 1.790683E+04, 
   1.802051E+04, 1.813457E+04, 1.824898E+04, 1.836376E+04, 1.847891E+04, 1.859442E+04, 
   1.871030E+04, 1.882654E+04, 1.894315E+04, 1.906012E+04, 1.917746E+04, 1.929516E+04, 
   1.941323E+04, 1.953167E+04, 1.965046E+04, 1.976963E+04, 1.988916E+04, 2.000906E+04, 
   2.012932E+04, 2.024994E+04, 2.037094E+04, 2.049230E+04, 2.061402E+04, 2.073611E+04, 
   2.085857E+04, 2.098139E+04, 2.110458E+04, 2.122813E+04, 2.135205E+04, 2.147634E+04, 
   2.160099E+04, 2.172601E+04, 2.185139E+04, 2.197715E+04, 2.210326E+04, 2.222974E+04, 
   2.235660E+04, 2.248381E+04, 2.261139E+04, 2.273934E+04, 2.286766E+04, 2.299634E+04, 
   2.312539E+04, 2.325480E+04, 2.338458E+04, 2.351473E+04, 2.364525E+04, 2.377613E+04, 
   2.390738E+04, 2.403899E+04, 2.417097E+04, 2.430332E+04, 2.443604E+04, 2.456912E+04, 
   2.470257E+04, 2.483639E+04, 2.497057E+04, 2.510512E+04, 2.524004E+04, 2.537533E+04, 
   2.551098E+04, 2.564700E+04, 2.578339E+04, 2.592014E+04, 2.605726E+04, 2.619475E+04, 
   2.633261E+04, 2.647084E+04, 2.660943E+04, 2.674839E+04, 2.688772E+04, 2.702741E+04, 
   2.716747E+04, 2.730790E+04, 2.744870E+04, 2.758987E+04, 2.773140E+04, 2.787330E+04, 
   2.801557E+04, 2.815821E+04, 2.830122E+04, 2.844459E+04, 2.858833E+04, 2.873244E+04, 
   2.887692E+04, 2.902177E+04, 2.916698E+04, 2.931256E+04, 2.945852E+04, 2.960483E+04, 
   2.975152E+04, 2.989858E+04, 3.004601E+04, 3.019380E+04, 3.034196E+04, 3.049049E+04, 
   3.063939E+04, 3.078866E+04, 3.093829E+04, 3.108830E+04, 3.123867E+04, 3.138942E+04, 
   3.154053E+04, 3.169201E+04, 3.184386E+04, 3.199608E+04, 3.214867E+04, 3.230162E+04, 
   3.245495E+04, 3.260864E+04, 3.276271E+04, 3.291714E+04, 3.307194E+04, 3.322711E+04, 
   3.338266E+04, 3.353857E+04, 3.369485E+04, 3.385150E+04, 3.400851E+04, 3.416590E+04, 
   3.432366E+04, 3.448179E+04, 3.464029E+04, 3.479915E+04, 3.495839E+04, 3.511800E+04, 
   3.527797E+04, 3.543832E+04, 3.559903E+04, 3.576012E+04, 3.592158E+04, 3.608340E+04, 
   3.624560E+04, 3.640816E+04, 3.657110E+04, 3.673440E+04, 3.689808E+04, 3.706213E+04, 
   3.722654E+04, 3.739133E+04, 3.755649E+04, 3.772202E+04, 3.788792E+04, 3.805418E+04, 
   3.822082E+04, 3.838783E+04, 3.855521E+04, 3.872296E+04, 3.889108E+04, 3.905958E+04, 
   3.922844E+04, 3.939767E+04, 3.956728E+04, 3.973725E+04, 3.990760E+04, 4.007832E+04, 
   4.024941E+04, 4.042086E+04, 4.059270E+04, 4.076490E+04, 4.093747E+04, 4.111041E+04, 
   4.128373E+04, 4.145741E+04, 4.163147E+04, 4.180590E+04, 4.198070E+04, 4.215587E+04, 
   4.233141E+04, 4.250733E+04, 4.268361E+04, 4.286027E+04, 4.303730E+04, 4.321470E+04, 
   4.339247E+04, 4.357062E+04, 4.374913E+04, 4.392802E+04, 4.410728E+04, 4.428691E+04, 
   4.446692E+04, 4.464729E+04, 4.482804E+04, 4.500916E+04, 4.519065E+04, 4.537251E+04, 
   4.555475E+04, 4.573736E+04, 4.592033E+04, 4.610369E+04, 4.628741E+04, 4.647151E+04, 
   4.665598E+04, 4.684082E+04, 4.702603E+04, 4.721162E+04, 4.739758E+04, 4.758391E+04, 
   4.777062E+04, 4.795769E+04, 4.814514E+04, 4.833297E+04, 4.852116E+04, 4.870973E+04, 
   4.889867E+04, 4.908799E+04, 4.927767E+04, 4.946773E+04, 4.965817E+04, 4.984897E+04, 
   5.004015E+04, 5.023170E+04, 5.042363E+04, 5.061593E+04, 5.080860E+04, 5.100164E+04, 
   5.119506E+04, 5.138886E+04, 5.158302E+04, 5.177756E+04, 5.197247E+04, 5.216776E+04, 
   5.236342E+04, 5.255945E+04, 5.275586E+04, 
])
# ============================ M = 50, I = 2 ============================
M = 50
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.783955E+00, 1.782252E+01, 4.343160E+01, 7.130060E+01, 9.995046E+01, 1.289581E+02, 
   1.581593E+02, 1.874773E+02, 2.168715E+02, 2.463182E+02, 2.758030E+02, 3.053164E+02, 
   3.348523E+02, 3.644063E+02, 3.939761E+02, 4.235603E+02, 4.531592E+02, 4.827741E+02, 
   5.124077E+02, 5.420639E+02, 5.717478E+02, 6.014659E+02, 6.312253E+02, 6.610344E+02, 
   6.909022E+02, 7.208386E+02, 7.508536E+02, 7.809580E+02, 8.111625E+02, 8.414781E+02, 
   8.719159E+02, 9.024867E+02, 9.332014E+02, 9.640703E+02, 9.951040E+02, 1.026312E+03, 
   1.057705E+03, 1.089291E+03, 1.121080E+03, 1.153080E+03, 1.185299E+03, 1.217745E+03, 
   1.250425E+03, 1.283347E+03, 1.316517E+03, 1.349942E+03, 1.383627E+03, 1.417578E+03, 
   1.451801E+03, 1.486300E+03, 1.521081E+03, 1.556148E+03, 1.591504E+03, 1.627155E+03, 
   1.663104E+03, 1.699355E+03, 1.735910E+03, 1.772774E+03, 1.809949E+03, 1.847438E+03, 
   1.885244E+03, 1.923369E+03, 1.961816E+03, 2.000587E+03, 2.039684E+03, 2.079109E+03, 
   2.118864E+03, 2.158952E+03, 2.199372E+03, 2.240128E+03, 2.281222E+03, 2.322653E+03, 
   2.364424E+03, 2.406536E+03, 2.448990E+03, 2.491787E+03, 2.534930E+03, 2.578418E+03, 
   2.622252E+03, 2.666434E+03, 2.710965E+03, 2.755845E+03, 2.801076E+03, 2.846657E+03, 
   2.892591E+03, 2.938877E+03, 2.985516E+03, 3.032509E+03, 3.079857E+03, 3.127560E+03, 
   3.175618E+03, 3.224033E+03, 3.272805E+03, 3.321934E+03, 3.371421E+03, 3.421266E+03, 
   3.471470E+03, 3.522032E+03, 3.572955E+03, 3.624237E+03, 3.675880E+03, 3.727884E+03, 
   3.780248E+03, 3.832974E+03, 3.886061E+03, 3.939511E+03, 3.993323E+03, 4.047497E+03, 
   4.102035E+03, 4.156935E+03, 4.212199E+03, 4.267827E+03, 4.323818E+03, 4.380174E+03, 
   4.436894E+03, 4.493978E+03, 4.551427E+03, 4.609242E+03, 4.667421E+03, 4.725966E+03, 
   4.784876E+03, 4.844153E+03, 4.903794E+03, 4.963803E+03, 5.024177E+03, 5.084918E+03, 
   5.146025E+03, 5.207500E+03, 5.269341E+03, 5.331549E+03, 5.394124E+03, 5.457066E+03, 
   5.520376E+03, 5.584054E+03, 5.648099E+03, 5.712512E+03, 5.777293E+03, 5.842442E+03, 
   5.907959E+03, 5.973845E+03, 6.040098E+03, 6.106721E+03, 6.173712E+03, 6.241071E+03, 
   6.308800E+03, 6.376897E+03, 6.445363E+03, 6.514199E+03, 6.583403E+03, 6.652977E+03, 
   6.722920E+03, 6.793233E+03, 6.863915E+03, 6.934967E+03, 7.006388E+03, 7.078179E+03, 
   7.150340E+03, 7.222872E+03, 7.295773E+03, 7.369044E+03, 7.442685E+03, 7.516697E+03, 
   7.591079E+03, 7.665831E+03, 7.740954E+03, 7.816447E+03, 7.892311E+03, 7.968546E+03, 
   8.045151E+03, 8.122127E+03, 8.199474E+03, 8.277192E+03, 8.355281E+03, 8.433741E+03, 
   8.512572E+03, 8.591774E+03, 8.671348E+03, 8.751293E+03, 8.831609E+03, 8.912297E+03, 
   8.993356E+03, 9.074787E+03, 9.156589E+03, 9.238763E+03, 9.321309E+03, 9.404226E+03, 
   9.487516E+03, 9.571177E+03, 9.655210E+03, 9.739615E+03, 9.824392E+03, 9.909541E+03, 
   9.995063E+03, 1.008096E+04, 1.016722E+04, 1.025386E+04, 1.034087E+04, 1.042825E+04, 
   1.051601E+04, 1.060414E+04, 1.069264E+04, 1.078151E+04, 1.087076E+04, 1.096038E+04, 
   1.105037E+04, 1.114073E+04, 1.123147E+04, 1.132258E+04, 1.141406E+04, 1.150592E+04, 
   1.159815E+04, 1.169075E+04, 1.178372E+04, 1.187707E+04, 1.197079E+04, 1.206489E+04, 
   1.215935E+04, 1.225420E+04, 1.234941E+04, 1.244500E+04, 1.254096E+04, 1.263730E+04, 
   1.273400E+04, 1.283109E+04, 1.292854E+04, 1.302637E+04, 1.312458E+04, 1.322316E+04, 
   1.332211E+04, 1.342143E+04, 1.352113E+04, 1.362121E+04, 1.372165E+04, 1.382248E+04, 
   1.392367E+04, 1.402524E+04, 1.412718E+04, 1.422950E+04, 1.433219E+04, 1.443526E+04, 
   1.453870E+04, 1.464252E+04, 1.474671E+04, 1.485127E+04, 1.495621E+04, 1.506153E+04, 
   1.516722E+04, 1.527328E+04, 1.537972E+04, 1.548653E+04, 1.559372E+04, 1.570128E+04, 
   1.580922E+04, 1.591753E+04, 1.602622E+04, 1.613528E+04, 1.624472E+04, 1.635453E+04, 
   1.646472E+04, 1.657528E+04, 1.668622E+04, 1.679753E+04, 1.690922E+04, 1.702128E+04, 
   1.713372E+04, 1.724654E+04, 1.735973E+04, 1.747329E+04, 1.758724E+04, 1.770155E+04, 
   1.781624E+04, 1.793131E+04, 1.804676E+04, 1.816258E+04, 1.827877E+04, 1.839534E+04, 
   1.851229E+04, 1.862961E+04, 1.874731E+04, 1.886539E+04, 1.898384E+04, 1.910267E+04, 
   1.922187E+04, 1.934145E+04, 1.946140E+04, 1.958174E+04, 1.970244E+04, 1.982353E+04, 
   1.994499E+04, 2.006683E+04, 2.018904E+04, 2.031163E+04, 2.043460E+04, 2.055794E+04, 
   2.068166E+04, 2.080576E+04, 2.093023E+04, 2.105508E+04, 2.118031E+04, 2.130591E+04, 
   2.143189E+04, 2.155825E+04, 2.168498E+04, 2.181209E+04, 2.193958E+04, 2.206744E+04, 
   2.219569E+04, 2.232431E+04, 2.245330E+04, 2.258268E+04, 2.271243E+04, 2.284256E+04, 
   2.297306E+04, 2.310394E+04, 2.323521E+04, 2.336684E+04, 2.349886E+04, 2.363125E+04, 
   2.376402E+04, 2.389717E+04, 2.403069E+04, 2.416460E+04, 2.429888E+04, 2.443354E+04, 
   2.456857E+04, 2.470399E+04, 2.483978E+04, 2.497595E+04, 2.511250E+04, 2.524942E+04, 
   2.538673E+04, 2.552441E+04, 2.566247E+04, 2.580091E+04, 2.593973E+04, 2.607892E+04, 
   2.621849E+04, 2.635844E+04, 2.649878E+04, 2.663948E+04, 2.678057E+04, 2.692203E+04, 
   2.706388E+04, 2.720610E+04, 2.734870E+04, 2.749168E+04, 2.763504E+04, 2.777878E+04, 
   2.792289E+04, 2.806739E+04, 2.821226E+04, 2.835751E+04, 2.850314E+04, 2.864915E+04, 
   2.879554E+04, 2.894231E+04, 2.908946E+04, 2.923698E+04, 2.938489E+04, 2.953317E+04, 
   2.968183E+04, 2.983088E+04, 2.998030E+04, 3.013010E+04, 3.028028E+04, 3.043084E+04, 
   3.058178E+04, 3.073310E+04, 3.088480E+04, 3.103688E+04, 3.118934E+04, 3.134217E+04, 
   3.149539E+04, 3.164899E+04, 3.180297E+04, 3.195732E+04, 3.211206E+04, 3.226718E+04, 
   3.242267E+04, 3.257855E+04, 3.273481E+04, 3.289144E+04, 3.304846E+04, 3.320586E+04, 
   3.336363E+04, 3.352179E+04, 3.368033E+04, 3.383925E+04, 3.399854E+04, 3.415822E+04, 
   3.431828E+04, 3.447872E+04, 3.463954E+04, 3.480074E+04, 3.496232E+04, 3.512429E+04, 
   3.528663E+04, 3.544935E+04, 3.561246E+04, 3.577594E+04, 3.593981E+04, 3.610405E+04, 
   3.626868E+04, 3.643369E+04, 3.659908E+04, 3.676485E+04, 3.693100E+04, 3.709753E+04, 
   3.726444E+04, 3.743174E+04, 3.759941E+04, 3.776747E+04, 3.793591E+04, 3.810473E+04, 
   3.827393E+04, 3.844351E+04, 3.861348E+04, 3.878382E+04, 3.895455E+04, 3.912566E+04, 
   3.929715E+04, 3.946902E+04, 3.964128E+04, 3.981391E+04, 3.998693E+04, 4.016033E+04, 
   4.033411E+04, 4.050827E+04, 4.068282E+04, 4.085774E+04, 4.103305E+04, 4.120874E+04, 
   4.138482E+04, 4.156127E+04, 4.173811E+04, 4.191533E+04, 4.209293E+04, 4.227092E+04, 
   4.244928E+04, 4.262803E+04, 4.280716E+04, 4.298668E+04, 4.316657E+04, 4.334685E+04, 
   4.352752E+04, 4.370856E+04, 4.388999E+04, 4.407180E+04, 4.425399E+04, 4.443657E+04, 
   4.461952E+04, 4.480286E+04, 4.498659E+04, 4.517069E+04, 4.535518E+04, 4.554006E+04, 
   4.572532E+04, 4.591095E+04, 4.609698E+04, 4.628338E+04, 4.647017E+04, 4.665734E+04, 
   4.684490E+04, 4.703284E+04, 4.722116E+04, 4.740987E+04, 4.759896E+04, 4.778843E+04, 
   4.797829E+04, 4.816853E+04, 4.835915E+04, 4.855016E+04, 4.874155E+04, 4.893332E+04, 
   4.912548E+04, 4.931802E+04, 4.951095E+04, 4.970426E+04, 4.989796E+04, 5.009204E+04, 
   5.028650E+04, 5.048135E+04, 5.067658E+04, 5.087219E+04, 5.106819E+04, 5.126457E+04, 
   5.146134E+04, 5.165849E+04, 5.185603E+04, 5.205395E+04, 5.225226E+04, 5.245095E+04, 
   5.265002E+04, 5.284948E+04, 5.304933E+04, 5.324956E+04, 5.345017E+04, 5.365117E+04, 
   5.385255E+04, 5.405432E+04, 5.425647E+04, 
])
# ============================ M = 50, I = 3 ============================
M = 50
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   1.829108E+00, 1.863492E+01, 4.566606E+01, 7.513556E+01, 1.054492E+02, 1.361498E+02, 
   1.670599E+02, 1.980964E+02, 2.292152E+02, 2.603910E+02, 2.916080E+02, 3.228561E+02, 
   3.541284E+02, 3.854207E+02, 4.167303E+02, 4.480564E+02, 4.793995E+02, 5.107617E+02, 
   5.421465E+02, 5.735591E+02, 6.050057E+02, 6.364940E+02, 6.680330E+02, 6.996323E+02, 
   7.313026E+02, 7.630552E+02, 7.949018E+02, 8.268545E+02, 8.589255E+02, 8.911272E+02, 
   9.234718E+02, 9.559713E+02, 9.886376E+02, 1.021482E+03, 1.054517E+03, 1.087751E+03, 
   1.121197E+03, 1.154863E+03, 1.188760E+03, 1.222896E+03, 1.257280E+03, 1.291921E+03, 
   1.326826E+03, 1.362002E+03, 1.397457E+03, 1.433198E+03, 1.469230E+03, 1.505559E+03, 
   1.542191E+03, 1.579132E+03, 1.616386E+03, 1.653958E+03, 1.691851E+03, 1.730072E+03, 
   1.768623E+03, 1.807507E+03, 1.846730E+03, 1.886293E+03, 1.926200E+03, 1.966453E+03, 
   2.007056E+03, 2.048012E+03, 2.089322E+03, 2.130988E+03, 2.173014E+03, 2.215401E+03, 
   2.258151E+03, 2.301265E+03, 2.344747E+03, 2.388597E+03, 2.432816E+03, 2.477407E+03, 
   2.522371E+03, 2.567708E+03, 2.613421E+03, 2.659511E+03, 2.705978E+03, 2.752824E+03, 
   2.800050E+03, 2.847656E+03, 2.895644E+03, 2.944014E+03, 2.992768E+03, 3.041906E+03, 
   3.091429E+03, 3.141338E+03, 3.191633E+03, 3.242315E+03, 3.293385E+03, 3.344843E+03, 
   3.396690E+03, 3.448926E+03, 3.501552E+03, 3.554568E+03, 3.607976E+03, 3.661774E+03, 
   3.715965E+03, 3.770548E+03, 3.825523E+03, 3.880892E+03, 3.936654E+03, 3.992810E+03, 
   4.049360E+03, 4.106304E+03, 4.163643E+03, 4.221378E+03, 4.279507E+03, 4.338033E+03, 
   4.396954E+03, 4.456272E+03, 4.515986E+03, 4.576097E+03, 4.636605E+03, 4.697510E+03, 
   4.758812E+03, 4.820513E+03, 4.882611E+03, 4.945107E+03, 5.008002E+03, 5.071295E+03, 
   5.134987E+03, 5.199078E+03, 5.263567E+03, 5.328456E+03, 5.393744E+03, 5.459432E+03, 
   5.525520E+03, 5.592007E+03, 5.658895E+03, 5.726182E+03, 5.793870E+03, 5.861958E+03, 
   5.930447E+03, 5.999337E+03, 6.068627E+03, 6.138318E+03, 6.208411E+03, 6.278904E+03, 
   6.349799E+03, 6.421096E+03, 6.492793E+03, 6.564893E+03, 6.637394E+03, 6.710297E+03, 
   6.783602E+03, 6.857309E+03, 6.931418E+03, 7.005930E+03, 7.080844E+03, 7.156160E+03, 
   7.231879E+03, 7.308000E+03, 7.384524E+03, 7.461451E+03, 7.538781E+03, 7.616514E+03, 
   7.694650E+03, 7.773189E+03, 7.852131E+03, 7.931476E+03, 8.011225E+03, 8.091377E+03, 
   8.171933E+03, 8.252892E+03, 8.334255E+03, 8.416022E+03, 8.498193E+03, 8.580767E+03, 
   8.663746E+03, 8.747128E+03, 8.830915E+03, 8.915105E+03, 8.999700E+03, 9.084700E+03, 
   9.170103E+03, 9.255911E+03, 9.342124E+03, 9.428741E+03, 9.515762E+03, 9.603189E+03, 
   9.691020E+03, 9.779255E+03, 9.867896E+03, 9.956942E+03, 1.004639E+04, 1.013625E+04, 
   1.022651E+04, 1.031717E+04, 1.040825E+04, 1.049972E+04, 1.059160E+04, 1.068389E+04, 
   1.077658E+04, 1.086968E+04, 1.096318E+04, 1.105709E+04, 1.115141E+04, 1.124613E+04, 
   1.134125E+04, 1.143679E+04, 1.153272E+04, 1.162907E+04, 1.172582E+04, 1.182297E+04, 
   1.192053E+04, 1.201850E+04, 1.211688E+04, 1.221566E+04, 1.231484E+04, 1.241444E+04, 
   1.251444E+04, 1.261484E+04, 1.271566E+04, 1.281687E+04, 1.291850E+04, 1.302053E+04, 
   1.312297E+04, 1.322582E+04, 1.332907E+04, 1.343273E+04, 1.353679E+04, 1.364126E+04, 
   1.374614E+04, 1.385143E+04, 1.395712E+04, 1.406323E+04, 1.416973E+04, 1.427665E+04, 
   1.438397E+04, 1.449170E+04, 1.459984E+04, 1.470838E+04, 1.481733E+04, 1.492669E+04, 
   1.503646E+04, 1.514664E+04, 1.525722E+04, 1.536821E+04, 1.547961E+04, 1.559141E+04, 
   1.570362E+04, 1.581625E+04, 1.592927E+04, 1.604271E+04, 1.615656E+04, 1.627081E+04, 
   1.638547E+04, 1.650054E+04, 1.661602E+04, 1.673190E+04, 1.684820E+04, 1.696490E+04, 
   1.708201E+04, 1.719953E+04, 1.731746E+04, 1.743579E+04, 1.755453E+04, 1.767369E+04, 
   1.779325E+04, 1.791322E+04, 1.803360E+04, 1.815439E+04, 1.827558E+04, 1.839719E+04, 
   1.851920E+04, 1.864162E+04, 1.876446E+04, 1.888770E+04, 1.901135E+04, 1.913541E+04, 
   1.925988E+04, 1.938475E+04, 1.951004E+04, 1.963574E+04, 1.976184E+04, 1.988836E+04, 
   2.001528E+04, 2.014261E+04, 2.027036E+04, 2.039851E+04, 2.052707E+04, 2.065604E+04, 
   2.078543E+04, 2.091522E+04, 2.104542E+04, 2.117603E+04, 2.130705E+04, 2.143848E+04, 
   2.157032E+04, 2.170257E+04, 2.183523E+04, 2.196830E+04, 2.210178E+04, 2.223567E+04, 
   2.236998E+04, 2.250469E+04, 2.263981E+04, 2.277534E+04, 2.291128E+04, 2.304763E+04, 
   2.318440E+04, 2.332157E+04, 2.345916E+04, 2.359715E+04, 2.373556E+04, 2.387437E+04, 
   2.401360E+04, 2.415324E+04, 2.429328E+04, 2.443374E+04, 2.457461E+04, 2.471590E+04, 
   2.485759E+04, 2.499969E+04, 2.514220E+04, 2.528513E+04, 2.542847E+04, 2.557221E+04, 
   2.571637E+04, 2.586094E+04, 2.600592E+04, 2.615131E+04, 2.629712E+04, 2.644333E+04, 
   2.658996E+04, 2.673700E+04, 2.688445E+04, 2.703231E+04, 2.718058E+04, 2.732927E+04, 
   2.747837E+04, 2.762788E+04, 2.777779E+04, 2.792813E+04, 2.807887E+04, 2.823003E+04, 
   2.838160E+04, 2.853358E+04, 2.868597E+04, 2.883877E+04, 2.899199E+04, 2.914562E+04, 
   2.929966E+04, 2.945411E+04, 2.960898E+04, 2.976426E+04, 2.991995E+04, 3.007605E+04, 
   3.023256E+04, 3.038949E+04, 3.054683E+04, 3.070459E+04, 3.086275E+04, 3.102133E+04, 
   3.118032E+04, 3.133973E+04, 3.149954E+04, 3.165978E+04, 3.182042E+04, 3.198147E+04, 
   3.214294E+04, 3.230483E+04, 3.246712E+04, 3.262983E+04, 3.279295E+04, 3.295649E+04, 
   3.312043E+04, 3.328479E+04, 3.344957E+04, 3.361476E+04, 3.378036E+04, 3.394638E+04, 
   3.411280E+04, 3.427965E+04, 3.444690E+04, 3.461457E+04, 3.478265E+04, 3.495115E+04, 
   3.512006E+04, 3.528939E+04, 3.545913E+04, 3.562928E+04, 3.579985E+04, 3.597083E+04, 
   3.614222E+04, 3.631403E+04, 3.648625E+04, 3.665889E+04, 3.683194E+04, 3.700541E+04, 
   3.717929E+04, 3.735358E+04, 3.752829E+04, 3.770342E+04, 3.787895E+04, 3.805491E+04, 
   3.823127E+04, 3.840806E+04, 3.858525E+04, 3.876286E+04, 3.894089E+04, 3.911933E+04, 
   3.929818E+04, 3.947746E+04, 3.965714E+04, 3.983724E+04, 4.001776E+04, 4.019869E+04, 
   4.038003E+04, 4.056179E+04, 4.074397E+04, 4.092656E+04, 4.110957E+04, 4.129299E+04, 
   4.147682E+04, 4.166108E+04, 4.184575E+04, 4.203083E+04, 4.221633E+04, 4.240224E+04, 
   4.258857E+04, 4.277532E+04, 4.296248E+04, 4.315006E+04, 4.333805E+04, 4.352646E+04, 
   4.371529E+04, 4.390453E+04, 4.409418E+04, 4.428426E+04, 4.447475E+04, 4.466565E+04, 
   4.485697E+04, 4.504871E+04, 4.524086E+04, 4.543343E+04, 4.562642E+04, 4.581982E+04, 
   4.601364E+04, 4.620788E+04, 4.640253E+04, 4.659760E+04, 4.679309E+04, 4.698899E+04, 
   4.718531E+04, 4.738204E+04, 4.757919E+04, 4.777676E+04, 4.797475E+04, 4.817315E+04, 
   4.837197E+04, 4.857121E+04, 4.877086E+04, 4.897093E+04, 4.917142E+04, 4.937233E+04, 
   4.957365E+04, 4.977539E+04, 4.997755E+04, 5.018012E+04, 5.038311E+04, 5.058652E+04, 
   5.079035E+04, 5.099460E+04, 5.119926E+04, 5.140434E+04, 5.160983E+04, 5.181575E+04, 
   5.202208E+04, 5.222883E+04, 5.243600E+04, 5.264359E+04, 5.285159E+04, 5.306001E+04, 
   5.326885E+04, 5.347811E+04, 5.368779E+04, 5.389788E+04, 5.410839E+04, 5.431933E+04, 
   5.453068E+04, 5.474244E+04, 5.495463E+04, 5.516723E+04, 5.538026E+04, 5.559370E+04, 
   5.580756E+04, 5.602184E+04, 5.623653E+04, 5.645165E+04, 5.666718E+04, 5.688314E+04, 
   5.709951E+04, 5.731630E+04, 5.753351E+04, 5.775114E+04, 5.796919E+04, 5.818765E+04, 
   5.840654E+04, 5.862584E+04, 5.884557E+04, 
])
# ============================ M = 51, I = 1 ============================
M = 51
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   7.261809E+00, 1.037883E+02, 2.646059E+02, 4.734199E+02, 7.232493E+02, 1.007900E+03, 
   1.323118E+03, 1.665993E+03, 2.034397E+03, 2.426674E+03, 2.841470E+03, 3.277651E+03, 
   3.734246E+03, 4.210423E+03, 4.705465E+03, 5.218762E+03, 5.749809E+03, 6.298203E+03, 
   6.863649E+03, 7.445962E+03, 8.045069E+03, 8.661013E+03, 9.293949E+03, 9.944151E+03, 
   1.061200E+04, 1.129801E+04, 1.200276E+04, 1.272698E+04, 1.347146E+04, 1.423709E+04, 
   1.502487E+04, 1.583586E+04, 1.667120E+04, 1.753211E+04, 1.841989E+04, 1.933588E+04, 
   2.028152E+04, 2.125828E+04, 2.226771E+04, 2.331142E+04, 2.439107E+04, 2.550839E+04, 
   2.666516E+04, 2.786323E+04, 2.910451E+04, 3.039095E+04, 3.172460E+04, 3.310756E+04, 
   3.454198E+04, 3.603011E+04, 3.757425E+04, 3.917679E+04, 4.084019E+04, 4.256698E+04, 
   4.435979E+04, 4.622133E+04, 4.815440E+04, 5.016189E+04, 5.224677E+04, 5.441215E+04, 
   5.666121E+04, 5.899725E+04, 6.142367E+04, 6.394399E+04, 6.656185E+04, 6.928101E+04, 
   7.210536E+04, 7.503891E+04, 7.808584E+04, 8.125041E+04, 8.453707E+04, 8.795042E+04, 
   9.149518E+04, 9.517626E+04, 9.899873E+04, 1.029678E+05, 1.070889E+05, 1.113676E+05, 
   1.158097E+05, 1.204212E+05, 1.252081E+05, 1.301769E+05, 1.353342E+05, 1.406867E+05, 
   1.462415E+05, 1.520057E+05, 1.579870E+05, 1.641929E+05, 1.706315E+05, 1.773110E+05, 
   1.842399E+05, 1.914270E+05, 1.988812E+05, 2.066120E+05, 2.146290E+05, 2.229421E+05, 
   2.315616E+05, 2.404981E+05, 2.497624E+05, 2.593659E+05, 2.693200E+05, 2.796369E+05, 
   2.903288E+05, 3.014085E+05, 3.128891E+05, 3.247841E+05, 3.371074E+05, 3.498734E+05, 
   3.630969E+05, 3.767931E+05, 3.909778E+05, 4.056671E+05, 4.208778E+05, 4.366269E+05, 
   4.529322E+05, 4.698118E+05, 4.872846E+05, 5.053698E+05, 5.240874E+05, 5.434578E+05, 
   5.635020E+05, 5.842418E+05, 6.056994E+05, 6.278979E+05, 6.508608E+05, 6.746125E+05, 
   6.991780E+05, 7.245831E+05, 7.508541E+05, 7.780184E+05, 8.061038E+05, 8.351393E+05, 
   8.651545E+05, 8.961797E+05, 9.282462E+05, 9.613864E+05, 9.956331E+05, 1.031020E+06, 
   1.067583E+06, 1.105358E+06, 1.144380E+06, 1.184689E+06, 1.226324E+06, 1.269323E+06, 
   1.313729E+06, 1.359583E+06, 1.406929E+06, 1.455812E+06, 1.506276E+06, 1.558370E+06, 
   1.612142E+06, 1.667640E+06, 1.724916E+06, 1.784021E+06, 1.845010E+06, 1.907938E+06, 
   1.972860E+06, 2.039834E+06, 2.108921E+06, 2.180181E+06, 2.253676E+06, 2.329470E+06, 
   2.407631E+06, 2.488224E+06, 2.571319E+06, 2.656987E+06, 2.745301E+06, 2.836336E+06, 
   2.930167E+06, 3.026874E+06, 3.126538E+06, 3.229240E+06, 3.335064E+06, 3.444099E+06, 
   3.556432E+06, 3.672155E+06, 3.791361E+06, 3.914145E+06, 4.040606E+06, 4.170843E+06, 
   4.304960E+06, 4.443062E+06, 4.585257E+06, 4.731656E+06, 4.882370E+06, 5.037518E+06, 
   5.197216E+06, 5.361588E+06, 5.530757E+06, 5.704851E+06, 5.884000E+06, 6.068337E+06, 
   6.258001E+06, 6.453130E+06, 6.653869E+06, 6.860363E+06, 7.072762E+06, 7.291222E+06, 
   7.515898E+06, 7.746951E+06, 7.984547E+06, 8.228852E+06, 8.480041E+06, 8.738289E+06, 
   9.003776E+06, 9.276687E+06, 9.557210E+06, 9.845538E+06, 1.014187E+07, 1.044640E+07, 
   1.075935E+07, 1.108092E+07, 1.141132E+07, 1.175078E+07, 1.209953E+07, 1.245779E+07, 
   1.282579E+07, 1.320379E+07, 1.359203E+07, 1.399075E+07, 1.440022E+07, 1.482069E+07, 
   1.525244E+07, 1.569574E+07, 1.615087E+07, 1.661812E+07, 1.709778E+07, 1.759014E+07, 
   1.809552E+07, 1.861423E+07, 1.914658E+07, 1.969290E+07, 2.025352E+07, 2.082879E+07, 
   2.141904E+07, 2.202463E+07, 2.264593E+07, 2.328330E+07, 2.393712E+07, 2.460778E+07, 
   2.529566E+07, 2.600117E+07, 2.672472E+07, 2.746672E+07, 2.822760E+07, 2.900780E+07, 
   2.980776E+07, 3.062794E+07, 3.146879E+07, 3.233079E+07, 3.321442E+07, 3.412018E+07, 
   3.504855E+07, 3.600007E+07, 3.697524E+07, 3.797460E+07, 3.899869E+07, 4.004808E+07, 
   4.112331E+07, 4.222497E+07, 4.335365E+07, 4.450995E+07, 4.569448E+07, 4.690786E+07, 
   4.815072E+07, 4.942372E+07, 5.072753E+07, 5.206280E+07, 5.343024E+07, 5.483053E+07, 
   5.626440E+07, 5.773257E+07, 5.923579E+07, 6.077481E+07, 6.235041E+07, 6.396337E+07, 
   6.561449E+07, 6.730458E+07, 6.903449E+07, 7.080506E+07, 7.261715E+07, 7.447165E+07, 
   7.636945E+07, 7.831146E+07, 8.029863E+07, 8.233189E+07, 8.441221E+07, 8.654058E+07, 
   8.871801E+07, 9.094551E+07, 9.322413E+07, 9.555492E+07, 9.793898E+07, 1.003774E+08, 
   1.028713E+08, 1.054218E+08, 1.080301E+08, 1.106973E+08, 1.134248E+08, 1.162136E+08, 
   1.190650E+08, 1.219804E+08, 1.249610E+08, 1.280082E+08, 1.311232E+08, 1.343074E+08, 
   1.375623E+08, 1.408892E+08, 1.442897E+08, 1.477650E+08, 1.513169E+08, 1.549467E+08, 
   1.586560E+08, 1.624464E+08, 1.663195E+08, 1.702770E+08, 1.743204E+08, 1.784515E+08, 
   1.826720E+08, 1.869836E+08, 1.913881E+08, 1.958874E+08, 2.004832E+08, 2.051775E+08, 
   2.099721E+08, 2.148690E+08, 2.198702E+08, 2.249776E+08, 2.301933E+08, 2.355194E+08, 
   2.409580E+08, 2.465113E+08, 2.521813E+08, 2.579704E+08, 2.638808E+08, 2.699148E+08, 
   2.760747E+08, 2.823629E+08, 2.887818E+08, 2.953339E+08, 3.020216E+08, 3.088476E+08, 
   3.158144E+08, 3.229245E+08, 3.301808E+08, 3.375858E+08, 3.451424E+08, 3.528533E+08, 
   3.607215E+08, 3.687497E+08, 3.769410E+08, 3.852982E+08, 3.938246E+08, 4.025231E+08, 
   4.113969E+08, 4.204492E+08, 4.296832E+08, 4.391022E+08, 4.487097E+08, 4.585089E+08, 
   4.685033E+08, 4.786965E+08, 4.890920E+08, 4.996935E+08, 5.105046E+08, 5.215292E+08, 
   5.327708E+08, 5.442336E+08, 5.559213E+08, 5.678380E+08, 5.799878E+08, 5.923746E+08, 
   6.050028E+08, 6.178766E+08, 6.310003E+08, 6.443782E+08, 6.580148E+08, 6.719147E+08, 
   6.860824E+08, 7.005226E+08, 7.152401E+08, 7.302395E+08, 7.455259E+08, 7.611042E+08, 
   7.769794E+08, 7.931566E+08, 8.096410E+08, 8.264379E+08, 8.435527E+08, 8.609907E+08, 
   8.787576E+08, 8.968588E+08, 9.153001E+08, 9.340872E+08, 9.532261E+08, 9.727227E+08, 
   9.925829E+08, 1.012813E+09, 1.033419E+09, 1.054408E+09, 1.075785E+09, 1.097558E+09, 
   1.119732E+09, 1.142315E+09, 1.165314E+09, 1.188734E+09, 1.212584E+09, 1.236871E+09, 
   1.261601E+09, 1.286782E+09, 1.312422E+09, 1.338528E+09, 1.365107E+09, 1.392167E+09, 
   1.419717E+09, 1.447764E+09, 1.476316E+09, 1.505382E+09, 1.534970E+09, 1.565088E+09, 
   1.595745E+09, 1.626950E+09, 1.658711E+09, 1.691038E+09, 1.723939E+09, 1.757424E+09, 
   1.791503E+09, 1.826184E+09, 1.861478E+09, 1.897394E+09, 1.933941E+09, 1.971131E+09, 
   2.008973E+09, 2.047478E+09, 2.086655E+09, 2.126517E+09, 2.167073E+09, 2.208334E+09, 
   2.250312E+09, 2.293018E+09, 2.336463E+09, 2.380659E+09, 2.425618E+09, 2.471350E+09, 
   2.517870E+09, 2.565187E+09, 2.613316E+09, 2.662269E+09, 2.712058E+09, 2.762696E+09, 
   2.814197E+09, 2.866574E+09, 2.919839E+09, 2.974008E+09, 3.029093E+09, 3.085109E+09, 
   3.142069E+09, 3.199989E+09, 3.258883E+09, 3.318766E+09, 3.379652E+09, 3.441557E+09, 
   3.504497E+09, 3.568487E+09, 3.633542E+09, 3.699680E+09, 3.766915E+09, 3.835265E+09, 
   3.904747E+09, 3.975377E+09, 4.047172E+09, 4.120150E+09, 4.194329E+09, 4.269726E+09, 
   4.346360E+09, 4.424248E+09, 4.503410E+09, 4.583864E+09, 4.665629E+09, 4.748725E+09, 
   4.833171E+09, 4.918987E+09, 5.006193E+09, 5.094810E+09, 5.184858E+09, 5.276358E+09, 
   5.369332E+09, 5.463800E+09, 5.559784E+09, 5.657308E+09, 5.756392E+09, 5.857059E+09, 
   5.959333E+09, 6.063236E+09, 6.168792E+09, 6.276025E+09, 6.384959E+09, 6.495619E+09, 
   6.608028E+09, 6.722212E+09, 6.838198E+09, 
])
# ============================ M = 52, I = 1 ============================
M = 52
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.594113E+00, 1.236785E+01, 2.929862E+01, 5.254960E+01, 8.022606E+01, 1.115873E+02, 
   1.462292E+02, 1.838669E+02, 2.242806E+02, 2.672935E+02, 3.127603E+02, 3.605600E+02, 
   4.105925E+02, 4.627760E+02, 5.170464E+02, 5.733560E+02, 6.316733E+02, 6.919821E+02, 
   7.542802E+02, 8.185783E+02, 8.848991E+02, 9.532754E+02, 1.023749E+03, 1.096370E+03, 
   1.171196E+03, 1.248287E+03, 1.327711E+03, 1.409540E+03, 1.493846E+03, 1.580709E+03, 
   1.670208E+03, 1.762425E+03, 1.857444E+03, 1.955351E+03, 2.056234E+03, 2.160182E+03, 
   2.267286E+03, 2.377638E+03, 2.491332E+03, 2.608465E+03, 2.729132E+03, 2.853433E+03, 
   2.981468E+03, 3.113341E+03, 3.249154E+03, 3.389014E+03, 3.533028E+03, 3.681307E+03, 
   3.833963E+03, 3.991109E+03, 4.152860E+03, 4.319337E+03, 4.490657E+03, 4.666945E+03, 
   4.848324E+03, 5.034922E+03, 5.226868E+03, 5.424293E+03, 5.627332E+03, 5.836120E+03, 
   6.050798E+03, 6.271505E+03, 6.498386E+03, 6.731588E+03, 6.971258E+03, 7.217549E+03, 
   7.470614E+03, 7.730610E+03, 7.997697E+03, 8.272036E+03, 8.553793E+03, 8.843133E+03, 
   9.140229E+03, 9.445252E+03, 9.758379E+03, 1.007979E+04, 1.040966E+04, 1.074818E+04, 
   1.109554E+04, 1.145192E+04, 1.181752E+04, 1.219254E+04, 1.257717E+04, 1.297162E+04, 
   1.337609E+04, 1.379079E+04, 1.421593E+04, 1.465174E+04, 1.509841E+04, 1.555619E+04, 
   1.602529E+04, 1.650593E+04, 1.699836E+04, 1.750281E+04, 1.801950E+04, 1.854869E+04, 
   1.909063E+04, 1.964555E+04, 2.021371E+04, 2.079537E+04, 2.139078E+04, 2.200021E+04, 
   2.262392E+04, 2.326219E+04, 2.391528E+04, 2.458348E+04, 2.526706E+04, 2.596631E+04, 
   2.668153E+04, 2.741299E+04, 2.816100E+04, 2.892586E+04, 2.970788E+04, 3.050735E+04, 
   3.132460E+04, 3.215993E+04, 3.301368E+04, 3.388616E+04, 3.477770E+04, 3.568864E+04, 
   3.661932E+04, 3.757007E+04, 3.854124E+04, 3.953318E+04, 4.054625E+04, 4.158079E+04, 
   4.263719E+04, 4.371580E+04, 4.481700E+04, 4.594116E+04, 4.708867E+04, 4.825990E+04, 
   4.945526E+04, 5.067513E+04, 5.191992E+04, 5.319003E+04, 5.448586E+04, 5.580785E+04, 
   5.715639E+04, 5.853192E+04, 5.993486E+04, 6.136566E+04, 6.282474E+04, 6.431255E+04, 
   6.582954E+04, 6.737616E+04, 6.895288E+04, 7.056015E+04, 7.219845E+04, 7.386825E+04, 
   7.557003E+04, 7.730427E+04, 7.907147E+04, 8.087212E+04, 8.270672E+04, 8.457578E+04, 
   8.647982E+04, 8.841934E+04, 9.039487E+04, 9.240694E+04, 9.445609E+04, 9.654286E+04, 
   9.866778E+04, 1.008314E+05, 1.030343E+05, 1.052771E+05, 1.075602E+05, 1.098844E+05, 
   1.122500E+05, 1.146579E+05, 1.171084E+05, 1.196023E+05, 1.221402E+05, 1.247226E+05, 
   1.273501E+05, 1.300235E+05, 1.327433E+05, 1.355101E+05, 1.383247E+05, 1.411876E+05, 
   1.440995E+05, 1.470611E+05, 1.500730E+05, 1.531360E+05, 1.562506E+05, 1.594176E+05, 
   1.626377E+05, 1.659115E+05, 1.692399E+05, 1.726234E+05, 1.760628E+05, 1.795589E+05, 
   1.831124E+05, 1.867240E+05, 1.903945E+05, 1.941246E+05, 1.979152E+05, 2.017668E+05, 
   2.056805E+05, 2.096568E+05, 2.136966E+05, 2.178008E+05, 2.219701E+05, 2.262054E+05, 
   2.305073E+05, 2.348769E+05, 2.393149E+05, 2.438222E+05, 2.483995E+05, 2.530479E+05, 
   2.577681E+05, 2.625610E+05, 2.674275E+05, 2.723684E+05, 2.773848E+05, 2.824774E+05, 
   2.876473E+05, 2.928952E+05, 2.982222E+05, 3.036292E+05, 3.091171E+05, 3.146869E+05, 
   3.203396E+05, 3.260760E+05, 3.318972E+05, 3.378042E+05, 3.437980E+05, 3.498795E+05, 
   3.560499E+05, 3.623100E+05, 3.686609E+05, 3.751037E+05, 3.816394E+05, 3.882691E+05, 
   3.949938E+05, 4.018146E+05, 4.087325E+05, 4.157487E+05, 4.228643E+05, 4.300803E+05, 
   4.373979E+05, 4.448182E+05, 4.523424E+05, 4.599715E+05, 4.677067E+05, 4.755492E+05, 
   4.835002E+05, 4.915608E+05, 4.997322E+05, 5.080157E+05, 5.164123E+05, 5.249234E+05, 
   5.335502E+05, 5.422938E+05, 5.511557E+05, 5.601369E+05, 5.692387E+05, 5.784625E+05, 
   5.878096E+05, 5.972811E+05, 6.068784E+05, 6.166029E+05, 6.264558E+05, 6.364385E+05, 
   6.465523E+05, 6.567986E+05, 6.671788E+05, 6.776941E+05, 6.883461E+05, 6.991360E+05, 
   7.100653E+05, 7.211354E+05, 7.323478E+05, 7.437038E+05, 7.552049E+05, 7.668526E+05, 
   7.786484E+05, 7.905936E+05, 8.026898E+05, 8.149386E+05, 8.273413E+05, 8.398996E+05, 
   8.526149E+05, 8.654888E+05, 8.785229E+05, 8.917187E+05, 9.050778E+05, 9.186017E+05, 
   9.322922E+05, 9.461507E+05, 9.601789E+05, 9.743784E+05, 9.887510E+05, 1.003298E+06, 
   1.018022E+06, 1.032923E+06, 1.048004E+06, 1.063267E+06, 1.078712E+06, 1.094342E+06, 
   1.110159E+06, 1.126164E+06, 1.142360E+06, 1.158747E+06, 1.175327E+06, 1.192103E+06, 
   1.209076E+06, 1.226249E+06, 1.243622E+06, 1.261198E+06, 1.278978E+06, 1.296964E+06, 
   1.315159E+06, 1.333565E+06, 1.352182E+06, 1.371013E+06, 1.390060E+06, 1.409326E+06, 
   1.428810E+06, 1.448517E+06, 1.468448E+06, 1.488604E+06, 1.508988E+06, 1.529602E+06, 
   1.550448E+06, 1.571528E+06, 1.592844E+06, 1.614397E+06, 1.636191E+06, 1.658227E+06, 
   1.680507E+06, 1.703034E+06, 1.725810E+06, 1.748836E+06, 1.772115E+06, 1.795649E+06, 
   1.819440E+06, 1.843491E+06, 1.867804E+06, 1.892380E+06, 1.917223E+06, 1.942334E+06, 
   1.967716E+06, 1.993371E+06, 2.019301E+06, 2.045509E+06, 2.071997E+06, 2.098767E+06, 
   2.125822E+06, 2.153164E+06, 2.180795E+06, 2.208719E+06, 2.236936E+06, 2.265450E+06, 
   2.294264E+06, 2.323379E+06, 2.352798E+06, 2.382524E+06, 2.412558E+06, 2.442905E+06, 
   2.473565E+06, 2.504542E+06, 2.535839E+06, 2.567457E+06, 2.599400E+06, 2.631670E+06, 
   2.664270E+06, 2.697202E+06, 2.730468E+06, 2.764073E+06, 2.798018E+06, 2.832306E+06, 
   2.866940E+06, 2.901922E+06, 2.937256E+06, 2.972943E+06, 3.008988E+06, 3.045392E+06, 
   3.082158E+06, 3.119290E+06, 3.156790E+06, 3.194660E+06, 3.232905E+06, 3.271527E+06, 
   3.310528E+06, 3.349911E+06, 3.389681E+06, 3.429838E+06, 3.470388E+06, 3.511331E+06, 
   3.552672E+06, 3.594414E+06, 3.636559E+06, 3.679111E+06, 3.722073E+06, 3.765447E+06, 
   3.809237E+06, 3.853447E+06, 3.898078E+06, 3.943135E+06, 3.988620E+06, 4.034537E+06, 
   4.080889E+06, 4.127679E+06, 4.174911E+06, 4.222587E+06, 4.270712E+06, 4.319287E+06, 
   4.368317E+06, 4.417805E+06, 4.467755E+06, 4.518169E+06, 4.569051E+06, 4.620404E+06, 
   4.672233E+06, 4.724540E+06, 4.777328E+06, 4.830602E+06, 4.884365E+06, 4.938620E+06, 
   4.993371E+06, 5.048621E+06, 5.104374E+06, 5.160634E+06, 5.217404E+06, 5.274688E+06, 
   5.332490E+06, 5.390812E+06, 5.449660E+06, 5.509035E+06, 5.568943E+06, 5.629388E+06, 
   5.690371E+06, 5.751899E+06, 5.813974E+06, 5.876600E+06, 5.939780E+06, 6.003520E+06, 
   6.067823E+06, 6.132692E+06, 6.198132E+06, 6.264146E+06, 6.330739E+06, 6.397914E+06, 
   6.465676E+06, 6.534028E+06, 6.602975E+06, 6.672520E+06, 6.742668E+06, 6.813423E+06, 
   6.884788E+06, 6.956769E+06, 7.029369E+06, 7.102592E+06, 7.176442E+06, 7.250924E+06, 
   7.326043E+06, 7.401802E+06, 7.478205E+06, 7.555257E+06, 7.632962E+06, 7.711325E+06, 
   7.790349E+06, 7.870040E+06, 7.950402E+06, 8.031438E+06, 8.113155E+06, 8.195555E+06, 
   8.278644E+06, 8.362426E+06, 8.446906E+06, 8.532087E+06, 8.617976E+06, 8.704576E+06, 
   8.791892E+06, 8.879928E+06, 8.968690E+06, 9.058182E+06, 9.148408E+06, 9.239374E+06, 
   9.331084E+06, 9.423543E+06, 9.516756E+06, 9.610727E+06, 9.705462E+06, 9.800965E+06, 
   9.897241E+06, 9.994296E+06, 1.009213E+07, 1.019076E+07, 1.029018E+07, 1.039039E+07, 
   1.049141E+07, 1.059324E+07, 1.069588E+07, 1.079934E+07, 1.090362E+07, 1.100873E+07, 
   1.111468E+07, 1.122146E+07, 1.132909E+07, 
])
# ============================ M = 52, I = 2 ============================
M = 52
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.594113E+00, 1.236785E+01, 2.929862E+01, 5.254960E+01, 8.022606E+01, 1.115873E+02, 
   1.462292E+02, 1.838669E+02, 2.242806E+02, 2.672935E+02, 3.127603E+02, 3.605600E+02, 
   4.105924E+02, 4.627758E+02, 5.170459E+02, 5.733551E+02, 6.316719E+02, 6.919799E+02, 
   7.542767E+02, 8.185733E+02, 8.848920E+02, 9.532658E+02, 1.023737E+03, 1.096354E+03, 
   1.171175E+03, 1.248261E+03, 1.327679E+03, 1.409501E+03, 1.493800E+03, 1.580655E+03, 
   1.670144E+03, 1.762350E+03, 1.857358E+03, 1.955252E+03, 2.056122E+03, 2.160055E+03, 
   2.267142E+03, 2.377477E+03, 2.491153E+03, 2.608266E+03, 2.728912E+03, 2.853191E+03, 
   2.981202E+03, 3.113049E+03, 3.248836E+03, 3.388667E+03, 3.532652E+03, 3.680900E+03, 
   3.833522E+03, 3.990633E+03, 4.152348E+03, 4.318786E+03, 4.490066E+03, 4.666311E+03, 
   4.847646E+03, 5.034198E+03, 5.226095E+03, 5.423470E+03, 5.626456E+03, 5.835189E+03, 
   6.049809E+03, 6.270457E+03, 6.497276E+03, 6.730412E+03, 6.970015E+03, 7.216235E+03, 
   7.469228E+03, 7.729148E+03, 7.996156E+03, 8.270413E+03, 8.552085E+03, 8.841337E+03, 
   9.138341E+03, 9.443270E+03, 9.756299E+03, 1.007761E+04, 1.040737E+04, 1.074578E+04, 
   1.109303E+04, 1.144929E+04, 1.181477E+04, 1.218967E+04, 1.257417E+04, 1.296848E+04, 
   1.337282E+04, 1.378738E+04, 1.421237E+04, 1.464802E+04, 1.509455E+04, 1.555216E+04, 
   1.602109E+04, 1.650157E+04, 1.699382E+04, 1.749808E+04, 1.801459E+04, 1.854359E+04, 
   1.908532E+04, 1.964004E+04, 2.020799E+04, 2.078943E+04, 2.138462E+04, 2.199381E+04, 
   2.261729E+04, 2.325531E+04, 2.390815E+04, 2.457610E+04, 2.525941E+04, 2.595839E+04, 
   2.667333E+04, 2.740450E+04, 2.815222E+04, 2.891677E+04, 2.969847E+04, 3.049763E+04, 
   3.131455E+04, 3.214955E+04, 3.300295E+04, 3.387507E+04, 3.476625E+04, 3.567682E+04, 
   3.660711E+04, 3.755747E+04, 3.852824E+04, 3.951976E+04, 4.053241E+04, 4.156652E+04, 
   4.262247E+04, 4.370063E+04, 4.480136E+04, 4.592504E+04, 4.707206E+04, 4.824279E+04, 
   4.943764E+04, 5.065698E+04, 5.190123E+04, 5.317079E+04, 5.446606E+04, 5.578747E+04, 
   5.713542E+04, 5.851034E+04, 5.991267E+04, 6.134283E+04, 6.280127E+04, 6.428842E+04, 
   6.580473E+04, 6.735066E+04, 6.892667E+04, 7.053323E+04, 7.217079E+04, 7.383983E+04, 
   7.554084E+04, 7.727429E+04, 7.904069E+04, 8.084052E+04, 8.267428E+04, 8.454248E+04, 
   8.644564E+04, 8.838427E+04, 9.035889E+04, 9.237003E+04, 9.441823E+04, 9.650402E+04, 
   9.862796E+04, 1.007906E+05, 1.029925E+05, 1.052342E+05, 1.075162E+05, 1.098393E+05, 
   1.122038E+05, 1.146105E+05, 1.170599E+05, 1.195526E+05, 1.220893E+05, 1.246704E+05, 
   1.272968E+05, 1.299688E+05, 1.326873E+05, 1.354529E+05, 1.382660E+05, 1.411276E+05, 
   1.440381E+05, 1.469982E+05, 1.500087E+05, 1.530701E+05, 1.561832E+05, 1.593487E+05, 
   1.625672E+05, 1.658394E+05, 1.691661E+05, 1.725480E+05, 1.759858E+05, 1.794801E+05, 
   1.830319E+05, 1.866417E+05, 1.903104E+05, 1.940386E+05, 1.978273E+05, 2.016770E+05, 
   2.055887E+05, 2.095630E+05, 2.136009E+05, 2.177030E+05, 2.218702E+05, 2.261033E+05, 
   2.304031E+05, 2.347705E+05, 2.392063E+05, 2.437112E+05, 2.482863E+05, 2.529323E+05, 
   2.576501E+05, 2.624405E+05, 2.673045E+05, 2.722430E+05, 2.772568E+05, 2.823468E+05, 
   2.875140E+05, 2.927593E+05, 2.980835E+05, 3.034878E+05, 3.089728E+05, 3.145398E+05, 
   3.201895E+05, 3.259230E+05, 3.317412E+05, 3.376451E+05, 3.436358E+05, 3.497142E+05, 
   3.558813E+05, 3.621382E+05, 3.684858E+05, 3.749252E+05, 3.814575E+05, 3.880837E+05, 
   3.948049E+05, 4.016221E+05, 4.085364E+05, 4.155490E+05, 4.226608E+05, 4.298730E+05, 
   4.371867E+05, 4.446031E+05, 4.521233E+05, 4.597484E+05, 4.674795E+05, 4.753179E+05, 
   4.832646E+05, 4.913209E+05, 4.994880E+05, 5.077670E+05, 5.161592E+05, 5.246658E+05, 
   5.332880E+05, 5.420269E+05, 5.508840E+05, 5.598604E+05, 5.689574E+05, 5.781763E+05, 
   5.875183E+05, 5.969847E+05, 6.065769E+05, 6.162961E+05, 6.261437E+05, 6.361210E+05, 
   6.462294E+05, 6.564702E+05, 6.668447E+05, 6.773543E+05, 6.880005E+05, 6.987846E+05, 
   7.097080E+05, 7.207721E+05, 7.319783E+05, 7.433282E+05, 7.548230E+05, 7.664644E+05, 
   7.782537E+05, 7.901924E+05, 8.022820E+05, 8.145241E+05, 8.269200E+05, 8.394714E+05, 
   8.521797E+05, 8.650466E+05, 8.780735E+05, 8.912620E+05, 9.046138E+05, 9.181303E+05, 
   9.318132E+05, 9.456641E+05, 9.596845E+05, 9.738762E+05, 9.882408E+05, 1.002780E+06, 
   1.017495E+06, 1.032388E+06, 1.047461E+06, 1.062715E+06, 1.078152E+06, 1.093774E+06, 
   1.109582E+06, 1.125578E+06, 1.141764E+06, 1.158142E+06, 1.174714E+06, 1.191480E+06, 
   1.208444E+06, 1.225606E+06, 1.242970E+06, 1.260536E+06, 1.278306E+06, 1.296283E+06, 
   1.314467E+06, 1.332862E+06, 1.351469E+06, 1.370290E+06, 1.389326E+06, 1.408581E+06, 
   1.428055E+06, 1.447750E+06, 1.467670E+06, 1.487815E+06, 1.508187E+06, 1.528790E+06, 
   1.549624E+06, 1.570691E+06, 1.591995E+06, 1.613536E+06, 1.635318E+06, 1.657342E+06, 
   1.679609E+06, 1.702123E+06, 1.724886E+06, 1.747899E+06, 1.771164E+06, 1.794685E+06, 
   1.818463E+06, 1.842500E+06, 1.866799E+06, 1.891362E+06, 1.916190E+06, 1.941287E+06, 
   1.966654E+06, 1.992295E+06, 2.018210E+06, 2.044403E+06, 2.070876E+06, 2.097630E+06, 
   2.124670E+06, 2.151996E+06, 2.179612E+06, 2.207519E+06, 2.235720E+06, 2.264218E+06, 
   2.293015E+06, 2.322113E+06, 2.351515E+06, 2.381224E+06, 2.411241E+06, 2.441570E+06, 
   2.472213E+06, 2.503172E+06, 2.534451E+06, 2.566051E+06, 2.597975E+06, 2.630227E+06, 
   2.662807E+06, 2.695720E+06, 2.728968E+06, 2.762553E+06, 2.796478E+06, 2.830746E+06, 
   2.865360E+06, 2.900322E+06, 2.935635E+06, 2.971302E+06, 3.007325E+06, 3.043708E+06, 
   3.080453E+06, 3.117563E+06, 3.155041E+06, 3.192890E+06, 3.231112E+06, 3.269712E+06, 
   3.308690E+06, 3.348050E+06, 3.387796E+06, 3.427931E+06, 3.468456E+06, 3.509376E+06, 
   3.550693E+06, 3.592410E+06, 3.634531E+06, 3.677058E+06, 3.719994E+06, 3.763343E+06, 
   3.807108E+06, 3.851291E+06, 3.895896E+06, 3.940927E+06, 3.986385E+06, 4.032275E+06, 
   4.078600E+06, 4.125363E+06, 4.172566E+06, 4.220215E+06, 4.268311E+06, 4.316858E+06, 
   4.365859E+06, 4.415318E+06, 4.465238E+06, 4.515622E+06, 4.566474E+06, 4.617797E+06, 
   4.669595E+06, 4.721871E+06, 4.774628E+06, 4.827871E+06, 4.881602E+06, 4.935825E+06, 
   4.990543E+06, 5.045761E+06, 5.101481E+06, 5.157708E+06, 5.214444E+06, 5.271694E+06, 
   5.329461E+06, 5.387749E+06, 5.446561E+06, 5.505902E+06, 5.565774E+06, 5.626182E+06, 
   5.687130E+06, 5.748621E+06, 5.810659E+06, 5.873247E+06, 5.936391E+06, 6.000092E+06, 
   6.064357E+06, 6.129187E+06, 6.194588E+06, 6.260563E+06, 6.327116E+06, 6.394251E+06, 
   6.461973E+06, 6.530284E+06, 6.599190E+06, 6.668693E+06, 6.738799E+06, 6.809511E+06, 
   6.880834E+06, 6.952772E+06, 7.025328E+06, 7.098507E+06, 7.172313E+06, 7.246751E+06, 
   7.321824E+06, 7.397537E+06, 7.473895E+06, 7.550900E+06, 7.628559E+06, 7.706875E+06, 
   7.785852E+06, 7.865495E+06, 7.945808E+06, 8.026796E+06, 8.108463E+06, 8.190814E+06, 
   8.273853E+06, 8.357585E+06, 8.442013E+06, 8.527144E+06, 8.612980E+06, 8.699528E+06, 
   8.786791E+06, 8.874775E+06, 8.963483E+06, 9.052920E+06, 9.143092E+06, 9.234003E+06, 
   9.325658E+06, 9.418061E+06, 9.511218E+06, 9.605132E+06, 9.699810E+06, 9.795255E+06, 
   9.891473E+06, 9.988469E+06, 1.008625E+07, 1.018481E+07, 1.028417E+07, 1.038433E+07, 
   1.048528E+07, 1.058705E+07, 1.068963E+07, 1.079303E+07, 1.089724E+07, 1.100229E+07, 
   1.110817E+07, 1.121489E+07, 1.132245E+07, 
])
# ============================ M = 52, I = 3 ============================
M = 52
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.594113E+00, 1.236785E+01, 2.929862E+01, 5.254960E+01, 8.022606E+01, 1.115873E+02, 
   1.462292E+02, 1.838669E+02, 2.242806E+02, 2.672935E+02, 3.127602E+02, 3.605599E+02, 
   4.105923E+02, 4.627755E+02, 5.170454E+02, 5.733542E+02, 6.316704E+02, 6.919775E+02, 
   7.542731E+02, 8.185680E+02, 8.848847E+02, 9.532558E+02, 1.023723E+03, 1.096337E+03, 
   1.171153E+03, 1.248234E+03, 1.327646E+03, 1.409460E+03, 1.493751E+03, 1.580597E+03, 
   1.670076E+03, 1.762271E+03, 1.857267E+03, 1.955148E+03, 2.056003E+03, 2.159920E+03, 
   2.266991E+03, 2.377308E+03, 2.490965E+03, 2.608056E+03, 2.728680E+03, 2.852935E+03, 
   2.980922E+03, 3.112742E+03, 3.248500E+03, 3.388302E+03, 3.532256E+03, 3.680470E+03, 
   3.833057E+03, 3.990131E+03, 4.151808E+03, 4.318205E+03, 4.489443E+03, 4.665644E+03, 
   4.846932E+03, 5.033435E+03, 5.225281E+03, 5.422602E+03, 5.625533E+03, 5.834208E+03, 
   6.048767E+03, 6.269352E+03, 6.496105E+03, 6.729173E+03, 6.968704E+03, 7.214851E+03, 
   7.467766E+03, 7.727606E+03, 7.994531E+03, 8.268702E+03, 8.550284E+03, 8.839444E+03, 
   9.136352E+03, 9.441180E+03, 9.754105E+03, 1.007531E+04, 1.040496E+04, 1.074326E+04, 
   1.109038E+04, 1.144653E+04, 1.181188E+04, 1.218664E+04, 1.257101E+04, 1.296518E+04, 
   1.336937E+04, 1.378378E+04, 1.420862E+04, 1.464411E+04, 1.509047E+04, 1.554791E+04, 
   1.601667E+04, 1.649696E+04, 1.698903E+04, 1.749310E+04, 1.800941E+04, 1.853820E+04, 
   1.907973E+04, 1.963423E+04, 2.020195E+04, 2.078316E+04, 2.137812E+04, 2.198707E+04, 
   2.261030E+04, 2.324806E+04, 2.390064E+04, 2.456831E+04, 2.525135E+04, 2.595004E+04, 
   2.666468E+04, 2.739555E+04, 2.814296E+04, 2.890719E+04, 2.968856E+04, 3.048738E+04, 
   3.130395E+04, 3.213860E+04, 3.299163E+04, 3.386338E+04, 3.475418E+04, 3.566435E+04, 
   3.659424E+04, 3.754418E+04, 3.851453E+04, 3.950562E+04, 4.051781E+04, 4.155147E+04, 
   4.260696E+04, 4.368463E+04, 4.478487E+04, 4.590805E+04, 4.705455E+04, 4.822475E+04, 
   4.941905E+04, 5.063785E+04, 5.188153E+04, 5.315050E+04, 5.444518E+04, 5.576598E+04, 
   5.711331E+04, 5.848760E+04, 5.988927E+04, 6.131877E+04, 6.277652E+04, 6.426297E+04, 
   6.577857E+04, 6.732378E+04, 6.889904E+04, 7.050483E+04, 7.214162E+04, 7.380987E+04, 
   7.551006E+04, 7.724269E+04, 7.900823E+04, 8.080720E+04, 8.264007E+04, 8.450737E+04, 
   8.640961E+04, 8.834729E+04, 9.032095E+04, 9.233111E+04, 9.437831E+04, 9.646308E+04, 
   9.858597E+04, 1.007475E+05, 1.029483E+05, 1.051889E+05, 1.074699E+05, 1.097917E+05, 
   1.121551E+05, 1.145606E+05, 1.170088E+05, 1.195002E+05, 1.220356E+05, 1.246155E+05, 
   1.272405E+05, 1.299112E+05, 1.326283E+05, 1.353924E+05, 1.382042E+05, 1.410643E+05, 
   1.439733E+05, 1.469319E+05, 1.499408E+05, 1.530007E+05, 1.561122E+05, 1.592760E+05, 
   1.624929E+05, 1.657634E+05, 1.690884E+05, 1.724685E+05, 1.759045E+05, 1.793971E+05, 
   1.829470E+05, 1.865549E+05, 1.902216E+05, 1.939480E+05, 1.977346E+05, 2.015823E+05, 
   2.054919E+05, 2.094642E+05, 2.134999E+05, 2.175999E+05, 2.217649E+05, 2.259957E+05, 
   2.302932E+05, 2.346583E+05, 2.390917E+05, 2.435943E+05, 2.481669E+05, 2.528104E+05, 
   2.575256E+05, 2.623135E+05, 2.671749E+05, 2.721107E+05, 2.771218E+05, 2.822091E+05, 
   2.873735E+05, 2.926160E+05, 2.979373E+05, 3.033386E+05, 3.088207E+05, 3.143846E+05, 
   3.200313E+05, 3.257616E+05, 3.315767E+05, 3.374774E+05, 3.434648E+05, 3.495398E+05, 
   3.557036E+05, 3.619570E+05, 3.683011E+05, 3.747370E+05, 3.812657E+05, 3.878883E+05, 
   3.946057E+05, 4.014192E+05, 4.083297E+05, 4.153383E+05, 4.224462E+05, 4.296544E+05, 
   4.369641E+05, 4.443763E+05, 4.518923E+05, 4.595131E+05, 4.672399E+05, 4.750739E+05, 
   4.830162E+05, 4.910680E+05, 4.992305E+05, 5.075049E+05, 5.158924E+05, 5.243942E+05, 
   5.330115E+05, 5.417455E+05, 5.505976E+05, 5.595689E+05, 5.686608E+05, 5.778744E+05, 
   5.872111E+05, 5.966722E+05, 6.062589E+05, 6.159727E+05, 6.258147E+05, 6.357863E+05, 
   6.458889E+05, 6.561238E+05, 6.664924E+05, 6.769961E+05, 6.876362E+05, 6.984141E+05, 
   7.093312E+05, 7.203890E+05, 7.315888E+05, 7.429321E+05, 7.544204E+05, 7.660550E+05, 
   7.778375E+05, 7.897694E+05, 8.018520E+05, 8.140870E+05, 8.264758E+05, 8.390199E+05, 
   8.517209E+05, 8.645803E+05, 8.775997E+05, 8.907806E+05, 9.041246E+05, 9.176332E+05, 
   9.313082E+05, 9.451510E+05, 9.591633E+05, 9.733467E+05, 9.877029E+05, 1.002234E+06, 
   1.016940E+06, 1.031825E+06, 1.046889E+06, 1.062134E+06, 1.077562E+06, 1.093174E+06, 
   1.108973E+06, 1.124960E+06, 1.141136E+06, 1.157505E+06, 1.174066E+06, 1.190823E+06, 
   1.207777E+06, 1.224929E+06, 1.242282E+06, 1.259838E+06, 1.277598E+06, 1.295564E+06, 
   1.313738E+06, 1.332122E+06, 1.350718E+06, 1.369527E+06, 1.388552E+06, 1.407795E+06, 
   1.427258E+06, 1.446942E+06, 1.466849E+06, 1.486982E+06, 1.507343E+06, 1.527933E+06, 
   1.548754E+06, 1.569810E+06, 1.591100E+06, 1.612629E+06, 1.634398E+06, 1.656408E+06, 
   1.678662E+06, 1.701163E+06, 1.723912E+06, 1.746911E+06, 1.770163E+06, 1.793669E+06, 
   1.817433E+06, 1.841456E+06, 1.865740E+06, 1.890288E+06, 1.915101E+06, 1.940183E+06, 
   1.965535E+06, 1.991160E+06, 2.017060E+06, 2.043237E+06, 2.069693E+06, 2.096432E+06, 
   2.123455E+06, 2.150765E+06, 2.178364E+06, 2.206254E+06, 2.234438E+06, 2.262919E+06, 
   2.291698E+06, 2.320779E+06, 2.350163E+06, 2.379854E+06, 2.409853E+06, 2.440163E+06, 
   2.470788E+06, 2.501728E+06, 2.532987E+06, 2.564568E+06, 2.596473E+06, 2.628705E+06, 
   2.661266E+06, 2.694158E+06, 2.727386E+06, 2.760950E+06, 2.794855E+06, 2.829102E+06, 
   2.863694E+06, 2.898635E+06, 2.933926E+06, 2.969571E+06, 3.005573E+06, 3.041933E+06, 
   3.078656E+06, 3.115743E+06, 3.153198E+06, 3.191024E+06, 3.229223E+06, 3.267798E+06, 
   3.306752E+06, 3.346089E+06, 3.385810E+06, 3.425920E+06, 3.466420E+06, 3.507315E+06, 
   3.548606E+06, 3.590298E+06, 3.632392E+06, 3.674893E+06, 3.717803E+06, 3.761125E+06, 
   3.804862E+06, 3.849018E+06, 3.893596E+06, 3.938598E+06, 3.984029E+06, 4.029890E+06, 
   4.076186E+06, 4.122920E+06, 4.170095E+06, 4.217713E+06, 4.265779E+06, 4.314296E+06, 
   4.363267E+06, 4.412695E+06, 4.462584E+06, 4.512937E+06, 4.563757E+06, 4.615048E+06, 
   4.666814E+06, 4.719057E+06, 4.771782E+06, 4.824991E+06, 4.878688E+06, 4.932878E+06, 
   4.987562E+06, 5.042745E+06, 5.098430E+06, 5.154622E+06, 5.211323E+06, 5.268537E+06, 
   5.326268E+06, 5.384519E+06, 5.443294E+06, 5.502598E+06, 5.562433E+06, 5.622803E+06, 
   5.683712E+06, 5.745165E+06, 5.807164E+06, 5.869713E+06, 5.932816E+06, 5.996478E+06, 
   6.060702E+06, 6.125492E+06, 6.190852E+06, 6.256785E+06, 6.323296E+06, 6.390389E+06, 
   6.458068E+06, 6.526336E+06, 6.595198E+06, 6.664658E+06, 6.734720E+06, 6.805387E+06, 
   6.876665E+06, 6.948557E+06, 7.021068E+06, 7.094200E+06, 7.167960E+06, 7.242351E+06, 
   7.317376E+06, 7.393042E+06, 7.469351E+06, 7.546308E+06, 7.623917E+06, 7.702183E+06, 
   7.781110E+06, 7.860703E+06, 7.940965E+06, 8.021902E+06, 8.103517E+06, 8.185816E+06, 
   8.268802E+06, 8.352480E+06, 8.436855E+06, 8.521931E+06, 8.607714E+06, 8.694206E+06, 
   8.781414E+06, 8.869341E+06, 8.957993E+06, 9.047374E+06, 9.137488E+06, 9.228341E+06, 
   9.319937E+06, 9.412282E+06, 9.505379E+06, 9.599233E+06, 9.693850E+06, 9.789235E+06, 
   9.885391E+06, 9.982325E+06, 1.008004E+07, 1.017854E+07, 1.027784E+07, 1.037793E+07, 
   1.047882E+07, 1.058052E+07, 1.068304E+07, 1.078637E+07, 1.089052E+07, 1.099550E+07, 
   1.110131E+07, 1.120796E+07, 1.131546E+07, 
])
# ============================ M = 52, I = 4 ============================
M = 52
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.594113E+01, 1.236785E+02, 2.929862E+02, 5.254960E+02, 8.022606E+02, 1.115873E+03, 
   1.462292E+03, 1.838669E+03, 2.242806E+03, 2.672935E+03, 3.127603E+03, 3.605600E+03, 
   4.105924E+03, 4.627759E+03, 5.170461E+03, 5.733556E+03, 6.316726E+03, 6.919810E+03, 
   7.542785E+03, 8.185758E+03, 8.848956E+03, 9.532707E+03, 1.023743E+04, 1.096362E+04, 
   1.171185E+04, 1.248274E+04, 1.327695E+04, 1.409520E+04, 1.493824E+04, 1.580682E+04, 
   1.670176E+04, 1.762388E+04, 1.857401E+04, 1.955303E+04, 2.056179E+04, 2.160119E+04, 
   2.267215E+04, 2.377559E+04, 2.491244E+04, 2.608367E+04, 2.729023E+04, 2.853313E+04, 
   2.981337E+04, 3.113197E+04, 3.248997E+04, 3.388843E+04, 3.532843E+04, 3.681107E+04, 
   3.833746E+04, 3.990874E+04, 4.152608E+04, 4.319065E+04, 4.490366E+04, 4.666633E+04, 
   4.847990E+04, 5.034565E+04, 5.226487E+04, 5.423888E+04, 5.626900E+04, 5.835662E+04, 
   6.050311E+04, 6.270989E+04, 6.497839E+04, 6.731009E+04, 6.970646E+04, 7.216902E+04, 
   7.469931E+04, 7.729890E+04, 7.996938E+04, 8.271237E+04, 8.552951E+04, 8.842248E+04, 
   9.139299E+04, 9.444276E+04, 9.757354E+04, 1.007871E+05, 1.040853E+05, 1.074700E+05, 
   1.109430E+05, 1.145063E+05, 1.181617E+05, 1.219112E+05, 1.257569E+05, 1.297007E+05, 
   1.337448E+05, 1.378911E+05, 1.421418E+05, 1.464991E+05, 1.509651E+05, 1.555420E+05, 
   1.602322E+05, 1.650378E+05, 1.699612E+05, 1.750048E+05, 1.801708E+05, 1.854618E+05, 
   1.908801E+05, 1.964283E+05, 2.021089E+05, 2.079244E+05, 2.138774E+05, 2.199706E+05, 
   2.262065E+05, 2.325880E+05, 2.391177E+05, 2.457984E+05, 2.526329E+05, 2.596241E+05, 
   2.667749E+05, 2.740881E+05, 2.815668E+05, 2.892138E+05, 2.970324E+05, 3.050256E+05, 
   3.131965E+05, 3.215482E+05, 3.300839E+05, 3.388070E+05, 3.477206E+05, 3.568282E+05, 
   3.661331E+05, 3.756386E+05, 3.853483E+05, 3.952657E+05, 4.053943E+05, 4.157376E+05, 
   4.262994E+05, 4.370833E+05, 4.480930E+05, 4.593322E+05, 4.708049E+05, 4.825148E+05, 
   4.944658E+05, 5.066619E+05, 5.191072E+05, 5.318055E+05, 5.447611E+05, 5.579781E+05, 
   5.714606E+05, 5.852129E+05, 5.992393E+05, 6.135442E+05, 6.281318E+05, 6.430066E+05, 
   6.581732E+05, 6.736360E+05, 6.893997E+05, 7.054689E+05, 7.218482E+05, 7.385425E+05, 
   7.555565E+05, 7.728951E+05, 7.905631E+05, 8.085655E+05, 8.269074E+05, 8.455938E+05, 
   8.646298E+05, 8.840206E+05, 9.037715E+05, 9.238876E+05, 9.443744E+05, 9.652373E+05, 
   9.864817E+05, 1.008113E+06, 1.030137E+06, 1.052559E+06, 1.075386E+06, 1.098621E+06, 
   1.122273E+06, 1.146345E+06, 1.170845E+06, 1.195778E+06, 1.221151E+06, 1.246969E+06, 
   1.273238E+06, 1.299966E+06, 1.327157E+06, 1.354819E+06, 1.382958E+06, 1.411580E+06, 
   1.440693E+06, 1.470301E+06, 1.500413E+06, 1.531035E+06, 1.562174E+06, 1.593836E+06, 
   1.626030E+06, 1.658760E+06, 1.692035E+06, 1.725863E+06, 1.760249E+06, 1.795201E+06, 
   1.830728E+06, 1.866835E+06, 1.903531E+06, 1.940823E+06, 1.978719E+06, 2.017226E+06, 
   2.056353E+06, 2.096106E+06, 2.136495E+06, 2.177526E+06, 2.219209E+06, 2.261551E+06, 
   2.304560E+06, 2.348245E+06, 2.392614E+06, 2.437675E+06, 2.483438E+06, 2.529910E+06, 
   2.577100E+06, 2.625017E+06, 2.673669E+06, 2.723067E+06, 2.773218E+06, 2.824131E+06, 
   2.875816E+06, 2.928283E+06, 2.981539E+06, 3.035596E+06, 3.090461E+06, 3.146145E+06, 
   3.202656E+06, 3.260006E+06, 3.318204E+06, 3.377259E+06, 3.437181E+06, 3.497981E+06, 
   3.559668E+06, 3.622254E+06, 3.685747E+06, 3.750158E+06, 3.815498E+06, 3.881778E+06, 
   3.949008E+06, 4.017198E+06, 4.086360E+06, 4.156504E+06, 4.227641E+06, 4.299782E+06, 
   4.372939E+06, 4.447123E+06, 4.522345E+06, 4.598616E+06, 4.675948E+06, 4.754353E+06, 
   4.833842E+06, 4.914427E+06, 4.996120E+06, 5.078932E+06, 5.162877E+06, 5.247966E+06, 
   5.334210E+06, 5.421624E+06, 5.510219E+06, 5.600007E+06, 5.691002E+06, 5.783216E+06, 
   5.876661E+06, 5.971351E+06, 6.067299E+06, 6.164518E+06, 6.263021E+06, 6.362822E+06, 
   6.463933E+06, 6.566369E+06, 6.670142E+06, 6.775268E+06, 6.881759E+06, 6.989629E+06, 
   7.098893E+06, 7.209565E+06, 7.321658E+06, 7.435188E+06, 7.550169E+06, 7.666614E+06, 
   7.784540E+06, 7.903960E+06, 8.024890E+06, 8.147344E+06, 8.271338E+06, 8.396887E+06, 
   8.524006E+06, 8.652710E+06, 8.783016E+06, 8.914938E+06, 9.048493E+06, 9.183696E+06, 
   9.320563E+06, 9.459110E+06, 9.599354E+06, 9.741311E+06, 9.884998E+06, 1.003043E+07, 
   1.017762E+07, 1.032660E+07, 1.047737E+07, 1.062995E+07, 1.078436E+07, 1.094062E+07, 
   1.109875E+07, 1.125876E+07, 1.142066E+07, 1.158449E+07, 1.175025E+07, 1.191796E+07, 
   1.208765E+07, 1.225932E+07, 1.243301E+07, 1.260872E+07, 1.278647E+07, 1.296629E+07, 
   1.314819E+07, 1.333219E+07, 1.351831E+07, 1.370657E+07, 1.389699E+07, 1.408959E+07, 
   1.428438E+07, 1.448140E+07, 1.468065E+07, 1.488215E+07, 1.508594E+07, 1.529202E+07, 
   1.550042E+07, 1.571116E+07, 1.592426E+07, 1.613973E+07, 1.635761E+07, 1.657791E+07, 
   1.680065E+07, 1.702586E+07, 1.725355E+07, 1.748374E+07, 1.771647E+07, 1.795174E+07, 
   1.818959E+07, 1.843003E+07, 1.867309E+07, 1.891879E+07, 1.916714E+07, 1.941818E+07, 
   1.967193E+07, 1.992841E+07, 2.018764E+07, 2.044964E+07, 2.071445E+07, 2.098208E+07, 
   2.125255E+07, 2.152589E+07, 2.180212E+07, 2.208128E+07, 2.236337E+07, 2.264844E+07, 
   2.293649E+07, 2.322756E+07, 2.352166E+07, 2.381884E+07, 2.411910E+07, 2.442248E+07, 
   2.472899E+07, 2.503868E+07, 2.535156E+07, 2.566765E+07, 2.598699E+07, 2.630959E+07, 
   2.663550E+07, 2.696472E+07, 2.729730E+07, 2.763324E+07, 2.797260E+07, 2.831538E+07, 
   2.866162E+07, 2.901134E+07, 2.936458E+07, 2.972135E+07, 3.008169E+07, 3.044563E+07, 
   3.081319E+07, 3.118440E+07, 3.155929E+07, 3.193789E+07, 3.232022E+07, 3.270633E+07, 
   3.309623E+07, 3.348995E+07, 3.388753E+07, 3.428899E+07, 3.469437E+07, 3.510368E+07, 
   3.551698E+07, 3.593427E+07, 3.635560E+07, 3.678100E+07, 3.721049E+07, 3.764411E+07, 
   3.808189E+07, 3.852385E+07, 3.897004E+07, 3.942048E+07, 3.987520E+07, 4.033423E+07, 
   4.079762E+07, 4.126539E+07, 4.173757E+07, 4.221419E+07, 4.269529E+07, 4.318091E+07, 
   4.367107E+07, 4.416580E+07, 4.466515E+07, 4.516915E+07, 4.567782E+07, 4.619121E+07, 
   4.670934E+07, 4.723226E+07, 4.775999E+07, 4.829257E+07, 4.883004E+07, 4.937244E+07, 
   4.991978E+07, 5.047213E+07, 5.102950E+07, 5.159193E+07, 5.215947E+07, 5.273214E+07, 
   5.330998E+07, 5.389304E+07, 5.448134E+07, 5.507492E+07, 5.567383E+07, 5.627809E+07, 
   5.688775E+07, 5.750285E+07, 5.812341E+07, 5.874949E+07, 5.938111E+07, 6.001832E+07, 
   6.066116E+07, 6.130966E+07, 6.196387E+07, 6.262382E+07, 6.328955E+07, 6.396111E+07, 
   6.463852E+07, 6.532185E+07, 6.601111E+07, 6.670636E+07, 6.740763E+07, 6.811497E+07, 
   6.882841E+07, 6.954801E+07, 7.027379E+07, 7.100580E+07, 7.174409E+07, 7.248870E+07, 
   7.323966E+07, 7.399702E+07, 7.476082E+07, 7.553112E+07, 7.630794E+07, 7.709133E+07, 
   7.788135E+07, 7.867802E+07, 7.948140E+07, 8.029153E+07, 8.110845E+07, 8.193221E+07, 
   8.276285E+07, 8.360042E+07, 8.444497E+07, 8.529653E+07, 8.615516E+07, 8.702090E+07, 
   8.789380E+07, 8.877390E+07, 8.966126E+07, 9.055591E+07, 9.145791E+07, 9.236729E+07, 
   9.328412E+07, 9.420844E+07, 9.514029E+07, 9.607972E+07, 9.702679E+07, 9.798153E+07, 
   9.894401E+07, 9.991426E+07, 1.008924E+08, 1.018783E+08, 1.028722E+08, 1.038741E+08, 
   1.048840E+08, 1.059019E+08, 1.069280E+08, 1.079623E+08, 1.090048E+08, 1.100556E+08, 
   1.111148E+08, 1.121823E+08, 1.132582E+08, 
])
# ============================ M = 52, I = 5 ============================
M = 52
I = 5
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   4.594113E+00, 1.236785E+01, 2.929862E+01, 5.254960E+01, 8.022606E+01, 1.115873E+02, 
   1.462292E+02, 1.838669E+02, 2.242806E+02, 2.672935E+02, 3.127603E+02, 3.605601E+02, 
   4.105926E+02, 4.627762E+02, 5.170468E+02, 5.733568E+02, 6.316747E+02, 6.919843E+02, 
   7.542835E+02, 8.185831E+02, 8.849057E+02, 9.532845E+02, 1.023761E+03, 1.096386E+03, 
   1.171215E+03, 1.248312E+03, 1.327742E+03, 1.409576E+03, 1.493890E+03, 1.580762E+03, 
   1.670269E+03, 1.762496E+03, 1.857526E+03, 1.955445E+03, 2.056341E+03, 2.160303E+03, 
   2.267422E+03, 2.377791E+03, 2.491503E+03, 2.608654E+03, 2.729341E+03, 2.853663E+03, 
   2.981721E+03, 3.113618E+03, 3.249456E+03, 3.389343E+03, 3.533386E+03, 3.681695E+03, 
   3.834382E+03, 3.991561E+03, 4.153348E+03, 4.319860E+03, 4.491220E+03, 4.667547E+03, 
   4.848969E+03, 5.035610E+03, 5.227602E+03, 5.425076E+03, 5.628165E+03, 5.837006E+03, 
   6.051738E+03, 6.272502E+03, 6.499443E+03, 6.732706E+03, 6.972440E+03, 7.218798E+03, 
   7.471932E+03, 7.732001E+03, 7.999162E+03, 8.273579E+03, 8.555416E+03, 8.844841E+03, 
   9.142023E+03, 9.447137E+03, 9.760357E+03, 1.008186E+04, 1.041183E+04, 1.075046E+04, 
   1.109792E+04, 1.145442E+04, 1.182013E+04, 1.219527E+04, 1.258002E+04, 1.297459E+04, 
   1.337920E+04, 1.379403E+04, 1.421932E+04, 1.465526E+04, 1.510209E+04, 1.556002E+04, 
   1.602927E+04, 1.651008E+04, 1.700268E+04, 1.750730E+04, 1.802417E+04, 1.855355E+04, 
   1.909567E+04, 1.965079E+04, 2.021915E+04, 2.080101E+04, 2.139664E+04, 2.200629E+04, 
   2.263023E+04, 2.326872E+04, 2.392206E+04, 2.459050E+04, 2.527433E+04, 2.597385E+04, 
   2.668932E+04, 2.742106E+04, 2.816935E+04, 2.893450E+04, 2.971681E+04, 3.051659E+04, 
   3.133415E+04, 3.216980E+04, 3.302388E+04, 3.389670E+04, 3.478859E+04, 3.569988E+04, 
   3.663092E+04, 3.758204E+04, 3.855360E+04, 3.954593E+04, 4.055940E+04, 4.159436E+04, 
   4.265118E+04, 4.373023E+04, 4.483187E+04, 4.595648E+04, 4.710445E+04, 4.827617E+04, 
   4.947201E+04, 5.069238E+04, 5.193768E+04, 5.320832E+04, 5.450469E+04, 5.582722E+04, 
   5.717632E+04, 5.855243E+04, 5.995596E+04, 6.138735E+04, 6.284705E+04, 6.433549E+04, 
   6.585312E+04, 6.740040E+04, 6.897779E+04, 7.058575E+04, 7.222475E+04, 7.389526E+04, 
   7.559777E+04, 7.733276E+04, 7.910073E+04, 8.090216E+04, 8.273756E+04, 8.460743E+04, 
   8.651230E+04, 8.845267E+04, 9.042907E+04, 9.244203E+04, 9.449208E+04, 9.657977E+04, 
   9.870563E+04, 1.008702E+05, 1.030741E+05, 1.053179E+05, 1.076020E+05, 1.099272E+05, 
   1.122939E+05, 1.147029E+05, 1.171545E+05, 1.196496E+05, 1.221886E+05, 1.247721E+05, 
   1.274009E+05, 1.300755E+05, 1.327965E+05, 1.355646E+05, 1.383805E+05, 1.412447E+05, 
   1.441579E+05, 1.471209E+05, 1.501342E+05, 1.531985E+05, 1.563146E+05, 1.594831E+05, 
   1.627046E+05, 1.659800E+05, 1.693099E+05, 1.726950E+05, 1.761361E+05, 1.796338E+05, 
   1.831890E+05, 1.868023E+05, 1.904745E+05, 1.942064E+05, 1.979987E+05, 2.018522E+05, 
   2.057677E+05, 2.097459E+05, 2.137877E+05, 2.178938E+05, 2.220651E+05, 2.263024E+05, 
   2.306064E+05, 2.349781E+05, 2.394182E+05, 2.439276E+05, 2.485072E+05, 2.531578E+05, 
   2.578802E+05, 2.626755E+05, 2.675443E+05, 2.724877E+05, 2.775065E+05, 2.826016E+05, 
   2.877739E+05, 2.930244E+05, 2.983540E+05, 3.037637E+05, 3.092543E+05, 3.148268E+05, 
   3.204822E+05, 3.262215E+05, 3.320455E+05, 3.379555E+05, 3.439522E+05, 3.500367E+05, 
   3.562101E+05, 3.624733E+05, 3.688274E+05, 3.752734E+05, 3.818123E+05, 3.884453E+05, 
   3.951733E+05, 4.019975E+05, 4.089189E+05, 4.159386E+05, 4.230578E+05, 4.302774E+05, 
   4.375987E+05, 4.450227E+05, 4.525506E+05, 4.601835E+05, 4.679227E+05, 4.757691E+05, 
   4.837241E+05, 4.917888E+05, 4.999643E+05, 5.082520E+05, 5.166529E+05, 5.251683E+05, 
   5.337994E+05, 5.425475E+05, 5.514138E+05, 5.603996E+05, 5.695061E+05, 5.787346E+05, 
   5.880864E+05, 5.975628E+05, 6.071650E+05, 6.168945E+05, 6.267524E+05, 6.367403E+05, 
   6.468593E+05, 6.571108E+05, 6.674963E+05, 6.780171E+05, 6.886745E+05, 6.994700E+05, 
   7.104049E+05, 7.214808E+05, 7.326989E+05, 7.440608E+05, 7.555679E+05, 7.672216E+05, 
   7.790235E+05, 7.909749E+05, 8.030774E+05, 8.153326E+05, 8.277418E+05, 8.403066E+05, 
   8.530285E+05, 8.659091E+05, 8.789500E+05, 8.921527E+05, 9.055188E+05, 9.190498E+05, 
   9.327474E+05, 9.466132E+05, 9.606488E+05, 9.748558E+05, 9.892359E+05, 1.003791E+06, 
   1.018522E+06, 1.033431E+06, 1.048520E+06, 1.063791E+06, 1.079244E+06, 1.094883E+06, 
   1.110708E+06, 1.126722E+06, 1.142926E+06, 1.159321E+06, 1.175911E+06, 1.192696E+06, 
   1.209678E+06, 1.226859E+06, 1.244241E+06, 1.261826E+06, 1.279616E+06, 1.297612E+06, 
   1.315817E+06, 1.334232E+06, 1.352859E+06, 1.371701E+06, 1.390758E+06, 1.410034E+06, 
   1.429529E+06, 1.449246E+06, 1.469188E+06, 1.489355E+06, 1.509750E+06, 1.530375E+06, 
   1.551232E+06, 1.572323E+06, 1.593650E+06, 1.615215E+06, 1.637021E+06, 1.659069E+06, 
   1.681361E+06, 1.703900E+06, 1.726688E+06, 1.749726E+06, 1.773018E+06, 1.796565E+06, 
   1.820369E+06, 1.844433E+06, 1.868759E+06, 1.893348E+06, 1.918205E+06, 1.943329E+06, 
   1.968725E+06, 1.994394E+06, 2.020338E+06, 2.046560E+06, 2.073063E+06, 2.099848E+06, 
   2.126917E+06, 2.154274E+06, 2.181920E+06, 2.209859E+06, 2.238092E+06, 2.266622E+06, 
   2.295451E+06, 2.324582E+06, 2.354017E+06, 2.383759E+06, 2.413810E+06, 2.444173E+06, 
   2.474850E+06, 2.505845E+06, 2.537158E+06, 2.568794E+06, 2.600754E+06, 2.633042E+06, 
   2.665660E+06, 2.698610E+06, 2.731895E+06, 2.765518E+06, 2.799482E+06, 2.833788E+06, 
   2.868441E+06, 2.903443E+06, 2.938796E+06, 2.974503E+06, 3.010568E+06, 3.046992E+06, 
   3.083778E+06, 3.120931E+06, 3.158451E+06, 3.196343E+06, 3.234609E+06, 3.273252E+06, 
   3.312274E+06, 3.351680E+06, 3.391471E+06, 3.431651E+06, 3.472223E+06, 3.513190E+06, 
   3.554554E+06, 3.596319E+06, 3.638487E+06, 3.681063E+06, 3.724048E+06, 3.767447E+06, 
   3.811261E+06, 3.855495E+06, 3.900152E+06, 3.945234E+06, 3.990744E+06, 4.036687E+06, 
   4.083065E+06, 4.129881E+06, 4.177139E+06, 4.224842E+06, 4.272994E+06, 4.321596E+06, 
   4.370654E+06, 4.420170E+06, 4.470147E+06, 4.520589E+06, 4.571500E+06, 4.622882E+06, 
   4.674740E+06, 4.727076E+06, 4.779894E+06, 4.833198E+06, 4.886991E+06, 4.941277E+06, 
   4.996059E+06, 5.051340E+06, 5.107125E+06, 5.163416E+06, 5.220218E+06, 5.277534E+06, 
   5.335368E+06, 5.393724E+06, 5.452604E+06, 5.512014E+06, 5.571956E+06, 5.632434E+06, 
   5.693452E+06, 5.755014E+06, 5.817124E+06, 5.879786E+06, 5.943003E+06, 6.006778E+06, 
   6.071118E+06, 6.136024E+06, 6.201500E+06, 6.267552E+06, 6.334183E+06, 6.401396E+06, 
   6.469196E+06, 6.537588E+06, 6.606573E+06, 6.676158E+06, 6.746346E+06, 6.817141E+06, 
   6.888547E+06, 6.960568E+06, 7.033209E+06, 7.106474E+06, 7.180367E+06, 7.254891E+06, 
   7.330053E+06, 7.405854E+06, 7.482301E+06, 7.559397E+06, 7.637147E+06, 7.715554E+06, 
   7.794624E+06, 7.874360E+06, 7.954768E+06, 8.035851E+06, 8.117614E+06, 8.200061E+06, 
   8.283198E+06, 8.367028E+06, 8.451556E+06, 8.536786E+06, 8.622724E+06, 8.709373E+06, 
   8.796739E+06, 8.884826E+06, 8.973639E+06, 9.063182E+06, 9.153460E+06, 9.244478E+06, 
   9.336241E+06, 9.428753E+06, 9.522019E+06, 9.616045E+06, 9.710834E+06, 9.806392E+06, 
   9.902724E+06, 9.999834E+06, 1.009773E+07, 1.019641E+07, 1.029589E+07, 1.039616E+07, 
   1.049724E+07, 1.059912E+07, 1.070182E+07, 1.080534E+07, 1.090968E+07, 1.101486E+07, 
   1.112086E+07, 1.122771E+07, 1.133540E+07, 
])
# ============================ M = 53, I = 1 ============================
M = 53
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   3.356653E+00, 3.201634E+01, 6.387248E+01, 9.573604E+01, 1.276072E+02, 1.594922E+02, 
   1.914279E+02, 2.235240E+02, 2.559897E+02, 2.891286E+02, 3.233114E+02, 3.589429E+02, 
   3.964357E+02, 4.361925E+02, 4.785969E+02, 5.240104E+02, 5.727728E+02, 6.252045E+02, 
   6.816097E+02, 7.422802E+02, 8.074983E+02, 8.775397E+02, 9.526762E+02, 1.033178E+03, 
   1.119315E+03, 1.211358E+03, 1.309582E+03, 1.414265E+03, 1.525689E+03, 1.644142E+03, 
   1.769918E+03, 1.903318E+03, 2.044649E+03, 2.194227E+03, 2.352373E+03, 2.519419E+03, 
   2.695705E+03, 2.881577E+03, 3.077393E+03, 3.283518E+03, 3.500327E+03, 3.728206E+03, 
   3.967546E+03, 4.218753E+03, 4.482240E+03, 4.758431E+03, 5.047759E+03, 5.350669E+03, 
   5.667614E+03, 5.999061E+03, 6.345484E+03, 6.707371E+03, 7.085219E+03, 7.479536E+03, 
   7.890842E+03, 8.319668E+03, 8.766557E+03, 9.232062E+03, 9.716749E+03, 1.022119E+04, 
   1.074599E+04, 1.129173E+04, 1.185903E+04, 1.244853E+04, 1.306084E+04, 1.369663E+04, 
   1.435656E+04, 1.504130E+04, 1.575154E+04, 1.648799E+04, 1.725134E+04, 1.804234E+04, 
   1.886172E+04, 1.971024E+04, 2.058865E+04, 2.149775E+04, 2.243833E+04, 2.341119E+04, 
   2.441716E+04, 2.545708E+04, 2.653178E+04, 2.764214E+04, 2.878904E+04, 2.997337E+04, 
   3.119604E+04, 3.245796E+04, 3.376009E+04, 3.510336E+04, 3.648876E+04, 3.791725E+04, 
   3.938985E+04, 4.090756E+04, 4.247141E+04, 4.408245E+04, 4.574174E+04, 4.745036E+04, 
   4.920940E+04, 5.101997E+04, 5.288319E+04, 5.480021E+04, 5.677218E+04, 5.880029E+04, 
   6.088571E+04, 6.302967E+04, 6.523339E+04, 6.749811E+04, 6.982510E+04, 7.221562E+04, 
   7.467098E+04, 7.719249E+04, 7.978148E+04, 8.243931E+04, 8.516732E+04, 8.796693E+04, 
   9.083951E+04, 9.378651E+04, 9.680936E+04, 9.990951E+04, 1.030884E+05, 1.063477E+05, 
   1.096887E+05, 1.131131E+05, 1.166223E+05, 1.202180E+05, 1.239018E+05, 1.276752E+05, 
   1.315399E+05, 1.354976E+05, 1.395499E+05, 1.436986E+05, 1.479452E+05, 1.522917E+05, 
   1.567396E+05, 1.612909E+05, 1.659473E+05, 1.707105E+05, 1.755825E+05, 1.805651E+05, 
   1.856602E+05, 1.908697E+05, 1.961955E+05, 2.016395E+05, 2.072038E+05, 2.128903E+05, 
   2.187010E+05, 2.246379E+05, 2.307031E+05, 2.368988E+05, 2.432269E+05, 2.496896E+05, 
   2.562891E+05, 2.630275E+05, 2.699070E+05, 2.769298E+05, 2.840982E+05, 2.914145E+05, 
   2.988808E+05, 3.064995E+05, 3.142730E+05, 3.222036E+05, 3.302937E+05, 3.385457E+05, 
   3.469620E+05, 3.555451E+05, 3.642974E+05, 3.732215E+05, 3.823198E+05, 3.915950E+05, 
   4.010497E+05, 4.106863E+05, 4.205075E+05, 4.305161E+05, 4.407147E+05, 4.511059E+05, 
   4.616925E+05, 4.724773E+05, 4.834630E+05, 4.946525E+05, 5.060486E+05, 5.176542E+05, 
   5.294721E+05, 5.415053E+05, 5.537566E+05, 5.662292E+05, 5.789259E+05, 5.918498E+05, 
   6.050039E+05, 6.183914E+05, 6.320153E+05, 6.458788E+05, 6.599849E+05, 6.743371E+05, 
   6.889383E+05, 7.037919E+05, 7.189012E+05, 7.342694E+05, 7.498999E+05, 7.657961E+05, 
   7.819613E+05, 7.983990E+05, 8.151126E+05, 8.321056E+05, 8.493814E+05, 8.669437E+05, 
   8.847960E+05, 9.029419E+05, 9.213850E+05, 9.401290E+05, 9.591776E+05, 9.785344E+05, 
   9.982033E+05, 1.018188E+06, 1.038492E+06, 1.059120E+06, 1.080075E+06, 1.101361E+06, 
   1.122983E+06, 1.144943E+06, 1.167247E+06, 1.189898E+06, 1.212900E+06, 1.236258E+06, 
   1.259974E+06, 1.284055E+06, 1.308503E+06, 1.333323E+06, 1.358519E+06, 1.384096E+06, 
   1.410058E+06, 1.436409E+06, 1.463154E+06, 1.490296E+06, 1.517841E+06, 1.545793E+06, 
   1.574156E+06, 1.602936E+06, 1.632136E+06, 1.661761E+06, 1.691816E+06, 1.722306E+06, 
   1.753235E+06, 1.784608E+06, 1.816429E+06, 1.848705E+06, 1.881439E+06, 1.914636E+06, 
   1.948302E+06, 1.982441E+06, 2.017058E+06, 2.052159E+06, 2.087748E+06, 2.123830E+06, 
   2.160412E+06, 2.197497E+06, 2.235091E+06, 2.273199E+06, 2.311827E+06, 2.350980E+06, 
   2.390664E+06, 2.430883E+06, 2.471643E+06, 2.512950E+06, 2.554809E+06, 2.597225E+06, 
   2.640205E+06, 2.683754E+06, 2.727877E+06, 2.772580E+06, 2.817870E+06, 2.863751E+06, 
   2.910229E+06, 2.957311E+06, 3.005002E+06, 3.053309E+06, 3.102237E+06, 3.151792E+06, 
   3.201980E+06, 3.252807E+06, 3.304280E+06, 3.356404E+06, 3.409186E+06, 3.462632E+06, 
   3.516748E+06, 3.571541E+06, 3.627017E+06, 3.683183E+06, 3.740044E+06, 3.797607E+06, 
   3.855879E+06, 3.914866E+06, 3.974576E+06, 4.035014E+06, 4.096187E+06, 4.158102E+06, 
   4.220766E+06, 4.284186E+06, 4.348367E+06, 4.413318E+06, 4.479046E+06, 4.545556E+06, 
   4.612856E+06, 4.680954E+06, 4.749856E+06, 4.819570E+06, 4.890102E+06, 4.961460E+06, 
   5.033650E+06, 5.106682E+06, 5.180561E+06, 5.255295E+06, 5.330892E+06, 5.407358E+06, 
   5.484703E+06, 5.562932E+06, 5.642054E+06, 5.722076E+06, 5.803007E+06, 5.884853E+06, 
   5.967622E+06, 6.051323E+06, 6.135964E+06, 6.221551E+06, 6.308093E+06, 6.395598E+06, 
   6.484075E+06, 6.573530E+06, 6.663973E+06, 6.755412E+06, 6.847854E+06, 6.941308E+06, 
   7.035782E+06, 7.131285E+06, 7.227825E+06, 7.325410E+06, 7.424049E+06, 7.523751E+06, 
   7.624523E+06, 7.726375E+06, 7.829316E+06, 7.933354E+06, 8.038497E+06, 8.144755E+06, 
   8.252136E+06, 8.360650E+06, 8.470304E+06, 8.581110E+06, 8.693074E+06, 8.806207E+06, 
   8.920517E+06, 9.036014E+06, 9.152706E+06, 9.270604E+06, 9.389716E+06, 9.510052E+06, 
   9.631622E+06, 9.754434E+06, 9.878498E+06, 1.000382E+07, 1.013042E+07, 1.025830E+07, 
   1.038747E+07, 1.051794E+07, 1.064972E+07, 1.078282E+07, 1.091724E+07, 1.105301E+07, 
   1.119013E+07, 1.132861E+07, 1.146846E+07, 1.160969E+07, 1.175231E+07, 1.189632E+07, 
   1.204175E+07, 1.218860E+07, 1.233689E+07, 1.248661E+07, 1.263779E+07, 1.279042E+07, 
   1.294454E+07, 1.310013E+07, 1.325722E+07, 1.341582E+07, 1.357593E+07, 1.373757E+07, 
   1.390074E+07, 1.406547E+07, 1.423175E+07, 1.439961E+07, 1.456904E+07, 1.474007E+07, 
   1.491270E+07, 1.508695E+07, 1.526282E+07, 1.544033E+07, 1.561949E+07, 1.580032E+07, 
   1.598281E+07, 1.616698E+07, 1.635285E+07, 1.654043E+07, 1.672972E+07, 1.692075E+07, 
   1.711351E+07, 1.730803E+07, 1.750432E+07, 1.770238E+07, 1.790223E+07, 1.810388E+07, 
   1.830734E+07, 1.851264E+07, 1.871976E+07, 1.892874E+07, 1.913958E+07, 1.935229E+07, 
   1.956689E+07, 1.978339E+07, 2.000180E+07, 2.022213E+07, 2.044440E+07, 2.066862E+07, 
   2.089480E+07, 2.112296E+07, 2.135310E+07, 2.158524E+07, 2.181940E+07, 2.205558E+07, 
   2.229380E+07, 2.253407E+07, 2.277641E+07, 2.302082E+07, 2.326733E+07, 2.351594E+07, 
   2.376667E+07, 2.401953E+07, 2.427454E+07, 2.453170E+07, 2.479103E+07, 2.505255E+07, 
   2.531627E+07, 2.558220E+07, 2.585036E+07, 2.612076E+07, 2.639341E+07, 2.666833E+07, 
   2.694553E+07, 2.722502E+07, 2.750683E+07, 2.779096E+07, 2.807742E+07, 2.836624E+07, 
   2.865742E+07, 2.895099E+07, 2.924695E+07, 2.954532E+07, 2.984611E+07, 3.014934E+07, 
   3.045502E+07, 3.076317E+07, 3.107380E+07, 3.138693E+07, 3.170257E+07, 3.202074E+07, 
   3.234145E+07, 3.266471E+07, 3.299055E+07, 3.331897E+07, 3.364999E+07, 3.398363E+07, 
   3.431990E+07, 3.465882E+07, 3.500040E+07, 3.534465E+07, 3.569160E+07, 3.604126E+07, 
   3.639364E+07, 3.674876E+07, 3.710664E+07, 3.746729E+07, 3.783072E+07, 3.819696E+07, 
   3.856601E+07, 3.893790E+07, 3.931263E+07, 3.969024E+07, 4.007072E+07, 4.045410E+07, 
   4.084040E+07, 4.122963E+07, 4.162181E+07, 4.201694E+07, 4.241506E+07, 4.281617E+07, 
   4.322030E+07, 4.362745E+07, 4.403765E+07, 4.445091E+07, 4.486726E+07, 4.528669E+07, 
   4.570924E+07, 4.613492E+07, 4.656374E+07, 
])
# ============================ M = 53, I = 2 ============================
M = 53
I = 2
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.906550E+00, 6.596858E+01, 1.316172E+02, 1.972811E+02, 2.629603E+02, 3.286614E+02, 
   3.944232E+02, 4.603607E+02, 5.266934E+02, 5.937410E+02, 6.618964E+02, 7.315933E+02, 
   8.032792E+02, 8.773970E+02, 9.543752E+02, 1.034623E+03, 1.118530E+03, 1.206466E+03, 
   1.298787E+03, 1.395833E+03, 1.497934E+03, 1.605413E+03, 1.718586E+03, 1.837766E+03, 
   1.963264E+03, 2.095392E+03, 2.234461E+03, 2.380787E+03, 2.534688E+03, 2.696489E+03, 
   2.866516E+03, 3.045106E+03, 3.232598E+03, 3.429341E+03, 3.635691E+03, 3.852012E+03, 
   4.078674E+03, 4.316060E+03, 4.564557E+03, 4.824565E+03, 5.096492E+03, 5.380755E+03, 
   5.677781E+03, 5.988008E+03, 6.311883E+03, 6.649864E+03, 7.002420E+03, 7.370028E+03, 
   7.753179E+03, 8.152373E+03, 8.568122E+03, 9.000949E+03, 9.451387E+03, 9.919983E+03, 
   1.040729E+04, 1.091389E+04, 1.144035E+04, 1.198726E+04, 1.255524E+04, 1.314489E+04, 
   1.375686E+04, 1.439177E+04, 1.505029E+04, 1.573308E+04, 1.644082E+04, 1.717420E+04, 
   1.793394E+04, 1.872074E+04, 1.953535E+04, 2.037851E+04, 2.125097E+04, 2.215352E+04, 
   2.308693E+04, 2.405201E+04, 2.504958E+04, 2.608047E+04, 2.714551E+04, 2.824557E+04, 
   2.938152E+04, 3.055424E+04, 3.176465E+04, 3.301365E+04, 3.430218E+04, 3.563119E+04, 
   3.700164E+04, 3.841451E+04, 3.987080E+04, 4.137150E+04, 4.291766E+04, 4.451031E+04, 
   4.615050E+04, 4.783932E+04, 4.957786E+04, 5.136721E+04, 5.320851E+04, 5.510290E+04, 
   5.705153E+04, 5.905557E+04, 6.111623E+04, 6.323470E+04, 6.541221E+04, 6.765001E+04, 
   6.994937E+04, 7.231155E+04, 7.473785E+04, 7.722960E+04, 7.978812E+04, 8.241476E+04, 
   8.511090E+04, 8.787793E+04, 9.071724E+04, 9.363028E+04, 9.661848E+04, 9.968331E+04, 
   1.028262E+05, 1.060488E+05, 1.093525E+05, 1.127389E+05, 1.162095E+05, 1.197659E+05, 
   1.234098E+05, 1.271426E+05, 1.309662E+05, 1.348822E+05, 1.388921E+05, 1.429978E+05, 
   1.472010E+05, 1.515034E+05, 1.559067E+05, 1.604128E+05, 1.650235E+05, 1.697406E+05, 
   1.745659E+05, 1.795014E+05, 1.845488E+05, 1.897102E+05, 1.949875E+05, 2.003827E+05, 
   2.058976E+05, 2.115344E+05, 2.172950E+05, 2.231816E+05, 2.291961E+05, 2.353408E+05, 
   2.416176E+05, 2.480288E+05, 2.545765E+05, 2.612629E+05, 2.680902E+05, 2.750607E+05, 
   2.821767E+05, 2.894403E+05, 2.968540E+05, 3.044200E+05, 3.121408E+05, 3.200187E+05, 
   3.280561E+05, 3.362554E+05, 3.446192E+05, 3.531498E+05, 3.618499E+05, 3.707219E+05, 
   3.797684E+05, 3.889920E+05, 3.983953E+05, 4.079809E+05, 4.177516E+05, 4.277100E+05, 
   4.378587E+05, 4.482007E+05, 4.587385E+05, 4.694751E+05, 4.804133E+05, 4.915558E+05, 
   5.029056E+05, 5.144657E+05, 5.262388E+05, 5.382281E+05, 5.504363E+05, 5.628668E+05, 
   5.755223E+05, 5.884060E+05, 6.015211E+05, 6.148706E+05, 6.284576E+05, 6.422855E+05, 
   6.563574E+05, 6.706765E+05, 6.852461E+05, 7.000695E+05, 7.151501E+05, 7.304912E+05, 
   7.460963E+05, 7.619686E+05, 7.781117E+05, 7.945291E+05, 8.112243E+05, 8.282008E+05, 
   8.454621E+05, 8.630119E+05, 8.808539E+05, 8.989917E+05, 9.174289E+05, 9.361694E+05, 
   9.552168E+05, 9.745749E+05, 9.942477E+05, 1.014239E+06, 1.034552E+06, 1.055192E+06, 
   1.076162E+06, 1.097466E+06, 1.119108E+06, 1.141093E+06, 1.163424E+06, 1.186105E+06, 
   1.209140E+06, 1.232535E+06, 1.256292E+06, 1.280417E+06, 1.304913E+06, 1.329785E+06, 
   1.355037E+06, 1.380673E+06, 1.406698E+06, 1.433117E+06, 1.459934E+06, 1.487153E+06, 
   1.514779E+06, 1.542816E+06, 1.571269E+06, 1.600144E+06, 1.629443E+06, 1.659173E+06, 
   1.689338E+06, 1.719942E+06, 1.750992E+06, 1.782490E+06, 1.814443E+06, 1.846856E+06, 
   1.879732E+06, 1.913078E+06, 1.946898E+06, 1.981197E+06, 2.015982E+06, 2.051255E+06, 
   2.087024E+06, 2.123293E+06, 2.160067E+06, 2.197352E+06, 2.235153E+06, 2.273476E+06, 
   2.312325E+06, 2.351707E+06, 2.391626E+06, 2.432089E+06, 2.473101E+06, 2.514668E+06, 
   2.556794E+06, 2.599487E+06, 2.642751E+06, 2.686593E+06, 2.731018E+06, 2.776032E+06, 
   2.821641E+06, 2.867851E+06, 2.914668E+06, 2.962098E+06, 3.010147E+06, 3.058820E+06, 
   3.108126E+06, 3.158068E+06, 3.208654E+06, 3.259889E+06, 3.311781E+06, 3.364335E+06, 
   3.417558E+06, 3.471457E+06, 3.526037E+06, 3.581305E+06, 3.637268E+06, 3.693933E+06, 
   3.751305E+06, 3.809392E+06, 3.868200E+06, 3.927737E+06, 3.988008E+06, 4.049022E+06, 
   4.110784E+06, 4.173301E+06, 4.236581E+06, 4.300631E+06, 4.365457E+06, 4.431068E+06, 
   4.497468E+06, 4.564668E+06, 4.632672E+06, 4.701489E+06, 4.771126E+06, 4.841590E+06, 
   4.912890E+06, 4.985031E+06, 5.058022E+06, 5.131870E+06, 5.206583E+06, 5.282169E+06, 
   5.358634E+06, 5.435988E+06, 5.514238E+06, 5.593390E+06, 5.673455E+06, 5.754439E+06, 
   5.836350E+06, 5.919196E+06, 6.002986E+06, 6.087727E+06, 6.173428E+06, 6.260096E+06, 
   6.347741E+06, 6.436370E+06, 6.525992E+06, 6.616615E+06, 6.708247E+06, 6.800898E+06, 
   6.894575E+06, 6.989288E+06, 7.085044E+06, 7.181852E+06, 7.279722E+06, 7.378662E+06, 
   7.478681E+06, 7.579788E+06, 7.681991E+06, 7.785300E+06, 7.889724E+06, 7.995271E+06, 
   8.101951E+06, 8.209774E+06, 8.318747E+06, 8.428882E+06, 8.540186E+06, 8.652669E+06, 
   8.766342E+06, 8.881212E+06, 8.997290E+06, 9.114586E+06, 9.233109E+06, 9.352868E+06, 
   9.473874E+06, 9.596136E+06, 9.719664E+06, 9.844468E+06, 9.970558E+06, 1.009794E+07, 
   1.022664E+07, 1.035664E+07, 1.048798E+07, 1.062065E+07, 1.075467E+07, 1.089004E+07, 
   1.102679E+07, 1.116491E+07, 1.130442E+07, 1.144532E+07, 1.158764E+07, 1.173138E+07, 
   1.187655E+07, 1.202316E+07, 1.217122E+07, 1.232075E+07, 1.247175E+07, 1.262423E+07, 
   1.277822E+07, 1.293370E+07, 1.309071E+07, 1.324925E+07, 1.340933E+07, 1.357096E+07, 
   1.373416E+07, 1.389893E+07, 1.406528E+07, 1.423324E+07, 1.440281E+07, 1.457399E+07, 
   1.474681E+07, 1.492128E+07, 1.509740E+07, 1.527519E+07, 1.545466E+07, 1.563582E+07, 
   1.581869E+07, 1.600328E+07, 1.618959E+07, 1.637765E+07, 1.656746E+07, 1.675903E+07, 
   1.695239E+07, 1.714753E+07, 1.734448E+07, 1.754324E+07, 1.774384E+07, 1.794627E+07, 
   1.815056E+07, 1.835671E+07, 1.856475E+07, 1.877468E+07, 1.898651E+07, 1.920026E+07, 
   1.941595E+07, 1.963358E+07, 1.985317E+07, 2.007473E+07, 2.029827E+07, 2.052382E+07, 
   2.075137E+07, 2.098096E+07, 2.121258E+07, 2.144625E+07, 2.168200E+07, 2.191982E+07, 
   2.215974E+07, 2.240176E+07, 2.264591E+07, 2.289220E+07, 2.314063E+07, 2.339124E+07, 
   2.364402E+07, 2.389899E+07, 2.415617E+07, 2.441558E+07, 2.467722E+07, 2.494111E+07, 
   2.520727E+07, 2.547572E+07, 2.574645E+07, 2.601950E+07, 2.629487E+07, 2.657259E+07, 
   2.685266E+07, 2.713510E+07, 2.741992E+07, 2.770715E+07, 2.799679E+07, 2.828887E+07, 
   2.858339E+07, 2.888038E+07, 2.917984E+07, 2.948180E+07, 2.978626E+07, 3.009326E+07, 
   3.040279E+07, 3.071489E+07, 3.102955E+07, 3.134681E+07, 3.166667E+07, 3.198915E+07, 
   3.231427E+07, 3.264204E+07, 3.297249E+07, 3.330562E+07, 3.364146E+07, 3.398001E+07, 
   3.432131E+07, 3.466535E+07, 3.501217E+07, 3.536178E+07, 3.571418E+07, 3.606942E+07, 
   3.642748E+07, 3.678841E+07, 3.715221E+07, 3.751890E+07, 3.788849E+07, 3.826101E+07, 
   3.863648E+07, 3.901490E+07, 3.939630E+07, 3.978070E+07, 4.016811E+07, 4.055855E+07, 
   4.095204E+07, 4.134860E+07, 4.174825E+07, 4.215100E+07, 4.255686E+07, 4.296587E+07, 
   4.337804E+07, 4.379339E+07, 4.421193E+07, 4.463368E+07, 4.505867E+07, 4.548691E+07, 
   4.591842E+07, 4.635322E+07, 4.679133E+07, 4.723277E+07, 4.767755E+07, 4.812570E+07, 
   4.857723E+07, 4.903217E+07, 4.949053E+07, 
])
# ============================ M = 53, I = 3 ============================
M = 53
I = 3
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   2.724569E+01, 2.600625E+02, 5.188440E+02, 7.776856E+02, 1.036588E+03, 1.295575E+03, 
   1.554801E+03, 1.814713E+03, 2.076173E+03, 2.340434E+03, 2.609037E+03, 2.883683E+03, 
   3.166126E+03, 3.458104E+03, 3.761298E+03, 4.077310E+03, 4.407669E+03, 4.753826E+03, 
   5.117172E+03, 5.499043E+03, 5.900734E+03, 6.323506E+03, 6.768600E+03, 7.237241E+03, 
   7.730649E+03, 8.250043E+03, 8.796647E+03, 9.371695E+03, 9.976435E+03, 1.061213E+04, 
   1.128007E+04, 1.198157E+04, 1.271795E+04, 1.349058E+04, 1.430086E+04, 1.515020E+04, 
   1.604006E+04, 1.697194E+04, 1.794735E+04, 1.896785E+04, 2.003504E+04, 2.115056E+04, 
   2.231607E+04, 2.353328E+04, 2.480395E+04, 2.612987E+04, 2.751286E+04, 2.895481E+04, 
   3.045762E+04, 3.202325E+04, 3.365371E+04, 3.535104E+04, 3.711733E+04, 3.895470E+04, 
   4.086535E+04, 4.285149E+04, 4.491540E+04, 4.705939E+04, 4.928583E+04, 5.159713E+04, 
   5.399576E+04, 5.648421E+04, 5.906505E+04, 6.174089E+04, 6.451439E+04, 6.738825E+04, 
   7.036524E+04, 7.344816E+04, 7.663989E+04, 7.994333E+04, 8.336146E+04, 8.689731E+04, 
   9.055394E+04, 9.433449E+04, 9.824214E+04, 1.022801E+05, 1.064518E+05, 1.107604E+05, 
   1.152095E+05, 1.198025E+05, 1.245428E+05, 1.294342E+05, 1.344802E+05, 1.396845E+05, 
   1.450510E+05, 1.505834E+05, 1.562856E+05, 1.621615E+05, 1.682153E+05, 1.744509E+05, 
   1.808724E+05, 1.874842E+05, 1.942904E+05, 2.012953E+05, 2.085035E+05, 2.159192E+05, 
   2.235471E+05, 2.313917E+05, 2.394576E+05, 2.477497E+05, 2.562727E+05, 2.650315E+05, 
   2.740309E+05, 2.832760E+05, 2.927719E+05, 3.025236E+05, 3.125365E+05, 3.228157E+05, 
   3.333667E+05, 3.441948E+05, 3.553056E+05, 3.667047E+05, 3.783976E+05, 3.903901E+05, 
   4.026880E+05, 4.152972E+05, 4.282236E+05, 4.414732E+05, 4.550522E+05, 4.689667E+05, 
   4.832230E+05, 4.978273E+05, 5.127862E+05, 5.281062E+05, 5.437937E+05, 5.598555E+05, 
   5.762983E+05, 5.931288E+05, 6.103541E+05, 6.279811E+05, 6.460168E+05, 6.644684E+05, 
   6.833432E+05, 7.026484E+05, 7.223915E+05, 7.425800E+05, 7.632214E+05, 7.843233E+05, 
   8.058936E+05, 8.279401E+05, 8.504707E+05, 8.734935E+05, 8.970164E+05, 9.210478E+05, 
   9.455959E+05, 9.706691E+05, 9.962758E+05, 1.022425E+06, 1.049124E+06, 1.076383E+06, 
   1.104211E+06, 1.132616E+06, 1.161607E+06, 1.191193E+06, 1.221384E+06, 1.252190E+06, 
   1.283618E+06, 1.315680E+06, 1.348383E+06, 1.381740E+06, 1.415758E+06, 1.450448E+06, 
   1.485820E+06, 1.521884E+06, 1.558650E+06, 1.596128E+06, 1.634330E+06, 1.673265E+06, 
   1.712944E+06, 1.753378E+06, 1.794578E+06, 1.836554E+06, 1.879317E+06, 1.922880E+06, 
   1.967252E+06, 2.012445E+06, 2.058471E+06, 2.105342E+06, 2.153068E+06, 2.201662E+06, 
   2.251136E+06, 2.301502E+06, 2.352771E+06, 2.404956E+06, 2.458070E+06, 2.512124E+06, 
   2.567132E+06, 2.623105E+06, 2.680057E+06, 2.738001E+06, 2.796950E+06, 2.856916E+06, 
   2.917914E+06, 2.979956E+06, 3.043055E+06, 3.107226E+06, 3.172483E+06, 3.238838E+06, 
   3.306306E+06, 3.374902E+06, 3.444638E+06, 3.515531E+06, 3.587593E+06, 3.660839E+06, 
   3.735285E+06, 3.810945E+06, 3.887834E+06, 3.965966E+06, 4.045358E+06, 4.126024E+06, 
   4.207980E+06, 4.291241E+06, 4.375824E+06, 4.461743E+06, 4.549015E+06, 4.637656E+06, 
   4.727682E+06, 4.819109E+06, 4.911954E+06, 5.006233E+06, 5.101963E+06, 5.199161E+06, 
   5.297844E+06, 5.398029E+06, 5.499733E+06, 5.602973E+06, 5.707768E+06, 5.814134E+06, 
   5.922089E+06, 6.031652E+06, 6.142840E+06, 6.255672E+06, 6.370166E+06, 6.486339E+06, 
   6.604212E+06, 6.723802E+06, 6.845129E+06, 6.968212E+06, 7.093069E+06, 7.219720E+06, 
   7.348184E+06, 7.478481E+06, 7.610631E+06, 7.744654E+06, 7.880569E+06, 8.018396E+06, 
   8.158157E+06, 8.299871E+06, 8.443559E+06, 8.589242E+06, 8.736940E+06, 8.886675E+06, 
   9.038468E+06, 9.192340E+06, 9.348312E+06, 9.506407E+06, 9.666646E+06, 9.829050E+06, 
   9.993643E+06, 1.016045E+07, 1.032948E+07, 1.050077E+07, 1.067434E+07, 1.085021E+07, 
   1.102840E+07, 1.120894E+07, 1.139185E+07, 1.157716E+07, 1.176488E+07, 1.195504E+07, 
   1.214767E+07, 1.234278E+07, 1.254041E+07, 1.274058E+07, 1.294331E+07, 1.314862E+07, 
   1.335655E+07, 1.356712E+07, 1.378035E+07, 1.399626E+07, 1.421489E+07, 1.443626E+07, 
   1.466039E+07, 1.488731E+07, 1.511705E+07, 1.534964E+07, 1.558509E+07, 1.582344E+07, 
   1.606472E+07, 1.630894E+07, 1.655614E+07, 1.680635E+07, 1.705959E+07, 1.731589E+07, 
   1.757528E+07, 1.783779E+07, 1.810344E+07, 1.837227E+07, 1.864429E+07, 1.891955E+07, 
   1.919807E+07, 1.947987E+07, 1.976500E+07, 2.005347E+07, 2.034532E+07, 2.064057E+07, 
   2.093927E+07, 2.124143E+07, 2.154708E+07, 2.185627E+07, 2.216901E+07, 2.248535E+07, 
   2.280530E+07, 2.312891E+07, 2.345620E+07, 2.378721E+07, 2.412196E+07, 2.446050E+07, 
   2.480284E+07, 2.514903E+07, 2.549909E+07, 2.585307E+07, 2.621098E+07, 2.657287E+07, 
   2.693877E+07, 2.730871E+07, 2.768273E+07, 2.806086E+07, 2.844313E+07, 2.882958E+07, 
   2.922024E+07, 2.961515E+07, 3.001434E+07, 3.041785E+07, 3.082571E+07, 3.123796E+07, 
   3.165463E+07, 3.207576E+07, 3.250138E+07, 3.293154E+07, 3.336627E+07, 3.380560E+07, 
   3.424957E+07, 3.469822E+07, 3.515159E+07, 3.560971E+07, 3.607262E+07, 3.654036E+07, 
   3.701296E+07, 3.749047E+07, 3.797292E+07, 3.846036E+07, 3.895282E+07, 3.945033E+07, 
   3.995295E+07, 4.046070E+07, 4.097364E+07, 4.149179E+07, 4.201520E+07, 4.254391E+07, 
   4.307796E+07, 4.361738E+07, 4.416223E+07, 4.471254E+07, 4.526836E+07, 4.582972E+07, 
   4.639666E+07, 4.696924E+07, 4.754748E+07, 4.813144E+07, 4.872116E+07, 4.931667E+07, 
   4.991803E+07, 5.052527E+07, 5.113844E+07, 5.175758E+07, 5.238274E+07, 5.301396E+07, 
   5.365129E+07, 5.429476E+07, 5.494443E+07, 5.560034E+07, 5.626254E+07, 5.693106E+07, 
   5.760597E+07, 5.828729E+07, 5.897508E+07, 5.966939E+07, 6.037026E+07, 6.107774E+07, 
   6.179187E+07, 6.251270E+07, 6.324029E+07, 6.397467E+07, 6.471590E+07, 6.546403E+07, 
   6.621909E+07, 6.698115E+07, 6.775025E+07, 6.852644E+07, 6.930977E+07, 7.010028E+07, 
   7.089804E+07, 7.170308E+07, 7.251546E+07, 7.333524E+07, 7.416245E+07, 7.499715E+07, 
   7.583940E+07, 7.668924E+07, 7.754673E+07, 7.841192E+07, 7.928485E+07, 8.016559E+07, 
   8.105418E+07, 8.195069E+07, 8.285515E+07, 8.376763E+07, 8.468818E+07, 8.561685E+07, 
   8.655370E+07, 8.749878E+07, 8.845215E+07, 8.941386E+07, 9.038396E+07, 9.136252E+07, 
   9.234958E+07, 9.334521E+07, 9.434946E+07, 9.536238E+07, 9.638404E+07, 9.741449E+07, 
   9.845378E+07, 9.950198E+07, 1.005591E+08, 1.016253E+08, 1.027006E+08, 1.037850E+08, 
   1.048786E+08, 1.059814E+08, 1.070936E+08, 1.082151E+08, 1.093461E+08, 1.104865E+08, 
   1.116366E+08, 1.127962E+08, 1.139655E+08, 1.151445E+08, 1.163334E+08, 1.175321E+08, 
   1.187407E+08, 1.199593E+08, 1.211879E+08, 1.224267E+08, 1.236756E+08, 1.249348E+08, 
   1.262042E+08, 1.274840E+08, 1.287743E+08, 1.300750E+08, 1.313863E+08, 1.327082E+08, 
   1.340408E+08, 1.353841E+08, 1.367383E+08, 1.381033E+08, 1.394793E+08, 1.408663E+08, 
   1.422643E+08, 1.436736E+08, 1.450940E+08, 1.465257E+08, 1.479688E+08, 1.494233E+08, 
   1.508892E+08, 1.523668E+08, 1.538559E+08, 1.553568E+08, 1.568694E+08, 1.583938E+08, 
   1.599302E+08, 1.614785E+08, 1.630388E+08, 1.646113E+08, 1.661960E+08, 1.677929E+08, 
   1.694022E+08, 1.710238E+08, 1.726579E+08, 1.743046E+08, 1.759639E+08, 1.776359E+08, 
   1.793206E+08, 1.810182E+08, 1.827287E+08, 1.844522E+08, 1.861887E+08, 1.879384E+08, 
   1.897013E+08, 1.914775E+08, 1.932671E+08, 
])
# ============================ M = 53, I = 4 ============================
M = 53
I = 4
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   6.713027E+00, 6.402990E+01, 1.277394E+02, 1.914638E+02, 2.552037E+02, 3.189770E+02, 
   3.828815E+02, 4.471891E+02, 5.123940E+02, 5.791861E+02, 6.483815E+02, 7.208506E+02, 
   7.974636E+02, 8.790581E+02, 9.664246E+02, 1.060303E+03, 1.161388E+03, 1.270333E+03, 
   1.387761E+03, 1.514270E+03, 1.650442E+03, 1.796845E+03, 1.954047E+03, 2.122608E+03, 
   2.303095E+03, 2.496078E+03, 2.702135E+03, 2.921855E+03, 3.155835E+03, 3.404688E+03, 
   3.669041E+03, 3.949533E+03, 4.246824E+03, 4.561585E+03, 4.894510E+03, 5.246306E+03, 
   5.617704E+03, 6.009449E+03, 6.422310E+03, 6.857072E+03, 7.314543E+03, 7.795551E+03, 
   8.300946E+03, 8.831598E+03, 9.388399E+03, 9.972263E+03, 1.058413E+04, 1.122495E+04, 
   1.189571E+04, 1.259742E+04, 1.333111E+04, 1.409782E+04, 1.489863E+04, 1.573465E+04, 
   1.660699E+04, 1.751681E+04, 1.846528E+04, 1.945360E+04, 2.048299E+04, 2.155471E+04, 
   2.267003E+04, 2.383025E+04, 2.503670E+04, 2.629074E+04, 2.759375E+04, 2.894713E+04, 
   3.035233E+04, 3.181081E+04, 3.332406E+04, 3.489361E+04, 3.652100E+04, 3.820780E+04, 
   3.995563E+04, 4.176613E+04, 4.364095E+04, 4.558180E+04, 4.759039E+04, 4.966849E+04, 
   5.181787E+04, 5.404036E+04, 5.633781E+04, 5.871209E+04, 6.116511E+04, 6.369882E+04, 
   6.631519E+04, 6.901623E+04, 7.180397E+04, 7.468049E+04, 7.764790E+04, 8.070834E+04, 
   8.386397E+04, 8.711700E+04, 9.046968E+04, 9.392428E+04, 9.748311E+04, 1.011485E+05, 
   1.049229E+05, 1.088086E+05, 1.128082E+05, 1.169240E+05, 1.211587E+05, 1.255148E+05, 
   1.299949E+05, 1.346017E+05, 1.393377E+05, 1.442058E+05, 1.492086E+05, 1.543489E+05, 
   1.596296E+05, 1.650536E+05, 1.706237E+05, 1.763429E+05, 1.822142E+05, 1.882405E+05, 
   1.944251E+05, 2.007708E+05, 2.072810E+05, 2.139587E+05, 2.208072E+05, 2.278299E+05, 
   2.350299E+05, 2.424106E+05, 2.499755E+05, 2.577279E+05, 2.656713E+05, 2.738094E+05, 
   2.821456E+05, 2.906835E+05, 2.994268E+05, 3.083792E+05, 3.175444E+05, 3.269263E+05, 
   3.365286E+05, 3.463552E+05, 3.564102E+05, 3.666973E+05, 3.772207E+05, 3.879843E+05, 
   3.989925E+05, 4.102491E+05, 4.217586E+05, 4.335251E+05, 4.455529E+05, 4.578463E+05, 
   4.704099E+05, 4.832479E+05, 4.963650E+05, 5.097656E+05, 5.234544E+05, 5.374360E+05, 
   5.517150E+05, 5.662962E+05, 5.811845E+05, 5.963847E+05, 6.119015E+05, 6.277401E+05, 
   6.439054E+05, 6.604025E+05, 6.772364E+05, 6.944124E+05, 7.119355E+05, 7.298112E+05, 
   7.480446E+05, 7.666413E+05, 7.856065E+05, 8.049458E+05, 8.246648E+05, 8.447689E+05, 
   8.652639E+05, 8.861554E+05, 9.074493E+05, 9.291512E+05, 9.512672E+05, 9.738030E+05, 
   9.967647E+05, 1.020158E+06, 1.043990E+06, 1.068266E+06, 1.092992E+06, 1.118175E+06, 
   1.143821E+06, 1.169936E+06, 1.196527E+06, 1.223600E+06, 1.251162E+06, 1.279220E+06, 
   1.307780E+06, 1.336849E+06, 1.366434E+06, 1.396541E+06, 1.427178E+06, 1.458351E+06, 
   1.490068E+06, 1.522336E+06, 1.555161E+06, 1.588552E+06, 1.622515E+06, 1.657058E+06, 
   1.692188E+06, 1.727913E+06, 1.764240E+06, 1.801178E+06, 1.838732E+06, 1.876913E+06, 
   1.915726E+06, 1.955180E+06, 1.995284E+06, 2.036044E+06, 2.077470E+06, 2.119569E+06, 
   2.162349E+06, 2.205819E+06, 2.249988E+06, 2.294863E+06, 2.340453E+06, 2.386766E+06, 
   2.433812E+06, 2.481599E+06, 2.530135E+06, 2.579430E+06, 2.629493E+06, 2.680332E+06, 
   2.731956E+06, 2.784375E+06, 2.837598E+06, 2.891634E+06, 2.946492E+06, 3.002182E+06, 
   3.058713E+06, 3.116095E+06, 3.174338E+06, 3.233450E+06, 3.293443E+06, 3.354325E+06, 
   3.416107E+06, 3.478798E+06, 3.542410E+06, 3.606951E+06, 3.672432E+06, 3.738863E+06, 
   3.806255E+06, 3.874619E+06, 3.943964E+06, 4.014301E+06, 4.085641E+06, 4.157995E+06, 
   4.231373E+06, 4.305787E+06, 4.381247E+06, 4.457764E+06, 4.535351E+06, 4.614016E+06, 
   4.693774E+06, 4.774633E+06, 4.856606E+06, 4.939705E+06, 5.023941E+06, 5.109325E+06, 
   5.195870E+06, 5.283588E+06, 5.372490E+06, 5.462587E+06, 5.553894E+06, 5.646421E+06, 
   5.740181E+06, 5.835186E+06, 5.931448E+06, 6.028981E+06, 6.127796E+06, 6.227907E+06, 
   6.329326E+06, 6.432066E+06, 6.536140E+06, 6.641561E+06, 6.748341E+06, 6.856495E+06, 
   6.966036E+06, 7.076976E+06, 7.189329E+06, 7.303110E+06, 7.418330E+06, 7.535005E+06, 
   7.653147E+06, 7.772771E+06, 7.893891E+06, 8.016521E+06, 8.140674E+06, 8.266366E+06, 
   8.393610E+06, 8.522420E+06, 8.652812E+06, 8.784800E+06, 8.918398E+06, 9.053621E+06, 
   9.190484E+06, 9.329003E+06, 9.469192E+06, 9.611065E+06, 9.754639E+06, 9.899929E+06, 
   1.004695E+07, 1.019572E+07, 1.034624E+07, 1.049855E+07, 1.065265E+07, 1.080856E+07, 
   1.096630E+07, 1.112587E+07, 1.128731E+07, 1.145061E+07, 1.161581E+07, 1.178291E+07, 
   1.195194E+07, 1.212291E+07, 1.229583E+07, 1.247072E+07, 1.264761E+07, 1.282650E+07, 
   1.300741E+07, 1.319037E+07, 1.337538E+07, 1.356247E+07, 1.375165E+07, 1.394295E+07, 
   1.413637E+07, 1.433193E+07, 1.452966E+07, 1.472958E+07, 1.493169E+07, 1.513602E+07, 
   1.534258E+07, 1.555140E+07, 1.576250E+07, 1.597588E+07, 1.619158E+07, 1.640960E+07, 
   1.662997E+07, 1.685270E+07, 1.707783E+07, 1.730535E+07, 1.753530E+07, 1.776770E+07, 
   1.800255E+07, 1.823989E+07, 1.847973E+07, 1.872209E+07, 1.896699E+07, 1.921445E+07, 
   1.946449E+07, 1.971714E+07, 1.997240E+07, 2.023031E+07, 2.049088E+07, 2.075413E+07, 
   2.102008E+07, 2.128876E+07, 2.156018E+07, 2.183437E+07, 2.211135E+07, 2.239113E+07, 
   2.267374E+07, 2.295920E+07, 2.324754E+07, 2.353877E+07, 2.383291E+07, 2.412999E+07, 
   2.443003E+07, 2.473305E+07, 2.503907E+07, 2.534812E+07, 2.566022E+07, 2.597538E+07, 
   2.629364E+07, 2.661501E+07, 2.693952E+07, 2.726718E+07, 2.759804E+07, 2.793210E+07, 
   2.826938E+07, 2.860992E+07, 2.895374E+07, 2.930086E+07, 2.965129E+07, 3.000508E+07, 
   3.036224E+07, 3.072278E+07, 3.108675E+07, 3.145416E+07, 3.182504E+07, 3.219941E+07, 
   3.257729E+07, 3.295872E+07, 3.334370E+07, 3.373228E+07, 3.412447E+07, 3.452030E+07, 
   3.491980E+07, 3.532298E+07, 3.572988E+07, 3.614051E+07, 3.655492E+07, 3.697311E+07, 
   3.739512E+07, 3.782098E+07, 3.825070E+07, 3.868432E+07, 3.912185E+07, 3.956334E+07, 
   4.000880E+07, 4.045825E+07, 4.091174E+07, 4.136928E+07, 4.183090E+07, 4.229662E+07, 
   4.276648E+07, 4.324050E+07, 4.371871E+07, 4.420113E+07, 4.468780E+07, 4.517874E+07, 
   4.567398E+07, 4.617354E+07, 4.667746E+07, 4.718576E+07, 4.769848E+07, 4.821563E+07, 
   4.873725E+07, 4.926336E+07, 4.979400E+07, 5.032919E+07, 5.086896E+07, 5.141335E+07, 
   5.196238E+07, 5.251607E+07, 5.307446E+07, 5.363758E+07, 5.420546E+07, 5.477812E+07, 
   5.535560E+07, 5.593793E+07, 5.652514E+07, 5.711725E+07, 5.771430E+07, 5.831631E+07, 
   5.892332E+07, 5.953536E+07, 6.015246E+07, 6.077465E+07, 6.140196E+07, 6.203442E+07, 
   6.267206E+07, 6.331492E+07, 6.396302E+07, 6.461640E+07, 6.527508E+07, 6.593911E+07, 
   6.660850E+07, 6.728330E+07, 6.796354E+07, 6.864924E+07, 6.934044E+07, 7.003718E+07, 
   7.073947E+07, 7.144737E+07, 7.216089E+07, 7.288008E+07, 7.360496E+07, 7.433558E+07, 
   7.507195E+07, 7.581412E+07, 7.656211E+07, 7.731597E+07, 7.807572E+07, 7.884140E+07, 
   7.961305E+07, 8.039069E+07, 8.117436E+07, 8.196410E+07, 8.275994E+07, 8.356191E+07, 
   8.437005E+07, 8.518439E+07, 8.600497E+07, 8.683182E+07, 8.766498E+07, 8.850449E+07, 
   8.935037E+07, 9.020266E+07, 9.106141E+07, 9.192664E+07, 9.279838E+07, 9.367669E+07, 
   9.456158E+07, 9.545310E+07, 9.635129E+07, 9.725618E+07, 9.816780E+07, 9.908620E+07, 
   1.000114E+08, 1.009435E+08, 1.018824E+08, 
])
# ============================ M = 54, I = 1 ============================
M = 54
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.564399E+01, 1.031202E+03, 2.666439E+03, 4.792731E+03, 7.338129E+03, 1.023914E+04, 
   1.345214E+04, 1.694741E+04, 2.070371E+04, 2.470550E+04, 2.894160E+04, 3.340460E+04, 
   3.809039E+04, 4.299775E+04, 4.812807E+04, 5.348497E+04, 5.907413E+04, 6.490301E+04, 
   7.098070E+04, 7.731780E+04, 8.392626E+04, 9.081932E+04, 9.801144E+04, 1.055183E+05, 
   1.133565E+05, 1.215441E+05, 1.300998E+05, 1.390438E+05, 1.483969E+05, 1.581814E+05, 
   1.684204E+05, 1.791381E+05, 1.903600E+05, 2.021124E+05, 2.144229E+05, 2.273204E+05, 
   2.408347E+05, 2.549972E+05, 2.698402E+05, 2.853977E+05, 3.017048E+05, 3.187980E+05, 
   3.367155E+05, 3.554967E+05, 3.751828E+05, 3.958167E+05, 4.174426E+05, 4.401069E+05, 
   4.638576E+05, 4.887447E+05, 5.148199E+05, 5.421372E+05, 5.707527E+05, 6.007245E+05, 
   6.321132E+05, 6.649816E+05, 6.993951E+05, 7.354214E+05, 7.731309E+05, 8.125970E+05, 
   8.538955E+05, 8.971053E+05, 9.423084E+05, 9.895898E+05, 1.039038E+06, 1.090744E+06, 
   1.144804E+06, 1.201316E+06, 1.260382E+06, 1.322110E+06, 1.386608E+06, 1.453991E+06, 
   1.524379E+06, 1.597893E+06, 1.674662E+06, 1.754818E+06, 1.838496E+06, 1.925841E+06, 
   2.016999E+06, 2.112121E+06, 2.211367E+06, 2.314900E+06, 2.422890E+06, 2.535511E+06, 
   2.652946E+06, 2.775384E+06, 2.903018E+06, 3.036051E+06, 3.174692E+06, 3.319156E+06, 
   3.469668E+06, 3.626458E+06, 3.789765E+06, 3.959838E+06, 4.136932E+06, 4.321312E+06, 
   4.513252E+06, 4.713034E+06, 4.920951E+06, 5.137305E+06, 5.362408E+06, 5.596584E+06, 
   5.840166E+06, 6.093498E+06, 6.356936E+06, 6.630848E+06, 6.915613E+06, 7.211623E+06, 
   7.519283E+06, 7.839010E+06, 8.171236E+06, 8.516406E+06, 8.874978E+06, 9.247427E+06, 
   9.634242E+06, 1.003593E+07, 1.045300E+07, 1.088600E+07, 1.133548E+07, 1.180201E+07, 
   1.228618E+07, 1.278860E+07, 1.330988E+07, 1.385068E+07, 1.441166E+07, 1.499350E+07, 
   1.559690E+07, 1.622261E+07, 1.687135E+07, 1.754391E+07, 1.824107E+07, 1.896366E+07, 
   1.971252E+07, 2.048851E+07, 2.129253E+07, 2.212549E+07, 2.298834E+07, 2.388205E+07, 
   2.480763E+07, 2.576610E+07, 2.675853E+07, 2.778601E+07, 2.884966E+07, 2.995062E+07, 
   3.109010E+07, 3.226932E+07, 3.348952E+07, 3.475199E+07, 3.605808E+07, 3.740913E+07, 
   3.880656E+07, 4.025180E+07, 4.174634E+07, 4.329170E+07, 4.488944E+07, 4.654118E+07, 
   4.824856E+07, 5.001328E+07, 5.183708E+07, 5.372176E+07, 5.566915E+07, 5.768114E+07, 
   5.975967E+07, 6.190674E+07, 6.412437E+07, 6.641468E+07, 6.877982E+07, 7.122200E+07, 
   7.374348E+07, 7.634659E+07, 7.903373E+07, 8.180734E+07, 8.466995E+07, 8.762413E+07, 
   9.067253E+07, 9.381786E+07, 9.706291E+07, 1.004105E+08, 1.038637E+08, 1.074253E+08, 
   1.110986E+08, 1.148865E+08, 1.187925E+08, 1.228198E+08, 1.269717E+08, 1.312519E+08, 
   1.356638E+08, 1.402111E+08, 1.448977E+08, 1.497272E+08, 1.547037E+08, 1.598312E+08, 
   1.651139E+08, 1.705559E+08, 1.761616E+08, 1.819354E+08, 1.878820E+08, 1.940059E+08, 
   2.003120E+08, 2.068050E+08, 2.134901E+08, 2.203724E+08, 2.274570E+08, 2.347493E+08, 
   2.422549E+08, 2.499794E+08, 2.579284E+08, 2.661080E+08, 2.745241E+08, 2.831830E+08, 
   2.920908E+08, 3.012540E+08, 3.106794E+08, 3.203735E+08, 3.303434E+08, 3.405961E+08, 
   3.511388E+08, 3.619788E+08, 3.731239E+08, 3.845817E+08, 3.963600E+08, 4.084671E+08, 
   4.209111E+08, 4.337004E+08, 4.468438E+08, 4.603501E+08, 4.742282E+08, 4.884874E+08, 
   5.031371E+08, 5.181870E+08, 5.336468E+08, 5.495267E+08, 5.658369E+08, 5.825879E+08, 
   5.997905E+08, 6.174555E+08, 6.355943E+08, 6.542181E+08, 6.733388E+08, 6.929681E+08, 
   7.131184E+08, 7.338020E+08, 7.550316E+08, 7.768203E+08, 7.991813E+08, 8.221280E+08, 
   8.456743E+08, 8.698344E+08, 8.946224E+08, 9.200533E+08, 9.461420E+08, 9.729037E+08, 
   1.000354E+09, 1.028509E+09, 1.057385E+09, 1.086998E+09, 1.117366E+09, 1.148506E+09, 
   1.180435E+09, 1.213171E+09, 1.246733E+09, 1.281140E+09, 1.316410E+09, 1.352563E+09, 
   1.389619E+09, 1.427598E+09, 1.466521E+09, 1.506410E+09, 1.547284E+09, 1.589167E+09, 
   1.632081E+09, 1.676048E+09, 1.721092E+09, 1.767237E+09, 1.814506E+09, 1.862925E+09, 
   1.912518E+09, 1.963312E+09, 2.015331E+09, 2.068603E+09, 2.123156E+09, 2.179015E+09, 
   2.236211E+09, 2.294770E+09, 2.354724E+09, 2.416100E+09, 2.478931E+09, 2.543246E+09, 
   2.609078E+09, 2.676458E+09, 2.745419E+09, 2.815995E+09, 2.888219E+09, 2.962126E+09, 
   3.037751E+09, 3.115130E+09, 3.194300E+09, 3.275298E+09, 3.358162E+09, 3.442930E+09, 
   3.529642E+09, 3.618337E+09, 3.709057E+09, 3.801843E+09, 3.896738E+09, 3.993783E+09, 
   4.093024E+09, 4.194504E+09, 4.298270E+09, 4.404367E+09, 4.512842E+09, 4.623744E+09, 
   4.737120E+09, 4.853021E+09, 4.971497E+09, 5.092599E+09, 5.216380E+09, 5.342893E+09, 
   5.472192E+09, 5.604332E+09, 5.739370E+09, 5.877361E+09, 6.018365E+09, 6.162440E+09, 
   6.309646E+09, 6.460045E+09, 6.613698E+09, 6.770669E+09, 6.931022E+09, 7.094823E+09, 
   7.262138E+09, 7.433035E+09, 7.607582E+09, 7.785850E+09, 7.967909E+09, 8.153833E+09, 
   8.343695E+09, 8.537569E+09, 8.735532E+09, 8.937661E+09, 9.144036E+09, 9.354734E+09, 
   9.569840E+09, 9.789434E+09, 1.001360E+10, 1.024243E+10, 1.047600E+10, 1.071440E+10, 
   1.095773E+10, 1.120608E+10, 1.145953E+10, 1.171818E+10, 1.198213E+10, 1.225147E+10, 
   1.252631E+10, 1.280674E+10, 1.309288E+10, 1.338480E+10, 1.368264E+10, 1.398649E+10, 
   1.429646E+10, 1.461267E+10, 1.493522E+10, 1.526423E+10, 1.559981E+10, 1.594209E+10, 
   1.629118E+10, 1.664721E+10, 1.701030E+10, 1.738056E+10, 1.775815E+10, 1.814317E+10, 
   1.853577E+10, 1.893607E+10, 1.934421E+10, 1.976034E+10, 2.018458E+10, 2.061709E+10, 
   2.105800E+10, 2.150747E+10, 2.196564E+10, 2.243266E+10, 2.290869E+10, 2.339388E+10, 
   2.388839E+10, 2.439238E+10, 2.490602E+10, 2.542946E+10, 2.596289E+10, 2.650646E+10, 
   2.706036E+10, 2.762475E+10, 2.819982E+10, 2.878574E+10, 2.938271E+10, 2.999090E+10, 
   3.061052E+10, 3.124174E+10, 3.188476E+10, 3.253979E+10, 3.320702E+10, 3.388666E+10, 
   3.457891E+10, 3.528399E+10, 3.600210E+10, 3.673347E+10, 3.747831E+10, 3.823684E+10, 
   3.900930E+10, 3.979591E+10, 4.059690E+10, 4.141251E+10, 4.224298E+10, 4.308855E+10, 
   4.394946E+10, 4.482598E+10, 4.571834E+10, 4.662681E+10, 4.755165E+10, 4.849312E+10, 
   4.945150E+10, 5.042704E+10, 5.142004E+10, 5.243076E+10, 5.345950E+10, 5.450654E+10, 
   5.557217E+10, 5.665669E+10, 5.776040E+10, 5.888360E+10, 6.002660E+10, 6.118972E+10, 
   6.237327E+10, 6.357758E+10, 6.480297E+10, 6.604977E+10, 6.731832E+10, 6.860895E+10, 
   6.992202E+10, 7.125787E+10, 7.261685E+10, 7.399933E+10, 7.540568E+10, 7.683625E+10, 
   7.829142E+10, 7.977158E+10, 8.127710E+10, 8.280839E+10, 8.436582E+10, 8.594981E+10, 
   8.756076E+10, 8.919908E+10, 9.086518E+10, 9.255950E+10, 9.428245E+10, 9.603448E+10, 
   9.781602E+10, 9.962752E+10, 1.014694E+11, 1.033422E+11, 1.052463E+11, 1.071822E+11, 
   1.091504E+11, 1.111513E+11, 1.131855E+11, 1.152535E+11, 1.173556E+11, 1.194926E+11, 
   1.216648E+11, 1.238728E+11, 1.261171E+11, 1.283983E+11, 1.307169E+11, 1.330734E+11, 
   1.354684E+11, 1.379025E+11, 1.403762E+11, 1.428902E+11, 1.454450E+11, 1.480411E+11, 
   1.506793E+11, 1.533601E+11, 1.560841E+11, 1.588519E+11, 1.616643E+11, 1.645217E+11, 
   1.674250E+11, 1.703746E+11, 1.733714E+11, 1.764160E+11, 1.795090E+11, 1.826511E+11, 
   1.858431E+11, 1.890857E+11, 1.923795E+11, 1.957253E+11, 1.991239E+11, 2.025759E+11, 
   2.060822E+11, 2.096435E+11, 2.132606E+11, 
])
# ============================ M = 54, I = 2 ============================
#M = 54
#I = 2
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.165434E+02, 2.171779E+03, 5.617372E+03, 1.009780E+04, 1.546146E+04, 2.157454E+04, 
#   2.834514E+04, 3.571078E+04, 4.362707E+04, 5.206194E+04, 6.099299E+04, 7.040600E+04, 
#   8.029393E+04, 9.065605E+04, 1.014971E+05, 1.128269E+05, 1.246590E+05, 1.370113E+05, 
#   1.499049E+05, 1.633639E+05, 1.774154E+05, 1.920894E+05, 2.074180E+05, 2.234362E+05, 
#   2.401812E+05, 2.576925E+05, 2.760120E+05, 2.951838E+05, 3.152544E+05, 3.362725E+05, 
#   3.582893E+05, 3.813582E+05, 4.055350E+05, 4.308780E+05, 4.574482E+05, 4.853090E+05, 
#   5.145265E+05, 5.451697E+05, 5.773102E+05, 6.110227E+05, 6.463851E+05, 6.834780E+05, 
#   7.223858E+05, 7.631960E+05, 8.059995E+05, 8.508913E+05, 8.979697E+05, 9.473372E+05, 
#   9.991003E+05, 1.053370E+06, 1.110261E+06, 1.169893E+06, 1.232391E+06, 1.297884E+06, 
#   1.366506E+06, 1.438398E+06, 1.513704E+06, 1.592576E+06, 1.675169E+06, 1.761648E+06, 
#   1.852181E+06, 1.946943E+06, 2.046118E+06, 2.149895E+06, 2.258471E+06, 2.372050E+06, 
#   2.490844E+06, 2.615074E+06, 2.744967E+06, 2.880762E+06, 3.022703E+06, 3.171047E+06, 
#   3.326059E+06, 3.488012E+06, 3.657192E+06, 3.833894E+06, 4.018424E+06, 4.211101E+06, 
#   4.412252E+06, 4.622219E+06, 4.841356E+06, 5.070028E+06, 5.308615E+06, 5.557510E+06, 
#   5.817121E+06, 6.087867E+06, 6.370187E+06, 6.664531E+06, 6.971367E+06, 7.291180E+06, 
#   7.624471E+06, 7.971758E+06, 8.333577E+06, 8.710484E+06, 9.103052E+06, 9.511874E+06, 
#   9.937566E+06, 1.038076E+07, 1.084212E+07, 1.132231E+07, 1.182204E+07, 1.234204E+07, 
#   1.288305E+07, 1.344584E+07, 1.403123E+07, 1.464002E+07, 1.527308E+07, 1.593128E+07, 
#   1.661553E+07, 1.732678E+07, 1.806599E+07, 1.883416E+07, 1.963233E+07, 2.046156E+07, 
#   2.132294E+07, 2.221763E+07, 2.314677E+07, 2.411159E+07, 2.511333E+07, 2.615326E+07, 
#   2.723272E+07, 2.835308E+07, 2.951573E+07, 3.072212E+07, 3.197377E+07, 3.327220E+07, 
#   3.461900E+07, 3.601581E+07, 3.746432E+07, 3.896626E+07, 4.052342E+07, 4.213765E+07, 
#   4.381084E+07, 4.554494E+07, 4.734198E+07, 4.920401E+07, 5.113318E+07, 5.313167E+07, 
#   5.520175E+07, 5.734575E+07, 5.956604E+07, 6.186510E+07, 6.424547E+07, 6.670973E+07, 
#   6.926058E+07, 7.190076E+07, 7.463312E+07, 7.746057E+07, 8.038611E+07, 8.341281E+07, 
#   8.654385E+07, 8.978248E+07, 9.313205E+07, 9.659600E+07, 1.001779E+08, 1.038813E+08, 
#   1.077100E+08, 1.116678E+08, 1.157586E+08, 1.199866E+08, 1.243558E+08, 1.288706E+08, 
#   1.335352E+08, 1.383543E+08, 1.433324E+08, 1.484742E+08, 1.537848E+08, 1.592689E+08, 
#   1.649319E+08, 1.707789E+08, 1.768154E+08, 1.830468E+08, 1.894790E+08, 1.961177E+08, 
#   2.029689E+08, 2.100388E+08, 2.173336E+08, 2.248600E+08, 2.326243E+08, 2.406336E+08, 
#   2.488947E+08, 2.574148E+08, 2.662012E+08, 2.752615E+08, 2.846033E+08, 2.942346E+08, 
#   3.041634E+08, 3.143981E+08, 3.249471E+08, 3.358191E+08, 3.470231E+08, 3.585682E+08, 
#   3.704639E+08, 3.827196E+08, 3.953452E+08, 4.083509E+08, 4.217468E+08, 4.355436E+08, 
#   4.497521E+08, 4.643834E+08, 4.794488E+08, 4.949598E+08, 5.109286E+08, 5.273671E+08, 
#   5.442878E+08, 5.617035E+08, 5.796273E+08, 5.980725E+08, 6.170527E+08, 6.365820E+08, 
#   6.566747E+08, 6.773454E+08, 6.986091E+08, 7.204811E+08, 7.429772E+08, 7.661132E+08, 
#   7.899058E+08, 8.143715E+08, 8.395276E+08, 8.653917E+08, 8.919816E+08, 9.193158E+08, 
#   9.474129E+08, 9.762922E+08, 1.005973E+09, 1.036476E+09, 1.067821E+09, 1.100030E+09, 
#   1.133123E+09, 1.167122E+09, 1.202051E+09, 1.237931E+09, 1.274786E+09, 1.312640E+09, 
#   1.351517E+09, 1.391443E+09, 1.432442E+09, 1.474540E+09, 1.517765E+09, 1.562143E+09, 
#   1.607702E+09, 1.654470E+09, 1.702476E+09, 1.751750E+09, 1.802321E+09, 1.854220E+09, 
#   1.907480E+09, 1.962131E+09, 2.018207E+09, 2.075741E+09, 2.134766E+09, 2.195319E+09, 
#   2.257434E+09, 2.321147E+09, 2.386496E+09, 2.453519E+09, 2.522254E+09, 2.592739E+09, 
#   2.665016E+09, 2.739125E+09, 2.815109E+09, 2.893009E+09, 2.972869E+09, 3.054733E+09, 
#   3.138648E+09, 3.224658E+09, 3.312810E+09, 3.403154E+09, 3.495738E+09, 3.590611E+09, 
#   3.687825E+09, 3.787432E+09, 3.889484E+09, 3.994035E+09, 4.101141E+09, 4.210858E+09, 
#   4.323242E+09, 4.438352E+09, 4.556247E+09, 4.676989E+09, 4.800638E+09, 4.927258E+09, 
#   5.056913E+09, 5.189667E+09, 5.325589E+09, 5.464745E+09, 5.607205E+09, 5.753039E+09, 
#   5.902319E+09, 6.055118E+09, 6.211512E+09, 6.371574E+09, 6.535384E+09, 6.703019E+09, 
#   6.874561E+09, 7.050090E+09, 7.229690E+09, 7.413447E+09, 7.601445E+09, 7.793774E+09, 
#   7.990522E+09, 8.191782E+09, 8.397645E+09, 8.608207E+09, 8.823564E+09, 9.043813E+09, 
#   9.269056E+09, 9.499392E+09, 9.734926E+09, 9.975764E+09, 1.022201E+10, 1.047378E+10, 
#   1.073118E+10, 1.099432E+10, 1.126332E+10, 1.153829E+10, 1.181936E+10, 1.210665E+10, 
#   1.240028E+10, 1.270037E+10, 1.300706E+10, 1.332046E+10, 1.364073E+10, 1.396798E+10, 
#   1.430236E+10, 1.464401E+10, 1.499307E+10, 1.534967E+10, 1.571398E+10, 1.608614E+10, 
#   1.646629E+10, 1.685460E+10, 1.725122E+10, 1.765632E+10, 1.807004E+10, 1.849256E+10, 
#   1.892406E+10, 1.936468E+10, 1.981462E+10, 2.027405E+10, 2.074314E+10, 2.122209E+10, 
#   2.171107E+10, 2.221027E+10, 2.271989E+10, 2.324012E+10, 2.377116E+10, 2.431321E+10, 
#   2.486647E+10, 2.543116E+10, 2.600749E+10, 2.659566E+10, 2.719591E+10, 2.780844E+10, 
#   2.843350E+10, 2.907130E+10, 2.972208E+10, 3.038608E+10, 3.106353E+10, 3.175469E+10, 
#   3.245980E+10, 3.317912E+10, 3.391290E+10, 3.466139E+10, 3.542488E+10, 3.620362E+10, 
#   3.699789E+10, 3.780796E+10, 3.863413E+10, 3.947668E+10, 4.033589E+10, 4.121206E+10, 
#   4.210550E+10, 4.301651E+10, 4.394539E+10, 4.489247E+10, 4.585806E+10, 4.684250E+10, 
#   4.784609E+10, 4.886920E+10, 4.991214E+10, 5.097527E+10, 5.205894E+10, 5.316350E+10, 
#   5.428932E+10, 5.543676E+10, 5.660620E+10, 5.779801E+10, 5.901258E+10, 6.025029E+10, 
#   6.151155E+10, 6.279675E+10, 6.410630E+10, 6.544062E+10, 6.680012E+10, 6.818523E+10, 
#   6.959639E+10, 7.103404E+10, 7.249861E+10, 7.399057E+10, 7.551037E+10, 7.705848E+10, 
#   7.863536E+10, 8.024151E+10, 8.187741E+10, 8.354354E+10, 8.524042E+10, 8.696855E+10, 
#   8.872845E+10, 9.052064E+10, 9.234565E+10, 9.420403E+10, 9.609631E+10, 9.802306E+10, 
#   9.998484E+10, 1.019822E+11, 1.040158E+11, 1.060861E+11, 1.081938E+11, 1.103394E+11, 
#   1.125236E+11, 1.147470E+11, 1.170103E+11, 1.193140E+11, 1.216588E+11, 1.240455E+11, 
#   1.264745E+11, 1.289467E+11, 1.314627E+11, 1.340232E+11, 1.366289E+11, 1.392806E+11, 
#   1.419789E+11, 1.447246E+11, 1.475184E+11, 1.503611E+11, 1.532535E+11, 1.561963E+11, 
#   1.591903E+11, 1.622364E+11, 1.653353E+11, 1.684879E+11, 1.716949E+11, 1.749573E+11, 
#   1.782759E+11, 1.816515E+11, 1.850851E+11, 1.885775E+11, 1.921296E+11, 1.957424E+11, 
#   1.994168E+11, 2.031537E+11, 2.069541E+11, 2.108189E+11, 2.147491E+11, 2.187458E+11, 
#   2.228098E+11, 2.269423E+11, 2.311443E+11, 2.354168E+11, 2.397608E+11, 2.441776E+11, 
#   2.486681E+11, 2.532334E+11, 2.578747E+11, 2.625931E+11, 2.673897E+11, 2.722658E+11, 
#   2.772224E+11, 2.822609E+11, 2.873823E+11, 2.925880E+11, 2.978791E+11, 3.032570E+11, 
#   3.087228E+11, 3.142779E+11, 3.199236E+11, 3.256613E+11, 3.314922E+11, 3.374177E+11, 
#   3.434392E+11, 3.495581E+11, 3.557758E+11, 3.620938E+11, 3.685134E+11, 3.750362E+11, 
#   3.816636E+11, 3.883971E+11, 3.952384E+11, 4.021888E+11, 4.092500E+11, 4.164235E+11, 
#   4.237110E+11, 4.311141E+11, 4.386344E+11, 4.462735E+11, 4.540333E+11, 4.619153E+11, 
#   4.699213E+11, 4.780531E+11, 4.863124E+11, 
#])
# ============================ M = 55, I = 1 ============================
M = 55
I = 1
TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
TIPS_2021_ISOQ_HASH[(M,I)] = float64([
   5.845627E+01, 1.667025E+03, 4.693504E+03, 8.609801E+03, 1.324637E+04, 1.850516E+04, 
   2.432025E+04, 3.064452E+04, 3.744479E+04, 4.470036E+04, 5.240261E+04, 6.055436E+04, 
   6.916896E+04, 7.826912E+04, 8.788573E+04, 9.805669E+04, 1.088261E+05, 1.202434E+05, 
   1.323631E+05, 1.452442E+05, 1.589504E+05, 1.735492E+05, 1.891130E+05, 2.057181E+05, 
   2.234454E+05, 2.423806E+05, 2.626138E+05, 2.842404E+05, 3.073608E+05, 3.320810E+05, 
   3.585126E+05, 3.867731E+05, 4.169863E+05, 4.492827E+05, 4.837994E+05, 5.206808E+05, 
   5.600788E+05, 6.021530E+05, 6.470715E+05, 6.950106E+05, 7.461558E+05, 8.007016E+05, 
   8.588524E+05, 9.208227E+05, 9.868374E+05, 1.057132E+06, 1.131955E+06, 1.211564E+06, 
   1.296231E+06, 1.386241E+06, 1.481889E+06, 1.583488E+06, 1.691363E+06, 1.805854E+06, 
   1.927316E+06, 2.056120E+06, 2.192654E+06, 2.337323E+06, 2.490550E+06, 2.652773E+06, 
   2.824453E+06, 3.006069E+06, 3.198119E+06, 3.401123E+06, 3.615623E+06, 3.842180E+06, 
   4.081382E+06, 4.333839E+06, 4.600184E+06, 4.881077E+06, 5.177203E+06, 5.489274E+06, 
   5.818030E+06, 6.164238E+06, 6.528697E+06, 6.912232E+06, 7.315703E+06, 7.740000E+06, 
   8.186046E+06, 8.654798E+06, 9.147247E+06, 9.664422E+06, 1.020739E+07, 1.077724E+07, 
   1.137513E+07, 1.200224E+07, 1.265978E+07, 1.334902E+07, 1.407126E+07, 1.482787E+07, 
   1.562024E+07, 1.644981E+07, 1.731807E+07, 1.822658E+07, 1.917691E+07, 2.017072E+07, 
   2.120970E+07, 2.229561E+07, 2.343025E+07, 2.461550E+07, 2.585327E+07, 2.714555E+07, 
   2.849439E+07, 2.990190E+07, 3.137025E+07, 3.290169E+07, 3.449852E+07, 3.616312E+07, 
   3.789795E+07, 3.970552E+07, 4.158844E+07, 4.354937E+07, 4.559106E+07, 4.771636E+07, 
   4.992817E+07, 5.222949E+07, 5.462340E+07, 5.711306E+07, 5.970175E+07, 6.239280E+07, 
   6.518965E+07, 6.809586E+07, 7.111503E+07, 7.425092E+07, 7.750736E+07, 8.088828E+07, 
   8.439773E+07, 8.803986E+07, 9.181894E+07, 9.573932E+07, 9.980551E+07, 1.040221E+08, 
   1.083938E+08, 1.129256E+08, 1.176222E+08, 1.224890E+08, 1.275310E+08, 1.327536E+08, 
   1.381624E+08, 1.437629E+08, 1.495609E+08, 1.555624E+08, 1.617734E+08, 1.682002E+08, 
   1.748489E+08, 1.817263E+08, 1.888389E+08, 1.961935E+08, 2.037972E+08, 2.116572E+08, 
   2.197806E+08, 2.281751E+08, 2.368483E+08, 2.458080E+08, 2.550623E+08, 2.646194E+08, 
   2.744877E+08, 2.846757E+08, 2.951924E+08, 3.060465E+08, 3.172474E+08, 3.288044E+08, 
   3.407271E+08, 3.530254E+08, 3.657092E+08, 3.787888E+08, 3.922746E+08, 4.061775E+08, 
   4.205082E+08, 4.352780E+08, 4.504982E+08, 4.661805E+08, 4.823368E+08, 4.989793E+08, 
   5.161202E+08, 5.337723E+08, 5.519485E+08, 5.706619E+08, 5.899261E+08, 6.097547E+08, 
   6.301618E+08, 6.511617E+08, 6.727689E+08, 6.949983E+08, 7.178651E+08, 7.413848E+08, 
   7.655732E+08, 7.904464E+08, 8.160208E+08, 8.423132E+08, 8.693405E+08, 8.971203E+08, 
   9.256702E+08, 9.550084E+08, 9.851533E+08, 1.016124E+09, 1.047938E+09, 1.080618E+09, 
   1.114181E+09, 1.148648E+09, 1.184040E+09, 1.220378E+09, 1.257683E+09, 1.295978E+09, 
   1.335284E+09, 1.375623E+09, 1.417020E+09, 1.459497E+09, 1.503078E+09, 1.547788E+09, 
   1.593651E+09, 1.640693E+09, 1.688939E+09, 1.738415E+09, 1.789148E+09, 1.841164E+09, 
   1.894492E+09, 1.949160E+09, 2.005195E+09, 2.062627E+09, 2.121486E+09, 2.181800E+09, 
   2.243602E+09, 2.306921E+09, 2.371789E+09, 2.438239E+09, 2.506303E+09, 2.576014E+09, 
   2.647407E+09, 2.720514E+09, 2.795372E+09, 2.872016E+09, 2.950481E+09, 3.030805E+09, 
   3.113024E+09, 3.197177E+09, 3.283302E+09, 3.371438E+09, 3.461625E+09, 3.553903E+09, 
   3.648314E+09, 3.744898E+09, 3.843699E+09, 3.944760E+09, 4.048124E+09, 4.153835E+09, 
   4.261939E+09, 4.372482E+09, 4.485510E+09, 4.601071E+09, 4.719212E+09, 4.839982E+09, 
   4.963432E+09, 5.089610E+09, 5.218568E+09, 5.350359E+09, 5.485034E+09, 5.622647E+09, 
   5.763253E+09, 5.906906E+09, 6.053663E+09, 6.203579E+09, 6.356714E+09, 6.513124E+09, 
   6.672870E+09, 6.836011E+09, 7.002610E+09, 7.172727E+09, 7.346426E+09, 7.523770E+09, 
   7.704824E+09, 7.889655E+09, 8.078328E+09, 8.270911E+09, 8.467473E+09, 8.668083E+09, 
   8.872813E+09, 9.081732E+09, 9.294915E+09, 9.512435E+09, 9.734366E+09, 9.960785E+09, 
   1.019177E+10, 1.042739E+10, 1.066774E+10, 1.091288E+10, 1.116291E+10, 1.141790E+10, 
   1.167794E+10, 1.194311E+10, 1.221350E+10, 1.248920E+10, 1.277028E+10, 1.305685E+10, 
   1.334900E+10, 1.364680E+10, 1.395037E+10, 1.425978E+10, 1.457514E+10, 1.489655E+10, 
   1.522410E+10, 1.555789E+10, 1.589802E+10, 1.624460E+10, 1.659772E+10, 1.695749E+10, 
   1.732403E+10, 1.769742E+10, 1.807780E+10, 1.846525E+10, 1.885990E+10, 1.926187E+10, 
   1.967125E+10, 2.008818E+10, 2.051276E+10, 2.094512E+10, 2.138538E+10, 2.183365E+10, 
   2.229007E+10, 2.275476E+10, 2.322784E+10, 2.370944E+10, 2.419970E+10, 2.469874E+10, 
   2.520670E+10, 2.572371E+10, 2.624991E+10, 2.678544E+10, 2.733043E+10, 2.788504E+10, 
   2.844939E+10, 2.902365E+10, 2.960794E+10, 3.020244E+10, 3.080727E+10, 3.142261E+10, 
   3.204859E+10, 3.268538E+10, 3.333313E+10, 3.399201E+10, 3.466217E+10, 3.534378E+10, 
   3.603701E+10, 3.674202E+10, 3.745898E+10, 3.818806E+10, 3.892944E+10, 3.968330E+10, 
   4.044980E+10, 4.122914E+10, 4.202148E+10, 4.282702E+10, 4.364594E+10, 4.447844E+10, 
   4.532469E+10, 4.618489E+10, 4.705924E+10, 4.794794E+10, 4.885118E+10, 4.976916E+10, 
   5.070210E+10, 5.165018E+10, 5.261363E+10, 5.359265E+10, 5.458746E+10, 5.559828E+10, 
   5.662531E+10, 5.766878E+10, 5.872892E+10, 5.980594E+10, 6.090008E+10, 6.201157E+10, 
   6.314064E+10, 6.428752E+10, 6.545245E+10, 6.663568E+10, 6.783744E+10, 6.905799E+10, 
   7.029757E+10, 7.155642E+10, 7.283482E+10, 7.413301E+10, 7.545124E+10, 7.678980E+10, 
   7.814893E+10, 7.952890E+10, 8.093000E+10, 8.235249E+10, 8.379664E+10, 8.526274E+10, 
   8.675107E+10, 8.826192E+10, 8.979556E+10, 9.135230E+10, 9.293244E+10, 9.453625E+10, 
   9.616406E+10, 9.781615E+10, 9.949284E+10, 1.011944E+11, 1.029213E+11, 1.046736E+11, 
   1.064518E+11, 1.082562E+11, 1.100871E+11, 1.119449E+11, 1.138298E+11, 1.157422E+11, 
   1.176824E+11, 1.196509E+11, 1.216478E+11, 1.236737E+11, 1.257288E+11, 1.278135E+11, 
   1.299282E+11, 1.320731E+11, 1.342488E+11, 1.364555E+11, 1.386937E+11, 1.409637E+11, 
   1.432658E+11, 1.456006E+11, 1.479683E+11, 1.503694E+11, 1.528043E+11, 1.552734E+11, 
   1.577770E+11, 1.603156E+11, 1.628896E+11, 1.654994E+11, 1.681455E+11, 1.708282E+11, 
   1.735481E+11, 1.763054E+11, 1.791008E+11, 1.819345E+11, 1.848071E+11, 1.877190E+11, 
   1.906707E+11, 1.936626E+11, 1.966952E+11, 1.997690E+11, 2.028844E+11, 2.060419E+11, 
   2.092420E+11, 2.124851E+11, 2.157718E+11, 2.191026E+11, 2.224780E+11, 2.258984E+11, 
   2.293644E+11, 2.328765E+11, 2.364352E+11, 2.400411E+11, 2.436946E+11, 2.473963E+11, 
   2.511467E+11, 2.549465E+11, 2.587960E+11, 2.626959E+11, 2.666468E+11, 2.706492E+11, 
   2.747036E+11, 2.788107E+11, 2.829710E+11, 2.871852E+11, 2.914537E+11, 2.957772E+11, 
   3.001563E+11, 3.045916E+11, 3.090837E+11, 3.136332E+11, 3.182408E+11, 3.229070E+11, 
   3.276326E+11, 3.324180E+11, 3.372641E+11, 3.421714E+11, 3.471406E+11, 3.521724E+11, 
   3.572674E+11, 3.624263E+11, 3.676497E+11, 3.729384E+11, 3.782930E+11, 3.837143E+11, 
   3.892030E+11, 3.947597E+11, 4.003851E+11, 4.060800E+11, 4.118452E+11, 4.176813E+11, 
   4.235891E+11, 4.295693E+11, 4.356227E+11, 4.417501E+11, 4.479522E+11, 4.542297E+11, 
   4.605835E+11, 4.670144E+11, 4.735230E+11, 
])
# ============================ M = 56, I = 1 ============================
#M = 56
#I = 1
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   1.640085E+01, 2.995820E+02, 7.723531E+02, 1.386065E+03, 2.120849E+03, 2.958902E+03, 
#   3.889100E+03, 4.906556E+03, 6.011409E+03, 7.207592E+03, 8.501785E+03, 9.902625E+03, 
#   1.142022E+04, 1.306585E+04, 1.485188E+04, 1.679172E+04, 1.889985E+04, 2.119191E+04, 
#   2.368480E+04, 2.639677E+04, 2.934758E+04, 3.255859E+04, 3.605292E+04, 3.985562E+04, 
#   4.399380E+04, 4.849681E+04, 5.339642E+04, 5.872701E+04, 6.452581E+04, 7.083304E+04, 
#   7.769223E+04, 8.515043E+04, 9.325845E+04, 1.020712E+05, 1.116480E+05, 1.220528E+05, 
#   1.333548E+05, 1.456282E+05, 1.589536E+05, 1.734175E+05, 1.891131E+05, 2.061411E+05, 
#   2.246095E+05, 2.446350E+05, 2.663429E+05, 2.898682E+05, 3.153558E+05, 3.429619E+05, 
#   3.728541E+05, 4.052125E+05, 4.402309E+05, 4.781170E+05, 5.190940E+05, 5.634015E+05, 
#   6.112965E+05, 6.630548E+05, 7.189721E+05, 7.793653E+05, 8.445742E+05, 9.149631E+05, 
#   9.909218E+05, 1.072868E+06, 1.161250E+06, 1.256544E+06, 1.359265E+06, 1.469959E+06, 
#   1.589214E+06, 1.717655E+06, 1.855952E+06, 2.004822E+06, 2.165030E+06, 2.337392E+06, 
#   2.522782E+06, 2.722130E+06, 2.936432E+06, 3.166747E+06, 3.414209E+06, 3.680024E+06, 
#   3.965479E+06, 4.271947E+06, 4.600890E+06, 4.953867E+06, 5.332538E+06, 5.738670E+06, 
#   6.174147E+06, 6.640972E+06, 7.141278E+06, 7.677334E+06, 8.251553E+06, 8.866504E+06, 
#   9.524915E+06, 1.022969E+07, 1.098391E+07, 1.179085E+07, 1.265400E+07, 1.357704E+07, 
#   1.456391E+07, 1.561878E+07, 1.674606E+07, 1.795046E+07, 1.923694E+07, 2.061079E+07, 
#   2.207761E+07, 2.364334E+07, 2.531428E+07, 2.709709E+07, 2.899884E+07, 3.102702E+07, 
#   3.318956E+07, 3.549485E+07, 3.795178E+07, 4.056976E+07, 4.335875E+07, 4.632929E+07, 
#   4.949250E+07, 5.286019E+07, 5.644481E+07, 6.025953E+07, 6.431828E+07, 6.863578E+07, 
#   7.322756E+07, 7.811006E+07, 8.330062E+07, 8.881755E+07, 9.468019E+07, 1.009090E+08, 
#   1.075254E+08, 1.145522E+08, 1.220134E+08, 1.299342E+08, 1.383414E+08, 1.472631E+08, 
#   1.567288E+08, 1.667699E+08, 1.774193E+08, 1.887116E+08, 2.006834E+08, 2.133733E+08, 
#   2.268217E+08, 2.410713E+08, 2.561671E+08, 2.721563E+08, 2.890887E+08, 3.070166E+08, 
#   3.259952E+08, 3.460824E+08, 3.673390E+08, 3.898291E+08, 4.136202E+08, 4.387829E+08, 
#   4.653916E+08, 4.935246E+08, 5.232638E+08, 5.546956E+08, 5.879106E+08, 6.230038E+08, 
#   6.600752E+08, 6.992295E+08, 7.405769E+08, 7.842327E+08, 8.303182E+08, 8.789605E+08, 
#   9.302928E+08, 9.844550E+08, 1.041594E+09, 1.101863E+09, 1.165423E+09, 1.232444E+09, 
#   1.303102E+09, 1.377583E+09, 1.456081E+09, 1.538800E+09, 1.625953E+09, 1.717763E+09, 
#   1.814464E+09, 1.916300E+09, 2.023529E+09, 2.136418E+09, 2.255247E+09, 2.380310E+09, 
#   2.511916E+09, 2.650383E+09, 2.796050E+09, 2.949266E+09, 3.110401E+09, 3.279837E+09, 
#   3.457976E+09, 3.645239E+09, 3.842064E+09, 4.048908E+09, 4.266252E+09, 4.494594E+09, 
#   4.734458E+09, 4.986390E+09, 5.250958E+09, 5.528758E+09, 5.820412E+09, 6.126567E+09, 
#   6.447900E+09, 6.785117E+09, 7.138955E+09, 7.510183E+09, 7.899602E+09, 8.308049E+09, 
#   8.736397E+09, 9.185554E+09, 9.656470E+09, 1.015013E+10, 1.066757E+10, 1.120987E+10, 
#   1.177813E+10, 1.237354E+10, 1.299730E+10, 1.365068E+10, 1.433500E+10, 1.505163E+10, 
#   1.580201E+10, 1.658763E+10, 1.741003E+10, 1.827084E+10, 1.917173E+10, 2.011446E+10, 
#   2.110084E+10, 2.213278E+10, 2.321224E+10, 2.434128E+10, 2.552204E+10, 2.675672E+10, 
#   2.804765E+10, 2.939722E+10, 3.080794E+10, 3.228239E+10, 3.382328E+10, 3.543342E+10, 
#   3.711572E+10, 3.887322E+10, 4.070906E+10, 4.262652E+10, 4.462899E+10, 4.672002E+10, 
#   4.890327E+10, 5.118256E+10, 5.356184E+10, 5.604522E+10, 5.863699E+10, 6.134155E+10, 
#   6.416353E+10, 6.710769E+10, 7.017899E+10, 7.338258E+10, 7.672380E+10, 8.020818E+10, 
#   8.384148E+10, 8.762966E+10, 9.157890E+10, 9.569562E+10, 9.998648E+10, 1.044584E+11, 
#   1.091185E+11, 1.139742E+11, 1.190332E+11, 1.243035E+11, 1.297933E+11, 1.355113E+11, 
#   1.414662E+11, 1.476674E+11, 1.541243E+11, 1.608468E+11, 1.678452E+11, 1.751300E+11, 
#   1.827122E+11, 1.906033E+11, 1.988149E+11, 2.073594E+11, 2.162492E+11, 2.254976E+11, 
#   2.351180E+11, 2.451245E+11, 2.555316E+11, 2.663542E+11, 2.776080E+11, 2.893090E+11, 
#   3.014739E+11, 3.141199E+11, 3.272648E+11, 3.409270E+11, 3.551256E+11, 3.698804E+11, 
#   3.852117E+11, 4.011406E+11, 4.176890E+11, 4.348794E+11, 4.527350E+11, 4.712802E+11, 
#   4.905396E+11, 5.105393E+11, 5.313056E+11, 5.528663E+11, 5.752497E+11, 5.984853E+11, 
#   6.226033E+11, 6.476352E+11, 6.736135E+11, 7.005715E+11, 7.285440E+11, 7.575666E+11, 
#   7.876762E+11, 8.189111E+11, 8.513105E+11, 8.849150E+11, 9.197668E+11, 9.559090E+11, 
#   9.933865E+11, 1.032245E+12, 1.072533E+12, 1.114300E+12, 1.157595E+12, 1.202472E+12, 
#   1.248984E+12, 1.297188E+12, 1.347140E+12, 1.398901E+12, 1.452532E+12, 1.508095E+12, 
#   1.565656E+12, 1.625283E+12, 1.687044E+12, 1.751012E+12, 1.817259E+12, 1.885862E+12, 
#   1.956900E+12, 2.030453E+12, 2.106605E+12, 2.185442E+12, 2.267051E+12, 2.351525E+12, 
#   2.438958E+12, 2.529446E+12, 2.623089E+12, 2.719991E+12, 2.820258E+12, 2.923999E+12, 
#   3.031326E+12, 3.142356E+12, 3.257208E+12, 3.376006E+12, 3.498876E+12, 3.625948E+12, 
#   3.757358E+12, 3.893245E+12, 4.033749E+12, 4.179020E+12, 4.329207E+12, 4.484466E+12, 
#   4.644959E+12, 4.810849E+12, 4.982308E+12, 5.159509E+12, 5.342633E+12, 5.531865E+12, 
#   5.727395E+12, 5.929421E+12, 6.138143E+12, 6.353770E+12, 6.576514E+12, 6.806597E+12, 
#   7.044245E+12, 7.289689E+12, 7.543170E+12, 7.804933E+12, 8.075232E+12, 8.354328E+12, 
#   8.642488E+12, 8.939987E+12, 9.247110E+12, 9.564146E+12, 9.891397E+12, 1.022917E+13, 
#   1.057778E+13, 1.093755E+13, 1.130882E+13, 1.169194E+13, 1.208725E+13, 1.249511E+13, 
#   1.291591E+13, 1.335003E+13, 1.379785E+13, 1.425979E+13, 1.473626E+13, 1.522769E+13, 
#   1.573451E+13, 1.625719E+13, 1.679618E+13, 1.735196E+13, 1.792502E+13, 1.851586E+13, 
#   1.912500E+13, 1.975296E+13, 2.040030E+13, 2.106757E+13, 2.175535E+13, 2.246422E+13, 
#   2.319479E+13, 2.394768E+13, 2.472354E+13, 2.552301E+13, 2.634677E+13, 2.719551E+13, 
#   2.806993E+13, 2.897077E+13, 2.989877E+13, 3.085470E+13, 3.183934E+13, 3.285350E+13, 
#   3.389801E+13, 3.497372E+13, 3.608150E+13, 3.722224E+13, 3.839686E+13, 3.960631E+13, 
#   4.085154E+13, 4.213356E+13, 4.345337E+13, 4.481202E+13, 4.621058E+13, 4.765015E+13, 
#   4.913184E+13, 5.065681E+13, 5.222626E+13, 5.384137E+13, 5.550342E+13, 5.721366E+13, 
#   5.897340E+13, 6.078399E+13, 6.264680E+13, 6.456324E+13, 6.653475E+13, 6.856281E+13, 
#   7.064895E+13, 7.279471E+13, 7.500169E+13, 7.727153E+13, 7.960590E+13, 8.200651E+13, 
#   8.447513E+13, 8.701355E+13, 8.962363E+13, 9.230725E+13, 9.506636E+13, 9.790293E+13, 
#   1.008190E+14, 1.038167E+14, 1.068981E+14, 1.100654E+14, 1.133208E+14, 1.166667E+14, 
#   1.201054E+14, 1.236393E+14, 1.272709E+14, 1.310027E+14, 1.348372E+14, 1.387772E+14, 
#   1.428254E+14, 1.469844E+14, 1.512572E+14, 1.556467E+14, 1.601558E+14, 1.647877E+14, 
#   1.695453E+14, 1.744320E+14, 1.794510E+14, 1.846056E+14, 1.898993E+14, 1.953356E+14, 
#   2.009181E+14, 2.066504E+14, 2.125364E+14, 2.185798E+14, 2.247846E+14, 2.311548E+14, 
#   2.376947E+14, 2.444083E+14, 2.513000E+14, 2.583743E+14, 2.656356E+14, 2.730886E+14, 
#   2.807380E+14, 2.885886E+14, 2.966455E+14, 3.049136E+14, 3.133982E+14, 3.221045E+14, 
#   3.310380E+14, 3.402042E+14, 3.496088E+14, 3.592576E+14, 3.691566E+14, 3.793116E+14, 
#   3.897291E+14, 4.004153E+14, 4.113768E+14, 
#])
# ============================ M = 57, I = 1 ============================
#M = 57
#I = 1
#TIPS_2021_ISOT_HASH[(M,I)] = TIPS_2021_ISOT[6]
#TIPS_2021_ISOQ_HASH[(M,I)] = float64([
#   0.000000E+00, 2.387312E+00, 9.439835E+00, 1.825869E+01, 2.871728E+01, 4.069890E+01, 
#   5.406319E+01, 6.869271E+01, 8.449555E+01, 1.013997E+02, 1.193488E+02, 1.382996E+02, 
#   1.582205E+02, 1.790899E+02, 2.008949E+02, 2.236301E+02, 2.472961E+02, 2.718986E+02, 
#   2.974472E+02, 3.239546E+02, 3.514358E+02, 3.799080E+02, 4.093897E+02, 4.399006E+02, 
#   4.714617E+02, 5.040943E+02, 5.378208E+02, 5.726640E+02, 6.086474E+02, 6.457948E+02, 
#   6.841307E+02, 7.236799E+02, 7.644677E+02, 8.065197E+02, 8.498620E+02, 8.945212E+02, 
#   9.405242E+02, 9.878982E+02, 1.036671E+03, 1.086871E+03, 1.138525E+03, 1.191664E+03, 
#   1.246317E+03, 1.302513E+03, 1.360281E+03, 1.419654E+03, 1.480661E+03, 1.543335E+03, 
#   1.607706E+03, 1.673807E+03, 1.741671E+03, 1.811332E+03, 1.882822E+03, 1.956176E+03, 
#   2.031428E+03, 2.108615E+03, 2.187770E+03, 2.268932E+03, 2.352137E+03, 2.437421E+03, 
#   2.524823E+03, 2.614382E+03, 2.706136E+03, 2.800126E+03, 2.896391E+03, 2.994973E+03, 
#   3.095913E+03, 3.199253E+03, 3.305037E+03, 3.413307E+03, 3.524108E+03, 3.637484E+03, 
#   3.753482E+03, 3.872147E+03, 3.993526E+03, 4.117667E+03, 4.244617E+03, 4.374427E+03, 
#   4.507146E+03, 4.642823E+03, 4.781511E+03, 4.923261E+03, 5.068125E+03, 5.216158E+03, 
#   5.367413E+03, 5.521945E+03, 5.679810E+03, 5.841063E+03, 6.005763E+03, 6.173968E+03, 
#   6.345735E+03, 6.521125E+03, 6.700197E+03, 6.883014E+03, 7.069636E+03, 7.260127E+03, 
#   7.454550E+03, 7.652970E+03, 7.855452E+03, 8.062061E+03, 8.272866E+03, 8.487932E+03, 
#   8.707330E+03, 8.931128E+03, 9.159397E+03, 9.392208E+03, 9.629633E+03, 9.871745E+03, 
#   1.011862E+04, 1.037033E+04, 1.062694E+04, 1.088855E+04, 1.115522E+04, 1.142704E+04, 
#   1.170407E+04, 1.198641E+04, 1.227414E+04, 1.256732E+04, 1.286606E+04, 1.317043E+04, 
#   1.348052E+04, 1.379641E+04, 1.411819E+04, 1.444595E+04, 1.477977E+04, 1.511975E+04, 
#   1.546598E+04, 1.581855E+04, 1.617755E+04, 1.654307E+04, 1.691522E+04, 1.729408E+04, 
#   1.767975E+04, 1.807234E+04, 1.847193E+04, 1.887863E+04, 1.929255E+04, 1.971377E+04, 
#   2.014242E+04, 2.057858E+04, 2.102237E+04, 2.147389E+04, 2.193326E+04, 2.240057E+04, 
#   2.287594E+04, 2.335948E+04, 2.385130E+04, 2.435151E+04, 2.486024E+04, 2.537759E+04, 
#   2.590368E+04, 2.643863E+04, 2.698256E+04, 2.753558E+04, 2.809783E+04, 2.866942E+04, 
#   2.925048E+04, 2.984113E+04, 3.044150E+04, 3.105171E+04, 3.167190E+04, 3.230219E+04, 
#   3.294272E+04, 3.359361E+04, 3.425501E+04, 3.492705E+04, 3.560985E+04, 3.630358E+04, 
#   3.700835E+04, 3.772431E+04, 3.845160E+04, 3.919037E+04, 3.994076E+04, 4.070292E+04, 
#   4.147698E+04, 4.226311E+04, 4.306145E+04, 4.387215E+04, 4.469536E+04, 4.553125E+04, 
#   4.637995E+04, 4.724164E+04, 4.811646E+04, 4.900458E+04, 4.990616E+04, 5.082136E+04, 
#   5.175035E+04, 5.269329E+04, 5.365034E+04, 5.462168E+04, 5.560747E+04, 5.660790E+04, 
#   5.762312E+04, 5.865331E+04, 5.969865E+04, 6.075932E+04, 6.183550E+04, 6.292736E+04, 
#   6.403509E+04, 6.515887E+04, 6.629888E+04, 6.745532E+04, 6.862837E+04, 6.981822E+04, 
#   7.102507E+04, 7.224910E+04, 7.349051E+04, 7.474949E+04, 7.602625E+04, 7.732098E+04, 
#   7.863389E+04, 7.996518E+04, 8.131504E+04, 8.268370E+04, 8.407135E+04, 8.547821E+04, 
#   8.690448E+04, 8.835039E+04, 8.981613E+04, 9.130194E+04, 9.280803E+04, 9.433461E+04, 
#   9.588192E+04, 9.745017E+04, 9.903959E+04, 1.006504E+05, 1.022828E+05, 1.039371E+05, 
#   1.056135E+05, 1.073122E+05, 1.090335E+05, 1.107775E+05, 1.125446E+05, 1.143349E+05, 
#   1.161488E+05, 1.179864E+05, 1.198481E+05, 1.217340E+05, 1.236444E+05, 1.255796E+05, 
#   1.275397E+05, 1.295252E+05, 1.315362E+05, 1.335729E+05, 1.356358E+05, 1.377249E+05, 
#   1.398406E+05, 1.419832E+05, 1.441529E+05, 1.463499E+05, 1.485747E+05, 1.508273E+05, 
#   1.531082E+05, 1.554176E+05, 1.577558E+05, 1.601230E+05, 1.625195E+05, 1.649457E+05, 
#   1.674018E+05, 1.698882E+05, 1.724050E+05, 1.749526E+05, 1.775314E+05, 1.801415E+05, 
#   1.827833E+05, 1.854572E+05, 1.881633E+05, 1.909021E+05, 1.936738E+05, 1.964787E+05, 
#   1.993172E+05, 2.021895E+05, 2.050961E+05, 2.080371E+05, 2.110130E+05, 2.140240E+05, 
#   2.170706E+05, 2.201529E+05, 2.232714E+05, 2.264263E+05, 2.296181E+05, 2.328470E+05, 
#   2.361134E+05, 2.394176E+05, 2.427600E+05, 2.461409E+05, 2.495607E+05, 2.530197E+05, 
#   2.565183E+05, 2.600568E+05, 2.636356E+05, 2.672551E+05, 2.709155E+05, 2.746173E+05, 
#   2.783609E+05, 2.821465E+05, 2.859747E+05, 2.898457E+05, 2.937599E+05, 2.977177E+05, 
#   3.017195E+05, 3.057656E+05, 3.098566E+05, 3.139926E+05, 3.181742E+05, 3.224017E+05, 
#   3.266755E+05, 3.309960E+05, 3.353636E+05, 3.397787E+05, 3.442417E+05, 3.487531E+05, 
#   3.533131E+05, 3.579223E+05, 3.625811E+05, 3.672898E+05, 3.720488E+05, 3.768587E+05, 
#   3.817198E+05, 3.866325E+05, 3.915973E+05, 3.966146E+05, 4.016848E+05, 4.068084E+05, 
#   4.119858E+05, 4.172175E+05, 4.225038E+05, 4.278452E+05, 4.332422E+05, 4.386952E+05, 
#   4.442047E+05, 4.497712E+05, 4.553950E+05, 4.610766E+05, 4.668165E+05, 4.726152E+05, 
#   4.784731E+05, 4.843907E+05, 4.903685E+05, 4.964070E+05, 5.025065E+05, 5.086676E+05, 
#   5.148908E+05, 5.211766E+05, 5.275254E+05, 5.339377E+05, 5.404141E+05, 5.469549E+05, 
#   5.535608E+05, 5.602322E+05, 5.669696E+05, 5.737735E+05, 5.806444E+05, 5.875829E+05, 
#   5.945894E+05, 6.016644E+05, 6.088085E+05, 6.160222E+05, 6.233060E+05, 6.306605E+05, 
#   6.380861E+05, 6.455834E+05, 6.531529E+05, 6.607951E+05, 6.685107E+05, 6.763000E+05, 
#   6.841638E+05, 6.921024E+05, 7.001166E+05, 7.082067E+05, 7.163734E+05, 7.246172E+05, 
#   7.329387E+05, 7.413384E+05, 7.498170E+05, 7.583749E+05, 7.670127E+05, 7.757311E+05, 
#   7.845305E+05, 7.934115E+05, 8.023748E+05, 8.114210E+05, 8.205505E+05, 8.297640E+05, 
#   8.390621E+05, 8.484453E+05, 8.579143E+05, 8.674697E+05, 8.771120E+05, 8.868419E+05, 
#   8.966600E+05, 9.065668E+05, 9.165630E+05, 9.266492E+05, 9.368261E+05, 9.470942E+05, 
#   9.574541E+05, 9.679065E+05, 9.784520E+05, 9.890913E+05, 9.998249E+05, 1.010654E+06, 
#   1.021578E+06, 1.032598E+06, 1.043716E+06, 1.054931E+06, 1.066244E+06, 1.077656E+06, 
#   1.089168E+06, 1.100780E+06, 1.112493E+06, 1.124307E+06, 1.136223E+06, 1.148243E+06, 
#   1.160365E+06, 1.172592E+06, 1.184924E+06, 1.197362E+06, 1.209905E+06, 1.222556E+06, 
#   1.235314E+06, 1.248180E+06, 1.261156E+06, 1.274241E+06, 1.287436E+06, 1.300743E+06, 
#   1.314161E+06, 1.327692E+06, 1.341336E+06, 1.355094E+06, 1.368966E+06, 1.382954E+06, 
#   1.397058E+06, 1.411279E+06, 1.425617E+06, 1.440074E+06, 1.454649E+06, 1.469344E+06, 
#   1.484160E+06, 1.499097E+06, 1.514156E+06, 1.529337E+06, 1.544642E+06, 1.560071E+06, 
#   1.575625E+06, 1.591305E+06, 1.607111E+06, 1.623045E+06, 1.639106E+06, 1.655296E+06, 
#   1.671616E+06, 1.688066E+06, 1.704647E+06, 1.721360E+06, 1.738206E+06, 1.755185E+06, 
#   1.772298E+06, 1.789547E+06, 1.806931E+06, 1.824451E+06, 1.842110E+06, 1.859906E+06, 
#   1.877841E+06, 1.895917E+06, 1.914133E+06, 1.932490E+06, 1.950989E+06, 1.969632E+06, 
#   1.988419E+06, 2.007350E+06, 2.026427E+06, 2.045651E+06, 2.065021E+06, 2.084540E+06, 
#   2.104208E+06, 2.124025E+06, 2.143993E+06, 2.164113E+06, 2.184385E+06, 2.204810E+06, 
#   2.225389E+06, 2.246123E+06, 2.267013E+06, 2.288059E+06, 2.309263E+06, 2.330625E+06, 
#   2.352147E+06, 2.373828E+06, 2.395671E+06, 2.417676E+06, 2.439843E+06, 2.462174E+06, 
#   2.484670E+06, 2.507331E+06, 2.530158E+06, 2.553153E+06, 2.576316E+06, 2.599648E+06, 
#   2.623150E+06, 2.646823E+06, 2.670668E+06, 
#])

def BD_TIPS_2021_PYTHON(M,I,T):
    # get temperature grid
    TT = TIPS_2021_ISOT_HASH[(M,I)]
    Tmin = min(TT); Tmax = max(TT)
    
    # out of temperature range
    if T<Tmin or T>Tmax:
        raise Exception('TIPS2021: T(%.1fK) must be between %.1fK and %.1fK.'%(T,Tmin,Tmax))
    
    try:
        # get statistical weight for specified isotopologue
        #gi = TIPS_GSI_HASH[(M,I)] # Take from TIPS-2011?
        # interpolate partition sum for specified isotopologue
        Qt = AtoB(T,TT,TIPS_2021_ISOQ_HASH[(M,I)],len(TT))
    except KeyError:
        raise Exception('TIPS2021: no data for M,I = %d,%d.' % (M,I))
    
    return None,Qt

# =========================/TIPS2021 PARTITION SUMS =========================

# ALIASES FOR TIPS
PYTIPS2011 = lambda M,I,T: BD_TIPS_2011_PYTHON(M,I,T)[1]
PYTIPS2017 = lambda M,I,T: BD_TIPS_2017_PYTHON(M,I,T)[1]
PYTIPS2017_SLICE = lambda M,I,T,n=20: BD_TIPS_2017_PYTHON_SLICE(M,I,T,n)[1]
PYTIPS2021 = lambda M,I,T: BD_TIPS_2021_PYTHON(M,I,T)[1]
PYTIPS = PYTIPS2021 # stub for backwards compatibility

# Total internal partition sum
# M - molecule number
# I - isotopologue number
# T - temperature (K)
# returns (StatWeight,PartitionSum)
def partitionSum(M,I,T,step=None,version=2021):
    """
    INPUT PARAMETERS: 
        M: HITRAN molecule number              (required)
        I: HITRAN isotopologue number          (required)
        T: temperature conditions              (required)
        step:  step to calculate temperatures  (optional)
    OUTPUT PARAMETERS:
        TT: list of temperatures (present only if T is a list)
        PartSum: partition sums calculated on a list of temperatures
    ---
    DESCRIPTION:
        Calculate range of partition sums at different temperatures.
        This function uses a python implementation of TIPS-2011 code:

        References:
        
            TIPS-2011:
            Laraia AL, Gamache RR, Lamouroux J, Gordon IE, Rothman LS. 
            Total internal partition sums to support planetary remote sensing. 
            Icarus 2011;215:391–400.
            http://dx.doi.org/10.1016/j.icarus.2011.06.004

            TIPS-2017:
            Gamache RR, Roller C, Lopes E, Gordon IE, Rothman LS, Polyansky OL, et al. 
            Total internal partition sums for 166 isotopologues of 51 molecules 
            important in planetary atmospheres: Application to HITRAN2016 and beyond. 
            J Quant Spectrosc Radiat Transf 2017;203:70–87. 
            http://dx.doi.org/10.1016/j.jqsrt.2017.03.045
            
            TIPS-2021:
            Gamache et al. (2021), JQSRT 271, 107713 (doi: 10.1016/j.jqsrt.2021.107713)
            Data obtained from: https://zenodo.org/record/4708099

        Output depends on a structure of input parameter T so that:
            1) If T is a scalar/list and step IS NOT provided,
                then calculate partition sums over each value of T.
            2) If T is a list and step parameter IS provided,
                then calculate partition sums between T[0] and T[1]
                with a given step.
    ---
    EXAMPLE OF USAGE:
        PartSum = partitionSum(1,1,[296,1000])
        TT,PartSum = partitionSum(1,1,[296,1000],step=0.1)
    ---
    """
    # version selector
    if version==2011:
        BD_TIPS = BD_TIPS_2011_PYTHON
    elif version==2017:
        BD_TIPS = BD_TIPS_2017_PYTHON
    elif version==2021:
        BD_TIPS = BD_TIPS_2021_PYTHON
    else:
        raise Exception('Unknown version of TIPS: %s'%str(version))
    # partitionSum
    if not step:
       if type(T) not in set([list,tuple]):
          return BD_TIPS(M,I,T)[1]
       else:
          return [BD_TIPS(M,I,temp)[1] for temp in T]
    else:
       TT = arange(T[0],T[1],step)
       return TT,array([BD_TIPS(M,I,temp)[1] for temp in TT])

# ------------------ partition sum --------------------------------------


# ------------------ LINESHAPES -----------------------------------------

# ------------------ complex probability function -----------------------
# define static data
zone = __ComplexType__(1.0e0 + 0.0e0j)
zi = __ComplexType__(0.0e0 + 1.0e0j)
tt = __FloatType__([0.5e0,1.5e0,2.5e0,3.5e0,4.5e0,5.5e0,6.5e0,7.5e0,8.5e0,9.5e0,10.5e0,11.5e0,12.5e0,13.5e0,14.5e0])
pipwoeronehalf = __FloatType__(0.564189583547756e0)

# "naive" implementation for benchmarks
def cpf3(X,Y):

    # X,Y,WR,WI - numpy arrays
    if type(X) != ndarray: 
        if type(X) not in set([list,tuple]): 
            X = array([X])
        else:
            X = array(X)
    if type(Y) != ndarray: 
        if type(Y) not in set([list,tuple]): 
            Y = array([Y])
        else:
            Y = array(Y)

    zm1 = zone/__ComplexType__(X + zi*Y) # maybe redundant
    zm2 = zm1**2
    zsum = zone
    zterm=zone

    for tt_i in tt:
        zterm *= zm2*tt_i
        zsum += zterm
    
    zsum *= zi*zm1*pipwoeronehalf
    
    return zsum.real, zsum.imag

T = __FloatType__([0.314240376e0,0.947788391e0,1.59768264e0,2.27950708e0,3.02063703e0,3.8897249e0])
U = __FloatType__([1.01172805e0,-0.75197147e0,1.2557727e-2,1.00220082e-2,-2.42068135e-4,5.00848061e-7])
S = __FloatType__([1.393237e0,0.231152406e0,-0.155351466e0,6.21836624e-3,9.19082986e-5,-6.27525958e-7])

# Complex probability function implementation (Humlicek)
def cpf(X,Y):

    # X,Y,WR,WI - numpy arrays
    if type(X) != ndarray: 
        if type(X) not in set([list,tuple]): 
            X = array([X])
        else:
            X = array(X)
    if type(Y) != ndarray: 
        if type(Y) not in set([list,tuple]): 
            Y = array([Y])
        else:
            Y = array(Y)
    
    # REGION3
    index_REGION3 = where(sqrt(X**2 + Y**2) > __FloatType__(8.0e0))
    X_REGION3 = X[index_REGION3]
    Y_REGION3 = Y[index_REGION3]
    zm1 = zone/__ComplexType__(X_REGION3 + zi*Y_REGION3)
    zm2 = zm1**2
    zsum_REGION3 = zone
    zterm=zone
    for tt_i in tt:
        zterm *= zm2*tt_i
        zsum_REGION3 += zterm
    zsum_REGION3 *= zi*zm1*pipwoeronehalf
    
    index_REGION12 = setdiff1d(array(arange(len(X))),array(index_REGION3))
    X_REGION12 = X[index_REGION12]
    Y_REGION12 = Y[index_REGION12]
    
    WR = __FloatType__(0.0e0)
    WI = __FloatType__(0.0e0)
    
    # REGION12
    Y1_REGION12 = Y_REGION12 + __FloatType__(1.5e0)
    Y2_REGION12 = Y1_REGION12**2

    # REGION2    
    subindex_REGION2 = where((Y_REGION12 <= 0.85e0) & 
                             (abs(X_REGION12) >= (18.1e0*Y_REGION12 + 1.65e0)))
    
    index_REGION2 = index_REGION12[subindex_REGION2]
    
    X_REGION2 = X[index_REGION2]
    Y_REGION2 = Y[index_REGION2]
    Y1_REGION2 = Y1_REGION12[subindex_REGION2]
    Y2_REGION2 = Y2_REGION12[subindex_REGION2]
    Y3_REGION2 = Y_REGION2 + __FloatType__(3.0e0)
    
    WR_REGION2 = WR
    WI_REGION2 = WI

    WR_REGION2 = zeros(len(X_REGION2))
    ii = abs(X_REGION2) < __FloatType__(12.0e0)
    WR_REGION2[ii] = exp(-X_REGION2[ii]**2)
    WR_REGION2[~ii] = WR
    
    for I in range(6):
        R_REGION2 = X_REGION2 - T[I]
        R2_REGION2 = R_REGION2**2
        D_REGION2 = __FloatType__(1.0e0) / (R2_REGION2 + Y2_REGION2)
        D1_REGION2 = Y1_REGION2 * D_REGION2
        D2_REGION2 = R_REGION2 * D_REGION2
        WR_REGION2 = WR_REGION2 + Y_REGION2 * (U[I]*(R_REGION2*D2_REGION2 - 1.5e0*D1_REGION2) + 
                                               S[I]*Y3_REGION2*D2_REGION2)/(R2_REGION2 + 2.25e0)
        R_REGION2 = X_REGION2 + T[I]
        R2_REGION2 = R_REGION2**2                
        D_REGION2 = __FloatType__(1.0e0) / (R2_REGION2 + Y2_REGION2)
        D3_REGION2 = Y1_REGION2 * D_REGION2
        D4_REGION2 = R_REGION2 * D_REGION2
        WR_REGION2 = WR_REGION2 + Y_REGION2 * (U[I]*(R_REGION2*D4_REGION2 - 1.5e0*D3_REGION2) - 
                                               S[I]*Y3_REGION2*D4_REGION2)/(R2_REGION2 + 2.25e0)
        WI_REGION2 = WI_REGION2 + U[I]*(D2_REGION2 + D4_REGION2) + S[I]*(D1_REGION2 - D3_REGION2)

    # REGION3
    index_REGION1 = setdiff1d(array(index_REGION12),array(index_REGION2))
    X_REGION1 = X[index_REGION1]
    Y_REGION1 = X[index_REGION1]
    
    subindex_REGION1 = setdiff1d(array(arange(len(index_REGION12))),array(subindex_REGION2))
    Y1_REGION1 = Y1_REGION12[subindex_REGION1]
    Y2_REGION1 = Y2_REGION12[subindex_REGION1]
    
    WR_REGION1 = WR
    WI_REGION1 = WI  
    
    for I in range(6):
        R_REGION1 = X_REGION1 - T[I]
        D_REGION1 = __FloatType__(1.0e0) / (R_REGION1**2 + Y2_REGION1)
        D1_REGION1 = Y1_REGION1 * D_REGION1
        D2_REGION1 = R_REGION1 * D_REGION1
        R_REGION1 = X_REGION1 + T[I]
        D_REGION1 = __FloatType__(1.0e0) / (R_REGION1**2 + Y2_REGION1)
        D3_REGION1 = Y1_REGION1 * D_REGION1
        D4_REGION1 = R_REGION1 * D_REGION1
        
        WR_REGION1 = WR_REGION1 + U[I]*(D1_REGION1 + D3_REGION1) - S[I]*(D2_REGION1 - D4_REGION1)
        WI_REGION1 = WI_REGION1 + U[I]*(D2_REGION1 + D4_REGION1) + S[I]*(D1_REGION1 - D3_REGION1)

    # total result
    WR_TOTAL = zeros(len(X))
    WI_TOTAL = zeros(len(X))
    # REGION3
    WR_TOTAL[index_REGION3] = zsum_REGION3.real
    WI_TOTAL[index_REGION3] = zsum_REGION3.imag
    # REGION2
    WR_TOTAL[index_REGION2] = WR_REGION2
    WI_TOTAL[index_REGION2] = WI_REGION2
    # REGION1
    WR_TOTAL[index_REGION1] = WR_REGION1
    WI_TOTAL[index_REGION1] = WI_REGION1
    
    return WR_TOTAL,WI_TOTAL


hcpf = cpf # stub for initial cpf
    
# ------------------ Schreier CPF ------------------------

# "Optimized implementations of rational approximations 
#  for the Voigt and complex error function".
# Franz Schreier. JQSRT 112 (2011) 1010-10250
# doi:10.1016/j.jqsrt.2010.12.010

# Enable this if numpy.polyval doesn't perform well.
"""    
def polyval(p, x):
    y = zeros(x.shape, dtype=float)
    for i, v in enumerate(p):
        y *= x
        y += v
    return y
""";
    
def cef(x,y,N):
    # Computes the function w(z) = exp(-zA2) erfc(-iz) using a rational
    # series with N terms. It is assumed that Im(z) > 0 or Im(z) = 0.
    z = x + 1.0j*y
    M = 2*N; M2 = 2*M; k = arange(-M+1,M) #'; # M2 = no. of sampling points.
    L = sqrt(N/sqrt(2)); # Optimal choice of L.
    theta = k*pi/M; t = L*tan(theta/2); # Variables theta and t.
    #f = exp(-t.A2)*(LA2+t.A2); f = [0; f]; # Function to be transformed.
    f = zeros(len(t)+1); f[0] = 0
    f[1:] = exp(-t**2)*(L**2+t**2)
    #f = insert(exp(-t**2)*(L**2+t**2),0,0)
    a = real(fft(fftshift(f)))/M2; # Coefficients of transform.
    a = flipud(a[1:N+1]); # Reorder coefficients.
    Z = (L+1.0j*z)/(L-1.0j*z); p = polyval(a,Z); # Polynomial evaluation.
    w = 2*p/(L-1.0j*z)**2+(1/sqrt(pi))/(L-1.0j*z); # Evaluate w(z).
    return w

# weideman24 by default    
#weideman24 = lambda x,y: cef(x,y,24)
weideman = lambda x,y,n: cef(x,y,n)

def hum1_wei(x,y,n=24):
    t = y-1.0j*x
    cerf=1/sqrt(pi)*t/(0.5+t**2)
    """
    z = x+1j*y
    cerf = 1j*z/sqrt(pi)/(z**2-0.5)
    """
    mask = abs(x)+y<15.0
    if any(mask):
        w24 = weideman(x[mask],y[mask],n)
        place(cerf,mask,w24)
    return cerf.real,cerf.imag

VARIABLES['CPF'] = hum1_wei
#VARIABLES['CPF'] = cpf
    
# ------------------ Hartmann-Tran Profile (HTP) ------------------------
def pcqsdhc(sg0,GamD,Gam0,Gam2,Shift0,Shift2,anuVC,eta,sg,Ylm=0.0):
    #-------------------------------------------------
    #      "pCqSDHC": partially-Correlated quadratic-Speed-Dependent Hard-Collision
    #      Subroutine to Compute the complex normalized spectral shape of an 
    #      isolated line by the pCqSDHC model
    #
    #      Reference:
    #      H. Tran, N.H. Ngo, J.-M. Hartmann.
    #      Efficient computation of some speed-dependent isolated line profiles.
    #      JQSRT, Volume 129, November 2013, Pages 199–203
    #      http://dx.doi.org/10.1016/j.jqsrt.2013.06.015
    #
    #      Input/Output Parameters of Routine (Arguments or Common)
    #      ---------------------------------
    #      T          : Temperature in Kelvin (Input).
    #      amM1       : Molar mass of the absorber in g/mol(Input).
    #      sg0        : Unperturbed line position in cm-1 (Input).
    #      GamD       : Doppler HWHM in cm-1 (Input)
    #      Gam0       : Speed-averaged line-width in cm-1 (Input).       
    #      Gam2       : Speed dependence of the line-width in cm-1 (Input).
    #      anuVC      : Velocity-changing frequency in cm-1 (Input).
    #      eta        : Correlation parameter, No unit (Input).
    #      Shift0     : Speed-averaged line-shift in cm-1 (Input).
    #      Shift2     : Speed dependence of the line-shift in cm-1 (Input)       
    #      sg         : Current WaveNumber of the Computation in cm-1 (Input).
    #      Ylm        : 1st order (Rosenkranz) line mixing coefficients in cm-1 (Input)
    #
    #      Output Quantities (through Common Statements)
    #      -----------------
    #      LS_pCqSDHC_R: Real part of the normalized spectral shape (cm)
    #      LS_pCqSDHC_I: Imaginary part of the normalized spectral shape (cm)
    #
    #      Called Routines: 'CPF'      (Complex Probability Function)
    #      ---------------  'CPF3'      (Complex Probability Function for the region 3)
    #
    #      Called By: Main Program
    #      ---------
    #
    #     Double Precision Version
    #
    #-------------------------------------------------
    
    # sg is the only vector argument which is passed to function
    
    if type(sg) not in set([array,ndarray,list,tuple]):
        sg = array([sg])
    
    number_of_points = len(sg)
    Aterm_GLOBAL = zeros(number_of_points,dtype=__ComplexType__)
    Bterm_GLOBAL = zeros(number_of_points,dtype=__ComplexType__)

    cte=sqrt(log(2.0e0))/GamD
    rpi=sqrt(pi)
    iz = __ComplexType__(0.0e0 + 1.0e0j)

    c0 = __ComplexType__(Gam0 + 1.0e0j*Shift0)
    c2 = __ComplexType__(Gam2 + 1.0e0j*Shift2)
    c0t = __ComplexType__((1.0e0 - eta) * (c0 - 1.5e0 * c2) + anuVC)
    c2t = __ComplexType__((1.0e0 - eta) * c2)

    # PART1
    if abs(c2t) == 0.0e0:
        Z1 = (iz*(sg0 - sg) + c0t) * cte
        xZ1 = -Z1.imag
        yZ1 = Z1.real
        WR1,WI1 = VARIABLES['CPF'](xZ1,yZ1)
        Aterm_GLOBAL = rpi*cte*__ComplexType__(WR1 + 1.0e0j*WI1)
        index_Z1 = abs(Z1) <= 4.0e3
        index_NOT_Z1 = ~index_Z1
        if any(index_Z1):
            Bterm_GLOBAL = rpi*cte*((1.0e0 - Z1**2)*__ComplexType__(WR1 + 1.0e0j*WI1) + Z1/rpi)
        if any(index_NOT_Z1):
            Bterm_GLOBAL = cte*(rpi*__ComplexType__(WR1 + 1.0e0j*WI1) + 0.5e0/Z1 - 0.75e0/(Z1**3))
    else:
        # PART2, PART3 AND PART4   (PART4 IS A MAIN PART)

        # X - vector, Y - scalar
        X = (iz * (sg0 - sg) + c0t) / c2t
        Y = __ComplexType__(1.0e0 / ((2.0e0*cte*c2t))**2)
        csqrtY = (Gam2 - iz*Shift2) / (2.0e0*cte*(1.0e0-eta) * (Gam2**2 + Shift2**2))

        index_PART2 = abs(X) <= 3.0e-8 * abs(Y)
        index_PART3 = (abs(Y) <= 1.0e-15 * abs(X)) & ~index_PART2
        index_PART4 = ~ (index_PART2 | index_PART3)
        
        # PART4
        if any(index_PART4):
            X_TMP = X[index_PART4]
            Z1 = sqrt(X_TMP + Y) - csqrtY
            Z2 = Z1 + __FloatType__(2.0e0) * csqrtY
            xZ1 = -Z1.imag
            yZ1 =  Z1.real
            xZ2 = -Z2.imag
            yZ2 =  Z2.real
            SZ1 = sqrt(xZ1**2 + yZ1**2)
            SZ2 = sqrt(xZ2**2 + yZ2**2)
            DSZ = abs(SZ1 - SZ2)
            SZmx = maximum(SZ1,SZ2)
            SZmn = minimum(SZ1,SZ2)
            length_PART4 = len(index_PART4)
            WR1_PART4 = zeros(length_PART4)
            WI1_PART4 = zeros(length_PART4)
            WR2_PART4 = zeros(length_PART4)
            WI2_PART4 = zeros(length_PART4)
            index_CPF3 = (DSZ <= 1.0e0) & (SZmx > 8.0e0) & (SZmn <= 8.0e0)
            index_CPF = ~index_CPF3 # can be removed
            if any(index_CPF3):
                WR1,WI1 = cpf3(xZ1[index_CPF3],yZ1[index_CPF3])
                WR2,WI2 = cpf3(xZ2[index_CPF3],yZ2[index_CPF3])
                WR1_PART4[index_CPF3] = WR1
                WI1_PART4[index_CPF3] = WI1
                WR2_PART4[index_CPF3] = WR2
                WI2_PART4[index_CPF3] = WI2
            if any(index_CPF):
                WR1,WI1 = VARIABLES['CPF'](xZ1[index_CPF],yZ1[index_CPF])
                WR2,WI2 = VARIABLES['CPF'](xZ2[index_CPF],yZ2[index_CPF])
                WR1_PART4[index_CPF] = WR1
                WI1_PART4[index_CPF] = WI1
                WR2_PART4[index_CPF] = WR2
                WI2_PART4[index_CPF] = WI2
            
            Aterm = rpi*cte*(__ComplexType__(WR1_PART4 + 1.0e0j*WI1_PART4) - __ComplexType__(WR2_PART4+1.0e0j*WI2_PART4))
            Bterm = (-1.0e0 +
                      rpi/(2.0e0*csqrtY)*(1.0e0 - Z1**2)*__ComplexType__(WR1_PART4 + 1.0e0j*WI1_PART4)-
                      rpi/(2.0e0*csqrtY)*(1.0e0 - Z2**2)*__ComplexType__(WR2_PART4 + 1.0e0j*WI2_PART4)) / c2t
            Aterm_GLOBAL[index_PART4] = Aterm
            Bterm_GLOBAL[index_PART4] = Bterm

        # PART2
        if any(index_PART2):
            X_TMP = X[index_PART2]
            Z1 = (iz*(sg0 - sg[index_PART2]) + c0t) * cte
            Z2 = sqrt(X_TMP + Y) + csqrtY
            xZ1 = -Z1.imag
            yZ1 = Z1.real
            xZ2 = -Z2.imag
            yZ2 = Z2.real
            WR1_PART2,WI1_PART2 = VARIABLES['CPF'](xZ1,yZ1)
            WR2_PART2,WI2_PART2 = VARIABLES['CPF'](xZ2,yZ2) 
            Aterm = rpi*cte*(__ComplexType__(WR1_PART2 + 1.0e0j*WI1_PART2) - __ComplexType__(WR2_PART2 + 1.0e0j*WI2_PART2))
            Bterm = (-1.0e0 +
                      rpi/(2.0e0*csqrtY)*(1.0e0 - Z1**2)*__ComplexType__(WR1_PART2 + 1.0e0j*WI1_PART2)-
                      rpi/(2.0e0*csqrtY)*(1.0e0 - Z2**2)*__ComplexType__(WR2_PART2 + 1.0e0j*WI2_PART2)) / c2t
            Aterm_GLOBAL[index_PART2] = Aterm
            Bterm_GLOBAL[index_PART2] = Bterm
            
        # PART3
        if any(index_PART3):
            X_TMP = X[index_PART3]
            xZ1 = -sqrt(X_TMP + Y).imag
            yZ1 = sqrt(X_TMP + Y).real
            WR1_PART3,WI1_PART3 =  VARIABLES['CPF'](xZ1,yZ1) 
            index_ABS = abs(sqrt(X_TMP)) <= 4.0e3
            index_NOT_ABS = ~index_ABS
            Aterm = zeros(len(index_PART3),dtype=__ComplexType__)
            Bterm = zeros(len(index_PART3),dtype=__ComplexType__)
            if any(index_ABS):
                xXb = -sqrt(X).imag
                yXb = sqrt(X).real
                WRb,WIb = VARIABLES['CPF'](xXb,yXb)
                Aterm[index_ABS] = (2.0e0*rpi/c2t)*(1.0e0/rpi - sqrt(X_TMP[index_ABS])*__ComplexType__(WRb + 1.0e0j*WIb))
                Bterm[index_ABS] = (1.0e0/c2t)*(-1.0e0+
                                  2.0e0*rpi*(1.0e0 - X_TMP[index_ABS]-2.0e0*Y)*(1.0e0/rpi-sqrt(X_TMP[index_ABS])*__ComplexType__(WRb + 1.0e0j*WIb))+
                                  2.0e0*rpi*sqrt(X_TMP[index_ABS] + Y)*__ComplexType__(WR1_PART3 + 1.0e0j*WI1_PART3))
            if any(index_NOT_ABS):
                Aterm[index_NOT_ABS] = (1.0e0/c2t)*(1.0e0/X_TMP[index_NOT_ABS] - 1.5e0/(X_TMP[index_NOT_ABS]**2))
                Bterm[index_NOT_ABS] = (1.0e0/c2t)*(-1.0e0 + (1.0e0 - X_TMP[index_NOT_ABS] - 2.0e0*Y)*
                                        (1.0e0/X_TMP[index_NOT_ABS] - 1.5e0/(X_TMP[index_NOT_ABS]**2))+
                                         2.0e0*rpi*sqrt(X_TMP[index_NOT_ABS] + Y)*__ComplexType__(WR1 + 1.0e0j*WI1))
            Aterm_GLOBAL[index_PART3] = Aterm
            Bterm_GLOBAL[index_PART3] = Bterm
            
    # common part
    # LINE MIXING PART NEEDS FURTHER TESTING!!!
    LS_pCqSDHC = (1.0e0/pi) * (Aterm_GLOBAL / (1.0e0 - (anuVC-eta*(c0-1.5e0*c2))*Aterm_GLOBAL + eta*c2*Bterm_GLOBAL))
    return LS_pCqSDHC.real + Ylm*LS_pCqSDHC.imag, LS_pCqSDHC.imag



# ------------------  CROSS-SECTIONS, XSECT.PY --------------------------------

# set interfaces for profiles

def PROFILE_HT(Nu,GammaD,Gamma0,Gamma2,Delta0,Delta2,NuVC,Eta,WnGrid,YRosen=0.0,Sw=1.0):
    """
    #-------------------------------------------------
    #      "pCqSDHC": partially-Correlated quadratic-Speed-Dependent "Hard-Collision"
    #      Subroutine to Compute the complex normalized spectral shape of an 
    #      isolated line by the pCqSDHC model
    #
    #      References:
    #
    #      1) N.H. Ngo, D. Lisak, H. Tran, J.-M. Hartmann.
    #         An isolated line-shape model to go beyond the Voigt profile in 
    #         spectroscopic databases and radiative transfer codes.
    #         JQSRT, Volume 129, November 2013, Pages 89–100
    #         http://dx.doi.org/10.1016/j.jqsrt.2013.05.034
    #
    #      2) H. Tran, N.H. Ngo, J.-M. Hartmann.
    #         Efficient computation of some speed-dependent isolated line profiles.
    #         JQSRT, Volume 129, November 2013, Pages 199–203
    #         http://dx.doi.org/10.1016/j.jqsrt.2013.06.015
    #
    #      3) H. Tran, N.H. Ngo, J.-M. Hartmann.
    #         Erratum to “Efficient computation of some speed-dependent isolated line profiles”.
    #         JQSRT, Volume 134, February 2014, Pages 104
    #         http://dx.doi.org/10.1016/j.jqsrt.2013.10.015
    #
    #      Input/Output Parameters of Routine (Arguments or Common)
    #      ---------------------------------
    #      Nu        : Unperturbed line position in cm-1 (Input).
    #      GammaD    : Doppler HWHM in cm-1 (Input)
    #      Gamma0    : Speed-averaged line-width in cm-1 (Input).       
    #      Gamma2    : Speed dependence of the line-width in cm-1 (Input).
    #      NuVC      : Velocity-changing frequency in cm-1 (Input).
    #      Eta       : Correlation parameter, No unit (Input).
    #      Delta0    : Speed-averaged line-shift in cm-1 (Input).
    #      Delta2    : Speed dependence of the line-shift in cm-1 (Input)       
    #      WnGrid    : Current WaveNumber of the Computation in cm-1 (Input).
    #      YRosen    : 1st order (Rosenkranz) line mixing coefficients in cm-1 (Input)
    #
    #      The function has two outputs:
    #      -----------------
    #      (1): Real part of the normalized spectral shape (cm)
    #      (2): Imaginary part of the normalized spectral shape (cm)
    #
    #      Called Routines: 'CPF'       (Complex Probability Function)
    #      ---------------  'CPF3'      (Complex Probability Function for the region 3)
    #
    #      Based on a double precision Fortran version
    #
    #-------------------------------------------------
    """
    return Sw*pcqsdhc(Nu,GammaD,Gamma0,Gamma2,Delta0,Delta2,NuVC,Eta,WnGrid,YRosen)[0]

PROFILE_HTP = PROFILE_HT # stub for backwards compatibility

def PROFILE_SDRAUTIAN(Nu,GammaD,Gamma0,Gamma2,Delta0,Delta2,NuVC,WnGrid,YRosen=0.0,Sw=1.0):
    """
    # Speed dependent Rautian profile based on HTP.
    # Input parameters:
    #      Nu        : Unperturbed line position in cm-1 (Input).
    #      GammaD    : Doppler HWHM in cm-1 (Input)
    #      Gamma0    : Speed-averaged line-width in cm-1 (Input).       
    #      Gamma2    : Speed dependence of the line-width in cm-1 (Input).
    #      NuVC      : Velocity-changing frequency in cm-1 (Input).
    #      Delta0    : Speed-averaged line-shift in cm-1 (Input).
    #      Delta2    : Speed dependence of the line-shift in cm-1 (Input)       
    #      WnGrid    : Current WaveNumber of the Computation in cm-1 (Input).
    #      YRosen    : 1st order (Rosenkranz) line mixing coefficients in cm-1 (Input)
    """
    return Sw*pcqsdhc(Nu,GammaD,Gamma0,Gamma2,Delta0,Delta2,NuVC,cZero,WnGrid,YRosen)[0]

def PROFILE_RAUTIAN(Nu,GammaD,Gamma0,Delta0,NuVC,WnGrid,Ylm=0.0,Sw=1.0):
    """
    # Rautian profile based on HTP.
    # Input parameters:
    #      Nu        : Unperturbed line position in cm-1 (Input).
    #      GammaD    : Doppler HWHM in cm-1 (Input)
    #      Gamma0    : Speed-averaged line-width in cm-1 (Input).       
    #      NuVC      : Velocity-changing frequency in cm-1 (Input).
    #      Delta0    : Speed-averaged line-shift in cm-1 (Input).
    #      WnGrid    : Current WaveNumber of the Computation in cm-1 (Input).
    #      YRosen    : 1st order (Rosenkranz) line mixing coefficients in cm-1 (Input)
    """
    return Sw*pcqsdhc(Nu,GammaD,Gamma0,cZero,Delta0,cZero,NuVC,cZero,WnGrid,YRosen)[0]

def PROFILE_SDVOIGT(Nu,GammaD,Gamma0,Gamma2,Delta0,Delta2,WnGrid,YRosen=0.0,Sw=1.0):
    """
    # Speed dependent Voigt profile based on HTP.
    # Input parameters:
    #      Nu        : Unperturbed line position in cm-1 (Input).
    #      GammaD    : Doppler HWHM in cm-1 (Input)
    #      Gamma0    : Speed-averaged line-width in cm-1 (Input).       
    #      Gamma2    : Speed dependence of the line-width in cm-1 (Input).
    #      Delta0    : Speed-averaged line-shift in cm-1 (Input).
    #      Delta2    : Speed dependence of the line-shift in cm-1 (Input)       
    #      WnGrid    : Current WaveNumber of the Computation in cm-1 (Input).
    #      YRosen    : 1st order (Rosenkranz) line mixing coefficients in cm-1 (Input)
    """
    if FLAG_DEBUG_PROFILE: 
        print('PROFILE_SDVOIGT>>>',Nu,GammaD,Gamma0,Gamma2,Delta0,Delta2,WnGrid,YRosen,Sw)
    return Sw*pcqsdhc(Nu,GammaD,Gamma0,Gamma2,Delta0,Delta2,cZero,cZero,WnGrid,YRosen)[0]
    
def PROFILE_VOIGT(Nu,GammaD,Gamma0,Delta0,WnGrid,YRosen=0.0,Sw=1.0):
    """
    # Voigt profile based on HTP.
    # Input parameters:
    #      Nu        : Unperturbed line position in cm-1 (Input).
    #      GammaD    : Doppler HWHM in cm-1 (Input)
    #      Gamma0    : Speed-averaged line-width in cm-1 (Input).       
    #      Delta0    : Speed-averaged line-shift in cm-1 (Input).
    #      WnGrid    : Current WaveNumber of the Computation in cm-1 (Input).
    #      YRosen    : 1st order (Rosenkranz) line mixing coefficients in cm-1 (Input)
    """
    #return PROFILE_HTP(Nu,GammaD,Gamma0,cZero,cZero,cZero,cZero,cZero,WnGrid,YRosen)[0]
    if FLAG_DEBUG_PROFILE: 
        print('PROFILE_VOIGT>>>',Nu,GammaD,Gamma0,Delta0,WnGrid,YRosen,Sw)
    return Sw*pcqsdhc(Nu,GammaD,Gamma0,cZero,Delta0,cZero,cZero,cZero,WnGrid,YRosen)[0]

def PROFILE_LORENTZ(Nu,Gamma0,Delta0,WnGrid,YRosen=0.0,Sw=1.0):
    """
    # Lorentz profile.
    # Input parameters:
    #      Nu        : Unperturbed line position in cm-1 (Input).
    #      Gamma0    : Speed-averaged line-width in cm-1 (Input).
    #      Delta0    : Speed-averaged line-shift in cm-1 (Input).
    #      WnGrid    : Current WaveNumber of the Computation in cm-1 (Input).
    #      YRosen    : 1st order (Rosenkranz) line mixing coefficients in cm-1 (Input)
    """
    # reduce the extra calculations in the case if YRosen is zero:
    if YRosen==0.0:
        return Sw*Gamma0/(pi*(Gamma0**2+(WnGrid+Delta0-Nu)**2))
    else:
        return Sw*(Gamma0+YRosen*(WnGrid+Delta0-Nu))/(pi*(Gamma0**2+(WnGrid+Delta0-Nu)**2))

def PROFILE_DOPPLER(Nu,GammaD,WnGrid,Sw=1.0):
    """
    # Doppler profile.
    # Input parameters:
    #      Nu        : Unperturbed line position in cm-1 (Input).
    #      GammaD    : Doppler HWHM in cm-1 (Input)
    #      WnGrid    : Current WaveNumber of the Computation in cm-1 (Input).
    """
    return Sw*cSqrtLn2divSqrtPi*exp(-cLn2*((WnGrid-Nu)/GammaD)**2)/GammaD

# Volume concentration of all gas molecules at the pressure p and temperature T
def volumeConcentration(p,T):
    return (p/9.869233e-7)/(cBolts*T) # CGS

# ------------------------------- PARAMETER DEPENDENCIES --------------------------------

# THE LOGIC OF THIS SECTION IS THAT NOTHING (OR AT LEAST MINUMUM) SHOULD BE HARD-CODED INTO THE GENERIC ABSCOEF ROUTINES
# TRYING TO AVOID THE OBJECT ORIENTED APPROACH HERE IN ORDER TO CORRESPOND TO THE OVERALL STYLE OF THE PACKAGE

def ladder(parname,species,envdep_presets,TRANS,flag_exception=False): # priority search for the parameters
    INFO = {}  
    if FLAG_DEBUG_LADDER: print('\nladder>>> ======================')
    if FLAG_DEBUG_LADDER: print('ladder>>> Calculating %s for %s broadener'%(parname,species))
    if FLAG_DEBUG_LADDER: print('ladder>>> Envdep presets: ',envdep_presets)
    for profile,envdep in envdep_presets:
        try:
            if FLAG_DEBUG_LADDER: print('\nladder>>> Trying: ',profile,envdep)
            INFO,ARGS = PRESSURE_INDUCED_ENVDEP[profile][parname][envdep]['getargs'](species,TRANS)
            parval_species = PRESSURE_INDUCED_ENVDEP[profile][parname][envdep]['depfunc'](**ARGS)
            if FLAG_DEBUG_LADDER: print('ladder>>> success!\n')
            return INFO,parval_species
        #except KeyError as e:
        except Exception as e:
            if flag_exception:
                raise e
            else:
                INFO['status'] = e.__class__.__name__+': '+str(e)
                if FLAG_DEBUG_LADDER: print('ladder>>>',e.__class__.__name__+': '+str(e),': ',parname,profile,envdep)
    if FLAG_DEBUG_LADDER: print('ladder>>> ')
    return INFO,0

def calculate_parameter_PI(parname,envdep_presets,TRANS,CALC_INFO):
    """
    Default function for calculating the pressure-induced parameters.
    Use this function only if the final lineshape parameter needs summation
    over the mixture/diluent components.
    """
    parval = 0
    Diluent = TRANS['Diluent']
    calc_info_flag = False
    if type(CALC_INFO) is dict:
        calc_info_flag = True
        CALC_INFO[parname] = {'mixture':{}}
    for species in Diluent:
        abun = Diluent[species]
        INFO,parval_species = ladder(parname,species,envdep_presets,TRANS)
        parval += abun*parval_species
        if calc_info_flag: 
            CALC_INFO[parname]['mixture'][species] = {'args':INFO,'value':parval_species}
    if calc_info_flag: 
        CALC_INFO[parname]['status'] = 'ok'
        CALC_INFO[parname]['value'] = parval
    return parval

def calculate_parameter_Nu(dummy,TRANS,CALC_INFO=None):
    nu = TRANS['nu']
    if type(CALC_INFO) is dict: 
        CALC_INFO['GammaD'] = {'value':nu,'mixture':{'generic':{'args':{}}}}
    return nu

def calculate_parameter_Sw(dummy,TRANS,CALC_INFO=None):
    molec_id = TRANS['molec_id']
    local_iso_id = TRANS['local_iso_id']
    nu = TRANS['nu']
    sw = TRANS['sw']
    T = TRANS['T']
    Tref = TRANS['T_ref']
    SigmaT = TRANS['SigmaT']
    SigmaTref = TRANS['SigmaT_ref']
    elower = TRANS['elower']
    Sw_calc = EnvironmentDependency_Intensity(sw,T,Tref,SigmaT,SigmaTref,elower,nu)
    if 'Abundances' in TRANS:
        Sw_calc *= TRANS['Abundances'][(molec_id,local_iso_id)]/abundance(molec_id,local_iso_id)
    if type(CALC_INFO) is dict:
        CALC_INFO['Sw'] = {
            'value':Sw_calc,
            'mixture':{
                'generic': {
                    'args': {
                        'SigmaT':{'value':SigmaT,'source':'<calc>'},
                        'SigmaT_ref':{'value':SigmaTref,'source':'<calc>'},
                        'elower':{'value':elower,'source':'elower'},
                        'nu':{'value':nu,'source':'nu'},
                    }
                }
            }
        }

    return Sw_calc
    #return {'value':TRANS['sw'],'info':{}} # SHOULD BE REDONE TO INCLUDE THE ABUNDANCES OF RADIATIVELY ACTIVE SPECIES
    
def calculate_parameter_GammaD(dummy,TRANS,CALC_INFO=None):
    """
    Calculate Doppler broadening HWHM for given environment.
    """
    Diluent = TRANS['Diluent']
    T = TRANS['T']
    p = TRANS['p']
    MoleculeNumberDB = TRANS['molec_id']
    IsoNumberDB = TRANS['local_iso_id']
    LineCenterDB = TRANS['nu']
    cMassMol = 1.66053873e-27
    molmass = molecularMass(MoleculeNumberDB,IsoNumberDB)
    fSqrtMass = sqrt(molmass)
    cc_ = 2.99792458e8
    cBolts_ = 1.3806503e-23
    
    #GammaD = (cSqrt2Ln2/cc_)*sqrt(cBolts_/cMassMol)*sqrt(T) * LineCenterDB/fSqrtMass
    # OR 
    m = molmass * cMassMol * 1000
    GammaD = sqrt(2*cBolts*T*log(2)/m/cc**2)*LineCenterDB
    
    if type(CALC_INFO) is dict: 
        CALC_INFO['GammaD'] = {
            'value':GammaD,
            'mixture':{
                'generic':{
                    'args':{
                        'T': {'value':T},
                        'p': {'value':p},
                        'molmass': {'value':molmass,'source':'<calc>'}
                    }
                }
            }
        }
    return GammaD
    
def calculate_parameter_Gamma0(envdep_presets,TRANS,CALC_INFO=None):
    """
    Calculate pressure-induced broadening HWHM for given Environment and TRANS.
    """
    parname = 'Gamma0'
    return calculate_parameter_PI(parname,envdep_presets,TRANS,CALC_INFO)

def calculate_parameter_Delta0(envdep_presets,TRANS,CALC_INFO=None):
    """
    Calculate pressure-induced line shift for given Environment and TRANS.
    """
    parname = 'Delta0'
    return calculate_parameter_PI(parname,envdep_presets,TRANS,CALC_INFO)

def calculate_parameter_Gamma2(envdep_presets,TRANS,CALC_INFO=None):
    """
    Calculate speed dependence of pressure-induced broadening HWHM for given Environment and TRANS.
    """
    parname = 'Gamma2'
    return calculate_parameter_PI(parname,envdep_presets,TRANS,CALC_INFO)

def calculate_parameter_Delta2(envdep_presets,TRANS,CALC_INFO=None):
    """
    Calculate speed dependence of pressure-induced line shift for given Environment and TRANS.
    """
    parname = 'Delta2'
    return calculate_parameter_PI(parname,envdep_presets,TRANS,CALC_INFO)

def calculate_parameter_Eta(envdep_presets,TRANS,CALC_INFO=None):
    """
    Calculate correlation parameter for given Environment and TRANS.
    """
    Diluent = TRANS['Diluent']
    if type(CALC_INFO) is not dict:
        return 0
    CALC_INFO['Eta'] = {'mixture':{}}
    Eta = 0
    Gamma2 = CALC_INFO['Gamma2']['value']
    Delta2 = CALC_INFO['Delta2']['value']
    Eta_denom = Gamma2-1j*Delta2
    for species in Diluent:
        abun = Diluent[species]
        EtaDB = TRANS.get('eta_HT_%s'%species,0)
        Gamma2T = CALC_INFO['Gamma2']['mixture'][species]['value']
        Delta2T = CALC_INFO['Delta2']['mixture'][species]['value']
        Eta_species = EtaDB*abun*(Gamma2T-1j*Delta2T)
        if Eta_denom!=0: Eta_species/=Eta_denom
        CALC_INFO['Eta']['mixture'][species] = {
            'value':Eta_species,
            'args':{
                'Gamma0':{'value':Gamma0,'source':'<calc>'},
                'Delta0':{'value':Delta0,'source':'<calc>'},
            }
        }
        Eta += Eta_species
    return Eta

def calculate_parameter_NuVC(envdep_presets,TRANS,CALC_INFO=None):
    """
    Calculate velocity collision frequency for given Environment and TRANS.
    """
    Diluent = TRANS['Diluent']
    if type(CALC_INFO) is not dict:
        return 0
    CALC_INFO['NuVC'] = {'mixture':{}}
    Gamma0 = CALC_INFO['Gamma0']['value']
    Delta0 = CALC_INFO['Delta0']['value']
    Eta = CALC_INFO['Eta']['value']
    NuVC = Eta*(Gamma0-1j*Shift0)
    p = TRANS['p']
    T = TRANS['T']
    Tref = CALC_INFO['Tref']['value']
    for species in Diluent:

        abun = Diluent[species]

        # 1st NuVC component: weighted sum of NuVC_i
        NuVCDB = TRANS.get('nu_HT_%s'%species,0)
        KappaDB = TRANS.get('kappa_HT_%s'%species,0)
        NuVC += NuVCDB*(Tref/T)**KappaDB*p

        # 2nd NuVC component (with negative sign)
        Gamma0T = CALC_INFO['Gamma0']['mixture'][species]['value']
        Delta0T = CALC_INFO['Delta0']['mixture'][species]['value']
        EtaDB = TRANS.get('eta_HT_%s'%species,0)
        NuVC -= EtaDB*abun*(Gamma0T-1j*Delta0T)

        CALC_INFO['NuVC']['mixture'][species] = {
            'value':NuVC_species,
            'args':{
                'Gamma0':{'value':Gamma0,'source':'<calc>'},
                'Delta0':{'value':Delta0,'source':'<calc>'},
            }
        }

        NuVC += NuVC_species

    return NuVC
    
def calculate_parameter_YRosen(envdep_presets,TRANS,CALC_INFO=None):
    """
    Calculate pressure-induced 1st order line mixing parameter for given Environment and TRANS.
    """
    parname = 'YRosen'
    return calculate_parameter_PI(parname,envdep_presets,TRANS,CALC_INFO)
    
def calculateProfileParameters(envdep_presets,parameters,TRANS,CALC_INFO=None,exclude=None):
    """
    Get the Line context on the input, and return the dictionary with the "abstract" parameters.
    """
    PARAMS = {}
    for pname,pfunc in parameters:
        if exclude and pname in exclude:
            pval_default = 0
            PARAMS[pname] = pval_default # don't calculate parameter if it is present in exclude set
            if type(CALC_INFO) is dict:
                CALC_INFO[pname] = {
                    'value': pval_default,
                    'status': 'excluded'
                }
        else:
            PARAMS[pname] = pfunc(envdep_presets,TRANS=TRANS,CALC_INFO=CALC_INFO)
    return PARAMS
    
def calculateProfileParametersDoppler(TRANS,CALC_INFO=None,exclude=None):
    """
    Get values for abstract profile parameters for Doppler profile.
    """
    envdep_presets = [('Doppler','default')]
    parameters = [
        ('Nu', calculate_parameter_Nu),
        ('Sw', calculate_parameter_Sw),
        ('GammaD', calculate_parameter_GammaD),
    ]
    return calculateProfileParameters(envdep_presets,parameters,CALC_INFO=CALC_INFO,TRANS=TRANS,exclude=exclude)
    
def calculateProfileParametersLorentz(TRANS,CALC_INFO=None,exclude=None):
    """
    Get values for abstract profile parameters for Lorentz profile.
    """
    envdep_presets = [('Lorentz','default')]
    parameters = [
        ('Nu', calculate_parameter_Nu),
        ('Sw', calculate_parameter_Sw),
        ('Gamma0', calculate_parameter_Gamma0),
        ('Delta0', calculate_parameter_Delta0),
        ('YRosen', calculate_parameter_YRosen),
    ]
    return calculateProfileParameters(envdep_presets,parameters,CALC_INFO=CALC_INFO,TRANS=TRANS,exclude=exclude)
    
def calculateProfileParametersVoigt(TRANS,CALC_INFO=None,exclude=None):
    """
    Get values for abstract profile parameters for Voigt profile.
    """
    envdep_presets = [('Voigt','default')]
    parameters = [
        ('Nu', calculate_parameter_Nu),
        ('Sw', calculate_parameter_Sw),
        ('GammaD', calculate_parameter_GammaD),
        ('Gamma0', calculate_parameter_Gamma0),
        ('Delta0', calculate_parameter_Delta0),
        ('YRosen', calculate_parameter_YRosen),
    ]
    return calculateProfileParameters(envdep_presets,parameters,CALC_INFO=CALC_INFO,TRANS=TRANS,exclude=exclude)

def calculateProfileParametersSDVoigt(TRANS,CALC_INFO=None,exclude=None):
    """
    Get values for abstract profile parameters for SDVoigt profile.
    """
    envdep_presets = [
        ('SDVoigt','default'),
        ('SDVoigt','dimensionless'),
        ('Voigt','default')
        ]
    parameters = [
        ('Nu', calculate_parameter_Nu),
        ('Sw', calculate_parameter_Sw),
        ('GammaD', calculate_parameter_GammaD),
        ('Gamma0', calculate_parameter_Gamma0),
        ('Delta0', calculate_parameter_Delta0),
        ('Gamma2', calculate_parameter_Gamma2),
        ('Delta2', calculate_parameter_Delta2),
        ('YRosen', calculate_parameter_YRosen),        
    ]
    return calculateProfileParameters(envdep_presets,parameters,CALC_INFO=CALC_INFO,TRANS=TRANS,exclude=exclude)

def calculateProfileParametersHT(TRANS,CALC_INFO=None,exclude=None):
    """
    Get values for abstract profile parameters for HT profile.
    """
    envdep_presets = [
        ('HT','multitemp'),
        ('HT','default'),
        ('Voigt','default')
        ]
    parameters = [
        ('Nu', calculate_parameter_Nu),
        ('Sw', calculate_parameter_Sw),
        ('GammaD', calculate_parameter_GammaD),
        ('Gamma0', calculate_parameter_Gamma0),
        ('Delta0', calculate_parameter_Delta0),
        ('Gamma2', calculate_parameter_Gamma2),
        ('Delta2', calculate_parameter_Delta2),
        ('Eta', calculate_parameter_Eta),
        ('NuVC', calculate_parameter_NuVC),
        ('YRosen', calculate_parameter_YRosen),
    ]
    return calculateProfileParameters(envdep_presets,parameters,CALC_INFO=CALC_INFO,TRANS=TRANS,exclude=exclude)
    
def calculateProfileParametersFullPriority(TRANS,CALC_INFO=None,exclude=None):
    """
    Get the Line context on the input, and return the dictionary with the "abstract" parameters.
    """
    envdep_presets = [
        ('HT','multitemp'),
        ('HT','default'),
        ('SDVoigt','default'),
        ('Voigt','default')
        ]
    parameters = [
        ('Nu', calculate_parameter_Nu),
        ('Sw', calculate_parameter_Sw),
        ('GammaD', calculate_parameter_GammaD),
        ('Gamma0', calculate_parameter_Gamma0),
        ('Delta0', calculate_parameter_Delta0),
        ('Gamma2', calculate_parameter_Gamma2),
        ('Delta2', calculate_parameter_Delta2),
        ('Eta', calculate_parameter_Eta),
        ('NuVC', calculate_parameter_NuVC),
        ('YRosen', calculate_parameter_YRosen),
    ]
    return calculateProfileParameters(envdep_presets,parameters,CALC_INFO=CALC_INFO,TRANS=TRANS,exclude=exclude)

VARIABLES['abscoef_debug'] = True

# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
# ENVIRONMENT DEPENDENCES (GENERIC)

def environGetArguments(abstract_parnames,lookup_cases,
        aux_args,TRANS):
    """
    Get the environment-dependent parameter names, along with auxiliary arguments 
    to use in the environment dependence function.
    INPUT:
        abstract_args: tuple containing the names of the abstract parameters 
                       for the environment dependence
        lookup_cases: list of dicts, each of which corresponds to the
                       mapping of abstract parname to particular value in the database.
                       The search goes accordingly to the 
                       order of the cases in the list, of none found then
                       the Exception is raised.
    OUTPUT:
        INFO: database names for some of the "abstract" arguments
                  for the environment dependence
        ARGS: values for the "abstract" arguments
    """    
    params_not_found = []
    
    for CASE in lookup_cases:
        
        casename = CASE['__case__']
    
        ARGS = aux_args
        INFO = {}
        
        flag = False
        for argname_abstract in set(CASE.keys())-set(['__case__']):
           
            argname_database = CASE[argname_abstract]['name']
            
            try:
                if argname_database not in TRANS or TRANS[argname_database] is np.ma.core.MaskedConstant:
                    if 'default' in CASE[argname_abstract]:
                        source = '<default>'
                        value = CASE[argname_abstract]['default']
                    else:
                        raise KeyError                    
                else:
                    source = argname_database
                    value = TRANS[argname_database]
                ARGS[argname_abstract] = value                   
                if VARIABLES['abscoef_debug']: 
                    INFO[argname_abstract]={'case':casename}
                    INFO[argname_abstract]['source'] = source
                    INFO[argname_abstract]['value'] = value
            except KeyError:                
                params_not_found.append(argname_database)
                flag = True
                
        if not flag: 
            return INFO,ARGS
        
    raise Exception('not found in DB: %s'%params_not_found)

# STANDARD ENVIRONMENT DEPENDENCE FUNCTIONS    
    
def environDependenceFn_PowerLaw(Par_ref,TempRatioPower,T,T_ref,p,p_ref):
    """
    Standard single power law environment dependence.
    """
    return Par_ref * ( T_ref/T )**TempRatioPower * p/p_ref
    
def environDependenceFn_LinearLaw(Par_ref,Coef,T,T_ref,p,p_ref):
    """
    Standard linear law environment dependence.    
    """ 
    return ( Par_ref + Coef*(T-T_ref) ) * p/p_ref
    
# ENVIRONMENT DEPENDENCES FOR LORENTZ PROFILE    
    
# Gamma0 =>
    
def environGetArguments_Lorentz_Gamma0_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Gamma0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'Lorentz 1',
            'Par_ref':{
                'name': 'gamma_%s'%broadener,                
            },
            'TempRatioPower':{
                'name': 'n_%s'%broadener,
            },
        },
        {
            '__case__': 'Lorentz 2',
            'Par_ref':{
                'name': 'gamma_%s'%broadener,
            },
            'TempRatioPower':{
                'name': 'n_air',
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
    
# Delta0 =>    
    
def environGetArguments_Lorentz_Delta0_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Delta0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','Coef']
    lookup_cases = [
        {
            '__case__': 'Lorentz 1',
            'Par_ref':{
                'name': 'delta_%s'%broadener,
                'default': 0,
            },
            'Coef':{
                'name': 'deltap_%s'%broadener,
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS

# YRosen =>

def environGetArguments_Lorentz_YRosen_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for YRosen.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'Lorentz 1',
            'Par_ref':{
                'name': 'y_%s'%broadener,  
                'default': 0,
            },
            'TempRatioPower':{
                'name': 'n_y_%s'%broadener,
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
        
# ENVIRONMENT DEPENDENCES FOR VOIGT PROFILE
    
# ... Gamma0, Delta0, and YRosen are the same as for Lorentz profile

# ENVIRONMENT DEPENDENCES FOR SDVOIGT PROFILE

# Gamma0 =>
    
def environGetArguments_SDVoigt_Gamma0_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Gamma0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'SDVoigt 1',
            'Par_ref':{
                'name': 'gamma_SDV_0_%s_%d'%(broadener,T_ref),
            },
            'TempRatioPower':{
                'name': 'n_SDV_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
    
# Delta0 =>
        
def environGetArguments_SDVoigt_Delta0_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Delta0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','Coef']
    lookup_cases = [
        {
            '__case__': 'SDVoigt 1',
            'Par_ref':{
                'name': 'delta_SDV_0_%s_%d'%(broadener,T_ref),
            },
            'Coef':{
                'name': 'deltap_SDV_%s_%d'%(broadener,T_ref),
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS

# Gamma2 =>
    
def environGetArguments_SDVoigt_Gamma2_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Gamma2.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'SDVoigt 1',
            'Par_ref':{
                'name': 'gamma_SDV_2_%s_%d'%(broadener,T_ref),
            },
            'TempRatioPower':{
                'name': 'n_gamma_SDV_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS

def environGetArguments_SDVoigt_Gamma2_dimensionless(broadener,TRANS): # avoid this;
    """
    Argument selector for the environment dependence function for Gamma2.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['gamma0','sd','n']
    lookup_cases = [
        {
            '__case__': 'SDVoigt 1',
            'gamma0':{
                'name': 'gamma_%s'%broadener,
            },
            'sd':{
                'name': 'SD_%s'%broadener,
            },
            'n':{
                'name': 'n_SD_%s'%broadener,
                'default': 0,
            },
        },
        {
            '__case__': 'SDVoigt 2',
            'gamma0':{
                'name': 'gamma_%s'%broadener,
            },
            'sd':{
                'name': 'SD_%s'%broadener,
            },
            'n':{
                'name': 'n_SD_air',
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
        
# Delta2 =>    
    
def environGetArguments_SDVoigt_Delta2_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Delta0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','Coef']
    lookup_cases = [
        {
            '__case__': 'SDVoigt 1',
            'Par_ref':{
                'name': 'delta_SDV_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
            'Coef':{
                'name': 'deltap_SDV_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS

# YRosen =>

def environGetArguments_SDVoigt_YRosen_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for YRosen.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'Lorentz 1',
            'Par_ref':{
                'name': 'Y_SDV_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
            'TempRatioPower':{
                'name': 'n_Y_SDV_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
    
# ENVIRONMENT DEPENDENCES FOR HT PROFILE

# Gamma0 =>

def get_T_ref_for_HT_multitemp(T):
    """
    Get the actual reference temperature for the multitemp HT preset.
    """
    TRanges = [(0,100),(100,200),(200,400),(400,float('inf'))]
    Trefs = [50.,150.,296.,700.]
    for TRange,TrefHT in zip(TRanges,Trefs):
        if T >= TRange[0] and T < TRange[1]:
            break
    return TrefHT
    
def environGetArguments_HT_Gamma0_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Gamma0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'gamma_HT_0_%s_%d'%(broadener,T_ref),
            },
            'TempRatioPower':{
                'name': 'n_HT_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
    
def environGetArguments_HT_Gamma0_multitemp(broadener,TRANS): # CUSTOM MULTITEMP PRESET
    """
    Argument selector for the environment dependence function for Gamma0.
    Search parameters for non-standard "Multi-temperature" environment dependence
    used in HITRAN for H2 molecule, as described in Wcislo et al., JQSRT 2016.
    """
    T_ref = get_T_ref_for_HT_multitemp(TRANS['T']); p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'gamma_HT_0_%s_%d'%(broadener,T_ref),
            },
            'TempRatioPower':{
                'name': 'n_HT_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
    
# Delta0 =>
        
def environGetArguments_HT_Delta0_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Delta0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','Coef']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'delta_HT_0_%s_%d'%(broadener,T_ref),
            },
            'Coef':{
                'name': 'deltap_HT_%s_%d'%(broadener,T_ref),
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)
    return CALC_INFO,ARGS

def environGetArguments_HT_Delta0_multitemp(broadener,TRANS): # CUSTOM MULTITEMP PRESET
    """
    Argument selector for the environment dependence function for Delta0.
    Search parameters for non-standard "Multi-temperature" environment dependence
    used in HITRAN for H2 molecule, as described in Wcislo et al., JQSRT 2016.
    """
    T_ref = get_T_ref_for_HT_multitemp(TRANS['T']); p_ref = 1.0
    abstract_args = ['Par_ref','Coef']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'delta_HT_0_%s_%d'%(broadener,T_ref),
            },
            'Coef':{
                'name': 'deltap_HT_%s_%d'%(broadener,T_ref),
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)
    return CALC_INFO,ARGS
    
# Gamma2 =>
    
def environGetArguments_HT_Gamma2_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Gamma2.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'gamma_HT_2_%s_%d'%(broadener,T_ref),
            },
            'TempRatioPower':{
                'name': 'n_gamma_HT_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)
    return CALC_INFO,ARGS

def environGetArguments_HT_Gamma2_multitemp(broadener,TRANS): # CUSTOM MULTITEMP PRESET
    """
    Argument selector for the environment dependence function for Gamma2.
    Search parameters for non-standard "Multi-temperature" environment dependence
    used in HITRAN for H2 molecule, as described in Wcislo et al., JQSRT 2016.
    """
    T_ref = get_T_ref_for_HT_multitemp(TRANS['T']); p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'gamma_HT_2_%s_%d'%(broadener,T_ref),
            },
            'TempRatioPower':{
                'name': 'n_gamma_HT_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)
    return CALC_INFO,ARGS
    
# Delta2 =>    
    
def environGetArguments_HT_Delta2_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for Delta0.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','Coef']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'delta_HT_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
            'Coef':{
                'name': 'deltap_HT_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS

def environGetArguments_HT_Delta2_multitemp(broadener,TRANS): # CUSTOM MULTITEMP PRESET
    """
    Argument selector for the environment dependence function for Delta0.
    Search parameters for non-standard "Multi-temperature" environment dependence
    used in HITRAN for H2 molecule, as described in Wcislo et al., JQSRT 2016.
    """
    T_ref = get_T_ref_for_HT_multitemp(TRANS['T']); p_ref = 1.0
    abstract_args = ['Par_ref','Coef']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'delta_HT_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
            'Coef':{
                'name': 'deltap_HT_2_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
    
# NuVC =>    
    
def environGetArguments_HT_NuVC_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for NuVC.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'HT 1',
            'Par_ref':{
                'name': 'nu_HT_%s'%broadener,
            },
            'TempRatioPower':{
                'name': 'kappa_HT_%s'%broadener,
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)
    return CALC_INFO,ARGS
    
# YRosen =>

def environGetArguments_HT_YRosen_default(broadener,TRANS):
    """
    Argument selector for the environment dependence function for YRosen.
    """
    T_ref = 296.0; p_ref = 1.0
    abstract_args = ['Par_ref','TempRatioPower']
    lookup_cases = [
        {
            '__case__': 'Lorentz 1',
            'Par_ref':{
                'name': 'Y_HT_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
            'TempRatioPower':{
                'name': 'n_Y_HT_%s_%d'%(broadener,T_ref),
                'default': 0,
            },
        },
    ]
    aux_args = {'T':TRANS['T'],'T_ref':T_ref,'p':TRANS['p'],'p_ref':p_ref}
    CALC_INFO,ARGS = environGetArguments(abstract_args,
        lookup_cases,aux_args,TRANS)        
    return CALC_INFO,ARGS
    
# //////////////////////////////////////////////////////////////////////
# REGISTRY FOR ENVIRONMENT DEPENDENCES FOR PRESSURE-INDUCED PARAMETERS 
PRESSURE_INDUCED_ENVDEP = {

    'Lorentz': {
        'Gamma0': { # name of the abstract parameter
            'default': { # name of the preset
                'getargs': environGetArguments_Lorentz_Gamma0_default, # convert abstract environment dependence arguments to the real database parameters
                'depfunc': environDependenceFn_PowerLaw, # calculate the environment-dependent parameter            
            },
        },
        'Delta0': {    
            'default': {
                'getargs': environGetArguments_Lorentz_Delta0_default,
                'depfunc': environDependenceFn_LinearLaw, 
            },
        },
        'YRosen': {    
            'default': {
                'getargs': environGetArguments_Lorentz_YRosen_default,
                'depfunc': environDependenceFn_LinearLaw, 
            },
        },
    },
    
    'Voigt': {
        'Gamma0': {
            'default': {
                'getargs': environGetArguments_Lorentz_Gamma0_default,
                'depfunc': environDependenceFn_PowerLaw,
            },
        },
        'Delta0': {    
            'default': {
                'getargs': environGetArguments_Lorentz_Delta0_default,
                'depfunc': environDependenceFn_LinearLaw, 
            },
        },
        'YRosen': {    
            'default': {
                'getargs': environGetArguments_Lorentz_YRosen_default,
                'depfunc': environDependenceFn_PowerLaw, 
            },
        },
    },

    'SDVoigt': {
        'Gamma0': {
            'default': {
                'getargs': environGetArguments_SDVoigt_Gamma0_default,
                'depfunc': environDependenceFn_PowerLaw,
            },
        },
        'Delta0': {    
            'default': {
                'getargs': environGetArguments_SDVoigt_Delta0_default,
                'depfunc': environDependenceFn_LinearLaw, 
            },
        },
        'Gamma2': {
            'default': {
                'getargs': environGetArguments_SDVoigt_Gamma2_default,
                'depfunc': environDependenceFn_PowerLaw,
            },
            'dimensionless': {
                'getargs': environGetArguments_SDVoigt_Gamma2_dimensionless,
                'depfunc': lambda gamma0,sd,n,T,T_ref,p,p_ref: environDependenceFn_PowerLaw(gamma0*sd,n,T,T_ref,p,p_ref),
            },
        },
        'Delta2': {    
            'default': {
                'getargs': environGetArguments_SDVoigt_Delta2_default,
                'depfunc': environDependenceFn_LinearLaw, 
            },
        },
        'YRosen': {    
            'default': {
                'getargs': environGetArguments_SDVoigt_YRosen_default,
                'depfunc': environDependenceFn_PowerLaw, 
            },
        },
    },
    
    'HT': {
        'Gamma0': {
            'default': {
                'getargs': environGetArguments_HT_Gamma0_default,
                'depfunc': environDependenceFn_PowerLaw,
            },
            'multitemp': {
                'getargs': environGetArguments_HT_Gamma0_multitemp,
                'depfunc': environDependenceFn_PowerLaw,
            },
        },
        'Delta0': {    
            'default': {
                'getargs': environGetArguments_HT_Delta0_default,
                'depfunc': environDependenceFn_LinearLaw, 
            },
            'multitemp': {
                'getargs': environGetArguments_HT_Delta0_multitemp,
                'depfunc': environDependenceFn_LinearLaw, 
            },
        },
        'Gamma2': {
            'default': {
                'getargs': environGetArguments_HT_Gamma2_default,
                'depfunc': environDependenceFn_PowerLaw,
            },
            'multitemp': {
                'getargs': environGetArguments_HT_Gamma2_multitemp,
                'depfunc': environDependenceFn_PowerLaw,
            },
        },
        'Delta2': {
            'default': {
                'getargs': environGetArguments_HT_Delta2_default,
                'depfunc': environDependenceFn_LinearLaw, 
            },
            'multitemp': {
                'getargs': environGetArguments_HT_Delta2_multitemp,
                'depfunc': environDependenceFn_LinearLaw, 
            },
        },
        'NuVC': {    
            'default': {
                'getargs': environGetArguments_HT_NuVC_default,
                'depfunc': environDependenceFn_PowerLaw, 
            },
        },
        'YRosen': {    
            'default': {
                'getargs': environGetArguments_HT_YRosen_default,
                'depfunc': environDependenceFn_PowerLaw, 
            },
        },
    },
    
}

# ////////////////////////////////////////////

# OLD TEMPERATURE AND PRESSURE DEPENDENCES MOSTLY FOR BACKWARDS COMPATIBILITY

# temperature dependence for intensities (HITRAN)
def EnvironmentDependency_Intensity(LineIntensityRef,T,Tref,SigmaT,SigmaTref,
                                    LowerStateEnergy,LineCenter):
    const = __FloatType__(1.4388028496642257)
    ch = exp(-const*LowerStateEnergy/T)*(1-exp(-const*LineCenter/T))
    zn = exp(-const*LowerStateEnergy/Tref)*(1-exp(-const*LineCenter/Tref))
    LineIntensity = LineIntensityRef*SigmaTref/SigmaT*ch/zn
    return LineIntensity

# environmental dependence for GammaD (HTP, Voigt)
def EnvironmentDependency_GammaD(GammaD_ref,T,Tref):
    # Doppler parameters do not depend on pressure!
    return GammaD_ref*sqrt(T/Tref)

# environmental dependence for Gamma0 (HTP, Voigt)
def EnvironmentDependency_Gamma0(Gamma0_ref,T,Tref,p,pref,TempRatioPower):
    return Gamma0_ref*p/pref*(Tref/T)**TempRatioPower

# environmental dependence for Gamma2 (HTP)
def EnvironmentDependency_Gamma2(Gamma2_ref,T,Tref,p,pref,TempRatioPower):
    return Gamma2_ref*p/pref*(Tref/T)**TempRatioPower
    #return Gamma2_ref*p/pref

# environmental dependence for Delta0 (HTP)
def EnvironmentDependency_Delta0(Delta0_ref,Deltap,T,Tref,p,pref):
    return (Delta0_ref + Deltap*(T-Tref))*p/pref

# environmental dependence for Delta2 (HTP)
def EnvironmentDependency_Delta2(Delta2_ref,T,Tref,p,pref,TempRatioPower):
    return Delta2_ref*p/pref*(Tref/T)**TempRatioPower
    #return Delta2_ref*p/pref

# environmental dependence for nuVC (HTP)
def EnvironmentDependency_nuVC(nuVC_ref,Kappa,T,Tref,p,pref):
    return nuVC_ref*(Tref/T)**Kappa*p/pref

# environmental dependence for nuVC (HTP)
def EnvironmentDependency_Eta(EtaDB,Gamma0,Shift0,Diluent,C):  # C=>CONTEXT
    Eta_Numer = 0
    for species in Diluent:
        abun = Diluent[species]
        Gamma0T = C['Gamma0T_%s'%species]
        Shift0T = C['Shift0T_%s'%species]
        Eta_Numer += EtaDB*abun*(Gamma0T+1j*Shift0T)
    Eta = Eta_Numer / (Gamma0 + 1j*Shift0)    
    return Eta
    
# ------------------------------- /PARAMETER DEPENDENCIES --------------------------------

# ------------------------------- BINGINGS --------------------------------

# default parameter bindings
DefaultParameterBindings = {}

# default temperature dependencies
DefaultEnvironmentDependencyBindings = {}

# ------------------------------- /BINGINGS --------------------------------

# default values for intensity threshold
DefaultIntensityThreshold = 0. # cm*molec

# default value for omega wing in halfwidths (from center)
DefaultOmegaWingHW = 50. # cm-1    HOTW default


# check and argument for being a tuple or list
# this is connected with a "bug" that in Python
# (val) is not a tuple, but (val,) is a tuple
def listOfTuples(a):
    if type(a) not in set([list,tuple]):
        a = [a]
    return a


# determine default parameters from those which are passed to absorptionCoefficient_...
def getDefaultValuesForXsect(Components,SourceTables,Environment,OmegaRange,
                             OmegaStep,OmegaWing,IntensityThreshold,Format):
    if SourceTables[0] == None:
        SourceTables = ['__BUFFER__',]
    if Environment == None:
        Environment = {'T':296., 'p':1.}
    if Components == [None]:
        CompDict = {}
        for TableName in SourceTables:
            # check table existance
            if TableName not in LOCAL_TABLE_CACHE.keys():
                raise Exception('%s: no such table. Check tableList() for more info.' % TableName)
            mol_ids = LOCAL_TABLE_CACHE[TableName]['data']['molec_id']
            iso_ids = LOCAL_TABLE_CACHE[TableName]['data']['local_iso_id']
            if len(mol_ids) != len(iso_ids):
                raise Exception('Lengths if mol_ids and iso_ids differ!')
            MI_zip = zip(mol_ids,iso_ids)
            MI_zip = set(MI_zip)
            for mol_id,iso_id in MI_zip:
                CompDict[(mol_id,iso_id)] = None
        Components = CompDict.keys()
    if OmegaRange == None:
        omega_min = float('inf')
        omega_max = float('-inf')
        for TableName in SourceTables:
            nu = LOCAL_TABLE_CACHE[TableName]['data']['nu']
            numin = min(nu)
            numax = max(nu)
            if omega_min > numin:
                omega_min = numin
            if omega_max < numax:
                omega_max = numax
        OmegaRange = (omega_min,omega_max)
    if OmegaStep == None:
        OmegaStep = 0.01 # cm-1
    if OmegaWing == None:
        OmegaWing = 0.0 # cm-1
    if not Format:
        Format = '%.12f %e'
    return Components,SourceTables,Environment,OmegaRange,\
           OmegaStep,OmegaWing,IntensityThreshold,Format

ABSCOEF_DOCSTRING_TEMPLATE = \
    """
    INPUT PARAMETERS: 
        Components:  list of tuples [(M,I,D)], where
                        M - HITRAN molecule number,
                        I - HITRAN isotopologue number,
                        D - relative abundance (optional)
        SourceTables:  list of tables from which to calculate cross-section   (optional)
        partitionFunction:  pointer to partition function (default is PYTIPS) (optional)
        Environment:  dictionary containing thermodynamic parameters.
                        'p' - pressure in atmospheres,
                        'T' - temperature in Kelvin
                        Default={{'p':1.,'T':296.}}
        WavenumberRange:  wavenumber range to consider.
        WavenumberStep:   wavenumber step to consider. 
        WavenumberWing:   absolute wing for calculating a lineshape (in cm-1) 
        WavenumberWingHW:  relative wing for calculating a lineshape (in halfwidths)
        IntensityThreshold:  threshold for intensities
        Diluent:  specifies broadening mixture composition, e.g. {{'air':0.7,'self':0.3}}
        HITRAN_units:  use cm2/molecule (True) or cm-1 (False) for absorption coefficient
        File:   write output to file (if specified)
        Format:  c-format of file output (accounts for significant digits in WavenumberStep)
        LineMixingRosen: include 1st order line mixing to calculation
    OUTPUT PARAMETERS: 
        Wavenum: wavenumber grid with respect to parameters WavenumberRange and WavenumberStep
        Xsect: absorption coefficient calculated on the grid
    ---
    DESCRIPTION:
        Calculate absorption coefficient using {profile}.
        Absorption coefficient is calculated at arbitrary temperature and pressure.
        User can vary a wide range of parameters to control a process of calculation.
        The choise of these parameters depends on properties of a particular linelist.
        Default values are a sort of guess which gives a decent precision (on average) 
        for a reasonable amount of cpu time. To increase calculation accuracy,
        user should use a trial and error method.
    ---
    EXAMPLE OF USAGE:
        {usage_example}
    ---
    """     
           
def absorptionCoefficient_Generic(Components=None,SourceTables=None,partitionFunction=PYTIPS,
                                  Environment=None,OmegaRange=None,OmegaStep=None,OmegaWing=None,
                                  IntensityThreshold=DefaultIntensityThreshold,
                                  OmegaWingHW=DefaultOmegaWingHW,
                                  GammaL='gamma_air', HITRAN_units=True, LineShift=True,
                                  File=None, Format=None, OmegaGrid=None,
                                  WavenumberRange=None,WavenumberStep=None,WavenumberWing=None,
                                  WavenumberWingHW=None,WavenumberGrid=None,
                                  Diluent={},LineMixingRosen=False,
                                  profile=None,calcpars=None,exclude=set(),
                                  DEBUG=None):
                                                              
    # Throw exception if profile or calcpars are empty.
    if profile is None: raise Exception('user must provide the line profile function')
    if calcpars is None: raise Exception('user must provide the function for calculating profile parameters')
        
    if DEBUG is not None: 
        VARIABLES['abscoef_debug'] = True
    else:
        VARIABLES['abscoef_debug'] = False
        
    if not LineMixingRosen: exclude.add('YRosen')
    if not LineShift: exclude.update({'Delta0','Delta2'})
    
    # Parameters OmegaRange,OmegaStep,OmegaWing,OmegaWingHW, and OmegaGrid
    # are deprecated and given for backward compatibility with the older versions.
    if WavenumberRange:  OmegaRange=WavenumberRange
    if WavenumberStep:   OmegaStep=WavenumberStep
    if WavenumberWing:   OmegaWing=WavenumberWing
    if WavenumberWingHW: OmegaWingHW=WavenumberWingHW
    if WavenumberGrid:   OmegaGrid=WavenumberGrid

    # "bug" with 1-element list
    Components = listOfTuples(Components)
    SourceTables = listOfTuples(SourceTables)
    
    # determine final input values
    Components,SourceTables,Environment,OmegaRange,OmegaStep,OmegaWing,\
    IntensityThreshold,Format = \
       getDefaultValuesForXsect(Components,SourceTables,Environment,OmegaRange,
                                OmegaStep,OmegaWing,IntensityThreshold,Format)
    
    # warn user about too large omega step
    if OmegaStep>0.005 and profile is PROFILE_DOPPLER: 
        warn('Big wavenumber step: possible accuracy decline')
    elif OmegaStep>0.1: 
        warn('Big wavenumber step: possible accuracy decline')

    # get uniform linespace for cross-section
    #number_of_points = (OmegaRange[1]-OmegaRange[0])/OmegaStep + 1
    #Omegas = linspace(OmegaRange[0],OmegaRange[1],number_of_points)
    if OmegaGrid is not None:
        Omegas = npsort(OmegaGrid)
    else:
        #Omegas = arange(OmegaRange[0],OmegaRange[1],OmegaStep)
        Omegas = arange_(OmegaRange[0],OmegaRange[1],OmegaStep) # fix
    number_of_points = len(Omegas)
    Xsect = zeros(number_of_points)
       
    # reference temperature and pressure
    T_ref_default = __FloatType__(296.) # K
    p_ref_default = __FloatType__(1.) # atm
    
    # actual temperature and pressure
    T = Environment['T'] # K
    p = Environment['p'] # atm
       
    # create dictionary from Components
    ABUNDANCES = {}
    NATURAL_ABUNDANCES = {}
    for Component in Components:
        M = Component[0]
        I = Component[1]
        if len(Component) >= 3:
            ni = Component[2]
        else:
            try:
                ni = ISO[(M,I)][ISO_INDEX['abundance']]
            except KeyError:
                raise Exception('cannot find component M,I = %d,%d.' % (M,I))
        ABUNDANCES[(M,I)] = ni
        NATURAL_ABUNDANCES[(M,I)] = ISO[(M,I)][ISO_INDEX['abundance']]
        
    # pre-calculation of volume concentration
    if HITRAN_units:
        factor = __FloatType__(1.0)
    else:
        factor = volumeConcentration(p,T)
        
    # setup the Diluent variable
    GammaL = GammaL.lower()
    if not Diluent:
        if GammaL == 'gamma_air':
            Diluent = {'air':1.}
        elif GammaL == 'gamma_self':
            Diluent = {'self':1.}
        else:
            raise Exception('Unknown GammaL value: %s' % GammaL)
        
    # Simple check
    print(Diluent)  # Added print statement # CHANGED RJH 23MAR18  # Simple check
    for key in Diluent:
        val = Diluent[key]
        if val < 0 or val > 1: # if val < 0 and val > 1:# CHANGED RJH 23MAR18
            raise Exception('Diluent fraction must be in [0,1]')
            
    # ================= HERE THE GENERIC PART STARTS =====================

    t = time()
    
    CALC_INFO_TOTAL = []
    
    # SourceTables contain multiple tables
    for TableName in SourceTables:
    
        # exclude parameters not involved in calculation
        DATA_DICT = LOCAL_TABLE_CACHE[TableName]['data']
        parnames_exclude = ['a','global_upper_quanta','global_lower_quanta',
            'local_upper_quanta','local_lower_quanta','ierr','iref','line_mixing_flag'] 
        parnames = set(DATA_DICT)-set(parnames_exclude)
        
        nlines = len(DATA_DICT['nu'])

        for RowID in range(nlines):
                            
            # create the transition object
            TRANS = CaselessDict({parname:DATA_DICT[parname][RowID] for parname in parnames}) # CORRECTLY HANDLES DIFFERENT SPELLING OF PARNAMES
            TRANS['T'] = T
            TRANS['p'] = p
            TRANS['T_ref'] = T_ref_default
            TRANS['p_ref'] = p_ref_default
            TRANS['Diluent'] = Diluent
            TRANS['Abundances'] = ABUNDANCES
            
            # filter by molecule and isotopologue
            if (TRANS['molec_id'],TRANS['local_iso_id']) not in ABUNDANCES: continue
                
            #   FILTER by LineIntensity: compare it with IntencityThreshold
            TRANS['SigmaT']     = partitionFunction(TRANS['molec_id'],TRANS['local_iso_id'],TRANS['T'])
            TRANS['SigmaT_ref'] = partitionFunction(TRANS['molec_id'],TRANS['local_iso_id'],TRANS['T_ref'])
            LineIntensity = calculate_parameter_Sw(None,TRANS)
            if LineIntensity < IntensityThreshold: continue

            # calculate profile parameters 
            if VARIABLES['abscoef_debug']:
                CALC_INFO = {}
            else:
                CALC_INFO = None                
            PARAMETERS = calcpars(TRANS=TRANS,CALC_INFO=CALC_INFO,exclude=exclude)
            
            # get final wing of the line according to max(Gamma0,GammaD), OmegaWingHW and OmegaWing
            try:
                GammaD = PARAMETERS['GammaD']
            except KeyError:
                GammaD = 0
            try:
                Gamma0 = PARAMETERS['Gamma0']
            except KeyError:
                Gamma0 = 0
            GammaMax = max(Gamma0,GammaD)
            if GammaMax==0 and OmegaWingHW==0:
                OmegaWing = 10.0 # 10 cm-1 default in case if Gamma0 and GammaD are missing
                warn('Gamma0 and GammaD are missing; setting OmegaWing to %f cm-1'%OmegaWing)
            OmegaWingF = max(OmegaWing,OmegaWingHW*GammaMax)
            
            # calculate profile on a grid            
            BoundIndexLower = bisect(Omegas,TRANS['nu']-OmegaWingF)
            BoundIndexUpper = bisect(Omegas,TRANS['nu']+OmegaWingF)
            PARAMETERS['WnGrid'] = Omegas[BoundIndexLower:BoundIndexUpper]
            lineshape_vals = profile(**PARAMETERS)
            Xsect[BoundIndexLower:BoundIndexUpper] += factor * lineshape_vals
                   
            # append debug information for the abscoef routine                
            if VARIABLES['abscoef_debug']: DEBUG.append(CALC_INFO)
        
    print('%f seconds elapsed for abscoef; nlines = %d'%(time()-t,nlines))
    
    if File: save_to_file(File,Format,Omegas,Xsect)
    return Omegas,Xsect    

def absorptionCoefficient_Priority(*args,**kwargs):
    return absorptionCoefficient_Generic(*args,**kwargs,
                                         profile=PROFILE_HT,
                                         calcpars=calculateProfileParametersFullPriority)    
    
def absorptionCoefficient_HT(*args,**kwargs):
    return absorptionCoefficient_Generic(*args,**kwargs,
                                         profile=PROFILE_HT,
                                         calcpars=calculateProfileParametersHT)    
                                   
def absorptionCoefficient_SDVoigt(*args,**kwargs):
    return absorptionCoefficient_Generic(*args,**kwargs,
                                          profile=PROFILE_SDVOIGT,
                                          calcpars=calculateProfileParametersSDVoigt)
        
def absorptionCoefficient_Voigt(*args,**kwargs):
    return absorptionCoefficient_Generic(*args,**kwargs,
                                          profile=PROFILE_VOIGT,
                                          calcpars=calculateProfileParametersVoigt)

def absorptionCoefficient_Lorentz(*args,**kwargs):
    return absorptionCoefficient_Generic(*args,**kwargs,
                                         profile=PROFILE_LORENTZ,
                                         calcpars=calculateProfileParametersLorentz)
     
def absorptionCoefficient_Doppler(*args,**kwargs):   
    return absorptionCoefficient_Generic(*args,**kwargs,
                                         profile=PROFILE_DOPPLER,
                                         calcpars=calculateProfileParametersDoppler)
    
absorptionCoefficient_Generic.__doc__ = ABSCOEF_DOCSTRING_TEMPLATE.format(
    profile='Generic',
    usage_example="""
        nu,coef = absorptionCoefficient_Generic(((2,1),),'co2',WavenumberStep=0.01,
                                              HITRAN_units=False,Diluent={'air':1},
                                              profile=PROFILE_VOIGT,
                                              calcpars=calcProfileParametersVoigt,
                                              DEBUG=None,
                                              )
    """
)

absorptionCoefficient_Priority.__doc__ = ABSCOEF_DOCSTRING_TEMPLATE.format(
    profile='Priority',
    usage_example="""
        nu,coef = absorptionCoefficient_Priority(((2,1),),'co2',WavenumberStep=0.01,
                                              HITRAN_units=False,Diluent={'air':1})
    """
)
                                   
absorptionCoefficient_HT.__doc__ = ABSCOEF_DOCSTRING_TEMPLATE.format(
    profile='HT',
    usage_example="""
        nu,coef = absorptionCoefficient_HT(((2,1),),'co2',WavenumberStep=0.01,
                                              HITRAN_units=False,Diluent={'air':1})
    """
)

absorptionCoefficient_SDVoigt.__doc__ = ABSCOEF_DOCSTRING_TEMPLATE.format(
    profile='SDVoigt',
    usage_example="""
        nu,coef = absorptionCoefficient_SDVoigt(((2,1),),'co2',WavenumberStep=0.01,
                                              HITRAN_units=False,Diluent={'air':1})
    """
)

absorptionCoefficient_Voigt.__doc__ = ABSCOEF_DOCSTRING_TEMPLATE.format(
    profile='Voigt',
    usage_example="""
        nu,coef = absorptionCoefficient_Voigt(((2,1),),'co2',WavenumberStep=0.01,
                                              HITRAN_units=False,Diluent={'air':1})
    """
)

absorptionCoefficient_Lorentz.__doc__ = ABSCOEF_DOCSTRING_TEMPLATE.format(
    profile='Lorentz',
    usage_example="""
        nu,coef = absorptionCoefficient_Lorentz(((2,1),),'co2',WavenumberStep=0.01,
                                              HITRAN_units=False,Diluent={'air':1})
    """
)

absorptionCoefficient_Doppler.__doc__ = ABSCOEF_DOCSTRING_TEMPLATE.format(
    profile='Doppler',
    usage_example="""
        nu,coef = absorptionCoefficient_Doppler(((2,1),),'co2',WavenumberStep=0.01,
                                              HITRAN_units=False,Diluent={'air':1})
    """
)    
    
# save numpy arrays to file
# arrays must have same dimensions
def save_to_file(fname,fformat,*arg):
    f = open(fname,'w')
    for i in range(len(arg[0])):
        argline = []
        for j in range(len(arg)):
            argline.append(arg[j][i])
        f.write((fformat+'\n') % tuple(argline))
    f.close()
    
# ---------------------------------------------------------------------------
# SHORTCUTS AND ALIASES FOR ABSORPTION COEFFICIENTS
# ---------------------------------------------------------------------------

absorptionCoefficient_Gauss = absorptionCoefficient_Doppler

def abscoef_HT(table=None,step=None,grid=None,env={'T':296.,'p':1.},file=None):
    return absorptionCoefficient_HT(SourceTables=table,OmegaStep=step,OmegaGrid=grid,Environment=env,File=file)

def abscoef_Voigt(table=None,step=None,grid=None,env={'T':296.,'p':1.},file=None):
    return absorptionCoefficient_Voigt(SourceTables=table,OmegaStep=step,OmegaGrid=grid,Environment=env,File=file)
    
def abscoef_Lorentz(table=None,step=None,grid=None,env={'T':296.,'p':1.},file=None):
    return absorptionCoefficient_Lorentz(SourceTables=table,OmegaStep=step,OmegaGrid=grid,Environment=env,File=file)

def abscoef_Doppler(table=None,step=None,grid=None,env={'T':296.,'p':1.},file=None):
    return absorptionCoefficient_Doppler(SourceTables=table,OmegaStep=step,OmegaGrid=grid,Environment=env,File=file)

abscoef_Gauss = abscoef_Doppler
    
def abscoef(table=None,step=None,grid=None,env={'T':296.,'p':1.},file=None): # default
    return absorptionCoefficient_Lorentz(SourceTables=table,OmegaStep=step,OmegaGrid=grid,Environment=env,File=file)
    
# ---------------------------------------------------------------------------
    
def transmittanceSpectrum(Omegas,AbsorptionCoefficient,Environment={'l':100.},
                          File=None, Format='%e %e', Wavenumber=None):
    """
    INPUT PARAMETERS: 
        Wavenumber/Omegas:   wavenumber grid                    (required)
        AbsorptionCoefficient:  absorption coefficient on grid  (required)
        Environment:  dictionary containing path length in cm.
                      Default={'l':100.}
        File:         name of the output file                 (optional) 
        Format: c format used in file output, default '%e %e' (optional)
    OUTPUT PARAMETERS: 
        Wavenum: wavenumber grid
        Xsect:  transmittance spectrum calculated on the grid
    ---
    DESCRIPTION:
        Calculate a transmittance spectrum (dimensionless) based
        on previously calculated absorption coefficient.
        Transmittance spectrum is calculated at an arbitrary
        optical path length 'l' (1 m by default)
    ---
    EXAMPLE OF USAGE:
        nu,trans = transmittanceSpectrum(nu,coef)
    ---
    """
    # compatibility with older versions
    if Wavenumber: Omegas=Wavenumber
    l = Environment['l']
    Xsect = exp(-AbsorptionCoefficient*l)
    if File: save_to_file(File,Format,Omegas,Xsect)
    return Omegas,Xsect

def absorptionSpectrum(Omegas,AbsorptionCoefficient,Environment={'l':100.},
                       File=None, Format='%e %e',Wavenumber=None):
    """
    INPUT PARAMETERS: 
        Wavenumber/Omegas:   wavenumber grid                    (required)
        AbsorptionCoefficient:  absorption coefficient on grid  (required)
        Environment:  dictionary containing path length in cm.
                      Default={'l':100.}
        File:         name of the output file                 (optional) 
        Format: c format used in file output, default '%e %e' (optional)
    OUTPUT PARAMETERS: 
        Wavenum: wavenumber grid
        Xsect:  absorption spectrum calculated on the grid
    ---
    DESCRIPTION:
        Calculate an absorption spectrum (dimensionless) based
        on previously calculated absorption coefficient.
        Absorption spectrum is calculated at an arbitrary
        optical path length 'l' (1 m by default)
    ---
    EXAMPLE OF USAGE:
        nu,absorp = absorptionSpectrum(nu,coef)
    ---
    """
    # compatibility with older versions
    if Wavenumber: Omegas=Wavenumber
    l = Environment['l']
    Xsect = 1-exp(-AbsorptionCoefficient*l)
    if File: save_to_file(File,Format,Omegas,Xsect)
    return Omegas,Xsect

def radianceSpectrum(Omegas,AbsorptionCoefficient,Environment={'l':100.,'T':296.},
                     File=None, Format='%e %e', Wavenumber=None):
    """
    INPUT PARAMETERS: 
        Wavenumber/Omegas:   wavenumber grid                   (required)
        AbsorptionCoefficient:  absorption coefficient on grid (required)
        Environment:  dictionary containing path length in cm.
                      and temperature in Kelvin.
                      Default={'l':100.,'T':296.}
        File:         name of the output file                 (optional) 
        Format: c format used in file output, default '%e %e' (optional)
    OUTPUT PARAMETERS: 
        Wavenum: wavenumber grid
        Xsect:  radiance spectrum calculated on the grid
    ---
    DESCRIPTION:
        Calculate a radiance spectrum (in W/sr/cm^2/cm-1) based
        on previously calculated absorption coefficient.
        Radiance spectrum is calculated at an arbitrary
        optical path length 'l' (1 m by default) and 
        temperature 'T' (296 K by default). For obtaining a
        physically meaningful result 'T' must be the same 
        as a temperature which was used in absorption coefficient.
    ---
    EXAMPLE OF USAGE:
        nu,radi = radianceSpectrum(nu,coef)
    ---
    """
    # compatibility with older versions
    if Wavenumber: Omegas=Wavenumber
    l = Environment['l']
    T = Environment['T']
    Alw = 1-exp(-AbsorptionCoefficient*l)
    LBBTw = 2*hh*cc**2*Omegas**3 / (exp(hh*cc*Omegas/(cBolts*T)) - 1) * 1.0E-7
    Xsect = Alw*LBBTw # W/sr/cm**2/cm**-1
    if File: save_to_file(File,Format,Omegas,Xsect)
    return Omegas,Xsect


# GET X,Y FOR FINE PLOTTING OF A STICK SPECTRUM
def getStickXY(TableName):
    """
    Get X and Y for fine plotting of a stick spectrum.
    Usage: X,Y = getStickXY(TableName).
    """
    cent,intens = getColumns(TableName,('nu','sw'))
    n = len(cent)
    cent_ = zeros(n*3)
    intens_ = zeros(n*3)
    for i in range(n):
        intens_[3*i] = 0
        intens_[3*i+1] = intens[i]
        intens_[3*i+2] = 0
        cent_[(3*i):(3*i+3)] = cent[i]
    return cent_,intens_
# /GET X,Y FOR FINE PLOTTING OF A STICK SPECTRUM


# LOW-RES SPECTRA (CONVOLUTION WITH APPARATUS FUNCTION)

# /LOW-RES SPECTRA (CONVOLUTION WITH APPARATUS FUNCTION)

# /----------------------------------------------------------------------------


# ------------------  HITRAN-ON-THE-WEB COMPATIBILITY -------------------------

def read_hotw(filename):
    """
    Read cross-section file fetched from HITRAN-on-the-Web.
    The format of the file line must be as follows: 
      nu, coef
    Other lines are omitted.
    """
    import sys
    f = open(filename,'r')
    nu = []
    coef = []
    for line in f:
        pars = line.split()
        try:
            nu.append(float(pars[0]))
            coef.append(float(pars[1]))
        except:
            if False:
                print(sys.exc_info())
            else:
                pass    
    return array(nu),array(coef)

# alias for read_hotw for backwards compatibility
read_xsect = read_hotw
    
# /----------------------------------------------------------------------------

# ------------------  SPECTRAL CONVOLUTION -------------------------

# rectangular slit function
def SLIT_RECTANGULAR(x,g):
    """
    Instrumental (slit) function.
    B(x) = 1/γ , if |x| ≤ γ/2 & B(x) = 0, if |x| > γ/2,
    where γ is a slit width or the instrumental resolution.
    """
    index_inner = abs(x) <= g/2
    index_outer = ~index_inner
    y = zeros(len(x))
    y[index_inner] = 1/g
    y[index_outer] = 0
    return y

# triangular slit function
def SLIT_TRIANGULAR(x,g):
    """
    Instrumental (slit) function.
    B(x) = 1/γ*(1-|x|/γ), if |x| ≤ γ & B(x) = 0, if |x| > γ,
    where γ is the line width equal to the half base of the triangle.
    """
    index_inner = abs(x) <= g
    index_outer = ~index_inner
    y = zeros(len(x))
    y[index_inner] = 1/g * (1 - abs(x[index_inner])/g)
    y[index_outer] = 0
    return y

# gaussian slit function
def SLIT_GAUSSIAN(x,g):
    """
    Instrumental (slit) function.
    B(x) = sqrt(ln(2)/pi)/γ*exp(-ln(2)*(x/γ)**2),
    where γ/2 is a gaussian half-width at half-maximum.
    """
    g /= 2
    return sqrt(log(2))/(sqrt(pi)*g)*exp(-log(2)*(x/g)**2)

# dispersion slit function
def SLIT_DISPERSION(x,g):
    """
    Instrumental (slit) function.
    B(x) = γ/pi/(x**2+γ**2),
    where γ/2 is a lorentzian half-width at half-maximum.
    """
    g /= 2
    return g/pi/(x**2+g**2)

# cosinus slit function
def SLIT_COSINUS(x,g):
    return (cos(pi/g*x)+1)/(2*g)

# diffraction slit function
def SLIT_DIFFRACTION(x,g):
    """
    Instrumental (slit) function.
    """
    y = zeros(len(x))
    index_zero = x==0
    index_nonzero = ~index_zero
    dk_ = pi/g
    x_ = dk_*x[index_nonzero]
    w_ = sin(x_)
    r_ = w_**2/x_**2
    y[index_zero] = 1
    y[index_nonzero] = r_/g
    return y

# apparatus function of the ideal Michelson interferometer
def SLIT_MICHELSON(x,g):
    """
    Instrumental (slit) function.
    B(x) = 2/γ*sin(2pi*x/γ)/(2pi*x/γ) if x!=0 else 1,
    where 1/γ is the maximum optical path difference.
    """
    y = zeros(len(x))
    index_zero = x==0
    index_nonzero = ~index_zero
    dk_ = 2*pi/g
    x_ = dk_*x[index_nonzero]
    y[index_zero] = 1
    y[index_nonzero] = 2/g*sin(x_)/x_
    return y

# spectral convolution with an apparatus (slit) function
def convolveSpectrum(Omega,CrossSection,Resolution=0.1,AF_wing=10.,
                     SlitFunction=SLIT_RECTANGULAR,Wavenumber=None):
    """
    INPUT PARAMETERS: 
        Wavenumber/Omega:    wavenumber grid                     (required)
        CrossSection:  high-res cross section calculated on grid (required)
        Resolution:    instrumental resolution γ                 (optional)
        AF_wing:       instrumental function wing                (optional)
        SlitFunction:  instrumental function for low-res spectra calculation (optional)
    OUTPUT PARAMETERS: 
        Wavenum: wavenumber grid
        CrossSection: low-res cross section calculated on grid
        i1: lower index in Omega input
        i2: higher index in Omega input
        slit: slit function calculated over grid [-AF_wing; AF_wing]
                with the step equal to instrumental resolution. 
    ---
    DESCRIPTION:
        Produce a simulation of experimental spectrum via the convolution 
        of a “dry” spectrum with an instrumental function.
        Instrumental function is provided as a parameter and
        is calculated in a grid with the width=AF_wing and step=Resolution.
    ---
    EXAMPLE OF USAGE:
        nu_,radi_,i,j,slit = convolveSpectrum(nu,radi,Resolution=2.0,AF_wing=10.0,
                                                SlitFunction=SLIT_MICHELSON)
    ---
    """    
    # compatibility with older versions
    if Wavenumber: Omega=Wavenumber
    step = Omega[1]-Omega[0]
    if step>=Resolution: raise Exception('step must be less than resolution')
    #x = arange(-AF_wing,AF_wing+step,step)
    x = arange_(-AF_wing,AF_wing+step,step) # fix
    slit = SlitFunction(x,Resolution)
    slit /= sum(slit)*step # simple normalization
    left_bnd = int(len(slit)/2) # new versions of Numpy don't support float indexing
    right_bnd = len(Omega) - int(len(slit)/2) # new versions of Numpy don't support float indexing
    CrossSectionLowRes = convolve(CrossSection,slit,mode='same')*step
    return Omega[left_bnd:right_bnd],CrossSectionLowRes[left_bnd:right_bnd],left_bnd,right_bnd,slit

# spectral convolution with an apparatus (slit) function
def convolveSpectrumSame(Omega,CrossSection,Resolution=0.1,AF_wing=10.,
                         SlitFunction=SLIT_RECTANGULAR,Wavenumber=None):
    """
    Convolves cross section with a slit function with given parameters.
    """
    # compatibility with older versions
    if Wavenumber: Omega=Wavenumber
    step = Omega[1]-Omega[0]
    if step>=Resolution: raise Exception('step must be less than resolution')
    #x = arange(-AF_wing,AF_wing+step,step)
    x = arange_(-AF_wing,AF_wing+step,step) # fix
    slit = SlitFunction(x,Resolution)
    slit /= sum(slit)*step # simple normalization
    left_bnd = 0
    right_bnd = len(Omega)
    CrossSectionLowRes = convolve(CrossSection,slit,mode='same')*step
    return Omega[left_bnd:right_bnd],CrossSectionLowRes[left_bnd:right_bnd],left_bnd,right_bnd,slit

def convolveSpectrumFull(Omega,CrossSection,Resolution=0.1,AF_wing=10.,SlitFunction=SLIT_RECTANGULAR):
    """
    Convolves cross section with a slit function with given parameters.
    """
    step = Omega[1]-Omega[0]
    x = arange(-AF_wing,AF_wing+step,step)
    slit = SlitFunction(x,Resolution)
    print('step=')
    print(step)
    print('x=')
    print(x)
    print('slitfunc=')
    print(SlitFunction)
    CrossSectionLowRes = convolve(CrossSection,slit,mode='full')*step
    return Omega,CrossSectionLowRes,None,None

# ------------------------------------------------------------------

# ------------------  SAVE CALC INFO IN CSV FORMAT -------------------------

def save_abscoef_calc_info(filename,parname,CALC_INFO_LIST,delim=';'):
    """
    This is an attempt to save the CALC_INFO from the 
    new versions of the abscoef function.
    Currently it saves one parameters per call.
    To build your own function that better suits your needs
    please consult the structure of the CALC_INFO elements.
    """
    col = [] # make empty collection
    order = []    
    for CALC_INFO in CALC_INFO_LIST:
        INFO = CALC_INFO[parname]
        item = {}
        item['val'] = INFO['value']
        if 'val' not in order: 
            order.append('val')
        for broadener in INFO['mixture']:
            for argname in INFO['mixture'][broadener]['args']:
                src_name = '%s_%s_src'%(argname,broadener)
                val_name = '%s_%s_val'%(argname,broadener)
                item[src_name] = INFO['mixture'][broadener]['args'][argname]['source']
                item[val_name] = INFO['mixture'][broadener]['args'][argname]['value']
                if src_name not in order:
                    order.append(src_name)
                if val_name not in order:
                    order.append(val_name)
        col.append(item)    
    # Export the result to the CSV file.
    #col.export_csv('test2.py_%s_%s.csv'%(TABLE_NAMETABLE_NAME,parname),order=order)    
    with open(filename,'w') as fout:
        header = ('%s'%delim).join(order)
        fout.write(header+'\n')
        for CALC_INFO in col:
            line = ('%s'%delim).join([str(CALC_INFO.get(pname,'')) for pname in order])
            fout.write(line+'\n')
